/* LanguageData.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from LanguageData.vala, do not modify */

/* LanguageData.vala
 *
 * Copyright (C) 2020-2025 Jerry Casiano
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.
 *
 * If not, see <http://www.gnu.org/licenses/gpl-3.0.txt>.
*/

#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib/gi18n-lib.h>
#include <gio/gio.h>
#include "font-manager.h"
#include <gtk/gtk.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define FONT_MANAGER_GOOGLE_FONTS_TYPE_LANGUAGE_DATA (font_manager_google_fonts_language_data_get_type ())
typedef struct _FontManagerGoogleFontsLanguageData FontManagerGoogleFontsLanguageData;
#define _g_free0(var) (var = (g_free (var), NULL))

#define FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE (font_manager_google_fonts_sample_get_type ())
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE, FontManagerGoogleFontsSample))
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE, FontManagerGoogleFontsSampleClass))
#define FONT_MANAGER_GOOGLE_FONTS_IS_SAMPLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE))
#define FONT_MANAGER_GOOGLE_FONTS_IS_SAMPLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE))
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE, FontManagerGoogleFontsSampleClass))

typedef struct _FontManagerGoogleFontsSample FontManagerGoogleFontsSample;
typedef struct _FontManagerGoogleFontsSampleClass FontManagerGoogleFontsSampleClass;
typedef struct _FontManagerGoogleFontsSamplePrivate FontManagerGoogleFontsSamplePrivate;
enum  {
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_0_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LABEL_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_NAME_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_SAMPLE_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_NUM_PROPERTIES
};
static GParamSpec* font_manager_google_fonts_sample_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_NUM_PROPERTIES];

#define FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL (font_manager_google_fonts_sample_model_get_type ())
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL, FontManagerGoogleFontsSampleModel))
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL, FontManagerGoogleFontsSampleModelClass))
#define FONT_MANAGER_GOOGLE_FONTS_IS_SAMPLE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL))
#define FONT_MANAGER_GOOGLE_FONTS_IS_SAMPLE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL))
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL, FontManagerGoogleFontsSampleModelClass))

typedef struct _FontManagerGoogleFontsSampleModel FontManagerGoogleFontsSampleModel;
typedef struct _FontManagerGoogleFontsSampleModelClass FontManagerGoogleFontsSampleModelClass;
typedef struct _FontManagerGoogleFontsSampleModelPrivate FontManagerGoogleFontsSampleModelPrivate;
enum  {
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_0_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_ITEMS_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_NUM_PROPERTIES
};
static GParamSpec* font_manager_google_fonts_sample_model_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_ROW (font_manager_google_fonts_sample_row_get_type ())
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_ROW, FontManagerGoogleFontsSampleRow))
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_ROW, FontManagerGoogleFontsSampleRowClass))
#define FONT_MANAGER_GOOGLE_FONTS_IS_SAMPLE_ROW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_ROW))
#define FONT_MANAGER_GOOGLE_FONTS_IS_SAMPLE_ROW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_ROW))
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_ROW, FontManagerGoogleFontsSampleRowClass))

typedef struct _FontManagerGoogleFontsSampleRow FontManagerGoogleFontsSampleRow;
typedef struct _FontManagerGoogleFontsSampleRowClass FontManagerGoogleFontsSampleRowClass;
typedef struct _FontManagerGoogleFontsSampleRowPrivate FontManagerGoogleFontsSampleRowPrivate;
enum  {
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_0_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_LABEL_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_SAMPLE_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_NUM_PROPERTIES
};
static GParamSpec* font_manager_google_fonts_sample_row_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_NUM_PROPERTIES];

#define FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST (font_manager_google_fonts_sample_list_get_type ())
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST, FontManagerGoogleFontsSampleList))
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST, FontManagerGoogleFontsSampleListClass))
#define FONT_MANAGER_GOOGLE_FONTS_IS_SAMPLE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST))
#define FONT_MANAGER_GOOGLE_FONTS_IS_SAMPLE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST))
#define FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST, FontManagerGoogleFontsSampleListClass))

typedef struct _FontManagerGoogleFontsSampleList FontManagerGoogleFontsSampleList;
typedef struct _FontManagerGoogleFontsSampleListClass FontManagerGoogleFontsSampleListClass;
typedef struct _FontManagerGoogleFontsSampleListPrivate FontManagerGoogleFontsSampleListPrivate;
enum  {
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_0_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_ITEMS_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_MODEL_PROPERTY,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_NUM_PROPERTIES
};
static GParamSpec* font_manager_google_fonts_sample_list_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_NUM_PROPERTIES];
enum  {
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_ROW_SELECTED_SIGNAL,
	FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_NUM_SIGNALS
};
static guint font_manager_google_fonts_sample_list_signals[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_NUM_SIGNALS] = {0};

struct _FontManagerGoogleFontsLanguageData {
	gchar* label;
	gchar* name;
	gchar* sample;
};

struct _FontManagerGoogleFontsSample {
	GObject parent_instance;
	FontManagerGoogleFontsSamplePrivate * priv;
};

struct _FontManagerGoogleFontsSampleClass {
	GObjectClass parent_class;
};

struct _FontManagerGoogleFontsSamplePrivate {
	gchar* _label;
	gchar* _name;
	gchar* _sample;
};

struct _FontManagerGoogleFontsSampleModel {
	GObject parent_instance;
	FontManagerGoogleFontsSampleModelPrivate * priv;
};

struct _FontManagerGoogleFontsSampleModelClass {
	GObjectClass parent_class;
};

struct _FontManagerGoogleFontsSampleModelPrivate {
	FontManagerStringSet* _items;
	guint n_items;
};

struct _FontManagerGoogleFontsSampleRow {
	GtkBox parent_instance;
	FontManagerGoogleFontsSampleRowPrivate * priv;
};

struct _FontManagerGoogleFontsSampleRowClass {
	GtkBoxClass parent_class;
};

struct _FontManagerGoogleFontsSampleRowPrivate {
	GtkLabel* _label;
	GtkInscription* _sample;
};

struct _FontManagerGoogleFontsSampleList {
	GtkPopover parent_instance;
	FontManagerGoogleFontsSampleListPrivate * priv;
};

struct _FontManagerGoogleFontsSampleListClass {
	GtkPopoverClass parent_class;
};

struct _FontManagerGoogleFontsSampleListPrivate {
	FontManagerStringSet* _items;
	FontManagerGoogleFontsSampleModel* _model;
	GtkListBox* sample_list;
};

static gint FontManagerGoogleFontsSample_private_offset;
static gpointer font_manager_google_fonts_sample_parent_class = NULL;
static gint FontManagerGoogleFontsSampleModel_private_offset;
static gpointer font_manager_google_fonts_sample_model_parent_class = NULL;
static GListModelInterface * font_manager_google_fonts_sample_model_g_list_model_parent_iface = NULL;
static gint FontManagerGoogleFontsSampleRow_private_offset;
static gpointer font_manager_google_fonts_sample_row_parent_class = NULL;
static gint FontManagerGoogleFontsSampleList_private_offset;
static gpointer font_manager_google_fonts_sample_list_parent_class = NULL;

VALA_EXTERN GType font_manager_google_fonts_language_data_get_type (void) G_GNUC_CONST ;
VALA_EXTERN FontManagerGoogleFontsLanguageData* font_manager_google_fonts_language_data_dup (const FontManagerGoogleFontsLanguageData* self);
VALA_EXTERN void font_manager_google_fonts_language_data_free (FontManagerGoogleFontsLanguageData* self);
VALA_EXTERN void font_manager_google_fonts_language_data_copy (const FontManagerGoogleFontsLanguageData* self,
                                                   FontManagerGoogleFontsLanguageData* dest);
VALA_EXTERN void font_manager_google_fonts_language_data_destroy (FontManagerGoogleFontsLanguageData* self);
G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC (FontManagerGoogleFontsLanguageData, font_manager_google_fonts_language_data_destroy)
VALA_EXTERN GType font_manager_google_fonts_sample_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerGoogleFontsSample, g_object_unref)
VALA_EXTERN FontManagerGoogleFontsSample* font_manager_google_fonts_sample_new (const gchar* lang);
VALA_EXTERN FontManagerGoogleFontsSample* font_manager_google_fonts_sample_construct (GType object_type,
                                                                          const gchar* lang);
VALA_EXTERN void font_manager_google_fonts_sample_set_label (FontManagerGoogleFontsSample* self,
                                                 const gchar* value);
VALA_EXTERN void font_manager_google_fonts_sample_set_name (FontManagerGoogleFontsSample* self,
                                                const gchar* value);
VALA_EXTERN void font_manager_google_fonts_sample_set_sample (FontManagerGoogleFontsSample* self,
                                                  const gchar* value);
VALA_EXTERN const gchar* font_manager_google_fonts_sample_get_label (FontManagerGoogleFontsSample* self);
VALA_EXTERN const gchar* font_manager_google_fonts_sample_get_name (FontManagerGoogleFontsSample* self);
VALA_EXTERN const gchar* font_manager_google_fonts_sample_get_sample (FontManagerGoogleFontsSample* self);
static void font_manager_google_fonts_sample_finalize (GObject * obj);
static GType font_manager_google_fonts_sample_get_type_once (void);
static void _vala_font_manager_google_fonts_sample_get_property (GObject * object,
                                                          guint property_id,
                                                          GValue * value,
                                                          GParamSpec * pspec);
static void _vala_font_manager_google_fonts_sample_set_property (GObject * object,
                                                          guint property_id,
                                                          const GValue * value,
                                                          GParamSpec * pspec);
VALA_EXTERN GType font_manager_google_fonts_sample_model_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerGoogleFontsSampleModel, g_object_unref)
static GType font_manager_google_fonts_sample_model_real_get_item_type (GListModel* base);
static guint font_manager_google_fonts_sample_model_real_get_n_items (GListModel* base);
VALA_EXTERN FontManagerStringSet* font_manager_google_fonts_sample_model_get_items (FontManagerGoogleFontsSampleModel* self);
static GObject* font_manager_google_fonts_sample_model_real_get_item (GListModel* base,
                                                               guint position);
VALA_EXTERN FontManagerGoogleFontsSampleModel* font_manager_google_fonts_sample_model_new (void);
VALA_EXTERN FontManagerGoogleFontsSampleModel* font_manager_google_fonts_sample_model_construct (GType object_type);
VALA_EXTERN void font_manager_google_fonts_sample_model_set_items (FontManagerGoogleFontsSampleModel* self,
                                                       FontManagerStringSet* value);
static GObject * font_manager_google_fonts_sample_model_constructor (GType type,
                                                              guint n_construct_properties,
                                                              GObjectConstructParam * construct_properties);
static void _font_manager_google_fonts_sample_model___lambda165_ (FontManagerGoogleFontsSampleModel* self);
static void __font_manager_google_fonts_sample_model___lambda165__g_object_notify (GObject* _sender,
                                                                            GParamSpec* pspec,
                                                                            gpointer self);
static void font_manager_google_fonts_sample_model_finalize (GObject * obj);
static GType font_manager_google_fonts_sample_model_get_type_once (void);
static void _vala_font_manager_google_fonts_sample_model_get_property (GObject * object,
                                                                guint property_id,
                                                                GValue * value,
                                                                GParamSpec * pspec);
static void _vala_font_manager_google_fonts_sample_model_set_property (GObject * object,
                                                                guint property_id,
                                                                const GValue * value,
                                                                GParamSpec * pspec);
VALA_EXTERN GType font_manager_google_fonts_sample_row_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerGoogleFontsSampleRow, g_object_unref)
VALA_EXTERN FontManagerGoogleFontsSampleRow* font_manager_google_fonts_sample_row_from_item (GObject* item);
VALA_EXTERN FontManagerGoogleFontsSampleRow* font_manager_google_fonts_sample_row_new (void);
VALA_EXTERN FontManagerGoogleFontsSampleRow* font_manager_google_fonts_sample_row_construct (GType object_type);
VALA_EXTERN GtkLabel* font_manager_google_fonts_sample_row_get_label (FontManagerGoogleFontsSampleRow* self);
VALA_EXTERN GtkInscription* font_manager_google_fonts_sample_row_get_sample (FontManagerGoogleFontsSampleRow* self);
static void font_manager_google_fonts_sample_row_finalize (GObject * obj);
static GType font_manager_google_fonts_sample_row_get_type_once (void);
static void _vala_font_manager_google_fonts_sample_row_get_property (GObject * object,
                                                              guint property_id,
                                                              GValue * value,
                                                              GParamSpec * pspec);
VALA_EXTERN GType font_manager_google_fonts_sample_list_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerGoogleFontsSampleList, g_object_unref)
static void font_manager_google_fonts_sample_list_on_row_activated (FontManagerGoogleFontsSampleList* self,
                                                             GtkListBox* list,
                                                             GtkListBoxRow* row);
VALA_EXTERN FontManagerGoogleFontsSampleModel* font_manager_google_fonts_sample_list_get_model (FontManagerGoogleFontsSampleList* self);
static void _font_manager_google_fonts_sample_list_on_row_activated_gtk_list_box_row_activated (GtkListBox* _sender,
                                                                                         GtkListBoxRow* row,
                                                                                         gpointer self);
VALA_EXTERN void font_manager_google_fonts_sample_list_unselect_all (FontManagerGoogleFontsSampleList* self);
VALA_EXTERN FontManagerGoogleFontsSampleList* font_manager_google_fonts_sample_list_new (void);
VALA_EXTERN FontManagerGoogleFontsSampleList* font_manager_google_fonts_sample_list_construct (GType object_type);
VALA_EXTERN FontManagerStringSet* font_manager_google_fonts_sample_list_get_items (FontManagerGoogleFontsSampleList* self);
VALA_EXTERN void font_manager_google_fonts_sample_list_set_items (FontManagerGoogleFontsSampleList* self,
                                                      FontManagerStringSet* value);
static void font_manager_google_fonts_sample_list_set_model (FontManagerGoogleFontsSampleList* self,
                                                      FontManagerGoogleFontsSampleModel* value);
static GObject * font_manager_google_fonts_sample_list_constructor (GType type,
                                                             guint n_construct_properties,
                                                             GObjectConstructParam * construct_properties);
static GtkWidget* _font_manager_google_fonts_sample_row_from_item_gtk_list_box_create_widget_func (GObject* item,
                                                                                            gpointer self);
static void font_manager_google_fonts_sample_list_finalize (GObject * obj);
static GType font_manager_google_fonts_sample_list_get_type_once (void);
static void _vala_font_manager_google_fonts_sample_list_get_property (GObject * object,
                                                               guint property_id,
                                                               GValue * value,
                                                               GParamSpec * pspec);
static void _vala_font_manager_google_fonts_sample_list_set_property (GObject * object,
                                                               guint property_id,
                                                               const GValue * value,
                                                               GParamSpec * pspec);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

const FontManagerGoogleFontsLanguageData FONT_MANAGER_GOOGLE_FONTS_Languages[142] = {{N_ ("Adlam"), "adlam", "𞤚𞤵𞥅𞤺𞤢𞥄𞤣𞤫 𞤱𞤮𞤲𞤣𞤫 𞤸𞤫𞤬𞤼𞤭𞤲𞤺𞤮𞤤 𞤸𞤮𞤪𞤥𞤢 𞤳𞤢𞤤𞤢 𞤲𞤫𞤯𞥆𞤮 𞤫"}, {N_ ("Anatolian Hieroglyphs"), "anatolian-heiroglyphs", "𔗷𔗬𔑈𔓯𔐤𔗷𔖶𔔆𔗐𔓱𔑣𔓢𔑈𔓷𔖻𔗔𔑏𔖱𔗷𔖶𔑦𔗬𔓯𔓷 𔖖𔓢𔕙𔑯𔗦 𔖪𔖱𔖪 𔑮𔐓𔗵𔗬 𔐱𔕬𔗬𔑰𔖱"}, {N_ ("Arabic"), "arabic", "الحب سماء لا تمطر غير الأحلام."}, {N_ ("Armenian"), "armenian", "Քանզի մարդկային ընտանիքի բոլոր անդամներին"}, {N_ ("Avestan"), "avestan", "𐬞𐬎𐬭𐬕𐬍𐬝 𐬛𐬁𐬢𐬁 𐬋 𐬨𐬀𐬌𐬢𐬌𐬌𐬋 𐬑𐬀𐬭𐬛 𐬐𐬎"}, {N_ ("Balinese"), "balinese", "ᬫᬦᬶᬫ᭄ᬩᬦ᭄ᬕ᭄ ᬭᬶᬅᬦ᭄ᬢᬸᬓᬦ᭄ ᬧᬦ᭄ᬕᬦ᭄ᬕ᭄ᬓᬾᬦ᭄ ᬭᬶᬦ᭄ᬕ᭄ ᬲᬸᬪᬓᬃᬫ"}, {N_ ("Bamum"), "bamum", "ꚳꚴ𖥉𖥊𖥋𖥌𖥍 𖡼𖡽𖠀𖠁𖠂𖠃𖠄 𖠎𖠏𖠐𖠑𖠒𖠓𖠔 ꚠꚡꚢꚣꚤꚥꚦ 𖨔𖨕𖨖𖨗𖨘𖨙𖨚 𖠣𖠤𖠥𖠦𖠧𖠨𖠩"}, {N_ ("Bassa Vah"), "bassa-vah", "𖫞𖫫𖫰 𖫐𖫭𖫱𖫐𖫗𖫭𖫰𖫞𖫭𖫰 𖫑𖫫𖫱 𖫔𖫬𖫱𖫞𖫬𖫱𖫭𖫱𖫐𖫕𖫭𖫰 𖫔𖫪𖫰𖫐𖫬𖫲𖫐 𖫞𖫫𖫰𖫬𖫱 𖫕𖫨𖫲𖫐𖫪𖫳𖫐𖫕𖫪𖫱"}, {N_ ("Batak"), "batak", "ᯘᯮᯂᯮᯖᯉ᯲ᯉᯉ᯲ᯐᯬᯔ᯲ᯅᯤᯇᯪᯂ᯲᯾"}, {N_ ("Bengali"), "bengali", "আগুনের শিখা নিভে গিয়েছিল, আর তিনি জানলা দিয়ে তারাদের দিকে তাকালেন৷"}, {N_ ("Bhaiksuki"), "bhaiksuki", "𑰧𑰝𑰿𑰨𑱃𑰕𑰐𑰝𑰰𑱃𑰫𑰯𑰡𑰿𑰝𑰰𑰡𑰿𑰧𑰯𑰧𑰭𑰿𑰪𑰯𑰝𑰡𑰿𑰝𑰿𑰨𑰿𑰧𑰯𑰜𑰯𑰽𑱃𑰁𑰠𑰯𑰨𑰾𑱃𑰦𑰯𑰡𑰪𑰢𑰨𑰰𑰪𑰯𑰨𑰭𑰿𑰧𑱃𑰭𑰨𑰿𑰪𑰸𑰬𑰯𑰦𑰢𑰰𑱃𑰭" \
"𑰟𑰭"}, {N_ ("Brahmi"), "brahmi", "𑀫𑀦 𑀱𑀩𑀩 𑀫𑀯𑀥𑀬𑀢𑀅 𑀩𑀅𑀬𑀔𑀭𑀅 𑀰𑀭𑀰𑀦𑀬𑀅 𑀮𑀓𑀮𑀳 𑀳𑀥𑀫𑀅 𑀥𑀓"}, {N_ ("Braille"), "braille", "⠁⠇⠇ ⠓⠥⠍⠁⠝ ⠃⠑⠊⠝⠛⠎ ⠁⠗⠑ ⠃⠕⠗⠝ ⠋⠗⠑⠑ ⠁⠝⠙ ⠑⠟⠥⠁⠇"}, {N_ ("Buginese"), "buginese", "Pura ritimbang nasengnge dipattongengngi"}, {N_ ("Caucasian Albanian"), "caucasian-albanian", "𐕗𐕘𐕙𐔷𐔸𐔹𐔺 𐔻𐔼𐔽𐕌𐕍𐕎𐕏 𐕂𐕃𐕄𐕅𐕆𐕇𐕈 𐔰𐔱𐔲𐔳𐔴𐔵𐔶 𐕡𐕢𐕣𐔾𐔿𐕀𐕁 𐕉𐕊𐕋𐕚𐕛𐕜𐕝"}, {N_ ("Canadian Aboriginal"), "canadian-aboriginal", "ᐃᒪᐃᒻᒪᑦ ᐃᓕᑕᖅᓯᒪᐅᑎᖃᑦᒪᑦ ᓯᕗᓕᕐᓂᓴᑐᖃᕐᓂᒃ ᓂᕐᓱᐃᓂᑦᒥᒃ"}, {N_ ("Carian"), "carian", "𐊪𐊾𐊠𐊽𐊾𐊲𐊸𐊫𐊷𐋉𐋃𐊷𐊲𐊷𐊲𐊰𐊫𐋇𐊫𐊰𐊪𐊫𐊣"}, {N_ ("Chakma"), "chakma", "𑄡𑄨𑄠𑄚𑄧𑄖𑄳𑄠𑄬 𑄟𑄚𑄬𑄭 𑄉𑄨𑄢𑄨𑄢𑄴 𑄝𑄬𑄇𑄴𑄅𑄚𑄧𑄖𑄳𑄠𑄴 𑄥𑄧𑄁 𑄃𑄮 𑄑𑄚𑄑𑄚𑄳𑄠𑄴 𑄝𑄚𑄝𑄚𑄳𑄠𑄴 𑄃𑄇𑄴𑄇𑄥𑄁𑄃𑄚𑄩"}, {N_ ("Cham"), "cham", "ꨢꨓꨴ ꨎꨈꨓꨪ ꨦꨩꩆꨓꨪꨘꨳꨩꨢꨧꨶꨩꨓꩆꨓꨴꨳꨩꨘꨩꩌ ꨀꨩꨖꨩꨣꩍ ꨠꨩꨘꨥꨚꨣꨪꨥꨩꨣꨧꨳ ꨧꨣꨶꨯꨮꨦꨩꨠꨚꨪ ꨧꨕꨧꨳꨩꨘꨩꩌ"}, {N_ ("Cherokee"), "cherokee", "ᎠᎣᎤᎴᎺᎾᏃᏆᏒᏔᏣᏫᏲᏴ"}, {N_ ("Chinese (Hong Kong)"), "chinese-hongkong", "他們所有的設備和儀器彷彿都是有生命的。"}, {N_ ("Chinese (Simplified)"), "chinese-simplified", "他们所有的设备和仪器彷佛都是有生命的。"}, {N_ ("Chinese (Traditional)"), "chinese-traditional", "他們所有的設備和儀器彷彿都是有生命的。"}, {N_ ("Chorasmian"), "chorasmian", "𐾽𐾾 𐾿𐾲𐾲 𐾽𐾶𐾴𐾺𐿄𐾰 𐾲𐾰𐾺𐾻𐿂𐾰 𐿃𐿂𐿃𐾾𐾺𐾰 𐾼𐾻𐾼𐾵 𐾵𐾴𐾽𐾰 𐾴𐾻"}, {N_ ("Coptic"), "coptic", "لمّا كان الاعتراف بالكرامة المتأصلة في جميع"}, {N_ ("Cuneiform"), "cuneiform", "𒆪𒌋𒀭𒆷𒈦𒇻𒀭𒆘𒀀𒁺𒀭𒀫𒌓𒀭𒈾𒀭𒇸𒀀𒉿𒈝𒀸"}, {N_ ("Cypriot"), "cypriot", "𐠀𐠜𐠍𐠚 𐠃𐠙𐠪𐠒𐠚 𐠰𐠜𐠙𐠪𐠎𐠡𐠦𐠚 𐠰𐠛 𐠅𐠮𐠣𐠚 𐠊𐠩 𐠰𐠩 𐠊𐠪𐠋𐠚𐠰𐠩"}, {N_ ("Cypro Minoan"), "cypro-minoan", "𒾙𒾚𒾛𒾜𒾝𒾞𒾟𒾠𒾡𒾢𒾣𒾤𒾥𒾦𒾧𒾨𒾩𒾪𒾫𒾬𒾭𒾮𒾯𒾰𒾱𒾲𒾳𒾴𒾵𒾶𒾷𒾸𒾹𒾺𒾻𒾼𒾽𒾾𒾿𒿀𒿁𒿂𒿃𒿄𒿅𒿆𒿇𒿈"}, {N_ ("Cyrillic"), "cyrillic", "Алая вспышка осветила силуэт зазубренного крыла."}, {N_ ("Cyrillic Extended"), "cyrillic-ext", "Видовище перед нашими очима справді вражало."}, {N_ ("Deseret"), "deseret", "𐐃𐑊 𐐸𐐷𐐭𐑋𐐲𐑌 𐐺𐐨𐐮𐑍𐑆 𐐪𐑉 𐐺𐐫𐑉𐑌 𐑁𐑉𐐨 𐐰𐑌𐐼 𐐨𐐿𐐶𐐲𐑊 𐐮𐑌"}, {N_ ("Devanagari"), "devanagari", "अंतरिक्ष यान से दूर नीचे पृथ्वी शानदार ढंग से जगमगा रही थी ।"}, {N_ ("Duployan"), "duployan", "𛱕𛱖𛱗𛱘𛱙𛱚 𛱩𛱪𛱰𛱱𛱲𛱳𛱴 𛰀𛰁𛰂𛰃𛰄𛰅𛰆 𛲔𛲕𛲖𛲗𛲘𛲙𛲜 𛰎𛰏𛰐𛰑𛰒𛰓𛰔 𛱢𛱣𛱤𛱥𛱦𛱧𛱨"}, {N_ ("Egyptian Hieroglyphs"), "egyptian-heiroglyphs", "𓈖𓆓 𓊽𓉐𓉐 𓈖𓏲𓇯𓂝𓏴𓃾 𓉐𓃾𓂝𓃻𓁶𓃾 𓌓𓁶𓌓𓆓𓂝𓃾 𓌅𓂧𓌅𓀠 𓀠𓇯𓈖𓃾 𓇯𓂧"}, {N_ ("Elbasan"), "elbasan", "𐔟 𐔁𐔀 𐔒𐔎𐔇𐔔 𐔏𐔇 𐔠𐔖 𐔀𐔝𐔎𐔇 𐔟 𐔒𐔁𐔟𐔛𐔌𐔔𐔈 𐔄𐔍𐔝𐔈 𐔝𐔈 𐔗𐔎𐔇𐔐𐔐𐔈"}, {N_ ("Elymaic"), "elymaic", "𐿬𐿭 𐿮𐿡𐿡 𐿬𐿥𐿣𐿩𐿵𐿠 𐿡𐿠𐿩𐿲𐿳𐿠 𐿴𐿳𐿴𐿭𐿩𐿠 𐿫𐿪𐿫𐿤 𐿤𐿣𐿬𐿠 𐿣𐿪"}, {N_ ("Ethiopic"), "ethiopic", "፤ ምድርም ሁሉ በአንድ ቋንቋና በአንድ ንግግር ነበረች።"}, {N_ ("Georgian"), "georgian", "ვინაიდან ადამიანთა ოჯახის ყველა წევრისათვის"}, {N_ ("Glagolitic"), "glagolitic", "ⰲⱐⱄⰻ ⰱⱁ ⰾⱓⰴⰻⰵ ⱃⱁⰴⱔⱅⱏ ⱄⱔ ⱄⰲⱁⰱⱁⰴⱐⱀⰻ ⰻ ⱃⰰⰲⱐⱀⰻ"}, {N_ ("Gothic"), "gothic", "𐌰𐌻𐌻𐌰𐌹 𐌼𐌰𐌽𐌽𐌰 𐍆𐍂𐌴𐌹𐌷𐌰𐌻𐍃 𐌾𐌰𐌷 𐍃𐌰𐌼𐌰𐌻𐌴𐌹𐌺𐍉 𐌹𐌽 𐍅𐌰𐌹𐍂𐌸𐌹𐌳𐌰𐌹"}, {N_ ("Grantha"), "grantha", "𑌯𑌤𑍍𑌰 𑌜𑌗𑌤𑌿 𑌶𑌾𑌨𑍍𑌤𑌿𑌨𑍍𑌯𑌾𑌯𑌸𑍍𑌵𑌾𑌤𑌨𑍍𑌤𑍍𑌰𑍍𑌯𑌾𑌣𑌾𑌂 𑌆𑌧𑌾𑌰𑌃 𑌮𑌾𑌨𑌵𑌪𑌰𑌿𑌵𑌾𑌰𑌸𑍍𑌯 𑌸𑌰𑍍𑌵𑍇𑌷𑌾𑌮𑌪𑌿"}, {N_ ("Greek"), "greek", "Ήταν απλώς θέμα χρόνου."}, {N_ ("Greek Extended"), "greek-ext", "Ήταν απλώς θέμα χρόνου."}, {N_ ("Gujarati"), "gujarati", "અમને તેની જાણ થાય તે પહેલાં જ, અમે જમીન છોડી દીધી હતી."}, {N_ ("Gunjala Gondi"), "gunjala-gondi", "𑵺𑶌𑵭 𑶀𑶌𑵭𑵳 𑶅𑶍𑵽-𑵶𑶂𑶗𑵰𑶋"}, {N_ ("Gurmukhi"), "gurmukhi", "ਸਵਾਲ ਸਿਰਫ਼ ਸਮੇਂ ਦਾ ਸੀ।"}, {N_ ("Hanifi Rohingya"), "hanifi-rohingya", "𐴀𐴞𐴕𐴐𐴝𐴦𐴕 𐴁𐴠𐴒𐴧𐴟𐴕 𐴀𐴝𐴎𐴝𐴊𐴢 𐴀𐴝𐴌 𐴀𐴠𐴑𐴧𐴟 𐴉𐴟𐴥𐴖𐴝𐴙𐴕𐴝"}, {N_ ("Hanunoo"), "hanunoo", "ᜣᜩᜳᜥ ᜩᜰᜳᜧᜳᜥ ᜵ ᜣᜲᜦ ᜢᜨ ᜫᜣᜪ ᜵"}, {N_ ("Hatran"), "hatran", "𐣬𐣭 𐣮𐣡𐣡 𐣬𐣥𐣣𐣩𐣵𐣠 𐣡𐣠𐣩𐣲𐣣𐣠 𐣴𐣣𐣴𐣭𐣩𐣠 𐣫𐣪𐣫𐣤 𐣤𐣣𐣬𐣠 𐣣𐣪"}, {N_ ("Hebrew"), "hebrew", "אז הגיע הלילה של כוכב השביט הראשון."}, {N_ ("Imperial Aramaic"), "imperial-aramaic", "𐡌𐡍 𐡎𐡁𐡁 𐡌𐡅𐡃𐡉𐡕𐡀 𐡁𐡀𐡉𐡒𐡓𐡀 𐡔𐡓𐡔𐡍𐡉𐡀 𐡋𐡊𐡋𐡄 𐡄𐡃𐡌𐡀 𐡃𐡊"}, {N_ ("Indic Siyaq Numbers"), "indic-siyac-numbers", "𞲝𞲞𞲟𞲠𞲡 𞲆𞲇𞲈𞲉𞲊𞲋𞲌 𞱸𞲂𞲃𞲄𞲅 𞲘 ا٠١٢٣٤٥"}, {N_ ("Inscriptional Pahlavi"), "inscriptional-pahlavi", "𐭬𐭭 𐭮𐭡𐭡 𐭬𐭥𐭣𐭩𐭲𐭠 𐭡𐭠𐭩𐭬𐭥𐭠 𐭱𐭥𐭱𐭭𐭩𐭠 𐭫𐭪𐭫𐭤 𐭤𐭣𐭬𐭠 𐭣𐭪"}, {N_ ("Inscriptional Parthian"), "inscriptional-parthian", "𐭌𐭍 𐭎𐭁𐭁 𐭌𐭅𐭃𐭉𐭕𐭀 𐭁𐭀𐭉𐭒𐭓𐭀 𐭔𐭓𐭔𐭍𐭉𐭀 𐭋𐭊𐭋𐭄 𐭄𐭃𐭌𐭀 𐭃𐭊"}, {N_ ("Japanese"), "japanese", "彼らの機器や装置はすべて生命体だ。"}, {N_ ("Javanese"), "javanese", "꧋ꦩꦤꦶꦩ꧀ꦧꦁꦩꦤꦮꦲꦏ꧀ꦲꦏ꧀ꦲꦸꦩꦠ꧀ꦩꦤꦸꦁꦱꦥꦼꦂꦭꦸꦲꦤ꧀ꦠꦸꦏ꧀ꦥꦔꦪꦺꦴꦩ꧀ꦩꦤ꧀ꦏꦤ꧀ꦛꦶꦥꦿꦤꦠꦤ꧀ꦭꦤ꧀ꦠꦠꦤꦤ"}, {N_ ("Kaithi"), "kaithi", "𑂉𑂍 𑂃𑂠𑂧𑂷 𑂍𑂵 𑂠𑂴 𑂥𑂵𑂗𑂰 𑂩𑂯𑂪 𑃀"}, {N_ ("Kannada"), "kannada", "ಇದು ಕೇವಲ ಸಮಯದ ಪ್ರಶ್ನೆಯಾಗಿದೆ."}, {N_ ("Kharoshthi"), "kharoshthi", "𐨩𐨟𐨿𐨪 𐨗𐨒𐨟𐨁 𐨭𐨌𐨣𐨿𐨟𐨁𐨣𐨿𐨩𐨌𐨩𐨯𐨿𐨬𐨌𐨟𐨣𐨿𐨟𐨿𐨪𐨿𐨩𐨌𐨞𐨌𐨎 𐨀𐨌𐨢𐨌𐨪𐨏 𐨨𐨌𐨣𐨬𐨤𐨪𐨁𐨬𐨌𐨪𐨯𐨿𐨩 𐨯𐨪𐨿𐨬𐨅𐨮𐨌𐨨𐨤𐨁"}, {N_ ("Kawi"), "kawi", "𑽅𑽎𑽅𑼆𑼯𑼒𑽉𑽑𑽒𑽑𑽔𑽉𑼙𑽂𑼫𑼾𑼰𑽂𑼝𑼪𑼴𑼱𑽉"}, {N_ ("Kayah Li"), "kayah-li", "ꤒꤟꤢꤧ꤬ꤚꤤ꤬ꤒꤟꤢꤧ꤬ꤊꤛꤢ꤭ ꤘꤣ ꤠꤢ꤭ ꤞꤢꤧꤐꤟꤢꤦ ꤟꤢꤩꤏꤥ꤬ ꤔꤟꤢꤧ꤬ ꤔꤌꤣ꤬ꤗꤢ꤬ ꤢ꤬ꤥ꤬"}, {N_ ("Khmer"), "khmer", "ខ្ញុំបានមើលព្យុះ ដែលមានភាពស្រស់ស្អាតណាស់ ប៉ុន្តែគួរឲ្យខ្លាច"}, {N_ ("Khojki"), "khojki", "𑈩𑈤𑈨𑈦𑈬𑈺𑈀𑈞𑈩𑈬𑈞𑈺𑈁𑈐𑈶𑈬𑈛𑈺𑈂𑈺𑈀𑈐𑈶𑈙𑈺𑈂𑈺𑈪𑈈𑈶𑈞𑈺𑈑𑈥𑈺𑈪𑈨𑈬𑈧𑈥𑈺𑈈𑈀𑈞"}, {N_ ("Khudawadi"), "khudawadi", "𑋝𑋗𑋛𑋙𑋠 𑊰𑋑𑋝𑋠𑋑 𑊱𑋂𑋩𑋠𑋏 𑊲 𑊰𑋂𑋩𑋍 𑊲 𑋞𑊺𑋩𑋑 ڄ𑋘 𑋞𑋛𑋠𑋚𑋘 𑊺𑊰𑋑"}, {N_ ("Korean"), "korean", "그들의 장비와 기구는 모두 살아 있다."}, {N_ ("Lao"), "lao", "ດ້ວຍເຫດວ່າ ການຮັບຮູ້ກຽດຕິສັກອັນມີປະຈຳຢູ່ຕົວບຸກຄົນໃນວົງສະກຸນຂອງມະນຸດທຸກ" \
"ໆຄົນ"}, {N_ ("Lepcha"), "lepcha", "᰿᱁᰿ ᰂᰦᰮᰛᰧᰶᰕᰩ ᰂᰦᰮᰛᰧᰶ ᰣᰦᰜᰬᰮᰌᰨ ᰌᰧᰜᰬ"}, {N_ ("Latin"), "latin", "Almost before we knew it, we had left the ground."}, {N_ ("Latin Extended"), "latin-ext", "Almost before we knew it, we had left the ground."}, {N_ ("Limbu"), "limbu", "ᤂᤧ᤹ᤕᤥ ᤁᤢᤴᤔᤢᤵᤔᤠᤸᤗᤧ ᤆᤠ ᤆᤥᤃᤢᤀᤠᤱ ᤐᤡᤖᤢ ᤜᤧᤰᤁᤩᤠᤱ ᤆᤠ ᤛᤢᤶᤒᤠᤰᤆᤧᤴ ᤑᤠ ᤏᤰᤁᤡᤸᤗᤧ ᤛᤢᤶᤒ" \
"ᤠᤰ ᤐᤡᤖᤢᤖᤧᤳᤇ॥"}, {N_ ("Linear A"), "linear-a", "𐘀𐘁𐘂𐘃𐘄𐘅𐘆 𐘣𐘤𐘥𐘦𐘧𐘨𐘩 𐚯𐚰𐚱𐚲𐚳𐚴𐚵 𐜟𐜠𐜡𐜢𐜣𐜤𐜥 𐚨𐚩𐚪𐚫𐚬𐚭𐚮 𐛋𐛌𐛍𐛎𐛏𐛐𐛑"}, {N_ ("Linear B"), "linear-b", "𐀴𐀪𐀡𐀆𐄀𐁁𐀐𐀄𐄀𐀐𐀩𐀯𐀍𐄀𐀸𐀐 𐃠 𐄈 𐀴𐀪𐀡𐄀𐀁𐀕𐄀𐀡𐀆𐄀𐀃𐀺𐀸 𐃠 𐄇 𐀴𐀪𐀡𐄀𐀐𐀩𐀯𐀍𐄀𐀸𐀐𐄀𐀀𐀢𐄀𐀐𐀏𐀄𐀕𐀜"}, {N_ ("Lisu"), "lisu", "ꓞꓳ ꓘꓹ ꓠꓯꓹꓼ ꓢꓲ ꓫꓬ ꓟ ꓙ ꓖꓴ ꓗꓪ ꓟꓬꓱꓽ ꓧꓳꓽ ꓢꓴ ꓠꓬ꓾"}, {N_ ("Lycian"), "lycian", "𐊁𐊂𐊚𐊑𐊏𐊚 𐊓𐊕𐊑𐊏𐊀𐊇𐊒 𐊎𐊚𐊏 𐊁𐊓𐊕𐊑𐊏𐊀𐊇𐊀𐊗𐊚 𐊛𐊀𐊏𐊀𐊅𐊀𐊈𐊀 𐊛𐊕𐊓𐊓𐊆𐊍𐊀𐊅𐊆"}, {N_ ("Lydian"), "lydian", "𐤠𐤨𐤯𐤦𐤫 𐤫𐤵𐤲𐤦𐤳 𐤲𐤤𐤩𐤷𐤨 𐤱𐤶𐤫𐤳𐤷𐤦𐤱𐤦𐤣 𐤱𐤠𐤨𐤪𐤷 𐤠𐤭𐤯𐤦𐤪𐤰𐤮"}, {N_ ("Mahajani"), "mahajani", "𑅙𑅒𑅧𑅕𑅑 𑅬𑅐𑅧𑅯 𑅨𑅭𑅑𑅯𑅐𑅭 𑅕𑅓 𑅰𑅫𑅑 𑅰𑅥𑅰𑅛𑅔𑅧 𑅕𑅓 𑅛𑅧𑅬𑅛𑅐𑅣 𑅗𑅒𑅭𑅯 𑅒𑅭 𑅰𑅬𑅐𑅧"}, {N_ ("Malayalam"), "malayalam", "അവരുടെ എല്ലാ ഉപകരണങ്ങളും യന്ത്രങ്ങളും ഏതെങ്കിലും രൂപത്തിൽ സജീവമാണ്."}, {N_ ("Mandaic"), "mandaic", "ࡕࡉࡁࡉࡋ ࡓࡌࡀ ࡀࡐࡀࡓࡀ ࡀࡎࡀࡓ ࡏࡅࡕࡓࡀ ࡂࡁࡓࡀ ࡍࡄࡅࡓࡀ ࡁࡓࡀࡄࡉࡌ"}, {N_ ("Manichaean"), "manichaean", "𐫖𐫗 𐫘𐫁𐫁 𐫖𐫇𐫅𐫏𐫤𐫀 𐫁𐫀𐫏𐫞𐫡𐫀 𐫢𐫡𐫢𐫗𐫏𐫀 𐫓𐫐𐫓𐫆 𐫆𐫅𐫖𐫀 𐫅𐫐"}, {N_ ("Math"), "math", "𝞉𝞩𝟃𞻰⟥⦀⦁ 𝚢𝚣𝚤𝖿𝗀𝗁𝗂 𝑻𝑼𝑽𝗔𝗕𝗖𝗗 ϑϕϰϱϵℊℎ ⊰⊱⊲⊳⊴⊵⫕ 𞹴𞹵𞹶𞹷𞹹𞹺𞹻"}, {N_ ("Marchen"), "marchen", "𑲉𑱺𑲪 𑱸𑱴𑱺𑲱 𑲌𑲰𑱽𑲚𑲱𑱽𑲩𑲰𑲉𑲍𑲥𑲰𑱺𑱽𑲚𑲪𑲩𑲰𑱽𑲰𑲵 𑲏𑲰𑱼𑲮𑲰𑲊𑲎 𑲁𑲰𑱽𑲅𑱾𑲊𑲱𑲅𑲰𑲊𑲍𑲩 𑲍𑲊𑲥𑲳𑲌𑲰𑲁𑱾𑲱 𑲍𑱼𑲍𑲩𑲰𑱽𑲰𑲵"}, {N_ ("Masaram Gondi"), "masaram-gondi", "𑴫𑴴𑴓𑴧𑴱𑴤𑵄 𑴫𑴴𑴡𑴧𑴱𑴤𑵄 𑴤𑴧𑴥𑴓𑴩𑴳𑴛𑴧𑴱𑴤𑵄 𑴫𑴫𑵅𑴥𑴩𑵅𑴧𑴱𑴤𑴧𑴱𑴤𑵄 𑴤𑴱𑴛𑴦𑴤𑵄 ॥"}, {N_ ("Mayan Numerals"), "mayan-numerals", "𝋠𝋡𝋢𝋣𝋤𝋥𝋦 𝋧𝋨𝋩𝋪𝋫𝋬𝋭"}, {N_ ("Medefaidrin"), "medefaidrin", "𖹀𖹦𖹻𖹧 𖹻 𖹫𖹠𖹦𖹤 𖹃𖹣𖹫 𖹤𖹨 𖹦𖹻𖹫𖹤 𖹣𖹫 𖹤𖹠 𖹛𖹫 𖹧𖹨𖹫𖹤𖹣 𖹫𖹤𖹣𖹧𖹨"}, {N_ ("Meetei Mayek"), "meetei-mayek", "ꯃꯤꯑꯣꯏꯕ ꯈꯨꯗꯤꯡꯃꯛ ꯄꯣꯛꯄ ꯃꯇꯝꯗ ꯅꯤꯡꯇꯝꯃ ꯑꯃꯗꯤ ꯏꯖꯖꯠꯑꯃꯁꯨꯡ ꯍꯛ"}, {N_ ("Mende Kikakui"), "mende-kikakui", "𞡥𞠖𞢻𞠢𞠮𞠣 𞢣𞠽 𞡅 𞡄 𞠺 𞡈 𞡗 𞢰𞠎 𞡔 𞡪, 𞡅𞠧 𞡄 𞡥𞢻𞠤 𞡖𞠢 𞠄𞠦"}, {N_ ("Meroitic"), "meroitic", "𐦣𐦤𐦥𐦦𐦮𐦯𐦰 𐦀𐦁𐦂𐦃𐦄𐦅𐦆 𐦱𐦲𐦳𐦴𐦵𐦶𐦷 𐦾𐦿𐦧𐦨𐦩𐦪𐦫 𐦑𐦒𐦓𐦔𐦠𐦡𐦢 𐦜𐦝𐦞𐦟𐦎𐦏𐦐"}, {N_ ("Miao"), "miao", "𖼐𖽪𖾐 𖼞𖽪 𖼷𖽷 𖽐𖼊𖽪𖾏 𖼷𖽷 𖼊𖽡 𖽐𖼞𖽻𖾏 𖼽𖽘 𖼮𖽷𖾑 𖼨𖽑𖽪𖾐 𖽐𖼊𖽪𖾏 𖼎𖽻 𖼡𖽑𖽔𖾑 𖼀𖽱 𖼎𖽻 𖼡𖽻𖾐 𖽐𖼊𖽪𖾏 𖼀𖽡𖾐 " \
"𖼳𖽔𖾐"}, {N_ ("Modi"), "modi", "𑘕𑘿𑘧𑘰 𑘀𑘨𑘿𑘞𑘲 𑘦𑘰𑘡𑘪 𑘎𑘳𑘘𑘳𑘽𑘪𑘰𑘝𑘲𑘩 𑘭𑘨𑘿𑘪 𑘪𑘿𑘧𑘎𑘿𑘝𑘲𑘽𑘓𑘲 𑘭𑘿𑘪𑘰𑘥𑘰𑘪𑘲𑘎 𑘢𑘿𑘨𑘝𑘲𑘬𑘿𑘙𑘰 𑘪"}, {N_ ("Mongolian"), "mongolian", "ᠬᠦᠮᠦᠨ ᠪᠦᠷ ᠲᠥᠷᠥᠵᠦ ᠮᠡᠨᠳᠡᠯᠡᠬᠦ ᠡᠷᠬᠡ ᠴᠢᠯᠥᠭᠡ ᠲᠡᠢ᠂"}, {N_ ("Mro"), "mro", "𖩏𖩖𖩔𖩆𖩊 𖩗𖩖𖩊 𖩍𖩖𖩌 𖩎𖩆𖩁 𖩋𖩖 𖩍𖩖𖩌𖩯"}, {N_ ("Multani"), "multani", "𑊡𑊖𑊢 𑊌𑊆𑊖 𑊥𑊚𑊖𑊚𑊡𑊡𑊥𑊤𑊖𑊚𑊖𑊢𑊡𑊕𑊚 𑊀𑊙𑊢𑊦 𑊠𑊚𑊤𑊛𑊢𑊤𑊢𑊥𑊡 𑊥𑊢𑊤𑊥𑊠𑊛"}, {N_ ("Music"), "music", "𝄆 𝄙𝆏 𝅗𝅘𝅥𝅘𝅥𝅯𝅘𝅥𝅱 𝄞𝄟𝄢 𝄾𝄿𝄎 𝄴 𝄶𝅁 𝄭𝄰 𝇛𝇜 𝄊 𝄇 𝀸𝀹𝀺𝀻𝀼𝀽 𝈀𝈁𝈂𝈃𝈄𝈅"}, {N_ ("Myanmar"), "myanmar", "သူတို့ရဲ့ စက်ပစ္စည်းတွေ၊ ကိရိယာတွေ အားလုံး အသက်ရှင်ကြတယ်။"}, {N_ ("Nabataean"), "nabataean", "𐢓𐢔 𐢖𐢃𐢂 𐢓𐢈𐢅𐢍𐢞𐢀 𐢃𐢁𐢍𐢚𐢛𐢀 𐢝𐢛𐢝𐢕𐢍𐢀 𐢑𐢏𐢑𐢆 𐢇𐢅𐢓𐢀 𐢅𐢏"}, {N_ ("Nag Mundari"), "nag-mundari", "𞓛𞓐𞓗𞓤𞓨 𞓙𞓐𞓡𞓐𞓢𞓐 𞓢𞓤 𞓧𞓕𞓨𞓣𞓕𞓔 𞓐𞓡𞓐𞓐 𞓕𞓢𞓝𞓚𞓓𞓕𞓣 𞓢𞓐 𞓣𞓤𞓕𞓙 𞓑𞓕𞓚𞓝𞓚𞓗𞓕𞓗𞓕𞓝 𞓣𞓤 𞓖𞓕𞓨𞓕𞓧"}, {N_ ("Nandinagari"), "nandinagari", "𑧇𑦽𑧠𑧈 𑦵𑦰𑦽𑧒 𑧋𑧑𑧞𑦽𑧒𑧁𑧠𑧇𑧑𑧇𑧍𑧠𑧊𑧑𑦽𑧞𑦽𑧠𑧈𑧠𑧇𑧑𑦼𑧑𑧞 𑦡𑧀𑧑𑧈𑧟 𑧆𑧑𑧁𑧊𑧂𑧈𑧒𑧊𑧑𑧈𑧍𑧠𑧇 𑧍𑧈𑧠𑧊𑧚𑧌𑧑𑧆𑧂𑧒"}, {N_ ("New Tai Lue"), "new-tai-lue", "ᦝᧂᦑᦸᦰ ᦍᦸᧆᦑᦲᧈᦷᦢᦆᧄ ᦅᧀᦂᦱᧂᦐᦸᧂ ᦂᦱᧁ ᦙᦸᧃᦟᦱᧆᦓᧄᧉ ᦶᦙᧈᦷᦎᦶᦂᧄᧉ"}, {N_ ("Newa"), "newa", "𑐳𑐎𑐮𑐾𑑄 𑐩𑐣𑐹𑐟 𑐳𑑂𑐰𑐟𑐣𑑂𑐟𑑂𑐬 𑐰 𑐖𑑂𑐰𑐮𑐶𑐖𑑂𑐰𑑅 𑐁𑐟𑑂𑐩𑐳𑐩𑑂𑐩𑐵𑐣 𑐰 𑐰𑐵𑑄 𑐡𑐂𑐎𑐠𑑄 𑐧𑐸𑐂"}, {N_ ("Nüshu"), "nushu", "𛇤𛅰𛈕𛅸𛇃𛆤𛈕 𛇤𛅰𛈕𛅸𛇃𛆤𛈕 𛇤𛅰𛈕𛅸𛇃𛆤𛈕 𛇤𛅰𛈕𛅸𛇃𛆤𛈕 𛇤𛅰𛈕𛅸𛇃𛆤𛈕 "}, {N_ ("Ogham"), "ogham", "᚛ᚌᚔᚚ ᚓ ᚈᚔᚄᚓᚇ ᚔᚅ ᚃᚐᚔᚇᚉᚆᚓ᚜ ᚛ᚇᚘᚐ ᚋᚁᚐ ᚌᚐᚄᚉᚓᚇᚐᚉᚆ᚜"}, {N_ ("Ol Chiki"), "ol-chiki", "ᱡᱚᱛᱚ ᱞᱮᱠᱟᱱᱚ ᱢᱳᱱᱚ ᱟᱨᱚ ᱚᱫᱷᱤᱠᱟᱨᱚ ᱨᱮᱭᱟᱠᱚ ᱟᱫᱷᱟᱨᱚ ᱨᱮ ᱢᱩᱪᱳᱛᱚ ᱫᱷᱟᱵᱤᱪᱚ ᱥᱣᱚᱛᱚᱱᱛᱨ" \
"ᱚ"}, {N_ ("Old Hungarian"), "old-hungarian", "𐲪𐲢𐲙𐲔 𐲥𐲬𐲖𐲦𐲤𐲦𐲬𐲖 𐲌𐲛𐲍𐲮𐲀𐲙 𐲐𐲢𐲙𐲔 𐲯𐲢𐲞𐲦  𐲥𐲀𐲯𐲎 𐲥𐲦𐲙𐲇𐲞𐲂𐲉 𐲘𐲀𐲨𐲤 𐲒𐲀𐲙𐲛𐲤 𐲤𐲨𐲦𐲙 𐲓𐲛𐲮𐲀𐲆 𐲆𐲐𐲙" \
"𐲀𐲖𐲦𐲔 𐲘𐲀𐲨𐲀𐲤𐲘𐲤𐲦𐲢 𐲍𐲢𐲍𐲗𐲘𐲤𐲦𐲢𐲆𐲐𐲙𐲀𐲖𐲦𐲀𐲔 𐲍 𐲐𐲒 𐲀 𐲤 𐲐 𐲗 𐲗 𐲖𐲦 𐲀"}, {N_ ("Old Italic"), "old-italic", "𐌆𐌀𐌌𐌈𐌉𐌂 𐌈𐌖𐌍 𐌗𐌀𐌓𐌖𐌍 𐌘𐌄𐌓𐌔𐌖 𐌆𐌀𐌌𐌀𐌈𐌉 𐌀𐌉𐌔 𐌑𐌀𐌔 𐌐𐌖𐌉𐌀"}, {N_ ("Old North Arabian"), "old-north-arabian", "𐪃𐪌 𐪊𐪈𐪈 𐪃𐪅𐪕𐪚𐪗𐪑 𐪈𐪑𐪚𐪄𐪇𐪑 𐪏𐪇𐪏𐪌𐪚𐪑 𐪁𐪋𐪁𐪀 𐪀𐪕𐪃𐪑 𐪕𐪋"}, {N_ ("Old Permic"), "old-permic", "𐍐𐍑𐍒𐍓𐍔𐍕𐍖 𐍞𐍟𐍠𐍡𐍢𐍣𐍤 𐍥𐍦𐍧𐍨𐍩𐍪𐍫 𐍗𐍘𐍙𐍚𐍛𐍜𐍝 𐍬𐍭𐍮𐍯𐍰𐍱𐍲"}, {N_ ("Old Persian"), "old-persian", "𐎧𐏁𐎹𐎠𐎼𐏁𐎠𐏐𐎧𐏁𐎹𐎰𐎡𐎹𐏐𐎺𐏀𐎼𐎣𐏐𐎧𐏁𐎠𐎹𐎰𐎡𐎹𐏐"}, {N_ ("Old Sogdian"), "old-sogdian", "𐼍𐼏 𐼑𐼂𐼃 𐼍𐼇𐼌𐼊𐼚𐼁 𐼂𐼀𐼊𐼋𐼘𐼁 𐼙𐼘𐼙𐼎𐼊𐼁 𐼌𐼋𐼌𐼅 𐼆𐼌𐼍𐼁 𐼌𐼋"}, {N_ ("Old South Arabian"), "old-south-arabian", "𐩣𐩬 𐩪𐩨𐩨 𐩣𐩥𐩵𐩺𐩩𐩱 𐩨𐩱𐩺𐩤𐩧𐩱 𐩦𐩧𐩦𐩬𐩺𐩱 𐩡𐩫𐩡𐩠 𐩠𐩵𐩣𐩱 𐩵𐩫"}, {N_ ("Old Turkic"), "old-turkic", "𐱅𐰇𐰼𐰜 𐰆𐰍𐰔 𐰋𐰏𐰠𐰼𐰃 𐰉𐰆𐰑𐰣 𐱁𐰃𐰓𐰤 𐰇𐰔𐰀 𐱅𐰭𐰼𐰃 𐰉𐰽𐰢𐰽𐰺 𐰽𐰺𐰀 𐰘𐰃𐰼 𐱅𐰠𐰃𐰤𐰢𐰾𐰼"}, {N_ ("Osage"), "osage", "𐒻𐓲𐓣𐓤𐓪 𐓰𐓘͘𐓤𐓘 𐓷𐓣͘ 𐓘𐓵𐓟 𐓘𐓬𐓘 𐓤𐓘𐓸𐓘 𐓤𐓯𐓣 𐓘𐓵𐓟 𐓘𐓬𐓘 𐓪𐓬𐓸𐓘"}, {N_ ("Osmanya"), "osmanya", "𐒛𐒆𐒖𐒒𐒖𐒔𐒖 𐒊𐒖𐒑𐒑𐒛𐒒𐒂𐒕𐒈 𐒓𐒚𐒄𐒓 𐒊𐒖𐒉𐒛 𐒘𐒈𐒖𐒌𐒝 𐒄𐒙𐒇 𐒖𐒔"}, {N_ ("Oriya"), "oriya", "ଏହା କେବଳ ଏକ ସମୟ କଥା ହିଁ ଥିଲା."}, {N_ ("Pahawh Hmong"), "pahawh-hmong", "𖬑𖬦𖬰 𖬇𖬰𖬧𖬵 𖬁𖬲𖬬 𖬇𖬲𖬤 𖬓𖬲𖬞 𖬐𖬰𖬦 𖬉 𖬘𖬲𖬤 𖬀𖬰𖬝𖬵 𖬔𖬟𖬰 𖬂𖬲𖬤𖬵 𖬅𖬲𖬨𖬵 𖬓𖬲𖬥𖬰 𖬄𖬲𖬟"}, {N_ ("Palmyrene"), "palmyrene", "𐡬𐡭 𐡯𐡡𐡡 𐡬𐡥𐡣𐡩𐡶𐡠 𐡡𐡠𐡩𐡳𐡴𐡠 𐡵𐡴𐡵𐡭𐡩𐡠 𐡫𐡪𐡫𐡤 𐡤𐡣𐡬𐡠 𐡣𐡪"}, {N_ ("Pau Cin Hau"), "pau-cin-hau", "𑫢𑫪𑫫𑫬𑫭𑫮𑫯 𑫸𑫱𑫲𑫳𑫴𑫵𑫶 𑫔𑫜𑫝𑫞𑫟𑫠𑫡 𑫀𑫁𑫂𑫃𑫄𑫅𑫆 𑫰𑫕𑫖𑫗𑫘𑫙𑫚 𑫣𑫤𑫥𑫦𑫧𑫨𑫩"}, {N_ ("Phags Pa"), "phags-pa", "ꡗ ꡈꡱ ᠂ ꡒ ꡂ ꡈꡞ ᠂ ꡚꡖꡋ ꡈꡞꡋꡨꡖ ꡗꡛꡧꡖ ꡈꡋ ꡈꡱꡨꡖ ꡳꡬꡖ"}, {N_ ("Phoenician"), "phoenician", "𐤌𐤍 𐤎𐤁𐤁 𐤌𐤅𐤃𐤉𐤕𐤀 𐤁𐤀𐤉𐤒𐤓𐤀 𐤔𐤓𐤔𐤍𐤉𐤀 𐤋𐤊𐤋𐤄 𐤄𐤃𐤌𐤀 𐤃𐤊"}, {N_ ("Psalter Pahlavi"), "psalter-pahlavi", "𐮋𐮌 𐮍𐮁𐮁 𐮋𐮅𐮃𐮈𐮑𐮀 𐮁𐮀𐮈𐮋𐮅𐮀 𐮐𐮅𐮐𐮌𐮈𐮀 𐮊𐮉𐮊𐮄 𐮄𐮃𐮋𐮀 𐮃𐮉"}, {N_ ("Rejang"), "rejang", "ꤰꥈꤳꥎ ꤳꥈꥐ ꤾꥁꥉꥑ ꤸꥎꥑꤴꥉꤰ ꤳ꥓ꤸꥈꥆꥐ ꥁꥋꤰ꥓ꥁꥋꤰ꥓ ꤴꥎ ꤼ꥓ꤽꥊ  ꤰꥈꤳꥎ ꤵꤱꥇꥒꤰ꥓ꤷꥒ ꥆꤰꥎꥒ ꤶꥉꤰꥉꥑ"}, {N_ ("Runic"), "runic", "ᚨᛚᛚᚨᛁ ᛗᚨᚾᚾᚨ ᚠᚱᛖᛁᚺᚨᛚᛋ ᛃᚨᚺ ᛋᚨᛗᚨᛚᛖᛁᚲᛟ ᛁᚾ ᚹᚨᛁᚱᚦᛁᛞᚨᛁ"}, {N_ ("Samaritan"), "samaritan", "ࠌࠍ ࠎࠁࠁ ࠌࠅࠃࠉࠕࠀ ࠁࠀࠉࠒࠓࠀ ࠔࠓࠔࠍࠉࠀ ࠋࠊࠋࠄ ࠄࠃࠌࠀ ࠃࠊ"}, {N_ ("Saurashtra"), "saurashtra", "ꢦꢶꢪ꣄ꢫꢳꢸ ꢚꢵꢞꢸ ꢥꢷꢞꢵꢪ꣄ ꢫꢶꢭ꣄ꢭꣁ ꢐꢠ꣄ꢜꢾꢥꣁ ꢨꢶꢱꢶꢬꢾꢱ꣄,"}, {N_ ("Sharada"), "sharada", "𑆪𑆠𑇀𑆫 𑆘𑆓𑆠𑆴 𑆯𑆳𑆤𑇀𑆠𑆴𑆤𑇀𑆪𑆳𑆪𑆱𑇀𑆮𑆳𑆠𑆤𑇀𑆠𑇀𑆫𑇀𑆪𑆳𑆟𑆳𑆁 𑆄𑆣𑆳𑆫𑆂 𑆩𑆳𑆤𑆮𑆥𑆫𑆴𑆮𑆳𑆫𑆱𑇀𑆪 𑆱𑆫𑇀𑆮𑆼𑆰𑆳𑆩𑆥𑆴"}, {N_ ("Shavian"), "shavian", "𐑢𐑺𐑨𐑟 𐑮𐑧𐑒𐑩𐑜𐑯𐑦𐑖𐑩𐑯 𐑝 𐑞 𐑦𐑯𐑣𐑧𐑮𐑩𐑯𐑑 𐑛𐑦𐑜𐑯𐑦𐑑𐑦 𐑯 𐑝"}, {N_ ("Siddham"), "siddham", "𑖧𑖝𑖿𑖨 𑖕𑖐𑖝𑖰 𑖫𑖯𑖡𑖿𑖝𑖰𑖡𑖿𑖧𑖯𑖧𑖭𑖿𑖪𑖯𑖝𑖡𑖿𑖝𑖿𑖨𑖿𑖧𑖯𑖜𑖯𑖽 𑖁𑖠𑖯𑖨𑖾 𑖦𑖯𑖡𑖪𑖢𑖨𑖰𑖪𑖯𑖨𑖭𑖿𑖧 𑖭𑖨𑖿𑖪𑖸𑖬𑖯𑖦𑖢𑖰"}, {N_ ("SignWriting"), "signwriting", "𝧿𝨊𝡝𝪜𝦦𝪬𝡝𝪩𝡝𝪡𝤅"}, {N_ ("Sinhala"), "sinhala", "එය කාලය පිළිබඳ ප්‍රශ්නයක් පමණක් විය."}, {N_ ("Sogdian"), "sogdian", "𐼺𐼻 𐼼𐼱𐼱 𐼺𐼴𐼹𐼷𐽂𐼰 𐼱𐼰𐼷𐼸𐽀𐼰 𐽁𐽀𐽁𐼻𐼷𐼰 𐽄𐼸𐽄𐼳 𐼳𐼹𐼺𐼰 𐼹𐼸"}, {N_ ("Sora Sompeng"), "sora-sompeng", "𑃜𑃑𑃝 𑃠𑃕𑃑𑃤 𑃐𑃠𑃢𑃙𑃑𑃤𑃙𑃜𑃢𑃜𑃐𑃚𑃢𑃑𑃙𑃑𑃝𑃜𑃢𑃙𑃨𑃢𑃖 𑃢𑃔𑃨𑃠𑃢𑃝𑃞"}, {N_ ("Soyombo"), "soyombo", "𑩻𑩫𑪙𑩼 𑩣𑩞𑩫𑩑 𑩿𑩛𑩯𑪙𑩫𑩑𑩯𑪙𑩻𑩛𑩻𑪁𑪙𑩾𑩛𑩫𑩯𑪙𑩫𑪙𑩼𑪙𑩻𑩛𑩪𑩛𑪖 𑩐𑩛𑩮𑩛𑩼𑪗"}, {N_ ("Sundanese"), "sundanese", "ᮚᮒᮢ ᮏᮌᮒᮤ ᮯᮔ᮪ᮒᮤᮔᮡᮚᮞ᮪ᮝᮒᮔ᮪ᮒᮢᮡᮔᮀ ᮃᮓᮛᮂ ᮙᮔᮝᮕᮛᮤᮝᮛᮞᮡ ᮞᮁᮝᮨᮯᮙᮕᮤ ᮞᮓᮞᮡᮔᮀ"}, {N_ ("Syloti Nagri"), "syloti-nagri", "ꠎꠔ꠆ꠞ ꠎꠉꠔꠤ ꠡꠣꠘ꠆ꠔꠤꠘ꠆ꠎꠣꠎꠡ꠆ꠛꠣꠔꠘ꠆ꠔ꠆ꠞ꠆ꠎꠣꠘꠣꠋ ꠀꠗꠣꠞꠢ꠆ ꠝꠣꠘꠛꠙꠞꠤꠛꠣꠞꠡ꠆ꠎ ꠡꠞ꠆ꠛꠦꠡꠣꠝꠙꠤ"}, {N_ ("Syriac"), "syriac", "ܟܠ ܒܪܢܫܐ ܒܪܝܠܗ ܚܐܪܐ ܘܒܪܒܪ ܓܘ ܐܝܩܪܐ ܘܙܕܩܐ."}, {N_ ("Tagalog"), "tagalog", "ᜀᜅ᜔ ᜎᜑᜆ᜔ ᜅ᜔ ᜆᜂᜌ᜔ ᜁᜐᜒᜈᜒᜎᜅ᜔ ᜈ ᜋᜎᜌ ᜀᜆ᜔ ᜉᜈ᜔ᜆᜌ᜔ᜉᜈ᜔ᜆᜌ᜔ ᜐ ᜃᜇᜅᜎᜈ᜔"}, {N_ ("Tagbanwa"), "tagbanwa", "ᝬᝦᝮ ᝧᝤᝦᝲ ᝰᝨᝦᝲᝨᝬᝬᝰᝯᝦᝨᝦᝮᝬᝨᝫ ᝠᝧᝮᝣ ᝫᝨᝯᝩᝮᝲᝯᝮᝰᝬ ᝰᝮᝯᝲᝰᝫᝩᝲ"}, {N_ ("Tai Le"), "tai-le", "ᥓᥣᥳ ᥞᥨᥛ ᥑᥤᥴ ᥘᥤ ᥞᥨᥛ ᥓᥨᥛᥰ ᥓᥣᥳ ᥙᥣᥰ ᥘᥤ ᥑᥤᥴ ᥙᥣᥰ"}, {N_ ("Tamil"), "tamil", "அந்திமாலையில், அலைகள் வேகமாக வீசத் தொடங்கின."}, {N_ ("Telugu"), "telugu", "ఆ రాత్రి మొదటిసారిగా ఒక నక్షత్రం నేలరాలింది."}, {N_ ("Thai"), "thai", "การเดินทางขากลับคงจะเหงา"}, {N_ ("Tibetan"), "tibetan", "ཁོ་ཚོའི་སྒྲིག་ཆས་དང་ལག་ཆ་ཡོད་ཚད་གསོན་པོ་རེད།"}, {N_ ("Vietnamese"), "vietnamese", "Bầu trời trong xanh thăm thẳm, không một gợn mây."}};

void
font_manager_google_fonts_language_data_copy (const FontManagerGoogleFontsLanguageData* self,
                                              FontManagerGoogleFontsLanguageData* dest)
{
	const gchar* _tmp0_;
	gchar* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	const gchar* _tmp4_;
	gchar* _tmp5_;
	_tmp0_ = (*self).label;
	_tmp1_ = g_strdup (_tmp0_);
	_g_free0 ((*dest).label);
	(*dest).label = _tmp1_;
	_tmp2_ = (*self).name;
	_tmp3_ = g_strdup (_tmp2_);
	_g_free0 ((*dest).name);
	(*dest).name = _tmp3_;
	_tmp4_ = (*self).sample;
	_tmp5_ = g_strdup (_tmp4_);
	_g_free0 ((*dest).sample);
	(*dest).sample = _tmp5_;
}

void
font_manager_google_fonts_language_data_destroy (FontManagerGoogleFontsLanguageData* self)
{
	_g_free0 ((*self).label);
	_g_free0 ((*self).name);
	_g_free0 ((*self).sample);
}

FontManagerGoogleFontsLanguageData*
font_manager_google_fonts_language_data_dup (const FontManagerGoogleFontsLanguageData* self)
{
	FontManagerGoogleFontsLanguageData* dup;
	dup = g_new0 (FontManagerGoogleFontsLanguageData, 1);
	font_manager_google_fonts_language_data_copy (self, dup);
	return dup;
}

void
font_manager_google_fonts_language_data_free (FontManagerGoogleFontsLanguageData* self)
{
	font_manager_google_fonts_language_data_destroy (self);
	g_free (self);
}

 G_GNUC_NO_INLINE static GType
font_manager_google_fonts_language_data_get_type_once (void)
{
	GType font_manager_google_fonts_language_data_type_id;
	font_manager_google_fonts_language_data_type_id = g_boxed_type_register_static ("FontManagerGoogleFontsLanguageData", (GBoxedCopyFunc) font_manager_google_fonts_language_data_dup, (GBoxedFreeFunc) font_manager_google_fonts_language_data_free);
	return font_manager_google_fonts_language_data_type_id;
}

GType
font_manager_google_fonts_language_data_get_type (void)
{
	static volatile gsize font_manager_google_fonts_language_data_type_id__once = 0;
	if (g_once_init_enter (&font_manager_google_fonts_language_data_type_id__once)) {
		GType font_manager_google_fonts_language_data_type_id;
		font_manager_google_fonts_language_data_type_id = font_manager_google_fonts_language_data_get_type_once ();
		g_once_init_leave (&font_manager_google_fonts_language_data_type_id__once, font_manager_google_fonts_language_data_type_id);
	}
	return font_manager_google_fonts_language_data_type_id__once;
}

static inline gpointer
font_manager_google_fonts_sample_get_instance_private (FontManagerGoogleFontsSample* self)
{
	return G_STRUCT_MEMBER_P (self, FontManagerGoogleFontsSample_private_offset);
}

FontManagerGoogleFontsSample*
font_manager_google_fonts_sample_construct (GType object_type,
                                            const gchar* lang)
{
	FontManagerGoogleFontsSample * self = NULL;
	gchar* _tmp0_;
	gchar* _tmp1_;
	g_return_val_if_fail (lang != NULL, NULL);
	self = (FontManagerGoogleFontsSample*) g_object_new (object_type, NULL);
	_tmp0_ = g_strdup_printf (_ ("No sample for %s available"), lang);
	_tmp1_ = _tmp0_;
	font_manager_google_fonts_sample_set_label (self, _tmp1_);
	_g_free0 (_tmp1_);
	font_manager_google_fonts_sample_set_name (self, "unknown");
	font_manager_google_fonts_sample_set_sample (self, _ ("Please file an issue requesting an update to available samples."));
	{
		FontManagerGoogleFontsLanguageData* entry_collection = NULL;
		gint entry_collection_length1 = 0;
		gint _entry_collection_size_ = 0;
		gint entry_it = 0;
		entry_collection = FONT_MANAGER_GOOGLE_FONTS_Languages;
		entry_collection_length1 = G_N_ELEMENTS (FONT_MANAGER_GOOGLE_FONTS_Languages);
		for (entry_it = 0; entry_it < entry_collection_length1; entry_it = entry_it + 1) {
			FontManagerGoogleFontsLanguageData entry = {0};
			entry = entry_collection[entry_it];
			{
				FontManagerGoogleFontsLanguageData _tmp2_;
				const gchar* _tmp3_;
				FontManagerGoogleFontsLanguageData _tmp4_;
				const gchar* _tmp5_;
				FontManagerGoogleFontsLanguageData _tmp6_;
				const gchar* _tmp7_;
				FontManagerGoogleFontsLanguageData _tmp8_;
				const gchar* _tmp9_;
				_tmp2_ = entry;
				_tmp3_ = _tmp2_.name;
				if (g_strcmp0 (_tmp3_, lang) != 0) {
					continue;
				}
				_tmp4_ = entry;
				_tmp5_ = _tmp4_.label;
				font_manager_google_fonts_sample_set_label (self, g_dgettext (NULL, _tmp5_));
				_tmp6_ = entry;
				_tmp7_ = _tmp6_.name;
				font_manager_google_fonts_sample_set_name (self, _tmp7_);
				_tmp8_ = entry;
				_tmp9_ = _tmp8_.sample;
				font_manager_google_fonts_sample_set_sample (self, _tmp9_);
				break;
			}
		}
	}
	return self;
}

FontManagerGoogleFontsSample*
font_manager_google_fonts_sample_new (const gchar* lang)
{
	return font_manager_google_fonts_sample_construct (FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE, lang);
}

const gchar*
font_manager_google_fonts_sample_get_label (FontManagerGoogleFontsSample* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_label;
	result = _tmp0_;
	return result;
}

void
font_manager_google_fonts_sample_set_label (FontManagerGoogleFontsSample* self,
                                            const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_google_fonts_sample_get_label (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_label);
		self->priv->_label = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_google_fonts_sample_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LABEL_PROPERTY]);
	}
}

const gchar*
font_manager_google_fonts_sample_get_name (FontManagerGoogleFontsSample* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_name;
	result = _tmp0_;
	return result;
}

void
font_manager_google_fonts_sample_set_name (FontManagerGoogleFontsSample* self,
                                           const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_google_fonts_sample_get_name (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_name);
		self->priv->_name = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_google_fonts_sample_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_NAME_PROPERTY]);
	}
}

const gchar*
font_manager_google_fonts_sample_get_sample (FontManagerGoogleFontsSample* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_sample;
	result = _tmp0_;
	return result;
}

void
font_manager_google_fonts_sample_set_sample (FontManagerGoogleFontsSample* self,
                                             const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_google_fonts_sample_get_sample (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_sample);
		self->priv->_sample = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_google_fonts_sample_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_SAMPLE_PROPERTY]);
	}
}

static void
font_manager_google_fonts_sample_class_init (FontManagerGoogleFontsSampleClass * klass,
                                             gpointer klass_data)
{
	font_manager_google_fonts_sample_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &FontManagerGoogleFontsSample_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_font_manager_google_fonts_sample_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_font_manager_google_fonts_sample_set_property;
	G_OBJECT_CLASS (klass)->finalize = font_manager_google_fonts_sample_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LABEL_PROPERTY, font_manager_google_fonts_sample_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LABEL_PROPERTY] = g_param_spec_string ("label", "label", "label", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_GOOGLE_FONTS_SAMPLE_NAME_PROPERTY, font_manager_google_fonts_sample_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_NAME_PROPERTY] = g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_GOOGLE_FONTS_SAMPLE_SAMPLE_PROPERTY, font_manager_google_fonts_sample_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_SAMPLE_PROPERTY] = g_param_spec_string ("sample", "sample", "sample", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
font_manager_google_fonts_sample_instance_init (FontManagerGoogleFontsSample * self,
                                                gpointer klass)
{
	self->priv = font_manager_google_fonts_sample_get_instance_private (self);
}

static void
font_manager_google_fonts_sample_finalize (GObject * obj)
{
	FontManagerGoogleFontsSample * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE, FontManagerGoogleFontsSample);
	_g_free0 (self->priv->_label);
	_g_free0 (self->priv->_name);
	_g_free0 (self->priv->_sample);
	G_OBJECT_CLASS (font_manager_google_fonts_sample_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
font_manager_google_fonts_sample_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (FontManagerGoogleFontsSampleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) font_manager_google_fonts_sample_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FontManagerGoogleFontsSample), 0, (GInstanceInitFunc) font_manager_google_fonts_sample_instance_init, NULL };
	GType font_manager_google_fonts_sample_type_id;
	font_manager_google_fonts_sample_type_id = g_type_register_static (G_TYPE_OBJECT, "FontManagerGoogleFontsSample", &g_define_type_info, 0);
	FontManagerGoogleFontsSample_private_offset = g_type_add_instance_private (font_manager_google_fonts_sample_type_id, sizeof (FontManagerGoogleFontsSamplePrivate));
	return font_manager_google_fonts_sample_type_id;
}

GType
font_manager_google_fonts_sample_get_type (void)
{
	static volatile gsize font_manager_google_fonts_sample_type_id__once = 0;
	if (g_once_init_enter (&font_manager_google_fonts_sample_type_id__once)) {
		GType font_manager_google_fonts_sample_type_id;
		font_manager_google_fonts_sample_type_id = font_manager_google_fonts_sample_get_type_once ();
		g_once_init_leave (&font_manager_google_fonts_sample_type_id__once, font_manager_google_fonts_sample_type_id);
	}
	return font_manager_google_fonts_sample_type_id__once;
}

static void
_vala_font_manager_google_fonts_sample_get_property (GObject * object,
                                                     guint property_id,
                                                     GValue * value,
                                                     GParamSpec * pspec)
{
	FontManagerGoogleFontsSample * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE, FontManagerGoogleFontsSample);
	switch (property_id) {
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LABEL_PROPERTY:
		g_value_set_string (value, font_manager_google_fonts_sample_get_label (self));
		break;
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_NAME_PROPERTY:
		g_value_set_string (value, font_manager_google_fonts_sample_get_name (self));
		break;
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_SAMPLE_PROPERTY:
		g_value_set_string (value, font_manager_google_fonts_sample_get_sample (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_font_manager_google_fonts_sample_set_property (GObject * object,
                                                     guint property_id,
                                                     const GValue * value,
                                                     GParamSpec * pspec)
{
	FontManagerGoogleFontsSample * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE, FontManagerGoogleFontsSample);
	switch (property_id) {
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LABEL_PROPERTY:
		font_manager_google_fonts_sample_set_label (self, g_value_get_string (value));
		break;
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_NAME_PROPERTY:
		font_manager_google_fonts_sample_set_name (self, g_value_get_string (value));
		break;
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_SAMPLE_PROPERTY:
		font_manager_google_fonts_sample_set_sample (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
font_manager_google_fonts_sample_model_get_instance_private (FontManagerGoogleFontsSampleModel* self)
{
	return G_STRUCT_MEMBER_P (self, FontManagerGoogleFontsSampleModel_private_offset);
}

static GType
font_manager_google_fonts_sample_model_real_get_item_type (GListModel* base)
{
	FontManagerGoogleFontsSampleModel * self;
	GType result;
	self = (FontManagerGoogleFontsSampleModel*) base;
	result = FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE;
	return result;
}

static guint
font_manager_google_fonts_sample_model_real_get_n_items (GListModel* base)
{
	FontManagerGoogleFontsSampleModel * self;
	guint _tmp0_ = 0U;
	FontManagerStringSet* _tmp1_;
	guint result;
	self = (FontManagerGoogleFontsSampleModel*) base;
	_tmp1_ = self->priv->_items;
	if (_tmp1_ != NULL) {
		FontManagerStringSet* _tmp2_;
		guint _tmp3_;
		guint _tmp4_;
		_tmp2_ = self->priv->_items;
		g_object_get (_tmp2_, "size", &_tmp3_, NULL);
		_tmp4_ = _tmp3_;
		_tmp0_ = _tmp4_;
	} else {
		_tmp0_ = (guint) 0;
	}
	result = _tmp0_;
	return result;
}

static GObject*
font_manager_google_fonts_sample_model_real_get_item (GListModel* base,
                                                      guint position)
{
	FontManagerGoogleFontsSampleModel * self;
	FontManagerStringSet* _tmp0_;
	const gchar* _tmp1_;
	FontManagerGoogleFontsSample* _tmp2_;
	GObject* result;
	self = (FontManagerGoogleFontsSampleModel*) base;
	_tmp0_ = self->priv->_items;
	_tmp1_ = font_manager_string_set_get (_tmp0_, position);
	_tmp2_ = font_manager_google_fonts_sample_new (_tmp1_);
	result = (GObject*) _tmp2_;
	return result;
}

FontManagerGoogleFontsSampleModel*
font_manager_google_fonts_sample_model_construct (GType object_type)
{
	FontManagerGoogleFontsSampleModel * self = NULL;
	self = (FontManagerGoogleFontsSampleModel*) g_object_new (object_type, NULL);
	return self;
}

FontManagerGoogleFontsSampleModel*
font_manager_google_fonts_sample_model_new (void)
{
	return font_manager_google_fonts_sample_model_construct (FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL);
}

FontManagerStringSet*
font_manager_google_fonts_sample_model_get_items (FontManagerGoogleFontsSampleModel* self)
{
	FontManagerStringSet* result;
	FontManagerStringSet* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_items;
	result = _tmp0_;
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

void
font_manager_google_fonts_sample_model_set_items (FontManagerGoogleFontsSampleModel* self,
                                                  FontManagerStringSet* value)
{
	FontManagerStringSet* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_google_fonts_sample_model_get_items (self);
	if (old_value != value) {
		FontManagerStringSet* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_items);
		self->priv->_items = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_google_fonts_sample_model_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_ITEMS_PROPERTY]);
	}
}

static void
_font_manager_google_fonts_sample_model___lambda165_ (FontManagerGoogleFontsSampleModel* self)
{
	g_list_model_items_changed ((GListModel*) self, (guint) 0, self->priv->n_items, g_list_model_get_n_items ((GListModel*) self));
	self->priv->n_items = g_list_model_get_n_items ((GListModel*) self);
}

static void
__font_manager_google_fonts_sample_model___lambda165__g_object_notify (GObject* _sender,
                                                                       GParamSpec* pspec,
                                                                       gpointer self)
{
	_font_manager_google_fonts_sample_model___lambda165_ ((FontManagerGoogleFontsSampleModel*) self);
}

static GObject *
font_manager_google_fonts_sample_model_constructor (GType type,
                                                    guint n_construct_properties,
                                                    GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	FontManagerGoogleFontsSampleModel * self;
	parent_class = G_OBJECT_CLASS (font_manager_google_fonts_sample_model_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL, FontManagerGoogleFontsSampleModel);
	g_signal_connect_object ((GObject*) self, "notify::items", (GCallback) __font_manager_google_fonts_sample_model___lambda165__g_object_notify, self, G_CONNECT_AFTER);
	return obj;
}

static void
font_manager_google_fonts_sample_model_class_init (FontManagerGoogleFontsSampleModelClass * klass,
                                                   gpointer klass_data)
{
	font_manager_google_fonts_sample_model_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &FontManagerGoogleFontsSampleModel_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_font_manager_google_fonts_sample_model_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_font_manager_google_fonts_sample_model_set_property;
	G_OBJECT_CLASS (klass)->constructor = font_manager_google_fonts_sample_model_constructor;
	G_OBJECT_CLASS (klass)->finalize = font_manager_google_fonts_sample_model_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_ITEMS_PROPERTY, font_manager_google_fonts_sample_model_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_ITEMS_PROPERTY] = g_param_spec_object ("items", "items", "items", font_manager_string_set_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
font_manager_google_fonts_sample_model_g_list_model_interface_init (GListModelInterface * iface,
                                                                    gpointer iface_data)
{
	font_manager_google_fonts_sample_model_g_list_model_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_item_type = (GType (*) (GListModel*)) font_manager_google_fonts_sample_model_real_get_item_type;
	iface->get_n_items = (guint (*) (GListModel*)) font_manager_google_fonts_sample_model_real_get_n_items;
	iface->get_item = (GObject* (*) (GListModel*, guint)) font_manager_google_fonts_sample_model_real_get_item;
}

static void
font_manager_google_fonts_sample_model_instance_init (FontManagerGoogleFontsSampleModel * self,
                                                      gpointer klass)
{
	self->priv = font_manager_google_fonts_sample_model_get_instance_private (self);
	self->priv->n_items = (guint) 0;
}

static void
font_manager_google_fonts_sample_model_finalize (GObject * obj)
{
	FontManagerGoogleFontsSampleModel * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL, FontManagerGoogleFontsSampleModel);
	_g_object_unref0 (self->priv->_items);
	G_OBJECT_CLASS (font_manager_google_fonts_sample_model_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
font_manager_google_fonts_sample_model_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (FontManagerGoogleFontsSampleModelClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) font_manager_google_fonts_sample_model_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FontManagerGoogleFontsSampleModel), 0, (GInstanceInitFunc) font_manager_google_fonts_sample_model_instance_init, NULL };
	static const GInterfaceInfo g_list_model_info = { (GInterfaceInitFunc) font_manager_google_fonts_sample_model_g_list_model_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType font_manager_google_fonts_sample_model_type_id;
	font_manager_google_fonts_sample_model_type_id = g_type_register_static (G_TYPE_OBJECT, "FontManagerGoogleFontsSampleModel", &g_define_type_info, 0);
	g_type_add_interface_static (font_manager_google_fonts_sample_model_type_id, g_list_model_get_type (), &g_list_model_info);
	FontManagerGoogleFontsSampleModel_private_offset = g_type_add_instance_private (font_manager_google_fonts_sample_model_type_id, sizeof (FontManagerGoogleFontsSampleModelPrivate));
	return font_manager_google_fonts_sample_model_type_id;
}

GType
font_manager_google_fonts_sample_model_get_type (void)
{
	static volatile gsize font_manager_google_fonts_sample_model_type_id__once = 0;
	if (g_once_init_enter (&font_manager_google_fonts_sample_model_type_id__once)) {
		GType font_manager_google_fonts_sample_model_type_id;
		font_manager_google_fonts_sample_model_type_id = font_manager_google_fonts_sample_model_get_type_once ();
		g_once_init_leave (&font_manager_google_fonts_sample_model_type_id__once, font_manager_google_fonts_sample_model_type_id);
	}
	return font_manager_google_fonts_sample_model_type_id__once;
}

static void
_vala_font_manager_google_fonts_sample_model_get_property (GObject * object,
                                                           guint property_id,
                                                           GValue * value,
                                                           GParamSpec * pspec)
{
	FontManagerGoogleFontsSampleModel * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL, FontManagerGoogleFontsSampleModel);
	switch (property_id) {
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_ITEMS_PROPERTY:
		g_value_set_object (value, font_manager_google_fonts_sample_model_get_items (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_font_manager_google_fonts_sample_model_set_property (GObject * object,
                                                           guint property_id,
                                                           const GValue * value,
                                                           GParamSpec * pspec)
{
	FontManagerGoogleFontsSampleModel * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL, FontManagerGoogleFontsSampleModel);
	switch (property_id) {
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_MODEL_ITEMS_PROPERTY:
		font_manager_google_fonts_sample_model_set_items (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
font_manager_google_fonts_sample_row_get_instance_private (FontManagerGoogleFontsSampleRow* self)
{
	return G_STRUCT_MEMBER_P (self, FontManagerGoogleFontsSampleRow_private_offset);
}

static const gchar*
string_to_string (const gchar* self)
{
	const gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self;
	return result;
}

FontManagerGoogleFontsSampleRow*
font_manager_google_fonts_sample_row_from_item (GObject* item)
{
	FontManagerGoogleFontsSampleRow* row = NULL;
	FontManagerGoogleFontsSampleRow* _tmp0_;
	FontManagerGoogleFontsSample* sample = NULL;
	FontManagerGoogleFontsSample* _tmp1_;
	GtkLabel* _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	gchar* sample_markup = NULL;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	gchar* _tmp7_;
	GtkInscription* _tmp8_;
	const gchar* _tmp9_;
	gchar* _tmp10_;
	gchar* _tmp11_;
	FontManagerGoogleFontsSampleRow* result;
	g_return_val_if_fail (item != NULL, NULL);
	_tmp0_ = font_manager_google_fonts_sample_row_new ();
	g_object_ref_sink (_tmp0_);
	row = _tmp0_;
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (item, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE, FontManagerGoogleFontsSample));
	sample = _tmp1_;
	_tmp2_ = row->priv->_label;
	_tmp3_ = font_manager_google_fonts_sample_get_label (sample);
	_tmp4_ = _tmp3_;
	gtk_label_set_label (_tmp2_, g_dgettext (NULL, _tmp4_));
	_tmp5_ = font_manager_google_fonts_sample_get_sample (sample);
	_tmp6_ = _tmp5_;
	_tmp7_ = g_markup_escape_text (_tmp6_, (gssize) -1);
	sample_markup = _tmp7_;
	_tmp8_ = row->priv->_sample;
	_tmp9_ = string_to_string (sample_markup);
	_tmp10_ = g_strconcat ("<small>", _tmp9_, "</small>", NULL);
	_tmp11_ = _tmp10_;
	gtk_inscription_set_markup (_tmp8_, _tmp11_);
	_g_free0 (_tmp11_);
	result = row;
	_g_free0 (sample_markup);
	_g_object_unref0 (sample);
	return result;
}

FontManagerGoogleFontsSampleRow*
font_manager_google_fonts_sample_row_construct (GType object_type)
{
	FontManagerGoogleFontsSampleRow * self = NULL;
	self = (FontManagerGoogleFontsSampleRow*) g_object_new (object_type, NULL);
	return self;
}

FontManagerGoogleFontsSampleRow*
font_manager_google_fonts_sample_row_new (void)
{
	return font_manager_google_fonts_sample_row_construct (FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_ROW);
}

GtkLabel*
font_manager_google_fonts_sample_row_get_label (FontManagerGoogleFontsSampleRow* self)
{
	GtkLabel* result;
	GtkLabel* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_label;
	result = _tmp0_;
	return result;
}

GtkInscription*
font_manager_google_fonts_sample_row_get_sample (FontManagerGoogleFontsSampleRow* self)
{
	GtkInscription* result;
	GtkInscription* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_sample;
	result = _tmp0_;
	return result;
}

static void
font_manager_google_fonts_sample_row_class_init (FontManagerGoogleFontsSampleRowClass * klass,
                                                 gpointer klass_data)
{
	font_manager_google_fonts_sample_row_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &FontManagerGoogleFontsSampleRow_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_font_manager_google_fonts_sample_row_get_property;
	G_OBJECT_CLASS (klass)->finalize = font_manager_google_fonts_sample_row_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_LABEL_PROPERTY, font_manager_google_fonts_sample_row_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_LABEL_PROPERTY] = g_param_spec_object ("label", "label", "label", gtk_label_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_SAMPLE_PROPERTY, font_manager_google_fonts_sample_row_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_SAMPLE_PROPERTY] = g_param_spec_object ("sample", "sample", "sample", gtk_inscription_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/com/github/FontManager/FontManager/web/google/ui/google-fonts-sample-row.ui");
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "label", FALSE, FontManagerGoogleFontsSampleRow_private_offset + G_STRUCT_OFFSET (FontManagerGoogleFontsSampleRowPrivate, _label));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "sample", FALSE, FontManagerGoogleFontsSampleRow_private_offset + G_STRUCT_OFFSET (FontManagerGoogleFontsSampleRowPrivate, _sample));
}

static void
font_manager_google_fonts_sample_row_instance_init (FontManagerGoogleFontsSampleRow * self,
                                                    gpointer klass)
{
	self->priv = font_manager_google_fonts_sample_row_get_instance_private (self);
	gtk_widget_init_template (GTK_WIDGET (self));
}

static void
font_manager_google_fonts_sample_row_finalize (GObject * obj)
{
	FontManagerGoogleFontsSampleRow * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_ROW, FontManagerGoogleFontsSampleRow);
	G_OBJECT_CLASS (font_manager_google_fonts_sample_row_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
font_manager_google_fonts_sample_row_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (FontManagerGoogleFontsSampleRowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) font_manager_google_fonts_sample_row_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FontManagerGoogleFontsSampleRow), 0, (GInstanceInitFunc) font_manager_google_fonts_sample_row_instance_init, NULL };
	GType font_manager_google_fonts_sample_row_type_id;
	font_manager_google_fonts_sample_row_type_id = g_type_register_static (gtk_box_get_type (), "FontManagerGoogleFontsSampleRow", &g_define_type_info, 0);
	FontManagerGoogleFontsSampleRow_private_offset = g_type_add_instance_private (font_manager_google_fonts_sample_row_type_id, sizeof (FontManagerGoogleFontsSampleRowPrivate));
	return font_manager_google_fonts_sample_row_type_id;
}

GType
font_manager_google_fonts_sample_row_get_type (void)
{
	static volatile gsize font_manager_google_fonts_sample_row_type_id__once = 0;
	if (g_once_init_enter (&font_manager_google_fonts_sample_row_type_id__once)) {
		GType font_manager_google_fonts_sample_row_type_id;
		font_manager_google_fonts_sample_row_type_id = font_manager_google_fonts_sample_row_get_type_once ();
		g_once_init_leave (&font_manager_google_fonts_sample_row_type_id__once, font_manager_google_fonts_sample_row_type_id);
	}
	return font_manager_google_fonts_sample_row_type_id__once;
}

static void
_vala_font_manager_google_fonts_sample_row_get_property (GObject * object,
                                                         guint property_id,
                                                         GValue * value,
                                                         GParamSpec * pspec)
{
	FontManagerGoogleFontsSampleRow * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_ROW, FontManagerGoogleFontsSampleRow);
	switch (property_id) {
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_LABEL_PROPERTY:
		g_value_set_object (value, font_manager_google_fonts_sample_row_get_label (self));
		break;
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_ROW_SAMPLE_PROPERTY:
		g_value_set_object (value, font_manager_google_fonts_sample_row_get_sample (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
font_manager_google_fonts_sample_list_get_instance_private (FontManagerGoogleFontsSampleList* self)
{
	return G_STRUCT_MEMBER_P (self, FontManagerGoogleFontsSampleList_private_offset);
}

static void
font_manager_google_fonts_sample_list_on_row_activated (FontManagerGoogleFontsSampleList* self,
                                                        GtkListBox* list,
                                                        GtkListBoxRow* row)
{
	guint position = 0U;
	FontManagerGoogleFontsSample* item = NULL;
	FontManagerGoogleFontsSampleModel* _tmp0_;
	GObject* _tmp1_;
	FontManagerGoogleFontsSample* _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (list != NULL);
	g_return_if_fail (row != NULL);
	if (row == NULL) {
		return;
	}
	position = (guint) gtk_list_box_row_get_index (row);
	_tmp0_ = self->priv->_model;
	_tmp1_ = g_list_model_get_item ((GListModel*) _tmp0_, position);
	item = G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE, FontManagerGoogleFontsSample);
	_tmp2_ = item;
	_tmp3_ = font_manager_google_fonts_sample_get_sample (_tmp2_);
	_tmp4_ = _tmp3_;
	g_signal_emit (self, font_manager_google_fonts_sample_list_signals[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_ROW_SELECTED_SIGNAL], 0, _tmp4_);
	gtk_popover_popdown ((GtkPopover*) self);
	_g_object_unref0 (item);
	return;
}

static void
_font_manager_google_fonts_sample_list_on_row_activated_gtk_list_box_row_activated (GtkListBox* _sender,
                                                                                    GtkListBoxRow* row,
                                                                                    gpointer self)
{
	font_manager_google_fonts_sample_list_on_row_activated ((FontManagerGoogleFontsSampleList*) self, _sender, row);
}

void
font_manager_google_fonts_sample_list_unselect_all (FontManagerGoogleFontsSampleList* self)
{
	GtkListBox* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->sample_list;
	gtk_list_box_unselect_all (_tmp0_);
	return;
}

FontManagerGoogleFontsSampleList*
font_manager_google_fonts_sample_list_construct (GType object_type)
{
	FontManagerGoogleFontsSampleList * self = NULL;
	self = (FontManagerGoogleFontsSampleList*) g_object_new (object_type, NULL);
	return self;
}

FontManagerGoogleFontsSampleList*
font_manager_google_fonts_sample_list_new (void)
{
	return font_manager_google_fonts_sample_list_construct (FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST);
}

FontManagerStringSet*
font_manager_google_fonts_sample_list_get_items (FontManagerGoogleFontsSampleList* self)
{
	FontManagerStringSet* result;
	FontManagerStringSet* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_items;
	result = _tmp0_;
	return result;
}

void
font_manager_google_fonts_sample_list_set_items (FontManagerGoogleFontsSampleList* self,
                                                 FontManagerStringSet* value)
{
	FontManagerStringSet* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_google_fonts_sample_list_get_items (self);
	if (old_value != value) {
		FontManagerStringSet* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_items);
		self->priv->_items = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_google_fonts_sample_list_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_ITEMS_PROPERTY]);
	}
}

FontManagerGoogleFontsSampleModel*
font_manager_google_fonts_sample_list_get_model (FontManagerGoogleFontsSampleList* self)
{
	FontManagerGoogleFontsSampleModel* result;
	FontManagerGoogleFontsSampleModel* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_model;
	result = _tmp0_;
	return result;
}

static void
font_manager_google_fonts_sample_list_set_model (FontManagerGoogleFontsSampleList* self,
                                                 FontManagerGoogleFontsSampleModel* value)
{
	FontManagerGoogleFontsSampleModel* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_google_fonts_sample_list_get_model (self);
	if (old_value != value) {
		FontManagerGoogleFontsSampleModel* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_model);
		self->priv->_model = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_google_fonts_sample_list_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_MODEL_PROPERTY]);
	}
}

static GtkWidget*
_font_manager_google_fonts_sample_row_from_item_gtk_list_box_create_widget_func (GObject* item,
                                                                                 gpointer self)
{
	GtkWidget* result;
	result = font_manager_google_fonts_sample_row_from_item (item);
	return result;
}

static GObject *
font_manager_google_fonts_sample_list_constructor (GType type,
                                                   guint n_construct_properties,
                                                   GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	FontManagerGoogleFontsSampleList * self;
	GtkListBox* _tmp0_;
	FontManagerGoogleFontsSampleModel* _tmp1_;
	FontManagerGoogleFontsSampleModel* _tmp2_;
	GtkListBox* _tmp3_;
	FontManagerGoogleFontsSampleModel* _tmp4_;
	GBindingFlags flags = 0U;
	FontManagerGoogleFontsSampleModel* _tmp5_;
	parent_class = G_OBJECT_CLASS (font_manager_google_fonts_sample_list_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST, FontManagerGoogleFontsSampleList);
	_tmp0_ = self->priv->sample_list;
	gtk_list_box_set_selection_mode (_tmp0_, GTK_SELECTION_NONE);
	font_manager_widget_set_name ((GtkWidget*) self, "FontManagerGoogleFontsSampleList");
	_tmp1_ = font_manager_google_fonts_sample_model_new ();
	_tmp2_ = _tmp1_;
	font_manager_google_fonts_sample_list_set_model (self, _tmp2_);
	_g_object_unref0 (_tmp2_);
	_tmp3_ = self->priv->sample_list;
	_tmp4_ = self->priv->_model;
	gtk_list_box_bind_model (_tmp3_, (GListModel*) _tmp4_, _font_manager_google_fonts_sample_row_from_item_gtk_list_box_create_widget_func, NULL, NULL);
	flags = G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE;
	_tmp5_ = self->priv->_model;
	g_object_bind_property_with_closures ((GObject*) self, "items", (GObject*) _tmp5_, "items", flags, (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, (GClosureNotify) NULL)), (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, (GClosureNotify) NULL)));
	return obj;
}

static void
font_manager_google_fonts_sample_list_class_init (FontManagerGoogleFontsSampleListClass * klass,
                                                  gpointer klass_data)
{
	font_manager_google_fonts_sample_list_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &FontManagerGoogleFontsSampleList_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_font_manager_google_fonts_sample_list_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_font_manager_google_fonts_sample_list_set_property;
	G_OBJECT_CLASS (klass)->constructor = font_manager_google_fonts_sample_list_constructor;
	G_OBJECT_CLASS (klass)->finalize = font_manager_google_fonts_sample_list_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_ITEMS_PROPERTY, font_manager_google_fonts_sample_list_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_ITEMS_PROPERTY] = g_param_spec_object ("items", "items", "items", font_manager_string_set_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_MODEL_PROPERTY, font_manager_google_fonts_sample_list_properties[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_MODEL_PROPERTY] = g_param_spec_object ("model", "model", "model", FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_MODEL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/com/github/FontManager/FontManager/web/google/ui/google-fonts-sample-list.ui");
	font_manager_google_fonts_sample_list_signals[FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_ROW_SELECTED_SIGNAL] = g_signal_new ("row-selected", FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "sample_list", FALSE, FontManagerGoogleFontsSampleList_private_offset + G_STRUCT_OFFSET (FontManagerGoogleFontsSampleListPrivate, sample_list));
	gtk_widget_class_bind_template_callback_full (GTK_WIDGET_CLASS (klass), "on_row_activated", G_CALLBACK(_font_manager_google_fonts_sample_list_on_row_activated_gtk_list_box_row_activated));
}

static void
font_manager_google_fonts_sample_list_instance_init (FontManagerGoogleFontsSampleList * self,
                                                     gpointer klass)
{
	self->priv = font_manager_google_fonts_sample_list_get_instance_private (self);
	gtk_widget_init_template (GTK_WIDGET (self));
}

static void
font_manager_google_fonts_sample_list_finalize (GObject * obj)
{
	FontManagerGoogleFontsSampleList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST, FontManagerGoogleFontsSampleList);
	_g_object_unref0 (self->priv->_items);
	_g_object_unref0 (self->priv->_model);
	G_OBJECT_CLASS (font_manager_google_fonts_sample_list_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
font_manager_google_fonts_sample_list_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (FontManagerGoogleFontsSampleListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) font_manager_google_fonts_sample_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FontManagerGoogleFontsSampleList), 0, (GInstanceInitFunc) font_manager_google_fonts_sample_list_instance_init, NULL };
	GType font_manager_google_fonts_sample_list_type_id;
	font_manager_google_fonts_sample_list_type_id = g_type_register_static (gtk_popover_get_type (), "FontManagerGoogleFontsSampleList", &g_define_type_info, 0);
	FontManagerGoogleFontsSampleList_private_offset = g_type_add_instance_private (font_manager_google_fonts_sample_list_type_id, sizeof (FontManagerGoogleFontsSampleListPrivate));
	return font_manager_google_fonts_sample_list_type_id;
}

GType
font_manager_google_fonts_sample_list_get_type (void)
{
	static volatile gsize font_manager_google_fonts_sample_list_type_id__once = 0;
	if (g_once_init_enter (&font_manager_google_fonts_sample_list_type_id__once)) {
		GType font_manager_google_fonts_sample_list_type_id;
		font_manager_google_fonts_sample_list_type_id = font_manager_google_fonts_sample_list_get_type_once ();
		g_once_init_leave (&font_manager_google_fonts_sample_list_type_id__once, font_manager_google_fonts_sample_list_type_id);
	}
	return font_manager_google_fonts_sample_list_type_id__once;
}

static void
_vala_font_manager_google_fonts_sample_list_get_property (GObject * object,
                                                          guint property_id,
                                                          GValue * value,
                                                          GParamSpec * pspec)
{
	FontManagerGoogleFontsSampleList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST, FontManagerGoogleFontsSampleList);
	switch (property_id) {
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_ITEMS_PROPERTY:
		g_value_set_object (value, font_manager_google_fonts_sample_list_get_items (self));
		break;
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_MODEL_PROPERTY:
		g_value_set_object (value, font_manager_google_fonts_sample_list_get_model (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_font_manager_google_fonts_sample_list_set_property (GObject * object,
                                                          guint property_id,
                                                          const GValue * value,
                                                          GParamSpec * pspec)
{
	FontManagerGoogleFontsSampleList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_GOOGLE_FONTS_TYPE_SAMPLE_LIST, FontManagerGoogleFontsSampleList);
	switch (property_id) {
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_ITEMS_PROPERTY:
		font_manager_google_fonts_sample_list_set_items (self, g_value_get_object (value));
		break;
		case FONT_MANAGER_GOOGLE_FONTS_SAMPLE_LIST_MODEL_PROPERTY:
		font_manager_google_fonts_sample_list_set_model (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

