/* Preferences.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from Preferences.vala, do not modify */

/* Preferences.vala
 *
 * Copyright (C) 2009-2025 Jerry Casiano
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.
 *
 * If not, see <http://www.gnu.org/licenses/gpl-3.0.txt>.
*/

#include <gtk/gtk.h>
#include <glib-object.h>
#include <gio/gio.h>
#include "font-manager.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib/gi18n-lib.h>

#define FONT_MANAGER_FONTCONFIG_DISCLAIMER _ ("A Fontconfig configuration file will be generated from these settings." \
"\n" \
"\n" \
"Running applications may require a restart to reflect any changes.\n" \
"\n" \
"Note that not all environments/applications will honor these settings.")
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define FONT_MANAGER_TYPE_PANED (font_manager_paned_get_type ())
#define FONT_MANAGER_PANED(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_PANED, FontManagerPaned))
#define FONT_MANAGER_PANED_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_PANED, FontManagerPanedClass))
#define FONT_MANAGER_IS_PANED(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_PANED))
#define FONT_MANAGER_IS_PANED_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_PANED))
#define FONT_MANAGER_PANED_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_PANED, FontManagerPanedClass))

typedef struct _FontManagerPaned FontManagerPaned;
typedef struct _FontManagerPanedClass FontManagerPanedClass;
typedef struct _FontManagerPanedPrivate FontManagerPanedPrivate;

#define FONT_MANAGER_TYPE_PREFERENCE_PANE (font_manager_preference_pane_get_type ())
#define FONT_MANAGER_PREFERENCE_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_PREFERENCE_PANE, FontManagerPreferencePane))
#define FONT_MANAGER_PREFERENCE_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_PREFERENCE_PANE, FontManagerPreferencePaneClass))
#define FONT_MANAGER_IS_PREFERENCE_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_PREFERENCE_PANE))
#define FONT_MANAGER_IS_PREFERENCE_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_PREFERENCE_PANE))
#define FONT_MANAGER_PREFERENCE_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_PREFERENCE_PANE, FontManagerPreferencePaneClass))

typedef struct _FontManagerPreferencePane FontManagerPreferencePane;
typedef struct _FontManagerPreferencePaneClass FontManagerPreferencePaneClass;
typedef struct _FontManagerPreferencePanePrivate FontManagerPreferencePanePrivate;

#define FONT_MANAGER_TYPE_USER_ACTION_MODEL (font_manager_user_action_model_get_type ())
#define FONT_MANAGER_USER_ACTION_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_USER_ACTION_MODEL, FontManagerUserActionModel))
#define FONT_MANAGER_USER_ACTION_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_USER_ACTION_MODEL, FontManagerUserActionModelClass))
#define FONT_MANAGER_IS_USER_ACTION_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_USER_ACTION_MODEL))
#define FONT_MANAGER_IS_USER_ACTION_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_USER_ACTION_MODEL))
#define FONT_MANAGER_USER_ACTION_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_USER_ACTION_MODEL, FontManagerUserActionModelClass))

typedef struct _FontManagerUserActionModel FontManagerUserActionModel;
typedef struct _FontManagerUserActionModelClass FontManagerUserActionModelClass;

#define FONT_MANAGER_TYPE_USER_SOURCE_MODEL (font_manager_user_source_model_get_type ())
#define FONT_MANAGER_USER_SOURCE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_USER_SOURCE_MODEL, FontManagerUserSourceModel))
#define FONT_MANAGER_USER_SOURCE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_USER_SOURCE_MODEL, FontManagerUserSourceModelClass))
#define FONT_MANAGER_IS_USER_SOURCE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_USER_SOURCE_MODEL))
#define FONT_MANAGER_IS_USER_SOURCE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_USER_SOURCE_MODEL))
#define FONT_MANAGER_USER_SOURCE_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_USER_SOURCE_MODEL, FontManagerUserSourceModelClass))

typedef struct _FontManagerUserSourceModel FontManagerUserSourceModel;
typedef struct _FontManagerUserSourceModelClass FontManagerUserSourceModelClass;

#define FONT_MANAGER_TYPE_PREFERENCE_LIST (font_manager_preference_list_get_type ())
#define FONT_MANAGER_PREFERENCE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_PREFERENCE_LIST, FontManagerPreferenceList))
#define FONT_MANAGER_PREFERENCE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_PREFERENCE_LIST, FontManagerPreferenceListClass))
#define FONT_MANAGER_IS_PREFERENCE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_PREFERENCE_LIST))
#define FONT_MANAGER_IS_PREFERENCE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_PREFERENCE_LIST))
#define FONT_MANAGER_PREFERENCE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_PREFERENCE_LIST, FontManagerPreferenceListClass))

typedef struct _FontManagerPreferenceList FontManagerPreferenceList;
typedef struct _FontManagerPreferenceListClass FontManagerPreferenceListClass;

#define FONT_MANAGER_TYPE_USER_INTERFACE_PREFERENCES (font_manager_user_interface_preferences_get_type ())
#define FONT_MANAGER_USER_INTERFACE_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_USER_INTERFACE_PREFERENCES, FontManagerUserInterfacePreferences))
#define FONT_MANAGER_USER_INTERFACE_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_USER_INTERFACE_PREFERENCES, FontManagerUserInterfacePreferencesClass))
#define FONT_MANAGER_IS_USER_INTERFACE_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_USER_INTERFACE_PREFERENCES))
#define FONT_MANAGER_IS_USER_INTERFACE_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_USER_INTERFACE_PREFERENCES))
#define FONT_MANAGER_USER_INTERFACE_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_USER_INTERFACE_PREFERENCES, FontManagerUserInterfacePreferencesClass))

typedef struct _FontManagerUserInterfacePreferences FontManagerUserInterfacePreferences;
typedef struct _FontManagerUserInterfacePreferencesClass FontManagerUserInterfacePreferencesClass;
enum  {
	FONT_MANAGER_PREFERENCE_PANE_0_PROPERTY,
	FONT_MANAGER_PREFERENCE_PANE_USER_ACTIONS_PROPERTY,
	FONT_MANAGER_PREFERENCE_PANE_USER_SOURCES_PROPERTY,
	FONT_MANAGER_PREFERENCE_PANE_INTERFACE_PREFERENCES_PROPERTY,
	FONT_MANAGER_PREFERENCE_PANE_NUM_PROPERTIES
};
static GParamSpec* font_manager_preference_pane_properties[FONT_MANAGER_PREFERENCE_PANE_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define FONT_MANAGER_TYPE_DESKTOP_PREFERENCES (font_manager_desktop_preferences_get_type ())
#define FONT_MANAGER_DESKTOP_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_DESKTOP_PREFERENCES, FontManagerDesktopPreferences))
#define FONT_MANAGER_DESKTOP_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_DESKTOP_PREFERENCES, FontManagerDesktopPreferencesClass))
#define FONT_MANAGER_IS_DESKTOP_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_DESKTOP_PREFERENCES))
#define FONT_MANAGER_IS_DESKTOP_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_DESKTOP_PREFERENCES))
#define FONT_MANAGER_DESKTOP_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_DESKTOP_PREFERENCES, FontManagerDesktopPreferencesClass))

typedef struct _FontManagerDesktopPreferences FontManagerDesktopPreferences;
typedef struct _FontManagerDesktopPreferencesClass FontManagerDesktopPreferencesClass;

#define FONT_MANAGER_TYPE_USER_ACTION_LIST (font_manager_user_action_list_get_type ())
#define FONT_MANAGER_USER_ACTION_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_USER_ACTION_LIST, FontManagerUserActionList))
#define FONT_MANAGER_USER_ACTION_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_USER_ACTION_LIST, FontManagerUserActionListClass))
#define FONT_MANAGER_IS_USER_ACTION_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_USER_ACTION_LIST))
#define FONT_MANAGER_IS_USER_ACTION_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_USER_ACTION_LIST))
#define FONT_MANAGER_USER_ACTION_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_USER_ACTION_LIST, FontManagerUserActionListClass))

typedef struct _FontManagerUserActionList FontManagerUserActionList;
typedef struct _FontManagerUserActionListClass FontManagerUserActionListClass;

#define FONT_MANAGER_TYPE_USER_SOURCE_LIST (font_manager_user_source_list_get_type ())
#define FONT_MANAGER_USER_SOURCE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_USER_SOURCE_LIST, FontManagerUserSourceList))
#define FONT_MANAGER_USER_SOURCE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_USER_SOURCE_LIST, FontManagerUserSourceListClass))
#define FONT_MANAGER_IS_USER_SOURCE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_USER_SOURCE_LIST))
#define FONT_MANAGER_IS_USER_SOURCE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_USER_SOURCE_LIST))
#define FONT_MANAGER_USER_SOURCE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_USER_SOURCE_LIST, FontManagerUserSourceListClass))

typedef struct _FontManagerUserSourceList FontManagerUserSourceList;
typedef struct _FontManagerUserSourceListClass FontManagerUserSourceListClass;

#define FONT_MANAGER_TYPE_SUBSTITUTE_LIST (font_manager_substitute_list_get_type ())
#define FONT_MANAGER_SUBSTITUTE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_SUBSTITUTE_LIST, FontManagerSubstituteList))
#define FONT_MANAGER_SUBSTITUTE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_SUBSTITUTE_LIST, FontManagerSubstituteListClass))
#define FONT_MANAGER_IS_SUBSTITUTE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_SUBSTITUTE_LIST))
#define FONT_MANAGER_IS_SUBSTITUTE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_SUBSTITUTE_LIST))
#define FONT_MANAGER_SUBSTITUTE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_SUBSTITUTE_LIST, FontManagerSubstituteListClass))

typedef struct _FontManagerSubstituteList FontManagerSubstituteList;
typedef struct _FontManagerSubstituteListClass FontManagerSubstituteListClass;

#define FONT_MANAGER_TYPE_DISPLAY_PREFERENCES (font_manager_display_preferences_get_type ())
#define FONT_MANAGER_DISPLAY_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_DISPLAY_PREFERENCES, FontManagerDisplayPreferences))
#define FONT_MANAGER_DISPLAY_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_DISPLAY_PREFERENCES, FontManagerDisplayPreferencesClass))
#define FONT_MANAGER_IS_DISPLAY_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_DISPLAY_PREFERENCES))
#define FONT_MANAGER_IS_DISPLAY_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_DISPLAY_PREFERENCES))
#define FONT_MANAGER_DISPLAY_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_DISPLAY_PREFERENCES, FontManagerDisplayPreferencesClass))

typedef struct _FontManagerDisplayPreferences FontManagerDisplayPreferences;
typedef struct _FontManagerDisplayPreferencesClass FontManagerDisplayPreferencesClass;

#define FONT_MANAGER_TYPE_RENDERING_PREFERENCES (font_manager_rendering_preferences_get_type ())
#define FONT_MANAGER_RENDERING_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_RENDERING_PREFERENCES, FontManagerRenderingPreferences))
#define FONT_MANAGER_RENDERING_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_RENDERING_PREFERENCES, FontManagerRenderingPreferencesClass))
#define FONT_MANAGER_IS_RENDERING_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_RENDERING_PREFERENCES))
#define FONT_MANAGER_IS_RENDERING_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_RENDERING_PREFERENCES))
#define FONT_MANAGER_RENDERING_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_RENDERING_PREFERENCES, FontManagerRenderingPreferencesClass))

typedef struct _FontManagerRenderingPreferences FontManagerRenderingPreferences;
typedef struct _FontManagerRenderingPreferencesClass FontManagerRenderingPreferencesClass;
typedef struct _FontManagerPreferenceListPrivate FontManagerPreferenceListPrivate;

#define FONT_MANAGER_TYPE_BASE_CONTROLS (font_manager_base_controls_get_type ())
#define FONT_MANAGER_BASE_CONTROLS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_BASE_CONTROLS, FontManagerBaseControls))
#define FONT_MANAGER_BASE_CONTROLS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_BASE_CONTROLS, FontManagerBaseControlsClass))
#define FONT_MANAGER_IS_BASE_CONTROLS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_BASE_CONTROLS))
#define FONT_MANAGER_IS_BASE_CONTROLS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_BASE_CONTROLS))
#define FONT_MANAGER_BASE_CONTROLS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_BASE_CONTROLS, FontManagerBaseControlsClass))

typedef struct _FontManagerBaseControls FontManagerBaseControls;
typedef struct _FontManagerBaseControlsClass FontManagerBaseControlsClass;
enum  {
	FONT_MANAGER_PREFERENCE_LIST_0_PROPERTY,
	FONT_MANAGER_PREFERENCE_LIST_NUM_PROPERTIES
};
static GParamSpec* font_manager_preference_list_properties[FONT_MANAGER_PREFERENCE_LIST_NUM_PROPERTIES];

#define FONT_MANAGER_TYPE_FONTCONFIG_FOOTER (font_manager_fontconfig_footer_get_type ())
#define FONT_MANAGER_FONTCONFIG_FOOTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_FONTCONFIG_FOOTER, FontManagerFontconfigFooter))
#define FONT_MANAGER_FONTCONFIG_FOOTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_FONTCONFIG_FOOTER, FontManagerFontconfigFooterClass))
#define FONT_MANAGER_IS_FONTCONFIG_FOOTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_FONTCONFIG_FOOTER))
#define FONT_MANAGER_IS_FONTCONFIG_FOOTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_FONTCONFIG_FOOTER))
#define FONT_MANAGER_FONTCONFIG_FOOTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_FONTCONFIG_FOOTER, FontManagerFontconfigFooterClass))

typedef struct _FontManagerFontconfigFooter FontManagerFontconfigFooter;
typedef struct _FontManagerFontconfigFooterClass FontManagerFontconfigFooterClass;
typedef struct _FontManagerFontconfigFooterPrivate FontManagerFontconfigFooterPrivate;
enum  {
	FONT_MANAGER_FONTCONFIG_FOOTER_0_PROPERTY,
	FONT_MANAGER_FONTCONFIG_FOOTER_NUM_PROPERTIES
};
static GParamSpec* font_manager_fontconfig_footer_properties[FONT_MANAGER_FONTCONFIG_FOOTER_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
enum  {
	FONT_MANAGER_FONTCONFIG_FOOTER_RESET_REQUESTED_SIGNAL,
	FONT_MANAGER_FONTCONFIG_FOOTER_NUM_SIGNALS
};
static guint font_manager_fontconfig_footer_signals[FONT_MANAGER_FONTCONFIG_FOOTER_NUM_SIGNALS] = {0};

#define FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON (font_manager_subpixel_geometry_icon_get_type ())
#define FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON, FontManagerSubpixelGeometryIcon))
#define FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON, FontManagerSubpixelGeometryIconClass))
#define FONT_MANAGER_IS_SUBPIXEL_GEOMETRY_ICON(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON))
#define FONT_MANAGER_IS_SUBPIXEL_GEOMETRY_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON))
#define FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON, FontManagerSubpixelGeometryIconClass))

typedef struct _FontManagerSubpixelGeometryIcon FontManagerSubpixelGeometryIcon;
typedef struct _FontManagerSubpixelGeometryIconClass FontManagerSubpixelGeometryIconClass;
typedef struct _FontManagerSubpixelGeometryIconPrivate FontManagerSubpixelGeometryIconPrivate;
enum  {
	FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_0_PROPERTY,
	FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_PREFERRED_SIZE_PROPERTY,
	FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_NUM_PROPERTIES
};
static GParamSpec* font_manager_subpixel_geometry_icon_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_NUM_PROPERTIES];

#define FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY (font_manager_subpixel_geometry_get_type ())
#define FONT_MANAGER_SUBPIXEL_GEOMETRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY, FontManagerSubpixelGeometry))
#define FONT_MANAGER_SUBPIXEL_GEOMETRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY, FontManagerSubpixelGeometryClass))
#define FONT_MANAGER_IS_SUBPIXEL_GEOMETRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY))
#define FONT_MANAGER_IS_SUBPIXEL_GEOMETRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY))
#define FONT_MANAGER_SUBPIXEL_GEOMETRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY, FontManagerSubpixelGeometryClass))

typedef struct _FontManagerSubpixelGeometry FontManagerSubpixelGeometry;
typedef struct _FontManagerSubpixelGeometryClass FontManagerSubpixelGeometryClass;
typedef struct _FontManagerSubpixelGeometryPrivate FontManagerSubpixelGeometryPrivate;
enum  {
	FONT_MANAGER_SUBPIXEL_GEOMETRY_0_PROPERTY,
	FONT_MANAGER_SUBPIXEL_GEOMETRY_SELECTED_PROPERTY,
	FONT_MANAGER_SUBPIXEL_GEOMETRY_PREFERRED_SIZE_PROPERTY,
	FONT_MANAGER_SUBPIXEL_GEOMETRY_OPTIONS_PROPERTY,
	FONT_MANAGER_SUBPIXEL_GEOMETRY_NUM_PROPERTIES
};
static GParamSpec* font_manager_subpixel_geometry_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_NUM_PROPERTIES];
#define _g_ptr_array_unref0(var) ((var == NULL) ? NULL : (var = (g_ptr_array_unref (var), NULL)))
typedef struct _Block27Data Block27Data;

struct _FontManagerPaned {
	GtkWidget parent_instance;
	FontManagerPanedPrivate * priv;
};

struct _FontManagerPanedClass {
	GtkWidgetClass parent_class;
};

struct _FontManagerPreferencePane {
	FontManagerPaned parent_instance;
	FontManagerPreferencePanePrivate * priv;
};

struct _FontManagerPreferencePaneClass {
	FontManagerPanedClass parent_class;
};

struct _FontManagerPreferencePanePrivate {
	FontManagerUserActionModel* _user_actions;
	FontManagerUserSourceModel* _user_sources;
	FontManagerUserInterfacePreferences* _interface_preferences;
	GtkStack* stack;
	GtkStackSidebar* sidebar;
};

struct _FontManagerPreferenceList {
	GtkBox parent_instance;
	FontManagerPreferenceListPrivate * priv;
	GtkListBox* list;
	FontManagerBaseControls* controls;
	GtkSeparator* separator;
};

struct _FontManagerPreferenceListClass {
	GtkBoxClass parent_class;
	void (*on_add_selected) (FontManagerPreferenceList* self);
	void (*on_list_row_selected) (FontManagerPreferenceList* self, GtkListBox* box, GtkListBoxRow* row);
	void (*on_map) (FontManagerPreferenceList* self);
	void (*on_remove_selected) (FontManagerPreferenceList* self);
	void (*on_unmap) (FontManagerPreferenceList* self);
};

struct _FontManagerFontconfigFooter {
	GtkBox parent_instance;
	FontManagerFontconfigFooterPrivate * priv;
};

struct _FontManagerFontconfigFooterClass {
	GtkBoxClass parent_class;
};

struct _FontManagerSubpixelGeometryIcon {
	GtkBox parent_instance;
	FontManagerSubpixelGeometryIconPrivate * priv;
};

struct _FontManagerSubpixelGeometryIconClass {
	GtkBoxClass parent_class;
};

struct _FontManagerSubpixelGeometryIconPrivate {
	gint _preferred_size;
};

struct _FontManagerSubpixelGeometry {
	GtkBox parent_instance;
	FontManagerSubpixelGeometryPrivate * priv;
};

struct _FontManagerSubpixelGeometryClass {
	GtkBoxClass parent_class;
};

struct _FontManagerSubpixelGeometryPrivate {
	gint _preferred_size;
	GPtrArray* _options;
	guint _rgba;
};

struct _Block27Data {
	int _ref_count_;
	FontManagerSubpixelGeometry* self;
	GtkCheckButton* button;
};

static gint FontManagerPreferencePane_private_offset;
static gpointer font_manager_preference_pane_parent_class = NULL;
static gpointer font_manager_preference_list_parent_class = NULL;
static gpointer font_manager_fontconfig_footer_parent_class = NULL;
static gint FontManagerSubpixelGeometryIcon_private_offset;
static gpointer font_manager_subpixel_geometry_icon_parent_class = NULL;
static gint FontManagerSubpixelGeometry_private_offset;
static gpointer font_manager_subpixel_geometry_parent_class = NULL;

VALA_EXTERN GType font_manager_paned_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerPaned, g_object_unref)
VALA_EXTERN GType font_manager_preference_pane_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerPreferencePane, g_object_unref)
VALA_EXTERN GType font_manager_user_action_model_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerUserActionModel, g_object_unref)
VALA_EXTERN GType font_manager_user_source_model_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerUserSourceModel, g_object_unref)
VALA_EXTERN GType font_manager_preference_list_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerPreferenceList, g_object_unref)
VALA_EXTERN GType font_manager_user_interface_preferences_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerUserInterfacePreferences, g_object_unref)
VALA_EXTERN FontManagerPreferencePane* font_manager_preference_pane_new (GSettings* settings);
VALA_EXTERN FontManagerPreferencePane* font_manager_preference_pane_construct (GType object_type,
                                                                   GSettings* settings);
VALA_EXTERN FontManagerPaned* font_manager_paned_new (void);
VALA_EXTERN FontManagerPaned* font_manager_paned_construct (GType object_type);
VALA_EXTERN void font_manager_paned_set_start_child (FontManagerPaned* self,
                                         GtkWidget* widget);
VALA_EXTERN void font_manager_paned_set_end_child (FontManagerPaned* self,
                                       GtkWidget* widget);
VALA_EXTERN FontManagerUserInterfacePreferences* font_manager_user_interface_preferences_new (GSettings* settings);
VALA_EXTERN FontManagerUserInterfacePreferences* font_manager_user_interface_preferences_construct (GType object_type,
                                                                                        GSettings* settings);
static void font_manager_preference_pane_set_interface_preferences (FontManagerPreferencePane* self,
                                                             FontManagerUserInterfacePreferences* value);
VALA_EXTERN void font_manager_preference_pane_add_page (FontManagerPreferencePane* self,
                                            GtkWidget* widget,
                                            const gchar* name,
                                            const gchar* title);
VALA_EXTERN FontManagerUserInterfacePreferences* font_manager_preference_pane_get_interface_preferences (FontManagerPreferencePane* self);
VALA_EXTERN FontManagerDesktopPreferences* font_manager_desktop_preferences_new (void);
VALA_EXTERN FontManagerDesktopPreferences* font_manager_desktop_preferences_construct (GType object_type);
VALA_EXTERN GType font_manager_desktop_preferences_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerDesktopPreferences, g_object_unref)
VALA_EXTERN GType font_manager_user_action_list_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerUserActionList, g_object_unref)
VALA_EXTERN FontManagerUserActionList* font_manager_user_action_list_new (void);
VALA_EXTERN FontManagerUserActionList* font_manager_user_action_list_construct (GType object_type);
VALA_EXTERN FontManagerUserActionModel* font_manager_user_action_list_get_model (FontManagerUserActionList* self);
static void font_manager_preference_pane_set_user_actions (FontManagerPreferencePane* self,
                                                    FontManagerUserActionModel* value);
VALA_EXTERN GType font_manager_user_source_list_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerUserSourceList, g_object_unref)
VALA_EXTERN FontManagerUserSourceList* font_manager_user_source_list_new (void);
VALA_EXTERN FontManagerUserSourceList* font_manager_user_source_list_construct (GType object_type);
VALA_EXTERN FontManagerUserSourceModel* font_manager_user_source_list_get_model (FontManagerUserSourceList* self);
static void font_manager_preference_pane_set_user_sources (FontManagerPreferencePane* self,
                                                    FontManagerUserSourceModel* value);
VALA_EXTERN FontManagerSubstituteList* font_manager_substitute_list_new (void);
VALA_EXTERN FontManagerSubstituteList* font_manager_substitute_list_construct (GType object_type);
VALA_EXTERN GType font_manager_substitute_list_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerSubstituteList, g_object_unref)
VALA_EXTERN FontManagerDisplayPreferences* font_manager_display_preferences_new (void);
VALA_EXTERN FontManagerDisplayPreferences* font_manager_display_preferences_construct (GType object_type);
VALA_EXTERN GType font_manager_display_preferences_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerDisplayPreferences, g_object_unref)
VALA_EXTERN FontManagerRenderingPreferences* font_manager_rendering_preferences_new (void);
VALA_EXTERN FontManagerRenderingPreferences* font_manager_rendering_preferences_construct (GType object_type);
VALA_EXTERN GType font_manager_rendering_preferences_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerRenderingPreferences, g_object_unref)
VALA_EXTERN GtkWidget* font_manager_preference_pane_get (FontManagerPreferencePane* self,
                                             const gchar* name);
VALA_EXTERN FontManagerUserActionModel* font_manager_preference_pane_get_user_actions (FontManagerPreferencePane* self);
VALA_EXTERN FontManagerUserSourceModel* font_manager_preference_pane_get_user_sources (FontManagerPreferencePane* self);
static void font_manager_preference_pane_finalize (GObject * obj);
static GType font_manager_preference_pane_get_type_once (void);
static void _vala_font_manager_preference_pane_get_property (GObject * object,
                                                      guint property_id,
                                                      GValue * value,
                                                      GParamSpec * pspec);
static void _vala_font_manager_preference_pane_set_property (GObject * object,
                                                      guint property_id,
                                                      const GValue * value,
                                                      GParamSpec * pspec);
VALA_EXTERN GType font_manager_base_controls_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerBaseControls, g_object_unref)
VALA_EXTERN void font_manager_preference_list_on_add_selected (FontManagerPreferenceList* self);
VALA_EXTERN void font_manager_preference_list_on_list_row_selected (FontManagerPreferenceList* self,
                                                        GtkListBox* box,
                                                        GtkListBoxRow* row);
VALA_EXTERN void font_manager_preference_list_on_map (FontManagerPreferenceList* self);
VALA_EXTERN void font_manager_preference_list_on_remove_selected (FontManagerPreferenceList* self);
VALA_EXTERN void font_manager_preference_list_on_unmap (FontManagerPreferenceList* self);
static void font_manager_preference_list_real_on_add_selected (FontManagerPreferenceList* self);
static void _font_manager_preference_list_on_add_selected_font_manager_base_controls_add_selected (FontManagerBaseControls* _sender,
                                                                                            gpointer self);
static void font_manager_preference_list_real_on_list_row_selected (FontManagerPreferenceList* self,
                                                             GtkListBox* box,
                                                             GtkListBoxRow* row);
VALA_EXTERN void font_manager_set_control_sensitivity (GtkWidget* widget,
                                           gboolean sensitive);
VALA_EXTERN GtkButton* font_manager_base_controls_get_remove_button (FontManagerBaseControls* self);
static void _font_manager_preference_list_on_list_row_selected_gtk_list_box_row_selected (GtkListBox* _sender,
                                                                                   GtkListBoxRow* row,
                                                                                   gpointer self);
static void font_manager_preference_list_real_on_map (FontManagerPreferenceList* self);
VALA_EXTERN void font_manager_preference_list_select_first_row (FontManagerPreferenceList* self);
static void _font_manager_preference_list_on_map_gtk_widget_map (GtkWidget* _sender,
                                                          gpointer self);
static void font_manager_preference_list_real_on_remove_selected (FontManagerPreferenceList* self);
static void _font_manager_preference_list_on_remove_selected_font_manager_base_controls_remove_selected (FontManagerBaseControls* _sender,
                                                                                                  gpointer self);
static void font_manager_preference_list_real_on_unmap (FontManagerPreferenceList* self);
static void _font_manager_preference_list_on_unmap_gtk_widget_unmap (GtkWidget* _sender,
                                                              gpointer self);
VALA_EXTERN GtkSwitch* font_manager_preference_list_add_preference_switch (FontManagerPreferenceList* self,
                                                               const gchar* name,
                                                               const gchar* tooltip);
VALA_EXTERN void font_manager_preference_list_append_row (FontManagerPreferenceList* self,
                                              GtkWidget* widget);
VALA_EXTERN FontManagerPreferenceList* font_manager_preference_list_new (void);
VALA_EXTERN FontManagerPreferenceList* font_manager_preference_list_construct (GType object_type);
static GObject * font_manager_preference_list_constructor (GType type,
                                                    guint n_construct_properties,
                                                    GObjectConstructParam * construct_properties);
VALA_EXTERN GtkButton* font_manager_base_controls_get_add_button (FontManagerBaseControls* self);
static void font_manager_preference_list_finalize (GObject * obj);
static GType font_manager_preference_list_get_type_once (void);
VALA_EXTERN GType font_manager_fontconfig_footer_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerFontconfigFooter, g_object_unref)
VALA_EXTERN FontManagerFontconfigFooter* font_manager_fontconfig_footer_new (void);
VALA_EXTERN FontManagerFontconfigFooter* font_manager_fontconfig_footer_construct (GType object_type);
VALA_EXTERN GtkImage* font_manager_inline_help_widget (const gchar* message);
static void __lambda162_ (FontManagerFontconfigFooter* self);
static void ___lambda162__gtk_button_clicked (GtkButton* _sender,
                                       gpointer self);
static GType font_manager_fontconfig_footer_get_type_once (void);
VALA_EXTERN GType font_manager_subpixel_geometry_icon_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerSubpixelGeometryIcon, g_object_unref)
VALA_EXTERN FontManagerSubpixelGeometryIcon* font_manager_subpixel_geometry_icon_new (FontManagerSubpixelOrder rgba);
VALA_EXTERN FontManagerSubpixelGeometryIcon* font_manager_subpixel_geometry_icon_construct (GType object_type,
                                                                                FontManagerSubpixelOrder rgba);
static void font_manager_subpixel_geometry_icon_set_child_size_request (FontManagerSubpixelGeometryIcon* self);
static void __lambda150_ (FontManagerSubpixelGeometryIcon* self);
static void ___lambda150__g_object_notify (GObject* _sender,
                                    GParamSpec* pspec,
                                    gpointer self);
VALA_EXTERN gint font_manager_subpixel_geometry_icon_get_preferred_size (FontManagerSubpixelGeometryIcon* self);
static void font_manager_subpixel_geometry_icon_real_measure (GtkWidget* base,
                                                       GtkOrientation orientation,
                                                       gint for_size,
                                                       gint* minimum,
                                                       gint* natural,
                                                       gint* minimum_baseline,
                                                       gint* natural_baseline);
VALA_EXTERN void font_manager_subpixel_geometry_icon_set_preferred_size (FontManagerSubpixelGeometryIcon* self,
                                                             gint value);
static void font_manager_subpixel_geometry_icon_finalize (GObject * obj);
static GType font_manager_subpixel_geometry_icon_get_type_once (void);
static void _vala_font_manager_subpixel_geometry_icon_get_property (GObject * object,
                                                             guint property_id,
                                                             GValue * value,
                                                             GParamSpec * pspec);
static void _vala_font_manager_subpixel_geometry_icon_set_property (GObject * object,
                                                             guint property_id,
                                                             const GValue * value,
                                                             GParamSpec * pspec);
VALA_EXTERN GType font_manager_subpixel_geometry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerSubpixelGeometry, g_object_unref)
VALA_EXTERN FontManagerSubpixelGeometry* font_manager_subpixel_geometry_new (void);
VALA_EXTERN FontManagerSubpixelGeometry* font_manager_subpixel_geometry_construct (GType object_type);
static void _g_object_unref0_ (gpointer var);
static void font_manager_subpixel_geometry_set_options (FontManagerSubpixelGeometry* self,
                                                 GPtrArray* value);
static Block27Data* block27_data_ref (Block27Data* _data27_);
static void block27_data_unref (void * _userdata_);
VALA_EXTERN GPtrArray* font_manager_subpixel_geometry_get_options (FontManagerSubpixelGeometry* self);
static void _____lambda151_ (Block27Data* _data27_);
VALA_EXTERN void font_manager_subpixel_geometry_set_selected (FontManagerSubpixelGeometry* self,
                                                  guint value);
static void ______lambda151__gtk_check_button_toggled (GtkCheckButton* _sender,
                                                gpointer self);
VALA_EXTERN guint font_manager_subpixel_geometry_get_selected (FontManagerSubpixelGeometry* self);
VALA_EXTERN gint font_manager_subpixel_geometry_get_preferred_size (FontManagerSubpixelGeometry* self);
VALA_EXTERN void font_manager_subpixel_geometry_set_preferred_size (FontManagerSubpixelGeometry* self,
                                                        gint value);
static void font_manager_subpixel_geometry_finalize (GObject * obj);
static GType font_manager_subpixel_geometry_get_type_once (void);
static void _vala_font_manager_subpixel_geometry_get_property (GObject * object,
                                                        guint property_id,
                                                        GValue * value,
                                                        GParamSpec * pspec);
static void _vala_font_manager_subpixel_geometry_set_property (GObject * object,
                                                        guint property_id,
                                                        const GValue * value,
                                                        GParamSpec * pspec);
static void _vala_array_destroy (gpointer array,
                          gssize array_length,
                          GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array,
                       gssize array_length,
                       GDestroyNotify destroy_func);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static inline gpointer
font_manager_preference_pane_get_instance_private (FontManagerPreferencePane* self)
{
	return G_STRUCT_MEMBER_P (self, FontManagerPreferencePane_private_offset);
}

FontManagerPreferencePane*
font_manager_preference_pane_construct (GType object_type,
                                        GSettings* settings)
{
	FontManagerPreferencePane * self = NULL;
	GtkStack* _tmp0_;
	GtkStackSidebar* _tmp1_;
	GtkStackSidebar* _tmp2_;
	GtkStackSidebar* _tmp3_;
	GtkStack* _tmp4_;
	GtkStackSidebar* _tmp5_;
	GtkStack* _tmp6_;
	FontManagerUserInterfacePreferences* _tmp7_;
	FontManagerUserInterfacePreferences* _tmp8_;
	FontManagerUserInterfacePreferences* _tmp9_;
	FontManagerDesktopPreferences* _tmp10_;
	FontManagerDesktopPreferences* _tmp11_;
	FontManagerUserActionList* actions = NULL;
	FontManagerUserActionList* _tmp12_;
	FontManagerUserActionModel* _tmp13_;
	FontManagerUserActionModel* _tmp14_;
	FontManagerUserSourceList* sources = NULL;
	FontManagerUserSourceList* _tmp15_;
	FontManagerUserSourceModel* _tmp16_;
	FontManagerUserSourceModel* _tmp17_;
	FontManagerSubstituteList* _tmp18_;
	FontManagerSubstituteList* _tmp19_;
	FontManagerDisplayPreferences* _tmp20_;
	FontManagerDisplayPreferences* _tmp21_;
	FontManagerRenderingPreferences* _tmp22_;
	FontManagerRenderingPreferences* _tmp23_;
	self = (FontManagerPreferencePane*) font_manager_paned_construct (object_type);
	font_manager_widget_set_name ((GtkWidget*) self, "FontManagerPreferencePane");
	_tmp0_ = (GtkStack*) gtk_stack_new ();
	g_object_ref_sink (_tmp0_);
	_g_object_unref0 (self->priv->stack);
	self->priv->stack = _tmp0_;
	_tmp1_ = (GtkStackSidebar*) gtk_stack_sidebar_new ();
	g_object_ref_sink (_tmp1_);
	_g_object_unref0 (self->priv->sidebar);
	self->priv->sidebar = _tmp1_;
	_tmp2_ = self->priv->sidebar;
	gtk_widget_add_css_class ((GtkWidget*) _tmp2_, "view");
	_tmp3_ = self->priv->sidebar;
	_tmp4_ = self->priv->stack;
	gtk_stack_sidebar_set_stack (_tmp3_, _tmp4_);
	_tmp5_ = self->priv->sidebar;
	font_manager_paned_set_start_child ((FontManagerPaned*) self, (GtkWidget*) _tmp5_);
	_tmp6_ = self->priv->stack;
	font_manager_paned_set_end_child ((FontManagerPaned*) self, (GtkWidget*) _tmp6_);
	_tmp7_ = font_manager_user_interface_preferences_new (settings);
	g_object_ref_sink (_tmp7_);
	_tmp8_ = _tmp7_;
	font_manager_preference_pane_set_interface_preferences (self, _tmp8_);
	_g_object_unref0 (_tmp8_);
	_tmp9_ = self->priv->_interface_preferences;
	font_manager_preference_pane_add_page (self, (GtkWidget*) _tmp9_, "Interface", _ ("Interface"));
	_tmp10_ = font_manager_desktop_preferences_new ();
	g_object_ref_sink (_tmp10_);
	_tmp11_ = _tmp10_;
	font_manager_preference_pane_add_page (self, (GtkWidget*) _tmp11_, "Desktop", _ ("Desktop"));
	_g_object_unref0 (_tmp11_);
	_tmp12_ = font_manager_user_action_list_new ();
	g_object_ref_sink (_tmp12_);
	actions = _tmp12_;
	_tmp13_ = font_manager_user_action_list_get_model (actions);
	_tmp14_ = _tmp13_;
	font_manager_preference_pane_set_user_actions (self, _tmp14_);
	font_manager_preference_pane_add_page (self, (GtkWidget*) actions, "UserActions", _ ("Actions"));
	_tmp15_ = font_manager_user_source_list_new ();
	g_object_ref_sink (_tmp15_);
	sources = _tmp15_;
	_tmp16_ = font_manager_user_source_list_get_model (sources);
	_tmp17_ = _tmp16_;
	font_manager_preference_pane_set_user_sources (self, _tmp17_);
	font_manager_preference_pane_add_page (self, (GtkWidget*) sources, "Sources", _ ("Sources"));
	_tmp18_ = font_manager_substitute_list_new ();
	g_object_ref_sink (_tmp18_);
	_tmp19_ = _tmp18_;
	font_manager_preference_pane_add_page (self, (GtkWidget*) _tmp19_, "Substitutions", _ ("Substitutions"));
	_g_object_unref0 (_tmp19_);
	_tmp20_ = font_manager_display_preferences_new ();
	g_object_ref_sink (_tmp20_);
	_tmp21_ = _tmp20_;
	font_manager_preference_pane_add_page (self, (GtkWidget*) _tmp21_, "Display", _ ("Display"));
	_g_object_unref0 (_tmp21_);
	_tmp22_ = font_manager_rendering_preferences_new ();
	g_object_ref_sink (_tmp22_);
	_tmp23_ = _tmp22_;
	font_manager_preference_pane_add_page (self, (GtkWidget*) _tmp23_, "Rendering", _ ("Rendering"));
	_g_object_unref0 (_tmp23_);
	_g_object_unref0 (sources);
	_g_object_unref0 (actions);
	return self;
}

FontManagerPreferencePane*
font_manager_preference_pane_new (GSettings* settings)
{
	return font_manager_preference_pane_construct (FONT_MANAGER_TYPE_PREFERENCE_PANE, settings);
}

void
font_manager_preference_pane_add_page (FontManagerPreferencePane* self,
                                       GtkWidget* widget,
                                       const gchar* name,
                                       const gchar* title)
{
	GtkScrolledWindow* scroll = NULL;
	GtkScrolledWindow* _tmp0_;
	GtkStack* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	g_return_if_fail (name != NULL);
	g_return_if_fail (title != NULL);
	_tmp0_ = (GtkScrolledWindow*) gtk_scrolled_window_new ();
	g_object_ref_sink (_tmp0_);
	scroll = _tmp0_;
	gtk_scrolled_window_set_policy (scroll, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_child (scroll, widget);
	_tmp1_ = self->priv->stack;
	gtk_stack_add_titled (_tmp1_, (GtkWidget*) scroll, name, title);
	_g_object_unref0 (scroll);
	return;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

GtkWidget*
font_manager_preference_pane_get (FontManagerPreferencePane* self,
                                  const gchar* name)
{
	GtkWidget* child = NULL;
	GtkStack* _tmp0_;
	GtkWidget* _tmp1_;
	GtkWidget* _tmp2_;
	GtkWidget* _tmp3_;
	GtkWidget* _tmp4_;
	GtkWidget* _tmp8_ = NULL;
	GtkWidget* _tmp9_;
	GtkWidget* _tmp13_;
	GtkWidget* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	_tmp0_ = self->priv->stack;
	_tmp1_ = gtk_stack_get_child_by_name (_tmp0_, name);
	_tmp2_ = _g_object_ref0 (_tmp1_);
	child = _tmp2_;
	_tmp3_ = child;
	g_return_val_if_fail (_tmp3_ != NULL, NULL);
	_tmp4_ = child;
	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp4_, gtk_scrolled_window_get_type ())) {
		GtkWidget* _tmp5_;
		GtkWidget* _tmp6_;
		GtkWidget* _tmp7_;
		_tmp5_ = child;
		_tmp6_ = gtk_scrolled_window_get_child ((GtkScrolledWindow*) _tmp5_);
		_tmp7_ = _g_object_ref0 (_tmp6_);
		_g_object_unref0 (child);
		child = _tmp7_;
	}
	_tmp9_ = child;
	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp9_, gtk_viewport_get_type ())) {
		GtkWidget* _tmp10_;
		GtkWidget* _tmp11_;
		_tmp10_ = child;
		_tmp11_ = gtk_viewport_get_child ((GtkViewport*) _tmp10_);
		_tmp8_ = _tmp11_;
	} else {
		GtkWidget* _tmp12_;
		_tmp12_ = child;
		_tmp8_ = _tmp12_;
	}
	_tmp13_ = _g_object_ref0 (_tmp8_);
	result = _tmp13_;
	_g_object_unref0 (child);
	return result;
}

FontManagerUserActionModel*
font_manager_preference_pane_get_user_actions (FontManagerPreferencePane* self)
{
	FontManagerUserActionModel* result;
	FontManagerUserActionModel* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_user_actions;
	result = _tmp0_;
	return result;
}

static void
font_manager_preference_pane_set_user_actions (FontManagerPreferencePane* self,
                                               FontManagerUserActionModel* value)
{
	FontManagerUserActionModel* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_preference_pane_get_user_actions (self);
	if (old_value != value) {
		FontManagerUserActionModel* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_user_actions);
		self->priv->_user_actions = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_preference_pane_properties[FONT_MANAGER_PREFERENCE_PANE_USER_ACTIONS_PROPERTY]);
	}
}

FontManagerUserSourceModel*
font_manager_preference_pane_get_user_sources (FontManagerPreferencePane* self)
{
	FontManagerUserSourceModel* result;
	FontManagerUserSourceModel* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_user_sources;
	result = _tmp0_;
	return result;
}

static void
font_manager_preference_pane_set_user_sources (FontManagerPreferencePane* self,
                                               FontManagerUserSourceModel* value)
{
	FontManagerUserSourceModel* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_preference_pane_get_user_sources (self);
	if (old_value != value) {
		FontManagerUserSourceModel* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_user_sources);
		self->priv->_user_sources = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_preference_pane_properties[FONT_MANAGER_PREFERENCE_PANE_USER_SOURCES_PROPERTY]);
	}
}

FontManagerUserInterfacePreferences*
font_manager_preference_pane_get_interface_preferences (FontManagerPreferencePane* self)
{
	FontManagerUserInterfacePreferences* result;
	FontManagerUserInterfacePreferences* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_interface_preferences;
	result = _tmp0_;
	return result;
}

static void
font_manager_preference_pane_set_interface_preferences (FontManagerPreferencePane* self,
                                                        FontManagerUserInterfacePreferences* value)
{
	FontManagerUserInterfacePreferences* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_preference_pane_get_interface_preferences (self);
	if (old_value != value) {
		FontManagerUserInterfacePreferences* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_interface_preferences);
		self->priv->_interface_preferences = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_preference_pane_properties[FONT_MANAGER_PREFERENCE_PANE_INTERFACE_PREFERENCES_PROPERTY]);
	}
}

static void
font_manager_preference_pane_class_init (FontManagerPreferencePaneClass * klass,
                                         gpointer klass_data)
{
	font_manager_preference_pane_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &FontManagerPreferencePane_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_font_manager_preference_pane_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_font_manager_preference_pane_set_property;
	G_OBJECT_CLASS (klass)->finalize = font_manager_preference_pane_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_PREFERENCE_PANE_USER_ACTIONS_PROPERTY, font_manager_preference_pane_properties[FONT_MANAGER_PREFERENCE_PANE_USER_ACTIONS_PROPERTY] = g_param_spec_object ("user-actions", "user-actions", "user-actions", FONT_MANAGER_TYPE_USER_ACTION_MODEL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_PREFERENCE_PANE_USER_SOURCES_PROPERTY, font_manager_preference_pane_properties[FONT_MANAGER_PREFERENCE_PANE_USER_SOURCES_PROPERTY] = g_param_spec_object ("user-sources", "user-sources", "user-sources", FONT_MANAGER_TYPE_USER_SOURCE_MODEL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_PREFERENCE_PANE_INTERFACE_PREFERENCES_PROPERTY, font_manager_preference_pane_properties[FONT_MANAGER_PREFERENCE_PANE_INTERFACE_PREFERENCES_PROPERTY] = g_param_spec_object ("interface-preferences", "interface-preferences", "interface-preferences", FONT_MANAGER_TYPE_USER_INTERFACE_PREFERENCES, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
font_manager_preference_pane_instance_init (FontManagerPreferencePane * self,
                                            gpointer klass)
{
	self->priv = font_manager_preference_pane_get_instance_private (self);
}

static void
font_manager_preference_pane_finalize (GObject * obj)
{
	FontManagerPreferencePane * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_TYPE_PREFERENCE_PANE, FontManagerPreferencePane);
	_g_object_unref0 (self->priv->_user_actions);
	_g_object_unref0 (self->priv->_user_sources);
	_g_object_unref0 (self->priv->_interface_preferences);
	_g_object_unref0 (self->priv->stack);
	_g_object_unref0 (self->priv->sidebar);
	G_OBJECT_CLASS (font_manager_preference_pane_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
font_manager_preference_pane_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (FontManagerPreferencePaneClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) font_manager_preference_pane_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FontManagerPreferencePane), 0, (GInstanceInitFunc) font_manager_preference_pane_instance_init, NULL };
	GType font_manager_preference_pane_type_id;
	font_manager_preference_pane_type_id = g_type_register_static (FONT_MANAGER_TYPE_PANED, "FontManagerPreferencePane", &g_define_type_info, 0);
	FontManagerPreferencePane_private_offset = g_type_add_instance_private (font_manager_preference_pane_type_id, sizeof (FontManagerPreferencePanePrivate));
	return font_manager_preference_pane_type_id;
}

GType
font_manager_preference_pane_get_type (void)
{
	static volatile gsize font_manager_preference_pane_type_id__once = 0;
	if (g_once_init_enter (&font_manager_preference_pane_type_id__once)) {
		GType font_manager_preference_pane_type_id;
		font_manager_preference_pane_type_id = font_manager_preference_pane_get_type_once ();
		g_once_init_leave (&font_manager_preference_pane_type_id__once, font_manager_preference_pane_type_id);
	}
	return font_manager_preference_pane_type_id__once;
}

static void
_vala_font_manager_preference_pane_get_property (GObject * object,
                                                 guint property_id,
                                                 GValue * value,
                                                 GParamSpec * pspec)
{
	FontManagerPreferencePane * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_TYPE_PREFERENCE_PANE, FontManagerPreferencePane);
	switch (property_id) {
		case FONT_MANAGER_PREFERENCE_PANE_USER_ACTIONS_PROPERTY:
		g_value_set_object (value, font_manager_preference_pane_get_user_actions (self));
		break;
		case FONT_MANAGER_PREFERENCE_PANE_USER_SOURCES_PROPERTY:
		g_value_set_object (value, font_manager_preference_pane_get_user_sources (self));
		break;
		case FONT_MANAGER_PREFERENCE_PANE_INTERFACE_PREFERENCES_PROPERTY:
		g_value_set_object (value, font_manager_preference_pane_get_interface_preferences (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_font_manager_preference_pane_set_property (GObject * object,
                                                 guint property_id,
                                                 const GValue * value,
                                                 GParamSpec * pspec)
{
	FontManagerPreferencePane * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_TYPE_PREFERENCE_PANE, FontManagerPreferencePane);
	switch (property_id) {
		case FONT_MANAGER_PREFERENCE_PANE_USER_ACTIONS_PROPERTY:
		font_manager_preference_pane_set_user_actions (self, g_value_get_object (value));
		break;
		case FONT_MANAGER_PREFERENCE_PANE_USER_SOURCES_PROPERTY:
		font_manager_preference_pane_set_user_sources (self, g_value_get_object (value));
		break;
		case FONT_MANAGER_PREFERENCE_PANE_INTERFACE_PREFERENCES_PROPERTY:
		font_manager_preference_pane_set_interface_preferences (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
font_manager_preference_list_real_on_add_selected (FontManagerPreferenceList* self)
{
}

void
font_manager_preference_list_on_add_selected (FontManagerPreferenceList* self)
{
	FontManagerPreferenceListClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = FONT_MANAGER_PREFERENCE_LIST_GET_CLASS (self);
	if (_klass_->on_add_selected) {
		_klass_->on_add_selected (self);
	}
}

static void
_font_manager_preference_list_on_add_selected_font_manager_base_controls_add_selected (FontManagerBaseControls* _sender,
                                                                                       gpointer self)
{
	font_manager_preference_list_on_add_selected ((FontManagerPreferenceList*) self);
}

static void
font_manager_preference_list_real_on_list_row_selected (FontManagerPreferenceList* self,
                                                        GtkListBox* box,
                                                        GtkListBoxRow* row)
{
	FontManagerBaseControls* _tmp0_;
	GtkButton* _tmp1_;
	GtkButton* _tmp2_;
	g_return_if_fail (box != NULL);
	_tmp0_ = self->controls;
	_tmp1_ = font_manager_base_controls_get_remove_button (_tmp0_);
	_tmp2_ = _tmp1_;
	font_manager_set_control_sensitivity ((GtkWidget*) _tmp2_, row != NULL);
	return;
}

void
font_manager_preference_list_on_list_row_selected (FontManagerPreferenceList* self,
                                                   GtkListBox* box,
                                                   GtkListBoxRow* row)
{
	FontManagerPreferenceListClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = FONT_MANAGER_PREFERENCE_LIST_GET_CLASS (self);
	if (_klass_->on_list_row_selected) {
		_klass_->on_list_row_selected (self, box, row);
	}
}

static void
_font_manager_preference_list_on_list_row_selected_gtk_list_box_row_selected (GtkListBox* _sender,
                                                                              GtkListBoxRow* row,
                                                                              gpointer self)
{
	font_manager_preference_list_on_list_row_selected ((FontManagerPreferenceList*) self, _sender, row);
}

static void
font_manager_preference_list_real_on_map (FontManagerPreferenceList* self)
{
	font_manager_preference_list_select_first_row (self);
	return;
}

void
font_manager_preference_list_on_map (FontManagerPreferenceList* self)
{
	FontManagerPreferenceListClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = FONT_MANAGER_PREFERENCE_LIST_GET_CLASS (self);
	if (_klass_->on_map) {
		_klass_->on_map (self);
	}
}

static void
_font_manager_preference_list_on_map_gtk_widget_map (GtkWidget* _sender,
                                                     gpointer self)
{
	font_manager_preference_list_on_map ((FontManagerPreferenceList*) self);
}

static void
font_manager_preference_list_real_on_remove_selected (FontManagerPreferenceList* self)
{
}

void
font_manager_preference_list_on_remove_selected (FontManagerPreferenceList* self)
{
	FontManagerPreferenceListClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = FONT_MANAGER_PREFERENCE_LIST_GET_CLASS (self);
	if (_klass_->on_remove_selected) {
		_klass_->on_remove_selected (self);
	}
}

static void
_font_manager_preference_list_on_remove_selected_font_manager_base_controls_remove_selected (FontManagerBaseControls* _sender,
                                                                                             gpointer self)
{
	font_manager_preference_list_on_remove_selected ((FontManagerPreferenceList*) self);
}

static void
font_manager_preference_list_real_on_unmap (FontManagerPreferenceList* self)
{
}

void
font_manager_preference_list_on_unmap (FontManagerPreferenceList* self)
{
	FontManagerPreferenceListClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = FONT_MANAGER_PREFERENCE_LIST_GET_CLASS (self);
	if (_klass_->on_unmap) {
		_klass_->on_unmap (self);
	}
}

static void
_font_manager_preference_list_on_unmap_gtk_widget_unmap (GtkWidget* _sender,
                                                         gpointer self)
{
	font_manager_preference_list_on_unmap ((FontManagerPreferenceList*) self);
}

void
font_manager_preference_list_select_first_row (FontManagerPreferenceList* self)
{
	GtkListBox* _tmp0_;
	GtkListBox* _tmp1_;
	GtkListBoxRow* _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->list;
	_tmp1_ = self->list;
	_tmp2_ = gtk_list_box_get_row_at_index (_tmp1_, 0);
	gtk_list_box_select_row (_tmp0_, _tmp2_);
	return;
}

GtkSwitch*
font_manager_preference_list_add_preference_switch (FontManagerPreferenceList* self,
                                                    const gchar* name,
                                                    const gchar* tooltip)
{
	GtkSwitch* control = NULL;
	GtkSwitch* _tmp0_;
	FontManagerPreferenceRow* row = NULL;
	FontManagerPreferenceRow* _tmp1_;
	GtkSwitch* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	_tmp0_ = (GtkSwitch*) gtk_switch_new ();
	g_object_ref_sink (_tmp0_);
	control = _tmp0_;
	_tmp1_ = (FontManagerPreferenceRow*) font_manager_preference_row_new (name, NULL, NULL, (GtkWidget*) control);
	g_object_ref_sink (_tmp1_);
	row = _tmp1_;
	gtk_widget_set_tooltip_text ((GtkWidget*) row, tooltip);
	font_manager_preference_list_append_row (self, (GtkWidget*) row);
	result = control;
	_g_object_unref0 (row);
	return result;
}

void
font_manager_preference_list_append_row (FontManagerPreferenceList* self,
                                         GtkWidget* widget)
{
	GtkListBoxRow* row = NULL;
	GtkListBoxRow* _tmp0_ = NULL;
	GtkListBox* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	_tmp0_ = (GtkListBoxRow*) gtk_list_box_row_new ();
	gtk_list_box_row_set_activatable (_tmp0_, FALSE);
	gtk_list_box_row_set_selectable (_tmp0_, FALSE);
	g_object_ref_sink (_tmp0_);
	row = _tmp0_;
	gtk_list_box_row_set_child (row, widget);
	_tmp1_ = self->list;
	gtk_list_box_insert (_tmp1_, (GtkWidget*) row, -1);
	_g_object_unref0 (row);
	return;
}

FontManagerPreferenceList*
font_manager_preference_list_construct (GType object_type)
{
	FontManagerPreferenceList * self = NULL;
	self = (FontManagerPreferenceList*) g_object_new (object_type, NULL);
	return self;
}

FontManagerPreferenceList*
font_manager_preference_list_new (void)
{
	return font_manager_preference_list_construct (FONT_MANAGER_TYPE_PREFERENCE_LIST);
}

static GObject *
font_manager_preference_list_constructor (GType type,
                                          guint n_construct_properties,
                                          GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	FontManagerPreferenceList * self;
	FontManagerBaseControls* _tmp0_;
	GtkButton* _tmp1_;
	GtkButton* _tmp2_;
	FontManagerBaseControls* _tmp3_;
	GtkButton* _tmp4_;
	GtkButton* _tmp5_;
	GBindingFlags flags = 0U;
	FontManagerBaseControls* _tmp6_;
	GtkSeparator* _tmp7_;
	parent_class = G_OBJECT_CLASS (font_manager_preference_list_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_TYPE_PREFERENCE_LIST, FontManagerPreferenceList);
	_tmp0_ = self->controls;
	_tmp1_ = font_manager_base_controls_get_add_button (_tmp0_);
	_tmp2_ = _tmp1_;
	font_manager_set_control_sensitivity ((GtkWidget*) _tmp2_, TRUE);
	_tmp3_ = self->controls;
	_tmp4_ = font_manager_base_controls_get_remove_button (_tmp3_);
	_tmp5_ = _tmp4_;
	font_manager_set_control_sensitivity ((GtkWidget*) _tmp5_, FALSE);
	flags = G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE;
	_tmp6_ = self->controls;
	_tmp7_ = self->separator;
	g_object_bind_property_with_closures ((GObject*) _tmp6_, "visible", (GObject*) _tmp7_, "visible", flags, (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, (GClosureNotify) NULL)), (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, (GClosureNotify) NULL)));
	return obj;
}

static void
font_manager_preference_list_class_init (FontManagerPreferenceListClass * klass,
                                         gpointer klass_data)
{
	font_manager_preference_list_parent_class = g_type_class_peek_parent (klass);
	((FontManagerPreferenceListClass *) klass)->on_add_selected = (void (*) (FontManagerPreferenceList*)) font_manager_preference_list_real_on_add_selected;
	((FontManagerPreferenceListClass *) klass)->on_list_row_selected = (void (*) (FontManagerPreferenceList*, GtkListBox*, GtkListBoxRow*)) font_manager_preference_list_real_on_list_row_selected;
	((FontManagerPreferenceListClass *) klass)->on_map = (void (*) (FontManagerPreferenceList*)) font_manager_preference_list_real_on_map;
	((FontManagerPreferenceListClass *) klass)->on_remove_selected = (void (*) (FontManagerPreferenceList*)) font_manager_preference_list_real_on_remove_selected;
	((FontManagerPreferenceListClass *) klass)->on_unmap = (void (*) (FontManagerPreferenceList*)) font_manager_preference_list_real_on_unmap;
	G_OBJECT_CLASS (klass)->constructor = font_manager_preference_list_constructor;
	G_OBJECT_CLASS (klass)->finalize = font_manager_preference_list_finalize;
	gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/com/github/FontManager/FontManager/ui/font-manager-preference-list.ui");
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "list", FALSE, G_STRUCT_OFFSET (FontManagerPreferenceList, list));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "controls", FALSE, G_STRUCT_OFFSET (FontManagerPreferenceList, controls));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "separator", FALSE, G_STRUCT_OFFSET (FontManagerPreferenceList, separator));
	gtk_widget_class_bind_template_callback_full (GTK_WIDGET_CLASS (klass), "on_add_selected", G_CALLBACK(_font_manager_preference_list_on_add_selected_font_manager_base_controls_add_selected));
	gtk_widget_class_bind_template_callback_full (GTK_WIDGET_CLASS (klass), "on_list_row_selected", G_CALLBACK(_font_manager_preference_list_on_list_row_selected_gtk_list_box_row_selected));
	gtk_widget_class_bind_template_callback_full (GTK_WIDGET_CLASS (klass), "on_map", G_CALLBACK(_font_manager_preference_list_on_map_gtk_widget_map));
	gtk_widget_class_bind_template_callback_full (GTK_WIDGET_CLASS (klass), "on_remove_selected", G_CALLBACK(_font_manager_preference_list_on_remove_selected_font_manager_base_controls_remove_selected));
	gtk_widget_class_bind_template_callback_full (GTK_WIDGET_CLASS (klass), "on_unmap", G_CALLBACK(_font_manager_preference_list_on_unmap_gtk_widget_unmap));
}

static void
font_manager_preference_list_instance_init (FontManagerPreferenceList * self,
                                            gpointer klass)
{
	g_type_ensure (FONT_MANAGER_TYPE_BASE_CONTROLS);
	gtk_widget_init_template (GTK_WIDGET (self));
}

static void
font_manager_preference_list_finalize (GObject * obj)
{
	FontManagerPreferenceList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_TYPE_PREFERENCE_LIST, FontManagerPreferenceList);
	G_OBJECT_CLASS (font_manager_preference_list_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
font_manager_preference_list_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (FontManagerPreferenceListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) font_manager_preference_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FontManagerPreferenceList), 0, (GInstanceInitFunc) font_manager_preference_list_instance_init, NULL };
	GType font_manager_preference_list_type_id;
	font_manager_preference_list_type_id = g_type_register_static (gtk_box_get_type (), "FontManagerPreferenceList", &g_define_type_info, 0);
	return font_manager_preference_list_type_id;
}

GType
font_manager_preference_list_get_type (void)
{
	static volatile gsize font_manager_preference_list_type_id__once = 0;
	if (g_once_init_enter (&font_manager_preference_list_type_id__once)) {
		GType font_manager_preference_list_type_id;
		font_manager_preference_list_type_id = font_manager_preference_list_get_type_once ();
		g_once_init_leave (&font_manager_preference_list_type_id__once, font_manager_preference_list_type_id);
	}
	return font_manager_preference_list_type_id__once;
}

static void
__lambda162_ (FontManagerFontconfigFooter* self)
{
	g_signal_emit (self, font_manager_fontconfig_footer_signals[FONT_MANAGER_FONTCONFIG_FOOTER_RESET_REQUESTED_SIGNAL], 0);
}

static void
___lambda162__gtk_button_clicked (GtkButton* _sender,
                                  gpointer self)
{
	__lambda162_ ((FontManagerFontconfigFooter*) self);
}

FontManagerFontconfigFooter*
font_manager_fontconfig_footer_construct (GType object_type)
{
	FontManagerFontconfigFooter * self = NULL;
	GtkImage* help = NULL;
	GtkImage* _tmp0_;
	GtkButton* reset = NULL;
	gchar* _tmp1_;
	GtkButton* _tmp2_ = NULL;
	self = (FontManagerFontconfigFooter*) g_object_new (object_type, NULL);
	gtk_orientable_set_orientation ((GtkOrientable*) self, GTK_ORIENTATION_HORIZONTAL);
	_tmp0_ = font_manager_inline_help_widget (FONT_MANAGER_FONTCONFIG_DISCLAIMER);
	help = _tmp0_;
	gtk_widget_set_halign ((GtkWidget*) help, GTK_ALIGN_START);
	font_manager_widget_set_margin ((GtkWidget*) help, FONT_MANAGER_DEFAULT_MARGIN * 2);
	gtk_widget_set_margin_start ((GtkWidget*) help, (FONT_MANAGER_DEFAULT_MARGIN * 3) - 2);
	gtk_widget_set_margin_bottom ((GtkWidget*) help, FONT_MANAGER_DEFAULT_MARGIN);
	gtk_box_append ((GtkBox*) self, (GtkWidget*) help);
	_tmp1_ = g_strdup (_ ("Reset all values to default"));
	_tmp2_ = (GtkButton*) gtk_button_new_from_icon_name ("edit-undo-symbolic");
	gtk_widget_set_opacity ((GtkWidget*) _tmp2_, 0.65);
	gtk_widget_set_hexpand ((GtkWidget*) _tmp2_, TRUE);
	gtk_widget_set_tooltip_text ((GtkWidget*) _tmp2_, _tmp1_);
	_g_free0 (_tmp1_);
	g_object_ref_sink (_tmp2_);
	reset = _tmp2_;
	font_manager_widget_set_align ((GtkWidget*) reset, GTK_ALIGN_END);
	font_manager_widget_set_margin ((GtkWidget*) reset, FONT_MANAGER_DEFAULT_MARGIN * 2);
	gtk_widget_add_css_class ((GtkWidget*) reset, "rounded");
	g_signal_connect_object (reset, "clicked", (GCallback) ___lambda162__gtk_button_clicked, self, 0);
	gtk_box_append ((GtkBox*) self, (GtkWidget*) reset);
	_g_object_unref0 (reset);
	_g_object_unref0 (help);
	return self;
}

FontManagerFontconfigFooter*
font_manager_fontconfig_footer_new (void)
{
	return font_manager_fontconfig_footer_construct (FONT_MANAGER_TYPE_FONTCONFIG_FOOTER);
}

static void
font_manager_fontconfig_footer_class_init (FontManagerFontconfigFooterClass * klass,
                                           gpointer klass_data)
{
	font_manager_fontconfig_footer_parent_class = g_type_class_peek_parent (klass);
	font_manager_fontconfig_footer_signals[FONT_MANAGER_FONTCONFIG_FOOTER_RESET_REQUESTED_SIGNAL] = g_signal_new ("reset-requested", FONT_MANAGER_TYPE_FONTCONFIG_FOOTER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}

static void
font_manager_fontconfig_footer_instance_init (FontManagerFontconfigFooter * self,
                                              gpointer klass)
{
}

 G_GNUC_NO_INLINE static GType
font_manager_fontconfig_footer_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (FontManagerFontconfigFooterClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) font_manager_fontconfig_footer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FontManagerFontconfigFooter), 0, (GInstanceInitFunc) font_manager_fontconfig_footer_instance_init, NULL };
	GType font_manager_fontconfig_footer_type_id;
	font_manager_fontconfig_footer_type_id = g_type_register_static (gtk_box_get_type (), "FontManagerFontconfigFooter", &g_define_type_info, 0);
	return font_manager_fontconfig_footer_type_id;
}

GType
font_manager_fontconfig_footer_get_type (void)
{
	static volatile gsize font_manager_fontconfig_footer_type_id__once = 0;
	if (g_once_init_enter (&font_manager_fontconfig_footer_type_id__once)) {
		GType font_manager_fontconfig_footer_type_id;
		font_manager_fontconfig_footer_type_id = font_manager_fontconfig_footer_get_type_once ();
		g_once_init_leave (&font_manager_fontconfig_footer_type_id__once, font_manager_fontconfig_footer_type_id);
	}
	return font_manager_fontconfig_footer_type_id__once;
}

static inline gpointer
font_manager_subpixel_geometry_icon_get_instance_private (FontManagerSubpixelGeometryIcon* self)
{
	return G_STRUCT_MEMBER_P (self, FontManagerSubpixelGeometryIcon_private_offset);
}

static void
__lambda150_ (FontManagerSubpixelGeometryIcon* self)
{
	font_manager_subpixel_geometry_icon_set_child_size_request (self);
}

static void
___lambda150__g_object_notify (GObject* _sender,
                               GParamSpec* pspec,
                               gpointer self)
{
	__lambda150_ ((FontManagerSubpixelGeometryIcon*) self);
}

FontManagerSubpixelGeometryIcon*
font_manager_subpixel_geometry_icon_construct (GType object_type,
                                               FontManagerSubpixelOrder rgba)
{
	FontManagerSubpixelGeometryIcon * self = NULL;
	gchar** colors = NULL;
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	gchar* _tmp7_;
	gchar* _tmp8_;
	gchar* _tmp9_;
	gchar* _tmp10_;
	gchar* _tmp11_;
	gchar* _tmp12_;
	gchar* _tmp13_;
	gchar* _tmp14_;
	gchar* _tmp15_;
	gchar* _tmp16_;
	gchar* _tmp17_;
	gchar** _tmp18_;
	gint colors_length1;
	gint colors_length2;
	gboolean _tmp19_ = FALSE;
	gboolean vertical = FALSE;
	GtkOrientation _tmp20_ = 0;
	self = (FontManagerSubpixelGeometryIcon*) g_object_new (object_type, NULL);
	gtk_box_set_homogeneous ((GtkBox*) self, TRUE);
	gtk_widget_set_vexpand ((GtkWidget*) self, TRUE);
	gtk_widget_set_hexpand ((GtkWidget*) self, TRUE);
	gtk_widget_set_valign ((GtkWidget*) self, GTK_ALIGN_CENTER);
	gtk_widget_set_halign ((GtkWidget*) self, GTK_ALIGN_CENTER);
	gtk_widget_set_margin_end ((GtkWidget*) self, FONT_MANAGER_DEFAULT_MARGIN);
	gtk_widget_set_margin_start ((GtkWidget*) self, FONT_MANAGER_DEFAULT_MARGIN);
	_tmp0_ = g_strdup ("gray");
	_tmp1_ = g_strdup ("gray");
	_tmp2_ = g_strdup ("gray");
	_tmp3_ = g_strdup ("red");
	_tmp4_ = g_strdup ("green");
	_tmp5_ = g_strdup ("blue");
	_tmp6_ = g_strdup ("blue");
	_tmp7_ = g_strdup ("green");
	_tmp8_ = g_strdup ("red");
	_tmp9_ = g_strdup ("red");
	_tmp10_ = g_strdup ("green");
	_tmp11_ = g_strdup ("blue");
	_tmp12_ = g_strdup ("blue");
	_tmp13_ = g_strdup ("green");
	_tmp14_ = g_strdup ("red");
	_tmp15_ = g_strdup ("gray");
	_tmp16_ = g_strdup ("gray");
	_tmp17_ = g_strdup ("gray");
	_tmp18_ = g_new0 (gchar*, (6 * 3) + 1);
	_tmp18_[0] = _tmp0_;
	_tmp18_[1] = _tmp1_;
	_tmp18_[2] = _tmp2_;
	_tmp18_[3] = _tmp3_;
	_tmp18_[4] = _tmp4_;
	_tmp18_[5] = _tmp5_;
	_tmp18_[6] = _tmp6_;
	_tmp18_[7] = _tmp7_;
	_tmp18_[8] = _tmp8_;
	_tmp18_[9] = _tmp9_;
	_tmp18_[10] = _tmp10_;
	_tmp18_[11] = _tmp11_;
	_tmp18_[12] = _tmp12_;
	_tmp18_[13] = _tmp13_;
	_tmp18_[14] = _tmp14_;
	_tmp18_[15] = _tmp15_;
	_tmp18_[16] = _tmp16_;
	_tmp18_[17] = _tmp17_;
	colors = _tmp18_;
	colors_length1 = 6;
	colors_length2 = 3;
	if (rgba == FONT_MANAGER_SUBPIXEL_ORDER_VBGR) {
		_tmp19_ = TRUE;
	} else {
		_tmp19_ = rgba == FONT_MANAGER_SUBPIXEL_ORDER_VRGB;
	}
	vertical = _tmp19_;
	if (vertical) {
		_tmp20_ = GTK_ORIENTATION_VERTICAL;
	} else {
		_tmp20_ = GTK_ORIENTATION_HORIZONTAL;
	}
	gtk_orientable_set_orientation ((GtkOrientable*) self, _tmp20_);
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp21_ = FALSE;
			_tmp21_ = TRUE;
			while (TRUE) {
				GtkDrawingArea* pixel = NULL;
				GtkDrawingArea* _tmp23_;
				GtkDrawingArea* _tmp24_;
				GtkDrawingArea* _tmp25_;
				gchar** _tmp26_;
				gint _tmp26__length1;
				gint _tmp26__length2;
				const gchar* _tmp27_;
				if (!_tmp21_) {
					gint _tmp22_;
					_tmp22_ = i;
					i = _tmp22_ + 1;
				}
				_tmp21_ = FALSE;
				if (!(i < 3)) {
					break;
				}
				_tmp23_ = (GtkDrawingArea*) gtk_drawing_area_new ();
				g_object_ref_sink (_tmp23_);
				pixel = _tmp23_;
				_tmp24_ = pixel;
				gtk_box_append ((GtkBox*) self, (GtkWidget*) _tmp24_);
				_tmp25_ = pixel;
				_tmp26_ = colors;
				_tmp26__length1 = colors_length1;
				_tmp26__length2 = colors_length2;
				_tmp27_ = _tmp26_[(rgba * _tmp26__length2) + i];
				gtk_widget_add_css_class ((GtkWidget*) _tmp25_, _tmp27_);
				_g_object_unref0 (pixel);
			}
		}
	}
	font_manager_subpixel_geometry_icon_set_child_size_request (self);
	g_signal_connect_object ((GObject*) self, "notify::preferred-size", (GCallback) ___lambda150__g_object_notify, self, 0);
	colors = (_vala_array_free (colors, colors_length1 * colors_length2, (GDestroyNotify) g_free), NULL);
	return self;
}

FontManagerSubpixelGeometryIcon*
font_manager_subpixel_geometry_icon_new (FontManagerSubpixelOrder rgba)
{
	return font_manager_subpixel_geometry_icon_construct (FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON, rgba);
}

static void
font_manager_subpixel_geometry_icon_set_child_size_request (FontManagerSubpixelGeometryIcon* self)
{
	gboolean vertical = FALSE;
	GtkOrientation _tmp0_;
	GtkOrientation _tmp1_;
	GtkWidget* child = NULL;
	GtkWidget* _tmp2_;
	GtkWidget* _tmp3_;
	g_return_if_fail (self != NULL);
	_tmp0_ = gtk_orientable_get_orientation ((GtkOrientable*) self);
	_tmp1_ = _tmp0_;
	vertical = _tmp1_ == GTK_ORIENTATION_VERTICAL;
	_tmp2_ = gtk_widget_get_first_child (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget));
	_tmp3_ = _g_object_ref0 (_tmp2_);
	child = _tmp3_;
	while (TRUE) {
		GtkWidget* _tmp4_;
		gint _tmp5_ = 0;
		gint _tmp8_ = 0;
		GtkWidget* _tmp11_;
		GtkWidget* _tmp12_;
		GtkWidget* _tmp13_;
		GtkWidget* _tmp14_;
		_tmp4_ = child;
		if (!(_tmp4_ != NULL)) {
			break;
		}
		if (vertical) {
			gint _tmp6_;
			_tmp6_ = self->priv->_preferred_size;
			_tmp5_ = _tmp6_;
		} else {
			gint _tmp7_;
			_tmp7_ = self->priv->_preferred_size;
			_tmp5_ = _tmp7_ / 3;
		}
		if (vertical) {
			gint _tmp9_;
			_tmp9_ = self->priv->_preferred_size;
			_tmp8_ = _tmp9_ / 3;
		} else {
			gint _tmp10_;
			_tmp10_ = self->priv->_preferred_size;
			_tmp8_ = _tmp10_;
		}
		_tmp11_ = child;
		gtk_widget_set_size_request (_tmp11_, _tmp5_, _tmp8_);
		_tmp12_ = child;
		_tmp13_ = gtk_widget_get_next_sibling (_tmp12_);
		_tmp14_ = _g_object_ref0 (_tmp13_);
		_g_object_unref0 (child);
		child = _tmp14_;
	}
	_g_object_unref0 (child);
	return;
}

static void
font_manager_subpixel_geometry_icon_real_measure (GtkWidget* base,
                                                  GtkOrientation orientation,
                                                  gint for_size,
                                                  gint* minimum,
                                                  gint* natural,
                                                  gint* minimum_baseline,
                                                  gint* natural_baseline)
{
	FontManagerSubpixelGeometryIcon * self;
	gint _vala_minimum = 0;
	gint _vala_natural = 0;
	gint _vala_minimum_baseline = 0;
	gint _vala_natural_baseline = 0;
	gint _tmp0_;
	self = (FontManagerSubpixelGeometryIcon*) base;
	_tmp0_ = self->priv->_preferred_size;
	_vala_natural = _tmp0_;
	_vala_minimum = _vala_natural;
	_vala_natural_baseline = -1;
	_vala_minimum_baseline = _vala_natural_baseline;
	if (minimum) {
		*minimum = _vala_minimum;
	}
	if (natural) {
		*natural = _vala_natural;
	}
	if (minimum_baseline) {
		*minimum_baseline = _vala_minimum_baseline;
	}
	if (natural_baseline) {
		*natural_baseline = _vala_natural_baseline;
	}
	return;
}

gint
font_manager_subpixel_geometry_icon_get_preferred_size (FontManagerSubpixelGeometryIcon* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_preferred_size;
	return result;
}

void
font_manager_subpixel_geometry_icon_set_preferred_size (FontManagerSubpixelGeometryIcon* self,
                                                        gint value)
{
	gint old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_subpixel_geometry_icon_get_preferred_size (self);
	if (old_value != value) {
		self->priv->_preferred_size = value;
		g_object_notify_by_pspec ((GObject *) self, font_manager_subpixel_geometry_icon_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_PREFERRED_SIZE_PROPERTY]);
	}
}

static void
font_manager_subpixel_geometry_icon_class_init (FontManagerSubpixelGeometryIconClass * klass,
                                                gpointer klass_data)
{
	font_manager_subpixel_geometry_icon_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &FontManagerSubpixelGeometryIcon_private_offset);
	((GtkWidgetClass *) klass)->measure = (void (*) (GtkWidget*, GtkOrientation, gint, gint*, gint*, gint*, gint*)) font_manager_subpixel_geometry_icon_real_measure;
	G_OBJECT_CLASS (klass)->get_property = _vala_font_manager_subpixel_geometry_icon_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_font_manager_subpixel_geometry_icon_set_property;
	G_OBJECT_CLASS (klass)->finalize = font_manager_subpixel_geometry_icon_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_PREFERRED_SIZE_PROPERTY, font_manager_subpixel_geometry_icon_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_PREFERRED_SIZE_PROPERTY] = g_param_spec_int ("preferred-size", "preferred-size", "preferred-size", G_MININT, G_MAXINT, 28, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
font_manager_subpixel_geometry_icon_instance_init (FontManagerSubpixelGeometryIcon * self,
                                                   gpointer klass)
{
	self->priv = font_manager_subpixel_geometry_icon_get_instance_private (self);
	self->priv->_preferred_size = 28;
}

static void
font_manager_subpixel_geometry_icon_finalize (GObject * obj)
{
	FontManagerSubpixelGeometryIcon * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON, FontManagerSubpixelGeometryIcon);
	G_OBJECT_CLASS (font_manager_subpixel_geometry_icon_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
font_manager_subpixel_geometry_icon_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (FontManagerSubpixelGeometryIconClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) font_manager_subpixel_geometry_icon_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FontManagerSubpixelGeometryIcon), 0, (GInstanceInitFunc) font_manager_subpixel_geometry_icon_instance_init, NULL };
	GType font_manager_subpixel_geometry_icon_type_id;
	font_manager_subpixel_geometry_icon_type_id = g_type_register_static (gtk_box_get_type (), "FontManagerSubpixelGeometryIcon", &g_define_type_info, 0);
	FontManagerSubpixelGeometryIcon_private_offset = g_type_add_instance_private (font_manager_subpixel_geometry_icon_type_id, sizeof (FontManagerSubpixelGeometryIconPrivate));
	return font_manager_subpixel_geometry_icon_type_id;
}

GType
font_manager_subpixel_geometry_icon_get_type (void)
{
	static volatile gsize font_manager_subpixel_geometry_icon_type_id__once = 0;
	if (g_once_init_enter (&font_manager_subpixel_geometry_icon_type_id__once)) {
		GType font_manager_subpixel_geometry_icon_type_id;
		font_manager_subpixel_geometry_icon_type_id = font_manager_subpixel_geometry_icon_get_type_once ();
		g_once_init_leave (&font_manager_subpixel_geometry_icon_type_id__once, font_manager_subpixel_geometry_icon_type_id);
	}
	return font_manager_subpixel_geometry_icon_type_id__once;
}

static void
_vala_font_manager_subpixel_geometry_icon_get_property (GObject * object,
                                                        guint property_id,
                                                        GValue * value,
                                                        GParamSpec * pspec)
{
	FontManagerSubpixelGeometryIcon * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON, FontManagerSubpixelGeometryIcon);
	switch (property_id) {
		case FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_PREFERRED_SIZE_PROPERTY:
		g_value_set_int (value, font_manager_subpixel_geometry_icon_get_preferred_size (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_font_manager_subpixel_geometry_icon_set_property (GObject * object,
                                                        guint property_id,
                                                        const GValue * value,
                                                        GParamSpec * pspec)
{
	FontManagerSubpixelGeometryIcon * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY_ICON, FontManagerSubpixelGeometryIcon);
	switch (property_id) {
		case FONT_MANAGER_SUBPIXEL_GEOMETRY_ICON_PREFERRED_SIZE_PROPERTY:
		font_manager_subpixel_geometry_icon_set_preferred_size (self, g_value_get_int (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
font_manager_subpixel_geometry_get_instance_private (FontManagerSubpixelGeometry* self)
{
	return G_STRUCT_MEMBER_P (self, FontManagerSubpixelGeometry_private_offset);
}

static void
_g_object_unref0_ (gpointer var)
{
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
}

static Block27Data*
block27_data_ref (Block27Data* _data27_)
{
	g_atomic_int_inc (&_data27_->_ref_count_);
	return _data27_;
}

static void
block27_data_unref (void * _userdata_)
{
	Block27Data* _data27_;
	_data27_ = (Block27Data*) _userdata_;
	if (g_atomic_int_dec_and_test (&_data27_->_ref_count_)) {
		FontManagerSubpixelGeometry* self;
		self = _data27_->self;
		_g_object_unref0 (_data27_->button);
		_g_object_unref0 (self);
		g_slice_free (Block27Data, _data27_);
	}
}

static void
_____lambda151_ (Block27Data* _data27_)
{
	FontManagerSubpixelGeometry* self;
	GtkCheckButton* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	self = _data27_->self;
	_tmp0_ = _data27_->button;
	_tmp1_ = gtk_check_button_get_active (_tmp0_);
	_tmp2_ = _tmp1_;
	if (_tmp2_) {
		guint index = 0U;
		GPtrArray* _tmp3_;
		GtkCheckButton* _tmp4_;
		guint _tmp5_ = 0U;
		_tmp3_ = self->priv->_options;
		_tmp4_ = _data27_->button;
		g_ptr_array_find (_tmp3_, _tmp4_, &_tmp5_);
		index = _tmp5_;
		font_manager_subpixel_geometry_set_selected (self, (guint) ((gint) index));
	}
}

static void
______lambda151__gtk_check_button_toggled (GtkCheckButton* _sender,
                                           gpointer self)
{
	_____lambda151_ (self);
}

FontManagerSubpixelGeometry*
font_manager_subpixel_geometry_construct (GType object_type)
{
	FontManagerSubpixelGeometry * self = NULL;
	GPtrArray* _tmp0_;
	GPtrArray* _tmp1_;
	self = (FontManagerSubpixelGeometry*) g_object_new (object_type, NULL);
	gtk_widget_set_hexpand ((GtkWidget*) self, TRUE);
	gtk_widget_set_vexpand ((GtkWidget*) self, FALSE);
	gtk_widget_set_valign ((GtkWidget*) self, GTK_ALIGN_CENTER);
	gtk_widget_set_halign ((GtkWidget*) self, GTK_ALIGN_CENTER);
	gtk_box_set_spacing ((GtkBox*) self, FONT_MANAGER_DEFAULT_MARGIN * 3);
	_tmp0_ = g_ptr_array_new_full ((guint) 0, _g_object_unref0_);
	_tmp1_ = _tmp0_;
	font_manager_subpixel_geometry_set_options (self, _tmp1_);
	_g_ptr_array_unref0 (_tmp1_);
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				Block27Data* _data27_;
				FontManagerSubpixelGeometryIcon* icon = NULL;
				FontManagerSubpixelGeometryIcon* _tmp4_;
				FontManagerSubpixelGeometryIcon* _tmp5_;
				GtkWidget* _tmp6_;
				const gchar* _tmp7_;
				gchar* _tmp8_;
				GtkCheckButton* _tmp9_ = NULL;
				GPtrArray* _tmp10_;
				GtkCheckButton* _tmp11_;
				GtkCheckButton* _tmp12_;
				GtkCheckButton* _tmp13_ = NULL;
				GtkCheckButton* _tmp16_;
				GtkCheckButton* _tmp17_;
				GtkCheckButton* _tmp18_;
				GBindingFlags flags = 0U;
				FontManagerSubpixelGeometryIcon* _tmp19_;
				_data27_ = g_slice_new0 (Block27Data);
				_data27_->_ref_count_ = 1;
				_data27_->self = g_object_ref (self);
				if (!_tmp2_) {
					gint _tmp3_;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				if (!(i <= ((gint) FONT_MANAGER_SUBPIXEL_ORDER_NONE))) {
					block27_data_unref (_data27_);
					_data27_ = NULL;
					break;
				}
				_tmp4_ = font_manager_subpixel_geometry_icon_new ((FontManagerSubpixelOrder) i);
				g_object_ref_sink (_tmp4_);
				icon = _tmp4_;
				_tmp5_ = icon;
				_tmp6_ = _g_object_ref0 ((GtkWidget*) _tmp5_);
				_tmp7_ = font_manager_subpixel_order_to_string ((FontManagerSubpixelOrder) i);
				_tmp8_ = g_strdup (_tmp7_);
				_tmp9_ = (GtkCheckButton*) gtk_check_button_new ();
				gtk_widget_set_margin_start ((GtkWidget*) _tmp9_, FONT_MANAGER_DEFAULT_MARGIN);
				gtk_widget_set_margin_end ((GtkWidget*) _tmp9_, FONT_MANAGER_DEFAULT_MARGIN);
				gtk_check_button_set_child (_tmp9_, _tmp6_);
				_g_object_unref0 (_tmp6_);
				gtk_widget_set_tooltip_text ((GtkWidget*) _tmp9_, _tmp8_);
				_g_free0 (_tmp8_);
				g_object_ref_sink (_tmp9_);
				_data27_->button = _tmp9_;
				_tmp10_ = self->priv->_options;
				_tmp11_ = _data27_->button;
				_tmp12_ = _g_object_ref0 (_tmp11_);
				g_ptr_array_insert (_tmp10_, i, _tmp12_);
				if (i != 0) {
					GPtrArray* _tmp14_;
					gconstpointer _tmp15_;
					_tmp14_ = self->priv->_options;
					_tmp15_ = g_ptr_array_index (_tmp14_, (guint) 0);
					_tmp13_ = (GtkCheckButton*) _tmp15_;
				} else {
					_tmp13_ = NULL;
				}
				_tmp16_ = _data27_->button;
				gtk_check_button_set_group (_tmp16_, _tmp13_);
				_tmp17_ = _data27_->button;
				g_signal_connect_data (_tmp17_, "toggled", (GCallback) ______lambda151__gtk_check_button_toggled, block27_data_ref (_data27_), (GClosureNotify) block27_data_unref, 0);
				_tmp18_ = _data27_->button;
				gtk_box_append ((GtkBox*) self, (GtkWidget*) _tmp18_);
				flags = G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE;
				_tmp19_ = icon;
				g_object_bind_property_with_closures ((GObject*) self, "preferred-size", (GObject*) _tmp19_, "preferred-size", flags, (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, (GClosureNotify) NULL)), (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, (GClosureNotify) NULL)));
				_g_object_unref0 (icon);
				block27_data_unref (_data27_);
				_data27_ = NULL;
			}
		}
	}
	return self;
}

FontManagerSubpixelGeometry*
font_manager_subpixel_geometry_new (void)
{
	return font_manager_subpixel_geometry_construct (FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY);
}

guint
font_manager_subpixel_geometry_get_selected (FontManagerSubpixelGeometry* self)
{
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_rgba;
	return result;
}

static gint
vala_g_ptr_array_get_length (GPtrArray* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = (gint) self->len;
	return result;
}

void
font_manager_subpixel_geometry_set_selected (FontManagerSubpixelGeometry* self,
                                             guint value)
{
	GPtrArray* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	GPtrArray* _tmp3_;
	gconstpointer _tmp4_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->_options;
	_tmp1_ = vala_g_ptr_array_get_length (_tmp0_);
	_tmp2_ = _tmp1_;
	self->priv->_rgba = CLAMP (value, (guint) 0, (guint) _tmp2_);
	_tmp3_ = self->priv->_options;
	_tmp4_ = g_ptr_array_index (_tmp3_, self->priv->_rgba);
	gtk_check_button_set_active ((GtkCheckButton*) _tmp4_, TRUE);
	g_object_notify ((GObject*) self, "selected");
	g_object_notify_by_pspec ((GObject *) self, font_manager_subpixel_geometry_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_SELECTED_PROPERTY]);
}

gint
font_manager_subpixel_geometry_get_preferred_size (FontManagerSubpixelGeometry* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_preferred_size;
	return result;
}

void
font_manager_subpixel_geometry_set_preferred_size (FontManagerSubpixelGeometry* self,
                                                   gint value)
{
	gint old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_subpixel_geometry_get_preferred_size (self);
	if (old_value != value) {
		self->priv->_preferred_size = value;
		g_object_notify_by_pspec ((GObject *) self, font_manager_subpixel_geometry_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_PREFERRED_SIZE_PROPERTY]);
	}
}

GPtrArray*
font_manager_subpixel_geometry_get_options (FontManagerSubpixelGeometry* self)
{
	GPtrArray* result;
	GPtrArray* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_options;
	result = _tmp0_;
	return result;
}

static gpointer
_g_ptr_array_ref0 (gpointer self)
{
	return self ? g_ptr_array_ref (self) : NULL;
}

static void
font_manager_subpixel_geometry_set_options (FontManagerSubpixelGeometry* self,
                                            GPtrArray* value)
{
	GPtrArray* old_value;
	g_return_if_fail (self != NULL);
	old_value = font_manager_subpixel_geometry_get_options (self);
	if (old_value != value) {
		GPtrArray* _tmp0_;
		_tmp0_ = _g_ptr_array_ref0 (value);
		_g_ptr_array_unref0 (self->priv->_options);
		self->priv->_options = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, font_manager_subpixel_geometry_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_OPTIONS_PROPERTY]);
	}
}

static void
font_manager_subpixel_geometry_class_init (FontManagerSubpixelGeometryClass * klass,
                                           gpointer klass_data)
{
	font_manager_subpixel_geometry_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &FontManagerSubpixelGeometry_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_font_manager_subpixel_geometry_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_font_manager_subpixel_geometry_set_property;
	G_OBJECT_CLASS (klass)->finalize = font_manager_subpixel_geometry_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_SUBPIXEL_GEOMETRY_SELECTED_PROPERTY, font_manager_subpixel_geometry_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_SELECTED_PROPERTY] = g_param_spec_uint ("selected", "selected", "selected", 0, G_MAXUINT, 0U, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_SUBPIXEL_GEOMETRY_PREFERRED_SIZE_PROPERTY, font_manager_subpixel_geometry_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_PREFERRED_SIZE_PROPERTY] = g_param_spec_int ("preferred-size", "preferred-size", "preferred-size", G_MININT, G_MAXINT, 28, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), FONT_MANAGER_SUBPIXEL_GEOMETRY_OPTIONS_PROPERTY, font_manager_subpixel_geometry_properties[FONT_MANAGER_SUBPIXEL_GEOMETRY_OPTIONS_PROPERTY] = g_param_spec_boxed ("options", "options", "options", G_TYPE_PTR_ARRAY, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
font_manager_subpixel_geometry_instance_init (FontManagerSubpixelGeometry * self,
                                              gpointer klass)
{
	self->priv = font_manager_subpixel_geometry_get_instance_private (self);
	self->priv->_preferred_size = 28;
}

static void
font_manager_subpixel_geometry_finalize (GObject * obj)
{
	FontManagerSubpixelGeometry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY, FontManagerSubpixelGeometry);
	_g_ptr_array_unref0 (self->priv->_options);
	G_OBJECT_CLASS (font_manager_subpixel_geometry_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
font_manager_subpixel_geometry_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (FontManagerSubpixelGeometryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) font_manager_subpixel_geometry_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FontManagerSubpixelGeometry), 0, (GInstanceInitFunc) font_manager_subpixel_geometry_instance_init, NULL };
	GType font_manager_subpixel_geometry_type_id;
	font_manager_subpixel_geometry_type_id = g_type_register_static (gtk_box_get_type (), "FontManagerSubpixelGeometry", &g_define_type_info, 0);
	FontManagerSubpixelGeometry_private_offset = g_type_add_instance_private (font_manager_subpixel_geometry_type_id, sizeof (FontManagerSubpixelGeometryPrivate));
	return font_manager_subpixel_geometry_type_id;
}

GType
font_manager_subpixel_geometry_get_type (void)
{
	static volatile gsize font_manager_subpixel_geometry_type_id__once = 0;
	if (g_once_init_enter (&font_manager_subpixel_geometry_type_id__once)) {
		GType font_manager_subpixel_geometry_type_id;
		font_manager_subpixel_geometry_type_id = font_manager_subpixel_geometry_get_type_once ();
		g_once_init_leave (&font_manager_subpixel_geometry_type_id__once, font_manager_subpixel_geometry_type_id);
	}
	return font_manager_subpixel_geometry_type_id__once;
}

static void
_vala_font_manager_subpixel_geometry_get_property (GObject * object,
                                                   guint property_id,
                                                   GValue * value,
                                                   GParamSpec * pspec)
{
	FontManagerSubpixelGeometry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY, FontManagerSubpixelGeometry);
	switch (property_id) {
		case FONT_MANAGER_SUBPIXEL_GEOMETRY_SELECTED_PROPERTY:
		g_value_set_uint (value, font_manager_subpixel_geometry_get_selected (self));
		break;
		case FONT_MANAGER_SUBPIXEL_GEOMETRY_PREFERRED_SIZE_PROPERTY:
		g_value_set_int (value, font_manager_subpixel_geometry_get_preferred_size (self));
		break;
		case FONT_MANAGER_SUBPIXEL_GEOMETRY_OPTIONS_PROPERTY:
		g_value_set_boxed (value, font_manager_subpixel_geometry_get_options (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_font_manager_subpixel_geometry_set_property (GObject * object,
                                                   guint property_id,
                                                   const GValue * value,
                                                   GParamSpec * pspec)
{
	FontManagerSubpixelGeometry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, FONT_MANAGER_TYPE_SUBPIXEL_GEOMETRY, FontManagerSubpixelGeometry);
	switch (property_id) {
		case FONT_MANAGER_SUBPIXEL_GEOMETRY_SELECTED_PROPERTY:
		font_manager_subpixel_geometry_set_selected (self, g_value_get_uint (value));
		break;
		case FONT_MANAGER_SUBPIXEL_GEOMETRY_PREFERRED_SIZE_PROPERTY:
		font_manager_subpixel_geometry_set_preferred_size (self, g_value_get_int (value));
		break;
		case FONT_MANAGER_SUBPIXEL_GEOMETRY_OPTIONS_PROPERTY:
		font_manager_subpixel_geometry_set_options (self, g_value_get_boxed (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_array_destroy (gpointer array,
                     gssize array_length,
                     GDestroyNotify destroy_func)
{
	if ((array != NULL) && (destroy_func != NULL)) {
		gssize i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}

static void
_vala_array_free (gpointer array,
                  gssize array_length,
                  GDestroyNotify destroy_func)
{
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

