#include <gio/gio.h>

#if defined (__ELF__) && ( __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 6))
# define SECTION __attribute__ ((section (".gresource.vala_common_gresources"), aligned (sizeof(void *) > 8 ? sizeof(void *) : 8)))
#else
# define SECTION
#endif

static const SECTION union { const guint8 data[697]; const double alignment; void * const ptr;}  vala_common_gresources_resource_data = {
  "\107\126\141\162\151\141\156\164\000\000\000\000\000\000\000\000"
  "\030\000\000\000\344\000\000\000\000\000\000\050\007\000\000\000"
  "\000\000\000\000\000\000\000\000\001\000\000\000\002\000\000\000"
  "\004\000\000\000\006\000\000\000\007\000\000\000\125\016\070\250"
  "\006\000\000\000\344\000\000\000\014\000\114\000\360\000\000\000"
  "\364\000\000\000\324\265\002\000\377\377\377\377\364\000\000\000"
  "\001\000\114\000\370\000\000\000\374\000\000\000\302\257\211\013"
  "\001\000\000\000\374\000\000\000\004\000\114\000\000\001\000\000"
  "\004\001\000\000\234\221\103\062\005\000\000\000\004\001\000\000"
  "\036\000\166\000\050\001\000\000\222\002\000\000\226\350\101\106"
  "\000\000\000\000\222\002\000\000\014\000\114\000\240\002\000\000"
  "\244\002\000\000\103\050\055\252\004\000\000\000\244\002\000\000"
  "\003\000\114\000\250\002\000\000\254\002\000\000\224\135\334\227"
  "\002\000\000\000\254\002\000\000\007\000\114\000\264\002\000\000"
  "\270\002\000\000\106\157\156\164\115\141\156\141\147\145\162\057"
  "\004\000\000\000\057\000\000\000\002\000\000\000\143\157\155\057"
  "\006\000\000\000\146\157\156\164\055\155\141\156\141\147\145\162"
  "\055\160\162\145\166\151\145\167\055\143\157\154\157\162\163\056"
  "\165\151\000\000\000\000\000\000\171\004\000\000\001\000\000\000"
  "\170\332\315\224\135\113\303\060\024\206\357\367\053\302\271\122"
  "\330\326\155\070\360\042\315\140\312\274\022\004\365\132\322\364"
  "\264\213\113\223\232\236\316\355\337\233\356\303\311\324\202\103"
  "\120\102\102\070\071\357\223\234\274\044\174\262\052\014\133\242"
  "\257\264\263\061\014\373\003\140\150\225\113\265\315\143\170\174"
  "\230\365\056\141\042\072\134\133\102\237\111\205\054\165\205\324"
  "\041\065\163\226\172\205\264\062\107\017\202\173\174\251\265\307"
  "\212\031\235\304\220\323\002\016\324\213\376\160\004\221\340\204"
  "\105\151\044\041\123\106\126\125\014\263\200\270\335\022\356\074"
  "\056\065\276\136\071\343\174\005\254\224\036\055\305\160\103\213"
  "\251\133\005\274\232\153\223\012\356\222\147\124\264\327\207\325"
  "\215\340\132\113\343\362\151\115\344\054\060\235\306\220\344\117"
  "\252\131\171\112\266\101\301\113\357\112\364\264\146\126\026\030"
  "\103\272\221\100\073\361\263\214\064\031\004\106\136\332\052\124"
  "\042\023\023\202\153\254\100\334\243\151\070\211\124\213\334\273"
  "\332\246\154\163\000\036\355\011\202\107\333\275\304\307\330\021"
  "\077\163\252\256\032\052\210\141\113\332\134\032\235\207\252\024"
  "\066\266\264\044\372\074\221\040\302\170\066\032\217\273\273\176"
  "\336\042\040\347\014\351\262\107\270\242\123\353\074\102\056\277"
  "\075\354\341\102\166\366\376\320\345\354\257\134\156\156\347\237"
  "\371\073\350\206\366\033\316\176\135\333\351\236\106\373\127\037"
  "\246\357\277\210\350\274\001\111\331\236\003\000\050\165\165\141"
  "\171\051\106\157\156\164\115\141\156\141\147\145\162\057\000\000"
  "\005\000\000\000\165\151\057\000\003\000\000\000\147\151\164\150"
  "\165\142\057\000\000\000\000\000" };

static GStaticResource static_resource = { vala_common_gresources_resource_data.data, sizeof (vala_common_gresources_resource_data.data) - 1 /* nul terminator */, NULL, NULL, NULL };

G_GNUC_INTERNAL
GResource *vala_common_gresources_get_resource (void);
GResource *vala_common_gresources_get_resource (void)
{
  return g_static_resource_get_resource (&static_resource);
}
/* GLIB - Library of useful routines for C programming
 * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GLib Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GLib at ftp://ftp.gtk.org/pub/gtk/.
 */

#ifndef __G_CONSTRUCTOR_H__
#define __G_CONSTRUCTOR_H__

/*
  If G_HAS_CONSTRUCTORS is true then the compiler support *both* constructors and
  destructors, in a usable way, including e.g. on library unload. If not you're on
  your own.

  Some compilers need #pragma to handle this, which does not work with macros,
  so the way you need to use this is (for constructors):

  #ifdef G_DEFINE_CONSTRUCTOR_NEEDS_PRAGMA
  #pragma G_DEFINE_CONSTRUCTOR_PRAGMA_ARGS(my_constructor)
  #endif
  G_DEFINE_CONSTRUCTOR(my_constructor)
  static void my_constructor(void) {
   ...
  }

*/

#ifndef __GTK_DOC_IGNORE__

#if  __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7)

#define G_HAS_CONSTRUCTORS 1

#define G_DEFINE_CONSTRUCTOR(_func) static void __attribute__((constructor)) _func (void);
#define G_DEFINE_DESTRUCTOR(_func) static void __attribute__((destructor)) _func (void);

#elif defined (_MSC_VER)

/*
 * Only try to include gslist.h if not already included via glib.h,
 * so that items using gconstructor.h outside of GLib (such as
 * GResources) continue to build properly.
 */
#ifndef __G_LIB_H__
#include "gslist.h"
#endif

#include <stdlib.h>

#define G_HAS_CONSTRUCTORS 1

/* We do some weird things to avoid the constructors being optimized
 * away on VS2015 if WholeProgramOptimization is enabled. First we
 * make a reference to the array from the wrapper to make sure its
 * references. Then we use a pragma to make sure the wrapper function
 * symbol is always included at the link stage. Also, the symbols
 * need to be extern (but not dllexport), even though they are not
 * really used from another object file.
 */

/* We need to account for differences between the mangling of symbols
 * for x86 and x64/ARM/ARM64 programs, as symbols on x86 are prefixed
 * with an underscore but symbols on x64/ARM/ARM64 are not.
 */
#ifdef _M_IX86
#define G_MSVC_SYMBOL_PREFIX "_"
#else
#define G_MSVC_SYMBOL_PREFIX ""
#endif

#define G_DEFINE_CONSTRUCTOR(_func) G_MSVC_CTOR (_func, G_MSVC_SYMBOL_PREFIX)
#define G_DEFINE_DESTRUCTOR(_func) G_MSVC_DTOR (_func, G_MSVC_SYMBOL_PREFIX)

#define G_MSVC_CTOR(_func,_sym_prefix) \
  static void _func(void); \
  extern int (* _array ## _func)(void);              \
  int _func ## _wrapper(void);              \
  int _func ## _wrapper(void) { _func(); g_slist_find (NULL,  _array ## _func); return 0; } \
  __pragma(comment(linker,"/include:" _sym_prefix # _func "_wrapper")) \
  __pragma(section(".CRT$XCU",read)) \
  __declspec(allocate(".CRT$XCU")) int (* _array ## _func)(void) = _func ## _wrapper;

#define G_MSVC_DTOR(_func,_sym_prefix) \
  static void _func(void); \
  extern int (* _array ## _func)(void);              \
  int _func ## _constructor(void);              \
  int _func ## _constructor(void) { atexit (_func); g_slist_find (NULL,  _array ## _func); return 0; } \
   __pragma(comment(linker,"/include:" _sym_prefix # _func "_constructor")) \
  __pragma(section(".CRT$XCU",read)) \
  __declspec(allocate(".CRT$XCU")) int (* _array ## _func)(void) = _func ## _constructor;

#elif defined(__SUNPRO_C)

/* This is not tested, but i believe it should work, based on:
 * http://opensource.apple.com/source/OpenSSL098/OpenSSL098-35/src/fips/fips_premain.c
 */

#define G_HAS_CONSTRUCTORS 1

#define G_DEFINE_CONSTRUCTOR_NEEDS_PRAGMA 1
#define G_DEFINE_DESTRUCTOR_NEEDS_PRAGMA 1

#define G_DEFINE_CONSTRUCTOR_PRAGMA_ARGS(_func) \
  init(_func)
#define G_DEFINE_CONSTRUCTOR(_func) \
  static void _func(void);

#define G_DEFINE_DESTRUCTOR_PRAGMA_ARGS(_func) \
  fini(_func)
#define G_DEFINE_DESTRUCTOR(_func) \
  static void _func(void);

#else

/* constructors not supported for this compiler */

#endif

#endif /* __GTK_DOC_IGNORE__ */
#endif /* __G_CONSTRUCTOR_H__ */

#ifdef G_HAS_CONSTRUCTORS

#ifdef G_DEFINE_CONSTRUCTOR_NEEDS_PRAGMA
#pragma G_DEFINE_CONSTRUCTOR_PRAGMA_ARGS(vala_common_gresourcesresource_constructor)
#endif
G_DEFINE_CONSTRUCTOR(vala_common_gresourcesresource_constructor)
#ifdef G_DEFINE_DESTRUCTOR_NEEDS_PRAGMA
#pragma G_DEFINE_DESTRUCTOR_PRAGMA_ARGS(vala_common_gresourcesresource_destructor)
#endif
G_DEFINE_DESTRUCTOR(vala_common_gresourcesresource_destructor)

#else
#warning "Constructor not supported on this compiler, linking in resources will not work"
#endif

static void vala_common_gresourcesresource_constructor (void)
{
  g_static_resource_init (&static_resource);
}

static void vala_common_gresourcesresource_destructor (void)
{
  g_static_resource_fini (&static_resource);
}
