/* vala-common.h generated by valac 0.56.18-dirty, the Vala compiler, do not modify */

#ifndef __SRC_COMMON_VALA_COMMON_H__
#define __SRC_COMMON_VALA_COMMON_H__

#include <gtk/gtk.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include "font-manager.h"

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define FONT_MANAGER_TYPE_PREVIEW_COLORS (font_manager_preview_colors_get_type ())
#define FONT_MANAGER_PREVIEW_COLORS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_PREVIEW_COLORS, FontManagerPreviewColors))
#define FONT_MANAGER_PREVIEW_COLORS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_PREVIEW_COLORS, FontManagerPreviewColorsClass))
#define FONT_MANAGER_IS_PREVIEW_COLORS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_PREVIEW_COLORS))
#define FONT_MANAGER_IS_PREVIEW_COLORS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_PREVIEW_COLORS))
#define FONT_MANAGER_PREVIEW_COLORS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_PREVIEW_COLORS, FontManagerPreviewColorsClass))

typedef struct _FontManagerPreviewColors FontManagerPreviewColors;
typedef struct _FontManagerPreviewColorsClass FontManagerPreviewColorsClass;
typedef struct _FontManagerPreviewColorsPrivate FontManagerPreviewColorsPrivate;
typedef enum  {
	FONT_MANAGER_PREDEFINED_WATERFALL_SIZE_LINEAR_48,
	FONT_MANAGER_PREDEFINED_WATERFALL_SIZE_72_11,
	FONT_MANAGER_PREDEFINED_WATERFALL_SIZE_LINEAR_96,
	FONT_MANAGER_PREDEFINED_WATERFALL_SIZE_96_11,
	FONT_MANAGER_PREDEFINED_WATERFALL_SIZE_120_12,
	FONT_MANAGER_PREDEFINED_WATERFALL_SIZE_144_13,
	FONT_MANAGER_PREDEFINED_WATERFALL_SIZE_192_14,
	FONT_MANAGER_PREDEFINED_WATERFALL_SIZE_CUSTOM
} FontManagerPredefinedWaterfallSize;

#define FONT_MANAGER_TYPE_PREDEFINED_WATERFALL_SIZE (font_manager_predefined_waterfall_size_get_type ())

#define FONT_MANAGER_TYPE_WATERFALL_SETTINGS (font_manager_waterfall_settings_get_type ())
#define FONT_MANAGER_WATERFALL_SETTINGS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_MANAGER_TYPE_WATERFALL_SETTINGS, FontManagerWaterfallSettings))
#define FONT_MANAGER_WATERFALL_SETTINGS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FONT_MANAGER_TYPE_WATERFALL_SETTINGS, FontManagerWaterfallSettingsClass))
#define FONT_MANAGER_IS_WATERFALL_SETTINGS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FONT_MANAGER_TYPE_WATERFALL_SETTINGS))
#define FONT_MANAGER_IS_WATERFALL_SETTINGS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FONT_MANAGER_TYPE_WATERFALL_SETTINGS))
#define FONT_MANAGER_WATERFALL_SETTINGS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FONT_MANAGER_TYPE_WATERFALL_SETTINGS, FontManagerWaterfallSettingsClass))

typedef struct _FontManagerWaterfallSettings FontManagerWaterfallSettings;
typedef struct _FontManagerWaterfallSettingsClass FontManagerWaterfallSettingsClass;
typedef struct _FontManagerWaterfallSettingsPrivate FontManagerWaterfallSettingsPrivate;

struct _FontManagerPreviewColors {
	GtkBox parent_instance;
	FontManagerPreviewColorsPrivate * priv;
};

struct _FontManagerPreviewColorsClass {
	GtkBoxClass parent_class;
};

struct _FontManagerWaterfallSettings {
	GObject parent_instance;
	FontManagerWaterfallSettingsPrivate * priv;
};

struct _FontManagerWaterfallSettingsClass {
	GObjectClass parent_class;
};

VALA_EXTERN GType font_manager_preview_colors_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerPreviewColors, g_object_unref)
VALA_EXTERN void font_manager_preview_colors_update_style (FontManagerPreviewColors* self);
VALA_EXTERN void font_manager_preview_colors_restore_state (FontManagerPreviewColors* self,
                                                GSettings* settings);
VALA_EXTERN GVariant* font_manager_preview_colors_to_setting (GValue* val,
                                                  const GVariantType* type);
VALA_EXTERN gboolean font_manager_preview_colors_from_setting (GValue* val,
                                                   GVariant* variant);
VALA_EXTERN FontManagerPreviewColors* font_manager_preview_colors_new (void);
VALA_EXTERN FontManagerPreviewColors* font_manager_preview_colors_construct (GType object_type);
VALA_EXTERN void font_manager_preview_colors_get_foreground_color (FontManagerPreviewColors* self,
                                                       GdkRGBA * result);
VALA_EXTERN void font_manager_preview_colors_set_foreground_color (FontManagerPreviewColors* self,
                                                       GdkRGBA * value);
VALA_EXTERN void font_manager_preview_colors_get_background_color (FontManagerPreviewColors* self,
                                                       GdkRGBA * result);
VALA_EXTERN void font_manager_preview_colors_set_background_color (FontManagerPreviewColors* self,
                                                       GdkRGBA * value);
VALA_EXTERN GSettings* font_manager_preview_colors_get_settings (FontManagerPreviewColors* self);
VALA_EXTERN void font_manager_preview_colors_set_settings (FontManagerPreviewColors* self,
                                               GSettings* value);
VALA_EXTERN GType font_manager_predefined_waterfall_size_get_type (void) G_GNUC_CONST ;
VALA_EXTERN gchar* font_manager_predefined_waterfall_size_to_string (FontManagerPredefinedWaterfallSize self);
VALA_EXTERN gdouble* font_manager_predefined_waterfall_size_to_size_array (FontManagerPredefinedWaterfallSize self,
                                                               gint* result_length1);
VALA_EXTERN GVariant* font_manager_predefined_waterfall_size_to_setting (GValue* val,
                                                             const GVariantType* type);
VALA_EXTERN gboolean font_manager_predefined_waterfall_size_from_setting (GValue* val,
                                                              GVariant* variant);
VALA_EXTERN GType font_manager_waterfall_settings_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FontManagerWaterfallSettings, g_object_unref)
VALA_EXTERN FontManagerWaterfallSettings* font_manager_waterfall_settings_new (GSettings* settings);
VALA_EXTERN FontManagerWaterfallSettings* font_manager_waterfall_settings_construct (GType object_type,
                                                                         GSettings* settings);
VALA_EXTERN void font_manager_waterfall_settings_on_selection_changed (FontManagerWaterfallSettings* self);
VALA_EXTERN GSettings* font_manager_waterfall_settings_get_settings (FontManagerWaterfallSettings* self);
VALA_EXTERN void font_manager_waterfall_settings_set_settings (FontManagerWaterfallSettings* self,
                                                   GSettings* value);
VALA_EXTERN gint font_manager_waterfall_settings_get_predefined_size (FontManagerWaterfallSettings* self);
VALA_EXTERN void font_manager_waterfall_settings_set_predefined_size (FontManagerWaterfallSettings* self,
                                                          gint value);
VALA_EXTERN gdouble font_manager_waterfall_settings_get_minimum (FontManagerWaterfallSettings* self);
VALA_EXTERN void font_manager_waterfall_settings_set_minimum (FontManagerWaterfallSettings* self,
                                                  gdouble value);
VALA_EXTERN gdouble font_manager_waterfall_settings_get_maximum (FontManagerWaterfallSettings* self);
VALA_EXTERN void font_manager_waterfall_settings_set_maximum (FontManagerWaterfallSettings* self,
                                                  gdouble value);
VALA_EXTERN gdouble font_manager_waterfall_settings_get_ratio (FontManagerWaterfallSettings* self);
VALA_EXTERN void font_manager_waterfall_settings_set_ratio (FontManagerWaterfallSettings* self,
                                                gdouble value);
VALA_EXTERN gboolean font_manager_waterfall_settings_get_show_line_size (FontManagerWaterfallSettings* self);
VALA_EXTERN void font_manager_waterfall_settings_set_show_line_size (FontManagerWaterfallSettings* self,
                                                         gboolean value);
VALA_EXTERN gint font_manager_waterfall_settings_get_line_spacing (FontManagerWaterfallSettings* self);
VALA_EXTERN void font_manager_waterfall_settings_set_line_spacing (FontManagerWaterfallSettings* self,
                                                       gint value);
VALA_EXTERN GtkPopoverMenu* font_manager_waterfall_settings_get_context_menu (FontManagerWaterfallSettings* self);
VALA_EXTERN FontManagerPreferenceRow* font_manager_waterfall_settings_get_preference_row (FontManagerWaterfallSettings* self);

G_END_DECLS

#endif
