/* unicode-names.h */
/* THIS IS A GENERATED FILE. CHANGES WILL BE OVERWRITTEN. */
/* Generated by ./gen-unicode-headers.pl */
/* Generated from UCD version 16.0.0 */

#pragma once

#include <glib.h>

#ifndef __GTK_DOC_IGNORE__

static const char unicode_names_strings[] = \
  "<control>\0"
  "ABACUS\0"
  "AC CURRENT\0"
  "ACCORDION\0"
  "ACCOUNT OF\0"
  "ACTIVATE ARABIC FORM SHAPING\0"
  "ACTIVATE SYMMETRIC SWAPPING\0"
  "ACUTE ACCENT\0"
  "ACUTE ANGLE\0"
  "ADDRESSED TO THE SUBJECT\0"
  "ADHESIVE BANDAGE\0"
  "ADI SHAKTI\0"
  "ADLAM ALIF LENGTHENER\0"
  "ADLAM CAPITAL LETTER ALIF\0"
  "ADLAM CAPITAL LETTER BA\0"
  "ADLAM CAPITAL LETTER BHE\0"
  "ADLAM CAPITAL LETTER CHI\0"
  "ADLAM CAPITAL LETTER DAALI\0"
  "ADLAM CAPITAL LETTER DHA\0"
  "ADLAM CAPITAL LETTER E\0"
  "ADLAM CAPITAL LETTER FA\0"
  "ADLAM CAPITAL LETTER GA\0"
  "ADLAM CAPITAL LETTER GBE\0"
  "ADLAM CAPITAL LETTER HA\0"
  "ADLAM CAPITAL LETTER I\0"
  "ADLAM CAPITAL LETTER JIIM\0"
  "ADLAM CAPITAL LETTER KAF\0"
  "ADLAM CAPITAL LETTER KHA\0"
  "ADLAM CAPITAL LETTER KPO\0"
  "ADLAM CAPITAL LETTER LAAM\0"
  "ADLAM CAPITAL LETTER MIIM\0"
  "ADLAM CAPITAL LETTER NHA\0"
  "ADLAM CAPITAL LETTER NUN\0"
  "ADLAM CAPITAL LETTER NYA\0"
  "ADLAM CAPITAL LETTER O\0"
  "ADLAM CAPITAL LETTER PE\0"
  "ADLAM CAPITAL LETTER QAAF\0"
  "ADLAM CAPITAL LETTER RA\0"
  "ADLAM CAPITAL LETTER SHA\0"
  "ADLAM CAPITAL LETTER SINNYIIYHE\0"
  "ADLAM CAPITAL LETTER TU\0"
  "ADLAM CAPITAL LETTER U\0"
  "ADLAM CAPITAL LETTER VA\0"
  "ADLAM CAPITAL LETTER WAW\0"
  "ADLAM CAPITAL LETTER YA\0"
  "ADLAM CAPITAL LETTER YHE\0"
  "ADLAM CAPITAL LETTER ZAL\0"
  "ADLAM CONSONANT MODIFIER\0"
  "ADLAM DIGIT EIGHT\0"
  "ADLAM DIGIT FIVE\0"
  "ADLAM DIGIT FOUR\0"
  "ADLAM DIGIT NINE\0"
  "ADLAM DIGIT ONE\0"
  "ADLAM DIGIT SEVEN\0"
  "ADLAM DIGIT SIX\0"
  "ADLAM DIGIT THREE\0"
  "ADLAM DIGIT TWO\0"
  "ADLAM DIGIT ZERO\0"
  "ADLAM GEMINATE CONSONANT MODIFIER\0"
  "ADLAM GEMINATION MARK\0"
  "ADLAM HAMZA\0"
  "ADLAM INITIAL EXCLAMATION MARK\0"
  "ADLAM INITIAL QUESTION MARK\0"
  "ADLAM NASALIZATION MARK\0"
  "ADLAM NUKTA\0"
  "ADLAM SMALL LETTER ALIF\0"
  "ADLAM SMALL LETTER BA\0"
  "ADLAM SMALL LETTER BHE\0"
  "ADLAM SMALL LETTER CHI\0"
  "ADLAM SMALL LETTER DAALI\0"
  "ADLAM SMALL LETTER DHA\0"
  "ADLAM SMALL LETTER E\0"
  "ADLAM SMALL LETTER FA\0"
  "ADLAM SMALL LETTER GA\0"
  "ADLAM SMALL LETTER GBE\0"
  "ADLAM SMALL LETTER HA\0"
  "ADLAM SMALL LETTER I\0"
  "ADLAM SMALL LETTER JIIM\0"
  "ADLAM SMALL LETTER KAF\0"
  "ADLAM SMALL LETTER KHA\0"
  "ADLAM SMALL LETTER KPO\0"
  "ADLAM SMALL LETTER LAAM\0"
  "ADLAM SMALL LETTER MIIM\0"
  "ADLAM SMALL LETTER NHA\0"
  "ADLAM SMALL LETTER NUN\0"
  "ADLAM SMALL LETTER NYA\0"
  "ADLAM SMALL LETTER O\0"
  "ADLAM SMALL LETTER PE\0"
  "ADLAM SMALL LETTER QAAF\0"
  "ADLAM SMALL LETTER RA\0"
  "ADLAM SMALL LETTER SHA\0"
  "ADLAM SMALL LETTER SINNYIIYHE\0"
  "ADLAM SMALL LETTER TU\0"
  "ADLAM SMALL LETTER U\0"
  "ADLAM SMALL LETTER VA\0"
  "ADLAM SMALL LETTER WAW\0"
  "ADLAM SMALL LETTER YA\0"
  "ADLAM SMALL LETTER YHE\0"
  "ADLAM SMALL LETTER ZAL\0"
  "ADLAM VOWEL LENGTHENER\0"
  "ADMETOS\0"
  "ADMISSION TICKETS\0"
  "ADULT\0"
  "AEGEAN CHECK MARK\0"
  "AEGEAN DRY MEASURE FIRST SUBUNIT\0"
  "AEGEAN LIQUID MEASURE FIRST SUBUNIT\0"
  "AEGEAN MEASURE SECOND SUBUNIT\0"
  "AEGEAN MEASURE THIRD SUBUNIT\0"
  "AEGEAN NUMBER EIGHT\0"
  "AEGEAN NUMBER EIGHT HUNDRED\0"
  "AEGEAN NUMBER EIGHT THOUSAND\0"
  "AEGEAN NUMBER EIGHTY\0"
  "AEGEAN NUMBER EIGHTY THOUSAND\0"
  "AEGEAN NUMBER FIFTY\0"
  "AEGEAN NUMBER FIFTY THOUSAND\0"
  "AEGEAN NUMBER FIVE\0"
  "AEGEAN NUMBER FIVE HUNDRED\0"
  "AEGEAN NUMBER FIVE THOUSAND\0"
  "AEGEAN NUMBER FORTY\0"
  "AEGEAN NUMBER FORTY THOUSAND\0"
  "AEGEAN NUMBER FOUR\0"
  "AEGEAN NUMBER FOUR HUNDRED\0"
  "AEGEAN NUMBER FOUR THOUSAND\0"
  "AEGEAN NUMBER NINE\0"
  "AEGEAN NUMBER NINE HUNDRED\0"
  "AEGEAN NUMBER NINE THOUSAND\0"
  "AEGEAN NUMBER NINETY\0"
  "AEGEAN NUMBER NINETY THOUSAND\0"
  "AEGEAN NUMBER ONE\0"
  "AEGEAN NUMBER ONE HUNDRED\0"
  "AEGEAN NUMBER ONE THOUSAND\0"
  "AEGEAN NUMBER SEVEN\0"
  "AEGEAN NUMBER SEVEN HUNDRED\0"
  "AEGEAN NUMBER SEVEN THOUSAND\0"
  "AEGEAN NUMBER SEVENTY\0"
  "AEGEAN NUMBER SEVENTY THOUSAND\0"
  "AEGEAN NUMBER SIX\0"
  "AEGEAN NUMBER SIX HUNDRED\0"
  "AEGEAN NUMBER SIX THOUSAND\0"
  "AEGEAN NUMBER SIXTY\0"
  "AEGEAN NUMBER SIXTY THOUSAND\0"
  "AEGEAN NUMBER TEN\0"
  "AEGEAN NUMBER TEN THOUSAND\0"
  "AEGEAN NUMBER THIRTY\0"
  "AEGEAN NUMBER THIRTY THOUSAND\0"
  "AEGEAN NUMBER THREE\0"
  "AEGEAN NUMBER THREE HUNDRED\0"
  "AEGEAN NUMBER THREE THOUSAND\0"
  "AEGEAN NUMBER TWENTY\0"
  "AEGEAN NUMBER TWENTY THOUSAND\0"
  "AEGEAN NUMBER TWO\0"
  "AEGEAN NUMBER TWO HUNDRED\0"
  "AEGEAN NUMBER TWO THOUSAND\0"
  "AEGEAN WEIGHT BASE UNIT\0"
  "AEGEAN WEIGHT FIRST SUBUNIT\0"
  "AEGEAN WEIGHT FOURTH SUBUNIT\0"
  "AEGEAN WEIGHT SECOND SUBUNIT\0"
  "AEGEAN WEIGHT THIRD SUBUNIT\0"
  "AEGEAN WORD SEPARATOR DOT\0"
  "AEGEAN WORD SEPARATOR LINE\0"
  "AERIAL TRAMWAY\0"
  "AFGHANI SIGN\0"
  "AHOM CONSONANT SIGN MEDIAL LA\0"
  "AHOM CONSONANT SIGN MEDIAL LIGATING RA\0"
  "AHOM CONSONANT SIGN MEDIAL RA\0"
  "AHOM DIGIT EIGHT\0"
  "AHOM DIGIT FIVE\0"
  "AHOM DIGIT FOUR\0"
  "AHOM DIGIT NINE\0"
  "AHOM DIGIT ONE\0"
  "AHOM DIGIT SEVEN\0"
  "AHOM DIGIT SIX\0"
  "AHOM DIGIT THREE\0"
  "AHOM DIGIT TWO\0"
  "AHOM DIGIT ZERO\0"
  "AHOM LETTER A\0"
  "AHOM LETTER ALTERNATE BA\0"
  "AHOM LETTER ALTERNATE GA\0"
  "AHOM LETTER ALTERNATE TA\0"
  "AHOM LETTER BA\0"
  "AHOM LETTER BHA\0"
  "AHOM LETTER CA\0"
  "AHOM LETTER CHA\0"
  "AHOM LETTER DA\0"
  "AHOM LETTER DDA\0"
  "AHOM LETTER DDHA\0"
  "AHOM LETTER DHA\0"
  "AHOM LETTER GA\0"
  "AHOM LETTER GHA\0"
  "AHOM LETTER HA\0"
  "AHOM LETTER JA\0"
  "AHOM LETTER JHA\0"
  "AHOM LETTER KA\0"
  "AHOM LETTER KHA\0"
  "AHOM LETTER LA\0"
  "AHOM LETTER LLA\0"
  "AHOM LETTER MA\0"
  "AHOM LETTER NA\0"
  "AHOM LETTER NGA\0"
  "AHOM LETTER NNA\0"
  "AHOM LETTER NYA\0"
  "AHOM LETTER PA\0"
  "AHOM LETTER PHA\0"
  "AHOM LETTER RA\0"
  "AHOM LETTER SA\0"
  "AHOM LETTER TA\0"
  "AHOM LETTER THA\0"
  "AHOM LETTER TTA\0"
  "AHOM LETTER TTHA\0"
  "AHOM NUMBER TEN\0"
  "AHOM NUMBER TWENTY\0"
  "AHOM SIGN KILLER\0"
  "AHOM SIGN RULAI\0"
  "AHOM SIGN SECTION\0"
  "AHOM SIGN SMALL SECTION\0"
  "AHOM SYMBOL VI\0"
  "AHOM VOWEL SIGN A\0"
  "AHOM VOWEL SIGN AA\0"
  "AHOM VOWEL SIGN AI\0"
  "AHOM VOWEL SIGN AM\0"
  "AHOM VOWEL SIGN AW\0"
  "AHOM VOWEL SIGN E\0"
  "AHOM VOWEL SIGN I\0"
  "AHOM VOWEL SIGN II\0"
  "AHOM VOWEL SIGN O\0"
  "AHOM VOWEL SIGN U\0"
  "AHOM VOWEL SIGN UU\0"
  "AIRPLANE\0"
  "AIRPLANE ARRIVING\0"
  "AIRPLANE DEPARTURE\0"
  "AKTIESELSKAB\0"
  "ALARM CLOCK\0"
  "ALCHEMICAL SYMBOL FOR AIR\0"
  "ALCHEMICAL SYMBOL FOR ALEMBIC\0"
  "ALCHEMICAL SYMBOL FOR ALKALI\0"
  "ALCHEMICAL SYMBOL FOR ALKALI-2\0"
  "ALCHEMICAL SYMBOL FOR ALUM\0"
  "ALCHEMICAL SYMBOL FOR AMALGAM\0"
  "ALCHEMICAL SYMBOL FOR ANTIMONY ORE\0"
  "ALCHEMICAL SYMBOL FOR AQUA REGIA\0"
  "ALCHEMICAL SYMBOL FOR AQUA REGIA-2\0"
  "ALCHEMICAL SYMBOL FOR AQUA VITAE\0"
  "ALCHEMICAL SYMBOL FOR AQUA VITAE-2\0"
  "ALCHEMICAL SYMBOL FOR AQUAFORTIS\0"
  "ALCHEMICAL SYMBOL FOR ARSENIC\0"
  "ALCHEMICAL SYMBOL FOR ASHES\0"
  "ALCHEMICAL SYMBOL FOR AURIPIGMENT\0"
  "ALCHEMICAL SYMBOL FOR BATH OF MARY\0"
  "ALCHEMICAL SYMBOL FOR BATH OF VAPOURS\0"
  "ALCHEMICAL SYMBOL FOR BISMUTH ORE\0"
  "ALCHEMICAL SYMBOL FOR BLACK SULFUR\0"
  "ALCHEMICAL SYMBOL FOR BORAX\0"
  "ALCHEMICAL SYMBOL FOR BORAX-2\0"
  "ALCHEMICAL SYMBOL FOR BORAX-3\0"
  "ALCHEMICAL SYMBOL FOR BRICK\0"
  "ALCHEMICAL SYMBOL FOR CADUCEUS\0"
  "ALCHEMICAL SYMBOL FOR CALX\0"
  "ALCHEMICAL SYMBOL FOR CAPUT MORTUUM\0"
  "ALCHEMICAL SYMBOL FOR CINNABAR\0"
  "ALCHEMICAL SYMBOL FOR COPPER ANTIMONIATE\0"
  "ALCHEMICAL SYMBOL FOR COPPER ORE\0"
  "ALCHEMICAL SYMBOL FOR CROCUS OF COPPER\0"
  "ALCHEMICAL SYMBOL FOR CROCUS OF COPPER-2\0"
  "ALCHEMICAL SYMBOL FOR CROCUS OF IRON\0"
  "ALCHEMICAL SYMBOL FOR CRUCIBLE\0"
  "ALCHEMICAL SYMBOL FOR CRUCIBLE-2\0"
  "ALCHEMICAL SYMBOL FOR CRUCIBLE-3\0"
  "ALCHEMICAL SYMBOL FOR CRUCIBLE-4\0"
  "ALCHEMICAL SYMBOL FOR CRUCIBLE-5\0"
  "ALCHEMICAL SYMBOL FOR DAY-NIGHT\0"
  "ALCHEMICAL SYMBOL FOR DISSOLVE\0"
  "ALCHEMICAL SYMBOL FOR DISSOLVE-2\0"
  "ALCHEMICAL SYMBOL FOR DISTILL\0"
  "ALCHEMICAL SYMBOL FOR EARTH\0"
  "ALCHEMICAL SYMBOL FOR FIRE\0"
  "ALCHEMICAL SYMBOL FOR GOLD\0"
  "ALCHEMICAL SYMBOL FOR GUM\0"
  "ALCHEMICAL SYMBOL FOR HALF DRAM\0"
  "ALCHEMICAL SYMBOL FOR HALF OUNCE\0"
  "ALCHEMICAL SYMBOL FOR HORSE DUNG\0"
  "ALCHEMICAL SYMBOL FOR HOUR\0"
  "ALCHEMICAL SYMBOL FOR IRON ORE\0"
  "ALCHEMICAL SYMBOL FOR IRON ORE-2\0"
  "ALCHEMICAL SYMBOL FOR IRON-COPPER ORE\0"
  "ALCHEMICAL SYMBOL FOR LEAD ORE\0"
  "ALCHEMICAL SYMBOL FOR LODESTONE\0"
  "ALCHEMICAL SYMBOL FOR MARCASITE\0"
  "ALCHEMICAL SYMBOL FOR MERCURY SUBLIMATE\0"
  "ALCHEMICAL SYMBOL FOR MERCURY SUBLIMATE-2\0"
  "ALCHEMICAL SYMBOL FOR MERCURY SUBLIMATE-3\0"
  "ALCHEMICAL SYMBOL FOR MONTH\0"
  "ALCHEMICAL SYMBOL FOR NIGHT\0"
  "ALCHEMICAL SYMBOL FOR NITRE\0"
  "ALCHEMICAL SYMBOL FOR OIL\0"
  "ALCHEMICAL SYMBOL FOR PHILOSOPHERS SULFUR\0"
  "ALCHEMICAL SYMBOL FOR POT ASHES\0"
  "ALCHEMICAL SYMBOL FOR POWDER\0"
  "ALCHEMICAL SYMBOL FOR POWDERED BRICK\0"
  "ALCHEMICAL SYMBOL FOR PRECIPITATE\0"
  "ALCHEMICAL SYMBOL FOR PURIFY\0"
  "ALCHEMICAL SYMBOL FOR PUTREFACTION\0"
  "ALCHEMICAL SYMBOL FOR QUICK LIME\0"
  "ALCHEMICAL SYMBOL FOR QUINTESSENCE\0"
  "ALCHEMICAL SYMBOL FOR REALGAR\0"
  "ALCHEMICAL SYMBOL FOR REALGAR-2\0"
  "ALCHEMICAL SYMBOL FOR REGULUS\0"
  "ALCHEMICAL SYMBOL FOR REGULUS OF ANTIMONY\0"
  "ALCHEMICAL SYMBOL FOR REGULUS OF ANTIMONY-2\0"
  "ALCHEMICAL SYMBOL FOR REGULUS OF IRON\0"
  "ALCHEMICAL SYMBOL FOR REGULUS-2\0"
  "ALCHEMICAL SYMBOL FOR REGULUS-3\0"
  "ALCHEMICAL SYMBOL FOR REGULUS-4\0"
  "ALCHEMICAL SYMBOL FOR RETORT\0"
  "ALCHEMICAL SYMBOL FOR ROCK SALT\0"
  "ALCHEMICAL SYMBOL FOR ROCK SALT-2\0"
  "ALCHEMICAL SYMBOL FOR SAL-AMMONIAC\0"
  "ALCHEMICAL SYMBOL FOR SALT\0"
  "ALCHEMICAL SYMBOL FOR SALT OF ANTIMONY\0"
  "ALCHEMICAL SYMBOL FOR SALT OF COPPER ANTIMONIATE\0"
  "ALCHEMICAL SYMBOL FOR SCEPTER OF JOVE\0"
  "ALCHEMICAL SYMBOL FOR SILVER\0"
  "ALCHEMICAL SYMBOL FOR SOAP\0"
  "ALCHEMICAL SYMBOL FOR SPIRIT\0"
  "ALCHEMICAL SYMBOL FOR STARRED TRIDENT\0"
  "ALCHEMICAL SYMBOL FOR STRATUM SUPER STRATUM\0"
  "ALCHEMICAL SYMBOL FOR STRATUM SUPER STRATUM-2\0"
  "ALCHEMICAL SYMBOL FOR SUBLIMATE OF ANTIMONY\0"
  "ALCHEMICAL SYMBOL FOR SUBLIMATE OF COPPER\0"
  "ALCHEMICAL SYMBOL FOR SUBLIMATE OF SALT OF ANTIMONY\0"
  "ALCHEMICAL SYMBOL FOR SUBLIMATE OF SALT OF COPPER\0"
  "ALCHEMICAL SYMBOL FOR SUBLIMATION\0"
  "ALCHEMICAL SYMBOL FOR SULFUR\0"
  "ALCHEMICAL SYMBOL FOR TARTAR\0"
  "ALCHEMICAL SYMBOL FOR TARTAR-2\0"
  "ALCHEMICAL SYMBOL FOR TIN ORE\0"
  "ALCHEMICAL SYMBOL FOR TINCTURE\0"
  "ALCHEMICAL SYMBOL FOR TRIDENT\0"
  "ALCHEMICAL SYMBOL FOR TUTTY\0"
  "ALCHEMICAL SYMBOL FOR URINE\0"
  "ALCHEMICAL SYMBOL FOR VERDIGRIS\0"
  "ALCHEMICAL SYMBOL FOR VINEGAR\0"
  "ALCHEMICAL SYMBOL FOR VINEGAR OF ANTIMONY\0"
  "ALCHEMICAL SYMBOL FOR VINEGAR-2\0"
  "ALCHEMICAL SYMBOL FOR VINEGAR-3\0"
  "ALCHEMICAL SYMBOL FOR VITRIOL\0"
  "ALCHEMICAL SYMBOL FOR VITRIOL-2\0"
  "ALCHEMICAL SYMBOL FOR WATER\0"
  "ALCHEMICAL SYMBOL FOR WAX\0"
  "ALEF SYMBOL\0"
  "ALEMBIC\0"
  "ALIEN CRAB STEPPING LEFT\0"
  "ALIEN CRAB STEPPING RIGHT\0"
  "ALIEN MONSTER\0"
  "ALIEN MONSTER CLOSED JAWS\0"
  "ALIEN MONSTER OPEN JAWS\0"
  "ALIEN MONSTER STEP-1\0"
  "ALIEN MONSTER STEP-2\0"
  "ALIEN SPIDER CROUCHING\0"
  "ALIEN SPIDER SPREAD\0"
  "ALIEN SQUID CLOSED TENTACLES\0"
  "ALIEN SQUID OPEN TENTACLES\0"
  "ALL AROUND-PROFILE\0"
  "ALL EQUAL TO\0"
  "ALMOST EQUAL OR EQUAL TO\0"
  "ALMOST EQUAL TO\0"
  "ALMOST EQUAL TO WITH CIRCUMFLEX ACCENT\0"
  "ALTERNATE ONE-WAY LEFT WAY TRAFFIC\0"
  "ALTERNATIVE KEY SYMBOL\0"
  "AMALGAMATION OR COPRODUCT\0"
  "AMBULANCE\0"
  "AMERICAN FOOTBALL\0"
  "AMPERSAND\0"
  "AMPHORA\0"
  "ANATOLIAN HIEROGLYPH A001\0"
  "ANATOLIAN HIEROGLYPH A002\0"
  "ANATOLIAN HIEROGLYPH A003\0"
  "ANATOLIAN HIEROGLYPH A004\0"
  "ANATOLIAN HIEROGLYPH A005\0"
  "ANATOLIAN HIEROGLYPH A006\0"
  "ANATOLIAN HIEROGLYPH A007\0"
  "ANATOLIAN HIEROGLYPH A008\0"
  "ANATOLIAN HIEROGLYPH A009\0"
  "ANATOLIAN HIEROGLYPH A010\0"
  "ANATOLIAN HIEROGLYPH A010A\0"
  "ANATOLIAN HIEROGLYPH A011\0"
  "ANATOLIAN HIEROGLYPH A012\0"
  "ANATOLIAN HIEROGLYPH A013\0"
  "ANATOLIAN HIEROGLYPH A014\0"
  "ANATOLIAN HIEROGLYPH A015\0"
  "ANATOLIAN HIEROGLYPH A016\0"
  "ANATOLIAN HIEROGLYPH A017\0"
  "ANATOLIAN HIEROGLYPH A018\0"
  "ANATOLIAN HIEROGLYPH A019\0"
  "ANATOLIAN HIEROGLYPH A020\0"
  "ANATOLIAN HIEROGLYPH A021\0"
  "ANATOLIAN HIEROGLYPH A022\0"
  "ANATOLIAN HIEROGLYPH A023\0"
  "ANATOLIAN HIEROGLYPH A024\0"
  "ANATOLIAN HIEROGLYPH A025\0"
  "ANATOLIAN HIEROGLYPH A026\0"
  "ANATOLIAN HIEROGLYPH A026A\0"
  "ANATOLIAN HIEROGLYPH A027\0"
  "ANATOLIAN HIEROGLYPH A028\0"
  "ANATOLIAN HIEROGLYPH A029\0"
  "ANATOLIAN HIEROGLYPH A030\0"
  "ANATOLIAN HIEROGLYPH A031\0"
  "ANATOLIAN HIEROGLYPH A032\0"
  "ANATOLIAN HIEROGLYPH A033\0"
  "ANATOLIAN HIEROGLYPH A034\0"
  "ANATOLIAN HIEROGLYPH A035\0"
  "ANATOLIAN HIEROGLYPH A036\0"
  "ANATOLIAN HIEROGLYPH A037\0"
  "ANATOLIAN HIEROGLYPH A038\0"
  "ANATOLIAN HIEROGLYPH A039\0"
  "ANATOLIAN HIEROGLYPH A039A\0"
  "ANATOLIAN HIEROGLYPH A040\0"
  "ANATOLIAN HIEROGLYPH A041\0"
  "ANATOLIAN HIEROGLYPH A041A\0"
  "ANATOLIAN HIEROGLYPH A042\0"
  "ANATOLIAN HIEROGLYPH A043\0"
  "ANATOLIAN HIEROGLYPH A044\0"
  "ANATOLIAN HIEROGLYPH A045\0"
  "ANATOLIAN HIEROGLYPH A045A\0"
  "ANATOLIAN HIEROGLYPH A046\0"
  "ANATOLIAN HIEROGLYPH A046A\0"
  "ANATOLIAN HIEROGLYPH A046B\0"
  "ANATOLIAN HIEROGLYPH A047\0"
  "ANATOLIAN HIEROGLYPH A048\0"
  "ANATOLIAN HIEROGLYPH A049\0"
  "ANATOLIAN HIEROGLYPH A050\0"
  "ANATOLIAN HIEROGLYPH A051\0"
  "ANATOLIAN HIEROGLYPH A052\0"
  "ANATOLIAN HIEROGLYPH A053\0"
  "ANATOLIAN HIEROGLYPH A054\0"
  "ANATOLIAN HIEROGLYPH A055\0"
  "ANATOLIAN HIEROGLYPH A056\0"
  "ANATOLIAN HIEROGLYPH A057\0"
  "ANATOLIAN HIEROGLYPH A058\0"
  "ANATOLIAN HIEROGLYPH A059\0"
  "ANATOLIAN HIEROGLYPH A060\0"
  "ANATOLIAN HIEROGLYPH A061\0"
  "ANATOLIAN HIEROGLYPH A062\0"
  "ANATOLIAN HIEROGLYPH A063\0"
  "ANATOLIAN HIEROGLYPH A064\0"
  "ANATOLIAN HIEROGLYPH A065\0"
  "ANATOLIAN HIEROGLYPH A066\0"
  "ANATOLIAN HIEROGLYPH A066A\0"
  "ANATOLIAN HIEROGLYPH A066B\0"
  "ANATOLIAN HIEROGLYPH A066C\0"
  "ANATOLIAN HIEROGLYPH A067\0"
  "ANATOLIAN HIEROGLYPH A068\0"
  "ANATOLIAN HIEROGLYPH A069\0"
  "ANATOLIAN HIEROGLYPH A070\0"
  "ANATOLIAN HIEROGLYPH A071\0"
  "ANATOLIAN HIEROGLYPH A072\0"
  "ANATOLIAN HIEROGLYPH A073\0"
  "ANATOLIAN HIEROGLYPH A074\0"
  "ANATOLIAN HIEROGLYPH A075\0"
  "ANATOLIAN HIEROGLYPH A076\0"
  "ANATOLIAN HIEROGLYPH A077\0"
  "ANATOLIAN HIEROGLYPH A078\0"
  "ANATOLIAN HIEROGLYPH A079\0"
  "ANATOLIAN HIEROGLYPH A080\0"
  "ANATOLIAN HIEROGLYPH A081\0"
  "ANATOLIAN HIEROGLYPH A082\0"
  "ANATOLIAN HIEROGLYPH A083\0"
  "ANATOLIAN HIEROGLYPH A084\0"
  "ANATOLIAN HIEROGLYPH A085\0"
  "ANATOLIAN HIEROGLYPH A086\0"
  "ANATOLIAN HIEROGLYPH A087\0"
  "ANATOLIAN HIEROGLYPH A088\0"
  "ANATOLIAN HIEROGLYPH A089\0"
  "ANATOLIAN HIEROGLYPH A090\0"
  "ANATOLIAN HIEROGLYPH A091\0"
  "ANATOLIAN HIEROGLYPH A092\0"
  "ANATOLIAN HIEROGLYPH A093\0"
  "ANATOLIAN HIEROGLYPH A094\0"
  "ANATOLIAN HIEROGLYPH A095\0"
  "ANATOLIAN HIEROGLYPH A096\0"
  "ANATOLIAN HIEROGLYPH A097\0"
  "ANATOLIAN HIEROGLYPH A097A\0"
  "ANATOLIAN HIEROGLYPH A098\0"
  "ANATOLIAN HIEROGLYPH A098A\0"
  "ANATOLIAN HIEROGLYPH A099\0"
  "ANATOLIAN HIEROGLYPH A100\0"
  "ANATOLIAN HIEROGLYPH A100A\0"
  "ANATOLIAN HIEROGLYPH A101\0"
  "ANATOLIAN HIEROGLYPH A101A\0"
  "ANATOLIAN HIEROGLYPH A102\0"
  "ANATOLIAN HIEROGLYPH A102A\0"
  "ANATOLIAN HIEROGLYPH A103\0"
  "ANATOLIAN HIEROGLYPH A104\0"
  "ANATOLIAN HIEROGLYPH A104A\0"
  "ANATOLIAN HIEROGLYPH A104B\0"
  "ANATOLIAN HIEROGLYPH A104C\0"
  "ANATOLIAN HIEROGLYPH A105\0"
  "ANATOLIAN HIEROGLYPH A105A\0"
  "ANATOLIAN HIEROGLYPH A105B\0"
  "ANATOLIAN HIEROGLYPH A106\0"
  "ANATOLIAN HIEROGLYPH A107\0"
  "ANATOLIAN HIEROGLYPH A107A\0"
  "ANATOLIAN HIEROGLYPH A107B\0"
  "ANATOLIAN HIEROGLYPH A107C\0"
  "ANATOLIAN HIEROGLYPH A108\0"
  "ANATOLIAN HIEROGLYPH A109\0"
  "ANATOLIAN HIEROGLYPH A110\0"
  "ANATOLIAN HIEROGLYPH A110A\0"
  "ANATOLIAN HIEROGLYPH A110B\0"
  "ANATOLIAN HIEROGLYPH A111\0"
  "ANATOLIAN HIEROGLYPH A112\0"
  "ANATOLIAN HIEROGLYPH A113\0"
  "ANATOLIAN HIEROGLYPH A114\0"
  "ANATOLIAN HIEROGLYPH A115\0"
  "ANATOLIAN HIEROGLYPH A115A\0"
  "ANATOLIAN HIEROGLYPH A116\0"
  "ANATOLIAN HIEROGLYPH A117\0"
  "ANATOLIAN HIEROGLYPH A118\0"
  "ANATOLIAN HIEROGLYPH A119\0"
  "ANATOLIAN HIEROGLYPH A120\0"
  "ANATOLIAN HIEROGLYPH A121\0"
  "ANATOLIAN HIEROGLYPH A122\0"
  "ANATOLIAN HIEROGLYPH A123\0"
  "ANATOLIAN HIEROGLYPH A124\0"
  "ANATOLIAN HIEROGLYPH A125\0"
  "ANATOLIAN HIEROGLYPH A125A\0"
  "ANATOLIAN HIEROGLYPH A126\0"
  "ANATOLIAN HIEROGLYPH A127\0"
  "ANATOLIAN HIEROGLYPH A128\0"
  "ANATOLIAN HIEROGLYPH A129\0"
  "ANATOLIAN HIEROGLYPH A130\0"
  "ANATOLIAN HIEROGLYPH A131\0"
  "ANATOLIAN HIEROGLYPH A132\0"
  "ANATOLIAN HIEROGLYPH A133\0"
  "ANATOLIAN HIEROGLYPH A134\0"
  "ANATOLIAN HIEROGLYPH A135\0"
  "ANATOLIAN HIEROGLYPH A135A\0"
  "ANATOLIAN HIEROGLYPH A136\0"
  "ANATOLIAN HIEROGLYPH A137\0"
  "ANATOLIAN HIEROGLYPH A138\0"
  "ANATOLIAN HIEROGLYPH A139\0"
  "ANATOLIAN HIEROGLYPH A140\0"
  "ANATOLIAN HIEROGLYPH A141\0"
  "ANATOLIAN HIEROGLYPH A142\0"
  "ANATOLIAN HIEROGLYPH A143\0"
  "ANATOLIAN HIEROGLYPH A144\0"
  "ANATOLIAN HIEROGLYPH A145\0"
  "ANATOLIAN HIEROGLYPH A146\0"
  "ANATOLIAN HIEROGLYPH A147\0"
  "ANATOLIAN HIEROGLYPH A148\0"
  "ANATOLIAN HIEROGLYPH A149\0"
  "ANATOLIAN HIEROGLYPH A150\0"
  "ANATOLIAN HIEROGLYPH A151\0"
  "ANATOLIAN HIEROGLYPH A152\0"
  "ANATOLIAN HIEROGLYPH A153\0"
  "ANATOLIAN HIEROGLYPH A154\0"
  "ANATOLIAN HIEROGLYPH A155\0"
  "ANATOLIAN HIEROGLYPH A156\0"
  "ANATOLIAN HIEROGLYPH A157\0"
  "ANATOLIAN HIEROGLYPH A158\0"
  "ANATOLIAN HIEROGLYPH A159\0"
  "ANATOLIAN HIEROGLYPH A160\0"
  "ANATOLIAN HIEROGLYPH A161\0"
  "ANATOLIAN HIEROGLYPH A162\0"
  "ANATOLIAN HIEROGLYPH A163\0"
  "ANATOLIAN HIEROGLYPH A164\0"
  "ANATOLIAN HIEROGLYPH A165\0"
  "ANATOLIAN HIEROGLYPH A166\0"
  "ANATOLIAN HIEROGLYPH A167\0"
  "ANATOLIAN HIEROGLYPH A168\0"
  "ANATOLIAN HIEROGLYPH A169\0"
  "ANATOLIAN HIEROGLYPH A170\0"
  "ANATOLIAN HIEROGLYPH A171\0"
  "ANATOLIAN HIEROGLYPH A172\0"
  "ANATOLIAN HIEROGLYPH A173\0"
  "ANATOLIAN HIEROGLYPH A174\0"
  "ANATOLIAN HIEROGLYPH A175\0"
  "ANATOLIAN HIEROGLYPH A176\0"
  "ANATOLIAN HIEROGLYPH A177\0"
  "ANATOLIAN HIEROGLYPH A178\0"
  "ANATOLIAN HIEROGLYPH A179\0"
  "ANATOLIAN HIEROGLYPH A180\0"
  "ANATOLIAN HIEROGLYPH A181\0"
  "ANATOLIAN HIEROGLYPH A182\0"
  "ANATOLIAN HIEROGLYPH A183\0"
  "ANATOLIAN HIEROGLYPH A184\0"
  "ANATOLIAN HIEROGLYPH A185\0"
  "ANATOLIAN HIEROGLYPH A186\0"
  "ANATOLIAN HIEROGLYPH A187\0"
  "ANATOLIAN HIEROGLYPH A188\0"
  "ANATOLIAN HIEROGLYPH A189\0"
  "ANATOLIAN HIEROGLYPH A190\0"
  "ANATOLIAN HIEROGLYPH A191\0"
  "ANATOLIAN HIEROGLYPH A192\0"
  "ANATOLIAN HIEROGLYPH A193\0"
  "ANATOLIAN HIEROGLYPH A194\0"
  "ANATOLIAN HIEROGLYPH A195\0"
  "ANATOLIAN HIEROGLYPH A196\0"
  "ANATOLIAN HIEROGLYPH A197\0"
  "ANATOLIAN HIEROGLYPH A198\0"
  "ANATOLIAN HIEROGLYPH A199\0"
  "ANATOLIAN HIEROGLYPH A200\0"
  "ANATOLIAN HIEROGLYPH A201\0"
  "ANATOLIAN HIEROGLYPH A202\0"
  "ANATOLIAN HIEROGLYPH A202A\0"
  "ANATOLIAN HIEROGLYPH A202B\0"
  "ANATOLIAN HIEROGLYPH A203\0"
  "ANATOLIAN HIEROGLYPH A204\0"
  "ANATOLIAN HIEROGLYPH A205\0"
  "ANATOLIAN HIEROGLYPH A206\0"
  "ANATOLIAN HIEROGLYPH A207\0"
  "ANATOLIAN HIEROGLYPH A207A\0"
  "ANATOLIAN HIEROGLYPH A208\0"
  "ANATOLIAN HIEROGLYPH A209\0"
  "ANATOLIAN HIEROGLYPH A209A\0"
  "ANATOLIAN HIEROGLYPH A210\0"
  "ANATOLIAN HIEROGLYPH A211\0"
  "ANATOLIAN HIEROGLYPH A212\0"
  "ANATOLIAN HIEROGLYPH A213\0"
  "ANATOLIAN HIEROGLYPH A214\0"
  "ANATOLIAN HIEROGLYPH A215\0"
  "ANATOLIAN HIEROGLYPH A215A\0"
  "ANATOLIAN HIEROGLYPH A216\0"
  "ANATOLIAN HIEROGLYPH A216A\0"
  "ANATOLIAN HIEROGLYPH A217\0"
  "ANATOLIAN HIEROGLYPH A218\0"
  "ANATOLIAN HIEROGLYPH A219\0"
  "ANATOLIAN HIEROGLYPH A220\0"
  "ANATOLIAN HIEROGLYPH A221\0"
  "ANATOLIAN HIEROGLYPH A222\0"
  "ANATOLIAN HIEROGLYPH A223\0"
  "ANATOLIAN HIEROGLYPH A224\0"
  "ANATOLIAN HIEROGLYPH A225\0"
  "ANATOLIAN HIEROGLYPH A226\0"
  "ANATOLIAN HIEROGLYPH A227\0"
  "ANATOLIAN HIEROGLYPH A227A\0"
  "ANATOLIAN HIEROGLYPH A228\0"
  "ANATOLIAN HIEROGLYPH A229\0"
  "ANATOLIAN HIEROGLYPH A230\0"
  "ANATOLIAN HIEROGLYPH A231\0"
  "ANATOLIAN HIEROGLYPH A232\0"
  "ANATOLIAN HIEROGLYPH A233\0"
  "ANATOLIAN HIEROGLYPH A234\0"
  "ANATOLIAN HIEROGLYPH A235\0"
  "ANATOLIAN HIEROGLYPH A236\0"
  "ANATOLIAN HIEROGLYPH A237\0"
  "ANATOLIAN HIEROGLYPH A238\0"
  "ANATOLIAN HIEROGLYPH A239\0"
  "ANATOLIAN HIEROGLYPH A240\0"
  "ANATOLIAN HIEROGLYPH A241\0"
  "ANATOLIAN HIEROGLYPH A242\0"
  "ANATOLIAN HIEROGLYPH A243\0"
  "ANATOLIAN HIEROGLYPH A244\0"
  "ANATOLIAN HIEROGLYPH A245\0"
  "ANATOLIAN HIEROGLYPH A246\0"
  "ANATOLIAN HIEROGLYPH A247\0"
  "ANATOLIAN HIEROGLYPH A248\0"
  "ANATOLIAN HIEROGLYPH A249\0"
  "ANATOLIAN HIEROGLYPH A250\0"
  "ANATOLIAN HIEROGLYPH A251\0"
  "ANATOLIAN HIEROGLYPH A252\0"
  "ANATOLIAN HIEROGLYPH A253\0"
  "ANATOLIAN HIEROGLYPH A254\0"
  "ANATOLIAN HIEROGLYPH A255\0"
  "ANATOLIAN HIEROGLYPH A256\0"
  "ANATOLIAN HIEROGLYPH A257\0"
  "ANATOLIAN HIEROGLYPH A258\0"
  "ANATOLIAN HIEROGLYPH A259\0"
  "ANATOLIAN HIEROGLYPH A260\0"
  "ANATOLIAN HIEROGLYPH A261\0"
  "ANATOLIAN HIEROGLYPH A262\0"
  "ANATOLIAN HIEROGLYPH A263\0"
  "ANATOLIAN HIEROGLYPH A264\0"
  "ANATOLIAN HIEROGLYPH A265\0"
  "ANATOLIAN HIEROGLYPH A266\0"
  "ANATOLIAN HIEROGLYPH A267\0"
  "ANATOLIAN HIEROGLYPH A267A\0"
  "ANATOLIAN HIEROGLYPH A268\0"
  "ANATOLIAN HIEROGLYPH A269\0"
  "ANATOLIAN HIEROGLYPH A270\0"
  "ANATOLIAN HIEROGLYPH A271\0"
  "ANATOLIAN HIEROGLYPH A272\0"
  "ANATOLIAN HIEROGLYPH A273\0"
  "ANATOLIAN HIEROGLYPH A274\0"
  "ANATOLIAN HIEROGLYPH A275\0"
  "ANATOLIAN HIEROGLYPH A276\0"
  "ANATOLIAN HIEROGLYPH A277\0"
  "ANATOLIAN HIEROGLYPH A278\0"
  "ANATOLIAN HIEROGLYPH A279\0"
  "ANATOLIAN HIEROGLYPH A280\0"
  "ANATOLIAN HIEROGLYPH A281\0"
  "ANATOLIAN HIEROGLYPH A282\0"
  "ANATOLIAN HIEROGLYPH A283\0"
  "ANATOLIAN HIEROGLYPH A284\0"
  "ANATOLIAN HIEROGLYPH A285\0"
  "ANATOLIAN HIEROGLYPH A286\0"
  "ANATOLIAN HIEROGLYPH A287\0"
  "ANATOLIAN HIEROGLYPH A288\0"
  "ANATOLIAN HIEROGLYPH A289\0"
  "ANATOLIAN HIEROGLYPH A289A\0"
  "ANATOLIAN HIEROGLYPH A290\0"
  "ANATOLIAN HIEROGLYPH A291\0"
  "ANATOLIAN HIEROGLYPH A292\0"
  "ANATOLIAN HIEROGLYPH A293\0"
  "ANATOLIAN HIEROGLYPH A294\0"
  "ANATOLIAN HIEROGLYPH A294A\0"
  "ANATOLIAN HIEROGLYPH A295\0"
  "ANATOLIAN HIEROGLYPH A296\0"
  "ANATOLIAN HIEROGLYPH A297\0"
  "ANATOLIAN HIEROGLYPH A298\0"
  "ANATOLIAN HIEROGLYPH A299\0"
  "ANATOLIAN HIEROGLYPH A299A\0"
  "ANATOLIAN HIEROGLYPH A300\0"
  "ANATOLIAN HIEROGLYPH A301\0"
  "ANATOLIAN HIEROGLYPH A302\0"
  "ANATOLIAN HIEROGLYPH A303\0"
  "ANATOLIAN HIEROGLYPH A304\0"
  "ANATOLIAN HIEROGLYPH A305\0"
  "ANATOLIAN HIEROGLYPH A306\0"
  "ANATOLIAN HIEROGLYPH A307\0"
  "ANATOLIAN HIEROGLYPH A308\0"
  "ANATOLIAN HIEROGLYPH A309\0"
  "ANATOLIAN HIEROGLYPH A309A\0"
  "ANATOLIAN HIEROGLYPH A310\0"
  "ANATOLIAN HIEROGLYPH A311\0"
  "ANATOLIAN HIEROGLYPH A312\0"
  "ANATOLIAN HIEROGLYPH A313\0"
  "ANATOLIAN HIEROGLYPH A314\0"
  "ANATOLIAN HIEROGLYPH A315\0"
  "ANATOLIAN HIEROGLYPH A316\0"
  "ANATOLIAN HIEROGLYPH A317\0"
  "ANATOLIAN HIEROGLYPH A318\0"
  "ANATOLIAN HIEROGLYPH A319\0"
  "ANATOLIAN HIEROGLYPH A320\0"
  "ANATOLIAN HIEROGLYPH A321\0"
  "ANATOLIAN HIEROGLYPH A322\0"
  "ANATOLIAN HIEROGLYPH A323\0"
  "ANATOLIAN HIEROGLYPH A324\0"
  "ANATOLIAN HIEROGLYPH A325\0"
  "ANATOLIAN HIEROGLYPH A326\0"
  "ANATOLIAN HIEROGLYPH A327\0"
  "ANATOLIAN HIEROGLYPH A328\0"
  "ANATOLIAN HIEROGLYPH A329\0"
  "ANATOLIAN HIEROGLYPH A329A\0"
  "ANATOLIAN HIEROGLYPH A330\0"
  "ANATOLIAN HIEROGLYPH A331\0"
  "ANATOLIAN HIEROGLYPH A332A\0"
  "ANATOLIAN HIEROGLYPH A332B\0"
  "ANATOLIAN HIEROGLYPH A332C\0"
  "ANATOLIAN HIEROGLYPH A333\0"
  "ANATOLIAN HIEROGLYPH A334\0"
  "ANATOLIAN HIEROGLYPH A335\0"
  "ANATOLIAN HIEROGLYPH A336\0"
  "ANATOLIAN HIEROGLYPH A336A\0"
  "ANATOLIAN HIEROGLYPH A336B\0"
  "ANATOLIAN HIEROGLYPH A336C\0"
  "ANATOLIAN HIEROGLYPH A337\0"
  "ANATOLIAN HIEROGLYPH A338\0"
  "ANATOLIAN HIEROGLYPH A339\0"
  "ANATOLIAN HIEROGLYPH A340\0"
  "ANATOLIAN HIEROGLYPH A341\0"
  "ANATOLIAN HIEROGLYPH A342\0"
  "ANATOLIAN HIEROGLYPH A343\0"
  "ANATOLIAN HIEROGLYPH A344\0"
  "ANATOLIAN HIEROGLYPH A345\0"
  "ANATOLIAN HIEROGLYPH A346\0"
  "ANATOLIAN HIEROGLYPH A347\0"
  "ANATOLIAN HIEROGLYPH A348\0"
  "ANATOLIAN HIEROGLYPH A349\0"
  "ANATOLIAN HIEROGLYPH A350\0"
  "ANATOLIAN HIEROGLYPH A351\0"
  "ANATOLIAN HIEROGLYPH A352\0"
  "ANATOLIAN HIEROGLYPH A353\0"
  "ANATOLIAN HIEROGLYPH A354\0"
  "ANATOLIAN HIEROGLYPH A355\0"
  "ANATOLIAN HIEROGLYPH A356\0"
  "ANATOLIAN HIEROGLYPH A357\0"
  "ANATOLIAN HIEROGLYPH A358\0"
  "ANATOLIAN HIEROGLYPH A359\0"
  "ANATOLIAN HIEROGLYPH A359A\0"
  "ANATOLIAN HIEROGLYPH A360\0"
  "ANATOLIAN HIEROGLYPH A361\0"
  "ANATOLIAN HIEROGLYPH A362\0"
  "ANATOLIAN HIEROGLYPH A363\0"
  "ANATOLIAN HIEROGLYPH A364\0"
  "ANATOLIAN HIEROGLYPH A364A\0"
  "ANATOLIAN HIEROGLYPH A365\0"
  "ANATOLIAN HIEROGLYPH A366\0"
  "ANATOLIAN HIEROGLYPH A367\0"
  "ANATOLIAN HIEROGLYPH A368\0"
  "ANATOLIAN HIEROGLYPH A368A\0"
  "ANATOLIAN HIEROGLYPH A369\0"
  "ANATOLIAN HIEROGLYPH A370\0"
  "ANATOLIAN HIEROGLYPH A371\0"
  "ANATOLIAN HIEROGLYPH A371A\0"
  "ANATOLIAN HIEROGLYPH A372\0"
  "ANATOLIAN HIEROGLYPH A373\0"
  "ANATOLIAN HIEROGLYPH A374\0"
  "ANATOLIAN HIEROGLYPH A375\0"
  "ANATOLIAN HIEROGLYPH A376\0"
  "ANATOLIAN HIEROGLYPH A377\0"
  "ANATOLIAN HIEROGLYPH A378\0"
  "ANATOLIAN HIEROGLYPH A379\0"
  "ANATOLIAN HIEROGLYPH A380\0"
  "ANATOLIAN HIEROGLYPH A381\0"
  "ANATOLIAN HIEROGLYPH A381A\0"
  "ANATOLIAN HIEROGLYPH A382\0"
  "ANATOLIAN HIEROGLYPH A383 RA OR RI\0"
  "ANATOLIAN HIEROGLYPH A383A\0"
  "ANATOLIAN HIEROGLYPH A384\0"
  "ANATOLIAN HIEROGLYPH A385\0"
  "ANATOLIAN HIEROGLYPH A386\0"
  "ANATOLIAN HIEROGLYPH A386A\0"
  "ANATOLIAN HIEROGLYPH A387\0"
  "ANATOLIAN HIEROGLYPH A388\0"
  "ANATOLIAN HIEROGLYPH A389\0"
  "ANATOLIAN HIEROGLYPH A390\0"
  "ANATOLIAN HIEROGLYPH A391\0"
  "ANATOLIAN HIEROGLYPH A392\0"
  "ANATOLIAN HIEROGLYPH A393 EIGHT\0"
  "ANATOLIAN HIEROGLYPH A394\0"
  "ANATOLIAN HIEROGLYPH A395\0"
  "ANATOLIAN HIEROGLYPH A396\0"
  "ANATOLIAN HIEROGLYPH A397\0"
  "ANATOLIAN HIEROGLYPH A398\0"
  "ANATOLIAN HIEROGLYPH A399\0"
  "ANATOLIAN HIEROGLYPH A400\0"
  "ANATOLIAN HIEROGLYPH A401\0"
  "ANATOLIAN HIEROGLYPH A402\0"
  "ANATOLIAN HIEROGLYPH A403\0"
  "ANATOLIAN HIEROGLYPH A404\0"
  "ANATOLIAN HIEROGLYPH A405\0"
  "ANATOLIAN HIEROGLYPH A406\0"
  "ANATOLIAN HIEROGLYPH A407\0"
  "ANATOLIAN HIEROGLYPH A408\0"
  "ANATOLIAN HIEROGLYPH A409\0"
  "ANATOLIAN HIEROGLYPH A410 BEGIN LOGOGRAM MARK\0"
  "ANATOLIAN HIEROGLYPH A410A END LOGOGRAM MARK\0"
  "ANATOLIAN HIEROGLYPH A411\0"
  "ANATOLIAN HIEROGLYPH A412\0"
  "ANATOLIAN HIEROGLYPH A413\0"
  "ANATOLIAN HIEROGLYPH A414\0"
  "ANATOLIAN HIEROGLYPH A415\0"
  "ANATOLIAN HIEROGLYPH A416\0"
  "ANATOLIAN HIEROGLYPH A417\0"
  "ANATOLIAN HIEROGLYPH A418\0"
  "ANATOLIAN HIEROGLYPH A419\0"
  "ANATOLIAN HIEROGLYPH A420\0"
  "ANATOLIAN HIEROGLYPH A421\0"
  "ANATOLIAN HIEROGLYPH A422\0"
  "ANATOLIAN HIEROGLYPH A423\0"
  "ANATOLIAN HIEROGLYPH A424\0"
  "ANATOLIAN HIEROGLYPH A425\0"
  "ANATOLIAN HIEROGLYPH A426\0"
  "ANATOLIAN HIEROGLYPH A427\0"
  "ANATOLIAN HIEROGLYPH A428\0"
  "ANATOLIAN HIEROGLYPH A429\0"
  "ANATOLIAN HIEROGLYPH A430\0"
  "ANATOLIAN HIEROGLYPH A431\0"
  "ANATOLIAN HIEROGLYPH A432\0"
  "ANATOLIAN HIEROGLYPH A433\0"
  "ANATOLIAN HIEROGLYPH A434\0"
  "ANATOLIAN HIEROGLYPH A435\0"
  "ANATOLIAN HIEROGLYPH A436\0"
  "ANATOLIAN HIEROGLYPH A437\0"
  "ANATOLIAN HIEROGLYPH A438\0"
  "ANATOLIAN HIEROGLYPH A439\0"
  "ANATOLIAN HIEROGLYPH A440\0"
  "ANATOLIAN HIEROGLYPH A441\0"
  "ANATOLIAN HIEROGLYPH A442\0"
  "ANATOLIAN HIEROGLYPH A443\0"
  "ANATOLIAN HIEROGLYPH A444\0"
  "ANATOLIAN HIEROGLYPH A445\0"
  "ANATOLIAN HIEROGLYPH A446\0"
  "ANATOLIAN HIEROGLYPH A447\0"
  "ANATOLIAN HIEROGLYPH A448\0"
  "ANATOLIAN HIEROGLYPH A449\0"
  "ANATOLIAN HIEROGLYPH A450\0"
  "ANATOLIAN HIEROGLYPH A450A\0"
  "ANATOLIAN HIEROGLYPH A451\0"
  "ANATOLIAN HIEROGLYPH A452\0"
  "ANATOLIAN HIEROGLYPH A453\0"
  "ANATOLIAN HIEROGLYPH A454\0"
  "ANATOLIAN HIEROGLYPH A455\0"
  "ANATOLIAN HIEROGLYPH A456\0"
  "ANATOLIAN HIEROGLYPH A457\0"
  "ANATOLIAN HIEROGLYPH A457A\0"
  "ANATOLIAN HIEROGLYPH A458\0"
  "ANATOLIAN HIEROGLYPH A459\0"
  "ANATOLIAN HIEROGLYPH A460\0"
  "ANATOLIAN HIEROGLYPH A461\0"
  "ANATOLIAN HIEROGLYPH A462\0"
  "ANATOLIAN HIEROGLYPH A463\0"
  "ANATOLIAN HIEROGLYPH A464\0"
  "ANATOLIAN HIEROGLYPH A465\0"
  "ANATOLIAN HIEROGLYPH A466\0"
  "ANATOLIAN HIEROGLYPH A467\0"
  "ANATOLIAN HIEROGLYPH A468\0"
  "ANATOLIAN HIEROGLYPH A469\0"
  "ANATOLIAN HIEROGLYPH A470\0"
  "ANATOLIAN HIEROGLYPH A471\0"
  "ANATOLIAN HIEROGLYPH A472\0"
  "ANATOLIAN HIEROGLYPH A473\0"
  "ANATOLIAN HIEROGLYPH A474\0"
  "ANATOLIAN HIEROGLYPH A475\0"
  "ANATOLIAN HIEROGLYPH A476\0"
  "ANATOLIAN HIEROGLYPH A477\0"
  "ANATOLIAN HIEROGLYPH A478\0"
  "ANATOLIAN HIEROGLYPH A479\0"
  "ANATOLIAN HIEROGLYPH A480\0"
  "ANATOLIAN HIEROGLYPH A481\0"
  "ANATOLIAN HIEROGLYPH A482\0"
  "ANATOLIAN HIEROGLYPH A483\0"
  "ANATOLIAN HIEROGLYPH A484\0"
  "ANATOLIAN HIEROGLYPH A485\0"
  "ANATOLIAN HIEROGLYPH A486\0"
  "ANATOLIAN HIEROGLYPH A487\0"
  "ANATOLIAN HIEROGLYPH A488\0"
  "ANATOLIAN HIEROGLYPH A489\0"
  "ANATOLIAN HIEROGLYPH A490\0"
  "ANATOLIAN HIEROGLYPH A491\0"
  "ANATOLIAN HIEROGLYPH A492\0"
  "ANATOLIAN HIEROGLYPH A493\0"
  "ANATOLIAN HIEROGLYPH A494\0"
  "ANATOLIAN HIEROGLYPH A495\0"
  "ANATOLIAN HIEROGLYPH A496\0"
  "ANATOLIAN HIEROGLYPH A497\0"
  "ANATOLIAN HIEROGLYPH A501\0"
  "ANATOLIAN HIEROGLYPH A502\0"
  "ANATOLIAN HIEROGLYPH A503\0"
  "ANATOLIAN HIEROGLYPH A504\0"
  "ANATOLIAN HIEROGLYPH A505\0"
  "ANATOLIAN HIEROGLYPH A506\0"
  "ANATOLIAN HIEROGLYPH A507\0"
  "ANATOLIAN HIEROGLYPH A508\0"
  "ANATOLIAN HIEROGLYPH A509\0"
  "ANATOLIAN HIEROGLYPH A510\0"
  "ANATOLIAN HIEROGLYPH A511\0"
  "ANATOLIAN HIEROGLYPH A512\0"
  "ANATOLIAN HIEROGLYPH A513\0"
  "ANATOLIAN HIEROGLYPH A514\0"
  "ANATOLIAN HIEROGLYPH A515\0"
  "ANATOLIAN HIEROGLYPH A516\0"
  "ANATOLIAN HIEROGLYPH A517\0"
  "ANATOLIAN HIEROGLYPH A518\0"
  "ANATOLIAN HIEROGLYPH A519\0"
  "ANATOLIAN HIEROGLYPH A520\0"
  "ANATOLIAN HIEROGLYPH A521\0"
  "ANATOLIAN HIEROGLYPH A522\0"
  "ANATOLIAN HIEROGLYPH A523\0"
  "ANATOLIAN HIEROGLYPH A524\0"
  "ANATOLIAN HIEROGLYPH A525\0"
  "ANATOLIAN HIEROGLYPH A526\0"
  "ANATOLIAN HIEROGLYPH A527\0"
  "ANATOLIAN HIEROGLYPH A528\0"
  "ANATOLIAN HIEROGLYPH A529\0"
  "ANATOLIAN HIEROGLYPH A530\0"
  "ANATOMICAL HEART\0"
  "ANCHOR\0"
  "AND WITH DOT\0"
  "ANGER SYMBOL\0"
  "ANGLE\0"
  "ANGLE WITH S INSIDE\0"
  "ANGLE WITH UNDERBAR\0"
  "ANGRY FACE\0"
  "ANGSTROM SIGN\0"
  "ANGUISHED FACE\0"
  "ANKH\0"
  "ANT\0"
  "ANTENNA\0"
  "ANTENNA WITH BARS\0"
  "ANTICLOCKWISE CLOSED CIRCLE ARROW\0"
  "ANTICLOCKWISE CONTOUR INTEGRAL\0"
  "ANTICLOCKWISE DOWNWARDS AND UPWARDS OPEN CIRCLE ARROWS\0"
  "ANTICLOCKWISE GAPPED CIRCLE ARROW\0"
  "ANTICLOCKWISE INTEGRATION\0"
  "ANTICLOCKWISE OPEN CIRCLE ARROW\0"
  "ANTICLOCKWISE TOP SEMICIRCLE ARROW\0"
  "ANTICLOCKWISE TRIANGLE-HEADED BOTTOM U-SHAPED ARROW\0"
  "ANTICLOCKWISE TRIANGLE-HEADED LEFT U-SHAPED ARROW\0"
  "ANTICLOCKWISE TRIANGLE-HEADED OPEN CIRCLE ARROW\0"
  "ANTICLOCKWISE TRIANGLE-HEADED RIGHT U-SHAPED ARROW\0"
  "ANTICLOCKWISE TRIANGLE-HEADED TOP U-SHAPED ARROW\0"
  "APL FUNCTIONAL SYMBOL ALPHA\0"
  "APL FUNCTIONAL SYMBOL ALPHA UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL BACKSLASH BAR\0"
  "APL FUNCTIONAL SYMBOL CIRCLE BACKSLASH\0"
  "APL FUNCTIONAL SYMBOL CIRCLE DIAERESIS\0"
  "APL FUNCTIONAL SYMBOL CIRCLE JOT\0"
  "APL FUNCTIONAL SYMBOL CIRCLE STAR\0"
  "APL FUNCTIONAL SYMBOL CIRCLE STILE\0"
  "APL FUNCTIONAL SYMBOL CIRCLE UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL COMMA BAR\0"
  "APL FUNCTIONAL SYMBOL DEL DIAERESIS\0"
  "APL FUNCTIONAL SYMBOL DEL STILE\0"
  "APL FUNCTIONAL SYMBOL DEL TILDE\0"
  "APL FUNCTIONAL SYMBOL DELTA STILE\0"
  "APL FUNCTIONAL SYMBOL DELTA UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL DIAMOND UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL DOWN CARET TILDE\0"
  "APL FUNCTIONAL SYMBOL DOWN SHOE STILE\0"
  "APL FUNCTIONAL SYMBOL DOWN TACK JOT\0"
  "APL FUNCTIONAL SYMBOL DOWN TACK UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL DOWNWARDS VANE\0"
  "APL FUNCTIONAL SYMBOL EPSILON UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL GREATER-THAN DIAERESIS\0"
  "APL FUNCTIONAL SYMBOL I-BEAM\0"
  "APL FUNCTIONAL SYMBOL IOTA\0"
  "APL FUNCTIONAL SYMBOL IOTA UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL JOT DIAERESIS\0"
  "APL FUNCTIONAL SYMBOL JOT UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL LEFT SHOE STILE\0"
  "APL FUNCTIONAL SYMBOL LEFTWARDS VANE\0"
  "APL FUNCTIONAL SYMBOL OMEGA\0"
  "APL FUNCTIONAL SYMBOL OMEGA UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL QUAD\0"
  "APL FUNCTIONAL SYMBOL QUAD BACKSLASH\0"
  "APL FUNCTIONAL SYMBOL QUAD CIRCLE\0"
  "APL FUNCTIONAL SYMBOL QUAD COLON\0"
  "APL FUNCTIONAL SYMBOL QUAD DEL\0"
  "APL FUNCTIONAL SYMBOL QUAD DELTA\0"
  "APL FUNCTIONAL SYMBOL QUAD DIAMOND\0"
  "APL FUNCTIONAL SYMBOL QUAD DIVIDE\0"
  "APL FUNCTIONAL SYMBOL QUAD DOWN CARET\0"
  "APL FUNCTIONAL SYMBOL QUAD DOWNWARDS ARROW\0"
  "APL FUNCTIONAL SYMBOL QUAD EQUAL\0"
  "APL FUNCTIONAL SYMBOL QUAD GREATER-THAN\0"
  "APL FUNCTIONAL SYMBOL QUAD JOT\0"
  "APL FUNCTIONAL SYMBOL QUAD LEFTWARDS ARROW\0"
  "APL FUNCTIONAL SYMBOL QUAD LESS-THAN\0"
  "APL FUNCTIONAL SYMBOL QUAD NOT EQUAL\0"
  "APL FUNCTIONAL SYMBOL QUAD QUESTION\0"
  "APL FUNCTIONAL SYMBOL QUAD RIGHTWARDS ARROW\0"
  "APL FUNCTIONAL SYMBOL QUAD SLASH\0"
  "APL FUNCTIONAL SYMBOL QUAD UP CARET\0"
  "APL FUNCTIONAL SYMBOL QUAD UPWARDS ARROW\0"
  "APL FUNCTIONAL SYMBOL QUOTE QUAD\0"
  "APL FUNCTIONAL SYMBOL QUOTE UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL RHO\0"
  "APL FUNCTIONAL SYMBOL RIGHTWARDS VANE\0"
  "APL FUNCTIONAL SYMBOL SEMICOLON UNDERBAR\0"
  "APL FUNCTIONAL SYMBOL SLASH BAR\0"
  "APL FUNCTIONAL SYMBOL SQUISH QUAD\0"
  "APL FUNCTIONAL SYMBOL STAR DIAERESIS\0"
  "APL FUNCTIONAL SYMBOL STILE TILDE\0"
  "APL FUNCTIONAL SYMBOL TILDE DIAERESIS\0"
  "APL FUNCTIONAL SYMBOL UP CARET TILDE\0"
  "APL FUNCTIONAL SYMBOL UP SHOE JOT\0"
  "APL FUNCTIONAL SYMBOL UP TACK DIAERESIS\0"
  "APL FUNCTIONAL SYMBOL UP TACK JOT\0"
  "APL FUNCTIONAL SYMBOL UP TACK OVERBAR\0"
  "APL FUNCTIONAL SYMBOL UPWARDS VANE\0"
  "APL FUNCTIONAL SYMBOL ZILDE\0"
  "APOLLON\0"
  "APOSTROPHE\0"
  "APPROACHES THE LIMIT\0"
  "APPROXIMATELY BUT NOT ACTUALLY EQUAL TO\0"
  "APPROXIMATELY EQUAL OR EQUAL TO\0"
  "APPROXIMATELY EQUAL TO\0"
  "APPROXIMATELY EQUAL TO OR THE IMAGE OF\0"
  "AQUARIUS\0"
  "ARABIC BASELINE ROUND DOT\0"
  "ARABIC COMBINING ALEF OVERLAY\0"
  "ARABIC COMMA\0"
  "ARABIC CURLY DAMMA\0"
  "ARABIC CURLY DAMMATAN\0"
  "ARABIC CURLY FATHA\0"
  "ARABIC CURLY FATHATAN\0"
  "ARABIC CURLY KASRA\0"
  "ARABIC CURLY KASRATAN\0"
  "ARABIC DAMMA\0"
  "ARABIC DAMMA ISOLATED FORM\0"
  "ARABIC DAMMA MEDIAL FORM\0"
  "ARABIC DAMMA WITH DOT\0"
  "ARABIC DAMMATAN\0"
  "ARABIC DAMMATAN ISOLATED FORM\0"
  "ARABIC DATE SEPARATOR\0"
  "ARABIC DECIMAL SEPARATOR\0"
  "ARABIC DISPUTED END OF AYAH\0"
  "ARABIC DOUBLE RIGHT ARROWHEAD ABOVE\0"
  "ARABIC DOUBLE RIGHT ARROWHEAD ABOVE WITH DOT\0"
  "ARABIC DOUBLED MADDA\0"
  "ARABIC EMPTY CENTRE HIGH STOP\0"
  "ARABIC EMPTY CENTRE LOW STOP\0"
  "ARABIC END OF AYAH\0"
  "ARABIC END OF TEXT MARK\0"
  "ARABIC FATHA\0"
  "ARABIC FATHA ISOLATED FORM\0"
  "ARABIC FATHA MEDIAL FORM\0"
  "ARABIC FATHA WITH DOT ABOVE\0"
  "ARABIC FATHA WITH RING\0"
  "ARABIC FATHA WITH TWO DOTS\0"
  "ARABIC FATHATAN\0"
  "ARABIC FATHATAN ISOLATED FORM\0"
  "ARABIC FIVE POINTED STAR\0"
  "ARABIC FOOTNOTE MARKER\0"
  "ARABIC FULL STOP\0"
  "ARABIC HALF MADDA OVER MADDA\0"
  "ARABIC HAMZA ABOVE\0"
  "ARABIC HAMZA BELOW\0"
  "ARABIC INVERTED DAMMA\0"
  "ARABIC KASRA\0"
  "ARABIC KASRA ISOLATED FORM\0"
  "ARABIC KASRA MEDIAL FORM\0"
  "ARABIC KASRA WITH DOT BELOW\0"
  "ARABIC KASRATAN\0"
  "ARABIC KASRATAN ISOLATED FORM\0"
  "ARABIC LARGE CIRCLE BELOW\0"
  "ARABIC LARGE ROUND DOT ABOVE\0"
  "ARABIC LARGE ROUND DOT BELOW\0"
  "ARABIC LARGE ROUND DOT INSIDE CIRCLE BELOW\0"
  "ARABIC LEFT ARROWHEAD ABOVE\0"
  "ARABIC LEFT ARROWHEAD BELOW\0"
  "ARABIC LETTER AE\0"
  "ARABIC LETTER AFRICAN FEH\0"
  "ARABIC LETTER AFRICAN NOON\0"
  "ARABIC LETTER AFRICAN QAF\0"
  "ARABIC LETTER AFRICAN QAF WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER AIN\0"
  "ARABIC LETTER AIN FINAL FORM\0"
  "ARABIC LETTER AIN INITIAL FORM\0"
  "ARABIC LETTER AIN ISOLATED FORM\0"
  "ARABIC LETTER AIN MEDIAL FORM\0"
  "ARABIC LETTER AIN WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER AIN WITH THREE DOTS BELOW\0"
  "ARABIC LETTER AIN WITH THREE DOTS POINTING DOWNWARDS ABOVE\0"
  "ARABIC LETTER AIN WITH TWO DOTS ABOVE\0"
  "ARABIC LETTER AIN WITH TWO DOTS VERTICALLY ABOVE\0"
  "ARABIC LETTER ALEF\0"
  "ARABIC LETTER ALEF FINAL FORM\0"
  "ARABIC LETTER ALEF ISOLATED FORM\0"
  "ARABIC LETTER ALEF MAKSURA\0"
  "ARABIC LETTER ALEF MAKSURA FINAL FORM\0"
  "ARABIC LETTER ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LETTER ALEF WASLA\0"
  "ARABIC LETTER ALEF WASLA FINAL FORM\0"
  "ARABIC LETTER ALEF WASLA ISOLATED FORM\0"
  "ARABIC LETTER ALEF WITH ATTACHED BOTTOM RIGHT KASRA\0"
  "ARABIC LETTER ALEF WITH ATTACHED BOTTOM RIGHT KASRA AND DOT ABOVE\0"
  "ARABIC LETTER ALEF WITH ATTACHED BOTTOM RIGHT KASRA AND LEFT RING\0"
  "ARABIC LETTER ALEF WITH ATTACHED FATHA\0"
  "ARABIC LETTER ALEF WITH ATTACHED KASRA\0"
  "ARABIC LETTER ALEF WITH ATTACHED LEFT HAMZA\0"
  "ARABIC LETTER ALEF WITH ATTACHED LEFT ROUND DOT\0"
  "ARABIC LETTER ALEF WITH ATTACHED RIGHT HAMZA\0"
  "ARABIC LETTER ALEF WITH ATTACHED RIGHT ROUND DOT\0"
  "ARABIC LETTER ALEF WITH ATTACHED ROUND DOT ABOVE\0"
  "ARABIC LETTER ALEF WITH ATTACHED ROUND DOT BELOW\0"
  "ARABIC LETTER ALEF WITH ATTACHED TOP RIGHT FATHA\0"
  "ARABIC LETTER ALEF WITH ATTACHED TOP RIGHT FATHA AND DOT ABOVE\0"
  "ARABIC LETTER ALEF WITH ATTACHED TOP RIGHT FATHA AND LEFT RING\0"
  "ARABIC LETTER ALEF WITH DOT ABOVE\0"
  "ARABIC LETTER ALEF WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE\0"
  "ARABIC LETTER ALEF WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE\0"
  "ARABIC LETTER ALEF WITH HAMZA ABOVE\0"
  "ARABIC LETTER ALEF WITH HAMZA ABOVE FINAL FORM\0"
  "ARABIC LETTER ALEF WITH HAMZA ABOVE ISOLATED FORM\0"
  "ARABIC LETTER ALEF WITH HAMZA BELOW\0"
  "ARABIC LETTER ALEF WITH HAMZA BELOW FINAL FORM\0"
  "ARABIC LETTER ALEF WITH HAMZA BELOW ISOLATED FORM\0"
  "ARABIC LETTER ALEF WITH LEFT MIDDLE STROKE\0"
  "ARABIC LETTER ALEF WITH MADDA ABOVE\0"
  "ARABIC LETTER ALEF WITH MADDA ABOVE FINAL FORM\0"
  "ARABIC LETTER ALEF WITH MADDA ABOVE ISOLATED FORM\0"
  "ARABIC LETTER ALEF WITH RIGHT MIDDLE STROKE\0"
  "ARABIC LETTER ALEF WITH RIGHT MIDDLE STROKE AND DOT ABOVE\0"
  "ARABIC LETTER ALEF WITH RIGHT MIDDLE STROKE AND LEFT RING\0"
  "ARABIC LETTER ALEF WITH WAVY HAMZA ABOVE\0"
  "ARABIC LETTER ALEF WITH WAVY HAMZA BELOW\0"
  "ARABIC LETTER BEEH\0"
  "ARABIC LETTER BEEH FINAL FORM\0"
  "ARABIC LETTER BEEH INITIAL FORM\0"
  "ARABIC LETTER BEEH ISOLATED FORM\0"
  "ARABIC LETTER BEEH MEDIAL FORM\0"
  "ARABIC LETTER BEH\0"
  "ARABIC LETTER BEH FINAL FORM\0"
  "ARABIC LETTER BEH INITIAL FORM\0"
  "ARABIC LETTER BEH ISOLATED FORM\0"
  "ARABIC LETTER BEH MEDIAL FORM\0"
  "ARABIC LETTER BEH WITH DOT BELOW AND THREE DOTS ABOVE\0"
  "ARABIC LETTER BEH WITH HAMZA ABOVE\0"
  "ARABIC LETTER BEH WITH INVERTED SMALL V BELOW\0"
  "ARABIC LETTER BEH WITH SMALL MEEM ABOVE\0"
  "ARABIC LETTER BEH WITH SMALL V\0"
  "ARABIC LETTER BEH WITH SMALL V BELOW\0"
  "ARABIC LETTER BEH WITH THREE DOTS HORIZONTALLY BELOW\0"
  "ARABIC LETTER BEH WITH THREE DOTS POINTING UPWARDS BELOW\0"
  "ARABIC LETTER BEH WITH THREE DOTS POINTING UPWARDS BELOW AND TWO DOTS ABOVE\0"
  "ARABIC LETTER BEH WITH TWO DOTS BELOW AND DOT ABOVE\0"
  "ARABIC LETTER BEHEH\0"
  "ARABIC LETTER BEHEH FINAL FORM\0"
  "ARABIC LETTER BEHEH INITIAL FORM\0"
  "ARABIC LETTER BEHEH ISOLATED FORM\0"
  "ARABIC LETTER BEHEH MEDIAL FORM\0"
  "ARABIC LETTER DAD\0"
  "ARABIC LETTER DAD FINAL FORM\0"
  "ARABIC LETTER DAD INITIAL FORM\0"
  "ARABIC LETTER DAD ISOLATED FORM\0"
  "ARABIC LETTER DAD MEDIAL FORM\0"
  "ARABIC LETTER DAD WITH DOT BELOW\0"
  "ARABIC LETTER DAHAL\0"
  "ARABIC LETTER DAHAL FINAL FORM\0"
  "ARABIC LETTER DAHAL ISOLATED FORM\0"
  "ARABIC LETTER DAL\0"
  "ARABIC LETTER DAL FINAL FORM\0"
  "ARABIC LETTER DAL ISOLATED FORM\0"
  "ARABIC LETTER DAL WITH DOT BELOW\0"
  "ARABIC LETTER DAL WITH DOT BELOW AND SMALL TAH\0"
  "ARABIC LETTER DAL WITH FOUR DOTS ABOVE\0"
  "ARABIC LETTER DAL WITH INVERTED SMALL V BELOW\0"
  "ARABIC LETTER DAL WITH INVERTED V\0"
  "ARABIC LETTER DAL WITH RING\0"
  "ARABIC LETTER DAL WITH THREE DOTS ABOVE DOWNWARDS\0"
  "ARABIC LETTER DAL WITH THREE DOTS BELOW\0"
  "ARABIC LETTER DAL WITH TWO DOTS VERTICALLY BELOW\0"
  "ARABIC LETTER DAL WITH TWO DOTS VERTICALLY BELOW AND SMALL TAH\0"
  "ARABIC LETTER DDAHAL\0"
  "ARABIC LETTER DDAHAL FINAL FORM\0"
  "ARABIC LETTER DDAHAL ISOLATED FORM\0"
  "ARABIC LETTER DDAL\0"
  "ARABIC LETTER DDAL FINAL FORM\0"
  "ARABIC LETTER DDAL ISOLATED FORM\0"
  "ARABIC LETTER DOTLESS BEH\0"
  "ARABIC LETTER DOTLESS FEH\0"
  "ARABIC LETTER DOTLESS QAF\0"
  "ARABIC LETTER DUL\0"
  "ARABIC LETTER DUL FINAL FORM\0"
  "ARABIC LETTER DUL ISOLATED FORM\0"
  "ARABIC LETTER DYEH\0"
  "ARABIC LETTER DYEH FINAL FORM\0"
  "ARABIC LETTER DYEH INITIAL FORM\0"
  "ARABIC LETTER DYEH ISOLATED FORM\0"
  "ARABIC LETTER DYEH MEDIAL FORM\0"
  "ARABIC LETTER E\0"
  "ARABIC LETTER E FINAL FORM\0"
  "ARABIC LETTER E INITIAL FORM\0"
  "ARABIC LETTER E ISOLATED FORM\0"
  "ARABIC LETTER E MEDIAL FORM\0"
  "ARABIC LETTER FARSI YEH\0"
  "ARABIC LETTER FARSI YEH FINAL FORM\0"
  "ARABIC LETTER FARSI YEH INITIAL FORM\0"
  "ARABIC LETTER FARSI YEH ISOLATED FORM\0"
  "ARABIC LETTER FARSI YEH MEDIAL FORM\0"
  "ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT FOUR BELOW\0"
  "ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE\0"
  "ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE\0"
  "ARABIC LETTER FARSI YEH WITH INVERTED V\0"
  "ARABIC LETTER FARSI YEH WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER FARSI YEH WITH TWO DOTS ABOVE\0"
  "ARABIC LETTER FEH\0"
  "ARABIC LETTER FEH FINAL FORM\0"
  "ARABIC LETTER FEH INITIAL FORM\0"
  "ARABIC LETTER FEH ISOLATED FORM\0"
  "ARABIC LETTER FEH MEDIAL FORM\0"
  "ARABIC LETTER FEH WITH DOT BELOW\0"
  "ARABIC LETTER FEH WITH DOT BELOW AND THREE DOTS ABOVE\0"
  "ARABIC LETTER FEH WITH DOT MOVED BELOW\0"
  "ARABIC LETTER FEH WITH THREE DOTS BELOW\0"
  "ARABIC LETTER FEH WITH THREE DOTS POINTING UPWARDS BELOW\0"
  "ARABIC LETTER FEH WITH TWO DOTS BELOW\0"
  "ARABIC LETTER GAF\0"
  "ARABIC LETTER GAF FINAL FORM\0"
  "ARABIC LETTER GAF INITIAL FORM\0"
  "ARABIC LETTER GAF ISOLATED FORM\0"
  "ARABIC LETTER GAF MEDIAL FORM\0"
  "ARABIC LETTER GAF WITH INVERTED STROKE\0"
  "ARABIC LETTER GAF WITH RING\0"
  "ARABIC LETTER GAF WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER GAF WITH TWO DOTS BELOW\0"
  "ARABIC LETTER GHAIN\0"
  "ARABIC LETTER GHAIN FINAL FORM\0"
  "ARABIC LETTER GHAIN INITIAL FORM\0"
  "ARABIC LETTER GHAIN ISOLATED FORM\0"
  "ARABIC LETTER GHAIN MEDIAL FORM\0"
  "ARABIC LETTER GHAIN WITH DOT BELOW\0"
  "ARABIC LETTER GHAIN WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER GRAF\0"
  "ARABIC LETTER GUEH\0"
  "ARABIC LETTER GUEH FINAL FORM\0"
  "ARABIC LETTER GUEH INITIAL FORM\0"
  "ARABIC LETTER GUEH ISOLATED FORM\0"
  "ARABIC LETTER GUEH MEDIAL FORM\0"
  "ARABIC LETTER HAH\0"
  "ARABIC LETTER HAH FINAL FORM\0"
  "ARABIC LETTER HAH INITIAL FORM\0"
  "ARABIC LETTER HAH ISOLATED FORM\0"
  "ARABIC LETTER HAH MEDIAL FORM\0"
  "ARABIC LETTER HAH WITH EXTENDED ARABIC-INDIC DIGIT FOUR BELOW\0"
  "ARABIC LETTER HAH WITH HAMZA ABOVE\0"
  "ARABIC LETTER HAH WITH INVERTED SMALL V BELOW\0"
  "ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH ABOVE\0"
  "ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH AND TWO DOTS\0"
  "ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH BELOW\0"
  "ARABIC LETTER HAH WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER HAH WITH THREE DOTS POINTING UPWARDS BELOW\0"
  "ARABIC LETTER HAH WITH TWO DOTS ABOVE\0"
  "ARABIC LETTER HAH WITH TWO DOTS VERTICAL ABOVE\0"
  "ARABIC LETTER HAMZA\0"
  "ARABIC LETTER HAMZA ISOLATED FORM\0"
  "ARABIC LETTER HEH\0"
  "ARABIC LETTER HEH DOACHASHMEE\0"
  "ARABIC LETTER HEH DOACHASHMEE FINAL FORM\0"
  "ARABIC LETTER HEH DOACHASHMEE INITIAL FORM\0"
  "ARABIC LETTER HEH DOACHASHMEE ISOLATED FORM\0"
  "ARABIC LETTER HEH DOACHASHMEE MEDIAL FORM\0"
  "ARABIC LETTER HEH FINAL FORM\0"
  "ARABIC LETTER HEH GOAL\0"
  "ARABIC LETTER HEH GOAL FINAL FORM\0"
  "ARABIC LETTER HEH GOAL INITIAL FORM\0"
  "ARABIC LETTER HEH GOAL ISOLATED FORM\0"
  "ARABIC LETTER HEH GOAL MEDIAL FORM\0"
  "ARABIC LETTER HEH GOAL WITH HAMZA ABOVE\0"
  "ARABIC LETTER HEH INITIAL FORM\0"
  "ARABIC LETTER HEH ISOLATED FORM\0"
  "ARABIC LETTER HEH MEDIAL FORM\0"
  "ARABIC LETTER HEH WITH INVERTED V\0"
  "ARABIC LETTER HEH WITH YEH ABOVE\0"
  "ARABIC LETTER HEH WITH YEH ABOVE FINAL FORM\0"
  "ARABIC LETTER HEH WITH YEH ABOVE ISOLATED FORM\0"
  "ARABIC LETTER HIGH HAMZA\0"
  "ARABIC LETTER HIGH HAMZA ALEF\0"
  "ARABIC LETTER HIGH HAMZA WAW\0"
  "ARABIC LETTER HIGH HAMZA YEH\0"
  "ARABIC LETTER JEEM\0"
  "ARABIC LETTER JEEM FINAL FORM\0"
  "ARABIC LETTER JEEM INITIAL FORM\0"
  "ARABIC LETTER JEEM ISOLATED FORM\0"
  "ARABIC LETTER JEEM MEDIAL FORM\0"
  "ARABIC LETTER JEEM WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER JEEM WITH THREE DOTS BELOW\0"
  "ARABIC LETTER JEEM WITH TWO DOTS ABOVE\0"
  "ARABIC LETTER JEH\0"
  "ARABIC LETTER JEH FINAL FORM\0"
  "ARABIC LETTER JEH ISOLATED FORM\0"
  "ARABIC LETTER KAF\0"
  "ARABIC LETTER KAF FINAL FORM\0"
  "ARABIC LETTER KAF INITIAL FORM\0"
  "ARABIC LETTER KAF ISOLATED FORM\0"
  "ARABIC LETTER KAF MEDIAL FORM\0"
  "ARABIC LETTER KAF WITH DOT ABOVE\0"
  "ARABIC LETTER KAF WITH DOT BELOW\0"
  "ARABIC LETTER KAF WITH RING\0"
  "ARABIC LETTER KAF WITH THREE DOTS BELOW\0"
  "ARABIC LETTER KAF WITH TWO DOTS ABOVE\0"
  "ARABIC LETTER KAF WITH TWO DOTS VERTICALLY BELOW\0"
  "ARABIC LETTER KASHMIRI YEH\0"
  "ARABIC LETTER KEHEH\0"
  "ARABIC LETTER KEHEH FINAL FORM\0"
  "ARABIC LETTER KEHEH INITIAL FORM\0"
  "ARABIC LETTER KEHEH ISOLATED FORM\0"
  "ARABIC LETTER KEHEH MEDIAL FORM\0"
  "ARABIC LETTER KEHEH WITH DOT ABOVE\0"
  "ARABIC LETTER KEHEH WITH SMALL V\0"
  "ARABIC LETTER KEHEH WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER KEHEH WITH THREE DOTS BELOW\0"
  "ARABIC LETTER KEHEH WITH THREE DOTS POINTING UPWARDS BELOW\0"
  "ARABIC LETTER KEHEH WITH TWO DOTS ABOVE\0"
  "ARABIC LETTER KEHEH WITH TWO DOTS VERTICALLY BELOW\0"
  "ARABIC LETTER KHAH\0"
  "ARABIC LETTER KHAH FINAL FORM\0"
  "ARABIC LETTER KHAH INITIAL FORM\0"
  "ARABIC LETTER KHAH ISOLATED FORM\0"
  "ARABIC LETTER KHAH MEDIAL FORM\0"
  "ARABIC LETTER KIRGHIZ OE\0"
  "ARABIC LETTER KIRGHIZ OE FINAL FORM\0"
  "ARABIC LETTER KIRGHIZ OE ISOLATED FORM\0"
  "ARABIC LETTER KIRGHIZ YU\0"
  "ARABIC LETTER KIRGHIZ YU FINAL FORM\0"
  "ARABIC LETTER KIRGHIZ YU ISOLATED FORM\0"
  "ARABIC LETTER LAM\0"
  "ARABIC LETTER LAM FINAL FORM\0"
  "ARABIC LETTER LAM INITIAL FORM\0"
  "ARABIC LETTER LAM ISOLATED FORM\0"
  "ARABIC LETTER LAM MEDIAL FORM\0"
  "ARABIC LETTER LAM WITH BAR\0"
  "ARABIC LETTER LAM WITH DOT ABOVE\0"
  "ARABIC LETTER LAM WITH DOUBLE BAR\0"
  "ARABIC LETTER LAM WITH SMALL ARABIC LETTER TAH ABOVE\0"
  "ARABIC LETTER LAM WITH SMALL V\0"
  "ARABIC LETTER LAM WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER LAM WITH THREE DOTS BELOW\0"
  "ARABIC LETTER LOW ALEF\0"
  "ARABIC LETTER MARK\0"
  "ARABIC LETTER MEEM\0"
  "ARABIC LETTER MEEM FINAL FORM\0"
  "ARABIC LETTER MEEM INITIAL FORM\0"
  "ARABIC LETTER MEEM ISOLATED FORM\0"
  "ARABIC LETTER MEEM MEDIAL FORM\0"
  "ARABIC LETTER MEEM WITH DOT ABOVE\0"
  "ARABIC LETTER MEEM WITH DOT BELOW\0"
  "ARABIC LETTER MEEM WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER NG\0"
  "ARABIC LETTER NG FINAL FORM\0"
  "ARABIC LETTER NG INITIAL FORM\0"
  "ARABIC LETTER NG ISOLATED FORM\0"
  "ARABIC LETTER NG MEDIAL FORM\0"
  "ARABIC LETTER NGOEH\0"
  "ARABIC LETTER NGOEH FINAL FORM\0"
  "ARABIC LETTER NGOEH INITIAL FORM\0"
  "ARABIC LETTER NGOEH ISOLATED FORM\0"
  "ARABIC LETTER NGOEH MEDIAL FORM\0"
  "ARABIC LETTER NOON\0"
  "ARABIC LETTER NOON FINAL FORM\0"
  "ARABIC LETTER NOON GHUNNA\0"
  "ARABIC LETTER NOON GHUNNA FINAL FORM\0"
  "ARABIC LETTER NOON GHUNNA ISOLATED FORM\0"
  "ARABIC LETTER NOON INITIAL FORM\0"
  "ARABIC LETTER NOON ISOLATED FORM\0"
  "ARABIC LETTER NOON MEDIAL FORM\0"
  "ARABIC LETTER NOON WITH DOT BELOW\0"
  "ARABIC LETTER NOON WITH INVERTED SMALL V\0"
  "ARABIC LETTER NOON WITH RING\0"
  "ARABIC LETTER NOON WITH SMALL TAH\0"
  "ARABIC LETTER NOON WITH SMALL V\0"
  "ARABIC LETTER NOON WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER NOON WITH TWO DOTS BELOW\0"
  "ARABIC LETTER NYEH\0"
  "ARABIC LETTER NYEH FINAL FORM\0"
  "ARABIC LETTER NYEH INITIAL FORM\0"
  "ARABIC LETTER NYEH ISOLATED FORM\0"
  "ARABIC LETTER NYEH MEDIAL FORM\0"
  "ARABIC LETTER OE\0"
  "ARABIC LETTER OE FINAL FORM\0"
  "ARABIC LETTER OE ISOLATED FORM\0"
  "ARABIC LETTER PEH\0"
  "ARABIC LETTER PEH FINAL FORM\0"
  "ARABIC LETTER PEH INITIAL FORM\0"
  "ARABIC LETTER PEH ISOLATED FORM\0"
  "ARABIC LETTER PEH MEDIAL FORM\0"
  "ARABIC LETTER PEH WITH SMALL MEEM ABOVE\0"
  "ARABIC LETTER PEH WITH SMALL V\0"
  "ARABIC LETTER PEHEH\0"
  "ARABIC LETTER PEHEH FINAL FORM\0"
  "ARABIC LETTER PEHEH INITIAL FORM\0"
  "ARABIC LETTER PEHEH ISOLATED FORM\0"
  "ARABIC LETTER PEHEH MEDIAL FORM\0"
  "ARABIC LETTER QAF\0"
  "ARABIC LETTER QAF FINAL FORM\0"
  "ARABIC LETTER QAF INITIAL FORM\0"
  "ARABIC LETTER QAF ISOLATED FORM\0"
  "ARABIC LETTER QAF MEDIAL FORM\0"
  "ARABIC LETTER QAF WITH DOT ABOVE\0"
  "ARABIC LETTER QAF WITH DOT BELOW\0"
  "ARABIC LETTER QAF WITH DOT BELOW AND NO DOTS ABOVE\0"
  "ARABIC LETTER QAF WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER REH\0"
  "ARABIC LETTER REH FINAL FORM\0"
  "ARABIC LETTER REH ISOLATED FORM\0"
  "ARABIC LETTER REH WITH DOT BELOW\0"
  "ARABIC LETTER REH WITH DOT BELOW AND DOT ABOVE\0"
  "ARABIC LETTER REH WITH FOUR DOTS ABOVE\0"
  "ARABIC LETTER REH WITH HAMZA ABOVE\0"
  "ARABIC LETTER REH WITH INVERTED V\0"
  "ARABIC LETTER REH WITH LOOP\0"
  "ARABIC LETTER REH WITH RING\0"
  "ARABIC LETTER REH WITH SMALL ARABIC LETTER TAH AND TWO DOTS\0"
  "ARABIC LETTER REH WITH SMALL NOON ABOVE\0"
  "ARABIC LETTER REH WITH SMALL V\0"
  "ARABIC LETTER REH WITH SMALL V BELOW\0"
  "ARABIC LETTER REH WITH STROKE\0"
  "ARABIC LETTER REH WITH TWO DOTS ABOVE\0"
  "ARABIC LETTER REH WITH TWO DOTS VERTICALLY ABOVE\0"
  "ARABIC LETTER RNOON\0"
  "ARABIC LETTER RNOON FINAL FORM\0"
  "ARABIC LETTER RNOON INITIAL FORM\0"
  "ARABIC LETTER RNOON ISOLATED FORM\0"
  "ARABIC LETTER RNOON MEDIAL FORM\0"
  "ARABIC LETTER ROHINGYA YEH\0"
  "ARABIC LETTER RREH\0"
  "ARABIC LETTER RREH FINAL FORM\0"
  "ARABIC LETTER RREH ISOLATED FORM\0"
  "ARABIC LETTER SAD\0"
  "ARABIC LETTER SAD FINAL FORM\0"
  "ARABIC LETTER SAD INITIAL FORM\0"
  "ARABIC LETTER SAD ISOLATED FORM\0"
  "ARABIC LETTER SAD MEDIAL FORM\0"
  "ARABIC LETTER SAD WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER SAD WITH THREE DOTS BELOW\0"
  "ARABIC LETTER SAD WITH TWO DOTS BELOW\0"
  "ARABIC LETTER SEEN\0"
  "ARABIC LETTER SEEN FINAL FORM\0"
  "ARABIC LETTER SEEN INITIAL FORM\0"
  "ARABIC LETTER SEEN ISOLATED FORM\0"
  "ARABIC LETTER SEEN MEDIAL FORM\0"
  "ARABIC LETTER SEEN WITH DOT BELOW AND DOT ABOVE\0"
  "ARABIC LETTER SEEN WITH EXTENDED ARABIC-INDIC DIGIT FOUR ABOVE\0"
  "ARABIC LETTER SEEN WITH FOUR DOTS ABOVE\0"
  "ARABIC LETTER SEEN WITH INVERTED V\0"
  "ARABIC LETTER SEEN WITH SMALL ARABIC LETTER TAH AND TWO DOTS\0"
  "ARABIC LETTER SEEN WITH THREE DOTS BELOW\0"
  "ARABIC LETTER SEEN WITH THREE DOTS BELOW AND THREE DOTS ABOVE\0"
  "ARABIC LETTER SEEN WITH TWO DOTS VERTICALLY ABOVE\0"
  "ARABIC LETTER SHEEN\0"
  "ARABIC LETTER SHEEN FINAL FORM\0"
  "ARABIC LETTER SHEEN INITIAL FORM\0"
  "ARABIC LETTER SHEEN ISOLATED FORM\0"
  "ARABIC LETTER SHEEN MEDIAL FORM\0"
  "ARABIC LETTER SHEEN WITH DOT BELOW\0"
  "ARABIC LETTER STRAIGHT WAW\0"
  "ARABIC LETTER SUPERSCRIPT ALEF\0"
  "ARABIC LETTER SWASH KAF\0"
  "ARABIC LETTER TAH\0"
  "ARABIC LETTER TAH FINAL FORM\0"
  "ARABIC LETTER TAH INITIAL FORM\0"
  "ARABIC LETTER TAH ISOLATED FORM\0"
  "ARABIC LETTER TAH MEDIAL FORM\0"
  "ARABIC LETTER TAH WITH DOT BELOW\0"
  "ARABIC LETTER TAH WITH THREE DOTS ABOVE\0"
  "ARABIC LETTER TAH WITH THREE DOTS BELOW\0"
  "ARABIC LETTER TAH WITH TWO DOTS ABOVE\0"
  "ARABIC LETTER TAH WITH TWO DOTS VERTICALLY BELOW\0"
  "ARABIC LETTER TCHEH\0"
  "ARABIC LETTER TCHEH FINAL FORM\0"
  "ARABIC LETTER TCHEH INITIAL FORM\0"
  "ARABIC LETTER TCHEH ISOLATED FORM\0"
  "ARABIC LETTER TCHEH MEDIAL FORM\0"
  "ARABIC LETTER TCHEH WITH DOT ABOVE\0"
  "ARABIC LETTER TCHEH WITH SMALL V\0"
  "ARABIC LETTER TCHEHEH\0"
  "ARABIC LETTER TCHEHEH FINAL FORM\0"
  "ARABIC LETTER TCHEHEH INITIAL FORM\0"
  "ARABIC LETTER TCHEHEH ISOLATED FORM\0"
  "ARABIC LETTER TCHEHEH MEDIAL FORM\0"
  "ARABIC LETTER TEH\0"
  "ARABIC LETTER TEH FINAL FORM\0"
  "ARABIC LETTER TEH INITIAL FORM\0"
  "ARABIC LETTER TEH ISOLATED FORM\0"
  "ARABIC LETTER TEH MARBUTA\0"
  "ARABIC LETTER TEH MARBUTA FINAL FORM\0"
  "ARABIC LETTER TEH MARBUTA GOAL\0"
  "ARABIC LETTER TEH MARBUTA ISOLATED FORM\0"
  "ARABIC LETTER TEH MEDIAL FORM\0"
  "ARABIC LETTER TEH WITH RING\0"
  "ARABIC LETTER TEH WITH SMALL TEH ABOVE\0"
  "ARABIC LETTER TEH WITH SMALL V\0"
  "ARABIC LETTER TEH WITH THREE DOTS ABOVE DOWNWARDS\0"
  "ARABIC LETTER TEHEH\0"
  "ARABIC LETTER TEHEH FINAL FORM\0"
  "ARABIC LETTER TEHEH INITIAL FORM\0"
  "ARABIC LETTER TEHEH ISOLATED FORM\0"
  "ARABIC LETTER TEHEH MEDIAL FORM\0"
  "ARABIC LETTER THAL\0"
  "ARABIC LETTER THAL FINAL FORM\0"
  "ARABIC LETTER THAL ISOLATED FORM\0"
  "ARABIC LETTER THEH\0"
  "ARABIC LETTER THEH FINAL FORM\0"
  "ARABIC LETTER THEH INITIAL FORM\0"
  "ARABIC LETTER THEH ISOLATED FORM\0"
  "ARABIC LETTER THEH MEDIAL FORM\0"
  "ARABIC LETTER THIN YEH\0"
  "ARABIC LETTER TTEH\0"
  "ARABIC LETTER TTEH FINAL FORM\0"
  "ARABIC LETTER TTEH INITIAL FORM\0"
  "ARABIC LETTER TTEH ISOLATED FORM\0"
  "ARABIC LETTER TTEH MEDIAL FORM\0"
  "ARABIC LETTER TTEH WITH SMALL V\0"
  "ARABIC LETTER TTEHEH\0"
  "ARABIC LETTER TTEHEH FINAL FORM\0"
  "ARABIC LETTER TTEHEH INITIAL FORM\0"
  "ARABIC LETTER TTEHEH ISOLATED FORM\0"
  "ARABIC LETTER TTEHEH MEDIAL FORM\0"
  "ARABIC LETTER U\0"
  "ARABIC LETTER U FINAL FORM\0"
  "ARABIC LETTER U ISOLATED FORM\0"
  "ARABIC LETTER U WITH HAMZA ABOVE\0"
  "ARABIC LETTER U WITH HAMZA ABOVE ISOLATED FORM\0"
  "ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA INITIAL FORM\0"
  "ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA MEDIAL FORM\0"
  "ARABIC LETTER VE\0"
  "ARABIC LETTER VE FINAL FORM\0"
  "ARABIC LETTER VE ISOLATED FORM\0"
  "ARABIC LETTER VEH\0"
  "ARABIC LETTER VEH FINAL FORM\0"
  "ARABIC LETTER VEH INITIAL FORM\0"
  "ARABIC LETTER VEH ISOLATED FORM\0"
  "ARABIC LETTER VEH MEDIAL FORM\0"
  "ARABIC LETTER WAW\0"
  "ARABIC LETTER WAW FINAL FORM\0"
  "ARABIC LETTER WAW ISOLATED FORM\0"
  "ARABIC LETTER WAW WITH DOT ABOVE\0"
  "ARABIC LETTER WAW WITH DOT WITHIN\0"
  "ARABIC LETTER WAW WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE\0"
  "ARABIC LETTER WAW WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE\0"
  "ARABIC LETTER WAW WITH HAMZA ABOVE\0"
  "ARABIC LETTER WAW WITH HAMZA ABOVE FINAL FORM\0"
  "ARABIC LETTER WAW WITH HAMZA ABOVE ISOLATED FORM\0"
  "ARABIC LETTER WAW WITH RING\0"
  "ARABIC LETTER WAW WITH TWO DOTS ABOVE\0"
  "ARABIC LETTER YEH\0"
  "ARABIC LETTER YEH BARREE\0"
  "ARABIC LETTER YEH BARREE FINAL FORM\0"
  "ARABIC LETTER YEH BARREE ISOLATED FORM\0"
  "ARABIC LETTER YEH BARREE WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE\0"
  "ARABIC LETTER YEH BARREE WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE\0"
  "ARABIC LETTER YEH BARREE WITH HAMZA ABOVE\0"
  "ARABIC LETTER YEH BARREE WITH HAMZA ABOVE FINAL FORM\0"
  "ARABIC LETTER YEH BARREE WITH HAMZA ABOVE ISOLATED FORM\0"
  "ARABIC LETTER YEH FINAL FORM\0"
  "ARABIC LETTER YEH INITIAL FORM\0"
  "ARABIC LETTER YEH ISOLATED FORM\0"
  "ARABIC LETTER YEH MEDIAL FORM\0"
  "ARABIC LETTER YEH WITH HAMZA ABOVE\0"
  "ARABIC LETTER YEH WITH HAMZA ABOVE FINAL FORM\0"
  "ARABIC LETTER YEH WITH HAMZA ABOVE INITIAL FORM\0"
  "ARABIC LETTER YEH WITH HAMZA ABOVE ISOLATED FORM\0"
  "ARABIC LETTER YEH WITH HAMZA ABOVE MEDIAL FORM\0"
  "ARABIC LETTER YEH WITH SMALL V\0"
  "ARABIC LETTER YEH WITH TAIL\0"
  "ARABIC LETTER YEH WITH THREE DOTS BELOW\0"
  "ARABIC LETTER YEH WITH TWO DOTS BELOW AND DOT ABOVE\0"
  "ARABIC LETTER YEH WITH TWO DOTS BELOW AND HAMZA ABOVE\0"
  "ARABIC LETTER YEH WITH TWO DOTS BELOW AND SMALL NOON ABOVE\0"
  "ARABIC LETTER YU\0"
  "ARABIC LETTER YU FINAL FORM\0"
  "ARABIC LETTER YU ISOLATED FORM\0"
  "ARABIC LETTER ZAH\0"
  "ARABIC LETTER ZAH FINAL FORM\0"
  "ARABIC LETTER ZAH INITIAL FORM\0"
  "ARABIC LETTER ZAH ISOLATED FORM\0"
  "ARABIC LETTER ZAH MEDIAL FORM\0"
  "ARABIC LETTER ZAIN\0"
  "ARABIC LETTER ZAIN FINAL FORM\0"
  "ARABIC LETTER ZAIN ISOLATED FORM\0"
  "ARABIC LETTER ZAIN WITH INVERTED V ABOVE\0"
  "ARABIC LIGATURE AIN WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE AIN WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE AIN WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE AIN WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE AIN WITH JEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE AIN WITH JEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE AIN WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE AIN WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE AIN WITH MEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE AIN WITH MEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE AIN WITH MEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE AIN WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE AIN WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE AIN WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE AKBAR ISOLATED FORM\0"
  "ARABIC LIGATURE ALAYHAA AS-SALAAM\0"
  "ARABIC LIGATURE ALAYHE ISOLATED FORM\0"
  "ARABIC LIGATURE ALAYHI AS-SALAAM\0"
  "ARABIC LIGATURE ALAYHI AS-SALAATU WAS-SALAAM\0"
  "ARABIC LIGATURE ALAYHIM AS-SALAAM\0"
  "ARABIC LIGATURE ALAYHIMAA AS-SALAAM\0"
  "ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF FINAL FORM\0"
  "ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF ISOLATED FORM\0"
  "ARABIC LIGATURE ALEF WITH FATHATAN FINAL FORM\0"
  "ARABIC LIGATURE ALEF WITH FATHATAN ISOLATED FORM\0"
  "ARABIC LIGATURE ALLAH ISOLATED FORM\0"
  "ARABIC LIGATURE AZZA WA JALL\0"
  "ARABIC LIGATURE BEH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE BEH WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE BEH WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE BEH WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE BEH WITH HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE BEH WITH HEH INITIAL FORM\0"
  "ARABIC LIGATURE BEH WITH HEH MEDIAL FORM\0"
  "ARABIC LIGATURE BEH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE BEH WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE BEH WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE BEH WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE BEH WITH KHAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE BEH WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE BEH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE BEH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE BEH WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE BEH WITH NOON FINAL FORM\0"
  "ARABIC LIGATURE BEH WITH REH FINAL FORM\0"
  "ARABIC LIGATURE BEH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE BEH WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE BEH WITH ZAIN FINAL FORM\0"
  "ARABIC LIGATURE BISMILLAH AR-RAHMAN AR-RAHEEM\0"
  "ARABIC LIGATURE DAD WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE DAD WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE DAD WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE DAD WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE DAD WITH HAH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE DAD WITH HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE DAD WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE DAD WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE DAD WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE DAD WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE DAD WITH KHAH WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE DAD WITH KHAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE DAD WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE DAD WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE DAD WITH REH FINAL FORM\0"
  "ARABIC LIGATURE DAD WITH REH ISOLATED FORM\0"
  "ARABIC LIGATURE DAD WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE DAD WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE FEH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE FEH WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE FEH WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE FEH WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE FEH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE FEH WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE FEH WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE FEH WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE FEH WITH KHAH WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE FEH WITH KHAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE FEH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE FEH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE FEH WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE FEH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE FEH WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE GHAIN WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE GHAIN WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE GHAIN WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE GHAIN WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE GHAIN WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE GHAIN WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE GHAIN WITH MEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE GHAIN WITH MEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE GHAIN WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE GHAIN WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE GHAIN WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE HAH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE HAH WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE HAH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE HAH WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE HAH WITH JEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE HAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE HAH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE HAH WITH MEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE HAH WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE HAH WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE HEH WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE HEH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE HEH WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE HEH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE HEH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE HEH WITH MEEM WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE HEH WITH MEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE HEH WITH SUPERSCRIPT ALEF INITIAL FORM\0"
  "ARABIC LIGATURE HEH WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE JALLAJALALOUHOU\0"
  "ARABIC LIGATURE JEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE JEEM WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE JEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE JEEM WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE JEEM WITH HAH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE JEEM WITH HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE JEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE JEEM WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE JEEM WITH MEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE JEEM WITH MEEM WITH HAH FINAL FORM\0"
  "ARABIC LIGATURE JEEM WITH MEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE JEEM WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE JEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE JEEM WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE KAF WITH ALEF FINAL FORM\0"
  "ARABIC LIGATURE KAF WITH ALEF ISOLATED FORM\0"
  "ARABIC LIGATURE KAF WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE KAF WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE KAF WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE KAF WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE KAF WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE KAF WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE KAF WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE KAF WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE KAF WITH LAM FINAL FORM\0"
  "ARABIC LIGATURE KAF WITH LAM INITIAL FORM\0"
  "ARABIC LIGATURE KAF WITH LAM ISOLATED FORM\0"
  "ARABIC LIGATURE KAF WITH LAM MEDIAL FORM\0"
  "ARABIC LIGATURE KAF WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE KAF WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE KAF WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE KAF WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE KAF WITH MEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE KAF WITH MEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE KAF WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE KAF WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE KAF WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE KHAH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE KHAH WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE KHAH WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE KHAH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE KHAH WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE KHAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE KHAH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE KHAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE KHAH WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF WITH HAMZA ABOVE FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF WITH HAMZA ABOVE ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF WITH HAMZA BELOW FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF WITH HAMZA BELOW ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF WITH MADDA ABOVE FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH ALEF WITH MADDA ABOVE ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH HAH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH HAH WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH HAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH HEH INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH JEEM WITH JEEM FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH JEEM WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH JEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH JEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH JEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH KHAH WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH KHAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE LAM WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE LAM WITH MEEM WITH HAH FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH MEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE LAM WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE LAM WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE MEEM WITH ALEF FINAL FORM\0"
  "ARABIC LIGATURE MEEM WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE MEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE MEEM WITH HAH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH HAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE MEEM WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE MEEM WITH JEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH JEEM WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH JEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH JEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE MEEM WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE MEEM WITH KHAH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH KHAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH KHAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE MEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE MEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE MEEM WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE MEEM WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE MEEM WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE MOHAMMAD ISOLATED FORM\0"
  "ARABIC LIGATURE NOON WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE NOON WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE NOON WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE NOON WITH HAH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH HAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE NOON WITH HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH HEH INITIAL FORM\0"
  "ARABIC LIGATURE NOON WITH HEH MEDIAL FORM\0"
  "ARABIC LIGATURE NOON WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE NOON WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE NOON WITH JEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH JEEM WITH HAH FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH JEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE NOON WITH JEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH JEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE NOON WITH JEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE NOON WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE NOON WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE NOON WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE NOON WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE NOON WITH MEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH NOON FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH REH FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE NOON WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE NOON WITH ZAIN FINAL FORM\0"
  "ARABIC LIGATURE QAF WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE QAF WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE QAF WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE QAF WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE QAF WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE QAF WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE QAF WITH MEEM WITH HAH FINAL FORM\0"
  "ARABIC LIGATURE QAF WITH MEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE QAF WITH MEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE QAF WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE QAF WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE QAF WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE QALA USED AS KORANIC STOP SIGN ISOLATED FORM\0"
  "ARABIC LIGATURE QUDDISA SIRRAH\0"
  "ARABIC LIGATURE RADI ALLAAHU ANH\0"
  "ARABIC LIGATURE RADI ALLAAHU ANHAA\0"
  "ARABIC LIGATURE RADI ALLAAHU ANHUM\0"
  "ARABIC LIGATURE RADI ALLAAHU ANHUMAA\0"
  "ARABIC LIGATURE RADI ALLAAHU ANHUNNA\0"
  "ARABIC LIGATURE RAHIMAHU ALLAAH\0"
  "ARABIC LIGATURE RAHIMAHUM ALLAAH\0"
  "ARABIC LIGATURE RASOUL ISOLATED FORM\0"
  "ARABIC LIGATURE REH WITH SUPERSCRIPT ALEF ISOLATED FORM\0"
  "ARABIC LIGATURE SAD WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE SAD WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE SAD WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE SAD WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE SAD WITH HAH WITH HAH FINAL FORM\0"
  "ARABIC LIGATURE SAD WITH HAH WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE SAD WITH HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE SAD WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE SAD WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE SAD WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE SAD WITH MEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE SAD WITH MEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE SAD WITH REH FINAL FORM\0"
  "ARABIC LIGATURE SAD WITH REH ISOLATED FORM\0"
  "ARABIC LIGATURE SAD WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE SAD WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE SALAAMUHU ALAYNAA\0"
  "ARABIC LIGATURE SALAM ISOLATED FORM\0"
  "ARABIC LIGATURE SALLA ISOLATED FORM\0"
  "ARABIC LIGATURE SALLA USED AS KORANIC STOP SIGN ISOLATED FORM\0"
  "ARABIC LIGATURE SALLALLAAHU ALAYHI WA-AALIH\0"
  "ARABIC LIGATURE SALLALLAHOU ALAYHE WASALLAM\0"
  "ARABIC LIGATURE SALLALLAHU ALAYHI WAAALIHEE WA-SALLAM\0"
  "ARABIC LIGATURE SEEN WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE SEEN WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE SEEN WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE SEEN WITH HAH MEDIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH HAH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH HEH INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH HEH MEDIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE SEEN WITH JEEM MEDIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH JEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE SEEN WITH JEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE SEEN WITH KHAH MEDIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH KHAH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE SEEN WITH KHAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE SEEN WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE SEEN WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH MEEM WITH HAH FINAL FORM\0"
  "ARABIC LIGATURE SEEN WITH MEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH MEEM WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH MEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE SEEN WITH MEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE SEEN WITH REH FINAL FORM\0"
  "ARABIC LIGATURE SEEN WITH REH ISOLATED FORM\0"
  "ARABIC LIGATURE SEEN WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE SEEN WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE SHADDA WITH DAMMA ISOLATED FORM\0"
  "ARABIC LIGATURE SHADDA WITH DAMMA MEDIAL FORM\0"
  "ARABIC LIGATURE SHADDA WITH DAMMATAN ISOLATED FORM\0"
  "ARABIC LIGATURE SHADDA WITH FATHA ISOLATED FORM\0"
  "ARABIC LIGATURE SHADDA WITH FATHA MEDIAL FORM\0"
  "ARABIC LIGATURE SHADDA WITH KASRA ISOLATED FORM\0"
  "ARABIC LIGATURE SHADDA WITH KASRA MEDIAL FORM\0"
  "ARABIC LIGATURE SHADDA WITH KASRATAN ISOLATED FORM\0"
  "ARABIC LIGATURE SHADDA WITH SUPERSCRIPT ALEF ISOLATED FORM\0"
  "ARABIC LIGATURE SHEEN WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE SHEEN WITH HAH FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE SHEEN WITH HAH MEDIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH HAH WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH HAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH HEH INITIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH HEH MEDIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH JEEM FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE SHEEN WITH JEEM MEDIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH JEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH KHAH FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE SHEEN WITH KHAH MEDIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE SHEEN WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH REH FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH REH ISOLATED FORM\0"
  "ARABIC LIGATURE SHEEN WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE SHEEN WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE SUBHAANAHU WA TAAALAA\0"
  "ARABIC LIGATURE TABAARAKA WA-TAAALAA\0"
  "ARABIC LIGATURE TAH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE TAH WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE TAH WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE TAH WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE TAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE TAH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE TAH WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE TAH WITH MEEM WITH HAH FINAL FORM\0"
  "ARABIC LIGATURE TAH WITH MEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE TAH WITH MEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE TAH WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE TAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE TAH WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE TEH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE TEH WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE TEH WITH HAH WITH JEEM FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH HAH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH HAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH HEH INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH HEH MEDIAL FORM\0"
  "ARABIC LIGATURE TEH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE TEH WITH JEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH JEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH JEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE TEH WITH KHAH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH KHAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH KHAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE TEH WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE TEH WITH MEEM WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH MEEM WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH MEEM WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH MEEM WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE TEH WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH NOON FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH REH FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE TEH WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE TEH WITH ZAIN FINAL FORM\0"
  "ARABIC LIGATURE THAL WITH SUPERSCRIPT ALEF ISOLATED FORM\0"
  "ARABIC LIGATURE THEH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE THEH WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE THEH WITH HEH MEDIAL FORM\0"
  "ARABIC LIGATURE THEH WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE THEH WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE THEH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE THEH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE THEH WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE THEH WITH NOON FINAL FORM\0"
  "ARABIC LIGATURE THEH WITH REH FINAL FORM\0"
  "ARABIC LIGATURE THEH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE THEH WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE THEH WITH ZAIN FINAL FORM\0"
  "ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA INITIAL FORM\0"
  "ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE WASALLAM ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH MEDIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH NOON FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH REH FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ZAIN FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH HEH INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH HEH MEDIAL FORM\0"
  "ARABIC LIGATURE YEH WITH JEEM INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH JEEM ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH JEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH KHAH INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH KHAH ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH MEEM MEDIAL FORM\0"
  "ARABIC LIGATURE YEH WITH MEEM WITH MEEM FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH MEEM WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE YEH WITH MEEM WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH NOON FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH REH FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH YEH FINAL FORM\0"
  "ARABIC LIGATURE YEH WITH YEH ISOLATED FORM\0"
  "ARABIC LIGATURE YEH WITH ZAIN FINAL FORM\0"
  "ARABIC LIGATURE ZAH WITH MEEM INITIAL FORM\0"
  "ARABIC LIGATURE ZAH WITH MEEM ISOLATED FORM\0"
  "ARABIC LIGATURE ZAH WITH MEEM MEDIAL FORM\0"
  "ARABIC MADDA WAAJIB\0"
  "ARABIC MADDAH ABOVE\0"
  "ARABIC MARK NOON GHUNNA\0"
  "ARABIC MARK SIDEWAYS NOON GHUNNA\0"
  "ARABIC MATHEMATICAL AIN\0"
  "ARABIC MATHEMATICAL ALEF\0"
  "ARABIC MATHEMATICAL BEH\0"
  "ARABIC MATHEMATICAL DAD\0"
  "ARABIC MATHEMATICAL DAL\0"
  "ARABIC MATHEMATICAL DOTLESS BEH\0"
  "ARABIC MATHEMATICAL DOTLESS FEH\0"
  "ARABIC MATHEMATICAL DOTLESS NOON\0"
  "ARABIC MATHEMATICAL DOTLESS QAF\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK AIN\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK BEH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK DAD\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK DAL\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK FEH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK GHAIN\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK HAH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK JEEM\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK KHAH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK LAM\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK MEEM\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK NOON\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK QAF\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK REH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK SAD\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK SEEN\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK SHEEN\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK TAH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK TEH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK THAL\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK THEH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK WAW\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK YEH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK ZAH\0"
  "ARABIC MATHEMATICAL DOUBLE-STRUCK ZAIN\0"
  "ARABIC MATHEMATICAL FEH\0"
  "ARABIC MATHEMATICAL GHAIN\0"
  "ARABIC MATHEMATICAL HAH\0"
  "ARABIC MATHEMATICAL INITIAL AIN\0"
  "ARABIC MATHEMATICAL INITIAL BEH\0"
  "ARABIC MATHEMATICAL INITIAL DAD\0"
  "ARABIC MATHEMATICAL INITIAL FEH\0"
  "ARABIC MATHEMATICAL INITIAL GHAIN\0"
  "ARABIC MATHEMATICAL INITIAL HAH\0"
  "ARABIC MATHEMATICAL INITIAL HEH\0"
  "ARABIC MATHEMATICAL INITIAL JEEM\0"
  "ARABIC MATHEMATICAL INITIAL KAF\0"
  "ARABIC MATHEMATICAL INITIAL KHAH\0"
  "ARABIC MATHEMATICAL INITIAL LAM\0"
  "ARABIC MATHEMATICAL INITIAL MEEM\0"
  "ARABIC MATHEMATICAL INITIAL NOON\0"
  "ARABIC MATHEMATICAL INITIAL QAF\0"
  "ARABIC MATHEMATICAL INITIAL SAD\0"
  "ARABIC MATHEMATICAL INITIAL SEEN\0"
  "ARABIC MATHEMATICAL INITIAL SHEEN\0"
  "ARABIC MATHEMATICAL INITIAL TEH\0"
  "ARABIC MATHEMATICAL INITIAL THEH\0"
  "ARABIC MATHEMATICAL INITIAL YEH\0"
  "ARABIC MATHEMATICAL JEEM\0"
  "ARABIC MATHEMATICAL KAF\0"
  "ARABIC MATHEMATICAL KHAH\0"
  "ARABIC MATHEMATICAL LAM\0"
  "ARABIC MATHEMATICAL LOOPED AIN\0"
  "ARABIC MATHEMATICAL LOOPED ALEF\0"
  "ARABIC MATHEMATICAL LOOPED BEH\0"
  "ARABIC MATHEMATICAL LOOPED DAD\0"
  "ARABIC MATHEMATICAL LOOPED DAL\0"
  "ARABIC MATHEMATICAL LOOPED FEH\0"
  "ARABIC MATHEMATICAL LOOPED GHAIN\0"
  "ARABIC MATHEMATICAL LOOPED HAH\0"
  "ARABIC MATHEMATICAL LOOPED HEH\0"
  "ARABIC MATHEMATICAL LOOPED JEEM\0"
  "ARABIC MATHEMATICAL LOOPED KHAH\0"
  "ARABIC MATHEMATICAL LOOPED LAM\0"
  "ARABIC MATHEMATICAL LOOPED MEEM\0"
  "ARABIC MATHEMATICAL LOOPED NOON\0"
  "ARABIC MATHEMATICAL LOOPED QAF\0"
  "ARABIC MATHEMATICAL LOOPED REH\0"
  "ARABIC MATHEMATICAL LOOPED SAD\0"
  "ARABIC MATHEMATICAL LOOPED SEEN\0"
  "ARABIC MATHEMATICAL LOOPED SHEEN\0"
  "ARABIC MATHEMATICAL LOOPED TAH\0"
  "ARABIC MATHEMATICAL LOOPED TEH\0"
  "ARABIC MATHEMATICAL LOOPED THAL\0"
  "ARABIC MATHEMATICAL LOOPED THEH\0"
  "ARABIC MATHEMATICAL LOOPED WAW\0"
  "ARABIC MATHEMATICAL LOOPED YEH\0"
  "ARABIC MATHEMATICAL LOOPED ZAH\0"
  "ARABIC MATHEMATICAL LOOPED ZAIN\0"
  "ARABIC MATHEMATICAL MEEM\0"
  "ARABIC MATHEMATICAL NOON\0"
  "ARABIC MATHEMATICAL OPERATOR HAH WITH DAL\0"
  "ARABIC MATHEMATICAL OPERATOR MEEM WITH HAH WITH TATWEEL\0"
  "ARABIC MATHEMATICAL QAF\0"
  "ARABIC MATHEMATICAL REH\0"
  "ARABIC MATHEMATICAL SAD\0"
  "ARABIC MATHEMATICAL SEEN\0"
  "ARABIC MATHEMATICAL SHEEN\0"
  "ARABIC MATHEMATICAL STRETCHED AIN\0"
  "ARABIC MATHEMATICAL STRETCHED BEH\0"
  "ARABIC MATHEMATICAL STRETCHED DAD\0"
  "ARABIC MATHEMATICAL STRETCHED DOTLESS BEH\0"
  "ARABIC MATHEMATICAL STRETCHED DOTLESS FEH\0"
  "ARABIC MATHEMATICAL STRETCHED FEH\0"
  "ARABIC MATHEMATICAL STRETCHED GHAIN\0"
  "ARABIC MATHEMATICAL STRETCHED HAH\0"
  "ARABIC MATHEMATICAL STRETCHED HEH\0"
  "ARABIC MATHEMATICAL STRETCHED JEEM\0"
  "ARABIC MATHEMATICAL STRETCHED KAF\0"
  "ARABIC MATHEMATICAL STRETCHED KHAH\0"
  "ARABIC MATHEMATICAL STRETCHED MEEM\0"
  "ARABIC MATHEMATICAL STRETCHED NOON\0"
  "ARABIC MATHEMATICAL STRETCHED QAF\0"
  "ARABIC MATHEMATICAL STRETCHED SAD\0"
  "ARABIC MATHEMATICAL STRETCHED SEEN\0"
  "ARABIC MATHEMATICAL STRETCHED SHEEN\0"
  "ARABIC MATHEMATICAL STRETCHED TAH\0"
  "ARABIC MATHEMATICAL STRETCHED TEH\0"
  "ARABIC MATHEMATICAL STRETCHED THEH\0"
  "ARABIC MATHEMATICAL STRETCHED YEH\0"
  "ARABIC MATHEMATICAL STRETCHED ZAH\0"
  "ARABIC MATHEMATICAL TAH\0"
  "ARABIC MATHEMATICAL TAILED AIN\0"
  "ARABIC MATHEMATICAL TAILED DAD\0"
  "ARABIC MATHEMATICAL TAILED DOTLESS NOON\0"
  "ARABIC MATHEMATICAL TAILED DOTLESS QAF\0"
  "ARABIC MATHEMATICAL TAILED GHAIN\0"
  "ARABIC MATHEMATICAL TAILED HAH\0"
  "ARABIC MATHEMATICAL TAILED JEEM\0"
  "ARABIC MATHEMATICAL TAILED KHAH\0"
  "ARABIC MATHEMATICAL TAILED LAM\0"
  "ARABIC MATHEMATICAL TAILED NOON\0"
  "ARABIC MATHEMATICAL TAILED QAF\0"
  "ARABIC MATHEMATICAL TAILED SAD\0"
  "ARABIC MATHEMATICAL TAILED SEEN\0"
  "ARABIC MATHEMATICAL TAILED SHEEN\0"
  "ARABIC MATHEMATICAL TAILED YEH\0"
  "ARABIC MATHEMATICAL TEH\0"
  "ARABIC MATHEMATICAL THAL\0"
  "ARABIC MATHEMATICAL THEH\0"
  "ARABIC MATHEMATICAL WAW\0"
  "ARABIC MATHEMATICAL YEH\0"
  "ARABIC MATHEMATICAL ZAH\0"
  "ARABIC MATHEMATICAL ZAIN\0"
  "ARABIC NUMBER MARK ABOVE\0"
  "ARABIC NUMBER SIGN\0"
  "ARABIC OPEN DAMMATAN\0"
  "ARABIC OPEN FATHATAN\0"
  "ARABIC OPEN KASRATAN\0"
  "ARABIC PEPET\0"
  "ARABIC PERCENT SIGN\0"
  "ARABIC PIASTRE MARK ABOVE\0"
  "ARABIC PLACE OF SAJDAH\0"
  "ARABIC POETIC VERSE SIGN\0"
  "ARABIC POUND MARK ABOVE\0"
  "ARABIC QUESTION MARK\0"
  "ARABIC RAISED ROUND DOT\0"
  "ARABIC RAY\0"
  "ARABIC REVERSED DAMMA\0"
  "ARABIC RIGHT ARROWHEAD ABOVE\0"
  "ARABIC RIGHT ARROWHEAD ABOVE WITH DOT\0"
  "ARABIC RIGHT ARROWHEAD BELOW\0"
  "ARABIC ROUNDED HIGH STOP WITH FILLED CENTRE\0"
  "ARABIC SEMICOLON\0"
  "ARABIC SHADDA\0"
  "ARABIC SHADDA ISOLATED FORM\0"
  "ARABIC SHADDA MEDIAL FORM\0"
  "ARABIC SIGN ALAYHE ASSALLAM\0"
  "ARABIC SIGN MISRA\0"
  "ARABIC SIGN RADI ALLAHOU ANHU\0"
  "ARABIC SIGN RAHMATULLAH ALAYHE\0"
  "ARABIC SIGN SAFHA\0"
  "ARABIC SIGN SALLALLAHOU ALAYHE WASSALLAM\0"
  "ARABIC SIGN SAMVAT\0"
  "ARABIC SIGN SANAH\0"
  "ARABIC SIGN SINDHI AMPERSAND\0"
  "ARABIC SIGN SINDHI POSTPOSITION MEN\0"
  "ARABIC SIGN TAKHALLUS\0"
  "ARABIC SMALL DAMMA\0"
  "ARABIC SMALL FARSI YEH\0"
  "ARABIC SMALL FATHA\0"
  "ARABIC SMALL HIGH AIN\0"
  "ARABIC SMALL HIGH DOTLESS HEAD OF KHAH\0"
  "ARABIC SMALL HIGH FARSI YEH\0"
  "ARABIC SMALL HIGH FOOTNOTE MARKER\0"
  "ARABIC SMALL HIGH JEEM\0"
  "ARABIC SMALL HIGH LAM ALEF\0"
  "ARABIC SMALL HIGH LIGATURE ALEF WITH LAM WITH YEH\0"
  "ARABIC SMALL HIGH LIGATURE QAF WITH LAM WITH ALEF MAKSURA\0"
  "ARABIC SMALL HIGH LIGATURE SAD WITH LAM WITH ALEF MAKSURA\0"
  "ARABIC SMALL HIGH MADDA\0"
  "ARABIC SMALL HIGH MEEM INITIAL FORM\0"
  "ARABIC SMALL HIGH MEEM ISOLATED FORM\0"
  "ARABIC SMALL HIGH NOON\0"
  "ARABIC SMALL HIGH NOON WITH KASRA\0"
  "ARABIC SMALL HIGH QAF\0"
  "ARABIC SMALL HIGH ROUNDED ZERO\0"
  "ARABIC SMALL HIGH SAD\0"
  "ARABIC SMALL HIGH SEEN\0"
  "ARABIC SMALL HIGH SIGN SAFHA\0"
  "ARABIC SMALL HIGH TAH\0"
  "ARABIC SMALL HIGH THREE DOTS\0"
  "ARABIC SMALL HIGH UPRIGHT RECTANGULAR ZERO\0"
  "ARABIC SMALL HIGH WAW\0"
  "ARABIC SMALL HIGH WORD AL-JUZ\0"
  "ARABIC SMALL HIGH WORD AN-NISF\0"
  "ARABIC SMALL HIGH WORD AR-RUB\0"
  "ARABIC SMALL HIGH WORD AS-SAJDA\0"
  "ARABIC SMALL HIGH WORD ATH-THALATHA\0"
  "ARABIC SMALL HIGH WORD QIF\0"
  "ARABIC SMALL HIGH WORD SAH\0"
  "ARABIC SMALL HIGH WORD SAKTA\0"
  "ARABIC SMALL HIGH WORD WAQFA\0"
  "ARABIC SMALL HIGH YEH\0"
  "ARABIC SMALL HIGH YEH BARREE WITH TWO DOTS BELOW\0"
  "ARABIC SMALL HIGH ZAH\0"
  "ARABIC SMALL HIGH ZAIN\0"
  "ARABIC SMALL KASRA\0"
  "ARABIC SMALL LOW MEEM\0"
  "ARABIC SMALL LOW NOON WITH KASRA\0"
  "ARABIC SMALL LOW SEEN\0"
  "ARABIC SMALL LOW WAW\0"
  "ARABIC SMALL LOW WORD IMAALA\0"
  "ARABIC SMALL LOW WORD ISHMAAM\0"
  "ARABIC SMALL LOW WORD MADDA\0"
  "ARABIC SMALL LOW WORD QASR\0"
  "ARABIC SMALL LOW WORD SAKTA\0"
  "ARABIC SMALL LOW WORD TASHEEL\0"
  "ARABIC SMALL WAW\0"
  "ARABIC SMALL YEH\0"
  "ARABIC START OF RUB EL HIZB\0"
  "ARABIC SUBSCRIPT ALEF\0"
  "ARABIC SUKUN\0"
  "ARABIC SUKUN BELOW\0"
  "ARABIC SUKUN ISOLATED FORM\0"
  "ARABIC SUKUN MEDIAL FORM\0"
  "ARABIC SUPERSCRIPT ALEF MOKHASSAS\0"
  "ARABIC SYMBOL DOT ABOVE\0"
  "ARABIC SYMBOL DOT BELOW\0"
  "ARABIC SYMBOL DOUBLE VERTICAL BAR BELOW\0"
  "ARABIC SYMBOL FOUR DOTS ABOVE\0"
  "ARABIC SYMBOL FOUR DOTS BELOW\0"
  "ARABIC SYMBOL RING\0"
  "ARABIC SYMBOL SMALL TAH ABOVE\0"
  "ARABIC SYMBOL SMALL TAH BELOW\0"
  "ARABIC SYMBOL THREE DOTS ABOVE\0"
  "ARABIC SYMBOL THREE DOTS BELOW\0"
  "ARABIC SYMBOL THREE DOTS POINTING DOWNWARDS ABOVE\0"
  "ARABIC SYMBOL THREE DOTS POINTING DOWNWARDS BELOW\0"
  "ARABIC SYMBOL TWO DOTS ABOVE\0"
  "ARABIC SYMBOL TWO DOTS BELOW\0"
  "ARABIC SYMBOL TWO DOTS VERTICALLY ABOVE\0"
  "ARABIC SYMBOL TWO DOTS VERTICALLY BELOW\0"
  "ARABIC SYMBOL WASLA ABOVE\0"
  "ARABIC TAIL FRAGMENT\0"
  "ARABIC TATWEEL\0"
  "ARABIC TATWEEL WITH FATHATAN ABOVE\0"
  "ARABIC TATWEEL WITH OVERSTRUCK HAMZA\0"
  "ARABIC TATWEEL WITH OVERSTRUCK WAW\0"
  "ARABIC TATWEEL WITH TWO DOTS BELOW\0"
  "ARABIC THOUSANDS SEPARATOR\0"
  "ARABIC TONE LOOP ABOVE\0"
  "ARABIC TONE LOOP BELOW\0"
  "ARABIC TONE ONE DOT ABOVE\0"
  "ARABIC TONE ONE DOT BELOW\0"
  "ARABIC TONE TWO DOTS ABOVE\0"
  "ARABIC TONE TWO DOTS BELOW\0"
  "ARABIC TRIPLE DOT PUNCTUATION MARK\0"
  "ARABIC TURNED DAMMA BELOW\0"
  "ARABIC VERTICAL TAIL\0"
  "ARABIC VOWEL SIGN DOT BELOW\0"
  "ARABIC VOWEL SIGN INVERTED SMALL V ABOVE\0"
  "ARABIC VOWEL SIGN SMALL V ABOVE\0"
  "ARABIC WAVY HAMZA BELOW\0"
  "ARABIC ZWARAKAY\0"
  "ARABIC-INDIC CUBE ROOT\0"
  "ARABIC-INDIC DIGIT EIGHT\0"
  "ARABIC-INDIC DIGIT FIVE\0"
  "ARABIC-INDIC DIGIT FOUR\0"
  "ARABIC-INDIC DIGIT NINE\0"
  "ARABIC-INDIC DIGIT ONE\0"
  "ARABIC-INDIC DIGIT SEVEN\0"
  "ARABIC-INDIC DIGIT SIX\0"
  "ARABIC-INDIC DIGIT THREE\0"
  "ARABIC-INDIC DIGIT TWO\0"
  "ARABIC-INDIC DIGIT ZERO\0"
  "ARABIC-INDIC FOURTH ROOT\0"
  "ARABIC-INDIC PER MILLE SIGN\0"
  "ARABIC-INDIC PER TEN THOUSAND SIGN\0"
  "ARC\0"
  "ARIES\0"
  "ARMENIAN ABBREVIATION MARK\0"
  "ARMENIAN APOSTROPHE\0"
  "ARMENIAN CAPITAL LETTER AYB\0"
  "ARMENIAN CAPITAL LETTER BEN\0"
  "ARMENIAN CAPITAL LETTER CA\0"
  "ARMENIAN CAPITAL LETTER CHA\0"
  "ARMENIAN CAPITAL LETTER CHEH\0"
  "ARMENIAN CAPITAL LETTER CO\0"
  "ARMENIAN CAPITAL LETTER DA\0"
  "ARMENIAN CAPITAL LETTER ECH\0"
  "ARMENIAN CAPITAL LETTER EH\0"
  "ARMENIAN CAPITAL LETTER ET\0"
  "ARMENIAN CAPITAL LETTER FEH\0"
  "ARMENIAN CAPITAL LETTER GHAD\0"
  "ARMENIAN CAPITAL LETTER GIM\0"
  "ARMENIAN CAPITAL LETTER HO\0"
  "ARMENIAN CAPITAL LETTER INI\0"
  "ARMENIAN CAPITAL LETTER JA\0"
  "ARMENIAN CAPITAL LETTER JHEH\0"
  "ARMENIAN CAPITAL LETTER KEH\0"
  "ARMENIAN CAPITAL LETTER KEN\0"
  "ARMENIAN CAPITAL LETTER LIWN\0"
  "ARMENIAN CAPITAL LETTER MEN\0"
  "ARMENIAN CAPITAL LETTER NOW\0"
  "ARMENIAN CAPITAL LETTER OH\0"
  "ARMENIAN CAPITAL LETTER PEH\0"
  "ARMENIAN CAPITAL LETTER PIWR\0"
  "ARMENIAN CAPITAL LETTER RA\0"
  "ARMENIAN CAPITAL LETTER REH\0"
  "ARMENIAN CAPITAL LETTER SEH\0"
  "ARMENIAN CAPITAL LETTER SHA\0"
  "ARMENIAN CAPITAL LETTER TIWN\0"
  "ARMENIAN CAPITAL LETTER TO\0"
  "ARMENIAN CAPITAL LETTER VEW\0"
  "ARMENIAN CAPITAL LETTER VO\0"
  "ARMENIAN CAPITAL LETTER XEH\0"
  "ARMENIAN CAPITAL LETTER YI\0"
  "ARMENIAN CAPITAL LETTER YIWN\0"
  "ARMENIAN CAPITAL LETTER ZA\0"
  "ARMENIAN CAPITAL LETTER ZHE\0"
  "ARMENIAN COMMA\0"
  "ARMENIAN DRAM SIGN\0"
  "ARMENIAN EMPHASIS MARK\0"
  "ARMENIAN EXCLAMATION MARK\0"
  "ARMENIAN FULL STOP\0"
  "ARMENIAN HYPHEN\0"
  "ARMENIAN MODIFIER LETTER LEFT HALF RING\0"
  "ARMENIAN QUESTION MARK\0"
  "ARMENIAN SMALL LETTER AYB\0"
  "ARMENIAN SMALL LETTER BEN\0"
  "ARMENIAN SMALL LETTER CA\0"
  "ARMENIAN SMALL LETTER CHA\0"
  "ARMENIAN SMALL LETTER CHEH\0"
  "ARMENIAN SMALL LETTER CO\0"
  "ARMENIAN SMALL LETTER DA\0"
  "ARMENIAN SMALL LETTER ECH\0"
  "ARMENIAN SMALL LETTER EH\0"
  "ARMENIAN SMALL LETTER ET\0"
  "ARMENIAN SMALL LETTER FEH\0"
  "ARMENIAN SMALL LETTER GHAD\0"
  "ARMENIAN SMALL LETTER GIM\0"
  "ARMENIAN SMALL LETTER HO\0"
  "ARMENIAN SMALL LETTER INI\0"
  "ARMENIAN SMALL LETTER JA\0"
  "ARMENIAN SMALL LETTER JHEH\0"
  "ARMENIAN SMALL LETTER KEH\0"
  "ARMENIAN SMALL LETTER KEN\0"
  "ARMENIAN SMALL LETTER LIWN\0"
  "ARMENIAN SMALL LETTER MEN\0"
  "ARMENIAN SMALL LETTER NOW\0"
  "ARMENIAN SMALL LETTER OH\0"
  "ARMENIAN SMALL LETTER PEH\0"
  "ARMENIAN SMALL LETTER PIWR\0"
  "ARMENIAN SMALL LETTER RA\0"
  "ARMENIAN SMALL LETTER REH\0"
  "ARMENIAN SMALL LETTER SEH\0"
  "ARMENIAN SMALL LETTER SHA\0"
  "ARMENIAN SMALL LETTER TIWN\0"
  "ARMENIAN SMALL LETTER TO\0"
  "ARMENIAN SMALL LETTER TURNED AYB\0"
  "ARMENIAN SMALL LETTER VEW\0"
  "ARMENIAN SMALL LETTER VO\0"
  "ARMENIAN SMALL LETTER XEH\0"
  "ARMENIAN SMALL LETTER YI\0"
  "ARMENIAN SMALL LETTER YI WITH STROKE\0"
  "ARMENIAN SMALL LETTER YIWN\0"
  "ARMENIAN SMALL LETTER ZA\0"
  "ARMENIAN SMALL LETTER ZHE\0"
  "ARMENIAN SMALL LIGATURE ECH YIWN\0"
  "ARMENIAN SMALL LIGATURE MEN ECH\0"
  "ARMENIAN SMALL LIGATURE MEN INI\0"
  "ARMENIAN SMALL LIGATURE MEN NOW\0"
  "ARMENIAN SMALL LIGATURE MEN XEH\0"
  "ARMENIAN SMALL LIGATURE VEW NOW\0"
  "ARROW POINTING DOWNWARDS THEN CURVING LEFTWARDS\0"
  "ARROW POINTING DOWNWARDS THEN CURVING RIGHTWARDS\0"
  "ARROW POINTING RIGHTWARDS THEN CURVING DOWNWARDS\0"
  "ARROW POINTING RIGHTWARDS THEN CURVING SOUTH WEST\0"
  "ARROW POINTING RIGHTWARDS THEN CURVING UPWARDS\0"
  "ARROW POINTING UPWARDS THEN NORTH WEST\0"
  "ARROWHEAD-SHAPED POINTER\0"
  "ARTICULATED LORRY\0"
  "ARTIST PALETTE\0"
  "ASCENDING NODE\0"
  "ASCIA SYMBOL\0"
  "ASSERTION\0"
  "ASTERISK\0"
  "ASTERISK OPERATOR\0"
  "ASTERISM\0"
  "ASTONISHED FACE\0"
  "ASTRAEA\0"
  "ASTRONOMICAL SYMBOL FOR URANUS\0"
  "ASYMPTOTICALLY EQUAL TO\0"
  "ATHLETIC SHOE\0"
  "ATOM SYMBOL\0"
  "AUBERGINE\0"
  "AUSTRAL SIGN\0"
  "AUTO RICKSHAW\0"
  "AUTOMATED TELLER MACHINE\0"
  "AUTOMOBILE\0"
  "AVESTAN ABBREVIATION MARK\0"
  "AVESTAN LETTER A\0"
  "AVESTAN LETTER AA\0"
  "AVESTAN LETTER AAN\0"
  "AVESTAN LETTER AAO\0"
  "AVESTAN LETTER AE\0"
  "AVESTAN LETTER AEE\0"
  "AVESTAN LETTER AN\0"
  "AVESTAN LETTER AO\0"
  "AVESTAN LETTER BE\0"
  "AVESTAN LETTER BHE\0"
  "AVESTAN LETTER CE\0"
  "AVESTAN LETTER DE\0"
  "AVESTAN LETTER DHE\0"
  "AVESTAN LETTER E\0"
  "AVESTAN LETTER EE\0"
  "AVESTAN LETTER FE\0"
  "AVESTAN LETTER GE\0"
  "AVESTAN LETTER GGE\0"
  "AVESTAN LETTER GHE\0"
  "AVESTAN LETTER HE\0"
  "AVESTAN LETTER HME\0"
  "AVESTAN LETTER I\0"
  "AVESTAN LETTER II\0"
  "AVESTAN LETTER JE\0"
  "AVESTAN LETTER KE\0"
  "AVESTAN LETTER LE\0"
  "AVESTAN LETTER ME\0"
  "AVESTAN LETTER NE\0"
  "AVESTAN LETTER NGE\0"
  "AVESTAN LETTER NGVE\0"
  "AVESTAN LETTER NGYE\0"
  "AVESTAN LETTER NNE\0"
  "AVESTAN LETTER NYE\0"
  "AVESTAN LETTER O\0"
  "AVESTAN LETTER OO\0"
  "AVESTAN LETTER PE\0"
  "AVESTAN LETTER RE\0"
  "AVESTAN LETTER SE\0"
  "AVESTAN LETTER SHE\0"
  "AVESTAN LETTER SHYE\0"
  "AVESTAN LETTER SSHE\0"
  "AVESTAN LETTER TE\0"
  "AVESTAN LETTER THE\0"
  "AVESTAN LETTER TTE\0"
  "AVESTAN LETTER U\0"
  "AVESTAN LETTER UU\0"
  "AVESTAN LETTER VE\0"
  "AVESTAN LETTER XE\0"
  "AVESTAN LETTER XVE\0"
  "AVESTAN LETTER XYE\0"
  "AVESTAN LETTER YE\0"
  "AVESTAN LETTER YYE\0"
  "AVESTAN LETTER ZE\0"
  "AVESTAN LETTER ZHE\0"
  "AVOCADO\0"
  "AXE\0"
  "BABY\0"
  "BABY ANGEL\0"
  "BABY BOTTLE\0"
  "BABY CHICK\0"
  "BABY SYMBOL\0"
  "BACK OF ENVELOPE\0"
  "BACK WITH LEFTWARDS ARROW ABOVE\0"
  "BACK-TILTED SHADOWED WHITE RIGHTWARDS ARROW\0"
  "BACKSLANTED SOUTH ARROW WITH HOOKED TAIL\0"
  "BACKSLANTED SOUTH ARROW WITH HORIZONTAL TAIL\0"
  "BACON\0"
  "BACTRIAN CAMEL\0"
  "BADGER\0"
  "BADMINTON RACQUET AND SHUTTLECOCK\0"
  "BAGEL\0"
  "BAGGAGE CLAIM\0"
  "BAGUETTE BREAD\0"
  "BALINESE ADEG ADEG\0"
  "BALINESE CARIK PAMUNGKAH\0"
  "BALINESE CARIK PAREREN\0"
  "BALINESE CARIK SIKI\0"
  "BALINESE DIGIT EIGHT\0"
  "BALINESE DIGIT FIVE\0"
  "BALINESE DIGIT FOUR\0"
  "BALINESE DIGIT NINE\0"
  "BALINESE DIGIT ONE\0"
  "BALINESE DIGIT SEVEN\0"
  "BALINESE DIGIT SIX\0"
  "BALINESE DIGIT THREE\0"
  "BALINESE DIGIT TWO\0"
  "BALINESE DIGIT ZERO\0"
  "BALINESE INVERTED CARIK PAREREN\0"
  "BALINESE INVERTED CARIK SIKI\0"
  "BALINESE LETTER AIKARA\0"
  "BALINESE LETTER AKARA\0"
  "BALINESE LETTER AKARA TEDUNG\0"
  "BALINESE LETTER ARCHAIC JNYA\0"
  "BALINESE LETTER ASYURA SASAK\0"
  "BALINESE LETTER BA\0"
  "BALINESE LETTER BA KEMBANG\0"
  "BALINESE LETTER CA\0"
  "BALINESE LETTER CA LACA\0"
  "BALINESE LETTER DA\0"
  "BALINESE LETTER DA MADU\0"
  "BALINESE LETTER DA MURDA ALPAPRANA\0"
  "BALINESE LETTER DA MURDA MAHAPRANA\0"
  "BALINESE LETTER EF SASAK\0"
  "BALINESE LETTER EKARA\0"
  "BALINESE LETTER GA\0"
  "BALINESE LETTER GA GORA\0"
  "BALINESE LETTER HA\0"
  "BALINESE LETTER IKARA\0"
  "BALINESE LETTER IKARA TEDUNG\0"
  "BALINESE LETTER JA\0"
  "BALINESE LETTER JA JERA\0"
  "BALINESE LETTER KA\0"
  "BALINESE LETTER KA MAHAPRANA\0"
  "BALINESE LETTER KAF SASAK\0"
  "BALINESE LETTER KHOT SASAK\0"
  "BALINESE LETTER LA\0"
  "BALINESE LETTER LA LENGA\0"
  "BALINESE LETTER LA LENGA TEDUNG\0"
  "BALINESE LETTER MA\0"
  "BALINESE LETTER NA\0"
  "BALINESE LETTER NA RAMBAT\0"
  "BALINESE LETTER NGA\0"
  "BALINESE LETTER NYA\0"
  "BALINESE LETTER OKARA\0"
  "BALINESE LETTER OKARA TEDUNG\0"
  "BALINESE LETTER PA\0"
  "BALINESE LETTER PA KAPAL\0"
  "BALINESE LETTER RA\0"
  "BALINESE LETTER RA REPA\0"
  "BALINESE LETTER RA REPA TEDUNG\0"
  "BALINESE LETTER SA\0"
  "BALINESE LETTER SA SAGA\0"
  "BALINESE LETTER SA SAPA\0"
  "BALINESE LETTER TA\0"
  "BALINESE LETTER TA LATIK\0"
  "BALINESE LETTER TA MURDA MAHAPRANA\0"
  "BALINESE LETTER TA TAWA\0"
  "BALINESE LETTER TZIR SASAK\0"
  "BALINESE LETTER UKARA\0"
  "BALINESE LETTER UKARA TEDUNG\0"
  "BALINESE LETTER VE SASAK\0"
  "BALINESE LETTER WA\0"
  "BALINESE LETTER YA\0"
  "BALINESE LETTER ZAL SASAK\0"
  "BALINESE MUSICAL SYMBOL COMBINING BENDE\0"
  "BALINESE MUSICAL SYMBOL COMBINING ENDEP\0"
  "BALINESE MUSICAL SYMBOL COMBINING GONG\0"
  "BALINESE MUSICAL SYMBOL COMBINING JEGOGAN\0"
  "BALINESE MUSICAL SYMBOL COMBINING KEMPLI\0"
  "BALINESE MUSICAL SYMBOL COMBINING KEMPLI WITH JEGOGAN\0"
  "BALINESE MUSICAL SYMBOL COMBINING KEMPUL\0"
  "BALINESE MUSICAL SYMBOL COMBINING KEMPUL WITH JEGOGAN\0"
  "BALINESE MUSICAL SYMBOL COMBINING TEGEH\0"
  "BALINESE MUSICAL SYMBOL DAENG\0"
  "BALINESE MUSICAL SYMBOL DAING\0"
  "BALINESE MUSICAL SYMBOL DANG\0"
  "BALINESE MUSICAL SYMBOL DANG GEDE\0"
  "BALINESE MUSICAL SYMBOL DANG SURANG\0"
  "BALINESE MUSICAL SYMBOL DENG\0"
  "BALINESE MUSICAL SYMBOL DEUNG\0"
  "BALINESE MUSICAL SYMBOL DING\0"
  "BALINESE MUSICAL SYMBOL DONG\0"
  "BALINESE MUSICAL SYMBOL DUNG\0"
  "BALINESE MUSICAL SYMBOL LEFT-HAND CLOSED PLAK\0"
  "BALINESE MUSICAL SYMBOL LEFT-HAND CLOSED PLUK\0"
  "BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PANG\0"
  "BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PING\0"
  "BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PUNG\0"
  "BALINESE MUSICAL SYMBOL RIGHT-HAND CLOSED TAK\0"
  "BALINESE MUSICAL SYMBOL RIGHT-HAND CLOSED TUK\0"
  "BALINESE MUSICAL SYMBOL RIGHT-HAND OPEN DAG\0"
  "BALINESE MUSICAL SYMBOL RIGHT-HAND OPEN DUG\0"
  "BALINESE PAMADA\0"
  "BALINESE PAMADA LANTANG\0"
  "BALINESE PAMENENG\0"
  "BALINESE PANTI\0"
  "BALINESE PANTI BAWAK\0"
  "BALINESE PANTI LANTANG\0"
  "BALINESE SIGN BISAH\0"
  "BALINESE SIGN CECEK\0"
  "BALINESE SIGN REREKAN\0"
  "BALINESE SIGN SURANG\0"
  "BALINESE SIGN ULU CANDRA\0"
  "BALINESE SIGN ULU RICEM\0"
  "BALINESE VOWEL SIGN LA LENGA\0"
  "BALINESE VOWEL SIGN LA LENGA TEDUNG\0"
  "BALINESE VOWEL SIGN PEPET\0"
  "BALINESE VOWEL SIGN PEPET TEDUNG\0"
  "BALINESE VOWEL SIGN RA REPA\0"
  "BALINESE VOWEL SIGN RA REPA TEDUNG\0"
  "BALINESE VOWEL SIGN SUKU\0"
  "BALINESE VOWEL SIGN SUKU ILUT\0"
  "BALINESE VOWEL SIGN TALING\0"
  "BALINESE VOWEL SIGN TALING REPA\0"
  "BALINESE VOWEL SIGN TALING REPA TEDUNG\0"
  "BALINESE VOWEL SIGN TALING TEDUNG\0"
  "BALINESE VOWEL SIGN TEDUNG\0"
  "BALINESE VOWEL SIGN ULU\0"
  "BALINESE VOWEL SIGN ULU SARI\0"
  "BALINESE WINDU\0"
  "BALL OF YARN\0"
  "BALLET SHOES\0"
  "BALLOON\0"
  "BALLOON-SPOKED ASTERISK\0"
  "BALLOT BOLD SCRIPT X\0"
  "BALLOT BOX\0"
  "BALLOT BOX WITH BALLOT\0"
  "BALLOT BOX WITH BOLD CHECK\0"
  "BALLOT BOX WITH BOLD SCRIPT X\0"
  "BALLOT BOX WITH CHECK\0"
  "BALLOT BOX WITH LIGHT X\0"
  "BALLOT BOX WITH SCRIPT X\0"
  "BALLOT BOX WITH X\0"
  "BALLOT SCRIPT X\0"
  "BALLOT X\0"
  "BAMUM COLON\0"
  "BAMUM COMBINING MARK KOQNDON\0"
  "BAMUM COMBINING MARK TUKWENTIS\0"
  "BAMUM COMMA\0"
  "BAMUM FULL STOP\0"
  "BAMUM LETTER A\0"
  "BAMUM LETTER EE\0"
  "BAMUM LETTER FAAMAE\0"
  "BAMUM LETTER FEE\0"
  "BAMUM LETTER FOM\0"
  "BAMUM LETTER FU\0"
  "BAMUM LETTER I\0"
  "BAMUM LETTER KA\0"
  "BAMUM LETTER KEN\0"
  "BAMUM LETTER KET\0"
  "BAMUM LETTER KEUX\0"
  "BAMUM LETTER KI\0"
  "BAMUM LETTER KO\0"
  "BAMUM LETTER KOGHOM\0"
  "BAMUM LETTER KOVUU\0"
  "BAMUM LETTER KPA\0"
  "BAMUM LETTER KU\0"
  "BAMUM LETTER KYEE\0"
  "BAMUM LETTER LA\0"
  "BAMUM LETTER LEEEE\0"
  "BAMUM LETTER LI\0"
  "BAMUM LETTER LOQ\0"
  "BAMUM LETTER LU\0"
  "BAMUM LETTER M\0"
  "BAMUM LETTER MA\0"
  "BAMUM LETTER MBAA\0"
  "BAMUM LETTER MBEN\0"
  "BAMUM LETTER MEEEE\0"
  "BAMUM LETTER MEN\0"
  "BAMUM LETTER MI\0"
  "BAMUM LETTER MO\0"
  "BAMUM LETTER MU\0"
  "BAMUM LETTER NA\0"
  "BAMUM LETTER NDAA\0"
  "BAMUM LETTER NGA\0"
  "BAMUM LETTER NGGA\0"
  "BAMUM LETTER NGKWAEN\0"
  "BAMUM LETTER NI\0"
  "BAMUM LETTER NJAEM\0"
  "BAMUM LETTER NJEE\0"
  "BAMUM LETTER NJUAE\0"
  "BAMUM LETTER NSHA\0"
  "BAMUM LETTER NTEE\0"
  "BAMUM LETTER NTUU\0"
  "BAMUM LETTER NU\0"
  "BAMUM LETTER NUAE\0"
  "BAMUM LETTER NYI\0"
  "BAMUM LETTER O\0"
  "BAMUM LETTER PA\0"
  "BAMUM LETTER PEE\0"
  "BAMUM LETTER PEUX\0"
  "BAMUM LETTER PHASE-A FIRI\0"
  "BAMUM LETTER PHASE-A GBIEE FON\0"
  "BAMUM LETTER PHASE-A GHEUAEGHEUAE\0"
  "BAMUM LETTER PHASE-A GHEUAERAE\0"
  "BAMUM LETTER PHASE-A KAFA\0"
  "BAMUM LETTER PHASE-A KAQ\0"
  "BAMUM LETTER PHASE-A KET\0"
  "BAMUM LETTER PHASE-A KEUKEUTNDA\0"
  "BAMUM LETTER PHASE-A KPOQ\0"
  "BAMUM LETTER PHASE-A KUOQ\0"
  "BAMUM LETTER PHASE-A LAPAQ\0"
  "BAMUM LETTER PHASE-A LET KUT\0"
  "BAMUM LETTER PHASE-A LOMMAE\0"
  "BAMUM LETTER PHASE-A LU\0"
  "BAMUM LETTER PHASE-A LUAEP\0"
  "BAMUM LETTER PHASE-A MAEKEUP\0"
  "BAMUM LETTER PHASE-A MAEM\0"
  "BAMUM LETTER PHASE-A MAEMBGBIEE\0"
  "BAMUM LETTER PHASE-A MAEMKPEN\0"
  "BAMUM LETTER PHASE-A MAEMVEUX\0"
  "BAMUM LETTER PHASE-A MAENYI\0"
  "BAMUM LETTER PHASE-A MAESI\0"
  "BAMUM LETTER PHASE-A MANSUAE\0"
  "BAMUM LETTER PHASE-A MAP PIEET\0"
  "BAMUM LETTER PHASE-A MBANYI\0"
  "BAMUM LETTER PHASE-A MBAQ\0"
  "BAMUM LETTER PHASE-A MEUNJOMNDEUQ\0"
  "BAMUM LETTER PHASE-A MGBASA\0"
  "BAMUM LETTER PHASE-A MON NGGEUAET\0"
  "BAMUM LETTER PHASE-A MOOMEUT\0"
  "BAMUM LETTER PHASE-A MOOMPUQ\0"
  "BAMUM LETTER PHASE-A MVEUAENGAM\0"
  "BAMUM LETTER PHASE-A NAA MFON\0"
  "BAMUM LETTER PHASE-A NAQ\0"
  "BAMUM LETTER PHASE-A NDA LEERAEWA\0"
  "BAMUM LETTER PHASE-A NDAANGGEUAET\0"
  "BAMUM LETTER PHASE-A NEN\0"
  "BAMUM LETTER PHASE-A NGANGU\0"
  "BAMUM LETTER PHASE-A NGGEN\0"
  "BAMUM LETTER PHASE-A NGKUE MFON\0"
  "BAMUM LETTER PHASE-A NGKUENZEUM\0"
  "BAMUM LETTER PHASE-A NIKA\0"
  "BAMUM LETTER PHASE-A NJAM\0"
  "BAMUM LETTER PHASE-A NKAARAE\0"
  "BAMUM LETTER PHASE-A NKINDI\0"
  "BAMUM LETTER PHASE-A NSHA\0"
  "BAMUM LETTER PHASE-A NSHIEE\0"
  "BAMUM LETTER PHASE-A NTAP\0"
  "BAMUM LETTER PHASE-A NTAP MFAA\0"
  "BAMUM LETTER PHASE-A NTAP NTAA\0"
  "BAMUM LETTER PHASE-A NTOQPEN\0"
  "BAMUM LETTER PHASE-A NYET\0"
  "BAMUM LETTER PHASE-A NYI\0"
  "BAMUM LETTER PHASE-A NYIT MONGKEUAEQ\0"
  "BAMUM LETTER PHASE-A NZA MFON\0"
  "BAMUM LETTER PHASE-A NZUN MEUT\0"
  "BAMUM LETTER PHASE-A PA LEERAEWA\0"
  "BAMUM LETTER PHASE-A PAAM\0"
  "BAMUM LETTER PHASE-A PAARAE\0"
  "BAMUM LETTER PHASE-A PAMSHAE\0"
  "BAMUM LETTER PHASE-A PASHAE\0"
  "BAMUM LETTER PHASE-A PET\0"
  "BAMUM LETTER PHASE-A PON MFON PIPAEMBA\0"
  "BAMUM LETTER PHASE-A PON MFON PIPAEMGBIEE\0"
  "BAMUM LETTER PHASE-A PON PA NJI PIPAEMBA\0"
  "BAMUM LETTER PHASE-A PON PA NJI PIPAEMGBIEE\0"
  "BAMUM LETTER PHASE-A PUP\0"
  "BAMUM LETTER PHASE-A ROM\0"
  "BAMUM LETTER PHASE-A SEUNYAM\0"
  "BAMUM LETTER PHASE-A SHINDA PA NJI\0"
  "BAMUM LETTER PHASE-A SHIRAE\0"
  "BAMUM LETTER PHASE-A SHOQ NSHUT YUM\0"
  "BAMUM LETTER PHASE-A SHUENSHUET\0"
  "BAMUM LETTER PHASE-A SHUM\0"
  "BAMUM LETTER PHASE-A SISA\0"
  "BAMUM LETTER PHASE-A SONJAM\0"
  "BAMUM LETTER PHASE-A SOQ\0"
  "BAMUM LETTER PHASE-A SOT\0"
  "BAMUM LETTER PHASE-A SUU\0"
  "BAMUM LETTER PHASE-A TEUAEN\0"
  "BAMUM LETTER PHASE-A TEUTEUWEN\0"
  "BAMUM LETTER PHASE-A TITA MFON\0"
  "BAMUM LETTER PHASE-A TU MAEMBA\0"
  "BAMUM LETTER PHASE-A TUAEP\0"
  "BAMUM LETTER PHASE-A U YUQ NAE\0"
  "BAMUM LETTER PHASE-A UNKNOWN\0"
  "BAMUM LETTER PHASE-A VEE\0"
  "BAMUM LETTER PHASE-B FEE\0"
  "BAMUM LETTER PHASE-B FEUX\0"
  "BAMUM LETTER PHASE-B GHEUGHEN\0"
  "BAMUM LETTER PHASE-B GHEUGHEUAEM\0"
  "BAMUM LETTER PHASE-B KAM\0"
  "BAMUM LETTER PHASE-B KEUAEM\0"
  "BAMUM LETTER PHASE-B KEUPUQ\0"
  "BAMUM LETTER PHASE-B KEUYEUX\0"
  "BAMUM LETTER PHASE-B KIEEM\0"
  "BAMUM LETTER PHASE-B KIQ\0"
  "BAMUM LETTER PHASE-B LAANAE\0"
  "BAMUM LETTER PHASE-B LAM NSHUT NYAM\0"
  "BAMUM LETTER PHASE-B LET\0"
  "BAMUM LETTER PHASE-B LOM NTEUM\0"
  "BAMUM LETTER PHASE-B MA\0"
  "BAMUM LETTER PHASE-B MA NJEUAENA\0"
  "BAMUM LETTER PHASE-B MA NJUQA\0"
  "BAMUM LETTER PHASE-B MBA MAELEE\0"
  "BAMUM LETTER PHASE-B MBAARAE\0"
  "BAMUM LETTER PHASE-B MBEURI\0"
  "BAMUM LETTER PHASE-B MBIT MBAAKET\0"
  "BAMUM LETTER PHASE-B MBUOQ\0"
  "BAMUM LETTER PHASE-B MEUQ\0"
  "BAMUM LETTER PHASE-B MEUT NGGEET\0"
  "BAMUM LETTER PHASE-B MFIYAQ\0"
  "BAMUM LETTER PHASE-B MFON TEUAEQ\0"
  "BAMUM LETTER PHASE-B MONTIEEN\0"
  "BAMUM LETTER PHASE-B NDU NJAA\0"
  "BAMUM LETTER PHASE-B NGGAAM\0"
  "BAMUM LETTER PHASE-B NGGEU MBU\0"
  "BAMUM LETTER PHASE-B NGGUOQ\0"
  "BAMUM LETTER PHASE-B NGGUOQ LARGE\0"
  "BAMUM LETTER PHASE-B NGKINDI MVOP\0"
  "BAMUM LETTER PHASE-B NGOM\0"
  "BAMUM LETTER PHASE-B NSEN\0"
  "BAMUM LETTER PHASE-B NSHUET\0"
  "BAMUM LETTER PHASE-B NTIEE SHEUOQ\0"
  "BAMUM LETTER PHASE-B NYAEMAE\0"
  "BAMUM LETTER PHASE-B NYI NTEUM\0"
  "BAMUM LETTER PHASE-B PARUM\0"
  "BAMUM LETTER PHASE-B PEESHI\0"
  "BAMUM LETTER PHASE-B PIT\0"
  "BAMUM LETTER PHASE-B PUNGAAM\0"
  "BAMUM LETTER PHASE-B SAKEUAE\0"
  "BAMUM LETTER PHASE-B SET TU\0"
  "BAMUM LETTER PHASE-B SHET NJAQ\0"
  "BAMUM LETTER PHASE-B SHEUAEQTU\0"
  "BAMUM LETTER PHASE-B SIEE\0"
  "BAMUM LETTER PHASE-B SUE\0"
  "BAMUM LETTER PHASE-B TAAM\0"
  "BAMUM LETTER PHASE-B TU MAEMGBIEE\0"
  "BAMUM LETTER PHASE-B TU NSIEE\0"
  "BAMUM LETTER PHASE-B VEUM\0"
  "BAMUM LETTER PHASE-B WUAET\0"
  "BAMUM LETTER PHASE-B YAFU LEERAEWA\0"
  "BAMUM LETTER PHASE-B YEURAE\0"
  "BAMUM LETTER PHASE-C BUNG\0"
  "BAMUM LETTER PHASE-C FUE\0"
  "BAMUM LETTER PHASE-C GBAYI\0"
  "BAMUM LETTER PHASE-C GHAP\0"
  "BAMUM LETTER PHASE-C GHARAE\0"
  "BAMUM LETTER PHASE-C KAA\0"
  "BAMUM LETTER PHASE-C KEN FATIGUE\0"
  "BAMUM LETTER PHASE-C KEN LAW\0"
  "BAMUM LETTER PHASE-C KET\0"
  "BAMUM LETTER PHASE-C KEUKAQ\0"
  "BAMUM LETTER PHASE-C KEUM\0"
  "BAMUM LETTER PHASE-C KEUSEUX\0"
  "BAMUM LETTER PHASE-C KEUSHEUAEP\0"
  "BAMUM LETTER PHASE-C KPARAQ\0"
  "BAMUM LETTER PHASE-C KUOP NKAARAE\0"
  "BAMUM LETTER PHASE-C KUT\0"
  "BAMUM LETTER PHASE-C LAM\0"
  "BAMUM LETTER PHASE-C LAP\0"
  "BAMUM LETTER PHASE-C LIQ\0"
  "BAMUM LETTER PHASE-C LU\0"
  "BAMUM LETTER PHASE-C MA KEUAERI\0"
  "BAMUM LETTER PHASE-C MA NSIEE\0"
  "BAMUM LETTER PHASE-C MAEMBA\0"
  "BAMUM LETTER PHASE-C MAESI\0"
  "BAMUM LETTER PHASE-C MBAA CABBAGE-TREE\0"
  "BAMUM LETTER PHASE-C MBAA PICKET\0"
  "BAMUM LETTER PHASE-C MBANYI\0"
  "BAMUM LETTER PHASE-C MBEEKEET\0"
  "BAMUM LETTER PHASE-C MBERAE\0"
  "BAMUM LETTER PHASE-C MBEUM\0"
  "BAMUM LETTER PHASE-C MBEUX\0"
  "BAMUM LETTER PHASE-C MBI\0"
  "BAMUM LETTER PHASE-C MBIRIEEN\0"
  "BAMUM LETTER PHASE-C MBIT\0"
  "BAMUM LETTER PHASE-C MBUAEM\0"
  "BAMUM LETTER PHASE-C MBUE\0"
  "BAMUM LETTER PHASE-C MGBASAQ\0"
  "BAMUM LETTER PHASE-C MIEE\0"
  "BAMUM LETTER PHASE-C MUAE\0"
  "BAMUM LETTER PHASE-C NANSANAQ\0"
  "BAMUM LETTER PHASE-C NAQ\0"
  "BAMUM LETTER PHASE-C NDAM\0"
  "BAMUM LETTER PHASE-C NDAP\0"
  "BAMUM LETTER PHASE-C NDEUAEREE\0"
  "BAMUM LETTER PHASE-C NDEUT\0"
  "BAMUM LETTER PHASE-C NDIDA\0"
  "BAMUM LETTER PHASE-C NDOMBU\0"
  "BAMUM LETTER PHASE-C NGAQ\0"
  "BAMUM LETTER PHASE-C NGGEN\0"
  "BAMUM LETTER PHASE-C NGGU\0"
  "BAMUM LETTER PHASE-C NGGUAEN NYAM\0"
  "BAMUM LETTER PHASE-C NGGUEET\0"
  "BAMUM LETTER PHASE-C NGGUM\0"
  "BAMUM LETTER PHASE-C NGGUON\0"
  "BAMUM LETTER PHASE-C NGKAQ\0"
  "BAMUM LETTER PHASE-C NGKUE MAEMBA\0"
  "BAMUM LETTER PHASE-C NGOM\0"
  "BAMUM LETTER PHASE-C NJAEM\0"
  "BAMUM LETTER PHASE-C NJAM\0"
  "BAMUM LETTER PHASE-C NJEEEE\0"
  "BAMUM LETTER PHASE-C NJEUX\0"
  "BAMUM LETTER PHASE-C NJUEQ\0"
  "BAMUM LETTER PHASE-C NSA\0"
  "BAMUM LETTER PHASE-C NSEUAEN\0"
  "BAMUM LETTER PHASE-C NSHAQ\0"
  "BAMUM LETTER PHASE-C NSOM\0"
  "BAMUM LETTER PHASE-C NSUN\0"
  "BAMUM LETTER PHASE-C NSUOT NGOM\0"
  "BAMUM LETTER PHASE-C NTAA\0"
  "BAMUM LETTER PHASE-C NTEN\0"
  "BAMUM LETTER PHASE-C NTEUNGBA\0"
  "BAMUM LETTER PHASE-C NTU MBIT\0"
  "BAMUM LETTER PHASE-C NYIR MKPARAQ MEUN\0"
  "BAMUM LETTER PHASE-C NZA\0"
  "BAMUM LETTER PHASE-C NZEUM\0"
  "BAMUM LETTER PHASE-C PEN\0"
  "BAMUM LETTER PHASE-C PIN\0"
  "BAMUM LETTER PHASE-C PIRIEEN\0"
  "BAMUM LETTER PHASE-C RU\0"
  "BAMUM LETTER PHASE-C SETFON\0"
  "BAMUM LETTER PHASE-C SEUX\0"
  "BAMUM LETTER PHASE-C SHIQ\0"
  "BAMUM LETTER PHASE-C SHUEQ\0"
  "BAMUM LETTER PHASE-C SUAET\0"
  "BAMUM LETTER PHASE-C TAASHAE\0"
  "BAMUM LETTER PHASE-C TET\0"
  "BAMUM LETTER PHASE-C TEUTEUX\0"
  "BAMUM LETTER PHASE-C TITA YUE\0"
  "BAMUM LETTER PHASE-C TITUAEP\0"
  "BAMUM LETTER PHASE-C VEUAEPEN\0"
  "BAMUM LETTER PHASE-C VEUX\0"
  "BAMUM LETTER PHASE-C WANGKUOQ\0"
  "BAMUM LETTER PHASE-C WUP\0"
  "BAMUM LETTER PHASE-C YAA\0"
  "BAMUM LETTER PHASE-C YEUQ\0"
  "BAMUM LETTER PHASE-C YU MUOMAE\0"
  "BAMUM LETTER PHASE-C YUM\0"
  "BAMUM LETTER PHASE-C YUWOQ\0"
  "BAMUM LETTER PHASE-D FAA\0"
  "BAMUM LETTER PHASE-D FEUFEUAET\0"
  "BAMUM LETTER PHASE-D GHAA\0"
  "BAMUM LETTER PHASE-D GHEUAE\0"
  "BAMUM LETTER PHASE-D KET\0"
  "BAMUM LETTER PHASE-D KEUAETMEUN\0"
  "BAMUM LETTER PHASE-D KEUM\0"
  "BAMUM LETTER PHASE-D KEUOT MBUAE\0"
  "BAMUM LETTER PHASE-D KEUP\0"
  "BAMUM LETTER PHASE-D KU\0"
  "BAMUM LETTER PHASE-D KUN\0"
  "BAMUM LETTER PHASE-D KUOM\0"
  "BAMUM LETTER PHASE-D KUQ\0"
  "BAMUM LETTER PHASE-D KWAET\0"
  "BAMUM LETTER PHASE-D KYEE\0"
  "BAMUM LETTER PHASE-D LEEEE\0"
  "BAMUM LETTER PHASE-D LET\0"
  "BAMUM LETTER PHASE-D LEUAEP\0"
  "BAMUM LETTER PHASE-D LEUM\0"
  "BAMUM LETTER PHASE-D LIEE\0"
  "BAMUM LETTER PHASE-D LOQ\0"
  "BAMUM LETTER PHASE-D LUM\0"
  "BAMUM LETTER PHASE-D M\0"
  "BAMUM LETTER PHASE-D MAENJET\0"
  "BAMUM LETTER PHASE-D MALEERI\0"
  "BAMUM LETTER PHASE-D MBAA\0"
  "BAMUM LETTER PHASE-D MBAA SEVEN\0"
  "BAMUM LETTER PHASE-D MBUO\0"
  "BAMUM LETTER PHASE-D MEEEE\0"
  "BAMUM LETTER PHASE-D MEUN\0"
  "BAMUM LETTER PHASE-D MEUT\0"
  "BAMUM LETTER PHASE-D MFEUAE\0"
  "BAMUM LETTER PHASE-D MFEUT\0"
  "BAMUM LETTER PHASE-D MFIEE\0"
  "BAMUM LETTER PHASE-D MFO\0"
  "BAMUM LETTER PHASE-D MFON\0"
  "BAMUM LETTER PHASE-D MGBEUN\0"
  "BAMUM LETTER PHASE-D MGBIEE\0"
  "BAMUM LETTER PHASE-D MGBOFUM\0"
  "BAMUM LETTER PHASE-D MONI\0"
  "BAMUM LETTER PHASE-D MU\0"
  "BAMUM LETTER PHASE-D MVOP\0"
  "BAMUM LETTER PHASE-D NDAM\0"
  "BAMUM LETTER PHASE-D NDEE\0"
  "BAMUM LETTER PHASE-D NDEUX\0"
  "BAMUM LETTER PHASE-D NDON\0"
  "BAMUM LETTER PHASE-D NGGAAM\0"
  "BAMUM LETTER PHASE-D NGGAAMAE\0"
  "BAMUM LETTER PHASE-D NGGAP\0"
  "BAMUM LETTER PHASE-D NGGEUX\0"
  "BAMUM LETTER PHASE-D NGGUOM\0"
  "BAMUM LETTER PHASE-D NGGWAEN\0"
  "BAMUM LETTER PHASE-D NGKAP\0"
  "BAMUM LETTER PHASE-D NGKEUAEQ\0"
  "BAMUM LETTER PHASE-D NGKEURI\0"
  "BAMUM LETTER PHASE-D NGKIEE\0"
  "BAMUM LETTER PHASE-D NGKUN\0"
  "BAMUM LETTER PHASE-D NGKYEE\0"
  "BAMUM LETTER PHASE-D NI\0"
  "BAMUM LETTER PHASE-D NJAP\0"
  "BAMUM LETTER PHASE-D NJEUAEM\0"
  "BAMUM LETTER PHASE-D NJEUT\0"
  "BAMUM LETTER PHASE-D NJI\0"
  "BAMUM LETTER PHASE-D NJIEE\0"
  "BAMUM LETTER PHASE-D NJUEQ\0"
  "BAMUM LETTER PHASE-D NSHEE\0"
  "BAMUM LETTER PHASE-D NSHUT\0"
  "BAMUM LETTER PHASE-D NSIEEP\0"
  "BAMUM LETTER PHASE-D NSIEET\0"
  "BAMUM LETTER PHASE-D NSUM\0"
  "BAMUM LETTER PHASE-D NTEE\0"
  "BAMUM LETTER PHASE-D NTEUM\0"
  "BAMUM LETTER PHASE-D NTUU\0"
  "BAMUM LETTER PHASE-D NU\0"
  "BAMUM LETTER PHASE-D NYAM\0"
  "BAMUM LETTER PHASE-D NYET\0"
  "BAMUM LETTER PHASE-D NYI\0"
  "BAMUM LETTER PHASE-D NYUE\0"
  "BAMUM LETTER PHASE-D PAP\0"
  "BAMUM LETTER PHASE-D PEE\0"
  "BAMUM LETTER PHASE-D PEUTAE\0"
  "BAMUM LETTER PHASE-D PIP\0"
  "BAMUM LETTER PHASE-D PUQ\0"
  "BAMUM LETTER PHASE-D PUUT\0"
  "BAMUM LETTER PHASE-D RAEM\0"
  "BAMUM LETTER PHASE-D REN MUCH\0"
  "BAMUM LETTER PHASE-D REN OLD\0"
  "BAMUM LETTER PHASE-D RII\0"
  "BAMUM LETTER PHASE-D SAP\0"
  "BAMUM LETTER PHASE-D SAQ\0"
  "BAMUM LETTER PHASE-D SEUAEQ\0"
  "BAMUM LETTER PHASE-D SHEE\0"
  "BAMUM LETTER PHASE-D SHEUAE\0"
  "BAMUM LETTER PHASE-D SHEUX\0"
  "BAMUM LETTER PHASE-D SHII\0"
  "BAMUM LETTER PHASE-D SHOQ\0"
  "BAMUM LETTER PHASE-D SHU\0"
  "BAMUM LETTER PHASE-D SOT\0"
  "BAMUM LETTER PHASE-D SUAE\0"
  "BAMUM LETTER PHASE-D SUE\0"
  "BAMUM LETTER PHASE-D SUU\0"
  "BAMUM LETTER PHASE-D TAE\0"
  "BAMUM LETTER PHASE-D TEEEE\0"
  "BAMUM LETTER PHASE-D TEUAEN\0"
  "BAMUM LETTER PHASE-D TEUN\0"
  "BAMUM LETTER PHASE-D TEUT\0"
  "BAMUM LETTER PHASE-D TI\0"
  "BAMUM LETTER PHASE-D TOQ\0"
  "BAMUM LETTER PHASE-D TU\0"
  "BAMUM LETTER PHASE-D TUOT\0"
  "BAMUM LETTER PHASE-D WAP\0"
  "BAMUM LETTER PHASE-D WUAEN\0"
  "BAMUM LETTER PHASE-D YAEMMAE\0"
  "BAMUM LETTER PHASE-D YEN\0"
  "BAMUM LETTER PHASE-D YUOM\0"
  "BAMUM LETTER PHASE-D YUOP\0"
  "BAMUM LETTER PHASE-D YUWOQ\0"
  "BAMUM LETTER PHASE-E A\0"
  "BAMUM LETTER PHASE-E FA\0"
  "BAMUM LETTER PHASE-E FAQ\0"
  "BAMUM LETTER PHASE-E FEE\0"
  "BAMUM LETTER PHASE-E FOM\0"
  "BAMUM LETTER PHASE-E FU CALL\0"
  "BAMUM LETTER PHASE-E FU I\0"
  "BAMUM LETTER PHASE-E FU REMEDY\0"
  "BAMUM LETTER PHASE-E FUE\0"
  "BAMUM LETTER PHASE-E FUET\0"
  "BAMUM LETTER PHASE-E GBET\0"
  "BAMUM LETTER PHASE-E GBEUX\0"
  "BAMUM LETTER PHASE-E GHAAMAE\0"
  "BAMUM LETTER PHASE-E GHET\0"
  "BAMUM LETTER PHASE-E GHEUAE\0"
  "BAMUM LETTER PHASE-E GHEUN\0"
  "BAMUM LETTER PHASE-E GHEUX\0"
  "BAMUM LETTER PHASE-E GHOM\0"
  "BAMUM LETTER PHASE-E I\0"
  "BAMUM LETTER PHASE-E KET\0"
  "BAMUM LETTER PHASE-E KEUAE\0"
  "BAMUM LETTER PHASE-E KEUX\0"
  "BAMUM LETTER PHASE-E KI\0"
  "BAMUM LETTER PHASE-E KO\0"
  "BAMUM LETTER PHASE-E KPEUX\0"
  "BAMUM LETTER PHASE-E KUET\0"
  "BAMUM LETTER PHASE-E KUOP\0"
  "BAMUM LETTER PHASE-E KUT\0"
  "BAMUM LETTER PHASE-E LAAM\0"
  "BAMUM LETTER PHASE-E LAP\0"
  "BAMUM LETTER PHASE-E LAQ\0"
  "BAMUM LETTER PHASE-E LEUAEM\0"
  "BAMUM LETTER PHASE-E LOM\0"
  "BAMUM LETTER PHASE-E LOON\0"
  "BAMUM LETTER PHASE-E LOOT\0"
  "BAMUM LETTER PHASE-E LOQ\0"
  "BAMUM LETTER PHASE-E LU\0"
  "BAMUM LETTER PHASE-E MA\0"
  "BAMUM LETTER PHASE-E MAE\0"
  "BAMUM LETTER PHASE-E MAEM\0"
  "BAMUM LETTER PHASE-E MAP\0"
  "BAMUM LETTER PHASE-E MAQ\0"
  "BAMUM LETTER PHASE-E MBEE\0"
  "BAMUM LETTER PHASE-E MBEUM\0"
  "BAMUM LETTER PHASE-E MEN\0"
  "BAMUM LETTER PHASE-E MFEUQ\0"
  "BAMUM LETTER PHASE-E MGBA\0"
  "BAMUM LETTER PHASE-E MGBEN\0"
  "BAMUM LETTER PHASE-E MI\0"
  "BAMUM LETTER PHASE-E MIEE\0"
  "BAMUM LETTER PHASE-E MON\0"
  "BAMUM LETTER PHASE-E MUAE\0"
  "BAMUM LETTER PHASE-E MVI\0"
  "BAMUM LETTER PHASE-E NA\0"
  "BAMUM LETTER PHASE-E NAE\0"
  "BAMUM LETTER PHASE-E NDAA MY HOUSE\0"
  "BAMUM LETTER PHASE-E NDAA SOFTNESS\0"
  "BAMUM LETTER PHASE-E NDAP\0"
  "BAMUM LETTER PHASE-E NDIAQ\0"
  "BAMUM LETTER PHASE-E NDIQ\0"
  "BAMUM LETTER PHASE-E NDUN\0"
  "BAMUM LETTER PHASE-E NGA\0"
  "BAMUM LETTER PHASE-E NGEUREUT\0"
  "BAMUM LETTER PHASE-E NGGEEEE\0"
  "BAMUM LETTER PHASE-E NGGEUAE\0"
  "BAMUM LETTER PHASE-E NGGEUAET\0"
  "BAMUM LETTER PHASE-E NGGUAESHAE NYAM\0"
  "BAMUM LETTER PHASE-E NGGUP\0"
  "BAMUM LETTER PHASE-E NGGURAE\0"
  "BAMUM LETTER PHASE-E NGKA\0"
  "BAMUM LETTER PHASE-E NGKAAMI\0"
  "BAMUM LETTER PHASE-E NGKEUAEM\0"
  "BAMUM LETTER PHASE-E NGKEUX\0"
  "BAMUM LETTER PHASE-E NGKUM\0"
  "BAMUM LETTER PHASE-E NGKUP\0"
  "BAMUM LETTER PHASE-E NGOP\0"
  "BAMUM LETTER PHASE-E NGOQ\0"
  "BAMUM LETTER PHASE-E NGUAE\0"
  "BAMUM LETTER PHASE-E NGUAET\0"
  "BAMUM LETTER PHASE-E NJAEMLI\0"
  "BAMUM LETTER PHASE-E NJEE\0"
  "BAMUM LETTER PHASE-E NJEE EPOCH\0"
  "BAMUM LETTER PHASE-E NJEUX\0"
  "BAMUM LETTER PHASE-E NKOM\0"
  "BAMUM LETTER PHASE-E NSHIEE\0"
  "BAMUM LETTER PHASE-E NSHUE\0"
  "BAMUM LETTER PHASE-E NSHUOP\0"
  "BAMUM LETTER PHASE-E NTAP\0"
  "BAMUM LETTER PHASE-E NTUM\0"
  "BAMUM LETTER PHASE-E NYI BETWEEN\0"
  "BAMUM LETTER PHASE-E NYI CLEAVER\0"
  "BAMUM LETTER PHASE-E NZAQ\0"
  "BAMUM LETTER PHASE-E NZUQ\0"
  "BAMUM LETTER PHASE-E O\0"
  "BAMUM LETTER PHASE-E PA PEOPLE\0"
  "BAMUM LETTER PHASE-E PA PLURAL\0"
  "BAMUM LETTER PHASE-E PAA\0"
  "BAMUM LETTER PHASE-E PAAM\0"
  "BAMUM LETTER PHASE-E PEEM\0"
  "BAMUM LETTER PHASE-E PEUT\0"
  "BAMUM LETTER PHASE-E PEUX\0"
  "BAMUM LETTER PHASE-E PI\0"
  "BAMUM LETTER PHASE-E PIEEQ\0"
  "BAMUM LETTER PHASE-E PIET\0"
  "BAMUM LETTER PHASE-E PO\0"
  "BAMUM LETTER PHASE-E POON\0"
  "BAMUM LETTER PHASE-E PU\0"
  "BAMUM LETTER PHASE-E PUAE\0"
  "BAMUM LETTER PHASE-E PUAQ\0"
  "BAMUM LETTER PHASE-E PUE\0"
  "BAMUM LETTER PHASE-E PUM\0"
  "BAMUM LETTER PHASE-E RAE\0"
  "BAMUM LETTER PHASE-E RAQ\0"
  "BAMUM LETTER PHASE-E REUX\0"
  "BAMUM LETTER PHASE-E RIMGBA\0"
  "BAMUM LETTER PHASE-E SAA\0"
  "BAMUM LETTER PHASE-E SEE\0"
  "BAMUM LETTER PHASE-E SET\0"
  "BAMUM LETTER PHASE-E SHEUAEQ\0"
  "BAMUM LETTER PHASE-E SHIQ\0"
  "BAMUM LETTER PHASE-E SHO\0"
  "BAMUM LETTER PHASE-E SHOQ\0"
  "BAMUM LETTER PHASE-E SOM\0"
  "BAMUM LETTER PHASE-E SUAEN\0"
  "BAMUM LETTER PHASE-E TAA\0"
  "BAMUM LETTER PHASE-E TAAQ\0"
  "BAMUM LETTER PHASE-E TAEN NTEUM\0"
  "BAMUM LETTER PHASE-E TAM\0"
  "BAMUM LETTER PHASE-E TAQ\0"
  "BAMUM LETTER PHASE-E TEN\0"
  "BAMUM LETTER PHASE-E TEU\0"
  "BAMUM LETTER PHASE-E TEUAEQ\0"
  "BAMUM LETTER PHASE-E TOO\0"
  "BAMUM LETTER PHASE-E TOON\0"
  "BAMUM LETTER PHASE-E TOQ\0"
  "BAMUM LETTER PHASE-E TUAE\0"
  "BAMUM LETTER PHASE-E TUM\0"
  "BAMUM LETTER PHASE-E TUMAE\0"
  "BAMUM LETTER PHASE-E VEE\0"
  "BAMUM LETTER PHASE-E VEUAE\0"
  "BAMUM LETTER PHASE-E VOM\0"
  "BAMUM LETTER PHASE-E WEUX\0"
  "BAMUM LETTER PHASE-E WUE\0"
  "BAMUM LETTER PHASE-E WUO\0"
  "BAMUM LETTER PHASE-E YAP\0"
  "BAMUM LETTER PHASE-E YEUAE\0"
  "BAMUM LETTER PHASE-E YEUAET\0"
  "BAMUM LETTER PHASE-E YEUM\0"
  "BAMUM LETTER PHASE-E YEUX\0"
  "BAMUM LETTER PHASE-E YIEE\0"
  "BAMUM LETTER PHASE-E YIT\0"
  "BAMUM LETTER PHASE-E YOQ COVER\0"
  "BAMUM LETTER PHASE-E YOQ SWIMMING\0"
  "BAMUM LETTER PHASE-E YUAEN\0"
  "BAMUM LETTER PHASE-E YUEQ\0"
  "BAMUM LETTER PHASE-E YUN\0"
  "BAMUM LETTER PHASE-E YUQ\0"
  "BAMUM LETTER PHASE-F EE\0"
  "BAMUM LETTER PHASE-F FOM\0"
  "BAMUM LETTER PHASE-F KA\0"
  "BAMUM LETTER PHASE-F KEN\0"
  "BAMUM LETTER PHASE-F KET\0"
  "BAMUM LETTER PHASE-F KO\0"
  "BAMUM LETTER PHASE-F KPA\0"
  "BAMUM LETTER PHASE-F KU\0"
  "BAMUM LETTER PHASE-F KYEE\0"
  "BAMUM LETTER PHASE-F LA\0"
  "BAMUM LETTER PHASE-F LI\0"
  "BAMUM LETTER PHASE-F LOQ\0"
  "BAMUM LETTER PHASE-F M\0"
  "BAMUM LETTER PHASE-F MA\0"
  "BAMUM LETTER PHASE-F MBAA\0"
  "BAMUM LETTER PHASE-F MBEN\0"
  "BAMUM LETTER PHASE-F MEEEE\0"
  "BAMUM LETTER PHASE-F MO\0"
  "BAMUM LETTER PHASE-F NDAA\0"
  "BAMUM LETTER PHASE-F NGGA\0"
  "BAMUM LETTER PHASE-F NGKWAEN\0"
  "BAMUM LETTER PHASE-F NI\0"
  "BAMUM LETTER PHASE-F NJAEM\0"
  "BAMUM LETTER PHASE-F NJUAE\0"
  "BAMUM LETTER PHASE-F NSHA\0"
  "BAMUM LETTER PHASE-F NTEE\0"
  "BAMUM LETTER PHASE-F NU\0"
  "BAMUM LETTER PHASE-F NUAE\0"
  "BAMUM LETTER PHASE-F NYI\0"
  "BAMUM LETTER PHASE-F PEE\0"
  "BAMUM LETTER PHASE-F PEUX\0"
  "BAMUM LETTER PHASE-F PUAE\0"
  "BAMUM LETTER PHASE-F REE\0"
  "BAMUM LETTER PHASE-F REN\0"
  "BAMUM LETTER PHASE-F REUX\0"
  "BAMUM LETTER PHASE-F RIEE\0"
  "BAMUM LETTER PHASE-F RII\0"
  "BAMUM LETTER PHASE-F RU\0"
  "BAMUM LETTER PHASE-F SAMBA\0"
  "BAMUM LETTER PHASE-F SEUX\0"
  "BAMUM LETTER PHASE-F SHII\0"
  "BAMUM LETTER PHASE-F SHO\0"
  "BAMUM LETTER PHASE-F SHU\0"
  "BAMUM LETTER PHASE-F SI\0"
  "BAMUM LETTER PHASE-F SUU\0"
  "BAMUM LETTER PHASE-F TAA\0"
  "BAMUM LETTER PHASE-F TAE\0"
  "BAMUM LETTER PHASE-F TET\0"
  "BAMUM LETTER PHASE-F U\0"
  "BAMUM LETTER PHASE-F VUEQ\0"
  "BAMUM LETTER PHASE-F WA\0"
  "BAMUM LETTER PHASE-F WUE\0"
  "BAMUM LETTER PHASE-F YA\0"
  "BAMUM LETTER PHASE-F YOQ\0"
  "BAMUM LETTER PI\0"
  "BAMUM LETTER PUAE\0"
  "BAMUM LETTER PUE\0"
  "BAMUM LETTER RAE\0"
  "BAMUM LETTER REE\0"
  "BAMUM LETTER REN\0"
  "BAMUM LETTER REUX\0"
  "BAMUM LETTER RIEE\0"
  "BAMUM LETTER RII\0"
  "BAMUM LETTER RU\0"
  "BAMUM LETTER SAMBA\0"
  "BAMUM LETTER SEUX\0"
  "BAMUM LETTER SHEUX\0"
  "BAMUM LETTER SHII\0"
  "BAMUM LETTER SHO\0"
  "BAMUM LETTER SHU\0"
  "BAMUM LETTER SI\0"
  "BAMUM LETTER SUU\0"
  "BAMUM LETTER TAA\0"
  "BAMUM LETTER TAE\0"
  "BAMUM LETTER TEN\0"
  "BAMUM LETTER TET\0"
  "BAMUM LETTER TI\0"
  "BAMUM LETTER U\0"
  "BAMUM LETTER WA\0"
  "BAMUM LETTER WUE\0"
  "BAMUM LETTER YA\0"
  "BAMUM LETTER YOQ\0"
  "BAMUM LETTER YUQ\0"
  "BAMUM NJAEMLI\0"
  "BAMUM QUESTION MARK\0"
  "BAMUM SEMICOLON\0"
  "BANANA\0"
  "BANJO\0"
  "BANK\0"
  "BANKNOTE WITH DOLLAR SIGN\0"
  "BANKNOTE WITH EURO SIGN\0"
  "BANKNOTE WITH POUND SIGN\0"
  "BANKNOTE WITH YEN SIGN\0"
  "BAR CHART\0"
  "BAR OF SOAP\0"
  "BARBER POLE\0"
  "BASEBALL\0"
  "BASKET\0"
  "BASKETBALL AND HOOP\0"
  "BASSA VAH COMBINING HIGH TONE\0"
  "BASSA VAH COMBINING HIGH-LOW TONE\0"
  "BASSA VAH COMBINING LOW TONE\0"
  "BASSA VAH COMBINING LOW-MID TONE\0"
  "BASSA VAH COMBINING MID TONE\0"
  "BASSA VAH FULL STOP\0"
  "BASSA VAH LETTER A\0"
  "BASSA VAH LETTER BA\0"
  "BASSA VAH LETTER CE\0"
  "BASSA VAH LETTER DHII\0"
  "BASSA VAH LETTER DO\0"
  "BASSA VAH LETTER E\0"
  "BASSA VAH LETTER EE\0"
  "BASSA VAH LETTER ENNI\0"
  "BASSA VAH LETTER FA\0"
  "BASSA VAH LETTER GAH\0"
  "BASSA VAH LETTER GBU\0"
  "BASSA VAH LETTER HWAH\0"
  "BASSA VAH LETTER I\0"
  "BASSA VAH LETTER JO\0"
  "BASSA VAH LETTER KA\0"
  "BASSA VAH LETTER KPAH\0"
  "BASSA VAH LETTER MBE\0"
  "BASSA VAH LETTER O\0"
  "BASSA VAH LETTER OO\0"
  "BASSA VAH LETTER PA\0"
  "BASSA VAH LETTER SE\0"
  "BASSA VAH LETTER TO\0"
  "BASSA VAH LETTER U\0"
  "BASSA VAH LETTER UWU\0"
  "BASSA VAH LETTER VU\0"
  "BASSA VAH LETTER WA\0"
  "BASSA VAH LETTER WADDA\0"
  "BASSA VAH LETTER YEIN\0"
  "BASSA VAH LETTER YIE\0"
  "BASSA VAH LETTER ZO\0"
  "BAT\0"
  "BATAK CONSONANT SIGN H\0"
  "BATAK CONSONANT SIGN NG\0"
  "BATAK LETTER A\0"
  "BATAK LETTER BA\0"
  "BATAK LETTER CA\0"
  "BATAK LETTER DA\0"
  "BATAK LETTER GA\0"
  "BATAK LETTER HA\0"
  "BATAK LETTER I\0"
  "BATAK LETTER JA\0"
  "BATAK LETTER KARO BA\0"
  "BATAK LETTER LA\0"
  "BATAK LETTER MA\0"
  "BATAK LETTER MANDAILING HA\0"
  "BATAK LETTER MANDAILING NA\0"
  "BATAK LETTER MANDAILING SA\0"
  "BATAK LETTER MBA\0"
  "BATAK LETTER NA\0"
  "BATAK LETTER NDA\0"
  "BATAK LETTER NGA\0"
  "BATAK LETTER NORTHERN TA\0"
  "BATAK LETTER NYA\0"
  "BATAK LETTER PA\0"
  "BATAK LETTER PAKPAK WA\0"
  "BATAK LETTER RA\0"
  "BATAK LETTER SA\0"
  "BATAK LETTER SIMALUNGUN A\0"
  "BATAK LETTER SIMALUNGUN GA\0"
  "BATAK LETTER SIMALUNGUN HA\0"
  "BATAK LETTER SIMALUNGUN LA\0"
  "BATAK LETTER SIMALUNGUN MA\0"
  "BATAK LETTER SIMALUNGUN PA\0"
  "BATAK LETTER SIMALUNGUN RA\0"
  "BATAK LETTER SIMALUNGUN SA\0"
  "BATAK LETTER SIMALUNGUN WA\0"
  "BATAK LETTER SIMALUNGUN YA\0"
  "BATAK LETTER SOUTHERN TA\0"
  "BATAK LETTER U\0"
  "BATAK LETTER WA\0"
  "BATAK LETTER YA\0"
  "BATAK PANGOLAT\0"
  "BATAK PANONGONAN\0"
  "BATAK SIGN TOMPI\0"
  "BATAK SYMBOL BINDU JUDUL\0"
  "BATAK SYMBOL BINDU NA METEK\0"
  "BATAK SYMBOL BINDU PANGOLAT\0"
  "BATAK SYMBOL BINDU PINARBORAS\0"
  "BATAK VOWEL SIGN E\0"
  "BATAK VOWEL SIGN EE\0"
  "BATAK VOWEL SIGN I\0"
  "BATAK VOWEL SIGN KARO I\0"
  "BATAK VOWEL SIGN KARO O\0"
  "BATAK VOWEL SIGN O\0"
  "BATAK VOWEL SIGN PAKPAK E\0"
  "BATAK VOWEL SIGN U\0"
  "BATAK VOWEL SIGN U FOR SIMALUNGUN SA\0"
  "BATH\0"
  "BATHTUB\0"
  "BATTERY\0"
  "BEACH WITH UMBRELLA\0"
  "BEAMED ASCENDING MUSICAL NOTES\0"
  "BEAMED DESCENDING MUSICAL NOTES\0"
  "BEAMED EIGHTH NOTES\0"
  "BEAMED SIXTEENTH NOTES\0"
  "BEANS\0"
  "BEAR FACE\0"
  "BEARDED PERSON\0"
  "BEATING HEART\0"
  "BEAVER\0"
  "BECAUSE\0"
  "BED\0"
  "BEER MUG\0"
  "BEETLE\0"
  "BELL\0"
  "BELL PEPPER\0"
  "BELL SYMBOL\0"
  "BELL WITH CANCELLATION STROKE\0"
  "BELLHOP BELL\0"
  "BENGALI ABBREVIATION SIGN\0"
  "BENGALI ANJI\0"
  "BENGALI AU LENGTH MARK\0"
  "BENGALI CURRENCY DENOMINATOR SIXTEEN\0"
  "BENGALI CURRENCY NUMERATOR FOUR\0"
  "BENGALI CURRENCY NUMERATOR ONE\0"
  "BENGALI CURRENCY NUMERATOR ONE LESS THAN THE DENOMINATOR\0"
  "BENGALI CURRENCY NUMERATOR THREE\0"
  "BENGALI CURRENCY NUMERATOR TWO\0"
  "BENGALI DIGIT EIGHT\0"
  "BENGALI DIGIT FIVE\0"
  "BENGALI DIGIT FOUR\0"
  "BENGALI DIGIT NINE\0"
  "BENGALI DIGIT ONE\0"
  "BENGALI DIGIT SEVEN\0"
  "BENGALI DIGIT SIX\0"
  "BENGALI DIGIT THREE\0"
  "BENGALI DIGIT TWO\0"
  "BENGALI DIGIT ZERO\0"
  "BENGALI GANDA MARK\0"
  "BENGALI ISSHAR\0"
  "BENGALI LETTER A\0"
  "BENGALI LETTER AA\0"
  "BENGALI LETTER AI\0"
  "BENGALI LETTER AU\0"
  "BENGALI LETTER BA\0"
  "BENGALI LETTER BHA\0"
  "BENGALI LETTER CA\0"
  "BENGALI LETTER CHA\0"
  "BENGALI LETTER DA\0"
  "BENGALI LETTER DDA\0"
  "BENGALI LETTER DDHA\0"
  "BENGALI LETTER DHA\0"
  "BENGALI LETTER E\0"
  "BENGALI LETTER GA\0"
  "BENGALI LETTER GHA\0"
  "BENGALI LETTER HA\0"
  "BENGALI LETTER I\0"
  "BENGALI LETTER II\0"
  "BENGALI LETTER JA\0"
  "BENGALI LETTER JHA\0"
  "BENGALI LETTER KA\0"
  "BENGALI LETTER KHA\0"
  "BENGALI LETTER KHANDA TA\0"
  "BENGALI LETTER LA\0"
  "BENGALI LETTER MA\0"
  "BENGALI LETTER NA\0"
  "BENGALI LETTER NGA\0"
  "BENGALI LETTER NNA\0"
  "BENGALI LETTER NYA\0"
  "BENGALI LETTER O\0"
  "BENGALI LETTER PA\0"
  "BENGALI LETTER PHA\0"
  "BENGALI LETTER RA\0"
  "BENGALI LETTER RA WITH LOWER DIAGONAL\0"
  "BENGALI LETTER RA WITH MIDDLE DIAGONAL\0"
  "BENGALI LETTER RHA\0"
  "BENGALI LETTER RRA\0"
  "BENGALI LETTER SA\0"
  "BENGALI LETTER SHA\0"
  "BENGALI LETTER SSA\0"
  "BENGALI LETTER TA\0"
  "BENGALI LETTER THA\0"
  "BENGALI LETTER TTA\0"
  "BENGALI LETTER TTHA\0"
  "BENGALI LETTER U\0"
  "BENGALI LETTER UU\0"
  "BENGALI LETTER VEDIC ANUSVARA\0"
  "BENGALI LETTER VOCALIC L\0"
  "BENGALI LETTER VOCALIC LL\0"
  "BENGALI LETTER VOCALIC R\0"
  "BENGALI LETTER VOCALIC RR\0"
  "BENGALI LETTER YA\0"
  "BENGALI LETTER YYA\0"
  "BENGALI RUPEE MARK\0"
  "BENGALI RUPEE SIGN\0"
  "BENGALI SANDHI MARK\0"
  "BENGALI SIGN ANUSVARA\0"
  "BENGALI SIGN AVAGRAHA\0"
  "BENGALI SIGN CANDRABINDU\0"
  "BENGALI SIGN NUKTA\0"
  "BENGALI SIGN VIRAMA\0"
  "BENGALI SIGN VISARGA\0"
  "BENGALI VOWEL SIGN AA\0"
  "BENGALI VOWEL SIGN AI\0"
  "BENGALI VOWEL SIGN AU\0"
  "BENGALI VOWEL SIGN E\0"
  "BENGALI VOWEL SIGN I\0"
  "BENGALI VOWEL SIGN II\0"
  "BENGALI VOWEL SIGN O\0"
  "BENGALI VOWEL SIGN U\0"
  "BENGALI VOWEL SIGN UU\0"
  "BENGALI VOWEL SIGN VOCALIC L\0"
  "BENGALI VOWEL SIGN VOCALIC LL\0"
  "BENGALI VOWEL SIGN VOCALIC R\0"
  "BENGALI VOWEL SIGN VOCALIC RR\0"
  "BENT ARROW POINTING DOWNWARDS THEN NORTH EAST\0"
  "BENTO BOX\0"
  "BENZENE RING\0"
  "BENZENE RING WITH CIRCLE\0"
  "BET SYMBOL\0"
  "BETWEEN\0"
  "BEVERAGE BOX\0"
  "BHAIKSUKI DANDA\0"
  "BHAIKSUKI DIGIT EIGHT\0"
  "BHAIKSUKI DIGIT FIVE\0"
  "BHAIKSUKI DIGIT FOUR\0"
  "BHAIKSUKI DIGIT NINE\0"
  "BHAIKSUKI DIGIT ONE\0"
  "BHAIKSUKI DIGIT SEVEN\0"
  "BHAIKSUKI DIGIT SIX\0"
  "BHAIKSUKI DIGIT THREE\0"
  "BHAIKSUKI DIGIT TWO\0"
  "BHAIKSUKI DIGIT ZERO\0"
  "BHAIKSUKI DOUBLE DANDA\0"
  "BHAIKSUKI GAP FILLER-1\0"
  "BHAIKSUKI GAP FILLER-2\0"
  "BHAIKSUKI HUNDREDS UNIT MARK\0"
  "BHAIKSUKI LETTER A\0"
  "BHAIKSUKI LETTER AA\0"
  "BHAIKSUKI LETTER AI\0"
  "BHAIKSUKI LETTER AU\0"
  "BHAIKSUKI LETTER BA\0"
  "BHAIKSUKI LETTER BHA\0"
  "BHAIKSUKI LETTER CA\0"
  "BHAIKSUKI LETTER CHA\0"
  "BHAIKSUKI LETTER DA\0"
  "BHAIKSUKI LETTER DDA\0"
  "BHAIKSUKI LETTER DDHA\0"
  "BHAIKSUKI LETTER DHA\0"
  "BHAIKSUKI LETTER E\0"
  "BHAIKSUKI LETTER GA\0"
  "BHAIKSUKI LETTER GHA\0"
  "BHAIKSUKI LETTER HA\0"
  "BHAIKSUKI LETTER I\0"
  "BHAIKSUKI LETTER II\0"
  "BHAIKSUKI LETTER JA\0"
  "BHAIKSUKI LETTER JHA\0"
  "BHAIKSUKI LETTER KA\0"
  "BHAIKSUKI LETTER KHA\0"
  "BHAIKSUKI LETTER LA\0"
  "BHAIKSUKI LETTER MA\0"
  "BHAIKSUKI LETTER NA\0"
  "BHAIKSUKI LETTER NGA\0"
  "BHAIKSUKI LETTER NNA\0"
  "BHAIKSUKI LETTER NYA\0"
  "BHAIKSUKI LETTER O\0"
  "BHAIKSUKI LETTER PA\0"
  "BHAIKSUKI LETTER PHA\0"
  "BHAIKSUKI LETTER RA\0"
  "BHAIKSUKI LETTER SA\0"
  "BHAIKSUKI LETTER SHA\0"
  "BHAIKSUKI LETTER SSA\0"
  "BHAIKSUKI LETTER TA\0"
  "BHAIKSUKI LETTER THA\0"
  "BHAIKSUKI LETTER TTA\0"
  "BHAIKSUKI LETTER TTHA\0"
  "BHAIKSUKI LETTER U\0"
  "BHAIKSUKI LETTER UU\0"
  "BHAIKSUKI LETTER VA\0"
  "BHAIKSUKI LETTER VOCALIC L\0"
  "BHAIKSUKI LETTER VOCALIC R\0"
  "BHAIKSUKI LETTER VOCALIC RR\0"
  "BHAIKSUKI LETTER YA\0"
  "BHAIKSUKI NUMBER EIGHT\0"
  "BHAIKSUKI NUMBER EIGHTY\0"
  "BHAIKSUKI NUMBER FIFTY\0"
  "BHAIKSUKI NUMBER FIVE\0"
  "BHAIKSUKI NUMBER FORTY\0"
  "BHAIKSUKI NUMBER FOUR\0"
  "BHAIKSUKI NUMBER NINE\0"
  "BHAIKSUKI NUMBER NINETY\0"
  "BHAIKSUKI NUMBER ONE\0"
  "BHAIKSUKI NUMBER SEVEN\0"
  "BHAIKSUKI NUMBER SEVENTY\0"
  "BHAIKSUKI NUMBER SIX\0"
  "BHAIKSUKI NUMBER SIXTY\0"
  "BHAIKSUKI NUMBER TEN\0"
  "BHAIKSUKI NUMBER THIRTY\0"
  "BHAIKSUKI NUMBER THREE\0"
  "BHAIKSUKI NUMBER TWENTY\0"
  "BHAIKSUKI NUMBER TWO\0"
  "BHAIKSUKI SIGN ANUSVARA\0"
  "BHAIKSUKI SIGN AVAGRAHA\0"
  "BHAIKSUKI SIGN CANDRABINDU\0"
  "BHAIKSUKI SIGN VIRAMA\0"
  "BHAIKSUKI SIGN VISARGA\0"
  "BHAIKSUKI VOWEL SIGN AA\0"
  "BHAIKSUKI VOWEL SIGN AI\0"
  "BHAIKSUKI VOWEL SIGN AU\0"
  "BHAIKSUKI VOWEL SIGN E\0"
  "BHAIKSUKI VOWEL SIGN I\0"
  "BHAIKSUKI VOWEL SIGN II\0"
  "BHAIKSUKI VOWEL SIGN O\0"
  "BHAIKSUKI VOWEL SIGN U\0"
  "BHAIKSUKI VOWEL SIGN UU\0"
  "BHAIKSUKI VOWEL SIGN VOCALIC L\0"
  "BHAIKSUKI VOWEL SIGN VOCALIC R\0"
  "BHAIKSUKI VOWEL SIGN VOCALIC RR\0"
  "BHAIKSUKI WORD SEPARATOR\0"
  "BICYCLE\0"
  "BICYCLIST\0"
  "BIG REVERSE SOLIDUS\0"
  "BIG SOLIDUS\0"
  "BIKINI\0"
  "BILLED CAP\0"
  "BILLIARDS\0"
  "BIOHAZARD SIGN\0"
  "BIRD\0"
  "BIRTHDAY CAKE\0"
  "BISON\0"
  "BITCOIN SIGN\0"
  "BITING LIP\0"
  "BLACK BOWTIE\0"
  "BLACK CENTRE WHITE STAR\0"
  "BLACK CHESS BISHOP\0"
  "BLACK CHESS BISHOP ROTATED NINETY DEGREES\0"
  "BLACK CHESS BISHOP ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "BLACK CHESS EQUIHOPPER\0"
  "BLACK CHESS EQUIHOPPER ROTATED NINETY DEGREES\0"
  "BLACK CHESS KING\0"
  "BLACK CHESS KING ROTATED NINETY DEGREES\0"
  "BLACK CHESS KING ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "BLACK CHESS KNIGHT\0"
  "BLACK CHESS KNIGHT ROTATED FORTY-FIVE DEGREES\0"
  "BLACK CHESS KNIGHT ROTATED NINETY DEGREES\0"
  "BLACK CHESS KNIGHT ROTATED ONE HUNDRED THIRTY-FIVE DEGREES\0"
  "BLACK CHESS KNIGHT ROTATED THREE HUNDRED FIFTEEN DEGREES\0"
  "BLACK CHESS KNIGHT ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "BLACK CHESS KNIGHT ROTATED TWO HUNDRED TWENTY-FIVE DEGREES\0"
  "BLACK CHESS KNIGHT-BISHOP\0"
  "BLACK CHESS KNIGHT-QUEEN\0"
  "BLACK CHESS KNIGHT-ROOK\0"
  "BLACK CHESS PAWN\0"
  "BLACK CHESS PAWN ROTATED NINETY DEGREES\0"
  "BLACK CHESS PAWN ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "BLACK CHESS QUEEN\0"
  "BLACK CHESS QUEEN ROTATED NINETY DEGREES\0"
  "BLACK CHESS QUEEN ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "BLACK CHESS ROOK\0"
  "BLACK CHESS ROOK ROTATED NINETY DEGREES\0"
  "BLACK CHESS ROOK ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "BLACK CHESS TURNED BISHOP\0"
  "BLACK CHESS TURNED KING\0"
  "BLACK CHESS TURNED KNIGHT\0"
  "BLACK CHESS TURNED PAWN\0"
  "BLACK CHESS TURNED QUEEN\0"
  "BLACK CHESS TURNED ROOK\0"
  "BLACK CIRCLE\0"
  "BLACK CIRCLE FOR RECORD\0"
  "BLACK CIRCLE WITH DOWN ARROW\0"
  "BLACK CIRCLE WITH TWO WHITE DOTS\0"
  "BLACK CIRCLE WITH WHITE DOT RIGHT\0"
  "BLACK CLUB SUIT\0"
  "BLACK CROSS ON SHIELD\0"
  "BLACK CURVED DOWNWARDS AND LEFTWARDS ARROW\0"
  "BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW\0"
  "BLACK CURVED LEFTWARDS AND DOWNWARDS ARROW\0"
  "BLACK CURVED LEFTWARDS AND UPWARDS ARROW\0"
  "BLACK CURVED RIGHTWARDS AND DOWNWARDS ARROW\0"
  "BLACK CURVED RIGHTWARDS AND UPWARDS ARROW\0"
  "BLACK CURVED UPWARDS AND LEFTWARDS ARROW\0"
  "BLACK CURVED UPWARDS AND RIGHTWARDS ARROW\0"
  "BLACK DIAMOND\0"
  "BLACK DIAMOND CENTRED\0"
  "BLACK DIAMOND MINUS WHITE X\0"
  "BLACK DIAMOND ON CROSS\0"
  "BLACK DIAMOND SUIT\0"
  "BLACK DIAMOND WITH DOWN ARROW\0"
  "BLACK DOWN POINTING BACKHAND INDEX\0"
  "BLACK DOWN-POINTING DOUBLE TRIANGLE\0"
  "BLACK DOWN-POINTING ISOSCELES RIGHT TRIANGLE\0"
  "BLACK DOWN-POINTING SMALL TRIANGLE\0"
  "BLACK DOWN-POINTING TRIANGLE\0"
  "BLACK DOWNWARDS EQUILATERAL ARROWHEAD\0"
  "BLACK DRAUGHTS KING\0"
  "BLACK DRAUGHTS MAN\0"
  "BLACK DROPLET\0"
  "BLACK FLAG\0"
  "BLACK FLORETTE\0"
  "BLACK FOLDER\0"
  "BLACK FOUR POINTED STAR\0"
  "BLACK HARD SHELL FLOPPY DISK\0"
  "BLACK HEART\0"
  "BLACK HEART SUIT\0"
  "BLACK HEXAGON\0"
  "BLACK HORIZONTAL ELLIPSE\0"
  "BLACK HOURGLASS\0"
  "BLACK LARGE CIRCLE\0"
  "BLACK LARGE CIRCLE MINUS LEFT QUARTER SECTION\0"
  "BLACK LARGE CIRCLE MINUS LOWER QUARTER SECTION\0"
  "BLACK LARGE CIRCLE MINUS RIGHT QUARTER SECTION\0"
  "BLACK LARGE CIRCLE MINUS UPPER QUARTER SECTION\0"
  "BLACK LARGE SQUARE\0"
  "BLACK LEFT LANE MERGE\0"
  "BLACK LEFT POINTING BACKHAND INDEX\0"
  "BLACK LEFT POINTING INDEX\0"
  "BLACK LEFT-POINTING DOUBLE TRIANGLE\0"
  "BLACK LEFT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR\0"
  "BLACK LEFT-POINTING ISOSCELES RIGHT TRIANGLE\0"
  "BLACK LEFT-POINTING POINTER\0"
  "BLACK LEFT-POINTING SMALL TRIANGLE\0"
  "BLACK LEFT-POINTING TRIANGLE\0"
  "BLACK LEFTWARDS BULLET\0"
  "BLACK LEFTWARDS EQUILATERAL ARROWHEAD\0"
  "BLACK LOWER LEFT TRIANGLE\0"
  "BLACK LOWER RIGHT TRIANGLE\0"
  "BLACK LOZENGE\0"
  "BLACK MEDIUM DIAMOND\0"
  "BLACK MEDIUM DOWN-POINTING TRIANGLE\0"
  "BLACK MEDIUM DOWN-POINTING TRIANGLE CENTRED\0"
  "BLACK MEDIUM LEFT-POINTING TRIANGLE\0"
  "BLACK MEDIUM LEFT-POINTING TRIANGLE CENTRED\0"
  "BLACK MEDIUM LOZENGE\0"
  "BLACK MEDIUM RIGHT-POINTING TRIANGLE\0"
  "BLACK MEDIUM RIGHT-POINTING TRIANGLE CENTRED\0"
  "BLACK MEDIUM SMALL DIAMOND\0"
  "BLACK MEDIUM SMALL LOZENGE\0"
  "BLACK MEDIUM SMALL SQUARE\0"
  "BLACK MEDIUM SQUARE\0"
  "BLACK MEDIUM UP-POINTING TRIANGLE\0"
  "BLACK MEDIUM UP-POINTING TRIANGLE CENTRED\0"
  "BLACK MOON LILITH\0"
  "BLACK NEUTRAL FACE\0"
  "BLACK NIB\0"
  "BLACK OCTAGON\0"
  "BLACK PARALLELOGRAM\0"
  "BLACK PENNANT\0"
  "BLACK PENTAGON\0"
  "BLACK PUSHPIN\0"
  "BLACK QUESTION MARK ORNAMENT\0"
  "BLACK RECTANGLE\0"
  "BLACK RIGHT POINTING BACKHAND INDEX\0"
  "BLACK RIGHT POINTING INDEX\0"
  "BLACK RIGHT TRIANGLE CARET\0"
  "BLACK RIGHT-POINTING DOUBLE TRIANGLE\0"
  "BLACK RIGHT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR\0"
  "BLACK RIGHT-POINTING ISOSCELES RIGHT TRIANGLE\0"
  "BLACK RIGHT-POINTING PENTAGON\0"
  "BLACK RIGHT-POINTING POINTER\0"
  "BLACK RIGHT-POINTING SMALL TRIANGLE\0"
  "BLACK RIGHT-POINTING TRIANGLE\0"
  "BLACK RIGHT-POINTING TRIANGLE WITH DOUBLE VERTICAL BAR\0"
  "BLACK RIGHTWARDS ARROW\0"
  "BLACK RIGHTWARDS ARROWHEAD\0"
  "BLACK RIGHTWARDS BULLET\0"
  "BLACK RIGHTWARDS EQUILATERAL ARROWHEAD\0"
  "BLACK ROSETTE\0"
  "BLACK SAFETY SCISSORS\0"
  "BLACK SCISSORS\0"
  "BLACK SHOGI PIECE\0"
  "BLACK SKULL AND CROSSBONES\0"
  "BLACK SLIGHTLY SMALL CIRCLE\0"
  "BLACK SLIGHTLY SMALL SQUARE\0"
  "BLACK SMALL DIAMOND\0"
  "BLACK SMALL LOZENGE\0"
  "BLACK SMALL SQUARE\0"
  "BLACK SMALL STAR\0"
  "BLACK SMALL UP-POINTING CHEVRON\0"
  "BLACK SMILING FACE\0"
  "BLACK SNOWMAN\0"
  "BLACK SPADE SUIT\0"
  "BLACK SQUARE\0"
  "BLACK SQUARE BUTTON\0"
  "BLACK SQUARE CENTRED\0"
  "BLACK SQUARE FOR STOP\0"
  "BLACK STAR\0"
  "BLACK SUN WITH RAYS\0"
  "BLACK TELEPHONE\0"
  "BLACK TINY DIAMOND\0"
  "BLACK TINY LOZENGE\0"
  "BLACK TINY SQUARE\0"
  "BLACK TOUCHTONE TELEPHONE\0"
  "BLACK TRUCK\0"
  "BLACK TWO-WAY LEFT WAY TRAFFIC\0"
  "BLACK UNIVERSAL RECYCLING SYMBOL\0"
  "BLACK UP POINTING BACKHAND INDEX\0"
  "BLACK UP-POINTING DOUBLE TRIANGLE\0"
  "BLACK UP-POINTING ISOSCELES RIGHT TRIANGLE\0"
  "BLACK UP-POINTING SMALL TRIANGLE\0"
  "BLACK UP-POINTING TRIANGLE\0"
  "BLACK UPPER LEFT TRIANGLE\0"
  "BLACK UPPER RIGHT TRIANGLE\0"
  "BLACK UPWARDS EQUILATERAL ARROWHEAD\0"
  "BLACK VERTICAL ELLIPSE\0"
  "BLACK VERTICAL RECTANGLE\0"
  "BLACK VERY SMALL DIAMOND\0"
  "BLACK VERY SMALL LOZENGE\0"
  "BLACK VERY SMALL SQUARE\0"
  "BLACK-FEATHERED NORTH EAST ARROW\0"
  "BLACK-FEATHERED RIGHTWARDS ARROW\0"
  "BLACK-FEATHERED SOUTH EAST ARROW\0"
  "BLACK-LETTER CAPITAL C\0"
  "BLACK-LETTER CAPITAL H\0"
  "BLACK-LETTER CAPITAL I\0"
  "BLACK-LETTER CAPITAL R\0"
  "BLACK-LETTER CAPITAL Z\0"
  "BLANK SYMBOL\0"
  "BLOCK OCTANT-123\0"
  "BLOCK OCTANT-12345\0"
  "BLOCK OCTANT-1234567\0"
  "BLOCK OCTANT-1234568\0"
  "BLOCK OCTANT-1234578\0"
  "BLOCK OCTANT-123458\0"
  "BLOCK OCTANT-12346\0"
  "BLOCK OCTANT-123467\0"
  "BLOCK OCTANT-1234678\0"
  "BLOCK OCTANT-12347\0"
  "BLOCK OCTANT-123478\0"
  "BLOCK OCTANT-12348\0"
  "BLOCK OCTANT-1235\0"
  "BLOCK OCTANT-12356\0"
  "BLOCK OCTANT-123567\0"
  "BLOCK OCTANT-1235678\0"
  "BLOCK OCTANT-123568\0"
  "BLOCK OCTANT-12357\0"
  "BLOCK OCTANT-123578\0"
  "BLOCK OCTANT-12358\0"
  "BLOCK OCTANT-1236\0"
  "BLOCK OCTANT-12367\0"
  "BLOCK OCTANT-123678\0"
  "BLOCK OCTANT-12368\0"
  "BLOCK OCTANT-1237\0"
  "BLOCK OCTANT-12378\0"
  "BLOCK OCTANT-1238\0"
  "BLOCK OCTANT-124\0"
  "BLOCK OCTANT-1245\0"
  "BLOCK OCTANT-12456\0"
  "BLOCK OCTANT-124567\0"
  "BLOCK OCTANT-1245678\0"
  "BLOCK OCTANT-124568\0"
  "BLOCK OCTANT-12457\0"
  "BLOCK OCTANT-124578\0"
  "BLOCK OCTANT-12458\0"
  "BLOCK OCTANT-1246\0"
  "BLOCK OCTANT-12467\0"
  "BLOCK OCTANT-124678\0"
  "BLOCK OCTANT-12468\0"
  "BLOCK OCTANT-1247\0"
  "BLOCK OCTANT-12478\0"
  "BLOCK OCTANT-1248\0"
  "BLOCK OCTANT-125\0"
  "BLOCK OCTANT-1256\0"
  "BLOCK OCTANT-12567\0"
  "BLOCK OCTANT-125678\0"
  "BLOCK OCTANT-12568\0"
  "BLOCK OCTANT-1257\0"
  "BLOCK OCTANT-12578\0"
  "BLOCK OCTANT-1258\0"
  "BLOCK OCTANT-126\0"
  "BLOCK OCTANT-1267\0"
  "BLOCK OCTANT-12678\0"
  "BLOCK OCTANT-1268\0"
  "BLOCK OCTANT-127\0"
  "BLOCK OCTANT-1278\0"
  "BLOCK OCTANT-128\0"
  "BLOCK OCTANT-134\0"
  "BLOCK OCTANT-1345\0"
  "BLOCK OCTANT-13456\0"
  "BLOCK OCTANT-134567\0"
  "BLOCK OCTANT-1345678\0"
  "BLOCK OCTANT-134568\0"
  "BLOCK OCTANT-13457\0"
  "BLOCK OCTANT-134578\0"
  "BLOCK OCTANT-13458\0"
  "BLOCK OCTANT-1346\0"
  "BLOCK OCTANT-13467\0"
  "BLOCK OCTANT-134678\0"
  "BLOCK OCTANT-13468\0"
  "BLOCK OCTANT-1347\0"
  "BLOCK OCTANT-13478\0"
  "BLOCK OCTANT-1348\0"
  "BLOCK OCTANT-135\0"
  "BLOCK OCTANT-1356\0"
  "BLOCK OCTANT-13567\0"
  "BLOCK OCTANT-13568\0"
  "BLOCK OCTANT-13578\0"
  "BLOCK OCTANT-1358\0"
  "BLOCK OCTANT-136\0"
  "BLOCK OCTANT-1367\0"
  "BLOCK OCTANT-13678\0"
  "BLOCK OCTANT-137\0"
  "BLOCK OCTANT-1378\0"
  "BLOCK OCTANT-138\0"
  "BLOCK OCTANT-14\0"
  "BLOCK OCTANT-145\0"
  "BLOCK OCTANT-1456\0"
  "BLOCK OCTANT-14567\0"
  "BLOCK OCTANT-145678\0"
  "BLOCK OCTANT-14568\0"
  "BLOCK OCTANT-1457\0"
  "BLOCK OCTANT-14578\0"
  "BLOCK OCTANT-1458\0"
  "BLOCK OCTANT-146\0"
  "BLOCK OCTANT-1467\0"
  "BLOCK OCTANT-14678\0"
  "BLOCK OCTANT-1468\0"
  "BLOCK OCTANT-147\0"
  "BLOCK OCTANT-1478\0"
  "BLOCK OCTANT-148\0"
  "BLOCK OCTANT-15\0"
  "BLOCK OCTANT-156\0"
  "BLOCK OCTANT-1567\0"
  "BLOCK OCTANT-15678\0"
  "BLOCK OCTANT-1568\0"
  "BLOCK OCTANT-157\0"
  "BLOCK OCTANT-1578\0"
  "BLOCK OCTANT-158\0"
  "BLOCK OCTANT-16\0"
  "BLOCK OCTANT-167\0"
  "BLOCK OCTANT-1678\0"
  "BLOCK OCTANT-168\0"
  "BLOCK OCTANT-17\0"
  "BLOCK OCTANT-178\0"
  "BLOCK OCTANT-18\0"
  "BLOCK OCTANT-23\0"
  "BLOCK OCTANT-234\0"
  "BLOCK OCTANT-2345\0"
  "BLOCK OCTANT-23456\0"
  "BLOCK OCTANT-234567\0"
  "BLOCK OCTANT-2345678\0"
  "BLOCK OCTANT-234568\0"
  "BLOCK OCTANT-23457\0"
  "BLOCK OCTANT-234578\0"
  "BLOCK OCTANT-23458\0"
  "BLOCK OCTANT-2346\0"
  "BLOCK OCTANT-23467\0"
  "BLOCK OCTANT-234678\0"
  "BLOCK OCTANT-23468\0"
  "BLOCK OCTANT-2347\0"
  "BLOCK OCTANT-23478\0"
  "BLOCK OCTANT-2348\0"
  "BLOCK OCTANT-235\0"
  "BLOCK OCTANT-2356\0"
  "BLOCK OCTANT-23567\0"
  "BLOCK OCTANT-235678\0"
  "BLOCK OCTANT-23568\0"
  "BLOCK OCTANT-2357\0"
  "BLOCK OCTANT-23578\0"
  "BLOCK OCTANT-2358\0"
  "BLOCK OCTANT-236\0"
  "BLOCK OCTANT-2367\0"
  "BLOCK OCTANT-23678\0"
  "BLOCK OCTANT-2368\0"
  "BLOCK OCTANT-237\0"
  "BLOCK OCTANT-2378\0"
  "BLOCK OCTANT-238\0"
  "BLOCK OCTANT-245\0"
  "BLOCK OCTANT-2456\0"
  "BLOCK OCTANT-24567\0"
  "BLOCK OCTANT-24568\0"
  "BLOCK OCTANT-24578\0"
  "BLOCK OCTANT-2458\0"
  "BLOCK OCTANT-246\0"
  "BLOCK OCTANT-2467\0"
  "BLOCK OCTANT-24678\0"
  "BLOCK OCTANT-247\0"
  "BLOCK OCTANT-2478\0"
  "BLOCK OCTANT-248\0"
  "BLOCK OCTANT-25\0"
  "BLOCK OCTANT-256\0"
  "BLOCK OCTANT-2567\0"
  "BLOCK OCTANT-25678\0"
  "BLOCK OCTANT-2568\0"
  "BLOCK OCTANT-257\0"
  "BLOCK OCTANT-2578\0"
  "BLOCK OCTANT-258\0"
  "BLOCK OCTANT-26\0"
  "BLOCK OCTANT-267\0"
  "BLOCK OCTANT-2678\0"
  "BLOCK OCTANT-268\0"
  "BLOCK OCTANT-27\0"
  "BLOCK OCTANT-278\0"
  "BLOCK OCTANT-28\0"
  "BLOCK OCTANT-3\0"
  "BLOCK OCTANT-34\0"
  "BLOCK OCTANT-345\0"
  "BLOCK OCTANT-3456\0"
  "BLOCK OCTANT-34567\0"
  "BLOCK OCTANT-34568\0"
  "BLOCK OCTANT-3457\0"
  "BLOCK OCTANT-34578\0"
  "BLOCK OCTANT-3458\0"
  "BLOCK OCTANT-346\0"
  "BLOCK OCTANT-3467\0"
  "BLOCK OCTANT-34678\0"
  "BLOCK OCTANT-3468\0"
  "BLOCK OCTANT-347\0"
  "BLOCK OCTANT-3478\0"
  "BLOCK OCTANT-348\0"
  "BLOCK OCTANT-356\0"
  "BLOCK OCTANT-3567\0"
  "BLOCK OCTANT-35678\0"
  "BLOCK OCTANT-3568\0"
  "BLOCK OCTANT-357\0"
  "BLOCK OCTANT-3578\0"
  "BLOCK OCTANT-358\0"
  "BLOCK OCTANT-36\0"
  "BLOCK OCTANT-367\0"
  "BLOCK OCTANT-3678\0"
  "BLOCK OCTANT-368\0"
  "BLOCK OCTANT-37\0"
  "BLOCK OCTANT-378\0"
  "BLOCK OCTANT-38\0"
  "BLOCK OCTANT-4\0"
  "BLOCK OCTANT-45\0"
  "BLOCK OCTANT-456\0"
  "BLOCK OCTANT-4567\0"
  "BLOCK OCTANT-45678\0"
  "BLOCK OCTANT-4568\0"
  "BLOCK OCTANT-457\0"
  "BLOCK OCTANT-4578\0"
  "BLOCK OCTANT-458\0"
  "BLOCK OCTANT-467\0"
  "BLOCK OCTANT-4678\0"
  "BLOCK OCTANT-468\0"
  "BLOCK OCTANT-47\0"
  "BLOCK OCTANT-478\0"
  "BLOCK OCTANT-48\0"
  "BLOCK OCTANT-5\0"
  "BLOCK OCTANT-56\0"
  "BLOCK OCTANT-567\0"
  "BLOCK OCTANT-568\0"
  "BLOCK OCTANT-578\0"
  "BLOCK OCTANT-58\0"
  "BLOCK OCTANT-6\0"
  "BLOCK OCTANT-67\0"
  "BLOCK OCTANT-678\0"
  "BLOCK SEXTANT-1\0"
  "BLOCK SEXTANT-12\0"
  "BLOCK SEXTANT-123\0"
  "BLOCK SEXTANT-1234\0"
  "BLOCK SEXTANT-12345\0"
  "BLOCK SEXTANT-12346\0"
  "BLOCK SEXTANT-1235\0"
  "BLOCK SEXTANT-12356\0"
  "BLOCK SEXTANT-1236\0"
  "BLOCK SEXTANT-124\0"
  "BLOCK SEXTANT-1245\0"
  "BLOCK SEXTANT-12456\0"
  "BLOCK SEXTANT-1246\0"
  "BLOCK SEXTANT-125\0"
  "BLOCK SEXTANT-1256\0"
  "BLOCK SEXTANT-126\0"
  "BLOCK SEXTANT-13\0"
  "BLOCK SEXTANT-134\0"
  "BLOCK SEXTANT-1345\0"
  "BLOCK SEXTANT-13456\0"
  "BLOCK SEXTANT-1346\0"
  "BLOCK SEXTANT-1356\0"
  "BLOCK SEXTANT-136\0"
  "BLOCK SEXTANT-14\0"
  "BLOCK SEXTANT-145\0"
  "BLOCK SEXTANT-1456\0"
  "BLOCK SEXTANT-146\0"
  "BLOCK SEXTANT-15\0"
  "BLOCK SEXTANT-156\0"
  "BLOCK SEXTANT-16\0"
  "BLOCK SEXTANT-2\0"
  "BLOCK SEXTANT-23\0"
  "BLOCK SEXTANT-234\0"
  "BLOCK SEXTANT-2345\0"
  "BLOCK SEXTANT-23456\0"
  "BLOCK SEXTANT-2346\0"
  "BLOCK SEXTANT-235\0"
  "BLOCK SEXTANT-2356\0"
  "BLOCK SEXTANT-236\0"
  "BLOCK SEXTANT-24\0"
  "BLOCK SEXTANT-245\0"
  "BLOCK SEXTANT-2456\0"
  "BLOCK SEXTANT-25\0"
  "BLOCK SEXTANT-256\0"
  "BLOCK SEXTANT-26\0"
  "BLOCK SEXTANT-3\0"
  "BLOCK SEXTANT-34\0"
  "BLOCK SEXTANT-345\0"
  "BLOCK SEXTANT-3456\0"
  "BLOCK SEXTANT-346\0"
  "BLOCK SEXTANT-35\0"
  "BLOCK SEXTANT-356\0"
  "BLOCK SEXTANT-36\0"
  "BLOCK SEXTANT-4\0"
  "BLOCK SEXTANT-45\0"
  "BLOCK SEXTANT-456\0"
  "BLOCK SEXTANT-46\0"
  "BLOCK SEXTANT-5\0"
  "BLOCK SEXTANT-56\0"
  "BLOCK SEXTANT-6\0"
  "BLOSSOM\0"
  "BLOWFISH\0"
  "BLUE BOOK\0"
  "BLUE HEART\0"
  "BLUEBERRIES\0"
  "BOAR\0"
  "BOLD EIGHT SPOKED ASTERISK\0"
  "BOLD FIVE SPOKED ASTERISK\0"
  "BOLD GREEK CROSS\0"
  "BOLD SALTIRE\0"
  "BOLD SIX SPOKED ASTERISK\0"
  "BOLD WHITE CIRCLE\0"
  "BOLD WHITE SQUARE\0"
  "BOMB\0"
  "BONE\0"
  "BOOK\0"
  "BOOKMARK\0"
  "BOOKMARK TABS\0"
  "BOOKS\0"
  "BOOMERANG\0"
  "BOPOMOFO FINAL LETTER G\0"
  "BOPOMOFO FINAL LETTER H\0"
  "BOPOMOFO FINAL LETTER K\0"
  "BOPOMOFO FINAL LETTER P\0"
  "BOPOMOFO FINAL LETTER T\0"
  "BOPOMOFO LETTER A\0"
  "BOPOMOFO LETTER AH\0"
  "BOPOMOFO LETTER AI\0"
  "BOPOMOFO LETTER AINN\0"
  "BOPOMOFO LETTER AM\0"
  "BOPOMOFO LETTER AN\0"
  "BOPOMOFO LETTER ANG\0"
  "BOPOMOFO LETTER ANN\0"
  "BOPOMOFO LETTER AU\0"
  "BOPOMOFO LETTER AUNN\0"
  "BOPOMOFO LETTER B\0"
  "BOPOMOFO LETTER BU\0"
  "BOPOMOFO LETTER C\0"
  "BOPOMOFO LETTER CH\0"
  "BOPOMOFO LETTER D\0"
  "BOPOMOFO LETTER E\0"
  "BOPOMOFO LETTER EE\0"
  "BOPOMOFO LETTER EH\0"
  "BOPOMOFO LETTER EI\0"
  "BOPOMOFO LETTER EN\0"
  "BOPOMOFO LETTER ENG\0"
  "BOPOMOFO LETTER ENN\0"
  "BOPOMOFO LETTER ER\0"
  "BOPOMOFO LETTER F\0"
  "BOPOMOFO LETTER G\0"
  "BOPOMOFO LETTER GH\0"
  "BOPOMOFO LETTER GN\0"
  "BOPOMOFO LETTER GU\0"
  "BOPOMOFO LETTER GW\0"
  "BOPOMOFO LETTER H\0"
  "BOPOMOFO LETTER I\0"
  "BOPOMOFO LETTER IH\0"
  "BOPOMOFO LETTER IM\0"
  "BOPOMOFO LETTER INN\0"
  "BOPOMOFO LETTER INNN\0"
  "BOPOMOFO LETTER IR\0"
  "BOPOMOFO LETTER IU\0"
  "BOPOMOFO LETTER J\0"
  "BOPOMOFO LETTER JI\0"
  "BOPOMOFO LETTER K\0"
  "BOPOMOFO LETTER KW\0"
  "BOPOMOFO LETTER L\0"
  "BOPOMOFO LETTER LH\0"
  "BOPOMOFO LETTER M\0"
  "BOPOMOFO LETTER N\0"
  "BOPOMOFO LETTER NG\0"
  "BOPOMOFO LETTER NGG\0"
  "BOPOMOFO LETTER NN\0"
  "BOPOMOFO LETTER O\0"
  "BOPOMOFO LETTER O WITH DOT ABOVE\0"
  "BOPOMOFO LETTER OE\0"
  "BOPOMOFO LETTER OM\0"
  "BOPOMOFO LETTER ONG\0"
  "BOPOMOFO LETTER ONN\0"
  "BOPOMOFO LETTER OO\0"
  "BOPOMOFO LETTER OU\0"
  "BOPOMOFO LETTER P\0"
  "BOPOMOFO LETTER Q\0"
  "BOPOMOFO LETTER R\0"
  "BOPOMOFO LETTER S\0"
  "BOPOMOFO LETTER SH\0"
  "BOPOMOFO LETTER T\0"
  "BOPOMOFO LETTER U\0"
  "BOPOMOFO LETTER UNN\0"
  "BOPOMOFO LETTER V\0"
  "BOPOMOFO LETTER X\0"
  "BOPOMOFO LETTER Z\0"
  "BOPOMOFO LETTER ZH\0"
  "BOPOMOFO LETTER ZI\0"
  "BOPOMOFO LETTER ZY\0"
  "BOTTLE WITH POPPING CORK\0"
  "BOTTOM ARC ANTICLOCKWISE ARROW\0"
  "BOTTOM CURLY BRACKET\0"
  "BOTTOM HALF BLACK CIRCLE\0"
  "BOTTOM HALF FORWARD-FACING ROBOT\0"
  "BOTTOM HALF FORWARD-FACING RUNNER FRAME-1\0"
  "BOTTOM HALF FORWARD-FACING RUNNER FRAME-2\0"
  "BOTTOM HALF FORWARD-FACING RUNNER FRAME-3\0"
  "BOTTOM HALF FORWARD-FACING RUNNER FRAME-4\0"
  "BOTTOM HALF INTEGRAL\0"
  "BOTTOM HALF LEFT PARENTHESIS\0"
  "BOTTOM HALF LEFT-FACING ROBOT\0"
  "BOTTOM HALF LEFT-FACING RUNNER FRAME-1\0"
  "BOTTOM HALF LEFT-FACING RUNNER FRAME-2\0"
  "BOTTOM HALF RIGHT PARENTHESIS\0"
  "BOTTOM HALF RIGHT-FACING ROBOT\0"
  "BOTTOM HALF RIGHT-FACING RUNNER FRAME-1\0"
  "BOTTOM HALF RIGHT-FACING RUNNER FRAME-2\0"
  "BOTTOM HALF STANDING PERSON\0"
  "BOTTOM JUSTIFIED UPPER HALF BLACK CIRCLE\0"
  "BOTTOM JUSTIFIED UPPER HALF WHITE CIRCLE\0"
  "BOTTOM LEFT CORNER\0"
  "BOTTOM LEFT CROP\0"
  "BOTTOM LEFT HALF BRACKET\0"
  "BOTTOM LEFT JUSTIFIED UPPER RIGHT QUARTER BLACK CIRCLE\0"
  "BOTTOM PARENTHESIS\0"
  "BOTTOM RIGHT CORNER\0"
  "BOTTOM RIGHT CROP\0"
  "BOTTOM RIGHT HALF BRACKET\0"
  "BOTTOM RIGHT JUSTIFIED UPPER LEFT QUARTER BLACK CIRCLE\0"
  "BOTTOM SQUARE BRACKET\0"
  "BOTTOM SQUARE BRACKET OVER TOP SQUARE BRACKET\0"
  "BOTTOM TORTOISE SHELL BRACKET\0"
  "BOUQUET\0"
  "BOUQUET OF FLOWERS\0"
  "BOW AND ARROW\0"
  "BOWL OF HYGIEIA\0"
  "BOWL WITH SPOON\0"
  "BOWLING\0"
  "BOWTIE\0"
  "BOWTIE WITH LEFT HALF BLACK\0"
  "BOWTIE WITH RIGHT HALF BLACK\0"
  "BOX DRAWINGS DOUBLE DIAGONAL LOWER LEFT TO MIDDLE CENTRE TO LOWER RIGHT\0"
  "BOX DRAWINGS DOUBLE DIAGONAL UPPER LEFT TO LOWER RIGHT\0"
  "BOX DRAWINGS DOUBLE DIAGONAL UPPER LEFT TO MIDDLE CENTRE TO UPPER RIGHT\0"
  "BOX DRAWINGS DOUBLE DIAGONAL UPPER RIGHT TO LOWER LEFT\0"
  "BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL\0"
  "BOX DRAWINGS DOUBLE DOWN AND LEFT\0"
  "BOX DRAWINGS DOUBLE DOWN AND RIGHT\0"
  "BOX DRAWINGS DOUBLE HORIZONTAL\0"
  "BOX DRAWINGS DOUBLE UP AND HORIZONTAL\0"
  "BOX DRAWINGS DOUBLE UP AND LEFT\0"
  "BOX DRAWINGS DOUBLE UP AND RIGHT\0"
  "BOX DRAWINGS DOUBLE VERTICAL\0"
  "BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL\0"
  "BOX DRAWINGS DOUBLE VERTICAL AND LEFT\0"
  "BOX DRAWINGS DOUBLE VERTICAL AND RIGHT\0"
  "BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE\0"
  "BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE\0"
  "BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE\0"
  "BOX DRAWINGS DOWN HEAVY AND HORIZONTAL LIGHT\0"
  "BOX DRAWINGS DOWN HEAVY AND LEFT LIGHT\0"
  "BOX DRAWINGS DOWN HEAVY AND LEFT UP LIGHT\0"
  "BOX DRAWINGS DOWN HEAVY AND RIGHT LIGHT\0"
  "BOX DRAWINGS DOWN HEAVY AND RIGHT UP LIGHT\0"
  "BOX DRAWINGS DOWN HEAVY AND UP HORIZONTAL LIGHT\0"
  "BOX DRAWINGS DOWN LIGHT AND HORIZONTAL HEAVY\0"
  "BOX DRAWINGS DOWN LIGHT AND LEFT HEAVY\0"
  "BOX DRAWINGS DOWN LIGHT AND LEFT UP HEAVY\0"
  "BOX DRAWINGS DOWN LIGHT AND RIGHT HEAVY\0"
  "BOX DRAWINGS DOWN LIGHT AND RIGHT UP HEAVY\0"
  "BOX DRAWINGS DOWN LIGHT AND UP HORIZONTAL HEAVY\0"
  "BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE\0"
  "BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE\0"
  "BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE\0"
  "BOX DRAWINGS HEAVY DOUBLE DASH HORIZONTAL\0"
  "BOX DRAWINGS HEAVY DOUBLE DASH VERTICAL\0"
  "BOX DRAWINGS HEAVY DOWN\0"
  "BOX DRAWINGS HEAVY DOWN AND HORIZONTAL\0"
  "BOX DRAWINGS HEAVY DOWN AND LEFT\0"
  "BOX DRAWINGS HEAVY DOWN AND RIGHT\0"
  "BOX DRAWINGS HEAVY HORIZONTAL\0"
  "BOX DRAWINGS HEAVY LEFT\0"
  "BOX DRAWINGS HEAVY LEFT AND LIGHT RIGHT\0"
  "BOX DRAWINGS HEAVY QUADRUPLE DASH HORIZONTAL\0"
  "BOX DRAWINGS HEAVY QUADRUPLE DASH VERTICAL\0"
  "BOX DRAWINGS HEAVY RIGHT\0"
  "BOX DRAWINGS HEAVY TRIPLE DASH HORIZONTAL\0"
  "BOX DRAWINGS HEAVY TRIPLE DASH VERTICAL\0"
  "BOX DRAWINGS HEAVY UP\0"
  "BOX DRAWINGS HEAVY UP AND HORIZONTAL\0"
  "BOX DRAWINGS HEAVY UP AND LEFT\0"
  "BOX DRAWINGS HEAVY UP AND LIGHT DOWN\0"
  "BOX DRAWINGS HEAVY UP AND RIGHT\0"
  "BOX DRAWINGS HEAVY VERTICAL\0"
  "BOX DRAWINGS HEAVY VERTICAL AND HORIZONTAL\0"
  "BOX DRAWINGS HEAVY VERTICAL AND LEFT\0"
  "BOX DRAWINGS HEAVY VERTICAL AND RIGHT\0"
  "BOX DRAWINGS LEFT DOWN HEAVY AND RIGHT UP LIGHT\0"
  "BOX DRAWINGS LEFT HEAVY AND RIGHT DOWN LIGHT\0"
  "BOX DRAWINGS LEFT HEAVY AND RIGHT UP LIGHT\0"
  "BOX DRAWINGS LEFT HEAVY AND RIGHT VERTICAL LIGHT\0"
  "BOX DRAWINGS LEFT LIGHT AND RIGHT DOWN HEAVY\0"
  "BOX DRAWINGS LEFT LIGHT AND RIGHT UP HEAVY\0"
  "BOX DRAWINGS LEFT LIGHT AND RIGHT VERTICAL HEAVY\0"
  "BOX DRAWINGS LEFT UP HEAVY AND RIGHT DOWN LIGHT\0"
  "BOX DRAWINGS LIGHT ARC DOWN AND LEFT\0"
  "BOX DRAWINGS LIGHT ARC DOWN AND RIGHT\0"
  "BOX DRAWINGS LIGHT ARC UP AND LEFT\0"
  "BOX DRAWINGS LIGHT ARC UP AND RIGHT\0"
  "BOX DRAWINGS LIGHT BOTTOM AND LOWER LEFT\0"
  "BOX DRAWINGS LIGHT DIAGONAL CROSS\0"
  "BOX DRAWINGS LIGHT DIAGONAL DIAMOND\0"
  "BOX DRAWINGS LIGHT DIAGONAL LOWER LEFT TO MIDDLE CENTRE TO LOWER RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL LOWER LEFT TO UPPER CENTRE TO LOWER RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO LOWER CENTRE TO MIDDLE RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO LOWER RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO UPPER CENTRE TO MIDDLE RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO UPPER CENTRE TO MIDDLE RIGHT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL MIDDLE RIGHT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL MIDDLE RIGHT TO LOWER LEFT\0"
  "BOX DRAWINGS LIGHT DIAGONAL MIDDLE RIGHT TO UPPER CENTRE TO MIDDLE LEFT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO LOWER LEFT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO LOWER RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT AND MIDDLE RIGHT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT TO LOWER CENTRE TO MIDDLE RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT AND MIDDLE LEFT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT TO LOWER CENTRE TO MIDDLE LEFT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER CENTRE TO UPPER RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO MIDDLE CENTRE TO LOWER LEFT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO MIDDLE CENTRE TO UPPER RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO MIDDLE RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO MIDDLE RIGHT TO LOWER LEFT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO LOWER CENTRE\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO LOWER LEFT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO MIDDLE CENTRE TO LOWER RIGHT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO MIDDLE LEFT\0"
  "BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO MIDDLE LEFT TO LOWER RIGHT\0"
  "BOX DRAWINGS LIGHT DOUBLE DASH HORIZONTAL\0"
  "BOX DRAWINGS LIGHT DOUBLE DASH VERTICAL\0"
  "BOX DRAWINGS LIGHT DOWN\0"
  "BOX DRAWINGS LIGHT DOWN AND HORIZONTAL\0"
  "BOX DRAWINGS LIGHT DOWN AND LEFT\0"
  "BOX DRAWINGS LIGHT DOWN AND RIGHT\0"
  "BOX DRAWINGS LIGHT HORIZONTAL\0"
  "BOX DRAWINGS LIGHT HORIZONTAL AND LOWER RIGHT\0"
  "BOX DRAWINGS LIGHT HORIZONTAL AND UPPER RIGHT\0"
  "BOX DRAWINGS LIGHT HORIZONTAL WITH VERTICAL STROKE\0"
  "BOX DRAWINGS LIGHT LEFT\0"
  "BOX DRAWINGS LIGHT LEFT AND HEAVY RIGHT\0"
  "BOX DRAWINGS LIGHT QUADRUPLE DASH HORIZONTAL\0"
  "BOX DRAWINGS LIGHT QUADRUPLE DASH VERTICAL\0"
  "BOX DRAWINGS LIGHT RIGHT\0"
  "BOX DRAWINGS LIGHT TOP AND UPPER LEFT\0"
  "BOX DRAWINGS LIGHT TRIPLE DASH HORIZONTAL\0"
  "BOX DRAWINGS LIGHT TRIPLE DASH VERTICAL\0"
  "BOX DRAWINGS LIGHT UP\0"
  "BOX DRAWINGS LIGHT UP AND HEAVY DOWN\0"
  "BOX DRAWINGS LIGHT UP AND HORIZONTAL\0"
  "BOX DRAWINGS LIGHT UP AND LEFT\0"
  "BOX DRAWINGS LIGHT UP AND RIGHT\0"
  "BOX DRAWINGS LIGHT VERTICAL\0"
  "BOX DRAWINGS LIGHT VERTICAL AND BOTTOM LEFT\0"
  "BOX DRAWINGS LIGHT VERTICAL AND BOTTOM RIGHT\0"
  "BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL\0"
  "BOX DRAWINGS LIGHT VERTICAL AND LEFT\0"
  "BOX DRAWINGS LIGHT VERTICAL AND RIGHT\0"
  "BOX DRAWINGS LIGHT VERTICAL AND TOP LEFT\0"
  "BOX DRAWINGS LIGHT VERTICAL AND TOP RIGHT\0"
  "BOX DRAWINGS RIGHT DOWN HEAVY AND LEFT UP LIGHT\0"
  "BOX DRAWINGS RIGHT HEAVY AND LEFT DOWN LIGHT\0"
  "BOX DRAWINGS RIGHT HEAVY AND LEFT UP LIGHT\0"
  "BOX DRAWINGS RIGHT HEAVY AND LEFT VERTICAL LIGHT\0"
  "BOX DRAWINGS RIGHT LIGHT AND LEFT DOWN HEAVY\0"
  "BOX DRAWINGS RIGHT LIGHT AND LEFT UP HEAVY\0"
  "BOX DRAWINGS RIGHT LIGHT AND LEFT VERTICAL HEAVY\0"
  "BOX DRAWINGS RIGHT UP HEAVY AND LEFT DOWN LIGHT\0"
  "BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE\0"
  "BOX DRAWINGS UP DOUBLE AND LEFT SINGLE\0"
  "BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE\0"
  "BOX DRAWINGS UP HEAVY AND DOWN HORIZONTAL LIGHT\0"
  "BOX DRAWINGS UP HEAVY AND HORIZONTAL LIGHT\0"
  "BOX DRAWINGS UP HEAVY AND LEFT DOWN LIGHT\0"
  "BOX DRAWINGS UP HEAVY AND LEFT LIGHT\0"
  "BOX DRAWINGS UP HEAVY AND RIGHT DOWN LIGHT\0"
  "BOX DRAWINGS UP HEAVY AND RIGHT LIGHT\0"
  "BOX DRAWINGS UP LIGHT AND DOWN HORIZONTAL HEAVY\0"
  "BOX DRAWINGS UP LIGHT AND HORIZONTAL HEAVY\0"
  "BOX DRAWINGS UP LIGHT AND LEFT DOWN HEAVY\0"
  "BOX DRAWINGS UP LIGHT AND LEFT HEAVY\0"
  "BOX DRAWINGS UP LIGHT AND RIGHT DOWN HEAVY\0"
  "BOX DRAWINGS UP LIGHT AND RIGHT HEAVY\0"
  "BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE\0"
  "BOX DRAWINGS UP SINGLE AND LEFT DOUBLE\0"
  "BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE\0"
  "BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE\0"
  "BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE\0"
  "BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE\0"
  "BOX DRAWINGS VERTICAL HEAVY AND HORIZONTAL LIGHT\0"
  "BOX DRAWINGS VERTICAL HEAVY AND LEFT LIGHT\0"
  "BOX DRAWINGS VERTICAL HEAVY AND RIGHT LIGHT\0"
  "BOX DRAWINGS VERTICAL LIGHT AND HORIZONTAL HEAVY\0"
  "BOX DRAWINGS VERTICAL LIGHT AND LEFT HEAVY\0"
  "BOX DRAWINGS VERTICAL LIGHT AND RIGHT HEAVY\0"
  "BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE\0"
  "BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE\0"
  "BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE\0"
  "BOXING GLOVE\0"
  "BOY\0"
  "BOYS SYMBOL\0"
  "BRAHMI DANDA\0"
  "BRAHMI DIGIT EIGHT\0"
  "BRAHMI DIGIT FIVE\0"
  "BRAHMI DIGIT FOUR\0"
  "BRAHMI DIGIT NINE\0"
  "BRAHMI DIGIT ONE\0"
  "BRAHMI DIGIT SEVEN\0"
  "BRAHMI DIGIT SIX\0"
  "BRAHMI DIGIT THREE\0"
  "BRAHMI DIGIT TWO\0"
  "BRAHMI DIGIT ZERO\0"
  "BRAHMI DOUBLE DANDA\0"
  "BRAHMI LETTER A\0"
  "BRAHMI LETTER AA\0"
  "BRAHMI LETTER AI\0"
  "BRAHMI LETTER AU\0"
  "BRAHMI LETTER BA\0"
  "BRAHMI LETTER BHA\0"
  "BRAHMI LETTER CA\0"
  "BRAHMI LETTER CHA\0"
  "BRAHMI LETTER DA\0"
  "BRAHMI LETTER DDA\0"
  "BRAHMI LETTER DDHA\0"
  "BRAHMI LETTER DHA\0"
  "BRAHMI LETTER E\0"
  "BRAHMI LETTER GA\0"
  "BRAHMI LETTER GHA\0"
  "BRAHMI LETTER HA\0"
  "BRAHMI LETTER I\0"
  "BRAHMI LETTER II\0"
  "BRAHMI LETTER JA\0"
  "BRAHMI LETTER JHA\0"
  "BRAHMI LETTER KA\0"
  "BRAHMI LETTER KHA\0"
  "BRAHMI LETTER LA\0"
  "BRAHMI LETTER LLA\0"
  "BRAHMI LETTER MA\0"
  "BRAHMI LETTER NA\0"
  "BRAHMI LETTER NGA\0"
  "BRAHMI LETTER NNA\0"
  "BRAHMI LETTER NYA\0"
  "BRAHMI LETTER O\0"
  "BRAHMI LETTER OLD TAMIL LLA\0"
  "BRAHMI LETTER OLD TAMIL LLLA\0"
  "BRAHMI LETTER OLD TAMIL NNNA\0"
  "BRAHMI LETTER OLD TAMIL RRA\0"
  "BRAHMI LETTER OLD TAMIL SHORT E\0"
  "BRAHMI LETTER OLD TAMIL SHORT O\0"
  "BRAHMI LETTER PA\0"
  "BRAHMI LETTER PHA\0"
  "BRAHMI LETTER RA\0"
  "BRAHMI LETTER SA\0"
  "BRAHMI LETTER SHA\0"
  "BRAHMI LETTER SSA\0"
  "BRAHMI LETTER TA\0"
  "BRAHMI LETTER THA\0"
  "BRAHMI LETTER TTA\0"
  "BRAHMI LETTER TTHA\0"
  "BRAHMI LETTER U\0"
  "BRAHMI LETTER UU\0"
  "BRAHMI LETTER VA\0"
  "BRAHMI LETTER VOCALIC L\0"
  "BRAHMI LETTER VOCALIC LL\0"
  "BRAHMI LETTER VOCALIC R\0"
  "BRAHMI LETTER VOCALIC RR\0"
  "BRAHMI LETTER YA\0"
  "BRAHMI NUMBER EIGHT\0"
  "BRAHMI NUMBER EIGHTY\0"
  "BRAHMI NUMBER FIFTY\0"
  "BRAHMI NUMBER FIVE\0"
  "BRAHMI NUMBER FORTY\0"
  "BRAHMI NUMBER FOUR\0"
  "BRAHMI NUMBER JOINER\0"
  "BRAHMI NUMBER NINE\0"
  "BRAHMI NUMBER NINETY\0"
  "BRAHMI NUMBER ONE\0"
  "BRAHMI NUMBER ONE HUNDRED\0"
  "BRAHMI NUMBER ONE THOUSAND\0"
  "BRAHMI NUMBER SEVEN\0"
  "BRAHMI NUMBER SEVENTY\0"
  "BRAHMI NUMBER SIX\0"
  "BRAHMI NUMBER SIXTY\0"
  "BRAHMI NUMBER TEN\0"
  "BRAHMI NUMBER THIRTY\0"
  "BRAHMI NUMBER THREE\0"
  "BRAHMI NUMBER TWENTY\0"
  "BRAHMI NUMBER TWO\0"
  "BRAHMI PUNCTUATION CRESCENT BAR\0"
  "BRAHMI PUNCTUATION DOT\0"
  "BRAHMI PUNCTUATION DOUBLE DOT\0"
  "BRAHMI PUNCTUATION LINE\0"
  "BRAHMI PUNCTUATION LOTUS\0"
  "BRAHMI SIGN ANUSVARA\0"
  "BRAHMI SIGN CANDRABINDU\0"
  "BRAHMI SIGN JIHVAMULIYA\0"
  "BRAHMI SIGN OLD TAMIL VIRAMA\0"
  "BRAHMI SIGN UPADHMANIYA\0"
  "BRAHMI SIGN VISARGA\0"
  "BRAHMI VIRAMA\0"
  "BRAHMI VOWEL SIGN AA\0"
  "BRAHMI VOWEL SIGN AI\0"
  "BRAHMI VOWEL SIGN AU\0"
  "BRAHMI VOWEL SIGN BHATTIPROLU AA\0"
  "BRAHMI VOWEL SIGN E\0"
  "BRAHMI VOWEL SIGN I\0"
  "BRAHMI VOWEL SIGN II\0"
  "BRAHMI VOWEL SIGN O\0"
  "BRAHMI VOWEL SIGN OLD TAMIL SHORT E\0"
  "BRAHMI VOWEL SIGN OLD TAMIL SHORT O\0"
  "BRAHMI VOWEL SIGN U\0"
  "BRAHMI VOWEL SIGN UU\0"
  "BRAHMI VOWEL SIGN VOCALIC L\0"
  "BRAHMI VOWEL SIGN VOCALIC LL\0"
  "BRAHMI VOWEL SIGN VOCALIC R\0"
  "BRAHMI VOWEL SIGN VOCALIC RR\0"
  "BRAILLE PATTERN BLANK\0"
  "BRAILLE PATTERN DOTS-1\0"
  "BRAILLE PATTERN DOTS-12\0"
  "BRAILLE PATTERN DOTS-123\0"
  "BRAILLE PATTERN DOTS-1234\0"
  "BRAILLE PATTERN DOTS-12345\0"
  "BRAILLE PATTERN DOTS-123456\0"
  "BRAILLE PATTERN DOTS-1234567\0"
  "BRAILLE PATTERN DOTS-12345678\0"
  "BRAILLE PATTERN DOTS-1234568\0"
  "BRAILLE PATTERN DOTS-123457\0"
  "BRAILLE PATTERN DOTS-1234578\0"
  "BRAILLE PATTERN DOTS-123458\0"
  "BRAILLE PATTERN DOTS-12346\0"
  "BRAILLE PATTERN DOTS-123467\0"
  "BRAILLE PATTERN DOTS-1234678\0"
  "BRAILLE PATTERN DOTS-123468\0"
  "BRAILLE PATTERN DOTS-12347\0"
  "BRAILLE PATTERN DOTS-123478\0"
  "BRAILLE PATTERN DOTS-12348\0"
  "BRAILLE PATTERN DOTS-1235\0"
  "BRAILLE PATTERN DOTS-12356\0"
  "BRAILLE PATTERN DOTS-123567\0"
  "BRAILLE PATTERN DOTS-1235678\0"
  "BRAILLE PATTERN DOTS-123568\0"
  "BRAILLE PATTERN DOTS-12357\0"
  "BRAILLE PATTERN DOTS-123578\0"
  "BRAILLE PATTERN DOTS-12358\0"
  "BRAILLE PATTERN DOTS-1236\0"
  "BRAILLE PATTERN DOTS-12367\0"
  "BRAILLE PATTERN DOTS-123678\0"
  "BRAILLE PATTERN DOTS-12368\0"
  "BRAILLE PATTERN DOTS-1237\0"
  "BRAILLE PATTERN DOTS-12378\0"
  "BRAILLE PATTERN DOTS-1238\0"
  "BRAILLE PATTERN DOTS-124\0"
  "BRAILLE PATTERN DOTS-1245\0"
  "BRAILLE PATTERN DOTS-12456\0"
  "BRAILLE PATTERN DOTS-124567\0"
  "BRAILLE PATTERN DOTS-1245678\0"
  "BRAILLE PATTERN DOTS-124568\0"
  "BRAILLE PATTERN DOTS-12457\0"
  "BRAILLE PATTERN DOTS-124578\0"
  "BRAILLE PATTERN DOTS-12458\0"
  "BRAILLE PATTERN DOTS-1246\0"
  "BRAILLE PATTERN DOTS-12467\0"
  "BRAILLE PATTERN DOTS-124678\0"
  "BRAILLE PATTERN DOTS-12468\0"
  "BRAILLE PATTERN DOTS-1247\0"
  "BRAILLE PATTERN DOTS-12478\0"
  "BRAILLE PATTERN DOTS-1248\0"
  "BRAILLE PATTERN DOTS-125\0"
  "BRAILLE PATTERN DOTS-1256\0"
  "BRAILLE PATTERN DOTS-12567\0"
  "BRAILLE PATTERN DOTS-125678\0"
  "BRAILLE PATTERN DOTS-12568\0"
  "BRAILLE PATTERN DOTS-1257\0"
  "BRAILLE PATTERN DOTS-12578\0"
  "BRAILLE PATTERN DOTS-1258\0"
  "BRAILLE PATTERN DOTS-126\0"
  "BRAILLE PATTERN DOTS-1267\0"
  "BRAILLE PATTERN DOTS-12678\0"
  "BRAILLE PATTERN DOTS-1268\0"
  "BRAILLE PATTERN DOTS-127\0"
  "BRAILLE PATTERN DOTS-1278\0"
  "BRAILLE PATTERN DOTS-128\0"
  "BRAILLE PATTERN DOTS-13\0"
  "BRAILLE PATTERN DOTS-134\0"
  "BRAILLE PATTERN DOTS-1345\0"
  "BRAILLE PATTERN DOTS-13456\0"
  "BRAILLE PATTERN DOTS-134567\0"
  "BRAILLE PATTERN DOTS-1345678\0"
  "BRAILLE PATTERN DOTS-134568\0"
  "BRAILLE PATTERN DOTS-13457\0"
  "BRAILLE PATTERN DOTS-134578\0"
  "BRAILLE PATTERN DOTS-13458\0"
  "BRAILLE PATTERN DOTS-1346\0"
  "BRAILLE PATTERN DOTS-13467\0"
  "BRAILLE PATTERN DOTS-134678\0"
  "BRAILLE PATTERN DOTS-13468\0"
  "BRAILLE PATTERN DOTS-1347\0"
  "BRAILLE PATTERN DOTS-13478\0"
  "BRAILLE PATTERN DOTS-1348\0"
  "BRAILLE PATTERN DOTS-135\0"
  "BRAILLE PATTERN DOTS-1356\0"
  "BRAILLE PATTERN DOTS-13567\0"
  "BRAILLE PATTERN DOTS-135678\0"
  "BRAILLE PATTERN DOTS-13568\0"
  "BRAILLE PATTERN DOTS-1357\0"
  "BRAILLE PATTERN DOTS-13578\0"
  "BRAILLE PATTERN DOTS-1358\0"
  "BRAILLE PATTERN DOTS-136\0"
  "BRAILLE PATTERN DOTS-1367\0"
  "BRAILLE PATTERN DOTS-13678\0"
  "BRAILLE PATTERN DOTS-1368\0"
  "BRAILLE PATTERN DOTS-137\0"
  "BRAILLE PATTERN DOTS-1378\0"
  "BRAILLE PATTERN DOTS-138\0"
  "BRAILLE PATTERN DOTS-14\0"
  "BRAILLE PATTERN DOTS-145\0"
  "BRAILLE PATTERN DOTS-1456\0"
  "BRAILLE PATTERN DOTS-14567\0"
  "BRAILLE PATTERN DOTS-145678\0"
  "BRAILLE PATTERN DOTS-14568\0"
  "BRAILLE PATTERN DOTS-1457\0"
  "BRAILLE PATTERN DOTS-14578\0"
  "BRAILLE PATTERN DOTS-1458\0"
  "BRAILLE PATTERN DOTS-146\0"
  "BRAILLE PATTERN DOTS-1467\0"
  "BRAILLE PATTERN DOTS-14678\0"
  "BRAILLE PATTERN DOTS-1468\0"
  "BRAILLE PATTERN DOTS-147\0"
  "BRAILLE PATTERN DOTS-1478\0"
  "BRAILLE PATTERN DOTS-148\0"
  "BRAILLE PATTERN DOTS-15\0"
  "BRAILLE PATTERN DOTS-156\0"
  "BRAILLE PATTERN DOTS-1567\0"
  "BRAILLE PATTERN DOTS-15678\0"
  "BRAILLE PATTERN DOTS-1568\0"
  "BRAILLE PATTERN DOTS-157\0"
  "BRAILLE PATTERN DOTS-1578\0"
  "BRAILLE PATTERN DOTS-158\0"
  "BRAILLE PATTERN DOTS-16\0"
  "BRAILLE PATTERN DOTS-167\0"
  "BRAILLE PATTERN DOTS-1678\0"
  "BRAILLE PATTERN DOTS-168\0"
  "BRAILLE PATTERN DOTS-17\0"
  "BRAILLE PATTERN DOTS-178\0"
  "BRAILLE PATTERN DOTS-18\0"
  "BRAILLE PATTERN DOTS-2\0"
  "BRAILLE PATTERN DOTS-23\0"
  "BRAILLE PATTERN DOTS-234\0"
  "BRAILLE PATTERN DOTS-2345\0"
  "BRAILLE PATTERN DOTS-23456\0"
  "BRAILLE PATTERN DOTS-234567\0"
  "BRAILLE PATTERN DOTS-2345678\0"
  "BRAILLE PATTERN DOTS-234568\0"
  "BRAILLE PATTERN DOTS-23457\0"
  "BRAILLE PATTERN DOTS-234578\0"
  "BRAILLE PATTERN DOTS-23458\0"
  "BRAILLE PATTERN DOTS-2346\0"
  "BRAILLE PATTERN DOTS-23467\0"
  "BRAILLE PATTERN DOTS-234678\0"
  "BRAILLE PATTERN DOTS-23468\0"
  "BRAILLE PATTERN DOTS-2347\0"
  "BRAILLE PATTERN DOTS-23478\0"
  "BRAILLE PATTERN DOTS-2348\0"
  "BRAILLE PATTERN DOTS-235\0"
  "BRAILLE PATTERN DOTS-2356\0"
  "BRAILLE PATTERN DOTS-23567\0"
  "BRAILLE PATTERN DOTS-235678\0"
  "BRAILLE PATTERN DOTS-23568\0"
  "BRAILLE PATTERN DOTS-2357\0"
  "BRAILLE PATTERN DOTS-23578\0"
  "BRAILLE PATTERN DOTS-2358\0"
  "BRAILLE PATTERN DOTS-236\0"
  "BRAILLE PATTERN DOTS-2367\0"
  "BRAILLE PATTERN DOTS-23678\0"
  "BRAILLE PATTERN DOTS-2368\0"
  "BRAILLE PATTERN DOTS-237\0"
  "BRAILLE PATTERN DOTS-2378\0"
  "BRAILLE PATTERN DOTS-238\0"
  "BRAILLE PATTERN DOTS-24\0"
  "BRAILLE PATTERN DOTS-245\0"
  "BRAILLE PATTERN DOTS-2456\0"
  "BRAILLE PATTERN DOTS-24567\0"
  "BRAILLE PATTERN DOTS-245678\0"
  "BRAILLE PATTERN DOTS-24568\0"
  "BRAILLE PATTERN DOTS-2457\0"
  "BRAILLE PATTERN DOTS-24578\0"
  "BRAILLE PATTERN DOTS-2458\0"
  "BRAILLE PATTERN DOTS-246\0"
  "BRAILLE PATTERN DOTS-2467\0"
  "BRAILLE PATTERN DOTS-24678\0"
  "BRAILLE PATTERN DOTS-2468\0"
  "BRAILLE PATTERN DOTS-247\0"
  "BRAILLE PATTERN DOTS-2478\0"
  "BRAILLE PATTERN DOTS-248\0"
  "BRAILLE PATTERN DOTS-25\0"
  "BRAILLE PATTERN DOTS-256\0"
  "BRAILLE PATTERN DOTS-2567\0"
  "BRAILLE PATTERN DOTS-25678\0"
  "BRAILLE PATTERN DOTS-2568\0"
  "BRAILLE PATTERN DOTS-257\0"
  "BRAILLE PATTERN DOTS-2578\0"
  "BRAILLE PATTERN DOTS-258\0"
  "BRAILLE PATTERN DOTS-26\0"
  "BRAILLE PATTERN DOTS-267\0"
  "BRAILLE PATTERN DOTS-2678\0"
  "BRAILLE PATTERN DOTS-268\0"
  "BRAILLE PATTERN DOTS-27\0"
  "BRAILLE PATTERN DOTS-278\0"
  "BRAILLE PATTERN DOTS-28\0"
  "BRAILLE PATTERN DOTS-3\0"
  "BRAILLE PATTERN DOTS-34\0"
  "BRAILLE PATTERN DOTS-345\0"
  "BRAILLE PATTERN DOTS-3456\0"
  "BRAILLE PATTERN DOTS-34567\0"
  "BRAILLE PATTERN DOTS-345678\0"
  "BRAILLE PATTERN DOTS-34568\0"
  "BRAILLE PATTERN DOTS-3457\0"
  "BRAILLE PATTERN DOTS-34578\0"
  "BRAILLE PATTERN DOTS-3458\0"
  "BRAILLE PATTERN DOTS-346\0"
  "BRAILLE PATTERN DOTS-3467\0"
  "BRAILLE PATTERN DOTS-34678\0"
  "BRAILLE PATTERN DOTS-3468\0"
  "BRAILLE PATTERN DOTS-347\0"
  "BRAILLE PATTERN DOTS-3478\0"
  "BRAILLE PATTERN DOTS-348\0"
  "BRAILLE PATTERN DOTS-35\0"
  "BRAILLE PATTERN DOTS-356\0"
  "BRAILLE PATTERN DOTS-3567\0"
  "BRAILLE PATTERN DOTS-35678\0"
  "BRAILLE PATTERN DOTS-3568\0"
  "BRAILLE PATTERN DOTS-357\0"
  "BRAILLE PATTERN DOTS-3578\0"
  "BRAILLE PATTERN DOTS-358\0"
  "BRAILLE PATTERN DOTS-36\0"
  "BRAILLE PATTERN DOTS-367\0"
  "BRAILLE PATTERN DOTS-3678\0"
  "BRAILLE PATTERN DOTS-368\0"
  "BRAILLE PATTERN DOTS-37\0"
  "BRAILLE PATTERN DOTS-378\0"
  "BRAILLE PATTERN DOTS-38\0"
  "BRAILLE PATTERN DOTS-4\0"
  "BRAILLE PATTERN DOTS-45\0"
  "BRAILLE PATTERN DOTS-456\0"
  "BRAILLE PATTERN DOTS-4567\0"
  "BRAILLE PATTERN DOTS-45678\0"
  "BRAILLE PATTERN DOTS-4568\0"
  "BRAILLE PATTERN DOTS-457\0"
  "BRAILLE PATTERN DOTS-4578\0"
  "BRAILLE PATTERN DOTS-458\0"
  "BRAILLE PATTERN DOTS-46\0"
  "BRAILLE PATTERN DOTS-467\0"
  "BRAILLE PATTERN DOTS-4678\0"
  "BRAILLE PATTERN DOTS-468\0"
  "BRAILLE PATTERN DOTS-47\0"
  "BRAILLE PATTERN DOTS-478\0"
  "BRAILLE PATTERN DOTS-48\0"
  "BRAILLE PATTERN DOTS-5\0"
  "BRAILLE PATTERN DOTS-56\0"
  "BRAILLE PATTERN DOTS-567\0"
  "BRAILLE PATTERN DOTS-5678\0"
  "BRAILLE PATTERN DOTS-568\0"
  "BRAILLE PATTERN DOTS-57\0"
  "BRAILLE PATTERN DOTS-578\0"
  "BRAILLE PATTERN DOTS-58\0"
  "BRAILLE PATTERN DOTS-6\0"
  "BRAILLE PATTERN DOTS-67\0"
  "BRAILLE PATTERN DOTS-678\0"
  "BRAILLE PATTERN DOTS-68\0"
  "BRAILLE PATTERN DOTS-7\0"
  "BRAILLE PATTERN DOTS-78\0"
  "BRAILLE PATTERN DOTS-8\0"
  "BRAIN\0"
  "BREAD\0"
  "BREAST-FEEDING\0"
  "BREVE\0"
  "BRICK\0"
  "BRIDE WITH VEIL\0"
  "BRIDGE AT NIGHT\0"
  "BRIEFCASE\0"
  "BRIEFS\0"
  "BROCCOLI\0"
  "BROKEN BAR\0"
  "BROKEN CIRCLE WITH NORTHWEST ARROW\0"
  "BROKEN HEART\0"
  "BROOM\0"
  "BROWN HEART\0"
  "BUBBLE TEA\0"
  "BUBBLES\0"
  "BUCKET\0"
  "BUG\0"
  "BUGINESE END OF SECTION\0"
  "BUGINESE LETTER A\0"
  "BUGINESE LETTER BA\0"
  "BUGINESE LETTER CA\0"
  "BUGINESE LETTER DA\0"
  "BUGINESE LETTER GA\0"
  "BUGINESE LETTER HA\0"
  "BUGINESE LETTER JA\0"
  "BUGINESE LETTER KA\0"
  "BUGINESE LETTER LA\0"
  "BUGINESE LETTER MA\0"
  "BUGINESE LETTER MPA\0"
  "BUGINESE LETTER NA\0"
  "BUGINESE LETTER NGA\0"
  "BUGINESE LETTER NGKA\0"
  "BUGINESE LETTER NRA\0"
  "BUGINESE LETTER NYA\0"
  "BUGINESE LETTER NYCA\0"
  "BUGINESE LETTER PA\0"
  "BUGINESE LETTER RA\0"
  "BUGINESE LETTER SA\0"
  "BUGINESE LETTER TA\0"
  "BUGINESE LETTER VA\0"
  "BUGINESE LETTER YA\0"
  "BUGINESE PALLAWA\0"
  "BUGINESE VOWEL SIGN AE\0"
  "BUGINESE VOWEL SIGN E\0"
  "BUGINESE VOWEL SIGN I\0"
  "BUGINESE VOWEL SIGN O\0"
  "BUGINESE VOWEL SIGN U\0"
  "BUHID LETTER A\0"
  "BUHID LETTER BA\0"
  "BUHID LETTER DA\0"
  "BUHID LETTER GA\0"
  "BUHID LETTER HA\0"
  "BUHID LETTER I\0"
  "BUHID LETTER KA\0"
  "BUHID LETTER LA\0"
  "BUHID LETTER MA\0"
  "BUHID LETTER NA\0"
  "BUHID LETTER NGA\0"
  "BUHID LETTER PA\0"
  "BUHID LETTER RA\0"
  "BUHID LETTER SA\0"
  "BUHID LETTER TA\0"
  "BUHID LETTER U\0"
  "BUHID LETTER WA\0"
  "BUHID LETTER YA\0"
  "BUHID VOWEL SIGN I\0"
  "BUHID VOWEL SIGN U\0"
  "BUILDING CONSTRUCTION\0"
  "BULLET\0"
  "BULLET OPERATOR\0"
  "BULLHORN\0"
  "BULLHORN WITH SOUND WAVES\0"
  "BULLSEYE\0"
  "BURRITO\0"
  "BUS\0"
  "BUS STOP\0"
  "BUST IN SILHOUETTE\0"
  "BUSTS IN SILHOUETTE\0"
  "BUTTER\0"
  "BUTTERFLY\0"
  "BYZANTINE MUSICAL SYMBOL AGOGI ARGI\0"
  "BYZANTINE MUSICAL SYMBOL AGOGI ARGOTERI\0"
  "BYZANTINE MUSICAL SYMBOL AGOGI GORGI\0"
  "BYZANTINE MUSICAL SYMBOL AGOGI GORGOTERI\0"
  "BYZANTINE MUSICAL SYMBOL AGOGI MESI\0"
  "BYZANTINE MUSICAL SYMBOL AGOGI METRIA\0"
  "BYZANTINE MUSICAL SYMBOL AGOGI POLI ARGI\0"
  "BYZANTINE MUSICAL SYMBOL AGOGI POLI GORGI\0"
  "BYZANTINE MUSICAL SYMBOL ANATRICHISMA\0"
  "BYZANTINE MUSICAL SYMBOL ANTIKENOKYLISMA\0"
  "BYZANTINE MUSICAL SYMBOL ANTIKENOMA\0"
  "BYZANTINE MUSICAL SYMBOL APESO EKFONITIKON\0"
  "BYZANTINE MUSICAL SYMBOL APESO EXO NEO\0"
  "BYZANTINE MUSICAL SYMBOL APLI\0"
  "BYZANTINE MUSICAL SYMBOL APODERMA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL APODERMA NEO\0"
  "BYZANTINE MUSICAL SYMBOL APOSTROFOI SYNDESMOS NEO\0"
  "BYZANTINE MUSICAL SYMBOL APOSTROFOI TELOUS ICHIMATOS\0"
  "BYZANTINE MUSICAL SYMBOL APOSTROFOS\0"
  "BYZANTINE MUSICAL SYMBOL APOSTROFOS DIPLI\0"
  "BYZANTINE MUSICAL SYMBOL APOSTROFOS NEO\0"
  "BYZANTINE MUSICAL SYMBOL APOTHEMA\0"
  "BYZANTINE MUSICAL SYMBOL ARGON\0"
  "BYZANTINE MUSICAL SYMBOL ARGOSYNTHETON\0"
  "BYZANTINE MUSICAL SYMBOL ARKTIKO DI\0"
  "BYZANTINE MUSICAL SYMBOL ARKTIKO GA\0"
  "BYZANTINE MUSICAL SYMBOL ARKTIKO KE\0"
  "BYZANTINE MUSICAL SYMBOL ARKTIKO NI\0"
  "BYZANTINE MUSICAL SYMBOL ARKTIKO PA\0"
  "BYZANTINE MUSICAL SYMBOL ARKTIKO VOU\0"
  "BYZANTINE MUSICAL SYMBOL ARKTIKO ZO\0"
  "BYZANTINE MUSICAL SYMBOL CHAMILI\0"
  "BYZANTINE MUSICAL SYMBOL CHAMILON\0"
  "BYZANTINE MUSICAL SYMBOL CHOREVMA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL CHOREVMA NEO\0"
  "BYZANTINE MUSICAL SYMBOL CHROA KLITON\0"
  "BYZANTINE MUSICAL SYMBOL CHROA SPATHI\0"
  "BYZANTINE MUSICAL SYMBOL CHROA ZYGOS\0"
  "BYZANTINE MUSICAL SYMBOL DASEIA\0"
  "BYZANTINE MUSICAL SYMBOL DIARGON\0"
  "BYZANTINE MUSICAL SYMBOL DIASTOLI APLI MEGALI\0"
  "BYZANTINE MUSICAL SYMBOL DIASTOLI APLI MIKRI\0"
  "BYZANTINE MUSICAL SYMBOL DIASTOLI DIPLI\0"
  "BYZANTINE MUSICAL SYMBOL DIASTOLI THESEOS\0"
  "BYZANTINE MUSICAL SYMBOL DIESIS APLI DYO DODEKATA\0"
  "BYZANTINE MUSICAL SYMBOL DIESIS DIGRAMMOS EX DODEKATA\0"
  "BYZANTINE MUSICAL SYMBOL DIESIS MONOGRAMMOS TESSERA DODEKATA\0"
  "BYZANTINE MUSICAL SYMBOL DIESIS TETARTIMORION\0"
  "BYZANTINE MUSICAL SYMBOL DIESIS TRIGRAMMOS OKTO DODEKATA\0"
  "BYZANTINE MUSICAL SYMBOL DIESIS TRITIMORION\0"
  "BYZANTINE MUSICAL SYMBOL DIFTOGGOS OU\0"
  "BYZANTINE MUSICAL SYMBOL DIGORGON\0"
  "BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON ARISTERA ANO\0"
  "BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON ARISTERA KATO\0"
  "BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON DEXIA\0"
  "BYZANTINE MUSICAL SYMBOL DIGRAMMA GG\0"
  "BYZANTINE MUSICAL SYMBOL DIPLI\0"
  "BYZANTINE MUSICAL SYMBOL DIPLI ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL DYO\0"
  "BYZANTINE MUSICAL SYMBOL EKSTREPTON\0"
  "BYZANTINE MUSICAL SYMBOL ELAFRON\0"
  "BYZANTINE MUSICAL SYMBOL ENARXIS KAI FTHORA VOU\0"
  "BYZANTINE MUSICAL SYMBOL ENDOFONON\0"
  "BYZANTINE MUSICAL SYMBOL EPEGERMA\0"
  "BYZANTINE MUSICAL SYMBOL ETERON ARGOSYNTHETON\0"
  "BYZANTINE MUSICAL SYMBOL ETERON PARAKALESMA\0"
  "BYZANTINE MUSICAL SYMBOL EXO EKFONITIKON\0"
  "BYZANTINE MUSICAL SYMBOL FANEROSIS DIFONIAS\0"
  "BYZANTINE MUSICAL SYMBOL FANEROSIS MONOFONIAS\0"
  "BYZANTINE MUSICAL SYMBOL FANEROSIS TETRAFONIAS\0"
  "BYZANTINE MUSICAL SYMBOL FHTORA SKLIRON CHROMA VASIS\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA ARCHAION DEYTEROU ICHOU\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI DI\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI KE\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NANA\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NI ANO\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NI KATO\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI PA\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI ZO\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA ENARMONIOS ANTIFONIA\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA I YFESIS TETARTIMORION\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA MALAKON CHROMA DIFONIAS\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA MALAKON CHROMA MONOFONIAS\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA NAOS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA NENANO\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA SKLIRON CHROMA SYNAFI\0"
  "BYZANTINE MUSICAL SYMBOL FTHORA SKLIRON DIATONON DI\0"
  "BYZANTINE MUSICAL SYMBOL GENIKI DIESIS\0"
  "BYZANTINE MUSICAL SYMBOL GENIKI YFESIS\0"
  "BYZANTINE MUSICAL SYMBOL GORGON ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL GORGON NEO ANO\0"
  "BYZANTINE MUSICAL SYMBOL GORGON NEO KATO\0"
  "BYZANTINE MUSICAL SYMBOL GORGON PARESTIGMENON ARISTERA\0"
  "BYZANTINE MUSICAL SYMBOL GORGON PARESTIGMENON DEXIA\0"
  "BYZANTINE MUSICAL SYMBOL GORGOSYNTHETON\0"
  "BYZANTINE MUSICAL SYMBOL GORTHMIKON N APLOUN\0"
  "BYZANTINE MUSICAL SYMBOL GORTHMIKON N DIPLOUN\0"
  "BYZANTINE MUSICAL SYMBOL GRONTHISMATA\0"
  "BYZANTINE MUSICAL SYMBOL ICHADIN\0"
  "BYZANTINE MUSICAL SYMBOL IMIDIARGON\0"
  "BYZANTINE MUSICAL SYMBOL IMIFONON\0"
  "BYZANTINE MUSICAL SYMBOL IMIFTHORA\0"
  "BYZANTINE MUSICAL SYMBOL IMIFTHORON\0"
  "BYZANTINE MUSICAL SYMBOL ISAKIA TELOUS ICHIMATOS\0"
  "BYZANTINE MUSICAL SYMBOL ISON ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL ISON NEO\0"
  "BYZANTINE MUSICAL SYMBOL KATAVA TROMIKON\0"
  "BYZANTINE MUSICAL SYMBOL KATAVASMA\0"
  "BYZANTINE MUSICAL SYMBOL KATHISTI\0"
  "BYZANTINE MUSICAL SYMBOL KENTIMA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL KENTIMA NEO ANO\0"
  "BYZANTINE MUSICAL SYMBOL KENTIMA NEO KATO\0"
  "BYZANTINE MUSICAL SYMBOL KENTIMA NEO MESO\0"
  "BYZANTINE MUSICAL SYMBOL KENTIMATA\0"
  "BYZANTINE MUSICAL SYMBOL KENTIMATA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL KENTIMATA NEO ANO\0"
  "BYZANTINE MUSICAL SYMBOL KENTIMATA NEO KATO\0"
  "BYZANTINE MUSICAL SYMBOL KENTIMATA NEO MESO\0"
  "BYZANTINE MUSICAL SYMBOL KLASMA\0"
  "BYZANTINE MUSICAL SYMBOL KLASMA ANO\0"
  "BYZANTINE MUSICAL SYMBOL KLASMA KATO\0"
  "BYZANTINE MUSICAL SYMBOL KONTEVMA\0"
  "BYZANTINE MUSICAL SYMBOL KONTEVMA ALLO\0"
  "BYZANTINE MUSICAL SYMBOL KORONIS\0"
  "BYZANTINE MUSICAL SYMBOL KOUFISMA\0"
  "BYZANTINE MUSICAL SYMBOL KRATIMA ALLO\0"
  "BYZANTINE MUSICAL SYMBOL KRATIMA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL KRATIMA NEO\0"
  "BYZANTINE MUSICAL SYMBOL KRATIMATA\0"
  "BYZANTINE MUSICAL SYMBOL KRATIMOKOUFISMA\0"
  "BYZANTINE MUSICAL SYMBOL KRATIMOYPORROON\0"
  "BYZANTINE MUSICAL SYMBOL KREMASTI\0"
  "BYZANTINE MUSICAL SYMBOL KYLISMA\0"
  "BYZANTINE MUSICAL SYMBOL LEIMMA DYO CHRONON\0"
  "BYZANTINE MUSICAL SYMBOL LEIMMA ENOS CHRONOU\0"
  "BYZANTINE MUSICAL SYMBOL LEIMMA IMISEOS CHRONOU\0"
  "BYZANTINE MUSICAL SYMBOL LEIMMA TESSARON CHRONON\0"
  "BYZANTINE MUSICAL SYMBOL LEIMMA TRION CHRONON\0"
  "BYZANTINE MUSICAL SYMBOL LEMOI\0"
  "BYZANTINE MUSICAL SYMBOL LYGISMA\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA ALLI DEYTEROS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA ALLI PROTOS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA DEYTEROS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA LEGETOS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA PLAGIOS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA PLAGIOS TETARTOS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA PROTOS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA PROTOVARYS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA TETARTOS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA TETARTOS LEGETOS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA TRIFONIAS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA TRITOS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MARTYRIA VARYS ICHOS\0"
  "BYZANTINE MUSICAL SYMBOL MIKRON ISON\0"
  "BYZANTINE MUSICAL SYMBOL NANA\0"
  "BYZANTINE MUSICAL SYMBOL OLIGON ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL OLIGON NEO\0"
  "BYZANTINE MUSICAL SYMBOL OMALON\0"
  "BYZANTINE MUSICAL SYMBOL OXEIA DIPLI\0"
  "BYZANTINE MUSICAL SYMBOL OXEIA EKFONITIKON\0"
  "BYZANTINE MUSICAL SYMBOL OXEIA NEO\0"
  "BYZANTINE MUSICAL SYMBOL OXEIAI ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL OYRANISMA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL OYRANISMA NEO\0"
  "BYZANTINE MUSICAL SYMBOL PARAKALESMA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL PARAKALESMA NEO\0"
  "BYZANTINE MUSICAL SYMBOL PARAKLITIKI\0"
  "BYZANTINE MUSICAL SYMBOL PARAKLITIKI ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL PARAKLITIKI NEO\0"
  "BYZANTINE MUSICAL SYMBOL PARICHON\0"
  "BYZANTINE MUSICAL SYMBOL PELASTON\0"
  "BYZANTINE MUSICAL SYMBOL PELASTON NEO\0"
  "BYZANTINE MUSICAL SYMBOL PERISPOMENI\0"
  "BYZANTINE MUSICAL SYMBOL PETASMA\0"
  "BYZANTINE MUSICAL SYMBOL PETASTI\0"
  "BYZANTINE MUSICAL SYMBOL PETASTOKOUFISMA\0"
  "BYZANTINE MUSICAL SYMBOL PIASMA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL PIASMA NEO\0"
  "BYZANTINE MUSICAL SYMBOL PSIFISTOLYGISMA\0"
  "BYZANTINE MUSICAL SYMBOL PSIFISTON\0"
  "BYZANTINE MUSICAL SYMBOL PSIFISTON NEO\0"
  "BYZANTINE MUSICAL SYMBOL PSIFISTOPARAKALESMA\0"
  "BYZANTINE MUSICAL SYMBOL PSIFISTOSYNAGMA\0"
  "BYZANTINE MUSICAL SYMBOL PSILI\0"
  "BYZANTINE MUSICAL SYMBOL PSILON\0"
  "BYZANTINE MUSICAL SYMBOL RAPISMA\0"
  "BYZANTINE MUSICAL SYMBOL REVMA\0"
  "BYZANTINE MUSICAL SYMBOL SAXIMATA\0"
  "BYZANTINE MUSICAL SYMBOL SEISMA\0"
  "BYZANTINE MUSICAL SYMBOL SEISMA NEO\0"
  "BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS\0"
  "BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS DISIMOU\0"
  "BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS TETRASIMOU\0"
  "BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS TRISIMOU\0"
  "BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS\0"
  "BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS DISIMOU\0"
  "BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS TETRASIMOU\0"
  "BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS TRISIMOU\0"
  "BYZANTINE MUSICAL SYMBOL STAVROS\0"
  "BYZANTINE MUSICAL SYMBOL STAVROS APODEXIA\0"
  "BYZANTINE MUSICAL SYMBOL STIGMA\0"
  "BYZANTINE MUSICAL SYMBOL STRAGGISMATA\0"
  "BYZANTINE MUSICAL SYMBOL SYNAGMA ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL SYNAGMA META STAVROU\0"
  "BYZANTINE MUSICAL SYMBOL SYNAGMA NEO\0"
  "BYZANTINE MUSICAL SYMBOL SYNEVMA\0"
  "BYZANTINE MUSICAL SYMBOL SYRMA\0"
  "BYZANTINE MUSICAL SYMBOL SYRMATIKI\0"
  "BYZANTINE MUSICAL SYMBOL TELEIA\0"
  "BYZANTINE MUSICAL SYMBOL TESSERA\0"
  "BYZANTINE MUSICAL SYMBOL TETRAPLI\0"
  "BYZANTINE MUSICAL SYMBOL THEMA\0"
  "BYZANTINE MUSICAL SYMBOL THEMA APLOUN\0"
  "BYZANTINE MUSICAL SYMBOL THEMATISMOS ESO\0"
  "BYZANTINE MUSICAL SYMBOL THEMATISMOS EXO\0"
  "BYZANTINE MUSICAL SYMBOL THES KAI APOTHES\0"
  "BYZANTINE MUSICAL SYMBOL THITA\0"
  "BYZANTINE MUSICAL SYMBOL TINAGMA\0"
  "BYZANTINE MUSICAL SYMBOL TRIA\0"
  "BYZANTINE MUSICAL SYMBOL TRIGORGON\0"
  "BYZANTINE MUSICAL SYMBOL TRIPLI\0"
  "BYZANTINE MUSICAL SYMBOL TROMIKOLYGISMA\0"
  "BYZANTINE MUSICAL SYMBOL TROMIKON ALLO\0"
  "BYZANTINE MUSICAL SYMBOL TROMIKON ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL TROMIKON NEO\0"
  "BYZANTINE MUSICAL SYMBOL TROMIKOPARAKALESMA\0"
  "BYZANTINE MUSICAL SYMBOL TROMIKOPSIFISTON\0"
  "BYZANTINE MUSICAL SYMBOL TROMIKOSYNAGMA\0"
  "BYZANTINE MUSICAL SYMBOL VAREIA DIPLI\0"
  "BYZANTINE MUSICAL SYMBOL VAREIA EKFONITIKON\0"
  "BYZANTINE MUSICAL SYMBOL VAREIA NEO\0"
  "BYZANTINE MUSICAL SYMBOL VAREIAI ARCHAION\0"
  "BYZANTINE MUSICAL SYMBOL VATHY\0"
  "BYZANTINE MUSICAL SYMBOL XIRON KLASMA\0"
  "BYZANTINE MUSICAL SYMBOL YFEN ANO\0"
  "BYZANTINE MUSICAL SYMBOL YFEN KATO\0"
  "BYZANTINE MUSICAL SYMBOL YFESIS APLI DYO DODEKATA\0"
  "BYZANTINE MUSICAL SYMBOL YFESIS DIGRAMMOS EX DODEKATA\0"
  "BYZANTINE MUSICAL SYMBOL YFESIS MONOGRAMMOS TESSERA DODEKATA\0"
  "BYZANTINE MUSICAL SYMBOL YFESIS TRIGRAMMOS OKTO DODEKATA\0"
  "BYZANTINE MUSICAL SYMBOL YFESIS TRITIMORION\0"
  "BYZANTINE MUSICAL SYMBOL YPOKRISIS\0"
  "BYZANTINE MUSICAL SYMBOL YPOKRISIS DIPLI\0"
  "BYZANTINE MUSICAL SYMBOL YPORROI\0"
  "BYZANTINE MUSICAL SYMBOL YPSILI\0"
  "CACTUS\0"
  "CADA UNA\0"
  "CADUCEUS\0"
  "CALENDAR\0"
  "CALL ME HAND\0"
  "CAMERA\0"
  "CAMERA WITH FLASH\0"
  "CAMPING\0"
  "CANADIAN SYLLABICS A\0"
  "CANADIAN SYLLABICS AA\0"
  "CANADIAN SYLLABICS AAI\0"
  "CANADIAN SYLLABICS AAY\0"
  "CANADIAN SYLLABICS AI\0"
  "CANADIAN SYLLABICS AIVILIK B\0"
  "CANADIAN SYLLABICS AN\0"
  "CANADIAN SYLLABICS ATHAPASCAN M\0"
  "CANADIAN SYLLABICS ATHAPASCAN S\0"
  "CANADIAN SYLLABICS AY\0"
  "CANADIAN SYLLABICS BEAVER DENE L\0"
  "CANADIAN SYLLABICS BEAVER DENE R\0"
  "CANADIAN SYLLABICS BIBLE-CREE Y\0"
  "CANADIAN SYLLABICS BLACKFOOT A\0"
  "CANADIAN SYLLABICS BLACKFOOT E\0"
  "CANADIAN SYLLABICS BLACKFOOT I\0"
  "CANADIAN SYLLABICS BLACKFOOT KA\0"
  "CANADIAN SYLLABICS BLACKFOOT KE\0"
  "CANADIAN SYLLABICS BLACKFOOT KI\0"
  "CANADIAN SYLLABICS BLACKFOOT KO\0"
  "CANADIAN SYLLABICS BLACKFOOT NA\0"
  "CANADIAN SYLLABICS BLACKFOOT NE\0"
  "CANADIAN SYLLABICS BLACKFOOT NI\0"
  "CANADIAN SYLLABICS BLACKFOOT NO\0"
  "CANADIAN SYLLABICS BLACKFOOT O\0"
  "CANADIAN SYLLABICS BLACKFOOT S\0"
  "CANADIAN SYLLABICS BLACKFOOT W\0"
  "CANADIAN SYLLABICS BLACKFOOT WA\0"
  "CANADIAN SYLLABICS BLACKFOOT WE\0"
  "CANADIAN SYLLABICS BLACKFOOT WI\0"
  "CANADIAN SYLLABICS BLACKFOOT WO\0"
  "CANADIAN SYLLABICS C\0"
  "CANADIAN SYLLABICS CA\0"
  "CANADIAN SYLLABICS CAA\0"
  "CANADIAN SYLLABICS CAAI\0"
  "CANADIAN SYLLABICS CARRIER CHA\0"
  "CANADIAN SYLLABICS CARRIER CHE\0"
  "CANADIAN SYLLABICS CARRIER CHEE\0"
  "CANADIAN SYLLABICS CARRIER CHI\0"
  "CANADIAN SYLLABICS CARRIER CHO\0"
  "CANADIAN SYLLABICS CARRIER CHU\0"
  "CANADIAN SYLLABICS CARRIER DEE\0"
  "CANADIAN SYLLABICS CARRIER DENE GEE\0"
  "CANADIAN SYLLABICS CARRIER DENTAL S\0"
  "CANADIAN SYLLABICS CARRIER DI\0"
  "CANADIAN SYLLABICS CARRIER DLA\0"
  "CANADIAN SYLLABICS CARRIER DLE\0"
  "CANADIAN SYLLABICS CARRIER DLEE\0"
  "CANADIAN SYLLABICS CARRIER DLI\0"
  "CANADIAN SYLLABICS CARRIER DLO\0"
  "CANADIAN SYLLABICS CARRIER DLU\0"
  "CANADIAN SYLLABICS CARRIER DZA\0"
  "CANADIAN SYLLABICS CARRIER DZE\0"
  "CANADIAN SYLLABICS CARRIER DZEE\0"
  "CANADIAN SYLLABICS CARRIER DZI\0"
  "CANADIAN SYLLABICS CARRIER DZO\0"
  "CANADIAN SYLLABICS CARRIER DZU\0"
  "CANADIAN SYLLABICS CARRIER EE\0"
  "CANADIAN SYLLABICS CARRIER GA\0"
  "CANADIAN SYLLABICS CARRIER GAA\0"
  "CANADIAN SYLLABICS CARRIER GE\0"
  "CANADIAN SYLLABICS CARRIER GEE\0"
  "CANADIAN SYLLABICS CARRIER GHA\0"
  "CANADIAN SYLLABICS CARRIER GHE\0"
  "CANADIAN SYLLABICS CARRIER GHEE\0"
  "CANADIAN SYLLABICS CARRIER GHI\0"
  "CANADIAN SYLLABICS CARRIER GHO\0"
  "CANADIAN SYLLABICS CARRIER GHU\0"
  "CANADIAN SYLLABICS CARRIER GI\0"
  "CANADIAN SYLLABICS CARRIER GO\0"
  "CANADIAN SYLLABICS CARRIER GU\0"
  "CANADIAN SYLLABICS CARRIER GWA\0"
  "CANADIAN SYLLABICS CARRIER GWU\0"
  "CANADIAN SYLLABICS CARRIER H\0"
  "CANADIAN SYLLABICS CARRIER HEE\0"
  "CANADIAN SYLLABICS CARRIER HI\0"
  "CANADIAN SYLLABICS CARRIER HWA\0"
  "CANADIAN SYLLABICS CARRIER HWE\0"
  "CANADIAN SYLLABICS CARRIER HWEE\0"
  "CANADIAN SYLLABICS CARRIER HWI\0"
  "CANADIAN SYLLABICS CARRIER HWO\0"
  "CANADIAN SYLLABICS CARRIER HWU\0"
  "CANADIAN SYLLABICS CARRIER I\0"
  "CANADIAN SYLLABICS CARRIER INITIAL Z\0"
  "CANADIAN SYLLABICS CARRIER JA\0"
  "CANADIAN SYLLABICS CARRIER JE\0"
  "CANADIAN SYLLABICS CARRIER JEE\0"
  "CANADIAN SYLLABICS CARRIER JI\0"
  "CANADIAN SYLLABICS CARRIER JJA\0"
  "CANADIAN SYLLABICS CARRIER JJE\0"
  "CANADIAN SYLLABICS CARRIER JJEE\0"
  "CANADIAN SYLLABICS CARRIER JJI\0"
  "CANADIAN SYLLABICS CARRIER JJO\0"
  "CANADIAN SYLLABICS CARRIER JJU\0"
  "CANADIAN SYLLABICS CARRIER JO\0"
  "CANADIAN SYLLABICS CARRIER JU\0"
  "CANADIAN SYLLABICS CARRIER JWA\0"
  "CANADIAN SYLLABICS CARRIER KHA\0"
  "CANADIAN SYLLABICS CARRIER KHE\0"
  "CANADIAN SYLLABICS CARRIER KHEE\0"
  "CANADIAN SYLLABICS CARRIER KHI\0"
  "CANADIAN SYLLABICS CARRIER KHO\0"
  "CANADIAN SYLLABICS CARRIER KHU\0"
  "CANADIAN SYLLABICS CARRIER KK\0"
  "CANADIAN SYLLABICS CARRIER KKA\0"
  "CANADIAN SYLLABICS CARRIER KKE\0"
  "CANADIAN SYLLABICS CARRIER KKEE\0"
  "CANADIAN SYLLABICS CARRIER KKI\0"
  "CANADIAN SYLLABICS CARRIER KKO\0"
  "CANADIAN SYLLABICS CARRIER KKU\0"
  "CANADIAN SYLLABICS CARRIER LA\0"
  "CANADIAN SYLLABICS CARRIER LE\0"
  "CANADIAN SYLLABICS CARRIER LEE\0"
  "CANADIAN SYLLABICS CARRIER LHA\0"
  "CANADIAN SYLLABICS CARRIER LHE\0"
  "CANADIAN SYLLABICS CARRIER LHEE\0"
  "CANADIAN SYLLABICS CARRIER LHI\0"
  "CANADIAN SYLLABICS CARRIER LHO\0"
  "CANADIAN SYLLABICS CARRIER LHU\0"
  "CANADIAN SYLLABICS CARRIER LI\0"
  "CANADIAN SYLLABICS CARRIER LO\0"
  "CANADIAN SYLLABICS CARRIER LU\0"
  "CANADIAN SYLLABICS CARRIER MA\0"
  "CANADIAN SYLLABICS CARRIER ME\0"
  "CANADIAN SYLLABICS CARRIER MEE\0"
  "CANADIAN SYLLABICS CARRIER MI\0"
  "CANADIAN SYLLABICS CARRIER MO\0"
  "CANADIAN SYLLABICS CARRIER MU\0"
  "CANADIAN SYLLABICS CARRIER NA\0"
  "CANADIAN SYLLABICS CARRIER NE\0"
  "CANADIAN SYLLABICS CARRIER NEE\0"
  "CANADIAN SYLLABICS CARRIER NG\0"
  "CANADIAN SYLLABICS CARRIER NI\0"
  "CANADIAN SYLLABICS CARRIER NO\0"
  "CANADIAN SYLLABICS CARRIER NU\0"
  "CANADIAN SYLLABICS CARRIER P\0"
  "CANADIAN SYLLABICS CARRIER PA\0"
  "CANADIAN SYLLABICS CARRIER PE\0"
  "CANADIAN SYLLABICS CARRIER PEE\0"
  "CANADIAN SYLLABICS CARRIER PI\0"
  "CANADIAN SYLLABICS CARRIER PO\0"
  "CANADIAN SYLLABICS CARRIER PU\0"
  "CANADIAN SYLLABICS CARRIER RA\0"
  "CANADIAN SYLLABICS CARRIER RE\0"
  "CANADIAN SYLLABICS CARRIER REE\0"
  "CANADIAN SYLLABICS CARRIER RI\0"
  "CANADIAN SYLLABICS CARRIER RO\0"
  "CANADIAN SYLLABICS CARRIER RU\0"
  "CANADIAN SYLLABICS CARRIER SA\0"
  "CANADIAN SYLLABICS CARRIER SE\0"
  "CANADIAN SYLLABICS CARRIER SEE\0"
  "CANADIAN SYLLABICS CARRIER SH\0"
  "CANADIAN SYLLABICS CARRIER SHA\0"
  "CANADIAN SYLLABICS CARRIER SHE\0"
  "CANADIAN SYLLABICS CARRIER SHEE\0"
  "CANADIAN SYLLABICS CARRIER SHI\0"
  "CANADIAN SYLLABICS CARRIER SHO\0"
  "CANADIAN SYLLABICS CARRIER SHU\0"
  "CANADIAN SYLLABICS CARRIER SI\0"
  "CANADIAN SYLLABICS CARRIER SO\0"
  "CANADIAN SYLLABICS CARRIER SU\0"
  "CANADIAN SYLLABICS CARRIER THA\0"
  "CANADIAN SYLLABICS CARRIER THE\0"
  "CANADIAN SYLLABICS CARRIER THEE\0"
  "CANADIAN SYLLABICS CARRIER THI\0"
  "CANADIAN SYLLABICS CARRIER THO\0"
  "CANADIAN SYLLABICS CARRIER THU\0"
  "CANADIAN SYLLABICS CARRIER TLA\0"
  "CANADIAN SYLLABICS CARRIER TLE\0"
  "CANADIAN SYLLABICS CARRIER TLEE\0"
  "CANADIAN SYLLABICS CARRIER TLHA\0"
  "CANADIAN SYLLABICS CARRIER TLHE\0"
  "CANADIAN SYLLABICS CARRIER TLHEE\0"
  "CANADIAN SYLLABICS CARRIER TLHI\0"
  "CANADIAN SYLLABICS CARRIER TLHO\0"
  "CANADIAN SYLLABICS CARRIER TLHU\0"
  "CANADIAN SYLLABICS CARRIER TLI\0"
  "CANADIAN SYLLABICS CARRIER TLO\0"
  "CANADIAN SYLLABICS CARRIER TLU\0"
  "CANADIAN SYLLABICS CARRIER TSA\0"
  "CANADIAN SYLLABICS CARRIER TSE\0"
  "CANADIAN SYLLABICS CARRIER TSEE\0"
  "CANADIAN SYLLABICS CARRIER TSI\0"
  "CANADIAN SYLLABICS CARRIER TSO\0"
  "CANADIAN SYLLABICS CARRIER TSU\0"
  "CANADIAN SYLLABICS CARRIER TTA\0"
  "CANADIAN SYLLABICS CARRIER TTE\0"
  "CANADIAN SYLLABICS CARRIER TTEE\0"
  "CANADIAN SYLLABICS CARRIER TTI\0"
  "CANADIAN SYLLABICS CARRIER TTO\0"
  "CANADIAN SYLLABICS CARRIER TTSA\0"
  "CANADIAN SYLLABICS CARRIER TTSE\0"
  "CANADIAN SYLLABICS CARRIER TTSEE\0"
  "CANADIAN SYLLABICS CARRIER TTSI\0"
  "CANADIAN SYLLABICS CARRIER TTSO\0"
  "CANADIAN SYLLABICS CARRIER TTSU\0"
  "CANADIAN SYLLABICS CARRIER TTU\0"
  "CANADIAN SYLLABICS CARRIER WA\0"
  "CANADIAN SYLLABICS CARRIER WE\0"
  "CANADIAN SYLLABICS CARRIER WEE\0"
  "CANADIAN SYLLABICS CARRIER WI\0"
  "CANADIAN SYLLABICS CARRIER WO\0"
  "CANADIAN SYLLABICS CARRIER WU\0"
  "CANADIAN SYLLABICS CARRIER YA\0"
  "CANADIAN SYLLABICS CARRIER YE\0"
  "CANADIAN SYLLABICS CARRIER YEE\0"
  "CANADIAN SYLLABICS CARRIER YI\0"
  "CANADIAN SYLLABICS CARRIER YO\0"
  "CANADIAN SYLLABICS CARRIER YU\0"
  "CANADIAN SYLLABICS CARRIER Z\0"
  "CANADIAN SYLLABICS CARRIER ZA\0"
  "CANADIAN SYLLABICS CARRIER ZE\0"
  "CANADIAN SYLLABICS CARRIER ZEE\0"
  "CANADIAN SYLLABICS CARRIER ZI\0"
  "CANADIAN SYLLABICS CARRIER ZO\0"
  "CANADIAN SYLLABICS CARRIER ZU\0"
  "CANADIAN SYLLABICS CE\0"
  "CANADIAN SYLLABICS CHI SIGN\0"
  "CANADIAN SYLLABICS CI\0"
  "CANADIAN SYLLABICS CII\0"
  "CANADIAN SYLLABICS CO\0"
  "CANADIAN SYLLABICS COO\0"
  "CANADIAN SYLLABICS CWA\0"
  "CANADIAN SYLLABICS CWAA\0"
  "CANADIAN SYLLABICS CWE\0"
  "CANADIAN SYLLABICS CWI\0"
  "CANADIAN SYLLABICS CWII\0"
  "CANADIAN SYLLABICS CWO\0"
  "CANADIAN SYLLABICS CWOO\0"
  "CANADIAN SYLLABICS E\0"
  "CANADIAN SYLLABICS EASTERN W\0"
  "CANADIAN SYLLABICS EN\0"
  "CANADIAN SYLLABICS F\0"
  "CANADIAN SYLLABICS FA\0"
  "CANADIAN SYLLABICS FAA\0"
  "CANADIAN SYLLABICS FAAI\0"
  "CANADIAN SYLLABICS FE\0"
  "CANADIAN SYLLABICS FI\0"
  "CANADIAN SYLLABICS FII\0"
  "CANADIAN SYLLABICS FINAL ACUTE\0"
  "CANADIAN SYLLABICS FINAL BOTTOM HALF RING\0"
  "CANADIAN SYLLABICS FINAL DOUBLE ACUTE\0"
  "CANADIAN SYLLABICS FINAL DOUBLE SHORT VERTICAL STROKES\0"
  "CANADIAN SYLLABICS FINAL DOWN TACK\0"
  "CANADIAN SYLLABICS FINAL GRAVE\0"
  "CANADIAN SYLLABICS FINAL MIDDLE DOT\0"
  "CANADIAN SYLLABICS FINAL PLUS\0"
  "CANADIAN SYLLABICS FINAL RAISED DOT\0"
  "CANADIAN SYLLABICS FINAL RIGHT HALF RING\0"
  "CANADIAN SYLLABICS FINAL RING\0"
  "CANADIAN SYLLABICS FINAL SHORT HORIZONTAL STROKE\0"
  "CANADIAN SYLLABICS FINAL SMALL RING\0"
  "CANADIAN SYLLABICS FINAL TOP HALF RING\0"
  "CANADIAN SYLLABICS FO\0"
  "CANADIAN SYLLABICS FOO\0"
  "CANADIAN SYLLABICS FULL STOP\0"
  "CANADIAN SYLLABICS FWAA\0"
  "CANADIAN SYLLABICS GLOTTAL STOP\0"
  "CANADIAN SYLLABICS HK\0"
  "CANADIAN SYLLABICS HYPHEN\0"
  "CANADIAN SYLLABICS I\0"
  "CANADIAN SYLLABICS II\0"
  "CANADIAN SYLLABICS IN\0"
  "CANADIAN SYLLABICS K\0"
  "CANADIAN SYLLABICS KA\0"
  "CANADIAN SYLLABICS KAA\0"
  "CANADIAN SYLLABICS KAAI\0"
  "CANADIAN SYLLABICS KAY\0"
  "CANADIAN SYLLABICS KE\0"
  "CANADIAN SYLLABICS KI\0"
  "CANADIAN SYLLABICS KII\0"
  "CANADIAN SYLLABICS KO\0"
  "CANADIAN SYLLABICS KOO\0"
  "CANADIAN SYLLABICS KW\0"
  "CANADIAN SYLLABICS KWA\0"
  "CANADIAN SYLLABICS KWAA\0"
  "CANADIAN SYLLABICS KWAY\0"
  "CANADIAN SYLLABICS KWE\0"
  "CANADIAN SYLLABICS KWI\0"
  "CANADIAN SYLLABICS KWII\0"
  "CANADIAN SYLLABICS KWO\0"
  "CANADIAN SYLLABICS KWOO\0"
  "CANADIAN SYLLABICS L\0"
  "CANADIAN SYLLABICS LA\0"
  "CANADIAN SYLLABICS LAA\0"
  "CANADIAN SYLLABICS LAAI\0"
  "CANADIAN SYLLABICS LAY\0"
  "CANADIAN SYLLABICS LE\0"
  "CANADIAN SYLLABICS LH\0"
  "CANADIAN SYLLABICS LHA\0"
  "CANADIAN SYLLABICS LHAA\0"
  "CANADIAN SYLLABICS LHI\0"
  "CANADIAN SYLLABICS LHII\0"
  "CANADIAN SYLLABICS LHO\0"
  "CANADIAN SYLLABICS LHOO\0"
  "CANADIAN SYLLABICS LI\0"
  "CANADIAN SYLLABICS LII\0"
  "CANADIAN SYLLABICS LO\0"
  "CANADIAN SYLLABICS LOO\0"
  "CANADIAN SYLLABICS LWA\0"
  "CANADIAN SYLLABICS LWAA\0"
  "CANADIAN SYLLABICS LWE\0"
  "CANADIAN SYLLABICS LWI\0"
  "CANADIAN SYLLABICS LWII\0"
  "CANADIAN SYLLABICS LWO\0"
  "CANADIAN SYLLABICS LWOO\0"
  "CANADIAN SYLLABICS M\0"
  "CANADIAN SYLLABICS MA\0"
  "CANADIAN SYLLABICS MAA\0"
  "CANADIAN SYLLABICS MAAI\0"
  "CANADIAN SYLLABICS MAY\0"
  "CANADIAN SYLLABICS ME\0"
  "CANADIAN SYLLABICS MEDIAL L\0"
  "CANADIAN SYLLABICS MEDIAL R\0"
  "CANADIAN SYLLABICS MH\0"
  "CANADIAN SYLLABICS MI\0"
  "CANADIAN SYLLABICS MII\0"
  "CANADIAN SYLLABICS MO\0"
  "CANADIAN SYLLABICS MOO\0"
  "CANADIAN SYLLABICS MOOSE-CREE SK\0"
  "CANADIAN SYLLABICS MWA\0"
  "CANADIAN SYLLABICS MWAA\0"
  "CANADIAN SYLLABICS MWE\0"
  "CANADIAN SYLLABICS MWI\0"
  "CANADIAN SYLLABICS MWII\0"
  "CANADIAN SYLLABICS MWO\0"
  "CANADIAN SYLLABICS MWOO\0"
  "CANADIAN SYLLABICS N\0"
  "CANADIAN SYLLABICS N-CREE THE\0"
  "CANADIAN SYLLABICS N-CREE THI\0"
  "CANADIAN SYLLABICS N-CREE THII\0"
  "CANADIAN SYLLABICS NA\0"
  "CANADIAN SYLLABICS NAA\0"
  "CANADIAN SYLLABICS NAAI\0"
  "CANADIAN SYLLABICS NASKAPI CWAA\0"
  "CANADIAN SYLLABICS NASKAPI KWAA\0"
  "CANADIAN SYLLABICS NASKAPI MWAA\0"
  "CANADIAN SYLLABICS NASKAPI NWAA\0"
  "CANADIAN SYLLABICS NASKAPI S-W\0"
  "CANADIAN SYLLABICS NASKAPI SCWA\0"
  "CANADIAN SYLLABICS NASKAPI SKW\0"
  "CANADIAN SYLLABICS NASKAPI SKWA\0"
  "CANADIAN SYLLABICS NASKAPI SPWA\0"
  "CANADIAN SYLLABICS NASKAPI STWA\0"
  "CANADIAN SYLLABICS NASKAPI SWAA\0"
  "CANADIAN SYLLABICS NASKAPI TWAA\0"
  "CANADIAN SYLLABICS NASKAPI WAA\0"
  "CANADIAN SYLLABICS NASKAPI WOO\0"
  "CANADIAN SYLLABICS NASKAPI YWAA\0"
  "CANADIAN SYLLABICS NATTILIK HA\0"
  "CANADIAN SYLLABICS NATTILIK HAA\0"
  "CANADIAN SYLLABICS NATTILIK HI\0"
  "CANADIAN SYLLABICS NATTILIK HII\0"
  "CANADIAN SYLLABICS NATTILIK HO\0"
  "CANADIAN SYLLABICS NATTILIK HOO\0"
  "CANADIAN SYLLABICS NATTILIK SHRA\0"
  "CANADIAN SYLLABICS NATTILIK SHRAA\0"
  "CANADIAN SYLLABICS NATTILIK SHRI\0"
  "CANADIAN SYLLABICS NATTILIK SHRII\0"
  "CANADIAN SYLLABICS NATTILIK SHRO\0"
  "CANADIAN SYLLABICS NATTILIK SHROO\0"
  "CANADIAN SYLLABICS NAY\0"
  "CANADIAN SYLLABICS NE\0"
  "CANADIAN SYLLABICS NG\0"
  "CANADIAN SYLLABICS NGA\0"
  "CANADIAN SYLLABICS NGAA\0"
  "CANADIAN SYLLABICS NGAAI\0"
  "CANADIAN SYLLABICS NGAI\0"
  "CANADIAN SYLLABICS NGI\0"
  "CANADIAN SYLLABICS NGII\0"
  "CANADIAN SYLLABICS NGO\0"
  "CANADIAN SYLLABICS NGOO\0"
  "CANADIAN SYLLABICS NH\0"
  "CANADIAN SYLLABICS NI\0"
  "CANADIAN SYLLABICS NII\0"
  "CANADIAN SYLLABICS NNG\0"
  "CANADIAN SYLLABICS NNGA\0"
  "CANADIAN SYLLABICS NNGAA\0"
  "CANADIAN SYLLABICS NNGI\0"
  "CANADIAN SYLLABICS NNGII\0"
  "CANADIAN SYLLABICS NNGO\0"
  "CANADIAN SYLLABICS NNGOO\0"
  "CANADIAN SYLLABICS NO\0"
  "CANADIAN SYLLABICS NOO\0"
  "CANADIAN SYLLABICS NOY\0"
  "CANADIAN SYLLABICS NUNAVIK H\0"
  "CANADIAN SYLLABICS NUNAVIK HA\0"
  "CANADIAN SYLLABICS NUNAVIK HAA\0"
  "CANADIAN SYLLABICS NUNAVIK HE\0"
  "CANADIAN SYLLABICS NUNAVIK HI\0"
  "CANADIAN SYLLABICS NUNAVIK HII\0"
  "CANADIAN SYLLABICS NUNAVIK HO\0"
  "CANADIAN SYLLABICS NUNAVIK HOO\0"
  "CANADIAN SYLLABICS NUNAVUT H\0"
  "CANADIAN SYLLABICS NWA\0"
  "CANADIAN SYLLABICS NWAA\0"
  "CANADIAN SYLLABICS NWE\0"
  "CANADIAN SYLLABICS NWI\0"
  "CANADIAN SYLLABICS NWII\0"
  "CANADIAN SYLLABICS NWO\0"
  "CANADIAN SYLLABICS NWOO\0"
  "CANADIAN SYLLABICS O\0"
  "CANADIAN SYLLABICS OJIBWAY C\0"
  "CANADIAN SYLLABICS OJIBWAY K\0"
  "CANADIAN SYLLABICS OJIBWAY M\0"
  "CANADIAN SYLLABICS OJIBWAY N\0"
  "CANADIAN SYLLABICS OJIBWAY NWI\0"
  "CANADIAN SYLLABICS OJIBWAY NWII\0"
  "CANADIAN SYLLABICS OJIBWAY NWO\0"
  "CANADIAN SYLLABICS OJIBWAY NWOO\0"
  "CANADIAN SYLLABICS OJIBWAY P\0"
  "CANADIAN SYLLABICS OJIBWAY S\0"
  "CANADIAN SYLLABICS OJIBWAY SH\0"
  "CANADIAN SYLLABICS OJIBWAY T\0"
  "CANADIAN SYLLABICS ON\0"
  "CANADIAN SYLLABICS OO\0"
  "CANADIAN SYLLABICS OY\0"
  "CANADIAN SYLLABICS P\0"
  "CANADIAN SYLLABICS PA\0"
  "CANADIAN SYLLABICS PAA\0"
  "CANADIAN SYLLABICS PAAI\0"
  "CANADIAN SYLLABICS PAY\0"
  "CANADIAN SYLLABICS PE\0"
  "CANADIAN SYLLABICS PI\0"
  "CANADIAN SYLLABICS PII\0"
  "CANADIAN SYLLABICS PO\0"
  "CANADIAN SYLLABICS POO\0"
  "CANADIAN SYLLABICS POY\0"
  "CANADIAN SYLLABICS PWA\0"
  "CANADIAN SYLLABICS PWAA\0"
  "CANADIAN SYLLABICS PWE\0"
  "CANADIAN SYLLABICS PWI\0"
  "CANADIAN SYLLABICS PWII\0"
  "CANADIAN SYLLABICS PWO\0"
  "CANADIAN SYLLABICS PWOO\0"
  "CANADIAN SYLLABICS PWOY\0"
  "CANADIAN SYLLABICS Q\0"
  "CANADIAN SYLLABICS QA\0"
  "CANADIAN SYLLABICS QAA\0"
  "CANADIAN SYLLABICS QAAI\0"
  "CANADIAN SYLLABICS QAI\0"
  "CANADIAN SYLLABICS QI\0"
  "CANADIAN SYLLABICS QII\0"
  "CANADIAN SYLLABICS QO\0"
  "CANADIAN SYLLABICS QOO\0"
  "CANADIAN SYLLABICS R\0"
  "CANADIAN SYLLABICS R-CREE RE\0"
  "CANADIAN SYLLABICS R-CREE RWE\0"
  "CANADIAN SYLLABICS RA\0"
  "CANADIAN SYLLABICS RAA\0"
  "CANADIAN SYLLABICS RAAI\0"
  "CANADIAN SYLLABICS RAY\0"
  "CANADIAN SYLLABICS RE\0"
  "CANADIAN SYLLABICS RI\0"
  "CANADIAN SYLLABICS RII\0"
  "CANADIAN SYLLABICS RO\0"
  "CANADIAN SYLLABICS ROO\0"
  "CANADIAN SYLLABICS RWA\0"
  "CANADIAN SYLLABICS RWAA\0"
  "CANADIAN SYLLABICS RWEE\0"
  "CANADIAN SYLLABICS RWI\0"
  "CANADIAN SYLLABICS RWII\0"
  "CANADIAN SYLLABICS RWO\0"
  "CANADIAN SYLLABICS RWOO\0"
  "CANADIAN SYLLABICS S\0"
  "CANADIAN SYLLABICS SA\0"
  "CANADIAN SYLLABICS SAA\0"
  "CANADIAN SYLLABICS SAAI\0"
  "CANADIAN SYLLABICS SAY\0"
  "CANADIAN SYLLABICS SAYISI HA\0"
  "CANADIAN SYLLABICS SAYISI HE\0"
  "CANADIAN SYLLABICS SAYISI HI\0"
  "CANADIAN SYLLABICS SAYISI HO\0"
  "CANADIAN SYLLABICS SAYISI HOO\0"
  "CANADIAN SYLLABICS SAYISI JI\0"
  "CANADIAN SYLLABICS SAYISI JU\0"
  "CANADIAN SYLLABICS SAYISI JUU\0"
  "CANADIAN SYLLABICS SAYISI M\0"
  "CANADIAN SYLLABICS SAYISI SHA\0"
  "CANADIAN SYLLABICS SAYISI SHE\0"
  "CANADIAN SYLLABICS SAYISI SHI\0"
  "CANADIAN SYLLABICS SAYISI SHO\0"
  "CANADIAN SYLLABICS SAYISI SHOO\0"
  "CANADIAN SYLLABICS SAYISI SHWE\0"
  "CANADIAN SYLLABICS SAYISI TH\0"
  "CANADIAN SYLLABICS SAYISI YI\0"
  "CANADIAN SYLLABICS SE\0"
  "CANADIAN SYLLABICS SH\0"
  "CANADIAN SYLLABICS SHA\0"
  "CANADIAN SYLLABICS SHAA\0"
  "CANADIAN SYLLABICS SHAY\0"
  "CANADIAN SYLLABICS SHE\0"
  "CANADIAN SYLLABICS SHI\0"
  "CANADIAN SYLLABICS SHII\0"
  "CANADIAN SYLLABICS SHO\0"
  "CANADIAN SYLLABICS SHOO\0"
  "CANADIAN SYLLABICS SHOY\0"
  "CANADIAN SYLLABICS SHWA\0"
  "CANADIAN SYLLABICS SHWAA\0"
  "CANADIAN SYLLABICS SHWE\0"
  "CANADIAN SYLLABICS SHWI\0"
  "CANADIAN SYLLABICS SHWII\0"
  "CANADIAN SYLLABICS SHWO\0"
  "CANADIAN SYLLABICS SHWOO\0"
  "CANADIAN SYLLABICS SHWOY\0"
  "CANADIAN SYLLABICS SI\0"
  "CANADIAN SYLLABICS SII\0"
  "CANADIAN SYLLABICS SO\0"
  "CANADIAN SYLLABICS SOO\0"
  "CANADIAN SYLLABICS SOUTH-SLAVEY KAH\0"
  "CANADIAN SYLLABICS SOUTH-SLAVEY KEH\0"
  "CANADIAN SYLLABICS SOUTH-SLAVEY KIH\0"
  "CANADIAN SYLLABICS SOUTH-SLAVEY KOH\0"
  "CANADIAN SYLLABICS SOY\0"
  "CANADIAN SYLLABICS SPA\0"
  "CANADIAN SYLLABICS SPE\0"
  "CANADIAN SYLLABICS SPI\0"
  "CANADIAN SYLLABICS SPO\0"
  "CANADIAN SYLLABICS SW\0"
  "CANADIAN SYLLABICS SWA\0"
  "CANADIAN SYLLABICS SWAA\0"
  "CANADIAN SYLLABICS SWE\0"
  "CANADIAN SYLLABICS SWI\0"
  "CANADIAN SYLLABICS SWII\0"
  "CANADIAN SYLLABICS SWO\0"
  "CANADIAN SYLLABICS SWOO\0"
  "CANADIAN SYLLABICS T\0"
  "CANADIAN SYLLABICS TA\0"
  "CANADIAN SYLLABICS TAA\0"
  "CANADIAN SYLLABICS TAAI\0"
  "CANADIAN SYLLABICS TAY\0"
  "CANADIAN SYLLABICS TE\0"
  "CANADIAN SYLLABICS TH\0"
  "CANADIAN SYLLABICS TH-CREE TH\0"
  "CANADIAN SYLLABICS TH-CREE THA\0"
  "CANADIAN SYLLABICS TH-CREE THAA\0"
  "CANADIAN SYLLABICS TH-CREE THE\0"
  "CANADIAN SYLLABICS TH-CREE THI\0"
  "CANADIAN SYLLABICS TH-CREE THII\0"
  "CANADIAN SYLLABICS TH-CREE THO\0"
  "CANADIAN SYLLABICS TH-CREE THOO\0"
  "CANADIAN SYLLABICS THA\0"
  "CANADIAN SYLLABICS THAA\0"
  "CANADIAN SYLLABICS THE\0"
  "CANADIAN SYLLABICS THI\0"
  "CANADIAN SYLLABICS THII\0"
  "CANADIAN SYLLABICS THO\0"
  "CANADIAN SYLLABICS THOO\0"
  "CANADIAN SYLLABICS THWA\0"
  "CANADIAN SYLLABICS THWAA\0"
  "CANADIAN SYLLABICS THWE\0"
  "CANADIAN SYLLABICS TI\0"
  "CANADIAN SYLLABICS TII\0"
  "CANADIAN SYLLABICS TLHA\0"
  "CANADIAN SYLLABICS TLHE\0"
  "CANADIAN SYLLABICS TLHI\0"
  "CANADIAN SYLLABICS TLHO\0"
  "CANADIAN SYLLABICS TLHOO\0"
  "CANADIAN SYLLABICS TLHWE\0"
  "CANADIAN SYLLABICS TO\0"
  "CANADIAN SYLLABICS TOO\0"
  "CANADIAN SYLLABICS TTA\0"
  "CANADIAN SYLLABICS TTE\0"
  "CANADIAN SYLLABICS TTH\0"
  "CANADIAN SYLLABICS TTHA\0"
  "CANADIAN SYLLABICS TTHAA\0"
  "CANADIAN SYLLABICS TTHE\0"
  "CANADIAN SYLLABICS TTHI\0"
  "CANADIAN SYLLABICS TTHO\0"
  "CANADIAN SYLLABICS TTHOO\0"
  "CANADIAN SYLLABICS TTHWE\0"
  "CANADIAN SYLLABICS TTI\0"
  "CANADIAN SYLLABICS TTO\0"
  "CANADIAN SYLLABICS TWA\0"
  "CANADIAN SYLLABICS TWAA\0"
  "CANADIAN SYLLABICS TWE\0"
  "CANADIAN SYLLABICS TWI\0"
  "CANADIAN SYLLABICS TWII\0"
  "CANADIAN SYLLABICS TWO\0"
  "CANADIAN SYLLABICS TWOO\0"
  "CANADIAN SYLLABICS TYA\0"
  "CANADIAN SYLLABICS TYE\0"
  "CANADIAN SYLLABICS TYI\0"
  "CANADIAN SYLLABICS TYO\0"
  "CANADIAN SYLLABICS WA\0"
  "CANADIAN SYLLABICS WAA\0"
  "CANADIAN SYLLABICS WAY\0"
  "CANADIAN SYLLABICS WE\0"
  "CANADIAN SYLLABICS WEST-CREE CWA\0"
  "CANADIAN SYLLABICS WEST-CREE CWAA\0"
  "CANADIAN SYLLABICS WEST-CREE CWE\0"
  "CANADIAN SYLLABICS WEST-CREE CWI\0"
  "CANADIAN SYLLABICS WEST-CREE CWII\0"
  "CANADIAN SYLLABICS WEST-CREE CWO\0"
  "CANADIAN SYLLABICS WEST-CREE CWOO\0"
  "CANADIAN SYLLABICS WEST-CREE FWAA\0"
  "CANADIAN SYLLABICS WEST-CREE KWA\0"
  "CANADIAN SYLLABICS WEST-CREE KWAA\0"
  "CANADIAN SYLLABICS WEST-CREE KWE\0"
  "CANADIAN SYLLABICS WEST-CREE KWI\0"
  "CANADIAN SYLLABICS WEST-CREE KWII\0"
  "CANADIAN SYLLABICS WEST-CREE KWO\0"
  "CANADIAN SYLLABICS WEST-CREE KWOO\0"
  "CANADIAN SYLLABICS WEST-CREE L\0"
  "CANADIAN SYLLABICS WEST-CREE LA\0"
  "CANADIAN SYLLABICS WEST-CREE LAA\0"
  "CANADIAN SYLLABICS WEST-CREE LE\0"
  "CANADIAN SYLLABICS WEST-CREE LO\0"
  "CANADIAN SYLLABICS WEST-CREE LOO\0"
  "CANADIAN SYLLABICS WEST-CREE LWA\0"
  "CANADIAN SYLLABICS WEST-CREE LWAA\0"
  "CANADIAN SYLLABICS WEST-CREE LWE\0"
  "CANADIAN SYLLABICS WEST-CREE LWI\0"
  "CANADIAN SYLLABICS WEST-CREE LWII\0"
  "CANADIAN SYLLABICS WEST-CREE LWO\0"
  "CANADIAN SYLLABICS WEST-CREE LWOO\0"
  "CANADIAN SYLLABICS WEST-CREE M\0"
  "CANADIAN SYLLABICS WEST-CREE MWA\0"
  "CANADIAN SYLLABICS WEST-CREE MWAA\0"
  "CANADIAN SYLLABICS WEST-CREE MWE\0"
  "CANADIAN SYLLABICS WEST-CREE MWI\0"
  "CANADIAN SYLLABICS WEST-CREE MWII\0"
  "CANADIAN SYLLABICS WEST-CREE MWO\0"
  "CANADIAN SYLLABICS WEST-CREE MWOO\0"
  "CANADIAN SYLLABICS WEST-CREE NWA\0"
  "CANADIAN SYLLABICS WEST-CREE NWAA\0"
  "CANADIAN SYLLABICS WEST-CREE NWE\0"
  "CANADIAN SYLLABICS WEST-CREE P\0"
  "CANADIAN SYLLABICS WEST-CREE PWA\0"
  "CANADIAN SYLLABICS WEST-CREE PWAA\0"
  "CANADIAN SYLLABICS WEST-CREE PWE\0"
  "CANADIAN SYLLABICS WEST-CREE PWI\0"
  "CANADIAN SYLLABICS WEST-CREE PWII\0"
  "CANADIAN SYLLABICS WEST-CREE PWO\0"
  "CANADIAN SYLLABICS WEST-CREE PWOO\0"
  "CANADIAN SYLLABICS WEST-CREE R\0"
  "CANADIAN SYLLABICS WEST-CREE RA\0"
  "CANADIAN SYLLABICS WEST-CREE RE\0"
  "CANADIAN SYLLABICS WEST-CREE RI\0"
  "CANADIAN SYLLABICS WEST-CREE RO\0"
  "CANADIAN SYLLABICS WEST-CREE RWAA\0"
  "CANADIAN SYLLABICS WEST-CREE SHWA\0"
  "CANADIAN SYLLABICS WEST-CREE SHWAA\0"
  "CANADIAN SYLLABICS WEST-CREE SHWE\0"
  "CANADIAN SYLLABICS WEST-CREE SHWI\0"
  "CANADIAN SYLLABICS WEST-CREE SHWII\0"
  "CANADIAN SYLLABICS WEST-CREE SHWO\0"
  "CANADIAN SYLLABICS WEST-CREE SHWOO\0"
  "CANADIAN SYLLABICS WEST-CREE SWA\0"
  "CANADIAN SYLLABICS WEST-CREE SWAA\0"
  "CANADIAN SYLLABICS WEST-CREE SWE\0"
  "CANADIAN SYLLABICS WEST-CREE SWI\0"
  "CANADIAN SYLLABICS WEST-CREE SWII\0"
  "CANADIAN SYLLABICS WEST-CREE SWO\0"
  "CANADIAN SYLLABICS WEST-CREE SWOO\0"
  "CANADIAN SYLLABICS WEST-CREE THWAA\0"
  "CANADIAN SYLLABICS WEST-CREE TWA\0"
  "CANADIAN SYLLABICS WEST-CREE TWAA\0"
  "CANADIAN SYLLABICS WEST-CREE TWE\0"
  "CANADIAN SYLLABICS WEST-CREE TWI\0"
  "CANADIAN SYLLABICS WEST-CREE TWII\0"
  "CANADIAN SYLLABICS WEST-CREE TWO\0"
  "CANADIAN SYLLABICS WEST-CREE TWOO\0"
  "CANADIAN SYLLABICS WEST-CREE WA\0"
  "CANADIAN SYLLABICS WEST-CREE WAA\0"
  "CANADIAN SYLLABICS WEST-CREE WE\0"
  "CANADIAN SYLLABICS WEST-CREE WI\0"
  "CANADIAN SYLLABICS WEST-CREE WII\0"
  "CANADIAN SYLLABICS WEST-CREE WO\0"
  "CANADIAN SYLLABICS WEST-CREE WOO\0"
  "CANADIAN SYLLABICS WEST-CREE Y\0"
  "CANADIAN SYLLABICS WEST-CREE YWA\0"
  "CANADIAN SYLLABICS WEST-CREE YWAA\0"
  "CANADIAN SYLLABICS WEST-CREE YWE\0"
  "CANADIAN SYLLABICS WEST-CREE YWI\0"
  "CANADIAN SYLLABICS WEST-CREE YWII\0"
  "CANADIAN SYLLABICS WEST-CREE YWO\0"
  "CANADIAN SYLLABICS WEST-CREE YWOO\0"
  "CANADIAN SYLLABICS WESTERN W\0"
  "CANADIAN SYLLABICS WI\0"
  "CANADIAN SYLLABICS WII\0"
  "CANADIAN SYLLABICS WO\0"
  "CANADIAN SYLLABICS WOO\0"
  "CANADIAN SYLLABICS WOODS-CREE FINAL TH\0"
  "CANADIAN SYLLABICS WOODS-CREE TH\0"
  "CANADIAN SYLLABICS WOODS-CREE THA\0"
  "CANADIAN SYLLABICS WOODS-CREE THE\0"
  "CANADIAN SYLLABICS WOODS-CREE THI\0"
  "CANADIAN SYLLABICS WOODS-CREE THO\0"
  "CANADIAN SYLLABICS WOODS-CREE THWA\0"
  "CANADIAN SYLLABICS WOODS-CREE THWAA\0"
  "CANADIAN SYLLABICS WOODS-CREE THWEE\0"
  "CANADIAN SYLLABICS WOODS-CREE THWI\0"
  "CANADIAN SYLLABICS WOODS-CREE THWII\0"
  "CANADIAN SYLLABICS WOODS-CREE THWO\0"
  "CANADIAN SYLLABICS WOODS-CREE THWOO\0"
  "CANADIAN SYLLABICS Y\0"
  "CANADIAN SYLLABICS Y-CREE COO\0"
  "CANADIAN SYLLABICS Y-CREE KOO\0"
  "CANADIAN SYLLABICS Y-CREE LOO\0"
  "CANADIAN SYLLABICS Y-CREE MOO\0"
  "CANADIAN SYLLABICS Y-CREE NOO\0"
  "CANADIAN SYLLABICS Y-CREE OO\0"
  "CANADIAN SYLLABICS Y-CREE POO\0"
  "CANADIAN SYLLABICS Y-CREE PWAA\0"
  "CANADIAN SYLLABICS Y-CREE SOO\0"
  "CANADIAN SYLLABICS Y-CREE TOO\0"
  "CANADIAN SYLLABICS Y-CREE W\0"
  "CANADIAN SYLLABICS Y-CREE YOO\0"
  "CANADIAN SYLLABICS YA\0"
  "CANADIAN SYLLABICS YAA\0"
  "CANADIAN SYLLABICS YAAI\0"
  "CANADIAN SYLLABICS YAY\0"
  "CANADIAN SYLLABICS YE\0"
  "CANADIAN SYLLABICS YI\0"
  "CANADIAN SYLLABICS YII\0"
  "CANADIAN SYLLABICS YO\0"
  "CANADIAN SYLLABICS YOO\0"
  "CANADIAN SYLLABICS YOY\0"
  "CANADIAN SYLLABICS YWA\0"
  "CANADIAN SYLLABICS YWAA\0"
  "CANADIAN SYLLABICS YWE\0"
  "CANADIAN SYLLABICS YWI\0"
  "CANADIAN SYLLABICS YWII\0"
  "CANADIAN SYLLABICS YWO\0"
  "CANADIAN SYLLABICS YWOO\0"
  "CANCEL TAG\0"
  "CANCELLATION X\0"
  "CANCER\0"
  "CANDLE\0"
  "CANDY\0"
  "CANNED FOOD\0"
  "CANOE\0"
  "CAPITULUM\0"
  "CAPRICORN\0"
  "CAR SLIDING\0"
  "CARD FILE BOX\0"
  "CARD INDEX\0"
  "CARD INDEX DIVIDERS\0"
  "CARE OF\0"
  "CARET\0"
  "CARET INSERTION POINT\0"
  "CARIAN LETTER A\0"
  "CARIAN LETTER A2\0"
  "CARIAN LETTER B\0"
  "CARIAN LETTER C-18\0"
  "CARIAN LETTER C-39\0"
  "CARIAN LETTER D\0"
  "CARIAN LETTER D2\0"
  "CARIAN LETTER E\0"
  "CARIAN LETTER E2\0"
  "CARIAN LETTER G\0"
  "CARIAN LETTER G2\0"
  "CARIAN LETTER I\0"
  "CARIAN LETTER II\0"
  "CARIAN LETTER K\0"
  "CARIAN LETTER K2\0"
  "CARIAN LETTER L\0"
  "CARIAN LETTER LD\0"
  "CARIAN LETTER LD2\0"
  "CARIAN LETTER M\0"
  "CARIAN LETTER MB\0"
  "CARIAN LETTER MB2\0"
  "CARIAN LETTER MB3\0"
  "CARIAN LETTER MB4\0"
  "CARIAN LETTER N\0"
  "CARIAN LETTER ND\0"
  "CARIAN LETTER NG\0"
  "CARIAN LETTER NN\0"
  "CARIAN LETTER O\0"
  "CARIAN LETTER P\0"
  "CARIAN LETTER P2\0"
  "CARIAN LETTER Q\0"
  "CARIAN LETTER R\0"
  "CARIAN LETTER RR\0"
  "CARIAN LETTER S\0"
  "CARIAN LETTER SH\0"
  "CARIAN LETTER SH2\0"
  "CARIAN LETTER SS\0"
  "CARIAN LETTER ST\0"
  "CARIAN LETTER ST2\0"
  "CARIAN LETTER T\0"
  "CARIAN LETTER TT\0"
  "CARIAN LETTER TT2\0"
  "CARIAN LETTER U\0"
  "CARIAN LETTER UU\0"
  "CARIAN LETTER UUU\0"
  "CARIAN LETTER UUU2\0"
  "CARIAN LETTER UUU3\0"
  "CARIAN LETTER UUUU\0"
  "CARIAN LETTER X\0"
  "CARON\0"
  "CAROUSEL HORSE\0"
  "CARP STREAMER\0"
  "CARPENTRY SAW\0"
  "CARROT\0"
  "CASTLE\0"
  "CAT\0"
  "CAT FACE\0"
  "CAT FACE WITH TEARS OF JOY\0"
  "CAT FACE WITH WRY SMILE\0"
  "CAUCASIAN ALBANIAN CITATION MARK\0"
  "CAUCASIAN ALBANIAN LETTER ALT\0"
  "CAUCASIAN ALBANIAN LETTER AOR\0"
  "CAUCASIAN ALBANIAN LETTER BET\0"
  "CAUCASIAN ALBANIAN LETTER CAR\0"
  "CAUCASIAN ALBANIAN LETTER CAYN\0"
  "CAUCASIAN ALBANIAN LETTER CHA\0"
  "CAUCASIAN ALBANIAN LETTER CHAT\0"
  "CAUCASIAN ALBANIAN LETTER CHI\0"
  "CAUCASIAN ALBANIAN LETTER CHOY\0"
  "CAUCASIAN ALBANIAN LETTER CYAW\0"
  "CAUCASIAN ALBANIAN LETTER CYAY\0"
  "CAUCASIAN ALBANIAN LETTER DAT\0"
  "CAUCASIAN ALBANIAN LETTER DYAN\0"
  "CAUCASIAN ALBANIAN LETTER DZAY\0"
  "CAUCASIAN ALBANIAN LETTER DZYAY\0"
  "CAUCASIAN ALBANIAN LETTER EB\0"
  "CAUCASIAN ALBANIAN LETTER EYN\0"
  "CAUCASIAN ALBANIAN LETTER FAM\0"
  "CAUCASIAN ALBANIAN LETTER GHEYS\0"
  "CAUCASIAN ALBANIAN LETTER GIM\0"
  "CAUCASIAN ALBANIAN LETTER HEYT\0"
  "CAUCASIAN ALBANIAN LETTER INYA\0"
  "CAUCASIAN ALBANIAN LETTER IRB\0"
  "CAUCASIAN ALBANIAN LETTER IWN\0"
  "CAUCASIAN ALBANIAN LETTER JAYN\0"
  "CAUCASIAN ALBANIAN LETTER JHOX\0"
  "CAUCASIAN ALBANIAN LETTER KAR\0"
  "CAUCASIAN ALBANIAN LETTER KIW\0"
  "CAUCASIAN ALBANIAN LETTER LAN\0"
  "CAUCASIAN ALBANIAN LETTER LYIT\0"
  "CAUCASIAN ALBANIAN LETTER MAQ\0"
  "CAUCASIAN ALBANIAN LETTER NOWC\0"
  "CAUCASIAN ALBANIAN LETTER ON\0"
  "CAUCASIAN ALBANIAN LETTER PEN\0"
  "CAUCASIAN ALBANIAN LETTER PIWR\0"
  "CAUCASIAN ALBANIAN LETTER QAR\0"
  "CAUCASIAN ALBANIAN LETTER QAY\0"
  "CAUCASIAN ALBANIAN LETTER RAT\0"
  "CAUCASIAN ALBANIAN LETTER SEYK\0"
  "CAUCASIAN ALBANIAN LETTER SHA\0"
  "CAUCASIAN ALBANIAN LETTER SHAK\0"
  "CAUCASIAN ALBANIAN LETTER SHOY\0"
  "CAUCASIAN ALBANIAN LETTER TAS\0"
  "CAUCASIAN ALBANIAN LETTER TIWR\0"
  "CAUCASIAN ALBANIAN LETTER TYAY\0"
  "CAUCASIAN ALBANIAN LETTER VEYZ\0"
  "CAUCASIAN ALBANIAN LETTER XEYN\0"
  "CAUCASIAN ALBANIAN LETTER YAYD\0"
  "CAUCASIAN ALBANIAN LETTER YOWD\0"
  "CAUCASIAN ALBANIAN LETTER ZARL\0"
  "CAUCASIAN ALBANIAN LETTER ZHA\0"
  "CAUCASIAN ALBANIAN LETTER ZHIL\0"
  "CAUTION SIGN\0"
  "CEDI SIGN\0"
  "CEDILLA\0"
  "CELTIC CROSS\0"
  "CENT SIGN\0"
  "CENTRE LINE SYMBOL\0"
  "CENTRELINE LOW LINE\0"
  "CENTRELINE OVERLINE\0"
  "CERES\0"
  "CHAINS\0"
  "CHAIR\0"
  "CHAKMA AU MARK\0"
  "CHAKMA DANDA\0"
  "CHAKMA DIGIT EIGHT\0"
  "CHAKMA DIGIT FIVE\0"
  "CHAKMA DIGIT FOUR\0"
  "CHAKMA DIGIT NINE\0"
  "CHAKMA DIGIT ONE\0"
  "CHAKMA DIGIT SEVEN\0"
  "CHAKMA DIGIT SIX\0"
  "CHAKMA DIGIT THREE\0"
  "CHAKMA DIGIT TWO\0"
  "CHAKMA DIGIT ZERO\0"
  "CHAKMA DOUBLE DANDA\0"
  "CHAKMA LETTER AA\0"
  "CHAKMA LETTER BAA\0"
  "CHAKMA LETTER BHAA\0"
  "CHAKMA LETTER CAA\0"
  "CHAKMA LETTER CHAA\0"
  "CHAKMA LETTER DAA\0"
  "CHAKMA LETTER DDAA\0"
  "CHAKMA LETTER DDHAA\0"
  "CHAKMA LETTER DHAA\0"
  "CHAKMA LETTER E\0"
  "CHAKMA LETTER GAA\0"
  "CHAKMA LETTER GHAA\0"
  "CHAKMA LETTER HAA\0"
  "CHAKMA LETTER I\0"
  "CHAKMA LETTER JAA\0"
  "CHAKMA LETTER JHAA\0"
  "CHAKMA LETTER KAA\0"
  "CHAKMA LETTER KHAA\0"
  "CHAKMA LETTER LAA\0"
  "CHAKMA LETTER LHAA\0"
  "CHAKMA LETTER MAA\0"
  "CHAKMA LETTER NAA\0"
  "CHAKMA LETTER NGAA\0"
  "CHAKMA LETTER NNAA\0"
  "CHAKMA LETTER NYAA\0"
  "CHAKMA LETTER PAA\0"
  "CHAKMA LETTER PHAA\0"
  "CHAKMA LETTER RAA\0"
  "CHAKMA LETTER SAA\0"
  "CHAKMA LETTER TAA\0"
  "CHAKMA LETTER THAA\0"
  "CHAKMA LETTER TTAA\0"
  "CHAKMA LETTER TTHAA\0"
  "CHAKMA LETTER U\0"
  "CHAKMA LETTER VAA\0"
  "CHAKMA LETTER WAA\0"
  "CHAKMA LETTER YAA\0"
  "CHAKMA LETTER YYAA\0"
  "CHAKMA MAAYYAA\0"
  "CHAKMA O MARK\0"
  "CHAKMA QUESTION MARK\0"
  "CHAKMA SECTION MARK\0"
  "CHAKMA SIGN ANUSVARA\0"
  "CHAKMA SIGN CANDRABINDU\0"
  "CHAKMA SIGN VISARGA\0"
  "CHAKMA VIRAMA\0"
  "CHAKMA VOWEL SIGN A\0"
  "CHAKMA VOWEL SIGN AA\0"
  "CHAKMA VOWEL SIGN AI\0"
  "CHAKMA VOWEL SIGN AU\0"
  "CHAKMA VOWEL SIGN E\0"
  "CHAKMA VOWEL SIGN EI\0"
  "CHAKMA VOWEL SIGN I\0"
  "CHAKMA VOWEL SIGN II\0"
  "CHAKMA VOWEL SIGN O\0"
  "CHAKMA VOWEL SIGN OI\0"
  "CHAKMA VOWEL SIGN U\0"
  "CHAKMA VOWEL SIGN UU\0"
  "CHAM CONSONANT SIGN FINAL H\0"
  "CHAM CONSONANT SIGN FINAL M\0"
  "CHAM CONSONANT SIGN FINAL NG\0"
  "CHAM CONSONANT SIGN LA\0"
  "CHAM CONSONANT SIGN RA\0"
  "CHAM CONSONANT SIGN WA\0"
  "CHAM CONSONANT SIGN YA\0"
  "CHAM DIGIT EIGHT\0"
  "CHAM DIGIT FIVE\0"
  "CHAM DIGIT FOUR\0"
  "CHAM DIGIT NINE\0"
  "CHAM DIGIT ONE\0"
  "CHAM DIGIT SEVEN\0"
  "CHAM DIGIT SIX\0"
  "CHAM DIGIT THREE\0"
  "CHAM DIGIT TWO\0"
  "CHAM DIGIT ZERO\0"
  "CHAM LETTER A\0"
  "CHAM LETTER AI\0"
  "CHAM LETTER BA\0"
  "CHAM LETTER BBA\0"
  "CHAM LETTER BHA\0"
  "CHAM LETTER CHA\0"
  "CHAM LETTER CHHA\0"
  "CHAM LETTER DA\0"
  "CHAM LETTER DDA\0"
  "CHAM LETTER DHA\0"
  "CHAM LETTER E\0"
  "CHAM LETTER FINAL CH\0"
  "CHAM LETTER FINAL G\0"
  "CHAM LETTER FINAL K\0"
  "CHAM LETTER FINAL L\0"
  "CHAM LETTER FINAL N\0"
  "CHAM LETTER FINAL NG\0"
  "CHAM LETTER FINAL P\0"
  "CHAM LETTER FINAL R\0"
  "CHAM LETTER FINAL SS\0"
  "CHAM LETTER FINAL T\0"
  "CHAM LETTER FINAL Y\0"
  "CHAM LETTER GA\0"
  "CHAM LETTER GHA\0"
  "CHAM LETTER HA\0"
  "CHAM LETTER I\0"
  "CHAM LETTER JA\0"
  "CHAM LETTER JHA\0"
  "CHAM LETTER KA\0"
  "CHAM LETTER KHA\0"
  "CHAM LETTER LA\0"
  "CHAM LETTER MA\0"
  "CHAM LETTER MUE\0"
  "CHAM LETTER NA\0"
  "CHAM LETTER NGA\0"
  "CHAM LETTER NGUE\0"
  "CHAM LETTER NHA\0"
  "CHAM LETTER NHJA\0"
  "CHAM LETTER NHUE\0"
  "CHAM LETTER NUE\0"
  "CHAM LETTER O\0"
  "CHAM LETTER PA\0"
  "CHAM LETTER PHA\0"
  "CHAM LETTER PPA\0"
  "CHAM LETTER RA\0"
  "CHAM LETTER SA\0"
  "CHAM LETTER SSA\0"
  "CHAM LETTER TA\0"
  "CHAM LETTER THA\0"
  "CHAM LETTER U\0"
  "CHAM LETTER VA\0"
  "CHAM LETTER YA\0"
  "CHAM PUNCTUATION DANDA\0"
  "CHAM PUNCTUATION DOUBLE DANDA\0"
  "CHAM PUNCTUATION SPIRAL\0"
  "CHAM PUNCTUATION TRIPLE DANDA\0"
  "CHAM VOWEL SIGN AA\0"
  "CHAM VOWEL SIGN AI\0"
  "CHAM VOWEL SIGN AU\0"
  "CHAM VOWEL SIGN EI\0"
  "CHAM VOWEL SIGN I\0"
  "CHAM VOWEL SIGN II\0"
  "CHAM VOWEL SIGN O\0"
  "CHAM VOWEL SIGN OE\0"
  "CHAM VOWEL SIGN U\0"
  "CHAM VOWEL SIGN UE\0"
  "CHARACTER TIE\0"
  "CHART WITH DOWNWARDS TREND\0"
  "CHART WITH UPWARDS TREND\0"
  "CHART WITH UPWARDS TREND AND YEN SIGN\0"
  "CHECK MARK\0"
  "CHECKER BOARD\0"
  "CHECKER BOARD FILL\0"
  "CHEERING MEGAPHONE\0"
  "CHEESE WEDGE\0"
  "CHEQUERED FLAG\0"
  "CHEROKEE LETTER A\0"
  "CHEROKEE LETTER DA\0"
  "CHEROKEE LETTER DE\0"
  "CHEROKEE LETTER DI\0"
  "CHEROKEE LETTER DLA\0"
  "CHEROKEE LETTER DO\0"
  "CHEROKEE LETTER DU\0"
  "CHEROKEE LETTER DV\0"
  "CHEROKEE LETTER E\0"
  "CHEROKEE LETTER GA\0"
  "CHEROKEE LETTER GE\0"
  "CHEROKEE LETTER GI\0"
  "CHEROKEE LETTER GO\0"
  "CHEROKEE LETTER GU\0"
  "CHEROKEE LETTER GV\0"
  "CHEROKEE LETTER HA\0"
  "CHEROKEE LETTER HE\0"
  "CHEROKEE LETTER HI\0"
  "CHEROKEE LETTER HNA\0"
  "CHEROKEE LETTER HO\0"
  "CHEROKEE LETTER HU\0"
  "CHEROKEE LETTER HV\0"
  "CHEROKEE LETTER I\0"
  "CHEROKEE LETTER KA\0"
  "CHEROKEE LETTER LA\0"
  "CHEROKEE LETTER LE\0"
  "CHEROKEE LETTER LI\0"
  "CHEROKEE LETTER LO\0"
  "CHEROKEE LETTER LU\0"
  "CHEROKEE LETTER LV\0"
  "CHEROKEE LETTER MA\0"
  "CHEROKEE LETTER ME\0"
  "CHEROKEE LETTER MI\0"
  "CHEROKEE LETTER MO\0"
  "CHEROKEE LETTER MU\0"
  "CHEROKEE LETTER MV\0"
  "CHEROKEE LETTER NA\0"
  "CHEROKEE LETTER NAH\0"
  "CHEROKEE LETTER NE\0"
  "CHEROKEE LETTER NI\0"
  "CHEROKEE LETTER NO\0"
  "CHEROKEE LETTER NU\0"
  "CHEROKEE LETTER NV\0"
  "CHEROKEE LETTER O\0"
  "CHEROKEE LETTER QUA\0"
  "CHEROKEE LETTER QUE\0"
  "CHEROKEE LETTER QUI\0"
  "CHEROKEE LETTER QUO\0"
  "CHEROKEE LETTER QUU\0"
  "CHEROKEE LETTER QUV\0"
  "CHEROKEE LETTER S\0"
  "CHEROKEE LETTER SA\0"
  "CHEROKEE LETTER SE\0"
  "CHEROKEE LETTER SI\0"
  "CHEROKEE LETTER SO\0"
  "CHEROKEE LETTER SU\0"
  "CHEROKEE LETTER SV\0"
  "CHEROKEE LETTER TA\0"
  "CHEROKEE LETTER TE\0"
  "CHEROKEE LETTER TI\0"
  "CHEROKEE LETTER TLA\0"
  "CHEROKEE LETTER TLE\0"
  "CHEROKEE LETTER TLI\0"
  "CHEROKEE LETTER TLO\0"
  "CHEROKEE LETTER TLU\0"
  "CHEROKEE LETTER TLV\0"
  "CHEROKEE LETTER TSA\0"
  "CHEROKEE LETTER TSE\0"
  "CHEROKEE LETTER TSI\0"
  "CHEROKEE LETTER TSO\0"
  "CHEROKEE LETTER TSU\0"
  "CHEROKEE LETTER TSV\0"
  "CHEROKEE LETTER U\0"
  "CHEROKEE LETTER V\0"
  "CHEROKEE LETTER WA\0"
  "CHEROKEE LETTER WE\0"
  "CHEROKEE LETTER WI\0"
  "CHEROKEE LETTER WO\0"
  "CHEROKEE LETTER WU\0"
  "CHEROKEE LETTER WV\0"
  "CHEROKEE LETTER YA\0"
  "CHEROKEE LETTER YE\0"
  "CHEROKEE LETTER YI\0"
  "CHEROKEE LETTER YO\0"
  "CHEROKEE LETTER YU\0"
  "CHEROKEE LETTER YV\0"
  "CHEROKEE SMALL LETTER A\0"
  "CHEROKEE SMALL LETTER DA\0"
  "CHEROKEE SMALL LETTER DE\0"
  "CHEROKEE SMALL LETTER DI\0"
  "CHEROKEE SMALL LETTER DLA\0"
  "CHEROKEE SMALL LETTER DO\0"
  "CHEROKEE SMALL LETTER DU\0"
  "CHEROKEE SMALL LETTER DV\0"
  "CHEROKEE SMALL LETTER E\0"
  "CHEROKEE SMALL LETTER GA\0"
  "CHEROKEE SMALL LETTER GE\0"
  "CHEROKEE SMALL LETTER GI\0"
  "CHEROKEE SMALL LETTER GO\0"
  "CHEROKEE SMALL LETTER GU\0"
  "CHEROKEE SMALL LETTER GV\0"
  "CHEROKEE SMALL LETTER HA\0"
  "CHEROKEE SMALL LETTER HE\0"
  "CHEROKEE SMALL LETTER HI\0"
  "CHEROKEE SMALL LETTER HNA\0"
  "CHEROKEE SMALL LETTER HO\0"
  "CHEROKEE SMALL LETTER HU\0"
  "CHEROKEE SMALL LETTER HV\0"
  "CHEROKEE SMALL LETTER I\0"
  "CHEROKEE SMALL LETTER KA\0"
  "CHEROKEE SMALL LETTER LA\0"
  "CHEROKEE SMALL LETTER LE\0"
  "CHEROKEE SMALL LETTER LI\0"
  "CHEROKEE SMALL LETTER LO\0"
  "CHEROKEE SMALL LETTER LU\0"
  "CHEROKEE SMALL LETTER LV\0"
  "CHEROKEE SMALL LETTER MA\0"
  "CHEROKEE SMALL LETTER ME\0"
  "CHEROKEE SMALL LETTER MI\0"
  "CHEROKEE SMALL LETTER MO\0"
  "CHEROKEE SMALL LETTER MU\0"
  "CHEROKEE SMALL LETTER MV\0"
  "CHEROKEE SMALL LETTER NA\0"
  "CHEROKEE SMALL LETTER NAH\0"
  "CHEROKEE SMALL LETTER NE\0"
  "CHEROKEE SMALL LETTER NI\0"
  "CHEROKEE SMALL LETTER NO\0"
  "CHEROKEE SMALL LETTER NU\0"
  "CHEROKEE SMALL LETTER NV\0"
  "CHEROKEE SMALL LETTER O\0"
  "CHEROKEE SMALL LETTER QUA\0"
  "CHEROKEE SMALL LETTER QUE\0"
  "CHEROKEE SMALL LETTER QUI\0"
  "CHEROKEE SMALL LETTER QUO\0"
  "CHEROKEE SMALL LETTER QUU\0"
  "CHEROKEE SMALL LETTER QUV\0"
  "CHEROKEE SMALL LETTER S\0"
  "CHEROKEE SMALL LETTER SA\0"
  "CHEROKEE SMALL LETTER SE\0"
  "CHEROKEE SMALL LETTER SI\0"
  "CHEROKEE SMALL LETTER SO\0"
  "CHEROKEE SMALL LETTER SU\0"
  "CHEROKEE SMALL LETTER SV\0"
  "CHEROKEE SMALL LETTER TA\0"
  "CHEROKEE SMALL LETTER TE\0"
  "CHEROKEE SMALL LETTER TI\0"
  "CHEROKEE SMALL LETTER TLA\0"
  "CHEROKEE SMALL LETTER TLE\0"
  "CHEROKEE SMALL LETTER TLI\0"
  "CHEROKEE SMALL LETTER TLO\0"
  "CHEROKEE SMALL LETTER TLU\0"
  "CHEROKEE SMALL LETTER TLV\0"
  "CHEROKEE SMALL LETTER TSA\0"
  "CHEROKEE SMALL LETTER TSE\0"
  "CHEROKEE SMALL LETTER TSI\0"
  "CHEROKEE SMALL LETTER TSO\0"
  "CHEROKEE SMALL LETTER TSU\0"
  "CHEROKEE SMALL LETTER TSV\0"
  "CHEROKEE SMALL LETTER U\0"
  "CHEROKEE SMALL LETTER V\0"
  "CHEROKEE SMALL LETTER WA\0"
  "CHEROKEE SMALL LETTER WE\0"
  "CHEROKEE SMALL LETTER WI\0"
  "CHEROKEE SMALL LETTER WO\0"
  "CHEROKEE SMALL LETTER WU\0"
  "CHEROKEE SMALL LETTER WV\0"
  "CHEROKEE SMALL LETTER YA\0"
  "CHEROKEE SMALL LETTER YE\0"
  "CHEROKEE SMALL LETTER YI\0"
  "CHEROKEE SMALL LETTER YO\0"
  "CHEROKEE SMALL LETTER YU\0"
  "CHEROKEE SMALL LETTER YV\0"
  "CHERRIES\0"
  "CHERRY BLOSSOM\0"
  "CHESTNUT\0"
  "CHI RHO\0"
  "CHICKEN\0"
  "CHILD\0"
  "CHILDREN CROSSING\0"
  "CHIPMUNK\0"
  "CHIRON\0"
  "CHOCOLATE BAR\0"
  "CHOPSTICKS\0"
  "CHORASMIAN LETTER ALEPH\0"
  "CHORASMIAN LETTER AYIN\0"
  "CHORASMIAN LETTER BETH\0"
  "CHORASMIAN LETTER CURLED WAW\0"
  "CHORASMIAN LETTER DALETH\0"
  "CHORASMIAN LETTER GIMEL\0"
  "CHORASMIAN LETTER HE\0"
  "CHORASMIAN LETTER HETH\0"
  "CHORASMIAN LETTER KAPH\0"
  "CHORASMIAN LETTER LAMEDH\0"
  "CHORASMIAN LETTER MEM\0"
  "CHORASMIAN LETTER NUN\0"
  "CHORASMIAN LETTER PE\0"
  "CHORASMIAN LETTER RESH\0"
  "CHORASMIAN LETTER SAMEKH\0"
  "CHORASMIAN LETTER SHIN\0"
  "CHORASMIAN LETTER SMALL ALEPH\0"
  "CHORASMIAN LETTER TAW\0"
  "CHORASMIAN LETTER WAW\0"
  "CHORASMIAN LETTER YODH\0"
  "CHORASMIAN LETTER ZAYIN\0"
  "CHORASMIAN NUMBER FOUR\0"
  "CHORASMIAN NUMBER ONE\0"
  "CHORASMIAN NUMBER ONE HUNDRED\0"
  "CHORASMIAN NUMBER TEN\0"
  "CHORASMIAN NUMBER THREE\0"
  "CHORASMIAN NUMBER TWENTY\0"
  "CHORASMIAN NUMBER TWO\0"
  "CHRISTMAS TREE\0"
  "CHURCH\0"
  "CINEMA\0"
  "CIRCLE DIVIDED BY HORIZONTAL BAR AND TOP HALF DIVIDED BY VERTICAL BAR\0"
  "CIRCLE WITH ALL BUT UPPER LEFT QUADRANT BLACK\0"
  "CIRCLE WITH HORIZONTAL BAR\0"
  "CIRCLE WITH LEFT HALF BLACK\0"
  "CIRCLE WITH LOWER HALF BLACK\0"
  "CIRCLE WITH RIGHT HALF BLACK\0"
  "CIRCLE WITH SMALL CIRCLE TO THE RIGHT\0"
  "CIRCLE WITH SUPERIMPOSED X\0"
  "CIRCLE WITH TWO HORIZONTAL STROKES TO THE RIGHT\0"
  "CIRCLE WITH UPPER HALF BLACK\0"
  "CIRCLE WITH UPPER RIGHT QUADRANT BLACK\0"
  "CIRCLE WITH VERTICAL FILL\0"
  "CIRCLED ANTICLOCKWISE ARROW\0"
  "CIRCLED ANTICLOCKWISE-ROTATED DIVISION SIGN\0"
  "CIRCLED ASTERISK OPERATOR\0"
  "CIRCLED BOLD X\0"
  "CIRCLED BULLET\0"
  "CIRCLED C WITH OVERLAID BACKSLASH\0"
  "CIRCLED CC\0"
  "CIRCLED CD\0"
  "CIRCLED CROSS FORMEE\0"
  "CIRCLED CROSS FORMEE WITH FOUR DOTS\0"
  "CIRCLED CROSS FORMEE WITH TWO DOTS\0"
  "CIRCLED CROSS POMMEE\0"
  "CIRCLED CROSSING LANES\0"
  "CIRCLED DASH\0"
  "CIRCLED DIGIT EIGHT\0"
  "CIRCLED DIGIT FIVE\0"
  "CIRCLED DIGIT FOUR\0"
  "CIRCLED DIGIT NINE\0"
  "CIRCLED DIGIT ONE\0"
  "CIRCLED DIGIT SEVEN\0"
  "CIRCLED DIGIT SIX\0"
  "CIRCLED DIGIT THREE\0"
  "CIRCLED DIGIT TWO\0"
  "CIRCLED DIGIT ZERO\0"
  "CIRCLED DIVISION SIGN\0"
  "CIRCLED DIVISION SLASH\0"
  "CIRCLED DOLLAR SIGN WITH OVERLAID BACKSLASH\0"
  "CIRCLED DOT OPERATOR\0"
  "CIRCLED EQUALS\0"
  "CIRCLED GREATER-THAN\0"
  "CIRCLED HANGUL CHIEUCH\0"
  "CIRCLED HANGUL CHIEUCH A\0"
  "CIRCLED HANGUL CIEUC\0"
  "CIRCLED HANGUL CIEUC A\0"
  "CIRCLED HANGUL HIEUH\0"
  "CIRCLED HANGUL HIEUH A\0"
  "CIRCLED HANGUL IEUNG\0"
  "CIRCLED HANGUL IEUNG A\0"
  "CIRCLED HANGUL IEUNG U\0"
  "CIRCLED HANGUL KHIEUKH\0"
  "CIRCLED HANGUL KHIEUKH A\0"
  "CIRCLED HANGUL KIYEOK\0"
  "CIRCLED HANGUL KIYEOK A\0"
  "CIRCLED HANGUL MIEUM\0"
  "CIRCLED HANGUL MIEUM A\0"
  "CIRCLED HANGUL NIEUN\0"
  "CIRCLED HANGUL NIEUN A\0"
  "CIRCLED HANGUL PHIEUPH\0"
  "CIRCLED HANGUL PHIEUPH A\0"
  "CIRCLED HANGUL PIEUP\0"
  "CIRCLED HANGUL PIEUP A\0"
  "CIRCLED HANGUL RIEUL\0"
  "CIRCLED HANGUL RIEUL A\0"
  "CIRCLED HANGUL SIOS\0"
  "CIRCLED HANGUL SIOS A\0"
  "CIRCLED HANGUL THIEUTH\0"
  "CIRCLED HANGUL THIEUTH A\0"
  "CIRCLED HANGUL TIKEUT\0"
  "CIRCLED HANGUL TIKEUT A\0"
  "CIRCLED HEAVY WHITE RIGHTWARDS ARROW\0"
  "CIRCLED HORIZONTAL BAR WITH NOTCH\0"
  "CIRCLED HUMAN FIGURE\0"
  "CIRCLED IDEOGRAPH ACCEPT\0"
  "CIRCLED IDEOGRAPH ADVANTAGE\0"
  "CIRCLED IDEOGRAPH ALLIANCE\0"
  "CIRCLED IDEOGRAPH ATTENTION\0"
  "CIRCLED IDEOGRAPH CENTRE\0"
  "CIRCLED IDEOGRAPH CONGRATULATION\0"
  "CIRCLED IDEOGRAPH COPY\0"
  "CIRCLED IDEOGRAPH CORRECT\0"
  "CIRCLED IDEOGRAPH EARTH\0"
  "CIRCLED IDEOGRAPH EIGHT\0"
  "CIRCLED IDEOGRAPH ENTERPRISE\0"
  "CIRCLED IDEOGRAPH EXCELLENT\0"
  "CIRCLED IDEOGRAPH FEMALE\0"
  "CIRCLED IDEOGRAPH FINANCIAL\0"
  "CIRCLED IDEOGRAPH FIRE\0"
  "CIRCLED IDEOGRAPH FIVE\0"
  "CIRCLED IDEOGRAPH FOUR\0"
  "CIRCLED IDEOGRAPH HAVE\0"
  "CIRCLED IDEOGRAPH HIGH\0"
  "CIRCLED IDEOGRAPH ITEM\0"
  "CIRCLED IDEOGRAPH KINDERGARTEN\0"
  "CIRCLED IDEOGRAPH KOTO\0"
  "CIRCLED IDEOGRAPH LABOR\0"
  "CIRCLED IDEOGRAPH LEFT\0"
  "CIRCLED IDEOGRAPH LOW\0"
  "CIRCLED IDEOGRAPH MALE\0"
  "CIRCLED IDEOGRAPH MEDICINE\0"
  "CIRCLED IDEOGRAPH METAL\0"
  "CIRCLED IDEOGRAPH MOON\0"
  "CIRCLED IDEOGRAPH NAME\0"
  "CIRCLED IDEOGRAPH NIGHT\0"
  "CIRCLED IDEOGRAPH NINE\0"
  "CIRCLED IDEOGRAPH ONE\0"
  "CIRCLED IDEOGRAPH PRINT\0"
  "CIRCLED IDEOGRAPH QUESTION\0"
  "CIRCLED IDEOGRAPH RELIGION\0"
  "CIRCLED IDEOGRAPH RESOURCE\0"
  "CIRCLED IDEOGRAPH REST\0"
  "CIRCLED IDEOGRAPH RIGHT\0"
  "CIRCLED IDEOGRAPH SCHOOL\0"
  "CIRCLED IDEOGRAPH SECRET\0"
  "CIRCLED IDEOGRAPH SEVEN\0"
  "CIRCLED IDEOGRAPH SIX\0"
  "CIRCLED IDEOGRAPH SOCIETY\0"
  "CIRCLED IDEOGRAPH SPECIAL\0"
  "CIRCLED IDEOGRAPH STOCK\0"
  "CIRCLED IDEOGRAPH STUDY\0"
  "CIRCLED IDEOGRAPH SUITABLE\0"
  "CIRCLED IDEOGRAPH SUN\0"
  "CIRCLED IDEOGRAPH SUPERVISE\0"
  "CIRCLED IDEOGRAPH TEN\0"
  "CIRCLED IDEOGRAPH THREE\0"
  "CIRCLED IDEOGRAPH TWO\0"
  "CIRCLED IDEOGRAPH WATER\0"
  "CIRCLED IDEOGRAPH WOOD\0"
  "CIRCLED INFORMATION SOURCE\0"
  "CIRCLED ITALIC LATIN CAPITAL LETTER C\0"
  "CIRCLED ITALIC LATIN CAPITAL LETTER R\0"
  "CIRCLED KATAKANA A\0"
  "CIRCLED KATAKANA E\0"
  "CIRCLED KATAKANA HA\0"
  "CIRCLED KATAKANA HE\0"
  "CIRCLED KATAKANA HI\0"
  "CIRCLED KATAKANA HO\0"
  "CIRCLED KATAKANA HU\0"
  "CIRCLED KATAKANA I\0"
  "CIRCLED KATAKANA KA\0"
  "CIRCLED KATAKANA KE\0"
  "CIRCLED KATAKANA KI\0"
  "CIRCLED KATAKANA KO\0"
  "CIRCLED KATAKANA KU\0"
  "CIRCLED KATAKANA MA\0"
  "CIRCLED KATAKANA ME\0"
  "CIRCLED KATAKANA MI\0"
  "CIRCLED KATAKANA MO\0"
  "CIRCLED KATAKANA MU\0"
  "CIRCLED KATAKANA NA\0"
  "CIRCLED KATAKANA NE\0"
  "CIRCLED KATAKANA NI\0"
  "CIRCLED KATAKANA NO\0"
  "CIRCLED KATAKANA NU\0"
  "CIRCLED KATAKANA O\0"
  "CIRCLED KATAKANA RA\0"
  "CIRCLED KATAKANA RE\0"
  "CIRCLED KATAKANA RI\0"
  "CIRCLED KATAKANA RO\0"
  "CIRCLED KATAKANA RU\0"
  "CIRCLED KATAKANA SA\0"
  "CIRCLED KATAKANA SE\0"
  "CIRCLED KATAKANA SI\0"
  "CIRCLED KATAKANA SO\0"
  "CIRCLED KATAKANA SU\0"
  "CIRCLED KATAKANA TA\0"
  "CIRCLED KATAKANA TE\0"
  "CIRCLED KATAKANA TI\0"
  "CIRCLED KATAKANA TO\0"
  "CIRCLED KATAKANA TU\0"
  "CIRCLED KATAKANA U\0"
  "CIRCLED KATAKANA WA\0"
  "CIRCLED KATAKANA WE\0"
  "CIRCLED KATAKANA WI\0"
  "CIRCLED KATAKANA WO\0"
  "CIRCLED KATAKANA YA\0"
  "CIRCLED KATAKANA YO\0"
  "CIRCLED KATAKANA YU\0"
  "CIRCLED KOREAN CHARACTER CHAMKO\0"
  "CIRCLED KOREAN CHARACTER JUEUI\0"
  "CIRCLED LATIN CAPITAL LETTER A\0"
  "CIRCLED LATIN CAPITAL LETTER B\0"
  "CIRCLED LATIN CAPITAL LETTER C\0"
  "CIRCLED LATIN CAPITAL LETTER D\0"
  "CIRCLED LATIN CAPITAL LETTER E\0"
  "CIRCLED LATIN CAPITAL LETTER F\0"
  "CIRCLED LATIN CAPITAL LETTER G\0"
  "CIRCLED LATIN CAPITAL LETTER H\0"
  "CIRCLED LATIN CAPITAL LETTER I\0"
  "CIRCLED LATIN CAPITAL LETTER J\0"
  "CIRCLED LATIN CAPITAL LETTER K\0"
  "CIRCLED LATIN CAPITAL LETTER L\0"
  "CIRCLED LATIN CAPITAL LETTER M\0"
  "CIRCLED LATIN CAPITAL LETTER N\0"
  "CIRCLED LATIN CAPITAL LETTER O\0"
  "CIRCLED LATIN CAPITAL LETTER P\0"
  "CIRCLED LATIN CAPITAL LETTER Q\0"
  "CIRCLED LATIN CAPITAL LETTER R\0"
  "CIRCLED LATIN CAPITAL LETTER S\0"
  "CIRCLED LATIN CAPITAL LETTER T\0"
  "CIRCLED LATIN CAPITAL LETTER U\0"
  "CIRCLED LATIN CAPITAL LETTER V\0"
  "CIRCLED LATIN CAPITAL LETTER W\0"
  "CIRCLED LATIN CAPITAL LETTER X\0"
  "CIRCLED LATIN CAPITAL LETTER Y\0"
  "CIRCLED LATIN CAPITAL LETTER Z\0"
  "CIRCLED LATIN SMALL LETTER A\0"
  "CIRCLED LATIN SMALL LETTER B\0"
  "CIRCLED LATIN SMALL LETTER C\0"
  "CIRCLED LATIN SMALL LETTER D\0"
  "CIRCLED LATIN SMALL LETTER E\0"
  "CIRCLED LATIN SMALL LETTER F\0"
  "CIRCLED LATIN SMALL LETTER G\0"
  "CIRCLED LATIN SMALL LETTER H\0"
  "CIRCLED LATIN SMALL LETTER I\0"
  "CIRCLED LATIN SMALL LETTER J\0"
  "CIRCLED LATIN SMALL LETTER K\0"
  "CIRCLED LATIN SMALL LETTER L\0"
  "CIRCLED LATIN SMALL LETTER M\0"
  "CIRCLED LATIN SMALL LETTER N\0"
  "CIRCLED LATIN SMALL LETTER O\0"
  "CIRCLED LATIN SMALL LETTER P\0"
  "CIRCLED LATIN SMALL LETTER Q\0"
  "CIRCLED LATIN SMALL LETTER R\0"
  "CIRCLED LATIN SMALL LETTER S\0"
  "CIRCLED LATIN SMALL LETTER T\0"
  "CIRCLED LATIN SMALL LETTER U\0"
  "CIRCLED LATIN SMALL LETTER V\0"
  "CIRCLED LATIN SMALL LETTER W\0"
  "CIRCLED LATIN SMALL LETTER X\0"
  "CIRCLED LATIN SMALL LETTER Y\0"
  "CIRCLED LATIN SMALL LETTER Z\0"
  "CIRCLED LESS-THAN\0"
  "CIRCLED MINUS\0"
  "CIRCLED MULTIPLICATION SIGN WITH CIRCUMFLEX ACCENT\0"
  "CIRCLED NUMBER EIGHTEEN\0"
  "CIRCLED NUMBER EIGHTY ON BLACK SQUARE\0"
  "CIRCLED NUMBER ELEVEN\0"
  "CIRCLED NUMBER FIFTEEN\0"
  "CIRCLED NUMBER FIFTY\0"
  "CIRCLED NUMBER FIFTY ON BLACK SQUARE\0"
  "CIRCLED NUMBER FORTY\0"
  "CIRCLED NUMBER FORTY EIGHT\0"
  "CIRCLED NUMBER FORTY FIVE\0"
  "CIRCLED NUMBER FORTY FOUR\0"
  "CIRCLED NUMBER FORTY NINE\0"
  "CIRCLED NUMBER FORTY ON BLACK SQUARE\0"
  "CIRCLED NUMBER FORTY ONE\0"
  "CIRCLED NUMBER FORTY SEVEN\0"
  "CIRCLED NUMBER FORTY SIX\0"
  "CIRCLED NUMBER FORTY THREE\0"
  "CIRCLED NUMBER FORTY TWO\0"
  "CIRCLED NUMBER FOURTEEN\0"
  "CIRCLED NUMBER NINETEEN\0"
  "CIRCLED NUMBER SEVENTEEN\0"
  "CIRCLED NUMBER SEVENTY ON BLACK SQUARE\0"
  "CIRCLED NUMBER SIXTEEN\0"
  "CIRCLED NUMBER SIXTY ON BLACK SQUARE\0"
  "CIRCLED NUMBER TEN\0"
  "CIRCLED NUMBER TEN ON BLACK SQUARE\0"
  "CIRCLED NUMBER THIRTEEN\0"
  "CIRCLED NUMBER THIRTY\0"
  "CIRCLED NUMBER THIRTY EIGHT\0"
  "CIRCLED NUMBER THIRTY FIVE\0"
  "CIRCLED NUMBER THIRTY FOUR\0"
  "CIRCLED NUMBER THIRTY NINE\0"
  "CIRCLED NUMBER THIRTY ON BLACK SQUARE\0"
  "CIRCLED NUMBER THIRTY ONE\0"
  "CIRCLED NUMBER THIRTY SEVEN\0"
  "CIRCLED NUMBER THIRTY SIX\0"
  "CIRCLED NUMBER THIRTY THREE\0"
  "CIRCLED NUMBER THIRTY TWO\0"
  "CIRCLED NUMBER TWELVE\0"
  "CIRCLED NUMBER TWENTY\0"
  "CIRCLED NUMBER TWENTY EIGHT\0"
  "CIRCLED NUMBER TWENTY FIVE\0"
  "CIRCLED NUMBER TWENTY FOUR\0"
  "CIRCLED NUMBER TWENTY NINE\0"
  "CIRCLED NUMBER TWENTY ON BLACK SQUARE\0"
  "CIRCLED NUMBER TWENTY ONE\0"
  "CIRCLED NUMBER TWENTY SEVEN\0"
  "CIRCLED NUMBER TWENTY SIX\0"
  "CIRCLED NUMBER TWENTY THREE\0"
  "CIRCLED NUMBER TWENTY TWO\0"
  "CIRCLED OPEN CENTRE EIGHT POINTED STAR\0"
  "CIRCLED PARALLEL\0"
  "CIRCLED PERPENDICULAR\0"
  "CIRCLED PLUS\0"
  "CIRCLED POSTAL MARK\0"
  "CIRCLED REVERSE SOLIDUS\0"
  "CIRCLED RING OPERATOR\0"
  "CIRCLED SQUARE\0"
  "CIRCLED TIMES\0"
  "CIRCLED TRIANGLE\0"
  "CIRCLED TRIANGLE DOWN\0"
  "CIRCLED VERTICAL BAR\0"
  "CIRCLED WHITE BULLET\0"
  "CIRCLED WHITE STAR\0"
  "CIRCLED WZ\0"
  "CIRCLED X\0"
  "CIRCLED ZERO WITH SLASH\0"
  "CIRCULATION FUNCTION\0"
  "CIRCUMFLEX ACCENT\0"
  "CIRCUS TENT\0"
  "CITYSCAPE\0"
  "CITYSCAPE AT DUSK\0"
  "CJK RADICAL BAMBOO\0"
  "CJK RADICAL BLUE\0"
  "CJK RADICAL BOLT OF CLOTH\0"
  "CJK RADICAL BONE\0"
  "CJK RADICAL BOX\0"
  "CJK RADICAL BRUSH ONE\0"
  "CJK RADICAL BRUSH TWO\0"
  "CJK RADICAL C-SIMPLIFIED BIRD\0"
  "CJK RADICAL C-SIMPLIFIED CART\0"
  "CJK RADICAL C-SIMPLIFIED DRAGON\0"
  "CJK RADICAL C-SIMPLIFIED EAT\0"
  "CJK RADICAL C-SIMPLIFIED EVEN\0"
  "CJK RADICAL C-SIMPLIFIED FISH\0"
  "CJK RADICAL C-SIMPLIFIED FLY\0"
  "CJK RADICAL C-SIMPLIFIED FROG\0"
  "CJK RADICAL C-SIMPLIFIED GATE\0"
  "CJK RADICAL C-SIMPLIFIED GOLD\0"
  "CJK RADICAL C-SIMPLIFIED HORSE\0"
  "CJK RADICAL C-SIMPLIFIED LEAF\0"
  "CJK RADICAL C-SIMPLIFIED LONG\0"
  "CJK RADICAL C-SIMPLIFIED SALT\0"
  "CJK RADICAL C-SIMPLIFIED SEE\0"
  "CJK RADICAL C-SIMPLIFIED SHELL\0"
  "CJK RADICAL C-SIMPLIFIED SILK\0"
  "CJK RADICAL C-SIMPLIFIED SPEECH\0"
  "CJK RADICAL C-SIMPLIFIED TANNED LEATHER\0"
  "CJK RADICAL C-SIMPLIFIED TOOTH\0"
  "CJK RADICAL C-SIMPLIFIED TURTLE\0"
  "CJK RADICAL C-SIMPLIFIED WIND\0"
  "CJK RADICAL CHOKE\0"
  "CJK RADICAL CITY\0"
  "CJK RADICAL CIVILIAN\0"
  "CJK RADICAL CLIFF\0"
  "CJK RADICAL CLOTHES\0"
  "CJK RADICAL COW\0"
  "CJK RADICAL DEATH\0"
  "CJK RADICAL DIVINATION\0"
  "CJK RADICAL DOG\0"
  "CJK RADICAL EAT ONE\0"
  "CJK RADICAL EAT THREE\0"
  "CJK RADICAL EAT TWO\0"
  "CJK RADICAL EWE\0"
  "CJK RADICAL EYE\0"
  "CJK RADICAL FIRE\0"
  "CJK RADICAL FOOT\0"
  "CJK RADICAL GHOST\0"
  "CJK RADICAL GRASS ONE\0"
  "CJK RADICAL GRASS THREE\0"
  "CJK RADICAL GRASS TWO\0"
  "CJK RADICAL HAND\0"
  "CJK RADICAL HEAD\0"
  "CJK RADICAL HEART ONE\0"
  "CJK RADICAL HEART TWO\0"
  "CJK RADICAL HORN\0"
  "CJK RADICAL J-SIMPLIFIED DRAGON\0"
  "CJK RADICAL J-SIMPLIFIED EVEN\0"
  "CJK RADICAL J-SIMPLIFIED TOOTH\0"
  "CJK RADICAL J-SIMPLIFIED TURTLE\0"
  "CJK RADICAL JADE\0"
  "CJK RADICAL KNIFE ONE\0"
  "CJK RADICAL KNIFE TWO\0"
  "CJK RADICAL LAME FOUR\0"
  "CJK RADICAL LAME ONE\0"
  "CJK RADICAL LAME THREE\0"
  "CJK RADICAL LAME TWO\0"
  "CJK RADICAL LONG ONE\0"
  "CJK RADICAL LONG TWO\0"
  "CJK RADICAL MEAT\0"
  "CJK RADICAL MESH\0"
  "CJK RADICAL MOON\0"
  "CJK RADICAL MORTAR\0"
  "CJK RADICAL MOTHER\0"
  "CJK RADICAL MOUND ONE\0"
  "CJK RADICAL MOUND TWO\0"
  "CJK RADICAL NET FOUR\0"
  "CJK RADICAL NET ONE\0"
  "CJK RADICAL NET THREE\0"
  "CJK RADICAL NET TWO\0"
  "CJK RADICAL OLD\0"
  "CJK RADICAL PAW ONE\0"
  "CJK RADICAL PAW TWO\0"
  "CJK RADICAL PERSON\0"
  "CJK RADICAL RAIN\0"
  "CJK RADICAL RAM\0"
  "CJK RADICAL RAP\0"
  "CJK RADICAL REPEAT\0"
  "CJK RADICAL SEAL\0"
  "CJK RADICAL SECOND ONE\0"
  "CJK RADICAL SECOND THREE\0"
  "CJK RADICAL SECOND TWO\0"
  "CJK RADICAL SHEEP\0"
  "CJK RADICAL SILK\0"
  "CJK RADICAL SIMPLIFIED HALF TREE TRUNK\0"
  "CJK RADICAL SIMPLIFIED HORN\0"
  "CJK RADICAL SIMPLIFIED WALK\0"
  "CJK RADICAL SIMPLIFIED WHEAT\0"
  "CJK RADICAL SIMPLIFIED YELLOW\0"
  "CJK RADICAL SMALL ONE\0"
  "CJK RADICAL SMALL TWO\0"
  "CJK RADICAL SNAKE\0"
  "CJK RADICAL SNOUT ONE\0"
  "CJK RADICAL SNOUT TWO\0"
  "CJK RADICAL SPIRIT ONE\0"
  "CJK RADICAL SPIRIT TWO\0"
  "CJK RADICAL SUN\0"
  "CJK RADICAL TABLE\0"
  "CJK RADICAL THREAD\0"
  "CJK RADICAL TIGER\0"
  "CJK RADICAL TURTLE\0"
  "CJK RADICAL WALK ONE\0"
  "CJK RADICAL WALK TWO\0"
  "CJK RADICAL WATER ONE\0"
  "CJK RADICAL WATER TWO\0"
  "CJK RADICAL WEST ONE\0"
  "CJK RADICAL WEST TWO\0"
  "CJK STROKE BXG\0"
  "CJK STROKE D\0"
  "CJK STROKE H\0"
  "CJK STROKE HG\0"
  "CJK STROKE HP\0"
  "CJK STROKE HPWG\0"
  "CJK STROKE HXG\0"
  "CJK STROKE HXWG\0"
  "CJK STROKE HZ\0"
  "CJK STROKE HZG\0"
  "CJK STROKE HZT\0"
  "CJK STROKE HZW\0"
  "CJK STROKE HZWG\0"
  "CJK STROKE HZZ\0"
  "CJK STROKE HZZP\0"
  "CJK STROKE HZZZ\0"
  "CJK STROKE HZZZG\0"
  "CJK STROKE N\0"
  "CJK STROKE P\0"
  "CJK STROKE PD\0"
  "CJK STROKE PG\0"
  "CJK STROKE PZ\0"
  "CJK STROKE Q\0"
  "CJK STROKE S\0"
  "CJK STROKE SG\0"
  "CJK STROKE SP\0"
  "CJK STROKE ST\0"
  "CJK STROKE SW\0"
  "CJK STROKE SWG\0"
  "CJK STROKE SWZ\0"
  "CJK STROKE SZ\0"
  "CJK STROKE SZP\0"
  "CJK STROKE SZWG\0"
  "CJK STROKE SZZ\0"
  "CJK STROKE T\0"
  "CJK STROKE TN\0"
  "CJK STROKE WG\0"
  "CJK STROKE XG\0"
  "CLAMSHELL MOBILE PHONE\0"
  "CLAPPER BOARD\0"
  "CLAPPING HANDS SIGN\0"
  "CLASSICAL BUILDING\0"
  "CLEAR SCREEN SYMBOL\0"
  "CLINKING BEER MUGS\0"
  "CLINKING GLASSES\0"
  "CLIPBOARD\0"
  "CLOCK FACE EIGHT OCLOCK\0"
  "CLOCK FACE EIGHT-THIRTY\0"
  "CLOCK FACE ELEVEN OCLOCK\0"
  "CLOCK FACE ELEVEN-THIRTY\0"
  "CLOCK FACE FIVE OCLOCK\0"
  "CLOCK FACE FIVE-THIRTY\0"
  "CLOCK FACE FOUR OCLOCK\0"
  "CLOCK FACE FOUR-THIRTY\0"
  "CLOCK FACE NINE OCLOCK\0"
  "CLOCK FACE NINE-THIRTY\0"
  "CLOCK FACE ONE OCLOCK\0"
  "CLOCK FACE ONE-THIRTY\0"
  "CLOCK FACE SEVEN OCLOCK\0"
  "CLOCK FACE SEVEN-THIRTY\0"
  "CLOCK FACE SIX OCLOCK\0"
  "CLOCK FACE SIX-THIRTY\0"
  "CLOCK FACE TEN OCLOCK\0"
  "CLOCK FACE TEN-THIRTY\0"
  "CLOCK FACE THREE OCLOCK\0"
  "CLOCK FACE THREE-THIRTY\0"
  "CLOCK FACE TWELVE OCLOCK\0"
  "CLOCK FACE TWELVE-THIRTY\0"
  "CLOCK FACE TWO OCLOCK\0"
  "CLOCK FACE TWO-THIRTY\0"
  "CLOCKWISE CLOSED CIRCLE ARROW\0"
  "CLOCKWISE CONTOUR INTEGRAL\0"
  "CLOCKWISE DOWNWARDS AND UPWARDS OPEN CIRCLE ARROWS\0"
  "CLOCKWISE GAPPED CIRCLE ARROW\0"
  "CLOCKWISE INTEGRAL\0"
  "CLOCKWISE OPEN CIRCLE ARROW\0"
  "CLOCKWISE RIGHT AND LEFT SEMICIRCLE ARROWS\0"
  "CLOCKWISE RIGHTWARDS AND LEFTWARDS OPEN CIRCLE ARROWS\0"
  "CLOCKWISE RIGHTWARDS AND LEFTWARDS OPEN CIRCLE ARROWS WITH CIRCLED ONE OVERLAY\0"
  "CLOCKWISE TOP SEMICIRCLE ARROW\0"
  "CLOCKWISE TRIANGLE-HEADED OPEN CIRCLE ARROW\0"
  "CLOSE UP\0"
  "CLOSED BOOK\0"
  "CLOSED INTERSECTION WITH SERIFS\0"
  "CLOSED LOCK WITH KEY\0"
  "CLOSED MAILBOX WITH LOWERED FLAG\0"
  "CLOSED MAILBOX WITH RAISED FLAG\0"
  "CLOSED SUBSET\0"
  "CLOSED SUBSET OR EQUAL TO\0"
  "CLOSED SUPERSET\0"
  "CLOSED SUPERSET OR EQUAL TO\0"
  "CLOSED UMBRELLA\0"
  "CLOSED UNION WITH SERIFS\0"
  "CLOSED UNION WITH SERIFS AND SMASH PRODUCT\0"
  "CLOUD\0"
  "CLOUD WITH LIGHTNING\0"
  "CLOUD WITH RAIN\0"
  "CLOUD WITH SNOW\0"
  "CLOUD WITH TORNADO\0"
  "CLOWN FACE\0"
  "COAT\0"
  "COCKROACH\0"
  "COCKTAIL GLASS\0"
  "COCONUT\0"
  "COFFIN\0"
  "COIN\0"
  "COLLISION SYMBOL\0"
  "COLON\0"
  "COLON EQUALS\0"
  "COLON SIGN\0"
  "COMBINING ACUTE ACCENT\0"
  "COMBINING ACUTE ACCENT BELOW\0"
  "COMBINING ACUTE TONE MARK\0"
  "COMBINING ACUTE-GRAVE-ACUTE\0"
  "COMBINING ACUTE-MACRON\0"
  "COMBINING ALMOST EQUAL TO ABOVE\0"
  "COMBINING ALMOST EQUAL TO BELOW\0"
  "COMBINING ANNUITY SYMBOL\0"
  "COMBINING ANTICLOCKWISE ARROW ABOVE\0"
  "COMBINING ANTICLOCKWISE RING OVERLAY\0"
  "COMBINING ASTERISK ABOVE\0"
  "COMBINING ASTERISK BELOW\0"
  "COMBINING BINDU BELOW\0"
  "COMBINING BREVE\0"
  "COMBINING BREVE BELOW\0"
  "COMBINING BREVE-MACRON\0"
  "COMBINING BRIDGE ABOVE\0"
  "COMBINING BRIDGE BELOW\0"
  "COMBINING CANDRABINDU\0"
  "COMBINING CARON\0"
  "COMBINING CARON BELOW\0"
  "COMBINING CEDILLA\0"
  "COMBINING CIRCUMFLEX ACCENT\0"
  "COMBINING CIRCUMFLEX ACCENT BELOW\0"
  "COMBINING CLOCKWISE ARROW ABOVE\0"
  "COMBINING CLOCKWISE RING OVERLAY\0"
  "COMBINING COMMA ABOVE\0"
  "COMBINING COMMA ABOVE RIGHT\0"
  "COMBINING COMMA BELOW\0"
  "COMBINING CONJOINING MACRON\0"
  "COMBINING CONJOINING MACRON BELOW\0"
  "COMBINING CYRILLIC DASIA PNEUMATA\0"
  "COMBINING CYRILLIC HUNDRED MILLIONS SIGN\0"
  "COMBINING CYRILLIC HUNDRED THOUSANDS SIGN\0"
  "COMBINING CYRILLIC KAVYKA\0"
  "COMBINING CYRILLIC LETTER A\0"
  "COMBINING CYRILLIC LETTER BE\0"
  "COMBINING CYRILLIC LETTER BIG YUS\0"
  "COMBINING CYRILLIC LETTER CHE\0"
  "COMBINING CYRILLIC LETTER DE\0"
  "COMBINING CYRILLIC LETTER DJERV\0"
  "COMBINING CYRILLIC LETTER EF\0"
  "COMBINING CYRILLIC LETTER EL\0"
  "COMBINING CYRILLIC LETTER EM\0"
  "COMBINING CYRILLIC LETTER EN\0"
  "COMBINING CYRILLIC LETTER ER\0"
  "COMBINING CYRILLIC LETTER ES\0"
  "COMBINING CYRILLIC LETTER ES-TE\0"
  "COMBINING CYRILLIC LETTER FITA\0"
  "COMBINING CYRILLIC LETTER GHE\0"
  "COMBINING CYRILLIC LETTER HA\0"
  "COMBINING CYRILLIC LETTER HARD SIGN\0"
  "COMBINING CYRILLIC LETTER I\0"
  "COMBINING CYRILLIC LETTER IE\0"
  "COMBINING CYRILLIC LETTER IOTIFIED A\0"
  "COMBINING CYRILLIC LETTER IOTIFIED BIG YUS\0"
  "COMBINING CYRILLIC LETTER IOTIFIED E\0"
  "COMBINING CYRILLIC LETTER KA\0"
  "COMBINING CYRILLIC LETTER LITTLE YUS\0"
  "COMBINING CYRILLIC LETTER MONOGRAPH UK\0"
  "COMBINING CYRILLIC LETTER O\0"
  "COMBINING CYRILLIC LETTER OMEGA\0"
  "COMBINING CYRILLIC LETTER PE\0"
  "COMBINING CYRILLIC LETTER SHA\0"
  "COMBINING CYRILLIC LETTER SHCHA\0"
  "COMBINING CYRILLIC LETTER SOFT SIGN\0"
  "COMBINING CYRILLIC LETTER TE\0"
  "COMBINING CYRILLIC LETTER TSE\0"
  "COMBINING CYRILLIC LETTER U\0"
  "COMBINING CYRILLIC LETTER UKRAINIAN IE\0"
  "COMBINING CYRILLIC LETTER VE\0"
  "COMBINING CYRILLIC LETTER YAT\0"
  "COMBINING CYRILLIC LETTER YERU\0"
  "COMBINING CYRILLIC LETTER YI\0"
  "COMBINING CYRILLIC LETTER YU\0"
  "COMBINING CYRILLIC LETTER ZE\0"
  "COMBINING CYRILLIC LETTER ZHE\0"
  "COMBINING CYRILLIC MILLIONS SIGN\0"
  "COMBINING CYRILLIC PALATALIZATION\0"
  "COMBINING CYRILLIC PAYEROK\0"
  "COMBINING CYRILLIC POKRYTIE\0"
  "COMBINING CYRILLIC PSILI PNEUMATA\0"
  "COMBINING CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I\0"
  "COMBINING CYRILLIC TEN MILLIONS SIGN\0"
  "COMBINING CYRILLIC THOUSAND MILLIONS SIGN\0"
  "COMBINING CYRILLIC TITLO\0"
  "COMBINING CYRILLIC TITLO LEFT HALF\0"
  "COMBINING CYRILLIC TITLO RIGHT HALF\0"
  "COMBINING CYRILLIC VZMET\0"
  "COMBINING DELETION MARK\0"
  "COMBINING DEVANAGARI DIGIT EIGHT\0"
  "COMBINING DEVANAGARI DIGIT FIVE\0"
  "COMBINING DEVANAGARI DIGIT FOUR\0"
  "COMBINING DEVANAGARI DIGIT NINE\0"
  "COMBINING DEVANAGARI DIGIT ONE\0"
  "COMBINING DEVANAGARI DIGIT SEVEN\0"
  "COMBINING DEVANAGARI DIGIT SIX\0"
  "COMBINING DEVANAGARI DIGIT THREE\0"
  "COMBINING DEVANAGARI DIGIT TWO\0"
  "COMBINING DEVANAGARI DIGIT ZERO\0"
  "COMBINING DEVANAGARI LETTER A\0"
  "COMBINING DEVANAGARI LETTER KA\0"
  "COMBINING DEVANAGARI LETTER NA\0"
  "COMBINING DEVANAGARI LETTER PA\0"
  "COMBINING DEVANAGARI LETTER RA\0"
  "COMBINING DEVANAGARI LETTER U\0"
  "COMBINING DEVANAGARI LETTER VI\0"
  "COMBINING DEVANAGARI SIGN AVAGRAHA\0"
  "COMBINING DIAERESIS\0"
  "COMBINING DIAERESIS BELOW\0"
  "COMBINING DIAERESIS-RING\0"
  "COMBINING DOT ABOVE\0"
  "COMBINING DOT ABOVE LEFT\0"
  "COMBINING DOT ABOVE RIGHT\0"
  "COMBINING DOT BELOW\0"
  "COMBINING DOT BELOW LEFT\0"
  "COMBINING DOTTED ACUTE ACCENT\0"
  "COMBINING DOTTED GRAVE ACCENT\0"
  "COMBINING DOUBLE ACUTE ACCENT\0"
  "COMBINING DOUBLE BREVE\0"
  "COMBINING DOUBLE BREVE BELOW\0"
  "COMBINING DOUBLE CIRCUMFLEX ABOVE\0"
  "COMBINING DOUBLE GRAVE ACCENT\0"
  "COMBINING DOUBLE INVERTED BREVE\0"
  "COMBINING DOUBLE INVERTED BREVE BELOW\0"
  "COMBINING DOUBLE LOW LINE\0"
  "COMBINING DOUBLE MACRON\0"
  "COMBINING DOUBLE MACRON BELOW\0"
  "COMBINING DOUBLE OPEN MARK BELOW\0"
  "COMBINING DOUBLE OVERLINE\0"
  "COMBINING DOUBLE PARENTHESES ABOVE\0"
  "COMBINING DOUBLE PLUS SIGN ABOVE\0"
  "COMBINING DOUBLE PLUS SIGN BELOW\0"
  "COMBINING DOUBLE RIGHTWARDS ARROW BELOW\0"
  "COMBINING DOUBLE RING BELOW\0"
  "COMBINING DOUBLE TILDE\0"
  "COMBINING DOUBLE TILDE LEFT HALF\0"
  "COMBINING DOUBLE TILDE RIGHT HALF\0"
  "COMBINING DOUBLE VERTICAL LINE ABOVE\0"
  "COMBINING DOUBLE VERTICAL LINE BELOW\0"
  "COMBINING DOUBLE VERTICAL STROKE OVERLAY\0"
  "COMBINING DOUBLED CIRCUMFLEX ACCENT\0"
  "COMBINING DOWN TACK BELOW\0"
  "COMBINING DOWNWARDS ARROW\0"
  "COMBINING ENCLOSING CIRCLE\0"
  "COMBINING ENCLOSING CIRCLE BACKSLASH\0"
  "COMBINING ENCLOSING DIAMOND\0"
  "COMBINING ENCLOSING KEYCAP\0"
  "COMBINING ENCLOSING SCREEN\0"
  "COMBINING ENCLOSING SQUARE\0"
  "COMBINING ENCLOSING UPWARD POINTING TRIANGLE\0"
  "COMBINING EQUALS SIGN BELOW\0"
  "COMBINING FERMATA\0"
  "COMBINING FOUR DOTS ABOVE\0"
  "COMBINING GLAGOLITIC LETTER AZU\0"
  "COMBINING GLAGOLITIC LETTER BIG YUS\0"
  "COMBINING GLAGOLITIC LETTER BUKY\0"
  "COMBINING GLAGOLITIC LETTER CHRIVI\0"
  "COMBINING GLAGOLITIC LETTER DJERVI\0"
  "COMBINING GLAGOLITIC LETTER DOBRO\0"
  "COMBINING GLAGOLITIC LETTER FITA\0"
  "COMBINING GLAGOLITIC LETTER FRITU\0"
  "COMBINING GLAGOLITIC LETTER GLAGOLI\0"
  "COMBINING GLAGOLITIC LETTER HERU\0"
  "COMBINING GLAGOLITIC LETTER I\0"
  "COMBINING GLAGOLITIC LETTER INITIAL IZHE\0"
  "COMBINING GLAGOLITIC LETTER IOTATED BIG YUS\0"
  "COMBINING GLAGOLITIC LETTER IOTATED SMALL YUS\0"
  "COMBINING GLAGOLITIC LETTER IZHE\0"
  "COMBINING GLAGOLITIC LETTER KAKO\0"
  "COMBINING GLAGOLITIC LETTER LJUDIJE\0"
  "COMBINING GLAGOLITIC LETTER MYSLITE\0"
  "COMBINING GLAGOLITIC LETTER NASHI\0"
  "COMBINING GLAGOLITIC LETTER ONU\0"
  "COMBINING GLAGOLITIC LETTER POKOJI\0"
  "COMBINING GLAGOLITIC LETTER RITSI\0"
  "COMBINING GLAGOLITIC LETTER SHA\0"
  "COMBINING GLAGOLITIC LETTER SHTA\0"
  "COMBINING GLAGOLITIC LETTER SLOVO\0"
  "COMBINING GLAGOLITIC LETTER SMALL YUS\0"
  "COMBINING GLAGOLITIC LETTER TSI\0"
  "COMBINING GLAGOLITIC LETTER TVRIDO\0"
  "COMBINING GLAGOLITIC LETTER UKU\0"
  "COMBINING GLAGOLITIC LETTER VEDE\0"
  "COMBINING GLAGOLITIC LETTER YATI\0"
  "COMBINING GLAGOLITIC LETTER YERI\0"
  "COMBINING GLAGOLITIC LETTER YERU\0"
  "COMBINING GLAGOLITIC LETTER YESTU\0"
  "COMBINING GLAGOLITIC LETTER YO\0"
  "COMBINING GLAGOLITIC LETTER YU\0"
  "COMBINING GLAGOLITIC LETTER ZEMLJA\0"
  "COMBINING GLAGOLITIC LETTER ZHIVETE\0"
  "COMBINING GRANTHA DIGIT FIVE\0"
  "COMBINING GRANTHA DIGIT FOUR\0"
  "COMBINING GRANTHA DIGIT ONE\0"
  "COMBINING GRANTHA DIGIT SIX\0"
  "COMBINING GRANTHA DIGIT THREE\0"
  "COMBINING GRANTHA DIGIT TWO\0"
  "COMBINING GRANTHA DIGIT ZERO\0"
  "COMBINING GRANTHA LETTER A\0"
  "COMBINING GRANTHA LETTER KA\0"
  "COMBINING GRANTHA LETTER NA\0"
  "COMBINING GRANTHA LETTER PA\0"
  "COMBINING GRANTHA LETTER VI\0"
  "COMBINING GRAPHEME JOINER\0"
  "COMBINING GRAVE ACCENT\0"
  "COMBINING GRAVE ACCENT BELOW\0"
  "COMBINING GRAVE TONE MARK\0"
  "COMBINING GRAVE-ACUTE-GRAVE\0"
  "COMBINING GRAVE-MACRON\0"
  "COMBINING GREEK DIALYTIKA TONOS\0"
  "COMBINING GREEK KORONIS\0"
  "COMBINING GREEK MUSICAL PENTASEME\0"
  "COMBINING GREEK MUSICAL TETRASEME\0"
  "COMBINING GREEK MUSICAL TRISEME\0"
  "COMBINING GREEK PERISPOMENI\0"
  "COMBINING GREEK YPOGEGRAMMENI\0"
  "COMBINING HOMOTHETIC ABOVE\0"
  "COMBINING HOOK ABOVE\0"
  "COMBINING HORN\0"
  "COMBINING INFINITY\0"
  "COMBINING INVERTED BREVE\0"
  "COMBINING INVERTED BREVE BELOW\0"
  "COMBINING INVERTED BRIDGE BELOW\0"
  "COMBINING INVERTED DOUBLE ARCH ABOVE\0"
  "COMBINING INVERTED DOUBLE ARCH BELOW\0"
  "COMBINING IS BELOW\0"
  "COMBINING KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK\0"
  "COMBINING KATAKANA-HIRAGANA VOICED SOUND MARK\0"
  "COMBINING KAVYKA ABOVE LEFT\0"
  "COMBINING KAVYKA ABOVE RIGHT\0"
  "COMBINING LATIN LETTER SMALL CAPITAL G\0"
  "COMBINING LATIN LETTER SMALL CAPITAL L\0"
  "COMBINING LATIN LETTER SMALL CAPITAL M\0"
  "COMBINING LATIN LETTER SMALL CAPITAL N\0"
  "COMBINING LATIN LETTER SMALL CAPITAL R\0"
  "COMBINING LATIN SMALL LETTER A\0"
  "COMBINING LATIN SMALL LETTER A WITH DIAERESIS\0"
  "COMBINING LATIN SMALL LETTER AE\0"
  "COMBINING LATIN SMALL LETTER ALPHA\0"
  "COMBINING LATIN SMALL LETTER AO\0"
  "COMBINING LATIN SMALL LETTER AV\0"
  "COMBINING LATIN SMALL LETTER B\0"
  "COMBINING LATIN SMALL LETTER BETA\0"
  "COMBINING LATIN SMALL LETTER C\0"
  "COMBINING LATIN SMALL LETTER C CEDILLA\0"
  "COMBINING LATIN SMALL LETTER D\0"
  "COMBINING LATIN SMALL LETTER E\0"
  "COMBINING LATIN SMALL LETTER ESH\0"
  "COMBINING LATIN SMALL LETTER ETH\0"
  "COMBINING LATIN SMALL LETTER F\0"
  "COMBINING LATIN SMALL LETTER FLATTENED OPEN A ABOVE\0"
  "COMBINING LATIN SMALL LETTER G\0"
  "COMBINING LATIN SMALL LETTER H\0"
  "COMBINING LATIN SMALL LETTER I\0"
  "COMBINING LATIN SMALL LETTER INSULAR D\0"
  "COMBINING LATIN SMALL LETTER INSULAR G\0"
  "COMBINING LATIN SMALL LETTER INSULAR R\0"
  "COMBINING LATIN SMALL LETTER INSULAR T\0"
  "COMBINING LATIN SMALL LETTER K\0"
  "COMBINING LATIN SMALL LETTER L\0"
  "COMBINING LATIN SMALL LETTER L WITH DOUBLE MIDDLE TILDE\0"
  "COMBINING LATIN SMALL LETTER LONG S\0"
  "COMBINING LATIN SMALL LETTER M\0"
  "COMBINING LATIN SMALL LETTER N\0"
  "COMBINING LATIN SMALL LETTER O\0"
  "COMBINING LATIN SMALL LETTER O WITH DIAERESIS\0"
  "COMBINING LATIN SMALL LETTER O WITH LIGHT CENTRALIZATION STROKE\0"
  "COMBINING LATIN SMALL LETTER P\0"
  "COMBINING LATIN SMALL LETTER R\0"
  "COMBINING LATIN SMALL LETTER R BELOW\0"
  "COMBINING LATIN SMALL LETTER R ROTUNDA\0"
  "COMBINING LATIN SMALL LETTER S\0"
  "COMBINING LATIN SMALL LETTER SCHWA\0"
  "COMBINING LATIN SMALL LETTER T\0"
  "COMBINING LATIN SMALL LETTER TURNED W BELOW\0"
  "COMBINING LATIN SMALL LETTER U\0"
  "COMBINING LATIN SMALL LETTER U WITH DIAERESIS\0"
  "COMBINING LATIN SMALL LETTER U WITH LIGHT CENTRALIZATION STROKE\0"
  "COMBINING LATIN SMALL LETTER V\0"
  "COMBINING LATIN SMALL LETTER W\0"
  "COMBINING LATIN SMALL LETTER W BELOW\0"
  "COMBINING LATIN SMALL LETTER X\0"
  "COMBINING LATIN SMALL LETTER Z\0"
  "COMBINING LEFT ANGLE ABOVE\0"
  "COMBINING LEFT ANGLE BELOW\0"
  "COMBINING LEFT ARROW ABOVE\0"
  "COMBINING LEFT ARROW BELOW\0"
  "COMBINING LEFT ARROWHEAD ABOVE\0"
  "COMBINING LEFT ARROWHEAD BELOW\0"
  "COMBINING LEFT HALF RING ABOVE\0"
  "COMBINING LEFT HALF RING BELOW\0"
  "COMBINING LEFT HARPOON ABOVE\0"
  "COMBINING LEFT PARENTHESIS ABOVE LEFT\0"
  "COMBINING LEFT PARENTHESIS BELOW LEFT\0"
  "COMBINING LEFT RIGHT ARROW ABOVE\0"
  "COMBINING LEFT RIGHT ARROW BELOW\0"
  "COMBINING LEFT TACK BELOW\0"
  "COMBINING LEFTWARDS ARROW OVERLAY\0"
  "COMBINING LEFTWARDS HARPOON WITH BARB DOWNWARDS\0"
  "COMBINING LIGATURE LEFT HALF\0"
  "COMBINING LIGATURE LEFT HALF BELOW\0"
  "COMBINING LIGATURE RIGHT HALF\0"
  "COMBINING LIGATURE RIGHT HALF BELOW\0"
  "COMBINING LIGHT CENTRALIZATION STROKE BELOW\0"
  "COMBINING LONG DOUBLE SOLIDUS OVERLAY\0"
  "COMBINING LONG SOLIDUS OVERLAY\0"
  "COMBINING LONG STROKE OVERLAY\0"
  "COMBINING LONG VERTICAL LINE OVERLAY\0"
  "COMBINING LOW LINE\0"
  "COMBINING MACRON\0"
  "COMBINING MACRON BELOW\0"
  "COMBINING MACRON LEFT HALF\0"
  "COMBINING MACRON LEFT HALF BELOW\0"
  "COMBINING MACRON RIGHT HALF\0"
  "COMBINING MACRON RIGHT HALF BELOW\0"
  "COMBINING MACRON-ACUTE\0"
  "COMBINING MACRON-BREVE\0"
  "COMBINING MACRON-GRAVE\0"
  "COMBINING MINUS SIGN BELOW\0"
  "COMBINING NOT TILDE ABOVE\0"
  "COMBINING NUMBER SIGN ABOVE\0"
  "COMBINING OGONEK\0"
  "COMBINING OGONEK ABOVE\0"
  "COMBINING OLD PERMIC LETTER AN\0"
  "COMBINING OLD PERMIC LETTER DOI\0"
  "COMBINING OLD PERMIC LETTER NENOE\0"
  "COMBINING OLD PERMIC LETTER SII\0"
  "COMBINING OLD PERMIC LETTER ZATA\0"
  "COMBINING OPEN MARK BELOW\0"
  "COMBINING OVERLINE\0"
  "COMBINING PALATALIZED HOOK BELOW\0"
  "COMBINING PARENTHESES ABOVE\0"
  "COMBINING PARENTHESES BELOW\0"
  "COMBINING PARENTHESES OVERLAY\0"
  "COMBINING PLUS SIGN ABOVE\0"
  "COMBINING PLUS SIGN BELOW\0"
  "COMBINING RETROFLEX HOOK BELOW\0"
  "COMBINING REVERSE SOLIDUS OVERLAY\0"
  "COMBINING REVERSED COMMA ABOVE\0"
  "COMBINING RIGHT ARROW ABOVE\0"
  "COMBINING RIGHT ARROW BELOW\0"
  "COMBINING RIGHT ARROWHEAD ABOVE\0"
  "COMBINING RIGHT ARROWHEAD AND DOWN ARROWHEAD BELOW\0"
  "COMBINING RIGHT ARROWHEAD AND UP ARROWHEAD BELOW\0"
  "COMBINING RIGHT ARROWHEAD BELOW\0"
  "COMBINING RIGHT HALF RING ABOVE\0"
  "COMBINING RIGHT HALF RING BELOW\0"
  "COMBINING RIGHT HARPOON ABOVE\0"
  "COMBINING RIGHT PARENTHESIS ABOVE RIGHT\0"
  "COMBINING RIGHT PARENTHESIS BELOW RIGHT\0"
  "COMBINING RIGHT TACK BELOW\0"
  "COMBINING RIGHTWARDS HARPOON WITH BARB DOWNWARDS\0"
  "COMBINING RING ABOVE\0"
  "COMBINING RING BELOW\0"
  "COMBINING RING OVERLAY\0"
  "COMBINING SEAGULL BELOW\0"
  "COMBINING SHORT SOLIDUS OVERLAY\0"
  "COMBINING SHORT STROKE OVERLAY\0"
  "COMBINING SHORT VERTICAL LINE OVERLAY\0"
  "COMBINING SNAKE BELOW\0"
  "COMBINING SQUARE BELOW\0"
  "COMBINING SQUARE BRACKETS ABOVE\0"
  "COMBINING STRONG CENTRALIZATION STROKE BELOW\0"
  "COMBINING SUSPENSION MARK\0"
  "COMBINING THREE DOTS ABOVE\0"
  "COMBINING TILDE\0"
  "COMBINING TILDE BELOW\0"
  "COMBINING TILDE LEFT HALF BELOW\0"
  "COMBINING TILDE OVERLAY\0"
  "COMBINING TILDE RIGHT HALF BELOW\0"
  "COMBINING TRIPLE ACUTE ACCENT\0"
  "COMBINING TRIPLE DOT\0"
  "COMBINING TRIPLE UNDERDOT\0"
  "COMBINING TURNED COMMA ABOVE\0"
  "COMBINING UP TACK ABOVE\0"
  "COMBINING UP TACK BELOW\0"
  "COMBINING UPWARDS ARROW BELOW\0"
  "COMBINING UR ABOVE\0"
  "COMBINING US ABOVE\0"
  "COMBINING VERTICAL LINE ABOVE\0"
  "COMBINING VERTICAL LINE BELOW\0"
  "COMBINING VERTICAL TILDE\0"
  "COMBINING WIDE BRIDGE ABOVE\0"
  "COMBINING WIDE INVERTED BRIDGE BELOW\0"
  "COMBINING WIGGLY LINE BELOW\0"
  "COMBINING X ABOVE\0"
  "COMBINING X BELOW\0"
  "COMBINING X-X BELOW\0"
  "COMBINING ZIGZAG ABOVE\0"
  "COMBINING ZIGZAG BELOW\0"
  "COMET\0"
  "COMMA\0"
  "COMMERCIAL AT\0"
  "COMMERCIAL MINUS SIGN\0"
  "COMPASS\0"
  "COMPLEMENT\0"
  "COMPOSITION SYMBOL\0"
  "COMPRESSION\0"
  "CONFETTI BALL\0"
  "CONFOUNDED FACE\0"
  "CONFUSED FACE\0"
  "CONGRUENT WITH DOT ABOVE\0"
  "CONICAL TAPER\0"
  "CONJUNCTION\0"
  "CONSTRUCTION SIGN\0"
  "CONSTRUCTION WORKER\0"
  "CONTAINS AS MEMBER\0"
  "CONTAINS AS NORMAL SUBGROUP\0"
  "CONTAINS AS NORMAL SUBGROUP OR EQUAL TO\0"
  "CONTAINS WITH LONG HORIZONTAL STROKE\0"
  "CONTAINS WITH OVERBAR\0"
  "CONTAINS WITH VERTICAL BAR AT END OF HORIZONTAL STROKE\0"
  "CONTINUOUS UNDERLINE SYMBOL\0"
  "CONTOUR INTEGRAL\0"
  "CONTROL KNOBS\0"
  "CONVENIENCE STORE\0"
  "COOKED RICE\0"
  "COOKIE\0"
  "COOKING\0"
  "COPTIC CAPITAL LETTER AKHMIMIC KHEI\0"
  "COPTIC CAPITAL LETTER ALFA\0"
  "COPTIC CAPITAL LETTER BOHAIRIC KHEI\0"
  "COPTIC CAPITAL LETTER CROSSED SHEI\0"
  "COPTIC CAPITAL LETTER CRYPTOGRAMMIC EIE\0"
  "COPTIC CAPITAL LETTER CRYPTOGRAMMIC GANGIA\0"
  "COPTIC CAPITAL LETTER CRYPTOGRAMMIC NI\0"
  "COPTIC CAPITAL LETTER CRYPTOGRAMMIC SHEI\0"
  "COPTIC CAPITAL LETTER DALDA\0"
  "COPTIC CAPITAL LETTER DEI\0"
  "COPTIC CAPITAL LETTER DIALECT-P ALEF\0"
  "COPTIC CAPITAL LETTER DIALECT-P HORI\0"
  "COPTIC CAPITAL LETTER DIALECT-P KAPA\0"
  "COPTIC CAPITAL LETTER DIALECT-P NI\0"
  "COPTIC CAPITAL LETTER EIE\0"
  "COPTIC CAPITAL LETTER FEI\0"
  "COPTIC CAPITAL LETTER FI\0"
  "COPTIC CAPITAL LETTER GAMMA\0"
  "COPTIC CAPITAL LETTER GANGIA\0"
  "COPTIC CAPITAL LETTER HATE\0"
  "COPTIC CAPITAL LETTER HORI\0"
  "COPTIC CAPITAL LETTER IAUDA\0"
  "COPTIC CAPITAL LETTER KAPA\0"
  "COPTIC CAPITAL LETTER KHEI\0"
  "COPTIC CAPITAL LETTER KHI\0"
  "COPTIC CAPITAL LETTER KSI\0"
  "COPTIC CAPITAL LETTER L-SHAPED HA\0"
  "COPTIC CAPITAL LETTER LAULA\0"
  "COPTIC CAPITAL LETTER MI\0"
  "COPTIC CAPITAL LETTER NI\0"
  "COPTIC CAPITAL LETTER O\0"
  "COPTIC CAPITAL LETTER OLD COPTIC AIN\0"
  "COPTIC CAPITAL LETTER OLD COPTIC DJA\0"
  "COPTIC CAPITAL LETTER OLD COPTIC ESH\0"
  "COPTIC CAPITAL LETTER OLD COPTIC GANGIA\0"
  "COPTIC CAPITAL LETTER OLD COPTIC HA\0"
  "COPTIC CAPITAL LETTER OLD COPTIC HAT\0"
  "COPTIC CAPITAL LETTER OLD COPTIC HEI\0"
  "COPTIC CAPITAL LETTER OLD COPTIC HORI\0"
  "COPTIC CAPITAL LETTER OLD COPTIC OOU\0"
  "COPTIC CAPITAL LETTER OLD COPTIC SHEI\0"
  "COPTIC CAPITAL LETTER OLD COPTIC SHIMA\0"
  "COPTIC CAPITAL LETTER OLD NUBIAN NGI\0"
  "COPTIC CAPITAL LETTER OLD NUBIAN NYI\0"
  "COPTIC CAPITAL LETTER OLD NUBIAN SHIMA\0"
  "COPTIC CAPITAL LETTER OLD NUBIAN WAU\0"
  "COPTIC CAPITAL LETTER OOU\0"
  "COPTIC CAPITAL LETTER PI\0"
  "COPTIC CAPITAL LETTER PSI\0"
  "COPTIC CAPITAL LETTER RO\0"
  "COPTIC CAPITAL LETTER SAMPI\0"
  "COPTIC CAPITAL LETTER SHEI\0"
  "COPTIC CAPITAL LETTER SHIMA\0"
  "COPTIC CAPITAL LETTER SIMA\0"
  "COPTIC CAPITAL LETTER SOU\0"
  "COPTIC CAPITAL LETTER TAU\0"
  "COPTIC CAPITAL LETTER THETHE\0"
  "COPTIC CAPITAL LETTER UA\0"
  "COPTIC CAPITAL LETTER VIDA\0"
  "COPTIC CAPITAL LETTER ZATA\0"
  "COPTIC COMBINING NI ABOVE\0"
  "COPTIC COMBINING SPIRITUS ASPER\0"
  "COPTIC COMBINING SPIRITUS LENIS\0"
  "COPTIC EPACT DIGIT EIGHT\0"
  "COPTIC EPACT DIGIT FIVE\0"
  "COPTIC EPACT DIGIT FOUR\0"
  "COPTIC EPACT DIGIT NINE\0"
  "COPTIC EPACT DIGIT ONE\0"
  "COPTIC EPACT DIGIT SEVEN\0"
  "COPTIC EPACT DIGIT SIX\0"
  "COPTIC EPACT DIGIT THREE\0"
  "COPTIC EPACT DIGIT TWO\0"
  "COPTIC EPACT NUMBER EIGHT HUNDRED\0"
  "COPTIC EPACT NUMBER EIGHTY\0"
  "COPTIC EPACT NUMBER FIFTY\0"
  "COPTIC EPACT NUMBER FIVE HUNDRED\0"
  "COPTIC EPACT NUMBER FORTY\0"
  "COPTIC EPACT NUMBER FOUR HUNDRED\0"
  "COPTIC EPACT NUMBER NINE HUNDRED\0"
  "COPTIC EPACT NUMBER NINETY\0"
  "COPTIC EPACT NUMBER ONE HUNDRED\0"
  "COPTIC EPACT NUMBER SEVEN HUNDRED\0"
  "COPTIC EPACT NUMBER SEVENTY\0"
  "COPTIC EPACT NUMBER SIX HUNDRED\0"
  "COPTIC EPACT NUMBER SIXTY\0"
  "COPTIC EPACT NUMBER TEN\0"
  "COPTIC EPACT NUMBER THIRTY\0"
  "COPTIC EPACT NUMBER THREE HUNDRED\0"
  "COPTIC EPACT NUMBER TWENTY\0"
  "COPTIC EPACT NUMBER TWO HUNDRED\0"
  "COPTIC EPACT THOUSANDS MARK\0"
  "COPTIC FRACTION ONE HALF\0"
  "COPTIC FULL STOP\0"
  "COPTIC MORPHOLOGICAL DIVIDER\0"
  "COPTIC OLD NUBIAN DIRECT QUESTION MARK\0"
  "COPTIC OLD NUBIAN FULL STOP\0"
  "COPTIC OLD NUBIAN INDIRECT QUESTION MARK\0"
  "COPTIC OLD NUBIAN VERSE DIVIDER\0"
  "COPTIC SMALL LETTER AKHMIMIC KHEI\0"
  "COPTIC SMALL LETTER ALFA\0"
  "COPTIC SMALL LETTER BOHAIRIC KHEI\0"
  "COPTIC SMALL LETTER CROSSED SHEI\0"
  "COPTIC SMALL LETTER CRYPTOGRAMMIC EIE\0"
  "COPTIC SMALL LETTER CRYPTOGRAMMIC GANGIA\0"
  "COPTIC SMALL LETTER CRYPTOGRAMMIC NI\0"
  "COPTIC SMALL LETTER CRYPTOGRAMMIC SHEI\0"
  "COPTIC SMALL LETTER DALDA\0"
  "COPTIC SMALL LETTER DEI\0"
  "COPTIC SMALL LETTER DIALECT-P ALEF\0"
  "COPTIC SMALL LETTER DIALECT-P HORI\0"
  "COPTIC SMALL LETTER DIALECT-P KAPA\0"
  "COPTIC SMALL LETTER DIALECT-P NI\0"
  "COPTIC SMALL LETTER EIE\0"
  "COPTIC SMALL LETTER FEI\0"
  "COPTIC SMALL LETTER FI\0"
  "COPTIC SMALL LETTER GAMMA\0"
  "COPTIC SMALL LETTER GANGIA\0"
  "COPTIC SMALL LETTER HATE\0"
  "COPTIC SMALL LETTER HORI\0"
  "COPTIC SMALL LETTER IAUDA\0"
  "COPTIC SMALL LETTER KAPA\0"
  "COPTIC SMALL LETTER KHEI\0"
  "COPTIC SMALL LETTER KHI\0"
  "COPTIC SMALL LETTER KSI\0"
  "COPTIC SMALL LETTER L-SHAPED HA\0"
  "COPTIC SMALL LETTER LAULA\0"
  "COPTIC SMALL LETTER MI\0"
  "COPTIC SMALL LETTER NI\0"
  "COPTIC SMALL LETTER O\0"
  "COPTIC SMALL LETTER OLD COPTIC AIN\0"
  "COPTIC SMALL LETTER OLD COPTIC DJA\0"
  "COPTIC SMALL LETTER OLD COPTIC ESH\0"
  "COPTIC SMALL LETTER OLD COPTIC GANGIA\0"
  "COPTIC SMALL LETTER OLD COPTIC HA\0"
  "COPTIC SMALL LETTER OLD COPTIC HAT\0"
  "COPTIC SMALL LETTER OLD COPTIC HEI\0"
  "COPTIC SMALL LETTER OLD COPTIC HORI\0"
  "COPTIC SMALL LETTER OLD COPTIC OOU\0"
  "COPTIC SMALL LETTER OLD COPTIC SHEI\0"
  "COPTIC SMALL LETTER OLD COPTIC SHIMA\0"
  "COPTIC SMALL LETTER OLD NUBIAN NGI\0"
  "COPTIC SMALL LETTER OLD NUBIAN NYI\0"
  "COPTIC SMALL LETTER OLD NUBIAN SHIMA\0"
  "COPTIC SMALL LETTER OLD NUBIAN WAU\0"
  "COPTIC SMALL LETTER OOU\0"
  "COPTIC SMALL LETTER PI\0"
  "COPTIC SMALL LETTER PSI\0"
  "COPTIC SMALL LETTER RO\0"
  "COPTIC SMALL LETTER SAMPI\0"
  "COPTIC SMALL LETTER SHEI\0"
  "COPTIC SMALL LETTER SHIMA\0"
  "COPTIC SMALL LETTER SIMA\0"
  "COPTIC SMALL LETTER SOU\0"
  "COPTIC SMALL LETTER TAU\0"
  "COPTIC SMALL LETTER THETHE\0"
  "COPTIC SMALL LETTER UA\0"
  "COPTIC SMALL LETTER VIDA\0"
  "COPTIC SMALL LETTER ZATA\0"
  "COPTIC SYMBOL KAI\0"
  "COPTIC SYMBOL KHI RO\0"
  "COPTIC SYMBOL MI RO\0"
  "COPTIC SYMBOL PI RO\0"
  "COPTIC SYMBOL SHIMA SIMA\0"
  "COPTIC SYMBOL STAUROS\0"
  "COPTIC SYMBOL TAU RO\0"
  "COPYLEFT SYMBOL\0"
  "COPYRIGHT SIGN\0"
  "CORAL\0"
  "CORNISH VERSE DIVIDER\0"
  "CORRESPONDS TO\0"
  "COUCH AND LAMP\0"
  "COUNTERBORE\0"
  "COUNTERSINK\0"
  "COUNTING ROD TENS DIGIT EIGHT\0"
  "COUNTING ROD TENS DIGIT FIVE\0"
  "COUNTING ROD TENS DIGIT FOUR\0"
  "COUNTING ROD TENS DIGIT NINE\0"
  "COUNTING ROD TENS DIGIT ONE\0"
  "COUNTING ROD TENS DIGIT SEVEN\0"
  "COUNTING ROD TENS DIGIT SIX\0"
  "COUNTING ROD TENS DIGIT THREE\0"
  "COUNTING ROD TENS DIGIT TWO\0"
  "COUNTING ROD UNIT DIGIT EIGHT\0"
  "COUNTING ROD UNIT DIGIT FIVE\0"
  "COUNTING ROD UNIT DIGIT FOUR\0"
  "COUNTING ROD UNIT DIGIT NINE\0"
  "COUNTING ROD UNIT DIGIT ONE\0"
  "COUNTING ROD UNIT DIGIT SEVEN\0"
  "COUNTING ROD UNIT DIGIT SIX\0"
  "COUNTING ROD UNIT DIGIT THREE\0"
  "COUNTING ROD UNIT DIGIT TWO\0"
  "COUPLE WITH HEART\0"
  "COW\0"
  "COW FACE\0"
  "CRAB\0"
  "CREDIT CARD\0"
  "CRESCENT MOON\0"
  "CRICKET\0"
  "CRICKET BAT AND BALL\0"
  "CROCODILE\0"
  "CROISSANT\0"
  "CROSS MARK\0"
  "CROSS OF JERUSALEM\0"
  "CROSS OF LORRAINE\0"
  "CROSS PATTY WITH LEFT CROSSBAR\0"
  "CROSS PATTY WITH RIGHT CROSSBAR\0"
  "CROSS POMMEE\0"
  "CROSS POMMEE WITH HALF-CIRCLE BELOW\0"
  "CROSSED FLAGS\0"
  "CROSSED NEGATIVE SQUARED LATIN CAPITAL LETTER P\0"
  "CROSSED SWORDS\0"
  "CROSSING LANES\0"
  "CROWN\0"
  "CRUTCH\0"
  "CRUZEIRO SIGN\0"
  "CRYING CAT FACE\0"
  "CRYING FACE\0"
  "CRYSTAL BALL\0"
  "CUBE ROOT\0"
  "CUCUMBER\0"
  "CUNEIFORM NUMERIC SIGN EIGHT ASH\0"
  "CUNEIFORM NUMERIC SIGN EIGHT DISH\0"
  "CUNEIFORM NUMERIC SIGN EIGHT GESH2\0"
  "CUNEIFORM NUMERIC SIGN EIGHT SHAR2\0"
  "CUNEIFORM NUMERIC SIGN EIGHT U\0"
  "CUNEIFORM NUMERIC SIGN EIGHT U VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN EIGHT VARIANT FORM USSU\0"
  "CUNEIFORM NUMERIC SIGN EIGHT VARIANT FORM USSU3\0"
  "CUNEIFORM NUMERIC SIGN ELAMITE FIFTY\0"
  "CUNEIFORM NUMERIC SIGN ELAMITE FORTY\0"
  "CUNEIFORM NUMERIC SIGN ELAMITE ONE THIRD\0"
  "CUNEIFORM NUMERIC SIGN ELAMITE TWO THIRDS\0"
  "CUNEIFORM NUMERIC SIGN FIVE ASH\0"
  "CUNEIFORM NUMERIC SIGN FIVE ASH TENU\0"
  "CUNEIFORM NUMERIC SIGN FIVE BAN2\0"
  "CUNEIFORM NUMERIC SIGN FIVE BAN2 VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN FIVE BURU\0"
  "CUNEIFORM NUMERIC SIGN FIVE DISH\0"
  "CUNEIFORM NUMERIC SIGN FIVE GESH2\0"
  "CUNEIFORM NUMERIC SIGN FIVE GESHU\0"
  "CUNEIFORM NUMERIC SIGN FIVE SHAR2\0"
  "CUNEIFORM NUMERIC SIGN FIVE SHARU\0"
  "CUNEIFORM NUMERIC SIGN FIVE SIXTHS DISH\0"
  "CUNEIFORM NUMERIC SIGN FIVE U\0"
  "CUNEIFORM NUMERIC SIGN FIVE U VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN FOUR ASH\0"
  "CUNEIFORM NUMERIC SIGN FOUR ASH TENU\0"
  "CUNEIFORM NUMERIC SIGN FOUR BAN2\0"
  "CUNEIFORM NUMERIC SIGN FOUR BAN2 VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN FOUR BURU\0"
  "CUNEIFORM NUMERIC SIGN FOUR DISH\0"
  "CUNEIFORM NUMERIC SIGN FOUR GESH2\0"
  "CUNEIFORM NUMERIC SIGN FOUR GESHU\0"
  "CUNEIFORM NUMERIC SIGN FOUR SHAR2\0"
  "CUNEIFORM NUMERIC SIGN FOUR SHARU\0"
  "CUNEIFORM NUMERIC SIGN FOUR U\0"
  "CUNEIFORM NUMERIC SIGN FOUR U VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU\0"
  "CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU A\0"
  "CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU B\0"
  "CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU4\0"
  "CUNEIFORM NUMERIC SIGN NIGIDAESH\0"
  "CUNEIFORM NUMERIC SIGN NIGIDAMIN\0"
  "CUNEIFORM NUMERIC SIGN NINE ASH\0"
  "CUNEIFORM NUMERIC SIGN NINE DISH\0"
  "CUNEIFORM NUMERIC SIGN NINE GESH2\0"
  "CUNEIFORM NUMERIC SIGN NINE SHAR2\0"
  "CUNEIFORM NUMERIC SIGN NINE U\0"
  "CUNEIFORM NUMERIC SIGN NINE U VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU\0"
  "CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU A\0"
  "CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU3\0"
  "CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU4\0"
  "CUNEIFORM NUMERIC SIGN OLD ASSYRIAN ONE QUARTER\0"
  "CUNEIFORM NUMERIC SIGN OLD ASSYRIAN ONE SIXTH\0"
  "CUNEIFORM NUMERIC SIGN ONE BAN2\0"
  "CUNEIFORM NUMERIC SIGN ONE BURU\0"
  "CUNEIFORM NUMERIC SIGN ONE EIGHTH ASH\0"
  "CUNEIFORM NUMERIC SIGN ONE ESHE3\0"
  "CUNEIFORM NUMERIC SIGN ONE GESH2\0"
  "CUNEIFORM NUMERIC SIGN ONE GESHU\0"
  "CUNEIFORM NUMERIC SIGN ONE HALF GUR\0"
  "CUNEIFORM NUMERIC SIGN ONE QUARTER ASH\0"
  "CUNEIFORM NUMERIC SIGN ONE QUARTER GUR\0"
  "CUNEIFORM NUMERIC SIGN ONE SHARU\0"
  "CUNEIFORM NUMERIC SIGN ONE THIRD DISH\0"
  "CUNEIFORM NUMERIC SIGN ONE THIRD VARIANT FORM A\0"
  "CUNEIFORM NUMERIC SIGN SEVEN ASH\0"
  "CUNEIFORM NUMERIC SIGN SEVEN DISH\0"
  "CUNEIFORM NUMERIC SIGN SEVEN GESH2\0"
  "CUNEIFORM NUMERIC SIGN SEVEN SHAR2\0"
  "CUNEIFORM NUMERIC SIGN SEVEN U\0"
  "CUNEIFORM NUMERIC SIGN SEVEN U VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN SEVEN VARIANT FORM IMIN A\0"
  "CUNEIFORM NUMERIC SIGN SEVEN VARIANT FORM IMIN B\0"
  "CUNEIFORM NUMERIC SIGN SEVEN VARIANT FORM IMIN3\0"
  "CUNEIFORM NUMERIC SIGN SHAR2 TIMES GAL PLUS DISH\0"
  "CUNEIFORM NUMERIC SIGN SHAR2 TIMES GAL PLUS MIN\0"
  "CUNEIFORM NUMERIC SIGN SIX ASH\0"
  "CUNEIFORM NUMERIC SIGN SIX ASH TENU\0"
  "CUNEIFORM NUMERIC SIGN SIX DISH\0"
  "CUNEIFORM NUMERIC SIGN SIX GESH2\0"
  "CUNEIFORM NUMERIC SIGN SIX SHAR2\0"
  "CUNEIFORM NUMERIC SIGN SIX U\0"
  "CUNEIFORM NUMERIC SIGN SIX U VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN SIX VARIANT FORM ASH9\0"
  "CUNEIFORM NUMERIC SIGN THREE ASH\0"
  "CUNEIFORM NUMERIC SIGN THREE ASH TENU\0"
  "CUNEIFORM NUMERIC SIGN THREE BAN2\0"
  "CUNEIFORM NUMERIC SIGN THREE BURU\0"
  "CUNEIFORM NUMERIC SIGN THREE BURU VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN THREE DISH\0"
  "CUNEIFORM NUMERIC SIGN THREE GESH2\0"
  "CUNEIFORM NUMERIC SIGN THREE GESHU\0"
  "CUNEIFORM NUMERIC SIGN THREE SHAR2\0"
  "CUNEIFORM NUMERIC SIGN THREE SHAR2 VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN THREE SHARU\0"
  "CUNEIFORM NUMERIC SIGN THREE SHARU VARIANT FORM\0"
  "CUNEIFORM NUMERIC SIGN THREE VARIANT FORM ESH16\0"
  "CUNEIFORM NUMERIC SIGN THREE VARIANT FORM ESH21\0"
  "CUNEIFORM NUMERIC SIGN TWO ASH\0"
  "CUNEIFORM NUMERIC SIGN TWO ASH TENU\0"
  "CUNEIFORM NUMERIC SIGN TWO BAN2\0"
  "CUNEIFORM NUMERIC SIGN TWO BURU\0"
  "CUNEIFORM NUMERIC SIGN TWO ESHE3\0"
  "CUNEIFORM NUMERIC SIGN TWO GESH2\0"
  "CUNEIFORM NUMERIC SIGN TWO GESHU\0"
  "CUNEIFORM NUMERIC SIGN TWO SHAR2\0"
  "CUNEIFORM NUMERIC SIGN TWO SHARU\0"
  "CUNEIFORM NUMERIC SIGN TWO THIRDS DISH\0"
  "CUNEIFORM NUMERIC SIGN TWO THIRDS VARIANT FORM A\0"
  "CUNEIFORM PUNCTUATION SIGN DIAGONAL COLON\0"
  "CUNEIFORM PUNCTUATION SIGN DIAGONAL QUADCOLON\0"
  "CUNEIFORM PUNCTUATION SIGN DIAGONAL TRICOLON\0"
  "CUNEIFORM PUNCTUATION SIGN OLD ASSYRIAN WORD DIVIDER\0"
  "CUNEIFORM PUNCTUATION SIGN VERTICAL COLON\0"
  "CUNEIFORM SIGN A\0"
  "CUNEIFORM SIGN A TIMES A\0"
  "CUNEIFORM SIGN A TIMES BAD\0"
  "CUNEIFORM SIGN A TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN A TIMES HA\0"
  "CUNEIFORM SIGN A TIMES IGI\0"
  "CUNEIFORM SIGN A TIMES LAGAR GUNU\0"
  "CUNEIFORM SIGN A TIMES MUSH\0"
  "CUNEIFORM SIGN A TIMES SAG\0"
  "CUNEIFORM SIGN A2\0"
  "CUNEIFORM SIGN AB\0"
  "CUNEIFORM SIGN AB GUNU\0"
  "CUNEIFORM SIGN AB TIMES ASH2\0"
  "CUNEIFORM SIGN AB TIMES DUN3 GUNU\0"
  "CUNEIFORM SIGN AB TIMES GAL\0"
  "CUNEIFORM SIGN AB TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN AB TIMES HA\0"
  "CUNEIFORM SIGN AB TIMES IGI GUNU\0"
  "CUNEIFORM SIGN AB TIMES IMIN\0"
  "CUNEIFORM SIGN AB TIMES LAGAB\0"
  "CUNEIFORM SIGN AB TIMES NUN\0"
  "CUNEIFORM SIGN AB TIMES NUN TENU\0"
  "CUNEIFORM SIGN AB TIMES SHESH\0"
  "CUNEIFORM SIGN AB TIMES SHU2\0"
  "CUNEIFORM SIGN AB TIMES U PLUS U PLUS U\0"
  "CUNEIFORM SIGN AB2\0"
  "CUNEIFORM SIGN AB2 TIMES A\0"
  "CUNEIFORM SIGN AB2 TIMES BALAG\0"
  "CUNEIFORM SIGN AB2 TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN AB2 TIMES ME PLUS EN\0"
  "CUNEIFORM SIGN AB2 TIMES SHA3\0"
  "CUNEIFORM SIGN AB2 TIMES TAK4\0"
  "CUNEIFORM SIGN AD\0"
  "CUNEIFORM SIGN AD TIMES ESH2\0"
  "CUNEIFORM SIGN AK\0"
  "CUNEIFORM SIGN AK TIMES ERIN2\0"
  "CUNEIFORM SIGN AK TIMES SHITA PLUS GISH\0"
  "CUNEIFORM SIGN AL\0"
  "CUNEIFORM SIGN AL TIMES AL\0"
  "CUNEIFORM SIGN AL TIMES DIM2\0"
  "CUNEIFORM SIGN AL TIMES GISH\0"
  "CUNEIFORM SIGN AL TIMES HA\0"
  "CUNEIFORM SIGN AL TIMES KAD3\0"
  "CUNEIFORM SIGN AL TIMES KI\0"
  "CUNEIFORM SIGN AL TIMES SHE\0"
  "CUNEIFORM SIGN AL TIMES USH\0"
  "CUNEIFORM SIGN ALAN\0"
  "CUNEIFORM SIGN ALEPH\0"
  "CUNEIFORM SIGN AMAR\0"
  "CUNEIFORM SIGN AMAR TIMES KUG\0"
  "CUNEIFORM SIGN AMAR TIMES SHE\0"
  "CUNEIFORM SIGN AN\0"
  "CUNEIFORM SIGN AN OVER AN\0"
  "CUNEIFORM SIGN AN PLUS NAGA OPPOSING AN PLUS NAGA\0"
  "CUNEIFORM SIGN AN PLUS NAGA SQUARED\0"
  "CUNEIFORM SIGN AN THREE TIMES\0"
  "CUNEIFORM SIGN ANSHE\0"
  "CUNEIFORM SIGN APIN\0"
  "CUNEIFORM SIGN ARAD\0"
  "CUNEIFORM SIGN ARAD TIMES KUR\0"
  "CUNEIFORM SIGN ARKAB\0"
  "CUNEIFORM SIGN ASAL2\0"
  "CUNEIFORM SIGN ASH\0"
  "CUNEIFORM SIGN ASH KABA TENU\0"
  "CUNEIFORM SIGN ASH OVER ASH OVER ASH\0"
  "CUNEIFORM SIGN ASH OVER ASH OVER ASH CROSSING ASH OVER ASH OVER ASH\0"
  "CUNEIFORM SIGN ASH OVER ASH TUG2 OVER TUG2 TUG2 OVER TUG2 PAP\0"
  "CUNEIFORM SIGN ASH ZIDA TENU\0"
  "CUNEIFORM SIGN ASH2\0"
  "CUNEIFORM SIGN ASHGAB\0"
  "CUNEIFORM SIGN BA\0"
  "CUNEIFORM SIGN BAD\0"
  "CUNEIFORM SIGN BAD TIMES DISH TENU\0"
  "CUNEIFORM SIGN BAG3\0"
  "CUNEIFORM SIGN BAHAR2\0"
  "CUNEIFORM SIGN BAHAR2 TIMES AB2\0"
  "CUNEIFORM SIGN BAHAR2 TIMES NI\0"
  "CUNEIFORM SIGN BAHAR2 TIMES ZA\0"
  "CUNEIFORM SIGN BAL\0"
  "CUNEIFORM SIGN BAL OVER BAL\0"
  "CUNEIFORM SIGN BALAG\0"
  "CUNEIFORM SIGN BAR\0"
  "CUNEIFORM SIGN BARA2\0"
  "CUNEIFORM SIGN BI\0"
  "CUNEIFORM SIGN BI TIMES A\0"
  "CUNEIFORM SIGN BI TIMES GAR\0"
  "CUNEIFORM SIGN BI TIMES IGI GUNU\0"
  "CUNEIFORM SIGN BU\0"
  "CUNEIFORM SIGN BU CROSSING BU\0"
  "CUNEIFORM SIGN BU OVER BU AB\0"
  "CUNEIFORM SIGN BU OVER BU TIMES NA2\0"
  "CUNEIFORM SIGN BU OVER BU UN\0"
  "CUNEIFORM SIGN BULUG\0"
  "CUNEIFORM SIGN BULUG OVER BULUG\0"
  "CUNEIFORM SIGN BUR\0"
  "CUNEIFORM SIGN BUR2\0"
  "CUNEIFORM SIGN DA\0"
  "CUNEIFORM SIGN DA TIMES TAK4\0"
  "CUNEIFORM SIGN DAG\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES A PLUS MASH\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES AMAR\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES BALAG\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES BI\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES GA\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES GA PLUS MASH\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES GI\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES GIR2\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES GUD\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES HA\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES IR\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES IR PLUS LU\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES KAK\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES LA\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES LU\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES LU PLUS MASH2\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES LUM\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES NE\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES PAP PLUS PAP\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES SI\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES TAK4\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES U2 PLUS GIR2\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES U2 PLUS MASH\0"
  "CUNEIFORM SIGN DAG KISIM5 TIMES USH\0"
  "CUNEIFORM SIGN DAG TIMES KUR\0"
  "CUNEIFORM SIGN DAG3\0"
  "CUNEIFORM SIGN DAM\0"
  "CUNEIFORM SIGN DAR\0"
  "CUNEIFORM SIGN DARA3\0"
  "CUNEIFORM SIGN DARA4\0"
  "CUNEIFORM SIGN DI\0"
  "CUNEIFORM SIGN DIB\0"
  "CUNEIFORM SIGN DIM\0"
  "CUNEIFORM SIGN DIM TIMES IGI\0"
  "CUNEIFORM SIGN DIM TIMES SHE\0"
  "CUNEIFORM SIGN DIM TIMES U U U\0"
  "CUNEIFORM SIGN DIM2\0"
  "CUNEIFORM SIGN DIM2 TIMES UD\0"
  "CUNEIFORM SIGN DIN\0"
  "CUNEIFORM SIGN DIN KASKAL U GUNU DISH\0"
  "CUNEIFORM SIGN DISH\0"
  "CUNEIFORM SIGN DISH PLUS SHU\0"
  "CUNEIFORM SIGN DU\0"
  "CUNEIFORM SIGN DU GUNU\0"
  "CUNEIFORM SIGN DU OVER DU\0"
  "CUNEIFORM SIGN DU SHESHIG\0"
  "CUNEIFORM SIGN DUB\0"
  "CUNEIFORM SIGN DUB TIMES ESH2\0"
  "CUNEIFORM SIGN DUB TIMES SHE\0"
  "CUNEIFORM SIGN DUB2\0"
  "CUNEIFORM SIGN DUG\0"
  "CUNEIFORM SIGN DUG TIMES ANSHE\0"
  "CUNEIFORM SIGN DUG TIMES ASH\0"
  "CUNEIFORM SIGN DUG TIMES ASH AT LEFT\0"
  "CUNEIFORM SIGN DUG TIMES DIN\0"
  "CUNEIFORM SIGN DUG TIMES DUN\0"
  "CUNEIFORM SIGN DUG TIMES ERIN2\0"
  "CUNEIFORM SIGN DUG TIMES GA\0"
  "CUNEIFORM SIGN DUG TIMES GI\0"
  "CUNEIFORM SIGN DUG TIMES GIR2 GUNU\0"
  "CUNEIFORM SIGN DUG TIMES GISH\0"
  "CUNEIFORM SIGN DUG TIMES HA\0"
  "CUNEIFORM SIGN DUG TIMES HI\0"
  "CUNEIFORM SIGN DUG TIMES IGI GUNU\0"
  "CUNEIFORM SIGN DUG TIMES KASKAL\0"
  "CUNEIFORM SIGN DUG TIMES KUR\0"
  "CUNEIFORM SIGN DUG TIMES KUSHU2\0"
  "CUNEIFORM SIGN DUG TIMES KUSHU2 PLUS KASKAL\0"
  "CUNEIFORM SIGN DUG TIMES LAK-020\0"
  "CUNEIFORM SIGN DUG TIMES LAM\0"
  "CUNEIFORM SIGN DUG TIMES LAM TIMES KUR\0"
  "CUNEIFORM SIGN DUG TIMES LUH PLUS GISH\0"
  "CUNEIFORM SIGN DUG TIMES MASH\0"
  "CUNEIFORM SIGN DUG TIMES MES\0"
  "CUNEIFORM SIGN DUG TIMES MI\0"
  "CUNEIFORM SIGN DUG TIMES NI\0"
  "CUNEIFORM SIGN DUG TIMES PI\0"
  "CUNEIFORM SIGN DUG TIMES SHE\0"
  "CUNEIFORM SIGN DUG TIMES SI GUNU\0"
  "CUNEIFORM SIGN DUGUD\0"
  "CUNEIFORM SIGN DUH\0"
  "CUNEIFORM SIGN DUN\0"
  "CUNEIFORM SIGN DUN3\0"
  "CUNEIFORM SIGN DUN3 GUNU\0"
  "CUNEIFORM SIGN DUN3 GUNU GUNU\0"
  "CUNEIFORM SIGN DUN4\0"
  "CUNEIFORM SIGN DUR2\0"
  "CUNEIFORM SIGN E\0"
  "CUNEIFORM SIGN E OVER E NUN OVER NUN\0"
  "CUNEIFORM SIGN E TIMES PAP\0"
  "CUNEIFORM SIGN E2\0"
  "CUNEIFORM SIGN E2 TIMES A PLUS HA PLUS DA\0"
  "CUNEIFORM SIGN E2 TIMES GAR\0"
  "CUNEIFORM SIGN E2 TIMES KUR\0"
  "CUNEIFORM SIGN E2 TIMES MI\0"
  "CUNEIFORM SIGN E2 TIMES PAP\0"
  "CUNEIFORM SIGN E2 TIMES SAL\0"
  "CUNEIFORM SIGN E2 TIMES SHE\0"
  "CUNEIFORM SIGN E2 TIMES U\0"
  "CUNEIFORM SIGN EDIN\0"
  "CUNEIFORM SIGN EGIR\0"
  "CUNEIFORM SIGN EL\0"
  "CUNEIFORM SIGN EN\0"
  "CUNEIFORM SIGN EN CROSSING EN\0"
  "CUNEIFORM SIGN EN OPPOSING EN\0"
  "CUNEIFORM SIGN EN SQUARED\0"
  "CUNEIFORM SIGN EN TIMES GAN2\0"
  "CUNEIFORM SIGN EN TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN EN TIMES ME\0"
  "CUNEIFORM SIGN EREN\0"
  "CUNEIFORM SIGN ERIN2\0"
  "CUNEIFORM SIGN ERIN2 X\0"
  "CUNEIFORM SIGN ESH2\0"
  "CUNEIFORM SIGN ESH2 CROSSING ESH2\0"
  "CUNEIFORM SIGN EZEN\0"
  "CUNEIFORM SIGN EZEN SHESHIG TIMES ASH\0"
  "CUNEIFORM SIGN EZEN SHESHIG TIMES HI\0"
  "CUNEIFORM SIGN EZEN SHESHIG TIMES IGI GUNU\0"
  "CUNEIFORM SIGN EZEN SHESHIG TIMES LA\0"
  "CUNEIFORM SIGN EZEN SHESHIG TIMES LAL\0"
  "CUNEIFORM SIGN EZEN SHESHIG TIMES ME\0"
  "CUNEIFORM SIGN EZEN SHESHIG TIMES MES\0"
  "CUNEIFORM SIGN EZEN SHESHIG TIMES SU\0"
  "CUNEIFORM SIGN EZEN TIMES A\0"
  "CUNEIFORM SIGN EZEN TIMES A PLUS LAL\0"
  "CUNEIFORM SIGN EZEN TIMES A PLUS LAL TIMES LAL\0"
  "CUNEIFORM SIGN EZEN TIMES AN\0"
  "CUNEIFORM SIGN EZEN TIMES BAD\0"
  "CUNEIFORM SIGN EZEN TIMES DUN3 GUNU\0"
  "CUNEIFORM SIGN EZEN TIMES DUN3 GUNU GUNU\0"
  "CUNEIFORM SIGN EZEN TIMES GUD\0"
  "CUNEIFORM SIGN EZEN TIMES HA\0"
  "CUNEIFORM SIGN EZEN TIMES HA GUNU\0"
  "CUNEIFORM SIGN EZEN TIMES IGI GUNU\0"
  "CUNEIFORM SIGN EZEN TIMES KASKAL\0"
  "CUNEIFORM SIGN EZEN TIMES KASKAL SQUARED\0"
  "CUNEIFORM SIGN EZEN TIMES KU3\0"
  "CUNEIFORM SIGN EZEN TIMES LA\0"
  "CUNEIFORM SIGN EZEN TIMES LAL TIMES LAL\0"
  "CUNEIFORM SIGN EZEN TIMES LI\0"
  "CUNEIFORM SIGN EZEN TIMES LU\0"
  "CUNEIFORM SIGN EZEN TIMES SHE\0"
  "CUNEIFORM SIGN EZEN TIMES SU\0"
  "CUNEIFORM SIGN EZEN TIMES U2\0"
  "CUNEIFORM SIGN EZEN TIMES UD\0"
  "CUNEIFORM SIGN GA\0"
  "CUNEIFORM SIGN GA GUNU\0"
  "CUNEIFORM SIGN GA2\0"
  "CUNEIFORM SIGN GA2 OVER GA2\0"
  "CUNEIFORM SIGN GA2 TIMES A PLUS DA PLUS HA\0"
  "CUNEIFORM SIGN GA2 TIMES A PLUS HA\0"
  "CUNEIFORM SIGN GA2 TIMES A PLUS IGI\0"
  "CUNEIFORM SIGN GA2 TIMES AB2 TENU PLUS TAB\0"
  "CUNEIFORM SIGN GA2 TIMES AN\0"
  "CUNEIFORM SIGN GA2 TIMES AN PLUS KAK PLUS A\0"
  "CUNEIFORM SIGN GA2 TIMES ASH\0"
  "CUNEIFORM SIGN GA2 TIMES ASH2\0"
  "CUNEIFORM SIGN GA2 TIMES ASH2 PLUS GAL\0"
  "CUNEIFORM SIGN GA2 TIMES BAD\0"
  "CUNEIFORM SIGN GA2 TIMES BAHAR2\0"
  "CUNEIFORM SIGN GA2 TIMES BAR PLUS RA\0"
  "CUNEIFORM SIGN GA2 TIMES BUR\0"
  "CUNEIFORM SIGN GA2 TIMES BUR PLUS RA\0"
  "CUNEIFORM SIGN GA2 TIMES DA\0"
  "CUNEIFORM SIGN GA2 TIMES DI\0"
  "CUNEIFORM SIGN GA2 TIMES DIM GUNU\0"
  "CUNEIFORM SIGN GA2 TIMES DIM TIMES SHE\0"
  "CUNEIFORM SIGN GA2 TIMES DUB\0"
  "CUNEIFORM SIGN GA2 TIMES DUG TIMES IGI GUNU\0"
  "CUNEIFORM SIGN GA2 TIMES DUG TIMES KASKAL\0"
  "CUNEIFORM SIGN GA2 TIMES EL\0"
  "CUNEIFORM SIGN GA2 TIMES EL PLUS LA\0"
  "CUNEIFORM SIGN GA2 TIMES EN\0"
  "CUNEIFORM SIGN GA2 TIMES EN TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN GA2 TIMES EREN\0"
  "CUNEIFORM SIGN GA2 TIMES GA\0"
  "CUNEIFORM SIGN GA2 TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN GA2 TIMES GAR\0"
  "CUNEIFORM SIGN GA2 TIMES GAR PLUS DI\0"
  "CUNEIFORM SIGN GA2 TIMES GAR PLUS NE\0"
  "CUNEIFORM SIGN GA2 TIMES GI\0"
  "CUNEIFORM SIGN GA2 TIMES GI4\0"
  "CUNEIFORM SIGN GA2 TIMES GI4 PLUS A\0"
  "CUNEIFORM SIGN GA2 TIMES GIR2 PLUS SU\0"
  "CUNEIFORM SIGN GA2 TIMES HA PLUS A\0"
  "CUNEIFORM SIGN GA2 TIMES HA PLUS LU PLUS ESH2\0"
  "CUNEIFORM SIGN GA2 TIMES HAL\0"
  "CUNEIFORM SIGN GA2 TIMES HAL PLUS LA\0"
  "CUNEIFORM SIGN GA2 TIMES HI PLUS LI\0"
  "CUNEIFORM SIGN GA2 TIMES HUB2\0"
  "CUNEIFORM SIGN GA2 TIMES IGI GUNU\0"
  "CUNEIFORM SIGN GA2 TIMES ISH PLUS HU PLUS ASH\0"
  "CUNEIFORM SIGN GA2 TIMES KAK\0"
  "CUNEIFORM SIGN GA2 TIMES KASKAL\0"
  "CUNEIFORM SIGN GA2 TIMES KID\0"
  "CUNEIFORM SIGN GA2 TIMES KID PLUS LAL\0"
  "CUNEIFORM SIGN GA2 TIMES KU3 PLUS AN\0"
  "CUNEIFORM SIGN GA2 TIMES KUSHU2 PLUS KASKAL\0"
  "CUNEIFORM SIGN GA2 TIMES LA\0"
  "CUNEIFORM SIGN GA2 TIMES LAM\0"
  "CUNEIFORM SIGN GA2 TIMES LAM TIMES KUR\0"
  "CUNEIFORM SIGN GA2 TIMES LUH\0"
  "CUNEIFORM SIGN GA2 TIMES ME PLUS EN\0"
  "CUNEIFORM SIGN GA2 TIMES MI\0"
  "CUNEIFORM SIGN GA2 TIMES MUSH\0"
  "CUNEIFORM SIGN GA2 TIMES NE\0"
  "CUNEIFORM SIGN GA2 TIMES NE PLUS E2\0"
  "CUNEIFORM SIGN GA2 TIMES NE PLUS GI\0"
  "CUNEIFORM SIGN GA2 TIMES NUN\0"
  "CUNEIFORM SIGN GA2 TIMES NUN OVER NUN\0"
  "CUNEIFORM SIGN GA2 TIMES PA\0"
  "CUNEIFORM SIGN GA2 TIMES SAL\0"
  "CUNEIFORM SIGN GA2 TIMES SAR\0"
  "CUNEIFORM SIGN GA2 TIMES SHE\0"
  "CUNEIFORM SIGN GA2 TIMES SHE PLUS TUR\0"
  "CUNEIFORM SIGN GA2 TIMES SHID\0"
  "CUNEIFORM SIGN GA2 TIMES SHIM\0"
  "CUNEIFORM SIGN GA2 TIMES SUM\0"
  "CUNEIFORM SIGN GA2 TIMES TAK4\0"
  "CUNEIFORM SIGN GA2 TIMES U\0"
  "CUNEIFORM SIGN GA2 TIMES UD\0"
  "CUNEIFORM SIGN GA2 TIMES UD PLUS DU\0"
  "CUNEIFORM SIGN GA2 TIMES ZIZ2\0"
  "CUNEIFORM SIGN GABA\0"
  "CUNEIFORM SIGN GABA CROSSING GABA\0"
  "CUNEIFORM SIGN GABA ROTATED NINETY DEGREES\0"
  "CUNEIFORM SIGN GAD\0"
  "CUNEIFORM SIGN GAD OVER GAD GAR OVER GAR\0"
  "CUNEIFORM SIGN GAL\0"
  "CUNEIFORM SIGN GAL GAD OVER GAD GAR OVER GAR\0"
  "CUNEIFORM SIGN GALAM\0"
  "CUNEIFORM SIGN GAM\0"
  "CUNEIFORM SIGN GAN\0"
  "CUNEIFORM SIGN GAN2\0"
  "CUNEIFORM SIGN GAN2 CROSSING GAN2\0"
  "CUNEIFORM SIGN GAN2 OVER GAN2\0"
  "CUNEIFORM SIGN GAN2 TENU\0"
  "CUNEIFORM SIGN GAR\0"
  "CUNEIFORM SIGN GAR3\0"
  "CUNEIFORM SIGN GASHAN\0"
  "CUNEIFORM SIGN GE22\0"
  "CUNEIFORM SIGN GESHTIN\0"
  "CUNEIFORM SIGN GESHTIN TIMES KUR\0"
  "CUNEIFORM SIGN GESHTIN TIMES U\0"
  "CUNEIFORM SIGN GI\0"
  "CUNEIFORM SIGN GI CROSSING GI\0"
  "CUNEIFORM SIGN GI TIMES E\0"
  "CUNEIFORM SIGN GI TIMES U\0"
  "CUNEIFORM SIGN GI4\0"
  "CUNEIFORM SIGN GI4 CROSSING GI4\0"
  "CUNEIFORM SIGN GI4 OVER GI4\0"
  "CUNEIFORM SIGN GIDIM\0"
  "CUNEIFORM SIGN GIG\0"
  "CUNEIFORM SIGN GIR2\0"
  "CUNEIFORM SIGN GIR2 GUNU\0"
  "CUNEIFORM SIGN GIR3\0"
  "CUNEIFORM SIGN GIR3 TIMES A PLUS IGI\0"
  "CUNEIFORM SIGN GIR3 TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN GIR3 TIMES IGI\0"
  "CUNEIFORM SIGN GIR3 TIMES LU PLUS IGI\0"
  "CUNEIFORM SIGN GIR3 TIMES PA\0"
  "CUNEIFORM SIGN GISAL\0"
  "CUNEIFORM SIGN GISH\0"
  "CUNEIFORM SIGN GISH CROSSING GISH\0"
  "CUNEIFORM SIGN GISH TENU\0"
  "CUNEIFORM SIGN GISH TIMES BAD\0"
  "CUNEIFORM SIGN GISH TIMES GISH CROSSING GISH\0"
  "CUNEIFORM SIGN GISH TIMES TAK4\0"
  "CUNEIFORM SIGN GU\0"
  "CUNEIFORM SIGN GU CROSSING GU\0"
  "CUNEIFORM SIGN GU2\0"
  "CUNEIFORM SIGN GU2 GUNU\0"
  "CUNEIFORM SIGN GU2 TIMES IGI GUNU\0"
  "CUNEIFORM SIGN GU2 TIMES KAK\0"
  "CUNEIFORM SIGN GU2 TIMES KAK TIMES IGI GUNU\0"
  "CUNEIFORM SIGN GU2 TIMES NUN\0"
  "CUNEIFORM SIGN GU2 TIMES SAL PLUS TUG2\0"
  "CUNEIFORM SIGN GUD\0"
  "CUNEIFORM SIGN GUD OVER GUD LUGAL\0"
  "CUNEIFORM SIGN GUD PLUS GISH TIMES TAK4\0"
  "CUNEIFORM SIGN GUD TIMES A PLUS KUR\0"
  "CUNEIFORM SIGN GUD TIMES KUR\0"
  "CUNEIFORM SIGN GUL\0"
  "CUNEIFORM SIGN GUM\0"
  "CUNEIFORM SIGN GUM TIMES SHE\0"
  "CUNEIFORM SIGN GUR\0"
  "CUNEIFORM SIGN GUR7\0"
  "CUNEIFORM SIGN GURUN\0"
  "CUNEIFORM SIGN GURUSH\0"
  "CUNEIFORM SIGN HA\0"
  "CUNEIFORM SIGN HA GUNU\0"
  "CUNEIFORM SIGN HA TENU\0"
  "CUNEIFORM SIGN HA TENU GUNU\0"
  "CUNEIFORM SIGN HAL\0"
  "CUNEIFORM SIGN HI\0"
  "CUNEIFORM SIGN HI TIMES ASH\0"
  "CUNEIFORM SIGN HI TIMES ASH OVER HI TIMES ASH\0"
  "CUNEIFORM SIGN HI TIMES ASH2\0"
  "CUNEIFORM SIGN HI TIMES BAD\0"
  "CUNEIFORM SIGN HI TIMES DISH\0"
  "CUNEIFORM SIGN HI TIMES GAD\0"
  "CUNEIFORM SIGN HI TIMES KIN\0"
  "CUNEIFORM SIGN HI TIMES NUN\0"
  "CUNEIFORM SIGN HI TIMES SHE\0"
  "CUNEIFORM SIGN HI TIMES U\0"
  "CUNEIFORM SIGN HU\0"
  "CUNEIFORM SIGN HUB2\0"
  "CUNEIFORM SIGN HUB2 TIMES AN\0"
  "CUNEIFORM SIGN HUB2 TIMES HAL\0"
  "CUNEIFORM SIGN HUB2 TIMES KASKAL\0"
  "CUNEIFORM SIGN HUB2 TIMES LISH\0"
  "CUNEIFORM SIGN HUB2 TIMES UD\0"
  "CUNEIFORM SIGN HUL2\0"
  "CUNEIFORM SIGN HUSH\0"
  "CUNEIFORM SIGN I\0"
  "CUNEIFORM SIGN I A\0"
  "CUNEIFORM SIGN IB\0"
  "CUNEIFORM SIGN IDIM\0"
  "CUNEIFORM SIGN IDIM OVER IDIM BUR\0"
  "CUNEIFORM SIGN IDIM OVER IDIM SQUARED\0"
  "CUNEIFORM SIGN IG\0"
  "CUNEIFORM SIGN IGI\0"
  "CUNEIFORM SIGN IGI DIB\0"
  "CUNEIFORM SIGN IGI GUNU\0"
  "CUNEIFORM SIGN IGI OVER IGI SHIR OVER SHIR UD OVER UD\0"
  "CUNEIFORM SIGN IGI RI\0"
  "CUNEIFORM SIGN IL\0"
  "CUNEIFORM SIGN IL TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN IL2\0"
  "CUNEIFORM SIGN IM\0"
  "CUNEIFORM SIGN IM CROSSING IM\0"
  "CUNEIFORM SIGN IM OPPOSING IM\0"
  "CUNEIFORM SIGN IM SQUARED\0"
  "CUNEIFORM SIGN IM TIMES TAK4\0"
  "CUNEIFORM SIGN IMIN\0"
  "CUNEIFORM SIGN IN\0"
  "CUNEIFORM SIGN IR\0"
  "CUNEIFORM SIGN ISH\0"
  "CUNEIFORM SIGN KA\0"
  "CUNEIFORM SIGN KA TIMES A\0"
  "CUNEIFORM SIGN KA TIMES AD\0"
  "CUNEIFORM SIGN KA TIMES AD PLUS KU3\0"
  "CUNEIFORM SIGN KA TIMES ANSHE\0"
  "CUNEIFORM SIGN KA TIMES ASH2\0"
  "CUNEIFORM SIGN KA TIMES ASH3\0"
  "CUNEIFORM SIGN KA TIMES BAD\0"
  "CUNEIFORM SIGN KA TIMES BALAG\0"
  "CUNEIFORM SIGN KA TIMES BAR\0"
  "CUNEIFORM SIGN KA TIMES BI\0"
  "CUNEIFORM SIGN KA TIMES BU\0"
  "CUNEIFORM SIGN KA TIMES ERIN2\0"
  "CUNEIFORM SIGN KA TIMES ESH2\0"
  "CUNEIFORM SIGN KA TIMES GA\0"
  "CUNEIFORM SIGN KA TIMES GAL\0"
  "CUNEIFORM SIGN KA TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN KA TIMES GAR\0"
  "CUNEIFORM SIGN KA TIMES GAR PLUS SHA3 PLUS A\0"
  "CUNEIFORM SIGN KA TIMES GI\0"
  "CUNEIFORM SIGN KA TIMES GIR2\0"
  "CUNEIFORM SIGN KA TIMES GISH\0"
  "CUNEIFORM SIGN KA TIMES GISH CROSSING GISH\0"
  "CUNEIFORM SIGN KA TIMES GISH PLUS SAR\0"
  "CUNEIFORM SIGN KA TIMES GU\0"
  "CUNEIFORM SIGN KA TIMES GUD\0"
  "CUNEIFORM SIGN KA TIMES GUR7\0"
  "CUNEIFORM SIGN KA TIMES HI TIMES ASH2\0"
  "CUNEIFORM SIGN KA TIMES IGI\0"
  "CUNEIFORM SIGN KA TIMES IM\0"
  "CUNEIFORM SIGN KA TIMES KA\0"
  "CUNEIFORM SIGN KA TIMES KAK\0"
  "CUNEIFORM SIGN KA TIMES KI\0"
  "CUNEIFORM SIGN KA TIMES KID\0"
  "CUNEIFORM SIGN KA TIMES LI\0"
  "CUNEIFORM SIGN KA TIMES LU\0"
  "CUNEIFORM SIGN KA TIMES LUM\0"
  "CUNEIFORM SIGN KA TIMES ME\0"
  "CUNEIFORM SIGN KA TIMES ME PLUS DU\0"
  "CUNEIFORM SIGN KA TIMES ME PLUS GI\0"
  "CUNEIFORM SIGN KA TIMES ME PLUS TE\0"
  "CUNEIFORM SIGN KA TIMES MI\0"
  "CUNEIFORM SIGN KA TIMES MI PLUS NUNUZ\0"
  "CUNEIFORM SIGN KA TIMES NE\0"
  "CUNEIFORM SIGN KA TIMES NUN\0"
  "CUNEIFORM SIGN KA TIMES PA\0"
  "CUNEIFORM SIGN KA TIMES PI\0"
  "CUNEIFORM SIGN KA TIMES RU\0"
  "CUNEIFORM SIGN KA TIMES SA\0"
  "CUNEIFORM SIGN KA TIMES SAR\0"
  "CUNEIFORM SIGN KA TIMES SHA\0"
  "CUNEIFORM SIGN KA TIMES SHE\0"
  "CUNEIFORM SIGN KA TIMES SHID\0"
  "CUNEIFORM SIGN KA TIMES SHU\0"
  "CUNEIFORM SIGN KA TIMES SHUL\0"
  "CUNEIFORM SIGN KA TIMES SIG\0"
  "CUNEIFORM SIGN KA TIMES SUHUR\0"
  "CUNEIFORM SIGN KA TIMES TAR\0"
  "CUNEIFORM SIGN KA TIMES TU\0"
  "CUNEIFORM SIGN KA TIMES U\0"
  "CUNEIFORM SIGN KA TIMES U U U\0"
  "CUNEIFORM SIGN KA TIMES U2\0"
  "CUNEIFORM SIGN KA TIMES UD\0"
  "CUNEIFORM SIGN KA TIMES UMUM TIMES PA\0"
  "CUNEIFORM SIGN KA TIMES UR\0"
  "CUNEIFORM SIGN KA TIMES UR2\0"
  "CUNEIFORM SIGN KA TIMES USH\0"
  "CUNEIFORM SIGN KA TIMES ZI\0"
  "CUNEIFORM SIGN KA2\0"
  "CUNEIFORM SIGN KA2 CROSSING KA2\0"
  "CUNEIFORM SIGN KAB\0"
  "CUNEIFORM SIGN KAD2\0"
  "CUNEIFORM SIGN KAD3\0"
  "CUNEIFORM SIGN KAD4\0"
  "CUNEIFORM SIGN KAD5\0"
  "CUNEIFORM SIGN KAD5 OVER KAD5\0"
  "CUNEIFORM SIGN KAK\0"
  "CUNEIFORM SIGN KAK TIMES IGI GUNU\0"
  "CUNEIFORM SIGN KAL\0"
  "CUNEIFORM SIGN KAL CROSSING KAL\0"
  "CUNEIFORM SIGN KAL TIMES BAD\0"
  "CUNEIFORM SIGN KAM2\0"
  "CUNEIFORM SIGN KAM4\0"
  "CUNEIFORM SIGN KAP ELAMITE\0"
  "CUNEIFORM SIGN KASKAL\0"
  "CUNEIFORM SIGN KASKAL LAGAB TIMES U OVER LAGAB TIMES U\0"
  "CUNEIFORM SIGN KASKAL OVER KASKAL LAGAB TIMES U OVER LAGAB TIMES U\0"
  "CUNEIFORM SIGN KESH2\0"
  "CUNEIFORM SIGN KI\0"
  "CUNEIFORM SIGN KI TIMES BAD\0"
  "CUNEIFORM SIGN KI TIMES U\0"
  "CUNEIFORM SIGN KI TIMES UD\0"
  "CUNEIFORM SIGN KID\0"
  "CUNEIFORM SIGN KIN\0"
  "CUNEIFORM SIGN KISAL\0"
  "CUNEIFORM SIGN KISH\0"
  "CUNEIFORM SIGN KISIM5\0"
  "CUNEIFORM SIGN KISIM5 OVER KISIM5\0"
  "CUNEIFORM SIGN KU\0"
  "CUNEIFORM SIGN KU OVER HI TIMES ASH2 KU OVER HI TIMES ASH2\0"
  "CUNEIFORM SIGN KU3\0"
  "CUNEIFORM SIGN KU4\0"
  "CUNEIFORM SIGN KU4 VARIANT FORM\0"
  "CUNEIFORM SIGN KU7\0"
  "CUNEIFORM SIGN KUL\0"
  "CUNEIFORM SIGN KUL GUNU\0"
  "CUNEIFORM SIGN KUN\0"
  "CUNEIFORM SIGN KUR\0"
  "CUNEIFORM SIGN KUR OPPOSING KUR\0"
  "CUNEIFORM SIGN KUSHU2\0"
  "CUNEIFORM SIGN KWU318\0"
  "CUNEIFORM SIGN LA\0"
  "CUNEIFORM SIGN LAGAB\0"
  "CUNEIFORM SIGN LAGAB SQUARED\0"
  "CUNEIFORM SIGN LAGAB TIMES A\0"
  "CUNEIFORM SIGN LAGAB TIMES A PLUS DA PLUS HA\0"
  "CUNEIFORM SIGN LAGAB TIMES A PLUS GAR\0"
  "CUNEIFORM SIGN LAGAB TIMES A PLUS LAL\0"
  "CUNEIFORM SIGN LAGAB TIMES AL\0"
  "CUNEIFORM SIGN LAGAB TIMES AN\0"
  "CUNEIFORM SIGN LAGAB TIMES ASH ZIDA TENU\0"
  "CUNEIFORM SIGN LAGAB TIMES BAD\0"
  "CUNEIFORM SIGN LAGAB TIMES BI\0"
  "CUNEIFORM SIGN LAGAB TIMES DAR\0"
  "CUNEIFORM SIGN LAGAB TIMES EN\0"
  "CUNEIFORM SIGN LAGAB TIMES GA\0"
  "CUNEIFORM SIGN LAGAB TIMES GAR\0"
  "CUNEIFORM SIGN LAGAB TIMES GI\0"
  "CUNEIFORM SIGN LAGAB TIMES GUD\0"
  "CUNEIFORM SIGN LAGAB TIMES GUD PLUS GUD\0"
  "CUNEIFORM SIGN LAGAB TIMES HA\0"
  "CUNEIFORM SIGN LAGAB TIMES HAL\0"
  "CUNEIFORM SIGN LAGAB TIMES HI TIMES NUN\0"
  "CUNEIFORM SIGN LAGAB TIMES IGI GUNU\0"
  "CUNEIFORM SIGN LAGAB TIMES IM\0"
  "CUNEIFORM SIGN LAGAB TIMES IM PLUS HA\0"
  "CUNEIFORM SIGN LAGAB TIMES IM PLUS LU\0"
  "CUNEIFORM SIGN LAGAB TIMES KI\0"
  "CUNEIFORM SIGN LAGAB TIMES KIN\0"
  "CUNEIFORM SIGN LAGAB TIMES KU3\0"
  "CUNEIFORM SIGN LAGAB TIMES KUL\0"
  "CUNEIFORM SIGN LAGAB TIMES KUL PLUS HI PLUS A\0"
  "CUNEIFORM SIGN LAGAB TIMES LAGAB\0"
  "CUNEIFORM SIGN LAGAB TIMES LISH\0"
  "CUNEIFORM SIGN LAGAB TIMES LU\0"
  "CUNEIFORM SIGN LAGAB TIMES LUL\0"
  "CUNEIFORM SIGN LAGAB TIMES ME\0"
  "CUNEIFORM SIGN LAGAB TIMES ME PLUS EN\0"
  "CUNEIFORM SIGN LAGAB TIMES MUSH\0"
  "CUNEIFORM SIGN LAGAB TIMES NE\0"
  "CUNEIFORM SIGN LAGAB TIMES SHE PLUS SUM\0"
  "CUNEIFORM SIGN LAGAB TIMES SHITA PLUS GISH PLUS ERIN2\0"
  "CUNEIFORM SIGN LAGAB TIMES SHITA PLUS GISH TENU\0"
  "CUNEIFORM SIGN LAGAB TIMES SHU2\0"
  "CUNEIFORM SIGN LAGAB TIMES SHU2 PLUS SHU2\0"
  "CUNEIFORM SIGN LAGAB TIMES SUM\0"
  "CUNEIFORM SIGN LAGAB TIMES TAG\0"
  "CUNEIFORM SIGN LAGAB TIMES TAK4\0"
  "CUNEIFORM SIGN LAGAB TIMES TE PLUS A PLUS SU PLUS NA\0"
  "CUNEIFORM SIGN LAGAB TIMES U\0"
  "CUNEIFORM SIGN LAGAB TIMES U PLUS A\0"
  "CUNEIFORM SIGN LAGAB TIMES U PLUS U PLUS U\0"
  "CUNEIFORM SIGN LAGAB TIMES U2 PLUS ASH\0"
  "CUNEIFORM SIGN LAGAB TIMES UD\0"
  "CUNEIFORM SIGN LAGAB TIMES USH\0"
  "CUNEIFORM SIGN LAGAB TIMES ZU OVER ZU\0"
  "CUNEIFORM SIGN LAGAR\0"
  "CUNEIFORM SIGN LAGAR GUNU\0"
  "CUNEIFORM SIGN LAGAR GUNU OVER LAGAR GUNU SHE\0"
  "CUNEIFORM SIGN LAGAR TIMES SHE\0"
  "CUNEIFORM SIGN LAGAR TIMES SHE PLUS SUM\0"
  "CUNEIFORM SIGN LAHSHU\0"
  "CUNEIFORM SIGN LAK-003\0"
  "CUNEIFORM SIGN LAK-021\0"
  "CUNEIFORM SIGN LAK-025\0"
  "CUNEIFORM SIGN LAK-030\0"
  "CUNEIFORM SIGN LAK-050\0"
  "CUNEIFORM SIGN LAK-051\0"
  "CUNEIFORM SIGN LAK-062\0"
  "CUNEIFORM SIGN LAK-079 OVER LAK-079 GUNU\0"
  "CUNEIFORM SIGN LAK-080\0"
  "CUNEIFORM SIGN LAK-081 OVER LAK-081\0"
  "CUNEIFORM SIGN LAK-092\0"
  "CUNEIFORM SIGN LAK-130\0"
  "CUNEIFORM SIGN LAK-142\0"
  "CUNEIFORM SIGN LAK-210\0"
  "CUNEIFORM SIGN LAK-219\0"
  "CUNEIFORM SIGN LAK-220\0"
  "CUNEIFORM SIGN LAK-225\0"
  "CUNEIFORM SIGN LAK-228\0"
  "CUNEIFORM SIGN LAK-238\0"
  "CUNEIFORM SIGN LAK-265\0"
  "CUNEIFORM SIGN LAK-266\0"
  "CUNEIFORM SIGN LAK-343\0"
  "CUNEIFORM SIGN LAK-347\0"
  "CUNEIFORM SIGN LAK-348\0"
  "CUNEIFORM SIGN LAK-383\0"
  "CUNEIFORM SIGN LAK-384\0"
  "CUNEIFORM SIGN LAK-390\0"
  "CUNEIFORM SIGN LAK-441\0"
  "CUNEIFORM SIGN LAK-449\0"
  "CUNEIFORM SIGN LAK-449 TIMES GU\0"
  "CUNEIFORM SIGN LAK-449 TIMES IGI\0"
  "CUNEIFORM SIGN LAK-449 TIMES PAP PLUS LU3\0"
  "CUNEIFORM SIGN LAK-449 TIMES PAP PLUS PAP PLUS LU3\0"
  "CUNEIFORM SIGN LAK-449 TIMES U2 PLUS BA\0"
  "CUNEIFORM SIGN LAK-450\0"
  "CUNEIFORM SIGN LAK-457\0"
  "CUNEIFORM SIGN LAK-470\0"
  "CUNEIFORM SIGN LAK-483\0"
  "CUNEIFORM SIGN LAK-490\0"
  "CUNEIFORM SIGN LAK-492\0"
  "CUNEIFORM SIGN LAK-493\0"
  "CUNEIFORM SIGN LAK-495\0"
  "CUNEIFORM SIGN LAK-550\0"
  "CUNEIFORM SIGN LAK-608\0"
  "CUNEIFORM SIGN LAK-617\0"
  "CUNEIFORM SIGN LAK-617 TIMES ASH\0"
  "CUNEIFORM SIGN LAK-617 TIMES BAD\0"
  "CUNEIFORM SIGN LAK-617 TIMES DUN3 GUNU GUNU\0"
  "CUNEIFORM SIGN LAK-617 TIMES KU3\0"
  "CUNEIFORM SIGN LAK-617 TIMES LA\0"
  "CUNEIFORM SIGN LAK-617 TIMES TAR\0"
  "CUNEIFORM SIGN LAK-617 TIMES TE\0"
  "CUNEIFORM SIGN LAK-617 TIMES U2\0"
  "CUNEIFORM SIGN LAK-617 TIMES UD\0"
  "CUNEIFORM SIGN LAK-617 TIMES URUDA\0"
  "CUNEIFORM SIGN LAK-636\0"
  "CUNEIFORM SIGN LAK-648\0"
  "CUNEIFORM SIGN LAK-648 TIMES DUB\0"
  "CUNEIFORM SIGN LAK-648 TIMES GA\0"
  "CUNEIFORM SIGN LAK-648 TIMES IGI\0"
  "CUNEIFORM SIGN LAK-648 TIMES IGI GUNU\0"
  "CUNEIFORM SIGN LAK-648 TIMES NI\0"
  "CUNEIFORM SIGN LAK-648 TIMES PAP PLUS PAP PLUS LU3\0"
  "CUNEIFORM SIGN LAK-648 TIMES SHESH PLUS KI\0"
  "CUNEIFORM SIGN LAK-648 TIMES UD\0"
  "CUNEIFORM SIGN LAK-648 TIMES URUDA\0"
  "CUNEIFORM SIGN LAK-724\0"
  "CUNEIFORM SIGN LAK-749\0"
  "CUNEIFORM SIGN LAL\0"
  "CUNEIFORM SIGN LAL TIMES LAL\0"
  "CUNEIFORM SIGN LAM\0"
  "CUNEIFORM SIGN LAM TIMES KUR\0"
  "CUNEIFORM SIGN LAM TIMES KUR PLUS RU\0"
  "CUNEIFORM SIGN LI\0"
  "CUNEIFORM SIGN LIL\0"
  "CUNEIFORM SIGN LIMMU2\0"
  "CUNEIFORM SIGN LISH\0"
  "CUNEIFORM SIGN LU\0"
  "CUNEIFORM SIGN LU TIMES BAD\0"
  "CUNEIFORM SIGN LU2\0"
  "CUNEIFORM SIGN LU2 CROSSING LU2\0"
  "CUNEIFORM SIGN LU2 GUNU TIMES ASH\0"
  "CUNEIFORM SIGN LU2 OPPOSING LU2\0"
  "CUNEIFORM SIGN LU2 SHESHIG\0"
  "CUNEIFORM SIGN LU2 SHESHIG TIMES BAD\0"
  "CUNEIFORM SIGN LU2 SQUARED\0"
  "CUNEIFORM SIGN LU2 TENU\0"
  "CUNEIFORM SIGN LU2 TIMES AL\0"
  "CUNEIFORM SIGN LU2 TIMES BAD\0"
  "CUNEIFORM SIGN LU2 TIMES DISH\0"
  "CUNEIFORM SIGN LU2 TIMES ESH2\0"
  "CUNEIFORM SIGN LU2 TIMES ESH2 PLUS LAL\0"
  "CUNEIFORM SIGN LU2 TIMES ESH2 TENU\0"
  "CUNEIFORM SIGN LU2 TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN LU2 TIMES HAL\0"
  "CUNEIFORM SIGN LU2 TIMES HI TIMES BAD\0"
  "CUNEIFORM SIGN LU2 TIMES IM\0"
  "CUNEIFORM SIGN LU2 TIMES KAD2\0"
  "CUNEIFORM SIGN LU2 TIMES KAD3\0"
  "CUNEIFORM SIGN LU2 TIMES KAD3 PLUS ASH\0"
  "CUNEIFORM SIGN LU2 TIMES KI\0"
  "CUNEIFORM SIGN LU2 TIMES LA PLUS ASH\0"
  "CUNEIFORM SIGN LU2 TIMES LAGAB\0"
  "CUNEIFORM SIGN LU2 TIMES ME PLUS EN\0"
  "CUNEIFORM SIGN LU2 TIMES NE\0"
  "CUNEIFORM SIGN LU2 TIMES NU\0"
  "CUNEIFORM SIGN LU2 TIMES PAP\0"
  "CUNEIFORM SIGN LU2 TIMES PAP PLUS PAP PLUS LU3\0"
  "CUNEIFORM SIGN LU2 TIMES SHU\0"
  "CUNEIFORM SIGN LU2 TIMES SI PLUS ASH\0"
  "CUNEIFORM SIGN LU2 TIMES SIK2 PLUS BU\0"
  "CUNEIFORM SIGN LU2 TIMES TAK4\0"
  "CUNEIFORM SIGN LU2 TIMES TUG2\0"
  "CUNEIFORM SIGN LU3\0"
  "CUNEIFORM SIGN LUGAL\0"
  "CUNEIFORM SIGN LUGAL OPPOSING LUGAL\0"
  "CUNEIFORM SIGN LUGAL OVER LUGAL\0"
  "CUNEIFORM SIGN LUGAL SHESHIG\0"
  "CUNEIFORM SIGN LUH\0"
  "CUNEIFORM SIGN LUL\0"
  "CUNEIFORM SIGN LUM\0"
  "CUNEIFORM SIGN LUM OVER LUM\0"
  "CUNEIFORM SIGN LUM OVER LUM GAR OVER GAR\0"
  "CUNEIFORM SIGN MA\0"
  "CUNEIFORM SIGN MA GUNU\0"
  "CUNEIFORM SIGN MA TIMES TAK4\0"
  "CUNEIFORM SIGN MA2\0"
  "CUNEIFORM SIGN MAH\0"
  "CUNEIFORM SIGN MAR\0"
  "CUNEIFORM SIGN MASH\0"
  "CUNEIFORM SIGN MASH2\0"
  "CUNEIFORM SIGN ME\0"
  "CUNEIFORM SIGN MES\0"
  "CUNEIFORM SIGN MESH\0"
  "CUNEIFORM SIGN MI\0"
  "CUNEIFORM SIGN MI PLUS ZA7\0"
  "CUNEIFORM SIGN MIN\0"
  "CUNEIFORM SIGN MU\0"
  "CUNEIFORM SIGN MU OVER MU\0"
  "CUNEIFORM SIGN MUG\0"
  "CUNEIFORM SIGN MUG GUNU\0"
  "CUNEIFORM SIGN MUNSUB\0"
  "CUNEIFORM SIGN MURGU2\0"
  "CUNEIFORM SIGN MUSH\0"
  "CUNEIFORM SIGN MUSH CROSSING MUSH\0"
  "CUNEIFORM SIGN MUSH OVER MUSH\0"
  "CUNEIFORM SIGN MUSH OVER MUSH TIMES A PLUS NA\0"
  "CUNEIFORM SIGN MUSH OVER MUSH TIMES GA\0"
  "CUNEIFORM SIGN MUSH OVER MUSH TIMES KAK\0"
  "CUNEIFORM SIGN MUSH TIMES A\0"
  "CUNEIFORM SIGN MUSH TIMES KUR\0"
  "CUNEIFORM SIGN MUSH TIMES ZA\0"
  "CUNEIFORM SIGN MUSH3\0"
  "CUNEIFORM SIGN MUSH3 GUNU\0"
  "CUNEIFORM SIGN MUSH3 TIMES A\0"
  "CUNEIFORM SIGN MUSH3 TIMES A PLUS DI\0"
  "CUNEIFORM SIGN MUSH3 TIMES DI\0"
  "CUNEIFORM SIGN MUSH3 TIMES ZA\0"
  "CUNEIFORM SIGN NA\0"
  "CUNEIFORM SIGN NA2\0"
  "CUNEIFORM SIGN NA4\0"
  "CUNEIFORM SIGN NAGA\0"
  "CUNEIFORM SIGN NAGA INVERTED\0"
  "CUNEIFORM SIGN NAGA OPPOSING NAGA\0"
  "CUNEIFORM SIGN NAGA TIMES SHU TENU\0"
  "CUNEIFORM SIGN NAGAR\0"
  "CUNEIFORM SIGN NAM\0"
  "CUNEIFORM SIGN NAM NUTILLU\0"
  "CUNEIFORM SIGN NAM2\0"
  "CUNEIFORM SIGN NE\0"
  "CUNEIFORM SIGN NE SHESHIG\0"
  "CUNEIFORM SIGN NE TIMES A\0"
  "CUNEIFORM SIGN NE TIMES UD\0"
  "CUNEIFORM SIGN NI\0"
  "CUNEIFORM SIGN NI TIMES E\0"
  "CUNEIFORM SIGN NI2\0"
  "CUNEIFORM SIGN NIM\0"
  "CUNEIFORM SIGN NIM TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN NIM TIMES GAR PLUS GAN2 TENU\0"
  "CUNEIFORM SIGN NIN\0"
  "CUNEIFORM SIGN NIN9\0"
  "CUNEIFORM SIGN NINDA2\0"
  "CUNEIFORM SIGN NINDA2 TIMES AN\0"
  "CUNEIFORM SIGN NINDA2 TIMES ASH\0"
  "CUNEIFORM SIGN NINDA2 TIMES ASH PLUS ASH\0"
  "CUNEIFORM SIGN NINDA2 TIMES BAL\0"
  "CUNEIFORM SIGN NINDA2 TIMES DIM GUNU\0"
  "CUNEIFORM SIGN NINDA2 TIMES GI\0"
  "CUNEIFORM SIGN NINDA2 TIMES GISH\0"
  "CUNEIFORM SIGN NINDA2 TIMES GUD\0"
  "CUNEIFORM SIGN NINDA2 TIMES GUL\0"
  "CUNEIFORM SIGN NINDA2 TIMES HI\0"
  "CUNEIFORM SIGN NINDA2 TIMES KESH2\0"
  "CUNEIFORM SIGN NINDA2 TIMES LAK-050\0"
  "CUNEIFORM SIGN NINDA2 TIMES MASH\0"
  "CUNEIFORM SIGN NINDA2 TIMES ME PLUS GAN2 TENU\0"
  "CUNEIFORM SIGN NINDA2 TIMES NE\0"
  "CUNEIFORM SIGN NINDA2 TIMES NUN\0"
  "CUNEIFORM SIGN NINDA2 TIMES PAP PLUS PAP\0"
  "CUNEIFORM SIGN NINDA2 TIMES SHE\0"
  "CUNEIFORM SIGN NINDA2 TIMES SHE PLUS A AN\0"
  "CUNEIFORM SIGN NINDA2 TIMES SHE PLUS ASH\0"
  "CUNEIFORM SIGN NINDA2 TIMES SHE PLUS ASH PLUS ASH\0"
  "CUNEIFORM SIGN NINDA2 TIMES U\0"
  "CUNEIFORM SIGN NINDA2 TIMES U PLUS U\0"
  "CUNEIFORM SIGN NINDA2 TIMES U2 PLUS ASH\0"
  "CUNEIFORM SIGN NINDA2 TIMES URUDA\0"
  "CUNEIFORM SIGN NINDA2 TIMES USH\0"
  "CUNEIFORM SIGN NISAG\0"
  "CUNEIFORM SIGN NU\0"
  "CUNEIFORM SIGN NU11\0"
  "CUNEIFORM SIGN NU11 ROTATED NINETY DEGREES\0"
  "CUNEIFORM SIGN NUN\0"
  "CUNEIFORM SIGN NUN CROSSING NUN\0"
  "CUNEIFORM SIGN NUN CROSSING NUN LAGAR OVER LAGAR\0"
  "CUNEIFORM SIGN NUN LAGAR TIMES GAR\0"
  "CUNEIFORM SIGN NUN LAGAR TIMES MASH\0"
  "CUNEIFORM SIGN NUN LAGAR TIMES SAL\0"
  "CUNEIFORM SIGN NUN LAGAR TIMES SAL OVER NUN LAGAR TIMES SAL\0"
  "CUNEIFORM SIGN NUN LAGAR TIMES USH\0"
  "CUNEIFORM SIGN NUN OVER NUN\0"
  "CUNEIFORM SIGN NUN TENU\0"
  "CUNEIFORM SIGN NUNUZ\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES ASHGAB\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES BI\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES DUG\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES GUD\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES IGI GUNU\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES KAD3\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES LA\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES NE\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES SILA3\0"
  "CUNEIFORM SIGN NUNUZ AB2 TIMES U2\0"
  "CUNEIFORM SIGN NUNUZ KISIM5 TIMES BI\0"
  "CUNEIFORM SIGN NUNUZ KISIM5 TIMES BI U\0"
  "CUNEIFORM SIGN PA\0"
  "CUNEIFORM SIGN PAD\0"
  "CUNEIFORM SIGN PAN\0"
  "CUNEIFORM SIGN PAP\0"
  "CUNEIFORM SIGN PESH2\0"
  "CUNEIFORM SIGN PESH2 ASTERISK\0"
  "CUNEIFORM SIGN PI\0"
  "CUNEIFORM SIGN PI CROSSING PI\0"
  "CUNEIFORM SIGN PI TIMES A\0"
  "CUNEIFORM SIGN PI TIMES AB\0"
  "CUNEIFORM SIGN PI TIMES BI\0"
  "CUNEIFORM SIGN PI TIMES BU\0"
  "CUNEIFORM SIGN PI TIMES E\0"
  "CUNEIFORM SIGN PI TIMES I\0"
  "CUNEIFORM SIGN PI TIMES IB\0"
  "CUNEIFORM SIGN PI TIMES U\0"
  "CUNEIFORM SIGN PI TIMES U2\0"
  "CUNEIFORM SIGN PIR2\0"
  "CUNEIFORM SIGN PIRIG\0"
  "CUNEIFORM SIGN PIRIG OPPOSING PIRIG\0"
  "CUNEIFORM SIGN PIRIG TIMES KAL\0"
  "CUNEIFORM SIGN PIRIG TIMES UD\0"
  "CUNEIFORM SIGN PIRIG TIMES ZA\0"
  "CUNEIFORM SIGN RA\0"
  "CUNEIFORM SIGN RAB\0"
  "CUNEIFORM SIGN RI\0"
  "CUNEIFORM SIGN RU\0"
  "CUNEIFORM SIGN SA\0"
  "CUNEIFORM SIGN SAG\0"
  "CUNEIFORM SIGN SAG GUNU\0"
  "CUNEIFORM SIGN SAG GUNU TIMES HA\0"
  "CUNEIFORM SIGN SAG NUTILLU\0"
  "CUNEIFORM SIGN SAG OVER SAG\0"
  "CUNEIFORM SIGN SAG TIMES A\0"
  "CUNEIFORM SIGN SAG TIMES DU\0"
  "CUNEIFORM SIGN SAG TIMES DUB\0"
  "CUNEIFORM SIGN SAG TIMES EN\0"
  "CUNEIFORM SIGN SAG TIMES HA\0"
  "CUNEIFORM SIGN SAG TIMES IGI GUNU\0"
  "CUNEIFORM SIGN SAG TIMES KAK\0"
  "CUNEIFORM SIGN SAG TIMES KUR\0"
  "CUNEIFORM SIGN SAG TIMES LUM\0"
  "CUNEIFORM SIGN SAG TIMES MI\0"
  "CUNEIFORM SIGN SAG TIMES NUN\0"
  "CUNEIFORM SIGN SAG TIMES SAL\0"
  "CUNEIFORM SIGN SAG TIMES SHE AT LEFT\0"
  "CUNEIFORM SIGN SAG TIMES SHID\0"
  "CUNEIFORM SIGN SAG TIMES TAB\0"
  "CUNEIFORM SIGN SAG TIMES TAK4\0"
  "CUNEIFORM SIGN SAG TIMES U2\0"
  "CUNEIFORM SIGN SAG TIMES UB\0"
  "CUNEIFORM SIGN SAG TIMES UM\0"
  "CUNEIFORM SIGN SAG TIMES UR\0"
  "CUNEIFORM SIGN SAG TIMES USH\0"
  "CUNEIFORM SIGN SAL\0"
  "CUNEIFORM SIGN SAL LAGAB TIMES ASH2\0"
  "CUNEIFORM SIGN SANGA2\0"
  "CUNEIFORM SIGN SAR\0"
  "CUNEIFORM SIGN SHA\0"
  "CUNEIFORM SIGN SHA3\0"
  "CUNEIFORM SIGN SHA3 TIMES A\0"
  "CUNEIFORM SIGN SHA3 TIMES BAD\0"
  "CUNEIFORM SIGN SHA3 TIMES GISH\0"
  "CUNEIFORM SIGN SHA3 TIMES NE\0"
  "CUNEIFORM SIGN SHA3 TIMES SHU2\0"
  "CUNEIFORM SIGN SHA3 TIMES TUR\0"
  "CUNEIFORM SIGN SHA3 TIMES U\0"
  "CUNEIFORM SIGN SHA3 TIMES U PLUS A\0"
  "CUNEIFORM SIGN SHA6\0"
  "CUNEIFORM SIGN SHA6 TENU\0"
  "CUNEIFORM SIGN SHAB6\0"
  "CUNEIFORM SIGN SHAR2\0"
  "CUNEIFORM SIGN SHE\0"
  "CUNEIFORM SIGN SHE HU\0"
  "CUNEIFORM SIGN SHE OVER SHE\0"
  "CUNEIFORM SIGN SHE OVER SHE GAD OVER GAD GAR OVER GAR\0"
  "CUNEIFORM SIGN SHE OVER SHE TAB OVER TAB GAR OVER GAR\0"
  "CUNEIFORM SIGN SHE PLUS HUB2\0"
  "CUNEIFORM SIGN SHE PLUS NAM2\0"
  "CUNEIFORM SIGN SHE PLUS SAR\0"
  "CUNEIFORM SIGN SHEG9\0"
  "CUNEIFORM SIGN SHEN\0"
  "CUNEIFORM SIGN SHESH\0"
  "CUNEIFORM SIGN SHESH2\0"
  "CUNEIFORM SIGN SHESHLAM\0"
  "CUNEIFORM SIGN SHID\0"
  "CUNEIFORM SIGN SHID TIMES A\0"
  "CUNEIFORM SIGN SHID TIMES IM\0"
  "CUNEIFORM SIGN SHIM\0"
  "CUNEIFORM SIGN SHIM TIMES A\0"
  "CUNEIFORM SIGN SHIM TIMES BAL\0"
  "CUNEIFORM SIGN SHIM TIMES BULUG\0"
  "CUNEIFORM SIGN SHIM TIMES DIN\0"
  "CUNEIFORM SIGN SHIM TIMES GAR\0"
  "CUNEIFORM SIGN SHIM TIMES IGI\0"
  "CUNEIFORM SIGN SHIM TIMES IGI GUNU\0"
  "CUNEIFORM SIGN SHIM TIMES KUSHU2\0"
  "CUNEIFORM SIGN SHIM TIMES LUL\0"
  "CUNEIFORM SIGN SHIM TIMES MUG\0"
  "CUNEIFORM SIGN SHIM TIMES SAL\0"
  "CUNEIFORM SIGN SHINIG\0"
  "CUNEIFORM SIGN SHIR\0"
  "CUNEIFORM SIGN SHIR OVER SHIR BUR OVER BUR\0"
  "CUNEIFORM SIGN SHIR TENU\0"
  "CUNEIFORM SIGN SHITA\0"
  "CUNEIFORM SIGN SHU\0"
  "CUNEIFORM SIGN SHU OVER INVERTED SHU\0"
  "CUNEIFORM SIGN SHU2\0"
  "CUNEIFORM SIGN SHU2 PLUS DUG TIMES NI\0"
  "CUNEIFORM SIGN SHU2 PLUS E2 TIMES AN\0"
  "CUNEIFORM SIGN SHUBUR\0"
  "CUNEIFORM SIGN SI\0"
  "CUNEIFORM SIGN SI GUNU\0"
  "CUNEIFORM SIGN SI TIMES TAK4\0"
  "CUNEIFORM SIGN SIG\0"
  "CUNEIFORM SIGN SIG4\0"
  "CUNEIFORM SIGN SIG4 OVER SIG4 SHU2\0"
  "CUNEIFORM SIGN SIK2\0"
  "CUNEIFORM SIGN SILA3\0"
  "CUNEIFORM SIGN SU\0"
  "CUNEIFORM SIGN SU OVER SU\0"
  "CUNEIFORM SIGN SUD\0"
  "CUNEIFORM SIGN SUD2\0"
  "CUNEIFORM SIGN SUHUR\0"
  "CUNEIFORM SIGN SUM\0"
  "CUNEIFORM SIGN SUMASH\0"
  "CUNEIFORM SIGN SUR\0"
  "CUNEIFORM SIGN SUR9\0"
  "CUNEIFORM SIGN TA\0"
  "CUNEIFORM SIGN TA ASTERISK\0"
  "CUNEIFORM SIGN TA GUNU\0"
  "CUNEIFORM SIGN TA TIMES HI\0"
  "CUNEIFORM SIGN TA TIMES MI\0"
  "CUNEIFORM SIGN TAB\0"
  "CUNEIFORM SIGN TAB OVER TAB NI OVER NI DISH OVER DISH\0"
  "CUNEIFORM SIGN TAB SQUARED\0"
  "CUNEIFORM SIGN TAG\0"
  "CUNEIFORM SIGN TAG TIMES BI\0"
  "CUNEIFORM SIGN TAG TIMES GUD\0"
  "CUNEIFORM SIGN TAG TIMES SHE\0"
  "CUNEIFORM SIGN TAG TIMES SHU\0"
  "CUNEIFORM SIGN TAG TIMES TUG2\0"
  "CUNEIFORM SIGN TAG TIMES UD\0"
  "CUNEIFORM SIGN TAK4\0"
  "CUNEIFORM SIGN TAK4 PLUS SAG\0"
  "CUNEIFORM SIGN TAR\0"
  "CUNEIFORM SIGN TE\0"
  "CUNEIFORM SIGN TE GUNU\0"
  "CUNEIFORM SIGN TI\0"
  "CUNEIFORM SIGN TI TENU\0"
  "CUNEIFORM SIGN TI2\0"
  "CUNEIFORM SIGN TIL\0"
  "CUNEIFORM SIGN TIR\0"
  "CUNEIFORM SIGN TIR OVER TIR\0"
  "CUNEIFORM SIGN TIR OVER TIR GAD OVER GAD GAR OVER GAR\0"
  "CUNEIFORM SIGN TIR TIMES TAK4\0"
  "CUNEIFORM SIGN TU\0"
  "CUNEIFORM SIGN TUG2\0"
  "CUNEIFORM SIGN TUK\0"
  "CUNEIFORM SIGN TUM\0"
  "CUNEIFORM SIGN TUM TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN TUM TIMES THREE DISH\0"
  "CUNEIFORM SIGN TUR\0"
  "CUNEIFORM SIGN TUR OVER TUR ZA OVER ZA\0"
  "CUNEIFORM SIGN U\0"
  "CUNEIFORM SIGN U GUD\0"
  "CUNEIFORM SIGN U OVER U PA OVER PA GAR OVER GAR\0"
  "CUNEIFORM SIGN U OVER U SUR OVER SUR\0"
  "CUNEIFORM SIGN U OVER U U REVERSED OVER U REVERSED\0"
  "CUNEIFORM SIGN U U\0"
  "CUNEIFORM SIGN U U U\0"
  "CUNEIFORM SIGN U2\0"
  "CUNEIFORM SIGN UB\0"
  "CUNEIFORM SIGN UD\0"
  "CUNEIFORM SIGN UD GUNU\0"
  "CUNEIFORM SIGN UD KUSHU2\0"
  "CUNEIFORM SIGN UD SHESHIG\0"
  "CUNEIFORM SIGN UD SHESHIG TIMES BAD\0"
  "CUNEIFORM SIGN UD TIMES BAD\0"
  "CUNEIFORM SIGN UD TIMES MI\0"
  "CUNEIFORM SIGN UD TIMES U PLUS U PLUS U\0"
  "CUNEIFORM SIGN UD TIMES U PLUS U PLUS U GUNU\0"
  "CUNEIFORM SIGN UDUG\0"
  "CUNEIFORM SIGN UM\0"
  "CUNEIFORM SIGN UM TIMES LAGAB\0"
  "CUNEIFORM SIGN UM TIMES ME\0"
  "CUNEIFORM SIGN UM TIMES ME PLUS DA\0"
  "CUNEIFORM SIGN UM TIMES SHA3\0"
  "CUNEIFORM SIGN UM TIMES U\0"
  "CUNEIFORM SIGN UMBIN\0"
  "CUNEIFORM SIGN UMUM\0"
  "CUNEIFORM SIGN UMUM TIMES KASKAL\0"
  "CUNEIFORM SIGN UMUM TIMES PA\0"
  "CUNEIFORM SIGN UN\0"
  "CUNEIFORM SIGN UN GUNU\0"
  "CUNEIFORM SIGN UR\0"
  "CUNEIFORM SIGN UR CROSSING UR\0"
  "CUNEIFORM SIGN UR SHESHIG\0"
  "CUNEIFORM SIGN UR2\0"
  "CUNEIFORM SIGN UR2 INVERTED\0"
  "CUNEIFORM SIGN UR2 TIMES A PLUS HA\0"
  "CUNEIFORM SIGN UR2 TIMES A PLUS NA\0"
  "CUNEIFORM SIGN UR2 TIMES AL\0"
  "CUNEIFORM SIGN UR2 TIMES HA\0"
  "CUNEIFORM SIGN UR2 TIMES NUN\0"
  "CUNEIFORM SIGN UR2 TIMES U2\0"
  "CUNEIFORM SIGN UR2 TIMES U2 PLUS ASH\0"
  "CUNEIFORM SIGN UR2 TIMES U2 PLUS BI\0"
  "CUNEIFORM SIGN UR2 TIMES UD\0"
  "CUNEIFORM SIGN UR4\0"
  "CUNEIFORM SIGN URI\0"
  "CUNEIFORM SIGN URI3\0"
  "CUNEIFORM SIGN URU\0"
  "CUNEIFORM SIGN URU TIMES A\0"
  "CUNEIFORM SIGN URU TIMES ASHGAB\0"
  "CUNEIFORM SIGN URU TIMES BAR\0"
  "CUNEIFORM SIGN URU TIMES DARA3\0"
  "CUNEIFORM SIGN URU TIMES DUN\0"
  "CUNEIFORM SIGN URU TIMES GA\0"
  "CUNEIFORM SIGN URU TIMES GAL\0"
  "CUNEIFORM SIGN URU TIMES GAN2 TENU\0"
  "CUNEIFORM SIGN URU TIMES GAR\0"
  "CUNEIFORM SIGN URU TIMES GU\0"
  "CUNEIFORM SIGN URU TIMES HA\0"
  "CUNEIFORM SIGN URU TIMES IGI\0"
  "CUNEIFORM SIGN URU TIMES IM\0"
  "CUNEIFORM SIGN URU TIMES ISH\0"
  "CUNEIFORM SIGN URU TIMES KI\0"
  "CUNEIFORM SIGN URU TIMES LAK-668\0"
  "CUNEIFORM SIGN URU TIMES LU3\0"
  "CUNEIFORM SIGN URU TIMES LUM\0"
  "CUNEIFORM SIGN URU TIMES MIN\0"
  "CUNEIFORM SIGN URU TIMES PA\0"
  "CUNEIFORM SIGN URU TIMES SHE\0"
  "CUNEIFORM SIGN URU TIMES SIG4\0"
  "CUNEIFORM SIGN URU TIMES TU\0"
  "CUNEIFORM SIGN URU TIMES U PLUS GUD\0"
  "CUNEIFORM SIGN URU TIMES UD\0"
  "CUNEIFORM SIGN URU TIMES URUDA\0"
  "CUNEIFORM SIGN URUDA\0"
  "CUNEIFORM SIGN URUDA TIMES U\0"
  "CUNEIFORM SIGN USH\0"
  "CUNEIFORM SIGN USH TIMES A\0"
  "CUNEIFORM SIGN USH TIMES KU\0"
  "CUNEIFORM SIGN USH TIMES KUR\0"
  "CUNEIFORM SIGN USH TIMES TAK4\0"
  "CUNEIFORM SIGN USH2\0"
  "CUNEIFORM SIGN USHUMX\0"
  "CUNEIFORM SIGN USHX\0"
  "CUNEIFORM SIGN UTUKI\0"
  "CUNEIFORM SIGN UZ3\0"
  "CUNEIFORM SIGN UZ3 TIMES KASKAL\0"
  "CUNEIFORM SIGN UZU\0"
  "CUNEIFORM SIGN ZA\0"
  "CUNEIFORM SIGN ZA SQUARED TIMES KUR\0"
  "CUNEIFORM SIGN ZA TENU\0"
  "CUNEIFORM SIGN ZA7\0"
  "CUNEIFORM SIGN ZAG\0"
  "CUNEIFORM SIGN ZAMX\0"
  "CUNEIFORM SIGN ZE2\0"
  "CUNEIFORM SIGN ZI\0"
  "CUNEIFORM SIGN ZI OVER ZI\0"
  "CUNEIFORM SIGN ZI3\0"
  "CUNEIFORM SIGN ZIB\0"
  "CUNEIFORM SIGN ZIB KABA TENU\0"
  "CUNEIFORM SIGN ZIG\0"
  "CUNEIFORM SIGN ZIZ2\0"
  "CUNEIFORM SIGN ZU\0"
  "CUNEIFORM SIGN ZU OVER ZU PLUS SAR\0"
  "CUNEIFORM SIGN ZU5\0"
  "CUNEIFORM SIGN ZU5 TIMES A\0"
  "CUNEIFORM SIGN ZU5 TIMES THREE DISH TENU\0"
  "CUNEIFORM SIGN ZUBUR\0"
  "CUNEIFORM SIGN ZUM\0"
  "CUP ON BLACK SQUARE\0"
  "CUP WITH STRAW\0"
  "CUPCAKE\0"
  "CUPIDO\0"
  "CURLING STONE\0"
  "CURLY BRACKET EXTENSION\0"
  "CURLY LOGICAL AND\0"
  "CURLY LOGICAL OR\0"
  "CURLY LOOP\0"
  "CURRENCY EXCHANGE\0"
  "CURRENCY SIGN\0"
  "CURRY AND RICE\0"
  "CURVED STEM PARAGRAPH SIGN ORNAMENT\0"
  "CUSTARD\0"
  "CUSTOMS\0"
  "CUT OF MEAT\0"
  "CYCLONE\0"
  "CYLINDRICITY\0"
  "CYPRIOT SYLLABLE A\0"
  "CYPRIOT SYLLABLE E\0"
  "CYPRIOT SYLLABLE I\0"
  "CYPRIOT SYLLABLE JA\0"
  "CYPRIOT SYLLABLE JO\0"
  "CYPRIOT SYLLABLE KA\0"
  "CYPRIOT SYLLABLE KE\0"
  "CYPRIOT SYLLABLE KI\0"
  "CYPRIOT SYLLABLE KO\0"
  "CYPRIOT SYLLABLE KU\0"
  "CYPRIOT SYLLABLE LA\0"
  "CYPRIOT SYLLABLE LE\0"
  "CYPRIOT SYLLABLE LI\0"
  "CYPRIOT SYLLABLE LO\0"
  "CYPRIOT SYLLABLE LU\0"
  "CYPRIOT SYLLABLE MA\0"
  "CYPRIOT SYLLABLE ME\0"
  "CYPRIOT SYLLABLE MI\0"
  "CYPRIOT SYLLABLE MO\0"
  "CYPRIOT SYLLABLE MU\0"
  "CYPRIOT SYLLABLE NA\0"
  "CYPRIOT SYLLABLE NE\0"
  "CYPRIOT SYLLABLE NI\0"
  "CYPRIOT SYLLABLE NO\0"
  "CYPRIOT SYLLABLE NU\0"
  "CYPRIOT SYLLABLE O\0"
  "CYPRIOT SYLLABLE PA\0"
  "CYPRIOT SYLLABLE PE\0"
  "CYPRIOT SYLLABLE PI\0"
  "CYPRIOT SYLLABLE PO\0"
  "CYPRIOT SYLLABLE PU\0"
  "CYPRIOT SYLLABLE RA\0"
  "CYPRIOT SYLLABLE RE\0"
  "CYPRIOT SYLLABLE RI\0"
  "CYPRIOT SYLLABLE RO\0"
  "CYPRIOT SYLLABLE RU\0"
  "CYPRIOT SYLLABLE SA\0"
  "CYPRIOT SYLLABLE SE\0"
  "CYPRIOT SYLLABLE SI\0"
  "CYPRIOT SYLLABLE SO\0"
  "CYPRIOT SYLLABLE SU\0"
  "CYPRIOT SYLLABLE TA\0"
  "CYPRIOT SYLLABLE TE\0"
  "CYPRIOT SYLLABLE TI\0"
  "CYPRIOT SYLLABLE TO\0"
  "CYPRIOT SYLLABLE TU\0"
  "CYPRIOT SYLLABLE U\0"
  "CYPRIOT SYLLABLE WA\0"
  "CYPRIOT SYLLABLE WE\0"
  "CYPRIOT SYLLABLE WI\0"
  "CYPRIOT SYLLABLE WO\0"
  "CYPRIOT SYLLABLE XA\0"
  "CYPRIOT SYLLABLE XE\0"
  "CYPRIOT SYLLABLE ZA\0"
  "CYPRIOT SYLLABLE ZO\0"
  "CYPRO-MINOAN SIGN CM001\0"
  "CYPRO-MINOAN SIGN CM002\0"
  "CYPRO-MINOAN SIGN CM004\0"
  "CYPRO-MINOAN SIGN CM005\0"
  "CYPRO-MINOAN SIGN CM006\0"
  "CYPRO-MINOAN SIGN CM007\0"
  "CYPRO-MINOAN SIGN CM008\0"
  "CYPRO-MINOAN SIGN CM009\0"
  "CYPRO-MINOAN SIGN CM010\0"
  "CYPRO-MINOAN SIGN CM011\0"
  "CYPRO-MINOAN SIGN CM012\0"
  "CYPRO-MINOAN SIGN CM012B\0"
  "CYPRO-MINOAN SIGN CM013\0"
  "CYPRO-MINOAN SIGN CM015\0"
  "CYPRO-MINOAN SIGN CM017\0"
  "CYPRO-MINOAN SIGN CM019\0"
  "CYPRO-MINOAN SIGN CM021\0"
  "CYPRO-MINOAN SIGN CM023\0"
  "CYPRO-MINOAN SIGN CM024\0"
  "CYPRO-MINOAN SIGN CM025\0"
  "CYPRO-MINOAN SIGN CM026\0"
  "CYPRO-MINOAN SIGN CM027\0"
  "CYPRO-MINOAN SIGN CM028\0"
  "CYPRO-MINOAN SIGN CM029\0"
  "CYPRO-MINOAN SIGN CM030\0"
  "CYPRO-MINOAN SIGN CM033\0"
  "CYPRO-MINOAN SIGN CM034\0"
  "CYPRO-MINOAN SIGN CM035\0"
  "CYPRO-MINOAN SIGN CM036\0"
  "CYPRO-MINOAN SIGN CM037\0"
  "CYPRO-MINOAN SIGN CM038\0"
  "CYPRO-MINOAN SIGN CM039\0"
  "CYPRO-MINOAN SIGN CM040\0"
  "CYPRO-MINOAN SIGN CM041\0"
  "CYPRO-MINOAN SIGN CM044\0"
  "CYPRO-MINOAN SIGN CM046\0"
  "CYPRO-MINOAN SIGN CM047\0"
  "CYPRO-MINOAN SIGN CM049\0"
  "CYPRO-MINOAN SIGN CM050\0"
  "CYPRO-MINOAN SIGN CM051\0"
  "CYPRO-MINOAN SIGN CM052\0"
  "CYPRO-MINOAN SIGN CM053\0"
  "CYPRO-MINOAN SIGN CM054\0"
  "CYPRO-MINOAN SIGN CM055\0"
  "CYPRO-MINOAN SIGN CM056\0"
  "CYPRO-MINOAN SIGN CM058\0"
  "CYPRO-MINOAN SIGN CM059\0"
  "CYPRO-MINOAN SIGN CM060\0"
  "CYPRO-MINOAN SIGN CM061\0"
  "CYPRO-MINOAN SIGN CM062\0"
  "CYPRO-MINOAN SIGN CM063\0"
  "CYPRO-MINOAN SIGN CM064\0"
  "CYPRO-MINOAN SIGN CM066\0"
  "CYPRO-MINOAN SIGN CM067\0"
  "CYPRO-MINOAN SIGN CM068\0"
  "CYPRO-MINOAN SIGN CM069\0"
  "CYPRO-MINOAN SIGN CM070\0"
  "CYPRO-MINOAN SIGN CM071\0"
  "CYPRO-MINOAN SIGN CM072\0"
  "CYPRO-MINOAN SIGN CM073\0"
  "CYPRO-MINOAN SIGN CM074\0"
  "CYPRO-MINOAN SIGN CM075\0"
  "CYPRO-MINOAN SIGN CM075B\0"
  "CYPRO-MINOAN SIGN CM076\0"
  "CYPRO-MINOAN SIGN CM078\0"
  "CYPRO-MINOAN SIGN CM079\0"
  "CYPRO-MINOAN SIGN CM080\0"
  "CYPRO-MINOAN SIGN CM081\0"
  "CYPRO-MINOAN SIGN CM082\0"
  "CYPRO-MINOAN SIGN CM083\0"
  "CYPRO-MINOAN SIGN CM084\0"
  "CYPRO-MINOAN SIGN CM085\0"
  "CYPRO-MINOAN SIGN CM086\0"
  "CYPRO-MINOAN SIGN CM087\0"
  "CYPRO-MINOAN SIGN CM088\0"
  "CYPRO-MINOAN SIGN CM089\0"
  "CYPRO-MINOAN SIGN CM090\0"
  "CYPRO-MINOAN SIGN CM091\0"
  "CYPRO-MINOAN SIGN CM092\0"
  "CYPRO-MINOAN SIGN CM094\0"
  "CYPRO-MINOAN SIGN CM095\0"
  "CYPRO-MINOAN SIGN CM096\0"
  "CYPRO-MINOAN SIGN CM097\0"
  "CYPRO-MINOAN SIGN CM098\0"
  "CYPRO-MINOAN SIGN CM099\0"
  "CYPRO-MINOAN SIGN CM100\0"
  "CYPRO-MINOAN SIGN CM101\0"
  "CYPRO-MINOAN SIGN CM102\0"
  "CYPRO-MINOAN SIGN CM103\0"
  "CYPRO-MINOAN SIGN CM104\0"
  "CYPRO-MINOAN SIGN CM105\0"
  "CYPRO-MINOAN SIGN CM107\0"
  "CYPRO-MINOAN SIGN CM108\0"
  "CYPRO-MINOAN SIGN CM109\0"
  "CYPRO-MINOAN SIGN CM110\0"
  "CYPRO-MINOAN SIGN CM112\0"
  "CYPRO-MINOAN SIGN CM114\0"
  "CYPRO-MINOAN SIGN CM301\0"
  "CYPRO-MINOAN SIGN CM302\0"
  "CYRILLIC CAPITAL LETTER A\0"
  "CYRILLIC CAPITAL LETTER A WITH BREVE\0"
  "CYRILLIC CAPITAL LETTER A WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER ABKHASIAN CHE\0"
  "CYRILLIC CAPITAL LETTER ABKHASIAN CHE WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER ABKHASIAN DZE\0"
  "CYRILLIC CAPITAL LETTER ABKHASIAN HA\0"
  "CYRILLIC CAPITAL LETTER ALEUT KA\0"
  "CYRILLIC CAPITAL LETTER BARRED O\0"
  "CYRILLIC CAPITAL LETTER BARRED O WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER BASHKIR KA\0"
  "CYRILLIC CAPITAL LETTER BE\0"
  "CYRILLIC CAPITAL LETTER BIG YUS\0"
  "CYRILLIC CAPITAL LETTER BINOCULAR O\0"
  "CYRILLIC CAPITAL LETTER BLENDED YUS\0"
  "CYRILLIC CAPITAL LETTER BROAD OMEGA\0"
  "CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I\0"
  "CYRILLIC CAPITAL LETTER CCHE\0"
  "CYRILLIC CAPITAL LETTER CHE\0"
  "CYRILLIC CAPITAL LETTER CHE WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER CHE WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER CHE WITH VERTICAL STROKE\0"
  "CYRILLIC CAPITAL LETTER CLOSED LITTLE YUS\0"
  "CYRILLIC CAPITAL LETTER CROSSED O\0"
  "CYRILLIC CAPITAL LETTER DCHE\0"
  "CYRILLIC CAPITAL LETTER DE\0"
  "CYRILLIC CAPITAL LETTER DJE\0"
  "CYRILLIC CAPITAL LETTER DJERV\0"
  "CYRILLIC CAPITAL LETTER DOUBLE MONOCULAR O\0"
  "CYRILLIC CAPITAL LETTER DOUBLE O\0"
  "CYRILLIC CAPITAL LETTER DWE\0"
  "CYRILLIC CAPITAL LETTER DZE\0"
  "CYRILLIC CAPITAL LETTER DZELO\0"
  "CYRILLIC CAPITAL LETTER DZHE\0"
  "CYRILLIC CAPITAL LETTER DZWE\0"
  "CYRILLIC CAPITAL LETTER DZZE\0"
  "CYRILLIC CAPITAL LETTER DZZHE\0"
  "CYRILLIC CAPITAL LETTER E\0"
  "CYRILLIC CAPITAL LETTER E WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER EF\0"
  "CYRILLIC CAPITAL LETTER EL\0"
  "CYRILLIC CAPITAL LETTER EL WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER EL WITH HOOK\0"
  "CYRILLIC CAPITAL LETTER EL WITH MIDDLE HOOK\0"
  "CYRILLIC CAPITAL LETTER EL WITH TAIL\0"
  "CYRILLIC CAPITAL LETTER EM\0"
  "CYRILLIC CAPITAL LETTER EM WITH TAIL\0"
  "CYRILLIC CAPITAL LETTER EN\0"
  "CYRILLIC CAPITAL LETTER EN WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER EN WITH HOOK\0"
  "CYRILLIC CAPITAL LETTER EN WITH LEFT HOOK\0"
  "CYRILLIC CAPITAL LETTER EN WITH MIDDLE HOOK\0"
  "CYRILLIC CAPITAL LETTER EN WITH TAIL\0"
  "CYRILLIC CAPITAL LETTER ER\0"
  "CYRILLIC CAPITAL LETTER ER WITH TICK\0"
  "CYRILLIC CAPITAL LETTER ES\0"
  "CYRILLIC CAPITAL LETTER ES WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER FITA\0"
  "CYRILLIC CAPITAL LETTER GHE\0"
  "CYRILLIC CAPITAL LETTER GHE WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER GHE WITH MIDDLE HOOK\0"
  "CYRILLIC CAPITAL LETTER GHE WITH STROKE\0"
  "CYRILLIC CAPITAL LETTER GHE WITH STROKE AND HOOK\0"
  "CYRILLIC CAPITAL LETTER GHE WITH UPTURN\0"
  "CYRILLIC CAPITAL LETTER GJE\0"
  "CYRILLIC CAPITAL LETTER HA\0"
  "CYRILLIC CAPITAL LETTER HA WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER HA WITH HOOK\0"
  "CYRILLIC CAPITAL LETTER HA WITH STROKE\0"
  "CYRILLIC CAPITAL LETTER HARD SIGN\0"
  "CYRILLIC CAPITAL LETTER HWE\0"
  "CYRILLIC CAPITAL LETTER I\0"
  "CYRILLIC CAPITAL LETTER I WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER I WITH GRAVE\0"
  "CYRILLIC CAPITAL LETTER I WITH MACRON\0"
  "CYRILLIC CAPITAL LETTER IE\0"
  "CYRILLIC CAPITAL LETTER IE WITH BREVE\0"
  "CYRILLIC CAPITAL LETTER IE WITH GRAVE\0"
  "CYRILLIC CAPITAL LETTER IO\0"
  "CYRILLIC CAPITAL LETTER IOTA\0"
  "CYRILLIC CAPITAL LETTER IOTIFIED A\0"
  "CYRILLIC CAPITAL LETTER IOTIFIED BIG YUS\0"
  "CYRILLIC CAPITAL LETTER IOTIFIED CLOSED LITTLE YUS\0"
  "CYRILLIC CAPITAL LETTER IOTIFIED E\0"
  "CYRILLIC CAPITAL LETTER IOTIFIED LITTLE YUS\0"
  "CYRILLIC CAPITAL LETTER IOTIFIED YAT\0"
  "CYRILLIC CAPITAL LETTER IZHITSA\0"
  "CYRILLIC CAPITAL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT\0"
  "CYRILLIC CAPITAL LETTER JE\0"
  "CYRILLIC CAPITAL LETTER KA\0"
  "CYRILLIC CAPITAL LETTER KA WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER KA WITH HOOK\0"
  "CYRILLIC CAPITAL LETTER KA WITH STROKE\0"
  "CYRILLIC CAPITAL LETTER KA WITH VERTICAL STROKE\0"
  "CYRILLIC CAPITAL LETTER KHAKASSIAN CHE\0"
  "CYRILLIC CAPITAL LETTER KJE\0"
  "CYRILLIC CAPITAL LETTER KOMI DE\0"
  "CYRILLIC CAPITAL LETTER KOMI DJE\0"
  "CYRILLIC CAPITAL LETTER KOMI DZJE\0"
  "CYRILLIC CAPITAL LETTER KOMI LJE\0"
  "CYRILLIC CAPITAL LETTER KOMI NJE\0"
  "CYRILLIC CAPITAL LETTER KOMI SJE\0"
  "CYRILLIC CAPITAL LETTER KOMI TJE\0"
  "CYRILLIC CAPITAL LETTER KOMI ZJE\0"
  "CYRILLIC CAPITAL LETTER KOPPA\0"
  "CYRILLIC CAPITAL LETTER KSI\0"
  "CYRILLIC CAPITAL LETTER LHA\0"
  "CYRILLIC CAPITAL LETTER LITTLE YUS\0"
  "CYRILLIC CAPITAL LETTER LJE\0"
  "CYRILLIC CAPITAL LETTER MONOCULAR O\0"
  "CYRILLIC CAPITAL LETTER MONOGRAPH UK\0"
  "CYRILLIC CAPITAL LETTER NEUTRAL YER\0"
  "CYRILLIC CAPITAL LETTER NJE\0"
  "CYRILLIC CAPITAL LETTER O\0"
  "CYRILLIC CAPITAL LETTER O WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER OMEGA\0"
  "CYRILLIC CAPITAL LETTER OMEGA WITH TITLO\0"
  "CYRILLIC CAPITAL LETTER OT\0"
  "CYRILLIC CAPITAL LETTER PE\0"
  "CYRILLIC CAPITAL LETTER PE WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER PE WITH MIDDLE HOOK\0"
  "CYRILLIC CAPITAL LETTER PSI\0"
  "CYRILLIC CAPITAL LETTER QA\0"
  "CYRILLIC CAPITAL LETTER REVERSED DZE\0"
  "CYRILLIC CAPITAL LETTER REVERSED TSE\0"
  "CYRILLIC CAPITAL LETTER REVERSED YU\0"
  "CYRILLIC CAPITAL LETTER REVERSED ZE\0"
  "CYRILLIC CAPITAL LETTER RHA\0"
  "CYRILLIC CAPITAL LETTER ROUND OMEGA\0"
  "CYRILLIC CAPITAL LETTER SCHWA\0"
  "CYRILLIC CAPITAL LETTER SCHWA WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER SEMISOFT SIGN\0"
  "CYRILLIC CAPITAL LETTER SHA\0"
  "CYRILLIC CAPITAL LETTER SHCHA\0"
  "CYRILLIC CAPITAL LETTER SHHA\0"
  "CYRILLIC CAPITAL LETTER SHHA WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER SHORT I\0"
  "CYRILLIC CAPITAL LETTER SHORT I WITH TAIL\0"
  "CYRILLIC CAPITAL LETTER SHORT U\0"
  "CYRILLIC CAPITAL LETTER SHWE\0"
  "CYRILLIC CAPITAL LETTER SOFT DE\0"
  "CYRILLIC CAPITAL LETTER SOFT EL\0"
  "CYRILLIC CAPITAL LETTER SOFT EM\0"
  "CYRILLIC CAPITAL LETTER SOFT SIGN\0"
  "CYRILLIC CAPITAL LETTER STRAIGHT U\0"
  "CYRILLIC CAPITAL LETTER STRAIGHT U WITH STROKE\0"
  "CYRILLIC CAPITAL LETTER TCHE\0"
  "CYRILLIC CAPITAL LETTER TE\0"
  "CYRILLIC CAPITAL LETTER TE WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER TE WITH MIDDLE HOOK\0"
  "CYRILLIC CAPITAL LETTER TJE\0"
  "CYRILLIC CAPITAL LETTER TSE\0"
  "CYRILLIC CAPITAL LETTER TSHE\0"
  "CYRILLIC CAPITAL LETTER TSSE\0"
  "CYRILLIC CAPITAL LETTER TSWE\0"
  "CYRILLIC CAPITAL LETTER TWE\0"
  "CYRILLIC CAPITAL LETTER U\0"
  "CYRILLIC CAPITAL LETTER U WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER U WITH DOUBLE ACUTE\0"
  "CYRILLIC CAPITAL LETTER U WITH MACRON\0"
  "CYRILLIC CAPITAL LETTER UK\0"
  "CYRILLIC CAPITAL LETTER UKRAINIAN IE\0"
  "CYRILLIC CAPITAL LETTER VE\0"
  "CYRILLIC CAPITAL LETTER WE\0"
  "CYRILLIC CAPITAL LETTER YA\0"
  "CYRILLIC CAPITAL LETTER YAE\0"
  "CYRILLIC CAPITAL LETTER YAT\0"
  "CYRILLIC CAPITAL LETTER YERU\0"
  "CYRILLIC CAPITAL LETTER YERU WITH BACK YER\0"
  "CYRILLIC CAPITAL LETTER YERU WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER YI\0"
  "CYRILLIC CAPITAL LETTER YN\0"
  "CYRILLIC CAPITAL LETTER YU\0"
  "CYRILLIC CAPITAL LETTER ZE\0"
  "CYRILLIC CAPITAL LETTER ZE WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER ZE WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER ZEMLYA\0"
  "CYRILLIC CAPITAL LETTER ZHE\0"
  "CYRILLIC CAPITAL LETTER ZHE WITH BREVE\0"
  "CYRILLIC CAPITAL LETTER ZHE WITH DESCENDER\0"
  "CYRILLIC CAPITAL LETTER ZHE WITH DIAERESIS\0"
  "CYRILLIC CAPITAL LETTER ZHWE\0"
  "CYRILLIC CAPITAL LIGATURE A IE\0"
  "CYRILLIC CAPITAL LIGATURE EN GHE\0"
  "CYRILLIC CAPITAL LIGATURE TE TSE\0"
  "CYRILLIC KAVYKA\0"
  "CYRILLIC LETTER MULTIOCULAR O\0"
  "CYRILLIC LETTER PALOCHKA\0"
  "CYRILLIC LETTER SMALL CAPITAL EL\0"
  "CYRILLIC PAYEROK\0"
  "CYRILLIC SMALL LETTER A\0"
  "CYRILLIC SMALL LETTER A WITH BREVE\0"
  "CYRILLIC SMALL LETTER A WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER ABKHASIAN CHE\0"
  "CYRILLIC SMALL LETTER ABKHASIAN CHE WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER ABKHASIAN DZE\0"
  "CYRILLIC SMALL LETTER ABKHASIAN HA\0"
  "CYRILLIC SMALL LETTER ALEUT KA\0"
  "CYRILLIC SMALL LETTER BARRED O\0"
  "CYRILLIC SMALL LETTER BARRED O WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER BASHKIR KA\0"
  "CYRILLIC SMALL LETTER BE\0"
  "CYRILLIC SMALL LETTER BIG YUS\0"
  "CYRILLIC SMALL LETTER BINOCULAR O\0"
  "CYRILLIC SMALL LETTER BLENDED YUS\0"
  "CYRILLIC SMALL LETTER BROAD OMEGA\0"
  "CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I\0"
  "CYRILLIC SMALL LETTER CCHE\0"
  "CYRILLIC SMALL LETTER CHE\0"
  "CYRILLIC SMALL LETTER CHE WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER CHE WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER CHE WITH VERTICAL STROKE\0"
  "CYRILLIC SMALL LETTER CLOSED LITTLE YUS\0"
  "CYRILLIC SMALL LETTER CROSSED O\0"
  "CYRILLIC SMALL LETTER DCHE\0"
  "CYRILLIC SMALL LETTER DE\0"
  "CYRILLIC SMALL LETTER DJE\0"
  "CYRILLIC SMALL LETTER DJERV\0"
  "CYRILLIC SMALL LETTER DOUBLE MONOCULAR O\0"
  "CYRILLIC SMALL LETTER DOUBLE O\0"
  "CYRILLIC SMALL LETTER DWE\0"
  "CYRILLIC SMALL LETTER DZE\0"
  "CYRILLIC SMALL LETTER DZELO\0"
  "CYRILLIC SMALL LETTER DZHE\0"
  "CYRILLIC SMALL LETTER DZWE\0"
  "CYRILLIC SMALL LETTER DZZE\0"
  "CYRILLIC SMALL LETTER DZZHE\0"
  "CYRILLIC SMALL LETTER E\0"
  "CYRILLIC SMALL LETTER E WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER EF\0"
  "CYRILLIC SMALL LETTER EL\0"
  "CYRILLIC SMALL LETTER EL WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER EL WITH HOOK\0"
  "CYRILLIC SMALL LETTER EL WITH MIDDLE HOOK\0"
  "CYRILLIC SMALL LETTER EL WITH TAIL\0"
  "CYRILLIC SMALL LETTER EM\0"
  "CYRILLIC SMALL LETTER EM WITH TAIL\0"
  "CYRILLIC SMALL LETTER EN\0"
  "CYRILLIC SMALL LETTER EN WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER EN WITH HOOK\0"
  "CYRILLIC SMALL LETTER EN WITH LEFT HOOK\0"
  "CYRILLIC SMALL LETTER EN WITH MIDDLE HOOK\0"
  "CYRILLIC SMALL LETTER EN WITH TAIL\0"
  "CYRILLIC SMALL LETTER ER\0"
  "CYRILLIC SMALL LETTER ER WITH TICK\0"
  "CYRILLIC SMALL LETTER ES\0"
  "CYRILLIC SMALL LETTER ES WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER FITA\0"
  "CYRILLIC SMALL LETTER GHE\0"
  "CYRILLIC SMALL LETTER GHE WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER GHE WITH MIDDLE HOOK\0"
  "CYRILLIC SMALL LETTER GHE WITH STROKE\0"
  "CYRILLIC SMALL LETTER GHE WITH STROKE AND HOOK\0"
  "CYRILLIC SMALL LETTER GHE WITH UPTURN\0"
  "CYRILLIC SMALL LETTER GJE\0"
  "CYRILLIC SMALL LETTER HA\0"
  "CYRILLIC SMALL LETTER HA WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER HA WITH HOOK\0"
  "CYRILLIC SMALL LETTER HA WITH STROKE\0"
  "CYRILLIC SMALL LETTER HARD SIGN\0"
  "CYRILLIC SMALL LETTER HWE\0"
  "CYRILLIC SMALL LETTER I\0"
  "CYRILLIC SMALL LETTER I WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER I WITH GRAVE\0"
  "CYRILLIC SMALL LETTER I WITH MACRON\0"
  "CYRILLIC SMALL LETTER IE\0"
  "CYRILLIC SMALL LETTER IE WITH BREVE\0"
  "CYRILLIC SMALL LETTER IE WITH GRAVE\0"
  "CYRILLIC SMALL LETTER IO\0"
  "CYRILLIC SMALL LETTER IOTA\0"
  "CYRILLIC SMALL LETTER IOTIFIED A\0"
  "CYRILLIC SMALL LETTER IOTIFIED BIG YUS\0"
  "CYRILLIC SMALL LETTER IOTIFIED CLOSED LITTLE YUS\0"
  "CYRILLIC SMALL LETTER IOTIFIED E\0"
  "CYRILLIC SMALL LETTER IOTIFIED LITTLE YUS\0"
  "CYRILLIC SMALL LETTER IOTIFIED YAT\0"
  "CYRILLIC SMALL LETTER IZHITSA\0"
  "CYRILLIC SMALL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT\0"
  "CYRILLIC SMALL LETTER JE\0"
  "CYRILLIC SMALL LETTER KA\0"
  "CYRILLIC SMALL LETTER KA WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER KA WITH HOOK\0"
  "CYRILLIC SMALL LETTER KA WITH STROKE\0"
  "CYRILLIC SMALL LETTER KA WITH VERTICAL STROKE\0"
  "CYRILLIC SMALL LETTER KHAKASSIAN CHE\0"
  "CYRILLIC SMALL LETTER KJE\0"
  "CYRILLIC SMALL LETTER KOMI DE\0"
  "CYRILLIC SMALL LETTER KOMI DJE\0"
  "CYRILLIC SMALL LETTER KOMI DZJE\0"
  "CYRILLIC SMALL LETTER KOMI LJE\0"
  "CYRILLIC SMALL LETTER KOMI NJE\0"
  "CYRILLIC SMALL LETTER KOMI SJE\0"
  "CYRILLIC SMALL LETTER KOMI TJE\0"
  "CYRILLIC SMALL LETTER KOMI ZJE\0"
  "CYRILLIC SMALL LETTER KOPPA\0"
  "CYRILLIC SMALL LETTER KSI\0"
  "CYRILLIC SMALL LETTER LHA\0"
  "CYRILLIC SMALL LETTER LITTLE YUS\0"
  "CYRILLIC SMALL LETTER LJE\0"
  "CYRILLIC SMALL LETTER LONG-LEGGED DE\0"
  "CYRILLIC SMALL LETTER MONOCULAR O\0"
  "CYRILLIC SMALL LETTER MONOGRAPH UK\0"
  "CYRILLIC SMALL LETTER NARROW O\0"
  "CYRILLIC SMALL LETTER NEUTRAL YER\0"
  "CYRILLIC SMALL LETTER NJE\0"
  "CYRILLIC SMALL LETTER O\0"
  "CYRILLIC SMALL LETTER O WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER OMEGA\0"
  "CYRILLIC SMALL LETTER OMEGA WITH TITLO\0"
  "CYRILLIC SMALL LETTER OT\0"
  "CYRILLIC SMALL LETTER PALOCHKA\0"
  "CYRILLIC SMALL LETTER PE\0"
  "CYRILLIC SMALL LETTER PE WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER PE WITH MIDDLE HOOK\0"
  "CYRILLIC SMALL LETTER PSI\0"
  "CYRILLIC SMALL LETTER QA\0"
  "CYRILLIC SMALL LETTER REVERSED DZE\0"
  "CYRILLIC SMALL LETTER REVERSED TSE\0"
  "CYRILLIC SMALL LETTER REVERSED YU\0"
  "CYRILLIC SMALL LETTER REVERSED ZE\0"
  "CYRILLIC SMALL LETTER RHA\0"
  "CYRILLIC SMALL LETTER ROUND OMEGA\0"
  "CYRILLIC SMALL LETTER ROUNDED VE\0"
  "CYRILLIC SMALL LETTER SCHWA\0"
  "CYRILLIC SMALL LETTER SCHWA WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER SEMISOFT SIGN\0"
  "CYRILLIC SMALL LETTER SHA\0"
  "CYRILLIC SMALL LETTER SHCHA\0"
  "CYRILLIC SMALL LETTER SHHA\0"
  "CYRILLIC SMALL LETTER SHHA WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER SHORT I\0"
  "CYRILLIC SMALL LETTER SHORT I WITH TAIL\0"
  "CYRILLIC SMALL LETTER SHORT U\0"
  "CYRILLIC SMALL LETTER SHWE\0"
  "CYRILLIC SMALL LETTER SOFT DE\0"
  "CYRILLIC SMALL LETTER SOFT EL\0"
  "CYRILLIC SMALL LETTER SOFT EM\0"
  "CYRILLIC SMALL LETTER SOFT SIGN\0"
  "CYRILLIC SMALL LETTER STRAIGHT U\0"
  "CYRILLIC SMALL LETTER STRAIGHT U WITH STROKE\0"
  "CYRILLIC SMALL LETTER TALL HARD SIGN\0"
  "CYRILLIC SMALL LETTER TALL TE\0"
  "CYRILLIC SMALL LETTER TALL YAT\0"
  "CYRILLIC SMALL LETTER TCHE\0"
  "CYRILLIC SMALL LETTER TE\0"
  "CYRILLIC SMALL LETTER TE WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER TE WITH MIDDLE HOOK\0"
  "CYRILLIC SMALL LETTER THREE-LEGGED TE\0"
  "CYRILLIC SMALL LETTER TJE\0"
  "CYRILLIC SMALL LETTER TSE\0"
  "CYRILLIC SMALL LETTER TSHE\0"
  "CYRILLIC SMALL LETTER TSSE\0"
  "CYRILLIC SMALL LETTER TSWE\0"
  "CYRILLIC SMALL LETTER TWE\0"
  "CYRILLIC SMALL LETTER U\0"
  "CYRILLIC SMALL LETTER U WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER U WITH DOUBLE ACUTE\0"
  "CYRILLIC SMALL LETTER U WITH MACRON\0"
  "CYRILLIC SMALL LETTER UK\0"
  "CYRILLIC SMALL LETTER UKRAINIAN IE\0"
  "CYRILLIC SMALL LETTER UNBLENDED UK\0"
  "CYRILLIC SMALL LETTER VE\0"
  "CYRILLIC SMALL LETTER WE\0"
  "CYRILLIC SMALL LETTER WIDE ES\0"
  "CYRILLIC SMALL LETTER YA\0"
  "CYRILLIC SMALL LETTER YAE\0"
  "CYRILLIC SMALL LETTER YAT\0"
  "CYRILLIC SMALL LETTER YERU\0"
  "CYRILLIC SMALL LETTER YERU WITH BACK YER\0"
  "CYRILLIC SMALL LETTER YERU WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER YI\0"
  "CYRILLIC SMALL LETTER YN\0"
  "CYRILLIC SMALL LETTER YU\0"
  "CYRILLIC SMALL LETTER ZE\0"
  "CYRILLIC SMALL LETTER ZE WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER ZE WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER ZEMLYA\0"
  "CYRILLIC SMALL LETTER ZHE\0"
  "CYRILLIC SMALL LETTER ZHE WITH BREVE\0"
  "CYRILLIC SMALL LETTER ZHE WITH DESCENDER\0"
  "CYRILLIC SMALL LETTER ZHE WITH DIAERESIS\0"
  "CYRILLIC SMALL LETTER ZHWE\0"
  "CYRILLIC SMALL LIGATURE A IE\0"
  "CYRILLIC SMALL LIGATURE EN GHE\0"
  "CYRILLIC SMALL LIGATURE TE TSE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER A\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER BE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER BYELORUSSIAN-UKRAINIAN I\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER CHE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER DE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER DZE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER DZHE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER EF\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER EL\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER ES\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER GHE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER GHE WITH UPTURN\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER HA\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER HARD SIGN\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER I\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER IE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER KA\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER O\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER PE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER SHA\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER TSE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER U\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER VE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER YERU\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER ZE\0"
  "CYRILLIC SUBSCRIPT SMALL LETTER ZHE\0"
  "CYRILLIC THOUSANDS SIGN\0"
  "DAGGER\0"
  "DAGGER KNIFE\0"
  "DAGGER WITH LEFT GUARD\0"
  "DAGGER WITH RIGHT GUARD\0"
  "DALET SYMBOL\0"
  "DANCER\0"
  "DANGO\0"
  "DARK SHADE\0"
  "DARK SUNGLASSES\0"
  "DASH SYMBOL\0"
  "DASH WITH LEFT UPTURN\0"
  "DASHED LOW LINE\0"
  "DASHED OVERLINE\0"
  "DASHED TRIANGLE-HEADED RIGHTWARDS ARROW\0"
  "DEAF PERSON\0"
  "DECIDUOUS TREE\0"
  "DECIMAL EXPONENT SYMBOL\0"
  "DECIMAL SEPARATOR KEY SYMBOL\0"
  "DECREASE FONT SIZE SYMBOL\0"
  "DEER\0"
  "DEGREE CELSIUS\0"
  "DEGREE FAHRENHEIT\0"
  "DEGREE SIGN\0"
  "DELIVERY TRUCK\0"
  "DELTA EQUAL TO\0"
  "DENSE HORIZONTAL FILL\0"
  "DENSE VERTICAL FILL\0"
  "DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL\0"
  "DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH CIRCLE\0"
  "DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH TRIANGLE\0"
  "DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH WAVE\0"
  "DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL\0"
  "DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH CIRCLE\0"
  "DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH TRIANGLE\0"
  "DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH WAVE\0"
  "DENTISTRY SYMBOL LIGHT VERTICAL AND BOTTOM LEFT\0"
  "DENTISTRY SYMBOL LIGHT VERTICAL AND BOTTOM RIGHT\0"
  "DENTISTRY SYMBOL LIGHT VERTICAL AND TOP LEFT\0"
  "DENTISTRY SYMBOL LIGHT VERTICAL AND TOP RIGHT\0"
  "DENTISTRY SYMBOL LIGHT VERTICAL AND WAVE\0"
  "DENTISTRY SYMBOL LIGHT VERTICAL WITH CIRCLE\0"
  "DENTISTRY SYMBOL LIGHT VERTICAL WITH TRIANGLE\0"
  "DEPARTMENT STORE\0"
  "DERELICT HOUSE BUILDING\0"
  "DESCENDING NODE\0"
  "DESERET CAPITAL LETTER AY\0"
  "DESERET CAPITAL LETTER BEE\0"
  "DESERET CAPITAL LETTER CHEE\0"
  "DESERET CAPITAL LETTER DEE\0"
  "DESERET CAPITAL LETTER EF\0"
  "DESERET CAPITAL LETTER EL\0"
  "DESERET CAPITAL LETTER EM\0"
  "DESERET CAPITAL LETTER EN\0"
  "DESERET CAPITAL LETTER ENG\0"
  "DESERET CAPITAL LETTER ER\0"
  "DESERET CAPITAL LETTER ES\0"
  "DESERET CAPITAL LETTER ESH\0"
  "DESERET CAPITAL LETTER ETH\0"
  "DESERET CAPITAL LETTER EW\0"
  "DESERET CAPITAL LETTER GAY\0"
  "DESERET CAPITAL LETTER H\0"
  "DESERET CAPITAL LETTER JEE\0"
  "DESERET CAPITAL LETTER KAY\0"
  "DESERET CAPITAL LETTER LONG A\0"
  "DESERET CAPITAL LETTER LONG AH\0"
  "DESERET CAPITAL LETTER LONG E\0"
  "DESERET CAPITAL LETTER LONG I\0"
  "DESERET CAPITAL LETTER LONG O\0"
  "DESERET CAPITAL LETTER LONG OO\0"
  "DESERET CAPITAL LETTER OI\0"
  "DESERET CAPITAL LETTER OW\0"
  "DESERET CAPITAL LETTER PEE\0"
  "DESERET CAPITAL LETTER SHORT A\0"
  "DESERET CAPITAL LETTER SHORT AH\0"
  "DESERET CAPITAL LETTER SHORT E\0"
  "DESERET CAPITAL LETTER SHORT I\0"
  "DESERET CAPITAL LETTER SHORT O\0"
  "DESERET CAPITAL LETTER SHORT OO\0"
  "DESERET CAPITAL LETTER TEE\0"
  "DESERET CAPITAL LETTER THEE\0"
  "DESERET CAPITAL LETTER VEE\0"
  "DESERET CAPITAL LETTER WU\0"
  "DESERET CAPITAL LETTER YEE\0"
  "DESERET CAPITAL LETTER ZEE\0"
  "DESERET CAPITAL LETTER ZHEE\0"
  "DESERET SMALL LETTER AY\0"
  "DESERET SMALL LETTER BEE\0"
  "DESERET SMALL LETTER CHEE\0"
  "DESERET SMALL LETTER DEE\0"
  "DESERET SMALL LETTER EF\0"
  "DESERET SMALL LETTER EL\0"
  "DESERET SMALL LETTER EM\0"
  "DESERET SMALL LETTER EN\0"
  "DESERET SMALL LETTER ENG\0"
  "DESERET SMALL LETTER ER\0"
  "DESERET SMALL LETTER ES\0"
  "DESERET SMALL LETTER ESH\0"
  "DESERET SMALL LETTER ETH\0"
  "DESERET SMALL LETTER EW\0"
  "DESERET SMALL LETTER GAY\0"
  "DESERET SMALL LETTER H\0"
  "DESERET SMALL LETTER JEE\0"
  "DESERET SMALL LETTER KAY\0"
  "DESERET SMALL LETTER LONG A\0"
  "DESERET SMALL LETTER LONG AH\0"
  "DESERET SMALL LETTER LONG E\0"
  "DESERET SMALL LETTER LONG I\0"
  "DESERET SMALL LETTER LONG O\0"
  "DESERET SMALL LETTER LONG OO\0"
  "DESERET SMALL LETTER OI\0"
  "DESERET SMALL LETTER OW\0"
  "DESERET SMALL LETTER PEE\0"
  "DESERET SMALL LETTER SHORT A\0"
  "DESERET SMALL LETTER SHORT AH\0"
  "DESERET SMALL LETTER SHORT E\0"
  "DESERET SMALL LETTER SHORT I\0"
  "DESERET SMALL LETTER SHORT O\0"
  "DESERET SMALL LETTER SHORT OO\0"
  "DESERET SMALL LETTER TEE\0"
  "DESERET SMALL LETTER THEE\0"
  "DESERET SMALL LETTER VEE\0"
  "DESERET SMALL LETTER WU\0"
  "DESERET SMALL LETTER YEE\0"
  "DESERET SMALL LETTER ZEE\0"
  "DESERET SMALL LETTER ZHEE\0"
  "DESERT\0"
  "DESERT ISLAND\0"
  "DESKTOP COMPUTER\0"
  "DESKTOP WINDOW\0"
  "DEVANAGARI ABBREVIATION SIGN\0"
  "DEVANAGARI ACUTE ACCENT\0"
  "DEVANAGARI CARET\0"
  "DEVANAGARI DANDA\0"
  "DEVANAGARI DIGIT EIGHT\0"
  "DEVANAGARI DIGIT FIVE\0"
  "DEVANAGARI DIGIT FOUR\0"
  "DEVANAGARI DIGIT NINE\0"
  "DEVANAGARI DIGIT ONE\0"
  "DEVANAGARI DIGIT SEVEN\0"
  "DEVANAGARI DIGIT SIX\0"
  "DEVANAGARI DIGIT THREE\0"
  "DEVANAGARI DIGIT TWO\0"
  "DEVANAGARI DIGIT ZERO\0"
  "DEVANAGARI DOUBLE DANDA\0"
  "DEVANAGARI GAP FILLER\0"
  "DEVANAGARI GRAVE ACCENT\0"
  "DEVANAGARI HEAD MARK\0"
  "DEVANAGARI HEAD MARK WITH HEADSTROKE\0"
  "DEVANAGARI HEADSTROKE\0"
  "DEVANAGARI JAIN OM\0"
  "DEVANAGARI LETTER A\0"
  "DEVANAGARI LETTER AA\0"
  "DEVANAGARI LETTER AI\0"
  "DEVANAGARI LETTER AU\0"
  "DEVANAGARI LETTER AW\0"
  "DEVANAGARI LETTER AY\0"
  "DEVANAGARI LETTER BA\0"
  "DEVANAGARI LETTER BBA\0"
  "DEVANAGARI LETTER BHA\0"
  "DEVANAGARI LETTER CA\0"
  "DEVANAGARI LETTER CANDRA A\0"
  "DEVANAGARI LETTER CANDRA E\0"
  "DEVANAGARI LETTER CANDRA O\0"
  "DEVANAGARI LETTER CHA\0"
  "DEVANAGARI LETTER DA\0"
  "DEVANAGARI LETTER DDA\0"
  "DEVANAGARI LETTER DDDA\0"
  "DEVANAGARI LETTER DDDHA\0"
  "DEVANAGARI LETTER DDHA\0"
  "DEVANAGARI LETTER DHA\0"
  "DEVANAGARI LETTER E\0"
  "DEVANAGARI LETTER FA\0"
  "DEVANAGARI LETTER GA\0"
  "DEVANAGARI LETTER GGA\0"
  "DEVANAGARI LETTER GHA\0"
  "DEVANAGARI LETTER GHHA\0"
  "DEVANAGARI LETTER GLOTTAL STOP\0"
  "DEVANAGARI LETTER HA\0"
  "DEVANAGARI LETTER HEAVY YA\0"
  "DEVANAGARI LETTER I\0"
  "DEVANAGARI LETTER II\0"
  "DEVANAGARI LETTER JA\0"
  "DEVANAGARI LETTER JHA\0"
  "DEVANAGARI LETTER JJA\0"
  "DEVANAGARI LETTER KA\0"
  "DEVANAGARI LETTER KHA\0"
  "DEVANAGARI LETTER KHHA\0"
  "DEVANAGARI LETTER LA\0"
  "DEVANAGARI LETTER LLA\0"
  "DEVANAGARI LETTER LLLA\0"
  "DEVANAGARI LETTER MA\0"
  "DEVANAGARI LETTER MARWARI DDA\0"
  "DEVANAGARI LETTER NA\0"
  "DEVANAGARI LETTER NGA\0"
  "DEVANAGARI LETTER NNA\0"
  "DEVANAGARI LETTER NNNA\0"
  "DEVANAGARI LETTER NYA\0"
  "DEVANAGARI LETTER O\0"
  "DEVANAGARI LETTER OE\0"
  "DEVANAGARI LETTER OOE\0"
  "DEVANAGARI LETTER PA\0"
  "DEVANAGARI LETTER PHA\0"
  "DEVANAGARI LETTER QA\0"
  "DEVANAGARI LETTER RA\0"
  "DEVANAGARI LETTER RHA\0"
  "DEVANAGARI LETTER RRA\0"
  "DEVANAGARI LETTER SA\0"
  "DEVANAGARI LETTER SHA\0"
  "DEVANAGARI LETTER SHORT A\0"
  "DEVANAGARI LETTER SHORT E\0"
  "DEVANAGARI LETTER SHORT O\0"
  "DEVANAGARI LETTER SSA\0"
  "DEVANAGARI LETTER TA\0"
  "DEVANAGARI LETTER THA\0"
  "DEVANAGARI LETTER TTA\0"
  "DEVANAGARI LETTER TTHA\0"
  "DEVANAGARI LETTER U\0"
  "DEVANAGARI LETTER UE\0"
  "DEVANAGARI LETTER UU\0"
  "DEVANAGARI LETTER UUE\0"
  "DEVANAGARI LETTER VA\0"
  "DEVANAGARI LETTER VOCALIC L\0"
  "DEVANAGARI LETTER VOCALIC LL\0"
  "DEVANAGARI LETTER VOCALIC R\0"
  "DEVANAGARI LETTER VOCALIC RR\0"
  "DEVANAGARI LETTER YA\0"
  "DEVANAGARI LETTER YYA\0"
  "DEVANAGARI LETTER ZA\0"
  "DEVANAGARI LETTER ZHA\0"
  "DEVANAGARI OM\0"
  "DEVANAGARI SIGN ANUSVARA\0"
  "DEVANAGARI SIGN AVAGRAHA\0"
  "DEVANAGARI SIGN BHALE\0"
  "DEVANAGARI SIGN BHALE WITH HOOK\0"
  "DEVANAGARI SIGN CANDRABINDU\0"
  "DEVANAGARI SIGN CANDRABINDU AVAGRAHA\0"
  "DEVANAGARI SIGN CANDRABINDU THREE\0"
  "DEVANAGARI SIGN CANDRABINDU TWO\0"
  "DEVANAGARI SIGN CANDRABINDU VIRAMA\0"
  "DEVANAGARI SIGN DOUBLE CANDRABINDU VIRAMA\0"
  "DEVANAGARI SIGN EXTENDED BHALE\0"
  "DEVANAGARI SIGN EXTENDED BHALE WITH HOOK\0"
  "DEVANAGARI SIGN HIGH SPACING DOT\0"
  "DEVANAGARI SIGN INVERTED CANDRABINDU\0"
  "DEVANAGARI SIGN MINDU\0"
  "DEVANAGARI SIGN NUKTA\0"
  "DEVANAGARI SIGN PUSHPIKA\0"
  "DEVANAGARI SIGN REVERSED NINE-LIKE BHALE\0"
  "DEVANAGARI SIGN SIDDHAM\0"
  "DEVANAGARI SIGN SPACING CANDRABINDU\0"
  "DEVANAGARI SIGN VIRAMA\0"
  "DEVANAGARI SIGN VISARGA\0"
  "DEVANAGARI SIGN WESTERN FIVE-LIKE BHALE\0"
  "DEVANAGARI SIGN WESTERN NINE-LIKE BHALE\0"
  "DEVANAGARI STRESS SIGN ANUDATTA\0"
  "DEVANAGARI STRESS SIGN UDATTA\0"
  "DEVANAGARI VOWEL SIGN AA\0"
  "DEVANAGARI VOWEL SIGN AI\0"
  "DEVANAGARI VOWEL SIGN AU\0"
  "DEVANAGARI VOWEL SIGN AW\0"
  "DEVANAGARI VOWEL SIGN AY\0"
  "DEVANAGARI VOWEL SIGN CANDRA E\0"
  "DEVANAGARI VOWEL SIGN CANDRA LONG E\0"
  "DEVANAGARI VOWEL SIGN CANDRA O\0"
  "DEVANAGARI VOWEL SIGN E\0"
  "DEVANAGARI VOWEL SIGN I\0"
  "DEVANAGARI VOWEL SIGN II\0"
  "DEVANAGARI VOWEL SIGN O\0"
  "DEVANAGARI VOWEL SIGN OE\0"
  "DEVANAGARI VOWEL SIGN OOE\0"
  "DEVANAGARI VOWEL SIGN PRISHTHAMATRA E\0"
  "DEVANAGARI VOWEL SIGN SHORT E\0"
  "DEVANAGARI VOWEL SIGN SHORT O\0"
  "DEVANAGARI VOWEL SIGN U\0"
  "DEVANAGARI VOWEL SIGN UE\0"
  "DEVANAGARI VOWEL SIGN UU\0"
  "DEVANAGARI VOWEL SIGN UUE\0"
  "DEVANAGARI VOWEL SIGN VOCALIC L\0"
  "DEVANAGARI VOWEL SIGN VOCALIC LL\0"
  "DEVANAGARI VOWEL SIGN VOCALIC R\0"
  "DEVANAGARI VOWEL SIGN VOCALIC RR\0"
  "DIAERESIS\0"
  "DIAGONAL CROSSHATCH FILL\0"
  "DIAMETER SIGN\0"
  "DIAMOND OPERATOR\0"
  "DIAMOND SHAPE WITH A DOT INSIDE\0"
  "DIAMOND TARGET\0"
  "DIAMOND WITH BOTTOM HALF BLACK\0"
  "DIAMOND WITH LEFT HALF BLACK\0"
  "DIAMOND WITH RIGHT HALF BLACK\0"
  "DIAMOND WITH TOP HALF BLACK\0"
  "DIE FACE-1\0"
  "DIE FACE-2\0"
  "DIE FACE-3\0"
  "DIE FACE-4\0"
  "DIE FACE-5\0"
  "DIE FACE-6\0"
  "DIESEL LOCOMOTIVE\0"
  "DIFFERENCE BETWEEN\0"
  "DIGIT EIGHT\0"
  "DIGIT EIGHT COMMA\0"
  "DIGIT EIGHT FULL STOP\0"
  "DIGIT FIVE\0"
  "DIGIT FIVE COMMA\0"
  "DIGIT FIVE FULL STOP\0"
  "DIGIT FOUR\0"
  "DIGIT FOUR COMMA\0"
  "DIGIT FOUR FULL STOP\0"
  "DIGIT NINE\0"
  "DIGIT NINE COMMA\0"
  "DIGIT NINE FULL STOP\0"
  "DIGIT ONE\0"
  "DIGIT ONE COMMA\0"
  "DIGIT ONE FULL STOP\0"
  "DIGIT SEVEN\0"
  "DIGIT SEVEN COMMA\0"
  "DIGIT SEVEN FULL STOP\0"
  "DIGIT SIX\0"
  "DIGIT SIX COMMA\0"
  "DIGIT SIX FULL STOP\0"
  "DIGIT THREE\0"
  "DIGIT THREE COMMA\0"
  "DIGIT THREE FULL STOP\0"
  "DIGIT TWO\0"
  "DIGIT TWO COMMA\0"
  "DIGIT TWO FULL STOP\0"
  "DIGIT ZERO\0"
  "DIGIT ZERO COMMA\0"
  "DIGIT ZERO FULL STOP\0"
  "DIGRAM FOR EARTH\0"
  "DIGRAM FOR EARTHLY HEAVEN\0"
  "DIGRAM FOR EARTHLY HUMAN\0"
  "DIGRAM FOR GREATER YANG\0"
  "DIGRAM FOR GREATER YIN\0"
  "DIGRAM FOR HEAVENLY EARTH\0"
  "DIGRAM FOR HUMAN EARTH\0"
  "DIGRAM FOR LESSER YANG\0"
  "DIGRAM FOR LESSER YIN\0"
  "DIMENSION ORIGIN\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT EIGHT\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT FIVE\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT FOUR\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT NINE\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT ONE\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT SEVEN\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT SIX\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT THREE\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT TWO\0"
  "DINGBAT CIRCLED SANS-SERIF DIGIT ZERO\0"
  "DINGBAT CIRCLED SANS-SERIF NUMBER TEN\0"
  "DINGBAT NEGATIVE CIRCLED DIGIT EIGHT\0"
  "DINGBAT NEGATIVE CIRCLED DIGIT FIVE\0"
  "DINGBAT NEGATIVE CIRCLED DIGIT FOUR\0"
  "DINGBAT NEGATIVE CIRCLED DIGIT NINE\0"
  "DINGBAT NEGATIVE CIRCLED DIGIT ONE\0"
  "DINGBAT NEGATIVE CIRCLED DIGIT SEVEN\0"
  "DINGBAT NEGATIVE CIRCLED DIGIT SIX\0"
  "DINGBAT NEGATIVE CIRCLED DIGIT THREE\0"
  "DINGBAT NEGATIVE CIRCLED DIGIT TWO\0"
  "DINGBAT NEGATIVE CIRCLED NUMBER TEN\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT EIGHT\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FIVE\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FOUR\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT NINE\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ONE\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SEVEN\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SIX\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT THREE\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT TWO\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ZERO\0"
  "DINGBAT NEGATIVE CIRCLED SANS-SERIF NUMBER TEN\0"
  "DIRECT CURRENT SYMBOL FORM TWO\0"
  "DIRECT HIT\0"
  "DISABLED CAR\0"
  "DISAPPOINTED BUT RELIEVED FACE\0"
  "DISAPPOINTED FACE\0"
  "DISCONTINUOUS UNDERLINE SYMBOL\0"
  "DISGUISED FACE\0"
  "DITTO MARK\0"
  "DIVES AKURU DIGIT EIGHT\0"
  "DIVES AKURU DIGIT FIVE\0"
  "DIVES AKURU DIGIT FOUR\0"
  "DIVES AKURU DIGIT NINE\0"
  "DIVES AKURU DIGIT ONE\0"
  "DIVES AKURU DIGIT SEVEN\0"
  "DIVES AKURU DIGIT SIX\0"
  "DIVES AKURU DIGIT THREE\0"
  "DIVES AKURU DIGIT TWO\0"
  "DIVES AKURU DIGIT ZERO\0"
  "DIVES AKURU DOUBLE DANDA\0"
  "DIVES AKURU END OF TEXT MARK\0"
  "DIVES AKURU GAP FILLER\0"
  "DIVES AKURU INITIAL RA\0"
  "DIVES AKURU LETTER A\0"
  "DIVES AKURU LETTER AA\0"
  "DIVES AKURU LETTER BA\0"
  "DIVES AKURU LETTER BHA\0"
  "DIVES AKURU LETTER CA\0"
  "DIVES AKURU LETTER CHA\0"
  "DIVES AKURU LETTER DA\0"
  "DIVES AKURU LETTER DDA\0"
  "DIVES AKURU LETTER DDHA\0"
  "DIVES AKURU LETTER DHA\0"
  "DIVES AKURU LETTER E\0"
  "DIVES AKURU LETTER GA\0"
  "DIVES AKURU LETTER GHA\0"
  "DIVES AKURU LETTER HA\0"
  "DIVES AKURU LETTER I\0"
  "DIVES AKURU LETTER II\0"
  "DIVES AKURU LETTER JA\0"
  "DIVES AKURU LETTER KA\0"
  "DIVES AKURU LETTER KHA\0"
  "DIVES AKURU LETTER LA\0"
  "DIVES AKURU LETTER LLA\0"
  "DIVES AKURU LETTER MA\0"
  "DIVES AKURU LETTER NA\0"
  "DIVES AKURU LETTER NGA\0"
  "DIVES AKURU LETTER NNA\0"
  "DIVES AKURU LETTER NYA\0"
  "DIVES AKURU LETTER O\0"
  "DIVES AKURU LETTER PA\0"
  "DIVES AKURU LETTER PHA\0"
  "DIVES AKURU LETTER RA\0"
  "DIVES AKURU LETTER SA\0"
  "DIVES AKURU LETTER SHA\0"
  "DIVES AKURU LETTER SSA\0"
  "DIVES AKURU LETTER TA\0"
  "DIVES AKURU LETTER THA\0"
  "DIVES AKURU LETTER TTA\0"
  "DIVES AKURU LETTER U\0"
  "DIVES AKURU LETTER UU\0"
  "DIVES AKURU LETTER VA\0"
  "DIVES AKURU LETTER YA\0"
  "DIVES AKURU LETTER YYA\0"
  "DIVES AKURU LETTER ZA\0"
  "DIVES AKURU MEDIAL RA\0"
  "DIVES AKURU MEDIAL YA\0"
  "DIVES AKURU PREFIXED NASAL SIGN\0"
  "DIVES AKURU SIGN ANUSVARA\0"
  "DIVES AKURU SIGN CANDRABINDU\0"
  "DIVES AKURU SIGN HALANTA\0"
  "DIVES AKURU SIGN NUKTA\0"
  "DIVES AKURU VIRAMA\0"
  "DIVES AKURU VOWEL SIGN AA\0"
  "DIVES AKURU VOWEL SIGN AI\0"
  "DIVES AKURU VOWEL SIGN E\0"
  "DIVES AKURU VOWEL SIGN I\0"
  "DIVES AKURU VOWEL SIGN II\0"
  "DIVES AKURU VOWEL SIGN O\0"
  "DIVES AKURU VOWEL SIGN U\0"
  "DIVES AKURU VOWEL SIGN UU\0"
  "DIVIDES\0"
  "DIVING MASK\0"
  "DIVISION SIGN\0"
  "DIVISION SLASH\0"
  "DIVISION TIMES\0"
  "DIVORCE SYMBOL\0"
  "DIYA LAMP\0"
  "DIZZY FACE\0"
  "DIZZY SYMBOL\0"
  "DNA DOUBLE HELIX\0"
  "DO NOT LITTER SYMBOL\0"
  "DOCUMENT\0"
  "DOCUMENT WITH PICTURE\0"
  "DOCUMENT WITH TEXT\0"
  "DOCUMENT WITH TEXT AND PICTURE\0"
  "DODO\0"
  "DOES NOT CONTAIN AS MEMBER\0"
  "DOES NOT CONTAIN AS NORMAL SUBGROUP\0"
  "DOES NOT CONTAIN AS NORMAL SUBGROUP OR EQUAL\0"
  "DOES NOT DIVIDE\0"
  "DOES NOT DIVIDE WITH REVERSED NEGATION SLASH\0"
  "DOES NOT FORCE\0"
  "DOES NOT PRECEDE\0"
  "DOES NOT PRECEDE OR EQUAL\0"
  "DOES NOT PROVE\0"
  "DOES NOT SUCCEED\0"
  "DOES NOT SUCCEED OR EQUAL\0"
  "DOG\0"
  "DOG FACE\0"
  "DOGRA ABBREVIATION SIGN\0"
  "DOGRA LETTER A\0"
  "DOGRA LETTER AA\0"
  "DOGRA LETTER AI\0"
  "DOGRA LETTER AU\0"
  "DOGRA LETTER BA\0"
  "DOGRA LETTER BHA\0"
  "DOGRA LETTER CA\0"
  "DOGRA LETTER CHA\0"
  "DOGRA LETTER DA\0"
  "DOGRA LETTER DDA\0"
  "DOGRA LETTER DDHA\0"
  "DOGRA LETTER DHA\0"
  "DOGRA LETTER E\0"
  "DOGRA LETTER GA\0"
  "DOGRA LETTER GHA\0"
  "DOGRA LETTER HA\0"
  "DOGRA LETTER I\0"
  "DOGRA LETTER II\0"
  "DOGRA LETTER JA\0"
  "DOGRA LETTER JHA\0"
  "DOGRA LETTER KA\0"
  "DOGRA LETTER KHA\0"
  "DOGRA LETTER LA\0"
  "DOGRA LETTER MA\0"
  "DOGRA LETTER NA\0"
  "DOGRA LETTER NGA\0"
  "DOGRA LETTER NNA\0"
  "DOGRA LETTER NYA\0"
  "DOGRA LETTER O\0"
  "DOGRA LETTER PA\0"
  "DOGRA LETTER PHA\0"
  "DOGRA LETTER RA\0"
  "DOGRA LETTER RRA\0"
  "DOGRA LETTER SA\0"
  "DOGRA LETTER SHA\0"
  "DOGRA LETTER SSA\0"
  "DOGRA LETTER TA\0"
  "DOGRA LETTER THA\0"
  "DOGRA LETTER TTA\0"
  "DOGRA LETTER TTHA\0"
  "DOGRA LETTER U\0"
  "DOGRA LETTER UU\0"
  "DOGRA LETTER VA\0"
  "DOGRA LETTER YA\0"
  "DOGRA SIGN ANUSVARA\0"
  "DOGRA SIGN NUKTA\0"
  "DOGRA SIGN VIRAMA\0"
  "DOGRA SIGN VISARGA\0"
  "DOGRA VOWEL SIGN AA\0"
  "DOGRA VOWEL SIGN AI\0"
  "DOGRA VOWEL SIGN AU\0"
  "DOGRA VOWEL SIGN E\0"
  "DOGRA VOWEL SIGN I\0"
  "DOGRA VOWEL SIGN II\0"
  "DOGRA VOWEL SIGN O\0"
  "DOGRA VOWEL SIGN U\0"
  "DOGRA VOWEL SIGN UU\0"
  "DOGRA VOWEL SIGN VOCALIC R\0"
  "DOGRA VOWEL SIGN VOCALIC RR\0"
  "DOLLAR SIGN\0"
  "DOLPHIN\0"
  "DOMINO TILE HORIZONTAL BACK\0"
  "DOMINO TILE HORIZONTAL-00-00\0"
  "DOMINO TILE HORIZONTAL-00-01\0"
  "DOMINO TILE HORIZONTAL-00-02\0"
  "DOMINO TILE HORIZONTAL-00-03\0"
  "DOMINO TILE HORIZONTAL-00-04\0"
  "DOMINO TILE HORIZONTAL-00-05\0"
  "DOMINO TILE HORIZONTAL-00-06\0"
  "DOMINO TILE HORIZONTAL-01-00\0"
  "DOMINO TILE HORIZONTAL-01-01\0"
  "DOMINO TILE HORIZONTAL-01-02\0"
  "DOMINO TILE HORIZONTAL-01-03\0"
  "DOMINO TILE HORIZONTAL-01-04\0"
  "DOMINO TILE HORIZONTAL-01-05\0"
  "DOMINO TILE HORIZONTAL-01-06\0"
  "DOMINO TILE HORIZONTAL-02-00\0"
  "DOMINO TILE HORIZONTAL-02-01\0"
  "DOMINO TILE HORIZONTAL-02-02\0"
  "DOMINO TILE HORIZONTAL-02-03\0"
  "DOMINO TILE HORIZONTAL-02-04\0"
  "DOMINO TILE HORIZONTAL-02-05\0"
  "DOMINO TILE HORIZONTAL-02-06\0"
  "DOMINO TILE HORIZONTAL-03-00\0"
  "DOMINO TILE HORIZONTAL-03-01\0"
  "DOMINO TILE HORIZONTAL-03-02\0"
  "DOMINO TILE HORIZONTAL-03-03\0"
  "DOMINO TILE HORIZONTAL-03-04\0"
  "DOMINO TILE HORIZONTAL-03-05\0"
  "DOMINO TILE HORIZONTAL-03-06\0"
  "DOMINO TILE HORIZONTAL-04-00\0"
  "DOMINO TILE HORIZONTAL-04-01\0"
  "DOMINO TILE HORIZONTAL-04-02\0"
  "DOMINO TILE HORIZONTAL-04-03\0"
  "DOMINO TILE HORIZONTAL-04-04\0"
  "DOMINO TILE HORIZONTAL-04-05\0"
  "DOMINO TILE HORIZONTAL-04-06\0"
  "DOMINO TILE HORIZONTAL-05-00\0"
  "DOMINO TILE HORIZONTAL-05-01\0"
  "DOMINO TILE HORIZONTAL-05-02\0"
  "DOMINO TILE HORIZONTAL-05-03\0"
  "DOMINO TILE HORIZONTAL-05-04\0"
  "DOMINO TILE HORIZONTAL-05-05\0"
  "DOMINO TILE HORIZONTAL-05-06\0"
  "DOMINO TILE HORIZONTAL-06-00\0"
  "DOMINO TILE HORIZONTAL-06-01\0"
  "DOMINO TILE HORIZONTAL-06-02\0"
  "DOMINO TILE HORIZONTAL-06-03\0"
  "DOMINO TILE HORIZONTAL-06-04\0"
  "DOMINO TILE HORIZONTAL-06-05\0"
  "DOMINO TILE HORIZONTAL-06-06\0"
  "DOMINO TILE VERTICAL BACK\0"
  "DOMINO TILE VERTICAL-00-00\0"
  "DOMINO TILE VERTICAL-00-01\0"
  "DOMINO TILE VERTICAL-00-02\0"
  "DOMINO TILE VERTICAL-00-03\0"
  "DOMINO TILE VERTICAL-00-04\0"
  "DOMINO TILE VERTICAL-00-05\0"
  "DOMINO TILE VERTICAL-00-06\0"
  "DOMINO TILE VERTICAL-01-00\0"
  "DOMINO TILE VERTICAL-01-01\0"
  "DOMINO TILE VERTICAL-01-02\0"
  "DOMINO TILE VERTICAL-01-03\0"
  "DOMINO TILE VERTICAL-01-04\0"
  "DOMINO TILE VERTICAL-01-05\0"
  "DOMINO TILE VERTICAL-01-06\0"
  "DOMINO TILE VERTICAL-02-00\0"
  "DOMINO TILE VERTICAL-02-01\0"
  "DOMINO TILE VERTICAL-02-02\0"
  "DOMINO TILE VERTICAL-02-03\0"
  "DOMINO TILE VERTICAL-02-04\0"
  "DOMINO TILE VERTICAL-02-05\0"
  "DOMINO TILE VERTICAL-02-06\0"
  "DOMINO TILE VERTICAL-03-00\0"
  "DOMINO TILE VERTICAL-03-01\0"
  "DOMINO TILE VERTICAL-03-02\0"
  "DOMINO TILE VERTICAL-03-03\0"
  "DOMINO TILE VERTICAL-03-04\0"
  "DOMINO TILE VERTICAL-03-05\0"
  "DOMINO TILE VERTICAL-03-06\0"
  "DOMINO TILE VERTICAL-04-00\0"
  "DOMINO TILE VERTICAL-04-01\0"
  "DOMINO TILE VERTICAL-04-02\0"
  "DOMINO TILE VERTICAL-04-03\0"
  "DOMINO TILE VERTICAL-04-04\0"
  "DOMINO TILE VERTICAL-04-05\0"
  "DOMINO TILE VERTICAL-04-06\0"
  "DOMINO TILE VERTICAL-05-00\0"
  "DOMINO TILE VERTICAL-05-01\0"
  "DOMINO TILE VERTICAL-05-02\0"
  "DOMINO TILE VERTICAL-05-03\0"
  "DOMINO TILE VERTICAL-05-04\0"
  "DOMINO TILE VERTICAL-05-05\0"
  "DOMINO TILE VERTICAL-05-06\0"
  "DOMINO TILE VERTICAL-06-00\0"
  "DOMINO TILE VERTICAL-06-01\0"
  "DOMINO TILE VERTICAL-06-02\0"
  "DOMINO TILE VERTICAL-06-03\0"
  "DOMINO TILE VERTICAL-06-04\0"
  "DOMINO TILE VERTICAL-06-05\0"
  "DOMINO TILE VERTICAL-06-06\0"
  "DONG SIGN\0"
  "DONKEY\0"
  "DOOR\0"
  "DOT ABOVE\0"
  "DOT MINUS\0"
  "DOT OPERATOR\0"
  "DOT PLUS\0"
  "DOTTED CIRCLE\0"
  "DOTTED CROSS\0"
  "DOTTED FENCE\0"
  "DOTTED LINE FACE\0"
  "DOTTED OBELOS\0"
  "DOTTED RIGHT-POINTING ANGLE\0"
  "DOTTED SOLIDUS\0"
  "DOTTED SQUARE\0"
  "DOTTED TRANSPOSITION MARKER\0"
  "DOUBLE ACUTE ACCENT\0"
  "DOUBLE CIRCLED DIGIT EIGHT\0"
  "DOUBLE CIRCLED DIGIT FIVE\0"
  "DOUBLE CIRCLED DIGIT FOUR\0"
  "DOUBLE CIRCLED DIGIT NINE\0"
  "DOUBLE CIRCLED DIGIT ONE\0"
  "DOUBLE CIRCLED DIGIT SEVEN\0"
  "DOUBLE CIRCLED DIGIT SIX\0"
  "DOUBLE CIRCLED DIGIT THREE\0"
  "DOUBLE CIRCLED DIGIT TWO\0"
  "DOUBLE CIRCLED NUMBER TEN\0"
  "DOUBLE COLON EQUAL\0"
  "DOUBLE CURLY LOOP\0"
  "DOUBLE DAGGER\0"
  "DOUBLE DOWN TACK\0"
  "DOUBLE EXCLAMATION MARK\0"
  "DOUBLE HIGH-REVERSED-9 QUOTATION MARK\0"
  "DOUBLE HYPHEN\0"
  "DOUBLE INTEGRAL\0"
  "DOUBLE INTERSECTION\0"
  "DOUBLE LEFT ARC GREATER-THAN BRACKET\0"
  "DOUBLE LOGICAL AND\0"
  "DOUBLE LOGICAL OR\0"
  "DOUBLE LOW LINE\0"
  "DOUBLE LOW-9 QUOTATION MARK\0"
  "DOUBLE LOW-REVERSED-9 QUOTATION MARK\0"
  "DOUBLE NESTED GREATER-THAN\0"
  "DOUBLE NESTED LESS-THAN\0"
  "DOUBLE NESTED LESS-THAN WITH UNDERBAR\0"
  "DOUBLE OBLIQUE HYPHEN\0"
  "DOUBLE PLUS\0"
  "DOUBLE PRECEDES\0"
  "DOUBLE PRIME\0"
  "DOUBLE PRIME QUOTATION MARK\0"
  "DOUBLE QUESTION MARK\0"
  "DOUBLE RIGHT ARC LESS-THAN BRACKET\0"
  "DOUBLE SOLIDUS OPERATOR\0"
  "DOUBLE SQUARE INTERSECTION\0"
  "DOUBLE SQUARE UNION\0"
  "DOUBLE STACKED COMMA\0"
  "DOUBLE STROKE NOT SIGN\0"
  "DOUBLE SUBSET\0"
  "DOUBLE SUCCEEDS\0"
  "DOUBLE SUPERSET\0"
  "DOUBLE SUSPENSION MARK\0"
  "DOUBLE UNION\0"
  "DOUBLE UP TACK\0"
  "DOUBLE VERTICAL BAR\0"
  "DOUBLE VERTICAL BAR DOUBLE LEFT TURNSTILE\0"
  "DOUBLE VERTICAL BAR DOUBLE RIGHT TURNSTILE\0"
  "DOUBLE VERTICAL BAR LEFT TURNSTILE\0"
  "DOUBLE VERTICAL LINE\0"
  "DOUBLE WAVY OVERLINE\0"
  "DOUBLE-ENDED MULTIMAP\0"
  "DOUBLE-LINE EQUAL TO OR GREATER-THAN\0"
  "DOUBLE-LINE EQUAL TO OR LESS-THAN\0"
  "DOUBLE-LINE SLANTED EQUAL TO OR GREATER-THAN\0"
  "DOUBLE-LINE SLANTED EQUAL TO OR LESS-THAN\0"
  "DOUBLE-LINE SLANTED GREATER-THAN OR EQUAL TO\0"
  "DOUBLE-LINE SLANTED LESS-THAN OR EQUAL TO\0"
  "DOUBLE-STRUCK CAPITAL C\0"
  "DOUBLE-STRUCK CAPITAL GAMMA\0"
  "DOUBLE-STRUCK CAPITAL H\0"
  "DOUBLE-STRUCK CAPITAL N\0"
  "DOUBLE-STRUCK CAPITAL P\0"
  "DOUBLE-STRUCK CAPITAL PI\0"
  "DOUBLE-STRUCK CAPITAL Q\0"
  "DOUBLE-STRUCK CAPITAL R\0"
  "DOUBLE-STRUCK CAPITAL Z\0"
  "DOUBLE-STRUCK ITALIC CAPITAL D\0"
  "DOUBLE-STRUCK ITALIC SMALL D\0"
  "DOUBLE-STRUCK ITALIC SMALL E\0"
  "DOUBLE-STRUCK ITALIC SMALL I\0"
  "DOUBLE-STRUCK ITALIC SMALL J\0"
  "DOUBLE-STRUCK N-ARY SUMMATION\0"
  "DOUBLE-STRUCK SMALL GAMMA\0"
  "DOUBLE-STRUCK SMALL PI\0"
  "DOUBLED FEMALE SIGN\0"
  "DOUBLED MALE SIGN\0"
  "DOUBLED SYMBOL\0"
  "DOUGHNUT\0"
  "DOVE OF PEACE\0"
  "DOWN ARROWHEAD\0"
  "DOWN FISH TAIL\0"
  "DOWN RIGHT DIAGONAL ELLIPSIS\0"
  "DOWN TACK\0"
  "DOWN TACK WITH CIRCLE BELOW\0"
  "DOWN-FACING SNAKE HEAD WITH CLOSED MOUTH\0"
  "DOWN-FACING SNAKE HEAD WITH OPEN MOUTH\0"
  "DOWN-POINTING AIRPLANE\0"
  "DOWN-POINTING ATOMIC BOMB\0"
  "DOWN-POINTING ENERGY WAVE\0"
  "DOWN-POINTING FROG\0"
  "DOWN-POINTING RACING CAR\0"
  "DOWN-POINTING RED TRIANGLE\0"
  "DOWN-POINTING RIFLE\0"
  "DOWN-POINTING ROCKET SHIP\0"
  "DOWN-POINTING SMALL RED TRIANGLE\0"
  "DOWN-POINTING STICK FIGURE\0"
  "DOWN-POINTING TRIANGLE WITH LEFT HALF BLACK\0"
  "DOWN-POINTING TRIANGLE WITH RIGHT HALF BLACK\0"
  "DOWNWARD FACING HOOK\0"
  "DOWNWARD FACING HOOK WITH DOT\0"
  "DOWNWARD FACING NOTCHED HOOK\0"
  "DOWNWARD FACING NOTCHED HOOK WITH DOT\0"
  "DOWNWARDS ANCORA\0"
  "DOWNWARDS ARROW\0"
  "DOWNWARDS ARROW AND RIGHT ONE EIGHTH BLOCK\0"
  "DOWNWARDS ARROW FROM BAR\0"
  "DOWNWARDS ARROW LEFTWARDS OF UPWARDS ARROW\0"
  "DOWNWARDS ARROW TO BAR\0"
  "DOWNWARDS ARROW WITH CORNER LEFTWARDS\0"
  "DOWNWARDS ARROW WITH DOUBLE STROKE\0"
  "DOWNWARDS ARROW WITH EQUILATERAL ARROWHEAD\0"
  "DOWNWARDS ARROW WITH HORIZONTAL STROKE\0"
  "DOWNWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD\0"
  "DOWNWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD\0"
  "DOWNWARDS ARROW WITH NOTCHED TAIL\0"
  "DOWNWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD\0"
  "DOWNWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD\0"
  "DOWNWARDS ARROW WITH TIP LEFTWARDS\0"
  "DOWNWARDS ARROW WITH TIP RIGHTWARDS\0"
  "DOWNWARDS BLACK ARROW\0"
  "DOWNWARDS BLACK ARROW TO BAR\0"
  "DOWNWARDS BLACK CIRCLED WHITE ARROW\0"
  "DOWNWARDS COMPRESSED ARROW\0"
  "DOWNWARDS DASHED ARROW\0"
  "DOWNWARDS DOUBLE ARROW\0"
  "DOWNWARDS FINGER-POST ARROW\0"
  "DOWNWARDS HARPOON WITH BARB LEFT BESIDE DOWNWARDS HARPOON WITH BARB RIGHT\0"
  "DOWNWARDS HARPOON WITH BARB LEFT BESIDE UPWARDS HARPOON WITH BARB RIGHT\0"
  "DOWNWARDS HARPOON WITH BARB LEFT FROM BAR\0"
  "DOWNWARDS HARPOON WITH BARB LEFT TO BAR\0"
  "DOWNWARDS HARPOON WITH BARB LEFTWARDS\0"
  "DOWNWARDS HARPOON WITH BARB RIGHT FROM BAR\0"
  "DOWNWARDS HARPOON WITH BARB RIGHT TO BAR\0"
  "DOWNWARDS HARPOON WITH BARB RIGHTWARDS\0"
  "DOWNWARDS HEAVY ARROW\0"
  "DOWNWARDS HEAVY COMPRESSED ARROW\0"
  "DOWNWARDS PAIRED ARROWS\0"
  "DOWNWARDS QUADRUPLE ARROW\0"
  "DOWNWARDS ROCKET\0"
  "DOWNWARDS SANS-SERIF ARROW\0"
  "DOWNWARDS SQUARED ARROW\0"
  "DOWNWARDS TRIANGLE ARROWHEAD\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW LEFTWARDS OF UPWARDS TRIANGLE-HEADED ARROW\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW TO BAR\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW WITH LONG TIP LEFTWARDS\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW WITH LONG TIP RIGHTWARDS\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT\0"
  "DOWNWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT\0"
  "DOWNWARDS TRIANGLE-HEADED DASHED ARROW\0"
  "DOWNWARDS TRIANGLE-HEADED PAIRED ARROWS\0"
  "DOWNWARDS TRIANGLE-HEADED ZIGZAG ARROW\0"
  "DOWNWARDS TRIPLE ARROW\0"
  "DOWNWARDS TWO HEADED ARROW\0"
  "DOWNWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS\0"
  "DOWNWARDS WHITE ARROW\0"
  "DOWNWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD\0"
  "DOWNWARDS ZIGZAG ARROW\0"
  "DRACHMA SIGN\0"
  "DRAFTING POINT RIGHTWARDS ARROW\0"
  "DRAGON\0"
  "DRAGON FACE\0"
  "DRESS\0"
  "DRIVE SLOW SIGN\0"
  "DROMEDARY CAMEL\0"
  "DROOLING FACE\0"
  "DROP OF BLOOD\0"
  "DROPLET\0"
  "DRUM WITH DRUMSTICKS\0"
  "DUCK\0"
  "DUMPLING\0"
  "DUPLOYAN AFFIX ATTACHED E HOOK\0"
  "DUPLOYAN AFFIX ATTACHED I HOOK\0"
  "DUPLOYAN AFFIX ATTACHED LEFT-TO-RIGHT SECANT\0"
  "DUPLOYAN AFFIX ATTACHED SECANT\0"
  "DUPLOYAN AFFIX ATTACHED TAIL\0"
  "DUPLOYAN AFFIX ATTACHED TANGENT\0"
  "DUPLOYAN AFFIX ATTACHED TANGENT HOOK\0"
  "DUPLOYAN AFFIX HIGH ACUTE\0"
  "DUPLOYAN AFFIX HIGH CIRCLE\0"
  "DUPLOYAN AFFIX HIGH DOT\0"
  "DUPLOYAN AFFIX HIGH GRAVE\0"
  "DUPLOYAN AFFIX HIGH LINE\0"
  "DUPLOYAN AFFIX HIGH LONG GRAVE\0"
  "DUPLOYAN AFFIX HIGH TIGHT ACUTE\0"
  "DUPLOYAN AFFIX HIGH VERTICAL\0"
  "DUPLOYAN AFFIX HIGH VERTICAL SECANT\0"
  "DUPLOYAN AFFIX HIGH WAVE\0"
  "DUPLOYAN AFFIX LEFT HORIZONTAL SECANT\0"
  "DUPLOYAN AFFIX LOW ACUTE\0"
  "DUPLOYAN AFFIX LOW ARROW\0"
  "DUPLOYAN AFFIX LOW CIRCLE\0"
  "DUPLOYAN AFFIX LOW DOT\0"
  "DUPLOYAN AFFIX LOW GRAVE\0"
  "DUPLOYAN AFFIX LOW LINE\0"
  "DUPLOYAN AFFIX LOW LONG GRAVE\0"
  "DUPLOYAN AFFIX LOW TIGHT ACUTE\0"
  "DUPLOYAN AFFIX LOW VERTICAL\0"
  "DUPLOYAN AFFIX LOW VERTICAL SECANT\0"
  "DUPLOYAN AFFIX LOW WAVE\0"
  "DUPLOYAN AFFIX MID HORIZONTAL SECANT\0"
  "DUPLOYAN AFFIX MID VERTICAL SECANT\0"
  "DUPLOYAN AFFIX RIGHT HORIZONTAL SECANT\0"
  "DUPLOYAN DOUBLE MARK\0"
  "DUPLOYAN LETTER A\0"
  "DUPLOYAN LETTER AOU\0"
  "DUPLOYAN LETTER B\0"
  "DUPLOYAN LETTER D\0"
  "DUPLOYAN LETTER D S\0"
  "DUPLOYAN LETTER DH\0"
  "DUPLOYAN LETTER E\0"
  "DUPLOYAN LETTER EE\0"
  "DUPLOYAN LETTER EU\0"
  "DUPLOYAN LETTER F\0"
  "DUPLOYAN LETTER F N\0"
  "DUPLOYAN LETTER G\0"
  "DUPLOYAN LETTER G R S\0"
  "DUPLOYAN LETTER H\0"
  "DUPLOYAN LETTER HL\0"
  "DUPLOYAN LETTER I\0"
  "DUPLOYAN LETTER IE\0"
  "DUPLOYAN LETTER J\0"
  "DUPLOYAN LETTER J M\0"
  "DUPLOYAN LETTER J M S\0"
  "DUPLOYAN LETTER J N\0"
  "DUPLOYAN LETTER J N S\0"
  "DUPLOYAN LETTER J S\0"
  "DUPLOYAN LETTER J S WITH DOT\0"
  "DUPLOYAN LETTER J WITH DOT\0"
  "DUPLOYAN LETTER J WITH DOTS INSIDE AND ABOVE\0"
  "DUPLOYAN LETTER K\0"
  "DUPLOYAN LETTER K M\0"
  "DUPLOYAN LETTER K R S\0"
  "DUPLOYAN LETTER KK\0"
  "DUPLOYAN LETTER L\0"
  "DUPLOYAN LETTER LH\0"
  "DUPLOYAN LETTER LONG I\0"
  "DUPLOYAN LETTER LONG U\0"
  "DUPLOYAN LETTER M\0"
  "DUPLOYAN LETTER M N\0"
  "DUPLOYAN LETTER M N S\0"
  "DUPLOYAN LETTER M S\0"
  "DUPLOYAN LETTER M WITH DOT\0"
  "DUPLOYAN LETTER N\0"
  "DUPLOYAN LETTER N M\0"
  "DUPLOYAN LETTER N M S\0"
  "DUPLOYAN LETTER N S\0"
  "DUPLOYAN LETTER N WITH DOT\0"
  "DUPLOYAN LETTER NASAL A\0"
  "DUPLOYAN LETTER NASAL I\0"
  "DUPLOYAN LETTER NASAL O\0"
  "DUPLOYAN LETTER NASAL U\0"
  "DUPLOYAN LETTER O\0"
  "DUPLOYAN LETTER OA\0"
  "DUPLOYAN LETTER OOH\0"
  "DUPLOYAN LETTER OU\0"
  "DUPLOYAN LETTER OW\0"
  "DUPLOYAN LETTER P\0"
  "DUPLOYAN LETTER P N\0"
  "DUPLOYAN LETTER PERNIN AM\0"
  "DUPLOYAN LETTER PERNIN AN\0"
  "DUPLOYAN LETTER R\0"
  "DUPLOYAN LETTER R S\0"
  "DUPLOYAN LETTER RH\0"
  "DUPLOYAN LETTER ROMANIAN I\0"
  "DUPLOYAN LETTER ROMANIAN U\0"
  "DUPLOYAN LETTER S\0"
  "DUPLOYAN LETTER S J\0"
  "DUPLOYAN LETTER S J S\0"
  "DUPLOYAN LETTER S K\0"
  "DUPLOYAN LETTER S K R\0"
  "DUPLOYAN LETTER S M\0"
  "DUPLOYAN LETTER S N\0"
  "DUPLOYAN LETTER S P\0"
  "DUPLOYAN LETTER S P R\0"
  "DUPLOYAN LETTER S S\0"
  "DUPLOYAN LETTER S T\0"
  "DUPLOYAN LETTER S T R\0"
  "DUPLOYAN LETTER S WITH DOT\0"
  "DUPLOYAN LETTER S WITH DOT BELOW\0"
  "DUPLOYAN LETTER SHORT I\0"
  "DUPLOYAN LETTER SLOAN AN\0"
  "DUPLOYAN LETTER SLOAN DH\0"
  "DUPLOYAN LETTER SLOAN EE\0"
  "DUPLOYAN LETTER SLOAN EH\0"
  "DUPLOYAN LETTER SLOAN EN\0"
  "DUPLOYAN LETTER SLOAN J\0"
  "DUPLOYAN LETTER SLOAN ON\0"
  "DUPLOYAN LETTER SLOAN OW\0"
  "DUPLOYAN LETTER SLOAN U\0"
  "DUPLOYAN LETTER T\0"
  "DUPLOYAN LETTER T R S\0"
  "DUPLOYAN LETTER T S\0"
  "DUPLOYAN LETTER TH\0"
  "DUPLOYAN LETTER U\0"
  "DUPLOYAN LETTER U N\0"
  "DUPLOYAN LETTER UH\0"
  "DUPLOYAN LETTER UI\0"
  "DUPLOYAN LETTER V\0"
  "DUPLOYAN LETTER VOCALIC M\0"
  "DUPLOYAN LETTER W\0"
  "DUPLOYAN LETTER W R\0"
  "DUPLOYAN LETTER WA\0"
  "DUPLOYAN LETTER WEI\0"
  "DUPLOYAN LETTER WH\0"
  "DUPLOYAN LETTER WI\0"
  "DUPLOYAN LETTER WO\0"
  "DUPLOYAN LETTER WOW\0"
  "DUPLOYAN LETTER X\0"
  "DUPLOYAN LETTER XW\0"
  "DUPLOYAN LETTER YE\0"
  "DUPLOYAN PUNCTUATION CHINOOK FULL STOP\0"
  "DUPLOYAN SIGN O WITH CROSS\0"
  "DUPLOYAN THICK LETTER SELECTOR\0"
  "DVD\0"
  "E-MAIL SYMBOL\0"
  "EAGLE\0"
  "EAR\0"
  "EAR OF MAIZE\0"
  "EAR OF RICE\0"
  "EAR WITH HEARING AID\0"
  "EARTH\0"
  "EARTH GLOBE AMERICAS\0"
  "EARTH GLOBE ASIA-AUSTRALIA\0"
  "EARTH GLOBE EUROPE-AFRICA\0"
  "EARTH GROUND\0"
  "EAST SYRIAC CROSS\0"
  "EDITORIAL CORONIS\0"
  "EGG\0"
  "EGYPTIAN HIEROGLYPH A001\0"
  "EGYPTIAN HIEROGLYPH A002\0"
  "EGYPTIAN HIEROGLYPH A003\0"
  "EGYPTIAN HIEROGLYPH A004\0"
  "EGYPTIAN HIEROGLYPH A005\0"
  "EGYPTIAN HIEROGLYPH A005A\0"
  "EGYPTIAN HIEROGLYPH A006\0"
  "EGYPTIAN HIEROGLYPH A006A\0"
  "EGYPTIAN HIEROGLYPH A006B\0"
  "EGYPTIAN HIEROGLYPH A007\0"
  "EGYPTIAN HIEROGLYPH A008\0"
  "EGYPTIAN HIEROGLYPH A009\0"
  "EGYPTIAN HIEROGLYPH A010\0"
  "EGYPTIAN HIEROGLYPH A011\0"
  "EGYPTIAN HIEROGLYPH A012\0"
  "EGYPTIAN HIEROGLYPH A013\0"
  "EGYPTIAN HIEROGLYPH A014\0"
  "EGYPTIAN HIEROGLYPH A014A\0"
  "EGYPTIAN HIEROGLYPH A015\0"
  "EGYPTIAN HIEROGLYPH A016\0"
  "EGYPTIAN HIEROGLYPH A017\0"
  "EGYPTIAN HIEROGLYPH A017A\0"
  "EGYPTIAN HIEROGLYPH A018\0"
  "EGYPTIAN HIEROGLYPH A019\0"
  "EGYPTIAN HIEROGLYPH A020\0"
  "EGYPTIAN HIEROGLYPH A021\0"
  "EGYPTIAN HIEROGLYPH A022\0"
  "EGYPTIAN HIEROGLYPH A023\0"
  "EGYPTIAN HIEROGLYPH A024\0"
  "EGYPTIAN HIEROGLYPH A025\0"
  "EGYPTIAN HIEROGLYPH A026\0"
  "EGYPTIAN HIEROGLYPH A027\0"
  "EGYPTIAN HIEROGLYPH A028\0"
  "EGYPTIAN HIEROGLYPH A029\0"
  "EGYPTIAN HIEROGLYPH A030\0"
  "EGYPTIAN HIEROGLYPH A031\0"
  "EGYPTIAN HIEROGLYPH A032\0"
  "EGYPTIAN HIEROGLYPH A032A\0"
  "EGYPTIAN HIEROGLYPH A033\0"
  "EGYPTIAN HIEROGLYPH A034\0"
  "EGYPTIAN HIEROGLYPH A035\0"
  "EGYPTIAN HIEROGLYPH A036\0"
  "EGYPTIAN HIEROGLYPH A037\0"
  "EGYPTIAN HIEROGLYPH A038\0"
  "EGYPTIAN HIEROGLYPH A039\0"
  "EGYPTIAN HIEROGLYPH A040\0"
  "EGYPTIAN HIEROGLYPH A040A\0"
  "EGYPTIAN HIEROGLYPH A041\0"
  "EGYPTIAN HIEROGLYPH A042\0"
  "EGYPTIAN HIEROGLYPH A042A\0"
  "EGYPTIAN HIEROGLYPH A043\0"
  "EGYPTIAN HIEROGLYPH A043A\0"
  "EGYPTIAN HIEROGLYPH A044\0"
  "EGYPTIAN HIEROGLYPH A045\0"
  "EGYPTIAN HIEROGLYPH A045A\0"
  "EGYPTIAN HIEROGLYPH A046\0"
  "EGYPTIAN HIEROGLYPH A047\0"
  "EGYPTIAN HIEROGLYPH A048\0"
  "EGYPTIAN HIEROGLYPH A049\0"
  "EGYPTIAN HIEROGLYPH A050\0"
  "EGYPTIAN HIEROGLYPH A051\0"
  "EGYPTIAN HIEROGLYPH A052\0"
  "EGYPTIAN HIEROGLYPH A053\0"
  "EGYPTIAN HIEROGLYPH A054\0"
  "EGYPTIAN HIEROGLYPH A055\0"
  "EGYPTIAN HIEROGLYPH A056\0"
  "EGYPTIAN HIEROGLYPH A057\0"
  "EGYPTIAN HIEROGLYPH A058\0"
  "EGYPTIAN HIEROGLYPH A059\0"
  "EGYPTIAN HIEROGLYPH A060\0"
  "EGYPTIAN HIEROGLYPH A061\0"
  "EGYPTIAN HIEROGLYPH A062\0"
  "EGYPTIAN HIEROGLYPH A063\0"
  "EGYPTIAN HIEROGLYPH A064\0"
  "EGYPTIAN HIEROGLYPH A065\0"
  "EGYPTIAN HIEROGLYPH A066\0"
  "EGYPTIAN HIEROGLYPH A067\0"
  "EGYPTIAN HIEROGLYPH A068\0"
  "EGYPTIAN HIEROGLYPH A069\0"
  "EGYPTIAN HIEROGLYPH A070\0"
  "EGYPTIAN HIEROGLYPH AA001\0"
  "EGYPTIAN HIEROGLYPH AA002\0"
  "EGYPTIAN HIEROGLYPH AA003\0"
  "EGYPTIAN HIEROGLYPH AA004\0"
  "EGYPTIAN HIEROGLYPH AA005\0"
  "EGYPTIAN HIEROGLYPH AA006\0"
  "EGYPTIAN HIEROGLYPH AA007\0"
  "EGYPTIAN HIEROGLYPH AA007A\0"
  "EGYPTIAN HIEROGLYPH AA007B\0"
  "EGYPTIAN HIEROGLYPH AA008\0"
  "EGYPTIAN HIEROGLYPH AA009\0"
  "EGYPTIAN HIEROGLYPH AA010\0"
  "EGYPTIAN HIEROGLYPH AA011\0"
  "EGYPTIAN HIEROGLYPH AA012\0"
  "EGYPTIAN HIEROGLYPH AA013\0"
  "EGYPTIAN HIEROGLYPH AA014\0"
  "EGYPTIAN HIEROGLYPH AA015\0"
  "EGYPTIAN HIEROGLYPH AA016\0"
  "EGYPTIAN HIEROGLYPH AA017\0"
  "EGYPTIAN HIEROGLYPH AA018\0"
  "EGYPTIAN HIEROGLYPH AA019\0"
  "EGYPTIAN HIEROGLYPH AA020\0"
  "EGYPTIAN HIEROGLYPH AA021\0"
  "EGYPTIAN HIEROGLYPH AA022\0"
  "EGYPTIAN HIEROGLYPH AA023\0"
  "EGYPTIAN HIEROGLYPH AA024\0"
  "EGYPTIAN HIEROGLYPH AA025\0"
  "EGYPTIAN HIEROGLYPH AA026\0"
  "EGYPTIAN HIEROGLYPH AA027\0"
  "EGYPTIAN HIEROGLYPH AA028\0"
  "EGYPTIAN HIEROGLYPH AA029\0"
  "EGYPTIAN HIEROGLYPH AA030\0"
  "EGYPTIAN HIEROGLYPH AA031\0"
  "EGYPTIAN HIEROGLYPH AA032\0"
  "EGYPTIAN HIEROGLYPH B001\0"
  "EGYPTIAN HIEROGLYPH B002\0"
  "EGYPTIAN HIEROGLYPH B003\0"
  "EGYPTIAN HIEROGLYPH B004\0"
  "EGYPTIAN HIEROGLYPH B005\0"
  "EGYPTIAN HIEROGLYPH B005A\0"
  "EGYPTIAN HIEROGLYPH B006\0"
  "EGYPTIAN HIEROGLYPH B007\0"
  "EGYPTIAN HIEROGLYPH B008\0"
  "EGYPTIAN HIEROGLYPH B009\0"
  "EGYPTIAN HIEROGLYPH BEGIN ENCLOSURE\0"
  "EGYPTIAN HIEROGLYPH BEGIN SEGMENT\0"
  "EGYPTIAN HIEROGLYPH BEGIN WALLED ENCLOSURE\0"
  "EGYPTIAN HIEROGLYPH C001\0"
  "EGYPTIAN HIEROGLYPH C002\0"
  "EGYPTIAN HIEROGLYPH C002A\0"
  "EGYPTIAN HIEROGLYPH C002B\0"
  "EGYPTIAN HIEROGLYPH C002C\0"
  "EGYPTIAN HIEROGLYPH C003\0"
  "EGYPTIAN HIEROGLYPH C004\0"
  "EGYPTIAN HIEROGLYPH C005\0"
  "EGYPTIAN HIEROGLYPH C006\0"
  "EGYPTIAN HIEROGLYPH C007\0"
  "EGYPTIAN HIEROGLYPH C008\0"
  "EGYPTIAN HIEROGLYPH C009\0"
  "EGYPTIAN HIEROGLYPH C010\0"
  "EGYPTIAN HIEROGLYPH C010A\0"
  "EGYPTIAN HIEROGLYPH C011\0"
  "EGYPTIAN HIEROGLYPH C012\0"
  "EGYPTIAN HIEROGLYPH C013\0"
  "EGYPTIAN HIEROGLYPH C014\0"
  "EGYPTIAN HIEROGLYPH C015\0"
  "EGYPTIAN HIEROGLYPH C016\0"
  "EGYPTIAN HIEROGLYPH C017\0"
  "EGYPTIAN HIEROGLYPH C018\0"
  "EGYPTIAN HIEROGLYPH C019\0"
  "EGYPTIAN HIEROGLYPH C020\0"
  "EGYPTIAN HIEROGLYPH C021\0"
  "EGYPTIAN HIEROGLYPH C022\0"
  "EGYPTIAN HIEROGLYPH C023\0"
  "EGYPTIAN HIEROGLYPH C024\0"
  "EGYPTIAN HIEROGLYPH D001\0"
  "EGYPTIAN HIEROGLYPH D002\0"
  "EGYPTIAN HIEROGLYPH D003\0"
  "EGYPTIAN HIEROGLYPH D004\0"
  "EGYPTIAN HIEROGLYPH D005\0"
  "EGYPTIAN HIEROGLYPH D006\0"
  "EGYPTIAN HIEROGLYPH D007\0"
  "EGYPTIAN HIEROGLYPH D008\0"
  "EGYPTIAN HIEROGLYPH D008A\0"
  "EGYPTIAN HIEROGLYPH D009\0"
  "EGYPTIAN HIEROGLYPH D010\0"
  "EGYPTIAN HIEROGLYPH D011\0"
  "EGYPTIAN HIEROGLYPH D012\0"
  "EGYPTIAN HIEROGLYPH D013\0"
  "EGYPTIAN HIEROGLYPH D014\0"
  "EGYPTIAN HIEROGLYPH D015\0"
  "EGYPTIAN HIEROGLYPH D016\0"
  "EGYPTIAN HIEROGLYPH D017\0"
  "EGYPTIAN HIEROGLYPH D018\0"
  "EGYPTIAN HIEROGLYPH D019\0"
  "EGYPTIAN HIEROGLYPH D020\0"
  "EGYPTIAN HIEROGLYPH D021\0"
  "EGYPTIAN HIEROGLYPH D022\0"
  "EGYPTIAN HIEROGLYPH D023\0"
  "EGYPTIAN HIEROGLYPH D024\0"
  "EGYPTIAN HIEROGLYPH D025\0"
  "EGYPTIAN HIEROGLYPH D026\0"
  "EGYPTIAN HIEROGLYPH D027\0"
  "EGYPTIAN HIEROGLYPH D027A\0"
  "EGYPTIAN HIEROGLYPH D028\0"
  "EGYPTIAN HIEROGLYPH D029\0"
  "EGYPTIAN HIEROGLYPH D030\0"
  "EGYPTIAN HIEROGLYPH D031\0"
  "EGYPTIAN HIEROGLYPH D031A\0"
  "EGYPTIAN HIEROGLYPH D032\0"
  "EGYPTIAN HIEROGLYPH D033\0"
  "EGYPTIAN HIEROGLYPH D034\0"
  "EGYPTIAN HIEROGLYPH D034A\0"
  "EGYPTIAN HIEROGLYPH D035\0"
  "EGYPTIAN HIEROGLYPH D036\0"
  "EGYPTIAN HIEROGLYPH D037\0"
  "EGYPTIAN HIEROGLYPH D038\0"
  "EGYPTIAN HIEROGLYPH D039\0"
  "EGYPTIAN HIEROGLYPH D040\0"
  "EGYPTIAN HIEROGLYPH D041\0"
  "EGYPTIAN HIEROGLYPH D042\0"
  "EGYPTIAN HIEROGLYPH D043\0"
  "EGYPTIAN HIEROGLYPH D044\0"
  "EGYPTIAN HIEROGLYPH D045\0"
  "EGYPTIAN HIEROGLYPH D046\0"
  "EGYPTIAN HIEROGLYPH D046A\0"
  "EGYPTIAN HIEROGLYPH D047\0"
  "EGYPTIAN HIEROGLYPH D048\0"
  "EGYPTIAN HIEROGLYPH D048A\0"
  "EGYPTIAN HIEROGLYPH D049\0"
  "EGYPTIAN HIEROGLYPH D050\0"
  "EGYPTIAN HIEROGLYPH D050A\0"
  "EGYPTIAN HIEROGLYPH D050B\0"
  "EGYPTIAN HIEROGLYPH D050C\0"
  "EGYPTIAN HIEROGLYPH D050D\0"
  "EGYPTIAN HIEROGLYPH D050E\0"
  "EGYPTIAN HIEROGLYPH D050F\0"
  "EGYPTIAN HIEROGLYPH D050G\0"
  "EGYPTIAN HIEROGLYPH D050H\0"
  "EGYPTIAN HIEROGLYPH D050I\0"
  "EGYPTIAN HIEROGLYPH D051\0"
  "EGYPTIAN HIEROGLYPH D052\0"
  "EGYPTIAN HIEROGLYPH D052A\0"
  "EGYPTIAN HIEROGLYPH D053\0"
  "EGYPTIAN HIEROGLYPH D054\0"
  "EGYPTIAN HIEROGLYPH D054A\0"
  "EGYPTIAN HIEROGLYPH D055\0"
  "EGYPTIAN HIEROGLYPH D056\0"
  "EGYPTIAN HIEROGLYPH D057\0"
  "EGYPTIAN HIEROGLYPH D058\0"
  "EGYPTIAN HIEROGLYPH D059\0"
  "EGYPTIAN HIEROGLYPH D060\0"
  "EGYPTIAN HIEROGLYPH D061\0"
  "EGYPTIAN HIEROGLYPH D062\0"
  "EGYPTIAN HIEROGLYPH D063\0"
  "EGYPTIAN HIEROGLYPH D064\0"
  "EGYPTIAN HIEROGLYPH D065\0"
  "EGYPTIAN HIEROGLYPH D066\0"
  "EGYPTIAN HIEROGLYPH D067\0"
  "EGYPTIAN HIEROGLYPH D067A\0"
  "EGYPTIAN HIEROGLYPH D067B\0"
  "EGYPTIAN HIEROGLYPH D067C\0"
  "EGYPTIAN HIEROGLYPH D067D\0"
  "EGYPTIAN HIEROGLYPH D067E\0"
  "EGYPTIAN HIEROGLYPH D067F\0"
  "EGYPTIAN HIEROGLYPH D067G\0"
  "EGYPTIAN HIEROGLYPH D067H\0"
  "EGYPTIAN HIEROGLYPH E001\0"
  "EGYPTIAN HIEROGLYPH E002\0"
  "EGYPTIAN HIEROGLYPH E003\0"
  "EGYPTIAN HIEROGLYPH E004\0"
  "EGYPTIAN HIEROGLYPH E005\0"
  "EGYPTIAN HIEROGLYPH E006\0"
  "EGYPTIAN HIEROGLYPH E007\0"
  "EGYPTIAN HIEROGLYPH E008\0"
  "EGYPTIAN HIEROGLYPH E008A\0"
  "EGYPTIAN HIEROGLYPH E009\0"
  "EGYPTIAN HIEROGLYPH E009A\0"
  "EGYPTIAN HIEROGLYPH E010\0"
  "EGYPTIAN HIEROGLYPH E011\0"
  "EGYPTIAN HIEROGLYPH E012\0"
  "EGYPTIAN HIEROGLYPH E013\0"
  "EGYPTIAN HIEROGLYPH E014\0"
  "EGYPTIAN HIEROGLYPH E015\0"
  "EGYPTIAN HIEROGLYPH E016\0"
  "EGYPTIAN HIEROGLYPH E016A\0"
  "EGYPTIAN HIEROGLYPH E017\0"
  "EGYPTIAN HIEROGLYPH E017A\0"
  "EGYPTIAN HIEROGLYPH E018\0"
  "EGYPTIAN HIEROGLYPH E019\0"
  "EGYPTIAN HIEROGLYPH E020\0"
  "EGYPTIAN HIEROGLYPH E020A\0"
  "EGYPTIAN HIEROGLYPH E021\0"
  "EGYPTIAN HIEROGLYPH E022\0"
  "EGYPTIAN HIEROGLYPH E023\0"
  "EGYPTIAN HIEROGLYPH E024\0"
  "EGYPTIAN HIEROGLYPH E025\0"
  "EGYPTIAN HIEROGLYPH E026\0"
  "EGYPTIAN HIEROGLYPH E027\0"
  "EGYPTIAN HIEROGLYPH E028\0"
  "EGYPTIAN HIEROGLYPH E028A\0"
  "EGYPTIAN HIEROGLYPH E029\0"
  "EGYPTIAN HIEROGLYPH E030\0"
  "EGYPTIAN HIEROGLYPH E031\0"
  "EGYPTIAN HIEROGLYPH E032\0"
  "EGYPTIAN HIEROGLYPH E033\0"
  "EGYPTIAN HIEROGLYPH E034\0"
  "EGYPTIAN HIEROGLYPH E034A\0"
  "EGYPTIAN HIEROGLYPH E036\0"
  "EGYPTIAN HIEROGLYPH E037\0"
  "EGYPTIAN HIEROGLYPH E038\0"
  "EGYPTIAN HIEROGLYPH END ENCLOSURE\0"
  "EGYPTIAN HIEROGLYPH END SEGMENT\0"
  "EGYPTIAN HIEROGLYPH END WALLED ENCLOSURE\0"
  "EGYPTIAN HIEROGLYPH F001\0"
  "EGYPTIAN HIEROGLYPH F001A\0"
  "EGYPTIAN HIEROGLYPH F002\0"
  "EGYPTIAN HIEROGLYPH F003\0"
  "EGYPTIAN HIEROGLYPH F004\0"
  "EGYPTIAN HIEROGLYPH F005\0"
  "EGYPTIAN HIEROGLYPH F006\0"
  "EGYPTIAN HIEROGLYPH F007\0"
  "EGYPTIAN HIEROGLYPH F008\0"
  "EGYPTIAN HIEROGLYPH F009\0"
  "EGYPTIAN HIEROGLYPH F010\0"
  "EGYPTIAN HIEROGLYPH F011\0"
  "EGYPTIAN HIEROGLYPH F012\0"
  "EGYPTIAN HIEROGLYPH F013\0"
  "EGYPTIAN HIEROGLYPH F013A\0"
  "EGYPTIAN HIEROGLYPH F014\0"
  "EGYPTIAN HIEROGLYPH F015\0"
  "EGYPTIAN HIEROGLYPH F016\0"
  "EGYPTIAN HIEROGLYPH F017\0"
  "EGYPTIAN HIEROGLYPH F018\0"
  "EGYPTIAN HIEROGLYPH F019\0"
  "EGYPTIAN HIEROGLYPH F020\0"
  "EGYPTIAN HIEROGLYPH F021\0"
  "EGYPTIAN HIEROGLYPH F021A\0"
  "EGYPTIAN HIEROGLYPH F022\0"
  "EGYPTIAN HIEROGLYPH F023\0"
  "EGYPTIAN HIEROGLYPH F024\0"
  "EGYPTIAN HIEROGLYPH F025\0"
  "EGYPTIAN HIEROGLYPH F026\0"
  "EGYPTIAN HIEROGLYPH F027\0"
  "EGYPTIAN HIEROGLYPH F028\0"
  "EGYPTIAN HIEROGLYPH F029\0"
  "EGYPTIAN HIEROGLYPH F030\0"
  "EGYPTIAN HIEROGLYPH F031\0"
  "EGYPTIAN HIEROGLYPH F031A\0"
  "EGYPTIAN HIEROGLYPH F032\0"
  "EGYPTIAN HIEROGLYPH F033\0"
  "EGYPTIAN HIEROGLYPH F034\0"
  "EGYPTIAN HIEROGLYPH F035\0"
  "EGYPTIAN HIEROGLYPH F036\0"
  "EGYPTIAN HIEROGLYPH F037\0"
  "EGYPTIAN HIEROGLYPH F037A\0"
  "EGYPTIAN HIEROGLYPH F038\0"
  "EGYPTIAN HIEROGLYPH F038A\0"
  "EGYPTIAN HIEROGLYPH F039\0"
  "EGYPTIAN HIEROGLYPH F040\0"
  "EGYPTIAN HIEROGLYPH F041\0"
  "EGYPTIAN HIEROGLYPH F042\0"
  "EGYPTIAN HIEROGLYPH F043\0"
  "EGYPTIAN HIEROGLYPH F044\0"
  "EGYPTIAN HIEROGLYPH F045\0"
  "EGYPTIAN HIEROGLYPH F045A\0"
  "EGYPTIAN HIEROGLYPH F046\0"
  "EGYPTIAN HIEROGLYPH F046A\0"
  "EGYPTIAN HIEROGLYPH F047\0"
  "EGYPTIAN HIEROGLYPH F047A\0"
  "EGYPTIAN HIEROGLYPH F048\0"
  "EGYPTIAN HIEROGLYPH F049\0"
  "EGYPTIAN HIEROGLYPH F050\0"
  "EGYPTIAN HIEROGLYPH F051\0"
  "EGYPTIAN HIEROGLYPH F051A\0"
  "EGYPTIAN HIEROGLYPH F051B\0"
  "EGYPTIAN HIEROGLYPH F051C\0"
  "EGYPTIAN HIEROGLYPH F052\0"
  "EGYPTIAN HIEROGLYPH F053\0"
  "EGYPTIAN HIEROGLYPH FULL BLANK\0"
  "EGYPTIAN HIEROGLYPH G001\0"
  "EGYPTIAN HIEROGLYPH G002\0"
  "EGYPTIAN HIEROGLYPH G003\0"
  "EGYPTIAN HIEROGLYPH G004\0"
  "EGYPTIAN HIEROGLYPH G005\0"
  "EGYPTIAN HIEROGLYPH G006\0"
  "EGYPTIAN HIEROGLYPH G006A\0"
  "EGYPTIAN HIEROGLYPH G007\0"
  "EGYPTIAN HIEROGLYPH G007A\0"
  "EGYPTIAN HIEROGLYPH G007B\0"
  "EGYPTIAN HIEROGLYPH G008\0"
  "EGYPTIAN HIEROGLYPH G009\0"
  "EGYPTIAN HIEROGLYPH G010\0"
  "EGYPTIAN HIEROGLYPH G011\0"
  "EGYPTIAN HIEROGLYPH G011A\0"
  "EGYPTIAN HIEROGLYPH G012\0"
  "EGYPTIAN HIEROGLYPH G013\0"
  "EGYPTIAN HIEROGLYPH G014\0"
  "EGYPTIAN HIEROGLYPH G015\0"
  "EGYPTIAN HIEROGLYPH G016\0"
  "EGYPTIAN HIEROGLYPH G017\0"
  "EGYPTIAN HIEROGLYPH G018\0"
  "EGYPTIAN HIEROGLYPH G019\0"
  "EGYPTIAN HIEROGLYPH G020\0"
  "EGYPTIAN HIEROGLYPH G020A\0"
  "EGYPTIAN HIEROGLYPH G021\0"
  "EGYPTIAN HIEROGLYPH G022\0"
  "EGYPTIAN HIEROGLYPH G023\0"
  "EGYPTIAN HIEROGLYPH G024\0"
  "EGYPTIAN HIEROGLYPH G025\0"
  "EGYPTIAN HIEROGLYPH G026\0"
  "EGYPTIAN HIEROGLYPH G026A\0"
  "EGYPTIAN HIEROGLYPH G027\0"
  "EGYPTIAN HIEROGLYPH G028\0"
  "EGYPTIAN HIEROGLYPH G029\0"
  "EGYPTIAN HIEROGLYPH G030\0"
  "EGYPTIAN HIEROGLYPH G031\0"
  "EGYPTIAN HIEROGLYPH G032\0"
  "EGYPTIAN HIEROGLYPH G033\0"
  "EGYPTIAN HIEROGLYPH G034\0"
  "EGYPTIAN HIEROGLYPH G035\0"
  "EGYPTIAN HIEROGLYPH G036\0"
  "EGYPTIAN HIEROGLYPH G036A\0"
  "EGYPTIAN HIEROGLYPH G037\0"
  "EGYPTIAN HIEROGLYPH G037A\0"
  "EGYPTIAN HIEROGLYPH G038\0"
  "EGYPTIAN HIEROGLYPH G039\0"
  "EGYPTIAN HIEROGLYPH G040\0"
  "EGYPTIAN HIEROGLYPH G041\0"
  "EGYPTIAN HIEROGLYPH G042\0"
  "EGYPTIAN HIEROGLYPH G043\0"
  "EGYPTIAN HIEROGLYPH G043A\0"
  "EGYPTIAN HIEROGLYPH G044\0"
  "EGYPTIAN HIEROGLYPH G045\0"
  "EGYPTIAN HIEROGLYPH G045A\0"
  "EGYPTIAN HIEROGLYPH G046\0"
  "EGYPTIAN HIEROGLYPH G047\0"
  "EGYPTIAN HIEROGLYPH G048\0"
  "EGYPTIAN HIEROGLYPH G049\0"
  "EGYPTIAN HIEROGLYPH G050\0"
  "EGYPTIAN HIEROGLYPH G051\0"
  "EGYPTIAN HIEROGLYPH G052\0"
  "EGYPTIAN HIEROGLYPH G053\0"
  "EGYPTIAN HIEROGLYPH G054\0"
  "EGYPTIAN HIEROGLYPH H001\0"
  "EGYPTIAN HIEROGLYPH H002\0"
  "EGYPTIAN HIEROGLYPH H003\0"
  "EGYPTIAN HIEROGLYPH H004\0"
  "EGYPTIAN HIEROGLYPH H005\0"
  "EGYPTIAN HIEROGLYPH H006\0"
  "EGYPTIAN HIEROGLYPH H006A\0"
  "EGYPTIAN HIEROGLYPH H007\0"
  "EGYPTIAN HIEROGLYPH H008\0"
  "EGYPTIAN HIEROGLYPH HALF BLANK\0"
  "EGYPTIAN HIEROGLYPH HALF LOST SIGN\0"
  "EGYPTIAN HIEROGLYPH HORIZONTAL JOINER\0"
  "EGYPTIAN HIEROGLYPH I001\0"
  "EGYPTIAN HIEROGLYPH I002\0"
  "EGYPTIAN HIEROGLYPH I003\0"
  "EGYPTIAN HIEROGLYPH I004\0"
  "EGYPTIAN HIEROGLYPH I005\0"
  "EGYPTIAN HIEROGLYPH I005A\0"
  "EGYPTIAN HIEROGLYPH I006\0"
  "EGYPTIAN HIEROGLYPH I007\0"
  "EGYPTIAN HIEROGLYPH I008\0"
  "EGYPTIAN HIEROGLYPH I009\0"
  "EGYPTIAN HIEROGLYPH I009A\0"
  "EGYPTIAN HIEROGLYPH I010\0"
  "EGYPTIAN HIEROGLYPH I010A\0"
  "EGYPTIAN HIEROGLYPH I011\0"
  "EGYPTIAN HIEROGLYPH I011A\0"
  "EGYPTIAN HIEROGLYPH I012\0"
  "EGYPTIAN HIEROGLYPH I013\0"
  "EGYPTIAN HIEROGLYPH I014\0"
  "EGYPTIAN HIEROGLYPH I015\0"
  "EGYPTIAN HIEROGLYPH INSERT AT BOTTOM\0"
  "EGYPTIAN HIEROGLYPH INSERT AT BOTTOM END\0"
  "EGYPTIAN HIEROGLYPH INSERT AT BOTTOM START\0"
  "EGYPTIAN HIEROGLYPH INSERT AT MIDDLE\0"
  "EGYPTIAN HIEROGLYPH INSERT AT TOP\0"
  "EGYPTIAN HIEROGLYPH INSERT AT TOP END\0"
  "EGYPTIAN HIEROGLYPH INSERT AT TOP START\0"
  "EGYPTIAN HIEROGLYPH K001\0"
  "EGYPTIAN HIEROGLYPH K002\0"
  "EGYPTIAN HIEROGLYPH K003\0"
  "EGYPTIAN HIEROGLYPH K004\0"
  "EGYPTIAN HIEROGLYPH K005\0"
  "EGYPTIAN HIEROGLYPH K006\0"
  "EGYPTIAN HIEROGLYPH K007\0"
  "EGYPTIAN HIEROGLYPH K008\0"
  "EGYPTIAN HIEROGLYPH L001\0"
  "EGYPTIAN HIEROGLYPH L002\0"
  "EGYPTIAN HIEROGLYPH L002A\0"
  "EGYPTIAN HIEROGLYPH L003\0"
  "EGYPTIAN HIEROGLYPH L004\0"
  "EGYPTIAN HIEROGLYPH L005\0"
  "EGYPTIAN HIEROGLYPH L006\0"
  "EGYPTIAN HIEROGLYPH L006A\0"
  "EGYPTIAN HIEROGLYPH L007\0"
  "EGYPTIAN HIEROGLYPH L008\0"
  "EGYPTIAN HIEROGLYPH LOST SIGN\0"
  "EGYPTIAN HIEROGLYPH M001\0"
  "EGYPTIAN HIEROGLYPH M001A\0"
  "EGYPTIAN HIEROGLYPH M001B\0"
  "EGYPTIAN HIEROGLYPH M002\0"
  "EGYPTIAN HIEROGLYPH M003\0"
  "EGYPTIAN HIEROGLYPH M003A\0"
  "EGYPTIAN HIEROGLYPH M004\0"
  "EGYPTIAN HIEROGLYPH M005\0"
  "EGYPTIAN HIEROGLYPH M006\0"
  "EGYPTIAN HIEROGLYPH M007\0"
  "EGYPTIAN HIEROGLYPH M008\0"
  "EGYPTIAN HIEROGLYPH M009\0"
  "EGYPTIAN HIEROGLYPH M010\0"
  "EGYPTIAN HIEROGLYPH M010A\0"
  "EGYPTIAN HIEROGLYPH M011\0"
  "EGYPTIAN HIEROGLYPH M012\0"
  "EGYPTIAN HIEROGLYPH M012A\0"
  "EGYPTIAN HIEROGLYPH M012B\0"
  "EGYPTIAN HIEROGLYPH M012C\0"
  "EGYPTIAN HIEROGLYPH M012D\0"
  "EGYPTIAN HIEROGLYPH M012E\0"
  "EGYPTIAN HIEROGLYPH M012F\0"
  "EGYPTIAN HIEROGLYPH M012G\0"
  "EGYPTIAN HIEROGLYPH M012H\0"
  "EGYPTIAN HIEROGLYPH M013\0"
  "EGYPTIAN HIEROGLYPH M014\0"
  "EGYPTIAN HIEROGLYPH M015\0"
  "EGYPTIAN HIEROGLYPH M015A\0"
  "EGYPTIAN HIEROGLYPH M016\0"
  "EGYPTIAN HIEROGLYPH M016A\0"
  "EGYPTIAN HIEROGLYPH M017\0"
  "EGYPTIAN HIEROGLYPH M017A\0"
  "EGYPTIAN HIEROGLYPH M018\0"
  "EGYPTIAN HIEROGLYPH M019\0"
  "EGYPTIAN HIEROGLYPH M020\0"
  "EGYPTIAN HIEROGLYPH M021\0"
  "EGYPTIAN HIEROGLYPH M022\0"
  "EGYPTIAN HIEROGLYPH M022A\0"
  "EGYPTIAN HIEROGLYPH M023\0"
  "EGYPTIAN HIEROGLYPH M024\0"
  "EGYPTIAN HIEROGLYPH M024A\0"
  "EGYPTIAN HIEROGLYPH M025\0"
  "EGYPTIAN HIEROGLYPH M026\0"
  "EGYPTIAN HIEROGLYPH M027\0"
  "EGYPTIAN HIEROGLYPH M028\0"
  "EGYPTIAN HIEROGLYPH M028A\0"
  "EGYPTIAN HIEROGLYPH M029\0"
  "EGYPTIAN HIEROGLYPH M030\0"
  "EGYPTIAN HIEROGLYPH M031\0"
  "EGYPTIAN HIEROGLYPH M031A\0"
  "EGYPTIAN HIEROGLYPH M032\0"
  "EGYPTIAN HIEROGLYPH M033\0"
  "EGYPTIAN HIEROGLYPH M033A\0"
  "EGYPTIAN HIEROGLYPH M033B\0"
  "EGYPTIAN HIEROGLYPH M034\0"
  "EGYPTIAN HIEROGLYPH M035\0"
  "EGYPTIAN HIEROGLYPH M036\0"
  "EGYPTIAN HIEROGLYPH M037\0"
  "EGYPTIAN HIEROGLYPH M038\0"
  "EGYPTIAN HIEROGLYPH M039\0"
  "EGYPTIAN HIEROGLYPH M040\0"
  "EGYPTIAN HIEROGLYPH M040A\0"
  "EGYPTIAN HIEROGLYPH M041\0"
  "EGYPTIAN HIEROGLYPH M042\0"
  "EGYPTIAN HIEROGLYPH M043\0"
  "EGYPTIAN HIEROGLYPH M044\0"
  "EGYPTIAN HIEROGLYPH MIRROR HORIZONTALLY\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM AND END\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM END\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM START\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM START AND TOP END\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT END\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT START\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT START AND BOTTOM\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT START AND TOP\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP AND END\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP END\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP START\0"
  "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP START AND BOTTOM END\0"
  "EGYPTIAN HIEROGLYPH N001\0"
  "EGYPTIAN HIEROGLYPH N002\0"
  "EGYPTIAN HIEROGLYPH N003\0"
  "EGYPTIAN HIEROGLYPH N004\0"
  "EGYPTIAN HIEROGLYPH N005\0"
  "EGYPTIAN HIEROGLYPH N006\0"
  "EGYPTIAN HIEROGLYPH N007\0"
  "EGYPTIAN HIEROGLYPH N008\0"
  "EGYPTIAN HIEROGLYPH N009\0"
  "EGYPTIAN HIEROGLYPH N010\0"
  "EGYPTIAN HIEROGLYPH N011\0"
  "EGYPTIAN HIEROGLYPH N012\0"
  "EGYPTIAN HIEROGLYPH N013\0"
  "EGYPTIAN HIEROGLYPH N014\0"
  "EGYPTIAN HIEROGLYPH N015\0"
  "EGYPTIAN HIEROGLYPH N016\0"
  "EGYPTIAN HIEROGLYPH N017\0"
  "EGYPTIAN HIEROGLYPH N018\0"
  "EGYPTIAN HIEROGLYPH N018A\0"
  "EGYPTIAN HIEROGLYPH N018B\0"
  "EGYPTIAN HIEROGLYPH N019\0"
  "EGYPTIAN HIEROGLYPH N020\0"
  "EGYPTIAN HIEROGLYPH N021\0"
  "EGYPTIAN HIEROGLYPH N022\0"
  "EGYPTIAN HIEROGLYPH N023\0"
  "EGYPTIAN HIEROGLYPH N024\0"
  "EGYPTIAN HIEROGLYPH N025\0"
  "EGYPTIAN HIEROGLYPH N025A\0"
  "EGYPTIAN HIEROGLYPH N026\0"
  "EGYPTIAN HIEROGLYPH N027\0"
  "EGYPTIAN HIEROGLYPH N028\0"
  "EGYPTIAN HIEROGLYPH N029\0"
  "EGYPTIAN HIEROGLYPH N030\0"
  "EGYPTIAN HIEROGLYPH N031\0"
  "EGYPTIAN HIEROGLYPH N032\0"
  "EGYPTIAN HIEROGLYPH N033\0"
  "EGYPTIAN HIEROGLYPH N033A\0"
  "EGYPTIAN HIEROGLYPH N034\0"
  "EGYPTIAN HIEROGLYPH N034A\0"
  "EGYPTIAN HIEROGLYPH N035\0"
  "EGYPTIAN HIEROGLYPH N035A\0"
  "EGYPTIAN HIEROGLYPH N036\0"
  "EGYPTIAN HIEROGLYPH N037\0"
  "EGYPTIAN HIEROGLYPH N037A\0"
  "EGYPTIAN HIEROGLYPH N038\0"
  "EGYPTIAN HIEROGLYPH N039\0"
  "EGYPTIAN HIEROGLYPH N040\0"
  "EGYPTIAN HIEROGLYPH N041\0"
  "EGYPTIAN HIEROGLYPH N042\0"
  "EGYPTIAN HIEROGLYPH NL001\0"
  "EGYPTIAN HIEROGLYPH NL002\0"
  "EGYPTIAN HIEROGLYPH NL003\0"
  "EGYPTIAN HIEROGLYPH NL004\0"
  "EGYPTIAN HIEROGLYPH NL005\0"
  "EGYPTIAN HIEROGLYPH NL005A\0"
  "EGYPTIAN HIEROGLYPH NL006\0"
  "EGYPTIAN HIEROGLYPH NL007\0"
  "EGYPTIAN HIEROGLYPH NL008\0"
  "EGYPTIAN HIEROGLYPH NL009\0"
  "EGYPTIAN HIEROGLYPH NL010\0"
  "EGYPTIAN HIEROGLYPH NL011\0"
  "EGYPTIAN HIEROGLYPH NL012\0"
  "EGYPTIAN HIEROGLYPH NL013\0"
  "EGYPTIAN HIEROGLYPH NL014\0"
  "EGYPTIAN HIEROGLYPH NL015\0"
  "EGYPTIAN HIEROGLYPH NL016\0"
  "EGYPTIAN HIEROGLYPH NL017\0"
  "EGYPTIAN HIEROGLYPH NL017A\0"
  "EGYPTIAN HIEROGLYPH NL018\0"
  "EGYPTIAN HIEROGLYPH NL019\0"
  "EGYPTIAN HIEROGLYPH NL020\0"
  "EGYPTIAN HIEROGLYPH NU001\0"
  "EGYPTIAN HIEROGLYPH NU002\0"
  "EGYPTIAN HIEROGLYPH NU003\0"
  "EGYPTIAN HIEROGLYPH NU004\0"
  "EGYPTIAN HIEROGLYPH NU005\0"
  "EGYPTIAN HIEROGLYPH NU006\0"
  "EGYPTIAN HIEROGLYPH NU007\0"
  "EGYPTIAN HIEROGLYPH NU008\0"
  "EGYPTIAN HIEROGLYPH NU009\0"
  "EGYPTIAN HIEROGLYPH NU010\0"
  "EGYPTIAN HIEROGLYPH NU010A\0"
  "EGYPTIAN HIEROGLYPH NU011\0"
  "EGYPTIAN HIEROGLYPH NU011A\0"
  "EGYPTIAN HIEROGLYPH NU012\0"
  "EGYPTIAN HIEROGLYPH NU013\0"
  "EGYPTIAN HIEROGLYPH NU014\0"
  "EGYPTIAN HIEROGLYPH NU015\0"
  "EGYPTIAN HIEROGLYPH NU016\0"
  "EGYPTIAN HIEROGLYPH NU017\0"
  "EGYPTIAN HIEROGLYPH NU018\0"
  "EGYPTIAN HIEROGLYPH NU018A\0"
  "EGYPTIAN HIEROGLYPH NU019\0"
  "EGYPTIAN HIEROGLYPH NU020\0"
  "EGYPTIAN HIEROGLYPH NU021\0"
  "EGYPTIAN HIEROGLYPH NU022\0"
  "EGYPTIAN HIEROGLYPH NU022A\0"
  "EGYPTIAN HIEROGLYPH O001\0"
  "EGYPTIAN HIEROGLYPH O001A\0"
  "EGYPTIAN HIEROGLYPH O002\0"
  "EGYPTIAN HIEROGLYPH O003\0"
  "EGYPTIAN HIEROGLYPH O004\0"
  "EGYPTIAN HIEROGLYPH O005\0"
  "EGYPTIAN HIEROGLYPH O005A\0"
  "EGYPTIAN HIEROGLYPH O006\0"
  "EGYPTIAN HIEROGLYPH O006A\0"
  "EGYPTIAN HIEROGLYPH O006B\0"
  "EGYPTIAN HIEROGLYPH O006C\0"
  "EGYPTIAN HIEROGLYPH O006D\0"
  "EGYPTIAN HIEROGLYPH O006E\0"
  "EGYPTIAN HIEROGLYPH O006F\0"
  "EGYPTIAN HIEROGLYPH O007\0"
  "EGYPTIAN HIEROGLYPH O008\0"
  "EGYPTIAN HIEROGLYPH O009\0"
  "EGYPTIAN HIEROGLYPH O010\0"
  "EGYPTIAN HIEROGLYPH O010A\0"
  "EGYPTIAN HIEROGLYPH O010B\0"
  "EGYPTIAN HIEROGLYPH O010C\0"
  "EGYPTIAN HIEROGLYPH O011\0"
  "EGYPTIAN HIEROGLYPH O012\0"
  "EGYPTIAN HIEROGLYPH O013\0"
  "EGYPTIAN HIEROGLYPH O014\0"
  "EGYPTIAN HIEROGLYPH O015\0"
  "EGYPTIAN HIEROGLYPH O016\0"
  "EGYPTIAN HIEROGLYPH O017\0"
  "EGYPTIAN HIEROGLYPH O018\0"
  "EGYPTIAN HIEROGLYPH O019\0"
  "EGYPTIAN HIEROGLYPH O019A\0"
  "EGYPTIAN HIEROGLYPH O020\0"
  "EGYPTIAN HIEROGLYPH O020A\0"
  "EGYPTIAN HIEROGLYPH O021\0"
  "EGYPTIAN HIEROGLYPH O022\0"
  "EGYPTIAN HIEROGLYPH O023\0"
  "EGYPTIAN HIEROGLYPH O024\0"
  "EGYPTIAN HIEROGLYPH O024A\0"
  "EGYPTIAN HIEROGLYPH O025\0"
  "EGYPTIAN HIEROGLYPH O025A\0"
  "EGYPTIAN HIEROGLYPH O026\0"
  "EGYPTIAN HIEROGLYPH O027\0"
  "EGYPTIAN HIEROGLYPH O028\0"
  "EGYPTIAN HIEROGLYPH O029\0"
  "EGYPTIAN HIEROGLYPH O029A\0"
  "EGYPTIAN HIEROGLYPH O030\0"
  "EGYPTIAN HIEROGLYPH O030A\0"
  "EGYPTIAN HIEROGLYPH O031\0"
  "EGYPTIAN HIEROGLYPH O032\0"
  "EGYPTIAN HIEROGLYPH O033\0"
  "EGYPTIAN HIEROGLYPH O033A\0"
  "EGYPTIAN HIEROGLYPH O034\0"
  "EGYPTIAN HIEROGLYPH O035\0"
  "EGYPTIAN HIEROGLYPH O036\0"
  "EGYPTIAN HIEROGLYPH O036A\0"
  "EGYPTIAN HIEROGLYPH O036B\0"
  "EGYPTIAN HIEROGLYPH O036C\0"
  "EGYPTIAN HIEROGLYPH O036D\0"
  "EGYPTIAN HIEROGLYPH O037\0"
  "EGYPTIAN HIEROGLYPH O038\0"
  "EGYPTIAN HIEROGLYPH O039\0"
  "EGYPTIAN HIEROGLYPH O040\0"
  "EGYPTIAN HIEROGLYPH O041\0"
  "EGYPTIAN HIEROGLYPH O042\0"
  "EGYPTIAN HIEROGLYPH O043\0"
  "EGYPTIAN HIEROGLYPH O044\0"
  "EGYPTIAN HIEROGLYPH O045\0"
  "EGYPTIAN HIEROGLYPH O046\0"
  "EGYPTIAN HIEROGLYPH O047\0"
  "EGYPTIAN HIEROGLYPH O048\0"
  "EGYPTIAN HIEROGLYPH O049\0"
  "EGYPTIAN HIEROGLYPH O050\0"
  "EGYPTIAN HIEROGLYPH O050A\0"
  "EGYPTIAN HIEROGLYPH O050B\0"
  "EGYPTIAN HIEROGLYPH O051\0"
  "EGYPTIAN HIEROGLYPH OVERLAY MIDDLE\0"
  "EGYPTIAN HIEROGLYPH P001\0"
  "EGYPTIAN HIEROGLYPH P001A\0"
  "EGYPTIAN HIEROGLYPH P002\0"
  "EGYPTIAN HIEROGLYPH P003\0"
  "EGYPTIAN HIEROGLYPH P003A\0"
  "EGYPTIAN HIEROGLYPH P004\0"
  "EGYPTIAN HIEROGLYPH P005\0"
  "EGYPTIAN HIEROGLYPH P006\0"
  "EGYPTIAN HIEROGLYPH P007\0"
  "EGYPTIAN HIEROGLYPH P008\0"
  "EGYPTIAN HIEROGLYPH P009\0"
  "EGYPTIAN HIEROGLYPH P010\0"
  "EGYPTIAN HIEROGLYPH P011\0"
  "EGYPTIAN HIEROGLYPH Q001\0"
  "EGYPTIAN HIEROGLYPH Q002\0"
  "EGYPTIAN HIEROGLYPH Q003\0"
  "EGYPTIAN HIEROGLYPH Q004\0"
  "EGYPTIAN HIEROGLYPH Q005\0"
  "EGYPTIAN HIEROGLYPH Q006\0"
  "EGYPTIAN HIEROGLYPH Q007\0"
  "EGYPTIAN HIEROGLYPH R001\0"
  "EGYPTIAN HIEROGLYPH R002\0"
  "EGYPTIAN HIEROGLYPH R002A\0"
  "EGYPTIAN HIEROGLYPH R003\0"
  "EGYPTIAN HIEROGLYPH R003A\0"
  "EGYPTIAN HIEROGLYPH R003B\0"
  "EGYPTIAN HIEROGLYPH R004\0"
  "EGYPTIAN HIEROGLYPH R005\0"
  "EGYPTIAN HIEROGLYPH R006\0"
  "EGYPTIAN HIEROGLYPH R007\0"
  "EGYPTIAN HIEROGLYPH R008\0"
  "EGYPTIAN HIEROGLYPH R009\0"
  "EGYPTIAN HIEROGLYPH R010\0"
  "EGYPTIAN HIEROGLYPH R010A\0"
  "EGYPTIAN HIEROGLYPH R011\0"
  "EGYPTIAN HIEROGLYPH R012\0"
  "EGYPTIAN HIEROGLYPH R013\0"
  "EGYPTIAN HIEROGLYPH R014\0"
  "EGYPTIAN HIEROGLYPH R015\0"
  "EGYPTIAN HIEROGLYPH R016\0"
  "EGYPTIAN HIEROGLYPH R016A\0"
  "EGYPTIAN HIEROGLYPH R017\0"
  "EGYPTIAN HIEROGLYPH R018\0"
  "EGYPTIAN HIEROGLYPH R019\0"
  "EGYPTIAN HIEROGLYPH R020\0"
  "EGYPTIAN HIEROGLYPH R021\0"
  "EGYPTIAN HIEROGLYPH R022\0"
  "EGYPTIAN HIEROGLYPH R023\0"
  "EGYPTIAN HIEROGLYPH R024\0"
  "EGYPTIAN HIEROGLYPH R025\0"
  "EGYPTIAN HIEROGLYPH R026\0"
  "EGYPTIAN HIEROGLYPH R027\0"
  "EGYPTIAN HIEROGLYPH R028\0"
  "EGYPTIAN HIEROGLYPH R029\0"
  "EGYPTIAN HIEROGLYPH S001\0"
  "EGYPTIAN HIEROGLYPH S002\0"
  "EGYPTIAN HIEROGLYPH S002A\0"
  "EGYPTIAN HIEROGLYPH S003\0"
  "EGYPTIAN HIEROGLYPH S004\0"
  "EGYPTIAN HIEROGLYPH S005\0"
  "EGYPTIAN HIEROGLYPH S006\0"
  "EGYPTIAN HIEROGLYPH S006A\0"
  "EGYPTIAN HIEROGLYPH S007\0"
  "EGYPTIAN HIEROGLYPH S008\0"
  "EGYPTIAN HIEROGLYPH S009\0"
  "EGYPTIAN HIEROGLYPH S010\0"
  "EGYPTIAN HIEROGLYPH S011\0"
  "EGYPTIAN HIEROGLYPH S012\0"
  "EGYPTIAN HIEROGLYPH S013\0"
  "EGYPTIAN HIEROGLYPH S014\0"
  "EGYPTIAN HIEROGLYPH S014A\0"
  "EGYPTIAN HIEROGLYPH S014B\0"
  "EGYPTIAN HIEROGLYPH S015\0"
  "EGYPTIAN HIEROGLYPH S016\0"
  "EGYPTIAN HIEROGLYPH S017\0"
  "EGYPTIAN HIEROGLYPH S017A\0"
  "EGYPTIAN HIEROGLYPH S018\0"
  "EGYPTIAN HIEROGLYPH S019\0"
  "EGYPTIAN HIEROGLYPH S020\0"
  "EGYPTIAN HIEROGLYPH S021\0"
  "EGYPTIAN HIEROGLYPH S022\0"
  "EGYPTIAN HIEROGLYPH S023\0"
  "EGYPTIAN HIEROGLYPH S024\0"
  "EGYPTIAN HIEROGLYPH S025\0"
  "EGYPTIAN HIEROGLYPH S026\0"
  "EGYPTIAN HIEROGLYPH S026A\0"
  "EGYPTIAN HIEROGLYPH S026B\0"
  "EGYPTIAN HIEROGLYPH S027\0"
  "EGYPTIAN HIEROGLYPH S028\0"
  "EGYPTIAN HIEROGLYPH S029\0"
  "EGYPTIAN HIEROGLYPH S030\0"
  "EGYPTIAN HIEROGLYPH S031\0"
  "EGYPTIAN HIEROGLYPH S032\0"
  "EGYPTIAN HIEROGLYPH S033\0"
  "EGYPTIAN HIEROGLYPH S034\0"
  "EGYPTIAN HIEROGLYPH S035\0"
  "EGYPTIAN HIEROGLYPH S035A\0"
  "EGYPTIAN HIEROGLYPH S036\0"
  "EGYPTIAN HIEROGLYPH S037\0"
  "EGYPTIAN HIEROGLYPH S038\0"
  "EGYPTIAN HIEROGLYPH S039\0"
  "EGYPTIAN HIEROGLYPH S040\0"
  "EGYPTIAN HIEROGLYPH S041\0"
  "EGYPTIAN HIEROGLYPH S042\0"
  "EGYPTIAN HIEROGLYPH S043\0"
  "EGYPTIAN HIEROGLYPH S044\0"
  "EGYPTIAN HIEROGLYPH S045\0"
  "EGYPTIAN HIEROGLYPH S046\0"
  "EGYPTIAN HIEROGLYPH T001\0"
  "EGYPTIAN HIEROGLYPH T002\0"
  "EGYPTIAN HIEROGLYPH T003\0"
  "EGYPTIAN HIEROGLYPH T003A\0"
  "EGYPTIAN HIEROGLYPH T004\0"
  "EGYPTIAN HIEROGLYPH T005\0"
  "EGYPTIAN HIEROGLYPH T006\0"
  "EGYPTIAN HIEROGLYPH T007\0"
  "EGYPTIAN HIEROGLYPH T007A\0"
  "EGYPTIAN HIEROGLYPH T008\0"
  "EGYPTIAN HIEROGLYPH T008A\0"
  "EGYPTIAN HIEROGLYPH T009\0"
  "EGYPTIAN HIEROGLYPH T009A\0"
  "EGYPTIAN HIEROGLYPH T010\0"
  "EGYPTIAN HIEROGLYPH T011\0"
  "EGYPTIAN HIEROGLYPH T011A\0"
  "EGYPTIAN HIEROGLYPH T012\0"
  "EGYPTIAN HIEROGLYPH T013\0"
  "EGYPTIAN HIEROGLYPH T014\0"
  "EGYPTIAN HIEROGLYPH T015\0"
  "EGYPTIAN HIEROGLYPH T016\0"
  "EGYPTIAN HIEROGLYPH T016A\0"
  "EGYPTIAN HIEROGLYPH T017\0"
  "EGYPTIAN HIEROGLYPH T018\0"
  "EGYPTIAN HIEROGLYPH T019\0"
  "EGYPTIAN HIEROGLYPH T020\0"
  "EGYPTIAN HIEROGLYPH T021\0"
  "EGYPTIAN HIEROGLYPH T022\0"
  "EGYPTIAN HIEROGLYPH T023\0"
  "EGYPTIAN HIEROGLYPH T024\0"
  "EGYPTIAN HIEROGLYPH T025\0"
  "EGYPTIAN HIEROGLYPH T026\0"
  "EGYPTIAN HIEROGLYPH T027\0"
  "EGYPTIAN HIEROGLYPH T028\0"
  "EGYPTIAN HIEROGLYPH T029\0"
  "EGYPTIAN HIEROGLYPH T030\0"
  "EGYPTIAN HIEROGLYPH T031\0"
  "EGYPTIAN HIEROGLYPH T032\0"
  "EGYPTIAN HIEROGLYPH T032A\0"
  "EGYPTIAN HIEROGLYPH T033\0"
  "EGYPTIAN HIEROGLYPH T033A\0"
  "EGYPTIAN HIEROGLYPH T034\0"
  "EGYPTIAN HIEROGLYPH T035\0"
  "EGYPTIAN HIEROGLYPH T036\0"
  "EGYPTIAN HIEROGLYPH TALL LOST SIGN\0"
  "EGYPTIAN HIEROGLYPH U001\0"
  "EGYPTIAN HIEROGLYPH U002\0"
  "EGYPTIAN HIEROGLYPH U003\0"
  "EGYPTIAN HIEROGLYPH U004\0"
  "EGYPTIAN HIEROGLYPH U005\0"
  "EGYPTIAN HIEROGLYPH U006\0"
  "EGYPTIAN HIEROGLYPH U006A\0"
  "EGYPTIAN HIEROGLYPH U006B\0"
  "EGYPTIAN HIEROGLYPH U007\0"
  "EGYPTIAN HIEROGLYPH U008\0"
  "EGYPTIAN HIEROGLYPH U009\0"
  "EGYPTIAN HIEROGLYPH U010\0"
  "EGYPTIAN HIEROGLYPH U011\0"
  "EGYPTIAN HIEROGLYPH U012\0"
  "EGYPTIAN HIEROGLYPH U013\0"
  "EGYPTIAN HIEROGLYPH U014\0"
  "EGYPTIAN HIEROGLYPH U015\0"
  "EGYPTIAN HIEROGLYPH U016\0"
  "EGYPTIAN HIEROGLYPH U017\0"
  "EGYPTIAN HIEROGLYPH U018\0"
  "EGYPTIAN HIEROGLYPH U019\0"
  "EGYPTIAN HIEROGLYPH U020\0"
  "EGYPTIAN HIEROGLYPH U021\0"
  "EGYPTIAN HIEROGLYPH U022\0"
  "EGYPTIAN HIEROGLYPH U023\0"
  "EGYPTIAN HIEROGLYPH U023A\0"
  "EGYPTIAN HIEROGLYPH U024\0"
  "EGYPTIAN HIEROGLYPH U025\0"
  "EGYPTIAN HIEROGLYPH U026\0"
  "EGYPTIAN HIEROGLYPH U027\0"
  "EGYPTIAN HIEROGLYPH U028\0"
  "EGYPTIAN HIEROGLYPH U029\0"
  "EGYPTIAN HIEROGLYPH U029A\0"
  "EGYPTIAN HIEROGLYPH U030\0"
  "EGYPTIAN HIEROGLYPH U031\0"
  "EGYPTIAN HIEROGLYPH U032\0"
  "EGYPTIAN HIEROGLYPH U032A\0"
  "EGYPTIAN HIEROGLYPH U033\0"
  "EGYPTIAN HIEROGLYPH U034\0"
  "EGYPTIAN HIEROGLYPH U035\0"
  "EGYPTIAN HIEROGLYPH U036\0"
  "EGYPTIAN HIEROGLYPH U037\0"
  "EGYPTIAN HIEROGLYPH U038\0"
  "EGYPTIAN HIEROGLYPH U039\0"
  "EGYPTIAN HIEROGLYPH U040\0"
  "EGYPTIAN HIEROGLYPH U041\0"
  "EGYPTIAN HIEROGLYPH U042\0"
  "EGYPTIAN HIEROGLYPH V001\0"
  "EGYPTIAN HIEROGLYPH V001A\0"
  "EGYPTIAN HIEROGLYPH V001B\0"
  "EGYPTIAN HIEROGLYPH V001C\0"
  "EGYPTIAN HIEROGLYPH V001D\0"
  "EGYPTIAN HIEROGLYPH V001E\0"
  "EGYPTIAN HIEROGLYPH V001F\0"
  "EGYPTIAN HIEROGLYPH V001G\0"
  "EGYPTIAN HIEROGLYPH V001H\0"
  "EGYPTIAN HIEROGLYPH V001I\0"
  "EGYPTIAN HIEROGLYPH V002\0"
  "EGYPTIAN HIEROGLYPH V002A\0"
  "EGYPTIAN HIEROGLYPH V003\0"
  "EGYPTIAN HIEROGLYPH V004\0"
  "EGYPTIAN HIEROGLYPH V005\0"
  "EGYPTIAN HIEROGLYPH V006\0"
  "EGYPTIAN HIEROGLYPH V007\0"
  "EGYPTIAN HIEROGLYPH V007A\0"
  "EGYPTIAN HIEROGLYPH V007B\0"
  "EGYPTIAN HIEROGLYPH V008\0"
  "EGYPTIAN HIEROGLYPH V009\0"
  "EGYPTIAN HIEROGLYPH V010\0"
  "EGYPTIAN HIEROGLYPH V011\0"
  "EGYPTIAN HIEROGLYPH V011A\0"
  "EGYPTIAN HIEROGLYPH V011B\0"
  "EGYPTIAN HIEROGLYPH V011C\0"
  "EGYPTIAN HIEROGLYPH V011D\0"
  "EGYPTIAN HIEROGLYPH V012\0"
  "EGYPTIAN HIEROGLYPH V012A\0"
  "EGYPTIAN HIEROGLYPH V012B\0"
  "EGYPTIAN HIEROGLYPH V013\0"
  "EGYPTIAN HIEROGLYPH V014\0"
  "EGYPTIAN HIEROGLYPH V015\0"
  "EGYPTIAN HIEROGLYPH V016\0"
  "EGYPTIAN HIEROGLYPH V017\0"
  "EGYPTIAN HIEROGLYPH V018\0"
  "EGYPTIAN HIEROGLYPH V019\0"
  "EGYPTIAN HIEROGLYPH V020\0"
  "EGYPTIAN HIEROGLYPH V020A\0"
  "EGYPTIAN HIEROGLYPH V020B\0"
  "EGYPTIAN HIEROGLYPH V020C\0"
  "EGYPTIAN HIEROGLYPH V020D\0"
  "EGYPTIAN HIEROGLYPH V020E\0"
  "EGYPTIAN HIEROGLYPH V020F\0"
  "EGYPTIAN HIEROGLYPH V020G\0"
  "EGYPTIAN HIEROGLYPH V020H\0"
  "EGYPTIAN HIEROGLYPH V020I\0"
  "EGYPTIAN HIEROGLYPH V020J\0"
  "EGYPTIAN HIEROGLYPH V020K\0"
  "EGYPTIAN HIEROGLYPH V020L\0"
  "EGYPTIAN HIEROGLYPH V021\0"
  "EGYPTIAN HIEROGLYPH V022\0"
  "EGYPTIAN HIEROGLYPH V023\0"
  "EGYPTIAN HIEROGLYPH V023A\0"
  "EGYPTIAN HIEROGLYPH V024\0"
  "EGYPTIAN HIEROGLYPH V025\0"
  "EGYPTIAN HIEROGLYPH V026\0"
  "EGYPTIAN HIEROGLYPH V027\0"
  "EGYPTIAN HIEROGLYPH V028\0"
  "EGYPTIAN HIEROGLYPH V028A\0"
  "EGYPTIAN HIEROGLYPH V029\0"
  "EGYPTIAN HIEROGLYPH V029A\0"
  "EGYPTIAN HIEROGLYPH V030\0"
  "EGYPTIAN HIEROGLYPH V030A\0"
  "EGYPTIAN HIEROGLYPH V031\0"
  "EGYPTIAN HIEROGLYPH V031A\0"
  "EGYPTIAN HIEROGLYPH V032\0"
  "EGYPTIAN HIEROGLYPH V033\0"
  "EGYPTIAN HIEROGLYPH V033A\0"
  "EGYPTIAN HIEROGLYPH V034\0"
  "EGYPTIAN HIEROGLYPH V035\0"
  "EGYPTIAN HIEROGLYPH V036\0"
  "EGYPTIAN HIEROGLYPH V037\0"
  "EGYPTIAN HIEROGLYPH V037A\0"
  "EGYPTIAN HIEROGLYPH V038\0"
  "EGYPTIAN HIEROGLYPH V039\0"
  "EGYPTIAN HIEROGLYPH V040\0"
  "EGYPTIAN HIEROGLYPH V040A\0"
  "EGYPTIAN HIEROGLYPH VERTICAL JOINER\0"
  "EGYPTIAN HIEROGLYPH W001\0"
  "EGYPTIAN HIEROGLYPH W002\0"
  "EGYPTIAN HIEROGLYPH W003\0"
  "EGYPTIAN HIEROGLYPH W003A\0"
  "EGYPTIAN HIEROGLYPH W004\0"
  "EGYPTIAN HIEROGLYPH W005\0"
  "EGYPTIAN HIEROGLYPH W006\0"
  "EGYPTIAN HIEROGLYPH W007\0"
  "EGYPTIAN HIEROGLYPH W008\0"
  "EGYPTIAN HIEROGLYPH W009\0"
  "EGYPTIAN HIEROGLYPH W009A\0"
  "EGYPTIAN HIEROGLYPH W010\0"
  "EGYPTIAN HIEROGLYPH W010A\0"
  "EGYPTIAN HIEROGLYPH W011\0"
  "EGYPTIAN HIEROGLYPH W012\0"
  "EGYPTIAN HIEROGLYPH W013\0"
  "EGYPTIAN HIEROGLYPH W014\0"
  "EGYPTIAN HIEROGLYPH W014A\0"
  "EGYPTIAN HIEROGLYPH W015\0"
  "EGYPTIAN HIEROGLYPH W016\0"
  "EGYPTIAN HIEROGLYPH W017\0"
  "EGYPTIAN HIEROGLYPH W017A\0"
  "EGYPTIAN HIEROGLYPH W018\0"
  "EGYPTIAN HIEROGLYPH W018A\0"
  "EGYPTIAN HIEROGLYPH W019\0"
  "EGYPTIAN HIEROGLYPH W020\0"
  "EGYPTIAN HIEROGLYPH W021\0"
  "EGYPTIAN HIEROGLYPH W022\0"
  "EGYPTIAN HIEROGLYPH W023\0"
  "EGYPTIAN HIEROGLYPH W024\0"
  "EGYPTIAN HIEROGLYPH W024A\0"
  "EGYPTIAN HIEROGLYPH W025\0"
  "EGYPTIAN HIEROGLYPH WIDE LOST SIGN\0"
  "EGYPTIAN HIEROGLYPH X001\0"
  "EGYPTIAN HIEROGLYPH X002\0"
  "EGYPTIAN HIEROGLYPH X003\0"
  "EGYPTIAN HIEROGLYPH X004\0"
  "EGYPTIAN HIEROGLYPH X004A\0"
  "EGYPTIAN HIEROGLYPH X004B\0"
  "EGYPTIAN HIEROGLYPH X005\0"
  "EGYPTIAN HIEROGLYPH X006\0"
  "EGYPTIAN HIEROGLYPH X006A\0"
  "EGYPTIAN HIEROGLYPH X007\0"
  "EGYPTIAN HIEROGLYPH X008\0"
  "EGYPTIAN HIEROGLYPH X008A\0"
  "EGYPTIAN HIEROGLYPH Y001\0"
  "EGYPTIAN HIEROGLYPH Y001A\0"
  "EGYPTIAN HIEROGLYPH Y002\0"
  "EGYPTIAN HIEROGLYPH Y003\0"
  "EGYPTIAN HIEROGLYPH Y004\0"
  "EGYPTIAN HIEROGLYPH Y005\0"
  "EGYPTIAN HIEROGLYPH Y006\0"
  "EGYPTIAN HIEROGLYPH Y007\0"
  "EGYPTIAN HIEROGLYPH Y008\0"
  "EGYPTIAN HIEROGLYPH Z001\0"
  "EGYPTIAN HIEROGLYPH Z002\0"
  "EGYPTIAN HIEROGLYPH Z002A\0"
  "EGYPTIAN HIEROGLYPH Z002B\0"
  "EGYPTIAN HIEROGLYPH Z002C\0"
  "EGYPTIAN HIEROGLYPH Z002D\0"
  "EGYPTIAN HIEROGLYPH Z003\0"
  "EGYPTIAN HIEROGLYPH Z003A\0"
  "EGYPTIAN HIEROGLYPH Z003B\0"
  "EGYPTIAN HIEROGLYPH Z004\0"
  "EGYPTIAN HIEROGLYPH Z004A\0"
  "EGYPTIAN HIEROGLYPH Z005\0"
  "EGYPTIAN HIEROGLYPH Z005A\0"
  "EGYPTIAN HIEROGLYPH Z006\0"
  "EGYPTIAN HIEROGLYPH Z007\0"
  "EGYPTIAN HIEROGLYPH Z008\0"
  "EGYPTIAN HIEROGLYPH Z009\0"
  "EGYPTIAN HIEROGLYPH Z010\0"
  "EGYPTIAN HIEROGLYPH Z011\0"
  "EGYPTIAN HIEROGLYPH Z012\0"
  "EGYPTIAN HIEROGLYPH Z013\0"
  "EGYPTIAN HIEROGLYPH Z014\0"
  "EGYPTIAN HIEROGLYPH Z015\0"
  "EGYPTIAN HIEROGLYPH Z015A\0"
  "EGYPTIAN HIEROGLYPH Z015B\0"
  "EGYPTIAN HIEROGLYPH Z015C\0"
  "EGYPTIAN HIEROGLYPH Z015D\0"
  "EGYPTIAN HIEROGLYPH Z015E\0"
  "EGYPTIAN HIEROGLYPH Z015F\0"
  "EGYPTIAN HIEROGLYPH Z015G\0"
  "EGYPTIAN HIEROGLYPH Z015H\0"
  "EGYPTIAN HIEROGLYPH Z015I\0"
  "EGYPTIAN HIEROGLYPH Z016\0"
  "EGYPTIAN HIEROGLYPH Z016A\0"
  "EGYPTIAN HIEROGLYPH Z016B\0"
  "EGYPTIAN HIEROGLYPH Z016C\0"
  "EGYPTIAN HIEROGLYPH Z016D\0"
  "EGYPTIAN HIEROGLYPH Z016E\0"
  "EGYPTIAN HIEROGLYPH Z016F\0"
  "EGYPTIAN HIEROGLYPH Z016G\0"
  "EGYPTIAN HIEROGLYPH Z016H\0"
  "EGYPTIAN HIEROGLYPH-13460\0"
  "EGYPTIAN HIEROGLYPH-13461\0"
  "EGYPTIAN HIEROGLYPH-13462\0"
  "EGYPTIAN HIEROGLYPH-13463\0"
  "EGYPTIAN HIEROGLYPH-13464\0"
  "EGYPTIAN HIEROGLYPH-13465\0"
  "EGYPTIAN HIEROGLYPH-13466\0"
  "EGYPTIAN HIEROGLYPH-13467\0"
  "EGYPTIAN HIEROGLYPH-13468\0"
  "EGYPTIAN HIEROGLYPH-13469\0"
  "EGYPTIAN HIEROGLYPH-1346A\0"
  "EGYPTIAN HIEROGLYPH-1346B\0"
  "EGYPTIAN HIEROGLYPH-1346C\0"
  "EGYPTIAN HIEROGLYPH-1346D\0"
  "EGYPTIAN HIEROGLYPH-1346E\0"
  "EGYPTIAN HIEROGLYPH-1346F\0"
  "EGYPTIAN HIEROGLYPH-13470\0"
  "EGYPTIAN HIEROGLYPH-13471\0"
  "EGYPTIAN HIEROGLYPH-13472\0"
  "EGYPTIAN HIEROGLYPH-13473\0"
  "EGYPTIAN HIEROGLYPH-13474\0"
  "EGYPTIAN HIEROGLYPH-13475\0"
  "EGYPTIAN HIEROGLYPH-13476\0"
  "EGYPTIAN HIEROGLYPH-13477\0"
  "EGYPTIAN HIEROGLYPH-13478\0"
  "EGYPTIAN HIEROGLYPH-13479\0"
  "EGYPTIAN HIEROGLYPH-1347A\0"
  "EGYPTIAN HIEROGLYPH-1347B\0"
  "EGYPTIAN HIEROGLYPH-1347C\0"
  "EGYPTIAN HIEROGLYPH-1347D\0"
  "EGYPTIAN HIEROGLYPH-1347E\0"
  "EGYPTIAN HIEROGLYPH-1347F\0"
  "EGYPTIAN HIEROGLYPH-13480\0"
  "EGYPTIAN HIEROGLYPH-13481\0"
  "EGYPTIAN HIEROGLYPH-13482\0"
  "EGYPTIAN HIEROGLYPH-13483\0"
  "EGYPTIAN HIEROGLYPH-13484\0"
  "EGYPTIAN HIEROGLYPH-13485\0"
  "EGYPTIAN HIEROGLYPH-13486\0"
  "EGYPTIAN HIEROGLYPH-13487\0"
  "EGYPTIAN HIEROGLYPH-13488\0"
  "EGYPTIAN HIEROGLYPH-13489\0"
  "EGYPTIAN HIEROGLYPH-1348A\0"
  "EGYPTIAN HIEROGLYPH-1348B\0"
  "EGYPTIAN HIEROGLYPH-1348C\0"
  "EGYPTIAN HIEROGLYPH-1348D\0"
  "EGYPTIAN HIEROGLYPH-1348E\0"
  "EGYPTIAN HIEROGLYPH-1348F\0"
  "EGYPTIAN HIEROGLYPH-13490\0"
  "EGYPTIAN HIEROGLYPH-13491\0"
  "EGYPTIAN HIEROGLYPH-13492\0"
  "EGYPTIAN HIEROGLYPH-13493\0"
  "EGYPTIAN HIEROGLYPH-13494\0"
  "EGYPTIAN HIEROGLYPH-13495\0"
  "EGYPTIAN HIEROGLYPH-13496\0"
  "EGYPTIAN HIEROGLYPH-13497\0"
  "EGYPTIAN HIEROGLYPH-13498\0"
  "EGYPTIAN HIEROGLYPH-13499\0"
  "EGYPTIAN HIEROGLYPH-1349A\0"
  "EGYPTIAN HIEROGLYPH-1349B\0"
  "EGYPTIAN HIEROGLYPH-1349C\0"
  "EGYPTIAN HIEROGLYPH-1349D\0"
  "EGYPTIAN HIEROGLYPH-1349E\0"
  "EGYPTIAN HIEROGLYPH-1349F\0"
  "EGYPTIAN HIEROGLYPH-134A0\0"
  "EGYPTIAN HIEROGLYPH-134A1\0"
  "EGYPTIAN HIEROGLYPH-134A2\0"
  "EGYPTIAN HIEROGLYPH-134A3\0"
  "EGYPTIAN HIEROGLYPH-134A4\0"
  "EGYPTIAN HIEROGLYPH-134A5\0"
  "EGYPTIAN HIEROGLYPH-134A6\0"
  "EGYPTIAN HIEROGLYPH-134A7\0"
  "EGYPTIAN HIEROGLYPH-134A8\0"
  "EGYPTIAN HIEROGLYPH-134A9\0"
  "EGYPTIAN HIEROGLYPH-134AA\0"
  "EGYPTIAN HIEROGLYPH-134AB\0"
  "EGYPTIAN HIEROGLYPH-134AC\0"
  "EGYPTIAN HIEROGLYPH-134AD\0"
  "EGYPTIAN HIEROGLYPH-134AE\0"
  "EGYPTIAN HIEROGLYPH-134AF\0"
  "EGYPTIAN HIEROGLYPH-134B0\0"
  "EGYPTIAN HIEROGLYPH-134B1\0"
  "EGYPTIAN HIEROGLYPH-134B2\0"
  "EGYPTIAN HIEROGLYPH-134B3\0"
  "EGYPTIAN HIEROGLYPH-134B4\0"
  "EGYPTIAN HIEROGLYPH-134B5\0"
  "EGYPTIAN HIEROGLYPH-134B6\0"
  "EGYPTIAN HIEROGLYPH-134B7\0"
  "EGYPTIAN HIEROGLYPH-134B8\0"
  "EGYPTIAN HIEROGLYPH-134B9\0"
  "EGYPTIAN HIEROGLYPH-134BA\0"
  "EGYPTIAN HIEROGLYPH-134BB\0"
  "EGYPTIAN HIEROGLYPH-134BC\0"
  "EGYPTIAN HIEROGLYPH-134BD\0"
  "EGYPTIAN HIEROGLYPH-134BE\0"
  "EGYPTIAN HIEROGLYPH-134BF\0"
  "EGYPTIAN HIEROGLYPH-134C0\0"
  "EGYPTIAN HIEROGLYPH-134C1\0"
  "EGYPTIAN HIEROGLYPH-134C2\0"
  "EGYPTIAN HIEROGLYPH-134C3\0"
  "EGYPTIAN HIEROGLYPH-134C4\0"
  "EGYPTIAN HIEROGLYPH-134C5\0"
  "EGYPTIAN HIEROGLYPH-134C6\0"
  "EGYPTIAN HIEROGLYPH-134C7\0"
  "EGYPTIAN HIEROGLYPH-134C8\0"
  "EGYPTIAN HIEROGLYPH-134C9\0"
  "EGYPTIAN HIEROGLYPH-134CA\0"
  "EGYPTIAN HIEROGLYPH-134CB\0"
  "EGYPTIAN HIEROGLYPH-134CC\0"
  "EGYPTIAN HIEROGLYPH-134CD\0"
  "EGYPTIAN HIEROGLYPH-134CE\0"
  "EGYPTIAN HIEROGLYPH-134CF\0"
  "EGYPTIAN HIEROGLYPH-134D0\0"
  "EGYPTIAN HIEROGLYPH-134D1\0"
  "EGYPTIAN HIEROGLYPH-134D2\0"
  "EGYPTIAN HIEROGLYPH-134D3\0"
  "EGYPTIAN HIEROGLYPH-134D4\0"
  "EGYPTIAN HIEROGLYPH-134D5\0"
  "EGYPTIAN HIEROGLYPH-134D6\0"
  "EGYPTIAN HIEROGLYPH-134D7\0"
  "EGYPTIAN HIEROGLYPH-134D8\0"
  "EGYPTIAN HIEROGLYPH-134D9\0"
  "EGYPTIAN HIEROGLYPH-134DA\0"
  "EGYPTIAN HIEROGLYPH-134DB\0"
  "EGYPTIAN HIEROGLYPH-134DC\0"
  "EGYPTIAN HIEROGLYPH-134DD\0"
  "EGYPTIAN HIEROGLYPH-134DE\0"
  "EGYPTIAN HIEROGLYPH-134DF\0"
  "EGYPTIAN HIEROGLYPH-134E0\0"
  "EGYPTIAN HIEROGLYPH-134E1\0"
  "EGYPTIAN HIEROGLYPH-134E2\0"
  "EGYPTIAN HIEROGLYPH-134E3\0"
  "EGYPTIAN HIEROGLYPH-134E4\0"
  "EGYPTIAN HIEROGLYPH-134E5\0"
  "EGYPTIAN HIEROGLYPH-134E6\0"
  "EGYPTIAN HIEROGLYPH-134E7\0"
  "EGYPTIAN HIEROGLYPH-134E8\0"
  "EGYPTIAN HIEROGLYPH-134E9\0"
  "EGYPTIAN HIEROGLYPH-134EA\0"
  "EGYPTIAN HIEROGLYPH-134EB\0"
  "EGYPTIAN HIEROGLYPH-134EC\0"
  "EGYPTIAN HIEROGLYPH-134ED\0"
  "EGYPTIAN HIEROGLYPH-134EE\0"
  "EGYPTIAN HIEROGLYPH-134EF\0"
  "EGYPTIAN HIEROGLYPH-134F0\0"
  "EGYPTIAN HIEROGLYPH-134F1\0"
  "EGYPTIAN HIEROGLYPH-134F2\0"
  "EGYPTIAN HIEROGLYPH-134F3\0"
  "EGYPTIAN HIEROGLYPH-134F4\0"
  "EGYPTIAN HIEROGLYPH-134F5\0"
  "EGYPTIAN HIEROGLYPH-134F6\0"
  "EGYPTIAN HIEROGLYPH-134F7\0"
  "EGYPTIAN HIEROGLYPH-134F8\0"
  "EGYPTIAN HIEROGLYPH-134F9\0"
  "EGYPTIAN HIEROGLYPH-134FA\0"
  "EGYPTIAN HIEROGLYPH-134FB\0"
  "EGYPTIAN HIEROGLYPH-134FC\0"
  "EGYPTIAN HIEROGLYPH-134FD\0"
  "EGYPTIAN HIEROGLYPH-134FE\0"
  "EGYPTIAN HIEROGLYPH-134FF\0"
  "EGYPTIAN HIEROGLYPH-13500\0"
  "EGYPTIAN HIEROGLYPH-13501\0"
  "EGYPTIAN HIEROGLYPH-13502\0"
  "EGYPTIAN HIEROGLYPH-13503\0"
  "EGYPTIAN HIEROGLYPH-13504\0"
  "EGYPTIAN HIEROGLYPH-13505\0"
  "EGYPTIAN HIEROGLYPH-13506\0"
  "EGYPTIAN HIEROGLYPH-13507\0"
  "EGYPTIAN HIEROGLYPH-13508\0"
  "EGYPTIAN HIEROGLYPH-13509\0"
  "EGYPTIAN HIEROGLYPH-1350A\0"
  "EGYPTIAN HIEROGLYPH-1350B\0"
  "EGYPTIAN HIEROGLYPH-1350C\0"
  "EGYPTIAN HIEROGLYPH-1350D\0"
  "EGYPTIAN HIEROGLYPH-1350E\0"
  "EGYPTIAN HIEROGLYPH-1350F\0"
  "EGYPTIAN HIEROGLYPH-13510\0"
  "EGYPTIAN HIEROGLYPH-13511\0"
  "EGYPTIAN HIEROGLYPH-13512\0"
  "EGYPTIAN HIEROGLYPH-13513\0"
  "EGYPTIAN HIEROGLYPH-13514\0"
  "EGYPTIAN HIEROGLYPH-13515\0"
  "EGYPTIAN HIEROGLYPH-13516\0"
  "EGYPTIAN HIEROGLYPH-13517\0"
  "EGYPTIAN HIEROGLYPH-13518\0"
  "EGYPTIAN HIEROGLYPH-13519\0"
  "EGYPTIAN HIEROGLYPH-1351A\0"
  "EGYPTIAN HIEROGLYPH-1351B\0"
  "EGYPTIAN HIEROGLYPH-1351C\0"
  "EGYPTIAN HIEROGLYPH-1351D\0"
  "EGYPTIAN HIEROGLYPH-1351E\0"
  "EGYPTIAN HIEROGLYPH-1351F\0"
  "EGYPTIAN HIEROGLYPH-13520\0"
  "EGYPTIAN HIEROGLYPH-13521\0"
  "EGYPTIAN HIEROGLYPH-13522\0"
  "EGYPTIAN HIEROGLYPH-13523\0"
  "EGYPTIAN HIEROGLYPH-13524\0"
  "EGYPTIAN HIEROGLYPH-13525\0"
  "EGYPTIAN HIEROGLYPH-13526\0"
  "EGYPTIAN HIEROGLYPH-13527\0"
  "EGYPTIAN HIEROGLYPH-13528\0"
  "EGYPTIAN HIEROGLYPH-13529\0"
  "EGYPTIAN HIEROGLYPH-1352A\0"
  "EGYPTIAN HIEROGLYPH-1352B\0"
  "EGYPTIAN HIEROGLYPH-1352C\0"
  "EGYPTIAN HIEROGLYPH-1352D\0"
  "EGYPTIAN HIEROGLYPH-1352E\0"
  "EGYPTIAN HIEROGLYPH-1352F\0"
  "EGYPTIAN HIEROGLYPH-13530\0"
  "EGYPTIAN HIEROGLYPH-13531\0"
  "EGYPTIAN HIEROGLYPH-13532\0"
  "EGYPTIAN HIEROGLYPH-13533\0"
  "EGYPTIAN HIEROGLYPH-13534\0"
  "EGYPTIAN HIEROGLYPH-13535\0"
  "EGYPTIAN HIEROGLYPH-13536\0"
  "EGYPTIAN HIEROGLYPH-13537\0"
  "EGYPTIAN HIEROGLYPH-13538\0"
  "EGYPTIAN HIEROGLYPH-13539\0"
  "EGYPTIAN HIEROGLYPH-1353A\0"
  "EGYPTIAN HIEROGLYPH-1353B\0"
  "EGYPTIAN HIEROGLYPH-1353C\0"
  "EGYPTIAN HIEROGLYPH-1353D\0"
  "EGYPTIAN HIEROGLYPH-1353E\0"
  "EGYPTIAN HIEROGLYPH-1353F\0"
  "EGYPTIAN HIEROGLYPH-13540\0"
  "EGYPTIAN HIEROGLYPH-13541\0"
  "EGYPTIAN HIEROGLYPH-13542\0"
  "EGYPTIAN HIEROGLYPH-13543\0"
  "EGYPTIAN HIEROGLYPH-13544\0"
  "EGYPTIAN HIEROGLYPH-13545\0"
  "EGYPTIAN HIEROGLYPH-13546\0"
  "EGYPTIAN HIEROGLYPH-13547\0"
  "EGYPTIAN HIEROGLYPH-13548\0"
  "EGYPTIAN HIEROGLYPH-13549\0"
  "EGYPTIAN HIEROGLYPH-1354A\0"
  "EGYPTIAN HIEROGLYPH-1354B\0"
  "EGYPTIAN HIEROGLYPH-1354C\0"
  "EGYPTIAN HIEROGLYPH-1354D\0"
  "EGYPTIAN HIEROGLYPH-1354E\0"
  "EGYPTIAN HIEROGLYPH-1354F\0"
  "EGYPTIAN HIEROGLYPH-13550\0"
  "EGYPTIAN HIEROGLYPH-13551\0"
  "EGYPTIAN HIEROGLYPH-13552\0"
  "EGYPTIAN HIEROGLYPH-13553\0"
  "EGYPTIAN HIEROGLYPH-13554\0"
  "EGYPTIAN HIEROGLYPH-13555\0"
  "EGYPTIAN HIEROGLYPH-13556\0"
  "EGYPTIAN HIEROGLYPH-13557\0"
  "EGYPTIAN HIEROGLYPH-13558\0"
  "EGYPTIAN HIEROGLYPH-13559\0"
  "EGYPTIAN HIEROGLYPH-1355A\0"
  "EGYPTIAN HIEROGLYPH-1355B\0"
  "EGYPTIAN HIEROGLYPH-1355C\0"
  "EGYPTIAN HIEROGLYPH-1355D\0"
  "EGYPTIAN HIEROGLYPH-1355E\0"
  "EGYPTIAN HIEROGLYPH-1355F\0"
  "EGYPTIAN HIEROGLYPH-13560\0"
  "EGYPTIAN HIEROGLYPH-13561\0"
  "EGYPTIAN HIEROGLYPH-13562\0"
  "EGYPTIAN HIEROGLYPH-13563\0"
  "EGYPTIAN HIEROGLYPH-13564\0"
  "EGYPTIAN HIEROGLYPH-13565\0"
  "EGYPTIAN HIEROGLYPH-13566\0"
  "EGYPTIAN HIEROGLYPH-13567\0"
  "EGYPTIAN HIEROGLYPH-13568\0"
  "EGYPTIAN HIEROGLYPH-13569\0"
  "EGYPTIAN HIEROGLYPH-1356A\0"
  "EGYPTIAN HIEROGLYPH-1356B\0"
  "EGYPTIAN HIEROGLYPH-1356C\0"
  "EGYPTIAN HIEROGLYPH-1356D\0"
  "EGYPTIAN HIEROGLYPH-1356E\0"
  "EGYPTIAN HIEROGLYPH-1356F\0"
  "EGYPTIAN HIEROGLYPH-13570\0"
  "EGYPTIAN HIEROGLYPH-13571\0"
  "EGYPTIAN HIEROGLYPH-13572\0"
  "EGYPTIAN HIEROGLYPH-13573\0"
  "EGYPTIAN HIEROGLYPH-13574\0"
  "EGYPTIAN HIEROGLYPH-13575\0"
  "EGYPTIAN HIEROGLYPH-13576\0"
  "EGYPTIAN HIEROGLYPH-13577\0"
  "EGYPTIAN HIEROGLYPH-13578\0"
  "EGYPTIAN HIEROGLYPH-13579\0"
  "EGYPTIAN HIEROGLYPH-1357A\0"
  "EGYPTIAN HIEROGLYPH-1357B\0"
  "EGYPTIAN HIEROGLYPH-1357C\0"
  "EGYPTIAN HIEROGLYPH-1357D\0"
  "EGYPTIAN HIEROGLYPH-1357E\0"
  "EGYPTIAN HIEROGLYPH-1357F\0"
  "EGYPTIAN HIEROGLYPH-13580\0"
  "EGYPTIAN HIEROGLYPH-13581\0"
  "EGYPTIAN HIEROGLYPH-13582\0"
  "EGYPTIAN HIEROGLYPH-13583\0"
  "EGYPTIAN HIEROGLYPH-13584\0"
  "EGYPTIAN HIEROGLYPH-13585\0"
  "EGYPTIAN HIEROGLYPH-13586\0"
  "EGYPTIAN HIEROGLYPH-13587\0"
  "EGYPTIAN HIEROGLYPH-13588\0"
  "EGYPTIAN HIEROGLYPH-13589\0"
  "EGYPTIAN HIEROGLYPH-1358A\0"
  "EGYPTIAN HIEROGLYPH-1358B\0"
  "EGYPTIAN HIEROGLYPH-1358C\0"
  "EGYPTIAN HIEROGLYPH-1358D\0"
  "EGYPTIAN HIEROGLYPH-1358E\0"
  "EGYPTIAN HIEROGLYPH-1358F\0"
  "EGYPTIAN HIEROGLYPH-13590\0"
  "EGYPTIAN HIEROGLYPH-13591\0"
  "EGYPTIAN HIEROGLYPH-13592\0"
  "EGYPTIAN HIEROGLYPH-13593\0"
  "EGYPTIAN HIEROGLYPH-13594\0"
  "EGYPTIAN HIEROGLYPH-13595\0"
  "EGYPTIAN HIEROGLYPH-13596\0"
  "EGYPTIAN HIEROGLYPH-13597\0"
  "EGYPTIAN HIEROGLYPH-13598\0"
  "EGYPTIAN HIEROGLYPH-13599\0"
  "EGYPTIAN HIEROGLYPH-1359A\0"
  "EGYPTIAN HIEROGLYPH-1359B\0"
  "EGYPTIAN HIEROGLYPH-1359C\0"
  "EGYPTIAN HIEROGLYPH-1359D\0"
  "EGYPTIAN HIEROGLYPH-1359E\0"
  "EGYPTIAN HIEROGLYPH-1359F\0"
  "EGYPTIAN HIEROGLYPH-135A0\0"
  "EGYPTIAN HIEROGLYPH-135A1\0"
  "EGYPTIAN HIEROGLYPH-135A2\0"
  "EGYPTIAN HIEROGLYPH-135A3\0"
  "EGYPTIAN HIEROGLYPH-135A4\0"
  "EGYPTIAN HIEROGLYPH-135A5\0"
  "EGYPTIAN HIEROGLYPH-135A6\0"
  "EGYPTIAN HIEROGLYPH-135A7\0"
  "EGYPTIAN HIEROGLYPH-135A8\0"
  "EGYPTIAN HIEROGLYPH-135A9\0"
  "EGYPTIAN HIEROGLYPH-135AA\0"
  "EGYPTIAN HIEROGLYPH-135AB\0"
  "EGYPTIAN HIEROGLYPH-135AC\0"
  "EGYPTIAN HIEROGLYPH-135AD\0"
  "EGYPTIAN HIEROGLYPH-135AE\0"
  "EGYPTIAN HIEROGLYPH-135AF\0"
  "EGYPTIAN HIEROGLYPH-135B0\0"
  "EGYPTIAN HIEROGLYPH-135B1\0"
  "EGYPTIAN HIEROGLYPH-135B2\0"
  "EGYPTIAN HIEROGLYPH-135B3\0"
  "EGYPTIAN HIEROGLYPH-135B4\0"
  "EGYPTIAN HIEROGLYPH-135B5\0"
  "EGYPTIAN HIEROGLYPH-135B6\0"
  "EGYPTIAN HIEROGLYPH-135B7\0"
  "EGYPTIAN HIEROGLYPH-135B8\0"
  "EGYPTIAN HIEROGLYPH-135B9\0"
  "EGYPTIAN HIEROGLYPH-135BA\0"
  "EGYPTIAN HIEROGLYPH-135BB\0"
  "EGYPTIAN HIEROGLYPH-135BC\0"
  "EGYPTIAN HIEROGLYPH-135BD\0"
  "EGYPTIAN HIEROGLYPH-135BE\0"
  "EGYPTIAN HIEROGLYPH-135BF\0"
  "EGYPTIAN HIEROGLYPH-135C0\0"
  "EGYPTIAN HIEROGLYPH-135C1\0"
  "EGYPTIAN HIEROGLYPH-135C2\0"
  "EGYPTIAN HIEROGLYPH-135C3\0"
  "EGYPTIAN HIEROGLYPH-135C4\0"
  "EGYPTIAN HIEROGLYPH-135C5\0"
  "EGYPTIAN HIEROGLYPH-135C6\0"
  "EGYPTIAN HIEROGLYPH-135C7\0"
  "EGYPTIAN HIEROGLYPH-135C8\0"
  "EGYPTIAN HIEROGLYPH-135C9\0"
  "EGYPTIAN HIEROGLYPH-135CA\0"
  "EGYPTIAN HIEROGLYPH-135CB\0"
  "EGYPTIAN HIEROGLYPH-135CC\0"
  "EGYPTIAN HIEROGLYPH-135CD\0"
  "EGYPTIAN HIEROGLYPH-135CE\0"
  "EGYPTIAN HIEROGLYPH-135CF\0"
  "EGYPTIAN HIEROGLYPH-135D0\0"
  "EGYPTIAN HIEROGLYPH-135D1\0"
  "EGYPTIAN HIEROGLYPH-135D2\0"
  "EGYPTIAN HIEROGLYPH-135D3\0"
  "EGYPTIAN HIEROGLYPH-135D4\0"
  "EGYPTIAN HIEROGLYPH-135D5\0"
  "EGYPTIAN HIEROGLYPH-135D6\0"
  "EGYPTIAN HIEROGLYPH-135D7\0"
  "EGYPTIAN HIEROGLYPH-135D8\0"
  "EGYPTIAN HIEROGLYPH-135D9\0"
  "EGYPTIAN HIEROGLYPH-135DA\0"
  "EGYPTIAN HIEROGLYPH-135DB\0"
  "EGYPTIAN HIEROGLYPH-135DC\0"
  "EGYPTIAN HIEROGLYPH-135DD\0"
  "EGYPTIAN HIEROGLYPH-135DE\0"
  "EGYPTIAN HIEROGLYPH-135DF\0"
  "EGYPTIAN HIEROGLYPH-135E0\0"
  "EGYPTIAN HIEROGLYPH-135E1\0"
  "EGYPTIAN HIEROGLYPH-135E2\0"
  "EGYPTIAN HIEROGLYPH-135E3\0"
  "EGYPTIAN HIEROGLYPH-135E4\0"
  "EGYPTIAN HIEROGLYPH-135E5\0"
  "EGYPTIAN HIEROGLYPH-135E6\0"
  "EGYPTIAN HIEROGLYPH-135E7\0"
  "EGYPTIAN HIEROGLYPH-135E8\0"
  "EGYPTIAN HIEROGLYPH-135E9\0"
  "EGYPTIAN HIEROGLYPH-135EA\0"
  "EGYPTIAN HIEROGLYPH-135EB\0"
  "EGYPTIAN HIEROGLYPH-135EC\0"
  "EGYPTIAN HIEROGLYPH-135ED\0"
  "EGYPTIAN HIEROGLYPH-135EE\0"
  "EGYPTIAN HIEROGLYPH-135EF\0"
  "EGYPTIAN HIEROGLYPH-135F0\0"
  "EGYPTIAN HIEROGLYPH-135F1\0"
  "EGYPTIAN HIEROGLYPH-135F2\0"
  "EGYPTIAN HIEROGLYPH-135F3\0"
  "EGYPTIAN HIEROGLYPH-135F4\0"
  "EGYPTIAN HIEROGLYPH-135F5\0"
  "EGYPTIAN HIEROGLYPH-135F6\0"
  "EGYPTIAN HIEROGLYPH-135F7\0"
  "EGYPTIAN HIEROGLYPH-135F8\0"
  "EGYPTIAN HIEROGLYPH-135F9\0"
  "EGYPTIAN HIEROGLYPH-135FA\0"
  "EGYPTIAN HIEROGLYPH-135FB\0"
  "EGYPTIAN HIEROGLYPH-135FC\0"
  "EGYPTIAN HIEROGLYPH-135FD\0"
  "EGYPTIAN HIEROGLYPH-135FE\0"
  "EGYPTIAN HIEROGLYPH-135FF\0"
  "EGYPTIAN HIEROGLYPH-13600\0"
  "EGYPTIAN HIEROGLYPH-13601\0"
  "EGYPTIAN HIEROGLYPH-13602\0"
  "EGYPTIAN HIEROGLYPH-13603\0"
  "EGYPTIAN HIEROGLYPH-13604\0"
  "EGYPTIAN HIEROGLYPH-13605\0"
  "EGYPTIAN HIEROGLYPH-13606\0"
  "EGYPTIAN HIEROGLYPH-13607\0"
  "EGYPTIAN HIEROGLYPH-13608\0"
  "EGYPTIAN HIEROGLYPH-13609\0"
  "EGYPTIAN HIEROGLYPH-1360A\0"
  "EGYPTIAN HIEROGLYPH-1360B\0"
  "EGYPTIAN HIEROGLYPH-1360C\0"
  "EGYPTIAN HIEROGLYPH-1360D\0"
  "EGYPTIAN HIEROGLYPH-1360E\0"
  "EGYPTIAN HIEROGLYPH-1360F\0"
  "EGYPTIAN HIEROGLYPH-13610\0"
  "EGYPTIAN HIEROGLYPH-13611\0"
  "EGYPTIAN HIEROGLYPH-13612\0"
  "EGYPTIAN HIEROGLYPH-13613\0"
  "EGYPTIAN HIEROGLYPH-13614\0"
  "EGYPTIAN HIEROGLYPH-13615\0"
  "EGYPTIAN HIEROGLYPH-13616\0"
  "EGYPTIAN HIEROGLYPH-13617\0"
  "EGYPTIAN HIEROGLYPH-13618\0"
  "EGYPTIAN HIEROGLYPH-13619\0"
  "EGYPTIAN HIEROGLYPH-1361A\0"
  "EGYPTIAN HIEROGLYPH-1361B\0"
  "EGYPTIAN HIEROGLYPH-1361C\0"
  "EGYPTIAN HIEROGLYPH-1361D\0"
  "EGYPTIAN HIEROGLYPH-1361E\0"
  "EGYPTIAN HIEROGLYPH-1361F\0"
  "EGYPTIAN HIEROGLYPH-13620\0"
  "EGYPTIAN HIEROGLYPH-13621\0"
  "EGYPTIAN HIEROGLYPH-13622\0"
  "EGYPTIAN HIEROGLYPH-13623\0"
  "EGYPTIAN HIEROGLYPH-13624\0"
  "EGYPTIAN HIEROGLYPH-13625\0"
  "EGYPTIAN HIEROGLYPH-13626\0"
  "EGYPTIAN HIEROGLYPH-13627\0"
  "EGYPTIAN HIEROGLYPH-13628\0"
  "EGYPTIAN HIEROGLYPH-13629\0"
  "EGYPTIAN HIEROGLYPH-1362A\0"
  "EGYPTIAN HIEROGLYPH-1362B\0"
  "EGYPTIAN HIEROGLYPH-1362C\0"
  "EGYPTIAN HIEROGLYPH-1362D\0"
  "EGYPTIAN HIEROGLYPH-1362E\0"
  "EGYPTIAN HIEROGLYPH-1362F\0"
  "EGYPTIAN HIEROGLYPH-13630\0"
  "EGYPTIAN HIEROGLYPH-13631\0"
  "EGYPTIAN HIEROGLYPH-13632\0"
  "EGYPTIAN HIEROGLYPH-13633\0"
  "EGYPTIAN HIEROGLYPH-13634\0"
  "EGYPTIAN HIEROGLYPH-13635\0"
  "EGYPTIAN HIEROGLYPH-13636\0"
  "EGYPTIAN HIEROGLYPH-13637\0"
  "EGYPTIAN HIEROGLYPH-13638\0"
  "EGYPTIAN HIEROGLYPH-13639\0"
  "EGYPTIAN HIEROGLYPH-1363A\0"
  "EGYPTIAN HIEROGLYPH-1363B\0"
  "EGYPTIAN HIEROGLYPH-1363C\0"
  "EGYPTIAN HIEROGLYPH-1363D\0"
  "EGYPTIAN HIEROGLYPH-1363E\0"
  "EGYPTIAN HIEROGLYPH-1363F\0"
  "EGYPTIAN HIEROGLYPH-13640\0"
  "EGYPTIAN HIEROGLYPH-13641\0"
  "EGYPTIAN HIEROGLYPH-13642\0"
  "EGYPTIAN HIEROGLYPH-13643\0"
  "EGYPTIAN HIEROGLYPH-13644\0"
  "EGYPTIAN HIEROGLYPH-13645\0"
  "EGYPTIAN HIEROGLYPH-13646\0"
  "EGYPTIAN HIEROGLYPH-13647\0"
  "EGYPTIAN HIEROGLYPH-13648\0"
  "EGYPTIAN HIEROGLYPH-13649\0"
  "EGYPTIAN HIEROGLYPH-1364A\0"
  "EGYPTIAN HIEROGLYPH-1364B\0"
  "EGYPTIAN HIEROGLYPH-1364C\0"
  "EGYPTIAN HIEROGLYPH-1364D\0"
  "EGYPTIAN HIEROGLYPH-1364E\0"
  "EGYPTIAN HIEROGLYPH-1364F\0"
  "EGYPTIAN HIEROGLYPH-13650\0"
  "EGYPTIAN HIEROGLYPH-13651\0"
  "EGYPTIAN HIEROGLYPH-13652\0"
  "EGYPTIAN HIEROGLYPH-13653\0"
  "EGYPTIAN HIEROGLYPH-13654\0"
  "EGYPTIAN HIEROGLYPH-13655\0"
  "EGYPTIAN HIEROGLYPH-13656\0"
  "EGYPTIAN HIEROGLYPH-13657\0"
  "EGYPTIAN HIEROGLYPH-13658\0"
  "EGYPTIAN HIEROGLYPH-13659\0"
  "EGYPTIAN HIEROGLYPH-1365A\0"
  "EGYPTIAN HIEROGLYPH-1365B\0"
  "EGYPTIAN HIEROGLYPH-1365C\0"
  "EGYPTIAN HIEROGLYPH-1365D\0"
  "EGYPTIAN HIEROGLYPH-1365E\0"
  "EGYPTIAN HIEROGLYPH-1365F\0"
  "EGYPTIAN HIEROGLYPH-13660\0"
  "EGYPTIAN HIEROGLYPH-13661\0"
  "EGYPTIAN HIEROGLYPH-13662\0"
  "EGYPTIAN HIEROGLYPH-13663\0"
  "EGYPTIAN HIEROGLYPH-13664\0"
  "EGYPTIAN HIEROGLYPH-13665\0"
  "EGYPTIAN HIEROGLYPH-13666\0"
  "EGYPTIAN HIEROGLYPH-13667\0"
  "EGYPTIAN HIEROGLYPH-13668\0"
  "EGYPTIAN HIEROGLYPH-13669\0"
  "EGYPTIAN HIEROGLYPH-1366A\0"
  "EGYPTIAN HIEROGLYPH-1366B\0"
  "EGYPTIAN HIEROGLYPH-1366C\0"
  "EGYPTIAN HIEROGLYPH-1366D\0"
  "EGYPTIAN HIEROGLYPH-1366E\0"
  "EGYPTIAN HIEROGLYPH-1366F\0"
  "EGYPTIAN HIEROGLYPH-13670\0"
  "EGYPTIAN HIEROGLYPH-13671\0"
  "EGYPTIAN HIEROGLYPH-13672\0"
  "EGYPTIAN HIEROGLYPH-13673\0"
  "EGYPTIAN HIEROGLYPH-13674\0"
  "EGYPTIAN HIEROGLYPH-13675\0"
  "EGYPTIAN HIEROGLYPH-13676\0"
  "EGYPTIAN HIEROGLYPH-13677\0"
  "EGYPTIAN HIEROGLYPH-13678\0"
  "EGYPTIAN HIEROGLYPH-13679\0"
  "EGYPTIAN HIEROGLYPH-1367A\0"
  "EGYPTIAN HIEROGLYPH-1367B\0"
  "EGYPTIAN HIEROGLYPH-1367C\0"
  "EGYPTIAN HIEROGLYPH-1367D\0"
  "EGYPTIAN HIEROGLYPH-1367E\0"
  "EGYPTIAN HIEROGLYPH-1367F\0"
  "EGYPTIAN HIEROGLYPH-13680\0"
  "EGYPTIAN HIEROGLYPH-13681\0"
  "EGYPTIAN HIEROGLYPH-13682\0"
  "EGYPTIAN HIEROGLYPH-13683\0"
  "EGYPTIAN HIEROGLYPH-13684\0"
  "EGYPTIAN HIEROGLYPH-13685\0"
  "EGYPTIAN HIEROGLYPH-13686\0"
  "EGYPTIAN HIEROGLYPH-13687\0"
  "EGYPTIAN HIEROGLYPH-13688\0"
  "EGYPTIAN HIEROGLYPH-13689\0"
  "EGYPTIAN HIEROGLYPH-1368A\0"
  "EGYPTIAN HIEROGLYPH-1368B\0"
  "EGYPTIAN HIEROGLYPH-1368C\0"
  "EGYPTIAN HIEROGLYPH-1368D\0"
  "EGYPTIAN HIEROGLYPH-1368E\0"
  "EGYPTIAN HIEROGLYPH-1368F\0"
  "EGYPTIAN HIEROGLYPH-13690\0"
  "EGYPTIAN HIEROGLYPH-13691\0"
  "EGYPTIAN HIEROGLYPH-13692\0"
  "EGYPTIAN HIEROGLYPH-13693\0"
  "EGYPTIAN HIEROGLYPH-13694\0"
  "EGYPTIAN HIEROGLYPH-13695\0"
  "EGYPTIAN HIEROGLYPH-13696\0"
  "EGYPTIAN HIEROGLYPH-13697\0"
  "EGYPTIAN HIEROGLYPH-13698\0"
  "EGYPTIAN HIEROGLYPH-13699\0"
  "EGYPTIAN HIEROGLYPH-1369A\0"
  "EGYPTIAN HIEROGLYPH-1369B\0"
  "EGYPTIAN HIEROGLYPH-1369C\0"
  "EGYPTIAN HIEROGLYPH-1369D\0"
  "EGYPTIAN HIEROGLYPH-1369E\0"
  "EGYPTIAN HIEROGLYPH-1369F\0"
  "EGYPTIAN HIEROGLYPH-136A0\0"
  "EGYPTIAN HIEROGLYPH-136A1\0"
  "EGYPTIAN HIEROGLYPH-136A2\0"
  "EGYPTIAN HIEROGLYPH-136A3\0"
  "EGYPTIAN HIEROGLYPH-136A4\0"
  "EGYPTIAN HIEROGLYPH-136A5\0"
  "EGYPTIAN HIEROGLYPH-136A6\0"
  "EGYPTIAN HIEROGLYPH-136A7\0"
  "EGYPTIAN HIEROGLYPH-136A8\0"
  "EGYPTIAN HIEROGLYPH-136A9\0"
  "EGYPTIAN HIEROGLYPH-136AA\0"
  "EGYPTIAN HIEROGLYPH-136AB\0"
  "EGYPTIAN HIEROGLYPH-136AC\0"
  "EGYPTIAN HIEROGLYPH-136AD\0"
  "EGYPTIAN HIEROGLYPH-136AE\0"
  "EGYPTIAN HIEROGLYPH-136AF\0"
  "EGYPTIAN HIEROGLYPH-136B0\0"
  "EGYPTIAN HIEROGLYPH-136B1\0"
  "EGYPTIAN HIEROGLYPH-136B2\0"
  "EGYPTIAN HIEROGLYPH-136B3\0"
  "EGYPTIAN HIEROGLYPH-136B4\0"
  "EGYPTIAN HIEROGLYPH-136B5\0"
  "EGYPTIAN HIEROGLYPH-136B6\0"
  "EGYPTIAN HIEROGLYPH-136B7\0"
  "EGYPTIAN HIEROGLYPH-136B8\0"
  "EGYPTIAN HIEROGLYPH-136B9\0"
  "EGYPTIAN HIEROGLYPH-136BA\0"
  "EGYPTIAN HIEROGLYPH-136BB\0"
  "EGYPTIAN HIEROGLYPH-136BC\0"
  "EGYPTIAN HIEROGLYPH-136BD\0"
  "EGYPTIAN HIEROGLYPH-136BE\0"
  "EGYPTIAN HIEROGLYPH-136BF\0"
  "EGYPTIAN HIEROGLYPH-136C0\0"
  "EGYPTIAN HIEROGLYPH-136C1\0"
  "EGYPTIAN HIEROGLYPH-136C2\0"
  "EGYPTIAN HIEROGLYPH-136C3\0"
  "EGYPTIAN HIEROGLYPH-136C4\0"
  "EGYPTIAN HIEROGLYPH-136C5\0"
  "EGYPTIAN HIEROGLYPH-136C6\0"
  "EGYPTIAN HIEROGLYPH-136C7\0"
  "EGYPTIAN HIEROGLYPH-136C8\0"
  "EGYPTIAN HIEROGLYPH-136C9\0"
  "EGYPTIAN HIEROGLYPH-136CA\0"
  "EGYPTIAN HIEROGLYPH-136CB\0"
  "EGYPTIAN HIEROGLYPH-136CC\0"
  "EGYPTIAN HIEROGLYPH-136CD\0"
  "EGYPTIAN HIEROGLYPH-136CE\0"
  "EGYPTIAN HIEROGLYPH-136CF\0"
  "EGYPTIAN HIEROGLYPH-136D0\0"
  "EGYPTIAN HIEROGLYPH-136D1\0"
  "EGYPTIAN HIEROGLYPH-136D2\0"
  "EGYPTIAN HIEROGLYPH-136D3\0"
  "EGYPTIAN HIEROGLYPH-136D4\0"
  "EGYPTIAN HIEROGLYPH-136D5\0"
  "EGYPTIAN HIEROGLYPH-136D6\0"
  "EGYPTIAN HIEROGLYPH-136D7\0"
  "EGYPTIAN HIEROGLYPH-136D8\0"
  "EGYPTIAN HIEROGLYPH-136D9\0"
  "EGYPTIAN HIEROGLYPH-136DA\0"
  "EGYPTIAN HIEROGLYPH-136DB\0"
  "EGYPTIAN HIEROGLYPH-136DC\0"
  "EGYPTIAN HIEROGLYPH-136DD\0"
  "EGYPTIAN HIEROGLYPH-136DE\0"
  "EGYPTIAN HIEROGLYPH-136DF\0"
  "EGYPTIAN HIEROGLYPH-136E0\0"
  "EGYPTIAN HIEROGLYPH-136E1\0"
  "EGYPTIAN HIEROGLYPH-136E2\0"
  "EGYPTIAN HIEROGLYPH-136E3\0"
  "EGYPTIAN HIEROGLYPH-136E4\0"
  "EGYPTIAN HIEROGLYPH-136E5\0"
  "EGYPTIAN HIEROGLYPH-136E6\0"
  "EGYPTIAN HIEROGLYPH-136E7\0"
  "EGYPTIAN HIEROGLYPH-136E8\0"
  "EGYPTIAN HIEROGLYPH-136E9\0"
  "EGYPTIAN HIEROGLYPH-136EA\0"
  "EGYPTIAN HIEROGLYPH-136EB\0"
  "EGYPTIAN HIEROGLYPH-136EC\0"
  "EGYPTIAN HIEROGLYPH-136ED\0"
  "EGYPTIAN HIEROGLYPH-136EE\0"
  "EGYPTIAN HIEROGLYPH-136EF\0"
  "EGYPTIAN HIEROGLYPH-136F0\0"
  "EGYPTIAN HIEROGLYPH-136F1\0"
  "EGYPTIAN HIEROGLYPH-136F2\0"
  "EGYPTIAN HIEROGLYPH-136F3\0"
  "EGYPTIAN HIEROGLYPH-136F4\0"
  "EGYPTIAN HIEROGLYPH-136F5\0"
  "EGYPTIAN HIEROGLYPH-136F6\0"
  "EGYPTIAN HIEROGLYPH-136F7\0"
  "EGYPTIAN HIEROGLYPH-136F8\0"
  "EGYPTIAN HIEROGLYPH-136F9\0"
  "EGYPTIAN HIEROGLYPH-136FA\0"
  "EGYPTIAN HIEROGLYPH-136FB\0"
  "EGYPTIAN HIEROGLYPH-136FC\0"
  "EGYPTIAN HIEROGLYPH-136FD\0"
  "EGYPTIAN HIEROGLYPH-136FE\0"
  "EGYPTIAN HIEROGLYPH-136FF\0"
  "EGYPTIAN HIEROGLYPH-13700\0"
  "EGYPTIAN HIEROGLYPH-13701\0"
  "EGYPTIAN HIEROGLYPH-13702\0"
  "EGYPTIAN HIEROGLYPH-13703\0"
  "EGYPTIAN HIEROGLYPH-13704\0"
  "EGYPTIAN HIEROGLYPH-13705\0"
  "EGYPTIAN HIEROGLYPH-13706\0"
  "EGYPTIAN HIEROGLYPH-13707\0"
  "EGYPTIAN HIEROGLYPH-13708\0"
  "EGYPTIAN HIEROGLYPH-13709\0"
  "EGYPTIAN HIEROGLYPH-1370A\0"
  "EGYPTIAN HIEROGLYPH-1370B\0"
  "EGYPTIAN HIEROGLYPH-1370C\0"
  "EGYPTIAN HIEROGLYPH-1370D\0"
  "EGYPTIAN HIEROGLYPH-1370E\0"
  "EGYPTIAN HIEROGLYPH-1370F\0"
  "EGYPTIAN HIEROGLYPH-13710\0"
  "EGYPTIAN HIEROGLYPH-13711\0"
  "EGYPTIAN HIEROGLYPH-13712\0"
  "EGYPTIAN HIEROGLYPH-13713\0"
  "EGYPTIAN HIEROGLYPH-13714\0"
  "EGYPTIAN HIEROGLYPH-13715\0"
  "EGYPTIAN HIEROGLYPH-13716\0"
  "EGYPTIAN HIEROGLYPH-13717\0"
  "EGYPTIAN HIEROGLYPH-13718\0"
  "EGYPTIAN HIEROGLYPH-13719\0"
  "EGYPTIAN HIEROGLYPH-1371A\0"
  "EGYPTIAN HIEROGLYPH-1371B\0"
  "EGYPTIAN HIEROGLYPH-1371C\0"
  "EGYPTIAN HIEROGLYPH-1371D\0"
  "EGYPTIAN HIEROGLYPH-1371E\0"
  "EGYPTIAN HIEROGLYPH-1371F\0"
  "EGYPTIAN HIEROGLYPH-13720\0"
  "EGYPTIAN HIEROGLYPH-13721\0"
  "EGYPTIAN HIEROGLYPH-13722\0"
  "EGYPTIAN HIEROGLYPH-13723\0"
  "EGYPTIAN HIEROGLYPH-13724\0"
  "EGYPTIAN HIEROGLYPH-13725\0"
  "EGYPTIAN HIEROGLYPH-13726\0"
  "EGYPTIAN HIEROGLYPH-13727\0"
  "EGYPTIAN HIEROGLYPH-13728\0"
  "EGYPTIAN HIEROGLYPH-13729\0"
  "EGYPTIAN HIEROGLYPH-1372A\0"
  "EGYPTIAN HIEROGLYPH-1372B\0"
  "EGYPTIAN HIEROGLYPH-1372C\0"
  "EGYPTIAN HIEROGLYPH-1372D\0"
  "EGYPTIAN HIEROGLYPH-1372E\0"
  "EGYPTIAN HIEROGLYPH-1372F\0"
  "EGYPTIAN HIEROGLYPH-13730\0"
  "EGYPTIAN HIEROGLYPH-13731\0"
  "EGYPTIAN HIEROGLYPH-13732\0"
  "EGYPTIAN HIEROGLYPH-13733\0"
  "EGYPTIAN HIEROGLYPH-13734\0"
  "EGYPTIAN HIEROGLYPH-13735\0"
  "EGYPTIAN HIEROGLYPH-13736\0"
  "EGYPTIAN HIEROGLYPH-13737\0"
  "EGYPTIAN HIEROGLYPH-13738\0"
  "EGYPTIAN HIEROGLYPH-13739\0"
  "EGYPTIAN HIEROGLYPH-1373A\0"
  "EGYPTIAN HIEROGLYPH-1373B\0"
  "EGYPTIAN HIEROGLYPH-1373C\0"
  "EGYPTIAN HIEROGLYPH-1373D\0"
  "EGYPTIAN HIEROGLYPH-1373E\0"
  "EGYPTIAN HIEROGLYPH-1373F\0"
  "EGYPTIAN HIEROGLYPH-13740\0"
  "EGYPTIAN HIEROGLYPH-13741\0"
  "EGYPTIAN HIEROGLYPH-13742\0"
  "EGYPTIAN HIEROGLYPH-13743\0"
  "EGYPTIAN HIEROGLYPH-13744\0"
  "EGYPTIAN HIEROGLYPH-13745\0"
  "EGYPTIAN HIEROGLYPH-13746\0"
  "EGYPTIAN HIEROGLYPH-13747\0"
  "EGYPTIAN HIEROGLYPH-13748\0"
  "EGYPTIAN HIEROGLYPH-13749\0"
  "EGYPTIAN HIEROGLYPH-1374A\0"
  "EGYPTIAN HIEROGLYPH-1374B\0"
  "EGYPTIAN HIEROGLYPH-1374C\0"
  "EGYPTIAN HIEROGLYPH-1374D\0"
  "EGYPTIAN HIEROGLYPH-1374E\0"
  "EGYPTIAN HIEROGLYPH-1374F\0"
  "EGYPTIAN HIEROGLYPH-13750\0"
  "EGYPTIAN HIEROGLYPH-13751\0"
  "EGYPTIAN HIEROGLYPH-13752\0"
  "EGYPTIAN HIEROGLYPH-13753\0"
  "EGYPTIAN HIEROGLYPH-13754\0"
  "EGYPTIAN HIEROGLYPH-13755\0"
  "EGYPTIAN HIEROGLYPH-13756\0"
  "EGYPTIAN HIEROGLYPH-13757\0"
  "EGYPTIAN HIEROGLYPH-13758\0"
  "EGYPTIAN HIEROGLYPH-13759\0"
  "EGYPTIAN HIEROGLYPH-1375A\0"
  "EGYPTIAN HIEROGLYPH-1375B\0"
  "EGYPTIAN HIEROGLYPH-1375C\0"
  "EGYPTIAN HIEROGLYPH-1375D\0"
  "EGYPTIAN HIEROGLYPH-1375E\0"
  "EGYPTIAN HIEROGLYPH-1375F\0"
  "EGYPTIAN HIEROGLYPH-13760\0"
  "EGYPTIAN HIEROGLYPH-13761\0"
  "EGYPTIAN HIEROGLYPH-13762\0"
  "EGYPTIAN HIEROGLYPH-13763\0"
  "EGYPTIAN HIEROGLYPH-13764\0"
  "EGYPTIAN HIEROGLYPH-13765\0"
  "EGYPTIAN HIEROGLYPH-13766\0"
  "EGYPTIAN HIEROGLYPH-13767\0"
  "EGYPTIAN HIEROGLYPH-13768\0"
  "EGYPTIAN HIEROGLYPH-13769\0"
  "EGYPTIAN HIEROGLYPH-1376A\0"
  "EGYPTIAN HIEROGLYPH-1376B\0"
  "EGYPTIAN HIEROGLYPH-1376C\0"
  "EGYPTIAN HIEROGLYPH-1376D\0"
  "EGYPTIAN HIEROGLYPH-1376E\0"
  "EGYPTIAN HIEROGLYPH-1376F\0"
  "EGYPTIAN HIEROGLYPH-13770\0"
  "EGYPTIAN HIEROGLYPH-13771\0"
  "EGYPTIAN HIEROGLYPH-13772\0"
  "EGYPTIAN HIEROGLYPH-13773\0"
  "EGYPTIAN HIEROGLYPH-13774\0"
  "EGYPTIAN HIEROGLYPH-13775\0"
  "EGYPTIAN HIEROGLYPH-13776\0"
  "EGYPTIAN HIEROGLYPH-13777\0"
  "EGYPTIAN HIEROGLYPH-13778\0"
  "EGYPTIAN HIEROGLYPH-13779\0"
  "EGYPTIAN HIEROGLYPH-1377A\0"
  "EGYPTIAN HIEROGLYPH-1377B\0"
  "EGYPTIAN HIEROGLYPH-1377C\0"
  "EGYPTIAN HIEROGLYPH-1377D\0"
  "EGYPTIAN HIEROGLYPH-1377E\0"
  "EGYPTIAN HIEROGLYPH-1377F\0"
  "EGYPTIAN HIEROGLYPH-13780\0"
  "EGYPTIAN HIEROGLYPH-13781\0"
  "EGYPTIAN HIEROGLYPH-13782\0"
  "EGYPTIAN HIEROGLYPH-13783\0"
  "EGYPTIAN HIEROGLYPH-13784\0"
  "EGYPTIAN HIEROGLYPH-13785\0"
  "EGYPTIAN HIEROGLYPH-13786\0"
  "EGYPTIAN HIEROGLYPH-13787\0"
  "EGYPTIAN HIEROGLYPH-13788\0"
  "EGYPTIAN HIEROGLYPH-13789\0"
  "EGYPTIAN HIEROGLYPH-1378A\0"
  "EGYPTIAN HIEROGLYPH-1378B\0"
  "EGYPTIAN HIEROGLYPH-1378C\0"
  "EGYPTIAN HIEROGLYPH-1378D\0"
  "EGYPTIAN HIEROGLYPH-1378E\0"
  "EGYPTIAN HIEROGLYPH-1378F\0"
  "EGYPTIAN HIEROGLYPH-13790\0"
  "EGYPTIAN HIEROGLYPH-13791\0"
  "EGYPTIAN HIEROGLYPH-13792\0"
  "EGYPTIAN HIEROGLYPH-13793\0"
  "EGYPTIAN HIEROGLYPH-13794\0"
  "EGYPTIAN HIEROGLYPH-13795\0"
  "EGYPTIAN HIEROGLYPH-13796\0"
  "EGYPTIAN HIEROGLYPH-13797\0"
  "EGYPTIAN HIEROGLYPH-13798\0"
  "EGYPTIAN HIEROGLYPH-13799\0"
  "EGYPTIAN HIEROGLYPH-1379A\0"
  "EGYPTIAN HIEROGLYPH-1379B\0"
  "EGYPTIAN HIEROGLYPH-1379C\0"
  "EGYPTIAN HIEROGLYPH-1379D\0"
  "EGYPTIAN HIEROGLYPH-1379E\0"
  "EGYPTIAN HIEROGLYPH-1379F\0"
  "EGYPTIAN HIEROGLYPH-137A0\0"
  "EGYPTIAN HIEROGLYPH-137A1\0"
  "EGYPTIAN HIEROGLYPH-137A2\0"
  "EGYPTIAN HIEROGLYPH-137A3\0"
  "EGYPTIAN HIEROGLYPH-137A4\0"
  "EGYPTIAN HIEROGLYPH-137A5\0"
  "EGYPTIAN HIEROGLYPH-137A6\0"
  "EGYPTIAN HIEROGLYPH-137A7\0"
  "EGYPTIAN HIEROGLYPH-137A8\0"
  "EGYPTIAN HIEROGLYPH-137A9\0"
  "EGYPTIAN HIEROGLYPH-137AA\0"
  "EGYPTIAN HIEROGLYPH-137AB\0"
  "EGYPTIAN HIEROGLYPH-137AC\0"
  "EGYPTIAN HIEROGLYPH-137AD\0"
  "EGYPTIAN HIEROGLYPH-137AE\0"
  "EGYPTIAN HIEROGLYPH-137AF\0"
  "EGYPTIAN HIEROGLYPH-137B0\0"
  "EGYPTIAN HIEROGLYPH-137B1\0"
  "EGYPTIAN HIEROGLYPH-137B2\0"
  "EGYPTIAN HIEROGLYPH-137B3\0"
  "EGYPTIAN HIEROGLYPH-137B4\0"
  "EGYPTIAN HIEROGLYPH-137B5\0"
  "EGYPTIAN HIEROGLYPH-137B6\0"
  "EGYPTIAN HIEROGLYPH-137B7\0"
  "EGYPTIAN HIEROGLYPH-137B8\0"
  "EGYPTIAN HIEROGLYPH-137B9\0"
  "EGYPTIAN HIEROGLYPH-137BA\0"
  "EGYPTIAN HIEROGLYPH-137BB\0"
  "EGYPTIAN HIEROGLYPH-137BC\0"
  "EGYPTIAN HIEROGLYPH-137BD\0"
  "EGYPTIAN HIEROGLYPH-137BE\0"
  "EGYPTIAN HIEROGLYPH-137BF\0"
  "EGYPTIAN HIEROGLYPH-137C0\0"
  "EGYPTIAN HIEROGLYPH-137C1\0"
  "EGYPTIAN HIEROGLYPH-137C2\0"
  "EGYPTIAN HIEROGLYPH-137C3\0"
  "EGYPTIAN HIEROGLYPH-137C4\0"
  "EGYPTIAN HIEROGLYPH-137C5\0"
  "EGYPTIAN HIEROGLYPH-137C6\0"
  "EGYPTIAN HIEROGLYPH-137C7\0"
  "EGYPTIAN HIEROGLYPH-137C8\0"
  "EGYPTIAN HIEROGLYPH-137C9\0"
  "EGYPTIAN HIEROGLYPH-137CA\0"
  "EGYPTIAN HIEROGLYPH-137CB\0"
  "EGYPTIAN HIEROGLYPH-137CC\0"
  "EGYPTIAN HIEROGLYPH-137CD\0"
  "EGYPTIAN HIEROGLYPH-137CE\0"
  "EGYPTIAN HIEROGLYPH-137CF\0"
  "EGYPTIAN HIEROGLYPH-137D0\0"
  "EGYPTIAN HIEROGLYPH-137D1\0"
  "EGYPTIAN HIEROGLYPH-137D2\0"
  "EGYPTIAN HIEROGLYPH-137D3\0"
  "EGYPTIAN HIEROGLYPH-137D4\0"
  "EGYPTIAN HIEROGLYPH-137D5\0"
  "EGYPTIAN HIEROGLYPH-137D6\0"
  "EGYPTIAN HIEROGLYPH-137D7\0"
  "EGYPTIAN HIEROGLYPH-137D8\0"
  "EGYPTIAN HIEROGLYPH-137D9\0"
  "EGYPTIAN HIEROGLYPH-137DA\0"
  "EGYPTIAN HIEROGLYPH-137DB\0"
  "EGYPTIAN HIEROGLYPH-137DC\0"
  "EGYPTIAN HIEROGLYPH-137DD\0"
  "EGYPTIAN HIEROGLYPH-137DE\0"
  "EGYPTIAN HIEROGLYPH-137DF\0"
  "EGYPTIAN HIEROGLYPH-137E0\0"
  "EGYPTIAN HIEROGLYPH-137E1\0"
  "EGYPTIAN HIEROGLYPH-137E2\0"
  "EGYPTIAN HIEROGLYPH-137E3\0"
  "EGYPTIAN HIEROGLYPH-137E4\0"
  "EGYPTIAN HIEROGLYPH-137E5\0"
  "EGYPTIAN HIEROGLYPH-137E6\0"
  "EGYPTIAN HIEROGLYPH-137E7\0"
  "EGYPTIAN HIEROGLYPH-137E8\0"
  "EGYPTIAN HIEROGLYPH-137E9\0"
  "EGYPTIAN HIEROGLYPH-137EA\0"
  "EGYPTIAN HIEROGLYPH-137EB\0"
  "EGYPTIAN HIEROGLYPH-137EC\0"
  "EGYPTIAN HIEROGLYPH-137ED\0"
  "EGYPTIAN HIEROGLYPH-137EE\0"
  "EGYPTIAN HIEROGLYPH-137EF\0"
  "EGYPTIAN HIEROGLYPH-137F0\0"
  "EGYPTIAN HIEROGLYPH-137F1\0"
  "EGYPTIAN HIEROGLYPH-137F2\0"
  "EGYPTIAN HIEROGLYPH-137F3\0"
  "EGYPTIAN HIEROGLYPH-137F4\0"
  "EGYPTIAN HIEROGLYPH-137F5\0"
  "EGYPTIAN HIEROGLYPH-137F6\0"
  "EGYPTIAN HIEROGLYPH-137F7\0"
  "EGYPTIAN HIEROGLYPH-137F8\0"
  "EGYPTIAN HIEROGLYPH-137F9\0"
  "EGYPTIAN HIEROGLYPH-137FA\0"
  "EGYPTIAN HIEROGLYPH-137FB\0"
  "EGYPTIAN HIEROGLYPH-137FC\0"
  "EGYPTIAN HIEROGLYPH-137FD\0"
  "EGYPTIAN HIEROGLYPH-137FE\0"
  "EGYPTIAN HIEROGLYPH-137FF\0"
  "EGYPTIAN HIEROGLYPH-13800\0"
  "EGYPTIAN HIEROGLYPH-13801\0"
  "EGYPTIAN HIEROGLYPH-13802\0"
  "EGYPTIAN HIEROGLYPH-13803\0"
  "EGYPTIAN HIEROGLYPH-13804\0"
  "EGYPTIAN HIEROGLYPH-13805\0"
  "EGYPTIAN HIEROGLYPH-13806\0"
  "EGYPTIAN HIEROGLYPH-13807\0"
  "EGYPTIAN HIEROGLYPH-13808\0"
  "EGYPTIAN HIEROGLYPH-13809\0"
  "EGYPTIAN HIEROGLYPH-1380A\0"
  "EGYPTIAN HIEROGLYPH-1380B\0"
  "EGYPTIAN HIEROGLYPH-1380C\0"
  "EGYPTIAN HIEROGLYPH-1380D\0"
  "EGYPTIAN HIEROGLYPH-1380E\0"
  "EGYPTIAN HIEROGLYPH-1380F\0"
  "EGYPTIAN HIEROGLYPH-13810\0"
  "EGYPTIAN HIEROGLYPH-13811\0"
  "EGYPTIAN HIEROGLYPH-13812\0"
  "EGYPTIAN HIEROGLYPH-13813\0"
  "EGYPTIAN HIEROGLYPH-13814\0"
  "EGYPTIAN HIEROGLYPH-13815\0"
  "EGYPTIAN HIEROGLYPH-13816\0"
  "EGYPTIAN HIEROGLYPH-13817\0"
  "EGYPTIAN HIEROGLYPH-13818\0"
  "EGYPTIAN HIEROGLYPH-13819\0"
  "EGYPTIAN HIEROGLYPH-1381A\0"
  "EGYPTIAN HIEROGLYPH-1381B\0"
  "EGYPTIAN HIEROGLYPH-1381C\0"
  "EGYPTIAN HIEROGLYPH-1381D\0"
  "EGYPTIAN HIEROGLYPH-1381E\0"
  "EGYPTIAN HIEROGLYPH-1381F\0"
  "EGYPTIAN HIEROGLYPH-13820\0"
  "EGYPTIAN HIEROGLYPH-13821\0"
  "EGYPTIAN HIEROGLYPH-13822\0"
  "EGYPTIAN HIEROGLYPH-13823\0"
  "EGYPTIAN HIEROGLYPH-13824\0"
  "EGYPTIAN HIEROGLYPH-13825\0"
  "EGYPTIAN HIEROGLYPH-13826\0"
  "EGYPTIAN HIEROGLYPH-13827\0"
  "EGYPTIAN HIEROGLYPH-13828\0"
  "EGYPTIAN HIEROGLYPH-13829\0"
  "EGYPTIAN HIEROGLYPH-1382A\0"
  "EGYPTIAN HIEROGLYPH-1382B\0"
  "EGYPTIAN HIEROGLYPH-1382C\0"
  "EGYPTIAN HIEROGLYPH-1382D\0"
  "EGYPTIAN HIEROGLYPH-1382E\0"
  "EGYPTIAN HIEROGLYPH-1382F\0"
  "EGYPTIAN HIEROGLYPH-13830\0"
  "EGYPTIAN HIEROGLYPH-13831\0"
  "EGYPTIAN HIEROGLYPH-13832\0"
  "EGYPTIAN HIEROGLYPH-13833\0"
  "EGYPTIAN HIEROGLYPH-13834\0"
  "EGYPTIAN HIEROGLYPH-13835\0"
  "EGYPTIAN HIEROGLYPH-13836\0"
  "EGYPTIAN HIEROGLYPH-13837\0"
  "EGYPTIAN HIEROGLYPH-13838\0"
  "EGYPTIAN HIEROGLYPH-13839\0"
  "EGYPTIAN HIEROGLYPH-1383A\0"
  "EGYPTIAN HIEROGLYPH-1383B\0"
  "EGYPTIAN HIEROGLYPH-1383C\0"
  "EGYPTIAN HIEROGLYPH-1383D\0"
  "EGYPTIAN HIEROGLYPH-1383E\0"
  "EGYPTIAN HIEROGLYPH-1383F\0"
  "EGYPTIAN HIEROGLYPH-13840\0"
  "EGYPTIAN HIEROGLYPH-13841\0"
  "EGYPTIAN HIEROGLYPH-13842\0"
  "EGYPTIAN HIEROGLYPH-13843\0"
  "EGYPTIAN HIEROGLYPH-13844\0"
  "EGYPTIAN HIEROGLYPH-13845\0"
  "EGYPTIAN HIEROGLYPH-13846\0"
  "EGYPTIAN HIEROGLYPH-13847\0"
  "EGYPTIAN HIEROGLYPH-13848\0"
  "EGYPTIAN HIEROGLYPH-13849\0"
  "EGYPTIAN HIEROGLYPH-1384A\0"
  "EGYPTIAN HIEROGLYPH-1384B\0"
  "EGYPTIAN HIEROGLYPH-1384C\0"
  "EGYPTIAN HIEROGLYPH-1384D\0"
  "EGYPTIAN HIEROGLYPH-1384E\0"
  "EGYPTIAN HIEROGLYPH-1384F\0"
  "EGYPTIAN HIEROGLYPH-13850\0"
  "EGYPTIAN HIEROGLYPH-13851\0"
  "EGYPTIAN HIEROGLYPH-13852\0"
  "EGYPTIAN HIEROGLYPH-13853\0"
  "EGYPTIAN HIEROGLYPH-13854\0"
  "EGYPTIAN HIEROGLYPH-13855\0"
  "EGYPTIAN HIEROGLYPH-13856\0"
  "EGYPTIAN HIEROGLYPH-13857\0"
  "EGYPTIAN HIEROGLYPH-13858\0"
  "EGYPTIAN HIEROGLYPH-13859\0"
  "EGYPTIAN HIEROGLYPH-1385A\0"
  "EGYPTIAN HIEROGLYPH-1385B\0"
  "EGYPTIAN HIEROGLYPH-1385C\0"
  "EGYPTIAN HIEROGLYPH-1385D\0"
  "EGYPTIAN HIEROGLYPH-1385E\0"
  "EGYPTIAN HIEROGLYPH-1385F\0"
  "EGYPTIAN HIEROGLYPH-13860\0"
  "EGYPTIAN HIEROGLYPH-13861\0"
  "EGYPTIAN HIEROGLYPH-13862\0"
  "EGYPTIAN HIEROGLYPH-13863\0"
  "EGYPTIAN HIEROGLYPH-13864\0"
  "EGYPTIAN HIEROGLYPH-13865\0"
  "EGYPTIAN HIEROGLYPH-13866\0"
  "EGYPTIAN HIEROGLYPH-13867\0"
  "EGYPTIAN HIEROGLYPH-13868\0"
  "EGYPTIAN HIEROGLYPH-13869\0"
  "EGYPTIAN HIEROGLYPH-1386A\0"
  "EGYPTIAN HIEROGLYPH-1386B\0"
  "EGYPTIAN HIEROGLYPH-1386C\0"
  "EGYPTIAN HIEROGLYPH-1386D\0"
  "EGYPTIAN HIEROGLYPH-1386E\0"
  "EGYPTIAN HIEROGLYPH-1386F\0"
  "EGYPTIAN HIEROGLYPH-13870\0"
  "EGYPTIAN HIEROGLYPH-13871\0"
  "EGYPTIAN HIEROGLYPH-13872\0"
  "EGYPTIAN HIEROGLYPH-13873\0"
  "EGYPTIAN HIEROGLYPH-13874\0"
  "EGYPTIAN HIEROGLYPH-13875\0"
  "EGYPTIAN HIEROGLYPH-13876\0"
  "EGYPTIAN HIEROGLYPH-13877\0"
  "EGYPTIAN HIEROGLYPH-13878\0"
  "EGYPTIAN HIEROGLYPH-13879\0"
  "EGYPTIAN HIEROGLYPH-1387A\0"
  "EGYPTIAN HIEROGLYPH-1387B\0"
  "EGYPTIAN HIEROGLYPH-1387C\0"
  "EGYPTIAN HIEROGLYPH-1387D\0"
  "EGYPTIAN HIEROGLYPH-1387E\0"
  "EGYPTIAN HIEROGLYPH-1387F\0"
  "EGYPTIAN HIEROGLYPH-13880\0"
  "EGYPTIAN HIEROGLYPH-13881\0"
  "EGYPTIAN HIEROGLYPH-13882\0"
  "EGYPTIAN HIEROGLYPH-13883\0"
  "EGYPTIAN HIEROGLYPH-13884\0"
  "EGYPTIAN HIEROGLYPH-13885\0"
  "EGYPTIAN HIEROGLYPH-13886\0"
  "EGYPTIAN HIEROGLYPH-13887\0"
  "EGYPTIAN HIEROGLYPH-13888\0"
  "EGYPTIAN HIEROGLYPH-13889\0"
  "EGYPTIAN HIEROGLYPH-1388A\0"
  "EGYPTIAN HIEROGLYPH-1388B\0"
  "EGYPTIAN HIEROGLYPH-1388C\0"
  "EGYPTIAN HIEROGLYPH-1388D\0"
  "EGYPTIAN HIEROGLYPH-1388E\0"
  "EGYPTIAN HIEROGLYPH-1388F\0"
  "EGYPTIAN HIEROGLYPH-13890\0"
  "EGYPTIAN HIEROGLYPH-13891\0"
  "EGYPTIAN HIEROGLYPH-13892\0"
  "EGYPTIAN HIEROGLYPH-13893\0"
  "EGYPTIAN HIEROGLYPH-13894\0"
  "EGYPTIAN HIEROGLYPH-13895\0"
  "EGYPTIAN HIEROGLYPH-13896\0"
  "EGYPTIAN HIEROGLYPH-13897\0"
  "EGYPTIAN HIEROGLYPH-13898\0"
  "EGYPTIAN HIEROGLYPH-13899\0"
  "EGYPTIAN HIEROGLYPH-1389A\0"
  "EGYPTIAN HIEROGLYPH-1389B\0"
  "EGYPTIAN HIEROGLYPH-1389C\0"
  "EGYPTIAN HIEROGLYPH-1389D\0"
  "EGYPTIAN HIEROGLYPH-1389E\0"
  "EGYPTIAN HIEROGLYPH-1389F\0"
  "EGYPTIAN HIEROGLYPH-138A0\0"
  "EGYPTIAN HIEROGLYPH-138A1\0"
  "EGYPTIAN HIEROGLYPH-138A2\0"
  "EGYPTIAN HIEROGLYPH-138A3\0"
  "EGYPTIAN HIEROGLYPH-138A4\0"
  "EGYPTIAN HIEROGLYPH-138A5\0"
  "EGYPTIAN HIEROGLYPH-138A6\0"
  "EGYPTIAN HIEROGLYPH-138A7\0"
  "EGYPTIAN HIEROGLYPH-138A8\0"
  "EGYPTIAN HIEROGLYPH-138A9\0"
  "EGYPTIAN HIEROGLYPH-138AA\0"
  "EGYPTIAN HIEROGLYPH-138AB\0"
  "EGYPTIAN HIEROGLYPH-138AC\0"
  "EGYPTIAN HIEROGLYPH-138AD\0"
  "EGYPTIAN HIEROGLYPH-138AE\0"
  "EGYPTIAN HIEROGLYPH-138AF\0"
  "EGYPTIAN HIEROGLYPH-138B0\0"
  "EGYPTIAN HIEROGLYPH-138B1\0"
  "EGYPTIAN HIEROGLYPH-138B2\0"
  "EGYPTIAN HIEROGLYPH-138B3\0"
  "EGYPTIAN HIEROGLYPH-138B4\0"
  "EGYPTIAN HIEROGLYPH-138B5\0"
  "EGYPTIAN HIEROGLYPH-138B6\0"
  "EGYPTIAN HIEROGLYPH-138B7\0"
  "EGYPTIAN HIEROGLYPH-138B8\0"
  "EGYPTIAN HIEROGLYPH-138B9\0"
  "EGYPTIAN HIEROGLYPH-138BA\0"
  "EGYPTIAN HIEROGLYPH-138BB\0"
  "EGYPTIAN HIEROGLYPH-138BC\0"
  "EGYPTIAN HIEROGLYPH-138BD\0"
  "EGYPTIAN HIEROGLYPH-138BE\0"
  "EGYPTIAN HIEROGLYPH-138BF\0"
  "EGYPTIAN HIEROGLYPH-138C0\0"
  "EGYPTIAN HIEROGLYPH-138C1\0"
  "EGYPTIAN HIEROGLYPH-138C2\0"
  "EGYPTIAN HIEROGLYPH-138C3\0"
  "EGYPTIAN HIEROGLYPH-138C4\0"
  "EGYPTIAN HIEROGLYPH-138C5\0"
  "EGYPTIAN HIEROGLYPH-138C6\0"
  "EGYPTIAN HIEROGLYPH-138C7\0"
  "EGYPTIAN HIEROGLYPH-138C8\0"
  "EGYPTIAN HIEROGLYPH-138C9\0"
  "EGYPTIAN HIEROGLYPH-138CA\0"
  "EGYPTIAN HIEROGLYPH-138CB\0"
  "EGYPTIAN HIEROGLYPH-138CC\0"
  "EGYPTIAN HIEROGLYPH-138CD\0"
  "EGYPTIAN HIEROGLYPH-138CE\0"
  "EGYPTIAN HIEROGLYPH-138CF\0"
  "EGYPTIAN HIEROGLYPH-138D0\0"
  "EGYPTIAN HIEROGLYPH-138D1\0"
  "EGYPTIAN HIEROGLYPH-138D2\0"
  "EGYPTIAN HIEROGLYPH-138D3\0"
  "EGYPTIAN HIEROGLYPH-138D4\0"
  "EGYPTIAN HIEROGLYPH-138D5\0"
  "EGYPTIAN HIEROGLYPH-138D6\0"
  "EGYPTIAN HIEROGLYPH-138D7\0"
  "EGYPTIAN HIEROGLYPH-138D8\0"
  "EGYPTIAN HIEROGLYPH-138D9\0"
  "EGYPTIAN HIEROGLYPH-138DA\0"
  "EGYPTIAN HIEROGLYPH-138DB\0"
  "EGYPTIAN HIEROGLYPH-138DC\0"
  "EGYPTIAN HIEROGLYPH-138DD\0"
  "EGYPTIAN HIEROGLYPH-138DE\0"
  "EGYPTIAN HIEROGLYPH-138DF\0"
  "EGYPTIAN HIEROGLYPH-138E0\0"
  "EGYPTIAN HIEROGLYPH-138E1\0"
  "EGYPTIAN HIEROGLYPH-138E2\0"
  "EGYPTIAN HIEROGLYPH-138E3\0"
  "EGYPTIAN HIEROGLYPH-138E4\0"
  "EGYPTIAN HIEROGLYPH-138E5\0"
  "EGYPTIAN HIEROGLYPH-138E6\0"
  "EGYPTIAN HIEROGLYPH-138E7\0"
  "EGYPTIAN HIEROGLYPH-138E8\0"
  "EGYPTIAN HIEROGLYPH-138E9\0"
  "EGYPTIAN HIEROGLYPH-138EA\0"
  "EGYPTIAN HIEROGLYPH-138EB\0"
  "EGYPTIAN HIEROGLYPH-138EC\0"
  "EGYPTIAN HIEROGLYPH-138ED\0"
  "EGYPTIAN HIEROGLYPH-138EE\0"
  "EGYPTIAN HIEROGLYPH-138EF\0"
  "EGYPTIAN HIEROGLYPH-138F0\0"
  "EGYPTIAN HIEROGLYPH-138F1\0"
  "EGYPTIAN HIEROGLYPH-138F2\0"
  "EGYPTIAN HIEROGLYPH-138F3\0"
  "EGYPTIAN HIEROGLYPH-138F4\0"
  "EGYPTIAN HIEROGLYPH-138F5\0"
  "EGYPTIAN HIEROGLYPH-138F6\0"
  "EGYPTIAN HIEROGLYPH-138F7\0"
  "EGYPTIAN HIEROGLYPH-138F8\0"
  "EGYPTIAN HIEROGLYPH-138F9\0"
  "EGYPTIAN HIEROGLYPH-138FA\0"
  "EGYPTIAN HIEROGLYPH-138FB\0"
  "EGYPTIAN HIEROGLYPH-138FC\0"
  "EGYPTIAN HIEROGLYPH-138FD\0"
  "EGYPTIAN HIEROGLYPH-138FE\0"
  "EGYPTIAN HIEROGLYPH-138FF\0"
  "EGYPTIAN HIEROGLYPH-13900\0"
  "EGYPTIAN HIEROGLYPH-13901\0"
  "EGYPTIAN HIEROGLYPH-13902\0"
  "EGYPTIAN HIEROGLYPH-13903\0"
  "EGYPTIAN HIEROGLYPH-13904\0"
  "EGYPTIAN HIEROGLYPH-13905\0"
  "EGYPTIAN HIEROGLYPH-13906\0"
  "EGYPTIAN HIEROGLYPH-13907\0"
  "EGYPTIAN HIEROGLYPH-13908\0"
  "EGYPTIAN HIEROGLYPH-13909\0"
  "EGYPTIAN HIEROGLYPH-1390A\0"
  "EGYPTIAN HIEROGLYPH-1390B\0"
  "EGYPTIAN HIEROGLYPH-1390C\0"
  "EGYPTIAN HIEROGLYPH-1390D\0"
  "EGYPTIAN HIEROGLYPH-1390E\0"
  "EGYPTIAN HIEROGLYPH-1390F\0"
  "EGYPTIAN HIEROGLYPH-13910\0"
  "EGYPTIAN HIEROGLYPH-13911\0"
  "EGYPTIAN HIEROGLYPH-13912\0"
  "EGYPTIAN HIEROGLYPH-13913\0"
  "EGYPTIAN HIEROGLYPH-13914\0"
  "EGYPTIAN HIEROGLYPH-13915\0"
  "EGYPTIAN HIEROGLYPH-13916\0"
  "EGYPTIAN HIEROGLYPH-13917\0"
  "EGYPTIAN HIEROGLYPH-13918\0"
  "EGYPTIAN HIEROGLYPH-13919\0"
  "EGYPTIAN HIEROGLYPH-1391A\0"
  "EGYPTIAN HIEROGLYPH-1391B\0"
  "EGYPTIAN HIEROGLYPH-1391C\0"
  "EGYPTIAN HIEROGLYPH-1391D\0"
  "EGYPTIAN HIEROGLYPH-1391E\0"
  "EGYPTIAN HIEROGLYPH-1391F\0"
  "EGYPTIAN HIEROGLYPH-13920\0"
  "EGYPTIAN HIEROGLYPH-13921\0"
  "EGYPTIAN HIEROGLYPH-13922\0"
  "EGYPTIAN HIEROGLYPH-13923\0"
  "EGYPTIAN HIEROGLYPH-13924\0"
  "EGYPTIAN HIEROGLYPH-13925\0"
  "EGYPTIAN HIEROGLYPH-13926\0"
  "EGYPTIAN HIEROGLYPH-13927\0"
  "EGYPTIAN HIEROGLYPH-13928\0"
  "EGYPTIAN HIEROGLYPH-13929\0"
  "EGYPTIAN HIEROGLYPH-1392A\0"
  "EGYPTIAN HIEROGLYPH-1392B\0"
  "EGYPTIAN HIEROGLYPH-1392C\0"
  "EGYPTIAN HIEROGLYPH-1392D\0"
  "EGYPTIAN HIEROGLYPH-1392E\0"
  "EGYPTIAN HIEROGLYPH-1392F\0"
  "EGYPTIAN HIEROGLYPH-13930\0"
  "EGYPTIAN HIEROGLYPH-13931\0"
  "EGYPTIAN HIEROGLYPH-13932\0"
  "EGYPTIAN HIEROGLYPH-13933\0"
  "EGYPTIAN HIEROGLYPH-13934\0"
  "EGYPTIAN HIEROGLYPH-13935\0"
  "EGYPTIAN HIEROGLYPH-13936\0"
  "EGYPTIAN HIEROGLYPH-13937\0"
  "EGYPTIAN HIEROGLYPH-13938\0"
  "EGYPTIAN HIEROGLYPH-13939\0"
  "EGYPTIAN HIEROGLYPH-1393A\0"
  "EGYPTIAN HIEROGLYPH-1393B\0"
  "EGYPTIAN HIEROGLYPH-1393C\0"
  "EGYPTIAN HIEROGLYPH-1393D\0"
  "EGYPTIAN HIEROGLYPH-1393E\0"
  "EGYPTIAN HIEROGLYPH-1393F\0"
  "EGYPTIAN HIEROGLYPH-13940\0"
  "EGYPTIAN HIEROGLYPH-13941\0"
  "EGYPTIAN HIEROGLYPH-13942\0"
  "EGYPTIAN HIEROGLYPH-13943\0"
  "EGYPTIAN HIEROGLYPH-13944\0"
  "EGYPTIAN HIEROGLYPH-13945\0"
  "EGYPTIAN HIEROGLYPH-13946\0"
  "EGYPTIAN HIEROGLYPH-13947\0"
  "EGYPTIAN HIEROGLYPH-13948\0"
  "EGYPTIAN HIEROGLYPH-13949\0"
  "EGYPTIAN HIEROGLYPH-1394A\0"
  "EGYPTIAN HIEROGLYPH-1394B\0"
  "EGYPTIAN HIEROGLYPH-1394C\0"
  "EGYPTIAN HIEROGLYPH-1394D\0"
  "EGYPTIAN HIEROGLYPH-1394E\0"
  "EGYPTIAN HIEROGLYPH-1394F\0"
  "EGYPTIAN HIEROGLYPH-13950\0"
  "EGYPTIAN HIEROGLYPH-13951\0"
  "EGYPTIAN HIEROGLYPH-13952\0"
  "EGYPTIAN HIEROGLYPH-13953\0"
  "EGYPTIAN HIEROGLYPH-13954\0"
  "EGYPTIAN HIEROGLYPH-13955\0"
  "EGYPTIAN HIEROGLYPH-13956\0"
  "EGYPTIAN HIEROGLYPH-13957\0"
  "EGYPTIAN HIEROGLYPH-13958\0"
  "EGYPTIAN HIEROGLYPH-13959\0"
  "EGYPTIAN HIEROGLYPH-1395A\0"
  "EGYPTIAN HIEROGLYPH-1395B\0"
  "EGYPTIAN HIEROGLYPH-1395C\0"
  "EGYPTIAN HIEROGLYPH-1395D\0"
  "EGYPTIAN HIEROGLYPH-1395E\0"
  "EGYPTIAN HIEROGLYPH-1395F\0"
  "EGYPTIAN HIEROGLYPH-13960\0"
  "EGYPTIAN HIEROGLYPH-13961\0"
  "EGYPTIAN HIEROGLYPH-13962\0"
  "EGYPTIAN HIEROGLYPH-13963\0"
  "EGYPTIAN HIEROGLYPH-13964\0"
  "EGYPTIAN HIEROGLYPH-13965\0"
  "EGYPTIAN HIEROGLYPH-13966\0"
  "EGYPTIAN HIEROGLYPH-13967\0"
  "EGYPTIAN HIEROGLYPH-13968\0"
  "EGYPTIAN HIEROGLYPH-13969\0"
  "EGYPTIAN HIEROGLYPH-1396A\0"
  "EGYPTIAN HIEROGLYPH-1396B\0"
  "EGYPTIAN HIEROGLYPH-1396C\0"
  "EGYPTIAN HIEROGLYPH-1396D\0"
  "EGYPTIAN HIEROGLYPH-1396E\0"
  "EGYPTIAN HIEROGLYPH-1396F\0"
  "EGYPTIAN HIEROGLYPH-13970\0"
  "EGYPTIAN HIEROGLYPH-13971\0"
  "EGYPTIAN HIEROGLYPH-13972\0"
  "EGYPTIAN HIEROGLYPH-13973\0"
  "EGYPTIAN HIEROGLYPH-13974\0"
  "EGYPTIAN HIEROGLYPH-13975\0"
  "EGYPTIAN HIEROGLYPH-13976\0"
  "EGYPTIAN HIEROGLYPH-13977\0"
  "EGYPTIAN HIEROGLYPH-13978\0"
  "EGYPTIAN HIEROGLYPH-13979\0"
  "EGYPTIAN HIEROGLYPH-1397A\0"
  "EGYPTIAN HIEROGLYPH-1397B\0"
  "EGYPTIAN HIEROGLYPH-1397C\0"
  "EGYPTIAN HIEROGLYPH-1397D\0"
  "EGYPTIAN HIEROGLYPH-1397E\0"
  "EGYPTIAN HIEROGLYPH-1397F\0"
  "EGYPTIAN HIEROGLYPH-13980\0"
  "EGYPTIAN HIEROGLYPH-13981\0"
  "EGYPTIAN HIEROGLYPH-13982\0"
  "EGYPTIAN HIEROGLYPH-13983\0"
  "EGYPTIAN HIEROGLYPH-13984\0"
  "EGYPTIAN HIEROGLYPH-13985\0"
  "EGYPTIAN HIEROGLYPH-13986\0"
  "EGYPTIAN HIEROGLYPH-13987\0"
  "EGYPTIAN HIEROGLYPH-13988\0"
  "EGYPTIAN HIEROGLYPH-13989\0"
  "EGYPTIAN HIEROGLYPH-1398A\0"
  "EGYPTIAN HIEROGLYPH-1398B\0"
  "EGYPTIAN HIEROGLYPH-1398C\0"
  "EGYPTIAN HIEROGLYPH-1398D\0"
  "EGYPTIAN HIEROGLYPH-1398E\0"
  "EGYPTIAN HIEROGLYPH-1398F\0"
  "EGYPTIAN HIEROGLYPH-13990\0"
  "EGYPTIAN HIEROGLYPH-13991\0"
  "EGYPTIAN HIEROGLYPH-13992\0"
  "EGYPTIAN HIEROGLYPH-13993\0"
  "EGYPTIAN HIEROGLYPH-13994\0"
  "EGYPTIAN HIEROGLYPH-13995\0"
  "EGYPTIAN HIEROGLYPH-13996\0"
  "EGYPTIAN HIEROGLYPH-13997\0"
  "EGYPTIAN HIEROGLYPH-13998\0"
  "EGYPTIAN HIEROGLYPH-13999\0"
  "EGYPTIAN HIEROGLYPH-1399A\0"
  "EGYPTIAN HIEROGLYPH-1399B\0"
  "EGYPTIAN HIEROGLYPH-1399C\0"
  "EGYPTIAN HIEROGLYPH-1399D\0"
  "EGYPTIAN HIEROGLYPH-1399E\0"
  "EGYPTIAN HIEROGLYPH-1399F\0"
  "EGYPTIAN HIEROGLYPH-139A0\0"
  "EGYPTIAN HIEROGLYPH-139A1\0"
  "EGYPTIAN HIEROGLYPH-139A2\0"
  "EGYPTIAN HIEROGLYPH-139A3\0"
  "EGYPTIAN HIEROGLYPH-139A4\0"
  "EGYPTIAN HIEROGLYPH-139A5\0"
  "EGYPTIAN HIEROGLYPH-139A6\0"
  "EGYPTIAN HIEROGLYPH-139A7\0"
  "EGYPTIAN HIEROGLYPH-139A8\0"
  "EGYPTIAN HIEROGLYPH-139A9\0"
  "EGYPTIAN HIEROGLYPH-139AA\0"
  "EGYPTIAN HIEROGLYPH-139AB\0"
  "EGYPTIAN HIEROGLYPH-139AC\0"
  "EGYPTIAN HIEROGLYPH-139AD\0"
  "EGYPTIAN HIEROGLYPH-139AE\0"
  "EGYPTIAN HIEROGLYPH-139AF\0"
  "EGYPTIAN HIEROGLYPH-139B0\0"
  "EGYPTIAN HIEROGLYPH-139B1\0"
  "EGYPTIAN HIEROGLYPH-139B2\0"
  "EGYPTIAN HIEROGLYPH-139B3\0"
  "EGYPTIAN HIEROGLYPH-139B4\0"
  "EGYPTIAN HIEROGLYPH-139B5\0"
  "EGYPTIAN HIEROGLYPH-139B6\0"
  "EGYPTIAN HIEROGLYPH-139B7\0"
  "EGYPTIAN HIEROGLYPH-139B8\0"
  "EGYPTIAN HIEROGLYPH-139B9\0"
  "EGYPTIAN HIEROGLYPH-139BA\0"
  "EGYPTIAN HIEROGLYPH-139BB\0"
  "EGYPTIAN HIEROGLYPH-139BC\0"
  "EGYPTIAN HIEROGLYPH-139BD\0"
  "EGYPTIAN HIEROGLYPH-139BE\0"
  "EGYPTIAN HIEROGLYPH-139BF\0"
  "EGYPTIAN HIEROGLYPH-139C0\0"
  "EGYPTIAN HIEROGLYPH-139C1\0"
  "EGYPTIAN HIEROGLYPH-139C2\0"
  "EGYPTIAN HIEROGLYPH-139C3\0"
  "EGYPTIAN HIEROGLYPH-139C4\0"
  "EGYPTIAN HIEROGLYPH-139C5\0"
  "EGYPTIAN HIEROGLYPH-139C6\0"
  "EGYPTIAN HIEROGLYPH-139C7\0"
  "EGYPTIAN HIEROGLYPH-139C8\0"
  "EGYPTIAN HIEROGLYPH-139C9\0"
  "EGYPTIAN HIEROGLYPH-139CA\0"
  "EGYPTIAN HIEROGLYPH-139CB\0"
  "EGYPTIAN HIEROGLYPH-139CC\0"
  "EGYPTIAN HIEROGLYPH-139CD\0"
  "EGYPTIAN HIEROGLYPH-139CE\0"
  "EGYPTIAN HIEROGLYPH-139CF\0"
  "EGYPTIAN HIEROGLYPH-139D0\0"
  "EGYPTIAN HIEROGLYPH-139D1\0"
  "EGYPTIAN HIEROGLYPH-139D2\0"
  "EGYPTIAN HIEROGLYPH-139D3\0"
  "EGYPTIAN HIEROGLYPH-139D4\0"
  "EGYPTIAN HIEROGLYPH-139D5\0"
  "EGYPTIAN HIEROGLYPH-139D6\0"
  "EGYPTIAN HIEROGLYPH-139D7\0"
  "EGYPTIAN HIEROGLYPH-139D8\0"
  "EGYPTIAN HIEROGLYPH-139D9\0"
  "EGYPTIAN HIEROGLYPH-139DA\0"
  "EGYPTIAN HIEROGLYPH-139DB\0"
  "EGYPTIAN HIEROGLYPH-139DC\0"
  "EGYPTIAN HIEROGLYPH-139DD\0"
  "EGYPTIAN HIEROGLYPH-139DE\0"
  "EGYPTIAN HIEROGLYPH-139DF\0"
  "EGYPTIAN HIEROGLYPH-139E0\0"
  "EGYPTIAN HIEROGLYPH-139E1\0"
  "EGYPTIAN HIEROGLYPH-139E2\0"
  "EGYPTIAN HIEROGLYPH-139E3\0"
  "EGYPTIAN HIEROGLYPH-139E4\0"
  "EGYPTIAN HIEROGLYPH-139E5\0"
  "EGYPTIAN HIEROGLYPH-139E6\0"
  "EGYPTIAN HIEROGLYPH-139E7\0"
  "EGYPTIAN HIEROGLYPH-139E8\0"
  "EGYPTIAN HIEROGLYPH-139E9\0"
  "EGYPTIAN HIEROGLYPH-139EA\0"
  "EGYPTIAN HIEROGLYPH-139EB\0"
  "EGYPTIAN HIEROGLYPH-139EC\0"
  "EGYPTIAN HIEROGLYPH-139ED\0"
  "EGYPTIAN HIEROGLYPH-139EE\0"
  "EGYPTIAN HIEROGLYPH-139EF\0"
  "EGYPTIAN HIEROGLYPH-139F0\0"
  "EGYPTIAN HIEROGLYPH-139F1\0"
  "EGYPTIAN HIEROGLYPH-139F2\0"
  "EGYPTIAN HIEROGLYPH-139F3\0"
  "EGYPTIAN HIEROGLYPH-139F4\0"
  "EGYPTIAN HIEROGLYPH-139F5\0"
  "EGYPTIAN HIEROGLYPH-139F6\0"
  "EGYPTIAN HIEROGLYPH-139F7\0"
  "EGYPTIAN HIEROGLYPH-139F8\0"
  "EGYPTIAN HIEROGLYPH-139F9\0"
  "EGYPTIAN HIEROGLYPH-139FA\0"
  "EGYPTIAN HIEROGLYPH-139FB\0"
  "EGYPTIAN HIEROGLYPH-139FC\0"
  "EGYPTIAN HIEROGLYPH-139FD\0"
  "EGYPTIAN HIEROGLYPH-139FE\0"
  "EGYPTIAN HIEROGLYPH-139FF\0"
  "EGYPTIAN HIEROGLYPH-13A00\0"
  "EGYPTIAN HIEROGLYPH-13A01\0"
  "EGYPTIAN HIEROGLYPH-13A02\0"
  "EGYPTIAN HIEROGLYPH-13A03\0"
  "EGYPTIAN HIEROGLYPH-13A04\0"
  "EGYPTIAN HIEROGLYPH-13A05\0"
  "EGYPTIAN HIEROGLYPH-13A06\0"
  "EGYPTIAN HIEROGLYPH-13A07\0"
  "EGYPTIAN HIEROGLYPH-13A08\0"
  "EGYPTIAN HIEROGLYPH-13A09\0"
  "EGYPTIAN HIEROGLYPH-13A0A\0"
  "EGYPTIAN HIEROGLYPH-13A0B\0"
  "EGYPTIAN HIEROGLYPH-13A0C\0"
  "EGYPTIAN HIEROGLYPH-13A0D\0"
  "EGYPTIAN HIEROGLYPH-13A0E\0"
  "EGYPTIAN HIEROGLYPH-13A0F\0"
  "EGYPTIAN HIEROGLYPH-13A10\0"
  "EGYPTIAN HIEROGLYPH-13A11\0"
  "EGYPTIAN HIEROGLYPH-13A12\0"
  "EGYPTIAN HIEROGLYPH-13A13\0"
  "EGYPTIAN HIEROGLYPH-13A14\0"
  "EGYPTIAN HIEROGLYPH-13A15\0"
  "EGYPTIAN HIEROGLYPH-13A16\0"
  "EGYPTIAN HIEROGLYPH-13A17\0"
  "EGYPTIAN HIEROGLYPH-13A18\0"
  "EGYPTIAN HIEROGLYPH-13A19\0"
  "EGYPTIAN HIEROGLYPH-13A1A\0"
  "EGYPTIAN HIEROGLYPH-13A1B\0"
  "EGYPTIAN HIEROGLYPH-13A1C\0"
  "EGYPTIAN HIEROGLYPH-13A1D\0"
  "EGYPTIAN HIEROGLYPH-13A1E\0"
  "EGYPTIAN HIEROGLYPH-13A1F\0"
  "EGYPTIAN HIEROGLYPH-13A20\0"
  "EGYPTIAN HIEROGLYPH-13A21\0"
  "EGYPTIAN HIEROGLYPH-13A22\0"
  "EGYPTIAN HIEROGLYPH-13A23\0"
  "EGYPTIAN HIEROGLYPH-13A24\0"
  "EGYPTIAN HIEROGLYPH-13A25\0"
  "EGYPTIAN HIEROGLYPH-13A26\0"
  "EGYPTIAN HIEROGLYPH-13A27\0"
  "EGYPTIAN HIEROGLYPH-13A28\0"
  "EGYPTIAN HIEROGLYPH-13A29\0"
  "EGYPTIAN HIEROGLYPH-13A2A\0"
  "EGYPTIAN HIEROGLYPH-13A2B\0"
  "EGYPTIAN HIEROGLYPH-13A2C\0"
  "EGYPTIAN HIEROGLYPH-13A2D\0"
  "EGYPTIAN HIEROGLYPH-13A2E\0"
  "EGYPTIAN HIEROGLYPH-13A2F\0"
  "EGYPTIAN HIEROGLYPH-13A30\0"
  "EGYPTIAN HIEROGLYPH-13A31\0"
  "EGYPTIAN HIEROGLYPH-13A32\0"
  "EGYPTIAN HIEROGLYPH-13A33\0"
  "EGYPTIAN HIEROGLYPH-13A34\0"
  "EGYPTIAN HIEROGLYPH-13A35\0"
  "EGYPTIAN HIEROGLYPH-13A36\0"
  "EGYPTIAN HIEROGLYPH-13A37\0"
  "EGYPTIAN HIEROGLYPH-13A38\0"
  "EGYPTIAN HIEROGLYPH-13A39\0"
  "EGYPTIAN HIEROGLYPH-13A3A\0"
  "EGYPTIAN HIEROGLYPH-13A3B\0"
  "EGYPTIAN HIEROGLYPH-13A3C\0"
  "EGYPTIAN HIEROGLYPH-13A3D\0"
  "EGYPTIAN HIEROGLYPH-13A3E\0"
  "EGYPTIAN HIEROGLYPH-13A3F\0"
  "EGYPTIAN HIEROGLYPH-13A40\0"
  "EGYPTIAN HIEROGLYPH-13A41\0"
  "EGYPTIAN HIEROGLYPH-13A42\0"
  "EGYPTIAN HIEROGLYPH-13A43\0"
  "EGYPTIAN HIEROGLYPH-13A44\0"
  "EGYPTIAN HIEROGLYPH-13A45\0"
  "EGYPTIAN HIEROGLYPH-13A46\0"
  "EGYPTIAN HIEROGLYPH-13A47\0"
  "EGYPTIAN HIEROGLYPH-13A48\0"
  "EGYPTIAN HIEROGLYPH-13A49\0"
  "EGYPTIAN HIEROGLYPH-13A4A\0"
  "EGYPTIAN HIEROGLYPH-13A4B\0"
  "EGYPTIAN HIEROGLYPH-13A4C\0"
  "EGYPTIAN HIEROGLYPH-13A4D\0"
  "EGYPTIAN HIEROGLYPH-13A4E\0"
  "EGYPTIAN HIEROGLYPH-13A4F\0"
  "EGYPTIAN HIEROGLYPH-13A50\0"
  "EGYPTIAN HIEROGLYPH-13A51\0"
  "EGYPTIAN HIEROGLYPH-13A52\0"
  "EGYPTIAN HIEROGLYPH-13A53\0"
  "EGYPTIAN HIEROGLYPH-13A54\0"
  "EGYPTIAN HIEROGLYPH-13A55\0"
  "EGYPTIAN HIEROGLYPH-13A56\0"
  "EGYPTIAN HIEROGLYPH-13A57\0"
  "EGYPTIAN HIEROGLYPH-13A58\0"
  "EGYPTIAN HIEROGLYPH-13A59\0"
  "EGYPTIAN HIEROGLYPH-13A5A\0"
  "EGYPTIAN HIEROGLYPH-13A5B\0"
  "EGYPTIAN HIEROGLYPH-13A5C\0"
  "EGYPTIAN HIEROGLYPH-13A5D\0"
  "EGYPTIAN HIEROGLYPH-13A5E\0"
  "EGYPTIAN HIEROGLYPH-13A5F\0"
  "EGYPTIAN HIEROGLYPH-13A60\0"
  "EGYPTIAN HIEROGLYPH-13A61\0"
  "EGYPTIAN HIEROGLYPH-13A62\0"
  "EGYPTIAN HIEROGLYPH-13A63\0"
  "EGYPTIAN HIEROGLYPH-13A64\0"
  "EGYPTIAN HIEROGLYPH-13A65\0"
  "EGYPTIAN HIEROGLYPH-13A66\0"
  "EGYPTIAN HIEROGLYPH-13A67\0"
  "EGYPTIAN HIEROGLYPH-13A68\0"
  "EGYPTIAN HIEROGLYPH-13A69\0"
  "EGYPTIAN HIEROGLYPH-13A6A\0"
  "EGYPTIAN HIEROGLYPH-13A6B\0"
  "EGYPTIAN HIEROGLYPH-13A6C\0"
  "EGYPTIAN HIEROGLYPH-13A6D\0"
  "EGYPTIAN HIEROGLYPH-13A6E\0"
  "EGYPTIAN HIEROGLYPH-13A6F\0"
  "EGYPTIAN HIEROGLYPH-13A70\0"
  "EGYPTIAN HIEROGLYPH-13A71\0"
  "EGYPTIAN HIEROGLYPH-13A72\0"
  "EGYPTIAN HIEROGLYPH-13A73\0"
  "EGYPTIAN HIEROGLYPH-13A74\0"
  "EGYPTIAN HIEROGLYPH-13A75\0"
  "EGYPTIAN HIEROGLYPH-13A76\0"
  "EGYPTIAN HIEROGLYPH-13A77\0"
  "EGYPTIAN HIEROGLYPH-13A78\0"
  "EGYPTIAN HIEROGLYPH-13A79\0"
  "EGYPTIAN HIEROGLYPH-13A7A\0"
  "EGYPTIAN HIEROGLYPH-13A7B\0"
  "EGYPTIAN HIEROGLYPH-13A7C\0"
  "EGYPTIAN HIEROGLYPH-13A7D\0"
  "EGYPTIAN HIEROGLYPH-13A7E\0"
  "EGYPTIAN HIEROGLYPH-13A7F\0"
  "EGYPTIAN HIEROGLYPH-13A80\0"
  "EGYPTIAN HIEROGLYPH-13A81\0"
  "EGYPTIAN HIEROGLYPH-13A82\0"
  "EGYPTIAN HIEROGLYPH-13A83\0"
  "EGYPTIAN HIEROGLYPH-13A84\0"
  "EGYPTIAN HIEROGLYPH-13A85\0"
  "EGYPTIAN HIEROGLYPH-13A86\0"
  "EGYPTIAN HIEROGLYPH-13A87\0"
  "EGYPTIAN HIEROGLYPH-13A88\0"
  "EGYPTIAN HIEROGLYPH-13A89\0"
  "EGYPTIAN HIEROGLYPH-13A8A\0"
  "EGYPTIAN HIEROGLYPH-13A8B\0"
  "EGYPTIAN HIEROGLYPH-13A8C\0"
  "EGYPTIAN HIEROGLYPH-13A8D\0"
  "EGYPTIAN HIEROGLYPH-13A8E\0"
  "EGYPTIAN HIEROGLYPH-13A8F\0"
  "EGYPTIAN HIEROGLYPH-13A90\0"
  "EGYPTIAN HIEROGLYPH-13A91\0"
  "EGYPTIAN HIEROGLYPH-13A92\0"
  "EGYPTIAN HIEROGLYPH-13A93\0"
  "EGYPTIAN HIEROGLYPH-13A94\0"
  "EGYPTIAN HIEROGLYPH-13A95\0"
  "EGYPTIAN HIEROGLYPH-13A96\0"
  "EGYPTIAN HIEROGLYPH-13A97\0"
  "EGYPTIAN HIEROGLYPH-13A98\0"
  "EGYPTIAN HIEROGLYPH-13A99\0"
  "EGYPTIAN HIEROGLYPH-13A9A\0"
  "EGYPTIAN HIEROGLYPH-13A9B\0"
  "EGYPTIAN HIEROGLYPH-13A9C\0"
  "EGYPTIAN HIEROGLYPH-13A9D\0"
  "EGYPTIAN HIEROGLYPH-13A9E\0"
  "EGYPTIAN HIEROGLYPH-13A9F\0"
  "EGYPTIAN HIEROGLYPH-13AA0\0"
  "EGYPTIAN HIEROGLYPH-13AA1\0"
  "EGYPTIAN HIEROGLYPH-13AA2\0"
  "EGYPTIAN HIEROGLYPH-13AA3\0"
  "EGYPTIAN HIEROGLYPH-13AA4\0"
  "EGYPTIAN HIEROGLYPH-13AA5\0"
  "EGYPTIAN HIEROGLYPH-13AA6\0"
  "EGYPTIAN HIEROGLYPH-13AA7\0"
  "EGYPTIAN HIEROGLYPH-13AA8\0"
  "EGYPTIAN HIEROGLYPH-13AA9\0"
  "EGYPTIAN HIEROGLYPH-13AAA\0"
  "EGYPTIAN HIEROGLYPH-13AAB\0"
  "EGYPTIAN HIEROGLYPH-13AAC\0"
  "EGYPTIAN HIEROGLYPH-13AAD\0"
  "EGYPTIAN HIEROGLYPH-13AAE\0"
  "EGYPTIAN HIEROGLYPH-13AAF\0"
  "EGYPTIAN HIEROGLYPH-13AB0\0"
  "EGYPTIAN HIEROGLYPH-13AB1\0"
  "EGYPTIAN HIEROGLYPH-13AB2\0"
  "EGYPTIAN HIEROGLYPH-13AB3\0"
  "EGYPTIAN HIEROGLYPH-13AB4\0"
  "EGYPTIAN HIEROGLYPH-13AB5\0"
  "EGYPTIAN HIEROGLYPH-13AB6\0"
  "EGYPTIAN HIEROGLYPH-13AB7\0"
  "EGYPTIAN HIEROGLYPH-13AB8\0"
  "EGYPTIAN HIEROGLYPH-13AB9\0"
  "EGYPTIAN HIEROGLYPH-13ABA\0"
  "EGYPTIAN HIEROGLYPH-13ABB\0"
  "EGYPTIAN HIEROGLYPH-13ABC\0"
  "EGYPTIAN HIEROGLYPH-13ABD\0"
  "EGYPTIAN HIEROGLYPH-13ABE\0"
  "EGYPTIAN HIEROGLYPH-13ABF\0"
  "EGYPTIAN HIEROGLYPH-13AC0\0"
  "EGYPTIAN HIEROGLYPH-13AC1\0"
  "EGYPTIAN HIEROGLYPH-13AC2\0"
  "EGYPTIAN HIEROGLYPH-13AC3\0"
  "EGYPTIAN HIEROGLYPH-13AC4\0"
  "EGYPTIAN HIEROGLYPH-13AC5\0"
  "EGYPTIAN HIEROGLYPH-13AC6\0"
  "EGYPTIAN HIEROGLYPH-13AC7\0"
  "EGYPTIAN HIEROGLYPH-13AC8\0"
  "EGYPTIAN HIEROGLYPH-13AC9\0"
  "EGYPTIAN HIEROGLYPH-13ACA\0"
  "EGYPTIAN HIEROGLYPH-13ACB\0"
  "EGYPTIAN HIEROGLYPH-13ACC\0"
  "EGYPTIAN HIEROGLYPH-13ACD\0"
  "EGYPTIAN HIEROGLYPH-13ACE\0"
  "EGYPTIAN HIEROGLYPH-13ACF\0"
  "EGYPTIAN HIEROGLYPH-13AD0\0"
  "EGYPTIAN HIEROGLYPH-13AD1\0"
  "EGYPTIAN HIEROGLYPH-13AD2\0"
  "EGYPTIAN HIEROGLYPH-13AD3\0"
  "EGYPTIAN HIEROGLYPH-13AD4\0"
  "EGYPTIAN HIEROGLYPH-13AD5\0"
  "EGYPTIAN HIEROGLYPH-13AD6\0"
  "EGYPTIAN HIEROGLYPH-13AD7\0"
  "EGYPTIAN HIEROGLYPH-13AD8\0"
  "EGYPTIAN HIEROGLYPH-13AD9\0"
  "EGYPTIAN HIEROGLYPH-13ADA\0"
  "EGYPTIAN HIEROGLYPH-13ADB\0"
  "EGYPTIAN HIEROGLYPH-13ADC\0"
  "EGYPTIAN HIEROGLYPH-13ADD\0"
  "EGYPTIAN HIEROGLYPH-13ADE\0"
  "EGYPTIAN HIEROGLYPH-13ADF\0"
  "EGYPTIAN HIEROGLYPH-13AE0\0"
  "EGYPTIAN HIEROGLYPH-13AE1\0"
  "EGYPTIAN HIEROGLYPH-13AE2\0"
  "EGYPTIAN HIEROGLYPH-13AE3\0"
  "EGYPTIAN HIEROGLYPH-13AE4\0"
  "EGYPTIAN HIEROGLYPH-13AE5\0"
  "EGYPTIAN HIEROGLYPH-13AE6\0"
  "EGYPTIAN HIEROGLYPH-13AE7\0"
  "EGYPTIAN HIEROGLYPH-13AE8\0"
  "EGYPTIAN HIEROGLYPH-13AE9\0"
  "EGYPTIAN HIEROGLYPH-13AEA\0"
  "EGYPTIAN HIEROGLYPH-13AEB\0"
  "EGYPTIAN HIEROGLYPH-13AEC\0"
  "EGYPTIAN HIEROGLYPH-13AED\0"
  "EGYPTIAN HIEROGLYPH-13AEE\0"
  "EGYPTIAN HIEROGLYPH-13AEF\0"
  "EGYPTIAN HIEROGLYPH-13AF0\0"
  "EGYPTIAN HIEROGLYPH-13AF1\0"
  "EGYPTIAN HIEROGLYPH-13AF2\0"
  "EGYPTIAN HIEROGLYPH-13AF3\0"
  "EGYPTIAN HIEROGLYPH-13AF4\0"
  "EGYPTIAN HIEROGLYPH-13AF5\0"
  "EGYPTIAN HIEROGLYPH-13AF6\0"
  "EGYPTIAN HIEROGLYPH-13AF7\0"
  "EGYPTIAN HIEROGLYPH-13AF8\0"
  "EGYPTIAN HIEROGLYPH-13AF9\0"
  "EGYPTIAN HIEROGLYPH-13AFA\0"
  "EGYPTIAN HIEROGLYPH-13AFB\0"
  "EGYPTIAN HIEROGLYPH-13AFC\0"
  "EGYPTIAN HIEROGLYPH-13AFD\0"
  "EGYPTIAN HIEROGLYPH-13AFE\0"
  "EGYPTIAN HIEROGLYPH-13AFF\0"
  "EGYPTIAN HIEROGLYPH-13B00\0"
  "EGYPTIAN HIEROGLYPH-13B01\0"
  "EGYPTIAN HIEROGLYPH-13B02\0"
  "EGYPTIAN HIEROGLYPH-13B03\0"
  "EGYPTIAN HIEROGLYPH-13B04\0"
  "EGYPTIAN HIEROGLYPH-13B05\0"
  "EGYPTIAN HIEROGLYPH-13B06\0"
  "EGYPTIAN HIEROGLYPH-13B07\0"
  "EGYPTIAN HIEROGLYPH-13B08\0"
  "EGYPTIAN HIEROGLYPH-13B09\0"
  "EGYPTIAN HIEROGLYPH-13B0A\0"
  "EGYPTIAN HIEROGLYPH-13B0B\0"
  "EGYPTIAN HIEROGLYPH-13B0C\0"
  "EGYPTIAN HIEROGLYPH-13B0D\0"
  "EGYPTIAN HIEROGLYPH-13B0E\0"
  "EGYPTIAN HIEROGLYPH-13B0F\0"
  "EGYPTIAN HIEROGLYPH-13B10\0"
  "EGYPTIAN HIEROGLYPH-13B11\0"
  "EGYPTIAN HIEROGLYPH-13B12\0"
  "EGYPTIAN HIEROGLYPH-13B13\0"
  "EGYPTIAN HIEROGLYPH-13B14\0"
  "EGYPTIAN HIEROGLYPH-13B15\0"
  "EGYPTIAN HIEROGLYPH-13B16\0"
  "EGYPTIAN HIEROGLYPH-13B17\0"
  "EGYPTIAN HIEROGLYPH-13B18\0"
  "EGYPTIAN HIEROGLYPH-13B19\0"
  "EGYPTIAN HIEROGLYPH-13B1A\0"
  "EGYPTIAN HIEROGLYPH-13B1B\0"
  "EGYPTIAN HIEROGLYPH-13B1C\0"
  "EGYPTIAN HIEROGLYPH-13B1D\0"
  "EGYPTIAN HIEROGLYPH-13B1E\0"
  "EGYPTIAN HIEROGLYPH-13B1F\0"
  "EGYPTIAN HIEROGLYPH-13B20\0"
  "EGYPTIAN HIEROGLYPH-13B21\0"
  "EGYPTIAN HIEROGLYPH-13B22\0"
  "EGYPTIAN HIEROGLYPH-13B23\0"
  "EGYPTIAN HIEROGLYPH-13B24\0"
  "EGYPTIAN HIEROGLYPH-13B25\0"
  "EGYPTIAN HIEROGLYPH-13B26\0"
  "EGYPTIAN HIEROGLYPH-13B27\0"
  "EGYPTIAN HIEROGLYPH-13B28\0"
  "EGYPTIAN HIEROGLYPH-13B29\0"
  "EGYPTIAN HIEROGLYPH-13B2A\0"
  "EGYPTIAN HIEROGLYPH-13B2B\0"
  "EGYPTIAN HIEROGLYPH-13B2C\0"
  "EGYPTIAN HIEROGLYPH-13B2D\0"
  "EGYPTIAN HIEROGLYPH-13B2E\0"
  "EGYPTIAN HIEROGLYPH-13B2F\0"
  "EGYPTIAN HIEROGLYPH-13B30\0"
  "EGYPTIAN HIEROGLYPH-13B31\0"
  "EGYPTIAN HIEROGLYPH-13B32\0"
  "EGYPTIAN HIEROGLYPH-13B33\0"
  "EGYPTIAN HIEROGLYPH-13B34\0"
  "EGYPTIAN HIEROGLYPH-13B35\0"
  "EGYPTIAN HIEROGLYPH-13B36\0"
  "EGYPTIAN HIEROGLYPH-13B37\0"
  "EGYPTIAN HIEROGLYPH-13B38\0"
  "EGYPTIAN HIEROGLYPH-13B39\0"
  "EGYPTIAN HIEROGLYPH-13B3A\0"
  "EGYPTIAN HIEROGLYPH-13B3B\0"
  "EGYPTIAN HIEROGLYPH-13B3C\0"
  "EGYPTIAN HIEROGLYPH-13B3D\0"
  "EGYPTIAN HIEROGLYPH-13B3E\0"
  "EGYPTIAN HIEROGLYPH-13B3F\0"
  "EGYPTIAN HIEROGLYPH-13B40\0"
  "EGYPTIAN HIEROGLYPH-13B41\0"
  "EGYPTIAN HIEROGLYPH-13B42\0"
  "EGYPTIAN HIEROGLYPH-13B43\0"
  "EGYPTIAN HIEROGLYPH-13B44\0"
  "EGYPTIAN HIEROGLYPH-13B45\0"
  "EGYPTIAN HIEROGLYPH-13B46\0"
  "EGYPTIAN HIEROGLYPH-13B47\0"
  "EGYPTIAN HIEROGLYPH-13B48\0"
  "EGYPTIAN HIEROGLYPH-13B49\0"
  "EGYPTIAN HIEROGLYPH-13B4A\0"
  "EGYPTIAN HIEROGLYPH-13B4B\0"
  "EGYPTIAN HIEROGLYPH-13B4C\0"
  "EGYPTIAN HIEROGLYPH-13B4D\0"
  "EGYPTIAN HIEROGLYPH-13B4E\0"
  "EGYPTIAN HIEROGLYPH-13B4F\0"
  "EGYPTIAN HIEROGLYPH-13B50\0"
  "EGYPTIAN HIEROGLYPH-13B51\0"
  "EGYPTIAN HIEROGLYPH-13B52\0"
  "EGYPTIAN HIEROGLYPH-13B53\0"
  "EGYPTIAN HIEROGLYPH-13B54\0"
  "EGYPTIAN HIEROGLYPH-13B55\0"
  "EGYPTIAN HIEROGLYPH-13B56\0"
  "EGYPTIAN HIEROGLYPH-13B57\0"
  "EGYPTIAN HIEROGLYPH-13B58\0"
  "EGYPTIAN HIEROGLYPH-13B59\0"
  "EGYPTIAN HIEROGLYPH-13B5A\0"
  "EGYPTIAN HIEROGLYPH-13B5B\0"
  "EGYPTIAN HIEROGLYPH-13B5C\0"
  "EGYPTIAN HIEROGLYPH-13B5D\0"
  "EGYPTIAN HIEROGLYPH-13B5E\0"
  "EGYPTIAN HIEROGLYPH-13B5F\0"
  "EGYPTIAN HIEROGLYPH-13B60\0"
  "EGYPTIAN HIEROGLYPH-13B61\0"
  "EGYPTIAN HIEROGLYPH-13B62\0"
  "EGYPTIAN HIEROGLYPH-13B63\0"
  "EGYPTIAN HIEROGLYPH-13B64\0"
  "EGYPTIAN HIEROGLYPH-13B65\0"
  "EGYPTIAN HIEROGLYPH-13B66\0"
  "EGYPTIAN HIEROGLYPH-13B67\0"
  "EGYPTIAN HIEROGLYPH-13B68\0"
  "EGYPTIAN HIEROGLYPH-13B69\0"
  "EGYPTIAN HIEROGLYPH-13B6A\0"
  "EGYPTIAN HIEROGLYPH-13B6B\0"
  "EGYPTIAN HIEROGLYPH-13B6C\0"
  "EGYPTIAN HIEROGLYPH-13B6D\0"
  "EGYPTIAN HIEROGLYPH-13B6E\0"
  "EGYPTIAN HIEROGLYPH-13B6F\0"
  "EGYPTIAN HIEROGLYPH-13B70\0"
  "EGYPTIAN HIEROGLYPH-13B71\0"
  "EGYPTIAN HIEROGLYPH-13B72\0"
  "EGYPTIAN HIEROGLYPH-13B73\0"
  "EGYPTIAN HIEROGLYPH-13B74\0"
  "EGYPTIAN HIEROGLYPH-13B75\0"
  "EGYPTIAN HIEROGLYPH-13B76\0"
  "EGYPTIAN HIEROGLYPH-13B77\0"
  "EGYPTIAN HIEROGLYPH-13B78\0"
  "EGYPTIAN HIEROGLYPH-13B79\0"
  "EGYPTIAN HIEROGLYPH-13B7A\0"
  "EGYPTIAN HIEROGLYPH-13B7B\0"
  "EGYPTIAN HIEROGLYPH-13B7C\0"
  "EGYPTIAN HIEROGLYPH-13B7D\0"
  "EGYPTIAN HIEROGLYPH-13B7E\0"
  "EGYPTIAN HIEROGLYPH-13B7F\0"
  "EGYPTIAN HIEROGLYPH-13B80\0"
  "EGYPTIAN HIEROGLYPH-13B81\0"
  "EGYPTIAN HIEROGLYPH-13B82\0"
  "EGYPTIAN HIEROGLYPH-13B83\0"
  "EGYPTIAN HIEROGLYPH-13B84\0"
  "EGYPTIAN HIEROGLYPH-13B85\0"
  "EGYPTIAN HIEROGLYPH-13B86\0"
  "EGYPTIAN HIEROGLYPH-13B87\0"
  "EGYPTIAN HIEROGLYPH-13B88\0"
  "EGYPTIAN HIEROGLYPH-13B89\0"
  "EGYPTIAN HIEROGLYPH-13B8A\0"
  "EGYPTIAN HIEROGLYPH-13B8B\0"
  "EGYPTIAN HIEROGLYPH-13B8C\0"
  "EGYPTIAN HIEROGLYPH-13B8D\0"
  "EGYPTIAN HIEROGLYPH-13B8E\0"
  "EGYPTIAN HIEROGLYPH-13B8F\0"
  "EGYPTIAN HIEROGLYPH-13B90\0"
  "EGYPTIAN HIEROGLYPH-13B91\0"
  "EGYPTIAN HIEROGLYPH-13B92\0"
  "EGYPTIAN HIEROGLYPH-13B93\0"
  "EGYPTIAN HIEROGLYPH-13B94\0"
  "EGYPTIAN HIEROGLYPH-13B95\0"
  "EGYPTIAN HIEROGLYPH-13B96\0"
  "EGYPTIAN HIEROGLYPH-13B97\0"
  "EGYPTIAN HIEROGLYPH-13B98\0"
  "EGYPTIAN HIEROGLYPH-13B99\0"
  "EGYPTIAN HIEROGLYPH-13B9A\0"
  "EGYPTIAN HIEROGLYPH-13B9B\0"
  "EGYPTIAN HIEROGLYPH-13B9C\0"
  "EGYPTIAN HIEROGLYPH-13B9D\0"
  "EGYPTIAN HIEROGLYPH-13B9E\0"
  "EGYPTIAN HIEROGLYPH-13B9F\0"
  "EGYPTIAN HIEROGLYPH-13BA0\0"
  "EGYPTIAN HIEROGLYPH-13BA1\0"
  "EGYPTIAN HIEROGLYPH-13BA2\0"
  "EGYPTIAN HIEROGLYPH-13BA3\0"
  "EGYPTIAN HIEROGLYPH-13BA4\0"
  "EGYPTIAN HIEROGLYPH-13BA5\0"
  "EGYPTIAN HIEROGLYPH-13BA6\0"
  "EGYPTIAN HIEROGLYPH-13BA7\0"
  "EGYPTIAN HIEROGLYPH-13BA8\0"
  "EGYPTIAN HIEROGLYPH-13BA9\0"
  "EGYPTIAN HIEROGLYPH-13BAA\0"
  "EGYPTIAN HIEROGLYPH-13BAB\0"
  "EGYPTIAN HIEROGLYPH-13BAC\0"
  "EGYPTIAN HIEROGLYPH-13BAD\0"
  "EGYPTIAN HIEROGLYPH-13BAE\0"
  "EGYPTIAN HIEROGLYPH-13BAF\0"
  "EGYPTIAN HIEROGLYPH-13BB0\0"
  "EGYPTIAN HIEROGLYPH-13BB1\0"
  "EGYPTIAN HIEROGLYPH-13BB2\0"
  "EGYPTIAN HIEROGLYPH-13BB3\0"
  "EGYPTIAN HIEROGLYPH-13BB4\0"
  "EGYPTIAN HIEROGLYPH-13BB5\0"
  "EGYPTIAN HIEROGLYPH-13BB6\0"
  "EGYPTIAN HIEROGLYPH-13BB7\0"
  "EGYPTIAN HIEROGLYPH-13BB8\0"
  "EGYPTIAN HIEROGLYPH-13BB9\0"
  "EGYPTIAN HIEROGLYPH-13BBA\0"
  "EGYPTIAN HIEROGLYPH-13BBB\0"
  "EGYPTIAN HIEROGLYPH-13BBC\0"
  "EGYPTIAN HIEROGLYPH-13BBD\0"
  "EGYPTIAN HIEROGLYPH-13BBE\0"
  "EGYPTIAN HIEROGLYPH-13BBF\0"
  "EGYPTIAN HIEROGLYPH-13BC0\0"
  "EGYPTIAN HIEROGLYPH-13BC1\0"
  "EGYPTIAN HIEROGLYPH-13BC2\0"
  "EGYPTIAN HIEROGLYPH-13BC3\0"
  "EGYPTIAN HIEROGLYPH-13BC4\0"
  "EGYPTIAN HIEROGLYPH-13BC5\0"
  "EGYPTIAN HIEROGLYPH-13BC6\0"
  "EGYPTIAN HIEROGLYPH-13BC7\0"
  "EGYPTIAN HIEROGLYPH-13BC8\0"
  "EGYPTIAN HIEROGLYPH-13BC9\0"
  "EGYPTIAN HIEROGLYPH-13BCA\0"
  "EGYPTIAN HIEROGLYPH-13BCB\0"
  "EGYPTIAN HIEROGLYPH-13BCC\0"
  "EGYPTIAN HIEROGLYPH-13BCD\0"
  "EGYPTIAN HIEROGLYPH-13BCE\0"
  "EGYPTIAN HIEROGLYPH-13BCF\0"
  "EGYPTIAN HIEROGLYPH-13BD0\0"
  "EGYPTIAN HIEROGLYPH-13BD1\0"
  "EGYPTIAN HIEROGLYPH-13BD2\0"
  "EGYPTIAN HIEROGLYPH-13BD3\0"
  "EGYPTIAN HIEROGLYPH-13BD4\0"
  "EGYPTIAN HIEROGLYPH-13BD5\0"
  "EGYPTIAN HIEROGLYPH-13BD6\0"
  "EGYPTIAN HIEROGLYPH-13BD7\0"
  "EGYPTIAN HIEROGLYPH-13BD8\0"
  "EGYPTIAN HIEROGLYPH-13BD9\0"
  "EGYPTIAN HIEROGLYPH-13BDA\0"
  "EGYPTIAN HIEROGLYPH-13BDB\0"
  "EGYPTIAN HIEROGLYPH-13BDC\0"
  "EGYPTIAN HIEROGLYPH-13BDD\0"
  "EGYPTIAN HIEROGLYPH-13BDE\0"
  "EGYPTIAN HIEROGLYPH-13BDF\0"
  "EGYPTIAN HIEROGLYPH-13BE0\0"
  "EGYPTIAN HIEROGLYPH-13BE1\0"
  "EGYPTIAN HIEROGLYPH-13BE2\0"
  "EGYPTIAN HIEROGLYPH-13BE3\0"
  "EGYPTIAN HIEROGLYPH-13BE4\0"
  "EGYPTIAN HIEROGLYPH-13BE5\0"
  "EGYPTIAN HIEROGLYPH-13BE6\0"
  "EGYPTIAN HIEROGLYPH-13BE7\0"
  "EGYPTIAN HIEROGLYPH-13BE8\0"
  "EGYPTIAN HIEROGLYPH-13BE9\0"
  "EGYPTIAN HIEROGLYPH-13BEA\0"
  "EGYPTIAN HIEROGLYPH-13BEB\0"
  "EGYPTIAN HIEROGLYPH-13BEC\0"
  "EGYPTIAN HIEROGLYPH-13BED\0"
  "EGYPTIAN HIEROGLYPH-13BEE\0"
  "EGYPTIAN HIEROGLYPH-13BEF\0"
  "EGYPTIAN HIEROGLYPH-13BF0\0"
  "EGYPTIAN HIEROGLYPH-13BF1\0"
  "EGYPTIAN HIEROGLYPH-13BF2\0"
  "EGYPTIAN HIEROGLYPH-13BF3\0"
  "EGYPTIAN HIEROGLYPH-13BF4\0"
  "EGYPTIAN HIEROGLYPH-13BF5\0"
  "EGYPTIAN HIEROGLYPH-13BF6\0"
  "EGYPTIAN HIEROGLYPH-13BF7\0"
  "EGYPTIAN HIEROGLYPH-13BF8\0"
  "EGYPTIAN HIEROGLYPH-13BF9\0"
  "EGYPTIAN HIEROGLYPH-13BFA\0"
  "EGYPTIAN HIEROGLYPH-13BFB\0"
  "EGYPTIAN HIEROGLYPH-13BFC\0"
  "EGYPTIAN HIEROGLYPH-13BFD\0"
  "EGYPTIAN HIEROGLYPH-13BFE\0"
  "EGYPTIAN HIEROGLYPH-13BFF\0"
  "EGYPTIAN HIEROGLYPH-13C00\0"
  "EGYPTIAN HIEROGLYPH-13C01\0"
  "EGYPTIAN HIEROGLYPH-13C02\0"
  "EGYPTIAN HIEROGLYPH-13C03\0"
  "EGYPTIAN HIEROGLYPH-13C04\0"
  "EGYPTIAN HIEROGLYPH-13C05\0"
  "EGYPTIAN HIEROGLYPH-13C06\0"
  "EGYPTIAN HIEROGLYPH-13C07\0"
  "EGYPTIAN HIEROGLYPH-13C08\0"
  "EGYPTIAN HIEROGLYPH-13C09\0"
  "EGYPTIAN HIEROGLYPH-13C0A\0"
  "EGYPTIAN HIEROGLYPH-13C0B\0"
  "EGYPTIAN HIEROGLYPH-13C0C\0"
  "EGYPTIAN HIEROGLYPH-13C0D\0"
  "EGYPTIAN HIEROGLYPH-13C0E\0"
  "EGYPTIAN HIEROGLYPH-13C0F\0"
  "EGYPTIAN HIEROGLYPH-13C10\0"
  "EGYPTIAN HIEROGLYPH-13C11\0"
  "EGYPTIAN HIEROGLYPH-13C12\0"
  "EGYPTIAN HIEROGLYPH-13C13\0"
  "EGYPTIAN HIEROGLYPH-13C14\0"
  "EGYPTIAN HIEROGLYPH-13C15\0"
  "EGYPTIAN HIEROGLYPH-13C16\0"
  "EGYPTIAN HIEROGLYPH-13C17\0"
  "EGYPTIAN HIEROGLYPH-13C18\0"
  "EGYPTIAN HIEROGLYPH-13C19\0"
  "EGYPTIAN HIEROGLYPH-13C1A\0"
  "EGYPTIAN HIEROGLYPH-13C1B\0"
  "EGYPTIAN HIEROGLYPH-13C1C\0"
  "EGYPTIAN HIEROGLYPH-13C1D\0"
  "EGYPTIAN HIEROGLYPH-13C1E\0"
  "EGYPTIAN HIEROGLYPH-13C1F\0"
  "EGYPTIAN HIEROGLYPH-13C20\0"
  "EGYPTIAN HIEROGLYPH-13C21\0"
  "EGYPTIAN HIEROGLYPH-13C22\0"
  "EGYPTIAN HIEROGLYPH-13C23\0"
  "EGYPTIAN HIEROGLYPH-13C24\0"
  "EGYPTIAN HIEROGLYPH-13C25\0"
  "EGYPTIAN HIEROGLYPH-13C26\0"
  "EGYPTIAN HIEROGLYPH-13C27\0"
  "EGYPTIAN HIEROGLYPH-13C28\0"
  "EGYPTIAN HIEROGLYPH-13C29\0"
  "EGYPTIAN HIEROGLYPH-13C2A\0"
  "EGYPTIAN HIEROGLYPH-13C2B\0"
  "EGYPTIAN HIEROGLYPH-13C2C\0"
  "EGYPTIAN HIEROGLYPH-13C2D\0"
  "EGYPTIAN HIEROGLYPH-13C2E\0"
  "EGYPTIAN HIEROGLYPH-13C2F\0"
  "EGYPTIAN HIEROGLYPH-13C30\0"
  "EGYPTIAN HIEROGLYPH-13C31\0"
  "EGYPTIAN HIEROGLYPH-13C32\0"
  "EGYPTIAN HIEROGLYPH-13C33\0"
  "EGYPTIAN HIEROGLYPH-13C34\0"
  "EGYPTIAN HIEROGLYPH-13C35\0"
  "EGYPTIAN HIEROGLYPH-13C36\0"
  "EGYPTIAN HIEROGLYPH-13C37\0"
  "EGYPTIAN HIEROGLYPH-13C38\0"
  "EGYPTIAN HIEROGLYPH-13C39\0"
  "EGYPTIAN HIEROGLYPH-13C3A\0"
  "EGYPTIAN HIEROGLYPH-13C3B\0"
  "EGYPTIAN HIEROGLYPH-13C3C\0"
  "EGYPTIAN HIEROGLYPH-13C3D\0"
  "EGYPTIAN HIEROGLYPH-13C3E\0"
  "EGYPTIAN HIEROGLYPH-13C3F\0"
  "EGYPTIAN HIEROGLYPH-13C40\0"
  "EGYPTIAN HIEROGLYPH-13C41\0"
  "EGYPTIAN HIEROGLYPH-13C42\0"
  "EGYPTIAN HIEROGLYPH-13C43\0"
  "EGYPTIAN HIEROGLYPH-13C44\0"
  "EGYPTIAN HIEROGLYPH-13C45\0"
  "EGYPTIAN HIEROGLYPH-13C46\0"
  "EGYPTIAN HIEROGLYPH-13C47\0"
  "EGYPTIAN HIEROGLYPH-13C48\0"
  "EGYPTIAN HIEROGLYPH-13C49\0"
  "EGYPTIAN HIEROGLYPH-13C4A\0"
  "EGYPTIAN HIEROGLYPH-13C4B\0"
  "EGYPTIAN HIEROGLYPH-13C4C\0"
  "EGYPTIAN HIEROGLYPH-13C4D\0"
  "EGYPTIAN HIEROGLYPH-13C4E\0"
  "EGYPTIAN HIEROGLYPH-13C4F\0"
  "EGYPTIAN HIEROGLYPH-13C50\0"
  "EGYPTIAN HIEROGLYPH-13C51\0"
  "EGYPTIAN HIEROGLYPH-13C52\0"
  "EGYPTIAN HIEROGLYPH-13C53\0"
  "EGYPTIAN HIEROGLYPH-13C54\0"
  "EGYPTIAN HIEROGLYPH-13C55\0"
  "EGYPTIAN HIEROGLYPH-13C56\0"
  "EGYPTIAN HIEROGLYPH-13C57\0"
  "EGYPTIAN HIEROGLYPH-13C58\0"
  "EGYPTIAN HIEROGLYPH-13C59\0"
  "EGYPTIAN HIEROGLYPH-13C5A\0"
  "EGYPTIAN HIEROGLYPH-13C5B\0"
  "EGYPTIAN HIEROGLYPH-13C5C\0"
  "EGYPTIAN HIEROGLYPH-13C5D\0"
  "EGYPTIAN HIEROGLYPH-13C5E\0"
  "EGYPTIAN HIEROGLYPH-13C5F\0"
  "EGYPTIAN HIEROGLYPH-13C60\0"
  "EGYPTIAN HIEROGLYPH-13C61\0"
  "EGYPTIAN HIEROGLYPH-13C62\0"
  "EGYPTIAN HIEROGLYPH-13C63\0"
  "EGYPTIAN HIEROGLYPH-13C64\0"
  "EGYPTIAN HIEROGLYPH-13C65\0"
  "EGYPTIAN HIEROGLYPH-13C66\0"
  "EGYPTIAN HIEROGLYPH-13C67\0"
  "EGYPTIAN HIEROGLYPH-13C68\0"
  "EGYPTIAN HIEROGLYPH-13C69\0"
  "EGYPTIAN HIEROGLYPH-13C6A\0"
  "EGYPTIAN HIEROGLYPH-13C6B\0"
  "EGYPTIAN HIEROGLYPH-13C6C\0"
  "EGYPTIAN HIEROGLYPH-13C6D\0"
  "EGYPTIAN HIEROGLYPH-13C6E\0"
  "EGYPTIAN HIEROGLYPH-13C6F\0"
  "EGYPTIAN HIEROGLYPH-13C70\0"
  "EGYPTIAN HIEROGLYPH-13C71\0"
  "EGYPTIAN HIEROGLYPH-13C72\0"
  "EGYPTIAN HIEROGLYPH-13C73\0"
  "EGYPTIAN HIEROGLYPH-13C74\0"
  "EGYPTIAN HIEROGLYPH-13C75\0"
  "EGYPTIAN HIEROGLYPH-13C76\0"
  "EGYPTIAN HIEROGLYPH-13C77\0"
  "EGYPTIAN HIEROGLYPH-13C78\0"
  "EGYPTIAN HIEROGLYPH-13C79\0"
  "EGYPTIAN HIEROGLYPH-13C7A\0"
  "EGYPTIAN HIEROGLYPH-13C7B\0"
  "EGYPTIAN HIEROGLYPH-13C7C\0"
  "EGYPTIAN HIEROGLYPH-13C7D\0"
  "EGYPTIAN HIEROGLYPH-13C7E\0"
  "EGYPTIAN HIEROGLYPH-13C7F\0"
  "EGYPTIAN HIEROGLYPH-13C80\0"
  "EGYPTIAN HIEROGLYPH-13C81\0"
  "EGYPTIAN HIEROGLYPH-13C82\0"
  "EGYPTIAN HIEROGLYPH-13C83\0"
  "EGYPTIAN HIEROGLYPH-13C84\0"
  "EGYPTIAN HIEROGLYPH-13C85\0"
  "EGYPTIAN HIEROGLYPH-13C86\0"
  "EGYPTIAN HIEROGLYPH-13C87\0"
  "EGYPTIAN HIEROGLYPH-13C88\0"
  "EGYPTIAN HIEROGLYPH-13C89\0"
  "EGYPTIAN HIEROGLYPH-13C8A\0"
  "EGYPTIAN HIEROGLYPH-13C8B\0"
  "EGYPTIAN HIEROGLYPH-13C8C\0"
  "EGYPTIAN HIEROGLYPH-13C8D\0"
  "EGYPTIAN HIEROGLYPH-13C8E\0"
  "EGYPTIAN HIEROGLYPH-13C8F\0"
  "EGYPTIAN HIEROGLYPH-13C90\0"
  "EGYPTIAN HIEROGLYPH-13C91\0"
  "EGYPTIAN HIEROGLYPH-13C92\0"
  "EGYPTIAN HIEROGLYPH-13C93\0"
  "EGYPTIAN HIEROGLYPH-13C94\0"
  "EGYPTIAN HIEROGLYPH-13C95\0"
  "EGYPTIAN HIEROGLYPH-13C96\0"
  "EGYPTIAN HIEROGLYPH-13C97\0"
  "EGYPTIAN HIEROGLYPH-13C98\0"
  "EGYPTIAN HIEROGLYPH-13C99\0"
  "EGYPTIAN HIEROGLYPH-13C9A\0"
  "EGYPTIAN HIEROGLYPH-13C9B\0"
  "EGYPTIAN HIEROGLYPH-13C9C\0"
  "EGYPTIAN HIEROGLYPH-13C9D\0"
  "EGYPTIAN HIEROGLYPH-13C9E\0"
  "EGYPTIAN HIEROGLYPH-13C9F\0"
  "EGYPTIAN HIEROGLYPH-13CA0\0"
  "EGYPTIAN HIEROGLYPH-13CA1\0"
  "EGYPTIAN HIEROGLYPH-13CA2\0"
  "EGYPTIAN HIEROGLYPH-13CA3\0"
  "EGYPTIAN HIEROGLYPH-13CA4\0"
  "EGYPTIAN HIEROGLYPH-13CA5\0"
  "EGYPTIAN HIEROGLYPH-13CA6\0"
  "EGYPTIAN HIEROGLYPH-13CA7\0"
  "EGYPTIAN HIEROGLYPH-13CA8\0"
  "EGYPTIAN HIEROGLYPH-13CA9\0"
  "EGYPTIAN HIEROGLYPH-13CAA\0"
  "EGYPTIAN HIEROGLYPH-13CAB\0"
  "EGYPTIAN HIEROGLYPH-13CAC\0"
  "EGYPTIAN HIEROGLYPH-13CAD\0"
  "EGYPTIAN HIEROGLYPH-13CAE\0"
  "EGYPTIAN HIEROGLYPH-13CAF\0"
  "EGYPTIAN HIEROGLYPH-13CB0\0"
  "EGYPTIAN HIEROGLYPH-13CB1\0"
  "EGYPTIAN HIEROGLYPH-13CB2\0"
  "EGYPTIAN HIEROGLYPH-13CB3\0"
  "EGYPTIAN HIEROGLYPH-13CB4\0"
  "EGYPTIAN HIEROGLYPH-13CB5\0"
  "EGYPTIAN HIEROGLYPH-13CB6\0"
  "EGYPTIAN HIEROGLYPH-13CB7\0"
  "EGYPTIAN HIEROGLYPH-13CB8\0"
  "EGYPTIAN HIEROGLYPH-13CB9\0"
  "EGYPTIAN HIEROGLYPH-13CBA\0"
  "EGYPTIAN HIEROGLYPH-13CBB\0"
  "EGYPTIAN HIEROGLYPH-13CBC\0"
  "EGYPTIAN HIEROGLYPH-13CBD\0"
  "EGYPTIAN HIEROGLYPH-13CBE\0"
  "EGYPTIAN HIEROGLYPH-13CBF\0"
  "EGYPTIAN HIEROGLYPH-13CC0\0"
  "EGYPTIAN HIEROGLYPH-13CC1\0"
  "EGYPTIAN HIEROGLYPH-13CC2\0"
  "EGYPTIAN HIEROGLYPH-13CC3\0"
  "EGYPTIAN HIEROGLYPH-13CC4\0"
  "EGYPTIAN HIEROGLYPH-13CC5\0"
  "EGYPTIAN HIEROGLYPH-13CC6\0"
  "EGYPTIAN HIEROGLYPH-13CC7\0"
  "EGYPTIAN HIEROGLYPH-13CC8\0"
  "EGYPTIAN HIEROGLYPH-13CC9\0"
  "EGYPTIAN HIEROGLYPH-13CCA\0"
  "EGYPTIAN HIEROGLYPH-13CCB\0"
  "EGYPTIAN HIEROGLYPH-13CCC\0"
  "EGYPTIAN HIEROGLYPH-13CCD\0"
  "EGYPTIAN HIEROGLYPH-13CCE\0"
  "EGYPTIAN HIEROGLYPH-13CCF\0"
  "EGYPTIAN HIEROGLYPH-13CD0\0"
  "EGYPTIAN HIEROGLYPH-13CD1\0"
  "EGYPTIAN HIEROGLYPH-13CD2\0"
  "EGYPTIAN HIEROGLYPH-13CD3\0"
  "EGYPTIAN HIEROGLYPH-13CD4\0"
  "EGYPTIAN HIEROGLYPH-13CD5\0"
  "EGYPTIAN HIEROGLYPH-13CD6\0"
  "EGYPTIAN HIEROGLYPH-13CD7\0"
  "EGYPTIAN HIEROGLYPH-13CD8\0"
  "EGYPTIAN HIEROGLYPH-13CD9\0"
  "EGYPTIAN HIEROGLYPH-13CDA\0"
  "EGYPTIAN HIEROGLYPH-13CDB\0"
  "EGYPTIAN HIEROGLYPH-13CDC\0"
  "EGYPTIAN HIEROGLYPH-13CDD\0"
  "EGYPTIAN HIEROGLYPH-13CDE\0"
  "EGYPTIAN HIEROGLYPH-13CDF\0"
  "EGYPTIAN HIEROGLYPH-13CE0\0"
  "EGYPTIAN HIEROGLYPH-13CE1\0"
  "EGYPTIAN HIEROGLYPH-13CE2\0"
  "EGYPTIAN HIEROGLYPH-13CE3\0"
  "EGYPTIAN HIEROGLYPH-13CE4\0"
  "EGYPTIAN HIEROGLYPH-13CE5\0"
  "EGYPTIAN HIEROGLYPH-13CE6\0"
  "EGYPTIAN HIEROGLYPH-13CE7\0"
  "EGYPTIAN HIEROGLYPH-13CE8\0"
  "EGYPTIAN HIEROGLYPH-13CE9\0"
  "EGYPTIAN HIEROGLYPH-13CEA\0"
  "EGYPTIAN HIEROGLYPH-13CEB\0"
  "EGYPTIAN HIEROGLYPH-13CEC\0"
  "EGYPTIAN HIEROGLYPH-13CED\0"
  "EGYPTIAN HIEROGLYPH-13CEE\0"
  "EGYPTIAN HIEROGLYPH-13CEF\0"
  "EGYPTIAN HIEROGLYPH-13CF0\0"
  "EGYPTIAN HIEROGLYPH-13CF1\0"
  "EGYPTIAN HIEROGLYPH-13CF2\0"
  "EGYPTIAN HIEROGLYPH-13CF3\0"
  "EGYPTIAN HIEROGLYPH-13CF4\0"
  "EGYPTIAN HIEROGLYPH-13CF5\0"
  "EGYPTIAN HIEROGLYPH-13CF6\0"
  "EGYPTIAN HIEROGLYPH-13CF7\0"
  "EGYPTIAN HIEROGLYPH-13CF8\0"
  "EGYPTIAN HIEROGLYPH-13CF9\0"
  "EGYPTIAN HIEROGLYPH-13CFA\0"
  "EGYPTIAN HIEROGLYPH-13CFB\0"
  "EGYPTIAN HIEROGLYPH-13CFC\0"
  "EGYPTIAN HIEROGLYPH-13CFD\0"
  "EGYPTIAN HIEROGLYPH-13CFE\0"
  "EGYPTIAN HIEROGLYPH-13CFF\0"
  "EGYPTIAN HIEROGLYPH-13D00\0"
  "EGYPTIAN HIEROGLYPH-13D01\0"
  "EGYPTIAN HIEROGLYPH-13D02\0"
  "EGYPTIAN HIEROGLYPH-13D03\0"
  "EGYPTIAN HIEROGLYPH-13D04\0"
  "EGYPTIAN HIEROGLYPH-13D05\0"
  "EGYPTIAN HIEROGLYPH-13D06\0"
  "EGYPTIAN HIEROGLYPH-13D07\0"
  "EGYPTIAN HIEROGLYPH-13D08\0"
  "EGYPTIAN HIEROGLYPH-13D09\0"
  "EGYPTIAN HIEROGLYPH-13D0A\0"
  "EGYPTIAN HIEROGLYPH-13D0B\0"
  "EGYPTIAN HIEROGLYPH-13D0C\0"
  "EGYPTIAN HIEROGLYPH-13D0D\0"
  "EGYPTIAN HIEROGLYPH-13D0E\0"
  "EGYPTIAN HIEROGLYPH-13D0F\0"
  "EGYPTIAN HIEROGLYPH-13D10\0"
  "EGYPTIAN HIEROGLYPH-13D11\0"
  "EGYPTIAN HIEROGLYPH-13D12\0"
  "EGYPTIAN HIEROGLYPH-13D13\0"
  "EGYPTIAN HIEROGLYPH-13D14\0"
  "EGYPTIAN HIEROGLYPH-13D15\0"
  "EGYPTIAN HIEROGLYPH-13D16\0"
  "EGYPTIAN HIEROGLYPH-13D17\0"
  "EGYPTIAN HIEROGLYPH-13D18\0"
  "EGYPTIAN HIEROGLYPH-13D19\0"
  "EGYPTIAN HIEROGLYPH-13D1A\0"
  "EGYPTIAN HIEROGLYPH-13D1B\0"
  "EGYPTIAN HIEROGLYPH-13D1C\0"
  "EGYPTIAN HIEROGLYPH-13D1D\0"
  "EGYPTIAN HIEROGLYPH-13D1E\0"
  "EGYPTIAN HIEROGLYPH-13D1F\0"
  "EGYPTIAN HIEROGLYPH-13D20\0"
  "EGYPTIAN HIEROGLYPH-13D21\0"
  "EGYPTIAN HIEROGLYPH-13D22\0"
  "EGYPTIAN HIEROGLYPH-13D23\0"
  "EGYPTIAN HIEROGLYPH-13D24\0"
  "EGYPTIAN HIEROGLYPH-13D25\0"
  "EGYPTIAN HIEROGLYPH-13D26\0"
  "EGYPTIAN HIEROGLYPH-13D27\0"
  "EGYPTIAN HIEROGLYPH-13D28\0"
  "EGYPTIAN HIEROGLYPH-13D29\0"
  "EGYPTIAN HIEROGLYPH-13D2A\0"
  "EGYPTIAN HIEROGLYPH-13D2B\0"
  "EGYPTIAN HIEROGLYPH-13D2C\0"
  "EGYPTIAN HIEROGLYPH-13D2D\0"
  "EGYPTIAN HIEROGLYPH-13D2E\0"
  "EGYPTIAN HIEROGLYPH-13D2F\0"
  "EGYPTIAN HIEROGLYPH-13D30\0"
  "EGYPTIAN HIEROGLYPH-13D31\0"
  "EGYPTIAN HIEROGLYPH-13D32\0"
  "EGYPTIAN HIEROGLYPH-13D33\0"
  "EGYPTIAN HIEROGLYPH-13D34\0"
  "EGYPTIAN HIEROGLYPH-13D35\0"
  "EGYPTIAN HIEROGLYPH-13D36\0"
  "EGYPTIAN HIEROGLYPH-13D37\0"
  "EGYPTIAN HIEROGLYPH-13D38\0"
  "EGYPTIAN HIEROGLYPH-13D39\0"
  "EGYPTIAN HIEROGLYPH-13D3A\0"
  "EGYPTIAN HIEROGLYPH-13D3B\0"
  "EGYPTIAN HIEROGLYPH-13D3C\0"
  "EGYPTIAN HIEROGLYPH-13D3D\0"
  "EGYPTIAN HIEROGLYPH-13D3E\0"
  "EGYPTIAN HIEROGLYPH-13D3F\0"
  "EGYPTIAN HIEROGLYPH-13D40\0"
  "EGYPTIAN HIEROGLYPH-13D41\0"
  "EGYPTIAN HIEROGLYPH-13D42\0"
  "EGYPTIAN HIEROGLYPH-13D43\0"
  "EGYPTIAN HIEROGLYPH-13D44\0"
  "EGYPTIAN HIEROGLYPH-13D45\0"
  "EGYPTIAN HIEROGLYPH-13D46\0"
  "EGYPTIAN HIEROGLYPH-13D47\0"
  "EGYPTIAN HIEROGLYPH-13D48\0"
  "EGYPTIAN HIEROGLYPH-13D49\0"
  "EGYPTIAN HIEROGLYPH-13D4A\0"
  "EGYPTIAN HIEROGLYPH-13D4B\0"
  "EGYPTIAN HIEROGLYPH-13D4C\0"
  "EGYPTIAN HIEROGLYPH-13D4D\0"
  "EGYPTIAN HIEROGLYPH-13D4E\0"
  "EGYPTIAN HIEROGLYPH-13D4F\0"
  "EGYPTIAN HIEROGLYPH-13D50\0"
  "EGYPTIAN HIEROGLYPH-13D51\0"
  "EGYPTIAN HIEROGLYPH-13D52\0"
  "EGYPTIAN HIEROGLYPH-13D53\0"
  "EGYPTIAN HIEROGLYPH-13D54\0"
  "EGYPTIAN HIEROGLYPH-13D55\0"
  "EGYPTIAN HIEROGLYPH-13D56\0"
  "EGYPTIAN HIEROGLYPH-13D57\0"
  "EGYPTIAN HIEROGLYPH-13D58\0"
  "EGYPTIAN HIEROGLYPH-13D59\0"
  "EGYPTIAN HIEROGLYPH-13D5A\0"
  "EGYPTIAN HIEROGLYPH-13D5B\0"
  "EGYPTIAN HIEROGLYPH-13D5C\0"
  "EGYPTIAN HIEROGLYPH-13D5D\0"
  "EGYPTIAN HIEROGLYPH-13D5E\0"
  "EGYPTIAN HIEROGLYPH-13D5F\0"
  "EGYPTIAN HIEROGLYPH-13D60\0"
  "EGYPTIAN HIEROGLYPH-13D61\0"
  "EGYPTIAN HIEROGLYPH-13D62\0"
  "EGYPTIAN HIEROGLYPH-13D63\0"
  "EGYPTIAN HIEROGLYPH-13D64\0"
  "EGYPTIAN HIEROGLYPH-13D65\0"
  "EGYPTIAN HIEROGLYPH-13D66\0"
  "EGYPTIAN HIEROGLYPH-13D67\0"
  "EGYPTIAN HIEROGLYPH-13D68\0"
  "EGYPTIAN HIEROGLYPH-13D69\0"
  "EGYPTIAN HIEROGLYPH-13D6A\0"
  "EGYPTIAN HIEROGLYPH-13D6B\0"
  "EGYPTIAN HIEROGLYPH-13D6C\0"
  "EGYPTIAN HIEROGLYPH-13D6D\0"
  "EGYPTIAN HIEROGLYPH-13D6E\0"
  "EGYPTIAN HIEROGLYPH-13D6F\0"
  "EGYPTIAN HIEROGLYPH-13D70\0"
  "EGYPTIAN HIEROGLYPH-13D71\0"
  "EGYPTIAN HIEROGLYPH-13D72\0"
  "EGYPTIAN HIEROGLYPH-13D73\0"
  "EGYPTIAN HIEROGLYPH-13D74\0"
  "EGYPTIAN HIEROGLYPH-13D75\0"
  "EGYPTIAN HIEROGLYPH-13D76\0"
  "EGYPTIAN HIEROGLYPH-13D77\0"
  "EGYPTIAN HIEROGLYPH-13D78\0"
  "EGYPTIAN HIEROGLYPH-13D79\0"
  "EGYPTIAN HIEROGLYPH-13D7A\0"
  "EGYPTIAN HIEROGLYPH-13D7B\0"
  "EGYPTIAN HIEROGLYPH-13D7C\0"
  "EGYPTIAN HIEROGLYPH-13D7D\0"
  "EGYPTIAN HIEROGLYPH-13D7E\0"
  "EGYPTIAN HIEROGLYPH-13D7F\0"
  "EGYPTIAN HIEROGLYPH-13D80\0"
  "EGYPTIAN HIEROGLYPH-13D81\0"
  "EGYPTIAN HIEROGLYPH-13D82\0"
  "EGYPTIAN HIEROGLYPH-13D83\0"
  "EGYPTIAN HIEROGLYPH-13D84\0"
  "EGYPTIAN HIEROGLYPH-13D85\0"
  "EGYPTIAN HIEROGLYPH-13D86\0"
  "EGYPTIAN HIEROGLYPH-13D87\0"
  "EGYPTIAN HIEROGLYPH-13D88\0"
  "EGYPTIAN HIEROGLYPH-13D89\0"
  "EGYPTIAN HIEROGLYPH-13D8A\0"
  "EGYPTIAN HIEROGLYPH-13D8B\0"
  "EGYPTIAN HIEROGLYPH-13D8C\0"
  "EGYPTIAN HIEROGLYPH-13D8D\0"
  "EGYPTIAN HIEROGLYPH-13D8E\0"
  "EGYPTIAN HIEROGLYPH-13D8F\0"
  "EGYPTIAN HIEROGLYPH-13D90\0"
  "EGYPTIAN HIEROGLYPH-13D91\0"
  "EGYPTIAN HIEROGLYPH-13D92\0"
  "EGYPTIAN HIEROGLYPH-13D93\0"
  "EGYPTIAN HIEROGLYPH-13D94\0"
  "EGYPTIAN HIEROGLYPH-13D95\0"
  "EGYPTIAN HIEROGLYPH-13D96\0"
  "EGYPTIAN HIEROGLYPH-13D97\0"
  "EGYPTIAN HIEROGLYPH-13D98\0"
  "EGYPTIAN HIEROGLYPH-13D99\0"
  "EGYPTIAN HIEROGLYPH-13D9A\0"
  "EGYPTIAN HIEROGLYPH-13D9B\0"
  "EGYPTIAN HIEROGLYPH-13D9C\0"
  "EGYPTIAN HIEROGLYPH-13D9D\0"
  "EGYPTIAN HIEROGLYPH-13D9E\0"
  "EGYPTIAN HIEROGLYPH-13D9F\0"
  "EGYPTIAN HIEROGLYPH-13DA0\0"
  "EGYPTIAN HIEROGLYPH-13DA1\0"
  "EGYPTIAN HIEROGLYPH-13DA2\0"
  "EGYPTIAN HIEROGLYPH-13DA3\0"
  "EGYPTIAN HIEROGLYPH-13DA4\0"
  "EGYPTIAN HIEROGLYPH-13DA5\0"
  "EGYPTIAN HIEROGLYPH-13DA6\0"
  "EGYPTIAN HIEROGLYPH-13DA7\0"
  "EGYPTIAN HIEROGLYPH-13DA8\0"
  "EGYPTIAN HIEROGLYPH-13DA9\0"
  "EGYPTIAN HIEROGLYPH-13DAA\0"
  "EGYPTIAN HIEROGLYPH-13DAB\0"
  "EGYPTIAN HIEROGLYPH-13DAC\0"
  "EGYPTIAN HIEROGLYPH-13DAD\0"
  "EGYPTIAN HIEROGLYPH-13DAE\0"
  "EGYPTIAN HIEROGLYPH-13DAF\0"
  "EGYPTIAN HIEROGLYPH-13DB0\0"
  "EGYPTIAN HIEROGLYPH-13DB1\0"
  "EGYPTIAN HIEROGLYPH-13DB2\0"
  "EGYPTIAN HIEROGLYPH-13DB3\0"
  "EGYPTIAN HIEROGLYPH-13DB4\0"
  "EGYPTIAN HIEROGLYPH-13DB5\0"
  "EGYPTIAN HIEROGLYPH-13DB6\0"
  "EGYPTIAN HIEROGLYPH-13DB7\0"
  "EGYPTIAN HIEROGLYPH-13DB8\0"
  "EGYPTIAN HIEROGLYPH-13DB9\0"
  "EGYPTIAN HIEROGLYPH-13DBA\0"
  "EGYPTIAN HIEROGLYPH-13DBB\0"
  "EGYPTIAN HIEROGLYPH-13DBC\0"
  "EGYPTIAN HIEROGLYPH-13DBD\0"
  "EGYPTIAN HIEROGLYPH-13DBE\0"
  "EGYPTIAN HIEROGLYPH-13DBF\0"
  "EGYPTIAN HIEROGLYPH-13DC0\0"
  "EGYPTIAN HIEROGLYPH-13DC1\0"
  "EGYPTIAN HIEROGLYPH-13DC2\0"
  "EGYPTIAN HIEROGLYPH-13DC3\0"
  "EGYPTIAN HIEROGLYPH-13DC4\0"
  "EGYPTIAN HIEROGLYPH-13DC5\0"
  "EGYPTIAN HIEROGLYPH-13DC6\0"
  "EGYPTIAN HIEROGLYPH-13DC7\0"
  "EGYPTIAN HIEROGLYPH-13DC8\0"
  "EGYPTIAN HIEROGLYPH-13DC9\0"
  "EGYPTIAN HIEROGLYPH-13DCA\0"
  "EGYPTIAN HIEROGLYPH-13DCB\0"
  "EGYPTIAN HIEROGLYPH-13DCC\0"
  "EGYPTIAN HIEROGLYPH-13DCD\0"
  "EGYPTIAN HIEROGLYPH-13DCE\0"
  "EGYPTIAN HIEROGLYPH-13DCF\0"
  "EGYPTIAN HIEROGLYPH-13DD0\0"
  "EGYPTIAN HIEROGLYPH-13DD1\0"
  "EGYPTIAN HIEROGLYPH-13DD2\0"
  "EGYPTIAN HIEROGLYPH-13DD3\0"
  "EGYPTIAN HIEROGLYPH-13DD4\0"
  "EGYPTIAN HIEROGLYPH-13DD5\0"
  "EGYPTIAN HIEROGLYPH-13DD6\0"
  "EGYPTIAN HIEROGLYPH-13DD7\0"
  "EGYPTIAN HIEROGLYPH-13DD8\0"
  "EGYPTIAN HIEROGLYPH-13DD9\0"
  "EGYPTIAN HIEROGLYPH-13DDA\0"
  "EGYPTIAN HIEROGLYPH-13DDB\0"
  "EGYPTIAN HIEROGLYPH-13DDC\0"
  "EGYPTIAN HIEROGLYPH-13DDD\0"
  "EGYPTIAN HIEROGLYPH-13DDE\0"
  "EGYPTIAN HIEROGLYPH-13DDF\0"
  "EGYPTIAN HIEROGLYPH-13DE0\0"
  "EGYPTIAN HIEROGLYPH-13DE1\0"
  "EGYPTIAN HIEROGLYPH-13DE2\0"
  "EGYPTIAN HIEROGLYPH-13DE3\0"
  "EGYPTIAN HIEROGLYPH-13DE4\0"
  "EGYPTIAN HIEROGLYPH-13DE5\0"
  "EGYPTIAN HIEROGLYPH-13DE6\0"
  "EGYPTIAN HIEROGLYPH-13DE7\0"
  "EGYPTIAN HIEROGLYPH-13DE8\0"
  "EGYPTIAN HIEROGLYPH-13DE9\0"
  "EGYPTIAN HIEROGLYPH-13DEA\0"
  "EGYPTIAN HIEROGLYPH-13DEB\0"
  "EGYPTIAN HIEROGLYPH-13DEC\0"
  "EGYPTIAN HIEROGLYPH-13DED\0"
  "EGYPTIAN HIEROGLYPH-13DEE\0"
  "EGYPTIAN HIEROGLYPH-13DEF\0"
  "EGYPTIAN HIEROGLYPH-13DF0\0"
  "EGYPTIAN HIEROGLYPH-13DF1\0"
  "EGYPTIAN HIEROGLYPH-13DF2\0"
  "EGYPTIAN HIEROGLYPH-13DF3\0"
  "EGYPTIAN HIEROGLYPH-13DF4\0"
  "EGYPTIAN HIEROGLYPH-13DF5\0"
  "EGYPTIAN HIEROGLYPH-13DF6\0"
  "EGYPTIAN HIEROGLYPH-13DF7\0"
  "EGYPTIAN HIEROGLYPH-13DF8\0"
  "EGYPTIAN HIEROGLYPH-13DF9\0"
  "EGYPTIAN HIEROGLYPH-13DFA\0"
  "EGYPTIAN HIEROGLYPH-13DFB\0"
  "EGYPTIAN HIEROGLYPH-13DFC\0"
  "EGYPTIAN HIEROGLYPH-13DFD\0"
  "EGYPTIAN HIEROGLYPH-13DFE\0"
  "EGYPTIAN HIEROGLYPH-13DFF\0"
  "EGYPTIAN HIEROGLYPH-13E00\0"
  "EGYPTIAN HIEROGLYPH-13E01\0"
  "EGYPTIAN HIEROGLYPH-13E02\0"
  "EGYPTIAN HIEROGLYPH-13E03\0"
  "EGYPTIAN HIEROGLYPH-13E04\0"
  "EGYPTIAN HIEROGLYPH-13E05\0"
  "EGYPTIAN HIEROGLYPH-13E06\0"
  "EGYPTIAN HIEROGLYPH-13E07\0"
  "EGYPTIAN HIEROGLYPH-13E08\0"
  "EGYPTIAN HIEROGLYPH-13E09\0"
  "EGYPTIAN HIEROGLYPH-13E0A\0"
  "EGYPTIAN HIEROGLYPH-13E0B\0"
  "EGYPTIAN HIEROGLYPH-13E0C\0"
  "EGYPTIAN HIEROGLYPH-13E0D\0"
  "EGYPTIAN HIEROGLYPH-13E0E\0"
  "EGYPTIAN HIEROGLYPH-13E0F\0"
  "EGYPTIAN HIEROGLYPH-13E10\0"
  "EGYPTIAN HIEROGLYPH-13E11\0"
  "EGYPTIAN HIEROGLYPH-13E12\0"
  "EGYPTIAN HIEROGLYPH-13E13\0"
  "EGYPTIAN HIEROGLYPH-13E14\0"
  "EGYPTIAN HIEROGLYPH-13E15\0"
  "EGYPTIAN HIEROGLYPH-13E16\0"
  "EGYPTIAN HIEROGLYPH-13E17\0"
  "EGYPTIAN HIEROGLYPH-13E18\0"
  "EGYPTIAN HIEROGLYPH-13E19\0"
  "EGYPTIAN HIEROGLYPH-13E1A\0"
  "EGYPTIAN HIEROGLYPH-13E1B\0"
  "EGYPTIAN HIEROGLYPH-13E1C\0"
  "EGYPTIAN HIEROGLYPH-13E1D\0"
  "EGYPTIAN HIEROGLYPH-13E1E\0"
  "EGYPTIAN HIEROGLYPH-13E1F\0"
  "EGYPTIAN HIEROGLYPH-13E20\0"
  "EGYPTIAN HIEROGLYPH-13E21\0"
  "EGYPTIAN HIEROGLYPH-13E22\0"
  "EGYPTIAN HIEROGLYPH-13E23\0"
  "EGYPTIAN HIEROGLYPH-13E24\0"
  "EGYPTIAN HIEROGLYPH-13E25\0"
  "EGYPTIAN HIEROGLYPH-13E26\0"
  "EGYPTIAN HIEROGLYPH-13E27\0"
  "EGYPTIAN HIEROGLYPH-13E28\0"
  "EGYPTIAN HIEROGLYPH-13E29\0"
  "EGYPTIAN HIEROGLYPH-13E2A\0"
  "EGYPTIAN HIEROGLYPH-13E2B\0"
  "EGYPTIAN HIEROGLYPH-13E2C\0"
  "EGYPTIAN HIEROGLYPH-13E2D\0"
  "EGYPTIAN HIEROGLYPH-13E2E\0"
  "EGYPTIAN HIEROGLYPH-13E2F\0"
  "EGYPTIAN HIEROGLYPH-13E30\0"
  "EGYPTIAN HIEROGLYPH-13E31\0"
  "EGYPTIAN HIEROGLYPH-13E32\0"
  "EGYPTIAN HIEROGLYPH-13E33\0"
  "EGYPTIAN HIEROGLYPH-13E34\0"
  "EGYPTIAN HIEROGLYPH-13E35\0"
  "EGYPTIAN HIEROGLYPH-13E36\0"
  "EGYPTIAN HIEROGLYPH-13E37\0"
  "EGYPTIAN HIEROGLYPH-13E38\0"
  "EGYPTIAN HIEROGLYPH-13E39\0"
  "EGYPTIAN HIEROGLYPH-13E3A\0"
  "EGYPTIAN HIEROGLYPH-13E3B\0"
  "EGYPTIAN HIEROGLYPH-13E3C\0"
  "EGYPTIAN HIEROGLYPH-13E3D\0"
  "EGYPTIAN HIEROGLYPH-13E3E\0"
  "EGYPTIAN HIEROGLYPH-13E3F\0"
  "EGYPTIAN HIEROGLYPH-13E40\0"
  "EGYPTIAN HIEROGLYPH-13E41\0"
  "EGYPTIAN HIEROGLYPH-13E42\0"
  "EGYPTIAN HIEROGLYPH-13E43\0"
  "EGYPTIAN HIEROGLYPH-13E44\0"
  "EGYPTIAN HIEROGLYPH-13E45\0"
  "EGYPTIAN HIEROGLYPH-13E46\0"
  "EGYPTIAN HIEROGLYPH-13E47\0"
  "EGYPTIAN HIEROGLYPH-13E48\0"
  "EGYPTIAN HIEROGLYPH-13E49\0"
  "EGYPTIAN HIEROGLYPH-13E4A\0"
  "EGYPTIAN HIEROGLYPH-13E4B\0"
  "EGYPTIAN HIEROGLYPH-13E4C\0"
  "EGYPTIAN HIEROGLYPH-13E4D\0"
  "EGYPTIAN HIEROGLYPH-13E4E\0"
  "EGYPTIAN HIEROGLYPH-13E4F\0"
  "EGYPTIAN HIEROGLYPH-13E50\0"
  "EGYPTIAN HIEROGLYPH-13E51\0"
  "EGYPTIAN HIEROGLYPH-13E52\0"
  "EGYPTIAN HIEROGLYPH-13E53\0"
  "EGYPTIAN HIEROGLYPH-13E54\0"
  "EGYPTIAN HIEROGLYPH-13E55\0"
  "EGYPTIAN HIEROGLYPH-13E56\0"
  "EGYPTIAN HIEROGLYPH-13E57\0"
  "EGYPTIAN HIEROGLYPH-13E58\0"
  "EGYPTIAN HIEROGLYPH-13E59\0"
  "EGYPTIAN HIEROGLYPH-13E5A\0"
  "EGYPTIAN HIEROGLYPH-13E5B\0"
  "EGYPTIAN HIEROGLYPH-13E5C\0"
  "EGYPTIAN HIEROGLYPH-13E5D\0"
  "EGYPTIAN HIEROGLYPH-13E5E\0"
  "EGYPTIAN HIEROGLYPH-13E5F\0"
  "EGYPTIAN HIEROGLYPH-13E60\0"
  "EGYPTIAN HIEROGLYPH-13E61\0"
  "EGYPTIAN HIEROGLYPH-13E62\0"
  "EGYPTIAN HIEROGLYPH-13E63\0"
  "EGYPTIAN HIEROGLYPH-13E64\0"
  "EGYPTIAN HIEROGLYPH-13E65\0"
  "EGYPTIAN HIEROGLYPH-13E66\0"
  "EGYPTIAN HIEROGLYPH-13E67\0"
  "EGYPTIAN HIEROGLYPH-13E68\0"
  "EGYPTIAN HIEROGLYPH-13E69\0"
  "EGYPTIAN HIEROGLYPH-13E6A\0"
  "EGYPTIAN HIEROGLYPH-13E6B\0"
  "EGYPTIAN HIEROGLYPH-13E6C\0"
  "EGYPTIAN HIEROGLYPH-13E6D\0"
  "EGYPTIAN HIEROGLYPH-13E6E\0"
  "EGYPTIAN HIEROGLYPH-13E6F\0"
  "EGYPTIAN HIEROGLYPH-13E70\0"
  "EGYPTIAN HIEROGLYPH-13E71\0"
  "EGYPTIAN HIEROGLYPH-13E72\0"
  "EGYPTIAN HIEROGLYPH-13E73\0"
  "EGYPTIAN HIEROGLYPH-13E74\0"
  "EGYPTIAN HIEROGLYPH-13E75\0"
  "EGYPTIAN HIEROGLYPH-13E76\0"
  "EGYPTIAN HIEROGLYPH-13E77\0"
  "EGYPTIAN HIEROGLYPH-13E78\0"
  "EGYPTIAN HIEROGLYPH-13E79\0"
  "EGYPTIAN HIEROGLYPH-13E7A\0"
  "EGYPTIAN HIEROGLYPH-13E7B\0"
  "EGYPTIAN HIEROGLYPH-13E7C\0"
  "EGYPTIAN HIEROGLYPH-13E7D\0"
  "EGYPTIAN HIEROGLYPH-13E7E\0"
  "EGYPTIAN HIEROGLYPH-13E7F\0"
  "EGYPTIAN HIEROGLYPH-13E80\0"
  "EGYPTIAN HIEROGLYPH-13E81\0"
  "EGYPTIAN HIEROGLYPH-13E82\0"
  "EGYPTIAN HIEROGLYPH-13E83\0"
  "EGYPTIAN HIEROGLYPH-13E84\0"
  "EGYPTIAN HIEROGLYPH-13E85\0"
  "EGYPTIAN HIEROGLYPH-13E86\0"
  "EGYPTIAN HIEROGLYPH-13E87\0"
  "EGYPTIAN HIEROGLYPH-13E88\0"
  "EGYPTIAN HIEROGLYPH-13E89\0"
  "EGYPTIAN HIEROGLYPH-13E8A\0"
  "EGYPTIAN HIEROGLYPH-13E8B\0"
  "EGYPTIAN HIEROGLYPH-13E8C\0"
  "EGYPTIAN HIEROGLYPH-13E8D\0"
  "EGYPTIAN HIEROGLYPH-13E8E\0"
  "EGYPTIAN HIEROGLYPH-13E8F\0"
  "EGYPTIAN HIEROGLYPH-13E90\0"
  "EGYPTIAN HIEROGLYPH-13E91\0"
  "EGYPTIAN HIEROGLYPH-13E92\0"
  "EGYPTIAN HIEROGLYPH-13E93\0"
  "EGYPTIAN HIEROGLYPH-13E94\0"
  "EGYPTIAN HIEROGLYPH-13E95\0"
  "EGYPTIAN HIEROGLYPH-13E96\0"
  "EGYPTIAN HIEROGLYPH-13E97\0"
  "EGYPTIAN HIEROGLYPH-13E98\0"
  "EGYPTIAN HIEROGLYPH-13E99\0"
  "EGYPTIAN HIEROGLYPH-13E9A\0"
  "EGYPTIAN HIEROGLYPH-13E9B\0"
  "EGYPTIAN HIEROGLYPH-13E9C\0"
  "EGYPTIAN HIEROGLYPH-13E9D\0"
  "EGYPTIAN HIEROGLYPH-13E9E\0"
  "EGYPTIAN HIEROGLYPH-13E9F\0"
  "EGYPTIAN HIEROGLYPH-13EA0\0"
  "EGYPTIAN HIEROGLYPH-13EA1\0"
  "EGYPTIAN HIEROGLYPH-13EA2\0"
  "EGYPTIAN HIEROGLYPH-13EA3\0"
  "EGYPTIAN HIEROGLYPH-13EA4\0"
  "EGYPTIAN HIEROGLYPH-13EA5\0"
  "EGYPTIAN HIEROGLYPH-13EA6\0"
  "EGYPTIAN HIEROGLYPH-13EA7\0"
  "EGYPTIAN HIEROGLYPH-13EA8\0"
  "EGYPTIAN HIEROGLYPH-13EA9\0"
  "EGYPTIAN HIEROGLYPH-13EAA\0"
  "EGYPTIAN HIEROGLYPH-13EAB\0"
  "EGYPTIAN HIEROGLYPH-13EAC\0"
  "EGYPTIAN HIEROGLYPH-13EAD\0"
  "EGYPTIAN HIEROGLYPH-13EAE\0"
  "EGYPTIAN HIEROGLYPH-13EAF\0"
  "EGYPTIAN HIEROGLYPH-13EB0\0"
  "EGYPTIAN HIEROGLYPH-13EB1\0"
  "EGYPTIAN HIEROGLYPH-13EB2\0"
  "EGYPTIAN HIEROGLYPH-13EB3\0"
  "EGYPTIAN HIEROGLYPH-13EB4\0"
  "EGYPTIAN HIEROGLYPH-13EB5\0"
  "EGYPTIAN HIEROGLYPH-13EB6\0"
  "EGYPTIAN HIEROGLYPH-13EB7\0"
  "EGYPTIAN HIEROGLYPH-13EB8\0"
  "EGYPTIAN HIEROGLYPH-13EB9\0"
  "EGYPTIAN HIEROGLYPH-13EBA\0"
  "EGYPTIAN HIEROGLYPH-13EBB\0"
  "EGYPTIAN HIEROGLYPH-13EBC\0"
  "EGYPTIAN HIEROGLYPH-13EBD\0"
  "EGYPTIAN HIEROGLYPH-13EBE\0"
  "EGYPTIAN HIEROGLYPH-13EBF\0"
  "EGYPTIAN HIEROGLYPH-13EC0\0"
  "EGYPTIAN HIEROGLYPH-13EC1\0"
  "EGYPTIAN HIEROGLYPH-13EC2\0"
  "EGYPTIAN HIEROGLYPH-13EC3\0"
  "EGYPTIAN HIEROGLYPH-13EC4\0"
  "EGYPTIAN HIEROGLYPH-13EC5\0"
  "EGYPTIAN HIEROGLYPH-13EC6\0"
  "EGYPTIAN HIEROGLYPH-13EC7\0"
  "EGYPTIAN HIEROGLYPH-13EC8\0"
  "EGYPTIAN HIEROGLYPH-13EC9\0"
  "EGYPTIAN HIEROGLYPH-13ECA\0"
  "EGYPTIAN HIEROGLYPH-13ECB\0"
  "EGYPTIAN HIEROGLYPH-13ECC\0"
  "EGYPTIAN HIEROGLYPH-13ECD\0"
  "EGYPTIAN HIEROGLYPH-13ECE\0"
  "EGYPTIAN HIEROGLYPH-13ECF\0"
  "EGYPTIAN HIEROGLYPH-13ED0\0"
  "EGYPTIAN HIEROGLYPH-13ED1\0"
  "EGYPTIAN HIEROGLYPH-13ED2\0"
  "EGYPTIAN HIEROGLYPH-13ED3\0"
  "EGYPTIAN HIEROGLYPH-13ED4\0"
  "EGYPTIAN HIEROGLYPH-13ED5\0"
  "EGYPTIAN HIEROGLYPH-13ED6\0"
  "EGYPTIAN HIEROGLYPH-13ED7\0"
  "EGYPTIAN HIEROGLYPH-13ED8\0"
  "EGYPTIAN HIEROGLYPH-13ED9\0"
  "EGYPTIAN HIEROGLYPH-13EDA\0"
  "EGYPTIAN HIEROGLYPH-13EDB\0"
  "EGYPTIAN HIEROGLYPH-13EDC\0"
  "EGYPTIAN HIEROGLYPH-13EDD\0"
  "EGYPTIAN HIEROGLYPH-13EDE\0"
  "EGYPTIAN HIEROGLYPH-13EDF\0"
  "EGYPTIAN HIEROGLYPH-13EE0\0"
  "EGYPTIAN HIEROGLYPH-13EE1\0"
  "EGYPTIAN HIEROGLYPH-13EE2\0"
  "EGYPTIAN HIEROGLYPH-13EE3\0"
  "EGYPTIAN HIEROGLYPH-13EE4\0"
  "EGYPTIAN HIEROGLYPH-13EE5\0"
  "EGYPTIAN HIEROGLYPH-13EE6\0"
  "EGYPTIAN HIEROGLYPH-13EE7\0"
  "EGYPTIAN HIEROGLYPH-13EE8\0"
  "EGYPTIAN HIEROGLYPH-13EE9\0"
  "EGYPTIAN HIEROGLYPH-13EEA\0"
  "EGYPTIAN HIEROGLYPH-13EEB\0"
  "EGYPTIAN HIEROGLYPH-13EEC\0"
  "EGYPTIAN HIEROGLYPH-13EED\0"
  "EGYPTIAN HIEROGLYPH-13EEE\0"
  "EGYPTIAN HIEROGLYPH-13EEF\0"
  "EGYPTIAN HIEROGLYPH-13EF0\0"
  "EGYPTIAN HIEROGLYPH-13EF1\0"
  "EGYPTIAN HIEROGLYPH-13EF2\0"
  "EGYPTIAN HIEROGLYPH-13EF3\0"
  "EGYPTIAN HIEROGLYPH-13EF4\0"
  "EGYPTIAN HIEROGLYPH-13EF5\0"
  "EGYPTIAN HIEROGLYPH-13EF6\0"
  "EGYPTIAN HIEROGLYPH-13EF7\0"
  "EGYPTIAN HIEROGLYPH-13EF8\0"
  "EGYPTIAN HIEROGLYPH-13EF9\0"
  "EGYPTIAN HIEROGLYPH-13EFA\0"
  "EGYPTIAN HIEROGLYPH-13EFB\0"
  "EGYPTIAN HIEROGLYPH-13EFC\0"
  "EGYPTIAN HIEROGLYPH-13EFD\0"
  "EGYPTIAN HIEROGLYPH-13EFE\0"
  "EGYPTIAN HIEROGLYPH-13EFF\0"
  "EGYPTIAN HIEROGLYPH-13F00\0"
  "EGYPTIAN HIEROGLYPH-13F01\0"
  "EGYPTIAN HIEROGLYPH-13F02\0"
  "EGYPTIAN HIEROGLYPH-13F03\0"
  "EGYPTIAN HIEROGLYPH-13F04\0"
  "EGYPTIAN HIEROGLYPH-13F05\0"
  "EGYPTIAN HIEROGLYPH-13F06\0"
  "EGYPTIAN HIEROGLYPH-13F07\0"
  "EGYPTIAN HIEROGLYPH-13F08\0"
  "EGYPTIAN HIEROGLYPH-13F09\0"
  "EGYPTIAN HIEROGLYPH-13F0A\0"
  "EGYPTIAN HIEROGLYPH-13F0B\0"
  "EGYPTIAN HIEROGLYPH-13F0C\0"
  "EGYPTIAN HIEROGLYPH-13F0D\0"
  "EGYPTIAN HIEROGLYPH-13F0E\0"
  "EGYPTIAN HIEROGLYPH-13F0F\0"
  "EGYPTIAN HIEROGLYPH-13F10\0"
  "EGYPTIAN HIEROGLYPH-13F11\0"
  "EGYPTIAN HIEROGLYPH-13F12\0"
  "EGYPTIAN HIEROGLYPH-13F13\0"
  "EGYPTIAN HIEROGLYPH-13F14\0"
  "EGYPTIAN HIEROGLYPH-13F15\0"
  "EGYPTIAN HIEROGLYPH-13F16\0"
  "EGYPTIAN HIEROGLYPH-13F17\0"
  "EGYPTIAN HIEROGLYPH-13F18\0"
  "EGYPTIAN HIEROGLYPH-13F19\0"
  "EGYPTIAN HIEROGLYPH-13F1A\0"
  "EGYPTIAN HIEROGLYPH-13F1B\0"
  "EGYPTIAN HIEROGLYPH-13F1C\0"
  "EGYPTIAN HIEROGLYPH-13F1D\0"
  "EGYPTIAN HIEROGLYPH-13F1E\0"
  "EGYPTIAN HIEROGLYPH-13F1F\0"
  "EGYPTIAN HIEROGLYPH-13F20\0"
  "EGYPTIAN HIEROGLYPH-13F21\0"
  "EGYPTIAN HIEROGLYPH-13F22\0"
  "EGYPTIAN HIEROGLYPH-13F23\0"
  "EGYPTIAN HIEROGLYPH-13F24\0"
  "EGYPTIAN HIEROGLYPH-13F25\0"
  "EGYPTIAN HIEROGLYPH-13F26\0"
  "EGYPTIAN HIEROGLYPH-13F27\0"
  "EGYPTIAN HIEROGLYPH-13F28\0"
  "EGYPTIAN HIEROGLYPH-13F29\0"
  "EGYPTIAN HIEROGLYPH-13F2A\0"
  "EGYPTIAN HIEROGLYPH-13F2B\0"
  "EGYPTIAN HIEROGLYPH-13F2C\0"
  "EGYPTIAN HIEROGLYPH-13F2D\0"
  "EGYPTIAN HIEROGLYPH-13F2E\0"
  "EGYPTIAN HIEROGLYPH-13F2F\0"
  "EGYPTIAN HIEROGLYPH-13F30\0"
  "EGYPTIAN HIEROGLYPH-13F31\0"
  "EGYPTIAN HIEROGLYPH-13F32\0"
  "EGYPTIAN HIEROGLYPH-13F33\0"
  "EGYPTIAN HIEROGLYPH-13F34\0"
  "EGYPTIAN HIEROGLYPH-13F35\0"
  "EGYPTIAN HIEROGLYPH-13F36\0"
  "EGYPTIAN HIEROGLYPH-13F37\0"
  "EGYPTIAN HIEROGLYPH-13F38\0"
  "EGYPTIAN HIEROGLYPH-13F39\0"
  "EGYPTIAN HIEROGLYPH-13F3A\0"
  "EGYPTIAN HIEROGLYPH-13F3B\0"
  "EGYPTIAN HIEROGLYPH-13F3C\0"
  "EGYPTIAN HIEROGLYPH-13F3D\0"
  "EGYPTIAN HIEROGLYPH-13F3E\0"
  "EGYPTIAN HIEROGLYPH-13F3F\0"
  "EGYPTIAN HIEROGLYPH-13F40\0"
  "EGYPTIAN HIEROGLYPH-13F41\0"
  "EGYPTIAN HIEROGLYPH-13F42\0"
  "EGYPTIAN HIEROGLYPH-13F43\0"
  "EGYPTIAN HIEROGLYPH-13F44\0"
  "EGYPTIAN HIEROGLYPH-13F45\0"
  "EGYPTIAN HIEROGLYPH-13F46\0"
  "EGYPTIAN HIEROGLYPH-13F47\0"
  "EGYPTIAN HIEROGLYPH-13F48\0"
  "EGYPTIAN HIEROGLYPH-13F49\0"
  "EGYPTIAN HIEROGLYPH-13F4A\0"
  "EGYPTIAN HIEROGLYPH-13F4B\0"
  "EGYPTIAN HIEROGLYPH-13F4C\0"
  "EGYPTIAN HIEROGLYPH-13F4D\0"
  "EGYPTIAN HIEROGLYPH-13F4E\0"
  "EGYPTIAN HIEROGLYPH-13F4F\0"
  "EGYPTIAN HIEROGLYPH-13F50\0"
  "EGYPTIAN HIEROGLYPH-13F51\0"
  "EGYPTIAN HIEROGLYPH-13F52\0"
  "EGYPTIAN HIEROGLYPH-13F53\0"
  "EGYPTIAN HIEROGLYPH-13F54\0"
  "EGYPTIAN HIEROGLYPH-13F55\0"
  "EGYPTIAN HIEROGLYPH-13F56\0"
  "EGYPTIAN HIEROGLYPH-13F57\0"
  "EGYPTIAN HIEROGLYPH-13F58\0"
  "EGYPTIAN HIEROGLYPH-13F59\0"
  "EGYPTIAN HIEROGLYPH-13F5A\0"
  "EGYPTIAN HIEROGLYPH-13F5B\0"
  "EGYPTIAN HIEROGLYPH-13F5C\0"
  "EGYPTIAN HIEROGLYPH-13F5D\0"
  "EGYPTIAN HIEROGLYPH-13F5E\0"
  "EGYPTIAN HIEROGLYPH-13F5F\0"
  "EGYPTIAN HIEROGLYPH-13F60\0"
  "EGYPTIAN HIEROGLYPH-13F61\0"
  "EGYPTIAN HIEROGLYPH-13F62\0"
  "EGYPTIAN HIEROGLYPH-13F63\0"
  "EGYPTIAN HIEROGLYPH-13F64\0"
  "EGYPTIAN HIEROGLYPH-13F65\0"
  "EGYPTIAN HIEROGLYPH-13F66\0"
  "EGYPTIAN HIEROGLYPH-13F67\0"
  "EGYPTIAN HIEROGLYPH-13F68\0"
  "EGYPTIAN HIEROGLYPH-13F69\0"
  "EGYPTIAN HIEROGLYPH-13F6A\0"
  "EGYPTIAN HIEROGLYPH-13F6B\0"
  "EGYPTIAN HIEROGLYPH-13F6C\0"
  "EGYPTIAN HIEROGLYPH-13F6D\0"
  "EGYPTIAN HIEROGLYPH-13F6E\0"
  "EGYPTIAN HIEROGLYPH-13F6F\0"
  "EGYPTIAN HIEROGLYPH-13F70\0"
  "EGYPTIAN HIEROGLYPH-13F71\0"
  "EGYPTIAN HIEROGLYPH-13F72\0"
  "EGYPTIAN HIEROGLYPH-13F73\0"
  "EGYPTIAN HIEROGLYPH-13F74\0"
  "EGYPTIAN HIEROGLYPH-13F75\0"
  "EGYPTIAN HIEROGLYPH-13F76\0"
  "EGYPTIAN HIEROGLYPH-13F77\0"
  "EGYPTIAN HIEROGLYPH-13F78\0"
  "EGYPTIAN HIEROGLYPH-13F79\0"
  "EGYPTIAN HIEROGLYPH-13F7A\0"
  "EGYPTIAN HIEROGLYPH-13F7B\0"
  "EGYPTIAN HIEROGLYPH-13F7C\0"
  "EGYPTIAN HIEROGLYPH-13F7D\0"
  "EGYPTIAN HIEROGLYPH-13F7E\0"
  "EGYPTIAN HIEROGLYPH-13F7F\0"
  "EGYPTIAN HIEROGLYPH-13F80\0"
  "EGYPTIAN HIEROGLYPH-13F81\0"
  "EGYPTIAN HIEROGLYPH-13F82\0"
  "EGYPTIAN HIEROGLYPH-13F83\0"
  "EGYPTIAN HIEROGLYPH-13F84\0"
  "EGYPTIAN HIEROGLYPH-13F85\0"
  "EGYPTIAN HIEROGLYPH-13F86\0"
  "EGYPTIAN HIEROGLYPH-13F87\0"
  "EGYPTIAN HIEROGLYPH-13F88\0"
  "EGYPTIAN HIEROGLYPH-13F89\0"
  "EGYPTIAN HIEROGLYPH-13F8A\0"
  "EGYPTIAN HIEROGLYPH-13F8B\0"
  "EGYPTIAN HIEROGLYPH-13F8C\0"
  "EGYPTIAN HIEROGLYPH-13F8D\0"
  "EGYPTIAN HIEROGLYPH-13F8E\0"
  "EGYPTIAN HIEROGLYPH-13F8F\0"
  "EGYPTIAN HIEROGLYPH-13F90\0"
  "EGYPTIAN HIEROGLYPH-13F91\0"
  "EGYPTIAN HIEROGLYPH-13F92\0"
  "EGYPTIAN HIEROGLYPH-13F93\0"
  "EGYPTIAN HIEROGLYPH-13F94\0"
  "EGYPTIAN HIEROGLYPH-13F95\0"
  "EGYPTIAN HIEROGLYPH-13F96\0"
  "EGYPTIAN HIEROGLYPH-13F97\0"
  "EGYPTIAN HIEROGLYPH-13F98\0"
  "EGYPTIAN HIEROGLYPH-13F99\0"
  "EGYPTIAN HIEROGLYPH-13F9A\0"
  "EGYPTIAN HIEROGLYPH-13F9B\0"
  "EGYPTIAN HIEROGLYPH-13F9C\0"
  "EGYPTIAN HIEROGLYPH-13F9D\0"
  "EGYPTIAN HIEROGLYPH-13F9E\0"
  "EGYPTIAN HIEROGLYPH-13F9F\0"
  "EGYPTIAN HIEROGLYPH-13FA0\0"
  "EGYPTIAN HIEROGLYPH-13FA1\0"
  "EGYPTIAN HIEROGLYPH-13FA2\0"
  "EGYPTIAN HIEROGLYPH-13FA3\0"
  "EGYPTIAN HIEROGLYPH-13FA4\0"
  "EGYPTIAN HIEROGLYPH-13FA5\0"
  "EGYPTIAN HIEROGLYPH-13FA6\0"
  "EGYPTIAN HIEROGLYPH-13FA7\0"
  "EGYPTIAN HIEROGLYPH-13FA8\0"
  "EGYPTIAN HIEROGLYPH-13FA9\0"
  "EGYPTIAN HIEROGLYPH-13FAA\0"
  "EGYPTIAN HIEROGLYPH-13FAB\0"
  "EGYPTIAN HIEROGLYPH-13FAC\0"
  "EGYPTIAN HIEROGLYPH-13FAD\0"
  "EGYPTIAN HIEROGLYPH-13FAE\0"
  "EGYPTIAN HIEROGLYPH-13FAF\0"
  "EGYPTIAN HIEROGLYPH-13FB0\0"
  "EGYPTIAN HIEROGLYPH-13FB1\0"
  "EGYPTIAN HIEROGLYPH-13FB2\0"
  "EGYPTIAN HIEROGLYPH-13FB3\0"
  "EGYPTIAN HIEROGLYPH-13FB4\0"
  "EGYPTIAN HIEROGLYPH-13FB5\0"
  "EGYPTIAN HIEROGLYPH-13FB6\0"
  "EGYPTIAN HIEROGLYPH-13FB7\0"
  "EGYPTIAN HIEROGLYPH-13FB8\0"
  "EGYPTIAN HIEROGLYPH-13FB9\0"
  "EGYPTIAN HIEROGLYPH-13FBA\0"
  "EGYPTIAN HIEROGLYPH-13FBB\0"
  "EGYPTIAN HIEROGLYPH-13FBC\0"
  "EGYPTIAN HIEROGLYPH-13FBD\0"
  "EGYPTIAN HIEROGLYPH-13FBE\0"
  "EGYPTIAN HIEROGLYPH-13FBF\0"
  "EGYPTIAN HIEROGLYPH-13FC0\0"
  "EGYPTIAN HIEROGLYPH-13FC1\0"
  "EGYPTIAN HIEROGLYPH-13FC2\0"
  "EGYPTIAN HIEROGLYPH-13FC3\0"
  "EGYPTIAN HIEROGLYPH-13FC4\0"
  "EGYPTIAN HIEROGLYPH-13FC5\0"
  "EGYPTIAN HIEROGLYPH-13FC6\0"
  "EGYPTIAN HIEROGLYPH-13FC7\0"
  "EGYPTIAN HIEROGLYPH-13FC8\0"
  "EGYPTIAN HIEROGLYPH-13FC9\0"
  "EGYPTIAN HIEROGLYPH-13FCA\0"
  "EGYPTIAN HIEROGLYPH-13FCB\0"
  "EGYPTIAN HIEROGLYPH-13FCC\0"
  "EGYPTIAN HIEROGLYPH-13FCD\0"
  "EGYPTIAN HIEROGLYPH-13FCE\0"
  "EGYPTIAN HIEROGLYPH-13FCF\0"
  "EGYPTIAN HIEROGLYPH-13FD0\0"
  "EGYPTIAN HIEROGLYPH-13FD1\0"
  "EGYPTIAN HIEROGLYPH-13FD2\0"
  "EGYPTIAN HIEROGLYPH-13FD3\0"
  "EGYPTIAN HIEROGLYPH-13FD4\0"
  "EGYPTIAN HIEROGLYPH-13FD5\0"
  "EGYPTIAN HIEROGLYPH-13FD6\0"
  "EGYPTIAN HIEROGLYPH-13FD7\0"
  "EGYPTIAN HIEROGLYPH-13FD8\0"
  "EGYPTIAN HIEROGLYPH-13FD9\0"
  "EGYPTIAN HIEROGLYPH-13FDA\0"
  "EGYPTIAN HIEROGLYPH-13FDB\0"
  "EGYPTIAN HIEROGLYPH-13FDC\0"
  "EGYPTIAN HIEROGLYPH-13FDD\0"
  "EGYPTIAN HIEROGLYPH-13FDE\0"
  "EGYPTIAN HIEROGLYPH-13FDF\0"
  "EGYPTIAN HIEROGLYPH-13FE0\0"
  "EGYPTIAN HIEROGLYPH-13FE1\0"
  "EGYPTIAN HIEROGLYPH-13FE2\0"
  "EGYPTIAN HIEROGLYPH-13FE3\0"
  "EGYPTIAN HIEROGLYPH-13FE4\0"
  "EGYPTIAN HIEROGLYPH-13FE5\0"
  "EGYPTIAN HIEROGLYPH-13FE6\0"
  "EGYPTIAN HIEROGLYPH-13FE7\0"
  "EGYPTIAN HIEROGLYPH-13FE8\0"
  "EGYPTIAN HIEROGLYPH-13FE9\0"
  "EGYPTIAN HIEROGLYPH-13FEA\0"
  "EGYPTIAN HIEROGLYPH-13FEB\0"
  "EGYPTIAN HIEROGLYPH-13FEC\0"
  "EGYPTIAN HIEROGLYPH-13FED\0"
  "EGYPTIAN HIEROGLYPH-13FEE\0"
  "EGYPTIAN HIEROGLYPH-13FEF\0"
  "EGYPTIAN HIEROGLYPH-13FF0\0"
  "EGYPTIAN HIEROGLYPH-13FF1\0"
  "EGYPTIAN HIEROGLYPH-13FF2\0"
  "EGYPTIAN HIEROGLYPH-13FF3\0"
  "EGYPTIAN HIEROGLYPH-13FF4\0"
  "EGYPTIAN HIEROGLYPH-13FF5\0"
  "EGYPTIAN HIEROGLYPH-13FF6\0"
  "EGYPTIAN HIEROGLYPH-13FF7\0"
  "EGYPTIAN HIEROGLYPH-13FF8\0"
  "EGYPTIAN HIEROGLYPH-13FF9\0"
  "EGYPTIAN HIEROGLYPH-13FFA\0"
  "EGYPTIAN HIEROGLYPH-13FFB\0"
  "EGYPTIAN HIEROGLYPH-13FFC\0"
  "EGYPTIAN HIEROGLYPH-13FFD\0"
  "EGYPTIAN HIEROGLYPH-13FFE\0"
  "EGYPTIAN HIEROGLYPH-13FFF\0"
  "EGYPTIAN HIEROGLYPH-14000\0"
  "EGYPTIAN HIEROGLYPH-14001\0"
  "EGYPTIAN HIEROGLYPH-14002\0"
  "EGYPTIAN HIEROGLYPH-14003\0"
  "EGYPTIAN HIEROGLYPH-14004\0"
  "EGYPTIAN HIEROGLYPH-14005\0"
  "EGYPTIAN HIEROGLYPH-14006\0"
  "EGYPTIAN HIEROGLYPH-14007\0"
  "EGYPTIAN HIEROGLYPH-14008\0"
  "EGYPTIAN HIEROGLYPH-14009\0"
  "EGYPTIAN HIEROGLYPH-1400A\0"
  "EGYPTIAN HIEROGLYPH-1400B\0"
  "EGYPTIAN HIEROGLYPH-1400C\0"
  "EGYPTIAN HIEROGLYPH-1400D\0"
  "EGYPTIAN HIEROGLYPH-1400E\0"
  "EGYPTIAN HIEROGLYPH-1400F\0"
  "EGYPTIAN HIEROGLYPH-14010\0"
  "EGYPTIAN HIEROGLYPH-14011\0"
  "EGYPTIAN HIEROGLYPH-14012\0"
  "EGYPTIAN HIEROGLYPH-14013\0"
  "EGYPTIAN HIEROGLYPH-14014\0"
  "EGYPTIAN HIEROGLYPH-14015\0"
  "EGYPTIAN HIEROGLYPH-14016\0"
  "EGYPTIAN HIEROGLYPH-14017\0"
  "EGYPTIAN HIEROGLYPH-14018\0"
  "EGYPTIAN HIEROGLYPH-14019\0"
  "EGYPTIAN HIEROGLYPH-1401A\0"
  "EGYPTIAN HIEROGLYPH-1401B\0"
  "EGYPTIAN HIEROGLYPH-1401C\0"
  "EGYPTIAN HIEROGLYPH-1401D\0"
  "EGYPTIAN HIEROGLYPH-1401E\0"
  "EGYPTIAN HIEROGLYPH-1401F\0"
  "EGYPTIAN HIEROGLYPH-14020\0"
  "EGYPTIAN HIEROGLYPH-14021\0"
  "EGYPTIAN HIEROGLYPH-14022\0"
  "EGYPTIAN HIEROGLYPH-14023\0"
  "EGYPTIAN HIEROGLYPH-14024\0"
  "EGYPTIAN HIEROGLYPH-14025\0"
  "EGYPTIAN HIEROGLYPH-14026\0"
  "EGYPTIAN HIEROGLYPH-14027\0"
  "EGYPTIAN HIEROGLYPH-14028\0"
  "EGYPTIAN HIEROGLYPH-14029\0"
  "EGYPTIAN HIEROGLYPH-1402A\0"
  "EGYPTIAN HIEROGLYPH-1402B\0"
  "EGYPTIAN HIEROGLYPH-1402C\0"
  "EGYPTIAN HIEROGLYPH-1402D\0"
  "EGYPTIAN HIEROGLYPH-1402E\0"
  "EGYPTIAN HIEROGLYPH-1402F\0"
  "EGYPTIAN HIEROGLYPH-14030\0"
  "EGYPTIAN HIEROGLYPH-14031\0"
  "EGYPTIAN HIEROGLYPH-14032\0"
  "EGYPTIAN HIEROGLYPH-14033\0"
  "EGYPTIAN HIEROGLYPH-14034\0"
  "EGYPTIAN HIEROGLYPH-14035\0"
  "EGYPTIAN HIEROGLYPH-14036\0"
  "EGYPTIAN HIEROGLYPH-14037\0"
  "EGYPTIAN HIEROGLYPH-14038\0"
  "EGYPTIAN HIEROGLYPH-14039\0"
  "EGYPTIAN HIEROGLYPH-1403A\0"
  "EGYPTIAN HIEROGLYPH-1403B\0"
  "EGYPTIAN HIEROGLYPH-1403C\0"
  "EGYPTIAN HIEROGLYPH-1403D\0"
  "EGYPTIAN HIEROGLYPH-1403E\0"
  "EGYPTIAN HIEROGLYPH-1403F\0"
  "EGYPTIAN HIEROGLYPH-14040\0"
  "EGYPTIAN HIEROGLYPH-14041\0"
  "EGYPTIAN HIEROGLYPH-14042\0"
  "EGYPTIAN HIEROGLYPH-14043\0"
  "EGYPTIAN HIEROGLYPH-14044\0"
  "EGYPTIAN HIEROGLYPH-14045\0"
  "EGYPTIAN HIEROGLYPH-14046\0"
  "EGYPTIAN HIEROGLYPH-14047\0"
  "EGYPTIAN HIEROGLYPH-14048\0"
  "EGYPTIAN HIEROGLYPH-14049\0"
  "EGYPTIAN HIEROGLYPH-1404A\0"
  "EGYPTIAN HIEROGLYPH-1404B\0"
  "EGYPTIAN HIEROGLYPH-1404C\0"
  "EGYPTIAN HIEROGLYPH-1404D\0"
  "EGYPTIAN HIEROGLYPH-1404E\0"
  "EGYPTIAN HIEROGLYPH-1404F\0"
  "EGYPTIAN HIEROGLYPH-14050\0"
  "EGYPTIAN HIEROGLYPH-14051\0"
  "EGYPTIAN HIEROGLYPH-14052\0"
  "EGYPTIAN HIEROGLYPH-14053\0"
  "EGYPTIAN HIEROGLYPH-14054\0"
  "EGYPTIAN HIEROGLYPH-14055\0"
  "EGYPTIAN HIEROGLYPH-14056\0"
  "EGYPTIAN HIEROGLYPH-14057\0"
  "EGYPTIAN HIEROGLYPH-14058\0"
  "EGYPTIAN HIEROGLYPH-14059\0"
  "EGYPTIAN HIEROGLYPH-1405A\0"
  "EGYPTIAN HIEROGLYPH-1405B\0"
  "EGYPTIAN HIEROGLYPH-1405C\0"
  "EGYPTIAN HIEROGLYPH-1405D\0"
  "EGYPTIAN HIEROGLYPH-1405E\0"
  "EGYPTIAN HIEROGLYPH-1405F\0"
  "EGYPTIAN HIEROGLYPH-14060\0"
  "EGYPTIAN HIEROGLYPH-14061\0"
  "EGYPTIAN HIEROGLYPH-14062\0"
  "EGYPTIAN HIEROGLYPH-14063\0"
  "EGYPTIAN HIEROGLYPH-14064\0"
  "EGYPTIAN HIEROGLYPH-14065\0"
  "EGYPTIAN HIEROGLYPH-14066\0"
  "EGYPTIAN HIEROGLYPH-14067\0"
  "EGYPTIAN HIEROGLYPH-14068\0"
  "EGYPTIAN HIEROGLYPH-14069\0"
  "EGYPTIAN HIEROGLYPH-1406A\0"
  "EGYPTIAN HIEROGLYPH-1406B\0"
  "EGYPTIAN HIEROGLYPH-1406C\0"
  "EGYPTIAN HIEROGLYPH-1406D\0"
  "EGYPTIAN HIEROGLYPH-1406E\0"
  "EGYPTIAN HIEROGLYPH-1406F\0"
  "EGYPTIAN HIEROGLYPH-14070\0"
  "EGYPTIAN HIEROGLYPH-14071\0"
  "EGYPTIAN HIEROGLYPH-14072\0"
  "EGYPTIAN HIEROGLYPH-14073\0"
  "EGYPTIAN HIEROGLYPH-14074\0"
  "EGYPTIAN HIEROGLYPH-14075\0"
  "EGYPTIAN HIEROGLYPH-14076\0"
  "EGYPTIAN HIEROGLYPH-14077\0"
  "EGYPTIAN HIEROGLYPH-14078\0"
  "EGYPTIAN HIEROGLYPH-14079\0"
  "EGYPTIAN HIEROGLYPH-1407A\0"
  "EGYPTIAN HIEROGLYPH-1407B\0"
  "EGYPTIAN HIEROGLYPH-1407C\0"
  "EGYPTIAN HIEROGLYPH-1407D\0"
  "EGYPTIAN HIEROGLYPH-1407E\0"
  "EGYPTIAN HIEROGLYPH-1407F\0"
  "EGYPTIAN HIEROGLYPH-14080\0"
  "EGYPTIAN HIEROGLYPH-14081\0"
  "EGYPTIAN HIEROGLYPH-14082\0"
  "EGYPTIAN HIEROGLYPH-14083\0"
  "EGYPTIAN HIEROGLYPH-14084\0"
  "EGYPTIAN HIEROGLYPH-14085\0"
  "EGYPTIAN HIEROGLYPH-14086\0"
  "EGYPTIAN HIEROGLYPH-14087\0"
  "EGYPTIAN HIEROGLYPH-14088\0"
  "EGYPTIAN HIEROGLYPH-14089\0"
  "EGYPTIAN HIEROGLYPH-1408A\0"
  "EGYPTIAN HIEROGLYPH-1408B\0"
  "EGYPTIAN HIEROGLYPH-1408C\0"
  "EGYPTIAN HIEROGLYPH-1408D\0"
  "EGYPTIAN HIEROGLYPH-1408E\0"
  "EGYPTIAN HIEROGLYPH-1408F\0"
  "EGYPTIAN HIEROGLYPH-14090\0"
  "EGYPTIAN HIEROGLYPH-14091\0"
  "EGYPTIAN HIEROGLYPH-14092\0"
  "EGYPTIAN HIEROGLYPH-14093\0"
  "EGYPTIAN HIEROGLYPH-14094\0"
  "EGYPTIAN HIEROGLYPH-14095\0"
  "EGYPTIAN HIEROGLYPH-14096\0"
  "EGYPTIAN HIEROGLYPH-14097\0"
  "EGYPTIAN HIEROGLYPH-14098\0"
  "EGYPTIAN HIEROGLYPH-14099\0"
  "EGYPTIAN HIEROGLYPH-1409A\0"
  "EGYPTIAN HIEROGLYPH-1409B\0"
  "EGYPTIAN HIEROGLYPH-1409C\0"
  "EGYPTIAN HIEROGLYPH-1409D\0"
  "EGYPTIAN HIEROGLYPH-1409E\0"
  "EGYPTIAN HIEROGLYPH-1409F\0"
  "EGYPTIAN HIEROGLYPH-140A0\0"
  "EGYPTIAN HIEROGLYPH-140A1\0"
  "EGYPTIAN HIEROGLYPH-140A2\0"
  "EGYPTIAN HIEROGLYPH-140A3\0"
  "EGYPTIAN HIEROGLYPH-140A4\0"
  "EGYPTIAN HIEROGLYPH-140A5\0"
  "EGYPTIAN HIEROGLYPH-140A6\0"
  "EGYPTIAN HIEROGLYPH-140A7\0"
  "EGYPTIAN HIEROGLYPH-140A8\0"
  "EGYPTIAN HIEROGLYPH-140A9\0"
  "EGYPTIAN HIEROGLYPH-140AA\0"
  "EGYPTIAN HIEROGLYPH-140AB\0"
  "EGYPTIAN HIEROGLYPH-140AC\0"
  "EGYPTIAN HIEROGLYPH-140AD\0"
  "EGYPTIAN HIEROGLYPH-140AE\0"
  "EGYPTIAN HIEROGLYPH-140AF\0"
  "EGYPTIAN HIEROGLYPH-140B0\0"
  "EGYPTIAN HIEROGLYPH-140B1\0"
  "EGYPTIAN HIEROGLYPH-140B2\0"
  "EGYPTIAN HIEROGLYPH-140B3\0"
  "EGYPTIAN HIEROGLYPH-140B4\0"
  "EGYPTIAN HIEROGLYPH-140B5\0"
  "EGYPTIAN HIEROGLYPH-140B6\0"
  "EGYPTIAN HIEROGLYPH-140B7\0"
  "EGYPTIAN HIEROGLYPH-140B8\0"
  "EGYPTIAN HIEROGLYPH-140B9\0"
  "EGYPTIAN HIEROGLYPH-140BA\0"
  "EGYPTIAN HIEROGLYPH-140BB\0"
  "EGYPTIAN HIEROGLYPH-140BC\0"
  "EGYPTIAN HIEROGLYPH-140BD\0"
  "EGYPTIAN HIEROGLYPH-140BE\0"
  "EGYPTIAN HIEROGLYPH-140BF\0"
  "EGYPTIAN HIEROGLYPH-140C0\0"
  "EGYPTIAN HIEROGLYPH-140C1\0"
  "EGYPTIAN HIEROGLYPH-140C2\0"
  "EGYPTIAN HIEROGLYPH-140C3\0"
  "EGYPTIAN HIEROGLYPH-140C4\0"
  "EGYPTIAN HIEROGLYPH-140C5\0"
  "EGYPTIAN HIEROGLYPH-140C6\0"
  "EGYPTIAN HIEROGLYPH-140C7\0"
  "EGYPTIAN HIEROGLYPH-140C8\0"
  "EGYPTIAN HIEROGLYPH-140C9\0"
  "EGYPTIAN HIEROGLYPH-140CA\0"
  "EGYPTIAN HIEROGLYPH-140CB\0"
  "EGYPTIAN HIEROGLYPH-140CC\0"
  "EGYPTIAN HIEROGLYPH-140CD\0"
  "EGYPTIAN HIEROGLYPH-140CE\0"
  "EGYPTIAN HIEROGLYPH-140CF\0"
  "EGYPTIAN HIEROGLYPH-140D0\0"
  "EGYPTIAN HIEROGLYPH-140D1\0"
  "EGYPTIAN HIEROGLYPH-140D2\0"
  "EGYPTIAN HIEROGLYPH-140D3\0"
  "EGYPTIAN HIEROGLYPH-140D4\0"
  "EGYPTIAN HIEROGLYPH-140D5\0"
  "EGYPTIAN HIEROGLYPH-140D6\0"
  "EGYPTIAN HIEROGLYPH-140D7\0"
  "EGYPTIAN HIEROGLYPH-140D8\0"
  "EGYPTIAN HIEROGLYPH-140D9\0"
  "EGYPTIAN HIEROGLYPH-140DA\0"
  "EGYPTIAN HIEROGLYPH-140DB\0"
  "EGYPTIAN HIEROGLYPH-140DC\0"
  "EGYPTIAN HIEROGLYPH-140DD\0"
  "EGYPTIAN HIEROGLYPH-140DE\0"
  "EGYPTIAN HIEROGLYPH-140DF\0"
  "EGYPTIAN HIEROGLYPH-140E0\0"
  "EGYPTIAN HIEROGLYPH-140E1\0"
  "EGYPTIAN HIEROGLYPH-140E2\0"
  "EGYPTIAN HIEROGLYPH-140E3\0"
  "EGYPTIAN HIEROGLYPH-140E4\0"
  "EGYPTIAN HIEROGLYPH-140E5\0"
  "EGYPTIAN HIEROGLYPH-140E6\0"
  "EGYPTIAN HIEROGLYPH-140E7\0"
  "EGYPTIAN HIEROGLYPH-140E8\0"
  "EGYPTIAN HIEROGLYPH-140E9\0"
  "EGYPTIAN HIEROGLYPH-140EA\0"
  "EGYPTIAN HIEROGLYPH-140EB\0"
  "EGYPTIAN HIEROGLYPH-140EC\0"
  "EGYPTIAN HIEROGLYPH-140ED\0"
  "EGYPTIAN HIEROGLYPH-140EE\0"
  "EGYPTIAN HIEROGLYPH-140EF\0"
  "EGYPTIAN HIEROGLYPH-140F0\0"
  "EGYPTIAN HIEROGLYPH-140F1\0"
  "EGYPTIAN HIEROGLYPH-140F2\0"
  "EGYPTIAN HIEROGLYPH-140F3\0"
  "EGYPTIAN HIEROGLYPH-140F4\0"
  "EGYPTIAN HIEROGLYPH-140F5\0"
  "EGYPTIAN HIEROGLYPH-140F6\0"
  "EGYPTIAN HIEROGLYPH-140F7\0"
  "EGYPTIAN HIEROGLYPH-140F8\0"
  "EGYPTIAN HIEROGLYPH-140F9\0"
  "EGYPTIAN HIEROGLYPH-140FA\0"
  "EGYPTIAN HIEROGLYPH-140FB\0"
  "EGYPTIAN HIEROGLYPH-140FC\0"
  "EGYPTIAN HIEROGLYPH-140FD\0"
  "EGYPTIAN HIEROGLYPH-140FE\0"
  "EGYPTIAN HIEROGLYPH-140FF\0"
  "EGYPTIAN HIEROGLYPH-14100\0"
  "EGYPTIAN HIEROGLYPH-14101\0"
  "EGYPTIAN HIEROGLYPH-14102\0"
  "EGYPTIAN HIEROGLYPH-14103\0"
  "EGYPTIAN HIEROGLYPH-14104\0"
  "EGYPTIAN HIEROGLYPH-14105\0"
  "EGYPTIAN HIEROGLYPH-14106\0"
  "EGYPTIAN HIEROGLYPH-14107\0"
  "EGYPTIAN HIEROGLYPH-14108\0"
  "EGYPTIAN HIEROGLYPH-14109\0"
  "EGYPTIAN HIEROGLYPH-1410A\0"
  "EGYPTIAN HIEROGLYPH-1410B\0"
  "EGYPTIAN HIEROGLYPH-1410C\0"
  "EGYPTIAN HIEROGLYPH-1410D\0"
  "EGYPTIAN HIEROGLYPH-1410E\0"
  "EGYPTIAN HIEROGLYPH-1410F\0"
  "EGYPTIAN HIEROGLYPH-14110\0"
  "EGYPTIAN HIEROGLYPH-14111\0"
  "EGYPTIAN HIEROGLYPH-14112\0"
  "EGYPTIAN HIEROGLYPH-14113\0"
  "EGYPTIAN HIEROGLYPH-14114\0"
  "EGYPTIAN HIEROGLYPH-14115\0"
  "EGYPTIAN HIEROGLYPH-14116\0"
  "EGYPTIAN HIEROGLYPH-14117\0"
  "EGYPTIAN HIEROGLYPH-14118\0"
  "EGYPTIAN HIEROGLYPH-14119\0"
  "EGYPTIAN HIEROGLYPH-1411A\0"
  "EGYPTIAN HIEROGLYPH-1411B\0"
  "EGYPTIAN HIEROGLYPH-1411C\0"
  "EGYPTIAN HIEROGLYPH-1411D\0"
  "EGYPTIAN HIEROGLYPH-1411E\0"
  "EGYPTIAN HIEROGLYPH-1411F\0"
  "EGYPTIAN HIEROGLYPH-14120\0"
  "EGYPTIAN HIEROGLYPH-14121\0"
  "EGYPTIAN HIEROGLYPH-14122\0"
  "EGYPTIAN HIEROGLYPH-14123\0"
  "EGYPTIAN HIEROGLYPH-14124\0"
  "EGYPTIAN HIEROGLYPH-14125\0"
  "EGYPTIAN HIEROGLYPH-14126\0"
  "EGYPTIAN HIEROGLYPH-14127\0"
  "EGYPTIAN HIEROGLYPH-14128\0"
  "EGYPTIAN HIEROGLYPH-14129\0"
  "EGYPTIAN HIEROGLYPH-1412A\0"
  "EGYPTIAN HIEROGLYPH-1412B\0"
  "EGYPTIAN HIEROGLYPH-1412C\0"
  "EGYPTIAN HIEROGLYPH-1412D\0"
  "EGYPTIAN HIEROGLYPH-1412E\0"
  "EGYPTIAN HIEROGLYPH-1412F\0"
  "EGYPTIAN HIEROGLYPH-14130\0"
  "EGYPTIAN HIEROGLYPH-14131\0"
  "EGYPTIAN HIEROGLYPH-14132\0"
  "EGYPTIAN HIEROGLYPH-14133\0"
  "EGYPTIAN HIEROGLYPH-14134\0"
  "EGYPTIAN HIEROGLYPH-14135\0"
  "EGYPTIAN HIEROGLYPH-14136\0"
  "EGYPTIAN HIEROGLYPH-14137\0"
  "EGYPTIAN HIEROGLYPH-14138\0"
  "EGYPTIAN HIEROGLYPH-14139\0"
  "EGYPTIAN HIEROGLYPH-1413A\0"
  "EGYPTIAN HIEROGLYPH-1413B\0"
  "EGYPTIAN HIEROGLYPH-1413C\0"
  "EGYPTIAN HIEROGLYPH-1413D\0"
  "EGYPTIAN HIEROGLYPH-1413E\0"
  "EGYPTIAN HIEROGLYPH-1413F\0"
  "EGYPTIAN HIEROGLYPH-14140\0"
  "EGYPTIAN HIEROGLYPH-14141\0"
  "EGYPTIAN HIEROGLYPH-14142\0"
  "EGYPTIAN HIEROGLYPH-14143\0"
  "EGYPTIAN HIEROGLYPH-14144\0"
  "EGYPTIAN HIEROGLYPH-14145\0"
  "EGYPTIAN HIEROGLYPH-14146\0"
  "EGYPTIAN HIEROGLYPH-14147\0"
  "EGYPTIAN HIEROGLYPH-14148\0"
  "EGYPTIAN HIEROGLYPH-14149\0"
  "EGYPTIAN HIEROGLYPH-1414A\0"
  "EGYPTIAN HIEROGLYPH-1414B\0"
  "EGYPTIAN HIEROGLYPH-1414C\0"
  "EGYPTIAN HIEROGLYPH-1414D\0"
  "EGYPTIAN HIEROGLYPH-1414E\0"
  "EGYPTIAN HIEROGLYPH-1414F\0"
  "EGYPTIAN HIEROGLYPH-14150\0"
  "EGYPTIAN HIEROGLYPH-14151\0"
  "EGYPTIAN HIEROGLYPH-14152\0"
  "EGYPTIAN HIEROGLYPH-14153\0"
  "EGYPTIAN HIEROGLYPH-14154\0"
  "EGYPTIAN HIEROGLYPH-14155\0"
  "EGYPTIAN HIEROGLYPH-14156\0"
  "EGYPTIAN HIEROGLYPH-14157\0"
  "EGYPTIAN HIEROGLYPH-14158\0"
  "EGYPTIAN HIEROGLYPH-14159\0"
  "EGYPTIAN HIEROGLYPH-1415A\0"
  "EGYPTIAN HIEROGLYPH-1415B\0"
  "EGYPTIAN HIEROGLYPH-1415C\0"
  "EGYPTIAN HIEROGLYPH-1415D\0"
  "EGYPTIAN HIEROGLYPH-1415E\0"
  "EGYPTIAN HIEROGLYPH-1415F\0"
  "EGYPTIAN HIEROGLYPH-14160\0"
  "EGYPTIAN HIEROGLYPH-14161\0"
  "EGYPTIAN HIEROGLYPH-14162\0"
  "EGYPTIAN HIEROGLYPH-14163\0"
  "EGYPTIAN HIEROGLYPH-14164\0"
  "EGYPTIAN HIEROGLYPH-14165\0"
  "EGYPTIAN HIEROGLYPH-14166\0"
  "EGYPTIAN HIEROGLYPH-14167\0"
  "EGYPTIAN HIEROGLYPH-14168\0"
  "EGYPTIAN HIEROGLYPH-14169\0"
  "EGYPTIAN HIEROGLYPH-1416A\0"
  "EGYPTIAN HIEROGLYPH-1416B\0"
  "EGYPTIAN HIEROGLYPH-1416C\0"
  "EGYPTIAN HIEROGLYPH-1416D\0"
  "EGYPTIAN HIEROGLYPH-1416E\0"
  "EGYPTIAN HIEROGLYPH-1416F\0"
  "EGYPTIAN HIEROGLYPH-14170\0"
  "EGYPTIAN HIEROGLYPH-14171\0"
  "EGYPTIAN HIEROGLYPH-14172\0"
  "EGYPTIAN HIEROGLYPH-14173\0"
  "EGYPTIAN HIEROGLYPH-14174\0"
  "EGYPTIAN HIEROGLYPH-14175\0"
  "EGYPTIAN HIEROGLYPH-14176\0"
  "EGYPTIAN HIEROGLYPH-14177\0"
  "EGYPTIAN HIEROGLYPH-14178\0"
  "EGYPTIAN HIEROGLYPH-14179\0"
  "EGYPTIAN HIEROGLYPH-1417A\0"
  "EGYPTIAN HIEROGLYPH-1417B\0"
  "EGYPTIAN HIEROGLYPH-1417C\0"
  "EGYPTIAN HIEROGLYPH-1417D\0"
  "EGYPTIAN HIEROGLYPH-1417E\0"
  "EGYPTIAN HIEROGLYPH-1417F\0"
  "EGYPTIAN HIEROGLYPH-14180\0"
  "EGYPTIAN HIEROGLYPH-14181\0"
  "EGYPTIAN HIEROGLYPH-14182\0"
  "EGYPTIAN HIEROGLYPH-14183\0"
  "EGYPTIAN HIEROGLYPH-14184\0"
  "EGYPTIAN HIEROGLYPH-14185\0"
  "EGYPTIAN HIEROGLYPH-14186\0"
  "EGYPTIAN HIEROGLYPH-14187\0"
  "EGYPTIAN HIEROGLYPH-14188\0"
  "EGYPTIAN HIEROGLYPH-14189\0"
  "EGYPTIAN HIEROGLYPH-1418A\0"
  "EGYPTIAN HIEROGLYPH-1418B\0"
  "EGYPTIAN HIEROGLYPH-1418C\0"
  "EGYPTIAN HIEROGLYPH-1418D\0"
  "EGYPTIAN HIEROGLYPH-1418E\0"
  "EGYPTIAN HIEROGLYPH-1418F\0"
  "EGYPTIAN HIEROGLYPH-14190\0"
  "EGYPTIAN HIEROGLYPH-14191\0"
  "EGYPTIAN HIEROGLYPH-14192\0"
  "EGYPTIAN HIEROGLYPH-14193\0"
  "EGYPTIAN HIEROGLYPH-14194\0"
  "EGYPTIAN HIEROGLYPH-14195\0"
  "EGYPTIAN HIEROGLYPH-14196\0"
  "EGYPTIAN HIEROGLYPH-14197\0"
  "EGYPTIAN HIEROGLYPH-14198\0"
  "EGYPTIAN HIEROGLYPH-14199\0"
  "EGYPTIAN HIEROGLYPH-1419A\0"
  "EGYPTIAN HIEROGLYPH-1419B\0"
  "EGYPTIAN HIEROGLYPH-1419C\0"
  "EGYPTIAN HIEROGLYPH-1419D\0"
  "EGYPTIAN HIEROGLYPH-1419E\0"
  "EGYPTIAN HIEROGLYPH-1419F\0"
  "EGYPTIAN HIEROGLYPH-141A0\0"
  "EGYPTIAN HIEROGLYPH-141A1\0"
  "EGYPTIAN HIEROGLYPH-141A2\0"
  "EGYPTIAN HIEROGLYPH-141A3\0"
  "EGYPTIAN HIEROGLYPH-141A4\0"
  "EGYPTIAN HIEROGLYPH-141A5\0"
  "EGYPTIAN HIEROGLYPH-141A6\0"
  "EGYPTIAN HIEROGLYPH-141A7\0"
  "EGYPTIAN HIEROGLYPH-141A8\0"
  "EGYPTIAN HIEROGLYPH-141A9\0"
  "EGYPTIAN HIEROGLYPH-141AA\0"
  "EGYPTIAN HIEROGLYPH-141AB\0"
  "EGYPTIAN HIEROGLYPH-141AC\0"
  "EGYPTIAN HIEROGLYPH-141AD\0"
  "EGYPTIAN HIEROGLYPH-141AE\0"
  "EGYPTIAN HIEROGLYPH-141AF\0"
  "EGYPTIAN HIEROGLYPH-141B0\0"
  "EGYPTIAN HIEROGLYPH-141B1\0"
  "EGYPTIAN HIEROGLYPH-141B2\0"
  "EGYPTIAN HIEROGLYPH-141B3\0"
  "EGYPTIAN HIEROGLYPH-141B4\0"
  "EGYPTIAN HIEROGLYPH-141B5\0"
  "EGYPTIAN HIEROGLYPH-141B6\0"
  "EGYPTIAN HIEROGLYPH-141B7\0"
  "EGYPTIAN HIEROGLYPH-141B8\0"
  "EGYPTIAN HIEROGLYPH-141B9\0"
  "EGYPTIAN HIEROGLYPH-141BA\0"
  "EGYPTIAN HIEROGLYPH-141BB\0"
  "EGYPTIAN HIEROGLYPH-141BC\0"
  "EGYPTIAN HIEROGLYPH-141BD\0"
  "EGYPTIAN HIEROGLYPH-141BE\0"
  "EGYPTIAN HIEROGLYPH-141BF\0"
  "EGYPTIAN HIEROGLYPH-141C0\0"
  "EGYPTIAN HIEROGLYPH-141C1\0"
  "EGYPTIAN HIEROGLYPH-141C2\0"
  "EGYPTIAN HIEROGLYPH-141C3\0"
  "EGYPTIAN HIEROGLYPH-141C4\0"
  "EGYPTIAN HIEROGLYPH-141C5\0"
  "EGYPTIAN HIEROGLYPH-141C6\0"
  "EGYPTIAN HIEROGLYPH-141C7\0"
  "EGYPTIAN HIEROGLYPH-141C8\0"
  "EGYPTIAN HIEROGLYPH-141C9\0"
  "EGYPTIAN HIEROGLYPH-141CA\0"
  "EGYPTIAN HIEROGLYPH-141CB\0"
  "EGYPTIAN HIEROGLYPH-141CC\0"
  "EGYPTIAN HIEROGLYPH-141CD\0"
  "EGYPTIAN HIEROGLYPH-141CE\0"
  "EGYPTIAN HIEROGLYPH-141CF\0"
  "EGYPTIAN HIEROGLYPH-141D0\0"
  "EGYPTIAN HIEROGLYPH-141D1\0"
  "EGYPTIAN HIEROGLYPH-141D2\0"
  "EGYPTIAN HIEROGLYPH-141D3\0"
  "EGYPTIAN HIEROGLYPH-141D4\0"
  "EGYPTIAN HIEROGLYPH-141D5\0"
  "EGYPTIAN HIEROGLYPH-141D6\0"
  "EGYPTIAN HIEROGLYPH-141D7\0"
  "EGYPTIAN HIEROGLYPH-141D8\0"
  "EGYPTIAN HIEROGLYPH-141D9\0"
  "EGYPTIAN HIEROGLYPH-141DA\0"
  "EGYPTIAN HIEROGLYPH-141DB\0"
  "EGYPTIAN HIEROGLYPH-141DC\0"
  "EGYPTIAN HIEROGLYPH-141DD\0"
  "EGYPTIAN HIEROGLYPH-141DE\0"
  "EGYPTIAN HIEROGLYPH-141DF\0"
  "EGYPTIAN HIEROGLYPH-141E0\0"
  "EGYPTIAN HIEROGLYPH-141E1\0"
  "EGYPTIAN HIEROGLYPH-141E2\0"
  "EGYPTIAN HIEROGLYPH-141E3\0"
  "EGYPTIAN HIEROGLYPH-141E4\0"
  "EGYPTIAN HIEROGLYPH-141E5\0"
  "EGYPTIAN HIEROGLYPH-141E6\0"
  "EGYPTIAN HIEROGLYPH-141E7\0"
  "EGYPTIAN HIEROGLYPH-141E8\0"
  "EGYPTIAN HIEROGLYPH-141E9\0"
  "EGYPTIAN HIEROGLYPH-141EA\0"
  "EGYPTIAN HIEROGLYPH-141EB\0"
  "EGYPTIAN HIEROGLYPH-141EC\0"
  "EGYPTIAN HIEROGLYPH-141ED\0"
  "EGYPTIAN HIEROGLYPH-141EE\0"
  "EGYPTIAN HIEROGLYPH-141EF\0"
  "EGYPTIAN HIEROGLYPH-141F0\0"
  "EGYPTIAN HIEROGLYPH-141F1\0"
  "EGYPTIAN HIEROGLYPH-141F2\0"
  "EGYPTIAN HIEROGLYPH-141F3\0"
  "EGYPTIAN HIEROGLYPH-141F4\0"
  "EGYPTIAN HIEROGLYPH-141F5\0"
  "EGYPTIAN HIEROGLYPH-141F6\0"
  "EGYPTIAN HIEROGLYPH-141F7\0"
  "EGYPTIAN HIEROGLYPH-141F8\0"
  "EGYPTIAN HIEROGLYPH-141F9\0"
  "EGYPTIAN HIEROGLYPH-141FA\0"
  "EGYPTIAN HIEROGLYPH-141FB\0"
  "EGYPTIAN HIEROGLYPH-141FC\0"
  "EGYPTIAN HIEROGLYPH-141FD\0"
  "EGYPTIAN HIEROGLYPH-141FE\0"
  "EGYPTIAN HIEROGLYPH-141FF\0"
  "EGYPTIAN HIEROGLYPH-14200\0"
  "EGYPTIAN HIEROGLYPH-14201\0"
  "EGYPTIAN HIEROGLYPH-14202\0"
  "EGYPTIAN HIEROGLYPH-14203\0"
  "EGYPTIAN HIEROGLYPH-14204\0"
  "EGYPTIAN HIEROGLYPH-14205\0"
  "EGYPTIAN HIEROGLYPH-14206\0"
  "EGYPTIAN HIEROGLYPH-14207\0"
  "EGYPTIAN HIEROGLYPH-14208\0"
  "EGYPTIAN HIEROGLYPH-14209\0"
  "EGYPTIAN HIEROGLYPH-1420A\0"
  "EGYPTIAN HIEROGLYPH-1420B\0"
  "EGYPTIAN HIEROGLYPH-1420C\0"
  "EGYPTIAN HIEROGLYPH-1420D\0"
  "EGYPTIAN HIEROGLYPH-1420E\0"
  "EGYPTIAN HIEROGLYPH-1420F\0"
  "EGYPTIAN HIEROGLYPH-14210\0"
  "EGYPTIAN HIEROGLYPH-14211\0"
  "EGYPTIAN HIEROGLYPH-14212\0"
  "EGYPTIAN HIEROGLYPH-14213\0"
  "EGYPTIAN HIEROGLYPH-14214\0"
  "EGYPTIAN HIEROGLYPH-14215\0"
  "EGYPTIAN HIEROGLYPH-14216\0"
  "EGYPTIAN HIEROGLYPH-14217\0"
  "EGYPTIAN HIEROGLYPH-14218\0"
  "EGYPTIAN HIEROGLYPH-14219\0"
  "EGYPTIAN HIEROGLYPH-1421A\0"
  "EGYPTIAN HIEROGLYPH-1421B\0"
  "EGYPTIAN HIEROGLYPH-1421C\0"
  "EGYPTIAN HIEROGLYPH-1421D\0"
  "EGYPTIAN HIEROGLYPH-1421E\0"
  "EGYPTIAN HIEROGLYPH-1421F\0"
  "EGYPTIAN HIEROGLYPH-14220\0"
  "EGYPTIAN HIEROGLYPH-14221\0"
  "EGYPTIAN HIEROGLYPH-14222\0"
  "EGYPTIAN HIEROGLYPH-14223\0"
  "EGYPTIAN HIEROGLYPH-14224\0"
  "EGYPTIAN HIEROGLYPH-14225\0"
  "EGYPTIAN HIEROGLYPH-14226\0"
  "EGYPTIAN HIEROGLYPH-14227\0"
  "EGYPTIAN HIEROGLYPH-14228\0"
  "EGYPTIAN HIEROGLYPH-14229\0"
  "EGYPTIAN HIEROGLYPH-1422A\0"
  "EGYPTIAN HIEROGLYPH-1422B\0"
  "EGYPTIAN HIEROGLYPH-1422C\0"
  "EGYPTIAN HIEROGLYPH-1422D\0"
  "EGYPTIAN HIEROGLYPH-1422E\0"
  "EGYPTIAN HIEROGLYPH-1422F\0"
  "EGYPTIAN HIEROGLYPH-14230\0"
  "EGYPTIAN HIEROGLYPH-14231\0"
  "EGYPTIAN HIEROGLYPH-14232\0"
  "EGYPTIAN HIEROGLYPH-14233\0"
  "EGYPTIAN HIEROGLYPH-14234\0"
  "EGYPTIAN HIEROGLYPH-14235\0"
  "EGYPTIAN HIEROGLYPH-14236\0"
  "EGYPTIAN HIEROGLYPH-14237\0"
  "EGYPTIAN HIEROGLYPH-14238\0"
  "EGYPTIAN HIEROGLYPH-14239\0"
  "EGYPTIAN HIEROGLYPH-1423A\0"
  "EGYPTIAN HIEROGLYPH-1423B\0"
  "EGYPTIAN HIEROGLYPH-1423C\0"
  "EGYPTIAN HIEROGLYPH-1423D\0"
  "EGYPTIAN HIEROGLYPH-1423E\0"
  "EGYPTIAN HIEROGLYPH-1423F\0"
  "EGYPTIAN HIEROGLYPH-14240\0"
  "EGYPTIAN HIEROGLYPH-14241\0"
  "EGYPTIAN HIEROGLYPH-14242\0"
  "EGYPTIAN HIEROGLYPH-14243\0"
  "EGYPTIAN HIEROGLYPH-14244\0"
  "EGYPTIAN HIEROGLYPH-14245\0"
  "EGYPTIAN HIEROGLYPH-14246\0"
  "EGYPTIAN HIEROGLYPH-14247\0"
  "EGYPTIAN HIEROGLYPH-14248\0"
  "EGYPTIAN HIEROGLYPH-14249\0"
  "EGYPTIAN HIEROGLYPH-1424A\0"
  "EGYPTIAN HIEROGLYPH-1424B\0"
  "EGYPTIAN HIEROGLYPH-1424C\0"
  "EGYPTIAN HIEROGLYPH-1424D\0"
  "EGYPTIAN HIEROGLYPH-1424E\0"
  "EGYPTIAN HIEROGLYPH-1424F\0"
  "EGYPTIAN HIEROGLYPH-14250\0"
  "EGYPTIAN HIEROGLYPH-14251\0"
  "EGYPTIAN HIEROGLYPH-14252\0"
  "EGYPTIAN HIEROGLYPH-14253\0"
  "EGYPTIAN HIEROGLYPH-14254\0"
  "EGYPTIAN HIEROGLYPH-14255\0"
  "EGYPTIAN HIEROGLYPH-14256\0"
  "EGYPTIAN HIEROGLYPH-14257\0"
  "EGYPTIAN HIEROGLYPH-14258\0"
  "EGYPTIAN HIEROGLYPH-14259\0"
  "EGYPTIAN HIEROGLYPH-1425A\0"
  "EGYPTIAN HIEROGLYPH-1425B\0"
  "EGYPTIAN HIEROGLYPH-1425C\0"
  "EGYPTIAN HIEROGLYPH-1425D\0"
  "EGYPTIAN HIEROGLYPH-1425E\0"
  "EGYPTIAN HIEROGLYPH-1425F\0"
  "EGYPTIAN HIEROGLYPH-14260\0"
  "EGYPTIAN HIEROGLYPH-14261\0"
  "EGYPTIAN HIEROGLYPH-14262\0"
  "EGYPTIAN HIEROGLYPH-14263\0"
  "EGYPTIAN HIEROGLYPH-14264\0"
  "EGYPTIAN HIEROGLYPH-14265\0"
  "EGYPTIAN HIEROGLYPH-14266\0"
  "EGYPTIAN HIEROGLYPH-14267\0"
  "EGYPTIAN HIEROGLYPH-14268\0"
  "EGYPTIAN HIEROGLYPH-14269\0"
  "EGYPTIAN HIEROGLYPH-1426A\0"
  "EGYPTIAN HIEROGLYPH-1426B\0"
  "EGYPTIAN HIEROGLYPH-1426C\0"
  "EGYPTIAN HIEROGLYPH-1426D\0"
  "EGYPTIAN HIEROGLYPH-1426E\0"
  "EGYPTIAN HIEROGLYPH-1426F\0"
  "EGYPTIAN HIEROGLYPH-14270\0"
  "EGYPTIAN HIEROGLYPH-14271\0"
  "EGYPTIAN HIEROGLYPH-14272\0"
  "EGYPTIAN HIEROGLYPH-14273\0"
  "EGYPTIAN HIEROGLYPH-14274\0"
  "EGYPTIAN HIEROGLYPH-14275\0"
  "EGYPTIAN HIEROGLYPH-14276\0"
  "EGYPTIAN HIEROGLYPH-14277\0"
  "EGYPTIAN HIEROGLYPH-14278\0"
  "EGYPTIAN HIEROGLYPH-14279\0"
  "EGYPTIAN HIEROGLYPH-1427A\0"
  "EGYPTIAN HIEROGLYPH-1427B\0"
  "EGYPTIAN HIEROGLYPH-1427C\0"
  "EGYPTIAN HIEROGLYPH-1427D\0"
  "EGYPTIAN HIEROGLYPH-1427E\0"
  "EGYPTIAN HIEROGLYPH-1427F\0"
  "EGYPTIAN HIEROGLYPH-14280\0"
  "EGYPTIAN HIEROGLYPH-14281\0"
  "EGYPTIAN HIEROGLYPH-14282\0"
  "EGYPTIAN HIEROGLYPH-14283\0"
  "EGYPTIAN HIEROGLYPH-14284\0"
  "EGYPTIAN HIEROGLYPH-14285\0"
  "EGYPTIAN HIEROGLYPH-14286\0"
  "EGYPTIAN HIEROGLYPH-14287\0"
  "EGYPTIAN HIEROGLYPH-14288\0"
  "EGYPTIAN HIEROGLYPH-14289\0"
  "EGYPTIAN HIEROGLYPH-1428A\0"
  "EGYPTIAN HIEROGLYPH-1428B\0"
  "EGYPTIAN HIEROGLYPH-1428C\0"
  "EGYPTIAN HIEROGLYPH-1428D\0"
  "EGYPTIAN HIEROGLYPH-1428E\0"
  "EGYPTIAN HIEROGLYPH-1428F\0"
  "EGYPTIAN HIEROGLYPH-14290\0"
  "EGYPTIAN HIEROGLYPH-14291\0"
  "EGYPTIAN HIEROGLYPH-14292\0"
  "EGYPTIAN HIEROGLYPH-14293\0"
  "EGYPTIAN HIEROGLYPH-14294\0"
  "EGYPTIAN HIEROGLYPH-14295\0"
  "EGYPTIAN HIEROGLYPH-14296\0"
  "EGYPTIAN HIEROGLYPH-14297\0"
  "EGYPTIAN HIEROGLYPH-14298\0"
  "EGYPTIAN HIEROGLYPH-14299\0"
  "EGYPTIAN HIEROGLYPH-1429A\0"
  "EGYPTIAN HIEROGLYPH-1429B\0"
  "EGYPTIAN HIEROGLYPH-1429C\0"
  "EGYPTIAN HIEROGLYPH-1429D\0"
  "EGYPTIAN HIEROGLYPH-1429E\0"
  "EGYPTIAN HIEROGLYPH-1429F\0"
  "EGYPTIAN HIEROGLYPH-142A0\0"
  "EGYPTIAN HIEROGLYPH-142A1\0"
  "EGYPTIAN HIEROGLYPH-142A2\0"
  "EGYPTIAN HIEROGLYPH-142A3\0"
  "EGYPTIAN HIEROGLYPH-142A4\0"
  "EGYPTIAN HIEROGLYPH-142A5\0"
  "EGYPTIAN HIEROGLYPH-142A6\0"
  "EGYPTIAN HIEROGLYPH-142A7\0"
  "EGYPTIAN HIEROGLYPH-142A8\0"
  "EGYPTIAN HIEROGLYPH-142A9\0"
  "EGYPTIAN HIEROGLYPH-142AA\0"
  "EGYPTIAN HIEROGLYPH-142AB\0"
  "EGYPTIAN HIEROGLYPH-142AC\0"
  "EGYPTIAN HIEROGLYPH-142AD\0"
  "EGYPTIAN HIEROGLYPH-142AE\0"
  "EGYPTIAN HIEROGLYPH-142AF\0"
  "EGYPTIAN HIEROGLYPH-142B0\0"
  "EGYPTIAN HIEROGLYPH-142B1\0"
  "EGYPTIAN HIEROGLYPH-142B2\0"
  "EGYPTIAN HIEROGLYPH-142B3\0"
  "EGYPTIAN HIEROGLYPH-142B4\0"
  "EGYPTIAN HIEROGLYPH-142B5\0"
  "EGYPTIAN HIEROGLYPH-142B6\0"
  "EGYPTIAN HIEROGLYPH-142B7\0"
  "EGYPTIAN HIEROGLYPH-142B8\0"
  "EGYPTIAN HIEROGLYPH-142B9\0"
  "EGYPTIAN HIEROGLYPH-142BA\0"
  "EGYPTIAN HIEROGLYPH-142BB\0"
  "EGYPTIAN HIEROGLYPH-142BC\0"
  "EGYPTIAN HIEROGLYPH-142BD\0"
  "EGYPTIAN HIEROGLYPH-142BE\0"
  "EGYPTIAN HIEROGLYPH-142BF\0"
  "EGYPTIAN HIEROGLYPH-142C0\0"
  "EGYPTIAN HIEROGLYPH-142C1\0"
  "EGYPTIAN HIEROGLYPH-142C2\0"
  "EGYPTIAN HIEROGLYPH-142C3\0"
  "EGYPTIAN HIEROGLYPH-142C4\0"
  "EGYPTIAN HIEROGLYPH-142C5\0"
  "EGYPTIAN HIEROGLYPH-142C6\0"
  "EGYPTIAN HIEROGLYPH-142C7\0"
  "EGYPTIAN HIEROGLYPH-142C8\0"
  "EGYPTIAN HIEROGLYPH-142C9\0"
  "EGYPTIAN HIEROGLYPH-142CA\0"
  "EGYPTIAN HIEROGLYPH-142CB\0"
  "EGYPTIAN HIEROGLYPH-142CC\0"
  "EGYPTIAN HIEROGLYPH-142CD\0"
  "EGYPTIAN HIEROGLYPH-142CE\0"
  "EGYPTIAN HIEROGLYPH-142CF\0"
  "EGYPTIAN HIEROGLYPH-142D0\0"
  "EGYPTIAN HIEROGLYPH-142D1\0"
  "EGYPTIAN HIEROGLYPH-142D2\0"
  "EGYPTIAN HIEROGLYPH-142D3\0"
  "EGYPTIAN HIEROGLYPH-142D4\0"
  "EGYPTIAN HIEROGLYPH-142D5\0"
  "EGYPTIAN HIEROGLYPH-142D6\0"
  "EGYPTIAN HIEROGLYPH-142D7\0"
  "EGYPTIAN HIEROGLYPH-142D8\0"
  "EGYPTIAN HIEROGLYPH-142D9\0"
  "EGYPTIAN HIEROGLYPH-142DA\0"
  "EGYPTIAN HIEROGLYPH-142DB\0"
  "EGYPTIAN HIEROGLYPH-142DC\0"
  "EGYPTIAN HIEROGLYPH-142DD\0"
  "EGYPTIAN HIEROGLYPH-142DE\0"
  "EGYPTIAN HIEROGLYPH-142DF\0"
  "EGYPTIAN HIEROGLYPH-142E0\0"
  "EGYPTIAN HIEROGLYPH-142E1\0"
  "EGYPTIAN HIEROGLYPH-142E2\0"
  "EGYPTIAN HIEROGLYPH-142E3\0"
  "EGYPTIAN HIEROGLYPH-142E4\0"
  "EGYPTIAN HIEROGLYPH-142E5\0"
  "EGYPTIAN HIEROGLYPH-142E6\0"
  "EGYPTIAN HIEROGLYPH-142E7\0"
  "EGYPTIAN HIEROGLYPH-142E8\0"
  "EGYPTIAN HIEROGLYPH-142E9\0"
  "EGYPTIAN HIEROGLYPH-142EA\0"
  "EGYPTIAN HIEROGLYPH-142EB\0"
  "EGYPTIAN HIEROGLYPH-142EC\0"
  "EGYPTIAN HIEROGLYPH-142ED\0"
  "EGYPTIAN HIEROGLYPH-142EE\0"
  "EGYPTIAN HIEROGLYPH-142EF\0"
  "EGYPTIAN HIEROGLYPH-142F0\0"
  "EGYPTIAN HIEROGLYPH-142F1\0"
  "EGYPTIAN HIEROGLYPH-142F2\0"
  "EGYPTIAN HIEROGLYPH-142F3\0"
  "EGYPTIAN HIEROGLYPH-142F4\0"
  "EGYPTIAN HIEROGLYPH-142F5\0"
  "EGYPTIAN HIEROGLYPH-142F6\0"
  "EGYPTIAN HIEROGLYPH-142F7\0"
  "EGYPTIAN HIEROGLYPH-142F8\0"
  "EGYPTIAN HIEROGLYPH-142F9\0"
  "EGYPTIAN HIEROGLYPH-142FA\0"
  "EGYPTIAN HIEROGLYPH-142FB\0"
  "EGYPTIAN HIEROGLYPH-142FC\0"
  "EGYPTIAN HIEROGLYPH-142FD\0"
  "EGYPTIAN HIEROGLYPH-142FE\0"
  "EGYPTIAN HIEROGLYPH-142FF\0"
  "EGYPTIAN HIEROGLYPH-14300\0"
  "EGYPTIAN HIEROGLYPH-14301\0"
  "EGYPTIAN HIEROGLYPH-14302\0"
  "EGYPTIAN HIEROGLYPH-14303\0"
  "EGYPTIAN HIEROGLYPH-14304\0"
  "EGYPTIAN HIEROGLYPH-14305\0"
  "EGYPTIAN HIEROGLYPH-14306\0"
  "EGYPTIAN HIEROGLYPH-14307\0"
  "EGYPTIAN HIEROGLYPH-14308\0"
  "EGYPTIAN HIEROGLYPH-14309\0"
  "EGYPTIAN HIEROGLYPH-1430A\0"
  "EGYPTIAN HIEROGLYPH-1430B\0"
  "EGYPTIAN HIEROGLYPH-1430C\0"
  "EGYPTIAN HIEROGLYPH-1430D\0"
  "EGYPTIAN HIEROGLYPH-1430E\0"
  "EGYPTIAN HIEROGLYPH-1430F\0"
  "EGYPTIAN HIEROGLYPH-14310\0"
  "EGYPTIAN HIEROGLYPH-14311\0"
  "EGYPTIAN HIEROGLYPH-14312\0"
  "EGYPTIAN HIEROGLYPH-14313\0"
  "EGYPTIAN HIEROGLYPH-14314\0"
  "EGYPTIAN HIEROGLYPH-14315\0"
  "EGYPTIAN HIEROGLYPH-14316\0"
  "EGYPTIAN HIEROGLYPH-14317\0"
  "EGYPTIAN HIEROGLYPH-14318\0"
  "EGYPTIAN HIEROGLYPH-14319\0"
  "EGYPTIAN HIEROGLYPH-1431A\0"
  "EGYPTIAN HIEROGLYPH-1431B\0"
  "EGYPTIAN HIEROGLYPH-1431C\0"
  "EGYPTIAN HIEROGLYPH-1431D\0"
  "EGYPTIAN HIEROGLYPH-1431E\0"
  "EGYPTIAN HIEROGLYPH-1431F\0"
  "EGYPTIAN HIEROGLYPH-14320\0"
  "EGYPTIAN HIEROGLYPH-14321\0"
  "EGYPTIAN HIEROGLYPH-14322\0"
  "EGYPTIAN HIEROGLYPH-14323\0"
  "EGYPTIAN HIEROGLYPH-14324\0"
  "EGYPTIAN HIEROGLYPH-14325\0"
  "EGYPTIAN HIEROGLYPH-14326\0"
  "EGYPTIAN HIEROGLYPH-14327\0"
  "EGYPTIAN HIEROGLYPH-14328\0"
  "EGYPTIAN HIEROGLYPH-14329\0"
  "EGYPTIAN HIEROGLYPH-1432A\0"
  "EGYPTIAN HIEROGLYPH-1432B\0"
  "EGYPTIAN HIEROGLYPH-1432C\0"
  "EGYPTIAN HIEROGLYPH-1432D\0"
  "EGYPTIAN HIEROGLYPH-1432E\0"
  "EGYPTIAN HIEROGLYPH-1432F\0"
  "EGYPTIAN HIEROGLYPH-14330\0"
  "EGYPTIAN HIEROGLYPH-14331\0"
  "EGYPTIAN HIEROGLYPH-14332\0"
  "EGYPTIAN HIEROGLYPH-14333\0"
  "EGYPTIAN HIEROGLYPH-14334\0"
  "EGYPTIAN HIEROGLYPH-14335\0"
  "EGYPTIAN HIEROGLYPH-14336\0"
  "EGYPTIAN HIEROGLYPH-14337\0"
  "EGYPTIAN HIEROGLYPH-14338\0"
  "EGYPTIAN HIEROGLYPH-14339\0"
  "EGYPTIAN HIEROGLYPH-1433A\0"
  "EGYPTIAN HIEROGLYPH-1433B\0"
  "EGYPTIAN HIEROGLYPH-1433C\0"
  "EGYPTIAN HIEROGLYPH-1433D\0"
  "EGYPTIAN HIEROGLYPH-1433E\0"
  "EGYPTIAN HIEROGLYPH-1433F\0"
  "EGYPTIAN HIEROGLYPH-14340\0"
  "EGYPTIAN HIEROGLYPH-14341\0"
  "EGYPTIAN HIEROGLYPH-14342\0"
  "EGYPTIAN HIEROGLYPH-14343\0"
  "EGYPTIAN HIEROGLYPH-14344\0"
  "EGYPTIAN HIEROGLYPH-14345\0"
  "EGYPTIAN HIEROGLYPH-14346\0"
  "EGYPTIAN HIEROGLYPH-14347\0"
  "EGYPTIAN HIEROGLYPH-14348\0"
  "EGYPTIAN HIEROGLYPH-14349\0"
  "EGYPTIAN HIEROGLYPH-1434A\0"
  "EGYPTIAN HIEROGLYPH-1434B\0"
  "EGYPTIAN HIEROGLYPH-1434C\0"
  "EGYPTIAN HIEROGLYPH-1434D\0"
  "EGYPTIAN HIEROGLYPH-1434E\0"
  "EGYPTIAN HIEROGLYPH-1434F\0"
  "EGYPTIAN HIEROGLYPH-14350\0"
  "EGYPTIAN HIEROGLYPH-14351\0"
  "EGYPTIAN HIEROGLYPH-14352\0"
  "EGYPTIAN HIEROGLYPH-14353\0"
  "EGYPTIAN HIEROGLYPH-14354\0"
  "EGYPTIAN HIEROGLYPH-14355\0"
  "EGYPTIAN HIEROGLYPH-14356\0"
  "EGYPTIAN HIEROGLYPH-14357\0"
  "EGYPTIAN HIEROGLYPH-14358\0"
  "EGYPTIAN HIEROGLYPH-14359\0"
  "EGYPTIAN HIEROGLYPH-1435A\0"
  "EGYPTIAN HIEROGLYPH-1435B\0"
  "EGYPTIAN HIEROGLYPH-1435C\0"
  "EGYPTIAN HIEROGLYPH-1435D\0"
  "EGYPTIAN HIEROGLYPH-1435E\0"
  "EGYPTIAN HIEROGLYPH-1435F\0"
  "EGYPTIAN HIEROGLYPH-14360\0"
  "EGYPTIAN HIEROGLYPH-14361\0"
  "EGYPTIAN HIEROGLYPH-14362\0"
  "EGYPTIAN HIEROGLYPH-14363\0"
  "EGYPTIAN HIEROGLYPH-14364\0"
  "EGYPTIAN HIEROGLYPH-14365\0"
  "EGYPTIAN HIEROGLYPH-14366\0"
  "EGYPTIAN HIEROGLYPH-14367\0"
  "EGYPTIAN HIEROGLYPH-14368\0"
  "EGYPTIAN HIEROGLYPH-14369\0"
  "EGYPTIAN HIEROGLYPH-1436A\0"
  "EGYPTIAN HIEROGLYPH-1436B\0"
  "EGYPTIAN HIEROGLYPH-1436C\0"
  "EGYPTIAN HIEROGLYPH-1436D\0"
  "EGYPTIAN HIEROGLYPH-1436E\0"
  "EGYPTIAN HIEROGLYPH-1436F\0"
  "EGYPTIAN HIEROGLYPH-14370\0"
  "EGYPTIAN HIEROGLYPH-14371\0"
  "EGYPTIAN HIEROGLYPH-14372\0"
  "EGYPTIAN HIEROGLYPH-14373\0"
  "EGYPTIAN HIEROGLYPH-14374\0"
  "EGYPTIAN HIEROGLYPH-14375\0"
  "EGYPTIAN HIEROGLYPH-14376\0"
  "EGYPTIAN HIEROGLYPH-14377\0"
  "EGYPTIAN HIEROGLYPH-14378\0"
  "EGYPTIAN HIEROGLYPH-14379\0"
  "EGYPTIAN HIEROGLYPH-1437A\0"
  "EGYPTIAN HIEROGLYPH-1437B\0"
  "EGYPTIAN HIEROGLYPH-1437C\0"
  "EGYPTIAN HIEROGLYPH-1437D\0"
  "EGYPTIAN HIEROGLYPH-1437E\0"
  "EGYPTIAN HIEROGLYPH-1437F\0"
  "EGYPTIAN HIEROGLYPH-14380\0"
  "EGYPTIAN HIEROGLYPH-14381\0"
  "EGYPTIAN HIEROGLYPH-14382\0"
  "EGYPTIAN HIEROGLYPH-14383\0"
  "EGYPTIAN HIEROGLYPH-14384\0"
  "EGYPTIAN HIEROGLYPH-14385\0"
  "EGYPTIAN HIEROGLYPH-14386\0"
  "EGYPTIAN HIEROGLYPH-14387\0"
  "EGYPTIAN HIEROGLYPH-14388\0"
  "EGYPTIAN HIEROGLYPH-14389\0"
  "EGYPTIAN HIEROGLYPH-1438A\0"
  "EGYPTIAN HIEROGLYPH-1438B\0"
  "EGYPTIAN HIEROGLYPH-1438C\0"
  "EGYPTIAN HIEROGLYPH-1438D\0"
  "EGYPTIAN HIEROGLYPH-1438E\0"
  "EGYPTIAN HIEROGLYPH-1438F\0"
  "EGYPTIAN HIEROGLYPH-14390\0"
  "EGYPTIAN HIEROGLYPH-14391\0"
  "EGYPTIAN HIEROGLYPH-14392\0"
  "EGYPTIAN HIEROGLYPH-14393\0"
  "EGYPTIAN HIEROGLYPH-14394\0"
  "EGYPTIAN HIEROGLYPH-14395\0"
  "EGYPTIAN HIEROGLYPH-14396\0"
  "EGYPTIAN HIEROGLYPH-14397\0"
  "EGYPTIAN HIEROGLYPH-14398\0"
  "EGYPTIAN HIEROGLYPH-14399\0"
  "EGYPTIAN HIEROGLYPH-1439A\0"
  "EGYPTIAN HIEROGLYPH-1439B\0"
  "EGYPTIAN HIEROGLYPH-1439C\0"
  "EGYPTIAN HIEROGLYPH-1439D\0"
  "EGYPTIAN HIEROGLYPH-1439E\0"
  "EGYPTIAN HIEROGLYPH-1439F\0"
  "EGYPTIAN HIEROGLYPH-143A0\0"
  "EGYPTIAN HIEROGLYPH-143A1\0"
  "EGYPTIAN HIEROGLYPH-143A2\0"
  "EGYPTIAN HIEROGLYPH-143A3\0"
  "EGYPTIAN HIEROGLYPH-143A4\0"
  "EGYPTIAN HIEROGLYPH-143A5\0"
  "EGYPTIAN HIEROGLYPH-143A6\0"
  "EGYPTIAN HIEROGLYPH-143A7\0"
  "EGYPTIAN HIEROGLYPH-143A8\0"
  "EGYPTIAN HIEROGLYPH-143A9\0"
  "EGYPTIAN HIEROGLYPH-143AA\0"
  "EGYPTIAN HIEROGLYPH-143AB\0"
  "EGYPTIAN HIEROGLYPH-143AC\0"
  "EGYPTIAN HIEROGLYPH-143AD\0"
  "EGYPTIAN HIEROGLYPH-143AE\0"
  "EGYPTIAN HIEROGLYPH-143AF\0"
  "EGYPTIAN HIEROGLYPH-143B0\0"
  "EGYPTIAN HIEROGLYPH-143B1\0"
  "EGYPTIAN HIEROGLYPH-143B2\0"
  "EGYPTIAN HIEROGLYPH-143B3\0"
  "EGYPTIAN HIEROGLYPH-143B4\0"
  "EGYPTIAN HIEROGLYPH-143B5\0"
  "EGYPTIAN HIEROGLYPH-143B6\0"
  "EGYPTIAN HIEROGLYPH-143B7\0"
  "EGYPTIAN HIEROGLYPH-143B8\0"
  "EGYPTIAN HIEROGLYPH-143B9\0"
  "EGYPTIAN HIEROGLYPH-143BA\0"
  "EGYPTIAN HIEROGLYPH-143BB\0"
  "EGYPTIAN HIEROGLYPH-143BC\0"
  "EGYPTIAN HIEROGLYPH-143BD\0"
  "EGYPTIAN HIEROGLYPH-143BE\0"
  "EGYPTIAN HIEROGLYPH-143BF\0"
  "EGYPTIAN HIEROGLYPH-143C0\0"
  "EGYPTIAN HIEROGLYPH-143C1\0"
  "EGYPTIAN HIEROGLYPH-143C2\0"
  "EGYPTIAN HIEROGLYPH-143C3\0"
  "EGYPTIAN HIEROGLYPH-143C4\0"
  "EGYPTIAN HIEROGLYPH-143C5\0"
  "EGYPTIAN HIEROGLYPH-143C6\0"
  "EGYPTIAN HIEROGLYPH-143C7\0"
  "EGYPTIAN HIEROGLYPH-143C8\0"
  "EGYPTIAN HIEROGLYPH-143C9\0"
  "EGYPTIAN HIEROGLYPH-143CA\0"
  "EGYPTIAN HIEROGLYPH-143CB\0"
  "EGYPTIAN HIEROGLYPH-143CC\0"
  "EGYPTIAN HIEROGLYPH-143CD\0"
  "EGYPTIAN HIEROGLYPH-143CE\0"
  "EGYPTIAN HIEROGLYPH-143CF\0"
  "EGYPTIAN HIEROGLYPH-143D0\0"
  "EGYPTIAN HIEROGLYPH-143D1\0"
  "EGYPTIAN HIEROGLYPH-143D2\0"
  "EGYPTIAN HIEROGLYPH-143D3\0"
  "EGYPTIAN HIEROGLYPH-143D4\0"
  "EGYPTIAN HIEROGLYPH-143D5\0"
  "EGYPTIAN HIEROGLYPH-143D6\0"
  "EGYPTIAN HIEROGLYPH-143D7\0"
  "EGYPTIAN HIEROGLYPH-143D8\0"
  "EGYPTIAN HIEROGLYPH-143D9\0"
  "EGYPTIAN HIEROGLYPH-143DA\0"
  "EGYPTIAN HIEROGLYPH-143DB\0"
  "EGYPTIAN HIEROGLYPH-143DC\0"
  "EGYPTIAN HIEROGLYPH-143DD\0"
  "EGYPTIAN HIEROGLYPH-143DE\0"
  "EGYPTIAN HIEROGLYPH-143DF\0"
  "EGYPTIAN HIEROGLYPH-143E0\0"
  "EGYPTIAN HIEROGLYPH-143E1\0"
  "EGYPTIAN HIEROGLYPH-143E2\0"
  "EGYPTIAN HIEROGLYPH-143E3\0"
  "EGYPTIAN HIEROGLYPH-143E4\0"
  "EGYPTIAN HIEROGLYPH-143E5\0"
  "EGYPTIAN HIEROGLYPH-143E6\0"
  "EGYPTIAN HIEROGLYPH-143E7\0"
  "EGYPTIAN HIEROGLYPH-143E8\0"
  "EGYPTIAN HIEROGLYPH-143E9\0"
  "EGYPTIAN HIEROGLYPH-143EA\0"
  "EGYPTIAN HIEROGLYPH-143EB\0"
  "EGYPTIAN HIEROGLYPH-143EC\0"
  "EGYPTIAN HIEROGLYPH-143ED\0"
  "EGYPTIAN HIEROGLYPH-143EE\0"
  "EGYPTIAN HIEROGLYPH-143EF\0"
  "EGYPTIAN HIEROGLYPH-143F0\0"
  "EGYPTIAN HIEROGLYPH-143F1\0"
  "EGYPTIAN HIEROGLYPH-143F2\0"
  "EGYPTIAN HIEROGLYPH-143F3\0"
  "EGYPTIAN HIEROGLYPH-143F4\0"
  "EGYPTIAN HIEROGLYPH-143F5\0"
  "EGYPTIAN HIEROGLYPH-143F6\0"
  "EGYPTIAN HIEROGLYPH-143F7\0"
  "EGYPTIAN HIEROGLYPH-143F8\0"
  "EGYPTIAN HIEROGLYPH-143F9\0"
  "EGYPTIAN HIEROGLYPH-143FA\0"
  "EIGHT PETALLED OUTLINED BLACK FLORETTE\0"
  "EIGHT POINTED BLACK STAR\0"
  "EIGHT POINTED PINWHEEL STAR\0"
  "EIGHT POINTED RECTILINEAR BLACK STAR\0"
  "EIGHT RAYS INWARD\0"
  "EIGHT RAYS OUTWARD\0"
  "EIGHT SPOKED ASTERISK\0"
  "EIGHT TEARDROP-SPOKED PROPELLER ASTERISK\0"
  "EIGHTH NOTE\0"
  "EJECT SYMBOL\0"
  "ELBASAN LETTER A\0"
  "ELBASAN LETTER BE\0"
  "ELBASAN LETTER CE\0"
  "ELBASAN LETTER CHE\0"
  "ELBASAN LETTER DE\0"
  "ELBASAN LETTER DHE\0"
  "ELBASAN LETTER E\0"
  "ELBASAN LETTER EI\0"
  "ELBASAN LETTER FE\0"
  "ELBASAN LETTER GE\0"
  "ELBASAN LETTER GHAMMA\0"
  "ELBASAN LETTER GHE\0"
  "ELBASAN LETTER GJE\0"
  "ELBASAN LETTER HE\0"
  "ELBASAN LETTER I\0"
  "ELBASAN LETTER JE\0"
  "ELBASAN LETTER KE\0"
  "ELBASAN LETTER KHE\0"
  "ELBASAN LETTER LE\0"
  "ELBASAN LETTER LLE\0"
  "ELBASAN LETTER ME\0"
  "ELBASAN LETTER NA\0"
  "ELBASAN LETTER NDE\0"
  "ELBASAN LETTER NE\0"
  "ELBASAN LETTER NJE\0"
  "ELBASAN LETTER O\0"
  "ELBASAN LETTER PE\0"
  "ELBASAN LETTER QE\0"
  "ELBASAN LETTER RE\0"
  "ELBASAN LETTER RRE\0"
  "ELBASAN LETTER SE\0"
  "ELBASAN LETTER SHE\0"
  "ELBASAN LETTER TE\0"
  "ELBASAN LETTER THE\0"
  "ELBASAN LETTER U\0"
  "ELBASAN LETTER VE\0"
  "ELBASAN LETTER XE\0"
  "ELBASAN LETTER Y\0"
  "ELBASAN LETTER ZE\0"
  "ELBASAN LETTER ZHE\0"
  "ELECTRIC ARROW\0"
  "ELECTRIC LIGHT BULB\0"
  "ELECTRIC PLUG\0"
  "ELECTRIC TORCH\0"
  "ELECTRICAL INTERSECTION\0"
  "ELEMENT OF\0"
  "ELEMENT OF OPENING DOWNWARDS\0"
  "ELEMENT OF OPENING UPWARDS\0"
  "ELEMENT OF WITH DOT ABOVE\0"
  "ELEMENT OF WITH LONG HORIZONTAL STROKE\0"
  "ELEMENT OF WITH OVERBAR\0"
  "ELEMENT OF WITH TWO HORIZONTAL STROKES\0"
  "ELEMENT OF WITH UNDERBAR\0"
  "ELEMENT OF WITH VERTICAL BAR AT END OF HORIZONTAL STROKE\0"
  "ELEPHANT\0"
  "ELEVATOR\0"
  "ELF\0"
  "ELYMAIC LETTER ALEPH\0"
  "ELYMAIC LETTER AYIN\0"
  "ELYMAIC LETTER BETH\0"
  "ELYMAIC LETTER DALETH\0"
  "ELYMAIC LETTER GIMEL\0"
  "ELYMAIC LETTER HE\0"
  "ELYMAIC LETTER HETH\0"
  "ELYMAIC LETTER KAPH\0"
  "ELYMAIC LETTER LAMEDH\0"
  "ELYMAIC LETTER MEM\0"
  "ELYMAIC LETTER NUN\0"
  "ELYMAIC LETTER PE\0"
  "ELYMAIC LETTER QOPH\0"
  "ELYMAIC LETTER RESH\0"
  "ELYMAIC LETTER SADHE\0"
  "ELYMAIC LETTER SAMEKH\0"
  "ELYMAIC LETTER SHIN\0"
  "ELYMAIC LETTER TAW\0"
  "ELYMAIC LETTER TETH\0"
  "ELYMAIC LETTER WAW\0"
  "ELYMAIC LETTER YODH\0"
  "ELYMAIC LETTER ZAYIN\0"
  "ELYMAIC LIGATURE ZAYIN-YODH\0"
  "EM DASH\0"
  "EM QUAD\0"
  "EM SPACE\0"
  "EMOJI COMPONENT BALD\0"
  "EMOJI COMPONENT CURLY HAIR\0"
  "EMOJI COMPONENT RED HAIR\0"
  "EMOJI COMPONENT WHITE HAIR\0"
  "EMOJI MODIFIER FITZPATRICK TYPE-1-2\0"
  "EMOJI MODIFIER FITZPATRICK TYPE-3\0"
  "EMOJI MODIFIER FITZPATRICK TYPE-4\0"
  "EMOJI MODIFIER FITZPATRICK TYPE-5\0"
  "EMOJI MODIFIER FITZPATRICK TYPE-6\0"
  "EMPHASIS SYMBOL\0"
  "EMPTY DOCUMENT\0"
  "EMPTY NEST\0"
  "EMPTY NOTE\0"
  "EMPTY NOTE PAD\0"
  "EMPTY NOTE PAGE\0"
  "EMPTY PAGE\0"
  "EMPTY PAGES\0"
  "EMPTY SET\0"
  "EMPTY SET WITH LEFT ARROW ABOVE\0"
  "EMPTY SET WITH OVERBAR\0"
  "EMPTY SET WITH RIGHT ARROW ABOVE\0"
  "EMPTY SET WITH SMALL CIRCLE ABOVE\0"
  "EN DASH\0"
  "EN QUAD\0"
  "EN SPACE\0"
  "END OF PROOF\0"
  "END WITH LEFTWARDS ARROW ABOVE\0"
  "ENTER SYMBOL\0"
  "ENVELOPE\0"
  "ENVELOPE WITH DOWNWARDS ARROW ABOVE\0"
  "ENVELOPE WITH LIGHTNING\0"
  "EQUAL AND PARALLEL TO\0"
  "EQUAL TO BY DEFINITION\0"
  "EQUAL TO OR GREATER-THAN\0"
  "EQUAL TO OR LESS-THAN\0"
  "EQUAL TO OR PRECEDES\0"
  "EQUAL TO OR SUCCEEDS\0"
  "EQUALS COLON\0"
  "EQUALS SIGN\0"
  "EQUALS SIGN ABOVE LEFTWARDS ARROW\0"
  "EQUALS SIGN ABOVE PLUS SIGN\0"
  "EQUALS SIGN ABOVE RIGHTWARDS ARROW\0"
  "EQUALS SIGN ABOVE TILDE OPERATOR\0"
  "EQUALS SIGN AND SLANTED PARALLEL\0"
  "EQUALS SIGN AND SLANTED PARALLEL WITH TILDE ABOVE\0"
  "EQUALS SIGN WITH BUMPY ABOVE\0"
  "EQUALS SIGN WITH DOT BELOW\0"
  "EQUALS SIGN WITH INFINITY BELOW\0"
  "EQUALS SIGN WITH TWO DOTS ABOVE AND TWO DOTS BELOW\0"
  "EQUALS WITH ASTERISK\0"
  "EQUIANGULAR TO\0"
  "EQUIVALENT TO\0"
  "EQUIVALENT WITH FOUR DOTS ABOVE\0"
  "ERASE TO THE LEFT\0"
  "ERASE TO THE RIGHT\0"
  "ERIS FORM ONE\0"
  "ERIS FORM TWO\0"
  "ERROR-BARRED BLACK CIRCLE\0"
  "ERROR-BARRED BLACK DIAMOND\0"
  "ERROR-BARRED BLACK SQUARE\0"
  "ERROR-BARRED WHITE CIRCLE\0"
  "ERROR-BARRED WHITE DIAMOND\0"
  "ERROR-BARRED WHITE SQUARE\0"
  "ESTIMATED SYMBOL\0"
  "ESTIMATES\0"
  "ETHIOPIC COLON\0"
  "ETHIOPIC COMBINING GEMINATION AND VOWEL LENGTH MARK\0"
  "ETHIOPIC COMBINING GEMINATION MARK\0"
  "ETHIOPIC COMBINING VOWEL LENGTH MARK\0"
  "ETHIOPIC COMMA\0"
  "ETHIOPIC DIGIT EIGHT\0"
  "ETHIOPIC DIGIT FIVE\0"
  "ETHIOPIC DIGIT FOUR\0"
  "ETHIOPIC DIGIT NINE\0"
  "ETHIOPIC DIGIT ONE\0"
  "ETHIOPIC DIGIT SEVEN\0"
  "ETHIOPIC DIGIT SIX\0"
  "ETHIOPIC DIGIT THREE\0"
  "ETHIOPIC DIGIT TWO\0"
  "ETHIOPIC FULL STOP\0"
  "ETHIOPIC NUMBER EIGHTY\0"
  "ETHIOPIC NUMBER FIFTY\0"
  "ETHIOPIC NUMBER FORTY\0"
  "ETHIOPIC NUMBER HUNDRED\0"
  "ETHIOPIC NUMBER NINETY\0"
  "ETHIOPIC NUMBER SEVENTY\0"
  "ETHIOPIC NUMBER SIXTY\0"
  "ETHIOPIC NUMBER TEN\0"
  "ETHIOPIC NUMBER TEN THOUSAND\0"
  "ETHIOPIC NUMBER THIRTY\0"
  "ETHIOPIC NUMBER TWENTY\0"
  "ETHIOPIC PARAGRAPH SEPARATOR\0"
  "ETHIOPIC PREFACE COLON\0"
  "ETHIOPIC QUESTION MARK\0"
  "ETHIOPIC SECTION MARK\0"
  "ETHIOPIC SEMICOLON\0"
  "ETHIOPIC SYLLABLE BA\0"
  "ETHIOPIC SYLLABLE BAA\0"
  "ETHIOPIC SYLLABLE BBA\0"
  "ETHIOPIC SYLLABLE BBAA\0"
  "ETHIOPIC SYLLABLE BBE\0"
  "ETHIOPIC SYLLABLE BBEE\0"
  "ETHIOPIC SYLLABLE BBI\0"
  "ETHIOPIC SYLLABLE BBO\0"
  "ETHIOPIC SYLLABLE BBU\0"
  "ETHIOPIC SYLLABLE BE\0"
  "ETHIOPIC SYLLABLE BEE\0"
  "ETHIOPIC SYLLABLE BI\0"
  "ETHIOPIC SYLLABLE BO\0"
  "ETHIOPIC SYLLABLE BOA\0"
  "ETHIOPIC SYLLABLE BU\0"
  "ETHIOPIC SYLLABLE BWA\0"
  "ETHIOPIC SYLLABLE BWE\0"
  "ETHIOPIC SYLLABLE BWEE\0"
  "ETHIOPIC SYLLABLE BWI\0"
  "ETHIOPIC SYLLABLE CA\0"
  "ETHIOPIC SYLLABLE CAA\0"
  "ETHIOPIC SYLLABLE CCA\0"
  "ETHIOPIC SYLLABLE CCAA\0"
  "ETHIOPIC SYLLABLE CCE\0"
  "ETHIOPIC SYLLABLE CCEE\0"
  "ETHIOPIC SYLLABLE CCHA\0"
  "ETHIOPIC SYLLABLE CCHAA\0"
  "ETHIOPIC SYLLABLE CCHE\0"
  "ETHIOPIC SYLLABLE CCHEE\0"
  "ETHIOPIC SYLLABLE CCHHA\0"
  "ETHIOPIC SYLLABLE CCHHAA\0"
  "ETHIOPIC SYLLABLE CCHHE\0"
  "ETHIOPIC SYLLABLE CCHHEE\0"
  "ETHIOPIC SYLLABLE CCHHI\0"
  "ETHIOPIC SYLLABLE CCHHO\0"
  "ETHIOPIC SYLLABLE CCHHU\0"
  "ETHIOPIC SYLLABLE CCHI\0"
  "ETHIOPIC SYLLABLE CCHO\0"
  "ETHIOPIC SYLLABLE CCHU\0"
  "ETHIOPIC SYLLABLE CCI\0"
  "ETHIOPIC SYLLABLE CCO\0"
  "ETHIOPIC SYLLABLE CCU\0"
  "ETHIOPIC SYLLABLE CE\0"
  "ETHIOPIC SYLLABLE CEE\0"
  "ETHIOPIC SYLLABLE CHA\0"
  "ETHIOPIC SYLLABLE CHAA\0"
  "ETHIOPIC SYLLABLE CHE\0"
  "ETHIOPIC SYLLABLE CHEE\0"
  "ETHIOPIC SYLLABLE CHI\0"
  "ETHIOPIC SYLLABLE CHO\0"
  "ETHIOPIC SYLLABLE CHOA\0"
  "ETHIOPIC SYLLABLE CHU\0"
  "ETHIOPIC SYLLABLE CHWA\0"
  "ETHIOPIC SYLLABLE CI\0"
  "ETHIOPIC SYLLABLE CO\0"
  "ETHIOPIC SYLLABLE COA\0"
  "ETHIOPIC SYLLABLE CU\0"
  "ETHIOPIC SYLLABLE CWA\0"
  "ETHIOPIC SYLLABLE DA\0"
  "ETHIOPIC SYLLABLE DAA\0"
  "ETHIOPIC SYLLABLE DDA\0"
  "ETHIOPIC SYLLABLE DDAA\0"
  "ETHIOPIC SYLLABLE DDE\0"
  "ETHIOPIC SYLLABLE DDEE\0"
  "ETHIOPIC SYLLABLE DDHAA\0"
  "ETHIOPIC SYLLABLE DDHE\0"
  "ETHIOPIC SYLLABLE DDHEE\0"
  "ETHIOPIC SYLLABLE DDHI\0"
  "ETHIOPIC SYLLABLE DDHO\0"
  "ETHIOPIC SYLLABLE DDHU\0"
  "ETHIOPIC SYLLABLE DDI\0"
  "ETHIOPIC SYLLABLE DDO\0"
  "ETHIOPIC SYLLABLE DDOA\0"
  "ETHIOPIC SYLLABLE DDU\0"
  "ETHIOPIC SYLLABLE DDWA\0"
  "ETHIOPIC SYLLABLE DE\0"
  "ETHIOPIC SYLLABLE DEE\0"
  "ETHIOPIC SYLLABLE DI\0"
  "ETHIOPIC SYLLABLE DO\0"
  "ETHIOPIC SYLLABLE DOA\0"
  "ETHIOPIC SYLLABLE DU\0"
  "ETHIOPIC SYLLABLE DWA\0"
  "ETHIOPIC SYLLABLE DZAA\0"
  "ETHIOPIC SYLLABLE DZE\0"
  "ETHIOPIC SYLLABLE DZEE\0"
  "ETHIOPIC SYLLABLE DZI\0"
  "ETHIOPIC SYLLABLE DZO\0"
  "ETHIOPIC SYLLABLE DZU\0"
  "ETHIOPIC SYLLABLE FA\0"
  "ETHIOPIC SYLLABLE FAA\0"
  "ETHIOPIC SYLLABLE FE\0"
  "ETHIOPIC SYLLABLE FEE\0"
  "ETHIOPIC SYLLABLE FI\0"
  "ETHIOPIC SYLLABLE FO\0"
  "ETHIOPIC SYLLABLE FU\0"
  "ETHIOPIC SYLLABLE FWA\0"
  "ETHIOPIC SYLLABLE FWE\0"
  "ETHIOPIC SYLLABLE FWEE\0"
  "ETHIOPIC SYLLABLE FWI\0"
  "ETHIOPIC SYLLABLE FYA\0"
  "ETHIOPIC SYLLABLE GA\0"
  "ETHIOPIC SYLLABLE GAA\0"
  "ETHIOPIC SYLLABLE GE\0"
  "ETHIOPIC SYLLABLE GEE\0"
  "ETHIOPIC SYLLABLE GGA\0"
  "ETHIOPIC SYLLABLE GGAA\0"
  "ETHIOPIC SYLLABLE GGE\0"
  "ETHIOPIC SYLLABLE GGEE\0"
  "ETHIOPIC SYLLABLE GGI\0"
  "ETHIOPIC SYLLABLE GGO\0"
  "ETHIOPIC SYLLABLE GGU\0"
  "ETHIOPIC SYLLABLE GGWA\0"
  "ETHIOPIC SYLLABLE GGWAA\0"
  "ETHIOPIC SYLLABLE GGWE\0"
  "ETHIOPIC SYLLABLE GGWEE\0"
  "ETHIOPIC SYLLABLE GGWI\0"
  "ETHIOPIC SYLLABLE GI\0"
  "ETHIOPIC SYLLABLE GLOTTAL A\0"
  "ETHIOPIC SYLLABLE GLOTTAL AA\0"
  "ETHIOPIC SYLLABLE GLOTTAL E\0"
  "ETHIOPIC SYLLABLE GLOTTAL EE\0"
  "ETHIOPIC SYLLABLE GLOTTAL I\0"
  "ETHIOPIC SYLLABLE GLOTTAL O\0"
  "ETHIOPIC SYLLABLE GLOTTAL OA\0"
  "ETHIOPIC SYLLABLE GLOTTAL U\0"
  "ETHIOPIC SYLLABLE GLOTTAL WA\0"
  "ETHIOPIC SYLLABLE GO\0"
  "ETHIOPIC SYLLABLE GOA\0"
  "ETHIOPIC SYLLABLE GU\0"
  "ETHIOPIC SYLLABLE GURAGE BWEE\0"
  "ETHIOPIC SYLLABLE GURAGE BWI\0"
  "ETHIOPIC SYLLABLE GURAGE FWEE\0"
  "ETHIOPIC SYLLABLE GURAGE FWI\0"
  "ETHIOPIC SYLLABLE GURAGE GWE\0"
  "ETHIOPIC SYLLABLE GURAGE GWEE\0"
  "ETHIOPIC SYLLABLE GURAGE GWI\0"
  "ETHIOPIC SYLLABLE GURAGE HHWA\0"
  "ETHIOPIC SYLLABLE GURAGE KWE\0"
  "ETHIOPIC SYLLABLE GURAGE KWEE\0"
  "ETHIOPIC SYLLABLE GURAGE KWI\0"
  "ETHIOPIC SYLLABLE GURAGE MWEE\0"
  "ETHIOPIC SYLLABLE GURAGE MWI\0"
  "ETHIOPIC SYLLABLE GURAGE PWEE\0"
  "ETHIOPIC SYLLABLE GURAGE PWI\0"
  "ETHIOPIC SYLLABLE GURAGE QWE\0"
  "ETHIOPIC SYLLABLE GURAGE QWEE\0"
  "ETHIOPIC SYLLABLE GURAGE QWI\0"
  "ETHIOPIC SYLLABLE GWA\0"
  "ETHIOPIC SYLLABLE GWAA\0"
  "ETHIOPIC SYLLABLE GWE\0"
  "ETHIOPIC SYLLABLE GWEE\0"
  "ETHIOPIC SYLLABLE GWI\0"
  "ETHIOPIC SYLLABLE GYA\0"
  "ETHIOPIC SYLLABLE GYAA\0"
  "ETHIOPIC SYLLABLE GYE\0"
  "ETHIOPIC SYLLABLE GYEE\0"
  "ETHIOPIC SYLLABLE GYI\0"
  "ETHIOPIC SYLLABLE GYO\0"
  "ETHIOPIC SYLLABLE GYU\0"
  "ETHIOPIC SYLLABLE HA\0"
  "ETHIOPIC SYLLABLE HAA\0"
  "ETHIOPIC SYLLABLE HE\0"
  "ETHIOPIC SYLLABLE HEE\0"
  "ETHIOPIC SYLLABLE HHA\0"
  "ETHIOPIC SYLLABLE HHAA\0"
  "ETHIOPIC SYLLABLE HHE\0"
  "ETHIOPIC SYLLABLE HHEE\0"
  "ETHIOPIC SYLLABLE HHI\0"
  "ETHIOPIC SYLLABLE HHO\0"
  "ETHIOPIC SYLLABLE HHU\0"
  "ETHIOPIC SYLLABLE HHWA\0"
  "ETHIOPIC SYLLABLE HHWE\0"
  "ETHIOPIC SYLLABLE HHWEE\0"
  "ETHIOPIC SYLLABLE HHWI\0"
  "ETHIOPIC SYLLABLE HHYA\0"
  "ETHIOPIC SYLLABLE HHYAA\0"
  "ETHIOPIC SYLLABLE HHYE\0"
  "ETHIOPIC SYLLABLE HHYEE\0"
  "ETHIOPIC SYLLABLE HHYI\0"
  "ETHIOPIC SYLLABLE HHYO\0"
  "ETHIOPIC SYLLABLE HHYU\0"
  "ETHIOPIC SYLLABLE HI\0"
  "ETHIOPIC SYLLABLE HO\0"
  "ETHIOPIC SYLLABLE HOA\0"
  "ETHIOPIC SYLLABLE HU\0"
  "ETHIOPIC SYLLABLE JA\0"
  "ETHIOPIC SYLLABLE JAA\0"
  "ETHIOPIC SYLLABLE JE\0"
  "ETHIOPIC SYLLABLE JEE\0"
  "ETHIOPIC SYLLABLE JI\0"
  "ETHIOPIC SYLLABLE JO\0"
  "ETHIOPIC SYLLABLE JOA\0"
  "ETHIOPIC SYLLABLE JU\0"
  "ETHIOPIC SYLLABLE JWA\0"
  "ETHIOPIC SYLLABLE KA\0"
  "ETHIOPIC SYLLABLE KAA\0"
  "ETHIOPIC SYLLABLE KE\0"
  "ETHIOPIC SYLLABLE KEE\0"
  "ETHIOPIC SYLLABLE KI\0"
  "ETHIOPIC SYLLABLE KO\0"
  "ETHIOPIC SYLLABLE KOA\0"
  "ETHIOPIC SYLLABLE KU\0"
  "ETHIOPIC SYLLABLE KWA\0"
  "ETHIOPIC SYLLABLE KWAA\0"
  "ETHIOPIC SYLLABLE KWE\0"
  "ETHIOPIC SYLLABLE KWEE\0"
  "ETHIOPIC SYLLABLE KWI\0"
  "ETHIOPIC SYLLABLE KXA\0"
  "ETHIOPIC SYLLABLE KXAA\0"
  "ETHIOPIC SYLLABLE KXE\0"
  "ETHIOPIC SYLLABLE KXEE\0"
  "ETHIOPIC SYLLABLE KXI\0"
  "ETHIOPIC SYLLABLE KXO\0"
  "ETHIOPIC SYLLABLE KXU\0"
  "ETHIOPIC SYLLABLE KXWA\0"
  "ETHIOPIC SYLLABLE KXWAA\0"
  "ETHIOPIC SYLLABLE KXWE\0"
  "ETHIOPIC SYLLABLE KXWEE\0"
  "ETHIOPIC SYLLABLE KXWI\0"
  "ETHIOPIC SYLLABLE KYA\0"
  "ETHIOPIC SYLLABLE KYAA\0"
  "ETHIOPIC SYLLABLE KYE\0"
  "ETHIOPIC SYLLABLE KYEE\0"
  "ETHIOPIC SYLLABLE KYI\0"
  "ETHIOPIC SYLLABLE KYO\0"
  "ETHIOPIC SYLLABLE KYU\0"
  "ETHIOPIC SYLLABLE LA\0"
  "ETHIOPIC SYLLABLE LAA\0"
  "ETHIOPIC SYLLABLE LE\0"
  "ETHIOPIC SYLLABLE LEE\0"
  "ETHIOPIC SYLLABLE LI\0"
  "ETHIOPIC SYLLABLE LO\0"
  "ETHIOPIC SYLLABLE LOA\0"
  "ETHIOPIC SYLLABLE LU\0"
  "ETHIOPIC SYLLABLE LWA\0"
  "ETHIOPIC SYLLABLE MA\0"
  "ETHIOPIC SYLLABLE MAA\0"
  "ETHIOPIC SYLLABLE ME\0"
  "ETHIOPIC SYLLABLE MEE\0"
  "ETHIOPIC SYLLABLE MI\0"
  "ETHIOPIC SYLLABLE MO\0"
  "ETHIOPIC SYLLABLE MOA\0"
  "ETHIOPIC SYLLABLE MU\0"
  "ETHIOPIC SYLLABLE MWA\0"
  "ETHIOPIC SYLLABLE MWE\0"
  "ETHIOPIC SYLLABLE MWEE\0"
  "ETHIOPIC SYLLABLE MWI\0"
  "ETHIOPIC SYLLABLE MYA\0"
  "ETHIOPIC SYLLABLE NA\0"
  "ETHIOPIC SYLLABLE NAA\0"
  "ETHIOPIC SYLLABLE NE\0"
  "ETHIOPIC SYLLABLE NEE\0"
  "ETHIOPIC SYLLABLE NI\0"
  "ETHIOPIC SYLLABLE NO\0"
  "ETHIOPIC SYLLABLE NOA\0"
  "ETHIOPIC SYLLABLE NU\0"
  "ETHIOPIC SYLLABLE NWA\0"
  "ETHIOPIC SYLLABLE NYA\0"
  "ETHIOPIC SYLLABLE NYAA\0"
  "ETHIOPIC SYLLABLE NYE\0"
  "ETHIOPIC SYLLABLE NYEE\0"
  "ETHIOPIC SYLLABLE NYI\0"
  "ETHIOPIC SYLLABLE NYO\0"
  "ETHIOPIC SYLLABLE NYOA\0"
  "ETHIOPIC SYLLABLE NYU\0"
  "ETHIOPIC SYLLABLE NYWA\0"
  "ETHIOPIC SYLLABLE PA\0"
  "ETHIOPIC SYLLABLE PAA\0"
  "ETHIOPIC SYLLABLE PE\0"
  "ETHIOPIC SYLLABLE PEE\0"
  "ETHIOPIC SYLLABLE PHA\0"
  "ETHIOPIC SYLLABLE PHAA\0"
  "ETHIOPIC SYLLABLE PHARYNGEAL A\0"
  "ETHIOPIC SYLLABLE PHARYNGEAL AA\0"
  "ETHIOPIC SYLLABLE PHARYNGEAL E\0"
  "ETHIOPIC SYLLABLE PHARYNGEAL EE\0"
  "ETHIOPIC SYLLABLE PHARYNGEAL I\0"
  "ETHIOPIC SYLLABLE PHARYNGEAL O\0"
  "ETHIOPIC SYLLABLE PHARYNGEAL U\0"
  "ETHIOPIC SYLLABLE PHE\0"
  "ETHIOPIC SYLLABLE PHEE\0"
  "ETHIOPIC SYLLABLE PHI\0"
  "ETHIOPIC SYLLABLE PHO\0"
  "ETHIOPIC SYLLABLE PHOA\0"
  "ETHIOPIC SYLLABLE PHU\0"
  "ETHIOPIC SYLLABLE PHWA\0"
  "ETHIOPIC SYLLABLE PI\0"
  "ETHIOPIC SYLLABLE PO\0"
  "ETHIOPIC SYLLABLE POA\0"
  "ETHIOPIC SYLLABLE PU\0"
  "ETHIOPIC SYLLABLE PWA\0"
  "ETHIOPIC SYLLABLE PWE\0"
  "ETHIOPIC SYLLABLE PWEE\0"
  "ETHIOPIC SYLLABLE PWI\0"
  "ETHIOPIC SYLLABLE QA\0"
  "ETHIOPIC SYLLABLE QAA\0"
  "ETHIOPIC SYLLABLE QE\0"
  "ETHIOPIC SYLLABLE QEE\0"
  "ETHIOPIC SYLLABLE QHA\0"
  "ETHIOPIC SYLLABLE QHAA\0"
  "ETHIOPIC SYLLABLE QHE\0"
  "ETHIOPIC SYLLABLE QHEE\0"
  "ETHIOPIC SYLLABLE QHI\0"
  "ETHIOPIC SYLLABLE QHO\0"
  "ETHIOPIC SYLLABLE QHU\0"
  "ETHIOPIC SYLLABLE QHWA\0"
  "ETHIOPIC SYLLABLE QHWAA\0"
  "ETHIOPIC SYLLABLE QHWE\0"
  "ETHIOPIC SYLLABLE QHWEE\0"
  "ETHIOPIC SYLLABLE QHWI\0"
  "ETHIOPIC SYLLABLE QI\0"
  "ETHIOPIC SYLLABLE QO\0"
  "ETHIOPIC SYLLABLE QOA\0"
  "ETHIOPIC SYLLABLE QU\0"
  "ETHIOPIC SYLLABLE QWA\0"
  "ETHIOPIC SYLLABLE QWAA\0"
  "ETHIOPIC SYLLABLE QWE\0"
  "ETHIOPIC SYLLABLE QWEE\0"
  "ETHIOPIC SYLLABLE QWI\0"
  "ETHIOPIC SYLLABLE QYA\0"
  "ETHIOPIC SYLLABLE QYAA\0"
  "ETHIOPIC SYLLABLE QYE\0"
  "ETHIOPIC SYLLABLE QYEE\0"
  "ETHIOPIC SYLLABLE QYI\0"
  "ETHIOPIC SYLLABLE QYO\0"
  "ETHIOPIC SYLLABLE QYU\0"
  "ETHIOPIC SYLLABLE RA\0"
  "ETHIOPIC SYLLABLE RAA\0"
  "ETHIOPIC SYLLABLE RE\0"
  "ETHIOPIC SYLLABLE REE\0"
  "ETHIOPIC SYLLABLE RI\0"
  "ETHIOPIC SYLLABLE RO\0"
  "ETHIOPIC SYLLABLE ROA\0"
  "ETHIOPIC SYLLABLE RU\0"
  "ETHIOPIC SYLLABLE RWA\0"
  "ETHIOPIC SYLLABLE RYA\0"
  "ETHIOPIC SYLLABLE SA\0"
  "ETHIOPIC SYLLABLE SAA\0"
  "ETHIOPIC SYLLABLE SE\0"
  "ETHIOPIC SYLLABLE SEBATBEIT BWA\0"
  "ETHIOPIC SYLLABLE SEBATBEIT FWA\0"
  "ETHIOPIC SYLLABLE SEBATBEIT MWA\0"
  "ETHIOPIC SYLLABLE SEBATBEIT PWA\0"
  "ETHIOPIC SYLLABLE SEE\0"
  "ETHIOPIC SYLLABLE SHA\0"
  "ETHIOPIC SYLLABLE SHAA\0"
  "ETHIOPIC SYLLABLE SHE\0"
  "ETHIOPIC SYLLABLE SHEE\0"
  "ETHIOPIC SYLLABLE SHI\0"
  "ETHIOPIC SYLLABLE SHO\0"
  "ETHIOPIC SYLLABLE SHOA\0"
  "ETHIOPIC SYLLABLE SHU\0"
  "ETHIOPIC SYLLABLE SHWA\0"
  "ETHIOPIC SYLLABLE SI\0"
  "ETHIOPIC SYLLABLE SO\0"
  "ETHIOPIC SYLLABLE SOA\0"
  "ETHIOPIC SYLLABLE SSA\0"
  "ETHIOPIC SYLLABLE SSAA\0"
  "ETHIOPIC SYLLABLE SSE\0"
  "ETHIOPIC SYLLABLE SSEE\0"
  "ETHIOPIC SYLLABLE SSI\0"
  "ETHIOPIC SYLLABLE SSO\0"
  "ETHIOPIC SYLLABLE SSU\0"
  "ETHIOPIC SYLLABLE SU\0"
  "ETHIOPIC SYLLABLE SWA\0"
  "ETHIOPIC SYLLABLE SZA\0"
  "ETHIOPIC SYLLABLE SZAA\0"
  "ETHIOPIC SYLLABLE SZE\0"
  "ETHIOPIC SYLLABLE SZEE\0"
  "ETHIOPIC SYLLABLE SZI\0"
  "ETHIOPIC SYLLABLE SZO\0"
  "ETHIOPIC SYLLABLE SZU\0"
  "ETHIOPIC SYLLABLE SZWA\0"
  "ETHIOPIC SYLLABLE TA\0"
  "ETHIOPIC SYLLABLE TAA\0"
  "ETHIOPIC SYLLABLE TE\0"
  "ETHIOPIC SYLLABLE TEE\0"
  "ETHIOPIC SYLLABLE THA\0"
  "ETHIOPIC SYLLABLE THAA\0"
  "ETHIOPIC SYLLABLE THE\0"
  "ETHIOPIC SYLLABLE THEE\0"
  "ETHIOPIC SYLLABLE THI\0"
  "ETHIOPIC SYLLABLE THO\0"
  "ETHIOPIC SYLLABLE THOA\0"
  "ETHIOPIC SYLLABLE THU\0"
  "ETHIOPIC SYLLABLE THWA\0"
  "ETHIOPIC SYLLABLE TI\0"
  "ETHIOPIC SYLLABLE TO\0"
  "ETHIOPIC SYLLABLE TOA\0"
  "ETHIOPIC SYLLABLE TSA\0"
  "ETHIOPIC SYLLABLE TSAA\0"
  "ETHIOPIC SYLLABLE TSE\0"
  "ETHIOPIC SYLLABLE TSEE\0"
  "ETHIOPIC SYLLABLE TSI\0"
  "ETHIOPIC SYLLABLE TSO\0"
  "ETHIOPIC SYLLABLE TSU\0"
  "ETHIOPIC SYLLABLE TSWA\0"
  "ETHIOPIC SYLLABLE TTHAA\0"
  "ETHIOPIC SYLLABLE TTHE\0"
  "ETHIOPIC SYLLABLE TTHEE\0"
  "ETHIOPIC SYLLABLE TTHI\0"
  "ETHIOPIC SYLLABLE TTHO\0"
  "ETHIOPIC SYLLABLE TTHU\0"
  "ETHIOPIC SYLLABLE TU\0"
  "ETHIOPIC SYLLABLE TWA\0"
  "ETHIOPIC SYLLABLE TZA\0"
  "ETHIOPIC SYLLABLE TZAA\0"
  "ETHIOPIC SYLLABLE TZE\0"
  "ETHIOPIC SYLLABLE TZEE\0"
  "ETHIOPIC SYLLABLE TZI\0"
  "ETHIOPIC SYLLABLE TZO\0"
  "ETHIOPIC SYLLABLE TZOA\0"
  "ETHIOPIC SYLLABLE TZU\0"
  "ETHIOPIC SYLLABLE VA\0"
  "ETHIOPIC SYLLABLE VAA\0"
  "ETHIOPIC SYLLABLE VE\0"
  "ETHIOPIC SYLLABLE VEE\0"
  "ETHIOPIC SYLLABLE VI\0"
  "ETHIOPIC SYLLABLE VO\0"
  "ETHIOPIC SYLLABLE VU\0"
  "ETHIOPIC SYLLABLE VWA\0"
  "ETHIOPIC SYLLABLE WA\0"
  "ETHIOPIC SYLLABLE WAA\0"
  "ETHIOPIC SYLLABLE WE\0"
  "ETHIOPIC SYLLABLE WEE\0"
  "ETHIOPIC SYLLABLE WI\0"
  "ETHIOPIC SYLLABLE WO\0"
  "ETHIOPIC SYLLABLE WOA\0"
  "ETHIOPIC SYLLABLE WU\0"
  "ETHIOPIC SYLLABLE XA\0"
  "ETHIOPIC SYLLABLE XAA\0"
  "ETHIOPIC SYLLABLE XE\0"
  "ETHIOPIC SYLLABLE XEE\0"
  "ETHIOPIC SYLLABLE XI\0"
  "ETHIOPIC SYLLABLE XO\0"
  "ETHIOPIC SYLLABLE XOA\0"
  "ETHIOPIC SYLLABLE XU\0"
  "ETHIOPIC SYLLABLE XWA\0"
  "ETHIOPIC SYLLABLE XWAA\0"
  "ETHIOPIC SYLLABLE XWE\0"
  "ETHIOPIC SYLLABLE XWEE\0"
  "ETHIOPIC SYLLABLE XWI\0"
  "ETHIOPIC SYLLABLE XYA\0"
  "ETHIOPIC SYLLABLE XYAA\0"
  "ETHIOPIC SYLLABLE XYE\0"
  "ETHIOPIC SYLLABLE XYEE\0"
  "ETHIOPIC SYLLABLE XYI\0"
  "ETHIOPIC SYLLABLE XYO\0"
  "ETHIOPIC SYLLABLE XYU\0"
  "ETHIOPIC SYLLABLE YA\0"
  "ETHIOPIC SYLLABLE YAA\0"
  "ETHIOPIC SYLLABLE YE\0"
  "ETHIOPIC SYLLABLE YEE\0"
  "ETHIOPIC SYLLABLE YI\0"
  "ETHIOPIC SYLLABLE YO\0"
  "ETHIOPIC SYLLABLE YOA\0"
  "ETHIOPIC SYLLABLE YU\0"
  "ETHIOPIC SYLLABLE ZA\0"
  "ETHIOPIC SYLLABLE ZAA\0"
  "ETHIOPIC SYLLABLE ZE\0"
  "ETHIOPIC SYLLABLE ZEE\0"
  "ETHIOPIC SYLLABLE ZHA\0"
  "ETHIOPIC SYLLABLE ZHAA\0"
  "ETHIOPIC SYLLABLE ZHE\0"
  "ETHIOPIC SYLLABLE ZHEE\0"
  "ETHIOPIC SYLLABLE ZHI\0"
  "ETHIOPIC SYLLABLE ZHO\0"
  "ETHIOPIC SYLLABLE ZHU\0"
  "ETHIOPIC SYLLABLE ZHWA\0"
  "ETHIOPIC SYLLABLE ZI\0"
  "ETHIOPIC SYLLABLE ZO\0"
  "ETHIOPIC SYLLABLE ZOA\0"
  "ETHIOPIC SYLLABLE ZU\0"
  "ETHIOPIC SYLLABLE ZWA\0"
  "ETHIOPIC SYLLABLE ZZA\0"
  "ETHIOPIC SYLLABLE ZZAA\0"
  "ETHIOPIC SYLLABLE ZZE\0"
  "ETHIOPIC SYLLABLE ZZEE\0"
  "ETHIOPIC SYLLABLE ZZI\0"
  "ETHIOPIC SYLLABLE ZZO\0"
  "ETHIOPIC SYLLABLE ZZU\0"
  "ETHIOPIC TONAL MARK CHIRET\0"
  "ETHIOPIC TONAL MARK DERET\0"
  "ETHIOPIC TONAL MARK DERET-HIDET\0"
  "ETHIOPIC TONAL MARK DIFAT\0"
  "ETHIOPIC TONAL MARK HIDET\0"
  "ETHIOPIC TONAL MARK KENAT\0"
  "ETHIOPIC TONAL MARK KURT\0"
  "ETHIOPIC TONAL MARK RIKRIK\0"
  "ETHIOPIC TONAL MARK SHORT RIKRIK\0"
  "ETHIOPIC TONAL MARK YIZET\0"
  "ETHIOPIC WORDSPACE\0"
  "EULER CONSTANT\0"
  "EURO SIGN\0"
  "EURO-CURRENCY SIGN\0"
  "EUROPEAN CASTLE\0"
  "EUROPEAN POST OFFICE\0"
  "EVERGREEN TREE\0"
  "EXCESS\0"
  "EXCLAMATION MARK\0"
  "EXCLAMATION QUESTION MARK\0"
  "EXPLOSION AT HORIZON\0"
  "EXPLOSION FRAME-1\0"
  "EXPLOSION FRAME-2\0"
  "EXPLOSION FRAME-3\0"
  "EXPRESSIONLESS FACE\0"
  "EXTENDED ARABIC-INDIC DIGIT EIGHT\0"
  "EXTENDED ARABIC-INDIC DIGIT FIVE\0"
  "EXTENDED ARABIC-INDIC DIGIT FOUR\0"
  "EXTENDED ARABIC-INDIC DIGIT NINE\0"
  "EXTENDED ARABIC-INDIC DIGIT ONE\0"
  "EXTENDED ARABIC-INDIC DIGIT SEVEN\0"
  "EXTENDED ARABIC-INDIC DIGIT SIX\0"
  "EXTENDED ARABIC-INDIC DIGIT THREE\0"
  "EXTENDED ARABIC-INDIC DIGIT TWO\0"
  "EXTENDED ARABIC-INDIC DIGIT ZERO\0"
  "EXTRATERRESTRIAL ALIEN\0"
  "EXTREMELY HEAVY FIVE SPOKED ASTERISK\0"
  "EXTREMELY HEAVY GREEK CROSS\0"
  "EXTREMELY HEAVY SALTIRE\0"
  "EXTREMELY HEAVY SIX SPOKED ASTERISK\0"
  "EXTREMELY HEAVY WHITE CIRCLE\0"
  "EXTREMELY HEAVY WHITE SQUARE\0"
  "EYE\0"
  "EYEGLASSES\0"
  "EYES\0"
  "FACE HOLDING BACK TEARS\0"
  "FACE MASSAGE\0"
  "FACE PALM\0"
  "FACE SAVOURING DELICIOUS FOOD\0"
  "FACE SCREAMING IN FEAR\0"
  "FACE THROWING A KISS\0"
  "FACE WITH BAGS UNDER EYES\0"
  "FACE WITH COLD SWEAT\0"
  "FACE WITH COWBOY HAT\0"
  "FACE WITH DIAGONAL MOUTH\0"
  "FACE WITH FINGER COVERING CLOSED LIPS\0"
  "FACE WITH HEAD-BANDAGE\0"
  "FACE WITH LOOK OF TRIUMPH\0"
  "FACE WITH MEDICAL MASK\0"
  "FACE WITH MONOCLE\0"
  "FACE WITH NO GOOD GESTURE\0"
  "FACE WITH OK GESTURE\0"
  "FACE WITH ONE EYEBROW RAISED\0"
  "FACE WITH OPEN EYES AND HAND OVER MOUTH\0"
  "FACE WITH OPEN MOUTH\0"
  "FACE WITH OPEN MOUTH AND COLD SWEAT\0"
  "FACE WITH OPEN MOUTH VOMITING\0"
  "FACE WITH PARTY HORN AND PARTY HAT\0"
  "FACE WITH PEEKING EYE\0"
  "FACE WITH PLEADING EYES\0"
  "FACE WITH ROLLING EYES\0"
  "FACE WITH STUCK-OUT TONGUE\0"
  "FACE WITH STUCK-OUT TONGUE AND TIGHTLY-CLOSED EYES\0"
  "FACE WITH STUCK-OUT TONGUE AND WINKING EYE\0"
  "FACE WITH TEARS OF JOY\0"
  "FACE WITH THERMOMETER\0"
  "FACE WITH UNEVEN EYES AND WAVY MOUTH\0"
  "FACE WITHOUT MOUTH\0"
  "FACSIMILE SIGN\0"
  "FACTORY\0"
  "FAIRY\0"
  "FALAFEL\0"
  "FALLEN LEAF\0"
  "FALLING DIAGONAL CROSSING NORTH EAST ARROW\0"
  "FALLING DIAGONAL CROSSING RISING DIAGONAL\0"
  "FALLING DIAGONAL IN WHITE CIRCLE IN BLACK SQUARE\0"
  "FAMILY\0"
  "FARSI SYMBOL\0"
  "FATHER CHRISTMAS\0"
  "FAX ICON\0"
  "FAX MACHINE\0"
  "FEARFUL FACE\0"
  "FEATHER\0"
  "FEMALE SIGN\0"
  "FEMININE ORDINAL INDICATOR\0"
  "FENCER\0"
  "FERRIS WHEEL\0"
  "FERRY\0"
  "FIELD HOCKEY STICK AND BALL\0"
  "FIGURE DASH\0"
  "FIGURE SPACE\0"
  "FILE CABINET\0"
  "FILE FOLDER\0"
  "FILM FRAMES\0"
  "FILM PROJECTOR\0"
  "FINGERPRINT\0"
  "FINITE PART INTEGRAL\0"
  "FIRE\0"
  "FIRE ENGINE\0"
  "FIRE EXTINGUISHER\0"
  "FIRECRACKER\0"
  "FIREWORK SPARKLER\0"
  "FIREWORKS\0"
  "FIRST PLACE MEDAL\0"
  "FIRST QUARTER MOON\0"
  "FIRST QUARTER MOON SYMBOL\0"
  "FIRST QUARTER MOON WITH FACE\0"
  "FIRST STRONG ISOLATE\0"
  "FISH\0"
  "FISH CAKE WITH SWIRL DESIGN\0"
  "FISHEYE\0"
  "FISHING POLE AND FISH\0"
  "FISTED HAND SIGN\0"
  "FIVE DOT MARK\0"
  "FIVE DOT PUNCTUATION\0"
  "FLAG IN HOLE\0"
  "FLAMINGO\0"
  "FLAPPING BIRD\0"
  "FLAT SHOE\0"
  "FLATBREAD\0"
  "FLATNESS\0"
  "FLEUR-DE-LIS\0"
  "FLEXED BICEPS\0"
  "FLOPPY DISK\0"
  "FLORAL HEART\0"
  "FLOWER\0"
  "FLOWER PLAYING CARDS\0"
  "FLOWER PUNCTUATION MARK\0"
  "FLUSHED FACE\0"
  "FLUTE\0"
  "FLY\0"
  "FLYING DISC\0"
  "FLYING ENVELOPE\0"
  "FLYING SAUCER\0"
  "FLYING SAUCER WITH BEAMS\0"
  "FLYING SAUCER WITHOUT BEAMS\0"
  "FOG\0"
  "FOGGY\0"
  "FOLDER\0"
  "FOLDING HAND FAN\0"
  "FONDUE\0"
  "FOOT\0"
  "FOOTPRINTS\0"
  "FOR ALL\0"
  "FORCES\0"
  "FORK AND KNIFE\0"
  "FORK AND KNIFE WITH PLATE\0"
  "FORKED PARAGRAPHOS\0"
  "FORKING\0"
  "FORTUNE COOKIE\0"
  "FOUNTAIN\0"
  "FOUR BALLOON-SPOKED ASTERISK\0"
  "FOUR CLUB-SPOKED ASTERISK\0"
  "FOUR CORNER ARROWS CIRCLING ANTICLOCKWISE\0"
  "FOUR DOT MARK\0"
  "FOUR DOT PUNCTUATION\0"
  "FOUR LEAF CLOVER\0"
  "FOUR POINTED BLACK STAR\0"
  "FOUR TEARDROP-SPOKED ASTERISK\0"
  "FOUR-PER-EM SPACE\0"
  "FOURTH ROOT\0"
  "FOX FACE\0"
  "FRACTION NUMERATOR ONE\0"
  "FRACTION SLASH\0"
  "FRAME WITH AN X\0"
  "FRAME WITH PICTURE\0"
  "FRAME WITH TILES\0"
  "FREEZING FACE\0"
  "FRENCH FRANC SIGN\0"
  "FRENCH FRIES\0"
  "FRIED SHRIMP\0"
  "FROG FACE\0"
  "FRONT-FACING BABY CHICK\0"
  "FRONT-TILTED SHADOWED WHITE RIGHTWARDS ARROW\0"
  "FROWN\0"
  "FROWNING FACE WITH OPEN MOUTH\0"
  "FUEL PUMP\0"
  "FULL BLOCK\0"
  "FULL MOON SYMBOL\0"
  "FULL MOON WITH FACE\0"
  "FULL OUTER JOIN\0"
  "FULL STOP\0"
  "FULLWIDTH AMPERSAND\0"
  "FULLWIDTH APOSTROPHE\0"
  "FULLWIDTH ASTERISK\0"
  "FULLWIDTH BROKEN BAR\0"
  "FULLWIDTH CENT SIGN\0"
  "FULLWIDTH CIRCUMFLEX ACCENT\0"
  "FULLWIDTH COLON\0"
  "FULLWIDTH COMMA\0"
  "FULLWIDTH COMMERCIAL AT\0"
  "FULLWIDTH DIGIT EIGHT\0"
  "FULLWIDTH DIGIT FIVE\0"
  "FULLWIDTH DIGIT FOUR\0"
  "FULLWIDTH DIGIT NINE\0"
  "FULLWIDTH DIGIT ONE\0"
  "FULLWIDTH DIGIT SEVEN\0"
  "FULLWIDTH DIGIT SIX\0"
  "FULLWIDTH DIGIT THREE\0"
  "FULLWIDTH DIGIT TWO\0"
  "FULLWIDTH DIGIT ZERO\0"
  "FULLWIDTH DOLLAR SIGN\0"
  "FULLWIDTH EQUALS SIGN\0"
  "FULLWIDTH EXCLAMATION MARK\0"
  "FULLWIDTH FULL STOP\0"
  "FULLWIDTH GRAVE ACCENT\0"
  "FULLWIDTH GREATER-THAN SIGN\0"
  "FULLWIDTH HYPHEN-MINUS\0"
  "FULLWIDTH LATIN CAPITAL LETTER A\0"
  "FULLWIDTH LATIN CAPITAL LETTER B\0"
  "FULLWIDTH LATIN CAPITAL LETTER C\0"
  "FULLWIDTH LATIN CAPITAL LETTER D\0"
  "FULLWIDTH LATIN CAPITAL LETTER E\0"
  "FULLWIDTH LATIN CAPITAL LETTER F\0"
  "FULLWIDTH LATIN CAPITAL LETTER G\0"
  "FULLWIDTH LATIN CAPITAL LETTER H\0"
  "FULLWIDTH LATIN CAPITAL LETTER I\0"
  "FULLWIDTH LATIN CAPITAL LETTER J\0"
  "FULLWIDTH LATIN CAPITAL LETTER K\0"
  "FULLWIDTH LATIN CAPITAL LETTER L\0"
  "FULLWIDTH LATIN CAPITAL LETTER M\0"
  "FULLWIDTH LATIN CAPITAL LETTER N\0"
  "FULLWIDTH LATIN CAPITAL LETTER O\0"
  "FULLWIDTH LATIN CAPITAL LETTER P\0"
  "FULLWIDTH LATIN CAPITAL LETTER Q\0"
  "FULLWIDTH LATIN CAPITAL LETTER R\0"
  "FULLWIDTH LATIN CAPITAL LETTER S\0"
  "FULLWIDTH LATIN CAPITAL LETTER T\0"
  "FULLWIDTH LATIN CAPITAL LETTER U\0"
  "FULLWIDTH LATIN CAPITAL LETTER V\0"
  "FULLWIDTH LATIN CAPITAL LETTER W\0"
  "FULLWIDTH LATIN CAPITAL LETTER X\0"
  "FULLWIDTH LATIN CAPITAL LETTER Y\0"
  "FULLWIDTH LATIN CAPITAL LETTER Z\0"
  "FULLWIDTH LATIN SMALL LETTER A\0"
  "FULLWIDTH LATIN SMALL LETTER B\0"
  "FULLWIDTH LATIN SMALL LETTER C\0"
  "FULLWIDTH LATIN SMALL LETTER D\0"
  "FULLWIDTH LATIN SMALL LETTER E\0"
  "FULLWIDTH LATIN SMALL LETTER F\0"
  "FULLWIDTH LATIN SMALL LETTER G\0"
  "FULLWIDTH LATIN SMALL LETTER H\0"
  "FULLWIDTH LATIN SMALL LETTER I\0"
  "FULLWIDTH LATIN SMALL LETTER J\0"
  "FULLWIDTH LATIN SMALL LETTER K\0"
  "FULLWIDTH LATIN SMALL LETTER L\0"
  "FULLWIDTH LATIN SMALL LETTER M\0"
  "FULLWIDTH LATIN SMALL LETTER N\0"
  "FULLWIDTH LATIN SMALL LETTER O\0"
  "FULLWIDTH LATIN SMALL LETTER P\0"
  "FULLWIDTH LATIN SMALL LETTER Q\0"
  "FULLWIDTH LATIN SMALL LETTER R\0"
  "FULLWIDTH LATIN SMALL LETTER S\0"
  "FULLWIDTH LATIN SMALL LETTER T\0"
  "FULLWIDTH LATIN SMALL LETTER U\0"
  "FULLWIDTH LATIN SMALL LETTER V\0"
  "FULLWIDTH LATIN SMALL LETTER W\0"
  "FULLWIDTH LATIN SMALL LETTER X\0"
  "FULLWIDTH LATIN SMALL LETTER Y\0"
  "FULLWIDTH LATIN SMALL LETTER Z\0"
  "FULLWIDTH LEFT CURLY BRACKET\0"
  "FULLWIDTH LEFT PARENTHESIS\0"
  "FULLWIDTH LEFT SQUARE BRACKET\0"
  "FULLWIDTH LEFT WHITE PARENTHESIS\0"
  "FULLWIDTH LESS-THAN SIGN\0"
  "FULLWIDTH LOW LINE\0"
  "FULLWIDTH MACRON\0"
  "FULLWIDTH NOT SIGN\0"
  "FULLWIDTH NUMBER SIGN\0"
  "FULLWIDTH PERCENT SIGN\0"
  "FULLWIDTH PLUS SIGN\0"
  "FULLWIDTH POUND SIGN\0"
  "FULLWIDTH QUESTION MARK\0"
  "FULLWIDTH QUOTATION MARK\0"
  "FULLWIDTH REVERSE SOLIDUS\0"
  "FULLWIDTH RIGHT CURLY BRACKET\0"
  "FULLWIDTH RIGHT PARENTHESIS\0"
  "FULLWIDTH RIGHT SQUARE BRACKET\0"
  "FULLWIDTH RIGHT WHITE PARENTHESIS\0"
  "FULLWIDTH SEMICOLON\0"
  "FULLWIDTH SOLIDUS\0"
  "FULLWIDTH TILDE\0"
  "FULLWIDTH VERTICAL LINE\0"
  "FULLWIDTH WON SIGN\0"
  "FULLWIDTH YEN SIGN\0"
  "FUNCTION APPLICATION\0"
  "FUNERAL URN\0"
  "FUNNEL\0"
  "FUSE\0"
  "GAME DIE\0"
  "GARAY CAPITAL LETTER A\0"
  "GARAY CAPITAL LETTER BA\0"
  "GARAY CAPITAL LETTER CA\0"
  "GARAY CAPITAL LETTER DA\0"
  "GARAY CAPITAL LETTER FA\0"
  "GARAY CAPITAL LETTER GA\0"
  "GARAY CAPITAL LETTER HA\0"
  "GARAY CAPITAL LETTER JA\0"
  "GARAY CAPITAL LETTER KA\0"
  "GARAY CAPITAL LETTER LA\0"
  "GARAY CAPITAL LETTER MA\0"
  "GARAY CAPITAL LETTER NA\0"
  "GARAY CAPITAL LETTER NYA\0"
  "GARAY CAPITAL LETTER OLD KA\0"
  "GARAY CAPITAL LETTER OLD NA\0"
  "GARAY CAPITAL LETTER PA\0"
  "GARAY CAPITAL LETTER RA\0"
  "GARAY CAPITAL LETTER SA\0"
  "GARAY CAPITAL LETTER TA\0"
  "GARAY CAPITAL LETTER WA\0"
  "GARAY CAPITAL LETTER XA\0"
  "GARAY CAPITAL LETTER YA\0"
  "GARAY COMBINING DOT ABOVE\0"
  "GARAY COMBINING DOUBLE DOT ABOVE\0"
  "GARAY CONSONANT GEMINATION MARK\0"
  "GARAY CONSONANT NASALIZATION MARK\0"
  "GARAY DIGIT EIGHT\0"
  "GARAY DIGIT FIVE\0"
  "GARAY DIGIT FOUR\0"
  "GARAY DIGIT NINE\0"
  "GARAY DIGIT ONE\0"
  "GARAY DIGIT SEVEN\0"
  "GARAY DIGIT SIX\0"
  "GARAY DIGIT THREE\0"
  "GARAY DIGIT TWO\0"
  "GARAY DIGIT ZERO\0"
  "GARAY HYPHEN\0"
  "GARAY MINUS SIGN\0"
  "GARAY PLUS SIGN\0"
  "GARAY REDUPLICATION MARK\0"
  "GARAY SMALL LETTER A\0"
  "GARAY SMALL LETTER BA\0"
  "GARAY SMALL LETTER CA\0"
  "GARAY SMALL LETTER DA\0"
  "GARAY SMALL LETTER FA\0"
  "GARAY SMALL LETTER GA\0"
  "GARAY SMALL LETTER HA\0"
  "GARAY SMALL LETTER JA\0"
  "GARAY SMALL LETTER KA\0"
  "GARAY SMALL LETTER LA\0"
  "GARAY SMALL LETTER MA\0"
  "GARAY SMALL LETTER NA\0"
  "GARAY SMALL LETTER NYA\0"
  "GARAY SMALL LETTER OLD KA\0"
  "GARAY SMALL LETTER OLD NA\0"
  "GARAY SMALL LETTER PA\0"
  "GARAY SMALL LETTER RA\0"
  "GARAY SMALL LETTER SA\0"
  "GARAY SMALL LETTER TA\0"
  "GARAY SMALL LETTER WA\0"
  "GARAY SMALL LETTER XA\0"
  "GARAY SMALL LETTER YA\0"
  "GARAY SUKUN\0"
  "GARAY VOWEL LENGTH MARK\0"
  "GARAY VOWEL SIGN A\0"
  "GARAY VOWEL SIGN E\0"
  "GARAY VOWEL SIGN EE\0"
  "GARAY VOWEL SIGN I\0"
  "GARAY VOWEL SIGN O\0"
  "GARLIC\0"
  "GEAR\0"
  "GEAR WITH HANDLES\0"
  "GEAR WITHOUT HUB\0"
  "GEM STONE\0"
  "GEMINI\0"
  "GENIE\0"
  "GEOMETRIC PROPORTION\0"
  "GEOMETRICALLY EQUAL TO\0"
  "GEOMETRICALLY EQUIVALENT TO\0"
  "GEORGIAN CAPITAL LETTER AEN\0"
  "GEORGIAN CAPITAL LETTER AN\0"
  "GEORGIAN CAPITAL LETTER BAN\0"
  "GEORGIAN CAPITAL LETTER CAN\0"
  "GEORGIAN CAPITAL LETTER CHAR\0"
  "GEORGIAN CAPITAL LETTER CHIN\0"
  "GEORGIAN CAPITAL LETTER CIL\0"
  "GEORGIAN CAPITAL LETTER DON\0"
  "GEORGIAN CAPITAL LETTER EN\0"
  "GEORGIAN CAPITAL LETTER GAN\0"
  "GEORGIAN CAPITAL LETTER GHAN\0"
  "GEORGIAN CAPITAL LETTER HAE\0"
  "GEORGIAN CAPITAL LETTER HAR\0"
  "GEORGIAN CAPITAL LETTER HE\0"
  "GEORGIAN CAPITAL LETTER HIE\0"
  "GEORGIAN CAPITAL LETTER HOE\0"
  "GEORGIAN CAPITAL LETTER IN\0"
  "GEORGIAN CAPITAL LETTER JHAN\0"
  "GEORGIAN CAPITAL LETTER JIL\0"
  "GEORGIAN CAPITAL LETTER KAN\0"
  "GEORGIAN CAPITAL LETTER KHAR\0"
  "GEORGIAN CAPITAL LETTER LAS\0"
  "GEORGIAN CAPITAL LETTER MAN\0"
  "GEORGIAN CAPITAL LETTER NAR\0"
  "GEORGIAN CAPITAL LETTER ON\0"
  "GEORGIAN CAPITAL LETTER PAR\0"
  "GEORGIAN CAPITAL LETTER PHAR\0"
  "GEORGIAN CAPITAL LETTER QAR\0"
  "GEORGIAN CAPITAL LETTER RAE\0"
  "GEORGIAN CAPITAL LETTER SAN\0"
  "GEORGIAN CAPITAL LETTER SHIN\0"
  "GEORGIAN CAPITAL LETTER TAN\0"
  "GEORGIAN CAPITAL LETTER TAR\0"
  "GEORGIAN CAPITAL LETTER UN\0"
  "GEORGIAN CAPITAL LETTER VIN\0"
  "GEORGIAN CAPITAL LETTER WE\0"
  "GEORGIAN CAPITAL LETTER XAN\0"
  "GEORGIAN CAPITAL LETTER YN\0"
  "GEORGIAN CAPITAL LETTER ZEN\0"
  "GEORGIAN CAPITAL LETTER ZHAR\0"
  "GEORGIAN LETTER AEN\0"
  "GEORGIAN LETTER AIN\0"
  "GEORGIAN LETTER AN\0"
  "GEORGIAN LETTER BAN\0"
  "GEORGIAN LETTER CAN\0"
  "GEORGIAN LETTER CHAR\0"
  "GEORGIAN LETTER CHIN\0"
  "GEORGIAN LETTER CIL\0"
  "GEORGIAN LETTER DON\0"
  "GEORGIAN LETTER ELIFI\0"
  "GEORGIAN LETTER EN\0"
  "GEORGIAN LETTER FI\0"
  "GEORGIAN LETTER GAN\0"
  "GEORGIAN LETTER GHAN\0"
  "GEORGIAN LETTER HAE\0"
  "GEORGIAN LETTER HAR\0"
  "GEORGIAN LETTER HARD SIGN\0"
  "GEORGIAN LETTER HE\0"
  "GEORGIAN LETTER HIE\0"
  "GEORGIAN LETTER HOE\0"
  "GEORGIAN LETTER IN\0"
  "GEORGIAN LETTER JHAN\0"
  "GEORGIAN LETTER JIL\0"
  "GEORGIAN LETTER KAN\0"
  "GEORGIAN LETTER KHAR\0"
  "GEORGIAN LETTER LABIAL SIGN\0"
  "GEORGIAN LETTER LAS\0"
  "GEORGIAN LETTER MAN\0"
  "GEORGIAN LETTER NAR\0"
  "GEORGIAN LETTER ON\0"
  "GEORGIAN LETTER PAR\0"
  "GEORGIAN LETTER PHAR\0"
  "GEORGIAN LETTER QAR\0"
  "GEORGIAN LETTER RAE\0"
  "GEORGIAN LETTER SAN\0"
  "GEORGIAN LETTER SHIN\0"
  "GEORGIAN LETTER TAN\0"
  "GEORGIAN LETTER TAR\0"
  "GEORGIAN LETTER TURNED GAN\0"
  "GEORGIAN LETTER UN\0"
  "GEORGIAN LETTER VIN\0"
  "GEORGIAN LETTER WE\0"
  "GEORGIAN LETTER XAN\0"
  "GEORGIAN LETTER YN\0"
  "GEORGIAN LETTER ZEN\0"
  "GEORGIAN LETTER ZHAR\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER AEN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER AIN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER AN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER BAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER CAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER CHAR\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER CHIN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER CIL\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER DON\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER ELIFI\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER EN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER FI\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER GAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER GHAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER HAE\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER HAR\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER HARD SIGN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER HE\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER HIE\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER HOE\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER IN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER JHAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER JIL\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER KAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER KHAR\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER LABIAL SIGN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER LAS\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER MAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER NAR\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER ON\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER PAR\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER PHAR\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER QAR\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER RAE\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER SAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER SHIN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER TAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER TAR\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER TURNED GAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER UN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER VIN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER WE\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER XAN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER YN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER ZEN\0"
  "GEORGIAN MTAVRULI CAPITAL LETTER ZHAR\0"
  "GEORGIAN PARAGRAPH SEPARATOR\0"
  "GEORGIAN SMALL LETTER AEN\0"
  "GEORGIAN SMALL LETTER AN\0"
  "GEORGIAN SMALL LETTER BAN\0"
  "GEORGIAN SMALL LETTER CAN\0"
  "GEORGIAN SMALL LETTER CHAR\0"
  "GEORGIAN SMALL LETTER CHIN\0"
  "GEORGIAN SMALL LETTER CIL\0"
  "GEORGIAN SMALL LETTER DON\0"
  "GEORGIAN SMALL LETTER EN\0"
  "GEORGIAN SMALL LETTER GAN\0"
  "GEORGIAN SMALL LETTER GHAN\0"
  "GEORGIAN SMALL LETTER HAE\0"
  "GEORGIAN SMALL LETTER HAR\0"
  "GEORGIAN SMALL LETTER HE\0"
  "GEORGIAN SMALL LETTER HIE\0"
  "GEORGIAN SMALL LETTER HOE\0"
  "GEORGIAN SMALL LETTER IN\0"
  "GEORGIAN SMALL LETTER JHAN\0"
  "GEORGIAN SMALL LETTER JIL\0"
  "GEORGIAN SMALL LETTER KAN\0"
  "GEORGIAN SMALL LETTER KHAR\0"
  "GEORGIAN SMALL LETTER LAS\0"
  "GEORGIAN SMALL LETTER MAN\0"
  "GEORGIAN SMALL LETTER NAR\0"
  "GEORGIAN SMALL LETTER ON\0"
  "GEORGIAN SMALL LETTER PAR\0"
  "GEORGIAN SMALL LETTER PHAR\0"
  "GEORGIAN SMALL LETTER QAR\0"
  "GEORGIAN SMALL LETTER RAE\0"
  "GEORGIAN SMALL LETTER SAN\0"
  "GEORGIAN SMALL LETTER SHIN\0"
  "GEORGIAN SMALL LETTER TAN\0"
  "GEORGIAN SMALL LETTER TAR\0"
  "GEORGIAN SMALL LETTER UN\0"
  "GEORGIAN SMALL LETTER VIN\0"
  "GEORGIAN SMALL LETTER WE\0"
  "GEORGIAN SMALL LETTER XAN\0"
  "GEORGIAN SMALL LETTER YN\0"
  "GEORGIAN SMALL LETTER ZEN\0"
  "GEORGIAN SMALL LETTER ZHAR\0"
  "GERMAN PENNY SIGN\0"
  "GETA MARK\0"
  "GHOST\0"
  "GIMEL SYMBOL\0"
  "GINGER ROOT\0"
  "GIRAFFE FACE\0"
  "GIRL\0"
  "GIRLS SYMBOL\0"
  "GLAGOLITIC CAPITAL LETTER AZU\0"
  "GLAGOLITIC CAPITAL LETTER BIG YUS\0"
  "GLAGOLITIC CAPITAL LETTER BUKY\0"
  "GLAGOLITIC CAPITAL LETTER CAUDATE CHRIVI\0"
  "GLAGOLITIC CAPITAL LETTER CHRIVI\0"
  "GLAGOLITIC CAPITAL LETTER DJERVI\0"
  "GLAGOLITIC CAPITAL LETTER DOBRO\0"
  "GLAGOLITIC CAPITAL LETTER DZELO\0"
  "GLAGOLITIC CAPITAL LETTER FITA\0"
  "GLAGOLITIC CAPITAL LETTER FRITU\0"
  "GLAGOLITIC CAPITAL LETTER GLAGOLI\0"
  "GLAGOLITIC CAPITAL LETTER HERU\0"
  "GLAGOLITIC CAPITAL LETTER I\0"
  "GLAGOLITIC CAPITAL LETTER INITIAL IZHE\0"
  "GLAGOLITIC CAPITAL LETTER IOTATED BIG YUS\0"
  "GLAGOLITIC CAPITAL LETTER IOTATED SMALL YUS\0"
  "GLAGOLITIC CAPITAL LETTER IZHE\0"
  "GLAGOLITIC CAPITAL LETTER IZHITSA\0"
  "GLAGOLITIC CAPITAL LETTER KAKO\0"
  "GLAGOLITIC CAPITAL LETTER LATINATE MYSLITE\0"
  "GLAGOLITIC CAPITAL LETTER LJUDIJE\0"
  "GLAGOLITIC CAPITAL LETTER MYSLITE\0"
  "GLAGOLITIC CAPITAL LETTER NASHI\0"
  "GLAGOLITIC CAPITAL LETTER ONU\0"
  "GLAGOLITIC CAPITAL LETTER OTU\0"
  "GLAGOLITIC CAPITAL LETTER PE\0"
  "GLAGOLITIC CAPITAL LETTER POKOJI\0"
  "GLAGOLITIC CAPITAL LETTER RITSI\0"
  "GLAGOLITIC CAPITAL LETTER SHA\0"
  "GLAGOLITIC CAPITAL LETTER SHTA\0"
  "GLAGOLITIC CAPITAL LETTER SHTAPIC\0"
  "GLAGOLITIC CAPITAL LETTER SLOVO\0"
  "GLAGOLITIC CAPITAL LETTER SMALL YUS\0"
  "GLAGOLITIC CAPITAL LETTER SMALL YUS WITH TAIL\0"
  "GLAGOLITIC CAPITAL LETTER SPIDERY HA\0"
  "GLAGOLITIC CAPITAL LETTER TROKUTASTI A\0"
  "GLAGOLITIC CAPITAL LETTER TSI\0"
  "GLAGOLITIC CAPITAL LETTER TVRIDO\0"
  "GLAGOLITIC CAPITAL LETTER UKU\0"
  "GLAGOLITIC CAPITAL LETTER VEDE\0"
  "GLAGOLITIC CAPITAL LETTER YATI\0"
  "GLAGOLITIC CAPITAL LETTER YERI\0"
  "GLAGOLITIC CAPITAL LETTER YERU\0"
  "GLAGOLITIC CAPITAL LETTER YESTU\0"
  "GLAGOLITIC CAPITAL LETTER YO\0"
  "GLAGOLITIC CAPITAL LETTER YU\0"
  "GLAGOLITIC CAPITAL LETTER ZEMLJA\0"
  "GLAGOLITIC CAPITAL LETTER ZHIVETE\0"
  "GLAGOLITIC SMALL LETTER AZU\0"
  "GLAGOLITIC SMALL LETTER BIG YUS\0"
  "GLAGOLITIC SMALL LETTER BUKY\0"
  "GLAGOLITIC SMALL LETTER CAUDATE CHRIVI\0"
  "GLAGOLITIC SMALL LETTER CHRIVI\0"
  "GLAGOLITIC SMALL LETTER DJERVI\0"
  "GLAGOLITIC SMALL LETTER DOBRO\0"
  "GLAGOLITIC SMALL LETTER DZELO\0"
  "GLAGOLITIC SMALL LETTER FITA\0"
  "GLAGOLITIC SMALL LETTER FRITU\0"
  "GLAGOLITIC SMALL LETTER GLAGOLI\0"
  "GLAGOLITIC SMALL LETTER HERU\0"
  "GLAGOLITIC SMALL LETTER I\0"
  "GLAGOLITIC SMALL LETTER INITIAL IZHE\0"
  "GLAGOLITIC SMALL LETTER IOTATED BIG YUS\0"
  "GLAGOLITIC SMALL LETTER IOTATED SMALL YUS\0"
  "GLAGOLITIC SMALL LETTER IZHE\0"
  "GLAGOLITIC SMALL LETTER IZHITSA\0"
  "GLAGOLITIC SMALL LETTER KAKO\0"
  "GLAGOLITIC SMALL LETTER LATINATE MYSLITE\0"
  "GLAGOLITIC SMALL LETTER LJUDIJE\0"
  "GLAGOLITIC SMALL LETTER MYSLITE\0"
  "GLAGOLITIC SMALL LETTER NASHI\0"
  "GLAGOLITIC SMALL LETTER ONU\0"
  "GLAGOLITIC SMALL LETTER OTU\0"
  "GLAGOLITIC SMALL LETTER PE\0"
  "GLAGOLITIC SMALL LETTER POKOJI\0"
  "GLAGOLITIC SMALL LETTER RITSI\0"
  "GLAGOLITIC SMALL LETTER SHA\0"
  "GLAGOLITIC SMALL LETTER SHTA\0"
  "GLAGOLITIC SMALL LETTER SHTAPIC\0"
  "GLAGOLITIC SMALL LETTER SLOVO\0"
  "GLAGOLITIC SMALL LETTER SMALL YUS\0"
  "GLAGOLITIC SMALL LETTER SMALL YUS WITH TAIL\0"
  "GLAGOLITIC SMALL LETTER SPIDERY HA\0"
  "GLAGOLITIC SMALL LETTER TROKUTASTI A\0"
  "GLAGOLITIC SMALL LETTER TSI\0"
  "GLAGOLITIC SMALL LETTER TVRIDO\0"
  "GLAGOLITIC SMALL LETTER UKU\0"
  "GLAGOLITIC SMALL LETTER VEDE\0"
  "GLAGOLITIC SMALL LETTER YATI\0"
  "GLAGOLITIC SMALL LETTER YERI\0"
  "GLAGOLITIC SMALL LETTER YERU\0"
  "GLAGOLITIC SMALL LETTER YESTU\0"
  "GLAGOLITIC SMALL LETTER YO\0"
  "GLAGOLITIC SMALL LETTER YU\0"
  "GLAGOLITIC SMALL LETTER ZEMLJA\0"
  "GLAGOLITIC SMALL LETTER ZHIVETE\0"
  "GLASS OF MILK\0"
  "GLEICH STARK\0"
  "GLOBE WITH MERIDIANS\0"
  "GLOVES\0"
  "GLOWING STAR\0"
  "GOAL NET\0"
  "GOAT\0"
  "GOGGLES\0"
  "GOLFER\0"
  "GONGGONG\0"
  "GOOSE\0"
  "GORILLA\0"
  "GOTHIC LETTER AHSA\0"
  "GOTHIC LETTER AIHVUS\0"
  "GOTHIC LETTER BAIRKAN\0"
  "GOTHIC LETTER DAGS\0"
  "GOTHIC LETTER EIS\0"
  "GOTHIC LETTER FAIHU\0"
  "GOTHIC LETTER GIBA\0"
  "GOTHIC LETTER HAGL\0"
  "GOTHIC LETTER HWAIR\0"
  "GOTHIC LETTER IGGWS\0"
  "GOTHIC LETTER IUJA\0"
  "GOTHIC LETTER JER\0"
  "GOTHIC LETTER KUSMA\0"
  "GOTHIC LETTER LAGUS\0"
  "GOTHIC LETTER MANNA\0"
  "GOTHIC LETTER NAUTHS\0"
  "GOTHIC LETTER NINE HUNDRED\0"
  "GOTHIC LETTER NINETY\0"
  "GOTHIC LETTER OTHAL\0"
  "GOTHIC LETTER PAIRTHRA\0"
  "GOTHIC LETTER QAIRTHRA\0"
  "GOTHIC LETTER RAIDA\0"
  "GOTHIC LETTER SAUIL\0"
  "GOTHIC LETTER TEIWS\0"
  "GOTHIC LETTER THIUTH\0"
  "GOTHIC LETTER URUS\0"
  "GOTHIC LETTER WINJA\0"
  "GRADUATION CAP\0"
  "GRANTHA AU LENGTH MARK\0"
  "GRANTHA LETTER A\0"
  "GRANTHA LETTER AA\0"
  "GRANTHA LETTER AI\0"
  "GRANTHA LETTER AU\0"
  "GRANTHA LETTER BA\0"
  "GRANTHA LETTER BHA\0"
  "GRANTHA LETTER CA\0"
  "GRANTHA LETTER CHA\0"
  "GRANTHA LETTER DA\0"
  "GRANTHA LETTER DDA\0"
  "GRANTHA LETTER DDHA\0"
  "GRANTHA LETTER DHA\0"
  "GRANTHA LETTER EE\0"
  "GRANTHA LETTER GA\0"
  "GRANTHA LETTER GHA\0"
  "GRANTHA LETTER HA\0"
  "GRANTHA LETTER I\0"
  "GRANTHA LETTER II\0"
  "GRANTHA LETTER JA\0"
  "GRANTHA LETTER JHA\0"
  "GRANTHA LETTER KA\0"
  "GRANTHA LETTER KHA\0"
  "GRANTHA LETTER LA\0"
  "GRANTHA LETTER LLA\0"
  "GRANTHA LETTER MA\0"
  "GRANTHA LETTER NA\0"
  "GRANTHA LETTER NGA\0"
  "GRANTHA LETTER NNA\0"
  "GRANTHA LETTER NYA\0"
  "GRANTHA LETTER OO\0"
  "GRANTHA LETTER PA\0"
  "GRANTHA LETTER PHA\0"
  "GRANTHA LETTER RA\0"
  "GRANTHA LETTER SA\0"
  "GRANTHA LETTER SHA\0"
  "GRANTHA LETTER SSA\0"
  "GRANTHA LETTER TA\0"
  "GRANTHA LETTER THA\0"
  "GRANTHA LETTER TTA\0"
  "GRANTHA LETTER TTHA\0"
  "GRANTHA LETTER U\0"
  "GRANTHA LETTER UU\0"
  "GRANTHA LETTER VA\0"
  "GRANTHA LETTER VEDIC ANUSVARA\0"
  "GRANTHA LETTER VEDIC DOUBLE ANUSVARA\0"
  "GRANTHA LETTER VOCALIC L\0"
  "GRANTHA LETTER VOCALIC LL\0"
  "GRANTHA LETTER VOCALIC R\0"
  "GRANTHA LETTER VOCALIC RR\0"
  "GRANTHA LETTER YA\0"
  "GRANTHA OM\0"
  "GRANTHA SIGN ANUSVARA\0"
  "GRANTHA SIGN AVAGRAHA\0"
  "GRANTHA SIGN CANDRABINDU\0"
  "GRANTHA SIGN COMBINING ANUSVARA ABOVE\0"
  "GRANTHA SIGN NUKTA\0"
  "GRANTHA SIGN PLUTA\0"
  "GRANTHA SIGN VIRAMA\0"
  "GRANTHA SIGN VISARGA\0"
  "GRANTHA VOWEL SIGN AA\0"
  "GRANTHA VOWEL SIGN AI\0"
  "GRANTHA VOWEL SIGN AU\0"
  "GRANTHA VOWEL SIGN EE\0"
  "GRANTHA VOWEL SIGN I\0"
  "GRANTHA VOWEL SIGN II\0"
  "GRANTHA VOWEL SIGN OO\0"
  "GRANTHA VOWEL SIGN U\0"
  "GRANTHA VOWEL SIGN UU\0"
  "GRANTHA VOWEL SIGN VOCALIC L\0"
  "GRANTHA VOWEL SIGN VOCALIC LL\0"
  "GRANTHA VOWEL SIGN VOCALIC R\0"
  "GRANTHA VOWEL SIGN VOCALIC RR\0"
  "GRAPES\0"
  "GRAVE ACCENT\0"
  "GREATER-THAN ABOVE DOUBLE-LINE EQUAL ABOVE LESS-THAN\0"
  "GREATER-THAN ABOVE LESS-THAN ABOVE DOUBLE-LINE EQUAL\0"
  "GREATER-THAN ABOVE RIGHTWARDS ARROW\0"
  "GREATER-THAN ABOVE SIMILAR ABOVE LESS-THAN\0"
  "GREATER-THAN ABOVE SIMILAR OR EQUAL\0"
  "GREATER-THAN ABOVE SLANTED EQUAL ABOVE LESS-THAN ABOVE SLANTED EQUAL\0"
  "GREATER-THAN AND NOT APPROXIMATE\0"
  "GREATER-THAN AND SINGLE-LINE NOT EQUAL TO\0"
  "GREATER-THAN BESIDE LESS-THAN\0"
  "GREATER-THAN BUT NOT EQUAL TO\0"
  "GREATER-THAN BUT NOT EQUIVALENT TO\0"
  "GREATER-THAN CLOSED BY CURVE\0"
  "GREATER-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL\0"
  "GREATER-THAN EQUAL TO OR LESS-THAN\0"
  "GREATER-THAN OR APPROXIMATE\0"
  "GREATER-THAN OR EQUAL TO\0"
  "GREATER-THAN OR EQUIVALENT TO\0"
  "GREATER-THAN OR LESS-THAN\0"
  "GREATER-THAN OR SLANTED EQUAL TO\0"
  "GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE\0"
  "GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE LEFT\0"
  "GREATER-THAN OR SLANTED EQUAL TO WITH DOT INSIDE\0"
  "GREATER-THAN OVER EQUAL TO\0"
  "GREATER-THAN OVERLAPPING LESS-THAN\0"
  "GREATER-THAN SIGN\0"
  "GREATER-THAN WITH CIRCLE INSIDE\0"
  "GREATER-THAN WITH DOT\0"
  "GREATER-THAN WITH QUESTION MARK ABOVE\0"
  "GREEK ACROPHONIC ATTIC FIFTY\0"
  "GREEK ACROPHONIC ATTIC FIFTY STATERS\0"
  "GREEK ACROPHONIC ATTIC FIFTY TALENTS\0"
  "GREEK ACROPHONIC ATTIC FIFTY THOUSAND\0"
  "GREEK ACROPHONIC ATTIC FIFTY THOUSAND STATERS\0"
  "GREEK ACROPHONIC ATTIC FIVE\0"
  "GREEK ACROPHONIC ATTIC FIVE HUNDRED\0"
  "GREEK ACROPHONIC ATTIC FIVE HUNDRED STATERS\0"
  "GREEK ACROPHONIC ATTIC FIVE HUNDRED TALENTS\0"
  "GREEK ACROPHONIC ATTIC FIVE STATERS\0"
  "GREEK ACROPHONIC ATTIC FIVE TALENTS\0"
  "GREEK ACROPHONIC ATTIC FIVE THOUSAND\0"
  "GREEK ACROPHONIC ATTIC FIVE THOUSAND TALENTS\0"
  "GREEK ACROPHONIC ATTIC ONE DRACHMA\0"
  "GREEK ACROPHONIC ATTIC ONE HALF\0"
  "GREEK ACROPHONIC ATTIC ONE HUNDRED STATERS\0"
  "GREEK ACROPHONIC ATTIC ONE HUNDRED TALENTS\0"
  "GREEK ACROPHONIC ATTIC ONE QUARTER\0"
  "GREEK ACROPHONIC ATTIC ONE THOUSAND STATERS\0"
  "GREEK ACROPHONIC ATTIC ONE THOUSAND TALENTS\0"
  "GREEK ACROPHONIC ATTIC TEN MNAS\0"
  "GREEK ACROPHONIC ATTIC TEN STATERS\0"
  "GREEK ACROPHONIC ATTIC TEN TALENTS\0"
  "GREEK ACROPHONIC ATTIC TEN THOUSAND STATERS\0"
  "GREEK ACROPHONIC CARYSTIAN FIVE HUNDRED\0"
  "GREEK ACROPHONIC CYRENAIC TWO DRACHMAS\0"
  "GREEK ACROPHONIC DELPHIC FIVE MNAS\0"
  "GREEK ACROPHONIC EPIDAUREAN FIVE HUNDRED\0"
  "GREEK ACROPHONIC EPIDAUREAN TWO\0"
  "GREEK ACROPHONIC EPIDAUREAN TWO DRACHMAS\0"
  "GREEK ACROPHONIC HERAEUM ONE PLETHRON\0"
  "GREEK ACROPHONIC HERMIONIAN FIFTY\0"
  "GREEK ACROPHONIC HERMIONIAN ONE\0"
  "GREEK ACROPHONIC HERMIONIAN TEN\0"
  "GREEK ACROPHONIC MESSENIAN TEN\0"
  "GREEK ACROPHONIC NAXIAN FIVE HUNDRED\0"
  "GREEK ACROPHONIC STRATIAN FIFTY MNAS\0"
  "GREEK ACROPHONIC THESPIAN FIFTY\0"
  "GREEK ACROPHONIC THESPIAN FIVE HUNDRED\0"
  "GREEK ACROPHONIC THESPIAN FIVE THOUSAND\0"
  "GREEK ACROPHONIC THESPIAN ONE\0"
  "GREEK ACROPHONIC THESPIAN ONE HUNDRED\0"
  "GREEK ACROPHONIC THESPIAN ONE THOUSAND\0"
  "GREEK ACROPHONIC THESPIAN TEN\0"
  "GREEK ACROPHONIC THESPIAN THIRTY\0"
  "GREEK ACROPHONIC THESPIAN THREE HUNDRED\0"
  "GREEK ACROPHONIC THESPIAN TWO\0"
  "GREEK ACROPHONIC TROEZENIAN FIFTY\0"
  "GREEK ACROPHONIC TROEZENIAN FIFTY ALTERNATE FORM\0"
  "GREEK ACROPHONIC TROEZENIAN FIVE\0"
  "GREEK ACROPHONIC TROEZENIAN FIVE HUNDRED\0"
  "GREEK ACROPHONIC TROEZENIAN TEN\0"
  "GREEK ACROPHONIC TROEZENIAN TEN ALTERNATE FORM\0"
  "GREEK ANO TELEIA\0"
  "GREEK AROURA SIGN\0"
  "GREEK ARTABE SIGN\0"
  "GREEK BETA SYMBOL\0"
  "GREEK CAPITAL DOTTED LUNATE SIGMA SYMBOL\0"
  "GREEK CAPITAL KAI SYMBOL\0"
  "GREEK CAPITAL LETTER ALPHA\0"
  "GREEK CAPITAL LETTER ALPHA WITH DASIA\0"
  "GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA\0"
  "GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH DASIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA\0"
  "GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH MACRON\0"
  "GREEK CAPITAL LETTER ALPHA WITH OXIA\0"
  "GREEK CAPITAL LETTER ALPHA WITH PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH PSILI\0"
  "GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA\0"
  "GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH PSILI AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA\0"
  "GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ALPHA WITH TONOS\0"
  "GREEK CAPITAL LETTER ALPHA WITH VARIA\0"
  "GREEK CAPITAL LETTER ALPHA WITH VRACHY\0"
  "GREEK CAPITAL LETTER ARCHAIC SAMPI\0"
  "GREEK CAPITAL LETTER BETA\0"
  "GREEK CAPITAL LETTER CHI\0"
  "GREEK CAPITAL LETTER DELTA\0"
  "GREEK CAPITAL LETTER EPSILON\0"
  "GREEK CAPITAL LETTER EPSILON WITH DASIA\0"
  "GREEK CAPITAL LETTER EPSILON WITH DASIA AND OXIA\0"
  "GREEK CAPITAL LETTER EPSILON WITH DASIA AND VARIA\0"
  "GREEK CAPITAL LETTER EPSILON WITH OXIA\0"
  "GREEK CAPITAL LETTER EPSILON WITH PSILI\0"
  "GREEK CAPITAL LETTER EPSILON WITH PSILI AND OXIA\0"
  "GREEK CAPITAL LETTER EPSILON WITH PSILI AND VARIA\0"
  "GREEK CAPITAL LETTER EPSILON WITH TONOS\0"
  "GREEK CAPITAL LETTER EPSILON WITH VARIA\0"
  "GREEK CAPITAL LETTER ETA\0"
  "GREEK CAPITAL LETTER ETA WITH DASIA\0"
  "GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA\0"
  "GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI\0"
  "GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ETA WITH DASIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA\0"
  "GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ETA WITH OXIA\0"
  "GREEK CAPITAL LETTER ETA WITH PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ETA WITH PSILI\0"
  "GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA\0"
  "GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI\0"
  "GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ETA WITH PSILI AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA\0"
  "GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER ETA WITH TONOS\0"
  "GREEK CAPITAL LETTER ETA WITH VARIA\0"
  "GREEK CAPITAL LETTER GAMMA\0"
  "GREEK CAPITAL LETTER HETA\0"
  "GREEK CAPITAL LETTER IOTA\0"
  "GREEK CAPITAL LETTER IOTA WITH DASIA\0"
  "GREEK CAPITAL LETTER IOTA WITH DASIA AND OXIA\0"
  "GREEK CAPITAL LETTER IOTA WITH DASIA AND PERISPOMENI\0"
  "GREEK CAPITAL LETTER IOTA WITH DASIA AND VARIA\0"
  "GREEK CAPITAL LETTER IOTA WITH DIALYTIKA\0"
  "GREEK CAPITAL LETTER IOTA WITH MACRON\0"
  "GREEK CAPITAL LETTER IOTA WITH OXIA\0"
  "GREEK CAPITAL LETTER IOTA WITH PSILI\0"
  "GREEK CAPITAL LETTER IOTA WITH PSILI AND OXIA\0"
  "GREEK CAPITAL LETTER IOTA WITH PSILI AND PERISPOMENI\0"
  "GREEK CAPITAL LETTER IOTA WITH PSILI AND VARIA\0"
  "GREEK CAPITAL LETTER IOTA WITH TONOS\0"
  "GREEK CAPITAL LETTER IOTA WITH VARIA\0"
  "GREEK CAPITAL LETTER IOTA WITH VRACHY\0"
  "GREEK CAPITAL LETTER KAPPA\0"
  "GREEK CAPITAL LETTER LAMDA\0"
  "GREEK CAPITAL LETTER MU\0"
  "GREEK CAPITAL LETTER NU\0"
  "GREEK CAPITAL LETTER OMEGA\0"
  "GREEK CAPITAL LETTER OMEGA WITH DASIA\0"
  "GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA\0"
  "GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH DASIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA\0"
  "GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH OXIA\0"
  "GREEK CAPITAL LETTER OMEGA WITH PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH PSILI\0"
  "GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA\0"
  "GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH PSILI AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA\0"
  "GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA AND PROSGEGRAMMENI\0"
  "GREEK CAPITAL LETTER OMEGA WITH TONOS\0"
  "GREEK CAPITAL LETTER OMEGA WITH VARIA\0"
  "GREEK CAPITAL LETTER OMICRON\0"
  "GREEK CAPITAL LETTER OMICRON WITH DASIA\0"
  "GREEK CAPITAL LETTER OMICRON WITH DASIA AND OXIA\0"
  "GREEK CAPITAL LETTER OMICRON WITH DASIA AND VARIA\0"
  "GREEK CAPITAL LETTER OMICRON WITH OXIA\0"
  "GREEK CAPITAL LETTER OMICRON WITH PSILI\0"
  "GREEK CAPITAL LETTER OMICRON WITH PSILI AND OXIA\0"
  "GREEK CAPITAL LETTER OMICRON WITH PSILI AND VARIA\0"
  "GREEK CAPITAL LETTER OMICRON WITH TONOS\0"
  "GREEK CAPITAL LETTER OMICRON WITH VARIA\0"
  "GREEK CAPITAL LETTER PAMPHYLIAN DIGAMMA\0"
  "GREEK CAPITAL LETTER PHI\0"
  "GREEK CAPITAL LETTER PI\0"
  "GREEK CAPITAL LETTER PSI\0"
  "GREEK CAPITAL LETTER RHO\0"
  "GREEK CAPITAL LETTER RHO WITH DASIA\0"
  "GREEK CAPITAL LETTER SAN\0"
  "GREEK CAPITAL LETTER SHO\0"
  "GREEK CAPITAL LETTER SIGMA\0"
  "GREEK CAPITAL LETTER TAU\0"
  "GREEK CAPITAL LETTER THETA\0"
  "GREEK CAPITAL LETTER UPSILON\0"
  "GREEK CAPITAL LETTER UPSILON WITH DASIA\0"
  "GREEK CAPITAL LETTER UPSILON WITH DASIA AND OXIA\0"
  "GREEK CAPITAL LETTER UPSILON WITH DASIA AND PERISPOMENI\0"
  "GREEK CAPITAL LETTER UPSILON WITH DASIA AND VARIA\0"
  "GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA\0"
  "GREEK CAPITAL LETTER UPSILON WITH MACRON\0"
  "GREEK CAPITAL LETTER UPSILON WITH OXIA\0"
  "GREEK CAPITAL LETTER UPSILON WITH TONOS\0"
  "GREEK CAPITAL LETTER UPSILON WITH VARIA\0"
  "GREEK CAPITAL LETTER UPSILON WITH VRACHY\0"
  "GREEK CAPITAL LETTER XI\0"
  "GREEK CAPITAL LETTER YOT\0"
  "GREEK CAPITAL LETTER ZETA\0"
  "GREEK CAPITAL LUNATE SIGMA SYMBOL\0"
  "GREEK CAPITAL REVERSED DOTTED LUNATE SIGMA SYMBOL\0"
  "GREEK CAPITAL REVERSED LUNATE SIGMA SYMBOL\0"
  "GREEK CAPITAL THETA SYMBOL\0"
  "GREEK DASIA\0"
  "GREEK DASIA AND OXIA\0"
  "GREEK DASIA AND PERISPOMENI\0"
  "GREEK DASIA AND VARIA\0"
  "GREEK DIALYTIKA AND OXIA\0"
  "GREEK DIALYTIKA AND PERISPOMENI\0"
  "GREEK DIALYTIKA AND VARIA\0"
  "GREEK DIALYTIKA TONOS\0"
  "GREEK DRACHMA SIGN\0"
  "GREEK FIVE OBOLS SIGN\0"
  "GREEK FOUR OBOLS SIGN\0"
  "GREEK GRAMMA SIGN\0"
  "GREEK INDICTION SIGN\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-1\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-11\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-12\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-13\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-14\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-17\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-18\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-19\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-2\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-23\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-24\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-25\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-26\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-27\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-29\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-30\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-32\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-36\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-37\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-38\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-39\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-4\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-40\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-42\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-43\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-45\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-47\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-48\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-49\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-5\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-50\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-51\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-52\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-53\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-54\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-7\0"
  "GREEK INSTRUMENTAL NOTATION SYMBOL-8\0"
  "GREEK KAI SYMBOL\0"
  "GREEK KAPPA SYMBOL\0"
  "GREEK KORONIS\0"
  "GREEK KYATHOS BASE SIGN\0"
  "GREEK LETTER ARCHAIC KOPPA\0"
  "GREEK LETTER DIGAMMA\0"
  "GREEK LETTER KOPPA\0"
  "GREEK LETTER SAMPI\0"
  "GREEK LETTER SMALL CAPITAL GAMMA\0"
  "GREEK LETTER SMALL CAPITAL LAMDA\0"
  "GREEK LETTER SMALL CAPITAL OMEGA\0"
  "GREEK LETTER SMALL CAPITAL PI\0"
  "GREEK LETTER SMALL CAPITAL PSI\0"
  "GREEK LETTER SMALL CAPITAL RHO\0"
  "GREEK LETTER STIGMA\0"
  "GREEK LETTER YOT\0"
  "GREEK LITRA SIGN\0"
  "GREEK LOWER NUMERAL SIGN\0"
  "GREEK LUNATE EPSILON SYMBOL\0"
  "GREEK LUNATE SIGMA SYMBOL\0"
  "GREEK METRETES SIGN\0"
  "GREEK MUSICAL LEIMMA\0"
  "GREEK NUMERAL SIGN\0"
  "GREEK OBOL SIGN\0"
  "GREEK ONE HALF SIGN\0"
  "GREEK ONE HALF SIGN ALTERNATE FORM\0"
  "GREEK ONE QUARTER SIGN\0"
  "GREEK OUNKIA SIGN\0"
  "GREEK OXIA\0"
  "GREEK PERISPOMENI\0"
  "GREEK PHI SYMBOL\0"
  "GREEK PI SYMBOL\0"
  "GREEK PROSGEGRAMMENI\0"
  "GREEK PSILI\0"
  "GREEK PSILI AND OXIA\0"
  "GREEK PSILI AND PERISPOMENI\0"
  "GREEK PSILI AND VARIA\0"
  "GREEK QUESTION MARK\0"
  "GREEK REVERSED LUNATE EPSILON SYMBOL\0"
  "GREEK RHO SYMBOL\0"
  "GREEK RHO WITH STROKE SYMBOL\0"
  "GREEK SINUSOID SIGN\0"
  "GREEK SMALL DOTTED LUNATE SIGMA SYMBOL\0"
  "GREEK SMALL LETTER ALPHA\0"
  "GREEK SMALL LETTER ALPHA WITH DASIA\0"
  "GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA\0"
  "GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI\0"
  "GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA\0"
  "GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH DASIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH MACRON\0"
  "GREEK SMALL LETTER ALPHA WITH OXIA\0"
  "GREEK SMALL LETTER ALPHA WITH OXIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH PERISPOMENI\0"
  "GREEK SMALL LETTER ALPHA WITH PERISPOMENI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH PSILI\0"
  "GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA\0"
  "GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI\0"
  "GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA\0"
  "GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH PSILI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH TONOS\0"
  "GREEK SMALL LETTER ALPHA WITH VARIA\0"
  "GREEK SMALL LETTER ALPHA WITH VARIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ALPHA WITH VRACHY\0"
  "GREEK SMALL LETTER ALPHA WITH YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ARCHAIC KOPPA\0"
  "GREEK SMALL LETTER ARCHAIC SAMPI\0"
  "GREEK SMALL LETTER BETA\0"
  "GREEK SMALL LETTER CHI\0"
  "GREEK SMALL LETTER DELTA\0"
  "GREEK SMALL LETTER DIGAMMA\0"
  "GREEK SMALL LETTER EPSILON\0"
  "GREEK SMALL LETTER EPSILON WITH DASIA\0"
  "GREEK SMALL LETTER EPSILON WITH DASIA AND OXIA\0"
  "GREEK SMALL LETTER EPSILON WITH DASIA AND VARIA\0"
  "GREEK SMALL LETTER EPSILON WITH OXIA\0"
  "GREEK SMALL LETTER EPSILON WITH PSILI\0"
  "GREEK SMALL LETTER EPSILON WITH PSILI AND OXIA\0"
  "GREEK SMALL LETTER EPSILON WITH PSILI AND VARIA\0"
  "GREEK SMALL LETTER EPSILON WITH TONOS\0"
  "GREEK SMALL LETTER EPSILON WITH VARIA\0"
  "GREEK SMALL LETTER ETA\0"
  "GREEK SMALL LETTER ETA WITH DASIA\0"
  "GREEK SMALL LETTER ETA WITH DASIA AND OXIA\0"
  "GREEK SMALL LETTER ETA WITH DASIA AND OXIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI\0"
  "GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH DASIA AND VARIA\0"
  "GREEK SMALL LETTER ETA WITH DASIA AND VARIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH DASIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH OXIA\0"
  "GREEK SMALL LETTER ETA WITH OXIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH PERISPOMENI\0"
  "GREEK SMALL LETTER ETA WITH PERISPOMENI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH PSILI\0"
  "GREEK SMALL LETTER ETA WITH PSILI AND OXIA\0"
  "GREEK SMALL LETTER ETA WITH PSILI AND OXIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI\0"
  "GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH PSILI AND VARIA\0"
  "GREEK SMALL LETTER ETA WITH PSILI AND VARIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH PSILI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH TONOS\0"
  "GREEK SMALL LETTER ETA WITH VARIA\0"
  "GREEK SMALL LETTER ETA WITH VARIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER ETA WITH YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER FINAL SIGMA\0"
  "GREEK SMALL LETTER GAMMA\0"
  "GREEK SMALL LETTER HETA\0"
  "GREEK SMALL LETTER IOTA\0"
  "GREEK SMALL LETTER IOTA WITH DASIA\0"
  "GREEK SMALL LETTER IOTA WITH DASIA AND OXIA\0"
  "GREEK SMALL LETTER IOTA WITH DASIA AND PERISPOMENI\0"
  "GREEK SMALL LETTER IOTA WITH DASIA AND VARIA\0"
  "GREEK SMALL LETTER IOTA WITH DIALYTIKA\0"
  "GREEK SMALL LETTER IOTA WITH DIALYTIKA AND OXIA\0"
  "GREEK SMALL LETTER IOTA WITH DIALYTIKA AND PERISPOMENI\0"
  "GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS\0"
  "GREEK SMALL LETTER IOTA WITH DIALYTIKA AND VARIA\0"
  "GREEK SMALL LETTER IOTA WITH MACRON\0"
  "GREEK SMALL LETTER IOTA WITH OXIA\0"
  "GREEK SMALL LETTER IOTA WITH PERISPOMENI\0"
  "GREEK SMALL LETTER IOTA WITH PSILI\0"
  "GREEK SMALL LETTER IOTA WITH PSILI AND OXIA\0"
  "GREEK SMALL LETTER IOTA WITH PSILI AND PERISPOMENI\0"
  "GREEK SMALL LETTER IOTA WITH PSILI AND VARIA\0"
  "GREEK SMALL LETTER IOTA WITH TONOS\0"
  "GREEK SMALL LETTER IOTA WITH VARIA\0"
  "GREEK SMALL LETTER IOTA WITH VRACHY\0"
  "GREEK SMALL LETTER KAPPA\0"
  "GREEK SMALL LETTER KOPPA\0"
  "GREEK SMALL LETTER LAMDA\0"
  "GREEK SMALL LETTER MU\0"
  "GREEK SMALL LETTER NU\0"
  "GREEK SMALL LETTER OMEGA\0"
  "GREEK SMALL LETTER OMEGA WITH DASIA\0"
  "GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA\0"
  "GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI\0"
  "GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA\0"
  "GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH DASIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH OXIA\0"
  "GREEK SMALL LETTER OMEGA WITH OXIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH PERISPOMENI\0"
  "GREEK SMALL LETTER OMEGA WITH PERISPOMENI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH PSILI\0"
  "GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA\0"
  "GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI\0"
  "GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA\0"
  "GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH PSILI AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH TONOS\0"
  "GREEK SMALL LETTER OMEGA WITH VARIA\0"
  "GREEK SMALL LETTER OMEGA WITH VARIA AND YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMEGA WITH YPOGEGRAMMENI\0"
  "GREEK SMALL LETTER OMICRON\0"
  "GREEK SMALL LETTER OMICRON WITH DASIA\0"
  "GREEK SMALL LETTER OMICRON WITH DASIA AND OXIA\0"
  "GREEK SMALL LETTER OMICRON WITH DASIA AND VARIA\0"
  "GREEK SMALL LETTER OMICRON WITH OXIA\0"
  "GREEK SMALL LETTER OMICRON WITH PSILI\0"
  "GREEK SMALL LETTER OMICRON WITH PSILI AND OXIA\0"
  "GREEK SMALL LETTER OMICRON WITH PSILI AND VARIA\0"
  "GREEK SMALL LETTER OMICRON WITH TONOS\0"
  "GREEK SMALL LETTER OMICRON WITH VARIA\0"
  "GREEK SMALL LETTER PAMPHYLIAN DIGAMMA\0"
  "GREEK SMALL LETTER PHI\0"
  "GREEK SMALL LETTER PI\0"
  "GREEK SMALL LETTER PSI\0"
  "GREEK SMALL LETTER RHO\0"
  "GREEK SMALL LETTER RHO WITH DASIA\0"
  "GREEK SMALL LETTER RHO WITH PSILI\0"
  "GREEK SMALL LETTER SAMPI\0"
  "GREEK SMALL LETTER SAN\0"
  "GREEK SMALL LETTER SHO\0"
  "GREEK SMALL LETTER SIGMA\0"
  "GREEK SMALL LETTER STIGMA\0"
  "GREEK SMALL LETTER TAU\0"
  "GREEK SMALL LETTER THETA\0"
  "GREEK SMALL LETTER UPSILON\0"
  "GREEK SMALL LETTER UPSILON WITH DASIA\0"
  "GREEK SMALL LETTER UPSILON WITH DASIA AND OXIA\0"
  "GREEK SMALL LETTER UPSILON WITH DASIA AND PERISPOMENI\0"
  "GREEK SMALL LETTER UPSILON WITH DASIA AND VARIA\0"
  "GREEK SMALL LETTER UPSILON WITH DIALYTIKA\0"
  "GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND OXIA\0"
  "GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND PERISPOMENI\0"
  "GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS\0"
  "GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND VARIA\0"
  "GREEK SMALL LETTER UPSILON WITH MACRON\0"
  "GREEK SMALL LETTER UPSILON WITH OXIA\0"
  "GREEK SMALL LETTER UPSILON WITH PERISPOMENI\0"
  "GREEK SMALL LETTER UPSILON WITH PSILI\0"
  "GREEK SMALL LETTER UPSILON WITH PSILI AND OXIA\0"
  "GREEK SMALL LETTER UPSILON WITH PSILI AND PERISPOMENI\0"
  "GREEK SMALL LETTER UPSILON WITH PSILI AND VARIA\0"
  "GREEK SMALL LETTER UPSILON WITH TONOS\0"
  "GREEK SMALL LETTER UPSILON WITH VARIA\0"
  "GREEK SMALL LETTER UPSILON WITH VRACHY\0"
  "GREEK SMALL LETTER XI\0"
  "GREEK SMALL LETTER ZETA\0"
  "GREEK SMALL REVERSED DOTTED LUNATE SIGMA SYMBOL\0"
  "GREEK SMALL REVERSED LUNATE SIGMA SYMBOL\0"
  "GREEK SUBSCRIPT SMALL LETTER BETA\0"
  "GREEK SUBSCRIPT SMALL LETTER CHI\0"
  "GREEK SUBSCRIPT SMALL LETTER GAMMA\0"
  "GREEK SUBSCRIPT SMALL LETTER PHI\0"
  "GREEK SUBSCRIPT SMALL LETTER RHO\0"
  "GREEK SYMBOL TAU RHO\0"
  "GREEK TALENT SIGN\0"
  "GREEK THETA SYMBOL\0"
  "GREEK THREE OBOLS SIGN\0"
  "GREEK THREE QUARTERS SIGN\0"
  "GREEK TONOS\0"
  "GREEK TRYBLION BASE SIGN\0"
  "GREEK TWO OBOLS SIGN\0"
  "GREEK TWO THIRDS SIGN\0"
  "GREEK UPSILON WITH ACUTE AND HOOK SYMBOL\0"
  "GREEK UPSILON WITH DIAERESIS AND HOOK SYMBOL\0"
  "GREEK UPSILON WITH HOOK SYMBOL\0"
  "GREEK VARIA\0"
  "GREEK VOCAL NOTATION SYMBOL-1\0"
  "GREEK VOCAL NOTATION SYMBOL-10\0"
  "GREEK VOCAL NOTATION SYMBOL-11\0"
  "GREEK VOCAL NOTATION SYMBOL-12\0"
  "GREEK VOCAL NOTATION SYMBOL-13\0"
  "GREEK VOCAL NOTATION SYMBOL-14\0"
  "GREEK VOCAL NOTATION SYMBOL-15\0"
  "GREEK VOCAL NOTATION SYMBOL-16\0"
  "GREEK VOCAL NOTATION SYMBOL-17\0"
  "GREEK VOCAL NOTATION SYMBOL-18\0"
  "GREEK VOCAL NOTATION SYMBOL-19\0"
  "GREEK VOCAL NOTATION SYMBOL-2\0"
  "GREEK VOCAL NOTATION SYMBOL-20\0"
  "GREEK VOCAL NOTATION SYMBOL-21\0"
  "GREEK VOCAL NOTATION SYMBOL-22\0"
  "GREEK VOCAL NOTATION SYMBOL-23\0"
  "GREEK VOCAL NOTATION SYMBOL-24\0"
  "GREEK VOCAL NOTATION SYMBOL-3\0"
  "GREEK VOCAL NOTATION SYMBOL-4\0"
  "GREEK VOCAL NOTATION SYMBOL-5\0"
  "GREEK VOCAL NOTATION SYMBOL-50\0"
  "GREEK VOCAL NOTATION SYMBOL-51\0"
  "GREEK VOCAL NOTATION SYMBOL-52\0"
  "GREEK VOCAL NOTATION SYMBOL-53\0"
  "GREEK VOCAL NOTATION SYMBOL-54\0"
  "GREEK VOCAL NOTATION SYMBOL-6\0"
  "GREEK VOCAL NOTATION SYMBOL-7\0"
  "GREEK VOCAL NOTATION SYMBOL-8\0"
  "GREEK VOCAL NOTATION SYMBOL-9\0"
  "GREEK XESTES SIGN\0"
  "GREEK YEAR SIGN\0"
  "GREEK YPOGEGRAMMENI\0"
  "GREEK ZERO SIGN\0"
  "GREEN APPLE\0"
  "GREEN BOOK\0"
  "GREEN HEART\0"
  "GREEN SALAD\0"
  "GREY HEART\0"
  "GRIMACING FACE\0"
  "GRINNING CAT FACE WITH SMILING EYES\0"
  "GRINNING FACE\0"
  "GRINNING FACE WITH ONE LARGE AND ONE SMALL EYE\0"
  "GRINNING FACE WITH SMILING EYES\0"
  "GRINNING FACE WITH STAR EYES\0"
  "GROUP MARK\0"
  "GROWING HEART\0"
  "GUARANI SIGN\0"
  "GUARDSMAN\0"
  "GUIDE DOG\0"
  "GUITAR\0"
  "GUJARATI ABBREVIATION SIGN\0"
  "GUJARATI DIGIT EIGHT\0"
  "GUJARATI DIGIT FIVE\0"
  "GUJARATI DIGIT FOUR\0"
  "GUJARATI DIGIT NINE\0"
  "GUJARATI DIGIT ONE\0"
  "GUJARATI DIGIT SEVEN\0"
  "GUJARATI DIGIT SIX\0"
  "GUJARATI DIGIT THREE\0"
  "GUJARATI DIGIT TWO\0"
  "GUJARATI DIGIT ZERO\0"
  "GUJARATI LETTER A\0"
  "GUJARATI LETTER AA\0"
  "GUJARATI LETTER AI\0"
  "GUJARATI LETTER AU\0"
  "GUJARATI LETTER BA\0"
  "GUJARATI LETTER BHA\0"
  "GUJARATI LETTER CA\0"
  "GUJARATI LETTER CHA\0"
  "GUJARATI LETTER DA\0"
  "GUJARATI LETTER DDA\0"
  "GUJARATI LETTER DDHA\0"
  "GUJARATI LETTER DHA\0"
  "GUJARATI LETTER E\0"
  "GUJARATI LETTER GA\0"
  "GUJARATI LETTER GHA\0"
  "GUJARATI LETTER HA\0"
  "GUJARATI LETTER I\0"
  "GUJARATI LETTER II\0"
  "GUJARATI LETTER JA\0"
  "GUJARATI LETTER JHA\0"
  "GUJARATI LETTER KA\0"
  "GUJARATI LETTER KHA\0"
  "GUJARATI LETTER LA\0"
  "GUJARATI LETTER LLA\0"
  "GUJARATI LETTER MA\0"
  "GUJARATI LETTER NA\0"
  "GUJARATI LETTER NGA\0"
  "GUJARATI LETTER NNA\0"
  "GUJARATI LETTER NYA\0"
  "GUJARATI LETTER O\0"
  "GUJARATI LETTER PA\0"
  "GUJARATI LETTER PHA\0"
  "GUJARATI LETTER RA\0"
  "GUJARATI LETTER SA\0"
  "GUJARATI LETTER SHA\0"
  "GUJARATI LETTER SSA\0"
  "GUJARATI LETTER TA\0"
  "GUJARATI LETTER THA\0"
  "GUJARATI LETTER TTA\0"
  "GUJARATI LETTER TTHA\0"
  "GUJARATI LETTER U\0"
  "GUJARATI LETTER UU\0"
  "GUJARATI LETTER VA\0"
  "GUJARATI LETTER VOCALIC L\0"
  "GUJARATI LETTER VOCALIC LL\0"
  "GUJARATI LETTER VOCALIC R\0"
  "GUJARATI LETTER VOCALIC RR\0"
  "GUJARATI LETTER YA\0"
  "GUJARATI LETTER ZHA\0"
  "GUJARATI OM\0"
  "GUJARATI RUPEE SIGN\0"
  "GUJARATI SIGN ANUSVARA\0"
  "GUJARATI SIGN AVAGRAHA\0"
  "GUJARATI SIGN CANDRABINDU\0"
  "GUJARATI SIGN CIRCLE NUKTA ABOVE\0"
  "GUJARATI SIGN MADDAH\0"
  "GUJARATI SIGN NUKTA\0"
  "GUJARATI SIGN SHADDA\0"
  "GUJARATI SIGN SUKUN\0"
  "GUJARATI SIGN THREE-DOT NUKTA ABOVE\0"
  "GUJARATI SIGN TWO-CIRCLE NUKTA ABOVE\0"
  "GUJARATI SIGN VIRAMA\0"
  "GUJARATI SIGN VISARGA\0"
  "GUJARATI VOWEL CANDRA E\0"
  "GUJARATI VOWEL CANDRA O\0"
  "GUJARATI VOWEL SIGN AA\0"
  "GUJARATI VOWEL SIGN AI\0"
  "GUJARATI VOWEL SIGN AU\0"
  "GUJARATI VOWEL SIGN CANDRA E\0"
  "GUJARATI VOWEL SIGN CANDRA O\0"
  "GUJARATI VOWEL SIGN E\0"
  "GUJARATI VOWEL SIGN I\0"
  "GUJARATI VOWEL SIGN II\0"
  "GUJARATI VOWEL SIGN O\0"
  "GUJARATI VOWEL SIGN U\0"
  "GUJARATI VOWEL SIGN UU\0"
  "GUJARATI VOWEL SIGN VOCALIC L\0"
  "GUJARATI VOWEL SIGN VOCALIC LL\0"
  "GUJARATI VOWEL SIGN VOCALIC R\0"
  "GUJARATI VOWEL SIGN VOCALIC RR\0"
  "GUNJALA GONDI DIGIT EIGHT\0"
  "GUNJALA GONDI DIGIT FIVE\0"
  "GUNJALA GONDI DIGIT FOUR\0"
  "GUNJALA GONDI DIGIT NINE\0"
  "GUNJALA GONDI DIGIT ONE\0"
  "GUNJALA GONDI DIGIT SEVEN\0"
  "GUNJALA GONDI DIGIT SIX\0"
  "GUNJALA GONDI DIGIT THREE\0"
  "GUNJALA GONDI DIGIT TWO\0"
  "GUNJALA GONDI DIGIT ZERO\0"
  "GUNJALA GONDI LETTER A\0"
  "GUNJALA GONDI LETTER AA\0"
  "GUNJALA GONDI LETTER AI\0"
  "GUNJALA GONDI LETTER AU\0"
  "GUNJALA GONDI LETTER BA\0"
  "GUNJALA GONDI LETTER BHA\0"
  "GUNJALA GONDI LETTER CA\0"
  "GUNJALA GONDI LETTER CHA\0"
  "GUNJALA GONDI LETTER DA\0"
  "GUNJALA GONDI LETTER DDA\0"
  "GUNJALA GONDI LETTER DDHA\0"
  "GUNJALA GONDI LETTER DHA\0"
  "GUNJALA GONDI LETTER EE\0"
  "GUNJALA GONDI LETTER GA\0"
  "GUNJALA GONDI LETTER GHA\0"
  "GUNJALA GONDI LETTER HA\0"
  "GUNJALA GONDI LETTER I\0"
  "GUNJALA GONDI LETTER II\0"
  "GUNJALA GONDI LETTER JA\0"
  "GUNJALA GONDI LETTER JHA\0"
  "GUNJALA GONDI LETTER KA\0"
  "GUNJALA GONDI LETTER KHA\0"
  "GUNJALA GONDI LETTER LA\0"
  "GUNJALA GONDI LETTER LLA\0"
  "GUNJALA GONDI LETTER MA\0"
  "GUNJALA GONDI LETTER NA\0"
  "GUNJALA GONDI LETTER NGA\0"
  "GUNJALA GONDI LETTER OO\0"
  "GUNJALA GONDI LETTER PA\0"
  "GUNJALA GONDI LETTER PHA\0"
  "GUNJALA GONDI LETTER RA\0"
  "GUNJALA GONDI LETTER SA\0"
  "GUNJALA GONDI LETTER TA\0"
  "GUNJALA GONDI LETTER THA\0"
  "GUNJALA GONDI LETTER TTA\0"
  "GUNJALA GONDI LETTER TTHA\0"
  "GUNJALA GONDI LETTER U\0"
  "GUNJALA GONDI LETTER UU\0"
  "GUNJALA GONDI LETTER VA\0"
  "GUNJALA GONDI LETTER YA\0"
  "GUNJALA GONDI OM\0"
  "GUNJALA GONDI SIGN ANUSVARA\0"
  "GUNJALA GONDI SIGN VISARGA\0"
  "GUNJALA GONDI VIRAMA\0"
  "GUNJALA GONDI VOWEL SIGN AA\0"
  "GUNJALA GONDI VOWEL SIGN AI\0"
  "GUNJALA GONDI VOWEL SIGN AU\0"
  "GUNJALA GONDI VOWEL SIGN EE\0"
  "GUNJALA GONDI VOWEL SIGN I\0"
  "GUNJALA GONDI VOWEL SIGN II\0"
  "GUNJALA GONDI VOWEL SIGN OO\0"
  "GUNJALA GONDI VOWEL SIGN U\0"
  "GUNJALA GONDI VOWEL SIGN UU\0"
  "GURMUKHI ABBREVIATION SIGN\0"
  "GURMUKHI ADDAK\0"
  "GURMUKHI DIGIT EIGHT\0"
  "GURMUKHI DIGIT FIVE\0"
  "GURMUKHI DIGIT FOUR\0"
  "GURMUKHI DIGIT NINE\0"
  "GURMUKHI DIGIT ONE\0"
  "GURMUKHI DIGIT SEVEN\0"
  "GURMUKHI DIGIT SIX\0"
  "GURMUKHI DIGIT THREE\0"
  "GURMUKHI DIGIT TWO\0"
  "GURMUKHI DIGIT ZERO\0"
  "GURMUKHI EK ONKAR\0"
  "GURMUKHI IRI\0"
  "GURMUKHI LETTER A\0"
  "GURMUKHI LETTER AA\0"
  "GURMUKHI LETTER AI\0"
  "GURMUKHI LETTER AU\0"
  "GURMUKHI LETTER BA\0"
  "GURMUKHI LETTER BHA\0"
  "GURMUKHI LETTER CA\0"
  "GURMUKHI LETTER CHA\0"
  "GURMUKHI LETTER DA\0"
  "GURMUKHI LETTER DDA\0"
  "GURMUKHI LETTER DDHA\0"
  "GURMUKHI LETTER DHA\0"
  "GURMUKHI LETTER EE\0"
  "GURMUKHI LETTER FA\0"
  "GURMUKHI LETTER GA\0"
  "GURMUKHI LETTER GHA\0"
  "GURMUKHI LETTER GHHA\0"
  "GURMUKHI LETTER HA\0"
  "GURMUKHI LETTER I\0"
  "GURMUKHI LETTER II\0"
  "GURMUKHI LETTER JA\0"
  "GURMUKHI LETTER JHA\0"
  "GURMUKHI LETTER KA\0"
  "GURMUKHI LETTER KHA\0"
  "GURMUKHI LETTER KHHA\0"
  "GURMUKHI LETTER LA\0"
  "GURMUKHI LETTER LLA\0"
  "GURMUKHI LETTER MA\0"
  "GURMUKHI LETTER NA\0"
  "GURMUKHI LETTER NGA\0"
  "GURMUKHI LETTER NNA\0"
  "GURMUKHI LETTER NYA\0"
  "GURMUKHI LETTER OO\0"
  "GURMUKHI LETTER PA\0"
  "GURMUKHI LETTER PHA\0"
  "GURMUKHI LETTER RA\0"
  "GURMUKHI LETTER RRA\0"
  "GURMUKHI LETTER SA\0"
  "GURMUKHI LETTER SHA\0"
  "GURMUKHI LETTER TA\0"
  "GURMUKHI LETTER THA\0"
  "GURMUKHI LETTER TTA\0"
  "GURMUKHI LETTER TTHA\0"
  "GURMUKHI LETTER U\0"
  "GURMUKHI LETTER UU\0"
  "GURMUKHI LETTER VA\0"
  "GURMUKHI LETTER YA\0"
  "GURMUKHI LETTER ZA\0"
  "GURMUKHI SIGN ADAK BINDI\0"
  "GURMUKHI SIGN BINDI\0"
  "GURMUKHI SIGN NUKTA\0"
  "GURMUKHI SIGN UDAAT\0"
  "GURMUKHI SIGN VIRAMA\0"
  "GURMUKHI SIGN VISARGA\0"
  "GURMUKHI SIGN YAKASH\0"
  "GURMUKHI TIPPI\0"
  "GURMUKHI URA\0"
  "GURMUKHI VOWEL SIGN AA\0"
  "GURMUKHI VOWEL SIGN AI\0"
  "GURMUKHI VOWEL SIGN AU\0"
  "GURMUKHI VOWEL SIGN EE\0"
  "GURMUKHI VOWEL SIGN I\0"
  "GURMUKHI VOWEL SIGN II\0"
  "GURMUKHI VOWEL SIGN OO\0"
  "GURMUKHI VOWEL SIGN U\0"
  "GURMUKHI VOWEL SIGN UU\0"
  "GURUNG KHEMA CONSONANT SIGN MEDIAL HA\0"
  "GURUNG KHEMA CONSONANT SIGN MEDIAL RA\0"
  "GURUNG KHEMA CONSONANT SIGN MEDIAL VA\0"
  "GURUNG KHEMA CONSONANT SIGN MEDIAL YA\0"
  "GURUNG KHEMA DIGIT EIGHT\0"
  "GURUNG KHEMA DIGIT FIVE\0"
  "GURUNG KHEMA DIGIT FOUR\0"
  "GURUNG KHEMA DIGIT NINE\0"
  "GURUNG KHEMA DIGIT ONE\0"
  "GURUNG KHEMA DIGIT SEVEN\0"
  "GURUNG KHEMA DIGIT SIX\0"
  "GURUNG KHEMA DIGIT THREE\0"
  "GURUNG KHEMA DIGIT TWO\0"
  "GURUNG KHEMA DIGIT ZERO\0"
  "GURUNG KHEMA LETTER A\0"
  "GURUNG KHEMA LETTER BA\0"
  "GURUNG KHEMA LETTER BHA\0"
  "GURUNG KHEMA LETTER CA\0"
  "GURUNG KHEMA LETTER CHA\0"
  "GURUNG KHEMA LETTER DA\0"
  "GURUNG KHEMA LETTER DDA\0"
  "GURUNG KHEMA LETTER DDHA\0"
  "GURUNG KHEMA LETTER DHA\0"
  "GURUNG KHEMA LETTER GA\0"
  "GURUNG KHEMA LETTER GHA\0"
  "GURUNG KHEMA LETTER HA\0"
  "GURUNG KHEMA LETTER JA\0"
  "GURUNG KHEMA LETTER JHA\0"
  "GURUNG KHEMA LETTER KA\0"
  "GURUNG KHEMA LETTER KHA\0"
  "GURUNG KHEMA LETTER LA\0"
  "GURUNG KHEMA LETTER MA\0"
  "GURUNG KHEMA LETTER NA\0"
  "GURUNG KHEMA LETTER NGA\0"
  "GURUNG KHEMA LETTER PA\0"
  "GURUNG KHEMA LETTER PHA\0"
  "GURUNG KHEMA LETTER RA\0"
  "GURUNG KHEMA LETTER SA\0"
  "GURUNG KHEMA LETTER TA\0"
  "GURUNG KHEMA LETTER THA\0"
  "GURUNG KHEMA LETTER TTA\0"
  "GURUNG KHEMA LETTER TTHA\0"
  "GURUNG KHEMA LETTER VA\0"
  "GURUNG KHEMA LETTER YA\0"
  "GURUNG KHEMA SIGN ANUSVARA\0"
  "GURUNG KHEMA SIGN THOLHOMA\0"
  "GURUNG KHEMA VOWEL LENGTH MARK\0"
  "GURUNG KHEMA VOWEL SIGN AA\0"
  "GURUNG KHEMA VOWEL SIGN AI\0"
  "GURUNG KHEMA VOWEL SIGN AU\0"
  "GURUNG KHEMA VOWEL SIGN E\0"
  "GURUNG KHEMA VOWEL SIGN EE\0"
  "GURUNG KHEMA VOWEL SIGN I\0"
  "GURUNG KHEMA VOWEL SIGN II\0"
  "GURUNG KHEMA VOWEL SIGN O\0"
  "GURUNG KHEMA VOWEL SIGN OO\0"
  "GURUNG KHEMA VOWEL SIGN U\0"
  "GURUNG KHEMA VOWEL SIGN UU\0"
  "HADES\0"
  "HAIR PICK\0"
  "HAIR SPACE\0"
  "HAIRCUT\0"
  "HALFWIDTH BLACK SQUARE\0"
  "HALFWIDTH DOWNWARDS ARROW\0"
  "HALFWIDTH FORMS LIGHT VERTICAL\0"
  "HALFWIDTH HANGUL FILLER\0"
  "HALFWIDTH HANGUL LETTER A\0"
  "HALFWIDTH HANGUL LETTER AE\0"
  "HALFWIDTH HANGUL LETTER CHIEUCH\0"
  "HALFWIDTH HANGUL LETTER CIEUC\0"
  "HALFWIDTH HANGUL LETTER E\0"
  "HALFWIDTH HANGUL LETTER EO\0"
  "HALFWIDTH HANGUL LETTER EU\0"
  "HALFWIDTH HANGUL LETTER HIEUH\0"
  "HALFWIDTH HANGUL LETTER I\0"
  "HALFWIDTH HANGUL LETTER IEUNG\0"
  "HALFWIDTH HANGUL LETTER KHIEUKH\0"
  "HALFWIDTH HANGUL LETTER KIYEOK\0"
  "HALFWIDTH HANGUL LETTER KIYEOK-SIOS\0"
  "HALFWIDTH HANGUL LETTER MIEUM\0"
  "HALFWIDTH HANGUL LETTER NIEUN\0"
  "HALFWIDTH HANGUL LETTER NIEUN-CIEUC\0"
  "HALFWIDTH HANGUL LETTER NIEUN-HIEUH\0"
  "HALFWIDTH HANGUL LETTER O\0"
  "HALFWIDTH HANGUL LETTER OE\0"
  "HALFWIDTH HANGUL LETTER PHIEUPH\0"
  "HALFWIDTH HANGUL LETTER PIEUP\0"
  "HALFWIDTH HANGUL LETTER PIEUP-SIOS\0"
  "HALFWIDTH HANGUL LETTER RIEUL\0"
  "HALFWIDTH HANGUL LETTER RIEUL-HIEUH\0"
  "HALFWIDTH HANGUL LETTER RIEUL-KIYEOK\0"
  "HALFWIDTH HANGUL LETTER RIEUL-MIEUM\0"
  "HALFWIDTH HANGUL LETTER RIEUL-PHIEUPH\0"
  "HALFWIDTH HANGUL LETTER RIEUL-PIEUP\0"
  "HALFWIDTH HANGUL LETTER RIEUL-SIOS\0"
  "HALFWIDTH HANGUL LETTER RIEUL-THIEUTH\0"
  "HALFWIDTH HANGUL LETTER SIOS\0"
  "HALFWIDTH HANGUL LETTER SSANGCIEUC\0"
  "HALFWIDTH HANGUL LETTER SSANGKIYEOK\0"
  "HALFWIDTH HANGUL LETTER SSANGPIEUP\0"
  "HALFWIDTH HANGUL LETTER SSANGSIOS\0"
  "HALFWIDTH HANGUL LETTER SSANGTIKEUT\0"
  "HALFWIDTH HANGUL LETTER THIEUTH\0"
  "HALFWIDTH HANGUL LETTER TIKEUT\0"
  "HALFWIDTH HANGUL LETTER U\0"
  "HALFWIDTH HANGUL LETTER WA\0"
  "HALFWIDTH HANGUL LETTER WAE\0"
  "HALFWIDTH HANGUL LETTER WE\0"
  "HALFWIDTH HANGUL LETTER WEO\0"
  "HALFWIDTH HANGUL LETTER WI\0"
  "HALFWIDTH HANGUL LETTER YA\0"
  "HALFWIDTH HANGUL LETTER YAE\0"
  "HALFWIDTH HANGUL LETTER YE\0"
  "HALFWIDTH HANGUL LETTER YEO\0"
  "HALFWIDTH HANGUL LETTER YI\0"
  "HALFWIDTH HANGUL LETTER YO\0"
  "HALFWIDTH HANGUL LETTER YU\0"
  "HALFWIDTH IDEOGRAPHIC COMMA\0"
  "HALFWIDTH IDEOGRAPHIC FULL STOP\0"
  "HALFWIDTH KATAKANA LETTER A\0"
  "HALFWIDTH KATAKANA LETTER E\0"
  "HALFWIDTH KATAKANA LETTER HA\0"
  "HALFWIDTH KATAKANA LETTER HE\0"
  "HALFWIDTH KATAKANA LETTER HI\0"
  "HALFWIDTH KATAKANA LETTER HO\0"
  "HALFWIDTH KATAKANA LETTER HU\0"
  "HALFWIDTH KATAKANA LETTER I\0"
  "HALFWIDTH KATAKANA LETTER KA\0"
  "HALFWIDTH KATAKANA LETTER KE\0"
  "HALFWIDTH KATAKANA LETTER KI\0"
  "HALFWIDTH KATAKANA LETTER KO\0"
  "HALFWIDTH KATAKANA LETTER KU\0"
  "HALFWIDTH KATAKANA LETTER MA\0"
  "HALFWIDTH KATAKANA LETTER ME\0"
  "HALFWIDTH KATAKANA LETTER MI\0"
  "HALFWIDTH KATAKANA LETTER MO\0"
  "HALFWIDTH KATAKANA LETTER MU\0"
  "HALFWIDTH KATAKANA LETTER N\0"
  "HALFWIDTH KATAKANA LETTER NA\0"
  "HALFWIDTH KATAKANA LETTER NE\0"
  "HALFWIDTH KATAKANA LETTER NI\0"
  "HALFWIDTH KATAKANA LETTER NO\0"
  "HALFWIDTH KATAKANA LETTER NU\0"
  "HALFWIDTH KATAKANA LETTER O\0"
  "HALFWIDTH KATAKANA LETTER RA\0"
  "HALFWIDTH KATAKANA LETTER RE\0"
  "HALFWIDTH KATAKANA LETTER RI\0"
  "HALFWIDTH KATAKANA LETTER RO\0"
  "HALFWIDTH KATAKANA LETTER RU\0"
  "HALFWIDTH KATAKANA LETTER SA\0"
  "HALFWIDTH KATAKANA LETTER SE\0"
  "HALFWIDTH KATAKANA LETTER SI\0"
  "HALFWIDTH KATAKANA LETTER SMALL A\0"
  "HALFWIDTH KATAKANA LETTER SMALL E\0"
  "HALFWIDTH KATAKANA LETTER SMALL I\0"
  "HALFWIDTH KATAKANA LETTER SMALL O\0"
  "HALFWIDTH KATAKANA LETTER SMALL TU\0"
  "HALFWIDTH KATAKANA LETTER SMALL U\0"
  "HALFWIDTH KATAKANA LETTER SMALL YA\0"
  "HALFWIDTH KATAKANA LETTER SMALL YO\0"
  "HALFWIDTH KATAKANA LETTER SMALL YU\0"
  "HALFWIDTH KATAKANA LETTER SO\0"
  "HALFWIDTH KATAKANA LETTER SU\0"
  "HALFWIDTH KATAKANA LETTER TA\0"
  "HALFWIDTH KATAKANA LETTER TE\0"
  "HALFWIDTH KATAKANA LETTER TI\0"
  "HALFWIDTH KATAKANA LETTER TO\0"
  "HALFWIDTH KATAKANA LETTER TU\0"
  "HALFWIDTH KATAKANA LETTER U\0"
  "HALFWIDTH KATAKANA LETTER WA\0"
  "HALFWIDTH KATAKANA LETTER WO\0"
  "HALFWIDTH KATAKANA LETTER YA\0"
  "HALFWIDTH KATAKANA LETTER YO\0"
  "HALFWIDTH KATAKANA LETTER YU\0"
  "HALFWIDTH KATAKANA MIDDLE DOT\0"
  "HALFWIDTH KATAKANA SEMI-VOICED SOUND MARK\0"
  "HALFWIDTH KATAKANA VOICED SOUND MARK\0"
  "HALFWIDTH KATAKANA-HIRAGANA PROLONGED SOUND MARK\0"
  "HALFWIDTH LEFT CORNER BRACKET\0"
  "HALFWIDTH LEFTWARDS ARROW\0"
  "HALFWIDTH RIGHT CORNER BRACKET\0"
  "HALFWIDTH RIGHTWARDS ARROW\0"
  "HALFWIDTH UPWARDS ARROW\0"
  "HALFWIDTH WHITE CIRCLE\0"
  "HAMBURGER\0"
  "HAMMER\0"
  "HAMMER AND PICK\0"
  "HAMMER AND SICKLE\0"
  "HAMMER AND WRENCH\0"
  "HAMSA\0"
  "HAMSTER FACE\0"
  "HAND WITH INDEX AND MIDDLE FINGERS CROSSED\0"
  "HAND WITH INDEX FINGER AND THUMB CROSSED\0"
  "HANDBAG\0"
  "HANDBALL\0"
  "HANDSHAKE\0"
  "HANGUL CHOSEONG CEONGCHIEUMCHIEUCH\0"
  "HANGUL CHOSEONG CEONGCHIEUMCIEUC\0"
  "HANGUL CHOSEONG CEONGCHIEUMSIOS\0"
  "HANGUL CHOSEONG CEONGCHIEUMSSANGCIEUC\0"
  "HANGUL CHOSEONG CEONGCHIEUMSSANGSIOS\0"
  "HANGUL CHOSEONG CHIEUCH\0"
  "HANGUL CHOSEONG CHIEUCH-HIEUH\0"
  "HANGUL CHOSEONG CHIEUCH-KHIEUKH\0"
  "HANGUL CHOSEONG CHITUEUMCHIEUCH\0"
  "HANGUL CHOSEONG CHITUEUMCIEUC\0"
  "HANGUL CHOSEONG CHITUEUMSIOS\0"
  "HANGUL CHOSEONG CHITUEUMSSANGCIEUC\0"
  "HANGUL CHOSEONG CHITUEUMSSANGSIOS\0"
  "HANGUL CHOSEONG CIEUC\0"
  "HANGUL CHOSEONG CIEUC-IEUNG\0"
  "HANGUL CHOSEONG FILLER\0"
  "HANGUL CHOSEONG HIEUH\0"
  "HANGUL CHOSEONG HIEUH-SIOS\0"
  "HANGUL CHOSEONG IEUNG\0"
  "HANGUL CHOSEONG IEUNG-CHIEUCH\0"
  "HANGUL CHOSEONG IEUNG-CIEUC\0"
  "HANGUL CHOSEONG IEUNG-HIEUH\0"
  "HANGUL CHOSEONG IEUNG-KIYEOK\0"
  "HANGUL CHOSEONG IEUNG-MIEUM\0"
  "HANGUL CHOSEONG IEUNG-PANSIOS\0"
  "HANGUL CHOSEONG IEUNG-PHIEUPH\0"
  "HANGUL CHOSEONG IEUNG-PIEUP\0"
  "HANGUL CHOSEONG IEUNG-RIEUL\0"
  "HANGUL CHOSEONG IEUNG-SIOS\0"
  "HANGUL CHOSEONG IEUNG-THIEUTH\0"
  "HANGUL CHOSEONG IEUNG-TIKEUT\0"
  "HANGUL CHOSEONG KAPYEOUNMIEUM\0"
  "HANGUL CHOSEONG KAPYEOUNPHIEUPH\0"
  "HANGUL CHOSEONG KAPYEOUNPIEUP\0"
  "HANGUL CHOSEONG KAPYEOUNRIEUL\0"
  "HANGUL CHOSEONG KAPYEOUNSSANGPIEUP\0"
  "HANGUL CHOSEONG KHIEUKH\0"
  "HANGUL CHOSEONG KIYEOK\0"
  "HANGUL CHOSEONG KIYEOK-TIKEUT\0"
  "HANGUL CHOSEONG MIEUM\0"
  "HANGUL CHOSEONG MIEUM-KIYEOK\0"
  "HANGUL CHOSEONG MIEUM-PIEUP\0"
  "HANGUL CHOSEONG MIEUM-SIOS\0"
  "HANGUL CHOSEONG MIEUM-TIKEUT\0"
  "HANGUL CHOSEONG NIEUN\0"
  "HANGUL CHOSEONG NIEUN-CIEUC\0"
  "HANGUL CHOSEONG NIEUN-HIEUH\0"
  "HANGUL CHOSEONG NIEUN-KIYEOK\0"
  "HANGUL CHOSEONG NIEUN-PIEUP\0"
  "HANGUL CHOSEONG NIEUN-SIOS\0"
  "HANGUL CHOSEONG NIEUN-TIKEUT\0"
  "HANGUL CHOSEONG PANSIOS\0"
  "HANGUL CHOSEONG PHIEUPH\0"
  "HANGUL CHOSEONG PHIEUPH-HIEUH\0"
  "HANGUL CHOSEONG PHIEUPH-PIEUP\0"
  "HANGUL CHOSEONG PIEUP\0"
  "HANGUL CHOSEONG PIEUP-CHIEUCH\0"
  "HANGUL CHOSEONG PIEUP-CIEUC\0"
  "HANGUL CHOSEONG PIEUP-HIEUH\0"
  "HANGUL CHOSEONG PIEUP-KHIEUKH\0"
  "HANGUL CHOSEONG PIEUP-KIYEOK\0"
  "HANGUL CHOSEONG PIEUP-NIEUN\0"
  "HANGUL CHOSEONG PIEUP-PHIEUPH\0"
  "HANGUL CHOSEONG PIEUP-SIOS\0"
  "HANGUL CHOSEONG PIEUP-SIOS-CIEUC\0"
  "HANGUL CHOSEONG PIEUP-SIOS-KIYEOK\0"
  "HANGUL CHOSEONG PIEUP-SIOS-PIEUP\0"
  "HANGUL CHOSEONG PIEUP-SIOS-THIEUTH\0"
  "HANGUL CHOSEONG PIEUP-SIOS-TIKEUT\0"
  "HANGUL CHOSEONG PIEUP-SSANGSIOS\0"
  "HANGUL CHOSEONG PIEUP-THIEUTH\0"
  "HANGUL CHOSEONG PIEUP-TIKEUT\0"
  "HANGUL CHOSEONG RIEUL\0"
  "HANGUL CHOSEONG RIEUL-CIEUC\0"
  "HANGUL CHOSEONG RIEUL-HIEUH\0"
  "HANGUL CHOSEONG RIEUL-KAPYEOUNPIEUP\0"
  "HANGUL CHOSEONG RIEUL-KHIEUKH\0"
  "HANGUL CHOSEONG RIEUL-KIYEOK\0"
  "HANGUL CHOSEONG RIEUL-MIEUM\0"
  "HANGUL CHOSEONG RIEUL-NIEUN\0"
  "HANGUL CHOSEONG RIEUL-PIEUP\0"
  "HANGUL CHOSEONG RIEUL-SIOS\0"
  "HANGUL CHOSEONG RIEUL-SSANGKIYEOK\0"
  "HANGUL CHOSEONG RIEUL-SSANGPIEUP\0"
  "HANGUL CHOSEONG RIEUL-SSANGTIKEUT\0"
  "HANGUL CHOSEONG RIEUL-TIKEUT\0"
  "HANGUL CHOSEONG SIOS\0"
  "HANGUL CHOSEONG SIOS-CHIEUCH\0"
  "HANGUL CHOSEONG SIOS-CIEUC\0"
  "HANGUL CHOSEONG SIOS-HIEUH\0"
  "HANGUL CHOSEONG SIOS-IEUNG\0"
  "HANGUL CHOSEONG SIOS-KHIEUKH\0"
  "HANGUL CHOSEONG SIOS-KIYEOK\0"
  "HANGUL CHOSEONG SIOS-MIEUM\0"
  "HANGUL CHOSEONG SIOS-NIEUN\0"
  "HANGUL CHOSEONG SIOS-PHIEUPH\0"
  "HANGUL CHOSEONG SIOS-PIEUP\0"
  "HANGUL CHOSEONG SIOS-PIEUP-KIYEOK\0"
  "HANGUL CHOSEONG SIOS-RIEUL\0"
  "HANGUL CHOSEONG SIOS-SSANGSIOS\0"
  "HANGUL CHOSEONG SIOS-THIEUTH\0"
  "HANGUL CHOSEONG SIOS-TIKEUT\0"
  "HANGUL CHOSEONG SSANGCIEUC\0"
  "HANGUL CHOSEONG SSANGCIEUC-HIEUH\0"
  "HANGUL CHOSEONG SSANGHIEUH\0"
  "HANGUL CHOSEONG SSANGIEUNG\0"
  "HANGUL CHOSEONG SSANGKIYEOK\0"
  "HANGUL CHOSEONG SSANGNIEUN\0"
  "HANGUL CHOSEONG SSANGPIEUP\0"
  "HANGUL CHOSEONG SSANGRIEUL\0"
  "HANGUL CHOSEONG SSANGSIOS\0"
  "HANGUL CHOSEONG SSANGSIOS-PIEUP\0"
  "HANGUL CHOSEONG SSANGTHIEUTH\0"
  "HANGUL CHOSEONG SSANGTIKEUT\0"
  "HANGUL CHOSEONG SSANGYEORINHIEUH\0"
  "HANGUL CHOSEONG THIEUTH\0"
  "HANGUL CHOSEONG TIKEUT\0"
  "HANGUL CHOSEONG TIKEUT-CIEUC\0"
  "HANGUL CHOSEONG TIKEUT-KIYEOK\0"
  "HANGUL CHOSEONG TIKEUT-MIEUM\0"
  "HANGUL CHOSEONG TIKEUT-PIEUP\0"
  "HANGUL CHOSEONG TIKEUT-RIEUL\0"
  "HANGUL CHOSEONG TIKEUT-SIOS\0"
  "HANGUL CHOSEONG YEORINHIEUH\0"
  "HANGUL CHOSEONG YESIEUNG\0"
  "HANGUL DOUBLE DOT TONE MARK\0"
  "HANGUL FILLER\0"
  "HANGUL JONGSEONG CHIEUCH\0"
  "HANGUL JONGSEONG CIEUC\0"
  "HANGUL JONGSEONG CIEUC-PIEUP\0"
  "HANGUL JONGSEONG CIEUC-SSANGPIEUP\0"
  "HANGUL JONGSEONG HIEUH\0"
  "HANGUL JONGSEONG HIEUH-MIEUM\0"
  "HANGUL JONGSEONG HIEUH-NIEUN\0"
  "HANGUL JONGSEONG HIEUH-PIEUP\0"
  "HANGUL JONGSEONG HIEUH-RIEUL\0"
  "HANGUL JONGSEONG IEUNG\0"
  "HANGUL JONGSEONG IEUNG-KHIEUKH\0"
  "HANGUL JONGSEONG IEUNG-KIYEOK\0"
  "HANGUL JONGSEONG IEUNG-SSANGKIYEOK\0"
  "HANGUL JONGSEONG KAPYEOUNMIEUM\0"
  "HANGUL JONGSEONG KAPYEOUNPHIEUPH\0"
  "HANGUL JONGSEONG KAPYEOUNPIEUP\0"
  "HANGUL JONGSEONG KAPYEOUNRIEUL\0"
  "HANGUL JONGSEONG KHIEUKH\0"
  "HANGUL JONGSEONG KIYEOK\0"
  "HANGUL JONGSEONG KIYEOK-CHIEUCH\0"
  "HANGUL JONGSEONG KIYEOK-HIEUH\0"
  "HANGUL JONGSEONG KIYEOK-KHIEUKH\0"
  "HANGUL JONGSEONG KIYEOK-NIEUN\0"
  "HANGUL JONGSEONG KIYEOK-PIEUP\0"
  "HANGUL JONGSEONG KIYEOK-RIEUL\0"
  "HANGUL JONGSEONG KIYEOK-SIOS\0"
  "HANGUL JONGSEONG KIYEOK-SIOS-KIYEOK\0"
  "HANGUL JONGSEONG MIEUM\0"
  "HANGUL JONGSEONG MIEUM-CHIEUCH\0"
  "HANGUL JONGSEONG MIEUM-CIEUC\0"
  "HANGUL JONGSEONG MIEUM-HIEUH\0"
  "HANGUL JONGSEONG MIEUM-KIYEOK\0"
  "HANGUL JONGSEONG MIEUM-NIEUN\0"
  "HANGUL JONGSEONG MIEUM-PANSIOS\0"
  "HANGUL JONGSEONG MIEUM-PIEUP\0"
  "HANGUL JONGSEONG MIEUM-PIEUP-SIOS\0"
  "HANGUL JONGSEONG MIEUM-RIEUL\0"
  "HANGUL JONGSEONG MIEUM-SIOS\0"
  "HANGUL JONGSEONG MIEUM-SSANGNIEUN\0"
  "HANGUL JONGSEONG MIEUM-SSANGSIOS\0"
  "HANGUL JONGSEONG NIEUN\0"
  "HANGUL JONGSEONG NIEUN-CHIEUCH\0"
  "HANGUL JONGSEONG NIEUN-CIEUC\0"
  "HANGUL JONGSEONG NIEUN-HIEUH\0"
  "HANGUL JONGSEONG NIEUN-KIYEOK\0"
  "HANGUL JONGSEONG NIEUN-PANSIOS\0"
  "HANGUL JONGSEONG NIEUN-RIEUL\0"
  "HANGUL JONGSEONG NIEUN-SIOS\0"
  "HANGUL JONGSEONG NIEUN-THIEUTH\0"
  "HANGUL JONGSEONG NIEUN-TIKEUT\0"
  "HANGUL JONGSEONG PANSIOS\0"
  "HANGUL JONGSEONG PANSIOS-KAPYEOUNPIEUP\0"
  "HANGUL JONGSEONG PANSIOS-PIEUP\0"
  "HANGUL JONGSEONG PHIEUPH\0"
  "HANGUL JONGSEONG PHIEUPH-PIEUP\0"
  "HANGUL JONGSEONG PHIEUPH-SIOS\0"
  "HANGUL JONGSEONG PHIEUPH-THIEUTH\0"
  "HANGUL JONGSEONG PIEUP\0"
  "HANGUL JONGSEONG PIEUP-CHIEUCH\0"
  "HANGUL JONGSEONG PIEUP-CIEUC\0"
  "HANGUL JONGSEONG PIEUP-HIEUH\0"
  "HANGUL JONGSEONG PIEUP-MIEUM\0"
  "HANGUL JONGSEONG PIEUP-PHIEUPH\0"
  "HANGUL JONGSEONG PIEUP-RIEUL\0"
  "HANGUL JONGSEONG PIEUP-RIEUL-PHIEUPH\0"
  "HANGUL JONGSEONG PIEUP-SIOS\0"
  "HANGUL JONGSEONG PIEUP-SIOS-TIKEUT\0"
  "HANGUL JONGSEONG PIEUP-TIKEUT\0"
  "HANGUL JONGSEONG RIEUL\0"
  "HANGUL JONGSEONG RIEUL-HIEUH\0"
  "HANGUL JONGSEONG RIEUL-KAPYEOUNPIEUP\0"
  "HANGUL JONGSEONG RIEUL-KHIEUKH\0"
  "HANGUL JONGSEONG RIEUL-KIYEOK\0"
  "HANGUL JONGSEONG RIEUL-KIYEOK-HIEUH\0"
  "HANGUL JONGSEONG RIEUL-KIYEOK-SIOS\0"
  "HANGUL JONGSEONG RIEUL-MIEUM\0"
  "HANGUL JONGSEONG RIEUL-MIEUM-HIEUH\0"
  "HANGUL JONGSEONG RIEUL-MIEUM-KIYEOK\0"
  "HANGUL JONGSEONG RIEUL-MIEUM-SIOS\0"
  "HANGUL JONGSEONG RIEUL-NIEUN\0"
  "HANGUL JONGSEONG RIEUL-PANSIOS\0"
  "HANGUL JONGSEONG RIEUL-PHIEUPH\0"
  "HANGUL JONGSEONG RIEUL-PIEUP\0"
  "HANGUL JONGSEONG RIEUL-PIEUP-HIEUH\0"
  "HANGUL JONGSEONG RIEUL-PIEUP-PHIEUPH\0"
  "HANGUL JONGSEONG RIEUL-PIEUP-SIOS\0"
  "HANGUL JONGSEONG RIEUL-PIEUP-TIKEUT\0"
  "HANGUL JONGSEONG RIEUL-SIOS\0"
  "HANGUL JONGSEONG RIEUL-SSANGKIYEOK\0"
  "HANGUL JONGSEONG RIEUL-SSANGSIOS\0"
  "HANGUL JONGSEONG RIEUL-THIEUTH\0"
  "HANGUL JONGSEONG RIEUL-TIKEUT\0"
  "HANGUL JONGSEONG RIEUL-TIKEUT-HIEUH\0"
  "HANGUL JONGSEONG RIEUL-YEORINHIEUH\0"
  "HANGUL JONGSEONG RIEUL-YEORINHIEUH-HIEUH\0"
  "HANGUL JONGSEONG RIEUL-YESIEUNG\0"
  "HANGUL JONGSEONG SIOS\0"
  "HANGUL JONGSEONG SIOS-CHIEUCH\0"
  "HANGUL JONGSEONG SIOS-CIEUC\0"
  "HANGUL JONGSEONG SIOS-HIEUH\0"
  "HANGUL JONGSEONG SIOS-KAPYEOUNPIEUP\0"
  "HANGUL JONGSEONG SIOS-KIYEOK\0"
  "HANGUL JONGSEONG SIOS-MIEUM\0"
  "HANGUL JONGSEONG SIOS-PANSIOS\0"
  "HANGUL JONGSEONG SIOS-PIEUP\0"
  "HANGUL JONGSEONG SIOS-RIEUL\0"
  "HANGUL JONGSEONG SIOS-THIEUTH\0"
  "HANGUL JONGSEONG SIOS-TIKEUT\0"
  "HANGUL JONGSEONG SSANGCIEUC\0"
  "HANGUL JONGSEONG SSANGIEUNG\0"
  "HANGUL JONGSEONG SSANGKIYEOK\0"
  "HANGUL JONGSEONG SSANGMIEUM\0"
  "HANGUL JONGSEONG SSANGNIEUN\0"
  "HANGUL JONGSEONG SSANGPIEUP\0"
  "HANGUL JONGSEONG SSANGRIEUL\0"
  "HANGUL JONGSEONG SSANGRIEUL-KHIEUKH\0"
  "HANGUL JONGSEONG SSANGSIOS\0"
  "HANGUL JONGSEONG SSANGSIOS-KIYEOK\0"
  "HANGUL JONGSEONG SSANGSIOS-TIKEUT\0"
  "HANGUL JONGSEONG SSANGTIKEUT\0"
  "HANGUL JONGSEONG SSANGTIKEUT-PIEUP\0"
  "HANGUL JONGSEONG THIEUTH\0"
  "HANGUL JONGSEONG TIKEUT\0"
  "HANGUL JONGSEONG TIKEUT-CHIEUCH\0"
  "HANGUL JONGSEONG TIKEUT-CIEUC\0"
  "HANGUL JONGSEONG TIKEUT-KIYEOK\0"
  "HANGUL JONGSEONG TIKEUT-PIEUP\0"
  "HANGUL JONGSEONG TIKEUT-RIEUL\0"
  "HANGUL JONGSEONG TIKEUT-SIOS\0"
  "HANGUL JONGSEONG TIKEUT-SIOS-KIYEOK\0"
  "HANGUL JONGSEONG TIKEUT-THIEUTH\0"
  "HANGUL JONGSEONG YEORINHIEUH\0"
  "HANGUL JONGSEONG YESIEUNG\0"
  "HANGUL JONGSEONG YESIEUNG-HIEUH\0"
  "HANGUL JONGSEONG YESIEUNG-MIEUM\0"
  "HANGUL JONGSEONG YESIEUNG-PANSIOS\0"
  "HANGUL JONGSEONG YESIEUNG-SIOS\0"
  "HANGUL JUNGSEONG A\0"
  "HANGUL JUNGSEONG A-EU\0"
  "HANGUL JUNGSEONG A-O\0"
  "HANGUL JUNGSEONG A-U\0"
  "HANGUL JUNGSEONG AE\0"
  "HANGUL JUNGSEONG ARAEA\0"
  "HANGUL JUNGSEONG ARAEA-A\0"
  "HANGUL JUNGSEONG ARAEA-E\0"
  "HANGUL JUNGSEONG ARAEA-EO\0"
  "HANGUL JUNGSEONG ARAEA-I\0"
  "HANGUL JUNGSEONG ARAEA-U\0"
  "HANGUL JUNGSEONG E\0"
  "HANGUL JUNGSEONG EO\0"
  "HANGUL JUNGSEONG EO-EU\0"
  "HANGUL JUNGSEONG EO-O\0"
  "HANGUL JUNGSEONG EO-U\0"
  "HANGUL JUNGSEONG EU\0"
  "HANGUL JUNGSEONG EU-A\0"
  "HANGUL JUNGSEONG EU-E\0"
  "HANGUL JUNGSEONG EU-EO\0"
  "HANGUL JUNGSEONG EU-EU\0"
  "HANGUL JUNGSEONG EU-O\0"
  "HANGUL JUNGSEONG EU-U\0"
  "HANGUL JUNGSEONG FILLER\0"
  "HANGUL JUNGSEONG I\0"
  "HANGUL JUNGSEONG I-A\0"
  "HANGUL JUNGSEONG I-ARAEA\0"
  "HANGUL JUNGSEONG I-EU\0"
  "HANGUL JUNGSEONG I-I\0"
  "HANGUL JUNGSEONG I-O\0"
  "HANGUL JUNGSEONG I-O-I\0"
  "HANGUL JUNGSEONG I-U\0"
  "HANGUL JUNGSEONG I-YA\0"
  "HANGUL JUNGSEONG I-YA-O\0"
  "HANGUL JUNGSEONG I-YAE\0"
  "HANGUL JUNGSEONG I-YE\0"
  "HANGUL JUNGSEONG I-YEO\0"
  "HANGUL JUNGSEONG I-YO\0"
  "HANGUL JUNGSEONG I-YU\0"
  "HANGUL JUNGSEONG O\0"
  "HANGUL JUNGSEONG O-E\0"
  "HANGUL JUNGSEONG O-EO\0"
  "HANGUL JUNGSEONG O-O\0"
  "HANGUL JUNGSEONG O-O-I\0"
  "HANGUL JUNGSEONG O-U\0"
  "HANGUL JUNGSEONG O-YA\0"
  "HANGUL JUNGSEONG O-YAE\0"
  "HANGUL JUNGSEONG O-YE\0"
  "HANGUL JUNGSEONG O-YEO\0"
  "HANGUL JUNGSEONG OE\0"
  "HANGUL JUNGSEONG SSANGARAEA\0"
  "HANGUL JUNGSEONG U\0"
  "HANGUL JUNGSEONG U-A\0"
  "HANGUL JUNGSEONG U-AE\0"
  "HANGUL JUNGSEONG U-EO-EU\0"
  "HANGUL JUNGSEONG U-I-I\0"
  "HANGUL JUNGSEONG U-U\0"
  "HANGUL JUNGSEONG U-YE\0"
  "HANGUL JUNGSEONG U-YEO\0"
  "HANGUL JUNGSEONG WA\0"
  "HANGUL JUNGSEONG WAE\0"
  "HANGUL JUNGSEONG WE\0"
  "HANGUL JUNGSEONG WEO\0"
  "HANGUL JUNGSEONG WI\0"
  "HANGUL JUNGSEONG YA\0"
  "HANGUL JUNGSEONG YA-O\0"
  "HANGUL JUNGSEONG YA-U\0"
  "HANGUL JUNGSEONG YA-YO\0"
  "HANGUL JUNGSEONG YAE\0"
  "HANGUL JUNGSEONG YE\0"
  "HANGUL JUNGSEONG YEO\0"
  "HANGUL JUNGSEONG YEO-O\0"
  "HANGUL JUNGSEONG YEO-U\0"
  "HANGUL JUNGSEONG YEO-YA\0"
  "HANGUL JUNGSEONG YI\0"
  "HANGUL JUNGSEONG YI-U\0"
  "HANGUL JUNGSEONG YO\0"
  "HANGUL JUNGSEONG YO-A\0"
  "HANGUL JUNGSEONG YO-AE\0"
  "HANGUL JUNGSEONG YO-EO\0"
  "HANGUL JUNGSEONG YO-I\0"
  "HANGUL JUNGSEONG YO-O\0"
  "HANGUL JUNGSEONG YO-YA\0"
  "HANGUL JUNGSEONG YO-YAE\0"
  "HANGUL JUNGSEONG YO-YEO\0"
  "HANGUL JUNGSEONG YU\0"
  "HANGUL JUNGSEONG YU-A\0"
  "HANGUL JUNGSEONG YU-AE\0"
  "HANGUL JUNGSEONG YU-E\0"
  "HANGUL JUNGSEONG YU-EO\0"
  "HANGUL JUNGSEONG YU-I\0"
  "HANGUL JUNGSEONG YU-O\0"
  "HANGUL JUNGSEONG YU-U\0"
  "HANGUL JUNGSEONG YU-YE\0"
  "HANGUL JUNGSEONG YU-YEO\0"
  "HANGUL LETTER A\0"
  "HANGUL LETTER AE\0"
  "HANGUL LETTER ARAEA\0"
  "HANGUL LETTER ARAEAE\0"
  "HANGUL LETTER CHIEUCH\0"
  "HANGUL LETTER CIEUC\0"
  "HANGUL LETTER E\0"
  "HANGUL LETTER EO\0"
  "HANGUL LETTER EU\0"
  "HANGUL LETTER HIEUH\0"
  "HANGUL LETTER I\0"
  "HANGUL LETTER IEUNG\0"
  "HANGUL LETTER KAPYEOUNMIEUM\0"
  "HANGUL LETTER KAPYEOUNPHIEUPH\0"
  "HANGUL LETTER KAPYEOUNPIEUP\0"
  "HANGUL LETTER KAPYEOUNSSANGPIEUP\0"
  "HANGUL LETTER KHIEUKH\0"
  "HANGUL LETTER KIYEOK\0"
  "HANGUL LETTER KIYEOK-SIOS\0"
  "HANGUL LETTER MIEUM\0"
  "HANGUL LETTER MIEUM-PANSIOS\0"
  "HANGUL LETTER MIEUM-PIEUP\0"
  "HANGUL LETTER MIEUM-SIOS\0"
  "HANGUL LETTER NIEUN\0"
  "HANGUL LETTER NIEUN-CIEUC\0"
  "HANGUL LETTER NIEUN-HIEUH\0"
  "HANGUL LETTER NIEUN-PANSIOS\0"
  "HANGUL LETTER NIEUN-SIOS\0"
  "HANGUL LETTER NIEUN-TIKEUT\0"
  "HANGUL LETTER O\0"
  "HANGUL LETTER OE\0"
  "HANGUL LETTER PANSIOS\0"
  "HANGUL LETTER PHIEUPH\0"
  "HANGUL LETTER PIEUP\0"
  "HANGUL LETTER PIEUP-CIEUC\0"
  "HANGUL LETTER PIEUP-KIYEOK\0"
  "HANGUL LETTER PIEUP-SIOS\0"
  "HANGUL LETTER PIEUP-SIOS-KIYEOK\0"
  "HANGUL LETTER PIEUP-SIOS-TIKEUT\0"
  "HANGUL LETTER PIEUP-THIEUTH\0"
  "HANGUL LETTER PIEUP-TIKEUT\0"
  "HANGUL LETTER RIEUL\0"
  "HANGUL LETTER RIEUL-HIEUH\0"
  "HANGUL LETTER RIEUL-KIYEOK\0"
  "HANGUL LETTER RIEUL-KIYEOK-SIOS\0"
  "HANGUL LETTER RIEUL-MIEUM\0"
  "HANGUL LETTER RIEUL-PANSIOS\0"
  "HANGUL LETTER RIEUL-PHIEUPH\0"
  "HANGUL LETTER RIEUL-PIEUP\0"
  "HANGUL LETTER RIEUL-PIEUP-SIOS\0"
  "HANGUL LETTER RIEUL-SIOS\0"
  "HANGUL LETTER RIEUL-THIEUTH\0"
  "HANGUL LETTER RIEUL-TIKEUT\0"
  "HANGUL LETTER RIEUL-YEORINHIEUH\0"
  "HANGUL LETTER SIOS\0"
  "HANGUL LETTER SIOS-CIEUC\0"
  "HANGUL LETTER SIOS-KIYEOK\0"
  "HANGUL LETTER SIOS-NIEUN\0"
  "HANGUL LETTER SIOS-PIEUP\0"
  "HANGUL LETTER SIOS-TIKEUT\0"
  "HANGUL LETTER SSANGCIEUC\0"
  "HANGUL LETTER SSANGHIEUH\0"
  "HANGUL LETTER SSANGIEUNG\0"
  "HANGUL LETTER SSANGKIYEOK\0"
  "HANGUL LETTER SSANGNIEUN\0"
  "HANGUL LETTER SSANGPIEUP\0"
  "HANGUL LETTER SSANGSIOS\0"
  "HANGUL LETTER SSANGTIKEUT\0"
  "HANGUL LETTER THIEUTH\0"
  "HANGUL LETTER TIKEUT\0"
  "HANGUL LETTER U\0"
  "HANGUL LETTER WA\0"
  "HANGUL LETTER WAE\0"
  "HANGUL LETTER WE\0"
  "HANGUL LETTER WEO\0"
  "HANGUL LETTER WI\0"
  "HANGUL LETTER YA\0"
  "HANGUL LETTER YAE\0"
  "HANGUL LETTER YE\0"
  "HANGUL LETTER YEO\0"
  "HANGUL LETTER YEORINHIEUH\0"
  "HANGUL LETTER YESIEUNG\0"
  "HANGUL LETTER YESIEUNG-PANSIOS\0"
  "HANGUL LETTER YESIEUNG-SIOS\0"
  "HANGUL LETTER YI\0"
  "HANGUL LETTER YO\0"
  "HANGUL LETTER YO-I\0"
  "HANGUL LETTER YO-YA\0"
  "HANGUL LETTER YO-YAE\0"
  "HANGUL LETTER YU\0"
  "HANGUL LETTER YU-I\0"
  "HANGUL LETTER YU-YE\0"
  "HANGUL LETTER YU-YEO\0"
  "HANGUL SINGLE DOT TONE MARK\0"
  "HANGZHOU NUMERAL EIGHT\0"
  "HANGZHOU NUMERAL FIVE\0"
  "HANGZHOU NUMERAL FOUR\0"
  "HANGZHOU NUMERAL NINE\0"
  "HANGZHOU NUMERAL ONE\0"
  "HANGZHOU NUMERAL SEVEN\0"
  "HANGZHOU NUMERAL SIX\0"
  "HANGZHOU NUMERAL TEN\0"
  "HANGZHOU NUMERAL THIRTY\0"
  "HANGZHOU NUMERAL THREE\0"
  "HANGZHOU NUMERAL TWENTY\0"
  "HANGZHOU NUMERAL TWO\0"
  "HANIFI ROHINGYA DIGIT EIGHT\0"
  "HANIFI ROHINGYA DIGIT FIVE\0"
  "HANIFI ROHINGYA DIGIT FOUR\0"
  "HANIFI ROHINGYA DIGIT NINE\0"
  "HANIFI ROHINGYA DIGIT ONE\0"
  "HANIFI ROHINGYA DIGIT SEVEN\0"
  "HANIFI ROHINGYA DIGIT SIX\0"
  "HANIFI ROHINGYA DIGIT THREE\0"
  "HANIFI ROHINGYA DIGIT TWO\0"
  "HANIFI ROHINGYA DIGIT ZERO\0"
  "HANIFI ROHINGYA LETTER A\0"
  "HANIFI ROHINGYA LETTER BA\0"
  "HANIFI ROHINGYA LETTER CA\0"
  "HANIFI ROHINGYA LETTER DA\0"
  "HANIFI ROHINGYA LETTER DDA\0"
  "HANIFI ROHINGYA LETTER FA\0"
  "HANIFI ROHINGYA LETTER GA\0"
  "HANIFI ROHINGYA LETTER HA\0"
  "HANIFI ROHINGYA LETTER JA\0"
  "HANIFI ROHINGYA LETTER KA\0"
  "HANIFI ROHINGYA LETTER KHA\0"
  "HANIFI ROHINGYA LETTER KINNA WA\0"
  "HANIFI ROHINGYA LETTER KINNA YA\0"
  "HANIFI ROHINGYA LETTER LA\0"
  "HANIFI ROHINGYA LETTER MA\0"
  "HANIFI ROHINGYA LETTER NA\0"
  "HANIFI ROHINGYA LETTER NGA\0"
  "HANIFI ROHINGYA LETTER NYA\0"
  "HANIFI ROHINGYA LETTER PA\0"
  "HANIFI ROHINGYA LETTER RA\0"
  "HANIFI ROHINGYA LETTER RRA\0"
  "HANIFI ROHINGYA LETTER SA\0"
  "HANIFI ROHINGYA LETTER SHA\0"
  "HANIFI ROHINGYA LETTER TA\0"
  "HANIFI ROHINGYA LETTER TTA\0"
  "HANIFI ROHINGYA LETTER VA\0"
  "HANIFI ROHINGYA LETTER WA\0"
  "HANIFI ROHINGYA LETTER YA\0"
  "HANIFI ROHINGYA LETTER ZA\0"
  "HANIFI ROHINGYA MARK NA KHONNA\0"
  "HANIFI ROHINGYA MARK SAKIN\0"
  "HANIFI ROHINGYA SIGN HARBAHAY\0"
  "HANIFI ROHINGYA SIGN TAHALA\0"
  "HANIFI ROHINGYA SIGN TANA\0"
  "HANIFI ROHINGYA SIGN TASSI\0"
  "HANIFI ROHINGYA VOWEL A\0"
  "HANIFI ROHINGYA VOWEL E\0"
  "HANIFI ROHINGYA VOWEL I\0"
  "HANIFI ROHINGYA VOWEL O\0"
  "HANIFI ROHINGYA VOWEL U\0"
  "HANUNOO LETTER A\0"
  "HANUNOO LETTER BA\0"
  "HANUNOO LETTER DA\0"
  "HANUNOO LETTER GA\0"
  "HANUNOO LETTER HA\0"
  "HANUNOO LETTER I\0"
  "HANUNOO LETTER KA\0"
  "HANUNOO LETTER LA\0"
  "HANUNOO LETTER MA\0"
  "HANUNOO LETTER NA\0"
  "HANUNOO LETTER NGA\0"
  "HANUNOO LETTER PA\0"
  "HANUNOO LETTER RA\0"
  "HANUNOO LETTER SA\0"
  "HANUNOO LETTER TA\0"
  "HANUNOO LETTER U\0"
  "HANUNOO LETTER WA\0"
  "HANUNOO LETTER YA\0"
  "HANUNOO SIGN PAMUDPOD\0"
  "HANUNOO VOWEL SIGN I\0"
  "HANUNOO VOWEL SIGN U\0"
  "HAPPY PERSON RAISING ONE HAND\0"
  "HARD DISK\0"
  "HARP\0"
  "HATCHING CHICK\0"
  "HATRAN LETTER ALEPH\0"
  "HATRAN LETTER AYN\0"
  "HATRAN LETTER BETH\0"
  "HATRAN LETTER DALETH-RESH\0"
  "HATRAN LETTER GIMEL\0"
  "HATRAN LETTER HE\0"
  "HATRAN LETTER HETH\0"
  "HATRAN LETTER KAPH\0"
  "HATRAN LETTER LAMEDH\0"
  "HATRAN LETTER MEM\0"
  "HATRAN LETTER NUN\0"
  "HATRAN LETTER PE\0"
  "HATRAN LETTER QOPH\0"
  "HATRAN LETTER SADHE\0"
  "HATRAN LETTER SAMEKH\0"
  "HATRAN LETTER SHIN\0"
  "HATRAN LETTER TAW\0"
  "HATRAN LETTER TETH\0"
  "HATRAN LETTER WAW\0"
  "HATRAN LETTER YODH\0"
  "HATRAN LETTER ZAYN\0"
  "HATRAN NUMBER FIVE\0"
  "HATRAN NUMBER ONE\0"
  "HATRAN NUMBER ONE HUNDRED\0"
  "HATRAN NUMBER TEN\0"
  "HATRAN NUMBER TWENTY\0"
  "HAUMEA\0"
  "HEADPHONE\0"
  "HEADSTONE\0"
  "HEADSTONE GRAVEYARD SYMBOL\0"
  "HEAR-NO-EVIL MONKEY\0"
  "HEART DECORATION\0"
  "HEART HANDS\0"
  "HEART WITH ARROW\0"
  "HEART WITH RIBBON\0"
  "HEART WITH TIP ON THE LEFT\0"
  "HEAVY AMPERSAND ORNAMENT\0"
  "HEAVY ARROW SHAFT WIDTH ONE\0"
  "HEAVY ARROW SHAFT WIDTH ONE HALF\0"
  "HEAVY ARROW SHAFT WIDTH ONE THIRD\0"
  "HEAVY ARROW SHAFT WIDTH TWO THIRDS\0"
  "HEAVY ASTERISK\0"
  "HEAVY BALLOT X\0"
  "HEAVY BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW\0"
  "HEAVY BLACK CURVED UPWARDS AND RIGHTWARDS ARROW\0"
  "HEAVY BLACK HEART\0"
  "HEAVY BLACK-FEATHERED NORTH EAST ARROW\0"
  "HEAVY BLACK-FEATHERED RIGHTWARDS ARROW\0"
  "HEAVY BLACK-FEATHERED SOUTH EAST ARROW\0"
  "HEAVY CHECK MARK\0"
  "HEAVY CHEVRON SNOWFLAKE\0"
  "HEAVY CIRCLE\0"
  "HEAVY CIRCLE WITH CIRCLE INSIDE\0"
  "HEAVY CIRCLE WITH STROKE AND TWO DOTS ABOVE\0"
  "HEAVY CIRCLED SALTIRE\0"
  "HEAVY CONCAVE-POINTED BLACK RIGHTWARDS ARROW\0"
  "HEAVY DASHED TRIANGLE-HEADED RIGHTWARDS ARROW\0"
  "HEAVY DIVISION SIGN\0"
  "HEAVY DOLLAR SIGN\0"
  "HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT\0"
  "HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT\0"
  "HEAVY DOWNWARDS ARROW WITH EQUILATERAL ARROWHEAD\0"
  "HEAVY DOWNWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD\0"
  "HEAVY EIGHT POINTED BLACK STAR\0"
  "HEAVY EIGHT POINTED PINWHEEL STAR\0"
  "HEAVY EIGHT POINTED RECTILINEAR BLACK STAR\0"
  "HEAVY EIGHT SPOKED ASTERISK\0"
  "HEAVY EIGHT TEARDROP-SPOKED PROPELLER ASTERISK\0"
  "HEAVY EQUALS SIGN\0"
  "HEAVY EXCLAMATION MARK ORNAMENT\0"
  "HEAVY EXCLAMATION MARK SYMBOL\0"
  "HEAVY FIVE POINTED BLACK STAR\0"
  "HEAVY FIVE SPOKED ASTERISK\0"
  "HEAVY FOUR BALLOON-SPOKED ASTERISK\0"
  "HEAVY GREEK CROSS\0"
  "HEAVY HEART EXCLAMATION MARK ORNAMENT\0"
  "HEAVY HORIZONTAL FILL\0"
  "HEAVY INTERROBANG ORNAMENT\0"
  "HEAVY LARGE CIRCLE\0"
  "HEAVY LATIN CROSS\0"
  "HEAVY LEFT-POINTING ANGLE BRACKET ORNAMENT\0"
  "HEAVY LEFT-POINTING ANGLE QUOTATION MARK ORNAMENT\0"
  "HEAVY LEFTWARDS ARROW WITH EQUILATERAL ARROWHEAD\0"
  "HEAVY LEFTWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD\0"
  "HEAVY LIGATURE OPEN ET ORNAMENT\0"
  "HEAVY LOW DOUBLE COMMA QUOTATION MARK ORNAMENT\0"
  "HEAVY LOW SINGLE COMMA QUOTATION MARK ORNAMENT\0"
  "HEAVY LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW\0"
  "HEAVY MINUS SIGN\0"
  "HEAVY MULTIPLICATION X\0"
  "HEAVY NORTH EAST ARROW\0"
  "HEAVY NORTH EAST POINTING BUD\0"
  "HEAVY NORTH EAST POINTING VINE LEAF\0"
  "HEAVY NORTH WEST POINTING BUD\0"
  "HEAVY NORTH WEST POINTING VINE LEAF\0"
  "HEAVY OPEN CENTRE CROSS\0"
  "HEAVY OUTLINED BLACK STAR\0"
  "HEAVY OVAL WITH OVAL INSIDE\0"
  "HEAVY PLUS SIGN\0"
  "HEAVY RIGHT-POINTING ANGLE BRACKET ORNAMENT\0"
  "HEAVY RIGHT-POINTING ANGLE QUOTATION MARK ORNAMENT\0"
  "HEAVY RIGHTWARDS ARROW\0"
  "HEAVY RIGHTWARDS ARROW WITH EQUILATERAL ARROWHEAD\0"
  "HEAVY RIGHTWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD\0"
  "HEAVY ROUND-TIPPED RIGHTWARDS ARROW\0"
  "HEAVY SALTIRE\0"
  "HEAVY SANS-SERIF INTERROBANG ORNAMENT\0"
  "HEAVY SCRIPT LIGATURE ET ORNAMENT\0"
  "HEAVY SINGLE COMMA QUOTATION MARK ORNAMENT\0"
  "HEAVY SINGLE TURNED COMMA QUOTATION MARK ORNAMENT\0"
  "HEAVY SIX POINTED BLACK STAR\0"
  "HEAVY SIX SPOKED ASTERISK\0"
  "HEAVY SOUTH EAST ARROW\0"
  "HEAVY SOUTH EAST POINTING BUD\0"
  "HEAVY SOUTH EAST POINTING VINE LEAF\0"
  "HEAVY SOUTH WEST POINTING BUD\0"
  "HEAVY SOUTH WEST POINTING VINE LEAF\0"
  "HEAVY SPARKLE\0"
  "HEAVY TEARDROP-SHANKED RIGHTWARDS ARROW\0"
  "HEAVY TEARDROP-SPOKED ASTERISK\0"
  "HEAVY TEARDROP-SPOKED PINWHEEL ASTERISK\0"
  "HEAVY TRIANGLE-HEADED RIGHTWARDS ARROW\0"
  "HEAVY TWELVE POINTED BLACK STAR\0"
  "HEAVY TWELVE POINTED PINWHEEL STAR\0"
  "HEAVY UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW\0"
  "HEAVY UPWARDS ARROW WITH EQUILATERAL ARROWHEAD\0"
  "HEAVY UPWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD\0"
  "HEAVY VERTICAL BAR\0"
  "HEAVY WEDGE-TAILED RIGHTWARDS ARROW\0"
  "HEAVY WHITE CIRCLE\0"
  "HEAVY WHITE DOWN-POINTING TRIANGLE\0"
  "HEAVY WHITE SQUARE\0"
  "HEAVY WHITE SQUARE CONTAINING BLACK VERY SMALL SQUARE\0"
  "HEAVY WIDE-HEADED RIGHTWARDS ARROW\0"
  "HEBREW ACCENT ATNAH HAFUKH\0"
  "HEBREW ACCENT DARGA\0"
  "HEBREW ACCENT DEHI\0"
  "HEBREW ACCENT ETNAHTA\0"
  "HEBREW ACCENT GERESH\0"
  "HEBREW ACCENT GERESH MUQDAM\0"
  "HEBREW ACCENT GERSHAYIM\0"
  "HEBREW ACCENT ILUY\0"
  "HEBREW ACCENT MAHAPAKH\0"
  "HEBREW ACCENT MERKHA\0"
  "HEBREW ACCENT MERKHA KEFULA\0"
  "HEBREW ACCENT MUNAH\0"
  "HEBREW ACCENT OLE\0"
  "HEBREW ACCENT PASHTA\0"
  "HEBREW ACCENT PAZER\0"
  "HEBREW ACCENT QADMA\0"
  "HEBREW ACCENT QARNEY PARA\0"
  "HEBREW ACCENT REVIA\0"
  "HEBREW ACCENT SEGOL\0"
  "HEBREW ACCENT SHALSHELET\0"
  "HEBREW ACCENT TELISHA GEDOLA\0"
  "HEBREW ACCENT TELISHA QETANA\0"
  "HEBREW ACCENT TEVIR\0"
  "HEBREW ACCENT TIPEHA\0"
  "HEBREW ACCENT YERAH BEN YOMO\0"
  "HEBREW ACCENT YETIV\0"
  "HEBREW ACCENT ZAQEF GADOL\0"
  "HEBREW ACCENT ZAQEF QATAN\0"
  "HEBREW ACCENT ZARQA\0"
  "HEBREW ACCENT ZINOR\0"
  "HEBREW LETTER ALEF\0"
  "HEBREW LETTER ALEF WITH MAPIQ\0"
  "HEBREW LETTER ALEF WITH PATAH\0"
  "HEBREW LETTER ALEF WITH QAMATS\0"
  "HEBREW LETTER ALTERNATIVE AYIN\0"
  "HEBREW LETTER ALTERNATIVE PLUS SIGN\0"
  "HEBREW LETTER AYIN\0"
  "HEBREW LETTER BET\0"
  "HEBREW LETTER BET WITH DAGESH\0"
  "HEBREW LETTER BET WITH RAFE\0"
  "HEBREW LETTER DALET\0"
  "HEBREW LETTER DALET WITH DAGESH\0"
  "HEBREW LETTER FINAL KAF\0"
  "HEBREW LETTER FINAL KAF WITH DAGESH\0"
  "HEBREW LETTER FINAL MEM\0"
  "HEBREW LETTER FINAL NUN\0"
  "HEBREW LETTER FINAL PE\0"
  "HEBREW LETTER FINAL PE WITH DAGESH\0"
  "HEBREW LETTER FINAL TSADI\0"
  "HEBREW LETTER GIMEL\0"
  "HEBREW LETTER GIMEL WITH DAGESH\0"
  "HEBREW LETTER HE\0"
  "HEBREW LETTER HE WITH MAPIQ\0"
  "HEBREW LETTER HET\0"
  "HEBREW LETTER KAF\0"
  "HEBREW LETTER KAF WITH DAGESH\0"
  "HEBREW LETTER KAF WITH RAFE\0"
  "HEBREW LETTER LAMED\0"
  "HEBREW LETTER LAMED WITH DAGESH\0"
  "HEBREW LETTER MEM\0"
  "HEBREW LETTER MEM WITH DAGESH\0"
  "HEBREW LETTER NUN\0"
  "HEBREW LETTER NUN WITH DAGESH\0"
  "HEBREW LETTER PE\0"
  "HEBREW LETTER PE WITH DAGESH\0"
  "HEBREW LETTER PE WITH RAFE\0"
  "HEBREW LETTER QOF\0"
  "HEBREW LETTER QOF WITH DAGESH\0"
  "HEBREW LETTER RESH\0"
  "HEBREW LETTER RESH WITH DAGESH\0"
  "HEBREW LETTER SAMEKH\0"
  "HEBREW LETTER SAMEKH WITH DAGESH\0"
  "HEBREW LETTER SHIN\0"
  "HEBREW LETTER SHIN WITH DAGESH\0"
  "HEBREW LETTER SHIN WITH DAGESH AND SHIN DOT\0"
  "HEBREW LETTER SHIN WITH DAGESH AND SIN DOT\0"
  "HEBREW LETTER SHIN WITH SHIN DOT\0"
  "HEBREW LETTER SHIN WITH SIN DOT\0"
  "HEBREW LETTER TAV\0"
  "HEBREW LETTER TAV WITH DAGESH\0"
  "HEBREW LETTER TET\0"
  "HEBREW LETTER TET WITH DAGESH\0"
  "HEBREW LETTER TSADI\0"
  "HEBREW LETTER TSADI WITH DAGESH\0"
  "HEBREW LETTER VAV\0"
  "HEBREW LETTER VAV WITH DAGESH\0"
  "HEBREW LETTER VAV WITH HOLAM\0"
  "HEBREW LETTER WIDE ALEF\0"
  "HEBREW LETTER WIDE DALET\0"
  "HEBREW LETTER WIDE FINAL MEM\0"
  "HEBREW LETTER WIDE HE\0"
  "HEBREW LETTER WIDE KAF\0"
  "HEBREW LETTER WIDE LAMED\0"
  "HEBREW LETTER WIDE RESH\0"
  "HEBREW LETTER WIDE TAV\0"
  "HEBREW LETTER YOD\0"
  "HEBREW LETTER YOD WITH DAGESH\0"
  "HEBREW LETTER YOD WITH HIRIQ\0"
  "HEBREW LETTER ZAYIN\0"
  "HEBREW LETTER ZAYIN WITH DAGESH\0"
  "HEBREW LIGATURE ALEF LAMED\0"
  "HEBREW LIGATURE YIDDISH DOUBLE VAV\0"
  "HEBREW LIGATURE YIDDISH DOUBLE YOD\0"
  "HEBREW LIGATURE YIDDISH VAV YOD\0"
  "HEBREW LIGATURE YIDDISH YOD YOD PATAH\0"
  "HEBREW MARK LOWER DOT\0"
  "HEBREW MARK MASORA CIRCLE\0"
  "HEBREW MARK UPPER DOT\0"
  "HEBREW POINT DAGESH OR MAPIQ\0"
  "HEBREW POINT HATAF PATAH\0"
  "HEBREW POINT HATAF QAMATS\0"
  "HEBREW POINT HATAF SEGOL\0"
  "HEBREW POINT HIRIQ\0"
  "HEBREW POINT HOLAM\0"
  "HEBREW POINT HOLAM HASER FOR VAV\0"
  "HEBREW POINT JUDEO-SPANISH VARIKA\0"
  "HEBREW POINT METEG\0"
  "HEBREW POINT PATAH\0"
  "HEBREW POINT QAMATS\0"
  "HEBREW POINT QAMATS QATAN\0"
  "HEBREW POINT QUBUTS\0"
  "HEBREW POINT RAFE\0"
  "HEBREW POINT SEGOL\0"
  "HEBREW POINT SHEVA\0"
  "HEBREW POINT SHIN DOT\0"
  "HEBREW POINT SIN DOT\0"
  "HEBREW POINT TSERE\0"
  "HEBREW PUNCTUATION GERESH\0"
  "HEBREW PUNCTUATION GERSHAYIM\0"
  "HEBREW PUNCTUATION MAQAF\0"
  "HEBREW PUNCTUATION NUN HAFUKHA\0"
  "HEBREW PUNCTUATION PASEQ\0"
  "HEBREW PUNCTUATION SOF PASUQ\0"
  "HEBREW YOD TRIANGLE\0"
  "HEDGEHOG\0"
  "HELICOPTER\0"
  "HELLSCHREIBER PAUSE SYMBOL\0"
  "HELM SYMBOL\0"
  "HELMET WITH WHITE CROSS\0"
  "HENTAIGANA LETTER A-1\0"
  "HENTAIGANA LETTER A-2\0"
  "HENTAIGANA LETTER A-3\0"
  "HENTAIGANA LETTER A-WO\0"
  "HENTAIGANA LETTER E-2\0"
  "HENTAIGANA LETTER E-3\0"
  "HENTAIGANA LETTER E-4\0"
  "HENTAIGANA LETTER E-5\0"
  "HENTAIGANA LETTER E-6\0"
  "HENTAIGANA LETTER HA-1\0"
  "HENTAIGANA LETTER HA-10\0"
  "HENTAIGANA LETTER HA-11\0"
  "HENTAIGANA LETTER HA-2\0"
  "HENTAIGANA LETTER HA-3\0"
  "HENTAIGANA LETTER HA-4\0"
  "HENTAIGANA LETTER HA-5\0"
  "HENTAIGANA LETTER HA-6\0"
  "HENTAIGANA LETTER HA-7\0"
  "HENTAIGANA LETTER HA-8\0"
  "HENTAIGANA LETTER HA-9\0"
  "HENTAIGANA LETTER HE-1\0"
  "HENTAIGANA LETTER HE-2\0"
  "HENTAIGANA LETTER HE-3\0"
  "HENTAIGANA LETTER HE-4\0"
  "HENTAIGANA LETTER HE-5\0"
  "HENTAIGANA LETTER HE-6\0"
  "HENTAIGANA LETTER HE-7\0"
  "HENTAIGANA LETTER HI-1\0"
  "HENTAIGANA LETTER HI-2\0"
  "HENTAIGANA LETTER HI-3\0"
  "HENTAIGANA LETTER HI-4\0"
  "HENTAIGANA LETTER HI-5\0"
  "HENTAIGANA LETTER HI-6\0"
  "HENTAIGANA LETTER HI-7\0"
  "HENTAIGANA LETTER HO-1\0"
  "HENTAIGANA LETTER HO-2\0"
  "HENTAIGANA LETTER HO-3\0"
  "HENTAIGANA LETTER HO-4\0"
  "HENTAIGANA LETTER HO-5\0"
  "HENTAIGANA LETTER HO-6\0"
  "HENTAIGANA LETTER HO-7\0"
  "HENTAIGANA LETTER HO-8\0"
  "HENTAIGANA LETTER HU-1\0"
  "HENTAIGANA LETTER HU-2\0"
  "HENTAIGANA LETTER HU-3\0"
  "HENTAIGANA LETTER I-1\0"
  "HENTAIGANA LETTER I-2\0"
  "HENTAIGANA LETTER I-3\0"
  "HENTAIGANA LETTER I-4\0"
  "HENTAIGANA LETTER KA-1\0"
  "HENTAIGANA LETTER KA-10\0"
  "HENTAIGANA LETTER KA-11\0"
  "HENTAIGANA LETTER KA-2\0"
  "HENTAIGANA LETTER KA-3\0"
  "HENTAIGANA LETTER KA-4\0"
  "HENTAIGANA LETTER KA-5\0"
  "HENTAIGANA LETTER KA-6\0"
  "HENTAIGANA LETTER KA-7\0"
  "HENTAIGANA LETTER KA-8\0"
  "HENTAIGANA LETTER KA-9\0"
  "HENTAIGANA LETTER KA-KE\0"
  "HENTAIGANA LETTER KE-1\0"
  "HENTAIGANA LETTER KE-2\0"
  "HENTAIGANA LETTER KE-3\0"
  "HENTAIGANA LETTER KE-4\0"
  "HENTAIGANA LETTER KE-5\0"
  "HENTAIGANA LETTER KE-6\0"
  "HENTAIGANA LETTER KI-1\0"
  "HENTAIGANA LETTER KI-2\0"
  "HENTAIGANA LETTER KI-3\0"
  "HENTAIGANA LETTER KI-4\0"
  "HENTAIGANA LETTER KI-5\0"
  "HENTAIGANA LETTER KI-6\0"
  "HENTAIGANA LETTER KI-7\0"
  "HENTAIGANA LETTER KI-8\0"
  "HENTAIGANA LETTER KO-1\0"
  "HENTAIGANA LETTER KO-2\0"
  "HENTAIGANA LETTER KO-3\0"
  "HENTAIGANA LETTER KO-KI\0"
  "HENTAIGANA LETTER KU-1\0"
  "HENTAIGANA LETTER KU-2\0"
  "HENTAIGANA LETTER KU-3\0"
  "HENTAIGANA LETTER KU-4\0"
  "HENTAIGANA LETTER KU-5\0"
  "HENTAIGANA LETTER KU-6\0"
  "HENTAIGANA LETTER KU-7\0"
  "HENTAIGANA LETTER MA-1\0"
  "HENTAIGANA LETTER MA-2\0"
  "HENTAIGANA LETTER MA-3\0"
  "HENTAIGANA LETTER MA-4\0"
  "HENTAIGANA LETTER MA-5\0"
  "HENTAIGANA LETTER MA-6\0"
  "HENTAIGANA LETTER MA-7\0"
  "HENTAIGANA LETTER ME-1\0"
  "HENTAIGANA LETTER ME-2\0"
  "HENTAIGANA LETTER ME-MA\0"
  "HENTAIGANA LETTER MI-1\0"
  "HENTAIGANA LETTER MI-2\0"
  "HENTAIGANA LETTER MI-3\0"
  "HENTAIGANA LETTER MI-4\0"
  "HENTAIGANA LETTER MI-5\0"
  "HENTAIGANA LETTER MI-6\0"
  "HENTAIGANA LETTER MI-7\0"
  "HENTAIGANA LETTER MO-1\0"
  "HENTAIGANA LETTER MO-2\0"
  "HENTAIGANA LETTER MO-3\0"
  "HENTAIGANA LETTER MO-4\0"
  "HENTAIGANA LETTER MO-5\0"
  "HENTAIGANA LETTER MO-6\0"
  "HENTAIGANA LETTER MU-1\0"
  "HENTAIGANA LETTER MU-2\0"
  "HENTAIGANA LETTER MU-3\0"
  "HENTAIGANA LETTER MU-4\0"
  "HENTAIGANA LETTER N-MU-MO-1\0"
  "HENTAIGANA LETTER N-MU-MO-2\0"
  "HENTAIGANA LETTER NA-1\0"
  "HENTAIGANA LETTER NA-2\0"
  "HENTAIGANA LETTER NA-3\0"
  "HENTAIGANA LETTER NA-4\0"
  "HENTAIGANA LETTER NA-5\0"
  "HENTAIGANA LETTER NA-6\0"
  "HENTAIGANA LETTER NA-7\0"
  "HENTAIGANA LETTER NA-8\0"
  "HENTAIGANA LETTER NA-9\0"
  "HENTAIGANA LETTER NE-1\0"
  "HENTAIGANA LETTER NE-2\0"
  "HENTAIGANA LETTER NE-3\0"
  "HENTAIGANA LETTER NE-4\0"
  "HENTAIGANA LETTER NE-5\0"
  "HENTAIGANA LETTER NE-6\0"
  "HENTAIGANA LETTER NE-KO\0"
  "HENTAIGANA LETTER NI-1\0"
  "HENTAIGANA LETTER NI-2\0"
  "HENTAIGANA LETTER NI-3\0"
  "HENTAIGANA LETTER NI-4\0"
  "HENTAIGANA LETTER NI-5\0"
  "HENTAIGANA LETTER NI-6\0"
  "HENTAIGANA LETTER NI-7\0"
  "HENTAIGANA LETTER NI-TE\0"
  "HENTAIGANA LETTER NO-1\0"
  "HENTAIGANA LETTER NO-2\0"
  "HENTAIGANA LETTER NO-3\0"
  "HENTAIGANA LETTER NO-4\0"
  "HENTAIGANA LETTER NO-5\0"
  "HENTAIGANA LETTER NU-1\0"
  "HENTAIGANA LETTER NU-2\0"
  "HENTAIGANA LETTER NU-3\0"
  "HENTAIGANA LETTER O-1\0"
  "HENTAIGANA LETTER O-2\0"
  "HENTAIGANA LETTER O-3\0"
  "HENTAIGANA LETTER RA-1\0"
  "HENTAIGANA LETTER RA-2\0"
  "HENTAIGANA LETTER RA-3\0"
  "HENTAIGANA LETTER RA-4\0"
  "HENTAIGANA LETTER RE-1\0"
  "HENTAIGANA LETTER RE-2\0"
  "HENTAIGANA LETTER RE-3\0"
  "HENTAIGANA LETTER RE-4\0"
  "HENTAIGANA LETTER RI-1\0"
  "HENTAIGANA LETTER RI-2\0"
  "HENTAIGANA LETTER RI-3\0"
  "HENTAIGANA LETTER RI-4\0"
  "HENTAIGANA LETTER RI-5\0"
  "HENTAIGANA LETTER RI-6\0"
  "HENTAIGANA LETTER RI-7\0"
  "HENTAIGANA LETTER RO-1\0"
  "HENTAIGANA LETTER RO-2\0"
  "HENTAIGANA LETTER RO-3\0"
  "HENTAIGANA LETTER RO-4\0"
  "HENTAIGANA LETTER RO-5\0"
  "HENTAIGANA LETTER RO-6\0"
  "HENTAIGANA LETTER RU-1\0"
  "HENTAIGANA LETTER RU-2\0"
  "HENTAIGANA LETTER RU-3\0"
  "HENTAIGANA LETTER RU-4\0"
  "HENTAIGANA LETTER RU-5\0"
  "HENTAIGANA LETTER RU-6\0"
  "HENTAIGANA LETTER SA-1\0"
  "HENTAIGANA LETTER SA-2\0"
  "HENTAIGANA LETTER SA-3\0"
  "HENTAIGANA LETTER SA-4\0"
  "HENTAIGANA LETTER SA-5\0"
  "HENTAIGANA LETTER SA-6\0"
  "HENTAIGANA LETTER SA-7\0"
  "HENTAIGANA LETTER SA-8\0"
  "HENTAIGANA LETTER SE-1\0"
  "HENTAIGANA LETTER SE-2\0"
  "HENTAIGANA LETTER SE-3\0"
  "HENTAIGANA LETTER SE-4\0"
  "HENTAIGANA LETTER SE-5\0"
  "HENTAIGANA LETTER SI-1\0"
  "HENTAIGANA LETTER SI-2\0"
  "HENTAIGANA LETTER SI-3\0"
  "HENTAIGANA LETTER SI-4\0"
  "HENTAIGANA LETTER SI-5\0"
  "HENTAIGANA LETTER SI-6\0"
  "HENTAIGANA LETTER SO-1\0"
  "HENTAIGANA LETTER SO-2\0"
  "HENTAIGANA LETTER SO-3\0"
  "HENTAIGANA LETTER SO-4\0"
  "HENTAIGANA LETTER SO-5\0"
  "HENTAIGANA LETTER SO-6\0"
  "HENTAIGANA LETTER SO-7\0"
  "HENTAIGANA LETTER SU-1\0"
  "HENTAIGANA LETTER SU-2\0"
  "HENTAIGANA LETTER SU-3\0"
  "HENTAIGANA LETTER SU-4\0"
  "HENTAIGANA LETTER SU-5\0"
  "HENTAIGANA LETTER SU-6\0"
  "HENTAIGANA LETTER SU-7\0"
  "HENTAIGANA LETTER SU-8\0"
  "HENTAIGANA LETTER TA-1\0"
  "HENTAIGANA LETTER TA-2\0"
  "HENTAIGANA LETTER TA-3\0"
  "HENTAIGANA LETTER TA-4\0"
  "HENTAIGANA LETTER TE-1\0"
  "HENTAIGANA LETTER TE-2\0"
  "HENTAIGANA LETTER TE-3\0"
  "HENTAIGANA LETTER TE-4\0"
  "HENTAIGANA LETTER TE-5\0"
  "HENTAIGANA LETTER TE-6\0"
  "HENTAIGANA LETTER TE-7\0"
  "HENTAIGANA LETTER TE-8\0"
  "HENTAIGANA LETTER TE-9\0"
  "HENTAIGANA LETTER TI-1\0"
  "HENTAIGANA LETTER TI-2\0"
  "HENTAIGANA LETTER TI-3\0"
  "HENTAIGANA LETTER TI-4\0"
  "HENTAIGANA LETTER TI-5\0"
  "HENTAIGANA LETTER TI-6\0"
  "HENTAIGANA LETTER TI-7\0"
  "HENTAIGANA LETTER TO-1\0"
  "HENTAIGANA LETTER TO-2\0"
  "HENTAIGANA LETTER TO-3\0"
  "HENTAIGANA LETTER TO-4\0"
  "HENTAIGANA LETTER TO-5\0"
  "HENTAIGANA LETTER TO-6\0"
  "HENTAIGANA LETTER TO-RA\0"
  "HENTAIGANA LETTER TU-1\0"
  "HENTAIGANA LETTER TU-2\0"
  "HENTAIGANA LETTER TU-3\0"
  "HENTAIGANA LETTER TU-4\0"
  "HENTAIGANA LETTER TU-TO\0"
  "HENTAIGANA LETTER U-1\0"
  "HENTAIGANA LETTER U-2\0"
  "HENTAIGANA LETTER U-3\0"
  "HENTAIGANA LETTER U-4\0"
  "HENTAIGANA LETTER U-5\0"
  "HENTAIGANA LETTER WA-1\0"
  "HENTAIGANA LETTER WA-2\0"
  "HENTAIGANA LETTER WA-3\0"
  "HENTAIGANA LETTER WA-4\0"
  "HENTAIGANA LETTER WA-5\0"
  "HENTAIGANA LETTER WE-1\0"
  "HENTAIGANA LETTER WE-2\0"
  "HENTAIGANA LETTER WE-3\0"
  "HENTAIGANA LETTER WE-4\0"
  "HENTAIGANA LETTER WI-1\0"
  "HENTAIGANA LETTER WI-2\0"
  "HENTAIGANA LETTER WI-3\0"
  "HENTAIGANA LETTER WI-4\0"
  "HENTAIGANA LETTER WI-5\0"
  "HENTAIGANA LETTER WO-1\0"
  "HENTAIGANA LETTER WO-2\0"
  "HENTAIGANA LETTER WO-3\0"
  "HENTAIGANA LETTER WO-4\0"
  "HENTAIGANA LETTER WO-5\0"
  "HENTAIGANA LETTER WO-6\0"
  "HENTAIGANA LETTER WO-7\0"
  "HENTAIGANA LETTER YA-1\0"
  "HENTAIGANA LETTER YA-2\0"
  "HENTAIGANA LETTER YA-3\0"
  "HENTAIGANA LETTER YA-4\0"
  "HENTAIGANA LETTER YA-5\0"
  "HENTAIGANA LETTER YA-YO\0"
  "HENTAIGANA LETTER YO-1\0"
  "HENTAIGANA LETTER YO-2\0"
  "HENTAIGANA LETTER YO-3\0"
  "HENTAIGANA LETTER YO-4\0"
  "HENTAIGANA LETTER YO-5\0"
  "HENTAIGANA LETTER YO-6\0"
  "HENTAIGANA LETTER YU-1\0"
  "HENTAIGANA LETTER YU-2\0"
  "HENTAIGANA LETTER YU-3\0"
  "HENTAIGANA LETTER YU-4\0"
  "HERB\0"
  "HERMITIAN CONJUGATE MATRIX\0"
  "HEXAGRAM FOR ABUNDANCE\0"
  "HEXAGRAM FOR AFTER COMPLETION\0"
  "HEXAGRAM FOR APPROACH\0"
  "HEXAGRAM FOR BEFORE COMPLETION\0"
  "HEXAGRAM FOR BITING THROUGH\0"
  "HEXAGRAM FOR BREAKTHROUGH\0"
  "HEXAGRAM FOR COMING TO MEET\0"
  "HEXAGRAM FOR CONFLICT\0"
  "HEXAGRAM FOR CONTEMPLATION\0"
  "HEXAGRAM FOR DARKENING OF THE LIGHT\0"
  "HEXAGRAM FOR DECREASE\0"
  "HEXAGRAM FOR DELIVERANCE\0"
  "HEXAGRAM FOR DEVELOPMENT\0"
  "HEXAGRAM FOR DIFFICULTY AT THE BEGINNING\0"
  "HEXAGRAM FOR DISPERSION\0"
  "HEXAGRAM FOR DURATION\0"
  "HEXAGRAM FOR ENTHUSIASM\0"
  "HEXAGRAM FOR FELLOWSHIP\0"
  "HEXAGRAM FOR FOLLOWING\0"
  "HEXAGRAM FOR GATHERING TOGETHER\0"
  "HEXAGRAM FOR GRACE\0"
  "HEXAGRAM FOR GREAT POSSESSION\0"
  "HEXAGRAM FOR GREAT POWER\0"
  "HEXAGRAM FOR GREAT PREPONDERANCE\0"
  "HEXAGRAM FOR GREAT TAMING\0"
  "HEXAGRAM FOR HOLDING TOGETHER\0"
  "HEXAGRAM FOR INCREASE\0"
  "HEXAGRAM FOR INFLUENCE\0"
  "HEXAGRAM FOR INNER TRUTH\0"
  "HEXAGRAM FOR INNOCENCE\0"
  "HEXAGRAM FOR LIMITATION\0"
  "HEXAGRAM FOR MODESTY\0"
  "HEXAGRAM FOR MOUTH CORNERS\0"
  "HEXAGRAM FOR OBSTRUCTION\0"
  "HEXAGRAM FOR OPPOSITION\0"
  "HEXAGRAM FOR OPPRESSION\0"
  "HEXAGRAM FOR PEACE\0"
  "HEXAGRAM FOR PROGRESS\0"
  "HEXAGRAM FOR PUSHING UPWARD\0"
  "HEXAGRAM FOR RETREAT\0"
  "HEXAGRAM FOR RETURN\0"
  "HEXAGRAM FOR REVOLUTION\0"
  "HEXAGRAM FOR SMALL PREPONDERANCE\0"
  "HEXAGRAM FOR SMALL TAMING\0"
  "HEXAGRAM FOR SPLITTING APART\0"
  "HEXAGRAM FOR STANDSTILL\0"
  "HEXAGRAM FOR THE ABYSMAL WATER\0"
  "HEXAGRAM FOR THE ARMY\0"
  "HEXAGRAM FOR THE AROUSING THUNDER\0"
  "HEXAGRAM FOR THE CAULDRON\0"
  "HEXAGRAM FOR THE CLINGING FIRE\0"
  "HEXAGRAM FOR THE CREATIVE HEAVEN\0"
  "HEXAGRAM FOR THE FAMILY\0"
  "HEXAGRAM FOR THE GENTLE WIND\0"
  "HEXAGRAM FOR THE JOYOUS LAKE\0"
  "HEXAGRAM FOR THE KEEPING STILL MOUNTAIN\0"
  "HEXAGRAM FOR THE MARRYING MAIDEN\0"
  "HEXAGRAM FOR THE RECEPTIVE EARTH\0"
  "HEXAGRAM FOR THE WANDERER\0"
  "HEXAGRAM FOR THE WELL\0"
  "HEXAGRAM FOR TREADING\0"
  "HEXAGRAM FOR WAITING\0"
  "HEXAGRAM FOR WORK ON THE DECAYED\0"
  "HEXAGRAM FOR YOUTHFUL FOLLY\0"
  "HIBISCUS\0"
  "HIGH BRIGHTNESS SYMBOL\0"
  "HIGH VOLTAGE SIGN\0"
  "HIGH-HEELED SHOE\0"
  "HIGH-SPEED TRAIN\0"
  "HIGH-SPEED TRAIN WITH BULLET NOSE\0"
  "HIKING BOOT\0"
  "HINDU TEMPLE\0"
  "HIPPOPOTAMUS\0"
  "HIRAGANA DIGRAPH YORI\0"
  "HIRAGANA ITERATION MARK\0"
  "HIRAGANA LETTER A\0"
  "HIRAGANA LETTER ARCHAIC WU\0"
  "HIRAGANA LETTER ARCHAIC YE\0"
  "HIRAGANA LETTER BA\0"
  "HIRAGANA LETTER BE\0"
  "HIRAGANA LETTER BI\0"
  "HIRAGANA LETTER BO\0"
  "HIRAGANA LETTER BU\0"
  "HIRAGANA LETTER DA\0"
  "HIRAGANA LETTER DE\0"
  "HIRAGANA LETTER DI\0"
  "HIRAGANA LETTER DO\0"
  "HIRAGANA LETTER DU\0"
  "HIRAGANA LETTER E\0"
  "HIRAGANA LETTER GA\0"
  "HIRAGANA LETTER GE\0"
  "HIRAGANA LETTER GI\0"
  "HIRAGANA LETTER GO\0"
  "HIRAGANA LETTER GU\0"
  "HIRAGANA LETTER HA\0"
  "HIRAGANA LETTER HE\0"
  "HIRAGANA LETTER HI\0"
  "HIRAGANA LETTER HO\0"
  "HIRAGANA LETTER HU\0"
  "HIRAGANA LETTER I\0"
  "HIRAGANA LETTER KA\0"
  "HIRAGANA LETTER KE\0"
  "HIRAGANA LETTER KI\0"
  "HIRAGANA LETTER KO\0"
  "HIRAGANA LETTER KU\0"
  "HIRAGANA LETTER MA\0"
  "HIRAGANA LETTER ME\0"
  "HIRAGANA LETTER MI\0"
  "HIRAGANA LETTER MO\0"
  "HIRAGANA LETTER MU\0"
  "HIRAGANA LETTER N\0"
  "HIRAGANA LETTER NA\0"
  "HIRAGANA LETTER NE\0"
  "HIRAGANA LETTER NI\0"
  "HIRAGANA LETTER NO\0"
  "HIRAGANA LETTER NU\0"
  "HIRAGANA LETTER O\0"
  "HIRAGANA LETTER PA\0"
  "HIRAGANA LETTER PE\0"
  "HIRAGANA LETTER PI\0"
  "HIRAGANA LETTER PO\0"
  "HIRAGANA LETTER PU\0"
  "HIRAGANA LETTER RA\0"
  "HIRAGANA LETTER RE\0"
  "HIRAGANA LETTER RI\0"
  "HIRAGANA LETTER RO\0"
  "HIRAGANA LETTER RU\0"
  "HIRAGANA LETTER SA\0"
  "HIRAGANA LETTER SE\0"
  "HIRAGANA LETTER SI\0"
  "HIRAGANA LETTER SMALL A\0"
  "HIRAGANA LETTER SMALL E\0"
  "HIRAGANA LETTER SMALL I\0"
  "HIRAGANA LETTER SMALL KA\0"
  "HIRAGANA LETTER SMALL KE\0"
  "HIRAGANA LETTER SMALL KO\0"
  "HIRAGANA LETTER SMALL O\0"
  "HIRAGANA LETTER SMALL TU\0"
  "HIRAGANA LETTER SMALL U\0"
  "HIRAGANA LETTER SMALL WA\0"
  "HIRAGANA LETTER SMALL WE\0"
  "HIRAGANA LETTER SMALL WI\0"
  "HIRAGANA LETTER SMALL WO\0"
  "HIRAGANA LETTER SMALL YA\0"
  "HIRAGANA LETTER SMALL YO\0"
  "HIRAGANA LETTER SMALL YU\0"
  "HIRAGANA LETTER SO\0"
  "HIRAGANA LETTER SU\0"
  "HIRAGANA LETTER TA\0"
  "HIRAGANA LETTER TE\0"
  "HIRAGANA LETTER TI\0"
  "HIRAGANA LETTER TO\0"
  "HIRAGANA LETTER TU\0"
  "HIRAGANA LETTER U\0"
  "HIRAGANA LETTER VU\0"
  "HIRAGANA LETTER WA\0"
  "HIRAGANA LETTER WE\0"
  "HIRAGANA LETTER WI\0"
  "HIRAGANA LETTER WO\0"
  "HIRAGANA LETTER YA\0"
  "HIRAGANA LETTER YO\0"
  "HIRAGANA LETTER YU\0"
  "HIRAGANA LETTER ZA\0"
  "HIRAGANA LETTER ZE\0"
  "HIRAGANA LETTER ZI\0"
  "HIRAGANA LETTER ZO\0"
  "HIRAGANA LETTER ZU\0"
  "HIRAGANA VOICED ITERATION MARK\0"
  "HISTORIC SITE\0"
  "HOCHO\0"
  "HOLE\0"
  "HOLLOW QUILT SQUARE ORNAMENT\0"
  "HOLLOW QUILT SQUARE ORNAMENT IN BLACK SQUARE\0"
  "HOMOTHETIC\0"
  "HONEY POT\0"
  "HONEYBEE\0"
  "HOOK\0"
  "HORIZONTAL BAR\0"
  "HORIZONTAL BLACK HEXAGON\0"
  "HORIZONTAL BLACK OCTAGON\0"
  "HORIZONTAL CAPACITOR\0"
  "HORIZONTAL ELLIPSIS\0"
  "HORIZONTAL LADDER\0"
  "HORIZONTAL LINE EXTENSION\0"
  "HORIZONTAL LINE WITH FOUR TICK MARKS\0"
  "HORIZONTAL LINE WITH THREE TICK MARKS\0"
  "HORIZONTAL LINE WITH TICK MARK\0"
  "HORIZONTAL MALE WITH STROKE SIGN\0"
  "HORIZONTAL ONE EIGHTH BLOCK-1358\0"
  "HORIZONTAL ONE EIGHTH BLOCK-2\0"
  "HORIZONTAL ONE EIGHTH BLOCK-3\0"
  "HORIZONTAL ONE EIGHTH BLOCK-4\0"
  "HORIZONTAL ONE EIGHTH BLOCK-5\0"
  "HORIZONTAL ONE EIGHTH BLOCK-6\0"
  "HORIZONTAL ONE EIGHTH BLOCK-7\0"
  "HORIZONTAL RACING CAR\0"
  "HORIZONTAL RESISTOR SEGMENT\0"
  "HORIZONTAL SCAN LINE-1\0"
  "HORIZONTAL SCAN LINE-3\0"
  "HORIZONTAL SCAN LINE-7\0"
  "HORIZONTAL SCAN LINE-9\0"
  "HORIZONTAL TAB KEY\0"
  "HORIZONTAL TRAFFIC LIGHT\0"
  "HORIZONTAL ZIGZAG LINE\0"
  "HORSE\0"
  "HORSE FACE\0"
  "HORSE RACING\0"
  "HOSPITAL\0"
  "HOT BEVERAGE\0"
  "HOT DOG\0"
  "HOT PEPPER\0"
  "HOT SPRINGS\0"
  "HOTEL\0"
  "HOURGLASS\0"
  "HOURGLASS WITH FLOWING SAND\0"
  "HOUSE\0"
  "HOUSE BUILDING\0"
  "HOUSE BUILDINGS\0"
  "HOUSE WITH GARDEN\0"
  "HRYVNIA SIGN\0"
  "HUGGING FACE\0"
  "HUNDRED POINTS SYMBOL\0"
  "HUSHED FACE\0"
  "HUT\0"
  "HYACINTH\0"
  "HYGIEA\0"
  "HYPHEN\0"
  "HYPHEN BULLET\0"
  "HYPHEN WITH DIAERESIS\0"
  "HYPHEN-MINUS\0"
  "HYPHENATION POINT\0"
  "HYPODIASTOLE\0"
  "HYSTERESIS SYMBOL\0"
  "I LOVE YOU HAND SIGN\0"
  "ICE CREAM\0"
  "ICE CUBE\0"
  "ICE HOCKEY STICK AND PUCK\0"
  "ICE SKATE\0"
  "IDENTICAL TO\0"
  "IDENTICAL TO AND SLANTED PARALLEL\0"
  "IDENTICAL WITH DOT ABOVE\0"
  "IDENTIFICATION CARD\0"
  "IDEOGRAPHIC ANNOTATION BOTTOM MARK\0"
  "IDEOGRAPHIC ANNOTATION EARTH MARK\0"
  "IDEOGRAPHIC ANNOTATION FIRST MARK\0"
  "IDEOGRAPHIC ANNOTATION FOUR MARK\0"
  "IDEOGRAPHIC ANNOTATION FOURTH MARK\0"
  "IDEOGRAPHIC ANNOTATION HEAVEN MARK\0"
  "IDEOGRAPHIC ANNOTATION LINKING MARK\0"
  "IDEOGRAPHIC ANNOTATION MAN MARK\0"
  "IDEOGRAPHIC ANNOTATION MIDDLE MARK\0"
  "IDEOGRAPHIC ANNOTATION ONE MARK\0"
  "IDEOGRAPHIC ANNOTATION REVERSE MARK\0"
  "IDEOGRAPHIC ANNOTATION SECOND MARK\0"
  "IDEOGRAPHIC ANNOTATION THIRD MARK\0"
  "IDEOGRAPHIC ANNOTATION THREE MARK\0"
  "IDEOGRAPHIC ANNOTATION TOP MARK\0"
  "IDEOGRAPHIC ANNOTATION TWO MARK\0"
  "IDEOGRAPHIC CLOSING MARK\0"
  "IDEOGRAPHIC COMMA\0"
  "IDEOGRAPHIC DEPARTING TONE MARK\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER ABOVE TO BELOW\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER ABOVE TO MIDDLE AND BELOW\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER FULL SURROUND\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER HORIZONTAL REFLECTION\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER LEFT TO MIDDLE AND RIGHT\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER LEFT TO RIGHT\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER OVERLAID\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER ROTATION\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER SUBTRACTION\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM ABOVE\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM BELOW\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LEFT\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LOWER LEFT\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LOWER RIGHT\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM RIGHT\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM UPPER LEFT\0"
  "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM UPPER RIGHT\0"
  "IDEOGRAPHIC ENTERING TONE MARK\0"
  "IDEOGRAPHIC FULL STOP\0"
  "IDEOGRAPHIC HALF FILL SPACE\0"
  "IDEOGRAPHIC ITERATION MARK\0"
  "IDEOGRAPHIC LEVEL TONE MARK\0"
  "IDEOGRAPHIC NUMBER ZERO\0"
  "IDEOGRAPHIC RISING TONE MARK\0"
  "IDEOGRAPHIC SPACE\0"
  "IDEOGRAPHIC TALLY MARK FIVE\0"
  "IDEOGRAPHIC TALLY MARK FOUR\0"
  "IDEOGRAPHIC TALLY MARK ONE\0"
  "IDEOGRAPHIC TALLY MARK THREE\0"
  "IDEOGRAPHIC TALLY MARK TWO\0"
  "IDEOGRAPHIC TELEGRAPH LINE FEED SEPARATOR SYMBOL\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR APRIL\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR AUGUST\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHT\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ELEVEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIFTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIVE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOUR\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOURTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINETEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ONE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVENTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIX\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIXTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY-ONE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THREE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWELVE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-EIGHT\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FIVE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FOUR\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-NINE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-ONE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SEVEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SIX\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-THREE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-TWO\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWO\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DECEMBER\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR FEBRUARY\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHT\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ELEVEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIFTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIVE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOUR\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOURTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINETEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ONE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVENTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIX\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIXTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THIRTEEN\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THREE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWELVE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-FOUR\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-ONE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-THREE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-TWO\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWO\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ZERO\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR JANUARY\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR JULY\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR JUNE\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR MARCH\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR MAY\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR NOVEMBER\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR OCTOBER\0"
  "IDEOGRAPHIC TELEGRAPH SYMBOL FOR SEPTEMBER\0"
  "IDEOGRAPHIC VARIATION INDICATOR\0"
  "IMAGE OF\0"
  "IMAGE OF OR APPROXIMATELY EQUAL TO\0"
  "IMP\0"
  "IMPERIAL ARAMAIC LETTER ALEPH\0"
  "IMPERIAL ARAMAIC LETTER AYIN\0"
  "IMPERIAL ARAMAIC LETTER BETH\0"
  "IMPERIAL ARAMAIC LETTER DALETH\0"
  "IMPERIAL ARAMAIC LETTER GIMEL\0"
  "IMPERIAL ARAMAIC LETTER HE\0"
  "IMPERIAL ARAMAIC LETTER HETH\0"
  "IMPERIAL ARAMAIC LETTER KAPH\0"
  "IMPERIAL ARAMAIC LETTER LAMEDH\0"
  "IMPERIAL ARAMAIC LETTER MEM\0"
  "IMPERIAL ARAMAIC LETTER NUN\0"
  "IMPERIAL ARAMAIC LETTER PE\0"
  "IMPERIAL ARAMAIC LETTER QOPH\0"
  "IMPERIAL ARAMAIC LETTER RESH\0"
  "IMPERIAL ARAMAIC LETTER SADHE\0"
  "IMPERIAL ARAMAIC LETTER SAMEKH\0"
  "IMPERIAL ARAMAIC LETTER SHIN\0"
  "IMPERIAL ARAMAIC LETTER TAW\0"
  "IMPERIAL ARAMAIC LETTER TETH\0"
  "IMPERIAL ARAMAIC LETTER WAW\0"
  "IMPERIAL ARAMAIC LETTER YODH\0"
  "IMPERIAL ARAMAIC LETTER ZAYIN\0"
  "IMPERIAL ARAMAIC NUMBER ONE\0"
  "IMPERIAL ARAMAIC NUMBER ONE HUNDRED\0"
  "IMPERIAL ARAMAIC NUMBER ONE THOUSAND\0"
  "IMPERIAL ARAMAIC NUMBER TEN\0"
  "IMPERIAL ARAMAIC NUMBER TEN THOUSAND\0"
  "IMPERIAL ARAMAIC NUMBER THREE\0"
  "IMPERIAL ARAMAIC NUMBER TWENTY\0"
  "IMPERIAL ARAMAIC NUMBER TWO\0"
  "IMPERIAL ARAMAIC SECTION SIGN\0"
  "INBOX TRAY\0"
  "INCOMING ENVELOPE\0"
  "INCOMPLETE INFINITY\0"
  "INCREASE FONT SIZE SYMBOL\0"
  "INCREASES AS\0"
  "INCREMENT\0"
  "INDEX POINTING AT THE VIEWER\0"
  "INDIAN RUPEE SIGN\0"
  "INDIC SIYAQ ALTERNATE LAKH MARK\0"
  "INDIC SIYAQ FRACTION ONE HALF\0"
  "INDIC SIYAQ FRACTION ONE QUARTER\0"
  "INDIC SIYAQ FRACTION THREE QUARTERS\0"
  "INDIC SIYAQ LAKH MARK\0"
  "INDIC SIYAQ NUMBER ALTERNATE ONE\0"
  "INDIC SIYAQ NUMBER ALTERNATE TEN THOUSAND\0"
  "INDIC SIYAQ NUMBER ALTERNATE TWO\0"
  "INDIC SIYAQ NUMBER EIGHT\0"
  "INDIC SIYAQ NUMBER EIGHT HUNDRED\0"
  "INDIC SIYAQ NUMBER EIGHT THOUSAND\0"
  "INDIC SIYAQ NUMBER EIGHTY\0"
  "INDIC SIYAQ NUMBER EIGHTY THOUSAND\0"
  "INDIC SIYAQ NUMBER FIFTY\0"
  "INDIC SIYAQ NUMBER FIFTY THOUSAND\0"
  "INDIC SIYAQ NUMBER FIVE\0"
  "INDIC SIYAQ NUMBER FIVE HUNDRED\0"
  "INDIC SIYAQ NUMBER FIVE THOUSAND\0"
  "INDIC SIYAQ NUMBER FORTY\0"
  "INDIC SIYAQ NUMBER FORTY THOUSAND\0"
  "INDIC SIYAQ NUMBER FOUR\0"
  "INDIC SIYAQ NUMBER FOUR HUNDRED\0"
  "INDIC SIYAQ NUMBER FOUR THOUSAND\0"
  "INDIC SIYAQ NUMBER KAROR\0"
  "INDIC SIYAQ NUMBER KARORAN\0"
  "INDIC SIYAQ NUMBER LAKH\0"
  "INDIC SIYAQ NUMBER LAKHAN\0"
  "INDIC SIYAQ NUMBER NINE\0"
  "INDIC SIYAQ NUMBER NINE HUNDRED\0"
  "INDIC SIYAQ NUMBER NINE THOUSAND\0"
  "INDIC SIYAQ NUMBER NINETY\0"
  "INDIC SIYAQ NUMBER NINETY THOUSAND\0"
  "INDIC SIYAQ NUMBER ONE\0"
  "INDIC SIYAQ NUMBER ONE HUNDRED\0"
  "INDIC SIYAQ NUMBER ONE THOUSAND\0"
  "INDIC SIYAQ NUMBER PREFIXED EIGHT\0"
  "INDIC SIYAQ NUMBER PREFIXED FIVE\0"
  "INDIC SIYAQ NUMBER PREFIXED FOUR\0"
  "INDIC SIYAQ NUMBER PREFIXED NINE\0"
  "INDIC SIYAQ NUMBER PREFIXED ONE\0"
  "INDIC SIYAQ NUMBER PREFIXED SEVEN\0"
  "INDIC SIYAQ NUMBER PREFIXED SIX\0"
  "INDIC SIYAQ NUMBER PREFIXED THREE\0"
  "INDIC SIYAQ NUMBER PREFIXED TWO\0"
  "INDIC SIYAQ NUMBER SEVEN\0"
  "INDIC SIYAQ NUMBER SEVEN HUNDRED\0"
  "INDIC SIYAQ NUMBER SEVEN THOUSAND\0"
  "INDIC SIYAQ NUMBER SEVENTY\0"
  "INDIC SIYAQ NUMBER SEVENTY THOUSAND\0"
  "INDIC SIYAQ NUMBER SIX\0"
  "INDIC SIYAQ NUMBER SIX HUNDRED\0"
  "INDIC SIYAQ NUMBER SIX THOUSAND\0"
  "INDIC SIYAQ NUMBER SIXTY\0"
  "INDIC SIYAQ NUMBER SIXTY THOUSAND\0"
  "INDIC SIYAQ NUMBER TEN\0"
  "INDIC SIYAQ NUMBER TEN THOUSAND\0"
  "INDIC SIYAQ NUMBER THIRTY\0"
  "INDIC SIYAQ NUMBER THIRTY THOUSAND\0"
  "INDIC SIYAQ NUMBER THREE\0"
  "INDIC SIYAQ NUMBER THREE HUNDRED\0"
  "INDIC SIYAQ NUMBER THREE THOUSAND\0"
  "INDIC SIYAQ NUMBER TWENTY\0"
  "INDIC SIYAQ NUMBER TWENTY THOUSAND\0"
  "INDIC SIYAQ NUMBER TWO\0"
  "INDIC SIYAQ NUMBER TWO HUNDRED\0"
  "INDIC SIYAQ NUMBER TWO THOUSAND\0"
  "INDIC SIYAQ PLACEHOLDER\0"
  "INDIC SIYAQ RUPEE MARK\0"
  "INFINITY\0"
  "INFINITY NEGATED WITH VERTICAL BAR\0"
  "INFORMATION DESK PERSON\0"
  "INFORMATION SOURCE\0"
  "INHIBIT ARABIC FORM SHAPING\0"
  "INHIBIT SYMMETRIC SWAPPING\0"
  "INPUT SYMBOL FOR LATIN CAPITAL LETTERS\0"
  "INPUT SYMBOL FOR LATIN LETTERS\0"
  "INPUT SYMBOL FOR LATIN SMALL LETTERS\0"
  "INPUT SYMBOL FOR NUMBERS\0"
  "INPUT SYMBOL FOR SYMBOLS\0"
  "INSCRIPTIONAL PAHLAVI LETTER ALEPH\0"
  "INSCRIPTIONAL PAHLAVI LETTER BETH\0"
  "INSCRIPTIONAL PAHLAVI LETTER DALETH\0"
  "INSCRIPTIONAL PAHLAVI LETTER GIMEL\0"
  "INSCRIPTIONAL PAHLAVI LETTER HE\0"
  "INSCRIPTIONAL PAHLAVI LETTER HETH\0"
  "INSCRIPTIONAL PAHLAVI LETTER KAPH\0"
  "INSCRIPTIONAL PAHLAVI LETTER LAMEDH\0"
  "INSCRIPTIONAL PAHLAVI LETTER MEM-QOPH\0"
  "INSCRIPTIONAL PAHLAVI LETTER NUN\0"
  "INSCRIPTIONAL PAHLAVI LETTER PE\0"
  "INSCRIPTIONAL PAHLAVI LETTER SADHE\0"
  "INSCRIPTIONAL PAHLAVI LETTER SAMEKH\0"
  "INSCRIPTIONAL PAHLAVI LETTER SHIN\0"
  "INSCRIPTIONAL PAHLAVI LETTER TAW\0"
  "INSCRIPTIONAL PAHLAVI LETTER TETH\0"
  "INSCRIPTIONAL PAHLAVI LETTER WAW-AYIN-RESH\0"
  "INSCRIPTIONAL PAHLAVI LETTER YODH\0"
  "INSCRIPTIONAL PAHLAVI LETTER ZAYIN\0"
  "INSCRIPTIONAL PAHLAVI NUMBER FOUR\0"
  "INSCRIPTIONAL PAHLAVI NUMBER ONE\0"
  "INSCRIPTIONAL PAHLAVI NUMBER ONE HUNDRED\0"
  "INSCRIPTIONAL PAHLAVI NUMBER ONE THOUSAND\0"
  "INSCRIPTIONAL PAHLAVI NUMBER TEN\0"
  "INSCRIPTIONAL PAHLAVI NUMBER THREE\0"
  "INSCRIPTIONAL PAHLAVI NUMBER TWENTY\0"
  "INSCRIPTIONAL PAHLAVI NUMBER TWO\0"
  "INSCRIPTIONAL PARTHIAN LETTER ALEPH\0"
  "INSCRIPTIONAL PARTHIAN LETTER AYIN\0"
  "INSCRIPTIONAL PARTHIAN LETTER BETH\0"
  "INSCRIPTIONAL PARTHIAN LETTER DALETH\0"
  "INSCRIPTIONAL PARTHIAN LETTER GIMEL\0"
  "INSCRIPTIONAL PARTHIAN LETTER HE\0"
  "INSCRIPTIONAL PARTHIAN LETTER HETH\0"
  "INSCRIPTIONAL PARTHIAN LETTER KAPH\0"
  "INSCRIPTIONAL PARTHIAN LETTER LAMEDH\0"
  "INSCRIPTIONAL PARTHIAN LETTER MEM\0"
  "INSCRIPTIONAL PARTHIAN LETTER NUN\0"
  "INSCRIPTIONAL PARTHIAN LETTER PE\0"
  "INSCRIPTIONAL PARTHIAN LETTER QOPH\0"
  "INSCRIPTIONAL PARTHIAN LETTER RESH\0"
  "INSCRIPTIONAL PARTHIAN LETTER SADHE\0"
  "INSCRIPTIONAL PARTHIAN LETTER SAMEKH\0"
  "INSCRIPTIONAL PARTHIAN LETTER SHIN\0"
  "INSCRIPTIONAL PARTHIAN LETTER TAW\0"
  "INSCRIPTIONAL PARTHIAN LETTER TETH\0"
  "INSCRIPTIONAL PARTHIAN LETTER WAW\0"
  "INSCRIPTIONAL PARTHIAN LETTER YODH\0"
  "INSCRIPTIONAL PARTHIAN LETTER ZAYIN\0"
  "INSCRIPTIONAL PARTHIAN NUMBER FOUR\0"
  "INSCRIPTIONAL PARTHIAN NUMBER ONE\0"
  "INSCRIPTIONAL PARTHIAN NUMBER ONE HUNDRED\0"
  "INSCRIPTIONAL PARTHIAN NUMBER ONE THOUSAND\0"
  "INSCRIPTIONAL PARTHIAN NUMBER TEN\0"
  "INSCRIPTIONAL PARTHIAN NUMBER THREE\0"
  "INSCRIPTIONAL PARTHIAN NUMBER TWENTY\0"
  "INSCRIPTIONAL PARTHIAN NUMBER TWO\0"
  "INSERTION SYMBOL\0"
  "INTEGRAL\0"
  "INTEGRAL AROUND A POINT OPERATOR\0"
  "INTEGRAL AVERAGE WITH SLASH\0"
  "INTEGRAL EXTENSION\0"
  "INTEGRAL WITH DOUBLE STROKE\0"
  "INTEGRAL WITH INTERSECTION\0"
  "INTEGRAL WITH LEFTWARDS ARROW WITH HOOK\0"
  "INTEGRAL WITH OVERBAR\0"
  "INTEGRAL WITH TIMES SIGN\0"
  "INTEGRAL WITH UNDERBAR\0"
  "INTEGRAL WITH UNION\0"
  "INTERCALATE\0"
  "INTERIOR PRODUCT\0"
  "INTERLINEAR ANNOTATION ANCHOR\0"
  "INTERLINEAR ANNOTATION SEPARATOR\0"
  "INTERLINEAR ANNOTATION TERMINATOR\0"
  "INTERLOCKED FEMALE AND MALE SIGN\0"
  "INTERROBANG\0"
  "INTERSECTION\0"
  "INTERSECTION ABOVE BAR ABOVE UNION\0"
  "INTERSECTION ABOVE UNION\0"
  "INTERSECTION BESIDE AND JOINED WITH INTERSECTION\0"
  "INTERSECTION WITH DOT\0"
  "INTERSECTION WITH LOGICAL AND\0"
  "INTERSECTION WITH OVERBAR\0"
  "INVERSE BLACK DIAMOND\0"
  "INVERSE BLACK SMALL SQUARE\0"
  "INVERSE BULLET\0"
  "INVERSE CHECK MARK\0"
  "INVERSE CHECKER BOARD FILL\0"
  "INVERSE DOWNWARDS ARROW WITH TIP LEFTWARDS\0"
  "INVERSE MEDIUM SHADE\0"
  "INVERSE WHITE CIRCLE\0"
  "INVERTED EXCLAMATION MARK\0"
  "INVERTED INTERROBANG\0"
  "INVERTED LAZY S\0"
  "INVERTED LOW KAVYKA\0"
  "INVERTED LOW KAVYKA WITH KAVYKA ABOVE\0"
  "INVERTED OHM SIGN\0"
  "INVERTED PENTAGRAM\0"
  "INVERTED QUESTION MARK\0"
  "INVERTED UNDERTIE\0"
  "INVISIBLE PLUS\0"
  "INVISIBLE SEPARATOR\0"
  "INVISIBLE TIMES\0"
  "IZAKAYA LANTERN\0"
  "JACK-O-LANTERN\0"
  "JAPANESE BANK SYMBOL\0"
  "JAPANESE CASTLE\0"
  "JAPANESE DOLLS\0"
  "JAPANESE GOBLIN\0"
  "JAPANESE INDUSTRIAL STANDARD SYMBOL\0"
  "JAPANESE OGRE\0"
  "JAPANESE POST OFFICE\0"
  "JAPANESE SYMBOL FOR BEGINNER\0"
  "JAR\0"
  "JAVANESE CONSONANT SIGN CAKRA\0"
  "JAVANESE CONSONANT SIGN KERET\0"
  "JAVANESE CONSONANT SIGN PENGKAL\0"
  "JAVANESE DIGIT EIGHT\0"
  "JAVANESE DIGIT FIVE\0"
  "JAVANESE DIGIT FOUR\0"
  "JAVANESE DIGIT NINE\0"
  "JAVANESE DIGIT ONE\0"
  "JAVANESE DIGIT SEVEN\0"
  "JAVANESE DIGIT SIX\0"
  "JAVANESE DIGIT THREE\0"
  "JAVANESE DIGIT TWO\0"
  "JAVANESE DIGIT ZERO\0"
  "JAVANESE LEFT RERENGGAN\0"
  "JAVANESE LETTER A\0"
  "JAVANESE LETTER AI\0"
  "JAVANESE LETTER BA\0"
  "JAVANESE LETTER BA MURDA\0"
  "JAVANESE LETTER CA\0"
  "JAVANESE LETTER CA MURDA\0"
  "JAVANESE LETTER DA\0"
  "JAVANESE LETTER DA MAHAPRANA\0"
  "JAVANESE LETTER DDA\0"
  "JAVANESE LETTER DDA MAHAPRANA\0"
  "JAVANESE LETTER E\0"
  "JAVANESE LETTER GA\0"
  "JAVANESE LETTER GA MURDA\0"
  "JAVANESE LETTER HA\0"
  "JAVANESE LETTER I\0"
  "JAVANESE LETTER I KAWI\0"
  "JAVANESE LETTER II\0"
  "JAVANESE LETTER JA\0"
  "JAVANESE LETTER JA MAHAPRANA\0"
  "JAVANESE LETTER KA\0"
  "JAVANESE LETTER KA MURDA\0"
  "JAVANESE LETTER KA SASAK\0"
  "JAVANESE LETTER LA\0"
  "JAVANESE LETTER MA\0"
  "JAVANESE LETTER NA\0"
  "JAVANESE LETTER NA MURDA\0"
  "JAVANESE LETTER NGA\0"
  "JAVANESE LETTER NGA LELET\0"
  "JAVANESE LETTER NGA LELET RASWADI\0"
  "JAVANESE LETTER NYA\0"
  "JAVANESE LETTER NYA MURDA\0"
  "JAVANESE LETTER O\0"
  "JAVANESE LETTER PA\0"
  "JAVANESE LETTER PA CEREK\0"
  "JAVANESE LETTER PA MURDA\0"
  "JAVANESE LETTER RA\0"
  "JAVANESE LETTER RA AGUNG\0"
  "JAVANESE LETTER SA\0"
  "JAVANESE LETTER SA MAHAPRANA\0"
  "JAVANESE LETTER SA MURDA\0"
  "JAVANESE LETTER TA\0"
  "JAVANESE LETTER TA MURDA\0"
  "JAVANESE LETTER TTA\0"
  "JAVANESE LETTER TTA MAHAPRANA\0"
  "JAVANESE LETTER U\0"
  "JAVANESE LETTER WA\0"
  "JAVANESE LETTER YA\0"
  "JAVANESE PADA ADEG\0"
  "JAVANESE PADA ADEG ADEG\0"
  "JAVANESE PADA ANDAP\0"
  "JAVANESE PADA ISEN-ISEN\0"
  "JAVANESE PADA LINGSA\0"
  "JAVANESE PADA LUHUR\0"
  "JAVANESE PADA LUNGSI\0"
  "JAVANESE PADA MADYA\0"
  "JAVANESE PADA PANGKAT\0"
  "JAVANESE PADA PISELEH\0"
  "JAVANESE PADA TIRTA TUMETES\0"
  "JAVANESE PADA WINDU\0"
  "JAVANESE PANGKON\0"
  "JAVANESE PANGRANGKEP\0"
  "JAVANESE RIGHT RERENGGAN\0"
  "JAVANESE SIGN CECAK\0"
  "JAVANESE SIGN CECAK TELU\0"
  "JAVANESE SIGN LAYAR\0"
  "JAVANESE SIGN PANYANGGA\0"
  "JAVANESE SIGN WIGNYAN\0"
  "JAVANESE TURNED PADA PISELEH\0"
  "JAVANESE VOWEL SIGN DIRGA MURE\0"
  "JAVANESE VOWEL SIGN PEPET\0"
  "JAVANESE VOWEL SIGN SUKU\0"
  "JAVANESE VOWEL SIGN SUKU MENDUT\0"
  "JAVANESE VOWEL SIGN TALING\0"
  "JAVANESE VOWEL SIGN TARUNG\0"
  "JAVANESE VOWEL SIGN TOLONG\0"
  "JAVANESE VOWEL SIGN WULU\0"
  "JAVANESE VOWEL SIGN WULU MELIK\0"
  "JEANS\0"
  "JELLYFISH\0"
  "JIGSAW PUZZLE PIECE\0"
  "JOIN\0"
  "JOYSTICK\0"
  "JUGGLING\0"
  "JUNO\0"
  "JUPITER\0"
  "KAABA\0"
  "KAITHI ABBREVIATION SIGN\0"
  "KAITHI DANDA\0"
  "KAITHI DOUBLE DANDA\0"
  "KAITHI DOUBLE SECTION MARK\0"
  "KAITHI ENUMERATION SIGN\0"
  "KAITHI LETTER A\0"
  "KAITHI LETTER AA\0"
  "KAITHI LETTER AI\0"
  "KAITHI LETTER AU\0"
  "KAITHI LETTER BA\0"
  "KAITHI LETTER BHA\0"
  "KAITHI LETTER CA\0"
  "KAITHI LETTER CHA\0"
  "KAITHI LETTER DA\0"
  "KAITHI LETTER DDA\0"
  "KAITHI LETTER DDDHA\0"
  "KAITHI LETTER DDHA\0"
  "KAITHI LETTER DHA\0"
  "KAITHI LETTER E\0"
  "KAITHI LETTER GA\0"
  "KAITHI LETTER GHA\0"
  "KAITHI LETTER HA\0"
  "KAITHI LETTER I\0"
  "KAITHI LETTER II\0"
  "KAITHI LETTER JA\0"
  "KAITHI LETTER JHA\0"
  "KAITHI LETTER KA\0"
  "KAITHI LETTER KHA\0"
  "KAITHI LETTER LA\0"
  "KAITHI LETTER MA\0"
  "KAITHI LETTER NA\0"
  "KAITHI LETTER NGA\0"
  "KAITHI LETTER NNA\0"
  "KAITHI LETTER NYA\0"
  "KAITHI LETTER O\0"
  "KAITHI LETTER PA\0"
  "KAITHI LETTER PHA\0"
  "KAITHI LETTER RA\0"
  "KAITHI LETTER RHA\0"
  "KAITHI LETTER SA\0"
  "KAITHI LETTER SHA\0"
  "KAITHI LETTER SSA\0"
  "KAITHI LETTER TA\0"
  "KAITHI LETTER THA\0"
  "KAITHI LETTER TTA\0"
  "KAITHI LETTER TTHA\0"
  "KAITHI LETTER U\0"
  "KAITHI LETTER UU\0"
  "KAITHI LETTER VA\0"
  "KAITHI LETTER YA\0"
  "KAITHI NUMBER SIGN\0"
  "KAITHI NUMBER SIGN ABOVE\0"
  "KAITHI SECTION MARK\0"
  "KAITHI SIGN ANUSVARA\0"
  "KAITHI SIGN CANDRABINDU\0"
  "KAITHI SIGN NUKTA\0"
  "KAITHI SIGN VIRAMA\0"
  "KAITHI SIGN VISARGA\0"
  "KAITHI VOWEL SIGN AA\0"
  "KAITHI VOWEL SIGN AI\0"
  "KAITHI VOWEL SIGN AU\0"
  "KAITHI VOWEL SIGN E\0"
  "KAITHI VOWEL SIGN I\0"
  "KAITHI VOWEL SIGN II\0"
  "KAITHI VOWEL SIGN O\0"
  "KAITHI VOWEL SIGN U\0"
  "KAITHI VOWEL SIGN UU\0"
  "KAITHI VOWEL SIGN VOCALIC R\0"
  "KAKTOVIK NUMERAL EIGHT\0"
  "KAKTOVIK NUMERAL EIGHTEEN\0"
  "KAKTOVIK NUMERAL ELEVEN\0"
  "KAKTOVIK NUMERAL FIFTEEN\0"
  "KAKTOVIK NUMERAL FIVE\0"
  "KAKTOVIK NUMERAL FOUR\0"
  "KAKTOVIK NUMERAL FOURTEEN\0"
  "KAKTOVIK NUMERAL NINE\0"
  "KAKTOVIK NUMERAL NINETEEN\0"
  "KAKTOVIK NUMERAL ONE\0"
  "KAKTOVIK NUMERAL SEVEN\0"
  "KAKTOVIK NUMERAL SEVENTEEN\0"
  "KAKTOVIK NUMERAL SIX\0"
  "KAKTOVIK NUMERAL SIXTEEN\0"
  "KAKTOVIK NUMERAL TEN\0"
  "KAKTOVIK NUMERAL THIRTEEN\0"
  "KAKTOVIK NUMERAL THREE\0"
  "KAKTOVIK NUMERAL TWELVE\0"
  "KAKTOVIK NUMERAL TWO\0"
  "KAKTOVIK NUMERAL ZERO\0"
  "KANGAROO\0"
  "KANGXI RADICAL AGAIN\0"
  "KANGXI RADICAL AND\0"
  "KANGXI RADICAL ARRIVE\0"
  "KANGXI RADICAL ARROW\0"
  "KANGXI RADICAL AXE\0"
  "KANGXI RADICAL BADGER\0"
  "KANGXI RADICAL BAMBOO\0"
  "KANGXI RADICAL BEAN\0"
  "KANGXI RADICAL BIG\0"
  "KANGXI RADICAL BIRD\0"
  "KANGXI RADICAL BITTER\0"
  "KANGXI RADICAL BLACK\0"
  "KANGXI RADICAL BLOOD\0"
  "KANGXI RADICAL BLUE\0"
  "KANGXI RADICAL BOAT\0"
  "KANGXI RADICAL BODY\0"
  "KANGXI RADICAL BOLT OF CLOTH\0"
  "KANGXI RADICAL BONE\0"
  "KANGXI RADICAL BOW\0"
  "KANGXI RADICAL BRANCH\0"
  "KANGXI RADICAL BRISTLE\0"
  "KANGXI RADICAL BRUSH\0"
  "KANGXI RADICAL CART\0"
  "KANGXI RADICAL CAULDRON\0"
  "KANGXI RADICAL CAVE\0"
  "KANGXI RADICAL CHILD\0"
  "KANGXI RADICAL CITY\0"
  "KANGXI RADICAL CLAN\0"
  "KANGXI RADICAL CLAW\0"
  "KANGXI RADICAL CLIFF\0"
  "KANGXI RADICAL CLOTHES\0"
  "KANGXI RADICAL COLOR\0"
  "KANGXI RADICAL COMPARE\0"
  "KANGXI RADICAL CORPSE\0"
  "KANGXI RADICAL COVER\0"
  "KANGXI RADICAL COW\0"
  "KANGXI RADICAL DEATH\0"
  "KANGXI RADICAL DEER\0"
  "KANGXI RADICAL DIPPER\0"
  "KANGXI RADICAL DISH\0"
  "KANGXI RADICAL DISTINGUISH\0"
  "KANGXI RADICAL DIVINATION\0"
  "KANGXI RADICAL DO NOT\0"
  "KANGXI RADICAL DOG\0"
  "KANGXI RADICAL DOOR\0"
  "KANGXI RADICAL DOT\0"
  "KANGXI RADICAL DOTTED CLIFF\0"
  "KANGXI RADICAL DOTTED TENT\0"
  "KANGXI RADICAL DOUBLE X\0"
  "KANGXI RADICAL DOWN BOX\0"
  "KANGXI RADICAL DRAGON\0"
  "KANGXI RADICAL DRUM\0"
  "KANGXI RADICAL DRY\0"
  "KANGXI RADICAL EAR\0"
  "KANGXI RADICAL EARTH\0"
  "KANGXI RADICAL EAT\0"
  "KANGXI RADICAL EIGHT\0"
  "KANGXI RADICAL EMBROIDERY\0"
  "KANGXI RADICAL ENCLOSURE\0"
  "KANGXI RADICAL ENTER\0"
  "KANGXI RADICAL EVEN\0"
  "KANGXI RADICAL EVENING\0"
  "KANGXI RADICAL EYE\0"
  "KANGXI RADICAL FACE\0"
  "KANGXI RADICAL FANG\0"
  "KANGXI RADICAL FATHER\0"
  "KANGXI RADICAL FEATHER\0"
  "KANGXI RADICAL FIELD\0"
  "KANGXI RADICAL FIGHT\0"
  "KANGXI RADICAL FIRE\0"
  "KANGXI RADICAL FISH\0"
  "KANGXI RADICAL FLUTE\0"
  "KANGXI RADICAL FLY\0"
  "KANGXI RADICAL FOOT\0"
  "KANGXI RADICAL FRAGRANT\0"
  "KANGXI RADICAL FROG\0"
  "KANGXI RADICAL FUR\0"
  "KANGXI RADICAL GATE\0"
  "KANGXI RADICAL GHOST\0"
  "KANGXI RADICAL GO\0"
  "KANGXI RADICAL GO SLOWLY\0"
  "KANGXI RADICAL GOLD\0"
  "KANGXI RADICAL GRAIN\0"
  "KANGXI RADICAL GRASS\0"
  "KANGXI RADICAL HAIR\0"
  "KANGXI RADICAL HALBERD\0"
  "KANGXI RADICAL HALF TREE TRUNK\0"
  "KANGXI RADICAL HAND\0"
  "KANGXI RADICAL HEAD\0"
  "KANGXI RADICAL HEART\0"
  "KANGXI RADICAL HEMP\0"
  "KANGXI RADICAL HIDING ENCLOSURE\0"
  "KANGXI RADICAL HOOK\0"
  "KANGXI RADICAL HORN\0"
  "KANGXI RADICAL HORSE\0"
  "KANGXI RADICAL ICE\0"
  "KANGXI RADICAL INCH\0"
  "KANGXI RADICAL INSECT\0"
  "KANGXI RADICAL JADE\0"
  "KANGXI RADICAL JAR\0"
  "KANGXI RADICAL KNIFE\0"
  "KANGXI RADICAL LACK\0"
  "KANGXI RADICAL LAME\0"
  "KANGXI RADICAL LEAF\0"
  "KANGXI RADICAL LEATHER\0"
  "KANGXI RADICAL LEEK\0"
  "KANGXI RADICAL LEGS\0"
  "KANGXI RADICAL LID\0"
  "KANGXI RADICAL LIFE\0"
  "KANGXI RADICAL LINE\0"
  "KANGXI RADICAL LONG\0"
  "KANGXI RADICAL LONG STRIDE\0"
  "KANGXI RADICAL MAN\0"
  "KANGXI RADICAL MEAT\0"
  "KANGXI RADICAL MELON\0"
  "KANGXI RADICAL MILLET\0"
  "KANGXI RADICAL MINISTER\0"
  "KANGXI RADICAL MOON\0"
  "KANGXI RADICAL MORNING\0"
  "KANGXI RADICAL MORTAR\0"
  "KANGXI RADICAL MOUND\0"
  "KANGXI RADICAL MOUNTAIN\0"
  "KANGXI RADICAL MOUTH\0"
  "KANGXI RADICAL NET\0"
  "KANGXI RADICAL NOSE\0"
  "KANGXI RADICAL NOT\0"
  "KANGXI RADICAL OLD\0"
  "KANGXI RADICAL ONE\0"
  "KANGXI RADICAL ONESELF\0"
  "KANGXI RADICAL OPEN BOX\0"
  "KANGXI RADICAL OPPOSE\0"
  "KANGXI RADICAL PIG\0"
  "KANGXI RADICAL PLOW\0"
  "KANGXI RADICAL POWER\0"
  "KANGXI RADICAL PRIVATE\0"
  "KANGXI RADICAL PROFOUND\0"
  "KANGXI RADICAL RAIN\0"
  "KANGXI RADICAL RAP\0"
  "KANGXI RADICAL RAT\0"
  "KANGXI RADICAL RED\0"
  "KANGXI RADICAL RICE\0"
  "KANGXI RADICAL RIGHT OPEN BOX\0"
  "KANGXI RADICAL RIVER\0"
  "KANGXI RADICAL ROOF\0"
  "KANGXI RADICAL RUN\0"
  "KANGXI RADICAL SACRIFICIAL WINE\0"
  "KANGXI RADICAL SALT\0"
  "KANGXI RADICAL SAY\0"
  "KANGXI RADICAL SCHOLAR\0"
  "KANGXI RADICAL SCRIPT\0"
  "KANGXI RADICAL SEAL\0"
  "KANGXI RADICAL SECOND\0"
  "KANGXI RADICAL SEE\0"
  "KANGXI RADICAL SELF\0"
  "KANGXI RADICAL SHEEP\0"
  "KANGXI RADICAL SHELL\0"
  "KANGXI RADICAL SHOOT\0"
  "KANGXI RADICAL SHORT TAILED BIRD\0"
  "KANGXI RADICAL SHORT THREAD\0"
  "KANGXI RADICAL SICKNESS\0"
  "KANGXI RADICAL SILK\0"
  "KANGXI RADICAL SKIN\0"
  "KANGXI RADICAL SLASH\0"
  "KANGXI RADICAL SLAVE\0"
  "KANGXI RADICAL SLICE\0"
  "KANGXI RADICAL SMALL\0"
  "KANGXI RADICAL SNOUT\0"
  "KANGXI RADICAL SOUND\0"
  "KANGXI RADICAL SPEAR\0"
  "KANGXI RADICAL SPEECH\0"
  "KANGXI RADICAL SPIRIT\0"
  "KANGXI RADICAL SPOON\0"
  "KANGXI RADICAL SPROUT\0"
  "KANGXI RADICAL SQUARE\0"
  "KANGXI RADICAL STAND\0"
  "KANGXI RADICAL STEAM\0"
  "KANGXI RADICAL STEP\0"
  "KANGXI RADICAL STONE\0"
  "KANGXI RADICAL STOP\0"
  "KANGXI RADICAL STOPPING\0"
  "KANGXI RADICAL SUN\0"
  "KANGXI RADICAL SWEET\0"
  "KANGXI RADICAL TABLE\0"
  "KANGXI RADICAL TALL\0"
  "KANGXI RADICAL TANNED LEATHER\0"
  "KANGXI RADICAL TEN\0"
  "KANGXI RADICAL TIGER\0"
  "KANGXI RADICAL TILE\0"
  "KANGXI RADICAL TONGUE\0"
  "KANGXI RADICAL TOOTH\0"
  "KANGXI RADICAL TRACK\0"
  "KANGXI RADICAL TREE\0"
  "KANGXI RADICAL TRIPOD\0"
  "KANGXI RADICAL TURBAN\0"
  "KANGXI RADICAL TURTLE\0"
  "KANGXI RADICAL TWO\0"
  "KANGXI RADICAL TWO HANDS\0"
  "KANGXI RADICAL USE\0"
  "KANGXI RADICAL VALLEY\0"
  "KANGXI RADICAL VILLAGE\0"
  "KANGXI RADICAL WALK\0"
  "KANGXI RADICAL WALK ENCLOSURE\0"
  "KANGXI RADICAL WATER\0"
  "KANGXI RADICAL WEAPON\0"
  "KANGXI RADICAL WEST\0"
  "KANGXI RADICAL WHEAT\0"
  "KANGXI RADICAL WHITE\0"
  "KANGXI RADICAL WIND\0"
  "KANGXI RADICAL WINE\0"
  "KANGXI RADICAL WOMAN\0"
  "KANGXI RADICAL WORK\0"
  "KANGXI RADICAL WRAP\0"
  "KANGXI RADICAL WRONG\0"
  "KANGXI RADICAL YELLOW\0"
  "KANNADA AI LENGTH MARK\0"
  "KANNADA DIGIT EIGHT\0"
  "KANNADA DIGIT FIVE\0"
  "KANNADA DIGIT FOUR\0"
  "KANNADA DIGIT NINE\0"
  "KANNADA DIGIT ONE\0"
  "KANNADA DIGIT SEVEN\0"
  "KANNADA DIGIT SIX\0"
  "KANNADA DIGIT THREE\0"
  "KANNADA DIGIT TWO\0"
  "KANNADA DIGIT ZERO\0"
  "KANNADA LENGTH MARK\0"
  "KANNADA LETTER A\0"
  "KANNADA LETTER AA\0"
  "KANNADA LETTER AI\0"
  "KANNADA LETTER AU\0"
  "KANNADA LETTER BA\0"
  "KANNADA LETTER BHA\0"
  "KANNADA LETTER CA\0"
  "KANNADA LETTER CHA\0"
  "KANNADA LETTER DA\0"
  "KANNADA LETTER DDA\0"
  "KANNADA LETTER DDHA\0"
  "KANNADA LETTER DHA\0"
  "KANNADA LETTER E\0"
  "KANNADA LETTER EE\0"
  "KANNADA LETTER FA\0"
  "KANNADA LETTER GA\0"
  "KANNADA LETTER GHA\0"
  "KANNADA LETTER HA\0"
  "KANNADA LETTER I\0"
  "KANNADA LETTER II\0"
  "KANNADA LETTER JA\0"
  "KANNADA LETTER JHA\0"
  "KANNADA LETTER KA\0"
  "KANNADA LETTER KHA\0"
  "KANNADA LETTER LA\0"
  "KANNADA LETTER LLA\0"
  "KANNADA LETTER MA\0"
  "KANNADA LETTER NA\0"
  "KANNADA LETTER NAKAARA POLLU\0"
  "KANNADA LETTER NGA\0"
  "KANNADA LETTER NNA\0"
  "KANNADA LETTER NYA\0"
  "KANNADA LETTER O\0"
  "KANNADA LETTER OO\0"
  "KANNADA LETTER PA\0"
  "KANNADA LETTER PHA\0"
  "KANNADA LETTER RA\0"
  "KANNADA LETTER RRA\0"
  "KANNADA LETTER SA\0"
  "KANNADA LETTER SHA\0"
  "KANNADA LETTER SSA\0"
  "KANNADA LETTER TA\0"
  "KANNADA LETTER THA\0"
  "KANNADA LETTER TTA\0"
  "KANNADA LETTER TTHA\0"
  "KANNADA LETTER U\0"
  "KANNADA LETTER UU\0"
  "KANNADA LETTER VA\0"
  "KANNADA LETTER VOCALIC L\0"
  "KANNADA LETTER VOCALIC LL\0"
  "KANNADA LETTER VOCALIC R\0"
  "KANNADA LETTER VOCALIC RR\0"
  "KANNADA LETTER YA\0"
  "KANNADA SIGN ANUSVARA\0"
  "KANNADA SIGN AVAGRAHA\0"
  "KANNADA SIGN CANDRABINDU\0"
  "KANNADA SIGN COMBINING ANUSVARA ABOVE RIGHT\0"
  "KANNADA SIGN JIHVAMULIYA\0"
  "KANNADA SIGN NUKTA\0"
  "KANNADA SIGN SIDDHAM\0"
  "KANNADA SIGN SPACING CANDRABINDU\0"
  "KANNADA SIGN UPADHMANIYA\0"
  "KANNADA SIGN VIRAMA\0"
  "KANNADA SIGN VISARGA\0"
  "KANNADA VOWEL SIGN AA\0"
  "KANNADA VOWEL SIGN AI\0"
  "KANNADA VOWEL SIGN AU\0"
  "KANNADA VOWEL SIGN E\0"
  "KANNADA VOWEL SIGN EE\0"
  "KANNADA VOWEL SIGN I\0"
  "KANNADA VOWEL SIGN II\0"
  "KANNADA VOWEL SIGN O\0"
  "KANNADA VOWEL SIGN OO\0"
  "KANNADA VOWEL SIGN U\0"
  "KANNADA VOWEL SIGN UU\0"
  "KANNADA VOWEL SIGN VOCALIC L\0"
  "KANNADA VOWEL SIGN VOCALIC LL\0"
  "KANNADA VOWEL SIGN VOCALIC R\0"
  "KANNADA VOWEL SIGN VOCALIC RR\0"
  "KATAKANA DIGRAPH KOTO\0"
  "KATAKANA ITERATION MARK\0"
  "KATAKANA LETTER A\0"
  "KATAKANA LETTER ARCHAIC E\0"
  "KATAKANA LETTER ARCHAIC WU\0"
  "KATAKANA LETTER ARCHAIC YE\0"
  "KATAKANA LETTER ARCHAIC YI\0"
  "KATAKANA LETTER BA\0"
  "KATAKANA LETTER BE\0"
  "KATAKANA LETTER BI\0"
  "KATAKANA LETTER BO\0"
  "KATAKANA LETTER BU\0"
  "KATAKANA LETTER DA\0"
  "KATAKANA LETTER DE\0"
  "KATAKANA LETTER DI\0"
  "KATAKANA LETTER DO\0"
  "KATAKANA LETTER DU\0"
  "KATAKANA LETTER E\0"
  "KATAKANA LETTER GA\0"
  "KATAKANA LETTER GE\0"
  "KATAKANA LETTER GI\0"
  "KATAKANA LETTER GO\0"
  "KATAKANA LETTER GU\0"
  "KATAKANA LETTER HA\0"
  "KATAKANA LETTER HE\0"
  "KATAKANA LETTER HI\0"
  "KATAKANA LETTER HO\0"
  "KATAKANA LETTER HU\0"
  "KATAKANA LETTER I\0"
  "KATAKANA LETTER KA\0"
  "KATAKANA LETTER KE\0"
  "KATAKANA LETTER KI\0"
  "KATAKANA LETTER KO\0"
  "KATAKANA LETTER KU\0"
  "KATAKANA LETTER MA\0"
  "KATAKANA LETTER ME\0"
  "KATAKANA LETTER MI\0"
  "KATAKANA LETTER MINNAN NASALIZED TONE-1\0"
  "KATAKANA LETTER MINNAN NASALIZED TONE-2\0"
  "KATAKANA LETTER MINNAN NASALIZED TONE-3\0"
  "KATAKANA LETTER MINNAN NASALIZED TONE-4\0"
  "KATAKANA LETTER MINNAN NASALIZED TONE-5\0"
  "KATAKANA LETTER MINNAN NASALIZED TONE-7\0"
  "KATAKANA LETTER MINNAN NASALIZED TONE-8\0"
  "KATAKANA LETTER MINNAN TONE-2\0"
  "KATAKANA LETTER MINNAN TONE-3\0"
  "KATAKANA LETTER MINNAN TONE-4\0"
  "KATAKANA LETTER MINNAN TONE-5\0"
  "KATAKANA LETTER MINNAN TONE-7\0"
  "KATAKANA LETTER MINNAN TONE-8\0"
  "KATAKANA LETTER MO\0"
  "KATAKANA LETTER MU\0"
  "KATAKANA LETTER N\0"
  "KATAKANA LETTER NA\0"
  "KATAKANA LETTER NE\0"
  "KATAKANA LETTER NI\0"
  "KATAKANA LETTER NO\0"
  "KATAKANA LETTER NU\0"
  "KATAKANA LETTER O\0"
  "KATAKANA LETTER PA\0"
  "KATAKANA LETTER PE\0"
  "KATAKANA LETTER PI\0"
  "KATAKANA LETTER PO\0"
  "KATAKANA LETTER PU\0"
  "KATAKANA LETTER RA\0"
  "KATAKANA LETTER RE\0"
  "KATAKANA LETTER RI\0"
  "KATAKANA LETTER RO\0"
  "KATAKANA LETTER RU\0"
  "KATAKANA LETTER SA\0"
  "KATAKANA LETTER SE\0"
  "KATAKANA LETTER SI\0"
  "KATAKANA LETTER SMALL A\0"
  "KATAKANA LETTER SMALL E\0"
  "KATAKANA LETTER SMALL HA\0"
  "KATAKANA LETTER SMALL HE\0"
  "KATAKANA LETTER SMALL HI\0"
  "KATAKANA LETTER SMALL HO\0"
  "KATAKANA LETTER SMALL HU\0"
  "KATAKANA LETTER SMALL I\0"
  "KATAKANA LETTER SMALL KA\0"
  "KATAKANA LETTER SMALL KE\0"
  "KATAKANA LETTER SMALL KO\0"
  "KATAKANA LETTER SMALL KU\0"
  "KATAKANA LETTER SMALL MU\0"
  "KATAKANA LETTER SMALL N\0"
  "KATAKANA LETTER SMALL NU\0"
  "KATAKANA LETTER SMALL O\0"
  "KATAKANA LETTER SMALL RA\0"
  "KATAKANA LETTER SMALL RE\0"
  "KATAKANA LETTER SMALL RI\0"
  "KATAKANA LETTER SMALL RO\0"
  "KATAKANA LETTER SMALL RU\0"
  "KATAKANA LETTER SMALL SI\0"
  "KATAKANA LETTER SMALL SU\0"
  "KATAKANA LETTER SMALL TO\0"
  "KATAKANA LETTER SMALL TU\0"
  "KATAKANA LETTER SMALL U\0"
  "KATAKANA LETTER SMALL WA\0"
  "KATAKANA LETTER SMALL WE\0"
  "KATAKANA LETTER SMALL WI\0"
  "KATAKANA LETTER SMALL WO\0"
  "KATAKANA LETTER SMALL YA\0"
  "KATAKANA LETTER SMALL YO\0"
  "KATAKANA LETTER SMALL YU\0"
  "KATAKANA LETTER SO\0"
  "KATAKANA LETTER SU\0"
  "KATAKANA LETTER TA\0"
  "KATAKANA LETTER TE\0"
  "KATAKANA LETTER TI\0"
  "KATAKANA LETTER TO\0"
  "KATAKANA LETTER TU\0"
  "KATAKANA LETTER U\0"
  "KATAKANA LETTER VA\0"
  "KATAKANA LETTER VE\0"
  "KATAKANA LETTER VI\0"
  "KATAKANA LETTER VO\0"
  "KATAKANA LETTER VU\0"
  "KATAKANA LETTER WA\0"
  "KATAKANA LETTER WE\0"
  "KATAKANA LETTER WI\0"
  "KATAKANA LETTER WO\0"
  "KATAKANA LETTER YA\0"
  "KATAKANA LETTER YO\0"
  "KATAKANA LETTER YU\0"
  "KATAKANA LETTER ZA\0"
  "KATAKANA LETTER ZE\0"
  "KATAKANA LETTER ZI\0"
  "KATAKANA LETTER ZO\0"
  "KATAKANA LETTER ZU\0"
  "KATAKANA MIDDLE DOT\0"
  "KATAKANA VOICED ITERATION MARK\0"
  "KATAKANA-HIRAGANA DOUBLE HYPHEN\0"
  "KATAKANA-HIRAGANA PROLONGED SOUND MARK\0"
  "KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK\0"
  "KATAKANA-HIRAGANA VOICED SOUND MARK\0"
  "KAWI CONJOINER\0"
  "KAWI DANDA\0"
  "KAWI DIGIT EIGHT\0"
  "KAWI DIGIT FIVE\0"
  "KAWI DIGIT FOUR\0"
  "KAWI DIGIT NINE\0"
  "KAWI DIGIT ONE\0"
  "KAWI DIGIT SEVEN\0"
  "KAWI DIGIT SIX\0"
  "KAWI DIGIT THREE\0"
  "KAWI DIGIT TWO\0"
  "KAWI DIGIT ZERO\0"
  "KAWI DOUBLE DANDA\0"
  "KAWI LETTER A\0"
  "KAWI LETTER AA\0"
  "KAWI LETTER AI\0"
  "KAWI LETTER BA\0"
  "KAWI LETTER BHA\0"
  "KAWI LETTER CA\0"
  "KAWI LETTER CHA\0"
  "KAWI LETTER DA\0"
  "KAWI LETTER DDA\0"
  "KAWI LETTER DDHA\0"
  "KAWI LETTER DHA\0"
  "KAWI LETTER E\0"
  "KAWI LETTER GA\0"
  "KAWI LETTER GHA\0"
  "KAWI LETTER HA\0"
  "KAWI LETTER I\0"
  "KAWI LETTER II\0"
  "KAWI LETTER JA\0"
  "KAWI LETTER JHA\0"
  "KAWI LETTER JNYA\0"
  "KAWI LETTER KA\0"
  "KAWI LETTER KHA\0"
  "KAWI LETTER LA\0"
  "KAWI LETTER MA\0"
  "KAWI LETTER NA\0"
  "KAWI LETTER NGA\0"
  "KAWI LETTER NNA\0"
  "KAWI LETTER NYA\0"
  "KAWI LETTER O\0"
  "KAWI LETTER PA\0"
  "KAWI LETTER PHA\0"
  "KAWI LETTER RA\0"
  "KAWI LETTER SA\0"
  "KAWI LETTER SHA\0"
  "KAWI LETTER SSA\0"
  "KAWI LETTER TA\0"
  "KAWI LETTER THA\0"
  "KAWI LETTER TTA\0"
  "KAWI LETTER TTHA\0"
  "KAWI LETTER U\0"
  "KAWI LETTER UU\0"
  "KAWI LETTER VOCALIC L\0"
  "KAWI LETTER VOCALIC LL\0"
  "KAWI LETTER VOCALIC R\0"
  "KAWI LETTER VOCALIC RR\0"
  "KAWI LETTER WA\0"
  "KAWI LETTER YA\0"
  "KAWI PUNCTUATION ALTERNATE SECTION MARKER\0"
  "KAWI PUNCTUATION CIRCLE\0"
  "KAWI PUNCTUATION CLOSING SPIRAL\0"
  "KAWI PUNCTUATION DOT\0"
  "KAWI PUNCTUATION DOUBLE DOT\0"
  "KAWI PUNCTUATION FILLED CIRCLE\0"
  "KAWI PUNCTUATION FLOWER\0"
  "KAWI PUNCTUATION SECTION MARKER\0"
  "KAWI PUNCTUATION SPACE FILLER\0"
  "KAWI PUNCTUATION SPIRAL\0"
  "KAWI PUNCTUATION TRIPLE DOT\0"
  "KAWI SIGN ANUSVARA\0"
  "KAWI SIGN CANDRABINDU\0"
  "KAWI SIGN KILLER\0"
  "KAWI SIGN NUKTA\0"
  "KAWI SIGN REPHA\0"
  "KAWI SIGN VISARGA\0"
  "KAWI VOWEL SIGN AA\0"
  "KAWI VOWEL SIGN AI\0"
  "KAWI VOWEL SIGN ALTERNATE AA\0"
  "KAWI VOWEL SIGN E\0"
  "KAWI VOWEL SIGN EU\0"
  "KAWI VOWEL SIGN I\0"
  "KAWI VOWEL SIGN II\0"
  "KAWI VOWEL SIGN U\0"
  "KAWI VOWEL SIGN UU\0"
  "KAWI VOWEL SIGN VOCALIC R\0"
  "KAYAH LI DIGIT EIGHT\0"
  "KAYAH LI DIGIT FIVE\0"
  "KAYAH LI DIGIT FOUR\0"
  "KAYAH LI DIGIT NINE\0"
  "KAYAH LI DIGIT ONE\0"
  "KAYAH LI DIGIT SEVEN\0"
  "KAYAH LI DIGIT SIX\0"
  "KAYAH LI DIGIT THREE\0"
  "KAYAH LI DIGIT TWO\0"
  "KAYAH LI DIGIT ZERO\0"
  "KAYAH LI LETTER A\0"
  "KAYAH LI LETTER BA\0"
  "KAYAH LI LETTER CA\0"
  "KAYAH LI LETTER DA\0"
  "KAYAH LI LETTER GA\0"
  "KAYAH LI LETTER HA\0"
  "KAYAH LI LETTER HTA\0"
  "KAYAH LI LETTER I\0"
  "KAYAH LI LETTER KA\0"
  "KAYAH LI LETTER KHA\0"
  "KAYAH LI LETTER LA\0"
  "KAYAH LI LETTER MA\0"
  "KAYAH LI LETTER NA\0"
  "KAYAH LI LETTER NGA\0"
  "KAYAH LI LETTER NYA\0"
  "KAYAH LI LETTER OE\0"
  "KAYAH LI LETTER OO\0"
  "KAYAH LI LETTER PA\0"
  "KAYAH LI LETTER PHA\0"
  "KAYAH LI LETTER RA\0"
  "KAYAH LI LETTER SA\0"
  "KAYAH LI LETTER SHA\0"
  "KAYAH LI LETTER TA\0"
  "KAYAH LI LETTER THA\0"
  "KAYAH LI LETTER VA\0"
  "KAYAH LI LETTER WA\0"
  "KAYAH LI LETTER YA\0"
  "KAYAH LI LETTER ZA\0"
  "KAYAH LI SIGN CWI\0"
  "KAYAH LI SIGN SHYA\0"
  "KAYAH LI TONE CALYA\0"
  "KAYAH LI TONE CALYA PLOPHU\0"
  "KAYAH LI TONE PLOPHU\0"
  "KAYAH LI VOWEL E\0"
  "KAYAH LI VOWEL EE\0"
  "KAYAH LI VOWEL O\0"
  "KAYAH LI VOWEL U\0"
  "KAYAH LI VOWEL UE\0"
  "KELVIN SIGN\0"
  "KEY\0"
  "KEYBOARD\0"
  "KEYBOARD AND MOUSE\0"
  "KEYCAP TEN\0"
  "KEYHOLE\0"
  "KHANDA\0"
  "KHAROSHTHI DIGIT FOUR\0"
  "KHAROSHTHI DIGIT ONE\0"
  "KHAROSHTHI DIGIT THREE\0"
  "KHAROSHTHI DIGIT TWO\0"
  "KHAROSHTHI FRACTION ONE HALF\0"
  "KHAROSHTHI LETTER A\0"
  "KHAROSHTHI LETTER BA\0"
  "KHAROSHTHI LETTER BHA\0"
  "KHAROSHTHI LETTER CA\0"
  "KHAROSHTHI LETTER CHA\0"
  "KHAROSHTHI LETTER DA\0"
  "KHAROSHTHI LETTER DDA\0"
  "KHAROSHTHI LETTER DDHA\0"
  "KHAROSHTHI LETTER DHA\0"
  "KHAROSHTHI LETTER GA\0"
  "KHAROSHTHI LETTER GHA\0"
  "KHAROSHTHI LETTER HA\0"
  "KHAROSHTHI LETTER JA\0"
  "KHAROSHTHI LETTER KA\0"
  "KHAROSHTHI LETTER KHA\0"
  "KHAROSHTHI LETTER KKA\0"
  "KHAROSHTHI LETTER LA\0"
  "KHAROSHTHI LETTER MA\0"
  "KHAROSHTHI LETTER NA\0"
  "KHAROSHTHI LETTER NNA\0"
  "KHAROSHTHI LETTER NYA\0"
  "KHAROSHTHI LETTER PA\0"
  "KHAROSHTHI LETTER PHA\0"
  "KHAROSHTHI LETTER RA\0"
  "KHAROSHTHI LETTER SA\0"
  "KHAROSHTHI LETTER SHA\0"
  "KHAROSHTHI LETTER SSA\0"
  "KHAROSHTHI LETTER TA\0"
  "KHAROSHTHI LETTER THA\0"
  "KHAROSHTHI LETTER TTA\0"
  "KHAROSHTHI LETTER TTHA\0"
  "KHAROSHTHI LETTER TTTA\0"
  "KHAROSHTHI LETTER TTTHA\0"
  "KHAROSHTHI LETTER VA\0"
  "KHAROSHTHI LETTER VHA\0"
  "KHAROSHTHI LETTER YA\0"
  "KHAROSHTHI LETTER ZA\0"
  "KHAROSHTHI NUMBER ONE HUNDRED\0"
  "KHAROSHTHI NUMBER ONE THOUSAND\0"
  "KHAROSHTHI NUMBER TEN\0"
  "KHAROSHTHI NUMBER TWENTY\0"
  "KHAROSHTHI PUNCTUATION CIRCLE\0"
  "KHAROSHTHI PUNCTUATION CRESCENT BAR\0"
  "KHAROSHTHI PUNCTUATION DANDA\0"
  "KHAROSHTHI PUNCTUATION DOT\0"
  "KHAROSHTHI PUNCTUATION DOUBLE DANDA\0"
  "KHAROSHTHI PUNCTUATION LINES\0"
  "KHAROSHTHI PUNCTUATION LOTUS\0"
  "KHAROSHTHI PUNCTUATION MANGALAM\0"
  "KHAROSHTHI PUNCTUATION SMALL CIRCLE\0"
  "KHAROSHTHI SIGN ANUSVARA\0"
  "KHAROSHTHI SIGN BAR ABOVE\0"
  "KHAROSHTHI SIGN CAUDA\0"
  "KHAROSHTHI SIGN DOT BELOW\0"
  "KHAROSHTHI SIGN DOUBLE RING BELOW\0"
  "KHAROSHTHI SIGN VISARGA\0"
  "KHAROSHTHI VIRAMA\0"
  "KHAROSHTHI VOWEL LENGTH MARK\0"
  "KHAROSHTHI VOWEL SIGN E\0"
  "KHAROSHTHI VOWEL SIGN I\0"
  "KHAROSHTHI VOWEL SIGN O\0"
  "KHAROSHTHI VOWEL SIGN U\0"
  "KHAROSHTHI VOWEL SIGN VOCALIC R\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B00\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B01\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B02\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B03\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B04\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B05\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B06\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B07\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B08\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B09\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B0A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B0B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B0C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B0D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B0E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B0F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B10\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B11\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B12\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B13\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B14\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B15\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B16\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B17\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B18\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B19\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B1A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B1B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B1C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B1D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B1E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B1F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B20\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B21\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B22\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B23\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B24\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B25\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B26\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B27\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B28\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B29\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B2A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B2B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B2C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B2D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B2E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B2F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B30\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B31\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B32\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B33\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B34\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B35\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B36\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B37\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B38\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B39\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B3A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B3B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B3C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B3D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B3E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B3F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B40\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B41\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B42\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B43\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B44\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B45\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B46\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B47\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B48\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B49\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B4A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B4B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B4C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B4D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B4E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B4F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B50\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B51\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B52\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B53\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B54\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B55\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B56\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B57\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B58\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B59\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B5A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B5B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B5C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B5D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B5E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B5F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B60\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B61\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B62\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B63\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B64\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B65\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B66\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B67\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B68\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B69\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B6A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B6B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B6C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B6D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B6E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B6F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B70\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B71\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B72\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B73\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B74\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B75\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B76\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B77\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B78\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B79\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B7A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B7B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B7C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B7D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B7E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B7F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B80\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B81\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B82\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B83\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B84\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B85\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B86\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B87\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B88\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B89\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B8A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B8B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B8C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B8D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B8E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B8F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B90\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B91\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B92\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B93\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B94\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B95\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B96\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B97\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B98\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B99\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B9A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B9B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B9C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B9D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B9E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18B9F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA6\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA7\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA8\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BA9\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BAA\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BAB\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BAC\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BAD\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BAE\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BAF\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB6\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB7\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB8\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BB9\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BBA\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BBB\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BBC\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BBD\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BBE\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BBF\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC6\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC7\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC8\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BC9\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BCA\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BCB\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BCC\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BCD\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BCE\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BCF\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD6\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD7\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD8\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BD9\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BDA\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BDB\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BDC\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BDD\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BDE\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BDF\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE6\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE7\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE8\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BE9\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BEA\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BEB\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BEC\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BED\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BEE\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BEF\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF6\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF7\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF8\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BF9\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BFA\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BFB\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BFC\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BFD\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BFE\0"
  "KHITAN SMALL SCRIPT CHARACTER-18BFF\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C00\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C01\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C02\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C03\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C04\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C05\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C06\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C07\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C08\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C09\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C0A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C0B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C0C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C0D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C0E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C0F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C10\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C11\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C12\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C13\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C14\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C15\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C16\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C17\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C18\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C19\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C1A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C1B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C1C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C1D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C1E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C1F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C20\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C21\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C22\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C23\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C24\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C25\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C26\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C27\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C28\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C29\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C2A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C2B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C2C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C2D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C2E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C2F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C30\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C31\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C32\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C33\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C34\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C35\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C36\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C37\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C38\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C39\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C3A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C3B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C3C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C3D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C3E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C3F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C40\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C41\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C42\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C43\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C44\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C45\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C46\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C47\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C48\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C49\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C4A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C4B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C4C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C4D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C4E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C4F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C50\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C51\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C52\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C53\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C54\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C55\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C56\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C57\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C58\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C59\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C5A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C5B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C5C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C5D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C5E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C5F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C60\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C61\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C62\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C63\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C64\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C65\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C66\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C67\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C68\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C69\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C6A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C6B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C6C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C6D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C6E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C6F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C70\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C71\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C72\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C73\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C74\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C75\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C76\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C77\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C78\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C79\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C7A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C7B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C7C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C7D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C7E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C7F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C80\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C81\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C82\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C83\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C84\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C85\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C86\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C87\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C88\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C89\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C8A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C8B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C8C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C8D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C8E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C8F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C90\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C91\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C92\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C93\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C94\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C95\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C96\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C97\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C98\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C99\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C9A\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C9B\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C9C\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C9D\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C9E\0"
  "KHITAN SMALL SCRIPT CHARACTER-18C9F\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA6\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA7\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA8\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CA9\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CAA\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CAB\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CAC\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CAD\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CAE\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CAF\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB6\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB7\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB8\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CB9\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CBA\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CBB\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CBC\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CBD\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CBE\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CBF\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC6\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC7\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC8\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CC9\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CCA\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CCB\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CCC\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CCD\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CCE\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CCF\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CD0\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CD1\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CD2\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CD3\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CD4\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CD5\0"
  "KHITAN SMALL SCRIPT CHARACTER-18CFF\0"
  "KHITAN SMALL SCRIPT FILLER\0"
  "KHMER CURRENCY SYMBOL RIEL\0"
  "KHMER DIGIT EIGHT\0"
  "KHMER DIGIT FIVE\0"
  "KHMER DIGIT FOUR\0"
  "KHMER DIGIT NINE\0"
  "KHMER DIGIT ONE\0"
  "KHMER DIGIT SEVEN\0"
  "KHMER DIGIT SIX\0"
  "KHMER DIGIT THREE\0"
  "KHMER DIGIT TWO\0"
  "KHMER DIGIT ZERO\0"
  "KHMER INDEPENDENT VOWEL LY\0"
  "KHMER INDEPENDENT VOWEL LYY\0"
  "KHMER INDEPENDENT VOWEL QAA\0"
  "KHMER INDEPENDENT VOWEL QAI\0"
  "KHMER INDEPENDENT VOWEL QAQ\0"
  "KHMER INDEPENDENT VOWEL QAU\0"
  "KHMER INDEPENDENT VOWEL QE\0"
  "KHMER INDEPENDENT VOWEL QI\0"
  "KHMER INDEPENDENT VOWEL QII\0"
  "KHMER INDEPENDENT VOWEL QOO TYPE ONE\0"
  "KHMER INDEPENDENT VOWEL QOO TYPE TWO\0"
  "KHMER INDEPENDENT VOWEL QU\0"
  "KHMER INDEPENDENT VOWEL QUK\0"
  "KHMER INDEPENDENT VOWEL QUU\0"
  "KHMER INDEPENDENT VOWEL QUUV\0"
  "KHMER INDEPENDENT VOWEL RY\0"
  "KHMER INDEPENDENT VOWEL RYY\0"
  "KHMER LETTER BA\0"
  "KHMER LETTER CA\0"
  "KHMER LETTER CHA\0"
  "KHMER LETTER CHO\0"
  "KHMER LETTER CO\0"
  "KHMER LETTER DA\0"
  "KHMER LETTER DO\0"
  "KHMER LETTER HA\0"
  "KHMER LETTER KA\0"
  "KHMER LETTER KHA\0"
  "KHMER LETTER KHO\0"
  "KHMER LETTER KO\0"
  "KHMER LETTER LA\0"
  "KHMER LETTER LO\0"
  "KHMER LETTER MO\0"
  "KHMER LETTER NGO\0"
  "KHMER LETTER NNO\0"
  "KHMER LETTER NO\0"
  "KHMER LETTER NYO\0"
  "KHMER LETTER PHA\0"
  "KHMER LETTER PHO\0"
  "KHMER LETTER PO\0"
  "KHMER LETTER QA\0"
  "KHMER LETTER RO\0"
  "KHMER LETTER SA\0"
  "KHMER LETTER SHA\0"
  "KHMER LETTER SSO\0"
  "KHMER LETTER TA\0"
  "KHMER LETTER THA\0"
  "KHMER LETTER THO\0"
  "KHMER LETTER TO\0"
  "KHMER LETTER TTHA\0"
  "KHMER LETTER TTHO\0"
  "KHMER LETTER VO\0"
  "KHMER LETTER YO\0"
  "KHMER SIGN AHSDA\0"
  "KHMER SIGN ATTHACAN\0"
  "KHMER SIGN AVAKRAHASANYA\0"
  "KHMER SIGN BANTOC\0"
  "KHMER SIGN BARIYOOSAN\0"
  "KHMER SIGN BATHAMASAT\0"
  "KHMER SIGN BEYYAL\0"
  "KHMER SIGN CAMNUC PII KUUH\0"
  "KHMER SIGN COENG\0"
  "KHMER SIGN KAKABAT\0"
  "KHMER SIGN KHAN\0"
  "KHMER SIGN KOOMUUT\0"
  "KHMER SIGN LEK TOO\0"
  "KHMER SIGN MUUSIKATOAN\0"
  "KHMER SIGN NIKAHIT\0"
  "KHMER SIGN PHNAEK MUAN\0"
  "KHMER SIGN REAHMUK\0"
  "KHMER SIGN ROBAT\0"
  "KHMER SIGN SAMYOK SANNYA\0"
  "KHMER SIGN TOANDAKHIAT\0"
  "KHMER SIGN TRIISAP\0"
  "KHMER SIGN VIRIAM\0"
  "KHMER SIGN YUUKALEAPINTU\0"
  "KHMER SYMBOL BEI KOET\0"
  "KHMER SYMBOL BEI ROC\0"
  "KHMER SYMBOL BUON KOET\0"
  "KHMER SYMBOL BUON ROC\0"
  "KHMER SYMBOL DAP KOET\0"
  "KHMER SYMBOL DAP ROC\0"
  "KHMER SYMBOL DAP-BEI KOET\0"
  "KHMER SYMBOL DAP-BEI ROC\0"
  "KHMER SYMBOL DAP-BUON KOET\0"
  "KHMER SYMBOL DAP-BUON ROC\0"
  "KHMER SYMBOL DAP-MUOY KOET\0"
  "KHMER SYMBOL DAP-MUOY ROC\0"
  "KHMER SYMBOL DAP-PII KOET\0"
  "KHMER SYMBOL DAP-PII ROC\0"
  "KHMER SYMBOL DAP-PRAM KOET\0"
  "KHMER SYMBOL DAP-PRAM ROC\0"
  "KHMER SYMBOL LEK ATTAK BEI\0"
  "KHMER SYMBOL LEK ATTAK BUON\0"
  "KHMER SYMBOL LEK ATTAK MUOY\0"
  "KHMER SYMBOL LEK ATTAK PII\0"
  "KHMER SYMBOL LEK ATTAK PRAM\0"
  "KHMER SYMBOL LEK ATTAK PRAM-BEI\0"
  "KHMER SYMBOL LEK ATTAK PRAM-BUON\0"
  "KHMER SYMBOL LEK ATTAK PRAM-MUOY\0"
  "KHMER SYMBOL LEK ATTAK PRAM-PII\0"
  "KHMER SYMBOL LEK ATTAK SON\0"
  "KHMER SYMBOL MUOY KOET\0"
  "KHMER SYMBOL MUOY ROC\0"
  "KHMER SYMBOL PATHAMASAT\0"
  "KHMER SYMBOL PII KOET\0"
  "KHMER SYMBOL PII ROC\0"
  "KHMER SYMBOL PRAM KOET\0"
  "KHMER SYMBOL PRAM ROC\0"
  "KHMER SYMBOL PRAM-BEI KOET\0"
  "KHMER SYMBOL PRAM-BEI ROC\0"
  "KHMER SYMBOL PRAM-BUON KOET\0"
  "KHMER SYMBOL PRAM-BUON ROC\0"
  "KHMER SYMBOL PRAM-MUOY KOET\0"
  "KHMER SYMBOL PRAM-MUOY ROC\0"
  "KHMER SYMBOL PRAM-PII KOET\0"
  "KHMER SYMBOL PRAM-PII ROC\0"
  "KHMER SYMBOL TUTEYASAT\0"
  "KHMER VOWEL INHERENT AA\0"
  "KHMER VOWEL INHERENT AQ\0"
  "KHMER VOWEL SIGN AA\0"
  "KHMER VOWEL SIGN AE\0"
  "KHMER VOWEL SIGN AI\0"
  "KHMER VOWEL SIGN AU\0"
  "KHMER VOWEL SIGN E\0"
  "KHMER VOWEL SIGN I\0"
  "KHMER VOWEL SIGN IE\0"
  "KHMER VOWEL SIGN II\0"
  "KHMER VOWEL SIGN OE\0"
  "KHMER VOWEL SIGN OO\0"
  "KHMER VOWEL SIGN U\0"
  "KHMER VOWEL SIGN UA\0"
  "KHMER VOWEL SIGN UU\0"
  "KHMER VOWEL SIGN Y\0"
  "KHMER VOWEL SIGN YA\0"
  "KHMER VOWEL SIGN YY\0"
  "KHOJKI ABBREVIATION SIGN\0"
  "KHOJKI DANDA\0"
  "KHOJKI DOUBLE DANDA\0"
  "KHOJKI DOUBLE SECTION MARK\0"
  "KHOJKI LETTER A\0"
  "KHOJKI LETTER AA\0"
  "KHOJKI LETTER AI\0"
  "KHOJKI LETTER AU\0"
  "KHOJKI LETTER BA\0"
  "KHOJKI LETTER BBA\0"
  "KHOJKI LETTER BHA\0"
  "KHOJKI LETTER CA\0"
  "KHOJKI LETTER CHA\0"
  "KHOJKI LETTER DA\0"
  "KHOJKI LETTER DDA\0"
  "KHOJKI LETTER DDDA\0"
  "KHOJKI LETTER DDHA\0"
  "KHOJKI LETTER DHA\0"
  "KHOJKI LETTER E\0"
  "KHOJKI LETTER GA\0"
  "KHOJKI LETTER GGA\0"
  "KHOJKI LETTER GHA\0"
  "KHOJKI LETTER HA\0"
  "KHOJKI LETTER I\0"
  "KHOJKI LETTER JA\0"
  "KHOJKI LETTER JJA\0"
  "KHOJKI LETTER KA\0"
  "KHOJKI LETTER KHA\0"
  "KHOJKI LETTER LA\0"
  "KHOJKI LETTER LLA\0"
  "KHOJKI LETTER MA\0"
  "KHOJKI LETTER NA\0"
  "KHOJKI LETTER NGA\0"
  "KHOJKI LETTER NNA\0"
  "KHOJKI LETTER NYA\0"
  "KHOJKI LETTER O\0"
  "KHOJKI LETTER PA\0"
  "KHOJKI LETTER PHA\0"
  "KHOJKI LETTER QA\0"
  "KHOJKI LETTER RA\0"
  "KHOJKI LETTER SA\0"
  "KHOJKI LETTER SHORT I\0"
  "KHOJKI LETTER TA\0"
  "KHOJKI LETTER THA\0"
  "KHOJKI LETTER TTA\0"
  "KHOJKI LETTER TTHA\0"
  "KHOJKI LETTER U\0"
  "KHOJKI LETTER VA\0"
  "KHOJKI LETTER YA\0"
  "KHOJKI SECTION MARK\0"
  "KHOJKI SIGN ANUSVARA\0"
  "KHOJKI SIGN NUKTA\0"
  "KHOJKI SIGN SHADDA\0"
  "KHOJKI SIGN SUKUN\0"
  "KHOJKI SIGN VIRAMA\0"
  "KHOJKI VOWEL SIGN AA\0"
  "KHOJKI VOWEL SIGN AI\0"
  "KHOJKI VOWEL SIGN AU\0"
  "KHOJKI VOWEL SIGN E\0"
  "KHOJKI VOWEL SIGN I\0"
  "KHOJKI VOWEL SIGN II\0"
  "KHOJKI VOWEL SIGN O\0"
  "KHOJKI VOWEL SIGN U\0"
  "KHOJKI VOWEL SIGN VOCALIC R\0"
  "KHOJKI WORD SEPARATOR\0"
  "KHUDAWADI DIGIT EIGHT\0"
  "KHUDAWADI DIGIT FIVE\0"
  "KHUDAWADI DIGIT FOUR\0"
  "KHUDAWADI DIGIT NINE\0"
  "KHUDAWADI DIGIT ONE\0"
  "KHUDAWADI DIGIT SEVEN\0"
  "KHUDAWADI DIGIT SIX\0"
  "KHUDAWADI DIGIT THREE\0"
  "KHUDAWADI DIGIT TWO\0"
  "KHUDAWADI DIGIT ZERO\0"
  "KHUDAWADI LETTER A\0"
  "KHUDAWADI LETTER AA\0"
  "KHUDAWADI LETTER AI\0"
  "KHUDAWADI LETTER AU\0"
  "KHUDAWADI LETTER BA\0"
  "KHUDAWADI LETTER BBA\0"
  "KHUDAWADI LETTER BHA\0"
  "KHUDAWADI LETTER CA\0"
  "KHUDAWADI LETTER CHA\0"
  "KHUDAWADI LETTER DA\0"
  "KHUDAWADI LETTER DDA\0"
  "KHUDAWADI LETTER DDDA\0"
  "KHUDAWADI LETTER DDHA\0"
  "KHUDAWADI LETTER DHA\0"
  "KHUDAWADI LETTER E\0"
  "KHUDAWADI LETTER GA\0"
  "KHUDAWADI LETTER GGA\0"
  "KHUDAWADI LETTER GHA\0"
  "KHUDAWADI LETTER HA\0"
  "KHUDAWADI LETTER I\0"
  "KHUDAWADI LETTER II\0"
  "KHUDAWADI LETTER JA\0"
  "KHUDAWADI LETTER JHA\0"
  "KHUDAWADI LETTER JJA\0"
  "KHUDAWADI LETTER KA\0"
  "KHUDAWADI LETTER KHA\0"
  "KHUDAWADI LETTER LA\0"
  "KHUDAWADI LETTER MA\0"
  "KHUDAWADI LETTER NA\0"
  "KHUDAWADI LETTER NGA\0"
  "KHUDAWADI LETTER NNA\0"
  "KHUDAWADI LETTER NYA\0"
  "KHUDAWADI LETTER O\0"
  "KHUDAWADI LETTER PA\0"
  "KHUDAWADI LETTER PHA\0"
  "KHUDAWADI LETTER RA\0"
  "KHUDAWADI LETTER RRA\0"
  "KHUDAWADI LETTER SA\0"
  "KHUDAWADI LETTER SHA\0"
  "KHUDAWADI LETTER TA\0"
  "KHUDAWADI LETTER THA\0"
  "KHUDAWADI LETTER TTA\0"
  "KHUDAWADI LETTER TTHA\0"
  "KHUDAWADI LETTER U\0"
  "KHUDAWADI LETTER UU\0"
  "KHUDAWADI LETTER VA\0"
  "KHUDAWADI LETTER YA\0"
  "KHUDAWADI SIGN ANUSVARA\0"
  "KHUDAWADI SIGN NUKTA\0"
  "KHUDAWADI SIGN VIRAMA\0"
  "KHUDAWADI VOWEL SIGN AA\0"
  "KHUDAWADI VOWEL SIGN AI\0"
  "KHUDAWADI VOWEL SIGN AU\0"
  "KHUDAWADI VOWEL SIGN E\0"
  "KHUDAWADI VOWEL SIGN I\0"
  "KHUDAWADI VOWEL SIGN II\0"
  "KHUDAWADI VOWEL SIGN O\0"
  "KHUDAWADI VOWEL SIGN U\0"
  "KHUDAWADI VOWEL SIGN UU\0"
  "KIMONO\0"
  "KIP SIGN\0"
  "KIRAT RAI DANDA\0"
  "KIRAT RAI DIGIT EIGHT\0"
  "KIRAT RAI DIGIT FIVE\0"
  "KIRAT RAI DIGIT FOUR\0"
  "KIRAT RAI DIGIT NINE\0"
  "KIRAT RAI DIGIT ONE\0"
  "KIRAT RAI DIGIT SEVEN\0"
  "KIRAT RAI DIGIT SIX\0"
  "KIRAT RAI DIGIT THREE\0"
  "KIRAT RAI DIGIT TWO\0"
  "KIRAT RAI DIGIT ZERO\0"
  "KIRAT RAI DOUBLE DANDA\0"
  "KIRAT RAI LETTER A\0"
  "KIRAT RAI LETTER BA\0"
  "KIRAT RAI LETTER BHA\0"
  "KIRAT RAI LETTER CA\0"
  "KIRAT RAI LETTER CHA\0"
  "KIRAT RAI LETTER DA\0"
  "KIRAT RAI LETTER DDA\0"
  "KIRAT RAI LETTER DDHA\0"
  "KIRAT RAI LETTER DHA\0"
  "KIRAT RAI LETTER GA\0"
  "KIRAT RAI LETTER GHA\0"
  "KIRAT RAI LETTER HA\0"
  "KIRAT RAI LETTER JA\0"
  "KIRAT RAI LETTER JHA\0"
  "KIRAT RAI LETTER KA\0"
  "KIRAT RAI LETTER KHA\0"
  "KIRAT RAI LETTER LA\0"
  "KIRAT RAI LETTER MA\0"
  "KIRAT RAI LETTER NA\0"
  "KIRAT RAI LETTER NGA\0"
  "KIRAT RAI LETTER NYA\0"
  "KIRAT RAI LETTER PA\0"
  "KIRAT RAI LETTER PHA\0"
  "KIRAT RAI LETTER RA\0"
  "KIRAT RAI LETTER SA\0"
  "KIRAT RAI LETTER SHA\0"
  "KIRAT RAI LETTER TA\0"
  "KIRAT RAI LETTER THA\0"
  "KIRAT RAI LETTER TTA\0"
  "KIRAT RAI LETTER TTHA\0"
  "KIRAT RAI LETTER VA\0"
  "KIRAT RAI LETTER YA\0"
  "KIRAT RAI SIGN ANUSVARA\0"
  "KIRAT RAI SIGN SAAT\0"
  "KIRAT RAI SIGN TONPI\0"
  "KIRAT RAI SIGN VIRAMA\0"
  "KIRAT RAI SIGN VISARGA\0"
  "KIRAT RAI SIGN YUPI\0"
  "KIRAT RAI VOWEL SIGN AA\0"
  "KIRAT RAI VOWEL SIGN AI\0"
  "KIRAT RAI VOWEL SIGN AU\0"
  "KIRAT RAI VOWEL SIGN E\0"
  "KIRAT RAI VOWEL SIGN I\0"
  "KIRAT RAI VOWEL SIGN O\0"
  "KIRAT RAI VOWEL SIGN U\0"
  "KIRAT RAI VOWEL SIGN UE\0"
  "KISS\0"
  "KISS MARK\0"
  "KISSING CAT FACE WITH CLOSED EYES\0"
  "KISSING FACE\0"
  "KISSING FACE WITH CLOSED EYES\0"
  "KISSING FACE WITH SMILING EYES\0"
  "KITE\0"
  "KIWIFRUIT\0"
  "KNEELING PERSON\0"
  "KNOT\0"
  "KOALA\0"
  "KOREAN STANDARD SYMBOL\0"
  "KRONOS\0"
  "L B BAR SYMBOL\0"
  "LAB COAT\0"
  "LABEL\0"
  "LACROSSE STICK AND BALL\0"
  "LADDER\0"
  "LADY BEETLE\0"
  "LANGUAGE TAG\0"
  "LAO CANCELLATION MARK\0"
  "LAO DIGIT EIGHT\0"
  "LAO DIGIT FIVE\0"
  "LAO DIGIT FOUR\0"
  "LAO DIGIT NINE\0"
  "LAO DIGIT ONE\0"
  "LAO DIGIT SEVEN\0"
  "LAO DIGIT SIX\0"
  "LAO DIGIT THREE\0"
  "LAO DIGIT TWO\0"
  "LAO DIGIT ZERO\0"
  "LAO ELLIPSIS\0"
  "LAO HO MO\0"
  "LAO HO NO\0"
  "LAO KO LA\0"
  "LAO LETTER BO\0"
  "LAO LETTER CO\0"
  "LAO LETTER DO\0"
  "LAO LETTER FO SUNG\0"
  "LAO LETTER FO TAM\0"
  "LAO LETTER HO SUNG\0"
  "LAO LETTER HO TAM\0"
  "LAO LETTER KHMU GO\0"
  "LAO LETTER KHMU NYO\0"
  "LAO LETTER KHO SUNG\0"
  "LAO LETTER KHO TAM\0"
  "LAO LETTER KO\0"
  "LAO LETTER LO LING\0"
  "LAO LETTER LO LOOT\0"
  "LAO LETTER MO\0"
  "LAO LETTER NGO\0"
  "LAO LETTER NO\0"
  "LAO LETTER NYO\0"
  "LAO LETTER O\0"
  "LAO LETTER PALI BHA\0"
  "LAO LETTER PALI CHA\0"
  "LAO LETTER PALI DDA\0"
  "LAO LETTER PALI DDHA\0"
  "LAO LETTER PALI DHA\0"
  "LAO LETTER PALI GHA\0"
  "LAO LETTER PALI JHA\0"
  "LAO LETTER PALI LLA\0"
  "LAO LETTER PALI NNA\0"
  "LAO LETTER PALI NYA\0"
  "LAO LETTER PALI TTA\0"
  "LAO LETTER PALI TTHA\0"
  "LAO LETTER PHO SUNG\0"
  "LAO LETTER PHO TAM\0"
  "LAO LETTER PO\0"
  "LAO LETTER SANSKRIT SHA\0"
  "LAO LETTER SANSKRIT SSA\0"
  "LAO LETTER SO SUNG\0"
  "LAO LETTER SO TAM\0"
  "LAO LETTER THO SUNG\0"
  "LAO LETTER THO TAM\0"
  "LAO LETTER TO\0"
  "LAO LETTER WO\0"
  "LAO LETTER YO\0"
  "LAO NIGGAHITA\0"
  "LAO SEMIVOWEL SIGN LO\0"
  "LAO SEMIVOWEL SIGN NYO\0"
  "LAO SIGN PALI VIRAMA\0"
  "LAO TONE MAI CATAWA\0"
  "LAO TONE MAI EK\0"
  "LAO TONE MAI THO\0"
  "LAO TONE MAI TI\0"
  "LAO VOWEL SIGN A\0"
  "LAO VOWEL SIGN AA\0"
  "LAO VOWEL SIGN AI\0"
  "LAO VOWEL SIGN AM\0"
  "LAO VOWEL SIGN AY\0"
  "LAO VOWEL SIGN E\0"
  "LAO VOWEL SIGN EI\0"
  "LAO VOWEL SIGN I\0"
  "LAO VOWEL SIGN II\0"
  "LAO VOWEL SIGN MAI KAN\0"
  "LAO VOWEL SIGN MAI KON\0"
  "LAO VOWEL SIGN O\0"
  "LAO VOWEL SIGN U\0"
  "LAO VOWEL SIGN UU\0"
  "LAO VOWEL SIGN Y\0"
  "LAO VOWEL SIGN YY\0"
  "LAO YAMAKKAN\0"
  "LARGE BLUE CIRCLE\0"
  "LARGE BLUE DIAMOND\0"
  "LARGE BLUE SQUARE\0"
  "LARGE BROWN CIRCLE\0"
  "LARGE BROWN SQUARE\0"
  "LARGE CIRCLE\0"
  "LARGE DOWN TACK\0"
  "LARGE GREEN CIRCLE\0"
  "LARGE GREEN SQUARE\0"
  "LARGE LEFT TRIANGLE OPERATOR\0"
  "LARGE ONE DOT OVER TWO DOTS PUNCTUATION\0"
  "LARGE ONE RING OVER TWO RINGS PUNCTUATION\0"
  "LARGE ORANGE CIRCLE\0"
  "LARGE ORANGE DIAMOND\0"
  "LARGE ORANGE SQUARE\0"
  "LARGE PURPLE CIRCLE\0"
  "LARGE PURPLE SQUARE\0"
  "LARGE RED CIRCLE\0"
  "LARGE RED SQUARE\0"
  "LARGE TRIPLE VERTICAL BAR OPERATOR\0"
  "LARGE TWO DOTS OVER ONE DOT PUNCTUATION\0"
  "LARGE TWO RINGS OVER ONE RING PUNCTUATION\0"
  "LARGE TYPE PIECE CENTRE OF K\0"
  "LARGE TYPE PIECE CENTRE OF X\0"
  "LARGE TYPE PIECE CENTRE OF Y\0"
  "LARGE TYPE PIECE CENTRE OF Z WITH CROSSBAR\0"
  "LARGE TYPE PIECE CROSSBAR\0"
  "LARGE TYPE PIECE CROSSBAR WITH LOWER STEM\0"
  "LARGE TYPE PIECE CROSSBAR WITH UPPER STEM\0"
  "LARGE TYPE PIECE DIAGONAL LOWER LEFT\0"
  "LARGE TYPE PIECE DIAGONAL LOWER RIGHT\0"
  "LARGE TYPE PIECE DIAGONAL UPPER LEFT\0"
  "LARGE TYPE PIECE DIAGONAL UPPER LEFT AND LOWER LEFT\0"
  "LARGE TYPE PIECE DIAGONAL UPPER RIGHT\0"
  "LARGE TYPE PIECE DIAGONAL UPPER RIGHT AND LOWER RIGHT\0"
  "LARGE TYPE PIECE LEFT ARM\0"
  "LARGE TYPE PIECE LOWER HALF VERTEX OF M\0"
  "LARGE TYPE PIECE LOWER HALF VERTEX OF W\0"
  "LARGE TYPE PIECE LOWER LEFT AND UPPER LEFT ARC\0"
  "LARGE TYPE PIECE LOWER LEFT ARC\0"
  "LARGE TYPE PIECE LOWER LEFT CORNER\0"
  "LARGE TYPE PIECE LOWER LEFT CROTCH\0"
  "LARGE TYPE PIECE LOWER RIGHT AND UPPER RIGHT ARC\0"
  "LARGE TYPE PIECE LOWER RIGHT ARC\0"
  "LARGE TYPE PIECE LOWER RIGHT ARC WITH TAIL\0"
  "LARGE TYPE PIECE LOWER RIGHT CORNER\0"
  "LARGE TYPE PIECE LOWER RIGHT CROTCH\0"
  "LARGE TYPE PIECE LOWER TERMINAL\0"
  "LARGE TYPE PIECE RAISED UPPER LEFT ARC\0"
  "LARGE TYPE PIECE RIGHT ARM\0"
  "LARGE TYPE PIECE SHORT LOWER TERMINAL\0"
  "LARGE TYPE PIECE SHORT UPPER TERMINAL\0"
  "LARGE TYPE PIECE STEM\0"
  "LARGE TYPE PIECE STEM WITH CROSSBAR\0"
  "LARGE TYPE PIECE STEM WITH LEFT CROSSBAR\0"
  "LARGE TYPE PIECE STEM WITH LEFT JOINT\0"
  "LARGE TYPE PIECE STEM WITH RIGHT CROSSBAR\0"
  "LARGE TYPE PIECE STEM-12\0"
  "LARGE TYPE PIECE STEM-1234\0"
  "LARGE TYPE PIECE STEM-2\0"
  "LARGE TYPE PIECE STEM-23\0"
  "LARGE TYPE PIECE STEM-234\0"
  "LARGE TYPE PIECE STEM-2345\0"
  "LARGE TYPE PIECE STEM-3\0"
  "LARGE TYPE PIECE STEM-34\0"
  "LARGE TYPE PIECE STEM-4\0"
  "LARGE TYPE PIECE STEM-45\0"
  "LARGE TYPE PIECE UPPER HALF VERTEX OF M\0"
  "LARGE TYPE PIECE UPPER HALF VERTEX OF W\0"
  "LARGE TYPE PIECE UPPER LEFT ARC\0"
  "LARGE TYPE PIECE UPPER LEFT CORNER\0"
  "LARGE TYPE PIECE UPPER LEFT CROTCH\0"
  "LARGE TYPE PIECE UPPER RIGHT ARC\0"
  "LARGE TYPE PIECE UPPER RIGHT CORNER\0"
  "LARGE TYPE PIECE UPPER RIGHT CROTCH\0"
  "LARGE TYPE PIECE UPPER TERMINAL\0"
  "LARGE TYPE PIECE VERTEX OF V\0"
  "LARGE UP TACK\0"
  "LARGE YELLOW CIRCLE\0"
  "LARGE YELLOW SQUARE\0"
  "LARGER THAN\0"
  "LARGER THAN OR EQUAL TO\0"
  "LARI SIGN\0"
  "LAST QUARTER MOON\0"
  "LAST QUARTER MOON SYMBOL\0"
  "LAST QUARTER MOON WITH FACE\0"
  "LATIN CAPITAL LETTER A\0"
  "LATIN CAPITAL LETTER A WITH ACUTE\0"
  "LATIN CAPITAL LETTER A WITH BREVE\0"
  "LATIN CAPITAL LETTER A WITH BREVE AND ACUTE\0"
  "LATIN CAPITAL LETTER A WITH BREVE AND DOT BELOW\0"
  "LATIN CAPITAL LETTER A WITH BREVE AND GRAVE\0"
  "LATIN CAPITAL LETTER A WITH BREVE AND HOOK ABOVE\0"
  "LATIN CAPITAL LETTER A WITH BREVE AND TILDE\0"
  "LATIN CAPITAL LETTER A WITH CARON\0"
  "LATIN CAPITAL LETTER A WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND ACUTE\0"
  "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND DOT BELOW\0"
  "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND GRAVE\0"
  "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE\0"
  "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND TILDE\0"
  "LATIN CAPITAL LETTER A WITH DIAERESIS\0"
  "LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON\0"
  "LATIN CAPITAL LETTER A WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER A WITH DOT ABOVE AND MACRON\0"
  "LATIN CAPITAL LETTER A WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER A WITH DOUBLE GRAVE\0"
  "LATIN CAPITAL LETTER A WITH GRAVE\0"
  "LATIN CAPITAL LETTER A WITH HOOK ABOVE\0"
  "LATIN CAPITAL LETTER A WITH INVERTED BREVE\0"
  "LATIN CAPITAL LETTER A WITH MACRON\0"
  "LATIN CAPITAL LETTER A WITH OGONEK\0"
  "LATIN CAPITAL LETTER A WITH RING ABOVE\0"
  "LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE\0"
  "LATIN CAPITAL LETTER A WITH RING BELOW\0"
  "LATIN CAPITAL LETTER A WITH STROKE\0"
  "LATIN CAPITAL LETTER A WITH TILDE\0"
  "LATIN CAPITAL LETTER AA\0"
  "LATIN CAPITAL LETTER AE\0"
  "LATIN CAPITAL LETTER AE WITH ACUTE\0"
  "LATIN CAPITAL LETTER AE WITH MACRON\0"
  "LATIN CAPITAL LETTER AFRICAN D\0"
  "LATIN CAPITAL LETTER ALPHA\0"
  "LATIN CAPITAL LETTER ANGLICANA W\0"
  "LATIN CAPITAL LETTER AO\0"
  "LATIN CAPITAL LETTER AU\0"
  "LATIN CAPITAL LETTER AV\0"
  "LATIN CAPITAL LETTER AV WITH HORIZONTAL BAR\0"
  "LATIN CAPITAL LETTER AY\0"
  "LATIN CAPITAL LETTER B\0"
  "LATIN CAPITAL LETTER B WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER B WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER B WITH FLOURISH\0"
  "LATIN CAPITAL LETTER B WITH HOOK\0"
  "LATIN CAPITAL LETTER B WITH LINE BELOW\0"
  "LATIN CAPITAL LETTER B WITH STROKE\0"
  "LATIN CAPITAL LETTER B WITH TOPBAR\0"
  "LATIN CAPITAL LETTER BETA\0"
  "LATIN CAPITAL LETTER BROKEN L\0"
  "LATIN CAPITAL LETTER C\0"
  "LATIN CAPITAL LETTER C WITH ACUTE\0"
  "LATIN CAPITAL LETTER C WITH BAR\0"
  "LATIN CAPITAL LETTER C WITH CARON\0"
  "LATIN CAPITAL LETTER C WITH CEDILLA\0"
  "LATIN CAPITAL LETTER C WITH CEDILLA AND ACUTE\0"
  "LATIN CAPITAL LETTER C WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER C WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER C WITH HOOK\0"
  "LATIN CAPITAL LETTER C WITH PALATAL HOOK\0"
  "LATIN CAPITAL LETTER C WITH STROKE\0"
  "LATIN CAPITAL LETTER CHI\0"
  "LATIN CAPITAL LETTER CLOSED INSULAR G\0"
  "LATIN CAPITAL LETTER CON\0"
  "LATIN CAPITAL LETTER CUATRILLO\0"
  "LATIN CAPITAL LETTER CUATRILLO WITH COMMA\0"
  "LATIN CAPITAL LETTER D\0"
  "LATIN CAPITAL LETTER D WITH CARON\0"
  "LATIN CAPITAL LETTER D WITH CEDILLA\0"
  "LATIN CAPITAL LETTER D WITH CIRCUMFLEX BELOW\0"
  "LATIN CAPITAL LETTER D WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER D WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER D WITH HOOK\0"
  "LATIN CAPITAL LETTER D WITH LINE BELOW\0"
  "LATIN CAPITAL LETTER D WITH SHORT STROKE OVERLAY\0"
  "LATIN CAPITAL LETTER D WITH SMALL LETTER Z\0"
  "LATIN CAPITAL LETTER D WITH SMALL LETTER Z WITH CARON\0"
  "LATIN CAPITAL LETTER D WITH STROKE\0"
  "LATIN CAPITAL LETTER D WITH TOPBAR\0"
  "LATIN CAPITAL LETTER DZ\0"
  "LATIN CAPITAL LETTER DZ WITH CARON\0"
  "LATIN CAPITAL LETTER E\0"
  "LATIN CAPITAL LETTER E WITH ACUTE\0"
  "LATIN CAPITAL LETTER E WITH BREVE\0"
  "LATIN CAPITAL LETTER E WITH CARON\0"
  "LATIN CAPITAL LETTER E WITH CEDILLA\0"
  "LATIN CAPITAL LETTER E WITH CEDILLA AND BREVE\0"
  "LATIN CAPITAL LETTER E WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND ACUTE\0"
  "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND DOT BELOW\0"
  "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND GRAVE\0"
  "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE\0"
  "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND TILDE\0"
  "LATIN CAPITAL LETTER E WITH CIRCUMFLEX BELOW\0"
  "LATIN CAPITAL LETTER E WITH DIAERESIS\0"
  "LATIN CAPITAL LETTER E WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER E WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER E WITH DOUBLE GRAVE\0"
  "LATIN CAPITAL LETTER E WITH GRAVE\0"
  "LATIN CAPITAL LETTER E WITH HOOK ABOVE\0"
  "LATIN CAPITAL LETTER E WITH INVERTED BREVE\0"
  "LATIN CAPITAL LETTER E WITH MACRON\0"
  "LATIN CAPITAL LETTER E WITH MACRON AND ACUTE\0"
  "LATIN CAPITAL LETTER E WITH MACRON AND GRAVE\0"
  "LATIN CAPITAL LETTER E WITH OGONEK\0"
  "LATIN CAPITAL LETTER E WITH STROKE\0"
  "LATIN CAPITAL LETTER E WITH TILDE\0"
  "LATIN CAPITAL LETTER E WITH TILDE BELOW\0"
  "LATIN CAPITAL LETTER EGYPTOLOGICAL AIN\0"
  "LATIN CAPITAL LETTER EGYPTOLOGICAL ALEF\0"
  "LATIN CAPITAL LETTER ENG\0"
  "LATIN CAPITAL LETTER ESH\0"
  "LATIN CAPITAL LETTER ET\0"
  "LATIN CAPITAL LETTER ETH\0"
  "LATIN CAPITAL LETTER EZH\0"
  "LATIN CAPITAL LETTER EZH REVERSED\0"
  "LATIN CAPITAL LETTER EZH WITH CARON\0"
  "LATIN CAPITAL LETTER F\0"
  "LATIN CAPITAL LETTER F WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER F WITH HOOK\0"
  "LATIN CAPITAL LETTER F WITH STROKE\0"
  "LATIN CAPITAL LETTER G\0"
  "LATIN CAPITAL LETTER G WITH ACUTE\0"
  "LATIN CAPITAL LETTER G WITH BREVE\0"
  "LATIN CAPITAL LETTER G WITH CARON\0"
  "LATIN CAPITAL LETTER G WITH CEDILLA\0"
  "LATIN CAPITAL LETTER G WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER G WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER G WITH HOOK\0"
  "LATIN CAPITAL LETTER G WITH MACRON\0"
  "LATIN CAPITAL LETTER G WITH OBLIQUE STROKE\0"
  "LATIN CAPITAL LETTER G WITH STROKE\0"
  "LATIN CAPITAL LETTER GAMMA\0"
  "LATIN CAPITAL LETTER GLOTTAL A\0"
  "LATIN CAPITAL LETTER GLOTTAL I\0"
  "LATIN CAPITAL LETTER GLOTTAL STOP\0"
  "LATIN CAPITAL LETTER GLOTTAL U\0"
  "LATIN CAPITAL LETTER H\0"
  "LATIN CAPITAL LETTER H WITH BREVE BELOW\0"
  "LATIN CAPITAL LETTER H WITH CARON\0"
  "LATIN CAPITAL LETTER H WITH CEDILLA\0"
  "LATIN CAPITAL LETTER H WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER H WITH DESCENDER\0"
  "LATIN CAPITAL LETTER H WITH DIAERESIS\0"
  "LATIN CAPITAL LETTER H WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER H WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER H WITH HOOK\0"
  "LATIN CAPITAL LETTER H WITH STROKE\0"
  "LATIN CAPITAL LETTER HALF H\0"
  "LATIN CAPITAL LETTER HENG\0"
  "LATIN CAPITAL LETTER HWAIR\0"
  "LATIN CAPITAL LETTER I\0"
  "LATIN CAPITAL LETTER I WITH ACUTE\0"
  "LATIN CAPITAL LETTER I WITH BREVE\0"
  "LATIN CAPITAL LETTER I WITH CARON\0"
  "LATIN CAPITAL LETTER I WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER I WITH DIAERESIS\0"
  "LATIN CAPITAL LETTER I WITH DIAERESIS AND ACUTE\0"
  "LATIN CAPITAL LETTER I WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER I WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER I WITH DOUBLE GRAVE\0"
  "LATIN CAPITAL LETTER I WITH GRAVE\0"
  "LATIN CAPITAL LETTER I WITH HOOK ABOVE\0"
  "LATIN CAPITAL LETTER I WITH INVERTED BREVE\0"
  "LATIN CAPITAL LETTER I WITH MACRON\0"
  "LATIN CAPITAL LETTER I WITH OGONEK\0"
  "LATIN CAPITAL LETTER I WITH STROKE\0"
  "LATIN CAPITAL LETTER I WITH TILDE\0"
  "LATIN CAPITAL LETTER I WITH TILDE BELOW\0"
  "LATIN CAPITAL LETTER INSULAR D\0"
  "LATIN CAPITAL LETTER INSULAR F\0"
  "LATIN CAPITAL LETTER INSULAR G\0"
  "LATIN CAPITAL LETTER INSULAR R\0"
  "LATIN CAPITAL LETTER INSULAR S\0"
  "LATIN CAPITAL LETTER INSULAR T\0"
  "LATIN CAPITAL LETTER IOTA\0"
  "LATIN CAPITAL LETTER IS\0"
  "LATIN CAPITAL LETTER J\0"
  "LATIN CAPITAL LETTER J WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER J WITH CROSSED-TAIL\0"
  "LATIN CAPITAL LETTER J WITH STROKE\0"
  "LATIN CAPITAL LETTER K\0"
  "LATIN CAPITAL LETTER K WITH ACUTE\0"
  "LATIN CAPITAL LETTER K WITH CARON\0"
  "LATIN CAPITAL LETTER K WITH CEDILLA\0"
  "LATIN CAPITAL LETTER K WITH DESCENDER\0"
  "LATIN CAPITAL LETTER K WITH DIAGONAL STROKE\0"
  "LATIN CAPITAL LETTER K WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER K WITH HOOK\0"
  "LATIN CAPITAL LETTER K WITH LINE BELOW\0"
  "LATIN CAPITAL LETTER K WITH OBLIQUE STROKE\0"
  "LATIN CAPITAL LETTER K WITH STROKE\0"
  "LATIN CAPITAL LETTER K WITH STROKE AND DIAGONAL STROKE\0"
  "LATIN CAPITAL LETTER L\0"
  "LATIN CAPITAL LETTER L WITH ACUTE\0"
  "LATIN CAPITAL LETTER L WITH BAR\0"
  "LATIN CAPITAL LETTER L WITH BELT\0"
  "LATIN CAPITAL LETTER L WITH CARON\0"
  "LATIN CAPITAL LETTER L WITH CEDILLA\0"
  "LATIN CAPITAL LETTER L WITH CIRCUMFLEX BELOW\0"
  "LATIN CAPITAL LETTER L WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER L WITH DOT BELOW AND MACRON\0"
  "LATIN CAPITAL LETTER L WITH DOUBLE BAR\0"
  "LATIN CAPITAL LETTER L WITH HIGH STROKE\0"
  "LATIN CAPITAL LETTER L WITH LINE BELOW\0"
  "LATIN CAPITAL LETTER L WITH MIDDLE DOT\0"
  "LATIN CAPITAL LETTER L WITH MIDDLE TILDE\0"
  "LATIN CAPITAL LETTER L WITH SMALL LETTER J\0"
  "LATIN CAPITAL LETTER L WITH STROKE\0"
  "LATIN CAPITAL LETTER LAMBDA\0"
  "LATIN CAPITAL LETTER LAMBDA WITH STROKE\0"
  "LATIN CAPITAL LETTER LJ\0"
  "LATIN CAPITAL LETTER M\0"
  "LATIN CAPITAL LETTER M WITH ACUTE\0"
  "LATIN CAPITAL LETTER M WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER M WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER M WITH HOOK\0"
  "LATIN CAPITAL LETTER MIDDLE SCOTS S\0"
  "LATIN CAPITAL LETTER MIDDLE-WELSH LL\0"
  "LATIN CAPITAL LETTER MIDDLE-WELSH V\0"
  "LATIN CAPITAL LETTER N\0"
  "LATIN CAPITAL LETTER N WITH ACUTE\0"
  "LATIN CAPITAL LETTER N WITH CARON\0"
  "LATIN CAPITAL LETTER N WITH CEDILLA\0"
  "LATIN CAPITAL LETTER N WITH CIRCUMFLEX BELOW\0"
  "LATIN CAPITAL LETTER N WITH DESCENDER\0"
  "LATIN CAPITAL LETTER N WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER N WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER N WITH GRAVE\0"
  "LATIN CAPITAL LETTER N WITH LEFT HOOK\0"
  "LATIN CAPITAL LETTER N WITH LINE BELOW\0"
  "LATIN CAPITAL LETTER N WITH LONG RIGHT LEG\0"
  "LATIN CAPITAL LETTER N WITH OBLIQUE STROKE\0"
  "LATIN CAPITAL LETTER N WITH SMALL LETTER J\0"
  "LATIN CAPITAL LETTER N WITH TILDE\0"
  "LATIN CAPITAL LETTER NJ\0"
  "LATIN CAPITAL LETTER O\0"
  "LATIN CAPITAL LETTER O WITH ACUTE\0"
  "LATIN CAPITAL LETTER O WITH BREVE\0"
  "LATIN CAPITAL LETTER O WITH CARON\0"
  "LATIN CAPITAL LETTER O WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND ACUTE\0"
  "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW\0"
  "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND GRAVE\0"
  "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE\0"
  "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND TILDE\0"
  "LATIN CAPITAL LETTER O WITH DIAERESIS\0"
  "LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON\0"
  "LATIN CAPITAL LETTER O WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER O WITH DOT ABOVE AND MACRON\0"
  "LATIN CAPITAL LETTER O WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER O WITH DOUBLE ACUTE\0"
  "LATIN CAPITAL LETTER O WITH DOUBLE GRAVE\0"
  "LATIN CAPITAL LETTER O WITH GRAVE\0"
  "LATIN CAPITAL LETTER O WITH HOOK ABOVE\0"
  "LATIN CAPITAL LETTER O WITH HORN\0"
  "LATIN CAPITAL LETTER O WITH HORN AND ACUTE\0"
  "LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW\0"
  "LATIN CAPITAL LETTER O WITH HORN AND GRAVE\0"
  "LATIN CAPITAL LETTER O WITH HORN AND HOOK ABOVE\0"
  "LATIN CAPITAL LETTER O WITH HORN AND TILDE\0"
  "LATIN CAPITAL LETTER O WITH INVERTED BREVE\0"
  "LATIN CAPITAL LETTER O WITH LONG STROKE OVERLAY\0"
  "LATIN CAPITAL LETTER O WITH LOOP\0"
  "LATIN CAPITAL LETTER O WITH MACRON\0"
  "LATIN CAPITAL LETTER O WITH MACRON AND ACUTE\0"
  "LATIN CAPITAL LETTER O WITH MACRON AND GRAVE\0"
  "LATIN CAPITAL LETTER O WITH MIDDLE TILDE\0"
  "LATIN CAPITAL LETTER O WITH OGONEK\0"
  "LATIN CAPITAL LETTER O WITH OGONEK AND MACRON\0"
  "LATIN CAPITAL LETTER O WITH STROKE\0"
  "LATIN CAPITAL LETTER O WITH STROKE AND ACUTE\0"
  "LATIN CAPITAL LETTER O WITH TILDE\0"
  "LATIN CAPITAL LETTER O WITH TILDE AND ACUTE\0"
  "LATIN CAPITAL LETTER O WITH TILDE AND DIAERESIS\0"
  "LATIN CAPITAL LETTER O WITH TILDE AND MACRON\0"
  "LATIN CAPITAL LETTER OI\0"
  "LATIN CAPITAL LETTER OLD POLISH O\0"
  "LATIN CAPITAL LETTER OMEGA\0"
  "LATIN CAPITAL LETTER OO\0"
  "LATIN CAPITAL LETTER OPEN E\0"
  "LATIN CAPITAL LETTER OPEN O\0"
  "LATIN CAPITAL LETTER OU\0"
  "LATIN CAPITAL LETTER P\0"
  "LATIN CAPITAL LETTER P WITH ACUTE\0"
  "LATIN CAPITAL LETTER P WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER P WITH FLOURISH\0"
  "LATIN CAPITAL LETTER P WITH HOOK\0"
  "LATIN CAPITAL LETTER P WITH SQUIRREL TAIL\0"
  "LATIN CAPITAL LETTER P WITH STROKE\0"
  "LATIN CAPITAL LETTER P WITH STROKE THROUGH DESCENDER\0"
  "LATIN CAPITAL LETTER Q\0"
  "LATIN CAPITAL LETTER Q WITH DIAGONAL STROKE\0"
  "LATIN CAPITAL LETTER Q WITH STROKE THROUGH DESCENDER\0"
  "LATIN CAPITAL LETTER R\0"
  "LATIN CAPITAL LETTER R ROTUNDA\0"
  "LATIN CAPITAL LETTER R WITH ACUTE\0"
  "LATIN CAPITAL LETTER R WITH CARON\0"
  "LATIN CAPITAL LETTER R WITH CEDILLA\0"
  "LATIN CAPITAL LETTER R WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER R WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER R WITH DOT BELOW AND MACRON\0"
  "LATIN CAPITAL LETTER R WITH DOUBLE GRAVE\0"
  "LATIN CAPITAL LETTER R WITH INVERTED BREVE\0"
  "LATIN CAPITAL LETTER R WITH LINE BELOW\0"
  "LATIN CAPITAL LETTER R WITH OBLIQUE STROKE\0"
  "LATIN CAPITAL LETTER R WITH STROKE\0"
  "LATIN CAPITAL LETTER R WITH TAIL\0"
  "LATIN CAPITAL LETTER RAMS HORN\0"
  "LATIN CAPITAL LETTER REVERSED C WITH DOT\0"
  "LATIN CAPITAL LETTER REVERSED E\0"
  "LATIN CAPITAL LETTER REVERSED HALF H\0"
  "LATIN CAPITAL LETTER REVERSED OPEN E\0"
  "LATIN CAPITAL LETTER RUM ROTUNDA\0"
  "LATIN CAPITAL LETTER S\0"
  "LATIN CAPITAL LETTER S WITH ACUTE\0"
  "LATIN CAPITAL LETTER S WITH ACUTE AND DOT ABOVE\0"
  "LATIN CAPITAL LETTER S WITH CARON\0"
  "LATIN CAPITAL LETTER S WITH CARON AND DOT ABOVE\0"
  "LATIN CAPITAL LETTER S WITH CEDILLA\0"
  "LATIN CAPITAL LETTER S WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER S WITH COMMA BELOW\0"
  "LATIN CAPITAL LETTER S WITH DIAGONAL STROKE\0"
  "LATIN CAPITAL LETTER S WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER S WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER S WITH DOT BELOW AND DOT ABOVE\0"
  "LATIN CAPITAL LETTER S WITH HOOK\0"
  "LATIN CAPITAL LETTER S WITH OBLIQUE STROKE\0"
  "LATIN CAPITAL LETTER S WITH SHORT STROKE OVERLAY\0"
  "LATIN CAPITAL LETTER S WITH SWASH TAIL\0"
  "LATIN CAPITAL LETTER SALTILLO\0"
  "LATIN CAPITAL LETTER SCHWA\0"
  "LATIN CAPITAL LETTER SCRIPT G\0"
  "LATIN CAPITAL LETTER SHARP S\0"
  "LATIN CAPITAL LETTER SIGMOID S\0"
  "LATIN CAPITAL LETTER SMALL CAPITAL I\0"
  "LATIN CAPITAL LETTER SMALL Q WITH HOOK TAIL\0"
  "LATIN CAPITAL LETTER T\0"
  "LATIN CAPITAL LETTER T WITH CARON\0"
  "LATIN CAPITAL LETTER T WITH CEDILLA\0"
  "LATIN CAPITAL LETTER T WITH CIRCUMFLEX BELOW\0"
  "LATIN CAPITAL LETTER T WITH COMMA BELOW\0"
  "LATIN CAPITAL LETTER T WITH DIAGONAL STROKE\0"
  "LATIN CAPITAL LETTER T WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER T WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER T WITH HOOK\0"
  "LATIN CAPITAL LETTER T WITH LINE BELOW\0"
  "LATIN CAPITAL LETTER T WITH RETROFLEX HOOK\0"
  "LATIN CAPITAL LETTER T WITH STROKE\0"
  "LATIN CAPITAL LETTER THORN\0"
  "LATIN CAPITAL LETTER THORN WITH STROKE\0"
  "LATIN CAPITAL LETTER THORN WITH STROKE THROUGH DESCENDER\0"
  "LATIN CAPITAL LETTER TONE FIVE\0"
  "LATIN CAPITAL LETTER TONE SIX\0"
  "LATIN CAPITAL LETTER TONE TWO\0"
  "LATIN CAPITAL LETTER TRESILLO\0"
  "LATIN CAPITAL LETTER TURNED A\0"
  "LATIN CAPITAL LETTER TURNED ALPHA\0"
  "LATIN CAPITAL LETTER TURNED H\0"
  "LATIN CAPITAL LETTER TURNED INSULAR G\0"
  "LATIN CAPITAL LETTER TURNED K\0"
  "LATIN CAPITAL LETTER TURNED L\0"
  "LATIN CAPITAL LETTER TURNED M\0"
  "LATIN CAPITAL LETTER TURNED T\0"
  "LATIN CAPITAL LETTER TURNED V\0"
  "LATIN CAPITAL LETTER TZ\0"
  "LATIN CAPITAL LETTER U\0"
  "LATIN CAPITAL LETTER U BAR\0"
  "LATIN CAPITAL LETTER U WITH ACUTE\0"
  "LATIN CAPITAL LETTER U WITH BREVE\0"
  "LATIN CAPITAL LETTER U WITH CARON\0"
  "LATIN CAPITAL LETTER U WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER U WITH CIRCUMFLEX BELOW\0"
  "LATIN CAPITAL LETTER U WITH DIAERESIS\0"
  "LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE\0"
  "LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON\0"
  "LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE\0"
  "LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON\0"
  "LATIN CAPITAL LETTER U WITH DIAERESIS BELOW\0"
  "LATIN CAPITAL LETTER U WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER U WITH DOUBLE ACUTE\0"
  "LATIN CAPITAL LETTER U WITH DOUBLE GRAVE\0"
  "LATIN CAPITAL LETTER U WITH GRAVE\0"
  "LATIN CAPITAL LETTER U WITH HOOK ABOVE\0"
  "LATIN CAPITAL LETTER U WITH HORN\0"
  "LATIN CAPITAL LETTER U WITH HORN AND ACUTE\0"
  "LATIN CAPITAL LETTER U WITH HORN AND DOT BELOW\0"
  "LATIN CAPITAL LETTER U WITH HORN AND GRAVE\0"
  "LATIN CAPITAL LETTER U WITH HORN AND HOOK ABOVE\0"
  "LATIN CAPITAL LETTER U WITH HORN AND TILDE\0"
  "LATIN CAPITAL LETTER U WITH INVERTED BREVE\0"
  "LATIN CAPITAL LETTER U WITH MACRON\0"
  "LATIN CAPITAL LETTER U WITH MACRON AND DIAERESIS\0"
  "LATIN CAPITAL LETTER U WITH OGONEK\0"
  "LATIN CAPITAL LETTER U WITH RING ABOVE\0"
  "LATIN CAPITAL LETTER U WITH STROKE\0"
  "LATIN CAPITAL LETTER U WITH TILDE\0"
  "LATIN CAPITAL LETTER U WITH TILDE AND ACUTE\0"
  "LATIN CAPITAL LETTER U WITH TILDE BELOW\0"
  "LATIN CAPITAL LETTER UPSILON\0"
  "LATIN CAPITAL LETTER V\0"
  "LATIN CAPITAL LETTER V WITH DIAGONAL STROKE\0"
  "LATIN CAPITAL LETTER V WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER V WITH HOOK\0"
  "LATIN CAPITAL LETTER V WITH TILDE\0"
  "LATIN CAPITAL LETTER VEND\0"
  "LATIN CAPITAL LETTER VISIGOTHIC Z\0"
  "LATIN CAPITAL LETTER VOLAPUK AE\0"
  "LATIN CAPITAL LETTER VOLAPUK OE\0"
  "LATIN CAPITAL LETTER VOLAPUK UE\0"
  "LATIN CAPITAL LETTER VY\0"
  "LATIN CAPITAL LETTER W\0"
  "LATIN CAPITAL LETTER W WITH ACUTE\0"
  "LATIN CAPITAL LETTER W WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER W WITH DIAERESIS\0"
  "LATIN CAPITAL LETTER W WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER W WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER W WITH GRAVE\0"
  "LATIN CAPITAL LETTER W WITH HOOK\0"
  "LATIN CAPITAL LETTER WYNN\0"
  "LATIN CAPITAL LETTER X\0"
  "LATIN CAPITAL LETTER X WITH DIAERESIS\0"
  "LATIN CAPITAL LETTER X WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER Y\0"
  "LATIN CAPITAL LETTER Y WITH ACUTE\0"
  "LATIN CAPITAL LETTER Y WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER Y WITH DIAERESIS\0"
  "LATIN CAPITAL LETTER Y WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER Y WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER Y WITH GRAVE\0"
  "LATIN CAPITAL LETTER Y WITH HOOK\0"
  "LATIN CAPITAL LETTER Y WITH HOOK ABOVE\0"
  "LATIN CAPITAL LETTER Y WITH LOOP\0"
  "LATIN CAPITAL LETTER Y WITH MACRON\0"
  "LATIN CAPITAL LETTER Y WITH STROKE\0"
  "LATIN CAPITAL LETTER Y WITH TILDE\0"
  "LATIN CAPITAL LETTER YOGH\0"
  "LATIN CAPITAL LETTER Z\0"
  "LATIN CAPITAL LETTER Z WITH ACUTE\0"
  "LATIN CAPITAL LETTER Z WITH CARON\0"
  "LATIN CAPITAL LETTER Z WITH CIRCUMFLEX\0"
  "LATIN CAPITAL LETTER Z WITH DESCENDER\0"
  "LATIN CAPITAL LETTER Z WITH DOT ABOVE\0"
  "LATIN CAPITAL LETTER Z WITH DOT BELOW\0"
  "LATIN CAPITAL LETTER Z WITH HOOK\0"
  "LATIN CAPITAL LETTER Z WITH LINE BELOW\0"
  "LATIN CAPITAL LETTER Z WITH PALATAL HOOK\0"
  "LATIN CAPITAL LETTER Z WITH STROKE\0"
  "LATIN CAPITAL LETTER Z WITH SWASH TAIL\0"
  "LATIN CAPITAL LIGATURE IJ\0"
  "LATIN CAPITAL LIGATURE OE\0"
  "LATIN CROSS\0"
  "LATIN EPIGRAPHIC LETTER ARCHAIC M\0"
  "LATIN EPIGRAPHIC LETTER I LONGA\0"
  "LATIN EPIGRAPHIC LETTER INVERTED M\0"
  "LATIN EPIGRAPHIC LETTER REVERSED F\0"
  "LATIN EPIGRAPHIC LETTER REVERSED P\0"
  "LATIN EPIGRAPHIC LETTER SIDEWAYS I\0"
  "LATIN LETTER AIN\0"
  "LATIN LETTER ALVEOLAR CLICK\0"
  "LATIN LETTER BIDENTAL PERCUSSIVE\0"
  "LATIN LETTER BILABIAL CLICK\0"
  "LATIN LETTER BILABIAL PERCUSSIVE\0"
  "LATIN LETTER DENTAL CLICK\0"
  "LATIN LETTER GLOTTAL STOP\0"
  "LATIN LETTER GLOTTAL STOP WITH STROKE\0"
  "LATIN LETTER INVERTED GLOTTAL STOP\0"
  "LATIN LETTER INVERTED GLOTTAL STOP WITH CURL\0"
  "LATIN LETTER INVERTED GLOTTAL STOP WITH STROKE\0"
  "LATIN LETTER LATERAL CLICK\0"
  "LATIN LETTER PHARYNGEAL VOICED FRICATIVE\0"
  "LATIN LETTER RETROFLEX CLICK\0"
  "LATIN LETTER RETROFLEX CLICK WITH RETROFLEX HOOK\0"
  "LATIN LETTER REVERSED ESH LOOP\0"
  "LATIN LETTER REVERSED GLOTTAL STOP WITH STROKE\0"
  "LATIN LETTER SINOLOGICAL DOT\0"
  "LATIN LETTER SMALL CAPITAL A\0"
  "LATIN LETTER SMALL CAPITAL AE\0"
  "LATIN LETTER SMALL CAPITAL B\0"
  "LATIN LETTER SMALL CAPITAL BARRED B\0"
  "LATIN LETTER SMALL CAPITAL C\0"
  "LATIN LETTER SMALL CAPITAL D\0"
  "LATIN LETTER SMALL CAPITAL E\0"
  "LATIN LETTER SMALL CAPITAL ETH\0"
  "LATIN LETTER SMALL CAPITAL EZH\0"
  "LATIN LETTER SMALL CAPITAL F\0"
  "LATIN LETTER SMALL CAPITAL G\0"
  "LATIN LETTER SMALL CAPITAL G WITH HOOK\0"
  "LATIN LETTER SMALL CAPITAL H\0"
  "LATIN LETTER SMALL CAPITAL I\0"
  "LATIN LETTER SMALL CAPITAL INVERTED R\0"
  "LATIN LETTER SMALL CAPITAL J\0"
  "LATIN LETTER SMALL CAPITAL K\0"
  "LATIN LETTER SMALL CAPITAL L\0"
  "LATIN LETTER SMALL CAPITAL L WITH BELT\0"
  "LATIN LETTER SMALL CAPITAL L WITH STROKE\0"
  "LATIN LETTER SMALL CAPITAL M\0"
  "LATIN LETTER SMALL CAPITAL N\0"
  "LATIN LETTER SMALL CAPITAL O\0"
  "LATIN LETTER SMALL CAPITAL OE\0"
  "LATIN LETTER SMALL CAPITAL OPEN O\0"
  "LATIN LETTER SMALL CAPITAL OU\0"
  "LATIN LETTER SMALL CAPITAL P\0"
  "LATIN LETTER SMALL CAPITAL Q\0"
  "LATIN LETTER SMALL CAPITAL R\0"
  "LATIN LETTER SMALL CAPITAL R WITH RIGHT LEG\0"
  "LATIN LETTER SMALL CAPITAL REVERSED N\0"
  "LATIN LETTER SMALL CAPITAL REVERSED R\0"
  "LATIN LETTER SMALL CAPITAL RUM\0"
  "LATIN LETTER SMALL CAPITAL S\0"
  "LATIN LETTER SMALL CAPITAL T\0"
  "LATIN LETTER SMALL CAPITAL TURNED E\0"
  "LATIN LETTER SMALL CAPITAL TURNED G\0"
  "LATIN LETTER SMALL CAPITAL TURNED K\0"
  "LATIN LETTER SMALL CAPITAL TURNED M\0"
  "LATIN LETTER SMALL CAPITAL TURNED R\0"
  "LATIN LETTER SMALL CAPITAL U\0"
  "LATIN LETTER SMALL CAPITAL V\0"
  "LATIN LETTER SMALL CAPITAL W\0"
  "LATIN LETTER SMALL CAPITAL Y\0"
  "LATIN LETTER SMALL CAPITAL Z\0"
  "LATIN LETTER STRETCHED C\0"
  "LATIN LETTER STRETCHED C WITH CURL\0"
  "LATIN LETTER TWO WITH STROKE\0"
  "LATIN LETTER VOICED LARYNGEAL SPIRANT\0"
  "LATIN LETTER WYNN\0"
  "LATIN LETTER YR\0"
  "LATIN SMALL CAPITAL LETTER I WITH STROKE\0"
  "LATIN SMALL CAPITAL LETTER U WITH STROKE\0"
  "LATIN SMALL LETTER A\0"
  "LATIN SMALL LETTER A REVERSED-SCHWA\0"
  "LATIN SMALL LETTER A WITH ACUTE\0"
  "LATIN SMALL LETTER A WITH BREVE\0"
  "LATIN SMALL LETTER A WITH BREVE AND ACUTE\0"
  "LATIN SMALL LETTER A WITH BREVE AND DOT BELOW\0"
  "LATIN SMALL LETTER A WITH BREVE AND GRAVE\0"
  "LATIN SMALL LETTER A WITH BREVE AND HOOK ABOVE\0"
  "LATIN SMALL LETTER A WITH BREVE AND TILDE\0"
  "LATIN SMALL LETTER A WITH CARON\0"
  "LATIN SMALL LETTER A WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER A WITH CIRCUMFLEX AND ACUTE\0"
  "LATIN SMALL LETTER A WITH CIRCUMFLEX AND DOT BELOW\0"
  "LATIN SMALL LETTER A WITH CIRCUMFLEX AND GRAVE\0"
  "LATIN SMALL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE\0"
  "LATIN SMALL LETTER A WITH CIRCUMFLEX AND TILDE\0"
  "LATIN SMALL LETTER A WITH DIAERESIS\0"
  "LATIN SMALL LETTER A WITH DIAERESIS AND MACRON\0"
  "LATIN SMALL LETTER A WITH DOT ABOVE\0"
  "LATIN SMALL LETTER A WITH DOT ABOVE AND MACRON\0"
  "LATIN SMALL LETTER A WITH DOT BELOW\0"
  "LATIN SMALL LETTER A WITH DOUBLE GRAVE\0"
  "LATIN SMALL LETTER A WITH GRAVE\0"
  "LATIN SMALL LETTER A WITH HOOK ABOVE\0"
  "LATIN SMALL LETTER A WITH INVERTED BREVE\0"
  "LATIN SMALL LETTER A WITH MACRON\0"
  "LATIN SMALL LETTER A WITH OGONEK\0"
  "LATIN SMALL LETTER A WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER A WITH RIGHT HALF RING\0"
  "LATIN SMALL LETTER A WITH RING ABOVE\0"
  "LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE\0"
  "LATIN SMALL LETTER A WITH RING BELOW\0"
  "LATIN SMALL LETTER A WITH STROKE\0"
  "LATIN SMALL LETTER A WITH TILDE\0"
  "LATIN SMALL LETTER AA\0"
  "LATIN SMALL LETTER AE\0"
  "LATIN SMALL LETTER AE WITH ACUTE\0"
  "LATIN SMALL LETTER AE WITH MACRON\0"
  "LATIN SMALL LETTER ALPHA\0"
  "LATIN SMALL LETTER ALPHA WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER ANGLICANA W\0"
  "LATIN SMALL LETTER AO\0"
  "LATIN SMALL LETTER AU\0"
  "LATIN SMALL LETTER AV\0"
  "LATIN SMALL LETTER AV WITH HORIZONTAL BAR\0"
  "LATIN SMALL LETTER AY\0"
  "LATIN SMALL LETTER B\0"
  "LATIN SMALL LETTER B WITH DOT ABOVE\0"
  "LATIN SMALL LETTER B WITH DOT BELOW\0"
  "LATIN SMALL LETTER B WITH FLOURISH\0"
  "LATIN SMALL LETTER B WITH HOOK\0"
  "LATIN SMALL LETTER B WITH LINE BELOW\0"
  "LATIN SMALL LETTER B WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER B WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER B WITH STROKE\0"
  "LATIN SMALL LETTER B WITH TOPBAR\0"
  "LATIN SMALL LETTER BARRED ALPHA\0"
  "LATIN SMALL LETTER BARRED E\0"
  "LATIN SMALL LETTER BARRED O\0"
  "LATIN SMALL LETTER BASELINE ESH\0"
  "LATIN SMALL LETTER BETA\0"
  "LATIN SMALL LETTER BLACKLETTER E\0"
  "LATIN SMALL LETTER BLACKLETTER O\0"
  "LATIN SMALL LETTER BLACKLETTER O WITH STROKE\0"
  "LATIN SMALL LETTER BOTTOM HALF O\0"
  "LATIN SMALL LETTER BROKEN L\0"
  "LATIN SMALL LETTER C\0"
  "LATIN SMALL LETTER C WITH ACUTE\0"
  "LATIN SMALL LETTER C WITH BAR\0"
  "LATIN SMALL LETTER C WITH CARON\0"
  "LATIN SMALL LETTER C WITH CEDILLA\0"
  "LATIN SMALL LETTER C WITH CEDILLA AND ACUTE\0"
  "LATIN SMALL LETTER C WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER C WITH CURL\0"
  "LATIN SMALL LETTER C WITH DOT ABOVE\0"
  "LATIN SMALL LETTER C WITH HOOK\0"
  "LATIN SMALL LETTER C WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER C WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER C WITH STROKE\0"
  "LATIN SMALL LETTER CHI\0"
  "LATIN SMALL LETTER CHI WITH LOW LEFT SERIF\0"
  "LATIN SMALL LETTER CHI WITH LOW RIGHT RING\0"
  "LATIN SMALL LETTER CLOSED INSULAR G\0"
  "LATIN SMALL LETTER CLOSED OMEGA\0"
  "LATIN SMALL LETTER CLOSED OPEN E\0"
  "LATIN SMALL LETTER CLOSED REVERSED OPEN E\0"
  "LATIN SMALL LETTER CON\0"
  "LATIN SMALL LETTER CUATRILLO\0"
  "LATIN SMALL LETTER CUATRILLO WITH COMMA\0"
  "LATIN SMALL LETTER D\0"
  "LATIN SMALL LETTER D WITH CARON\0"
  "LATIN SMALL LETTER D WITH CEDILLA\0"
  "LATIN SMALL LETTER D WITH CIRCUMFLEX BELOW\0"
  "LATIN SMALL LETTER D WITH CURL\0"
  "LATIN SMALL LETTER D WITH DOT ABOVE\0"
  "LATIN SMALL LETTER D WITH DOT BELOW\0"
  "LATIN SMALL LETTER D WITH HOOK\0"
  "LATIN SMALL LETTER D WITH HOOK AND TAIL\0"
  "LATIN SMALL LETTER D WITH LINE BELOW\0"
  "LATIN SMALL LETTER D WITH MID-HEIGHT LEFT HOOK\0"
  "LATIN SMALL LETTER D WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER D WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER D WITH SHORT STROKE OVERLAY\0"
  "LATIN SMALL LETTER D WITH STROKE\0"
  "LATIN SMALL LETTER D WITH TAIL\0"
  "LATIN SMALL LETTER D WITH TOPBAR\0"
  "LATIN SMALL LETTER DB DIGRAPH\0"
  "LATIN SMALL LETTER DELTA\0"
  "LATIN SMALL LETTER DEZH DIGRAPH\0"
  "LATIN SMALL LETTER DEZH DIGRAPH WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER DEZH DIGRAPH WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER DOTLESS I\0"
  "LATIN SMALL LETTER DOTLESS J\0"
  "LATIN SMALL LETTER DOTLESS J WITH STROKE\0"
  "LATIN SMALL LETTER DOTLESS J WITH STROKE AND HOOK\0"
  "LATIN SMALL LETTER DOUBLE R\0"
  "LATIN SMALL LETTER DOUBLE R WITH CROSSED-TAIL\0"
  "LATIN SMALL LETTER DOUBLE THORN\0"
  "LATIN SMALL LETTER DOUBLE WYNN\0"
  "LATIN SMALL LETTER DUM\0"
  "LATIN SMALL LETTER DZ\0"
  "LATIN SMALL LETTER DZ DIGRAPH\0"
  "LATIN SMALL LETTER DZ DIGRAPH WITH CURL\0"
  "LATIN SMALL LETTER DZ DIGRAPH WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER DZ WITH CARON\0"
  "LATIN SMALL LETTER E\0"
  "LATIN SMALL LETTER E WITH ACUTE\0"
  "LATIN SMALL LETTER E WITH BREVE\0"
  "LATIN SMALL LETTER E WITH CARON\0"
  "LATIN SMALL LETTER E WITH CEDILLA\0"
  "LATIN SMALL LETTER E WITH CEDILLA AND BREVE\0"
  "LATIN SMALL LETTER E WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER E WITH CIRCUMFLEX AND ACUTE\0"
  "LATIN SMALL LETTER E WITH CIRCUMFLEX AND DOT BELOW\0"
  "LATIN SMALL LETTER E WITH CIRCUMFLEX AND GRAVE\0"
  "LATIN SMALL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE\0"
  "LATIN SMALL LETTER E WITH CIRCUMFLEX AND TILDE\0"
  "LATIN SMALL LETTER E WITH CIRCUMFLEX BELOW\0"
  "LATIN SMALL LETTER E WITH DIAERESIS\0"
  "LATIN SMALL LETTER E WITH DOT ABOVE\0"
  "LATIN SMALL LETTER E WITH DOT BELOW\0"
  "LATIN SMALL LETTER E WITH DOUBLE GRAVE\0"
  "LATIN SMALL LETTER E WITH FLOURISH\0"
  "LATIN SMALL LETTER E WITH GRAVE\0"
  "LATIN SMALL LETTER E WITH HOOK ABOVE\0"
  "LATIN SMALL LETTER E WITH INVERTED BREVE\0"
  "LATIN SMALL LETTER E WITH MACRON\0"
  "LATIN SMALL LETTER E WITH MACRON AND ACUTE\0"
  "LATIN SMALL LETTER E WITH MACRON AND GRAVE\0"
  "LATIN SMALL LETTER E WITH NOTCH\0"
  "LATIN SMALL LETTER E WITH OGONEK\0"
  "LATIN SMALL LETTER E WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER E WITH STROKE\0"
  "LATIN SMALL LETTER E WITH TILDE\0"
  "LATIN SMALL LETTER E WITH TILDE BELOW\0"
  "LATIN SMALL LETTER EGYPTOLOGICAL AIN\0"
  "LATIN SMALL LETTER EGYPTOLOGICAL ALEF\0"
  "LATIN SMALL LETTER ENG\0"
  "LATIN SMALL LETTER ENG WITH CROSSED-TAIL\0"
  "LATIN SMALL LETTER ENG WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER ESH\0"
  "LATIN SMALL LETTER ESH WITH CURL\0"
  "LATIN SMALL LETTER ESH WITH DOUBLE BAR\0"
  "LATIN SMALL LETTER ESH WITH DOUBLE BAR AND CURL\0"
  "LATIN SMALL LETTER ESH WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER ESH WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER ET\0"
  "LATIN SMALL LETTER ETH\0"
  "LATIN SMALL LETTER EZH\0"
  "LATIN SMALL LETTER EZH REVERSED\0"
  "LATIN SMALL LETTER EZH WITH CARON\0"
  "LATIN SMALL LETTER EZH WITH CURL\0"
  "LATIN SMALL LETTER EZH WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER EZH WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER EZH WITH TAIL\0"
  "LATIN SMALL LETTER F\0"
  "LATIN SMALL LETTER F WITH DOT ABOVE\0"
  "LATIN SMALL LETTER F WITH HOOK\0"
  "LATIN SMALL LETTER F WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER F WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER F WITH STROKE\0"
  "LATIN SMALL LETTER FENG DIGRAPH\0"
  "LATIN SMALL LETTER FENG DIGRAPH WITH TRILL\0"
  "LATIN SMALL LETTER G\0"
  "LATIN SMALL LETTER G WITH ACUTE\0"
  "LATIN SMALL LETTER G WITH BREVE\0"
  "LATIN SMALL LETTER G WITH CARON\0"
  "LATIN SMALL LETTER G WITH CEDILLA\0"
  "LATIN SMALL LETTER G WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER G WITH DOT ABOVE\0"
  "LATIN SMALL LETTER G WITH HOOK\0"
  "LATIN SMALL LETTER G WITH MACRON\0"
  "LATIN SMALL LETTER G WITH OBLIQUE STROKE\0"
  "LATIN SMALL LETTER G WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER G WITH STROKE\0"
  "LATIN SMALL LETTER GAMMA\0"
  "LATIN SMALL LETTER GLOTTAL A\0"
  "LATIN SMALL LETTER GLOTTAL I\0"
  "LATIN SMALL LETTER GLOTTAL STOP\0"
  "LATIN SMALL LETTER GLOTTAL U\0"
  "LATIN SMALL LETTER H\0"
  "LATIN SMALL LETTER H WITH BREVE BELOW\0"
  "LATIN SMALL LETTER H WITH CARON\0"
  "LATIN SMALL LETTER H WITH CEDILLA\0"
  "LATIN SMALL LETTER H WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER H WITH DESCENDER\0"
  "LATIN SMALL LETTER H WITH DIAERESIS\0"
  "LATIN SMALL LETTER H WITH DOT ABOVE\0"
  "LATIN SMALL LETTER H WITH DOT BELOW\0"
  "LATIN SMALL LETTER H WITH HOOK\0"
  "LATIN SMALL LETTER H WITH LINE BELOW\0"
  "LATIN SMALL LETTER H WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER H WITH STROKE\0"
  "LATIN SMALL LETTER HALF H\0"
  "LATIN SMALL LETTER HENG\0"
  "LATIN SMALL LETTER HENG WITH HOOK\0"
  "LATIN SMALL LETTER HV\0"
  "LATIN SMALL LETTER I\0"
  "LATIN SMALL LETTER I WITH ACUTE\0"
  "LATIN SMALL LETTER I WITH BREVE\0"
  "LATIN SMALL LETTER I WITH CARON\0"
  "LATIN SMALL LETTER I WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER I WITH DIAERESIS\0"
  "LATIN SMALL LETTER I WITH DIAERESIS AND ACUTE\0"
  "LATIN SMALL LETTER I WITH DOT BELOW\0"
  "LATIN SMALL LETTER I WITH DOUBLE GRAVE\0"
  "LATIN SMALL LETTER I WITH GRAVE\0"
  "LATIN SMALL LETTER I WITH HOOK ABOVE\0"
  "LATIN SMALL LETTER I WITH INVERTED BREVE\0"
  "LATIN SMALL LETTER I WITH MACRON\0"
  "LATIN SMALL LETTER I WITH OGONEK\0"
  "LATIN SMALL LETTER I WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER I WITH STROKE\0"
  "LATIN SMALL LETTER I WITH STROKE AND RETROFLEX HOOK\0"
  "LATIN SMALL LETTER I WITH TILDE\0"
  "LATIN SMALL LETTER I WITH TILDE BELOW\0"
  "LATIN SMALL LETTER INSULAR D\0"
  "LATIN SMALL LETTER INSULAR F\0"
  "LATIN SMALL LETTER INSULAR G\0"
  "LATIN SMALL LETTER INSULAR R\0"
  "LATIN SMALL LETTER INSULAR S\0"
  "LATIN SMALL LETTER INSULAR T\0"
  "LATIN SMALL LETTER INVERTED ALPHA\0"
  "LATIN SMALL LETTER INVERTED OE\0"
  "LATIN SMALL LETTER IOTA\0"
  "LATIN SMALL LETTER IOTA WITH STROKE\0"
  "LATIN SMALL LETTER IOTIFIED E\0"
  "LATIN SMALL LETTER IS\0"
  "LATIN SMALL LETTER J\0"
  "LATIN SMALL LETTER J WITH CARON\0"
  "LATIN SMALL LETTER J WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER J WITH CROSSED-TAIL\0"
  "LATIN SMALL LETTER J WITH STROKE\0"
  "LATIN SMALL LETTER K\0"
  "LATIN SMALL LETTER K WITH ACUTE\0"
  "LATIN SMALL LETTER K WITH CARON\0"
  "LATIN SMALL LETTER K WITH CEDILLA\0"
  "LATIN SMALL LETTER K WITH DESCENDER\0"
  "LATIN SMALL LETTER K WITH DIAGONAL STROKE\0"
  "LATIN SMALL LETTER K WITH DOT BELOW\0"
  "LATIN SMALL LETTER K WITH HOOK\0"
  "LATIN SMALL LETTER K WITH LINE BELOW\0"
  "LATIN SMALL LETTER K WITH OBLIQUE STROKE\0"
  "LATIN SMALL LETTER K WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER K WITH STROKE\0"
  "LATIN SMALL LETTER K WITH STROKE AND DIAGONAL STROKE\0"
  "LATIN SMALL LETTER KRA\0"
  "LATIN SMALL LETTER L\0"
  "LATIN SMALL LETTER L WITH ACUTE\0"
  "LATIN SMALL LETTER L WITH BAR\0"
  "LATIN SMALL LETTER L WITH BELT\0"
  "LATIN SMALL LETTER L WITH BELT AND PALATAL HOOK\0"
  "LATIN SMALL LETTER L WITH CARON\0"
  "LATIN SMALL LETTER L WITH CEDILLA\0"
  "LATIN SMALL LETTER L WITH CIRCUMFLEX BELOW\0"
  "LATIN SMALL LETTER L WITH CURL\0"
  "LATIN SMALL LETTER L WITH DOT BELOW\0"
  "LATIN SMALL LETTER L WITH DOT BELOW AND MACRON\0"
  "LATIN SMALL LETTER L WITH DOUBLE BAR\0"
  "LATIN SMALL LETTER L WITH DOUBLE MIDDLE TILDE\0"
  "LATIN SMALL LETTER L WITH FISHHOOK\0"
  "LATIN SMALL LETTER L WITH HIGH STROKE\0"
  "LATIN SMALL LETTER L WITH INVERTED LAZY S\0"
  "LATIN SMALL LETTER L WITH LINE BELOW\0"
  "LATIN SMALL LETTER L WITH MID-HEIGHT LEFT HOOK\0"
  "LATIN SMALL LETTER L WITH MIDDLE DOT\0"
  "LATIN SMALL LETTER L WITH MIDDLE RING\0"
  "LATIN SMALL LETTER L WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER L WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER L WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER L WITH RETROFLEX HOOK AND BELT\0"
  "LATIN SMALL LETTER L WITH STROKE\0"
  "LATIN SMALL LETTER LAMBDA\0"
  "LATIN SMALL LETTER LAMBDA WITH STROKE\0"
  "LATIN SMALL LETTER LENIS F\0"
  "LATIN SMALL LETTER LEZH\0"
  "LATIN SMALL LETTER LEZH WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER LJ\0"
  "LATIN SMALL LETTER LONG S\0"
  "LATIN SMALL LETTER LONG S WITH DIAGONAL STROKE\0"
  "LATIN SMALL LETTER LONG S WITH DOT ABOVE\0"
  "LATIN SMALL LETTER LONG S WITH HIGH STROKE\0"
  "LATIN SMALL LETTER LS DIGRAPH\0"
  "LATIN SMALL LETTER LUM\0"
  "LATIN SMALL LETTER LZ DIGRAPH\0"
  "LATIN SMALL LETTER M\0"
  "LATIN SMALL LETTER M WITH ACUTE\0"
  "LATIN SMALL LETTER M WITH CROSSED-TAIL\0"
  "LATIN SMALL LETTER M WITH DOT ABOVE\0"
  "LATIN SMALL LETTER M WITH DOT BELOW\0"
  "LATIN SMALL LETTER M WITH HOOK\0"
  "LATIN SMALL LETTER M WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER M WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER MIDDLE SCOTS S\0"
  "LATIN SMALL LETTER MIDDLE-WELSH LL\0"
  "LATIN SMALL LETTER MIDDLE-WELSH V\0"
  "LATIN SMALL LETTER MUM\0"
  "LATIN SMALL LETTER N\0"
  "LATIN SMALL LETTER N PRECEDED BY APOSTROPHE\0"
  "LATIN SMALL LETTER N WITH ACUTE\0"
  "LATIN SMALL LETTER N WITH CARON\0"
  "LATIN SMALL LETTER N WITH CEDILLA\0"
  "LATIN SMALL LETTER N WITH CIRCUMFLEX BELOW\0"
  "LATIN SMALL LETTER N WITH CROSSED-TAIL\0"
  "LATIN SMALL LETTER N WITH CURL\0"
  "LATIN SMALL LETTER N WITH DESCENDER\0"
  "LATIN SMALL LETTER N WITH DOT ABOVE\0"
  "LATIN SMALL LETTER N WITH DOT BELOW\0"
  "LATIN SMALL LETTER N WITH GRAVE\0"
  "LATIN SMALL LETTER N WITH LEFT HOOK\0"
  "LATIN SMALL LETTER N WITH LINE BELOW\0"
  "LATIN SMALL LETTER N WITH LONG RIGHT LEG\0"
  "LATIN SMALL LETTER N WITH MID-HEIGHT LEFT HOOK\0"
  "LATIN SMALL LETTER N WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER N WITH OBLIQUE STROKE\0"
  "LATIN SMALL LETTER N WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER N WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER N WITH TILDE\0"
  "LATIN SMALL LETTER NJ\0"
  "LATIN SMALL LETTER NUM\0"
  "LATIN SMALL LETTER O\0"
  "LATIN SMALL LETTER O WITH ACUTE\0"
  "LATIN SMALL LETTER O WITH BREVE\0"
  "LATIN SMALL LETTER O WITH CARON\0"
  "LATIN SMALL LETTER O WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER O WITH CIRCUMFLEX AND ACUTE\0"
  "LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW\0"
  "LATIN SMALL LETTER O WITH CIRCUMFLEX AND GRAVE\0"
  "LATIN SMALL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE\0"
  "LATIN SMALL LETTER O WITH CIRCUMFLEX AND TILDE\0"
  "LATIN SMALL LETTER O WITH DIAERESIS\0"
  "LATIN SMALL LETTER O WITH DIAERESIS AND MACRON\0"
  "LATIN SMALL LETTER O WITH DOT ABOVE\0"
  "LATIN SMALL LETTER O WITH DOT ABOVE AND MACRON\0"
  "LATIN SMALL LETTER O WITH DOT BELOW\0"
  "LATIN SMALL LETTER O WITH DOUBLE ACUTE\0"
  "LATIN SMALL LETTER O WITH DOUBLE GRAVE\0"
  "LATIN SMALL LETTER O WITH GRAVE\0"
  "LATIN SMALL LETTER O WITH HOOK ABOVE\0"
  "LATIN SMALL LETTER O WITH HORN\0"
  "LATIN SMALL LETTER O WITH HORN AND ACUTE\0"
  "LATIN SMALL LETTER O WITH HORN AND DOT BELOW\0"
  "LATIN SMALL LETTER O WITH HORN AND GRAVE\0"
  "LATIN SMALL LETTER O WITH HORN AND HOOK ABOVE\0"
  "LATIN SMALL LETTER O WITH HORN AND TILDE\0"
  "LATIN SMALL LETTER O WITH INVERTED BREVE\0"
  "LATIN SMALL LETTER O WITH LONG STROKE OVERLAY\0"
  "LATIN SMALL LETTER O WITH LOOP\0"
  "LATIN SMALL LETTER O WITH LOW RING INSIDE\0"
  "LATIN SMALL LETTER O WITH MACRON\0"
  "LATIN SMALL LETTER O WITH MACRON AND ACUTE\0"
  "LATIN SMALL LETTER O WITH MACRON AND GRAVE\0"
  "LATIN SMALL LETTER O WITH OGONEK\0"
  "LATIN SMALL LETTER O WITH OGONEK AND MACRON\0"
  "LATIN SMALL LETTER O WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER O WITH STROKE\0"
  "LATIN SMALL LETTER O WITH STROKE AND ACUTE\0"
  "LATIN SMALL LETTER O WITH TILDE\0"
  "LATIN SMALL LETTER O WITH TILDE AND ACUTE\0"
  "LATIN SMALL LETTER O WITH TILDE AND DIAERESIS\0"
  "LATIN SMALL LETTER O WITH TILDE AND MACRON\0"
  "LATIN SMALL LETTER OI\0"
  "LATIN SMALL LETTER OLD POLISH O\0"
  "LATIN SMALL LETTER OMEGA\0"
  "LATIN SMALL LETTER OO\0"
  "LATIN SMALL LETTER OPEN E\0"
  "LATIN SMALL LETTER OPEN E WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER OPEN O\0"
  "LATIN SMALL LETTER OPEN O WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER OPEN O WITH STROKE\0"
  "LATIN SMALL LETTER OPEN OE\0"
  "LATIN SMALL LETTER OU\0"
  "LATIN SMALL LETTER P\0"
  "LATIN SMALL LETTER P WITH ACUTE\0"
  "LATIN SMALL LETTER P WITH DOT ABOVE\0"
  "LATIN SMALL LETTER P WITH FLOURISH\0"
  "LATIN SMALL LETTER P WITH HOOK\0"
  "LATIN SMALL LETTER P WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER P WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER P WITH SQUIRREL TAIL\0"
  "LATIN SMALL LETTER P WITH STROKE\0"
  "LATIN SMALL LETTER P WITH STROKE THROUGH DESCENDER\0"
  "LATIN SMALL LETTER PHI\0"
  "LATIN SMALL LETTER Q\0"
  "LATIN SMALL LETTER Q WITH DIAGONAL STROKE\0"
  "LATIN SMALL LETTER Q WITH HOOK\0"
  "LATIN SMALL LETTER Q WITH HOOK TAIL\0"
  "LATIN SMALL LETTER Q WITH STROKE THROUGH DESCENDER\0"
  "LATIN SMALL LETTER QP DIGRAPH\0"
  "LATIN SMALL LETTER R\0"
  "LATIN SMALL LETTER R ROTUNDA\0"
  "LATIN SMALL LETTER R WITH ACUTE\0"
  "LATIN SMALL LETTER R WITH CARON\0"
  "LATIN SMALL LETTER R WITH CEDILLA\0"
  "LATIN SMALL LETTER R WITH CROSSED-TAIL\0"
  "LATIN SMALL LETTER R WITH DOT ABOVE\0"
  "LATIN SMALL LETTER R WITH DOT BELOW\0"
  "LATIN SMALL LETTER R WITH DOT BELOW AND MACRON\0"
  "LATIN SMALL LETTER R WITH DOUBLE GRAVE\0"
  "LATIN SMALL LETTER R WITH FISHHOOK\0"
  "LATIN SMALL LETTER R WITH FISHHOOK AND MIDDLE TILDE\0"
  "LATIN SMALL LETTER R WITH FISHHOOK AND PALATAL HOOK\0"
  "LATIN SMALL LETTER R WITH INVERTED BREVE\0"
  "LATIN SMALL LETTER R WITH LINE BELOW\0"
  "LATIN SMALL LETTER R WITH LONG LEG\0"
  "LATIN SMALL LETTER R WITH MID-HEIGHT LEFT HOOK\0"
  "LATIN SMALL LETTER R WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER R WITH OBLIQUE STROKE\0"
  "LATIN SMALL LETTER R WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER R WITH STROKE\0"
  "LATIN SMALL LETTER R WITH TAIL\0"
  "LATIN SMALL LETTER R WITHOUT HANDLE\0"
  "LATIN SMALL LETTER RAMS HORN\0"
  "LATIN SMALL LETTER REVERSED C\0"
  "LATIN SMALL LETTER REVERSED C WITH DOT\0"
  "LATIN SMALL LETTER REVERSED E\0"
  "LATIN SMALL LETTER REVERSED ENG\0"
  "LATIN SMALL LETTER REVERSED HALF H\0"
  "LATIN SMALL LETTER REVERSED K\0"
  "LATIN SMALL LETTER REVERSED OPEN E\0"
  "LATIN SMALL LETTER REVERSED OPEN E WITH HOOK\0"
  "LATIN SMALL LETTER REVERSED OPEN E WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER REVERSED R WITH FISHHOOK\0"
  "LATIN SMALL LETTER REVERSED SCRIPT G\0"
  "LATIN SMALL LETTER RUM\0"
  "LATIN SMALL LETTER RUM ROTUNDA\0"
  "LATIN SMALL LETTER S\0"
  "LATIN SMALL LETTER S WITH ACUTE\0"
  "LATIN SMALL LETTER S WITH ACUTE AND DOT ABOVE\0"
  "LATIN SMALL LETTER S WITH CARON\0"
  "LATIN SMALL LETTER S WITH CARON AND DOT ABOVE\0"
  "LATIN SMALL LETTER S WITH CEDILLA\0"
  "LATIN SMALL LETTER S WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER S WITH COMMA BELOW\0"
  "LATIN SMALL LETTER S WITH CURL\0"
  "LATIN SMALL LETTER S WITH DIAGONAL STROKE\0"
  "LATIN SMALL LETTER S WITH DOT ABOVE\0"
  "LATIN SMALL LETTER S WITH DOT BELOW\0"
  "LATIN SMALL LETTER S WITH DOT BELOW AND DOT ABOVE\0"
  "LATIN SMALL LETTER S WITH HOOK\0"
  "LATIN SMALL LETTER S WITH MID-HEIGHT LEFT HOOK\0"
  "LATIN SMALL LETTER S WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER S WITH OBLIQUE STROKE\0"
  "LATIN SMALL LETTER S WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER S WITH SHORT STROKE OVERLAY\0"
  "LATIN SMALL LETTER S WITH SWASH TAIL\0"
  "LATIN SMALL LETTER SAKHA YAT\0"
  "LATIN SMALL LETTER SALTILLO\0"
  "LATIN SMALL LETTER SCHWA\0"
  "LATIN SMALL LETTER SCHWA WITH HOOK\0"
  "LATIN SMALL LETTER SCHWA WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER SCRIPT G\0"
  "LATIN SMALL LETTER SCRIPT G WITH CROSSED-TAIL\0"
  "LATIN SMALL LETTER SCRIPT R\0"
  "LATIN SMALL LETTER SCRIPT R WITH RING\0"
  "LATIN SMALL LETTER SHARP S\0"
  "LATIN SMALL LETTER SIDEWAYS DIAERESIZED U\0"
  "LATIN SMALL LETTER SIDEWAYS O\0"
  "LATIN SMALL LETTER SIDEWAYS O WITH STROKE\0"
  "LATIN SMALL LETTER SIDEWAYS OPEN O\0"
  "LATIN SMALL LETTER SIDEWAYS TURNED M\0"
  "LATIN SMALL LETTER SIDEWAYS U\0"
  "LATIN SMALL LETTER SIGMOID S\0"
  "LATIN SMALL LETTER SQUAT REVERSED ESH\0"
  "LATIN SMALL LETTER STIRRUP R\0"
  "LATIN SMALL LETTER T\0"
  "LATIN SMALL LETTER T WITH CARON\0"
  "LATIN SMALL LETTER T WITH CEDILLA\0"
  "LATIN SMALL LETTER T WITH CIRCUMFLEX BELOW\0"
  "LATIN SMALL LETTER T WITH COMMA BELOW\0"
  "LATIN SMALL LETTER T WITH CURL\0"
  "LATIN SMALL LETTER T WITH DIAERESIS\0"
  "LATIN SMALL LETTER T WITH DIAGONAL STROKE\0"
  "LATIN SMALL LETTER T WITH DOT ABOVE\0"
  "LATIN SMALL LETTER T WITH DOT BELOW\0"
  "LATIN SMALL LETTER T WITH HOOK\0"
  "LATIN SMALL LETTER T WITH HOOK AND RETROFLEX HOOK\0"
  "LATIN SMALL LETTER T WITH LINE BELOW\0"
  "LATIN SMALL LETTER T WITH MID-HEIGHT LEFT HOOK\0"
  "LATIN SMALL LETTER T WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER T WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER T WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER T WITH STROKE\0"
  "LATIN SMALL LETTER TAILLESS PHI\0"
  "LATIN SMALL LETTER TC DIGRAPH WITH CURL\0"
  "LATIN SMALL LETTER TESH DIGRAPH\0"
  "LATIN SMALL LETTER TESH DIGRAPH WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER TESH DIGRAPH WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER TH WITH STRIKETHROUGH\0"
  "LATIN SMALL LETTER THORN\0"
  "LATIN SMALL LETTER THORN WITH STROKE\0"
  "LATIN SMALL LETTER THORN WITH STROKE THROUGH DESCENDER\0"
  "LATIN SMALL LETTER TONE FIVE\0"
  "LATIN SMALL LETTER TONE SIX\0"
  "LATIN SMALL LETTER TONE TWO\0"
  "LATIN SMALL LETTER TOP HALF O\0"
  "LATIN SMALL LETTER TRESILLO\0"
  "LATIN SMALL LETTER TS DIGRAPH\0"
  "LATIN SMALL LETTER TS DIGRAPH WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER TUM\0"
  "LATIN SMALL LETTER TURNED A\0"
  "LATIN SMALL LETTER TURNED AE\0"
  "LATIN SMALL LETTER TURNED ALPHA\0"
  "LATIN SMALL LETTER TURNED DELTA\0"
  "LATIN SMALL LETTER TURNED E\0"
  "LATIN SMALL LETTER TURNED G\0"
  "LATIN SMALL LETTER TURNED H\0"
  "LATIN SMALL LETTER TURNED H WITH FISHHOOK\0"
  "LATIN SMALL LETTER TURNED H WITH FISHHOOK AND TAIL\0"
  "LATIN SMALL LETTER TURNED I\0"
  "LATIN SMALL LETTER TURNED INSULAR G\0"
  "LATIN SMALL LETTER TURNED K\0"
  "LATIN SMALL LETTER TURNED L\0"
  "LATIN SMALL LETTER TURNED M\0"
  "LATIN SMALL LETTER TURNED M WITH LONG LEG\0"
  "LATIN SMALL LETTER TURNED O OPEN-O\0"
  "LATIN SMALL LETTER TURNED O OPEN-O WITH STROKE\0"
  "LATIN SMALL LETTER TURNED OE\0"
  "LATIN SMALL LETTER TURNED OE WITH HORIZONTAL STROKE\0"
  "LATIN SMALL LETTER TURNED OE WITH STROKE\0"
  "LATIN SMALL LETTER TURNED OPEN E\0"
  "LATIN SMALL LETTER TURNED R\0"
  "LATIN SMALL LETTER TURNED R WITH HOOK\0"
  "LATIN SMALL LETTER TURNED R WITH LONG LEG\0"
  "LATIN SMALL LETTER TURNED R WITH LONG LEG AND RETROFLEX HOOK\0"
  "LATIN SMALL LETTER TURNED R WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER TURNED R WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER TURNED R WITH TAIL\0"
  "LATIN SMALL LETTER TURNED T\0"
  "LATIN SMALL LETTER TURNED T WITH CURL\0"
  "LATIN SMALL LETTER TURNED UI\0"
  "LATIN SMALL LETTER TURNED V\0"
  "LATIN SMALL LETTER TURNED W\0"
  "LATIN SMALL LETTER TURNED Y\0"
  "LATIN SMALL LETTER TURNED Y WITH BELT\0"
  "LATIN SMALL LETTER TZ\0"
  "LATIN SMALL LETTER U\0"
  "LATIN SMALL LETTER U BAR\0"
  "LATIN SMALL LETTER U BAR WITH SHORT RIGHT LEG\0"
  "LATIN SMALL LETTER U WITH ACUTE\0"
  "LATIN SMALL LETTER U WITH BREVE\0"
  "LATIN SMALL LETTER U WITH CARON\0"
  "LATIN SMALL LETTER U WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER U WITH CIRCUMFLEX BELOW\0"
  "LATIN SMALL LETTER U WITH DIAERESIS\0"
  "LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE\0"
  "LATIN SMALL LETTER U WITH DIAERESIS AND CARON\0"
  "LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE\0"
  "LATIN SMALL LETTER U WITH DIAERESIS AND MACRON\0"
  "LATIN SMALL LETTER U WITH DIAERESIS BELOW\0"
  "LATIN SMALL LETTER U WITH DOT BELOW\0"
  "LATIN SMALL LETTER U WITH DOUBLE ACUTE\0"
  "LATIN SMALL LETTER U WITH DOUBLE GRAVE\0"
  "LATIN SMALL LETTER U WITH GRAVE\0"
  "LATIN SMALL LETTER U WITH HOOK ABOVE\0"
  "LATIN SMALL LETTER U WITH HORN\0"
  "LATIN SMALL LETTER U WITH HORN AND ACUTE\0"
  "LATIN SMALL LETTER U WITH HORN AND DOT BELOW\0"
  "LATIN SMALL LETTER U WITH HORN AND GRAVE\0"
  "LATIN SMALL LETTER U WITH HORN AND HOOK ABOVE\0"
  "LATIN SMALL LETTER U WITH HORN AND TILDE\0"
  "LATIN SMALL LETTER U WITH INVERTED BREVE\0"
  "LATIN SMALL LETTER U WITH LEFT HOOK\0"
  "LATIN SMALL LETTER U WITH MACRON\0"
  "LATIN SMALL LETTER U WITH MACRON AND DIAERESIS\0"
  "LATIN SMALL LETTER U WITH OGONEK\0"
  "LATIN SMALL LETTER U WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER U WITH RING ABOVE\0"
  "LATIN SMALL LETTER U WITH SHORT RIGHT LEG\0"
  "LATIN SMALL LETTER U WITH STROKE\0"
  "LATIN SMALL LETTER U WITH TILDE\0"
  "LATIN SMALL LETTER U WITH TILDE AND ACUTE\0"
  "LATIN SMALL LETTER U WITH TILDE BELOW\0"
  "LATIN SMALL LETTER UE\0"
  "LATIN SMALL LETTER UI\0"
  "LATIN SMALL LETTER UM\0"
  "LATIN SMALL LETTER UO\0"
  "LATIN SMALL LETTER UPSILON\0"
  "LATIN SMALL LETTER UPSILON WITH STROKE\0"
  "LATIN SMALL LETTER V\0"
  "LATIN SMALL LETTER V WITH CURL\0"
  "LATIN SMALL LETTER V WITH DIAGONAL STROKE\0"
  "LATIN SMALL LETTER V WITH DOT BELOW\0"
  "LATIN SMALL LETTER V WITH HOOK\0"
  "LATIN SMALL LETTER V WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER V WITH RIGHT HOOK\0"
  "LATIN SMALL LETTER V WITH TILDE\0"
  "LATIN SMALL LETTER VEND\0"
  "LATIN SMALL LETTER VISIGOTHIC Z\0"
  "LATIN SMALL LETTER VOLAPUK AE\0"
  "LATIN SMALL LETTER VOLAPUK OE\0"
  "LATIN SMALL LETTER VOLAPUK UE\0"
  "LATIN SMALL LETTER VY\0"
  "LATIN SMALL LETTER W\0"
  "LATIN SMALL LETTER W WITH ACUTE\0"
  "LATIN SMALL LETTER W WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER W WITH DIAERESIS\0"
  "LATIN SMALL LETTER W WITH DOT ABOVE\0"
  "LATIN SMALL LETTER W WITH DOT BELOW\0"
  "LATIN SMALL LETTER W WITH GRAVE\0"
  "LATIN SMALL LETTER W WITH HOOK\0"
  "LATIN SMALL LETTER W WITH RING ABOVE\0"
  "LATIN SMALL LETTER X\0"
  "LATIN SMALL LETTER X WITH DIAERESIS\0"
  "LATIN SMALL LETTER X WITH DOT ABOVE\0"
  "LATIN SMALL LETTER X WITH LONG LEFT LEG\0"
  "LATIN SMALL LETTER X WITH LONG LEFT LEG AND LOW RIGHT RING\0"
  "LATIN SMALL LETTER X WITH LONG LEFT LEG WITH SERIF\0"
  "LATIN SMALL LETTER X WITH LOW RIGHT RING\0"
  "LATIN SMALL LETTER X WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER Y\0"
  "LATIN SMALL LETTER Y WITH ACUTE\0"
  "LATIN SMALL LETTER Y WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER Y WITH DIAERESIS\0"
  "LATIN SMALL LETTER Y WITH DOT ABOVE\0"
  "LATIN SMALL LETTER Y WITH DOT BELOW\0"
  "LATIN SMALL LETTER Y WITH GRAVE\0"
  "LATIN SMALL LETTER Y WITH HOOK\0"
  "LATIN SMALL LETTER Y WITH HOOK ABOVE\0"
  "LATIN SMALL LETTER Y WITH LOOP\0"
  "LATIN SMALL LETTER Y WITH MACRON\0"
  "LATIN SMALL LETTER Y WITH RING ABOVE\0"
  "LATIN SMALL LETTER Y WITH SHORT RIGHT LEG\0"
  "LATIN SMALL LETTER Y WITH STROKE\0"
  "LATIN SMALL LETTER Y WITH TILDE\0"
  "LATIN SMALL LETTER YOGH\0"
  "LATIN SMALL LETTER Z\0"
  "LATIN SMALL LETTER Z WITH ACUTE\0"
  "LATIN SMALL LETTER Z WITH CARON\0"
  "LATIN SMALL LETTER Z WITH CIRCUMFLEX\0"
  "LATIN SMALL LETTER Z WITH CURL\0"
  "LATIN SMALL LETTER Z WITH DESCENDER\0"
  "LATIN SMALL LETTER Z WITH DOT ABOVE\0"
  "LATIN SMALL LETTER Z WITH DOT BELOW\0"
  "LATIN SMALL LETTER Z WITH HOOK\0"
  "LATIN SMALL LETTER Z WITH LINE BELOW\0"
  "LATIN SMALL LETTER Z WITH MIDDLE TILDE\0"
  "LATIN SMALL LETTER Z WITH PALATAL HOOK\0"
  "LATIN SMALL LETTER Z WITH RETROFLEX HOOK\0"
  "LATIN SMALL LETTER Z WITH STROKE\0"
  "LATIN SMALL LETTER Z WITH SWASH TAIL\0"
  "LATIN SMALL LIGATURE FF\0"
  "LATIN SMALL LIGATURE FFI\0"
  "LATIN SMALL LIGATURE FFL\0"
  "LATIN SMALL LIGATURE FI\0"
  "LATIN SMALL LIGATURE FL\0"
  "LATIN SMALL LIGATURE IJ\0"
  "LATIN SMALL LIGATURE LONG S T\0"
  "LATIN SMALL LIGATURE OE\0"
  "LATIN SMALL LIGATURE ST\0"
  "LATIN SUBSCRIPT SMALL LETTER A\0"
  "LATIN SUBSCRIPT SMALL LETTER E\0"
  "LATIN SUBSCRIPT SMALL LETTER H\0"
  "LATIN SUBSCRIPT SMALL LETTER I\0"
  "LATIN SUBSCRIPT SMALL LETTER J\0"
  "LATIN SUBSCRIPT SMALL LETTER K\0"
  "LATIN SUBSCRIPT SMALL LETTER L\0"
  "LATIN SUBSCRIPT SMALL LETTER M\0"
  "LATIN SUBSCRIPT SMALL LETTER N\0"
  "LATIN SUBSCRIPT SMALL LETTER O\0"
  "LATIN SUBSCRIPT SMALL LETTER P\0"
  "LATIN SUBSCRIPT SMALL LETTER R\0"
  "LATIN SUBSCRIPT SMALL LETTER S\0"
  "LATIN SUBSCRIPT SMALL LETTER SCHWA\0"
  "LATIN SUBSCRIPT SMALL LETTER T\0"
  "LATIN SUBSCRIPT SMALL LETTER U\0"
  "LATIN SUBSCRIPT SMALL LETTER V\0"
  "LATIN SUBSCRIPT SMALL LETTER X\0"
  "LEAF FLUTTERING IN WIND\0"
  "LEAFLESS TREE\0"
  "LEAFY GREEN\0"
  "LEDGER\0"
  "LEFT AND LOWER AND RIGHT TRIANGULAR THREE QUARTERS BLOCK\0"
  "LEFT AND LOWER ONE EIGHTH BLOCK\0"
  "LEFT AND RIGHT DOUBLE TURNSTILE\0"
  "LEFT AND RIGHT TACK\0"
  "LEFT AND RIGHT TRIANGULAR HALF BLOCK\0"
  "LEFT AND UPPER AND RIGHT TRIANGULAR THREE QUARTERS BLOCK\0"
  "LEFT AND UPPER ONE EIGHTH BLOCK\0"
  "LEFT AND UPPER ONE EIGHTH BLOCK CONTAINING BLACK SMALL SQUARE\0"
  "LEFT ANGER BUBBLE\0"
  "LEFT ANGLE BRACKET\0"
  "LEFT ANGLE BRACKET WITH DOT\0"
  "LEFT ARC LESS-THAN BRACKET\0"
  "LEFT ARROW WITH CIRCLED PLUS\0"
  "LEFT ARROW WITH SMALL CIRCLE\0"
  "LEFT BARB DOWN RIGHT BARB DOWN HARPOON\0"
  "LEFT BARB DOWN RIGHT BARB UP HARPOON\0"
  "LEFT BARB UP RIGHT BARB DOWN HARPOON\0"
  "LEFT BARB UP RIGHT BARB UP HARPOON\0"
  "LEFT BLACK LENTICULAR BRACKET\0"
  "LEFT BLACK TORTOISE SHELL BRACKET\0"
  "LEFT CEILING\0"
  "LEFT CLOSED ENTRY\0"
  "LEFT CORNER BRACKET\0"
  "LEFT CURLY BRACKET\0"
  "LEFT CURLY BRACKET LOWER HOOK\0"
  "LEFT CURLY BRACKET MIDDLE PIECE\0"
  "LEFT CURLY BRACKET UPPER HOOK\0"
  "LEFT DOTTED SUBSTITUTION BRACKET\0"
  "LEFT DOUBLE ANGLE BRACKET\0"
  "LEFT DOUBLE PARENTHESIS\0"
  "LEFT DOUBLE QUOTATION MARK\0"
  "LEFT DOUBLE WIGGLY FENCE\0"
  "LEFT FISH TAIL\0"
  "LEFT FIVE EIGHTHS BLOCK\0"
  "LEFT FLOOR\0"
  "LEFT HALF BLACK CIRCLE\0"
  "LEFT HALF BLACK STAR\0"
  "LEFT HALF BLOCK\0"
  "LEFT HALF CIRCLE\0"
  "LEFT HALF CIRCLE WITH DOT\0"
  "LEFT HALF CIRCLE WITH FOUR DOTS\0"
  "LEFT HALF CIRCLE WITH THREE DOTS\0"
  "LEFT HALF CIRCLE WITH TWO DOTS\0"
  "LEFT HALF FLYING SAUCER\0"
  "LEFT HALF FOLDER\0"
  "LEFT HALF HORIZONTAL LINE WITH THREE TICK MARKS\0"
  "LEFT HALF INVERSE MEDIUM SHADE AND RIGHT HALF BLOCK\0"
  "LEFT HALF LOWER ONE QUARTER BLOCK\0"
  "LEFT HALF MEDIUM SHADE\0"
  "LEFT HALF RUNNING MAN\0"
  "LEFT HALF TRIPLE DASH HORIZONTAL\0"
  "LEFT HALF UPPER ONE QUARTER BLOCK\0"
  "LEFT HALF WHITE ELLIPSE\0"
  "LEFT HAND TELEPHONE RECEIVER\0"
  "LEFT JUSTIFIED RIGHT HALF BLACK CIRCLE\0"
  "LEFT JUSTIFIED RIGHT HALF WHITE CIRCLE\0"
  "LEFT LOW PARAPHRASE BRACKET\0"
  "LEFT LUGGAGE\0"
  "LEFT MULTIMAP\0"
  "LEFT NORMAL FACTOR SEMIDIRECT PRODUCT\0"
  "LEFT ONE EIGHTH BLOCK\0"
  "LEFT ONE QUARTER BLOCK\0"
  "LEFT ONE THIRD BLOCK\0"
  "LEFT OUTER JOIN\0"
  "LEFT PARENTHESIS\0"
  "LEFT PARENTHESIS EXTENSION\0"
  "LEFT PARENTHESIS LOWER HOOK\0"
  "LEFT PARENTHESIS UPPER HOOK\0"
  "LEFT RAISED OMISSION BRACKET\0"
  "LEFT RIGHT ARROW\0"
  "LEFT RIGHT ARROW THROUGH SMALL CIRCLE\0"
  "LEFT RIGHT ARROW WITH DOUBLE VERTICAL STROKE\0"
  "LEFT RIGHT ARROW WITH STROKE\0"
  "LEFT RIGHT ARROW WITH VERTICAL STROKE\0"
  "LEFT RIGHT BLACK ARROW\0"
  "LEFT RIGHT DOUBLE ARROW\0"
  "LEFT RIGHT DOUBLE ARROW WITH STROKE\0"
  "LEFT RIGHT DOUBLE ARROW WITH VERTICAL STROKE\0"
  "LEFT RIGHT OPEN-HEADED ARROW\0"
  "LEFT RIGHT SANS-SERIF ARROW\0"
  "LEFT RIGHT TRIANGLE-HEADED ARROW\0"
  "LEFT RIGHT WAVE ARROW\0"
  "LEFT RIGHT WHITE ARROW\0"
  "LEFT S-SHAPED BAG DELIMITER\0"
  "LEFT SEMIDIRECT PRODUCT\0"
  "LEFT SEVEN EIGHTHS BLOCK\0"
  "LEFT SIDEWAYS U BRACKET\0"
  "LEFT SINGLE QUOTATION MARK\0"
  "LEFT SPEECH BUBBLE\0"
  "LEFT SQUARE BRACKET\0"
  "LEFT SQUARE BRACKET EXTENSION\0"
  "LEFT SQUARE BRACKET LOWER CORNER\0"
  "LEFT SQUARE BRACKET UPPER CORNER\0"
  "LEFT SQUARE BRACKET WITH DOUBLE STROKE\0"
  "LEFT SQUARE BRACKET WITH QUILL\0"
  "LEFT SQUARE BRACKET WITH STROKE\0"
  "LEFT SQUARE BRACKET WITH TICK IN BOTTOM CORNER\0"
  "LEFT SQUARE BRACKET WITH TICK IN TOP CORNER\0"
  "LEFT SQUARE BRACKET WITH UNDERBAR\0"
  "LEFT SUBSTITUTION BRACKET\0"
  "LEFT TACK\0"
  "LEFT THIRD INDUCTOR\0"
  "LEFT THIRD WHITE RIGHT POINTING INDEX\0"
  "LEFT THOUGHT BUBBLE\0"
  "LEFT THREE EIGHTHS BLOCK\0"
  "LEFT THREE QUARTERS BLOCK\0"
  "LEFT THREE QUARTERS LOWER ONE QUARTER BLOCK\0"
  "LEFT THREE QUARTERS UPPER ONE QUARTER BLOCK\0"
  "LEFT TORTOISE SHELL BRACKET\0"
  "LEFT TRANSPOSITION BRACKET\0"
  "LEFT TRIANGLE BESIDE VERTICAL BAR\0"
  "LEFT TRIANGULAR ONE QUARTER BLOCK\0"
  "LEFT TWO THIRDS BLOCK\0"
  "LEFT VERTICAL BAR WITH QUILL\0"
  "LEFT VERTICAL BOX LINE\0"
  "LEFT WHITE CORNER BRACKET\0"
  "LEFT WHITE CURLY BRACKET\0"
  "LEFT WHITE LENTICULAR BRACKET\0"
  "LEFT WHITE PARENTHESIS\0"
  "LEFT WHITE SQUARE BRACKET\0"
  "LEFT WHITE TORTOISE SHELL BRACKET\0"
  "LEFT WIGGLY FENCE\0"
  "LEFT WRITING HAND\0"
  "LEFT-FACING ARMENIAN ETERNITY SIGN\0"
  "LEFT-FACING BASSINET\0"
  "LEFT-FACING FISH\0"
  "LEFT-FACING FISH WITH OPEN MOUTH\0"
  "LEFT-FACING FIST\0"
  "LEFT-FACING SNAKE HEAD WITH CLOSED MOUTH\0"
  "LEFT-FACING SNAKE HEAD WITH OPEN MOUTH\0"
  "LEFT-FACING SVASTI SIGN\0"
  "LEFT-FACING SVASTI SIGN WITH DOTS\0"
  "LEFT-HANDED INTERLACED PENTAGRAM\0"
  "LEFT-POINTING AIRPLANE\0"
  "LEFT-POINTING ANGLE BRACKET\0"
  "LEFT-POINTING ATOMIC BOMB\0"
  "LEFT-POINTING CURVED ANGLE BRACKET\0"
  "LEFT-POINTING DIODE\0"
  "LEFT-POINTING DOUBLE ANGLE QUOTATION MARK\0"
  "LEFT-POINTING ENERGY WAVE\0"
  "LEFT-POINTING MAGNIFYING GLASS\0"
  "LEFT-POINTING RACING CAR\0"
  "LEFT-POINTING RIFLE\0"
  "LEFT-POINTING ROCKET BOOSTER\0"
  "LEFT-POINTING ROCKET SHIP\0"
  "LEFT-POINTING ROLLER COASTER CAR\0"
  "LEFT-POINTING STICK FIGURE\0"
  "LEFT-POINTING TANK\0"
  "LEFT-SHADED WHITE RIGHTWARDS ARROW\0"
  "LEFT-SIDE ARC ANTICLOCKWISE ARROW\0"
  "LEFT-TO-RIGHT EMBEDDING\0"
  "LEFT-TO-RIGHT ISOLATE\0"
  "LEFT-TO-RIGHT MARK\0"
  "LEFT-TO-RIGHT OVERRIDE\0"
  "LEFTWARDS ARROW\0"
  "LEFTWARDS ARROW ABOVE ALMOST EQUAL TO\0"
  "LEFTWARDS ARROW ABOVE REVERSE ALMOST EQUAL TO\0"
  "LEFTWARDS ARROW ABOVE REVERSE TILDE OPERATOR\0"
  "LEFTWARDS ARROW ABOVE SHORT RIGHTWARDS ARROW\0"
  "LEFTWARDS ARROW ABOVE TILDE OPERATOR\0"
  "LEFTWARDS ARROW AND UPPER AND LOWER ONE EIGHTH BLOCK\0"
  "LEFTWARDS ARROW FROM BAR\0"
  "LEFTWARDS ARROW FROM BAR TO BLACK DIAMOND\0"
  "LEFTWARDS ARROW FROM DOWNWARDS ARROW\0"
  "LEFTWARDS ARROW OVER RIGHTWARDS ARROW\0"
  "LEFTWARDS ARROW THROUGH LESS-THAN\0"
  "LEFTWARDS ARROW THROUGH SUBSET\0"
  "LEFTWARDS ARROW THROUGH X\0"
  "LEFTWARDS ARROW TO BAR\0"
  "LEFTWARDS ARROW TO BAR OVER RIGHTWARDS ARROW TO BAR\0"
  "LEFTWARDS ARROW TO BLACK DIAMOND\0"
  "LEFTWARDS ARROW WITH DOTTED STEM\0"
  "LEFTWARDS ARROW WITH DOUBLE VERTICAL STROKE\0"
  "LEFTWARDS ARROW WITH EQUILATERAL ARROWHEAD\0"
  "LEFTWARDS ARROW WITH HOOK\0"
  "LEFTWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD\0"
  "LEFTWARDS ARROW WITH LOOP\0"
  "LEFTWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD\0"
  "LEFTWARDS ARROW WITH NOTCHED TAIL\0"
  "LEFTWARDS ARROW WITH PLUS BELOW\0"
  "LEFTWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD\0"
  "LEFTWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD\0"
  "LEFTWARDS ARROW WITH STROKE\0"
  "LEFTWARDS ARROW WITH TAIL\0"
  "LEFTWARDS ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE\0"
  "LEFTWARDS ARROW WITH TAIL WITH VERTICAL STROKE\0"
  "LEFTWARDS ARROW WITH TIP DOWNWARDS\0"
  "LEFTWARDS ARROW WITH TIP UPWARDS\0"
  "LEFTWARDS ARROW WITH VERTICAL STROKE\0"
  "LEFTWARDS ARROW-TAIL\0"
  "LEFTWARDS BACK-TILTED SHADOWED WHITE ARROW\0"
  "LEFTWARDS BLACK ARROW\0"
  "LEFTWARDS BLACK CIRCLED WHITE ARROW\0"
  "LEFTWARDS BOTTOM-SHADED WHITE ARROW\0"
  "LEFTWARDS COMPRESSED ARROW\0"
  "LEFTWARDS DASHED ARROW\0"
  "LEFTWARDS DOUBLE ARROW\0"
  "LEFTWARDS DOUBLE ARROW FROM BAR\0"
  "LEFTWARDS DOUBLE ARROW WITH STROKE\0"
  "LEFTWARDS DOUBLE ARROW WITH VERTICAL STROKE\0"
  "LEFTWARDS DOUBLE ARROW-TAIL\0"
  "LEFTWARDS DOUBLE DASH ARROW\0"
  "LEFTWARDS FINGER-POST ARROW\0"
  "LEFTWARDS FRONT-TILTED SHADOWED WHITE ARROW\0"
  "LEFTWARDS HAND\0"
  "LEFTWARDS HARPOON OVER RIGHTWARDS HARPOON\0"
  "LEFTWARDS HARPOON WITH BARB DOWN ABOVE RIGHTWARDS HARPOON WITH BARB DOWN\0"
  "LEFTWARDS HARPOON WITH BARB DOWN BELOW LONG DASH\0"
  "LEFTWARDS HARPOON WITH BARB DOWN FROM BAR\0"
  "LEFTWARDS HARPOON WITH BARB DOWN TO BAR\0"
  "LEFTWARDS HARPOON WITH BARB DOWNWARDS\0"
  "LEFTWARDS HARPOON WITH BARB UP ABOVE LEFTWARDS HARPOON WITH BARB DOWN\0"
  "LEFTWARDS HARPOON WITH BARB UP ABOVE LONG DASH\0"
  "LEFTWARDS HARPOON WITH BARB UP ABOVE RIGHTWARDS HARPOON WITH BARB UP\0"
  "LEFTWARDS HARPOON WITH BARB UP FROM BAR\0"
  "LEFTWARDS HARPOON WITH BARB UP TO BAR\0"
  "LEFTWARDS HARPOON WITH BARB UPWARDS\0"
  "LEFTWARDS HEAVY ARROW\0"
  "LEFTWARDS HEAVY COMPRESSED ARROW\0"
  "LEFTWARDS LEFT-SHADED WHITE ARROW\0"
  "LEFTWARDS OPEN-HEADED ARROW\0"
  "LEFTWARDS PAIRED ARROWS\0"
  "LEFTWARDS PUSHING HAND\0"
  "LEFTWARDS QUADRUPLE ARROW\0"
  "LEFTWARDS RIGHT-SHADED WHITE ARROW\0"
  "LEFTWARDS ROCKET\0"
  "LEFTWARDS SANS-SERIF ARROW\0"
  "LEFTWARDS SQUARED ARROW\0"
  "LEFTWARDS SQUIGGLE ARROW\0"
  "LEFTWARDS TOP SHADED WHITE ARROW\0"
  "LEFTWARDS TRIANGLE ARROWHEAD\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW OVER RIGHTWARDS TRIANGLE-HEADED ARROW\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW TO BAR\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP DOWNWARDS\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP UPWARDS\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT\0"
  "LEFTWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT\0"
  "LEFTWARDS TRIANGLE-HEADED DASHED ARROW\0"
  "LEFTWARDS TRIANGLE-HEADED PAIRED ARROWS\0"
  "LEFTWARDS TRIPLE ARROW\0"
  "LEFTWARDS TRIPLE DASH ARROW\0"
  "LEFTWARDS TWO HEADED ARROW\0"
  "LEFTWARDS TWO-HEADED ARROW FROM BAR\0"
  "LEFTWARDS TWO-HEADED ARROW WITH DOUBLE VERTICAL STROKE\0"
  "LEFTWARDS TWO-HEADED ARROW WITH TAIL\0"
  "LEFTWARDS TWO-HEADED ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE\0"
  "LEFTWARDS TWO-HEADED ARROW WITH TAIL WITH VERTICAL STROKE\0"
  "LEFTWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS\0"
  "LEFTWARDS TWO-HEADED ARROW WITH VERTICAL STROKE\0"
  "LEFTWARDS TWO-HEADED TRIPLE DASH ARROW\0"
  "LEFTWARDS WAVE ARROW\0"
  "LEFTWARDS WHITE ARROW\0"
  "LEFTWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD\0"
  "LEG\0"
  "LEMON\0"
  "LEO\0"
  "LEOPARD\0"
  "LEPCHA CONSONANT SIGN K\0"
  "LEPCHA CONSONANT SIGN KANG\0"
  "LEPCHA CONSONANT SIGN L\0"
  "LEPCHA CONSONANT SIGN M\0"
  "LEPCHA CONSONANT SIGN N\0"
  "LEPCHA CONSONANT SIGN NYIN-DO\0"
  "LEPCHA CONSONANT SIGN P\0"
  "LEPCHA CONSONANT SIGN R\0"
  "LEPCHA CONSONANT SIGN T\0"
  "LEPCHA DIGIT EIGHT\0"
  "LEPCHA DIGIT FIVE\0"
  "LEPCHA DIGIT FOUR\0"
  "LEPCHA DIGIT NINE\0"
  "LEPCHA DIGIT ONE\0"
  "LEPCHA DIGIT SEVEN\0"
  "LEPCHA DIGIT SIX\0"
  "LEPCHA DIGIT THREE\0"
  "LEPCHA DIGIT TWO\0"
  "LEPCHA DIGIT ZERO\0"
  "LEPCHA LETTER A\0"
  "LEPCHA LETTER BA\0"
  "LEPCHA LETTER BLA\0"
  "LEPCHA LETTER CA\0"
  "LEPCHA LETTER CHA\0"
  "LEPCHA LETTER DA\0"
  "LEPCHA LETTER DDA\0"
  "LEPCHA LETTER DZA\0"
  "LEPCHA LETTER FA\0"
  "LEPCHA LETTER FLA\0"
  "LEPCHA LETTER GA\0"
  "LEPCHA LETTER GLA\0"
  "LEPCHA LETTER HA\0"
  "LEPCHA LETTER HLA\0"
  "LEPCHA LETTER JA\0"
  "LEPCHA LETTER KA\0"
  "LEPCHA LETTER KHA\0"
  "LEPCHA LETTER KLA\0"
  "LEPCHA LETTER LA\0"
  "LEPCHA LETTER MA\0"
  "LEPCHA LETTER MLA\0"
  "LEPCHA LETTER NA\0"
  "LEPCHA LETTER NGA\0"
  "LEPCHA LETTER NYA\0"
  "LEPCHA LETTER PA\0"
  "LEPCHA LETTER PHA\0"
  "LEPCHA LETTER PLA\0"
  "LEPCHA LETTER RA\0"
  "LEPCHA LETTER SA\0"
  "LEPCHA LETTER SHA\0"
  "LEPCHA LETTER TA\0"
  "LEPCHA LETTER THA\0"
  "LEPCHA LETTER TSA\0"
  "LEPCHA LETTER TSHA\0"
  "LEPCHA LETTER TTA\0"
  "LEPCHA LETTER TTHA\0"
  "LEPCHA LETTER VA\0"
  "LEPCHA LETTER WA\0"
  "LEPCHA LETTER YA\0"
  "LEPCHA PUNCTUATION CER-WA\0"
  "LEPCHA PUNCTUATION NYET THYOOM TA-ROL\0"
  "LEPCHA PUNCTUATION TA-ROL\0"
  "LEPCHA PUNCTUATION TSHOOK\0"
  "LEPCHA PUNCTUATION TSHOOK CER-WA\0"
  "LEPCHA SIGN NUKTA\0"
  "LEPCHA SIGN RAN\0"
  "LEPCHA SUBJOINED LETTER RA\0"
  "LEPCHA SUBJOINED LETTER YA\0"
  "LEPCHA VOWEL SIGN AA\0"
  "LEPCHA VOWEL SIGN E\0"
  "LEPCHA VOWEL SIGN I\0"
  "LEPCHA VOWEL SIGN O\0"
  "LEPCHA VOWEL SIGN OO\0"
  "LEPCHA VOWEL SIGN U\0"
  "LEPCHA VOWEL SIGN UU\0"
  "LESS-THAN ABOVE DOUBLE-LINE EQUAL ABOVE GREATER-THAN\0"
  "LESS-THAN ABOVE GREATER-THAN ABOVE DOUBLE-LINE EQUAL\0"
  "LESS-THAN ABOVE LEFTWARDS ARROW\0"
  "LESS-THAN ABOVE SIMILAR ABOVE GREATER-THAN\0"
  "LESS-THAN ABOVE SIMILAR OR EQUAL\0"
  "LESS-THAN ABOVE SLANTED EQUAL ABOVE GREATER-THAN ABOVE SLANTED EQUAL\0"
  "LESS-THAN AND NOT APPROXIMATE\0"
  "LESS-THAN AND SINGLE-LINE NOT EQUAL TO\0"
  "LESS-THAN BUT NOT EQUAL TO\0"
  "LESS-THAN BUT NOT EQUIVALENT TO\0"
  "LESS-THAN CLOSED BY CURVE\0"
  "LESS-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL\0"
  "LESS-THAN EQUAL TO OR GREATER-THAN\0"
  "LESS-THAN OR APPROXIMATE\0"
  "LESS-THAN OR EQUAL TO\0"
  "LESS-THAN OR EQUIVALENT TO\0"
  "LESS-THAN OR GREATER-THAN\0"
  "LESS-THAN OR SLANTED EQUAL TO\0"
  "LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE\0"
  "LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE RIGHT\0"
  "LESS-THAN OR SLANTED EQUAL TO WITH DOT INSIDE\0"
  "LESS-THAN OVER EQUAL TO\0"
  "LESS-THAN SIGN\0"
  "LESS-THAN WITH CIRCLE INSIDE\0"
  "LESS-THAN WITH DOT\0"
  "LESS-THAN WITH QUESTION MARK ABOVE\0"
  "LEVEL SLIDER\0"
  "LIBRA\0"
  "LIGATURE OPEN ET ORNAMENT\0"
  "LIGHT BLUE HEART\0"
  "LIGHT CHECK MARK\0"
  "LIGHT EIGHT SPOKED ASTERISK\0"
  "LIGHT FIVE POINTED BLACK STAR\0"
  "LIGHT FIVE SPOKED ASTERISK\0"
  "LIGHT FOUR POINTED BLACK CUSP\0"
  "LIGHT FOUR POINTED BLACK STAR\0"
  "LIGHT GREEK CROSS\0"
  "LIGHT LEFT TORTOISE SHELL BRACKET ORNAMENT\0"
  "LIGHT RAIL\0"
  "LIGHT RIGHT TORTOISE SHELL BRACKET ORNAMENT\0"
  "LIGHT SALTIRE\0"
  "LIGHT SHADE\0"
  "LIGHT SIX SPOKED ASTERISK\0"
  "LIGHT THREE POINTED BLACK STAR\0"
  "LIGHT TWELVE POINTED BLACK STAR\0"
  "LIGHT VERTICAL BAR\0"
  "LIGHT WHITE SQUARE\0"
  "LIGHTNING\0"
  "LIGHTNING MOOD\0"
  "LIGHTNING MOOD BUBBLE\0"
  "LIMBU DIGIT EIGHT\0"
  "LIMBU DIGIT FIVE\0"
  "LIMBU DIGIT FOUR\0"
  "LIMBU DIGIT NINE\0"
  "LIMBU DIGIT ONE\0"
  "LIMBU DIGIT SEVEN\0"
  "LIMBU DIGIT SIX\0"
  "LIMBU DIGIT THREE\0"
  "LIMBU DIGIT TWO\0"
  "LIMBU DIGIT ZERO\0"
  "LIMBU EXCLAMATION MARK\0"
  "LIMBU LETTER BA\0"
  "LIMBU LETTER BHA\0"
  "LIMBU LETTER CA\0"
  "LIMBU LETTER CHA\0"
  "LIMBU LETTER DA\0"
  "LIMBU LETTER DHA\0"
  "LIMBU LETTER GA\0"
  "LIMBU LETTER GHA\0"
  "LIMBU LETTER GYAN\0"
  "LIMBU LETTER HA\0"
  "LIMBU LETTER JA\0"
  "LIMBU LETTER JHA\0"
  "LIMBU LETTER KA\0"
  "LIMBU LETTER KHA\0"
  "LIMBU LETTER LA\0"
  "LIMBU LETTER MA\0"
  "LIMBU LETTER NA\0"
  "LIMBU LETTER NGA\0"
  "LIMBU LETTER PA\0"
  "LIMBU LETTER PHA\0"
  "LIMBU LETTER RA\0"
  "LIMBU LETTER SA\0"
  "LIMBU LETTER SHA\0"
  "LIMBU LETTER SSA\0"
  "LIMBU LETTER TA\0"
  "LIMBU LETTER THA\0"
  "LIMBU LETTER TRA\0"
  "LIMBU LETTER WA\0"
  "LIMBU LETTER YA\0"
  "LIMBU LETTER YAN\0"
  "LIMBU QUESTION MARK\0"
  "LIMBU SIGN KEMPHRENG\0"
  "LIMBU SIGN LOO\0"
  "LIMBU SIGN MUKPHRENG\0"
  "LIMBU SIGN SA-I\0"
  "LIMBU SMALL LETTER ANUSVARA\0"
  "LIMBU SMALL LETTER KA\0"
  "LIMBU SMALL LETTER LA\0"
  "LIMBU SMALL LETTER MA\0"
  "LIMBU SMALL LETTER NA\0"
  "LIMBU SMALL LETTER NGA\0"
  "LIMBU SMALL LETTER PA\0"
  "LIMBU SMALL LETTER RA\0"
  "LIMBU SMALL LETTER TA\0"
  "LIMBU SUBJOINED LETTER RA\0"
  "LIMBU SUBJOINED LETTER WA\0"
  "LIMBU SUBJOINED LETTER YA\0"
  "LIMBU VOWEL SIGN A\0"
  "LIMBU VOWEL SIGN AI\0"
  "LIMBU VOWEL SIGN AU\0"
  "LIMBU VOWEL SIGN E\0"
  "LIMBU VOWEL SIGN EE\0"
  "LIMBU VOWEL SIGN I\0"
  "LIMBU VOWEL SIGN O\0"
  "LIMBU VOWEL SIGN OO\0"
  "LIMBU VOWEL SIGN U\0"
  "LIMBU VOWEL-CARRIER LETTER\0"
  "LIMITED LIABILITY SIGN\0"
  "LINE INTEGRATION NOT INCLUDING THE POLE\0"
  "LINE INTEGRATION WITH RECTANGULAR PATH AROUND POLE\0"
  "LINE INTEGRATION WITH SEMICIRCULAR PATH AROUND POLE\0"
  "LINE SEPARATOR\0"
  "LINEAR A SIGN A028B\0"
  "LINEAR A SIGN A100-102\0"
  "LINEAR A SIGN A120B\0"
  "LINEAR A SIGN A131C\0"
  "LINEAR A SIGN A301\0"
  "LINEAR A SIGN A302\0"
  "LINEAR A SIGN A303\0"
  "LINEAR A SIGN A304\0"
  "LINEAR A SIGN A305\0"
  "LINEAR A SIGN A306\0"
  "LINEAR A SIGN A307\0"
  "LINEAR A SIGN A308\0"
  "LINEAR A SIGN A309A\0"
  "LINEAR A SIGN A309B\0"
  "LINEAR A SIGN A309C\0"
  "LINEAR A SIGN A310\0"
  "LINEAR A SIGN A311\0"
  "LINEAR A SIGN A312\0"
  "LINEAR A SIGN A313A\0"
  "LINEAR A SIGN A313B\0"
  "LINEAR A SIGN A313C\0"
  "LINEAR A SIGN A314\0"
  "LINEAR A SIGN A315\0"
  "LINEAR A SIGN A316\0"
  "LINEAR A SIGN A317\0"
  "LINEAR A SIGN A318\0"
  "LINEAR A SIGN A319\0"
  "LINEAR A SIGN A320\0"
  "LINEAR A SIGN A321\0"
  "LINEAR A SIGN A322\0"
  "LINEAR A SIGN A323\0"
  "LINEAR A SIGN A324\0"
  "LINEAR A SIGN A325\0"
  "LINEAR A SIGN A326\0"
  "LINEAR A SIGN A327\0"
  "LINEAR A SIGN A328\0"
  "LINEAR A SIGN A329\0"
  "LINEAR A SIGN A330\0"
  "LINEAR A SIGN A331\0"
  "LINEAR A SIGN A332\0"
  "LINEAR A SIGN A333\0"
  "LINEAR A SIGN A334\0"
  "LINEAR A SIGN A335\0"
  "LINEAR A SIGN A336\0"
  "LINEAR A SIGN A337\0"
  "LINEAR A SIGN A338\0"
  "LINEAR A SIGN A339\0"
  "LINEAR A SIGN A340\0"
  "LINEAR A SIGN A341\0"
  "LINEAR A SIGN A342\0"
  "LINEAR A SIGN A343\0"
  "LINEAR A SIGN A344\0"
  "LINEAR A SIGN A345\0"
  "LINEAR A SIGN A346\0"
  "LINEAR A SIGN A347\0"
  "LINEAR A SIGN A348\0"
  "LINEAR A SIGN A349\0"
  "LINEAR A SIGN A350\0"
  "LINEAR A SIGN A351\0"
  "LINEAR A SIGN A352\0"
  "LINEAR A SIGN A353\0"
  "LINEAR A SIGN A354\0"
  "LINEAR A SIGN A355\0"
  "LINEAR A SIGN A356\0"
  "LINEAR A SIGN A357\0"
  "LINEAR A SIGN A358\0"
  "LINEAR A SIGN A359\0"
  "LINEAR A SIGN A360\0"
  "LINEAR A SIGN A361\0"
  "LINEAR A SIGN A362\0"
  "LINEAR A SIGN A363\0"
  "LINEAR A SIGN A364\0"
  "LINEAR A SIGN A365\0"
  "LINEAR A SIGN A366\0"
  "LINEAR A SIGN A367\0"
  "LINEAR A SIGN A368\0"
  "LINEAR A SIGN A369\0"
  "LINEAR A SIGN A370\0"
  "LINEAR A SIGN A371\0"
  "LINEAR A SIGN A400-VAS\0"
  "LINEAR A SIGN A401-VAS\0"
  "LINEAR A SIGN A402-VAS\0"
  "LINEAR A SIGN A403-VAS\0"
  "LINEAR A SIGN A404-VAS\0"
  "LINEAR A SIGN A405-VAS\0"
  "LINEAR A SIGN A406-VAS\0"
  "LINEAR A SIGN A407-VAS\0"
  "LINEAR A SIGN A408-VAS\0"
  "LINEAR A SIGN A409-VAS\0"
  "LINEAR A SIGN A410-VAS\0"
  "LINEAR A SIGN A411-VAS\0"
  "LINEAR A SIGN A412-VAS\0"
  "LINEAR A SIGN A413-VAS\0"
  "LINEAR A SIGN A414-VAS\0"
  "LINEAR A SIGN A415-VAS\0"
  "LINEAR A SIGN A416-VAS\0"
  "LINEAR A SIGN A417-VAS\0"
  "LINEAR A SIGN A418-VAS\0"
  "LINEAR A SIGN A501\0"
  "LINEAR A SIGN A502\0"
  "LINEAR A SIGN A503\0"
  "LINEAR A SIGN A504\0"
  "LINEAR A SIGN A505\0"
  "LINEAR A SIGN A506\0"
  "LINEAR A SIGN A508\0"
  "LINEAR A SIGN A509\0"
  "LINEAR A SIGN A510\0"
  "LINEAR A SIGN A511\0"
  "LINEAR A SIGN A512\0"
  "LINEAR A SIGN A513\0"
  "LINEAR A SIGN A515\0"
  "LINEAR A SIGN A516\0"
  "LINEAR A SIGN A520\0"
  "LINEAR A SIGN A521\0"
  "LINEAR A SIGN A523\0"
  "LINEAR A SIGN A524\0"
  "LINEAR A SIGN A525\0"
  "LINEAR A SIGN A526\0"
  "LINEAR A SIGN A527\0"
  "LINEAR A SIGN A528\0"
  "LINEAR A SIGN A529\0"
  "LINEAR A SIGN A530\0"
  "LINEAR A SIGN A531\0"
  "LINEAR A SIGN A532\0"
  "LINEAR A SIGN A534\0"
  "LINEAR A SIGN A535\0"
  "LINEAR A SIGN A536\0"
  "LINEAR A SIGN A537\0"
  "LINEAR A SIGN A538\0"
  "LINEAR A SIGN A539\0"
  "LINEAR A SIGN A540\0"
  "LINEAR A SIGN A541\0"
  "LINEAR A SIGN A542\0"
  "LINEAR A SIGN A545\0"
  "LINEAR A SIGN A547\0"
  "LINEAR A SIGN A548\0"
  "LINEAR A SIGN A549\0"
  "LINEAR A SIGN A550\0"
  "LINEAR A SIGN A551\0"
  "LINEAR A SIGN A552\0"
  "LINEAR A SIGN A553\0"
  "LINEAR A SIGN A554\0"
  "LINEAR A SIGN A555\0"
  "LINEAR A SIGN A556\0"
  "LINEAR A SIGN A557\0"
  "LINEAR A SIGN A559\0"
  "LINEAR A SIGN A563\0"
  "LINEAR A SIGN A564\0"
  "LINEAR A SIGN A565\0"
  "LINEAR A SIGN A566\0"
  "LINEAR A SIGN A568\0"
  "LINEAR A SIGN A569\0"
  "LINEAR A SIGN A570\0"
  "LINEAR A SIGN A571\0"
  "LINEAR A SIGN A572\0"
  "LINEAR A SIGN A573\0"
  "LINEAR A SIGN A574\0"
  "LINEAR A SIGN A575\0"
  "LINEAR A SIGN A576\0"
  "LINEAR A SIGN A577\0"
  "LINEAR A SIGN A578\0"
  "LINEAR A SIGN A579\0"
  "LINEAR A SIGN A580\0"
  "LINEAR A SIGN A581\0"
  "LINEAR A SIGN A582\0"
  "LINEAR A SIGN A583\0"
  "LINEAR A SIGN A584\0"
  "LINEAR A SIGN A585\0"
  "LINEAR A SIGN A586\0"
  "LINEAR A SIGN A587\0"
  "LINEAR A SIGN A588\0"
  "LINEAR A SIGN A589\0"
  "LINEAR A SIGN A591\0"
  "LINEAR A SIGN A592\0"
  "LINEAR A SIGN A594\0"
  "LINEAR A SIGN A595\0"
  "LINEAR A SIGN A596\0"
  "LINEAR A SIGN A598\0"
  "LINEAR A SIGN A600\0"
  "LINEAR A SIGN A601\0"
  "LINEAR A SIGN A602\0"
  "LINEAR A SIGN A603\0"
  "LINEAR A SIGN A604\0"
  "LINEAR A SIGN A606\0"
  "LINEAR A SIGN A608\0"
  "LINEAR A SIGN A609\0"
  "LINEAR A SIGN A610\0"
  "LINEAR A SIGN A611\0"
  "LINEAR A SIGN A612\0"
  "LINEAR A SIGN A613\0"
  "LINEAR A SIGN A614\0"
  "LINEAR A SIGN A615\0"
  "LINEAR A SIGN A616\0"
  "LINEAR A SIGN A617\0"
  "LINEAR A SIGN A618\0"
  "LINEAR A SIGN A619\0"
  "LINEAR A SIGN A620\0"
  "LINEAR A SIGN A621\0"
  "LINEAR A SIGN A622\0"
  "LINEAR A SIGN A623\0"
  "LINEAR A SIGN A624\0"
  "LINEAR A SIGN A626\0"
  "LINEAR A SIGN A627\0"
  "LINEAR A SIGN A628\0"
  "LINEAR A SIGN A629\0"
  "LINEAR A SIGN A634\0"
  "LINEAR A SIGN A637\0"
  "LINEAR A SIGN A638\0"
  "LINEAR A SIGN A640\0"
  "LINEAR A SIGN A642\0"
  "LINEAR A SIGN A643\0"
  "LINEAR A SIGN A644\0"
  "LINEAR A SIGN A645\0"
  "LINEAR A SIGN A646\0"
  "LINEAR A SIGN A648\0"
  "LINEAR A SIGN A649\0"
  "LINEAR A SIGN A651\0"
  "LINEAR A SIGN A652\0"
  "LINEAR A SIGN A653\0"
  "LINEAR A SIGN A654\0"
  "LINEAR A SIGN A655\0"
  "LINEAR A SIGN A656\0"
  "LINEAR A SIGN A657\0"
  "LINEAR A SIGN A658\0"
  "LINEAR A SIGN A659\0"
  "LINEAR A SIGN A660\0"
  "LINEAR A SIGN A661\0"
  "LINEAR A SIGN A662\0"
  "LINEAR A SIGN A663\0"
  "LINEAR A SIGN A664\0"
  "LINEAR A SIGN A701 A\0"
  "LINEAR A SIGN A702 B\0"
  "LINEAR A SIGN A703 D\0"
  "LINEAR A SIGN A704 E\0"
  "LINEAR A SIGN A705 F\0"
  "LINEAR A SIGN A706 H\0"
  "LINEAR A SIGN A707 J\0"
  "LINEAR A SIGN A708 K\0"
  "LINEAR A SIGN A709 L\0"
  "LINEAR A SIGN A709-2 L2\0"
  "LINEAR A SIGN A709-3 L3\0"
  "LINEAR A SIGN A709-4 L4\0"
  "LINEAR A SIGN A709-6 L6\0"
  "LINEAR A SIGN A710 W\0"
  "LINEAR A SIGN A711 X\0"
  "LINEAR A SIGN A712 Y\0"
  "LINEAR A SIGN A713 OMEGA\0"
  "LINEAR A SIGN A714 ABB\0"
  "LINEAR A SIGN A715 BB\0"
  "LINEAR A SIGN A717 DD\0"
  "LINEAR A SIGN A726 EYYY\0"
  "LINEAR A SIGN A732 JE\0"
  "LINEAR A SIGN A800\0"
  "LINEAR A SIGN A801\0"
  "LINEAR A SIGN A802\0"
  "LINEAR A SIGN A803\0"
  "LINEAR A SIGN A804\0"
  "LINEAR A SIGN A805\0"
  "LINEAR A SIGN A806\0"
  "LINEAR A SIGN A807\0"
  "LINEAR A SIGN AB001\0"
  "LINEAR A SIGN AB002\0"
  "LINEAR A SIGN AB003\0"
  "LINEAR A SIGN AB004\0"
  "LINEAR A SIGN AB005\0"
  "LINEAR A SIGN AB006\0"
  "LINEAR A SIGN AB007\0"
  "LINEAR A SIGN AB008\0"
  "LINEAR A SIGN AB009\0"
  "LINEAR A SIGN AB010\0"
  "LINEAR A SIGN AB011\0"
  "LINEAR A SIGN AB013\0"
  "LINEAR A SIGN AB016\0"
  "LINEAR A SIGN AB017\0"
  "LINEAR A SIGN AB020\0"
  "LINEAR A SIGN AB021\0"
  "LINEAR A SIGN AB021F\0"
  "LINEAR A SIGN AB021M\0"
  "LINEAR A SIGN AB022\0"
  "LINEAR A SIGN AB022F\0"
  "LINEAR A SIGN AB022M\0"
  "LINEAR A SIGN AB023\0"
  "LINEAR A SIGN AB023M\0"
  "LINEAR A SIGN AB024\0"
  "LINEAR A SIGN AB026\0"
  "LINEAR A SIGN AB027\0"
  "LINEAR A SIGN AB028\0"
  "LINEAR A SIGN AB029\0"
  "LINEAR A SIGN AB030\0"
  "LINEAR A SIGN AB031\0"
  "LINEAR A SIGN AB034\0"
  "LINEAR A SIGN AB037\0"
  "LINEAR A SIGN AB038\0"
  "LINEAR A SIGN AB039\0"
  "LINEAR A SIGN AB040\0"
  "LINEAR A SIGN AB041\0"
  "LINEAR A SIGN AB044\0"
  "LINEAR A SIGN AB045\0"
  "LINEAR A SIGN AB046\0"
  "LINEAR A SIGN AB047\0"
  "LINEAR A SIGN AB048\0"
  "LINEAR A SIGN AB049\0"
  "LINEAR A SIGN AB050\0"
  "LINEAR A SIGN AB051\0"
  "LINEAR A SIGN AB053\0"
  "LINEAR A SIGN AB054\0"
  "LINEAR A SIGN AB055\0"
  "LINEAR A SIGN AB056\0"
  "LINEAR A SIGN AB057\0"
  "LINEAR A SIGN AB058\0"
  "LINEAR A SIGN AB059\0"
  "LINEAR A SIGN AB060\0"
  "LINEAR A SIGN AB061\0"
  "LINEAR A SIGN AB065\0"
  "LINEAR A SIGN AB066\0"
  "LINEAR A SIGN AB067\0"
  "LINEAR A SIGN AB069\0"
  "LINEAR A SIGN AB070\0"
  "LINEAR A SIGN AB073\0"
  "LINEAR A SIGN AB074\0"
  "LINEAR A SIGN AB076\0"
  "LINEAR A SIGN AB077\0"
  "LINEAR A SIGN AB078\0"
  "LINEAR A SIGN AB079\0"
  "LINEAR A SIGN AB080\0"
  "LINEAR A SIGN AB081\0"
  "LINEAR A SIGN AB082\0"
  "LINEAR A SIGN AB085\0"
  "LINEAR A SIGN AB086\0"
  "LINEAR A SIGN AB087\0"
  "LINEAR A SIGN AB118\0"
  "LINEAR A SIGN AB120\0"
  "LINEAR A SIGN AB122\0"
  "LINEAR A SIGN AB123\0"
  "LINEAR A SIGN AB131A\0"
  "LINEAR A SIGN AB131B\0"
  "LINEAR A SIGN AB164\0"
  "LINEAR A SIGN AB171\0"
  "LINEAR A SIGN AB180\0"
  "LINEAR A SIGN AB188\0"
  "LINEAR A SIGN AB191\0"
  "LINEAR B IDEOGRAM B100 MAN\0"
  "LINEAR B IDEOGRAM B102 WOMAN\0"
  "LINEAR B IDEOGRAM B104 DEER\0"
  "LINEAR B IDEOGRAM B105 EQUID\0"
  "LINEAR B IDEOGRAM B105F MARE\0"
  "LINEAR B IDEOGRAM B105M STALLION\0"
  "LINEAR B IDEOGRAM B106F EWE\0"
  "LINEAR B IDEOGRAM B106M RAM\0"
  "LINEAR B IDEOGRAM B107F SHE-GOAT\0"
  "LINEAR B IDEOGRAM B107M HE-GOAT\0"
  "LINEAR B IDEOGRAM B108F SOW\0"
  "LINEAR B IDEOGRAM B108M BOAR\0"
  "LINEAR B IDEOGRAM B109F COW\0"
  "LINEAR B IDEOGRAM B109M BULL\0"
  "LINEAR B IDEOGRAM B120 WHEAT\0"
  "LINEAR B IDEOGRAM B121 BARLEY\0"
  "LINEAR B IDEOGRAM B122 OLIVE\0"
  "LINEAR B IDEOGRAM B123 SPICE\0"
  "LINEAR B IDEOGRAM B125 CYPERUS\0"
  "LINEAR B IDEOGRAM B130 OIL\0"
  "LINEAR B IDEOGRAM B131 WINE\0"
  "LINEAR B IDEOGRAM B132\0"
  "LINEAR B IDEOGRAM B140 BRONZE\0"
  "LINEAR B IDEOGRAM B141 GOLD\0"
  "LINEAR B IDEOGRAM B142\0"
  "LINEAR B IDEOGRAM B145 WOOL\0"
  "LINEAR B IDEOGRAM B146\0"
  "LINEAR B IDEOGRAM B150\0"
  "LINEAR B IDEOGRAM B151 HORN\0"
  "LINEAR B IDEOGRAM B152\0"
  "LINEAR B IDEOGRAM B153\0"
  "LINEAR B IDEOGRAM B154\0"
  "LINEAR B IDEOGRAM B157\0"
  "LINEAR B IDEOGRAM B158\0"
  "LINEAR B IDEOGRAM B159 CLOTH\0"
  "LINEAR B IDEOGRAM B160\0"
  "LINEAR B IDEOGRAM B161\0"
  "LINEAR B IDEOGRAM B162 GARMENT\0"
  "LINEAR B IDEOGRAM B163 ARMOUR\0"
  "LINEAR B IDEOGRAM B164\0"
  "LINEAR B IDEOGRAM B165\0"
  "LINEAR B IDEOGRAM B166\0"
  "LINEAR B IDEOGRAM B167\0"
  "LINEAR B IDEOGRAM B168\0"
  "LINEAR B IDEOGRAM B169\0"
  "LINEAR B IDEOGRAM B170\0"
  "LINEAR B IDEOGRAM B171\0"
  "LINEAR B IDEOGRAM B172\0"
  "LINEAR B IDEOGRAM B173 MONTH\0"
  "LINEAR B IDEOGRAM B174\0"
  "LINEAR B IDEOGRAM B176 TREE\0"
  "LINEAR B IDEOGRAM B177\0"
  "LINEAR B IDEOGRAM B178\0"
  "LINEAR B IDEOGRAM B179\0"
  "LINEAR B IDEOGRAM B180\0"
  "LINEAR B IDEOGRAM B181\0"
  "LINEAR B IDEOGRAM B182\0"
  "LINEAR B IDEOGRAM B183\0"
  "LINEAR B IDEOGRAM B184\0"
  "LINEAR B IDEOGRAM B185\0"
  "LINEAR B IDEOGRAM B189\0"
  "LINEAR B IDEOGRAM B190\0"
  "LINEAR B IDEOGRAM B191 HELMET\0"
  "LINEAR B IDEOGRAM B220 FOOTSTOOL\0"
  "LINEAR B IDEOGRAM B225 BATHTUB\0"
  "LINEAR B IDEOGRAM B230 SPEAR\0"
  "LINEAR B IDEOGRAM B231 ARROW\0"
  "LINEAR B IDEOGRAM B232\0"
  "LINEAR B IDEOGRAM B233 SWORD\0"
  "LINEAR B IDEOGRAM B234\0"
  "LINEAR B IDEOGRAM B236\0"
  "LINEAR B IDEOGRAM B240 WHEELED CHARIOT\0"
  "LINEAR B IDEOGRAM B241 CHARIOT\0"
  "LINEAR B IDEOGRAM B242 CHARIOT FRAME\0"
  "LINEAR B IDEOGRAM B243 WHEEL\0"
  "LINEAR B IDEOGRAM B245\0"
  "LINEAR B IDEOGRAM B246\0"
  "LINEAR B IDEOGRAM B248\0"
  "LINEAR B IDEOGRAM B249\0"
  "LINEAR B IDEOGRAM B251\0"
  "LINEAR B IDEOGRAM B252\0"
  "LINEAR B IDEOGRAM B253\0"
  "LINEAR B IDEOGRAM B254 DART\0"
  "LINEAR B IDEOGRAM B255\0"
  "LINEAR B IDEOGRAM B256\0"
  "LINEAR B IDEOGRAM B257\0"
  "LINEAR B IDEOGRAM B258\0"
  "LINEAR B IDEOGRAM B259\0"
  "LINEAR B IDEOGRAM VESSEL B155\0"
  "LINEAR B IDEOGRAM VESSEL B200\0"
  "LINEAR B IDEOGRAM VESSEL B201\0"
  "LINEAR B IDEOGRAM VESSEL B202\0"
  "LINEAR B IDEOGRAM VESSEL B203\0"
  "LINEAR B IDEOGRAM VESSEL B204\0"
  "LINEAR B IDEOGRAM VESSEL B205\0"
  "LINEAR B IDEOGRAM VESSEL B206\0"
  "LINEAR B IDEOGRAM VESSEL B207\0"
  "LINEAR B IDEOGRAM VESSEL B208\0"
  "LINEAR B IDEOGRAM VESSEL B209\0"
  "LINEAR B IDEOGRAM VESSEL B210\0"
  "LINEAR B IDEOGRAM VESSEL B211\0"
  "LINEAR B IDEOGRAM VESSEL B212\0"
  "LINEAR B IDEOGRAM VESSEL B213\0"
  "LINEAR B IDEOGRAM VESSEL B214\0"
  "LINEAR B IDEOGRAM VESSEL B215\0"
  "LINEAR B IDEOGRAM VESSEL B216\0"
  "LINEAR B IDEOGRAM VESSEL B217\0"
  "LINEAR B IDEOGRAM VESSEL B218\0"
  "LINEAR B IDEOGRAM VESSEL B219\0"
  "LINEAR B IDEOGRAM VESSEL B221\0"
  "LINEAR B IDEOGRAM VESSEL B222\0"
  "LINEAR B IDEOGRAM VESSEL B226\0"
  "LINEAR B IDEOGRAM VESSEL B227\0"
  "LINEAR B IDEOGRAM VESSEL B228\0"
  "LINEAR B IDEOGRAM VESSEL B229\0"
  "LINEAR B IDEOGRAM VESSEL B250\0"
  "LINEAR B IDEOGRAM VESSEL B305\0"
  "LINEAR B MONOGRAM B127 KAPO\0"
  "LINEAR B MONOGRAM B128 KANAKO\0"
  "LINEAR B MONOGRAM B133 AREPA\0"
  "LINEAR B MONOGRAM B135 MERI\0"
  "LINEAR B MONOGRAM B156 TURO2\0"
  "LINEAR B MONOGRAM B247 DIPTE\0"
  "LINEAR B SYLLABLE B001 DA\0"
  "LINEAR B SYLLABLE B002 RO\0"
  "LINEAR B SYLLABLE B003 PA\0"
  "LINEAR B SYLLABLE B004 TE\0"
  "LINEAR B SYLLABLE B005 TO\0"
  "LINEAR B SYLLABLE B006 NA\0"
  "LINEAR B SYLLABLE B007 DI\0"
  "LINEAR B SYLLABLE B008 A\0"
  "LINEAR B SYLLABLE B009 SE\0"
  "LINEAR B SYLLABLE B010 U\0"
  "LINEAR B SYLLABLE B011 PO\0"
  "LINEAR B SYLLABLE B012 SO\0"
  "LINEAR B SYLLABLE B013 ME\0"
  "LINEAR B SYLLABLE B014 DO\0"
  "LINEAR B SYLLABLE B015 MO\0"
  "LINEAR B SYLLABLE B016 QA\0"
  "LINEAR B SYLLABLE B017 ZA\0"
  "LINEAR B SYLLABLE B020 ZO\0"
  "LINEAR B SYLLABLE B021 QI\0"
  "LINEAR B SYLLABLE B023 MU\0"
  "LINEAR B SYLLABLE B024 NE\0"
  "LINEAR B SYLLABLE B025 A2\0"
  "LINEAR B SYLLABLE B026 RU\0"
  "LINEAR B SYLLABLE B027 RE\0"
  "LINEAR B SYLLABLE B028 I\0"
  "LINEAR B SYLLABLE B029 PU2\0"
  "LINEAR B SYLLABLE B030 NI\0"
  "LINEAR B SYLLABLE B031 SA\0"
  "LINEAR B SYLLABLE B032 QO\0"
  "LINEAR B SYLLABLE B033 RA3\0"
  "LINEAR B SYLLABLE B036 JO\0"
  "LINEAR B SYLLABLE B037 TI\0"
  "LINEAR B SYLLABLE B038 E\0"
  "LINEAR B SYLLABLE B039 PI\0"
  "LINEAR B SYLLABLE B040 WI\0"
  "LINEAR B SYLLABLE B041 SI\0"
  "LINEAR B SYLLABLE B042 WO\0"
  "LINEAR B SYLLABLE B043 A3\0"
  "LINEAR B SYLLABLE B044 KE\0"
  "LINEAR B SYLLABLE B045 DE\0"
  "LINEAR B SYLLABLE B046 JE\0"
  "LINEAR B SYLLABLE B048 NWA\0"
  "LINEAR B SYLLABLE B050 PU\0"
  "LINEAR B SYLLABLE B051 DU\0"
  "LINEAR B SYLLABLE B052 NO\0"
  "LINEAR B SYLLABLE B053 RI\0"
  "LINEAR B SYLLABLE B054 WA\0"
  "LINEAR B SYLLABLE B055 NU\0"
  "LINEAR B SYLLABLE B057 JA\0"
  "LINEAR B SYLLABLE B058 SU\0"
  "LINEAR B SYLLABLE B059 TA\0"
  "LINEAR B SYLLABLE B060 RA\0"
  "LINEAR B SYLLABLE B061 O\0"
  "LINEAR B SYLLABLE B062 PTE\0"
  "LINEAR B SYLLABLE B065 JU\0"
  "LINEAR B SYLLABLE B066 TA2\0"
  "LINEAR B SYLLABLE B067 KI\0"
  "LINEAR B SYLLABLE B068 RO2\0"
  "LINEAR B SYLLABLE B069 TU\0"
  "LINEAR B SYLLABLE B070 KO\0"
  "LINEAR B SYLLABLE B071 DWE\0"
  "LINEAR B SYLLABLE B072 PE\0"
  "LINEAR B SYLLABLE B073 MI\0"
  "LINEAR B SYLLABLE B074 ZE\0"
  "LINEAR B SYLLABLE B075 WE\0"
  "LINEAR B SYLLABLE B076 RA2\0"
  "LINEAR B SYLLABLE B077 KA\0"
  "LINEAR B SYLLABLE B078 QE\0"
  "LINEAR B SYLLABLE B080 MA\0"
  "LINEAR B SYLLABLE B081 KU\0"
  "LINEAR B SYLLABLE B085 AU\0"
  "LINEAR B SYLLABLE B087 TWE\0"
  "LINEAR B SYLLABLE B090 DWO\0"
  "LINEAR B SYLLABLE B091 TWO\0"
  "LINEAR B SYMBOL B018\0"
  "LINEAR B SYMBOL B019\0"
  "LINEAR B SYMBOL B022\0"
  "LINEAR B SYMBOL B034\0"
  "LINEAR B SYMBOL B047\0"
  "LINEAR B SYMBOL B049\0"
  "LINEAR B SYMBOL B056\0"
  "LINEAR B SYMBOL B063\0"
  "LINEAR B SYMBOL B064\0"
  "LINEAR B SYMBOL B079\0"
  "LINEAR B SYMBOL B082\0"
  "LINEAR B SYMBOL B083\0"
  "LINEAR B SYMBOL B086\0"
  "LINEAR B SYMBOL B089\0"
  "LINK SYMBOL\0"
  "LINKED PAPERCLIPS\0"
  "LION FACE\0"
  "LIPS\0"
  "LIPSTICK\0"
  "LIRA SIGN\0"
  "LISU LETTER A\0"
  "LISU LETTER AE\0"
  "LISU LETTER BA\0"
  "LISU LETTER CA\0"
  "LISU LETTER CHA\0"
  "LISU LETTER DA\0"
  "LISU LETTER DZA\0"
  "LISU LETTER E\0"
  "LISU LETTER EU\0"
  "LISU LETTER FA\0"
  "LISU LETTER GA\0"
  "LISU LETTER GHA\0"
  "LISU LETTER HA\0"
  "LISU LETTER HHA\0"
  "LISU LETTER I\0"
  "LISU LETTER JA\0"
  "LISU LETTER KA\0"
  "LISU LETTER KHA\0"
  "LISU LETTER LA\0"
  "LISU LETTER MA\0"
  "LISU LETTER NA\0"
  "LISU LETTER NGA\0"
  "LISU LETTER O\0"
  "LISU LETTER OE\0"
  "LISU LETTER PA\0"
  "LISU LETTER PHA\0"
  "LISU LETTER SA\0"
  "LISU LETTER SHA\0"
  "LISU LETTER TA\0"
  "LISU LETTER THA\0"
  "LISU LETTER TONE MYA BO\0"
  "LISU LETTER TONE MYA CYA\0"
  "LISU LETTER TONE MYA JEU\0"
  "LISU LETTER TONE MYA NA\0"
  "LISU LETTER TONE MYA TI\0"
  "LISU LETTER TONE NA PO\0"
  "LISU LETTER TSA\0"
  "LISU LETTER TSHA\0"
  "LISU LETTER U\0"
  "LISU LETTER UE\0"
  "LISU LETTER UH\0"
  "LISU LETTER WA\0"
  "LISU LETTER XA\0"
  "LISU LETTER YA\0"
  "LISU LETTER YHA\0"
  "LISU LETTER ZA\0"
  "LISU LETTER ZHA\0"
  "LISU PUNCTUATION COMMA\0"
  "LISU PUNCTUATION FULL STOP\0"
  "LIVRE TOURNOIS SIGN\0"
  "LIZARD\0"
  "LLAMA\0"
  "LOBSTER\0"
  "LOCK\0"
  "LOCK WITH INK PEN\0"
  "LOGIC GATE AND\0"
  "LOGIC GATE BUFFER\0"
  "LOGIC GATE BUFFER WITH INVERTED INPUT\0"
  "LOGIC GATE INVERTED INPUTS\0"
  "LOGIC GATE INVERTED OUTPUT\0"
  "LOGIC GATE OR\0"
  "LOGICAL AND\0"
  "LOGICAL AND WITH DOT ABOVE\0"
  "LOGICAL AND WITH DOUBLE OVERBAR\0"
  "LOGICAL AND WITH DOUBLE UNDERBAR\0"
  "LOGICAL AND WITH HORIZONTAL DASH\0"
  "LOGICAL AND WITH MIDDLE STEM\0"
  "LOGICAL AND WITH UNDERBAR\0"
  "LOGICAL OR\0"
  "LOGICAL OR OVERLAPPING LOGICAL AND\0"
  "LOGICAL OR WITH DOT ABOVE\0"
  "LOGICAL OR WITH DOUBLE OVERBAR\0"
  "LOGICAL OR WITH DOUBLE UNDERBAR\0"
  "LOGICAL OR WITH HORIZONTAL DASH\0"
  "LOGICAL OR WITH MIDDLE STEM\0"
  "LOLLIPOP\0"
  "LONG DASH FROM LEFT MEMBER OF DOUBLE VERTICAL\0"
  "LONG DIVISION\0"
  "LONG DRUM\0"
  "LONG LEFT RIGHT ARROW\0"
  "LONG LEFT RIGHT DOUBLE ARROW\0"
  "LONG LEFT TACK\0"
  "LONG LEFTWARDS ARROW\0"
  "LONG LEFTWARDS ARROW FROM BAR\0"
  "LONG LEFTWARDS DOUBLE ARROW\0"
  "LONG LEFTWARDS DOUBLE ARROW FROM BAR\0"
  "LONG LEFTWARDS SQUIGGLE ARROW\0"
  "LONG RIGHT TACK\0"
  "LONG RIGHTWARDS ARROW\0"
  "LONG RIGHTWARDS ARROW FROM BAR\0"
  "LONG RIGHTWARDS DOUBLE ARROW\0"
  "LONG RIGHTWARDS DOUBLE ARROW FROM BAR\0"
  "LONG RIGHTWARDS SQUIGGLE ARROW\0"
  "LOT OF FORTUNE\0"
  "LOTION BOTTLE\0"
  "LOTUS\0"
  "LOUDLY CRYING FACE\0"
  "LOVE HOTEL\0"
  "LOVE LETTER\0"
  "LOW ASTERISK\0"
  "LOW BATTERY\0"
  "LOW BRIGHTNESS SYMBOL\0"
  "LOW DOUBLE PRIME QUOTATION MARK\0"
  "LOW KAVYKA\0"
  "LOW KAVYKA WITH DOT\0"
  "LOW LINE\0"
  "LOWER BLADE SCISSORS\0"
  "LOWER CENTRE LEFT ONE SIXTEENTH BLOCK\0"
  "LOWER CENTRE LEFT TWELFTH CIRCLE\0"
  "LOWER CENTRE ONE QUARTER BLOCK\0"
  "LOWER CENTRE RIGHT ONE SIXTEENTH BLOCK\0"
  "LOWER CENTRE RIGHT TWELFTH CIRCLE\0"
  "LOWER FIVE EIGHTHS BLOCK\0"
  "LOWER HALF AND UPPER HALF WHITE CIRCLE\0"
  "LOWER HALF BLOCK\0"
  "LOWER HALF CIRCLE\0"
  "LOWER HALF HEAVY WHITE SQUARE\0"
  "LOWER HALF INVERSE WHITE CIRCLE\0"
  "LOWER HALF LEFT ONE QUARTER BLOCK\0"
  "LOWER HALF MEDIUM SHADE\0"
  "LOWER HALF RIGHT ONE QUARTER BLOCK\0"
  "LOWER HALF VERTICAL LINE WITH THREE TICK MARKS\0"
  "LOWER HORIZONTAL RULER SEGMENT\0"
  "LOWER LEFT BALLPOINT PEN\0"
  "LOWER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER CENTRE\0"
  "LOWER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER RIGHT\0"
  "LOWER LEFT BLOCK DIAGONAL UPPER CENTRE TO LOWER MIDDLE RIGHT\0"
  "LOWER LEFT BLOCK DIAGONAL UPPER CENTRE TO LOWER RIGHT\0"
  "LOWER LEFT BLOCK DIAGONAL UPPER CENTRE TO UPPER MIDDLE RIGHT\0"
  "LOWER LEFT BLOCK DIAGONAL UPPER LEFT TO LOWER CENTRE\0"
  "LOWER LEFT BLOCK DIAGONAL UPPER LEFT TO LOWER MIDDLE RIGHT\0"
  "LOWER LEFT BLOCK DIAGONAL UPPER LEFT TO UPPER MIDDLE RIGHT\0"
  "LOWER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER CENTRE\0"
  "LOWER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER MIDDLE RIGHT\0"
  "LOWER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER RIGHT\0"
  "LOWER LEFT CRAYON\0"
  "LOWER LEFT FOUNTAIN PEN\0"
  "LOWER LEFT ONE SIXTEENTH BLOCK\0"
  "LOWER LEFT PAINTBRUSH\0"
  "LOWER LEFT PENCIL\0"
  "LOWER LEFT QUADRANT CHESS BISHOP\0"
  "LOWER LEFT QUADRANT CHESS KING\0"
  "LOWER LEFT QUADRANT CHESS KNIGHT\0"
  "LOWER LEFT QUADRANT CHESS PAWN\0"
  "LOWER LEFT QUADRANT CHESS QUEEN\0"
  "LOWER LEFT QUADRANT CHESS ROOK\0"
  "LOWER LEFT QUADRANT CIRCULAR ARC\0"
  "LOWER LEFT QUADRANT FACE WITH OPEN MOUTH\0"
  "LOWER LEFT QUADRANT FROWNING FACE\0"
  "LOWER LEFT QUADRANT MICROCOMPUTER\0"
  "LOWER LEFT QUADRANT NEUTRAL FACE\0"
  "LOWER LEFT QUADRANT SMILING FACE\0"
  "LOWER LEFT QUADRANT STANDING KNIGHT\0"
  "LOWER LEFT QUADRANT TELEVISION\0"
  "LOWER LEFT QUARTER CIRCLE\0"
  "LOWER LEFT SEMICIRCULAR ANTICLOCKWISE ARROW\0"
  "LOWER LEFT TRIANGLE\0"
  "LOWER LEFT TRIANGULAR MEDIUM SHADE\0"
  "LOWER LEFT TWELFTH CIRCLE\0"
  "LOWER MIDDLE CENTRE LEFT ONE SIXTEENTH BLOCK\0"
  "LOWER MIDDLE CENTRE RIGHT ONE SIXTEENTH BLOCK\0"
  "LOWER MIDDLE LEFT ONE SIXTEENTH BLOCK\0"
  "LOWER MIDDLE LEFT TWELFTH CIRCLE\0"
  "LOWER MIDDLE RIGHT ONE SIXTEENTH BLOCK\0"
  "LOWER MIDDLE RIGHT TWELFTH CIRCLE\0"
  "LOWER ONE EIGHTH BLOCK\0"
  "LOWER ONE QUARTER BLOCK\0"
  "LOWER RIGHT BLOCK DIAGONAL LOWER CENTRE TO LOWER MIDDLE RIGHT\0"
  "LOWER RIGHT BLOCK DIAGONAL LOWER CENTRE TO UPPER MIDDLE RIGHT\0"
  "LOWER RIGHT BLOCK DIAGONAL LOWER CENTRE TO UPPER RIGHT\0"
  "LOWER RIGHT BLOCK DIAGONAL LOWER LEFT TO LOWER MIDDLE RIGHT\0"
  "LOWER RIGHT BLOCK DIAGONAL LOWER LEFT TO UPPER CENTRE\0"
  "LOWER RIGHT BLOCK DIAGONAL LOWER LEFT TO UPPER MIDDLE RIGHT\0"
  "LOWER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER CENTRE\0"
  "LOWER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER MIDDLE RIGHT\0"
  "LOWER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER RIGHT\0"
  "LOWER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER CENTRE\0"
  "LOWER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER RIGHT\0"
  "LOWER RIGHT CORNER WITH DOT\0"
  "LOWER RIGHT DROP-SHADOWED WHITE SQUARE\0"
  "LOWER RIGHT ONE SIXTEENTH BLOCK\0"
  "LOWER RIGHT PENCIL\0"
  "LOWER RIGHT QUADRANT CHESS BISHOP\0"
  "LOWER RIGHT QUADRANT CHESS KING\0"
  "LOWER RIGHT QUADRANT CHESS KNIGHT\0"
  "LOWER RIGHT QUADRANT CHESS PAWN\0"
  "LOWER RIGHT QUADRANT CHESS QUEEN\0"
  "LOWER RIGHT QUADRANT CHESS ROOK\0"
  "LOWER RIGHT QUADRANT CIRCULAR ARC\0"
  "LOWER RIGHT QUADRANT FACE WITH OPEN MOUTH\0"
  "LOWER RIGHT QUADRANT FROWNING FACE\0"
  "LOWER RIGHT QUADRANT MICROCOMPUTER\0"
  "LOWER RIGHT QUADRANT NEUTRAL FACE\0"
  "LOWER RIGHT QUADRANT SMILING FACE\0"
  "LOWER RIGHT QUADRANT STANDING KNIGHT\0"
  "LOWER RIGHT QUADRANT TELEVISION\0"
  "LOWER RIGHT QUARTER CIRCLE\0"
  "LOWER RIGHT RULER SEGMENT\0"
  "LOWER RIGHT SEMICIRCULAR CLOCKWISE ARROW\0"
  "LOWER RIGHT SHADOWED WHITE CIRCLE\0"
  "LOWER RIGHT SHADOWED WHITE SQUARE\0"
  "LOWER RIGHT TRIANGLE\0"
  "LOWER RIGHT TRIANGULAR MEDIUM SHADE\0"
  "LOWER RIGHT TWELFTH CIRCLE\0"
  "LOWER SEVEN EIGHTHS BLOCK\0"
  "LOWER THREE EIGHTHS BLOCK\0"
  "LOWER THREE QUARTERS BLOCK\0"
  "LOWER THREE QUARTERS LEFT ONE QUARTER BLOCK\0"
  "LOWER THREE QUARTERS RIGHT ONE QUARTER BLOCK\0"
  "LOWER TRIANGULAR ONE QUARTER BLOCK\0"
  "LOZENGE\0"
  "LOZENGE DIVIDED BY HORIZONTAL RULE\0"
  "LUGGAGE\0"
  "LUNAR ECLIPSE\0"
  "LUNGS\0"
  "LYCIAN LETTER A\0"
  "LYCIAN LETTER AN\0"
  "LYCIAN LETTER B\0"
  "LYCIAN LETTER BH\0"
  "LYCIAN LETTER D\0"
  "LYCIAN LETTER E\0"
  "LYCIAN LETTER EN\0"
  "LYCIAN LETTER G\0"
  "LYCIAN LETTER H\0"
  "LYCIAN LETTER I\0"
  "LYCIAN LETTER J\0"
  "LYCIAN LETTER K\0"
  "LYCIAN LETTER KK\0"
  "LYCIAN LETTER L\0"
  "LYCIAN LETTER M\0"
  "LYCIAN LETTER MM\0"
  "LYCIAN LETTER N\0"
  "LYCIAN LETTER NN\0"
  "LYCIAN LETTER P\0"
  "LYCIAN LETTER Q\0"
  "LYCIAN LETTER R\0"
  "LYCIAN LETTER S\0"
  "LYCIAN LETTER T\0"
  "LYCIAN LETTER TH\0"
  "LYCIAN LETTER TT\0"
  "LYCIAN LETTER U\0"
  "LYCIAN LETTER W\0"
  "LYCIAN LETTER X\0"
  "LYCIAN LETTER Z\0"
  "LYDIAN LETTER A\0"
  "LYDIAN LETTER AN\0"
  "LYDIAN LETTER B\0"
  "LYDIAN LETTER C\0"
  "LYDIAN LETTER D\0"
  "LYDIAN LETTER E\0"
  "LYDIAN LETTER EN\0"
  "LYDIAN LETTER F\0"
  "LYDIAN LETTER G\0"
  "LYDIAN LETTER I\0"
  "LYDIAN LETTER K\0"
  "LYDIAN LETTER L\0"
  "LYDIAN LETTER LY\0"
  "LYDIAN LETTER M\0"
  "LYDIAN LETTER N\0"
  "LYDIAN LETTER NN\0"
  "LYDIAN LETTER O\0"
  "LYDIAN LETTER Q\0"
  "LYDIAN LETTER R\0"
  "LYDIAN LETTER S\0"
  "LYDIAN LETTER SS\0"
  "LYDIAN LETTER T\0"
  "LYDIAN LETTER TT\0"
  "LYDIAN LETTER U\0"
  "LYDIAN LETTER V\0"
  "LYDIAN LETTER Y\0"
  "LYDIAN TRIANGULAR MARK\0"
  "LYING FACE\0"
  "MACRON\0"
  "MAGE\0"
  "MAGIC WAND\0"
  "MAGNET\0"
  "MAHAJANI ABBREVIATION SIGN\0"
  "MAHAJANI LETTER A\0"
  "MAHAJANI LETTER BA\0"
  "MAHAJANI LETTER BHA\0"
  "MAHAJANI LETTER CA\0"
  "MAHAJANI LETTER CHA\0"
  "MAHAJANI LETTER DA\0"
  "MAHAJANI LETTER DDA\0"
  "MAHAJANI LETTER DDHA\0"
  "MAHAJANI LETTER DHA\0"
  "MAHAJANI LETTER E\0"
  "MAHAJANI LETTER GA\0"
  "MAHAJANI LETTER GHA\0"
  "MAHAJANI LETTER HA\0"
  "MAHAJANI LETTER I\0"
  "MAHAJANI LETTER JA\0"
  "MAHAJANI LETTER JHA\0"
  "MAHAJANI LETTER KA\0"
  "MAHAJANI LETTER KHA\0"
  "MAHAJANI LETTER LA\0"
  "MAHAJANI LETTER MA\0"
  "MAHAJANI LETTER NA\0"
  "MAHAJANI LETTER NNA\0"
  "MAHAJANI LETTER NYA\0"
  "MAHAJANI LETTER O\0"
  "MAHAJANI LETTER PA\0"
  "MAHAJANI LETTER PHA\0"
  "MAHAJANI LETTER RA\0"
  "MAHAJANI LETTER RRA\0"
  "MAHAJANI LETTER SA\0"
  "MAHAJANI LETTER TA\0"
  "MAHAJANI LETTER THA\0"
  "MAHAJANI LETTER TTA\0"
  "MAHAJANI LETTER TTHA\0"
  "MAHAJANI LETTER U\0"
  "MAHAJANI LETTER VA\0"
  "MAHAJANI LIGATURE SHRI\0"
  "MAHAJANI SECTION MARK\0"
  "MAHAJANI SIGN NUKTA\0"
  "MAHJONG TILE AUTUMN\0"
  "MAHJONG TILE BACK\0"
  "MAHJONG TILE BAMBOO\0"
  "MAHJONG TILE CHRYSANTHEMUM\0"
  "MAHJONG TILE EAST WIND\0"
  "MAHJONG TILE EIGHT OF BAMBOOS\0"
  "MAHJONG TILE EIGHT OF CHARACTERS\0"
  "MAHJONG TILE EIGHT OF CIRCLES\0"
  "MAHJONG TILE FIVE OF BAMBOOS\0"
  "MAHJONG TILE FIVE OF CHARACTERS\0"
  "MAHJONG TILE FIVE OF CIRCLES\0"
  "MAHJONG TILE FOUR OF BAMBOOS\0"
  "MAHJONG TILE FOUR OF CHARACTERS\0"
  "MAHJONG TILE FOUR OF CIRCLES\0"
  "MAHJONG TILE GREEN DRAGON\0"
  "MAHJONG TILE JOKER\0"
  "MAHJONG TILE NINE OF BAMBOOS\0"
  "MAHJONG TILE NINE OF CHARACTERS\0"
  "MAHJONG TILE NINE OF CIRCLES\0"
  "MAHJONG TILE NORTH WIND\0"
  "MAHJONG TILE ONE OF BAMBOOS\0"
  "MAHJONG TILE ONE OF CHARACTERS\0"
  "MAHJONG TILE ONE OF CIRCLES\0"
  "MAHJONG TILE ORCHID\0"
  "MAHJONG TILE PLUM\0"
  "MAHJONG TILE RED DRAGON\0"
  "MAHJONG TILE SEVEN OF BAMBOOS\0"
  "MAHJONG TILE SEVEN OF CHARACTERS\0"
  "MAHJONG TILE SEVEN OF CIRCLES\0"
  "MAHJONG TILE SIX OF BAMBOOS\0"
  "MAHJONG TILE SIX OF CHARACTERS\0"
  "MAHJONG TILE SIX OF CIRCLES\0"
  "MAHJONG TILE SOUTH WIND\0"
  "MAHJONG TILE SPRING\0"
  "MAHJONG TILE SUMMER\0"
  "MAHJONG TILE THREE OF BAMBOOS\0"
  "MAHJONG TILE THREE OF CHARACTERS\0"
  "MAHJONG TILE THREE OF CIRCLES\0"
  "MAHJONG TILE TWO OF BAMBOOS\0"
  "MAHJONG TILE TWO OF CHARACTERS\0"
  "MAHJONG TILE TWO OF CIRCLES\0"
  "MAHJONG TILE WEST WIND\0"
  "MAHJONG TILE WHITE DRAGON\0"
  "MAHJONG TILE WINTER\0"
  "MAKASAR ANGKA\0"
  "MAKASAR END OF SECTION\0"
  "MAKASAR LETTER A\0"
  "MAKASAR LETTER BA\0"
  "MAKASAR LETTER CA\0"
  "MAKASAR LETTER DA\0"
  "MAKASAR LETTER GA\0"
  "MAKASAR LETTER JA\0"
  "MAKASAR LETTER KA\0"
  "MAKASAR LETTER LA\0"
  "MAKASAR LETTER MA\0"
  "MAKASAR LETTER NA\0"
  "MAKASAR LETTER NGA\0"
  "MAKASAR LETTER NYA\0"
  "MAKASAR LETTER PA\0"
  "MAKASAR LETTER RA\0"
  "MAKASAR LETTER SA\0"
  "MAKASAR LETTER TA\0"
  "MAKASAR LETTER VA\0"
  "MAKASAR LETTER YA\0"
  "MAKASAR PASSIMBANG\0"
  "MAKASAR VOWEL SIGN E\0"
  "MAKASAR VOWEL SIGN I\0"
  "MAKASAR VOWEL SIGN O\0"
  "MAKASAR VOWEL SIGN U\0"
  "MAKEMAKE\0"
  "MALAYALAM AU LENGTH MARK\0"
  "MALAYALAM DATE MARK\0"
  "MALAYALAM DIGIT EIGHT\0"
  "MALAYALAM DIGIT FIVE\0"
  "MALAYALAM DIGIT FOUR\0"
  "MALAYALAM DIGIT NINE\0"
  "MALAYALAM DIGIT ONE\0"
  "MALAYALAM DIGIT SEVEN\0"
  "MALAYALAM DIGIT SIX\0"
  "MALAYALAM DIGIT THREE\0"
  "MALAYALAM DIGIT TWO\0"
  "MALAYALAM DIGIT ZERO\0"
  "MALAYALAM FRACTION ONE EIGHTH\0"
  "MALAYALAM FRACTION ONE FIFTH\0"
  "MALAYALAM FRACTION ONE FORTIETH\0"
  "MALAYALAM FRACTION ONE HALF\0"
  "MALAYALAM FRACTION ONE ONE-HUNDRED-AND-SIXTIETH\0"
  "MALAYALAM FRACTION ONE QUARTER\0"
  "MALAYALAM FRACTION ONE SIXTEENTH\0"
  "MALAYALAM FRACTION ONE TENTH\0"
  "MALAYALAM FRACTION ONE TWENTIETH\0"
  "MALAYALAM FRACTION THREE EIGHTIETHS\0"
  "MALAYALAM FRACTION THREE QUARTERS\0"
  "MALAYALAM FRACTION THREE SIXTEENTHS\0"
  "MALAYALAM FRACTION THREE TWENTIETHS\0"
  "MALAYALAM LETTER A\0"
  "MALAYALAM LETTER AA\0"
  "MALAYALAM LETTER AI\0"
  "MALAYALAM LETTER ARCHAIC II\0"
  "MALAYALAM LETTER AU\0"
  "MALAYALAM LETTER BA\0"
  "MALAYALAM LETTER BHA\0"
  "MALAYALAM LETTER CA\0"
  "MALAYALAM LETTER CHA\0"
  "MALAYALAM LETTER CHILLU K\0"
  "MALAYALAM LETTER CHILLU L\0"
  "MALAYALAM LETTER CHILLU LL\0"
  "MALAYALAM LETTER CHILLU LLL\0"
  "MALAYALAM LETTER CHILLU M\0"
  "MALAYALAM LETTER CHILLU N\0"
  "MALAYALAM LETTER CHILLU NN\0"
  "MALAYALAM LETTER CHILLU RR\0"
  "MALAYALAM LETTER CHILLU Y\0"
  "MALAYALAM LETTER DA\0"
  "MALAYALAM LETTER DDA\0"
  "MALAYALAM LETTER DDHA\0"
  "MALAYALAM LETTER DHA\0"
  "MALAYALAM LETTER DOT REPH\0"
  "MALAYALAM LETTER E\0"
  "MALAYALAM LETTER EE\0"
  "MALAYALAM LETTER GA\0"
  "MALAYALAM LETTER GHA\0"
  "MALAYALAM LETTER HA\0"
  "MALAYALAM LETTER I\0"
  "MALAYALAM LETTER II\0"
  "MALAYALAM LETTER JA\0"
  "MALAYALAM LETTER JHA\0"
  "MALAYALAM LETTER KA\0"
  "MALAYALAM LETTER KHA\0"
  "MALAYALAM LETTER LA\0"
  "MALAYALAM LETTER LLA\0"
  "MALAYALAM LETTER LLLA\0"
  "MALAYALAM LETTER MA\0"
  "MALAYALAM LETTER NA\0"
  "MALAYALAM LETTER NGA\0"
  "MALAYALAM LETTER NNA\0"
  "MALAYALAM LETTER NNNA\0"
  "MALAYALAM LETTER NYA\0"
  "MALAYALAM LETTER O\0"
  "MALAYALAM LETTER OO\0"
  "MALAYALAM LETTER PA\0"
  "MALAYALAM LETTER PHA\0"
  "MALAYALAM LETTER RA\0"
  "MALAYALAM LETTER RRA\0"
  "MALAYALAM LETTER SA\0"
  "MALAYALAM LETTER SHA\0"
  "MALAYALAM LETTER SSA\0"
  "MALAYALAM LETTER TA\0"
  "MALAYALAM LETTER THA\0"
  "MALAYALAM LETTER TTA\0"
  "MALAYALAM LETTER TTHA\0"
  "MALAYALAM LETTER TTTA\0"
  "MALAYALAM LETTER U\0"
  "MALAYALAM LETTER UU\0"
  "MALAYALAM LETTER VA\0"
  "MALAYALAM LETTER VEDIC ANUSVARA\0"
  "MALAYALAM LETTER VOCALIC L\0"
  "MALAYALAM LETTER VOCALIC LL\0"
  "MALAYALAM LETTER VOCALIC R\0"
  "MALAYALAM LETTER VOCALIC RR\0"
  "MALAYALAM LETTER YA\0"
  "MALAYALAM NUMBER ONE HUNDRED\0"
  "MALAYALAM NUMBER ONE THOUSAND\0"
  "MALAYALAM NUMBER TEN\0"
  "MALAYALAM SIGN ANUSVARA\0"
  "MALAYALAM SIGN AVAGRAHA\0"
  "MALAYALAM SIGN CANDRABINDU\0"
  "MALAYALAM SIGN CIRCULAR VIRAMA\0"
  "MALAYALAM SIGN COMBINING ANUSVARA ABOVE\0"
  "MALAYALAM SIGN PARA\0"
  "MALAYALAM SIGN VERTICAL BAR VIRAMA\0"
  "MALAYALAM SIGN VIRAMA\0"
  "MALAYALAM SIGN VISARGA\0"
  "MALAYALAM VOWEL SIGN AA\0"
  "MALAYALAM VOWEL SIGN AI\0"
  "MALAYALAM VOWEL SIGN AU\0"
  "MALAYALAM VOWEL SIGN E\0"
  "MALAYALAM VOWEL SIGN EE\0"
  "MALAYALAM VOWEL SIGN I\0"
  "MALAYALAM VOWEL SIGN II\0"
  "MALAYALAM VOWEL SIGN O\0"
  "MALAYALAM VOWEL SIGN OO\0"
  "MALAYALAM VOWEL SIGN U\0"
  "MALAYALAM VOWEL SIGN UU\0"
  "MALAYALAM VOWEL SIGN VOCALIC L\0"
  "MALAYALAM VOWEL SIGN VOCALIC LL\0"
  "MALAYALAM VOWEL SIGN VOCALIC R\0"
  "MALAYALAM VOWEL SIGN VOCALIC RR\0"
  "MALE AND FEMALE SIGN\0"
  "MALE SIGN\0"
  "MALE WITH STROKE AND MALE AND FEMALE SIGN\0"
  "MALE WITH STROKE SIGN\0"
  "MALTESE CROSS\0"
  "MAMMOTH\0"
  "MAN\0"
  "MAN AND WOMAN HOLDING HANDS\0"
  "MAN DANCING\0"
  "MAN IN BUSINESS SUIT LEVITATING\0"
  "MAN IN TUXEDO\0"
  "MAN WITH GUA PI MAO\0"
  "MAN WITH TURBAN\0"
  "MANAT SIGN\0"
  "MANDAIC AFFRICATION MARK\0"
  "MANDAIC GEMINATION MARK\0"
  "MANDAIC LETTER AB\0"
  "MANDAIC LETTER AD\0"
  "MANDAIC LETTER AG\0"
  "MANDAIC LETTER AH\0"
  "MANDAIC LETTER AIN\0"
  "MANDAIC LETTER AK\0"
  "MANDAIC LETTER AKSA\0"
  "MANDAIC LETTER AL\0"
  "MANDAIC LETTER AM\0"
  "MANDAIC LETTER AN\0"
  "MANDAIC LETTER AP\0"
  "MANDAIC LETTER AQ\0"
  "MANDAIC LETTER AR\0"
  "MANDAIC LETTER AS\0"
  "MANDAIC LETTER ASH\0"
  "MANDAIC LETTER ASZ\0"
  "MANDAIC LETTER AT\0"
  "MANDAIC LETTER ATT\0"
  "MANDAIC LETTER AZ\0"
  "MANDAIC LETTER DUSHENNA\0"
  "MANDAIC LETTER HALQA\0"
  "MANDAIC LETTER IN\0"
  "MANDAIC LETTER IT\0"
  "MANDAIC LETTER KAD\0"
  "MANDAIC LETTER USHENNA\0"
  "MANDAIC PUNCTUATION\0"
  "MANDAIC VOCALIZATION MARK\0"
  "MANGO\0"
  "MANICHAEAN ABBREVIATION MARK ABOVE\0"
  "MANICHAEAN ABBREVIATION MARK BELOW\0"
  "MANICHAEAN LETTER AAYIN\0"
  "MANICHAEAN LETTER ALEPH\0"
  "MANICHAEAN LETTER AYIN\0"
  "MANICHAEAN LETTER BETH\0"
  "MANICHAEAN LETTER BHETH\0"
  "MANICHAEAN LETTER DALETH\0"
  "MANICHAEAN LETTER DHAMEDH\0"
  "MANICHAEAN LETTER FE\0"
  "MANICHAEAN LETTER GHIMEL\0"
  "MANICHAEAN LETTER GIMEL\0"
  "MANICHAEAN LETTER HE\0"
  "MANICHAEAN LETTER HETH\0"
  "MANICHAEAN LETTER JAYIN\0"
  "MANICHAEAN LETTER JHAYIN\0"
  "MANICHAEAN LETTER KAPH\0"
  "MANICHAEAN LETTER KHAPH\0"
  "MANICHAEAN LETTER LAMEDH\0"
  "MANICHAEAN LETTER MEM\0"
  "MANICHAEAN LETTER NUN\0"
  "MANICHAEAN LETTER PE\0"
  "MANICHAEAN LETTER QHOPH\0"
  "MANICHAEAN LETTER QOPH\0"
  "MANICHAEAN LETTER RESH\0"
  "MANICHAEAN LETTER SADHE\0"
  "MANICHAEAN LETTER SAMEKH\0"
  "MANICHAEAN LETTER SHIN\0"
  "MANICHAEAN LETTER SSHIN\0"
  "MANICHAEAN LETTER TAW\0"
  "MANICHAEAN LETTER TETH\0"
  "MANICHAEAN LETTER THAMEDH\0"
  "MANICHAEAN LETTER WAW\0"
  "MANICHAEAN LETTER XAPH\0"
  "MANICHAEAN LETTER XOPH\0"
  "MANICHAEAN LETTER YODH\0"
  "MANICHAEAN LETTER ZAYIN\0"
  "MANICHAEAN LETTER ZHAYIN\0"
  "MANICHAEAN NUMBER FIVE\0"
  "MANICHAEAN NUMBER ONE\0"
  "MANICHAEAN NUMBER ONE HUNDRED\0"
  "MANICHAEAN NUMBER TEN\0"
  "MANICHAEAN NUMBER TWENTY\0"
  "MANICHAEAN PUNCTUATION DOT\0"
  "MANICHAEAN PUNCTUATION DOT WITHIN DOT\0"
  "MANICHAEAN PUNCTUATION DOUBLE DOT WITHIN DOT\0"
  "MANICHAEAN PUNCTUATION FLEURON\0"
  "MANICHAEAN PUNCTUATION LINE FILLER\0"
  "MANICHAEAN PUNCTUATION STAR\0"
  "MANICHAEAN PUNCTUATION TWO DOTS\0"
  "MANICHAEAN SIGN UD\0"
  "MANS SHOE\0"
  "MANTELPIECE CLOCK\0"
  "MANUAL WHEELCHAIR\0"
  "MAP SYMBOL FOR LIGHTHOUSE\0"
  "MAPLE LEAF\0"
  "MARACAS\0"
  "MARCHEN HEAD MARK\0"
  "MARCHEN LETTER -A\0"
  "MARCHEN LETTER A\0"
  "MARCHEN LETTER BA\0"
  "MARCHEN LETTER CA\0"
  "MARCHEN LETTER CHA\0"
  "MARCHEN LETTER DA\0"
  "MARCHEN LETTER DZA\0"
  "MARCHEN LETTER GA\0"
  "MARCHEN LETTER HA\0"
  "MARCHEN LETTER JA\0"
  "MARCHEN LETTER KA\0"
  "MARCHEN LETTER KHA\0"
  "MARCHEN LETTER LA\0"
  "MARCHEN LETTER MA\0"
  "MARCHEN LETTER NA\0"
  "MARCHEN LETTER NGA\0"
  "MARCHEN LETTER NYA\0"
  "MARCHEN LETTER PA\0"
  "MARCHEN LETTER PHA\0"
  "MARCHEN LETTER RA\0"
  "MARCHEN LETTER SA\0"
  "MARCHEN LETTER SHA\0"
  "MARCHEN LETTER TA\0"
  "MARCHEN LETTER THA\0"
  "MARCHEN LETTER TSA\0"
  "MARCHEN LETTER TSHA\0"
  "MARCHEN LETTER WA\0"
  "MARCHEN LETTER YA\0"
  "MARCHEN LETTER ZA\0"
  "MARCHEN LETTER ZHA\0"
  "MARCHEN MARK SHAD\0"
  "MARCHEN SIGN ANUSVARA\0"
  "MARCHEN SIGN CANDRABINDU\0"
  "MARCHEN SUBJOINED LETTER A\0"
  "MARCHEN SUBJOINED LETTER BA\0"
  "MARCHEN SUBJOINED LETTER CA\0"
  "MARCHEN SUBJOINED LETTER CHA\0"
  "MARCHEN SUBJOINED LETTER DA\0"
  "MARCHEN SUBJOINED LETTER DZA\0"
  "MARCHEN SUBJOINED LETTER GA\0"
  "MARCHEN SUBJOINED LETTER HA\0"
  "MARCHEN SUBJOINED LETTER JA\0"
  "MARCHEN SUBJOINED LETTER KA\0"
  "MARCHEN SUBJOINED LETTER KHA\0"
  "MARCHEN SUBJOINED LETTER LA\0"
  "MARCHEN SUBJOINED LETTER MA\0"
  "MARCHEN SUBJOINED LETTER NA\0"
  "MARCHEN SUBJOINED LETTER NGA\0"
  "MARCHEN SUBJOINED LETTER NYA\0"
  "MARCHEN SUBJOINED LETTER PA\0"
  "MARCHEN SUBJOINED LETTER PHA\0"
  "MARCHEN SUBJOINED LETTER RA\0"
  "MARCHEN SUBJOINED LETTER SA\0"
  "MARCHEN SUBJOINED LETTER SHA\0"
  "MARCHEN SUBJOINED LETTER TA\0"
  "MARCHEN SUBJOINED LETTER THA\0"
  "MARCHEN SUBJOINED LETTER TSA\0"
  "MARCHEN SUBJOINED LETTER TSHA\0"
  "MARCHEN SUBJOINED LETTER WA\0"
  "MARCHEN SUBJOINED LETTER YA\0"
  "MARCHEN SUBJOINED LETTER ZA\0"
  "MARCHEN SUBJOINED LETTER ZHA\0"
  "MARCHEN VOWEL SIGN AA\0"
  "MARCHEN VOWEL SIGN E\0"
  "MARCHEN VOWEL SIGN I\0"
  "MARCHEN VOWEL SIGN O\0"
  "MARCHEN VOWEL SIGN U\0"
  "MARRIAGE SYMBOL\0"
  "MARTIAL ARTS UNIFORM\0"
  "MASARAM GONDI DIGIT EIGHT\0"
  "MASARAM GONDI DIGIT FIVE\0"
  "MASARAM GONDI DIGIT FOUR\0"
  "MASARAM GONDI DIGIT NINE\0"
  "MASARAM GONDI DIGIT ONE\0"
  "MASARAM GONDI DIGIT SEVEN\0"
  "MASARAM GONDI DIGIT SIX\0"
  "MASARAM GONDI DIGIT THREE\0"
  "MASARAM GONDI DIGIT TWO\0"
  "MASARAM GONDI DIGIT ZERO\0"
  "MASARAM GONDI LETTER A\0"
  "MASARAM GONDI LETTER AA\0"
  "MASARAM GONDI LETTER AI\0"
  "MASARAM GONDI LETTER AU\0"
  "MASARAM GONDI LETTER BA\0"
  "MASARAM GONDI LETTER BHA\0"
  "MASARAM GONDI LETTER CA\0"
  "MASARAM GONDI LETTER CHA\0"
  "MASARAM GONDI LETTER DA\0"
  "MASARAM GONDI LETTER DDA\0"
  "MASARAM GONDI LETTER DDHA\0"
  "MASARAM GONDI LETTER DHA\0"
  "MASARAM GONDI LETTER E\0"
  "MASARAM GONDI LETTER GA\0"
  "MASARAM GONDI LETTER GHA\0"
  "MASARAM GONDI LETTER HA\0"
  "MASARAM GONDI LETTER I\0"
  "MASARAM GONDI LETTER II\0"
  "MASARAM GONDI LETTER JA\0"
  "MASARAM GONDI LETTER JHA\0"
  "MASARAM GONDI LETTER JNYA\0"
  "MASARAM GONDI LETTER KA\0"
  "MASARAM GONDI LETTER KHA\0"
  "MASARAM GONDI LETTER KSSA\0"
  "MASARAM GONDI LETTER LA\0"
  "MASARAM GONDI LETTER LLA\0"
  "MASARAM GONDI LETTER MA\0"
  "MASARAM GONDI LETTER NA\0"
  "MASARAM GONDI LETTER NGA\0"
  "MASARAM GONDI LETTER NNA\0"
  "MASARAM GONDI LETTER NYA\0"
  "MASARAM GONDI LETTER O\0"
  "MASARAM GONDI LETTER PA\0"
  "MASARAM GONDI LETTER PHA\0"
  "MASARAM GONDI LETTER RA\0"
  "MASARAM GONDI LETTER SA\0"
  "MASARAM GONDI LETTER SHA\0"
  "MASARAM GONDI LETTER SSA\0"
  "MASARAM GONDI LETTER TA\0"
  "MASARAM GONDI LETTER THA\0"
  "MASARAM GONDI LETTER TRA\0"
  "MASARAM GONDI LETTER TTA\0"
  "MASARAM GONDI LETTER TTHA\0"
  "MASARAM GONDI LETTER U\0"
  "MASARAM GONDI LETTER UU\0"
  "MASARAM GONDI LETTER VA\0"
  "MASARAM GONDI LETTER YA\0"
  "MASARAM GONDI RA-KARA\0"
  "MASARAM GONDI REPHA\0"
  "MASARAM GONDI SIGN ANUSVARA\0"
  "MASARAM GONDI SIGN CANDRA\0"
  "MASARAM GONDI SIGN HALANTA\0"
  "MASARAM GONDI SIGN NUKTA\0"
  "MASARAM GONDI SIGN VISARGA\0"
  "MASARAM GONDI VIRAMA\0"
  "MASARAM GONDI VOWEL SIGN AA\0"
  "MASARAM GONDI VOWEL SIGN AI\0"
  "MASARAM GONDI VOWEL SIGN AU\0"
  "MASARAM GONDI VOWEL SIGN E\0"
  "MASARAM GONDI VOWEL SIGN I\0"
  "MASARAM GONDI VOWEL SIGN II\0"
  "MASARAM GONDI VOWEL SIGN O\0"
  "MASARAM GONDI VOWEL SIGN U\0"
  "MASARAM GONDI VOWEL SIGN UU\0"
  "MASARAM GONDI VOWEL SIGN VOCALIC R\0"
  "MASCULINE ORDINAL INDICATOR\0"
  "MASK WORK SYMBOL\0"
  "MASU MARK\0"
  "MATE DRINK\0"
  "MATHEMATICAL BOLD CAPITAL A\0"
  "MATHEMATICAL BOLD CAPITAL ALPHA\0"
  "MATHEMATICAL BOLD CAPITAL B\0"
  "MATHEMATICAL BOLD CAPITAL BETA\0"
  "MATHEMATICAL BOLD CAPITAL C\0"
  "MATHEMATICAL BOLD CAPITAL CHI\0"
  "MATHEMATICAL BOLD CAPITAL D\0"
  "MATHEMATICAL BOLD CAPITAL DELTA\0"
  "MATHEMATICAL BOLD CAPITAL DIGAMMA\0"
  "MATHEMATICAL BOLD CAPITAL E\0"
  "MATHEMATICAL BOLD CAPITAL EPSILON\0"
  "MATHEMATICAL BOLD CAPITAL ETA\0"
  "MATHEMATICAL BOLD CAPITAL F\0"
  "MATHEMATICAL BOLD CAPITAL G\0"
  "MATHEMATICAL BOLD CAPITAL GAMMA\0"
  "MATHEMATICAL BOLD CAPITAL H\0"
  "MATHEMATICAL BOLD CAPITAL I\0"
  "MATHEMATICAL BOLD CAPITAL IOTA\0"
  "MATHEMATICAL BOLD CAPITAL J\0"
  "MATHEMATICAL BOLD CAPITAL K\0"
  "MATHEMATICAL BOLD CAPITAL KAPPA\0"
  "MATHEMATICAL BOLD CAPITAL L\0"
  "MATHEMATICAL BOLD CAPITAL LAMDA\0"
  "MATHEMATICAL BOLD CAPITAL M\0"
  "MATHEMATICAL BOLD CAPITAL MU\0"
  "MATHEMATICAL BOLD CAPITAL N\0"
  "MATHEMATICAL BOLD CAPITAL NU\0"
  "MATHEMATICAL BOLD CAPITAL O\0"
  "MATHEMATICAL BOLD CAPITAL OMEGA\0"
  "MATHEMATICAL BOLD CAPITAL OMICRON\0"
  "MATHEMATICAL BOLD CAPITAL P\0"
  "MATHEMATICAL BOLD CAPITAL PHI\0"
  "MATHEMATICAL BOLD CAPITAL PI\0"
  "MATHEMATICAL BOLD CAPITAL PSI\0"
  "MATHEMATICAL BOLD CAPITAL Q\0"
  "MATHEMATICAL BOLD CAPITAL R\0"
  "MATHEMATICAL BOLD CAPITAL RHO\0"
  "MATHEMATICAL BOLD CAPITAL S\0"
  "MATHEMATICAL BOLD CAPITAL SIGMA\0"
  "MATHEMATICAL BOLD CAPITAL T\0"
  "MATHEMATICAL BOLD CAPITAL TAU\0"
  "MATHEMATICAL BOLD CAPITAL THETA\0"
  "MATHEMATICAL BOLD CAPITAL THETA SYMBOL\0"
  "MATHEMATICAL BOLD CAPITAL U\0"
  "MATHEMATICAL BOLD CAPITAL UPSILON\0"
  "MATHEMATICAL BOLD CAPITAL V\0"
  "MATHEMATICAL BOLD CAPITAL W\0"
  "MATHEMATICAL BOLD CAPITAL X\0"
  "MATHEMATICAL BOLD CAPITAL XI\0"
  "MATHEMATICAL BOLD CAPITAL Y\0"
  "MATHEMATICAL BOLD CAPITAL Z\0"
  "MATHEMATICAL BOLD CAPITAL ZETA\0"
  "MATHEMATICAL BOLD DIGIT EIGHT\0"
  "MATHEMATICAL BOLD DIGIT FIVE\0"
  "MATHEMATICAL BOLD DIGIT FOUR\0"
  "MATHEMATICAL BOLD DIGIT NINE\0"
  "MATHEMATICAL BOLD DIGIT ONE\0"
  "MATHEMATICAL BOLD DIGIT SEVEN\0"
  "MATHEMATICAL BOLD DIGIT SIX\0"
  "MATHEMATICAL BOLD DIGIT THREE\0"
  "MATHEMATICAL BOLD DIGIT TWO\0"
  "MATHEMATICAL BOLD DIGIT ZERO\0"
  "MATHEMATICAL BOLD EPSILON SYMBOL\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL A\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL B\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL C\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL D\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL E\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL F\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL G\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL H\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL I\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL J\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL K\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL L\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL M\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL N\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL O\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL P\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL Q\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL R\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL S\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL T\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL U\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL V\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL W\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL X\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL Y\0"
  "MATHEMATICAL BOLD FRAKTUR CAPITAL Z\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL A\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL B\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL C\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL D\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL E\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL F\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL G\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL H\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL I\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL J\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL K\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL L\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL M\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL N\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL O\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL P\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL Q\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL R\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL S\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL T\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL U\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL V\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL W\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL X\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL Y\0"
  "MATHEMATICAL BOLD FRAKTUR SMALL Z\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL A\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL ALPHA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL B\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL BETA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL C\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL CHI\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL D\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL DELTA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL E\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL EPSILON\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL ETA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL F\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL G\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL GAMMA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL H\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL I\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL IOTA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL J\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL K\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL KAPPA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL L\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL LAMDA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL M\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL MU\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL N\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL NU\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL O\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL OMEGA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL OMICRON\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL P\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL PHI\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL PI\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL PSI\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL Q\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL R\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL RHO\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL S\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL SIGMA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL T\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL TAU\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL THETA\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL THETA SYMBOL\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL U\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL UPSILON\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL V\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL W\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL X\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL XI\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL Y\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL Z\0"
  "MATHEMATICAL BOLD ITALIC CAPITAL ZETA\0"
  "MATHEMATICAL BOLD ITALIC EPSILON SYMBOL\0"
  "MATHEMATICAL BOLD ITALIC KAPPA SYMBOL\0"
  "MATHEMATICAL BOLD ITALIC NABLA\0"
  "MATHEMATICAL BOLD ITALIC PARTIAL DIFFERENTIAL\0"
  "MATHEMATICAL BOLD ITALIC PHI SYMBOL\0"
  "MATHEMATICAL BOLD ITALIC PI SYMBOL\0"
  "MATHEMATICAL BOLD ITALIC RHO SYMBOL\0"
  "MATHEMATICAL BOLD ITALIC SMALL A\0"
  "MATHEMATICAL BOLD ITALIC SMALL ALPHA\0"
  "MATHEMATICAL BOLD ITALIC SMALL B\0"
  "MATHEMATICAL BOLD ITALIC SMALL BETA\0"
  "MATHEMATICAL BOLD ITALIC SMALL C\0"
  "MATHEMATICAL BOLD ITALIC SMALL CHI\0"
  "MATHEMATICAL BOLD ITALIC SMALL D\0"
  "MATHEMATICAL BOLD ITALIC SMALL DELTA\0"
  "MATHEMATICAL BOLD ITALIC SMALL E\0"
  "MATHEMATICAL BOLD ITALIC SMALL EPSILON\0"
  "MATHEMATICAL BOLD ITALIC SMALL ETA\0"
  "MATHEMATICAL BOLD ITALIC SMALL F\0"
  "MATHEMATICAL BOLD ITALIC SMALL FINAL SIGMA\0"
  "MATHEMATICAL BOLD ITALIC SMALL G\0"
  "MATHEMATICAL BOLD ITALIC SMALL GAMMA\0"
  "MATHEMATICAL BOLD ITALIC SMALL H\0"
  "MATHEMATICAL BOLD ITALIC SMALL I\0"
  "MATHEMATICAL BOLD ITALIC SMALL IOTA\0"
  "MATHEMATICAL BOLD ITALIC SMALL J\0"
  "MATHEMATICAL BOLD ITALIC SMALL K\0"
  "MATHEMATICAL BOLD ITALIC SMALL KAPPA\0"
  "MATHEMATICAL BOLD ITALIC SMALL L\0"
  "MATHEMATICAL BOLD ITALIC SMALL LAMDA\0"
  "MATHEMATICAL BOLD ITALIC SMALL M\0"
  "MATHEMATICAL BOLD ITALIC SMALL MU\0"
  "MATHEMATICAL BOLD ITALIC SMALL N\0"
  "MATHEMATICAL BOLD ITALIC SMALL NU\0"
  "MATHEMATICAL BOLD ITALIC SMALL O\0"
  "MATHEMATICAL BOLD ITALIC SMALL OMEGA\0"
  "MATHEMATICAL BOLD ITALIC SMALL OMICRON\0"
  "MATHEMATICAL BOLD ITALIC SMALL P\0"
  "MATHEMATICAL BOLD ITALIC SMALL PHI\0"
  "MATHEMATICAL BOLD ITALIC SMALL PI\0"
  "MATHEMATICAL BOLD ITALIC SMALL PSI\0"
  "MATHEMATICAL BOLD ITALIC SMALL Q\0"
  "MATHEMATICAL BOLD ITALIC SMALL R\0"
  "MATHEMATICAL BOLD ITALIC SMALL RHO\0"
  "MATHEMATICAL BOLD ITALIC SMALL S\0"
  "MATHEMATICAL BOLD ITALIC SMALL SIGMA\0"
  "MATHEMATICAL BOLD ITALIC SMALL T\0"
  "MATHEMATICAL BOLD ITALIC SMALL TAU\0"
  "MATHEMATICAL BOLD ITALIC SMALL THETA\0"
  "MATHEMATICAL BOLD ITALIC SMALL U\0"
  "MATHEMATICAL BOLD ITALIC SMALL UPSILON\0"
  "MATHEMATICAL BOLD ITALIC SMALL V\0"
  "MATHEMATICAL BOLD ITALIC SMALL W\0"
  "MATHEMATICAL BOLD ITALIC SMALL X\0"
  "MATHEMATICAL BOLD ITALIC SMALL XI\0"
  "MATHEMATICAL BOLD ITALIC SMALL Y\0"
  "MATHEMATICAL BOLD ITALIC SMALL Z\0"
  "MATHEMATICAL BOLD ITALIC SMALL ZETA\0"
  "MATHEMATICAL BOLD ITALIC THETA SYMBOL\0"
  "MATHEMATICAL BOLD KAPPA SYMBOL\0"
  "MATHEMATICAL BOLD NABLA\0"
  "MATHEMATICAL BOLD PARTIAL DIFFERENTIAL\0"
  "MATHEMATICAL BOLD PHI SYMBOL\0"
  "MATHEMATICAL BOLD PI SYMBOL\0"
  "MATHEMATICAL BOLD RHO SYMBOL\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL A\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL B\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL C\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL D\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL E\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL F\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL G\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL H\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL I\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL J\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL K\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL L\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL M\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL N\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL O\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL P\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL Q\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL R\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL S\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL T\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL U\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL V\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL W\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL X\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL Y\0"
  "MATHEMATICAL BOLD SCRIPT CAPITAL Z\0"
  "MATHEMATICAL BOLD SCRIPT SMALL A\0"
  "MATHEMATICAL BOLD SCRIPT SMALL B\0"
  "MATHEMATICAL BOLD SCRIPT SMALL C\0"
  "MATHEMATICAL BOLD SCRIPT SMALL D\0"
  "MATHEMATICAL BOLD SCRIPT SMALL E\0"
  "MATHEMATICAL BOLD SCRIPT SMALL F\0"
  "MATHEMATICAL BOLD SCRIPT SMALL G\0"
  "MATHEMATICAL BOLD SCRIPT SMALL H\0"
  "MATHEMATICAL BOLD SCRIPT SMALL I\0"
  "MATHEMATICAL BOLD SCRIPT SMALL J\0"
  "MATHEMATICAL BOLD SCRIPT SMALL K\0"
  "MATHEMATICAL BOLD SCRIPT SMALL L\0"
  "MATHEMATICAL BOLD SCRIPT SMALL M\0"
  "MATHEMATICAL BOLD SCRIPT SMALL N\0"
  "MATHEMATICAL BOLD SCRIPT SMALL O\0"
  "MATHEMATICAL BOLD SCRIPT SMALL P\0"
  "MATHEMATICAL BOLD SCRIPT SMALL Q\0"
  "MATHEMATICAL BOLD SCRIPT SMALL R\0"
  "MATHEMATICAL BOLD SCRIPT SMALL S\0"
  "MATHEMATICAL BOLD SCRIPT SMALL T\0"
  "MATHEMATICAL BOLD SCRIPT SMALL U\0"
  "MATHEMATICAL BOLD SCRIPT SMALL V\0"
  "MATHEMATICAL BOLD SCRIPT SMALL W\0"
  "MATHEMATICAL BOLD SCRIPT SMALL X\0"
  "MATHEMATICAL BOLD SCRIPT SMALL Y\0"
  "MATHEMATICAL BOLD SCRIPT SMALL Z\0"
  "MATHEMATICAL BOLD SMALL A\0"
  "MATHEMATICAL BOLD SMALL ALPHA\0"
  "MATHEMATICAL BOLD SMALL B\0"
  "MATHEMATICAL BOLD SMALL BETA\0"
  "MATHEMATICAL BOLD SMALL C\0"
  "MATHEMATICAL BOLD SMALL CHI\0"
  "MATHEMATICAL BOLD SMALL D\0"
  "MATHEMATICAL BOLD SMALL DELTA\0"
  "MATHEMATICAL BOLD SMALL DIGAMMA\0"
  "MATHEMATICAL BOLD SMALL E\0"
  "MATHEMATICAL BOLD SMALL EPSILON\0"
  "MATHEMATICAL BOLD SMALL ETA\0"
  "MATHEMATICAL BOLD SMALL F\0"
  "MATHEMATICAL BOLD SMALL FINAL SIGMA\0"
  "MATHEMATICAL BOLD SMALL G\0"
  "MATHEMATICAL BOLD SMALL GAMMA\0"
  "MATHEMATICAL BOLD SMALL H\0"
  "MATHEMATICAL BOLD SMALL I\0"
  "MATHEMATICAL BOLD SMALL IOTA\0"
  "MATHEMATICAL BOLD SMALL J\0"
  "MATHEMATICAL BOLD SMALL K\0"
  "MATHEMATICAL BOLD SMALL KAPPA\0"
  "MATHEMATICAL BOLD SMALL L\0"
  "MATHEMATICAL BOLD SMALL LAMDA\0"
  "MATHEMATICAL BOLD SMALL M\0"
  "MATHEMATICAL BOLD SMALL MU\0"
  "MATHEMATICAL BOLD SMALL N\0"
  "MATHEMATICAL BOLD SMALL NU\0"
  "MATHEMATICAL BOLD SMALL O\0"
  "MATHEMATICAL BOLD SMALL OMEGA\0"
  "MATHEMATICAL BOLD SMALL OMICRON\0"
  "MATHEMATICAL BOLD SMALL P\0"
  "MATHEMATICAL BOLD SMALL PHI\0"
  "MATHEMATICAL BOLD SMALL PI\0"
  "MATHEMATICAL BOLD SMALL PSI\0"
  "MATHEMATICAL BOLD SMALL Q\0"
  "MATHEMATICAL BOLD SMALL R\0"
  "MATHEMATICAL BOLD SMALL RHO\0"
  "MATHEMATICAL BOLD SMALL S\0"
  "MATHEMATICAL BOLD SMALL SIGMA\0"
  "MATHEMATICAL BOLD SMALL T\0"
  "MATHEMATICAL BOLD SMALL TAU\0"
  "MATHEMATICAL BOLD SMALL THETA\0"
  "MATHEMATICAL BOLD SMALL U\0"
  "MATHEMATICAL BOLD SMALL UPSILON\0"
  "MATHEMATICAL BOLD SMALL V\0"
  "MATHEMATICAL BOLD SMALL W\0"
  "MATHEMATICAL BOLD SMALL X\0"
  "MATHEMATICAL BOLD SMALL XI\0"
  "MATHEMATICAL BOLD SMALL Y\0"
  "MATHEMATICAL BOLD SMALL Z\0"
  "MATHEMATICAL BOLD SMALL ZETA\0"
  "MATHEMATICAL BOLD THETA SYMBOL\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL A\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL B\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL D\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL E\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL F\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL G\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL I\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL J\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL K\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL L\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL M\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL O\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL S\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL T\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL U\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL V\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL W\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL X\0"
  "MATHEMATICAL DOUBLE-STRUCK CAPITAL Y\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT EIGHT\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT FIVE\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT FOUR\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT NINE\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT ONE\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT SEVEN\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT SIX\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT THREE\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT TWO\0"
  "MATHEMATICAL DOUBLE-STRUCK DIGIT ZERO\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL A\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL B\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL C\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL D\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL E\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL F\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL G\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL H\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL I\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL J\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL K\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL L\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL M\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL N\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL O\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL P\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL Q\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL R\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL S\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL T\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL U\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL V\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL W\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL X\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL Y\0"
  "MATHEMATICAL DOUBLE-STRUCK SMALL Z\0"
  "MATHEMATICAL FALLING DIAGONAL\0"
  "MATHEMATICAL FRAKTUR CAPITAL A\0"
  "MATHEMATICAL FRAKTUR CAPITAL B\0"
  "MATHEMATICAL FRAKTUR CAPITAL D\0"
  "MATHEMATICAL FRAKTUR CAPITAL E\0"
  "MATHEMATICAL FRAKTUR CAPITAL F\0"
  "MATHEMATICAL FRAKTUR CAPITAL G\0"
  "MATHEMATICAL FRAKTUR CAPITAL J\0"
  "MATHEMATICAL FRAKTUR CAPITAL K\0"
  "MATHEMATICAL FRAKTUR CAPITAL L\0"
  "MATHEMATICAL FRAKTUR CAPITAL M\0"
  "MATHEMATICAL FRAKTUR CAPITAL N\0"
  "MATHEMATICAL FRAKTUR CAPITAL O\0"
  "MATHEMATICAL FRAKTUR CAPITAL P\0"
  "MATHEMATICAL FRAKTUR CAPITAL Q\0"
  "MATHEMATICAL FRAKTUR CAPITAL S\0"
  "MATHEMATICAL FRAKTUR CAPITAL T\0"
  "MATHEMATICAL FRAKTUR CAPITAL U\0"
  "MATHEMATICAL FRAKTUR CAPITAL V\0"
  "MATHEMATICAL FRAKTUR CAPITAL W\0"
  "MATHEMATICAL FRAKTUR CAPITAL X\0"
  "MATHEMATICAL FRAKTUR CAPITAL Y\0"
  "MATHEMATICAL FRAKTUR SMALL A\0"
  "MATHEMATICAL FRAKTUR SMALL B\0"
  "MATHEMATICAL FRAKTUR SMALL C\0"
  "MATHEMATICAL FRAKTUR SMALL D\0"
  "MATHEMATICAL FRAKTUR SMALL E\0"
  "MATHEMATICAL FRAKTUR SMALL F\0"
  "MATHEMATICAL FRAKTUR SMALL G\0"
  "MATHEMATICAL FRAKTUR SMALL H\0"
  "MATHEMATICAL FRAKTUR SMALL I\0"
  "MATHEMATICAL FRAKTUR SMALL J\0"
  "MATHEMATICAL FRAKTUR SMALL K\0"
  "MATHEMATICAL FRAKTUR SMALL L\0"
  "MATHEMATICAL FRAKTUR SMALL M\0"
  "MATHEMATICAL FRAKTUR SMALL N\0"
  "MATHEMATICAL FRAKTUR SMALL O\0"
  "MATHEMATICAL FRAKTUR SMALL P\0"
  "MATHEMATICAL FRAKTUR SMALL Q\0"
  "MATHEMATICAL FRAKTUR SMALL R\0"
  "MATHEMATICAL FRAKTUR SMALL S\0"
  "MATHEMATICAL FRAKTUR SMALL T\0"
  "MATHEMATICAL FRAKTUR SMALL U\0"
  "MATHEMATICAL FRAKTUR SMALL V\0"
  "MATHEMATICAL FRAKTUR SMALL W\0"
  "MATHEMATICAL FRAKTUR SMALL X\0"
  "MATHEMATICAL FRAKTUR SMALL Y\0"
  "MATHEMATICAL FRAKTUR SMALL Z\0"
  "MATHEMATICAL ITALIC CAPITAL A\0"
  "MATHEMATICAL ITALIC CAPITAL ALPHA\0"
  "MATHEMATICAL ITALIC CAPITAL B\0"
  "MATHEMATICAL ITALIC CAPITAL BETA\0"
  "MATHEMATICAL ITALIC CAPITAL C\0"
  "MATHEMATICAL ITALIC CAPITAL CHI\0"
  "MATHEMATICAL ITALIC CAPITAL D\0"
  "MATHEMATICAL ITALIC CAPITAL DELTA\0"
  "MATHEMATICAL ITALIC CAPITAL E\0"
  "MATHEMATICAL ITALIC CAPITAL EPSILON\0"
  "MATHEMATICAL ITALIC CAPITAL ETA\0"
  "MATHEMATICAL ITALIC CAPITAL F\0"
  "MATHEMATICAL ITALIC CAPITAL G\0"
  "MATHEMATICAL ITALIC CAPITAL GAMMA\0"
  "MATHEMATICAL ITALIC CAPITAL H\0"
  "MATHEMATICAL ITALIC CAPITAL I\0"
  "MATHEMATICAL ITALIC CAPITAL IOTA\0"
  "MATHEMATICAL ITALIC CAPITAL J\0"
  "MATHEMATICAL ITALIC CAPITAL K\0"
  "MATHEMATICAL ITALIC CAPITAL KAPPA\0"
  "MATHEMATICAL ITALIC CAPITAL L\0"
  "MATHEMATICAL ITALIC CAPITAL LAMDA\0"
  "MATHEMATICAL ITALIC CAPITAL M\0"
  "MATHEMATICAL ITALIC CAPITAL MU\0"
  "MATHEMATICAL ITALIC CAPITAL N\0"
  "MATHEMATICAL ITALIC CAPITAL NU\0"
  "MATHEMATICAL ITALIC CAPITAL O\0"
  "MATHEMATICAL ITALIC CAPITAL OMEGA\0"
  "MATHEMATICAL ITALIC CAPITAL OMICRON\0"
  "MATHEMATICAL ITALIC CAPITAL P\0"
  "MATHEMATICAL ITALIC CAPITAL PHI\0"
  "MATHEMATICAL ITALIC CAPITAL PI\0"
  "MATHEMATICAL ITALIC CAPITAL PSI\0"
  "MATHEMATICAL ITALIC CAPITAL Q\0"
  "MATHEMATICAL ITALIC CAPITAL R\0"
  "MATHEMATICAL ITALIC CAPITAL RHO\0"
  "MATHEMATICAL ITALIC CAPITAL S\0"
  "MATHEMATICAL ITALIC CAPITAL SIGMA\0"
  "MATHEMATICAL ITALIC CAPITAL T\0"
  "MATHEMATICAL ITALIC CAPITAL TAU\0"
  "MATHEMATICAL ITALIC CAPITAL THETA\0"
  "MATHEMATICAL ITALIC CAPITAL THETA SYMBOL\0"
  "MATHEMATICAL ITALIC CAPITAL U\0"
  "MATHEMATICAL ITALIC CAPITAL UPSILON\0"
  "MATHEMATICAL ITALIC CAPITAL V\0"
  "MATHEMATICAL ITALIC CAPITAL W\0"
  "MATHEMATICAL ITALIC CAPITAL X\0"
  "MATHEMATICAL ITALIC CAPITAL XI\0"
  "MATHEMATICAL ITALIC CAPITAL Y\0"
  "MATHEMATICAL ITALIC CAPITAL Z\0"
  "MATHEMATICAL ITALIC CAPITAL ZETA\0"
  "MATHEMATICAL ITALIC EPSILON SYMBOL\0"
  "MATHEMATICAL ITALIC KAPPA SYMBOL\0"
  "MATHEMATICAL ITALIC NABLA\0"
  "MATHEMATICAL ITALIC PARTIAL DIFFERENTIAL\0"
  "MATHEMATICAL ITALIC PHI SYMBOL\0"
  "MATHEMATICAL ITALIC PI SYMBOL\0"
  "MATHEMATICAL ITALIC RHO SYMBOL\0"
  "MATHEMATICAL ITALIC SMALL A\0"
  "MATHEMATICAL ITALIC SMALL ALPHA\0"
  "MATHEMATICAL ITALIC SMALL B\0"
  "MATHEMATICAL ITALIC SMALL BETA\0"
  "MATHEMATICAL ITALIC SMALL C\0"
  "MATHEMATICAL ITALIC SMALL CHI\0"
  "MATHEMATICAL ITALIC SMALL D\0"
  "MATHEMATICAL ITALIC SMALL DELTA\0"
  "MATHEMATICAL ITALIC SMALL DOTLESS I\0"
  "MATHEMATICAL ITALIC SMALL DOTLESS J\0"
  "MATHEMATICAL ITALIC SMALL E\0"
  "MATHEMATICAL ITALIC SMALL EPSILON\0"
  "MATHEMATICAL ITALIC SMALL ETA\0"
  "MATHEMATICAL ITALIC SMALL F\0"
  "MATHEMATICAL ITALIC SMALL FINAL SIGMA\0"
  "MATHEMATICAL ITALIC SMALL G\0"
  "MATHEMATICAL ITALIC SMALL GAMMA\0"
  "MATHEMATICAL ITALIC SMALL I\0"
  "MATHEMATICAL ITALIC SMALL IOTA\0"
  "MATHEMATICAL ITALIC SMALL J\0"
  "MATHEMATICAL ITALIC SMALL K\0"
  "MATHEMATICAL ITALIC SMALL KAPPA\0"
  "MATHEMATICAL ITALIC SMALL L\0"
  "MATHEMATICAL ITALIC SMALL LAMDA\0"
  "MATHEMATICAL ITALIC SMALL M\0"
  "MATHEMATICAL ITALIC SMALL MU\0"
  "MATHEMATICAL ITALIC SMALL N\0"
  "MATHEMATICAL ITALIC SMALL NU\0"
  "MATHEMATICAL ITALIC SMALL O\0"
  "MATHEMATICAL ITALIC SMALL OMEGA\0"
  "MATHEMATICAL ITALIC SMALL OMICRON\0"
  "MATHEMATICAL ITALIC SMALL P\0"
  "MATHEMATICAL ITALIC SMALL PHI\0"
  "MATHEMATICAL ITALIC SMALL PI\0"
  "MATHEMATICAL ITALIC SMALL PSI\0"
  "MATHEMATICAL ITALIC SMALL Q\0"
  "MATHEMATICAL ITALIC SMALL R\0"
  "MATHEMATICAL ITALIC SMALL RHO\0"
  "MATHEMATICAL ITALIC SMALL S\0"
  "MATHEMATICAL ITALIC SMALL SIGMA\0"
  "MATHEMATICAL ITALIC SMALL T\0"
  "MATHEMATICAL ITALIC SMALL TAU\0"
  "MATHEMATICAL ITALIC SMALL THETA\0"
  "MATHEMATICAL ITALIC SMALL U\0"
  "MATHEMATICAL ITALIC SMALL UPSILON\0"
  "MATHEMATICAL ITALIC SMALL V\0"
  "MATHEMATICAL ITALIC SMALL W\0"
  "MATHEMATICAL ITALIC SMALL X\0"
  "MATHEMATICAL ITALIC SMALL XI\0"
  "MATHEMATICAL ITALIC SMALL Y\0"
  "MATHEMATICAL ITALIC SMALL Z\0"
  "MATHEMATICAL ITALIC SMALL ZETA\0"
  "MATHEMATICAL ITALIC THETA SYMBOL\0"
  "MATHEMATICAL LEFT ANGLE BRACKET\0"
  "MATHEMATICAL LEFT DOUBLE ANGLE BRACKET\0"
  "MATHEMATICAL LEFT FLATTENED PARENTHESIS\0"
  "MATHEMATICAL LEFT WHITE SQUARE BRACKET\0"
  "MATHEMATICAL LEFT WHITE TORTOISE SHELL BRACKET\0"
  "MATHEMATICAL MONOSPACE CAPITAL A\0"
  "MATHEMATICAL MONOSPACE CAPITAL B\0"
  "MATHEMATICAL MONOSPACE CAPITAL C\0"
  "MATHEMATICAL MONOSPACE CAPITAL D\0"
  "MATHEMATICAL MONOSPACE CAPITAL E\0"
  "MATHEMATICAL MONOSPACE CAPITAL F\0"
  "MATHEMATICAL MONOSPACE CAPITAL G\0"
  "MATHEMATICAL MONOSPACE CAPITAL H\0"
  "MATHEMATICAL MONOSPACE CAPITAL I\0"
  "MATHEMATICAL MONOSPACE CAPITAL J\0"
  "MATHEMATICAL MONOSPACE CAPITAL K\0"
  "MATHEMATICAL MONOSPACE CAPITAL L\0"
  "MATHEMATICAL MONOSPACE CAPITAL M\0"
  "MATHEMATICAL MONOSPACE CAPITAL N\0"
  "MATHEMATICAL MONOSPACE CAPITAL O\0"
  "MATHEMATICAL MONOSPACE CAPITAL P\0"
  "MATHEMATICAL MONOSPACE CAPITAL Q\0"
  "MATHEMATICAL MONOSPACE CAPITAL R\0"
  "MATHEMATICAL MONOSPACE CAPITAL S\0"
  "MATHEMATICAL MONOSPACE CAPITAL T\0"
  "MATHEMATICAL MONOSPACE CAPITAL U\0"
  "MATHEMATICAL MONOSPACE CAPITAL V\0"
  "MATHEMATICAL MONOSPACE CAPITAL W\0"
  "MATHEMATICAL MONOSPACE CAPITAL X\0"
  "MATHEMATICAL MONOSPACE CAPITAL Y\0"
  "MATHEMATICAL MONOSPACE CAPITAL Z\0"
  "MATHEMATICAL MONOSPACE DIGIT EIGHT\0"
  "MATHEMATICAL MONOSPACE DIGIT FIVE\0"
  "MATHEMATICAL MONOSPACE DIGIT FOUR\0"
  "MATHEMATICAL MONOSPACE DIGIT NINE\0"
  "MATHEMATICAL MONOSPACE DIGIT ONE\0"
  "MATHEMATICAL MONOSPACE DIGIT SEVEN\0"
  "MATHEMATICAL MONOSPACE DIGIT SIX\0"
  "MATHEMATICAL MONOSPACE DIGIT THREE\0"
  "MATHEMATICAL MONOSPACE DIGIT TWO\0"
  "MATHEMATICAL MONOSPACE DIGIT ZERO\0"
  "MATHEMATICAL MONOSPACE SMALL A\0"
  "MATHEMATICAL MONOSPACE SMALL B\0"
  "MATHEMATICAL MONOSPACE SMALL C\0"
  "MATHEMATICAL MONOSPACE SMALL D\0"
  "MATHEMATICAL MONOSPACE SMALL E\0"
  "MATHEMATICAL MONOSPACE SMALL F\0"
  "MATHEMATICAL MONOSPACE SMALL G\0"
  "MATHEMATICAL MONOSPACE SMALL H\0"
  "MATHEMATICAL MONOSPACE SMALL I\0"
  "MATHEMATICAL MONOSPACE SMALL J\0"
  "MATHEMATICAL MONOSPACE SMALL K\0"
  "MATHEMATICAL MONOSPACE SMALL L\0"
  "MATHEMATICAL MONOSPACE SMALL M\0"
  "MATHEMATICAL MONOSPACE SMALL N\0"
  "MATHEMATICAL MONOSPACE SMALL O\0"
  "MATHEMATICAL MONOSPACE SMALL P\0"
  "MATHEMATICAL MONOSPACE SMALL Q\0"
  "MATHEMATICAL MONOSPACE SMALL R\0"
  "MATHEMATICAL MONOSPACE SMALL S\0"
  "MATHEMATICAL MONOSPACE SMALL T\0"
  "MATHEMATICAL MONOSPACE SMALL U\0"
  "MATHEMATICAL MONOSPACE SMALL V\0"
  "MATHEMATICAL MONOSPACE SMALL W\0"
  "MATHEMATICAL MONOSPACE SMALL X\0"
  "MATHEMATICAL MONOSPACE SMALL Y\0"
  "MATHEMATICAL MONOSPACE SMALL Z\0"
  "MATHEMATICAL RIGHT ANGLE BRACKET\0"
  "MATHEMATICAL RIGHT DOUBLE ANGLE BRACKET\0"
  "MATHEMATICAL RIGHT FLATTENED PARENTHESIS\0"
  "MATHEMATICAL RIGHT WHITE SQUARE BRACKET\0"
  "MATHEMATICAL RIGHT WHITE TORTOISE SHELL BRACKET\0"
  "MATHEMATICAL RISING DIAGONAL\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL A\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL ALPHA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL B\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL BETA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL C\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL CHI\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL D\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL DELTA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL E\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL EPSILON\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL ETA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL F\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL G\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL GAMMA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL H\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL I\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL IOTA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL J\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL K\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL KAPPA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL L\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL LAMDA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL M\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL MU\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL N\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL NU\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL O\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL OMEGA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL OMICRON\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL P\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL PHI\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL PI\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL PSI\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL Q\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL R\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL RHO\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL S\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL SIGMA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL T\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL TAU\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL THETA\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL THETA SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL U\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL UPSILON\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL V\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL W\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL X\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL XI\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL Y\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL Z\0"
  "MATHEMATICAL SANS-SERIF BOLD CAPITAL ZETA\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT EIGHT\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT FIVE\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT FOUR\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT NINE\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT ONE\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT SEVEN\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT SIX\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT THREE\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT TWO\0"
  "MATHEMATICAL SANS-SERIF BOLD DIGIT ZERO\0"
  "MATHEMATICAL SANS-SERIF BOLD EPSILON SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL A\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ALPHA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL B\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL BETA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL C\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL CHI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL D\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL DELTA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL E\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL EPSILON\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ETA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL F\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL G\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL GAMMA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL H\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL I\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL IOTA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL J\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL K\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL KAPPA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL L\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL LAMDA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL M\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL MU\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL N\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL NU\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL O\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL OMEGA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL OMICRON\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL P\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PHI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PSI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Q\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL R\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL RHO\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL S\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL SIGMA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL T\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL TAU\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL THETA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL THETA SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL U\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL UPSILON\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL V\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL W\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL X\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL XI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Y\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Z\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ZETA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC EPSILON SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC KAPPA SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC NABLA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC PARTIAL DIFFERENTIAL\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC PHI SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC PI SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC RHO SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL A\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ALPHA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL B\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL BETA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL C\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL CHI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL D\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL DELTA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL E\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL EPSILON\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ETA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL F\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL FINAL SIGMA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL G\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL GAMMA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL H\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL I\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL IOTA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL J\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL K\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL KAPPA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL L\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL LAMDA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL M\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL MU\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL N\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL NU\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL O\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL OMEGA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL OMICRON\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL P\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PHI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PSI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Q\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL R\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL RHO\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL S\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL SIGMA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL T\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL TAU\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL THETA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL U\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL UPSILON\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL V\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL W\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL X\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL XI\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Y\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Z\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ZETA\0"
  "MATHEMATICAL SANS-SERIF BOLD ITALIC THETA SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD KAPPA SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD NABLA\0"
  "MATHEMATICAL SANS-SERIF BOLD PARTIAL DIFFERENTIAL\0"
  "MATHEMATICAL SANS-SERIF BOLD PHI SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD PI SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD RHO SYMBOL\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL A\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL ALPHA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL B\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL BETA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL C\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL CHI\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL D\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL DELTA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL E\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL EPSILON\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL ETA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL F\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL FINAL SIGMA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL G\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL GAMMA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL H\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL I\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL IOTA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL J\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL K\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL KAPPA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL L\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL LAMDA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL M\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL MU\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL N\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL NU\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL O\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL OMEGA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL OMICRON\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL P\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL PHI\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL PI\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL PSI\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL Q\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL R\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL RHO\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL S\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL SIGMA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL T\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL TAU\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL THETA\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL U\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL UPSILON\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL V\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL W\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL X\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL XI\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL Y\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL Z\0"
  "MATHEMATICAL SANS-SERIF BOLD SMALL ZETA\0"
  "MATHEMATICAL SANS-SERIF BOLD THETA SYMBOL\0"
  "MATHEMATICAL SANS-SERIF CAPITAL A\0"
  "MATHEMATICAL SANS-SERIF CAPITAL B\0"
  "MATHEMATICAL SANS-SERIF CAPITAL C\0"
  "MATHEMATICAL SANS-SERIF CAPITAL D\0"
  "MATHEMATICAL SANS-SERIF CAPITAL E\0"
  "MATHEMATICAL SANS-SERIF CAPITAL F\0"
  "MATHEMATICAL SANS-SERIF CAPITAL G\0"
  "MATHEMATICAL SANS-SERIF CAPITAL H\0"
  "MATHEMATICAL SANS-SERIF CAPITAL I\0"
  "MATHEMATICAL SANS-SERIF CAPITAL J\0"
  "MATHEMATICAL SANS-SERIF CAPITAL K\0"
  "MATHEMATICAL SANS-SERIF CAPITAL L\0"
  "MATHEMATICAL SANS-SERIF CAPITAL M\0"
  "MATHEMATICAL SANS-SERIF CAPITAL N\0"
  "MATHEMATICAL SANS-SERIF CAPITAL O\0"
  "MATHEMATICAL SANS-SERIF CAPITAL P\0"
  "MATHEMATICAL SANS-SERIF CAPITAL Q\0"
  "MATHEMATICAL SANS-SERIF CAPITAL R\0"
  "MATHEMATICAL SANS-SERIF CAPITAL S\0"
  "MATHEMATICAL SANS-SERIF CAPITAL T\0"
  "MATHEMATICAL SANS-SERIF CAPITAL U\0"
  "MATHEMATICAL SANS-SERIF CAPITAL V\0"
  "MATHEMATICAL SANS-SERIF CAPITAL W\0"
  "MATHEMATICAL SANS-SERIF CAPITAL X\0"
  "MATHEMATICAL SANS-SERIF CAPITAL Y\0"
  "MATHEMATICAL SANS-SERIF CAPITAL Z\0"
  "MATHEMATICAL SANS-SERIF DIGIT EIGHT\0"
  "MATHEMATICAL SANS-SERIF DIGIT FIVE\0"
  "MATHEMATICAL SANS-SERIF DIGIT FOUR\0"
  "MATHEMATICAL SANS-SERIF DIGIT NINE\0"
  "MATHEMATICAL SANS-SERIF DIGIT ONE\0"
  "MATHEMATICAL SANS-SERIF DIGIT SEVEN\0"
  "MATHEMATICAL SANS-SERIF DIGIT SIX\0"
  "MATHEMATICAL SANS-SERIF DIGIT THREE\0"
  "MATHEMATICAL SANS-SERIF DIGIT TWO\0"
  "MATHEMATICAL SANS-SERIF DIGIT ZERO\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL A\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL B\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL C\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL D\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL E\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL F\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL G\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL H\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL I\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL J\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL K\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL L\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL M\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL N\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL O\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL P\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL Q\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL R\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL S\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL T\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL U\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL V\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL W\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL X\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL Y\0"
  "MATHEMATICAL SANS-SERIF ITALIC CAPITAL Z\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL A\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL B\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL C\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL D\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL E\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL F\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL G\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL H\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL I\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL J\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL K\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL L\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL M\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL N\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL O\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL P\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL Q\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL R\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL S\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL T\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL U\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL V\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL W\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL X\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL Y\0"
  "MATHEMATICAL SANS-SERIF ITALIC SMALL Z\0"
  "MATHEMATICAL SANS-SERIF SMALL A\0"
  "MATHEMATICAL SANS-SERIF SMALL B\0"
  "MATHEMATICAL SANS-SERIF SMALL C\0"
  "MATHEMATICAL SANS-SERIF SMALL D\0"
  "MATHEMATICAL SANS-SERIF SMALL E\0"
  "MATHEMATICAL SANS-SERIF SMALL F\0"
  "MATHEMATICAL SANS-SERIF SMALL G\0"
  "MATHEMATICAL SANS-SERIF SMALL H\0"
  "MATHEMATICAL SANS-SERIF SMALL I\0"
  "MATHEMATICAL SANS-SERIF SMALL J\0"
  "MATHEMATICAL SANS-SERIF SMALL K\0"
  "MATHEMATICAL SANS-SERIF SMALL L\0"
  "MATHEMATICAL SANS-SERIF SMALL M\0"
  "MATHEMATICAL SANS-SERIF SMALL N\0"
  "MATHEMATICAL SANS-SERIF SMALL O\0"
  "MATHEMATICAL SANS-SERIF SMALL P\0"
  "MATHEMATICAL SANS-SERIF SMALL Q\0"
  "MATHEMATICAL SANS-SERIF SMALL R\0"
  "MATHEMATICAL SANS-SERIF SMALL S\0"
  "MATHEMATICAL SANS-SERIF SMALL T\0"
  "MATHEMATICAL SANS-SERIF SMALL U\0"
  "MATHEMATICAL SANS-SERIF SMALL V\0"
  "MATHEMATICAL SANS-SERIF SMALL W\0"
  "MATHEMATICAL SANS-SERIF SMALL X\0"
  "MATHEMATICAL SANS-SERIF SMALL Y\0"
  "MATHEMATICAL SANS-SERIF SMALL Z\0"
  "MATHEMATICAL SCRIPT CAPITAL A\0"
  "MATHEMATICAL SCRIPT CAPITAL C\0"
  "MATHEMATICAL SCRIPT CAPITAL D\0"
  "MATHEMATICAL SCRIPT CAPITAL G\0"
  "MATHEMATICAL SCRIPT CAPITAL J\0"
  "MATHEMATICAL SCRIPT CAPITAL K\0"
  "MATHEMATICAL SCRIPT CAPITAL N\0"
  "MATHEMATICAL SCRIPT CAPITAL O\0"
  "MATHEMATICAL SCRIPT CAPITAL P\0"
  "MATHEMATICAL SCRIPT CAPITAL Q\0"
  "MATHEMATICAL SCRIPT CAPITAL S\0"
  "MATHEMATICAL SCRIPT CAPITAL T\0"
  "MATHEMATICAL SCRIPT CAPITAL U\0"
  "MATHEMATICAL SCRIPT CAPITAL V\0"
  "MATHEMATICAL SCRIPT CAPITAL W\0"
  "MATHEMATICAL SCRIPT CAPITAL X\0"
  "MATHEMATICAL SCRIPT CAPITAL Y\0"
  "MATHEMATICAL SCRIPT CAPITAL Z\0"
  "MATHEMATICAL SCRIPT SMALL A\0"
  "MATHEMATICAL SCRIPT SMALL B\0"
  "MATHEMATICAL SCRIPT SMALL C\0"
  "MATHEMATICAL SCRIPT SMALL D\0"
  "MATHEMATICAL SCRIPT SMALL F\0"
  "MATHEMATICAL SCRIPT SMALL H\0"
  "MATHEMATICAL SCRIPT SMALL I\0"
  "MATHEMATICAL SCRIPT SMALL J\0"
  "MATHEMATICAL SCRIPT SMALL K\0"
  "MATHEMATICAL SCRIPT SMALL L\0"
  "MATHEMATICAL SCRIPT SMALL M\0"
  "MATHEMATICAL SCRIPT SMALL N\0"
  "MATHEMATICAL SCRIPT SMALL P\0"
  "MATHEMATICAL SCRIPT SMALL Q\0"
  "MATHEMATICAL SCRIPT SMALL R\0"
  "MATHEMATICAL SCRIPT SMALL S\0"
  "MATHEMATICAL SCRIPT SMALL T\0"
  "MATHEMATICAL SCRIPT SMALL U\0"
  "MATHEMATICAL SCRIPT SMALL V\0"
  "MATHEMATICAL SCRIPT SMALL W\0"
  "MATHEMATICAL SCRIPT SMALL X\0"
  "MATHEMATICAL SCRIPT SMALL Y\0"
  "MATHEMATICAL SCRIPT SMALL Z\0"
  "MAXIMIZE\0"
  "MAYAN NUMERAL EIGHT\0"
  "MAYAN NUMERAL EIGHTEEN\0"
  "MAYAN NUMERAL ELEVEN\0"
  "MAYAN NUMERAL FIFTEEN\0"
  "MAYAN NUMERAL FIVE\0"
  "MAYAN NUMERAL FOUR\0"
  "MAYAN NUMERAL FOURTEEN\0"
  "MAYAN NUMERAL NINE\0"
  "MAYAN NUMERAL NINETEEN\0"
  "MAYAN NUMERAL ONE\0"
  "MAYAN NUMERAL SEVEN\0"
  "MAYAN NUMERAL SEVENTEEN\0"
  "MAYAN NUMERAL SIX\0"
  "MAYAN NUMERAL SIXTEEN\0"
  "MAYAN NUMERAL TEN\0"
  "MAYAN NUMERAL THIRTEEN\0"
  "MAYAN NUMERAL THREE\0"
  "MAYAN NUMERAL TWELVE\0"
  "MAYAN NUMERAL TWO\0"
  "MAYAN NUMERAL ZERO\0"
  "MEASURED ANGLE\0"
  "MEASURED ANGLE OPENING LEFT\0"
  "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND LEFT\0"
  "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND RIGHT\0"
  "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND DOWN\0"
  "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND UP\0"
  "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND DOWN\0"
  "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND UP\0"
  "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND LEFT\0"
  "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND RIGHT\0"
  "MEASURED BY\0"
  "MEASURED RIGHT ANGLE WITH DOT\0"
  "MEAT ON BONE\0"
  "MECHANICAL ARM\0"
  "MECHANICAL LEG\0"
  "MEDEFAIDRIN CAPITAL LETTER A\0"
  "MEDEFAIDRIN CAPITAL LETTER AI\0"
  "MEDEFAIDRIN CAPITAL LETTER ATIU\0"
  "MEDEFAIDRIN CAPITAL LETTER B\0"
  "MEDEFAIDRIN CAPITAL LETTER C\0"
  "MEDEFAIDRIN CAPITAL LETTER D\0"
  "MEDEFAIDRIN CAPITAL LETTER E\0"
  "MEDEFAIDRIN CAPITAL LETTER F\0"
  "MEDEFAIDRIN CAPITAL LETTER G\0"
  "MEDEFAIDRIN CAPITAL LETTER HP\0"
  "MEDEFAIDRIN CAPITAL LETTER I\0"
  "MEDEFAIDRIN CAPITAL LETTER J\0"
  "MEDEFAIDRIN CAPITAL LETTER K\0"
  "MEDEFAIDRIN CAPITAL LETTER KP\0"
  "MEDEFAIDRIN CAPITAL LETTER L\0"
  "MEDEFAIDRIN CAPITAL LETTER M\0"
  "MEDEFAIDRIN CAPITAL LETTER N\0"
  "MEDEFAIDRIN CAPITAL LETTER NY\0"
  "MEDEFAIDRIN CAPITAL LETTER O\0"
  "MEDEFAIDRIN CAPITAL LETTER OE\0"
  "MEDEFAIDRIN CAPITAL LETTER P\0"
  "MEDEFAIDRIN CAPITAL LETTER Q\0"
  "MEDEFAIDRIN CAPITAL LETTER R\0"
  "MEDEFAIDRIN CAPITAL LETTER S\0"
  "MEDEFAIDRIN CAPITAL LETTER T\0"
  "MEDEFAIDRIN CAPITAL LETTER U\0"
  "MEDEFAIDRIN CAPITAL LETTER V\0"
  "MEDEFAIDRIN CAPITAL LETTER W\0"
  "MEDEFAIDRIN CAPITAL LETTER X\0"
  "MEDEFAIDRIN CAPITAL LETTER Y\0"
  "MEDEFAIDRIN CAPITAL LETTER YU\0"
  "MEDEFAIDRIN CAPITAL LETTER Z\0"
  "MEDEFAIDRIN COMMA\0"
  "MEDEFAIDRIN DIGIT EIGHT\0"
  "MEDEFAIDRIN DIGIT FIVE\0"
  "MEDEFAIDRIN DIGIT FOUR\0"
  "MEDEFAIDRIN DIGIT NINE\0"
  "MEDEFAIDRIN DIGIT ONE\0"
  "MEDEFAIDRIN DIGIT ONE ALTERNATE FORM\0"
  "MEDEFAIDRIN DIGIT SEVEN\0"
  "MEDEFAIDRIN DIGIT SIX\0"
  "MEDEFAIDRIN DIGIT THREE\0"
  "MEDEFAIDRIN DIGIT THREE ALTERNATE FORM\0"
  "MEDEFAIDRIN DIGIT TWO\0"
  "MEDEFAIDRIN DIGIT TWO ALTERNATE FORM\0"
  "MEDEFAIDRIN DIGIT ZERO\0"
  "MEDEFAIDRIN EXCLAMATION OH\0"
  "MEDEFAIDRIN FULL STOP\0"
  "MEDEFAIDRIN NUMBER EIGHTEEN\0"
  "MEDEFAIDRIN NUMBER ELEVEN\0"
  "MEDEFAIDRIN NUMBER FIFTEEN\0"
  "MEDEFAIDRIN NUMBER FOURTEEN\0"
  "MEDEFAIDRIN NUMBER NINETEEN\0"
  "MEDEFAIDRIN NUMBER SEVENTEEN\0"
  "MEDEFAIDRIN NUMBER SIXTEEN\0"
  "MEDEFAIDRIN NUMBER TEN\0"
  "MEDEFAIDRIN NUMBER THIRTEEN\0"
  "MEDEFAIDRIN NUMBER TWELVE\0"
  "MEDEFAIDRIN SMALL LETTER A\0"
  "MEDEFAIDRIN SMALL LETTER AI\0"
  "MEDEFAIDRIN SMALL LETTER ATIU\0"
  "MEDEFAIDRIN SMALL LETTER B\0"
  "MEDEFAIDRIN SMALL LETTER C\0"
  "MEDEFAIDRIN SMALL LETTER D\0"
  "MEDEFAIDRIN SMALL LETTER E\0"
  "MEDEFAIDRIN SMALL LETTER F\0"
  "MEDEFAIDRIN SMALL LETTER G\0"
  "MEDEFAIDRIN SMALL LETTER HP\0"
  "MEDEFAIDRIN SMALL LETTER I\0"
  "MEDEFAIDRIN SMALL LETTER J\0"
  "MEDEFAIDRIN SMALL LETTER K\0"
  "MEDEFAIDRIN SMALL LETTER KP\0"
  "MEDEFAIDRIN SMALL LETTER L\0"
  "MEDEFAIDRIN SMALL LETTER M\0"
  "MEDEFAIDRIN SMALL LETTER N\0"
  "MEDEFAIDRIN SMALL LETTER NY\0"
  "MEDEFAIDRIN SMALL LETTER O\0"
  "MEDEFAIDRIN SMALL LETTER OE\0"
  "MEDEFAIDRIN SMALL LETTER P\0"
  "MEDEFAIDRIN SMALL LETTER Q\0"
  "MEDEFAIDRIN SMALL LETTER R\0"
  "MEDEFAIDRIN SMALL LETTER S\0"
  "MEDEFAIDRIN SMALL LETTER T\0"
  "MEDEFAIDRIN SMALL LETTER U\0"
  "MEDEFAIDRIN SMALL LETTER V\0"
  "MEDEFAIDRIN SMALL LETTER W\0"
  "MEDEFAIDRIN SMALL LETTER X\0"
  "MEDEFAIDRIN SMALL LETTER Y\0"
  "MEDEFAIDRIN SMALL LETTER YU\0"
  "MEDEFAIDRIN SMALL LETTER Z\0"
  "MEDEFAIDRIN SYMBOL AIVA\0"
  "MEDIEVAL COMMA\0"
  "MEDIEVAL EXCLAMATION MARK\0"
  "MEDIEVAL QUESTION MARK\0"
  "MEDIUM BLACK CIRCLE\0"
  "MEDIUM BOLD WHITE CIRCLE\0"
  "MEDIUM EIGHT POINTED BLACK STAR\0"
  "MEDIUM EIGHT SPOKED ASTERISK\0"
  "MEDIUM FIVE SPOKED ASTERISK\0"
  "MEDIUM FLATTENED LEFT PARENTHESIS ORNAMENT\0"
  "MEDIUM FLATTENED RIGHT PARENTHESIS ORNAMENT\0"
  "MEDIUM FOUR POINTED BLACK STAR\0"
  "MEDIUM FOUR POINTED PINWHEEL STAR\0"
  "MEDIUM GREEK CROSS\0"
  "MEDIUM LEFT CURLY BRACKET ORNAMENT\0"
  "MEDIUM LEFT PARENTHESIS ORNAMENT\0"
  "MEDIUM LEFT-POINTING ANGLE BRACKET ORNAMENT\0"
  "MEDIUM MATHEMATICAL SPACE\0"
  "MEDIUM RIGHT CURLY BRACKET ORNAMENT\0"
  "MEDIUM RIGHT PARENTHESIS ORNAMENT\0"
  "MEDIUM RIGHT-POINTING ANGLE BRACKET ORNAMENT\0"
  "MEDIUM SALTIRE\0"
  "MEDIUM SHADE\0"
  "MEDIUM SIX POINTED BLACK STAR\0"
  "MEDIUM SIX SPOKED ASTERISK\0"
  "MEDIUM SMALL WHITE CIRCLE\0"
  "MEDIUM THREE POINTED BLACK STAR\0"
  "MEDIUM THREE POINTED PINWHEEL STAR\0"
  "MEDIUM VERTICAL BAR\0"
  "MEDIUM WHITE CIRCLE\0"
  "MEDIUM WHITE SQUARE\0"
  "MEETEI MAYEK AHANG KHUDAM\0"
  "MEETEI MAYEK ANJI\0"
  "MEETEI MAYEK APUN IYEK\0"
  "MEETEI MAYEK CHEIKHAN\0"
  "MEETEI MAYEK CHEIKHEI\0"
  "MEETEI MAYEK DIGIT EIGHT\0"
  "MEETEI MAYEK DIGIT FIVE\0"
  "MEETEI MAYEK DIGIT FOUR\0"
  "MEETEI MAYEK DIGIT NINE\0"
  "MEETEI MAYEK DIGIT ONE\0"
  "MEETEI MAYEK DIGIT SEVEN\0"
  "MEETEI MAYEK DIGIT SIX\0"
  "MEETEI MAYEK DIGIT THREE\0"
  "MEETEI MAYEK DIGIT TWO\0"
  "MEETEI MAYEK DIGIT ZERO\0"
  "MEETEI MAYEK LETTER ATIYA\0"
  "MEETEI MAYEK LETTER BA\0"
  "MEETEI MAYEK LETTER BHAM\0"
  "MEETEI MAYEK LETTER CHA\0"
  "MEETEI MAYEK LETTER CHIL\0"
  "MEETEI MAYEK LETTER DDA\0"
  "MEETEI MAYEK LETTER DDHA\0"
  "MEETEI MAYEK LETTER DHOU\0"
  "MEETEI MAYEK LETTER DIL\0"
  "MEETEI MAYEK LETTER E\0"
  "MEETEI MAYEK LETTER GHOU\0"
  "MEETEI MAYEK LETTER GOK\0"
  "MEETEI MAYEK LETTER HUK\0"
  "MEETEI MAYEK LETTER I\0"
  "MEETEI MAYEK LETTER I LONSUM\0"
  "MEETEI MAYEK LETTER JHAM\0"
  "MEETEI MAYEK LETTER JIL\0"
  "MEETEI MAYEK LETTER KHOU\0"
  "MEETEI MAYEK LETTER KOK\0"
  "MEETEI MAYEK LETTER KOK LONSUM\0"
  "MEETEI MAYEK LETTER LAI\0"
  "MEETEI MAYEK LETTER LAI LONSUM\0"
  "MEETEI MAYEK LETTER MIT\0"
  "MEETEI MAYEK LETTER MIT LONSUM\0"
  "MEETEI MAYEK LETTER NA\0"
  "MEETEI MAYEK LETTER NA LONSUM\0"
  "MEETEI MAYEK LETTER NGOU\0"
  "MEETEI MAYEK LETTER NGOU LONSUM\0"
  "MEETEI MAYEK LETTER NNA\0"
  "MEETEI MAYEK LETTER NYA\0"
  "MEETEI MAYEK LETTER O\0"
  "MEETEI MAYEK LETTER PA\0"
  "MEETEI MAYEK LETTER PA LONSUM\0"
  "MEETEI MAYEK LETTER PHAM\0"
  "MEETEI MAYEK LETTER RAI\0"
  "MEETEI MAYEK LETTER SAM\0"
  "MEETEI MAYEK LETTER SHA\0"
  "MEETEI MAYEK LETTER SSA\0"
  "MEETEI MAYEK LETTER THOU\0"
  "MEETEI MAYEK LETTER TIL\0"
  "MEETEI MAYEK LETTER TIL LONSUM\0"
  "MEETEI MAYEK LETTER TTA\0"
  "MEETEI MAYEK LETTER TTHA\0"
  "MEETEI MAYEK LETTER UN\0"
  "MEETEI MAYEK LETTER WAI\0"
  "MEETEI MAYEK LETTER YANG\0"
  "MEETEI MAYEK LUM IYEK\0"
  "MEETEI MAYEK SYLLABLE REPETITION MARK\0"
  "MEETEI MAYEK VIRAMA\0"
  "MEETEI MAYEK VOWEL SIGN AAI\0"
  "MEETEI MAYEK VOWEL SIGN AAU\0"
  "MEETEI MAYEK VOWEL SIGN ANAP\0"
  "MEETEI MAYEK VOWEL SIGN AU\0"
  "MEETEI MAYEK VOWEL SIGN CHEINAP\0"
  "MEETEI MAYEK VOWEL SIGN II\0"
  "MEETEI MAYEK VOWEL SIGN INAP\0"
  "MEETEI MAYEK VOWEL SIGN NUNG\0"
  "MEETEI MAYEK VOWEL SIGN ONAP\0"
  "MEETEI MAYEK VOWEL SIGN SOUNAP\0"
  "MEETEI MAYEK VOWEL SIGN UNAP\0"
  "MEETEI MAYEK VOWEL SIGN UU\0"
  "MEETEI MAYEK VOWEL SIGN VISARGA\0"
  "MEETEI MAYEK VOWEL SIGN YENAP\0"
  "MEETEI MAYEK WORD REPETITION MARK\0"
  "MELON\0"
  "MELTING FACE\0"
  "MEMO\0"
  "MENDE KIKAKUI COMBINING NUMBER HUNDRED THOUSANDS\0"
  "MENDE KIKAKUI COMBINING NUMBER HUNDREDS\0"
  "MENDE KIKAKUI COMBINING NUMBER MILLIONS\0"
  "MENDE KIKAKUI COMBINING NUMBER TEENS\0"
  "MENDE KIKAKUI COMBINING NUMBER TEN THOUSANDS\0"
  "MENDE KIKAKUI COMBINING NUMBER TENS\0"
  "MENDE KIKAKUI COMBINING NUMBER THOUSANDS\0"
  "MENDE KIKAKUI DIGIT EIGHT\0"
  "MENDE KIKAKUI DIGIT FIVE\0"
  "MENDE KIKAKUI DIGIT FOUR\0"
  "MENDE KIKAKUI DIGIT NINE\0"
  "MENDE KIKAKUI DIGIT ONE\0"
  "MENDE KIKAKUI DIGIT SEVEN\0"
  "MENDE KIKAKUI DIGIT SIX\0"
  "MENDE KIKAKUI DIGIT THREE\0"
  "MENDE KIKAKUI DIGIT TWO\0"
  "MENDE KIKAKUI SYLLABLE M001 KI\0"
  "MENDE KIKAKUI SYLLABLE M002 KA\0"
  "MENDE KIKAKUI SYLLABLE M003 KU\0"
  "MENDE KIKAKUI SYLLABLE M004 WI\0"
  "MENDE KIKAKUI SYLLABLE M005 WA\0"
  "MENDE KIKAKUI SYLLABLE M006 WU\0"
  "MENDE KIKAKUI SYLLABLE M007 MIN\0"
  "MENDE KIKAKUI SYLLABLE M008 MAN\0"
  "MENDE KIKAKUI SYLLABLE M009 MUN\0"
  "MENDE KIKAKUI SYLLABLE M010 BI\0"
  "MENDE KIKAKUI SYLLABLE M011 BA\0"
  "MENDE KIKAKUI SYLLABLE M012 BU\0"
  "MENDE KIKAKUI SYLLABLE M013 I\0"
  "MENDE KIKAKUI SYLLABLE M014 A\0"
  "MENDE KIKAKUI SYLLABLE M015 U\0"
  "MENDE KIKAKUI SYLLABLE M016 DI\0"
  "MENDE KIKAKUI SYLLABLE M017 DA\0"
  "MENDE KIKAKUI SYLLABLE M018 DU\0"
  "MENDE KIKAKUI SYLLABLE M019 SI\0"
  "MENDE KIKAKUI SYLLABLE M020 SA\0"
  "MENDE KIKAKUI SYLLABLE M021 SU\0"
  "MENDE KIKAKUI SYLLABLE M022 TI\0"
  "MENDE KIKAKUI SYLLABLE M023 TA\0"
  "MENDE KIKAKUI SYLLABLE M024 TU\0"
  "MENDE KIKAKUI SYLLABLE M025 LI\0"
  "MENDE KIKAKUI SYLLABLE M026 LA\0"
  "MENDE KIKAKUI SYLLABLE M027 LU\0"
  "MENDE KIKAKUI SYLLABLE M028 JI\0"
  "MENDE KIKAKUI SYLLABLE M029 JA\0"
  "MENDE KIKAKUI SYLLABLE M030 JU\0"
  "MENDE KIKAKUI SYLLABLE M031 YI\0"
  "MENDE KIKAKUI SYLLABLE M032 YA\0"
  "MENDE KIKAKUI SYLLABLE M033 YU\0"
  "MENDE KIKAKUI SYLLABLE M034 FI\0"
  "MENDE KIKAKUI SYLLABLE M035 FA\0"
  "MENDE KIKAKUI SYLLABLE M036 FU\0"
  "MENDE KIKAKUI SYLLABLE M037 NIN\0"
  "MENDE KIKAKUI SYLLABLE M038 NAN\0"
  "MENDE KIKAKUI SYLLABLE M039 NUN\0"
  "MENDE KIKAKUI SYLLABLE M040 HEE\0"
  "MENDE KIKAKUI SYLLABLE M041 HA\0"
  "MENDE KIKAKUI SYLLABLE M042 HOO\0"
  "MENDE KIKAKUI SYLLABLE M043 NGGA\0"
  "MENDE KIKAKUI SYLLABLE M044 KPEE\0"
  "MENDE KIKAKUI SYLLABLE M045 WO\0"
  "MENDE KIKAKUI SYLLABLE M046 HUAN\0"
  "MENDE KIKAKUI SYLLABLE M047 MBEE\0"
  "MENDE KIKAKUI SYLLABLE M048 KO\0"
  "MENDE KIKAKUI SYLLABLE M049 WVA\0"
  "MENDE KIKAKUI SYLLABLE M050 PU\0"
  "MENDE KIKAKUI SYLLABLE M051 PE\0"
  "MENDE KIKAKUI SYLLABLE M052 HEN\0"
  "MENDE KIKAKUI SYLLABLE M053 HIN\0"
  "MENDE KIKAKUI SYLLABLE M054 LOO\0"
  "MENDE KIKAKUI SYLLABLE M055 TE\0"
  "MENDE KIKAKUI SYLLABLE M056 GBA\0"
  "MENDE KIKAKUI SYLLABLE M057 NGON\0"
  "MENDE KIKAKUI SYLLABLE M058 NYAN\0"
  "MENDE KIKAKUI SYLLABLE M059 MEN\0"
  "MENDE KIKAKUI SYLLABLE M060 NYON\0"
  "MENDE KIKAKUI SYLLABLE M061 WVI\0"
  "MENDE KIKAKUI SYLLABLE M062 MBA\0"
  "MENDE KIKAKUI SYLLABLE M063 JO\0"
  "MENDE KIKAKUI SYLLABLE M064 NDO\0"
  "MENDE KIKAKUI SYLLABLE M065 KEE\0"
  "MENDE KIKAKUI SYLLABLE M066 PO\0"
  "MENDE KIKAKUI SYLLABLE M067 NDOO\0"
  "MENDE KIKAKUI SYLLABLE M068 PI\0"
  "MENDE KIKAKUI SYLLABLE M069 TO\0"
  "MENDE KIKAKUI SYLLABLE M070 GBO\0"
  "MENDE KIKAKUI SYLLABLE M071 GBOO\0"
  "MENDE KIKAKUI SYLLABLE M072 MBE\0"
  "MENDE KIKAKUI SYLLABLE M073 LE\0"
  "MENDE KIKAKUI SYLLABLE M074 KPU\0"
  "MENDE KIKAKUI SYLLABLE M075 FE\0"
  "MENDE KIKAKUI SYLLABLE M076 KOO\0"
  "MENDE KIKAKUI SYLLABLE M077 VO\0"
  "MENDE KIKAKUI SYLLABLE M078 FEE\0"
  "MENDE KIKAKUI SYLLABLE M079 SO\0"
  "MENDE KIKAKUI SYLLABLE M080 YE\0"
  "MENDE KIKAKUI SYLLABLE M081 PEE\0"
  "MENDE KIKAKUI SYLLABLE M082 NGGU\0"
  "MENDE KIKAKUI SYLLABLE M083 HEEI\0"
  "MENDE KIKAKUI SYLLABLE M084 LEE\0"
  "MENDE KIKAKUI SYLLABLE M085 VE\0"
  "MENDE KIKAKUI SYLLABLE M086 LONG NGGE\0"
  "MENDE KIKAKUI SYLLABLE M087 HUN\0"
  "MENDE KIKAKUI SYLLABLE M088 FO\0"
  "MENDE KIKAKUI SYLLABLE M089 DEE\0"
  "MENDE KIKAKUI SYLLABLE M090 NGGI\0"
  "MENDE KIKAKUI SYLLABLE M091 TEE\0"
  "MENDE KIKAKUI SYLLABLE M092 KPA\0"
  "MENDE KIKAKUI SYLLABLE M093 GBEE\0"
  "MENDE KIKAKUI SYLLABLE M094 MON\0"
  "MENDE KIKAKUI SYLLABLE M095 KE\0"
  "MENDE KIKAKUI SYLLABLE M096 HE\0"
  "MENDE KIKAKUI SYLLABLE M097 BE\0"
  "MENDE KIKAKUI SYLLABLE M098 NYEN\0"
  "MENDE KIKAKUI SYLLABLE M099 PA\0"
  "MENDE KIKAKUI SYLLABLE M100 E\0"
  "MENDE KIKAKUI SYLLABLE M101 FAN\0"
  "MENDE KIKAKUI SYLLABLE M102 POO\0"
  "MENDE KIKAKUI SYLLABLE M103 BOO\0"
  "MENDE KIKAKUI SYLLABLE M104 TOO\0"
  "MENDE KIKAKUI SYLLABLE M105 LONG MBOO\0"
  "MENDE KIKAKUI SYLLABLE M106 LONG NGGOO\0"
  "MENDE KIKAKUI SYLLABLE M107 GBE\0"
  "MENDE KIKAKUI SYLLABLE M108 KPE\0"
  "MENDE KIKAKUI SYLLABLE M109 YEE\0"
  "MENDE KIKAKUI SYLLABLE M110 LONG LE\0"
  "MENDE KIKAKUI SYLLABLE M111 GA\0"
  "MENDE KIKAKUI SYLLABLE M112 KPOO\0"
  "MENDE KIKAKUI SYLLABLE M113 JE\0"
  "MENDE KIKAKUI SYLLABLE M114 WOO\0"
  "MENDE KIKAKUI SYLLABLE M115 NGGEE\0"
  "MENDE KIKAKUI SYLLABLE M116 SE\0"
  "MENDE KIKAKUI SYLLABLE M117 NEN\0"
  "MENDE KIKAKUI SYLLABLE M118 WE\0"
  "MENDE KIKAKUI SYLLABLE M119 NDE\0"
  "MENDE KIKAKUI SYLLABLE M120 NGGO\0"
  "MENDE KIKAKUI SYLLABLE M121 YO\0"
  "MENDE KIKAKUI SYLLABLE M122 MBU\0"
  "MENDE KIKAKUI SYLLABLE M123 NDI\0"
  "MENDE KIKAKUI SYLLABLE M124 GBI\0"
  "MENDE KIKAKUI SYLLABLE M125 NDU\0"
  "MENDE KIKAKUI SYLLABLE M126 WEE\0"
  "MENDE KIKAKUI SYLLABLE M127 NGGUA\0"
  "MENDE KIKAKUI SYLLABLE M128 HOOU\0"
  "MENDE KIKAKUI SYLLABLE M129 NDA\0"
  "MENDE KIKAKUI SYLLABLE M130 HAN\0"
  "MENDE KIKAKUI SYLLABLE M131 IN\0"
  "MENDE KIKAKUI SYLLABLE M132 KPI\0"
  "MENDE KIKAKUI SYLLABLE M133 FOO\0"
  "MENDE KIKAKUI SYLLABLE M134 NJOO\0"
  "MENDE KIKAKUI SYLLABLE M135 IN\0"
  "MENDE KIKAKUI SYLLABLE M136 SOO\0"
  "MENDE KIKAKUI SYLLABLE M137 EI\0"
  "MENDE KIKAKUI SYLLABLE M138 BO\0"
  "MENDE KIKAKUI SYLLABLE M139 WVE\0"
  "MENDE KIKAKUI SYLLABLE M140 HO\0"
  "MENDE KIKAKUI SYLLABLE M141 YOO\0"
  "MENDE KIKAKUI SYLLABLE M142 LONG MBO\0"
  "MENDE KIKAKUI SYLLABLE M143 WEI\0"
  "MENDE KIKAKUI SYLLABLE M144 VOO\0"
  "MENDE KIKAKUI SYLLABLE M145 MBI\0"
  "MENDE KIKAKUI SYLLABLE M146 NGGE\0"
  "MENDE KIKAKUI SYLLABLE M147 O\0"
  "MENDE KIKAKUI SYLLABLE M148 GBU\0"
  "MENDE KIKAKUI SYLLABLE M149 NJEE\0"
  "MENDE KIKAKUI SYLLABLE M150 BEE\0"
  "MENDE KIKAKUI SYLLABLE M151 VU\0"
  "MENDE KIKAKUI SYLLABLE M152 NJA\0"
  "MENDE KIKAKUI SYLLABLE M153 LO\0"
  "MENDE KIKAKUI SYLLABLE M154 MUAN\0"
  "MENDE KIKAKUI SYLLABLE M155 GI\0"
  "MENDE KIKAKUI SYLLABLE M156 NGGOO\0"
  "MENDE KIKAKUI SYLLABLE M157 JEE\0"
  "MENDE KIKAKUI SYLLABLE M158 KPO\0"
  "MENDE KIKAKUI SYLLABLE M159 NGGAA\0"
  "MENDE KIKAKUI SYLLABLE M160 JOO\0"
  "MENDE KIKAKUI SYLLABLE M161 LONG MBE\0"
  "MENDE KIKAKUI SYLLABLE M162 SEE\0"
  "MENDE KIKAKUI SYLLABLE M163 EE\0"
  "MENDE KIKAKUI SYLLABLE M164 NYIN\0"
  "MENDE KIKAKUI SYLLABLE M165 OO\0"
  "MENDE KIKAKUI SYLLABLE M166 GUEI\0"
  "MENDE KIKAKUI SYLLABLE M167 GUAN\0"
  "MENDE KIKAKUI SYLLABLE M168 GU\0"
  "MENDE KIKAKUI SYLLABLE M169 NON\0"
  "MENDE KIKAKUI SYLLABLE M170 NYUN\0"
  "MENDE KIKAKUI SYLLABLE M171 RA\0"
  "MENDE KIKAKUI SYLLABLE M172 MBOO\0"
  "MENDE KIKAKUI SYLLABLE M173 VEE\0"
  "MENDE KIKAKUI SYLLABLE M174 MBO\0"
  "MENDE KIKAKUI SYLLABLE M175 LONG JO\0"
  "MENDE KIKAKUI SYLLABLE M176 HI\0"
  "MENDE KIKAKUI SYLLABLE M177 NGUAN\0"
  "MENDE KIKAKUI SYLLABLE M178 EN\0"
  "MENDE KIKAKUI SYLLABLE M179 KUA\0"
  "MENDE KIKAKUI SYLLABLE M180 DOO\0"
  "MENDE KIKAKUI SYLLABLE M181 DO\0"
  "MENDE KIKAKUI SYLLABLE M182 VI\0"
  "MENDE KIKAKUI SYLLABLE M183 LONG NGGO\0"
  "MENDE KIKAKUI SYLLABLE M184 NGEN\0"
  "MENDE KIKAKUI SYLLABLE M185 VA\0"
  "MENDE KIKAKUI SYLLABLE M186 HU\0"
  "MENDE KIKAKUI SYLLABLE M187 MBUU\0"
  "MENDE KIKAKUI SYLLABLE M188 MBEE\0"
  "MENDE KIKAKUI SYLLABLE M189 MUEN\0"
  "MENDE KIKAKUI SYLLABLE M190 GEE\0"
  "MENDE KIKAKUI SYLLABLE M191 NDEE\0"
  "MENDE KIKAKUI SYLLABLE M192 NJU\0"
  "MENDE KIKAKUI SYLLABLE M193 HON\0"
  "MENDE KIKAKUI SYLLABLE M194 WUI\0"
  "MENDE KIKAKUI SYLLABLE M195 AN\0"
  "MENDE KIKAKUI SYLLABLE M196 SIA\0"
  "MENDE KIKAKUI SYLLABLE M197 FUA\0"
  "MENORAH WITH NINE BRANCHES\0"
  "MENS SYMBOL\0"
  "MERCURY\0"
  "MEROITIC CURSIVE FRACTION EIGHT TWELFTHS\0"
  "MEROITIC CURSIVE FRACTION ELEVEN TWELFTHS\0"
  "MEROITIC CURSIVE FRACTION FIVE TWELFTHS\0"
  "MEROITIC CURSIVE FRACTION FOUR TWELFTHS\0"
  "MEROITIC CURSIVE FRACTION NINE TWELFTHS\0"
  "MEROITIC CURSIVE FRACTION ONE HALF\0"
  "MEROITIC CURSIVE FRACTION ONE TWELFTH\0"
  "MEROITIC CURSIVE FRACTION SEVEN TWELFTHS\0"
  "MEROITIC CURSIVE FRACTION SIX TWELFTHS\0"
  "MEROITIC CURSIVE FRACTION TEN TWELFTHS\0"
  "MEROITIC CURSIVE FRACTION THREE TWELFTHS\0"
  "MEROITIC CURSIVE FRACTION TWO TWELFTHS\0"
  "MEROITIC CURSIVE LETTER A\0"
  "MEROITIC CURSIVE LETTER ARCHAIC SA\0"
  "MEROITIC CURSIVE LETTER BA\0"
  "MEROITIC CURSIVE LETTER DA\0"
  "MEROITIC CURSIVE LETTER E\0"
  "MEROITIC CURSIVE LETTER HHA\0"
  "MEROITIC CURSIVE LETTER I\0"
  "MEROITIC CURSIVE LETTER KA\0"
  "MEROITIC CURSIVE LETTER KHA\0"
  "MEROITIC CURSIVE LETTER LA\0"
  "MEROITIC CURSIVE LETTER MA\0"
  "MEROITIC CURSIVE LETTER NA\0"
  "MEROITIC CURSIVE LETTER NE\0"
  "MEROITIC CURSIVE LETTER O\0"
  "MEROITIC CURSIVE LETTER PA\0"
  "MEROITIC CURSIVE LETTER QA\0"
  "MEROITIC CURSIVE LETTER RA\0"
  "MEROITIC CURSIVE LETTER SA\0"
  "MEROITIC CURSIVE LETTER SE\0"
  "MEROITIC CURSIVE LETTER TA\0"
  "MEROITIC CURSIVE LETTER TE\0"
  "MEROITIC CURSIVE LETTER TO\0"
  "MEROITIC CURSIVE LETTER WA\0"
  "MEROITIC CURSIVE LETTER YA\0"
  "MEROITIC CURSIVE LOGOGRAM IMN\0"
  "MEROITIC CURSIVE LOGOGRAM RMT\0"
  "MEROITIC CURSIVE NUMBER EIGHT\0"
  "MEROITIC CURSIVE NUMBER EIGHT HUNDRED\0"
  "MEROITIC CURSIVE NUMBER EIGHT HUNDRED THOUSAND\0"
  "MEROITIC CURSIVE NUMBER EIGHT THOUSAND\0"
  "MEROITIC CURSIVE NUMBER EIGHTY THOUSAND\0"
  "MEROITIC CURSIVE NUMBER FIFTY\0"
  "MEROITIC CURSIVE NUMBER FIFTY THOUSAND\0"
  "MEROITIC CURSIVE NUMBER FIVE\0"
  "MEROITIC CURSIVE NUMBER FIVE HUNDRED\0"
  "MEROITIC CURSIVE NUMBER FIVE HUNDRED THOUSAND\0"
  "MEROITIC CURSIVE NUMBER FIVE THOUSAND\0"
  "MEROITIC CURSIVE NUMBER FORTY\0"
  "MEROITIC CURSIVE NUMBER FORTY THOUSAND\0"
  "MEROITIC CURSIVE NUMBER FOUR\0"
  "MEROITIC CURSIVE NUMBER FOUR HUNDRED\0"
  "MEROITIC CURSIVE NUMBER FOUR HUNDRED THOUSAND\0"
  "MEROITIC CURSIVE NUMBER FOUR THOUSAND\0"
  "MEROITIC CURSIVE NUMBER NINE\0"
  "MEROITIC CURSIVE NUMBER NINE HUNDRED\0"
  "MEROITIC CURSIVE NUMBER NINE HUNDRED THOUSAND\0"
  "MEROITIC CURSIVE NUMBER NINE THOUSAND\0"
  "MEROITIC CURSIVE NUMBER NINETY THOUSAND\0"
  "MEROITIC CURSIVE NUMBER ONE\0"
  "MEROITIC CURSIVE NUMBER ONE HUNDRED\0"
  "MEROITIC CURSIVE NUMBER ONE HUNDRED THOUSAND\0"
  "MEROITIC CURSIVE NUMBER ONE THOUSAND\0"
  "MEROITIC CURSIVE NUMBER SEVEN\0"
  "MEROITIC CURSIVE NUMBER SEVEN HUNDRED\0"
  "MEROITIC CURSIVE NUMBER SEVEN HUNDRED THOUSAND\0"
  "MEROITIC CURSIVE NUMBER SEVEN THOUSAND\0"
  "MEROITIC CURSIVE NUMBER SEVENTY\0"
  "MEROITIC CURSIVE NUMBER SEVENTY THOUSAND\0"
  "MEROITIC CURSIVE NUMBER SIX\0"
  "MEROITIC CURSIVE NUMBER SIX HUNDRED\0"
  "MEROITIC CURSIVE NUMBER SIX HUNDRED THOUSAND\0"
  "MEROITIC CURSIVE NUMBER SIX THOUSAND\0"
  "MEROITIC CURSIVE NUMBER SIXTY\0"
  "MEROITIC CURSIVE NUMBER SIXTY THOUSAND\0"
  "MEROITIC CURSIVE NUMBER TEN\0"
  "MEROITIC CURSIVE NUMBER TEN THOUSAND\0"
  "MEROITIC CURSIVE NUMBER THIRTY\0"
  "MEROITIC CURSIVE NUMBER THIRTY THOUSAND\0"
  "MEROITIC CURSIVE NUMBER THREE\0"
  "MEROITIC CURSIVE NUMBER THREE HUNDRED\0"
  "MEROITIC CURSIVE NUMBER THREE HUNDRED THOUSAND\0"
  "MEROITIC CURSIVE NUMBER THREE THOUSAND\0"
  "MEROITIC CURSIVE NUMBER TWENTY\0"
  "MEROITIC CURSIVE NUMBER TWENTY THOUSAND\0"
  "MEROITIC CURSIVE NUMBER TWO\0"
  "MEROITIC CURSIVE NUMBER TWO HUNDRED\0"
  "MEROITIC CURSIVE NUMBER TWO HUNDRED THOUSAND\0"
  "MEROITIC CURSIVE NUMBER TWO THOUSAND\0"
  "MEROITIC HIEROGLYPHIC LETTER A\0"
  "MEROITIC HIEROGLYPHIC LETTER BA\0"
  "MEROITIC HIEROGLYPHIC LETTER BA-2\0"
  "MEROITIC HIEROGLYPHIC LETTER DA\0"
  "MEROITIC HIEROGLYPHIC LETTER E\0"
  "MEROITIC HIEROGLYPHIC LETTER HHA\0"
  "MEROITIC HIEROGLYPHIC LETTER I\0"
  "MEROITIC HIEROGLYPHIC LETTER KA\0"
  "MEROITIC HIEROGLYPHIC LETTER KHA\0"
  "MEROITIC HIEROGLYPHIC LETTER LA\0"
  "MEROITIC HIEROGLYPHIC LETTER MA\0"
  "MEROITIC HIEROGLYPHIC LETTER NA\0"
  "MEROITIC HIEROGLYPHIC LETTER NA-2\0"
  "MEROITIC HIEROGLYPHIC LETTER NE\0"
  "MEROITIC HIEROGLYPHIC LETTER NE-2\0"
  "MEROITIC HIEROGLYPHIC LETTER O\0"
  "MEROITIC HIEROGLYPHIC LETTER PA\0"
  "MEROITIC HIEROGLYPHIC LETTER QA\0"
  "MEROITIC HIEROGLYPHIC LETTER RA\0"
  "MEROITIC HIEROGLYPHIC LETTER RA-2\0"
  "MEROITIC HIEROGLYPHIC LETTER SA\0"
  "MEROITIC HIEROGLYPHIC LETTER SA-2\0"
  "MEROITIC HIEROGLYPHIC LETTER SE\0"
  "MEROITIC HIEROGLYPHIC LETTER TA\0"
  "MEROITIC HIEROGLYPHIC LETTER TA-2\0"
  "MEROITIC HIEROGLYPHIC LETTER TE\0"
  "MEROITIC HIEROGLYPHIC LETTER TE-2\0"
  "MEROITIC HIEROGLYPHIC LETTER TO\0"
  "MEROITIC HIEROGLYPHIC LETTER WA\0"
  "MEROITIC HIEROGLYPHIC LETTER YA\0"
  "MEROITIC HIEROGLYPHIC SYMBOL VIDJ\0"
  "MEROITIC HIEROGLYPHIC SYMBOL VIDJ-2\0"
  "MERPERSON\0"
  "METRICAL BREVE\0"
  "METRICAL LONG OVER SHORT\0"
  "METRICAL LONG OVER TWO SHORTS\0"
  "METRICAL PENTASEME\0"
  "METRICAL SHORT OVER LONG\0"
  "METRICAL TETRASEME\0"
  "METRICAL TRISEME\0"
  "METRICAL TWO SHORTS JOINED\0"
  "METRICAL TWO SHORTS OVER LONG\0"
  "METRO\0"
  "MIAO LETTER AH\0"
  "MIAO LETTER ARCHAIC MA\0"
  "MIAO LETTER ARCHAIC NA\0"
  "MIAO LETTER ARCHAIC NGA\0"
  "MIAO LETTER ARCHAIC ZZA\0"
  "MIAO LETTER BA\0"
  "MIAO LETTER BRI\0"
  "MIAO LETTER DA\0"
  "MIAO LETTER DDA\0"
  "MIAO LETTER DLHA\0"
  "MIAO LETTER DLHYA\0"
  "MIAO LETTER DZA\0"
  "MIAO LETTER DZHA\0"
  "MIAO LETTER DZYI\0"
  "MIAO LETTER DZZA\0"
  "MIAO LETTER FA\0"
  "MIAO LETTER GA\0"
  "MIAO LETTER GHA\0"
  "MIAO LETTER GHHA\0"
  "MIAO LETTER HA\0"
  "MIAO LETTER HHA\0"
  "MIAO LETTER KA\0"
  "MIAO LETTER LA\0"
  "MIAO LETTER LHA\0"
  "MIAO LETTER LHYA\0"
  "MIAO LETTER LYA\0"
  "MIAO LETTER MA\0"
  "MIAO LETTER MHA\0"
  "MIAO LETTER NA\0"
  "MIAO LETTER NASALIZATION\0"
  "MIAO LETTER NGA\0"
  "MIAO LETTER NGHA\0"
  "MIAO LETTER NHA\0"
  "MIAO LETTER NNA\0"
  "MIAO LETTER NNHA\0"
  "MIAO LETTER NYA\0"
  "MIAO LETTER NYHA\0"
  "MIAO LETTER PA\0"
  "MIAO LETTER PLA\0"
  "MIAO LETTER QA\0"
  "MIAO LETTER QGA\0"
  "MIAO LETTER REFORMED TONE-1\0"
  "MIAO LETTER REFORMED TONE-2\0"
  "MIAO LETTER REFORMED TONE-4\0"
  "MIAO LETTER REFORMED TONE-5\0"
  "MIAO LETTER REFORMED TONE-6\0"
  "MIAO LETTER REFORMED TONE-8\0"
  "MIAO LETTER REFORMED TSHA\0"
  "MIAO LETTER RTE\0"
  "MIAO LETTER SA\0"
  "MIAO LETTER SHA\0"
  "MIAO LETTER SSA\0"
  "MIAO LETTER SYI\0"
  "MIAO LETTER TA\0"
  "MIAO LETTER TE\0"
  "MIAO LETTER TLHA\0"
  "MIAO LETTER TLHYA\0"
  "MIAO LETTER TONE-2\0"
  "MIAO LETTER TONE-3\0"
  "MIAO LETTER TONE-4\0"
  "MIAO LETTER TONE-5\0"
  "MIAO LETTER TONE-6\0"
  "MIAO LETTER TONE-7\0"
  "MIAO LETTER TONE-8\0"
  "MIAO LETTER TSA\0"
  "MIAO LETTER TSE\0"
  "MIAO LETTER TSHA\0"
  "MIAO LETTER TSSA\0"
  "MIAO LETTER TTA\0"
  "MIAO LETTER VA\0"
  "MIAO LETTER VFA\0"
  "MIAO LETTER WA\0"
  "MIAO LETTER XA\0"
  "MIAO LETTER YI DZHA\0"
  "MIAO LETTER YI KA\0"
  "MIAO LETTER YI NNA\0"
  "MIAO LETTER YI PA\0"
  "MIAO LETTER YI TA\0"
  "MIAO LETTER YI TSA\0"
  "MIAO LETTER YI TSHA\0"
  "MIAO LETTER YI TTA\0"
  "MIAO LETTER ZA\0"
  "MIAO LETTER ZHA\0"
  "MIAO LETTER ZSA\0"
  "MIAO LETTER ZSHA\0"
  "MIAO LETTER ZZA\0"
  "MIAO LETTER ZZSA\0"
  "MIAO LETTER ZZSYA\0"
  "MIAO LETTER ZZYA\0"
  "MIAO SIGN ASPIRATION\0"
  "MIAO SIGN CONSONANT MODIFIER BAR\0"
  "MIAO SIGN REFORMED ASPIRATION\0"
  "MIAO SIGN REFORMED VOICING\0"
  "MIAO TONE ABOVE\0"
  "MIAO TONE BELOW\0"
  "MIAO TONE RIGHT\0"
  "MIAO TONE TOP RIGHT\0"
  "MIAO VOWEL SIGN A\0"
  "MIAO VOWEL SIGN AA\0"
  "MIAO VOWEL SIGN AE\0"
  "MIAO VOWEL SIGN AEE\0"
  "MIAO VOWEL SIGN AHH\0"
  "MIAO VOWEL SIGN AI\0"
  "MIAO VOWEL SIGN AN\0"
  "MIAO VOWEL SIGN ANG\0"
  "MIAO VOWEL SIGN AU\0"
  "MIAO VOWEL SIGN E\0"
  "MIAO VOWEL SIGN EA\0"
  "MIAO VOWEL SIGN EI\0"
  "MIAO VOWEL SIGN EN\0"
  "MIAO VOWEL SIGN ENG\0"
  "MIAO VOWEL SIGN ER\0"
  "MIAO VOWEL SIGN ERR\0"
  "MIAO VOWEL SIGN I\0"
  "MIAO VOWEL SIGN IA\0"
  "MIAO VOWEL SIGN IAN\0"
  "MIAO VOWEL SIGN IANG\0"
  "MIAO VOWEL SIGN IE\0"
  "MIAO VOWEL SIGN IG\0"
  "MIAO VOWEL SIGN II\0"
  "MIAO VOWEL SIGN ING\0"
  "MIAO VOWEL SIGN IO\0"
  "MIAO VOWEL SIGN IONG\0"
  "MIAO VOWEL SIGN IU\0"
  "MIAO VOWEL SIGN N\0"
  "MIAO VOWEL SIGN NG\0"
  "MIAO VOWEL SIGN O\0"
  "MIAO VOWEL SIGN OER\0"
  "MIAO VOWEL SIGN OEY\0"
  "MIAO VOWEL SIGN OG\0"
  "MIAO VOWEL SIGN OO\0"
  "MIAO VOWEL SIGN OU\0"
  "MIAO VOWEL SIGN ROUNDED ER\0"
  "MIAO VOWEL SIGN ROUNDED ERR\0"
  "MIAO VOWEL SIGN U\0"
  "MIAO VOWEL SIGN UA\0"
  "MIAO VOWEL SIGN UAN\0"
  "MIAO VOWEL SIGN UANG\0"
  "MIAO VOWEL SIGN UEI\0"
  "MIAO VOWEL SIGN UI\0"
  "MIAO VOWEL SIGN UNG\0"
  "MIAO VOWEL SIGN UOG\0"
  "MIAO VOWEL SIGN UU\0"
  "MIAO VOWEL SIGN VW\0"
  "MIAO VOWEL SIGN W\0"
  "MIAO VOWEL SIGN WO\0"
  "MIAO VOWEL SIGN Y\0"
  "MIAO VOWEL SIGN YI\0"
  "MIAO VOWEL SIGN YUI\0"
  "MICRO SIGN\0"
  "MICROBE\0"
  "MICROPHONE\0"
  "MICROSCOPE\0"
  "MIDDLE DOT\0"
  "MIDDLE LEFT ONE QUARTER BLOCK\0"
  "MIDDLE RIGHT ONE QUARTER BLOCK\0"
  "MIDDLE THIRD INDUCTOR\0"
  "MIDDLE THIRD WHITE RIGHT POINTING INDEX\0"
  "MIDLINE HORIZONTAL ELLIPSIS\0"
  "MILITARY HELMET\0"
  "MILITARY MEDAL\0"
  "MILKY WAY\0"
  "MILL SIGN\0"
  "MINIBUS\0"
  "MINIDISC\0"
  "MINIMIZE\0"
  "MINUS SIGN\0"
  "MINUS SIGN IN TRIANGLE\0"
  "MINUS SIGN WITH COMMA ABOVE\0"
  "MINUS SIGN WITH DOT BELOW\0"
  "MINUS SIGN WITH FALLING DOTS\0"
  "MINUS SIGN WITH RISING DOTS\0"
  "MINUS TILDE\0"
  "MINUS-OR-PLUS SIGN\0"
  "MINY\0"
  "MIRROR\0"
  "MIRROR BALL\0"
  "MOBILE PHONE\0"
  "MOBILE PHONE OFF\0"
  "MOBILE PHONE WITH RIGHTWARDS ARROW AT LEFT\0"
  "MODELS\0"
  "MODERN PENTATHLON\0"
  "MODI ABBREVIATION SIGN\0"
  "MODI DANDA\0"
  "MODI DIGIT EIGHT\0"
  "MODI DIGIT FIVE\0"
  "MODI DIGIT FOUR\0"
  "MODI DIGIT NINE\0"
  "MODI DIGIT ONE\0"
  "MODI DIGIT SEVEN\0"
  "MODI DIGIT SIX\0"
  "MODI DIGIT THREE\0"
  "MODI DIGIT TWO\0"
  "MODI DIGIT ZERO\0"
  "MODI DOUBLE DANDA\0"
  "MODI LETTER A\0"
  "MODI LETTER AA\0"
  "MODI LETTER AI\0"
  "MODI LETTER AU\0"
  "MODI LETTER BA\0"
  "MODI LETTER BHA\0"
  "MODI LETTER CA\0"
  "MODI LETTER CHA\0"
  "MODI LETTER DA\0"
  "MODI LETTER DDA\0"
  "MODI LETTER DDHA\0"
  "MODI LETTER DHA\0"
  "MODI LETTER E\0"
  "MODI LETTER GA\0"
  "MODI LETTER GHA\0"
  "MODI LETTER HA\0"
  "MODI LETTER I\0"
  "MODI LETTER II\0"
  "MODI LETTER JA\0"
  "MODI LETTER JHA\0"
  "MODI LETTER KA\0"
  "MODI LETTER KHA\0"
  "MODI LETTER LA\0"
  "MODI LETTER LLA\0"
  "MODI LETTER MA\0"
  "MODI LETTER NA\0"
  "MODI LETTER NGA\0"
  "MODI LETTER NNA\0"
  "MODI LETTER NYA\0"
  "MODI LETTER O\0"
  "MODI LETTER PA\0"
  "MODI LETTER PHA\0"
  "MODI LETTER RA\0"
  "MODI LETTER SA\0"
  "MODI LETTER SHA\0"
  "MODI LETTER SSA\0"
  "MODI LETTER TA\0"
  "MODI LETTER THA\0"
  "MODI LETTER TTA\0"
  "MODI LETTER TTHA\0"
  "MODI LETTER U\0"
  "MODI LETTER UU\0"
  "MODI LETTER VA\0"
  "MODI LETTER VOCALIC L\0"
  "MODI LETTER VOCALIC LL\0"
  "MODI LETTER VOCALIC R\0"
  "MODI LETTER VOCALIC RR\0"
  "MODI LETTER YA\0"
  "MODI SIGN ANUSVARA\0"
  "MODI SIGN ARDHACANDRA\0"
  "MODI SIGN HUVA\0"
  "MODI SIGN VIRAMA\0"
  "MODI SIGN VISARGA\0"
  "MODI VOWEL SIGN AA\0"
  "MODI VOWEL SIGN AI\0"
  "MODI VOWEL SIGN AU\0"
  "MODI VOWEL SIGN E\0"
  "MODI VOWEL SIGN I\0"
  "MODI VOWEL SIGN II\0"
  "MODI VOWEL SIGN O\0"
  "MODI VOWEL SIGN U\0"
  "MODI VOWEL SIGN UU\0"
  "MODI VOWEL SIGN VOCALIC L\0"
  "MODI VOWEL SIGN VOCALIC LL\0"
  "MODI VOWEL SIGN VOCALIC R\0"
  "MODI VOWEL SIGN VOCALIC RR\0"
  "MODIFIER BREVE WITH INVERTED BREVE\0"
  "MODIFIER LETTER ACUTE ACCENT\0"
  "MODIFIER LETTER ALVEOLAR CLICK\0"
  "MODIFIER LETTER APOSTROPHE\0"
  "MODIFIER LETTER BEGIN HIGH TONE\0"
  "MODIFIER LETTER BEGIN LOW TONE\0"
  "MODIFIER LETTER BILABIAL CLICK\0"
  "MODIFIER LETTER CAPITAL A\0"
  "MODIFIER LETTER CAPITAL AE\0"
  "MODIFIER LETTER CAPITAL B\0"
  "MODIFIER LETTER CAPITAL BARRED B\0"
  "MODIFIER LETTER CAPITAL C\0"
  "MODIFIER LETTER CAPITAL D\0"
  "MODIFIER LETTER CAPITAL E\0"
  "MODIFIER LETTER CAPITAL F\0"
  "MODIFIER LETTER CAPITAL G\0"
  "MODIFIER LETTER CAPITAL H\0"
  "MODIFIER LETTER CAPITAL H WITH STROKE\0"
  "MODIFIER LETTER CAPITAL I\0"
  "MODIFIER LETTER CAPITAL J\0"
  "MODIFIER LETTER CAPITAL K\0"
  "MODIFIER LETTER CAPITAL L\0"
  "MODIFIER LETTER CAPITAL M\0"
  "MODIFIER LETTER CAPITAL N\0"
  "MODIFIER LETTER CAPITAL O\0"
  "MODIFIER LETTER CAPITAL OU\0"
  "MODIFIER LETTER CAPITAL P\0"
  "MODIFIER LETTER CAPITAL Q\0"
  "MODIFIER LETTER CAPITAL R\0"
  "MODIFIER LETTER CAPITAL REVERSED E\0"
  "MODIFIER LETTER CAPITAL REVERSED N\0"
  "MODIFIER LETTER CAPITAL T\0"
  "MODIFIER LETTER CAPITAL U\0"
  "MODIFIER LETTER CAPITAL V\0"
  "MODIFIER LETTER CAPITAL W\0"
  "MODIFIER LETTER CENTRED LEFT HALF RING\0"
  "MODIFIER LETTER CENTRED RIGHT HALF RING\0"
  "MODIFIER LETTER CHINESE TONE YANG PING\0"
  "MODIFIER LETTER CHINESE TONE YANG QU\0"
  "MODIFIER LETTER CHINESE TONE YANG RU\0"
  "MODIFIER LETTER CHINESE TONE YANG SHANG\0"
  "MODIFIER LETTER CHINESE TONE YIN PING\0"
  "MODIFIER LETTER CHINESE TONE YIN QU\0"
  "MODIFIER LETTER CHINESE TONE YIN RU\0"
  "MODIFIER LETTER CHINESE TONE YIN SHANG\0"
  "MODIFIER LETTER CIRCUMFLEX ACCENT\0"
  "MODIFIER LETTER COLON\0"
  "MODIFIER LETTER CROSS ACCENT\0"
  "MODIFIER LETTER CYRILLIC EN\0"
  "MODIFIER LETTER CYRILLIC HARD SIGN\0"
  "MODIFIER LETTER CYRILLIC SMALL A\0"
  "MODIFIER LETTER CYRILLIC SMALL BARRED O\0"
  "MODIFIER LETTER CYRILLIC SMALL BE\0"
  "MODIFIER LETTER CYRILLIC SMALL BYELORUSSIAN-UKRAINIAN I\0"
  "MODIFIER LETTER CYRILLIC SMALL CHE\0"
  "MODIFIER LETTER CYRILLIC SMALL DE\0"
  "MODIFIER LETTER CYRILLIC SMALL DZZE\0"
  "MODIFIER LETTER CYRILLIC SMALL E\0"
  "MODIFIER LETTER CYRILLIC SMALL EF\0"
  "MODIFIER LETTER CYRILLIC SMALL EL\0"
  "MODIFIER LETTER CYRILLIC SMALL EM\0"
  "MODIFIER LETTER CYRILLIC SMALL ER\0"
  "MODIFIER LETTER CYRILLIC SMALL ES\0"
  "MODIFIER LETTER CYRILLIC SMALL ES WITH DESCENDER\0"
  "MODIFIER LETTER CYRILLIC SMALL GHE\0"
  "MODIFIER LETTER CYRILLIC SMALL HA\0"
  "MODIFIER LETTER CYRILLIC SMALL I\0"
  "MODIFIER LETTER CYRILLIC SMALL IE\0"
  "MODIFIER LETTER CYRILLIC SMALL JE\0"
  "MODIFIER LETTER CYRILLIC SMALL KA\0"
  "MODIFIER LETTER CYRILLIC SMALL O\0"
  "MODIFIER LETTER CYRILLIC SMALL PALOCHKA\0"
  "MODIFIER LETTER CYRILLIC SMALL PE\0"
  "MODIFIER LETTER CYRILLIC SMALL SCHWA\0"
  "MODIFIER LETTER CYRILLIC SMALL SHA\0"
  "MODIFIER LETTER CYRILLIC SMALL STRAIGHT U\0"
  "MODIFIER LETTER CYRILLIC SMALL STRAIGHT U WITH STROKE\0"
  "MODIFIER LETTER CYRILLIC SMALL TE\0"
  "MODIFIER LETTER CYRILLIC SMALL TSE\0"
  "MODIFIER LETTER CYRILLIC SMALL U\0"
  "MODIFIER LETTER CYRILLIC SMALL VE\0"
  "MODIFIER LETTER CYRILLIC SMALL YERU\0"
  "MODIFIER LETTER CYRILLIC SMALL YERU WITH BACK YER\0"
  "MODIFIER LETTER CYRILLIC SMALL YU\0"
  "MODIFIER LETTER CYRILLIC SMALL ZE\0"
  "MODIFIER LETTER CYRILLIC SMALL ZHE\0"
  "MODIFIER LETTER CYRILLIC SOFT SIGN\0"
  "MODIFIER LETTER DENTAL CLICK\0"
  "MODIFIER LETTER DOT HORIZONTAL BAR\0"
  "MODIFIER LETTER DOT SLASH\0"
  "MODIFIER LETTER DOT VERTICAL BAR\0"
  "MODIFIER LETTER DOUBLE APOSTROPHE\0"
  "MODIFIER LETTER DOUBLE PRIME\0"
  "MODIFIER LETTER DOWN ARROWHEAD\0"
  "MODIFIER LETTER DOWN TACK\0"
  "MODIFIER LETTER END HIGH TONE\0"
  "MODIFIER LETTER END LOW TONE\0"
  "MODIFIER LETTER EXTRA-HIGH DOTTED LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER EXTRA-HIGH DOTTED TONE BAR\0"
  "MODIFIER LETTER EXTRA-HIGH LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER EXTRA-HIGH TONE BAR\0"
  "MODIFIER LETTER EXTRA-LOW DOTTED LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER EXTRA-LOW DOTTED TONE BAR\0"
  "MODIFIER LETTER EXTRA-LOW LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER EXTRA-LOW TONE BAR\0"
  "MODIFIER LETTER GEORGIAN NAR\0"
  "MODIFIER LETTER GLOTTAL STOP\0"
  "MODIFIER LETTER GLOTTAL STOP WITH STROKE\0"
  "MODIFIER LETTER GRAVE ACCENT\0"
  "MODIFIER LETTER HALF TRIANGULAR COLON\0"
  "MODIFIER LETTER HIGH DOTTED LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER HIGH DOTTED TONE BAR\0"
  "MODIFIER LETTER HIGH LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER HIGH TONE BAR\0"
  "MODIFIER LETTER LATERAL CLICK\0"
  "MODIFIER LETTER LEFT ARROWHEAD\0"
  "MODIFIER LETTER LEFT HALF RING\0"
  "MODIFIER LETTER LEFT TACK\0"
  "MODIFIER LETTER LOW ACUTE ACCENT\0"
  "MODIFIER LETTER LOW CIRCUMFLEX ACCENT\0"
  "MODIFIER LETTER LOW DOTTED LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER LOW DOTTED TONE BAR\0"
  "MODIFIER LETTER LOW DOWN ARROWHEAD\0"
  "MODIFIER LETTER LOW GRAVE ACCENT\0"
  "MODIFIER LETTER LOW INVERTED EXCLAMATION MARK\0"
  "MODIFIER LETTER LOW LEFT ARROW\0"
  "MODIFIER LETTER LOW LEFT ARROWHEAD\0"
  "MODIFIER LETTER LOW LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER LOW MACRON\0"
  "MODIFIER LETTER LOW RIGHT ARROWHEAD\0"
  "MODIFIER LETTER LOW RING\0"
  "MODIFIER LETTER LOW TILDE\0"
  "MODIFIER LETTER LOW TONE BAR\0"
  "MODIFIER LETTER LOW UP ARROWHEAD\0"
  "MODIFIER LETTER LOW VERTICAL LINE\0"
  "MODIFIER LETTER LOWER RIGHT CORNER ANGLE\0"
  "MODIFIER LETTER MACRON\0"
  "MODIFIER LETTER MID DOTTED LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER MID DOTTED TONE BAR\0"
  "MODIFIER LETTER MID LEFT-STEM TONE BAR\0"
  "MODIFIER LETTER MID TONE BAR\0"
  "MODIFIER LETTER MIDDLE DOUBLE ACUTE ACCENT\0"
  "MODIFIER LETTER MIDDLE DOUBLE GRAVE ACCENT\0"
  "MODIFIER LETTER MIDDLE GRAVE ACCENT\0"
  "MODIFIER LETTER MINUS SIGN\0"
  "MODIFIER LETTER OPEN SHELF\0"
  "MODIFIER LETTER PLUS SIGN\0"
  "MODIFIER LETTER PRIME\0"
  "MODIFIER LETTER RAISED COLON\0"
  "MODIFIER LETTER RAISED DOWN ARROW\0"
  "MODIFIER LETTER RAISED EXCLAMATION MARK\0"
  "MODIFIER LETTER RAISED INVERTED EXCLAMATION MARK\0"
  "MODIFIER LETTER RAISED UP ARROW\0"
  "MODIFIER LETTER RETROFLEX CLICK WITH RETROFLEX HOOK\0"
  "MODIFIER LETTER REVERSED COMMA\0"
  "MODIFIER LETTER REVERSED GLOTTAL STOP\0"
  "MODIFIER LETTER REVERSED GLOTTAL STOP WITH STROKE\0"
  "MODIFIER LETTER RHOTIC HOOK\0"
  "MODIFIER LETTER RIGHT ARROWHEAD\0"
  "MODIFIER LETTER RIGHT HALF RING\0"
  "MODIFIER LETTER RIGHT TACK\0"
  "MODIFIER LETTER SHELF\0"
  "MODIFIER LETTER SHORT EQUALS SIGN\0"
  "MODIFIER LETTER SMALL A\0"
  "MODIFIER LETTER SMALL AE\0"
  "MODIFIER LETTER SMALL AIN\0"
  "MODIFIER LETTER SMALL ALPHA\0"
  "MODIFIER LETTER SMALL B\0"
  "MODIFIER LETTER SMALL B WITH HOOK\0"
  "MODIFIER LETTER SMALL BARRED O\0"
  "MODIFIER LETTER SMALL BETA\0"
  "MODIFIER LETTER SMALL BOTTOM HALF O\0"
  "MODIFIER LETTER SMALL C\0"
  "MODIFIER LETTER SMALL C WITH CURL\0"
  "MODIFIER LETTER SMALL CAPITAL AA\0"
  "MODIFIER LETTER SMALL CAPITAL B\0"
  "MODIFIER LETTER SMALL CAPITAL G\0"
  "MODIFIER LETTER SMALL CAPITAL G WITH HOOK\0"
  "MODIFIER LETTER SMALL CAPITAL H\0"
  "MODIFIER LETTER SMALL CAPITAL I\0"
  "MODIFIER LETTER SMALL CAPITAL I WITH STROKE\0"
  "MODIFIER LETTER SMALL CAPITAL INVERTED R\0"
  "MODIFIER LETTER SMALL CAPITAL L\0"
  "MODIFIER LETTER SMALL CAPITAL L WITH BELT\0"
  "MODIFIER LETTER SMALL CAPITAL N\0"
  "MODIFIER LETTER SMALL CAPITAL OE\0"
  "MODIFIER LETTER SMALL CAPITAL R\0"
  "MODIFIER LETTER SMALL CAPITAL U\0"
  "MODIFIER LETTER SMALL CAPITAL Y\0"
  "MODIFIER LETTER SMALL CHI\0"
  "MODIFIER LETTER SMALL CLOSED OMEGA\0"
  "MODIFIER LETTER SMALL CLOSED REVERSED OPEN E\0"
  "MODIFIER LETTER SMALL D\0"
  "MODIFIER LETTER SMALL D WITH HOOK\0"
  "MODIFIER LETTER SMALL D WITH HOOK AND TAIL\0"
  "MODIFIER LETTER SMALL D WITH TAIL\0"
  "MODIFIER LETTER SMALL DELTA\0"
  "MODIFIER LETTER SMALL DEZH DIGRAPH\0"
  "MODIFIER LETTER SMALL DOTLESS J WITH STROKE\0"
  "MODIFIER LETTER SMALL DOTLESS J WITH STROKE AND HOOK\0"
  "MODIFIER LETTER SMALL DZ DIGRAPH\0"
  "MODIFIER LETTER SMALL DZ DIGRAPH WITH CURL\0"
  "MODIFIER LETTER SMALL DZ DIGRAPH WITH RETROFLEX HOOK\0"
  "MODIFIER LETTER SMALL E\0"
  "MODIFIER LETTER SMALL ENG\0"
  "MODIFIER LETTER SMALL ESH\0"
  "MODIFIER LETTER SMALL ETH\0"
  "MODIFIER LETTER SMALL EZH\0"
  "MODIFIER LETTER SMALL F\0"
  "MODIFIER LETTER SMALL FENG DIGRAPH\0"
  "MODIFIER LETTER SMALL G\0"
  "MODIFIER LETTER SMALL G WITH HOOK\0"
  "MODIFIER LETTER SMALL GAMMA\0"
  "MODIFIER LETTER SMALL GREEK GAMMA\0"
  "MODIFIER LETTER SMALL GREEK PHI\0"
  "MODIFIER LETTER SMALL H\0"
  "MODIFIER LETTER SMALL H WITH HOOK\0"
  "MODIFIER LETTER SMALL H WITH STROKE\0"
  "MODIFIER LETTER SMALL HENG\0"
  "MODIFIER LETTER SMALL HENG WITH HOOK\0"
  "MODIFIER LETTER SMALL I WITH STROKE\0"
  "MODIFIER LETTER SMALL IOTA\0"
  "MODIFIER LETTER SMALL J\0"
  "MODIFIER LETTER SMALL J WITH CROSSED-TAIL\0"
  "MODIFIER LETTER SMALL K\0"
  "MODIFIER LETTER SMALL L\0"
  "MODIFIER LETTER SMALL L WITH BELT\0"
  "MODIFIER LETTER SMALL L WITH INVERTED LAZY S\0"
  "MODIFIER LETTER SMALL L WITH MIDDLE TILDE\0"
  "MODIFIER LETTER SMALL L WITH PALATAL HOOK\0"
  "MODIFIER LETTER SMALL L WITH RETROFLEX HOOK\0"
  "MODIFIER LETTER SMALL L WITH RETROFLEX HOOK AND BELT\0"
  "MODIFIER LETTER SMALL LEZH\0"
  "MODIFIER LETTER SMALL LEZH WITH RETROFLEX HOOK\0"
  "MODIFIER LETTER SMALL LIGATURE OE\0"
  "MODIFIER LETTER SMALL LS DIGRAPH\0"
  "MODIFIER LETTER SMALL LZ DIGRAPH\0"
  "MODIFIER LETTER SMALL M\0"
  "MODIFIER LETTER SMALL M WITH HOOK\0"
  "MODIFIER LETTER SMALL N WITH LEFT HOOK\0"
  "MODIFIER LETTER SMALL N WITH RETROFLEX HOOK\0"
  "MODIFIER LETTER SMALL O\0"
  "MODIFIER LETTER SMALL O WITH STROKE\0"
  "MODIFIER LETTER SMALL OPEN E\0"
  "MODIFIER LETTER SMALL OPEN O\0"
  "MODIFIER LETTER SMALL P\0"
  "MODIFIER LETTER SMALL PHI\0"
  "MODIFIER LETTER SMALL Q\0"
  "MODIFIER LETTER SMALL R\0"
  "MODIFIER LETTER SMALL R WITH FISHHOOK\0"
  "MODIFIER LETTER SMALL R WITH TAIL\0"
  "MODIFIER LETTER SMALL RAMS HORN\0"
  "MODIFIER LETTER SMALL REVERSED E\0"
  "MODIFIER LETTER SMALL REVERSED GLOTTAL STOP\0"
  "MODIFIER LETTER SMALL REVERSED OPEN E\0"
  "MODIFIER LETTER SMALL S\0"
  "MODIFIER LETTER SMALL S WITH CURL\0"
  "MODIFIER LETTER SMALL S WITH HOOK\0"
  "MODIFIER LETTER SMALL SCHWA\0"
  "MODIFIER LETTER SMALL SCRIPT G\0"
  "MODIFIER LETTER SMALL SIDEWAYS U\0"
  "MODIFIER LETTER SMALL T\0"
  "MODIFIER LETTER SMALL T WITH PALATAL HOOK\0"
  "MODIFIER LETTER SMALL T WITH RETROFLEX HOOK\0"
  "MODIFIER LETTER SMALL TC DIGRAPH WITH CURL\0"
  "MODIFIER LETTER SMALL TESH DIGRAPH\0"
  "MODIFIER LETTER SMALL THETA\0"
  "MODIFIER LETTER SMALL TOP HALF O\0"
  "MODIFIER LETTER SMALL TS DIGRAPH\0"
  "MODIFIER LETTER SMALL TS DIGRAPH WITH RETROFLEX HOOK\0"
  "MODIFIER LETTER SMALL TURNED A\0"
  "MODIFIER LETTER SMALL TURNED AE\0"
  "MODIFIER LETTER SMALL TURNED ALPHA\0"
  "MODIFIER LETTER SMALL TURNED H\0"
  "MODIFIER LETTER SMALL TURNED I\0"
  "MODIFIER LETTER SMALL TURNED M\0"
  "MODIFIER LETTER SMALL TURNED M WITH LONG LEG\0"
  "MODIFIER LETTER SMALL TURNED OPEN E\0"
  "MODIFIER LETTER SMALL TURNED R\0"
  "MODIFIER LETTER SMALL TURNED R WITH HOOK\0"
  "MODIFIER LETTER SMALL TURNED R WITH LONG LEG\0"
  "MODIFIER LETTER SMALL TURNED R WITH LONG LEG AND RETROFLEX HOOK\0"
  "MODIFIER LETTER SMALL TURNED V\0"
  "MODIFIER LETTER SMALL TURNED W\0"
  "MODIFIER LETTER SMALL TURNED Y\0"
  "MODIFIER LETTER SMALL TURNED Y WITH BELT\0"
  "MODIFIER LETTER SMALL U\0"
  "MODIFIER LETTER SMALL U BAR\0"
  "MODIFIER LETTER SMALL U WITH LEFT HOOK\0"
  "MODIFIER LETTER SMALL UPSILON\0"
  "MODIFIER LETTER SMALL V\0"
  "MODIFIER LETTER SMALL V WITH HOOK\0"
  "MODIFIER LETTER SMALL V WITH RIGHT HOOK\0"
  "MODIFIER LETTER SMALL W\0"
  "MODIFIER LETTER SMALL X\0"
  "MODIFIER LETTER SMALL Y\0"
  "MODIFIER LETTER SMALL Z\0"
  "MODIFIER LETTER SMALL Z WITH CURL\0"
  "MODIFIER LETTER SMALL Z WITH RETROFLEX HOOK\0"
  "MODIFIER LETTER STRESS AND HIGH TONE\0"
  "MODIFIER LETTER STRESS AND LOW TONE\0"
  "MODIFIER LETTER SUPERSCRIPT HALF TRIANGULAR COLON\0"
  "MODIFIER LETTER SUPERSCRIPT TRIANGULAR COLON\0"
  "MODIFIER LETTER TRIANGULAR COLON\0"
  "MODIFIER LETTER TURNED COMMA\0"
  "MODIFIER LETTER UNASPIRATED\0"
  "MODIFIER LETTER UP ARROWHEAD\0"
  "MODIFIER LETTER UP TACK\0"
  "MODIFIER LETTER US\0"
  "MODIFIER LETTER VERTICAL LINE\0"
  "MODIFIER LETTER VOICING\0"
  "MODIFIER LETTER YANG DEPARTING TONE MARK\0"
  "MODIFIER LETTER YIN DEPARTING TONE MARK\0"
  "MODULO TWO SUM\0"
  "MONEY BAG\0"
  "MONEY WITH WINGS\0"
  "MONEY-MOUTH FACE\0"
  "MONGOLIAN BIRGA\0"
  "MONGOLIAN BIRGA WITH DOUBLE ORNAMENT\0"
  "MONGOLIAN BIRGA WITH ORNAMENT\0"
  "MONGOLIAN COLON\0"
  "MONGOLIAN COMMA\0"
  "MONGOLIAN DIGIT EIGHT\0"
  "MONGOLIAN DIGIT FIVE\0"
  "MONGOLIAN DIGIT FOUR\0"
  "MONGOLIAN DIGIT NINE\0"
  "MONGOLIAN DIGIT ONE\0"
  "MONGOLIAN DIGIT SEVEN\0"
  "MONGOLIAN DIGIT SIX\0"
  "MONGOLIAN DIGIT THREE\0"
  "MONGOLIAN DIGIT TWO\0"
  "MONGOLIAN DIGIT ZERO\0"
  "MONGOLIAN DOUBLE BIRGA WITH ORNAMENT\0"
  "MONGOLIAN ELLIPSIS\0"
  "MONGOLIAN FOUR DOTS\0"
  "MONGOLIAN FREE VARIATION SELECTOR FOUR\0"
  "MONGOLIAN FREE VARIATION SELECTOR ONE\0"
  "MONGOLIAN FREE VARIATION SELECTOR THREE\0"
  "MONGOLIAN FREE VARIATION SELECTOR TWO\0"
  "MONGOLIAN FULL STOP\0"
  "MONGOLIAN INVERTED BIRGA\0"
  "MONGOLIAN INVERTED BIRGA WITH DOUBLE ORNAMENT\0"
  "MONGOLIAN LETTER A\0"
  "MONGOLIAN LETTER ALI GALI A\0"
  "MONGOLIAN LETTER ALI GALI AH\0"
  "MONGOLIAN LETTER ALI GALI ANUSVARA ONE\0"
  "MONGOLIAN LETTER ALI GALI BALUDA\0"
  "MONGOLIAN LETTER ALI GALI CA\0"
  "MONGOLIAN LETTER ALI GALI DA\0"
  "MONGOLIAN LETTER ALI GALI DAGALGA\0"
  "MONGOLIAN LETTER ALI GALI DAMARU\0"
  "MONGOLIAN LETTER ALI GALI DDA\0"
  "MONGOLIAN LETTER ALI GALI HALF U\0"
  "MONGOLIAN LETTER ALI GALI HALF YA\0"
  "MONGOLIAN LETTER ALI GALI I\0"
  "MONGOLIAN LETTER ALI GALI INVERTED UBADAMA\0"
  "MONGOLIAN LETTER ALI GALI KA\0"
  "MONGOLIAN LETTER ALI GALI NGA\0"
  "MONGOLIAN LETTER ALI GALI NNA\0"
  "MONGOLIAN LETTER ALI GALI PA\0"
  "MONGOLIAN LETTER ALI GALI PHA\0"
  "MONGOLIAN LETTER ALI GALI SSA\0"
  "MONGOLIAN LETTER ALI GALI TA\0"
  "MONGOLIAN LETTER ALI GALI THREE BALUDA\0"
  "MONGOLIAN LETTER ALI GALI TTA\0"
  "MONGOLIAN LETTER ALI GALI TTHA\0"
  "MONGOLIAN LETTER ALI GALI UBADAMA\0"
  "MONGOLIAN LETTER ALI GALI VISARGA ONE\0"
  "MONGOLIAN LETTER ALI GALI ZA\0"
  "MONGOLIAN LETTER ALI GALI ZHA\0"
  "MONGOLIAN LETTER ANG\0"
  "MONGOLIAN LETTER BA\0"
  "MONGOLIAN LETTER CHA\0"
  "MONGOLIAN LETTER CHA WITH TWO DOTS\0"
  "MONGOLIAN LETTER CHI\0"
  "MONGOLIAN LETTER DA\0"
  "MONGOLIAN LETTER E\0"
  "MONGOLIAN LETTER EE\0"
  "MONGOLIAN LETTER FA\0"
  "MONGOLIAN LETTER GA\0"
  "MONGOLIAN LETTER HAA\0"
  "MONGOLIAN LETTER I\0"
  "MONGOLIAN LETTER JA\0"
  "MONGOLIAN LETTER KA\0"
  "MONGOLIAN LETTER KHA\0"
  "MONGOLIAN LETTER LA\0"
  "MONGOLIAN LETTER LHA\0"
  "MONGOLIAN LETTER MA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI BHA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI CA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI CYA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI DDHA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI DHA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI GHA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI JHA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI LHA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI NGA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI SSA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI TA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI TTA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI ZA\0"
  "MONGOLIAN LETTER MANCHU ALI GALI ZHA\0"
  "MONGOLIAN LETTER MANCHU FA\0"
  "MONGOLIAN LETTER MANCHU I\0"
  "MONGOLIAN LETTER MANCHU KA\0"
  "MONGOLIAN LETTER MANCHU RA\0"
  "MONGOLIAN LETTER MANCHU ZHA\0"
  "MONGOLIAN LETTER NA\0"
  "MONGOLIAN LETTER O\0"
  "MONGOLIAN LETTER OE\0"
  "MONGOLIAN LETTER PA\0"
  "MONGOLIAN LETTER QA\0"
  "MONGOLIAN LETTER RA\0"
  "MONGOLIAN LETTER SA\0"
  "MONGOLIAN LETTER SHA\0"
  "MONGOLIAN LETTER SIBE ANG\0"
  "MONGOLIAN LETTER SIBE CHA\0"
  "MONGOLIAN LETTER SIBE DA\0"
  "MONGOLIAN LETTER SIBE E\0"
  "MONGOLIAN LETTER SIBE FA\0"
  "MONGOLIAN LETTER SIBE GA\0"
  "MONGOLIAN LETTER SIBE GAA\0"
  "MONGOLIAN LETTER SIBE HA\0"
  "MONGOLIAN LETTER SIBE HAA\0"
  "MONGOLIAN LETTER SIBE I\0"
  "MONGOLIAN LETTER SIBE IY\0"
  "MONGOLIAN LETTER SIBE JA\0"
  "MONGOLIAN LETTER SIBE KA\0"
  "MONGOLIAN LETTER SIBE PA\0"
  "MONGOLIAN LETTER SIBE RAA\0"
  "MONGOLIAN LETTER SIBE SHA\0"
  "MONGOLIAN LETTER SIBE TA\0"
  "MONGOLIAN LETTER SIBE TSA\0"
  "MONGOLIAN LETTER SIBE U\0"
  "MONGOLIAN LETTER SIBE UE\0"
  "MONGOLIAN LETTER SIBE ZA\0"
  "MONGOLIAN LETTER SIBE ZHA\0"
  "MONGOLIAN LETTER TA\0"
  "MONGOLIAN LETTER TODO ALI GALI TA\0"
  "MONGOLIAN LETTER TODO ALI GALI ZHA\0"
  "MONGOLIAN LETTER TODO ANG\0"
  "MONGOLIAN LETTER TODO BA\0"
  "MONGOLIAN LETTER TODO CHA\0"
  "MONGOLIAN LETTER TODO DA\0"
  "MONGOLIAN LETTER TODO DZA\0"
  "MONGOLIAN LETTER TODO E\0"
  "MONGOLIAN LETTER TODO GA\0"
  "MONGOLIAN LETTER TODO GAA\0"
  "MONGOLIAN LETTER TODO HAA\0"
  "MONGOLIAN LETTER TODO I\0"
  "MONGOLIAN LETTER TODO JA\0"
  "MONGOLIAN LETTER TODO JIA\0"
  "MONGOLIAN LETTER TODO KA\0"
  "MONGOLIAN LETTER TODO LONG VOWEL SIGN\0"
  "MONGOLIAN LETTER TODO MA\0"
  "MONGOLIAN LETTER TODO NIA\0"
  "MONGOLIAN LETTER TODO O\0"
  "MONGOLIAN LETTER TODO OE\0"
  "MONGOLIAN LETTER TODO PA\0"
  "MONGOLIAN LETTER TODO QA\0"
  "MONGOLIAN LETTER TODO TA\0"
  "MONGOLIAN LETTER TODO TSA\0"
  "MONGOLIAN LETTER TODO U\0"
  "MONGOLIAN LETTER TODO UE\0"
  "MONGOLIAN LETTER TODO WA\0"
  "MONGOLIAN LETTER TODO YA\0"
  "MONGOLIAN LETTER TSA\0"
  "MONGOLIAN LETTER U\0"
  "MONGOLIAN LETTER UE\0"
  "MONGOLIAN LETTER WA\0"
  "MONGOLIAN LETTER YA\0"
  "MONGOLIAN LETTER ZA\0"
  "MONGOLIAN LETTER ZHI\0"
  "MONGOLIAN LETTER ZRA\0"
  "MONGOLIAN MANCHU COMMA\0"
  "MONGOLIAN MANCHU FULL STOP\0"
  "MONGOLIAN NIRUGU\0"
  "MONGOLIAN ROTATED BIRGA\0"
  "MONGOLIAN ROTATED BIRGA WITH DOUBLE ORNAMENT\0"
  "MONGOLIAN ROTATED BIRGA WITH ORNAMENT\0"
  "MONGOLIAN SIBE SYLLABLE BOUNDARY MARKER\0"
  "MONGOLIAN SWIRL BIRGA\0"
  "MONGOLIAN SWIRL BIRGA WITH DOUBLE ORNAMENT\0"
  "MONGOLIAN SWIRL BIRGA WITH ORNAMENT\0"
  "MONGOLIAN TODO SOFT HYPHEN\0"
  "MONGOLIAN TRIPLE BIRGA WITH ORNAMENT\0"
  "MONGOLIAN TURNED SWIRL BIRGA WITH DOUBLE ORNAMENT\0"
  "MONGOLIAN VOWEL SEPARATOR\0"
  "MONKEY\0"
  "MONKEY FACE\0"
  "MONOGRAM FOR EARTH\0"
  "MONOGRAM FOR YANG\0"
  "MONOGRAM FOR YIN\0"
  "MONORAIL\0"
  "MONOSTABLE SYMBOL\0"
  "MOOD BUBBLE\0"
  "MOON CAKE\0"
  "MOON LANDER\0"
  "MOON VIEWING CEREMONY\0"
  "MOOSE\0"
  "MOSQUE\0"
  "MOSQUITO\0"
  "MOTHER CHRISTMAS\0"
  "MOTOR BOAT\0"
  "MOTOR SCOOTER\0"
  "MOTORIZED WHEELCHAIR\0"
  "MOTORWAY\0"
  "MOUNT FUJI\0"
  "MOUNTAIN\0"
  "MOUNTAIN BICYCLIST\0"
  "MOUNTAIN CABLEWAY\0"
  "MOUNTAIN RAILWAY\0"
  "MOUSE\0"
  "MOUSE FACE\0"
  "MOUSE TRAP\0"
  "MOUTH\0"
  "MOVIE CAMERA\0"
  "MOYAI\0"
  "MRO DANDA\0"
  "MRO DIGIT EIGHT\0"
  "MRO DIGIT FIVE\0"
  "MRO DIGIT FOUR\0"
  "MRO DIGIT NINE\0"
  "MRO DIGIT ONE\0"
  "MRO DIGIT SEVEN\0"
  "MRO DIGIT SIX\0"
  "MRO DIGIT THREE\0"
  "MRO DIGIT TWO\0"
  "MRO DIGIT ZERO\0"
  "MRO DOUBLE DANDA\0"
  "MRO LETTER A\0"
  "MRO LETTER BA\0"
  "MRO LETTER CHU\0"
  "MRO LETTER DA\0"
  "MRO LETTER DAI\0"
  "MRO LETTER E\0"
  "MRO LETTER EA\0"
  "MRO LETTER HAI\0"
  "MRO LETTER HAO\0"
  "MRO LETTER KEAAE\0"
  "MRO LETTER KHAI\0"
  "MRO LETTER KO\0"
  "MRO LETTER LA\0"
  "MRO LETTER LAN\0"
  "MRO LETTER MAEM\0"
  "MRO LETTER MIM\0"
  "MRO LETTER NGI\0"
  "MRO LETTER NIN\0"
  "MRO LETTER O\0"
  "MRO LETTER OL\0"
  "MRO LETTER OO\0"
  "MRO LETTER PA\0"
  "MRO LETTER PHI\0"
  "MRO LETTER RI\0"
  "MRO LETTER RO\0"
  "MRO LETTER SHI\0"
  "MRO LETTER TA\0"
  "MRO LETTER TEK\0"
  "MRO LETTER THEA\0"
  "MRO LETTER WA\0"
  "MRO LETTER YO\0"
  "MUCH GREATER-THAN\0"
  "MUCH LESS-THAN\0"
  "MULTANI LETTER A\0"
  "MULTANI LETTER BA\0"
  "MULTANI LETTER BHA\0"
  "MULTANI LETTER CA\0"
  "MULTANI LETTER CHA\0"
  "MULTANI LETTER DA\0"
  "MULTANI LETTER DDA\0"
  "MULTANI LETTER DDDA\0"
  "MULTANI LETTER DDHA\0"
  "MULTANI LETTER DHA\0"
  "MULTANI LETTER E\0"
  "MULTANI LETTER GA\0"
  "MULTANI LETTER GHA\0"
  "MULTANI LETTER HA\0"
  "MULTANI LETTER I\0"
  "MULTANI LETTER JA\0"
  "MULTANI LETTER JJA\0"
  "MULTANI LETTER KA\0"
  "MULTANI LETTER KHA\0"
  "MULTANI LETTER LA\0"
  "MULTANI LETTER MA\0"
  "MULTANI LETTER NA\0"
  "MULTANI LETTER NNA\0"
  "MULTANI LETTER NYA\0"
  "MULTANI LETTER PA\0"
  "MULTANI LETTER PHA\0"
  "MULTANI LETTER RA\0"
  "MULTANI LETTER RHA\0"
  "MULTANI LETTER RRA\0"
  "MULTANI LETTER SA\0"
  "MULTANI LETTER TA\0"
  "MULTANI LETTER THA\0"
  "MULTANI LETTER TTA\0"
  "MULTANI LETTER TTHA\0"
  "MULTANI LETTER U\0"
  "MULTANI LETTER VA\0"
  "MULTANI LETTER YA\0"
  "MULTANI SECTION MARK\0"
  "MULTIMAP\0"
  "MULTIPLE MUSICAL NOTES\0"
  "MULTIPLICATION SIGN\0"
  "MULTIPLICATION SIGN IN DOUBLE CIRCLE\0"
  "MULTIPLICATION SIGN IN LEFT HALF CIRCLE\0"
  "MULTIPLICATION SIGN IN RIGHT HALF CIRCLE\0"
  "MULTIPLICATION SIGN IN TRIANGLE\0"
  "MULTIPLICATION SIGN WITH DOT ABOVE\0"
  "MULTIPLICATION SIGN WITH UNDERBAR\0"
  "MULTIPLICATION X\0"
  "MULTISET\0"
  "MULTISET MULTIPLICATION\0"
  "MULTISET UNION\0"
  "MUSHROOM\0"
  "MUSHROOM CLOUD\0"
  "MUSIC FLAT SIGN\0"
  "MUSIC NATURAL SIGN\0"
  "MUSIC SHARP SIGN\0"
  "MUSICAL KEYBOARD\0"
  "MUSICAL KEYBOARD WITH JACKS\0"
  "MUSICAL NOTE\0"
  "MUSICAL SCORE\0"
  "MUSICAL SYMBOL ARPEGGIATO DOWN\0"
  "MUSICAL SYMBOL ARPEGGIATO UP\0"
  "MUSICAL SYMBOL BEGIN BEAM\0"
  "MUSICAL SYMBOL BEGIN PHRASE\0"
  "MUSICAL SYMBOL BEGIN SLUR\0"
  "MUSICAL SYMBOL BEGIN TIE\0"
  "MUSICAL SYMBOL BRACE\0"
  "MUSICAL SYMBOL BRACKET\0"
  "MUSICAL SYMBOL BREATH MARK\0"
  "MUSICAL SYMBOL BREVE\0"
  "MUSICAL SYMBOL BREVIS\0"
  "MUSICAL SYMBOL BREVIS REST\0"
  "MUSICAL SYMBOL C CLEF\0"
  "MUSICAL SYMBOL CAESURA\0"
  "MUSICAL SYMBOL CIRCLE X NOTEHEAD\0"
  "MUSICAL SYMBOL CLIMACUS\0"
  "MUSICAL SYMBOL CLIVIS\0"
  "MUSICAL SYMBOL CLUSTER NOTEHEAD BLACK\0"
  "MUSICAL SYMBOL CLUSTER NOTEHEAD WHITE\0"
  "MUSICAL SYMBOL CODA\0"
  "MUSICAL SYMBOL COMBINING ACCENT\0"
  "MUSICAL SYMBOL COMBINING ACCENT-STACCATO\0"
  "MUSICAL SYMBOL COMBINING AUGMENTATION DOT\0"
  "MUSICAL SYMBOL COMBINING BEND\0"
  "MUSICAL SYMBOL COMBINING DOIT\0"
  "MUSICAL SYMBOL COMBINING DOUBLE TONGUE\0"
  "MUSICAL SYMBOL COMBINING DOWN BOW\0"
  "MUSICAL SYMBOL COMBINING FLAG-1\0"
  "MUSICAL SYMBOL COMBINING FLAG-2\0"
  "MUSICAL SYMBOL COMBINING FLAG-3\0"
  "MUSICAL SYMBOL COMBINING FLAG-4\0"
  "MUSICAL SYMBOL COMBINING FLAG-5\0"
  "MUSICAL SYMBOL COMBINING FLIP\0"
  "MUSICAL SYMBOL COMBINING HARMONIC\0"
  "MUSICAL SYMBOL COMBINING LOURE\0"
  "MUSICAL SYMBOL COMBINING MARCATO\0"
  "MUSICAL SYMBOL COMBINING MARCATO-STACCATO\0"
  "MUSICAL SYMBOL COMBINING RIP\0"
  "MUSICAL SYMBOL COMBINING SMEAR\0"
  "MUSICAL SYMBOL COMBINING SNAP PIZZICATO\0"
  "MUSICAL SYMBOL COMBINING SPRECHGESANG STEM\0"
  "MUSICAL SYMBOL COMBINING STACCATISSIMO\0"
  "MUSICAL SYMBOL COMBINING STACCATO\0"
  "MUSICAL SYMBOL COMBINING STEM\0"
  "MUSICAL SYMBOL COMBINING TENUTO\0"
  "MUSICAL SYMBOL COMBINING TREMOLO-1\0"
  "MUSICAL SYMBOL COMBINING TREMOLO-2\0"
  "MUSICAL SYMBOL COMBINING TREMOLO-3\0"
  "MUSICAL SYMBOL COMBINING TRIPLE TONGUE\0"
  "MUSICAL SYMBOL COMBINING UP BOW\0"
  "MUSICAL SYMBOL COMMON TIME\0"
  "MUSICAL SYMBOL CRESCENDO\0"
  "MUSICAL SYMBOL CROIX\0"
  "MUSICAL SYMBOL CUT TIME\0"
  "MUSICAL SYMBOL DA CAPO\0"
  "MUSICAL SYMBOL DAL SEGNO\0"
  "MUSICAL SYMBOL DAMP\0"
  "MUSICAL SYMBOL DAMP ALL\0"
  "MUSICAL SYMBOL DASHED BARLINE\0"
  "MUSICAL SYMBOL DECRESCENDO\0"
  "MUSICAL SYMBOL DEGREE SLASH\0"
  "MUSICAL SYMBOL DOUBLE BARLINE\0"
  "MUSICAL SYMBOL DOUBLE FLAT\0"
  "MUSICAL SYMBOL DOUBLE SHARP\0"
  "MUSICAL SYMBOL DRUM CLEF-1\0"
  "MUSICAL SYMBOL DRUM CLEF-2\0"
  "MUSICAL SYMBOL EIGHTH NOTE\0"
  "MUSICAL SYMBOL EIGHTH REST\0"
  "MUSICAL SYMBOL END BEAM\0"
  "MUSICAL SYMBOL END OF STIMME\0"
  "MUSICAL SYMBOL END PHRASE\0"
  "MUSICAL SYMBOL END SLUR\0"
  "MUSICAL SYMBOL END TIE\0"
  "MUSICAL SYMBOL F CLEF\0"
  "MUSICAL SYMBOL F CLEF OTTAVA ALTA\0"
  "MUSICAL SYMBOL F CLEF OTTAVA BASSA\0"
  "MUSICAL SYMBOL FERMATA\0"
  "MUSICAL SYMBOL FERMATA BELOW\0"
  "MUSICAL SYMBOL FINAL BARLINE\0"
  "MUSICAL SYMBOL FINGERED TREMOLO-1\0"
  "MUSICAL SYMBOL FINGERED TREMOLO-2\0"
  "MUSICAL SYMBOL FINGERED TREMOLO-3\0"
  "MUSICAL SYMBOL FIVE-LINE STAFF\0"
  "MUSICAL SYMBOL FLAT DOWN\0"
  "MUSICAL SYMBOL FLAT UP\0"
  "MUSICAL SYMBOL FORTE\0"
  "MUSICAL SYMBOL FOUR-LINE STAFF\0"
  "MUSICAL SYMBOL FOUR-STRING FRETBOARD\0"
  "MUSICAL SYMBOL FUSA BLACK\0"
  "MUSICAL SYMBOL FUSA WHITE\0"
  "MUSICAL SYMBOL G CLEF\0"
  "MUSICAL SYMBOL G CLEF OTTAVA ALTA\0"
  "MUSICAL SYMBOL G CLEF OTTAVA BASSA\0"
  "MUSICAL SYMBOL GLISSANDO DOWN\0"
  "MUSICAL SYMBOL GLISSANDO UP\0"
  "MUSICAL SYMBOL GRACE NOTE NO SLASH\0"
  "MUSICAL SYMBOL GRACE NOTE SLASH\0"
  "MUSICAL SYMBOL GREGORIAN C CLEF\0"
  "MUSICAL SYMBOL GREGORIAN F CLEF\0"
  "MUSICAL SYMBOL HALF NOTE\0"
  "MUSICAL SYMBOL HALF PEDAL MARK\0"
  "MUSICAL SYMBOL HALF REST\0"
  "MUSICAL SYMBOL HAUPTSTIMME\0"
  "MUSICAL SYMBOL INVERTED TURN\0"
  "MUSICAL SYMBOL KIEVAN C CLEF\0"
  "MUSICAL SYMBOL KIEVAN EIGHTH NOTE STEM DOWN\0"
  "MUSICAL SYMBOL KIEVAN EIGHTH NOTE STEM UP\0"
  "MUSICAL SYMBOL KIEVAN END OF PIECE\0"
  "MUSICAL SYMBOL KIEVAN FINAL NOTE\0"
  "MUSICAL SYMBOL KIEVAN FLAT SIGN\0"
  "MUSICAL SYMBOL KIEVAN HALF NOTE\0"
  "MUSICAL SYMBOL KIEVAN QUARTER NOTE STEM DOWN\0"
  "MUSICAL SYMBOL KIEVAN QUARTER NOTE STEM UP\0"
  "MUSICAL SYMBOL KIEVAN RECITATIVE MARK\0"
  "MUSICAL SYMBOL KIEVAN WHOLE NOTE\0"
  "MUSICAL SYMBOL KORON\0"
  "MUSICAL SYMBOL LEFT REPEAT SIGN\0"
  "MUSICAL SYMBOL LONGA\0"
  "MUSICAL SYMBOL LONGA IMPERFECTA REST\0"
  "MUSICAL SYMBOL LONGA PERFECTA REST\0"
  "MUSICAL SYMBOL MAXIMA\0"
  "MUSICAL SYMBOL MEZZO\0"
  "MUSICAL SYMBOL MINIMA\0"
  "MUSICAL SYMBOL MINIMA BLACK\0"
  "MUSICAL SYMBOL MINIMA REST\0"
  "MUSICAL SYMBOL MOON NOTEHEAD BLACK\0"
  "MUSICAL SYMBOL MOON NOTEHEAD WHITE\0"
  "MUSICAL SYMBOL MULTI REST\0"
  "MUSICAL SYMBOL MULTIPLE MEASURE REST\0"
  "MUSICAL SYMBOL NATURAL DOWN\0"
  "MUSICAL SYMBOL NATURAL UP\0"
  "MUSICAL SYMBOL NEBENSTIMME\0"
  "MUSICAL SYMBOL NOTEHEAD BLACK\0"
  "MUSICAL SYMBOL NULL NOTEHEAD\0"
  "MUSICAL SYMBOL ONE HUNDRED TWENTY-EIGHTH NOTE\0"
  "MUSICAL SYMBOL ONE HUNDRED TWENTY-EIGHTH REST\0"
  "MUSICAL SYMBOL ONE-LINE STAFF\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-1\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-10\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-11\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-2\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-3\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-4\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-5\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-6\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-7\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-8\0"
  "MUSICAL SYMBOL ORNAMENT STROKE-9\0"
  "MUSICAL SYMBOL OTTAVA ALTA\0"
  "MUSICAL SYMBOL OTTAVA BASSA\0"
  "MUSICAL SYMBOL PARENTHESIS NOTEHEAD\0"
  "MUSICAL SYMBOL PEDAL MARK\0"
  "MUSICAL SYMBOL PEDAL UP MARK\0"
  "MUSICAL SYMBOL PES SUBPUNCTIS\0"
  "MUSICAL SYMBOL PIANO\0"
  "MUSICAL SYMBOL PLUS NOTEHEAD\0"
  "MUSICAL SYMBOL PODATUS\0"
  "MUSICAL SYMBOL PORRECTUS\0"
  "MUSICAL SYMBOL PORRECTUS FLEXUS\0"
  "MUSICAL SYMBOL QUARTER NOTE\0"
  "MUSICAL SYMBOL QUARTER REST\0"
  "MUSICAL SYMBOL QUARTER TONE FLAT\0"
  "MUSICAL SYMBOL QUARTER TONE SHARP\0"
  "MUSICAL SYMBOL QUINDICESIMA ALTA\0"
  "MUSICAL SYMBOL QUINDICESIMA BASSA\0"
  "MUSICAL SYMBOL REPEAT DOTS\0"
  "MUSICAL SYMBOL REPEATED FIGURE-1\0"
  "MUSICAL SYMBOL REPEATED FIGURE-2\0"
  "MUSICAL SYMBOL REPEATED FIGURE-3\0"
  "MUSICAL SYMBOL REVERSE FINAL BARLINE\0"
  "MUSICAL SYMBOL RIGHT REPEAT SIGN\0"
  "MUSICAL SYMBOL RINFORZANDO\0"
  "MUSICAL SYMBOL SCANDICUS\0"
  "MUSICAL SYMBOL SCANDICUS FLEXUS\0"
  "MUSICAL SYMBOL SEGNO\0"
  "MUSICAL SYMBOL SEMIBREVIS BLACK\0"
  "MUSICAL SYMBOL SEMIBREVIS REST\0"
  "MUSICAL SYMBOL SEMIBREVIS WHITE\0"
  "MUSICAL SYMBOL SEMIMINIMA BLACK\0"
  "MUSICAL SYMBOL SEMIMINIMA REST\0"
  "MUSICAL SYMBOL SEMIMINIMA WHITE\0"
  "MUSICAL SYMBOL SHARP DOWN\0"
  "MUSICAL SYMBOL SHARP UP\0"
  "MUSICAL SYMBOL SHORT BARLINE\0"
  "MUSICAL SYMBOL SINGLE BARLINE\0"
  "MUSICAL SYMBOL SIX-LINE STAFF\0"
  "MUSICAL SYMBOL SIX-STRING FRETBOARD\0"
  "MUSICAL SYMBOL SIXTEENTH NOTE\0"
  "MUSICAL SYMBOL SIXTEENTH REST\0"
  "MUSICAL SYMBOL SIXTY-FOURTH NOTE\0"
  "MUSICAL SYMBOL SIXTY-FOURTH REST\0"
  "MUSICAL SYMBOL SORI\0"
  "MUSICAL SYMBOL SQUARE B\0"
  "MUSICAL SYMBOL SQUARE NOTEHEAD BLACK\0"
  "MUSICAL SYMBOL SQUARE NOTEHEAD WHITE\0"
  "MUSICAL SYMBOL SUBITO\0"
  "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA\0"
  "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA DIMINUTION-1\0"
  "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA DIMINUTION-2\0"
  "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA DIMINUTION-3\0"
  "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE PERFECTA\0"
  "MUSICAL SYMBOL TEMPUS PERFECTUM CUM PROLATIONE IMPERFECTA\0"
  "MUSICAL SYMBOL TEMPUS PERFECTUM CUM PROLATIONE PERFECTA\0"
  "MUSICAL SYMBOL TEMPUS PERFECTUM CUM PROLATIONE PERFECTA DIMINUTION-1\0"
  "MUSICAL SYMBOL THIRTY-SECOND NOTE\0"
  "MUSICAL SYMBOL THIRTY-SECOND REST\0"
  "MUSICAL SYMBOL THREE-LINE STAFF\0"
  "MUSICAL SYMBOL TORCULUS\0"
  "MUSICAL SYMBOL TORCULUS RESUPINUS\0"
  "MUSICAL SYMBOL TR\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD DOWN BLACK\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD DOWN WHITE\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD LEFT BLACK\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD LEFT WHITE\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD RIGHT BLACK\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD RIGHT WHITE\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD UP BLACK\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD UP RIGHT BLACK\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD UP RIGHT WHITE\0"
  "MUSICAL SYMBOL TRIANGLE NOTEHEAD UP WHITE\0"
  "MUSICAL SYMBOL TRIANGLE-ROUND NOTEHEAD DOWN BLACK\0"
  "MUSICAL SYMBOL TRIANGLE-ROUND NOTEHEAD DOWN WHITE\0"
  "MUSICAL SYMBOL TURN\0"
  "MUSICAL SYMBOL TURN SLASH\0"
  "MUSICAL SYMBOL TURN UP\0"
  "MUSICAL SYMBOL TWO-LINE STAFF\0"
  "MUSICAL SYMBOL VIRGA\0"
  "MUSICAL SYMBOL VOID NOTEHEAD\0"
  "MUSICAL SYMBOL WHOLE NOTE\0"
  "MUSICAL SYMBOL WHOLE REST\0"
  "MUSICAL SYMBOL WITH FINGERNAILS\0"
  "MUSICAL SYMBOL X NOTEHEAD\0"
  "MUSICAL SYMBOL Z\0"
  "MYANMAR CONSONANT SIGN MEDIAL HA\0"
  "MYANMAR CONSONANT SIGN MEDIAL RA\0"
  "MYANMAR CONSONANT SIGN MEDIAL WA\0"
  "MYANMAR CONSONANT SIGN MEDIAL YA\0"
  "MYANMAR CONSONANT SIGN MON MEDIAL LA\0"
  "MYANMAR CONSONANT SIGN MON MEDIAL MA\0"
  "MYANMAR CONSONANT SIGN MON MEDIAL NA\0"
  "MYANMAR CONSONANT SIGN SHAN MEDIAL WA\0"
  "MYANMAR DIGIT EIGHT\0"
  "MYANMAR DIGIT FIVE\0"
  "MYANMAR DIGIT FOUR\0"
  "MYANMAR DIGIT NINE\0"
  "MYANMAR DIGIT ONE\0"
  "MYANMAR DIGIT SEVEN\0"
  "MYANMAR DIGIT SIX\0"
  "MYANMAR DIGIT THREE\0"
  "MYANMAR DIGIT TWO\0"
  "MYANMAR DIGIT ZERO\0"
  "MYANMAR EASTERN PWO KAREN DIGIT EIGHT\0"
  "MYANMAR EASTERN PWO KAREN DIGIT FIVE\0"
  "MYANMAR EASTERN PWO KAREN DIGIT FOUR\0"
  "MYANMAR EASTERN PWO KAREN DIGIT NINE\0"
  "MYANMAR EASTERN PWO KAREN DIGIT ONE\0"
  "MYANMAR EASTERN PWO KAREN DIGIT SEVEN\0"
  "MYANMAR EASTERN PWO KAREN DIGIT SIX\0"
  "MYANMAR EASTERN PWO KAREN DIGIT THREE\0"
  "MYANMAR EASTERN PWO KAREN DIGIT TWO\0"
  "MYANMAR EASTERN PWO KAREN DIGIT ZERO\0"
  "MYANMAR LETTER A\0"
  "MYANMAR LETTER AITON RA\0"
  "MYANMAR LETTER AU\0"
  "MYANMAR LETTER BA\0"
  "MYANMAR LETTER BHA\0"
  "MYANMAR LETTER CA\0"
  "MYANMAR LETTER CHA\0"
  "MYANMAR LETTER DA\0"
  "MYANMAR LETTER DDA\0"
  "MYANMAR LETTER DDHA\0"
  "MYANMAR LETTER DHA\0"
  "MYANMAR LETTER E\0"
  "MYANMAR LETTER EASTERN PWO KAREN GHWA\0"
  "MYANMAR LETTER EASTERN PWO KAREN NNA\0"
  "MYANMAR LETTER EASTERN PWO KAREN YWA\0"
  "MYANMAR LETTER GA\0"
  "MYANMAR LETTER GHA\0"
  "MYANMAR LETTER GREAT SA\0"
  "MYANMAR LETTER HA\0"
  "MYANMAR LETTER I\0"
  "MYANMAR LETTER II\0"
  "MYANMAR LETTER JA\0"
  "MYANMAR LETTER JHA\0"
  "MYANMAR LETTER KA\0"
  "MYANMAR LETTER KHA\0"
  "MYANMAR LETTER KHAMTI CA\0"
  "MYANMAR LETTER KHAMTI CHA\0"
  "MYANMAR LETTER KHAMTI DDA\0"
  "MYANMAR LETTER KHAMTI DDHA\0"
  "MYANMAR LETTER KHAMTI DHA\0"
  "MYANMAR LETTER KHAMTI FA\0"
  "MYANMAR LETTER KHAMTI GA\0"
  "MYANMAR LETTER KHAMTI HA\0"
  "MYANMAR LETTER KHAMTI HHA\0"
  "MYANMAR LETTER KHAMTI JA\0"
  "MYANMAR LETTER KHAMTI JHA\0"
  "MYANMAR LETTER KHAMTI NA\0"
  "MYANMAR LETTER KHAMTI NYA\0"
  "MYANMAR LETTER KHAMTI RA\0"
  "MYANMAR LETTER KHAMTI SA\0"
  "MYANMAR LETTER KHAMTI TTA\0"
  "MYANMAR LETTER KHAMTI TTHA\0"
  "MYANMAR LETTER KHAMTI XA\0"
  "MYANMAR LETTER KHAMTI ZA\0"
  "MYANMAR LETTER LA\0"
  "MYANMAR LETTER LLA\0"
  "MYANMAR LETTER MA\0"
  "MYANMAR LETTER MON BBA\0"
  "MYANMAR LETTER MON BBE\0"
  "MYANMAR LETTER MON E\0"
  "MYANMAR LETTER MON JHA\0"
  "MYANMAR LETTER MON NGA\0"
  "MYANMAR LETTER NA\0"
  "MYANMAR LETTER NGA\0"
  "MYANMAR LETTER NNA\0"
  "MYANMAR LETTER NNYA\0"
  "MYANMAR LETTER NYA\0"
  "MYANMAR LETTER O\0"
  "MYANMAR LETTER PA\0"
  "MYANMAR LETTER PHA\0"
  "MYANMAR LETTER RA\0"
  "MYANMAR LETTER RUMAI PALAUNG FA\0"
  "MYANMAR LETTER SA\0"
  "MYANMAR LETTER SGAW KAREN SHA\0"
  "MYANMAR LETTER SHA\0"
  "MYANMAR LETTER SHAN A\0"
  "MYANMAR LETTER SHAN BA\0"
  "MYANMAR LETTER SHAN BHA\0"
  "MYANMAR LETTER SHAN CA\0"
  "MYANMAR LETTER SHAN CHA\0"
  "MYANMAR LETTER SHAN DA\0"
  "MYANMAR LETTER SHAN FA\0"
  "MYANMAR LETTER SHAN GA\0"
  "MYANMAR LETTER SHAN GHA\0"
  "MYANMAR LETTER SHAN HA\0"
  "MYANMAR LETTER SHAN JHA\0"
  "MYANMAR LETTER SHAN KA\0"
  "MYANMAR LETTER SHAN KHA\0"
  "MYANMAR LETTER SHAN NA\0"
  "MYANMAR LETTER SHAN NNA\0"
  "MYANMAR LETTER SHAN NYA\0"
  "MYANMAR LETTER SHAN PHA\0"
  "MYANMAR LETTER SHAN THA\0"
  "MYANMAR LETTER SHAN ZA\0"
  "MYANMAR LETTER SHWE PALAUNG CHA\0"
  "MYANMAR LETTER SHWE PALAUNG SHA\0"
  "MYANMAR LETTER SSA\0"
  "MYANMAR LETTER TA\0"
  "MYANMAR LETTER TAI LAING BA\0"
  "MYANMAR LETTER TAI LAING BHA\0"
  "MYANMAR LETTER TAI LAING DA\0"
  "MYANMAR LETTER TAI LAING DDA\0"
  "MYANMAR LETTER TAI LAING DDHA\0"
  "MYANMAR LETTER TAI LAING DHA\0"
  "MYANMAR LETTER TAI LAING FA\0"
  "MYANMAR LETTER TAI LAING GA\0"
  "MYANMAR LETTER TAI LAING GHA\0"
  "MYANMAR LETTER TAI LAING JA\0"
  "MYANMAR LETTER TAI LAING JHA\0"
  "MYANMAR LETTER TAI LAING LLA\0"
  "MYANMAR LETTER TAI LAING NNA\0"
  "MYANMAR LETTER TAI LAING NYA\0"
  "MYANMAR LETTER THA\0"
  "MYANMAR LETTER TTA\0"
  "MYANMAR LETTER TTHA\0"
  "MYANMAR LETTER U\0"
  "MYANMAR LETTER UU\0"
  "MYANMAR LETTER VOCALIC L\0"
  "MYANMAR LETTER VOCALIC LL\0"
  "MYANMAR LETTER VOCALIC R\0"
  "MYANMAR LETTER VOCALIC RR\0"
  "MYANMAR LETTER WA\0"
  "MYANMAR LETTER WESTERN PWO KAREN PWA\0"
  "MYANMAR LETTER WESTERN PWO KAREN THA\0"
  "MYANMAR LETTER YA\0"
  "MYANMAR LOGOGRAM KHAMTI HM\0"
  "MYANMAR LOGOGRAM KHAMTI OAY\0"
  "MYANMAR LOGOGRAM KHAMTI QN\0"
  "MYANMAR MODIFIER LETTER KHAMTI REDUPLICATION\0"
  "MYANMAR MODIFIER LETTER SHAN REDUPLICATION\0"
  "MYANMAR PAO DIGIT EIGHT\0"
  "MYANMAR PAO DIGIT FIVE\0"
  "MYANMAR PAO DIGIT FOUR\0"
  "MYANMAR PAO DIGIT NINE\0"
  "MYANMAR PAO DIGIT ONE\0"
  "MYANMAR PAO DIGIT SEVEN\0"
  "MYANMAR PAO DIGIT SIX\0"
  "MYANMAR PAO DIGIT THREE\0"
  "MYANMAR PAO DIGIT TWO\0"
  "MYANMAR PAO DIGIT ZERO\0"
  "MYANMAR SHAN DIGIT EIGHT\0"
  "MYANMAR SHAN DIGIT FIVE\0"
  "MYANMAR SHAN DIGIT FOUR\0"
  "MYANMAR SHAN DIGIT NINE\0"
  "MYANMAR SHAN DIGIT ONE\0"
  "MYANMAR SHAN DIGIT SEVEN\0"
  "MYANMAR SHAN DIGIT SIX\0"
  "MYANMAR SHAN DIGIT THREE\0"
  "MYANMAR SHAN DIGIT TWO\0"
  "MYANMAR SHAN DIGIT ZERO\0"
  "MYANMAR SIGN ANUSVARA\0"
  "MYANMAR SIGN ASAT\0"
  "MYANMAR SIGN DOT BELOW\0"
  "MYANMAR SIGN KHAMTI TONE-1\0"
  "MYANMAR SIGN KHAMTI TONE-3\0"
  "MYANMAR SIGN LITTLE SECTION\0"
  "MYANMAR SIGN PAO KAREN TONE\0"
  "MYANMAR SIGN RUMAI PALAUNG TONE-5\0"
  "MYANMAR SIGN SECTION\0"
  "MYANMAR SIGN SHAN COUNCIL EMPHATIC TONE\0"
  "MYANMAR SIGN SHAN COUNCIL TONE-2\0"
  "MYANMAR SIGN SHAN COUNCIL TONE-3\0"
  "MYANMAR SIGN SHAN SAW\0"
  "MYANMAR SIGN SHAN TONE-2\0"
  "MYANMAR SIGN SHAN TONE-3\0"
  "MYANMAR SIGN SHAN TONE-5\0"
  "MYANMAR SIGN SHAN TONE-6\0"
  "MYANMAR SIGN TAI LAING TONE-2\0"
  "MYANMAR SIGN TAI LAING TONE-5\0"
  "MYANMAR SIGN VIRAMA\0"
  "MYANMAR SIGN VISARGA\0"
  "MYANMAR SIGN WESTERN PWO KAREN TONE-1\0"
  "MYANMAR SIGN WESTERN PWO KAREN TONE-2\0"
  "MYANMAR SIGN WESTERN PWO KAREN TONE-3\0"
  "MYANMAR SIGN WESTERN PWO KAREN TONE-4\0"
  "MYANMAR SIGN WESTERN PWO KAREN TONE-5\0"
  "MYANMAR SYMBOL AFOREMENTIONED\0"
  "MYANMAR SYMBOL AITON EXCLAMATION\0"
  "MYANMAR SYMBOL AITON ONE\0"
  "MYANMAR SYMBOL AITON TWO\0"
  "MYANMAR SYMBOL COMPLETED\0"
  "MYANMAR SYMBOL GENITIVE\0"
  "MYANMAR SYMBOL LOCATIVE\0"
  "MYANMAR SYMBOL SHAN EXCLAMATION\0"
  "MYANMAR SYMBOL SHAN ONE\0"
  "MYANMAR TAI LAING DIGIT EIGHT\0"
  "MYANMAR TAI LAING DIGIT FIVE\0"
  "MYANMAR TAI LAING DIGIT FOUR\0"
  "MYANMAR TAI LAING DIGIT NINE\0"
  "MYANMAR TAI LAING DIGIT ONE\0"
  "MYANMAR TAI LAING DIGIT SEVEN\0"
  "MYANMAR TAI LAING DIGIT SIX\0"
  "MYANMAR TAI LAING DIGIT THREE\0"
  "MYANMAR TAI LAING DIGIT TWO\0"
  "MYANMAR TAI LAING DIGIT ZERO\0"
  "MYANMAR TONE MARK SGAW KAREN HATHI\0"
  "MYANMAR TONE MARK SGAW KAREN KE PHO\0"
  "MYANMAR VOWEL SIGN AA\0"
  "MYANMAR VOWEL SIGN AI\0"
  "MYANMAR VOWEL SIGN AITON A\0"
  "MYANMAR VOWEL SIGN AITON AI\0"
  "MYANMAR VOWEL SIGN E\0"
  "MYANMAR VOWEL SIGN E ABOVE\0"
  "MYANMAR VOWEL SIGN GEBA KAREN I\0"
  "MYANMAR VOWEL SIGN I\0"
  "MYANMAR VOWEL SIGN II\0"
  "MYANMAR VOWEL SIGN KAYAH EE\0"
  "MYANMAR VOWEL SIGN KAYAH OE\0"
  "MYANMAR VOWEL SIGN KAYAH U\0"
  "MYANMAR VOWEL SIGN MON II\0"
  "MYANMAR VOWEL SIGN MON O\0"
  "MYANMAR VOWEL SIGN SGAW KAREN EU\0"
  "MYANMAR VOWEL SIGN SHAN AA\0"
  "MYANMAR VOWEL SIGN SHAN E\0"
  "MYANMAR VOWEL SIGN SHAN E ABOVE\0"
  "MYANMAR VOWEL SIGN SHAN FINAL Y\0"
  "MYANMAR VOWEL SIGN TALL AA\0"
  "MYANMAR VOWEL SIGN U\0"
  "MYANMAR VOWEL SIGN UU\0"
  "MYANMAR VOWEL SIGN VOCALIC L\0"
  "MYANMAR VOWEL SIGN VOCALIC LL\0"
  "MYANMAR VOWEL SIGN VOCALIC R\0"
  "MYANMAR VOWEL SIGN VOCALIC RR\0"
  "MYANMAR VOWEL SIGN WESTERN PWO KAREN EU\0"
  "MYANMAR VOWEL SIGN WESTERN PWO KAREN UE\0"
  "N-ARY CIRCLED DOT OPERATOR\0"
  "N-ARY CIRCLED PLUS OPERATOR\0"
  "N-ARY CIRCLED TIMES OPERATOR\0"
  "N-ARY COPRODUCT\0"
  "N-ARY INTERSECTION\0"
  "N-ARY LOGICAL AND\0"
  "N-ARY LOGICAL OR\0"
  "N-ARY PRODUCT\0"
  "N-ARY SQUARE INTERSECTION OPERATOR\0"
  "N-ARY SQUARE UNION OPERATOR\0"
  "N-ARY SUMMATION\0"
  "N-ARY TIMES OPERATOR\0"
  "N-ARY UNION\0"
  "N-ARY UNION OPERATOR WITH DOT\0"
  "N-ARY UNION OPERATOR WITH PLUS\0"
  "N-ARY WHITE VERTICAL BAR\0"
  "NABATAEAN CRUCIFORM NUMBER FOUR\0"
  "NABATAEAN LETTER ALEPH\0"
  "NABATAEAN LETTER AYIN\0"
  "NABATAEAN LETTER BETH\0"
  "NABATAEAN LETTER DALETH\0"
  "NABATAEAN LETTER FINAL ALEPH\0"
  "NABATAEAN LETTER FINAL BETH\0"
  "NABATAEAN LETTER FINAL HE\0"
  "NABATAEAN LETTER FINAL KAPH\0"
  "NABATAEAN LETTER FINAL LAMEDH\0"
  "NABATAEAN LETTER FINAL MEM\0"
  "NABATAEAN LETTER FINAL NUN\0"
  "NABATAEAN LETTER FINAL SHIN\0"
  "NABATAEAN LETTER FINAL YODH\0"
  "NABATAEAN LETTER GIMEL\0"
  "NABATAEAN LETTER HE\0"
  "NABATAEAN LETTER HETH\0"
  "NABATAEAN LETTER KAPH\0"
  "NABATAEAN LETTER LAMEDH\0"
  "NABATAEAN LETTER MEM\0"
  "NABATAEAN LETTER NUN\0"
  "NABATAEAN LETTER PE\0"
  "NABATAEAN LETTER QOPH\0"
  "NABATAEAN LETTER RESH\0"
  "NABATAEAN LETTER SADHE\0"
  "NABATAEAN LETTER SAMEKH\0"
  "NABATAEAN LETTER SHIN\0"
  "NABATAEAN LETTER TAW\0"
  "NABATAEAN LETTER TETH\0"
  "NABATAEAN LETTER WAW\0"
  "NABATAEAN LETTER YODH\0"
  "NABATAEAN LETTER ZAYIN\0"
  "NABATAEAN NUMBER FIVE\0"
  "NABATAEAN NUMBER FOUR\0"
  "NABATAEAN NUMBER ONE\0"
  "NABATAEAN NUMBER ONE HUNDRED\0"
  "NABATAEAN NUMBER TEN\0"
  "NABATAEAN NUMBER THREE\0"
  "NABATAEAN NUMBER TWENTY\0"
  "NABATAEAN NUMBER TWO\0"
  "NABLA\0"
  "NAG MUNDARI DIGIT EIGHT\0"
  "NAG MUNDARI DIGIT FIVE\0"
  "NAG MUNDARI DIGIT FOUR\0"
  "NAG MUNDARI DIGIT NINE\0"
  "NAG MUNDARI DIGIT ONE\0"
  "NAG MUNDARI DIGIT SEVEN\0"
  "NAG MUNDARI DIGIT SIX\0"
  "NAG MUNDARI DIGIT THREE\0"
  "NAG MUNDARI DIGIT TWO\0"
  "NAG MUNDARI DIGIT ZERO\0"
  "NAG MUNDARI LETTER A\0"
  "NAG MUNDARI LETTER AB\0"
  "NAG MUNDARI LETTER AH\0"
  "NAG MUNDARI LETTER AJ\0"
  "NAG MUNDARI LETTER ANY\0"
  "NAG MUNDARI LETTER E\0"
  "NAG MUNDARI LETTER EG\0"
  "NAG MUNDARI LETTER ELL\0"
  "NAG MUNDARI LETTER EM\0"
  "NAG MUNDARI LETTER EN\0"
  "NAG MUNDARI LETTER ENN\0"
  "NAG MUNDARI LETTER ETT\0"
  "NAG MUNDARI LETTER I\0"
  "NAG MUNDARI LETTER IDD\0"
  "NAG MUNDARI LETTER IH\0"
  "NAG MUNDARI LETTER IS\0"
  "NAG MUNDARI LETTER IT\0"
  "NAG MUNDARI LETTER O\0"
  "NAG MUNDARI LETTER OL\0"
  "NAG MUNDARI LETTER ONG\0"
  "NAG MUNDARI LETTER OP\0"
  "NAG MUNDARI LETTER OY\0"
  "NAG MUNDARI LETTER U\0"
  "NAG MUNDARI LETTER UC\0"
  "NAG MUNDARI LETTER UD\0"
  "NAG MUNDARI LETTER UK\0"
  "NAG MUNDARI LETTER UR\0"
  "NAG MUNDARI SIGN IKIR\0"
  "NAG MUNDARI SIGN MUHOR\0"
  "NAG MUNDARI SIGN OJOD\0"
  "NAG MUNDARI SIGN SUTUH\0"
  "NAG MUNDARI SIGN TOYOR\0"
  "NAIL POLISH\0"
  "NAIRA SIGN\0"
  "NAME BADGE\0"
  "NAND\0"
  "NANDINAGARI HEADSTROKE\0"
  "NANDINAGARI LETTER A\0"
  "NANDINAGARI LETTER AA\0"
  "NANDINAGARI LETTER AI\0"
  "NANDINAGARI LETTER AU\0"
  "NANDINAGARI LETTER BA\0"
  "NANDINAGARI LETTER BHA\0"
  "NANDINAGARI LETTER CA\0"
  "NANDINAGARI LETTER CHA\0"
  "NANDINAGARI LETTER DA\0"
  "NANDINAGARI LETTER DDA\0"
  "NANDINAGARI LETTER DDHA\0"
  "NANDINAGARI LETTER DHA\0"
  "NANDINAGARI LETTER E\0"
  "NANDINAGARI LETTER GA\0"
  "NANDINAGARI LETTER GHA\0"
  "NANDINAGARI LETTER HA\0"
  "NANDINAGARI LETTER I\0"
  "NANDINAGARI LETTER II\0"
  "NANDINAGARI LETTER JA\0"
  "NANDINAGARI LETTER JHA\0"
  "NANDINAGARI LETTER KA\0"
  "NANDINAGARI LETTER KHA\0"
  "NANDINAGARI LETTER LA\0"
  "NANDINAGARI LETTER LLA\0"
  "NANDINAGARI LETTER MA\0"
  "NANDINAGARI LETTER NA\0"
  "NANDINAGARI LETTER NGA\0"
  "NANDINAGARI LETTER NNA\0"
  "NANDINAGARI LETTER NYA\0"
  "NANDINAGARI LETTER O\0"
  "NANDINAGARI LETTER PA\0"
  "NANDINAGARI LETTER PHA\0"
  "NANDINAGARI LETTER RA\0"
  "NANDINAGARI LETTER RRA\0"
  "NANDINAGARI LETTER SA\0"
  "NANDINAGARI LETTER SHA\0"
  "NANDINAGARI LETTER SSA\0"
  "NANDINAGARI LETTER TA\0"
  "NANDINAGARI LETTER THA\0"
  "NANDINAGARI LETTER TTA\0"
  "NANDINAGARI LETTER TTHA\0"
  "NANDINAGARI LETTER U\0"
  "NANDINAGARI LETTER UU\0"
  "NANDINAGARI LETTER VA\0"
  "NANDINAGARI LETTER VOCALIC R\0"
  "NANDINAGARI LETTER VOCALIC RR\0"
  "NANDINAGARI LETTER YA\0"
  "NANDINAGARI SIGN ANUSVARA\0"
  "NANDINAGARI SIGN AVAGRAHA\0"
  "NANDINAGARI SIGN SIDDHAM\0"
  "NANDINAGARI SIGN VIRAMA\0"
  "NANDINAGARI SIGN VISARGA\0"
  "NANDINAGARI VOWEL SIGN AA\0"
  "NANDINAGARI VOWEL SIGN AI\0"
  "NANDINAGARI VOWEL SIGN AU\0"
  "NANDINAGARI VOWEL SIGN E\0"
  "NANDINAGARI VOWEL SIGN I\0"
  "NANDINAGARI VOWEL SIGN II\0"
  "NANDINAGARI VOWEL SIGN O\0"
  "NANDINAGARI VOWEL SIGN PRISHTHAMATRA E\0"
  "NANDINAGARI VOWEL SIGN U\0"
  "NANDINAGARI VOWEL SIGN UU\0"
  "NANDINAGARI VOWEL SIGN VOCALIC R\0"
  "NANDINAGARI VOWEL SIGN VOCALIC RR\0"
  "NARROW NO-BREAK SPACE\0"
  "NATIONAL DIGIT SHAPES\0"
  "NATIONAL PARK\0"
  "NAUSEATED FACE\0"
  "NAZAR AMULET\0"
  "NECKTIE\0"
  "NEGATED DOUBLE VERTICAL BAR DOUBLE RIGHT TURNSTILE\0"
  "NEGATIVE CIRCLED DIGIT ZERO\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER A\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER B\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER C\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER D\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER E\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER F\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER G\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER H\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER I\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER J\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER K\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER L\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER M\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER N\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER O\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER P\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER Q\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER R\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER S\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER T\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER U\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER V\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER W\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER X\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER Y\0"
  "NEGATIVE CIRCLED LATIN CAPITAL LETTER Z\0"
  "NEGATIVE CIRCLED NUMBER EIGHTEEN\0"
  "NEGATIVE CIRCLED NUMBER ELEVEN\0"
  "NEGATIVE CIRCLED NUMBER FIFTEEN\0"
  "NEGATIVE CIRCLED NUMBER FOURTEEN\0"
  "NEGATIVE CIRCLED NUMBER NINETEEN\0"
  "NEGATIVE CIRCLED NUMBER SEVENTEEN\0"
  "NEGATIVE CIRCLED NUMBER SIXTEEN\0"
  "NEGATIVE CIRCLED NUMBER THIRTEEN\0"
  "NEGATIVE CIRCLED NUMBER TWELVE\0"
  "NEGATIVE CIRCLED NUMBER TWENTY\0"
  "NEGATIVE CIRCLED SQUARE\0"
  "NEGATIVE CIRCLED TRIANGLE\0"
  "NEGATIVE DIAGONAL CROSS\0"
  "NEGATIVE DIAGONAL DIAMOND\0"
  "NEGATIVE DIAGONAL MIDDLE RIGHT TO LOWER CENTRE\0"
  "NEGATIVE SQUARED AB\0"
  "NEGATIVE SQUARED CROSS MARK\0"
  "NEGATIVE SQUARED DOWNWARDS ARROW\0"
  "NEGATIVE SQUARED IC\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER A\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER B\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER C\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER D\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER E\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER F\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER G\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER H\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER I\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER J\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER K\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER L\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER M\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER N\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER O\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER P\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER Q\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER R\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER S\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER T\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER U\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER V\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER W\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER X\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER Y\0"
  "NEGATIVE SQUARED LATIN CAPITAL LETTER Z\0"
  "NEGATIVE SQUARED LEFTWARDS ARROW\0"
  "NEGATIVE SQUARED PA\0"
  "NEGATIVE SQUARED QUESTION MARK\0"
  "NEGATIVE SQUARED RIGHTWARDS ARROW\0"
  "NEGATIVE SQUARED SA\0"
  "NEGATIVE SQUARED UPWARDS ARROW\0"
  "NEGATIVE SQUARED WC\0"
  "NEITHER A SUBSET OF NOR EQUAL TO\0"
  "NEITHER A SUPERSET OF NOR EQUAL TO\0"
  "NEITHER APPROXIMATELY NOR ACTUALLY EQUAL TO\0"
  "NEITHER GREATER-THAN NOR EQUAL TO\0"
  "NEITHER GREATER-THAN NOR EQUIVALENT TO\0"
  "NEITHER GREATER-THAN NOR LESS-THAN\0"
  "NEITHER LESS-THAN NOR EQUAL TO\0"
  "NEITHER LESS-THAN NOR EQUIVALENT TO\0"
  "NEITHER LESS-THAN NOR GREATER-THAN\0"
  "NEPTUNE\0"
  "NEPTUNE FORM TWO\0"
  "NERD FACE\0"
  "NESSUS\0"
  "NEST WITH EGGS\0"
  "NESTING DOLLS\0"
  "NEUTER\0"
  "NEUTRAL CHESS BISHOP\0"
  "NEUTRAL CHESS BISHOP ROTATED NINETY DEGREES\0"
  "NEUTRAL CHESS BISHOP ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "NEUTRAL CHESS EQUIHOPPER\0"
  "NEUTRAL CHESS EQUIHOPPER ROTATED NINETY DEGREES\0"
  "NEUTRAL CHESS KING\0"
  "NEUTRAL CHESS KING ROTATED NINETY DEGREES\0"
  "NEUTRAL CHESS KING ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "NEUTRAL CHESS KNIGHT\0"
  "NEUTRAL CHESS KNIGHT ROTATED FORTY-FIVE DEGREES\0"
  "NEUTRAL CHESS KNIGHT ROTATED NINETY DEGREES\0"
  "NEUTRAL CHESS KNIGHT ROTATED ONE HUNDRED THIRTY-FIVE DEGREES\0"
  "NEUTRAL CHESS KNIGHT ROTATED THREE HUNDRED FIFTEEN DEGREES\0"
  "NEUTRAL CHESS KNIGHT ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "NEUTRAL CHESS KNIGHT ROTATED TWO HUNDRED TWENTY-FIVE DEGREES\0"
  "NEUTRAL CHESS PAWN\0"
  "NEUTRAL CHESS PAWN ROTATED NINETY DEGREES\0"
  "NEUTRAL CHESS PAWN ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "NEUTRAL CHESS QUEEN\0"
  "NEUTRAL CHESS QUEEN ROTATED NINETY DEGREES\0"
  "NEUTRAL CHESS QUEEN ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "NEUTRAL CHESS ROOK\0"
  "NEUTRAL CHESS ROOK ROTATED NINETY DEGREES\0"
  "NEUTRAL CHESS ROOK ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "NEUTRAL CHESS TURNED BISHOP\0"
  "NEUTRAL CHESS TURNED KING\0"
  "NEUTRAL CHESS TURNED KNIGHT\0"
  "NEUTRAL CHESS TURNED PAWN\0"
  "NEUTRAL CHESS TURNED QUEEN\0"
  "NEUTRAL CHESS TURNED ROOK\0"
  "NEUTRAL FACE\0"
  "NEW MOON SYMBOL\0"
  "NEW MOON WITH FACE\0"
  "NEW SHEQEL SIGN\0"
  "NEW TAI LUE DIGIT EIGHT\0"
  "NEW TAI LUE DIGIT FIVE\0"
  "NEW TAI LUE DIGIT FOUR\0"
  "NEW TAI LUE DIGIT NINE\0"
  "NEW TAI LUE DIGIT ONE\0"
  "NEW TAI LUE DIGIT SEVEN\0"
  "NEW TAI LUE DIGIT SIX\0"
  "NEW TAI LUE DIGIT THREE\0"
  "NEW TAI LUE DIGIT TWO\0"
  "NEW TAI LUE DIGIT ZERO\0"
  "NEW TAI LUE LETTER FINAL B\0"
  "NEW TAI LUE LETTER FINAL D\0"
  "NEW TAI LUE LETTER FINAL K\0"
  "NEW TAI LUE LETTER FINAL M\0"
  "NEW TAI LUE LETTER FINAL N\0"
  "NEW TAI LUE LETTER FINAL NG\0"
  "NEW TAI LUE LETTER FINAL V\0"
  "NEW TAI LUE LETTER HIGH BA\0"
  "NEW TAI LUE LETTER HIGH DA\0"
  "NEW TAI LUE LETTER HIGH FA\0"
  "NEW TAI LUE LETTER HIGH HA\0"
  "NEW TAI LUE LETTER HIGH KA\0"
  "NEW TAI LUE LETTER HIGH KVA\0"
  "NEW TAI LUE LETTER HIGH LA\0"
  "NEW TAI LUE LETTER HIGH MA\0"
  "NEW TAI LUE LETTER HIGH NA\0"
  "NEW TAI LUE LETTER HIGH NGA\0"
  "NEW TAI LUE LETTER HIGH PA\0"
  "NEW TAI LUE LETTER HIGH PHA\0"
  "NEW TAI LUE LETTER HIGH QA\0"
  "NEW TAI LUE LETTER HIGH SA\0"
  "NEW TAI LUE LETTER HIGH SUA\0"
  "NEW TAI LUE LETTER HIGH TA\0"
  "NEW TAI LUE LETTER HIGH THA\0"
  "NEW TAI LUE LETTER HIGH TSA\0"
  "NEW TAI LUE LETTER HIGH VA\0"
  "NEW TAI LUE LETTER HIGH XA\0"
  "NEW TAI LUE LETTER HIGH XVA\0"
  "NEW TAI LUE LETTER HIGH YA\0"
  "NEW TAI LUE LETTER LOW BA\0"
  "NEW TAI LUE LETTER LOW DA\0"
  "NEW TAI LUE LETTER LOW FA\0"
  "NEW TAI LUE LETTER LOW HA\0"
  "NEW TAI LUE LETTER LOW KA\0"
  "NEW TAI LUE LETTER LOW KVA\0"
  "NEW TAI LUE LETTER LOW LA\0"
  "NEW TAI LUE LETTER LOW MA\0"
  "NEW TAI LUE LETTER LOW NA\0"
  "NEW TAI LUE LETTER LOW NGA\0"
  "NEW TAI LUE LETTER LOW PA\0"
  "NEW TAI LUE LETTER LOW PHA\0"
  "NEW TAI LUE LETTER LOW QA\0"
  "NEW TAI LUE LETTER LOW SA\0"
  "NEW TAI LUE LETTER LOW SUA\0"
  "NEW TAI LUE LETTER LOW TA\0"
  "NEW TAI LUE LETTER LOW THA\0"
  "NEW TAI LUE LETTER LOW TSA\0"
  "NEW TAI LUE LETTER LOW VA\0"
  "NEW TAI LUE LETTER LOW XA\0"
  "NEW TAI LUE LETTER LOW XVA\0"
  "NEW TAI LUE LETTER LOW YA\0"
  "NEW TAI LUE SIGN LAE\0"
  "NEW TAI LUE SIGN LAEV\0"
  "NEW TAI LUE THAM DIGIT ONE\0"
  "NEW TAI LUE TONE MARK-1\0"
  "NEW TAI LUE TONE MARK-2\0"
  "NEW TAI LUE VOWEL SIGN AA\0"
  "NEW TAI LUE VOWEL SIGN AAY\0"
  "NEW TAI LUE VOWEL SIGN AE\0"
  "NEW TAI LUE VOWEL SIGN AY\0"
  "NEW TAI LUE VOWEL SIGN E\0"
  "NEW TAI LUE VOWEL SIGN II\0"
  "NEW TAI LUE VOWEL SIGN IY\0"
  "NEW TAI LUE VOWEL SIGN O\0"
  "NEW TAI LUE VOWEL SIGN OA\0"
  "NEW TAI LUE VOWEL SIGN OAY\0"
  "NEW TAI LUE VOWEL SIGN OY\0"
  "NEW TAI LUE VOWEL SIGN U\0"
  "NEW TAI LUE VOWEL SIGN UE\0"
  "NEW TAI LUE VOWEL SIGN UEY\0"
  "NEW TAI LUE VOWEL SIGN UU\0"
  "NEW TAI LUE VOWEL SIGN UY\0"
  "NEW TAI LUE VOWEL SIGN VOWEL SHORTENER\0"
  "NEWA ABBREVIATION SIGN\0"
  "NEWA COMMA\0"
  "NEWA DANDA\0"
  "NEWA DIGIT EIGHT\0"
  "NEWA DIGIT FIVE\0"
  "NEWA DIGIT FOUR\0"
  "NEWA DIGIT NINE\0"
  "NEWA DIGIT ONE\0"
  "NEWA DIGIT SEVEN\0"
  "NEWA DIGIT SIX\0"
  "NEWA DIGIT THREE\0"
  "NEWA DIGIT TWO\0"
  "NEWA DIGIT ZERO\0"
  "NEWA DOUBLE COMMA\0"
  "NEWA DOUBLE DANDA\0"
  "NEWA GAP FILLER\0"
  "NEWA INSERTION SIGN\0"
  "NEWA LETTER A\0"
  "NEWA LETTER AA\0"
  "NEWA LETTER AI\0"
  "NEWA LETTER AU\0"
  "NEWA LETTER BA\0"
  "NEWA LETTER BHA\0"
  "NEWA LETTER CA\0"
  "NEWA LETTER CHA\0"
  "NEWA LETTER DA\0"
  "NEWA LETTER DDA\0"
  "NEWA LETTER DDHA\0"
  "NEWA LETTER DHA\0"
  "NEWA LETTER E\0"
  "NEWA LETTER GA\0"
  "NEWA LETTER GHA\0"
  "NEWA LETTER HA\0"
  "NEWA LETTER I\0"
  "NEWA LETTER II\0"
  "NEWA LETTER JA\0"
  "NEWA LETTER JHA\0"
  "NEWA LETTER KA\0"
  "NEWA LETTER KHA\0"
  "NEWA LETTER LA\0"
  "NEWA LETTER LHA\0"
  "NEWA LETTER MA\0"
  "NEWA LETTER MHA\0"
  "NEWA LETTER NA\0"
  "NEWA LETTER NGA\0"
  "NEWA LETTER NGHA\0"
  "NEWA LETTER NHA\0"
  "NEWA LETTER NNA\0"
  "NEWA LETTER NYA\0"
  "NEWA LETTER NYHA\0"
  "NEWA LETTER O\0"
  "NEWA LETTER PA\0"
  "NEWA LETTER PHA\0"
  "NEWA LETTER RA\0"
  "NEWA LETTER RHA\0"
  "NEWA LETTER SA\0"
  "NEWA LETTER SHA\0"
  "NEWA LETTER SSA\0"
  "NEWA LETTER TA\0"
  "NEWA LETTER THA\0"
  "NEWA LETTER TTA\0"
  "NEWA LETTER TTHA\0"
  "NEWA LETTER U\0"
  "NEWA LETTER UU\0"
  "NEWA LETTER VEDIC ANUSVARA\0"
  "NEWA LETTER VOCALIC L\0"
  "NEWA LETTER VOCALIC LL\0"
  "NEWA LETTER VOCALIC R\0"
  "NEWA LETTER VOCALIC RR\0"
  "NEWA LETTER WA\0"
  "NEWA LETTER YA\0"
  "NEWA OM\0"
  "NEWA PLACEHOLDER MARK\0"
  "NEWA SANDHI MARK\0"
  "NEWA SIDDHI\0"
  "NEWA SIGN ANUSVARA\0"
  "NEWA SIGN AVAGRAHA\0"
  "NEWA SIGN CANDRABINDU\0"
  "NEWA SIGN FINAL ANUSVARA\0"
  "NEWA SIGN JIHVAMULIYA\0"
  "NEWA SIGN NUKTA\0"
  "NEWA SIGN UPADHMANIYA\0"
  "NEWA SIGN VIRAMA\0"
  "NEWA SIGN VISARGA\0"
  "NEWA VOWEL SIGN AA\0"
  "NEWA VOWEL SIGN AI\0"
  "NEWA VOWEL SIGN AU\0"
  "NEWA VOWEL SIGN E\0"
  "NEWA VOWEL SIGN I\0"
  "NEWA VOWEL SIGN II\0"
  "NEWA VOWEL SIGN O\0"
  "NEWA VOWEL SIGN U\0"
  "NEWA VOWEL SIGN UU\0"
  "NEWA VOWEL SIGN VOCALIC L\0"
  "NEWA VOWEL SIGN VOCALIC LL\0"
  "NEWA VOWEL SIGN VOCALIC R\0"
  "NEWA VOWEL SIGN VOCALIC RR\0"
  "NEWLINE LEFT\0"
  "NEWLINE RIGHT\0"
  "NEWSPAPER\0"
  "NEXT PAGE\0"
  "NIGHT WITH STARS\0"
  "NINE POINTED WHITE STAR\0"
  "NINJA\0"
  "NKO COMBINING DOUBLE DOT ABOVE\0"
  "NKO COMBINING LONG DESCENDING TONE\0"
  "NKO COMBINING LONG HIGH TONE\0"
  "NKO COMBINING LONG LOW TONE\0"
  "NKO COMBINING LONG RISING TONE\0"
  "NKO COMBINING NASALIZATION MARK\0"
  "NKO COMBINING SHORT HIGH TONE\0"
  "NKO COMBINING SHORT LOW TONE\0"
  "NKO COMBINING SHORT RISING TONE\0"
  "NKO COMMA\0"
  "NKO DANTAYALAN\0"
  "NKO DIGIT EIGHT\0"
  "NKO DIGIT FIVE\0"
  "NKO DIGIT FOUR\0"
  "NKO DIGIT NINE\0"
  "NKO DIGIT ONE\0"
  "NKO DIGIT SEVEN\0"
  "NKO DIGIT SIX\0"
  "NKO DIGIT THREE\0"
  "NKO DIGIT TWO\0"
  "NKO DIGIT ZERO\0"
  "NKO DOROME SIGN\0"
  "NKO EXCLAMATION MARK\0"
  "NKO HIGH TONE APOSTROPHE\0"
  "NKO LAJANYALAN\0"
  "NKO LETTER A\0"
  "NKO LETTER BA\0"
  "NKO LETTER CHA\0"
  "NKO LETTER DA\0"
  "NKO LETTER DAGBASINNA\0"
  "NKO LETTER E\0"
  "NKO LETTER EE\0"
  "NKO LETTER FA\0"
  "NKO LETTER GBA\0"
  "NKO LETTER HA\0"
  "NKO LETTER I\0"
  "NKO LETTER JA\0"
  "NKO LETTER JONA CHA\0"
  "NKO LETTER JONA JA\0"
  "NKO LETTER JONA RA\0"
  "NKO LETTER KA\0"
  "NKO LETTER LA\0"
  "NKO LETTER MA\0"
  "NKO LETTER N\0"
  "NKO LETTER NA\0"
  "NKO LETTER NA WOLOSO\0"
  "NKO LETTER NYA\0"
  "NKO LETTER NYA WOLOSO\0"
  "NKO LETTER O\0"
  "NKO LETTER OO\0"
  "NKO LETTER PA\0"
  "NKO LETTER RA\0"
  "NKO LETTER RRA\0"
  "NKO LETTER SA\0"
  "NKO LETTER TA\0"
  "NKO LETTER U\0"
  "NKO LETTER WA\0"
  "NKO LETTER YA\0"
  "NKO LOW TONE APOSTROPHE\0"
  "NKO SYMBOL GBAKURUNEN\0"
  "NKO SYMBOL OO DENNEN\0"
  "NKO TAMAN SIGN\0"
  "NO BICYCLES\0"
  "NO ENTRY\0"
  "NO ENTRY SIGN\0"
  "NO MOBILE PHONES\0"
  "NO ONE UNDER EIGHTEEN SYMBOL\0"
  "NO PEDESTRIANS\0"
  "NO PIRACY\0"
  "NO SMOKING SYMBOL\0"
  "NO-BREAK SPACE\0"
  "NOMINAL DIGIT SHAPES\0"
  "NOMISMA SIGN\0"
  "NON-BREAKING HYPHEN\0"
  "NON-POTABLE WATER SYMBOL\0"
  "NONFORKING\0"
  "NOR\0"
  "NORDIC MARK SIGN\0"
  "NORMAL SUBGROUP OF\0"
  "NORMAL SUBGROUP OF OR EQUAL TO\0"
  "NORTH EAST AND SOUTH WEST ARROW\0"
  "NORTH EAST ARROW\0"
  "NORTH EAST ARROW AND SOUTH EAST ARROW\0"
  "NORTH EAST ARROW CROSSING NORTH WEST ARROW\0"
  "NORTH EAST ARROW CROSSING SOUTH EAST ARROW\0"
  "NORTH EAST ARROW FROM BAR\0"
  "NORTH EAST ARROW WITH HOOK\0"
  "NORTH EAST BLACK ARROW\0"
  "NORTH EAST DOUBLE ARROW\0"
  "NORTH EAST POINTING BUD\0"
  "NORTH EAST POINTING LEAF\0"
  "NORTH EAST POINTING VINE LEAF\0"
  "NORTH EAST SANS-SERIF ARROW\0"
  "NORTH EAST TRIANGLE-HEADED ARROW\0"
  "NORTH EAST TRIANGLE-HEADED ARROW TO BAR\0"
  "NORTH EAST WHITE ARROW\0"
  "NORTH INDIC FRACTION ONE EIGHTH\0"
  "NORTH INDIC FRACTION ONE HALF\0"
  "NORTH INDIC FRACTION ONE QUARTER\0"
  "NORTH INDIC FRACTION ONE SIXTEENTH\0"
  "NORTH INDIC FRACTION THREE QUARTERS\0"
  "NORTH INDIC FRACTION THREE SIXTEENTHS\0"
  "NORTH INDIC PLACEHOLDER MARK\0"
  "NORTH INDIC QUANTITY MARK\0"
  "NORTH INDIC QUARTER MARK\0"
  "NORTH INDIC RUPEE MARK\0"
  "NORTH WEST AND SOUTH EAST ARROW\0"
  "NORTH WEST ARROW\0"
  "NORTH WEST ARROW AND NORTH EAST ARROW\0"
  "NORTH WEST ARROW CROSSING NORTH EAST ARROW\0"
  "NORTH WEST ARROW FROM BAR\0"
  "NORTH WEST ARROW TO CORNER\0"
  "NORTH WEST ARROW TO LONG BAR\0"
  "NORTH WEST ARROW WITH HOOK\0"
  "NORTH WEST BLACK ARROW\0"
  "NORTH WEST DOUBLE ARROW\0"
  "NORTH WEST POINTING BUD\0"
  "NORTH WEST POINTING LEAF\0"
  "NORTH WEST POINTING VINE LEAF\0"
  "NORTH WEST SANS-SERIF ARROW\0"
  "NORTH WEST TRIANGLE-HEADED ARROW\0"
  "NORTH WEST TRIANGLE-HEADED ARROW TO BAR\0"
  "NORTH WEST WHITE ARROW\0"
  "NORTHEAST-POINTING AIRPLANE\0"
  "NOSE\0"
  "NOT A SUBSET OF\0"
  "NOT A SUPERSET OF\0"
  "NOT ALMOST EQUAL TO\0"
  "NOT AN ELEMENT OF\0"
  "NOT ASYMPTOTICALLY EQUAL TO\0"
  "NOT CHECK MARK\0"
  "NOT EQUAL TO\0"
  "NOT EQUIVALENT TO\0"
  "NOT GREATER-THAN\0"
  "NOT IDENTICAL TO\0"
  "NOT LESS-THAN\0"
  "NOT NORMAL SUBGROUP OF\0"
  "NOT NORMAL SUBGROUP OF OR EQUAL TO\0"
  "NOT PARALLEL TO\0"
  "NOT SIGN\0"
  "NOT SQUARE IMAGE OF OR EQUAL TO\0"
  "NOT SQUARE ORIGINAL OF OR EQUAL TO\0"
  "NOT TILDE\0"
  "NOT TRUE\0"
  "NOTCHED LEFT SEMICIRCLE WITH THREE DOTS\0"
  "NOTCHED LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW\0"
  "NOTCHED RIGHT SEMICIRCLE WITH THREE DOTS\0"
  "NOTCHED UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW\0"
  "NOTE\0"
  "NOTE PAD\0"
  "NOTE PAGE\0"
  "NOTEBOOK\0"
  "NOTEBOOK WITH DECORATIVE COVER\0"
  "NPN TRANSISTOR\0"
  "NUMBER EIGHTEEN FULL STOP\0"
  "NUMBER ELEVEN FULL STOP\0"
  "NUMBER FIFTEEN FULL STOP\0"
  "NUMBER FOURTEEN FULL STOP\0"
  "NUMBER NINETEEN FULL STOP\0"
  "NUMBER SEVENTEEN FULL STOP\0"
  "NUMBER SIGN\0"
  "NUMBER SIXTEEN FULL STOP\0"
  "NUMBER TEN FULL STOP\0"
  "NUMBER THIRTEEN FULL STOP\0"
  "NUMBER TWELVE FULL STOP\0"
  "NUMBER TWENTY FULL STOP\0"
  "NUMERO SIGN\0"
  "NUSHU CHARACTER-1B170\0"
  "NUSHU CHARACTER-1B171\0"
  "NUSHU CHARACTER-1B172\0"
  "NUSHU CHARACTER-1B173\0"
  "NUSHU CHARACTER-1B174\0"
  "NUSHU CHARACTER-1B175\0"
  "NUSHU CHARACTER-1B176\0"
  "NUSHU CHARACTER-1B177\0"
  "NUSHU CHARACTER-1B178\0"
  "NUSHU CHARACTER-1B179\0"
  "NUSHU CHARACTER-1B17A\0"
  "NUSHU CHARACTER-1B17B\0"
  "NUSHU CHARACTER-1B17C\0"
  "NUSHU CHARACTER-1B17D\0"
  "NUSHU CHARACTER-1B17E\0"
  "NUSHU CHARACTER-1B17F\0"
  "NUSHU CHARACTER-1B180\0"
  "NUSHU CHARACTER-1B181\0"
  "NUSHU CHARACTER-1B182\0"
  "NUSHU CHARACTER-1B183\0"
  "NUSHU CHARACTER-1B184\0"
  "NUSHU CHARACTER-1B185\0"
  "NUSHU CHARACTER-1B186\0"
  "NUSHU CHARACTER-1B187\0"
  "NUSHU CHARACTER-1B188\0"
  "NUSHU CHARACTER-1B189\0"
  "NUSHU CHARACTER-1B18A\0"
  "NUSHU CHARACTER-1B18B\0"
  "NUSHU CHARACTER-1B18C\0"
  "NUSHU CHARACTER-1B18D\0"
  "NUSHU CHARACTER-1B18E\0"
  "NUSHU CHARACTER-1B18F\0"
  "NUSHU CHARACTER-1B190\0"
  "NUSHU CHARACTER-1B191\0"
  "NUSHU CHARACTER-1B192\0"
  "NUSHU CHARACTER-1B193\0"
  "NUSHU CHARACTER-1B194\0"
  "NUSHU CHARACTER-1B195\0"
  "NUSHU CHARACTER-1B196\0"
  "NUSHU CHARACTER-1B197\0"
  "NUSHU CHARACTER-1B198\0"
  "NUSHU CHARACTER-1B199\0"
  "NUSHU CHARACTER-1B19A\0"
  "NUSHU CHARACTER-1B19B\0"
  "NUSHU CHARACTER-1B19C\0"
  "NUSHU CHARACTER-1B19D\0"
  "NUSHU CHARACTER-1B19E\0"
  "NUSHU CHARACTER-1B19F\0"
  "NUSHU CHARACTER-1B1A0\0"
  "NUSHU CHARACTER-1B1A1\0"
  "NUSHU CHARACTER-1B1A2\0"
  "NUSHU CHARACTER-1B1A3\0"
  "NUSHU CHARACTER-1B1A4\0"
  "NUSHU CHARACTER-1B1A5\0"
  "NUSHU CHARACTER-1B1A6\0"
  "NUSHU CHARACTER-1B1A7\0"
  "NUSHU CHARACTER-1B1A8\0"
  "NUSHU CHARACTER-1B1A9\0"
  "NUSHU CHARACTER-1B1AA\0"
  "NUSHU CHARACTER-1B1AB\0"
  "NUSHU CHARACTER-1B1AC\0"
  "NUSHU CHARACTER-1B1AD\0"
  "NUSHU CHARACTER-1B1AE\0"
  "NUSHU CHARACTER-1B1AF\0"
  "NUSHU CHARACTER-1B1B0\0"
  "NUSHU CHARACTER-1B1B1\0"
  "NUSHU CHARACTER-1B1B2\0"
  "NUSHU CHARACTER-1B1B3\0"
  "NUSHU CHARACTER-1B1B4\0"
  "NUSHU CHARACTER-1B1B5\0"
  "NUSHU CHARACTER-1B1B6\0"
  "NUSHU CHARACTER-1B1B7\0"
  "NUSHU CHARACTER-1B1B8\0"
  "NUSHU CHARACTER-1B1B9\0"
  "NUSHU CHARACTER-1B1BA\0"
  "NUSHU CHARACTER-1B1BB\0"
  "NUSHU CHARACTER-1B1BC\0"
  "NUSHU CHARACTER-1B1BD\0"
  "NUSHU CHARACTER-1B1BE\0"
  "NUSHU CHARACTER-1B1BF\0"
  "NUSHU CHARACTER-1B1C0\0"
  "NUSHU CHARACTER-1B1C1\0"
  "NUSHU CHARACTER-1B1C2\0"
  "NUSHU CHARACTER-1B1C3\0"
  "NUSHU CHARACTER-1B1C4\0"
  "NUSHU CHARACTER-1B1C5\0"
  "NUSHU CHARACTER-1B1C6\0"
  "NUSHU CHARACTER-1B1C7\0"
  "NUSHU CHARACTER-1B1C8\0"
  "NUSHU CHARACTER-1B1C9\0"
  "NUSHU CHARACTER-1B1CA\0"
  "NUSHU CHARACTER-1B1CB\0"
  "NUSHU CHARACTER-1B1CC\0"
  "NUSHU CHARACTER-1B1CD\0"
  "NUSHU CHARACTER-1B1CE\0"
  "NUSHU CHARACTER-1B1CF\0"
  "NUSHU CHARACTER-1B1D0\0"
  "NUSHU CHARACTER-1B1D1\0"
  "NUSHU CHARACTER-1B1D2\0"
  "NUSHU CHARACTER-1B1D3\0"
  "NUSHU CHARACTER-1B1D4\0"
  "NUSHU CHARACTER-1B1D5\0"
  "NUSHU CHARACTER-1B1D6\0"
  "NUSHU CHARACTER-1B1D7\0"
  "NUSHU CHARACTER-1B1D8\0"
  "NUSHU CHARACTER-1B1D9\0"
  "NUSHU CHARACTER-1B1DA\0"
  "NUSHU CHARACTER-1B1DB\0"
  "NUSHU CHARACTER-1B1DC\0"
  "NUSHU CHARACTER-1B1DD\0"
  "NUSHU CHARACTER-1B1DE\0"
  "NUSHU CHARACTER-1B1DF\0"
  "NUSHU CHARACTER-1B1E0\0"
  "NUSHU CHARACTER-1B1E1\0"
  "NUSHU CHARACTER-1B1E2\0"
  "NUSHU CHARACTER-1B1E3\0"
  "NUSHU CHARACTER-1B1E4\0"
  "NUSHU CHARACTER-1B1E5\0"
  "NUSHU CHARACTER-1B1E6\0"
  "NUSHU CHARACTER-1B1E7\0"
  "NUSHU CHARACTER-1B1E8\0"
  "NUSHU CHARACTER-1B1E9\0"
  "NUSHU CHARACTER-1B1EA\0"
  "NUSHU CHARACTER-1B1EB\0"
  "NUSHU CHARACTER-1B1EC\0"
  "NUSHU CHARACTER-1B1ED\0"
  "NUSHU CHARACTER-1B1EE\0"
  "NUSHU CHARACTER-1B1EF\0"
  "NUSHU CHARACTER-1B1F0\0"
  "NUSHU CHARACTER-1B1F1\0"
  "NUSHU CHARACTER-1B1F2\0"
  "NUSHU CHARACTER-1B1F3\0"
  "NUSHU CHARACTER-1B1F4\0"
  "NUSHU CHARACTER-1B1F5\0"
  "NUSHU CHARACTER-1B1F6\0"
  "NUSHU CHARACTER-1B1F7\0"
  "NUSHU CHARACTER-1B1F8\0"
  "NUSHU CHARACTER-1B1F9\0"
  "NUSHU CHARACTER-1B1FA\0"
  "NUSHU CHARACTER-1B1FB\0"
  "NUSHU CHARACTER-1B1FC\0"
  "NUSHU CHARACTER-1B1FD\0"
  "NUSHU CHARACTER-1B1FE\0"
  "NUSHU CHARACTER-1B1FF\0"
  "NUSHU CHARACTER-1B200\0"
  "NUSHU CHARACTER-1B201\0"
  "NUSHU CHARACTER-1B202\0"
  "NUSHU CHARACTER-1B203\0"
  "NUSHU CHARACTER-1B204\0"
  "NUSHU CHARACTER-1B205\0"
  "NUSHU CHARACTER-1B206\0"
  "NUSHU CHARACTER-1B207\0"
  "NUSHU CHARACTER-1B208\0"
  "NUSHU CHARACTER-1B209\0"
  "NUSHU CHARACTER-1B20A\0"
  "NUSHU CHARACTER-1B20B\0"
  "NUSHU CHARACTER-1B20C\0"
  "NUSHU CHARACTER-1B20D\0"
  "NUSHU CHARACTER-1B20E\0"
  "NUSHU CHARACTER-1B20F\0"
  "NUSHU CHARACTER-1B210\0"
  "NUSHU CHARACTER-1B211\0"
  "NUSHU CHARACTER-1B212\0"
  "NUSHU CHARACTER-1B213\0"
  "NUSHU CHARACTER-1B214\0"
  "NUSHU CHARACTER-1B215\0"
  "NUSHU CHARACTER-1B216\0"
  "NUSHU CHARACTER-1B217\0"
  "NUSHU CHARACTER-1B218\0"
  "NUSHU CHARACTER-1B219\0"
  "NUSHU CHARACTER-1B21A\0"
  "NUSHU CHARACTER-1B21B\0"
  "NUSHU CHARACTER-1B21C\0"
  "NUSHU CHARACTER-1B21D\0"
  "NUSHU CHARACTER-1B21E\0"
  "NUSHU CHARACTER-1B21F\0"
  "NUSHU CHARACTER-1B220\0"
  "NUSHU CHARACTER-1B221\0"
  "NUSHU CHARACTER-1B222\0"
  "NUSHU CHARACTER-1B223\0"
  "NUSHU CHARACTER-1B224\0"
  "NUSHU CHARACTER-1B225\0"
  "NUSHU CHARACTER-1B226\0"
  "NUSHU CHARACTER-1B227\0"
  "NUSHU CHARACTER-1B228\0"
  "NUSHU CHARACTER-1B229\0"
  "NUSHU CHARACTER-1B22A\0"
  "NUSHU CHARACTER-1B22B\0"
  "NUSHU CHARACTER-1B22C\0"
  "NUSHU CHARACTER-1B22D\0"
  "NUSHU CHARACTER-1B22E\0"
  "NUSHU CHARACTER-1B22F\0"
  "NUSHU CHARACTER-1B230\0"
  "NUSHU CHARACTER-1B231\0"
  "NUSHU CHARACTER-1B232\0"
  "NUSHU CHARACTER-1B233\0"
  "NUSHU CHARACTER-1B234\0"
  "NUSHU CHARACTER-1B235\0"
  "NUSHU CHARACTER-1B236\0"
  "NUSHU CHARACTER-1B237\0"
  "NUSHU CHARACTER-1B238\0"
  "NUSHU CHARACTER-1B239\0"
  "NUSHU CHARACTER-1B23A\0"
  "NUSHU CHARACTER-1B23B\0"
  "NUSHU CHARACTER-1B23C\0"
  "NUSHU CHARACTER-1B23D\0"
  "NUSHU CHARACTER-1B23E\0"
  "NUSHU CHARACTER-1B23F\0"
  "NUSHU CHARACTER-1B240\0"
  "NUSHU CHARACTER-1B241\0"
  "NUSHU CHARACTER-1B242\0"
  "NUSHU CHARACTER-1B243\0"
  "NUSHU CHARACTER-1B244\0"
  "NUSHU CHARACTER-1B245\0"
  "NUSHU CHARACTER-1B246\0"
  "NUSHU CHARACTER-1B247\0"
  "NUSHU CHARACTER-1B248\0"
  "NUSHU CHARACTER-1B249\0"
  "NUSHU CHARACTER-1B24A\0"
  "NUSHU CHARACTER-1B24B\0"
  "NUSHU CHARACTER-1B24C\0"
  "NUSHU CHARACTER-1B24D\0"
  "NUSHU CHARACTER-1B24E\0"
  "NUSHU CHARACTER-1B24F\0"
  "NUSHU CHARACTER-1B250\0"
  "NUSHU CHARACTER-1B251\0"
  "NUSHU CHARACTER-1B252\0"
  "NUSHU CHARACTER-1B253\0"
  "NUSHU CHARACTER-1B254\0"
  "NUSHU CHARACTER-1B255\0"
  "NUSHU CHARACTER-1B256\0"
  "NUSHU CHARACTER-1B257\0"
  "NUSHU CHARACTER-1B258\0"
  "NUSHU CHARACTER-1B259\0"
  "NUSHU CHARACTER-1B25A\0"
  "NUSHU CHARACTER-1B25B\0"
  "NUSHU CHARACTER-1B25C\0"
  "NUSHU CHARACTER-1B25D\0"
  "NUSHU CHARACTER-1B25E\0"
  "NUSHU CHARACTER-1B25F\0"
  "NUSHU CHARACTER-1B260\0"
  "NUSHU CHARACTER-1B261\0"
  "NUSHU CHARACTER-1B262\0"
  "NUSHU CHARACTER-1B263\0"
  "NUSHU CHARACTER-1B264\0"
  "NUSHU CHARACTER-1B265\0"
  "NUSHU CHARACTER-1B266\0"
  "NUSHU CHARACTER-1B267\0"
  "NUSHU CHARACTER-1B268\0"
  "NUSHU CHARACTER-1B269\0"
  "NUSHU CHARACTER-1B26A\0"
  "NUSHU CHARACTER-1B26B\0"
  "NUSHU CHARACTER-1B26C\0"
  "NUSHU CHARACTER-1B26D\0"
  "NUSHU CHARACTER-1B26E\0"
  "NUSHU CHARACTER-1B26F\0"
  "NUSHU CHARACTER-1B270\0"
  "NUSHU CHARACTER-1B271\0"
  "NUSHU CHARACTER-1B272\0"
  "NUSHU CHARACTER-1B273\0"
  "NUSHU CHARACTER-1B274\0"
  "NUSHU CHARACTER-1B275\0"
  "NUSHU CHARACTER-1B276\0"
  "NUSHU CHARACTER-1B277\0"
  "NUSHU CHARACTER-1B278\0"
  "NUSHU CHARACTER-1B279\0"
  "NUSHU CHARACTER-1B27A\0"
  "NUSHU CHARACTER-1B27B\0"
  "NUSHU CHARACTER-1B27C\0"
  "NUSHU CHARACTER-1B27D\0"
  "NUSHU CHARACTER-1B27E\0"
  "NUSHU CHARACTER-1B27F\0"
  "NUSHU CHARACTER-1B280\0"
  "NUSHU CHARACTER-1B281\0"
  "NUSHU CHARACTER-1B282\0"
  "NUSHU CHARACTER-1B283\0"
  "NUSHU CHARACTER-1B284\0"
  "NUSHU CHARACTER-1B285\0"
  "NUSHU CHARACTER-1B286\0"
  "NUSHU CHARACTER-1B287\0"
  "NUSHU CHARACTER-1B288\0"
  "NUSHU CHARACTER-1B289\0"
  "NUSHU CHARACTER-1B28A\0"
  "NUSHU CHARACTER-1B28B\0"
  "NUSHU CHARACTER-1B28C\0"
  "NUSHU CHARACTER-1B28D\0"
  "NUSHU CHARACTER-1B28E\0"
  "NUSHU CHARACTER-1B28F\0"
  "NUSHU CHARACTER-1B290\0"
  "NUSHU CHARACTER-1B291\0"
  "NUSHU CHARACTER-1B292\0"
  "NUSHU CHARACTER-1B293\0"
  "NUSHU CHARACTER-1B294\0"
  "NUSHU CHARACTER-1B295\0"
  "NUSHU CHARACTER-1B296\0"
  "NUSHU CHARACTER-1B297\0"
  "NUSHU CHARACTER-1B298\0"
  "NUSHU CHARACTER-1B299\0"
  "NUSHU CHARACTER-1B29A\0"
  "NUSHU CHARACTER-1B29B\0"
  "NUSHU CHARACTER-1B29C\0"
  "NUSHU CHARACTER-1B29D\0"
  "NUSHU CHARACTER-1B29E\0"
  "NUSHU CHARACTER-1B29F\0"
  "NUSHU CHARACTER-1B2A0\0"
  "NUSHU CHARACTER-1B2A1\0"
  "NUSHU CHARACTER-1B2A2\0"
  "NUSHU CHARACTER-1B2A3\0"
  "NUSHU CHARACTER-1B2A4\0"
  "NUSHU CHARACTER-1B2A5\0"
  "NUSHU CHARACTER-1B2A6\0"
  "NUSHU CHARACTER-1B2A7\0"
  "NUSHU CHARACTER-1B2A8\0"
  "NUSHU CHARACTER-1B2A9\0"
  "NUSHU CHARACTER-1B2AA\0"
  "NUSHU CHARACTER-1B2AB\0"
  "NUSHU CHARACTER-1B2AC\0"
  "NUSHU CHARACTER-1B2AD\0"
  "NUSHU CHARACTER-1B2AE\0"
  "NUSHU CHARACTER-1B2AF\0"
  "NUSHU CHARACTER-1B2B0\0"
  "NUSHU CHARACTER-1B2B1\0"
  "NUSHU CHARACTER-1B2B2\0"
  "NUSHU CHARACTER-1B2B3\0"
  "NUSHU CHARACTER-1B2B4\0"
  "NUSHU CHARACTER-1B2B5\0"
  "NUSHU CHARACTER-1B2B6\0"
  "NUSHU CHARACTER-1B2B7\0"
  "NUSHU CHARACTER-1B2B8\0"
  "NUSHU CHARACTER-1B2B9\0"
  "NUSHU CHARACTER-1B2BA\0"
  "NUSHU CHARACTER-1B2BB\0"
  "NUSHU CHARACTER-1B2BC\0"
  "NUSHU CHARACTER-1B2BD\0"
  "NUSHU CHARACTER-1B2BE\0"
  "NUSHU CHARACTER-1B2BF\0"
  "NUSHU CHARACTER-1B2C0\0"
  "NUSHU CHARACTER-1B2C1\0"
  "NUSHU CHARACTER-1B2C2\0"
  "NUSHU CHARACTER-1B2C3\0"
  "NUSHU CHARACTER-1B2C4\0"
  "NUSHU CHARACTER-1B2C5\0"
  "NUSHU CHARACTER-1B2C6\0"
  "NUSHU CHARACTER-1B2C7\0"
  "NUSHU CHARACTER-1B2C8\0"
  "NUSHU CHARACTER-1B2C9\0"
  "NUSHU CHARACTER-1B2CA\0"
  "NUSHU CHARACTER-1B2CB\0"
  "NUSHU CHARACTER-1B2CC\0"
  "NUSHU CHARACTER-1B2CD\0"
  "NUSHU CHARACTER-1B2CE\0"
  "NUSHU CHARACTER-1B2CF\0"
  "NUSHU CHARACTER-1B2D0\0"
  "NUSHU CHARACTER-1B2D1\0"
  "NUSHU CHARACTER-1B2D2\0"
  "NUSHU CHARACTER-1B2D3\0"
  "NUSHU CHARACTER-1B2D4\0"
  "NUSHU CHARACTER-1B2D5\0"
  "NUSHU CHARACTER-1B2D6\0"
  "NUSHU CHARACTER-1B2D7\0"
  "NUSHU CHARACTER-1B2D8\0"
  "NUSHU CHARACTER-1B2D9\0"
  "NUSHU CHARACTER-1B2DA\0"
  "NUSHU CHARACTER-1B2DB\0"
  "NUSHU CHARACTER-1B2DC\0"
  "NUSHU CHARACTER-1B2DD\0"
  "NUSHU CHARACTER-1B2DE\0"
  "NUSHU CHARACTER-1B2DF\0"
  "NUSHU CHARACTER-1B2E0\0"
  "NUSHU CHARACTER-1B2E1\0"
  "NUSHU CHARACTER-1B2E2\0"
  "NUSHU CHARACTER-1B2E3\0"
  "NUSHU CHARACTER-1B2E4\0"
  "NUSHU CHARACTER-1B2E5\0"
  "NUSHU CHARACTER-1B2E6\0"
  "NUSHU CHARACTER-1B2E7\0"
  "NUSHU CHARACTER-1B2E8\0"
  "NUSHU CHARACTER-1B2E9\0"
  "NUSHU CHARACTER-1B2EA\0"
  "NUSHU CHARACTER-1B2EB\0"
  "NUSHU CHARACTER-1B2EC\0"
  "NUSHU CHARACTER-1B2ED\0"
  "NUSHU CHARACTER-1B2EE\0"
  "NUSHU CHARACTER-1B2EF\0"
  "NUSHU CHARACTER-1B2F0\0"
  "NUSHU CHARACTER-1B2F1\0"
  "NUSHU CHARACTER-1B2F2\0"
  "NUSHU CHARACTER-1B2F3\0"
  "NUSHU CHARACTER-1B2F4\0"
  "NUSHU CHARACTER-1B2F5\0"
  "NUSHU CHARACTER-1B2F6\0"
  "NUSHU CHARACTER-1B2F7\0"
  "NUSHU CHARACTER-1B2F8\0"
  "NUSHU CHARACTER-1B2F9\0"
  "NUSHU CHARACTER-1B2FA\0"
  "NUSHU CHARACTER-1B2FB\0"
  "NUSHU ITERATION MARK\0"
  "NUT AND BOLT\0"
  "NYIAKENG PUACHUE HMONG CIRCLED CA\0"
  "NYIAKENG PUACHUE HMONG DIGIT EIGHT\0"
  "NYIAKENG PUACHUE HMONG DIGIT FIVE\0"
  "NYIAKENG PUACHUE HMONG DIGIT FOUR\0"
  "NYIAKENG PUACHUE HMONG DIGIT NINE\0"
  "NYIAKENG PUACHUE HMONG DIGIT ONE\0"
  "NYIAKENG PUACHUE HMONG DIGIT SEVEN\0"
  "NYIAKENG PUACHUE HMONG DIGIT SIX\0"
  "NYIAKENG PUACHUE HMONG DIGIT THREE\0"
  "NYIAKENG PUACHUE HMONG DIGIT TWO\0"
  "NYIAKENG PUACHUE HMONG DIGIT ZERO\0"
  "NYIAKENG PUACHUE HMONG LETTER A\0"
  "NYIAKENG PUACHUE HMONG LETTER AA\0"
  "NYIAKENG PUACHUE HMONG LETTER CA\0"
  "NYIAKENG PUACHUE HMONG LETTER DA\0"
  "NYIAKENG PUACHUE HMONG LETTER DLA\0"
  "NYIAKENG PUACHUE HMONG LETTER E\0"
  "NYIAKENG PUACHUE HMONG LETTER EE\0"
  "NYIAKENG PUACHUE HMONG LETTER FA\0"
  "NYIAKENG PUACHUE HMONG LETTER GA\0"
  "NYIAKENG PUACHUE HMONG LETTER HA\0"
  "NYIAKENG PUACHUE HMONG LETTER HAH\0"
  "NYIAKENG PUACHUE HMONG LETTER I\0"
  "NYIAKENG PUACHUE HMONG LETTER KA\0"
  "NYIAKENG PUACHUE HMONG LETTER LA\0"
  "NYIAKENG PUACHUE HMONG LETTER MA\0"
  "NYIAKENG PUACHUE HMONG LETTER MLA\0"
  "NYIAKENG PUACHUE HMONG LETTER NA\0"
  "NYIAKENG PUACHUE HMONG LETTER NCA\0"
  "NYIAKENG PUACHUE HMONG LETTER NKA\0"
  "NYIAKENG PUACHUE HMONG LETTER NPA\0"
  "NYIAKENG PUACHUE HMONG LETTER NPLA\0"
  "NYIAKENG PUACHUE HMONG LETTER NQA\0"
  "NYIAKENG PUACHUE HMONG LETTER NRA\0"
  "NYIAKENG PUACHUE HMONG LETTER NTA\0"
  "NYIAKENG PUACHUE HMONG LETTER NTSA\0"
  "NYIAKENG PUACHUE HMONG LETTER NTXA\0"
  "NYIAKENG PUACHUE HMONG LETTER NYA\0"
  "NYIAKENG PUACHUE HMONG LETTER O\0"
  "NYIAKENG PUACHUE HMONG LETTER OO\0"
  "NYIAKENG PUACHUE HMONG LETTER PA\0"
  "NYIAKENG PUACHUE HMONG LETTER PLA\0"
  "NYIAKENG PUACHUE HMONG LETTER QA\0"
  "NYIAKENG PUACHUE HMONG LETTER RA\0"
  "NYIAKENG PUACHUE HMONG LETTER RRA\0"
  "NYIAKENG PUACHUE HMONG LETTER SA\0"
  "NYIAKENG PUACHUE HMONG LETTER TA\0"
  "NYIAKENG PUACHUE HMONG LETTER TSA\0"
  "NYIAKENG PUACHUE HMONG LETTER TXA\0"
  "NYIAKENG PUACHUE HMONG LETTER U\0"
  "NYIAKENG PUACHUE HMONG LETTER VA\0"
  "NYIAKENG PUACHUE HMONG LETTER W\0"
  "NYIAKENG PUACHUE HMONG LETTER XA\0"
  "NYIAKENG PUACHUE HMONG LETTER XYA\0"
  "NYIAKENG PUACHUE HMONG LETTER YA\0"
  "NYIAKENG PUACHUE HMONG LETTER ZA\0"
  "NYIAKENG PUACHUE HMONG LOGOGRAM NYAJ\0"
  "NYIAKENG PUACHUE HMONG SIGN FOR ANIMAL\0"
  "NYIAKENG PUACHUE HMONG SIGN FOR INVERTEBRATE\0"
  "NYIAKENG PUACHUE HMONG SIGN FOR LOCATION\0"
  "NYIAKENG PUACHUE HMONG SIGN FOR PERSON\0"
  "NYIAKENG PUACHUE HMONG SIGN FOR THING\0"
  "NYIAKENG PUACHUE HMONG SIGN XW XW\0"
  "NYIAKENG PUACHUE HMONG SYLLABLE LENGTHENER\0"
  "NYIAKENG PUACHUE HMONG TONE-B\0"
  "NYIAKENG PUACHUE HMONG TONE-D\0"
  "NYIAKENG PUACHUE HMONG TONE-G\0"
  "NYIAKENG PUACHUE HMONG TONE-J\0"
  "NYIAKENG PUACHUE HMONG TONE-M\0"
  "NYIAKENG PUACHUE HMONG TONE-S\0"
  "NYIAKENG PUACHUE HMONG TONE-V\0"
  "OBJECT REPLACEMENT CHARACTER\0"
  "OBLIQUE ANGLE OPENING DOWN\0"
  "OBLIQUE ANGLE OPENING UP\0"
  "OBLIQUE HYPHEN\0"
  "OBSERVER EYE SYMBOL\0"
  "OCCULTATION\0"
  "OCR AMOUNT OF CHECK\0"
  "OCR BELT BUCKLE\0"
  "OCR BOW TIE\0"
  "OCR BRANCH BANK IDENTIFICATION\0"
  "OCR CHAIR\0"
  "OCR CUSTOMER ACCOUNT NUMBER\0"
  "OCR DASH\0"
  "OCR DOUBLE BACKSLASH\0"
  "OCR FORK\0"
  "OCR HOOK\0"
  "OCR INVERTED FORK\0"
  "OCTAGONAL SIGN\0"
  "OCTOPUS\0"
  "ODEN\0"
  "OFFICE BUILDING\0"
  "OGHAM FEATHER MARK\0"
  "OGHAM LETTER AILM\0"
  "OGHAM LETTER BEITH\0"
  "OGHAM LETTER CEIRT\0"
  "OGHAM LETTER COLL\0"
  "OGHAM LETTER DAIR\0"
  "OGHAM LETTER EABHADH\0"
  "OGHAM LETTER EADHADH\0"
  "OGHAM LETTER EAMHANCHOLL\0"
  "OGHAM LETTER FEARN\0"
  "OGHAM LETTER GORT\0"
  "OGHAM LETTER IFIN\0"
  "OGHAM LETTER IODHADH\0"
  "OGHAM LETTER LUIS\0"
  "OGHAM LETTER MUIN\0"
  "OGHAM LETTER NGEADAL\0"
  "OGHAM LETTER NION\0"
  "OGHAM LETTER ONN\0"
  "OGHAM LETTER OR\0"
  "OGHAM LETTER PEITH\0"
  "OGHAM LETTER RUIS\0"
  "OGHAM LETTER SAIL\0"
  "OGHAM LETTER STRAIF\0"
  "OGHAM LETTER TINNE\0"
  "OGHAM LETTER UATH\0"
  "OGHAM LETTER UILLEANN\0"
  "OGHAM LETTER UR\0"
  "OGHAM REVERSED FEATHER MARK\0"
  "OGHAM SPACE MARK\0"
  "OGONEK\0"
  "OHM SIGN\0"
  "OIL DRUM\0"
  "OK HAND SIGN\0"
  "OL CHIKI AHAD\0"
  "OL CHIKI DIGIT EIGHT\0"
  "OL CHIKI DIGIT FIVE\0"
  "OL CHIKI DIGIT FOUR\0"
  "OL CHIKI DIGIT NINE\0"
  "OL CHIKI DIGIT ONE\0"
  "OL CHIKI DIGIT SEVEN\0"
  "OL CHIKI DIGIT SIX\0"
  "OL CHIKI DIGIT THREE\0"
  "OL CHIKI DIGIT TWO\0"
  "OL CHIKI DIGIT ZERO\0"
  "OL CHIKI GAAHLAA TTUDDAAG\0"
  "OL CHIKI LETTER AAJ\0"
  "OL CHIKI LETTER AAK\0"
  "OL CHIKI LETTER AAM\0"
  "OL CHIKI LETTER AAW\0"
  "OL CHIKI LETTER AG\0"
  "OL CHIKI LETTER AL\0"
  "OL CHIKI LETTER ANG\0"
  "OL CHIKI LETTER AT\0"
  "OL CHIKI LETTER EDD\0"
  "OL CHIKI LETTER EN\0"
  "OL CHIKI LETTER EP\0"
  "OL CHIKI LETTER ERR\0"
  "OL CHIKI LETTER IH\0"
  "OL CHIKI LETTER INY\0"
  "OL CHIKI LETTER IR\0"
  "OL CHIKI LETTER IS\0"
  "OL CHIKI LETTER LA\0"
  "OL CHIKI LETTER LAA\0"
  "OL CHIKI LETTER LE\0"
  "OL CHIKI LETTER LI\0"
  "OL CHIKI LETTER LO\0"
  "OL CHIKI LETTER LU\0"
  "OL CHIKI LETTER OB\0"
  "OL CHIKI LETTER OH\0"
  "OL CHIKI LETTER OTT\0"
  "OL CHIKI LETTER OV\0"
  "OL CHIKI LETTER UC\0"
  "OL CHIKI LETTER UD\0"
  "OL CHIKI LETTER UNN\0"
  "OL CHIKI LETTER UY\0"
  "OL CHIKI MU TTUDDAG\0"
  "OL CHIKI MU-GAAHLAA TTUDDAAG\0"
  "OL CHIKI PHAARKAA\0"
  "OL CHIKI PUNCTUATION DOUBLE MUCAAD\0"
  "OL CHIKI PUNCTUATION MUCAAD\0"
  "OL CHIKI RELAA\0"
  "OL ONAL ABBREVIATION SIGN\0"
  "OL ONAL DIGIT EIGHT\0"
  "OL ONAL DIGIT FIVE\0"
  "OL ONAL DIGIT FOUR\0"
  "OL ONAL DIGIT NINE\0"
  "OL ONAL DIGIT ONE\0"
  "OL ONAL DIGIT SEVEN\0"
  "OL ONAL DIGIT SIX\0"
  "OL ONAL DIGIT THREE\0"
  "OL ONAL DIGIT TWO\0"
  "OL ONAL DIGIT ZERO\0"
  "OL ONAL LETTER A\0"
  "OL ONAL LETTER AB\0"
  "OL ONAL LETTER AD\0"
  "OL ONAL LETTER AH\0"
  "OL ONAL LETTER AL\0"
  "OL ONAL LETTER AW\0"
  "OL ONAL LETTER E\0"
  "OL ONAL LETTER EC\0"
  "OL ONAL LETTER EG\0"
  "OL ONAL LETTER EH\0"
  "OL ONAL LETTER ENN\0"
  "OL ONAL LETTER ES\0"
  "OL ONAL LETTER I\0"
  "OL ONAL LETTER ID\0"
  "OL ONAL LETTER IN\0"
  "OL ONAL LETTER IP\0"
  "OL ONAL LETTER IT\0"
  "OL ONAL LETTER ITT\0"
  "OL ONAL LETTER O\0"
  "OL ONAL LETTER OM\0"
  "OL ONAL LETTER ONG\0"
  "OL ONAL LETTER OO\0"
  "OL ONAL LETTER ORR\0"
  "OL ONAL LETTER OY\0"
  "OL ONAL LETTER U\0"
  "OL ONAL LETTER UDD\0"
  "OL ONAL LETTER UJ\0"
  "OL ONAL LETTER UK\0"
  "OL ONAL LETTER UNY\0"
  "OL ONAL LETTER UR\0"
  "OL ONAL SIGN HODDOND\0"
  "OL ONAL SIGN IKIR\0"
  "OL ONAL SIGN MU\0"
  "OLD CHINESE HOOK MARK\0"
  "OLD CHINESE ITERATION MARK\0"
  "OLD HUNGARIAN CAPITAL LETTER A\0"
  "OLD HUNGARIAN CAPITAL LETTER AA\0"
  "OLD HUNGARIAN CAPITAL LETTER AK\0"
  "OLD HUNGARIAN CAPITAL LETTER AMB\0"
  "OLD HUNGARIAN CAPITAL LETTER AND\0"
  "OLD HUNGARIAN CAPITAL LETTER CLOSE E\0"
  "OLD HUNGARIAN CAPITAL LETTER E\0"
  "OLD HUNGARIAN CAPITAL LETTER EB\0"
  "OLD HUNGARIAN CAPITAL LETTER EC\0"
  "OLD HUNGARIAN CAPITAL LETTER ECH\0"
  "OLD HUNGARIAN CAPITAL LETTER ECS\0"
  "OLD HUNGARIAN CAPITAL LETTER ED\0"
  "OLD HUNGARIAN CAPITAL LETTER EE\0"
  "OLD HUNGARIAN CAPITAL LETTER EF\0"
  "OLD HUNGARIAN CAPITAL LETTER EG\0"
  "OLD HUNGARIAN CAPITAL LETTER EGY\0"
  "OLD HUNGARIAN CAPITAL LETTER EH\0"
  "OLD HUNGARIAN CAPITAL LETTER EJ\0"
  "OLD HUNGARIAN CAPITAL LETTER EK\0"
  "OLD HUNGARIAN CAPITAL LETTER EL\0"
  "OLD HUNGARIAN CAPITAL LETTER ELY\0"
  "OLD HUNGARIAN CAPITAL LETTER EM\0"
  "OLD HUNGARIAN CAPITAL LETTER EMP\0"
  "OLD HUNGARIAN CAPITAL LETTER EN\0"
  "OLD HUNGARIAN CAPITAL LETTER ENC\0"
  "OLD HUNGARIAN CAPITAL LETTER ENT\0"
  "OLD HUNGARIAN CAPITAL LETTER ENT-SHAPED SIGN\0"
  "OLD HUNGARIAN CAPITAL LETTER ENY\0"
  "OLD HUNGARIAN CAPITAL LETTER EP\0"
  "OLD HUNGARIAN CAPITAL LETTER ER\0"
  "OLD HUNGARIAN CAPITAL LETTER ES\0"
  "OLD HUNGARIAN CAPITAL LETTER ESZ\0"
  "OLD HUNGARIAN CAPITAL LETTER ET\0"
  "OLD HUNGARIAN CAPITAL LETTER ETY\0"
  "OLD HUNGARIAN CAPITAL LETTER EV\0"
  "OLD HUNGARIAN CAPITAL LETTER EZ\0"
  "OLD HUNGARIAN CAPITAL LETTER EZS\0"
  "OLD HUNGARIAN CAPITAL LETTER I\0"
  "OLD HUNGARIAN CAPITAL LETTER II\0"
  "OLD HUNGARIAN CAPITAL LETTER NIKOLSBURG OE\0"
  "OLD HUNGARIAN CAPITAL LETTER NIKOLSBURG UE\0"
  "OLD HUNGARIAN CAPITAL LETTER O\0"
  "OLD HUNGARIAN CAPITAL LETTER OEE\0"
  "OLD HUNGARIAN CAPITAL LETTER OO\0"
  "OLD HUNGARIAN CAPITAL LETTER RUDIMENTA OE\0"
  "OLD HUNGARIAN CAPITAL LETTER RUDIMENTA UE\0"
  "OLD HUNGARIAN CAPITAL LETTER SHORT ER\0"
  "OLD HUNGARIAN CAPITAL LETTER U\0"
  "OLD HUNGARIAN CAPITAL LETTER UNK\0"
  "OLD HUNGARIAN CAPITAL LETTER US\0"
  "OLD HUNGARIAN CAPITAL LETTER UU\0"
  "OLD HUNGARIAN NUMBER FIFTY\0"
  "OLD HUNGARIAN NUMBER FIVE\0"
  "OLD HUNGARIAN NUMBER ONE\0"
  "OLD HUNGARIAN NUMBER ONE HUNDRED\0"
  "OLD HUNGARIAN NUMBER ONE THOUSAND\0"
  "OLD HUNGARIAN NUMBER TEN\0"
  "OLD HUNGARIAN SMALL LETTER A\0"
  "OLD HUNGARIAN SMALL LETTER AA\0"
  "OLD HUNGARIAN SMALL LETTER AK\0"
  "OLD HUNGARIAN SMALL LETTER AMB\0"
  "OLD HUNGARIAN SMALL LETTER AND\0"
  "OLD HUNGARIAN SMALL LETTER CLOSE E\0"
  "OLD HUNGARIAN SMALL LETTER E\0"
  "OLD HUNGARIAN SMALL LETTER EB\0"
  "OLD HUNGARIAN SMALL LETTER EC\0"
  "OLD HUNGARIAN SMALL LETTER ECH\0"
  "OLD HUNGARIAN SMALL LETTER ECS\0"
  "OLD HUNGARIAN SMALL LETTER ED\0"
  "OLD HUNGARIAN SMALL LETTER EE\0"
  "OLD HUNGARIAN SMALL LETTER EF\0"
  "OLD HUNGARIAN SMALL LETTER EG\0"
  "OLD HUNGARIAN SMALL LETTER EGY\0"
  "OLD HUNGARIAN SMALL LETTER EH\0"
  "OLD HUNGARIAN SMALL LETTER EJ\0"
  "OLD HUNGARIAN SMALL LETTER EK\0"
  "OLD HUNGARIAN SMALL LETTER EL\0"
  "OLD HUNGARIAN SMALL LETTER ELY\0"
  "OLD HUNGARIAN SMALL LETTER EM\0"
  "OLD HUNGARIAN SMALL LETTER EMP\0"
  "OLD HUNGARIAN SMALL LETTER EN\0"
  "OLD HUNGARIAN SMALL LETTER ENC\0"
  "OLD HUNGARIAN SMALL LETTER ENT\0"
  "OLD HUNGARIAN SMALL LETTER ENT-SHAPED SIGN\0"
  "OLD HUNGARIAN SMALL LETTER ENY\0"
  "OLD HUNGARIAN SMALL LETTER EP\0"
  "OLD HUNGARIAN SMALL LETTER ER\0"
  "OLD HUNGARIAN SMALL LETTER ES\0"
  "OLD HUNGARIAN SMALL LETTER ESZ\0"
  "OLD HUNGARIAN SMALL LETTER ET\0"
  "OLD HUNGARIAN SMALL LETTER ETY\0"
  "OLD HUNGARIAN SMALL LETTER EV\0"
  "OLD HUNGARIAN SMALL LETTER EZ\0"
  "OLD HUNGARIAN SMALL LETTER EZS\0"
  "OLD HUNGARIAN SMALL LETTER I\0"
  "OLD HUNGARIAN SMALL LETTER II\0"
  "OLD HUNGARIAN SMALL LETTER NIKOLSBURG OE\0"
  "OLD HUNGARIAN SMALL LETTER NIKOLSBURG UE\0"
  "OLD HUNGARIAN SMALL LETTER O\0"
  "OLD HUNGARIAN SMALL LETTER OEE\0"
  "OLD HUNGARIAN SMALL LETTER OO\0"
  "OLD HUNGARIAN SMALL LETTER RUDIMENTA OE\0"
  "OLD HUNGARIAN SMALL LETTER RUDIMENTA UE\0"
  "OLD HUNGARIAN SMALL LETTER SHORT ER\0"
  "OLD HUNGARIAN SMALL LETTER U\0"
  "OLD HUNGARIAN SMALL LETTER UNK\0"
  "OLD HUNGARIAN SMALL LETTER US\0"
  "OLD HUNGARIAN SMALL LETTER UU\0"
  "OLD ITALIC LETTER A\0"
  "OLD ITALIC LETTER BE\0"
  "OLD ITALIC LETTER CHE\0"
  "OLD ITALIC LETTER DE\0"
  "OLD ITALIC LETTER E\0"
  "OLD ITALIC LETTER EF\0"
  "OLD ITALIC LETTER EKS\0"
  "OLD ITALIC LETTER EL\0"
  "OLD ITALIC LETTER EM\0"
  "OLD ITALIC LETTER EN\0"
  "OLD ITALIC LETTER ER\0"
  "OLD ITALIC LETTER ERS\0"
  "OLD ITALIC LETTER ES\0"
  "OLD ITALIC LETTER ESH\0"
  "OLD ITALIC LETTER ESS\0"
  "OLD ITALIC LETTER HE\0"
  "OLD ITALIC LETTER I\0"
  "OLD ITALIC LETTER II\0"
  "OLD ITALIC LETTER KA\0"
  "OLD ITALIC LETTER KE\0"
  "OLD ITALIC LETTER KHE\0"
  "OLD ITALIC LETTER KU\0"
  "OLD ITALIC LETTER NORTHERN TSE\0"
  "OLD ITALIC LETTER O\0"
  "OLD ITALIC LETTER PE\0"
  "OLD ITALIC LETTER PHE\0"
  "OLD ITALIC LETTER SHE\0"
  "OLD ITALIC LETTER SOUTHERN TSE\0"
  "OLD ITALIC LETTER TE\0"
  "OLD ITALIC LETTER THE\0"
  "OLD ITALIC LETTER U\0"
  "OLD ITALIC LETTER UU\0"
  "OLD ITALIC LETTER VE\0"
  "OLD ITALIC LETTER YE\0"
  "OLD ITALIC LETTER ZE\0"
  "OLD ITALIC NUMERAL FIFTY\0"
  "OLD ITALIC NUMERAL FIVE\0"
  "OLD ITALIC NUMERAL ONE\0"
  "OLD ITALIC NUMERAL TEN\0"
  "OLD KEY\0"
  "OLD NORTH ARABIAN LETTER AIN\0"
  "OLD NORTH ARABIAN LETTER ALEF\0"
  "OLD NORTH ARABIAN LETTER BEH\0"
  "OLD NORTH ARABIAN LETTER DAD\0"
  "OLD NORTH ARABIAN LETTER DAL\0"
  "OLD NORTH ARABIAN LETTER ES-1\0"
  "OLD NORTH ARABIAN LETTER ES-2\0"
  "OLD NORTH ARABIAN LETTER ES-3\0"
  "OLD NORTH ARABIAN LETTER FEH\0"
  "OLD NORTH ARABIAN LETTER GEEM\0"
  "OLD NORTH ARABIAN LETTER GHAIN\0"
  "OLD NORTH ARABIAN LETTER HAH\0"
  "OLD NORTH ARABIAN LETTER HEH\0"
  "OLD NORTH ARABIAN LETTER KAF\0"
  "OLD NORTH ARABIAN LETTER KHAH\0"
  "OLD NORTH ARABIAN LETTER LAM\0"
  "OLD NORTH ARABIAN LETTER MEEM\0"
  "OLD NORTH ARABIAN LETTER NOON\0"
  "OLD NORTH ARABIAN LETTER QAF\0"
  "OLD NORTH ARABIAN LETTER REH\0"
  "OLD NORTH ARABIAN LETTER SAD\0"
  "OLD NORTH ARABIAN LETTER TAH\0"
  "OLD NORTH ARABIAN LETTER TEH\0"
  "OLD NORTH ARABIAN LETTER THAL\0"
  "OLD NORTH ARABIAN LETTER THEH\0"
  "OLD NORTH ARABIAN LETTER WAW\0"
  "OLD NORTH ARABIAN LETTER YEH\0"
  "OLD NORTH ARABIAN LETTER ZAH\0"
  "OLD NORTH ARABIAN LETTER ZAIN\0"
  "OLD NORTH ARABIAN NUMBER ONE\0"
  "OLD NORTH ARABIAN NUMBER TEN\0"
  "OLD NORTH ARABIAN NUMBER TWENTY\0"
  "OLD PERMIC LETTER AN\0"
  "OLD PERMIC LETTER BUR\0"
  "OLD PERMIC LETTER CHERY\0"
  "OLD PERMIC LETTER DOI\0"
  "OLD PERMIC LETTER DZHOI\0"
  "OLD PERMIC LETTER DZITA\0"
  "OLD PERMIC LETTER E\0"
  "OLD PERMIC LETTER EF\0"
  "OLD PERMIC LETTER GAI\0"
  "OLD PERMIC LETTER HA\0"
  "OLD PERMIC LETTER I\0"
  "OLD PERMIC LETTER IA\0"
  "OLD PERMIC LETTER IE\0"
  "OLD PERMIC LETTER KOKE\0"
  "OLD PERMIC LETTER LEI\0"
  "OLD PERMIC LETTER MENOE\0"
  "OLD PERMIC LETTER NENOE\0"
  "OLD PERMIC LETTER O\0"
  "OLD PERMIC LETTER OO\0"
  "OLD PERMIC LETTER PEEI\0"
  "OLD PERMIC LETTER REI\0"
  "OLD PERMIC LETTER SHCHOOI\0"
  "OLD PERMIC LETTER SHOOI\0"
  "OLD PERMIC LETTER SII\0"
  "OLD PERMIC LETTER TAI\0"
  "OLD PERMIC LETTER TSIU\0"
  "OLD PERMIC LETTER U\0"
  "OLD PERMIC LETTER VER\0"
  "OLD PERMIC LETTER VOOI\0"
  "OLD PERMIC LETTER YA\0"
  "OLD PERMIC LETTER YAT\0"
  "OLD PERMIC LETTER YER\0"
  "OLD PERMIC LETTER YERI\0"
  "OLD PERMIC LETTER YERU\0"
  "OLD PERMIC LETTER YRY\0"
  "OLD PERMIC LETTER YU\0"
  "OLD PERMIC LETTER ZATA\0"
  "OLD PERMIC LETTER ZHOI\0"
  "OLD PERSIAN NUMBER HUNDRED\0"
  "OLD PERSIAN NUMBER ONE\0"
  "OLD PERSIAN NUMBER TEN\0"
  "OLD PERSIAN NUMBER TWENTY\0"
  "OLD PERSIAN NUMBER TWO\0"
  "OLD PERSIAN SIGN A\0"
  "OLD PERSIAN SIGN AURAMAZDAA\0"
  "OLD PERSIAN SIGN AURAMAZDAA-2\0"
  "OLD PERSIAN SIGN AURAMAZDAAHA\0"
  "OLD PERSIAN SIGN BA\0"
  "OLD PERSIAN SIGN BAGA\0"
  "OLD PERSIAN SIGN BUUMISH\0"
  "OLD PERSIAN SIGN CA\0"
  "OLD PERSIAN SIGN DA\0"
  "OLD PERSIAN SIGN DAHYAAUSH\0"
  "OLD PERSIAN SIGN DAHYAAUSH-2\0"
  "OLD PERSIAN SIGN DI\0"
  "OLD PERSIAN SIGN DU\0"
  "OLD PERSIAN SIGN FA\0"
  "OLD PERSIAN SIGN GA\0"
  "OLD PERSIAN SIGN GU\0"
  "OLD PERSIAN SIGN HA\0"
  "OLD PERSIAN SIGN I\0"
  "OLD PERSIAN SIGN JA\0"
  "OLD PERSIAN SIGN JI\0"
  "OLD PERSIAN SIGN KA\0"
  "OLD PERSIAN SIGN KU\0"
  "OLD PERSIAN SIGN LA\0"
  "OLD PERSIAN SIGN MA\0"
  "OLD PERSIAN SIGN MI\0"
  "OLD PERSIAN SIGN MU\0"
  "OLD PERSIAN SIGN NA\0"
  "OLD PERSIAN SIGN NU\0"
  "OLD PERSIAN SIGN PA\0"
  "OLD PERSIAN SIGN RA\0"
  "OLD PERSIAN SIGN RU\0"
  "OLD PERSIAN SIGN SA\0"
  "OLD PERSIAN SIGN SHA\0"
  "OLD PERSIAN SIGN SSA\0"
  "OLD PERSIAN SIGN TA\0"
  "OLD PERSIAN SIGN THA\0"
  "OLD PERSIAN SIGN TU\0"
  "OLD PERSIAN SIGN U\0"
  "OLD PERSIAN SIGN VA\0"
  "OLD PERSIAN SIGN VI\0"
  "OLD PERSIAN SIGN XA\0"
  "OLD PERSIAN SIGN XSHAAYATHIYA\0"
  "OLD PERSIAN SIGN YA\0"
  "OLD PERSIAN SIGN ZA\0"
  "OLD PERSIAN WORD DIVIDER\0"
  "OLD PERSONAL COMPUTER\0"
  "OLD PERSONAL COMPUTER WITH MONITOR IN PORTRAIT ORIENTATION\0"
  "OLD SOGDIAN FRACTION ONE HALF\0"
  "OLD SOGDIAN LETTER ALEPH\0"
  "OLD SOGDIAN LETTER ALTERNATE AYIN\0"
  "OLD SOGDIAN LETTER AYIN\0"
  "OLD SOGDIAN LETTER BETH\0"
  "OLD SOGDIAN LETTER FINAL ALEPH\0"
  "OLD SOGDIAN LETTER FINAL BETH\0"
  "OLD SOGDIAN LETTER FINAL HE\0"
  "OLD SOGDIAN LETTER FINAL NUN\0"
  "OLD SOGDIAN LETTER FINAL NUN WITH VERTICAL TAIL\0"
  "OLD SOGDIAN LETTER FINAL SADHE\0"
  "OLD SOGDIAN LETTER FINAL SADHE WITH VERTICAL TAIL\0"
  "OLD SOGDIAN LETTER FINAL TAW\0"
  "OLD SOGDIAN LETTER FINAL TAW WITH VERTICAL TAIL\0"
  "OLD SOGDIAN LETTER GIMEL\0"
  "OLD SOGDIAN LETTER HE\0"
  "OLD SOGDIAN LETTER HETH\0"
  "OLD SOGDIAN LETTER KAPH\0"
  "OLD SOGDIAN LETTER LAMEDH\0"
  "OLD SOGDIAN LETTER MEM\0"
  "OLD SOGDIAN LETTER NUN\0"
  "OLD SOGDIAN LETTER PE\0"
  "OLD SOGDIAN LETTER RESH-AYIN-DALETH\0"
  "OLD SOGDIAN LETTER SADHE\0"
  "OLD SOGDIAN LETTER SAMEKH\0"
  "OLD SOGDIAN LETTER SHIN\0"
  "OLD SOGDIAN LETTER TAW\0"
  "OLD SOGDIAN LETTER WAW\0"
  "OLD SOGDIAN LETTER YODH\0"
  "OLD SOGDIAN LETTER ZAYIN\0"
  "OLD SOGDIAN LIGATURE AYIN-DALETH\0"
  "OLD SOGDIAN NUMBER FIVE\0"
  "OLD SOGDIAN NUMBER FOUR\0"
  "OLD SOGDIAN NUMBER ONE\0"
  "OLD SOGDIAN NUMBER ONE HUNDRED\0"
  "OLD SOGDIAN NUMBER TEN\0"
  "OLD SOGDIAN NUMBER THIRTY\0"
  "OLD SOGDIAN NUMBER THREE\0"
  "OLD SOGDIAN NUMBER TWENTY\0"
  "OLD SOGDIAN NUMBER TWO\0"
  "OLD SOUTH ARABIAN LETTER ALEF\0"
  "OLD SOUTH ARABIAN LETTER AYN\0"
  "OLD SOUTH ARABIAN LETTER BETH\0"
  "OLD SOUTH ARABIAN LETTER DALETH\0"
  "OLD SOUTH ARABIAN LETTER DHADHE\0"
  "OLD SOUTH ARABIAN LETTER DHALETH\0"
  "OLD SOUTH ARABIAN LETTER FE\0"
  "OLD SOUTH ARABIAN LETTER GHAYN\0"
  "OLD SOUTH ARABIAN LETTER GIMEL\0"
  "OLD SOUTH ARABIAN LETTER HE\0"
  "OLD SOUTH ARABIAN LETTER HETH\0"
  "OLD SOUTH ARABIAN LETTER KAPH\0"
  "OLD SOUTH ARABIAN LETTER KHETH\0"
  "OLD SOUTH ARABIAN LETTER LAMEDH\0"
  "OLD SOUTH ARABIAN LETTER MEM\0"
  "OLD SOUTH ARABIAN LETTER NUN\0"
  "OLD SOUTH ARABIAN LETTER QOPH\0"
  "OLD SOUTH ARABIAN LETTER RESH\0"
  "OLD SOUTH ARABIAN LETTER SADHE\0"
  "OLD SOUTH ARABIAN LETTER SAMEKH\0"
  "OLD SOUTH ARABIAN LETTER SAT\0"
  "OLD SOUTH ARABIAN LETTER SHIN\0"
  "OLD SOUTH ARABIAN LETTER TAW\0"
  "OLD SOUTH ARABIAN LETTER TETH\0"
  "OLD SOUTH ARABIAN LETTER THAW\0"
  "OLD SOUTH ARABIAN LETTER THETH\0"
  "OLD SOUTH ARABIAN LETTER WAW\0"
  "OLD SOUTH ARABIAN LETTER YODH\0"
  "OLD SOUTH ARABIAN LETTER ZAYN\0"
  "OLD SOUTH ARABIAN NUMBER FIFTY\0"
  "OLD SOUTH ARABIAN NUMBER ONE\0"
  "OLD SOUTH ARABIAN NUMERIC INDICATOR\0"
  "OLD TURKIC LETTER ORKHON A\0"
  "OLD TURKIC LETTER ORKHON AB\0"
  "OLD TURKIC LETTER ORKHON AD\0"
  "OLD TURKIC LETTER ORKHON AEB\0"
  "OLD TURKIC LETTER ORKHON AED\0"
  "OLD TURKIC LETTER ORKHON AEG\0"
  "OLD TURKIC LETTER ORKHON AEK\0"
  "OLD TURKIC LETTER ORKHON AEL\0"
  "OLD TURKIC LETTER ORKHON AEN\0"
  "OLD TURKIC LETTER ORKHON AER\0"
  "OLD TURKIC LETTER ORKHON AES\0"
  "OLD TURKIC LETTER ORKHON AET\0"
  "OLD TURKIC LETTER ORKHON AEY\0"
  "OLD TURKIC LETTER ORKHON AG\0"
  "OLD TURKIC LETTER ORKHON AL\0"
  "OLD TURKIC LETTER ORKHON AN\0"
  "OLD TURKIC LETTER ORKHON AQ\0"
  "OLD TURKIC LETTER ORKHON AR\0"
  "OLD TURKIC LETTER ORKHON AS\0"
  "OLD TURKIC LETTER ORKHON ASH\0"
  "OLD TURKIC LETTER ORKHON AT\0"
  "OLD TURKIC LETTER ORKHON AY\0"
  "OLD TURKIC LETTER ORKHON BASH\0"
  "OLD TURKIC LETTER ORKHON EC\0"
  "OLD TURKIC LETTER ORKHON ELT\0"
  "OLD TURKIC LETTER ORKHON EM\0"
  "OLD TURKIC LETTER ORKHON ENC\0"
  "OLD TURKIC LETTER ORKHON ENG\0"
  "OLD TURKIC LETTER ORKHON ENT\0"
  "OLD TURKIC LETTER ORKHON ENY\0"
  "OLD TURKIC LETTER ORKHON EP\0"
  "OLD TURKIC LETTER ORKHON ESH\0"
  "OLD TURKIC LETTER ORKHON EZ\0"
  "OLD TURKIC LETTER ORKHON I\0"
  "OLD TURKIC LETTER ORKHON IC\0"
  "OLD TURKIC LETTER ORKHON IQ\0"
  "OLD TURKIC LETTER ORKHON O\0"
  "OLD TURKIC LETTER ORKHON OE\0"
  "OLD TURKIC LETTER ORKHON OEK\0"
  "OLD TURKIC LETTER ORKHON OP\0"
  "OLD TURKIC LETTER ORKHON OQ\0"
  "OLD TURKIC LETTER ORKHON OT\0"
  "OLD TURKIC LETTER YENISEI A\0"
  "OLD TURKIC LETTER YENISEI AB\0"
  "OLD TURKIC LETTER YENISEI AD\0"
  "OLD TURKIC LETTER YENISEI AE\0"
  "OLD TURKIC LETTER YENISEI AEB\0"
  "OLD TURKIC LETTER YENISEI AEG\0"
  "OLD TURKIC LETTER YENISEI AEK\0"
  "OLD TURKIC LETTER YENISEI AEN\0"
  "OLD TURKIC LETTER YENISEI AENG\0"
  "OLD TURKIC LETTER YENISEI AET\0"
  "OLD TURKIC LETTER YENISEI AEY\0"
  "OLD TURKIC LETTER YENISEI AG\0"
  "OLD TURKIC LETTER YENISEI AL\0"
  "OLD TURKIC LETTER YENISEI ANG\0"
  "OLD TURKIC LETTER YENISEI AQ\0"
  "OLD TURKIC LETTER YENISEI AR\0"
  "OLD TURKIC LETTER YENISEI ASH\0"
  "OLD TURKIC LETTER YENISEI AT\0"
  "OLD TURKIC LETTER YENISEI AY\0"
  "OLD TURKIC LETTER YENISEI E\0"
  "OLD TURKIC LETTER YENISEI EC\0"
  "OLD TURKIC LETTER YENISEI ENC\0"
  "OLD TURKIC LETTER YENISEI ENT\0"
  "OLD TURKIC LETTER YENISEI ENY\0"
  "OLD TURKIC LETTER YENISEI ESH\0"
  "OLD TURKIC LETTER YENISEI EZ\0"
  "OLD TURKIC LETTER YENISEI I\0"
  "OLD TURKIC LETTER YENISEI IQ\0"
  "OLD TURKIC LETTER YENISEI OE\0"
  "OLD TURKIC LETTER YENISEI OEK\0"
  "OLD TURKIC LETTER YENISEI OQ\0"
  "OLD UYGHUR COMBINING DOT ABOVE\0"
  "OLD UYGHUR COMBINING DOT BELOW\0"
  "OLD UYGHUR COMBINING TWO DOTS ABOVE\0"
  "OLD UYGHUR COMBINING TWO DOTS BELOW\0"
  "OLD UYGHUR LETTER ALEPH\0"
  "OLD UYGHUR LETTER BETH\0"
  "OLD UYGHUR LETTER FINAL HETH\0"
  "OLD UYGHUR LETTER GIMEL-HETH\0"
  "OLD UYGHUR LETTER KAPH\0"
  "OLD UYGHUR LETTER LAMEDH\0"
  "OLD UYGHUR LETTER LESH\0"
  "OLD UYGHUR LETTER MEM\0"
  "OLD UYGHUR LETTER NUN\0"
  "OLD UYGHUR LETTER PE\0"
  "OLD UYGHUR LETTER RESH\0"
  "OLD UYGHUR LETTER SADHE\0"
  "OLD UYGHUR LETTER SAMEKH\0"
  "OLD UYGHUR LETTER SHIN\0"
  "OLD UYGHUR LETTER TAW\0"
  "OLD UYGHUR LETTER WAW\0"
  "OLD UYGHUR LETTER YODH\0"
  "OLD UYGHUR LETTER ZAYIN\0"
  "OLD UYGHUR PUNCTUATION BAR\0"
  "OLD UYGHUR PUNCTUATION FOUR DOTS\0"
  "OLD UYGHUR PUNCTUATION TWO BARS\0"
  "OLD UYGHUR PUNCTUATION TWO DOTS\0"
  "OLDER ADULT\0"
  "OLDER MAN\0"
  "OLDER WOMAN\0"
  "OLIVE\0"
  "OM SYMBOL\0"
  "ON WITH EXCLAMATION MARK WITH LEFT RIGHT ARROW ABOVE\0"
  "ONCOMING AUTOMOBILE\0"
  "ONCOMING BUS\0"
  "ONCOMING FIRE ENGINE\0"
  "ONCOMING POLICE CAR\0"
  "ONCOMING TAXI\0"
  "ONE BUTTON MOUSE\0"
  "ONE DOT LEADER\0"
  "ONE DOT OVER TWO DOTS PUNCTUATION\0"
  "ONE-PIECE SWIMSUIT\0"
  "ONION\0"
  "OPEN BOOK\0"
  "OPEN BOX\0"
  "OPEN CENTRE ASTERISK\0"
  "OPEN CENTRE BLACK STAR\0"
  "OPEN CENTRE CROSS\0"
  "OPEN CENTRE TEARDROP-SPOKED ASTERISK\0"
  "OPEN FILE FOLDER\0"
  "OPEN FOLDER\0"
  "OPEN HANDS SIGN\0"
  "OPEN LOCK\0"
  "OPEN MAILBOX WITH LOWERED FLAG\0"
  "OPEN MAILBOX WITH RAISED FLAG\0"
  "OPEN SUBSET\0"
  "OPEN SUPERSET\0"
  "OPEN-CIRCUIT-OUTPUT H-TYPE SYMBOL\0"
  "OPEN-CIRCUIT-OUTPUT L-TYPE SYMBOL\0"
  "OPEN-OUTLINED RIGHTWARDS ARROW\0"
  "OPHIUCHUS\0"
  "OPPOSITION\0"
  "OPTICAL DISC\0"
  "OPTICAL DISC ICON\0"
  "OPTION KEY\0"
  "OR WITH DOT INSIDE\0"
  "ORANGE BOOK\0"
  "ORANGE HEART\0"
  "ORANGUTAN\0"
  "ORCUS\0"
  "ORIGINAL OF\0"
  "ORIYA AI LENGTH MARK\0"
  "ORIYA AU LENGTH MARK\0"
  "ORIYA DIGIT EIGHT\0"
  "ORIYA DIGIT FIVE\0"
  "ORIYA DIGIT FOUR\0"
  "ORIYA DIGIT NINE\0"
  "ORIYA DIGIT ONE\0"
  "ORIYA DIGIT SEVEN\0"
  "ORIYA DIGIT SIX\0"
  "ORIYA DIGIT THREE\0"
  "ORIYA DIGIT TWO\0"
  "ORIYA DIGIT ZERO\0"
  "ORIYA FRACTION ONE EIGHTH\0"
  "ORIYA FRACTION ONE HALF\0"
  "ORIYA FRACTION ONE QUARTER\0"
  "ORIYA FRACTION ONE SIXTEENTH\0"
  "ORIYA FRACTION THREE QUARTERS\0"
  "ORIYA FRACTION THREE SIXTEENTHS\0"
  "ORIYA ISSHAR\0"
  "ORIYA LETTER A\0"
  "ORIYA LETTER AA\0"
  "ORIYA LETTER AI\0"
  "ORIYA LETTER AU\0"
  "ORIYA LETTER BA\0"
  "ORIYA LETTER BHA\0"
  "ORIYA LETTER CA\0"
  "ORIYA LETTER CHA\0"
  "ORIYA LETTER DA\0"
  "ORIYA LETTER DDA\0"
  "ORIYA LETTER DDHA\0"
  "ORIYA LETTER DHA\0"
  "ORIYA LETTER E\0"
  "ORIYA LETTER GA\0"
  "ORIYA LETTER GHA\0"
  "ORIYA LETTER HA\0"
  "ORIYA LETTER I\0"
  "ORIYA LETTER II\0"
  "ORIYA LETTER JA\0"
  "ORIYA LETTER JHA\0"
  "ORIYA LETTER KA\0"
  "ORIYA LETTER KHA\0"
  "ORIYA LETTER LA\0"
  "ORIYA LETTER LLA\0"
  "ORIYA LETTER MA\0"
  "ORIYA LETTER NA\0"
  "ORIYA LETTER NGA\0"
  "ORIYA LETTER NNA\0"
  "ORIYA LETTER NYA\0"
  "ORIYA LETTER O\0"
  "ORIYA LETTER PA\0"
  "ORIYA LETTER PHA\0"
  "ORIYA LETTER RA\0"
  "ORIYA LETTER RHA\0"
  "ORIYA LETTER RRA\0"
  "ORIYA LETTER SA\0"
  "ORIYA LETTER SHA\0"
  "ORIYA LETTER SSA\0"
  "ORIYA LETTER TA\0"
  "ORIYA LETTER THA\0"
  "ORIYA LETTER TTA\0"
  "ORIYA LETTER TTHA\0"
  "ORIYA LETTER U\0"
  "ORIYA LETTER UU\0"
  "ORIYA LETTER VA\0"
  "ORIYA LETTER VOCALIC L\0"
  "ORIYA LETTER VOCALIC LL\0"
  "ORIYA LETTER VOCALIC R\0"
  "ORIYA LETTER VOCALIC RR\0"
  "ORIYA LETTER WA\0"
  "ORIYA LETTER YA\0"
  "ORIYA LETTER YYA\0"
  "ORIYA SIGN ANUSVARA\0"
  "ORIYA SIGN AVAGRAHA\0"
  "ORIYA SIGN CANDRABINDU\0"
  "ORIYA SIGN NUKTA\0"
  "ORIYA SIGN OVERLINE\0"
  "ORIYA SIGN VIRAMA\0"
  "ORIYA SIGN VISARGA\0"
  "ORIYA VOWEL SIGN AA\0"
  "ORIYA VOWEL SIGN AI\0"
  "ORIYA VOWEL SIGN AU\0"
  "ORIYA VOWEL SIGN E\0"
  "ORIYA VOWEL SIGN I\0"
  "ORIYA VOWEL SIGN II\0"
  "ORIYA VOWEL SIGN O\0"
  "ORIYA VOWEL SIGN U\0"
  "ORIYA VOWEL SIGN UU\0"
  "ORIYA VOWEL SIGN VOCALIC L\0"
  "ORIYA VOWEL SIGN VOCALIC LL\0"
  "ORIYA VOWEL SIGN VOCALIC R\0"
  "ORIYA VOWEL SIGN VOCALIC RR\0"
  "ORNATE LEFT PARENTHESIS\0"
  "ORNATE RIGHT PARENTHESIS\0"
  "ORTHODOX CROSS\0"
  "ORTHOGONAL CROSSHATCH FILL\0"
  "OSAGE CAPITAL LETTER A\0"
  "OSAGE CAPITAL LETTER AH\0"
  "OSAGE CAPITAL LETTER AI\0"
  "OSAGE CAPITAL LETTER AIN\0"
  "OSAGE CAPITAL LETTER BRA\0"
  "OSAGE CAPITAL LETTER CHA\0"
  "OSAGE CAPITAL LETTER DHA\0"
  "OSAGE CAPITAL LETTER E\0"
  "OSAGE CAPITAL LETTER EHCHA\0"
  "OSAGE CAPITAL LETTER EHKA\0"
  "OSAGE CAPITAL LETTER EHPA\0"
  "OSAGE CAPITAL LETTER EHTA\0"
  "OSAGE CAPITAL LETTER EHTSA\0"
  "OSAGE CAPITAL LETTER EIN\0"
  "OSAGE CAPITAL LETTER GHA\0"
  "OSAGE CAPITAL LETTER HA\0"
  "OSAGE CAPITAL LETTER HYA\0"
  "OSAGE CAPITAL LETTER I\0"
  "OSAGE CAPITAL LETTER KA\0"
  "OSAGE CAPITAL LETTER KHA\0"
  "OSAGE CAPITAL LETTER KYA\0"
  "OSAGE CAPITAL LETTER LA\0"
  "OSAGE CAPITAL LETTER MA\0"
  "OSAGE CAPITAL LETTER NA\0"
  "OSAGE CAPITAL LETTER O\0"
  "OSAGE CAPITAL LETTER OIN\0"
  "OSAGE CAPITAL LETTER PA\0"
  "OSAGE CAPITAL LETTER SA\0"
  "OSAGE CAPITAL LETTER SHA\0"
  "OSAGE CAPITAL LETTER TA\0"
  "OSAGE CAPITAL LETTER TSA\0"
  "OSAGE CAPITAL LETTER TSHA\0"
  "OSAGE CAPITAL LETTER U\0"
  "OSAGE CAPITAL LETTER WA\0"
  "OSAGE CAPITAL LETTER ZA\0"
  "OSAGE CAPITAL LETTER ZHA\0"
  "OSAGE SMALL LETTER A\0"
  "OSAGE SMALL LETTER AH\0"
  "OSAGE SMALL LETTER AI\0"
  "OSAGE SMALL LETTER AIN\0"
  "OSAGE SMALL LETTER BRA\0"
  "OSAGE SMALL LETTER CHA\0"
  "OSAGE SMALL LETTER DHA\0"
  "OSAGE SMALL LETTER E\0"
  "OSAGE SMALL LETTER EHCHA\0"
  "OSAGE SMALL LETTER EHKA\0"
  "OSAGE SMALL LETTER EHPA\0"
  "OSAGE SMALL LETTER EHTA\0"
  "OSAGE SMALL LETTER EHTSA\0"
  "OSAGE SMALL LETTER EIN\0"
  "OSAGE SMALL LETTER GHA\0"
  "OSAGE SMALL LETTER HA\0"
  "OSAGE SMALL LETTER HYA\0"
  "OSAGE SMALL LETTER I\0"
  "OSAGE SMALL LETTER KA\0"
  "OSAGE SMALL LETTER KHA\0"
  "OSAGE SMALL LETTER KYA\0"
  "OSAGE SMALL LETTER LA\0"
  "OSAGE SMALL LETTER MA\0"
  "OSAGE SMALL LETTER NA\0"
  "OSAGE SMALL LETTER O\0"
  "OSAGE SMALL LETTER OIN\0"
  "OSAGE SMALL LETTER PA\0"
  "OSAGE SMALL LETTER SA\0"
  "OSAGE SMALL LETTER SHA\0"
  "OSAGE SMALL LETTER TA\0"
  "OSAGE SMALL LETTER TSA\0"
  "OSAGE SMALL LETTER TSHA\0"
  "OSAGE SMALL LETTER U\0"
  "OSAGE SMALL LETTER WA\0"
  "OSAGE SMALL LETTER ZA\0"
  "OSAGE SMALL LETTER ZHA\0"
  "OSMANYA DIGIT EIGHT\0"
  "OSMANYA DIGIT FIVE\0"
  "OSMANYA DIGIT FOUR\0"
  "OSMANYA DIGIT NINE\0"
  "OSMANYA DIGIT ONE\0"
  "OSMANYA DIGIT SEVEN\0"
  "OSMANYA DIGIT SIX\0"
  "OSMANYA DIGIT THREE\0"
  "OSMANYA DIGIT TWO\0"
  "OSMANYA DIGIT ZERO\0"
  "OSMANYA LETTER A\0"
  "OSMANYA LETTER AA\0"
  "OSMANYA LETTER ALEF\0"
  "OSMANYA LETTER BA\0"
  "OSMANYA LETTER CAYN\0"
  "OSMANYA LETTER DEEL\0"
  "OSMANYA LETTER DHA\0"
  "OSMANYA LETTER E\0"
  "OSMANYA LETTER EE\0"
  "OSMANYA LETTER FA\0"
  "OSMANYA LETTER GA\0"
  "OSMANYA LETTER HA\0"
  "OSMANYA LETTER I\0"
  "OSMANYA LETTER JA\0"
  "OSMANYA LETTER KAAF\0"
  "OSMANYA LETTER KHA\0"
  "OSMANYA LETTER LAAN\0"
  "OSMANYA LETTER MIIN\0"
  "OSMANYA LETTER NUUN\0"
  "OSMANYA LETTER O\0"
  "OSMANYA LETTER OO\0"
  "OSMANYA LETTER QAAF\0"
  "OSMANYA LETTER RA\0"
  "OSMANYA LETTER SA\0"
  "OSMANYA LETTER SHIIN\0"
  "OSMANYA LETTER TA\0"
  "OSMANYA LETTER U\0"
  "OSMANYA LETTER WAW\0"
  "OSMANYA LETTER XA\0"
  "OSMANYA LETTER YA\0"
  "OTTER\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER EIGHT\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER FIVE\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER FOUR\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER FOUR HUNDRED\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER NINE\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER SEVEN\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER SIX\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER SIX HUNDRED\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER TEN\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER TEN THOUSAND\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER THREE\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER TWO\0"
  "OTTOMAN SIYAQ ALTERNATE NUMBER TWO THOUSAND\0"
  "OTTOMAN SIYAQ FRACTION ONE HALF\0"
  "OTTOMAN SIYAQ FRACTION ONE SIXTH\0"
  "OTTOMAN SIYAQ MARRATAN\0"
  "OTTOMAN SIYAQ NUMBER EIGHT\0"
  "OTTOMAN SIYAQ NUMBER EIGHT HUNDRED\0"
  "OTTOMAN SIYAQ NUMBER EIGHT THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER EIGHTY\0"
  "OTTOMAN SIYAQ NUMBER EIGHTY THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER FIFTY\0"
  "OTTOMAN SIYAQ NUMBER FIFTY THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER FIVE\0"
  "OTTOMAN SIYAQ NUMBER FIVE HUNDRED\0"
  "OTTOMAN SIYAQ NUMBER FIVE THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER FORTY\0"
  "OTTOMAN SIYAQ NUMBER FORTY THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER FOUR\0"
  "OTTOMAN SIYAQ NUMBER FOUR HUNDRED\0"
  "OTTOMAN SIYAQ NUMBER FOUR THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER NINE\0"
  "OTTOMAN SIYAQ NUMBER NINE HUNDRED\0"
  "OTTOMAN SIYAQ NUMBER NINE THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER NINETY\0"
  "OTTOMAN SIYAQ NUMBER NINETY THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER ONE\0"
  "OTTOMAN SIYAQ NUMBER ONE HUNDRED\0"
  "OTTOMAN SIYAQ NUMBER ONE THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER SEVEN\0"
  "OTTOMAN SIYAQ NUMBER SEVEN HUNDRED\0"
  "OTTOMAN SIYAQ NUMBER SEVEN THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER SEVENTY\0"
  "OTTOMAN SIYAQ NUMBER SEVENTY THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER SIX\0"
  "OTTOMAN SIYAQ NUMBER SIX HUNDRED\0"
  "OTTOMAN SIYAQ NUMBER SIX THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER SIXTY\0"
  "OTTOMAN SIYAQ NUMBER SIXTY THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER TEN\0"
  "OTTOMAN SIYAQ NUMBER TEN THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER THIRTY\0"
  "OTTOMAN SIYAQ NUMBER THIRTY THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER THREE\0"
  "OTTOMAN SIYAQ NUMBER THREE HUNDRED\0"
  "OTTOMAN SIYAQ NUMBER THREE THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER TWENTY\0"
  "OTTOMAN SIYAQ NUMBER TWENTY THOUSAND\0"
  "OTTOMAN SIYAQ NUMBER TWO\0"
  "OTTOMAN SIYAQ NUMBER TWO HUNDRED\0"
  "OTTOMAN SIYAQ NUMBER TWO THOUSAND\0"
  "OUNCE SIGN\0"
  "OUTBOX TRAY\0"
  "OUTLINED BLACK STAR\0"
  "OUTLINED DIGIT EIGHT\0"
  "OUTLINED DIGIT FIVE\0"
  "OUTLINED DIGIT FOUR\0"
  "OUTLINED DIGIT NINE\0"
  "OUTLINED DIGIT ONE\0"
  "OUTLINED DIGIT SEVEN\0"
  "OUTLINED DIGIT SIX\0"
  "OUTLINED DIGIT THREE\0"
  "OUTLINED DIGIT TWO\0"
  "OUTLINED DIGIT ZERO\0"
  "OUTLINED GREEK CROSS\0"
  "OUTLINED LATIN CAPITAL LETTER A\0"
  "OUTLINED LATIN CAPITAL LETTER B\0"
  "OUTLINED LATIN CAPITAL LETTER C\0"
  "OUTLINED LATIN CAPITAL LETTER D\0"
  "OUTLINED LATIN CAPITAL LETTER E\0"
  "OUTLINED LATIN CAPITAL LETTER F\0"
  "OUTLINED LATIN CAPITAL LETTER G\0"
  "OUTLINED LATIN CAPITAL LETTER H\0"
  "OUTLINED LATIN CAPITAL LETTER I\0"
  "OUTLINED LATIN CAPITAL LETTER J\0"
  "OUTLINED LATIN CAPITAL LETTER K\0"
  "OUTLINED LATIN CAPITAL LETTER L\0"
  "OUTLINED LATIN CAPITAL LETTER M\0"
  "OUTLINED LATIN CAPITAL LETTER N\0"
  "OUTLINED LATIN CAPITAL LETTER O\0"
  "OUTLINED LATIN CAPITAL LETTER P\0"
  "OUTLINED LATIN CAPITAL LETTER Q\0"
  "OUTLINED LATIN CAPITAL LETTER R\0"
  "OUTLINED LATIN CAPITAL LETTER S\0"
  "OUTLINED LATIN CAPITAL LETTER T\0"
  "OUTLINED LATIN CAPITAL LETTER U\0"
  "OUTLINED LATIN CAPITAL LETTER V\0"
  "OUTLINED LATIN CAPITAL LETTER W\0"
  "OUTLINED LATIN CAPITAL LETTER X\0"
  "OUTLINED LATIN CAPITAL LETTER Y\0"
  "OUTLINED LATIN CAPITAL LETTER Z\0"
  "OUTLINED LATIN CROSS\0"
  "OUTLINED WHITE STAR\0"
  "OVERHEATED FACE\0"
  "OVERLAP\0"
  "OVERLAPPING BLACK SQUARES\0"
  "OVERLAPPING WHITE AND BLACK SQUARES\0"
  "OVERLAPPING WHITE SQUARES\0"
  "OVERLINE\0"
  "OWL\0"
  "OX\0"
  "OYSTER\0"
  "PACKAGE\0"
  "PAGE\0"
  "PAGE FACING UP\0"
  "PAGE WITH CIRCLED TEXT\0"
  "PAGE WITH CURL\0"
  "PAGER\0"
  "PAGES\0"
  "PAGODA\0"
  "PAHAWH HMONG CLAN SIGN FAJ\0"
  "PAHAWH HMONG CLAN SIGN HAM\0"
  "PAHAWH HMONG CLAN SIGN HAWJ\0"
  "PAHAWH HMONG CLAN SIGN KHAB\0"
  "PAHAWH HMONG CLAN SIGN KOO\0"
  "PAHAWH HMONG CLAN SIGN KWM\0"
  "PAHAWH HMONG CLAN SIGN LAUJ\0"
  "PAHAWH HMONG CLAN SIGN LIS\0"
  "PAHAWH HMONG CLAN SIGN MUAS\0"
  "PAHAWH HMONG CLAN SIGN PHAB\0"
  "PAHAWH HMONG CLAN SIGN THOJ\0"
  "PAHAWH HMONG CLAN SIGN TSAB\0"
  "PAHAWH HMONG CLAN SIGN TSHEEJ\0"
  "PAHAWH HMONG CLAN SIGN TSWB\0"
  "PAHAWH HMONG CLAN SIGN VAJ\0"
  "PAHAWH HMONG CLAN SIGN VWJ\0"
  "PAHAWH HMONG CLAN SIGN XYOOJ\0"
  "PAHAWH HMONG CLAN SIGN YAJ\0"
  "PAHAWH HMONG CLAN SIGN YEEG\0"
  "PAHAWH HMONG CONSONANT AU\0"
  "PAHAWH HMONG CONSONANT CAU\0"
  "PAHAWH HMONG CONSONANT CHAU\0"
  "PAHAWH HMONG CONSONANT HAU\0"
  "PAHAWH HMONG CONSONANT HLAU\0"
  "PAHAWH HMONG CONSONANT HNAU\0"
  "PAHAWH HMONG CONSONANT LAU\0"
  "PAHAWH HMONG CONSONANT MAU\0"
  "PAHAWH HMONG CONSONANT NAU\0"
  "PAHAWH HMONG CONSONANT NCHAU\0"
  "PAHAWH HMONG CONSONANT NKAU\0"
  "PAHAWH HMONG CONSONANT NLAU\0"
  "PAHAWH HMONG CONSONANT NTHAU\0"
  "PAHAWH HMONG CONSONANT NTSAU\0"
  "PAHAWH HMONG CONSONANT PLHAU\0"
  "PAHAWH HMONG CONSONANT QHAU\0"
  "PAHAWH HMONG CONSONANT RAU\0"
  "PAHAWH HMONG CONSONANT VAU\0"
  "PAHAWH HMONG CONSONANT XAU\0"
  "PAHAWH HMONG CONSONANT YAU\0"
  "PAHAWH HMONG DIGIT EIGHT\0"
  "PAHAWH HMONG DIGIT FIVE\0"
  "PAHAWH HMONG DIGIT FOUR\0"
  "PAHAWH HMONG DIGIT NINE\0"
  "PAHAWH HMONG DIGIT ONE\0"
  "PAHAWH HMONG DIGIT SEVEN\0"
  "PAHAWH HMONG DIGIT SIX\0"
  "PAHAWH HMONG DIGIT THREE\0"
  "PAHAWH HMONG DIGIT TWO\0"
  "PAHAWH HMONG DIGIT ZERO\0"
  "PAHAWH HMONG MARK CIM HOM\0"
  "PAHAWH HMONG MARK CIM KES\0"
  "PAHAWH HMONG MARK CIM KHAV\0"
  "PAHAWH HMONG MARK CIM SO\0"
  "PAHAWH HMONG MARK CIM SUAM\0"
  "PAHAWH HMONG MARK CIM TAUM\0"
  "PAHAWH HMONG MARK CIM TUB\0"
  "PAHAWH HMONG NUMBER HUNDRED MILLIONS\0"
  "PAHAWH HMONG NUMBER HUNDREDS\0"
  "PAHAWH HMONG NUMBER MILLIONS\0"
  "PAHAWH HMONG NUMBER TEN BILLIONS\0"
  "PAHAWH HMONG NUMBER TEN THOUSANDS\0"
  "PAHAWH HMONG NUMBER TENS\0"
  "PAHAWH HMONG NUMBER TRILLIONS\0"
  "PAHAWH HMONG SIGN AV\0"
  "PAHAWH HMONG SIGN CIM CHEEM\0"
  "PAHAWH HMONG SIGN CIM CUAM TSHOOJ\0"
  "PAHAWH HMONG SIGN CIM HAIS LUS NTOG NTOG\0"
  "PAHAWH HMONG SIGN CIM NRES TOS\0"
  "PAHAWH HMONG SIGN CIM PUB DAWB\0"
  "PAHAWH HMONG SIGN CIM TSOV ROG\0"
  "PAHAWH HMONG SIGN CIM TXWV\0"
  "PAHAWH HMONG SIGN CIM TXWV CHWV\0"
  "PAHAWH HMONG SIGN HLI\0"
  "PAHAWH HMONG SIGN HNUB\0"
  "PAHAWH HMONG SIGN IB YAM\0"
  "PAHAWH HMONG SIGN LOS\0"
  "PAHAWH HMONG SIGN MEEJ SUAB\0"
  "PAHAWH HMONG SIGN MEEJ TSEEB\0"
  "PAHAWH HMONG SIGN MUS\0"
  "PAHAWH HMONG SIGN NQIG\0"
  "PAHAWH HMONG SIGN NTUJ\0"
  "PAHAWH HMONG SIGN TAU\0"
  "PAHAWH HMONG SIGN THIRD-STAGE HLI\0"
  "PAHAWH HMONG SIGN TXHEEJ CEEV\0"
  "PAHAWH HMONG SIGN VOS FEEM\0"
  "PAHAWH HMONG SIGN VOS LUB\0"
  "PAHAWH HMONG SIGN VOS NRUA\0"
  "PAHAWH HMONG SIGN VOS SEEV\0"
  "PAHAWH HMONG SIGN VOS THIAB\0"
  "PAHAWH HMONG SIGN VOS THOM\0"
  "PAHAWH HMONG SIGN VOS TSHAB CEEB\0"
  "PAHAWH HMONG SIGN XAUS\0"
  "PAHAWH HMONG SIGN XIAB\0"
  "PAHAWH HMONG SIGN XYEEM FAIB\0"
  "PAHAWH HMONG SIGN XYEEM NTXIV\0"
  "PAHAWH HMONG SIGN XYEEM RHO\0"
  "PAHAWH HMONG SIGN XYEEM TOV\0"
  "PAHAWH HMONG SIGN XYOO\0"
  "PAHAWH HMONG SIGN ZWJ THAJ\0"
  "PAHAWH HMONG VOWEL KAAB\0"
  "PAHAWH HMONG VOWEL KAAV\0"
  "PAHAWH HMONG VOWEL KAB\0"
  "PAHAWH HMONG VOWEL KAIB\0"
  "PAHAWH HMONG VOWEL KAIV\0"
  "PAHAWH HMONG VOWEL KAUB\0"
  "PAHAWH HMONG VOWEL KAUV\0"
  "PAHAWH HMONG VOWEL KAV\0"
  "PAHAWH HMONG VOWEL KAWB\0"
  "PAHAWH HMONG VOWEL KAWV\0"
  "PAHAWH HMONG VOWEL KEB\0"
  "PAHAWH HMONG VOWEL KEEB\0"
  "PAHAWH HMONG VOWEL KEEV\0"
  "PAHAWH HMONG VOWEL KEV\0"
  "PAHAWH HMONG VOWEL KIAB\0"
  "PAHAWH HMONG VOWEL KIAV\0"
  "PAHAWH HMONG VOWEL KIB\0"
  "PAHAWH HMONG VOWEL KIV\0"
  "PAHAWH HMONG VOWEL KOB\0"
  "PAHAWH HMONG VOWEL KOOB\0"
  "PAHAWH HMONG VOWEL KOOV\0"
  "PAHAWH HMONG VOWEL KOV\0"
  "PAHAWH HMONG VOWEL KUAB\0"
  "PAHAWH HMONG VOWEL KUAV\0"
  "PAHAWH HMONG VOWEL KUB\0"
  "PAHAWH HMONG VOWEL KUV\0"
  "PAHAWH HMONG VOWEL KWB\0"
  "PAHAWH HMONG VOWEL KWV\0"
  "PALLAS\0"
  "PALM BRANCH\0"
  "PALM DOWN HAND\0"
  "PALM TREE\0"
  "PALM UP HAND\0"
  "PALMS UP TOGETHER\0"
  "PALMYRENE LEFT-POINTING FLEURON\0"
  "PALMYRENE LETTER ALEPH\0"
  "PALMYRENE LETTER AYIN\0"
  "PALMYRENE LETTER BETH\0"
  "PALMYRENE LETTER DALETH\0"
  "PALMYRENE LETTER FINAL NUN\0"
  "PALMYRENE LETTER GIMEL\0"
  "PALMYRENE LETTER HE\0"
  "PALMYRENE LETTER HETH\0"
  "PALMYRENE LETTER KAPH\0"
  "PALMYRENE LETTER LAMEDH\0"
  "PALMYRENE LETTER MEM\0"
  "PALMYRENE LETTER NUN\0"
  "PALMYRENE LETTER PE\0"
  "PALMYRENE LETTER QOPH\0"
  "PALMYRENE LETTER RESH\0"
  "PALMYRENE LETTER SADHE\0"
  "PALMYRENE LETTER SAMEKH\0"
  "PALMYRENE LETTER SHIN\0"
  "PALMYRENE LETTER TAW\0"
  "PALMYRENE LETTER TETH\0"
  "PALMYRENE LETTER WAW\0"
  "PALMYRENE LETTER YODH\0"
  "PALMYRENE LETTER ZAYIN\0"
  "PALMYRENE NUMBER FIVE\0"
  "PALMYRENE NUMBER FOUR\0"
  "PALMYRENE NUMBER ONE\0"
  "PALMYRENE NUMBER TEN\0"
  "PALMYRENE NUMBER THREE\0"
  "PALMYRENE NUMBER TWENTY\0"
  "PALMYRENE NUMBER TWO\0"
  "PALMYRENE RIGHT-POINTING FLEURON\0"
  "PANCAKES\0"
  "PANDA FACE\0"
  "PAPERCLIP\0"
  "PARACHUTE\0"
  "PARAGRAPH SEPARATOR\0"
  "PARAGRAPHOS\0"
  "PARAGRAPHUS MARK\0"
  "PARALLEL TO\0"
  "PARALLEL WITH HORIZONTAL STROKE\0"
  "PARALLEL WITH TILDE OPERATOR\0"
  "PARENTHESIZED DIGIT EIGHT\0"
  "PARENTHESIZED DIGIT FIVE\0"
  "PARENTHESIZED DIGIT FOUR\0"
  "PARENTHESIZED DIGIT NINE\0"
  "PARENTHESIZED DIGIT ONE\0"
  "PARENTHESIZED DIGIT SEVEN\0"
  "PARENTHESIZED DIGIT SIX\0"
  "PARENTHESIZED DIGIT THREE\0"
  "PARENTHESIZED DIGIT TWO\0"
  "PARENTHESIZED HANGUL CHIEUCH\0"
  "PARENTHESIZED HANGUL CHIEUCH A\0"
  "PARENTHESIZED HANGUL CIEUC\0"
  "PARENTHESIZED HANGUL CIEUC A\0"
  "PARENTHESIZED HANGUL CIEUC U\0"
  "PARENTHESIZED HANGUL HIEUH\0"
  "PARENTHESIZED HANGUL HIEUH A\0"
  "PARENTHESIZED HANGUL IEUNG\0"
  "PARENTHESIZED HANGUL IEUNG A\0"
  "PARENTHESIZED HANGUL KHIEUKH\0"
  "PARENTHESIZED HANGUL KHIEUKH A\0"
  "PARENTHESIZED HANGUL KIYEOK\0"
  "PARENTHESIZED HANGUL KIYEOK A\0"
  "PARENTHESIZED HANGUL MIEUM\0"
  "PARENTHESIZED HANGUL MIEUM A\0"
  "PARENTHESIZED HANGUL NIEUN\0"
  "PARENTHESIZED HANGUL NIEUN A\0"
  "PARENTHESIZED HANGUL PHIEUPH\0"
  "PARENTHESIZED HANGUL PHIEUPH A\0"
  "PARENTHESIZED HANGUL PIEUP\0"
  "PARENTHESIZED HANGUL PIEUP A\0"
  "PARENTHESIZED HANGUL RIEUL\0"
  "PARENTHESIZED HANGUL RIEUL A\0"
  "PARENTHESIZED HANGUL SIOS\0"
  "PARENTHESIZED HANGUL SIOS A\0"
  "PARENTHESIZED HANGUL THIEUTH\0"
  "PARENTHESIZED HANGUL THIEUTH A\0"
  "PARENTHESIZED HANGUL TIKEUT\0"
  "PARENTHESIZED HANGUL TIKEUT A\0"
  "PARENTHESIZED IDEOGRAPH ALLIANCE\0"
  "PARENTHESIZED IDEOGRAPH CALL\0"
  "PARENTHESIZED IDEOGRAPH CONGRATULATION\0"
  "PARENTHESIZED IDEOGRAPH EARTH\0"
  "PARENTHESIZED IDEOGRAPH EIGHT\0"
  "PARENTHESIZED IDEOGRAPH ENTERPRISE\0"
  "PARENTHESIZED IDEOGRAPH FESTIVAL\0"
  "PARENTHESIZED IDEOGRAPH FINANCIAL\0"
  "PARENTHESIZED IDEOGRAPH FIRE\0"
  "PARENTHESIZED IDEOGRAPH FIVE\0"
  "PARENTHESIZED IDEOGRAPH FOUR\0"
  "PARENTHESIZED IDEOGRAPH HAVE\0"
  "PARENTHESIZED IDEOGRAPH LABOR\0"
  "PARENTHESIZED IDEOGRAPH METAL\0"
  "PARENTHESIZED IDEOGRAPH MOON\0"
  "PARENTHESIZED IDEOGRAPH NAME\0"
  "PARENTHESIZED IDEOGRAPH NINE\0"
  "PARENTHESIZED IDEOGRAPH ONE\0"
  "PARENTHESIZED IDEOGRAPH REACH\0"
  "PARENTHESIZED IDEOGRAPH REPRESENT\0"
  "PARENTHESIZED IDEOGRAPH RESOURCE\0"
  "PARENTHESIZED IDEOGRAPH REST\0"
  "PARENTHESIZED IDEOGRAPH SELF\0"
  "PARENTHESIZED IDEOGRAPH SEVEN\0"
  "PARENTHESIZED IDEOGRAPH SIX\0"
  "PARENTHESIZED IDEOGRAPH SOCIETY\0"
  "PARENTHESIZED IDEOGRAPH SPECIAL\0"
  "PARENTHESIZED IDEOGRAPH STOCK\0"
  "PARENTHESIZED IDEOGRAPH STUDY\0"
  "PARENTHESIZED IDEOGRAPH SUN\0"
  "PARENTHESIZED IDEOGRAPH SUPERVISE\0"
  "PARENTHESIZED IDEOGRAPH TEN\0"
  "PARENTHESIZED IDEOGRAPH THREE\0"
  "PARENTHESIZED IDEOGRAPH TWO\0"
  "PARENTHESIZED IDEOGRAPH WATER\0"
  "PARENTHESIZED IDEOGRAPH WOOD\0"
  "PARENTHESIZED KOREAN CHARACTER O HU\0"
  "PARENTHESIZED KOREAN CHARACTER OJEON\0"
  "PARENTHESIZED LATIN CAPITAL LETTER A\0"
  "PARENTHESIZED LATIN CAPITAL LETTER B\0"
  "PARENTHESIZED LATIN CAPITAL LETTER C\0"
  "PARENTHESIZED LATIN CAPITAL LETTER D\0"
  "PARENTHESIZED LATIN CAPITAL LETTER E\0"
  "PARENTHESIZED LATIN CAPITAL LETTER F\0"
  "PARENTHESIZED LATIN CAPITAL LETTER G\0"
  "PARENTHESIZED LATIN CAPITAL LETTER H\0"
  "PARENTHESIZED LATIN CAPITAL LETTER I\0"
  "PARENTHESIZED LATIN CAPITAL LETTER J\0"
  "PARENTHESIZED LATIN CAPITAL LETTER K\0"
  "PARENTHESIZED LATIN CAPITAL LETTER L\0"
  "PARENTHESIZED LATIN CAPITAL LETTER M\0"
  "PARENTHESIZED LATIN CAPITAL LETTER N\0"
  "PARENTHESIZED LATIN CAPITAL LETTER O\0"
  "PARENTHESIZED LATIN CAPITAL LETTER P\0"
  "PARENTHESIZED LATIN CAPITAL LETTER Q\0"
  "PARENTHESIZED LATIN CAPITAL LETTER R\0"
  "PARENTHESIZED LATIN CAPITAL LETTER S\0"
  "PARENTHESIZED LATIN CAPITAL LETTER T\0"
  "PARENTHESIZED LATIN CAPITAL LETTER U\0"
  "PARENTHESIZED LATIN CAPITAL LETTER V\0"
  "PARENTHESIZED LATIN CAPITAL LETTER W\0"
  "PARENTHESIZED LATIN CAPITAL LETTER X\0"
  "PARENTHESIZED LATIN CAPITAL LETTER Y\0"
  "PARENTHESIZED LATIN CAPITAL LETTER Z\0"
  "PARENTHESIZED LATIN SMALL LETTER A\0"
  "PARENTHESIZED LATIN SMALL LETTER B\0"
  "PARENTHESIZED LATIN SMALL LETTER C\0"
  "PARENTHESIZED LATIN SMALL LETTER D\0"
  "PARENTHESIZED LATIN SMALL LETTER E\0"
  "PARENTHESIZED LATIN SMALL LETTER F\0"
  "PARENTHESIZED LATIN SMALL LETTER G\0"
  "PARENTHESIZED LATIN SMALL LETTER H\0"
  "PARENTHESIZED LATIN SMALL LETTER I\0"
  "PARENTHESIZED LATIN SMALL LETTER J\0"
  "PARENTHESIZED LATIN SMALL LETTER K\0"
  "PARENTHESIZED LATIN SMALL LETTER L\0"
  "PARENTHESIZED LATIN SMALL LETTER M\0"
  "PARENTHESIZED LATIN SMALL LETTER N\0"
  "PARENTHESIZED LATIN SMALL LETTER O\0"
  "PARENTHESIZED LATIN SMALL LETTER P\0"
  "PARENTHESIZED LATIN SMALL LETTER Q\0"
  "PARENTHESIZED LATIN SMALL LETTER R\0"
  "PARENTHESIZED LATIN SMALL LETTER S\0"
  "PARENTHESIZED LATIN SMALL LETTER T\0"
  "PARENTHESIZED LATIN SMALL LETTER U\0"
  "PARENTHESIZED LATIN SMALL LETTER V\0"
  "PARENTHESIZED LATIN SMALL LETTER W\0"
  "PARENTHESIZED LATIN SMALL LETTER X\0"
  "PARENTHESIZED LATIN SMALL LETTER Y\0"
  "PARENTHESIZED LATIN SMALL LETTER Z\0"
  "PARENTHESIZED NUMBER EIGHTEEN\0"
  "PARENTHESIZED NUMBER ELEVEN\0"
  "PARENTHESIZED NUMBER FIFTEEN\0"
  "PARENTHESIZED NUMBER FOURTEEN\0"
  "PARENTHESIZED NUMBER NINETEEN\0"
  "PARENTHESIZED NUMBER SEVENTEEN\0"
  "PARENTHESIZED NUMBER SIXTEEN\0"
  "PARENTHESIZED NUMBER TEN\0"
  "PARENTHESIZED NUMBER THIRTEEN\0"
  "PARENTHESIZED NUMBER TWELVE\0"
  "PARENTHESIZED NUMBER TWENTY\0"
  "PARROT\0"
  "PART ALTERNATION MARK\0"
  "PARTIAL DIFFERENTIAL\0"
  "PARTIALLY-RECYCLED PAPER SYMBOL\0"
  "PARTNERSHIP SIGN\0"
  "PARTY POPPER\0"
  "PASSED SYMBOL\0"
  "PASSENGER SHIP\0"
  "PASSIVE-PULL-DOWN-OUTPUT SYMBOL\0"
  "PASSIVE-PULL-UP-OUTPUT SYMBOL\0"
  "PASSPORT CONTROL\0"
  "PAU CIN HAU GLOTTAL STOP\0"
  "PAU CIN HAU GLOTTAL STOP FINAL\0"
  "PAU CIN HAU GLOTTAL STOP VARIANT\0"
  "PAU CIN HAU LETTER A\0"
  "PAU CIN HAU LETTER BA\0"
  "PAU CIN HAU LETTER CA\0"
  "PAU CIN HAU LETTER CHA\0"
  "PAU CIN HAU LETTER DA\0"
  "PAU CIN HAU LETTER E\0"
  "PAU CIN HAU LETTER FA\0"
  "PAU CIN HAU LETTER FINAL K\0"
  "PAU CIN HAU LETTER FINAL L\0"
  "PAU CIN HAU LETTER FINAL M\0"
  "PAU CIN HAU LETTER FINAL N\0"
  "PAU CIN HAU LETTER FINAL NG\0"
  "PAU CIN HAU LETTER FINAL P\0"
  "PAU CIN HAU LETTER FINAL T\0"
  "PAU CIN HAU LETTER FINAL W\0"
  "PAU CIN HAU LETTER FINAL Y\0"
  "PAU CIN HAU LETTER GA\0"
  "PAU CIN HAU LETTER HA\0"
  "PAU CIN HAU LETTER I\0"
  "PAU CIN HAU LETTER IA\0"
  "PAU CIN HAU LETTER KA\0"
  "PAU CIN HAU LETTER KHA\0"
  "PAU CIN HAU LETTER LA\0"
  "PAU CIN HAU LETTER MA\0"
  "PAU CIN HAU LETTER NA\0"
  "PAU CIN HAU LETTER NGA\0"
  "PAU CIN HAU LETTER O\0"
  "PAU CIN HAU LETTER PA\0"
  "PAU CIN HAU LETTER PHA\0"
  "PAU CIN HAU LETTER RA\0"
  "PAU CIN HAU LETTER SA\0"
  "PAU CIN HAU LETTER TA\0"
  "PAU CIN HAU LETTER THA\0"
  "PAU CIN HAU LETTER U\0"
  "PAU CIN HAU LETTER UA\0"
  "PAU CIN HAU LETTER VA\0"
  "PAU CIN HAU LETTER ZA\0"
  "PAU CIN HAU LOW-FALLING TONE\0"
  "PAU CIN HAU LOW-FALLING TONE FINAL\0"
  "PAU CIN HAU LOW-FALLING TONE LONG\0"
  "PAU CIN HAU LOW-FALLING TONE LONG FINAL\0"
  "PAU CIN HAU MID-LEVEL TONE\0"
  "PAU CIN HAU MID-LEVEL TONE FINAL\0"
  "PAU CIN HAU MID-LEVEL TONE LONG FINAL\0"
  "PAU CIN HAU RISING TONE\0"
  "PAU CIN HAU RISING TONE FINAL\0"
  "PAU CIN HAU RISING TONE LONG\0"
  "PAU CIN HAU RISING TONE LONG FINAL\0"
  "PAU CIN HAU SANDHI GLOTTAL STOP\0"
  "PAU CIN HAU SANDHI GLOTTAL STOP FINAL\0"
  "PAU CIN HAU SANDHI TONE\0"
  "PAU CIN HAU SANDHI TONE FINAL\0"
  "PAU CIN HAU SANDHI TONE LONG\0"
  "PAU CIN HAU SANDHI TONE LONG FINAL\0"
  "PAW PRINTS\0"
  "PEA POD\0"
  "PEACE SYMBOL\0"
  "PEACH\0"
  "PEACOCK\0"
  "PEANUTS\0"
  "PEAR\0"
  "PEDESTRIAN\0"
  "PEN OVER STAMPED ENVELOPE\0"
  "PENCIL\0"
  "PENGUIN\0"
  "PENSIVE FACE\0"
  "PENTAGRAM\0"
  "PEOPLE HUGGING\0"
  "PER MILLE SIGN\0"
  "PER SIGN\0"
  "PER TEN THOUSAND SIGN\0"
  "PERCENT SIGN\0"
  "PERFORMING ARTS\0"
  "PERMANENT PAPER SIGN\0"
  "PERPENDICULAR\0"
  "PERPENDICULAR WITH S\0"
  "PERSEVERING FACE\0"
  "PERSON BOWING DEEPLY\0"
  "PERSON CLIMBING\0"
  "PERSON DOING CARTWHEEL\0"
  "PERSON FROWNING\0"
  "PERSON IN LOTUS POSITION\0"
  "PERSON IN STEAMY ROOM\0"
  "PERSON RAISING BOTH HANDS IN CELEBRATION\0"
  "PERSON WITH BALL\0"
  "PERSON WITH BLOND HAIR\0"
  "PERSON WITH CROWN\0"
  "PERSON WITH FOLDED HANDS\0"
  "PERSON WITH HEADSCARF\0"
  "PERSON WITH POUTING FACE\0"
  "PERSONAL COMPUTER\0"
  "PERSPECTIVE\0"
  "PESETA SIGN\0"
  "PESO SIGN\0"
  "PETRI DISH\0"
  "PHAGS-PA DOUBLE HEAD MARK\0"
  "PHAGS-PA LETTER A\0"
  "PHAGS-PA LETTER ALTERNATE YA\0"
  "PHAGS-PA LETTER ASPIRATED FA\0"
  "PHAGS-PA LETTER BA\0"
  "PHAGS-PA LETTER CA\0"
  "PHAGS-PA LETTER CANDRABINDU\0"
  "PHAGS-PA LETTER CHA\0"
  "PHAGS-PA LETTER DA\0"
  "PHAGS-PA LETTER DDA\0"
  "PHAGS-PA LETTER DZA\0"
  "PHAGS-PA LETTER E\0"
  "PHAGS-PA LETTER EE\0"
  "PHAGS-PA LETTER FA\0"
  "PHAGS-PA LETTER GA\0"
  "PHAGS-PA LETTER GGA\0"
  "PHAGS-PA LETTER HA\0"
  "PHAGS-PA LETTER I\0"
  "PHAGS-PA LETTER JA\0"
  "PHAGS-PA LETTER KA\0"
  "PHAGS-PA LETTER KHA\0"
  "PHAGS-PA LETTER LA\0"
  "PHAGS-PA LETTER MA\0"
  "PHAGS-PA LETTER NA\0"
  "PHAGS-PA LETTER NGA\0"
  "PHAGS-PA LETTER NNA\0"
  "PHAGS-PA LETTER NYA\0"
  "PHAGS-PA LETTER O\0"
  "PHAGS-PA LETTER PA\0"
  "PHAGS-PA LETTER PHA\0"
  "PHAGS-PA LETTER QA\0"
  "PHAGS-PA LETTER RA\0"
  "PHAGS-PA LETTER SA\0"
  "PHAGS-PA LETTER SHA\0"
  "PHAGS-PA LETTER SMALL A\0"
  "PHAGS-PA LETTER TA\0"
  "PHAGS-PA LETTER THA\0"
  "PHAGS-PA LETTER TSA\0"
  "PHAGS-PA LETTER TSHA\0"
  "PHAGS-PA LETTER TTA\0"
  "PHAGS-PA LETTER TTHA\0"
  "PHAGS-PA LETTER U\0"
  "PHAGS-PA LETTER VOICED HA\0"
  "PHAGS-PA LETTER VOICELESS SHA\0"
  "PHAGS-PA LETTER WA\0"
  "PHAGS-PA LETTER XA\0"
  "PHAGS-PA LETTER YA\0"
  "PHAGS-PA LETTER ZA\0"
  "PHAGS-PA LETTER ZHA\0"
  "PHAGS-PA MARK DOUBLE SHAD\0"
  "PHAGS-PA MARK SHAD\0"
  "PHAGS-PA SINGLE HEAD MARK\0"
  "PHAGS-PA SUBJOINED LETTER RA\0"
  "PHAGS-PA SUBJOINED LETTER WA\0"
  "PHAGS-PA SUBJOINED LETTER YA\0"
  "PHAGS-PA SUPERFIXED LETTER RA\0"
  "PHAISTOS DISC SIGN ARROW\0"
  "PHAISTOS DISC SIGN BEE\0"
  "PHAISTOS DISC SIGN BEEHIVE\0"
  "PHAISTOS DISC SIGN BOOMERANG\0"
  "PHAISTOS DISC SIGN BOW\0"
  "PHAISTOS DISC SIGN BULLS LEG\0"
  "PHAISTOS DISC SIGN CAPTIVE\0"
  "PHAISTOS DISC SIGN CARPENTRY PLANE\0"
  "PHAISTOS DISC SIGN CAT\0"
  "PHAISTOS DISC SIGN CHILD\0"
  "PHAISTOS DISC SIGN CLUB\0"
  "PHAISTOS DISC SIGN COLUMN\0"
  "PHAISTOS DISC SIGN COMB\0"
  "PHAISTOS DISC SIGN COMBINING OBLIQUE STROKE\0"
  "PHAISTOS DISC SIGN DOLIUM\0"
  "PHAISTOS DISC SIGN DOVE\0"
  "PHAISTOS DISC SIGN EAGLE\0"
  "PHAISTOS DISC SIGN FLUTE\0"
  "PHAISTOS DISC SIGN GAUNTLET\0"
  "PHAISTOS DISC SIGN GRATER\0"
  "PHAISTOS DISC SIGN HELMET\0"
  "PHAISTOS DISC SIGN HIDE\0"
  "PHAISTOS DISC SIGN HORN\0"
  "PHAISTOS DISC SIGN LID\0"
  "PHAISTOS DISC SIGN LILY\0"
  "PHAISTOS DISC SIGN MANACLES\0"
  "PHAISTOS DISC SIGN MATTOCK\0"
  "PHAISTOS DISC SIGN OX BACK\0"
  "PHAISTOS DISC SIGN PAPYRUS\0"
  "PHAISTOS DISC SIGN PEDESTRIAN\0"
  "PHAISTOS DISC SIGN PLANE TREE\0"
  "PHAISTOS DISC SIGN PLUMED HEAD\0"
  "PHAISTOS DISC SIGN RAM\0"
  "PHAISTOS DISC SIGN ROSETTE\0"
  "PHAISTOS DISC SIGN SAW\0"
  "PHAISTOS DISC SIGN SHIELD\0"
  "PHAISTOS DISC SIGN SHIP\0"
  "PHAISTOS DISC SIGN SLING\0"
  "PHAISTOS DISC SIGN SMALL AXE\0"
  "PHAISTOS DISC SIGN STRAINER\0"
  "PHAISTOS DISC SIGN TATTOOED HEAD\0"
  "PHAISTOS DISC SIGN TIARA\0"
  "PHAISTOS DISC SIGN TUNNY\0"
  "PHAISTOS DISC SIGN VINE\0"
  "PHAISTOS DISC SIGN WAVY BAND\0"
  "PHAISTOS DISC SIGN WOMAN\0"
  "PHILIPPINE DOUBLE PUNCTUATION\0"
  "PHILIPPINE SINGLE PUNCTUATION\0"
  "PHOENICIAN LETTER AIN\0"
  "PHOENICIAN LETTER ALF\0"
  "PHOENICIAN LETTER BET\0"
  "PHOENICIAN LETTER DELT\0"
  "PHOENICIAN LETTER GAML\0"
  "PHOENICIAN LETTER HE\0"
  "PHOENICIAN LETTER HET\0"
  "PHOENICIAN LETTER KAF\0"
  "PHOENICIAN LETTER LAMD\0"
  "PHOENICIAN LETTER MEM\0"
  "PHOENICIAN LETTER NUN\0"
  "PHOENICIAN LETTER PE\0"
  "PHOENICIAN LETTER QOF\0"
  "PHOENICIAN LETTER ROSH\0"
  "PHOENICIAN LETTER SADE\0"
  "PHOENICIAN LETTER SEMK\0"
  "PHOENICIAN LETTER SHIN\0"
  "PHOENICIAN LETTER TAU\0"
  "PHOENICIAN LETTER TET\0"
  "PHOENICIAN LETTER WAU\0"
  "PHOENICIAN LETTER YOD\0"
  "PHOENICIAN LETTER ZAI\0"
  "PHOENICIAN NUMBER ONE\0"
  "PHOENICIAN NUMBER ONE HUNDRED\0"
  "PHOENICIAN NUMBER TEN\0"
  "PHOENICIAN NUMBER THREE\0"
  "PHOENICIAN NUMBER TWENTY\0"
  "PHOENICIAN NUMBER TWO\0"
  "PHOENICIAN WORD SEPARATOR\0"
  "PHOLUS\0"
  "PICK\0"
  "PICKUP TRUCK\0"
  "PIE\0"
  "PIG\0"
  "PIG FACE\0"
  "PIG NOSE\0"
  "PILCROW SIGN\0"
  "PILE OF POO\0"
  "PILL\0"
  "PINATA\0"
  "PINCHED FINGERS\0"
  "PINCHING HAND\0"
  "PINE DECORATION\0"
  "PINEAPPLE\0"
  "PINK HEART\0"
  "PINWHEEL STAR\0"
  "PISCES\0"
  "PISTOL\0"
  "PITCHFORK\0"
  "PITCHFORK WITH TEE TOP\0"
  "PLACARD\0"
  "PLACE OF INTEREST SIGN\0"
  "PLACE OF WORSHIP\0"
  "PLANCK CONSTANT\0"
  "PLANCK CONSTANT OVER TWO PI\0"
  "PLAYGROUND SLIDE\0"
  "PLAYING CARD ACE OF CLUBS\0"
  "PLAYING CARD ACE OF DIAMONDS\0"
  "PLAYING CARD ACE OF HEARTS\0"
  "PLAYING CARD ACE OF SPADES\0"
  "PLAYING CARD BACK\0"
  "PLAYING CARD BLACK JOKER\0"
  "PLAYING CARD EIGHT OF CLUBS\0"
  "PLAYING CARD EIGHT OF DIAMONDS\0"
  "PLAYING CARD EIGHT OF HEARTS\0"
  "PLAYING CARD EIGHT OF SPADES\0"
  "PLAYING CARD FIVE OF CLUBS\0"
  "PLAYING CARD FIVE OF DIAMONDS\0"
  "PLAYING CARD FIVE OF HEARTS\0"
  "PLAYING CARD FIVE OF SPADES\0"
  "PLAYING CARD FOOL\0"
  "PLAYING CARD FOUR OF CLUBS\0"
  "PLAYING CARD FOUR OF DIAMONDS\0"
  "PLAYING CARD FOUR OF HEARTS\0"
  "PLAYING CARD FOUR OF SPADES\0"
  "PLAYING CARD JACK OF CLUBS\0"
  "PLAYING CARD JACK OF DIAMONDS\0"
  "PLAYING CARD JACK OF HEARTS\0"
  "PLAYING CARD JACK OF SPADES\0"
  "PLAYING CARD KING OF CLUBS\0"
  "PLAYING CARD KING OF DIAMONDS\0"
  "PLAYING CARD KING OF HEARTS\0"
  "PLAYING CARD KING OF SPADES\0"
  "PLAYING CARD KNIGHT OF CLUBS\0"
  "PLAYING CARD KNIGHT OF DIAMONDS\0"
  "PLAYING CARD KNIGHT OF HEARTS\0"
  "PLAYING CARD KNIGHT OF SPADES\0"
  "PLAYING CARD NINE OF CLUBS\0"
  "PLAYING CARD NINE OF DIAMONDS\0"
  "PLAYING CARD NINE OF HEARTS\0"
  "PLAYING CARD NINE OF SPADES\0"
  "PLAYING CARD QUEEN OF CLUBS\0"
  "PLAYING CARD QUEEN OF DIAMONDS\0"
  "PLAYING CARD QUEEN OF HEARTS\0"
  "PLAYING CARD QUEEN OF SPADES\0"
  "PLAYING CARD RED JOKER\0"
  "PLAYING CARD SEVEN OF CLUBS\0"
  "PLAYING CARD SEVEN OF DIAMONDS\0"
  "PLAYING CARD SEVEN OF HEARTS\0"
  "PLAYING CARD SEVEN OF SPADES\0"
  "PLAYING CARD SIX OF CLUBS\0"
  "PLAYING CARD SIX OF DIAMONDS\0"
  "PLAYING CARD SIX OF HEARTS\0"
  "PLAYING CARD SIX OF SPADES\0"
  "PLAYING CARD TEN OF CLUBS\0"
  "PLAYING CARD TEN OF DIAMONDS\0"
  "PLAYING CARD TEN OF HEARTS\0"
  "PLAYING CARD TEN OF SPADES\0"
  "PLAYING CARD THREE OF CLUBS\0"
  "PLAYING CARD THREE OF DIAMONDS\0"
  "PLAYING CARD THREE OF HEARTS\0"
  "PLAYING CARD THREE OF SPADES\0"
  "PLAYING CARD TRUMP-1\0"
  "PLAYING CARD TRUMP-10\0"
  "PLAYING CARD TRUMP-11\0"
  "PLAYING CARD TRUMP-12\0"
  "PLAYING CARD TRUMP-13\0"
  "PLAYING CARD TRUMP-14\0"
  "PLAYING CARD TRUMP-15\0"
  "PLAYING CARD TRUMP-16\0"
  "PLAYING CARD TRUMP-17\0"
  "PLAYING CARD TRUMP-18\0"
  "PLAYING CARD TRUMP-19\0"
  "PLAYING CARD TRUMP-2\0"
  "PLAYING CARD TRUMP-20\0"
  "PLAYING CARD TRUMP-21\0"
  "PLAYING CARD TRUMP-3\0"
  "PLAYING CARD TRUMP-4\0"
  "PLAYING CARD TRUMP-5\0"
  "PLAYING CARD TRUMP-6\0"
  "PLAYING CARD TRUMP-7\0"
  "PLAYING CARD TRUMP-8\0"
  "PLAYING CARD TRUMP-9\0"
  "PLAYING CARD TWO OF CLUBS\0"
  "PLAYING CARD TWO OF DIAMONDS\0"
  "PLAYING CARD TWO OF HEARTS\0"
  "PLAYING CARD TWO OF SPADES\0"
  "PLAYING CARD WHITE JOKER\0"
  "PLUNGER\0"
  "PLUS SIGN\0"
  "PLUS SIGN ABOVE EQUALS SIGN\0"
  "PLUS SIGN IN LEFT HALF CIRCLE\0"
  "PLUS SIGN IN RIGHT HALF CIRCLE\0"
  "PLUS SIGN IN TRIANGLE\0"
  "PLUS SIGN WITH BLACK TRIANGLE\0"
  "PLUS SIGN WITH CIRCUMFLEX ACCENT ABOVE\0"
  "PLUS SIGN WITH DOT BELOW\0"
  "PLUS SIGN WITH SMALL CIRCLE ABOVE\0"
  "PLUS SIGN WITH SUBSCRIPT TWO\0"
  "PLUS SIGN WITH TILDE ABOVE\0"
  "PLUS SIGN WITH TILDE BELOW\0"
  "PLUS-MINUS SIGN\0"
  "PLUTO\0"
  "PLUTO FORM FIVE\0"
  "PLUTO FORM FOUR\0"
  "PLUTO FORM THREE\0"
  "PLUTO FORM TWO\0"
  "PNP TRANSISTOR\0"
  "POCKET CALCULATOR\0"
  "POLICE CAR\0"
  "POLICE CARS REVOLVING LIGHT\0"
  "POLICE OFFICER\0"
  "POODLE\0"
  "POP DIRECTIONAL FORMATTING\0"
  "POP DIRECTIONAL ISOLATE\0"
  "POPCORN\0"
  "PORTABLE STEREO\0"
  "POSEIDON\0"
  "POSITION INDICATOR\0"
  "POSTAL HORN\0"
  "POSTAL MARK\0"
  "POSTAL MARK FACE\0"
  "POSTBOX\0"
  "POT OF FOOD\0"
  "POTABLE WATER SYMBOL\0"
  "POTATO\0"
  "POTTED PLANT\0"
  "POUCH\0"
  "POULTRY LEG\0"
  "POUND SIGN\0"
  "POURING LIQUID\0"
  "POUTING CAT FACE\0"
  "POUTING FACE\0"
  "POWER ON SYMBOL\0"
  "POWER ON-OFF SYMBOL\0"
  "POWER SLEEP SYMBOL\0"
  "POWER SYMBOL\0"
  "PRAYER BEADS\0"
  "PRECEDES\0"
  "PRECEDES ABOVE ALMOST EQUAL TO\0"
  "PRECEDES ABOVE EQUALS SIGN\0"
  "PRECEDES ABOVE NOT ALMOST EQUAL TO\0"
  "PRECEDES ABOVE NOT EQUAL TO\0"
  "PRECEDES ABOVE SINGLE-LINE EQUALS SIGN\0"
  "PRECEDES ABOVE SINGLE-LINE NOT EQUAL TO\0"
  "PRECEDES BUT NOT EQUIVALENT TO\0"
  "PRECEDES OR EQUAL TO\0"
  "PRECEDES OR EQUIVALENT TO\0"
  "PRECEDES UNDER RELATION\0"
  "PREGNANT MAN\0"
  "PREGNANT PERSON\0"
  "PREGNANT WOMAN\0"
  "PRESCRIPTION TAKE\0"
  "PRESENTATION FORM FOR VERTICAL COLON\0"
  "PRESENTATION FORM FOR VERTICAL COMMA\0"
  "PRESENTATION FORM FOR VERTICAL EM DASH\0"
  "PRESENTATION FORM FOR VERTICAL EN DASH\0"
  "PRESENTATION FORM FOR VERTICAL EXCLAMATION MARK\0"
  "PRESENTATION FORM FOR VERTICAL HORIZONTAL ELLIPSIS\0"
  "PRESENTATION FORM FOR VERTICAL IDEOGRAPHIC COMMA\0"
  "PRESENTATION FORM FOR VERTICAL IDEOGRAPHIC FULL STOP\0"
  "PRESENTATION FORM FOR VERTICAL LEFT ANGLE BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL LEFT BLACK LENTICULAR BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL LEFT CORNER BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL LEFT CURLY BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL LEFT DOUBLE ANGLE BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL LEFT PARENTHESIS\0"
  "PRESENTATION FORM FOR VERTICAL LEFT SQUARE BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL LEFT TORTOISE SHELL BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL LEFT WHITE CORNER BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL LEFT WHITE LENTICULAR BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL LOW LINE\0"
  "PRESENTATION FORM FOR VERTICAL QUESTION MARK\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT ANGLE BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT BLACK LENTICULAR BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT CORNER BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT CURLY BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT DOUBLE ANGLE BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT PARENTHESIS\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT SQUARE BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT TORTOISE SHELL BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT WHITE CORNER BRACKET\0"
  "PRESENTATION FORM FOR VERTICAL RIGHT WHITE LENTICULAR BRAKCET\0"
  "PRESENTATION FORM FOR VERTICAL SEMICOLON\0"
  "PRESENTATION FORM FOR VERTICAL TWO DOT LEADER\0"
  "PRESENTATION FORM FOR VERTICAL WAVY LOW LINE\0"
  "PRETZEL\0"
  "PREVIOUS PAGE\0"
  "PRIME\0"
  "PRINCE\0"
  "PRINCESS\0"
  "PRINT SCREEN SYMBOL\0"
  "PRINTER\0"
  "PRINTER ICON\0"
  "PROBING CANE\0"
  "PROHIBITED SIGN\0"
  "PROJECTIVE\0"
  "PROPERTY LINE\0"
  "PROPORTION\0"
  "PROPORTIONAL TO\0"
  "PROSERPINA\0"
  "PSALTER PAHLAVI FOUR DOTS WITH CROSS\0"
  "PSALTER PAHLAVI FOUR DOTS WITH DOT\0"
  "PSALTER PAHLAVI LETTER ALEPH\0"
  "PSALTER PAHLAVI LETTER BETH\0"
  "PSALTER PAHLAVI LETTER DALETH\0"
  "PSALTER PAHLAVI LETTER GIMEL\0"
  "PSALTER PAHLAVI LETTER HE\0"
  "PSALTER PAHLAVI LETTER HETH\0"
  "PSALTER PAHLAVI LETTER KAPH\0"
  "PSALTER PAHLAVI LETTER LAMEDH\0"
  "PSALTER PAHLAVI LETTER MEM-QOPH\0"
  "PSALTER PAHLAVI LETTER NUN\0"
  "PSALTER PAHLAVI LETTER PE\0"
  "PSALTER PAHLAVI LETTER SADHE\0"
  "PSALTER PAHLAVI LETTER SAMEKH\0"
  "PSALTER PAHLAVI LETTER SHIN\0"
  "PSALTER PAHLAVI LETTER TAW\0"
  "PSALTER PAHLAVI LETTER WAW-AYIN-RESH\0"
  "PSALTER PAHLAVI LETTER YODH\0"
  "PSALTER PAHLAVI LETTER ZAYIN\0"
  "PSALTER PAHLAVI NUMBER FOUR\0"
  "PSALTER PAHLAVI NUMBER ONE\0"
  "PSALTER PAHLAVI NUMBER ONE HUNDRED\0"
  "PSALTER PAHLAVI NUMBER TEN\0"
  "PSALTER PAHLAVI NUMBER THREE\0"
  "PSALTER PAHLAVI NUMBER TWENTY\0"
  "PSALTER PAHLAVI NUMBER TWO\0"
  "PSALTER PAHLAVI SECTION MARK\0"
  "PSALTER PAHLAVI TURNED SECTION MARK\0"
  "PUBLIC ADDRESS LOUDSPEAKER\0"
  "PUNCTUATION SPACE\0"
  "PUNCTUS ELEVATUS MARK\0"
  "PURPLE HEART\0"
  "PURSE\0"
  "PUSHPIN\0"
  "PUT LITTER IN ITS PLACE SYMBOL\0"
  "QUADRANT LOWER LEFT\0"
  "QUADRANT LOWER RIGHT\0"
  "QUADRANT UPPER LEFT\0"
  "QUADRANT UPPER LEFT AND LOWER LEFT AND LOWER RIGHT\0"
  "QUADRANT UPPER LEFT AND LOWER RIGHT\0"
  "QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER LEFT\0"
  "QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER RIGHT\0"
  "QUADRANT UPPER RIGHT\0"
  "QUADRANT UPPER RIGHT AND LOWER LEFT\0"
  "QUADRANT UPPER RIGHT AND LOWER LEFT AND LOWER RIGHT\0"
  "QUADRUPLE INTEGRAL OPERATOR\0"
  "QUADRUPLE PRIME\0"
  "QUAOAR\0"
  "QUARTER NOTE\0"
  "QUATERNION INTEGRAL OPERATOR\0"
  "QUESTION EXCLAMATION MARK\0"
  "QUESTION MARK\0"
  "QUESTIONED EQUAL TO\0"
  "QUINCUNX\0"
  "QUOTATION MARK\0"
  "RABBIT\0"
  "RABBIT FACE\0"
  "RACCOON\0"
  "RACING CAR\0"
  "RACING MOTORCYCLE\0"
  "RADICAL SYMBOL BOTTOM\0"
  "RADIO\0"
  "RADIO BUTTON\0"
  "RADIOACTIVE SIGN\0"
  "RAILWAY CAR\0"
  "RAILWAY TRACK\0"
  "RAIN\0"
  "RAINBOW\0"
  "RAISED BACK OF HAND\0"
  "RAISED COMMA\0"
  "RAISED DOT\0"
  "RAISED DOTTED INTERPOLATION MARKER\0"
  "RAISED FIST\0"
  "RAISED HAND\0"
  "RAISED HAND WITH FINGERS SPLAYED\0"
  "RAISED HAND WITH PART BETWEEN MIDDLE AND RING FINGERS\0"
  "RAISED INTERPOLATION MARKER\0"
  "RAISED MC SIGN\0"
  "RAISED MD SIGN\0"
  "RAISED MR SIGN\0"
  "RAISED SMALL LEFT SQUARE BRACKET\0"
  "RAISED SQUARE\0"
  "RAM\0"
  "RAT\0"
  "RATIO\0"
  "RAZOR\0"
  "RECEIPT\0"
  "RECEPTACLE\0"
  "RECREATIONAL VEHICLE\0"
  "RECYCLED PAPER SYMBOL\0"
  "RECYCLING SYMBOL FOR GENERIC MATERIALS\0"
  "RECYCLING SYMBOL FOR TYPE-1 PLASTICS\0"
  "RECYCLING SYMBOL FOR TYPE-2 PLASTICS\0"
  "RECYCLING SYMBOL FOR TYPE-3 PLASTICS\0"
  "RECYCLING SYMBOL FOR TYPE-4 PLASTICS\0"
  "RECYCLING SYMBOL FOR TYPE-5 PLASTICS\0"
  "RECYCLING SYMBOL FOR TYPE-6 PLASTICS\0"
  "RECYCLING SYMBOL FOR TYPE-7 PLASTICS\0"
  "RED APPLE\0"
  "RED GIFT ENVELOPE\0"
  "REFERENCE MARK\0"
  "REGIONAL INDICATOR SYMBOL LETTER A\0"
  "REGIONAL INDICATOR SYMBOL LETTER B\0"
  "REGIONAL INDICATOR SYMBOL LETTER C\0"
  "REGIONAL INDICATOR SYMBOL LETTER D\0"
  "REGIONAL INDICATOR SYMBOL LETTER E\0"
  "REGIONAL INDICATOR SYMBOL LETTER F\0"
  "REGIONAL INDICATOR SYMBOL LETTER G\0"
  "REGIONAL INDICATOR SYMBOL LETTER H\0"
  "REGIONAL INDICATOR SYMBOL LETTER I\0"
  "REGIONAL INDICATOR SYMBOL LETTER J\0"
  "REGIONAL INDICATOR SYMBOL LETTER K\0"
  "REGIONAL INDICATOR SYMBOL LETTER L\0"
  "REGIONAL INDICATOR SYMBOL LETTER M\0"
  "REGIONAL INDICATOR SYMBOL LETTER N\0"
  "REGIONAL INDICATOR SYMBOL LETTER O\0"
  "REGIONAL INDICATOR SYMBOL LETTER P\0"
  "REGIONAL INDICATOR SYMBOL LETTER Q\0"
  "REGIONAL INDICATOR SYMBOL LETTER R\0"
  "REGIONAL INDICATOR SYMBOL LETTER S\0"
  "REGIONAL INDICATOR SYMBOL LETTER T\0"
  "REGIONAL INDICATOR SYMBOL LETTER U\0"
  "REGIONAL INDICATOR SYMBOL LETTER V\0"
  "REGIONAL INDICATOR SYMBOL LETTER W\0"
  "REGIONAL INDICATOR SYMBOL LETTER X\0"
  "REGIONAL INDICATOR SYMBOL LETTER Y\0"
  "REGIONAL INDICATOR SYMBOL LETTER Z\0"
  "REGISTERED SIGN\0"
  "REJANG CONSONANT SIGN H\0"
  "REJANG CONSONANT SIGN N\0"
  "REJANG CONSONANT SIGN NG\0"
  "REJANG CONSONANT SIGN R\0"
  "REJANG LETTER A\0"
  "REJANG LETTER BA\0"
  "REJANG LETTER CA\0"
  "REJANG LETTER DA\0"
  "REJANG LETTER GA\0"
  "REJANG LETTER HA\0"
  "REJANG LETTER JA\0"
  "REJANG LETTER KA\0"
  "REJANG LETTER LA\0"
  "REJANG LETTER MA\0"
  "REJANG LETTER MBA\0"
  "REJANG LETTER NA\0"
  "REJANG LETTER NDA\0"
  "REJANG LETTER NGA\0"
  "REJANG LETTER NGGA\0"
  "REJANG LETTER NYA\0"
  "REJANG LETTER NYJA\0"
  "REJANG LETTER PA\0"
  "REJANG LETTER RA\0"
  "REJANG LETTER SA\0"
  "REJANG LETTER TA\0"
  "REJANG LETTER WA\0"
  "REJANG LETTER YA\0"
  "REJANG SECTION MARK\0"
  "REJANG VIRAMA\0"
  "REJANG VOWEL SIGN AI\0"
  "REJANG VOWEL SIGN AU\0"
  "REJANG VOWEL SIGN E\0"
  "REJANG VOWEL SIGN EA\0"
  "REJANG VOWEL SIGN EU\0"
  "REJANG VOWEL SIGN I\0"
  "REJANG VOWEL SIGN O\0"
  "REJANG VOWEL SIGN U\0"
  "RELIEVED FACE\0"
  "REMINDER RIBBON\0"
  "REPLACEMENT CHARACTER\0"
  "RESPONSE\0"
  "RESTRICTED LEFT ENTRY-1\0"
  "RESTRICTED LEFT ENTRY-2\0"
  "RESTROOM\0"
  "RETURN LEFT\0"
  "RETURN RIGHT\0"
  "RETURN SYMBOL\0"
  "REVERSE CHECKER BOARD\0"
  "REVERSE LIGHT FOUR POINTED PINWHEEL STAR\0"
  "REVERSE SOLIDUS\0"
  "REVERSE SOLIDUS OPERATOR\0"
  "REVERSE SOLIDUS PRECEDING SUBSET\0"
  "REVERSE SOLIDUS WITH HORIZONTAL STROKE\0"
  "REVERSE TILDE OPERATOR ABOVE LEFTWARDS ARROW\0"
  "REVERSE TILDE OPERATOR ABOVE RIGHTWARDS ARROW\0"
  "REVERSED ANGLE\0"
  "REVERSED ANGLE WITH UNDERBAR\0"
  "REVERSED COMMA\0"
  "REVERSED DOUBLE PRIME\0"
  "REVERSED DOUBLE PRIME QUOTATION MARK\0"
  "REVERSED DOUBLE STROKE NOT SIGN\0"
  "REVERSED EMPTY SET\0"
  "REVERSED FORKED PARAGRAPHOS\0"
  "REVERSED HAND WITH MIDDLE FINGER EXTENDED\0"
  "REVERSED NOT SIGN\0"
  "REVERSED PILCROW SIGN\0"
  "REVERSED PRIME\0"
  "REVERSED QUESTION MARK\0"
  "REVERSED RAISED HAND WITH FINGERS SPLAYED\0"
  "REVERSED RIGHT ANGLE\0"
  "REVERSED ROTATED FLORAL HEART BULLET\0"
  "REVERSED SANS-SERIF CAPITAL L\0"
  "REVERSED SEMICOLON\0"
  "REVERSED THUMBS DOWN SIGN\0"
  "REVERSED THUMBS UP SIGN\0"
  "REVERSED TILDE\0"
  "REVERSED TILDE EQUALS\0"
  "REVERSED TRIPLE PRIME\0"
  "REVERSED VICTORY HAND\0"
  "REVOLVING HEARTS\0"
  "RHINOCEROS\0"
  "RIAL SIGN\0"
  "RIBBON\0"
  "RIBBON ARROW DOWN LEFT\0"
  "RIBBON ARROW DOWN RIGHT\0"
  "RIBBON ARROW LEFT DOWN\0"
  "RIBBON ARROW LEFT UP\0"
  "RIBBON ARROW RIGHT DOWN\0"
  "RIBBON ARROW RIGHT UP\0"
  "RIBBON ARROW UP LEFT\0"
  "RIBBON ARROW UP RIGHT\0"
  "RICE BALL\0"
  "RICE CRACKER\0"
  "RIFLE\0"
  "RIGHT AND LOWER ONE EIGHTH BLOCK\0"
  "RIGHT AND UPPER ONE EIGHTH BLOCK\0"
  "RIGHT ANGER BUBBLE\0"
  "RIGHT ANGLE\0"
  "RIGHT ANGLE BRACKET\0"
  "RIGHT ANGLE BRACKET WITH DOT\0"
  "RIGHT ANGLE DOTTED SUBSTITUTION MARKER\0"
  "RIGHT ANGLE SUBSTITUTION MARKER\0"
  "RIGHT ANGLE VARIANT WITH SQUARE\0"
  "RIGHT ANGLE WITH ARC\0"
  "RIGHT ANGLE WITH DOWNWARDS ZIGZAG ARROW\0"
  "RIGHT ARC GREATER-THAN BRACKET\0"
  "RIGHT ARROW WITH CIRCLED PLUS\0"
  "RIGHT ARROW WITH SMALL CIRCLE\0"
  "RIGHT BLACK LENTICULAR BRACKET\0"
  "RIGHT BLACK TORTOISE SHELL BRACKET\0"
  "RIGHT CEILING\0"
  "RIGHT CORNER BRACKET\0"
  "RIGHT CURLY BRACKET\0"
  "RIGHT CURLY BRACKET LOWER HOOK\0"
  "RIGHT CURLY BRACKET MIDDLE PIECE\0"
  "RIGHT CURLY BRACKET UPPER HOOK\0"
  "RIGHT DOTTED SUBSTITUTION BRACKET\0"
  "RIGHT DOUBLE ANGLE BRACKET\0"
  "RIGHT DOUBLE ARROW WITH ROUNDED HEAD\0"
  "RIGHT DOUBLE PARENTHESIS\0"
  "RIGHT DOUBLE QUOTATION MARK\0"
  "RIGHT DOUBLE WIGGLY FENCE\0"
  "RIGHT FISH TAIL\0"
  "RIGHT FIVE EIGHTHS BLOCK\0"
  "RIGHT FLOOR\0"
  "RIGHT HALF AND LEFT HALF WHITE CIRCLE\0"
  "RIGHT HALF BLACK CIRCLE\0"
  "RIGHT HALF BLACK STAR\0"
  "RIGHT HALF BLOCK\0"
  "RIGHT HALF FLYING SAUCER\0"
  "RIGHT HALF FOLDER\0"
  "RIGHT HALF HORIZONTAL LINE WITH THREE TICK MARKS\0"
  "RIGHT HALF LOWER ONE QUARTER BLOCK\0"
  "RIGHT HALF MEDIUM SHADE\0"
  "RIGHT HALF RUNNING MAN\0"
  "RIGHT HALF TRIPLE DASH HORIZONTAL\0"
  "RIGHT HALF UPPER ONE QUARTER BLOCK\0"
  "RIGHT HALF WHITE ELLIPSE\0"
  "RIGHT HAND TELEPHONE RECEIVER\0"
  "RIGHT JUSTIFIED LEFT HALF BLACK CIRCLE\0"
  "RIGHT JUSTIFIED LEFT HALF WHITE CIRCLE\0"
  "RIGHT LOW PARAPHRASE BRACKET\0"
  "RIGHT NORMAL FACTOR SEMIDIRECT PRODUCT\0"
  "RIGHT ONE EIGHTH BLOCK\0"
  "RIGHT ONE QUARTER BLOCK\0"
  "RIGHT OPEN SQUARED DOT\0"
  "RIGHT OUTER JOIN\0"
  "RIGHT PARENTHESIS\0"
  "RIGHT PARENTHESIS EXTENSION\0"
  "RIGHT PARENTHESIS LOWER HOOK\0"
  "RIGHT PARENTHESIS UPPER HOOK\0"
  "RIGHT RAISED OMISSION BRACKET\0"
  "RIGHT S-SHAPED BAG DELIMITER\0"
  "RIGHT SEMIDIRECT PRODUCT\0"
  "RIGHT SEVEN EIGHTHS BLOCK\0"
  "RIGHT SIDEWAYS U BRACKET\0"
  "RIGHT SINGLE QUOTATION MARK\0"
  "RIGHT SPEAKER\0"
  "RIGHT SPEAKER WITH ONE SOUND WAVE\0"
  "RIGHT SPEAKER WITH THREE SOUND WAVES\0"
  "RIGHT SPEECH BUBBLE\0"
  "RIGHT SQUARE BRACKET\0"
  "RIGHT SQUARE BRACKET EXTENSION\0"
  "RIGHT SQUARE BRACKET LOWER CORNER\0"
  "RIGHT SQUARE BRACKET UPPER CORNER\0"
  "RIGHT SQUARE BRACKET WITH DOUBLE STROKE\0"
  "RIGHT SQUARE BRACKET WITH QUILL\0"
  "RIGHT SQUARE BRACKET WITH STROKE\0"
  "RIGHT SQUARE BRACKET WITH TICK IN BOTTOM CORNER\0"
  "RIGHT SQUARE BRACKET WITH TICK IN TOP CORNER\0"
  "RIGHT SQUARE BRACKET WITH UNDERBAR\0"
  "RIGHT SUBSTITUTION BRACKET\0"
  "RIGHT TACK\0"
  "RIGHT THIRD INDUCTOR\0"
  "RIGHT THIRD WHITE RIGHT POINTING INDEX\0"
  "RIGHT THOUGHT BUBBLE\0"
  "RIGHT THREE EIGHTHS BLOCK\0"
  "RIGHT THREE QUARTERS BLOCK\0"
  "RIGHT THREE QUARTERS LOWER ONE QUARTER BLOCK\0"
  "RIGHT THREE QUARTERS UPPER ONE QUARTER BLOCK\0"
  "RIGHT TORTOISE SHELL BRACKET\0"
  "RIGHT TRANSPOSITION BRACKET\0"
  "RIGHT TRIANGLE\0"
  "RIGHT TRIANGLE ABOVE LEFT TRIANGLE\0"
  "RIGHT TRIANGULAR ONE QUARTER BLOCK\0"
  "RIGHT VERTICAL BAR WITH QUILL\0"
  "RIGHT VERTICAL BOX LINE\0"
  "RIGHT VERTICAL RULER SEGMENT\0"
  "RIGHT WHITE CORNER BRACKET\0"
  "RIGHT WHITE CURLY BRACKET\0"
  "RIGHT WHITE LENTICULAR BRACKET\0"
  "RIGHT WHITE PARENTHESIS\0"
  "RIGHT WHITE SQUARE BRACKET\0"
  "RIGHT WHITE TORTOISE SHELL BRACKET\0"
  "RIGHT WIGGLY FENCE\0"
  "RIGHT-FACING ARMENIAN ETERNITY SIGN\0"
  "RIGHT-FACING BASSINET\0"
  "RIGHT-FACING FISH\0"
  "RIGHT-FACING FISH WITH OPEN MOUTH\0"
  "RIGHT-FACING FIST\0"
  "RIGHT-FACING SNAKE HEAD WITH CLOSED MOUTH\0"
  "RIGHT-FACING SNAKE HEAD WITH OPEN MOUTH\0"
  "RIGHT-FACING SVASTI SIGN\0"
  "RIGHT-FACING SVASTI SIGN WITH DOTS\0"
  "RIGHT-HANDED INTERLACED PENTAGRAM\0"
  "RIGHT-POINTING ANGLE BRACKET\0"
  "RIGHT-POINTING ATOMIC BOMB\0"
  "RIGHT-POINTING CURVED ANGLE BRACKET\0"
  "RIGHT-POINTING DIODE\0"
  "RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK\0"
  "RIGHT-POINTING ENERGY WAVE\0"
  "RIGHT-POINTING GO-KART\0"
  "RIGHT-POINTING MAGNIFYING GLASS\0"
  "RIGHT-POINTING RACING CAR\0"
  "RIGHT-POINTING RIFLE\0"
  "RIGHT-POINTING ROCKET BOOSTER\0"
  "RIGHT-POINTING ROCKET SHIP\0"
  "RIGHT-POINTING ROLLER COASTER CAR\0"
  "RIGHT-POINTING STICK FIGURE\0"
  "RIGHT-POINTING TANK\0"
  "RIGHT-SHADED WHITE RIGHTWARDS ARROW\0"
  "RIGHT-SIDE ARC CLOCKWISE ARROW\0"
  "RIGHT-TO-LEFT EMBEDDING\0"
  "RIGHT-TO-LEFT ISOLATE\0"
  "RIGHT-TO-LEFT MARK\0"
  "RIGHT-TO-LEFT OVERRIDE\0"
  "RIGHTHAND INTERIOR PRODUCT\0"
  "RIGHTWARDS ARROW\0"
  "RIGHTWARDS ARROW ABOVE ALMOST EQUAL TO\0"
  "RIGHTWARDS ARROW ABOVE REVERSE ALMOST EQUAL TO\0"
  "RIGHTWARDS ARROW ABOVE REVERSE TILDE OPERATOR\0"
  "RIGHTWARDS ARROW ABOVE SHORT LEFTWARDS ARROW\0"
  "RIGHTWARDS ARROW ABOVE TILDE OPERATOR\0"
  "RIGHTWARDS ARROW AND UPPER AND LOWER ONE EIGHTH BLOCK\0"
  "RIGHTWARDS ARROW FROM BAR\0"
  "RIGHTWARDS ARROW FROM BAR TO BLACK DIAMOND\0"
  "RIGHTWARDS ARROW FROM DOWNWARDS ARROW\0"
  "RIGHTWARDS ARROW OVER LEFTWARDS ARROW\0"
  "RIGHTWARDS ARROW THROUGH GREATER-THAN\0"
  "RIGHTWARDS ARROW THROUGH SUPERSET\0"
  "RIGHTWARDS ARROW THROUGH X\0"
  "RIGHTWARDS ARROW TO BAR\0"
  "RIGHTWARDS ARROW TO BLACK DIAMOND\0"
  "RIGHTWARDS ARROW WITH CORNER DOWNWARDS\0"
  "RIGHTWARDS ARROW WITH DOTTED STEM\0"
  "RIGHTWARDS ARROW WITH DOUBLE VERTICAL STROKE\0"
  "RIGHTWARDS ARROW WITH EQUILATERAL ARROWHEAD\0"
  "RIGHTWARDS ARROW WITH HOOK\0"
  "RIGHTWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD\0"
  "RIGHTWARDS ARROW WITH LOOP\0"
  "RIGHTWARDS ARROW WITH LOWER HOOK\0"
  "RIGHTWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD\0"
  "RIGHTWARDS ARROW WITH NOTCHED TAIL\0"
  "RIGHTWARDS ARROW WITH PLUS BELOW\0"
  "RIGHTWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD\0"
  "RIGHTWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD\0"
  "RIGHTWARDS ARROW WITH STROKE\0"
  "RIGHTWARDS ARROW WITH TAIL\0"
  "RIGHTWARDS ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE\0"
  "RIGHTWARDS ARROW WITH TAIL WITH VERTICAL STROKE\0"
  "RIGHTWARDS ARROW WITH TIP DOWNWARDS\0"
  "RIGHTWARDS ARROW WITH TIP UPWARDS\0"
  "RIGHTWARDS ARROW WITH VERTICAL STROKE\0"
  "RIGHTWARDS ARROW-TAIL\0"
  "RIGHTWARDS BACK-TILTED SHADOWED WHITE ARROW\0"
  "RIGHTWARDS BLACK ARROW\0"
  "RIGHTWARDS BLACK CIRCLED WHITE ARROW\0"
  "RIGHTWARDS BOTTOM SHADED WHITE ARROW\0"
  "RIGHTWARDS COMPRESSED ARROW\0"
  "RIGHTWARDS DASHED ARROW\0"
  "RIGHTWARDS DOUBLE ARROW\0"
  "RIGHTWARDS DOUBLE ARROW FROM BAR\0"
  "RIGHTWARDS DOUBLE ARROW WITH STROKE\0"
  "RIGHTWARDS DOUBLE ARROW WITH VERTICAL STROKE\0"
  "RIGHTWARDS DOUBLE ARROW-TAIL\0"
  "RIGHTWARDS DOUBLE DASH ARROW\0"
  "RIGHTWARDS FINGER-POST ARROW\0"
  "RIGHTWARDS FRONT-TILTED SHADOWED WHITE ARROW\0"
  "RIGHTWARDS HAND\0"
  "RIGHTWARDS HARPOON OVER LEFTWARDS HARPOON\0"
  "RIGHTWARDS HARPOON WITH BARB DOWN ABOVE LEFTWARDS HARPOON WITH BARB DOWN\0"
  "RIGHTWARDS HARPOON WITH BARB DOWN BELOW LONG DASH\0"
  "RIGHTWARDS HARPOON WITH BARB DOWN FROM BAR\0"
  "RIGHTWARDS HARPOON WITH BARB DOWN TO BAR\0"
  "RIGHTWARDS HARPOON WITH BARB DOWNWARDS\0"
  "RIGHTWARDS HARPOON WITH BARB UP ABOVE LEFTWARDS HARPOON WITH BARB UP\0"
  "RIGHTWARDS HARPOON WITH BARB UP ABOVE LONG DASH\0"
  "RIGHTWARDS HARPOON WITH BARB UP ABOVE RIGHTWARDS HARPOON WITH BARB DOWN\0"
  "RIGHTWARDS HARPOON WITH BARB UP FROM BAR\0"
  "RIGHTWARDS HARPOON WITH BARB UP TO BAR\0"
  "RIGHTWARDS HARPOON WITH BARB UPWARDS\0"
  "RIGHTWARDS HEAVY ARROW\0"
  "RIGHTWARDS HEAVY COMPRESSED ARROW\0"
  "RIGHTWARDS LEFT-SHADED WHITE ARROW\0"
  "RIGHTWARDS OPEN-HEADED ARROW\0"
  "RIGHTWARDS PAIRED ARROWS\0"
  "RIGHTWARDS PUSHING HAND\0"
  "RIGHTWARDS QUADRUPLE ARROW\0"
  "RIGHTWARDS RIGHT-SHADED WHITE ARROW\0"
  "RIGHTWARDS ROCKET\0"
  "RIGHTWARDS SANS-SERIF ARROW\0"
  "RIGHTWARDS SQUARED ARROW\0"
  "RIGHTWARDS SQUIGGLE ARROW\0"
  "RIGHTWARDS TOP SHADED WHITE ARROW\0"
  "RIGHTWARDS TRIANGLE ARROWHEAD\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW OVER LEFTWARDS TRIANGLE-HEADED ARROW\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW TO BAR\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP DOWNWARDS\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP UPWARDS\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT\0"
  "RIGHTWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT\0"
  "RIGHTWARDS TRIANGLE-HEADED DASHED ARROW\0"
  "RIGHTWARDS TRIANGLE-HEADED PAIRED ARROWS\0"
  "RIGHTWARDS TRIPLE ARROW\0"
  "RIGHTWARDS TRIPLE DASH ARROW\0"
  "RIGHTWARDS TWO HEADED ARROW\0"
  "RIGHTWARDS TWO-HEADED ARROW FROM BAR\0"
  "RIGHTWARDS TWO-HEADED ARROW WITH DOUBLE VERTICAL STROKE\0"
  "RIGHTWARDS TWO-HEADED ARROW WITH TAIL\0"
  "RIGHTWARDS TWO-HEADED ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE\0"
  "RIGHTWARDS TWO-HEADED ARROW WITH TAIL WITH VERTICAL STROKE\0"
  "RIGHTWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS\0"
  "RIGHTWARDS TWO-HEADED ARROW WITH VERTICAL STROKE\0"
  "RIGHTWARDS TWO-HEADED TRIPLE DASH ARROW\0"
  "RIGHTWARDS WAVE ARROW\0"
  "RIGHTWARDS WHITE ARROW\0"
  "RIGHTWARDS WHITE ARROW FROM WALL\0"
  "RIGHTWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD\0"
  "RING\0"
  "RING ABOVE\0"
  "RING BUOY\0"
  "RING EQUAL TO\0"
  "RING IN EQUAL TO\0"
  "RING OPERATOR\0"
  "RING POINT\0"
  "RINGED PLANET\0"
  "RINGING BELL\0"
  "RISING DIAGONAL CROSSING FALLING DIAGONAL\0"
  "RISING DIAGONAL CROSSING SOUTH EAST ARROW\0"
  "ROASTED SWEET POTATO\0"
  "ROBOT FACE\0"
  "ROCK\0"
  "ROCKET\0"
  "ROLL OF PAPER\0"
  "ROLLED-UP NEWSPAPER\0"
  "ROLLER COASTER\0"
  "ROLLER SKATE\0"
  "ROLLING ON THE FLOOR LAUGHING\0"
  "ROMAN AS SIGN\0"
  "ROMAN CENTURIAL SIGN\0"
  "ROMAN DENARIUS SIGN\0"
  "ROMAN DIMIDIA SEXTULA SIGN\0"
  "ROMAN DUPONDIUS SIGN\0"
  "ROMAN NUMERAL EIGHT\0"
  "ROMAN NUMERAL ELEVEN\0"
  "ROMAN NUMERAL FIFTY\0"
  "ROMAN NUMERAL FIFTY EARLY FORM\0"
  "ROMAN NUMERAL FIFTY THOUSAND\0"
  "ROMAN NUMERAL FIVE\0"
  "ROMAN NUMERAL FIVE HUNDRED\0"
  "ROMAN NUMERAL FIVE THOUSAND\0"
  "ROMAN NUMERAL FOUR\0"
  "ROMAN NUMERAL NINE\0"
  "ROMAN NUMERAL ONE\0"
  "ROMAN NUMERAL ONE HUNDRED\0"
  "ROMAN NUMERAL ONE HUNDRED THOUSAND\0"
  "ROMAN NUMERAL ONE THOUSAND\0"
  "ROMAN NUMERAL ONE THOUSAND C D\0"
  "ROMAN NUMERAL REVERSED ONE HUNDRED\0"
  "ROMAN NUMERAL SEVEN\0"
  "ROMAN NUMERAL SIX\0"
  "ROMAN NUMERAL SIX LATE FORM\0"
  "ROMAN NUMERAL TEN\0"
  "ROMAN NUMERAL TEN THOUSAND\0"
  "ROMAN NUMERAL THREE\0"
  "ROMAN NUMERAL TWELVE\0"
  "ROMAN NUMERAL TWO\0"
  "ROMAN QUINARIUS SIGN\0"
  "ROMAN SEMUNCIA SIGN\0"
  "ROMAN SESTERTIUS SIGN\0"
  "ROMAN SEXTANS SIGN\0"
  "ROMAN SEXTULA SIGN\0"
  "ROMAN SILIQUA SIGN\0"
  "ROMAN UNCIA SIGN\0"
  "ROOSTER\0"
  "ROOT VEGETABLE\0"
  "ROSE\0"
  "ROSETTE\0"
  "ROTATED CAPITAL Q\0"
  "ROTATED FLORAL HEART BULLET\0"
  "ROTATED HEAVY BLACK HEART BULLET\0"
  "ROTATED LIGHT FOUR POINTED BLACK CUSP\0"
  "ROTATED WHITE FOUR POINTED CUSP\0"
  "ROUND PUSHPIN\0"
  "ROUND TARGET\0"
  "ROUNDED SYMBOL FOR CAI\0"
  "ROUNDED SYMBOL FOR FU\0"
  "ROUNDED SYMBOL FOR LU\0"
  "ROUNDED SYMBOL FOR SHOU\0"
  "ROUNDED SYMBOL FOR SHUANGXI\0"
  "ROUNDED SYMBOL FOR XI\0"
  "ROWBOAT\0"
  "RUBLE SIGN\0"
  "RUGBY FOOTBALL\0"
  "RULE-DELAYED\0"
  "RUMI DIGIT EIGHT\0"
  "RUMI DIGIT FIVE\0"
  "RUMI DIGIT FOUR\0"
  "RUMI DIGIT NINE\0"
  "RUMI DIGIT ONE\0"
  "RUMI DIGIT SEVEN\0"
  "RUMI DIGIT SIX\0"
  "RUMI DIGIT THREE\0"
  "RUMI DIGIT TWO\0"
  "RUMI FRACTION ONE HALF\0"
  "RUMI FRACTION ONE QUARTER\0"
  "RUMI FRACTION ONE THIRD\0"
  "RUMI FRACTION TWO THIRDS\0"
  "RUMI NUMBER EIGHT HUNDRED\0"
  "RUMI NUMBER EIGHTY\0"
  "RUMI NUMBER FIFTY\0"
  "RUMI NUMBER FIVE HUNDRED\0"
  "RUMI NUMBER FORTY\0"
  "RUMI NUMBER FOUR HUNDRED\0"
  "RUMI NUMBER NINE HUNDRED\0"
  "RUMI NUMBER NINETY\0"
  "RUMI NUMBER ONE HUNDRED\0"
  "RUMI NUMBER SEVEN HUNDRED\0"
  "RUMI NUMBER SEVENTY\0"
  "RUMI NUMBER SIX HUNDRED\0"
  "RUMI NUMBER SIXTY\0"
  "RUMI NUMBER TEN\0"
  "RUMI NUMBER THIRTY\0"
  "RUMI NUMBER THREE HUNDRED\0"
  "RUMI NUMBER TWENTY\0"
  "RUMI NUMBER TWO HUNDRED\0"
  "RUNIC ARLAUG SYMBOL\0"
  "RUNIC BELGTHOR SYMBOL\0"
  "RUNIC CROSS PUNCTUATION\0"
  "RUNIC LETTER AC A\0"
  "RUNIC LETTER AESC\0"
  "RUNIC LETTER ALGIZ EOLHX\0"
  "RUNIC LETTER ANSUZ A\0"
  "RUNIC LETTER BERKANAN BEORC BJARKAN B\0"
  "RUNIC LETTER C\0"
  "RUNIC LETTER CALC\0"
  "RUNIC LETTER CEALC\0"
  "RUNIC LETTER CEN\0"
  "RUNIC LETTER CWEORTH\0"
  "RUNIC LETTER D\0"
  "RUNIC LETTER DAGAZ DAEG D\0"
  "RUNIC LETTER DOTTED-L\0"
  "RUNIC LETTER DOTTED-N\0"
  "RUNIC LETTER DOTTED-P\0"
  "RUNIC LETTER E\0"
  "RUNIC LETTER EAR\0"
  "RUNIC LETTER EHWAZ EH E\0"
  "RUNIC LETTER ENG\0"
  "RUNIC LETTER ETH\0"
  "RUNIC LETTER FEHU FEOH FE F\0"
  "RUNIC LETTER FRANKS CASKET AC\0"
  "RUNIC LETTER FRANKS CASKET AESC\0"
  "RUNIC LETTER FRANKS CASKET EH\0"
  "RUNIC LETTER FRANKS CASKET IS\0"
  "RUNIC LETTER FRANKS CASKET OS\0"
  "RUNIC LETTER G\0"
  "RUNIC LETTER GAR\0"
  "RUNIC LETTER GEBO GYFU G\0"
  "RUNIC LETTER GER\0"
  "RUNIC LETTER HAEGL H\0"
  "RUNIC LETTER HAGLAZ H\0"
  "RUNIC LETTER ICELANDIC-YR\0"
  "RUNIC LETTER ING\0"
  "RUNIC LETTER INGWAZ\0"
  "RUNIC LETTER IOR\0"
  "RUNIC LETTER ISAZ IS ISS I\0"
  "RUNIC LETTER IWAZ EOH\0"
  "RUNIC LETTER JERAN J\0"
  "RUNIC LETTER K\0"
  "RUNIC LETTER KAUN K\0"
  "RUNIC LETTER KAUNA\0"
  "RUNIC LETTER LAUKAZ LAGU LOGR L\0"
  "RUNIC LETTER LONG-BRANCH-AR AE\0"
  "RUNIC LETTER LONG-BRANCH-HAGALL H\0"
  "RUNIC LETTER LONG-BRANCH-MADR M\0"
  "RUNIC LETTER LONG-BRANCH-OSS O\0"
  "RUNIC LETTER LONG-BRANCH-YR\0"
  "RUNIC LETTER MANNAZ MAN M\0"
  "RUNIC LETTER NAUDIZ NYD NAUD N\0"
  "RUNIC LETTER O\0"
  "RUNIC LETTER OE\0"
  "RUNIC LETTER ON\0"
  "RUNIC LETTER OO\0"
  "RUNIC LETTER OPEN-P\0"
  "RUNIC LETTER OS O\0"
  "RUNIC LETTER OTHALAN ETHEL O\0"
  "RUNIC LETTER PERTHO PEORTH P\0"
  "RUNIC LETTER Q\0"
  "RUNIC LETTER RAIDO RAD REID R\0"
  "RUNIC LETTER SH\0"
  "RUNIC LETTER SHORT-TWIG-AR A\0"
  "RUNIC LETTER SHORT-TWIG-BJARKAN B\0"
  "RUNIC LETTER SHORT-TWIG-HAGALL H\0"
  "RUNIC LETTER SHORT-TWIG-MADR M\0"
  "RUNIC LETTER SHORT-TWIG-NAUD N\0"
  "RUNIC LETTER SHORT-TWIG-OSS O\0"
  "RUNIC LETTER SHORT-TWIG-SOL S\0"
  "RUNIC LETTER SHORT-TWIG-TYR T\0"
  "RUNIC LETTER SHORT-TWIG-YR\0"
  "RUNIC LETTER SIGEL LONG-BRANCH-SOL S\0"
  "RUNIC LETTER SOWILO S\0"
  "RUNIC LETTER STAN\0"
  "RUNIC LETTER THURISAZ THURS THORN\0"
  "RUNIC LETTER TIWAZ TIR TYR T\0"
  "RUNIC LETTER URUZ UR U\0"
  "RUNIC LETTER V\0"
  "RUNIC LETTER W\0"
  "RUNIC LETTER WUNJO WYNN W\0"
  "RUNIC LETTER X\0"
  "RUNIC LETTER Y\0"
  "RUNIC LETTER YR\0"
  "RUNIC LETTER Z\0"
  "RUNIC MULTIPLE PUNCTUATION\0"
  "RUNIC SINGLE PUNCTUATION\0"
  "RUNIC TVIMADUR SYMBOL\0"
  "RUNNER\0"
  "RUNNING SHIRT WITH SASH\0"
  "RUPEE SIGN\0"
  "RUSSIAN ASTROLOGICAL SYMBOL BINOVILE\0"
  "RUSSIAN ASTROLOGICAL SYMBOL NOVILE\0"
  "RUSSIAN ASTROLOGICAL SYMBOL QUINTILE\0"
  "RUSSIAN ASTROLOGICAL SYMBOL SENTAGON\0"
  "RUSSIAN ASTROLOGICAL SYMBOL TREDECILE\0"
  "RUSSIAN ASTROLOGICAL SYMBOL VIGINTILE\0"
  "S IN TRIANGLE\0"
  "SAFETY PIN\0"
  "SAFETY VEST\0"
  "SAGITTARIUS\0"
  "SAILBOAT\0"
  "SAKE BOTTLE AND CUP\0"
  "SALT SHAKER\0"
  "SALTIRE\0"
  "SALUTING FACE\0"
  "SAMARITAN ABBREVIATION MARK\0"
  "SAMARITAN LETTER ALAF\0"
  "SAMARITAN LETTER BAA\0"
  "SAMARITAN LETTER BIT\0"
  "SAMARITAN LETTER DALAT\0"
  "SAMARITAN LETTER FI\0"
  "SAMARITAN LETTER GAMAN\0"
  "SAMARITAN LETTER IN\0"
  "SAMARITAN LETTER IT\0"
  "SAMARITAN LETTER IY\0"
  "SAMARITAN LETTER KAAF\0"
  "SAMARITAN LETTER LABAT\0"
  "SAMARITAN LETTER MIM\0"
  "SAMARITAN LETTER NUN\0"
  "SAMARITAN LETTER QUF\0"
  "SAMARITAN LETTER RISH\0"
  "SAMARITAN LETTER SHAN\0"
  "SAMARITAN LETTER SINGAAT\0"
  "SAMARITAN LETTER TAAF\0"
  "SAMARITAN LETTER TIT\0"
  "SAMARITAN LETTER TSAADIY\0"
  "SAMARITAN LETTER YUT\0"
  "SAMARITAN LETTER ZEN\0"
  "SAMARITAN MARK DAGESH\0"
  "SAMARITAN MARK EPENTHETIC YUT\0"
  "SAMARITAN MARK IN\0"
  "SAMARITAN MARK IN-ALAF\0"
  "SAMARITAN MARK NEQUDAA\0"
  "SAMARITAN MARK OCCLUSION\0"
  "SAMARITAN MODIFIER LETTER EPENTHETIC YUT\0"
  "SAMARITAN MODIFIER LETTER I\0"
  "SAMARITAN MODIFIER LETTER SHORT A\0"
  "SAMARITAN PUNCTUATION AFSAAQ\0"
  "SAMARITAN PUNCTUATION ANGED\0"
  "SAMARITAN PUNCTUATION ANNAAU\0"
  "SAMARITAN PUNCTUATION ARKAANU\0"
  "SAMARITAN PUNCTUATION ATMAAU\0"
  "SAMARITAN PUNCTUATION BAU\0"
  "SAMARITAN PUNCTUATION MELODIC QITSA\0"
  "SAMARITAN PUNCTUATION NEQUDAA\0"
  "SAMARITAN PUNCTUATION QITSA\0"
  "SAMARITAN PUNCTUATION SHIYYAALAA\0"
  "SAMARITAN PUNCTUATION SOF MASHFAAT\0"
  "SAMARITAN PUNCTUATION TURU\0"
  "SAMARITAN PUNCTUATION ZAEF\0"
  "SAMARITAN PUNCTUATION ZIQAA\0"
  "SAMARITAN VOWEL SIGN A\0"
  "SAMARITAN VOWEL SIGN AA\0"
  "SAMARITAN VOWEL SIGN E\0"
  "SAMARITAN VOWEL SIGN I\0"
  "SAMARITAN VOWEL SIGN LONG A\0"
  "SAMARITAN VOWEL SIGN LONG AA\0"
  "SAMARITAN VOWEL SIGN LONG E\0"
  "SAMARITAN VOWEL SIGN LONG I\0"
  "SAMARITAN VOWEL SIGN LONG U\0"
  "SAMARITAN VOWEL SIGN O\0"
  "SAMARITAN VOWEL SIGN OVERLONG A\0"
  "SAMARITAN VOWEL SIGN OVERLONG AA\0"
  "SAMARITAN VOWEL SIGN SHORT A\0"
  "SAMARITAN VOWEL SIGN SUKUN\0"
  "SAMARITAN VOWEL SIGN U\0"
  "SANDWICH\0"
  "SANS-SERIF HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT\0"
  "SANS-SERIF HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT\0"
  "SANS-SERIF HEAVY LOW DOUBLE COMMA QUOTATION MARK ORNAMENT\0"
  "SANS-SERIF INTERROBANG ORNAMENT\0"
  "SARI\0"
  "SATELLITE\0"
  "SATELLITE ANTENNA\0"
  "SATURN\0"
  "SAURASHTRA CONSONANT SIGN HAARU\0"
  "SAURASHTRA DANDA\0"
  "SAURASHTRA DIGIT EIGHT\0"
  "SAURASHTRA DIGIT FIVE\0"
  "SAURASHTRA DIGIT FOUR\0"
  "SAURASHTRA DIGIT NINE\0"
  "SAURASHTRA DIGIT ONE\0"
  "SAURASHTRA DIGIT SEVEN\0"
  "SAURASHTRA DIGIT SIX\0"
  "SAURASHTRA DIGIT THREE\0"
  "SAURASHTRA DIGIT TWO\0"
  "SAURASHTRA DIGIT ZERO\0"
  "SAURASHTRA DOUBLE DANDA\0"
  "SAURASHTRA LETTER A\0"
  "SAURASHTRA LETTER AA\0"
  "SAURASHTRA LETTER AI\0"
  "SAURASHTRA LETTER AU\0"
  "SAURASHTRA LETTER BA\0"
  "SAURASHTRA LETTER BHA\0"
  "SAURASHTRA LETTER CA\0"
  "SAURASHTRA LETTER CHA\0"
  "SAURASHTRA LETTER DA\0"
  "SAURASHTRA LETTER DDA\0"
  "SAURASHTRA LETTER DDHA\0"
  "SAURASHTRA LETTER DHA\0"
  "SAURASHTRA LETTER E\0"
  "SAURASHTRA LETTER EE\0"
  "SAURASHTRA LETTER GA\0"
  "SAURASHTRA LETTER GHA\0"
  "SAURASHTRA LETTER HA\0"
  "SAURASHTRA LETTER I\0"
  "SAURASHTRA LETTER II\0"
  "SAURASHTRA LETTER JA\0"
  "SAURASHTRA LETTER JHA\0"
  "SAURASHTRA LETTER KA\0"
  "SAURASHTRA LETTER KHA\0"
  "SAURASHTRA LETTER LA\0"
  "SAURASHTRA LETTER LLA\0"
  "SAURASHTRA LETTER MA\0"
  "SAURASHTRA LETTER NA\0"
  "SAURASHTRA LETTER NGA\0"
  "SAURASHTRA LETTER NNA\0"
  "SAURASHTRA LETTER NYA\0"
  "SAURASHTRA LETTER O\0"
  "SAURASHTRA LETTER OO\0"
  "SAURASHTRA LETTER PA\0"
  "SAURASHTRA LETTER PHA\0"
  "SAURASHTRA LETTER RA\0"
  "SAURASHTRA LETTER SA\0"
  "SAURASHTRA LETTER SHA\0"
  "SAURASHTRA LETTER SSA\0"
  "SAURASHTRA LETTER TA\0"
  "SAURASHTRA LETTER THA\0"
  "SAURASHTRA LETTER TTA\0"
  "SAURASHTRA LETTER TTHA\0"
  "SAURASHTRA LETTER U\0"
  "SAURASHTRA LETTER UU\0"
  "SAURASHTRA LETTER VA\0"
  "SAURASHTRA LETTER VOCALIC L\0"
  "SAURASHTRA LETTER VOCALIC LL\0"
  "SAURASHTRA LETTER VOCALIC R\0"
  "SAURASHTRA LETTER VOCALIC RR\0"
  "SAURASHTRA LETTER YA\0"
  "SAURASHTRA SIGN ANUSVARA\0"
  "SAURASHTRA SIGN CANDRABINDU\0"
  "SAURASHTRA SIGN VIRAMA\0"
  "SAURASHTRA SIGN VISARGA\0"
  "SAURASHTRA VOWEL SIGN AA\0"
  "SAURASHTRA VOWEL SIGN AI\0"
  "SAURASHTRA VOWEL SIGN AU\0"
  "SAURASHTRA VOWEL SIGN E\0"
  "SAURASHTRA VOWEL SIGN EE\0"
  "SAURASHTRA VOWEL SIGN I\0"
  "SAURASHTRA VOWEL SIGN II\0"
  "SAURASHTRA VOWEL SIGN O\0"
  "SAURASHTRA VOWEL SIGN OO\0"
  "SAURASHTRA VOWEL SIGN U\0"
  "SAURASHTRA VOWEL SIGN UU\0"
  "SAURASHTRA VOWEL SIGN VOCALIC L\0"
  "SAURASHTRA VOWEL SIGN VOCALIC LL\0"
  "SAURASHTRA VOWEL SIGN VOCALIC R\0"
  "SAURASHTRA VOWEL SIGN VOCALIC RR\0"
  "SAUROPOD\0"
  "SAXOPHONE\0"
  "SCALES\0"
  "SCARF\0"
  "SCHOOL\0"
  "SCHOOL SATCHEL\0"
  "SCOOTER\0"
  "SCORPION\0"
  "SCORPIUS\0"
  "SCREEN\0"
  "SCREWDRIVER\0"
  "SCRIPT CAPITAL B\0"
  "SCRIPT CAPITAL E\0"
  "SCRIPT CAPITAL F\0"
  "SCRIPT CAPITAL H\0"
  "SCRIPT CAPITAL I\0"
  "SCRIPT CAPITAL L\0"
  "SCRIPT CAPITAL M\0"
  "SCRIPT CAPITAL P\0"
  "SCRIPT CAPITAL R\0"
  "SCRIPT LIGATURE ET ORNAMENT\0"
  "SCRIPT SMALL E\0"
  "SCRIPT SMALL G\0"
  "SCRIPT SMALL L\0"
  "SCRIPT SMALL O\0"
  "SCROLL\0"
  "SCRUPLE\0"
  "SEAL\0"
  "SEAT\0"
  "SECOND PLACE MEDAL\0"
  "SECTION SIGN\0"
  "SECTOR\0"
  "SEDNA\0"
  "SEE-NO-EVIL MONKEY\0"
  "SEEDLING\0"
  "SEGMENT\0"
  "SEGMENTED DIGIT EIGHT\0"
  "SEGMENTED DIGIT FIVE\0"
  "SEGMENTED DIGIT FOUR\0"
  "SEGMENTED DIGIT NINE\0"
  "SEGMENTED DIGIT ONE\0"
  "SEGMENTED DIGIT SEVEN\0"
  "SEGMENTED DIGIT SIX\0"
  "SEGMENTED DIGIT THREE\0"
  "SEGMENTED DIGIT TWO\0"
  "SEGMENTED DIGIT ZERO\0"
  "SELFIE\0"
  "SEMICOLON\0"
  "SEMIDIRECT PRODUCT WITH BOTTOM CLOSED\0"
  "SEMISEXTILE\0"
  "SEPARATED BLOCK QUADRANT-1\0"
  "SEPARATED BLOCK QUADRANT-12\0"
  "SEPARATED BLOCK QUADRANT-123\0"
  "SEPARATED BLOCK QUADRANT-1234\0"
  "SEPARATED BLOCK QUADRANT-124\0"
  "SEPARATED BLOCK QUADRANT-13\0"
  "SEPARATED BLOCK QUADRANT-134\0"
  "SEPARATED BLOCK QUADRANT-14\0"
  "SEPARATED BLOCK QUADRANT-2\0"
  "SEPARATED BLOCK QUADRANT-23\0"
  "SEPARATED BLOCK QUADRANT-234\0"
  "SEPARATED BLOCK QUADRANT-24\0"
  "SEPARATED BLOCK QUADRANT-3\0"
  "SEPARATED BLOCK QUADRANT-34\0"
  "SEPARATED BLOCK QUADRANT-4\0"
  "SEPARATED BLOCK SEXTANT-1\0"
  "SEPARATED BLOCK SEXTANT-12\0"
  "SEPARATED BLOCK SEXTANT-123\0"
  "SEPARATED BLOCK SEXTANT-1234\0"
  "SEPARATED BLOCK SEXTANT-12345\0"
  "SEPARATED BLOCK SEXTANT-123456\0"
  "SEPARATED BLOCK SEXTANT-12346\0"
  "SEPARATED BLOCK SEXTANT-1235\0"
  "SEPARATED BLOCK SEXTANT-12356\0"
  "SEPARATED BLOCK SEXTANT-1236\0"
  "SEPARATED BLOCK SEXTANT-124\0"
  "SEPARATED BLOCK SEXTANT-1245\0"
  "SEPARATED BLOCK SEXTANT-12456\0"
  "SEPARATED BLOCK SEXTANT-1246\0"
  "SEPARATED BLOCK SEXTANT-125\0"
  "SEPARATED BLOCK SEXTANT-1256\0"
  "SEPARATED BLOCK SEXTANT-126\0"
  "SEPARATED BLOCK SEXTANT-13\0"
  "SEPARATED BLOCK SEXTANT-134\0"
  "SEPARATED BLOCK SEXTANT-1345\0"
  "SEPARATED BLOCK SEXTANT-13456\0"
  "SEPARATED BLOCK SEXTANT-1346\0"
  "SEPARATED BLOCK SEXTANT-135\0"
  "SEPARATED BLOCK SEXTANT-1356\0"
  "SEPARATED BLOCK SEXTANT-136\0"
  "SEPARATED BLOCK SEXTANT-14\0"
  "SEPARATED BLOCK SEXTANT-145\0"
  "SEPARATED BLOCK SEXTANT-1456\0"
  "SEPARATED BLOCK SEXTANT-146\0"
  "SEPARATED BLOCK SEXTANT-15\0"
  "SEPARATED BLOCK SEXTANT-156\0"
  "SEPARATED BLOCK SEXTANT-16\0"
  "SEPARATED BLOCK SEXTANT-2\0"
  "SEPARATED BLOCK SEXTANT-23\0"
  "SEPARATED BLOCK SEXTANT-234\0"
  "SEPARATED BLOCK SEXTANT-2345\0"
  "SEPARATED BLOCK SEXTANT-23456\0"
  "SEPARATED BLOCK SEXTANT-2346\0"
  "SEPARATED BLOCK SEXTANT-235\0"
  "SEPARATED BLOCK SEXTANT-2356\0"
  "SEPARATED BLOCK SEXTANT-236\0"
  "SEPARATED BLOCK SEXTANT-24\0"
  "SEPARATED BLOCK SEXTANT-245\0"
  "SEPARATED BLOCK SEXTANT-2456\0"
  "SEPARATED BLOCK SEXTANT-246\0"
  "SEPARATED BLOCK SEXTANT-25\0"
  "SEPARATED BLOCK SEXTANT-256\0"
  "SEPARATED BLOCK SEXTANT-26\0"
  "SEPARATED BLOCK SEXTANT-3\0"
  "SEPARATED BLOCK SEXTANT-34\0"
  "SEPARATED BLOCK SEXTANT-345\0"
  "SEPARATED BLOCK SEXTANT-3456\0"
  "SEPARATED BLOCK SEXTANT-346\0"
  "SEPARATED BLOCK SEXTANT-35\0"
  "SEPARATED BLOCK SEXTANT-356\0"
  "SEPARATED BLOCK SEXTANT-36\0"
  "SEPARATED BLOCK SEXTANT-4\0"
  "SEPARATED BLOCK SEXTANT-45\0"
  "SEPARATED BLOCK SEXTANT-456\0"
  "SEPARATED BLOCK SEXTANT-46\0"
  "SEPARATED BLOCK SEXTANT-5\0"
  "SEPARATED BLOCK SEXTANT-56\0"
  "SEPARATED BLOCK SEXTANT-6\0"
  "SEPARATED SYMBOL\0"
  "SERIOUS FACE WITH SYMBOLS COVERING MOUTH\0"
  "SERVICE MARK\0"
  "SESAME DOT\0"
  "SESQUIQUADRATE\0"
  "SET MINUS\0"
  "SEWING NEEDLE\0"
  "SEXTILE\0"
  "SHADOWED WHITE CIRCLE\0"
  "SHADOWED WHITE LATIN CROSS\0"
  "SHADOWED WHITE STAR\0"
  "SHAKING FACE\0"
  "SHALLOW PAN OF FOOD\0"
  "SHAMROCK\0"
  "SHARADA ABBREVIATION SIGN\0"
  "SHARADA CONTINUATION SIGN\0"
  "SHARADA DANDA\0"
  "SHARADA DIGIT EIGHT\0"
  "SHARADA DIGIT FIVE\0"
  "SHARADA DIGIT FOUR\0"
  "SHARADA DIGIT NINE\0"
  "SHARADA DIGIT ONE\0"
  "SHARADA DIGIT SEVEN\0"
  "SHARADA DIGIT SIX\0"
  "SHARADA DIGIT THREE\0"
  "SHARADA DIGIT TWO\0"
  "SHARADA DIGIT ZERO\0"
  "SHARADA DOUBLE DANDA\0"
  "SHARADA EKAM\0"
  "SHARADA EXTRA SHORT VOWEL MARK\0"
  "SHARADA HEADSTROKE\0"
  "SHARADA LETTER A\0"
  "SHARADA LETTER AA\0"
  "SHARADA LETTER AI\0"
  "SHARADA LETTER AU\0"
  "SHARADA LETTER BA\0"
  "SHARADA LETTER BHA\0"
  "SHARADA LETTER CA\0"
  "SHARADA LETTER CHA\0"
  "SHARADA LETTER DA\0"
  "SHARADA LETTER DDA\0"
  "SHARADA LETTER DDHA\0"
  "SHARADA LETTER DHA\0"
  "SHARADA LETTER E\0"
  "SHARADA LETTER GA\0"
  "SHARADA LETTER GHA\0"
  "SHARADA LETTER HA\0"
  "SHARADA LETTER I\0"
  "SHARADA LETTER II\0"
  "SHARADA LETTER JA\0"
  "SHARADA LETTER JHA\0"
  "SHARADA LETTER KA\0"
  "SHARADA LETTER KHA\0"
  "SHARADA LETTER LA\0"
  "SHARADA LETTER LLA\0"
  "SHARADA LETTER MA\0"
  "SHARADA LETTER NA\0"
  "SHARADA LETTER NGA\0"
  "SHARADA LETTER NNA\0"
  "SHARADA LETTER NYA\0"
  "SHARADA LETTER O\0"
  "SHARADA LETTER PA\0"
  "SHARADA LETTER PHA\0"
  "SHARADA LETTER RA\0"
  "SHARADA LETTER SA\0"
  "SHARADA LETTER SHA\0"
  "SHARADA LETTER SSA\0"
  "SHARADA LETTER TA\0"
  "SHARADA LETTER THA\0"
  "SHARADA LETTER TTA\0"
  "SHARADA LETTER TTHA\0"
  "SHARADA LETTER U\0"
  "SHARADA LETTER UU\0"
  "SHARADA LETTER VA\0"
  "SHARADA LETTER VOCALIC L\0"
  "SHARADA LETTER VOCALIC LL\0"
  "SHARADA LETTER VOCALIC R\0"
  "SHARADA LETTER VOCALIC RR\0"
  "SHARADA LETTER YA\0"
  "SHARADA OM\0"
  "SHARADA SANDHI MARK\0"
  "SHARADA SECTION MARK-1\0"
  "SHARADA SECTION MARK-2\0"
  "SHARADA SEPARATOR\0"
  "SHARADA SIGN ANUSVARA\0"
  "SHARADA SIGN AVAGRAHA\0"
  "SHARADA SIGN CANDRABINDU\0"
  "SHARADA SIGN INVERTED CANDRABINDU\0"
  "SHARADA SIGN JIHVAMULIYA\0"
  "SHARADA SIGN NUKTA\0"
  "SHARADA SIGN SIDDHAM\0"
  "SHARADA SIGN UPADHMANIYA\0"
  "SHARADA SIGN VIRAMA\0"
  "SHARADA SIGN VISARGA\0"
  "SHARADA SUTRA MARK\0"
  "SHARADA VOWEL MODIFIER MARK\0"
  "SHARADA VOWEL SIGN AA\0"
  "SHARADA VOWEL SIGN AI\0"
  "SHARADA VOWEL SIGN AU\0"
  "SHARADA VOWEL SIGN E\0"
  "SHARADA VOWEL SIGN I\0"
  "SHARADA VOWEL SIGN II\0"
  "SHARADA VOWEL SIGN O\0"
  "SHARADA VOWEL SIGN PRISHTHAMATRA E\0"
  "SHARADA VOWEL SIGN U\0"
  "SHARADA VOWEL SIGN UU\0"
  "SHARADA VOWEL SIGN VOCALIC L\0"
  "SHARADA VOWEL SIGN VOCALIC LL\0"
  "SHARADA VOWEL SIGN VOCALIC R\0"
  "SHARADA VOWEL SIGN VOCALIC RR\0"
  "SHARK\0"
  "SHAVED ICE\0"
  "SHAVIAN LETTER ADO\0"
  "SHAVIAN LETTER AGE\0"
  "SHAVIAN LETTER AH\0"
  "SHAVIAN LETTER AIR\0"
  "SHAVIAN LETTER ARE\0"
  "SHAVIAN LETTER ARRAY\0"
  "SHAVIAN LETTER ASH\0"
  "SHAVIAN LETTER AWE\0"
  "SHAVIAN LETTER BIB\0"
  "SHAVIAN LETTER CHURCH\0"
  "SHAVIAN LETTER DEAD\0"
  "SHAVIAN LETTER EAR\0"
  "SHAVIAN LETTER EAT\0"
  "SHAVIAN LETTER EGG\0"
  "SHAVIAN LETTER ERR\0"
  "SHAVIAN LETTER FEE\0"
  "SHAVIAN LETTER GAG\0"
  "SHAVIAN LETTER HA-HA\0"
  "SHAVIAN LETTER HUNG\0"
  "SHAVIAN LETTER IAN\0"
  "SHAVIAN LETTER ICE\0"
  "SHAVIAN LETTER IF\0"
  "SHAVIAN LETTER JUDGE\0"
  "SHAVIAN LETTER KICK\0"
  "SHAVIAN LETTER LOLL\0"
  "SHAVIAN LETTER MEASURE\0"
  "SHAVIAN LETTER MIME\0"
  "SHAVIAN LETTER NUN\0"
  "SHAVIAN LETTER OAK\0"
  "SHAVIAN LETTER OIL\0"
  "SHAVIAN LETTER ON\0"
  "SHAVIAN LETTER OOZE\0"
  "SHAVIAN LETTER OR\0"
  "SHAVIAN LETTER OUT\0"
  "SHAVIAN LETTER PEEP\0"
  "SHAVIAN LETTER ROAR\0"
  "SHAVIAN LETTER SO\0"
  "SHAVIAN LETTER SURE\0"
  "SHAVIAN LETTER THEY\0"
  "SHAVIAN LETTER THIGH\0"
  "SHAVIAN LETTER TOT\0"
  "SHAVIAN LETTER UP\0"
  "SHAVIAN LETTER VOW\0"
  "SHAVIAN LETTER WOE\0"
  "SHAVIAN LETTER WOOL\0"
  "SHAVIAN LETTER YEA\0"
  "SHAVIAN LETTER YEW\0"
  "SHAVIAN LETTER ZOO\0"
  "SHEEP\0"
  "SHIELD\0"
  "SHINTO SHRINE\0"
  "SHIP\0"
  "SHOCKED FACE WITH EXPLODING HEAD\0"
  "SHOOTING STAR\0"
  "SHOPPING BAGS\0"
  "SHOPPING TROLLEY\0"
  "SHORT BACKSLANTED SOUTH ARROW\0"
  "SHORT BENT ARROW POINTING DOWNWARDS THEN NORTH EAST\0"
  "SHORT DOWN TACK\0"
  "SHORT DOWN TACK WITH OVERBAR\0"
  "SHORT LEFT TACK\0"
  "SHORT RIGHTWARDS ARROW ABOVE LEFTWARDS ARROW\0"
  "SHORT SLANTED NORTH ARROW\0"
  "SHORT UP TACK\0"
  "SHORT UP TACK ABOVE SHORT DOWN TACK\0"
  "SHORT UP TACK WITH UNDERBAR\0"
  "SHORTCAKE\0"
  "SHORTHAND FORMAT CONTINUING OVERLAP\0"
  "SHORTHAND FORMAT DOWN STEP\0"
  "SHORTHAND FORMAT LETTER OVERLAP\0"
  "SHORTHAND FORMAT UP STEP\0"
  "SHORTS\0"
  "SHOULDERED OPEN BOX\0"
  "SHOVEL\0"
  "SHOWER\0"
  "SHRIMP\0"
  "SHRUG\0"
  "SHUFFLE PRODUCT\0"
  "SIDDHAM DANDA\0"
  "SIDDHAM DOUBLE DANDA\0"
  "SIDDHAM END OF TEXT MARK\0"
  "SIDDHAM LETTER A\0"
  "SIDDHAM LETTER AA\0"
  "SIDDHAM LETTER AI\0"
  "SIDDHAM LETTER ALTERNATE U\0"
  "SIDDHAM LETTER AU\0"
  "SIDDHAM LETTER BA\0"
  "SIDDHAM LETTER BHA\0"
  "SIDDHAM LETTER CA\0"
  "SIDDHAM LETTER CHA\0"
  "SIDDHAM LETTER DA\0"
  "SIDDHAM LETTER DDA\0"
  "SIDDHAM LETTER DDHA\0"
  "SIDDHAM LETTER DHA\0"
  "SIDDHAM LETTER E\0"
  "SIDDHAM LETTER GA\0"
  "SIDDHAM LETTER GHA\0"
  "SIDDHAM LETTER HA\0"
  "SIDDHAM LETTER I\0"
  "SIDDHAM LETTER II\0"
  "SIDDHAM LETTER JA\0"
  "SIDDHAM LETTER JHA\0"
  "SIDDHAM LETTER KA\0"
  "SIDDHAM LETTER KHA\0"
  "SIDDHAM LETTER LA\0"
  "SIDDHAM LETTER MA\0"
  "SIDDHAM LETTER NA\0"
  "SIDDHAM LETTER NGA\0"
  "SIDDHAM LETTER NNA\0"
  "SIDDHAM LETTER NYA\0"
  "SIDDHAM LETTER O\0"
  "SIDDHAM LETTER PA\0"
  "SIDDHAM LETTER PHA\0"
  "SIDDHAM LETTER RA\0"
  "SIDDHAM LETTER SA\0"
  "SIDDHAM LETTER SHA\0"
  "SIDDHAM LETTER SSA\0"
  "SIDDHAM LETTER TA\0"
  "SIDDHAM LETTER THA\0"
  "SIDDHAM LETTER THREE-CIRCLE ALTERNATE I\0"
  "SIDDHAM LETTER TTA\0"
  "SIDDHAM LETTER TTHA\0"
  "SIDDHAM LETTER TWO-CIRCLE ALTERNATE I\0"
  "SIDDHAM LETTER TWO-CIRCLE ALTERNATE II\0"
  "SIDDHAM LETTER U\0"
  "SIDDHAM LETTER UU\0"
  "SIDDHAM LETTER VA\0"
  "SIDDHAM LETTER VOCALIC L\0"
  "SIDDHAM LETTER VOCALIC LL\0"
  "SIDDHAM LETTER VOCALIC R\0"
  "SIDDHAM LETTER VOCALIC RR\0"
  "SIDDHAM LETTER YA\0"
  "SIDDHAM REPETITION MARK-1\0"
  "SIDDHAM REPETITION MARK-2\0"
  "SIDDHAM REPETITION MARK-3\0"
  "SIDDHAM SECTION MARK DOUBLE RING\0"
  "SIDDHAM SECTION MARK DOUBLE RING WITH RAYS\0"
  "SIDDHAM SECTION MARK WITH CIRCLES AND FOUR ENCLOSURES\0"
  "SIDDHAM SECTION MARK WITH CIRCLES AND RAYS\0"
  "SIDDHAM SECTION MARK WITH CIRCLES AND TWO ENCLOSURES\0"
  "SIDDHAM SECTION MARK WITH DOUBLE CRESCENTS\0"
  "SIDDHAM SECTION MARK WITH QUADRUPLE CRESCENTS\0"
  "SIDDHAM SECTION MARK WITH RAYS AND DOTTED CRESCENTS\0"
  "SIDDHAM SECTION MARK WITH RAYS AND DOTTED DOUBLE CRESCENTS\0"
  "SIDDHAM SECTION MARK WITH RAYS AND DOTTED TRIPLE CRESCENTS\0"
  "SIDDHAM SECTION MARK WITH SEPTUPLE CRESCENTS\0"
  "SIDDHAM SECTION MARK WITH TRIDENT AND DOTTED CRESCENTS\0"
  "SIDDHAM SECTION MARK WITH TRIDENT AND U-SHAPED ORNAMENTS\0"
  "SIDDHAM SECTION MARK WITH TRIPLE CRESCENTS\0"
  "SIDDHAM SEPARATOR BAR\0"
  "SIDDHAM SEPARATOR DOT\0"
  "SIDDHAM SIGN ANUSVARA\0"
  "SIDDHAM SIGN CANDRABINDU\0"
  "SIDDHAM SIGN NUKTA\0"
  "SIDDHAM SIGN SIDDHAM\0"
  "SIDDHAM SIGN VIRAMA\0"
  "SIDDHAM SIGN VISARGA\0"
  "SIDDHAM VOWEL SIGN AA\0"
  "SIDDHAM VOWEL SIGN AI\0"
  "SIDDHAM VOWEL SIGN ALTERNATE U\0"
  "SIDDHAM VOWEL SIGN ALTERNATE UU\0"
  "SIDDHAM VOWEL SIGN AU\0"
  "SIDDHAM VOWEL SIGN E\0"
  "SIDDHAM VOWEL SIGN I\0"
  "SIDDHAM VOWEL SIGN II\0"
  "SIDDHAM VOWEL SIGN O\0"
  "SIDDHAM VOWEL SIGN U\0"
  "SIDDHAM VOWEL SIGN UU\0"
  "SIDDHAM VOWEL SIGN VOCALIC R\0"
  "SIDDHAM VOWEL SIGN VOCALIC RR\0"
  "SIDEWAYS BLACK DOWN POINTING INDEX\0"
  "SIDEWAYS BLACK LEFT POINTING INDEX\0"
  "SIDEWAYS BLACK RIGHT POINTING INDEX\0"
  "SIDEWAYS BLACK UP POINTING INDEX\0"
  "SIDEWAYS WHITE DOWN POINTING INDEX\0"
  "SIDEWAYS WHITE LEFT POINTING INDEX\0"
  "SIDEWAYS WHITE RIGHT POINTING INDEX\0"
  "SIDEWAYS WHITE UP POINTING INDEX\0"
  "SIGN OF THE HORNS\0"
  "SIGNWRITING AIR BLOW SMALL ROTATIONS\0"
  "SIGNWRITING AIR BLOWING OUT\0"
  "SIGNWRITING AIR SUCK SMALL ROTATIONS\0"
  "SIGNWRITING AIR SUCKING IN\0"
  "SIGNWRITING BREATH EXHALE\0"
  "SIGNWRITING BREATH INHALE\0"
  "SIGNWRITING BRUSH BETWEEN\0"
  "SIGNWRITING BRUSH MULTIPLE\0"
  "SIGNWRITING BRUSH SINGLE\0"
  "SIGNWRITING CHEEKS NEUTRAL\0"
  "SIGNWRITING CHEEKS PUFFED\0"
  "SIGNWRITING CHEEKS SUCKED\0"
  "SIGNWRITING COLON\0"
  "SIGNWRITING COMMA\0"
  "SIGNWRITING DREAMY EYEBROWS DOWN NEUTRAL\0"
  "SIGNWRITING DREAMY EYEBROWS NEUTRAL DOWN\0"
  "SIGNWRITING DREAMY EYEBROWS NEUTRAL UP\0"
  "SIGNWRITING DREAMY EYEBROWS UP NEUTRAL\0"
  "SIGNWRITING DYNAMIC ARROWHEAD LARGE\0"
  "SIGNWRITING DYNAMIC ARROWHEAD SMALL\0"
  "SIGNWRITING DYNAMIC EVERY OTHER TIME\0"
  "SIGNWRITING DYNAMIC FAST\0"
  "SIGNWRITING DYNAMIC GRADUAL\0"
  "SIGNWRITING DYNAMIC RELAXED\0"
  "SIGNWRITING DYNAMIC SIMULTANEOUS\0"
  "SIGNWRITING DYNAMIC SIMULTANEOUS ALTERNATING\0"
  "SIGNWRITING DYNAMIC SLOW\0"
  "SIGNWRITING DYNAMIC TENSE\0"
  "SIGNWRITING EARS\0"
  "SIGNWRITING EXCITEMENT\0"
  "SIGNWRITING EYE BLINK MULTIPLE\0"
  "SIGNWRITING EYE BLINK SINGLE\0"
  "SIGNWRITING EYE WINK\0"
  "SIGNWRITING EYEBROWS STRAIGHT DOWN\0"
  "SIGNWRITING EYEBROWS STRAIGHT NEUTRAL\0"
  "SIGNWRITING EYEBROWS STRAIGHT UP\0"
  "SIGNWRITING EYEGAZE-FLOORPLANE CURVED\0"
  "SIGNWRITING EYEGAZE-FLOORPLANE STRAIGHT\0"
  "SIGNWRITING EYEGAZE-FLOORPLANE STRAIGHT ALTERNATING\0"
  "SIGNWRITING EYEGAZE-FLOORPLANE STRAIGHT DOUBLE\0"
  "SIGNWRITING EYEGAZE-WALLPLANE CIRCLING\0"
  "SIGNWRITING EYEGAZE-WALLPLANE CURVED\0"
  "SIGNWRITING EYEGAZE-WALLPLANE STRAIGHT\0"
  "SIGNWRITING EYEGAZE-WALLPLANE STRAIGHT ALTERNATING\0"
  "SIGNWRITING EYEGAZE-WALLPLANE STRAIGHT DOUBLE\0"
  "SIGNWRITING EYELASHES DOWN\0"
  "SIGNWRITING EYELASHES FLUTTERING\0"
  "SIGNWRITING EYELASHES UP\0"
  "SIGNWRITING EYES CLOSED\0"
  "SIGNWRITING EYES HALF CLOSED\0"
  "SIGNWRITING EYES HALF OPEN\0"
  "SIGNWRITING EYES OPEN\0"
  "SIGNWRITING EYES SQUEEZED\0"
  "SIGNWRITING EYES WIDE OPEN\0"
  "SIGNWRITING EYES WIDENING MOVEMENT\0"
  "SIGNWRITING FACE DIRECTION POSITION NOSE FORWARD TILTING\0"
  "SIGNWRITING FACE DIRECTION POSITION NOSE UP OR DOWN\0"
  "SIGNWRITING FACE DIRECTION POSITION NOSE UP OR DOWN TILTING\0"
  "SIGNWRITING FILL MODIFIER-2\0"
  "SIGNWRITING FILL MODIFIER-3\0"
  "SIGNWRITING FILL MODIFIER-4\0"
  "SIGNWRITING FILL MODIFIER-5\0"
  "SIGNWRITING FILL MODIFIER-6\0"
  "SIGNWRITING FINGER\0"
  "SIGNWRITING FLICK LARGE MULTIPLE\0"
  "SIGNWRITING FLICK LARGE SINGLE\0"
  "SIGNWRITING FLICK SEQUENTIAL\0"
  "SIGNWRITING FLICK SMALL MULTIPLE\0"
  "SIGNWRITING FLICK SMALL SINGLE\0"
  "SIGNWRITING FLOORPLANE SHOULDER HIP MOVE\0"
  "SIGNWRITING FOREHEAD CONTACT\0"
  "SIGNWRITING FOREHEAD NEUTRAL\0"
  "SIGNWRITING FOREHEAD WRINKLED\0"
  "SIGNWRITING FULL STOP\0"
  "SIGNWRITING GRASP BETWEEN\0"
  "SIGNWRITING GRASP MULTIPLE\0"
  "SIGNWRITING GRASP SINGLE\0"
  "SIGNWRITING HAIR\0"
  "SIGNWRITING HAND-ANGLE\0"
  "SIGNWRITING HAND-ANGLE INDEX\0"
  "SIGNWRITING HAND-ANGLE INDEX MIDDLE LITTLE\0"
  "SIGNWRITING HAND-ANGLE INDEX MIDDLE RING\0"
  "SIGNWRITING HAND-ANGLE INDEX RING LITTLE\0"
  "SIGNWRITING HAND-ANGLE INDEX RING LITTLE OUT\0"
  "SIGNWRITING HAND-ANGLE INDEX THUMB\0"
  "SIGNWRITING HAND-ANGLE INDEX THUMB IN\0"
  "SIGNWRITING HAND-ANGLE INDEX THUMB OUT\0"
  "SIGNWRITING HAND-ANGLE LITTLE INDEX\0"
  "SIGNWRITING HAND-ANGLE LITTLE INDEX THUMB INDEX THUMB\0"
  "SIGNWRITING HAND-ANGLE LITTLE INDEX THUMB INDEX THUMB OUT\0"
  "SIGNWRITING HAND-ANGLE LITTLE UP\0"
  "SIGNWRITING HAND-ANGLE MIDDLE RING LITTLE\0"
  "SIGNWRITING HAND-ANGLE MIDDLE RING LITTLE IN\0"
  "SIGNWRITING HAND-ANGLE MIDDLE RING LITTLE OUT\0"
  "SIGNWRITING HAND-ANGLE RING DOWN MIDDLE THUMB INDEX CROSS\0"
  "SIGNWRITING HAND-ANGLE RING LITTLE\0"
  "SIGNWRITING HAND-CIRCLE\0"
  "SIGNWRITING HAND-CIRCLE INDEX\0"
  "SIGNWRITING HAND-CIRCLE INDEX BENT\0"
  "SIGNWRITING HAND-CIRCLE INDEX HINGE\0"
  "SIGNWRITING HAND-CIRCLE INDEX MIDDLE\0"
  "SIGNWRITING HAND-CIRCLE INDEX MIDDLE CROSS LITTLE\0"
  "SIGNWRITING HAND-CIRCLE INDEX MIDDLE CROSSED\0"
  "SIGNWRITING HAND-CIRCLE INDEX MIDDLE LITTLE\0"
  "SIGNWRITING HAND-CIRCLE INDEX MIDDLE RING\0"
  "SIGNWRITING HAND-CIRCLE INDEX MIDDLE THUMB\0"
  "SIGNWRITING HAND-CIRCLE INDEX RING LITTLE\0"
  "SIGNWRITING HAND-CIRCLE LITTLE INDEX\0"
  "SIGNWRITING HAND-CIRCLE LITTLE UP\0"
  "SIGNWRITING HAND-CIRCLE MIDDLE RING LITTLE\0"
  "SIGNWRITING HAND-CIRCLE MIDDLE RING LITTLE BENT\0"
  "SIGNWRITING HAND-CIRCLE MIDDLE UP\0"
  "SIGNWRITING HAND-CIRCLE RING LITTLE\0"
  "SIGNWRITING HAND-CLAW\0"
  "SIGNWRITING HAND-CLAW FOUR FINGERS CONJOINED\0"
  "SIGNWRITING HAND-CLAW INDEX THUMB CURVE THUMB INSIDE\0"
  "SIGNWRITING HAND-CLAW MIDDLE RING LITTLE CONJOINED\0"
  "SIGNWRITING HAND-CLAW MIDDLE RING LITTLE CONJOINED SIDE\0"
  "SIGNWRITING HAND-CLAW NO THUMB\0"
  "SIGNWRITING HAND-CLAW THUMB FORWARD\0"
  "SIGNWRITING HAND-CLAW THUMB SIDE\0"
  "SIGNWRITING HAND-CUP\0"
  "SIGNWRITING HAND-CUP FIVE FINGERS SPREAD\0"
  "SIGNWRITING HAND-CUP FIVE FINGERS SPREAD OPEN\0"
  "SIGNWRITING HAND-CUP INDEX\0"
  "SIGNWRITING HAND-CUP INDEX RING LITTLE\0"
  "SIGNWRITING HAND-CUP INDEX THUMB\0"
  "SIGNWRITING HAND-CUP INDEX THUMB OPEN\0"
  "SIGNWRITING HAND-CUP MIDDLE RING LITTLE\0"
  "SIGNWRITING HAND-CUP NO THUMB\0"
  "SIGNWRITING HAND-CUP OPEN\0"
  "SIGNWRITING HAND-CUP OPEN NO THUMB\0"
  "SIGNWRITING HAND-CUP OPEN THUMB FORWARD\0"
  "SIGNWRITING HAND-CUP OPEN THUMB SIDE\0"
  "SIGNWRITING HAND-CUP THUMB FORWARD\0"
  "SIGNWRITING HAND-CUP THUMB SIDE\0"
  "SIGNWRITING HAND-CURLICUE\0"
  "SIGNWRITING HAND-CURLICUE INDEX RING LITTLE ON\0"
  "SIGNWRITING HAND-CURLICUE MIDDLE RING LITTLE ON\0"
  "SIGNWRITING HAND-CURLICUE OPEN\0"
  "SIGNWRITING HAND-FIST\0"
  "SIGNWRITING HAND-FIST FOUR FINGERS CONJOINED BENT\0"
  "SIGNWRITING HAND-FIST HEEL\0"
  "SIGNWRITING HAND-FIST INDEX\0"
  "SIGNWRITING HAND-FIST INDEX BENT\0"
  "SIGNWRITING HAND-FIST INDEX BENT OVER MIDDLE\0"
  "SIGNWRITING HAND-FIST INDEX CUPPED\0"
  "SIGNWRITING HAND-FIST INDEX HINGED\0"
  "SIGNWRITING HAND-FIST INDEX HINGED LOW\0"
  "SIGNWRITING HAND-FIST INDEX HINGED MIDDLE UP\0"
  "SIGNWRITING HAND-FIST INDEX HINGED MIDDLE UP THUMB SIDE\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE BENT\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE BENT THUMB STRAIGHT\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED CUPPED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED CUPPED THUMB FORWARD\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED HINGED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED HINGED THUMB SIDE\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED INDEX BENT\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED MIDDLE BENT\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB FORWARD\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB SIDE\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB SIDE BENT\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB SIDE CONJOINED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CROSS LITTLE\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CROSSED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE CROSSED THUMB SIDE\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE HINGED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE HINGED SPREAD THUMB SIDE\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE LITTLE\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE RAISED KNUCKLES\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE RING\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE RING BENT\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE RING CONJOINED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE STRAIGHT THUMB BENT\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB ANGLED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB ANGLED OUT\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB BENT\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB CIRCLED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB CONJOINED HINGED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB CUPPED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB HINGED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB HOOKED\0"
  "SIGNWRITING HAND-FIST INDEX MIDDLE UP SPREAD THUMB FORWARD\0"
  "SIGNWRITING HAND-FIST INDEX RAISED KNUCKLE\0"
  "SIGNWRITING HAND-FIST INDEX RING LITTLE\0"
  "SIGNWRITING HAND-FIST INDEX THUMB ANGLED IN MIDDLE UP\0"
  "SIGNWRITING HAND-FIST INDEX THUMB ANGLED OUT MIDDLE UP\0"
  "SIGNWRITING HAND-FIST INDEX THUMB CIRCLE\0"
  "SIGNWRITING HAND-FIST INDEX THUMB CIRCLED MIDDLE UP\0"
  "SIGNWRITING HAND-FIST INDEX THUMB CUPPED MIDDLE UP\0"
  "SIGNWRITING HAND-FIST INDEX THUMB CURLICUE\0"
  "SIGNWRITING HAND-FIST INDEX THUMB CURVE THUMB INSIDE\0"
  "SIGNWRITING HAND-FIST INDEX THUMB CURVE THUMB UNDER\0"
  "SIGNWRITING HAND-FIST INDEX THUMB FORWARD INDEX BENT\0"
  "SIGNWRITING HAND-FIST INDEX THUMB FORWARD INDEX STRAIGHT\0"
  "SIGNWRITING HAND-FIST INDEX THUMB HOOK\0"
  "SIGNWRITING HAND-FIST INDEX THUMB HOOKED MIDDLE HINGED\0"
  "SIGNWRITING HAND-FIST INDEX THUMB HOOKED MIDDLE UP\0"
  "SIGNWRITING HAND-FIST INDEX THUMB SIDE\0"
  "SIGNWRITING HAND-FIST INDEX THUMB SIDE BOTH BENT\0"
  "SIGNWRITING HAND-FIST INDEX THUMB SIDE INDEX BENT\0"
  "SIGNWRITING HAND-FIST INDEX THUMB SIDE INDEX HINGE\0"
  "SIGNWRITING HAND-FIST INDEX THUMB SIDE THUMB BENT\0"
  "SIGNWRITING HAND-FIST INDEX THUMB SIDE THUMB CONJOINED\0"
  "SIGNWRITING HAND-FIST INDEX THUMB SIDE THUMB DIAGONAL\0"
  "SIGNWRITING HAND-FIST INDEX UP MIDDLE HINGED\0"
  "SIGNWRITING HAND-FIST INDEX UP MIDDLE HINGED THUMB CONJOINED\0"
  "SIGNWRITING HAND-FIST INDEX UP MIDDLE HINGED THUMB SIDE\0"
  "SIGNWRITING HAND-FIST LITTLE BENT\0"
  "SIGNWRITING HAND-FIST LITTLE DOWN\0"
  "SIGNWRITING HAND-FIST LITTLE DOWN OTHERS CIRCLED\0"
  "SIGNWRITING HAND-FIST LITTLE DOWN RIPPLE CURVED\0"
  "SIGNWRITING HAND-FIST LITTLE DOWN RIPPLE STRAIGHT\0"
  "SIGNWRITING HAND-FIST LITTLE INDEX\0"
  "SIGNWRITING HAND-FIST LITTLE INDEX THUMB\0"
  "SIGNWRITING HAND-FIST LITTLE RAISED KNUCKLE\0"
  "SIGNWRITING HAND-FIST LITTLE THUMB\0"
  "SIGNWRITING HAND-FIST LITTLE TOUCHES THUMB\0"
  "SIGNWRITING HAND-FIST LITTLE UP\0"
  "SIGNWRITING HAND-FIST MIDDLE BENT OVER INDEX\0"
  "SIGNWRITING HAND-FIST MIDDLE DOWN\0"
  "SIGNWRITING HAND-FIST MIDDLE LITTLE\0"
  "SIGNWRITING HAND-FIST MIDDLE RAISED KNUCKLE\0"
  "SIGNWRITING HAND-FIST MIDDLE RING LITTLE\0"
  "SIGNWRITING HAND-FIST MIDDLE THUMB ANGLED INDEX UP\0"
  "SIGNWRITING HAND-FIST MIDDLE THUMB ANGLED OUT INDEX CROSSED\0"
  "SIGNWRITING HAND-FIST MIDDLE THUMB ANGLED OUT INDEX UP\0"
  "SIGNWRITING HAND-FIST MIDDLE THUMB CIRCLED INDEX HINGED\0"
  "SIGNWRITING HAND-FIST MIDDLE THUMB CIRCLED INDEX UP\0"
  "SIGNWRITING HAND-FIST MIDDLE THUMB CUPPED INDEX UP\0"
  "SIGNWRITING HAND-FIST MIDDLE THUMB HOOKED INDEX UP\0"
  "SIGNWRITING HAND-FIST MIDDLE THUMB LITTLE\0"
  "SIGNWRITING HAND-FIST MIDDLE UP\0"
  "SIGNWRITING HAND-FIST MIDDLE UP THUMB SIDE\0"
  "SIGNWRITING HAND-FIST RING DOWN\0"
  "SIGNWRITING HAND-FIST RING INDEX\0"
  "SIGNWRITING HAND-FIST RING LITTLE\0"
  "SIGNWRITING HAND-FIST RING MIDDLE\0"
  "SIGNWRITING HAND-FIST RING MIDDLE CONJOINED\0"
  "SIGNWRITING HAND-FIST RING MIDDLE RAISED KNUCKLES\0"
  "SIGNWRITING HAND-FIST RING RAISED KNUCKLE\0"
  "SIGNWRITING HAND-FIST RING THUMB\0"
  "SIGNWRITING HAND-FIST RING UP\0"
  "SIGNWRITING HAND-FIST THUMB\0"
  "SIGNWRITING HAND-FIST THUMB BETWEEN INDEX MIDDLE\0"
  "SIGNWRITING HAND-FIST THUMB BETWEEN INDEX MIDDLE STRAIGHT\0"
  "SIGNWRITING HAND-FIST THUMB BETWEEN MIDDLE RING\0"
  "SIGNWRITING HAND-FIST THUMB BETWEEN RING LITTLE\0"
  "SIGNWRITING HAND-FIST THUMB FORWARD\0"
  "SIGNWRITING HAND-FIST THUMB HEEL\0"
  "SIGNWRITING HAND-FIST THUMB OVER FOUR RAISED KNUCKLES\0"
  "SIGNWRITING HAND-FIST THUMB OVER TWO FINGERS\0"
  "SIGNWRITING HAND-FIST THUMB SIDE BENT\0"
  "SIGNWRITING HAND-FIST THUMB SIDE CONJOINED\0"
  "SIGNWRITING HAND-FIST THUMB SIDE DIAGONAL\0"
  "SIGNWRITING HAND-FIST THUMB UNDER FOUR FINGERS\0"
  "SIGNWRITING HAND-FIST THUMB UNDER INDEX BENT\0"
  "SIGNWRITING HAND-FIST THUMB UNDER LITTLE UP\0"
  "SIGNWRITING HAND-FIST THUMB UNDER THREE FINGERS\0"
  "SIGNWRITING HAND-FIST THUMB UNDER TWO FINGERS\0"
  "SIGNWRITING HAND-FLAT\0"
  "SIGNWRITING HAND-FLAT BETWEEN PALM FACINGS\0"
  "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD\0"
  "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD BENT\0"
  "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD FOUR BENT\0"
  "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD HINGED\0"
  "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD HINGED NO THUMB\0"
  "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD HINGED THUMB SIDE\0"
  "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD THUMB FORWARD\0"
  "SIGNWRITING HAND-FLAT FOUR FINGERS\0"
  "SIGNWRITING HAND-FLAT FOUR FINGERS BENT\0"
  "SIGNWRITING HAND-FLAT FOUR FINGERS CONJOINED\0"
  "SIGNWRITING HAND-FLAT FOUR FINGERS CONJOINED SPLIT\0"
  "SIGNWRITING HAND-FLAT FOUR FINGERS HINGED\0"
  "SIGNWRITING HAND-FLAT HEEL\0"
  "SIGNWRITING HAND-FLAT HEEL FIVE FINGERS SPREAD\0"
  "SIGNWRITING HAND-FLAT HEEL FIVE FINGERS SPREAD BENT\0"
  "SIGNWRITING HAND-FLAT HEEL FIVE FINGERS SPREAD FOUR BENT\0"
  "SIGNWRITING HAND-FLAT HEEL THUMB SIDE\0"
  "SIGNWRITING HAND-FLAT SPLIT CENTRE\0"
  "SIGNWRITING HAND-FLAT SPLIT CENTRE THUMB SIDE\0"
  "SIGNWRITING HAND-FLAT SPLIT CENTRE THUMB SIDE BENT\0"
  "SIGNWRITING HAND-FLAT SPLIT INDEX THUMB SIDE\0"
  "SIGNWRITING HAND-FLAT SPLIT LITTLE\0"
  "SIGNWRITING HAND-FLAT THUMB BENT\0"
  "SIGNWRITING HAND-FLAT THUMB FORWARD\0"
  "SIGNWRITING HAND-FLAT THUMB SIDE\0"
  "SIGNWRITING HAND-HINGE\0"
  "SIGNWRITING HAND-HINGE FIVE FINGERS SPREAD OPEN\0"
  "SIGNWRITING HAND-HINGE FOUR FINGERS CONJOINED\0"
  "SIGNWRITING HAND-HINGE INDEX\0"
  "SIGNWRITING HAND-HINGE INDEX HINGED\0"
  "SIGNWRITING HAND-HINGE INDEX MIDDLE LITTLE\0"
  "SIGNWRITING HAND-HINGE INDEX MIDDLE RING\0"
  "SIGNWRITING HAND-HINGE INDEX MIDDLE RING CONJOINED\0"
  "SIGNWRITING HAND-HINGE INDEX RING LITTLE\0"
  "SIGNWRITING HAND-HINGE INDEX THUMB\0"
  "SIGNWRITING HAND-HINGE INDEX THUMB LARGE\0"
  "SIGNWRITING HAND-HINGE INDEX THUMB OPEN\0"
  "SIGNWRITING HAND-HINGE INDEX THUMB SIDE\0"
  "SIGNWRITING HAND-HINGE INDEX THUMB SMALL\0"
  "SIGNWRITING HAND-HINGE LITTLE\0"
  "SIGNWRITING HAND-HINGE LITTLE INDEX\0"
  "SIGNWRITING HAND-HINGE LITTLE INDEX THUMB\0"
  "SIGNWRITING HAND-HINGE LITTLE THUMB\0"
  "SIGNWRITING HAND-HINGE MIDDLE\0"
  "SIGNWRITING HAND-HINGE MIDDLE RING LITTLE\0"
  "SIGNWRITING HAND-HINGE NO THUMB\0"
  "SIGNWRITING HAND-HINGE OPEN\0"
  "SIGNWRITING HAND-HINGE OPEN NO THUMB\0"
  "SIGNWRITING HAND-HINGE OPEN THUMB FORWARD\0"
  "SIGNWRITING HAND-HINGE OPEN THUMB SIDE\0"
  "SIGNWRITING HAND-HINGE RING\0"
  "SIGNWRITING HAND-HINGE RING DOWN INDEX THUMB HOOK MIDDLE\0"
  "SIGNWRITING HAND-HINGE SMALL\0"
  "SIGNWRITING HAND-HINGE THUMB BETWEEN MIDDLE RING\0"
  "SIGNWRITING HAND-HINGE THUMB SIDE\0"
  "SIGNWRITING HAND-HINGE THUMB SIDE TOUCHING INDEX\0"
  "SIGNWRITING HAND-HOOK\0"
  "SIGNWRITING HAND-HOOK CURLICUE\0"
  "SIGNWRITING HAND-HOOK INDEX RING LITTLE IN\0"
  "SIGNWRITING HAND-HOOK INDEX RING LITTLE OUT\0"
  "SIGNWRITING HAND-HOOK INDEX RING LITTLE UNDER\0"
  "SIGNWRITING HAND-HOOK MIDDLE RING LITTLE CONJOINED\0"
  "SIGNWRITING HAND-HOOK MIDDLE RING LITTLE CONJOINED IN\0"
  "SIGNWRITING HAND-HOOK MIDDLE RING LITTLE CONJOINED OUT\0"
  "SIGNWRITING HAND-HOOK MIDDLE THUMB\0"
  "SIGNWRITING HAND-HOOK RING THUMB\0"
  "SIGNWRITING HAND-OVAL\0"
  "SIGNWRITING HAND-OVAL FIVE FINGERS SPREAD\0"
  "SIGNWRITING HAND-OVAL INDEX\0"
  "SIGNWRITING HAND-OVAL LITTLE UP\0"
  "SIGNWRITING HAND-OVAL NO THUMB\0"
  "SIGNWRITING HAND-OVAL RING LITTLE\0"
  "SIGNWRITING HAND-OVAL THUMB FORWARD\0"
  "SIGNWRITING HAND-OVAL THUMB SIDE\0"
  "SIGNWRITING HEAD\0"
  "SIGNWRITING HEAD MOVEMENT CIRCLE\0"
  "SIGNWRITING HEAD MOVEMENT-FLOORPLANE CURVE\0"
  "SIGNWRITING HEAD MOVEMENT-FLOORPLANE STRAIGHT\0"
  "SIGNWRITING HEAD MOVEMENT-WALLPLANE CURVE\0"
  "SIGNWRITING HEAD MOVEMENT-WALLPLANE STRAIGHT\0"
  "SIGNWRITING HEAD MOVEMENT-WALLPLANE TILT\0"
  "SIGNWRITING HEAD RIM\0"
  "SIGNWRITING LIMB COMBINATION\0"
  "SIGNWRITING LIMB LENGTH-1\0"
  "SIGNWRITING LIMB LENGTH-2\0"
  "SIGNWRITING LIMB LENGTH-3\0"
  "SIGNWRITING LIMB LENGTH-4\0"
  "SIGNWRITING LIMB LENGTH-5\0"
  "SIGNWRITING LIMB LENGTH-6\0"
  "SIGNWRITING LIMB LENGTH-7\0"
  "SIGNWRITING LIP LOWER OVER UPPER\0"
  "SIGNWRITING LIP UPPER OVER LOWER\0"
  "SIGNWRITING LIPS PRESSED TOGETHER\0"
  "SIGNWRITING LOCATION DEPTH\0"
  "SIGNWRITING LOCATION HEAD NECK\0"
  "SIGNWRITING LOCATION HEIGHT\0"
  "SIGNWRITING LOCATION LIMBS DIGITS\0"
  "SIGNWRITING LOCATION TORSO\0"
  "SIGNWRITING LOCATION WIDTH\0"
  "SIGNWRITING LOCATION-FLOORPLANE SPACE\0"
  "SIGNWRITING LOCATION-WALLPLANE SPACE\0"
  "SIGNWRITING MOUTH CLOSED CONTACT\0"
  "SIGNWRITING MOUTH CLOSED FORWARD\0"
  "SIGNWRITING MOUTH CLOSED NEUTRAL\0"
  "SIGNWRITING MOUTH CORNERS\0"
  "SIGNWRITING MOUTH FROWN\0"
  "SIGNWRITING MOUTH FROWN OPEN\0"
  "SIGNWRITING MOUTH FROWN WRINKLED\0"
  "SIGNWRITING MOUTH KISS\0"
  "SIGNWRITING MOUTH KISS FORWARD\0"
  "SIGNWRITING MOUTH KISS WRINKLED\0"
  "SIGNWRITING MOUTH OPEN CIRCLE\0"
  "SIGNWRITING MOUTH OPEN FORWARD\0"
  "SIGNWRITING MOUTH OPEN OVAL\0"
  "SIGNWRITING MOUTH OPEN OVAL WRINKLED\0"
  "SIGNWRITING MOUTH OPEN OVAL YAWN\0"
  "SIGNWRITING MOUTH OPEN RECTANGLE\0"
  "SIGNWRITING MOUTH OPEN RECTANGLE WRINKLED\0"
  "SIGNWRITING MOUTH OPEN RECTANGLE YAWN\0"
  "SIGNWRITING MOUTH OPEN WRINKLED\0"
  "SIGNWRITING MOUTH SMILE\0"
  "SIGNWRITING MOUTH SMILE OPEN\0"
  "SIGNWRITING MOUTH SMILE WRINKLED\0"
  "SIGNWRITING MOUTH TENSE\0"
  "SIGNWRITING MOUTH TENSE FORWARD\0"
  "SIGNWRITING MOUTH TENSE SUCKED\0"
  "SIGNWRITING MOUTH WRINKLES DOUBLE\0"
  "SIGNWRITING MOUTH WRINKLES SINGLE\0"
  "SIGNWRITING MOVEMENT-DIAGONAL AWAY LARGE\0"
  "SIGNWRITING MOVEMENT-DIAGONAL AWAY LARGEST\0"
  "SIGNWRITING MOVEMENT-DIAGONAL AWAY MEDIUM\0"
  "SIGNWRITING MOVEMENT-DIAGONAL AWAY SMALL\0"
  "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY LARGE\0"
  "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY LARGEST\0"
  "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY MEDIUM\0"
  "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY SMALL\0"
  "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS LARGE\0"
  "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS LARGEST\0"
  "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS MEDIUM\0"
  "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS SMALL\0"
  "SIGNWRITING MOVEMENT-DIAGONAL TOWARDS LARGE\0"
  "SIGNWRITING MOVEMENT-DIAGONAL TOWARDS LARGEST\0"
  "SIGNWRITING MOVEMENT-DIAGONAL TOWARDS MEDIUM\0"
  "SIGNWRITING MOVEMENT-DIAGONAL TOWARDS SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL LARGE DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL LARGE SINGLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL MEDIUM DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL MEDIUM SINGLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL SMALL DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL SMALL SINGLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE BEND\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE BOX LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE BOX MEDIUM\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE BOX SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CHECK\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CORNER LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CORNER MEDIUM\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CORNER SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CROSS\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CURVE COMBINED\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING CEILING LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING CEILING SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING FLOOR LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING FLOOR SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CURVE LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CURVE LARGEST\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CURVE MEDIUM\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE CURVE SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE ALTERNATING\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE ALTERNATING WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE STRAIGHT\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE FINGER CIRCLES HITTING WALL DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE FINGER CIRCLES HITTING WALL SINGLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE FINGER CONTACT\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING LARGE DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING LARGE TRIPLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING SMALL DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING SMALL TRIPLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR LARGE DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR SMALL DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR TRIPLE LARGE TRIPLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR TRIPLE SMALL TRIPLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE HUMP SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE JAW\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING LARGE DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING LARGE SINGLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING SMALL DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING SMALL SINGLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR LARGE DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR LARGE SINGLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR SMALL DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR SMALL SINGLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE LOOP SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE PEAKS LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE PEAKS MEDIUM\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE PEAKS SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE SHAKING PARALLEL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT LARGEST\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT MEDIUM\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE ALTERNATING MOVEMENT\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE ALTERNATING WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE STRAIGHT MOVEMENT\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING CEILING LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING CEILING SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING FLOOR LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING FLOOR SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE WAVE LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE WAVE SMALL\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE WAVE SNAKE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE WRIST CIRCLE HITTING WALL DOUBLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE WRIST CIRCLE HITTING WALL SINGLE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE ZIGZAG LARGE\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE ZIGZAG MEDIUM\0"
  "SIGNWRITING MOVEMENT-FLOORPLANE ZIGZAG SMALL\0"
  "SIGNWRITING MOVEMENT-HINGE DOWN SEQUENTIAL\0"
  "SIGNWRITING MOVEMENT-HINGE SIDE TO SIDE SCISSORS\0"
  "SIGNWRITING MOVEMENT-HINGE UP DOWN ALTERNATING LARGE\0"
  "SIGNWRITING MOVEMENT-HINGE UP DOWN ALTERNATING SMALL\0"
  "SIGNWRITING MOVEMENT-HINGE UP DOWN LARGE\0"
  "SIGNWRITING MOVEMENT-HINGE UP DOWN SMALL\0"
  "SIGNWRITING MOVEMENT-HINGE UP SEQUENTIAL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE MEDIUM DOUBLE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE MEDIUM SINGLE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE SMALL DOUBLE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE SMALL SINGLE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE BEND LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE BEND MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE BEND SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE BOX LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE BOX MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE BOX SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CHECK LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CHECK MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CHECK SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CORNER LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CORNER MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CORNER ROTATION\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CORNER SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CROSS\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE LARGEST\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE HITTING CHEST\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE HITTING FRONT WALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER LARGEST\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE THEN STRAIGHT\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE THREE-QUARTER CIRCLE MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVE THREE-QUARTER CIRCLE SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVED CROSS MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE CURVED CROSS SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE DOUBLE ALTERNATING\0"
  "SIGNWRITING MOVEMENT-WALLPLANE DOUBLE ALTERNATING WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-WALLPLANE DOUBLE STRAIGHT\0"
  "SIGNWRITING MOVEMENT-WALLPLANE DOUBLE WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-WALLPLANE FINGER CIRCLES DOUBLE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE FINGER CIRCLES SINGLE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE FINGER CONTACT\0"
  "SIGNWRITING MOVEMENT-WALLPLANE HUMP HITTING CHEST\0"
  "SIGNWRITING MOVEMENT-WALLPLANE HUMP HITTING FRONT WALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE HUMP LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE HUMP MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE HUMP SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE JAW\0"
  "SIGNWRITING MOVEMENT-WALLPLANE LOOP HITTING CHEST\0"
  "SIGNWRITING MOVEMENT-WALLPLANE LOOP HITTING FRONT WALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE LOOP LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE LOOP MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE LOOP SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE LOOP SMALL DOUBLE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE PEAKS LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE PEAKS MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE PEAKS SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE SHAKING\0"
  "SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT LARGEST\0"
  "SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE SINGLE WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-WALLPLANE TRIPLE ALTERNATING\0"
  "SIGNWRITING MOVEMENT-WALLPLANE TRIPLE ALTERNATING WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-WALLPLANE TRIPLE STRAIGHT MOVEMENT\0"
  "SIGNWRITING MOVEMENT-WALLPLANE TRIPLE WRIST FLEX\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE DOUBLE LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE DOUBLE MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE DOUBLE SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE TRIPLE LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE TRIPLE MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE TRIPLE SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE DIAGONAL PATH LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE DIAGONAL PATH MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE DIAGONAL PATH SMALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE HITTING CHEST\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WAVE HITTING FRONT WALL\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WRIST CIRCLE FRONT DOUBLE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE WRIST CIRCLE FRONT SINGLE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE ZIGZAG LARGE\0"
  "SIGNWRITING MOVEMENT-WALLPLANE ZIGZAG MEDIUM\0"
  "SIGNWRITING MOVEMENT-WALLPLANE ZIGZAG SMALL\0"
  "SIGNWRITING NECK\0"
  "SIGNWRITING NOSE CONTACT\0"
  "SIGNWRITING NOSE NEUTRAL\0"
  "SIGNWRITING NOSE WIGGLES\0"
  "SIGNWRITING NOSE WRINKLES\0"
  "SIGNWRITING PARENTHESIS\0"
  "SIGNWRITING ROTATION MODIFIER-10\0"
  "SIGNWRITING ROTATION MODIFIER-11\0"
  "SIGNWRITING ROTATION MODIFIER-12\0"
  "SIGNWRITING ROTATION MODIFIER-13\0"
  "SIGNWRITING ROTATION MODIFIER-14\0"
  "SIGNWRITING ROTATION MODIFIER-15\0"
  "SIGNWRITING ROTATION MODIFIER-16\0"
  "SIGNWRITING ROTATION MODIFIER-2\0"
  "SIGNWRITING ROTATION MODIFIER-3\0"
  "SIGNWRITING ROTATION MODIFIER-4\0"
  "SIGNWRITING ROTATION MODIFIER-5\0"
  "SIGNWRITING ROTATION MODIFIER-6\0"
  "SIGNWRITING ROTATION MODIFIER-7\0"
  "SIGNWRITING ROTATION MODIFIER-8\0"
  "SIGNWRITING ROTATION MODIFIER-9\0"
  "SIGNWRITING ROTATION-FLOORPLANE ALTERNATING\0"
  "SIGNWRITING ROTATION-FLOORPLANE ALTERNATING HITTING CEILING\0"
  "SIGNWRITING ROTATION-FLOORPLANE ALTERNATING HITTING FLOOR\0"
  "SIGNWRITING ROTATION-FLOORPLANE DOUBLE\0"
  "SIGNWRITING ROTATION-FLOORPLANE DOUBLE HITTING CEILING\0"
  "SIGNWRITING ROTATION-FLOORPLANE DOUBLE HITTING FLOOR\0"
  "SIGNWRITING ROTATION-FLOORPLANE SINGLE\0"
  "SIGNWRITING ROTATION-FLOORPLANE SINGLE HITTING CEILING\0"
  "SIGNWRITING ROTATION-FLOORPLANE SINGLE HITTING FLOOR\0"
  "SIGNWRITING ROTATION-WALLPLANE ALTERNATE\0"
  "SIGNWRITING ROTATION-WALLPLANE ALTERNATING HITTING CHEST\0"
  "SIGNWRITING ROTATION-WALLPLANE ALTERNATING HITTING FRONT WALL\0"
  "SIGNWRITING ROTATION-WALLPLANE DOUBLE\0"
  "SIGNWRITING ROTATION-WALLPLANE DOUBLE HITTING CHEST\0"
  "SIGNWRITING ROTATION-WALLPLANE DOUBLE HITTING FRONT WALL\0"
  "SIGNWRITING ROTATION-WALLPLANE SINGLE\0"
  "SIGNWRITING ROTATION-WALLPLANE SINGLE HITTING CHEST\0"
  "SIGNWRITING ROTATION-WALLPLANE SINGLE HITTING FRONT WALL\0"
  "SIGNWRITING RUB BETWEEN\0"
  "SIGNWRITING RUB MULTIPLE\0"
  "SIGNWRITING RUB SINGLE\0"
  "SIGNWRITING SEMICOLON\0"
  "SIGNWRITING SHOULDER HIP POSITIONS\0"
  "SIGNWRITING SHOULDER HIP SPINE\0"
  "SIGNWRITING SHOULDER TILTING FROM WAIST\0"
  "SIGNWRITING SQUEEZE FLICK ALTERNATING\0"
  "SIGNWRITING SQUEEZE LARGE MULTIPLE\0"
  "SIGNWRITING SQUEEZE LARGE SINGLE\0"
  "SIGNWRITING SQUEEZE SEQUENTIAL\0"
  "SIGNWRITING SQUEEZE SMALL MULTIPLE\0"
  "SIGNWRITING SQUEEZE SMALL SINGLE\0"
  "SIGNWRITING STRIKE BETWEEN\0"
  "SIGNWRITING STRIKE MULTIPLE\0"
  "SIGNWRITING STRIKE SINGLE\0"
  "SIGNWRITING SURFACE BETWEEN\0"
  "SIGNWRITING SURFACE SYMBOLS\0"
  "SIGNWRITING TEETH\0"
  "SIGNWRITING TEETH BITE LIPS\0"
  "SIGNWRITING TEETH MOVEMENT\0"
  "SIGNWRITING TEETH ON LIPS\0"
  "SIGNWRITING TEETH ON LIPS MOVEMENT\0"
  "SIGNWRITING TEETH ON TONGUE\0"
  "SIGNWRITING TEETH ON TONGUE MOVEMENT\0"
  "SIGNWRITING TENSE CHEEKS HIGH\0"
  "SIGNWRITING TENSE CHEEKS LOW\0"
  "SIGNWRITING TENSE CHEEKS MIDDLE\0"
  "SIGNWRITING TONGUE CENTRE INSIDE MOUTH\0"
  "SIGNWRITING TONGUE CENTRE STICKING OUT\0"
  "SIGNWRITING TONGUE INSIDE MOUTH RELAXED\0"
  "SIGNWRITING TONGUE LICKING LIPS\0"
  "SIGNWRITING TONGUE MOVES AGAINST CHEEK\0"
  "SIGNWRITING TONGUE STICKING OUT FAR\0"
  "SIGNWRITING TONGUE TIP BETWEEN LIPS\0"
  "SIGNWRITING TONGUE TIP TOUCHING INSIDE MOUTH\0"
  "SIGNWRITING TORSO-FLOORPLANE TWISTING\0"
  "SIGNWRITING TORSO-WALLPLANE CURVED BEND\0"
  "SIGNWRITING TORSO-WALLPLANE STRAIGHT STRETCH\0"
  "SIGNWRITING TOUCH BETWEEN\0"
  "SIGNWRITING TOUCH MULTIPLE\0"
  "SIGNWRITING TOUCH SINGLE\0"
  "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-FLOORPLANE ALTERNATING\0"
  "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-FLOORPLANE DOUBLE\0"
  "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-FLOORPLANE SINGLE\0"
  "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-WALLPLANE ALTERNATING\0"
  "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-WALLPLANE DOUBLE\0"
  "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-WALLPLANE SINGLE\0"
  "SIGNWRITING TRAVEL-FLOORPLANE SHAKING\0"
  "SIGNWRITING TRAVEL-WALLPLANE ARM SPIRAL DOUBLE\0"
  "SIGNWRITING TRAVEL-WALLPLANE ARM SPIRAL SINGLE\0"
  "SIGNWRITING TRAVEL-WALLPLANE ARM SPIRAL TRIPLE\0"
  "SIGNWRITING TRAVEL-WALLPLANE ROTATION-FLOORPLANE ALTERNATING\0"
  "SIGNWRITING TRAVEL-WALLPLANE ROTATION-FLOORPLANE DOUBLE\0"
  "SIGNWRITING TRAVEL-WALLPLANE ROTATION-FLOORPLANE SINGLE\0"
  "SIGNWRITING TRAVEL-WALLPLANE ROTATION-WALLPLANE ALTERNATING\0"
  "SIGNWRITING TRAVEL-WALLPLANE ROTATION-WALLPLANE DOUBLE\0"
  "SIGNWRITING TRAVEL-WALLPLANE ROTATION-WALLPLANE SINGLE\0"
  "SIGNWRITING TRAVEL-WALLPLANE SHAKING\0"
  "SIGNWRITING UPPER BODY TILTING FROM HIP JOINTS\0"
  "SIGNWRITING WALLPLANE SHOULDER HIP MOVE\0"
  "SILHOUETTE OF JAPAN\0"
  "SIMILAR ABOVE GREATER-THAN ABOVE EQUALS SIGN\0"
  "SIMILAR ABOVE LESS-THAN ABOVE EQUALS SIGN\0"
  "SIMILAR MINUS SIMILAR\0"
  "SIMILAR OR GREATER-THAN\0"
  "SIMILAR OR LESS-THAN\0"
  "SINE WAVE\0"
  "SINGLE HIGH-REVERSED-9 QUOTATION MARK\0"
  "SINGLE LEFT-POINTING ANGLE QUOTATION MARK\0"
  "SINGLE LOW-9 QUOTATION MARK\0"
  "SINGLE RIGHT-POINTING ANGLE QUOTATION MARK\0"
  "SINHALA ARCHAIC DIGIT EIGHT\0"
  "SINHALA ARCHAIC DIGIT FIVE\0"
  "SINHALA ARCHAIC DIGIT FOUR\0"
  "SINHALA ARCHAIC DIGIT NINE\0"
  "SINHALA ARCHAIC DIGIT ONE\0"
  "SINHALA ARCHAIC DIGIT SEVEN\0"
  "SINHALA ARCHAIC DIGIT SIX\0"
  "SINHALA ARCHAIC DIGIT THREE\0"
  "SINHALA ARCHAIC DIGIT TWO\0"
  "SINHALA ARCHAIC NUMBER EIGHTY\0"
  "SINHALA ARCHAIC NUMBER FIFTY\0"
  "SINHALA ARCHAIC NUMBER FORTY\0"
  "SINHALA ARCHAIC NUMBER NINETY\0"
  "SINHALA ARCHAIC NUMBER ONE HUNDRED\0"
  "SINHALA ARCHAIC NUMBER ONE THOUSAND\0"
  "SINHALA ARCHAIC NUMBER SEVENTY\0"
  "SINHALA ARCHAIC NUMBER SIXTY\0"
  "SINHALA ARCHAIC NUMBER TEN\0"
  "SINHALA ARCHAIC NUMBER THIRTY\0"
  "SINHALA ARCHAIC NUMBER TWENTY\0"
  "SINHALA LETTER AAYANNA\0"
  "SINHALA LETTER AEEYANNA\0"
  "SINHALA LETTER AEYANNA\0"
  "SINHALA LETTER AIYANNA\0"
  "SINHALA LETTER ALPAPRAANA BAYANNA\0"
  "SINHALA LETTER ALPAPRAANA CAYANNA\0"
  "SINHALA LETTER ALPAPRAANA DAYANNA\0"
  "SINHALA LETTER ALPAPRAANA DDAYANNA\0"
  "SINHALA LETTER ALPAPRAANA GAYANNA\0"
  "SINHALA LETTER ALPAPRAANA JAYANNA\0"
  "SINHALA LETTER ALPAPRAANA KAYANNA\0"
  "SINHALA LETTER ALPAPRAANA PAYANNA\0"
  "SINHALA LETTER ALPAPRAANA TAYANNA\0"
  "SINHALA LETTER ALPAPRAANA TTAYANNA\0"
  "SINHALA LETTER AMBA BAYANNA\0"
  "SINHALA LETTER AUYANNA\0"
  "SINHALA LETTER AYANNA\0"
  "SINHALA LETTER DANTAJA LAYANNA\0"
  "SINHALA LETTER DANTAJA NAYANNA\0"
  "SINHALA LETTER DANTAJA SAYANNA\0"
  "SINHALA LETTER EEYANNA\0"
  "SINHALA LETTER EYANNA\0"
  "SINHALA LETTER FAYANNA\0"
  "SINHALA LETTER HAYANNA\0"
  "SINHALA LETTER IIYANNA\0"
  "SINHALA LETTER ILUUYANNA\0"
  "SINHALA LETTER ILUYANNA\0"
  "SINHALA LETTER IRUUYANNA\0"
  "SINHALA LETTER IRUYANNA\0"
  "SINHALA LETTER IYANNA\0"
  "SINHALA LETTER KANTAJA NAASIKYAYA\0"
  "SINHALA LETTER MAHAAPRAANA BAYANNA\0"
  "SINHALA LETTER MAHAAPRAANA CAYANNA\0"
  "SINHALA LETTER MAHAAPRAANA DAYANNA\0"
  "SINHALA LETTER MAHAAPRAANA DDAYANNA\0"
  "SINHALA LETTER MAHAAPRAANA GAYANNA\0"
  "SINHALA LETTER MAHAAPRAANA JAYANNA\0"
  "SINHALA LETTER MAHAAPRAANA KAYANNA\0"
  "SINHALA LETTER MAHAAPRAANA PAYANNA\0"
  "SINHALA LETTER MAHAAPRAANA TAYANNA\0"
  "SINHALA LETTER MAHAAPRAANA TTAYANNA\0"
  "SINHALA LETTER MAYANNA\0"
  "SINHALA LETTER MUURDHAJA LAYANNA\0"
  "SINHALA LETTER MUURDHAJA NAYANNA\0"
  "SINHALA LETTER MUURDHAJA SAYANNA\0"
  "SINHALA LETTER OOYANNA\0"
  "SINHALA LETTER OYANNA\0"
  "SINHALA LETTER RAYANNA\0"
  "SINHALA LETTER SANYAKA DAYANNA\0"
  "SINHALA LETTER SANYAKA DDAYANNA\0"
  "SINHALA LETTER SANYAKA GAYANNA\0"
  "SINHALA LETTER SANYAKA JAYANNA\0"
  "SINHALA LETTER TAALUJA NAASIKYAYA\0"
  "SINHALA LETTER TAALUJA SANYOOGA NAAKSIKYAYA\0"
  "SINHALA LETTER TAALUJA SAYANNA\0"
  "SINHALA LETTER UUYANNA\0"
  "SINHALA LETTER UYANNA\0"
  "SINHALA LETTER VAYANNA\0"
  "SINHALA LETTER YAYANNA\0"
  "SINHALA LITH DIGIT EIGHT\0"
  "SINHALA LITH DIGIT FIVE\0"
  "SINHALA LITH DIGIT FOUR\0"
  "SINHALA LITH DIGIT NINE\0"
  "SINHALA LITH DIGIT ONE\0"
  "SINHALA LITH DIGIT SEVEN\0"
  "SINHALA LITH DIGIT SIX\0"
  "SINHALA LITH DIGIT THREE\0"
  "SINHALA LITH DIGIT TWO\0"
  "SINHALA LITH DIGIT ZERO\0"
  "SINHALA PUNCTUATION KUNDDALIYA\0"
  "SINHALA SIGN AL-LAKUNA\0"
  "SINHALA SIGN ANUSVARAYA\0"
  "SINHALA SIGN CANDRABINDU\0"
  "SINHALA SIGN VISARGAYA\0"
  "SINHALA VOWEL SIGN AELA-PILLA\0"
  "SINHALA VOWEL SIGN DIGA AEDA-PILLA\0"
  "SINHALA VOWEL SIGN DIGA GAETTA-PILLA\0"
  "SINHALA VOWEL SIGN DIGA GAYANUKITTA\0"
  "SINHALA VOWEL SIGN DIGA IS-PILLA\0"
  "SINHALA VOWEL SIGN DIGA KOMBUVA\0"
  "SINHALA VOWEL SIGN DIGA PAA-PILLA\0"
  "SINHALA VOWEL SIGN GAETTA-PILLA\0"
  "SINHALA VOWEL SIGN GAYANUKITTA\0"
  "SINHALA VOWEL SIGN KETTI AEDA-PILLA\0"
  "SINHALA VOWEL SIGN KETTI IS-PILLA\0"
  "SINHALA VOWEL SIGN KETTI PAA-PILLA\0"
  "SINHALA VOWEL SIGN KOMBU DEKA\0"
  "SINHALA VOWEL SIGN KOMBUVA\0"
  "SINHALA VOWEL SIGN KOMBUVA HAA AELA-PILLA\0"
  "SINHALA VOWEL SIGN KOMBUVA HAA DIGA AELA-PILLA\0"
  "SINHALA VOWEL SIGN KOMBUVA HAA GAYANUKITTA\0"
  "SIX PETALLED BLACK AND WHITE FLORETTE\0"
  "SIX POINTED BLACK STAR\0"
  "SIX POINTED PINWHEEL STAR\0"
  "SIX POINTED STAR WITH MIDDLE DOT\0"
  "SIX-PER-EM SPACE\0"
  "SIXTEEN POINTED ASTERISK\0"
  "SKATEBOARD\0"
  "SKI AND SKI BOOT\0"
  "SKIER\0"
  "SKULL\0"
  "SKULL AND CROSSBONES\0"
  "SKUNK\0"
  "SLANTED EQUAL TO OR GREATER-THAN\0"
  "SLANTED EQUAL TO OR GREATER-THAN WITH DOT INSIDE\0"
  "SLANTED EQUAL TO OR LESS-THAN\0"
  "SLANTED EQUAL TO OR LESS-THAN WITH DOT INSIDE\0"
  "SLANTED NORTH ARROW WITH HOOKED HEAD\0"
  "SLANTED NORTH ARROW WITH HORIZONTAL TAIL\0"
  "SLAVONIC ASTERISK\0"
  "SLED\0"
  "SLEEPING ACCOMMODATION\0"
  "SLEEPING FACE\0"
  "SLEEPING SYMBOL\0"
  "SLEEPY FACE\0"
  "SLEUTH OR SPY\0"
  "SLICE OF PIZZA\0"
  "SLIGHTLY FROWNING FACE\0"
  "SLIGHTLY SMILING FACE\0"
  "SLOPE\0"
  "SLOPING LARGE AND\0"
  "SLOPING LARGE OR\0"
  "SLOT MACHINE\0"
  "SLOTH\0"
  "SMALL AIRPLANE\0"
  "SMALL AMPERSAND\0"
  "SMALL ASTERISK\0"
  "SMALL BLUE DIAMOND\0"
  "SMALL COLON\0"
  "SMALL COMMA\0"
  "SMALL COMMERCIAL AT\0"
  "SMALL CONTAINS AS MEMBER\0"
  "SMALL CONTAINS WITH OVERBAR\0"
  "SMALL CONTAINS WITH VERTICAL BAR AT END OF HORIZONTAL STROKE\0"
  "SMALL DOLLAR SIGN\0"
  "SMALL ELEMENT OF\0"
  "SMALL ELEMENT OF WITH OVERBAR\0"
  "SMALL ELEMENT OF WITH VERTICAL BAR AT END OF HORIZONTAL STROKE\0"
  "SMALL EM DASH\0"
  "SMALL EQUALS SIGN\0"
  "SMALL EXCLAMATION MARK\0"
  "SMALL FULL STOP\0"
  "SMALL GREATER-THAN SIGN\0"
  "SMALL HYPHEN-MINUS\0"
  "SMALL IDEOGRAPHIC COMMA\0"
  "SMALL LEFT CURLY BRACKET\0"
  "SMALL LEFT PARENTHESIS\0"
  "SMALL LEFT TORTOISE SHELL BRACKET\0"
  "SMALL LESS-THAN SIGN\0"
  "SMALL NUMBER SIGN\0"
  "SMALL ORANGE DIAMOND\0"
  "SMALL PERCENT SIGN\0"
  "SMALL PLUS SIGN\0"
  "SMALL QUESTION MARK\0"
  "SMALL REVERSE SOLIDUS\0"
  "SMALL RIGHT CURLY BRACKET\0"
  "SMALL RIGHT PARENTHESIS\0"
  "SMALL RIGHT TORTOISE SHELL BRACKET\0"
  "SMALL ROMAN NUMERAL EIGHT\0"
  "SMALL ROMAN NUMERAL ELEVEN\0"
  "SMALL ROMAN NUMERAL FIFTY\0"
  "SMALL ROMAN NUMERAL FIVE\0"
  "SMALL ROMAN NUMERAL FIVE HUNDRED\0"
  "SMALL ROMAN NUMERAL FOUR\0"
  "SMALL ROMAN NUMERAL NINE\0"
  "SMALL ROMAN NUMERAL ONE\0"
  "SMALL ROMAN NUMERAL ONE HUNDRED\0"
  "SMALL ROMAN NUMERAL ONE THOUSAND\0"
  "SMALL ROMAN NUMERAL SEVEN\0"
  "SMALL ROMAN NUMERAL SIX\0"
  "SMALL ROMAN NUMERAL TEN\0"
  "SMALL ROMAN NUMERAL THREE\0"
  "SMALL ROMAN NUMERAL TWELVE\0"
  "SMALL ROMAN NUMERAL TWO\0"
  "SMALL SEMICOLON\0"
  "SMALL TILDE\0"
  "SMALL TWO DOTS OVER ONE DOT PUNCTUATION\0"
  "SMALL UP-POINTING AIRPLANE\0"
  "SMALL VEE WITH UNDERBAR\0"
  "SMALLER THAN\0"
  "SMALLER THAN OR EQUAL TO\0"
  "SMASH PRODUCT\0"
  "SMILE\0"
  "SMILING CAT FACE WITH HEART-SHAPED EYES\0"
  "SMILING CAT FACE WITH OPEN MOUTH\0"
  "SMILING FACE WITH HALO\0"
  "SMILING FACE WITH HEART-SHAPED EYES\0"
  "SMILING FACE WITH HORNS\0"
  "SMILING FACE WITH OPEN MOUTH\0"
  "SMILING FACE WITH OPEN MOUTH AND COLD SWEAT\0"
  "SMILING FACE WITH OPEN MOUTH AND SMILING EYES\0"
  "SMILING FACE WITH OPEN MOUTH AND TIGHTLY-CLOSED EYES\0"
  "SMILING FACE WITH SMILING EYES\0"
  "SMILING FACE WITH SMILING EYES AND HAND COVERING MOUTH\0"
  "SMILING FACE WITH SMILING EYES AND THREE HEARTS\0"
  "SMILING FACE WITH SUNGLASSES\0"
  "SMILING FACE WITH TEAR\0"
  "SMIRKING FACE\0"
  "SMOKING SYMBOL\0"
  "SNAIL\0"
  "SNAKE\0"
  "SNEEZING FACE\0"
  "SNOW CAPPED MOUNTAIN\0"
  "SNOWBOARDER\0"
  "SNOWFLAKE\0"
  "SNOWMAN\0"
  "SNOWMAN WITHOUT SNOW\0"
  "SOCCER BALL\0"
  "SOCKS\0"
  "SOFT HYPHEN\0"
  "SOFT ICE CREAM\0"
  "SOFT SHELL FLOPPY DISK\0"
  "SOFTBALL\0"
  "SOFTWARE-FUNCTION SYMBOL\0"
  "SOGDIAN COMBINING CURVE ABOVE\0"
  "SOGDIAN COMBINING CURVE BELOW\0"
  "SOGDIAN COMBINING DOT ABOVE\0"
  "SOGDIAN COMBINING DOT BELOW\0"
  "SOGDIAN COMBINING HOOK ABOVE\0"
  "SOGDIAN COMBINING HOOK BELOW\0"
  "SOGDIAN COMBINING LONG HOOK BELOW\0"
  "SOGDIAN COMBINING RESH BELOW\0"
  "SOGDIAN COMBINING STROKE BELOW\0"
  "SOGDIAN COMBINING TWO DOTS ABOVE\0"
  "SOGDIAN COMBINING TWO DOTS BELOW\0"
  "SOGDIAN INDEPENDENT SHIN\0"
  "SOGDIAN LETTER ALEPH\0"
  "SOGDIAN LETTER AYIN\0"
  "SOGDIAN LETTER BETH\0"
  "SOGDIAN LETTER FETH\0"
  "SOGDIAN LETTER GIMEL\0"
  "SOGDIAN LETTER HE\0"
  "SOGDIAN LETTER HETH\0"
  "SOGDIAN LETTER KAPH\0"
  "SOGDIAN LETTER LAMEDH\0"
  "SOGDIAN LETTER LESH\0"
  "SOGDIAN LETTER MEM\0"
  "SOGDIAN LETTER NUN\0"
  "SOGDIAN LETTER PE\0"
  "SOGDIAN LETTER RESH-AYIN\0"
  "SOGDIAN LETTER SADHE\0"
  "SOGDIAN LETTER SAMEKH\0"
  "SOGDIAN LETTER SHIN\0"
  "SOGDIAN LETTER TAW\0"
  "SOGDIAN LETTER WAW\0"
  "SOGDIAN LETTER YODH\0"
  "SOGDIAN LETTER ZAYIN\0"
  "SOGDIAN NUMBER ONE\0"
  "SOGDIAN NUMBER ONE HUNDRED\0"
  "SOGDIAN NUMBER TEN\0"
  "SOGDIAN NUMBER TWENTY\0"
  "SOGDIAN PUNCTUATION CIRCLE WITH DOT\0"
  "SOGDIAN PUNCTUATION HALF CIRCLE WITH DOT\0"
  "SOGDIAN PUNCTUATION TWO CIRCLES WITH DOTS\0"
  "SOGDIAN PUNCTUATION TWO VERTICAL BARS\0"
  "SOGDIAN PUNCTUATION TWO VERTICAL BARS WITH DOTS\0"
  "SOLID QUILT SQUARE ORNAMENT\0"
  "SOLID QUILT SQUARE ORNAMENT IN BLACK SQUARE\0"
  "SOLIDUS\0"
  "SOLIDUS WITH OVERBAR\0"
  "SOM SIGN\0"
  "SOON WITH RIGHTWARDS ARROW ABOVE\0"
  "SORA SOMPENG DIGIT EIGHT\0"
  "SORA SOMPENG DIGIT FIVE\0"
  "SORA SOMPENG DIGIT FOUR\0"
  "SORA SOMPENG DIGIT NINE\0"
  "SORA SOMPENG DIGIT ONE\0"
  "SORA SOMPENG DIGIT SEVEN\0"
  "SORA SOMPENG DIGIT SIX\0"
  "SORA SOMPENG DIGIT THREE\0"
  "SORA SOMPENG DIGIT TWO\0"
  "SORA SOMPENG DIGIT ZERO\0"
  "SORA SOMPENG LETTER AH\0"
  "SORA SOMPENG LETTER BAH\0"
  "SORA SOMPENG LETTER CAH\0"
  "SORA SOMPENG LETTER DAH\0"
  "SORA SOMPENG LETTER EEH\0"
  "SORA SOMPENG LETTER EH\0"
  "SORA SOMPENG LETTER GAH\0"
  "SORA SOMPENG LETTER HAH\0"
  "SORA SOMPENG LETTER IH\0"
  "SORA SOMPENG LETTER JAH\0"
  "SORA SOMPENG LETTER KAH\0"
  "SORA SOMPENG LETTER LAH\0"
  "SORA SOMPENG LETTER MAE\0"
  "SORA SOMPENG LETTER MAH\0"
  "SORA SOMPENG LETTER NAH\0"
  "SORA SOMPENG LETTER NGAH\0"
  "SORA SOMPENG LETTER NYAH\0"
  "SORA SOMPENG LETTER OH\0"
  "SORA SOMPENG LETTER PAH\0"
  "SORA SOMPENG LETTER RAH\0"
  "SORA SOMPENG LETTER SAH\0"
  "SORA SOMPENG LETTER TAH\0"
  "SORA SOMPENG LETTER UH\0"
  "SORA SOMPENG LETTER VAH\0"
  "SORA SOMPENG LETTER YAH\0"
  "SOUND RECORDING COPYRIGHT\0"
  "SOUTH EAST ARROW\0"
  "SOUTH EAST ARROW AND SOUTH WEST ARROW\0"
  "SOUTH EAST ARROW CROSSING NORTH EAST ARROW\0"
  "SOUTH EAST ARROW FROM BAR\0"
  "SOUTH EAST ARROW TO CORNER\0"
  "SOUTH EAST ARROW WITH HOOK\0"
  "SOUTH EAST BLACK ARROW\0"
  "SOUTH EAST DOUBLE ARROW\0"
  "SOUTH EAST POINTING BUD\0"
  "SOUTH EAST POINTING LEAF\0"
  "SOUTH EAST POINTING VINE LEAF\0"
  "SOUTH EAST SANS-SERIF ARROW\0"
  "SOUTH EAST TRIANGLE-HEADED ARROW\0"
  "SOUTH EAST TRIANGLE-HEADED ARROW TO BAR\0"
  "SOUTH EAST WHITE ARROW\0"
  "SOUTH WEST ARROW\0"
  "SOUTH WEST ARROW AND NORTH WEST ARROW\0"
  "SOUTH WEST ARROW FROM BAR\0"
  "SOUTH WEST ARROW WITH HOOK\0"
  "SOUTH WEST BLACK ARROW\0"
  "SOUTH WEST DOUBLE ARROW\0"
  "SOUTH WEST POINTING BUD\0"
  "SOUTH WEST POINTING LEAF\0"
  "SOUTH WEST POINTING VINE LEAF\0"
  "SOUTH WEST SANS-SERIF ARROW\0"
  "SOUTH WEST TRIANGLE-HEADED ARROW\0"
  "SOUTH WEST TRIANGLE-HEADED ARROW TO BAR\0"
  "SOUTH WEST WHITE ARROW\0"
  "SOYOMBO CLUSTER-INITIAL LETTER LA\0"
  "SOYOMBO CLUSTER-INITIAL LETTER RA\0"
  "SOYOMBO CLUSTER-INITIAL LETTER SA\0"
  "SOYOMBO CLUSTER-INITIAL LETTER SHA\0"
  "SOYOMBO FINAL CONSONANT SIGN -A\0"
  "SOYOMBO FINAL CONSONANT SIGN B\0"
  "SOYOMBO FINAL CONSONANT SIGN D\0"
  "SOYOMBO FINAL CONSONANT SIGN G\0"
  "SOYOMBO FINAL CONSONANT SIGN K\0"
  "SOYOMBO FINAL CONSONANT SIGN L\0"
  "SOYOMBO FINAL CONSONANT SIGN M\0"
  "SOYOMBO FINAL CONSONANT SIGN N\0"
  "SOYOMBO FINAL CONSONANT SIGN NG\0"
  "SOYOMBO FINAL CONSONANT SIGN R\0"
  "SOYOMBO FINAL CONSONANT SIGN S\0"
  "SOYOMBO FINAL CONSONANT SIGN SH\0"
  "SOYOMBO GEMINATION MARK\0"
  "SOYOMBO HEAD MARK WITH MOON AND SUN\0"
  "SOYOMBO HEAD MARK WITH MOON AND SUN AND FLAME\0"
  "SOYOMBO HEAD MARK WITH MOON AND SUN AND TRIPLE FLAME\0"
  "SOYOMBO LETTER -A\0"
  "SOYOMBO LETTER A\0"
  "SOYOMBO LETTER BA\0"
  "SOYOMBO LETTER BHA\0"
  "SOYOMBO LETTER CA\0"
  "SOYOMBO LETTER CHA\0"
  "SOYOMBO LETTER DA\0"
  "SOYOMBO LETTER DDA\0"
  "SOYOMBO LETTER DDHA\0"
  "SOYOMBO LETTER DHA\0"
  "SOYOMBO LETTER DZA\0"
  "SOYOMBO LETTER GA\0"
  "SOYOMBO LETTER GHA\0"
  "SOYOMBO LETTER HA\0"
  "SOYOMBO LETTER JA\0"
  "SOYOMBO LETTER JHA\0"
  "SOYOMBO LETTER KA\0"
  "SOYOMBO LETTER KHA\0"
  "SOYOMBO LETTER KSSA\0"
  "SOYOMBO LETTER LA\0"
  "SOYOMBO LETTER MA\0"
  "SOYOMBO LETTER NA\0"
  "SOYOMBO LETTER NGA\0"
  "SOYOMBO LETTER NNA\0"
  "SOYOMBO LETTER NYA\0"
  "SOYOMBO LETTER PA\0"
  "SOYOMBO LETTER PHA\0"
  "SOYOMBO LETTER RA\0"
  "SOYOMBO LETTER SA\0"
  "SOYOMBO LETTER SHA\0"
  "SOYOMBO LETTER SSA\0"
  "SOYOMBO LETTER TA\0"
  "SOYOMBO LETTER THA\0"
  "SOYOMBO LETTER TSA\0"
  "SOYOMBO LETTER TSHA\0"
  "SOYOMBO LETTER TTA\0"
  "SOYOMBO LETTER TTHA\0"
  "SOYOMBO LETTER VA\0"
  "SOYOMBO LETTER YA\0"
  "SOYOMBO LETTER ZA\0"
  "SOYOMBO LETTER ZHA\0"
  "SOYOMBO MARK DOUBLE SHAD\0"
  "SOYOMBO MARK PLUTA\0"
  "SOYOMBO MARK SHAD\0"
  "SOYOMBO MARK TSHEG\0"
  "SOYOMBO SIGN ANUSVARA\0"
  "SOYOMBO SIGN JIHVAMULIYA\0"
  "SOYOMBO SIGN UPADHMANIYA\0"
  "SOYOMBO SIGN VISARGA\0"
  "SOYOMBO SUBJOINER\0"
  "SOYOMBO TERMINAL MARK-1\0"
  "SOYOMBO TERMINAL MARK-2\0"
  "SOYOMBO VOWEL LENGTH MARK\0"
  "SOYOMBO VOWEL SIGN AI\0"
  "SOYOMBO VOWEL SIGN AU\0"
  "SOYOMBO VOWEL SIGN E\0"
  "SOYOMBO VOWEL SIGN I\0"
  "SOYOMBO VOWEL SIGN O\0"
  "SOYOMBO VOWEL SIGN OE\0"
  "SOYOMBO VOWEL SIGN U\0"
  "SOYOMBO VOWEL SIGN UE\0"
  "SOYOMBO VOWEL SIGN VOCALIC L\0"
  "SOYOMBO VOWEL SIGN VOCALIC R\0"
  "SPACE\0"
  "SPAGHETTI\0"
  "SPARKLE\0"
  "SPARKLES\0"
  "SPARKLING HEART\0"
  "SPARSE HORIZONTAL FILL\0"
  "SPARSE VERTICAL FILL\0"
  "SPEAK-NO-EVIL MONKEY\0"
  "SPEAKER\0"
  "SPEAKER WITH CANCELLATION STROKE\0"
  "SPEAKER WITH ONE SOUND WAVE\0"
  "SPEAKER WITH THREE SOUND WAVES\0"
  "SPEAKING HEAD IN SILHOUETTE\0"
  "SPECKLE FILL FRAME-1\0"
  "SPECKLE FILL FRAME-2\0"
  "SPEECH BALLOON\0"
  "SPEEDBOAT\0"
  "SPESMILO SIGN\0"
  "SPHERICAL ANGLE\0"
  "SPHERICAL ANGLE OPENING LEFT\0"
  "SPHERICAL ANGLE OPENING UP\0"
  "SPIDER\0"
  "SPIDER WEB\0"
  "SPIRAL CALENDAR PAD\0"
  "SPIRAL NOTE PAD\0"
  "SPIRAL SHELL\0"
  "SPLASHING SWEAT SYMBOL\0"
  "SPLATTER\0"
  "SPONGE\0"
  "SPOOL OF THREAD\0"
  "SPOON\0"
  "SPORTS MEDAL\0"
  "SPOUTING WHALE\0"
  "SQUARE A OVER M\0"
  "SQUARE AARU\0"
  "SQUARE AM\0"
  "SQUARE ANPEA\0"
  "SQUARE APAATO\0"
  "SQUARE APERTURE\0"
  "SQUARE ARUHUA\0"
  "SQUARE AU\0"
  "SQUARE BAARERU\0"
  "SQUARE BAR\0"
  "SQUARE BEETA\0"
  "SQUARE BIRU\0"
  "SQUARE BORUTO\0"
  "SQUARE BQ\0"
  "SQUARE BUSSYERU\0"
  "SQUARE C OVER KG\0"
  "SQUARE CAL\0"
  "SQUARE CAP\0"
  "SQUARE CC\0"
  "SQUARE CD\0"
  "SQUARE CM\0"
  "SQUARE CM CUBED\0"
  "SQUARE CM SQUARED\0"
  "SQUARE CO\0"
  "SQUARE CORPORATION\0"
  "SQUARE CUP\0"
  "SQUARE DA\0"
  "SQUARE DAASU\0"
  "SQUARE DB\0"
  "SQUARE DESI\0"
  "SQUARE DJ\0"
  "SQUARE DL\0"
  "SQUARE DM\0"
  "SQUARE DM CUBED\0"
  "SQUARE DM SQUARED\0"
  "SQUARE DORU\0"
  "SQUARE EEKAA\0"
  "SQUARE ERA NAME HEISEI\0"
  "SQUARE ERA NAME MEIZI\0"
  "SQUARE ERA NAME REIWA\0"
  "SQUARE ERA NAME SYOUWA\0"
  "SQUARE ERA NAME TAISYOU\0"
  "SQUARE ERG\0"
  "SQUARE ESUKUUDO\0"
  "SQUARE EV\0"
  "SQUARE FM\0"
  "SQUARE FOOT\0"
  "SQUARE FOUR CORNER BLACK TRIANGLES\0"
  "SQUARE FOUR CORNER DIAGONALS\0"
  "SQUARE FOUR CORNER SALTIRES\0"
  "SQUARE FOUR CORNERS\0"
  "SQUARE GAL\0"
  "SQUARE GANMA\0"
  "SQUARE GARON\0"
  "SQUARE GB\0"
  "SQUARE GHZ\0"
  "SQUARE GIGA\0"
  "SQUARE GINII\0"
  "SQUARE GIRUDAA\0"
  "SQUARE GPA\0"
  "SQUARE GURAMU\0"
  "SQUARE GURAMUTON\0"
  "SQUARE GY\0"
  "SQUARE HA\0"
  "SQUARE HAITU\0"
  "SQUARE HEKUTAARU\0"
  "SQUARE HERUTU\0"
  "SQUARE HG\0"
  "SQUARE HIRAGANA HOKA\0"
  "SQUARE HON\0"
  "SQUARE HOON\0"
  "SQUARE HOORU\0"
  "SQUARE HP\0"
  "SQUARE HPA\0"
  "SQUARE HUARADDO\0"
  "SQUARE HUIITO\0"
  "SQUARE HURAN\0"
  "SQUARE HZ\0"
  "SQUARE IMAGE OF\0"
  "SQUARE IMAGE OF OR EQUAL TO\0"
  "SQUARE IMAGE OF OR NOT EQUAL TO\0"
  "SQUARE IN\0"
  "SQUARE ININGU\0"
  "SQUARE INTI\0"
  "SQUARE IU\0"
  "SQUARE K OHM\0"
  "SQUARE KA\0"
  "SQUARE KAIRI\0"
  "SQUARE KARATTO\0"
  "SQUARE KARORII\0"
  "SQUARE KB\0"
  "SQUARE KCAL\0"
  "SQUARE KEESU\0"
  "SQUARE KG\0"
  "SQUARE KHZ\0"
  "SQUARE KIRO\0"
  "SQUARE KIROGURAMU\0"
  "SQUARE KIROMEETORU\0"
  "SQUARE KIROWATTO\0"
  "SQUARE KK\0"
  "SQUARE KL\0"
  "SQUARE KM\0"
  "SQUARE KM CAPITAL\0"
  "SQUARE KM CUBED\0"
  "SQUARE KM SQUARED\0"
  "SQUARE KOOPO\0"
  "SQUARE KORUNA\0"
  "SQUARE KPA\0"
  "SQUARE KT\0"
  "SQUARE KUROONE\0"
  "SQUARE KURUZEIRO\0"
  "SQUARE KV\0"
  "SQUARE KW\0"
  "SQUARE KYURII\0"
  "SQUARE LEFT OPEN BOX OPERATOR\0"
  "SQUARE LM\0"
  "SQUARE LN\0"
  "SQUARE LOG\0"
  "SQUARE LOZENGE\0"
  "SQUARE LX\0"
  "SQUARE M CUBED\0"
  "SQUARE M OHM\0"
  "SQUARE M OVER S\0"
  "SQUARE M OVER S SQUARED\0"
  "SQUARE M SQUARED\0"
  "SQUARE MA\0"
  "SQUARE MAHHA\0"
  "SQUARE MAIKURO\0"
  "SQUARE MAIRU\0"
  "SQUARE MANSYON\0"
  "SQUARE MARUKU\0"
  "SQUARE MB\0"
  "SQUARE MB SMALL\0"
  "SQUARE MEETORU\0"
  "SQUARE MEGA\0"
  "SQUARE MEGATON\0"
  "SQUARE MG\0"
  "SQUARE MHZ\0"
  "SQUARE MIKURON\0"
  "SQUARE MIL\0"
  "SQUARE MIRI\0"
  "SQUARE MIRIBAARU\0"
  "SQUARE ML\0"
  "SQUARE MM\0"
  "SQUARE MM CUBED\0"
  "SQUARE MM SQUARED\0"
  "SQUARE MOL\0"
  "SQUARE MPA\0"
  "SQUARE MS\0"
  "SQUARE MU A\0"
  "SQUARE MU F\0"
  "SQUARE MU G\0"
  "SQUARE MU L\0"
  "SQUARE MU M\0"
  "SQUARE MU S\0"
  "SQUARE MU V\0"
  "SQUARE MU W\0"
  "SQUARE MV\0"
  "SQUARE MV MEGA\0"
  "SQUARE MW\0"
  "SQUARE MW MEGA\0"
  "SQUARE NA\0"
  "SQUARE NANO\0"
  "SQUARE NF\0"
  "SQUARE NM\0"
  "SQUARE NOTTO\0"
  "SQUARE NS\0"
  "SQUARE NV\0"
  "SQUARE NW\0"
  "SQUARE ONSU\0"
  "SQUARE OOMU\0"
  "SQUARE ORIGINAL OF\0"
  "SQUARE ORIGINAL OF OR EQUAL TO\0"
  "SQUARE ORIGINAL OF OR NOT EQUAL TO\0"
  "SQUARE OV\0"
  "SQUARE PA\0"
  "SQUARE PA AMPS\0"
  "SQUARE PAASENTO\0"
  "SQUARE PAATU\0"
  "SQUARE PC\0"
  "SQUARE PEEZI\0"
  "SQUARE PENIHI\0"
  "SQUARE PENSU\0"
  "SQUARE PESO\0"
  "SQUARE PF\0"
  "SQUARE PH\0"
  "SQUARE PIASUTORU\0"
  "SQUARE PIKO\0"
  "SQUARE PIKURU\0"
  "SQUARE PM\0"
  "SQUARE POINTO\0"
  "SQUARE PONDO\0"
  "SQUARE POSITION INDICATOR\0"
  "SQUARE PPM\0"
  "SQUARE PR\0"
  "SQUARE PS\0"
  "SQUARE PV\0"
  "SQUARE PW\0"
  "SQUARE RAD\0"
  "SQUARE RAD OVER S\0"
  "SQUARE RAD OVER S SQUARED\0"
  "SQUARE REMU\0"
  "SQUARE RENTOGEN\0"
  "SQUARE RIGHT OPEN BOX OPERATOR\0"
  "SQUARE RIRA\0"
  "SQUARE RITTORU\0"
  "SQUARE ROOT\0"
  "SQUARE RUPII\0"
  "SQUARE RUUBURU\0"
  "SQUARE SAIKURU\0"
  "SQUARE SANTIIMU\0"
  "SQUARE SENTI\0"
  "SQUARE SENTO\0"
  "SQUARE SIRINGU\0"
  "SQUARE SPIRAL FROM BOTTOM LEFT\0"
  "SQUARE SPIRAL FROM BOTTOM RIGHT\0"
  "SQUARE SPIRAL FROM TOP LEFT\0"
  "SQUARE SPIRAL FROM TOP RIGHT\0"
  "SQUARE SR\0"
  "SQUARE SV\0"
  "SQUARE TARGET\0"
  "SQUARE THZ\0"
  "SQUARE TON\0"
  "SQUARE UON\0"
  "SQUARE V OVER M\0"
  "SQUARE WATTO\0"
  "SQUARE WB\0"
  "SQUARE WITH BOTTOM HALF BLACK\0"
  "SQUARE WITH CONTOURED OUTLINE\0"
  "SQUARE WITH DIAGONAL CROSSHATCH FILL\0"
  "SQUARE WITH HORIZONTAL FILL\0"
  "SQUARE WITH LEFT HALF BLACK\0"
  "SQUARE WITH LOWER LEFT DIAGONAL HALF BLACK\0"
  "SQUARE WITH LOWER RIGHT DIAGONAL HALF BLACK\0"
  "SQUARE WITH ORTHOGONAL CROSSHATCH FILL\0"
  "SQUARE WITH RIGHT HALF BLACK\0"
  "SQUARE WITH TOP HALF BLACK\0"
  "SQUARE WITH UPPER LEFT DIAGONAL HALF BLACK\0"
  "SQUARE WITH UPPER LEFT TO LOWER RIGHT FILL\0"
  "SQUARE WITH UPPER RIGHT DIAGONAL HALF BLACK\0"
  "SQUARE WITH UPPER RIGHT TO LOWER LEFT FILL\0"
  "SQUARE WITH VERTICAL FILL\0"
  "SQUARE YAADO\0"
  "SQUARE YAARU\0"
  "SQUARE YUAN\0"
  "SQUARED ASTERISK\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-4E00\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-4E09\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-4E2D\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-4E8C\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-4EA4\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-518D\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-521D\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-524D\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-5272\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-53CC\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-53F3\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-5408\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-5439\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-55B6\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-58F0\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-591A\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-5929\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-5B57\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-5DE6\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-5F8C\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-624B\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6253\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6295\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6307\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6355\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6599\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-65B0\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6620\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6708\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6709\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6E80\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-6F14\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-7121\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-751F\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-7533\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-7981\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-7A7A\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-7D42\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-89E3\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-8CA9\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-8D70\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-904A\0"
  "SQUARED CJK UNIFIED IDEOGRAPH-914D\0"
  "SQUARED CL\0"
  "SQUARED COOL\0"
  "SQUARED DOT OPERATOR\0"
  "SQUARED EIGHT K\0"
  "SQUARED FALLING DIAGONAL SLASH\0"
  "SQUARED FIVE POINT ONE\0"
  "SQUARED FOUR DOT PUNCTUATION\0"
  "SQUARED FOUR K\0"
  "SQUARED FREE\0"
  "SQUARED HC\0"
  "SQUARED HDR\0"
  "SQUARED HI-RES\0"
  "SQUARED HV\0"
  "SQUARED ID\0"
  "SQUARED KATAKANA DE\0"
  "SQUARED KATAKANA KOKO\0"
  "SQUARED KATAKANA SA\0"
  "SQUARED KEY\0"
  "SQUARED LATIN CAPITAL LETTER A\0"
  "SQUARED LATIN CAPITAL LETTER B\0"
  "SQUARED LATIN CAPITAL LETTER C\0"
  "SQUARED LATIN CAPITAL LETTER D\0"
  "SQUARED LATIN CAPITAL LETTER E\0"
  "SQUARED LATIN CAPITAL LETTER F\0"
  "SQUARED LATIN CAPITAL LETTER G\0"
  "SQUARED LATIN CAPITAL LETTER H\0"
  "SQUARED LATIN CAPITAL LETTER I\0"
  "SQUARED LATIN CAPITAL LETTER J\0"
  "SQUARED LATIN CAPITAL LETTER K\0"
  "SQUARED LATIN CAPITAL LETTER L\0"
  "SQUARED LATIN CAPITAL LETTER M\0"
  "SQUARED LATIN CAPITAL LETTER N\0"
  "SQUARED LATIN CAPITAL LETTER O\0"
  "SQUARED LATIN CAPITAL LETTER P\0"
  "SQUARED LATIN CAPITAL LETTER Q\0"
  "SQUARED LATIN CAPITAL LETTER R\0"
  "SQUARED LATIN CAPITAL LETTER S\0"
  "SQUARED LATIN CAPITAL LETTER T\0"
  "SQUARED LATIN CAPITAL LETTER U\0"
  "SQUARED LATIN CAPITAL LETTER V\0"
  "SQUARED LATIN CAPITAL LETTER W\0"
  "SQUARED LATIN CAPITAL LETTER X\0"
  "SQUARED LATIN CAPITAL LETTER Y\0"
  "SQUARED LATIN CAPITAL LETTER Z\0"
  "SQUARED LATIN SMALL LETTER D\0"
  "SQUARED LOGICAL AND\0"
  "SQUARED LOGICAL OR\0"
  "SQUARED LOSSLESS\0"
  "SQUARED MINUS\0"
  "SQUARED MV\0"
  "SQUARED NEW\0"
  "SQUARED NG\0"
  "SQUARED OK\0"
  "SQUARED ONE HUNDRED TWENTY P\0"
  "SQUARED PLUS\0"
  "SQUARED PPV\0"
  "SQUARED RISING DIAGONAL SLASH\0"
  "SQUARED SALTIRE\0"
  "SQUARED SD\0"
  "SQUARED SECOND SCREEN\0"
  "SQUARED SEVEN POINT ONE\0"
  "SQUARED SHV\0"
  "SQUARED SIXTY P\0"
  "SQUARED SMALL CIRCLE\0"
  "SQUARED SOS\0"
  "SQUARED SQUARE\0"
  "SQUARED SS\0"
  "SQUARED THREE D\0"
  "SQUARED TIMES\0"
  "SQUARED TWENTY-TWO POINT TWO\0"
  "SQUARED TWO K\0"
  "SQUARED UHD\0"
  "SQUARED UP WITH EXCLAMATION MARK\0"
  "SQUARED VOD\0"
  "SQUARED VS\0"
  "SQUARED WC\0"
  "SQUAT BLACK RIGHTWARDS ARROW\0"
  "SQUID\0"
  "STADIUM\0"
  "STAFF OF AESCULAPIUS\0"
  "STAFF OF HERMES\0"
  "STAMPED ENVELOPE\0"
  "STANDING PERSON\0"
  "STAR AND CRESCENT\0"
  "STAR EQUALS\0"
  "STAR OF DAVID\0"
  "STAR OPERATOR\0"
  "STAR WITH LEFT HALF BLACK\0"
  "STAR WITH RIGHT HALF BLACK\0"
  "STATION\0"
  "STATUE OF LIBERTY\0"
  "STEAM LOCOMOTIVE\0"
  "STEAMING BOWL\0"
  "STENOGRAPHIC FULL STOP\0"
  "STETHOSCOPE\0"
  "STICK FIGURE\0"
  "STICK FIGURE LEANING LEFT\0"
  "STICK FIGURE LEANING RIGHT\0"
  "STICK FIGURE WITH ARMS RAISED\0"
  "STICK FIGURE WITH DRESS\0"
  "STOCK CHART\0"
  "STOPWATCH\0"
  "STRAIGHT RULER\0"
  "STRAIGHTNESS\0"
  "STRAWBERRY\0"
  "STRESS OUTLINED WHITE STAR\0"
  "STRICTLY EQUIVALENT TO\0"
  "STRIPED DOWN-POINTING TRIANGLE\0"
  "STRIPED LEFT-POINTING TRIANGLE\0"
  "STRIPED RIGHT-POINTING TRIANGLE\0"
  "STRIPED UP-POINTING TRIANGLE\0"
  "STUDIO MICROPHONE\0"
  "STUFFED FLATBREAD\0"
  "STUPA\0"
  "SUBSCRIPT EIGHT\0"
  "SUBSCRIPT EQUALS SIGN\0"
  "SUBSCRIPT FIVE\0"
  "SUBSCRIPT FOUR\0"
  "SUBSCRIPT LEFT PARENTHESIS\0"
  "SUBSCRIPT MINUS\0"
  "SUBSCRIPT NINE\0"
  "SUBSCRIPT ONE\0"
  "SUBSCRIPT PLUS SIGN\0"
  "SUBSCRIPT RIGHT PARENTHESIS\0"
  "SUBSCRIPT SEVEN\0"
  "SUBSCRIPT SIX\0"
  "SUBSCRIPT THREE\0"
  "SUBSCRIPT TWO\0"
  "SUBSCRIPT ZERO\0"
  "SUBSET ABOVE RIGHTWARDS ARROW\0"
  "SUBSET ABOVE SUBSET\0"
  "SUBSET ABOVE SUPERSET\0"
  "SUBSET OF\0"
  "SUBSET OF ABOVE ALMOST EQUAL TO\0"
  "SUBSET OF ABOVE EQUALS SIGN\0"
  "SUBSET OF ABOVE NOT EQUAL TO\0"
  "SUBSET OF ABOVE TILDE OPERATOR\0"
  "SUBSET OF OR EQUAL TO\0"
  "SUBSET OF OR EQUAL TO WITH DOT ABOVE\0"
  "SUBSET OF WITH NOT EQUAL TO\0"
  "SUBSET WITH DOT\0"
  "SUBSET WITH MULTIPLICATION SIGN BELOW\0"
  "SUBSET WITH PLUS SIGN BELOW\0"
  "SUCCEEDS\0"
  "SUCCEEDS ABOVE ALMOST EQUAL TO\0"
  "SUCCEEDS ABOVE EQUALS SIGN\0"
  "SUCCEEDS ABOVE NOT ALMOST EQUAL TO\0"
  "SUCCEEDS ABOVE NOT EQUAL TO\0"
  "SUCCEEDS ABOVE SINGLE-LINE EQUALS SIGN\0"
  "SUCCEEDS ABOVE SINGLE-LINE NOT EQUAL TO\0"
  "SUCCEEDS BUT NOT EQUIVALENT TO\0"
  "SUCCEEDS OR EQUAL TO\0"
  "SUCCEEDS OR EQUIVALENT TO\0"
  "SUCCEEDS UNDER RELATION\0"
  "SUMMATION BOTTOM\0"
  "SUMMATION TOP\0"
  "SUMMATION WITH INTEGRAL\0"
  "SUN\0"
  "SUN BEHIND CLOUD\0"
  "SUN WITH FACE\0"
  "SUNDANESE AVAGRAHA\0"
  "SUNDANESE CONSONANT SIGN PAMINGKAL\0"
  "SUNDANESE CONSONANT SIGN PANYAKRA\0"
  "SUNDANESE CONSONANT SIGN PANYIKU\0"
  "SUNDANESE CONSONANT SIGN PASANGAN MA\0"
  "SUNDANESE CONSONANT SIGN PASANGAN WA\0"
  "SUNDANESE DIGIT EIGHT\0"
  "SUNDANESE DIGIT FIVE\0"
  "SUNDANESE DIGIT FOUR\0"
  "SUNDANESE DIGIT NINE\0"
  "SUNDANESE DIGIT ONE\0"
  "SUNDANESE DIGIT SEVEN\0"
  "SUNDANESE DIGIT SIX\0"
  "SUNDANESE DIGIT THREE\0"
  "SUNDANESE DIGIT TWO\0"
  "SUNDANESE DIGIT ZERO\0"
  "SUNDANESE LETTER A\0"
  "SUNDANESE LETTER AE\0"
  "SUNDANESE LETTER BA\0"
  "SUNDANESE LETTER BHA\0"
  "SUNDANESE LETTER CA\0"
  "SUNDANESE LETTER DA\0"
  "SUNDANESE LETTER E\0"
  "SUNDANESE LETTER EU\0"
  "SUNDANESE LETTER FA\0"
  "SUNDANESE LETTER FINAL K\0"
  "SUNDANESE LETTER FINAL M\0"
  "SUNDANESE LETTER GA\0"
  "SUNDANESE LETTER HA\0"
  "SUNDANESE LETTER I\0"
  "SUNDANESE LETTER JA\0"
  "SUNDANESE LETTER KA\0"
  "SUNDANESE LETTER KHA\0"
  "SUNDANESE LETTER LA\0"
  "SUNDANESE LETTER LEU\0"
  "SUNDANESE LETTER MA\0"
  "SUNDANESE LETTER NA\0"
  "SUNDANESE LETTER NGA\0"
  "SUNDANESE LETTER NYA\0"
  "SUNDANESE LETTER O\0"
  "SUNDANESE LETTER PA\0"
  "SUNDANESE LETTER QA\0"
  "SUNDANESE LETTER RA\0"
  "SUNDANESE LETTER REU\0"
  "SUNDANESE LETTER SA\0"
  "SUNDANESE LETTER SYA\0"
  "SUNDANESE LETTER TA\0"
  "SUNDANESE LETTER U\0"
  "SUNDANESE LETTER VA\0"
  "SUNDANESE LETTER WA\0"
  "SUNDANESE LETTER XA\0"
  "SUNDANESE LETTER YA\0"
  "SUNDANESE LETTER ZA\0"
  "SUNDANESE PUNCTUATION BINDU BA SATANGA\0"
  "SUNDANESE PUNCTUATION BINDU CAKRA\0"
  "SUNDANESE PUNCTUATION BINDU DA SATANGA\0"
  "SUNDANESE PUNCTUATION BINDU KA SATANGA\0"
  "SUNDANESE PUNCTUATION BINDU LEU SATANGA\0"
  "SUNDANESE PUNCTUATION BINDU PANGLONG\0"
  "SUNDANESE PUNCTUATION BINDU PURNAMA\0"
  "SUNDANESE PUNCTUATION BINDU SURYA\0"
  "SUNDANESE SIGN PAMAAEH\0"
  "SUNDANESE SIGN PANGLAYAR\0"
  "SUNDANESE SIGN PANGWISAD\0"
  "SUNDANESE SIGN PANYECEK\0"
  "SUNDANESE SIGN VIRAMA\0"
  "SUNDANESE VOWEL SIGN PAMEPET\0"
  "SUNDANESE VOWEL SIGN PANAELAENG\0"
  "SUNDANESE VOWEL SIGN PANEULEUNG\0"
  "SUNDANESE VOWEL SIGN PANGHULU\0"
  "SUNDANESE VOWEL SIGN PANOLONG\0"
  "SUNDANESE VOWEL SIGN PANYUKU\0"
  "SUNFLOWER\0"
  "SUNRISE\0"
  "SUNRISE OVER MOUNTAINS\0"
  "SUNSET OVER BUILDINGS\0"
  "SUNUWAR DIGIT EIGHT\0"
  "SUNUWAR DIGIT FIVE\0"
  "SUNUWAR DIGIT FOUR\0"
  "SUNUWAR DIGIT NINE\0"
  "SUNUWAR DIGIT ONE\0"
  "SUNUWAR DIGIT SEVEN\0"
  "SUNUWAR DIGIT SIX\0"
  "SUNUWAR DIGIT THREE\0"
  "SUNUWAR DIGIT TWO\0"
  "SUNUWAR DIGIT ZERO\0"
  "SUNUWAR LETTER AAL\0"
  "SUNUWAR LETTER APPHO\0"
  "SUNUWAR LETTER AVA\0"
  "SUNUWAR LETTER BUR\0"
  "SUNUWAR LETTER CARMI\0"
  "SUNUWAR LETTER CHELAP\0"
  "SUNUWAR LETTER DEVI\0"
  "SUNUWAR LETTER DONGA\0"
  "SUNUWAR LETTER EKO\0"
  "SUNUWAR LETTER GIL\0"
  "SUNUWAR LETTER HAMSO\0"
  "SUNUWAR LETTER IMAR\0"
  "SUNUWAR LETTER JYAH\0"
  "SUNUWAR LETTER KHA\0"
  "SUNUWAR LETTER KIK\0"
  "SUNUWAR LETTER KLOKO\0"
  "SUNUWAR LETTER LOACHA\0"
  "SUNUWAR LETTER MA\0"
  "SUNUWAR LETTER NAH\0"
  "SUNUWAR LETTER NGAR\0"
  "SUNUWAR LETTER OTTHI\0"
  "SUNUWAR LETTER PHAR\0"
  "SUNUWAR LETTER PIP\0"
  "SUNUWAR LETTER REU\0"
  "SUNUWAR LETTER SHYELE\0"
  "SUNUWAR LETTER SHYER\0"
  "SUNUWAR LETTER TASLA\0"
  "SUNUWAR LETTER TENTU\0"
  "SUNUWAR LETTER THARI\0"
  "SUNUWAR LETTER THELE\0"
  "SUNUWAR LETTER UTTHI\0"
  "SUNUWAR LETTER VARCA\0"
  "SUNUWAR LETTER YAT\0"
  "SUNUWAR SIGN PVO\0"
  "SUPERHERO\0"
  "SUPERSCRIPT EIGHT\0"
  "SUPERSCRIPT EQUALS SIGN\0"
  "SUPERSCRIPT FIVE\0"
  "SUPERSCRIPT FOUR\0"
  "SUPERSCRIPT LATIN SMALL LETTER I\0"
  "SUPERSCRIPT LATIN SMALL LETTER N\0"
  "SUPERSCRIPT LEFT PARENTHESIS\0"
  "SUPERSCRIPT MINUS\0"
  "SUPERSCRIPT NINE\0"
  "SUPERSCRIPT ONE\0"
  "SUPERSCRIPT PLUS SIGN\0"
  "SUPERSCRIPT RIGHT PARENTHESIS\0"
  "SUPERSCRIPT SEVEN\0"
  "SUPERSCRIPT SIX\0"
  "SUPERSCRIPT THREE\0"
  "SUPERSCRIPT TWO\0"
  "SUPERSCRIPT ZERO\0"
  "SUPERSET ABOVE LEFTWARDS ARROW\0"
  "SUPERSET ABOVE SUBSET\0"
  "SUPERSET ABOVE SUPERSET\0"
  "SUPERSET BESIDE AND JOINED BY DASH WITH SUBSET\0"
  "SUPERSET BESIDE SUBSET\0"
  "SUPERSET OF\0"
  "SUPERSET OF ABOVE ALMOST EQUAL TO\0"
  "SUPERSET OF ABOVE EQUALS SIGN\0"
  "SUPERSET OF ABOVE NOT EQUAL TO\0"
  "SUPERSET OF ABOVE TILDE OPERATOR\0"
  "SUPERSET OF OR EQUAL TO\0"
  "SUPERSET OF OR EQUAL TO WITH DOT ABOVE\0"
  "SUPERSET OF WITH NOT EQUAL TO\0"
  "SUPERSET PRECEDING SOLIDUS\0"
  "SUPERSET WITH DOT\0"
  "SUPERSET WITH MULTIPLICATION SIGN BELOW\0"
  "SUPERSET WITH PLUS SIGN BELOW\0"
  "SUPERVILLAIN\0"
  "SURFACE INTEGRAL\0"
  "SURFER\0"
  "SUSHI\0"
  "SUSPENSION RAILWAY\0"
  "SWAN\0"
  "SWASH AMPERSAND ORNAMENT\0"
  "SWIMMER\0"
  "SWUNG DASH\0"
  "SYLOTI NAGRI LETTER A\0"
  "SYLOTI NAGRI LETTER BHO\0"
  "SYLOTI NAGRI LETTER BO\0"
  "SYLOTI NAGRI LETTER CHO\0"
  "SYLOTI NAGRI LETTER CO\0"
  "SYLOTI NAGRI LETTER DDHO\0"
  "SYLOTI NAGRI LETTER DDO\0"
  "SYLOTI NAGRI LETTER DHO\0"
  "SYLOTI NAGRI LETTER DO\0"
  "SYLOTI NAGRI LETTER E\0"
  "SYLOTI NAGRI LETTER GHO\0"
  "SYLOTI NAGRI LETTER GO\0"
  "SYLOTI NAGRI LETTER HO\0"
  "SYLOTI NAGRI LETTER I\0"
  "SYLOTI NAGRI LETTER JHO\0"
  "SYLOTI NAGRI LETTER JO\0"
  "SYLOTI NAGRI LETTER KHO\0"
  "SYLOTI NAGRI LETTER KO\0"
  "SYLOTI NAGRI LETTER LO\0"
  "SYLOTI NAGRI LETTER MO\0"
  "SYLOTI NAGRI LETTER NO\0"
  "SYLOTI NAGRI LETTER O\0"
  "SYLOTI NAGRI LETTER PHO\0"
  "SYLOTI NAGRI LETTER PO\0"
  "SYLOTI NAGRI LETTER RO\0"
  "SYLOTI NAGRI LETTER RRO\0"
  "SYLOTI NAGRI LETTER SO\0"
  "SYLOTI NAGRI LETTER THO\0"
  "SYLOTI NAGRI LETTER TO\0"
  "SYLOTI NAGRI LETTER TTHO\0"
  "SYLOTI NAGRI LETTER TTO\0"
  "SYLOTI NAGRI LETTER U\0"
  "SYLOTI NAGRI POETRY MARK-1\0"
  "SYLOTI NAGRI POETRY MARK-2\0"
  "SYLOTI NAGRI POETRY MARK-3\0"
  "SYLOTI NAGRI POETRY MARK-4\0"
  "SYLOTI NAGRI SIGN ALTERNATE HASANTA\0"
  "SYLOTI NAGRI SIGN ANUSVARA\0"
  "SYLOTI NAGRI SIGN DVISVARA\0"
  "SYLOTI NAGRI SIGN HASANTA\0"
  "SYLOTI NAGRI VOWEL SIGN A\0"
  "SYLOTI NAGRI VOWEL SIGN E\0"
  "SYLOTI NAGRI VOWEL SIGN I\0"
  "SYLOTI NAGRI VOWEL SIGN OO\0"
  "SYLOTI NAGRI VOWEL SIGN U\0"
  "SYMBOL FOR ACKNOWLEDGE\0"
  "SYMBOL FOR BACKSPACE\0"
  "SYMBOL FOR BELL\0"
  "SYMBOL FOR CANCEL\0"
  "SYMBOL FOR CARRIAGE RETURN\0"
  "SYMBOL FOR DATA LINK ESCAPE\0"
  "SYMBOL FOR DELETE\0"
  "SYMBOL FOR DELETE FORM TWO\0"
  "SYMBOL FOR DELETE MEDIUM SHADE FORM\0"
  "SYMBOL FOR DELETE RECTANGULAR CHECKER BOARD FORM\0"
  "SYMBOL FOR DELETE SQUARE CHECKER BOARD FORM\0"
  "SYMBOL FOR DEVICE CONTROL FOUR\0"
  "SYMBOL FOR DEVICE CONTROL ONE\0"
  "SYMBOL FOR DEVICE CONTROL THREE\0"
  "SYMBOL FOR DEVICE CONTROL TWO\0"
  "SYMBOL FOR END OF MEDIUM\0"
  "SYMBOL FOR END OF TEXT\0"
  "SYMBOL FOR END OF TRANSMISSION\0"
  "SYMBOL FOR END OF TRANSMISSION BLOCK\0"
  "SYMBOL FOR ENQUIRY\0"
  "SYMBOL FOR ESCAPE\0"
  "SYMBOL FOR FILE SEPARATOR\0"
  "SYMBOL FOR FORM FEED\0"
  "SYMBOL FOR GROUP SEPARATOR\0"
  "SYMBOL FOR HORIZONTAL TABULATION\0"
  "SYMBOL FOR LINE FEED\0"
  "SYMBOL FOR MARKS CHAPTER\0"
  "SYMBOL FOR NEGATIVE ACKNOWLEDGE\0"
  "SYMBOL FOR NEWLINE\0"
  "SYMBOL FOR NULL\0"
  "SYMBOL FOR RECORD SEPARATOR\0"
  "SYMBOL FOR SAMARITAN SOURCE\0"
  "SYMBOL FOR SHIFT IN\0"
  "SYMBOL FOR SHIFT OUT\0"
  "SYMBOL FOR SPACE\0"
  "SYMBOL FOR START OF HEADING\0"
  "SYMBOL FOR START OF TEXT\0"
  "SYMBOL FOR SUBSTITUTE\0"
  "SYMBOL FOR SUBSTITUTE FORM TWO\0"
  "SYMBOL FOR SYNCHRONOUS IDLE\0"
  "SYMBOL FOR TYPE A ELECTRONICS\0"
  "SYMBOL FOR UNIT SEPARATOR\0"
  "SYMBOL FOR VERTICAL TABULATION\0"
  "SYMMETRY\0"
  "SYNAGOGUE\0"
  "SYRIAC ABBREVIATION MARK\0"
  "SYRIAC BARREKH\0"
  "SYRIAC COLON SKEWED LEFT\0"
  "SYRIAC COLON SKEWED RIGHT\0"
  "SYRIAC CONTRACTION\0"
  "SYRIAC DOTTED ZLAMA ANGULAR\0"
  "SYRIAC DOTTED ZLAMA HORIZONTAL\0"
  "SYRIAC END OF PARAGRAPH\0"
  "SYRIAC ESASA ABOVE\0"
  "SYRIAC ESASA BELOW\0"
  "SYRIAC FEMININE DOT\0"
  "SYRIAC HARKLEAN ASTERISCUS\0"
  "SYRIAC HARKLEAN METOBELUS\0"
  "SYRIAC HARKLEAN OBELUS\0"
  "SYRIAC HBASA ABOVE\0"
  "SYRIAC HBASA BELOW\0"
  "SYRIAC HBASA-ESASA DOTTED\0"
  "SYRIAC HORIZONTAL COLON\0"
  "SYRIAC LETTER ALAPH\0"
  "SYRIAC LETTER BETH\0"
  "SYRIAC LETTER DALATH\0"
  "SYRIAC LETTER DOTLESS DALATH RISH\0"
  "SYRIAC LETTER E\0"
  "SYRIAC LETTER FINAL SEMKATH\0"
  "SYRIAC LETTER GAMAL\0"
  "SYRIAC LETTER GAMAL GARSHUNI\0"
  "SYRIAC LETTER HE\0"
  "SYRIAC LETTER HETH\0"
  "SYRIAC LETTER KAPH\0"
  "SYRIAC LETTER LAMADH\0"
  "SYRIAC LETTER MALAYALAM BHA\0"
  "SYRIAC LETTER MALAYALAM JA\0"
  "SYRIAC LETTER MALAYALAM LLA\0"
  "SYRIAC LETTER MALAYALAM LLLA\0"
  "SYRIAC LETTER MALAYALAM NGA\0"
  "SYRIAC LETTER MALAYALAM NNA\0"
  "SYRIAC LETTER MALAYALAM NNNA\0"
  "SYRIAC LETTER MALAYALAM NYA\0"
  "SYRIAC LETTER MALAYALAM RA\0"
  "SYRIAC LETTER MALAYALAM SSA\0"
  "SYRIAC LETTER MALAYALAM TTA\0"
  "SYRIAC LETTER MIM\0"
  "SYRIAC LETTER NUN\0"
  "SYRIAC LETTER PE\0"
  "SYRIAC LETTER PERSIAN BHETH\0"
  "SYRIAC LETTER PERSIAN DHALATH\0"
  "SYRIAC LETTER PERSIAN GHAMAL\0"
  "SYRIAC LETTER QAPH\0"
  "SYRIAC LETTER REVERSED PE\0"
  "SYRIAC LETTER RISH\0"
  "SYRIAC LETTER SADHE\0"
  "SYRIAC LETTER SEMKATH\0"
  "SYRIAC LETTER SHIN\0"
  "SYRIAC LETTER SOGDIAN FE\0"
  "SYRIAC LETTER SOGDIAN KHAPH\0"
  "SYRIAC LETTER SOGDIAN ZHAIN\0"
  "SYRIAC LETTER SUPERSCRIPT ALAPH\0"
  "SYRIAC LETTER TAW\0"
  "SYRIAC LETTER TETH\0"
  "SYRIAC LETTER TETH GARSHUNI\0"
  "SYRIAC LETTER WAW\0"
  "SYRIAC LETTER YUDH\0"
  "SYRIAC LETTER YUDH HE\0"
  "SYRIAC LETTER ZAIN\0"
  "SYRIAC MUSIC\0"
  "SYRIAC OBLIQUE LINE ABOVE\0"
  "SYRIAC OBLIQUE LINE BELOW\0"
  "SYRIAC PTHAHA ABOVE\0"
  "SYRIAC PTHAHA BELOW\0"
  "SYRIAC PTHAHA DOTTED\0"
  "SYRIAC QUSHSHAYA\0"
  "SYRIAC RBASA ABOVE\0"
  "SYRIAC RBASA BELOW\0"
  "SYRIAC RUKKAKHA\0"
  "SYRIAC RWAHA\0"
  "SYRIAC SUBLINEAR COLON\0"
  "SYRIAC SUBLINEAR COLON SKEWED RIGHT\0"
  "SYRIAC SUBLINEAR FULL STOP\0"
  "SYRIAC SUPRALINEAR COLON\0"
  "SYRIAC SUPRALINEAR COLON SKEWED LEFT\0"
  "SYRIAC SUPRALINEAR FULL STOP\0"
  "SYRIAC THREE DOTS ABOVE\0"
  "SYRIAC THREE DOTS BELOW\0"
  "SYRIAC TWO VERTICAL DOTS ABOVE\0"
  "SYRIAC TWO VERTICAL DOTS BELOW\0"
  "SYRIAC ZQAPHA ABOVE\0"
  "SYRIAC ZQAPHA BELOW\0"
  "SYRIAC ZQAPHA DOTTED\0"
  "SYRINGE\0"
  "T-REX\0"
  "T-SHIRT\0"
  "TABLE TENNIS PADDLE AND BALL\0"
  "TACO\0"
  "TAG AMPERSAND\0"
  "TAG APOSTROPHE\0"
  "TAG ASTERISK\0"
  "TAG CIRCUMFLEX ACCENT\0"
  "TAG COLON\0"
  "TAG COMMA\0"
  "TAG COMMERCIAL AT\0"
  "TAG DIGIT EIGHT\0"
  "TAG DIGIT FIVE\0"
  "TAG DIGIT FOUR\0"
  "TAG DIGIT NINE\0"
  "TAG DIGIT ONE\0"
  "TAG DIGIT SEVEN\0"
  "TAG DIGIT SIX\0"
  "TAG DIGIT THREE\0"
  "TAG DIGIT TWO\0"
  "TAG DIGIT ZERO\0"
  "TAG DOLLAR SIGN\0"
  "TAG EQUALS SIGN\0"
  "TAG EXCLAMATION MARK\0"
  "TAG FULL STOP\0"
  "TAG GRAVE ACCENT\0"
  "TAG GREATER-THAN SIGN\0"
  "TAG HYPHEN-MINUS\0"
  "TAG LATIN CAPITAL LETTER A\0"
  "TAG LATIN CAPITAL LETTER B\0"
  "TAG LATIN CAPITAL LETTER C\0"
  "TAG LATIN CAPITAL LETTER D\0"
  "TAG LATIN CAPITAL LETTER E\0"
  "TAG LATIN CAPITAL LETTER F\0"
  "TAG LATIN CAPITAL LETTER G\0"
  "TAG LATIN CAPITAL LETTER H\0"
  "TAG LATIN CAPITAL LETTER I\0"
  "TAG LATIN CAPITAL LETTER J\0"
  "TAG LATIN CAPITAL LETTER K\0"
  "TAG LATIN CAPITAL LETTER L\0"
  "TAG LATIN CAPITAL LETTER M\0"
  "TAG LATIN CAPITAL LETTER N\0"
  "TAG LATIN CAPITAL LETTER O\0"
  "TAG LATIN CAPITAL LETTER P\0"
  "TAG LATIN CAPITAL LETTER Q\0"
  "TAG LATIN CAPITAL LETTER R\0"
  "TAG LATIN CAPITAL LETTER S\0"
  "TAG LATIN CAPITAL LETTER T\0"
  "TAG LATIN CAPITAL LETTER U\0"
  "TAG LATIN CAPITAL LETTER V\0"
  "TAG LATIN CAPITAL LETTER W\0"
  "TAG LATIN CAPITAL LETTER X\0"
  "TAG LATIN CAPITAL LETTER Y\0"
  "TAG LATIN CAPITAL LETTER Z\0"
  "TAG LATIN SMALL LETTER A\0"
  "TAG LATIN SMALL LETTER B\0"
  "TAG LATIN SMALL LETTER C\0"
  "TAG LATIN SMALL LETTER D\0"
  "TAG LATIN SMALL LETTER E\0"
  "TAG LATIN SMALL LETTER F\0"
  "TAG LATIN SMALL LETTER G\0"
  "TAG LATIN SMALL LETTER H\0"
  "TAG LATIN SMALL LETTER I\0"
  "TAG LATIN SMALL LETTER J\0"
  "TAG LATIN SMALL LETTER K\0"
  "TAG LATIN SMALL LETTER L\0"
  "TAG LATIN SMALL LETTER M\0"
  "TAG LATIN SMALL LETTER N\0"
  "TAG LATIN SMALL LETTER O\0"
  "TAG LATIN SMALL LETTER P\0"
  "TAG LATIN SMALL LETTER Q\0"
  "TAG LATIN SMALL LETTER R\0"
  "TAG LATIN SMALL LETTER S\0"
  "TAG LATIN SMALL LETTER T\0"
  "TAG LATIN SMALL LETTER U\0"
  "TAG LATIN SMALL LETTER V\0"
  "TAG LATIN SMALL LETTER W\0"
  "TAG LATIN SMALL LETTER X\0"
  "TAG LATIN SMALL LETTER Y\0"
  "TAG LATIN SMALL LETTER Z\0"
  "TAG LEFT CURLY BRACKET\0"
  "TAG LEFT PARENTHESIS\0"
  "TAG LEFT SQUARE BRACKET\0"
  "TAG LESS-THAN SIGN\0"
  "TAG LOW LINE\0"
  "TAG NUMBER SIGN\0"
  "TAG PERCENT SIGN\0"
  "TAG PLUS SIGN\0"
  "TAG QUESTION MARK\0"
  "TAG QUOTATION MARK\0"
  "TAG REVERSE SOLIDUS\0"
  "TAG RIGHT CURLY BRACKET\0"
  "TAG RIGHT PARENTHESIS\0"
  "TAG RIGHT SQUARE BRACKET\0"
  "TAG SEMICOLON\0"
  "TAG SOLIDUS\0"
  "TAG SPACE\0"
  "TAG TILDE\0"
  "TAG VERTICAL LINE\0"
  "TAGALOG LETTER A\0"
  "TAGALOG LETTER ARCHAIC RA\0"
  "TAGALOG LETTER BA\0"
  "TAGALOG LETTER DA\0"
  "TAGALOG LETTER GA\0"
  "TAGALOG LETTER HA\0"
  "TAGALOG LETTER I\0"
  "TAGALOG LETTER KA\0"
  "TAGALOG LETTER LA\0"
  "TAGALOG LETTER MA\0"
  "TAGALOG LETTER NA\0"
  "TAGALOG LETTER NGA\0"
  "TAGALOG LETTER PA\0"
  "TAGALOG LETTER RA\0"
  "TAGALOG LETTER SA\0"
  "TAGALOG LETTER TA\0"
  "TAGALOG LETTER U\0"
  "TAGALOG LETTER WA\0"
  "TAGALOG LETTER YA\0"
  "TAGALOG SIGN PAMUDPOD\0"
  "TAGALOG SIGN VIRAMA\0"
  "TAGALOG VOWEL SIGN I\0"
  "TAGALOG VOWEL SIGN U\0"
  "TAGBANWA LETTER A\0"
  "TAGBANWA LETTER BA\0"
  "TAGBANWA LETTER DA\0"
  "TAGBANWA LETTER GA\0"
  "TAGBANWA LETTER I\0"
  "TAGBANWA LETTER KA\0"
  "TAGBANWA LETTER LA\0"
  "TAGBANWA LETTER MA\0"
  "TAGBANWA LETTER NA\0"
  "TAGBANWA LETTER NGA\0"
  "TAGBANWA LETTER PA\0"
  "TAGBANWA LETTER SA\0"
  "TAGBANWA LETTER TA\0"
  "TAGBANWA LETTER U\0"
  "TAGBANWA LETTER WA\0"
  "TAGBANWA LETTER YA\0"
  "TAGBANWA VOWEL SIGN I\0"
  "TAGBANWA VOWEL SIGN U\0"
  "TAI LE LETTER A\0"
  "TAI LE LETTER AI\0"
  "TAI LE LETTER AUE\0"
  "TAI LE LETTER E\0"
  "TAI LE LETTER EE\0"
  "TAI LE LETTER EH\0"
  "TAI LE LETTER FA\0"
  "TAI LE LETTER HA\0"
  "TAI LE LETTER I\0"
  "TAI LE LETTER KA\0"
  "TAI LE LETTER KHA\0"
  "TAI LE LETTER LA\0"
  "TAI LE LETTER MA\0"
  "TAI LE LETTER NA\0"
  "TAI LE LETTER NGA\0"
  "TAI LE LETTER O\0"
  "TAI LE LETTER OO\0"
  "TAI LE LETTER PA\0"
  "TAI LE LETTER PHA\0"
  "TAI LE LETTER QA\0"
  "TAI LE LETTER SA\0"
  "TAI LE LETTER TA\0"
  "TAI LE LETTER THA\0"
  "TAI LE LETTER TONE-2\0"
  "TAI LE LETTER TONE-3\0"
  "TAI LE LETTER TONE-4\0"
  "TAI LE LETTER TONE-5\0"
  "TAI LE LETTER TONE-6\0"
  "TAI LE LETTER TSA\0"
  "TAI LE LETTER TSHA\0"
  "TAI LE LETTER U\0"
  "TAI LE LETTER UE\0"
  "TAI LE LETTER VA\0"
  "TAI LE LETTER XA\0"
  "TAI LE LETTER YA\0"
  "TAI THAM COMBINING CRYPTOGRAMMIC DOT\0"
  "TAI THAM CONSONANT SIGN BA\0"
  "TAI THAM CONSONANT SIGN FINAL NGA\0"
  "TAI THAM CONSONANT SIGN HIGH RATHA OR LOW PA\0"
  "TAI THAM CONSONANT SIGN LA TANG LAI\0"
  "TAI THAM CONSONANT SIGN LOW PA\0"
  "TAI THAM CONSONANT SIGN MA\0"
  "TAI THAM CONSONANT SIGN MEDIAL LA\0"
  "TAI THAM CONSONANT SIGN MEDIAL RA\0"
  "TAI THAM CONSONANT SIGN SA\0"
  "TAI THAM HORA DIGIT EIGHT\0"
  "TAI THAM HORA DIGIT FIVE\0"
  "TAI THAM HORA DIGIT FOUR\0"
  "TAI THAM HORA DIGIT NINE\0"
  "TAI THAM HORA DIGIT ONE\0"
  "TAI THAM HORA DIGIT SEVEN\0"
  "TAI THAM HORA DIGIT SIX\0"
  "TAI THAM HORA DIGIT THREE\0"
  "TAI THAM HORA DIGIT TWO\0"
  "TAI THAM HORA DIGIT ZERO\0"
  "TAI THAM LETTER A\0"
  "TAI THAM LETTER BA\0"
  "TAI THAM LETTER DA\0"
  "TAI THAM LETTER EE\0"
  "TAI THAM LETTER GREAT SA\0"
  "TAI THAM LETTER HIGH CA\0"
  "TAI THAM LETTER HIGH CHA\0"
  "TAI THAM LETTER HIGH FA\0"
  "TAI THAM LETTER HIGH HA\0"
  "TAI THAM LETTER HIGH KA\0"
  "TAI THAM LETTER HIGH KHA\0"
  "TAI THAM LETTER HIGH KXA\0"
  "TAI THAM LETTER HIGH PA\0"
  "TAI THAM LETTER HIGH PHA\0"
  "TAI THAM LETTER HIGH RATHA\0"
  "TAI THAM LETTER HIGH SA\0"
  "TAI THAM LETTER HIGH SHA\0"
  "TAI THAM LETTER HIGH SSA\0"
  "TAI THAM LETTER HIGH TA\0"
  "TAI THAM LETTER HIGH THA\0"
  "TAI THAM LETTER HIGH YA\0"
  "TAI THAM LETTER I\0"
  "TAI THAM LETTER II\0"
  "TAI THAM LETTER LA\0"
  "TAI THAM LETTER LAE\0"
  "TAI THAM LETTER LLA\0"
  "TAI THAM LETTER LOW CA\0"
  "TAI THAM LETTER LOW CHA\0"
  "TAI THAM LETTER LOW FA\0"
  "TAI THAM LETTER LOW HA\0"
  "TAI THAM LETTER LOW KA\0"
  "TAI THAM LETTER LOW KHA\0"
  "TAI THAM LETTER LOW KXA\0"
  "TAI THAM LETTER LOW PA\0"
  "TAI THAM LETTER LOW PHA\0"
  "TAI THAM LETTER LOW RATHA\0"
  "TAI THAM LETTER LOW SA\0"
  "TAI THAM LETTER LOW TA\0"
  "TAI THAM LETTER LOW THA\0"
  "TAI THAM LETTER LOW YA\0"
  "TAI THAM LETTER LUE\0"
  "TAI THAM LETTER MA\0"
  "TAI THAM LETTER NA\0"
  "TAI THAM LETTER NGA\0"
  "TAI THAM LETTER NYA\0"
  "TAI THAM LETTER OO\0"
  "TAI THAM LETTER RA\0"
  "TAI THAM LETTER RANA\0"
  "TAI THAM LETTER RATA\0"
  "TAI THAM LETTER RUE\0"
  "TAI THAM LETTER U\0"
  "TAI THAM LETTER UU\0"
  "TAI THAM LETTER WA\0"
  "TAI THAM SIGN CAANG\0"
  "TAI THAM SIGN DOKMAI\0"
  "TAI THAM SIGN HANG\0"
  "TAI THAM SIGN HOY\0"
  "TAI THAM SIGN KAAN\0"
  "TAI THAM SIGN KAANKUU\0"
  "TAI THAM SIGN KEOW\0"
  "TAI THAM SIGN KHUEN TONE-3\0"
  "TAI THAM SIGN KHUEN TONE-4\0"
  "TAI THAM SIGN KHUEN TONE-5\0"
  "TAI THAM SIGN KHUEN-LUE KARAN\0"
  "TAI THAM SIGN MAI KANG\0"
  "TAI THAM SIGN MAI KANG LAI\0"
  "TAI THAM SIGN MAI SAM\0"
  "TAI THAM SIGN MAI YAMOK\0"
  "TAI THAM SIGN RA HAAM\0"
  "TAI THAM SIGN REVERSED ROTATED RANA\0"
  "TAI THAM SIGN SAKOT\0"
  "TAI THAM SIGN SATKAAN\0"
  "TAI THAM SIGN SATKAANKUU\0"
  "TAI THAM SIGN SAWAN\0"
  "TAI THAM SIGN TONE-1\0"
  "TAI THAM SIGN TONE-2\0"
  "TAI THAM SIGN WIANG\0"
  "TAI THAM SIGN WIANGWAAK\0"
  "TAI THAM THAM DIGIT EIGHT\0"
  "TAI THAM THAM DIGIT FIVE\0"
  "TAI THAM THAM DIGIT FOUR\0"
  "TAI THAM THAM DIGIT NINE\0"
  "TAI THAM THAM DIGIT ONE\0"
  "TAI THAM THAM DIGIT SEVEN\0"
  "TAI THAM THAM DIGIT SIX\0"
  "TAI THAM THAM DIGIT THREE\0"
  "TAI THAM THAM DIGIT TWO\0"
  "TAI THAM THAM DIGIT ZERO\0"
  "TAI THAM VOWEL SIGN A\0"
  "TAI THAM VOWEL SIGN AA\0"
  "TAI THAM VOWEL SIGN AE\0"
  "TAI THAM VOWEL SIGN AI\0"
  "TAI THAM VOWEL SIGN E\0"
  "TAI THAM VOWEL SIGN I\0"
  "TAI THAM VOWEL SIGN II\0"
  "TAI THAM VOWEL SIGN MAI SAT\0"
  "TAI THAM VOWEL SIGN O\0"
  "TAI THAM VOWEL SIGN OA ABOVE\0"
  "TAI THAM VOWEL SIGN OA BELOW\0"
  "TAI THAM VOWEL SIGN OO\0"
  "TAI THAM VOWEL SIGN OY\0"
  "TAI THAM VOWEL SIGN TALL AA\0"
  "TAI THAM VOWEL SIGN THAM AI\0"
  "TAI THAM VOWEL SIGN U\0"
  "TAI THAM VOWEL SIGN UE\0"
  "TAI THAM VOWEL SIGN UU\0"
  "TAI THAM VOWEL SIGN UUE\0"
  "TAI VIET LETTER HIGH BO\0"
  "TAI VIET LETTER HIGH CHO\0"
  "TAI VIET LETTER HIGH CO\0"
  "TAI VIET LETTER HIGH DO\0"
  "TAI VIET LETTER HIGH FO\0"
  "TAI VIET LETTER HIGH GO\0"
  "TAI VIET LETTER HIGH HO\0"
  "TAI VIET LETTER HIGH KHHO\0"
  "TAI VIET LETTER HIGH KHO\0"
  "TAI VIET LETTER HIGH KO\0"
  "TAI VIET LETTER HIGH LO\0"
  "TAI VIET LETTER HIGH MO\0"
  "TAI VIET LETTER HIGH NGO\0"
  "TAI VIET LETTER HIGH NO\0"
  "TAI VIET LETTER HIGH NYO\0"
  "TAI VIET LETTER HIGH O\0"
  "TAI VIET LETTER HIGH PHO\0"
  "TAI VIET LETTER HIGH PO\0"
  "TAI VIET LETTER HIGH RO\0"
  "TAI VIET LETTER HIGH SO\0"
  "TAI VIET LETTER HIGH THO\0"
  "TAI VIET LETTER HIGH TO\0"
  "TAI VIET LETTER HIGH VO\0"
  "TAI VIET LETTER HIGH YO\0"
  "TAI VIET LETTER LOW BO\0"
  "TAI VIET LETTER LOW CHO\0"
  "TAI VIET LETTER LOW CO\0"
  "TAI VIET LETTER LOW DO\0"
  "TAI VIET LETTER LOW FO\0"
  "TAI VIET LETTER LOW GO\0"
  "TAI VIET LETTER LOW HO\0"
  "TAI VIET LETTER LOW KHHO\0"
  "TAI VIET LETTER LOW KHO\0"
  "TAI VIET LETTER LOW KO\0"
  "TAI VIET LETTER LOW LO\0"
  "TAI VIET LETTER LOW MO\0"
  "TAI VIET LETTER LOW NGO\0"
  "TAI VIET LETTER LOW NO\0"
  "TAI VIET LETTER LOW NYO\0"
  "TAI VIET LETTER LOW O\0"
  "TAI VIET LETTER LOW PHO\0"
  "TAI VIET LETTER LOW PO\0"
  "TAI VIET LETTER LOW RO\0"
  "TAI VIET LETTER LOW SO\0"
  "TAI VIET LETTER LOW THO\0"
  "TAI VIET LETTER LOW TO\0"
  "TAI VIET LETTER LOW VO\0"
  "TAI VIET LETTER LOW YO\0"
  "TAI VIET MAI KANG\0"
  "TAI VIET MAI KHIT\0"
  "TAI VIET SYMBOL HO HOI\0"
  "TAI VIET SYMBOL KOI KOI\0"
  "TAI VIET SYMBOL KON\0"
  "TAI VIET SYMBOL NUENG\0"
  "TAI VIET SYMBOL SAM\0"
  "TAI VIET TONE MAI EK\0"
  "TAI VIET TONE MAI NUENG\0"
  "TAI VIET TONE MAI SONG\0"
  "TAI VIET TONE MAI THO\0"
  "TAI VIET VOWEL AA\0"
  "TAI VIET VOWEL AM\0"
  "TAI VIET VOWEL AN\0"
  "TAI VIET VOWEL AUE\0"
  "TAI VIET VOWEL AY\0"
  "TAI VIET VOWEL E\0"
  "TAI VIET VOWEL I\0"
  "TAI VIET VOWEL IA\0"
  "TAI VIET VOWEL O\0"
  "TAI VIET VOWEL U\0"
  "TAI VIET VOWEL UA\0"
  "TAI VIET VOWEL UE\0"
  "TAI VIET VOWEL UEA\0"
  "TAKEOUT BOX\0"
  "TAKRI ABBREVIATION SIGN\0"
  "TAKRI DIGIT EIGHT\0"
  "TAKRI DIGIT FIVE\0"
  "TAKRI DIGIT FOUR\0"
  "TAKRI DIGIT NINE\0"
  "TAKRI DIGIT ONE\0"
  "TAKRI DIGIT SEVEN\0"
  "TAKRI DIGIT SIX\0"
  "TAKRI DIGIT THREE\0"
  "TAKRI DIGIT TWO\0"
  "TAKRI DIGIT ZERO\0"
  "TAKRI LETTER A\0"
  "TAKRI LETTER AA\0"
  "TAKRI LETTER AI\0"
  "TAKRI LETTER ARCHAIC KHA\0"
  "TAKRI LETTER AU\0"
  "TAKRI LETTER BA\0"
  "TAKRI LETTER BHA\0"
  "TAKRI LETTER CA\0"
  "TAKRI LETTER CHA\0"
  "TAKRI LETTER DA\0"
  "TAKRI LETTER DDA\0"
  "TAKRI LETTER DDHA\0"
  "TAKRI LETTER DHA\0"
  "TAKRI LETTER E\0"
  "TAKRI LETTER GA\0"
  "TAKRI LETTER GHA\0"
  "TAKRI LETTER HA\0"
  "TAKRI LETTER I\0"
  "TAKRI LETTER II\0"
  "TAKRI LETTER JA\0"
  "TAKRI LETTER JHA\0"
  "TAKRI LETTER KA\0"
  "TAKRI LETTER KHA\0"
  "TAKRI LETTER LA\0"
  "TAKRI LETTER MA\0"
  "TAKRI LETTER NA\0"
  "TAKRI LETTER NGA\0"
  "TAKRI LETTER NNA\0"
  "TAKRI LETTER NYA\0"
  "TAKRI LETTER O\0"
  "TAKRI LETTER PA\0"
  "TAKRI LETTER PHA\0"
  "TAKRI LETTER RA\0"
  "TAKRI LETTER RRA\0"
  "TAKRI LETTER SA\0"
  "TAKRI LETTER SHA\0"
  "TAKRI LETTER TA\0"
  "TAKRI LETTER THA\0"
  "TAKRI LETTER TTA\0"
  "TAKRI LETTER TTHA\0"
  "TAKRI LETTER U\0"
  "TAKRI LETTER UU\0"
  "TAKRI LETTER VA\0"
  "TAKRI LETTER YA\0"
  "TAKRI SIGN ANUSVARA\0"
  "TAKRI SIGN NUKTA\0"
  "TAKRI SIGN VIRAMA\0"
  "TAKRI SIGN VISARGA\0"
  "TAKRI VOWEL SIGN AA\0"
  "TAKRI VOWEL SIGN AI\0"
  "TAKRI VOWEL SIGN AU\0"
  "TAKRI VOWEL SIGN E\0"
  "TAKRI VOWEL SIGN I\0"
  "TAKRI VOWEL SIGN II\0"
  "TAKRI VOWEL SIGN O\0"
  "TAKRI VOWEL SIGN U\0"
  "TAKRI VOWEL SIGN UU\0"
  "TALLY MARK FIVE\0"
  "TALLY MARK ONE\0"
  "TAMALE\0"
  "TAMIL AND ODD SIGN\0"
  "TAMIL AS ABOVE SIGN\0"
  "TAMIL AU LENGTH MARK\0"
  "TAMIL CREDIT SIGN\0"
  "TAMIL CURRENT SIGN\0"
  "TAMIL DAY SIGN\0"
  "TAMIL DEBIT SIGN\0"
  "TAMIL DIGIT EIGHT\0"
  "TAMIL DIGIT FIVE\0"
  "TAMIL DIGIT FOUR\0"
  "TAMIL DIGIT NINE\0"
  "TAMIL DIGIT ONE\0"
  "TAMIL DIGIT SEVEN\0"
  "TAMIL DIGIT SIX\0"
  "TAMIL DIGIT THREE\0"
  "TAMIL DIGIT TWO\0"
  "TAMIL DIGIT ZERO\0"
  "TAMIL DRY CULTIVATION SIGN\0"
  "TAMIL FRACTION DOWNSCALING FACTOR KIIZH\0"
  "TAMIL FRACTION ONE EIGHTH\0"
  "TAMIL FRACTION ONE EIGHTIETH\0"
  "TAMIL FRACTION ONE FIFTH\0"
  "TAMIL FRACTION ONE FORTIETH\0"
  "TAMIL FRACTION ONE HALF-1\0"
  "TAMIL FRACTION ONE HALF-2\0"
  "TAMIL FRACTION ONE ONE-HUNDRED-AND-SIXTIETH\0"
  "TAMIL FRACTION ONE QUARTER\0"
  "TAMIL FRACTION ONE SIXTEENTH-1\0"
  "TAMIL FRACTION ONE SIXTEENTH-2\0"
  "TAMIL FRACTION ONE SIXTY-FOURTH\0"
  "TAMIL FRACTION ONE TENTH\0"
  "TAMIL FRACTION ONE THIRTY-SECOND\0"
  "TAMIL FRACTION ONE THREE-HUNDRED-AND-TWENTIETH\0"
  "TAMIL FRACTION ONE TWENTIETH\0"
  "TAMIL FRACTION THREE EIGHTIETHS\0"
  "TAMIL FRACTION THREE QUARTERS\0"
  "TAMIL FRACTION THREE SIXTEENTHS\0"
  "TAMIL FRACTION THREE SIXTY-FOURTHS\0"
  "TAMIL FRACTION THREE TWENTIETHS\0"
  "TAMIL IN POSSESSION SIGN\0"
  "TAMIL LAND SIGN\0"
  "TAMIL LETTER A\0"
  "TAMIL LETTER AA\0"
  "TAMIL LETTER AI\0"
  "TAMIL LETTER AU\0"
  "TAMIL LETTER CA\0"
  "TAMIL LETTER E\0"
  "TAMIL LETTER EE\0"
  "TAMIL LETTER HA\0"
  "TAMIL LETTER I\0"
  "TAMIL LETTER II\0"
  "TAMIL LETTER JA\0"
  "TAMIL LETTER KA\0"
  "TAMIL LETTER LA\0"
  "TAMIL LETTER LLA\0"
  "TAMIL LETTER LLLA\0"
  "TAMIL LETTER MA\0"
  "TAMIL LETTER NA\0"
  "TAMIL LETTER NGA\0"
  "TAMIL LETTER NNA\0"
  "TAMIL LETTER NNNA\0"
  "TAMIL LETTER NYA\0"
  "TAMIL LETTER O\0"
  "TAMIL LETTER OO\0"
  "TAMIL LETTER PA\0"
  "TAMIL LETTER RA\0"
  "TAMIL LETTER RRA\0"
  "TAMIL LETTER SA\0"
  "TAMIL LETTER SHA\0"
  "TAMIL LETTER SSA\0"
  "TAMIL LETTER TA\0"
  "TAMIL LETTER TTA\0"
  "TAMIL LETTER U\0"
  "TAMIL LETTER UU\0"
  "TAMIL LETTER VA\0"
  "TAMIL LETTER YA\0"
  "TAMIL MONTH SIGN\0"
  "TAMIL NUMBER ONE HUNDRED\0"
  "TAMIL NUMBER ONE THOUSAND\0"
  "TAMIL NUMBER SIGN\0"
  "TAMIL NUMBER TEN\0"
  "TAMIL OM\0"
  "TAMIL PUNCTUATION END OF TEXT\0"
  "TAMIL RUPEE SIGN\0"
  "TAMIL SALT PAN SIGN\0"
  "TAMIL SIGN AAZHAAKKU\0"
  "TAMIL SIGN ANUSVARA\0"
  "TAMIL SIGN CEVITU\0"
  "TAMIL SIGN KAACU\0"
  "TAMIL SIGN KURUNI\0"
  "TAMIL SIGN KUZHI\0"
  "TAMIL SIGN MUKKURUNI\0"
  "TAMIL SIGN MUTHALIYA\0"
  "TAMIL SIGN MUUVUZHAKKU\0"
  "TAMIL SIGN NEL\0"
  "TAMIL SIGN PAARAM\0"
  "TAMIL SIGN PANAM\0"
  "TAMIL SIGN PATHAKKU\0"
  "TAMIL SIGN PON\0"
  "TAMIL SIGN UZHAKKU\0"
  "TAMIL SIGN VAKAIYARAA\0"
  "TAMIL SIGN VARAAKAN\0"
  "TAMIL SIGN VELI\0"
  "TAMIL SIGN VIRAMA\0"
  "TAMIL SIGN VISARGA\0"
  "TAMIL SPENT SIGN\0"
  "TAMIL STARTING FROM SIGN\0"
  "TAMIL TOTAL SIGN\0"
  "TAMIL TRADITIONAL CREDIT SIGN\0"
  "TAMIL TRADITIONAL NUMBER SIGN\0"
  "TAMIL VOWEL SIGN AA\0"
  "TAMIL VOWEL SIGN AI\0"
  "TAMIL VOWEL SIGN AU\0"
  "TAMIL VOWEL SIGN E\0"
  "TAMIL VOWEL SIGN EE\0"
  "TAMIL VOWEL SIGN I\0"
  "TAMIL VOWEL SIGN II\0"
  "TAMIL VOWEL SIGN O\0"
  "TAMIL VOWEL SIGN OO\0"
  "TAMIL VOWEL SIGN U\0"
  "TAMIL VOWEL SIGN UU\0"
  "TAMIL WET CULTIVATION SIGN\0"
  "TAMIL YEAR SIGN\0"
  "TANABATA TREE\0"
  "TANGERINE\0"
  "TANGSA DIGIT EIGHT\0"
  "TANGSA DIGIT FIVE\0"
  "TANGSA DIGIT FOUR\0"
  "TANGSA DIGIT NINE\0"
  "TANGSA DIGIT ONE\0"
  "TANGSA DIGIT SEVEN\0"
  "TANGSA DIGIT SIX\0"
  "TANGSA DIGIT THREE\0"
  "TANGSA DIGIT TWO\0"
  "TANGSA DIGIT ZERO\0"
  "TANGSA LETTER AC\0"
  "TANGSA LETTER AQ\0"
  "TANGSA LETTER AWC\0"
  "TANGSA LETTER AWQ\0"
  "TANGSA LETTER AWX\0"
  "TANGSA LETTER AWZ\0"
  "TANGSA LETTER AX\0"
  "TANGSA LETTER AZ\0"
  "TANGSA LETTER BA\0"
  "TANGSA LETTER CA\0"
  "TANGSA LETTER CHA\0"
  "TANGSA LETTER DA\0"
  "TANGSA LETTER DHA\0"
  "TANGSA LETTER EC\0"
  "TANGSA LETTER EQ\0"
  "TANGSA LETTER EX\0"
  "TANGSA LETTER EZ\0"
  "TANGSA LETTER FA\0"
  "TANGSA LETTER FINAL NG\0"
  "TANGSA LETTER GA\0"
  "TANGSA LETTER GHA\0"
  "TANGSA LETTER HA\0"
  "TANGSA LETTER HTA\0"
  "TANGSA LETTER HTTA\0"
  "TANGSA LETTER IC\0"
  "TANGSA LETTER IQ\0"
  "TANGSA LETTER IX\0"
  "TANGSA LETTER IZ\0"
  "TANGSA LETTER KA\0"
  "TANGSA LETTER KHA\0"
  "TANGSA LETTER LA\0"
  "TANGSA LETTER LONG UEX\0"
  "TANGSA LETTER MA\0"
  "TANGSA LETTER MC\0"
  "TANGSA LETTER MQ\0"
  "TANGSA LETTER MX\0"
  "TANGSA LETTER MZ\0"
  "TANGSA LETTER NA\0"
  "TANGSA LETTER NGA\0"
  "TANGSA LETTER NHA\0"
  "TANGSA LETTER NYA\0"
  "TANGSA LETTER OC\0"
  "TANGSA LETTER OQ\0"
  "TANGSA LETTER OX\0"
  "TANGSA LETTER OZ\0"
  "TANGSA LETTER PA\0"
  "TANGSA LETTER PHA\0"
  "TANGSA LETTER RA\0"
  "TANGSA LETTER SA\0"
  "TANGSA LETTER SHA\0"
  "TANGSA LETTER SHORT AWX\0"
  "TANGSA LETTER SHORT UEZ\0"
  "TANGSA LETTER TA\0"
  "TANGSA LETTER THA\0"
  "TANGSA LETTER TSA\0"
  "TANGSA LETTER UC\0"
  "TANGSA LETTER UEC\0"
  "TANGSA LETTER UEQ\0"
  "TANGSA LETTER UEX\0"
  "TANGSA LETTER UEZ\0"
  "TANGSA LETTER UIC\0"
  "TANGSA LETTER UIQ\0"
  "TANGSA LETTER UIUC\0"
  "TANGSA LETTER UIUQ\0"
  "TANGSA LETTER UIUX\0"
  "TANGSA LETTER UIUZ\0"
  "TANGSA LETTER UIX\0"
  "TANGSA LETTER UIZ\0"
  "TANGSA LETTER UQ\0"
  "TANGSA LETTER UX\0"
  "TANGSA LETTER UZ\0"
  "TANGSA LETTER VC\0"
  "TANGSA LETTER VQ\0"
  "TANGSA LETTER VX\0"
  "TANGSA LETTER VZ\0"
  "TANGSA LETTER WA\0"
  "TANGSA LETTER XA\0"
  "TANGSA LETTER YA\0"
  "TANGSA LETTER ZA\0"
  "TANGUT ITERATION MARK\0"
  "TAPE CARTRIDGE\0"
  "TAPE DRIVE\0"
  "TAURUS\0"
  "TAXI\0"
  "TEACUP WITHOUT HANDLE\0"
  "TEAPOT\0"
  "TEAR-OFF CALENDAR\0"
  "TEARDROP-BARBED RIGHTWARDS ARROW\0"
  "TEARDROP-SPOKED ASTERISK\0"
  "TEDDY BEAR\0"
  "TELEPHONE LOCATION SIGN\0"
  "TELEPHONE ON TOP OF MODEM\0"
  "TELEPHONE RECEIVER\0"
  "TELEPHONE RECEIVER WITH PAGE\0"
  "TELEPHONE RECORDER\0"
  "TELEPHONE SIGN\0"
  "TELESCOPE\0"
  "TELEVISION\0"
  "TELUGU AI LENGTH MARK\0"
  "TELUGU DIGIT EIGHT\0"
  "TELUGU DIGIT FIVE\0"
  "TELUGU DIGIT FOUR\0"
  "TELUGU DIGIT NINE\0"
  "TELUGU DIGIT ONE\0"
  "TELUGU DIGIT SEVEN\0"
  "TELUGU DIGIT SIX\0"
  "TELUGU DIGIT THREE\0"
  "TELUGU DIGIT TWO\0"
  "TELUGU DIGIT ZERO\0"
  "TELUGU FRACTION DIGIT ONE FOR EVEN POWERS OF FOUR\0"
  "TELUGU FRACTION DIGIT ONE FOR ODD POWERS OF FOUR\0"
  "TELUGU FRACTION DIGIT THREE FOR EVEN POWERS OF FOUR\0"
  "TELUGU FRACTION DIGIT THREE FOR ODD POWERS OF FOUR\0"
  "TELUGU FRACTION DIGIT TWO FOR EVEN POWERS OF FOUR\0"
  "TELUGU FRACTION DIGIT TWO FOR ODD POWERS OF FOUR\0"
  "TELUGU FRACTION DIGIT ZERO FOR ODD POWERS OF FOUR\0"
  "TELUGU LENGTH MARK\0"
  "TELUGU LETTER A\0"
  "TELUGU LETTER AA\0"
  "TELUGU LETTER AI\0"
  "TELUGU LETTER AU\0"
  "TELUGU LETTER BA\0"
  "TELUGU LETTER BHA\0"
  "TELUGU LETTER CA\0"
  "TELUGU LETTER CHA\0"
  "TELUGU LETTER DA\0"
  "TELUGU LETTER DDA\0"
  "TELUGU LETTER DDHA\0"
  "TELUGU LETTER DHA\0"
  "TELUGU LETTER DZA\0"
  "TELUGU LETTER E\0"
  "TELUGU LETTER EE\0"
  "TELUGU LETTER GA\0"
  "TELUGU LETTER GHA\0"
  "TELUGU LETTER HA\0"
  "TELUGU LETTER I\0"
  "TELUGU LETTER II\0"
  "TELUGU LETTER JA\0"
  "TELUGU LETTER JHA\0"
  "TELUGU LETTER KA\0"
  "TELUGU LETTER KHA\0"
  "TELUGU LETTER LA\0"
  "TELUGU LETTER LLA\0"
  "TELUGU LETTER LLLA\0"
  "TELUGU LETTER MA\0"
  "TELUGU LETTER NA\0"
  "TELUGU LETTER NAKAARA POLLU\0"
  "TELUGU LETTER NGA\0"
  "TELUGU LETTER NNA\0"
  "TELUGU LETTER NYA\0"
  "TELUGU LETTER O\0"
  "TELUGU LETTER OO\0"
  "TELUGU LETTER PA\0"
  "TELUGU LETTER PHA\0"
  "TELUGU LETTER RA\0"
  "TELUGU LETTER RRA\0"
  "TELUGU LETTER RRRA\0"
  "TELUGU LETTER SA\0"
  "TELUGU LETTER SHA\0"
  "TELUGU LETTER SSA\0"
  "TELUGU LETTER TA\0"
  "TELUGU LETTER THA\0"
  "TELUGU LETTER TSA\0"
  "TELUGU LETTER TTA\0"
  "TELUGU LETTER TTHA\0"
  "TELUGU LETTER U\0"
  "TELUGU LETTER UU\0"
  "TELUGU LETTER VA\0"
  "TELUGU LETTER VOCALIC L\0"
  "TELUGU LETTER VOCALIC LL\0"
  "TELUGU LETTER VOCALIC R\0"
  "TELUGU LETTER VOCALIC RR\0"
  "TELUGU LETTER YA\0"
  "TELUGU SIGN ANUSVARA\0"
  "TELUGU SIGN AVAGRAHA\0"
  "TELUGU SIGN CANDRABINDU\0"
  "TELUGU SIGN COMBINING ANUSVARA ABOVE\0"
  "TELUGU SIGN COMBINING CANDRABINDU ABOVE\0"
  "TELUGU SIGN NUKTA\0"
  "TELUGU SIGN SIDDHAM\0"
  "TELUGU SIGN TUUMU\0"
  "TELUGU SIGN VIRAMA\0"
  "TELUGU SIGN VISARGA\0"
  "TELUGU VOWEL SIGN AA\0"
  "TELUGU VOWEL SIGN AI\0"
  "TELUGU VOWEL SIGN AU\0"
  "TELUGU VOWEL SIGN E\0"
  "TELUGU VOWEL SIGN EE\0"
  "TELUGU VOWEL SIGN I\0"
  "TELUGU VOWEL SIGN II\0"
  "TELUGU VOWEL SIGN O\0"
  "TELUGU VOWEL SIGN OO\0"
  "TELUGU VOWEL SIGN U\0"
  "TELUGU VOWEL SIGN UU\0"
  "TELUGU VOWEL SIGN VOCALIC L\0"
  "TELUGU VOWEL SIGN VOCALIC LL\0"
  "TELUGU VOWEL SIGN VOCALIC R\0"
  "TELUGU VOWEL SIGN VOCALIC RR\0"
  "TENGE SIGN\0"
  "TENNIS RACQUET AND BALL\0"
  "TENT\0"
  "TEST TUBE\0"
  "TETRAGRAM FOR ACCUMULATION\0"
  "TETRAGRAM FOR ADVANCE\0"
  "TETRAGRAM FOR AGGRAVATION\0"
  "TETRAGRAM FOR ASCENT\0"
  "TETRAGRAM FOR BARRIER\0"
  "TETRAGRAM FOR BOLD RESOLUTION\0"
  "TETRAGRAM FOR BRANCHING OUT\0"
  "TETRAGRAM FOR CENTRE\0"
  "TETRAGRAM FOR CHANGE\0"
  "TETRAGRAM FOR CLOSED MOUTH\0"
  "TETRAGRAM FOR CLOSENESS\0"
  "TETRAGRAM FOR CLOSURE\0"
  "TETRAGRAM FOR COMPLETION\0"
  "TETRAGRAM FOR COMPLIANCE\0"
  "TETRAGRAM FOR CONSTANCY\0"
  "TETRAGRAM FOR CONTACT\0"
  "TETRAGRAM FOR CONTENTION\0"
  "TETRAGRAM FOR CONTRARIETY\0"
  "TETRAGRAM FOR DARKENING\0"
  "TETRAGRAM FOR DECISIVENESS\0"
  "TETRAGRAM FOR DEFECTIVENESS OR DISTORTION\0"
  "TETRAGRAM FOR DEPARTURE\0"
  "TETRAGRAM FOR DIFFICULTIES\0"
  "TETRAGRAM FOR DIMINISHMENT\0"
  "TETRAGRAM FOR DIMMING\0"
  "TETRAGRAM FOR DIVERGENCE\0"
  "TETRAGRAM FOR DOUBT\0"
  "TETRAGRAM FOR DUTIES\0"
  "TETRAGRAM FOR EASE\0"
  "TETRAGRAM FOR EMBELLISHMENT\0"
  "TETRAGRAM FOR ENCOUNTERS\0"
  "TETRAGRAM FOR ENDEAVOUR\0"
  "TETRAGRAM FOR ENLARGEMENT\0"
  "TETRAGRAM FOR ETERNITY\0"
  "TETRAGRAM FOR EXHAUSTION\0"
  "TETRAGRAM FOR FAILURE\0"
  "TETRAGRAM FOR FLIGHT\0"
  "TETRAGRAM FOR FOLLOWING\0"
  "TETRAGRAM FOR FOSTERING\0"
  "TETRAGRAM FOR FULL CIRCLE\0"
  "TETRAGRAM FOR FULLNESS\0"
  "TETRAGRAM FOR GATHERING\0"
  "TETRAGRAM FOR GATHERING IN\0"
  "TETRAGRAM FOR GOING TO MEET\0"
  "TETRAGRAM FOR GREATNESS\0"
  "TETRAGRAM FOR GUARDEDNESS\0"
  "TETRAGRAM FOR HARDNESS\0"
  "TETRAGRAM FOR HOLDING BACK\0"
  "TETRAGRAM FOR INCREASE\0"
  "TETRAGRAM FOR INNER\0"
  "TETRAGRAM FOR JOY\0"
  "TETRAGRAM FOR KEEPING SMALL\0"
  "TETRAGRAM FOR KINSHIP\0"
  "TETRAGRAM FOR LABOURING\0"
  "TETRAGRAM FOR LAW OR MODEL\0"
  "TETRAGRAM FOR LEGION\0"
  "TETRAGRAM FOR MASSING\0"
  "TETRAGRAM FOR MEASURE\0"
  "TETRAGRAM FOR MIRED\0"
  "TETRAGRAM FOR ON THE VERGE\0"
  "TETRAGRAM FOR OPPOSITION\0"
  "TETRAGRAM FOR PACKING\0"
  "TETRAGRAM FOR PATTERN\0"
  "TETRAGRAM FOR PENETRATION\0"
  "TETRAGRAM FOR PURITY\0"
  "TETRAGRAM FOR REACH\0"
  "TETRAGRAM FOR RELEASE\0"
  "TETRAGRAM FOR RESIDENCE\0"
  "TETRAGRAM FOR RESISTANCE\0"
  "TETRAGRAM FOR RESPONSE\0"
  "TETRAGRAM FOR RITUAL\0"
  "TETRAGRAM FOR SEVERANCE\0"
  "TETRAGRAM FOR SINKING\0"
  "TETRAGRAM FOR STOPPAGE\0"
  "TETRAGRAM FOR STOVE\0"
  "TETRAGRAM FOR STRENGTH\0"
  "TETRAGRAM FOR UNITY\0"
  "TETRAGRAM FOR VASTNESS OR WASTING\0"
  "TETRAGRAM FOR WAITING\0"
  "TETRAGRAM FOR WATCH\0"
  "TETRAGRAM FOR YOUTHFULNESS\0"
  "THAANA AABAAFILI\0"
  "THAANA ABAFILI\0"
  "THAANA EBEFILI\0"
  "THAANA EEBEEFILI\0"
  "THAANA EYBEYFILI\0"
  "THAANA IBIFILI\0"
  "THAANA LETTER AINU\0"
  "THAANA LETTER ALIFU\0"
  "THAANA LETTER BAA\0"
  "THAANA LETTER CHAVIYANI\0"
  "THAANA LETTER DAADHU\0"
  "THAANA LETTER DAVIYANI\0"
  "THAANA LETTER DHAALU\0"
  "THAANA LETTER FAAFU\0"
  "THAANA LETTER GAAFU\0"
  "THAANA LETTER GHAINU\0"
  "THAANA LETTER GNAVIYANI\0"
  "THAANA LETTER HAA\0"
  "THAANA LETTER HHAA\0"
  "THAANA LETTER JAVIYANI\0"
  "THAANA LETTER KAAFU\0"
  "THAANA LETTER KHAA\0"
  "THAANA LETTER LAAMU\0"
  "THAANA LETTER LHAVIYANI\0"
  "THAANA LETTER MEEMU\0"
  "THAANA LETTER NAA\0"
  "THAANA LETTER NOONU\0"
  "THAANA LETTER PAVIYANI\0"
  "THAANA LETTER QAAFU\0"
  "THAANA LETTER RAA\0"
  "THAANA LETTER SAADHU\0"
  "THAANA LETTER SEENU\0"
  "THAANA LETTER SHAVIYANI\0"
  "THAANA LETTER SHEENU\0"
  "THAANA LETTER TAVIYANI\0"
  "THAANA LETTER THAA\0"
  "THAANA LETTER THAALU\0"
  "THAANA LETTER TO\0"
  "THAANA LETTER TTAA\0"
  "THAANA LETTER VAAVU\0"
  "THAANA LETTER WAAVU\0"
  "THAANA LETTER YAA\0"
  "THAANA LETTER ZAA\0"
  "THAANA LETTER ZAVIYANI\0"
  "THAANA LETTER ZO\0"
  "THAANA OABOAFILI\0"
  "THAANA OBOFILI\0"
  "THAANA OOBOOFILI\0"
  "THAANA SUKUN\0"
  "THAANA UBUFILI\0"
  "THAI CHARACTER ANGKHANKHU\0"
  "THAI CHARACTER BO BAIMAI\0"
  "THAI CHARACTER CHO CHAN\0"
  "THAI CHARACTER CHO CHANG\0"
  "THAI CHARACTER CHO CHING\0"
  "THAI CHARACTER CHO CHOE\0"
  "THAI CHARACTER DO CHADA\0"
  "THAI CHARACTER DO DEK\0"
  "THAI CHARACTER FO FA\0"
  "THAI CHARACTER FO FAN\0"
  "THAI CHARACTER FONGMAN\0"
  "THAI CHARACTER HO HIP\0"
  "THAI CHARACTER HO NOKHUK\0"
  "THAI CHARACTER KHO KHAI\0"
  "THAI CHARACTER KHO KHON\0"
  "THAI CHARACTER KHO KHUAT\0"
  "THAI CHARACTER KHO KHWAI\0"
  "THAI CHARACTER KHO RAKHANG\0"
  "THAI CHARACTER KHOMUT\0"
  "THAI CHARACTER KO KAI\0"
  "THAI CHARACTER LAKKHANGYAO\0"
  "THAI CHARACTER LO CHULA\0"
  "THAI CHARACTER LO LING\0"
  "THAI CHARACTER LU\0"
  "THAI CHARACTER MAI CHATTAWA\0"
  "THAI CHARACTER MAI EK\0"
  "THAI CHARACTER MAI HAN-AKAT\0"
  "THAI CHARACTER MAI THO\0"
  "THAI CHARACTER MAI TRI\0"
  "THAI CHARACTER MAITAIKHU\0"
  "THAI CHARACTER MAIYAMOK\0"
  "THAI CHARACTER MO MA\0"
  "THAI CHARACTER NGO NGU\0"
  "THAI CHARACTER NIKHAHIT\0"
  "THAI CHARACTER NO NEN\0"
  "THAI CHARACTER NO NU\0"
  "THAI CHARACTER O ANG\0"
  "THAI CHARACTER PAIYANNOI\0"
  "THAI CHARACTER PHINTHU\0"
  "THAI CHARACTER PHO PHAN\0"
  "THAI CHARACTER PHO PHUNG\0"
  "THAI CHARACTER PHO SAMPHAO\0"
  "THAI CHARACTER PO PLA\0"
  "THAI CHARACTER RO RUA\0"
  "THAI CHARACTER RU\0"
  "THAI CHARACTER SARA A\0"
  "THAI CHARACTER SARA AA\0"
  "THAI CHARACTER SARA AE\0"
  "THAI CHARACTER SARA AI MAIMALAI\0"
  "THAI CHARACTER SARA AI MAIMUAN\0"
  "THAI CHARACTER SARA AM\0"
  "THAI CHARACTER SARA E\0"
  "THAI CHARACTER SARA I\0"
  "THAI CHARACTER SARA II\0"
  "THAI CHARACTER SARA O\0"
  "THAI CHARACTER SARA U\0"
  "THAI CHARACTER SARA UE\0"
  "THAI CHARACTER SARA UEE\0"
  "THAI CHARACTER SARA UU\0"
  "THAI CHARACTER SO RUSI\0"
  "THAI CHARACTER SO SALA\0"
  "THAI CHARACTER SO SO\0"
  "THAI CHARACTER SO SUA\0"
  "THAI CHARACTER THANTHAKHAT\0"
  "THAI CHARACTER THO NANGMONTHO\0"
  "THAI CHARACTER THO PHUTHAO\0"
  "THAI CHARACTER THO THAHAN\0"
  "THAI CHARACTER THO THAN\0"
  "THAI CHARACTER THO THONG\0"
  "THAI CHARACTER THO THUNG\0"
  "THAI CHARACTER TO PATAK\0"
  "THAI CHARACTER TO TAO\0"
  "THAI CHARACTER WO WAEN\0"
  "THAI CHARACTER YAMAKKAN\0"
  "THAI CHARACTER YO YAK\0"
  "THAI CHARACTER YO YING\0"
  "THAI CURRENCY SYMBOL BAHT\0"
  "THAI DIGIT EIGHT\0"
  "THAI DIGIT FIVE\0"
  "THAI DIGIT FOUR\0"
  "THAI DIGIT NINE\0"
  "THAI DIGIT ONE\0"
  "THAI DIGIT SEVEN\0"
  "THAI DIGIT SIX\0"
  "THAI DIGIT THREE\0"
  "THAI DIGIT TWO\0"
  "THAI DIGIT ZERO\0"
  "THERE DOES NOT EXIST\0"
  "THERE EXISTS\0"
  "THEREFORE\0"
  "THERMODYNAMIC\0"
  "THERMOMETER\0"
  "THIN GREEK CROSS\0"
  "THIN SALTIRE\0"
  "THIN SPACE\0"
  "THINKING FACE\0"
  "THIRD PLACE MEDAL\0"
  "THONG SANDAL\0"
  "THOUGHT BALLOON\0"
  "THREE BUTTON MOUSE\0"
  "THREE CONSECUTIVE EQUALS SIGNS\0"
  "THREE DIMENSIONAL ANGLE\0"
  "THREE DOT PUNCTUATION\0"
  "THREE LEFTWARDS ARROWS\0"
  "THREE LINES CONVERGING LEFT\0"
  "THREE LINES CONVERGING RIGHT\0"
  "THREE NETWORKED COMPUTERS\0"
  "THREE POINTED BLACK STAR\0"
  "THREE RAYS ABOVE\0"
  "THREE RAYS BELOW\0"
  "THREE RAYS LEFT\0"
  "THREE RAYS RIGHT\0"
  "THREE RIGHTWARDS ARROWS\0"
  "THREE SPEECH BUBBLES\0"
  "THREE-D BOTTOM-LIGHTED RIGHTWARDS ARROWHEAD\0"
  "THREE-D LEFT-LIGHTED DOWNWARDS EQUILATERAL ARROWHEAD\0"
  "THREE-D RIGHT-LIGHTED UPWARDS EQUILATERAL ARROWHEAD\0"
  "THREE-D TOP-LIGHTED LEFTWARDS EQUILATERAL ARROWHEAD\0"
  "THREE-D TOP-LIGHTED RIGHTWARDS ARROWHEAD\0"
  "THREE-D TOP-LIGHTED RIGHTWARDS EQUILATERAL ARROWHEAD\0"
  "THREE-EM DASH\0"
  "THREE-PER-EM SPACE\0"
  "THUMBS DOWN SIGN\0"
  "THUMBS UP SIGN\0"
  "THUNDER CLOUD AND RAIN\0"
  "THUNDERSTORM\0"
  "TIBETAN ASTROLOGICAL SIGN -KHYUD PA\0"
  "TIBETAN ASTROLOGICAL SIGN SDONG TSHUGS\0"
  "TIBETAN ASTROLOGICAL SIGN SGRA GCAN -CHAR RTAGS\0"
  "TIBETAN CANTILLATION SIGN CANG TE-U\0"
  "TIBETAN CANTILLATION SIGN HEAVY BEAT\0"
  "TIBETAN CANTILLATION SIGN LIGHT BEAT\0"
  "TIBETAN CANTILLATION SIGN SBUB -CHAL\0"
  "TIBETAN DIGIT EIGHT\0"
  "TIBETAN DIGIT FIVE\0"
  "TIBETAN DIGIT FOUR\0"
  "TIBETAN DIGIT HALF EIGHT\0"
  "TIBETAN DIGIT HALF FIVE\0"
  "TIBETAN DIGIT HALF FOUR\0"
  "TIBETAN DIGIT HALF NINE\0"
  "TIBETAN DIGIT HALF ONE\0"
  "TIBETAN DIGIT HALF SEVEN\0"
  "TIBETAN DIGIT HALF SIX\0"
  "TIBETAN DIGIT HALF THREE\0"
  "TIBETAN DIGIT HALF TWO\0"
  "TIBETAN DIGIT HALF ZERO\0"
  "TIBETAN DIGIT NINE\0"
  "TIBETAN DIGIT ONE\0"
  "TIBETAN DIGIT SEVEN\0"
  "TIBETAN DIGIT SIX\0"
  "TIBETAN DIGIT THREE\0"
  "TIBETAN DIGIT TWO\0"
  "TIBETAN DIGIT ZERO\0"
  "TIBETAN KU RU KHA\0"
  "TIBETAN KU RU KHA BZHI MIG CAN\0"
  "TIBETAN LETTER -A\0"
  "TIBETAN LETTER A\0"
  "TIBETAN LETTER BA\0"
  "TIBETAN LETTER BHA\0"
  "TIBETAN LETTER CA\0"
  "TIBETAN LETTER CHA\0"
  "TIBETAN LETTER DA\0"
  "TIBETAN LETTER DDA\0"
  "TIBETAN LETTER DDHA\0"
  "TIBETAN LETTER DHA\0"
  "TIBETAN LETTER DZA\0"
  "TIBETAN LETTER DZHA\0"
  "TIBETAN LETTER FIXED-FORM RA\0"
  "TIBETAN LETTER GA\0"
  "TIBETAN LETTER GHA\0"
  "TIBETAN LETTER HA\0"
  "TIBETAN LETTER JA\0"
  "TIBETAN LETTER KA\0"
  "TIBETAN LETTER KHA\0"
  "TIBETAN LETTER KKA\0"
  "TIBETAN LETTER KSSA\0"
  "TIBETAN LETTER LA\0"
  "TIBETAN LETTER MA\0"
  "TIBETAN LETTER NA\0"
  "TIBETAN LETTER NGA\0"
  "TIBETAN LETTER NNA\0"
  "TIBETAN LETTER NYA\0"
  "TIBETAN LETTER PA\0"
  "TIBETAN LETTER PHA\0"
  "TIBETAN LETTER RA\0"
  "TIBETAN LETTER RRA\0"
  "TIBETAN LETTER SA\0"
  "TIBETAN LETTER SHA\0"
  "TIBETAN LETTER SSA\0"
  "TIBETAN LETTER TA\0"
  "TIBETAN LETTER THA\0"
  "TIBETAN LETTER TSA\0"
  "TIBETAN LETTER TSHA\0"
  "TIBETAN LETTER TTA\0"
  "TIBETAN LETTER TTHA\0"
  "TIBETAN LETTER WA\0"
  "TIBETAN LETTER YA\0"
  "TIBETAN LETTER ZA\0"
  "TIBETAN LETTER ZHA\0"
  "TIBETAN LOGOTYPE SIGN CHAD RTAGS\0"
  "TIBETAN LOGOTYPE SIGN LHAG RTAGS\0"
  "TIBETAN MARK ANG KHANG GYAS\0"
  "TIBETAN MARK ANG KHANG GYON\0"
  "TIBETAN MARK BKA- SHOG YIG MGO\0"
  "TIBETAN MARK BSDUS RTAGS\0"
  "TIBETAN MARK BSKA- SHOG GI MGO RGYAN\0"
  "TIBETAN MARK BSKUR YIG MGO\0"
  "TIBETAN MARK CARET -DZUD RTAGS BZHI MIG CAN\0"
  "TIBETAN MARK CARET -DZUD RTAGS ME LONG CAN\0"
  "TIBETAN MARK CARET YIG MGO PHUR SHAD MA\0"
  "TIBETAN MARK CHE MGO\0"
  "TIBETAN MARK CLOSING BRDA RNYING YIG MGO SGAB MA\0"
  "TIBETAN MARK CLOSING YIG MGO SGAB MA\0"
  "TIBETAN MARK DELIMITER TSHEG BSTAR\0"
  "TIBETAN MARK GTER TSHEG\0"
  "TIBETAN MARK GTER YIG MGO -UM GTER TSHEG MA\0"
  "TIBETAN MARK GTER YIG MGO -UM RNAM BCAD MA\0"
  "TIBETAN MARK GTER YIG MGO TRUNCATED A\0"
  "TIBETAN MARK GUG RTAGS GYAS\0"
  "TIBETAN MARK GUG RTAGS GYON\0"
  "TIBETAN MARK HALANTA\0"
  "TIBETAN MARK INITIAL BRDA RNYING YIG MGO MDUN MA\0"
  "TIBETAN MARK INITIAL YIG MGO MDUN MA\0"
  "TIBETAN MARK INTERSYLLABIC TSHEG\0"
  "TIBETAN MARK LEADING MCHAN RTAGS\0"
  "TIBETAN MARK MNYAM YIG GI MGO RGYAN\0"
  "TIBETAN MARK NGAS BZUNG NYI ZLA\0"
  "TIBETAN MARK NGAS BZUNG SGOR RTAGS\0"
  "TIBETAN MARK NYIS SHAD\0"
  "TIBETAN MARK NYIS TSHEG\0"
  "TIBETAN MARK NYIS TSHEG SHAD\0"
  "TIBETAN MARK PALUTA\0"
  "TIBETAN MARK RGYA GRAM SHAD\0"
  "TIBETAN MARK RIN CHEN SPUNGS SHAD\0"
  "TIBETAN MARK SBRUL SHAD\0"
  "TIBETAN MARK SHAD\0"
  "TIBETAN MARK TRAILING MCHAN RTAGS\0"
  "TIBETAN MARK TSA -PHRU\0"
  "TIBETAN MARK TSHEG SHAD\0"
  "TIBETAN MARK YIG MGO TSHEG SHAD MA\0"
  "TIBETAN SIGN GRU CAN RGYINGS\0"
  "TIBETAN SIGN GRU MED RGYINGS\0"
  "TIBETAN SIGN INVERTED MCHU CAN\0"
  "TIBETAN SIGN LCE TSA CAN\0"
  "TIBETAN SIGN LCI RTAGS\0"
  "TIBETAN SIGN MAR TSHES\0"
  "TIBETAN SIGN MCHU CAN\0"
  "TIBETAN SIGN NYI ZLA NAA DA\0"
  "TIBETAN SIGN RDEL DKAR GCIG\0"
  "TIBETAN SIGN RDEL DKAR GNYIS\0"
  "TIBETAN SIGN RDEL DKAR GSUM\0"
  "TIBETAN SIGN RDEL DKAR RDEL NAG\0"
  "TIBETAN SIGN RDEL NAG GCIG\0"
  "TIBETAN SIGN RDEL NAG GNYIS\0"
  "TIBETAN SIGN RDEL NAG GSUM\0"
  "TIBETAN SIGN RDEL NAG RDEL DKAR\0"
  "TIBETAN SIGN RJES SU NGA RO\0"
  "TIBETAN SIGN RNAM BCAD\0"
  "TIBETAN SIGN SNA LDAN\0"
  "TIBETAN SIGN YANG RTAGS\0"
  "TIBETAN SIGN YAR TSHES\0"
  "TIBETAN SUBJOINED LETTER -A\0"
  "TIBETAN SUBJOINED LETTER A\0"
  "TIBETAN SUBJOINED LETTER BA\0"
  "TIBETAN SUBJOINED LETTER BHA\0"
  "TIBETAN SUBJOINED LETTER CA\0"
  "TIBETAN SUBJOINED LETTER CHA\0"
  "TIBETAN SUBJOINED LETTER DA\0"
  "TIBETAN SUBJOINED LETTER DDA\0"
  "TIBETAN SUBJOINED LETTER DDHA\0"
  "TIBETAN SUBJOINED LETTER DHA\0"
  "TIBETAN SUBJOINED LETTER DZA\0"
  "TIBETAN SUBJOINED LETTER DZHA\0"
  "TIBETAN SUBJOINED LETTER FIXED-FORM RA\0"
  "TIBETAN SUBJOINED LETTER FIXED-FORM WA\0"
  "TIBETAN SUBJOINED LETTER FIXED-FORM YA\0"
  "TIBETAN SUBJOINED LETTER GA\0"
  "TIBETAN SUBJOINED LETTER GHA\0"
  "TIBETAN SUBJOINED LETTER HA\0"
  "TIBETAN SUBJOINED LETTER JA\0"
  "TIBETAN SUBJOINED LETTER KA\0"
  "TIBETAN SUBJOINED LETTER KHA\0"
  "TIBETAN SUBJOINED LETTER KSSA\0"
  "TIBETAN SUBJOINED LETTER LA\0"
  "TIBETAN SUBJOINED LETTER MA\0"
  "TIBETAN SUBJOINED LETTER NA\0"
  "TIBETAN SUBJOINED LETTER NGA\0"
  "TIBETAN SUBJOINED LETTER NNA\0"
  "TIBETAN SUBJOINED LETTER NYA\0"
  "TIBETAN SUBJOINED LETTER PA\0"
  "TIBETAN SUBJOINED LETTER PHA\0"
  "TIBETAN SUBJOINED LETTER RA\0"
  "TIBETAN SUBJOINED LETTER SA\0"
  "TIBETAN SUBJOINED LETTER SHA\0"
  "TIBETAN SUBJOINED LETTER SSA\0"
  "TIBETAN SUBJOINED LETTER TA\0"
  "TIBETAN SUBJOINED LETTER THA\0"
  "TIBETAN SUBJOINED LETTER TSA\0"
  "TIBETAN SUBJOINED LETTER TSHA\0"
  "TIBETAN SUBJOINED LETTER TTA\0"
  "TIBETAN SUBJOINED LETTER TTHA\0"
  "TIBETAN SUBJOINED LETTER WA\0"
  "TIBETAN SUBJOINED LETTER YA\0"
  "TIBETAN SUBJOINED LETTER ZA\0"
  "TIBETAN SUBJOINED LETTER ZHA\0"
  "TIBETAN SUBJOINED SIGN INVERTED MCHU CAN\0"
  "TIBETAN SUBJOINED SIGN LCE TSA CAN\0"
  "TIBETAN SUBJOINED SIGN MCHU CAN\0"
  "TIBETAN SYLLABLE OM\0"
  "TIBETAN SYMBOL DRIL BU\0"
  "TIBETAN SYMBOL NOR BU\0"
  "TIBETAN SYMBOL NOR BU BZHI -KHYIL\0"
  "TIBETAN SYMBOL NOR BU GSUM -KHYIL\0"
  "TIBETAN SYMBOL NOR BU NYIS -KHYIL\0"
  "TIBETAN SYMBOL PADMA GDAN\0"
  "TIBETAN SYMBOL PHUR PA\0"
  "TIBETAN SYMBOL RDO RJE\0"
  "TIBETAN SYMBOL RDO RJE RGYA GRAM\0"
  "TIBETAN VOWEL SIGN AA\0"
  "TIBETAN VOWEL SIGN E\0"
  "TIBETAN VOWEL SIGN EE\0"
  "TIBETAN VOWEL SIGN I\0"
  "TIBETAN VOWEL SIGN II\0"
  "TIBETAN VOWEL SIGN O\0"
  "TIBETAN VOWEL SIGN OO\0"
  "TIBETAN VOWEL SIGN REVERSED I\0"
  "TIBETAN VOWEL SIGN REVERSED II\0"
  "TIBETAN VOWEL SIGN U\0"
  "TIBETAN VOWEL SIGN UU\0"
  "TIBETAN VOWEL SIGN VOCALIC L\0"
  "TIBETAN VOWEL SIGN VOCALIC LL\0"
  "TIBETAN VOWEL SIGN VOCALIC R\0"
  "TIBETAN VOWEL SIGN VOCALIC RR\0"
  "TICKET\0"
  "TIE OVER INFINITY\0"
  "TIFINAGH CONSONANT JOINER\0"
  "TIFINAGH LETTER AHAGGAR YAZH\0"
  "TIFINAGH LETTER AYER YAGH\0"
  "TIFINAGH LETTER BERBER ACADEMY YAH\0"
  "TIFINAGH LETTER BERBER ACADEMY YAJ\0"
  "TIFINAGH LETTER TAWELLEMET YAZ\0"
  "TIFINAGH LETTER TUAREG YAGH\0"
  "TIFINAGH LETTER TUAREG YAGN\0"
  "TIFINAGH LETTER TUAREG YAH\0"
  "TIFINAGH LETTER TUAREG YAK\0"
  "TIFINAGH LETTER TUAREG YAKH\0"
  "TIFINAGH LETTER TUAREG YANG\0"
  "TIFINAGH LETTER TUAREG YAQ\0"
  "TIFINAGH LETTER TUAREG YAZH\0"
  "TIFINAGH LETTER YA\0"
  "TIFINAGH LETTER YAA\0"
  "TIFINAGH LETTER YAB\0"
  "TIFINAGH LETTER YABH\0"
  "TIFINAGH LETTER YACH\0"
  "TIFINAGH LETTER YAD\0"
  "TIFINAGH LETTER YADD\0"
  "TIFINAGH LETTER YADDH\0"
  "TIFINAGH LETTER YADH\0"
  "TIFINAGH LETTER YAF\0"
  "TIFINAGH LETTER YAG\0"
  "TIFINAGH LETTER YAGH\0"
  "TIFINAGH LETTER YAGHH\0"
  "TIFINAGH LETTER YAH\0"
  "TIFINAGH LETTER YAHH\0"
  "TIFINAGH LETTER YAJ\0"
  "TIFINAGH LETTER YAK\0"
  "TIFINAGH LETTER YAKH\0"
  "TIFINAGH LETTER YAKHH\0"
  "TIFINAGH LETTER YAL\0"
  "TIFINAGH LETTER YAM\0"
  "TIFINAGH LETTER YAN\0"
  "TIFINAGH LETTER YAP\0"
  "TIFINAGH LETTER YAQ\0"
  "TIFINAGH LETTER YAR\0"
  "TIFINAGH LETTER YARR\0"
  "TIFINAGH LETTER YAS\0"
  "TIFINAGH LETTER YASH\0"
  "TIFINAGH LETTER YASS\0"
  "TIFINAGH LETTER YAT\0"
  "TIFINAGH LETTER YATH\0"
  "TIFINAGH LETTER YATT\0"
  "TIFINAGH LETTER YAV\0"
  "TIFINAGH LETTER YAW\0"
  "TIFINAGH LETTER YAY\0"
  "TIFINAGH LETTER YAZ\0"
  "TIFINAGH LETTER YAZH\0"
  "TIFINAGH LETTER YAZZ\0"
  "TIFINAGH LETTER YE\0"
  "TIFINAGH LETTER YEY\0"
  "TIFINAGH LETTER YI\0"
  "TIFINAGH LETTER YO\0"
  "TIFINAGH LETTER YU\0"
  "TIFINAGH MODIFIER LETTER LABIALIZATION MARK\0"
  "TIFINAGH SEPARATOR MARK\0"
  "TIGER\0"
  "TIGER FACE\0"
  "TIGHT TRIFOLIATE SNOWFLAKE\0"
  "TILDE\0"
  "TILDE OPERATOR\0"
  "TILDE OPERATOR ABOVE LEFTWARDS ARROW\0"
  "TILDE OPERATOR ABOVE RIGHTWARDS ARROW\0"
  "TILDE OPERATOR WITH DOT ABOVE\0"
  "TILDE OPERATOR WITH RISING DOTS\0"
  "TILDE WITH DOT ABOVE\0"
  "TILDE WITH DOT BELOW\0"
  "TILDE WITH RING ABOVE\0"
  "TIMER CLOCK\0"
  "TIMES WITH LEFT HALF BLACK\0"
  "TIMES WITH RIGHT HALF BLACK\0"
  "TINY\0"
  "TINY TWO DOTS OVER ONE DOT PUNCTUATION\0"
  "TIRED FACE\0"
  "TIRHUTA ABBREVIATION SIGN\0"
  "TIRHUTA ANJI\0"
  "TIRHUTA DIGIT EIGHT\0"
  "TIRHUTA DIGIT FIVE\0"
  "TIRHUTA DIGIT FOUR\0"
  "TIRHUTA DIGIT NINE\0"
  "TIRHUTA DIGIT ONE\0"
  "TIRHUTA DIGIT SEVEN\0"
  "TIRHUTA DIGIT SIX\0"
  "TIRHUTA DIGIT THREE\0"
  "TIRHUTA DIGIT TWO\0"
  "TIRHUTA DIGIT ZERO\0"
  "TIRHUTA GVANG\0"
  "TIRHUTA LETTER A\0"
  "TIRHUTA LETTER AA\0"
  "TIRHUTA LETTER AI\0"
  "TIRHUTA LETTER AU\0"
  "TIRHUTA LETTER BA\0"
  "TIRHUTA LETTER BHA\0"
  "TIRHUTA LETTER CA\0"
  "TIRHUTA LETTER CHA\0"
  "TIRHUTA LETTER DA\0"
  "TIRHUTA LETTER DDA\0"
  "TIRHUTA LETTER DDHA\0"
  "TIRHUTA LETTER DHA\0"
  "TIRHUTA LETTER E\0"
  "TIRHUTA LETTER GA\0"
  "TIRHUTA LETTER GHA\0"
  "TIRHUTA LETTER HA\0"
  "TIRHUTA LETTER I\0"
  "TIRHUTA LETTER II\0"
  "TIRHUTA LETTER JA\0"
  "TIRHUTA LETTER JHA\0"
  "TIRHUTA LETTER KA\0"
  "TIRHUTA LETTER KHA\0"
  "TIRHUTA LETTER LA\0"
  "TIRHUTA LETTER MA\0"
  "TIRHUTA LETTER NA\0"
  "TIRHUTA LETTER NGA\0"
  "TIRHUTA LETTER NNA\0"
  "TIRHUTA LETTER NYA\0"
  "TIRHUTA LETTER O\0"
  "TIRHUTA LETTER PA\0"
  "TIRHUTA LETTER PHA\0"
  "TIRHUTA LETTER RA\0"
  "TIRHUTA LETTER SA\0"
  "TIRHUTA LETTER SHA\0"
  "TIRHUTA LETTER SSA\0"
  "TIRHUTA LETTER TA\0"
  "TIRHUTA LETTER THA\0"
  "TIRHUTA LETTER TTA\0"
  "TIRHUTA LETTER TTHA\0"
  "TIRHUTA LETTER U\0"
  "TIRHUTA LETTER UU\0"
  "TIRHUTA LETTER VA\0"
  "TIRHUTA LETTER VOCALIC L\0"
  "TIRHUTA LETTER VOCALIC LL\0"
  "TIRHUTA LETTER VOCALIC R\0"
  "TIRHUTA LETTER VOCALIC RR\0"
  "TIRHUTA LETTER YA\0"
  "TIRHUTA OM\0"
  "TIRHUTA SIGN ANUSVARA\0"
  "TIRHUTA SIGN AVAGRAHA\0"
  "TIRHUTA SIGN CANDRABINDU\0"
  "TIRHUTA SIGN NUKTA\0"
  "TIRHUTA SIGN VIRAMA\0"
  "TIRHUTA SIGN VISARGA\0"
  "TIRHUTA VOWEL SIGN AA\0"
  "TIRHUTA VOWEL SIGN AI\0"
  "TIRHUTA VOWEL SIGN AU\0"
  "TIRHUTA VOWEL SIGN E\0"
  "TIRHUTA VOWEL SIGN I\0"
  "TIRHUTA VOWEL SIGN II\0"
  "TIRHUTA VOWEL SIGN O\0"
  "TIRHUTA VOWEL SIGN SHORT E\0"
  "TIRHUTA VOWEL SIGN SHORT O\0"
  "TIRHUTA VOWEL SIGN U\0"
  "TIRHUTA VOWEL SIGN UU\0"
  "TIRHUTA VOWEL SIGN VOCALIC L\0"
  "TIRHUTA VOWEL SIGN VOCALIC LL\0"
  "TIRHUTA VOWEL SIGN VOCALIC R\0"
  "TIRHUTA VOWEL SIGN VOCALIC RR\0"
  "TIRONIAN SIGN CAPITAL ET\0"
  "TIRONIAN SIGN ET\0"
  "TODHRI LETTER A\0"
  "TODHRI LETTER AS\0"
  "TODHRI LETTER BA\0"
  "TODHRI LETTER CA\0"
  "TODHRI LETTER CHA\0"
  "TODHRI LETTER DA\0"
  "TODHRI LETTER DHA\0"
  "TODHRI LETTER E\0"
  "TODHRI LETTER EI\0"
  "TODHRI LETTER FA\0"
  "TODHRI LETTER GA\0"
  "TODHRI LETTER GHA\0"
  "TODHRI LETTER GJA\0"
  "TODHRI LETTER HA\0"
  "TODHRI LETTER HJA\0"
  "TODHRI LETTER I\0"
  "TODHRI LETTER JA\0"
  "TODHRI LETTER JY\0"
  "TODHRI LETTER KA\0"
  "TODHRI LETTER KHA\0"
  "TODHRI LETTER LA\0"
  "TODHRI LETTER LLA\0"
  "TODHRI LETTER MA\0"
  "TODHRI LETTER MBA\0"
  "TODHRI LETTER NA\0"
  "TODHRI LETTER NDA\0"
  "TODHRI LETTER NGA\0"
  "TODHRI LETTER NGJA\0"
  "TODHRI LETTER NJAN\0"
  "TODHRI LETTER NXA\0"
  "TODHRI LETTER NXHA\0"
  "TODHRI LETTER O\0"
  "TODHRI LETTER OO\0"
  "TODHRI LETTER PA\0"
  "TODHRI LETTER PSA\0"
  "TODHRI LETTER QA\0"
  "TODHRI LETTER RA\0"
  "TODHRI LETTER RRA\0"
  "TODHRI LETTER SA\0"
  "TODHRI LETTER SHA\0"
  "TODHRI LETTER SHTA\0"
  "TODHRI LETTER SKAN\0"
  "TODHRI LETTER STA\0"
  "TODHRI LETTER TA\0"
  "TODHRI LETTER THA\0"
  "TODHRI LETTER U\0"
  "TODHRI LETTER VA\0"
  "TODHRI LETTER XA\0"
  "TODHRI LETTER XHA\0"
  "TODHRI LETTER Y\0"
  "TODHRI LETTER ZA\0"
  "TODHRI LETTER ZHA\0"
  "TOILET\0"
  "TOKYO TOWER\0"
  "TOMATO\0"
  "TONGUE\0"
  "TOOLBOX\0"
  "TOOTH\0"
  "TOOTHBRUSH\0"
  "TOP ARC ANTICLOCKWISE ARROW\0"
  "TOP ARC ANTICLOCKWISE ARROW WITH PLUS\0"
  "TOP ARC CLOCKWISE ARROW WITH MINUS\0"
  "TOP CURLY BRACKET\0"
  "TOP HALF BLACK CIRCLE\0"
  "TOP HALF FLAILING ROBOT FRAME-1\0"
  "TOP HALF FLAILING ROBOT FRAME-2\0"
  "TOP HALF FORWARD-FACING ROBOT\0"
  "TOP HALF FORWARD-FACING RUNNER\0"
  "TOP HALF INTEGRAL\0"
  "TOP HALF LEFT PARENTHESIS\0"
  "TOP HALF LEFT-FACING ROBOT\0"
  "TOP HALF LEFT-FACING RUNNER FRAME-1\0"
  "TOP HALF LEFT-FACING RUNNER FRAME-2\0"
  "TOP HALF RIGHT PARENTHESIS\0"
  "TOP HALF RIGHT-FACING ROBOT\0"
  "TOP HALF RIGHT-FACING RUNNER FRAME-1\0"
  "TOP HALF RIGHT-FACING RUNNER FRAME-2\0"
  "TOP HALF SECTION SIGN\0"
  "TOP HALF STANDING PERSON\0"
  "TOP HAT\0"
  "TOP JUSTIFIED LOWER HALF BLACK CIRCLE\0"
  "TOP JUSTIFIED LOWER HALF WHITE CIRCLE\0"
  "TOP LEFT BLACK LEFT-POINTING SMALL TRIANGLE\0"
  "TOP LEFT CORNER\0"
  "TOP LEFT CROP\0"
  "TOP LEFT HALF BRACKET\0"
  "TOP LEFT JUSTIFIED LOWER RIGHT QUARTER BLACK CIRCLE\0"
  "TOP PARENTHESIS\0"
  "TOP RIGHT CORNER\0"
  "TOP RIGHT CROP\0"
  "TOP RIGHT HALF BRACKET\0"
  "TOP RIGHT JUSTIFIED LOWER LEFT QUARTER BLACK CIRCLE\0"
  "TOP SQUARE BRACKET\0"
  "TOP TORTOISE SHELL BRACKET\0"
  "TOP WITH UPWARDS ARROW ABOVE\0"
  "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-4E09\0"
  "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-4E8C\0"
  "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-52DD\0"
  "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-5B89\0"
  "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-6253\0"
  "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-6557\0"
  "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-672C\0"
  "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-70B9\0"
  "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-76D7\0"
  "TORTOISE SHELL BRACKETED LATIN CAPITAL LETTER S\0"
  "TOTAL RUNOUT\0"
  "TOTO LETTER A\0"
  "TOTO LETTER AE\0"
  "TOTO LETTER BA\0"
  "TOTO LETTER BREATHY AE\0"
  "TOTO LETTER BREATHY E\0"
  "TOTO LETTER BREATHY EO\0"
  "TOTO LETTER BREATHY I\0"
  "TOTO LETTER BREATHY IU\0"
  "TOTO LETTER CHA\0"
  "TOTO LETTER DA\0"
  "TOTO LETTER E\0"
  "TOTO LETTER EO\0"
  "TOTO LETTER GA\0"
  "TOTO LETTER HA\0"
  "TOTO LETTER I\0"
  "TOTO LETTER IU\0"
  "TOTO LETTER JA\0"
  "TOTO LETTER KA\0"
  "TOTO LETTER LA\0"
  "TOTO LETTER MA\0"
  "TOTO LETTER NA\0"
  "TOTO LETTER NGA\0"
  "TOTO LETTER O\0"
  "TOTO LETTER PA\0"
  "TOTO LETTER RA\0"
  "TOTO LETTER SA\0"
  "TOTO LETTER TA\0"
  "TOTO LETTER U\0"
  "TOTO LETTER WA\0"
  "TOTO LETTER YA\0"
  "TOTO SIGN RISING TONE\0"
  "TRACKBALL\0"
  "TRACTOR\0"
  "TRADE MARK SIGN\0"
  "TRAIN\0"
  "TRAM\0"
  "TRAM CAR\0"
  "TRANSPLUTO\0"
  "TRANSVERSAL INTERSECTION\0"
  "TRIANGLE WITH DOT ABOVE\0"
  "TRIANGLE WITH ROUNDED CORNERS\0"
  "TRIANGLE WITH SERIFS AT BOTTOM\0"
  "TRIANGLE WITH UNDERBAR\0"
  "TRIANGLE-HEADED RIGHTWARDS ARROW\0"
  "TRIANGULAR BULLET\0"
  "TRIANGULAR FLAG ON POST\0"
  "TRIANGULAR RULER\0"
  "TRICOLON\0"
  "TRIDENT EMBLEM\0"
  "TRIGRAM FOR EARTH\0"
  "TRIGRAM FOR FIRE\0"
  "TRIGRAM FOR HEAVEN\0"
  "TRIGRAM FOR LAKE\0"
  "TRIGRAM FOR MOUNTAIN\0"
  "TRIGRAM FOR THUNDER\0"
  "TRIGRAM FOR WATER\0"
  "TRIGRAM FOR WIND\0"
  "TRIPLE COLON OPERATOR\0"
  "TRIPLE DAGGER\0"
  "TRIPLE HORIZONTAL BAR WITH DOUBLE VERTICAL STROKE\0"
  "TRIPLE HORIZONTAL BAR WITH TRIPLE VERTICAL STROKE\0"
  "TRIPLE INTEGRAL\0"
  "TRIPLE NESTED GREATER-THAN\0"
  "TRIPLE NESTED LESS-THAN\0"
  "TRIPLE PLUS\0"
  "TRIPLE PRIME\0"
  "TRIPLE SOLIDUS BINARY RELATION\0"
  "TRIPLE TILDE\0"
  "TRIPLE VERTICAL BAR BINARY RELATION\0"
  "TRIPLE VERTICAL BAR DELIMITER\0"
  "TRIPLE VERTICAL BAR RIGHT TURNSTILE\0"
  "TRIPLE VERTICAL BAR WITH HORIZONTAL STROKE\0"
  "TROLL\0"
  "TROLLEYBUS\0"
  "TROPHY\0"
  "TROPICAL DRINK\0"
  "TROPICAL FISH\0"
  "TRUE\0"
  "TRUE LIGHT MOON ARTA\0"
  "TRUMPET\0"
  "TUGRIK SIGN\0"
  "TULIP\0"
  "TULU-TIGALARI AU LENGTH MARK\0"
  "TULU-TIGALARI CONJOINER\0"
  "TULU-TIGALARI DANDA\0"
  "TULU-TIGALARI DOUBLE DANDA\0"
  "TULU-TIGALARI GEMINATION MARK\0"
  "TULU-TIGALARI LETTER A\0"
  "TULU-TIGALARI LETTER AA\0"
  "TULU-TIGALARI LETTER AI\0"
  "TULU-TIGALARI LETTER AU\0"
  "TULU-TIGALARI LETTER BA\0"
  "TULU-TIGALARI LETTER BHA\0"
  "TULU-TIGALARI LETTER CA\0"
  "TULU-TIGALARI LETTER CHA\0"
  "TULU-TIGALARI LETTER DA\0"
  "TULU-TIGALARI LETTER DDA\0"
  "TULU-TIGALARI LETTER DDHA\0"
  "TULU-TIGALARI LETTER DHA\0"
  "TULU-TIGALARI LETTER EE\0"
  "TULU-TIGALARI LETTER GA\0"
  "TULU-TIGALARI LETTER GHA\0"
  "TULU-TIGALARI LETTER HA\0"
  "TULU-TIGALARI LETTER I\0"
  "TULU-TIGALARI LETTER II\0"
  "TULU-TIGALARI LETTER JA\0"
  "TULU-TIGALARI LETTER JHA\0"
  "TULU-TIGALARI LETTER KA\0"
  "TULU-TIGALARI LETTER KHA\0"
  "TULU-TIGALARI LETTER LA\0"
  "TULU-TIGALARI LETTER LLA\0"
  "TULU-TIGALARI LETTER LLLA\0"
  "TULU-TIGALARI LETTER MA\0"
  "TULU-TIGALARI LETTER NA\0"
  "TULU-TIGALARI LETTER NGA\0"
  "TULU-TIGALARI LETTER NNA\0"
  "TULU-TIGALARI LETTER NYA\0"
  "TULU-TIGALARI LETTER OO\0"
  "TULU-TIGALARI LETTER PA\0"
  "TULU-TIGALARI LETTER PHA\0"
  "TULU-TIGALARI LETTER RA\0"
  "TULU-TIGALARI LETTER RRA\0"
  "TULU-TIGALARI LETTER SA\0"
  "TULU-TIGALARI LETTER SHA\0"
  "TULU-TIGALARI LETTER SSA\0"
  "TULU-TIGALARI LETTER TA\0"
  "TULU-TIGALARI LETTER THA\0"
  "TULU-TIGALARI LETTER TTA\0"
  "TULU-TIGALARI LETTER TTHA\0"
  "TULU-TIGALARI LETTER U\0"
  "TULU-TIGALARI LETTER UU\0"
  "TULU-TIGALARI LETTER VA\0"
  "TULU-TIGALARI LETTER VOCALIC L\0"
  "TULU-TIGALARI LETTER VOCALIC LL\0"
  "TULU-TIGALARI LETTER VOCALIC R\0"
  "TULU-TIGALARI LETTER VOCALIC RR\0"
  "TULU-TIGALARI LETTER YA\0"
  "TULU-TIGALARI REPHA\0"
  "TULU-TIGALARI SIGN ANUSVARA\0"
  "TULU-TIGALARI SIGN AVAGRAHA\0"
  "TULU-TIGALARI SIGN CANDRA ANUNASIKA\0"
  "TULU-TIGALARI SIGN LOOPED VIRAMA\0"
  "TULU-TIGALARI SIGN OM PUSHPIKA\0"
  "TULU-TIGALARI SIGN PLUTA\0"
  "TULU-TIGALARI SIGN SHRII PUSHPIKA\0"
  "TULU-TIGALARI SIGN VIRAMA\0"
  "TULU-TIGALARI SIGN VISARGA\0"
  "TULU-TIGALARI VEDIC TONE ANUDATTA\0"
  "TULU-TIGALARI VEDIC TONE SVARITA\0"
  "TULU-TIGALARI VOWEL SIGN AA\0"
  "TULU-TIGALARI VOWEL SIGN AI\0"
  "TULU-TIGALARI VOWEL SIGN AU\0"
  "TULU-TIGALARI VOWEL SIGN EE\0"
  "TULU-TIGALARI VOWEL SIGN I\0"
  "TULU-TIGALARI VOWEL SIGN II\0"
  "TULU-TIGALARI VOWEL SIGN OO\0"
  "TULU-TIGALARI VOWEL SIGN U\0"
  "TULU-TIGALARI VOWEL SIGN UU\0"
  "TULU-TIGALARI VOWEL SIGN VOCALIC L\0"
  "TULU-TIGALARI VOWEL SIGN VOCALIC LL\0"
  "TULU-TIGALARI VOWEL SIGN VOCALIC R\0"
  "TULU-TIGALARI VOWEL SIGN VOCALIC RR\0"
  "TUMBLER GLASS\0"
  "TURKEY\0"
  "TURKISH LIRA SIGN\0"
  "TURNED AMPERSAND\0"
  "TURNED ANGLE\0"
  "TURNED BLACK PENTAGON\0"
  "TURNED BLACK SHOGI PIECE\0"
  "TURNED CAPITAL F\0"
  "TURNED COMMA\0"
  "TURNED DAGGER\0"
  "TURNED DIGIT THREE\0"
  "TURNED DIGIT TWO\0"
  "TURNED GREEK SMALL LETTER IOTA\0"
  "TURNED NORTH EAST POINTING LEAF\0"
  "TURNED NORTH WEST POINTING LEAF\0"
  "TURNED NOT SIGN\0"
  "TURNED OK HAND SIGN\0"
  "TURNED SANS-SERIF CAPITAL G\0"
  "TURNED SANS-SERIF CAPITAL L\0"
  "TURNED SANS-SERIF CAPITAL Y\0"
  "TURNED SEMICOLON\0"
  "TURNED SMALL F\0"
  "TURNED SOUTH EAST POINTING LEAF\0"
  "TURNED SOUTH WEST POINTING LEAF\0"
  "TURNED WHITE SHOGI PIECE\0"
  "TURTLE\0"
  "TWELVE POINTED BLACK STAR\0"
  "TWISTED RIGHTWARDS ARROWS\0"
  "TWO ASTERISKS ALIGNED VERTICALLY\0"
  "TWO BUTTON MOUSE\0"
  "TWO CONSECUTIVE EQUALS SIGNS\0"
  "TWO DOT LEADER\0"
  "TWO DOT PUNCTUATION\0"
  "TWO DOTS OVER ONE DOT PUNCTUATION\0"
  "TWO HEARTS\0"
  "TWO INTERSECTING LOGICAL AND\0"
  "TWO INTERSECTING LOGICAL OR\0"
  "TWO JOINED SQUARES\0"
  "TWO LOGICAL AND OPERATOR\0"
  "TWO LOGICAL OR OPERATOR\0"
  "TWO MEN HOLDING HANDS\0"
  "TWO RINGS ALIGNED HORIZONTALLY\0"
  "TWO SPEECH BUBBLES\0"
  "TWO WOMEN HOLDING HANDS\0"
  "TWO-EM DASH\0"
  "UGARITIC LETTER AIN\0"
  "UGARITIC LETTER ALPA\0"
  "UGARITIC LETTER BETA\0"
  "UGARITIC LETTER DELTA\0"
  "UGARITIC LETTER DHAL\0"
  "UGARITIC LETTER GAMLA\0"
  "UGARITIC LETTER GHAIN\0"
  "UGARITIC LETTER HO\0"
  "UGARITIC LETTER HOTA\0"
  "UGARITIC LETTER I\0"
  "UGARITIC LETTER KAF\0"
  "UGARITIC LETTER KHA\0"
  "UGARITIC LETTER LAMDA\0"
  "UGARITIC LETTER MEM\0"
  "UGARITIC LETTER NUN\0"
  "UGARITIC LETTER PU\0"
  "UGARITIC LETTER QOPA\0"
  "UGARITIC LETTER RASHA\0"
  "UGARITIC LETTER SADE\0"
  "UGARITIC LETTER SAMKA\0"
  "UGARITIC LETTER SHIN\0"
  "UGARITIC LETTER SSU\0"
  "UGARITIC LETTER TET\0"
  "UGARITIC LETTER THANNA\0"
  "UGARITIC LETTER TO\0"
  "UGARITIC LETTER U\0"
  "UGARITIC LETTER WO\0"
  "UGARITIC LETTER YOD\0"
  "UGARITIC LETTER ZETA\0"
  "UGARITIC LETTER ZU\0"
  "UGARITIC WORD DIVIDER\0"
  "UMBRELLA\0"
  "UMBRELLA ON GROUND\0"
  "UMBRELLA WITH RAIN DROPS\0"
  "UNAMUSED FACE\0"
  "UNCERTAINTY SIGN\0"
  "UNDERTIE\0"
  "UNDO SYMBOL\0"
  "UNICORN FACE\0"
  "UNION\0"
  "UNION ABOVE BAR ABOVE INTERSECTION\0"
  "UNION ABOVE INTERSECTION\0"
  "UNION BESIDE AND JOINED WITH UNION\0"
  "UNION WITH LOGICAL OR\0"
  "UNION WITH MINUS SIGN\0"
  "UNION WITH OVERBAR\0"
  "UNITED SYMBOL\0"
  "UNIVERSAL RECYCLING SYMBOL\0"
  "UNMARRIED PARTNERSHIP SYMBOL\0"
  "UP ARROW THROUGH CIRCLE\0"
  "UP ARROWHEAD\0"
  "UP ARROWHEAD BETWEEN TWO HORIZONTAL BARS\0"
  "UP ARROWHEAD IN A RECTANGLE BOX\0"
  "UP BARB LEFT DOWN BARB LEFT HARPOON\0"
  "UP BARB LEFT DOWN BARB RIGHT HARPOON\0"
  "UP BARB RIGHT DOWN BARB LEFT HARPOON\0"
  "UP BARB RIGHT DOWN BARB RIGHT HARPOON\0"
  "UP DOWN ARROW\0"
  "UP DOWN ARROW WITH BASE\0"
  "UP DOWN BLACK ARROW\0"
  "UP DOWN DOUBLE ARROW\0"
  "UP DOWN SANS-SERIF ARROW\0"
  "UP DOWN TRIANGLE-HEADED ARROW\0"
  "UP DOWN WHITE ARROW\0"
  "UP FISH TAIL\0"
  "UP RIGHT DIAGONAL ELLIPSIS\0"
  "UP TACK\0"
  "UP TACK WITH CIRCLE ABOVE\0"
  "UP-FACING SNAKE HEAD WITH CLOSED MOUTH\0"
  "UP-FACING SNAKE HEAD WITH OPEN MOUTH\0"
  "UP-POINTING AIRPLANE\0"
  "UP-POINTING ATOMIC BOMB\0"
  "UP-POINTING ENERGY WAVE\0"
  "UP-POINTING FROG\0"
  "UP-POINTING GO-KART\0"
  "UP-POINTING MILITARY AIRPLANE\0"
  "UP-POINTING RACING CAR\0"
  "UP-POINTING RED TRIANGLE\0"
  "UP-POINTING RIFLE\0"
  "UP-POINTING ROCKET SHIP\0"
  "UP-POINTING SMALL AIRPLANE\0"
  "UP-POINTING SMALL RED TRIANGLE\0"
  "UP-POINTING TRIANGLE WITH LEFT HALF BLACK\0"
  "UP-POINTING TRIANGLE WITH RIGHT HALF BLACK\0"
  "UPPER AND LEFT AND LOWER TRIANGULAR THREE QUARTERS BLOCK\0"
  "UPPER AND LOWER ONE EIGHTH BLOCK\0"
  "UPPER AND LOWER TRIANGULAR HALF BLOCK\0"
  "UPPER AND RIGHT AND LOWER TRIANGULAR THREE QUARTERS BLOCK\0"
  "UPPER BLADE SCISSORS\0"
  "UPPER CENTRE LEFT ONE SIXTEENTH BLOCK\0"
  "UPPER CENTRE LEFT TWELFTH CIRCLE\0"
  "UPPER CENTRE ONE QUARTER BLOCK\0"
  "UPPER CENTRE RIGHT ONE SIXTEENTH BLOCK\0"
  "UPPER CENTRE RIGHT TWELFTH CIRCLE\0"
  "UPPER FIVE EIGHTHS BLOCK\0"
  "UPPER HALF BLOCK\0"
  "UPPER HALF BLOCK AND LOWER HALF INVERSE MEDIUM SHADE\0"
  "UPPER HALF CIRCLE\0"
  "UPPER HALF HEAVY WHITE SQUARE\0"
  "UPPER HALF INVERSE MEDIUM SHADE AND LOWER HALF BLOCK\0"
  "UPPER HALF INVERSE WHITE CIRCLE\0"
  "UPPER HALF LEFT ONE QUARTER BLOCK\0"
  "UPPER HALF MEDIUM SHADE\0"
  "UPPER HALF RIGHT ONE QUARTER BLOCK\0"
  "UPPER HALF VERTICAL LINE WITH THREE TICK MARKS\0"
  "UPPER LEFT BLOCK DIAGONAL LOWER CENTRE TO LOWER MIDDLE RIGHT\0"
  "UPPER LEFT BLOCK DIAGONAL LOWER CENTRE TO UPPER MIDDLE RIGHT\0"
  "UPPER LEFT BLOCK DIAGONAL LOWER CENTRE TO UPPER RIGHT\0"
  "UPPER LEFT BLOCK DIAGONAL LOWER LEFT TO LOWER MIDDLE RIGHT\0"
  "UPPER LEFT BLOCK DIAGONAL LOWER LEFT TO UPPER CENTRE\0"
  "UPPER LEFT BLOCK DIAGONAL LOWER LEFT TO UPPER MIDDLE RIGHT\0"
  "UPPER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER CENTRE\0"
  "UPPER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER MIDDLE RIGHT\0"
  "UPPER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER RIGHT\0"
  "UPPER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER CENTRE\0"
  "UPPER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER RIGHT\0"
  "UPPER LEFT CORNER WITH DOT\0"
  "UPPER LEFT ONE SIXTEENTH BLOCK\0"
  "UPPER LEFT OR LOWER RIGHT CURLY BRACKET SECTION\0"
  "UPPER LEFT QUADRANT CHESS BISHOP\0"
  "UPPER LEFT QUADRANT CHESS KING\0"
  "UPPER LEFT QUADRANT CHESS KNIGHT\0"
  "UPPER LEFT QUADRANT CHESS PAWN\0"
  "UPPER LEFT QUADRANT CHESS QUEEN\0"
  "UPPER LEFT QUADRANT CHESS ROOK\0"
  "UPPER LEFT QUADRANT CIRCULAR ARC\0"
  "UPPER LEFT QUADRANT FACE WITH CLOSED EYES\0"
  "UPPER LEFT QUADRANT FACE WITH OPEN EYES\0"
  "UPPER LEFT QUADRANT MICROCOMPUTER\0"
  "UPPER LEFT QUADRANT STANDING KNIGHT\0"
  "UPPER LEFT QUADRANT TELEVISION\0"
  "UPPER LEFT QUARTER CIRCLE\0"
  "UPPER LEFT TO LOWER RIGHT FILL\0"
  "UPPER LEFT TRIANGLE\0"
  "UPPER LEFT TRIANGULAR MEDIUM SHADE\0"
  "UPPER LEFT TWELFTH CIRCLE\0"
  "UPPER MIDDLE CENTRE LEFT ONE SIXTEENTH BLOCK\0"
  "UPPER MIDDLE CENTRE RIGHT ONE SIXTEENTH BLOCK\0"
  "UPPER MIDDLE LEFT ONE SIXTEENTH BLOCK\0"
  "UPPER MIDDLE LEFT TWELFTH CIRCLE\0"
  "UPPER MIDDLE RIGHT ONE SIXTEENTH BLOCK\0"
  "UPPER MIDDLE RIGHT TWELFTH CIRCLE\0"
  "UPPER ONE EIGHTH BLOCK\0"
  "UPPER ONE QUARTER BLOCK\0"
  "UPPER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER CENTRE\0"
  "UPPER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER RIGHT\0"
  "UPPER RIGHT BLOCK DIAGONAL UPPER CENTRE TO LOWER MIDDLE RIGHT\0"
  "UPPER RIGHT BLOCK DIAGONAL UPPER CENTRE TO LOWER RIGHT\0"
  "UPPER RIGHT BLOCK DIAGONAL UPPER CENTRE TO UPPER MIDDLE RIGHT\0"
  "UPPER RIGHT BLOCK DIAGONAL UPPER LEFT TO LOWER CENTRE\0"
  "UPPER RIGHT BLOCK DIAGONAL UPPER LEFT TO LOWER MIDDLE RIGHT\0"
  "UPPER RIGHT BLOCK DIAGONAL UPPER LEFT TO UPPER MIDDLE RIGHT\0"
  "UPPER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER CENTRE\0"
  "UPPER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER MIDDLE RIGHT\0"
  "UPPER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER RIGHT\0"
  "UPPER RIGHT DROP-SHADOWED WHITE SQUARE\0"
  "UPPER RIGHT ONE SIXTEENTH BLOCK\0"
  "UPPER RIGHT OR LOWER LEFT CURLY BRACKET SECTION\0"
  "UPPER RIGHT PENCIL\0"
  "UPPER RIGHT QUADRANT CHESS BISHOP\0"
  "UPPER RIGHT QUADRANT CHESS KING\0"
  "UPPER RIGHT QUADRANT CHESS KNIGHT\0"
  "UPPER RIGHT QUADRANT CHESS PAWN\0"
  "UPPER RIGHT QUADRANT CHESS QUEEN\0"
  "UPPER RIGHT QUADRANT CHESS ROOK\0"
  "UPPER RIGHT QUADRANT CIRCULAR ARC\0"
  "UPPER RIGHT QUADRANT FACE WITH CLOSED EYES\0"
  "UPPER RIGHT QUADRANT FACE WITH OPEN EYES\0"
  "UPPER RIGHT QUADRANT MICROCOMPUTER\0"
  "UPPER RIGHT QUADRANT STANDING KNIGHT\0"
  "UPPER RIGHT QUADRANT TELEVISION\0"
  "UPPER RIGHT QUARTER CIRCLE\0"
  "UPPER RIGHT SHADOWED WHITE CIRCLE\0"
  "UPPER RIGHT SHADOWED WHITE SQUARE\0"
  "UPPER RIGHT TO LOWER LEFT FILL\0"
  "UPPER RIGHT TRIANGLE\0"
  "UPPER RIGHT TRIANGULAR MEDIUM SHADE\0"
  "UPPER RIGHT TWELFTH CIRCLE\0"
  "UPPER SEVEN EIGHTHS BLOCK\0"
  "UPPER THREE EIGHTHS BLOCK\0"
  "UPPER THREE QUARTERS BLOCK\0"
  "UPPER THREE QUARTERS LEFT ONE QUARTER BLOCK\0"
  "UPPER THREE QUARTERS RIGHT ONE QUARTER BLOCK\0"
  "UPPER TRIANGULAR ONE QUARTER BLOCK\0"
  "UPSIDE-DOWN FACE\0"
  "UPWARDS ANCORA\0"
  "UPWARDS ARROW\0"
  "UPWARDS ARROW AND RIGHT ONE EIGHTH BLOCK\0"
  "UPWARDS ARROW FROM BAR\0"
  "UPWARDS ARROW LEFTWARDS OF DOWNWARDS ARROW\0"
  "UPWARDS ARROW TO BAR\0"
  "UPWARDS ARROW WITH DOUBLE STROKE\0"
  "UPWARDS ARROW WITH EQUILATERAL ARROWHEAD\0"
  "UPWARDS ARROW WITH HORIZONTAL STROKE\0"
  "UPWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD\0"
  "UPWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD\0"
  "UPWARDS ARROW WITH NOTCHED TAIL\0"
  "UPWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD\0"
  "UPWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD\0"
  "UPWARDS ARROW WITH TIP LEFTWARDS\0"
  "UPWARDS ARROW WITH TIP RIGHTWARDS\0"
  "UPWARDS BLACK ARROW\0"
  "UPWARDS BLACK CIRCLED WHITE ARROW\0"
  "UPWARDS COMPRESSED ARROW\0"
  "UPWARDS DASHED ARROW\0"
  "UPWARDS DOUBLE ARROW\0"
  "UPWARDS FINGER-POST ARROW\0"
  "UPWARDS HARPOON WITH BARB LEFT BESIDE DOWNWARDS HARPOON WITH BARB RIGHT\0"
  "UPWARDS HARPOON WITH BARB LEFT BESIDE UPWARDS HARPOON WITH BARB RIGHT\0"
  "UPWARDS HARPOON WITH BARB LEFT FROM BAR\0"
  "UPWARDS HARPOON WITH BARB LEFT TO BAR\0"
  "UPWARDS HARPOON WITH BARB LEFTWARDS\0"
  "UPWARDS HARPOON WITH BARB RIGHT FROM BAR\0"
  "UPWARDS HARPOON WITH BARB RIGHT TO BAR\0"
  "UPWARDS HARPOON WITH BARB RIGHTWARDS\0"
  "UPWARDS HEAVY ARROW\0"
  "UPWARDS HEAVY COMPRESSED ARROW\0"
  "UPWARDS PAIRED ARROWS\0"
  "UPWARDS QUADRUPLE ARROW\0"
  "UPWARDS ROCKET\0"
  "UPWARDS SANS-SERIF ARROW\0"
  "UPWARDS SQUARED ARROW\0"
  "UPWARDS TRIANGLE ARROWHEAD\0"
  "UPWARDS TRIANGLE-HEADED ARROW\0"
  "UPWARDS TRIANGLE-HEADED ARROW LEFTWARDS OF DOWNWARDS TRIANGLE-HEADED ARROW\0"
  "UPWARDS TRIANGLE-HEADED ARROW TO BAR\0"
  "UPWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT\0"
  "UPWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE\0"
  "UPWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT\0"
  "UPWARDS TRIANGLE-HEADED ARROW WITH LONG TIP LEFTWARDS\0"
  "UPWARDS TRIANGLE-HEADED ARROW WITH LONG TIP RIGHTWARDS\0"
  "UPWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT\0"
  "UPWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT\0"
  "UPWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT\0"
  "UPWARDS TRIANGLE-HEADED DASHED ARROW\0"
  "UPWARDS TRIANGLE-HEADED PAIRED ARROWS\0"
  "UPWARDS TRIPLE ARROW\0"
  "UPWARDS TWO HEADED ARROW\0"
  "UPWARDS TWO-HEADED ARROW FROM SMALL CIRCLE\0"
  "UPWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS\0"
  "UPWARDS WHITE ARROW\0"
  "UPWARDS WHITE ARROW FROM BAR\0"
  "UPWARDS WHITE ARROW FROM BAR WITH HORIZONTAL BAR\0"
  "UPWARDS WHITE ARROW ON PEDESTAL\0"
  "UPWARDS WHITE ARROW ON PEDESTAL WITH HORIZONTAL BAR\0"
  "UPWARDS WHITE ARROW ON PEDESTAL WITH VERTICAL BAR\0"
  "UPWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD\0"
  "UPWARDS WHITE DOUBLE ARROW\0"
  "UPWARDS WHITE DOUBLE ARROW ON PEDESTAL\0"
  "URANUS\0"
  "VAI COMMA\0"
  "VAI DIGIT EIGHT\0"
  "VAI DIGIT FIVE\0"
  "VAI DIGIT FOUR\0"
  "VAI DIGIT NINE\0"
  "VAI DIGIT ONE\0"
  "VAI DIGIT SEVEN\0"
  "VAI DIGIT SIX\0"
  "VAI DIGIT THREE\0"
  "VAI DIGIT TWO\0"
  "VAI DIGIT ZERO\0"
  "VAI FULL STOP\0"
  "VAI QUESTION MARK\0"
  "VAI SYLLABLE A\0"
  "VAI SYLLABLE AN\0"
  "VAI SYLLABLE BA\0"
  "VAI SYLLABLE BE\0"
  "VAI SYLLABLE BEE\0"
  "VAI SYLLABLE BHA\0"
  "VAI SYLLABLE BHE\0"
  "VAI SYLLABLE BHEE\0"
  "VAI SYLLABLE BHI\0"
  "VAI SYLLABLE BHO\0"
  "VAI SYLLABLE BHOO\0"
  "VAI SYLLABLE BHU\0"
  "VAI SYLLABLE BI\0"
  "VAI SYLLABLE BO\0"
  "VAI SYLLABLE BOO\0"
  "VAI SYLLABLE BU\0"
  "VAI SYLLABLE CA\0"
  "VAI SYLLABLE CE\0"
  "VAI SYLLABLE CEE\0"
  "VAI SYLLABLE CI\0"
  "VAI SYLLABLE CO\0"
  "VAI SYLLABLE COO\0"
  "VAI SYLLABLE CU\0"
  "VAI SYLLABLE DA\0"
  "VAI SYLLABLE DE\0"
  "VAI SYLLABLE DEE\0"
  "VAI SYLLABLE DHA\0"
  "VAI SYLLABLE DHE\0"
  "VAI SYLLABLE DHEE\0"
  "VAI SYLLABLE DHHA\0"
  "VAI SYLLABLE DHHE\0"
  "VAI SYLLABLE DHHEE\0"
  "VAI SYLLABLE DHHI\0"
  "VAI SYLLABLE DHHO\0"
  "VAI SYLLABLE DHHOO\0"
  "VAI SYLLABLE DHHU\0"
  "VAI SYLLABLE DHI\0"
  "VAI SYLLABLE DHO\0"
  "VAI SYLLABLE DHOO\0"
  "VAI SYLLABLE DHU\0"
  "VAI SYLLABLE DI\0"
  "VAI SYLLABLE DO\0"
  "VAI SYLLABLE DOO\0"
  "VAI SYLLABLE DU\0"
  "VAI SYLLABLE E\0"
  "VAI SYLLABLE EE\0"
  "VAI SYLLABLE EEN\0"
  "VAI SYLLABLE EN\0"
  "VAI SYLLABLE FA\0"
  "VAI SYLLABLE FE\0"
  "VAI SYLLABLE FEE\0"
  "VAI SYLLABLE FI\0"
  "VAI SYLLABLE FO\0"
  "VAI SYLLABLE FOO\0"
  "VAI SYLLABLE FU\0"
  "VAI SYLLABLE GA\0"
  "VAI SYLLABLE GBA\0"
  "VAI SYLLABLE GBE\0"
  "VAI SYLLABLE GBEE\0"
  "VAI SYLLABLE GBEN\0"
  "VAI SYLLABLE GBI\0"
  "VAI SYLLABLE GBO\0"
  "VAI SYLLABLE GBON\0"
  "VAI SYLLABLE GBOO\0"
  "VAI SYLLABLE GBU\0"
  "VAI SYLLABLE GE\0"
  "VAI SYLLABLE GEE\0"
  "VAI SYLLABLE GEN\0"
  "VAI SYLLABLE GI\0"
  "VAI SYLLABLE GO\0"
  "VAI SYLLABLE GOO\0"
  "VAI SYLLABLE GU\0"
  "VAI SYLLABLE HA\0"
  "VAI SYLLABLE HAN\0"
  "VAI SYLLABLE HE\0"
  "VAI SYLLABLE HEE\0"
  "VAI SYLLABLE HEN\0"
  "VAI SYLLABLE HI\0"
  "VAI SYLLABLE HIN\0"
  "VAI SYLLABLE HO\0"
  "VAI SYLLABLE HON\0"
  "VAI SYLLABLE HOO\0"
  "VAI SYLLABLE HU\0"
  "VAI SYLLABLE HUN\0"
  "VAI SYLLABLE I\0"
  "VAI SYLLABLE IN\0"
  "VAI SYLLABLE JA\0"
  "VAI SYLLABLE JE\0"
  "VAI SYLLABLE JEE\0"
  "VAI SYLLABLE JI\0"
  "VAI SYLLABLE JO\0"
  "VAI SYLLABLE JOO\0"
  "VAI SYLLABLE JU\0"
  "VAI SYLLABLE KA\0"
  "VAI SYLLABLE KAN\0"
  "VAI SYLLABLE KE\0"
  "VAI SYLLABLE KEE\0"
  "VAI SYLLABLE KI\0"
  "VAI SYLLABLE KO\0"
  "VAI SYLLABLE KOO\0"
  "VAI SYLLABLE KPA\0"
  "VAI SYLLABLE KPAN\0"
  "VAI SYLLABLE KPE\0"
  "VAI SYLLABLE KPEE\0"
  "VAI SYLLABLE KPEN\0"
  "VAI SYLLABLE KPI\0"
  "VAI SYLLABLE KPO\0"
  "VAI SYLLABLE KPOO\0"
  "VAI SYLLABLE KPU\0"
  "VAI SYLLABLE KU\0"
  "VAI SYLLABLE LA\0"
  "VAI SYLLABLE LE\0"
  "VAI SYLLABLE LEE\0"
  "VAI SYLLABLE LENGTHENER\0"
  "VAI SYLLABLE LI\0"
  "VAI SYLLABLE LO\0"
  "VAI SYLLABLE LOO\0"
  "VAI SYLLABLE LU\0"
  "VAI SYLLABLE MA\0"
  "VAI SYLLABLE MBA\0"
  "VAI SYLLABLE MBE\0"
  "VAI SYLLABLE MBEE\0"
  "VAI SYLLABLE MBI\0"
  "VAI SYLLABLE MBO\0"
  "VAI SYLLABLE MBOO\0"
  "VAI SYLLABLE MBU\0"
  "VAI SYLLABLE ME\0"
  "VAI SYLLABLE MEE\0"
  "VAI SYLLABLE MGBA\0"
  "VAI SYLLABLE MGBE\0"
  "VAI SYLLABLE MGBEE\0"
  "VAI SYLLABLE MGBI\0"
  "VAI SYLLABLE MGBO\0"
  "VAI SYLLABLE MGBOO\0"
  "VAI SYLLABLE MGBU\0"
  "VAI SYLLABLE MI\0"
  "VAI SYLLABLE MO\0"
  "VAI SYLLABLE MOO\0"
  "VAI SYLLABLE MU\0"
  "VAI SYLLABLE NA\0"
  "VAI SYLLABLE NDA\0"
  "VAI SYLLABLE NDE\0"
  "VAI SYLLABLE NDEE\0"
  "VAI SYLLABLE NDI\0"
  "VAI SYLLABLE NDO\0"
  "VAI SYLLABLE NDOLE DO\0"
  "VAI SYLLABLE NDOLE FA\0"
  "VAI SYLLABLE NDOLE KA\0"
  "VAI SYLLABLE NDOLE MA\0"
  "VAI SYLLABLE NDOLE SOO\0"
  "VAI SYLLABLE NDOO\0"
  "VAI SYLLABLE NDU\0"
  "VAI SYLLABLE NE\0"
  "VAI SYLLABLE NEE\0"
  "VAI SYLLABLE NG\0"
  "VAI SYLLABLE NGAN\0"
  "VAI SYLLABLE NGEN\0"
  "VAI SYLLABLE NGGA\0"
  "VAI SYLLABLE NGGE\0"
  "VAI SYLLABLE NGGEE\0"
  "VAI SYLLABLE NGGEN\0"
  "VAI SYLLABLE NGGI\0"
  "VAI SYLLABLE NGGO\0"
  "VAI SYLLABLE NGGOO\0"
  "VAI SYLLABLE NGGU\0"
  "VAI SYLLABLE NGON\0"
  "VAI SYLLABLE NI\0"
  "VAI SYLLABLE NJA\0"
  "VAI SYLLABLE NJE\0"
  "VAI SYLLABLE NJEE\0"
  "VAI SYLLABLE NJI\0"
  "VAI SYLLABLE NJO\0"
  "VAI SYLLABLE NJOO\0"
  "VAI SYLLABLE NJU\0"
  "VAI SYLLABLE NO\0"
  "VAI SYLLABLE NOO\0"
  "VAI SYLLABLE NU\0"
  "VAI SYLLABLE NYA\0"
  "VAI SYLLABLE NYE\0"
  "VAI SYLLABLE NYEE\0"
  "VAI SYLLABLE NYI\0"
  "VAI SYLLABLE NYO\0"
  "VAI SYLLABLE NYOO\0"
  "VAI SYLLABLE NYU\0"
  "VAI SYLLABLE O\0"
  "VAI SYLLABLE ON\0"
  "VAI SYLLABLE OO\0"
  "VAI SYLLABLE OON\0"
  "VAI SYLLABLE PA\0"
  "VAI SYLLABLE PE\0"
  "VAI SYLLABLE PEE\0"
  "VAI SYLLABLE PI\0"
  "VAI SYLLABLE PO\0"
  "VAI SYLLABLE POO\0"
  "VAI SYLLABLE PU\0"
  "VAI SYLLABLE RA\0"
  "VAI SYLLABLE RE\0"
  "VAI SYLLABLE REE\0"
  "VAI SYLLABLE RI\0"
  "VAI SYLLABLE RO\0"
  "VAI SYLLABLE ROO\0"
  "VAI SYLLABLE RU\0"
  "VAI SYLLABLE SA\0"
  "VAI SYLLABLE SE\0"
  "VAI SYLLABLE SEE\0"
  "VAI SYLLABLE SHA\0"
  "VAI SYLLABLE SHE\0"
  "VAI SYLLABLE SHEE\0"
  "VAI SYLLABLE SHI\0"
  "VAI SYLLABLE SHO\0"
  "VAI SYLLABLE SHOO\0"
  "VAI SYLLABLE SHU\0"
  "VAI SYLLABLE SI\0"
  "VAI SYLLABLE SO\0"
  "VAI SYLLABLE SOO\0"
  "VAI SYLLABLE SU\0"
  "VAI SYLLABLE TA\0"
  "VAI SYLLABLE TE\0"
  "VAI SYLLABLE TEE\0"
  "VAI SYLLABLE THA\0"
  "VAI SYLLABLE THE\0"
  "VAI SYLLABLE THEE\0"
  "VAI SYLLABLE THI\0"
  "VAI SYLLABLE THO\0"
  "VAI SYLLABLE THOO\0"
  "VAI SYLLABLE THU\0"
  "VAI SYLLABLE TI\0"
  "VAI SYLLABLE TO\0"
  "VAI SYLLABLE TOO\0"
  "VAI SYLLABLE TU\0"
  "VAI SYLLABLE U\0"
  "VAI SYLLABLE UN\0"
  "VAI SYLLABLE VA\0"
  "VAI SYLLABLE VE\0"
  "VAI SYLLABLE VEE\0"
  "VAI SYLLABLE VI\0"
  "VAI SYLLABLE VO\0"
  "VAI SYLLABLE VOO\0"
  "VAI SYLLABLE VU\0"
  "VAI SYLLABLE WA\0"
  "VAI SYLLABLE WAN\0"
  "VAI SYLLABLE WE\0"
  "VAI SYLLABLE WEE\0"
  "VAI SYLLABLE WEEN\0"
  "VAI SYLLABLE WEN\0"
  "VAI SYLLABLE WI\0"
  "VAI SYLLABLE WIN\0"
  "VAI SYLLABLE WO\0"
  "VAI SYLLABLE WON\0"
  "VAI SYLLABLE WOO\0"
  "VAI SYLLABLE WOON\0"
  "VAI SYLLABLE WU\0"
  "VAI SYLLABLE WUN\0"
  "VAI SYLLABLE YA\0"
  "VAI SYLLABLE YE\0"
  "VAI SYLLABLE YEE\0"
  "VAI SYLLABLE YI\0"
  "VAI SYLLABLE YO\0"
  "VAI SYLLABLE YOO\0"
  "VAI SYLLABLE YU\0"
  "VAI SYLLABLE ZA\0"
  "VAI SYLLABLE ZE\0"
  "VAI SYLLABLE ZEE\0"
  "VAI SYLLABLE ZHA\0"
  "VAI SYLLABLE ZHE\0"
  "VAI SYLLABLE ZHEE\0"
  "VAI SYLLABLE ZHI\0"
  "VAI SYLLABLE ZHO\0"
  "VAI SYLLABLE ZHOO\0"
  "VAI SYLLABLE ZHU\0"
  "VAI SYLLABLE ZI\0"
  "VAI SYLLABLE ZO\0"
  "VAI SYLLABLE ZOO\0"
  "VAI SYLLABLE ZU\0"
  "VAI SYMBOL BANG\0"
  "VAI SYMBOL DANG\0"
  "VAI SYMBOL DO-O\0"
  "VAI SYMBOL DOONG\0"
  "VAI SYMBOL FAA\0"
  "VAI SYMBOL FEENG\0"
  "VAI SYMBOL JONG\0"
  "VAI SYMBOL KEENG\0"
  "VAI SYMBOL KUNG\0"
  "VAI SYMBOL NII\0"
  "VAI SYMBOL TAA\0"
  "VAI SYMBOL TING\0"
  "VAI SYMBOL TONG\0"
  "VAMPIRE\0"
  "VARIATION SELECTOR-1\0"
  "VARIATION SELECTOR-10\0"
  "VARIATION SELECTOR-100\0"
  "VARIATION SELECTOR-101\0"
  "VARIATION SELECTOR-102\0"
  "VARIATION SELECTOR-103\0"
  "VARIATION SELECTOR-104\0"
  "VARIATION SELECTOR-105\0"
  "VARIATION SELECTOR-106\0"
  "VARIATION SELECTOR-107\0"
  "VARIATION SELECTOR-108\0"
  "VARIATION SELECTOR-109\0"
  "VARIATION SELECTOR-11\0"
  "VARIATION SELECTOR-110\0"
  "VARIATION SELECTOR-111\0"
  "VARIATION SELECTOR-112\0"
  "VARIATION SELECTOR-113\0"
  "VARIATION SELECTOR-114\0"
  "VARIATION SELECTOR-115\0"
  "VARIATION SELECTOR-116\0"
  "VARIATION SELECTOR-117\0"
  "VARIATION SELECTOR-118\0"
  "VARIATION SELECTOR-119\0"
  "VARIATION SELECTOR-12\0"
  "VARIATION SELECTOR-120\0"
  "VARIATION SELECTOR-121\0"
  "VARIATION SELECTOR-122\0"
  "VARIATION SELECTOR-123\0"
  "VARIATION SELECTOR-124\0"
  "VARIATION SELECTOR-125\0"
  "VARIATION SELECTOR-126\0"
  "VARIATION SELECTOR-127\0"
  "VARIATION SELECTOR-128\0"
  "VARIATION SELECTOR-129\0"
  "VARIATION SELECTOR-13\0"
  "VARIATION SELECTOR-130\0"
  "VARIATION SELECTOR-131\0"
  "VARIATION SELECTOR-132\0"
  "VARIATION SELECTOR-133\0"
  "VARIATION SELECTOR-134\0"
  "VARIATION SELECTOR-135\0"
  "VARIATION SELECTOR-136\0"
  "VARIATION SELECTOR-137\0"
  "VARIATION SELECTOR-138\0"
  "VARIATION SELECTOR-139\0"
  "VARIATION SELECTOR-14\0"
  "VARIATION SELECTOR-140\0"
  "VARIATION SELECTOR-141\0"
  "VARIATION SELECTOR-142\0"
  "VARIATION SELECTOR-143\0"
  "VARIATION SELECTOR-144\0"
  "VARIATION SELECTOR-145\0"
  "VARIATION SELECTOR-146\0"
  "VARIATION SELECTOR-147\0"
  "VARIATION SELECTOR-148\0"
  "VARIATION SELECTOR-149\0"
  "VARIATION SELECTOR-15\0"
  "VARIATION SELECTOR-150\0"
  "VARIATION SELECTOR-151\0"
  "VARIATION SELECTOR-152\0"
  "VARIATION SELECTOR-153\0"
  "VARIATION SELECTOR-154\0"
  "VARIATION SELECTOR-155\0"
  "VARIATION SELECTOR-156\0"
  "VARIATION SELECTOR-157\0"
  "VARIATION SELECTOR-158\0"
  "VARIATION SELECTOR-159\0"
  "VARIATION SELECTOR-16\0"
  "VARIATION SELECTOR-160\0"
  "VARIATION SELECTOR-161\0"
  "VARIATION SELECTOR-162\0"
  "VARIATION SELECTOR-163\0"
  "VARIATION SELECTOR-164\0"
  "VARIATION SELECTOR-165\0"
  "VARIATION SELECTOR-166\0"
  "VARIATION SELECTOR-167\0"
  "VARIATION SELECTOR-168\0"
  "VARIATION SELECTOR-169\0"
  "VARIATION SELECTOR-17\0"
  "VARIATION SELECTOR-170\0"
  "VARIATION SELECTOR-171\0"
  "VARIATION SELECTOR-172\0"
  "VARIATION SELECTOR-173\0"
  "VARIATION SELECTOR-174\0"
  "VARIATION SELECTOR-175\0"
  "VARIATION SELECTOR-176\0"
  "VARIATION SELECTOR-177\0"
  "VARIATION SELECTOR-178\0"
  "VARIATION SELECTOR-179\0"
  "VARIATION SELECTOR-18\0"
  "VARIATION SELECTOR-180\0"
  "VARIATION SELECTOR-181\0"
  "VARIATION SELECTOR-182\0"
  "VARIATION SELECTOR-183\0"
  "VARIATION SELECTOR-184\0"
  "VARIATION SELECTOR-185\0"
  "VARIATION SELECTOR-186\0"
  "VARIATION SELECTOR-187\0"
  "VARIATION SELECTOR-188\0"
  "VARIATION SELECTOR-189\0"
  "VARIATION SELECTOR-19\0"
  "VARIATION SELECTOR-190\0"
  "VARIATION SELECTOR-191\0"
  "VARIATION SELECTOR-192\0"
  "VARIATION SELECTOR-193\0"
  "VARIATION SELECTOR-194\0"
  "VARIATION SELECTOR-195\0"
  "VARIATION SELECTOR-196\0"
  "VARIATION SELECTOR-197\0"
  "VARIATION SELECTOR-198\0"
  "VARIATION SELECTOR-199\0"
  "VARIATION SELECTOR-2\0"
  "VARIATION SELECTOR-20\0"
  "VARIATION SELECTOR-200\0"
  "VARIATION SELECTOR-201\0"
  "VARIATION SELECTOR-202\0"
  "VARIATION SELECTOR-203\0"
  "VARIATION SELECTOR-204\0"
  "VARIATION SELECTOR-205\0"
  "VARIATION SELECTOR-206\0"
  "VARIATION SELECTOR-207\0"
  "VARIATION SELECTOR-208\0"
  "VARIATION SELECTOR-209\0"
  "VARIATION SELECTOR-21\0"
  "VARIATION SELECTOR-210\0"
  "VARIATION SELECTOR-211\0"
  "VARIATION SELECTOR-212\0"
  "VARIATION SELECTOR-213\0"
  "VARIATION SELECTOR-214\0"
  "VARIATION SELECTOR-215\0"
  "VARIATION SELECTOR-216\0"
  "VARIATION SELECTOR-217\0"
  "VARIATION SELECTOR-218\0"
  "VARIATION SELECTOR-219\0"
  "VARIATION SELECTOR-22\0"
  "VARIATION SELECTOR-220\0"
  "VARIATION SELECTOR-221\0"
  "VARIATION SELECTOR-222\0"
  "VARIATION SELECTOR-223\0"
  "VARIATION SELECTOR-224\0"
  "VARIATION SELECTOR-225\0"
  "VARIATION SELECTOR-226\0"
  "VARIATION SELECTOR-227\0"
  "VARIATION SELECTOR-228\0"
  "VARIATION SELECTOR-229\0"
  "VARIATION SELECTOR-23\0"
  "VARIATION SELECTOR-230\0"
  "VARIATION SELECTOR-231\0"
  "VARIATION SELECTOR-232\0"
  "VARIATION SELECTOR-233\0"
  "VARIATION SELECTOR-234\0"
  "VARIATION SELECTOR-235\0"
  "VARIATION SELECTOR-236\0"
  "VARIATION SELECTOR-237\0"
  "VARIATION SELECTOR-238\0"
  "VARIATION SELECTOR-239\0"
  "VARIATION SELECTOR-24\0"
  "VARIATION SELECTOR-240\0"
  "VARIATION SELECTOR-241\0"
  "VARIATION SELECTOR-242\0"
  "VARIATION SELECTOR-243\0"
  "VARIATION SELECTOR-244\0"
  "VARIATION SELECTOR-245\0"
  "VARIATION SELECTOR-246\0"
  "VARIATION SELECTOR-247\0"
  "VARIATION SELECTOR-248\0"
  "VARIATION SELECTOR-249\0"
  "VARIATION SELECTOR-25\0"
  "VARIATION SELECTOR-250\0"
  "VARIATION SELECTOR-251\0"
  "VARIATION SELECTOR-252\0"
  "VARIATION SELECTOR-253\0"
  "VARIATION SELECTOR-254\0"
  "VARIATION SELECTOR-255\0"
  "VARIATION SELECTOR-256\0"
  "VARIATION SELECTOR-26\0"
  "VARIATION SELECTOR-27\0"
  "VARIATION SELECTOR-28\0"
  "VARIATION SELECTOR-29\0"
  "VARIATION SELECTOR-3\0"
  "VARIATION SELECTOR-30\0"
  "VARIATION SELECTOR-31\0"
  "VARIATION SELECTOR-32\0"
  "VARIATION SELECTOR-33\0"
  "VARIATION SELECTOR-34\0"
  "VARIATION SELECTOR-35\0"
  "VARIATION SELECTOR-36\0"
  "VARIATION SELECTOR-37\0"
  "VARIATION SELECTOR-38\0"
  "VARIATION SELECTOR-39\0"
  "VARIATION SELECTOR-4\0"
  "VARIATION SELECTOR-40\0"
  "VARIATION SELECTOR-41\0"
  "VARIATION SELECTOR-42\0"
  "VARIATION SELECTOR-43\0"
  "VARIATION SELECTOR-44\0"
  "VARIATION SELECTOR-45\0"
  "VARIATION SELECTOR-46\0"
  "VARIATION SELECTOR-47\0"
  "VARIATION SELECTOR-48\0"
  "VARIATION SELECTOR-49\0"
  "VARIATION SELECTOR-5\0"
  "VARIATION SELECTOR-50\0"
  "VARIATION SELECTOR-51\0"
  "VARIATION SELECTOR-52\0"
  "VARIATION SELECTOR-53\0"
  "VARIATION SELECTOR-54\0"
  "VARIATION SELECTOR-55\0"
  "VARIATION SELECTOR-56\0"
  "VARIATION SELECTOR-57\0"
  "VARIATION SELECTOR-58\0"
  "VARIATION SELECTOR-59\0"
  "VARIATION SELECTOR-6\0"
  "VARIATION SELECTOR-60\0"
  "VARIATION SELECTOR-61\0"
  "VARIATION SELECTOR-62\0"
  "VARIATION SELECTOR-63\0"
  "VARIATION SELECTOR-64\0"
  "VARIATION SELECTOR-65\0"
  "VARIATION SELECTOR-66\0"
  "VARIATION SELECTOR-67\0"
  "VARIATION SELECTOR-68\0"
  "VARIATION SELECTOR-69\0"
  "VARIATION SELECTOR-7\0"
  "VARIATION SELECTOR-70\0"
  "VARIATION SELECTOR-71\0"
  "VARIATION SELECTOR-72\0"
  "VARIATION SELECTOR-73\0"
  "VARIATION SELECTOR-74\0"
  "VARIATION SELECTOR-75\0"
  "VARIATION SELECTOR-76\0"
  "VARIATION SELECTOR-77\0"
  "VARIATION SELECTOR-78\0"
  "VARIATION SELECTOR-79\0"
  "VARIATION SELECTOR-8\0"
  "VARIATION SELECTOR-80\0"
  "VARIATION SELECTOR-81\0"
  "VARIATION SELECTOR-82\0"
  "VARIATION SELECTOR-83\0"
  "VARIATION SELECTOR-84\0"
  "VARIATION SELECTOR-85\0"
  "VARIATION SELECTOR-86\0"
  "VARIATION SELECTOR-87\0"
  "VARIATION SELECTOR-88\0"
  "VARIATION SELECTOR-89\0"
  "VARIATION SELECTOR-9\0"
  "VARIATION SELECTOR-90\0"
  "VARIATION SELECTOR-91\0"
  "VARIATION SELECTOR-92\0"
  "VARIATION SELECTOR-93\0"
  "VARIATION SELECTOR-94\0"
  "VARIATION SELECTOR-95\0"
  "VARIATION SELECTOR-96\0"
  "VARIATION SELECTOR-97\0"
  "VARIATION SELECTOR-98\0"
  "VARIATION SELECTOR-99\0"
  "VECTOR OR CROSS PRODUCT\0"
  "VEDIC SIGN ANUSVARA ANTARGOMUKHA\0"
  "VEDIC SIGN ANUSVARA BAHIRGOMUKHA\0"
  "VEDIC SIGN ANUSVARA UBHAYATO MUKHA\0"
  "VEDIC SIGN ANUSVARA VAMAGOMUKHA\0"
  "VEDIC SIGN ANUSVARA VAMAGOMUKHA WITH TAIL\0"
  "VEDIC SIGN ARDHAVISARGA\0"
  "VEDIC SIGN ATIKRAMA\0"
  "VEDIC SIGN DOUBLE ANUSVARA ANTARGOMUKHA\0"
  "VEDIC SIGN HEXIFORM LONG ANUSVARA\0"
  "VEDIC SIGN JIHVAMULIYA\0"
  "VEDIC SIGN LONG ANUSVARA\0"
  "VEDIC SIGN NIHSHVASA\0"
  "VEDIC SIGN REVERSED VISARGA ANUDATTA\0"
  "VEDIC SIGN REVERSED VISARGA UDATTA\0"
  "VEDIC SIGN ROTATED ARDHAVISARGA\0"
  "VEDIC SIGN RTHANG LONG ANUSVARA\0"
  "VEDIC SIGN TIRYAK\0"
  "VEDIC SIGN UPADHMANIYA\0"
  "VEDIC SIGN VISARGA ANUDATTA\0"
  "VEDIC SIGN VISARGA ANUDATTA WITH TAIL\0"
  "VEDIC SIGN VISARGA SVARITA\0"
  "VEDIC SIGN VISARGA UDATTA\0"
  "VEDIC SIGN VISARGA UDATTA WITH TAIL\0"
  "VEDIC SIGN YAJURVEDIC MIDLINE SVARITA\0"
  "VEDIC TONE ATHARVAVEDIC INDEPENDENT SVARITA\0"
  "VEDIC TONE CANDRA ABOVE\0"
  "VEDIC TONE CANDRA BELOW\0"
  "VEDIC TONE DOT BELOW\0"
  "VEDIC TONE DOUBLE RING ABOVE\0"
  "VEDIC TONE DOUBLE SVARITA\0"
  "VEDIC TONE KARSHANA\0"
  "VEDIC TONE KATHAKA ANUDATTA\0"
  "VEDIC TONE PRENKHA\0"
  "VEDIC TONE RIGVEDIC KASHMIRI INDEPENDENT SVARITA\0"
  "VEDIC TONE RING ABOVE\0"
  "VEDIC TONE SHARA\0"
  "VEDIC TONE THREE DOTS BELOW\0"
  "VEDIC TONE TRIPLE SVARITA\0"
  "VEDIC TONE TWO DOTS BELOW\0"
  "VEDIC TONE YAJURVEDIC AGGRAVATED INDEPENDENT SVARITA\0"
  "VEDIC TONE YAJURVEDIC INDEPENDENT SVARITA\0"
  "VEDIC TONE YAJURVEDIC KATHAKA INDEPENDENT SVARITA\0"
  "VEDIC TONE YAJURVEDIC KATHAKA INDEPENDENT SVARITA SCHROEDER\0"
  "VERSICLE\0"
  "VERTICAL BAR BESIDE RIGHT TRIANGLE\0"
  "VERTICAL BAR DOUBLE LEFT TURNSTILE\0"
  "VERTICAL BAR TRIPLE RIGHT TURNSTILE\0"
  "VERTICAL BAR WITH HORIZONTAL STROKE\0"
  "VERTICAL CAPACITOR\0"
  "VERTICAL ELLIPSIS\0"
  "VERTICAL FOUR DOTS\0"
  "VERTICAL GO-KART\0"
  "VERTICAL IDEOGRAPHIC ITERATION MARK\0"
  "VERTICAL KANA REPEAT MARK\0"
  "VERTICAL KANA REPEAT MARK LOWER HALF\0"
  "VERTICAL KANA REPEAT MARK UPPER HALF\0"
  "VERTICAL KANA REPEAT WITH VOICED SOUND MARK\0"
  "VERTICAL KANA REPEAT WITH VOICED SOUND MARK UPPER HALF\0"
  "VERTICAL LADDER\0"
  "VERTICAL LINE\0"
  "VERTICAL LINE EXTENSION\0"
  "VERTICAL LINE WITH CIRCLE ABOVE\0"
  "VERTICAL LINE WITH CIRCLE BELOW\0"
  "VERTICAL LINE WITH FOUR TICK MARKS\0"
  "VERTICAL LINE WITH MIDDLE DOT\0"
  "VERTICAL LINE WITH THREE TICK MARKS\0"
  "VERTICAL MALE WITH STROKE SIGN\0"
  "VERTICAL ONE EIGHTH BLOCK-2\0"
  "VERTICAL ONE EIGHTH BLOCK-3\0"
  "VERTICAL ONE EIGHTH BLOCK-4\0"
  "VERTICAL ONE EIGHTH BLOCK-5\0"
  "VERTICAL ONE EIGHTH BLOCK-6\0"
  "VERTICAL ONE EIGHTH BLOCK-7\0"
  "VERTICAL RACING CAR\0"
  "VERTICAL RESISTOR SEGMENT\0"
  "VERTICAL SIX DOTS\0"
  "VERTICAL TAB KEY\0"
  "VERTICAL TILDE\0"
  "VERTICAL TRAFFIC LIGHT\0"
  "VERTICAL ZIGZAG LINE\0"
  "VERY BOLD GREEK CROSS\0"
  "VERY HEAVY EIGHT POINTED BLACK STAR\0"
  "VERY HEAVY EIGHT SPOKED ASTERISK\0"
  "VERY HEAVY FIVE SPOKED ASTERISK\0"
  "VERY HEAVY GREEK CROSS\0"
  "VERY HEAVY REVERSE SOLIDUS\0"
  "VERY HEAVY SALTIRE\0"
  "VERY HEAVY SIX SPOKED ASTERISK\0"
  "VERY HEAVY SOLIDUS\0"
  "VERY HEAVY WHITE CIRCLE\0"
  "VERY HEAVY WHITE SQUARE\0"
  "VERY MUCH GREATER-THAN\0"
  "VERY MUCH LESS-THAN\0"
  "VESTA\0"
  "VIBRATION MODE\0"
  "VICTORY HAND\0"
  "VIDEO CAMERA\0"
  "VIDEO GAME\0"
  "VIDEOCASSETTE\0"
  "VIETNAMESE ALTERNATE READING MARK CA\0"
  "VIETNAMESE ALTERNATE READING MARK NHAY\0"
  "VIEWDATA SQUARE\0"
  "VIOLIN\0"
  "VIRGO\0"
  "VITHKUQI CAPITAL LETTER A\0"
  "VITHKUQI CAPITAL LETTER BBE\0"
  "VITHKUQI CAPITAL LETTER BE\0"
  "VITHKUQI CAPITAL LETTER CE\0"
  "VITHKUQI CAPITAL LETTER CHE\0"
  "VITHKUQI CAPITAL LETTER DE\0"
  "VITHKUQI CAPITAL LETTER DHE\0"
  "VITHKUQI CAPITAL LETTER E\0"
  "VITHKUQI CAPITAL LETTER EI\0"
  "VITHKUQI CAPITAL LETTER FE\0"
  "VITHKUQI CAPITAL LETTER GA\0"
  "VITHKUQI CAPITAL LETTER HA\0"
  "VITHKUQI CAPITAL LETTER HHA\0"
  "VITHKUQI CAPITAL LETTER I\0"
  "VITHKUQI CAPITAL LETTER IJE\0"
  "VITHKUQI CAPITAL LETTER JE\0"
  "VITHKUQI CAPITAL LETTER KA\0"
  "VITHKUQI CAPITAL LETTER LA\0"
  "VITHKUQI CAPITAL LETTER LLA\0"
  "VITHKUQI CAPITAL LETTER ME\0"
  "VITHKUQI CAPITAL LETTER NE\0"
  "VITHKUQI CAPITAL LETTER NJE\0"
  "VITHKUQI CAPITAL LETTER O\0"
  "VITHKUQI CAPITAL LETTER PE\0"
  "VITHKUQI CAPITAL LETTER QA\0"
  "VITHKUQI CAPITAL LETTER RE\0"
  "VITHKUQI CAPITAL LETTER SE\0"
  "VITHKUQI CAPITAL LETTER SHE\0"
  "VITHKUQI CAPITAL LETTER TE\0"
  "VITHKUQI CAPITAL LETTER THE\0"
  "VITHKUQI CAPITAL LETTER U\0"
  "VITHKUQI CAPITAL LETTER VE\0"
  "VITHKUQI CAPITAL LETTER XE\0"
  "VITHKUQI CAPITAL LETTER Y\0"
  "VITHKUQI CAPITAL LETTER ZE\0"
  "VITHKUQI SMALL LETTER A\0"
  "VITHKUQI SMALL LETTER BBE\0"
  "VITHKUQI SMALL LETTER BE\0"
  "VITHKUQI SMALL LETTER CE\0"
  "VITHKUQI SMALL LETTER CHE\0"
  "VITHKUQI SMALL LETTER DE\0"
  "VITHKUQI SMALL LETTER DHE\0"
  "VITHKUQI SMALL LETTER E\0"
  "VITHKUQI SMALL LETTER EI\0"
  "VITHKUQI SMALL LETTER FE\0"
  "VITHKUQI SMALL LETTER GA\0"
  "VITHKUQI SMALL LETTER HA\0"
  "VITHKUQI SMALL LETTER HHA\0"
  "VITHKUQI SMALL LETTER I\0"
  "VITHKUQI SMALL LETTER IJE\0"
  "VITHKUQI SMALL LETTER JE\0"
  "VITHKUQI SMALL LETTER KA\0"
  "VITHKUQI SMALL LETTER LA\0"
  "VITHKUQI SMALL LETTER LLA\0"
  "VITHKUQI SMALL LETTER ME\0"
  "VITHKUQI SMALL LETTER NE\0"
  "VITHKUQI SMALL LETTER NJE\0"
  "VITHKUQI SMALL LETTER O\0"
  "VITHKUQI SMALL LETTER PE\0"
  "VITHKUQI SMALL LETTER QA\0"
  "VITHKUQI SMALL LETTER RE\0"
  "VITHKUQI SMALL LETTER SE\0"
  "VITHKUQI SMALL LETTER SHE\0"
  "VITHKUQI SMALL LETTER TE\0"
  "VITHKUQI SMALL LETTER THE\0"
  "VITHKUQI SMALL LETTER U\0"
  "VITHKUQI SMALL LETTER VE\0"
  "VITHKUQI SMALL LETTER XE\0"
  "VITHKUQI SMALL LETTER Y\0"
  "VITHKUQI SMALL LETTER ZE\0"
  "VOIDED GREEK CROSS\0"
  "VOLCANO\0"
  "VOLLEYBALL\0"
  "VOLUME INTEGRAL\0"
  "VULCANUS\0"
  "VULGAR FRACTION FIVE EIGHTHS\0"
  "VULGAR FRACTION FIVE SIXTHS\0"
  "VULGAR FRACTION FOUR FIFTHS\0"
  "VULGAR FRACTION ONE EIGHTH\0"
  "VULGAR FRACTION ONE FIFTH\0"
  "VULGAR FRACTION ONE HALF\0"
  "VULGAR FRACTION ONE NINTH\0"
  "VULGAR FRACTION ONE QUARTER\0"
  "VULGAR FRACTION ONE SEVENTH\0"
  "VULGAR FRACTION ONE SIXTH\0"
  "VULGAR FRACTION ONE TENTH\0"
  "VULGAR FRACTION ONE THIRD\0"
  "VULGAR FRACTION SEVEN EIGHTHS\0"
  "VULGAR FRACTION THREE EIGHTHS\0"
  "VULGAR FRACTION THREE FIFTHS\0"
  "VULGAR FRACTION THREE QUARTERS\0"
  "VULGAR FRACTION TWO FIFTHS\0"
  "VULGAR FRACTION TWO THIRDS\0"
  "VULGAR FRACTION ZERO THIRDS\0"
  "WAFFLE\0"
  "WANCHO DIGIT EIGHT\0"
  "WANCHO DIGIT FIVE\0"
  "WANCHO DIGIT FOUR\0"
  "WANCHO DIGIT NINE\0"
  "WANCHO DIGIT ONE\0"
  "WANCHO DIGIT SEVEN\0"
  "WANCHO DIGIT SIX\0"
  "WANCHO DIGIT THREE\0"
  "WANCHO DIGIT TWO\0"
  "WANCHO DIGIT ZERO\0"
  "WANCHO LETTER A\0"
  "WANCHO LETTER AA\0"
  "WANCHO LETTER AAN\0"
  "WANCHO LETTER AANG\0"
  "WANCHO LETTER ANG\0"
  "WANCHO LETTER AU\0"
  "WANCHO LETTER BA\0"
  "WANCHO LETTER CA\0"
  "WANCHO LETTER DA\0"
  "WANCHO LETTER E\0"
  "WANCHO LETTER EN\0"
  "WANCHO LETTER FA\0"
  "WANCHO LETTER GA\0"
  "WANCHO LETTER HA\0"
  "WANCHO LETTER I\0"
  "WANCHO LETTER ING\0"
  "WANCHO LETTER JA\0"
  "WANCHO LETTER KA\0"
  "WANCHO LETTER KHA\0"
  "WANCHO LETTER LA\0"
  "WANCHO LETTER LLHA\0"
  "WANCHO LETTER MA\0"
  "WANCHO LETTER NA\0"
  "WANCHO LETTER NGA\0"
  "WANCHO LETTER NYA\0"
  "WANCHO LETTER O\0"
  "WANCHO LETTER ON\0"
  "WANCHO LETTER ONG\0"
  "WANCHO LETTER PA\0"
  "WANCHO LETTER PHA\0"
  "WANCHO LETTER RA\0"
  "WANCHO LETTER SA\0"
  "WANCHO LETTER SHA\0"
  "WANCHO LETTER TA\0"
  "WANCHO LETTER THA\0"
  "WANCHO LETTER TRA\0"
  "WANCHO LETTER TSA\0"
  "WANCHO LETTER U\0"
  "WANCHO LETTER UEN\0"
  "WANCHO LETTER VA\0"
  "WANCHO LETTER WA\0"
  "WANCHO LETTER YA\0"
  "WANCHO LETTER YIH\0"
  "WANCHO LETTER ZA\0"
  "WANCHO NGUN SIGN\0"
  "WANCHO TONE KOI\0"
  "WANCHO TONE KOINI\0"
  "WANCHO TONE TUP\0"
  "WANCHO TONE TUPNI\0"
  "WANING CRESCENT MOON SYMBOL\0"
  "WANING GIBBOUS MOON SYMBOL\0"
  "WARANG CITI CAPITAL LETTER A\0"
  "WARANG CITI CAPITAL LETTER AM\0"
  "WARANG CITI CAPITAL LETTER ANG\0"
  "WARANG CITI CAPITAL LETTER AT\0"
  "WARANG CITI CAPITAL LETTER BU\0"
  "WARANG CITI CAPITAL LETTER DA\0"
  "WARANG CITI CAPITAL LETTER E\0"
  "WARANG CITI CAPITAL LETTER ENN\0"
  "WARANG CITI CAPITAL LETTER ENY\0"
  "WARANG CITI CAPITAL LETTER GA\0"
  "WARANG CITI CAPITAL LETTER HAR\0"
  "WARANG CITI CAPITAL LETTER HIYO\0"
  "WARANG CITI CAPITAL LETTER HOLO\0"
  "WARANG CITI CAPITAL LETTER HORR\0"
  "WARANG CITI CAPITAL LETTER II\0"
  "WARANG CITI CAPITAL LETTER KO\0"
  "WARANG CITI CAPITAL LETTER NGAA\0"
  "WARANG CITI CAPITAL LETTER NUNG\0"
  "WARANG CITI CAPITAL LETTER O\0"
  "WARANG CITI CAPITAL LETTER ODD\0"
  "WARANG CITI CAPITAL LETTER PU\0"
  "WARANG CITI CAPITAL LETTER SII\0"
  "WARANG CITI CAPITAL LETTER SSUU\0"
  "WARANG CITI CAPITAL LETTER TTE\0"
  "WARANG CITI CAPITAL LETTER UC\0"
  "WARANG CITI CAPITAL LETTER UU\0"
  "WARANG CITI CAPITAL LETTER VIYO\0"
  "WARANG CITI CAPITAL LETTER WI\0"
  "WARANG CITI CAPITAL LETTER YA\0"
  "WARANG CITI CAPITAL LETTER YO\0"
  "WARANG CITI CAPITAL LETTER YU\0"
  "WARANG CITI CAPITAL LETTER YUJ\0"
  "WARANG CITI DIGIT EIGHT\0"
  "WARANG CITI DIGIT FIVE\0"
  "WARANG CITI DIGIT FOUR\0"
  "WARANG CITI DIGIT NINE\0"
  "WARANG CITI DIGIT ONE\0"
  "WARANG CITI DIGIT SEVEN\0"
  "WARANG CITI DIGIT SIX\0"
  "WARANG CITI DIGIT THREE\0"
  "WARANG CITI DIGIT TWO\0"
  "WARANG CITI DIGIT ZERO\0"
  "WARANG CITI NUMBER EIGHTY\0"
  "WARANG CITI NUMBER FIFTY\0"
  "WARANG CITI NUMBER FORTY\0"
  "WARANG CITI NUMBER NINETY\0"
  "WARANG CITI NUMBER SEVENTY\0"
  "WARANG CITI NUMBER SIXTY\0"
  "WARANG CITI NUMBER TEN\0"
  "WARANG CITI NUMBER THIRTY\0"
  "WARANG CITI NUMBER TWENTY\0"
  "WARANG CITI OM\0"
  "WARANG CITI SMALL LETTER A\0"
  "WARANG CITI SMALL LETTER AM\0"
  "WARANG CITI SMALL LETTER ANG\0"
  "WARANG CITI SMALL LETTER AT\0"
  "WARANG CITI SMALL LETTER BU\0"
  "WARANG CITI SMALL LETTER DA\0"
  "WARANG CITI SMALL LETTER E\0"
  "WARANG CITI SMALL LETTER ENN\0"
  "WARANG CITI SMALL LETTER ENY\0"
  "WARANG CITI SMALL LETTER GA\0"
  "WARANG CITI SMALL LETTER HAR\0"
  "WARANG CITI SMALL LETTER HIYO\0"
  "WARANG CITI SMALL LETTER HOLO\0"
  "WARANG CITI SMALL LETTER HORR\0"
  "WARANG CITI SMALL LETTER II\0"
  "WARANG CITI SMALL LETTER KO\0"
  "WARANG CITI SMALL LETTER NGAA\0"
  "WARANG CITI SMALL LETTER NUNG\0"
  "WARANG CITI SMALL LETTER O\0"
  "WARANG CITI SMALL LETTER ODD\0"
  "WARANG CITI SMALL LETTER PU\0"
  "WARANG CITI SMALL LETTER SII\0"
  "WARANG CITI SMALL LETTER SSUU\0"
  "WARANG CITI SMALL LETTER TTE\0"
  "WARANG CITI SMALL LETTER UC\0"
  "WARANG CITI SMALL LETTER UU\0"
  "WARANG CITI SMALL LETTER VIYO\0"
  "WARANG CITI SMALL LETTER WI\0"
  "WARANG CITI SMALL LETTER YA\0"
  "WARANG CITI SMALL LETTER YO\0"
  "WARANG CITI SMALL LETTER YU\0"
  "WARANG CITI SMALL LETTER YUJ\0"
  "WARNING SIGN\0"
  "WASTEBASKET\0"
  "WATCH\0"
  "WATER BUFFALO\0"
  "WATER CLOSET\0"
  "WATER POLO\0"
  "WATER WAVE\0"
  "WATERMELON\0"
  "WAVE ARROW POINTING DIRECTLY LEFT\0"
  "WAVE ARROW POINTING DIRECTLY RIGHT\0"
  "WAVE DASH\0"
  "WAVING BLACK FLAG\0"
  "WAVING HAND SIGN\0"
  "WAVING WHITE FLAG\0"
  "WAVY DASH\0"
  "WAVY LINE\0"
  "WAVY LOW LINE\0"
  "WAVY OVERLINE\0"
  "WAXING CRESCENT MOON SYMBOL\0"
  "WAXING GIBBOUS MOON SYMBOL\0"
  "WEARY CAT FACE\0"
  "WEARY FACE\0"
  "WEDDING\0"
  "WEDGE-TAILED RIGHTWARDS ARROW\0"
  "WEIGHT LIFTER\0"
  "WEST SYRIAC CROSS\0"
  "WHALE\0"
  "WHEEL\0"
  "WHEEL OF DHARMA\0"
  "WHEELCHAIR SYMBOL\0"
  "WHITE ARROW SHAFT WIDTH ONE\0"
  "WHITE ARROW SHAFT WIDTH TWO THIRDS\0"
  "WHITE BULLET\0"
  "WHITE CHESS BISHOP\0"
  "WHITE CHESS BISHOP ROTATED NINETY DEGREES\0"
  "WHITE CHESS BISHOP ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "WHITE CHESS EQUIHOPPER\0"
  "WHITE CHESS EQUIHOPPER ROTATED NINETY DEGREES\0"
  "WHITE CHESS KING\0"
  "WHITE CHESS KING ROTATED NINETY DEGREES\0"
  "WHITE CHESS KING ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "WHITE CHESS KNIGHT\0"
  "WHITE CHESS KNIGHT ROTATED FORTY-FIVE DEGREES\0"
  "WHITE CHESS KNIGHT ROTATED NINETY DEGREES\0"
  "WHITE CHESS KNIGHT ROTATED ONE HUNDRED THIRTY-FIVE DEGREES\0"
  "WHITE CHESS KNIGHT ROTATED THREE HUNDRED FIFTEEN DEGREES\0"
  "WHITE CHESS KNIGHT ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "WHITE CHESS KNIGHT ROTATED TWO HUNDRED TWENTY-FIVE DEGREES\0"
  "WHITE CHESS KNIGHT-BISHOP\0"
  "WHITE CHESS KNIGHT-QUEEN\0"
  "WHITE CHESS KNIGHT-ROOK\0"
  "WHITE CHESS PAWN\0"
  "WHITE CHESS PAWN ROTATED NINETY DEGREES\0"
  "WHITE CHESS PAWN ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "WHITE CHESS QUEEN\0"
  "WHITE CHESS QUEEN ROTATED NINETY DEGREES\0"
  "WHITE CHESS QUEEN ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "WHITE CHESS ROOK\0"
  "WHITE CHESS ROOK ROTATED NINETY DEGREES\0"
  "WHITE CHESS ROOK ROTATED TWO HUNDRED SEVENTY DEGREES\0"
  "WHITE CHESS TURNED BISHOP\0"
  "WHITE CHESS TURNED KING\0"
  "WHITE CHESS TURNED KNIGHT\0"
  "WHITE CHESS TURNED PAWN\0"
  "WHITE CHESS TURNED QUEEN\0"
  "WHITE CHESS TURNED ROOK\0"
  "WHITE CIRCLE\0"
  "WHITE CIRCLE CONTAINING BLACK SMALL CIRCLE\0"
  "WHITE CIRCLE WITH DOT RIGHT\0"
  "WHITE CIRCLE WITH DOWN ARROW\0"
  "WHITE CIRCLE WITH LOWER LEFT QUADRANT\0"
  "WHITE CIRCLE WITH LOWER RIGHT QUADRANT\0"
  "WHITE CIRCLE WITH TWO DOTS\0"
  "WHITE CIRCLE WITH UPPER LEFT QUADRANT\0"
  "WHITE CIRCLE WITH UPPER RIGHT QUADRANT\0"
  "WHITE CLUB SUIT\0"
  "WHITE CONCAVE-SIDED DIAMOND\0"
  "WHITE CONCAVE-SIDED DIAMOND WITH LEFTWARDS TICK\0"
  "WHITE CONCAVE-SIDED DIAMOND WITH RIGHTWARDS TICK\0"
  "WHITE CROSS MARK\0"
  "WHITE DIAMOND\0"
  "WHITE DIAMOND CONTAINING BLACK MEDIUM DIAMOND\0"
  "WHITE DIAMOND CONTAINING BLACK SMALL DIAMOND\0"
  "WHITE DIAMOND CONTAINING BLACK VERY SMALL DIAMOND\0"
  "WHITE DIAMOND IN SQUARE\0"
  "WHITE DIAMOND SUIT\0"
  "WHITE DIAMOND WITH CENTRED DOT\0"
  "WHITE DOWN POINTING BACKHAND INDEX\0"
  "WHITE DOWN POINTING INDEX\0"
  "WHITE DOWN POINTING LEFT HAND INDEX\0"
  "WHITE DOWN-POINTING SMALL TRIANGLE\0"
  "WHITE DOWN-POINTING TRIANGLE\0"
  "WHITE DRAUGHTS KING\0"
  "WHITE DRAUGHTS MAN\0"
  "WHITE EXCLAMATION MARK ORNAMENT\0"
  "WHITE FLAG\0"
  "WHITE FLAG WITH HORIZONTAL MIDDLE BLACK STRIPE\0"
  "WHITE FLORETTE\0"
  "WHITE FLOWER\0"
  "WHITE FOUR POINTED CUSP\0"
  "WHITE FOUR POINTED STAR\0"
  "WHITE FROWNING FACE\0"
  "WHITE HARD SHELL FLOPPY DISK\0"
  "WHITE HEART\0"
  "WHITE HEART SUIT\0"
  "WHITE HEAVY CHECK MARK\0"
  "WHITE HEAVY SALTIRE WITH ROUNDED CORNERS\0"
  "WHITE HEXAGON\0"
  "WHITE HORIZONTAL ELLIPSE\0"
  "WHITE HOURGLASS\0"
  "WHITE LARGE SQUARE\0"
  "WHITE LATIN CROSS\0"
  "WHITE LEFT LANE MERGE\0"
  "WHITE LEFT POINTING BACKHAND INDEX\0"
  "WHITE LEFT POINTING INDEX\0"
  "WHITE LEFT-POINTING POINTER\0"
  "WHITE LEFT-POINTING SMALL TRIANGLE\0"
  "WHITE LEFT-POINTING TRIANGLE\0"
  "WHITE LOWER LEFT POINTER\0"
  "WHITE LOWER RIGHT POINTER\0"
  "WHITE LOZENGE CONTAINING BLACK SMALL LOZENGE\0"
  "WHITE MEDIUM DIAMOND\0"
  "WHITE MEDIUM LOZENGE\0"
  "WHITE MEDIUM SMALL SQUARE\0"
  "WHITE MEDIUM SQUARE\0"
  "WHITE MEDIUM STAR\0"
  "WHITE MOON SELENA\0"
  "WHITE NIB\0"
  "WHITE PARALLELOGRAM\0"
  "WHITE PENNANT\0"
  "WHITE PENTAGON\0"
  "WHITE QUESTION MARK ORNAMENT\0"
  "WHITE RECTANGLE\0"
  "WHITE RIGHT POINTING BACKHAND INDEX\0"
  "WHITE RIGHT POINTING INDEX\0"
  "WHITE RIGHT-POINTING PENTAGON\0"
  "WHITE RIGHT-POINTING POINTER\0"
  "WHITE RIGHT-POINTING SMALL TRIANGLE\0"
  "WHITE RIGHT-POINTING TRIANGLE\0"
  "WHITE SCISSORS\0"
  "WHITE SESAME DOT\0"
  "WHITE SHOGI PIECE\0"
  "WHITE SMALL LOZENGE\0"
  "WHITE SMALL SQUARE\0"
  "WHITE SMALL STAR\0"
  "WHITE SMILING FACE\0"
  "WHITE SPADE SUIT\0"
  "WHITE SQUARE\0"
  "WHITE SQUARE BUTTON\0"
  "WHITE SQUARE CONTAINING BLACK MEDIUM SQUARE\0"
  "WHITE SQUARE CONTAINING BLACK SMALL SQUARE\0"
  "WHITE SQUARE CONTAINING BLACK VERY SMALL SQUARE\0"
  "WHITE SQUARE WITH CENTRE VERTICAL LINE\0"
  "WHITE SQUARE WITH LEFTWARDS TICK\0"
  "WHITE SQUARE WITH LOWER LEFT QUADRANT\0"
  "WHITE SQUARE WITH LOWER RIGHT QUADRANT\0"
  "WHITE SQUARE WITH RIGHTWARDS TICK\0"
  "WHITE SQUARE WITH ROUNDED CORNERS\0"
  "WHITE SQUARE WITH UPPER LEFT QUADRANT\0"
  "WHITE SQUARE WITH UPPER RIGHT QUADRANT\0"
  "WHITE SQUARE WITH VERTICAL BISECTING LINE\0"
  "WHITE STAR\0"
  "WHITE SUN\0"
  "WHITE SUN BEHIND CLOUD\0"
  "WHITE SUN BEHIND CLOUD WITH RAIN\0"
  "WHITE SUN WITH RAYS\0"
  "WHITE SUN WITH SMALL CLOUD\0"
  "WHITE TELEPHONE\0"
  "WHITE TOUCHTONE TELEPHONE\0"
  "WHITE TRAPEZIUM\0"
  "WHITE TRIANGLE CONTAINING SMALL WHITE TRIANGLE\0"
  "WHITE TWO-WAY LEFT WAY TRAFFIC\0"
  "WHITE UP POINTING BACKHAND INDEX\0"
  "WHITE UP POINTING INDEX\0"
  "WHITE UP-POINTING CHEVRON\0"
  "WHITE UP-POINTING SMALL TRIANGLE\0"
  "WHITE UP-POINTING TRIANGLE\0"
  "WHITE UP-POINTING TRIANGLE WITH DOT\0"
  "WHITE VERTICAL BAR\0"
  "WHITE VERTICAL ELLIPSE\0"
  "WHITE VERTICAL RECTANGLE\0"
  "WHITE VERTICAL RECTANGLE WITH HORIZONTAL BAR\0"
  "WHITE VERY SMALL SQUARE\0"
  "WHITE-FEATHERED RIGHTWARDS ARROW\0"
  "WIDE-HEADED DOWNWARDS BARB ARROW\0"
  "WIDE-HEADED DOWNWARDS HEAVY BARB ARROW\0"
  "WIDE-HEADED DOWNWARDS LIGHT BARB ARROW\0"
  "WIDE-HEADED DOWNWARDS MEDIUM BARB ARROW\0"
  "WIDE-HEADED DOWNWARDS VERY HEAVY BARB ARROW\0"
  "WIDE-HEADED LEFTWARDS BARB ARROW\0"
  "WIDE-HEADED LEFTWARDS HEAVY BARB ARROW\0"
  "WIDE-HEADED LEFTWARDS LIGHT BARB ARROW\0"
  "WIDE-HEADED LEFTWARDS MEDIUM BARB ARROW\0"
  "WIDE-HEADED LEFTWARDS VERY HEAVY BARB ARROW\0"
  "WIDE-HEADED NORTH EAST BARB ARROW\0"
  "WIDE-HEADED NORTH EAST HEAVY BARB ARROW\0"
  "WIDE-HEADED NORTH EAST LIGHT BARB ARROW\0"
  "WIDE-HEADED NORTH EAST MEDIUM BARB ARROW\0"
  "WIDE-HEADED NORTH EAST VERY HEAVY BARB ARROW\0"
  "WIDE-HEADED NORTH WEST BARB ARROW\0"
  "WIDE-HEADED NORTH WEST HEAVY BARB ARROW\0"
  "WIDE-HEADED NORTH WEST LIGHT BARB ARROW\0"
  "WIDE-HEADED NORTH WEST MEDIUM BARB ARROW\0"
  "WIDE-HEADED NORTH WEST VERY HEAVY BARB ARROW\0"
  "WIDE-HEADED RIGHTWARDS BARB ARROW\0"
  "WIDE-HEADED RIGHTWARDS HEAVY BARB ARROW\0"
  "WIDE-HEADED RIGHTWARDS LIGHT BARB ARROW\0"
  "WIDE-HEADED RIGHTWARDS MEDIUM BARB ARROW\0"
  "WIDE-HEADED RIGHTWARDS VERY HEAVY BARB ARROW\0"
  "WIDE-HEADED SOUTH EAST BARB ARROW\0"
  "WIDE-HEADED SOUTH EAST HEAVY BARB ARROW\0"
  "WIDE-HEADED SOUTH EAST LIGHT BARB ARROW\0"
  "WIDE-HEADED SOUTH EAST MEDIUM BARB ARROW\0"
  "WIDE-HEADED SOUTH EAST VERY HEAVY BARB ARROW\0"
  "WIDE-HEADED SOUTH WEST BARB ARROW\0"
  "WIDE-HEADED SOUTH WEST HEAVY BARB ARROW\0"
  "WIDE-HEADED SOUTH WEST LIGHT BARB ARROW\0"
  "WIDE-HEADED SOUTH WEST MEDIUM BARB ARROW\0"
  "WIDE-HEADED SOUTH WEST VERY HEAVY BARB ARROW\0"
  "WIDE-HEADED UPWARDS BARB ARROW\0"
  "WIDE-HEADED UPWARDS HEAVY BARB ARROW\0"
  "WIDE-HEADED UPWARDS LIGHT BARB ARROW\0"
  "WIDE-HEADED UPWARDS MEDIUM BARB ARROW\0"
  "WIDE-HEADED UPWARDS VERY HEAVY BARB ARROW\0"
  "WIGGLY VERTICAL LINE\0"
  "WILTED FLOWER\0"
  "WIND BLOWING FACE\0"
  "WIND CHIME\0"
  "WINDOW\0"
  "WINE GLASS\0"
  "WING\0"
  "WINKING FACE\0"
  "WIRED KEYBOARD\0"
  "WIRELESS\0"
  "WOLF FACE\0"
  "WOMAN\0"
  "WOMAN WITH BUNNY EARS\0"
  "WOMANS BOOTS\0"
  "WOMANS CLOTHES\0"
  "WOMANS HAT\0"
  "WOMANS SANDAL\0"
  "WOMENS SYMBOL\0"
  "WON SIGN\0"
  "WOOD\0"
  "WORD JOINER\0"
  "WORD SEPARATOR MIDDLE DOT\0"
  "WORLD MAP\0"
  "WORM\0"
  "WORRIED FACE\0"
  "WRAPPED PRESENT\0"
  "WREATH PRODUCT\0"
  "WRENCH\0"
  "WRESTLERS\0"
  "WRITING HAND\0"
  "X IN A RECTANGLE BOX\0"
  "X-RAY\0"
  "XIANGQI BLACK CANNON\0"
  "XIANGQI BLACK CHARIOT\0"
  "XIANGQI BLACK ELEPHANT\0"
  "XIANGQI BLACK GENERAL\0"
  "XIANGQI BLACK HORSE\0"
  "XIANGQI BLACK MANDARIN\0"
  "XIANGQI BLACK SOLDIER\0"
  "XIANGQI RED CANNON\0"
  "XIANGQI RED CHARIOT\0"
  "XIANGQI RED ELEPHANT\0"
  "XIANGQI RED GENERAL\0"
  "XIANGQI RED HORSE\0"
  "XIANGQI RED MANDARIN\0"
  "XIANGQI RED SOLDIER\0"
  "XOR\0"
  "YAWNING FACE\0"
  "YELLOW HEART\0"
  "YEN SIGN\0"
  "YEZIDI COMBINING HAMZA MARK\0"
  "YEZIDI COMBINING MADDA MARK\0"
  "YEZIDI HYPHENATION MARK\0"
  "YEZIDI LETTER BE\0"
  "YEZIDI LETTER CHHIM\0"
  "YEZIDI LETTER CHIM\0"
  "YEZIDI LETTER CIM\0"
  "YEZIDI LETTER DAD\0"
  "YEZIDI LETTER DAL\0"
  "YEZIDI LETTER ELIF\0"
  "YEZIDI LETTER ET\0"
  "YEZIDI LETTER EW\0"
  "YEZIDI LETTER EYN\0"
  "YEZIDI LETTER FA\0"
  "YEZIDI LETTER GAF\0"
  "YEZIDI LETTER HAY\0"
  "YEZIDI LETTER HHA\0"
  "YEZIDI LETTER JA\0"
  "YEZIDI LETTER KAF\0"
  "YEZIDI LETTER KHAF\0"
  "YEZIDI LETTER LAM\0"
  "YEZIDI LETTER LAM WITH DOT ABOVE\0"
  "YEZIDI LETTER MIM\0"
  "YEZIDI LETTER NUN\0"
  "YEZIDI LETTER OW\0"
  "YEZIDI LETTER PE\0"
  "YEZIDI LETTER PHE\0"
  "YEZIDI LETTER QAF\0"
  "YEZIDI LETTER RA\0"
  "YEZIDI LETTER RHA\0"
  "YEZIDI LETTER SAD\0"
  "YEZIDI LETTER SE\0"
  "YEZIDI LETTER SHIN\0"
  "YEZIDI LETTER SIN\0"
  "YEZIDI LETTER TA\0"
  "YEZIDI LETTER THE\0"
  "YEZIDI LETTER UM\0"
  "YEZIDI LETTER VA\0"
  "YEZIDI LETTER VA ALTERNATE FORM\0"
  "YEZIDI LETTER WAW\0"
  "YEZIDI LETTER XA\0"
  "YEZIDI LETTER XHEYN\0"
  "YEZIDI LETTER YOT\0"
  "YEZIDI LETTER YOT WITH CIRCUMFLEX ABOVE\0"
  "YEZIDI LETTER ZA\0"
  "YEZIDI LETTER ZAL\0"
  "YEZIDI LETTER ZE\0"
  "YI RADICAL BBUT\0"
  "YI RADICAL BUR\0"
  "YI RADICAL CHE\0"
  "YI RADICAL CIP\0"
  "YI RADICAL CUOP\0"
  "YI RADICAL CY\0"
  "YI RADICAL CYP\0"
  "YI RADICAL CYT\0"
  "YI RADICAL DDUR\0"
  "YI RADICAL DU\0"
  "YI RADICAL GA\0"
  "YI RADICAL GEP\0"
  "YI RADICAL GGOP\0"
  "YI RADICAL GGUO\0"
  "YI RADICAL GOT\0"
  "YI RADICAL HMO\0"
  "YI RADICAL HXIT\0"
  "YI RADICAL HXOP\0"
  "YI RADICAL HXUO\0"
  "YI RADICAL JJIE\0"
  "YI RADICAL JJUT\0"
  "YI RADICAL JJY\0"
  "YI RADICAL JO\0"
  "YI RADICAL KE\0"
  "YI RADICAL KIT\0"
  "YI RADICAL LI\0"
  "YI RADICAL LIE\0"
  "YI RADICAL LYR\0"
  "YI RADICAL MI\0"
  "YI RADICAL MOP\0"
  "YI RADICAL NBIE\0"
  "YI RADICAL NYIP\0"
  "YI RADICAL NYOP\0"
  "YI RADICAL NZUP\0"
  "YI RADICAL OP\0"
  "YI RADICAL PUT\0"
  "YI RADICAL PYT\0"
  "YI RADICAL QOT\0"
  "YI RADICAL SHAT\0"
  "YI RADICAL SHOP\0"
  "YI RADICAL SHUR\0"
  "YI RADICAL SHY\0"
  "YI RADICAL SSI\0"
  "YI RADICAL TAT\0"
  "YI RADICAL TU\0"
  "YI RADICAL VEP\0"
  "YI RADICAL VUR\0"
  "YI RADICAL WO\0"
  "YI RADICAL YIT\0"
  "YI RADICAL YO\0"
  "YI RADICAL ZA\0"
  "YI RADICAL ZOT\0"
  "YI RADICAL ZUP\0"
  "YI RADICAL ZUR\0"
  "YI RADICAL ZZIET\0"
  "YI SYLLABLE A\0"
  "YI SYLLABLE AP\0"
  "YI SYLLABLE AT\0"
  "YI SYLLABLE AX\0"
  "YI SYLLABLE BA\0"
  "YI SYLLABLE BAP\0"
  "YI SYLLABLE BAT\0"
  "YI SYLLABLE BAX\0"
  "YI SYLLABLE BBA\0"
  "YI SYLLABLE BBAP\0"
  "YI SYLLABLE BBAT\0"
  "YI SYLLABLE BBAX\0"
  "YI SYLLABLE BBE\0"
  "YI SYLLABLE BBEP\0"
  "YI SYLLABLE BBEX\0"
  "YI SYLLABLE BBI\0"
  "YI SYLLABLE BBIE\0"
  "YI SYLLABLE BBIEP\0"
  "YI SYLLABLE BBIET\0"
  "YI SYLLABLE BBIEX\0"
  "YI SYLLABLE BBIP\0"
  "YI SYLLABLE BBIT\0"
  "YI SYLLABLE BBIX\0"
  "YI SYLLABLE BBO\0"
  "YI SYLLABLE BBOP\0"
  "YI SYLLABLE BBOT\0"
  "YI SYLLABLE BBOX\0"
  "YI SYLLABLE BBU\0"
  "YI SYLLABLE BBUO\0"
  "YI SYLLABLE BBUOP\0"
  "YI SYLLABLE BBUOX\0"
  "YI SYLLABLE BBUP\0"
  "YI SYLLABLE BBUR\0"
  "YI SYLLABLE BBURX\0"
  "YI SYLLABLE BBUT\0"
  "YI SYLLABLE BBUX\0"
  "YI SYLLABLE BBY\0"
  "YI SYLLABLE BBYP\0"
  "YI SYLLABLE BBYT\0"
  "YI SYLLABLE BBYX\0"
  "YI SYLLABLE BE\0"
  "YI SYLLABLE BEP\0"
  "YI SYLLABLE BEX\0"
  "YI SYLLABLE BI\0"
  "YI SYLLABLE BIE\0"
  "YI SYLLABLE BIEP\0"
  "YI SYLLABLE BIET\0"
  "YI SYLLABLE BIEX\0"
  "YI SYLLABLE BIP\0"
  "YI SYLLABLE BIT\0"
  "YI SYLLABLE BIX\0"
  "YI SYLLABLE BO\0"
  "YI SYLLABLE BOP\0"
  "YI SYLLABLE BOT\0"
  "YI SYLLABLE BOX\0"
  "YI SYLLABLE BU\0"
  "YI SYLLABLE BUO\0"
  "YI SYLLABLE BUOP\0"
  "YI SYLLABLE BUOX\0"
  "YI SYLLABLE BUP\0"
  "YI SYLLABLE BUR\0"
  "YI SYLLABLE BURX\0"
  "YI SYLLABLE BUT\0"
  "YI SYLLABLE BUX\0"
  "YI SYLLABLE BY\0"
  "YI SYLLABLE BYP\0"
  "YI SYLLABLE BYR\0"
  "YI SYLLABLE BYRX\0"
  "YI SYLLABLE BYT\0"
  "YI SYLLABLE BYX\0"
  "YI SYLLABLE CA\0"
  "YI SYLLABLE CAP\0"
  "YI SYLLABLE CAT\0"
  "YI SYLLABLE CAX\0"
  "YI SYLLABLE CE\0"
  "YI SYLLABLE CEP\0"
  "YI SYLLABLE CEX\0"
  "YI SYLLABLE CHA\0"
  "YI SYLLABLE CHAP\0"
  "YI SYLLABLE CHAT\0"
  "YI SYLLABLE CHAX\0"
  "YI SYLLABLE CHE\0"
  "YI SYLLABLE CHEP\0"
  "YI SYLLABLE CHET\0"
  "YI SYLLABLE CHEX\0"
  "YI SYLLABLE CHO\0"
  "YI SYLLABLE CHOP\0"
  "YI SYLLABLE CHOT\0"
  "YI SYLLABLE CHOX\0"
  "YI SYLLABLE CHU\0"
  "YI SYLLABLE CHUO\0"
  "YI SYLLABLE CHUOP\0"
  "YI SYLLABLE CHUOT\0"
  "YI SYLLABLE CHUOX\0"
  "YI SYLLABLE CHUP\0"
  "YI SYLLABLE CHUR\0"
  "YI SYLLABLE CHURX\0"
  "YI SYLLABLE CHUX\0"
  "YI SYLLABLE CHY\0"
  "YI SYLLABLE CHYP\0"
  "YI SYLLABLE CHYR\0"
  "YI SYLLABLE CHYRX\0"
  "YI SYLLABLE CHYT\0"
  "YI SYLLABLE CHYX\0"
  "YI SYLLABLE CI\0"
  "YI SYLLABLE CIE\0"
  "YI SYLLABLE CIEP\0"
  "YI SYLLABLE CIET\0"
  "YI SYLLABLE CIEX\0"
  "YI SYLLABLE CIP\0"
  "YI SYLLABLE CIT\0"
  "YI SYLLABLE CIX\0"
  "YI SYLLABLE CO\0"
  "YI SYLLABLE COP\0"
  "YI SYLLABLE COT\0"
  "YI SYLLABLE COX\0"
  "YI SYLLABLE CU\0"
  "YI SYLLABLE CUO\0"
  "YI SYLLABLE CUOP\0"
  "YI SYLLABLE CUOX\0"
  "YI SYLLABLE CUP\0"
  "YI SYLLABLE CUR\0"
  "YI SYLLABLE CURX\0"
  "YI SYLLABLE CUT\0"
  "YI SYLLABLE CUX\0"
  "YI SYLLABLE CY\0"
  "YI SYLLABLE CYP\0"
  "YI SYLLABLE CYR\0"
  "YI SYLLABLE CYRX\0"
  "YI SYLLABLE CYT\0"
  "YI SYLLABLE CYX\0"
  "YI SYLLABLE DA\0"
  "YI SYLLABLE DAP\0"
  "YI SYLLABLE DAT\0"
  "YI SYLLABLE DAX\0"
  "YI SYLLABLE DDA\0"
  "YI SYLLABLE DDAP\0"
  "YI SYLLABLE DDAT\0"
  "YI SYLLABLE DDAX\0"
  "YI SYLLABLE DDE\0"
  "YI SYLLABLE DDEP\0"
  "YI SYLLABLE DDEX\0"
  "YI SYLLABLE DDI\0"
  "YI SYLLABLE DDIE\0"
  "YI SYLLABLE DDIEP\0"
  "YI SYLLABLE DDIEX\0"
  "YI SYLLABLE DDIP\0"
  "YI SYLLABLE DDIT\0"
  "YI SYLLABLE DDIX\0"
  "YI SYLLABLE DDO\0"
  "YI SYLLABLE DDOP\0"
  "YI SYLLABLE DDOT\0"
  "YI SYLLABLE DDOX\0"
  "YI SYLLABLE DDU\0"
  "YI SYLLABLE DDUO\0"
  "YI SYLLABLE DDUOP\0"
  "YI SYLLABLE DDUOX\0"
  "YI SYLLABLE DDUP\0"
  "YI SYLLABLE DDUR\0"
  "YI SYLLABLE DDURX\0"
  "YI SYLLABLE DDUT\0"
  "YI SYLLABLE DDUX\0"
  "YI SYLLABLE DE\0"
  "YI SYLLABLE DEP\0"
  "YI SYLLABLE DEX\0"
  "YI SYLLABLE DI\0"
  "YI SYLLABLE DIE\0"
  "YI SYLLABLE DIEP\0"
  "YI SYLLABLE DIEX\0"
  "YI SYLLABLE DIP\0"
  "YI SYLLABLE DIT\0"
  "YI SYLLABLE DIX\0"
  "YI SYLLABLE DO\0"
  "YI SYLLABLE DOP\0"
  "YI SYLLABLE DOT\0"
  "YI SYLLABLE DOX\0"
  "YI SYLLABLE DU\0"
  "YI SYLLABLE DUO\0"
  "YI SYLLABLE DUOX\0"
  "YI SYLLABLE DUP\0"
  "YI SYLLABLE DUR\0"
  "YI SYLLABLE DURX\0"
  "YI SYLLABLE DUT\0"
  "YI SYLLABLE DUX\0"
  "YI SYLLABLE E\0"
  "YI SYLLABLE EX\0"
  "YI SYLLABLE FA\0"
  "YI SYLLABLE FAP\0"
  "YI SYLLABLE FAT\0"
  "YI SYLLABLE FAX\0"
  "YI SYLLABLE FI\0"
  "YI SYLLABLE FIP\0"
  "YI SYLLABLE FIT\0"
  "YI SYLLABLE FIX\0"
  "YI SYLLABLE FO\0"
  "YI SYLLABLE FOP\0"
  "YI SYLLABLE FOX\0"
  "YI SYLLABLE FU\0"
  "YI SYLLABLE FUP\0"
  "YI SYLLABLE FUR\0"
  "YI SYLLABLE FURX\0"
  "YI SYLLABLE FUT\0"
  "YI SYLLABLE FUX\0"
  "YI SYLLABLE FY\0"
  "YI SYLLABLE FYP\0"
  "YI SYLLABLE FYT\0"
  "YI SYLLABLE FYX\0"
  "YI SYLLABLE GA\0"
  "YI SYLLABLE GAP\0"
  "YI SYLLABLE GAT\0"
  "YI SYLLABLE GAX\0"
  "YI SYLLABLE GE\0"
  "YI SYLLABLE GEP\0"
  "YI SYLLABLE GET\0"
  "YI SYLLABLE GEX\0"
  "YI SYLLABLE GGA\0"
  "YI SYLLABLE GGAP\0"
  "YI SYLLABLE GGAT\0"
  "YI SYLLABLE GGAX\0"
  "YI SYLLABLE GGE\0"
  "YI SYLLABLE GGEP\0"
  "YI SYLLABLE GGET\0"
  "YI SYLLABLE GGEX\0"
  "YI SYLLABLE GGI\0"
  "YI SYLLABLE GGIE\0"
  "YI SYLLABLE GGIEP\0"
  "YI SYLLABLE GGIEX\0"
  "YI SYLLABLE GGIT\0"
  "YI SYLLABLE GGIX\0"
  "YI SYLLABLE GGO\0"
  "YI SYLLABLE GGOP\0"
  "YI SYLLABLE GGOT\0"
  "YI SYLLABLE GGOX\0"
  "YI SYLLABLE GGU\0"
  "YI SYLLABLE GGUO\0"
  "YI SYLLABLE GGUOP\0"
  "YI SYLLABLE GGUOT\0"
  "YI SYLLABLE GGUOX\0"
  "YI SYLLABLE GGUP\0"
  "YI SYLLABLE GGUR\0"
  "YI SYLLABLE GGURX\0"
  "YI SYLLABLE GGUT\0"
  "YI SYLLABLE GGUX\0"
  "YI SYLLABLE GI\0"
  "YI SYLLABLE GIE\0"
  "YI SYLLABLE GIEP\0"
  "YI SYLLABLE GIET\0"
  "YI SYLLABLE GIEX\0"
  "YI SYLLABLE GIP\0"
  "YI SYLLABLE GIT\0"
  "YI SYLLABLE GIX\0"
  "YI SYLLABLE GO\0"
  "YI SYLLABLE GOP\0"
  "YI SYLLABLE GOT\0"
  "YI SYLLABLE GOX\0"
  "YI SYLLABLE GU\0"
  "YI SYLLABLE GUO\0"
  "YI SYLLABLE GUOP\0"
  "YI SYLLABLE GUOT\0"
  "YI SYLLABLE GUOX\0"
  "YI SYLLABLE GUP\0"
  "YI SYLLABLE GUR\0"
  "YI SYLLABLE GURX\0"
  "YI SYLLABLE GUT\0"
  "YI SYLLABLE GUX\0"
  "YI SYLLABLE HA\0"
  "YI SYLLABLE HAP\0"
  "YI SYLLABLE HAT\0"
  "YI SYLLABLE HAX\0"
  "YI SYLLABLE HE\0"
  "YI SYLLABLE HEP\0"
  "YI SYLLABLE HEX\0"
  "YI SYLLABLE HIE\0"
  "YI SYLLABLE HIEX\0"
  "YI SYLLABLE HIT\0"
  "YI SYLLABLE HLA\0"
  "YI SYLLABLE HLAP\0"
  "YI SYLLABLE HLAT\0"
  "YI SYLLABLE HLAX\0"
  "YI SYLLABLE HLE\0"
  "YI SYLLABLE HLEP\0"
  "YI SYLLABLE HLEX\0"
  "YI SYLLABLE HLI\0"
  "YI SYLLABLE HLIE\0"
  "YI SYLLABLE HLIEP\0"
  "YI SYLLABLE HLIEX\0"
  "YI SYLLABLE HLIP\0"
  "YI SYLLABLE HLIT\0"
  "YI SYLLABLE HLIX\0"
  "YI SYLLABLE HLO\0"
  "YI SYLLABLE HLOP\0"
  "YI SYLLABLE HLOX\0"
  "YI SYLLABLE HLU\0"
  "YI SYLLABLE HLUO\0"
  "YI SYLLABLE HLUOP\0"
  "YI SYLLABLE HLUOX\0"
  "YI SYLLABLE HLUP\0"
  "YI SYLLABLE HLUR\0"
  "YI SYLLABLE HLURX\0"
  "YI SYLLABLE HLUT\0"
  "YI SYLLABLE HLUX\0"
  "YI SYLLABLE HLY\0"
  "YI SYLLABLE HLYP\0"
  "YI SYLLABLE HLYR\0"
  "YI SYLLABLE HLYRX\0"
  "YI SYLLABLE HLYT\0"
  "YI SYLLABLE HLYX\0"
  "YI SYLLABLE HMA\0"
  "YI SYLLABLE HMAP\0"
  "YI SYLLABLE HMAT\0"
  "YI SYLLABLE HMAX\0"
  "YI SYLLABLE HMI\0"
  "YI SYLLABLE HMIE\0"
  "YI SYLLABLE HMIEP\0"
  "YI SYLLABLE HMIEX\0"
  "YI SYLLABLE HMIP\0"
  "YI SYLLABLE HMIT\0"
  "YI SYLLABLE HMIX\0"
  "YI SYLLABLE HMO\0"
  "YI SYLLABLE HMOP\0"
  "YI SYLLABLE HMOT\0"
  "YI SYLLABLE HMOX\0"
  "YI SYLLABLE HMU\0"
  "YI SYLLABLE HMUO\0"
  "YI SYLLABLE HMUOP\0"
  "YI SYLLABLE HMUOX\0"
  "YI SYLLABLE HMUP\0"
  "YI SYLLABLE HMUR\0"
  "YI SYLLABLE HMURX\0"
  "YI SYLLABLE HMUT\0"
  "YI SYLLABLE HMUX\0"
  "YI SYLLABLE HMY\0"
  "YI SYLLABLE HMYP\0"
  "YI SYLLABLE HMYR\0"
  "YI SYLLABLE HMYRX\0"
  "YI SYLLABLE HMYX\0"
  "YI SYLLABLE HNA\0"
  "YI SYLLABLE HNAP\0"
  "YI SYLLABLE HNAT\0"
  "YI SYLLABLE HNAX\0"
  "YI SYLLABLE HNE\0"
  "YI SYLLABLE HNEP\0"
  "YI SYLLABLE HNEX\0"
  "YI SYLLABLE HNI\0"
  "YI SYLLABLE HNIE\0"
  "YI SYLLABLE HNIEP\0"
  "YI SYLLABLE HNIET\0"
  "YI SYLLABLE HNIEX\0"
  "YI SYLLABLE HNIP\0"
  "YI SYLLABLE HNIT\0"
  "YI SYLLABLE HNIX\0"
  "YI SYLLABLE HNOP\0"
  "YI SYLLABLE HNOT\0"
  "YI SYLLABLE HNOX\0"
  "YI SYLLABLE HNUO\0"
  "YI SYLLABLE HNUOX\0"
  "YI SYLLABLE HNUT\0"
  "YI SYLLABLE HO\0"
  "YI SYLLABLE HOP\0"
  "YI SYLLABLE HOT\0"
  "YI SYLLABLE HOX\0"
  "YI SYLLABLE HUO\0"
  "YI SYLLABLE HUOP\0"
  "YI SYLLABLE HUOT\0"
  "YI SYLLABLE HUOX\0"
  "YI SYLLABLE HXA\0"
  "YI SYLLABLE HXAP\0"
  "YI SYLLABLE HXAT\0"
  "YI SYLLABLE HXAX\0"
  "YI SYLLABLE HXE\0"
  "YI SYLLABLE HXEP\0"
  "YI SYLLABLE HXEX\0"
  "YI SYLLABLE HXI\0"
  "YI SYLLABLE HXIE\0"
  "YI SYLLABLE HXIEP\0"
  "YI SYLLABLE HXIET\0"
  "YI SYLLABLE HXIEX\0"
  "YI SYLLABLE HXIP\0"
  "YI SYLLABLE HXIT\0"
  "YI SYLLABLE HXIX\0"
  "YI SYLLABLE HXO\0"
  "YI SYLLABLE HXOP\0"
  "YI SYLLABLE HXOT\0"
  "YI SYLLABLE HXOX\0"
  "YI SYLLABLE HXUO\0"
  "YI SYLLABLE HXUOP\0"
  "YI SYLLABLE HXUOT\0"
  "YI SYLLABLE HXUOX\0"
  "YI SYLLABLE I\0"
  "YI SYLLABLE IE\0"
  "YI SYLLABLE IEP\0"
  "YI SYLLABLE IET\0"
  "YI SYLLABLE IEX\0"
  "YI SYLLABLE IP\0"
  "YI SYLLABLE IT\0"
  "YI SYLLABLE IX\0"
  "YI SYLLABLE JI\0"
  "YI SYLLABLE JIE\0"
  "YI SYLLABLE JIEP\0"
  "YI SYLLABLE JIET\0"
  "YI SYLLABLE JIEX\0"
  "YI SYLLABLE JIP\0"
  "YI SYLLABLE JIT\0"
  "YI SYLLABLE JIX\0"
  "YI SYLLABLE JJI\0"
  "YI SYLLABLE JJIE\0"
  "YI SYLLABLE JJIEP\0"
  "YI SYLLABLE JJIET\0"
  "YI SYLLABLE JJIEX\0"
  "YI SYLLABLE JJIP\0"
  "YI SYLLABLE JJIT\0"
  "YI SYLLABLE JJIX\0"
  "YI SYLLABLE JJO\0"
  "YI SYLLABLE JJOP\0"
  "YI SYLLABLE JJOT\0"
  "YI SYLLABLE JJOX\0"
  "YI SYLLABLE JJU\0"
  "YI SYLLABLE JJUO\0"
  "YI SYLLABLE JJUOP\0"
  "YI SYLLABLE JJUOX\0"
  "YI SYLLABLE JJUP\0"
  "YI SYLLABLE JJUR\0"
  "YI SYLLABLE JJURX\0"
  "YI SYLLABLE JJUT\0"
  "YI SYLLABLE JJUX\0"
  "YI SYLLABLE JJY\0"
  "YI SYLLABLE JJYP\0"
  "YI SYLLABLE JJYT\0"
  "YI SYLLABLE JJYX\0"
  "YI SYLLABLE JO\0"
  "YI SYLLABLE JOP\0"
  "YI SYLLABLE JOT\0"
  "YI SYLLABLE JOX\0"
  "YI SYLLABLE JU\0"
  "YI SYLLABLE JUO\0"
  "YI SYLLABLE JUOP\0"
  "YI SYLLABLE JUOT\0"
  "YI SYLLABLE JUOX\0"
  "YI SYLLABLE JUP\0"
  "YI SYLLABLE JUR\0"
  "YI SYLLABLE JURX\0"
  "YI SYLLABLE JUT\0"
  "YI SYLLABLE JUX\0"
  "YI SYLLABLE JY\0"
  "YI SYLLABLE JYP\0"
  "YI SYLLABLE JYR\0"
  "YI SYLLABLE JYRX\0"
  "YI SYLLABLE JYT\0"
  "YI SYLLABLE JYX\0"
  "YI SYLLABLE KA\0"
  "YI SYLLABLE KAP\0"
  "YI SYLLABLE KAT\0"
  "YI SYLLABLE KAX\0"
  "YI SYLLABLE KE\0"
  "YI SYLLABLE KEP\0"
  "YI SYLLABLE KET\0"
  "YI SYLLABLE KEX\0"
  "YI SYLLABLE KI\0"
  "YI SYLLABLE KIE\0"
  "YI SYLLABLE KIEP\0"
  "YI SYLLABLE KIEX\0"
  "YI SYLLABLE KIP\0"
  "YI SYLLABLE KIT\0"
  "YI SYLLABLE KIX\0"
  "YI SYLLABLE KO\0"
  "YI SYLLABLE KOP\0"
  "YI SYLLABLE KOT\0"
  "YI SYLLABLE KOX\0"
  "YI SYLLABLE KU\0"
  "YI SYLLABLE KUO\0"
  "YI SYLLABLE KUOP\0"
  "YI SYLLABLE KUOX\0"
  "YI SYLLABLE KUP\0"
  "YI SYLLABLE KUR\0"
  "YI SYLLABLE KURX\0"
  "YI SYLLABLE KUT\0"
  "YI SYLLABLE KUX\0"
  "YI SYLLABLE LA\0"
  "YI SYLLABLE LAP\0"
  "YI SYLLABLE LAT\0"
  "YI SYLLABLE LAX\0"
  "YI SYLLABLE LE\0"
  "YI SYLLABLE LEP\0"
  "YI SYLLABLE LEX\0"
  "YI SYLLABLE LI\0"
  "YI SYLLABLE LIE\0"
  "YI SYLLABLE LIEP\0"
  "YI SYLLABLE LIET\0"
  "YI SYLLABLE LIEX\0"
  "YI SYLLABLE LIP\0"
  "YI SYLLABLE LIT\0"
  "YI SYLLABLE LIX\0"
  "YI SYLLABLE LO\0"
  "YI SYLLABLE LOP\0"
  "YI SYLLABLE LOT\0"
  "YI SYLLABLE LOX\0"
  "YI SYLLABLE LU\0"
  "YI SYLLABLE LUO\0"
  "YI SYLLABLE LUOP\0"
  "YI SYLLABLE LUOT\0"
  "YI SYLLABLE LUOX\0"
  "YI SYLLABLE LUP\0"
  "YI SYLLABLE LUR\0"
  "YI SYLLABLE LURX\0"
  "YI SYLLABLE LUT\0"
  "YI SYLLABLE LUX\0"
  "YI SYLLABLE LY\0"
  "YI SYLLABLE LYP\0"
  "YI SYLLABLE LYR\0"
  "YI SYLLABLE LYRX\0"
  "YI SYLLABLE LYT\0"
  "YI SYLLABLE LYX\0"
  "YI SYLLABLE MA\0"
  "YI SYLLABLE MAP\0"
  "YI SYLLABLE MAT\0"
  "YI SYLLABLE MAX\0"
  "YI SYLLABLE ME\0"
  "YI SYLLABLE MEX\0"
  "YI SYLLABLE MGA\0"
  "YI SYLLABLE MGAP\0"
  "YI SYLLABLE MGAT\0"
  "YI SYLLABLE MGAX\0"
  "YI SYLLABLE MGE\0"
  "YI SYLLABLE MGEP\0"
  "YI SYLLABLE MGEX\0"
  "YI SYLLABLE MGIE\0"
  "YI SYLLABLE MGIEX\0"
  "YI SYLLABLE MGO\0"
  "YI SYLLABLE MGOP\0"
  "YI SYLLABLE MGOT\0"
  "YI SYLLABLE MGOX\0"
  "YI SYLLABLE MGU\0"
  "YI SYLLABLE MGUO\0"
  "YI SYLLABLE MGUOP\0"
  "YI SYLLABLE MGUOX\0"
  "YI SYLLABLE MGUP\0"
  "YI SYLLABLE MGUR\0"
  "YI SYLLABLE MGURX\0"
  "YI SYLLABLE MGUT\0"
  "YI SYLLABLE MGUX\0"
  "YI SYLLABLE MI\0"
  "YI SYLLABLE MIE\0"
  "YI SYLLABLE MIEP\0"
  "YI SYLLABLE MIEX\0"
  "YI SYLLABLE MIP\0"
  "YI SYLLABLE MIT\0"
  "YI SYLLABLE MIX\0"
  "YI SYLLABLE MO\0"
  "YI SYLLABLE MOP\0"
  "YI SYLLABLE MOT\0"
  "YI SYLLABLE MOX\0"
  "YI SYLLABLE MU\0"
  "YI SYLLABLE MUO\0"
  "YI SYLLABLE MUOP\0"
  "YI SYLLABLE MUOT\0"
  "YI SYLLABLE MUOX\0"
  "YI SYLLABLE MUP\0"
  "YI SYLLABLE MUR\0"
  "YI SYLLABLE MURX\0"
  "YI SYLLABLE MUT\0"
  "YI SYLLABLE MUX\0"
  "YI SYLLABLE MY\0"
  "YI SYLLABLE MYP\0"
  "YI SYLLABLE MYT\0"
  "YI SYLLABLE MYX\0"
  "YI SYLLABLE NA\0"
  "YI SYLLABLE NAP\0"
  "YI SYLLABLE NAX\0"
  "YI SYLLABLE NBA\0"
  "YI SYLLABLE NBAP\0"
  "YI SYLLABLE NBAT\0"
  "YI SYLLABLE NBAX\0"
  "YI SYLLABLE NBI\0"
  "YI SYLLABLE NBIE\0"
  "YI SYLLABLE NBIEP\0"
  "YI SYLLABLE NBIEX\0"
  "YI SYLLABLE NBIP\0"
  "YI SYLLABLE NBIT\0"
  "YI SYLLABLE NBIX\0"
  "YI SYLLABLE NBO\0"
  "YI SYLLABLE NBOP\0"
  "YI SYLLABLE NBOT\0"
  "YI SYLLABLE NBOX\0"
  "YI SYLLABLE NBU\0"
  "YI SYLLABLE NBUP\0"
  "YI SYLLABLE NBUR\0"
  "YI SYLLABLE NBURX\0"
  "YI SYLLABLE NBUT\0"
  "YI SYLLABLE NBUX\0"
  "YI SYLLABLE NBY\0"
  "YI SYLLABLE NBYP\0"
  "YI SYLLABLE NBYR\0"
  "YI SYLLABLE NBYRX\0"
  "YI SYLLABLE NBYT\0"
  "YI SYLLABLE NBYX\0"
  "YI SYLLABLE NDA\0"
  "YI SYLLABLE NDAP\0"
  "YI SYLLABLE NDAT\0"
  "YI SYLLABLE NDAX\0"
  "YI SYLLABLE NDE\0"
  "YI SYLLABLE NDEP\0"
  "YI SYLLABLE NDEX\0"
  "YI SYLLABLE NDI\0"
  "YI SYLLABLE NDIE\0"
  "YI SYLLABLE NDIEX\0"
  "YI SYLLABLE NDIP\0"
  "YI SYLLABLE NDIT\0"
  "YI SYLLABLE NDIX\0"
  "YI SYLLABLE NDO\0"
  "YI SYLLABLE NDOP\0"
  "YI SYLLABLE NDOT\0"
  "YI SYLLABLE NDOX\0"
  "YI SYLLABLE NDU\0"
  "YI SYLLABLE NDUP\0"
  "YI SYLLABLE NDUR\0"
  "YI SYLLABLE NDURX\0"
  "YI SYLLABLE NDUT\0"
  "YI SYLLABLE NDUX\0"
  "YI SYLLABLE NE\0"
  "YI SYLLABLE NEP\0"
  "YI SYLLABLE NEX\0"
  "YI SYLLABLE NGA\0"
  "YI SYLLABLE NGAP\0"
  "YI SYLLABLE NGAT\0"
  "YI SYLLABLE NGAX\0"
  "YI SYLLABLE NGE\0"
  "YI SYLLABLE NGEP\0"
  "YI SYLLABLE NGEX\0"
  "YI SYLLABLE NGIE\0"
  "YI SYLLABLE NGIEP\0"
  "YI SYLLABLE NGIEX\0"
  "YI SYLLABLE NGO\0"
  "YI SYLLABLE NGOP\0"
  "YI SYLLABLE NGOT\0"
  "YI SYLLABLE NGOX\0"
  "YI SYLLABLE NGUO\0"
  "YI SYLLABLE NGUOT\0"
  "YI SYLLABLE NGUOX\0"
  "YI SYLLABLE NI\0"
  "YI SYLLABLE NIE\0"
  "YI SYLLABLE NIEP\0"
  "YI SYLLABLE NIEX\0"
  "YI SYLLABLE NIP\0"
  "YI SYLLABLE NIT\0"
  "YI SYLLABLE NIX\0"
  "YI SYLLABLE NJI\0"
  "YI SYLLABLE NJIE\0"
  "YI SYLLABLE NJIEP\0"
  "YI SYLLABLE NJIET\0"
  "YI SYLLABLE NJIEX\0"
  "YI SYLLABLE NJIP\0"
  "YI SYLLABLE NJIT\0"
  "YI SYLLABLE NJIX\0"
  "YI SYLLABLE NJO\0"
  "YI SYLLABLE NJOP\0"
  "YI SYLLABLE NJOT\0"
  "YI SYLLABLE NJOX\0"
  "YI SYLLABLE NJU\0"
  "YI SYLLABLE NJUO\0"
  "YI SYLLABLE NJUOX\0"
  "YI SYLLABLE NJUP\0"
  "YI SYLLABLE NJUR\0"
  "YI SYLLABLE NJURX\0"
  "YI SYLLABLE NJUX\0"
  "YI SYLLABLE NJY\0"
  "YI SYLLABLE NJYP\0"
  "YI SYLLABLE NJYR\0"
  "YI SYLLABLE NJYRX\0"
  "YI SYLLABLE NJYT\0"
  "YI SYLLABLE NJYX\0"
  "YI SYLLABLE NO\0"
  "YI SYLLABLE NOP\0"
  "YI SYLLABLE NOT\0"
  "YI SYLLABLE NOX\0"
  "YI SYLLABLE NRA\0"
  "YI SYLLABLE NRAP\0"
  "YI SYLLABLE NRAT\0"
  "YI SYLLABLE NRAX\0"
  "YI SYLLABLE NRE\0"
  "YI SYLLABLE NREP\0"
  "YI SYLLABLE NRET\0"
  "YI SYLLABLE NREX\0"
  "YI SYLLABLE NRO\0"
  "YI SYLLABLE NROP\0"
  "YI SYLLABLE NROX\0"
  "YI SYLLABLE NRU\0"
  "YI SYLLABLE NRUP\0"
  "YI SYLLABLE NRUR\0"
  "YI SYLLABLE NRURX\0"
  "YI SYLLABLE NRUT\0"
  "YI SYLLABLE NRUX\0"
  "YI SYLLABLE NRY\0"
  "YI SYLLABLE NRYP\0"
  "YI SYLLABLE NRYR\0"
  "YI SYLLABLE NRYRX\0"
  "YI SYLLABLE NRYT\0"
  "YI SYLLABLE NRYX\0"
  "YI SYLLABLE NU\0"
  "YI SYLLABLE NUO\0"
  "YI SYLLABLE NUOP\0"
  "YI SYLLABLE NUOX\0"
  "YI SYLLABLE NUP\0"
  "YI SYLLABLE NUR\0"
  "YI SYLLABLE NURX\0"
  "YI SYLLABLE NUT\0"
  "YI SYLLABLE NUX\0"
  "YI SYLLABLE NYI\0"
  "YI SYLLABLE NYIE\0"
  "YI SYLLABLE NYIEP\0"
  "YI SYLLABLE NYIET\0"
  "YI SYLLABLE NYIEX\0"
  "YI SYLLABLE NYIP\0"
  "YI SYLLABLE NYIT\0"
  "YI SYLLABLE NYIX\0"
  "YI SYLLABLE NYO\0"
  "YI SYLLABLE NYOP\0"
  "YI SYLLABLE NYOT\0"
  "YI SYLLABLE NYOX\0"
  "YI SYLLABLE NYU\0"
  "YI SYLLABLE NYUO\0"
  "YI SYLLABLE NYUOP\0"
  "YI SYLLABLE NYUOX\0"
  "YI SYLLABLE NYUP\0"
  "YI SYLLABLE NYUT\0"
  "YI SYLLABLE NYUX\0"
  "YI SYLLABLE NZA\0"
  "YI SYLLABLE NZAP\0"
  "YI SYLLABLE NZAT\0"
  "YI SYLLABLE NZAX\0"
  "YI SYLLABLE NZE\0"
  "YI SYLLABLE NZEX\0"
  "YI SYLLABLE NZI\0"
  "YI SYLLABLE NZIE\0"
  "YI SYLLABLE NZIEP\0"
  "YI SYLLABLE NZIEX\0"
  "YI SYLLABLE NZIP\0"
  "YI SYLLABLE NZIT\0"
  "YI SYLLABLE NZIX\0"
  "YI SYLLABLE NZOP\0"
  "YI SYLLABLE NZOX\0"
  "YI SYLLABLE NZU\0"
  "YI SYLLABLE NZUO\0"
  "YI SYLLABLE NZUOX\0"
  "YI SYLLABLE NZUP\0"
  "YI SYLLABLE NZUR\0"
  "YI SYLLABLE NZURX\0"
  "YI SYLLABLE NZUX\0"
  "YI SYLLABLE NZY\0"
  "YI SYLLABLE NZYP\0"
  "YI SYLLABLE NZYR\0"
  "YI SYLLABLE NZYRX\0"
  "YI SYLLABLE NZYT\0"
  "YI SYLLABLE NZYX\0"
  "YI SYLLABLE O\0"
  "YI SYLLABLE OP\0"
  "YI SYLLABLE OT\0"
  "YI SYLLABLE OX\0"
  "YI SYLLABLE PA\0"
  "YI SYLLABLE PAP\0"
  "YI SYLLABLE PAT\0"
  "YI SYLLABLE PAX\0"
  "YI SYLLABLE PI\0"
  "YI SYLLABLE PIE\0"
  "YI SYLLABLE PIEP\0"
  "YI SYLLABLE PIEX\0"
  "YI SYLLABLE PIP\0"
  "YI SYLLABLE PIT\0"
  "YI SYLLABLE PIX\0"
  "YI SYLLABLE PO\0"
  "YI SYLLABLE POP\0"
  "YI SYLLABLE POT\0"
  "YI SYLLABLE POX\0"
  "YI SYLLABLE PU\0"
  "YI SYLLABLE PUO\0"
  "YI SYLLABLE PUOP\0"
  "YI SYLLABLE PUOX\0"
  "YI SYLLABLE PUP\0"
  "YI SYLLABLE PUR\0"
  "YI SYLLABLE PURX\0"
  "YI SYLLABLE PUT\0"
  "YI SYLLABLE PUX\0"
  "YI SYLLABLE PY\0"
  "YI SYLLABLE PYP\0"
  "YI SYLLABLE PYR\0"
  "YI SYLLABLE PYRX\0"
  "YI SYLLABLE PYT\0"
  "YI SYLLABLE PYX\0"
  "YI SYLLABLE QI\0"
  "YI SYLLABLE QIE\0"
  "YI SYLLABLE QIEP\0"
  "YI SYLLABLE QIET\0"
  "YI SYLLABLE QIEX\0"
  "YI SYLLABLE QIP\0"
  "YI SYLLABLE QIT\0"
  "YI SYLLABLE QIX\0"
  "YI SYLLABLE QO\0"
  "YI SYLLABLE QOP\0"
  "YI SYLLABLE QOT\0"
  "YI SYLLABLE QOX\0"
  "YI SYLLABLE QU\0"
  "YI SYLLABLE QUO\0"
  "YI SYLLABLE QUOP\0"
  "YI SYLLABLE QUOT\0"
  "YI SYLLABLE QUOX\0"
  "YI SYLLABLE QUP\0"
  "YI SYLLABLE QUR\0"
  "YI SYLLABLE QURX\0"
  "YI SYLLABLE QUT\0"
  "YI SYLLABLE QUX\0"
  "YI SYLLABLE QY\0"
  "YI SYLLABLE QYP\0"
  "YI SYLLABLE QYR\0"
  "YI SYLLABLE QYRX\0"
  "YI SYLLABLE QYT\0"
  "YI SYLLABLE QYX\0"
  "YI SYLLABLE RA\0"
  "YI SYLLABLE RAP\0"
  "YI SYLLABLE RAT\0"
  "YI SYLLABLE RAX\0"
  "YI SYLLABLE RE\0"
  "YI SYLLABLE REP\0"
  "YI SYLLABLE REX\0"
  "YI SYLLABLE RO\0"
  "YI SYLLABLE ROP\0"
  "YI SYLLABLE ROT\0"
  "YI SYLLABLE ROX\0"
  "YI SYLLABLE RRA\0"
  "YI SYLLABLE RRAX\0"
  "YI SYLLABLE RRE\0"
  "YI SYLLABLE RREP\0"
  "YI SYLLABLE RRET\0"
  "YI SYLLABLE RREX\0"
  "YI SYLLABLE RRO\0"
  "YI SYLLABLE RROP\0"
  "YI SYLLABLE RROT\0"
  "YI SYLLABLE RROX\0"
  "YI SYLLABLE RRU\0"
  "YI SYLLABLE RRUO\0"
  "YI SYLLABLE RRUOX\0"
  "YI SYLLABLE RRUP\0"
  "YI SYLLABLE RRUR\0"
  "YI SYLLABLE RRURX\0"
  "YI SYLLABLE RRUT\0"
  "YI SYLLABLE RRUX\0"
  "YI SYLLABLE RRY\0"
  "YI SYLLABLE RRYP\0"
  "YI SYLLABLE RRYR\0"
  "YI SYLLABLE RRYRX\0"
  "YI SYLLABLE RRYT\0"
  "YI SYLLABLE RRYX\0"
  "YI SYLLABLE RU\0"
  "YI SYLLABLE RUO\0"
  "YI SYLLABLE RUOP\0"
  "YI SYLLABLE RUOX\0"
  "YI SYLLABLE RUP\0"
  "YI SYLLABLE RUR\0"
  "YI SYLLABLE RURX\0"
  "YI SYLLABLE RUT\0"
  "YI SYLLABLE RUX\0"
  "YI SYLLABLE RY\0"
  "YI SYLLABLE RYP\0"
  "YI SYLLABLE RYR\0"
  "YI SYLLABLE RYRX\0"
  "YI SYLLABLE RYT\0"
  "YI SYLLABLE RYX\0"
  "YI SYLLABLE SA\0"
  "YI SYLLABLE SAP\0"
  "YI SYLLABLE SAT\0"
  "YI SYLLABLE SAX\0"
  "YI SYLLABLE SE\0"
  "YI SYLLABLE SEP\0"
  "YI SYLLABLE SEX\0"
  "YI SYLLABLE SHA\0"
  "YI SYLLABLE SHAP\0"
  "YI SYLLABLE SHAT\0"
  "YI SYLLABLE SHAX\0"
  "YI SYLLABLE SHE\0"
  "YI SYLLABLE SHEP\0"
  "YI SYLLABLE SHET\0"
  "YI SYLLABLE SHEX\0"
  "YI SYLLABLE SHO\0"
  "YI SYLLABLE SHOP\0"
  "YI SYLLABLE SHOT\0"
  "YI SYLLABLE SHOX\0"
  "YI SYLLABLE SHU\0"
  "YI SYLLABLE SHUO\0"
  "YI SYLLABLE SHUOP\0"
  "YI SYLLABLE SHUOX\0"
  "YI SYLLABLE SHUP\0"
  "YI SYLLABLE SHUR\0"
  "YI SYLLABLE SHURX\0"
  "YI SYLLABLE SHUT\0"
  "YI SYLLABLE SHUX\0"
  "YI SYLLABLE SHY\0"
  "YI SYLLABLE SHYP\0"
  "YI SYLLABLE SHYR\0"
  "YI SYLLABLE SHYRX\0"
  "YI SYLLABLE SHYT\0"
  "YI SYLLABLE SHYX\0"
  "YI SYLLABLE SI\0"
  "YI SYLLABLE SIE\0"
  "YI SYLLABLE SIEP\0"
  "YI SYLLABLE SIEX\0"
  "YI SYLLABLE SIP\0"
  "YI SYLLABLE SIT\0"
  "YI SYLLABLE SIX\0"
  "YI SYLLABLE SO\0"
  "YI SYLLABLE SOP\0"
  "YI SYLLABLE SOT\0"
  "YI SYLLABLE SOX\0"
  "YI SYLLABLE SSA\0"
  "YI SYLLABLE SSAP\0"
  "YI SYLLABLE SSAT\0"
  "YI SYLLABLE SSAX\0"
  "YI SYLLABLE SSE\0"
  "YI SYLLABLE SSEP\0"
  "YI SYLLABLE SSEX\0"
  "YI SYLLABLE SSI\0"
  "YI SYLLABLE SSIE\0"
  "YI SYLLABLE SSIEP\0"
  "YI SYLLABLE SSIEX\0"
  "YI SYLLABLE SSIP\0"
  "YI SYLLABLE SSIT\0"
  "YI SYLLABLE SSIX\0"
  "YI SYLLABLE SSO\0"
  "YI SYLLABLE SSOP\0"
  "YI SYLLABLE SSOT\0"
  "YI SYLLABLE SSOX\0"
  "YI SYLLABLE SSU\0"
  "YI SYLLABLE SSUP\0"
  "YI SYLLABLE SSUT\0"
  "YI SYLLABLE SSUX\0"
  "YI SYLLABLE SSY\0"
  "YI SYLLABLE SSYP\0"
  "YI SYLLABLE SSYR\0"
  "YI SYLLABLE SSYRX\0"
  "YI SYLLABLE SSYT\0"
  "YI SYLLABLE SSYX\0"
  "YI SYLLABLE SU\0"
  "YI SYLLABLE SUO\0"
  "YI SYLLABLE SUOP\0"
  "YI SYLLABLE SUOX\0"
  "YI SYLLABLE SUP\0"
  "YI SYLLABLE SUR\0"
  "YI SYLLABLE SURX\0"
  "YI SYLLABLE SUT\0"
  "YI SYLLABLE SUX\0"
  "YI SYLLABLE SY\0"
  "YI SYLLABLE SYP\0"
  "YI SYLLABLE SYR\0"
  "YI SYLLABLE SYRX\0"
  "YI SYLLABLE SYT\0"
  "YI SYLLABLE SYX\0"
  "YI SYLLABLE TA\0"
  "YI SYLLABLE TAP\0"
  "YI SYLLABLE TAT\0"
  "YI SYLLABLE TAX\0"
  "YI SYLLABLE TE\0"
  "YI SYLLABLE TEP\0"
  "YI SYLLABLE TEX\0"
  "YI SYLLABLE TI\0"
  "YI SYLLABLE TIE\0"
  "YI SYLLABLE TIEP\0"
  "YI SYLLABLE TIEX\0"
  "YI SYLLABLE TIP\0"
  "YI SYLLABLE TIT\0"
  "YI SYLLABLE TIX\0"
  "YI SYLLABLE TO\0"
  "YI SYLLABLE TOP\0"
  "YI SYLLABLE TOT\0"
  "YI SYLLABLE TOX\0"
  "YI SYLLABLE TU\0"
  "YI SYLLABLE TUO\0"
  "YI SYLLABLE TUOP\0"
  "YI SYLLABLE TUOT\0"
  "YI SYLLABLE TUOX\0"
  "YI SYLLABLE TUP\0"
  "YI SYLLABLE TUR\0"
  "YI SYLLABLE TURX\0"
  "YI SYLLABLE TUT\0"
  "YI SYLLABLE TUX\0"
  "YI SYLLABLE UO\0"
  "YI SYLLABLE UOP\0"
  "YI SYLLABLE UOX\0"
  "YI SYLLABLE VA\0"
  "YI SYLLABLE VAP\0"
  "YI SYLLABLE VAT\0"
  "YI SYLLABLE VAX\0"
  "YI SYLLABLE VEP\0"
  "YI SYLLABLE VEX\0"
  "YI SYLLABLE VI\0"
  "YI SYLLABLE VIE\0"
  "YI SYLLABLE VIEP\0"
  "YI SYLLABLE VIET\0"
  "YI SYLLABLE VIEX\0"
  "YI SYLLABLE VIP\0"
  "YI SYLLABLE VIT\0"
  "YI SYLLABLE VIX\0"
  "YI SYLLABLE VO\0"
  "YI SYLLABLE VOP\0"
  "YI SYLLABLE VOT\0"
  "YI SYLLABLE VOX\0"
  "YI SYLLABLE VU\0"
  "YI SYLLABLE VUP\0"
  "YI SYLLABLE VUR\0"
  "YI SYLLABLE VURX\0"
  "YI SYLLABLE VUT\0"
  "YI SYLLABLE VUX\0"
  "YI SYLLABLE VY\0"
  "YI SYLLABLE VYP\0"
  "YI SYLLABLE VYR\0"
  "YI SYLLABLE VYRX\0"
  "YI SYLLABLE VYT\0"
  "YI SYLLABLE VYX\0"
  "YI SYLLABLE WA\0"
  "YI SYLLABLE WAP\0"
  "YI SYLLABLE WAT\0"
  "YI SYLLABLE WAX\0"
  "YI SYLLABLE WE\0"
  "YI SYLLABLE WEP\0"
  "YI SYLLABLE WEX\0"
  "YI SYLLABLE WO\0"
  "YI SYLLABLE WOP\0"
  "YI SYLLABLE WOX\0"
  "YI SYLLABLE WU\0"
  "YI SYLLABLE WUO\0"
  "YI SYLLABLE WUOP\0"
  "YI SYLLABLE WUOX\0"
  "YI SYLLABLE XI\0"
  "YI SYLLABLE XIE\0"
  "YI SYLLABLE XIEP\0"
  "YI SYLLABLE XIET\0"
  "YI SYLLABLE XIEX\0"
  "YI SYLLABLE XIP\0"
  "YI SYLLABLE XIT\0"
  "YI SYLLABLE XIX\0"
  "YI SYLLABLE XO\0"
  "YI SYLLABLE XOP\0"
  "YI SYLLABLE XOT\0"
  "YI SYLLABLE XOX\0"
  "YI SYLLABLE XUO\0"
  "YI SYLLABLE XUOX\0"
  "YI SYLLABLE XY\0"
  "YI SYLLABLE XYP\0"
  "YI SYLLABLE XYR\0"
  "YI SYLLABLE XYRX\0"
  "YI SYLLABLE XYT\0"
  "YI SYLLABLE XYX\0"
  "YI SYLLABLE YI\0"
  "YI SYLLABLE YIE\0"
  "YI SYLLABLE YIEP\0"
  "YI SYLLABLE YIET\0"
  "YI SYLLABLE YIEX\0"
  "YI SYLLABLE YIP\0"
  "YI SYLLABLE YIT\0"
  "YI SYLLABLE YIX\0"
  "YI SYLLABLE YO\0"
  "YI SYLLABLE YOP\0"
  "YI SYLLABLE YOT\0"
  "YI SYLLABLE YOX\0"
  "YI SYLLABLE YU\0"
  "YI SYLLABLE YUO\0"
  "YI SYLLABLE YUOP\0"
  "YI SYLLABLE YUOT\0"
  "YI SYLLABLE YUOX\0"
  "YI SYLLABLE YUP\0"
  "YI SYLLABLE YUR\0"
  "YI SYLLABLE YURX\0"
  "YI SYLLABLE YUT\0"
  "YI SYLLABLE YUX\0"
  "YI SYLLABLE YY\0"
  "YI SYLLABLE YYP\0"
  "YI SYLLABLE YYR\0"
  "YI SYLLABLE YYRX\0"
  "YI SYLLABLE YYT\0"
  "YI SYLLABLE YYX\0"
  "YI SYLLABLE ZA\0"
  "YI SYLLABLE ZAP\0"
  "YI SYLLABLE ZAT\0"
  "YI SYLLABLE ZAX\0"
  "YI SYLLABLE ZE\0"
  "YI SYLLABLE ZEP\0"
  "YI SYLLABLE ZEX\0"
  "YI SYLLABLE ZHA\0"
  "YI SYLLABLE ZHAP\0"
  "YI SYLLABLE ZHAT\0"
  "YI SYLLABLE ZHAX\0"
  "YI SYLLABLE ZHE\0"
  "YI SYLLABLE ZHEP\0"
  "YI SYLLABLE ZHET\0"
  "YI SYLLABLE ZHEX\0"
  "YI SYLLABLE ZHO\0"
  "YI SYLLABLE ZHOP\0"
  "YI SYLLABLE ZHOT\0"
  "YI SYLLABLE ZHOX\0"
  "YI SYLLABLE ZHU\0"
  "YI SYLLABLE ZHUO\0"
  "YI SYLLABLE ZHUOP\0"
  "YI SYLLABLE ZHUOX\0"
  "YI SYLLABLE ZHUP\0"
  "YI SYLLABLE ZHUR\0"
  "YI SYLLABLE ZHURX\0"
  "YI SYLLABLE ZHUT\0"
  "YI SYLLABLE ZHUX\0"
  "YI SYLLABLE ZHY\0"
  "YI SYLLABLE ZHYP\0"
  "YI SYLLABLE ZHYR\0"
  "YI SYLLABLE ZHYRX\0"
  "YI SYLLABLE ZHYT\0"
  "YI SYLLABLE ZHYX\0"
  "YI SYLLABLE ZI\0"
  "YI SYLLABLE ZIE\0"
  "YI SYLLABLE ZIEP\0"
  "YI SYLLABLE ZIEX\0"
  "YI SYLLABLE ZIP\0"
  "YI SYLLABLE ZIT\0"
  "YI SYLLABLE ZIX\0"
  "YI SYLLABLE ZO\0"
  "YI SYLLABLE ZOP\0"
  "YI SYLLABLE ZOT\0"
  "YI SYLLABLE ZOX\0"
  "YI SYLLABLE ZU\0"
  "YI SYLLABLE ZUO\0"
  "YI SYLLABLE ZUOP\0"
  "YI SYLLABLE ZUOX\0"
  "YI SYLLABLE ZUP\0"
  "YI SYLLABLE ZUR\0"
  "YI SYLLABLE ZURX\0"
  "YI SYLLABLE ZUT\0"
  "YI SYLLABLE ZUX\0"
  "YI SYLLABLE ZY\0"
  "YI SYLLABLE ZYP\0"
  "YI SYLLABLE ZYR\0"
  "YI SYLLABLE ZYRX\0"
  "YI SYLLABLE ZYT\0"
  "YI SYLLABLE ZYX\0"
  "YI SYLLABLE ZZA\0"
  "YI SYLLABLE ZZAP\0"
  "YI SYLLABLE ZZAT\0"
  "YI SYLLABLE ZZAX\0"
  "YI SYLLABLE ZZE\0"
  "YI SYLLABLE ZZEP\0"
  "YI SYLLABLE ZZEX\0"
  "YI SYLLABLE ZZI\0"
  "YI SYLLABLE ZZIE\0"
  "YI SYLLABLE ZZIEP\0"
  "YI SYLLABLE ZZIET\0"
  "YI SYLLABLE ZZIEX\0"
  "YI SYLLABLE ZZIP\0"
  "YI SYLLABLE ZZIT\0"
  "YI SYLLABLE ZZIX\0"
  "YI SYLLABLE ZZO\0"
  "YI SYLLABLE ZZOP\0"
  "YI SYLLABLE ZZOX\0"
  "YI SYLLABLE ZZU\0"
  "YI SYLLABLE ZZUP\0"
  "YI SYLLABLE ZZUR\0"
  "YI SYLLABLE ZZURX\0"
  "YI SYLLABLE ZZUX\0"
  "YI SYLLABLE ZZY\0"
  "YI SYLLABLE ZZYP\0"
  "YI SYLLABLE ZZYR\0"
  "YI SYLLABLE ZZYRX\0"
  "YI SYLLABLE ZZYT\0"
  "YI SYLLABLE ZZYX\0"
  "YIN YANG\0"
  "YO-YO\0"
  "Z NOTATION BAG MEMBERSHIP\0"
  "Z NOTATION DOMAIN ANTIRESTRICTION\0"
  "Z NOTATION LEFT BINDING BRACKET\0"
  "Z NOTATION LEFT IMAGE BRACKET\0"
  "Z NOTATION RANGE ANTIRESTRICTION\0"
  "Z NOTATION RELATIONAL COMPOSITION\0"
  "Z NOTATION RIGHT BINDING BRACKET\0"
  "Z NOTATION RIGHT IMAGE BRACKET\0"
  "Z NOTATION SCHEMA COMPOSITION\0"
  "Z NOTATION SCHEMA PIPING\0"
  "Z NOTATION SCHEMA PROJECTION\0"
  "Z NOTATION SPOT\0"
  "Z NOTATION TYPE COLON\0"
  "ZANABAZAR SQUARE CLOSING DOUBLE-LINED HEAD MARK\0"
  "ZANABAZAR SQUARE CLOSING HEAD MARK\0"
  "ZANABAZAR SQUARE CLUSTER-FINAL LETTER LA\0"
  "ZANABAZAR SQUARE CLUSTER-FINAL LETTER RA\0"
  "ZANABAZAR SQUARE CLUSTER-FINAL LETTER VA\0"
  "ZANABAZAR SQUARE CLUSTER-FINAL LETTER YA\0"
  "ZANABAZAR SQUARE CLUSTER-INITIAL LETTER RA\0"
  "ZANABAZAR SQUARE FINAL CONSONANT MARK\0"
  "ZANABAZAR SQUARE INITIAL DOUBLE-LINED HEAD MARK\0"
  "ZANABAZAR SQUARE INITIAL HEAD MARK\0"
  "ZANABAZAR SQUARE LETTER -A\0"
  "ZANABAZAR SQUARE LETTER A\0"
  "ZANABAZAR SQUARE LETTER BA\0"
  "ZANABAZAR SQUARE LETTER BHA\0"
  "ZANABAZAR SQUARE LETTER CA\0"
  "ZANABAZAR SQUARE LETTER CHA\0"
  "ZANABAZAR SQUARE LETTER DA\0"
  "ZANABAZAR SQUARE LETTER DDA\0"
  "ZANABAZAR SQUARE LETTER DDHA\0"
  "ZANABAZAR SQUARE LETTER DHA\0"
  "ZANABAZAR SQUARE LETTER DZA\0"
  "ZANABAZAR SQUARE LETTER DZHA\0"
  "ZANABAZAR SQUARE LETTER GA\0"
  "ZANABAZAR SQUARE LETTER GHA\0"
  "ZANABAZAR SQUARE LETTER HA\0"
  "ZANABAZAR SQUARE LETTER JA\0"
  "ZANABAZAR SQUARE LETTER KA\0"
  "ZANABAZAR SQUARE LETTER KHA\0"
  "ZANABAZAR SQUARE LETTER KSSA\0"
  "ZANABAZAR SQUARE LETTER LA\0"
  "ZANABAZAR SQUARE LETTER MA\0"
  "ZANABAZAR SQUARE LETTER NA\0"
  "ZANABAZAR SQUARE LETTER NGA\0"
  "ZANABAZAR SQUARE LETTER NNA\0"
  "ZANABAZAR SQUARE LETTER NYA\0"
  "ZANABAZAR SQUARE LETTER PA\0"
  "ZANABAZAR SQUARE LETTER PHA\0"
  "ZANABAZAR SQUARE LETTER RA\0"
  "ZANABAZAR SQUARE LETTER SA\0"
  "ZANABAZAR SQUARE LETTER SHA\0"
  "ZANABAZAR SQUARE LETTER SSA\0"
  "ZANABAZAR SQUARE LETTER TA\0"
  "ZANABAZAR SQUARE LETTER THA\0"
  "ZANABAZAR SQUARE LETTER TSA\0"
  "ZANABAZAR SQUARE LETTER TSHA\0"
  "ZANABAZAR SQUARE LETTER TTA\0"
  "ZANABAZAR SQUARE LETTER TTHA\0"
  "ZANABAZAR SQUARE LETTER VA\0"
  "ZANABAZAR SQUARE LETTER YA\0"
  "ZANABAZAR SQUARE LETTER ZA\0"
  "ZANABAZAR SQUARE LETTER ZHA\0"
  "ZANABAZAR SQUARE MARK DOUBLE SHAD\0"
  "ZANABAZAR SQUARE MARK LONG TSHEG\0"
  "ZANABAZAR SQUARE MARK SHAD\0"
  "ZANABAZAR SQUARE MARK TSHEG\0"
  "ZANABAZAR SQUARE SIGN ANUSVARA\0"
  "ZANABAZAR SQUARE SIGN CANDRA WITH ORNAMENT\0"
  "ZANABAZAR SQUARE SIGN CANDRABINDU\0"
  "ZANABAZAR SQUARE SIGN CANDRABINDU WITH ORNAMENT\0"
  "ZANABAZAR SQUARE SIGN VIRAMA\0"
  "ZANABAZAR SQUARE SIGN VISARGA\0"
  "ZANABAZAR SQUARE SUBJOINER\0"
  "ZANABAZAR SQUARE VOWEL LENGTH MARK\0"
  "ZANABAZAR SQUARE VOWEL SIGN AI\0"
  "ZANABAZAR SQUARE VOWEL SIGN AU\0"
  "ZANABAZAR SQUARE VOWEL SIGN E\0"
  "ZANABAZAR SQUARE VOWEL SIGN I\0"
  "ZANABAZAR SQUARE VOWEL SIGN O\0"
  "ZANABAZAR SQUARE VOWEL SIGN OE\0"
  "ZANABAZAR SQUARE VOWEL SIGN REVERSED I\0"
  "ZANABAZAR SQUARE VOWEL SIGN U\0"
  "ZANABAZAR SQUARE VOWEL SIGN UE\0"
  "ZEBRA FACE\0"
  "ZERO WIDTH JOINER\0"
  "ZERO WIDTH NO-BREAK SPACE\0"
  "ZERO WIDTH NON-JOINER\0"
  "ZERO WIDTH SPACE\0"
  "ZEUS\0"
  "ZIPPER-MOUTH FACE\0"
  "ZNAMENNY COMBINING ATTACHING VERTICAL OMET\0"
  "ZNAMENNY COMBINING LOWER TONAL RANGE INDICATOR\0"
  "ZNAMENNY COMBINING MARK BORZAYA\0"
  "ZNAMENNY COMBINING MARK CHASHKA\0"
  "ZNAMENNY COMBINING MARK CHASHKA POLNAYA\0"
  "ZNAMENNY COMBINING MARK CURVED OMET\0"
  "ZNAMENNY COMBINING MARK DEMESTVENNY ZADERZHKA\0"
  "ZNAMENNY COMBINING MARK DVOETOCHIE\0"
  "ZNAMENNY COMBINING MARK GORAZDO NIZKO ON LEFT\0"
  "ZNAMENNY COMBINING MARK GORAZDO NIZKO ON RIGHT\0"
  "ZNAMENNY COMBINING MARK GORAZDO NIZKO S KRYZHEM ON LEFT\0"
  "ZNAMENNY COMBINING MARK GORAZDO NIZKO S KRYZHEM ON RIGHT\0"
  "ZNAMENNY COMBINING MARK GORAZDO VYSOKO\0"
  "ZNAMENNY COMBINING MARK KACHKA\0"
  "ZNAMENNY COMBINING MARK KRYZH\0"
  "ZNAMENNY COMBINING MARK KRYZH ON LEFT\0"
  "ZNAMENNY COMBINING MARK KUPNAYA\0"
  "ZNAMENNY COMBINING MARK LOMKA\0"
  "ZNAMENNY COMBINING MARK MALO POVYSHE ON LEFT\0"
  "ZNAMENNY COMBINING MARK MALO POVYSHE ON RIGHT\0"
  "ZNAMENNY COMBINING MARK MALO POVYSHE S KHOKHLOM ON LEFT\0"
  "ZNAMENNY COMBINING MARK MALO POVYSHE S KHOKHLOM ON RIGHT\0"
  "ZNAMENNY COMBINING MARK MALO POVYSHE S KRYZHEM\0"
  "ZNAMENNY COMBINING MARK MALO POVYSHE STRANNO\0"
  "ZNAMENNY COMBINING MARK NIZKO ON LEFT\0"
  "ZNAMENNY COMBINING MARK NIZKO ON RIGHT\0"
  "ZNAMENNY COMBINING MARK NIZKO S KRYZHEM ON LEFT\0"
  "ZNAMENNY COMBINING MARK NIZKO S KRYZHEM ON RIGHT\0"
  "ZNAMENNY COMBINING MARK OBLACHKO\0"
  "ZNAMENNY COMBINING MARK ON\0"
  "ZNAMENNY COMBINING MARK OTSECHKA\0"
  "ZNAMENNY COMBINING MARK PODCHASHIE\0"
  "ZNAMENNY COMBINING MARK PODCHASHIE WITH VERTICAL STROKE\0"
  "ZNAMENNY COMBINING MARK PODVERTKA\0"
  "ZNAMENNY COMBINING MARK POVYSHE ON LEFT\0"
  "ZNAMENNY COMBINING MARK POVYSHE ON RIGHT\0"
  "ZNAMENNY COMBINING MARK POVYSHE S KHOKHLOM ON LEFT\0"
  "ZNAMENNY COMBINING MARK POVYSHE S KHOKHLOM ON RIGHT\0"
  "ZNAMENNY COMBINING MARK POVYSHE S KRYZHEM\0"
  "ZNAMENNY COMBINING MARK POVYSHE STRANNO\0"
  "ZNAMENNY COMBINING MARK RAVNO\0"
  "ZNAMENNY COMBINING MARK RAZSEKA\0"
  "ZNAMENNY COMBINING MARK SKOBA\0"
  "ZNAMENNY COMBINING MARK SOROCHYA NOZHKA\0"
  "ZNAMENNY COMBINING MARK SREDNE ON LEFT\0"
  "ZNAMENNY COMBINING MARK SREDNE ON RIGHT\0"
  "ZNAMENNY COMBINING MARK STRANNO MALO POVYSHE\0"
  "ZNAMENNY COMBINING MARK TIKHAYA\0"
  "ZNAMENNY COMBINING MARK TOCHKA\0"
  "ZNAMENNY COMBINING MARK TSATA ON LEFT\0"
  "ZNAMENNY COMBINING MARK TSATA ON RIGHT\0"
  "ZNAMENNY COMBINING MARK TSATA S KRYZHEM\0"
  "ZNAMENNY COMBINING MARK UDARKA\0"
  "ZNAMENNY COMBINING MARK VYSOKO ON LEFT\0"
  "ZNAMENNY COMBINING MARK VYSOKO ON RIGHT\0"
  "ZNAMENNY COMBINING MARK VYSOKO S KHOKHLOM ON LEFT\0"
  "ZNAMENNY COMBINING MARK VYSOKO S KHOKHLOM ON RIGHT\0"
  "ZNAMENNY COMBINING MARK VYSOKO S KRYZHEM\0"
  "ZNAMENNY COMBINING MARK ZADERZHKA\0"
  "ZNAMENNY COMBINING MARK ZELO\0"
  "ZNAMENNY COMBINING MARK ZEVOK\0"
  "ZNAMENNY COMBINING TONAL RANGE MARK MRACHNO\0"
  "ZNAMENNY COMBINING TONAL RANGE MARK SVETLO\0"
  "ZNAMENNY COMBINING TONAL RANGE MARK TRESVETLO\0"
  "ZNAMENNY NEUME CHASHKA\0"
  "ZNAMENNY NEUME CHELYUSTKA\0"
  "ZNAMENNY NEUME DEMESTVENNY KLYUCH\0"
  "ZNAMENNY NEUME DERBITSA\0"
  "ZNAMENNY NEUME DOUBLE ZAPYATAYA\0"
  "ZNAMENNY NEUME DUDA\0"
  "ZNAMENNY NEUME DVA V CHELNU\0"
  "ZNAMENNY NEUME FITA\0"
  "ZNAMENNY NEUME GOLUBCHIK BORZY\0"
  "ZNAMENNY NEUME GOLUBCHIK MRACHNY\0"
  "ZNAMENNY NEUME GOLUBCHIK SVETLY\0"
  "ZNAMENNY NEUME GOLUBCHIK TIKHY\0"
  "ZNAMENNY NEUME GOLUBCHIK TRESVETLY\0"
  "ZNAMENNY NEUME KHAMILO\0"
  "ZNAMENNY NEUME KLYUCH\0"
  "ZNAMENNY NEUME KOBYLA\0"
  "ZNAMENNY NEUME KRYUK\0"
  "ZNAMENNY NEUME KRYUK TIKHY\0"
  "ZNAMENNY NEUME KRYZH\0"
  "ZNAMENNY NEUME KUFISMA\0"
  "ZNAMENNY NEUME MECHIK\0"
  "ZNAMENNY NEUME MECHIK KLYUCHENEPOSTOYANNY\0"
  "ZNAMENNY NEUME MECHIK KLYUCHEPOVODNY\0"
  "ZNAMENNY NEUME MECHIK KLYUCHEVOY\0"
  "ZNAMENNY NEUME MECHIK POVODNY\0"
  "ZNAMENNY NEUME NEMKA\0"
  "ZNAMENNY NEUME OBLAKO\0"
  "ZNAMENNY NEUME OSOKA\0"
  "ZNAMENNY NEUME OSOKA KLYUCHEVAYA NEPOSTOYANNAYA\0"
  "ZNAMENNY NEUME OSOKA KLYUCHEVAYA SVETLAYA\0"
  "ZNAMENNY NEUME OSOKA KRYUKOVAYA SVETLAYA\0"
  "ZNAMENNY NEUME OSOKA SVETLAYA\0"
  "ZNAMENNY NEUME OSOKA TRESVETLAYA\0"
  "ZNAMENNY NEUME PALKA\0"
  "ZNAMENNY NEUME PARAKLIT\0"
  "ZNAMENNY NEUME PAUK\0"
  "ZNAMENNY NEUME PEREVODKA\0"
  "ZNAMENNY NEUME PEREVODKA NEPOSTOYANNAYA\0"
  "ZNAMENNY NEUME PODCHASHIE\0"
  "ZNAMENNY NEUME POLKULIZMY\0"
  "ZNAMENNY NEUME REVERSED CHELYUSTKA\0"
  "ZNAMENNY NEUME ROG\0"
  "ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA KLYUCHEVAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA NEPOSTOYANNAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA PROSTAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA SVETLAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA KLYUCHENEPOSTOYANNAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA KLYUCHEVAYA SVETLAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA KLYUCHEVAYA TIKHAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA MRACHNAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA SVETLAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA TIKHAYA\0"
  "ZNAMENNY NEUME SKAMEYTSA TRESVETLAYA\0"
  "ZNAMENNY NEUME SLOZHITIE\0"
  "ZNAMENNY NEUME SLOZHITIE S KRYZHEM\0"
  "ZNAMENNY NEUME SLOZHITIE S ZAPYATOY\0"
  "ZNAMENNY NEUME SLOZHITIE ZAKRYTOE\0"
  "ZNAMENNY NEUME STATYA\0"
  "ZNAMENNY NEUME STATYA NEPOSTOYANNAYA\0"
  "ZNAMENNY NEUME STATYA S DVUMYA ZAPYATYMI\0"
  "ZNAMENNY NEUME STATYA S KRYZHEM\0"
  "ZNAMENNY NEUME STATYA S KRYZHEM I ZAPYATOY\0"
  "ZNAMENNY NEUME STATYA S ROGOM\0"
  "ZNAMENNY NEUME STATYA S ZAPYATOY\0"
  "ZNAMENNY NEUME STATYA S ZAPYATOY I KRYZHEM\0"
  "ZNAMENNY NEUME STATYA S ZAPYATOY I PODCHASHIEM\0"
  "ZNAMENNY NEUME STATYA ZAKRYTAYA\0"
  "ZNAMENNY NEUME STATYA ZAKRYTAYA S ZAPYATOY\0"
  "ZNAMENNY NEUME STOPITSA\0"
  "ZNAMENNY NEUME STOPITSA S OCHKOM\0"
  "ZNAMENNY NEUME STOPITSA WITH SOROCHYA NOZHKA\0"
  "ZNAMENNY NEUME STRELA DVOECHELNAYA\0"
  "ZNAMENNY NEUME STRELA DVOECHELNAYA KLYUCHEVAYA\0"
  "ZNAMENNY NEUME STRELA DVOECHELNOKRYZHEVAYA\0"
  "ZNAMENNY NEUME STRELA DVOECHELNOPOVODNAYA\0"
  "ZNAMENNY NEUME STRELA DVOECHELNOPOVODNAYA KLYUCHEVAYA\0"
  "ZNAMENNY NEUME STRELA GROMNAYA\0"
  "ZNAMENNY NEUME STRELA GROMNAYA WITH SINGLE ZAPYATAYA\0"
  "ZNAMENNY NEUME STRELA GROMOKRYZHEVAYA\0"
  "ZNAMENNY NEUME STRELA GROMOKRYZHEVAYA POVODNAYA\0"
  "ZNAMENNY NEUME STRELA GROMOPOVODNAYA\0"
  "ZNAMENNY NEUME STRELA GROMOPOVODNAYA WITH DOUBLE ZAPYATAYA\0"
  "ZNAMENNY NEUME STRELA GROMOPOVODNAYA WITH SINGLE ZAPYATAYA\0"
  "ZNAMENNY NEUME STRELA KLYUCHENEPOSTOYANNAYA\0"
  "ZNAMENNY NEUME STRELA KLYUCHEPOVODNAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMNAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMNAYA WITH SINGLE ZAPYATAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOKRYZHEVAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOKRYZHEVAYA POVODNAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOPOVODNAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOPOVODNAYA WITH DOUBLE ZAPYATAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOPOVODNAYA WITH SINGLE ZAPYATAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA POVODNAYA\0"
  "ZNAMENNY NEUME STRELA KRYUKOVAYA TRYASKA\0"
  "ZNAMENNY NEUME STRELA KRYZHEVAYA\0"
  "ZNAMENNY NEUME STRELA MRACHNOTIKHAYA\0"
  "ZNAMENNY NEUME STRELA NEPOSTOYANNAYA\0"
  "ZNAMENNY NEUME STRELA POLUPOVODNAYA\0"
  "ZNAMENNY NEUME STRELA POVODNAYA\0"
  "ZNAMENNY NEUME STRELA PROSTAYA\0"
  "ZNAMENNY NEUME STRELA TIKHAYA PUTNAYA\0"
  "ZNAMENNY NEUME STRELA TRYASOGLASNAYA\0"
  "ZNAMENNY NEUME STRELA TRYASOPOVODNAYA\0"
  "ZNAMENNY NEUME STRELA TRYASOSTRELNAYA\0"
  "ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA MRACHNAYA\0"
  "ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA PROSTAYA\0"
  "ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA SVETLAYA\0"
  "ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA TRESVETLAYA\0"
  "ZNAMENNY NEUME VRAKHIYA MRACHNAYA\0"
  "ZNAMENNY NEUME VRAKHIYA PROSTAYA\0"
  "ZNAMENNY NEUME VRAKHIYA SVETLAYA\0"
  "ZNAMENNY NEUME VRAKHIYA TRESVETLAYA\0"
  "ZNAMENNY NEUME ZANOZHEK\0"
  "ZNAMENNY NEUME ZAPYATAYA\0"
  "ZNAMENNY NEUME ZMEYTSA\0"
  "ZNAMENNY PRIZNAK MODIFIER DIRECTION FLIP\0"
  "ZNAMENNY PRIZNAK MODIFIER KRYZH\0"
  "ZNAMENNY PRIZNAK MODIFIER LEVEL-2\0"
  "ZNAMENNY PRIZNAK MODIFIER LEVEL-3\0"
  "ZNAMENNY PRIZNAK MODIFIER ROG\0"
  "ZOMBIE\0"
;
typedef struct _FontManagerUnicodeName FontManagerUnicodeName;

static const struct _FontManagerUnicodeName
{
  gunichar index;
  guint32 name_offset;
} 
unicode_names[] =
{
  {0x0000, 0},
  {0x0001, 0},
  {0x0002, 0},
  {0x0003, 0},
  {0x0004, 0},
  {0x0005, 0},
  {0x0006, 0},
  {0x0007, 0},
  {0x0008, 0},
  {0x0009, 0},
  {0x000A, 0},
  {0x000B, 0},
  {0x000C, 0},
  {0x000D, 0},
  {0x000E, 0},
  {0x000F, 0},
  {0x0010, 0},
  {0x0011, 0},
  {0x0012, 0},
  {0x0013, 0},
  {0x0014, 0},
  {0x0015, 0},
  {0x0016, 0},
  {0x0017, 0},
  {0x0018, 0},
  {0x0019, 0},
  {0x001A, 0},
  {0x001B, 0},
  {0x001C, 0},
  {0x001D, 0},
  {0x001E, 0},
  {0x001F, 0},
  {0x0020, 899014},
  {0x0021, 441863},
  {0x0022, 830024},
  {0x0023, 774051},
  {0x0024, 282130},
  {0x0025, 819344},
  {0x0026, 9293},
  {0x0027, 27746},
  {0x0028, 627546},
  {0x0029, 835298},
  {0x002A, 82009},
  {0x002B, 825424},
  {0x002C, 208738},
  {0x002D, 524383},
  {0x002E, 445226},
  {0x002F, 895364},
  {0x0030, 277147},
  {0x0031, 276905},
  {0x0032, 277101},
  {0x0033, 277049},
  {0x0034, 276807},
  {0x0035, 276758},
  {0x0036, 277003},
  {0x0037, 276951},
  {0x0038, 276706},
  {0x0039, 276856},
  {0x003A, 196574},
  {0x003B, 850715},
  {0x003C, 636369},
  {0x003D, 429123},
  {0x003E, 460971},
  {0x003F, 829981},
  {0x0040, 208744},
  {0x0041, 582437},
  {0x0042, 584069},
  {0x0043, 584403},
  {0x0044, 584955},
  {0x0045, 585516},
  {0x0046, 586874},
  {0x0047, 587003},
  {0x0048, 587541},
  {0x0049, 588014},
  {0x004A, 588912},
  {0x004B, 589050},
  {0x004C, 589502},
  {0x004D, 590194},
  {0x004E, 590469},
  {0x004F, 591053},
  {0x0050, 592906},
  {0x0051, 593201},
  {0x0052, 593321},
  {0x0053, 594049},
  {0x0054, 594915},
  {0x0055, 595913},
  {0x0056, 597257},
  {0x0057, 597609},
  {0x0058, 597912},
  {0x0059, 598011},
  {0x005A, 598490},
  {0x005B, 628252},
  {0x005C, 832764},
  {0x005D, 835670},
  {0x005E, 191755},
  {0x005F, 652863},
  {0x0060, 460031},
  {0x0061, 601478},
  {0x0062, 603159},
  {0x0063, 603815},
  {0x0064, 604600},
  {0x0065, 605883},
  {0x0066, 607713},
  {0x0067, 607987},
  {0x0068, 608532},
  {0x0069, 609084},
  {0x006A, 610118},
  {0x006B, 610280},
  {0x006C, 610770},
  {0x006D, 612135},
  {0x006E, 612534},
  {0x006F, 613348},
  {0x0070, 615308},
  {0x0071, 615688},
  {0x0072, 615899},
  {0x0073, 617257},
  {0x0074, 618660},
  {0x0075, 621199},
  {0x0076, 622767},
  {0x0077, 623204},
  {0x0078, 623502},
  {0x0079, 623825},
  {0x007A, 624355},
  {0x007B, 626458},
  {0x007C, 975118},
  {0x007D, 834321},
  {0x007E, 945048},
  {0x007F, 0},
  {0x0080, 0},
  {0x0081, 0},
  {0x0082, 0},
  {0x0083, 0},
  {0x0084, 0},
  {0x0085, 0},
  {0x0086, 0},
  {0x0087, 0},
  {0x0088, 0},
  {0x0089, 0},
  {0x008A, 0},
  {0x008B, 0},
  {0x008C, 0},
  {0x008D, 0},
  {0x008E, 0},
  {0x008F, 0},
  {0x0090, 0},
  {0x0091, 0},
  {0x0092, 0},
  {0x0093, 0},
  {0x0094, 0},
  {0x0095, 0},
  {0x0096, 0},
  {0x0097, 0},
  {0x0098, 0},
  {0x0099, 0},
  {0x009A, 0},
  {0x009B, 0},
  {0x009C, 0},
  {0x009D, 0},
  {0x009E, 0},
  {0x009F, 0},
  {0x00A0, 771789},
  {0x00A1, 535923},
  {0x00A2, 176396},
  {0x00A3, 826159},
  {0x00A4, 251696},
  {0x00A5, 989289},
  {0x00A6, 141844},
  {0x00A7, 850436},
  {0x00A8, 276372},
  {0x00A9, 214171},
  {0x00AA, 443702},
  {0x00AB, 629671},
  {0x00AC, 773538},
  {0x00AD, 894132},
  {0x00AE, 831835},
  {0x00AF, 657796},
  {0x00B0, 269513},
  {0x00B1, 825756},
  {0x00B2, 911298},
  {0x00B3, 911280},
  {0x00B4, 106},
  {0x00B5, 725005},
  {0x00B6, 822990},
  {0x00B7, 725046},
  {0x00B8, 176375},
  {0x00B9, 911178},
  {0x00BA, 668797},
  {0x00BB, 837116},
  {0x00BC, 978248},
  {0x00BD, 978197},
  {0x00BE, 978471},
  {0x00BF, 536081},
  {0x00C0, 583337},
  {0x00C1, 582460},
  {0x00C2, 582791},
  {0x00C3, 583685},
  {0x00C4, 583084},
  {0x00C5, 583523},
  {0x00C6, 583743},
  {0x00C7, 584526},
  {0x00C8, 586216},
  {0x00C9, 585539},
  {0x00CA, 585723},
  {0x00CB, 586061},
  {0x00CC, 588381},
  {0x00CD, 588037},
  {0x00CE, 588139},
  {0x00CF, 588178},
  {0x00D0, 586754},
  {0x00D1, 590995},
  {0x00D2, 591765},
  {0x00D3, 591076},
  {0x00D4, 591178},
  {0x00D5, 592546},
  {0x00D6, 591471},
  {0x00D7, 743927},
  {0x00D8, 592466},
  {0x00D9, 596544},
  {0x00DA, 595963},
  {0x00DB, 596065},
  {0x00DC, 596149},
  {0x00DD, 598034},
  {0x00DE, 595363},
  {0x00DF, 618321},
  {0x00E0, 602372},
  {0x00E1, 601535},
  {0x00E2, 601850},
  {0x00E3, 602785},
  {0x00E4, 602131},
  {0x00E5, 602631},
  {0x00E6, 602839},
  {0x00E7, 603930},
  {0x00E8, 606584},
  {0x00E9, 605904},
  {0x00EA, 606078},
  {0x00EB, 606402},
  {0x00EC, 609395},
  {0x00ED, 609105},
  {0x00EE, 609201},
  {0x00EF, 609238},
  {0x00F0, 607451},
  {0x00F1, 613271},
  {0x00F2, 614026},
  {0x00F3, 613369},
  {0x00F4, 613465},
  {0x00F5, 614813},
  {0x00F6, 613746},
  {0x00F7, 280567},
  {0x00F8, 614737},
  {0x00F9, 621844},
  {0x00FA, 621291},
  {0x00FB, 621387},
  {0x00FC, 621467},
  {0x00FD, 623846},
  {0x00FE, 619573},
  {0x00FF, 623915},
  {0x0100, 583453},
  {0x0101, 602482},
  {0x0102, 582494},
  {0x0103, 601567},
  {0x0104, 583488},
  {0x0105, 602515},
  {0x0106, 584426},
  {0x0107, 603836},
  {0x0108, 584608},
  {0x0109, 604008},
  {0x010A, 584647},
  {0x010B, 604076},
  {0x010C, 584492},
  {0x010D, 603898},
  {0x010E, 584978},
  {0x010F, 604621},
  {0x0110, 585387},
  {0x0111, 605113},
  {0x0112, 586332},
  {0x0113, 606694},
  {0x0114, 585573},
  {0x0115, 605936},
  {0x0116, 586099},
  {0x0117, 606438},
  {0x0118, 586457},
  {0x0119, 606845},
  {0x011A, 585607},
  {0x011B, 605968},
  {0x011C, 587164},
  {0x011D, 608138},
  {0x011E, 587060},
  {0x011F, 608040},
  {0x0120, 587203},
  {0x0121, 608175},
  {0x0122, 587128},
  {0x0123, 608104},
  {0x0124, 587674},
  {0x0125, 608657},
  {0x0126, 587898},
  {0x0127, 608945},
  {0x0128, 588602},
  {0x0129, 609697},
  {0x012A, 588497},
  {0x012B, 609505},
  {0x012C, 588071},
  {0x012D, 609137},
  {0x012E, 588532},
  {0x012F, 609538},
  {0x0130, 588264},
  {0x0131, 605399},
  {0x0132, 598921},
  {0x0133, 624995},
  {0x0134, 588935},
  {0x0135, 610171},
  {0x0136, 589141},
  {0x0137, 610365},
  {0x0138, 610747},
  {0x0139, 589525},
  {0x013A, 610791},
  {0x013B, 589658},
  {0x013C, 610964},
  {0x013D, 589624},
  {0x013E, 610932},
  {0x013F, 589944},
  {0x0140, 611437},
  {0x0141, 590067},
  {0x0142, 611681},
  {0x0143, 590492},
  {0x0144, 612599},
  {0x0145, 590560},
  {0x0146, 612663},
  {0x0147, 590526},
  {0x0148, 612631},
  {0x0149, 612555},
  {0x014A, 586680},
  {0x014B, 607097},
  {0x014C, 592219},
  {0x014D, 614500},
  {0x014E, 591110},
  {0x014F, 613401},
  {0x0150, 591683},
  {0x0151, 613948},
  {0x0152, 598947},
  {0x0153, 625049},
  {0x0154, 593375},
  {0x0155, 615949},
  {0x0156, 593443},
  {0x0157, 616013},
  {0x0158, 593409},
  {0x0159, 615981},
  {0x015A, 594072},
  {0x015B, 617278},
  {0x015C, 594272},
  {0x015D, 617468},
  {0x015E, 594236},
  {0x015F, 617434},
  {0x0160, 594154},
  {0x0161, 617356},
  {0x0162, 594972},
  {0x0163, 618713},
  {0x0164, 594938},
  {0x0165, 618681},
  {0x0166, 595328},
  {0x0167, 619293},
  {0x0168, 597110},
  {0x0169, 622501},
  {0x016A, 596917},
  {0x016B, 622235},
  {0x016C, 595997},
  {0x016D, 621323},
  {0x016E, 597036},
  {0x016F, 622389},
  {0x0170, 596462},
  {0x0171, 621766},
  {0x0172, 597001},
  {0x0173, 622315},
  {0x0174, 597666},
  {0x0175, 623257},
  {0x0176, 598068},
  {0x0177, 623878},
  {0x0178, 598107},
  {0x0179, 598513},
  {0x017A, 624376},
  {0x017B, 598658},
  {0x017C, 624544},
  {0x017D, 598547},
  {0x017E, 624408},
  {0x017F, 611895},
  {0x0180, 603433},
  {0x0181, 584205},
  {0x0182, 584312},
  {0x0183, 603466},
  {0x0184, 595517},
  {0x0185, 619719},
  {0x0186, 592854},
  {0x0187, 584685},
  {0x0188, 604112},
  {0x0189, 583838},
  {0x018A, 585169},
  {0x018B, 585422},
  {0x018C, 605177},
  {0x018D, 620025},
  {0x018E, 593910},
  {0x018F, 594717},
  {0x0190, 592826},
  {0x0191, 586935},
  {0x0192, 607770},
  {0x0193, 587241},
  {0x0194, 587387},
  {0x0195, 609062},
  {0x0196, 588862},
  {0x0197, 588567},
  {0x0198, 589297},
  {0x0199, 610513},
  {0x019A, 610823},
  {0x019B, 611740},
  {0x019C, 595799},
  {0x019D, 590789},
  {0x019E, 613023},
  {0x019F, 592344},
  {0x01A0, 591838},
  {0x01A1, 614095},
  {0x01A2, 592717},
  {0x01A3, 614976},
  {0x01A4, 593038},
  {0x01A5, 615432},
  {0x01A6, 601380},
  {0x01A7, 595547},
  {0x01A8, 619747},
  {0x01A9, 586705},
  {0x01AA, 599693},
  {0x01AB, 619213},
  {0x01AC, 595213},
  {0x01AD, 619009},
  {0x01AE, 595285},
  {0x01AF, 596617},
  {0x01B0, 621913},
  {0x01B1, 597228},
  {0x01B2, 597362},
  {0x01B3, 598255},
  {0x01B4, 624055},
  {0x01B5, 598847},
  {0x01B6, 624803},
  {0x01B7, 586779},
  {0x01B8, 586804},
  {0x01B9, 607497},
  {0x01BA, 607680},
  {0x01BB, 601295},
  {0x01BC, 595486},
  {0x01BD, 619690},
  {0x01BE, 599500},
  {0x01BF, 601362},
  {0x01C0, 599330},
  {0x01C1, 599547},
  {0x01C2, 599208},
  {0x01C3, 599615},
  {0x01C4, 585481},
  {0x01C5, 585333},
  {0x01C6, 605850},
  {0x01C7, 590170},
  {0x01C8, 590024},
  {0x01C9, 611873},
  {0x01CA, 591029},
  {0x01CB, 590952},
  {0x01CC, 613303},
  {0x01CD, 582757},
  {0x01CE, 601818},
  {0x01CF, 588105},
  {0x01D0, 609169},
  {0x01D1, 591144},
  {0x01D2, 613433},
  {0x01D3, 596031},
  {0x01D4, 621355},
  {0x01D5, 596331},
  {0x01D6, 621641},
  {0x01D7, 596187},
  {0x01D8, 621503},
  {0x01D9, 596235},
  {0x01DA, 621549},
  {0x01DB, 596283},
  {0x01DC, 621595},
  {0x01DD, 620057},
  {0x01DE, 583122},
  {0x01DF, 602167},
  {0x01E0, 583209},
  {0x01E1, 602250},
  {0x01E2, 583802},
  {0x01E3, 602894},
  {0x01E4, 587352},
  {0x01E5, 608355},
  {0x01E6, 587094},
  {0x01E7, 608072},
  {0x01E8, 589107},
  {0x01E9, 610333},
  {0x01EA, 592385},
  {0x01EB, 614619},
  {0x01EC, 592420},
  {0x01ED, 614652},
  {0x01EE, 586838},
  {0x01EF, 607529},
  {0x01F0, 610139},
  {0x01F1, 585457},
  {0x01F2, 585290},
  {0x01F3, 605708},
  {0x01F4, 587026},
  {0x01F5, 608008},
  {0x01F6, 587987},
  {0x01F7, 597886},
  {0x01F8, 590755},
  {0x01F9, 612918},
  {0x01FA, 583562},
  {0x01FB, 602668},
  {0x01FC, 583767},
  {0x01FD, 602861},
  {0x01FE, 592501},
  {0x01FF, 614770},
  {0x0200, 583296},
  {0x0201, 602333},
  {0x0202, 583410},
  {0x0203, 602441},
  {0x0204, 586175},
  {0x0205, 606510},
  {0x0206, 586289},
  {0x0207, 606653},
  {0x0208, 588340},
  {0x0209, 609356},
  {0x020A, 588454},
  {0x020B, 609464},
  {0x020C, 591724},
  {0x020D, 613987},
  {0x020E, 592095},
  {0x020F, 614340},
  {0x0210, 593604},
  {0x0211, 616205},
  {0x0212, 593645},
  {0x0213, 616383},
  {0x0214, 596503},
  {0x0215, 621805},
  {0x0216, 596874},
  {0x0217, 622158},
  {0x0218, 594311},
  {0x0219, 617505},
  {0x021A, 595053},
  {0x021B, 618790},
  {0x021C, 598464},
  {0x021D, 624331},
  {0x021E, 587604},
  {0x021F, 608591},
  {0x0220, 590866},
  {0x0221, 604730},
  {0x0222, 592882},
  {0x0223, 615286},
  {0x0224, 598734},
  {0x0225, 624616},
  {0x0226, 583171},
  {0x0227, 602214},
  {0x0228, 585641},
  {0x0229, 606000},
  {0x022A, 591509},
  {0x022B, 613782},
  {0x022C, 592672},
  {0x022D, 614933},
  {0x022E, 591558},
  {0x022F, 613829},
  {0x0230, 591596},
  {0x0231, 613865},
  {0x0232, 598360},
  {0x0233, 624154},
  {0x0234, 611041},
  {0x0235, 612779},
  {0x0236, 618828},
  {0x0237, 605428},
  {0x0238, 605210},
  {0x0239, 615869},
  {0x023A, 583650},
  {0x023B, 584759},
  {0x023C, 604223},
  {0x023D, 589559},
  {0x023E, 595093},
  {0x023F, 617982},
  {0x0240, 624836},
  {0x0241, 587476},
  {0x0242, 608471},
  {0x0243, 584277},
  {0x0244, 595936},
  {0x0245, 595859},
  {0x0246, 586492},
  {0x0247, 606919},
  {0x0248, 589015},
  {0x0249, 610247},
  {0x024A, 594871},
  {0x024B, 615782},
  {0x024C, 593770},
  {0x024D, 616662},
  {0x024E, 598395},
  {0x024F, 624266},
  {0x0250, 619936},
  {0x0251, 602928},
  {0x0252, 619993},
  {0x0253, 603287},
  {0x0254, 615149},
  {0x0255, 604045},
  {0x0256, 605146},
  {0x0257, 604833},
  {0x0258, 616860},
  {0x0259, 618076},
  {0x025A, 618101},
  {0x025B, 615077},
  {0x025C, 616987},
  {0x025D, 617022},
  {0x025E, 604466},
  {0x025F, 605457},
  {0x0260, 608211},
  {0x0261, 618181},
  {0x0262, 600102},
  {0x0263, 608388},
  {0x0264, 616762},
  {0x0265, 620113},
  {0x0266, 608838},
  {0x0267, 609028},
  {0x0268, 609612},
  {0x0269, 610006},
  {0x026A, 600199},
  {0x026B, 611512},
  {0x026C, 610853},
  {0x026D, 611590},
  {0x026E, 611805},
  {0x026F, 620354},
  {0x0270, 620382},
  {0x0271, 612299},
  {0x0272, 612950},
  {0x0273, 613230},
  {0x0274, 600462},
  {0x0275, 603559},
  {0x0276, 600520},
  {0x0277, 604401},
  {0x0278, 615665},
  {0x0279, 620661},
  {0x027A, 620727},
  {0x027B, 620689},
  {0x027C, 616461},
  {0x027D, 616695},
  {0x027E, 616244},
  {0x027F, 617122},
  {0x0280, 600672},
  {0x0281, 600228},
  {0x0282, 617738},
  {0x0283, 607202},
  {0x0284, 605498},
  {0x0285, 618593},
  {0x0286, 607225},
  {0x0287, 620960},
  {0x0288, 619252},
  {0x0289, 621220},
  {0x028A, 622701},
  {0x028B, 622897},
  {0x028C, 621055},
  {0x028D, 621083},
  {0x028E, 621111},
  {0x028F, 601177},
  {0x0290, 624762},
  {0x0291, 624477},
  {0x0292, 607474},
  {0x0293, 607563},
  {0x0294, 599356},
  {0x0295, 599574},
  {0x0296, 599420},
  {0x0297, 601235},
  {0x0298, 599269},
  {0x0299, 599859},
  {0x029A, 604433},
  {0x029B, 600131},
  {0x029C, 600170},
  {0x029D, 610208},
  {0x029E, 620298},
  {0x029F, 600324},
  {0x02A0, 615751},
  {0x02A1, 599382},
  {0x02A2, 599724},
  {0x02A3, 605730},
  {0x02A4, 605265},
  {0x02A5, 605760},
  {0x02A6, 619833},
  {0x02A7, 619398},
  {0x02A8, 619358},
  {0x02A9, 607912},
  {0x02AA, 612052},
  {0x02AB, 612105},
  {0x02AC, 599297},
  {0x02AD, 599236},
  {0x02AE, 620141},
  {0x02AF, 620183},
  {0x02B0, 734142},
  {0x02B1, 734166},
  {0x02B2, 734363},
  {0x02B3, 735244},
  {0x02B4, 736278},
  {0x02B5, 736309},
  {0x02B6, 732997},
  {0x02B7, 736812},
  {0x02B8, 736860},
  {0x02B9, 731885},
  {0x02BA, 729952},
  {0x02BB, 737187},
  {0x02BC, 727019},
  {0x02BD, 732143},
  {0x02BE, 732322},
  {0x02BF, 730830},
  {0x02C0, 730478},
  {0x02C1, 732174},
  {0x02C2, 730799},
  {0x02C3, 732290},
  {0x02C4, 737244},
  {0x02C5, 729981},
  {0x02C6, 728288},
  {0x02C7, 174605},
  {0x02C8, 737316},
  {0x02C9, 731510},
  {0x02CA, 726959},
  {0x02CB, 730548},
  {0x02CC, 731435},
  {0x02CD, 731259},
  {0x02CE, 731075},
  {0x02CF, 730887},
  {0x02D0, 737154},
  {0x02D1, 730577},
  {0x02D2, 727946},
  {0x02D3, 727907},
  {0x02D4, 737273},
  {0x02D5, 730012},
  {0x02D6, 731859},
  {0x02D7, 731805},
  {0x02D8, 141774},
  {0x02D9, 284970},
  {0x02DA, 841728},
  {0x02DB, 786253},
  {0x02DC, 893312},
  {0x02DD, 285168},
  {0x02DE, 732262},
  {0x02DF, 728344},
  {0x02E0, 734048},
  {0x02E1, 734453},
  {0x02E2, 735487},
  {0x02E3, 736836},
  {0x02E4, 735405},
  {0x02E5, 730239},
  {0x02E6, 730739},
  {0x02E7, 731654},
  {0x02E8, 731373},
  {0x02E9, 730414},
  {0x02EA, 737411},
  {0x02EB, 737370},
  {0x02EC, 737346},
  {0x02ED, 737216},
  {0x02EE, 729918},
  {0x02EF, 731040},
  {0x02F0, 731402},
  {0x02F1, 731185},
  {0x02F2, 731286},
  {0x02F3, 731322},
  {0x02F4, 731769},
  {0x02F5, 731726},
  {0x02F6, 731683},
  {0x02F7, 731347},
  {0x02F8, 731907},
  {0x02F9, 727046},
  {0x02FA, 730038},
  {0x02FB, 727078},
  {0x02FC, 730068},
  {0x02FD, 732381},
  {0x02FE, 731832},
  {0x02FF, 731154},
  {0x0300, 202906},
  {0x0301, 196604},
  {0x0302, 197152},
  {0x0303, 208103},
  {0x0304, 206442},
  {0x0305, 206982},
  {0x0306, 196967},
  {0x0307, 199947},
  {0x0308, 199876},
  {0x0309, 203276},
  {0x030A, 207738},
  {0x030B, 200123},
  {0x030C, 197096},
  {0x030D, 208452},
  {0x030E, 200737},
  {0x030F, 200239},
  {0x0310, 197074},
  {0x0311, 203331},
  {0x0312, 208307},
  {0x0313, 197279},
  {0x0314, 207237},
  {0x0315, 197301},
  {0x0316, 202929},
  {0x0317, 196627},
  {0x0318, 206005},
  {0x0319, 207662},
  {0x031A, 205602},
  {0x031B, 203297},
  {0x031C, 205803},
  {0x031D, 208360},
  {0x031E, 200888},
  {0x031F, 207146},
  {0x0320, 206673},
  {0x0321, 207001},
  {0x0322, 207172},
  {0x0323, 200018},
  {0x0324, 199896},
  {0x0325, 207759},
  {0x0326, 197329},
  {0x0327, 197134},
  {0x0328, 206754},
  {0x0329, 208482},
  {0x032A, 197051},
  {0x032B, 203456},
  {0x032C, 197112},
  {0x032D, 197180},
  {0x032E, 196983},
  {0x032F, 203356},
  {0x0330, 208119},
  {0x0331, 206459},
  {0x0332, 206423},
  {0x0333, 200339},
  {0x0334, 208173},
  {0x0335, 207859},
  {0x0336, 206356},
  {0x0337, 207827},
  {0x0338, 206325},
  {0x0339, 207520},
  {0x033A, 203387},
  {0x033B, 207950},
  {0x033C, 207803},
  {0x033D, 208630},
  {0x033E, 208512},
  {0x033F, 200452},
  {0x0340, 202958},
  {0x0341, 196656},
  {0x0342, 203191},
  {0x0343, 203067},
  {0x0344, 203035},
  {0x0345, 203219},
  {0x0346, 197028},
  {0x0347, 201158},
  {0x0348, 200774},
  {0x0349, 205629},
  {0x034A, 206700},
  {0x034B, 203249},
  {0x034C, 196733},
  {0x034D, 205972},
  {0x034E, 208384},
  {0x034F, 202880},
  {0x0350, 207324},
  {0x0351, 205772},
  {0x0352, 201186},
  {0x0353, 208648},
  {0x0354, 205741},
  {0x0355, 207456},
  {0x0356, 207407},
  {0x0357, 207488},
  {0x0358, 199992},
  {0x0359, 196920},
  {0x035A, 200619},
  {0x035B, 208686},
  {0x035C, 200176},
  {0x035D, 200153},
  {0x035E, 200365},
  {0x035F, 200389},
  {0x0360, 200647},
  {0x0361, 200269},
  {0x0362, 200579},
  {0x0363, 203861},
  {0x0364, 204235},
  {0x0365, 204477},
  {0x0366, 204880},
  {0x0367, 205300},
  {0x0368, 204134},
  {0x0369, 204204},
  {0x036A, 204446},
  {0x036B, 204818},
  {0x036C, 205052},
  {0x036D, 205225},
  {0x036E, 205441},
  {0x036F, 205540},
  {0x0370, 465965},
  {0x0371, 475042},
  {0x0372, 464354},
  {0x0373, 473253},
  {0x0374, 471442},
  {0x0375, 471322},
  {0x0376, 468225},
  {0x0377, 477667},
  {0x037A, 480468},
  {0x037B, 479019},
  {0x037C, 471862},
  {0x037D, 478971},
  {0x037E, 471739},
  {0x037F, 469022},
  {0x0384, 479335},
  {0x0385, 469393},
  {0x0386, 464239},
  {0x0387, 463052},
  {0x0388, 464813},
  {0x0389, 465866},
  {0x038A, 466498},
  {0x038C, 468145},
  {0x038E, 468877},
  {0x038F, 467723},
  {0x0390, 475407},
  {0x0391, 463189},
  {0x0392, 464389},
  {0x0393, 465938},
  {0x0394, 464440},
  {0x0395, 464467},
  {0x0396, 469047},
  {0x0397, 464893},
  {0x0398, 468502},
  {0x0399, 465991},
  {0x039A, 466610},
  {0x039B, 466637},
  {0x039C, 466664},
  {0x039D, 466688},
  {0x039E, 468998},
  {0x039F, 467799},
  {0x03A0, 468290},
  {0x03A1, 468339},
  {0x03A3, 468450},
  {0x03A4, 468477},
  {0x03A5, 468529},
  {0x03A6, 468265},
  {0x03A7, 464415},
  {0x03A8, 468314},
  {0x03A9, 466712},
  {0x03AA, 466200},
  {0x03AB, 468753},
  {0x03AC, 473013},
  {0x03AD, 473715},
  {0x03AE, 474824},
  {0x03AF, 475791},
  {0x03B0, 478399},
  {0x03B1, 471901},
  {0x03B2, 473286},
  {0x03B3, 475017},
  {0x03B4, 473333},
  {0x03B5, 473385},
  {0x03B6, 478947},
  {0x03B7, 473791},
  {0x03B8, 478009},
  {0x03B9, 475066},
  {0x03BA, 475897},
  {0x03BB, 475947},
  {0x03BC, 475972},
  {0x03BD, 475994},
  {0x03BE, 478925},
  {0x03BF, 477261},
  {0x03C0, 477728},
  {0x03C1, 477773},
  {0x03C2, 474986},
  {0x03C3, 477935},
  {0x03C4, 477986},
  {0x03C5, 478034},
  {0x03C6, 477705},
  {0x03C7, 473310},
  {0x03C8, 477750},
  {0x03C9, 476016},
  {0x03CA, 475265},
  {0x03CB, 478248},
  {0x03CC, 477591},
  {0x03CD, 478810},
  {0x03CE, 477091},
  {0x03CF, 463164},
  {0x03D0, 463105},
  {0x03D1, 479267},
  {0x03D2, 479501},
  {0x03D3, 479415},
  {0x03D4, 479456},
  {0x03D5, 471602},
  {0x03D6, 471619},
  {0x03D7, 470917},
  {0x03D8, 470991},
  {0x03D9, 473220},
  {0x03DA, 471268},
  {0x03DB, 477960},
  {0x03DC, 471018},
  {0x03DD, 473358},
  {0x03DE, 471039},
  {0x03DF, 475922},
  {0x03E0, 471058},
  {0x03E1, 477864},
  {0x03E2, 210916},
  {0x03E3, 213784},
  {0x03E4, 209791},
  {0x03E5, 212731},
  {0x03E6, 210008},
  {0x03E7, 212932},
  {0x03E8, 209926},
  {0x03E9, 212856},
  {0x03EA, 209870},
  {0x03EB, 212804},
  {0x03EC, 210943},
  {0x03ED, 213809},
  {0x03EE, 209593},
  {0x03EF, 212545},
  {0x03F0, 470934},
  {0x03F1, 471796},
  {0x03F2, 471375},
  {0x03F3, 471288},
  {0x03F4, 469200},
  {0x03F5, 471347},
  {0x03F6, 471759},
  {0x03F7, 468425},
  {0x03F8, 477912},
  {0x03F9, 469073},
  {0x03FA, 468400},
  {0x03FB, 477889},
  {0x03FC, 471813},
  {0x03FD, 469157},
  {0x03FE, 463123},
  {0x03FF, 469107},
  {0x0400, 258002},
  {0x0401, 258040},
  {0x0402, 256242},
  {0x0403, 257560},
  {0x0404, 260888},
  {0x0405, 256404},
  {0x0406, 255869},
  {0x0407, 261178},
  {0x0408, 258428},
  {0x0409, 259100},
  {0x040A, 259237},
  {0x040B, 260597},
  {0x040C, 258687},
  {0x040D, 257862},
  {0x040E, 260126},
  {0x040F, 256462},
  {0x0410, 255283},
  {0x0411, 255702},
  {0x0412, 260925},
  {0x0413, 257315},
  {0x0414, 256215},
  {0x0415, 257937},
  {0x0416, 261401},
  {0x0417, 261259},
  {0x0418, 257795},
  {0x0419, 260052},
  {0x041A, 258455},
  {0x041B, 256673},
  {0x041C, 256860},
  {0x041D, 256924},
  {0x041E, 259265},
  {0x041F, 259430},
  {0x0420, 257153},
  {0x0421, 257217},
  {0x0422, 260428},
  {0x0423, 260712},
  {0x0424, 256646},
  {0x0425, 257588},
  {0x0426, 260569},
  {0x0427, 255947},
  {0x0428, 259921},
  {0x0429, 259949},
  {0x042A, 257733},
  {0x042B, 261062},
  {0x042C, 260283},
  {0x042D, 256579},
  {0x042E, 261232},
  {0x042F, 260979},
  {0x0430, 261801},
  {0x0431, 262198},
  {0x0432, 267422},
  {0x0433, 263717},
  {0x0434, 262683},
  {0x0435, 264305},
  {0x0436, 267898},
  {0x0437, 267764},
  {0x0438, 264171},
  {0x0439, 266430},
  {0x043A, 264795},
  {0x043B, 263111},
  {0x043C, 263288},
  {0x043D, 263348},
  {0x043E, 265625},
  {0x043F, 265811},
  {0x0440, 263565},
  {0x0441, 263625},
  {0x0442, 266882},
  {0x0443, 267186},
  {0x0444, 263086},
  {0x0445, 263976},
  {0x0446, 267053},
  {0x0447, 262429},
  {0x0448, 266307},
  {0x0449, 266333},
  {0x044A, 264113},
  {0x044B, 267579},
  {0x044C, 266647},
  {0x044D, 263023},
  {0x044E, 267739},
  {0x044F, 267502},
  {0x0450, 264366},
  {0x0451, 264402},
  {0x0452, 262708},
  {0x0453, 263950},
  {0x0454, 267352},
  {0x0455, 262860},
  {0x0456, 262355},
  {0x0457, 267689},
  {0x0458, 264770},
  {0x0459, 265402},
  {0x045A, 265599},
  {0x045B, 267079},
  {0x045C, 265015},
  {0x045D, 264234},
  {0x045E, 266500},
  {0x045F, 262914},
  {0x0460, 259332},
  {0x0461, 265688},
  {0x0462, 261034},
  {0x0463, 267553},
  {0x0464, 258223},
  {0x0465, 264575},
  {0x0466, 259065},
  {0x0467, 265369},
  {0x0468, 258258},
  {0x0469, 264608},
  {0x046A, 255729},
  {0x046B, 262223},
  {0x046C, 258131},
  {0x046D, 264487},
  {0x046E, 259009},
  {0x046F, 265317},
  {0x0470, 259543},
  {0x0471, 265918},
  {0x0472, 257286},
  {0x0473, 263690},
  {0x0474, 258339},
  {0x0475, 264685},
  {0x0476, 258371},
  {0x0477, 264715},
  {0x0478, 260861},
  {0x0479, 267327},
  {0x047A, 259772},
  {0x047B, 266133},
  {0x047C, 259362},
  {0x047D, 265716},
  {0x047E, 259403},
  {0x047F, 265755},
  {0x0480, 258979},
  {0x0481, 265289},
  {0x0482, 269119},
  {0x0483, 199161},
  {0x0484, 198902},
  {0x0485, 197413},
  {0x0486, 198991},
  {0x0487, 198963},
  {0x0488, 197488},
  {0x0489, 198869},
  {0x048A, 260084},
  {0x048B, 266460},
  {0x048C, 259883},
  {0x048D, 266271},
  {0x048E, 257180},
  {0x048F, 263590},
  {0x0490, 257520},
  {0x0491, 263912},
  {0x0492, 257431},
  {0x0493, 263827},
  {0x0494, 257386},
  {0x0495, 263784},
  {0x0496, 261468},
  {0x0497, 267961},
  {0x0498, 261286},
  {0x0499, 267789},
  {0x049A, 258482},
  {0x049B, 264820},
  {0x049C, 258600},
  {0x049D, 264932},
  {0x049E, 258561},
  {0x049F, 264895},
  {0x04A0, 255667},
  {0x04A1, 262165},
  {0x04A2, 256951},
  {0x04A3, 263373},
  {0x04A4, 261614},
  {0x04A5, 268099},
  {0x04A6, 259499},
  {0x04A7, 265876},
  {0x04A8, 255516},
  {0x04A9, 262022},
  {0x04AA, 257244},
  {0x04AB, 263650},
  {0x04AC, 260455},
  {0x04AD, 266907},
  {0x04AE, 260317},
  {0x04AF, 266679},
  {0x04B0, 260352},
  {0x04B1, 266712},
  {0x04B2, 257615},
  {0x04B3, 264001},
  {0x04B4, 261647},
  {0x04B5, 268130},
  {0x04B6, 255975},
  {0x04B7, 262455},
  {0x04B8, 256061},
  {0x04B9, 262537},
  {0x04BA, 259979},
  {0x04BB, 266361},
  {0x04BC, 255387},
  {0x04BD, 261899},
  {0x04BE, 255425},
  {0x04BF, 261935},
  {0x04C0, 261726},
  {0x04C1, 261429},
  {0x04C2, 267924},
  {0x04C3, 258524},
  {0x04C4, 264860},
  {0x04C5, 256823},
  {0x04C6, 263253},
  {0x04C7, 256993},
  {0x04C8, 263413},
  {0x04C9, 257116},
  {0x04CA, 263530},
  {0x04CB, 258648},
  {0x04CC, 264978},
  {0x04CD, 256887},
  {0x04CE, 263313},
  {0x04CF, 265780},
  {0x04D0, 255309},
  {0x04D1, 261825},
  {0x04D2, 255346},
  {0x04D3, 261860},
  {0x04D4, 261583},
  {0x04D5, 268070},
  {0x04D6, 257964},
  {0x04D7, 264330},
  {0x04D8, 259808},
  {0x04D9, 266200},
  {0x04DA, 259838},
  {0x04DB, 266228},
  {0x04DC, 261511},
  {0x04DD, 268002},
  {0x04DE, 261328},
  {0x04DF, 267829},
  {0x04E0, 255478},
  {0x04E1, 261986},
  {0x04E2, 257899},
  {0x04E3, 264269},
  {0x04E4, 257821},
  {0x04E5, 264195},
  {0x04E6, 259291},
  {0x04E7, 265649},
  {0x04E8, 255586},
  {0x04E9, 262088},
  {0x04EA, 255619},
  {0x04EB, 262119},
  {0x04EC, 256605},
  {0x04ED, 263047},
  {0x04EE, 260823},
  {0x04EF, 267291},
  {0x04F0, 260738},
  {0x04F1, 267210},
  {0x04F2, 260779},
  {0x04F3, 267249},
  {0x04F4, 256018},
  {0x04F5, 262496},
  {0x04F6, 257343},
  {0x04F7, 263743},
  {0x04F8, 261134},
  {0x04F9, 267647},
  {0x04FA, 257471},
  {0x04FB, 263865},
  {0x04FC, 257657},
  {0x04FD, 264041},
  {0x04FE, 257694},
  {0x04FF, 264076},
  {0x0500, 258715},
  {0x0501, 265041},
  {0x0502, 258747},
  {0x0503, 265071},
  {0x0504, 258946},
  {0x0505, 265258},
  {0x0506, 258780},
  {0x0507, 265102},
  {0x0508, 258814},
  {0x0509, 265134},
  {0x050A, 258847},
  {0x050B, 265165},
  {0x050C, 258880},
  {0x050D, 265196},
  {0x050E, 258913},
  {0x050F, 265227},
  {0x0510, 259708},
  {0x0511, 266073},
  {0x0512, 256742},
  {0x0513, 263176},
  {0x0514, 259037},
  {0x0515, 265343},
  {0x0516, 259744},
  {0x0517, 266107},
  {0x0518, 261006},
  {0x0519, 267527},
  {0x051A, 259571},
  {0x051B, 265944},
  {0x051C, 260952},
  {0x051D, 267447},
  {0x051E, 255553},
  {0x051F, 262057},
  {0x0520, 256779},
  {0x0521, 263211},
  {0x0522, 257072},
  {0x0523, 263488},
  {0x0524, 259457},
  {0x0525, 265836},
  {0x0526, 260008},
  {0x0527, 266388},
  {0x0528, 257030},
  {0x0529, 263448},
  {0x052A, 256549},
  {0x052B, 262995},
  {0x052C, 256186},
  {0x052D, 262656},
  {0x052E, 256700},
  {0x052F, 263136},
  {0x0531, 79147},
  {0x0532, 79175},
  {0x0533, 79480},
  {0x0534, 79314},
  {0x0535, 79341},
  {0x0536, 80150},
  {0x0537, 79369},
  {0x0538, 79396},
  {0x0539, 79984},
  {0x053A, 80177},
  {0x053B, 79535},
  {0x053C, 79675},
  {0x053D, 80066},
  {0x053E, 79203},
  {0x053F, 79647},
  {0x0540, 79508},
  {0x0541, 79563},
  {0x0542, 79451},
  {0x0543, 79258},
  {0x0544, 79704},
  {0x0545, 80094},
  {0x0546, 79732},
  {0x0547, 79927},
  {0x0548, 80039},
  {0x0549, 79230},
  {0x054A, 79787},
  {0x054B, 79590},
  {0x054C, 79844},
  {0x054D, 79899},
  {0x054E, 80011},
  {0x054F, 79955},
  {0x0550, 79871},
  {0x0551, 79287},
  {0x0552, 80121},
  {0x0553, 79815},
  {0x0554, 79619},
  {0x0555, 79760},
  {0x0556, 79423},
  {0x0559, 80323},
  {0x055A, 79127},
  {0x055B, 80239},
  {0x055C, 80262},
  {0x055D, 80205},
  {0x055E, 80363},
  {0x055F, 79100},
  {0x0560, 81188},
  {0x0561, 80386},
  {0x0562, 80412},
  {0x0563, 80695},
  {0x0564, 80541},
  {0x0565, 80566},
  {0x0566, 81387},
  {0x0567, 80592},
  {0x0568, 80617},
  {0x0569, 81163},
  {0x056A, 81412},
  {0x056B, 80746},
  {0x056C, 80876},
  {0x056D, 81272},
  {0x056E, 80438},
  {0x056F, 80850},
  {0x0570, 80721},
  {0x0571, 80772},
  {0x0572, 80668},
  {0x0573, 80489},
  {0x0574, 80903},
  {0x0575, 81298},
  {0x0576, 80929},
  {0x0577, 81110},
  {0x0578, 81247},
  {0x0579, 80463},
  {0x057A, 80980},
  {0x057B, 80797},
  {0x057C, 81033},
  {0x057D, 81084},
  {0x057E, 81221},
  {0x057F, 81136},
  {0x0580, 81058},
  {0x0581, 80516},
  {0x0582, 81360},
  {0x0583, 81006},
  {0x0584, 80824},
  {0x0585, 80955},
  {0x0586, 80642},
  {0x0587, 81438},
  {0x0588, 81323},
  {0x0589, 80288},
  {0x058A, 80307},
  {0x058D, 836699},
  {0x058E, 629245},
  {0x058F, 80220},
  {0x0591, 509479},
  {0x0592, 509810},
  {0x0593, 509830},
  {0x0594, 510029},
  {0x0595, 510003},
  {0x0596, 509933},
  {0x0597, 509790},
  {0x0598, 510055},
  {0x0599, 509703},
  {0x059A, 509983},
  {0x059B, 509913},
  {0x059C, 509501},
  {0x059D, 509522},
  {0x059E, 509550},
  {0x059F, 509764},
  {0x05A0, 509855},
  {0x05A1, 509724},
  {0x05A2, 509413},
  {0x05A3, 509665},
  {0x05A4, 509593},
  {0x05A5, 509616},
  {0x05A6, 509637},
  {0x05A7, 509440},
  {0x05A8, 509744},
  {0x05A9, 509884},
  {0x05AA, 509954},
  {0x05AB, 509685},
  {0x05AC, 509574},
  {0x05AD, 509460},
  {0x05AE, 510075},
  {0x05AF, 512115},
  {0x05B0, 512514},
  {0x05B1, 512243},
  {0x05B2, 512192},
  {0x05B3, 512217},
  {0x05B4, 512268},
  {0x05B5, 512576},
  {0x05B6, 512495},
  {0x05B7, 512392},
  {0x05B8, 512411},
  {0x05B9, 512287},
  {0x05BA, 512306},
  {0x05BB, 512457},
  {0x05BC, 512163},
  {0x05BD, 512373},
  {0x05BE, 512650},
  {0x05BF, 512477},
  {0x05C0, 512706},
  {0x05C1, 512533},
  {0x05C2, 512555},
  {0x05C3, 512731},
  {0x05C4, 512141},
  {0x05C5, 512093},
  {0x05C6, 512675},
  {0x05C7, 512431},
  {0x05D0, 510095},
  {0x05D1, 510291},
  {0x05D2, 510611},
  {0x05D3, 510367},
  {0x05D4, 510663},
  {0x05D5, 511525},
  {0x05D6, 511874},
  {0x05D7, 510708},
  {0x05D8, 511425},
  {0x05D9, 511797},
  {0x05DA, 510419},
  {0x05DB, 510726},
  {0x05DC, 510802},
  {0x05DD, 510479},
  {0x05DE, 510854},
  {0x05DF, 510503},
  {0x05E0, 510902},
  {0x05E1, 511121},
  {0x05E2, 510272},
  {0x05E3, 510527},
  {0x05E4, 510950},
  {0x05E5, 510585},
  {0x05E6, 511473},
  {0x05E7, 511023},
  {0x05E8, 511071},
  {0x05E9, 511175},
  {0x05EA, 511377},
  {0x05EF, 512760},
  {0x05F0, 511953},
  {0x05F1, 512023},
  {0x05F2, 511988},
  {0x05F3, 512595},
  {0x05F4, 512621},
  {0x0600, 75125},
  {0x0601, 75826},
  {0x0602, 28739},
  {0x0603, 75748},
  {0x0604, 75807},
  {0x0605, 75100},
  {0x0606, 78739},
  {0x0607, 79002},
  {0x0608, 75383},
  {0x0609, 79027},
  {0x060A, 79055},
  {0x060B, 3712},
  {0x060C, 27977},
  {0x060D, 28246},
  {0x060E, 75289},
  {0x060F, 75669},
  {0x0610, 75766},
  {0x0611, 75641},
  {0x0612, 75717},
  {0x0613, 75687},
  {0x0614, 75909},
  {0x0615, 76612},
  {0x0616, 76165},
  {0x0617, 77092},
  {0x0618, 75973},
  {0x0619, 75931},
  {0x061A, 77115},
  {0x061B, 75556},
  {0x061C, 38870},
  {0x061D, 28501},
  {0x061E, 78516},
  {0x061F, 75338},
  {0x0620, 37625},
  {0x0621, 36049},
  {0x0622, 31178},
  {0x0623, 30869},
  {0x0624, 45376},
  {0x0625, 31002},
  {0x0626, 46101},
  {0x0627, 29700},
  {0x0628, 31698},
  {0x0629, 43570},
  {0x062A, 43460},
  {0x062B, 44114},
  {0x062C, 36919},
  {0x062D, 35418},
  {0x062E, 38104},
  {0x062F, 32727},
  {0x0630, 44032},
  {0x0631, 40805},
  {0x0632, 46806},
  {0x0633, 41930},
  {0x0634, 42475},
  {0x0635, 41672},
  {0x0636, 32469},
  {0x0637, 42742},
  {0x0638, 46666},
  {0x0639, 29334},
  {0x063A, 35027},
  {0x063B, 38013},
  {0x063C, 37912},
  {0x063D, 34211},
  {0x063E, 34297},
  {0x063F, 34251},
  {0x0640, 78180},
  {0x0641, 34341},
  {0x0642, 40508},
  {0x0643, 37264},
  {0x0644, 38449},
  {0x0645, 38889},
  {0x0646, 39428},
  {0x0647, 36103},
  {0x0648, 45106},
  {0x0649, 29782},
  {0x064A, 45572},
  {0x064B, 28668},
  {0x064C, 28200},
  {0x064D, 28961},
  {0x064E, 28525},
  {0x064F, 28113},
  {0x0650, 28868},
  {0x0651, 75573},
  {0x0652, 77488},
  {0x0653, 70375},
  {0x0654, 28808},
  {0x0655, 28827},
  {0x0656, 77466},
  {0x0657, 28846},
  {0x0658, 70395},
  {0x0659, 78723},
  {0x065A, 78667},
  {0x065B, 78626},
  {0x065C, 78598},
  {0x065D, 75394},
  {0x065E, 28641},
  {0x065F, 78699},
  {0x0660, 78978},
  {0x0661, 78859},
  {0x0662, 78955},
  {0x0663, 78930},
  {0x0664, 78811},
  {0x0665, 78787},
  {0x0666, 78907},
  {0x0667, 78882},
  {0x0668, 78762},
  {0x0669, 78835},
  {0x066A, 75220},
  {0x066B, 28268},
  {0x066C, 78337},
  {0x066D, 28714},
  {0x066E, 33405},
  {0x066F, 33457},
  {0x0670, 42687},
  {0x0671, 29888},
  {0x0672, 31471},
  {0x0673, 31512},
  {0x0674, 36806},
  {0x0675, 36831},
  {0x0676, 36861},
  {0x0677, 44687},
  {0x0678, 36890},
  {0x0679, 44282},
  {0x067A, 44459},
  {0x067B, 31553},
  {0x067C, 43734},
  {0x067D, 43832},
  {0x067E, 40147},
  {0x067F, 43882},
  {0x0680, 32319},
  {0x0681, 35620},
  {0x0682, 36002},
  {0x0683, 39926},
  {0x0684, 33562},
  {0x0685, 35867},
  {0x0686, 43082},
  {0x0687, 43300},
  {0x0688, 33323},
  {0x0689, 33005},
  {0x068A, 32806},
  {0x068B, 32839},
  {0x068C, 32642},
  {0x068D, 33235},
  {0x068E, 33483},
  {0x068F, 33033},
  {0x0690, 32886},
  {0x0691, 41590},
  {0x0692, 41228},
  {0x0693, 41100},
  {0x0694, 40884},
  {0x0695, 41259},
  {0x0696, 40917},
  {0x0697, 41326},
  {0x0698, 37185},
  {0x0699, 40964},
  {0x069A, 42075},
  {0x069B, 42322},
  {0x069C, 42363},
  {0x069D, 41892},
  {0x069E, 41812},
  {0x069F, 42915},
  {0x06A0, 29474},
  {0x06A1, 33431},
  {0x06A2, 34568},
  {0x06A3, 34481},
  {0x06A4, 44966},
  {0x06A5, 34607},
  {0x06A6, 40358},
  {0x06A7, 40648},
  {0x06A8, 40765},
  {0x06A9, 37652},
  {0x06AA, 42718},
  {0x06AB, 37470},
  {0x06AC, 37404},
  {0x06AD, 39143},
  {0x06AE, 37498},
  {0x06AF, 34742},
  {0x06B0, 34921},
  {0x06B1, 39278},
  {0x06B2, 34989},
  {0x06B3, 35273},
  {0x06B4, 34949},
  {0x06B5, 38736},
  {0x06B6, 38616},
  {0x06B7, 38767},
  {0x06B8, 38807},
  {0x06B9, 39676},
  {0x06BA, 39477},
  {0x06BB, 41413},
  {0x06BC, 39751},
  {0x06BD, 39846},
  {0x06BE, 36121},
  {0x06BF, 43232},
  {0x06C0, 36682},
  {0x06C1, 36350},
  {0x06C2, 36515},
  {0x06C3, 43633},
  {0x06C4, 45506},
  {0x06C5, 38249},
  {0x06C6, 40071},
  {0x06C7, 44614},
  {0x06C8, 46590},
  {0x06C9, 38349},
  {0x06CA, 45534},
  {0x06CB, 44890},
  {0x06CC, 33837},
  {0x06CD, 46357},
  {0x06CE, 46326},
  {0x06CF, 45185},
  {0x06D0, 33707},
  {0x06D1, 46385},
  {0x06D2, 45590},
  {0x06D3, 45828},
  {0x06D4, 28762},
  {0x06D5, 29190},
  {0x06D6, 76273},
  {0x06D7, 76215},
  {0x06D8, 76355},
  {0x06D9, 76138},
  {0x06DA, 76115},
  {0x06DB, 76634},
  {0x06DC, 76560},
  {0x06DD, 28482},
  {0x06DE, 77438},
  {0x06DF, 76507},
  {0x06E0, 76663},
  {0x06E1, 76014},
  {0x06E2, 76391},
  {0x06E3, 77189},
  {0x06E4, 76331},
  {0x06E5, 77404},
  {0x06E6, 77421},
  {0x06E7, 76999},
  {0x06E8, 76428},
  {0x06E9, 75266},
  {0x06EA, 28453},
  {0x06EB, 28423},
  {0x06EC, 75512},
  {0x06ED, 77134},
  {0x06EE, 32971},
  {0x06EF, 41038},
  {0x06F0, 442298},
  {0x06F1, 442134},
  {0x06F2, 442266},
  {0x06F3, 442232},
  {0x06F4, 442068},
  {0x06F5, 442035},
  {0x06F6, 442200},
  {0x06F7, 442166},
  {0x06F8, 442001},
  {0x06F9, 442101},
  {0x06FA, 42625},
  {0x06FB, 32609},
  {0x06FC, 35177},
  {0x06FD, 75844},
  {0x06FE, 75873},
  {0x06FF, 36648},
  {0x0700, 914364},
  {0x0701, 916060},
  {0x0702, 915971},
  {0x0703, 915998},
  {0x0704, 915912},
  {0x0705, 914586},
  {0x0706, 914235},
  {0x0707, 914260},
  {0x0708, 916023},
  {0x0709, 915935},
  {0x070A, 914286},
  {0x070B, 914499},
  {0x070C, 914473},
  {0x070D, 914446},
  {0x070F, 914195},
  {0x0710, 914610},
  {0x0711, 915527},
  {0x0712, 914630},
  {0x0713, 914748},
  {0x0714, 914768},
  {0x0715, 914649},
  {0x0716, 914670},
  {0x0717, 914797},
  {0x0718, 915624},
  {0x0719, 915683},
  {0x071A, 914814},
  {0x071B, 915577},
  {0x071C, 915596},
  {0x071D, 915642},
  {0x071E, 915661},
  {0x071F, 914833},
  {0x0720, 914852},
  {0x0721, 915181},
  {0x0722, 915199},
  {0x0723, 915405},
  {0x0724, 914720},
  {0x0725, 914704},
  {0x0726, 915217},
  {0x0727, 915340},
  {0x0728, 915385},
  {0x0729, 915321},
  {0x072A, 915366},
  {0x072B, 915427},
  {0x072C, 915559},
  {0x072D, 915234},
  {0x072E, 915292},
  {0x072F, 915262},
  {0x0730, 915767},
  {0x0731, 915787},
  {0x0732, 915807},
  {0x0733, 916199},
  {0x0734, 916219},
  {0x0735, 916239},
  {0x0736, 915845},
  {0x0737, 915864},
  {0x0738, 914333},
  {0x0739, 914305},
  {0x073A, 914522},
  {0x073B, 914541},
  {0x073C, 914560},
  {0x073D, 914388},
  {0x073E, 914407},
  {0x073F, 915899},
  {0x0740, 914426},
  {0x0741, 915828},
  {0x0742, 915883},
  {0x0743, 916137},
  {0x0744, 916168},
  {0x0745, 916089},
  {0x0746, 916113},
  {0x0747, 915715},
  {0x0748, 915741},
  {0x0749, 915702},
  {0x074A, 914220},
  {0x074D, 915499},
  {0x074E, 915471},
  {0x074F, 915446},
  {0x0750, 32081},
  {0x0751, 31838},
  {0x0752, 32134},
  {0x0753, 32191},
  {0x0754, 32267},
  {0x0755, 31927},
  {0x0756, 32013},
  {0x0757, 35964},
  {0x0758, 35907},
  {0x0759, 33172},
  {0x075A, 32925},
  {0x075B, 41296},
  {0x075C, 42186},
  {0x075D, 29613},
  {0x075E, 29554},
  {0x075F, 29651},
  {0x0760, 34704},
  {0x0761, 34647},
  {0x0762, 37802},
  {0x0763, 37870},
  {0x0764, 37954},
  {0x0765, 39034},
  {0x0766, 39068},
  {0x0767, 39887},
  {0x0768, 39780},
  {0x0769, 39814},
  {0x076A, 38589},
  {0x076B, 41364},
  {0x076C, 41003},
  {0x076D, 42425},
  {0x076E, 35814},
  {0x076F, 35754},
  {0x0770, 42261},
  {0x0771, 41128},
  {0x0772, 35701},
  {0x0773, 30807},
  {0x0774, 30743},
  {0x0775, 34144},
  {0x0776, 34075},
  {0x0777, 34007},
  {0x0778, 45315},
  {0x0779, 45252},
  {0x077A, 45760},
  {0x077B, 45690},
  {0x077C, 35558},
  {0x077D, 42123},
  {0x077E, 42226},
  {0x077F, 37538},
  {0x0780, 934601},
  {0x0781, 934904},
  {0x0782, 934782},
  {0x0783, 934845},
  {0x0784, 934409},
  {0x0785, 934720},
  {0x0786, 934661},
  {0x0787, 934389},
  {0x0788, 935048},
  {0x0789, 934744},
  {0x078A, 934516},
  {0x078B, 934495},
  {0x078C, 934972},
  {0x078D, 934700},
  {0x078E, 934536},
  {0x078F, 934577},
  {0x0790, 934884},
  {0x0791, 934472},
  {0x0792, 935124},
  {0x0793, 934949},
  {0x0794, 935088},
  {0x0795, 934802},
  {0x0796, 934638},
  {0x0797, 934427},
  {0x0798, 935029},
  {0x0799, 934619},
  {0x079A, 934681},
  {0x079B, 934991},
  {0x079C, 935106},
  {0x079D, 934928},
  {0x079E, 934863},
  {0x079F, 934451},
  {0x07A0, 935012},
  {0x07A1, 935147},
  {0x07A2, 934370},
  {0x07A3, 934556},
  {0x07A4, 934825},
  {0x07A5, 935068},
  {0x07A6, 934291},
  {0x07A7, 934274},
  {0x07A8, 934355},
  {0x07A9, 934321},
  {0x07AA, 935226},
  {0x07AB, 935196},
  {0x07AC, 934306},
  {0x07AD, 934338},
  {0x07AE, 935181},
  {0x07AF, 935164},
  {0x07B0, 935213},
  {0x07B1, 934764},
  {0x07C0, 770992},
  {0x07C1, 770918},
  {0x07C2, 770978},
  {0x07C3, 770962},
  {0x07C4, 770888},
  {0x07C5, 770873},
  {0x07C6, 770948},
  {0x07C7, 770932},
  {0x07C8, 770857},
  {0x07C9, 770903},
  {0x07CA, 771084},
  {0x07CB, 771175},
  {0x07CC, 771232},
  {0x07CD, 771162},
  {0x07CE, 771542},
  {0x07CF, 771457},
  {0x07D0, 771444},
  {0x07D1, 771140},
  {0x07D2, 771359},
  {0x07D3, 771097},
  {0x07D4, 771471},
  {0x07D5, 771528},
  {0x07D6, 771245},
  {0x07D7, 771111},
  {0x07D8, 771126},
  {0x07D9, 771485},
  {0x07DA, 771499},
  {0x07DB, 771514},
  {0x07DC, 771203},
  {0x07DD, 771189},
  {0x07DE, 771317},
  {0x07DF, 771331},
  {0x07E0, 771386},
  {0x07E1, 771345},
  {0x07E2, 771407},
  {0x07E3, 771372},
  {0x07E4, 771218},
  {0x07E5, 771555},
  {0x07E6, 771569},
  {0x07E7, 771422},
  {0x07E8, 771279},
  {0x07E9, 771259},
  {0x07EA, 771298},
  {0x07EB, 770741},
  {0x07EC, 770771},
  {0x07ED, 770800},
  {0x07EE, 770586},
  {0x07EF, 770621},
  {0x07F0, 770650},
  {0x07F1, 770678},
  {0x07F2, 770709},
  {0x07F3, 770555},
  {0x07F4, 771044},
  {0x07F5, 771583},
  {0x07F6, 771629},
  {0x07F7, 771607},
  {0x07F8, 770832},
  {0x07F9, 771023},
  {0x07FA, 771069},
  {0x07FD, 770842},
  {0x07FE, 771007},
  {0x07FF, 771650},
  {0x0800, 846372},
  {0x0801, 846415},
  {0x0802, 846479},
  {0x0803, 846436},
  {0x0804, 846542},
  {0x0805, 846394},
  {0x0806, 846828},
  {0x0807, 846522},
  {0x0808, 846761},
  {0x0809, 846807},
  {0x080A, 846562},
  {0x080B, 846584},
  {0x080C, 846607},
  {0x080D, 846628},
  {0x080E, 846714},
  {0x080F, 846502},
  {0x0810, 846459},
  {0x0811, 846782},
  {0x0812, 846649},
  {0x0813, 846670},
  {0x0814, 846692},
  {0x0815, 846739},
  {0x0816, 846901},
  {0x0817, 846919},
  {0x0818, 846965},
  {0x0819, 846849},
  {0x081A, 846990},
  {0x081B, 846871},
  {0x081C, 847658},
  {0x081D, 847555},
  {0x081E, 847797},
  {0x081F, 847629},
  {0x0820, 847531},
  {0x0821, 847765},
  {0x0822, 847601},
  {0x0823, 847508},
  {0x0824, 847059},
  {0x0825, 847830},
  {0x0826, 847714},
  {0x0827, 847886},
  {0x0828, 847031},
  {0x0829, 847686},
  {0x082A, 847578},
  {0x082B, 847742},
  {0x082C, 847859},
  {0x082D, 846942},
  {0x0830, 847300},
  {0x0831, 847093},
  {0x0832, 847122},
  {0x0833, 847238},
  {0x0834, 847209},
  {0x0835, 847358},
  {0x0836, 846344},
  {0x0837, 847264},
  {0x0838, 847480},
  {0x0839, 847330},
  {0x083A, 847453},
  {0x083B, 847426},
  {0x083C, 847179},
  {0x083D, 847391},
  {0x083E, 847150},
  {0x0840, 663785},
  {0x0841, 663413},
  {0x0842, 663449},
  {0x0843, 663431},
  {0x0844, 663467},
  {0x0845, 663861},
  {0x0846, 663743},
  {0x0847, 663824},
  {0x0848, 663724},
  {0x0849, 663522},
  {0x084A, 663504},
  {0x084B, 663542},
  {0x084C, 663560},
  {0x084D, 663578},
  {0x084E, 663650},
  {0x084F, 663806},
  {0x0850, 663596},
  {0x0851, 663687},
  {0x0852, 663614},
  {0x0853, 663632},
  {0x0854, 663668},
  {0x0855, 663706},
  {0x0856, 663761},
  {0x0857, 663842},
  {0x0858, 663485},
  {0x0859, 663364},
  {0x085A, 663904},
  {0x085B, 663389},
  {0x085E, 663884},
  {0x0860, 914985},
  {0x0861, 914901},
  {0x0862, 915070},
  {0x0863, 915153},
  {0x0864, 915013},
  {0x0865, 915041},
  {0x0866, 914873},
  {0x0867, 915098},
  {0x0868, 914928},
  {0x0869, 914956},
  {0x086A, 915125},
  {0x0870, 30172},
  {0x0871, 30534},
  {0x0872, 31311},
  {0x0873, 31135},
  {0x0874, 30211},
  {0x0875, 29988},
  {0x0876, 30436},
  {0x0877, 30387},
  {0x0878, 30294},
  {0x0879, 30485},
  {0x087A, 30709},
  {0x087B, 30583},
  {0x087C, 31355},
  {0x087D, 30040},
  {0x087E, 30646},
  {0x087F, 31413},
  {0x0880, 30106},
  {0x0881, 30342},
  {0x0882, 30250},
  {0x0883, 78230},
  {0x0884, 78267},
  {0x0885, 78302},
  {0x0886, 44259},
  {0x0887, 27921},
  {0x0888, 75359},
  {0x0889, 39710},
  {0x088A, 35655},
  {0x088B, 42882},
  {0x088C, 42955},
  {0x088D, 38053},
  {0x088E, 78577},
  {0x0890, 75314},
  {0x0891, 75240},
  {0x0897, 75207},
  {0x0898, 76728},
  {0x0899, 77261},
  {0x089A, 77232},
  {0x089B, 77374},
  {0x089C, 70355},
  {0x089D, 77572},
  {0x089E, 28402},
  {0x089F, 28779},
  {0x08A0, 32044},
  {0x08A1, 31892},
  {0x08A2, 37146},
  {0x08A3, 42995},
  {0x08A4, 34514},
  {0x08A5, 40681},
  {0x08A6, 38649},
  {0x08A7, 39102},
  {0x08A8, 46477},
  {0x08A9, 46425},
  {0x08AA, 41072},
  {0x08AB, 45218},
  {0x08AC, 41563},
  {0x08AD, 38847},
  {0x08AE, 33083},
  {0x08AF, 41852},
  {0x08B0, 34882},
  {0x08B1, 42660},
  {0x08B2, 46888},
  {0x08B3, 29514},
  {0x08B4, 37437},
  {0x08B5, 40714},
  {0x08B6, 31973},
  {0x08B7, 40287},
  {0x08B8, 43762},
  {0x08B9, 41188},
  {0x08BA, 46531},
  {0x08BB, 29207},
  {0x08BC, 29260},
  {0x08BD, 29233},
  {0x08BE, 40327},
  {0x08BF, 43801},
  {0x08C0, 44427},
  {0x08C1, 43267},
  {0x08C2, 37837},
  {0x08C3, 35212},
  {0x08C4, 29286},
  {0x08C5, 37064},
  {0x08C6, 37105},
  {0x08C7, 38683},
  {0x08C8, 35254},
  {0x08C9, 75950},
  {0x08CA, 76053},
  {0x08CB, 77021},
  {0x08CC, 76914},
  {0x08CD, 77070},
  {0x08CE, 29033},
  {0x08CF, 29062},
  {0x08D0, 77501},
  {0x08D1, 29007},
  {0x08D2, 29091},
  {0x08D3, 77211},
  {0x08D4, 76789},
  {0x08D5, 76538},
  {0x08D6, 75992},
  {0x08D7, 76485},
  {0x08D8, 76451},
  {0x08D9, 77156},
  {0x08DA, 76851},
  {0x08DB, 76819},
  {0x08DC, 76758},
  {0x08DD, 76941},
  {0x08DE, 76887},
  {0x08DF, 76970},
  {0x08E0, 76081},
  {0x08E1, 76583},
  {0x08E2, 28293},
  {0x08E3, 78551},
  {0x08E4, 28031},
  {0x08E5, 27990},
  {0x08E6, 28072},
  {0x08E7, 28050},
  {0x08E8, 28009},
  {0x08E9, 28091},
  {0x08EA, 78410},
  {0x08EB, 78462},
  {0x08EC, 78364},
  {0x08ED, 78436},
  {0x08EE, 78489},
  {0x08EF, 78387},
  {0x08F0, 75165},
  {0x08F1, 75144},
  {0x08F2, 75186},
  {0x08F3, 76706},
  {0x08F4, 28618},
  {0x08F5, 28590},
  {0x08F6, 28933},
  {0x08F7, 29134},
  {0x08F8, 75416},
  {0x08F9, 29162},
  {0x08FA, 75483},
  {0x08FB, 28321},
  {0x08FC, 28357},
  {0x08FD, 75445},
  {0x08FE, 28178},
  {0x08FF, 70419},
  {0x0900, 275277},
  {0x0901, 274964},
  {0x0902, 274860},
  {0x0903, 275507},
  {0x0904, 274353},
  {0x0905, 273062},
  {0x0906, 273082},
  {0x0907, 273719},
  {0x0908, 273739},
  {0x0909, 274541},
  {0x090A, 274582},
  {0x090B, 274703},
  {0x090C, 274646},
  {0x090D, 273300},
  {0x090E, 274379},
  {0x090F, 273511},
  {0x0910, 273103},
  {0x0911, 273327},
  {0x0912, 274405},
  {0x0913, 274118},
  {0x0914, 273124},
  {0x0915, 273825},
  {0x0916, 273846},
  {0x0917, 273552},
  {0x0918, 273595},
  {0x0919, 274029},
  {0x091A, 273252},
  {0x091B, 273354},
  {0x091C, 273760},
  {0x091D, 273781},
  {0x091E, 274096},
  {0x091F, 274496},
  {0x0920, 274518},
  {0x0921, 273397},
  {0x0922, 273466},
  {0x0923, 274051},
  {0x0924, 274453},
  {0x0925, 274474},
  {0x0926, 273376},
  {0x0927, 273489},
  {0x0928, 274008},
  {0x0929, 274073},
  {0x092A, 274181},
  {0x092B, 274202},
  {0x092C, 273187},
  {0x092D, 273230},
  {0x092E, 273957},
  {0x092F, 274760},
  {0x0930, 274245},
  {0x0931, 274288},
  {0x0932, 273891},
  {0x0933, 273912},
  {0x0934, 273934},
  {0x0935, 274625},
  {0x0936, 274331},
  {0x0937, 274431},
  {0x0938, 274310},
  {0x0939, 273671},
  {0x093A, 275993},
  {0x093B, 276018},
  {0x093C, 275336},
  {0x093D, 274885},
  {0x093E, 275673},
  {0x093F, 275920},
  {0x0940, 275944},
  {0x0941, 276142},
  {0x0942, 276191},
  {0x0943, 276307},
  {0x0944, 276339},
  {0x0945, 275798},
  {0x0946, 276082},
  {0x0947, 275896},
  {0x0948, 275698},
  {0x0949, 275865},
  {0x094A, 276112},
  {0x094B, 275969},
  {0x094C, 275723},
  {0x094D, 275484},
  {0x094E, 276044},
  {0x094F, 275748},
  {0x0950, 274846},
  {0x0951, 275643},
  {0x0952, 275611},
  {0x0953, 272939},
  {0x0954, 272615},
  {0x0955, 275829},
  {0x0956, 276166},
  {0x0957, 276216},
  {0x0958, 274224},
  {0x0959, 273868},
  {0x095A, 273617},
  {0x095B, 274803},
  {0x095C, 273442},
  {0x095D, 274266},
  {0x095E, 273531},
  {0x095F, 274781},
  {0x0960, 274731},
  {0x0961, 274674},
  {0x0962, 276242},
  {0x0963, 276274},
  {0x0964, 272656},
  {0x0965, 272893},
  {0x0966, 272871},
  {0x0967, 272762},
  {0x0968, 272850},
  {0x0969, 272827},
  {0x096A, 272718},
  {0x096B, 272696},
  {0x096C, 272806},
  {0x096D, 272783},
  {0x096E, 272673},
  {0x096F, 272740},
  {0x0970, 272586},
  {0x0971, 275244},
  {0x0972, 273273},
  {0x0973, 274138},
  {0x0974, 274159},
  {0x0975, 273145},
  {0x0976, 274561},
  {0x0977, 274603},
  {0x0978, 273978},
  {0x0979, 274824},
  {0x097A, 273692},
  {0x097B, 273573},
  {0x097C, 273803},
  {0x097D, 273640},
  {0x097E, 273419},
  {0x097F, 273208},
  {0x0980, 106906},
  {0x0981, 108550},
  {0x0982, 108506},
  {0x0983, 108614},
  {0x0985, 107387},
  {0x0986, 107404},
  {0x0987, 107680},
  {0x0988, 107697},
  {0x0989, 108244},
  {0x098A, 108261},
  {0x098B, 108360},
  {0x098C, 108309},
  {0x098F, 107608},
  {0x0990, 107422},
  {0x0993, 107925},
  {0x0994, 107440},
  {0x0995, 107752},
  {0x0996, 107770},
  {0x0997, 107625},
  {0x0998, 107643},
  {0x0999, 107868},
  {0x099A, 107495},
  {0x099B, 107513},
  {0x099C, 107715},
  {0x099D, 107733},
  {0x099E, 107906},
  {0x099F, 108205},
  {0x09A0, 108224},
  {0x09A1, 107550},
  {0x09A2, 107569},
  {0x09A3, 107887},
  {0x09A4, 108168},
  {0x09A5, 108186},
  {0x09A6, 107532},
  {0x09A7, 107589},
  {0x09A8, 107850},
  {0x09AA, 107942},
  {0x09AB, 107960},
  {0x09AC, 107458},
  {0x09AD, 107476},
  {0x09AE, 107832},
  {0x09AF, 108411},
  {0x09B0, 107979},
  {0x09B2, 107814},
  {0x09B6, 108130},
  {0x09B7, 108149},
  {0x09B8, 108112},
  {0x09B9, 107662},
  {0x09BC, 108575},
  {0x09BD, 108528},
  {0x09BE, 108635},
  {0x09BF, 108722},
  {0x09C0, 108743},
  {0x09C1, 108786},
  {0x09C2, 108807},
  {0x09C3, 108888},
  {0x09C4, 108917},
  {0x09C7, 108701},
  {0x09C8, 108657},
  {0x09CB, 108765},
  {0x09CC, 108679},
  {0x09CD, 108594},
  {0x09CE, 107789},
  {0x09D7, 106919},
  {0x09DC, 108093},
  {0x09DD, 108074},
  {0x09DF, 108429},
  {0x09E0, 108385},
  {0x09E1, 108334},
  {0x09E2, 108829},
  {0x09E3, 108858},
  {0x09E6, 107334},
  {0x09E7, 107240},
  {0x09E8, 107316},
  {0x09E9, 107296},
  {0x09EA, 107202},
  {0x09EB, 107183},
  {0x09EC, 107278},
  {0x09ED, 107258},
  {0x09EE, 107163},
  {0x09EF, 107221},
  {0x09F0, 108035},
  {0x09F1, 107997},
  {0x09F2, 108448},
  {0x09F3, 108467},
  {0x09F4, 107011},
  {0x09F5, 107132},
  {0x09F6, 107099},
  {0x09F7, 106979},
  {0x09F8, 107042},
  {0x09F9, 106942},
  {0x09FA, 107372},
  {0x09FB, 107353},
  {0x09FC, 108279},
  {0x09FD, 106880},
  {0x09FE, 108486},
  {0x0A01, 485542},
  {0x0A02, 485567},
  {0x0A03, 485648},
  {0x0A05, 484609},
  {0x0A06, 484627},
  {0x0A07, 484959},
  {0x0A08, 484977},
  {0x0A09, 485448},
  {0x0A0A, 485466},
  {0x0A0F, 484842},
  {0x0A10, 484646},
  {0x0A13, 485232},
  {0x0A14, 484665},
  {0x0A15, 485035},
  {0x0A16, 485054},
  {0x0A17, 484880},
  {0x0A18, 484899},
  {0x0A19, 485172},
  {0x0A1A, 484723},
  {0x0A1B, 484742},
  {0x0A1C, 484996},
  {0x0A1D, 485015},
  {0x0A1E, 485212},
  {0x0A1F, 485407},
  {0x0A20, 485427},
  {0x0A21, 484781},
  {0x0A22, 484801},
  {0x0A23, 485192},
  {0x0A24, 485368},
  {0x0A25, 485387},
  {0x0A26, 484762},
  {0x0A27, 484822},
  {0x0A28, 485153},
  {0x0A2A, 485251},
  {0x0A2B, 485270},
  {0x0A2C, 484684},
  {0x0A2D, 484703},
  {0x0A2E, 485134},
  {0x0A2F, 485504},
  {0x0A30, 485290},
  {0x0A32, 485095},
  {0x0A33, 485114},
  {0x0A35, 485485},
  {0x0A36, 485348},
  {0x0A38, 485329},
  {0x0A39, 484940},
  {0x0A3C, 485587},
  {0x0A3E, 485719},
  {0x0A3F, 485811},
  {0x0A40, 485833},
  {0x0A41, 485879},
  {0x0A42, 485901},
  {0x0A47, 485788},
  {0x0A48, 485742},
  {0x0A4B, 485856},
  {0x0A4C, 485765},
  {0x0A4D, 485627},
  {0x0A51, 485607},
  {0x0A59, 485074},
  {0x0A5A, 484919},
  {0x0A5B, 485523},
  {0x0A5C, 485309},
  {0x0A5E, 484861},
  {0x0A66, 484558},
  {0x0A67, 484459},
  {0x0A68, 484539},
  {0x0A69, 484518},
  {0x0A6A, 484419},
  {0x0A6B, 484399},
  {0x0A6C, 484499},
  {0x0A6D, 484478},
  {0x0A6E, 484378},
  {0x0A6F, 484439},
  {0x0A70, 485691},
  {0x0A71, 484363},
  {0x0A72, 484596},
  {0x0A73, 485706},
  {0x0A74, 484578},
  {0x0A75, 485670},
  {0x0A76, 484336},
  {0x0A81, 482082},
  {0x0A82, 482036},
  {0x0A83, 482317},
  {0x0A85, 481027},
  {0x0A86, 481045},
  {0x0A87, 481336},
  {0x0A88, 481354},
  {0x0A89, 481803},
  {0x0A8A, 481821},
  {0x0A8B, 481912},
  {0x0A8C, 481859},
  {0x0A8D, 482339},
  {0x0A8F, 481260},
  {0x0A90, 481064},
  {0x0A91, 482363},
  {0x0A93, 481588},
  {0x0A94, 481083},
  {0x0A95, 481412},
  {0x0A96, 481431},
  {0x0A97, 481278},
  {0x0A98, 481297},
  {0x0A99, 481528},
  {0x0A9A, 481141},
  {0x0A9B, 481160},
  {0x0A9C, 481373},
  {0x0A9D, 481392},
  {0x0A9E, 481568},
  {0x0A9F, 481762},
  {0x0AA0, 481782},
  {0x0AA1, 481199},
  {0x0AA2, 481219},
  {0x0AA3, 481548},
  {0x0AA4, 481723},
  {0x0AA5, 481742},
  {0x0AA6, 481180},
  {0x0AA7, 481240},
  {0x0AA8, 481509},
  {0x0AAA, 481606},
  {0x0AAB, 481625},
  {0x0AAC, 481102},
  {0x0AAD, 481121},
  {0x0AAE, 481490},
  {0x0AAF, 481965},
  {0x0AB0, 481645},
  {0x0AB2, 481451},
  {0x0AB3, 481470},
  {0x0AB5, 481840},
  {0x0AB6, 481683},
  {0x0AB7, 481703},
  {0x0AB8, 481664},
  {0x0AB9, 481317},
  {0x0ABC, 482162},
  {0x0ABD, 482059},
  {0x0ABE, 482387},
  {0x0ABF, 482536},
  {0x0AC0, 482558},
  {0x0AC1, 482603},
  {0x0AC2, 482625},
  {0x0AC3, 482709},
  {0x0AC4, 482739},
  {0x0AC5, 482456},
  {0x0AC7, 482514},
  {0x0AC8, 482410},
  {0x0AC9, 482485},
  {0x0ACB, 482581},
  {0x0ACC, 482433},
  {0x0ACD, 482296},
  {0x0AD0, 482004},
  {0x0AE0, 481938},
  {0x0AE1, 481885},
  {0x0AE2, 482648},
  {0x0AE3, 482678},
  {0x0AE6, 481007},
  {0x0AE7, 480908},
  {0x0AE8, 480988},
  {0x0AE9, 480967},
  {0x0AEA, 480868},
  {0x0AEB, 480848},
  {0x0AEC, 480948},
  {0x0AED, 480927},
  {0x0AEE, 480827},
  {0x0AEF, 480888},
  {0x0AF0, 480800},
  {0x0AF1, 482016},
  {0x0AF9, 481984},
  {0x0AFA, 482203},
  {0x0AFB, 482182},
  {0x0AFC, 482141},
  {0x0AFD, 482223},
  {0x0AFE, 482108},
  {0x0AFF, 482259},
  {0x0B01, 802385},
  {0x0B02, 802345},
  {0x0B03, 802463},
  {0x0B05, 801465},
  {0x0B06, 801480},
  {0x0B07, 801726},
  {0x0B08, 801741},
  {0x0B09, 802155},
  {0x0B0A, 802170},
  {0x0B0B, 802249},
  {0x0B0C, 802202},
  {0x0B0F, 801662},
  {0x0B10, 801496},
  {0x0B13, 801939},
  {0x0B14, 801512},
  {0x0B15, 801790},
  {0x0B16, 801806},
  {0x0B17, 801677},
  {0x0B18, 801693},
  {0x0B19, 801888},
  {0x0B1A, 801561},
  {0x0B1B, 801577},
  {0x0B1C, 801757},
  {0x0B1D, 801773},
  {0x0B1E, 801922},
  {0x0B1F, 802120},
  {0x0B20, 802137},
  {0x0B21, 801610},
  {0x0B22, 801627},
  {0x0B23, 801905},
  {0x0B24, 802087},
  {0x0B25, 802103},
  {0x0B26, 801594},
  {0x0B27, 801645},
  {0x0B28, 801872},
  {0x0B2A, 801954},
  {0x0B2B, 801970},
  {0x0B2C, 801528},
  {0x0B2D, 801544},
  {0x0B2E, 801856},
  {0x0B2F, 802312},
  {0x0B30, 801987},
  {0x0B32, 801823},
  {0x0B33, 801839},
  {0x0B35, 802186},
  {0x0B36, 802053},
  {0x0B37, 802070},
  {0x0B38, 802037},
  {0x0B39, 801710},
  {0x0B3C, 802408},
  {0x0B3D, 802365},
  {0x0B3E, 802482},
  {0x0B3F, 802561},
  {0x0B40, 802580},
  {0x0B41, 802619},
  {0x0B42, 802638},
  {0x0B43, 802713},
  {0x0B44, 802740},
  {0x0B47, 802542},
  {0x0B48, 802502},
  {0x0B4B, 802600},
  {0x0B4C, 802522},
  {0x0B4D, 802445},
  {0x0B55, 802425},
  {0x0B56, 801072},
  {0x0B57, 801093},
  {0x0B5C, 802020},
  {0x0B5D, 802003},
  {0x0B5F, 802328},
  {0x0B60, 802272},
  {0x0B61, 802225},
  {0x0B62, 802658},
  {0x0B63, 802685},
  {0x0B66, 801267},
  {0x0B67, 801183},
  {0x0B68, 801251},
  {0x0B69, 801233},
  {0x0B6A, 801149},
  {0x0B6B, 801132},
  {0x0B6C, 801217},
  {0x0B6D, 801199},
  {0x0B6E, 801114},
  {0x0B6F, 801166},
  {0x0B70, 801452},
  {0x0B71, 802296},
  {0x0B72, 801334},
  {0x0B73, 801310},
  {0x0B74, 801390},
  {0x0B75, 801361},
  {0x0B76, 801284},
  {0x0B77, 801420},
  {0x0B82, 927444},
  {0x0B83, 927779},
  {0x0B85, 926677},
  {0x0B86, 926692},
  {0x0B87, 926803},
  {0x0B88, 926818},
  {0x0B89, 927181},
  {0x0B8A, 927196},
  {0x0B8E, 926756},
  {0x0B8F, 926771},
  {0x0B90, 926708},
  {0x0B92, 927018},
  {0x0B93, 927033},
  {0x0B94, 926724},
  {0x0B95, 926850},
  {0x0B99, 926949},
  {0x0B9A, 926740},
  {0x0B9C, 926834},
  {0x0B9E, 927001},
  {0x0B9F, 927164},
  {0x0BA3, 926966},
  {0x0BA4, 927148},
  {0x0BA8, 926933},
  {0x0BA9, 926983},
  {0x0BAA, 927049},
  {0x0BAE, 926917},
  {0x0BAF, 927228},
  {0x0BB0, 927065},
  {0x0BB1, 927081},
  {0x0BB2, 926866},
  {0x0BB3, 926882},
  {0x0BB4, 926899},
  {0x0BB5, 927212},
  {0x0BB6, 927114},
  {0x0BB7, 927131},
  {0x0BB8, 927098},
  {0x0BB9, 926787},
  {0x0BBE, 927917},
  {0x0BBF, 928016},
  {0x0BC0, 928035},
  {0x0BC1, 928094},
  {0x0BC2, 928113},
  {0x0BC6, 927977},
  {0x0BC7, 927996},
  {0x0BC8, 927937},
  {0x0BCA, 928055},
  {0x0BCB, 928074},
  {0x0BCC, 927957},
  {0x0BCD, 927761},
  {0x0BD0, 927347},
  {0x0BD7, 925689},
  {0x0BE6, 925932},
  {0x0BE7, 925848},
  {0x0BE8, 925916},
  {0x0BE9, 925898},
  {0x0BEA, 925814},
  {0x0BEB, 925797},
  {0x0BEC, 925882},
  {0x0BED, 925864},
  {0x0BEE, 925779},
  {0x0BEF, 925831},
  {0x0BF0, 927330},
  {0x0BF1, 927261},
  {0x0BF2, 927286},
  {0x0BF3, 925747},
  {0x0BF4, 927244},
  {0x0BF5, 928160},
  {0x0BF6, 925762},
  {0x0BF7, 925710},
  {0x0BF8, 925669},
  {0x0BF9, 927386},
  {0x0BFA, 927312},
  {0x0C00, 931802},
  {0x0C01, 931741},
  {0x0C02, 931699},
  {0x0C03, 931917},
  {0x0C04, 931765},
  {0x0C05, 930684},
  {0x0C06, 930700},
  {0x0C07, 930996},
  {0x0C08, 931012},
  {0x0C09, 931534},
  {0x0C0A, 931550},
  {0x0C0B, 931633},
  {0x0C0C, 931584},
  {0x0C0E, 930911},
  {0x0C0F, 930927},
  {0x0C10, 930717},
  {0x0C12, 931269},
  {0x0C13, 931285},
  {0x0C14, 930734},
  {0x0C15, 931064},
  {0x0C16, 931081},
  {0x0C17, 930944},
  {0x0C18, 930961},
  {0x0C19, 931215},
  {0x0C1A, 930786},
  {0x0C1B, 930803},
  {0x0C1C, 931029},
  {0x0C1D, 931046},
  {0x0C1E, 931251},
  {0x0C1F, 931497},
  {0x0C20, 931515},
  {0x0C21, 930838},
  {0x0C22, 930856},
  {0x0C23, 931233},
  {0x0C24, 931444},
  {0x0C25, 931461},
  {0x0C26, 930821},
  {0x0C27, 930875},
  {0x0C28, 931170},
  {0x0C2A, 931302},
  {0x0C2B, 931319},
  {0x0C2C, 930751},
  {0x0C2D, 930768},
  {0x0C2E, 931153},
  {0x0C2F, 931682},
  {0x0C30, 931337},
  {0x0C31, 931354},
  {0x0C32, 931099},
  {0x0C33, 931116},
  {0x0C34, 931134},
  {0x0C35, 931567},
  {0x0C36, 931408},
  {0x0C37, 931426},
  {0x0C38, 931391},
  {0x0C39, 930979},
  {0x0C3C, 931842},
  {0x0C3D, 931720},
  {0x0C3E, 931937},
  {0x0C3F, 932041},
  {0x0C40, 932061},
  {0x0C41, 932123},
  {0x0C42, 932143},
  {0x0C43, 932221},
  {0x0C44, 932249},
  {0x0C46, 932000},
  {0x0C47, 932020},
  {0x0C48, 931958},
  {0x0C4A, 932082},
  {0x0C4B, 932102},
  {0x0C4C, 931979},
  {0x0C4D, 931898},
  {0x0C55, 930665},
  {0x0C56, 930112},
  {0x0C58, 931479},
  {0x0C59, 930893},
  {0x0C5A, 931372},
  {0x0C5D, 931187},
  {0x0C60, 931657},
  {0x0C61, 931608},
  {0x0C62, 932164},
  {0x0C63, 932192},
  {0x0C66, 930296},
  {0x0C67, 930207},
  {0x0C68, 930279},
  {0x0C69, 930260},
  {0x0C6A, 930171},
  {0x0C6B, 930153},
  {0x0C6C, 930243},
  {0x0C6D, 930224},
  {0x0C6E, 930134},
  {0x0C6F, 930189},
  {0x0C77, 931860},
  {0x0C78, 930615},
  {0x0C79, 930364},
  {0x0C7A, 930566},
  {0x0C7B, 930465},
  {0x0C7C, 930314},
  {0x0C7D, 930516},
  {0x0C7E, 930413},
  {0x0C7F, 931880},
  {0x0C80, 546278},
  {0x0C81, 546144},
  {0x0C82, 546100},
  {0x0C83, 546356},
  {0x0C84, 546257},
  {0x0C85, 545089},
  {0x0C86, 545106},
  {0x0C87, 545418},
  {0x0C88, 545435},
  {0x0C89, 545927},
  {0x0C8A, 545944},
  {0x0C8B, 546031},
  {0x0C8C, 545980},
  {0x0C8E, 545310},
  {0x0C8F, 545327},
  {0x0C90, 545124},
  {0x0C92, 545686},
  {0x0C93, 545703},
  {0x0C94, 545142},
  {0x0C95, 545490},
  {0x0C96, 545508},
  {0x0C97, 545363},
  {0x0C98, 545381},
  {0x0C99, 545629},
  {0x0C9A, 545197},
  {0x0C9B, 545215},
  {0x0C9C, 545453},
  {0x0C9D, 545471},
  {0x0C9E, 545667},
  {0x0C9F, 545888},
  {0x0CA0, 545907},
  {0x0CA1, 545252},
  {0x0CA2, 545271},
  {0x0CA3, 545648},
  {0x0CA4, 545851},
  {0x0CA5, 545869},
  {0x0CA6, 545234},
  {0x0CA7, 545291},
  {0x0CA8, 545582},
  {0x0CAA, 545721},
  {0x0CAB, 545739},
  {0x0CAC, 545160},
  {0x0CAD, 545178},
  {0x0CAE, 545564},
  {0x0CAF, 546082},
  {0x0CB0, 545758},
  {0x0CB1, 545776},
  {0x0CB2, 545527},
  {0x0CB3, 545545},
  {0x0CB5, 545962},
  {0x0CB6, 545813},
  {0x0CB7, 545832},
  {0x0CB8, 545795},
  {0x0CB9, 545400},
  {0x0CBC, 546238},
  {0x0CBD, 546122},
  {0x0CBE, 546377},
  {0x0CBF, 546486},
  {0x0CC0, 546507},
  {0x0CC1, 546572},
  {0x0CC2, 546593},
  {0x0CC3, 546674},
  {0x0CC4, 546703},
  {0x0CC6, 546443},
  {0x0CC7, 546464},
  {0x0CC8, 546399},
  {0x0CCA, 546529},
  {0x0CCB, 546550},
  {0x0CCC, 546421},
  {0x0CCD, 546336},
  {0x0CD5, 545069},
  {0x0CD6, 544856},
  {0x0CDD, 545600},
  {0x0CDE, 545345},
  {0x0CE0, 546056},
  {0x0CE1, 546005},
  {0x0CE2, 546615},
  {0x0CE3, 546644},
  {0x0CE6, 545050},
  {0x0CE7, 544956},
  {0x0CE8, 545032},
  {0x0CE9, 545012},
  {0x0CEA, 544918},
  {0x0CEB, 544899},
  {0x0CEC, 544994},
  {0x0CED, 544974},
  {0x0CEE, 544879},
  {0x0CEF, 544937},
  {0x0CF1, 546213},
  {0x0CF2, 546311},
  {0x0CF3, 546169},
  {0x0D00, 662584},
  {0x0D01, 662526},
  {0x0D02, 662478},
  {0x0D03, 662701},
  {0x0D04, 662236},
  {0x0D05, 660941},
  {0x0D06, 660960},
  {0x0D07, 661579},
  {0x0D08, 661598},
  {0x0D09, 662177},
  {0x0D0A, 662196},
  {0x0D0B, 662323},
  {0x0D0C, 662268},
  {0x0D0E, 661479},
  {0x0D0F, 661498},
  {0x0D10, 660980},
  {0x0D12, 661888},
  {0x0D13, 661907},
  {0x0D14, 661028},
  {0x0D15, 661659},
  {0x0D16, 661679},
  {0x0D17, 661518},
  {0x0D18, 661538},
  {0x0D19, 661803},
  {0x0D1A, 661089},
  {0x0D1B, 661109},
  {0x0D1C, 661618},
  {0x0D1D, 661638},
  {0x0D1E, 661867},
  {0x0D1F, 662112},
  {0x0D20, 662133},
  {0x0D21, 661389},
  {0x0D22, 661410},
  {0x0D23, 661824},
  {0x0D24, 662071},
  {0x0D25, 662091},
  {0x0D26, 661369},
  {0x0D27, 661432},
  {0x0D28, 661783},
  {0x0D29, 661845},
  {0x0D2A, 661927},
  {0x0D2B, 661947},
  {0x0D2C, 661048},
  {0x0D2D, 661068},
  {0x0D2E, 661763},
  {0x0D2F, 662378},
  {0x0D30, 661968},
  {0x0D31, 661988},
  {0x0D32, 661700},
  {0x0D33, 661720},
  {0x0D34, 661741},
  {0x0D35, 662216},
  {0x0D36, 662029},
  {0x0D37, 662050},
  {0x0D38, 662009},
  {0x0D39, 661559},
  {0x0D3A, 662155},
  {0x0D3B, 662644},
  {0x0D3C, 662553},
  {0x0D3D, 662502},
  {0x0D3E, 662724},
  {0x0D3F, 662843},
  {0x0D40, 662866},
  {0x0D41, 662937},
  {0x0D42, 662960},
  {0x0D43, 663047},
  {0x0D44, 663078},
  {0x0D46, 662796},
  {0x0D47, 662819},
  {0x0D48, 662748},
  {0x0D4A, 662890},
  {0x0D4B, 662913},
  {0x0D4C, 662772},
  {0x0D4D, 662679},
  {0x0D4E, 661453},
  {0x0D4F, 662624},
  {0x0D54, 661237},
  {0x0D55, 661343},
  {0x0D56, 661209},
  {0x0D57, 660251},
  {0x0D58, 660625},
  {0x0D59, 660565},
  {0x0D5A, 660799},
  {0x0D5B, 660766},
  {0x0D5C, 660737},
  {0x0D5D, 660905},
  {0x0D5E, 660536},
  {0x0D5F, 661000},
  {0x0D60, 662350},
  {0x0D61, 662295},
  {0x0D62, 662984},
  {0x0D63, 663015},
  {0x0D66, 660485},
  {0x0D67, 660381},
  {0x0D68, 660465},
  {0x0D69, 660443},
  {0x0D6A, 660339},
  {0x0D6B, 660318},
  {0x0D6C, 660423},
  {0x0D6D, 660401},
  {0x0D6E, 660296},
  {0x0D6F, 660360},
  {0x0D70, 662457},
  {0x0D71, 662398},
  {0x0D72, 662427},
  {0x0D73, 660673},
  {0x0D74, 660597},
  {0x0D75, 660835},
  {0x0D76, 660704},
  {0x0D77, 660506},
  {0x0D78, 660869},
  {0x0D79, 660276},
  {0x0D7A, 661289},
  {0x0D7B, 661263},
  {0x0D7C, 661316},
  {0x0D7D, 661156},
  {0x0D7E, 661182},
  {0x0D7F, 661130},
  {0x0D81, 890747},
  {0x0D82, 890723},
  {0x0D83, 890772},
  {0x0D85, 889179},
  {0x0D86, 888693},
  {0x0D87, 888740},
  {0x0D88, 888716},
  {0x0D89, 889506},
  {0x0D8A, 889385},
  {0x0D8B, 890361},
  {0x0D8C, 890338},
  {0x0D8D, 889482},
  {0x0D8E, 889457},
  {0x0D8F, 889433},
  {0x0D90, 889408},
  {0x0D91, 889317},
  {0x0D92, 889294},
  {0x0D93, 888763},
  {0x0D94, 890059},
  {0x0D95, 890036},
  {0x0D96, 889156},
  {0x0D9A, 888991},
  {0x0D9B, 889773},
  {0x0D9C, 888923},
  {0x0D9D, 889703},
  {0x0D9E, 889528},
  {0x0D9F, 890167},
  {0x0DA0, 888820},
  {0x0DA1, 889597},
  {0x0DA2, 888957},
  {0x0DA3, 889738},
  {0x0DA4, 890229},
  {0x0DA5, 890263},
  {0x0DA6, 890198},
  {0x0DA7, 889093},
  {0x0DA8, 889878},
  {0x0DA9, 888888},
  {0x0DAA, 889667},
  {0x0DAB, 889970},
  {0x0DAC, 890135},
  {0x0DAD, 889059},
  {0x0DAE, 889843},
  {0x0DAF, 888854},
  {0x0DB0, 889632},
  {0x0DB1, 889232},
  {0x0DB3, 890104},
  {0x0DB4, 889025},
  {0x0DB5, 889808},
  {0x0DB6, 888786},
  {0x0DB7, 889562},
  {0x0DB8, 889914},
  {0x0DB9, 889128},
  {0x0DBA, 890406},
  {0x0DBB, 890081},
  {0x0DBD, 889201},
  {0x0DC0, 890383},
  {0x0DC1, 890307},
  {0x0DC2, 890003},
  {0x0DC3, 889263},
  {0x0DC4, 889362},
  {0x0DC5, 889937},
  {0x0DC6, 889339},
  {0x0DCA, 890700},
  {0x0DCF, 890795},
  {0x0DD0, 891095},
  {0x0DD1, 890825},
  {0x0DD2, 891131},
  {0x0DD3, 890933},
  {0x0DD4, 891165},
  {0x0DD6, 890998},
  {0x0DD8, 891032},
  {0x0DD9, 891230},
  {0x0DDA, 890966},
  {0x0DDB, 891200},
  {0x0DDC, 891257},
  {0x0DDD, 891299},
  {0x0DDE, 891346},
  {0x0DDF, 891064},
  {0x0DE6, 890645},
  {0x0DE7, 890526},
  {0x0DE8, 890622},
  {0x0DE9, 890597},
  {0x0DEA, 890478},
  {0x0DEB, 890454},
  {0x0DEC, 890574},
  {0x0DED, 890549},
  {0x0DEE, 890429},
  {0x0DEF, 890502},
  {0x0DF2, 890860},
  {0x0DF3, 890897},
  {0x0DF4, 890669},
  {0x0E01, 935696},
  {0x0E02, 935549},
  {0x0E03, 935597},
  {0x0E04, 935622},
  {0x0E05, 935573},
  {0x0E06, 935647},
  {0x0E07, 936004},
  {0x0E08, 935292},
  {0x0E09, 935341},
  {0x0E0A, 935316},
  {0x0E0B, 936682},
  {0x0E0C, 935366},
  {0x0E0D, 937024},
  {0x0E0E, 935390},
  {0x0E0F, 936909},
  {0x0E10, 936835},
  {0x0E11, 936752},
  {0x0E12, 936782},
  {0x0E13, 936051},
  {0x0E14, 935414},
  {0x0E15, 936933},
  {0x0E16, 936884},
  {0x0E17, 936809},
  {0x0E18, 936859},
  {0x0E19, 936073},
  {0x0E1A, 935267},
  {0x0E1B, 936239},
  {0x0E1C, 936187},
  {0x0E1D, 935436},
  {0x0E1E, 936163},
  {0x0E1F, 935457},
  {0x0E20, 936212},
  {0x0E21, 935983},
  {0x0E22, 937002},
  {0x0E23, 936261},
  {0x0E24, 936283},
  {0x0E25, 935769},
  {0x0E26, 935792},
  {0x0E27, 936955},
  {0x0E28, 936659},
  {0x0E29, 936636},
  {0x0E2A, 936703},
  {0x0E2B, 935502},
  {0x0E2C, 935745},
  {0x0E2D, 936094},
  {0x0E2E, 935524},
  {0x0E2F, 936115},
  {0x0E30, 936301},
  {0x0E31, 935860},
  {0x0E32, 936323},
  {0x0E33, 936432},
  {0x0E34, 936477},
  {0x0E35, 936499},
  {0x0E36, 936566},
  {0x0E37, 936589},
  {0x0E38, 936544},
  {0x0E39, 936613},
  {0x0E3A, 936140},
  {0x0E3F, 937047},
  {0x0E40, 936455},
  {0x0E41, 936346},
  {0x0E42, 936522},
  {0x0E43, 936401},
  {0x0E44, 936369},
  {0x0E45, 935718},
  {0x0E46, 935959},
  {0x0E47, 935934},
  {0x0E48, 935838},
  {0x0E49, 935888},
  {0x0E4A, 935911},
  {0x0E4B, 935810},
  {0x0E4C, 936725},
  {0x0E4D, 936027},
  {0x0E4E, 936978},
  {0x0E4F, 935479},
  {0x0E50, 937217},
  {0x0E51, 937138},
  {0x0E52, 937202},
  {0x0E53, 937185},
  {0x0E54, 937106},
  {0x0E55, 937090},
  {0x0E56, 937170},
  {0x0E57, 937153},
  {0x0E58, 937073},
  {0x0E59, 937122},
  {0x0E5A, 935241},
  {0x0E5B, 935674},
  {0x0E81, 578804},
  {0x0E82, 578765},
  {0x0E84, 578785},
  {0x0E86, 579028},
  {0x0E87, 578870},
  {0x0E88, 578624},
  {0x0E89, 578947},
  {0x0E8A, 579289},
  {0x0E8C, 579048},
  {0x0E8D, 578899},
  {0x0E8E, 579108},
  {0x0E8F, 579128},
  {0x0E90, 579148},
  {0x0E91, 578967},
  {0x0E92, 578987},
  {0x0E93, 579088},
  {0x0E94, 578638},
  {0x0E95, 579346},
  {0x0E96, 579307},
  {0x0E97, 579327},
  {0x0E98, 579008},
  {0x0E99, 578885},
  {0x0E9A, 578610},
  {0x0E9B, 579208},
  {0x0E9C, 579169},
  {0x0E9D, 578671},
  {0x0E9E, 579189},
  {0x0E9F, 578652},
  {0x0EA0, 578927},
  {0x0EA1, 578856},
  {0x0EA2, 579374},
  {0x0EA3, 578818},
  {0x0EA5, 578837},
  {0x0EA7, 579360},
  {0x0EA8, 579222},
  {0x0EA9, 579246},
  {0x0EAA, 579270},
  {0x0EAB, 578689},
  {0x0EAC, 579068},
  {0x0EAD, 578914},
  {0x0EAE, 578708},
  {0x0EAF, 578567},
  {0x0EB0, 579537},
  {0x0EB1, 579696},
  {0x0EB2, 579554},
  {0x0EB3, 579590},
  {0x0EB4, 579661},
  {0x0EB5, 579678},
  {0x0EB6, 579794},
  {0x0EB7, 579811},
  {0x0EB8, 579759},
  {0x0EB9, 579776},
  {0x0EBA, 579447},
  {0x0EBB, 579719},
  {0x0EBC, 579402},
  {0x0EBD, 579424},
  {0x0EC0, 579626},
  {0x0EC1, 579643},
  {0x0EC2, 579742},
  {0x0EC3, 579608},
  {0x0EC4, 579572},
  {0x0EC6, 578600},
  {0x0EC8, 579488},
  {0x0EC9, 579504},
  {0x0ECA, 579521},
  {0x0ECB, 579468},
  {0x0ECC, 578395},
  {0x0ECD, 579388},
  {0x0ECE, 579829},
  {0x0ED0, 578552},
  {0x0ED1, 578478},
  {0x0ED2, 578538},
  {0x0ED3, 578522},
  {0x0ED4, 578448},
  {0x0ED5, 578433},
  {0x0ED6, 578508},
  {0x0ED7, 578492},
  {0x0ED8, 578417},
  {0x0ED9, 578463},
  {0x0EDC, 578590},
  {0x0EDD, 578580},
  {0x0EDE, 578726},
  {0x0EDF, 578745},
  {0x0F00, 942988},
  {0x0F01, 940342},
  {0x0F02, 940299},
  {0x0F03, 940255},
  {0x0F04, 940506},
  {0x0F05, 940159},
  {0x0F06, 940049},
  {0x0F07, 940993},
  {0x0F08, 940870},
  {0x0F09, 939935},
  {0x0F0A, 939842},
  {0x0F0B, 940543},
  {0x0F0C, 940196},
  {0x0F0D, 940894},
  {0x0F0E, 940712},
  {0x0F0F, 940969},
  {0x0F10, 940759},
  {0x0F11, 940836},
  {0x0F12, 940808},
  {0x0F13, 940006},
  {0x0F14, 940231},
  {0x0F15, 939720},
  {0x0F16, 939753},
  {0x0F17, 938215},
  {0x0F18, 938140},
  {0x0F19, 938176},
  {0x0F1A, 941238},
  {0x0F1B, 941266},
  {0x0F1C, 941295},
  {0x0F1D, 941355},
  {0x0F1E, 941382},
  {0x0F1F, 941323},
  {0x0F20, 938821},
  {0x0F21, 938727},
  {0x0F22, 938803},
  {0x0F23, 938783},
  {0x0F24, 938449},
  {0x0F25, 938430},
  {0x0F26, 938765},
  {0x0F27, 938745},
  {0x0F28, 938410},
  {0x0F29, 938708},
  {0x0F2A, 938565},
  {0x0F2B, 938661},
  {0x0F2C, 938636},
  {0x0F2D, 938517},
  {0x0F2E, 938493},
  {0x0F2F, 938613},
  {0x0F30, 938588},
  {0x0F31, 938468},
  {0x0F32, 938541},
  {0x0F33, 938684},
  {0x0F34, 939873},
  {0x0F35, 940645},
  {0x0F36, 939962},
  {0x0F37, 940677},
  {0x0F38, 940089},
  {0x0F39, 940946},
  {0x0F3A, 940408},
  {0x0F3B, 940380},
  {0x0F3C, 939814},
  {0x0F3D, 939786},
  {0x0F3E, 941566},
  {0x0F3F, 941165},
  {0x0F40, 939215},
  {0x0F41, 939233},
  {0x0F42, 939142},
  {0x0F43, 939160},
  {0x0F44, 939345},
  {0x0F45, 938961},
  {0x0F46, 938979},
  {0x0F47, 939197},
  {0x0F49, 939383},
  {0x0F4A, 939608},
  {0x0F4B, 939627},
  {0x0F4C, 939016},
  {0x0F4D, 939035},
  {0x0F4E, 939364},
  {0x0F4F, 939532},
  {0x0F50, 939550},
  {0x0F51, 938998},
  {0x0F52, 939055},
  {0x0F53, 939327},
  {0x0F54, 939402},
  {0x0F55, 939420},
  {0x0F56, 938924},
  {0x0F57, 938942},
  {0x0F58, 939309},
  {0x0F59, 939569},
  {0x0F5A, 939588},
  {0x0F5B, 939074},
  {0x0F5C, 939093},
  {0x0F5D, 939647},
  {0x0F5E, 939701},
  {0x0F5F, 939683},
  {0x0F60, 938889},
  {0x0F61, 939665},
  {0x0F62, 939439},
  {0x0F63, 939291},
  {0x0F64, 939494},
  {0x0F65, 939513},
  {0x0F66, 939476},
  {0x0F67, 939179},
  {0x0F68, 938907},
  {0x0F69, 939271},
  {0x0F6A, 939113},
  {0x0F6B, 939252},
  {0x0F6C, 939457},
  {0x0F71, 943260},
  {0x0F72, 943325},
  {0x0F73, 943346},
  {0x0F74, 943472},
  {0x0F75, 943493},
  {0x0F76, 943574},
  {0x0F77, 943603},
  {0x0F78, 943515},
  {0x0F79, 943544},
  {0x0F7A, 943282},
  {0x0F7B, 943303},
  {0x0F7C, 943368},
  {0x0F7D, 943389},
  {0x0F7E, 941469},
  {0x0F7F, 941497},
  {0x0F80, 943411},
  {0x0F81, 943441},
  {0x0F82, 941210},
  {0x0F83, 941520},
  {0x0F84, 940436},
  {0x0F85, 940788},
  {0x0F86, 941142},
  {0x0F87, 941542},
  {0x0F88, 941117},
  {0x0F89, 941188},
  {0x0F8A, 941028},
  {0x0F8B, 941057},
  {0x0F8C, 941086},
  {0x0F8D, 942921},
  {0x0F8E, 942956},
  {0x0F8F, 942880},
  {0x0F90, 942163},
  {0x0F91, 942191},
  {0x0F92, 942050},
  {0x0F93, 942078},
  {0x0F94, 942334},
  {0x0F95, 941701},
  {0x0F96, 941729},
  {0x0F97, 942135},
  {0x0F99, 942392},
  {0x0F9A, 942708},
  {0x0F9B, 942737},
  {0x0F9C, 941786},
  {0x0F9D, 941815},
  {0x0F9E, 942363},
  {0x0F9F, 942592},
  {0x0FA0, 942620},
  {0x0FA1, 941758},
  {0x0FA2, 941845},
  {0x0FA3, 942306},
  {0x0FA4, 942421},
  {0x0FA5, 942449},
  {0x0FA6, 941644},
  {0x0FA7, 941672},
  {0x0FA8, 942278},
  {0x0FA9, 942649},
  {0x0FAA, 942678},
  {0x0FAB, 941874},
  {0x0FAC, 941903},
  {0x0FAD, 942767},
  {0x0FAE, 942851},
  {0x0FAF, 942823},
  {0x0FB0, 941589},
  {0x0FB1, 942795},
  {0x0FB2, 942478},
  {0x0FB3, 942250},
  {0x0FB4, 942534},
  {0x0FB5, 942563},
  {0x0FB6, 942506},
  {0x0FB7, 942107},
  {0x0FB8, 941617},
  {0x0FB9, 942220},
  {0x0FBA, 941972},
  {0x0FBB, 942011},
  {0x0FBC, 941933},
  {0x0FBE, 938840},
  {0x0FBF, 938858},
  {0x0FC0, 938299},
  {0x0FC1, 938336},
  {0x0FC2, 938263},
  {0x0FC3, 938373},
  {0x0FC4, 943008},
  {0x0FC5, 943204},
  {0x0FC6, 943155},
  {0x0FC7, 943227},
  {0x0FC8, 943181},
  {0x0FC9, 943031},
  {0x0FCA, 943121},
  {0x0FCB, 943087},
  {0x0FCC, 943053},
  {0x0FCE, 941437},
  {0x0FCF, 941410},
  {0x0FD0, 939898},
  {0x0FD1, 940609},
  {0x0FD2, 940735},
  {0x0FD3, 940457},
  {0x0FD4, 940110},
  {0x0FD5, 836909},
  {0x0FD6, 629448},
  {0x0FD7, 836934},
  {0x0FD8, 629472},
  {0x0FD9, 940576},
  {0x0FDA, 940912},
  {0x1000, 753149},
  {0x1001, 753167},
  {0x1002, 752998},
  {0x1003, 753016},
  {0x1004, 753858},
  {0x1005, 752756},
  {0x1006, 752774},
  {0x1007, 753112},
  {0x1008, 753130},
  {0x1009, 753916},
  {0x100A, 753896},
  {0x100B, 755073},
  {0x100C, 755092},
  {0x100D, 752811},
  {0x100E, 752830},
  {0x100F, 753877},
  {0x1010, 754634},
  {0x1011, 755054},
  {0x1012, 752793},
  {0x1013, 752850},
  {0x1014, 753840},
  {0x1015, 753952},
  {0x1016, 753970},
  {0x1017, 752719},
  {0x1018, 752737},
  {0x1019, 753709},
  {0x101A, 755341},
  {0x101B, 753989},
  {0x101C, 753672},
  {0x101D, 755249},
  {0x101E, 754039},
  {0x101F, 753059},
  {0x1020, 753690},
  {0x1021, 752660},
  {0x1022, 754106},
  {0x1023, 753077},
  {0x1024, 753094},
  {0x1025, 755112},
  {0x1026, 755129},
  {0x1027, 752869},
  {0x1028, 753773},
  {0x1029, 753935},
  {0x102A, 752701},
  {0x102B, 757855},
  {0x102C, 757349},
  {0x102D, 757528},
  {0x102E, 757549},
  {0x102F, 757882},
  {0x1030, 757903},
  {0x1031, 757448},
  {0x1032, 757371},
  {0x1033, 757654},
  {0x1034, 757680},
  {0x1035, 757469},
  {0x1036, 755999},
  {0x1037, 756039},
  {0x1038, 756535},
  {0x1039, 756515},
  {0x103A, 756021},
  {0x103B, 751918},
  {0x103C, 751852},
  {0x103D, 751885},
  {0x103E, 751819},
  {0x103F, 753035},
  {0x1040, 752271},
  {0x1041, 752177},
  {0x1042, 752253},
  {0x1043, 752233},
  {0x1044, 752139},
  {0x1045, 752120},
  {0x1046, 752215},
  {0x1047, 752195},
  {0x1048, 752100},
  {0x1049, 752158},
  {0x104A, 756116},
  {0x104B, 756206},
  {0x104C, 756908},
  {0x104D, 756859},
  {0x104E, 756746},
  {0x104F, 756884},
  {0x1050, 754087},
  {0x1051, 754615},
  {0x1052, 755198},
  {0x1053, 755223},
  {0x1054, 755147},
  {0x1055, 755172},
  {0x1056, 757984},
  {0x1057, 758013},
  {0x1058, 757925},
  {0x1059, 757954},
  {0x105A, 753817},
  {0x105B, 753794},
  {0x105C, 753727},
  {0x105D, 753750},
  {0x105E, 752025},
  {0x105F, 751988},
  {0x1060, 751951},
  {0x1061, 754057},
  {0x1062, 757705},
  {0x1063, 757278},
  {0x1064, 757313},
  {0x1065, 755304},
  {0x1066, 755267},
  {0x1067, 758043},
  {0x1068, 758083},
  {0x1069, 756556},
  {0x106A, 756594},
  {0x106B, 756632},
  {0x106C, 756670},
  {0x106D, 756708},
  {0x106E, 752924},
  {0x106F, 752961},
  {0x1070, 752886},
  {0x1071, 757496},
  {0x1072, 757599},
  {0x1073, 757627},
  {0x1074, 757571},
  {0x1075, 754362},
  {0x1076, 754385},
  {0x1077, 754268},
  {0x1078, 754175},
  {0x1079, 754528},
  {0x107A, 754456},
  {0x107B, 754222},
  {0x107C, 754409},
  {0x107D, 754480},
  {0x107E, 754245},
  {0x107F, 754128},
  {0x1080, 754504},
  {0x1081, 754315},
  {0x1082, 752062},
  {0x1083, 757738},
  {0x1084, 757765},
  {0x1085, 757791},
  {0x1086, 757823},
  {0x1087, 756355},
  {0x1088, 756380},
  {0x1089, 756405},
  {0x108A, 756430},
  {0x108B, 756267},
  {0x108C, 756300},
  {0x108D, 756227},
  {0x108E, 754007},
  {0x108F, 756172},
  {0x1090, 755975},
  {0x1091, 755856},
  {0x1092, 755952},
  {0x1093, 755927},
  {0x1094, 755808},
  {0x1095, 755784},
  {0x1096, 755904},
  {0x1097, 755879},
  {0x1098, 755759},
  {0x1099, 755832},
  {0x109A, 756062},
  {0x109B, 756089},
  {0x109C, 757393},
  {0x109D, 757420},
  {0x109E, 756964},
  {0x109F, 756932},
  {0x10A0, 449809},
  {0x10A1, 449836},
  {0x10A2, 450033},
  {0x10A3, 449978},
  {0x10A4, 450006},
  {0x10A5, 450734},
  {0x10A6, 450844},
  {0x10A7, 450651},
  {0x10A8, 450229},
  {0x10A9, 450313},
  {0x10AA, 450370},
  {0x10AB, 450398},
  {0x10AC, 450426},
  {0x10AD, 450454},
  {0x10AE, 450481},
  {0x10AF, 450872},
  {0x10B0, 450566},
  {0x10B1, 450594},
  {0x10B2, 450679},
  {0x10B3, 450707},
  {0x10B4, 450509},
  {0x10B5, 450341},
  {0x10B6, 450061},
  {0x10B7, 450538},
  {0x10B8, 450622},
  {0x10B9, 449921},
  {0x10BA, 449864},
  {0x10BB, 450285},
  {0x10BC, 449950},
  {0x10BD, 449892},
  {0x10BE, 450789},
  {0x10BF, 450256},
  {0x10C0, 450090},
  {0x10C1, 450146},
  {0x10C2, 450173},
  {0x10C3, 450762},
  {0x10C4, 450118},
  {0x10C5, 450201},
  {0x10C7, 450817},
  {0x10CD, 449781},
  {0x10D0, 450941},
  {0x10D1, 450960},
  {0x10D2, 451142},
  {0x10D3, 451062},
  {0x10D4, 451104},
  {0x10D5, 451724},
  {0x10D6, 451802},
  {0x10D7, 451638},
  {0x10D8, 451308},
  {0x10D9, 451368},
  {0x10DA, 451437},
  {0x10DB, 451457},
  {0x10DC, 451477},
  {0x10DD, 451497},
  {0x10DE, 451516},
  {0x10DF, 451822},
  {0x10E0, 451577},
  {0x10E1, 451597},
  {0x10E2, 451658},
  {0x10E3, 451705},
  {0x10E4, 451536},
  {0x10E5, 451388},
  {0x10E6, 451162},
  {0x10E7, 451557},
  {0x10E8, 451617},
  {0x10E9, 451021},
  {0x10EA, 450980},
  {0x10EB, 451348},
  {0x10EC, 451042},
  {0x10ED, 451000},
  {0x10EE, 451763},
  {0x10EF, 451327},
  {0x10F0, 451183},
  {0x10F1, 451249},
  {0x10F2, 451268},
  {0x10F3, 451744},
  {0x10F4, 451203},
  {0x10F5, 451288},
  {0x10F6, 451123},
  {0x10F7, 451783},
  {0x10F8, 451082},
  {0x10F9, 451678},
  {0x10FA, 450921},
  {0x10FB, 453567},
  {0x10FC, 730449},
  {0x10FD, 450901},
  {0x10FE, 451223},
  {0x10FF, 451409},
  {0x1100, 492414},
  {0x1101, 494388},
  {0x1102, 492602},
  {0x1103, 494669},
  {0x1104, 494584},
  {0x1105, 493413},
  {0x1106, 492467},
  {0x1107, 492901},
  {0x1108, 494443},
  {0x1109, 493827},
  {0x110A, 494497},
  {0x110B, 491866},
  {0x110C, 491744},
  {0x110D, 494274},
  {0x110E, 491498},
  {0x110F, 492390},
  {0x1110, 494645},
  {0x1111, 492817},
  {0x1112, 491817},
  {0x1113, 492680},
  {0x1114, 494416},
  {0x1115, 492764},
  {0x1116, 492709},
  {0x1117, 494721},
  {0x1118, 493614},
  {0x1119, 494470},
  {0x111A, 493463},
  {0x111B, 492325},
  {0x111C, 492518},
  {0x111D, 492233},
  {0x111E, 493039},
  {0x111F, 493068},
  {0x1120, 493384},
  {0x1121, 493126},
  {0x1122, 493186},
  {0x1123, 493288},
  {0x1124, 493220},
  {0x1125, 493322},
  {0x1126, 493153},
  {0x1127, 492953},
  {0x1128, 492923},
  {0x1129, 493354},
  {0x112A, 493096},
  {0x112B, 492295},
  {0x112C, 492355},
  {0x112D, 493987},
  {0x112E, 494042},
  {0x112F, 494246},
  {0x1130, 494159},
  {0x1131, 494015},
  {0x1132, 494098},
  {0x1133, 494125},
  {0x1134, 494186},
  {0x1135, 493931},
  {0x1136, 493877},
  {0x1137, 493848},
  {0x1138, 493958},
  {0x1139, 494217},
  {0x113A, 494069},
  {0x113B, 493904},
  {0x113C, 491646},
  {0x113D, 491710},
  {0x113E, 491391},
  {0x113F, 491461},
  {0x1140, 492793},
  {0x1141, 491974},
  {0x1142, 492204},
  {0x1143, 492003},
  {0x1144, 492091},
  {0x1145, 492147},
  {0x1146, 492031},
  {0x1147, 494361},
  {0x1148, 491918},
  {0x1149, 491888},
  {0x114A, 492174},
  {0x114B, 492061},
  {0x114C, 494894},
  {0x114D, 491766},
  {0x114E, 491616},
  {0x114F, 491675},
  {0x1150, 491358},
  {0x1151, 491423},
  {0x1152, 491552},
  {0x1153, 491522},
  {0x1154, 491584},
  {0x1155, 491323},
  {0x1156, 492871},
  {0x1157, 492263},
  {0x1158, 494334},
  {0x1159, 494866},
  {0x115A, 492437},
  {0x115B, 492737},
  {0x115C, 492624},
  {0x115D, 492652},
  {0x115E, 494809},
  {0x115F, 491794},
  {0x1160, 499630},
  {0x1161, 499118},
  {0x1162, 499201},
  {0x1163, 500528},
  {0x1164, 500615},
  {0x1165, 499389},
  {0x1166, 499370},
  {0x1167, 500656},
  {0x1168, 500636},
  {0x1169, 499985},
  {0x116A, 500426},
  {0x116B, 500446},
  {0x116C, 500202},
  {0x116D, 500789},
  {0x116E, 500250},
  {0x116F, 500487},
  {0x1170, 500467},
  {0x1171, 500508},
  {0x1172, 500992},
  {0x1173, 499476},
  {0x1174, 500747},
  {0x1175, 499654},
  {0x1176, 499159},
  {0x1177, 499180},
  {0x1178, 500548},
  {0x1179, 500592},
  {0x117A, 499432},
  {0x117B, 499454},
  {0x117C, 499409},
  {0x117D, 500677},
  {0x117E, 500700},
  {0x117F, 500025},
  {0x1180, 500004},
  {0x1181, 500157},
  {0x1182, 500047},
  {0x1183, 500091},
  {0x1184, 500921},
  {0x1185, 500944},
  {0x1186, 500968},
  {0x1187, 500899},
  {0x1188, 500877},
  {0x1189, 500269},
  {0x118A, 500290},
  {0x118B, 500312},
  {0x118C, 500381},
  {0x118D, 500360},
  {0x118E, 501012},
  {0x118F, 501079},
  {0x1190, 501057},
  {0x1191, 501191},
  {0x1192, 501168},
  {0x1193, 501146},
  {0x1194, 501102},
  {0x1195, 499608},
  {0x1196, 499563},
  {0x1197, 500767},
  {0x1198, 499673},
  {0x1199, 499827},
  {0x119A, 499762},
  {0x119B, 499806},
  {0x119C, 499719},
  {0x119D, 499694},
  {0x119E, 499221},
  {0x119F, 499294},
  {0x11A0, 499345},
  {0x11A1, 499320},
  {0x11A2, 500222},
  {0x11A3, 499137},
  {0x11A4, 500570},
  {0x11A5, 500723},
  {0x11A6, 500112},
  {0x11A7, 500134},
  {0x11A8, 495481},
  {0x11A9, 498299},
  {0x11AA, 495689},
  {0x11AB, 496143},
  {0x11AC, 496197},
  {0x11AD, 496226},
  {0x11AE, 498660},
  {0x11AF, 496979},
  {0x11B0, 497099},
  {0x11B1, 497200},
  {0x11B2, 497425},
  {0x11B3, 497596},
  {0x11B4, 497692},
  {0x11B5, 497394},
  {0x11B6, 497002},
  {0x11B7, 495754},
  {0x11B8, 496648},
  {0x11B9, 496886},
  {0x11BA, 497897},
  {0x11BB, 498476},
  {0x11BC, 495211},
  {0x11BD, 494986},
  {0x11BE, 494961},
  {0x11BF, 495456},
  {0x11C0, 498635},
  {0x11C1, 496529},
  {0x11C2, 495072},
  {0x11C3, 495659},
  {0x11C4, 495718},
  {0x11C5, 496255},
  {0x11C6, 496404},
  {0x11C7, 496345},
  {0x11C8, 496285},
  {0x11C9, 496373},
  {0x11CA, 498746},
  {0x11CB, 498807},
  {0x11CC, 497165},
  {0x11CD, 497334},
  {0x11CE, 497723},
  {0x11CF, 497753},
  {0x11D0, 498412},
  {0x11D1, 497264},
  {0x11D2, 497300},
  {0x11D3, 497526},
  {0x11D4, 497454},
  {0x11D5, 497031},
  {0x11D6, 497659},
  {0x11D7, 497363},
  {0x11D8, 497068},
  {0x11D9, 497789},
  {0x11DA, 495866},
  {0x11DB, 496019},
  {0x11DC, 495956},
  {0x11DD, 496048},
  {0x11DE, 496110},
  {0x11DF, 495925},
  {0x11E0, 495777},
  {0x11E1, 495837},
  {0x11E2, 495330},
  {0x11E3, 496820},
  {0x11E4, 496789},
  {0x11E5, 496731},
  {0x11E6, 495394},
  {0x11E7, 498041},
  {0x11E8, 498214},
  {0x11E9, 498156},
  {0x11EA, 498128},
  {0x11EB, 496434},
  {0x11EC, 495265},
  {0x11ED, 495295},
  {0x11EE, 498271},
  {0x11EF, 495234},
  {0x11F0, 498963},
  {0x11F1, 499087},
  {0x11F2, 499053},
  {0x11F3, 496554},
  {0x11F4, 495361},
  {0x11F5, 495124},
  {0x11F6, 495182},
  {0x11F7, 495095},
  {0x11F8, 495153},
  {0x11F9, 498934},
  {0x11FA, 495599},
  {0x11FB, 495629},
  {0x11FC, 495505},
  {0x11FD, 495567},
  {0x11FE, 495537},
  {0x11FF, 498356},
  {0x1200, 434270},
  {0x1201, 434832},
  {0x1202, 434768},
  {0x1203, 434291},
  {0x1204, 434334},
  {0x1205, 434313},
  {0x1206, 434789},
  {0x1207, 434810},
  {0x1208, 435758},
  {0x1209, 435908},
  {0x120A, 435844},
  {0x120B, 435779},
  {0x120C, 435822},
  {0x120D, 435801},
  {0x120E, 435865},
  {0x120F, 435929},
  {0x1210, 434356},
  {0x1211, 434490},
  {0x1212, 434446},
  {0x1213, 434378},
  {0x1214, 434423},
  {0x1215, 434401},
  {0x1216, 434468},
  {0x1217, 434512},
  {0x1218, 435951},
  {0x1219, 436101},
  {0x121A, 436037},
  {0x121B, 435972},
  {0x121C, 436015},
  {0x121D, 435994},
  {0x121E, 436058},
  {0x121F, 436122},
  {0x1220, 438915},
  {0x1221, 439049},
  {0x1222, 439005},
  {0x1223, 438937},
  {0x1224, 438982},
  {0x1225, 438960},
  {0x1226, 439027},
  {0x1227, 439071},
  {0x1228, 438021},
  {0x1229, 438171},
  {0x122A, 438107},
  {0x122B, 438042},
  {0x122C, 438085},
  {0x122D, 438064},
  {0x122E, 438128},
  {0x122F, 438192},
  {0x1230, 438236},
  {0x1231, 438872},
  {0x1232, 438652},
  {0x1233, 438257},
  {0x1234, 438428},
  {0x1235, 438279},
  {0x1236, 438673},
  {0x1237, 438893},
  {0x1238, 438450},
  {0x1239, 438607},
  {0x123A, 438540},
  {0x123B, 438472},
  {0x123C, 438517},
  {0x123D, 438495},
  {0x123E, 438562},
  {0x123F, 438629},
  {0x1240, 437309},
  {0x1241, 437732},
  {0x1242, 437668},
  {0x1243, 437330},
  {0x1244, 437373},
  {0x1245, 437352},
  {0x1246, 437689},
  {0x1247, 437710},
  {0x1248, 437753},
  {0x124A, 437843},
  {0x124B, 437775},
  {0x124C, 437820},
  {0x124D, 437798},
  {0x1250, 437395},
  {0x1251, 437529},
  {0x1252, 437485},
  {0x1253, 437417},
  {0x1254, 437462},
  {0x1255, 437440},
  {0x1256, 437507},
  {0x1258, 437551},
  {0x125A, 437645},
  {0x125B, 437574},
  {0x125C, 437621},
  {0x125D, 437598},
  {0x1260, 430543},
  {0x1261, 430849},
  {0x1262, 430785},
  {0x1263, 430564},
  {0x1264, 430763},
  {0x1265, 430742},
  {0x1266, 430806},
  {0x1267, 430870},
  {0x1268, 439987},
  {0x1269, 440115},
  {0x126A, 440073},
  {0x126B, 440008},
  {0x126C, 440051},
  {0x126D, 440030},
  {0x126E, 440094},
  {0x126F, 440136},
  {0x1270, 439094},
  {0x1271, 439765},
  {0x1272, 439382},
  {0x1273, 439115},
  {0x1274, 439158},
  {0x1275, 439137},
  {0x1276, 439403},
  {0x1277, 439786},
  {0x1278, 430959},
  {0x1279, 431800},
  {0x127A, 431736},
  {0x127B, 430980},
  {0x127C, 431512},
  {0x127D, 431491},
  {0x127E, 431757},
  {0x127F, 431821},
  {0x1280, 440329},
  {0x1281, 440479},
  {0x1282, 440415},
  {0x1283, 440350},
  {0x1284, 440393},
  {0x1285, 440372},
  {0x1286, 440436},
  {0x1287, 440457},
  {0x1288, 440500},
  {0x128A, 440590},
  {0x128B, 440522},
  {0x128C, 440567},
  {0x128D, 440545},
  {0x1290, 436233},
  {0x1291, 436383},
  {0x1292, 436319},
  {0x1293, 436254},
  {0x1294, 436297},
  {0x1295, 436276},
  {0x1296, 436340},
  {0x1297, 436404},
  {0x1298, 436426},
  {0x1299, 436583},
  {0x129A, 436516},
  {0x129B, 436448},
  {0x129C, 436493},
  {0x129D, 436471},
  {0x129E, 436538},
  {0x129F, 436605},
  {0x12A0, 433152},
  {0x12A1, 433351},
  {0x12A2, 433266},
  {0x12A3, 433180},
  {0x12A4, 433237},
  {0x12A5, 433209},
  {0x12A6, 433294},
  {0x12A7, 433379},
  {0x12A8, 435046},
  {0x12A9, 435196},
  {0x12AA, 435132},
  {0x12AB, 435067},
  {0x12AC, 435110},
  {0x12AD, 435089},
  {0x12AE, 435153},
  {0x12AF, 435174},
  {0x12B0, 435217},
  {0x12B2, 435307},
  {0x12B3, 435239},
  {0x12B4, 435284},
  {0x12B5, 435262},
  {0x12B8, 435329},
  {0x12B9, 435463},
  {0x12BA, 435419},
  {0x12BB, 435351},
  {0x12BC, 435396},
  {0x12BD, 435374},
  {0x12BE, 435441},
  {0x12C0, 435485},
  {0x12C2, 435579},
  {0x12C3, 435508},
  {0x12C4, 435555},
  {0x12C5, 435532},
  {0x12C8, 440158},
  {0x12C9, 440308},
  {0x12CA, 440244},
  {0x12CB, 440179},
  {0x12CC, 440222},
  {0x12CD, 440201},
  {0x12CE, 440265},
  {0x12CF, 440286},
  {0x12D0, 436759},
  {0x12D1, 436947},
  {0x12D2, 436885},
  {0x12D3, 436790},
  {0x12D4, 436853},
  {0x12D5, 436822},
  {0x12D6, 436916},
  {0x12D8, 440939},
  {0x12D9, 441268},
  {0x12DA, 441204},
  {0x12DB, 440960},
  {0x12DC, 441003},
  {0x12DD, 440982},
  {0x12DE, 441225},
  {0x12DF, 441289},
  {0x12E0, 441025},
  {0x12E1, 441159},
  {0x12E2, 441115},
  {0x12E3, 441047},
  {0x12E4, 441092},
  {0x12E5, 441070},
  {0x12E6, 441137},
  {0x12E7, 441181},
  {0x12E8, 440768},
  {0x12E9, 440918},
  {0x12EA, 440854},
  {0x12EB, 440789},
  {0x12EC, 440832},
  {0x12ED, 440811},
  {0x12EE, 440875},
  {0x12EF, 440896},
  {0x12F0, 431843},
  {0x12F1, 432335},
  {0x12F2, 432271},
  {0x12F3, 431864},
  {0x12F4, 432249},
  {0x12F5, 432228},
  {0x12F6, 432292},
  {0x12F7, 432356},
  {0x12F8, 431886},
  {0x12F9, 432183},
  {0x12FA, 432116},
  {0x12FB, 431908},
  {0x12FC, 431953},
  {0x12FD, 431931},
  {0x12FE, 432138},
  {0x12FF, 432205},
  {0x1300, 434853},
  {0x1301, 435003},
  {0x1302, 434939},
  {0x1303, 434874},
  {0x1304, 434917},
  {0x1305, 434896},
  {0x1306, 434960},
  {0x1307, 435024},
  {0x1308, 432772},
  {0x1309, 433451},
  {0x130A, 433131},
  {0x130B, 432793},
  {0x130C, 432836},
  {0x130D, 432815},
  {0x130E, 433408},
  {0x130F, 433429},
  {0x1310, 434002},
  {0x1312, 434092},
  {0x1313, 434024},
  {0x1314, 434069},
  {0x1315, 434047},
  {0x1318, 432858},
  {0x1319, 432992},
  {0x131A, 432948},
  {0x131B, 432880},
  {0x131C, 432925},
  {0x131D, 432903},
  {0x131E, 432970},
  {0x131F, 433037},
  {0x1320, 439180},
  {0x1321, 439337},
  {0x1322, 439270},
  {0x1323, 439202},
  {0x1324, 439247},
  {0x1325, 439225},
  {0x1326, 439292},
  {0x1327, 439359},
  {0x1328, 431534},
  {0x1329, 431691},
  {0x132A, 431624},
  {0x132B, 431556},
  {0x132C, 431601},
  {0x132D, 431579},
  {0x132E, 431646},
  {0x132F, 431713},
  {0x1330, 436714},
  {0x1331, 437090},
  {0x1332, 437023},
  {0x1333, 436736},
  {0x1334, 437000},
  {0x1335, 436978},
  {0x1336, 437045},
  {0x1337, 437112},
  {0x1338, 439446},
  {0x1339, 439580},
  {0x133A, 439536},
  {0x133B, 439468},
  {0x133C, 439513},
  {0x133D, 439491},
  {0x133E, 439558},
  {0x133F, 439602},
  {0x1340, 439808},
  {0x1341, 439965},
  {0x1342, 439898},
  {0x1343, 439830},
  {0x1344, 439875},
  {0x1345, 439853},
  {0x1346, 439920},
  {0x1347, 439942},
  {0x1348, 432512},
  {0x1349, 432640},
  {0x134A, 432598},
  {0x134B, 432533},
  {0x134C, 432576},
  {0x134D, 432555},
  {0x134E, 432619},
  {0x134F, 432661},
  {0x1350, 436628},
  {0x1351, 437199},
  {0x1352, 437135},
  {0x1353, 436649},
  {0x1354, 436692},
  {0x1355, 436671},
  {0x1356, 437156},
  {0x1357, 437220},
  {0x1358, 438214},
  {0x1359, 436211},
  {0x135A, 432750},
  {0x135D, 429834},
  {0x135E, 429921},
  {0x135F, 429886},
  {0x1360, 430502},
  {0x1361, 441741},
  {0x1362, 430153},
  {0x1363, 429958},
  {0x1364, 430524},
  {0x1365, 429819},
  {0x1366, 430456},
  {0x1367, 430479},
  {0x1368, 430427},
  {0x1369, 430054},
  {0x136A, 430134},
  {0x136B, 430113},
  {0x136C, 430014},
  {0x136D, 429994},
  {0x136E, 430094},
  {0x136F, 430073},
  {0x1370, 429973},
  {0x1371, 430034},
  {0x1372, 430332},
  {0x1373, 430404},
  {0x1374, 430381},
  {0x1375, 430217},
  {0x1376, 430195},
  {0x1377, 430310},
  {0x1378, 430286},
  {0x1379, 430172},
  {0x137A, 430263},
  {0x137B, 430239},
  {0x137C, 430352},
  {0x1380, 438364},
  {0x1381, 436189},
  {0x1382, 436166},
  {0x1383, 436144},
  {0x1384, 438300},
  {0x1385, 430937},
  {0x1386, 430914},
  {0x1387, 430892},
  {0x1388, 438332},
  {0x1389, 432728},
  {0x138A, 432705},
  {0x138B, 432683},
  {0x138C, 438396},
  {0x138D, 437287},
  {0x138E, 437264},
  {0x138F, 437242},
  {0x1390, 441715},
  {0x1391, 441494},
  {0x1392, 441655},
  {0x1393, 441682},
  {0x1394, 441552},
  {0x1395, 441604},
  {0x1396, 441467},
  {0x1397, 441578},
  {0x1398, 441520},
  {0x1399, 441630},
  {0x13A0, 179487},
  {0x13A1, 179639},
  {0x13A2, 179905},
  {0x13A3, 180304},
  {0x13A4, 180871},
  {0x13A5, 180889},
  {0x13A6, 179657},
  {0x13A7, 179923},
  {0x13A8, 179676},
  {0x13A9, 179695},
  {0x13AA, 179714},
  {0x13AB, 179733},
  {0x13AC, 179752},
  {0x13AD, 179771},
  {0x13AE, 179790},
  {0x13AF, 179809},
  {0x13B0, 179848},
  {0x13B1, 179867},
  {0x13B2, 179886},
  {0x13B3, 179942},
  {0x13B4, 179961},
  {0x13B5, 179980},
  {0x13B6, 179999},
  {0x13B7, 180018},
  {0x13B8, 180037},
  {0x13B9, 180056},
  {0x13BA, 180075},
  {0x13BB, 180094},
  {0x13BC, 180113},
  {0x13BD, 180132},
  {0x13BE, 180170},
  {0x13BF, 179828},
  {0x13C0, 180189},
  {0x13C1, 180209},
  {0x13C2, 180228},
  {0x13C3, 180247},
  {0x13C4, 180266},
  {0x13C5, 180285},
  {0x13C6, 180322},
  {0x13C7, 180342},
  {0x13C8, 180362},
  {0x13C9, 180382},
  {0x13CA, 180402},
  {0x13CB, 180422},
  {0x13CC, 180460},
  {0x13CD, 180442},
  {0x13CE, 180479},
  {0x13CF, 180498},
  {0x13D0, 180517},
  {0x13D1, 180536},
  {0x13D2, 180555},
  {0x13D3, 179505},
  {0x13D4, 180574},
  {0x13D5, 179524},
  {0x13D6, 180593},
  {0x13D7, 179543},
  {0x13D8, 180612},
  {0x13D9, 179582},
  {0x13DA, 179601},
  {0x13DB, 179620},
  {0x13DC, 179562},
  {0x13DD, 180631},
  {0x13DE, 180651},
  {0x13DF, 180671},
  {0x13E0, 180691},
  {0x13E1, 180711},
  {0x13E2, 180731},
  {0x13E3, 180751},
  {0x13E4, 180771},
  {0x13E5, 180791},
  {0x13E6, 180811},
  {0x13E7, 180831},
  {0x13E8, 180851},
  {0x13E9, 180907},
  {0x13EA, 180926},
  {0x13EB, 180945},
  {0x13EC, 180964},
  {0x13ED, 180983},
  {0x13EE, 181002},
  {0x13EF, 181021},
  {0x13F0, 181040},
  {0x13F1, 181059},
  {0x13F2, 181078},
  {0x13F3, 181097},
  {0x13F4, 181116},
  {0x13F5, 180151},
  {0x13F8, 183174},
  {0x13F9, 183199},
  {0x13FA, 183224},
  {0x13FB, 183249},
  {0x13FC, 183274},
  {0x13FD, 182009},
  {0x1400, 160991},
  {0x1401, 160081},
  {0x1402, 153228},
  {0x1403, 161017},
  {0x1404, 161038},
  {0x1405, 164613},
  {0x1406, 165015},
  {0x1407, 172996},
  {0x1408, 154885},
  {0x1409, 155653},
  {0x140A, 153185},
  {0x140B, 153206},
  {0x140C, 169239},
  {0x140D, 171822},
  {0x140E, 172278},
  {0x140F, 171854},
  {0x1410, 172300},
  {0x1411, 171886},
  {0x1412, 172323},
  {0x1413, 171919},
  {0x1414, 172345},
  {0x1415, 171951},
  {0x1416, 163163},
  {0x1417, 169171},
  {0x1418, 171757},
  {0x1419, 169193},
  {0x141A, 171789},
  {0x141B, 163132},
  {0x141C, 153274},
  {0x141D, 173146},
  {0x141E, 160937},
  {0x141F, 160310},
  {0x1420, 160511},
  {0x1421, 160341},
  {0x1422, 160800},
  {0x1423, 160644},
  {0x1424, 160685},
  {0x1425, 160383},
  {0x1426, 160421},
  {0x1427, 160542},
  {0x1428, 160715},
  {0x1429, 160578},
  {0x142A, 160476},
  {0x142B, 160131},
  {0x142C, 161060},
  {0x142D, 164993},
  {0x142E, 153325},
  {0x142F, 165172},
  {0x1430, 165125},
  {0x1431, 165194},
  {0x1432, 165216},
  {0x1433, 165239},
  {0x1434, 165261},
  {0x1435, 173025},
  {0x1436, 155405},
  {0x1437, 155436},
  {0x1438, 165080},
  {0x1439, 165102},
  {0x143A, 165354},
  {0x143B, 170653},
  {0x143C, 165377},
  {0x143D, 170686},
  {0x143E, 165400},
  {0x143F, 170719},
  {0x1440, 165424},
  {0x1441, 170753},
  {0x1442, 165447},
  {0x1443, 170786},
  {0x1444, 165307},
  {0x1445, 170586},
  {0x1446, 165330},
  {0x1447, 170619},
  {0x1448, 173055},
  {0x1449, 165059},
  {0x144A, 170555},
  {0x144B, 155376},
  {0x144C, 167862},
  {0x144D, 167815},
  {0x144E, 168393},
  {0x144F, 168415},
  {0x1450, 168584},
  {0x1451, 168606},
  {0x1452, 173116},
  {0x1453, 154378},
  {0x1454, 154481},
  {0x1455, 167770},
  {0x1456, 167792},
  {0x1457, 168962},
  {0x1458, 171590},
  {0x1459, 168985},
  {0x145A, 171623},
  {0x145B, 169008},
  {0x145C, 171656},
  {0x145D, 169032},
  {0x145E, 171690},
  {0x145F, 169055},
  {0x1460, 171723},
  {0x1461, 168915},
  {0x1462, 171523},
  {0x1463, 168938},
  {0x1464, 171556},
  {0x1465, 163100},
  {0x1466, 167749},
  {0x1467, 168652},
  {0x1468, 168869},
  {0x1469, 168892},
  {0x146A, 168629},
  {0x146B, 161195},
  {0x146C, 161148},
  {0x146D, 161217},
  {0x146E, 161239},
  {0x146F, 161262},
  {0x1470, 161284},
  {0x1471, 172876},
  {0x1472, 161103},
  {0x1473, 161125},
  {0x1474, 161400},
  {0x1475, 169596},
  {0x1476, 161423},
  {0x1477, 169629},
  {0x1478, 161446},
  {0x1479, 169662},
  {0x147A, 161470},
  {0x147B, 169696},
  {0x147C, 161493},
  {0x147D, 169729},
  {0x147E, 161329},
  {0x147F, 169529},
  {0x1480, 161352},
  {0x1481, 169562},
  {0x1482, 162782},
  {0x1483, 161082},
  {0x1484, 161307},
  {0x1485, 167340},
  {0x1486, 167376},
  {0x1487, 167412},
  {0x1488, 167304},
  {0x1489, 159777},
  {0x148A, 154167},
  {0x148B, 159827},
  {0x148C, 159849},
  {0x148D, 159872},
  {0x148E, 159894},
  {0x148F, 172846},
  {0x1490, 154122},
  {0x1491, 154144},
  {0x1492, 159964},
  {0x1493, 169328},
  {0x1494, 159987},
  {0x1495, 169361},
  {0x1496, 160010},
  {0x1497, 169394},
  {0x1498, 160034},
  {0x1499, 169428},
  {0x149A, 160057},
  {0x149B, 169461},
  {0x149C, 159917},
  {0x149D, 169261},
  {0x149E, 159940},
  {0x149F, 169294},
  {0x14A0, 162750},
  {0x14A1, 154101},
  {0x14A2, 166704},
  {0x14A3, 162182},
  {0x14A4, 162135},
  {0x14A5, 162282},
  {0x14A6, 162304},
  {0x14A7, 162327},
  {0x14A8, 162349},
  {0x14A9, 172936},
  {0x14AA, 162090},
  {0x14AB, 162112},
  {0x14AC, 162452},
  {0x14AD, 170288},
  {0x14AE, 162475},
  {0x14AF, 170321},
  {0x14B0, 162498},
  {0x14B1, 170354},
  {0x14B2, 162522},
  {0x14B3, 170388},
  {0x14B4, 162545},
  {0x14B5, 170421},
  {0x14B6, 162405},
  {0x14B7, 170221},
  {0x14B8, 162428},
  {0x14B9, 170254},
  {0x14BA, 162814},
  {0x14BB, 162069},
  {0x14BC, 170190},
  {0x14BD, 162260},
  {0x14BE, 153347},
  {0x14BF, 166494},
  {0x14C0, 163639},
  {0x14C1, 162726},
  {0x14C2, 163895},
  {0x14C3, 163917},
  {0x14C4, 164110},
  {0x14C5, 164132},
  {0x14C6, 172966},
  {0x14C7, 162681},
  {0x14C8, 162703},
  {0x14C9, 164496},
  {0x14CA, 170522},
  {0x14CB, 164449},
  {0x14CC, 170455},
  {0x14CD, 164472},
  {0x14CE, 170488},
  {0x14CF, 162846},
  {0x14D0, 162569},
  {0x14D1, 157162},
  {0x14D2, 163873},
  {0x14D3, 161630},
  {0x14D4, 161583},
  {0x14D5, 161815},
  {0x14D6, 161837},
  {0x14D7, 161860},
  {0x14D8, 161882},
  {0x14D9, 172906},
  {0x14DA, 161538},
  {0x14DB, 161560},
  {0x14DC, 161952},
  {0x14DD, 170023},
  {0x14DE, 161975},
  {0x14DF, 170056},
  {0x14E0, 161998},
  {0x14E1, 170089},
  {0x14E2, 162022},
  {0x14E3, 170123},
  {0x14E4, 162045},
  {0x14E5, 170156},
  {0x14E6, 161905},
  {0x14E7, 169956},
  {0x14E8, 161928},
  {0x14E9, 169989},
  {0x14EA, 161517},
  {0x14EB, 169763},
  {0x14EC, 162204},
  {0x14ED, 166762},
  {0x14EE, 166213},
  {0x14EF, 167214},
  {0x14F0, 167236},
  {0x14F1, 167259},
  {0x14F2, 167281},
  {0x14F3, 173086},
  {0x14F4, 166168},
  {0x14F5, 166190},
  {0x14F6, 167632},
  {0x14F7, 171321},
  {0x14F8, 167655},
  {0x14F9, 171354},
  {0x14FA, 167678},
  {0x14FB, 171387},
  {0x14FC, 167702},
  {0x14FD, 171421},
  {0x14FE, 167725},
  {0x14FF, 171454},
  {0x1500, 167585},
  {0x1501, 171254},
  {0x1502, 167608},
  {0x1503, 171287},
  {0x1504, 163068},
  {0x1505, 166147},
  {0x1506, 153379},
  {0x1507, 167563},
  {0x1508, 153911},
  {0x1509, 162372},
  {0x150A, 162941},
  {0x150B, 162878},
  {0x150C, 163004},
  {0x150D, 163036},
  {0x150E, 162972},
  {0x150F, 162909},
  {0x1510, 166877},
  {0x1511, 166900},
  {0x1512, 166923},
  {0x1513, 166947},
  {0x1514, 166970},
  {0x1515, 166806},
  {0x1516, 166829},
  {0x1517, 167067},
  {0x1518, 171082},
  {0x1519, 167091},
  {0x151A, 171116},
  {0x151B, 167115},
  {0x151C, 171150},
  {0x151D, 167140},
  {0x151E, 171185},
  {0x151F, 167164},
  {0x1520, 171219},
  {0x1521, 167018},
  {0x1522, 171013},
  {0x1523, 167042},
  {0x1524, 171047},
  {0x1525, 166784},
  {0x1526, 173296},
  {0x1527, 173249},
  {0x1528, 173318},
  {0x1529, 173340},
  {0x152A, 173363},
  {0x152B, 173385},
  {0x152C, 173174},
  {0x152D, 173204},
  {0x152E, 173226},
  {0x152F, 173478},
  {0x1530, 172082},
  {0x1531, 173501},
  {0x1532, 172115},
  {0x1533, 173524},
  {0x1534, 172148},
  {0x1535, 173548},
  {0x1536, 172182},
  {0x1537, 173571},
  {0x1538, 172215},
  {0x1539, 173431},
  {0x153A, 172015},
  {0x153B, 173454},
  {0x153C, 172048},
  {0x153D, 163194},
  {0x153E, 172825},
  {0x153F, 153499},
  {0x1540, 171984},
  {0x1541, 166733},
  {0x1542, 165870},
  {0x1543, 165719},
  {0x1544, 169859},
  {0x1545, 165823},
  {0x1546, 165892},
  {0x1547, 165914},
  {0x1548, 165937},
  {0x1549, 165959},
  {0x154A, 169891},
  {0x154B, 165778},
  {0x154C, 165800},
  {0x154D, 169794},
  {0x154E, 166005},
  {0x154F, 170979},
  {0x1550, 165698},
  {0x1551, 170820},
  {0x1552, 162232},
  {0x1553, 160243},
  {0x1554, 160219},
  {0x1555, 160265},
  {0x1556, 160287},
  {0x1557, 160839},
  {0x1558, 160861},
  {0x1559, 160174},
  {0x155A, 160196},
  {0x155B, 160913},
  {0x155C, 169495},
  {0x155D, 160153},
  {0x155E, 168203},
  {0x155F, 162590},
  {0x1560, 168226},
  {0x1561, 162620},
  {0x1562, 168249},
  {0x1563, 162650},
  {0x1564, 168273},
  {0x1565, 168296},
  {0x1566, 168156},
  {0x1567, 168179},
  {0x1568, 168344},
  {0x1569, 171488},
  {0x156A, 167884},
  {0x156B, 168747},
  {0x156C, 168771},
  {0x156D, 168795},
  {0x156E, 168698},
  {0x156F, 168675},
  {0x1570, 169102},
  {0x1571, 169125},
  {0x1572, 169148},
  {0x1573, 169079},
  {0x1574, 164268},
  {0x1575, 164298},
  {0x1576, 164328},
  {0x1577, 164359},
  {0x1578, 164389},
  {0x1579, 164207},
  {0x157A, 164237},
  {0x157B, 164178},
  {0x157C, 164420},
  {0x157D, 160969},
  {0x157E, 165561},
  {0x157F, 165608},
  {0x1580, 165630},
  {0x1581, 165653},
  {0x1582, 165675},
  {0x1583, 165516},
  {0x1584, 165538},
  {0x1585, 165495},
  {0x1586, 168462},
  {0x1587, 168486},
  {0x1588, 168510},
  {0x1589, 168438},
  {0x158A, 170883},
  {0x158B, 170915},
  {0x158C, 170947},
  {0x158D, 170851},
  {0x158E, 163730},
  {0x158F, 163779},
  {0x1590, 163802},
  {0x1591, 163826},
  {0x1592, 163849},
  {0x1593, 163683},
  {0x1594, 163706},
  {0x1595, 163661},
  {0x1596, 163940},
  {0x1597, 166552},
  {0x1598, 166582},
  {0x1599, 166612},
  {0x159A, 166522},
  {0x159B, 172474},
  {0x159C, 172508},
  {0x159D, 172542},
  {0x159E, 172440},
  {0x159F, 172407},
  {0x15A0, 161721},
  {0x15A1, 161744},
  {0x15A2, 161768},
  {0x15A3, 161791},
  {0x15A4, 161674},
  {0x15A5, 161697},
  {0x15A6, 161652},
  {0x15A7, 167999},
  {0x15A8, 168030},
  {0x15A9, 168061},
  {0x15AA, 168093},
  {0x15AB, 168124},
  {0x15AC, 167936},
  {0x15AD, 167967},
  {0x15AE, 167906},
  {0x15AF, 153296},
  {0x15B0, 153562},
  {0x15B1, 153593},
  {0x15B2, 153880},
  {0x15B3, 153531},
  {0x15B4, 154005},
  {0x15B5, 154037},
  {0x15B6, 154069},
  {0x15B7, 153973},
  {0x15B8, 153784},
  {0x15B9, 153816},
  {0x15BA, 153848},
  {0x15BB, 153752},
  {0x15BC, 153656},
  {0x15BD, 153688},
  {0x15BE, 153720},
  {0x15BF, 153624},
  {0x15C0, 166289},
  {0x15C1, 166318},
  {0x15C2, 166347},
  {0x15C3, 166260},
  {0x15C4, 155193},
  {0x15C5, 155162},
  {0x15C6, 155068},
  {0x15C7, 155099},
  {0x15C8, 155131},
  {0x15C9, 155037},
  {0x15CA, 157643},
  {0x15CB, 157613},
  {0x15CC, 157522},
  {0x15CD, 157552},
  {0x15CE, 157583},
  {0x15CF, 157492},
  {0x15D0, 159356},
  {0x15D1, 159326},
  {0x15D2, 159235},
  {0x15D3, 159265},
  {0x15D4, 159296},
  {0x15D5, 159205},
  {0x15D6, 155622},
  {0x15D7, 155591},
  {0x15D8, 155497},
  {0x15D9, 155528},
  {0x15DA, 155560},
  {0x15DB, 155466},
  {0x15DC, 158227},
  {0x15DD, 158196},
  {0x15DE, 158102},
  {0x15DF, 158133},
  {0x15E0, 158165},
  {0x15E1, 158071},
  {0x15E2, 159174},
  {0x15E3, 158950},
  {0x15E4, 158856},
  {0x15E5, 158887},
  {0x15E6, 158919},
  {0x15E7, 158825},
  {0x15E8, 157462},
  {0x15E9, 157432},
  {0x15EA, 157341},
  {0x15EB, 157371},
  {0x15EC, 157402},
  {0x15ED, 157311},
  {0x15EE, 157282},
  {0x15EF, 155284},
  {0x15F0, 155254},
  {0x15F1, 154976},
  {0x15F2, 155006},
  {0x15F3, 155224},
  {0x15F4, 154915},
  {0x15F5, 156274},
  {0x15F6, 156243},
  {0x15F7, 156149},
  {0x15F8, 156180},
  {0x15F9, 156212},
  {0x15FA, 156118},
  {0x15FB, 156491},
  {0x15FC, 156460},
  {0x15FD, 156366},
  {0x15FE, 156397},
  {0x15FF, 156429},
  {0x1600, 156335},
  {0x1601, 156305},
  {0x1602, 157252},
  {0x1603, 157222},
  {0x1604, 157101},
  {0x1605, 157131},
  {0x1606, 157192},
  {0x1607, 157071},
  {0x1608, 157041},
  {0x1609, 157011},
  {0x160A, 156920},
  {0x160B, 156950},
  {0x160C, 156981},
  {0x160D, 156890},
  {0x160E, 159537},
  {0x160F, 159507},
  {0x1610, 159416},
  {0x1611, 159446},
  {0x1612, 159477},
  {0x1613, 159386},
  {0x1614, 156057},
  {0x1615, 166435},
  {0x1616, 156027},
  {0x1617, 155749},
  {0x1618, 155779},
  {0x1619, 155810},
  {0x161A, 166406},
  {0x161B, 155719},
  {0x161C, 155996},
  {0x161D, 155965},
  {0x161E, 155871},
  {0x161F, 155902},
  {0x1620, 155934},
  {0x1621, 155840},
  {0x1622, 156860},
  {0x1623, 156830},
  {0x1624, 156552},
  {0x1625, 156582},
  {0x1626, 156800},
  {0x1627, 156522},
  {0x1628, 154667},
  {0x1629, 154636},
  {0x162A, 154542},
  {0x162B, 154573},
  {0x162C, 154605},
  {0x162D, 154511},
  {0x162E, 156769},
  {0x162F, 156738},
  {0x1630, 156644},
  {0x1631, 156675},
  {0x1632, 156707},
  {0x1633, 156613},
  {0x1634, 158513},
  {0x1635, 158481},
  {0x1636, 158384},
  {0x1637, 158416},
  {0x1638, 158449},
  {0x1639, 158352},
  {0x163A, 158607},
  {0x163B, 158576},
  {0x163C, 158289},
  {0x163D, 158320},
  {0x163E, 158545},
  {0x163F, 158258},
  {0x1640, 159747},
  {0x1641, 159717},
  {0x1642, 159626},
  {0x1643, 159656},
  {0x1644, 159687},
  {0x1645, 159596},
  {0x1646, 159567},
  {0x1647, 155682},
  {0x1648, 154854},
  {0x1649, 154823},
  {0x164A, 154729},
  {0x164B, 154760},
  {0x164C, 154792},
  {0x164D, 154698},
  {0x164E, 158041},
  {0x164F, 158011},
  {0x1650, 157703},
  {0x1651, 157733},
  {0x1652, 157981},
  {0x1653, 157673},
  {0x1654, 157950},
  {0x1655, 157919},
  {0x1656, 157825},
  {0x1657, 157856},
  {0x1658, 157888},
  {0x1659, 157794},
  {0x165A, 157764},
  {0x165B, 158794},
  {0x165C, 158763},
  {0x165D, 158669},
  {0x165E, 158700},
  {0x165F, 158732},
  {0x1660, 158638},
  {0x1661, 154347},
  {0x1662, 154316},
  {0x1663, 154222},
  {0x1664, 154253},
  {0x1665, 154285},
  {0x1666, 154191},
  {0x1667, 159142},
  {0x1668, 159110},
  {0x1669, 159013},
  {0x166A, 159045},
  {0x166B, 159078},
  {0x166C, 158981},
  {0x166D, 159799},
  {0x166E, 160884},
  {0x166F, 165585},
  {0x1670, 163755},
  {0x1671, 164012},
  {0x1672, 164036},
  {0x1673, 164061},
  {0x1674, 164085},
  {0x1675, 163963},
  {0x1676, 163987},
  {0x1677, 172647},
  {0x1678, 172683},
  {0x1679, 172718},
  {0x167A, 172754},
  {0x167B, 172789},
  {0x167C, 172576},
  {0x167D, 172611},
  {0x167E, 172368},
  {0x167F, 153942},
  {0x1680, 786236},
  {0x1681, 785733},
  {0x1682, 785950},
  {0x1683, 785874},
  {0x1684, 786095},
  {0x1685, 786007},
  {0x1686, 786152},
  {0x1687, 785789},
  {0x1688, 786133},
  {0x1689, 785771},
  {0x168A, 785752},
  {0x168B, 785968},
  {0x168C, 785893},
  {0x168D, 785986},
  {0x168E, 786113},
  {0x168F, 786077},
  {0x1690, 785715},
  {0x1691, 786025},
  {0x1692, 786192},
  {0x1693, 785828},
  {0x1694, 785929},
  {0x1695, 785807},
  {0x1696, 786042},
  {0x1697, 786170},
  {0x1698, 785911},
  {0x1699, 785849},
  {0x169A, 786058},
  {0x169B, 785696},
  {0x169C, 786208},
  {0x16A0, 844373},
  {0x16A1, 845777},
  {0x16A2, 845754},
  {0x16A3, 845863},
  {0x16A4, 845848},
  {0x16A5, 845792},
  {0x16A6, 845691},
  {0x16A7, 844356},
  {0x16A8, 844027},
  {0x16A9, 845202},
  {0x16AA, 843966},
  {0x16AB, 843984},
  {0x16AC, 845003},
  {0x16AD, 845497},
  {0x16AE, 845119},
  {0x16AF, 845134},
  {0x16B0, 845150},
  {0x16B1, 845293},
  {0x16B2, 844855},
  {0x16B3, 844138},
  {0x16B4, 844835},
  {0x16B5, 844553},
  {0x16B6, 844339},
  {0x16B7, 844585},
  {0x16B8, 844568},
  {0x16B9, 845807},
  {0x16BA, 844648},
  {0x16BB, 844627},
  {0x16BC, 844937},
  {0x16BD, 845402},
  {0x16BE, 845088},
  {0x16BF, 845466},
  {0x16C0, 844239},
  {0x16C1, 844750},
  {0x16C2, 844283},
  {0x16C3, 844799},
  {0x16C4, 844610},
  {0x16C5, 844906},
  {0x16C6, 845339},
  {0x16C7, 844777},
  {0x16C8, 845249},
  {0x16C9, 844002},
  {0x16CA, 845651},
  {0x16CB, 845614},
  {0x16CC, 845527},
  {0x16CD, 844086},
  {0x16CE, 845879},
  {0x16CF, 845725},
  {0x16D0, 845557},
  {0x16D1, 844176},
  {0x16D2, 844048},
  {0x16D3, 845368},
  {0x16D4, 844261},
  {0x16D5, 845182},
  {0x16D6, 844315},
  {0x16D7, 845062},
  {0x16D8, 844971},
  {0x16D9, 845435},
  {0x16DA, 844874},
  {0x16DB, 844217},
  {0x16DC, 844713},
  {0x16DD, 844696},
  {0x16DE, 844191},
  {0x16DF, 845220},
  {0x16E0, 844298},
  {0x16E1, 844733},
  {0x16E2, 844155},
  {0x16E3, 844101},
  {0x16E4, 844119},
  {0x16E5, 845673},
  {0x16E6, 845034},
  {0x16E7, 845587},
  {0x16E8, 844670},
  {0x16E9, 845278},
  {0x16EA, 845833},
  {0x16EB, 845921},
  {0x16EC, 845894},
  {0x16ED, 843942},
  {0x16EE, 843900},
  {0x16EF, 845946},
  {0x16F0, 843920},
  {0x16F1, 844820},
  {0x16F2, 845323},
  {0x16F3, 845166},
  {0x16F4, 844523},
  {0x16F5, 844493},
  {0x16F6, 844463},
  {0x16F7, 844401},
  {0x16F8, 844431},
  {0x1700, 918382},
  {0x1701, 918497},
  {0x1702, 918677},
  {0x1703, 918514},
  {0x1704, 918461},
  {0x1705, 918586},
  {0x1706, 918659},
  {0x1707, 918443},
  {0x1708, 918568},
  {0x1709, 918605},
  {0x170A, 918425},
  {0x170B, 918550},
  {0x170C, 918712},
  {0x170D, 918623},
  {0x170E, 918532},
  {0x170F, 918694},
  {0x1710, 918641},
  {0x1711, 918479},
  {0x1712, 918772},
  {0x1713, 918793},
  {0x1714, 918752},
  {0x1715, 918730},
  {0x171F, 918399},
  {0x1720, 504994},
  {0x1721, 505083},
  {0x1722, 505263},
  {0x1723, 505100},
  {0x1724, 505047},
  {0x1725, 505172},
  {0x1726, 505245},
  {0x1727, 505029},
  {0x1728, 505154},
  {0x1729, 505191},
  {0x172A, 505011},
  {0x172B, 505136},
  {0x172C, 505298},
  {0x172D, 505209},
  {0x172E, 505118},
  {0x172F, 505280},
  {0x1730, 505227},
  {0x1731, 505065},
  {0x1732, 505338},
  {0x1733, 505359},
  {0x1734, 505316},
  {0x1735, 822249},
  {0x1736, 822219},
  {0x1740, 142547},
  {0x1741, 142626},
  {0x1742, 142786},
  {0x1743, 142641},
  {0x1744, 142594},
  {0x1745, 142705},
  {0x1746, 142770},
  {0x1747, 142578},
  {0x1748, 142689},
  {0x1749, 142722},
  {0x174A, 142562},
  {0x174B, 142673},
  {0x174C, 142817},
  {0x174D, 142738},
  {0x174E, 142657},
  {0x174F, 142801},
  {0x1750, 142754},
  {0x1751, 142610},
  {0x1752, 142833},
  {0x1753, 142852},
  {0x1760, 918814},
  {0x1761, 918889},
  {0x1762, 919060},
  {0x1763, 918907},
  {0x1764, 918870},
  {0x1765, 918983},
  {0x1766, 919041},
  {0x1767, 918851},
  {0x1768, 918964},
  {0x1769, 919003},
  {0x176A, 918832},
  {0x176B, 918945},
  {0x176C, 919097},
  {0x176E, 918926},
  {0x176F, 919078},
  {0x1770, 919022},
  {0x1772, 919116},
  {0x1773, 919138},
  {0x1780, 571862},
  {0x1781, 571878},
  {0x1782, 571912},
  {0x1783, 571895},
  {0x1784, 571976},
  {0x1785, 571748},
  {0x1786, 571764},
  {0x1787, 571798},
  {0x1788, 571781},
  {0x1789, 572026},
  {0x178A, 571814},
  {0x178B, 572241},
  {0x178C, 571830},
  {0x178D, 572259},
  {0x178E, 571993},
  {0x178F, 572175},
  {0x1790, 572191},
  {0x1791, 572225},
  {0x1792, 572208},
  {0x1793, 572010},
  {0x1794, 571732},
  {0x1795, 572043},
  {0x1796, 572077},
  {0x1797, 572060},
  {0x1798, 571960},
  {0x1799, 572293},
  {0x179A, 572109},
  {0x179B, 571944},
  {0x179C, 572277},
  {0x179D, 572141},
  {0x179E, 572158},
  {0x179F, 572125},
  {0x17A0, 571846},
  {0x17A1, 571928},
  {0x17A2, 572093},
  {0x17A3, 571353},
  {0x17A4, 571297},
  {0x17A5, 571436},
  {0x17A6, 571463},
  {0x17A7, 571565},
  {0x17A8, 571592},
  {0x17A9, 571620},
  {0x17AA, 571648},
  {0x17AB, 571677},
  {0x17AC, 571704},
  {0x17AD, 571242},
  {0x17AE, 571269},
  {0x17AF, 571409},
  {0x17B0, 571325},
  {0x17B1, 571491},
  {0x17B2, 571528},
  {0x17B3, 571381},
  {0x17B4, 573886},
  {0x17B5, 573862},
  {0x17B6, 573910},
  {0x17B7, 574009},
  {0x17B8, 574048},
  {0x17B9, 574167},
  {0x17BA, 574206},
  {0x17BB, 574108},
  {0x17BC, 574147},
  {0x17BD, 574127},
  {0x17BE, 574068},
  {0x17BF, 574186},
  {0x17C0, 574028},
  {0x17C1, 573990},
  {0x17C2, 573930},
  {0x17C3, 573950},
  {0x17C4, 574088},
  {0x17C5, 573970},
  {0x17C6, 572591},
  {0x17C7, 572633},
  {0x17C8, 572754},
  {0x17C9, 572568},
  {0x17CA, 572717},
  {0x17CB, 572371},
  {0x17CC, 572652},
  {0x17CD, 572694},
  {0x17CE, 572495},
  {0x17CF, 572309},
  {0x17D0, 572669},
  {0x17D1, 572736},
  {0x17D2, 572478},
  {0x17D3, 572411},
  {0x17D4, 572514},
  {0x17D5, 572389},
  {0x17D6, 572451},
  {0x17D7, 572549},
  {0x17D8, 572433},
  {0x17D9, 572610},
  {0x17DA, 572530},
  {0x17DB, 571045},
  {0x17DC, 572346},
  {0x17DD, 572326},
  {0x17E0, 571225},
  {0x17E1, 571141},
  {0x17E2, 571209},
  {0x17E3, 571191},
  {0x17E4, 571107},
  {0x17E5, 571090},
  {0x17E6, 571175},
  {0x17E7, 571157},
  {0x17E8, 571072},
  {0x17E9, 571124},
  {0x17F0, 573439},
  {0x17F1, 573226},
  {0x17F2, 573254},
  {0x17F3, 573171},
  {0x17F4, 573198},
  {0x17F5, 573281},
  {0x17F6, 573374},
  {0x17F7, 573407},
  {0x17F8, 573309},
  {0x17F9, 573341},
  {0x1800, 737510},
  {0x1801, 737872},
  {0x1802, 737609},
  {0x1803, 738066},
  {0x1804, 737593},
  {0x1805, 737891},
  {0x1806, 742014},
  {0x1807, 741873},
  {0x1808, 741699},
  {0x1809, 741722},
  {0x180A, 741749},
  {0x180B, 737950},
  {0x180C, 738028},
  {0x180D, 737988},
  {0x180E, 742128},
  {0x180F, 737911},
  {0x1810, 737814},
  {0x1811, 737710},
  {0x1812, 737794},
  {0x1813, 737772},
  {0x1814, 737668},
  {0x1815, 737647},
  {0x1816, 737752},
  {0x1817, 737730},
  {0x1818, 737625},
  {0x1819, 737689},
  {0x1820, 738157},
  {0x1821, 739174},
  {0x1822, 739274},
  {0x1823, 740086},
  {0x1824, 741558},
  {0x1825, 740105},
  {0x1826, 741577},
  {0x1827, 739193},
  {0x1828, 740066},
  {0x1829, 739036},
  {0x182A, 739057},
  {0x182B, 740125},
  {0x182C, 740145},
  {0x182D, 739233},
  {0x182E, 739395},
  {0x182F, 739354},
  {0x1830, 740185},
  {0x1831, 740205},
  {0x1832, 740781},
  {0x1833, 739154},
  {0x1834, 739077},
  {0x1835, 739293},
  {0x1836, 741617},
  {0x1837, 740165},
  {0x1838, 741597},
  {0x1839, 739213},
  {0x183A, 739313},
  {0x183B, 739333},
  {0x183C, 741537},
  {0x183D, 741637},
  {0x183E, 739253},
  {0x183F, 741678},
  {0x1840, 739374},
  {0x1841, 741657},
  {0x1842, 739133},
  {0x1843, 741199},
  {0x1844, 740998},
  {0x1845, 741099},
  {0x1846, 741288},
  {0x1847, 741438},
  {0x1848, 741312},
  {0x1849, 741462},
  {0x184A, 740870},
  {0x184B, 740896},
  {0x184C, 741337},
  {0x184D, 741362},
  {0x184E, 741022},
  {0x184F, 741237},
  {0x1850, 741387},
  {0x1851, 740947},
  {0x1852, 740921},
  {0x1853, 741123},
  {0x1854, 741412},
  {0x1855, 741512},
  {0x1856, 741487},
  {0x1857, 741174},
  {0x1858, 741047},
  {0x1859, 741073},
  {0x185A, 741148},
  {0x185B, 741262},
  {0x185C, 740972},
  {0x185D, 740303},
  {0x185E, 740454},
  {0x185F, 740478},
  {0x1860, 740705},
  {0x1861, 740681},
  {0x1862, 740226},
  {0x1863, 740528},
  {0x1864, 740352},
  {0x1865, 740403},
  {0x1866, 740553},
  {0x1867, 740604},
  {0x1868, 740630},
  {0x1869, 740278},
  {0x186A, 740503},
  {0x186B, 740327},
  {0x186C, 740377},
  {0x186D, 740428},
  {0x186E, 740655},
  {0x186F, 740730},
  {0x1870, 740578},
  {0x1871, 740252},
  {0x1872, 740755},
  {0x1873, 739958},
  {0x1874, 739984},
  {0x1875, 740011},
  {0x1876, 739931},
  {0x1877, 740038},
  {0x1878, 739098},
  {0x1880, 738233},
  {0x1881, 738939},
  {0x1882, 738397},
  {0x1883, 738905},
  {0x1884, 738555},
  {0x1885, 738272},
  {0x1886, 738805},
  {0x1887, 738176},
  {0x1888, 738527},
  {0x1889, 738598},
  {0x188A, 738627},
  {0x188B, 738305},
  {0x188C, 738844},
  {0x188D, 738874},
  {0x188E, 738430},
  {0x188F, 738657},
  {0x1890, 738776},
  {0x1891, 738334},
  {0x1892, 738687},
  {0x1893, 738716},
  {0x1894, 738746},
  {0x1895, 739006},
  {0x1896, 738977},
  {0x1897, 738204},
  {0x1898, 740801},
  {0x1899, 740835},
  {0x189A, 739600},
  {0x189B, 739711},
  {0x189C, 739452},
  {0x189D, 739637},
  {0x189E, 739821},
  {0x189F, 739525},
  {0x18A0, 739785},
  {0x18A1, 739563},
  {0x18A2, 739748},
  {0x18A3, 739488},
  {0x18A4, 739894},
  {0x18A5, 739858},
  {0x18A6, 738460},
  {0x18A7, 738493},
  {0x18A8, 739415},
  {0x18A9, 738363},
  {0x18AA, 739674},
  {0x18B0, 165037},
  {0x18B1, 153411},
  {0x18B2, 153251},
  {0x18B3, 169216},
  {0x18B4, 165284},
  {0x18B5, 165149},
  {0x18B6, 165471},
  {0x18B7, 167839},
  {0x18B8, 161172},
  {0x18B9, 161376},
  {0x18BA, 162159},
  {0x18BB, 164155},
  {0x18BC, 163616},
  {0x18BD, 161607},
  {0x18BE, 167448},
  {0x18BF, 166237},
  {0x18C0, 166994},
  {0x18C1, 166853},
  {0x18C2, 167189},
  {0x18C3, 173408},
  {0x18C4, 173273},
  {0x18C5, 165847},
  {0x18C6, 164519},
  {0x18C7, 164750},
  {0x18C8, 164542},
  {0x18C9, 164781},
  {0x18CA, 164566},
  {0x18CB, 164813},
  {0x18CC, 164589},
  {0x18CD, 164844},
  {0x18CE, 166029},
  {0x18CF, 166053},
  {0x18D0, 166076},
  {0x18D1, 166100},
  {0x18D2, 166123},
  {0x18D3, 165982},
  {0x18D4, 164876},
  {0x18D5, 164964},
  {0x18D6, 164663},
  {0x18D7, 164634},
  {0x18D8, 164692},
  {0x18D9, 164721},
  {0x18DA, 164905},
  {0x18DB, 164934},
  {0x18DC, 160102},
  {0x18DD, 172249},
  {0x18DE, 160764},
  {0x18DF, 160608},
  {0x18E0, 165748},
  {0x18E1, 169923},
  {0x18E2, 169826},
  {0x18E3, 168369},
  {0x18E4, 168320},
  {0x18E5, 168844},
  {0x18E6, 168819},
  {0x18E7, 168722},
  {0x18E8, 168559},
  {0x18E9, 168534},
  {0x18EA, 166673},
  {0x18EB, 166642},
  {0x18EC, 166376},
  {0x18ED, 155345},
  {0x18EE, 154409},
  {0x18EF, 154945},
  {0x18F0, 155314},
  {0x18F1, 166464},
  {0x18F2, 156087},
  {0x18F3, 153433},
  {0x18F4, 153466},
  {0x18F5, 154445},
  {0x1900, 638246},
  {0x1901, 637399},
  {0x1902, 637415},
  {0x1903, 637299},
  {0x1904, 637315},
  {0x1905, 637480},
  {0x1906, 637233},
  {0x1907, 637249},
  {0x1908, 637366},
  {0x1909, 637382},
  {0x190A, 637678},
  {0x190B, 637596},
  {0x190C, 637612},
  {0x190D, 637266},
  {0x190E, 637282},
  {0x190F, 637464},
  {0x1910, 637497},
  {0x1911, 637513},
  {0x1912, 637200},
  {0x1913, 637216},
  {0x1914, 637448},
  {0x1915, 637662},
  {0x1916, 637530},
  {0x1917, 637432},
  {0x1918, 637646},
  {0x1919, 637562},
  {0x191A, 637579},
  {0x191B, 637546},
  {0x191C, 637350},
  {0x191D, 637332},
  {0x191E, 637629},
  {0x1920, 638071},
  {0x1921, 638169},
  {0x1922, 638227},
  {0x1923, 638149},
  {0x1924, 638090},
  {0x1925, 638207},
  {0x1926, 638110},
  {0x1927, 638130},
  {0x1928, 638188},
  {0x1929, 638045},
  {0x192A, 637993},
  {0x192B, 638019},
  {0x1930, 637816},
  {0x1931, 637904},
  {0x1932, 637788},
  {0x1933, 637971},
  {0x1934, 637882},
  {0x1935, 637927},
  {0x1936, 637860},
  {0x1937, 637949},
  {0x1938, 637838},
  {0x1939, 637751},
  {0x193A, 637715},
  {0x193B, 637772},
  {0x1940, 637736},
  {0x1944, 637177},
  {0x1945, 637695},
  {0x1946, 637160},
  {0x1947, 637076},
  {0x1948, 637144},
  {0x1949, 637126},
  {0x194A, 637042},
  {0x194B, 637025},
  {0x194C, 637110},
  {0x194D, 637092},
  {0x194E, 637007},
  {0x194F, 637059},
  {0x1950, 919311},
  {0x1951, 919744},
  {0x1952, 919397},
  {0x1953, 919657},
  {0x1954, 919500},
  {0x1955, 919761},
  {0x1956, 919517},
  {0x1957, 919534},
  {0x1958, 919346},
  {0x1959, 919448},
  {0x195A, 919465},
  {0x195B, 919363},
  {0x195C, 919261},
  {0x195D, 919727},
  {0x195E, 919278},
  {0x195F, 919483},
  {0x1960, 919328},
  {0x1961, 919675},
  {0x1962, 919380},
  {0x1963, 919160},
  {0x1964, 919295},
  {0x1965, 919227},
  {0x1966, 919244},
  {0x1967, 919694},
  {0x1968, 919431},
  {0x1969, 919415},
  {0x196A, 919710},
  {0x196B, 919211},
  {0x196C, 919193},
  {0x196D, 919176},
  {0x1970, 919552},
  {0x1971, 919573},
  {0x1972, 919594},
  {0x1973, 919615},
  {0x1974, 919636},
  {0x1980, 767372},
  {0x1981, 767961},
  {0x1982, 767153},
  {0x1983, 767564},
  {0x1984, 767289},
  {0x1985, 767750},
  {0x1986, 768146},
  {0x1987, 767881},
  {0x1988, 767509},
  {0x1989, 767399},
  {0x198A, 767619},
  {0x198B, 768093},
  {0x198C, 767987},
  {0x198D, 768199},
  {0x198E, 767454},
  {0x198F, 767481},
  {0x1990, 767262},
  {0x1991, 768040},
  {0x1992, 768066},
  {0x1993, 767855},
  {0x1994, 767317},
  {0x1995, 767344},
  {0x1996, 767235},
  {0x1997, 767908},
  {0x1998, 767934},
  {0x1999, 767829},
  {0x199A, 767099},
  {0x199B, 767537},
  {0x199C, 767208},
  {0x199D, 767698},
  {0x199E, 768120},
  {0x199F, 767803},
  {0x19A0, 767126},
  {0x19A1, 767072},
  {0x19A2, 767045},
  {0x19A3, 767724},
  {0x19A4, 767672},
  {0x19A5, 767646},
  {0x19A6, 767180},
  {0x19A7, 767591},
  {0x19A8, 767776},
  {0x19A9, 768172},
  {0x19AA, 767426},
  {0x19AB, 768013},
  {0x19B0, 768759},
  {0x19B1, 768343},
  {0x19B2, 768473},
  {0x19B3, 768629},
  {0x19B4, 768707},
  {0x19B5, 768448},
  {0x19B6, 768396},
  {0x19B7, 768525},
  {0x19B8, 768550},
  {0x19B9, 768654},
  {0x19BA, 768422},
  {0x19BB, 768369},
  {0x19BC, 768733},
  {0x19BD, 768603},
  {0x19BE, 768576},
  {0x19BF, 768680},
  {0x19C0, 768499},
  {0x19C1, 767018},
  {0x19C2, 766990},
  {0x19C3, 766963},
  {0x19C4, 766936},
  {0x19C5, 766909},
  {0x19C6, 766882},
  {0x19C7, 766855},
  {0x19C8, 768295},
  {0x19C9, 768319},
  {0x19D0, 766832},
  {0x19D1, 766718},
  {0x19D2, 766810},
  {0x19D3, 766786},
  {0x19D4, 766672},
  {0x19D5, 766649},
  {0x19D6, 766764},
  {0x19D7, 766740},
  {0x19D8, 766625},
  {0x19D9, 766695},
  {0x19DA, 768268},
  {0x19DE, 768225},
  {0x19DF, 768246},
  {0x19E0, 573511},
  {0x19E1, 573466},
  {0x19E2, 573535},
  {0x19E3, 572779},
  {0x19E4, 572822},
  {0x19E5, 573578},
  {0x19E6, 573731},
  {0x19E7, 573786},
  {0x19E8, 573623},
  {0x19E9, 573676},
  {0x19EA, 572867},
  {0x19EB, 573014},
  {0x19EC, 573067},
  {0x19ED, 572910},
  {0x19EE, 572961},
  {0x19EF, 573118},
  {0x19F0, 573839},
  {0x19F1, 573489},
  {0x19F2, 573557},
  {0x19F3, 572801},
  {0x19F4, 572845},
  {0x19F5, 573601},
  {0x19F6, 573759},
  {0x19F7, 573813},
  {0x19F8, 573650},
  {0x19F9, 573704},
  {0x19FA, 572889},
  {0x19FB, 573041},
  {0x19FC, 573093},
  {0x19FD, 572936},
  {0x19FE, 572988},
  {0x19FF, 573145},
  {0x1A00, 142107},
  {0x1A01, 142050},
  {0x1A02, 142203},
  {0x1A03, 142223},
  {0x1A04, 142305},
  {0x1A05, 141993},
  {0x1A06, 142145},
  {0x1A07, 142164},
  {0x1A08, 142362},
  {0x1A09, 142031},
  {0x1A0A, 142184},
  {0x1A0B, 142244},
  {0x1A0C, 142012},
  {0x1A0D, 142088},
  {0x1A0E, 142264},
  {0x1A0F, 142284},
  {0x1A10, 142400},
  {0x1A11, 142324},
  {0x1A12, 142126},
  {0x1A13, 142381},
  {0x1A14, 142343},
  {0x1A15, 141975},
  {0x1A16, 142069},
  {0x1A17, 142481},
  {0x1A18, 142525},
  {0x1A19, 142459},
  {0x1A1A, 142503},
  {0x1A1B, 142436},
  {0x1A1E, 142419},
  {0x1A1F, 141951},
  {0x1A20, 920557},
  {0x1A21, 920581},
  {0x1A22, 920606},
  {0x1A23, 921043},
  {0x1A24, 921090},
  {0x1A25, 921066},
  {0x1A26, 921338},
  {0x1A27, 920460},
  {0x1A28, 920484},
  {0x1A29, 920950},
  {0x1A2A, 921187},
  {0x1A2B, 920973},
  {0x1A2C, 921358},
  {0x1A2D, 921437},
  {0x1A2E, 920680},
  {0x1A2F, 920397},
  {0x1A30, 921161},
  {0x1A31, 921416},
  {0x1A32, 920781},
  {0x1A33, 920805},
  {0x1A34, 921210},
  {0x1A35, 921233},
  {0x1A36, 921319},
  {0x1A37, 920378},
  {0x1A38, 920631},
  {0x1A39, 920655},
  {0x1A3A, 920509},
  {0x1A3B, 921114},
  {0x1A3C, 920997},
  {0x1A3D, 921137},
  {0x1A3E, 921300},
  {0x1A3F, 921257},
  {0x1A40, 920830},
  {0x1A41, 921397},
  {0x1A42, 921458},
  {0x1A43, 920891},
  {0x1A44, 921280},
  {0x1A45, 921515},
  {0x1A46, 920731},
  {0x1A47, 920756},
  {0x1A48, 920707},
  {0x1A49, 920533},
  {0x1A4A, 920930},
  {0x1A4B, 920360},
  {0x1A4C, 921020},
  {0x1A4D, 920854},
  {0x1A4E, 920872},
  {0x1A4F, 921478},
  {0x1A50, 921496},
  {0x1A51, 920416},
  {0x1A52, 921378},
  {0x1A53, 920910},
  {0x1A54, 920435},
  {0x1A55, 920049},
  {0x1A56, 920015},
  {0x1A57, 919921},
  {0x1A58, 921806},
  {0x1A59, 919842},
  {0x1A5A, 919957},
  {0x1A5B, 919876},
  {0x1A5C, 919988},
  {0x1A5D, 919815},
  {0x1A5E, 920083},
  {0x1A60, 921937},
  {0x1A61, 922360},
  {0x1A62, 922518},
  {0x1A63, 922382},
  {0x1A64, 922672},
  {0x1A65, 922473},
  {0x1A66, 922495},
  {0x1A67, 922750},
  {0x1A68, 922796},
  {0x1A69, 922728},
  {0x1A6A, 922773},
  {0x1A6B, 922546},
  {0x1A6C, 922597},
  {0x1A6D, 922649},
  {0x1A6E, 922451},
  {0x1A6F, 922405},
  {0x1A70, 922626},
  {0x1A71, 922428},
  {0x1A72, 922700},
  {0x1A73, 922568},
  {0x1A74, 921783},
  {0x1A75, 922024},
  {0x1A76, 922045},
  {0x1A77, 921672},
  {0x1A78, 921699},
  {0x1A79, 921726},
  {0x1A7A, 921879},
  {0x1A7B, 921833},
  {0x1A7C, 921753},
  {0x1A7F, 919778},
  {0x1A80, 920335},
  {0x1A81, 920211},
  {0x1A82, 920311},
  {0x1A83, 920285},
  {0x1A84, 920161},
  {0x1A85, 920136},
  {0x1A86, 920261},
  {0x1A87, 920235},
  {0x1A88, 920110},
  {0x1A89, 920186},
  {0x1A90, 922335},
  {0x1A91, 922211},
  {0x1A92, 922311},
  {0x1A93, 922285},
  {0x1A94, 922161},
  {0x1A95, 922136},
  {0x1A96, 922261},
  {0x1A97, 922235},
  {0x1A98, 922110},
  {0x1A99, 922186},
  {0x1AA0, 922066},
  {0x1AA1, 922086},
  {0x1AA2, 922004},
  {0x1AA3, 921653},
  {0x1AA4, 921594},
  {0x1AA5, 921554},
  {0x1AA6, 921901},
  {0x1AA7, 921855},
  {0x1AA8, 921612},
  {0x1AA9, 921631},
  {0x1AAA, 921957},
  {0x1AAB, 921979},
  {0x1AAC, 921575},
  {0x1AAD, 921534},
  {0x1AB0, 200852},
  {0x1AB1, 199922},
  {0x1AB2, 203312},
  {0x1AB3, 200914},
  {0x1AB4, 208260},
  {0x1AB5, 208666},
  {0x1AB6, 208602},
  {0x1AB7, 206956},
  {0x1AB8, 200419},
  {0x1AB9, 206243},
  {0x1ABA, 208005},
  {0x1ABB, 207034},
  {0x1ABC, 200478},
  {0x1ABD, 207062},
  {0x1ABE, 207090},
  {0x1ABF, 205503},
  {0x1AC0, 205256},
  {0x1AC1, 205863},
  {0x1AC2, 207582},
  {0x1AC3, 205901},
  {0x1AC4, 207622},
  {0x1AC5, 207973},
  {0x1AC6, 206726},
  {0x1AC7, 203419},
  {0x1AC8, 207120},
  {0x1AC9, 200513},
  {0x1ACA, 200546},
  {0x1ACB, 208230},
  {0x1ACC, 204547},
  {0x1ACD, 204586},
  {0x1ACE, 204625},
  {0x1B00, 86578},
  {0x1B01, 86553},
  {0x1B02, 86490},
  {0x1B03, 86532},
  {0x1B04, 86470},
  {0x1B05, 83952},
  {0x1B06, 83974},
  {0x1B07, 84372},
  {0x1B08, 84394},
  {0x1B09, 85113},
  {0x1B0A, 85135},
  {0x1B0B, 84861},
  {0x1B0C, 84885},
  {0x1B0D, 84586},
  {0x1B0E, 84611},
  {0x1B0F, 84288},
  {0x1B10, 83929},
  {0x1B11, 84747},
  {0x1B12, 84769},
  {0x1B13, 84466},
  {0x1B14, 84485},
  {0x1B15, 84310},
  {0x1B16, 84329},
  {0x1B17, 84707},
  {0x1B18, 84107},
  {0x1B19, 84126},
  {0x1B1A, 84423},
  {0x1B1B, 84442},
  {0x1B1C, 84727},
  {0x1B1D, 85002},
  {0x1B1E, 85027},
  {0x1B1F, 84193},
  {0x1B20, 84228},
  {0x1B21, 84681},
  {0x1B22, 84983},
  {0x1B23, 85062},
  {0x1B24, 84150},
  {0x1B25, 84169},
  {0x1B26, 84662},
  {0x1B27, 84798},
  {0x1B28, 84817},
  {0x1B29, 84061},
  {0x1B2A, 84080},
  {0x1B2B, 84643},
  {0x1B2C, 85208},
  {0x1B2D, 84842},
  {0x1B2E, 84567},
  {0x1B2F, 85189},
  {0x1B30, 84935},
  {0x1B31, 84959},
  {0x1B32, 84916},
  {0x1B33, 84353},
  {0x1B34, 86510},
  {0x1B35, 86976},
  {0x1B36, 87003},
  {0x1B37, 87027},
  {0x1B38, 86789},
  {0x1B39, 86814},
  {0x1B3A, 86726},
  {0x1B3B, 86754},
  {0x1B3C, 86602},
  {0x1B3D, 86631},
  {0x1B3E, 86844},
  {0x1B3F, 86871},
  {0x1B40, 86942},
  {0x1B41, 86903},
  {0x1B42, 86667},
  {0x1B43, 86693},
  {0x1B44, 83581},
  {0x1B45, 84514},
  {0x1B46, 84540},
  {0x1B47, 85086},
  {0x1B48, 84263},
  {0x1B49, 85164},
  {0x1B4A, 85227},
  {0x1B4B, 84032},
  {0x1B4C, 84003},
  {0x1B4E, 83900},
  {0x1B4F, 83868},
  {0x1B50, 83848},
  {0x1B51, 83749},
  {0x1B52, 83829},
  {0x1B53, 83808},
  {0x1B54, 83709},
  {0x1B55, 83689},
  {0x1B56, 83789},
  {0x1B57, 83768},
  {0x1B58, 83668},
  {0x1B59, 83729},
  {0x1B5A, 86411},
  {0x1B5B, 86353},
  {0x1B5C, 87056},
  {0x1B5D, 83600},
  {0x1B5E, 83648},
  {0x1B5F, 83625},
  {0x1B60, 86393},
  {0x1B61, 85891},
  {0x1B62, 85803},
  {0x1B63, 85920},
  {0x1B64, 85704},
  {0x1B65, 85767},
  {0x1B66, 85862},
  {0x1B67, 85644},
  {0x1B68, 85832},
  {0x1B69, 85674},
  {0x1B6A, 85733},
  {0x1B6B, 85604},
  {0x1B6C, 85293},
  {0x1B6D, 85509},
  {0x1B6E, 85414},
  {0x1B6F, 85372},
  {0x1B70, 85550},
  {0x1B71, 85455},
  {0x1B72, 85253},
  {0x1B73, 85333},
  {0x1B74, 86309},
  {0x1B75, 86265},
  {0x1B76, 86219},
  {0x1B77, 86173},
  {0x1B78, 86041},
  {0x1B79, 86129},
  {0x1B7A, 85949},
  {0x1B7B, 85995},
  {0x1B7C, 86085},
  {0x1B7D, 86447},
  {0x1B7E, 86369},
  {0x1B7F, 86426},
  {0x1B80, 909798},
  {0x1B81, 909748},
  {0x1B82, 909773},
  {0x1B83, 908675},
  {0x1B84, 908944},
  {0x1B85, 909308},
  {0x1B86, 908694},
  {0x1B87, 909147},
  {0x1B88, 908795},
  {0x1B89, 908814},
  {0x1B8A, 908983},
  {0x1B8B, 909186},
  {0x1B8C, 908904},
  {0x1B8D, 909105},
  {0x1B8E, 908755},
  {0x1B8F, 908963},
  {0x1B90, 909407},
  {0x1B91, 909126},
  {0x1B92, 909288},
  {0x1B93, 908775},
  {0x1B94, 909085},
  {0x1B95, 909166},
  {0x1B96, 908834},
  {0x1B97, 909327},
  {0x1B98, 908714},
  {0x1B99, 909065},
  {0x1B9A, 909387},
  {0x1B9B, 909206},
  {0x1B9C, 909024},
  {0x1B9D, 909347},
  {0x1B9E, 909247},
  {0x1B9F, 909367},
  {0x1BA0, 908924},
  {0x1BA1, 908289},
  {0x1BA2, 908324},
  {0x1BA3, 908358},
  {0x1BA4, 909937},
  {0x1BA5, 909997},
  {0x1BA6, 909873},
  {0x1BA7, 909967},
  {0x1BA8, 909844},
  {0x1BA9, 909905},
  {0x1BAA, 909725},
  {0x1BAB, 909822},
  {0x1BAC, 908391},
  {0x1BAD, 908428},
  {0x1BAE, 909003},
  {0x1BAF, 909267},
  {0x1BB0, 908654},
  {0x1BB1, 908550},
  {0x1BB2, 908634},
  {0x1BB3, 908612},
  {0x1BB4, 908508},
  {0x1BB5, 908487},
  {0x1BB6, 908592},
  {0x1BB7, 908570},
  {0x1BB8, 908465},
  {0x1BB9, 908529},
  {0x1BBA, 908270},
  {0x1BBB, 909226},
  {0x1BBC, 909044},
  {0x1BBD, 908734},
  {0x1BBE, 908854},
  {0x1BBF, 908879},
  {0x1BC0, 105433},
  {0x1BC1, 105873},
  {0x1BC2, 105512},
  {0x1BC3, 105926},
  {0x1BC4, 105612},
  {0x1BC5, 105448},
  {0x1BC6, 105559},
  {0x1BC7, 105802},
  {0x1BC8, 106007},
  {0x1BC9, 105710},
  {0x1BCA, 105639},
  {0x1BCB, 106182},
  {0x1BCC, 106088},
  {0x1BCD, 105818},
  {0x1BCE, 105496},
  {0x1BCF, 105899},
  {0x1BD0, 105543},
  {0x1BD1, 105480},
  {0x1BD2, 105841},
  {0x1BD3, 106034},
  {0x1BD4, 105596},
  {0x1BD5, 105980},
  {0x1BD6, 106142},
  {0x1BD7, 105760},
  {0x1BD8, 105857},
  {0x1BD9, 106061},
  {0x1BDA, 105666},
  {0x1BDB, 106198},
  {0x1BDC, 106115},
  {0x1BDD, 105743},
  {0x1BDE, 105580},
  {0x1BDF, 105953},
  {0x1BE0, 105785},
  {0x1BE1, 105464},
  {0x1BE2, 105726},
  {0x1BE3, 105693},
  {0x1BE4, 105528},
  {0x1BE5, 106167},
  {0x1BE6, 106246},
  {0x1BE7, 106374},
  {0x1BE8, 106499},
  {0x1BE9, 106393},
  {0x1BEA, 106413},
  {0x1BEB, 106432},
  {0x1BEC, 106480},
  {0x1BED, 106456},
  {0x1BEE, 106525},
  {0x1BEF, 106544},
  {0x1BF0, 105409},
  {0x1BF1, 105386},
  {0x1BF2, 106214},
  {0x1BF3, 106229},
  {0x1BFC, 106288},
  {0x1BFD, 106344},
  {0x1BFE, 106263},
  {0x1BFF, 106316},
  {0x1C00, 634752},
  {0x1C01, 634787},
  {0x1C02, 634769},
  {0x1C03, 634665},
  {0x1C04, 634682},
  {0x1C05, 634874},
  {0x1C06, 634542},
  {0x1C07, 634559},
  {0x1C08, 634735},
  {0x1C09, 634892},
  {0x1C0A, 635015},
  {0x1C0B, 635032},
  {0x1C0C, 634577},
  {0x1C0D, 634857},
  {0x1C0E, 634910},
  {0x1C0F, 634945},
  {0x1C10, 634927},
  {0x1C11, 634630},
  {0x1C12, 634647},
  {0x1C13, 634507},
  {0x1C14, 634524},
  {0x1C15, 634822},
  {0x1C16, 634839},
  {0x1C17, 635050},
  {0x1C18, 635068},
  {0x1C19, 634612},
  {0x1C1A, 635158},
  {0x1C1B, 634963},
  {0x1C1C, 634805},
  {0x1C1D, 634700},
  {0x1C1E, 634717},
  {0x1C1F, 635124},
  {0x1C20, 634980},
  {0x1C21, 634997},
  {0x1C22, 635141},
  {0x1C23, 634491},
  {0x1C24, 635385},
  {0x1C25, 635358},
  {0x1C26, 635412},
  {0x1C27, 635453},
  {0x1C28, 635473},
  {0x1C29, 635493},
  {0x1C2A, 635514},
  {0x1C2B, 635534},
  {0x1C2C, 635433},
  {0x1C2D, 634086},
  {0x1C2E, 634161},
  {0x1C2F, 634137},
  {0x1C30, 634185},
  {0x1C31, 634239},
  {0x1C32, 634263},
  {0x1C33, 634287},
  {0x1C34, 634209},
  {0x1C35, 634110},
  {0x1C36, 635342},
  {0x1C37, 635324},
  {0x1C3B, 635239},
  {0x1C3C, 635201},
  {0x1C3D, 635175},
  {0x1C3E, 635291},
  {0x1C3F, 635265},
  {0x1C40, 634473},
  {0x1C41, 634384},
  {0x1C42, 634456},
  {0x1C43, 634437},
  {0x1C44, 634348},
  {0x1C45, 634330},
  {0x1C46, 634420},
  {0x1C47, 634401},
  {0x1C48, 634311},
  {0x1C49, 634366},
  {0x1C4D, 635087},
  {0x1C4E, 635105},
  {0x1C4F, 634594},
  {0x1C50, 786485},
  {0x1C51, 786386},
  {0x1C52, 786466},
  {0x1C53, 786445},
  {0x1C54, 786346},
  {0x1C55, 786326},
  {0x1C56, 786426},
  {0x1C57, 786405},
  {0x1C58, 786305},
  {0x1C59, 786366},
  {0x1C5A, 786843},
  {0x1C5B, 786669},
  {0x1C5C, 786611},
  {0x1C5D, 786649},
  {0x1C5E, 786630},
  {0x1C5F, 786862},
  {0x1C60, 786551},
  {0x1C61, 786531},
  {0x1C62, 786571},
  {0x1C63, 786591},
  {0x1C64, 786901},
  {0x1C65, 786824},
  {0x1C66, 786766},
  {0x1C67, 786785},
  {0x1C68, 786805},
  {0x1C69, 786939},
  {0x1C6A, 787035},
  {0x1C6B, 787054},
  {0x1C6C, 787073},
  {0x1C6D, 787093},
  {0x1C6E, 786882},
  {0x1C6F, 786727},
  {0x1C70, 786688},
  {0x1C71, 786708},
  {0x1C72, 786746},
  {0x1C73, 786920},
  {0x1C74, 786996},
  {0x1C75, 786958},
  {0x1C76, 787016},
  {0x1C77, 786977},
  {0x1C78, 787112},
  {0x1C79, 786505},
  {0x1C7A, 787132},
  {0x1C7B, 787242},
  {0x1C7C, 787161},
  {0x1C7D, 786291},
  {0x1C7E, 787214},
  {0x1C7F, 787179},
  {0x1C80, 266167},
  {0x1C81, 265428},
  {0x1C82, 265534},
  {0x1C83, 267472},
  {0x1C84, 266794},
  {0x1C85, 266989},
  {0x1C86, 266757},
  {0x1C87, 266824},
  {0x1C88, 267387},
  {0x1C89, 260541},
  {0x1C8A, 267027},
  {0x1C90, 451917},
  {0x1C91, 451953},
  {0x1C92, 452288},
  {0x1C93, 452140},
  {0x1C94, 452216},
  {0x1C95, 453346},
  {0x1C96, 453492},
  {0x1C97, 453192},
  {0x1C98, 452590},
  {0x1C99, 452701},
  {0x1C9A, 452821},
  {0x1C9B, 452858},
  {0x1C9C, 452895},
  {0x1C9D, 452932},
  {0x1C9E, 452968},
  {0x1C9F, 453529},
  {0x1CA0, 453080},
  {0x1CA1, 453117},
  {0x1CA2, 453229},
  {0x1CA3, 453310},
  {0x1CA4, 453005},
  {0x1CA5, 452738},
  {0x1CA6, 452325},
  {0x1CA7, 453043},
  {0x1CA8, 453154},
  {0x1CA9, 452065},
  {0x1CAA, 451990},
  {0x1CAB, 452664},
  {0x1CAC, 452103},
  {0x1CAD, 452027},
  {0x1CAE, 453419},
  {0x1CAF, 452626},
  {0x1CB0, 452363},
  {0x1CB1, 452480},
  {0x1CB2, 452516},
  {0x1CB3, 453383},
  {0x1CB4, 452400},
  {0x1CB5, 452553},
  {0x1CB6, 452252},
  {0x1CB7, 453456},
  {0x1CB8, 452177},
  {0x1CB9, 453266},
  {0x1CBA, 451880},
  {0x1CBD, 451843},
  {0x1CBE, 452437},
  {0x1CBF, 452776},
  {0x1CC0, 909691},
  {0x1CC1, 909618},
  {0x1CC2, 909655},
  {0x1CC3, 909466},
  {0x1CC4, 909578},
  {0x1CC5, 909539},
  {0x1CC6, 909500},
  {0x1CC7, 909427},
  {0x1CD0, 974203},
  {0x1CD1, 974341},
  {0x1CD2, 974251},
  {0x1CD3, 973644},
  {0x1CD4, 973997},
  {0x1CD5, 974438},
  {0x1CD6, 974491},
  {0x1CD7, 974533},
  {0x1CD8, 974103},
  {0x1CD9, 974583},
  {0x1CDA, 974177},
  {0x1CDB, 974386},
  {0x1CDC, 974223},
  {0x1CDD, 974127},
  {0x1CDE, 974412},
  {0x1CDF, 974358},
  {0x1CE0, 974270},
  {0x1CE1, 974035},
  {0x1CE2, 973908},
  {0x1CE3, 973935},
  {0x1CE4, 973702},
  {0x1CE5, 973842},
  {0x1CE6, 973665},
  {0x1CE7, 973961},
  {0x1CE8, 973870},
  {0x1CE9, 973303},
  {0x1CEA, 973336},
  {0x1CEB, 973404},
  {0x1CEC, 973436},
  {0x1CED, 973801},
  {0x1CEE, 973562},
  {0x1CEF, 973619},
  {0x1CF0, 973769},
  {0x1CF1, 973369},
  {0x1CF2, 973478},
  {0x1CF3, 973737},
  {0x1CF4, 974079},
  {0x1CF5, 973596},
  {0x1CF6, 973819},
  {0x1CF7, 973502},
  {0x1CF8, 974319},
  {0x1CF9, 974148},
  {0x1CFA, 973522},
  {0x1D00, 599800},
  {0x1D01, 599829},
  {0x1D02, 619964},
  {0x1D03, 599888},
  {0x1D04, 599924},
  {0x1D05, 599953},
  {0x1D06, 600011},
  {0x1D07, 599982},
  {0x1D08, 620628},
  {0x1D09, 620234},
  {0x1D0A, 600266},
  {0x1D0B, 600295},
  {0x1D0C, 600392},
  {0x1D0D, 600433},
  {0x1D0E, 600745},
  {0x1D0F, 600491},
  {0x1D10, 600550},
  {0x1D11, 618390},
  {0x1D12, 618462},
  {0x1D13, 618420},
  {0x1D14, 620506},
  {0x1D15, 600584},
  {0x1D16, 619775},
  {0x1D17, 603754},
  {0x1D18, 600614},
  {0x1D19, 600783},
  {0x1D1A, 601054},
  {0x1D1B, 600881},
  {0x1D1C, 601090},
  {0x1D1D, 618534},
  {0x1D1E, 618348},
  {0x1D1F, 618497},
  {0x1D20, 601119},
  {0x1D21, 601148},
  {0x1D22, 601206},
  {0x1D23, 600042},
  {0x1D24, 601324},
  {0x1D25, 599191},
  {0x1D26, 471077},
  {0x1D27, 471110},
  {0x1D28, 471176},
  {0x1D29, 471237},
  {0x1D2A, 471206},
  {0x1D2B, 261751},
  {0x1D2C, 727140},
  {0x1D2D, 727166},
  {0x1D2E, 727193},
  {0x1D2F, 727219},
  {0x1D30, 727278},
  {0x1D31, 727304},
  {0x1D32, 727733},
  {0x1D33, 727356},
  {0x1D34, 727382},
  {0x1D35, 727446},
  {0x1D36, 727472},
  {0x1D37, 727498},
  {0x1D38, 727524},
  {0x1D39, 727550},
  {0x1D3A, 727576},
  {0x1D3B, 727768},
  {0x1D3C, 727602},
  {0x1D3D, 727628},
  {0x1D3E, 727655},
  {0x1D3F, 727707},
  {0x1D40, 727803},
  {0x1D41, 727829},
  {0x1D42, 727881},
  {0x1D43, 732437},
  {0x1D44, 736006},
  {0x1D45, 732512},
  {0x1D46, 736037},
  {0x1D47, 732540},
  {0x1D48, 733379},
  {0x1D49, 733803},
  {0x1D4A, 735579},
  {0x1D4B, 735112},
  {0x1D4C, 736242},
  {0x1D4D, 733990},
  {0x1D4E, 736135},
  {0x1D4F, 734429},
  {0x1D50, 734911},
  {0x1D51, 733827},
  {0x1D52, 735052},
  {0x1D53, 735141},
  {0x1D54, 735887},
  {0x1D55, 732656},
  {0x1D56, 735170},
  {0x1D57, 735671},
  {0x1D58, 736593},
  {0x1D59, 735638},
  {0x1D5A, 736166},
  {0x1D5B, 736714},
  {0x1D5C, 732486},
  {0x1D5D, 732629},
  {0x1D5E, 734076},
  {0x1D5F, 733514},
  {0x1D60, 734110},
  {0x1D61, 733273},
  {0x1D62, 625190},
  {0x1D63, 625438},
  {0x1D64, 625566},
  {0x1D65, 625597},
  {0x1D66, 479060},
  {0x1D67, 479127},
  {0x1D68, 479195},
  {0x1D69, 479162},
  {0x1D6A, 479094},
  {0x1D6B, 622613},
  {0x1D6C, 603355},
  {0x1D6D, 604988},
  {0x1D6E, 607801},
  {0x1D6F, 612330},
  {0x1D70, 613111},
  {0x1D71, 615463},
  {0x1D72, 616543},
  {0x1D73, 616279},
  {0x1D74, 617816},
  {0x1D75, 619174},
  {0x1D76, 624684},
  {0x1D77, 620085},
  {0x1D78, 728373},
  {0x1D79, 609825},
  {0x1D7A, 619532},
  {0x1D7B, 601396},
  {0x1D7C, 610030},
  {0x1D7D, 615581},
  {0x1D7E, 601437},
  {0x1D7F, 622728},
  {0x1D80, 603394},
  {0x1D81, 605027},
  {0x1D82, 607840},
  {0x1D83, 608316},
  {0x1D84, 610622},
  {0x1D85, 611551},
  {0x1D86, 612369},
  {0x1D87, 613191},
  {0x1D88, 615502},
  {0x1D89, 616623},
  {0x1D8A, 617896},
  {0x1D8B, 607345},
  {0x1D8C, 622928},
  {0x1D8D, 623786},
  {0x1D8E, 624723},
  {0x1D8F, 602548},
  {0x1D90, 602953},
  {0x1D91, 604864},
  {0x1D92, 606878},
  {0x1D93, 615103},
  {0x1D94, 617067},
  {0x1D95, 618136},
  {0x1D96, 609571},
  {0x1D97, 615175},
  {0x1D98, 607386},
  {0x1D99, 622348},
  {0x1D9A, 607637},
  {0x1D9B, 736069},
  {0x1D9C, 732692},
  {0x1D9D, 732716},
  {0x1D9E, 733879},
  {0x1D9F, 735449},
  {0x1DA0, 733931},
  {0x1DA1, 733577},
  {0x1DA2, 735607},
  {0x1DA3, 736104},
  {0x1DA4, 734300},
  {0x1DA5, 734336},
  {0x1DA6, 732921},
  {0x1DA7, 732953},
  {0x1DA8, 734387},
  {0x1DA9, 734640},
  {0x1DAA, 734598},
  {0x1DAB, 733038},
  {0x1DAC, 734935},
  {0x1DAD, 736197},
  {0x1DAE, 734969},
  {0x1DAF, 735008},
  {0x1DB0, 733112},
  {0x1DB1, 732598},
  {0x1DB2, 735194},
  {0x1DB3, 735545},
  {0x1DB4, 733853},
  {0x1DB5, 735695},
  {0x1DB6, 736617},
  {0x1DB7, 736684},
  {0x1DB8, 733209},
  {0x1DB9, 736738},
  {0x1DBA, 736459},
  {0x1DBB, 736884},
  {0x1DBC, 736942},
  {0x1DBD, 736908},
  {0x1DBE, 733905},
  {0x1DBF, 735859},
  {0x1DC0, 200093},
  {0x1DC1, 200063},
  {0x1DC2, 207928},
  {0x1DC3, 208050},
  {0x1DC4, 206604},
  {0x1DC5, 203012},
  {0x1DC6, 206650},
  {0x1DC7, 196710},
  {0x1DC8, 202984},
  {0x1DC9, 196682},
  {0x1DCA, 205083},
  {0x1DCB, 197005},
  {0x1DCC, 206627},
  {0x1DCD, 200205},
  {0x1DCE, 206771},
  {0x1DCF, 208709},
  {0x1DD0, 203493},
  {0x1DD1, 208414},
  {0x1DD2, 208433},
  {0x1DD3, 204363},
  {0x1DD4, 203938},
  {0x1DD5, 204005},
  {0x1DD6, 204037},
  {0x1DD7, 204165},
  {0x1DD8, 204508},
  {0x1DD9, 204299},
  {0x1DDA, 204415},
  {0x1DDB, 203666},
  {0x1DDC, 204664},
  {0x1DDD, 204695},
  {0x1DDE, 203705},
  {0x1DDF, 203744},
  {0x1DE0, 204849},
  {0x1DE1, 203783},
  {0x1DE2, 203822},
  {0x1DE3, 205120},
  {0x1DE4, 205159},
  {0x1DE5, 204782},
  {0x1DE6, 205571},
  {0x1DE7, 203970},
  {0x1DE8, 204069},
  {0x1DE9, 204100},
  {0x1DEA, 205190},
  {0x1DEB, 204332},
  {0x1DEC, 204726},
  {0x1DED, 204957},
  {0x1DEE, 205021},
  {0x1DEF, 204266},
  {0x1DF0, 205377},
  {0x1DF1, 205472},
  {0x1DF2, 203892},
  {0x1DF3, 204911},
  {0x1DF4, 205331},
  {0x1DF5, 208336},
  {0x1DF6, 203637},
  {0x1DF7, 203609},
  {0x1DF8, 199967},
  {0x1DF9, 208565},
  {0x1DFA, 200038},
  {0x1DFB, 199282},
  {0x1DFC, 200301},
  {0x1DFD, 196765},
  {0x1DFE, 205710},
  {0x1DFF, 207356},
  {0x1E00, 583611},
  {0x1E01, 602715},
  {0x1E02, 584092},
  {0x1E03, 603180},
  {0x1E04, 584130},
  {0x1E05, 603216},
  {0x1E06, 584238},
  {0x1E07, 603318},
  {0x1E08, 584562},
  {0x1E09, 603964},
  {0x1E0A, 585093},
  {0x1E0B, 604761},
  {0x1E0C, 585131},
  {0x1E0D, 604797},
  {0x1E0E, 585202},
  {0x1E0F, 604904},
  {0x1E10, 585012},
  {0x1E11, 604653},
  {0x1E12, 585048},
  {0x1E13, 604687},
  {0x1E14, 586412},
  {0x1E15, 606770},
  {0x1E16, 586367},
  {0x1E17, 606727},
  {0x1E18, 586016},
  {0x1E19, 606359},
  {0x1E1A, 586561},
  {0x1E1B, 606984},
  {0x1E1C, 585677},
  {0x1E1D, 606034},
  {0x1E1E, 586897},
  {0x1E1F, 607734},
  {0x1E20, 587274},
  {0x1E21, 608242},
  {0x1E22, 587789},
  {0x1E23, 608766},
  {0x1E24, 587827},
  {0x1E25, 608802},
  {0x1E26, 587751},
  {0x1E27, 608730},
  {0x1E28, 587638},
  {0x1E29, 608623},
  {0x1E2A, 587564},
  {0x1E2B, 608553},
  {0x1E2C, 588636},
  {0x1E2D, 609729},
  {0x1E2E, 588216},
  {0x1E2F, 609274},
  {0x1E30, 589073},
  {0x1E31, 610301},
  {0x1E32, 589259},
  {0x1E33, 610477},
  {0x1E34, 589330},
  {0x1E35, 610544},
  {0x1E36, 589739},
  {0x1E37, 611072},
  {0x1E38, 589777},
  {0x1E39, 611108},
  {0x1E3A, 589905},
  {0x1E3B, 611353},
  {0x1E3C, 589694},
  {0x1E3D, 610998},
  {0x1E3E, 590217},
  {0x1E3F, 612156},
  {0x1E40, 590251},
  {0x1E41, 612227},
  {0x1E42, 590289},
  {0x1E43, 612263},
  {0x1E44, 590679},
  {0x1E45, 612846},
  {0x1E46, 590717},
  {0x1E47, 612882},
  {0x1E48, 590827},
  {0x1E49, 612986},
  {0x1E4A, 590596},
  {0x1E4B, 612697},
  {0x1E4C, 592580},
  {0x1E4D, 614845},
  {0x1E4E, 592624},
  {0x1E4F, 614887},
  {0x1E50, 592299},
  {0x1E51, 614576},
  {0x1E52, 592254},
  {0x1E53, 614533},
  {0x1E54, 592929},
  {0x1E55, 615329},
  {0x1E56, 592963},
  {0x1E57, 615361},
  {0x1E58, 593479},
  {0x1E59, 616086},
  {0x1E5A, 593517},
  {0x1E5B, 616122},
  {0x1E5C, 593555},
  {0x1E5D, 616158},
  {0x1E5E, 593688},
  {0x1E5F, 616424},
  {0x1E60, 594395},
  {0x1E61, 617616},
  {0x1E62, 594433},
  {0x1E63, 617652},
  {0x1E64, 594106},
  {0x1E65, 617310},
  {0x1E66, 594188},
  {0x1E67, 617388},
  {0x1E68, 594471},
  {0x1E69, 617688},
  {0x1E6A, 595137},
  {0x1E6B, 618937},
  {0x1E6C, 595175},
  {0x1E6D, 618973},
  {0x1E6E, 595246},
  {0x1E6F, 619090},
  {0x1E70, 595008},
  {0x1E71, 618747},
  {0x1E72, 596380},
  {0x1E73, 621688},
  {0x1E74, 597188},
  {0x1E75, 622575},
  {0x1E76, 596104},
  {0x1E77, 621424},
  {0x1E78, 597144},
  {0x1E79, 622533},
  {0x1E7A, 596952},
  {0x1E7B, 622268},
  {0x1E7C, 597395},
  {0x1E7D, 623004},
  {0x1E7E, 597324},
  {0x1E7F, 622861},
  {0x1E80, 597819},
  {0x1E81, 623402},
  {0x1E82, 597632},
  {0x1E83, 623225},
  {0x1E84, 597705},
  {0x1E85, 623294},
  {0x1E86, 597743},
  {0x1E87, 623330},
  {0x1E88, 597781},
  {0x1E89, 623366},
  {0x1E8A, 597973},
  {0x1E8B, 623559},
  {0x1E8C, 597935},
  {0x1E8D, 623523},
  {0x1E8E, 598145},
  {0x1E8F, 623951},
  {0x1E90, 598581},
  {0x1E91, 624440},
  {0x1E92, 598696},
  {0x1E93, 624580},
  {0x1E94, 598767},
  {0x1E95, 624647},
  {0x1E96, 608869},
  {0x1E97, 618859},
  {0x1E98, 623465},
  {0x1E99, 624187},
  {0x1E9A, 602589},
  {0x1E9B, 611968},
  {0x1E9C, 611921},
  {0x1E9D, 612009},
  {0x1E9E, 594774},
  {0x1E9F, 605240},
  {0x1EA0, 583258},
  {0x1EA1, 602297},
  {0x1EA2, 583371},
  {0x1EA3, 602404},
  {0x1EA4, 582830},
  {0x1EA5, 601887},
  {0x1EA6, 582932},
  {0x1EA7, 601985},
  {0x1EA8, 582981},
  {0x1EA9, 602032},
  {0x1EAA, 583035},
  {0x1EAB, 602084},
  {0x1EAC, 582879},
  {0x1EAD, 601934},
  {0x1EAE, 582528},
  {0x1EAF, 601599},
  {0x1EB0, 582620},
  {0x1EB1, 601687},
  {0x1EB2, 582664},
  {0x1EB3, 601729},
  {0x1EB4, 582713},
  {0x1EB5, 601776},
  {0x1EB6, 582572},
  {0x1EB7, 601641},
  {0x1EB8, 586137},
  {0x1EB9, 606474},
  {0x1EBA, 586250},
  {0x1EBB, 606616},
  {0x1EBC, 586527},
  {0x1EBD, 606952},
  {0x1EBE, 585762},
  {0x1EBF, 606115},
  {0x1EC0, 585864},
  {0x1EC1, 606213},
  {0x1EC2, 585913},
  {0x1EC3, 606260},
  {0x1EC4, 585967},
  {0x1EC5, 606312},
  {0x1EC6, 585811},
  {0x1EC7, 606162},
  {0x1EC8, 588415},
  {0x1EC9, 609427},
  {0x1ECA, 588302},
  {0x1ECB, 609320},
  {0x1ECC, 591645},
  {0x1ECD, 613912},
  {0x1ECE, 591799},
  {0x1ECF, 614058},
  {0x1ED0, 591217},
  {0x1ED1, 613502},
  {0x1ED2, 591319},
  {0x1ED3, 613600},
  {0x1ED4, 591368},
  {0x1ED5, 613647},
  {0x1ED6, 591422},
  {0x1ED7, 613699},
  {0x1ED8, 591266},
  {0x1ED9, 613549},
  {0x1EDA, 591871},
  {0x1EDB, 614126},
  {0x1EDC, 591961},
  {0x1EDD, 614212},
  {0x1EDE, 592004},
  {0x1EDF, 614253},
  {0x1EE0, 592052},
  {0x1EE1, 614299},
  {0x1EE2, 591914},
  {0x1EE3, 614167},
  {0x1EE4, 596424},
  {0x1EE5, 621730},
  {0x1EE6, 596578},
  {0x1EE7, 621876},
  {0x1EE8, 596650},
  {0x1EE9, 621944},
  {0x1EEA, 596740},
  {0x1EEB, 622030},
  {0x1EEC, 596783},
  {0x1EED, 622071},
  {0x1EEE, 596831},
  {0x1EEF, 622117},
  {0x1EF0, 596693},
  {0x1EF1, 621985},
  {0x1EF2, 598221},
  {0x1EF3, 624023},
  {0x1EF4, 598183},
  {0x1EF5, 623987},
  {0x1EF6, 598288},
  {0x1EF7, 624086},
  {0x1EF8, 598430},
  {0x1EF9, 624299},
  {0x1EFA, 590396},
  {0x1EFB, 612442},
  {0x1EFC, 590433},
  {0x1EFD, 612477},
  {0x1EFE, 598327},
  {0x1EFF, 624123},
  {0x1F00, 472583},
  {0x1F01, 471926},
  {0x1F02, 472849},
  {0x1F03, 472192},
  {0x1F04, 472619},
  {0x1F05, 471962},
  {0x1F06, 472727},
  {0x1F07, 472070},
  {0x1F08, 463789},
  {0x1F09, 463216},
  {0x1F0A, 464124},
  {0x1F0B, 463551},
  {0x1F0C, 463827},
  {0x1F0D, 463254},
  {0x1F0E, 463940},
  {0x1F0F, 463367},
  {0x1F10, 473582},
  {0x1F11, 473412},
  {0x1F12, 473667},
  {0x1F13, 473497},
  {0x1F14, 473620},
  {0x1F15, 473450},
  {0x1F18, 464674},
  {0x1F19, 464496},
  {0x1F1A, 464763},
  {0x1F1B, 464585},
  {0x1F1C, 464714},
  {0x1F1D, 464536},
  {0x1F20, 474410},
  {0x1F21, 473814},
  {0x1F22, 474666},
  {0x1F23, 474070},
  {0x1F24, 474444},
  {0x1F25, 473848},
  {0x1F26, 474548},
  {0x1F27, 473952},
  {0x1F28, 465432},
  {0x1F29, 464918},
  {0x1F2A, 465755},
  {0x1F2B, 465241},
  {0x1F2C, 465468},
  {0x1F2D, 464954},
  {0x1F2E, 465577},
  {0x1F2F, 465063},
  {0x1F30, 475616},
  {0x1F31, 475090},
  {0x1F32, 475746},
  {0x1F33, 475220},
  {0x1F34, 475651},
  {0x1F35, 475125},
  {0x1F36, 475695},
  {0x1F37, 475169},
  {0x1F38, 466315},
  {0x1F39, 466017},
  {0x1F3A, 466451},
  {0x1F3B, 466153},
  {0x1F3C, 466352},
  {0x1F3D, 466054},
  {0x1F3E, 466398},
  {0x1F3F, 466100},
  {0x1F40, 477458},
  {0x1F41, 477288},
  {0x1F42, 477543},
  {0x1F43, 477373},
  {0x1F44, 477496},
  {0x1F45, 477326},
  {0x1F48, 468006},
  {0x1F49, 467828},
  {0x1F4A, 468095},
  {0x1F4B, 467917},
  {0x1F4C, 468046},
  {0x1F4D, 467868},
  {0x1F50, 478623},
  {0x1F51, 478061},
  {0x1F52, 478762},
  {0x1F53, 478200},
  {0x1F54, 478661},
  {0x1F55, 478099},
  {0x1F56, 478708},
  {0x1F57, 478146},
  {0x1F59, 468558},
  {0x1F5B, 468703},
  {0x1F5D, 468598},
  {0x1F5F, 468647},
  {0x1F60, 476661},
  {0x1F61, 476041},
  {0x1F62, 476927},
  {0x1F63, 476307},
  {0x1F64, 476697},
  {0x1F65, 476077},
  {0x1F66, 476805},
  {0x1F67, 476185},
  {0x1F68, 467273},
  {0x1F69, 466739},
  {0x1F6A, 467608},
  {0x1F6B, 467074},
  {0x1F6C, 467311},
  {0x1F6D, 466777},
  {0x1F6E, 467424},
  {0x1F6F, 466890},
  {0x1F70, 473049},
  {0x1F71, 472393},
  {0x1F72, 473753},
  {0x1F73, 473545},
  {0x1F74, 474858},
  {0x1F75, 474228},
  {0x1F76, 475826},
  {0x1F77, 475541},
  {0x1F78, 477629},
  {0x1F79, 477421},
  {0x1F7A, 478848},
  {0x1F7B, 478542},
  {0x1F7C, 477127},
  {0x1F7D, 476471},
  {0x1F80, 472959},
  {0x1F81, 472302},
  {0x1F82, 472895},
  {0x1F83, 472238},
  {0x1F84, 472664},
  {0x1F85, 472007},
  {0x1F86, 472779},
  {0x1F87, 472122},
  {0x1F88, 464067},
  {0x1F89, 463494},
  {0x1F8A, 464172},
  {0x1F8B, 463599},
  {0x1F8C, 463874},
  {0x1F8D, 463301},
  {0x1F8E, 463994},
  {0x1F8F, 463421},
  {0x1F90, 474772},
  {0x1F91, 474176},
  {0x1F92, 474710},
  {0x1F93, 474114},
  {0x1F94, 474487},
  {0x1F95, 473891},
  {0x1F96, 474598},
  {0x1F97, 474002},
  {0x1F98, 465700},
  {0x1F99, 465186},
  {0x1F9A, 465801},
  {0x1F9B, 465287},
  {0x1F9C, 465513},
  {0x1F9D, 464999},
  {0x1F9E, 465629},
  {0x1F9F, 465115},
  {0x1FA0, 477037},
  {0x1FA1, 476417},
  {0x1FA2, 476973},
  {0x1FA3, 476353},
  {0x1FA4, 476742},
  {0x1FA5, 476122},
  {0x1FA6, 476857},
  {0x1FA7, 476237},
  {0x1FA8, 467551},
  {0x1FA9, 467017},
  {0x1FAA, 467656},
  {0x1FAB, 467122},
  {0x1FAC, 467358},
  {0x1FAD, 466824},
  {0x1FAE, 467478},
  {0x1FAF, 466944},
  {0x1FB0, 473139},
  {0x1FB1, 472356},
  {0x1FB2, 473085},
  {0x1FB3, 473176},
  {0x1FB4, 472428},
  {0x1FB6, 472481},
  {0x1FB7, 472523},
  {0x1FB8, 464315},
  {0x1FB9, 463666},
  {0x1FBA, 464277},
  {0x1FBB, 463705},
  {0x1FBC, 463742},
  {0x1FBD, 470953},
  {0x1FBE, 471635},
  {0x1FBF, 471656},
  {0x1FC0, 471584},
  {0x1FC1, 469335},
  {0x1FC2, 474892},
  {0x1FC3, 474944},
  {0x1FC4, 474261},
  {0x1FC6, 474312},
  {0x1FC7, 474352},
  {0x1FC8, 464853},
  {0x1FC9, 464635},
  {0x1FCA, 465902},
  {0x1FCB, 465352},
  {0x1FCC, 465387},
  {0x1FCD, 471717},
  {0x1FCE, 471668},
  {0x1FCF, 471689},
  {0x1FD0, 475861},
  {0x1FD1, 475505},
  {0x1FD2, 475456},
  {0x1FD3, 475304},
  {0x1FD6, 475575},
  {0x1FD7, 475352},
  {0x1FD8, 466572},
  {0x1FD9, 466241},
  {0x1FDA, 466535},
  {0x1FDB, 466279},
  {0x1FDD, 469288},
  {0x1FDE, 469239},
  {0x1FDF, 469260},
  {0x1FE0, 478886},
  {0x1FE1, 478503},
  {0x1FE2, 478451},
  {0x1FE3, 478290},
  {0x1FE4, 477830},
  {0x1FE5, 477796},
  {0x1FE6, 478579},
  {0x1FE7, 478341},
  {0x1FE8, 468957},
  {0x1FE9, 468797},
  {0x1FEA, 468917},
  {0x1FEB, 468838},
  {0x1FEC, 468364},
  {0x1FED, 469367},
  {0x1FEE, 469310},
  {0x1FEF, 479532},
  {0x1FF2, 477163},
  {0x1FF3, 477217},
  {0x1FF4, 476506},
  {0x1FF6, 476559},
  {0x1FF7, 476601},
  {0x1FF8, 468185},
  {0x1FF9, 467967},
  {0x1FFA, 467761},
  {0x1FFB, 467189},
  {0x1FFC, 467226},
  {0x1FFD, 471573},
  {0x1FFE, 469227},
  {0x2000, 428833},
  {0x2001, 428297},
  {0x2002, 428841},
  {0x2003, 428305},
  {0x2004, 938053},
  {0x2005, 444850},
  {0x2006, 891509},
  {0x2007, 443795},
  {0x2008, 829404},
  {0x2009, 937333},
  {0x200A, 487414},
  {0x200B, 1012934},
  {0x200C, 1012912},
  {0x200D, 1012868},
  {0x200E, 630064},
  {0x200F, 837540},
  {0x2010, 524340},
  {0x2011, 771838},
  {0x2012, 443783},
  {0x2013, 428825},
  {0x2014, 428289},
  {0x2015, 523367},
  {0x2016, 286371},
  {0x2017, 285702},
  {0x2018, 628206},
  {0x2019, 835537},
  {0x201A, 888043},
  {0x201B, 887963},
  {0x201C, 626652},
  {0x201D, 834559},
  {0x201E, 285718},
  {0x201F, 285540},
  {0x2020, 269143},
  {0x2021, 285485},
  {0x2022, 142893},
  {0x2023, 950319},
  {0x2024, 800504},
  {0x2025, 953868},
  {0x2026, 523453},
  {0x2027, 524396},
  {0x2028, 638439},
  {0x2029, 812899},
  {0x202A, 630018},
  {0x202B, 837494},
  {0x202C, 825936},
  {0x202D, 630083},
  {0x202E, 837559},
  {0x202F, 761974},
  {0x2030, 819298},
  {0x2031, 819322},
  {0x2032, 828361},
  {0x2033, 285922},
  {0x2034, 950764},
  {0x2035, 833247},
  {0x2036, 833027},
  {0x2037, 833521},
  {0x2038, 173744},
  {0x2039, 888001},
  {0x203A, 888071},
  {0x203B, 830910},
  {0x203C, 285516},
  {0x203D, 535516},
  {0x203E, 808578},
  {0x203F, 954901},
  {0x2040, 179292},
  {0x2041, 173750},
  {0x2042, 82036},
  {0x2043, 524347},
  {0x2044, 444912},
  {0x2045, 628407},
  {0x2046, 835830},
  {0x2047, 285963},
  {0x2048, 829955},
  {0x2049, 441880},
  {0x204A, 947056},
  {0x204B, 833225},
  {0x204C, 114210},
  {0x204D, 115425},
  {0x204E, 652753},
  {0x204F, 833415},
  {0x2050, 196111},
  {0x2051, 953789},
  {0x2052, 208758},
  {0x2053, 911926},
  {0x2054, 536104},
  {0x2055, 444341},
  {0x2056, 937479},
  {0x2057, 829890},
  {0x2058, 444758},
  {0x2059, 444175},
  {0x205A, 953883},
  {0x205B, 444744},
  {0x205C, 285026},
  {0x205D, 950378},
  {0x205E, 974831},
  {0x205F, 708548},
  {0x2060, 988813},
  {0x2061, 448059},
  {0x2062, 536157},
  {0x2063, 536137},
  {0x2064, 536122},
  {0x2066, 630042},
  {0x2067, 837518},
  {0x2068, 444060},
  {0x2069, 825963},
  {0x206A, 532865},
  {0x206B, 78},
  {0x206C, 532837},
  {0x206D, 49},
  {0x206E, 761996},
  {0x206F, 771804},
  {0x2070, 911314},
  {0x2071, 911048},
  {0x2074, 911031},
  {0x2075, 911014},
  {0x2076, 911264},
  {0x2077, 911246},
  {0x2078, 910972},
  {0x2079, 911161},
  {0x207A, 911194},
  {0x207B, 911143},
  {0x207C, 910990},
  {0x207D, 911114},
  {0x207E, 911216},
  {0x207F, 911081},
  {0x2080, 907483},
  {0x2081, 907361},
  {0x2082, 907469},
  {0x2083, 907453},
  {0x2084, 907288},
  {0x2085, 907273},
  {0x2086, 907439},
  {0x2087, 907423},
  {0x2088, 907235},
  {0x2089, 907346},
  {0x208A, 907375},
  {0x208B, 907330},
  {0x208C, 907251},
  {0x208D, 907303},
  {0x208E, 907395},
  {0x2090, 625097},
  {0x2091, 625128},
  {0x2092, 625376},
  {0x2093, 625628},
  {0x2094, 625500},
  {0x2095, 625159},
  {0x2096, 625252},
  {0x2097, 625283},
  {0x2098, 625314},
  {0x2099, 625345},
  {0x209A, 625407},
  {0x209B, 625469},
  {0x209C, 625535},
  {0x20A0, 441785},
  {0x20A1, 196593},
  {0x20A2, 215166},
  {0x20A3, 444993},
  {0x20A4, 650799},
  {0x20A5, 725249},
  {0x20A6, 760397},
  {0x20A7, 819770},
  {0x20A8, 845999},
  {0x20A9, 988799},
  {0x20AA, 766609},
  {0x20AB, 284948},
  {0x20AC, 441775},
  {0x20AD, 576882},
  {0x20AE, 951053},
  {0x20AF, 290118},
  {0x20B0, 454636},
  {0x20B1, 819782},
  {0x20B2, 480760},
  {0x20B3, 82160},
  {0x20B4, 524260},
  {0x20B5, 176365},
  {0x20B6, 651628},
  {0x20B7, 899323},
  {0x20B8, 932278},
  {0x20B9, 530685},
  {0x20BA, 953219},
  {0x20BB, 771898},
  {0x20BC, 663353},
  {0x20BD, 843228},
  {0x20BE, 582356},
  {0x20BF, 111331},
  {0x20C0, 895393},
  {0x20D0, 205834},
  {0x20D1, 207552},
  {0x20D2, 206386},
  {0x20D3, 207890},
  {0x20D4, 196822},
  {0x20D5, 197214},
  {0x20D6, 205656},
  {0x20D7, 207268},
  {0x20D8, 207780},
  {0x20D9, 197246},
  {0x20DA, 196858},
  {0x20DB, 208076},
  {0x20DC, 201204},
  {0x20DD, 200940},
  {0x20DE, 201086},
  {0x20DF, 201004},
  {0x20E0, 200967},
  {0x20E1, 205939},
  {0x20E2, 201059},
  {0x20E3, 201032},
  {0x20E4, 201113},
  {0x20E5, 207203},
  {0x20E6, 200811},
  {0x20E7, 196797},
  {0x20E8, 208281},
  {0x20E9, 208537},
  {0x20EA, 206031},
  {0x20EB, 206287},
  {0x20EC, 207689},
  {0x20ED, 206065},
  {0x20EE, 205683},
  {0x20EF, 207296},
  {0x20F0, 196895},
  {0x2100, 38},
  {0x2101, 131},
  {0x2102, 286680},
  {0x2103, 269480},
  {0x2104, 176406},
  {0x2105, 173736},
  {0x2106, 153112},
  {0x2107, 441760},
  {0x2108, 850399},
  {0x2109, 269495},
  {0x210A, 850347},
  {0x210B, 850202},
  {0x210C, 116582},
  {0x210D, 286732},
  {0x210E, 823203},
  {0x210F, 823219},
  {0x2110, 850219},
  {0x2111, 116605},
  {0x2112, 850236},
  {0x2113, 850362},
  {0x2114, 578309},
  {0x2115, 286756},
  {0x2116, 774183},
  {0x2117, 896272},
  {0x2118, 850270},
  {0x2119, 286780},
  {0x211A, 286829},
  {0x211B, 850287},
  {0x211C, 116628},
  {0x211D, 286853},
  {0x211E, 826651},
  {0x211F, 832596},
  {0x2120, 853022},
  {0x2121, 930076},
  {0x2122, 950106},
  {0x2123, 974643},
  {0x2124, 286877},
  {0x2125, 807329},
  {0x2126, 786260},
  {0x2127, 536044},
  {0x2128, 116651},
  {0x2129, 953394},
  {0x212A, 552352},
  {0x212B, 24686},
  {0x212C, 850151},
  {0x212D, 116559},
  {0x212E, 429792},
  {0x212F, 850332},
  {0x2130, 850168},
  {0x2131, 850185},
  {0x2132, 953314},
  {0x2133, 850253},
  {0x2134, 850377},
  {0x2135, 8793},
  {0x2136, 109041},
  {0x2137, 454670},
  {0x2138, 269210},
  {0x2139, 532818},
  {0x213A, 842903},
  {0x213B, 443428},
  {0x213C, 287104},
  {0x213D, 287078},
  {0x213E, 286704},
  {0x213F, 286804},
  {0x2140, 287048},
  {0x2141, 953525},
  {0x2142, 953553},
  {0x2143, 833385},
  {0x2144, 953581},
  {0x2145, 286901},
  {0x2146, 286932},
  {0x2147, 286961},
  {0x2148, 286990},
  {0x2149, 287019},
  {0x214A, 828464},
  {0x214B, 953237},
  {0x214C, 819313},
  {0x214D, 4916},
  {0x214E, 953626},
  {0x214F, 913869},
  {0x2150, 978276},
  {0x2151, 978222},
  {0x2152, 978330},
  {0x2153, 978356},
  {0x2154, 978529},
  {0x2155, 978171},
  {0x2156, 978502},
  {0x2157, 978442},
  {0x2158, 978116},
  {0x2159, 978304},
  {0x215A, 978088},
  {0x215B, 978144},
  {0x215C, 978412},
  {0x215D, 978059},
  {0x215E, 978382},
  {0x215F, 444889},
  {0x2160, 842388},
  {0x2161, 842712},
  {0x2162, 842671},
  {0x2163, 842350},
  {0x2164, 842276},
  {0x2165, 842580},
  {0x2166, 842560},
  {0x2167, 842155},
  {0x2168, 842369},
  {0x2169, 842626},
  {0x216A, 842175},
  {0x216B, 842691},
  {0x216C, 842196},
  {0x216D, 842406},
  {0x216E, 842295},
  {0x216F, 842467},
  {0x2170, 893056},
  {0x2171, 893272},
  {0x2172, 893219},
  {0x2173, 893006},
  {0x2174, 892948},
  {0x2175, 893171},
  {0x2176, 893145},
  {0x2177, 892869},
  {0x2178, 893031},
  {0x2179, 893195},
  {0x217A, 892895},
  {0x217B, 893245},
  {0x217C, 892922},
  {0x217D, 893080},
  {0x217E, 892973},
  {0x217F, 893112},
  {0x2180, 842494},
  {0x2181, 842322},
  {0x2182, 842644},
  {0x2183, 842525},
  {0x2184, 616791},
  {0x2185, 842598},
  {0x2186, 842216},
  {0x2187, 842247},
  {0x2188, 842432},
  {0x2189, 978556},
  {0x218A, 953377},
  {0x218B, 953358},
  {0x2190, 630106},
  {0x2191, 960098},
  {0x2192, 837609},
  {0x2193, 287856},
  {0x2194, 627675},
  {0x2195, 955427},
  {0x2196, 772780},
  {0x2197, 771997},
  {0x2198, 896298},
  {0x2199, 896726},
  {0x219A, 631153},
  {0x219B, 838706},
  {0x219C, 633973},
  {0x219D, 841596},
  {0x219E, 633556},
  {0x219F, 961984},
  {0x21A0, 841170},
  {0x21A1, 289946},
  {0x21A2, 631181},
  {0x21A3, 838735},
  {0x21A4, 630386},
  {0x21A5, 960153},
  {0x21A6, 837895},
  {0x21A7, 287915},
  {0x21A8, 955441},
  {0x21A9, 630847},
  {0x21AA, 838359},
  {0x21AB, 630919},
  {0x21AC, 838433},
  {0x21AD, 628060},
  {0x21AE, 627775},
  {0x21AF, 290095},
  {0x21B0, 960563},
  {0x21B1, 960596},
  {0x21B2, 288383},
  {0x21B3, 288418},
  {0x21B4, 838197},
  {0x21B5, 288006},
  {0x21B6, 24962},
  {0x21B7, 196036},
  {0x21B8, 772931},
  {0x21B9, 630642},
  {0x21BA, 24930},
  {0x21BB, 195832},
  {0x21BC, 632446},
  {0x21BD, 632144},
  {0x21BE, 961113},
  {0x21BF, 960997},
  {0x21C0, 840031},
  {0x21C1, 839723},
  {0x21C2, 288992},
  {0x21C3, 288870},
  {0x21C4, 838002},
  {0x21C5, 960176},
  {0x21C6, 630490},
  {0x21C7, 632599},
  {0x21C8, 961201},
  {0x21C9, 840189},
  {0x21CA, 289086},
  {0x21CB, 631898},
  {0x21CC, 839474},
  {0x21CD, 631676},
  {0x21CE, 627889},
  {0x21CF, 839245},
  {0x21D0, 631621},
  {0x21D1, 960730},
  {0x21D2, 839188},
  {0x21D3, 288591},
  {0x21D4, 627865},
  {0x21D5, 955485},
  {0x21D6, 773010},
  {0x21D7, 772214},
  {0x21D8, 896499},
  {0x21D9, 896857},
  {0x21DA, 633505},
  {0x21DB, 841117},
  {0x21DC, 632775},
  {0x21DD, 840372},
  {0x21DE, 960240},
  {0x21DF, 288044},
  {0x21E0, 631598},
  {0x21E1, 960709},
  {0x21E2, 839164},
  {0x21E3, 288568},
  {0x21E4, 630619},
  {0x21E5, 838139},
  {0x21E6, 633994},
  {0x21E7, 962102},
  {0x21E8, 841618},
  {0x21E9, 290025},
  {0x21EA, 962122},
  {0x21EB, 962200},
  {0x21EC, 962232},
  {0x21ED, 962284},
  {0x21EE, 962380},
  {0x21EF, 962407},
  {0x21F0, 841641},
  {0x21F1, 772904},
  {0x21F2, 896422},
  {0x21F3, 955561},
  {0x21F4, 834190},
  {0x21F5, 287940},
  {0x21F6, 937699},
  {0x21F7, 631376},
  {0x21F8, 838935},
  {0x21F9, 627804},
  {0x21FA, 630760},
  {0x21FB, 838270},
  {0x21FC, 627730},
  {0x21FD, 632571},
  {0x21FE, 840160},
  {0x21FF, 627970},
  {0x2200, 444540},
  {0x2201, 208788},
  {0x2202, 817467},
  {0x2203, 937254},
  {0x2204, 937233},
  {0x2205, 428693},
  {0x2206, 530646},
  {0x2207, 759441},
  {0x2208, 427520},
  {0x2209, 773324},
  {0x220A, 892317},
  {0x220B, 208963},
  {0x220C, 280784},
  {0x220D, 892185},
  {0x220E, 428850},
  {0x220F, 758277},
  {0x2210, 758207},
  {0x2211, 758354},
  {0x2212, 725285},
  {0x2213, 725442},
  {0x2214, 285003},
  {0x2215, 280581},
  {0x2216, 853061},
  {0x2217, 82018},
  {0x2218, 841780},
  {0x2219, 142900},
  {0x221A, 902439},
  {0x221B, 215221},
  {0x221C, 444868},
  {0x221D, 828489},
  {0x221E, 532750},
  {0x221F, 833904},
  {0x2220, 24629},
  {0x2221, 704885},
  {0x2222, 899337},
  {0x2223, 280547},
  {0x2224, 280892},
  {0x2225, 812948},
  {0x2226, 773522},
  {0x2227, 651831},
  {0x2228, 652023},
  {0x2229, 535528},
  {0x222A, 954935},
  {0x222B, 535083},
  {0x222C, 285592},
  {0x222D, 950685},
  {0x222E, 209192},
  {0x222F, 911839},
  {0x2230, 978034},
  {0x2231, 195813},
  {0x2232, 195705},
  {0x2233, 24784},
  {0x2234, 937267},
  {0x2235, 106780},
  {0x2236, 830510},
  {0x2237, 828478},
  {0x2238, 284980},
  {0x2239, 441856},
  {0x223A, 449709},
  {0x223B, 523332},
  {0x223C, 945054},
  {0x223D, 833484},
  {0x223E, 535970},
  {0x223F, 887953},
  {0x2240, 988895},
  {0x2241, 773614},
  {0x2242, 725430},
  {0x2243, 82100},
  {0x2244, 773342},
  {0x2245, 27850},
  {0x2246, 27778},
  {0x2247, 765055},
  {0x2248, 9126},
  {0x2249, 773304},
  {0x224A, 9101},
  {0x224B, 950808},
  {0x224C, 9088},
  {0x224D, 429523},
  {0x224E, 449753},
  {0x224F, 276687},
  {0x2250, 27757},
  {0x2251, 449730},
  {0x2252, 27873},
  {0x2253, 529594},
  {0x2254, 196580},
  {0x2255, 429110},
  {0x2256, 841763},
  {0x2257, 841749},
  {0x2258, 214214},
  {0x2259, 429809},
  {0x225A, 429508},
  {0x225B, 906654},
  {0x225C, 269540},
  {0x225D, 428998},
  {0x225E, 705468},
  {0x225F, 829995},
  {0x2260, 773385},
  {0x2261, 524521},
  {0x2262, 773433},
  {0x2263, 907047},
  {0x2264, 636098},
  {0x2265, 460645},
  {0x2266, 636345},
  {0x2267, 460909},
  {0x2268, 635907},
  {0x2269, 460439},
  {0x226A, 743177},
  {0x226B, 743159},
  {0x226C, 109052},
  {0x226D, 773398},
  {0x226E, 773450},
  {0x226F, 773416},
  {0x2270, 765207},
  {0x2271, 765099},
  {0x2272, 636120},
  {0x2273, 460670},
  {0x2274, 765238},
  {0x2275, 765133},
  {0x2276, 636147},
  {0x2277, 460700},
  {0x2278, 765274},
  {0x2279, 765172},
  {0x227A, 826296},
  {0x227B, 907869},
  {0x227C, 826536},
  {0x227D, 908109},
  {0x227E, 826557},
  {0x227F, 908130},
  {0x2280, 280968},
  {0x2281, 281026},
  {0x2282, 907570},
  {0x2283, 911478},
  {0x2284, 773270},
  {0x2285, 773286},
  {0x2286, 907700},
  {0x2287, 911618},
  {0x2288, 764987},
  {0x2289, 765020},
  {0x228A, 907759},
  {0x228B, 911681},
  {0x228C, 744183},
  {0x228D, 744192},
  {0x228E, 744216},
  {0x228F, 900668},
  {0x2290, 901910},
  {0x2291, 900684},
  {0x2292, 901929},
  {0x2293, 899789},
  {0x2294, 899893},
  {0x2295, 191481},
  {0x2296, 190003},
  {0x2297, 191575},
  {0x2298, 185088},
  {0x2299, 185155},
  {0x229A, 191538},
  {0x229B, 184615},
  {0x229C, 185176},
  {0x229D, 184863},
  {0x229E, 906156},
  {0x229F, 906068},
  {0x22A0, 906387},
  {0x22A1, 904895},
  {0x22A2, 836050},
  {0x22A3, 628621},
  {0x22A4, 287262},
  {0x22A5, 955621},
  {0x22A6, 81999},
  {0x22A7, 725558},
  {0x22A8, 951019},
  {0x22A9, 444548},
  {0x22AA, 950887},
  {0x22AB, 286293},
  {0x22AC, 281011},
  {0x22AD, 773624},
  {0x22AE, 280953},
  {0x22AF, 762068},
  {0x22B0, 826583},
  {0x22B1, 908156},
  {0x22B2, 771915},
  {0x22B3, 208982},
  {0x22B4, 771934},
  {0x22B5, 209010},
  {0x22B6, 801060},
  {0x22B7, 529585},
  {0x22B8, 743895},
  {0x22B9, 519425},
  {0x22BA, 535357},
  {0x22BB, 989259},
  {0x22BC, 760419},
  {0x22BD, 771894},
  {0x22BE, 834068},
  {0x22BF, 836342},
  {0x22C0, 758242},
  {0x22C1, 758260},
  {0x22C2, 758223},
  {0x22C3, 758391},
  {0x22C4, 276421},
  {0x22C5, 284990},
  {0x22C6, 906680},
  {0x22C7, 280596},
  {0x22C8, 124776},
  {0x22C9, 627426},
  {0x22CA, 835172},
  {0x22CB, 628133},
  {0x22CC, 835461},
  {0x22CD, 833499},
  {0x22CE, 251650},
  {0x22CF, 251632},
  {0x22D0, 286134},
  {0x22D1, 286164},
  {0x22D2, 285608},
  {0x22D3, 286203},
  {0x22D4, 823122},
  {0x22D5, 428976},
  {0x22D6, 636413},
  {0x22D7, 461021},
  {0x22D8, 975973},
  {0x22D9, 975950},
  {0x22DA, 636038},
  {0x22DB, 460582},
  {0x22DC, 429046},
  {0x22DD, 429021},
  {0x22DE, 429068},
  {0x22DF, 429089},
  {0x22E0, 280985},
  {0x22E1, 281043},
  {0x22E2, 773547},
  {0x22E3, 773579},
  {0x22E4, 900712},
  {0x22E5, 901960},
  {0x22E6, 635934},
  {0x22E7, 460469},
  {0x22E8, 826505},
  {0x22E9, 908078},
  {0x22EA, 773464},
  {0x22EB, 280811},
  {0x22EC, 773487},
  {0x22ED, 280847},
  {0x22EE, 974813},
  {0x22EF, 725180},
  {0x22F0, 955594},
  {0x22F1, 287233},
  {0x22F2, 427613},
  {0x22F3, 427740},
  {0x22F4, 892364},
  {0x22F5, 427587},
  {0x22F6, 427652},
  {0x22F7, 892334},
  {0x22F8, 427715},
  {0x22F9, 427676},
  {0x22FA, 209050},
  {0x22FB, 209109},
  {0x22FC, 892238},
  {0x22FD, 209087},
  {0x22FE, 892210},
  {0x22FF, 1010256},
  {0x2300, 276407},
  {0x2301, 427432},
  {0x2302, 524205},
  {0x2303, 955193},
  {0x2304, 287203},
  {0x2305, 828453},
  {0x2306, 819758},
  {0x2307, 982273},
  {0x2308, 626407},
  {0x2309, 834286},
  {0x230A, 626743},
  {0x230B, 834654},
  {0x230C, 124498},
  {0x230D, 124362},
  {0x230E, 948887},
  {0x230F, 948766},
  {0x2310, 833207},
  {0x2311, 901235},
  {0x2312, 79090},
  {0x2313, 850490},
  {0x2314, 850449},
  {0x2315, 930057},
  {0x2316, 826020},
  {0x2317, 976141},
  {0x2318, 823163},
  {0x2319, 953489},
  {0x231A, 982065},
  {0x231B, 524167},
  {0x231C, 948750},
  {0x231D, 948870},
  {0x231E, 124343},
  {0x231F, 124478},
  {0x2320, 948303},
  {0x2321, 123934},
  {0x2322, 445116},
  {0x2323, 893467},
  {0x2324, 955206},
  {0x2325, 800989},
  {0x2326, 429587},
  {0x2327, 988940},
  {0x2328, 552368},
  {0x2329, 629562},
  {0x232A, 837003},
  {0x232B, 429569},
  {0x232C, 109003},
  {0x232D, 251797},
  {0x232E, 9069},
  {0x232F, 914176},
  {0x2330, 949568},
  {0x2331, 277405},
  {0x2332, 208899},
  {0x2333, 892016},
  {0x2334, 214244},
  {0x2335, 214256},
  {0x2336, 26083},
  {0x2337, 27349},
  {0x2338, 26731},
  {0x2339, 26616},
  {0x233A, 26581},
  {0x233B, 26804},
  {0x233C, 26450},
  {0x233D, 25493},
  {0x233E, 25426},
  {0x233F, 27317},
  {0x2340, 25312},
  {0x2341, 27032},
  {0x2342, 26413},
  {0x2343, 26878},
  {0x2344, 26764},
  {0x2345, 26284},
  {0x2346, 27238},
  {0x2347, 26835},
  {0x2348, 26988},
  {0x2349, 25348},
  {0x234A, 25921},
  {0x234B, 25698},
  {0x234C, 26650},
  {0x234D, 26548},
  {0x234E, 25885},
  {0x234F, 27675},
  {0x2350, 27101},
  {0x2351, 27637},
  {0x2352, 25634},
  {0x2353, 27065},
  {0x2354, 26517},
  {0x2355, 27603},
  {0x2356, 25962},
  {0x2357, 26688},
  {0x2358, 27175},
  {0x2359, 25732},
  {0x235A, 25769},
  {0x235B, 26211},
  {0x235C, 25528},
  {0x235D, 27529},
  {0x235E, 27142},
  {0x235F, 25459},
  {0x2360, 26484},
  {0x2361, 27563},
  {0x2362, 25598},
  {0x2363, 27383},
  {0x2364, 26175},
  {0x2365, 25387},
  {0x2366, 25847},
  {0x2367, 26246},
  {0x2368, 27454},
  {0x2369, 26038},
  {0x236A, 25566},
  {0x236B, 25666},
  {0x236C, 27710},
  {0x236D, 27420},
  {0x236E, 27276},
  {0x236F, 26915},
  {0x2370, 26952},
  {0x2371, 25808},
  {0x2372, 27492},
  {0x2373, 26112},
  {0x2374, 27212},
  {0x2375, 26321},
  {0x2376, 25275},
  {0x2377, 25999},
  {0x2378, 26139},
  {0x2379, 26349},
  {0x237A, 25247},
  {0x237B, 773370},
  {0x237C, 834089},
  {0x237D, 856664},
  {0x237E, 106825},
  {0x237F, 975255},
  {0x2380, 535066},
  {0x2381, 209164},
  {0x2382, 278821},
  {0x2383, 428586},
  {0x2384, 208799},
  {0x2385, 986051},
  {0x2386, 428894},
  {0x2387, 9216},
  {0x2388, 512827},
  {0x2389, 185906},
  {0x238A, 191606},
  {0x238B, 141855},
  {0x238C, 954910},
  {0x238D, 742236},
  {0x238E, 524427},
  {0x238F, 800838},
  {0x2390, 800872},
  {0x2391, 817579},
  {0x2392, 817611},
  {0x2393, 278717},
  {0x2394, 894191},
  {0x2395, 26386},
  {0x2396, 269420},
  {0x2397, 828347},
  {0x2398, 770498},
  {0x2399, 828383},
  {0x239A, 195051},
  {0x239B, 627618},
  {0x239C, 627563},
  {0x239D, 627590},
  {0x239E, 835373},
  {0x239F, 835316},
  {0x23A0, 835344},
  {0x23A1, 628335},
  {0x23A2, 628272},
  {0x23A3, 628302},
  {0x23A4, 835756},
  {0x23A5, 835691},
  {0x23A6, 835722},
  {0x23A7, 626539},
  {0x23A8, 626507},
  {0x23A9, 626477},
  {0x23AA, 251608},
  {0x23AB, 834405},
  {0x23AC, 834372},
  {0x23AD, 834341},
  {0x23AE, 535153},
  {0x23AF, 523491},
  {0x23B0, 957559},
  {0x23B1, 959167},
  {0x23B2, 908197},
  {0x23B3, 908180},
  {0x23B4, 948977},
  {0x23B5, 124597},
  {0x23B6, 124619},
  {0x23B7, 830095},
  {0x23B8, 629022},
  {0x23B9, 836457},
  {0x23BA, 523919},
  {0x23BB, 523942},
  {0x23BC, 523965},
  {0x23BD, 523988},
  {0x23BE, 270147},
  {0x23BF, 270053},
  {0x23C0, 270234},
  {0x23C1, 269640},
  {0x23C2, 269846},
  {0x23C3, 270278},
  {0x23C4, 269695},
  {0x23C5, 269899},
  {0x23C6, 270193},
  {0x23C7, 269752},
  {0x23C8, 269954},
  {0x23C9, 269597},
  {0x23CA, 269805},
  {0x23CB, 270102},
  {0x23CC, 270005},
  {0x23CD, 900199},
  {0x23CE, 832687},
  {0x23CF, 426689},
  {0x23D0, 975132},
  {0x23D1, 722005},
  {0x23D2, 722020},
  {0x23D3, 722094},
  {0x23D4, 722045},
  {0x23D5, 722182},
  {0x23D6, 722155},
  {0x23D7, 722138},
  {0x23D8, 722119},
  {0x23D9, 722075},
  {0x23DA, 293799},
  {0x23DB, 448099},
  {0x23DC, 948854},
  {0x23DD, 124459},
  {0x23DE, 948138},
  {0x23DF, 123687},
  {0x23E0, 948996},
  {0x23E1, 124665},
  {0x23E2, 986553},
  {0x23E3, 109016},
  {0x23E4, 906996},
  {0x23E5, 444252},
  {0x23E6, 17},
  {0x23E7, 427496},
  {0x23E8, 269396},
  {0x23E9, 115057},
  {0x23EA, 113983},
  {0x23EB, 116112},
  {0x23EC, 113263},
  {0x23ED, 115094},
  {0x23EE, 114019},
  {0x23EF, 115320},
  {0x23F0, 4929},
  {0x23F1, 906971},
  {0x23F2, 945270},
  {0x23F3, 524177},
  {0x23F4, 114439},
  {0x23F5, 114540},
  {0x23F6, 114722},
  {0x23F7, 114359},
  {0x23F8, 286231},
  {0x23F9, 115852},
  {0x23FA, 112594},
  {0x23FB, 826270},
  {0x23FC, 826231},
  {0x23FD, 826215},
  {0x23FE, 826251},
  {0x23FF, 785437},
  {0x2400, 913825},
  {0x2401, 913955},
  {0x2402, 913983},
  {0x2403, 913493},
  {0x2404, 913516},
  {0x2405, 913584},
  {0x2406, 913038},
  {0x2407, 913082},
  {0x2408, 913061},
  {0x2409, 913695},
  {0x240A, 913728},
  {0x240B, 914145},
  {0x240C, 913647},
  {0x240D, 913116},
  {0x240E, 913917},
  {0x240F, 913897},
  {0x2410, 913143},
  {0x2411, 913376},
  {0x2412, 913438},
  {0x2413, 913406},
  {0x2414, 913345},
  {0x2415, 913774},
  {0x2416, 914061},
  {0x2417, 913547},
  {0x2418, 913098},
  {0x2419, 913468},
  {0x241A, 914008},
  {0x241B, 913603},
  {0x241C, 913621},
  {0x241D, 913668},
  {0x241E, 913841},
  {0x241F, 914119},
  {0x2420, 913938},
  {0x2421, 913171},
  {0x2422, 116674},
  {0x2423, 800588},
  {0x2424, 913806},
  {0x2425, 913189},
  {0x2426, 914030},
  {0x2427, 913301},
  {0x2428, 913252},
  {0x2429, 913216},
  {0x2440, 785625},
  {0x2441, 785548},
  {0x2442, 785616},
  {0x2443, 785634},
  {0x2444, 785489},
  {0x2445, 785505},
  {0x2446, 785517},
  {0x2447, 785469},
  {0x2448, 785586},
  {0x2449, 785558},
  {0x244A, 785595},
  {0x2460, 184953},
  {0x2461, 185029},
  {0x2462, 185009},
  {0x2463, 184915},
  {0x2464, 184896},
  {0x2465, 184991},
  {0x2466, 184971},
  {0x2467, 184876},
  {0x2468, 184934},
  {0x2469, 190697},
  {0x246A, 190130},
  {0x246B, 191078},
  {0x246C, 190751},
  {0x246D, 190525},
  {0x246E, 190152},
  {0x246F, 190637},
  {0x2470, 190573},
  {0x2471, 190068},
  {0x2472, 190549},
  {0x2473, 191100},
  {0x2474, 813122},
  {0x2475, 813222},
  {0x2476, 813196},
  {0x2477, 813072},
  {0x2478, 813047},
  {0x2479, 813172},
  {0x247A, 813146},
  {0x247B, 813021},
  {0x247C, 813097},
  {0x247D, 817327},
  {0x247E, 817150},
  {0x247F, 817382},
  {0x2480, 817352},
  {0x2481, 817207},
  {0x2482, 817178},
  {0x2483, 817298},
  {0x2484, 817267},
  {0x2485, 817120},
  {0x2486, 817237},
  {0x2487, 817410},
  {0x2488, 276931},
  {0x2489, 277127},
  {0x248A, 277079},
  {0x248B, 276835},
  {0x248C, 276786},
  {0x248D, 277029},
  {0x248E, 276981},
  {0x248F, 276736},
  {0x2490, 276884},
  {0x2491, 774088},
  {0x2492, 773923},
  {0x2493, 774135},
  {0x2494, 774109},
  {0x2495, 773972},
  {0x2496, 773947},
  {0x2497, 774063},
  {0x2498, 774024},
  {0x2499, 773897},
  {0x249A, 773998},
  {0x249B, 774159},
  {0x249C, 816210},
  {0x249D, 816245},
  {0x249E, 816280},
  {0x249F, 816315},
  {0x24A0, 816350},
  {0x24A1, 816385},
  {0x24A2, 816420},
  {0x24A3, 816455},
  {0x24A4, 816490},
  {0x24A5, 816525},
  {0x24A6, 816560},
  {0x24A7, 816595},
  {0x24A8, 816630},
  {0x24A9, 816665},
  {0x24AA, 816700},
  {0x24AB, 816735},
  {0x24AC, 816770},
  {0x24AD, 816805},
  {0x24AE, 816840},
  {0x24AF, 816875},
  {0x24B0, 816910},
  {0x24B1, 816945},
  {0x24B2, 816980},
  {0x24B3, 817015},
  {0x24B4, 817050},
  {0x24B5, 817085},
  {0x24B6, 188425},
  {0x24B7, 188456},
  {0x24B8, 188487},
  {0x24B9, 188518},
  {0x24BA, 188549},
  {0x24BB, 188580},
  {0x24BC, 188611},
  {0x24BD, 188642},
  {0x24BE, 188673},
  {0x24BF, 188704},
  {0x24C0, 188735},
  {0x24C1, 188766},
  {0x24C2, 188797},
  {0x24C3, 188828},
  {0x24C4, 188859},
  {0x24C5, 188890},
  {0x24C6, 188921},
  {0x24C7, 188952},
  {0x24C8, 188983},
  {0x24C9, 189014},
  {0x24CA, 189045},
  {0x24CB, 189076},
  {0x24CC, 189107},
  {0x24CD, 189138},
  {0x24CE, 189169},
  {0x24CF, 189200},
  {0x24D0, 189231},
  {0x24D1, 189260},
  {0x24D2, 189289},
  {0x24D3, 189318},
  {0x24D4, 189347},
  {0x24D5, 189376},
  {0x24D6, 189405},
  {0x24D7, 189434},
  {0x24D8, 189463},
  {0x24D9, 189492},
  {0x24DA, 189521},
  {0x24DB, 189550},
  {0x24DC, 189579},
  {0x24DD, 189608},
  {0x24DE, 189637},
  {0x24DF, 189666},
  {0x24E0, 189695},
  {0x24E1, 189724},
  {0x24E2, 189753},
  {0x24E3, 189782},
  {0x24E4, 189811},
  {0x24E5, 189840},
  {0x24E6, 189869},
  {0x24E7, 189898},
  {0x24E8, 189927},
  {0x24E9, 189956},
  {0x24EA, 185047},
  {0x24EB, 763220},
  {0x24EC, 763448},
  {0x24ED, 763415},
  {0x24EE, 763283},
  {0x24EF, 763251},
  {0x24F0, 763383},
  {0x24F1, 763349},
  {0x24F2, 763187},
  {0x24F3, 763316},
  {0x24F4, 763479},
  {0x24F5, 285293},
  {0x24F6, 285397},
  {0x24F7, 285370},
  {0x24F8, 285241},
  {0x24F9, 285215},
  {0x24FA, 285345},
  {0x24FB, 285318},
  {0x24FC, 285188},
  {0x24FD, 285267},
  {0x24FE, 285422},
  {0x24FF, 762119},
  {0x2500, 130064},
  {0x2501, 126473},
  {0x2502, 130693},
  {0x2503, 126921},
  {0x2504, 130452},
  {0x2505, 126680},
  {0x2506, 130494},
  {0x2507, 126722},
  {0x2508, 130301},
  {0x2509, 126567},
  {0x250A, 130346},
  {0x250B, 126612},
  {0x250C, 130030},
  {0x250D, 126000},
  {0x250E, 125743},
  {0x250F, 126439},
  {0x2510, 129997},
  {0x2511, 125919},
  {0x2512, 125662},
  {0x2513, 126406},
  {0x2514, 130661},
  {0x2515, 131969},
  {0x2516, 131718},
  {0x2517, 126889},
  {0x2518, 130630},
  {0x2519, 131889},
  {0x251A, 131638},
  {0x251B, 126821},
  {0x251C, 130890},
  {0x251D, 132501},
  {0x251E, 131675},
  {0x251F, 125783},
  {0x2520, 132365},
  {0x2521, 126040},
  {0x2522, 131926},
  {0x2523, 127029},
  {0x2524, 130853},
  {0x2525, 132458},
  {0x2526, 131596},
  {0x2527, 125701},
  {0x2528, 132322},
  {0x2529, 125958},
  {0x252A, 131847},
  {0x252B, 126992},
  {0x252C, 129958},
  {0x252D, 127115},
  {0x252E, 131059},
  {0x252F, 125874},
  {0x2530, 125617},
  {0x2531, 131196},
  {0x2532, 127252},
  {0x2533, 126367},
  {0x2534, 130593},
  {0x2535, 127160},
  {0x2536, 131104},
  {0x2537, 131804},
  {0x2538, 131553},
  {0x2539, 131241},
  {0x253A, 127297},
  {0x253B, 126784},
  {0x253C, 130810},
  {0x253D, 127203},
  {0x253E, 131147},
  {0x253F, 132409},
  {0x2540, 131505},
  {0x2541, 125826},
  {0x2542, 132273},
  {0x2543, 127389},
  {0x2544, 131333},
  {0x2545, 127067},
  {0x2546, 131011},
  {0x2547, 126083},
  {0x2548, 131756},
  {0x2549, 131284},
  {0x254A, 127340},
  {0x254B, 126949},
  {0x254C, 129852},
  {0x254D, 126261},
  {0x254E, 129894},
  {0x254F, 126303},
  {0x2550, 125203},
  {0x2551, 125337},
  {0x2552, 126219},
  {0x2553, 125575},
  {0x2554, 125168},
  {0x2555, 126178},
  {0x2556, 125534},
  {0x2557, 125134},
  {0x2558, 132091},
  {0x2559, 131465},
  {0x255A, 125304},
  {0x255B, 132052},
  {0x255C, 131426},
  {0x255D, 125272},
  {0x255E, 132641},
  {0x255F, 132227},
  {0x2560, 125448},
  {0x2561, 132596},
  {0x2562, 132182},
  {0x2563, 125410},
  {0x2564, 126131},
  {0x2565, 125487},
  {0x2566, 125094},
  {0x2567, 132007},
  {0x2568, 131381},
  {0x2569, 125234},
  {0x256A, 132545},
  {0x256B, 132131},
  {0x256C, 125366},
  {0x256D, 127474},
  {0x256E, 127437},
  {0x256F, 127512},
  {0x2570, 127547},
  {0x2571, 129601},
  {0x2572, 129226},
  {0x2573, 127624},
  {0x2574, 130237},
  {0x2575, 130534},
  {0x2576, 130389},
  {0x2577, 129934},
  {0x2578, 126503},
  {0x2579, 126762},
  {0x257A, 126655},
  {0x257B, 126343},
  {0x257C, 130261},
  {0x257D, 130556},
  {0x257E, 126527},
  {0x257F, 126852},
  {0x2580, 956507},
  {0x2581, 655041},
  {0x2582, 655064},
  {0x2583, 656620},
  {0x2584, 653132},
  {0x2585, 653068},
  {0x2586, 656646},
  {0x2587, 656594},
  {0x2588, 445162},
  {0x2589, 628157},
  {0x258A, 628734},
  {0x258B, 626719},
  {0x258C, 626798},
  {0x258D, 628709},
  {0x258E, 627486},
  {0x258F, 627464},
  {0x2590, 834750},
  {0x2591, 636821},
  {0x2592, 708704},
  {0x2593, 269236},
  {0x2594, 958387},
  {0x2595, 835211},
  {0x2596, 829502},
  {0x2597, 829522},
  {0x2598, 829543},
  {0x2599, 829563},
  {0x259A, 829614},
  {0x259B, 829650},
  {0x259C, 829701},
  {0x259D, 829753},
  {0x259E, 829774},
  {0x259F, 829810},
  {0x25A0, 115798},
  {0x25A1, 985883},
  {0x25A2, 986234},
  {0x25A3, 985960},
  {0x25A4, 902874},
  {0x25A5, 903285},
  {0x25A6, 903017},
  {0x25A7, 903155},
  {0x25A8, 903242},
  {0x25A9, 902837},
  {0x25AA, 115680},
  {0x25AB, 985811},
  {0x25AC, 114951},
  {0x25AD, 985537},
  {0x25AE, 116361},
  {0x25AF, 986868},
  {0x25B0, 114859},
  {0x25B1, 985459},
  {0x25B2, 116222},
  {0x25B3, 986763},
  {0x25B4, 116189},
  {0x25B5, 986730},
  {0x25B6, 115290},
  {0x25B7, 985711},
  {0x25B8, 115254},
  {0x25B9, 985675},
  {0x25BA, 115225},
  {0x25BB, 985646},
  {0x25BC, 113379},
  {0x25BD, 984586},
  {0x25BE, 113344},
  {0x25BF, 984551},
  {0x25C0, 114181},
  {0x25C1, 985200},
  {0x25C2, 114146},
  {0x25C3, 985165},
  {0x25C4, 114118},
  {0x25C5, 985137},
  {0x25C6, 113092},
  {0x25C7, 984225},
  {0x25C8, 984285},
  {0x25C9, 444114},
  {0x25CA, 656797},
  {0x25CB, 983773},
  {0x25CC, 285012},
  {0x25CD, 184517},
  {0x25CE, 142951},
  {0x25CF, 112581},
  {0x25D0, 184250},
  {0x25D1, 184307},
  {0x25D2, 184278},
  {0x25D3, 184449},
  {0x25D4, 184478},
  {0x25D5, 184177},
  {0x25D6, 626754},
  {0x25D7, 834704},
  {0x25D8, 535777},
  {0x25D9, 535902},
  {0x25DA, 956678},
  {0x25DB, 653197},
  {0x25DC, 957798},
  {0x25DD, 959431},
  {0x25DE, 656065},
  {0x25DF, 654380},
  {0x25E0, 956577},
  {0x25E1, 653149},
  {0x25E2, 114297},
  {0x25E3, 114271},
  {0x25E4, 116249},
  {0x25E5, 116275},
  {0x25E6, 982571},
  {0x25E7, 902902},
  {0x25E8, 903056},
  {0x25E9, 903112},
  {0x25EA, 902973},
  {0x25EB, 986345},
  {0x25EC, 986790},
  {0x25ED, 956015},
  {0x25EE, 956057},
  {0x25EF, 579935},
  {0x25F0, 986268},
  {0x25F1, 986123},
  {0x25F2, 986161},
  {0x25F3, 986306},
  {0x25F4, 983990},
  {0x25F5, 983886},
  {0x25F6, 983924},
  {0x25F7, 984028},
  {0x25F8, 958071},
  {0x25F9, 959779},
  {0x25FA, 654725},
  {0x25FB, 985393},
  {0x25FC, 114702},
  {0x25FD, 985367},
  {0x25FE, 114676},
  {0x25FF, 656510},
  {0x2600, 115885},
  {0x2601, 196418},
  {0x2602, 954817},
  {0x2603, 894085},
  {0x2604, 208732},
  {0x2605, 115874},
  {0x2606, 986387},
  {0x2607, 636960},
  {0x2608, 938127},
  {0x2609, 908235},
  {0x260A, 81971},
  {0x260B, 270365},
  {0x260C, 208913},
  {0x260D, 800947},
  {0x260E, 115905},
  {0x260F, 986511},
  {0x2610, 87150},
  {0x2611, 87241},
  {0x2612, 87312},
  {0x2613, 846322},
  {0x2614, 954845},
  {0x2615, 524117},
  {0x2616, 985773},
  {0x2617, 115539},
  {0x2618, 853195},
  {0x2619, 833348},
  {0x261A, 113957},
  {0x261B, 115003},
  {0x261C, 985111},
  {0x261D, 986680},
  {0x261E, 985589},
  {0x261F, 984489},
  {0x2620, 891591},
  {0x2621, 176352},
  {0x2622, 830136},
  {0x2623, 111291},
  {0x2624, 153121},
  {0x2625, 24715},
  {0x2626, 802817},
  {0x2627, 183332},
  {0x2628, 214931},
  {0x2629, 214912},
  {0x262A, 906636},
  {0x262B, 443618},
  {0x262C, 173},
  {0x262D, 491157},
  {0x262E, 819168},
  {0x262F, 1010241},
  {0x2630, 950437},
  {0x2631, 950456},
  {0x2632, 950420},
  {0x2633, 950494},
  {0x2634, 950532},
  {0x2635, 950514},
  {0x2636, 950473},
  {0x2637, 950402},
  {0x2638, 982474},
  {0x2639, 984820},
  {0x263A, 985847},
  {0x263B, 115748},
  {0x263C, 986464},
  {0x263D, 443986},
  {0x263E, 582366},
  {0x263F, 717839},
  {0x2640, 443690},
  {0x2641, 293719},
  {0x2642, 663131},
  {0x2643, 538514},
  {0x2644, 848156},
  {0x2645, 962446},
  {0x2646, 765309},
  {0x2647, 825772},
  {0x2648, 79094},
  {0x2649, 929831},
  {0x264A, 449696},
  {0x264B, 173621},
  {0x264C, 634074},
  {0x264D, 976164},
  {0x264E, 636480},
  {0x264F, 850123},
  {0x2650, 846269},
  {0x2651, 173669},
  {0x2652, 27912},
  {0x2653, 823108},
  {0x2654, 982769},
  {0x2655, 983401},
  {0x2656, 983514},
  {0x2657, 982584},
  {0x2658, 982879},
  {0x2659, 983291},
  {0x265A, 111577},
  {0x265B, 112209},
  {0x265C, 112322},
  {0x265D, 111392},
  {0x265E, 111687},
  {0x265F, 112099},
  {0x2660, 115781},
  {0x2661, 984881},
  {0x2662, 984404},
  {0x2663, 112714},
  {0x2664, 985866},
  {0x2665, 113603},
  {0x2666, 113179},
  {0x2667, 984067},
  {0x2668, 524149},
  {0x2669, 829913},
  {0x266A, 426677},
  {0x266B, 106685},
  {0x266C, 106705},
  {0x266D, 744255},
  {0x266E, 744271},
  {0x266F, 744290},
  {0x2670, 982444},
  {0x2671, 293812},
  {0x2672, 955113},
  {0x2673, 830623},
  {0x2674, 830660},
  {0x2675, 830697},
  {0x2676, 830734},
  {0x2677, 830771},
  {0x2678, 830808},
  {0x2679, 830845},
  {0x267A, 830584},
  {0x267B, 116046},
  {0x267C, 830562},
  {0x267D, 817488},
  {0x267E, 819373},
  {0x267F, 982490},
  {0x2680, 276603},
  {0x2681, 276614},
  {0x2682, 276625},
  {0x2683, 276636},
  {0x2684, 276647},
  {0x2685, 276658},
  {0x2686, 983829},
  {0x2687, 983963},
  {0x2688, 112680},
  {0x2689, 112647},
  {0x268A, 742192},
  {0x268B, 742210},
  {0x268C, 277264},
  {0x268D, 277383},
  {0x268E, 277360},
  {0x268F, 277288},
  {0x2690, 984686},
  {0x2691, 113499},
  {0x2692, 491141},
  {0x2693, 24596},
  {0x2694, 215123},
  {0x2695, 906566},
  {0x2696, 850071},
  {0x2697, 8805},
  {0x2698, 444313},
  {0x2699, 449646},
  {0x269A, 906587},
  {0x269B, 82138},
  {0x269C, 444261},
  {0x269D, 808446},
  {0x269E, 937552},
  {0x269F, 937524},
  {0x26A0, 982040},
  {0x26A1, 521183},
  {0x26A2, 287127},
  {0x26A3, 287147},
  {0x26A4, 535483},
  {0x26A5, 663110},
  {0x26A6, 663183},
  {0x26A7, 663141},
  {0x26A8, 975321},
  {0x26A9, 523623},
  {0x26AA, 708887},
  {0x26AB, 708131},
  {0x26AC, 708774},
  {0x26AD, 666883},
  {0x26AE, 280611},
  {0x26AF, 955140},
  {0x26B0, 196545},
  {0x26B1, 448080},
  {0x26B2, 765380},
  {0x26B3, 176465},
  {0x26B4, 812052},
  {0x26B5, 538509},
  {0x26B6, 975993},
  {0x26B7, 183381},
  {0x26B8, 114798},
  {0x26B9, 853085},
  {0x26BA, 850763},
  {0x26BB, 830015},
  {0x26BC, 853046},
  {0x26BD, 894114},
  {0x26BE, 104558},
  {0x26BF, 905165},
  {0x26C0, 984635},
  {0x26C1, 984615},
  {0x26C2, 113466},
  {0x26C3, 113446},
  {0x26C4, 894093},
  {0x26C5, 908239},
  {0x26C6, 830179},
  {0x26C7, 115767},
  {0x26C8, 938104},
  {0x26C9, 953705},
  {0x26CA, 953289},
  {0x26CB, 984380},
  {0x26CC, 215138},
  {0x26CD, 278759},
  {0x26CE, 800937},
  {0x26CF, 822946},
  {0x26D0, 173679},
  {0x26D1, 512839},
  {0x26D2, 184840},
  {0x26D3, 176471},
  {0x26D4, 771677},
  {0x26D5, 9181},
  {0x26D6, 116015},
  {0x26D7, 986616},
  {0x26D8, 113900},
  {0x26D9, 985054},
  {0x26DA, 290188},
  {0x26DB, 509270},
  {0x26DC, 626420},
  {0x26DD, 906211},
  {0x26DE, 443562},
  {0x26DF, 116003},
  {0x26E0, 832605},
  {0x26E1, 832629},
  {0x26E2, 82069},
  {0x26E3, 506615},
  {0x26E4, 819273},
  {0x26E5, 836969},
  {0x26E6, 629506},
  {0x26E7, 536062},
  {0x26E8, 112730},
  {0x26E9, 856138},
  {0x26EA, 184093},
  {0x26EB, 174661},
  {0x26EC, 523233},
  {0x26ED, 449669},
  {0x26EE, 449651},
  {0x26EF, 665275},
  {0x26F0, 742415},
  {0x26F1, 954826},
  {0x26F2, 444638},
  {0x26F3, 444196},
  {0x26F4, 443749},
  {0x26F5, 846281},
  {0x26F6, 900303},
  {0x26F7, 891579},
  {0x26F8, 524511},
  {0x26F9, 819610},
  {0x26FA, 932313},
  {0x26FB, 536204},
  {0x26FC, 505973},
  {0x26FD, 445152},
  {0x26FE, 251544},
  {0x26FF, 984697},
  {0x2700, 115502},
  {0x2701, 956286},
  {0x2702, 115524},
  {0x2703, 652872},
  {0x2704, 985741},
  {0x2705, 984898},
  {0x2706, 929959},
  {0x2707, 929820},
  {0x2708, 4870},
  {0x2709, 428907},
  {0x270A, 830271},
  {0x270B, 830283},
  {0x270C, 976014},
  {0x270D, 988927},
  {0x270E, 655849},
  {0x270F, 819245},
  {0x2710, 959215},
  {0x2711, 985449},
  {0x2712, 114835},
  {0x2713, 179396},
  {0x2714, 506529},
  {0x2715, 744166},
  {0x2716, 507894},
  {0x2717, 87346},
  {0x2718, 506281},
  {0x2719, 807572},
  {0x271A, 507362},
  {0x271B, 800641},
  {0x271C, 508072},
  {0x271D, 598973},
  {0x271E, 853115},
  {0x271F, 808425},
  {0x2720, 663205},
  {0x2721, 906666},
  {0x2722, 444820},
  {0x2723, 444647},
  {0x2724, 507327},
  {0x2725, 444676},
  {0x2726, 113538},
  {0x2727, 984796},
  {0x2728, 899038},
  {0x2729, 907020},
  {0x272A, 191670},
  {0x272B, 800618},
  {0x272C, 111368},
  {0x272D, 807352},
  {0x272E, 508096},
  {0x272F, 823094},
  {0x2730, 853142},
  {0x2731, 506266},
  {0x2732, 800597},
  {0x2733, 426614},
  {0x2734, 426487},
  {0x2735, 426512},
  {0x2736, 891427},
  {0x2737, 426540},
  {0x2738, 507072},
  {0x2739, 953737},
  {0x273A, 891526},
  {0x273B, 929923},
  {0x273C, 800659},
  {0x273D, 508869},
  {0x273E, 891389},
  {0x273F, 113510},
  {0x2740, 984744},
  {0x2741, 426448},
  {0x2742, 191403},
  {0x2743, 508900},
  {0x2744, 894075},
  {0x2745, 945021},
  {0x2746, 506546},
  {0x2747, 899030},
  {0x2748, 508815},
  {0x2749, 87105},
  {0x274A, 426636},
  {0x274B, 507143},
  {0x274C, 214901},
  {0x274D, 853093},
  {0x274E, 763677},
  {0x274F, 655778},
  {0x2750, 959096},
  {0x2751, 656476},
  {0x2752, 959714},
  {0x2753, 114922},
  {0x2754, 985508},
  {0x2755, 984654},
  {0x2756, 113128},
  {0x2757, 507240},
  {0x2758, 636922},
  {0x2759, 708867},
  {0x275A, 509196},
  {0x275B, 508555},
  {0x275C, 508512},
  {0x275D, 506853},
  {0x275E, 506810},
  {0x275F, 507780},
  {0x2760, 507733},
  {0x2761, 251725},
  {0x2762, 507208},
  {0x2763, 507380},
  {0x2764, 506394},
  {0x2765, 842949},
  {0x2766, 444300},
  {0x2767, 842921},
  {0x2768, 708471},
  {0x2769, 708610},
  {0x276A, 708265},
  {0x276B, 708308},
  {0x276C, 708504},
  {0x276D, 708644},
  {0x276E, 507547},
  {0x276F, 508210},
  {0x2770, 507504},
  {0x2771, 508166},
  {0x2772, 636709},
  {0x2773, 636763},
  {0x2774, 708436},
  {0x2775, 708574},
  {0x2776, 277985},
  {0x2777, 278129},
  {0x2778, 278092},
  {0x2779, 277913},
  {0x277A, 277877},
  {0x277B, 278057},
  {0x277C, 278020},
  {0x277D, 277840},
  {0x277E, 277949},
  {0x277F, 278164},
  {0x2780, 277575},
  {0x2781, 277727},
  {0x2782, 277688},
  {0x2783, 277499},
  {0x2784, 277461},
  {0x2785, 277651},
  {0x2786, 277612},
  {0x2787, 277422},
  {0x2788, 277537},
  {0x2789, 277802},
  {0x278A, 278389},
  {0x278B, 278577},
  {0x278C, 278529},
  {0x278D, 278295},
  {0x278E, 278248},
  {0x278F, 278483},
  {0x2790, 278435},
  {0x2791, 278200},
  {0x2792, 278342},
  {0x2793, 278670},
  {0x2794, 509378},
  {0x2795, 508150},
  {0x2796, 507877},
  {0x2797, 506772},
  {0x2798, 508660},
  {0x2799, 508261},
  {0x279A, 507917},
  {0x279B, 290131},
  {0x279C, 508390},
  {0x279D, 950286},
  {0x279E, 508940},
  {0x279F, 269329},
  {0x27A0, 506726},
  {0x27A1, 115375},
  {0x27A2, 937945},
  {0x27A3, 937744},
  {0x27A4, 115398},
  {0x27A5, 506296},
  {0x27A6, 506346},
  {0x27A7, 906523},
  {0x27A8, 506681},
  {0x27A9, 837427},
  {0x27AA, 629949},
  {0x27AB, 83354},
  {0x27AC, 445071},
  {0x27AD, 507827},
  {0x27AE, 509046},
  {0x27AF, 773673},
  {0x27B0, 251667},
  {0x27B1, 773766},
  {0x27B2, 185869},
  {0x27B3, 986962},
  {0x27B4, 116526},
  {0x27B5, 116493},
  {0x27B6, 116460},
  {0x27B7, 506490},
  {0x27B8, 506451},
  {0x27B9, 506412},
  {0x27BA, 929890},
  {0x27BB, 508829},
  {0x27BC, 982400},
  {0x27BD, 509215},
  {0x27BE, 800906},
  {0x27BF, 285467},
  {0x27C0, 937455},
  {0x27C1, 986569},
  {0x27C2, 819394},
  {0x27C3, 800812},
  {0x27C4, 800824},
  {0x27C5, 628105},
  {0x27C6, 835432},
  {0x27C7, 801000},
  {0x27C8, 832805},
  {0x27C9, 911711},
  {0x27CA, 974758},
  {0x27CB, 689179},
  {0x27CC, 652273},
  {0x27CD, 681908},
  {0x27CE, 906012},
  {0x27CF, 906032},
  {0x27D0, 984423},
  {0x27D1, 24603},
  {0x27D2, 427560},
  {0x27D3, 655750},
  {0x27D4, 957501},
  {0x27D5, 627530},
  {0x27D6, 835281},
  {0x27D7, 445210},
  {0x27D8, 582266},
  {0x27D9, 579948},
  {0x27DA, 625805},
  {0x27DB, 625837},
  {0x27DC, 627412},
  {0x27DD, 652509},
  {0x27DE, 652348},
  {0x27DF, 955629},
  {0x27E0, 656805},
  {0x27E1, 984083},
  {0x27E2, 984111},
  {0x27E3, 984159},
  {0x27E4, 986090},
  {0x27E5, 986200},
  {0x27E6, 686887},
  {0x27E7, 689091},
  {0x27E8, 686776},
  {0x27E9, 688977},
  {0x27EA, 686808},
  {0x27EB, 689010},
  {0x27EC, 686926},
  {0x27ED, 689131},
  {0x27EE, 686847},
  {0x27EF, 689050},
  {0x27F0, 961223},
  {0x27F1, 289110},
  {0x27F2, 24870},
  {0x27F3, 195783},
  {0x27F4, 834160},
  {0x27F5, 652363},
  {0x27F6, 652525},
  {0x27F7, 652297},
  {0x27F8, 652414},
  {0x27F9, 652578},
  {0x27FA, 652319},
  {0x27FB, 652384},
  {0x27FC, 652547},
  {0x27FD, 652442},
  {0x27FE, 652607},
  {0x27FF, 652645},
  {0x2800, 135091},
  {0x2801, 135113},
  {0x2802, 138505},
  {0x2803, 135136},
  {0x2804, 140169},
  {0x2805, 136864},
  {0x2806, 138528},
  {0x2807, 135160},
  {0x2808, 140985},
  {0x2809, 137712},
  {0x280A, 139376},
  {0x280B, 136040},
  {0x280C, 140192},
  {0x280D, 136888},
  {0x280E, 138552},
  {0x280F, 135185},
  {0x2810, 141385},
  {0x2811, 138128},
  {0x2812, 139792},
  {0x2813, 136472},
  {0x2814, 140608},
  {0x2815, 137320},
  {0x2816, 138984},
  {0x2817, 135633},
  {0x2818, 141008},
  {0x2819, 137736},
  {0x281A, 139400},
  {0x281B, 136065},
  {0x281C, 140216},
  {0x281D, 136913},
  {0x281E, 138577},
  {0x281F, 135211},
  {0x2820, 141581},
  {0x2821, 138332},
  {0x2822, 139996},
  {0x2823, 136684},
  {0x2824, 140812},
  {0x2825, 137532},
  {0x2826, 139196},
  {0x2827, 135853},
  {0x2828, 141212},
  {0x2829, 137948},
  {0x282A, 139612},
  {0x282B, 136285},
  {0x282C, 140428},
  {0x282D, 137133},
  {0x282E, 138797},
  {0x282F, 135439},
  {0x2830, 141408},
  {0x2831, 138152},
  {0x2832, 139816},
  {0x2833, 136497},
  {0x2834, 140632},
  {0x2835, 137345},
  {0x2836, 139009},
  {0x2837, 135659},
  {0x2838, 141032},
  {0x2839, 137761},
  {0x283A, 139425},
  {0x283B, 136091},
  {0x283C, 140241},
  {0x283D, 136939},
  {0x283E, 138603},
  {0x283F, 135238},
  {0x2840, 141677},
  {0x2841, 138432},
  {0x2842, 140096},
  {0x2843, 136788},
  {0x2844, 140912},
  {0x2845, 137636},
  {0x2846, 139300},
  {0x2847, 135961},
  {0x2848, 141312},
  {0x2849, 138052},
  {0x284A, 139716},
  {0x284B, 136393},
  {0x284C, 140532},
  {0x284D, 137241},
  {0x284E, 138905},
  {0x284F, 135551},
  {0x2850, 141508},
  {0x2851, 138256},
  {0x2852, 139920},
  {0x2853, 136605},
  {0x2854, 140736},
  {0x2855, 137453},
  {0x2856, 139117},
  {0x2857, 135771},
  {0x2858, 141136},
  {0x2859, 137869},
  {0x285A, 139533},
  {0x285B, 136203},
  {0x285C, 140349},
  {0x285D, 137051},
  {0x285E, 138715},
  {0x285F, 135354},
  {0x2860, 141604},
  {0x2861, 138356},
  {0x2862, 140020},
  {0x2863, 136709},
  {0x2864, 140836},
  {0x2865, 137557},
  {0x2866, 139221},
  {0x2867, 135879},
  {0x2868, 141236},
  {0x2869, 137973},
  {0x286A, 139637},
  {0x286B, 136311},
  {0x286C, 140453},
  {0x286D, 137159},
  {0x286E, 138823},
  {0x286F, 135466},
  {0x2870, 141432},
  {0x2871, 138177},
  {0x2872, 139841},
  {0x2873, 136523},
  {0x2874, 140657},
  {0x2875, 137371},
  {0x2876, 139035},
  {0x2877, 135686},
  {0x2878, 141057},
  {0x2879, 137787},
  {0x287A, 139451},
  {0x287B, 136118},
  {0x287C, 140267},
  {0x287D, 136966},
  {0x287E, 138630},
  {0x287F, 135266},
  {0x2880, 141724},
  {0x2881, 138481},
  {0x2882, 140145},
  {0x2883, 136839},
  {0x2884, 140961},
  {0x2885, 137687},
  {0x2886, 139351},
  {0x2887, 136014},
  {0x2888, 141361},
  {0x2889, 138103},
  {0x288A, 139767},
  {0x288B, 136446},
  {0x288C, 140583},
  {0x288D, 137294},
  {0x288E, 138958},
  {0x288F, 135606},
  {0x2890, 141557},
  {0x2891, 138307},
  {0x2892, 139971},
  {0x2893, 136658},
  {0x2894, 140787},
  {0x2895, 137506},
  {0x2896, 139170},
  {0x2897, 135826},
  {0x2898, 141187},
  {0x2899, 137922},
  {0x289A, 139586},
  {0x289B, 136258},
  {0x289C, 140402},
  {0x289D, 137106},
  {0x289E, 138770},
  {0x289F, 135411},
  {0x28A0, 141653},
  {0x28A1, 138407},
  {0x28A2, 140071},
  {0x28A3, 136762},
  {0x28A4, 140887},
  {0x28A5, 137610},
  {0x28A6, 139274},
  {0x28A7, 135934},
  {0x28A8, 141287},
  {0x28A9, 138026},
  {0x28AA, 139690},
  {0x28AB, 136366},
  {0x28AC, 140506},
  {0x28AD, 137214},
  {0x28AE, 138878},
  {0x28AF, 135523},
  {0x28B0, 141483},
  {0x28B1, 138230},
  {0x28B2, 139894},
  {0x28B3, 136578},
  {0x28B4, 140710},
  {0x28B5, 137426},
  {0x28B6, 139090},
  {0x28B7, 135743},
  {0x28B8, 141110},
  {0x28B9, 137842},
  {0x28BA, 139506},
  {0x28BB, 136175},
  {0x28BC, 140322},
  {0x28BD, 137023},
  {0x28BE, 138687},
  {0x28BF, 135325},
  {0x28C0, 141700},
  {0x28C1, 138456},
  {0x28C2, 140120},
  {0x28C3, 136813},
  {0x28C4, 140936},
  {0x28C5, 137661},
  {0x28C6, 139325},
  {0x28C7, 135987},
  {0x28C8, 141336},
  {0x28C9, 138077},
  {0x28CA, 139741},
  {0x28CB, 136419},
  {0x28CC, 140557},
  {0x28CD, 137267},
  {0x28CE, 138931},
  {0x28CF, 135578},
  {0x28D0, 141532},
  {0x28D1, 138281},
  {0x28D2, 139945},
  {0x28D3, 136631},
  {0x28D4, 140761},
  {0x28D5, 137479},
  {0x28D6, 139143},
  {0x28D7, 135798},
  {0x28D8, 141161},
  {0x28D9, 137895},
  {0x28DA, 139559},
  {0x28DB, 136230},
  {0x28DC, 140375},
  {0x28DD, 137078},
  {0x28DE, 138742},
  {0x28DF, 135382},
  {0x28E0, 141628},
  {0x28E1, 138381},
  {0x28E2, 140045},
  {0x28E3, 136735},
  {0x28E4, 140861},
  {0x28E5, 137583},
  {0x28E6, 139247},
  {0x28E7, 135906},
  {0x28E8, 141261},
  {0x28E9, 137999},
  {0x28EA, 139663},
  {0x28EB, 136338},
  {0x28EC, 140479},
  {0x28ED, 137186},
  {0x28EE, 138850},
  {0x28EF, 135494},
  {0x28F0, 141457},
  {0x28F1, 138203},
  {0x28F2, 139867},
  {0x28F3, 136550},
  {0x28F4, 140683},
  {0x28F5, 137398},
  {0x28F6, 139062},
  {0x28F7, 135714},
  {0x28F8, 141083},
  {0x28F9, 137814},
  {0x28FA, 139478},
  {0x28FB, 136146},
  {0x28FC, 140294},
  {0x28FD, 136994},
  {0x28FE, 138658},
  {0x28FF, 135295},
  {0x2900, 841507},
  {0x2901, 841235},
  {0x2902, 631711},
  {0x2903, 839281},
  {0x2904, 627925},
  {0x2905, 841198},
  {0x2906, 631644},
  {0x2907, 839212},
  {0x2908, 288122},
  {0x2909, 960314},
  {0x290A, 961963},
  {0x290B, 289923},
  {0x290C, 631783},
  {0x290D, 839355},
  {0x290E, 633528},
  {0x290F, 841141},
  {0x2910, 841556},
  {0x2911, 838236},
  {0x2912, 960219},
  {0x2913, 287983},
  {0x2914, 838817},
  {0x2915, 838762},
  {0x2916, 841291},
  {0x2917, 841395},
  {0x2918, 841329},
  {0x2919, 631413},
  {0x291A, 838973},
  {0x291B, 631755},
  {0x291C, 839326},
  {0x291D, 630694},
  {0x291E, 838163},
  {0x291F, 630411},
  {0x2920, 837921},
  {0x2921, 772748},
  {0x2922, 771965},
  {0x2923, 772960},
  {0x2924, 772164},
  {0x2925, 896449},
  {0x2926, 896807},
  {0x2927, 772797},
  {0x2928, 772014},
  {0x2929, 896315},
  {0x292A, 896743},
  {0x292B, 841832},
  {0x292C, 443520},
  {0x292D, 896353},
  {0x292E, 772095},
  {0x292F, 443477},
  {0x2930, 841874},
  {0x2931, 772052},
  {0x2932, 772835},
  {0x2933, 982165},
  {0x2934, 81827},
  {0x2935, 81728},
  {0x2936, 81631},
  {0x2937, 81679},
  {0x2938, 837463},
  {0x2939, 629984},
  {0x293A, 948037},
  {0x293B, 123656},
  {0x293C, 948103},
  {0x293D, 948065},
  {0x293E, 656401},
  {0x293F, 654681},
  {0x2940, 24750},
  {0x2941, 195675},
  {0x2942, 837758},
  {0x2943, 630251},
  {0x2944, 856378},
  {0x2945, 838576},
  {0x2946, 631026},
  {0x2947, 838112},
  {0x2948, 627692},
  {0x2949, 962009},
  {0x294A, 626271},
  {0x294B, 626234},
  {0x294C, 955352},
  {0x294D, 955315},
  {0x294E, 626308},
  {0x294F, 955389},
  {0x2950, 626195},
  {0x2951, 955279},
  {0x2952, 632408},
  {0x2953, 839992},
  {0x2954, 961074},
  {0x2955, 288951},
  {0x2956, 632104},
  {0x2957, 839682},
  {0x2958, 960959},
  {0x2959, 288830},
  {0x295A, 632368},
  {0x295B, 839951},
  {0x295C, 961033},
  {0x295D, 288908},
  {0x295E, 632062},
  {0x295F, 839639},
  {0x2960, 960919},
  {0x2961, 288788},
  {0x2962, 632182},
  {0x2963, 960849},
  {0x2964, 839879},
  {0x2965, 288642},
  {0x2966, 632299},
  {0x2967, 631940},
  {0x2968, 839762},
  {0x2969, 839516},
  {0x296A, 632252},
  {0x296B, 632013},
  {0x296C, 839831},
  {0x296D, 839589},
  {0x296E, 960777},
  {0x296F, 288716},
  {0x2970, 834497},
  {0x2971, 429197},
  {0x2972, 945106},
  {0x2973, 630296},
  {0x2974, 837803},
  {0x2975, 837626},
  {0x2976, 635661},
  {0x2977, 630528},
  {0x2978, 460150},
  {0x2979, 907498},
  {0x297A, 630562},
  {0x297B, 911331},
  {0x297C, 626704},
  {0x297D, 834613},
  {0x297E, 955581},
  {0x297F, 287218},
  {0x2980, 950857},
  {0x2981, 1010593},
  {0x2982, 1010609},
  {0x2983, 629071},
  {0x2984, 836537},
  {0x2985, 629126},
  {0x2986, 836594},
  {0x2987, 1010348},
  {0x2988, 1010478},
  {0x2989, 1010316},
  {0x298A, 1010445},
  {0x298B, 628561},
  {0x298C, 835988},
  {0x298D, 628517},
  {0x298E, 835895},
  {0x298F, 628470},
  {0x2990, 835943},
  {0x2991, 626082},
  {0x2992, 833936},
  {0x2993, 626110},
  {0x2994, 834129},
  {0x2995, 285628},
  {0x2996, 285984},
  {0x2997, 626373},
  {0x2998, 834251},
  {0x2999, 285039},
  {0x299A, 975639},
  {0x299B, 704900},
  {0x299C, 834036},
  {0x299D, 705480},
  {0x299E, 24635},
  {0x299F, 119},
  {0x29A0, 899353},
  {0x29A1, 899382},
  {0x29A2, 953254},
  {0x29A3, 832968},
  {0x29A4, 24655},
  {0x29A5, 832983},
  {0x29A6, 785397},
  {0x29A7, 785370},
  {0x29A8, 705401},
  {0x29A9, 705335},
  {0x29AA, 704996},
  {0x29AB, 704928},
  {0x29AC, 705268},
  {0x29AD, 705133},
  {0x29AE, 705199},
  {0x29AF, 705065},
  {0x29B0, 833118},
  {0x29B1, 428735},
  {0x29B2, 428791},
  {0x29B3, 428758},
  {0x29B4, 428703},
  {0x29B5, 184223},
  {0x29B6, 191628},
  {0x29B7, 191442},
  {0x29B8, 191514},
  {0x29B9, 191459},
  {0x29BA, 184107},
  {0x29BB, 184374},
  {0x29BC, 184571},
  {0x29BD, 955169},
  {0x29BE, 191649},
  {0x29BF, 184656},
  {0x29C0, 189985},
  {0x29C1, 185191},
  {0x29C2, 184336},
  {0x29C3, 184401},
  {0x29C4, 906181},
  {0x29C5, 904932},
  {0x29C6, 903349},
  {0x29C7, 906312},
  {0x29C8, 906345},
  {0x29C9, 954005},
  {0x29CA, 950178},
  {0x29CB, 950263},
  {0x29CC, 846232},
  {0x29CD, 950232},
  {0x29CE, 836357},
  {0x29CF, 628903},
  {0x29D0, 974652},
  {0x29D1, 124783},
  {0x29D2, 124811},
  {0x29D3, 111355},
  {0x29D4, 945282},
  {0x29D5, 945309},
  {0x29D6, 985001},
  {0x29D7, 113659},
  {0x29D8, 629209},
  {0x29D9, 836680},
  {0x29DA, 626679},
  {0x29DB, 834587},
  {0x29DC, 530587},
  {0x29DD, 943640},
  {0x29DE, 532759},
  {0x29DF, 286413},
  {0x29E0, 902807},
  {0x29E1, 530633},
  {0x29E2, 856711},
  {0x29E3, 429265},
  {0x29E4, 429298},
  {0x29E5, 524534},
  {0x29E6, 457842},
  {0x29E7, 937277},
  {0x29E8, 287632},
  {0x29E9, 287676},
  {0x29EA, 113198},
  {0x29EB, 114324},
  {0x29EC, 983857},
  {0x29ED, 112618},
  {0x29EE, 429766},
  {0x29EF, 429687},
  {0x29F0, 429739},
  {0x29F1, 429660},
  {0x29F2, 429713},
  {0x29F3, 429634},
  {0x29F4, 843254},
  {0x29F5, 832780},
  {0x29F6, 895372},
  {0x29F7, 832838},
  {0x29F8, 111251},
  {0x29F9, 111231},
  {0x29FA, 285894},
  {0x29FB, 950752},
  {0x29FC, 629616},
  {0x29FD, 837059},
  {0x29FE, 945337},
  {0x29FF, 725461},
  {0x2A00, 758123},
  {0x2A01, 758150},
  {0x2A02, 758178},
  {0x2A03, 758403},
  {0x2A04, 758433},
  {0x2A05, 758291},
  {0x2A06, 758326},
  {0x2A07, 954024},
  {0x2A08, 954049},
  {0x2A09, 758370},
  {0x2A0A, 737451},
  {0x2A0B, 908211},
  {0x2A0C, 829862},
  {0x2A0D, 443872},
  {0x2A0E, 535172},
  {0x2A0F, 535125},
  {0x2A10, 191734},
  {0x2A11, 24904},
  {0x2A12, 638336},
  {0x2A13, 638387},
  {0x2A14, 638296},
  {0x2A15, 535092},
  {0x2A16, 829926},
  {0x2A17, 535227},
  {0x2A18, 535289},
  {0x2A19, 535200},
  {0x2A1A, 535337},
  {0x2A1B, 535267},
  {0x2A1C, 535314},
  {0x2A1D, 538486},
  {0x2A1E, 580002},
  {0x2A1F, 1010509},
  {0x2A20, 1010539},
  {0x2A21, 1010564},
  {0x2A22, 825639},
  {0x2A23, 825575},
  {0x2A24, 825702},
  {0x2A25, 825614},
  {0x2A26, 825729},
  {0x2A27, 825673},
  {0x2A28, 825545},
  {0x2A29, 725319},
  {0x2A2A, 725347},
  {0x2A2B, 725373},
  {0x2A2C, 725402},
  {0x2A2D, 825462},
  {0x2A2E, 825492},
  {0x2A2F, 973279},
  {0x2A30, 744097},
  {0x2A31, 744132},
  {0x2A32, 850725},
  {0x2A33, 893453},
  {0x2A34, 743984},
  {0x2A35, 744024},
  {0x2A36, 190017},
  {0x2A37, 743947},
  {0x2A38, 185066},
  {0x2A39, 825523},
  {0x2A3A, 725296},
  {0x2A3B, 744065},
  {0x2A3C, 535369},
  {0x2A3D, 837582},
  {0x2A3E, 1010411},
  {0x2A3F, 9239},
  {0x2A40, 535650},
  {0x2A41, 955058},
  {0x2A42, 955080},
  {0x2A43, 535702},
  {0x2A44, 535672},
  {0x2A45, 955036},
  {0x2A46, 954976},
  {0x2A47, 535576},
  {0x2A48, 954941},
  {0x2A49, 535541},
  {0x2A4A, 955001},
  {0x2A4B, 535601},
  {0x2A4C, 196350},
  {0x2A4D, 196132},
  {0x2A4E, 286043},
  {0x2A4F, 286070},
  {0x2A50, 196375},
  {0x2A51, 651843},
  {0x2A52, 652069},
  {0x2A53, 285665},
  {0x2A54, 285684},
  {0x2A55, 953948},
  {0x2A56, 953977},
  {0x2A57, 892040},
  {0x2A58, 892022},
  {0x2A59, 652034},
  {0x2A5A, 651968},
  {0x2A5B, 652190},
  {0x2A5C, 651935},
  {0x2A5D, 652158},
  {0x2A5E, 651870},
  {0x2A5F, 651997},
  {0x2A60, 651902},
  {0x2A61, 893391},
  {0x2A62, 652095},
  {0x2A63, 652126},
  {0x2A64, 1010282},
  {0x2A65, 1010378},
  {0x2A66, 429377},
  {0x2A67, 524568},
  {0x2A68, 950585},
  {0x2A69, 950635},
  {0x2A6A, 945144},
  {0x2A6B, 945174},
  {0x2A6C, 887886},
  {0x2A6D, 208874},
  {0x2A6E, 429487},
  {0x2A6F, 9142},
  {0x2A70, 27818},
  {0x2A71, 429169},
  {0x2A72, 825434},
  {0x2A73, 429232},
  {0x2A74, 285448},
  {0x2A75, 953839},
  {0x2A76, 937424},
  {0x2A77, 429436},
  {0x2A78, 429537},
  {0x2A79, 636384},
  {0x2A7A, 460989},
  {0x2A7B, 636432},
  {0x2A7C, 461043},
  {0x2A7D, 636173},
  {0x2A7E, 460726},
  {0x2A7F, 636299},
  {0x2A80, 460860},
  {0x2A81, 636203},
  {0x2A82, 460759},
  {0x2A83, 636248},
  {0x2A84, 460807},
  {0x2A85, 636073},
  {0x2A86, 460617},
  {0x2A87, 635868},
  {0x2A88, 460367},
  {0x2A89, 635838},
  {0x2A8A, 460334},
  {0x2A8B, 635555},
  {0x2A8C, 460044},
  {0x2A8D, 635736},
  {0x2A8E, 460229},
  {0x2A8F, 635693},
  {0x2A90, 460186},
  {0x2A91, 635608},
  {0x2A92, 460097},
  {0x2A93, 635769},
  {0x2A94, 460265},
  {0x2A95, 891700},
  {0x2A96, 891618},
  {0x2A97, 891730},
  {0x2A98, 891651},
  {0x2A99, 286472},
  {0x2A9A, 286435},
  {0x2A9B, 286551},
  {0x2A9C, 286506},
  {0x2A9D, 887932},
  {0x2A9E, 887908},
  {0x2A9F, 887844},
  {0x2AA0, 887799},
  {0x2AA1, 285810},
  {0x2AA2, 285783},
  {0x2AA3, 285834},
  {0x2AA4, 460936},
  {0x2AA5, 460409},
  {0x2AA6, 635966},
  {0x2AA7, 460504},
  {0x2AA8, 635992},
  {0x2AA9, 460533},
  {0x2AAA, 893415},
  {0x2AAB, 582320},
  {0x2AAC, 893428},
  {0x2AAD, 582332},
  {0x2AAE, 429348},
  {0x2AAF, 826426},
  {0x2AB0, 907999},
  {0x2AB1, 826465},
  {0x2AB2, 908038},
  {0x2AB3, 826336},
  {0x2AB4, 907909},
  {0x2AB5, 826398},
  {0x2AB6, 907971},
  {0x2AB7, 826305},
  {0x2AB8, 907878},
  {0x2AB9, 826363},
  {0x2ABA, 907936},
  {0x2ABB, 285906},
  {0x2ABC, 286148},
  {0x2ABD, 907787},
  {0x2ABE, 911738},
  {0x2ABF, 907841},
  {0x2AC0, 911796},
  {0x2AC1, 907803},
  {0x2AC2, 911756},
  {0x2AC3, 907722},
  {0x2AC4, 911642},
  {0x2AC5, 907612},
  {0x2AC6, 911524},
  {0x2AC7, 907669},
  {0x2AC8, 911585},
  {0x2AC9, 907580},
  {0x2ACA, 911490},
  {0x2ACB, 907640},
  {0x2ACC, 911554},
  {0x2ACD, 901174},
  {0x2ACE, 902381},
  {0x2ACF, 196250},
  {0x2AD0, 196290},
  {0x2AD1, 196264},
  {0x2AD2, 196306},
  {0x2AD3, 907548},
  {0x2AD4, 911362},
  {0x2AD5, 907528},
  {0x2AD6, 911384},
  {0x2AD7, 911455},
  {0x2AD8, 911408},
  {0x2AD9, 427531},
  {0x2ADA, 823132},
  {0x2ADB, 950153},
  {0x2ADC, 444615},
  {0x2ADD, 771883},
  {0x2ADE, 856362},
  {0x2ADF, 856317},
  {0x2AE0, 856449},
  {0x2AE1, 819408},
  {0x2AE2, 974722},
  {0x2AE3, 286336},
  {0x2AE4, 974687},
  {0x2AE5, 286251},
  {0x2AE6, 652227},
  {0x2AE7, 856333},
  {0x2AE8, 856499},
  {0x2AE9, 856463},
  {0x2AEA, 285499},
  {0x2AEB, 286216},
  {0x2AEC, 286111},
  {0x2AED, 833086},
  {0x2AEE, 280908},
  {0x2AEF, 975156},
  {0x2AF0, 975188},
  {0x2AF1, 287272},
  {0x2AF2, 812960},
  {0x2AF3, 812992},
  {0x2AF4, 950821},
  {0x2AF5, 950923},
  {0x2AF6, 950549},
  {0x2AF7, 950728},
  {0x2AF8, 950701},
  {0x2AF9, 286638},
  {0x2AFA, 286593},
  {0x2AFB, 950777},
  {0x2AFC, 580248},
  {0x2AFD, 286019},
  {0x2AFE, 986826},
  {0x2AFF, 758464},
  {0x2B00, 772418},
  {0x2B01, 773214},
  {0x2B02, 896703},
  {0x2B03, 897061},
  {0x2B04, 628082},
  {0x2B05, 631477},
  {0x2B06, 960630},
  {0x2B07, 288454},
  {0x2B08, 772191},
  {0x2B09, 772987},
  {0x2B0A, 896476},
  {0x2B0B, 896834},
  {0x2B0C, 627842},
  {0x2B0D, 955465},
  {0x2B0E, 838865},
  {0x2B0F, 838901},
  {0x2B10, 631308},
  {0x2B11, 631343},
  {0x2B12, 903085},
  {0x2B13, 902777},
  {0x2B14, 903198},
  {0x2B15, 902930},
  {0x2B16, 276516},
  {0x2B17, 276545},
  {0x2B18, 276575},
  {0x2B19, 276485},
  {0x2B1A, 285126},
  {0x2B1B, 113881},
  {0x2B1C, 985017},
  {0x2B1D, 116436},
  {0x2B1E, 986938},
  {0x2B1F, 114893},
  {0x2B20, 985493},
  {0x2B21, 984962},
  {0x2B22, 113620},
  {0x2B23, 523382},
  {0x2B24, 113675},
  {0x2B25, 114338},
  {0x2B26, 985325},
  {0x2B27, 114519},
  {0x2B28, 985346},
  {0x2B29, 115640},
  {0x2B2A, 115660},
  {0x2B2B, 985791},
  {0x2B2C, 113634},
  {0x2B2D, 984976},
  {0x2B2E, 116338},
  {0x2B2F, 986845},
  {0x2B30, 626166},
  {0x2B31, 937501},
  {0x2B32, 626137},
  {0x2B33, 652479},
  {0x2B34, 633886},
  {0x2B35, 633619},
  {0x2B36, 633583},
  {0x2B37, 633934},
  {0x2B38, 630727},
  {0x2B39, 631261},
  {0x2B3A, 631207},
  {0x2B3B, 633674},
  {0x2B3C, 633776},
  {0x2B3D, 633711},
  {0x2B3E, 630593},
  {0x2B3F, 982131},
  {0x2B40, 429135},
  {0x2B41, 832877},
  {0x2B42, 630160},
  {0x2B43, 838040},
  {0x2B44, 838078},
  {0x2B45, 632646},
  {0x2B46, 840238},
  {0x2B47, 832922},
  {0x2B48, 837665},
  {0x2B49, 945069},
  {0x2B4A, 630122},
  {0x2B4B, 630206},
  {0x2B4C, 837712},
  {0x2B4D, 289884},
  {0x2B4E, 856423},
  {0x2B4F, 856235},
  {0x2B50, 985413},
  {0x2B51, 115699},
  {0x2B52, 985830},
  {0x2B53, 115195},
  {0x2B54, 985616},
  {0x2B55, 507467},
  {0x2B56, 508122},
  {0x2B57, 506583},
  {0x2B58, 506570},
  {0x2B59, 506659},
  {0x2B5A, 891776},
  {0x2B5B, 83398},
  {0x2B5C, 891813},
  {0x2B5D, 83439},
  {0x2B5E, 108947},
  {0x2B5F, 856265},
  {0x2B60, 632862},
  {0x2B61, 961336},
  {0x2B62, 840462},
  {0x2B63, 289233},
  {0x2B64, 628027},
  {0x2B65, 955531},
  {0x2B66, 773141},
  {0x2B67, 772345},
  {0x2B68, 896630},
  {0x2B69, 896988},
  {0x2B6A, 633426},
  {0x2B6B, 961888},
  {0x2B6C, 841036},
  {0x2B6D, 289805},
  {0x2B6E, 196067},
  {0x2B6F, 25099},
  {0x2B70, 632964},
  {0x2B71, 961441},
  {0x2B72, 840565},
  {0x2B73, 289340},
  {0x2B76, 773174},
  {0x2B77, 772378},
  {0x2B78, 896663},
  {0x2B79, 897021},
  {0x2B7A, 633051},
  {0x2B7B, 961524},
  {0x2B7C, 840654},
  {0x2B7D, 289427},
  {0x2B7E, 524011},
  {0x2B7F, 975584},
  {0x2B80, 632894},
  {0x2B81, 961366},
  {0x2B82, 840495},
  {0x2B83, 289265},
  {0x2B84, 633465},
  {0x2B85, 961925},
  {0x2B86, 841076},
  {0x2B87, 289844},
  {0x2B88, 631499},
  {0x2B89, 960650},
  {0x2B8A, 839062},
  {0x2B8B, 288505},
  {0x2B8C, 25147},
  {0x2B8D, 24997},
  {0x2B8E, 25049},
  {0x2B8F, 25198},
  {0x2B90, 832662},
  {0x2B91, 832674},
  {0x2B92, 770461},
  {0x2B93, 770474},
  {0x2B94, 444702},
  {0x2B95, 839039},
  {0x2B97, 914089},
  {0x2B98, 937893},
  {0x2B99, 937841},
  {0x2B9A, 937986},
  {0x2B9B, 937788},
  {0x2B9C, 114233},
  {0x2B9D, 116302},
  {0x2B9E, 115449},
  {0x2B9F, 113408},
  {0x2BA0, 289538},
  {0x2BA1, 289594},
  {0x2BA2, 961631},
  {0x2BA3, 961685},
  {0x2BA4, 633218},
  {0x2BA5, 840824},
  {0x2BA6, 633162},
  {0x2BA7, 840767},
  {0x2BA8, 112752},
  {0x2BA9, 112795},
  {0x2BAA, 113009},
  {0x2BAB, 113050},
  {0x2BAC, 112882},
  {0x2BAD, 112967},
  {0x2BAE, 112839},
  {0x2BAF, 112923},
  {0x2BB0, 833610},
  {0x2BB1, 833633},
  {0x2BB2, 833747},
  {0x2BB3, 833768},
  {0x2BB4, 833680},
  {0x2BB5, 833725},
  {0x2BB6, 833657},
  {0x2BB7, 833701},
  {0x2BB8, 962151},
  {0x2BB9, 955247},
  {0x2BBA, 808552},
  {0x2BBB, 808516},
  {0x2BBC, 808490},
  {0x2BBD, 87263},
  {0x2BBE, 191700},
  {0x2BBF, 184641},
  {0x2BC0, 115831},
  {0x2BC1, 113106},
  {0x2BC2, 953267},
  {0x2BC3, 523407},
  {0x2BC4, 114845},
  {0x2BC5, 114756},
  {0x2BC6, 114395},
  {0x2BC7, 114475},
  {0x2BC8, 114577},
  {0x2BC9, 765317},
  {0x2BCA, 948156},
  {0x2BCB, 123708},
  {0x2BCC, 636631},
  {0x2BCD, 842982},
  {0x2BCE, 984772},
  {0x2BCF, 843020},
  {0x2BD0, 902221},
  {0x2BD1, 954884},
  {0x2BD2, 480735},
  {0x2BD3, 825827},
  {0x2BD4, 825810},
  {0x2BD5, 825794},
  {0x2BD6, 825778},
  {0x2BD7, 950142},
  {0x2BD8, 828505},
  {0x2BD9, 82061},
  {0x2BDA, 524333},
  {0x2BDB, 822939},
  {0x2BDC, 765344},
  {0x2BDD, 985431},
  {0x2BDE, 113156},
  {0x2BDF, 951024},
  {0x2BE0, 251587},
  {0x2BE1, 487398},
  {0x2BE2, 1012951},
  {0x2BE3, 578302},
  {0x2BE4, 27738},
  {0x2BE5, 2213},
  {0x2BE6, 978050},
  {0x2BE7, 826011},
  {0x2BE8, 626777},
  {0x2BE9, 834728},
  {0x2BEA, 906694},
  {0x2BEB, 906720},
  {0x2BEC, 633834},
  {0x2BED, 962052},
  {0x2BEE, 841454},
  {0x2BEF, 289973},
  {0x2BF0, 429606},
  {0x2BF1, 429620},
  {0x2BF2, 850456},
  {0x2BF3, 846194},
  {0x2BF4, 846047},
  {0x2BF5, 846082},
  {0x2BF6, 846010},
  {0x2BF7, 846119},
  {0x2BF8, 846156},
  {0x2BF9, 429404},
  {0x2BFA, 955099},
  {0x2BFB, 852964},
  {0x2BFC, 287165},
  {0x2BFD, 817550},
  {0x2BFE, 833327},
  {0x2BFF, 512800},
  {0x2C00, 454726},
  {0x2C01, 454790},
  {0x2C02, 456044},
  {0x2C03, 455055},
  {0x2C04, 454928},
  {0x2C05, 456168},
  {0x2C06, 456291},
  {0x2C07, 454960},
  {0x2C08, 456258},
  {0x2C09, 455273},
  {0x2C0A, 455148},
  {0x2C0B, 455120},
  {0x2C0C, 454895},
  {0x2C0D, 455338},
  {0x2C0E, 455412},
  {0x2C0F, 455446},
  {0x2C10, 455480},
  {0x2C11, 455512},
  {0x2C12, 455601},
  {0x2C13, 455634},
  {0x2C14, 455761},
  {0x2C15, 455981},
  {0x2C16, 456014},
  {0x2C17, 455023},
  {0x2C18, 455089},
  {0x2C19, 455542},
  {0x2C1A, 455572},
  {0x2C1B, 455696},
  {0x2C1C, 455951},
  {0x2C1D, 454862},
  {0x2C1E, 455666},
  {0x2C1F, 456137},
  {0x2C20, 456106},
  {0x2C21, 456075},
  {0x2C22, 455875},
  {0x2C23, 456229},
  {0x2C24, 455793},
  {0x2C25, 455829},
  {0x2C26, 456200},
  {0x2C27, 455229},
  {0x2C28, 454756},
  {0x2C29, 455187},
  {0x2C2A, 454992},
  {0x2C2B, 455304},
  {0x2C2C, 455727},
  {0x2C2D, 455912},
  {0x2C2E, 455369},
  {0x2C2F, 454821},
  {0x2C30, 456325},
  {0x2C31, 456385},
  {0x2C32, 457565},
  {0x2C33, 456634},
  {0x2C34, 456515},
  {0x2C35, 457681},
  {0x2C36, 457796},
  {0x2C37, 456545},
  {0x2C38, 457765},
  {0x2C39, 456840},
  {0x2C3A, 456721},
  {0x2C3B, 456695},
  {0x2C3C, 456484},
  {0x2C3D, 456901},
  {0x2C3E, 456971},
  {0x2C3F, 457003},
  {0x2C40, 457035},
  {0x2C41, 457065},
  {0x2C42, 457148},
  {0x2C43, 457179},
  {0x2C44, 457298},
  {0x2C45, 457506},
  {0x2C46, 457537},
  {0x2C47, 456604},
  {0x2C48, 456666},
  {0x2C49, 457093},
  {0x2C4A, 457121},
  {0x2C4B, 457237},
  {0x2C4C, 457478},
  {0x2C4D, 456453},
  {0x2C4E, 457209},
  {0x2C4F, 457652},
  {0x2C50, 457623},
  {0x2C51, 457594},
  {0x2C52, 457406},
  {0x2C53, 457738},
  {0x2C54, 457328},
  {0x2C55, 457362},
  {0x2C56, 457711},
  {0x2C57, 456798},
  {0x2C58, 456353},
  {0x2C59, 456758},
  {0x2C5A, 456575},
  {0x2C5B, 456869},
  {0x2C5C, 457266},
  {0x2C5D, 457441},
  {0x2C5E, 456930},
  {0x2C5F, 456414},
  {0x2C60, 589826},
  {0x2C61, 611155},
  {0x2C62, 589983},
  {0x2C63, 593113},
  {0x2C64, 593805},
  {0x2C65, 602752},
  {0x2C66, 618895},
  {0x2C67, 587713},
  {0x2C68, 608694},
  {0x2C69, 589177},
  {0x2C6A, 610399},
  {0x2C6B, 598620},
  {0x2C6C, 624508},
  {0x2C6D, 583869},
  {0x2C6E, 590327},
  {0x2C6F, 595607},
  {0x2C70, 595637},
  {0x2C71, 622967},
  {0x2C72, 597853},
  {0x2C73, 623434},
  {0x2C74, 622788},
  {0x2C75, 587933},
  {0x2C76, 608978},
  {0x2C77, 619326},
  {0x2C78, 606813},
  {0x2C79, 620922},
  {0x2C7A, 614458},
  {0x2C7B, 600910},
  {0x2C7C, 625221},
  {0x2C7D, 727855},
  {0x2C7E, 594648},
  {0x2C7F, 598882},
  {0x2C80, 209304},
  {0x2C81, 212272},
  {0x2C82, 211104},
  {0x2C83, 213958},
  {0x2C84, 209842},
  {0x2C85, 212778},
  {0x2C86, 209565},
  {0x2C87, 212519},
  {0x2C88, 209765},
  {0x2C89, 212707},
  {0x2C8A, 210998},
  {0x2C8B, 213860},
  {0x2C8C, 211131},
  {0x2C8D, 213983},
  {0x2C8E, 209899},
  {0x2C8F, 212831},
  {0x2C90, 211050},
  {0x2C91, 213908},
  {0x2C92, 209953},
  {0x2C93, 212881},
  {0x2C94, 209981},
  {0x2C95, 212907},
  {0x2C96, 210121},
  {0x2C97, 213037},
  {0x2C98, 210149},
  {0x2C99, 213063},
  {0x2C9A, 210174},
  {0x2C9B, 213086},
  {0x2C9C, 210061},
  {0x2C9D, 212981},
  {0x2C9E, 210199},
  {0x2C9F, 213109},
  {0x2CA0, 210812},
  {0x2CA1, 213688},
  {0x2CA2, 210863},
  {0x2CA3, 213735},
  {0x2CA4, 210971},
  {0x2CA5, 213835},
  {0x2CA6, 211024},
  {0x2CA7, 213884},
  {0x2CA8, 211079},
  {0x2CA9, 213935},
  {0x2CAA, 209817},
  {0x2CAB, 212755},
  {0x2CAC, 210035},
  {0x2CAD, 212957},
  {0x2CAE, 210837},
  {0x2CAF, 213711},
  {0x2CB0, 210786},
  {0x2CB1, 213664},
  {0x2CB2, 209619},
  {0x2CB3, 212569},
  {0x2CB4, 210223},
  {0x2CB5, 213131},
  {0x2CB6, 209402},
  {0x2CB7, 212364},
  {0x2CB8, 209693},
  {0x2CB9, 212639},
  {0x2CBA, 209730},
  {0x2CBB, 212674},
  {0x2CBC, 209485},
  {0x2CBD, 212443},
  {0x2CBE, 210522},
  {0x2CBF, 213414},
  {0x2CC0, 210888},
  {0x2CC1, 213758},
  {0x2CC2, 209367},
  {0x2CC3, 212331},
  {0x2CC4, 210559},
  {0x2CC5, 213449},
  {0x2CC6, 210297},
  {0x2CC7, 213201},
  {0x2CC8, 209268},
  {0x2CC9, 212238},
  {0x2CCA, 209656},
  {0x2CCB, 212604},
  {0x2CCC, 210484},
  {0x2CCD, 213378},
  {0x2CCE, 210374},
  {0x2CCF, 213274},
  {0x2CD0, 210087},
  {0x2CD1, 213005},
  {0x2CD2, 210447},
  {0x2CD3, 213343},
  {0x2CD4, 210410},
  {0x2CD5, 213308},
  {0x2CD6, 210334},
  {0x2CD7, 213236},
  {0x2CD8, 210260},
  {0x2CD9, 213166},
  {0x2CDA, 210597},
  {0x2CDB, 213485},
  {0x2CDC, 210710},
  {0x2CDD, 213592},
  {0x2CDE, 210636},
  {0x2CDF, 213522},
  {0x2CE0, 210673},
  {0x2CE1, 213557},
  {0x2CE2, 210749},
  {0x2CE3, 213629},
  {0x2CE4, 214008},
  {0x2CE5, 214047},
  {0x2CE6, 214067},
  {0x2CE7, 214112},
  {0x2CE8, 214134},
  {0x2CE9, 214026},
  {0x2CEA, 214087},
  {0x2CEB, 209524},
  {0x2CEC, 212480},
  {0x2CED, 209442},
  {0x2CEE, 212402},
  {0x2CEF, 211158},
  {0x2CF0, 211184},
  {0x2CF1, 211216},
  {0x2CF2, 209331},
  {0x2CF3, 212297},
  {0x2CF9, 212137},
  {0x2CFA, 212098},
  {0x2CFB, 212165},
  {0x2CFC, 212206},
  {0x2CFD, 212027},
  {0x2CFE, 212052},
  {0x2CFF, 212069},
  {0x2D00, 453622},
  {0x2D01, 453647},
  {0x2D02, 453830},
  {0x2D03, 453779},
  {0x2D04, 453805},
  {0x2D05, 454481},
  {0x2D06, 454583},
  {0x2D07, 454404},
  {0x2D08, 454012},
  {0x2D09, 454090},
  {0x2D0A, 454143},
  {0x2D0B, 454169},
  {0x2D0C, 454195},
  {0x2D0D, 454221},
  {0x2D0E, 454246},
  {0x2D0F, 454609},
  {0x2D10, 454325},
  {0x2D11, 454351},
  {0x2D12, 454430},
  {0x2D13, 454456},
  {0x2D14, 454272},
  {0x2D15, 454116},
  {0x2D16, 453856},
  {0x2D17, 454299},
  {0x2D18, 454377},
  {0x2D19, 453726},
  {0x2D1A, 453673},
  {0x2D1B, 454064},
  {0x2D1C, 453753},
  {0x2D1D, 453699},
  {0x2D1E, 454532},
  {0x2D1F, 454037},
  {0x2D20, 453883},
  {0x2D21, 453935},
  {0x2D22, 453960},
  {0x2D23, 454507},
  {0x2D24, 453909},
  {0x2D25, 453986},
  {0x2D27, 454558},
  {0x2D2D, 453596},
  {0x2D30, 944061},
  {0x2D31, 944100},
  {0x2D32, 944120},
  {0x2D33, 944266},
  {0x2D34, 944307},
  {0x2D35, 943774},
  {0x2D36, 944370},
  {0x2D37, 944162},
  {0x2D38, 944225},
  {0x2D39, 944182},
  {0x2D3A, 944203},
  {0x2D3B, 944859},
  {0x2D3C, 944246},
  {0x2D3D, 944390},
  {0x2D3E, 943923},
  {0x2D3F, 944431},
  {0x2D40, 944329},
  {0x2D41, 943739},
  {0x2D42, 943896},
  {0x2D43, 944349},
  {0x2D44, 944080},
  {0x2D45, 944410},
  {0x2D46, 943950},
  {0x2D47, 944533},
  {0x2D48, 944006},
  {0x2D49, 944879},
  {0x2D4A, 944798},
  {0x2D4B, 943684},
  {0x2D4C, 944033},
  {0x2D4D, 944453},
  {0x2D4E, 944473},
  {0x2D4F, 944493},
  {0x2D50, 943868},
  {0x2D51, 943978},
  {0x2D52, 944513},
  {0x2D53, 944917},
  {0x2D54, 944553},
  {0x2D55, 944573},
  {0x2D56, 944286},
  {0x2D57, 943840},
  {0x2D58, 943713},
  {0x2D59, 944594},
  {0x2D5A, 944635},
  {0x2D5B, 944614},
  {0x2D5C, 944656},
  {0x2D5D, 944676},
  {0x2D5E, 944141},
  {0x2D5F, 944697},
  {0x2D60, 944718},
  {0x2D61, 944738},
  {0x2D62, 944758},
  {0x2D63, 944778},
  {0x2D64, 943809},
  {0x2D65, 944819},
  {0x2D66, 944840},
  {0x2D67, 944898},
  {0x2D6F, 944936},
  {0x2D70, 944980},
  {0x2D7F, 943658},
  {0x2D80, 435886},
  {0x2D81, 436079},
  {0x2D82, 438149},
  {0x2D83, 438694},
  {0x2D84, 438584},
  {0x2D85, 430827},
  {0x2D86, 439424},
  {0x2D87, 431778},
  {0x2D88, 436361},
  {0x2D89, 436560},
  {0x2D8A, 433322},
  {0x2D8B, 441246},
  {0x2D8C, 432313},
  {0x2D8D, 432160},
  {0x2D8E, 434981},
  {0x2D8F, 439314},
  {0x2D90, 431668},
  {0x2D91, 437067},
  {0x2D92, 437177},
  {0x2D93, 433014},
  {0x2D94, 433108},
  {0x2D95, 433084},
  {0x2D96, 433061},
  {0x2DA0, 438716},
  {0x2DA1, 438850},
  {0x2DA2, 438806},
  {0x2DA3, 438738},
  {0x2DA4, 438783},
  {0x2DA5, 438761},
  {0x2DA6, 438828},
  {0x2DA8, 431002},
  {0x2DA9, 431469},
  {0x2DAA, 431425},
  {0x2DAB, 431024},
  {0x2DAC, 431069},
  {0x2DAD, 431047},
  {0x2DAE, 431447},
  {0x2DB0, 441311},
  {0x2DB1, 441445},
  {0x2DB2, 441401},
  {0x2DB3, 441333},
  {0x2DB4, 441378},
  {0x2DB5, 441356},
  {0x2DB6, 441423},
  {0x2DB8, 431092},
  {0x2DB9, 431402},
  {0x2DBA, 431356},
  {0x2DBB, 431115},
  {0x2DBC, 431162},
  {0x2DBD, 431139},
  {0x2DBE, 431379},
  {0x2DC0, 437865},
  {0x2DC1, 437999},
  {0x2DC2, 437955},
  {0x2DC3, 437887},
  {0x2DC4, 437932},
  {0x2DC5, 437910},
  {0x2DC6, 437977},
  {0x2DC8, 435602},
  {0x2DC9, 435736},
  {0x2DCA, 435692},
  {0x2DCB, 435624},
  {0x2DCC, 435669},
  {0x2DCD, 435647},
  {0x2DCE, 435714},
  {0x2DD0, 440612},
  {0x2DD1, 440746},
  {0x2DD2, 440702},
  {0x2DD3, 440634},
  {0x2DD4, 440679},
  {0x2DD5, 440657},
  {0x2DD6, 440724},
  {0x2DD8, 434114},
  {0x2DD9, 434248},
  {0x2DDA, 434204},
  {0x2DDB, 434136},
  {0x2DDC, 434181},
  {0x2DDD, 434159},
  {0x2DDE, 434226},
  {0x2DE0, 197584},
  {0x2DE1, 198662},
  {0x2DE2, 197975},
  {0x2DE3, 197677},
  {0x2DE4, 198839},
  {0x2DE5, 198810},
  {0x2DE6, 198244},
  {0x2DE7, 197767},
  {0x2DE8, 197796},
  {0x2DE9, 197825},
  {0x2DEA, 198349},
  {0x2DEB, 198409},
  {0x2DEC, 197854},
  {0x2DED, 197883},
  {0x2DEE, 198536},
  {0x2DEF, 198005},
  {0x2DF0, 198565},
  {0x2DF1, 197647},
  {0x2DF2, 198438},
  {0x2DF3, 198468},
  {0x2DF4, 197944},
  {0x2DF5, 197912},
  {0x2DF6, 197556},
  {0x2DF7, 198098},
  {0x2DF8, 197706},
  {0x2DF9, 198310},
  {0x2DFA, 198691},
  {0x2DFB, 198781},
  {0x2DFC, 198127},
  {0x2DFD, 198273},
  {0x2DFE, 197613},
  {0x2DFF, 198164},
  {0x2E00, 834004},
  {0x2E01, 833965},
  {0x2E02, 628595},
  {0x2E03, 836023},
  {0x2E04, 626569},
  {0x2E05, 834436},
  {0x2E06, 830382},
  {0x2E07, 830236},
  {0x2E08, 285140},
  {0x2E09, 628876},
  {0x2E0A, 836314},
  {0x2E0B, 830488},
  {0x2E0C, 627646},
  {0x2E0D, 835402},
  {0x2E0E, 293830},
  {0x2E0F, 812919},
  {0x2E10, 444596},
  {0x2E11, 833137},
  {0x2E12, 524414},
  {0x2E13, 285069},
  {0x2E14, 287839},
  {0x2E15, 960083},
  {0x2E16, 285083},
  {0x2E17, 285872},
  {0x2E18, 535949},
  {0x2E19, 812059},
  {0x2E1A, 524361},
  {0x2E1B, 945248},
  {0x2E1C, 627371},
  {0x2E1D, 835143},
  {0x2E1E, 945206},
  {0x2E1F, 945227},
  {0x2E20, 628993},
  {0x2E21, 836427},
  {0x2E22, 948780},
  {0x2E23, 948902},
  {0x2E24, 124379},
  {0x2E25, 124516},
  {0x2E26, 628182},
  {0x2E27, 835512},
  {0x2E28, 626628},
  {0x2E29, 834534},
  {0x2E2A, 953903},
  {0x2E2B, 800519},
  {0x2E2C, 904986},
  {0x2E2D, 444161},
  {0x2E2E, 833262},
  {0x2E2F, 975601},
  {0x2E30, 841794},
  {0x2E31, 988825},
  {0x2E32, 953331},
  {0x2E33, 830225},
  {0x2E34, 830212},
  {0x2E35, 953609},
  {0x2E36, 269163},
  {0x2E37, 269186},
  {0x2E38, 953344},
  {0x2E39, 948575},
  {0x2E3A, 954169},
  {0x2E3B, 938039},
  {0x2E3C, 906804},
  {0x2E3D, 975566},
  {0x2E3E, 988570},
  {0x2E3F, 173659},
  {0x2E40, 285578},
  {0x2E41, 833012},
  {0x2E42, 285746},
  {0x2E43, 269275},
  {0x2E44, 286180},
  {0x2E45, 535986},
  {0x2E46, 536006},
  {0x2E47, 652832},
  {0x2E48, 652843},
  {0x2E49, 286090},
  {0x2E4A, 285111},
  {0x2E4B, 950571},
  {0x2E4C, 708067},
  {0x2E4D, 812931},
  {0x2E4E, 829422},
  {0x2E4F, 214192},
  {0x2E50, 214980},
  {0x2E51, 214949},
  {0x2E52, 947031},
  {0x2E53, 708082},
  {0x2E54, 708108},
  {0x2E55, 628438},
  {0x2E56, 835862},
  {0x2E57, 628368},
  {0x2E58, 835790},
  {0x2E59, 948321},
  {0x2E5A, 948446},
  {0x2E5B, 123955},
  {0x2E5C, 124092},
  {0x2E5D, 785422},
  {0x2E80, 193759},
  {0x2E81, 192684},
  {0x2E82, 193795},
  {0x2E83, 193843},
  {0x2E84, 193818},
  {0x2E85, 193691},
  {0x2E86, 191892},
  {0x2E87, 194223},
  {0x2E88, 193246},
  {0x2E89, 193268},
  {0x2E8A, 192756},
  {0x2E8B, 193778},
  {0x2E8C, 194055},
  {0x2E8D, 194077},
  {0x2E8E, 193312},
  {0x2E8F, 193356},
  {0x2E90, 193333},
  {0x2E91, 193290},
  {0x2E92, 194099},
  {0x2E93, 194241},
  {0x2E94, 194117},
  {0x2E95, 194139},
  {0x2E96, 193043},
  {0x2E97, 193065},
  {0x2E98, 193009},
  {0x2E99, 193743},
  {0x2E9B, 192628},
  {0x2E9C, 194207},
  {0x2E9D, 193453},
  {0x2E9E, 192738},
  {0x2E9F, 193489},
  {0x2EA0, 192663},
  {0x2EA1, 194339},
  {0x2EA2, 194361},
  {0x2EA3, 192889},
  {0x2EA4, 193651},
  {0x2EA5, 193671},
  {0x2EA6, 193901},
  {0x2EA7, 192722},
  {0x2EA8, 192779},
  {0x2EA9, 193229},
  {0x2EAA, 191849},
  {0x2EAB, 192873},
  {0x2EAC, 194161},
  {0x2EAD, 194184},
  {0x2EAE, 191813},
  {0x2EAF, 193884},
  {0x2EB0, 192433},
  {0x2EB1, 193573},
  {0x2EB2, 193615},
  {0x2EB3, 193593},
  {0x2EB4, 193552},
  {0x2EB5, 193436},
  {0x2EB6, 193866},
  {0x2EB7, 193727},
  {0x2EB8, 192857},
  {0x2EB9, 193635},
  {0x2EBA, 191908},
  {0x2EBB, 191930},
  {0x2EBC, 193419},
  {0x2EBD, 193470},
  {0x2EBE, 192941},
  {0x2EBF, 192987},
  {0x2EC0, 192963},
  {0x2EC1, 194260},
  {0x2EC2, 192702},
  {0x2EC3, 194383},
  {0x2EC4, 194404},
  {0x2EC5, 192373},
  {0x2EC6, 193940},
  {0x2EC7, 193087},
  {0x2EC8, 192463},
  {0x2EC9, 192402},
  {0x2ECA, 192906},
  {0x2ECB, 191982},
  {0x2ECC, 193968},
  {0x2ECD, 194297},
  {0x2ECE, 194318},
  {0x2ECF, 192646},
  {0x2ED0, 192222},
  {0x2ED1, 193377},
  {0x2ED2, 193398},
  {0x2ED3, 192313},
  {0x2ED4, 192192},
  {0x2ED5, 193508},
  {0x2ED6, 193530},
  {0x2ED7, 193710},
  {0x2ED8, 191832},
  {0x2ED9, 192495},
  {0x2EDA, 192283},
  {0x2EDB, 192598},
  {0x2EDC, 192133},
  {0x2EDD, 192795},
  {0x2EDE, 192837},
  {0x2EDF, 192815},
  {0x2EE0, 192044},
  {0x2EE1, 193026},
  {0x2EE2, 192252},
  {0x2EE3, 191875},
  {0x2EE4, 192923},
  {0x2EE5, 192103},
  {0x2EE6, 191952},
  {0x2EE7, 192343},
  {0x2EE8, 193996},
  {0x2EE9, 194025},
  {0x2EEA, 192162},
  {0x2EEB, 193136},
  {0x2EEC, 192073},
  {0x2EED, 193166},
  {0x2EEE, 192535},
  {0x2EEF, 193104},
  {0x2EF0, 192012},
  {0x2EF1, 194278},
  {0x2EF2, 193197},
  {0x2EF3, 192566},
  {0x2F00, 542978},
  {0x2F01, 542597},
  {0x2F02, 541233},
  {0x2F03, 543745},
  {0x2F04, 543496},
  {0x2F05, 542253},
  {0x2F06, 544449},
  {0x2F07, 542558},
  {0x2F08, 542664},
  {0x2F09, 542538},
  {0x2F0A, 541547},
  {0x2F0B, 541475},
  {0x2F0C, 541331},
  {0x2F0D, 540996},
  {0x2F0E, 542314},
  {0x2F0F, 544168},
  {0x2F10, 543020},
  {0x2F11, 542414},
  {0x2F12, 543105},
  {0x2F13, 544793},
  {0x2F14, 543936},
  {0x2F15, 543270},
  {0x2F16, 542221},
  {0x2F17, 544239},
  {0x2F18, 541146},
  {0x2F19, 543476},
  {0x2F1A, 540886},
  {0x2F1B, 543126},
  {0x2F1C, 540278},
  {0x2F1D, 542880},
  {0x2F1E, 541522},
  {0x2F1F, 541435},
  {0x2F20, 543431},
  {0x2F21, 541961},
  {0x2F22, 541979},
  {0x2F23, 541588},
  {0x2F24, 540444},
  {0x2F25, 544752},
  {0x2F26, 540805},
  {0x2F27, 543321},
  {0x2F28, 542333},
  {0x2F29, 543808},
  {0x2F2A, 542455},
  {0x2F2B, 540974},
  {0x2F2C, 543957},
  {0x2F2D, 542856},
  {0x2F2E, 543300},
  {0x2F2F, 544773},
  {0x2F30, 542997},
  {0x2F31, 544405},
  {0x2F32, 541397},
  {0x2F33, 543653},
  {0x2F34, 541252},
  {0x2F35, 542637},
  {0x2F36, 544468},
  {0x2F37, 543599},
  {0x2F38, 540656},
  {0x2F39, 543829},
  {0x2F3A, 540697},
  {0x2F3B, 544043},
  {0x2F3C, 542180},
  {0x2F3D, 542086},
  {0x2F3E, 541213},
  {0x2F3F, 542140},
  {0x2F40, 540675},
  {0x2F41, 543193},
  {0x2F42, 543454},
  {0x2F43, 541077},
  {0x2F44, 540361},
  {0x2F45, 543979},
  {0x2F46, 542940},
  {0x2F47, 544128},
  {0x2F48, 543412},
  {0x2F49, 542770},
  {0x2F4A, 544363},
  {0x2F4B, 542435},
  {0x2F4C, 544084},
  {0x2F4D, 541036},
  {0x2F4E, 544628},
  {0x2F4F, 541172},
  {0x2F50, 540951},
  {0x2F51, 541901},
  {0x2F52, 540846},
  {0x2F53, 544022},
  {0x2F54, 544607},
  {0x2F55, 541757},
  {0x2F56, 540866},
  {0x2F57, 541670},
  {0x2F58, 541307},
  {0x2F59, 542109},
  {0x2F5A, 543787},
  {0x2F5B, 541650},
  {0x2F5C, 541017},
  {0x2F5D, 541194},
  {0x2F5E, 543149},
  {0x2F5F, 542375},
  {0x2F60, 542703},
  {0x2F61, 544279},
  {0x2F62, 544147},
  {0x2F63, 542577},
  {0x2F64, 544493},
  {0x2F65, 541715},
  {0x2F66, 540607},
  {0x2F67, 543681},
  {0x2F68, 541280},
  {0x2F69, 544691},
  {0x2F6A, 543725},
  {0x2F6B, 541099},
  {0x2F6C, 541611},
  {0x2F6D, 543871},
  {0x2F6E, 540340},
  {0x2F6F, 544063},
  {0x2F70, 543914},
  {0x2F71, 544342},
  {0x2F72, 542024},
  {0x2F73, 540785},
  {0x2F74, 544001},
  {0x2F75, 540402},
  {0x2F76, 543250},
  {0x2F77, 543705},
  {0x2F78, 542395},
  {0x2F79, 542901},
  {0x2F7A, 543557},
  {0x2F7B, 541692},
  {0x2F7C, 542959},
  {0x2F7D, 540299},
  {0x2F7E, 543085},
  {0x2F7F, 541416},
  {0x2F80, 540720},
  {0x2F81, 542683},
  {0x2F82, 542746},
  {0x2F83, 543537},
  {0x2F84, 540318},
  {0x2F85, 542813},
  {0x2F86, 544299},
  {0x2F87, 543044},
  {0x2F88, 540567},
  {0x2F89, 544104},
  {0x2F8A, 540930},
  {0x2F8B, 542045},
  {0x2F8C, 544258},
  {0x2F8D, 542353},
  {0x2F8E, 540526},
  {0x2F8F, 544577},
  {0x2F90, 540907},
  {0x2F91, 544650},
  {0x2F92, 543518},
  {0x2F93, 542273},
  {0x2F94, 543892},
  {0x2F95, 544512},
  {0x2F96, 540424},
  {0x2F97, 543066},
  {0x2F98, 540380},
  {0x2F99, 543578},
  {0x2F9A, 543231},
  {0x2F9B, 543341},
  {0x2F9C, 541837},
  {0x2F9D, 540587},
  {0x2F9E, 540741},
  {0x2F9F, 540483},
  {0x2FA0, 542790},
  {0x2FA1, 544557},
  {0x2FA2, 540826},
  {0x2FA3, 544732},
  {0x2FA4, 541119},
  {0x2FA5, 544534},
  {0x2FA6, 542004},
  {0x2FA7, 542617},
  {0x2FA8, 541920},
  {0x2FA9, 542835},
  {0x2FAA, 543766},
  {0x2FAB, 543620},
  {0x2FAC, 543173},
  {0x2FAD, 540547},
  {0x2FAE, 544813},
  {0x2FAF, 541630},
  {0x2FB0, 542495},
  {0x2FB1, 544209},
  {0x2FB2, 542518},
  {0x2FB3, 543850},
  {0x2FB4, 542475},
  {0x2FB5, 544712},
  {0x2FB6, 541818},
  {0x2FB7, 541456},
  {0x2FB8, 542160},
  {0x2FB9, 541857},
  {0x2FBA, 542293},
  {0x2FBB, 540636},
  {0x2FBC, 544189},
  {0x2FBD, 542066},
  {0x2FBE, 541736},
  {0x2FBF, 543360},
  {0x2FC0, 540761},
  {0x2FC1, 541940},
  {0x2FC2, 541777},
  {0x2FC3, 540463},
  {0x2FC4, 543392},
  {0x2FC5, 541057},
  {0x2FC6, 544670},
  {0x2FC7, 542201},
  {0x2FC8, 544834},
  {0x2FC9, 542724},
  {0x2FCA, 540505},
  {0x2FCB, 541496},
  {0x2FCC, 541881},
  {0x2FCD, 544383},
  {0x2FCE, 541377},
  {0x2FCF, 543212},
  {0x2FD0, 542920},
  {0x2FD1, 541568},
  {0x2FD2, 544321},
  {0x2FD3, 541355},
  {0x2FD4, 544427},
  {0x2FD5, 541797},
  {0x2FF0, 525504},
  {0x2FF1, 525232},
  {0x2FF2, 525445},
  {0x2FF3, 525281},
  {0x2FF4, 525341},
  {0x2FF5, 525684},
  {0x2FF6, 525738},
  {0x2FF7, 525792},
  {0x2FF8, 526018},
  {0x2FF9, 526077},
  {0x2FFA, 525845},
  {0x2FFB, 525552},
  {0x2FFC, 525964},
  {0x2FFD, 525904},
  {0x2FFE, 525389},
  {0x2FFF, 525595},
  {0x3000, 526326},
  {0x3001, 525182},
  {0x3002, 526168},
  {0x3003, 278867},
  {0x3004, 536272},
  {0x3005, 526218},
  {0x3006, 525157},
  {0x3007, 526273},
  {0x3008, 626063},
  {0x3009, 833916},
  {0x300A, 626602},
  {0x300B, 834470},
  {0x300C, 626438},
  {0x300D, 834300},
  {0x300E, 629045},
  {0x300F, 836510},
  {0x3010, 626343},
  {0x3011, 834220},
  {0x3012, 826051},
  {0x3013, 454654},
  {0x3014, 628848},
  {0x3015, 836285},
  {0x3016, 629096},
  {0x3017, 836563},
  {0x3018, 629175},
  {0x3019, 836645},
  {0x301A, 629149},
  {0x301B, 836618},
  {0x301C, 982200},
  {0x301D, 833049},
  {0x301E, 285935},
  {0x301F, 652800},
  {0x3020, 826063},
  {0x3021, 503485},
  {0x3022, 503642},
  {0x3023, 503595},
  {0x3024, 503441},
  {0x3025, 503419},
  {0x3026, 503529},
  {0x3027, 503506},
  {0x3028, 503396},
  {0x3029, 503463},
  {0x302A, 526245},
  {0x302B, 526297},
  {0x302C, 525200},
  {0x302D, 526137},
  {0x302E, 503368},
  {0x302F, 494919},
  {0x3030, 982263},
  {0x3031, 974903},
  {0x3032, 975003},
  {0x3033, 974966},
  {0x3034, 975047},
  {0x3035, 974929},
  {0x3036, 191494},
  {0x3037, 526483},
  {0x3038, 503550},
  {0x3039, 503618},
  {0x303A, 503571},
  {0x303B, 974867},
  {0x303C, 668842},
  {0x303D, 817445},
  {0x303E, 529553},
  {0x303F, 526190},
  {0x3041, 522409},
  {0x3042, 521353},
  {0x3043, 522457},
  {0x3044, 521823},
  {0x3045, 522605},
  {0x3046, 522937},
  {0x3047, 522433},
  {0x3048, 521615},
  {0x3049, 522556},
  {0x304A, 522144},
  {0x304B, 521841},
  {0x304C, 521633},
  {0x304D, 521879},
  {0x304E, 521671},
  {0x304F, 521917},
  {0x3050, 521709},
  {0x3051, 521860},
  {0x3052, 521652},
  {0x3053, 521898},
  {0x3054, 521690},
  {0x3055, 522352},
  {0x3056, 523107},
  {0x3057, 522390},
  {0x3058, 523145},
  {0x3059, 522823},
  {0x305A, 523183},
  {0x305B, 522371},
  {0x305C, 523126},
  {0x305D, 522804},
  {0x305E, 523164},
  {0x305F, 522842},
  {0x3060, 521520},
  {0x3061, 522880},
  {0x3062, 521558},
  {0x3063, 522580},
  {0x3064, 522918},
  {0x3065, 521596},
  {0x3066, 522861},
  {0x3067, 521539},
  {0x3068, 522899},
  {0x3069, 521577},
  {0x306A, 522049},
  {0x306B, 522087},
  {0x306C, 522125},
  {0x306D, 522068},
  {0x306E, 522106},
  {0x306F, 521728},
  {0x3070, 521425},
  {0x3071, 522162},
  {0x3072, 521766},
  {0x3073, 521463},
  {0x3074, 522200},
  {0x3075, 521804},
  {0x3076, 521501},
  {0x3077, 522238},
  {0x3078, 521747},
  {0x3079, 521444},
  {0x307A, 522181},
  {0x307B, 521785},
  {0x307C, 521482},
  {0x307D, 522219},
  {0x307E, 521936},
  {0x307F, 521974},
  {0x3080, 522012},
  {0x3081, 521955},
  {0x3082, 521993},
  {0x3083, 522729},
  {0x3084, 523050},
  {0x3085, 522779},
  {0x3086, 523088},
  {0x3087, 522754},
  {0x3088, 523069},
  {0x3089, 522257},
  {0x308A, 522295},
  {0x308B, 522333},
  {0x308C, 522276},
  {0x308D, 522314},
  {0x308E, 522629},
  {0x308F, 522974},
  {0x3090, 523012},
  {0x3091, 522993},
  {0x3092, 523031},
  {0x3093, 522031},
  {0x3094, 522955},
  {0x3095, 522481},
  {0x3096, 522506},
  {0x3099, 203563},
  {0x309A, 203512},
  {0x309B, 549804},
  {0x309C, 549763},
  {0x309D, 521329},
  {0x309E, 523202},
  {0x309F, 521307},
  {0x30A0, 549692},
  {0x30A1, 548348},
  {0x30A2, 546779},
  {0x30A3, 548521},
  {0x30A4, 547302},
  {0x30A5, 548968},
  {0x30A6, 549300},
  {0x30A7, 548372},
  {0x30A8, 547094},
  {0x30A9, 548719},
  {0x30AA, 548083},
  {0x30AB, 547320},
  {0x30AC, 547112},
  {0x30AD, 547358},
  {0x30AE, 547150},
  {0x30AF, 547396},
  {0x30B0, 547188},
  {0x30B1, 547339},
  {0x30B2, 547131},
  {0x30B3, 547377},
  {0x30B4, 547169},
  {0x30B5, 548291},
  {0x30B6, 549546},
  {0x30B7, 548329},
  {0x30B8, 549584},
  {0x30B9, 549186},
  {0x30BA, 549622},
  {0x30BB, 548310},
  {0x30BC, 549565},
  {0x30BD, 549167},
  {0x30BE, 549603},
  {0x30BF, 549205},
  {0x30C0, 546999},
  {0x30C1, 549243},
  {0x30C2, 547037},
  {0x30C3, 548943},
  {0x30C4, 549281},
  {0x30C5, 547075},
  {0x30C6, 549224},
  {0x30C7, 547018},
  {0x30C8, 549262},
  {0x30C9, 547056},
  {0x30CA, 547988},
  {0x30CB, 548026},
  {0x30CC, 548064},
  {0x30CD, 548007},
  {0x30CE, 548045},
  {0x30CF, 547207},
  {0x30D0, 546904},
  {0x30D1, 548101},
  {0x30D2, 547245},
  {0x30D3, 546942},
  {0x30D4, 548139},
  {0x30D5, 547283},
  {0x30D6, 546980},
  {0x30D7, 548177},
  {0x30D8, 547226},
  {0x30D9, 546923},
  {0x30DA, 548120},
  {0x30DB, 547264},
  {0x30DC, 546961},
  {0x30DD, 548158},
  {0x30DE, 547415},
  {0x30DF, 547453},
  {0x30E0, 547951},
  {0x30E1, 547434},
  {0x30E2, 547932},
  {0x30E3, 549092},
  {0x30E4, 549489},
  {0x30E5, 549142},
  {0x30E6, 549527},
  {0x30E7, 549117},
  {0x30E8, 549508},
  {0x30E9, 548196},
  {0x30EA, 548234},
  {0x30EB, 548272},
  {0x30EC, 548215},
  {0x30ED, 548253},
  {0x30EE, 548992},
  {0x30EF, 549413},
  {0x30F0, 549451},
  {0x30F1, 549432},
  {0x30F2, 549470},
  {0x30F3, 547970},
  {0x30F4, 549394},
  {0x30F5, 548545},
  {0x30F6, 548570},
  {0x30F7, 549318},
  {0x30F8, 549356},
  {0x30F9, 549337},
  {0x30FA, 549375},
  {0x30FB, 549641},
  {0x30FC, 549724},
  {0x30FD, 546755},
  {0x30FE, 549661},
  {0x30FF, 546733},
  {0x3105, 122491},
  {0x3106, 123373},
  {0x3107, 123112},
  {0x3108, 122736},
  {0x3109, 122565},
  {0x310A, 123464},
  {0x310B, 123130},
  {0x310C, 123075},
  {0x310D, 122754},
  {0x310E, 123038},
  {0x310F, 122848},
  {0x3110, 123001},
  {0x3111, 123391},
  {0x3112, 123538},
  {0x3113, 123574},
  {0x3114, 122546},
  {0x3115, 123445},
  {0x3116, 123409},
  {0x3117, 123556},
  {0x3118, 122528},
  {0x3119, 123427},
  {0x311A, 122296},
  {0x311B, 123206},
  {0x311C, 122583},
  {0x311D, 122620},
  {0x311E, 122333},
  {0x311F, 122639},
  {0x3120, 122451},
  {0x3121, 123354},
  {0x3122, 122392},
  {0x3123, 122658},
  {0x3124, 122411},
  {0x3125, 122677},
  {0x3126, 122717},
  {0x3127, 122866},
  {0x3128, 123482},
  {0x3129, 122982},
  {0x312A, 123520},
  {0x312B, 123148},
  {0x312C, 122791},
  {0x312D, 122884},
  {0x312E, 123224},
  {0x312F, 123187},
  {0x3131, 501578},
  {0x3132, 502747},
  {0x3133, 501599},
  {0x3134, 501724},
  {0x3135, 501744},
  {0x3136, 501770},
  {0x3137, 502895},
  {0x3138, 502847},
  {0x3139, 502170},
  {0x313A, 502216},
  {0x313B, 502275},
  {0x313C, 502357},
  {0x313D, 502414},
  {0x313E, 502439},
  {0x313F, 502329},
  {0x3140, 502190},
  {0x3141, 501625},
  {0x3142, 501953},
  {0x3143, 502798},
  {0x3144, 502026},
  {0x3145, 502526},
  {0x3146, 502823},
  {0x3147, 501417},
  {0x3148, 501311},
  {0x3149, 502672},
  {0x314A, 501289},
  {0x314B, 501556},
  {0x314C, 502873},
  {0x314D, 501931},
  {0x314E, 501381},
  {0x314F, 501215},
  {0x3150, 501231},
  {0x3151, 503019},
  {0x3152, 503036},
  {0x3153, 501347},
  {0x3154, 501331},
  {0x3155, 503071},
  {0x3156, 503054},
  {0x3157, 501876},
  {0x3158, 502932},
  {0x3159, 502949},
  {0x315A, 501892},
  {0x315B, 503214},
  {0x315C, 502916},
  {0x315D, 502984},
  {0x315E, 502967},
  {0x315F, 503002},
  {0x3160, 503291},
  {0x3161, 501364},
  {0x3162, 503197},
  {0x3163, 501401},
  {0x3164, 494947},
  {0x3165, 502773},
  {0x3166, 501849},
  {0x3167, 501824},
  {0x3168, 501796},
  {0x3169, 502243},
  {0x316A, 502467},
  {0x316B, 502383},
  {0x316C, 502301},
  {0x316D, 502494},
  {0x316E, 501673},
  {0x316F, 501699},
  {0x3170, 501645},
  {0x3171, 501437},
  {0x3172, 501999},
  {0x3173, 502143},
  {0x3174, 502051},
  {0x3175, 502083},
  {0x3176, 501973},
  {0x3177, 502115},
  {0x3178, 501495},
  {0x3179, 501523},
  {0x317A, 502570},
  {0x317B, 502596},
  {0x317C, 502646},
  {0x317D, 502621},
  {0x317E, 502545},
  {0x317F, 501909},
  {0x3180, 502722},
  {0x3181, 503115},
  {0x3182, 503169},
  {0x3183, 503138},
  {0x3184, 501465},
  {0x3185, 502697},
  {0x3186, 503089},
  {0x3187, 503250},
  {0x3188, 503270},
  {0x3189, 503231},
  {0x318A, 503347},
  {0x318B, 503327},
  {0x318C, 503308},
  {0x318D, 501248},
  {0x318E, 501268},
  {0x3190, 524819},
  {0x3191, 524954},
  {0x3192, 524922},
  {0x3193, 525125},
  {0x3194, 525059},
  {0x3195, 524716},
  {0x3196, 525093},
  {0x3197, 524887},
  {0x3198, 524613},
  {0x3199, 524682},
  {0x319A, 524990},
  {0x319B, 525025},
  {0x319C, 524749},
  {0x319D, 524784},
  {0x319E, 524648},
  {0x319F, 524855},
  {0x31A0, 122509},
  {0x31A1, 123593},
  {0x31A2, 123019},
  {0x31A3, 122810},
  {0x31A4, 122601},
  {0x31A5, 122697},
  {0x31A6, 123335},
  {0x31A7, 123315},
  {0x31A8, 122963},
  {0x31A9, 122431},
  {0x31AA, 122922},
  {0x31AB, 123500},
  {0x31AC, 122903},
  {0x31AD, 123167},
  {0x31AE, 122352},
  {0x31AF, 122470},
  {0x31B0, 122373},
  {0x31B1, 123276},
  {0x31B2, 123295},
  {0x31B3, 122942},
  {0x31B4, 122248},
  {0x31B5, 122272},
  {0x31B6, 122224},
  {0x31B7, 122200},
  {0x31B8, 122772},
  {0x31B9, 123093},
  {0x31BA, 123612},
  {0x31BB, 122176},
  {0x31BC, 122829},
  {0x31BD, 123056},
  {0x31BE, 123257},
  {0x31BF, 122314},
  {0x31C0, 194920},
  {0x31C1, 194947},
  {0x31C2, 194961},
  {0x31C3, 194425},
  {0x31C4, 194816},
  {0x31C5, 194616},
  {0x31C6, 194555},
  {0x31C7, 194480},
  {0x31C8, 194600},
  {0x31C9, 194889},
  {0x31CA, 194570},
  {0x31CB, 194631},
  {0x31CC, 194494},
  {0x31CD, 194585},
  {0x31CE, 194647},
  {0x31CF, 194680},
  {0x31D0, 194453},
  {0x31D1, 194761},
  {0x31D2, 194693},
  {0x31D3, 194788},
  {0x31D4, 194440},
  {0x31D5, 194541},
  {0x31D6, 194466},
  {0x31D7, 194860},
  {0x31D8, 194845},
  {0x31D9, 194802},
  {0x31DA, 194774},
  {0x31DB, 194706},
  {0x31DC, 194734},
  {0x31DD, 194933},
  {0x31DE, 194905},
  {0x31DF, 194830},
  {0x31E0, 194525},
  {0x31E1, 194663},
  {0x31E2, 194720},
  {0x31E3, 194748},
  {0x31E4, 194510},
  {0x31E5, 194874},
  {0x31EF, 525638},
  {0x31F0, 548620},
  {0x31F1, 548868},
  {0x31F2, 548893},
  {0x31F3, 548918},
  {0x31F4, 548694},
  {0x31F5, 548396},
  {0x31F6, 548446},
  {0x31F7, 548496},
  {0x31F8, 548421},
  {0x31F9, 548471},
  {0x31FA, 548645},
  {0x31FB, 548743},
  {0x31FC, 548793},
  {0x31FD, 548843},
  {0x31FE, 548768},
  {0x31FF, 548818},
  {0x3200, 813563},
  {0x3201, 813677},
  {0x3202, 814019},
  {0x3203, 813849},
  {0x3204, 813621},
  {0x3205, 813793},
  {0x3206, 813905},
  {0x3207, 813447},
  {0x3208, 813306},
  {0x3209, 813246},
  {0x320A, 813503},
  {0x320B, 813959},
  {0x320C, 813733},
  {0x320D, 813391},
  {0x320E, 813591},
  {0x320F, 813704},
  {0x3210, 814047},
  {0x3211, 813876},
  {0x3212, 813648},
  {0x3213, 813820},
  {0x3214, 813931},
  {0x3215, 813474},
  {0x3216, 813333},
  {0x3217, 813275},
  {0x3218, 813532},
  {0x3219, 813988},
  {0x321A, 813762},
  {0x321B, 813418},
  {0x321C, 813362},
  {0x321D, 815211},
  {0x321E, 815175},
  {0x3220, 814603},
  {0x3221, 815088},
  {0x3222, 815058},
  {0x3223, 814398},
  {0x3224, 814369},
  {0x3225, 814816},
  {0x3226, 814786},
  {0x3227, 814208},
  {0x3228, 814574},
  {0x3229, 815030},
  {0x322A, 814516},
  {0x322B, 814340},
  {0x322C, 815116},
  {0x322D, 815146},
  {0x322E, 814486},
  {0x322F, 814178},
  {0x3230, 814968},
  {0x3231, 814908},
  {0x3232, 814427},
  {0x3233, 814844},
  {0x3234, 814545},
  {0x3235, 814876},
  {0x3236, 814306},
  {0x3237, 814139},
  {0x3238, 814456},
  {0x3239, 814661},
  {0x323A, 814110},
  {0x323B, 814938},
  {0x323C, 814996},
  {0x323D, 814238},
  {0x323E, 814695},
  {0x323F, 814077},
  {0x3240, 814273},
  {0x3241, 814728},
  {0x3242, 814757},
  {0x3243, 814631},
  {0x3244, 186808},
  {0x3245, 186472},
  {0x3246, 186936},
  {0x3247, 186503},
  {0x3248, 190716},
  {0x3249, 191231},
  {0x324A, 190906},
  {0x324B, 190359},
  {0x324C, 190196},
  {0x324D, 190660},
  {0x324E, 190598},
  {0x324F, 190092},
  {0x3250, 817520},
  {0x3251, 191269},
  {0x3252, 191377},
  {0x3253, 191349},
  {0x3254, 191177},
  {0x3255, 191150},
  {0x3256, 191323},
  {0x3257, 191295},
  {0x3258, 191122},
  {0x3259, 191204},
  {0x325A, 190775},
  {0x325B, 190944},
  {0x325C, 191052},
  {0x325D, 191024},
  {0x325E, 190852},
  {0x325F, 190825},
  {0x3260, 185463},
  {0x3261, 185553},
  {0x3262, 185823},
  {0x3263, 185689},
  {0x3264, 185509},
  {0x3265, 185645},
  {0x3266, 185733},
  {0x3267, 185348},
  {0x3268, 185260},
  {0x3269, 185212},
  {0x326A, 185415},
  {0x326B, 185775},
  {0x326C, 185597},
  {0x326D, 185304},
  {0x326E, 185485},
  {0x326F, 185574},
  {0x3270, 185845},
  {0x3271, 185710},
  {0x3272, 185530},
  {0x3273, 185666},
  {0x3274, 185753},
  {0x3275, 185369},
  {0x3276, 185281},
  {0x3277, 185235},
  {0x3278, 185438},
  {0x3279, 185798},
  {0x327A, 185620},
  {0x327B, 185325},
  {0x327C, 188362},
  {0x327D, 188394},
  {0x327E, 185392},
  {0x327F, 578279},
  {0x3280, 186762},
  {0x3281, 187255},
  {0x3282, 187231},
  {0x3283, 186380},
  {0x3284, 186357},
  {0x3285, 187010},
  {0x3286, 186986},
  {0x3287, 186200},
  {0x3288, 186739},
  {0x3289, 187209},
  {0x328A, 186669},
  {0x328B, 186334},
  {0x328C, 187277},
  {0x328D, 187301},
  {0x328E, 186645},
  {0x328F, 186176},
  {0x3290, 187159},
  {0x3291, 187084},
  {0x3292, 186403},
  {0x3293, 187032},
  {0x3294, 186692},
  {0x3295, 187058},
  {0x3296, 186306},
  {0x3297, 186094},
  {0x3298, 186526},
  {0x3299, 186961},
  {0x329A, 186595},
  {0x329B, 186281},
  {0x329C, 187132},
  {0x329D, 186253},
  {0x329E, 186784},
  {0x329F, 186041},
  {0x32A0, 186449},
  {0x32A1, 186889},
  {0x32A2, 186127},
  {0x32A3, 186150},
  {0x32A4, 186426},
  {0x32A5, 186069},
  {0x32A6, 186573},
  {0x32A7, 186550},
  {0x32A8, 186912},
  {0x32A9, 186618},
  {0x32AA, 186835},
  {0x32AB, 187108},
  {0x32AC, 187181},
  {0x32AD, 186224},
  {0x32AE, 186862},
  {0x32AF, 186014},
  {0x32B0, 186715},
  {0x32B1, 190998},
  {0x32B2, 190970},
  {0x32B3, 190797},
  {0x32B4, 190879},
  {0x32B5, 190233},
  {0x32B6, 190396},
  {0x32B7, 190500},
  {0x32B8, 190473},
  {0x32B9, 190307},
  {0x32BA, 190281},
  {0x32BB, 190448},
  {0x32BC, 190421},
  {0x32BD, 190254},
  {0x32BE, 190333},
  {0x32BF, 190175},
  {0x32C0, 529234},
  {0x32C1, 528058},
  {0x32C2, 529351},
  {0x32C3, 526532},
  {0x32C4, 529390},
  {0x32C5, 529313},
  {0x32C6, 529275},
  {0x32C7, 526571},
  {0x32C8, 529510},
  {0x32C9, 529469},
  {0x32CA, 529427},
  {0x32CB, 528016},
  {0x32CC, 900527},
  {0x32CD, 900152},
  {0x32CE, 900179},
  {0x32CF, 638273},
  {0x32D0, 187427},
  {0x32D1, 187565},
  {0x32D2, 188203},
  {0x32D3, 187446},
  {0x32D4, 187884},
  {0x32D5, 187584},
  {0x32D6, 187624},
  {0x32D7, 187664},
  {0x32D8, 187604},
  {0x32D9, 187644},
  {0x32DA, 188003},
  {0x32DB, 188043},
  {0x32DC, 188083},
  {0x32DD, 188023},
  {0x32DE, 188063},
  {0x32DF, 188103},
  {0x32E0, 188143},
  {0x32E1, 188183},
  {0x32E2, 188123},
  {0x32E3, 188163},
  {0x32E4, 187784},
  {0x32E5, 187824},
  {0x32E6, 187864},
  {0x32E7, 187804},
  {0x32E8, 187844},
  {0x32E9, 187465},
  {0x32EA, 187505},
  {0x32EB, 187545},
  {0x32EC, 187485},
  {0x32ED, 187525},
  {0x32EE, 187684},
  {0x32EF, 187724},
  {0x32F0, 187764},
  {0x32F1, 187704},
  {0x32F2, 187744},
  {0x32F3, 188302},
  {0x32F4, 188342},
  {0x32F5, 188322},
  {0x32F6, 187903},
  {0x32F7, 187943},
  {0x32F8, 187983},
  {0x32F9, 187923},
  {0x32FA, 187963},
  {0x32FB, 188222},
  {0x32FC, 188262},
  {0x32FD, 188242},
  {0x32FE, 188282},
  {0x32FF, 900083},
  {0x3300, 899616},
  {0x3301, 899646},
  {0x3302, 899603},
  {0x3303, 899581},
  {0x3304, 900754},
  {0x3305, 900768},
  {0x3306, 902727},
  {0x3307, 900163},
  {0x3308, 900025},
  {0x3309, 901886},
  {0x330A, 901898},
  {0x330B, 900813},
  {0x330C, 900826},
  {0x330D, 900841},
  {0x330E, 900347},
  {0x330F, 900334},
  {0x3310, 900381},
  {0x3311, 900393},
  {0x3312, 901160},
  {0x3313, 900406},
  {0x3314, 900912},
  {0x3315, 900924},
  {0x3316, 900942},
  {0x3317, 900961},
  {0x3318, 900432},
  {0x3319, 900446},
  {0x331A, 901123},
  {0x331B, 901108},
  {0x331C, 900878},
  {0x331D, 901073},
  {0x331E, 901060},
  {0x331F, 902479},
  {0x3320, 902494},
  {0x3321, 902536},
  {0x3322, 902510},
  {0x3323, 902523},
  {0x3324, 899914},
  {0x3325, 899937},
  {0x3326, 900013},
  {0x3327, 902716},
  {0x3328, 901811},
  {0x3329, 901843},
  {0x332A, 900483},
  {0x332B, 902030},
  {0x332C, 902046},
  {0x332D, 899670},
  {0x332E, 902141},
  {0x332F, 902170},
  {0x3330, 902158},
  {0x3331, 899709},
  {0x3332, 900615},
  {0x3333, 900631},
  {0x3334, 899745},
  {0x3335, 900645},
  {0x3336, 900496},
  {0x3337, 902109},
  {0x3338, 902082},
  {0x3339, 900513},
  {0x333A, 902096},
  {0x333B, 902069},
  {0x333C, 899696},
  {0x333D, 902194},
  {0x333E, 899721},
  {0x333F, 900558},
  {0x3340, 902208},
  {0x3341, 900581},
  {0x3342, 900569},
  {0x3343, 901368},
  {0x3344, 901383},
  {0x3345, 901355},
  {0x3346, 901411},
  {0x3347, 901396},
  {0x3348, 901514},
  {0x3349, 901540},
  {0x334A, 901552},
  {0x334B, 901466},
  {0x334C, 901478},
  {0x334D, 901451},
  {0x334E, 903311},
  {0x334F, 903324},
  {0x3350, 903337},
  {0x3351, 902424},
  {0x3352, 902412},
  {0x3353, 902451},
  {0x3354, 902464},
  {0x3355, 902353},
  {0x3356, 902365},
  {0x3357, 902754},
  {0x3358, 529191},
  {0x3359, 528505},
  {0x335A, 529149},
  {0x335B, 528816},
  {0x335C, 528325},
  {0x335D, 528282},
  {0x335E, 528639},
  {0x335F, 528547},
  {0x3360, 528100},
  {0x3361, 528415},
  {0x3362, 528727},
  {0x3363, 528191},
  {0x3364, 528860},
  {0x3365, 528769},
  {0x3366, 528368},
  {0x3367, 528236},
  {0x3368, 528681},
  {0x3369, 528591},
  {0x336A, 528144},
  {0x336B, 528458},
  {0x336C, 528905},
  {0x336D, 529000},
  {0x336E, 529100},
  {0x336F, 529049},
  {0x3370, 528950},
  {0x3371, 900604},
  {0x3372, 899904},
  {0x3373, 899660},
  {0x3374, 899685},
  {0x3375, 901995},
  {0x3376, 902059},
  {0x3377, 899969},
  {0x3378, 899995},
  {0x3379, 899979},
  {0x337A, 900780},
  {0x337B, 900038},
  {0x337C, 900105},
  {0x337D, 900128},
  {0x337E, 900061},
  {0x337F, 899874},
  {0x3380, 902015},
  {0x3381, 901801},
  {0x3382, 901655},
  {0x3383, 901345},
  {0x3384, 900803},
  {0x3385, 900856},
  {0x3386, 901425},
  {0x3387, 900360},
  {0x3388, 899778},
  {0x3389, 900866},
  {0x338A, 902121},
  {0x338B, 901823},
  {0x338C, 901667},
  {0x338D, 901679},
  {0x338E, 901493},
  {0x338F, 900891},
  {0x3390, 900658},
  {0x3391, 900901},
  {0x3392, 901503},
  {0x3393, 900370},
  {0x3394, 902705},
  {0x3395, 901691},
  {0x3396, 901569},
  {0x3397, 899959},
  {0x3398, 900988},
  {0x3399, 900189},
  {0x339A, 901833},
  {0x339B, 901703},
  {0x339C, 901579},
  {0x339D, 899820},
  {0x339E, 900998},
  {0x339F, 901605},
  {0x33A0, 899846},
  {0x33A1, 901328},
  {0x33A2, 901042},
  {0x33A3, 901589},
  {0x33A4, 899830},
  {0x33A5, 901260},
  {0x33A6, 901026},
  {0x33A7, 901288},
  {0x33A8, 901304},
  {0x33A9, 902005},
  {0x33AA, 901087},
  {0x33AB, 901634},
  {0x33AC, 900421},
  {0x33AD, 902298},
  {0x33AE, 902309},
  {0x33AF, 902327},
  {0x33B0, 902268},
  {0x33B1, 901856},
  {0x33B2, 901715},
  {0x33B3, 901645},
  {0x33B4, 902278},
  {0x33B5, 901866},
  {0x33B6, 901727},
  {0x33B7, 901751},
  {0x33B8, 901140},
  {0x33B9, 901761},
  {0x33BA, 902288},
  {0x33BB, 901876},
  {0x33BC, 901739},
  {0x33BD, 901776},
  {0x33BE, 901150},
  {0x33BF, 901786},
  {0x33C0, 900790},
  {0x33C1, 901275},
  {0x33C2, 899593},
  {0x33C3, 899735},
  {0x33C4, 899800},
  {0x33C5, 899810},
  {0x33C6, 899761},
  {0x33C7, 899864},
  {0x33C8, 899927},
  {0x33C9, 900463},
  {0x33CA, 900473},
  {0x33CB, 900594},
  {0x33CC, 900744},
  {0x33CD, 900978},
  {0x33CE, 901008},
  {0x33CF, 901098},
  {0x33D0, 901204},
  {0x33D1, 901214},
  {0x33D2, 901224},
  {0x33D3, 901250},
  {0x33D4, 901435},
  {0x33D5, 901529},
  {0x33D6, 901623},
  {0x33D7, 902131},
  {0x33D8, 902184},
  {0x33D9, 902247},
  {0x33DA, 902258},
  {0x33DB, 902671},
  {0x33DC, 902681},
  {0x33DD, 902767},
  {0x33DE, 902738},
  {0x33DF, 899565},
  {0x33E0, 527007},
  {0x33E1, 527975},
  {0x33E2, 527403},
  {0x33E3, 526831},
  {0x33E4, 526789},
  {0x33E5, 527138},
  {0x33E6, 527048},
  {0x33E7, 526611},
  {0x33E8, 526919},
  {0x33E9, 527224},
  {0x33EA, 526700},
  {0x33EB, 527446},
  {0x33EC, 527265},
  {0x33ED, 526873},
  {0x33EE, 526744},
  {0x33EF, 527179},
  {0x33F0, 527091},
  {0x33F1, 526654},
  {0x33F2, 526961},
  {0x33F3, 527490},
  {0x33F4, 527731},
  {0x33F5, 527927},
  {0x33F6, 527877},
  {0x33F7, 527633},
  {0x33F8, 527584},
  {0x33F9, 527829},
  {0x33FA, 527779},
  {0x33FB, 527534},
  {0x33FC, 527682},
  {0x33FD, 527311},
  {0x33FE, 527355},
  {0x33FF, 900323},
  {0x4DC0, 520778},
  {0x4DC1, 520966},
  {0x4DC2, 519797},
  {0x4DC3, 521123},
  {0x4DC4, 521069},
  {0x4DC5, 519640},
  {0x4DC6, 520665},
  {0x4DC7, 520120},
  {0x4DC8, 520555},
  {0x4DC9, 521047},
  {0x4DCA, 520388},
  {0x4DCB, 520610},
  {0x4DCC, 519908},
  {0x4DCD, 520006},
  {0x4DCE, 520267},
  {0x4DCF, 519884},
  {0x4DD0, 519932},
  {0x4DD1, 521090},
  {0x4DD2, 519505},
  {0x4DD3, 519662},
  {0x4DD4, 519558},
  {0x4DD5, 519987},
  {0x4DD6, 520581},
  {0x4DD7, 520478},
  {0x4DD8, 520220},
  {0x4DD9, 520094},
  {0x4DDA, 520288},
  {0x4DDB, 520061},
  {0x4DDC, 520634},
  {0x4DDD, 520747},
  {0x4DDE, 520172},
  {0x4DDF, 519862},
  {0x4DE0, 520457},
  {0x4DE1, 520036},
  {0x4DE2, 520407},
  {0x4DE3, 519689},
  {0x4DE4, 520811},
  {0x4DE5, 520340},
  {0x4DE6, 520315},
  {0x4DE7, 519747},
  {0x4DE8, 519725},
  {0x4DE9, 520150},
  {0x4DEA, 519586},
  {0x4DEB, 519612},
  {0x4DEC, 519955},
  {0x4DED, 520429},
  {0x4DEE, 520364},
  {0x4DEF, 521025},
  {0x4DF0, 520498},
  {0x4DF1, 520721},
  {0x4DF2, 520687},
  {0x4DF3, 520893},
  {0x4DF4, 519772},
  {0x4DF5, 520933},
  {0x4DF6, 519452},
  {0x4DF7, 520999},
  {0x4DF8, 520835},
  {0x4DF9, 520864},
  {0x4DFA, 519838},
  {0x4DFB, 520243},
  {0x4DFC, 520195},
  {0x4DFD, 520522},
  {0x4DFE, 519475},
  {0x4DFF, 519527},
  {0xA000, 997562},
  {0xA001, 997577},
  {0xA002, 997470},
  {0xA003, 997547},
  {0xA004, 997515},
  {0xA005, 997531},
  {0xA006, 997484},
  {0xA007, 997499},
  {0xA008, 991075},
  {0xA009, 991090},
  {0xA00A, 991046},
  {0xA00B, 991060},
  {0xA00C, 1007271},
  {0xA00D, 1007240},
  {0xA00E, 1007255},
  {0xA00F, 1003548},
  {0xA010, 1003563},
  {0xA011, 1003519},
  {0xA012, 1003533},
  {0xA013, 994086},
  {0xA014, 994072},
  {0xA015, 1007924},
  {0xA016, 991857},
  {0xA017, 991873},
  {0xA018, 991759},
  {0xA019, 991841},
  {0xA01A, 991807},
  {0xA01B, 991824},
  {0xA01C, 991774},
  {0xA01D, 991790},
  {0xA01E, 991136},
  {0xA01F, 991152},
  {0xA020, 991105},
  {0xA021, 991120},
  {0xA022, 992000},
  {0xA023, 991967},
  {0xA024, 991983},
  {0xA025, 991920},
  {0xA026, 991936},
  {0xA027, 991889},
  {0xA028, 991904},
  {0xA029, 991743},
  {0xA02A, 991712},
  {0xA02B, 991727},
  {0xA02C, 992066},
  {0xA02D, 992082},
  {0xA02E, 991952},
  {0xA02F, 992017},
  {0xA030, 992049},
  {0xA031, 992033},
  {0xA032, 992162},
  {0xA033, 992178},
  {0xA034, 992098},
  {0xA035, 992113},
  {0xA036, 992145},
  {0xA037, 992129},
  {0xA038, 1003722},
  {0xA039, 1003738},
  {0xA03A, 1003641},
  {0xA03B, 1003706},
  {0xA03C, 1003689},
  {0xA03D, 1003656},
  {0xA03E, 1003672},
  {0xA03F, 1003609},
  {0xA040, 1003625},
  {0xA041, 1003578},
  {0xA042, 1003593},
  {0xA043, 1003865},
  {0xA044, 1003832},
  {0xA045, 1003848},
  {0xA046, 1003785},
  {0xA047, 1003801},
  {0xA048, 1003754},
  {0xA049, 1003769},
  {0xA04A, 1003931},
  {0xA04B, 1003947},
  {0xA04C, 1003817},
  {0xA04D, 1003882},
  {0xA04E, 1003914},
  {0xA04F, 1003898},
  {0xA050, 1004027},
  {0xA051, 1004043},
  {0xA052, 1003963},
  {0xA053, 1003978},
  {0xA054, 1004010},
  {0xA055, 1003994},
  {0xA056, 991389},
  {0xA057, 991406},
  {0xA058, 991285},
  {0xA059, 991372},
  {0xA05A, 991336},
  {0xA05B, 991354},
  {0xA05C, 991301},
  {0xA05D, 991318},
  {0xA05E, 991201},
  {0xA05F, 991218},
  {0xA060, 991168},
  {0xA061, 991184},
  {0xA062, 991541},
  {0xA063, 991506},
  {0xA064, 991523},
  {0xA065, 991456},
  {0xA066, 991473},
  {0xA067, 991423},
  {0xA068, 991439},
  {0xA069, 991268},
  {0xA06A, 991235},
  {0xA06B, 991251},
  {0xA06C, 991611},
  {0xA06D, 991628},
  {0xA06E, 991490},
  {0xA06F, 991559},
  {0xA070, 991593},
  {0xA071, 991576},
  {0xA072, 991678},
  {0xA073, 991695},
  {0xA074, 991645},
  {0xA075, 991661},
  {0xA076, 1000551},
  {0xA077, 1000568},
  {0xA078, 1000465},
  {0xA079, 1000534},
  {0xA07A, 1000516},
  {0xA07B, 1000481},
  {0xA07C, 1000498},
  {0xA07D, 1000431},
  {0xA07E, 1000448},
  {0xA07F, 1000398},
  {0xA080, 1000414},
  {0xA081, 1000618},
  {0xA082, 1000635},
  {0xA083, 1000585},
  {0xA084, 1000601},
  {0xA085, 1000720},
  {0xA086, 1000737},
  {0xA087, 1000652},
  {0xA088, 1000668},
  {0xA089, 1000702},
  {0xA08A, 1000685},
  {0xA08B, 1000822},
  {0xA08C, 1000839},
  {0xA08D, 1000754},
  {0xA08E, 1000770},
  {0xA08F, 1000804},
  {0xA090, 1000787},
  {0xA091, 996248},
  {0xA092, 996265},
  {0xA093, 996162},
  {0xA094, 996231},
  {0xA095, 996213},
  {0xA096, 996178},
  {0xA097, 996195},
  {0xA098, 996128},
  {0xA099, 996145},
  {0xA09A, 996095},
  {0xA09B, 996111},
  {0xA09C, 996400},
  {0xA09D, 996365},
  {0xA09E, 996382},
  {0xA09F, 996315},
  {0xA0A0, 996332},
  {0xA0A1, 996282},
  {0xA0A2, 996298},
  {0xA0A3, 996470},
  {0xA0A4, 996487},
  {0xA0A5, 996349},
  {0xA0A6, 996418},
  {0xA0A7, 996452},
  {0xA0A8, 996435},
  {0xA0A9, 996572},
  {0xA0AA, 996504},
  {0xA0AB, 996520},
  {0xA0AC, 996554},
  {0xA0AD, 996537},
  {0xA0AE, 1000030},
  {0xA0AF, 1000046},
  {0xA0B0, 999949},
  {0xA0B1, 1000014},
  {0xA0B2, 999997},
  {0xA0B3, 999964},
  {0xA0B4, 999980},
  {0xA0B5, 999512},
  {0xA0B6, 999528},
  {0xA0B7, 999481},
  {0xA0B8, 999496},
  {0xA0B9, 1000173},
  {0xA0BA, 1000190},
  {0xA0BB, 1000140},
  {0xA0BC, 1000156},
  {0xA0BD, 1000093},
  {0xA0BE, 1000109},
  {0xA0BF, 1000062},
  {0xA0C0, 1000077},
  {0xA0C1, 999559},
  {0xA0C2, 999544},
  {0xA0C3, 1000256},
  {0xA0C4, 1000272},
  {0xA0C5, 1000125},
  {0xA0C6, 1000207},
  {0xA0C7, 1000239},
  {0xA0C8, 1000223},
  {0xA0C9, 1000319},
  {0xA0CA, 1000335},
  {0xA0CB, 1000288},
  {0xA0CC, 1000303},
  {0xA0CD, 994195},
  {0xA0CE, 994211},
  {0xA0CF, 994164},
  {0xA0D0, 994179},
  {0xA0D1, 994132},
  {0xA0D2, 994148},
  {0xA0D3, 994101},
  {0xA0D4, 994116},
  {0xA0D5, 994258},
  {0xA0D6, 994227},
  {0xA0D7, 994242},
  {0xA0D8, 994338},
  {0xA0D9, 994354},
  {0xA0DA, 994274},
  {0xA0DB, 994289},
  {0xA0DC, 994321},
  {0xA0DD, 994305},
  {0xA0DE, 994401},
  {0xA0DF, 994417},
  {0xA0E0, 994370},
  {0xA0E1, 994385},
  {0xA0E2, 1007480},
  {0xA0E3, 1007496},
  {0xA0E4, 1007382},
  {0xA0E5, 1007464},
  {0xA0E6, 1007430},
  {0xA0E7, 1007447},
  {0xA0E8, 1007397},
  {0xA0E9, 1007413},
  {0xA0EA, 1007318},
  {0xA0EB, 1007334},
  {0xA0EC, 1007287},
  {0xA0ED, 1007302},
  {0xA0EE, 1007543},
  {0xA0EF, 1007559},
  {0xA0F0, 1007512},
  {0xA0F1, 1007527},
  {0xA0F2, 1007366},
  {0xA0F3, 1007350},
  {0xA0F4, 1007639},
  {0xA0F5, 1007655},
  {0xA0F6, 1007575},
  {0xA0F7, 1007590},
  {0xA0F8, 1007622},
  {0xA0F9, 1007606},
  {0xA0FA, 1007735},
  {0xA0FB, 1007751},
  {0xA0FC, 1007671},
  {0xA0FD, 1007686},
  {0xA0FE, 1007718},
  {0xA0FF, 1007702},
  {0xA100, 993848},
  {0xA101, 993864},
  {0xA102, 993767},
  {0xA103, 993832},
  {0xA104, 993815},
  {0xA105, 993782},
  {0xA106, 993798},
  {0xA107, 993229},
  {0xA108, 993245},
  {0xA109, 993198},
  {0xA10A, 993213},
  {0xA10B, 993974},
  {0xA10C, 993958},
  {0xA10D, 993911},
  {0xA10E, 993927},
  {0xA10F, 993880},
  {0xA110, 993895},
  {0xA111, 993751},
  {0xA112, 993720},
  {0xA113, 993735},
  {0xA114, 994040},
  {0xA115, 994056},
  {0xA116, 993943},
  {0xA117, 993991},
  {0xA118, 994023},
  {0xA119, 994007},
  {0xA11A, 1006982},
  {0xA11B, 1006998},
  {0xA11C, 1006901},
  {0xA11D, 1006966},
  {0xA11E, 1006949},
  {0xA11F, 1006916},
  {0xA120, 1006932},
  {0xA121, 1006822},
  {0xA122, 1006838},
  {0xA123, 1006791},
  {0xA124, 1006806},
  {0xA125, 1007125},
  {0xA126, 1007142},
  {0xA127, 1007092},
  {0xA128, 1007108},
  {0xA129, 1007045},
  {0xA12A, 1007061},
  {0xA12B, 1007014},
  {0xA12C, 1007029},
  {0xA12D, 1006885},
  {0xA12E, 1006854},
  {0xA12F, 1006869},
  {0xA130, 1007208},
  {0xA131, 1007224},
  {0xA132, 1007077},
  {0xA133, 1007159},
  {0xA134, 1007191},
  {0xA135, 1007175},
  {0xA136, 993464},
  {0xA137, 993481},
  {0xA138, 993378},
  {0xA139, 993447},
  {0xA13A, 993429},
  {0xA13B, 993394},
  {0xA13C, 993411},
  {0xA13D, 993294},
  {0xA13E, 993311},
  {0xA13F, 993261},
  {0xA140, 993277},
  {0xA141, 993616},
  {0xA142, 993581},
  {0xA143, 993598},
  {0xA144, 993531},
  {0xA145, 993548},
  {0xA146, 993498},
  {0xA147, 993514},
  {0xA148, 993361},
  {0xA149, 993328},
  {0xA14A, 993344},
  {0xA14B, 993686},
  {0xA14C, 993703},
  {0xA14D, 993565},
  {0xA14E, 993634},
  {0xA14F, 993668},
  {0xA150, 993651},
  {0xA151, 1001041},
  {0xA152, 1001058},
  {0xA153, 1000973},
  {0xA154, 1001024},
  {0xA155, 1001006},
  {0xA156, 1000989},
  {0xA157, 1000889},
  {0xA158, 1000906},
  {0xA159, 1000856},
  {0xA15A, 1000872},
  {0xA15B, 1001108},
  {0xA15C, 1001125},
  {0xA15D, 1001075},
  {0xA15E, 1001091},
  {0xA15F, 1000956},
  {0xA160, 1000923},
  {0xA161, 1000939},
  {0xA162, 1001210},
  {0xA163, 1001227},
  {0xA164, 1001142},
  {0xA165, 1001158},
  {0xA166, 1001192},
  {0xA167, 1001175},
  {0xA168, 996810},
  {0xA169, 996827},
  {0xA16A, 996706},
  {0xA16B, 996793},
  {0xA16C, 996757},
  {0xA16D, 996775},
  {0xA16E, 996722},
  {0xA16F, 996739},
  {0xA170, 996622},
  {0xA171, 996639},
  {0xA172, 996589},
  {0xA173, 996605},
  {0xA174, 996912},
  {0xA175, 996895},
  {0xA176, 996861},
  {0xA177, 996878},
  {0xA178, 996844},
  {0xA179, 996689},
  {0xA17A, 996656},
  {0xA17B, 996672},
  {0xA17C, 996930},
  {0xA17D, 1001662},
  {0xA17E, 1001678},
  {0xA17F, 1001581},
  {0xA180, 1001646},
  {0xA181, 1001629},
  {0xA182, 1001596},
  {0xA183, 1001612},
  {0xA184, 1000382},
  {0xA185, 1000351},
  {0xA186, 1000366},
  {0xA187, 1002620},
  {0xA188, 1002587},
  {0xA189, 1002603},
  {0xA18A, 1002152},
  {0xA18B, 1002168},
  {0xA18C, 1002121},
  {0xA18D, 1002136},
  {0xA18E, 1001275},
  {0xA18F, 1001244},
  {0xA190, 1001259},
  {0xA191, 1002686},
  {0xA192, 1002702},
  {0xA193, 1002572},
  {0xA194, 1002637},
  {0xA195, 1002669},
  {0xA196, 1002653},
  {0xA197, 995754},
  {0xA198, 995771},
  {0xA199, 995668},
  {0xA19A, 995737},
  {0xA19B, 995719},
  {0xA19C, 995684},
  {0xA19D, 995701},
  {0xA19E, 995584},
  {0xA19F, 995601},
  {0xA1A0, 995551},
  {0xA1A1, 995567},
  {0xA1A2, 995889},
  {0xA1A3, 995854},
  {0xA1A4, 995871},
  {0xA1A5, 995821},
  {0xA1A6, 995788},
  {0xA1A7, 995804},
  {0xA1A8, 995651},
  {0xA1A9, 995618},
  {0xA1AA, 995634},
  {0xA1AB, 995959},
  {0xA1AC, 995976},
  {0xA1AD, 995838},
  {0xA1AE, 995907},
  {0xA1AF, 995941},
  {0xA1B0, 995924},
  {0xA1B1, 996061},
  {0xA1B2, 996078},
  {0xA1B3, 995993},
  {0xA1B4, 996009},
  {0xA1B5, 996043},
  {0xA1B6, 996026},
  {0xA1B7, 999127},
  {0xA1B8, 999143},
  {0xA1B9, 999029},
  {0xA1BA, 999111},
  {0xA1BB, 999077},
  {0xA1BC, 999094},
  {0xA1BD, 999044},
  {0xA1BE, 999060},
  {0xA1BF, 998950},
  {0xA1C0, 998966},
  {0xA1C1, 998919},
  {0xA1C2, 998934},
  {0xA1C3, 999270},
  {0xA1C4, 999287},
  {0xA1C5, 999237},
  {0xA1C6, 999253},
  {0xA1C7, 999190},
  {0xA1C8, 999206},
  {0xA1C9, 999159},
  {0xA1CA, 999174},
  {0xA1CB, 999013},
  {0xA1CC, 998982},
  {0xA1CD, 998997},
  {0xA1CE, 999353},
  {0xA1CF, 999369},
  {0xA1D0, 999222},
  {0xA1D1, 999304},
  {0xA1D2, 999336},
  {0xA1D3, 999320},
  {0xA1D4, 999449},
  {0xA1D5, 999465},
  {0xA1D6, 999385},
  {0xA1D7, 999400},
  {0xA1D8, 999432},
  {0xA1D9, 999416},
  {0xA1DA, 995134},
  {0xA1DB, 995150},
  {0xA1DC, 995036},
  {0xA1DD, 995118},
  {0xA1DE, 995084},
  {0xA1DF, 995101},
  {0xA1E0, 995051},
  {0xA1E1, 995067},
  {0xA1E2, 994464},
  {0xA1E3, 994480},
  {0xA1E4, 994433},
  {0xA1E5, 994448},
  {0xA1E6, 995277},
  {0xA1E7, 995294},
  {0xA1E8, 995244},
  {0xA1E9, 995260},
  {0xA1EA, 995197},
  {0xA1EB, 995213},
  {0xA1EC, 995166},
  {0xA1ED, 995181},
  {0xA1EE, 994527},
  {0xA1EF, 994543},
  {0xA1F0, 994496},
  {0xA1F1, 994511},
  {0xA1F2, 995360},
  {0xA1F3, 995376},
  {0xA1F4, 995229},
  {0xA1F5, 995311},
  {0xA1F6, 995343},
  {0xA1F7, 995327},
  {0xA1F8, 998678},
  {0xA1F9, 998694},
  {0xA1FA, 998597},
  {0xA1FB, 998662},
  {0xA1FC, 998645},
  {0xA1FD, 998612},
  {0xA1FE, 998628},
  {0xA1FF, 998502},
  {0xA200, 998518},
  {0xA201, 998471},
  {0xA202, 998486},
  {0xA203, 998821},
  {0xA204, 998788},
  {0xA205, 998804},
  {0xA206, 998741},
  {0xA207, 998757},
  {0xA208, 998710},
  {0xA209, 998725},
  {0xA20A, 998565},
  {0xA20B, 998581},
  {0xA20C, 998534},
  {0xA20D, 998549},
  {0xA20E, 998887},
  {0xA20F, 998903},
  {0xA210, 998773},
  {0xA211, 998838},
  {0xA212, 998870},
  {0xA213, 998854},
  {0xA214, 994762},
  {0xA215, 994779},
  {0xA216, 994693},
  {0xA217, 994744},
  {0xA218, 994709},
  {0xA219, 994726},
  {0xA21A, 994592},
  {0xA21B, 994609},
  {0xA21C, 994559},
  {0xA21D, 994575},
  {0xA21E, 994914},
  {0xA21F, 994932},
  {0xA220, 994879},
  {0xA221, 994896},
  {0xA222, 994829},
  {0xA223, 994846},
  {0xA224, 994796},
  {0xA225, 994812},
  {0xA226, 994659},
  {0xA227, 994676},
  {0xA228, 994626},
  {0xA229, 994642},
  {0xA22A, 995002},
  {0xA22B, 995019},
  {0xA22C, 994863},
  {0xA22D, 994950},
  {0xA22E, 994984},
  {0xA22F, 994967},
  {0xA230, 999709},
  {0xA231, 999692},
  {0xA232, 999608},
  {0xA233, 999625},
  {0xA234, 999575},
  {0xA235, 999591},
  {0xA236, 999845},
  {0xA237, 999810},
  {0xA238, 999827},
  {0xA239, 999760},
  {0xA23A, 999777},
  {0xA23B, 999727},
  {0xA23C, 999743},
  {0xA23D, 999675},
  {0xA23E, 999642},
  {0xA23F, 999658},
  {0xA240, 999915},
  {0xA241, 999932},
  {0xA242, 999794},
  {0xA243, 999863},
  {0xA244, 999897},
  {0xA245, 999880},
  {0xA246, 997298},
  {0xA247, 997315},
  {0xA248, 997194},
  {0xA249, 997281},
  {0xA24A, 997245},
  {0xA24B, 997263},
  {0xA24C, 997210},
  {0xA24D, 997227},
  {0xA24E, 997110},
  {0xA24F, 997127},
  {0xA250, 997077},
  {0xA251, 997093},
  {0xA252, 997434},
  {0xA253, 997452},
  {0xA254, 997399},
  {0xA255, 997416},
  {0xA256, 997365},
  {0xA257, 997382},
  {0xA258, 997332},
  {0xA259, 997348},
  {0xA25A, 997177},
  {0xA25B, 997144},
  {0xA25C, 997160},
  {0xA25D, 1001443},
  {0xA25E, 1001408},
  {0xA25F, 1001425},
  {0xA260, 1001324},
  {0xA261, 1001341},
  {0xA262, 1001291},
  {0xA263, 1001307},
  {0xA264, 1001545},
  {0xA265, 1001563},
  {0xA266, 1001528},
  {0xA267, 1001494},
  {0xA268, 1001511},
  {0xA269, 1001461},
  {0xA26A, 1001477},
  {0xA26B, 1001391},
  {0xA26C, 1001358},
  {0xA26D, 1001374},
  {0xA26E, 995535},
  {0xA26F, 995518},
  {0xA270, 995502},
  {0xA271, 995423},
  {0xA272, 995439},
  {0xA273, 995392},
  {0xA274, 995407},
  {0xA275, 997043},
  {0xA276, 997060},
  {0xA277, 997010},
  {0xA278, 997026},
  {0xA279, 996978},
  {0xA27A, 996994},
  {0xA27B, 996947},
  {0xA27C, 996962},
  {0xA27D, 995486},
  {0xA27E, 995455},
  {0xA27F, 995470},
  {0xA280, 1007798},
  {0xA281, 1007814},
  {0xA282, 1007767},
  {0xA283, 1007782},
  {0xA284, 1007972},
  {0xA285, 1007939},
  {0xA286, 1007955},
  {0xA287, 1007908},
  {0xA288, 1007877},
  {0xA289, 1007892},
  {0xA28A, 1007861},
  {0xA28B, 1007830},
  {0xA28C, 1007845},
  {0xA28D, 1009412},
  {0xA28E, 1009428},
  {0xA28F, 1009331},
  {0xA290, 1009396},
  {0xA291, 1009379},
  {0xA292, 1009346},
  {0xA293, 1009362},
  {0xA294, 1008794},
  {0xA295, 1008810},
  {0xA296, 1008763},
  {0xA297, 1008778},
  {0xA298, 1009555},
  {0xA299, 1009522},
  {0xA29A, 1009538},
  {0xA29B, 1009475},
  {0xA29C, 1009491},
  {0xA29D, 1009444},
  {0xA29E, 1009459},
  {0xA29F, 1008857},
  {0xA2A0, 1008826},
  {0xA2A1, 1008841},
  {0xA2A2, 1009621},
  {0xA2A3, 1009637},
  {0xA2A4, 1009507},
  {0xA2A5, 1009572},
  {0xA2A6, 1009604},
  {0xA2A7, 1009588},
  {0xA2A8, 1009717},
  {0xA2A9, 1009733},
  {0xA2AA, 1009653},
  {0xA2AB, 1009668},
  {0xA2AC, 1009700},
  {0xA2AD, 1009684},
  {0xA2AE, 992861},
  {0xA2AF, 992877},
  {0xA2B0, 992763},
  {0xA2B1, 992845},
  {0xA2B2, 992811},
  {0xA2B3, 992828},
  {0xA2B4, 992778},
  {0xA2B5, 992794},
  {0xA2B6, 992225},
  {0xA2B7, 992241},
  {0xA2B8, 992194},
  {0xA2B9, 992209},
  {0xA2BA, 993004},
  {0xA2BB, 992971},
  {0xA2BC, 992987},
  {0xA2BD, 992924},
  {0xA2BE, 992940},
  {0xA2BF, 992893},
  {0xA2C0, 992908},
  {0xA2C1, 992288},
  {0xA2C2, 992257},
  {0xA2C3, 992272},
  {0xA2C4, 993070},
  {0xA2C5, 993086},
  {0xA2C6, 992956},
  {0xA2C7, 993021},
  {0xA2C8, 993053},
  {0xA2C9, 993037},
  {0xA2CA, 993166},
  {0xA2CB, 993182},
  {0xA2CC, 993102},
  {0xA2CD, 993117},
  {0xA2CE, 993149},
  {0xA2CF, 993133},
  {0xA2D0, 1009970},
  {0xA2D1, 1009987},
  {0xA2D2, 1009866},
  {0xA2D3, 1009953},
  {0xA2D4, 1009917},
  {0xA2D5, 1009935},
  {0xA2D6, 1009882},
  {0xA2D7, 1009899},
  {0xA2D8, 1009782},
  {0xA2D9, 1009799},
  {0xA2DA, 1009749},
  {0xA2DB, 1009765},
  {0xA2DC, 1010037},
  {0xA2DD, 1010004},
  {0xA2DE, 1010020},
  {0xA2DF, 1009849},
  {0xA2E0, 1009816},
  {0xA2E1, 1009832},
  {0xA2E2, 1010122},
  {0xA2E3, 1010054},
  {0xA2E4, 1010070},
  {0xA2E5, 1010104},
  {0xA2E6, 1010087},
  {0xA2E7, 1010207},
  {0xA2E8, 1010224},
  {0xA2E9, 1010139},
  {0xA2EA, 1010155},
  {0xA2EB, 1010189},
  {0xA2EC, 1010172},
  {0xA2ED, 1003229},
  {0xA2EE, 1003246},
  {0xA2EF, 1003143},
  {0xA2F0, 1003212},
  {0xA2F1, 1003194},
  {0xA2F2, 1003159},
  {0xA2F3, 1003176},
  {0xA2F4, 1003076},
  {0xA2F5, 1003093},
  {0xA2F6, 1003043},
  {0xA2F7, 1003059},
  {0xA2F8, 1003330},
  {0xA2F9, 1003313},
  {0xA2FA, 1003280},
  {0xA2FB, 1003263},
  {0xA2FC, 1003126},
  {0xA2FD, 1003110},
  {0xA2FE, 1003400},
  {0xA2FF, 1003297},
  {0xA300, 1003348},
  {0xA301, 1003382},
  {0xA302, 1003365},
  {0xA303, 1003485},
  {0xA304, 1003502},
  {0xA305, 1003417},
  {0xA306, 1003433},
  {0xA307, 1003467},
  {0xA308, 1003450},
  {0xA309, 1005981},
  {0xA30A, 1005997},
  {0xA30B, 1005900},
  {0xA30C, 1005965},
  {0xA30D, 1005948},
  {0xA30E, 1005915},
  {0xA30F, 1005931},
  {0xA310, 1005363},
  {0xA311, 1005379},
  {0xA312, 1005332},
  {0xA313, 1005347},
  {0xA314, 1006597},
  {0xA315, 1006564},
  {0xA316, 1006580},
  {0xA317, 1006044},
  {0xA318, 1006060},
  {0xA319, 1006013},
  {0xA31A, 1006028},
  {0xA31B, 1005426},
  {0xA31C, 1005395},
  {0xA31D, 1005410},
  {0xA31E, 1006663},
  {0xA31F, 1006679},
  {0xA320, 1006549},
  {0xA321, 1006614},
  {0xA322, 1006646},
  {0xA323, 1006630},
  {0xA324, 1006759},
  {0xA325, 1006775},
  {0xA326, 1006695},
  {0xA327, 1006710},
  {0xA328, 1006742},
  {0xA329, 1006726},
  {0xA32A, 1006279},
  {0xA32B, 1006296},
  {0xA32C, 1006193},
  {0xA32D, 1006262},
  {0xA32E, 1006244},
  {0xA32F, 1006209},
  {0xA330, 1006226},
  {0xA331, 1006109},
  {0xA332, 1006126},
  {0xA333, 1006076},
  {0xA334, 1006092},
  {0xA335, 1006346},
  {0xA336, 1006363},
  {0xA337, 1006313},
  {0xA338, 1006329},
  {0xA339, 1006176},
  {0xA33A, 1006143},
  {0xA33B, 1006159},
  {0xA33C, 1006413},
  {0xA33D, 1006430},
  {0xA33E, 1006380},
  {0xA33F, 1006396},
  {0xA340, 1006515},
  {0xA341, 1006532},
  {0xA342, 1006447},
  {0xA343, 1006463},
  {0xA344, 1006497},
  {0xA345, 1006480},
  {0xA346, 1008906},
  {0xA347, 1008923},
  {0xA348, 1008873},
  {0xA349, 1008889},
  {0xA34A, 1009125},
  {0xA34B, 1009090},
  {0xA34C, 1009107},
  {0xA34D, 1009040},
  {0xA34E, 1009057},
  {0xA34F, 1009007},
  {0xA350, 1009023},
  {0xA351, 1008973},
  {0xA352, 1008990},
  {0xA353, 1008940},
  {0xA354, 1008956},
  {0xA355, 1009195},
  {0xA356, 1009212},
  {0xA357, 1009074},
  {0xA358, 1009143},
  {0xA359, 1009177},
  {0xA35A, 1009160},
  {0xA35B, 1009297},
  {0xA35C, 1009314},
  {0xA35D, 1009229},
  {0xA35E, 1009245},
  {0xA35F, 1009279},
  {0xA360, 1009262},
  {0xA361, 992337},
  {0xA362, 992354},
  {0xA363, 992304},
  {0xA364, 992320},
  {0xA365, 992556},
  {0xA366, 992574},
  {0xA367, 992521},
  {0xA368, 992538},
  {0xA369, 992471},
  {0xA36A, 992488},
  {0xA36B, 992438},
  {0xA36C, 992454},
  {0xA36D, 992404},
  {0xA36E, 992421},
  {0xA36F, 992371},
  {0xA370, 992387},
  {0xA371, 992644},
  {0xA372, 992505},
  {0xA373, 992592},
  {0xA374, 992626},
  {0xA375, 992609},
  {0xA376, 992729},
  {0xA377, 992746},
  {0xA378, 992661},
  {0xA379, 992677},
  {0xA37A, 992711},
  {0xA37B, 992694},
  {0xA37C, 1004700},
  {0xA37D, 1004684},
  {0xA37E, 1004884},
  {0xA37F, 1004867},
  {0xA380, 1004817},
  {0xA381, 1004834},
  {0xA382, 1004784},
  {0xA383, 1004800},
  {0xA384, 1004750},
  {0xA385, 1004767},
  {0xA386, 1004717},
  {0xA387, 1004733},
  {0xA388, 1004954},
  {0xA389, 1004971},
  {0xA38A, 1004851},
  {0xA38B, 1004902},
  {0xA38C, 1004936},
  {0xA38D, 1004919},
  {0xA38E, 1005056},
  {0xA38F, 1005073},
  {0xA390, 1004988},
  {0xA391, 1005004},
  {0xA392, 1005038},
  {0xA393, 1005021},
  {0xA394, 1002217},
  {0xA395, 1002234},
  {0xA396, 1002184},
  {0xA397, 1002200},
  {0xA398, 1002351},
  {0xA399, 1002318},
  {0xA39A, 1002334},
  {0xA39B, 1002284},
  {0xA39C, 1002301},
  {0xA39D, 1002251},
  {0xA39E, 1002267},
  {0xA39F, 1002436},
  {0xA3A0, 1002453},
  {0xA3A1, 1002368},
  {0xA3A2, 1002384},
  {0xA3A3, 1002418},
  {0xA3A4, 1002401},
  {0xA3A5, 1002538},
  {0xA3A6, 1002555},
  {0xA3A7, 1002470},
  {0xA3A8, 1002486},
  {0xA3A9, 1002520},
  {0xA3AA, 1002503},
  {0xA3AB, 1005475},
  {0xA3AC, 1005492},
  {0xA3AD, 1005442},
  {0xA3AE, 1005458},
  {0xA3AF, 1005694},
  {0xA3B0, 1005659},
  {0xA3B1, 1005676},
  {0xA3B2, 1005609},
  {0xA3B3, 1005626},
  {0xA3B4, 1005576},
  {0xA3B5, 1005592},
  {0xA3B6, 1005542},
  {0xA3B7, 1005559},
  {0xA3B8, 1005509},
  {0xA3B9, 1005525},
  {0xA3BA, 1005764},
  {0xA3BB, 1005781},
  {0xA3BC, 1005643},
  {0xA3BD, 1005712},
  {0xA3BE, 1005746},
  {0xA3BF, 1005729},
  {0xA3C0, 1005866},
  {0xA3C1, 1005883},
  {0xA3C2, 1005798},
  {0xA3C3, 1005814},
  {0xA3C4, 1005848},
  {0xA3C5, 1005831},
  {0xA3C6, 1004542},
  {0xA3C7, 1004558},
  {0xA3C8, 1004511},
  {0xA3C9, 1004526},
  {0xA3CA, 1005138},
  {0xA3CB, 1005105},
  {0xA3CC, 1005121},
  {0xA3CD, 1004652},
  {0xA3CE, 1004668},
  {0xA3CF, 1004621},
  {0xA3D0, 1004636},
  {0xA3D1, 1004605},
  {0xA3D2, 1004574},
  {0xA3D3, 1004589},
  {0xA3D4, 1005204},
  {0xA3D5, 1005220},
  {0xA3D6, 1005090},
  {0xA3D7, 1005155},
  {0xA3D8, 1005187},
  {0xA3D9, 1005171},
  {0xA3DA, 1005300},
  {0xA3DB, 1005316},
  {0xA3DC, 1005236},
  {0xA3DD, 1005251},
  {0xA3DE, 1005283},
  {0xA3DF, 1005267},
  {0xA3E0, 997690},
  {0xA3E1, 997706},
  {0xA3E2, 997592},
  {0xA3E3, 997674},
  {0xA3E4, 997640},
  {0xA3E5, 997657},
  {0xA3E6, 997607},
  {0xA3E7, 997623},
  {0xA3E8, 998260},
  {0xA3E9, 998277},
  {0xA3EA, 998227},
  {0xA3EB, 998243},
  {0xA3EC, 998180},
  {0xA3ED, 998196},
  {0xA3EE, 998149},
  {0xA3EF, 998164},
  {0xA3F0, 998343},
  {0xA3F1, 998359},
  {0xA3F2, 998212},
  {0xA3F3, 998294},
  {0xA3F4, 998326},
  {0xA3F5, 998310},
  {0xA3F6, 998439},
  {0xA3F7, 998455},
  {0xA3F8, 998375},
  {0xA3F9, 998390},
  {0xA3FA, 998422},
  {0xA3FB, 998406},
  {0xA3FC, 1004157},
  {0xA3FD, 1004173},
  {0xA3FE, 1004059},
  {0xA3FF, 1004141},
  {0xA400, 1004107},
  {0xA401, 1004124},
  {0xA402, 1004074},
  {0xA403, 1004090},
  {0xA404, 1004300},
  {0xA405, 1004317},
  {0xA406, 1004267},
  {0xA407, 1004283},
  {0xA408, 1004220},
  {0xA409, 1004236},
  {0xA40A, 1004189},
  {0xA40B, 1004204},
  {0xA40C, 1004383},
  {0xA40D, 1004399},
  {0xA40E, 1004252},
  {0xA40F, 1004334},
  {0xA410, 1004366},
  {0xA411, 1004350},
  {0xA412, 1004479},
  {0xA413, 1004495},
  {0xA414, 1004415},
  {0xA415, 1004430},
  {0xA416, 1004462},
  {0xA417, 1004446},
  {0xA418, 997826},
  {0xA419, 997843},
  {0xA41A, 997722},
  {0xA41B, 997809},
  {0xA41C, 997773},
  {0xA41D, 997791},
  {0xA41E, 997738},
  {0xA41F, 997755},
  {0xA420, 997978},
  {0xA421, 997943},
  {0xA422, 997960},
  {0xA423, 997893},
  {0xA424, 997910},
  {0xA425, 997860},
  {0xA426, 997876},
  {0xA427, 998048},
  {0xA428, 998065},
  {0xA429, 997927},
  {0xA42A, 997996},
  {0xA42B, 998030},
  {0xA42C, 998013},
  {0xA42D, 998115},
  {0xA42E, 998132},
  {0xA42F, 998082},
  {0xA430, 998098},
  {0xA431, 1001798},
  {0xA432, 1001815},
  {0xA433, 1001694},
  {0xA434, 1001781},
  {0xA435, 1001745},
  {0xA436, 1001763},
  {0xA437, 1001710},
  {0xA438, 1001727},
  {0xA439, 1001932},
  {0xA43A, 1001915},
  {0xA43B, 1001865},
  {0xA43C, 1001882},
  {0xA43D, 1001832},
  {0xA43E, 1001848},
  {0xA43F, 1002002},
  {0xA440, 1001899},
  {0xA441, 1001950},
  {0xA442, 1001984},
  {0xA443, 1001967},
  {0xA444, 1002087},
  {0xA445, 1002104},
  {0xA446, 1002019},
  {0xA447, 1002035},
  {0xA448, 1002069},
  {0xA449, 1002052},
  {0xA44A, 1002822},
  {0xA44B, 1002839},
  {0xA44C, 1002718},
  {0xA44D, 1002805},
  {0xA44E, 1002769},
  {0xA44F, 1002787},
  {0xA450, 1002734},
  {0xA451, 1002751},
  {0xA452, 1002974},
  {0xA453, 1002939},
  {0xA454, 1002956},
  {0xA455, 1002889},
  {0xA456, 1002906},
  {0xA457, 1002856},
  {0xA458, 1002872},
  {0xA459, 1003009},
  {0xA45A, 1003026},
  {0xA45B, 1002923},
  {0xA45C, 1002992},
  {0xA45D, 1008087},
  {0xA45E, 1008103},
  {0xA45F, 1007989},
  {0xA460, 1008071},
  {0xA461, 1008037},
  {0xA462, 1008054},
  {0xA463, 1008004},
  {0xA464, 1008020},
  {0xA465, 1008198},
  {0xA466, 1008182},
  {0xA467, 1008150},
  {0xA468, 1008166},
  {0xA469, 1008119},
  {0xA46A, 1008134},
  {0xA46B, 1008279},
  {0xA46C, 1008295},
  {0xA46D, 1008215},
  {0xA46E, 1008230},
  {0xA46F, 1008262},
  {0xA470, 1008246},
  {0xA471, 1008409},
  {0xA472, 1008425},
  {0xA473, 1008311},
  {0xA474, 1008393},
  {0xA475, 1008359},
  {0xA476, 1008376},
  {0xA477, 1008326},
  {0xA478, 1008342},
  {0xA479, 1008552},
  {0xA47A, 1008569},
  {0xA47B, 1008519},
  {0xA47C, 1008535},
  {0xA47D, 1008472},
  {0xA47E, 1008488},
  {0xA47F, 1008441},
  {0xA480, 1008456},
  {0xA481, 1008635},
  {0xA482, 1008651},
  {0xA483, 1008504},
  {0xA484, 1008586},
  {0xA485, 1008618},
  {0xA486, 1008602},
  {0xA487, 1008731},
  {0xA488, 1008747},
  {0xA489, 1008667},
  {0xA48A, 1008682},
  {0xA48B, 1008714},
  {0xA48C, 1008698},
  {0xA490, 990775},
  {0xA491, 990594},
  {0xA492, 990579},
  {0xA493, 990683},
  {0xA494, 990305},
  {0xA495, 990853},
  {0xA496, 990394},
  {0xA497, 990379},
  {0xA498, 990638},
  {0xA499, 990456},
  {0xA49A, 990623},
  {0xA49B, 990214},
  {0xA49C, 990652},
  {0xA49D, 990956},
  {0xA49E, 990745},
  {0xA49F, 990488},
  {0xA4A0, 990868},
  {0xA4A1, 990365},
  {0xA4A2, 990999},
  {0xA4A3, 990320},
  {0xA4A4, 990335},
  {0xA4A5, 990230},
  {0xA4A6, 990410},
  {0xA4A7, 990699},
  {0xA4A8, 990883},
  {0xA4A9, 990731},
  {0xA4AA, 990520},
  {0xA4AB, 990984},
  {0xA4AC, 990760},
  {0xA4AD, 990441},
  {0xA4AE, 990941},
  {0xA4AF, 990912},
  {0xA4B0, 990838},
  {0xA4B1, 990897},
  {0xA4B2, 990970},
  {0xA4B3, 990551},
  {0xA4B4, 990715},
  {0xA4B5, 990536},
  {0xA4B6, 990426},
  {0xA4B7, 990504},
  {0xA4B8, 990927},
  {0xA4B9, 990351},
  {0xA4BA, 990822},
  {0xA4BB, 990608},
  {0xA4BC, 990291},
  {0xA4BD, 990275},
  {0xA4BE, 990260},
  {0xA4BF, 990472},
  {0xA4C0, 990790},
  {0xA4C1, 991014},
  {0xA4C2, 990806},
  {0xA4C3, 990245},
  {0xA4C4, 991029},
  {0xA4C5, 990667},
  {0xA4C6, 990565},
  {0xA4D0, 650838},
  {0xA4D1, 651171},
  {0xA4D2, 651186},
  {0xA4D3, 650884},
  {0xA4D4, 651233},
  {0xA4D5, 651248},
  {0xA4D6, 650959},
  {0xA4D7, 651050},
  {0xA4D8, 651065},
  {0xA4D9, 651035},
  {0xA4DA, 650853},
  {0xA4DB, 650868},
  {0xA4DC, 650899},
  {0xA4DD, 651409},
  {0xA4DE, 651425},
  {0xA4DF, 651096},
  {0xA4E0, 651111},
  {0xA4E1, 651081},
  {0xA4E2, 651202},
  {0xA4E3, 651562},
  {0xA4E4, 651547},
  {0xA4E5, 651126},
  {0xA4E6, 650990},
  {0xA4E7, 651501},
  {0xA4E8, 651005},
  {0xA4E9, 650944},
  {0xA4EA, 651486},
  {0xA4EB, 651217},
  {0xA4EC, 651516},
  {0xA4ED, 650974},
  {0xA4EE, 650809},
  {0xA4EF, 650823},
  {0xA4F0, 650915},
  {0xA4F1, 650929},
  {0xA4F2, 651021},
  {0xA4F3, 651142},
  {0xA4F4, 651442},
  {0xA4F5, 651456},
  {0xA4F6, 651471},
  {0xA4F7, 651156},
  {0xA4F8, 651362},
  {0xA4F9, 651386},
  {0xA4FA, 651288},
  {0xA4FB, 651264},
  {0xA4FC, 651338},
  {0xA4FD, 651313},
  {0xA4FE, 651578},
  {0xA4FF, 651601},
  {0xA500, 963403},
  {0xA501, 963419},
  {0xA502, 963903},
  {0xA503, 966748},
  {0xA504, 966765},
  {0xA505, 965888},
  {0xA506, 962759},
  {0xA507, 962708},
  {0xA508, 964674},
  {0xA509, 964365},
  {0xA50A, 964830},
  {0xA50B, 963616},
  {0xA50C, 963484},
  {0xA50D, 966617},
  {0xA50E, 966351},
  {0xA50F, 966402},
  {0xA510, 963108},
  {0xA511, 963162},
  {0xA512, 964518},
  {0xA513, 966002},
  {0xA514, 963057},
  {0xA515, 965037},
  {0xA516, 966116},
  {0xA517, 966167},
  {0xA518, 967080},
  {0xA519, 967131},
  {0xA51A, 962943},
  {0xA51B, 964116},
  {0xA51C, 965535},
  {0xA51D, 966966},
  {0xA51E, 964247},
  {0xA51F, 965356},
  {0xA520, 963755},
  {0xA521, 964777},
  {0xA522, 965251},
  {0xA523, 965705},
  {0xA524, 964053},
  {0xA525, 964068},
  {0xA526, 963937},
  {0xA527, 963953},
  {0xA528, 966800},
  {0xA529, 966816},
  {0xA52A, 965905},
  {0xA52B, 962777},
  {0xA52C, 962846},
  {0xA52D, 964692},
  {0xA52E, 964401},
  {0xA52F, 964849},
  {0xA530, 963652},
  {0xA531, 963501},
  {0xA532, 966634},
  {0xA533, 966489},
  {0xA534, 966420},
  {0xA535, 963254},
  {0xA536, 963181},
  {0xA537, 964559},
  {0xA538, 966019},
  {0xA539, 963323},
  {0xA53A, 965055},
  {0xA53B, 966254},
  {0xA53C, 966185},
  {0xA53D, 967218},
  {0xA53E, 967149},
  {0xA53F, 962960},
  {0xA540, 964133},
  {0xA541, 965553},
  {0xA542, 966983},
  {0xA543, 964264},
  {0xA544, 965394},
  {0xA545, 963789},
  {0xA546, 964922},
  {0xA547, 965485},
  {0xA548, 965723},
  {0xA549, 962645},
  {0xA54A, 962660},
  {0xA54B, 965284},
  {0xA54C, 963854},
  {0xA54D, 963870},
  {0xA54E, 966699},
  {0xA54F, 966715},
  {0xA550, 965856},
  {0xA551, 962725},
  {0xA552, 962676},
  {0xA553, 964640},
  {0xA554, 964313},
  {0xA555, 964330},
  {0xA556, 964794},
  {0xA557, 963582},
  {0xA558, 963452},
  {0xA559, 966585},
  {0xA55A, 966319},
  {0xA55B, 966368},
  {0xA55C, 963074},
  {0xA55D, 963126},
  {0xA55E, 964486},
  {0xA55F, 965970},
  {0xA560, 963025},
  {0xA561, 965003},
  {0xA562, 966084},
  {0xA563, 966133},
  {0xA564, 967048},
  {0xA565, 967097},
  {0xA566, 962911},
  {0xA567, 964084},
  {0xA568, 965501},
  {0xA569, 966934},
  {0xA56A, 964198},
  {0xA56B, 964214},
  {0xA56C, 965320},
  {0xA56D, 963566},
  {0xA56E, 964624},
  {0xA56F, 964987},
  {0xA570, 965671},
  {0xA571, 965823},
  {0xA572, 965839},
  {0xA573, 964003},
  {0xA574, 966866},
  {0xA575, 966883},
  {0xA576, 965937},
  {0xA577, 962811},
  {0xA578, 962878},
  {0xA579, 964726},
  {0xA57A, 964435},
  {0xA57B, 964885},
  {0xA57C, 963704},
  {0xA57D, 963533},
  {0xA57E, 966666},
  {0xA57F, 966521},
  {0xA580, 966454},
  {0xA581, 963288},
  {0xA582, 963217},
  {0xA583, 964591},
  {0xA584, 966051},
  {0xA585, 963355},
  {0xA586, 965200},
  {0xA587, 966286},
  {0xA588, 966219},
  {0xA589, 967250},
  {0xA58A, 967183},
  {0xA58B, 962992},
  {0xA58C, 964165},
  {0xA58D, 965587},
  {0xA58E, 967015},
  {0xA58F, 964296},
  {0xA590, 965430},
  {0xA591, 963821},
  {0xA592, 964954},
  {0xA593, 965638},
  {0xA594, 965757},
  {0xA595, 966554},
  {0xA596, 966569},
  {0xA597, 964020},
  {0xA598, 964036},
  {0xA599, 966901},
  {0xA59A, 966917},
  {0xA59B, 965954},
  {0xA59C, 962829},
  {0xA59D, 962895},
  {0xA59E, 964744},
  {0xA59F, 964453},
  {0xA5A0, 964904},
  {0xA5A1, 963722},
  {0xA5A2, 963550},
  {0xA5A3, 966683},
  {0xA5A4, 966538},
  {0xA5A5, 966472},
  {0xA5A6, 963306},
  {0xA5A7, 963236},
  {0xA5A8, 964608},
  {0xA5A9, 966068},
  {0xA5AA, 963372},
  {0xA5AB, 965218},
  {0xA5AC, 966303},
  {0xA5AD, 966237},
  {0xA5AE, 967267},
  {0xA5AF, 967201},
  {0xA5B0, 963009},
  {0xA5B1, 964182},
  {0xA5B2, 965605},
  {0xA5B3, 967032},
  {0xA5B4, 964470},
  {0xA5B5, 965449},
  {0xA5B6, 963838},
  {0xA5B7, 964971},
  {0xA5B8, 965655},
  {0xA5B9, 965775},
  {0xA5BA, 965792},
  {0xA5BB, 965807},
  {0xA5BC, 965467},
  {0xA5BD, 963970},
  {0xA5BE, 963986},
  {0xA5BF, 966833},
  {0xA5C0, 966849},
  {0xA5C1, 965921},
  {0xA5C2, 962794},
  {0xA5C3, 962862},
  {0xA5C4, 964709},
  {0xA5C5, 964418},
  {0xA5C6, 964867},
  {0xA5C7, 963669},
  {0xA5C8, 963686},
  {0xA5C9, 963517},
  {0xA5CA, 966650},
  {0xA5CB, 966505},
  {0xA5CC, 966437},
  {0xA5CD, 963271},
  {0xA5CE, 963199},
  {0xA5CF, 964575},
  {0xA5D0, 966035},
  {0xA5D1, 963339},
  {0xA5D2, 965072},
  {0xA5D3, 966270},
  {0xA5D4, 966202},
  {0xA5D5, 967234},
  {0xA5D6, 967166},
  {0xA5D7, 962976},
  {0xA5D8, 964149},
  {0xA5D9, 965570},
  {0xA5DA, 966999},
  {0xA5DB, 964280},
  {0xA5DC, 965412},
  {0xA5DD, 963805},
  {0xA5DE, 964938},
  {0xA5DF, 965622},
  {0xA5E0, 965740},
  {0xA5E1, 963388},
  {0xA5E2, 963436},
  {0xA5E3, 965302},
  {0xA5E4, 963887},
  {0xA5E5, 963920},
  {0xA5E6, 966732},
  {0xA5E7, 966783},
  {0xA5E8, 965872},
  {0xA5E9, 962742},
  {0xA5EA, 962692},
  {0xA5EB, 964657},
  {0xA5EC, 964348},
  {0xA5ED, 964383},
  {0xA5EE, 964812},
  {0xA5EF, 963599},
  {0xA5F0, 963634},
  {0xA5F1, 963468},
  {0xA5F2, 966601},
  {0xA5F3, 966335},
  {0xA5F4, 966385},
  {0xA5F5, 963091},
  {0xA5F6, 963144},
  {0xA5F7, 964502},
  {0xA5F8, 965986},
  {0xA5F9, 963041},
  {0xA5FA, 965020},
  {0xA5FB, 966100},
  {0xA5FC, 966150},
  {0xA5FD, 967064},
  {0xA5FE, 967114},
  {0xA5FF, 962927},
  {0xA600, 964100},
  {0xA601, 965518},
  {0xA602, 966950},
  {0xA603, 964231},
  {0xA604, 965338},
  {0xA605, 965375},
  {0xA606, 963739},
  {0xA607, 963772},
  {0xA608, 964761},
  {0xA609, 965235},
  {0xA60A, 965688},
  {0xA60B, 965268},
  {0xA60C, 964535},
  {0xA60D, 962453},
  {0xA60E, 962613},
  {0xA60F, 962627},
  {0xA610, 965111},
  {0xA611, 965133},
  {0xA612, 965177},
  {0xA613, 967363},
  {0xA614, 967396},
  {0xA615, 967459},
  {0xA616, 967429},
  {0xA617, 967283},
  {0xA618, 967348},
  {0xA619, 967444},
  {0xA61A, 967299},
  {0xA61B, 967331},
  {0xA61C, 967413},
  {0xA61D, 967475},
  {0xA61E, 967315},
  {0xA61F, 967380},
  {0xA620, 962598},
  {0xA621, 962524},
  {0xA622, 962584},
  {0xA623, 962568},
  {0xA624, 962494},
  {0xA625, 962479},
  {0xA626, 962554},
  {0xA627, 962538},
  {0xA628, 962463},
  {0xA629, 962509},
  {0xA62A, 965155},
  {0xA62B, 965089},
  {0xA640, 261370},
  {0xA641, 267869},
  {0xA642, 256432},
  {0xA643, 262886},
  {0xA644, 259598},
  {0xA645, 265969},
  {0xA646, 258067},
  {0xA647, 264427},
  {0xA648, 256270},
  {0xA649, 262734},
  {0xA64A, 259164},
  {0xA64B, 265499},
  {0xA64C, 255833},
  {0xA64D, 262321},
  {0xA64E, 259201},
  {0xA64F, 265565},
  {0xA650, 261091},
  {0xA651, 267606},
  {0xA652, 258302},
  {0xA653, 264650},
  {0xA654, 259672},
  {0xA655, 266039},
  {0xA656, 258096},
  {0xA657, 264454},
  {0xA658, 256110},
  {0xA659, 262584},
  {0xA65A, 255797},
  {0xA65B, 262287},
  {0xA65C, 258172},
  {0xA65D, 264526},
  {0xA65E, 261205},
  {0xA65F, 267714},
  {0xA660, 259635},
  {0xA661, 266004},
  {0xA662, 260187},
  {0xA663, 266557},
  {0xA664, 260219},
  {0xA665, 266587},
  {0xA666, 260251},
  {0xA667, 266617},
  {0xA668, 259128},
  {0xA669, 265465},
  {0xA66A, 255761},
  {0xA66B, 262253},
  {0xA66C, 256300},
  {0xA66D, 262762},
  {0xA66E, 261696},
  {0xA66F, 199257},
  {0xA670, 199082},
  {0xA671, 197447},
  {0xA672, 199119},
  {0xA673, 891854},
  {0xA674, 198623},
  {0xA675, 198070},
  {0xA676, 198752},
  {0xA677, 198595},
  {0xA678, 198034},
  {0xA679, 198721},
  {0xA67A, 198500},
  {0xA67B, 198377},
  {0xA67C, 197530},
  {0xA67D, 198936},
  {0xA67E, 261680},
  {0xA67F, 261784},
  {0xA680, 256376},
  {0xA681, 262834},
  {0xA682, 256491},
  {0xA683, 262941},
  {0xA684, 261554},
  {0xA685, 268043},
  {0xA686, 255918},
  {0xA687, 262402},
  {0xA688, 256520},
  {0xA689, 262968},
  {0xA68A, 260497},
  {0xA68B, 266947},
  {0xA68C, 260684},
  {0xA68D, 267160},
  {0xA68E, 260655},
  {0xA68F, 267133},
  {0xA690, 260626},
  {0xA691, 267106},
  {0xA692, 260399},
  {0xA693, 266855},
  {0xA694, 257767},
  {0xA695, 264145},
  {0xA696, 260158},
  {0xA697, 266530},
  {0xA698, 256343},
  {0xA699, 262803},
  {0xA69A, 256152},
  {0xA69B, 262624},
  {0xA69C, 728401},
  {0xA69D, 729760},
  {0xA69E, 197738},
  {0xA69F, 198207},
  {0xA6A0, 87455},
  {0xA6A1, 87571},
  {0xA6A2, 104260},
  {0xA6A3, 87727},
  {0xA6A4, 87470},
  {0xA6A5, 103932},
  {0xA6A6, 104193},
  {0xA6A7, 88263},
  {0xA6A8, 88246},
  {0xA6A9, 87556},
  {0xA6AA, 87761},
  {0xA6AB, 88278},
  {0xA6AC, 104002},
  {0xA6AD, 103984},
  {0xA6AE, 87777},
  {0xA6AF, 87912},
  {0xA6B0, 104176},
  {0xA6B1, 88012},
  {0xA6B2, 88102},
  {0xA6B3, 87845},
  {0xA6B4, 104159},
  {0xA6B5, 87980},
  {0xA6B6, 104091},
  {0xA6B7, 104143},
  {0xA6B8, 104072},
  {0xA6B9, 104054},
  {0xA6BA, 87743},
  {0xA6BB, 87604},
  {0xA6BC, 88228},
  {0xA6BD, 88212},
  {0xA6BE, 88139},
  {0xA6BF, 104324},
  {0xA6C0, 104126},
  {0xA6C1, 104341},
  {0xA6C2, 104308},
  {0xA6C3, 88158},
  {0xA6C4, 87621},
  {0xA6C5, 88311},
  {0xA6C6, 88121},
  {0xA6C7, 88176},
  {0xA6C8, 103898},
  {0xA6C9, 104291},
  {0xA6CA, 88294},
  {0xA6CB, 87506},
  {0xA6CC, 104019},
  {0xA6CD, 87829},
  {0xA6CE, 87948},
  {0xA6CF, 88086},
  {0xA6D0, 103966},
  {0xA6D1, 103915},
  {0xA6D2, 87587},
  {0xA6D3, 88065},
  {0xA6D4, 88047},
  {0xA6D5, 88030},
  {0xA6D6, 104109},
  {0xA6D7, 103880},
  {0xA6D8, 87540},
  {0xA6D9, 87523},
  {0xA6DA, 104275},
  {0xA6DB, 87996},
  {0xA6DC, 87796},
  {0xA6DD, 103864},
  {0xA6DE, 87812},
  {0xA6DF, 87655},
  {0xA6E0, 87894},
  {0xA6E1, 103949},
  {0xA6E2, 87931},
  {0xA6E3, 87860},
  {0xA6E4, 104244},
  {0xA6E5, 87639},
  {0xA6E6, 87964},
  {0xA6E7, 87876},
  {0xA6E8, 104227},
  {0xA6E9, 87710},
  {0xA6EA, 104210},
  {0xA6EB, 88194},
  {0xA6EC, 104035},
  {0xA6ED, 87486},
  {0xA6EE, 87691},
  {0xA6EF, 87671},
  {0xA6F0, 87367},
  {0xA6F1, 87396},
  {0xA6F2, 104358},
  {0xA6F3, 87439},
  {0xA6F4, 87355},
  {0xA6F5, 87427},
  {0xA6F6, 104392},
  {0xA6F7, 104372},
  {0xA700, 728139},
  {0xA701, 727986},
  {0xA702, 728249},
  {0xA703, 728099},
  {0xA704, 728177},
  {0xA705, 728025},
  {0xA706, 728213},
  {0xA707, 728062},
  {0xA708, 730150},
  {0xA709, 730662},
  {0xA70A, 731579},
  {0xA70B, 731004},
  {0xA70C, 730327},
  {0xA70D, 730097},
  {0xA70E, 730615},
  {0xA70F, 731533},
  {0xA710, 730958},
  {0xA711, 730275},
  {0xA712, 730193},
  {0xA713, 730699},
  {0xA714, 731615},
  {0xA715, 731220},
  {0xA716, 730369},
  {0xA717, 729885},
  {0xA718, 729859},
  {0xA719, 729824},
  {0xA71A, 731469},
  {0xA71B, 732059},
  {0xA71C, 731936},
  {0xA71D, 731970},
  {0xA71E, 732010},
  {0xA71F, 731108},
  {0xA720, 736986},
  {0xA721, 737023},
  {0xA722, 586640},
  {0xA723, 607059},
  {0xA724, 586601},
  {0xA725, 607022},
  {0xA726, 587961},
  {0xA727, 609004},
  {0xA728, 595889},
  {0xA729, 621177},
  {0xA72A, 595577},
  {0xA72B, 619805},
  {0xA72C, 584882},
  {0xA72D, 604531},
  {0xA72E, 584913},
  {0xA72F, 604560},
  {0xA730, 600073},
  {0xA731, 600852},
  {0xA732, 583719},
  {0xA733, 602817},
  {0xA734, 583929},
  {0xA735, 603029},
  {0xA736, 583953},
  {0xA737, 603051},
  {0xA738, 583977},
  {0xA739, 603073},
  {0xA73A, 584001},
  {0xA73B, 603095},
  {0xA73C, 584045},
  {0xA73D, 603137},
  {0xA73E, 593869},
  {0xA73F, 616821},
  {0xA740, 589412},
  {0xA741, 610661},
  {0xA742, 589215},
  {0xA743, 610435},
  {0xA744, 589447},
  {0xA745, 610694},
  {0xA746, 584373},
  {0xA747, 603787},
  {0xA748, 589865},
  {0xA749, 611273},
  {0xA74A, 592138},
  {0xA74B, 614381},
  {0xA74C, 592186},
  {0xA74D, 614427},
  {0xA74E, 592802},
  {0xA74F, 615055},
  {0xA750, 593148},
  {0xA751, 615614},
  {0xA752, 593001},
  {0xA753, 615397},
  {0xA754, 593071},
  {0xA755, 615541},
  {0xA756, 593268},
  {0xA757, 615818},
  {0xA758, 593224},
  {0xA759, 615709},
  {0xA75A, 593344},
  {0xA75B, 615920},
  {0xA75C, 594016},
  {0xA75D, 617226},
  {0xA75E, 597280},
  {0xA75F, 622819},
  {0xA760, 597585},
  {0xA761, 623182},
  {0xA762, 597455},
  {0xA763, 623060},
  {0xA764, 595390},
  {0xA765, 619598},
  {0xA766, 595429},
  {0xA767, 619635},
  {0xA768, 597429},
  {0xA769, 623036},
  {0xA76A, 586730},
  {0xA76B, 607429},
  {0xA76C, 588888},
  {0xA76D, 610096},
  {0xA76E, 584857},
  {0xA76F, 604508},
  {0xA770, 737297},
  {0xA771, 605685},
  {0xA772, 612082},
  {0xA773, 612511},
  {0xA774, 613325},
  {0xA775, 617203},
  {0xA776, 600821},
  {0xA777, 619913},
  {0xA778, 622657},
  {0xA779, 588676},
  {0xA77A, 609767},
  {0xA77B, 588707},
  {0xA77C, 609796},
  {0xA77D, 588738},
  {0xA77E, 595701},
  {0xA77F, 620262},
  {0xA780, 595769},
  {0xA781, 620326},
  {0xA782, 588769},
  {0xA783, 609854},
  {0xA784, 588800},
  {0xA785, 609883},
  {0xA786, 588831},
  {0xA787, 609912},
  {0xA788, 730920},
  {0xA789, 728322},
  {0xA78A, 732403},
  {0xA78B, 594687},
  {0xA78C, 618048},
  {0xA78D, 595671},
  {0xA78E, 611631},
  {0xA78F, 599771},
  {0xA790, 590641},
  {0xA791, 612810},
  {0xA792, 584460},
  {0xA793, 603868},
  {0xA794, 604143},
  {0xA795, 608906},
  {0xA796, 584168},
  {0xA797, 603252},
  {0xA798, 586968},
  {0xA799, 607879},
  {0xA79A, 597489},
  {0xA79B, 623092},
  {0xA79C, 597521},
  {0xA79D, 623122},
  {0xA79E, 597553},
  {0xA79F, 623152},
  {0xA7A0, 587309},
  {0xA7A1, 608275},
  {0xA7A2, 589369},
  {0xA7A3, 610581},
  {0xA7A4, 590909},
  {0xA7A5, 613150},
  {0xA7A6, 593727},
  {0xA7A7, 616582},
  {0xA7A8, 594556},
  {0xA7A9, 617855},
  {0xA7AA, 587865},
  {0xA7AB, 593979},
  {0xA7AC, 594744},
  {0xA7AD, 589591},
  {0xA7AE, 594834},
  {0xA7AF, 600643},
  {0xA7B0, 595739},
  {0xA7B1, 595829},
  {0xA7B2, 588974},
  {0xA7B3, 584794},
  {0xA7B4, 584347},
  {0xA7B5, 603619},
  {0xA7B6, 592775},
  {0xA7B7, 615030},
  {0xA7B8, 597075},
  {0xA7B9, 622468},
  {0xA7BA, 587414},
  {0xA7BB, 608413},
  {0xA7BC, 587445},
  {0xA7BD, 608442},
  {0xA7BE, 587510},
  {0xA7BF, 608503},
  {0xA7C0, 592741},
  {0xA7C1, 614998},
  {0xA7C2, 583896},
  {0xA7C3, 602998},
  {0xA7C4, 584718},
  {0xA7C5, 594523},
  {0xA7C6, 598806},
  {0xA7C7, 585241},
  {0xA7C8, 605066},
  {0xA7C9, 594599},
  {0xA7CA, 617935},
  {0xA7CB, 593838},
  {0xA7CC, 594351},
  {0xA7CD, 617574},
  {0xA7D0, 584819},
  {0xA7D1, 604365},
  {0xA7D3, 605622},
  {0xA7D5, 605654},
  {0xA7D6, 590360},
  {0xA7D7, 612408},
  {0xA7D8, 594803},
  {0xA7D9, 618564},
  {0xA7DA, 590102},
  {0xA7DB, 611714},
  {0xA7DC, 590130},
  {0xA7F2, 727252},
  {0xA7F3, 727330},
  {0xA7F4, 727681},
  {0xA7F5, 593942},
  {0xA7F6, 616922},
  {0xA7F7, 599156},
  {0xA7F8, 727408},
  {0xA7F9, 734811},
  {0xA7FA, 601018},
  {0xA7FB, 599086},
  {0xA7FC, 599121},
  {0xA7FD, 599051},
  {0xA7FE, 599019},
  {0xA7FF, 598985},
  {0xA800, 911937},
  {0xA801, 912241},
  {0xA802, 912854},
  {0xA803, 912661},
  {0xA804, 912149},
  {0xA805, 912426},
  {0xA806, 912881},
  {0xA807, 912334},
  {0xA808, 912310},
  {0xA809, 912195},
  {0xA80A, 912171},
  {0xA80B, 912827},
  {0xA80C, 912030},
  {0xA80D, 912006},
  {0xA80E, 912287},
  {0xA80F, 912263},
  {0xA810, 912637},
  {0xA811, 912612},
  {0xA812, 912078},
  {0xA813, 912053},
  {0xA814, 912589},
  {0xA815, 912565},
  {0xA816, 912126},
  {0xA817, 912102},
  {0xA818, 912403},
  {0xA819, 912472},
  {0xA81A, 912448},
  {0xA81B, 911983},
  {0xA81C, 911959},
  {0xA81D, 912380},
  {0xA81E, 912495},
  {0xA81F, 912357},
  {0xA820, 912518},
  {0xA821, 912542},
  {0xA822, 912218},
  {0xA823, 912907},
  {0xA824, 912959},
  {0xA825, 913012},
  {0xA826, 912933},
  {0xA827, 912985},
  {0xA828, 912683},
  {0xA829, 912710},
  {0xA82A, 912737},
  {0xA82B, 912764},
  {0xA82C, 912791},
  {0xA830, 772503},
  {0xA831, 772473},
  {0xA832, 772571},
  {0xA833, 772536},
  {0xA834, 772441},
  {0xA835, 772607},
  {0xA836, 772700},
  {0xA837, 772645},
  {0xA838, 772725},
  {0xA839, 772674},
  {0xA840, 820201},
  {0xA841, 820220},
  {0xA842, 820106},
  {0xA843, 820297},
  {0xA844, 819924},
  {0xA845, 819971},
  {0xA846, 820182},
  {0xA847, 820337},
  {0xA848, 820515},
  {0xA849, 820534},
  {0xA84A, 819991},
  {0xA84B, 820278},
  {0xA84C, 820375},
  {0xA84D, 820394},
  {0xA84E, 819905},
  {0xA84F, 820259},
  {0xA850, 820554},
  {0xA851, 820574},
  {0xA852, 820030},
  {0xA853, 820710},
  {0xA854, 820786},
  {0xA855, 820767},
  {0xA856, 820491},
  {0xA857, 820748},
  {0xA858, 820433},
  {0xA859, 820240},
  {0xA85A, 820471},
  {0xA85B, 820452},
  {0xA85C, 820145},
  {0xA85D, 819829},
  {0xA85E, 820164},
  {0xA85F, 820636},
  {0xA860, 820050},
  {0xA861, 820357},
  {0xA862, 820414},
  {0xA863, 820729},
  {0xA864, 820087},
  {0xA865, 820125},
  {0xA866, 820068},
  {0xA867, 820906},
  {0xA868, 820935},
  {0xA869, 820595},
  {0xA86A, 820615},
  {0xA86B, 820010},
  {0xA86C, 820317},
  {0xA86D, 819847},
  {0xA86E, 820680},
  {0xA86F, 820654},
  {0xA870, 819876},
  {0xA871, 820877},
  {0xA872, 820964},
  {0xA873, 819943},
  {0xA874, 820851},
  {0xA875, 819803},
  {0xA876, 820832},
  {0xA877, 820806},
  {0xA880, 849551},
  {0xA881, 849627},
  {0xA882, 848456},
  {0xA883, 848476},
  {0xA884, 848818},
  {0xA885, 848838},
  {0xA886, 849354},
  {0xA887, 849374},
  {0xA888, 849473},
  {0xA889, 849501},
  {0xA88A, 849416},
  {0xA88B, 849444},
  {0xA88C, 848713},
  {0xA88D, 848733},
  {0xA88E, 848497},
  {0xA88F, 849096},
  {0xA890, 849116},
  {0xA891, 848518},
  {0xA892, 848902},
  {0xA893, 848923},
  {0xA894, 848754},
  {0xA895, 848775},
  {0xA896, 849030},
  {0xA897, 848582},
  {0xA898, 848603},
  {0xA899, 848859},
  {0xA89A, 848880},
  {0xA89B, 849074},
  {0xA89C, 849309},
  {0xA89D, 849331},
  {0xA89E, 848646},
  {0xA89F, 848668},
  {0xA8A0, 849052},
  {0xA8A1, 849266},
  {0xA8A2, 849287},
  {0xA8A3, 848625},
  {0xA8A4, 848691},
  {0xA8A5, 849009},
  {0xA8A6, 849137},
  {0xA8A7, 849158},
  {0xA8A8, 848539},
  {0xA8A9, 848560},
  {0xA8AA, 848988},
  {0xA8AB, 849530},
  {0xA8AC, 849180},
  {0xA8AD, 848945},
  {0xA8AE, 849395},
  {0xA8AF, 849222},
  {0xA8B0, 849244},
  {0xA8B1, 849201},
  {0xA8B2, 848797},
  {0xA8B3, 848966},
  {0xA8B4, 848163},
  {0xA8B5, 849651},
  {0xA8B6, 849775},
  {0xA8B7, 849799},
  {0xA8B8, 849873},
  {0xA8B9, 849897},
  {0xA8BA, 849987},
  {0xA8BB, 850019},
  {0xA8BC, 849922},
  {0xA8BD, 849954},
  {0xA8BE, 849726},
  {0xA8BF, 849750},
  {0xA8C0, 849676},
  {0xA8C1, 849824},
  {0xA8C2, 849848},
  {0xA8C3, 849701},
  {0xA8C4, 849604},
  {0xA8C5, 849576},
  {0xA8CE, 848195},
  {0xA8CF, 848432},
  {0xA8D0, 848410},
  {0xA8D1, 848301},
  {0xA8D2, 848389},
  {0xA8D3, 848366},
  {0xA8D4, 848257},
  {0xA8D5, 848235},
  {0xA8D6, 848345},
  {0xA8D7, 848322},
  {0xA8D8, 848212},
  {0xA8D9, 848279},
  {0xA8E0, 199594},
  {0xA8E1, 199435},
  {0xA8E2, 199563},
  {0xA8E3, 199530},
  {0xA8E4, 199371},
  {0xA8E5, 199339},
  {0xA8E6, 199499},
  {0xA8E7, 199466},
  {0xA8E8, 199306},
  {0xA8E9, 199403},
  {0xA8EA, 199626},
  {0xA8EB, 199780},
  {0xA8EC, 199656},
  {0xA8ED, 199687},
  {0xA8EE, 199718},
  {0xA8EF, 199749},
  {0xA8F0, 199810},
  {0xA8F1, 199841},
  {0xA8F2, 275448},
  {0xA8F3, 275095},
  {0xA8F4, 275130},
  {0xA8F5, 275063},
  {0xA8F6, 275029},
  {0xA8F7, 274992},
  {0xA8F8, 275358},
  {0xA8F9, 272917},
  {0xA8FA, 272639},
  {0xA8FB, 273021},
  {0xA8FC, 275424},
  {0xA8FD, 273043},
  {0xA8FE, 273166},
  {0xA8FF, 275773},
  {0xA900, 551603},
  {0xA901, 551504},
  {0xA902, 551584},
  {0xA903, 551563},
  {0xA904, 551464},
  {0xA905, 551444},
  {0xA906, 551544},
  {0xA907, 551523},
  {0xA908, 551423},
  {0xA909, 551484},
  {0xA90A, 551774},
  {0xA90B, 551793},
  {0xA90C, 551698},
  {0xA90D, 551870},
  {0xA90E, 552006},
  {0xA90F, 552025},
  {0xA910, 552141},
  {0xA911, 551890},
  {0xA912, 552045},
  {0xA913, 551736},
  {0xA914, 551851},
  {0xA915, 551948},
  {0xA916, 551967},
  {0xA917, 551832},
  {0xA918, 551679},
  {0xA919, 551641},
  {0xA91A, 551987},
  {0xA91B, 552122},
  {0xA91C, 551813},
  {0xA91D, 552103},
  {0xA91E, 552064},
  {0xA91F, 551717},
  {0xA920, 552084},
  {0xA921, 551660},
  {0xA922, 551623},
  {0xA923, 551910},
  {0xA924, 551756},
  {0xA925, 551929},
  {0xA926, 552334},
  {0xA927, 552265},
  {0xA928, 552317},
  {0xA929, 552282},
  {0xA92A, 552300},
  {0xA92B, 552244},
  {0xA92C, 552197},
  {0xA92D, 552217},
  {0xA92E, 552160},
  {0xA92F, 552178},
  {0xA930, 832066},
  {0xA931, 832015},
  {0xA932, 832170},
  {0xA933, 832295},
  {0xA934, 831998},
  {0xA935, 832135},
  {0xA936, 832244},
  {0xA937, 831964},
  {0xA938, 832100},
  {0xA939, 831981},
  {0xA93A, 832049},
  {0xA93B, 832207},
  {0xA93C, 832278},
  {0xA93D, 832261},
  {0xA93E, 832083},
  {0xA93F, 832329},
  {0xA940, 832312},
  {0xA941, 832032},
  {0xA942, 832117},
  {0xA943, 832188},
  {0xA944, 832152},
  {0xA945, 832225},
  {0xA946, 831948},
  {0xA947, 832484},
  {0xA948, 832524},
  {0xA949, 832422},
  {0xA94A, 832380},
  {0xA94B, 832504},
  {0xA94C, 832401},
  {0xA94D, 832463},
  {0xA94E, 832442},
  {0xA94F, 831899},
  {0xA950, 831875},
  {0xA951, 831924},
  {0xA952, 831851},
  {0xA953, 832366},
  {0xA95F, 832346},
  {0xA960, 494751},
  {0xA961, 494780},
  {0xA962, 494838},
  {0xA963, 494692},
  {0xA964, 493557},
  {0xA965, 493697},
  {0xA966, 493798},
  {0xA967, 493764},
  {0xA968, 493586},
  {0xA969, 493642},
  {0xA96A, 493731},
  {0xA96B, 493491},
  {0xA96C, 493670},
  {0xA96D, 493435},
  {0xA96E, 493527},
  {0xA96F, 492489},
  {0xA970, 492573},
  {0xA971, 492546},
  {0xA972, 493253},
  {0xA973, 493009},
  {0xA974, 492981},
  {0xA975, 494523},
  {0xA976, 492119},
  {0xA977, 491946},
  {0xA978, 494301},
  {0xA979, 494555},
  {0xA97A, 492841},
  {0xA97B, 491839},
  {0xA97C, 494612},
  {0xA980, 538124},
  {0xA981, 538059},
  {0xA982, 538104},
  {0xA983, 538148},
  {0xA984, 536692},
  {0xA985, 537014},
  {0xA986, 536996},
  {0xA987, 537037},
  {0xA988, 537679},
  {0xA989, 537418},
  {0xA98A, 537275},
  {0xA98B, 537301},
  {0xA98C, 536915},
  {0xA98D, 536710},
  {0xA98E, 537381},
  {0xA98F, 537104},
  {0xA990, 537148},
  {0xA991, 537123},
  {0xA992, 536933},
  {0xA993, 536952},
  {0xA994, 537255},
  {0xA995, 536773},
  {0xA996, 536792},
  {0xA997, 537056},
  {0xA998, 537355},
  {0xA999, 537075},
  {0xA99A, 537335},
  {0xA99B, 537629},
  {0xA99C, 537649},
  {0xA99D, 536865},
  {0xA99E, 536885},
  {0xA99F, 537230},
  {0xA9A0, 537585},
  {0xA9A1, 537604},
  {0xA9A2, 536817},
  {0xA9A3, 536836},
  {0xA9A4, 537211},
  {0xA9A5, 537399},
  {0xA9A6, 537443},
  {0xA9A7, 536729},
  {0xA9A8, 536748},
  {0xA9A9, 537192},
  {0xA9AA, 537716},
  {0xA9AB, 537468},
  {0xA9AC, 537487},
  {0xA9AD, 537173},
  {0xA9AE, 537697},
  {0xA9AF, 537560},
  {0xA9B0, 537531},
  {0xA9B1, 537512},
  {0xA9B2, 536977},
  {0xA9B3, 538079},
  {0xA9B4, 538340},
  {0xA9B5, 538367},
  {0xA9B6, 538394},
  {0xA9B7, 538419},
  {0xA9B8, 538256},
  {0xA9B9, 538281},
  {0xA9BA, 538313},
  {0xA9BB, 538199},
  {0xA9BC, 538230},
  {0xA9BD, 536406},
  {0xA9BE, 536436},
  {0xA9BF, 536376},
  {0xA9C0, 537996},
  {0xA9C1, 536668},
  {0xA9C2, 538034},
  {0xA9C3, 537778},
  {0xA9C4, 537884},
  {0xA9C5, 537843},
  {0xA9C6, 537976},
  {0xA9C7, 537904},
  {0xA9C8, 537822},
  {0xA9C9, 537863},
  {0xA9CA, 537735},
  {0xA9CB, 537754},
  {0xA9CC, 537926},
  {0xA9CD, 538170},
  {0xA9CF, 538013},
  {0xA9D0, 536648},
  {0xA9D1, 536549},
  {0xA9D2, 536629},
  {0xA9D3, 536608},
  {0xA9D4, 536509},
  {0xA9D5, 536489},
  {0xA9D6, 536589},
  {0xA9D7, 536568},
  {0xA9D8, 536468},
  {0xA9D9, 536529},
  {0xA9DE, 537948},
  {0xA9DF, 537798},
  {0xA9E0, 754291},
  {0xA9E1, 754198},
  {0xA9E2, 754338},
  {0xA9E3, 754432},
  {0xA9E4, 754151},
  {0xA9E5, 756333},
  {0xA9E6, 755486},
  {0xA9E7, 755025},
  {0xA9E8, 754825},
  {0xA9E9, 754853},
  {0xA9EA, 754881},
  {0xA9EB, 754910},
  {0xA9EC, 754938},
  {0xA9ED, 754737},
  {0xA9EE, 754766},
  {0xA9EF, 754996},
  {0xA9F0, 757249},
  {0xA9F1, 757105},
  {0xA9F2, 757221},
  {0xA9F3, 757191},
  {0xA9F4, 757047},
  {0xA9F5, 757018},
  {0xA9F6, 757163},
  {0xA9F7, 757133},
  {0xA9F8, 756988},
  {0xA9F9, 757076},
  {0xA9FA, 754967},
  {0xA9FB, 754709},
  {0xA9FC, 754796},
  {0xA9FD, 754652},
  {0xA9FE, 754680},
  {0xAA00, 178141},
  {0xAA01, 178580},
  {0xAA02, 178954},
  {0xAA03, 178297},
  {0xAA04, 178155},
  {0xAA05, 178816},
  {0xAA06, 178625},
  {0xAA07, 178640},
  {0xAA08, 178534},
  {0xAA09, 178549},
  {0xAA0A, 178733},
  {0xAA0B, 178717},
  {0xAA0C, 178217},
  {0xAA0D, 178233},
  {0xAA0E, 178594},
  {0xAA0F, 178609},
  {0xAA10, 178783},
  {0xAA11, 178750},
  {0xAA12, 178766},
  {0xAA13, 178923},
  {0xAA14, 178938},
  {0xAA15, 178250},
  {0xAA16, 178281},
  {0xAA17, 178800},
  {0xAA18, 178702},
  {0xAA19, 178265},
  {0xAA1A, 178830},
  {0xAA1B, 178861},
  {0xAA1C, 178845},
  {0xAA1D, 178170},
  {0xAA1E, 178201},
  {0xAA1F, 178686},
  {0xAA20, 178671},
  {0xAA21, 178185},
  {0xAA22, 178983},
  {0xAA23, 178877},
  {0xAA24, 178656},
  {0xAA25, 178968},
  {0xAA26, 178907},
  {0xAA27, 178892},
  {0xAA28, 178565},
  {0xAA29, 179105},
  {0xAA2A, 179181},
  {0xAA2B, 179199},
  {0xAA2C, 179162},
  {0xAA2D, 179255},
  {0xAA2E, 179236},
  {0xAA2F, 179218},
  {0xAA30, 179124},
  {0xAA31, 179143},
  {0xAA32, 179273},
  {0xAA33, 177958},
  {0xAA34, 177912},
  {0xAA35, 177889},
  {0xAA36, 177935},
  {0xAA40, 178352},
  {0xAA41, 178332},
  {0xAA42, 178412},
  {0xAA43, 177860},
  {0xAA44, 178311},
  {0xAA45, 178494},
  {0xAA46, 178392},
  {0xAA47, 178433},
  {0xAA48, 178514},
  {0xAA49, 178453},
  {0xAA4A, 178372},
  {0xAA4B, 178473},
  {0xAA4C, 177832},
  {0xAA4D, 177804},
  {0xAA50, 178125},
  {0xAA51, 178046},
  {0xAA52, 178110},
  {0xAA53, 178093},
  {0xAA54, 178014},
  {0xAA55, 177998},
  {0xAA56, 178078},
  {0xAA57, 178061},
  {0xAA58, 177981},
  {0xAA59, 178030},
  {0xAA5C, 179051},
  {0xAA5D, 178998},
  {0xAA5E, 179021},
  {0xAA5F, 179075},
  {0xAA60, 753341},
  {0xAA61, 753186},
  {0xAA62, 753211},
  {0xAA63, 753417},
  {0xAA64, 753442},
  {0xAA65, 753493},
  {0xAA66, 753569},
  {0xAA67, 753595},
  {0xAA68, 753237},
  {0xAA69, 753263},
  {0xAA6A, 753290},
  {0xAA6B, 753468},
  {0xAA6C, 753544},
  {0xAA6D, 753366},
  {0xAA6E, 753391},
  {0xAA6F, 753316},
  {0xAA70, 755441},
  {0xAA71, 753622},
  {0xAA72, 753647},
  {0xAA73, 753519},
  {0xAA74, 755386},
  {0xAA75, 755414},
  {0xAA76, 755359},
  {0xAA77, 756776},
  {0xAA78, 756809},
  {0xAA79, 756834},
  {0xAA7A, 752677},
  {0xAA7B, 756144},
  {0xAA7C, 756455},
  {0xAA7D, 756485},
  {0xAA7E, 754551},
  {0xAA7F, 754583},
  {0xAA80, 923614},
  {0xAA81, 923040},
  {0xAA82, 923590},
  {0xAA83, 923015},
  {0xAA84, 923565},
  {0xAA85, 922989},
  {0xAA86, 923519},
  {0xAA87, 922941},
  {0xAA88, 923683},
  {0xAA89, 923112},
  {0xAA8A, 923450},
  {0xAA8B, 922869},
  {0xAA8C, 923426},
  {0xAA8D, 922844},
  {0xAA8E, 923846},
  {0xAA8F, 923282},
  {0xAA90, 923730},
  {0xAA91, 923161},
  {0xAA92, 923473},
  {0xAA93, 922893},
  {0xAA94, 923893},
  {0xAA95, 923331},
  {0xAA96, 923869},
  {0xAA97, 923306},
  {0xAA98, 923707},
  {0xAA99, 923137},
  {0xAA9A, 923403},
  {0xAA9B, 922820},
  {0xAA9C, 923800},
  {0xAA9D, 923234},
  {0xAA9E, 923776},
  {0xAA9F, 923209},
  {0xAAA0, 923496},
  {0xAAA1, 922917},
  {0xAAA2, 923660},
  {0xAAA3, 923088},
  {0xAAA4, 923939},
  {0xAAA5, 923379},
  {0xAAA6, 923823},
  {0xAAA7, 923258},
  {0xAAA8, 923637},
  {0xAAA9, 923064},
  {0xAAAA, 923916},
  {0xAAAB, 923355},
  {0xAAAC, 923542},
  {0xAAAD, 922965},
  {0xAAAE, 923754},
  {0xAAAF, 923186},
  {0xAAB0, 923962},
  {0xAAB1, 924197},
  {0xAAB2, 924305},
  {0xAAB3, 924392},
  {0xAAB4, 924357},
  {0xAAB5, 924288},
  {0xAAB6, 924340},
  {0xAAB7, 923980},
  {0xAAB8, 924322},
  {0xAAB9, 924410},
  {0xAABA, 924374},
  {0xAABB, 924251},
  {0xAABC, 924270},
  {0xAABD, 924233},
  {0xAABE, 924215},
  {0xAABF, 924107},
  {0xAAC0, 924128},
  {0xAAC1, 924175},
  {0xAAC2, 924152},
  {0xAADB, 924045},
  {0xAADC, 924065},
  {0xAADD, 924087},
  {0xAADE, 923998},
  {0xAADF, 924021},
  {0xAAE0, 709499},
  {0xAAE1, 710042},
  {0xAAE2, 709352},
  {0xAAE3, 710018},
  {0xAAE4, 710318},
  {0xAAE5, 710342},
  {0xAAE6, 709401},
  {0xAAE7, 709425},
  {0xAAE8, 709994},
  {0xAAE9, 710190},
  {0xAAEA, 710214},
  {0xAAEB, 710663},
  {0xAAEC, 710837},
  {0xAAED, 710519},
  {0xAAEE, 710604},
  {0xAAEF, 710547},
  {0xAAF0, 708994},
  {0xAAF1, 708927},
  {0xAAF2, 708953},
  {0xAAF3, 710461},
  {0xAAF4, 710926},
  {0xAAF5, 710864},
  {0xAAF6, 710499},
  {0xAB01, 439742},
  {0xAB02, 439696},
  {0xAB03, 439625},
  {0xAB04, 439672},
  {0xAB05, 439649},
  {0xAB06, 439719},
  {0xAB09, 432093},
  {0xAB0A, 432047},
  {0xAB0B, 431976},
  {0xAB0C, 432023},
  {0xAB0D, 432000},
  {0xAB0E, 432070},
  {0xAB11, 432490},
  {0xAB12, 432446},
  {0xAB13, 432378},
  {0xAB14, 432423},
  {0xAB15, 432401},
  {0xAB16, 432468},
  {0xAB20, 431186},
  {0xAB21, 431332},
  {0xAB22, 431284},
  {0xAB23, 431210},
  {0xAB24, 431259},
  {0xAB25, 431235},
  {0xAB26, 431308},
  {0xAB28, 430586},
  {0xAB29, 430720},
  {0xAB2A, 430676},
  {0xAB2B, 430608},
  {0xAB2C, 430653},
  {0xAB2D, 430631},
  {0xAB2E, 430698},
  {0xAB30, 603499},
  {0xAB31, 601499},
  {0xAB32, 603643},
  {0xAB33, 603531},
  {0xAB34, 606549},
  {0xAB35, 611778},
  {0xAB36, 618209},
  {0xAB37, 611311},
  {0xAB38, 611192},
  {0xAB39, 611474},
  {0xAB3A, 612188},
  {0xAB3B, 612740},
  {0xAB3C, 607120},
  {0xAB3D, 603676},
  {0xAB3E, 603709},
  {0xAB3F, 615221},
  {0xAB40, 609975},
  {0xAB41, 620587},
  {0xAB42, 620535},
  {0xAB43, 620424},
  {0xAB44, 620459},
  {0xAB45, 618631},
  {0xAB46, 600701},
  {0xAB47, 616726},
  {0xAB48, 605548},
  {0xAB49, 616047},
  {0xAB4A, 605576},
  {0xAB4B, 618255},
  {0xAB4C, 618283},
  {0xAB4D, 603587},
  {0xAB4E, 622426},
  {0xAB4F, 621245},
  {0xAB50, 622635},
  {0xAB51, 621026},
  {0xAB52, 622199},
  {0xAB53, 604256},
  {0xAB54, 604322},
  {0xAB55, 604279},
  {0xAB56, 623745},
  {0xAB57, 623595},
  {0xAB58, 623635},
  {0xAB59, 623694},
  {0xAB5A, 624224},
  {0xAB5B, 726924},
  {0xAB5C, 734236},
  {0xAB5D, 734511},
  {0xAB5E, 734556},
  {0xAB5F, 736645},
  {0xAB60, 618019},
  {0xAB61, 610066},
  {0xAB62, 615259},
  {0xAB63, 622679},
  {0xAB64, 609941},
  {0xAB65, 471143},
  {0xAB66, 605800},
  {0xAB67, 619863},
  {0xAB68, 620830},
  {0xAB69, 736490},
  {0xAB6A, 730861},
  {0xAB6B, 732354},
  {0xAB70, 181135},
  {0xAB71, 181335},
  {0xAB72, 181685},
  {0xAB73, 182210},
  {0xAB74, 182951},
  {0xAB75, 182975},
  {0xAB76, 181359},
  {0xAB77, 181709},
  {0xAB78, 181384},
  {0xAB79, 181409},
  {0xAB7A, 181434},
  {0xAB7B, 181459},
  {0xAB7C, 181484},
  {0xAB7D, 181509},
  {0xAB7E, 181534},
  {0xAB7F, 181559},
  {0xAB80, 181610},
  {0xAB81, 181635},
  {0xAB82, 181660},
  {0xAB83, 181734},
  {0xAB84, 181759},
  {0xAB85, 181784},
  {0xAB86, 181809},
  {0xAB87, 181834},
  {0xAB88, 181859},
  {0xAB89, 181884},
  {0xAB8A, 181909},
  {0xAB8B, 181934},
  {0xAB8C, 181959},
  {0xAB8D, 181984},
  {0xAB8E, 182034},
  {0xAB8F, 181584},
  {0xAB90, 182059},
  {0xAB91, 182085},
  {0xAB92, 182110},
  {0xAB93, 182135},
  {0xAB94, 182160},
  {0xAB95, 182185},
  {0xAB96, 182234},
  {0xAB97, 182260},
  {0xAB98, 182286},
  {0xAB99, 182312},
  {0xAB9A, 182338},
  {0xAB9B, 182364},
  {0xAB9C, 182414},
  {0xAB9D, 182390},
  {0xAB9E, 182439},
  {0xAB9F, 182464},
  {0xABA0, 182489},
  {0xABA1, 182514},
  {0xABA2, 182539},
  {0xABA3, 181159},
  {0xABA4, 182564},
  {0xABA5, 181184},
  {0xABA6, 182589},
  {0xABA7, 181209},
  {0xABA8, 182614},
  {0xABA9, 181260},
  {0xABAA, 181285},
  {0xABAB, 181310},
  {0xABAC, 181234},
  {0xABAD, 182639},
  {0xABAE, 182665},
  {0xABAF, 182691},
  {0xABB0, 182717},
  {0xABB1, 182743},
  {0xABB2, 182769},
  {0xABB3, 182795},
  {0xABB4, 182821},
  {0xABB5, 182847},
  {0xABB6, 182873},
  {0xABB7, 182899},
  {0xABB8, 182925},
  {0xABB9, 182999},
  {0xABBA, 183024},
  {0xABBB, 183049},
  {0xABBC, 183074},
  {0xABBD, 183099},
  {0xABBE, 183124},
  {0xABBF, 183149},
  {0xABC0, 709719},
  {0xABC1, 710166},
  {0xABC2, 709774},
  {0xABC3, 709829},
  {0xABC4, 710064},
  {0xABC5, 709884},
  {0xABC6, 709376},
  {0xABC7, 710263},
  {0xABC8, 709694},
  {0xABC9, 709937},
  {0xABCA, 710238},
  {0xABCB, 710390},
  {0xABCC, 710414},
  {0xABCD, 709570},
  {0xABCE, 710367},
  {0xABCF, 709594},
  {0xABD0, 710117},
  {0xABD1, 709278},
  {0xABD2, 709546},
  {0xABD3, 709645},
  {0xABD4, 710142},
  {0xABD5, 709304},
  {0xABD6, 709670},
  {0xABD7, 709475},
  {0xABD8, 709521},
  {0xABD9, 709450},
  {0xABDA, 709327},
  {0xABDB, 709743},
  {0xABDC, 709798},
  {0xABDD, 709853},
  {0xABDE, 710087},
  {0xABDF, 709907},
  {0xABE0, 710287},
  {0xABE1, 709962},
  {0xABE2, 709616},
  {0xABE3, 710748},
  {0xABE4, 710690},
  {0xABE5, 710575},
  {0xABE6, 710896},
  {0xABE7, 710777},
  {0xABE8, 710808},
  {0xABE9, 710631},
  {0xABEA, 710719},
  {0xABEB, 709016},
  {0xABEC, 710439},
  {0xABED, 708971},
  {0xABF0, 709254},
  {0xABF1, 709135},
  {0xABF2, 709231},
  {0xABF3, 709206},
  {0xABF4, 709087},
  {0xABF5, 709063},
  {0xABF6, 709183},
  {0xABF7, 709158},
  {0xABF8, 709038},
  {0xABF9, 709111},
  {0xD7B0, 500179},
  {0xD7B1, 500068},
  {0xD7B2, 500809},
  {0xD7B3, 500831},
  {0xD7B4, 500854},
  {0xD7B5, 500403},
  {0xD7B6, 500337},
  {0xD7B7, 501034},
  {0xD7B8, 501124},
  {0xD7B9, 499496},
  {0xD7BA, 499540},
  {0xD7BB, 499518},
  {0xD7BC, 499586},
  {0xD7BD, 499849},
  {0xD7BE, 499873},
  {0xD7BF, 499918},
  {0xD7C0, 499896},
  {0xD7C1, 499783},
  {0xD7C2, 499941},
  {0xD7C3, 499963},
  {0xD7C4, 499741},
  {0xD7C5, 499244},
  {0xD7C6, 499269},
  {0xD7CB, 496316},
  {0xD7CC, 496166},
  {0xD7CD, 498571},
  {0xD7CE, 498600},
  {0xD7CF, 498777},
  {0xD7D0, 498837},
  {0xD7D1, 498866},
  {0xD7D2, 498716},
  {0xD7D3, 498684},
  {0xD7D4, 498902},
  {0xD7D5, 497624},
  {0xD7D6, 497129},
  {0xD7D7, 498440},
  {0xD7D8, 497229},
  {0xD7D9, 497560},
  {0xD7DA, 497489},
  {0xD7DB, 497865},
  {0xD7DC, 497824},
  {0xD7DD, 495425},
  {0xD7DE, 495896},
  {0xD7DF, 496076},
  {0xD7E0, 498328},
  {0xD7E1, 495985},
  {0xD7E2, 495808},
  {0xD7E3, 496949},
  {0xD7E4, 496849},
  {0xD7E5, 496760},
  {0xD7E6, 498384},
  {0xD7E7, 496914},
  {0xD7E8, 496702},
  {0xD7E9, 496671},
  {0xD7EA, 498070},
  {0xD7EB, 498005},
  {0xD7EC, 498503},
  {0xD7ED, 498537},
  {0xD7EE, 498098},
  {0xD7EF, 497949},
  {0xD7F0, 497919},
  {0xD7F1, 498184},
  {0xD7F2, 497977},
  {0xD7F3, 496498},
  {0xD7F4, 496459},
  {0xD7F5, 499021},
  {0xD7F6, 498989},
  {0xD7F7, 495009},
  {0xD7F8, 495038},
  {0xD7F9, 498243},
  {0xD7FA, 496585},
  {0xD7FB, 496615},
  {0xFB00, 624873},
  {0xFB01, 624947},
  {0xFB02, 624971},
  {0xFB03, 624897},
  {0xFB04, 624922},
  {0xFB05, 625019},
  {0xFB06, 625073},
  {0xFB13, 81535},
  {0xFB14, 81471},
  {0xFB15, 81503},
  {0xFB16, 81599},
  {0xFB17, 81567},
  {0xFB1D, 511845},
  {0xFB1E, 512339},
  {0xFB1F, 512055},
  {0xFB20, 510205},
  {0xFB21, 511602},
  {0xFB22, 511626},
  {0xFB23, 511680},
  {0xFB24, 511702},
  {0xFB25, 511725},
  {0xFB26, 511651},
  {0xFB27, 511750},
  {0xFB28, 511774},
  {0xFB29, 510236},
  {0xFB2A, 511312},
  {0xFB2B, 511345},
  {0xFB2C, 511225},
  {0xFB2D, 511269},
  {0xFB2E, 510144},
  {0xFB2F, 510174},
  {0xFB30, 510114},
  {0xFB31, 510309},
  {0xFB32, 510631},
  {0xFB33, 510387},
  {0xFB34, 510680},
  {0xFB35, 511543},
  {0xFB36, 511894},
  {0xFB38, 511443},
  {0xFB39, 511815},
  {0xFB3A, 510443},
  {0xFB3B, 510744},
  {0xFB3C, 510822},
  {0xFB3E, 510872},
  {0xFB40, 510920},
  {0xFB41, 511142},
  {0xFB43, 510550},
  {0xFB44, 510967},
  {0xFB46, 511493},
  {0xFB47, 511041},
  {0xFB48, 511090},
  {0xFB49, 511194},
  {0xFB4A, 511395},
  {0xFB4B, 511573},
  {0xFB4C, 510339},
  {0xFB4D, 510774},
  {0xFB4E, 510996},
  {0xFB4F, 511926},
  {0xFB50, 29949},
  {0xFB51, 29913},
  {0xFB52, 31634},
  {0xFB53, 31572},
  {0xFB54, 31602},
  {0xFB55, 31667},
  {0xFB56, 40225},
  {0xFB57, 40165},
  {0xFB58, 40194},
  {0xFB59, 40257},
  {0xFB5A, 32403},
  {0xFB5B, 32339},
  {0xFB5C, 32370},
  {0xFB5D, 32437},
  {0xFB5E, 44546},
  {0xFB5F, 44480},
  {0xFB60, 44512},
  {0xFB61, 44581},
  {0xFB62, 43966},
  {0xFB63, 43902},
  {0xFB64, 43933},
  {0xFB65, 44000},
  {0xFB66, 44363},
  {0xFB67, 44301},
  {0xFB68, 44331},
  {0xFB69, 44396},
  {0xFB6A, 45044},
  {0xFB6B, 44984},
  {0xFB6C, 45013},
  {0xFB6D, 45076},
  {0xFB6E, 40442},
  {0xFB6F, 40378},
  {0xFB70, 40409},
  {0xFB71, 40476},
  {0xFB72, 33643},
  {0xFB73, 33581},
  {0xFB74, 33611},
  {0xFB75, 33676},
  {0xFB76, 40007},
  {0xFB77, 39945},
  {0xFB78, 39975},
  {0xFB79, 40040},
  {0xFB7A, 43166},
  {0xFB7B, 43102},
  {0xFB7C, 43133},
  {0xFB7D, 43200},
  {0xFB7E, 43390},
  {0xFB7F, 43322},
  {0xFB80, 43355},
  {0xFB81, 43426},
  {0xFB82, 33288},
  {0xFB83, 33256},
  {0xFB84, 32693},
  {0xFB85, 32662},
  {0xFB86, 33530},
  {0xFB87, 33501},
  {0xFB88, 33372},
  {0xFB89, 33342},
  {0xFB8A, 37232},
  {0xFB8B, 37203},
  {0xFB8C, 41639},
  {0xFB8D, 41609},
  {0xFB8E, 37736},
  {0xFB8F, 37672},
  {0xFB90, 37703},
  {0xFB91, 37770},
  {0xFB92, 34820},
  {0xFB93, 34760},
  {0xFB94, 34789},
  {0xFB95, 34852},
  {0xFB96, 35354},
  {0xFB97, 35292},
  {0xFB98, 35322},
  {0xFB99, 35387},
  {0xFB9A, 39362},
  {0xFB9B, 39298},
  {0xFB9C, 39329},
  {0xFB9D, 39396},
  {0xFB9E, 39540},
  {0xFB9F, 39503},
  {0xFBA0, 41497},
  {0xFBA1, 41433},
  {0xFBA2, 41464},
  {0xFBA3, 41531},
  {0xFBA4, 36759},
  {0xFBA5, 36715},
  {0xFBA6, 36443},
  {0xFBA7, 36373},
  {0xFBA8, 36407},
  {0xFBA9, 36480},
  {0xFBAA, 36235},
  {0xFBAB, 36151},
  {0xFBAC, 36192},
  {0xFBAD, 36279},
  {0xFBAE, 45651},
  {0xFBAF, 45615},
  {0xFBB0, 45923},
  {0xFBB1, 45870},
  {0xFBB2, 77606},
  {0xFBB3, 77630},
  {0xFBB4, 77995},
  {0xFBB5, 78024},
  {0xFBB6, 77833},
  {0xFBB7, 77864},
  {0xFBB8, 77895},
  {0xFBB9, 77945},
  {0xFBBA, 77694},
  {0xFBBB, 77724},
  {0xFBBC, 77654},
  {0xFBBD, 78053},
  {0xFBBE, 78093},
  {0xFBBF, 77754},
  {0xFBC0, 77773},
  {0xFBC1, 77803},
  {0xFBC2, 78133},
  {0xFBD3, 39218},
  {0xFBD4, 39160},
  {0xFBD5, 39188},
  {0xFBD6, 39249},
  {0xFBD7, 44657},
  {0xFBD8, 44630},
  {0xFBD9, 40116},
  {0xFBDA, 40088},
  {0xFBDB, 46635},
  {0xFBDC, 46607},
  {0xFBDD, 44720},
  {0xFBDE, 44935},
  {0xFBDF, 44907},
  {0xFBE0, 38310},
  {0xFBE1, 38274},
  {0xFBE2, 38410},
  {0xFBE3, 38374},
  {0xFBE4, 33779},
  {0xFBE5, 33723},
  {0xFBE6, 33750},
  {0xFBE7, 33809},
  {0xFBE8, 44767},
  {0xFBE9, 44829},
  {0xFBEA, 67619},
  {0xFBEB, 67561},
  {0xFBEC, 67502},
  {0xFBED, 67446},
  {0xFBEE, 69040},
  {0xFBEF, 68983},
  {0xFBF0, 68925},
  {0xFBF1, 68870},
  {0xFBF2, 68754},
  {0xFBF3, 68698},
  {0xFBF4, 69273},
  {0xFBF5, 69217},
  {0xFBF6, 67927},
  {0xFBF7, 67815},
  {0xFBF8, 67870},
  {0xFBF9, 67088},
  {0xFBFA, 66924},
  {0xFBFB, 67005},
  {0xFBFC, 33933},
  {0xFBFD, 33861},
  {0xFBFE, 33896},
  {0xFBFF, 33971},
  {0xFC00, 68281},
  {0xFC01, 68044},
  {0xFC02, 68520},
  {0xFC03, 67746},
  {0xFC04, 69157},
  {0xFC05, 48507},
  {0xFC06, 48289},
  {0xFC07, 48594},
  {0xFC08, 48772},
  {0xFC09, 48195},
  {0xFC0A, 48979},
  {0xFC0B, 65196},
  {0xFC0C, 64873},
  {0xFC0D, 65445},
  {0xFC0E, 65735},
  {0xFC0F, 64779},
  {0xFC10, 66209},
  {0xFC11, 66495},
  {0xFC12, 66626},
  {0xFC13, 66400},
  {0xFC14, 66838},
  {0xFC15, 52282},
  {0xFC16, 52479},
  {0xFC17, 51301},
  {0xFC18, 51438},
  {0xFC19, 54033},
  {0xFC1A, 53945},
  {0xFC1B, 54122},
  {0xFC1C, 61100},
  {0xFC1D, 60832},
  {0xFC1E, 61345},
  {0xFC1F, 61588},
  {0xFC20, 59784},
  {0xFC21, 60062},
  {0xFC22, 49445},
  {0xFC23, 49252},
  {0xFC24, 49532},
  {0xFC25, 49723},
  {0xFC26, 64270},
  {0xFC27, 64356},
  {0xFC28, 70269},
  {0xFC29, 47073},
  {0xFC2A, 47264},
  {0xFC2B, 50767},
  {0xFC2C, 50858},
  {0xFC2D, 50162},
  {0xFC2E, 50076},
  {0xFC2F, 50249},
  {0xFC30, 50440},
  {0xFC31, 49982},
  {0xFC32, 50574},
  {0xFC33, 58798},
  {0xFC34, 58884},
  {0xFC35, 58704},
  {0xFC36, 59171},
  {0xFC37, 52865},
  {0xFC38, 53138},
  {0xFC39, 53052},
  {0xFC3A, 53225},
  {0xFC3B, 53351},
  {0xFC3C, 53519},
  {0xFC3D, 52958},
  {0xFC3E, 53799},
  {0xFC3F, 55174},
  {0xFC40, 54837},
  {0xFC41, 55519},
  {0xFC42, 55751},
  {0xFC43, 54386},
  {0xFC44, 56029},
  {0xFC45, 56454},
  {0xFC46, 56210},
  {0xFC47, 56755},
  {0xFC48, 57045},
  {0xFC49, 56114},
  {0xFC4A, 57141},
  {0xFC4B, 57705},
  {0xFC4C, 57370},
  {0xFC4D, 58115},
  {0xFC4E, 58246},
  {0xFC4F, 57274},
  {0xFC50, 58569},
  {0xFC51, 51769},
  {0xFC52, 51856},
  {0xFC53, 51674},
  {0xFC54, 52061},
  {0xFC55, 69516},
  {0xFC56, 67354},
  {0xFC57, 69653},
  {0xFC58, 69781},
  {0xFC59, 67260},
  {0xFC5A, 70142},
  {0xFC5B, 66293},
  {0xFC5C, 59585},
  {0xFC5D, 47921},
  {0xFC5E, 62204},
  {0xFC5F, 62443},
  {0xFC60, 62255},
  {0xFC61, 62110},
  {0xFC62, 62349},
  {0xFC63, 62494},
  {0xFC64, 68813},
  {0xFC65, 69332},
  {0xFC66, 68402},
  {0xFC67, 68640},
  {0xFC68, 67680},
  {0xFC69, 69100},
  {0xFC6A, 48899},
  {0xFC6B, 49022},
  {0xFC6C, 48688},
  {0xFC6D, 48858},
  {0xFC6E, 48146},
  {0xFC6F, 48939},
  {0xFC70, 66129},
  {0xFC71, 66252},
  {0xFC72, 65651},
  {0xFC73, 66088},
  {0xFC74, 64730},
  {0xFC75, 66169},
  {0xFC76, 66756},
  {0xFC77, 66882},
  {0xFC78, 66540},
  {0xFC79, 66714},
  {0xFC7A, 66350},
  {0xFC7B, 66797},
  {0xFC7C, 49933},
  {0xFC7D, 50534},
  {0xFC7E, 58655},
  {0xFC7F, 59131},
  {0xFC80, 52824},
  {0xFC81, 53269},
  {0xFC82, 53435},
  {0xFC83, 52909},
  {0xFC84, 53759},
  {0xFC85, 55667},
  {0xFC86, 54337},
  {0xFC87, 55989},
  {0xFC88, 56072},
  {0xFC89, 56959},
  {0xFC8A, 58487},
  {0xFC8B, 58613},
  {0xFC8C, 58160},
  {0xFC8D, 58445},
  {0xFC8E, 57224},
  {0xFC8F, 58528},
  {0xFC90, 47859},
  {0xFC91, 70062},
  {0xFC92, 70185},
  {0xFC93, 69697},
  {0xFC94, 70021},
  {0xFC95, 67211},
  {0xFC96, 70102},
  {0xFC97, 68221},
  {0xFC98, 67985},
  {0xFC99, 68342},
  {0xFC9A, 68460},
  {0xFC9B, 68104},
  {0xFC9C, 48464},
  {0xFC9D, 48247},
  {0xFC9E, 48551},
  {0xFC9F, 48729},
  {0xFCA0, 48381},
  {0xFCA1, 65153},
  {0xFCA2, 64831},
  {0xFCA3, 65402},
  {0xFCA4, 65692},
  {0xFCA5, 65070},
  {0xFCA6, 66582},
  {0xFCA7, 52239},
  {0xFCA8, 52435},
  {0xFCA9, 51258},
  {0xFCAA, 51395},
  {0xFCAB, 53989},
  {0xFCAC, 54078},
  {0xFCAD, 61056},
  {0xFCAE, 60789},
  {0xFCAF, 61301},
  {0xFCB0, 61544},
  {0xFCB1, 59742},
  {0xFCB2, 59976},
  {0xFCB3, 60019},
  {0xFCB4, 49402},
  {0xFCB5, 49210},
  {0xFCB6, 49489},
  {0xFCB7, 49680},
  {0xFCB8, 64228},
  {0xFCB9, 70226},
  {0xFCBA, 47030},
  {0xFCBB, 47221},
  {0xFCBC, 50722},
  {0xFCBD, 50813},
  {0xFCBE, 50119},
  {0xFCBF, 50034},
  {0xFCC0, 50206},
  {0xFCC1, 50397},
  {0xFCC2, 58756},
  {0xFCC3, 58841},
  {0xFCC4, 53095},
  {0xFCC5, 53010},
  {0xFCC6, 53182},
  {0xFCC7, 53309},
  {0xFCC8, 53476},
  {0xFCC9, 55131},
  {0xFCCA, 54795},
  {0xFCCB, 55476},
  {0xFCCC, 55708},
  {0xFCCD, 55089},
  {0xFCCE, 56410},
  {0xFCCF, 56167},
  {0xFCD0, 56711},
  {0xFCD1, 57001},
  {0xFCD2, 57661},
  {0xFCD3, 57327},
  {0xFCD4, 58071},
  {0xFCD5, 58202},
  {0xFCD6, 57576},
  {0xFCD7, 51726},
  {0xFCD8, 51813},
  {0xFCD9, 52006},
  {0xFCDA, 69473},
  {0xFCDB, 67312},
  {0xFCDC, 69610},
  {0xFCDD, 69738},
  {0xFCDE, 69390},
  {0xFCDF, 68581},
  {0xFCE0, 68163},
  {0xFCE1, 48816},
  {0xFCE2, 48423},
  {0xFCE3, 65779},
  {0xFCE4, 65112},
  {0xFCE5, 66671},
  {0xFCE6, 66453},
  {0xFCE7, 61633},
  {0xFCE8, 61014},
  {0xFCE9, 63618},
  {0xFCEA, 63033},
  {0xFCEB, 53394},
  {0xFCEC, 53563},
  {0xFCED, 55795},
  {0xFCEE, 58291},
  {0xFCEF, 57619},
  {0xFCF0, 69825},
  {0xFCF1, 69432},
  {0xFCF2, 62303},
  {0xFCF3, 62158},
  {0xFCF4, 62397},
  {0xFCF5, 64176},
  {0xFCF6, 64687},
  {0xFCF7, 46978},
  {0xFCF8, 47561},
  {0xFCF9, 50668},
  {0xFCFA, 51112},
  {0xFCFB, 60736},
  {0xFCFC, 62066},
  {0xFCFD, 62604},
  {0xFCFE, 64007},
  {0xFCFF, 51206},
  {0xFD00, 51631},
  {0xFD01, 52186},
  {0xFD02, 52780},
  {0xFD03, 53892},
  {0xFD04, 54208},
  {0xFD05, 59690},
  {0xFD06, 60333},
  {0xFD07, 49158},
  {0xFD08, 49890},
  {0xFD09, 63164},
  {0xFD0A, 62744},
  {0xFD0B, 63394},
  {0xFD0C, 63572},
  {0xFD0D, 63920},
  {0xFD0E, 61981},
  {0xFD0F, 60250},
  {0xFD10, 49807},
  {0xFD11, 64127},
  {0xFD12, 64647},
  {0xFD13, 46929},
  {0xFD14, 47521},
  {0xFD15, 50617},
  {0xFD16, 51070},
  {0xFD17, 60686},
  {0xFD18, 62025},
  {0xFD19, 62553},
  {0xFD1A, 63965},
  {0xFD1B, 51157},
  {0xFD1C, 51591},
  {0xFD1D, 52136},
  {0xFD1E, 52739},
  {0xFD1F, 53842},
  {0xFD20, 54167},
  {0xFD21, 59641},
  {0xFD22, 60293},
  {0xFD23, 49109},
  {0xFD24, 49850},
  {0xFD25, 63076},
  {0xFD26, 62658},
  {0xFD27, 63306},
  {0xFD28, 63484},
  {0xFD29, 63878},
  {0xFD2A, 61940},
  {0xFD2B, 60210},
  {0xFD2C, 49767},
  {0xFD2D, 63119},
  {0xFD2E, 62700},
  {0xFD2F, 63349},
  {0xFD30, 63527},
  {0xFD31, 60971},
  {0xFD32, 62989},
  {0xFD33, 64313},
  {0xFD34, 61145},
  {0xFD35, 60876},
  {0xFD36, 61390},
  {0xFD37, 63210},
  {0xFD38, 62789},
  {0xFD39, 63440},
  {0xFD3A, 64400},
  {0xFD3B, 70313},
  {0xFD3C, 47986},
  {0xFD3D, 48032},
  {0xFD3E, 802768},
  {0xFD3F, 802792},
  {0xFD40, 59483},
  {0xFD41, 59306},
  {0xFD42, 59339},
  {0xFD43, 59374},
  {0xFD44, 59409},
  {0xFD45, 59446},
  {0xFD46, 60544},
  {0xFD47, 47711},
  {0xFD48, 47789},
  {0xFD49, 47823},
  {0xFD4A, 47744},
  {0xFD4B, 59275},
  {0xFD4C, 60632},
  {0xFD4D, 47640},
  {0xFD4E, 64090},
  {0xFD4F, 59515},
  {0xFD50, 65299},
  {0xFD51, 64916},
  {0xFD52, 64966},
  {0xFD53, 65018},
  {0xFD54, 65548},
  {0xFD55, 65932},
  {0xFD56, 65880},
  {0xFD57, 65985},
  {0xFD58, 52584},
  {0xFD59, 52635},
  {0xFD5A, 51541},
  {0xFD5B, 51482},
  {0xFD5C, 60918},
  {0xFD5D, 61248},
  {0xFD5E, 61188},
  {0xFD5F, 61676},
  {0xFD60, 61727},
  {0xFD61, 61780},
  {0xFD62, 61834},
  {0xFD63, 61886},
  {0xFD64, 59827},
  {0xFD65, 59876},
  {0xFD66, 60106},
  {0xFD67, 62832},
  {0xFD68, 62884},
  {0xFD69, 63254},
  {0xFD6A, 63662},
  {0xFD6B, 63715},
  {0xFD6C, 63770},
  {0xFD6D, 63823},
  {0xFD6E, 49295},
  {0xFD6F, 49576},
  {0xFD70, 49627},
  {0xFD71, 64442},
  {0xFD72, 64492},
  {0xFD73, 64544},
  {0xFD74, 64597},
  {0xFD75, 47117},
  {0xFD76, 47367},
  {0xFD77, 47418},
  {0xFD78, 47308},
  {0xFD79, 50965},
  {0xFD7A, 51018},
  {0xFD7B, 50904},
  {0xFD7C, 50293},
  {0xFD7D, 50344},
  {0xFD7E, 58928},
  {0xFD7F, 59030},
  {0xFD80, 54938},
  {0xFD81, 55040},
  {0xFD82, 54880},
  {0xFD83, 55269},
  {0xFD84, 55218},
  {0xFD85, 55563},
  {0xFD86, 55614},
  {0xFD87, 55837},
  {0xFD88, 55887},
  {0xFD89, 56254},
  {0xFD8A, 56307},
  {0xFD8B, 56360},
  {0xFD8C, 56499},
  {0xFD8D, 56606},
  {0xFD8E, 56800},
  {0xFD8F, 56854},
  {0xFD92, 56552},
  {0xFD93, 51900},
  {0xFD94, 51953},
  {0xFD95, 57473},
  {0xFD96, 57414},
  {0xFD97, 57914},
  {0xFD98, 57966},
  {0xFD99, 57750},
  {0xFD9A, 58394},
  {0xFD9B, 58334},
  {0xFD9C, 69867},
  {0xFD9D, 69918},
  {0xFD9E, 48638},
  {0xFD9F, 65352},
  {0xFDA0, 65240},
  {0xFDA1, 65601},
  {0xFDA2, 65489},
  {0xFDA3, 66038},
  {0xFDA4, 65821},
  {0xFDA5, 52688},
  {0xFDA6, 52326},
  {0xFDA7, 52524},
  {0xFDA8, 61433},
  {0xFDA9, 59927},
  {0xFDAA, 62938},
  {0xFDAB, 49353},
  {0xFDAC, 55426},
  {0xFDAD, 55939},
  {0xFDAE, 67397},
  {0xFDAF, 69560},
  {0xFDB0, 69971},
  {0xFDB1, 57090},
  {0xFDB2, 59081},
  {0xFDB3, 57526},
  {0xFDB4, 58978},
  {0xFDB5, 54988},
  {0xFDB6, 47471},
  {0xFDB7, 53709},
  {0xFDB8, 57861},
  {0xFDB9, 56908},
  {0xFDBA, 55373},
  {0xFDBB, 53605},
  {0xFDBC, 55322},
  {0xFDBD, 57810},
  {0xFDBE, 52385},
  {0xFDBF, 51345},
  {0xFDC0, 56660},
  {0xFDC1, 50484},
  {0xFDC2, 48332},
  {0xFDC3, 53656},
  {0xFDC4, 47168},
  {0xFDC5, 60157},
  {0xFDC6, 61493},
  {0xFDC7, 58020},
  {0xFDCF, 60376},
  {0xFDF0, 60482},
  {0xFDF1, 59214},
  {0xFDF2, 48081},
  {0xFDF3, 47604},
  {0xFDF4, 57185},
  {0xFDF5, 60410},
  {0xFDF6, 59548},
  {0xFDF7, 47674},
  {0xFDF8, 67172},
  {0xFDF9, 60446},
  {0xFDFA, 60588},
  {0xFDFB, 52104},
  {0xFDFC, 833593},
  {0xFDFD, 49063},
  {0xFDFE, 64052},
  {0xFDFF, 48117},
  {0xFE00, 967499},
  {0xFE01, 970040},
  {0xFE02, 971592},
  {0xFE03, 971833},
  {0xFE04, 972074},
  {0xFE05, 972315},
  {0xFE06, 972556},
  {0xFE07, 972797},
  {0xFE08, 973038},
  {0xFE09, 967520},
  {0xFE0A, 967772},
  {0xFE0B, 968024},
  {0xFE0C, 968276},
  {0xFE0D, 968528},
  {0xFE0E, 968780},
  {0xFE0F, 969032},
  {0xFE10, 826706},
  {0xFE11, 826920},
  {0xFE12, 826969},
  {0xFE13, 826669},
  {0xFE14, 828207},
  {0xFE15, 826821},
  {0xFE16, 827607},
  {0xFE17, 827506},
  {0xFE18, 828145},
  {0xFE19, 826869},
  {0xFE20, 206113},
  {0xFE21, 206177},
  {0xFE22, 200670},
  {0xFE23, 200703},
  {0xFE24, 206482},
  {0xFE25, 206542},
  {0xFE26, 197351},
  {0xFE27, 206142},
  {0xFE28, 206207},
  {0xFE29, 208141},
  {0xFE2A, 208197},
  {0xFE2B, 206509},
  {0xFE2C, 206570},
  {0xFE2D, 197379},
  {0xFE2E, 199186},
  {0xFE2F, 199221},
  {0xFE30, 828248},
  {0xFE31, 826743},
  {0xFE32, 826782},
  {0xFE33, 827567},
  {0xFE34, 828294},
  {0xFE35, 827291},
  {0xFE36, 827926},
  {0xFE37, 827184},
  {0xFE38, 827817},
  {0xFE39, 827390},
  {0xFE3A, 828027},
  {0xFE3B, 827072},
  {0xFE3C, 827703},
  {0xFE3D, 827234},
  {0xFE3E, 827868},
  {0xFE3F, 827022},
  {0xFE40, 827652},
  {0xFE41, 827133},
  {0xFE42, 827765},
  {0xFE43, 827449},
  {0xFE44, 828087},
  {0xFE45, 853035},
  {0xFE46, 985756},
  {0xFE47, 827339},
  {0xFE48, 827975},
  {0xFE49, 269313},
  {0xFE4A, 176445},
  {0xFE4B, 982297},
  {0xFE4C, 286392},
  {0xFE4D, 269297},
  {0xFE4E, 176425},
  {0xFE4F, 982283},
  {0xFE50, 892153},
  {0xFE51, 892541},
  {0xFE52, 892482},
  {0xFE54, 893296},
  {0xFE55, 892141},
  {0xFE56, 892742},
  {0xFE57, 892459},
  {0xFE58, 892427},
  {0xFE59, 892590},
  {0xFE5A, 892810},
  {0xFE5B, 892565},
  {0xFE5C, 892784},
  {0xFE5D, 892613},
  {0xFE5E, 892834},
  {0xFE5F, 892668},
  {0xFE60, 892091},
  {0xFE61, 892107},
  {0xFE62, 892726},
  {0xFE63, 892522},
  {0xFE64, 892647},
  {0xFE65, 892498},
  {0xFE66, 892441},
  {0xFE68, 892762},
  {0xFE69, 892299},
  {0xFE6A, 892707},
  {0xFE6B, 892165},
  {0xFE70, 28684},
  {0xFE71, 78195},
  {0xFE72, 28216},
  {0xFE73, 78159},
  {0xFE74, 28977},
  {0xFE76, 28538},
  {0xFE77, 28565},
  {0xFE78, 28126},
  {0xFE79, 28153},
  {0xFE7A, 28881},
  {0xFE7B, 28908},
  {0xFE7C, 75587},
  {0xFE7D, 75615},
  {0xFE7E, 77520},
  {0xFE7F, 77547},
  {0xFE80, 36069},
  {0xFE81, 31261},
  {0xFE82, 31214},
  {0xFE83, 30952},
  {0xFE84, 30905},
  {0xFE85, 45457},
  {0xFE86, 45411},
  {0xFE87, 31085},
  {0xFE88, 31038},
  {0xFE89, 46230},
  {0xFE8A, 46136},
  {0xFE8B, 46182},
  {0xFE8C, 46279},
  {0xFE8D, 29749},
  {0xFE8E, 29719},
  {0xFE8F, 31776},
  {0xFE90, 31716},
  {0xFE91, 31745},
  {0xFE92, 31808},
  {0xFE93, 43664},
  {0xFE94, 43596},
  {0xFE95, 43538},
  {0xFE96, 43478},
  {0xFE97, 43507},
  {0xFE98, 43704},
  {0xFE99, 44195},
  {0xFE9A, 44133},
  {0xFE9B, 44163},
  {0xFE9C, 44228},
  {0xFE9D, 37000},
  {0xFE9E, 36938},
  {0xFE9F, 36968},
  {0xFEA0, 37033},
  {0xFEA1, 35496},
  {0xFEA2, 35436},
  {0xFEA3, 35465},
  {0xFEA4, 35528},
  {0xFEA5, 38185},
  {0xFEA6, 38123},
  {0xFEA7, 38153},
  {0xFEA8, 38218},
  {0xFEA9, 32774},
  {0xFEAA, 32745},
  {0xFEAB, 44081},
  {0xFEAC, 44051},
  {0xFEAD, 40852},
  {0xFEAE, 40823},
  {0xFEAF, 46855},
  {0xFEB0, 46825},
  {0xFEB1, 42011},
  {0xFEB2, 41949},
  {0xFEB3, 41979},
  {0xFEB4, 42044},
  {0xFEB5, 42559},
  {0xFEB6, 42495},
  {0xFEB7, 42526},
  {0xFEB8, 42593},
  {0xFEB9, 41750},
  {0xFEBA, 41690},
  {0xFEBB, 41719},
  {0xFEBC, 41782},
  {0xFEBD, 32547},
  {0xFEBE, 32487},
  {0xFEBF, 32516},
  {0xFEC0, 32579},
  {0xFEC1, 42820},
  {0xFEC2, 42760},
  {0xFEC3, 42789},
  {0xFEC4, 42852},
  {0xFEC5, 46744},
  {0xFEC6, 46684},
  {0xFEC7, 46713},
  {0xFEC8, 46776},
  {0xFEC9, 29412},
  {0xFECA, 29352},
  {0xFECB, 29381},
  {0xFECC, 29444},
  {0xFECD, 35111},
  {0xFECE, 35047},
  {0xFECF, 35078},
  {0xFED0, 35145},
  {0xFED1, 34419},
  {0xFED2, 34359},
  {0xFED3, 34388},
  {0xFED4, 34451},
  {0xFED5, 40586},
  {0xFED6, 40526},
  {0xFED7, 40555},
  {0xFED8, 40618},
  {0xFED9, 37342},
  {0xFEDA, 37282},
  {0xFEDB, 37311},
  {0xFEDC, 37374},
  {0xFEDD, 38527},
  {0xFEDE, 38467},
  {0xFEDF, 38496},
  {0xFEE0, 38559},
  {0xFEE1, 38970},
  {0xFEE2, 38908},
  {0xFEE3, 38938},
  {0xFEE4, 39003},
  {0xFEE5, 39612},
  {0xFEE6, 39447},
  {0xFEE7, 39580},
  {0xFEE8, 39645},
  {0xFEE9, 36586},
  {0xFEEA, 36321},
  {0xFEEB, 36555},
  {0xFEEC, 36618},
  {0xFEED, 45153},
  {0xFEEE, 45124},
  {0xFEEF, 29847},
  {0xFEF0, 29809},
  {0xFEF1, 46039},
  {0xFEF2, 45979},
  {0xFEF3, 46008},
  {0xFEF4, 46071},
  {0xFEF5, 54734},
  {0xFEF6, 54676},
  {0xFEF7, 54496},
  {0xFEF8, 54438},
  {0xFEF9, 54615},
  {0xFEFA, 54557},
  {0xFEFB, 54293},
  {0xFEFC, 54252},
  {0xFEFF, 1012886},
  {0xFF01, 445675},
  {0xFF02, 447769},
  {0xFF03, 447659},
  {0xFF04, 445631},
  {0xFF05, 447681},
  {0xFF06, 445236},
  {0xFF07, 445256},
  {0xFF08, 447489},
  {0xFF09, 447850},
  {0xFF0A, 445277},
  {0xFF0B, 447704},
  {0xFF0C, 445381},
  {0xFF0D, 445773},
  {0xFF0E, 445702},
  {0xFF0F, 447963},
  {0xFF10, 445610},
  {0xFF11, 445506},
  {0xFF12, 445590},
  {0xFF13, 445568},
  {0xFF14, 445464},
  {0xFF15, 445443},
  {0xFF16, 445548},
  {0xFF17, 445526},
  {0xFF18, 445421},
  {0xFF19, 445485},
  {0xFF1A, 445365},
  {0xFF1B, 447943},
  {0xFF1C, 447579},
  {0xFF1D, 445653},
  {0xFF1E, 445745},
  {0xFF1F, 447745},
  {0xFF20, 445397},
  {0xFF21, 445796},
  {0xFF22, 445829},
  {0xFF23, 445862},
  {0xFF24, 445895},
  {0xFF25, 445928},
  {0xFF26, 445961},
  {0xFF27, 445994},
  {0xFF28, 446027},
  {0xFF29, 446060},
  {0xFF2A, 446093},
  {0xFF2B, 446126},
  {0xFF2C, 446159},
  {0xFF2D, 446192},
  {0xFF2E, 446225},
  {0xFF2F, 446258},
  {0xFF30, 446291},
  {0xFF31, 446324},
  {0xFF32, 446357},
  {0xFF33, 446390},
  {0xFF34, 446423},
  {0xFF35, 446456},
  {0xFF36, 446489},
  {0xFF37, 446522},
  {0xFF38, 446555},
  {0xFF39, 446588},
  {0xFF3A, 446621},
  {0xFF3B, 447516},
  {0xFF3C, 447794},
  {0xFF3D, 447878},
  {0xFF3E, 445337},
  {0xFF3F, 447604},
  {0xFF40, 445722},
  {0xFF41, 446654},
  {0xFF42, 446685},
  {0xFF43, 446716},
  {0xFF44, 446747},
  {0xFF45, 446778},
  {0xFF46, 446809},
  {0xFF47, 446840},
  {0xFF48, 446871},
  {0xFF49, 446902},
  {0xFF4A, 446933},
  {0xFF4B, 446964},
  {0xFF4C, 446995},
  {0xFF4D, 447026},
  {0xFF4E, 447057},
  {0xFF4F, 447088},
  {0xFF50, 447119},
  {0xFF51, 447150},
  {0xFF52, 447181},
  {0xFF53, 447212},
  {0xFF54, 447243},
  {0xFF55, 447274},
  {0xFF56, 447305},
  {0xFF57, 447336},
  {0xFF58, 447367},
  {0xFF59, 447398},
  {0xFF5A, 447429},
  {0xFF5B, 447460},
  {0xFF5C, 447997},
  {0xFF5D, 447820},
  {0xFF5E, 447981},
  {0xFF5F, 447546},
  {0xFF60, 447909},
  {0xFF61, 489135},
  {0xFF62, 490963},
  {0xFF63, 491019},
  {0xFF64, 489107},
  {0xFF65, 490805},
  {0xFF66, 490689},
  {0xFF67, 490119},
  {0xFF68, 490187},
  {0xFF69, 490290},
  {0xFF6A, 490153},
  {0xFF6B, 490221},
  {0xFF6C, 490324},
  {0xFF6D, 490394},
  {0xFF6E, 490359},
  {0xFF6F, 490255},
  {0xFF70, 490914},
  {0xFF71, 489167},
  {0xFF72, 489368},
  {0xFF73, 490632},
  {0xFF74, 489195},
  {0xFF75, 489859},
  {0xFF76, 489396},
  {0xFF77, 489454},
  {0xFF78, 489512},
  {0xFF79, 489425},
  {0xFF7A, 489483},
  {0xFF7B, 490032},
  {0xFF7C, 490090},
  {0xFF7D, 490458},
  {0xFF7E, 490061},
  {0xFF7F, 490429},
  {0xFF80, 490487},
  {0xFF81, 490545},
  {0xFF82, 490603},
  {0xFF83, 490516},
  {0xFF84, 490574},
  {0xFF85, 489714},
  {0xFF86, 489772},
  {0xFF87, 489830},
  {0xFF88, 489743},
  {0xFF89, 489801},
  {0xFF8A, 489223},
  {0xFF8B, 489281},
  {0xFF8C, 489339},
  {0xFF8D, 489252},
  {0xFF8E, 489310},
  {0xFF8F, 489541},
  {0xFF90, 489599},
  {0xFF91, 489657},
  {0xFF92, 489570},
  {0xFF93, 489628},
  {0xFF94, 490718},
  {0xFF95, 490776},
  {0xFF96, 490747},
  {0xFF97, 489887},
  {0xFF98, 489945},
  {0xFF99, 490003},
  {0xFF9A, 489916},
  {0xFF9B, 489974},
  {0xFF9C, 490660},
  {0xFF9D, 489686},
  {0xFF9E, 490877},
  {0xFF9F, 490835},
  {0xFFA0, 487513},
  {0xFFA1, 487850},
  {0xFFA2, 488549},
  {0xFFA3, 487881},
  {0xFFA4, 487947},
  {0xFFA5, 487977},
  {0xFFA6, 488013},
  {0xFFA7, 488722},
  {0xFFA8, 488654},
  {0xFFA9, 488199},
  {0xFFAA, 488265},
  {0xFFAB, 488302},
  {0xFFAC, 488376},
  {0xFFAD, 488412},
  {0xFFAE, 488447},
  {0xFFAF, 488338},
  {0xFFB0, 488229},
  {0xFFB1, 487917},
  {0xFFB2, 488134},
  {0xFFB3, 488585},
  {0xFFB4, 488164},
  {0xFFB5, 488485},
  {0xFFB6, 488620},
  {0xFFB7, 487788},
  {0xFFB8, 487622},
  {0xFFB9, 488514},
  {0xFFBA, 487590},
  {0xFFBB, 487818},
  {0xFFBC, 488690},
  {0xFFBD, 488102},
  {0xFFBE, 487732},
  {0xFFC2, 487537},
  {0xFFC3, 487563},
  {0xFFC4, 488916},
  {0xFFC5, 488943},
  {0xFFC6, 487678},
  {0xFFC7, 487652},
  {0xFFCA, 488998},
  {0xFFCB, 488971},
  {0xFFCC, 488049},
  {0xFFCD, 488779},
  {0xFFCE, 488806},
  {0xFFCF, 488075},
  {0xFFD2, 489053},
  {0xFFD3, 488753},
  {0xFFD4, 488861},
  {0xFFD5, 488834},
  {0xFFD6, 488889},
  {0xFFD7, 489080},
  {0xFFDA, 487705},
  {0xFFDB, 489026},
  {0xFFDC, 487762},
  {0xFFE0, 445317},
  {0xFFE1, 447724},
  {0xFFE2, 447640},
  {0xFFE3, 447623},
  {0xFFE4, 445296},
  {0xFFE5, 448040},
  {0xFFE6, 448021},
  {0xFFE8, 487482},
  {0xFFE9, 490993},
  {0xFFEA, 491077},
  {0xFFEB, 491050},
  {0xFFEC, 487456},
  {0xFFED, 487433},
  {0xFFEE, 491101},
  {0xFFF9, 535386},
  {0xFFFA, 535416},
  {0xFFFB, 535449},
  {0xFFFC, 785341},
  {0xFFFD, 832574},
  {0x10000, 648703},
  {0x10001, 649352},
  {0x10002, 649143},
  {0x10003, 649872},
  {0x10004, 648754},
  {0x10005, 648521},
  {0x10006, 649533},
  {0x10007, 648677},
  {0x10008, 648857},
  {0x10009, 649638},
  {0x1000A, 649768},
  {0x1000B, 649559},
  {0x1000D, 649300},
  {0x1000E, 649924},
  {0x1000F, 650240},
  {0x10010, 649507},
  {0x10011, 649977},
  {0x10012, 650056},
  {0x10013, 650318},
  {0x10014, 650292},
  {0x10015, 648831},
  {0x10016, 650135},
  {0x10017, 648883},
  {0x10018, 649013},
  {0x10019, 648651},
  {0x1001A, 649039},
  {0x1001B, 649195},
  {0x1001C, 649664},
  {0x1001D, 649742},
  {0x1001E, 648573},
  {0x1001F, 650109},
  {0x10020, 649377},
  {0x10021, 648779},
  {0x10022, 649612},
  {0x10023, 648909},
  {0x10024, 650266},
  {0x10025, 648987},
  {0x10026, 649247},
  {0x10028, 649846},
  {0x10029, 649117},
  {0x1002A, 649690},
  {0x1002B, 648547},
  {0x1002C, 649091},
  {0x1002D, 649221},
  {0x1002E, 648728},
  {0x1002F, 649429},
  {0x10030, 648805},
  {0x10031, 649794},
  {0x10032, 649820},
  {0x10033, 648599},
  {0x10034, 649326},
  {0x10035, 648625},
  {0x10036, 650030},
  {0x10037, 649716},
  {0x10038, 650187},
  {0x10039, 649403},
  {0x1003A, 649455},
  {0x1003C, 648935},
  {0x1003D, 650161},
  {0x1003F, 648961},
  {0x10040, 649065},
  {0x10041, 649481},
  {0x10042, 650344},
  {0x10043, 650082},
  {0x10044, 650397},
  {0x10045, 649585},
  {0x10046, 649168},
  {0x10047, 649897},
  {0x10048, 650213},
  {0x10049, 649273},
  {0x1004A, 650003},
  {0x1004B, 649950},
  {0x1004C, 650370},
  {0x1004D, 650424},
  {0x10050, 650451},
  {0x10051, 650472},
  {0x10052, 650493},
  {0x10053, 650514},
  {0x10054, 650535},
  {0x10055, 650556},
  {0x10056, 650577},
  {0x10057, 650598},
  {0x10058, 650619},
  {0x10059, 650640},
  {0x1005A, 650661},
  {0x1005B, 650682},
  {0x1005C, 650703},
  {0x1005D, 650724},
  {0x10080, 645178},
  {0x10081, 645205},
  {0x10082, 645234},
  {0x10083, 645262},
  {0x10084, 645291},
  {0x10085, 645320},
  {0x10086, 645353},
  {0x10087, 645381},
  {0x10088, 645409},
  {0x10089, 645442},
  {0x1008A, 645474},
  {0x1008B, 645502},
  {0x1008C, 645531},
  {0x1008D, 645559},
  {0x1008E, 645588},
  {0x1008F, 645617},
  {0x10090, 645647},
  {0x10091, 645676},
  {0x10092, 645705},
  {0x10093, 648348},
  {0x10094, 648376},
  {0x10095, 645736},
  {0x10096, 645763},
  {0x10097, 645791},
  {0x10098, 648406},
  {0x10099, 648435},
  {0x1009A, 645814},
  {0x1009B, 645844},
  {0x1009C, 645872},
  {0x1009D, 645895},
  {0x1009E, 645923},
  {0x1009F, 645946},
  {0x100A0, 645969},
  {0x100A1, 645997},
  {0x100A2, 646020},
  {0x100A3, 646043},
  {0x100A4, 648463},
  {0x100A5, 646066},
  {0x100A6, 646089},
  {0x100A7, 646112},
  {0x100A8, 646141},
  {0x100A9, 646164},
  {0x100AA, 646187},
  {0x100AB, 646218},
  {0x100AC, 646248},
  {0x100AD, 646271},
  {0x100AE, 646294},
  {0x100AF, 646317},
  {0x100B0, 646340},
  {0x100B1, 646363},
  {0x100B2, 646386},
  {0x100B3, 646409},
  {0x100B4, 646432},
  {0x100B5, 646455},
  {0x100B6, 646484},
  {0x100B7, 646507},
  {0x100B8, 646535},
  {0x100B9, 646558},
  {0x100BA, 646581},
  {0x100BB, 646604},
  {0x100BC, 646627},
  {0x100BD, 646650},
  {0x100BE, 646673},
  {0x100BF, 646696},
  {0x100C0, 646719},
  {0x100C1, 646742},
  {0x100C2, 646765},
  {0x100C3, 646788},
  {0x100C4, 646818},
  {0x100C5, 646851},
  {0x100C6, 646882},
  {0x100C7, 646911},
  {0x100C8, 646940},
  {0x100C9, 646963},
  {0x100CA, 646992},
  {0x100CB, 647015},
  {0x100CC, 647038},
  {0x100CD, 647077},
  {0x100CE, 647108},
  {0x100CF, 647145},
  {0x100D0, 647174},
  {0x100D1, 647197},
  {0x100D2, 648492},
  {0x100D3, 647220},
  {0x100D4, 647243},
  {0x100D5, 647266},
  {0x100D6, 647289},
  {0x100D7, 647312},
  {0x100D8, 647335},
  {0x100D9, 647363},
  {0x100DA, 647386},
  {0x100DB, 647409},
  {0x100DC, 647432},
  {0x100DD, 647455},
  {0x100DE, 647478},
  {0x100DF, 647508},
  {0x100E0, 647538},
  {0x100E1, 647568},
  {0x100E2, 647598},
  {0x100E3, 647628},
  {0x100E4, 647658},
  {0x100E5, 647688},
  {0x100E6, 647718},
  {0x100E7, 647748},
  {0x100E8, 647778},
  {0x100E9, 647808},
  {0x100EA, 647838},
  {0x100EB, 647868},
  {0x100EC, 647898},
  {0x100ED, 647928},
  {0x100EE, 647958},
  {0x100EF, 647988},
  {0x100F0, 648018},
  {0x100F1, 648048},
  {0x100F2, 648078},
  {0x100F3, 648108},
  {0x100F4, 648138},
  {0x100F5, 648168},
  {0x100F6, 648198},
  {0x100F7, 648228},
  {0x100F8, 648258},
  {0x100F9, 648288},
  {0x100FA, 648318},
  {0x10100, 3670},
  {0x10101, 3644},
  {0x10102, 2245},
  {0x10107, 2890},
  {0x10108, 3435},
  {0x10109, 3307},
  {0x1010A, 2691},
  {0x1010B, 2568},
  {0x1010C, 3091},
  {0x1010D, 2961},
  {0x1010E, 2391},
  {0x1010F, 2765},
  {0x10110, 3211},
  {0x10111, 3384},
  {0x10112, 3256},
  {0x10113, 2642},
  {0x10114, 2519},
  {0x10115, 3162},
  {0x10116, 3038},
  {0x10117, 2468},
  {0x10118, 2839},
  {0x10119, 2908},
  {0x1011A, 3453},
  {0x1011B, 3327},
  {0x1011C, 2710},
  {0x1011D, 2587},
  {0x1011E, 3109},
  {0x1011F, 2981},
  {0x10120, 2411},
  {0x10121, 2784},
  {0x10122, 2934},
  {0x10123, 3479},
  {0x10124, 3355},
  {0x10125, 2737},
  {0x10126, 2614},
  {0x10127, 3135},
  {0x10128, 3009},
  {0x10129, 2439},
  {0x1012A, 2811},
  {0x1012B, 3229},
  {0x1012C, 3405},
  {0x1012D, 3277},
  {0x1012E, 2662},
  {0x1012F, 2539},
  {0x10130, 3182},
  {0x10131, 3060},
  {0x10132, 2489},
  {0x10133, 2860},
  {0x10137, 3506},
  {0x10138, 3530},
  {0x10139, 3587},
  {0x1013A, 3616},
  {0x1013B, 3558},
  {0x1013C, 2263},
  {0x1013D, 2296},
  {0x1013E, 2332},
  {0x1013F, 2362},
  {0x10140, 461727},
  {0x10141, 461609},
  {0x10142, 461574},
  {0x10143, 461268},
  {0x10144, 461081},
  {0x10145, 461296},
  {0x10146, 461492},
  {0x10147, 461184},
  {0x10148, 461456},
  {0x10149, 461917},
  {0x1014A, 461147},
  {0x1014B, 461684},
  {0x1014C, 461376},
  {0x1014D, 461806},
  {0x1014E, 461529},
  {0x1014F, 461420},
  {0x10150, 461882},
  {0x10151, 461110},
  {0x10152, 461641},
  {0x10153, 461332},
  {0x10154, 461762},
  {0x10155, 461952},
  {0x10156, 461222},
  {0x10157, 461850},
  {0x10158, 462224},
  {0x10159, 462576},
  {0x1015A, 462296},
  {0x1015B, 462151},
  {0x1015C, 462786},
  {0x1015D, 462036},
  {0x1015E, 462183},
  {0x1015F, 462899},
  {0x10160, 462973},
  {0x10161, 463005},
  {0x10162, 462328},
  {0x10163, 462360},
  {0x10164, 462683},
  {0x10165, 462713},
  {0x10166, 462816},
  {0x10167, 462850},
  {0x10168, 462262},
  {0x10169, 462465},
  {0x1016A, 462606},
  {0x1016B, 462746},
  {0x1016C, 462110},
  {0x1016D, 462932},
  {0x1016E, 462497},
  {0x1016F, 461996},
  {0x10170, 462391},
  {0x10171, 462644},
  {0x10172, 462536},
  {0x10173, 462075},
  {0x10174, 462428},
  {0x10175, 471477},
  {0x10176, 471497},
  {0x10177, 479393},
  {0x10178, 479309},
  {0x10179, 480452},
  {0x1017A, 479249},
  {0x1017B, 469415},
  {0x1017C, 471461},
  {0x1017D, 479372},
  {0x1017E, 479286},
  {0x1017F, 469456},
  {0x10180, 469434},
  {0x10181, 471401},
  {0x10182, 470967},
  {0x10183, 471305},
  {0x10184, 471555},
  {0x10185, 480434},
  {0x10186, 463087},
  {0x10187, 463069},
  {0x10188, 469478},
  {0x10189, 479347},
  {0x1018A, 480488},
  {0x1018B, 471532},
  {0x1018C, 471842},
  {0x1018D, 469496},
  {0x1018E, 771825},
  {0x10190, 842793},
  {0x10191, 842850},
  {0x10192, 842751},
  {0x10193, 842812},
  {0x10194, 842107},
  {0x10195, 842831},
  {0x10196, 842087},
  {0x10197, 842730},
  {0x10198, 842771},
  {0x10199, 842134},
  {0x1019A, 842052},
  {0x1019B, 842066},
  {0x1019C, 81986},
  {0x101A0, 479228},
  {0x101D0, 821762},
  {0x101D1, 821822},
  {0x101D2, 822058},
  {0x101D3, 821150},
  {0x101D4, 821235},
  {0x101D5, 822194},
  {0x101D6, 821532},
  {0x101D7, 821478},
  {0x101D8, 822091},
  {0x101D9, 820994},
  {0x101DA, 821098},
  {0x101DB, 821926},
  {0x101DC, 821260},
  {0x101DD, 821653},
  {0x101DE, 821681},
  {0x101DF, 821903},
  {0x101E0, 821606},
  {0x101E1, 821069},
  {0x101E2, 821177},
  {0x101E3, 821378},
  {0x101E4, 821310},
  {0x101E5, 821976},
  {0x101E6, 821284},
  {0x101E7, 821042},
  {0x101E8, 821952},
  {0x101E9, 821582},
  {0x101EA, 821558},
  {0x101EB, 821121},
  {0x101EC, 821212},
  {0x101ED, 821853},
  {0x101EE, 821428},
  {0x101EF, 821404},
  {0x101F0, 822116},
  {0x101F1, 821019},
  {0x101F2, 821792},
  {0x101F3, 822141},
  {0x101F4, 821735},
  {0x101F5, 821876},
  {0x101F6, 821629},
  {0x101F7, 821708},
  {0x101F8, 821453},
  {0x101F9, 821506},
  {0x101FA, 822030},
  {0x101FB, 822001},
  {0x101FC, 822165},
  {0x101FD, 821334},
  {0x10280, 656868},
  {0x10281, 656950},
  {0x10282, 656901},
  {0x10283, 656917},
  {0x10284, 656983},
  {0x10285, 656934},
  {0x10286, 657015},
  {0x10287, 657292},
  {0x10288, 657324},
  {0x10289, 657242},
  {0x1028A, 657031},
  {0x1028B, 657047},
  {0x1028C, 657178},
  {0x1028D, 657080},
  {0x1028E, 657096},
  {0x1028F, 657129},
  {0x10290, 657112},
  {0x10291, 657145},
  {0x10292, 657276},
  {0x10293, 657162},
  {0x10294, 657063},
  {0x10295, 657194},
  {0x10296, 657210},
  {0x10297, 657226},
  {0x10298, 657259},
  {0x10299, 656884},
  {0x1029A, 656966},
  {0x1029B, 656999},
  {0x1029C, 657308},
  {0x102A0, 173772},
  {0x102A1, 174261},
  {0x102A2, 173859},
  {0x102A3, 174024},
  {0x102A4, 174514},
  {0x102A5, 174294},
  {0x102A6, 174040},
  {0x102A7, 173788},
  {0x102A8, 174278},
  {0x102A9, 173805},
  {0x102AA, 174075},
  {0x102AB, 174229},
  {0x102AC, 173875},
  {0x102AD, 174430},
  {0x102AE, 174343},
  {0x102AF, 174360},
  {0x102B0, 174327},
  {0x102B1, 173821},
  {0x102B2, 174481},
  {0x102B3, 174212},
  {0x102B4, 174589},
  {0x102B5, 174162},
  {0x102B6, 174463},
  {0x102B7, 174245},
  {0x102B8, 174378},
  {0x102B9, 173958},
  {0x102BA, 173892},
  {0x102BB, 174570},
  {0x102BC, 173991},
  {0x102BD, 174007},
  {0x102BE, 174178},
  {0x102BF, 174497},
  {0x102C0, 173925},
  {0x102C1, 173941},
  {0x102C2, 174395},
  {0x102C3, 174412},
  {0x102C4, 174195},
  {0x102C5, 173974},
  {0x102C6, 173840},
  {0x102C7, 174446},
  {0x102C8, 174532},
  {0x102C9, 174310},
  {0x102CA, 174091},
  {0x102CB, 174108},
  {0x102CC, 174126},
  {0x102CD, 174144},
  {0x102CE, 174057},
  {0x102CF, 173908},
  {0x102D0, 174551},
  {0x102E0, 211999},
  {0x102E1, 211345},
  {0x102E2, 211441},
  {0x102E3, 211416},
  {0x102E4, 211297},
  {0x102E5, 211273},
  {0x102E6, 211393},
  {0x102E7, 211368},
  {0x102E8, 211248},
  {0x102E9, 211321},
  {0x102EA, 211855},
  {0x102EB, 211940},
  {0x102EC, 211879},
  {0x102ED, 211584},
  {0x102EE, 211525},
  {0x102EF, 211829},
  {0x102F0, 211769},
  {0x102F1, 211498},
  {0x102F2, 211676},
  {0x102F3, 211703},
  {0x102F4, 211967},
  {0x102F5, 211906},
  {0x102F6, 211610},
  {0x102F7, 211551},
  {0x102F8, 211797},
  {0x102F9, 211735},
  {0x102FA, 211464},
  {0x102FB, 211643},
  {0x10300, 791602},
  {0x10301, 791622},
  {0x10302, 792003},
  {0x10303, 791665},
  {0x10304, 791686},
  {0x10305, 792298},
  {0x10306, 792340},
  {0x10307, 791920},
  {0x10308, 792235},
  {0x10309, 791941},
  {0x1030A, 791982},
  {0x1030B, 791749},
  {0x1030C, 791770},
  {0x1030D, 791791},
  {0x1030E, 791876},
  {0x1030F, 792098},
  {0x10310, 792118},
  {0x10311, 792161},
  {0x10312, 792046},
  {0x10313, 791812},
  {0x10314, 791855},
  {0x10315, 792214},
  {0x10316, 792257},
  {0x10317, 791727},
  {0x10318, 792139},
  {0x10319, 792024},
  {0x1031A, 791706},
  {0x1031B, 791833},
  {0x1031C, 791643},
  {0x1031D, 791961},
  {0x1031E, 792277},
  {0x1031F, 791898},
  {0x10320, 792410},
  {0x10321, 792386},
  {0x10322, 792433},
  {0x10323, 792361},
  {0x1032D, 792319},
  {0x1032E, 792067},
  {0x1032F, 792183},
  {0x10330, 457948},
  {0x10331, 457988},
  {0x10332, 458067},
  {0x10333, 458010},
  {0x10334, 457967},
  {0x10335, 458354},
  {0x10336, 458145},
  {0x10337, 458086},
  {0x10338, 458437},
  {0x10339, 458029},
  {0x1033A, 458182},
  {0x1033B, 458202},
  {0x1033C, 458222},
  {0x1033D, 458242},
  {0x1033E, 458164},
  {0x1033F, 458458},
  {0x10340, 458331},
  {0x10341, 458290},
  {0x10342, 458377},
  {0x10343, 458397},
  {0x10344, 458417},
  {0x10345, 458477},
  {0x10346, 458047},
  {0x10347, 458125},
  {0x10348, 458105},
  {0x10349, 458311},
  {0x1034A, 458263},
  {0x10350, 793408},
  {0x10351, 793429},
  {0x10352, 793586},
  {0x10353, 793475},
  {0x10354, 793545},
  {0x10355, 794229},
  {0x10356, 793497},
  {0x10357, 794206},
  {0x10358, 793521},
  {0x10359, 793629},
  {0x1035A, 793691},
  {0x1035B, 793714},
  {0x1035C, 793736},
  {0x1035D, 793760},
  {0x1035E, 794029},
  {0x1035F, 793825},
  {0x10360, 793848},
  {0x10361, 793920},
  {0x10362, 793942},
  {0x10363, 793987},
  {0x10364, 793451},
  {0x10365, 793896},
  {0x10366, 793870},
  {0x10367, 794163},
  {0x10368, 794140},
  {0x10369, 793784},
  {0x1036A, 793804},
  {0x1036B, 793565},
  {0x1036C, 793608},
  {0x1036D, 793964},
  {0x1036E, 794007},
  {0x1036F, 794095},
  {0x10370, 794117},
  {0x10371, 794073},
  {0x10372, 793670},
  {0x10373, 794185},
  {0x10374, 794052},
  {0x10375, 793649},
  {0x10376, 206794},
  {0x10377, 206825},
  {0x10378, 206923},
  {0x10379, 206857},
  {0x1037A, 206891},
  {0x10380, 954201},
  {0x10381, 954222},
  {0x10382, 954286},
  {0x10383, 954408},
  {0x10384, 954243},
  {0x10385, 954330},
  {0x10386, 954716},
  {0x10387, 954755},
  {0x10388, 954349},
  {0x10389, 954636},
  {0x1038A, 954735},
  {0x1038B, 954388},
  {0x1038C, 954595},
  {0x1038D, 954428},
  {0x1038E, 954450},
  {0x1038F, 954265},
  {0x10390, 954470},
  {0x10391, 954776},
  {0x10392, 954573},
  {0x10393, 954181},
  {0x10394, 954490},
  {0x10395, 954552},
  {0x10396, 954509},
  {0x10397, 954530},
  {0x10398, 954656},
  {0x10399, 954308},
  {0x1039A, 954679},
  {0x1039B, 954370},
  {0x1039C, 954698},
  {0x1039D, 954616},
  {0x1039F, 954795},
  {0x103A0, 794374},
  {0x103A1, 794764},
  {0x103A2, 795166},
  {0x103A3, 794823},
  {0x103A4, 794843},
  {0x103A5, 794704},
  {0x103A6, 794724},
  {0x103A7, 795225},
  {0x103A8, 794548},
  {0x103A9, 794783},
  {0x103AA, 794803},
  {0x103AB, 795105},
  {0x103AC, 795146},
  {0x103AD, 794568},
  {0x103AE, 794644},
  {0x103AF, 794664},
  {0x103B0, 795125},
  {0x103B1, 794983},
  {0x103B2, 794481},
  {0x103B3, 794684},
  {0x103B4, 794943},
  {0x103B5, 794963},
  {0x103B6, 794883},
  {0x103B7, 794903},
  {0x103B8, 794923},
  {0x103B9, 795275},
  {0x103BA, 795185},
  {0x103BB, 795205},
  {0x103BC, 795003},
  {0x103BD, 795023},
  {0x103BE, 794863},
  {0x103BF, 795043},
  {0x103C0, 795295},
  {0x103C1, 795063},
  {0x103C2, 795084},
  {0x103C3, 794744},
  {0x103C8, 794393},
  {0x103C9, 794421},
  {0x103CA, 794451},
  {0x103CB, 795245},
  {0x103CC, 794588},
  {0x103CD, 794615},
  {0x103CE, 794501},
  {0x103CF, 794523},
  {0x103D0, 795315},
  {0x103D1, 794279},
  {0x103D2, 794351},
  {0x103D3, 794302},
  {0x103D4, 794325},
  {0x103D5, 794252},
  {0x10400, 270949},
  {0x10401, 270919},
  {0x10402, 270858},
  {0x10403, 270888},
  {0x10404, 270979},
  {0x10405, 271009},
  {0x10406, 271213},
  {0x10407, 271182},
  {0x10408, 271119},
  {0x10409, 271150},
  {0x1040A, 271244},
  {0x1040B, 271275},
  {0x1040C, 270381},
  {0x1040D, 271066},
  {0x1040E, 271389},
  {0x1040F, 271415},
  {0x10410, 270779},
  {0x10411, 271092},
  {0x10412, 270407},
  {0x10413, 271307},
  {0x10414, 270462},
  {0x10415, 270434},
  {0x10416, 270804},
  {0x10417, 270831},
  {0x10418, 270752},
  {0x10419, 270489},
  {0x1041A, 271362},
  {0x1041B, 270699},
  {0x1041C, 271334},
  {0x1041D, 270646},
  {0x1041E, 271442},
  {0x1041F, 270672},
  {0x10420, 271469},
  {0x10421, 270620},
  {0x10422, 270515},
  {0x10423, 270541},
  {0x10424, 270567},
  {0x10425, 270593},
  {0x10426, 271040},
  {0x10427, 270726},
  {0x10428, 272023},
  {0x10429, 271995},
  {0x1042A, 271938},
  {0x1042B, 271966},
  {0x1042C, 272051},
  {0x1042D, 272079},
  {0x1042E, 272269},
  {0x1042F, 272240},
  {0x10430, 272181},
  {0x10431, 272210},
  {0x10432, 272298},
  {0x10433, 272327},
  {0x10434, 271497},
  {0x10435, 272132},
  {0x10436, 272433},
  {0x10437, 272457},
  {0x10438, 271865},
  {0x10439, 272156},
  {0x1043A, 271521},
  {0x1043B, 272357},
  {0x1043C, 271572},
  {0x1043D, 271546},
  {0x1043E, 271888},
  {0x1043F, 271913},
  {0x10440, 271840},
  {0x10441, 271597},
  {0x10442, 272408},
  {0x10443, 271791},
  {0x10444, 272382},
  {0x10445, 271742},
  {0x10446, 272482},
  {0x10447, 271766},
  {0x10448, 272507},
  {0x10449, 271718},
  {0x1044A, 271621},
  {0x1044B, 271645},
  {0x1044C, 271669},
  {0x1044D, 271693},
  {0x1044E, 272108},
  {0x1044F, 271816},
  {0x10450, 855854},
  {0x10451, 855973},
  {0x10452, 855639},
  {0x10453, 855483},
  {0x10454, 855952},
  {0x10455, 855894},
  {0x10456, 855912},
  {0x10457, 855365},
  {0x10458, 856068},
  {0x10459, 855542},
  {0x1045A, 855346},
  {0x1045B, 855387},
  {0x1045C, 855502},
  {0x1045D, 856010},
  {0x1045E, 855932},
  {0x1045F, 856106},
  {0x10460, 855679},
  {0x10461, 855618},
  {0x10462, 856029},
  {0x10463, 855521},
  {0x10464, 855659},
  {0x10465, 855702},
  {0x10466, 855600},
  {0x10467, 855445},
  {0x10468, 855308},
  {0x10469, 855193},
  {0x1046A, 855779},
  {0x1046B, 856048},
  {0x1046C, 855835},
  {0x1046D, 855231},
  {0x1046E, 855874},
  {0x1046F, 855722},
  {0x10470, 855426},
  {0x10471, 855212},
  {0x10472, 855581},
  {0x10473, 855992},
  {0x10474, 855741},
  {0x10475, 855797},
  {0x10476, 855760},
  {0x10477, 855327},
  {0x10478, 855268},
  {0x10479, 855817},
  {0x1047A, 855249},
  {0x1047B, 855464},
  {0x1047C, 855287},
  {0x1047D, 855407},
  {0x1047E, 855562},
  {0x1047F, 856087},
  {0x10480, 804784},
  {0x10481, 804804},
  {0x10482, 805216},
  {0x10483, 804987},
  {0x10484, 805270},
  {0x10485, 805025},
  {0x10486, 804842},
  {0x10487, 805159},
  {0x10488, 805177},
  {0x10489, 805195},
  {0x1048A, 804862},
  {0x1048B, 804822},
  {0x1048C, 804934},
  {0x1048D, 804916},
  {0x1048E, 805139},
  {0x1048F, 805005},
  {0x10490, 805044},
  {0x10491, 805064},
  {0x10492, 805084},
  {0x10493, 805251},
  {0x10494, 804952},
  {0x10495, 805288},
  {0x10496, 804749},
  {0x10497, 804881},
  {0x10498, 804970},
  {0x10499, 805104},
  {0x1049A, 805234},
  {0x1049B, 804766},
  {0x1049C, 804898},
  {0x1049D, 805121},
  {0x104A0, 804730},
  {0x104A1, 804636},
  {0x104A2, 804712},
  {0x104A3, 804692},
  {0x104A4, 804598},
  {0x104A5, 804579},
  {0x104A6, 804674},
  {0x104A7, 804654},
  {0x104A8, 804559},
  {0x104A9, 804617},
  {0x104B0, 802859},
  {0x104B1, 802906},
  {0x104B2, 802930},
  {0x104B3, 802882},
  {0x104B4, 802955},
  {0x104B5, 802980},
  {0x104B6, 803053},
  {0x104B7, 803030},
  {0x104B8, 803185},
  {0x104B9, 803235},
  {0x104BA, 803259},
  {0x104BB, 803284},
  {0x104BC, 803307},
  {0x104BD, 803080},
  {0x104BE, 803356},
  {0x104BF, 803381},
  {0x104C0, 803405},
  {0x104C1, 803429},
  {0x104C2, 803453},
  {0x104C3, 803476},
  {0x104C4, 803501},
  {0x104C5, 803106},
  {0x104C6, 803525},
  {0x104C7, 803549},
  {0x104C8, 803574},
  {0x104C9, 803132},
  {0x104CA, 803598},
  {0x104CB, 803158},
  {0x104CC, 803623},
  {0x104CD, 803005},
  {0x104CE, 803649},
  {0x104CF, 803672},
  {0x104D0, 803331},
  {0x104D1, 803210},
  {0x104D2, 803696},
  {0x104D3, 803720},
  {0x104D8, 803745},
  {0x104D9, 803788},
  {0x104DA, 803810},
  {0x104DB, 803766},
  {0x104DC, 803833},
  {0x104DD, 803856},
  {0x104DE, 803923},
  {0x104DF, 803902},
  {0x104E0, 804045},
  {0x104E1, 804091},
  {0x104E2, 804113},
  {0x104E3, 804136},
  {0x104E4, 804157},
  {0x104E5, 803948},
  {0x104E6, 804202},
  {0x104E7, 804225},
  {0x104E8, 804247},
  {0x104E9, 804269},
  {0x104EA, 804291},
  {0x104EB, 804312},
  {0x104EC, 804335},
  {0x104ED, 803972},
  {0x104EE, 804357},
  {0x104EF, 804379},
  {0x104F0, 804402},
  {0x104F1, 803996},
  {0x104F2, 804424},
  {0x104F3, 804020},
  {0x104F4, 804447},
  {0x104F5, 803879},
  {0x104F6, 804471},
  {0x104F7, 804492},
  {0x104F8, 804179},
  {0x104F9, 804068},
  {0x104FA, 804514},
  {0x104FB, 804536},
  {0x10500, 426702},
  {0x10501, 426719},
  {0x10502, 426737},
  {0x10503, 426755},
  {0x10504, 426774},
  {0x10505, 427105},
  {0x10506, 426792},
  {0x10507, 426828},
  {0x10508, 426811},
  {0x10509, 426846},
  {0x1050A, 426864},
  {0x1050B, 426923},
  {0x1050C, 426942},
  {0x1050D, 426960},
  {0x1050E, 426977},
  {0x1050F, 426995},
  {0x10510, 427032},
  {0x10511, 427050},
  {0x10512, 427069},
  {0x10513, 427124},
  {0x10514, 427087},
  {0x10515, 427142},
  {0x10516, 427161},
  {0x10517, 427178},
  {0x10518, 427196},
  {0x10519, 427214},
  {0x1051A, 427232},
  {0x1051B, 427251},
  {0x1051C, 427269},
  {0x1051D, 427288},
  {0x1051E, 427306},
  {0x1051F, 427325},
  {0x10520, 427342},
  {0x10521, 427360},
  {0x10522, 427378},
  {0x10523, 427395},
  {0x10524, 427413},
  {0x10525, 426904},
  {0x10526, 426882},
  {0x10527, 427013},
  {0x10530, 174765},
  {0x10531, 174825},
  {0x10532, 175345},
  {0x10533, 175100},
  {0x10534, 175224},
  {0x10535, 176260},
  {0x10536, 175253},
  {0x10537, 176321},
  {0x10538, 176044},
  {0x10539, 174916},
  {0x1053A, 176229},
  {0x1053B, 176291},
  {0x1053C, 175437},
  {0x1053D, 175952},
  {0x1053E, 175619},
  {0x1053F, 175406},
  {0x10540, 176167},
  {0x10541, 175130},
  {0x10542, 174855},
  {0x10543, 175528},
  {0x10544, 175559},
  {0x10545, 175649},
  {0x10546, 175375},
  {0x10547, 175861},
  {0x10548, 174795},
  {0x10549, 175007},
  {0x1054A, 174977},
  {0x1054B, 175069},
  {0x1054C, 175680},
  {0x1054D, 175831},
  {0x1054E, 175710},
  {0x1054F, 175192},
  {0x10550, 175982},
  {0x10551, 175497},
  {0x10552, 175741},
  {0x10553, 176105},
  {0x10554, 175283},
  {0x10555, 175161},
  {0x10556, 174946},
  {0x10557, 175770},
  {0x10558, 175313},
  {0x10559, 175891},
  {0x1055A, 175921},
  {0x1055B, 176136},
  {0x1055C, 176074},
  {0x1055D, 176013},
  {0x1055E, 175467},
  {0x1055F, 175038},
  {0x10560, 174885},
  {0x10561, 176198},
  {0x10562, 175800},
  {0x10563, 175589},
  {0x1056F, 174732},
  {0x10570, 976170},
  {0x10571, 976196},
  {0x10572, 976224},
  {0x10573, 976251},
  {0x10574, 976278},
  {0x10575, 976306},
  {0x10576, 976333},
  {0x10577, 976387},
  {0x10578, 976361},
  {0x10579, 976414},
  {0x1057A, 976441},
  {0x1057C, 976468},
  {0x1057D, 976495},
  {0x1057E, 976523},
  {0x1057F, 976549},
  {0x10580, 976577},
  {0x10581, 976604},
  {0x10582, 976631},
  {0x10583, 976658},
  {0x10584, 976686},
  {0x10585, 976713},
  {0x10586, 976740},
  {0x10587, 976768},
  {0x10588, 976794},
  {0x10589, 976821},
  {0x1058A, 976848},
  {0x1058C, 976875},
  {0x1058D, 976902},
  {0x1058E, 976930},
  {0x1058F, 976957},
  {0x10590, 976985},
  {0x10591, 977011},
  {0x10592, 977038},
  {0x10594, 977065},
  {0x10595, 977091},
  {0x10597, 977118},
  {0x10598, 977142},
  {0x10599, 977168},
  {0x1059A, 977193},
  {0x1059B, 977218},
  {0x1059C, 977244},
  {0x1059D, 977269},
  {0x1059E, 977319},
  {0x1059F, 977295},
  {0x105A0, 977344},
  {0x105A1, 977369},
  {0x105A3, 977394},
  {0x105A4, 977419},
  {0x105A5, 977445},
  {0x105A6, 977469},
  {0x105A7, 977495},
  {0x105A8, 977520},
  {0x105A9, 977545},
  {0x105AA, 977570},
  {0x105AB, 977596},
  {0x105AC, 977621},
  {0x105AD, 977646},
  {0x105AE, 977672},
  {0x105AF, 977696},
  {0x105B0, 977721},
  {0x105B1, 977746},
  {0x105B3, 977771},
  {0x105B4, 977796},
  {0x105B5, 977822},
  {0x105B6, 977847},
  {0x105B7, 977873},
  {0x105B8, 977897},
  {0x105B9, 977922},
  {0x105BB, 977947},
  {0x105BC, 977971},
  {0x105C0, 947073},
  {0x105C1, 947089},
  {0x105C2, 947106},
  {0x105C3, 947468},
  {0x105C4, 947123},
  {0x105C5, 947140},
  {0x105C6, 947158},
  {0x105C7, 947503},
  {0x105C8, 947175},
  {0x105C9, 947209},
  {0x105CA, 947193},
  {0x105CB, 947226},
  {0x105CC, 947243},
  {0x105CD, 947521},
  {0x105CE, 947278},
  {0x105CF, 947539},
  {0x105D0, 947296},
  {0x105D1, 947313},
  {0x105D2, 947331},
  {0x105D3, 947347},
  {0x105D4, 947381},
  {0x105D5, 947416},
  {0x105D6, 947433},
  {0x105D7, 947451},
  {0x105D8, 947486},
  {0x105D9, 947558},
  {0x105DA, 947614},
  {0x105DB, 947647},
  {0x105DC, 947682},
  {0x105DD, 947699},
  {0x105DE, 947716},
  {0x105DF, 947734},
  {0x105E0, 947751},
  {0x105E1, 947769},
  {0x105E2, 947825},
  {0x105E3, 947842},
  {0x105E4, 947860},
  {0x105E5, 947876},
  {0x105E6, 947893},
  {0x105E7, 947577},
  {0x105E8, 947910},
  {0x105E9, 947595},
  {0x105EA, 947928},
  {0x105EB, 947364},
  {0x105EC, 947944},
  {0x105ED, 947961},
  {0x105EE, 947260},
  {0x105EF, 947807},
  {0x105F0, 947788},
  {0x105F1, 947398},
  {0x105F2, 947664},
  {0x105F3, 947630},
  {0x10600, 643551},
  {0x10601, 643571},
  {0x10602, 643591},
  {0x10603, 643611},
  {0x10604, 643631},
  {0x10605, 643651},
  {0x10606, 643671},
  {0x10607, 643691},
  {0x10608, 643711},
  {0x10609, 643731},
  {0x1060A, 643751},
  {0x1060B, 643771},
  {0x1060C, 643791},
  {0x1060D, 643811},
  {0x1060E, 643831},
  {0x1060F, 643851},
  {0x10610, 643871},
  {0x10611, 643892},
  {0x10612, 643913},
  {0x10613, 643933},
  {0x10614, 643954},
  {0x10615, 643975},
  {0x10616, 643995},
  {0x10617, 644016},
  {0x10618, 644036},
  {0x10619, 644056},
  {0x1061A, 644076},
  {0x1061B, 638454},
  {0x1061C, 644096},
  {0x1061D, 644116},
  {0x1061E, 644136},
  {0x1061F, 644156},
  {0x10620, 644176},
  {0x10621, 644196},
  {0x10622, 644216},
  {0x10623, 644236},
  {0x10624, 644256},
  {0x10625, 644276},
  {0x10626, 644296},
  {0x10627, 644316},
  {0x10628, 644336},
  {0x10629, 644356},
  {0x1062A, 644376},
  {0x1062B, 644396},
  {0x1062C, 644416},
  {0x1062D, 644436},
  {0x1062E, 644456},
  {0x1062F, 644476},
  {0x10630, 644496},
  {0x10631, 644516},
  {0x10632, 644536},
  {0x10633, 644556},
  {0x10634, 644576},
  {0x10635, 644596},
  {0x10636, 644616},
  {0x10637, 644636},
  {0x10638, 644656},
  {0x10639, 644676},
  {0x1063A, 644696},
  {0x1063B, 644716},
  {0x1063C, 644736},
  {0x1063D, 644756},
  {0x1063E, 644776},
  {0x1063F, 644796},
  {0x10640, 644816},
  {0x10641, 644836},
  {0x10642, 644856},
  {0x10643, 644876},
  {0x10644, 644896},
  {0x10645, 644916},
  {0x10646, 644936},
  {0x10647, 638474},
  {0x10648, 644956},
  {0x10649, 644976},
  {0x1064A, 638497},
  {0x1064B, 644996},
  {0x1064C, 645016},
  {0x1064D, 645036},
  {0x1064E, 645057},
  {0x1064F, 638517},
  {0x10650, 645078},
  {0x10651, 645098},
  {0x10652, 645118},
  {0x10653, 645138},
  {0x10654, 645158},
  {0x10655, 638537},
  {0x10656, 638556},
  {0x10657, 638575},
  {0x10658, 638594},
  {0x10659, 638613},
  {0x1065A, 638632},
  {0x1065B, 638651},
  {0x1065C, 638670},
  {0x1065D, 638689},
  {0x1065E, 638709},
  {0x1065F, 638729},
  {0x10660, 638749},
  {0x10661, 638768},
  {0x10662, 638787},
  {0x10663, 638806},
  {0x10664, 638826},
  {0x10665, 638846},
  {0x10666, 638866},
  {0x10667, 638885},
  {0x10668, 638904},
  {0x10669, 638923},
  {0x1066A, 638942},
  {0x1066B, 638961},
  {0x1066C, 638980},
  {0x1066D, 638999},
  {0x1066E, 639018},
  {0x1066F, 639037},
  {0x10670, 639056},
  {0x10671, 639075},
  {0x10672, 639094},
  {0x10673, 639113},
  {0x10674, 639132},
  {0x10675, 639151},
  {0x10676, 639170},
  {0x10677, 639189},
  {0x10678, 639208},
  {0x10679, 639227},
  {0x1067A, 639246},
  {0x1067B, 639265},
  {0x1067C, 639284},
  {0x1067D, 639303},
  {0x1067E, 639322},
  {0x1067F, 639341},
  {0x10680, 639360},
  {0x10681, 639379},
  {0x10682, 639398},
  {0x10683, 639417},
  {0x10684, 639436},
  {0x10685, 639455},
  {0x10686, 639474},
  {0x10687, 639493},
  {0x10688, 639512},
  {0x10689, 639531},
  {0x1068A, 639550},
  {0x1068B, 639569},
  {0x1068C, 639588},
  {0x1068D, 639607},
  {0x1068E, 639626},
  {0x1068F, 639645},
  {0x10690, 639664},
  {0x10691, 639683},
  {0x10692, 639702},
  {0x10693, 639721},
  {0x10694, 639740},
  {0x10695, 639759},
  {0x10696, 639778},
  {0x10697, 639797},
  {0x10698, 639816},
  {0x10699, 639835},
  {0x1069A, 639854},
  {0x1069B, 639873},
  {0x1069C, 639892},
  {0x1069D, 639911},
  {0x1069E, 639930},
  {0x1069F, 639949},
  {0x106A0, 639968},
  {0x106A1, 639991},
  {0x106A2, 640014},
  {0x106A3, 640037},
  {0x106A4, 640060},
  {0x106A5, 640083},
  {0x106A6, 640106},
  {0x106A7, 640129},
  {0x106A8, 640152},
  {0x106A9, 640175},
  {0x106AA, 640198},
  {0x106AB, 640221},
  {0x106AC, 640244},
  {0x106AD, 640267},
  {0x106AE, 640290},
  {0x106AF, 640313},
  {0x106B0, 640336},
  {0x106B1, 640359},
  {0x106B2, 640382},
  {0x106B3, 640405},
  {0x106B4, 640424},
  {0x106B5, 640443},
  {0x106B6, 640462},
  {0x106B7, 640481},
  {0x106B8, 640500},
  {0x106B9, 640519},
  {0x106BA, 640538},
  {0x106BB, 640557},
  {0x106BC, 640576},
  {0x106BD, 640595},
  {0x106BE, 640614},
  {0x106BF, 640633},
  {0x106C0, 640652},
  {0x106C1, 640671},
  {0x106C2, 640690},
  {0x106C3, 640709},
  {0x106C4, 640728},
  {0x106C5, 640747},
  {0x106C6, 640766},
  {0x106C7, 640785},
  {0x106C8, 640804},
  {0x106C9, 640823},
  {0x106CA, 640842},
  {0x106CB, 640861},
  {0x106CC, 640880},
  {0x106CD, 640899},
  {0x106CE, 640918},
  {0x106CF, 640937},
  {0x106D0, 640956},
  {0x106D1, 640975},
  {0x106D2, 640994},
  {0x106D3, 641013},
  {0x106D4, 641032},
  {0x106D5, 641051},
  {0x106D6, 641070},
  {0x106D7, 641089},
  {0x106D8, 641108},
  {0x106D9, 641127},
  {0x106DA, 641146},
  {0x106DB, 641165},
  {0x106DC, 641184},
  {0x106DD, 641203},
  {0x106DE, 641222},
  {0x106DF, 641241},
  {0x106E0, 641260},
  {0x106E1, 641279},
  {0x106E2, 641298},
  {0x106E3, 641317},
  {0x106E4, 641336},
  {0x106E5, 641355},
  {0x106E6, 641374},
  {0x106E7, 641393},
  {0x106E8, 641412},
  {0x106E9, 641431},
  {0x106EA, 641450},
  {0x106EB, 641469},
  {0x106EC, 641488},
  {0x106ED, 641507},
  {0x106EE, 641526},
  {0x106EF, 641545},
  {0x106F0, 641564},
  {0x106F1, 641583},
  {0x106F2, 641602},
  {0x106F3, 641621},
  {0x106F4, 641640},
  {0x106F5, 641659},
  {0x106F6, 641678},
  {0x106F7, 641697},
  {0x106F8, 641716},
  {0x106F9, 641735},
  {0x106FA, 641754},
  {0x106FB, 641773},
  {0x106FC, 641792},
  {0x106FD, 641811},
  {0x106FE, 641830},
  {0x106FF, 641849},
  {0x10700, 641868},
  {0x10701, 641887},
  {0x10702, 641906},
  {0x10703, 641925},
  {0x10704, 641944},
  {0x10705, 641963},
  {0x10706, 641982},
  {0x10707, 642001},
  {0x10708, 642020},
  {0x10709, 642039},
  {0x1070A, 642058},
  {0x1070B, 642077},
  {0x1070C, 642096},
  {0x1070D, 642115},
  {0x1070E, 642134},
  {0x1070F, 642153},
  {0x10710, 642172},
  {0x10711, 642191},
  {0x10712, 642210},
  {0x10713, 642229},
  {0x10714, 642248},
  {0x10715, 642267},
  {0x10716, 642286},
  {0x10717, 642305},
  {0x10718, 642324},
  {0x10719, 642343},
  {0x1071A, 642362},
  {0x1071B, 642381},
  {0x1071C, 642400},
  {0x1071D, 642419},
  {0x1071E, 642438},
  {0x1071F, 642457},
  {0x10720, 642476},
  {0x10721, 642495},
  {0x10722, 642514},
  {0x10723, 642533},
  {0x10724, 642552},
  {0x10725, 642571},
  {0x10726, 642590},
  {0x10727, 642609},
  {0x10728, 642628},
  {0x10729, 642647},
  {0x1072A, 642666},
  {0x1072B, 642685},
  {0x1072C, 642704},
  {0x1072D, 642723},
  {0x1072E, 642742},
  {0x1072F, 642761},
  {0x10730, 642780},
  {0x10731, 642799},
  {0x10732, 642818},
  {0x10733, 642837},
  {0x10734, 642856},
  {0x10735, 642875},
  {0x10736, 642894},
  {0x10740, 642913},
  {0x10741, 642934},
  {0x10742, 642955},
  {0x10743, 642976},
  {0x10744, 642997},
  {0x10745, 643018},
  {0x10746, 643039},
  {0x10747, 643060},
  {0x10748, 643081},
  {0x10749, 643102},
  {0x1074A, 643126},
  {0x1074B, 643150},
  {0x1074C, 643174},
  {0x1074D, 643198},
  {0x1074E, 643219},
  {0x1074F, 643240},
  {0x10750, 643261},
  {0x10751, 643286},
  {0x10752, 643309},
  {0x10753, 643331},
  {0x10754, 643353},
  {0x10755, 643377},
  {0x10760, 643399},
  {0x10761, 643418},
  {0x10762, 643437},
  {0x10763, 643456},
  {0x10764, 643475},
  {0x10765, 643494},
  {0x10766, 643513},
  {0x10767, 643532},
  {0x10780, 732750},
  {0x10781, 737109},
  {0x10782, 737059},
  {0x10783, 732461},
  {0x10784, 732783},
  {0x10785, 732564},
  {0x10787, 733674},
  {0x10788, 733750},
  {0x10789, 733707},
  {0x1078A, 733542},
  {0x1078B, 733480},
  {0x1078C, 733403},
  {0x1078D, 733437},
  {0x1078E, 735372},
  {0x1078F, 733334},
  {0x10790, 733955},
  {0x10791, 735340},
  {0x10792, 732815},
  {0x10793, 734014},
  {0x10794, 732847},
  {0x10795, 734200},
  {0x10796, 732889},
  {0x10797, 734263},
  {0x10798, 733621},
  {0x10799, 734845},
  {0x1079A, 734878},
  {0x1079B, 734477},
  {0x1079C, 733070},
  {0x1079D, 734684},
  {0x1079E, 734737},
  {0x1079F, 734764},
  {0x107A0, 736521},
  {0x107A1, 736552},
  {0x107A2, 735076},
  {0x107A3, 733144},
  {0x107A4, 733299},
  {0x107A5, 735220},
  {0x107A6, 736350},
  {0x107A7, 736395},
  {0x107A8, 735306},
  {0x107A9, 735268},
  {0x107AA, 733177},
  {0x107AB, 735781},
  {0x107AC, 735920},
  {0x107AD, 735953},
  {0x107AE, 735824},
  {0x107AF, 735737},
  {0x107B0, 736772},
  {0x107B2, 733241},
  {0x107B3, 730507},
  {0x107B4, 732212},
  {0x107B5, 727109},
  {0x107B6, 729795},
  {0x107B7, 730769},
  {0x107B8, 726988},
  {0x107B9, 732091},
  {0x107BA, 735511},
  {0x10800, 251810},
  {0x10801, 251829},
  {0x10802, 251848},
  {0x10803, 252307},
  {0x10804, 252726},
  {0x10805, 251867},
  {0x10808, 251887},
  {0x1080A, 251907},
  {0x1080B, 251927},
  {0x1080C, 251947},
  {0x1080D, 251967},
  {0x1080E, 251987},
  {0x1080F, 252007},
  {0x10810, 252027},
  {0x10811, 252047},
  {0x10812, 252067},
  {0x10813, 252087},
  {0x10814, 252107},
  {0x10815, 252127},
  {0x10816, 252147},
  {0x10817, 252167},
  {0x10818, 252187},
  {0x10819, 252207},
  {0x1081A, 252227},
  {0x1081B, 252247},
  {0x1081C, 252267},
  {0x1081D, 252287},
  {0x1081E, 252326},
  {0x1081F, 252346},
  {0x10820, 252366},
  {0x10821, 252386},
  {0x10822, 252406},
  {0x10823, 252426},
  {0x10824, 252446},
  {0x10825, 252466},
  {0x10826, 252486},
  {0x10827, 252506},
  {0x10828, 252526},
  {0x10829, 252546},
  {0x1082A, 252566},
  {0x1082B, 252586},
  {0x1082C, 252606},
  {0x1082D, 252626},
  {0x1082E, 252646},
  {0x1082F, 252666},
  {0x10830, 252686},
  {0x10831, 252706},
  {0x10832, 252745},
  {0x10833, 252765},
  {0x10834, 252785},
  {0x10835, 252805},
  {0x10837, 252825},
  {0x10838, 252845},
  {0x1083C, 252865},
  {0x1083F, 252885},
  {0x10840, 529633},
  {0x10841, 529692},
  {0x10842, 529752},
  {0x10843, 529721},
  {0x10844, 529782},
  {0x10845, 530186},
  {0x10846, 530243},
  {0x10847, 529809},
  {0x10848, 530157},
  {0x10849, 530214},
  {0x1084A, 529838},
  {0x1084B, 529867},
  {0x1084C, 529898},
  {0x1084D, 529926},
  {0x1084E, 530069},
  {0x1084F, 529663},
  {0x10850, 529954},
  {0x10851, 530039},
  {0x10852, 529981},
  {0x10853, 530010},
  {0x10854, 530100},
  {0x10855, 530129},
  {0x10857, 530528},
  {0x10858, 530273},
  {0x10859, 530500},
  {0x1085A, 530439},
  {0x1085B, 530374},
  {0x1085C, 530469},
  {0x1085D, 530301},
  {0x1085E, 530337},
  {0x1085F, 530402},
  {0x10860, 812159},
  {0x10861, 812204},
  {0x10862, 812277},
  {0x10863, 812226},
  {0x10864, 812300},
  {0x10865, 812606},
  {0x10866, 812649},
  {0x10867, 812320},
  {0x10868, 812584},
  {0x10869, 812627},
  {0x1086A, 812342},
  {0x1086B, 812364},
  {0x1086C, 812388},
  {0x1086D, 812250},
  {0x1086E, 812409},
  {0x1086F, 812517},
  {0x10870, 812182},
  {0x10871, 812430},
  {0x10872, 812494},
  {0x10873, 812450},
  {0x10874, 812472},
  {0x10875, 812541},
  {0x10876, 812563},
  {0x10877, 812127},
  {0x10878, 812826},
  {0x10879, 812716},
  {0x1087A, 812805},
  {0x1087B, 812758},
  {0x1087C, 812694},
  {0x1087D, 812672},
  {0x1087E, 812737},
  {0x1087F, 812781},
  {0x10880, 758612},
  {0x10881, 758521},
  {0x10882, 758641},
  {0x10883, 758566},
  {0x10884, 758863},
  {0x10885, 758588},
  {0x10886, 758669},
  {0x10887, 758886},
  {0x10888, 759192},
  {0x10889, 759235},
  {0x1088A, 758906},
  {0x1088B, 759170},
  {0x1088C, 758835},
  {0x1088D, 759213},
  {0x1088E, 758695},
  {0x1088F, 758928},
  {0x10890, 758723},
  {0x10891, 758950},
  {0x10892, 758753},
  {0x10893, 758974},
  {0x10894, 758780},
  {0x10895, 758995},
  {0x10896, 759103},
  {0x10897, 758544},
  {0x10898, 759016},
  {0x10899, 759080},
  {0x1089A, 759036},
  {0x1089B, 759058},
  {0x1089C, 758807},
  {0x1089D, 759127},
  {0x1089E, 759149},
  {0x108A7, 759302},
  {0x108A8, 759420},
  {0x108A9, 759373},
  {0x108AA, 759280},
  {0x108AB, 758489},
  {0x108AC, 759258},
  {0x108AD, 759352},
  {0x108AE, 759396},
  {0x108AF, 759323},
  {0x108E0, 505440},
  {0x108E1, 505478},
  {0x108E2, 505523},
  {0x108E3, 505497},
  {0x108E4, 505543},
  {0x108E5, 505788},
  {0x108E6, 505825},
  {0x108E7, 505560},
  {0x108E8, 505769},
  {0x108E9, 505806},
  {0x108EA, 505579},
  {0x108EB, 505598},
  {0x108EC, 505619},
  {0x108ED, 505637},
  {0x108EE, 505711},
  {0x108EF, 505460},
  {0x108F0, 505655},
  {0x108F1, 505691},
  {0x108F2, 505672},
  {0x108F4, 505732},
  {0x108F5, 505751},
  {0x108FB, 505863},
  {0x108FC, 505844},
  {0x108FD, 505907},
  {0x108FE, 505925},
  {0x108FF, 505881},
  {0x10900, 822301},
  {0x10901, 822323},
  {0x10902, 822368},
  {0x10903, 822345},
  {0x10904, 822391},
  {0x10905, 822702},
  {0x10906, 822746},
  {0x10907, 822412},
  {0x10908, 822680},
  {0x10909, 822724},
  {0x1090A, 822434},
  {0x1090B, 822456},
  {0x1090C, 822479},
  {0x1090D, 822501},
  {0x1090E, 822612},
  {0x1090F, 822279},
  {0x10910, 822523},
  {0x10911, 822589},
  {0x10912, 822544},
  {0x10913, 822566},
  {0x10914, 822635},
  {0x10915, 822658},
  {0x10916, 822768},
  {0x10917, 822820},
  {0x10918, 822866},
  {0x10919, 822790},
  {0x1091A, 822891},
  {0x1091B, 822842},
  {0x1091F, 822913},
  {0x10920, 657340},
  {0x10921, 657373},
  {0x10922, 657470},
  {0x10923, 657405},
  {0x10924, 657421},
  {0x10925, 657730},
  {0x10926, 657486},
  {0x10927, 657746},
  {0x10928, 657502},
  {0x10929, 657518},
  {0x1092A, 657551},
  {0x1092B, 657567},
  {0x1092C, 657600},
  {0x1092D, 657632},
  {0x1092E, 657664},
  {0x1092F, 657681},
  {0x10930, 657714},
  {0x10931, 657454},
  {0x10932, 657616},
  {0x10933, 657648},
  {0x10934, 657697},
  {0x10935, 657356},
  {0x10936, 657437},
  {0x10937, 657534},
  {0x10938, 657583},
  {0x10939, 657389},
  {0x1093F, 657762},
  {0x10980, 720953},
  {0x10981, 721082},
  {0x10982, 721146},
  {0x10983, 721438},
  {0x10984, 721893},
  {0x10985, 721861},
  {0x10986, 720984},
  {0x10987, 721016},
  {0x10988, 721469},
  {0x10989, 721274},
  {0x1098A, 721306},
  {0x1098B, 721338},
  {0x1098C, 721372},
  {0x1098D, 721404},
  {0x1098E, 721533},
  {0x1098F, 721565},
  {0x10990, 721242},
  {0x10991, 721209},
  {0x10992, 721113},
  {0x10993, 721599},
  {0x10994, 721631},
  {0x10995, 721665},
  {0x10996, 721177},
  {0x10997, 721501},
  {0x10998, 721697},
  {0x10999, 721729},
  {0x1099A, 721763},
  {0x1099B, 721795},
  {0x1099C, 721829},
  {0x1099D, 721050},
  {0x1099E, 721925},
  {0x1099F, 721959},
  {0x109A0, 718322},
  {0x109A1, 718437},
  {0x109A2, 718491},
  {0x109A3, 718680},
  {0x109A4, 718949},
  {0x109A5, 718922},
  {0x109A6, 718383},
  {0x109A7, 718706},
  {0x109A8, 718599},
  {0x109A9, 718626},
  {0x109AA, 718653},
  {0x109AB, 718760},
  {0x109AC, 718572},
  {0x109AD, 718544},
  {0x109AE, 718463},
  {0x109AF, 718787},
  {0x109B0, 718348},
  {0x109B1, 718814},
  {0x109B2, 718517},
  {0x109B3, 718733},
  {0x109B4, 718841},
  {0x109B5, 718868},
  {0x109B6, 718895},
  {0x109B7, 718410},
  {0x109BC, 717888},
  {0x109BD, 718050},
  {0x109BE, 719006},
  {0x109BF, 718976},
  {0x109C0, 719858},
  {0x109C1, 720807},
  {0x109C2, 720582},
  {0x109C3, 719518},
  {0x109C4, 719299},
  {0x109C5, 720231},
  {0x109C6, 720004},
  {0x109C7, 719036},
  {0x109C8, 719668},
  {0x109C9, 720446},
  {0x109CA, 720736},
  {0x109CB, 720511},
  {0x109CC, 719449},
  {0x109CD, 719230},
  {0x109CE, 720377},
  {0x109CF, 720158},
  {0x109D2, 719886},
  {0x109D3, 720835},
  {0x109D4, 720612},
  {0x109D5, 719547},
  {0x109D6, 719328},
  {0x109D7, 720259},
  {0x109D8, 720034},
  {0x109D9, 719066},
  {0x109DA, 719697},
  {0x109DB, 719967},
  {0x109DC, 720916},
  {0x109DD, 720697},
  {0x109DE, 719630},
  {0x109DF, 719411},
  {0x109E0, 720340},
  {0x109E1, 720119},
  {0x109E2, 719151},
  {0x109E3, 719780},
  {0x109E4, 720474},
  {0x109E5, 720767},
  {0x109E6, 720542},
  {0x109E7, 719479},
  {0x109E8, 719260},
  {0x109E9, 720407},
  {0x109EA, 720190},
  {0x109EB, 719190},
  {0x109EC, 719818},
  {0x109ED, 719922},
  {0x109EE, 720871},
  {0x109EF, 720650},
  {0x109F0, 719584},
  {0x109F1, 719365},
  {0x109F2, 720295},
  {0x109F3, 720072},
  {0x109F4, 719104},
  {0x109F5, 719734},
  {0x109F6, 718085},
  {0x109F7, 718283},
  {0x109F8, 718242},
  {0x109F9, 717970},
  {0x109FA, 717930},
  {0x109FB, 718164},
  {0x109FC, 718123},
  {0x109FD, 717847},
  {0x109FE, 718010},
  {0x109FF, 718203},
  {0x10A00, 552538},
  {0x10A01, 553958},
  {0x10A02, 554006},
  {0x10A03, 554030},
  {0x10A05, 553934},
  {0x10A06, 553982},
  {0x10A0C, 553905},
  {0x10A0D, 553829},
  {0x10A0E, 553730},
  {0x10A0F, 553863},
  {0x10A10, 552817},
  {0x10A11, 552838},
  {0x10A12, 552732},
  {0x10A13, 552753},
  {0x10A15, 552601},
  {0x10A16, 552622},
  {0x10A17, 552796},
  {0x10A19, 552967},
  {0x10A1A, 553161},
  {0x10A1B, 553183},
  {0x10A1C, 552665},
  {0x10A1D, 552687},
  {0x10A1E, 552945},
  {0x10A1F, 553118},
  {0x10A20, 553139},
  {0x10A21, 552644},
  {0x10A22, 552710},
  {0x10A23, 552924},
  {0x10A24, 552989},
  {0x10A25, 553010},
  {0x10A26, 552558},
  {0x10A27, 552579},
  {0x10A28, 552903},
  {0x10A29, 553296},
  {0x10A2A, 553032},
  {0x10A2B, 552882},
  {0x10A2C, 553253},
  {0x10A2D, 553074},
  {0x10A2E, 553096},
  {0x10A2F, 553053},
  {0x10A30, 553317},
  {0x10A31, 552775},
  {0x10A32, 552860},
  {0x10A33, 553229},
  {0x10A34, 553206},
  {0x10A35, 553274},
  {0x10A38, 553755},
  {0x10A39, 553781},
  {0x10A3A, 553803},
  {0x10A3F, 553887},
  {0x10A40, 552444},
  {0x10A41, 552488},
  {0x10A42, 552465},
  {0x10A43, 552422},
  {0x10A44, 553399},
  {0x10A45, 553421},
  {0x10A46, 553338},
  {0x10A47, 553368},
  {0x10A48, 552509},
  {0x10A50, 553541},
  {0x10A51, 553694},
  {0x10A52, 553446},
  {0x10A53, 553476},
  {0x10A54, 553662},
  {0x10A55, 553633},
  {0x10A56, 553512},
  {0x10A57, 553568},
  {0x10A58, 553604},
  {0x10A60, 796811},
  {0x10A61, 796930},
  {0x10A62, 796839},
  {0x10A63, 796962},
  {0x10A64, 797020},
  {0x10A65, 797322},
  {0x10A66, 797172},
  {0x10A67, 797050},
  {0x10A68, 796594},
  {0x10A69, 797202},
  {0x10A6A, 797143},
  {0x10A6B, 796869},
  {0x10A6C, 796991},
  {0x10A6D, 796899},
  {0x10A6E, 797080},
  {0x10A6F, 797111},
  {0x10A70, 796721},
  {0x10A71, 796535},
  {0x10A72, 796565},
  {0x10A73, 796656},
  {0x10A74, 796780},
  {0x10A75, 796624},
  {0x10A76, 796749},
  {0x10A77, 797231},
  {0x10A78, 797381},
  {0x10A79, 796688},
  {0x10A7A, 797351},
  {0x10A7B, 797261},
  {0x10A7C, 797291},
  {0x10A7D, 797442},
  {0x10A7E, 797411},
  {0x10A7F, 797471},
  {0x10A80, 792819},
  {0x10A81, 792907},
  {0x10A82, 792790},
  {0x10A83, 792936},
  {0x10A84, 792996},
  {0x10A85, 793201},
  {0x10A86, 792640},
  {0x10A87, 793025},
  {0x10A88, 792523},
  {0x10A89, 793112},
  {0x10A8A, 792610},
  {0x10A8B, 792848},
  {0x10A8C, 792966},
  {0x10A8D, 792877},
  {0x10A8E, 793054},
  {0x10A8F, 792670},
  {0x10A90, 792700},
  {0x10A91, 792493},
  {0x10A92, 792464},
  {0x10A93, 792552},
  {0x10A94, 792729},
  {0x10A95, 792581},
  {0x10A96, 792759},
  {0x10A97, 793083},
  {0x10A98, 793288},
  {0x10A99, 793141},
  {0x10A9A, 793230},
  {0x10A9B, 793171},
  {0x10A9C, 793259},
  {0x10A9D, 793318},
  {0x10A9E, 793347},
  {0x10A9F, 793376},
  {0x10AC0, 664030},
  {0x10AC1, 664077},
  {0x10AC2, 664100},
  {0x10AC3, 664221},
  {0x10AC4, 664196},
  {0x10AC5, 664124},
  {0x10AC6, 664245},
  {0x10AC7, 664712},
  {0x10AC8, 665210},
  {0x10AC9, 664803},
  {0x10ACA, 664827},
  {0x10ACB, 664289},
  {0x10ACC, 664313},
  {0x10ACD, 664266},
  {0x10ACE, 664663},
  {0x10ACF, 664780},
  {0x10AD0, 664338},
  {0x10AD1, 664734},
  {0x10AD2, 664361},
  {0x10AD3, 664385},
  {0x10AD4, 664149},
  {0x10AD5, 664686},
  {0x10AD6, 664410},
  {0x10AD7, 664432},
  {0x10AD8, 664569},
  {0x10AD9, 664054},
  {0x10ADA, 664006},
  {0x10ADB, 664454},
  {0x10ADC, 664175},
  {0x10ADD, 664545},
  {0x10ADE, 664499},
  {0x10ADF, 664757},
  {0x10AE0, 664475},
  {0x10AE1, 664522},
  {0x10AE2, 664594},
  {0x10AE3, 664617},
  {0x10AE4, 664641},
  {0x10AE5, 663936},
  {0x10AE6, 663971},
  {0x10AEB, 664875},
  {0x10AEC, 664852},
  {0x10AED, 664927},
  {0x10AEE, 664949},
  {0x10AEF, 664897},
  {0x10AF0, 665150},
  {0x10AF1, 665084},
  {0x10AF2, 665039},
  {0x10AF3, 665001},
  {0x10AF4, 664974},
  {0x10AF5, 665178},
  {0x10AF6, 665115},
  {0x10B00, 82249},
  {0x10B01, 82266},
  {0x10B02, 82377},
  {0x10B03, 82303},
  {0x10B04, 82359},
  {0x10B05, 82284},
  {0x10B06, 82322},
  {0x10B07, 82340},
  {0x10B08, 82487},
  {0x10B09, 82504},
  {0x10B0A, 82855},
  {0x10B0B, 82872},
  {0x10B0C, 82633},
  {0x10B0D, 82650},
  {0x10B0E, 83059},
  {0x10B0F, 83076},
  {0x10B10, 82686},
  {0x10B11, 83112},
  {0x10B12, 83149},
  {0x10B13, 83130},
  {0x10B14, 82540},
  {0x10B15, 82558},
  {0x10B16, 82577},
  {0x10B17, 82432},
  {0x10B18, 82668},
  {0x10B19, 83003},
  {0x10B1A, 83021},
  {0x10B1B, 82450},
  {0x10B1C, 82468},
  {0x10B1D, 83040},
  {0x10B1E, 82890},
  {0x10B1F, 82522},
  {0x10B20, 82395},
  {0x10B21, 82413},
  {0x10B22, 82758},
  {0x10B23, 82797},
  {0x10B24, 82777},
  {0x10B25, 82740},
  {0x10B26, 82836},
  {0x10B27, 82817},
  {0x10B28, 82722},
  {0x10B29, 82614},
  {0x10B2A, 83186},
  {0x10B2B, 83168},
  {0x10B2C, 83094},
  {0x10B2D, 82908},
  {0x10B2E, 82704},
  {0x10B2F, 82926},
  {0x10B30, 83205},
  {0x10B31, 82944},
  {0x10B32, 83223},
  {0x10B33, 82963},
  {0x10B34, 82983},
  {0x10B35, 82596},
  {0x10B39, 82223},
  {0x10B3A, 945342},
  {0x10B3B, 893324},
  {0x10B3C, 580283},
  {0x10B3D, 580031},
  {0x10B3E, 580323},
  {0x10B3F, 580071},
  {0x10B40, 533999},
  {0x10B41, 534070},
  {0x10B42, 534142},
  {0x10B43, 534105},
  {0x10B44, 534178},
  {0x10B45, 534666},
  {0x10B46, 534735},
  {0x10B47, 534211},
  {0x10B48, 534631},
  {0x10B49, 534700},
  {0x10B4A, 534246},
  {0x10B4B, 534281},
  {0x10B4C, 534318},
  {0x10B4D, 534352},
  {0x10B4E, 534525},
  {0x10B4F, 534035},
  {0x10B50, 534386},
  {0x10B51, 534489},
  {0x10B52, 534419},
  {0x10B53, 534454},
  {0x10B54, 534562},
  {0x10B55, 534597},
  {0x10B58, 534806},
  {0x10B59, 535032},
  {0x10B5A, 534959},
  {0x10B5B, 534771},
  {0x10B5C, 534925},
  {0x10B5D, 534995},
  {0x10B5E, 534840},
  {0x10B5F, 534882},
  {0x10B60, 533049},
  {0x10B61, 533084},
  {0x10B62, 533154},
  {0x10B63, 533118},
  {0x10B64, 533189},
  {0x10B65, 533600},
  {0x10B66, 533677},
  {0x10B67, 533221},
  {0x10B68, 533566},
  {0x10B69, 533643},
  {0x10B6A, 533255},
  {0x10B6B, 533289},
  {0x10B6C, 533325},
  {0x10B6D, 533363},
  {0x10B6E, 533463},
  {0x10B6F, 533396},
  {0x10B70, 533428},
  {0x10B71, 533499},
  {0x10B72, 533533},
  {0x10B78, 533746},
  {0x10B79, 533966},
  {0x10B7A, 533895},
  {0x10B7B, 533712},
  {0x10B7C, 533862},
  {0x10B7D, 533930},
  {0x10B7E, 533779},
  {0x10B7F, 533820},
  {0x10B80, 828588},
  {0x10B81, 828617},
  {0x10B82, 828675},
  {0x10B83, 828645},
  {0x10B84, 828704},
  {0x10B85, 829015},
  {0x10B86, 829080},
  {0x10B87, 828730},
  {0x10B88, 829052},
  {0x10B89, 828758},
  {0x10B8A, 828786},
  {0x10B8B, 828816},
  {0x10B8C, 828848},
  {0x10B8D, 828930},
  {0x10B8E, 828875},
  {0x10B8F, 828901},
  {0x10B90, 828960},
  {0x10B91, 828988},
  {0x10B99, 829312},
  {0x10B9A, 829341},
  {0x10B9B, 828516},
  {0x10B9C, 828553},
  {0x10BA9, 829137},
  {0x10BAA, 829285},
  {0x10BAB, 829226},
  {0x10BAC, 829109},
  {0x10BAD, 829199},
  {0x10BAE, 829255},
  {0x10BAF, 829164},
  {0x10C00, 797507},
  {0x10C01, 798700},
  {0x10C02, 798786},
  {0x10C03, 798449},
  {0x10C04, 799466},
  {0x10C05, 799260},
  {0x10C06, 798532},
  {0x10C07, 798559},
  {0x10C08, 799523},
  {0x10C09, 797534},
  {0x10C0A, 798728},
  {0x10C0B, 797590},
  {0x10C0C, 798815},
  {0x10C0D, 797880},
  {0x10C0E, 799026},
  {0x10C0F, 797648},
  {0x10C10, 798845},
  {0x10C11, 797562},
  {0x10C12, 798757},
  {0x10C13, 797619},
  {0x10C14, 798421},
  {0x10C15, 799437},
  {0x10C16, 798105},
  {0x10C17, 799231},
  {0x10C18, 797851},
  {0x10C19, 798996},
  {0x10C1A, 797677},
  {0x10C1B, 798875},
  {0x10C1C, 798587},
  {0x10C1D, 799552},
  {0x10C1E, 797908},
  {0x10C1F, 799055},
  {0x10C20, 797706},
  {0x10C21, 798191},
  {0x10C22, 798220},
  {0x10C23, 797936},
  {0x10C24, 797735},
  {0x10C25, 798905},
  {0x10C26, 798306},
  {0x10C27, 799347},
  {0x10C28, 798248},
  {0x10C29, 799317},
  {0x10C2A, 798335},
  {0x10C2B, 799377},
  {0x10C2C, 799084},
  {0x10C2D, 798277},
  {0x10C2E, 798935},
  {0x10C2F, 798364},
  {0x10C30, 798616},
  {0x10C31, 798476},
  {0x10C32, 798163},
  {0x10C33, 799288},
  {0x10C34, 797964},
  {0x10C35, 799114},
  {0x10C36, 798504},
  {0x10C37, 799494},
  {0x10C38, 798644},
  {0x10C39, 799582},
  {0x10C3A, 797992},
  {0x10C3B, 799143},
  {0x10C3C, 797764},
  {0x10C3D, 798020},
  {0x10C3E, 797793},
  {0x10C3F, 798048},
  {0x10C40, 799172},
  {0x10C41, 798392},
  {0x10C42, 799407},
  {0x10C43, 798077},
  {0x10C44, 799202},
  {0x10C45, 797822},
  {0x10C46, 798966},
  {0x10C47, 798672},
  {0x10C48, 798133},
  {0x10C80, 788118},
  {0x10C81, 788149},
  {0x10C82, 788347},
  {0x10C83, 788213},
  {0x10C84, 788379},
  {0x10C85, 788896},
  {0x10C86, 788444},
  {0x10C87, 788477},
  {0x10C88, 788246},
  {0x10C89, 788316},
  {0x10C8A, 788279},
  {0x10C8B, 788509},
  {0x10C8C, 788541},
  {0x10C8D, 788573},
  {0x10C8E, 788605},
  {0x10C8F, 788638},
  {0x10C90, 789331},
  {0x10C91, 789362},
  {0x10C92, 788670},
  {0x10C93, 788702},
  {0x10C94, 788181},
  {0x10C95, 789729},
  {0x10C96, 788734},
  {0x10C97, 788766},
  {0x10C98, 788799},
  {0x10C99, 788864},
  {0x10C9A, 789007},
  {0x10C9B, 789480},
  {0x10C9C, 789544},
  {0x10C9D, 789394},
  {0x10C9E, 789576},
  {0x10C9F, 789511},
  {0x10CA0, 789040},
  {0x10CA1, 788831},
  {0x10CA2, 789072},
  {0x10CA3, 789660},
  {0x10CA4, 789104},
  {0x10CA5, 789136},
  {0x10CA6, 789169},
  {0x10CA7, 788929},
  {0x10CA8, 789201},
  {0x10CA9, 788411},
  {0x10CAA, 789698},
  {0x10CAB, 789794},
  {0x10CAC, 789437},
  {0x10CAD, 789618},
  {0x10CAE, 789234},
  {0x10CAF, 789266},
  {0x10CB0, 789298},
  {0x10CB1, 788962},
  {0x10CB2, 789762},
  {0x10CC0, 789996},
  {0x10CC1, 790025},
  {0x10CC2, 790211},
  {0x10CC3, 790085},
  {0x10CC4, 790241},
  {0x10CC5, 790726},
  {0x10CC6, 790302},
  {0x10CC7, 790333},
  {0x10CC8, 790116},
  {0x10CC9, 790182},
  {0x10CCA, 790147},
  {0x10CCB, 790363},
  {0x10CCC, 790393},
  {0x10CCD, 790423},
  {0x10CCE, 790453},
  {0x10CCF, 790484},
  {0x10CD0, 791135},
  {0x10CD1, 791164},
  {0x10CD2, 790514},
  {0x10CD3, 790544},
  {0x10CD4, 790055},
  {0x10CD5, 791511},
  {0x10CD6, 790574},
  {0x10CD7, 790604},
  {0x10CD8, 790635},
  {0x10CD9, 790696},
  {0x10CDA, 790831},
  {0x10CDB, 791276},
  {0x10CDC, 791336},
  {0x10CDD, 791194},
  {0x10CDE, 791366},
  {0x10CDF, 791305},
  {0x10CE0, 790862},
  {0x10CE1, 790665},
  {0x10CE2, 790892},
  {0x10CE3, 791446},
  {0x10CE4, 790922},
  {0x10CE5, 790952},
  {0x10CE6, 790983},
  {0x10CE7, 790757},
  {0x10CE8, 791013},
  {0x10CE9, 790271},
  {0x10CEA, 791482},
  {0x10CEB, 791572},
  {0x10CEC, 791235},
  {0x10CED, 791406},
  {0x10CEE, 791044},
  {0x10CEF, 791074},
  {0x10CF0, 791104},
  {0x10CF1, 790788},
  {0x10CF2, 791542},
  {0x10CFA, 789879},
  {0x10CFB, 789853},
  {0x10CFC, 789971},
  {0x10CFD, 789826},
  {0x10CFE, 789904},
  {0x10CFF, 789937},
  {0x10D00, 503933},
  {0x10D01, 503958},
  {0x10D02, 504416},
  {0x10D03, 504548},
  {0x10D04, 504574},
  {0x10D05, 504141},
  {0x10D06, 503984},
  {0x10D07, 504115},
  {0x10D08, 504193},
  {0x10D09, 504063},
  {0x10D0A, 504010},
  {0x10D0B, 504036},
  {0x10D0C, 504442},
  {0x10D0D, 504468},
  {0x10D0E, 504679},
  {0x10D0F, 504495},
  {0x10D10, 504521},
  {0x10D11, 504167},
  {0x10D12, 504089},
  {0x10D13, 504284},
  {0x10D14, 504310},
  {0x10D15, 504336},
  {0x10D16, 504627},
  {0x10D17, 504220},
  {0x10D18, 504653},
  {0x10D19, 504252},
  {0x10D1A, 504362},
  {0x10D1B, 504389},
  {0x10D1C, 504601},
  {0x10D1D, 504874},
  {0x10D1E, 504922},
  {0x10D1F, 504970},
  {0x10D20, 504898},
  {0x10D21, 504946},
  {0x10D22, 504736},
  {0x10D23, 504705},
  {0x10D24, 504763},
  {0x10D25, 504793},
  {0x10D26, 504821},
  {0x10D27, 504847},
  {0x10D30, 503906},
  {0x10D31, 503772},
  {0x10D32, 503880},
  {0x10D33, 503852},
  {0x10D34, 503718},
  {0x10D35, 503691},
  {0x10D36, 503826},
  {0x10D37, 503798},
  {0x10D38, 503663},
  {0x10D39, 503745},
  {0x10D40, 448927},
  {0x10D41, 448843},
  {0x10D42, 448911},
  {0x10D43, 448893},
  {0x10D44, 448809},
  {0x10D45, 448792},
  {0x10D46, 448877},
  {0x10D47, 448859},
  {0x10D48, 448774},
  {0x10D49, 448826},
  {0x10D4A, 449543},
  {0x10D4B, 449601},
  {0x10D4C, 449620},
  {0x10D4D, 449581},
  {0x10D4E, 449519},
  {0x10D4F, 449507},
  {0x10D50, 448113},
  {0x10D51, 448160},
  {0x10D52, 448352},
  {0x10D53, 448304},
  {0x10D54, 448136},
  {0x10D55, 448280},
  {0x10D56, 448529},
  {0x10D57, 448577},
  {0x10D58, 448328},
  {0x10D59, 448232},
  {0x10D5A, 448184},
  {0x10D5B, 448601},
  {0x10D5C, 448625},
  {0x10D5D, 448553},
  {0x10D5E, 448505},
  {0x10D5F, 448400},
  {0x10D60, 448208},
  {0x10D61, 448376},
  {0x10D62, 448481},
  {0x10D63, 448256},
  {0x10D64, 448425},
  {0x10D65, 448453},
  {0x10D69, 449562},
  {0x10D6A, 448708},
  {0x10D6B, 448649},
  {0x10D6C, 448675},
  {0x10D6D, 448740},
  {0x10D6E, 448944},
  {0x10D6F, 448990},
  {0x10D70, 449015},
  {0x10D71, 449058},
  {0x10D72, 449234},
  {0x10D73, 449190},
  {0x10D74, 449036},
  {0x10D75, 449168},
  {0x10D76, 449397},
  {0x10D77, 449441},
  {0x10D78, 449212},
  {0x10D79, 449124},
  {0x10D7A, 449080},
  {0x10D7B, 449463},
  {0x10D7C, 449485},
  {0x10D7D, 449419},
  {0x10D7E, 449375},
  {0x10D7F, 449278},
  {0x10D80, 449102},
  {0x10D81, 449256},
  {0x10D82, 449353},
  {0x10D83, 449146},
  {0x10D84, 449301},
  {0x10D85, 449327},
  {0x10D8E, 448974},
  {0x10D8F, 448957},
  {0x10E60, 843332},
  {0x10E61, 843396},
  {0x10E62, 843379},
  {0x10E63, 843300},
  {0x10E64, 843284},
  {0x10E65, 843364},
  {0x10E66, 843347},
  {0x10E67, 843267},
  {0x10E68, 843316},
  {0x10E69, 843796},
  {0x10E6A, 843857},
  {0x10E6B, 843812},
  {0x10E6C, 843597},
  {0x10E6D, 843554},
  {0x10E6E, 843778},
  {0x10E6F, 843734},
  {0x10E70, 843535},
  {0x10E71, 843665},
  {0x10E72, 843684},
  {0x10E73, 843876},
  {0x10E74, 843831},
  {0x10E75, 843615},
  {0x10E76, 843572},
  {0x10E77, 843754},
  {0x10E78, 843708},
  {0x10E79, 843509},
  {0x10E7A, 843640},
  {0x10E7B, 843411},
  {0x10E7C, 843434},
  {0x10E7D, 843460},
  {0x10E7E, 843484},
  {0x10E80, 989488},
  {0x10E81, 989378},
  {0x10E82, 989788},
  {0x10E83, 989805},
  {0x10E84, 989965},
  {0x10E85, 989894},
  {0x10E86, 989434},
  {0x10E87, 989415},
  {0x10E88, 989395},
  {0x10E89, 989612},
  {0x10E8A, 990067},
  {0x10E8B, 989470},
  {0x10E8C, 990179},
  {0x10E8D, 989841},
  {0x10E8E, 989858},
  {0x10E8F, 990162},
  {0x10E90, 989630},
  {0x10E91, 989930},
  {0x10E92, 989911},
  {0x10E93, 989876},
  {0x10E94, 989452},
  {0x10E95, 989948},
  {0x10E96, 990197},
  {0x10E97, 989541},
  {0x10E98, 990084},
  {0x10E99, 989559},
  {0x10E9A, 990000},
  {0x10E9B, 990017},
  {0x10E9C, 989823},
  {0x10E9D, 989647},
  {0x10E9E, 989665},
  {0x10E9F, 989576},
  {0x10EA0, 989684},
  {0x10EA1, 989735},
  {0x10EA2, 989753},
  {0x10EA3, 989983},
  {0x10EA4, 990049},
  {0x10EA5, 989771},
  {0x10EA6, 989524},
  {0x10EA7, 989594},
  {0x10EA8, 990104},
  {0x10EA9, 989507},
  {0x10EAB, 989298},
  {0x10EAC, 989326},
  {0x10EAD, 989354},
  {0x10EB0, 989702},
  {0x10EB1, 990122},
  {0x10EC2, 33123},
  {0x10EC3, 43033},
  {0x10EC4, 37576},
  {0x10EFC, 27947},
  {0x10EFD, 77346},
  {0x10EFE, 77319},
  {0x10EFF, 77291},
  {0x10F00, 795451},
  {0x10F01, 795558},
  {0x10F02, 795534},
  {0x10F03, 795589},
  {0x10F04, 795882},
  {0x10F05, 795907},
  {0x10F06, 795619},
  {0x10F07, 796205},
  {0x10F08, 796252},
  {0x10F09, 795929},
  {0x10F0A, 796228},
  {0x10F0B, 795953},
  {0x10F0C, 795977},
  {0x10F0D, 796003},
  {0x10F0E, 796026},
  {0x10F0F, 795647},
  {0x10F10, 795676},
  {0x10F11, 796132},
  {0x10F12, 795510},
  {0x10F13, 795476},
  {0x10F14, 796049},
  {0x10F15, 796107},
  {0x10F16, 795724},
  {0x10F17, 795755},
  {0x10F18, 796071},
  {0x10F19, 796158},
  {0x10F1A, 796182},
  {0x10F1B, 795805},
  {0x10F1C, 795834},
  {0x10F1D, 796358},
  {0x10F1E, 796512},
  {0x10F1F, 796461},
  {0x10F20, 796334},
  {0x10F21, 796310},
  {0x10F22, 796412},
  {0x10F23, 796486},
  {0x10F24, 796435},
  {0x10F25, 796381},
  {0x10F26, 795421},
  {0x10F27, 796277},
  {0x10F30, 894575},
  {0x10F31, 894616},
  {0x10F32, 894656},
  {0x10F33, 894677},
  {0x10F34, 894940},
  {0x10F35, 894979},
  {0x10F36, 894695},
  {0x10F37, 894959},
  {0x10F38, 894715},
  {0x10F39, 894735},
  {0x10F3A, 894777},
  {0x10F3B, 894796},
  {0x10F3C, 894879},
  {0x10F3D, 894596},
  {0x10F3E, 894815},
  {0x10F3F, 894858},
  {0x10F40, 894833},
  {0x10F41, 894901},
  {0x10F42, 894921},
  {0x10F43, 894636},
  {0x10F44, 894757},
  {0x10F45, 894550},
  {0x10F46, 894304},
  {0x10F47, 894517},
  {0x10F48, 894276},
  {0x10F49, 894484},
  {0x10F4A, 894216},
  {0x10F4B, 894246},
  {0x10F4C, 894332},
  {0x10F4D, 894361},
  {0x10F4E, 894390},
  {0x10F4F, 894424},
  {0x10F50, 894453},
  {0x10F51, 895000},
  {0x10F52, 895046},
  {0x10F53, 895065},
  {0x10F54, 895019},
  {0x10F55, 895206},
  {0x10F56, 895244},
  {0x10F57, 895087},
  {0x10F58, 895164},
  {0x10F59, 895123},
  {0x10F70, 799745},
  {0x10F71, 799769},
  {0x10F72, 799821},
  {0x10F73, 800103},
  {0x10F74, 800148},
  {0x10F75, 799792},
  {0x10F76, 800125},
  {0x10F77, 799850},
  {0x10F78, 799873},
  {0x10F79, 799921},
  {0x10F7A, 799943},
  {0x10F7B, 800033},
  {0x10F7C, 799965},
  {0x10F7D, 800009},
  {0x10F7E, 799986},
  {0x10F7F, 800058},
  {0x10F80, 800081},
  {0x10F81, 799898},
  {0x10F82, 799611},
  {0x10F83, 799642},
  {0x10F84, 799673},
  {0x10F85, 799709},
  {0x10F86, 800172},
  {0x10F87, 800232},
  {0x10F88, 800264},
  {0x10F89, 800199},
  {0x10FB0, 183413},
  {0x10FB1, 183789},
  {0x10FB2, 183460},
  {0x10FB3, 183537},
  {0x10FB4, 183512},
  {0x10FB5, 183561},
  {0x10FB6, 183841},
  {0x10FB7, 183483},
  {0x10FB8, 183886},
  {0x10FB9, 183582},
  {0x10FBA, 183863},
  {0x10FBB, 183605},
  {0x10FBC, 183628},
  {0x10FBD, 183653},
  {0x10FBE, 183675},
  {0x10FBF, 183741},
  {0x10FC0, 183437},
  {0x10FC1, 183697},
  {0x10FC2, 183718},
  {0x10FC3, 183766},
  {0x10FC4, 183819},
  {0x10FC5, 183933},
  {0x10FC6, 184056},
  {0x10FC7, 184007},
  {0x10FC8, 183910},
  {0x10FC9, 183985},
  {0x10FCA, 184031},
  {0x10FCB, 183955},
  {0x10FE0, 427819},
  {0x10FE1, 427860},
  {0x10FE2, 427902},
  {0x10FE3, 427880},
  {0x10FE4, 427923},
  {0x10FE5, 428201},
  {0x10FE6, 428240},
  {0x10FE7, 427941},
  {0x10FE8, 428181},
  {0x10FE9, 428220},
  {0x10FEA, 427961},
  {0x10FEB, 427981},
  {0x10FEC, 428003},
  {0x10FED, 428022},
  {0x10FEE, 428120},
  {0x10FEF, 427840},
  {0x10FF0, 428041},
  {0x10FF1, 428099},
  {0x10FF2, 428059},
  {0x10FF3, 428079},
  {0x10FF4, 428142},
  {0x10FF5, 428162},
  {0x10FF6, 428261},
  {0x11000, 134552},
  {0x11001, 134531},
  {0x11002, 134653},
  {0x11003, 134576},
  {0x11004, 134629},
  {0x11005, 132929},
  {0x11006, 132945},
  {0x11007, 133206},
  {0x11008, 133222},
  {0x11009, 133803},
  {0x1100A, 133819},
  {0x1100B, 133902},
  {0x1100C, 133926},
  {0x1100D, 133853},
  {0x1100E, 133877},
  {0x1100F, 133138},
  {0x11010, 132962},
  {0x11011, 133432},
  {0x11012, 132979},
  {0x11013, 133274},
  {0x11014, 133291},
  {0x11015, 133154},
  {0x11016, 133171},
  {0x11017, 133378},
  {0x11018, 133031},
  {0x11019, 133048},
  {0x1101A, 133239},
  {0x1101B, 133256},
  {0x1101C, 133414},
  {0x1101D, 133766},
  {0x1101E, 133784},
  {0x1101F, 133083},
  {0x11020, 133101},
  {0x11021, 133396},
  {0x11022, 133731},
  {0x11023, 133748},
  {0x11024, 133066},
  {0x11025, 133120},
  {0x11026, 133361},
  {0x11027, 133626},
  {0x11028, 133643},
  {0x11029, 132996},
  {0x1102A, 133013},
  {0x1102B, 133344},
  {0x1102C, 133951},
  {0x1102D, 133661},
  {0x1102E, 133309},
  {0x1102F, 133836},
  {0x11030, 133695},
  {0x11031, 133713},
  {0x11032, 133678},
  {0x11033, 133189},
  {0x11034, 133326},
  {0x11035, 133476},
  {0x11036, 133534},
  {0x11037, 133505},
  {0x11038, 134687},
  {0x11039, 134750},
  {0x1103A, 134803},
  {0x1103B, 134823},
  {0x1103C, 134936},
  {0x1103D, 134956},
  {0x1103E, 135034},
  {0x1103F, 135062},
  {0x11040, 134977},
  {0x11041, 135005},
  {0x11042, 134783},
  {0x11043, 134708},
  {0x11044, 134844},
  {0x11045, 134729},
  {0x11046, 134673},
  {0x11047, 132716},
  {0x11048, 132909},
  {0x11049, 134429},
  {0x1104A, 134452},
  {0x1104B, 134482},
  {0x1104C, 134397},
  {0x1104D, 134506},
  {0x11052, 134148},
  {0x11053, 134379},
  {0x11054, 134338},
  {0x11055, 134068},
  {0x11056, 134029},
  {0x11057, 134261},
  {0x11058, 134219},
  {0x11059, 133968},
  {0x1105A, 134108},
  {0x1105B, 134299},
  {0x1105C, 134358},
  {0x1105D, 134317},
  {0x1105E, 134048},
  {0x1105F, 134009},
  {0x11060, 134279},
  {0x11061, 134239},
  {0x11062, 133988},
  {0x11063, 134127},
  {0x11064, 134166},
  {0x11065, 134192},
  {0x11066, 132891},
  {0x11067, 132802},
  {0x11068, 132874},
  {0x11069, 132855},
  {0x1106A, 132766},
  {0x1106B, 132748},
  {0x1106C, 132838},
  {0x1106D, 132819},
  {0x1106E, 132729},
  {0x1106F, 132784},
  {0x11070, 134600},
  {0x11071, 133562},
  {0x11072, 133594},
  {0x11073, 134864},
  {0x11074, 134900},
  {0x11075, 133448},
  {0x1107F, 134087},
  {0x11080, 539505},
  {0x11081, 539484},
  {0x11082, 539566},
  {0x11083, 538637},
  {0x11084, 538653},
  {0x11085, 538934},
  {0x11086, 538950},
  {0x11087, 539353},
  {0x11088, 539369},
  {0x11089, 538866},
  {0x1108A, 538670},
  {0x1108B, 539142},
  {0x1108C, 538687},
  {0x1108D, 539002},
  {0x1108E, 539019},
  {0x1108F, 538882},
  {0x11090, 538899},
  {0x11091, 539088},
  {0x11092, 538739},
  {0x11093, 538756},
  {0x11094, 538967},
  {0x11095, 538984},
  {0x11096, 539124},
  {0x11097, 539316},
  {0x11098, 539334},
  {0x11099, 538791},
  {0x1109A, 538809},
  {0x1109B, 538829},
  {0x1109C, 539210},
  {0x1109D, 539106},
  {0x1109E, 539281},
  {0x1109F, 539298},
  {0x110A0, 538774},
  {0x110A1, 538848},
  {0x110A2, 539071},
  {0x110A3, 539158},
  {0x110A4, 539175},
  {0x110A5, 538704},
  {0x110A6, 538721},
  {0x110A7, 539054},
  {0x110A8, 539403},
  {0x110A9, 539193},
  {0x110AA, 539037},
  {0x110AB, 539386},
  {0x110AC, 539245},
  {0x110AD, 539263},
  {0x110AE, 539228},
  {0x110AF, 538917},
  {0x110B0, 539586},
  {0x110B1, 539669},
  {0x110B2, 539689},
  {0x110B3, 539730},
  {0x110B4, 539750},
  {0x110B5, 539649},
  {0x110B6, 539607},
  {0x110B7, 539710},
  {0x110B8, 539628},
  {0x110B9, 539547},
  {0x110BA, 539529},
  {0x110BB, 538528},
  {0x110BC, 538613},
  {0x110BD, 539420},
  {0x110BE, 539464},
  {0x110BF, 538586},
  {0x110C0, 538553},
  {0x110C1, 538566},
  {0x110C2, 539771},
  {0x110CD, 539439},
  {0x110D0, 896153},
  {0x110D1, 896177},
  {0x110D2, 895698},
  {0x110D3, 895722},
  {0x110D4, 895746},
  {0x110D5, 895817},
  {0x110D6, 895984},
  {0x110D7, 896032},
  {0x110D8, 895936},
  {0x110D9, 896008},
  {0x110DA, 896224},
  {0x110DB, 896105},
  {0x110DC, 896248},
  {0x110DD, 896129},
  {0x110DE, 895841},
  {0x110DF, 895912},
  {0x110E0, 895888},
  {0x110E1, 896057},
  {0x110E2, 895675},
  {0x110E3, 895770},
  {0x110E4, 895865},
  {0x110E5, 896201},
  {0x110E6, 896082},
  {0x110E7, 895794},
  {0x110E8, 895960},
  {0x110F0, 895651},
  {0x110F1, 895532},
  {0x110F2, 895628},
  {0x110F3, 895603},
  {0x110F4, 895484},
  {0x110F5, 895460},
  {0x110F6, 895580},
  {0x110F7, 895555},
  {0x110F8, 895435},
  {0x110F9, 895508},
  {0x11100, 177499},
  {0x11101, 177478},
  {0x11102, 177523},
  {0x11103, 176712},
  {0x11104, 176950},
  {0x11105, 177319},
  {0x11106, 176879},
  {0x11107, 177003},
  {0x11108, 177021},
  {0x11109, 176895},
  {0x1110A, 176913},
  {0x1110B, 177113},
  {0x1110C, 176766},
  {0x1110D, 176784},
  {0x1110E, 176966},
  {0x1110F, 176984},
  {0x11110, 177151},
  {0x11111, 177280},
  {0x11112, 177299},
  {0x11113, 176821},
  {0x11114, 176840},
  {0x11115, 177132},
  {0x11116, 177243},
  {0x11117, 177261},
  {0x11118, 176803},
  {0x11119, 176860},
  {0x1111A, 177095},
  {0x1111B, 177170},
  {0x1111C, 177188},
  {0x1111D, 176729},
  {0x1111E, 176747},
  {0x1111F, 177077},
  {0x11120, 177389},
  {0x11121, 177371},
  {0x11122, 177207},
  {0x11123, 177040},
  {0x11124, 177353},
  {0x11125, 177225},
  {0x11126, 176932},
  {0x11127, 177557},
  {0x11128, 177681},
  {0x11129, 177701},
  {0x1112A, 177763},
  {0x1112B, 177783},
  {0x1112C, 177640},
  {0x1112D, 177598},
  {0x1112E, 177722},
  {0x1112F, 177619},
  {0x11130, 177742},
  {0x11131, 177423},
  {0x11132, 176484},
  {0x11133, 177543},
  {0x11134, 177408},
  {0x11136, 176674},
  {0x11137, 176585},
  {0x11138, 176657},
  {0x11139, 176638},
  {0x1113A, 176549},
  {0x1113B, 176531},
  {0x1113C, 176621},
  {0x1113D, 176602},
  {0x1113E, 176512},
  {0x1113F, 176567},
  {0x11140, 177458},
  {0x11141, 176499},
  {0x11142, 176692},
  {0x11143, 177437},
  {0x11144, 177058},
  {0x11145, 177577},
  {0x11146, 177660},
  {0x11147, 177335},
  {0x11150, 657853},
  {0x11151, 658105},
  {0x11152, 658493},
  {0x11153, 658029},
  {0x11154, 658298},
  {0x11155, 658162},
  {0x11156, 658181},
  {0x11157, 658047},
  {0x11158, 658066},
  {0x11159, 657910},
  {0x1115A, 657929},
  {0x1115B, 658123},
  {0x1115C, 658142},
  {0x1115D, 658278},
  {0x1115E, 658452},
  {0x1115F, 658472},
  {0x11160, 657968},
  {0x11161, 657988},
  {0x11162, 658258},
  {0x11163, 658413},
  {0x11164, 658432},
  {0x11165, 657949},
  {0x11166, 658009},
  {0x11167, 658239},
  {0x11168, 658316},
  {0x11169, 658335},
  {0x1116A, 657871},
  {0x1116B, 657890},
  {0x1116C, 658220},
  {0x1116D, 658355},
  {0x1116E, 658201},
  {0x1116F, 658511},
  {0x11170, 658394},
  {0x11171, 658086},
  {0x11172, 658374},
  {0x11173, 658575},
  {0x11174, 657826},
  {0x11175, 658553},
  {0x11176, 658530},
  {0x11180, 854592},
  {0x11181, 854548},
  {0x11182, 854761},
  {0x11183, 853544},
  {0x11184, 853561},
  {0x11185, 853837},
  {0x11186, 853854},
  {0x11187, 854280},
  {0x11188, 854297},
  {0x11189, 854384},
  {0x1118A, 854409},
  {0x1118B, 854333},
  {0x1118C, 854358},
  {0x1118D, 853765},
  {0x1118E, 853579},
  {0x1118F, 854076},
  {0x11190, 853597},
  {0x11191, 853909},
  {0x11192, 853927},
  {0x11193, 853782},
  {0x11194, 853800},
  {0x11195, 854019},
  {0x11196, 853652},
  {0x11197, 853670},
  {0x11198, 853872},
  {0x11199, 853890},
  {0x1119A, 854057},
  {0x1119B, 854241},
  {0x1119C, 854260},
  {0x1119D, 853707},
  {0x1119E, 853726},
  {0x1119F, 854038},
  {0x111A0, 854204},
  {0x111A1, 854222},
  {0x111A2, 853689},
  {0x111A3, 853746},
  {0x111A4, 854001},
  {0x111A5, 854093},
  {0x111A6, 854111},
  {0x111A7, 853615},
  {0x111A8, 853633},
  {0x111A9, 853983},
  {0x111AA, 854435},
  {0x111AB, 854130},
  {0x111AC, 853946},
  {0x111AD, 853964},
  {0x111AE, 854315},
  {0x111AF, 854166},
  {0x111B0, 854185},
  {0x111B1, 854148},
  {0x111B2, 853819},
  {0x111B3, 854829},
  {0x111B4, 854916},
  {0x111B5, 854937},
  {0x111B6, 855015},
  {0x111B7, 855036},
  {0x111B8, 855117},
  {0x111B9, 855146},
  {0x111BA, 855058},
  {0x111BB, 855087},
  {0x111BC, 854895},
  {0x111BD, 854851},
  {0x111BE, 854959},
  {0x111BF, 854873},
  {0x111C0, 854741},
  {0x111C1, 854570},
  {0x111C2, 854651},
  {0x111C3, 854716},
  {0x111C4, 854453},
  {0x111C5, 853256},
  {0x111C6, 853460},
  {0x111C7, 853204},
  {0x111C8, 854530},
  {0x111C9, 854464},
  {0x111CA, 854676},
  {0x111CB, 854801},
  {0x111CC, 853494},
  {0x111CD, 854782},
  {0x111CE, 854980},
  {0x111CF, 854617},
  {0x111D0, 853441},
  {0x111D1, 853347},
  {0x111D2, 853423},
  {0x111D3, 853403},
  {0x111D4, 853309},
  {0x111D5, 853290},
  {0x111D6, 853385},
  {0x111D7, 853365},
  {0x111D8, 853270},
  {0x111D9, 853328},
  {0x111DA, 853481},
  {0x111DB, 854695},
  {0x111DC, 853525},
  {0x111DD, 853230},
  {0x111DE, 854484},
  {0x111DF, 854507},
  {0x111E1, 888223},
  {0x111E2, 888331},
  {0x111E3, 888303},
  {0x111E4, 888169},
  {0x111E5, 888142},
  {0x111E6, 888277},
  {0x111E7, 888249},
  {0x111E8, 888114},
  {0x111E9, 888196},
  {0x111EA, 888606},
  {0x111EB, 888663},
  {0x111EC, 888633},
  {0x111ED, 888416},
  {0x111EE, 888387},
  {0x111EF, 888577},
  {0x111F0, 888546},
  {0x111F1, 888357},
  {0x111F2, 888445},
  {0x111F3, 888475},
  {0x111F4, 888510},
  {0x11200, 574311},
  {0x11201, 574327},
  {0x11202, 574643},
  {0x11203, 575048},
  {0x11204, 574557},
  {0x11205, 574344},
  {0x11206, 574852},
  {0x11207, 574361},
  {0x11208, 574694},
  {0x11209, 574711},
  {0x1120A, 574573},
  {0x1120B, 574590},
  {0x1120C, 574608},
  {0x1120D, 574798},
  {0x1120E, 574431},
  {0x1120F, 574448},
  {0x11210, 574659},
  {0x11211, 574676},
  {0x11213, 574834},
  {0x11214, 575011},
  {0x11215, 575029},
  {0x11216, 574483},
  {0x11217, 574520},
  {0x11218, 574816},
  {0x11219, 574976},
  {0x1121A, 574993},
  {0x1121B, 574466},
  {0x1121C, 574501},
  {0x1121D, 574539},
  {0x1121E, 574781},
  {0x1121F, 574868},
  {0x11220, 574885},
  {0x11221, 574378},
  {0x11222, 574395},
  {0x11223, 574413},
  {0x11224, 574764},
  {0x11225, 575081},
  {0x11226, 574920},
  {0x11227, 574729},
  {0x11228, 575064},
  {0x11229, 574937},
  {0x1122A, 574626},
  {0x1122B, 574746},
  {0x1122C, 575213},
  {0x1122D, 575296},
  {0x1122E, 575316},
  {0x1122F, 575357},
  {0x11230, 575276},
  {0x11231, 575234},
  {0x11232, 575337},
  {0x11233, 575255},
  {0x11234, 575118},
  {0x11235, 575194},
  {0x11236, 575139},
  {0x11237, 575157},
  {0x11238, 574251},
  {0x11239, 574264},
  {0x1123A, 575405},
  {0x1123B, 575098},
  {0x1123C, 574284},
  {0x1123D, 574226},
  {0x1123E, 575176},
  {0x1123F, 574903},
  {0x11240, 574954},
  {0x11241, 575377},
  {0x11280, 743192},
  {0x11281, 743451},
  {0x11282, 743821},
  {0x11283, 743379},
  {0x11284, 743505},
  {0x11285, 743523},
  {0x11286, 743396},
  {0x11288, 743414},
  {0x1128A, 743246},
  {0x1128B, 743264},
  {0x1128C, 743468},
  {0x1128D, 743486},
  {0x1128F, 743615},
  {0x11290, 743782},
  {0x11291, 743801},
  {0x11292, 743301},
  {0x11293, 743320},
  {0x11294, 743340},
  {0x11295, 743596},
  {0x11296, 743745},
  {0x11297, 743763},
  {0x11298, 743283},
  {0x11299, 743360},
  {0x1129A, 743578},
  {0x1129B, 743634},
  {0x1129C, 743652},
  {0x1129D, 743209},
  {0x1129F, 743227},
  {0x112A0, 743560},
  {0x112A1, 743856},
  {0x112A2, 743671},
  {0x112A3, 743542},
  {0x112A4, 743838},
  {0x112A5, 743727},
  {0x112A6, 743433},
  {0x112A7, 743708},
  {0x112A8, 743689},
  {0x112A9, 743874},
  {0x112B0, 575637},
  {0x112B1, 575656},
  {0x112B2, 576026},
  {0x112B3, 576045},
  {0x112B4, 576517},
  {0x112B5, 576536},
  {0x112B6, 575925},
  {0x112B7, 575676},
  {0x112B8, 576291},
  {0x112B9, 575696},
  {0x112BA, 576127},
  {0x112BB, 576147},
  {0x112BC, 575944},
  {0x112BD, 575964},
  {0x112BE, 575985},
  {0x112BF, 576228},
  {0x112C0, 575778},
  {0x112C1, 575798},
  {0x112C2, 576065},
  {0x112C3, 576106},
  {0x112C4, 576085},
  {0x112C5, 576270},
  {0x112C6, 576474},
  {0x112C7, 576495},
  {0x112C8, 575839},
  {0x112C9, 575860},
  {0x112CA, 576371},
  {0x112CB, 575882},
  {0x112CC, 576249},
  {0x112CD, 576433},
  {0x112CE, 576453},
  {0x112CF, 575819},
  {0x112D0, 575904},
  {0x112D1, 576208},
  {0x112D2, 576310},
  {0x112D3, 576330},
  {0x112D4, 575716},
  {0x112D5, 575736},
  {0x112D6, 575757},
  {0x112D7, 576188},
  {0x112D8, 576576},
  {0x112D9, 576351},
  {0x112DA, 576168},
  {0x112DB, 576556},
  {0x112DC, 576412},
  {0x112DD, 576392},
  {0x112DE, 576006},
  {0x112DF, 576596},
  {0x112E0, 576663},
  {0x112E1, 576758},
  {0x112E2, 576781},
  {0x112E3, 576828},
  {0x112E4, 576851},
  {0x112E5, 576735},
  {0x112E6, 576687},
  {0x112E7, 576805},
  {0x112E8, 576711},
  {0x112E9, 576620},
  {0x112EA, 576641},
  {0x112F0, 575616},
  {0x112F1, 575512},
  {0x112F2, 575596},
  {0x112F3, 575574},
  {0x112F4, 575470},
  {0x112F5, 575449},
  {0x112F6, 575554},
  {0x112F7, 575532},
  {0x112F8, 575427},
  {0x112F9, 575491},
  {0x11300, 459593},
  {0x11301, 459568},
  {0x11302, 459524},
  {0x11303, 459689},
  {0x11305, 458535},
  {0x11306, 458552},
  {0x11307, 458829},
  {0x11308, 458846},
  {0x11309, 459273},
  {0x1130A, 459290},
  {0x1130B, 459444},
  {0x1130C, 459393},
  {0x1130F, 458756},
  {0x11310, 458570},
  {0x11313, 459068},
  {0x11314, 458588},
  {0x11315, 458901},
  {0x11316, 458919},
  {0x11317, 458774},
  {0x11318, 458792},
  {0x11319, 459011},
  {0x1131A, 458643},
  {0x1131B, 458661},
  {0x1131C, 458864},
  {0x1131D, 458882},
  {0x1131E, 459049},
  {0x1131F, 459234},
  {0x11320, 459253},
  {0x11321, 458698},
  {0x11322, 458717},
  {0x11323, 459030},
  {0x11324, 459197},
  {0x11325, 459215},
  {0x11326, 458680},
  {0x11327, 458737},
  {0x11328, 458993},
  {0x1132A, 459086},
  {0x1132B, 459104},
  {0x1132C, 458606},
  {0x1132D, 458624},
  {0x1132E, 458975},
  {0x1132F, 459495},
  {0x11330, 459123},
  {0x11332, 458938},
  {0x11333, 458956},
  {0x11335, 459308},
  {0x11336, 459159},
  {0x11337, 459178},
  {0x11338, 459141},
  {0x11339, 458811},
  {0x1133B, 196945},
  {0x1133C, 459631},
  {0x1133D, 459546},
  {0x1133E, 459710},
  {0x1133F, 459798},
  {0x11340, 459819},
  {0x11341, 459863},
  {0x11342, 459884},
  {0x11343, 459965},
  {0x11344, 459994},
  {0x11347, 459776},
  {0x11348, 459732},
  {0x1134B, 459841},
  {0x1134C, 459754},
  {0x1134D, 459669},
  {0x11350, 459513},
  {0x11357, 458512},
  {0x1135D, 459650},
  {0x1135E, 459326},
  {0x1135F, 459356},
  {0x11360, 459469},
  {0x11361, 459418},
  {0x11362, 459906},
  {0x11363, 459935},
  {0x11366, 202712},
  {0x11367, 202598},
  {0x11368, 202684},
  {0x11369, 202654},
  {0x1136A, 202569},
  {0x1136B, 202540},
  {0x1136C, 202626},
  {0x11370, 202741},
  {0x11371, 202768},
  {0x11372, 202796},
  {0x11373, 202852},
  {0x11374, 202824},
  {0x11380, 951201},
  {0x11381, 951224},
  {0x11382, 951591},
  {0x11383, 951614},
  {0x11384, 952230},
  {0x11385, 952253},
  {0x11386, 952364},
  {0x11387, 952395},
  {0x11388, 952301},
  {0x11389, 952332},
  {0x1138B, 951494},
  {0x1138E, 951248},
  {0x11390, 951934},
  {0x11391, 951272},
  {0x11392, 951687},
  {0x11393, 951711},
  {0x11394, 951518},
  {0x11395, 951542},
  {0x11396, 951859},
  {0x11397, 951345},
  {0x11398, 951369},
  {0x11399, 951638},
  {0x1139A, 951662},
  {0x1139B, 951909},
  {0x1139C, 952179},
  {0x1139D, 952204},
  {0x1139E, 951418},
  {0x1139F, 951443},
  {0x113A0, 951884},
  {0x113A1, 952130},
  {0x113A2, 952154},
  {0x113A3, 951394},
  {0x113A4, 951469},
  {0x113A5, 951835},
  {0x113A6, 951958},
  {0x113A7, 951982},
  {0x113A8, 951296},
  {0x113A9, 951320},
  {0x113AA, 951811},
  {0x113AB, 952427},
  {0x113AC, 952007},
  {0x113AD, 951736},
  {0x113AE, 952277},
  {0x113AF, 952080},
  {0x113B0, 952105},
  {0x113B1, 952056},
  {0x113B2, 951567},
  {0x113B3, 951760},
  {0x113B4, 952031},
  {0x113B5, 951785},
  {0x113B7, 952499},
  {0x113B8, 952806},
  {0x113B9, 952918},
  {0x113BA, 952945},
  {0x113BB, 953001},
  {0x113BC, 953028},
  {0x113BD, 953127},
  {0x113BE, 953162},
  {0x113BF, 953056},
  {0x113C0, 953091},
  {0x113C2, 952890},
  {0x113C5, 952834},
  {0x113C7, 952973},
  {0x113C8, 952862},
  {0x113C9, 951071},
  {0x113CA, 952527},
  {0x113CC, 952471},
  {0x113CD, 952712},
  {0x113CE, 952686},
  {0x113CF, 952563},
  {0x113D0, 951100},
  {0x113D1, 952451},
  {0x113D2, 951171},
  {0x113D3, 952627},
  {0x113D4, 951124},
  {0x113D5, 951144},
  {0x113D7, 952596},
  {0x113D8, 952652},
  {0x113E1, 952773},
  {0x113E2, 952739},
  {0x11400, 769075},
  {0x11401, 769089},
  {0x11402, 769320},
  {0x11403, 769334},
  {0x11404, 769773},
  {0x11405, 769787},
  {0x11406, 769874},
  {0x11407, 769896},
  {0x11408, 769829},
  {0x11409, 769851},
  {0x1140A, 769260},
  {0x1140B, 769104},
  {0x1140C, 769586},
  {0x1140D, 769119},
  {0x1140E, 769380},
  {0x1140F, 769395},
  {0x11410, 769274},
  {0x11411, 769289},
  {0x11412, 769488},
  {0x11413, 769504},
  {0x11414, 769165},
  {0x11415, 769180},
  {0x11416, 769349},
  {0x11417, 769364},
  {0x11418, 769553},
  {0x11419, 769569},
  {0x1141A, 769740},
  {0x1141B, 769756},
  {0x1141C, 769211},
  {0x1141D, 769227},
  {0x1141E, 769537},
  {0x1141F, 769709},
  {0x11420, 769724},
  {0x11421, 769196},
  {0x11422, 769244},
  {0x11423, 769473},
  {0x11424, 769521},
  {0x11425, 769600},
  {0x11426, 769615},
  {0x11427, 769134},
  {0x11428, 769149},
  {0x11429, 769442},
  {0x1142A, 769457},
  {0x1142B, 769934},
  {0x1142C, 769631},
  {0x1142D, 769646},
  {0x1142E, 769411},
  {0x1142F, 769426},
  {0x11430, 769919},
  {0x11431, 769677},
  {0x11432, 769693},
  {0x11433, 769662},
  {0x11434, 769305},
  {0x11435, 770188},
  {0x11436, 770263},
  {0x11437, 770281},
  {0x11438, 770318},
  {0x11439, 770336},
  {0x1143A, 770408},
  {0x1143B, 770434},
  {0x1143C, 770355},
  {0x1143D, 770381},
  {0x1143E, 770245},
  {0x1143F, 770207},
  {0x11440, 770300},
  {0x11441, 770226},
  {0x11442, 770153},
  {0x11443, 770046},
  {0x11444, 770008},
  {0x11445, 770170},
  {0x11446, 770115},
  {0x11447, 770027},
  {0x11448, 770068},
  {0x11449, 769949},
  {0x1144A, 769996},
  {0x1144B, 768832},
  {0x1144C, 769021},
  {0x1144D, 768821},
  {0x1144E, 769039},
  {0x1144F, 768798},
  {0x11450, 768987},
  {0x11451, 768908},
  {0x11452, 768972},
  {0x11453, 768955},
  {0x11454, 768876},
  {0x11455, 768860},
  {0x11456, 768940},
  {0x11457, 768923},
  {0x11458, 768843},
  {0x11459, 768892},
  {0x1145A, 769003},
  {0x1145B, 769957},
  {0x1145D, 769055},
  {0x1145E, 769979},
  {0x1145F, 769802},
  {0x11460, 770093},
  {0x11461, 770131},
  {0x11480, 945418},
  {0x11481, 945635},
  {0x11482, 945652},
  {0x11483, 945928},
  {0x11484, 945945},
  {0x11485, 946352},
  {0x11486, 946369},
  {0x11487, 946456},
  {0x11488, 946481},
  {0x11489, 946405},
  {0x1148A, 946430},
  {0x1148B, 945856},
  {0x1148C, 945670},
  {0x1148D, 946148},
  {0x1148E, 945688},
  {0x1148F, 946000},
  {0x11490, 946018},
  {0x11491, 945873},
  {0x11492, 945891},
  {0x11493, 946091},
  {0x11494, 945743},
  {0x11495, 945761},
  {0x11496, 945963},
  {0x11497, 945981},
  {0x11498, 946129},
  {0x11499, 946313},
  {0x1149A, 946332},
  {0x1149B, 945798},
  {0x1149C, 945817},
  {0x1149D, 946110},
  {0x1149E, 946276},
  {0x1149F, 946294},
  {0x114A0, 945780},
  {0x114A1, 945837},
  {0x114A2, 946073},
  {0x114A3, 946165},
  {0x114A4, 946183},
  {0x114A5, 945706},
  {0x114A6, 945724},
  {0x114A7, 946055},
  {0x114A8, 946507},
  {0x114A9, 946202},
  {0x114AA, 946037},
  {0x114AB, 946387},
  {0x114AC, 946238},
  {0x114AD, 946257},
  {0x114AE, 946220},
  {0x114AF, 945910},
  {0x114B0, 946665},
  {0x114B1, 946752},
  {0x114B2, 946773},
  {0x114B3, 946870},
  {0x114B4, 946891},
  {0x114B5, 946972},
  {0x114B6, 947001},
  {0x114B7, 946913},
  {0x114B8, 946942},
  {0x114B9, 946731},
  {0x114BA, 946816},
  {0x114BB, 946687},
  {0x114BC, 946795},
  {0x114BD, 946843},
  {0x114BE, 946709},
  {0x114BF, 946580},
  {0x114C0, 946536},
  {0x114C1, 946644},
  {0x114C2, 946624},
  {0x114C3, 946605},
  {0x114C4, 946558},
  {0x114C5, 945621},
  {0x114C6, 945392},
  {0x114C7, 946525},
  {0x114D0, 945602},
  {0x114D1, 945508},
  {0x114D2, 945584},
  {0x114D3, 945564},
  {0x114D4, 945470},
  {0x114D5, 945451},
  {0x114D6, 945546},
  {0x114D7, 945526},
  {0x114D8, 945431},
  {0x114D9, 945489},
  {0x11580, 856787},
  {0x11581, 856804},
  {0x11582, 857107},
  {0x11583, 857124},
  {0x11584, 857648},
  {0x11585, 857665},
  {0x11586, 857752},
  {0x11587, 857777},
  {0x11588, 857701},
  {0x11589, 857726},
  {0x1158A, 857035},
  {0x1158B, 856822},
  {0x1158C, 857327},
  {0x1158D, 856867},
  {0x1158E, 857179},
  {0x1158F, 857197},
  {0x11590, 857052},
  {0x11591, 857070},
  {0x11592, 857270},
  {0x11593, 856922},
  {0x11594, 856940},
  {0x11595, 857142},
  {0x11596, 857160},
  {0x11597, 857308},
  {0x11598, 857532},
  {0x11599, 857551},
  {0x1159A, 856977},
  {0x1159B, 856996},
  {0x1159C, 857289},
  {0x1159D, 857455},
  {0x1159E, 857473},
  {0x1159F, 856959},
  {0x115A0, 857016},
  {0x115A1, 857252},
  {0x115A2, 857344},
  {0x115A3, 857362},
  {0x115A4, 856885},
  {0x115A5, 856903},
  {0x115A6, 857234},
  {0x115A7, 857803},
  {0x115A8, 857381},
  {0x115A9, 857216},
  {0x115AA, 857683},
  {0x115AB, 857417},
  {0x115AC, 857436},
  {0x115AD, 857399},
  {0x115AE, 857089},
  {0x115AF, 858756},
  {0x115B0, 858906},
  {0x115B1, 858927},
  {0x115B2, 858970},
  {0x115B3, 858991},
  {0x115B4, 859013},
  {0x115B5, 859042},
  {0x115B8, 858885},
  {0x115B9, 858778},
  {0x115BA, 858949},
  {0x115BB, 858863},
  {0x115BC, 858650},
  {0x115BD, 858628},
  {0x115BE, 858735},
  {0x115BF, 858715},
  {0x115C0, 858675},
  {0x115C1, 858694},
  {0x115C2, 856727},
  {0x115C3, 856741},
  {0x115C4, 858606},
  {0x115C5, 858584},
  {0x115C6, 857821},
  {0x115C7, 857847},
  {0x115C8, 857873},
  {0x115C9, 856762},
  {0x115CA, 858484},
  {0x115CB, 858429},
  {0x115CC, 858214},
  {0x115CD, 858266},
  {0x115CE, 858325},
  {0x115CF, 857899},
  {0x115D0, 857932},
  {0x115D1, 858125},
  {0x115D2, 858541},
  {0x115D3, 858168},
  {0x115D4, 858384},
  {0x115D5, 858029},
  {0x115D6, 858072},
  {0x115D7, 857975},
  {0x115D8, 857492},
  {0x115D9, 857571},
  {0x115DA, 857609},
  {0x115DB, 856840},
  {0x115DC, 858800},
  {0x115DD, 858831},
  {0x11600, 725795},
  {0x11601, 725809},
  {0x11602, 726040},
  {0x11603, 726054},
  {0x11604, 726411},
  {0x11605, 726425},
  {0x11606, 726500},
  {0x11607, 726522},
  {0x11608, 726455},
  {0x11609, 726477},
  {0x1160A, 725980},
  {0x1160B, 725824},
  {0x1160C, 726240},
  {0x1160D, 725839},
  {0x1160E, 726100},
  {0x1160F, 726115},
  {0x11610, 725994},
  {0x11611, 726009},
  {0x11612, 726192},
  {0x11613, 725885},
  {0x11614, 725900},
  {0x11615, 726069},
  {0x11616, 726084},
  {0x11617, 726224},
  {0x11618, 726378},
  {0x11619, 726394},
  {0x1161A, 725931},
  {0x1161B, 725947},
  {0x1161C, 726208},
  {0x1161D, 726347},
  {0x1161E, 726362},
  {0x1161F, 725916},
  {0x11620, 725964},
  {0x11621, 726177},
  {0x11622, 726254},
  {0x11623, 726269},
  {0x11624, 725854},
  {0x11625, 725869},
  {0x11626, 726162},
  {0x11627, 726545},
  {0x11628, 726285},
  {0x11629, 726131},
  {0x1162A, 726440},
  {0x1162B, 726315},
  {0x1162C, 726331},
  {0x1162D, 726300},
  {0x1162E, 726025},
  {0x1162F, 726146},
  {0x11630, 726651},
  {0x11631, 726726},
  {0x11632, 726744},
  {0x11633, 726781},
  {0x11634, 726799},
  {0x11635, 726871},
  {0x11636, 726897},
  {0x11637, 726818},
  {0x11638, 726844},
  {0x11639, 726708},
  {0x1163A, 726670},
  {0x1163B, 726763},
  {0x1163C, 726689},
  {0x1163D, 726560},
  {0x1163E, 726633},
  {0x1163F, 726616},
  {0x11640, 726579},
  {0x11641, 725606},
  {0x11642, 725777},
  {0x11643, 725583},
  {0x11644, 726601},
  {0x11650, 725761},
  {0x11651, 725682},
  {0x11652, 725746},
  {0x11653, 725729},
  {0x11654, 725650},
  {0x11655, 725634},
  {0x11656, 725714},
  {0x11657, 725697},
  {0x11658, 725617},
  {0x11659, 725666},
  {0x11660, 737563},
  {0x11661, 741766},
  {0x11662, 737835},
  {0x11663, 742041},
  {0x11664, 737526},
  {0x11665, 741835},
  {0x11666, 741790},
  {0x11667, 738086},
  {0x11668, 738111},
  {0x11669, 741913},
  {0x1166A, 741978},
  {0x1166B, 741935},
  {0x1166C, 742078},
  {0x11680, 924635},
  {0x11681, 924650},
  {0x11682, 924921},
  {0x11683, 924936},
  {0x11684, 925299},
  {0x11685, 925314},
  {0x11686, 924857},
  {0x11687, 924666},
  {0x11688, 925117},
  {0x11689, 924707},
  {0x1168A, 924985},
  {0x1168B, 925001},
  {0x1168C, 924872},
  {0x1168D, 924888},
  {0x1168E, 925066},
  {0x1168F, 924756},
  {0x11690, 924772},
  {0x11691, 924952},
  {0x11692, 924968},
  {0x11693, 925100},
  {0x11694, 925264},
  {0x11695, 925281},
  {0x11696, 924805},
  {0x11697, 924822},
  {0x11698, 925083},
  {0x11699, 925231},
  {0x1169A, 925247},
  {0x1169B, 924789},
  {0x1169C, 924840},
  {0x1169D, 925050},
  {0x1169E, 925132},
  {0x1169F, 925148},
  {0x116A0, 924723},
  {0x116A1, 924739},
  {0x116A2, 925034},
  {0x116A3, 925346},
  {0x116A4, 925165},
  {0x116A5, 925018},
  {0x116A6, 925330},
  {0x116A7, 925214},
  {0x116A8, 925198},
  {0x116A9, 924905},
  {0x116AA, 925181},
  {0x116AB, 925362},
  {0x116AC, 925417},
  {0x116AD, 925436},
  {0x116AE, 925515},
  {0x116AF, 925534},
  {0x116B0, 925573},
  {0x116B1, 925592},
  {0x116B2, 925496},
  {0x116B3, 925456},
  {0x116B4, 925554},
  {0x116B5, 925476},
  {0x116B6, 925399},
  {0x116B7, 925382},
  {0x116B8, 924682},
  {0x116B9, 924441},
  {0x116C0, 924618},
  {0x116C1, 924534},
  {0x116C2, 924602},
  {0x116C3, 924584},
  {0x116C4, 924500},
  {0x116C5, 924483},
  {0x116C6, 924568},
  {0x116C7, 924550},
  {0x116C8, 924465},
  {0x116C9, 924517},
  {0x116D0, 755736},
  {0x116D1, 755622},
  {0x116D2, 755714},
  {0x116D3, 755690},
  {0x116D4, 755576},
  {0x116D5, 755553},
  {0x116D6, 755668},
  {0x116D7, 755644},
  {0x116D8, 755529},
  {0x116D9, 755599},
  {0x116DA, 752623},
  {0x116DB, 752439},
  {0x116DC, 752587},
  {0x116DD, 752549},
  {0x116DE, 752365},
  {0x116DF, 752328},
  {0x116E0, 752513},
  {0x116E1, 752475},
  {0x116E2, 752290},
  {0x116E3, 752402},
  {0x11700, 4276},
  {0x11701, 4291},
  {0x11702, 4368},
  {0x11703, 4353},
  {0x11704, 4477},
  {0x11705, 4048},
  {0x11706, 4416},
  {0x11707, 4431},
  {0x11708, 4073},
  {0x11709, 4338},
  {0x1170A, 4245},
  {0x1170B, 4119},
  {0x1170C, 4492},
  {0x1170D, 4447},
  {0x1170E, 4307},
  {0x1170F, 4462},
  {0x11710, 4400},
  {0x11711, 4230},
  {0x11712, 3984},
  {0x11713, 4135},
  {0x11714, 4183},
  {0x11715, 4199},
  {0x11716, 4023},
  {0x11717, 4214},
  {0x11718, 4088},
  {0x11719, 4260},
  {0x1171A, 3998},
  {0x1171D, 3725},
  {0x1171E, 3794},
  {0x1171F, 3755},
  {0x11720, 4666},
  {0x11721, 4684},
  {0x11722, 4778},
  {0x11723, 4796},
  {0x11724, 4833},
  {0x11725, 4851},
  {0x11726, 4760},
  {0x11727, 4741},
  {0x11728, 4815},
  {0x11729, 4703},
  {0x1172A, 4722},
  {0x1172B, 4576},
  {0x11730, 3968},
  {0x11731, 3889},
  {0x11732, 3953},
  {0x11733, 3936},
  {0x11734, 3857},
  {0x11735, 3841},
  {0x11736, 3921},
  {0x11737, 3904},
  {0x11738, 3824},
  {0x11739, 3873},
  {0x1173A, 4541},
  {0x1173B, 4557},
  {0x1173C, 4627},
  {0x1173D, 4609},
  {0x1173E, 4593},
  {0x1173F, 4651},
  {0x11740, 4104},
  {0x11741, 4508},
  {0x11742, 4524},
  {0x11743, 4150},
  {0x11744, 4166},
  {0x11745, 4384},
  {0x11746, 4322},
  {0x11800, 281106},
  {0x11801, 281121},
  {0x11802, 281367},
  {0x11803, 281382},
  {0x11804, 281762},
  {0x11805, 281777},
  {0x11806, 281303},
  {0x11807, 281137},
  {0x11808, 281563},
  {0x11809, 281153},
  {0x1180A, 281431},
  {0x1180B, 281447},
  {0x1180C, 281318},
  {0x1180D, 281334},
  {0x1180E, 281512},
  {0x1180F, 281202},
  {0x11810, 281218},
  {0x11811, 281398},
  {0x11812, 281414},
  {0x11813, 281546},
  {0x11814, 281727},
  {0x11815, 281744},
  {0x11816, 281251},
  {0x11817, 281268},
  {0x11818, 281529},
  {0x11819, 281694},
  {0x1181A, 281710},
  {0x1181B, 281235},
  {0x1181C, 281286},
  {0x1181D, 281496},
  {0x1181E, 281578},
  {0x1181F, 281594},
  {0x11820, 281169},
  {0x11821, 281185},
  {0x11822, 281480},
  {0x11823, 281809},
  {0x11824, 281611},
  {0x11825, 281464},
  {0x11826, 281793},
  {0x11827, 281660},
  {0x11828, 281677},
  {0x11829, 281644},
  {0x1182A, 281351},
  {0x1182B, 281627},
  {0x1182C, 281899},
  {0x1182D, 281978},
  {0x1182E, 281997},
  {0x1182F, 282036},
  {0x11830, 282055},
  {0x11831, 282075},
  {0x11832, 282102},
  {0x11833, 281959},
  {0x11834, 281919},
  {0x11835, 282017},
  {0x11836, 281939},
  {0x11837, 281825},
  {0x11838, 281880},
  {0x11839, 281862},
  {0x1183A, 281845},
  {0x1183B, 281082},
  {0x118A0, 980160},
  {0x118A1, 979672},
  {0x118A2, 980500},
  {0x118A3, 980590},
  {0x118A4, 980530},
  {0x118A5, 980560},
  {0x118A6, 980100},
  {0x118A7, 980438},
  {0x118A8, 979852},
  {0x118A9, 980224},
  {0x118AA, 979731},
  {0x118AB, 979943},
  {0x118AC, 980130},
  {0x118AD, 979912},
  {0x118AE, 980620},
  {0x118AF, 980408},
  {0x118B0, 979881},
  {0x118B1, 980253},
  {0x118B2, 980377},
  {0x118B3, 980192},
  {0x118B4, 979822},
  {0x118B5, 979762},
  {0x118B6, 979701},
  {0x118B7, 979792},
  {0x118B8, 980284},
  {0x118B9, 980004},
  {0x118BA, 980036},
  {0x118BB, 980068},
  {0x118BC, 979973},
  {0x118BD, 980345},
  {0x118BE, 980314},
  {0x118BF, 980468},
  {0x118C0, 981581},
  {0x118C1, 981125},
  {0x118C2, 981899},
  {0x118C3, 981983},
  {0x118C4, 981927},
  {0x118C5, 981955},
  {0x118C6, 981525},
  {0x118C7, 981841},
  {0x118C8, 981293},
  {0x118C9, 981641},
  {0x118CA, 981180},
  {0x118CB, 981378},
  {0x118CC, 981553},
  {0x118CD, 981349},
  {0x118CE, 982011},
  {0x118CF, 981813},
  {0x118D0, 981320},
  {0x118D1, 981668},
  {0x118D2, 981784},
  {0x118D3, 981611},
  {0x118D4, 981265},
  {0x118D5, 981209},
  {0x118D6, 981152},
  {0x118D7, 981237},
  {0x118D8, 981697},
  {0x118D9, 981435},
  {0x118DA, 981465},
  {0x118DB, 981495},
  {0x118DC, 981406},
  {0x118DD, 981754},
  {0x118DE, 981725},
  {0x118DF, 981869},
  {0x118E0, 980858},
  {0x118E1, 980744},
  {0x118E2, 980836},
  {0x118E3, 980812},
  {0x118E4, 980698},
  {0x118E5, 980675},
  {0x118E6, 980790},
  {0x118E7, 980766},
  {0x118E8, 980651},
  {0x118E9, 980721},
  {0x118EA, 981035},
  {0x118EB, 981084},
  {0x118EC, 981058},
  {0x118ED, 980932},
  {0x118EE, 980907},
  {0x118EF, 981010},
  {0x118F0, 980983},
  {0x118F1, 980881},
  {0x118F2, 980957},
  {0x118FF, 981110},
  {0x11900, 279208},
  {0x11901, 279229},
  {0x11902, 279521},
  {0x11903, 279542},
  {0x11904, 280013},
  {0x11905, 280034},
  {0x11906, 279433},
  {0x11909, 279789},
  {0x1190C, 279586},
  {0x1190D, 279608},
  {0x1190E, 279454},
  {0x1190F, 279476},
  {0x11910, 279720},
  {0x11911, 279296},
  {0x11912, 279318},
  {0x11913, 279564},
  {0x11915, 279766},
  {0x11916, 279990},
  {0x11918, 279363},
  {0x11919, 279386},
  {0x1191A, 279743},
  {0x1191B, 279945},
  {0x1191C, 279967},
  {0x1191D, 279341},
  {0x1191E, 279410},
  {0x1191F, 279698},
  {0x11920, 279810},
  {0x11921, 279832},
  {0x11922, 279251},
  {0x11923, 279273},
  {0x11924, 279676},
  {0x11925, 280078},
  {0x11926, 280100},
  {0x11927, 279855},
  {0x11928, 279631},
  {0x11929, 280056},
  {0x1192A, 279899},
  {0x1192B, 279922},
  {0x1192C, 279877},
  {0x1192D, 279499},
  {0x1192E, 279653},
  {0x1192F, 280123},
  {0x11930, 280343},
  {0x11931, 280420},
  {0x11932, 280445},
  {0x11933, 280496},
  {0x11934, 280521},
  {0x11935, 280395},
  {0x11937, 280369},
  {0x11938, 280471},
  {0x1193B, 280221},
  {0x1193C, 280247},
  {0x1193D, 280276},
  {0x1193E, 280324},
  {0x1193F, 280189},
  {0x11940, 280167},
  {0x11941, 279185},
  {0x11942, 280145},
  {0x11943, 280301},
  {0x11944, 279108},
  {0x11945, 279162},
  {0x11946, 279133},
  {0x11950, 279085},
  {0x11951, 278971},
  {0x11952, 279063},
  {0x11953, 279039},
  {0x11954, 278925},
  {0x11955, 278902},
  {0x11956, 279017},
  {0x11957, 278993},
  {0x11958, 278878},
  {0x11959, 278948},
  {0x119A0, 760447},
  {0x119A1, 760468},
  {0x119A2, 760804},
  {0x119A3, 760825},
  {0x119A4, 761366},
  {0x119A5, 761387},
  {0x119A6, 761431},
  {0x119A7, 761460},
  {0x119AA, 760716},
  {0x119AB, 760490},
  {0x119AC, 761095},
  {0x119AD, 760512},
  {0x119AE, 760892},
  {0x119AF, 760914},
  {0x119B0, 760737},
  {0x119B1, 760759},
  {0x119B2, 761026},
  {0x119B3, 760579},
  {0x119B4, 760601},
  {0x119B5, 760847},
  {0x119B6, 760869},
  {0x119B7, 761072},
  {0x119B8, 761319},
  {0x119B9, 761342},
  {0x119BA, 760646},
  {0x119BB, 760669},
  {0x119BC, 761049},
  {0x119BD, 761274},
  {0x119BE, 761296},
  {0x119BF, 760624},
  {0x119C0, 760693},
  {0x119C1, 761004},
  {0x119C2, 761116},
  {0x119C3, 761138},
  {0x119C4, 760534},
  {0x119C5, 760556},
  {0x119C6, 760982},
  {0x119C7, 761490},
  {0x119C8, 761161},
  {0x119C9, 760937},
  {0x119CA, 761409},
  {0x119CB, 761228},
  {0x119CC, 761251},
  {0x119CD, 761206},
  {0x119CE, 760782},
  {0x119CF, 760959},
  {0x119D0, 761183},
  {0x119D1, 761638},
  {0x119D2, 761741},
  {0x119D3, 761766},
  {0x119D4, 761856},
  {0x119D5, 761881},
  {0x119D6, 761907},
  {0x119D7, 761940},
  {0x119DA, 761716},
  {0x119DB, 761664},
  {0x119DC, 761792},
  {0x119DD, 761690},
  {0x119DE, 761512},
  {0x119DF, 761613},
  {0x119E0, 761589},
  {0x119E1, 761538},
  {0x119E2, 761564},
  {0x119E3, 760424},
  {0x119E4, 761817},
  {0x11A00, 1011069},
  {0x11A01, 1012666},
  {0x11A02, 1012826},
  {0x11A03, 1012796},
  {0x11A04, 1012636},
  {0x11A05, 1012726},
  {0x11A06, 1012696},
  {0x11A07, 1012574},
  {0x11A08, 1012605},
  {0x11A09, 1012757},
  {0x11A0A, 1012539},
  {0x11A0B, 1011483},
  {0x11A0C, 1011510},
  {0x11A0D, 1011374},
  {0x11A0E, 1011401},
  {0x11A0F, 1011648},
  {0x11A10, 1011150},
  {0x11A11, 1011177},
  {0x11A12, 1011456},
  {0x11A13, 1011704},
  {0x11A14, 1012009},
  {0x11A15, 1012037},
  {0x11A16, 1011232},
  {0x11A17, 1011260},
  {0x11A18, 1011676},
  {0x11A19, 1011897},
  {0x11A1A, 1011924},
  {0x11A1B, 1011205},
  {0x11A1C, 1011289},
  {0x11A1D, 1011621},
  {0x11A1E, 1011732},
  {0x11A1F, 1011759},
  {0x11A20, 1011095},
  {0x11A21, 1011122},
  {0x11A22, 1011594},
  {0x11A23, 1011952},
  {0x11A24, 1011980},
  {0x11A25, 1011317},
  {0x11A26, 1011345},
  {0x11A27, 1012147},
  {0x11A28, 1012120},
  {0x11A29, 1011042},
  {0x11A2A, 1012093},
  {0x11A2B, 1011787},
  {0x11A2C, 1011567},
  {0x11A2D, 1012066},
  {0x11A2E, 1011841},
  {0x11A2F, 1011869},
  {0x11A30, 1011814},
  {0x11A31, 1011429},
  {0x11A32, 1011538},
  {0x11A33, 1010921},
  {0x11A34, 1012453},
  {0x11A35, 1012371},
  {0x11A36, 1012405},
  {0x11A37, 1012328},
  {0x11A38, 1012297},
  {0x11A39, 1012482},
  {0x11A3A, 1010878},
  {0x11A3B, 1010837},
  {0x11A3C, 1010755},
  {0x11A3D, 1010714},
  {0x11A3E, 1010796},
  {0x11A3F, 1011007},
  {0x11A40, 1010679},
  {0x11A41, 1012269},
  {0x11A42, 1012242},
  {0x11A43, 1012175},
  {0x11A44, 1012209},
  {0x11A45, 1010959},
  {0x11A46, 1010631},
  {0x11A47, 1012512},
  {0x11A50, 897773},
  {0x11A51, 898849},
  {0x11A52, 898934},
  {0x11A53, 898913},
  {0x11A54, 898828},
  {0x11A55, 898870},
  {0x11A56, 898891},
  {0x11A57, 898784},
  {0x11A58, 898806},
  {0x11A59, 898985},
  {0x11A5A, 898956},
  {0x11A5B, 898758},
  {0x11A5C, 898051},
  {0x11A5D, 898069},
  {0x11A5E, 897959},
  {0x11A5F, 897977},
  {0x11A60, 898162},
  {0x11A61, 897827},
  {0x11A62, 897845},
  {0x11A63, 898014},
  {0x11A64, 898032},
  {0x11A65, 898200},
  {0x11A66, 898406},
  {0x11A67, 898425},
  {0x11A68, 897882},
  {0x11A69, 897901},
  {0x11A6A, 898181},
  {0x11A6B, 898330},
  {0x11A6C, 898348},
  {0x11A6D, 897864},
  {0x11A6E, 897921},
  {0x11A6F, 898144},
  {0x11A70, 898219},
  {0x11A71, 898237},
  {0x11A72, 897790},
  {0x11A73, 897808},
  {0x11A74, 898126},
  {0x11A75, 898367},
  {0x11A76, 898386},
  {0x11A77, 897940},
  {0x11A78, 898499},
  {0x11A79, 898481},
  {0x11A7A, 897755},
  {0x11A7B, 898463},
  {0x11A7C, 898256},
  {0x11A7D, 898108},
  {0x11A7E, 898445},
  {0x11A7F, 898292},
  {0x11A80, 898311},
  {0x11A81, 898274},
  {0x11A82, 897996},
  {0x11A83, 898088},
  {0x11A84, 898621},
  {0x11A85, 898646},
  {0x11A86, 897118},
  {0x11A87, 897084},
  {0x11A88, 897186},
  {0x11A89, 897152},
  {0x11A8A, 897315},
  {0x11A8B, 897346},
  {0x11A8C, 897470},
  {0x11A8D, 897284},
  {0x11A8E, 897439},
  {0x11A8F, 897253},
  {0x11A90, 897408},
  {0x11A91, 897502},
  {0x11A92, 897377},
  {0x11A93, 897564},
  {0x11A94, 897533},
  {0x11A95, 897221},
  {0x11A96, 898599},
  {0x11A97, 898671},
  {0x11A98, 897596},
  {0x11A99, 898692},
  {0x11A9A, 898580},
  {0x11A9B, 898562},
  {0x11A9C, 898518},
  {0x11A9D, 898543},
  {0x11A9E, 897702},
  {0x11A9F, 897656},
  {0x11AA0, 897620},
  {0x11AA1, 898710},
  {0x11AA2, 898734},
  {0x11AB0, 163289},
  {0x11AB1, 163320},
  {0x11AB2, 163352},
  {0x11AB3, 163383},
  {0x11AB4, 163226},
  {0x11AB5, 163257},
  {0x11AB6, 163482},
  {0x11AB7, 163515},
  {0x11AB8, 163549},
  {0x11AB9, 163582},
  {0x11ABA, 163415},
  {0x11ABB, 163448},
  {0x11ABC, 167494},
  {0x11ABD, 167517},
  {0x11ABE, 167540},
  {0x11ABF, 167471},
  {0x11AC0, 818386},
  {0x11AC1, 818231},
  {0x11AC2, 818276},
  {0x11AC3, 818298},
  {0x11AC4, 817835},
  {0x11AC5, 818585},
  {0x11AC6, 818563},
  {0x11AC7, 818342},
  {0x11AC8, 818166},
  {0x11AC9, 818144},
  {0x11ACA, 818253},
  {0x11ACB, 818453},
  {0x11ACC, 817768},
  {0x11ACD, 817790},
  {0x11ACE, 818475},
  {0x11ACF, 818497},
  {0x11AD0, 818320},
  {0x11AD1, 818408},
  {0x11AD2, 818431},
  {0x11AD3, 817878},
  {0x11AD4, 817812},
  {0x11AD5, 817747},
  {0x11AD6, 817857},
  {0x11AD7, 818188},
  {0x11AD8, 818365},
  {0x11AD9, 818520},
  {0x11ADA, 818541},
  {0x11ADB, 818209},
  {0x11ADC, 818036},
  {0x11ADD, 817900},
  {0x11ADE, 818063},
  {0x11ADF, 817954},
  {0x11AE0, 817981},
  {0x11AE1, 817927},
  {0x11AE2, 818090},
  {0x11AE3, 818008},
  {0x11AE4, 818117},
  {0x11AE5, 818897},
  {0x11AE6, 818843},
  {0x11AE7, 818961},
  {0x11AE8, 818926},
  {0x11AE9, 818867},
  {0x11AEA, 818993},
  {0x11AEB, 819085},
  {0x11AEC, 819031},
  {0x11AED, 819114},
  {0x11AEE, 819055},
  {0x11AEF, 818745},
  {0x11AF0, 817714},
  {0x11AF1, 818805},
  {0x11AF2, 818772},
  {0x11AF3, 818671},
  {0x11AF4, 818607},
  {0x11AF5, 817658},
  {0x11AF6, 818705},
  {0x11AF7, 818636},
  {0x11AF8, 817683},
  {0x11B00, 272963},
  {0x11B01, 272984},
  {0x11B02, 274910},
  {0x11B03, 274932},
  {0x11B04, 275172},
  {0x11B05, 275203},
  {0x11B06, 275531},
  {0x11B07, 275571},
  {0x11B08, 275383},
  {0x11B09, 275314},
  {0x11BC0, 910400},
  {0x11BC1, 910800},
  {0x11BC2, 910441},
  {0x11BC3, 910500},
  {0x11BC4, 910738},
  {0x11BC5, 910884},
  {0x11BC6, 910559},
  {0x11BC7, 910621},
  {0x11BC8, 910298},
  {0x11BC9, 910719},
  {0x11BCA, 910460},
  {0x11BCB, 910479},
  {0x11BCC, 910357},
  {0x11BCD, 910639},
  {0x11BCE, 910338},
  {0x11BCF, 910520},
  {0x11BD0, 910599},
  {0x11BD1, 910678},
  {0x11BD2, 910757},
  {0x11BD3, 910905},
  {0x11BD4, 910926},
  {0x11BD5, 910319},
  {0x11BD6, 910279},
  {0x11BD7, 910420},
  {0x11BD8, 910842},
  {0x11BD9, 910699},
  {0x11BDA, 910658},
  {0x11BDB, 910540},
  {0x11BDC, 910779},
  {0x11BDD, 910378},
  {0x11BDE, 910821},
  {0x11BDF, 910863},
  {0x11BE0, 910578},
  {0x11BE1, 910945},
  {0x11BF0, 910260},
  {0x11BF1, 910166},
  {0x11BF2, 910242},
  {0x11BF3, 910222},
  {0x11BF4, 910128},
  {0x11BF5, 910109},
  {0x11BF6, 910204},
  {0x11BF7, 910184},
  {0x11BF8, 910089},
  {0x11BF9, 910147},
  {0x11C00, 109397},
  {0x11C01, 109416},
  {0x11C02, 109722},
  {0x11C03, 109741},
  {0x11C04, 110192},
  {0x11C05, 110211},
  {0x11C06, 110278},
  {0x11C07, 110305},
  {0x11C08, 110251},
  {0x11C0A, 109642},
  {0x11C0B, 109436},
  {0x11C0C, 109966},
  {0x11C0D, 109456},
  {0x11C0E, 109802},
  {0x11C0F, 109822},
  {0x11C10, 109661},
  {0x11C11, 109681},
  {0x11C12, 109903},
  {0x11C13, 109517},
  {0x11C14, 109537},
  {0x11C15, 109761},
  {0x11C16, 109781},
  {0x11C17, 109945},
  {0x11C18, 110149},
  {0x11C19, 110170},
  {0x11C1A, 109578},
  {0x11C1B, 109599},
  {0x11C1C, 109924},
  {0x11C1D, 110108},
  {0x11C1E, 110128},
  {0x11C1F, 109558},
  {0x11C20, 109621},
  {0x11C21, 109883},
  {0x11C22, 109985},
  {0x11C23, 110005},
  {0x11C24, 109476},
  {0x11C25, 109496},
  {0x11C26, 109863},
  {0x11C27, 110333},
  {0x11C28, 110026},
  {0x11C29, 109843},
  {0x11C2A, 110231},
  {0x11C2B, 110066},
  {0x11C2C, 110087},
  {0x11C2D, 110046},
  {0x11C2E, 109702},
  {0x11C2F, 110882},
  {0x11C30, 110977},
  {0x11C31, 111000},
  {0x11C32, 111047},
  {0x11C33, 111070},
  {0x11C34, 111125},
  {0x11C35, 111156},
  {0x11C36, 111094},
  {0x11C38, 110954},
  {0x11C39, 110906},
  {0x11C3A, 111024},
  {0x11C3B, 110930},
  {0x11C3C, 110810},
  {0x11C3D, 110762},
  {0x11C3E, 110859},
  {0x11C3F, 110837},
  {0x11C40, 110786},
  {0x11C41, 109073},
  {0x11C42, 109299},
  {0x11C43, 111188},
  {0x11C44, 109322},
  {0x11C45, 109345},
  {0x11C50, 109278},
  {0x11C51, 109174},
  {0x11C52, 109258},
  {0x11C53, 109236},
  {0x11C54, 109132},
  {0x11C55, 109111},
  {0x11C56, 109216},
  {0x11C57, 109194},
  {0x11C58, 109089},
  {0x11C59, 109153},
  {0x11C5A, 110536},
  {0x11C5B, 110741},
  {0x11C5C, 110694},
  {0x11C5D, 110468},
  {0x11C5E, 110423},
  {0x11C5F, 110605},
  {0x11C60, 110557},
  {0x11C61, 110353},
  {0x11C62, 110490},
  {0x11C63, 110649},
  {0x11C64, 110717},
  {0x11C65, 110670},
  {0x11C66, 110445},
  {0x11C67, 110400},
  {0x11C68, 110626},
  {0x11C69, 110580},
  {0x11C6A, 110376},
  {0x11C6B, 110512},
  {0x11C6C, 109368},
  {0x11C70, 665320},
  {0x11C71, 665889},
  {0x11C72, 665519},
  {0x11C73, 665537},
  {0x11C74, 665465},
  {0x11C75, 665610},
  {0x11C76, 665391},
  {0x11C77, 665409},
  {0x11C78, 665501},
  {0x11C79, 665629},
  {0x11C7A, 665740},
  {0x11C7B, 665758},
  {0x11C7C, 665428},
  {0x11C7D, 665592},
  {0x11C7E, 665648},
  {0x11C7F, 665666},
  {0x11C80, 665373},
  {0x11C81, 665574},
  {0x11C82, 665777},
  {0x11C83, 665796},
  {0x11C84, 665446},
  {0x11C85, 665816},
  {0x11C86, 665870},
  {0x11C87, 665852},
  {0x11C88, 665338},
  {0x11C89, 665834},
  {0x11C8A, 665685},
  {0x11C8B, 665556},
  {0x11C8C, 665721},
  {0x11C8D, 665703},
  {0x11C8E, 665483},
  {0x11C8F, 665356},
  {0x11C92, 666207},
  {0x11C93, 666235},
  {0x11C94, 666123},
  {0x11C95, 666348},
  {0x11C96, 666009},
  {0x11C97, 666037},
  {0x11C98, 666179},
  {0x11C99, 666377},
  {0x11C9A, 666548},
  {0x11C9B, 666576},
  {0x11C9C, 666066},
  {0x11C9D, 666320},
  {0x11C9E, 666406},
  {0x11C9F, 666434},
  {0x11CA0, 665981},
  {0x11CA1, 666292},
  {0x11CA2, 666605},
  {0x11CA3, 666634},
  {0x11CA4, 666094},
  {0x11CA5, 666664},
  {0x11CA6, 666748},
  {0x11CA7, 666720},
  {0x11CA9, 666692},
  {0x11CAA, 666463},
  {0x11CAB, 666264},
  {0x11CAC, 666519},
  {0x11CAD, 666491},
  {0x11CAE, 666151},
  {0x11CAF, 665954},
  {0x11CB0, 666777},
  {0x11CB1, 666820},
  {0x11CB2, 666862},
  {0x11CB3, 666799},
  {0x11CB4, 666841},
  {0x11CB5, 665907},
  {0x11CB6, 665929},
  {0x11D00, 667170},
  {0x11D01, 667193},
  {0x11D02, 667559},
  {0x11D03, 667582},
  {0x11D04, 668223},
  {0x11D05, 668246},
  {0x11D06, 667463},
  {0x11D08, 667217},
  {0x11D09, 667928},
  {0x11D0B, 667241},
  {0x11D0C, 667681},
  {0x11D0D, 667705},
  {0x11D0E, 667486},
  {0x11D0F, 667510},
  {0x11D10, 667853},
  {0x11D11, 667314},
  {0x11D12, 667338},
  {0x11D13, 667606},
  {0x11D14, 667630},
  {0x11D15, 667903},
  {0x11D16, 668172},
  {0x11D17, 668197},
  {0x11D18, 667387},
  {0x11D19, 667412},
  {0x11D1A, 667878},
  {0x11D1B, 668098},
  {0x11D1C, 668122},
  {0x11D1D, 667363},
  {0x11D1E, 667438},
  {0x11D1F, 667829},
  {0x11D20, 667951},
  {0x11D21, 667975},
  {0x11D22, 667265},
  {0x11D23, 667289},
  {0x11D24, 667805},
  {0x11D25, 668294},
  {0x11D26, 668000},
  {0x11D27, 667756},
  {0x11D28, 668270},
  {0x11D29, 668048},
  {0x11D2A, 668073},
  {0x11D2B, 668024},
  {0x11D2C, 667535},
  {0x11D2D, 667780},
  {0x11D2E, 667730},
  {0x11D2F, 667655},
  {0x11D30, 668147},
  {0x11D31, 668514},
  {0x11D32, 668625},
  {0x11D33, 668652},
  {0x11D34, 668707},
  {0x11D35, 668734},
  {0x11D36, 668762},
  {0x11D3A, 668598},
  {0x11D3C, 668542},
  {0x11D3D, 668680},
  {0x11D3F, 668570},
  {0x11D40, 668360},
  {0x11D41, 668466},
  {0x11D42, 668441},
  {0x11D43, 668388},
  {0x11D44, 668414},
  {0x11D45, 668493},
  {0x11D46, 668340},
  {0x11D47, 668318},
  {0x11D50, 667145},
  {0x11D51, 667021},
  {0x11D52, 667121},
  {0x11D53, 667095},
  {0x11D54, 666971},
  {0x11D55, 666946},
  {0x11D56, 667071},
  {0x11D57, 667045},
  {0x11D58, 666920},
  {0x11D59, 666996},
  {0x11D60, 483020},
  {0x11D61, 483043},
  {0x11D62, 483410},
  {0x11D63, 483433},
  {0x11D64, 483898},
  {0x11D65, 483921},
  {0x11D67, 483313},
  {0x11D68, 483067},
  {0x11D6A, 483677},
  {0x11D6B, 483091},
  {0x11D6C, 483969},
  {0x11D6D, 483945},
  {0x11D6E, 483115},
  {0x11D6F, 483139},
  {0x11D70, 483604},
  {0x11D71, 483506},
  {0x11D72, 483530},
  {0x11D73, 483798},
  {0x11D74, 483822},
  {0x11D75, 483555},
  {0x11D76, 483337},
  {0x11D77, 483361},
  {0x11D78, 483213},
  {0x11D79, 483288},
  {0x11D7A, 483628},
  {0x11D7B, 483164},
  {0x11D7C, 483188},
  {0x11D7D, 483847},
  {0x11D7E, 483872},
  {0x11D7F, 483579},
  {0x11D80, 483457},
  {0x11D81, 483481},
  {0x11D82, 483237},
  {0x11D83, 483262},
  {0x11D84, 483652},
  {0x11D85, 483701},
  {0x11D86, 483725},
  {0x11D87, 483386},
  {0x11D88, 483750},
  {0x11D89, 483774},
  {0x11D8A, 484086},
  {0x11D8B, 484198},
  {0x11D8C, 484225},
  {0x11D8D, 484281},
  {0x11D8E, 484308},
  {0x11D90, 484170},
  {0x11D91, 484114},
  {0x11D93, 484253},
  {0x11D94, 484142},
  {0x11D95, 484010},
  {0x11D96, 484038},
  {0x11D97, 484065},
  {0x11D98, 483993},
  {0x11DA0, 482995},
  {0x11DA1, 482871},
  {0x11DA2, 482971},
  {0x11DA3, 482945},
  {0x11DA4, 482821},
  {0x11DA5, 482796},
  {0x11DA6, 482921},
  {0x11DA7, 482895},
  {0x11DA8, 482770},
  {0x11DA9, 482846},
  {0x11EE0, 659921},
  {0x11EE1, 659885},
  {0x11EE2, 659993},
  {0x11EE3, 660031},
  {0x11EE4, 659831},
  {0x11EE5, 659957},
  {0x11EE6, 660085},
  {0x11EE7, 659867},
  {0x11EE8, 659975},
  {0x11EE9, 659849},
  {0x11EEA, 659903},
  {0x11EEB, 660012},
  {0x11EEC, 660121},
  {0x11EED, 660049},
  {0x11EEE, 659939},
  {0x11EEF, 660103},
  {0x11EF0, 660067},
  {0x11EF1, 659814},
  {0x11EF2, 659777},
  {0x11EF3, 660179},
  {0x11EF4, 660221},
  {0x11EF5, 660158},
  {0x11EF6, 660200},
  {0x11EF7, 660139},
  {0x11EF8, 659791},
  {0x11F00, 551130},
  {0x11F01, 551111},
  {0x11F02, 551185},
  {0x11F03, 551201},
  {0x11F04, 550044},
  {0x11F05, 550058},
  {0x11F06, 550274},
  {0x11F07, 550288},
  {0x11F08, 550646},
  {0x11F09, 550660},
  {0x11F0A, 550720},
  {0x11F0B, 550742},
  {0x11F0C, 550675},
  {0x11F0D, 550697},
  {0x11F0E, 550214},
  {0x11F0F, 550073},
  {0x11F10, 550475},
  {0x11F12, 550351},
  {0x11F13, 550366},
  {0x11F14, 550228},
  {0x11F15, 550243},
  {0x11F16, 550427},
  {0x11F17, 550119},
  {0x11F18, 550134},
  {0x11F19, 550303},
  {0x11F1A, 550318},
  {0x11F1B, 550459},
  {0x11F1C, 550613},
  {0x11F1D, 550629},
  {0x11F1E, 550165},
  {0x11F1F, 550181},
  {0x11F20, 550443},
  {0x11F21, 550582},
  {0x11F22, 550597},
  {0x11F23, 550150},
  {0x11F24, 550198},
  {0x11F25, 550412},
  {0x11F26, 550489},
  {0x11F27, 550504},
  {0x11F28, 550088},
  {0x11F29, 550103},
  {0x11F2A, 550397},
  {0x11F2B, 550780},
  {0x11F2C, 550520},
  {0x11F2D, 550382},
  {0x11F2E, 550765},
  {0x11F2F, 550550},
  {0x11F30, 550566},
  {0x11F31, 550535},
  {0x11F32, 550259},
  {0x11F33, 550334},
  {0x11F34, 551219},
  {0x11F35, 551257},
  {0x11F36, 551323},
  {0x11F37, 551341},
  {0x11F38, 551360},
  {0x11F39, 551378},
  {0x11F3A, 551397},
  {0x11F3E, 551286},
  {0x11F3F, 551238},
  {0x11F40, 551304},
  {0x11F41, 551152},
  {0x11F42, 549840},
  {0x11F43, 549855},
  {0x11F44, 550026},
  {0x11F45, 550997},
  {0x11F46, 550795},
  {0x11F47, 550973},
  {0x11F48, 551029},
  {0x11F49, 550893},
  {0x11F4A, 550914},
  {0x11F4B, 551083},
  {0x11F4C, 550837},
  {0x11F4D, 550942},
  {0x11F4E, 551059},
  {0x11F4F, 550861},
  {0x11F50, 550010},
  {0x11F51, 549931},
  {0x11F52, 549995},
  {0x11F53, 549978},
  {0x11F54, 549899},
  {0x11F55, 549883},
  {0x11F56, 549963},
  {0x11F57, 549946},
  {0x11F58, 549866},
  {0x11F59, 549915},
  {0x11F5A, 551169},
  {0x11FB0, 651531},
  {0x11FC0, 926399},
  {0x11FC1, 926176},
  {0x11FC2, 926042},
  {0x11FC3, 926309},
  {0x11FC4, 926096},
  {0x11FC5, 926366},
  {0x11FC6, 926475},
  {0x11FC7, 926569},
  {0x11FC8, 926446},
  {0x11FC9, 926247},
  {0x11FCA, 926278},
  {0x11FCB, 926341},
  {0x11FCC, 926016},
  {0x11FCD, 926604},
  {0x11FCE, 926537},
  {0x11FCF, 926071},
  {0x11FD0, 926220},
  {0x11FD1, 926124},
  {0x11FD2, 926150},
  {0x11FD3, 926507},
  {0x11FD4, 925976},
  {0x11FD5, 927599},
  {0x11FD6, 927464},
  {0x11FD7, 927423},
  {0x11FD8, 927684},
  {0x11FD9, 927576},
  {0x11FDA, 927499},
  {0x11FDB, 927649},
  {0x11FDC, 927534},
  {0x11FDD, 927482},
  {0x11FDE, 927632},
  {0x11FDF, 927669},
  {0x11FE0, 927725},
  {0x11FE1, 927614},
  {0x11FE2, 927517},
  {0x11FE3, 927745},
  {0x11FE4, 928133},
  {0x11FE5, 925949},
  {0x11FE6, 926661},
  {0x11FE7, 927403},
  {0x11FE8, 927857},
  {0x11FE9, 927887},
  {0x11FEA, 925728},
  {0x11FEB, 925650},
  {0x11FEC, 927798},
  {0x11FED, 927840},
  {0x11FEE, 926636},
  {0x11FEF, 927815},
  {0x11FF0, 927555},
  {0x11FF1, 927703},
  {0x11FFF, 927356},
  {0x12000, 219679},
  {0x12001, 219696},
  {0x12002, 219721},
  {0x12003, 219748},
  {0x12004, 219781},
  {0x12005, 219807},
  {0x12006, 219834},
  {0x12007, 219868},
  {0x12008, 219896},
  {0x12009, 219923},
  {0x1200A, 219941},
  {0x1200B, 219982},
  {0x1200C, 220011},
  {0x1200D, 220045},
  {0x1200E, 220073},
  {0x1200F, 220107},
  {0x12010, 220134},
  {0x12011, 220167},
  {0x12012, 220196},
  {0x12013, 220287},
  {0x12014, 220346},
  {0x12015, 219959},
  {0x12016, 220386},
  {0x12017, 220432},
  {0x12018, 220463},
  {0x12019, 220498},
  {0x1201A, 220534},
  {0x1201B, 220564},
  {0x1201C, 220594},
  {0x1201D, 220641},
  {0x1201E, 220659},
  {0x1201F, 220689},
  {0x12020, 220729},
  {0x12021, 220747},
  {0x12022, 220774},
  {0x12023, 220803},
  {0x12024, 220832},
  {0x12025, 220859},
  {0x12026, 220888},
  {0x12027, 220915},
  {0x12028, 220943},
  {0x12029, 220971},
  {0x1202A, 220991},
  {0x1202B, 221012},
  {0x1202C, 221062},
  {0x1202D, 221092},
  {0x1202E, 221110},
  {0x1202F, 221222},
  {0x12030, 221136},
  {0x12031, 221186},
  {0x12032, 221252},
  {0x12033, 221273},
  {0x12034, 221293},
  {0x12035, 221313},
  {0x12036, 221343},
  {0x12037, 221364},
  {0x12038, 221385},
  {0x12039, 221600},
  {0x1203A, 221404},
  {0x1203B, 221538},
  {0x1203C, 221433},
  {0x1203D, 221470},
  {0x1203E, 221629},
  {0x1203F, 221649},
  {0x12040, 221671},
  {0x12041, 221689},
  {0x12042, 221743},
  {0x12043, 221763},
  {0x12044, 221879},
  {0x12045, 221898},
  {0x12046, 221926},
  {0x12047, 221947},
  {0x12048, 221966},
  {0x12049, 221987},
  {0x1204A, 222005},
  {0x1204B, 222031},
  {0x1204C, 222059},
  {0x1204D, 222092},
  {0x1204E, 222140},
  {0x1204F, 222205},
  {0x12050, 222110},
  {0x12051, 222234},
  {0x12052, 222255},
  {0x12053, 222287},
  {0x12054, 222306},
  {0x12055, 222326},
  {0x12056, 222373},
  {0x12057, 222392},
  {0x12058, 222436},
  {0x12059, 222473},
  {0x1205A, 222511},
  {0x1205B, 222546},
  {0x1205C, 222581},
  {0x1205D, 222626},
  {0x1205E, 222661},
  {0x1205F, 222698},
  {0x12060, 222734},
  {0x12061, 222769},
  {0x12062, 222804},
  {0x12063, 222847},
  {0x12064, 222883},
  {0x12065, 222918},
  {0x12066, 222953},
  {0x12067, 222999},
  {0x12068, 223035},
  {0x12069, 223070},
  {0x1206A, 223115},
  {0x1206B, 223150},
  {0x1206C, 223187},
  {0x1206D, 223277},
  {0x1206E, 223362},
  {0x1206F, 223381},
  {0x12070, 223400},
  {0x12071, 223421},
  {0x12072, 223442},
  {0x12073, 223460},
  {0x12074, 223479},
  {0x12075, 223527},
  {0x12076, 223587},
  {0x12077, 223636},
  {0x12078, 223655},
  {0x12079, 223693},
  {0x1207A, 223742},
  {0x1207B, 223783},
  {0x1207C, 223760},
  {0x1207D, 223809},
  {0x1207E, 223835},
  {0x1207F, 223854},
  {0x12080, 223913},
  {0x12081, 223933},
  {0x12082, 224831},
  {0x12083, 224852},
  {0x12084, 224871},
  {0x12085, 224890},
  {0x12086, 224910},
  {0x12087, 224935},
  {0x12088, 224965},
  {0x12089, 224985},
  {0x1208A, 225005},
  {0x1208B, 225059},
  {0x1208C, 225022},
  {0x1208D, 225086},
  {0x1208E, 225104},
  {0x1208F, 225146},
  {0x12090, 225202},
  {0x12091, 225257},
  {0x12092, 225285},
  {0x12093, 225313},
  {0x12094, 225339},
  {0x12095, 225359},
  {0x12096, 225379},
  {0x12097, 225397},
  {0x12098, 225501},
  {0x12099, 225530},
  {0x1209A, 225564},
  {0x1209B, 225415},
  {0x1209C, 225445},
  {0x1209D, 225475},
  {0x1209E, 225591},
  {0x1209F, 225611},
  {0x120A0, 225655},
  {0x120A1, 225709},
  {0x120A2, 226034},
  {0x120A3, 226062},
  {0x120A4, 226099},
  {0x120A5, 226146},
  {0x120A6, 226175},
  {0x120A7, 226205},
  {0x120A8, 226241},
  {0x120A9, 226312},
  {0x120AA, 226341},
  {0x120AB, 226375},
  {0x120AC, 226410},
  {0x120AD, 226443},
  {0x120AE, 226484},
  {0x120AF, 226514},
  {0x120B0, 226543},
  {0x120B1, 226583},
  {0x120B2, 226612},
  {0x120B3, 226700},
  {0x120B4, 226729},
  {0x120B5, 226758},
  {0x120B6, 226776},
  {0x120B7, 226799},
  {0x120B8, 226846},
  {0x120B9, 226889},
  {0x120BA, 226924},
  {0x120BB, 226960},
  {0x120BC, 227003},
  {0x120BD, 227075},
  {0x120BE, 227134},
  {0x120BF, 227173},
  {0x120C0, 227234},
  {0x120C1, 227271},
  {0x120C2, 227300},
  {0x120C3, 227337},
  {0x120C4, 227365},
  {0x120C5, 227427},
  {0x120C6, 227466},
  {0x120C7, 227581},
  {0x120C8, 227609},
  {0x120C9, 227645},
  {0x120CA, 227673},
  {0x120CB, 227775},
  {0x120CC, 227810},
  {0x120CD, 227913},
  {0x120CE, 227941},
  {0x120CF, 227970},
  {0x120D0, 228006},
  {0x120D1, 228079},
  {0x120D2, 228125},
  {0x120D3, 228154},
  {0x120D4, 228191},
  {0x120D5, 228227},
  {0x120D6, 228257},
  {0x120D7, 228291},
  {0x120D8, 228337},
  {0x120D9, 228366},
  {0x120DA, 228398},
  {0x120DB, 228427},
  {0x120DC, 228465},
  {0x120DD, 228546},
  {0x120DE, 228671},
  {0x120DF, 228707},
  {0x120E0, 228865},
  {0x120E1, 228894},
  {0x120E2, 228932},
  {0x120E3, 228960},
  {0x120E4, 228989},
  {0x120E5, 229018},
  {0x120E6, 229047},
  {0x120E7, 229085},
  {0x120E8, 229145},
  {0x120E9, 229174},
  {0x120EA, 229204},
  {0x120EB, 229231},
  {0x120EC, 229259},
  {0x120ED, 226818},
  {0x120EE, 229325},
  {0x120EF, 229345},
  {0x120F0, 229422},
  {0x120F1, 229441},
  {0x120F2, 229482},
  {0x120F3, 229501},
  {0x120F4, 229546},
  {0x120F5, 229567},
  {0x120F6, 229586},
  {0x120F7, 229605},
  {0x120F8, 229689},
  {0x120F9, 229659},
  {0x120FA, 229625},
  {0x120FB, 229714},
  {0x120FC, 229733},
  {0x120FD, 229753},
  {0x120FE, 229795},
  {0x120FF, 229818},
  {0x12100, 229882},
  {0x12101, 229930},
  {0x12102, 229956},
  {0x12103, 229900},
  {0x12104, 229982},
  {0x12105, 230033},
  {0x12106, 230001},
  {0x12107, 230061},
  {0x12108, 230101},
  {0x12109, 230121},
  {0x1210A, 230146},
  {0x1210B, 230166},
  {0x1210C, 230203},
  {0x1210D, 230239},
  {0x1210E, 230269},
  {0x1210F, 230307},
  {0x12110, 230336},
  {0x12111, 230357},
  {0x12112, 230377},
  {0x12113, 230436},
  {0x12114, 230511},
  {0x12115, 230411},
  {0x12116, 230542},
  {0x12117, 230560},
  {0x12118, 230590},
  {0x12119, 230667},
  {0x1211A, 230696},
  {0x1211B, 230740},
  {0x1211C, 230769},
  {0x1211D, 230609},
  {0x1211E, 230808},
  {0x1211F, 230901},
  {0x12120, 230937},
  {0x12121, 230827},
  {0x12122, 230966},
  {0x12123, 230985},
  {0x12124, 231004},
  {0x12125, 231033},
  {0x12126, 231052},
  {0x12127, 231072},
  {0x12128, 231093},
  {0x12129, 231115},
  {0x1212A, 231156},
  {0x1212B, 231133},
  {0x1212C, 231207},
  {0x1212D, 231226},
  {0x1212E, 231244},
  {0x1212F, 231318},
  {0x12130, 231347},
  {0x12131, 231375},
  {0x12132, 231404},
  {0x12133, 231432},
  {0x12134, 231460},
  {0x12135, 231488},
  {0x12136, 231516},
  {0x12137, 231542},
  {0x12138, 231560},
  {0x12139, 231580},
  {0x1213A, 231609},
  {0x1213B, 231639},
  {0x1213C, 231672},
  {0x1213D, 231703},
  {0x1213E, 231732},
  {0x1213F, 231772},
  {0x12140, 231789},
  {0x12141, 231808},
  {0x12142, 231826},
  {0x12143, 231846},
  {0x12144, 231880},
  {0x12145, 231918},
  {0x12146, 231936},
  {0x12147, 231955},
  {0x12148, 232056},
  {0x12149, 232002},
  {0x1214A, 231978},
  {0x1214B, 232078},
  {0x1214C, 232096},
  {0x1214D, 232130},
  {0x1214E, 232149},
  {0x1214F, 232253},
  {0x12150, 232167},
  {0x12151, 232197},
  {0x12152, 232227},
  {0x12153, 232282},
  {0x12154, 232302},
  {0x12155, 232320},
  {0x12156, 232338},
  {0x12157, 232357},
  {0x12158, 232375},
  {0x12159, 232401},
  {0x1215A, 232428},
  {0x1215B, 232494},
  {0x1215C, 232552},
  {0x1215D, 232580},
  {0x1215E, 232610},
  {0x1215F, 232638},
  {0x12160, 232692},
  {0x12161, 232722},
  {0x12162, 232751},
  {0x12163, 232778},
  {0x12164, 232806},
  {0x12165, 232840},
  {0x12166, 232868},
  {0x12167, 232913},
  {0x12168, 232940},
  {0x12169, 233041},
  {0x1216A, 232998},
  {0x1216B, 233079},
  {0x1216C, 233134},
  {0x1216D, 233201},
  {0x1216E, 233229},
  {0x1216F, 233283},
  {0x12170, 233311},
  {0x12171, 233338},
  {0x12172, 233366},
  {0x12173, 233393},
  {0x12174, 233448},
  {0x12175, 233475},
  {0x12176, 233510},
  {0x12177, 233545},
  {0x12178, 233580},
  {0x12179, 233607},
  {0x1217A, 233645},
  {0x1217B, 233672},
  {0x1217C, 233727},
  {0x1217D, 233754},
  {0x1217E, 233781},
  {0x1217F, 233808},
  {0x12180, 233836},
  {0x12181, 233864},
  {0x12182, 233892},
  {0x12183, 233921},
  {0x12184, 233978},
  {0x12185, 234006},
  {0x12186, 234036},
  {0x12187, 234091},
  {0x12188, 234147},
  {0x12189, 234174},
  {0x1218A, 234201},
  {0x1218B, 234294},
  {0x1218C, 234322},
  {0x1218D, 234349},
  {0x1218E, 234368},
  {0x1218F, 234400},
  {0x12190, 234419},
  {0x12191, 234439},
  {0x12192, 234459},
  {0x12193, 234479},
  {0x12194, 234499},
  {0x12195, 234529},
  {0x12196, 234548},
  {0x12197, 234582},
  {0x12198, 234633},
  {0x12199, 234601},
  {0x1219A, 234662},
  {0x1219B, 234682},
  {0x1219C, 234729},
  {0x1219D, 234751},
  {0x1219E, 234806},
  {0x1219F, 234873},
  {0x121A0, 234894},
  {0x121A1, 234912},
  {0x121A2, 234940},
  {0x121A3, 234966},
  {0x121A4, 234993},
  {0x121A5, 235012},
  {0x121A6, 235031},
  {0x121A7, 235052},
  {0x121A8, 235072},
  {0x121A9, 235094},
  {0x121AA, 235128},
  {0x121AB, 235146},
  {0x121AC, 235205},
  {0x121AD, 235224},
  {0x121AE, 235243},
  {0x121AF, 235275},
  {0x121B0, 235294},
  {0x121B1, 235313},
  {0x121B2, 235337},
  {0x121B3, 235356},
  {0x121B4, 235375},
  {0x121B5, 235407},
  {0x121B6, 235429},
  {0x121B7, 235451},
  {0x121B8, 235469},
  {0x121B9, 235519},
  {0x121BA, 235548},
  {0x121BB, 235593},
  {0x121BC, 235631},
  {0x121BD, 235669},
  {0x121BE, 235699},
  {0x121BF, 235729},
  {0x121C0, 235770},
  {0x121C1, 235801},
  {0x121C2, 235831},
  {0x121C3, 235862},
  {0x121C4, 235892},
  {0x121C5, 235922},
  {0x121C6, 235983},
  {0x121C7, 236014},
  {0x121C8, 236054},
  {0x121C9, 236084},
  {0x121CA, 236115},
  {0x121CB, 236155},
  {0x121CC, 236191},
  {0x121CD, 236221},
  {0x121CE, 236259},
  {0x121CF, 236297},
  {0x121D0, 236327},
  {0x121D1, 236358},
  {0x121D2, 236389},
  {0x121D3, 236420},
  {0x121D4, 236466},
  {0x121D5, 236499},
  {0x121D6, 236531},
  {0x121D7, 236561},
  {0x121D8, 236592},
  {0x121D9, 236622},
  {0x121DA, 236660},
  {0x121DB, 236692},
  {0x121DC, 236722},
  {0x121DD, 236762},
  {0x121DE, 236816},
  {0x121DF, 236864},
  {0x121E0, 236896},
  {0x121E1, 236938},
  {0x121E2, 236969},
  {0x121E3, 237000},
  {0x121E4, 237032},
  {0x121E5, 237085},
  {0x121E6, 237114},
  {0x121E7, 237150},
  {0x121E8, 237193},
  {0x121E9, 237232},
  {0x121EA, 237262},
  {0x121EB, 235490},
  {0x121EC, 237331},
  {0x121ED, 237424},
  {0x121EE, 237455},
  {0x121EF, 237352},
  {0x121F0, 237378},
  {0x121F1, 237495},
  {0x121F2, 239426},
  {0x121F3, 239445},
  {0x121F4, 239474},
  {0x121F5, 239493},
  {0x121F6, 239522},
  {0x121F7, 239559},
  {0x121F8, 239577},
  {0x121F9, 239596},
  {0x121FA, 239618},
  {0x121FB, 239638},
  {0x121FC, 239656},
  {0x121FD, 239684},
  {0x121FE, 239916},
  {0x121FF, 239944},
  {0x12200, 240003},
  {0x12201, 240072},
  {0x12202, 240107},
  {0x12203, 240171},
  {0x12204, 240209},
  {0x12205, 240237},
  {0x12206, 240267},
  {0x12207, 240297},
  {0x12208, 240336},
  {0x12209, 240364},
  {0x1220A, 240401},
  {0x1220B, 240432},
  {0x1220C, 240468},
  {0x1220D, 240496},
  {0x1220E, 240629},
  {0x1220F, 240666},
  {0x12210, 240734},
  {0x12211, 239892},
  {0x12212, 239703},
  {0x12213, 239769},
  {0x12214, 239865},
  {0x12215, 239801},
  {0x12216, 240764},
  {0x12217, 240783},
  {0x12218, 240840},
  {0x12219, 240804},
  {0x1221A, 240872},
  {0x1221B, 240901},
  {0x1221C, 240920},
  {0x1221D, 240939},
  {0x1221E, 240958},
  {0x1221F, 240986},
  {0x12220, 241027},
  {0x12221, 241068},
  {0x12222, 241045},
  {0x12223, 241097},
  {0x12224, 241116},
  {0x12225, 241135},
  {0x12226, 241154},
  {0x12227, 241174},
  {0x12228, 241195},
  {0x12229, 241213},
  {0x1222A, 241252},
  {0x1222B, 241297},
  {0x1222C, 241316},
  {0x1222D, 241334},
  {0x1222E, 241360},
  {0x1222F, 241379},
  {0x12230, 241403},
  {0x12231, 241425},
  {0x12232, 241447},
  {0x12233, 241656},
  {0x12234, 241684},
  {0x12235, 241714},
  {0x12236, 241501},
  {0x12237, 241531},
  {0x12238, 241467},
  {0x12239, 241743},
  {0x1223A, 241790},
  {0x1223B, 241819},
  {0x1223C, 241856},
  {0x1223D, 241764},
  {0x1223E, 241916},
  {0x1223F, 241934},
  {0x12240, 241972},
  {0x12241, 241992},
  {0x12242, 242055},
  {0x12243, 242021},
  {0x12244, 242090},
  {0x12245, 242130},
  {0x12246, 242111},
  {0x12247, 242157},
  {0x12248, 242177},
  {0x12249, 242221},
  {0x1224A, 242247},
  {0x1224B, 242195},
  {0x1224C, 242274},
  {0x1224D, 242292},
  {0x1224E, 242318},
  {0x1224F, 242337},
  {0x12250, 242356},
  {0x12251, 242391},
  {0x12252, 242474},
  {0x12253, 242496},
  {0x12254, 242527},
  {0x12255, 242559},
  {0x12256, 242733},
  {0x12257, 242931},
  {0x12258, 242977},
  {0x12259, 243008},
  {0x1225A, 243081},
  {0x1225B, 243113},
  {0x1225C, 243155},
  {0x1225D, 243196},
  {0x1225E, 243313},
  {0x1225F, 243387},
  {0x12260, 243419},
  {0x12261, 243440},
  {0x12262, 243458},
  {0x12263, 243521},
  {0x12264, 243621},
  {0x12265, 243656},
  {0x12266, 243692},
  {0x12267, 243727},
  {0x12268, 243787},
  {0x12269, 243850},
  {0x1226A, 243822},
  {0x1226B, 243540},
  {0x1226C, 243572},
  {0x1226D, 243874},
  {0x1226E, 243895},
  {0x1226F, 243933},
  {0x12270, 243967},
  {0x12271, 244002},
  {0x12272, 244037},
  {0x12273, 244077},
  {0x12274, 244113},
  {0x12275, 244147},
  {0x12276, 244181},
  {0x12277, 244218},
  {0x12278, 244252},
  {0x12279, 244289},
  {0x1227A, 244328},
  {0x1227B, 244346},
  {0x1227C, 244365},
  {0x1227D, 244384},
  {0x1227E, 244403},
  {0x1227F, 244454},
  {0x12280, 244502},
  {0x12281, 244528},
  {0x12282, 244555},
  {0x12283, 244582},
  {0x12284, 244609},
  {0x12285, 244635},
  {0x12286, 244661},
  {0x12287, 244688},
  {0x12288, 244714},
  {0x12289, 244472},
  {0x1228A, 244761},
  {0x1228B, 244818},
  {0x1228C, 244849},
  {0x1228D, 244879},
  {0x1228E, 244782},
  {0x1228F, 244909},
  {0x12290, 244927},
  {0x12291, 244946},
  {0x12292, 244964},
  {0x12293, 244982},
  {0x12294, 245076},
  {0x12295, 245000},
  {0x12296, 245131},
  {0x12297, 245158},
  {0x12298, 245186},
  {0x12299, 245243},
  {0x1229A, 245305},
  {0x1229B, 245334},
  {0x1229C, 245363},
  {0x1229D, 245392},
  {0x1229E, 245420},
  {0x1229F, 245449},
  {0x122A0, 245515},
  {0x122A1, 245545},
  {0x122A2, 245604},
  {0x122A3, 245632},
  {0x122A4, 245660},
  {0x122A5, 245688},
  {0x122A6, 245716},
  {0x122A7, 245103},
  {0x122A8, 245019},
  {0x122A9, 245745},
  {0x122AA, 245764},
  {0x122AB, 245800},
  {0x122AC, 245822},
  {0x122AD, 245841},
  {0x122AE, 245860},
  {0x122AF, 245880},
  {0x122B0, 245908},
  {0x122B1, 245938},
  {0x122B2, 245969},
  {0x122B3, 245998},
  {0x122B4, 246029},
  {0x122B5, 246059},
  {0x122B6, 246087},
  {0x122B7, 246122},
  {0x122B8, 246167},
  {0x122B9, 246188},
  {0x122BA, 246209},
  {0x122BB, 246228},
  {0x122BC, 246278},
  {0x122BD, 246332},
  {0x122BE, 246472},
  {0x122BF, 246493},
  {0x122C0, 246513},
  {0x122C1, 246534},
  {0x122C2, 246556},
  {0x122C3, 246580},
  {0x122C4, 246600},
  {0x122C5, 246628},
  {0x122C6, 246657},
  {0x122C7, 246677},
  {0x122C8, 246705},
  {0x122C9, 246735},
  {0x122CA, 246767},
  {0x122CB, 246797},
  {0x122CC, 246827},
  {0x122CD, 246857},
  {0x122CE, 246892},
  {0x122CF, 246925},
  {0x122D0, 246955},
  {0x122D1, 246985},
  {0x122D2, 247015},
  {0x122D3, 247037},
  {0x122D4, 247100},
  {0x122D5, 247057},
  {0x122D6, 247125},
  {0x122D7, 247146},
  {0x122D8, 247165},
  {0x122D9, 247202},
  {0x122DA, 247297},
  {0x122DB, 247319},
  {0x122DC, 247337},
  {0x122DD, 247389},
  {0x122DE, 247408},
  {0x122DF, 247428},
  {0x122E0, 247463},
  {0x122E1, 247483},
  {0x122E2, 247504},
  {0x122E3, 247522},
  {0x122E4, 247548},
  {0x122E5, 247567},
  {0x122E6, 247587},
  {0x122E7, 247608},
  {0x122E8, 247627},
  {0x122E9, 247649},
  {0x122EA, 247668},
  {0x122EB, 247688},
  {0x122EC, 247706},
  {0x122ED, 247756},
  {0x122EE, 247783},
  {0x122EF, 247733},
  {0x122F0, 247810},
  {0x122F1, 247829},
  {0x122F2, 247883},
  {0x122F3, 247910},
  {0x122F4, 247929},
  {0x122F5, 247957},
  {0x122F6, 247986},
  {0x122F7, 248015},
  {0x122F8, 248044},
  {0x122F9, 248074},
  {0x122FA, 248102},
  {0x122FB, 248151},
  {0x122FC, 248170},
  {0x122FD, 248188},
  {0x122FE, 248211},
  {0x122FF, 248229},
  {0x12300, 248271},
  {0x12301, 248290},
  {0x12302, 248391},
  {0x12303, 248309},
  {0x12304, 248337},
  {0x12305, 248421},
  {0x12306, 248439},
  {0x12307, 248459},
  {0x12308, 248478},
  {0x12309, 248568},
  {0x1230A, 248587},
  {0x1230B, 248626},
  {0x1230C, 248643},
  {0x1230D, 248819},
  {0x1230E, 248664},
  {0x1230F, 248712},
  {0x12310, 248749},
  {0x12311, 248840},
  {0x12312, 248858},
  {0x12313, 248876},
  {0x12314, 248917},
  {0x12315, 249004},
  {0x12316, 249032},
  {0x12317, 249059},
  {0x12318, 249099},
  {0x12319, 248894},
  {0x1231A, 248942},
  {0x1231B, 248968},
  {0x1231C, 249144},
  {0x1231D, 249164},
  {0x1231E, 249182},
  {0x1231F, 249239},
  {0x12320, 249274},
  {0x12321, 249303},
  {0x12322, 249329},
  {0x12323, 249350},
  {0x12324, 249370},
  {0x12325, 249403},
  {0x12326, 249432},
  {0x12327, 249450},
  {0x12328, 249473},
  {0x12329, 249491},
  {0x1232A, 249521},
  {0x1232B, 249547},
  {0x1232C, 249594},
  {0x1232D, 249629},
  {0x1232E, 249664},
  {0x1232F, 249692},
  {0x12330, 249720},
  {0x12331, 249749},
  {0x12332, 249777},
  {0x12333, 249814},
  {0x12334, 249878},
  {0x12335, 249897},
  {0x12336, 249916},
  {0x12337, 249936},
  {0x12338, 249955},
  {0x12339, 249982},
  {0x1233A, 250014},
  {0x1233B, 250074},
  {0x1233C, 250103},
  {0x1233D, 250131},
  {0x1233E, 250160},
  {0x1233F, 250195},
  {0x12340, 250224},
  {0x12341, 250252},
  {0x12342, 250280},
  {0x12343, 250309},
  {0x12344, 250337},
  {0x12345, 250366},
  {0x12346, 250456},
  {0x12347, 250485},
  {0x12348, 250514},
  {0x12349, 250542},
  {0x1234A, 250571},
  {0x1234B, 250601},
  {0x1234C, 250629},
  {0x1234D, 250665},
  {0x1234E, 250693},
  {0x1234F, 250724},
  {0x12350, 250745},
  {0x12351, 250774},
  {0x12352, 250793},
  {0x12353, 250820},
  {0x12354, 250848},
  {0x12355, 250877},
  {0x12356, 250949},
  {0x12357, 250907},
  {0x12358, 250927},
  {0x12359, 250969},
  {0x1235A, 250990},
  {0x1235B, 251009},
  {0x1235C, 251041},
  {0x1235D, 251060},
  {0x1235E, 251114},
  {0x1235F, 251078},
  {0x12360, 251156},
  {0x12361, 251175},
  {0x12362, 251195},
  {0x12363, 251214},
  {0x12364, 251232},
  {0x12365, 251258},
  {0x12366, 251277},
  {0x12367, 251296},
  {0x12368, 251325},
  {0x12369, 251344},
  {0x1236A, 251364},
  {0x1236B, 251417},
  {0x1236C, 251436},
  {0x1236D, 251504},
  {0x1236E, 251525},
  {0x1236F, 234702},
  {0x12370, 220226},
  {0x12371, 220405},
  {0x12372, 221032},
  {0x12373, 223232},
  {0x12374, 223342},
  {0x12375, 223713},
  {0x12376, 223884},
  {0x12377, 226282},
  {0x12378, 226641},
  {0x12379, 227031},
  {0x1237A, 227104},
  {0x1237B, 229775},
  {0x1237C, 230082},
  {0x1237D, 231752},
  {0x1237E, 232464},
  {0x1237F, 232523},
  {0x12380, 232969},
  {0x12381, 233106},
  {0x12382, 233163},
  {0x12383, 233420},
  {0x12384, 233700},
  {0x12385, 233949},
  {0x12386, 234064},
  {0x12387, 234266},
  {0x12388, 235953},
  {0x12389, 239828},
  {0x1238A, 240033},
  {0x1238B, 240600},
  {0x1238C, 241232},
  {0x1238D, 241886},
  {0x1238E, 241953},
  {0x1238F, 242435},
  {0x12390, 242454},
  {0x12391, 242600},
  {0x12392, 242669},
  {0x12393, 243478},
  {0x12394, 244424},
  {0x12395, 244741},
  {0x12396, 245271},
  {0x12397, 248252},
  {0x12398, 249212},
  {0x12399, 248800},
  {0x12400, 219067},
  {0x12401, 218515},
  {0x12402, 216167},
  {0x12403, 215704},
  {0x12404, 218234},
  {0x12405, 217779},
  {0x12406, 215240},
  {0x12407, 216849},
  {0x12408, 218701},
  {0x12409, 216348},
  {0x1240A, 215885},
  {0x1240B, 218301},
  {0x1240C, 217812},
  {0x1240D, 215273},
  {0x1240E, 216881},
  {0x1240F, 216517},
  {0x12410, 216094},
  {0x12411, 218399},
  {0x12412, 217916},
  {0x12413, 215377},
  {0x12414, 216982},
  {0x12415, 217480},
  {0x12416, 219231},
  {0x12417, 218735},
  {0x12418, 216381},
  {0x12419, 215918},
  {0x1241A, 218333},
  {0x1241B, 217846},
  {0x1241C, 215307},
  {0x1241D, 216914},
  {0x1241E, 217513},
  {0x1241F, 219264},
  {0x12420, 218770},
  {0x12421, 216415},
  {0x12422, 215952},
  {0x12423, 219297},
  {0x12424, 218805},
  {0x12425, 218840},
  {0x12426, 216449},
  {0x12427, 215986},
  {0x12428, 218366},
  {0x12429, 217881},
  {0x1242A, 215342},
  {0x1242B, 216948},
  {0x1242C, 217660},
  {0x1242D, 219330},
  {0x1242E, 218888},
  {0x1242F, 218923},
  {0x12430, 216483},
  {0x12431, 216020},
  {0x12432, 218137},
  {0x12433, 218186},
  {0x12434, 217377},
  {0x12435, 219166},
  {0x12436, 218620},
  {0x12437, 218654},
  {0x12438, 216315},
  {0x12439, 215852},
  {0x1243A, 218971},
  {0x1243B, 219019},
  {0x1243C, 216590},
  {0x1243D, 216735},
  {0x1243E, 216637},
  {0x1243F, 216686},
  {0x12440, 218470},
  {0x12441, 218089},
  {0x12442, 217991},
  {0x12443, 218040},
  {0x12444, 215452},
  {0x12445, 215499},
  {0x12446, 217055},
  {0x12447, 217153},
  {0x12448, 217202},
  {0x12449, 217103},
  {0x1244A, 219098},
  {0x1244B, 218548},
  {0x1244C, 216199},
  {0x1244D, 215736},
  {0x1244E, 218265},
  {0x1244F, 217345},
  {0x12450, 219134},
  {0x12451, 218586},
  {0x12452, 216236},
  {0x12453, 216269},
  {0x12454, 215773},
  {0x12455, 215806},
  {0x12456, 216816},
  {0x12457, 216783},
  {0x12458, 217447},
  {0x12459, 219198},
  {0x1245A, 217693},
  {0x1245B, 219363},
  {0x1245C, 216054},
  {0x1245D, 217731},
  {0x1245E, 219402},
  {0x1245F, 217409},
  {0x12460, 217582},
  {0x12461, 217299},
  {0x12462, 217251},
  {0x12463, 217621},
  {0x12464, 217546},
  {0x12465, 215621},
  {0x12466, 215662},
  {0x12467, 215584},
  {0x12468, 215547},
  {0x12469, 216547},
  {0x1246A, 216124},
  {0x1246B, 218428},
  {0x1246C, 217947},
  {0x1246D, 215408},
  {0x1246E, 217012},
  {0x12470, 219584},
  {0x12471, 219637},
  {0x12472, 219451},
  {0x12473, 219539},
  {0x12474, 219493},
  {0x12480, 220254},
  {0x12481, 220317},
  {0x12482, 220612},
  {0x12483, 221708},
  {0x12484, 221785},
  {0x12485, 221817},
  {0x12486, 221848},
  {0x12487, 222169},
  {0x12488, 222344},
  {0x12489, 223313},
  {0x1248A, 223498},
  {0x1248B, 223556},
  {0x1248C, 223607},
  {0x1248D, 223952},
  {0x1248E, 223983},
  {0x1248F, 224012},
  {0x12490, 224049},
  {0x12491, 224078},
  {0x12492, 224107},
  {0x12493, 224138},
  {0x12494, 224166},
  {0x12495, 224194},
  {0x12496, 224229},
  {0x12497, 224259},
  {0x12498, 224287},
  {0x12499, 224315},
  {0x1249A, 224349},
  {0x1249B, 224381},
  {0x1249C, 224410},
  {0x1249D, 224442},
  {0x1249E, 224486},
  {0x1249F, 224519},
  {0x124A0, 224548},
  {0x124A1, 224587},
  {0x124A2, 224626},
  {0x124A3, 224656},
  {0x124A4, 224685},
  {0x124A5, 224713},
  {0x124A6, 224741},
  {0x124A7, 224769},
  {0x124A8, 224798},
  {0x124A9, 225174},
  {0x124AA, 225229},
  {0x124AB, 225632},
  {0x124AC, 225675},
  {0x124AD, 225729},
  {0x124AE, 225767},
  {0x124AF, 225804},
  {0x124B0, 225847},
  {0x124B1, 225884},
  {0x124B2, 225922},
  {0x124B3, 225959},
  {0x124B4, 225997},
  {0x124B5, 226671},
  {0x124B6, 227202},
  {0x124B7, 227393},
  {0x124B8, 227495},
  {0x124B9, 227539},
  {0x124BA, 227717},
  {0x124BB, 227747},
  {0x124BC, 227839},
  {0x124BD, 227876},
  {0x124BE, 228044},
  {0x124BF, 228502},
  {0x124C0, 228574},
  {0x124C1, 228603},
  {0x124C2, 228642},
  {0x124C3, 228735},
  {0x124C4, 228765},
  {0x124C5, 228793},
  {0x124C6, 228829},
  {0x124C7, 229115},
  {0x124C8, 229295},
  {0x124C9, 229379},
  {0x124CA, 229851},
  {0x124CB, 230466},
  {0x124CC, 230633},
  {0x124CD, 230861},
  {0x124CE, 231179},
  {0x124CF, 231272},
  {0x124D0, 232665},
  {0x124D1, 233256},
  {0x124D2, 234117},
  {0x124D3, 234239},
  {0x124D4, 237293},
  {0x124D5, 237517},
  {0x124D6, 237540},
  {0x124D7, 237563},
  {0x124D8, 237586},
  {0x124D9, 237609},
  {0x124DA, 237632},
  {0x124DB, 237655},
  {0x124DC, 237678},
  {0x124DD, 237719},
  {0x124DE, 237742},
  {0x124DF, 237778},
  {0x124E0, 237801},
  {0x124E1, 237824},
  {0x124E2, 237847},
  {0x124E3, 237870},
  {0x124E4, 237893},
  {0x124E5, 237916},
  {0x124E6, 237939},
  {0x124E7, 237962},
  {0x124E8, 237985},
  {0x124E9, 238008},
  {0x124EA, 238031},
  {0x124EB, 238054},
  {0x124EC, 238077},
  {0x124ED, 238100},
  {0x124EE, 238123},
  {0x124EF, 238146},
  {0x124F0, 238169},
  {0x124F1, 238192},
  {0x124F2, 238215},
  {0x124F3, 238247},
  {0x124F4, 238280},
  {0x124F5, 238322},
  {0x124F6, 238373},
  {0x124F7, 238413},
  {0x124F8, 238436},
  {0x124F9, 238459},
  {0x124FA, 238482},
  {0x124FB, 238505},
  {0x124FC, 238528},
  {0x124FD, 238551},
  {0x124FE, 238574},
  {0x124FF, 238597},
  {0x12500, 238620},
  {0x12501, 238643},
  {0x12502, 238666},
  {0x12503, 238699},
  {0x12504, 238732},
  {0x12505, 238776},
  {0x12506, 238809},
  {0x12507, 238841},
  {0x12508, 238874},
  {0x12509, 238906},
  {0x1250A, 238938},
  {0x1250B, 238970},
  {0x1250C, 239005},
  {0x1250D, 239028},
  {0x1250E, 239051},
  {0x1250F, 239084},
  {0x12510, 239116},
  {0x12511, 239149},
  {0x12512, 239187},
  {0x12513, 239219},
  {0x12514, 239270},
  {0x12515, 239313},
  {0x12516, 239345},
  {0x12517, 239380},
  {0x12518, 239403},
  {0x12519, 239735},
  {0x1251A, 239973},
  {0x1251B, 240142},
  {0x1251C, 240524},
  {0x1251D, 240553},
  {0x1251E, 240704},
  {0x1251F, 241270},
  {0x12520, 241577},
  {0x12521, 241616},
  {0x12522, 242632},
  {0x12523, 242700},
  {0x12524, 242765},
  {0x12525, 242797},
  {0x12526, 242828},
  {0x12527, 242862},
  {0x12528, 242898},
  {0x12529, 243040},
  {0x1252A, 243246},
  {0x1252B, 243276},
  {0x1252C, 243353},
  {0x1252D, 245043},
  {0x1252E, 245215},
  {0x1252F, 245478},
  {0x12530, 245574},
  {0x12531, 246142},
  {0x12532, 246250},
  {0x12533, 246386},
  {0x12534, 246415},
  {0x12535, 246444},
  {0x12536, 247222},
  {0x12537, 247260},
  {0x12538, 247360},
  {0x12539, 248122},
  {0x1253A, 248497},
  {0x1253B, 248532},
  {0x1253C, 249566},
  {0x1253D, 249850},
  {0x1253E, 250043},
  {0x1253F, 250394},
  {0x12540, 250427},
  {0x12541, 251137},
  {0x12542, 251382},
  {0x12543, 251463},
  {0x12F90, 252905},
  {0x12F91, 252929},
  {0x12F92, 252953},
  {0x12F93, 252977},
  {0x12F94, 253001},
  {0x12F95, 253025},
  {0x12F96, 253049},
  {0x12F97, 253073},
  {0x12F98, 253097},
  {0x12F99, 253121},
  {0x12F9A, 253145},
  {0x12F9B, 253169},
  {0x12F9C, 253194},
  {0x12F9D, 253218},
  {0x12F9E, 253242},
  {0x12F9F, 253266},
  {0x12FA0, 253290},
  {0x12FA1, 253314},
  {0x12FA2, 253338},
  {0x12FA3, 253362},
  {0x12FA4, 253386},
  {0x12FA5, 253410},
  {0x12FA6, 253434},
  {0x12FA7, 253458},
  {0x12FA8, 253482},
  {0x12FA9, 253506},
  {0x12FAA, 253530},
  {0x12FAB, 253554},
  {0x12FAC, 253578},
  {0x12FAD, 253602},
  {0x12FAE, 253626},
  {0x12FAF, 253650},
  {0x12FB0, 253674},
  {0x12FB1, 253698},
  {0x12FB2, 253722},
  {0x12FB3, 253746},
  {0x12FB4, 253770},
  {0x12FB5, 253794},
  {0x12FB6, 253818},
  {0x12FB7, 253842},
  {0x12FB8, 253866},
  {0x12FB9, 253890},
  {0x12FBA, 253914},
  {0x12FBB, 253938},
  {0x12FBC, 253962},
  {0x12FBD, 253986},
  {0x12FBE, 254010},
  {0x12FBF, 254034},
  {0x12FC0, 254058},
  {0x12FC1, 254082},
  {0x12FC2, 254106},
  {0x12FC3, 254130},
  {0x12FC4, 254154},
  {0x12FC5, 254178},
  {0x12FC6, 254202},
  {0x12FC7, 254226},
  {0x12FC8, 254250},
  {0x12FC9, 254274},
  {0x12FCA, 254298},
  {0x12FCB, 254322},
  {0x12FCC, 254346},
  {0x12FCD, 254370},
  {0x12FCE, 254394},
  {0x12FCF, 254419},
  {0x12FD0, 254443},
  {0x12FD1, 254467},
  {0x12FD2, 254491},
  {0x12FD3, 254515},
  {0x12FD4, 254539},
  {0x12FD5, 254563},
  {0x12FD6, 254587},
  {0x12FD7, 254611},
  {0x12FD8, 254635},
  {0x12FD9, 254659},
  {0x12FDA, 254683},
  {0x12FDB, 254707},
  {0x12FDC, 254731},
  {0x12FDD, 254755},
  {0x12FDE, 254779},
  {0x12FDF, 254803},
  {0x12FE0, 254827},
  {0x12FE1, 254851},
  {0x12FE2, 254875},
  {0x12FE3, 254899},
  {0x12FE4, 254923},
  {0x12FE5, 254947},
  {0x12FE6, 254971},
  {0x12FE7, 254995},
  {0x12FE8, 255019},
  {0x12FE9, 255043},
  {0x12FEA, 255067},
  {0x12FEB, 255091},
  {0x12FEC, 255115},
  {0x12FED, 255139},
  {0x12FEE, 255163},
  {0x12FEF, 255187},
  {0x12FF0, 255211},
  {0x12FF1, 255235},
  {0x12FF2, 255259},
  {0x13000, 293852},
  {0x13001, 293877},
  {0x13002, 293902},
  {0x13003, 293927},
  {0x13004, 293952},
  {0x13005, 293977},
  {0x13006, 294003},
  {0x13007, 294028},
  {0x13008, 294054},
  {0x13009, 294080},
  {0x1300A, 294105},
  {0x1300B, 294130},
  {0x1300C, 294155},
  {0x1300D, 294180},
  {0x1300E, 294205},
  {0x1300F, 294230},
  {0x13010, 294255},
  {0x13011, 294280},
  {0x13012, 294306},
  {0x13013, 294331},
  {0x13014, 294356},
  {0x13015, 294381},
  {0x13016, 294407},
  {0x13017, 294432},
  {0x13018, 294457},
  {0x13019, 294482},
  {0x1301A, 294507},
  {0x1301B, 294532},
  {0x1301C, 294557},
  {0x1301D, 294582},
  {0x1301E, 294607},
  {0x1301F, 294632},
  {0x13020, 294657},
  {0x13021, 294682},
  {0x13022, 294707},
  {0x13023, 294732},
  {0x13024, 294757},
  {0x13025, 294782},
  {0x13026, 294808},
  {0x13027, 294833},
  {0x13028, 294858},
  {0x13029, 294883},
  {0x1302A, 294908},
  {0x1302B, 294933},
  {0x1302C, 294958},
  {0x1302D, 294983},
  {0x1302E, 295008},
  {0x1302F, 295034},
  {0x13030, 295059},
  {0x13031, 295084},
  {0x13032, 295110},
  {0x13033, 295135},
  {0x13034, 295161},
  {0x13035, 295186},
  {0x13036, 295211},
  {0x13037, 295237},
  {0x13038, 295262},
  {0x13039, 295287},
  {0x1303A, 295312},
  {0x1303B, 295337},
  {0x1303C, 295362},
  {0x1303D, 295387},
  {0x1303E, 295412},
  {0x1303F, 295437},
  {0x13040, 295462},
  {0x13041, 295487},
  {0x13042, 295512},
  {0x13043, 295537},
  {0x13044, 295562},
  {0x13045, 295587},
  {0x13046, 295612},
  {0x13047, 295637},
  {0x13048, 295662},
  {0x13049, 295687},
  {0x1304A, 295712},
  {0x1304B, 295737},
  {0x1304C, 295762},
  {0x1304D, 295787},
  {0x1304E, 295812},
  {0x1304F, 295837},
  {0x13050, 296748},
  {0x13051, 296773},
  {0x13052, 296798},
  {0x13053, 296823},
  {0x13054, 296848},
  {0x13055, 296873},
  {0x13056, 296899},
  {0x13057, 296924},
  {0x13058, 296949},
  {0x13059, 296974},
  {0x1305A, 297112},
  {0x1305B, 297137},
  {0x1305C, 297162},
  {0x1305D, 297188},
  {0x1305E, 297214},
  {0x1305F, 297240},
  {0x13060, 297265},
  {0x13061, 297290},
  {0x13062, 297315},
  {0x13063, 297340},
  {0x13064, 297365},
  {0x13065, 297390},
  {0x13066, 297415},
  {0x13067, 297440},
  {0x13068, 297466},
  {0x13069, 297491},
  {0x1306A, 297516},
  {0x1306B, 297541},
  {0x1306C, 297566},
  {0x1306D, 297591},
  {0x1306E, 297616},
  {0x1306F, 297641},
  {0x13070, 297666},
  {0x13071, 297691},
  {0x13072, 297716},
  {0x13073, 297741},
  {0x13074, 297766},
  {0x13075, 297791},
  {0x13076, 297816},
  {0x13077, 297841},
  {0x13078, 297866},
  {0x13079, 297891},
  {0x1307A, 297916},
  {0x1307B, 297941},
  {0x1307C, 297966},
  {0x1307D, 297991},
  {0x1307E, 298016},
  {0x1307F, 298042},
  {0x13080, 298067},
  {0x13081, 298092},
  {0x13082, 298117},
  {0x13083, 298142},
  {0x13084, 298167},
  {0x13085, 298192},
  {0x13086, 298217},
  {0x13087, 298242},
  {0x13088, 298267},
  {0x13089, 298292},
  {0x1308A, 298317},
  {0x1308B, 298342},
  {0x1308C, 298367},
  {0x1308D, 298392},
  {0x1308E, 298417},
  {0x1308F, 298442},
  {0x13090, 298467},
  {0x13091, 298492},
  {0x13092, 298517},
  {0x13093, 298543},
  {0x13094, 298568},
  {0x13095, 298593},
  {0x13096, 298618},
  {0x13097, 298643},
  {0x13098, 298669},
  {0x13099, 298694},
  {0x1309A, 298719},
  {0x1309B, 298744},
  {0x1309C, 298770},
  {0x1309D, 298795},
  {0x1309E, 298820},
  {0x1309F, 298845},
  {0x130A0, 298870},
  {0x130A1, 298895},
  {0x130A2, 298920},
  {0x130A3, 298945},
  {0x130A4, 298970},
  {0x130A5, 298995},
  {0x130A6, 299020},
  {0x130A7, 299045},
  {0x130A8, 299070},
  {0x130A9, 299096},
  {0x130AA, 299121},
  {0x130AB, 299146},
  {0x130AC, 299172},
  {0x130AD, 299197},
  {0x130AE, 299222},
  {0x130AF, 299248},
  {0x130B0, 299274},
  {0x130B1, 299300},
  {0x130B2, 299326},
  {0x130B3, 299352},
  {0x130B4, 299378},
  {0x130B5, 299404},
  {0x130B6, 299430},
  {0x130B7, 299456},
  {0x130B8, 299481},
  {0x130B9, 299506},
  {0x130BA, 299532},
  {0x130BB, 299557},
  {0x130BC, 299582},
  {0x130BD, 299608},
  {0x130BE, 299633},
  {0x130BF, 299658},
  {0x130C0, 299683},
  {0x130C1, 299708},
  {0x130C2, 299733},
  {0x130C3, 299758},
  {0x130C4, 299783},
  {0x130C5, 299808},
  {0x130C6, 299833},
  {0x130C7, 299858},
  {0x130C8, 299883},
  {0x130C9, 299908},
  {0x130CA, 299933},
  {0x130CB, 299959},
  {0x130CC, 299985},
  {0x130CD, 300011},
  {0x130CE, 300037},
  {0x130CF, 300063},
  {0x130D0, 300089},
  {0x130D1, 300115},
  {0x130D2, 300141},
  {0x130D3, 300166},
  {0x130D4, 300191},
  {0x130D5, 300216},
  {0x130D6, 300241},
  {0x130D7, 300266},
  {0x130D8, 300291},
  {0x130D9, 300316},
  {0x130DA, 300341},
  {0x130DB, 300367},
  {0x130DC, 300392},
  {0x130DD, 300418},
  {0x130DE, 300443},
  {0x130DF, 300468},
  {0x130E0, 300493},
  {0x130E1, 300518},
  {0x130E2, 300543},
  {0x130E3, 300568},
  {0x130E4, 300593},
  {0x130E5, 300619},
  {0x130E6, 300644},
  {0x130E7, 300670},
  {0x130E8, 300695},
  {0x130E9, 300720},
  {0x130EA, 300745},
  {0x130EB, 300771},
  {0x130EC, 300796},
  {0x130ED, 300821},
  {0x130EE, 300846},
  {0x130EF, 300871},
  {0x130F0, 300896},
  {0x130F1, 300921},
  {0x130F2, 300946},
  {0x130F3, 300971},
  {0x130F4, 300997},
  {0x130F5, 301022},
  {0x130F6, 301047},
  {0x130F7, 301072},
  {0x130F8, 301097},
  {0x130F9, 301122},
  {0x130FA, 301147},
  {0x130FB, 301173},
  {0x130FC, 301198},
  {0x130FD, 301223},
  {0x130FE, 301355},
  {0x130FF, 301380},
  {0x13100, 301406},
  {0x13101, 301431},
  {0x13102, 301456},
  {0x13103, 301481},
  {0x13104, 301506},
  {0x13105, 301531},
  {0x13106, 301556},
  {0x13107, 301581},
  {0x13108, 301606},
  {0x13109, 301631},
  {0x1310A, 301656},
  {0x1310B, 301681},
  {0x1310C, 301706},
  {0x1310D, 301732},
  {0x1310E, 301757},
  {0x1310F, 301782},
  {0x13110, 301807},
  {0x13111, 301832},
  {0x13112, 301857},
  {0x13113, 301882},
  {0x13114, 301907},
  {0x13115, 301932},
  {0x13116, 301958},
  {0x13117, 301983},
  {0x13118, 302008},
  {0x13119, 302033},
  {0x1311A, 302058},
  {0x1311B, 302083},
  {0x1311C, 302108},
  {0x1311D, 302133},
  {0x1311E, 302158},
  {0x1311F, 302183},
  {0x13120, 302208},
  {0x13121, 302234},
  {0x13122, 302259},
  {0x13123, 302284},
  {0x13124, 302309},
  {0x13125, 302334},
  {0x13126, 302359},
  {0x13127, 302384},
  {0x13128, 302410},
  {0x13129, 302435},
  {0x1312A, 302461},
  {0x1312B, 302486},
  {0x1312C, 302511},
  {0x1312D, 302536},
  {0x1312E, 302561},
  {0x1312F, 302586},
  {0x13130, 302611},
  {0x13131, 302636},
  {0x13132, 302662},
  {0x13133, 302687},
  {0x13134, 302713},
  {0x13135, 302738},
  {0x13136, 302764},
  {0x13137, 302789},
  {0x13138, 302814},
  {0x13139, 302839},
  {0x1313A, 302864},
  {0x1313B, 302890},
  {0x1313C, 302916},
  {0x1313D, 302942},
  {0x1313E, 302967},
  {0x1313F, 303023},
  {0x13140, 303048},
  {0x13141, 303073},
  {0x13142, 303098},
  {0x13143, 303123},
  {0x13144, 303148},
  {0x13145, 303173},
  {0x13146, 303199},
  {0x13147, 303224},
  {0x13148, 303250},
  {0x13149, 303276},
  {0x1314A, 303301},
  {0x1314B, 303326},
  {0x1314C, 303351},
  {0x1314D, 303376},
  {0x1314E, 303402},
  {0x1314F, 303427},
  {0x13150, 303452},
  {0x13151, 303477},
  {0x13152, 303502},
  {0x13153, 303527},
  {0x13154, 303552},
  {0x13155, 303577},
  {0x13156, 303602},
  {0x13157, 303627},
  {0x13158, 303653},
  {0x13159, 303678},
  {0x1315A, 303703},
  {0x1315B, 303728},
  {0x1315C, 303753},
  {0x1315D, 303778},
  {0x1315E, 303803},
  {0x1315F, 303829},
  {0x13160, 303854},
  {0x13161, 303879},
  {0x13162, 303904},
  {0x13163, 303929},
  {0x13164, 303954},
  {0x13165, 303979},
  {0x13166, 304004},
  {0x13167, 304029},
  {0x13168, 304054},
  {0x13169, 304079},
  {0x1316A, 304105},
  {0x1316B, 304130},
  {0x1316C, 304156},
  {0x1316D, 304181},
  {0x1316E, 304206},
  {0x1316F, 304231},
  {0x13170, 304256},
  {0x13171, 304281},
  {0x13172, 304306},
  {0x13173, 304332},
  {0x13174, 304357},
  {0x13175, 304382},
  {0x13176, 304408},
  {0x13177, 304433},
  {0x13178, 304458},
  {0x13179, 304483},
  {0x1317A, 304508},
  {0x1317B, 304533},
  {0x1317C, 304558},
  {0x1317D, 304583},
  {0x1317E, 304608},
  {0x1317F, 304633},
  {0x13180, 304658},
  {0x13181, 304683},
  {0x13182, 304708},
  {0x13183, 304733},
  {0x13184, 304758},
  {0x13185, 304783},
  {0x13186, 304809},
  {0x13187, 304834},
  {0x13188, 304963},
  {0x13189, 304988},
  {0x1318A, 305013},
  {0x1318B, 305038},
  {0x1318C, 305063},
  {0x1318D, 305088},
  {0x1318E, 305114},
  {0x1318F, 305139},
  {0x13190, 305164},
  {0x13191, 305189},
  {0x13192, 305214},
  {0x13193, 305240},
  {0x13194, 305265},
  {0x13195, 305291},
  {0x13196, 305316},
  {0x13197, 305342},
  {0x13198, 305367},
  {0x13199, 305392},
  {0x1319A, 305417},
  {0x1319B, 305712},
  {0x1319C, 305737},
  {0x1319D, 305762},
  {0x1319E, 305787},
  {0x1319F, 305812},
  {0x131A0, 305837},
  {0x131A1, 305862},
  {0x131A2, 305887},
  {0x131A3, 305912},
  {0x131A4, 305937},
  {0x131A5, 305962},
  {0x131A6, 305988},
  {0x131A7, 306013},
  {0x131A8, 306038},
  {0x131A9, 306063},
  {0x131AA, 306088},
  {0x131AB, 306114},
  {0x131AC, 306139},
  {0x131AD, 306194},
  {0x131AE, 306219},
  {0x131AF, 306245},
  {0x131B0, 306271},
  {0x131B1, 306296},
  {0x131B2, 306321},
  {0x131B3, 306347},
  {0x131B4, 306372},
  {0x131B5, 306397},
  {0x131B6, 306422},
  {0x131B7, 306447},
  {0x131B8, 306472},
  {0x131B9, 306497},
  {0x131BA, 306522},
  {0x131BB, 306548},
  {0x131BC, 306573},
  {0x131BD, 306598},
  {0x131BE, 306624},
  {0x131BF, 306650},
  {0x131C0, 306676},
  {0x131C1, 306702},
  {0x131C2, 306728},
  {0x131C3, 306754},
  {0x131C4, 306780},
  {0x131C5, 306806},
  {0x131C6, 306831},
  {0x131C7, 306856},
  {0x131C8, 306881},
  {0x131C9, 306907},
  {0x131CA, 306932},
  {0x131CB, 306958},
  {0x131CC, 306983},
  {0x131CD, 307009},
  {0x131CE, 307034},
  {0x131CF, 307059},
  {0x131D0, 307084},
  {0x131D1, 307109},
  {0x131D2, 307134},
  {0x131D3, 307160},
  {0x131D4, 307185},
  {0x131D5, 307210},
  {0x131D6, 307236},
  {0x131D7, 307261},
  {0x131D8, 307286},
  {0x131D9, 307311},
  {0x131DA, 307336},
  {0x131DB, 307362},
  {0x131DC, 307387},
  {0x131DD, 307412},
  {0x131DE, 307437},
  {0x131DF, 307463},
  {0x131E0, 307488},
  {0x131E1, 307513},
  {0x131E2, 307539},
  {0x131E3, 307565},
  {0x131E4, 307590},
  {0x131E5, 307615},
  {0x131E6, 307640},
  {0x131E7, 307665},
  {0x131E8, 307690},
  {0x131E9, 307715},
  {0x131EA, 307740},
  {0x131EB, 307766},
  {0x131EC, 307791},
  {0x131ED, 307816},
  {0x131EE, 307841},
  {0x131EF, 308674},
  {0x131F0, 308699},
  {0x131F1, 308724},
  {0x131F2, 308749},
  {0x131F3, 308774},
  {0x131F4, 308799},
  {0x131F5, 308824},
  {0x131F6, 308849},
  {0x131F7, 308874},
  {0x131F8, 308899},
  {0x131F9, 308924},
  {0x131FA, 308949},
  {0x131FB, 308974},
  {0x131FC, 308999},
  {0x131FD, 309024},
  {0x131FE, 309049},
  {0x131FF, 309074},
  {0x13200, 309099},
  {0x13201, 309124},
  {0x13202, 309150},
  {0x13203, 309176},
  {0x13204, 309201},
  {0x13205, 309226},
  {0x13206, 309251},
  {0x13207, 309276},
  {0x13208, 309301},
  {0x13209, 309326},
  {0x1320A, 309351},
  {0x1320B, 309377},
  {0x1320C, 309402},
  {0x1320D, 309427},
  {0x1320E, 309452},
  {0x1320F, 309477},
  {0x13210, 309502},
  {0x13211, 309527},
  {0x13212, 309552},
  {0x13213, 309577},
  {0x13214, 309603},
  {0x13215, 309628},
  {0x13216, 309654},
  {0x13217, 309679},
  {0x13218, 309705},
  {0x13219, 309730},
  {0x1321A, 309755},
  {0x1321B, 309781},
  {0x1321C, 309806},
  {0x1321D, 309831},
  {0x1321E, 309856},
  {0x1321F, 309881},
  {0x13220, 309906},
  {0x13221, 309932},
  {0x13222, 309958},
  {0x13223, 309984},
  {0x13224, 310010},
  {0x13225, 310036},
  {0x13226, 310063},
  {0x13227, 310089},
  {0x13228, 310115},
  {0x13229, 310141},
  {0x1322A, 310167},
  {0x1322B, 310193},
  {0x1322C, 310219},
  {0x1322D, 310245},
  {0x1322E, 310271},
  {0x1322F, 310297},
  {0x13230, 310323},
  {0x13231, 310349},
  {0x13232, 310375},
  {0x13233, 310402},
  {0x13234, 310428},
  {0x13235, 310454},
  {0x13236, 310480},
  {0x13237, 310506},
  {0x13238, 310532},
  {0x13239, 310558},
  {0x1323A, 310584},
  {0x1323B, 310610},
  {0x1323C, 310636},
  {0x1323D, 310662},
  {0x1323E, 310688},
  {0x1323F, 310714},
  {0x13240, 310740},
  {0x13241, 310767},
  {0x13242, 310793},
  {0x13243, 310820},
  {0x13244, 310846},
  {0x13245, 310872},
  {0x13246, 310898},
  {0x13247, 310924},
  {0x13248, 310950},
  {0x13249, 310976},
  {0x1324A, 311002},
  {0x1324B, 311029},
  {0x1324C, 311055},
  {0x1324D, 311081},
  {0x1324E, 311107},
  {0x1324F, 311133},
  {0x13250, 311160},
  {0x13251, 311185},
  {0x13252, 311211},
  {0x13253, 311236},
  {0x13254, 311261},
  {0x13255, 311286},
  {0x13256, 311311},
  {0x13257, 311337},
  {0x13258, 311362},
  {0x13259, 311388},
  {0x1325A, 311414},
  {0x1325B, 311440},
  {0x1325C, 311466},
  {0x1325D, 311492},
  {0x1325E, 311518},
  {0x1325F, 311543},
  {0x13260, 311568},
  {0x13261, 311593},
  {0x13262, 311618},
  {0x13263, 311644},
  {0x13264, 311670},
  {0x13265, 311696},
  {0x13266, 311721},
  {0x13267, 311746},
  {0x13268, 311771},
  {0x13269, 311796},
  {0x1326A, 311821},
  {0x1326B, 311846},
  {0x1326C, 311871},
  {0x1326D, 311896},
  {0x1326E, 311921},
  {0x1326F, 311947},
  {0x13270, 311972},
  {0x13271, 311998},
  {0x13272, 312023},
  {0x13273, 312048},
  {0x13274, 312073},
  {0x13275, 312098},
  {0x13276, 312124},
  {0x13277, 312149},
  {0x13278, 312175},
  {0x13279, 312200},
  {0x1327A, 312225},
  {0x1327B, 312250},
  {0x1327C, 312275},
  {0x1327D, 312301},
  {0x1327E, 312326},
  {0x1327F, 312352},
  {0x13280, 312377},
  {0x13281, 312402},
  {0x13282, 312427},
  {0x13283, 312453},
  {0x13284, 312478},
  {0x13285, 312503},
  {0x13286, 312528},
  {0x13287, 312554},
  {0x13288, 312580},
  {0x13289, 312606},
  {0x1328A, 312632},
  {0x1328B, 312657},
  {0x1328C, 312682},
  {0x1328D, 312707},
  {0x1328E, 312732},
  {0x1328F, 312757},
  {0x13290, 312782},
  {0x13291, 312807},
  {0x13292, 312832},
  {0x13293, 312857},
  {0x13294, 312882},
  {0x13295, 312907},
  {0x13296, 312932},
  {0x13297, 312957},
  {0x13298, 312982},
  {0x13299, 313008},
  {0x1329A, 313034},
  {0x1329B, 313094},
  {0x1329C, 313119},
  {0x1329D, 313145},
  {0x1329E, 313170},
  {0x1329F, 313195},
  {0x132A0, 313221},
  {0x132A1, 313246},
  {0x132A2, 313271},
  {0x132A3, 313296},
  {0x132A4, 313321},
  {0x132A5, 313346},
  {0x132A6, 313371},
  {0x132A7, 313396},
  {0x132A8, 313421},
  {0x132A9, 313446},
  {0x132AA, 313471},
  {0x132AB, 313496},
  {0x132AC, 313521},
  {0x132AD, 313546},
  {0x132AE, 313571},
  {0x132AF, 313596},
  {0x132B0, 313621},
  {0x132B1, 313646},
  {0x132B2, 313672},
  {0x132B3, 313697},
  {0x132B4, 313723},
  {0x132B5, 313749},
  {0x132B6, 313774},
  {0x132B7, 313799},
  {0x132B8, 313824},
  {0x132B9, 313849},
  {0x132BA, 313874},
  {0x132BB, 313899},
  {0x132BC, 313924},
  {0x132BD, 313950},
  {0x132BE, 313975},
  {0x132BF, 314000},
  {0x132C0, 314025},
  {0x132C1, 314050},
  {0x132C2, 314075},
  {0x132C3, 314100},
  {0x132C4, 314126},
  {0x132C5, 314151},
  {0x132C6, 314176},
  {0x132C7, 314201},
  {0x132C8, 314226},
  {0x132C9, 314251},
  {0x132CA, 314276},
  {0x132CB, 314301},
  {0x132CC, 314326},
  {0x132CD, 314351},
  {0x132CE, 314376},
  {0x132CF, 314401},
  {0x132D0, 314426},
  {0x132D1, 314451},
  {0x132D2, 314476},
  {0x132D3, 314501},
  {0x132D4, 314527},
  {0x132D5, 314552},
  {0x132D6, 314577},
  {0x132D7, 314602},
  {0x132D8, 314627},
  {0x132D9, 314653},
  {0x132DA, 314678},
  {0x132DB, 314703},
  {0x132DC, 314728},
  {0x132DD, 314753},
  {0x132DE, 314778},
  {0x132DF, 314803},
  {0x132E0, 314828},
  {0x132E1, 314853},
  {0x132E2, 314879},
  {0x132E3, 314905},
  {0x132E4, 314930},
  {0x132E5, 314955},
  {0x132E6, 314980},
  {0x132E7, 315006},
  {0x132E8, 315031},
  {0x132E9, 315056},
  {0x132EA, 315081},
  {0x132EB, 315106},
  {0x132EC, 315131},
  {0x132ED, 315156},
  {0x132EE, 315181},
  {0x132EF, 315206},
  {0x132F0, 315231},
  {0x132F1, 315257},
  {0x132F2, 315283},
  {0x132F3, 315308},
  {0x132F4, 315333},
  {0x132F5, 315358},
  {0x132F6, 315383},
  {0x132F7, 315408},
  {0x132F8, 315433},
  {0x132F9, 315458},
  {0x132FA, 315483},
  {0x132FB, 315508},
  {0x132FC, 315534},
  {0x132FD, 315559},
  {0x132FE, 315584},
  {0x132FF, 315609},
  {0x13300, 315634},
  {0x13301, 315659},
  {0x13302, 315684},
  {0x13303, 315709},
  {0x13304, 315734},
  {0x13305, 315759},
  {0x13306, 315784},
  {0x13307, 315809},
  {0x13308, 315834},
  {0x13309, 315859},
  {0x1330A, 315884},
  {0x1330B, 315910},
  {0x1330C, 315935},
  {0x1330D, 315960},
  {0x1330E, 315985},
  {0x1330F, 316010},
  {0x13310, 316036},
  {0x13311, 316061},
  {0x13312, 316087},
  {0x13313, 316112},
  {0x13314, 316138},
  {0x13315, 316163},
  {0x13316, 316188},
  {0x13317, 316214},
  {0x13318, 316239},
  {0x13319, 316264},
  {0x1331A, 316289},
  {0x1331B, 316314},
  {0x1331C, 316339},
  {0x1331D, 316365},
  {0x1331E, 316390},
  {0x1331F, 316415},
  {0x13320, 316440},
  {0x13321, 316465},
  {0x13322, 316490},
  {0x13323, 316515},
  {0x13324, 316540},
  {0x13325, 316565},
  {0x13326, 316590},
  {0x13327, 316615},
  {0x13328, 316640},
  {0x13329, 316665},
  {0x1332A, 316690},
  {0x1332B, 316715},
  {0x1332C, 316740},
  {0x1332D, 316765},
  {0x1332E, 316791},
  {0x1332F, 316816},
  {0x13330, 316842},
  {0x13331, 316867},
  {0x13332, 316892},
  {0x13333, 316952},
  {0x13334, 316977},
  {0x13335, 317002},
  {0x13336, 317027},
  {0x13337, 317052},
  {0x13338, 317077},
  {0x13339, 317102},
  {0x1333A, 317128},
  {0x1333B, 317154},
  {0x1333C, 317179},
  {0x1333D, 317204},
  {0x1333E, 317229},
  {0x1333F, 317254},
  {0x13340, 317279},
  {0x13341, 317304},
  {0x13342, 317329},
  {0x13343, 317354},
  {0x13344, 317379},
  {0x13345, 317404},
  {0x13346, 317429},
  {0x13347, 317454},
  {0x13348, 317479},
  {0x13349, 317504},
  {0x1334A, 317529},
  {0x1334B, 317554},
  {0x1334C, 317579},
  {0x1334D, 317605},
  {0x1334E, 317630},
  {0x1334F, 317655},
  {0x13350, 317680},
  {0x13351, 317705},
  {0x13352, 317730},
  {0x13353, 317755},
  {0x13354, 317781},
  {0x13355, 317806},
  {0x13356, 317831},
  {0x13357, 317856},
  {0x13358, 317882},
  {0x13359, 317907},
  {0x1335A, 317932},
  {0x1335B, 317957},
  {0x1335C, 317982},
  {0x1335D, 318007},
  {0x1335E, 318032},
  {0x1335F, 318057},
  {0x13360, 318082},
  {0x13361, 318107},
  {0x13362, 318132},
  {0x13363, 318157},
  {0x13364, 318183},
  {0x13365, 318209},
  {0x13366, 318235},
  {0x13367, 318261},
  {0x13368, 318287},
  {0x13369, 318313},
  {0x1336A, 318339},
  {0x1336B, 318365},
  {0x1336C, 318391},
  {0x1336D, 318416},
  {0x1336E, 318442},
  {0x1336F, 318467},
  {0x13370, 318492},
  {0x13371, 318517},
  {0x13372, 318542},
  {0x13373, 318567},
  {0x13374, 318593},
  {0x13375, 318619},
  {0x13376, 318644},
  {0x13377, 318669},
  {0x13378, 318694},
  {0x13379, 318719},
  {0x1337A, 318745},
  {0x1337B, 318771},
  {0x1337C, 318823},
  {0x1337D, 318848},
  {0x1337E, 318874},
  {0x1337F, 318900},
  {0x13380, 318925},
  {0x13381, 318950},
  {0x13382, 318975},
  {0x13383, 319000},
  {0x13384, 319025},
  {0x13385, 319050},
  {0x13386, 319075},
  {0x13387, 319100},
  {0x13388, 319126},
  {0x13389, 319152},
  {0x1338A, 319178},
  {0x1338B, 319204},
  {0x1338C, 319230},
  {0x1338D, 319256},
  {0x1338E, 319282},
  {0x1338F, 319308},
  {0x13390, 319334},
  {0x13391, 319360},
  {0x13392, 319386},
  {0x13393, 319412},
  {0x13394, 319437},
  {0x13395, 319462},
  {0x13396, 319487},
  {0x13397, 319513},
  {0x13398, 319538},
  {0x13399, 319563},
  {0x1339A, 319588},
  {0x1339B, 319613},
  {0x1339C, 319638},
  {0x1339D, 319664},
  {0x1339E, 319689},
  {0x1339F, 319715},
  {0x133A0, 319740},
  {0x133A1, 319766},
  {0x133A2, 319791},
  {0x133A3, 319817},
  {0x133A4, 319842},
  {0x133A5, 319867},
  {0x133A6, 319893},
  {0x133A7, 319918},
  {0x133A8, 319943},
  {0x133A9, 319968},
  {0x133AA, 319993},
  {0x133AB, 320019},
  {0x133AC, 320044},
  {0x133AD, 320069},
  {0x133AE, 320094},
  {0x133AF, 320156},
  {0x133B0, 320181},
  {0x133B1, 320206},
  {0x133B2, 320231},
  {0x133B3, 320257},
  {0x133B4, 320282},
  {0x133B5, 320307},
  {0x133B6, 320332},
  {0x133B7, 320357},
  {0x133B8, 320382},
  {0x133B9, 320407},
  {0x133BA, 320433},
  {0x133BB, 320458},
  {0x133BC, 320484},
  {0x133BD, 320509},
  {0x133BE, 320534},
  {0x133BF, 320559},
  {0x133C0, 320584},
  {0x133C1, 320610},
  {0x133C2, 320635},
  {0x133C3, 320660},
  {0x133C4, 320685},
  {0x133C5, 320711},
  {0x133C6, 320736},
  {0x133C7, 320762},
  {0x133C8, 320787},
  {0x133C9, 320812},
  {0x133CA, 320837},
  {0x133CB, 320862},
  {0x133CC, 320887},
  {0x133CD, 320912},
  {0x133CE, 320938},
  {0x133CF, 320998},
  {0x133D0, 321023},
  {0x133D1, 321048},
  {0x133D2, 321073},
  {0x133D3, 321098},
  {0x133D4, 321124},
  {0x133D5, 321150},
  {0x133D6, 321175},
  {0x133D7, 321200},
  {0x133D8, 321226},
  {0x133D9, 321251},
  {0x133DA, 321276},
  {0x133DB, 321302},
  {0x133DC, 321327},
  {0x133DD, 321353},
  {0x133DE, 321378},
  {0x133DF, 321403},
  {0x133E0, 321428},
  {0x133E1, 321453},
  {0x133E2, 321478},
  {0x133E3, 321503},
  {0x133E4, 321528},
  {0x133E5, 321553},
  {0x133E6, 321578},
  {0x133E7, 321604},
  {0x133E8, 321630},
  {0x133E9, 321656},
  {0x133EA, 321682},
  {0x133EB, 321707},
  {0x133EC, 321733},
  {0x133ED, 321759},
  {0x133EE, 321784},
  {0x133EF, 321810},
  {0x133F0, 321835},
  {0x133F1, 321861},
  {0x133F2, 321886},
  {0x133F3, 321911},
  {0x133F4, 321936},
  {0x133F5, 321961},
  {0x133F6, 321986},
  {0x133F7, 322011},
  {0x133F8, 322036},
  {0x133F9, 322061},
  {0x133FA, 322086},
  {0x133FB, 322111},
  {0x133FC, 322137},
  {0x133FD, 322163},
  {0x133FE, 322189},
  {0x133FF, 322215},
  {0x13400, 322241},
  {0x13401, 322267},
  {0x13402, 322293},
  {0x13403, 322319},
  {0x13404, 322345},
  {0x13405, 322370},
  {0x13406, 322396},
  {0x13407, 322422},
  {0x13408, 322448},
  {0x13409, 322474},
  {0x1340A, 322500},
  {0x1340B, 322526},
  {0x1340C, 322552},
  {0x1340D, 295862},
  {0x1340E, 295888},
  {0x1340F, 295914},
  {0x13410, 295940},
  {0x13411, 295966},
  {0x13412, 295992},
  {0x13413, 296018},
  {0x13414, 296044},
  {0x13415, 296071},
  {0x13416, 296098},
  {0x13417, 296124},
  {0x13418, 296150},
  {0x13419, 296176},
  {0x1341A, 296202},
  {0x1341B, 296228},
  {0x1341C, 296254},
  {0x1341D, 296280},
  {0x1341E, 296306},
  {0x1341F, 296332},
  {0x13420, 296358},
  {0x13421, 296384},
  {0x13422, 296410},
  {0x13423, 296436},
  {0x13424, 296462},
  {0x13425, 296488},
  {0x13426, 296514},
  {0x13427, 296540},
  {0x13428, 296566},
  {0x13429, 296592},
  {0x1342A, 296618},
  {0x1342B, 296644},
  {0x1342C, 296670},
  {0x1342D, 296696},
  {0x1342E, 296722},
  {0x1342F, 318797},
  {0x13430, 320120},
  {0x13431, 304925},
  {0x13432, 305672},
  {0x13433, 305520},
  {0x13434, 305634},
  {0x13435, 305479},
  {0x13436, 313059},
  {0x13437, 297035},
  {0x13438, 301282},
  {0x13439, 305563},
  {0x1343A, 305600},
  {0x1343B, 305442},
  {0x1343C, 296999},
  {0x1343D, 301248},
  {0x1343E, 297069},
  {0x1343F, 301314},
  {0x13440, 307866},
  {0x13441, 302992},
  {0x13442, 304859},
  {0x13443, 306164},
  {0x13444, 304890},
  {0x13445, 316917},
  {0x13446, 320963},
  {0x13447, 308559},
  {0x13448, 308096},
  {0x13449, 308258},
  {0x1344A, 308511},
  {0x1344B, 308415},
  {0x1344C, 308149},
  {0x1344D, 308361},
  {0x1344E, 308045},
  {0x1344F, 308609},
  {0x13450, 307943},
  {0x13451, 308304},
  {0x13452, 308214},
  {0x13453, 308459},
  {0x13454, 307990},
  {0x13455, 307906},
  {0x13460, 322578},
  {0x13461, 322604},
  {0x13462, 322630},
  {0x13463, 322656},
  {0x13464, 322682},
  {0x13465, 322708},
  {0x13466, 322734},
  {0x13467, 322760},
  {0x13468, 322786},
  {0x13469, 322812},
  {0x1346A, 322838},
  {0x1346B, 322864},
  {0x1346C, 322890},
  {0x1346D, 322916},
  {0x1346E, 322942},
  {0x1346F, 322968},
  {0x13470, 322994},
  {0x13471, 323020},
  {0x13472, 323046},
  {0x13473, 323072},
  {0x13474, 323098},
  {0x13475, 323124},
  {0x13476, 323150},
  {0x13477, 323176},
  {0x13478, 323202},
  {0x13479, 323228},
  {0x1347A, 323254},
  {0x1347B, 323280},
  {0x1347C, 323306},
  {0x1347D, 323332},
  {0x1347E, 323358},
  {0x1347F, 323384},
  {0x13480, 323410},
  {0x13481, 323436},
  {0x13482, 323462},
  {0x13483, 323488},
  {0x13484, 323514},
  {0x13485, 323540},
  {0x13486, 323566},
  {0x13487, 323592},
  {0x13488, 323618},
  {0x13489, 323644},
  {0x1348A, 323670},
  {0x1348B, 323696},
  {0x1348C, 323722},
  {0x1348D, 323748},
  {0x1348E, 323774},
  {0x1348F, 323800},
  {0x13490, 323826},
  {0x13491, 323852},
  {0x13492, 323878},
  {0x13493, 323904},
  {0x13494, 323930},
  {0x13495, 323956},
  {0x13496, 323982},
  {0x13497, 324008},
  {0x13498, 324034},
  {0x13499, 324060},
  {0x1349A, 324086},
  {0x1349B, 324112},
  {0x1349C, 324138},
  {0x1349D, 324164},
  {0x1349E, 324190},
  {0x1349F, 324216},
  {0x134A0, 324242},
  {0x134A1, 324268},
  {0x134A2, 324294},
  {0x134A3, 324320},
  {0x134A4, 324346},
  {0x134A5, 324372},
  {0x134A6, 324398},
  {0x134A7, 324424},
  {0x134A8, 324450},
  {0x134A9, 324476},
  {0x134AA, 324502},
  {0x134AB, 324528},
  {0x134AC, 324554},
  {0x134AD, 324580},
  {0x134AE, 324606},
  {0x134AF, 324632},
  {0x134B0, 324658},
  {0x134B1, 324684},
  {0x134B2, 324710},
  {0x134B3, 324736},
  {0x134B4, 324762},
  {0x134B5, 324788},
  {0x134B6, 324814},
  {0x134B7, 324840},
  {0x134B8, 324866},
  {0x134B9, 324892},
  {0x134BA, 324918},
  {0x134BB, 324944},
  {0x134BC, 324970},
  {0x134BD, 324996},
  {0x134BE, 325022},
  {0x134BF, 325048},
  {0x134C0, 325074},
  {0x134C1, 325100},
  {0x134C2, 325126},
  {0x134C3, 325152},
  {0x134C4, 325178},
  {0x134C5, 325204},
  {0x134C6, 325230},
  {0x134C7, 325256},
  {0x134C8, 325282},
  {0x134C9, 325308},
  {0x134CA, 325334},
  {0x134CB, 325360},
  {0x134CC, 325386},
  {0x134CD, 325412},
  {0x134CE, 325438},
  {0x134CF, 325464},
  {0x134D0, 325490},
  {0x134D1, 325516},
  {0x134D2, 325542},
  {0x134D3, 325568},
  {0x134D4, 325594},
  {0x134D5, 325620},
  {0x134D6, 325646},
  {0x134D7, 325672},
  {0x134D8, 325698},
  {0x134D9, 325724},
  {0x134DA, 325750},
  {0x134DB, 325776},
  {0x134DC, 325802},
  {0x134DD, 325828},
  {0x134DE, 325854},
  {0x134DF, 325880},
  {0x134E0, 325906},
  {0x134E1, 325932},
  {0x134E2, 325958},
  {0x134E3, 325984},
  {0x134E4, 326010},
  {0x134E5, 326036},
  {0x134E6, 326062},
  {0x134E7, 326088},
  {0x134E8, 326114},
  {0x134E9, 326140},
  {0x134EA, 326166},
  {0x134EB, 326192},
  {0x134EC, 326218},
  {0x134ED, 326244},
  {0x134EE, 326270},
  {0x134EF, 326296},
  {0x134F0, 326322},
  {0x134F1, 326348},
  {0x134F2, 326374},
  {0x134F3, 326400},
  {0x134F4, 326426},
  {0x134F5, 326452},
  {0x134F6, 326478},
  {0x134F7, 326504},
  {0x134F8, 326530},
  {0x134F9, 326556},
  {0x134FA, 326582},
  {0x134FB, 326608},
  {0x134FC, 326634},
  {0x134FD, 326660},
  {0x134FE, 326686},
  {0x134FF, 326712},
  {0x13500, 326738},
  {0x13501, 326764},
  {0x13502, 326790},
  {0x13503, 326816},
  {0x13504, 326842},
  {0x13505, 326868},
  {0x13506, 326894},
  {0x13507, 326920},
  {0x13508, 326946},
  {0x13509, 326972},
  {0x1350A, 326998},
  {0x1350B, 327024},
  {0x1350C, 327050},
  {0x1350D, 327076},
  {0x1350E, 327102},
  {0x1350F, 327128},
  {0x13510, 327154},
  {0x13511, 327180},
  {0x13512, 327206},
  {0x13513, 327232},
  {0x13514, 327258},
  {0x13515, 327284},
  {0x13516, 327310},
  {0x13517, 327336},
  {0x13518, 327362},
  {0x13519, 327388},
  {0x1351A, 327414},
  {0x1351B, 327440},
  {0x1351C, 327466},
  {0x1351D, 327492},
  {0x1351E, 327518},
  {0x1351F, 327544},
  {0x13520, 327570},
  {0x13521, 327596},
  {0x13522, 327622},
  {0x13523, 327648},
  {0x13524, 327674},
  {0x13525, 327700},
  {0x13526, 327726},
  {0x13527, 327752},
  {0x13528, 327778},
  {0x13529, 327804},
  {0x1352A, 327830},
  {0x1352B, 327856},
  {0x1352C, 327882},
  {0x1352D, 327908},
  {0x1352E, 327934},
  {0x1352F, 327960},
  {0x13530, 327986},
  {0x13531, 328012},
  {0x13532, 328038},
  {0x13533, 328064},
  {0x13534, 328090},
  {0x13535, 328116},
  {0x13536, 328142},
  {0x13537, 328168},
  {0x13538, 328194},
  {0x13539, 328220},
  {0x1353A, 328246},
  {0x1353B, 328272},
  {0x1353C, 328298},
  {0x1353D, 328324},
  {0x1353E, 328350},
  {0x1353F, 328376},
  {0x13540, 328402},
  {0x13541, 328428},
  {0x13542, 328454},
  {0x13543, 328480},
  {0x13544, 328506},
  {0x13545, 328532},
  {0x13546, 328558},
  {0x13547, 328584},
  {0x13548, 328610},
  {0x13549, 328636},
  {0x1354A, 328662},
  {0x1354B, 328688},
  {0x1354C, 328714},
  {0x1354D, 328740},
  {0x1354E, 328766},
  {0x1354F, 328792},
  {0x13550, 328818},
  {0x13551, 328844},
  {0x13552, 328870},
  {0x13553, 328896},
  {0x13554, 328922},
  {0x13555, 328948},
  {0x13556, 328974},
  {0x13557, 329000},
  {0x13558, 329026},
  {0x13559, 329052},
  {0x1355A, 329078},
  {0x1355B, 329104},
  {0x1355C, 329130},
  {0x1355D, 329156},
  {0x1355E, 329182},
  {0x1355F, 329208},
  {0x13560, 329234},
  {0x13561, 329260},
  {0x13562, 329286},
  {0x13563, 329312},
  {0x13564, 329338},
  {0x13565, 329364},
  {0x13566, 329390},
  {0x13567, 329416},
  {0x13568, 329442},
  {0x13569, 329468},
  {0x1356A, 329494},
  {0x1356B, 329520},
  {0x1356C, 329546},
  {0x1356D, 329572},
  {0x1356E, 329598},
  {0x1356F, 329624},
  {0x13570, 329650},
  {0x13571, 329676},
  {0x13572, 329702},
  {0x13573, 329728},
  {0x13574, 329754},
  {0x13575, 329780},
  {0x13576, 329806},
  {0x13577, 329832},
  {0x13578, 329858},
  {0x13579, 329884},
  {0x1357A, 329910},
  {0x1357B, 329936},
  {0x1357C, 329962},
  {0x1357D, 329988},
  {0x1357E, 330014},
  {0x1357F, 330040},
  {0x13580, 330066},
  {0x13581, 330092},
  {0x13582, 330118},
  {0x13583, 330144},
  {0x13584, 330170},
  {0x13585, 330196},
  {0x13586, 330222},
  {0x13587, 330248},
  {0x13588, 330274},
  {0x13589, 330300},
  {0x1358A, 330326},
  {0x1358B, 330352},
  {0x1358C, 330378},
  {0x1358D, 330404},
  {0x1358E, 330430},
  {0x1358F, 330456},
  {0x13590, 330482},
  {0x13591, 330508},
  {0x13592, 330534},
  {0x13593, 330560},
  {0x13594, 330586},
  {0x13595, 330612},
  {0x13596, 330638},
  {0x13597, 330664},
  {0x13598, 330690},
  {0x13599, 330716},
  {0x1359A, 330742},
  {0x1359B, 330768},
  {0x1359C, 330794},
  {0x1359D, 330820},
  {0x1359E, 330846},
  {0x1359F, 330872},
  {0x135A0, 330898},
  {0x135A1, 330924},
  {0x135A2, 330950},
  {0x135A3, 330976},
  {0x135A4, 331002},
  {0x135A5, 331028},
  {0x135A6, 331054},
  {0x135A7, 331080},
  {0x135A8, 331106},
  {0x135A9, 331132},
  {0x135AA, 331158},
  {0x135AB, 331184},
  {0x135AC, 331210},
  {0x135AD, 331236},
  {0x135AE, 331262},
  {0x135AF, 331288},
  {0x135B0, 331314},
  {0x135B1, 331340},
  {0x135B2, 331366},
  {0x135B3, 331392},
  {0x135B4, 331418},
  {0x135B5, 331444},
  {0x135B6, 331470},
  {0x135B7, 331496},
  {0x135B8, 331522},
  {0x135B9, 331548},
  {0x135BA, 331574},
  {0x135BB, 331600},
  {0x135BC, 331626},
  {0x135BD, 331652},
  {0x135BE, 331678},
  {0x135BF, 331704},
  {0x135C0, 331730},
  {0x135C1, 331756},
  {0x135C2, 331782},
  {0x135C3, 331808},
  {0x135C4, 331834},
  {0x135C5, 331860},
  {0x135C6, 331886},
  {0x135C7, 331912},
  {0x135C8, 331938},
  {0x135C9, 331964},
  {0x135CA, 331990},
  {0x135CB, 332016},
  {0x135CC, 332042},
  {0x135CD, 332068},
  {0x135CE, 332094},
  {0x135CF, 332120},
  {0x135D0, 332146},
  {0x135D1, 332172},
  {0x135D2, 332198},
  {0x135D3, 332224},
  {0x135D4, 332250},
  {0x135D5, 332276},
  {0x135D6, 332302},
  {0x135D7, 332328},
  {0x135D8, 332354},
  {0x135D9, 332380},
  {0x135DA, 332406},
  {0x135DB, 332432},
  {0x135DC, 332458},
  {0x135DD, 332484},
  {0x135DE, 332510},
  {0x135DF, 332536},
  {0x135E0, 332562},
  {0x135E1, 332588},
  {0x135E2, 332614},
  {0x135E3, 332640},
  {0x135E4, 332666},
  {0x135E5, 332692},
  {0x135E6, 332718},
  {0x135E7, 332744},
  {0x135E8, 332770},
  {0x135E9, 332796},
  {0x135EA, 332822},
  {0x135EB, 332848},
  {0x135EC, 332874},
  {0x135ED, 332900},
  {0x135EE, 332926},
  {0x135EF, 332952},
  {0x135F0, 332978},
  {0x135F1, 333004},
  {0x135F2, 333030},
  {0x135F3, 333056},
  {0x135F4, 333082},
  {0x135F5, 333108},
  {0x135F6, 333134},
  {0x135F7, 333160},
  {0x135F8, 333186},
  {0x135F9, 333212},
  {0x135FA, 333238},
  {0x135FB, 333264},
  {0x135FC, 333290},
  {0x135FD, 333316},
  {0x135FE, 333342},
  {0x135FF, 333368},
  {0x13600, 333394},
  {0x13601, 333420},
  {0x13602, 333446},
  {0x13603, 333472},
  {0x13604, 333498},
  {0x13605, 333524},
  {0x13606, 333550},
  {0x13607, 333576},
  {0x13608, 333602},
  {0x13609, 333628},
  {0x1360A, 333654},
  {0x1360B, 333680},
  {0x1360C, 333706},
  {0x1360D, 333732},
  {0x1360E, 333758},
  {0x1360F, 333784},
  {0x13610, 333810},
  {0x13611, 333836},
  {0x13612, 333862},
  {0x13613, 333888},
  {0x13614, 333914},
  {0x13615, 333940},
  {0x13616, 333966},
  {0x13617, 333992},
  {0x13618, 334018},
  {0x13619, 334044},
  {0x1361A, 334070},
  {0x1361B, 334096},
  {0x1361C, 334122},
  {0x1361D, 334148},
  {0x1361E, 334174},
  {0x1361F, 334200},
  {0x13620, 334226},
  {0x13621, 334252},
  {0x13622, 334278},
  {0x13623, 334304},
  {0x13624, 334330},
  {0x13625, 334356},
  {0x13626, 334382},
  {0x13627, 334408},
  {0x13628, 334434},
  {0x13629, 334460},
  {0x1362A, 334486},
  {0x1362B, 334512},
  {0x1362C, 334538},
  {0x1362D, 334564},
  {0x1362E, 334590},
  {0x1362F, 334616},
  {0x13630, 334642},
  {0x13631, 334668},
  {0x13632, 334694},
  {0x13633, 334720},
  {0x13634, 334746},
  {0x13635, 334772},
  {0x13636, 334798},
  {0x13637, 334824},
  {0x13638, 334850},
  {0x13639, 334876},
  {0x1363A, 334902},
  {0x1363B, 334928},
  {0x1363C, 334954},
  {0x1363D, 334980},
  {0x1363E, 335006},
  {0x1363F, 335032},
  {0x13640, 335058},
  {0x13641, 335084},
  {0x13642, 335110},
  {0x13643, 335136},
  {0x13644, 335162},
  {0x13645, 335188},
  {0x13646, 335214},
  {0x13647, 335240},
  {0x13648, 335266},
  {0x13649, 335292},
  {0x1364A, 335318},
  {0x1364B, 335344},
  {0x1364C, 335370},
  {0x1364D, 335396},
  {0x1364E, 335422},
  {0x1364F, 335448},
  {0x13650, 335474},
  {0x13651, 335500},
  {0x13652, 335526},
  {0x13653, 335552},
  {0x13654, 335578},
  {0x13655, 335604},
  {0x13656, 335630},
  {0x13657, 335656},
  {0x13658, 335682},
  {0x13659, 335708},
  {0x1365A, 335734},
  {0x1365B, 335760},
  {0x1365C, 335786},
  {0x1365D, 335812},
  {0x1365E, 335838},
  {0x1365F, 335864},
  {0x13660, 335890},
  {0x13661, 335916},
  {0x13662, 335942},
  {0x13663, 335968},
  {0x13664, 335994},
  {0x13665, 336020},
  {0x13666, 336046},
  {0x13667, 336072},
  {0x13668, 336098},
  {0x13669, 336124},
  {0x1366A, 336150},
  {0x1366B, 336176},
  {0x1366C, 336202},
  {0x1366D, 336228},
  {0x1366E, 336254},
  {0x1366F, 336280},
  {0x13670, 336306},
  {0x13671, 336332},
  {0x13672, 336358},
  {0x13673, 336384},
  {0x13674, 336410},
  {0x13675, 336436},
  {0x13676, 336462},
  {0x13677, 336488},
  {0x13678, 336514},
  {0x13679, 336540},
  {0x1367A, 336566},
  {0x1367B, 336592},
  {0x1367C, 336618},
  {0x1367D, 336644},
  {0x1367E, 336670},
  {0x1367F, 336696},
  {0x13680, 336722},
  {0x13681, 336748},
  {0x13682, 336774},
  {0x13683, 336800},
  {0x13684, 336826},
  {0x13685, 336852},
  {0x13686, 336878},
  {0x13687, 336904},
  {0x13688, 336930},
  {0x13689, 336956},
  {0x1368A, 336982},
  {0x1368B, 337008},
  {0x1368C, 337034},
  {0x1368D, 337060},
  {0x1368E, 337086},
  {0x1368F, 337112},
  {0x13690, 337138},
  {0x13691, 337164},
  {0x13692, 337190},
  {0x13693, 337216},
  {0x13694, 337242},
  {0x13695, 337268},
  {0x13696, 337294},
  {0x13697, 337320},
  {0x13698, 337346},
  {0x13699, 337372},
  {0x1369A, 337398},
  {0x1369B, 337424},
  {0x1369C, 337450},
  {0x1369D, 337476},
  {0x1369E, 337502},
  {0x1369F, 337528},
  {0x136A0, 337554},
  {0x136A1, 337580},
  {0x136A2, 337606},
  {0x136A3, 337632},
  {0x136A4, 337658},
  {0x136A5, 337684},
  {0x136A6, 337710},
  {0x136A7, 337736},
  {0x136A8, 337762},
  {0x136A9, 337788},
  {0x136AA, 337814},
  {0x136AB, 337840},
  {0x136AC, 337866},
  {0x136AD, 337892},
  {0x136AE, 337918},
  {0x136AF, 337944},
  {0x136B0, 337970},
  {0x136B1, 337996},
  {0x136B2, 338022},
  {0x136B3, 338048},
  {0x136B4, 338074},
  {0x136B5, 338100},
  {0x136B6, 338126},
  {0x136B7, 338152},
  {0x136B8, 338178},
  {0x136B9, 338204},
  {0x136BA, 338230},
  {0x136BB, 338256},
  {0x136BC, 338282},
  {0x136BD, 338308},
  {0x136BE, 338334},
  {0x136BF, 338360},
  {0x136C0, 338386},
  {0x136C1, 338412},
  {0x136C2, 338438},
  {0x136C3, 338464},
  {0x136C4, 338490},
  {0x136C5, 338516},
  {0x136C6, 338542},
  {0x136C7, 338568},
  {0x136C8, 338594},
  {0x136C9, 338620},
  {0x136CA, 338646},
  {0x136CB, 338672},
  {0x136CC, 338698},
  {0x136CD, 338724},
  {0x136CE, 338750},
  {0x136CF, 338776},
  {0x136D0, 338802},
  {0x136D1, 338828},
  {0x136D2, 338854},
  {0x136D3, 338880},
  {0x136D4, 338906},
  {0x136D5, 338932},
  {0x136D6, 338958},
  {0x136D7, 338984},
  {0x136D8, 339010},
  {0x136D9, 339036},
  {0x136DA, 339062},
  {0x136DB, 339088},
  {0x136DC, 339114},
  {0x136DD, 339140},
  {0x136DE, 339166},
  {0x136DF, 339192},
  {0x136E0, 339218},
  {0x136E1, 339244},
  {0x136E2, 339270},
  {0x136E3, 339296},
  {0x136E4, 339322},
  {0x136E5, 339348},
  {0x136E6, 339374},
  {0x136E7, 339400},
  {0x136E8, 339426},
  {0x136E9, 339452},
  {0x136EA, 339478},
  {0x136EB, 339504},
  {0x136EC, 339530},
  {0x136ED, 339556},
  {0x136EE, 339582},
  {0x136EF, 339608},
  {0x136F0, 339634},
  {0x136F1, 339660},
  {0x136F2, 339686},
  {0x136F3, 339712},
  {0x136F4, 339738},
  {0x136F5, 339764},
  {0x136F6, 339790},
  {0x136F7, 339816},
  {0x136F8, 339842},
  {0x136F9, 339868},
  {0x136FA, 339894},
  {0x136FB, 339920},
  {0x136FC, 339946},
  {0x136FD, 339972},
  {0x136FE, 339998},
  {0x136FF, 340024},
  {0x13700, 340050},
  {0x13701, 340076},
  {0x13702, 340102},
  {0x13703, 340128},
  {0x13704, 340154},
  {0x13705, 340180},
  {0x13706, 340206},
  {0x13707, 340232},
  {0x13708, 340258},
  {0x13709, 340284},
  {0x1370A, 340310},
  {0x1370B, 340336},
  {0x1370C, 340362},
  {0x1370D, 340388},
  {0x1370E, 340414},
  {0x1370F, 340440},
  {0x13710, 340466},
  {0x13711, 340492},
  {0x13712, 340518},
  {0x13713, 340544},
  {0x13714, 340570},
  {0x13715, 340596},
  {0x13716, 340622},
  {0x13717, 340648},
  {0x13718, 340674},
  {0x13719, 340700},
  {0x1371A, 340726},
  {0x1371B, 340752},
  {0x1371C, 340778},
  {0x1371D, 340804},
  {0x1371E, 340830},
  {0x1371F, 340856},
  {0x13720, 340882},
  {0x13721, 340908},
  {0x13722, 340934},
  {0x13723, 340960},
  {0x13724, 340986},
  {0x13725, 341012},
  {0x13726, 341038},
  {0x13727, 341064},
  {0x13728, 341090},
  {0x13729, 341116},
  {0x1372A, 341142},
  {0x1372B, 341168},
  {0x1372C, 341194},
  {0x1372D, 341220},
  {0x1372E, 341246},
  {0x1372F, 341272},
  {0x13730, 341298},
  {0x13731, 341324},
  {0x13732, 341350},
  {0x13733, 341376},
  {0x13734, 341402},
  {0x13735, 341428},
  {0x13736, 341454},
  {0x13737, 341480},
  {0x13738, 341506},
  {0x13739, 341532},
  {0x1373A, 341558},
  {0x1373B, 341584},
  {0x1373C, 341610},
  {0x1373D, 341636},
  {0x1373E, 341662},
  {0x1373F, 341688},
  {0x13740, 341714},
  {0x13741, 341740},
  {0x13742, 341766},
  {0x13743, 341792},
  {0x13744, 341818},
  {0x13745, 341844},
  {0x13746, 341870},
  {0x13747, 341896},
  {0x13748, 341922},
  {0x13749, 341948},
  {0x1374A, 341974},
  {0x1374B, 342000},
  {0x1374C, 342026},
  {0x1374D, 342052},
  {0x1374E, 342078},
  {0x1374F, 342104},
  {0x13750, 342130},
  {0x13751, 342156},
  {0x13752, 342182},
  {0x13753, 342208},
  {0x13754, 342234},
  {0x13755, 342260},
  {0x13756, 342286},
  {0x13757, 342312},
  {0x13758, 342338},
  {0x13759, 342364},
  {0x1375A, 342390},
  {0x1375B, 342416},
  {0x1375C, 342442},
  {0x1375D, 342468},
  {0x1375E, 342494},
  {0x1375F, 342520},
  {0x13760, 342546},
  {0x13761, 342572},
  {0x13762, 342598},
  {0x13763, 342624},
  {0x13764, 342650},
  {0x13765, 342676},
  {0x13766, 342702},
  {0x13767, 342728},
  {0x13768, 342754},
  {0x13769, 342780},
  {0x1376A, 342806},
  {0x1376B, 342832},
  {0x1376C, 342858},
  {0x1376D, 342884},
  {0x1376E, 342910},
  {0x1376F, 342936},
  {0x13770, 342962},
  {0x13771, 342988},
  {0x13772, 343014},
  {0x13773, 343040},
  {0x13774, 343066},
  {0x13775, 343092},
  {0x13776, 343118},
  {0x13777, 343144},
  {0x13778, 343170},
  {0x13779, 343196},
  {0x1377A, 343222},
  {0x1377B, 343248},
  {0x1377C, 343274},
  {0x1377D, 343300},
  {0x1377E, 343326},
  {0x1377F, 343352},
  {0x13780, 343378},
  {0x13781, 343404},
  {0x13782, 343430},
  {0x13783, 343456},
  {0x13784, 343482},
  {0x13785, 343508},
  {0x13786, 343534},
  {0x13787, 343560},
  {0x13788, 343586},
  {0x13789, 343612},
  {0x1378A, 343638},
  {0x1378B, 343664},
  {0x1378C, 343690},
  {0x1378D, 343716},
  {0x1378E, 343742},
  {0x1378F, 343768},
  {0x13790, 343794},
  {0x13791, 343820},
  {0x13792, 343846},
  {0x13793, 343872},
  {0x13794, 343898},
  {0x13795, 343924},
  {0x13796, 343950},
  {0x13797, 343976},
  {0x13798, 344002},
  {0x13799, 344028},
  {0x1379A, 344054},
  {0x1379B, 344080},
  {0x1379C, 344106},
  {0x1379D, 344132},
  {0x1379E, 344158},
  {0x1379F, 344184},
  {0x137A0, 344210},
  {0x137A1, 344236},
  {0x137A2, 344262},
  {0x137A3, 344288},
  {0x137A4, 344314},
  {0x137A5, 344340},
  {0x137A6, 344366},
  {0x137A7, 344392},
  {0x137A8, 344418},
  {0x137A9, 344444},
  {0x137AA, 344470},
  {0x137AB, 344496},
  {0x137AC, 344522},
  {0x137AD, 344548},
  {0x137AE, 344574},
  {0x137AF, 344600},
  {0x137B0, 344626},
  {0x137B1, 344652},
  {0x137B2, 344678},
  {0x137B3, 344704},
  {0x137B4, 344730},
  {0x137B5, 344756},
  {0x137B6, 344782},
  {0x137B7, 344808},
  {0x137B8, 344834},
  {0x137B9, 344860},
  {0x137BA, 344886},
  {0x137BB, 344912},
  {0x137BC, 344938},
  {0x137BD, 344964},
  {0x137BE, 344990},
  {0x137BF, 345016},
  {0x137C0, 345042},
  {0x137C1, 345068},
  {0x137C2, 345094},
  {0x137C3, 345120},
  {0x137C4, 345146},
  {0x137C5, 345172},
  {0x137C6, 345198},
  {0x137C7, 345224},
  {0x137C8, 345250},
  {0x137C9, 345276},
  {0x137CA, 345302},
  {0x137CB, 345328},
  {0x137CC, 345354},
  {0x137CD, 345380},
  {0x137CE, 345406},
  {0x137CF, 345432},
  {0x137D0, 345458},
  {0x137D1, 345484},
  {0x137D2, 345510},
  {0x137D3, 345536},
  {0x137D4, 345562},
  {0x137D5, 345588},
  {0x137D6, 345614},
  {0x137D7, 345640},
  {0x137D8, 345666},
  {0x137D9, 345692},
  {0x137DA, 345718},
  {0x137DB, 345744},
  {0x137DC, 345770},
  {0x137DD, 345796},
  {0x137DE, 345822},
  {0x137DF, 345848},
  {0x137E0, 345874},
  {0x137E1, 345900},
  {0x137E2, 345926},
  {0x137E3, 345952},
  {0x137E4, 345978},
  {0x137E5, 346004},
  {0x137E6, 346030},
  {0x137E7, 346056},
  {0x137E8, 346082},
  {0x137E9, 346108},
  {0x137EA, 346134},
  {0x137EB, 346160},
  {0x137EC, 346186},
  {0x137ED, 346212},
  {0x137EE, 346238},
  {0x137EF, 346264},
  {0x137F0, 346290},
  {0x137F1, 346316},
  {0x137F2, 346342},
  {0x137F3, 346368},
  {0x137F4, 346394},
  {0x137F5, 346420},
  {0x137F6, 346446},
  {0x137F7, 346472},
  {0x137F8, 346498},
  {0x137F9, 346524},
  {0x137FA, 346550},
  {0x137FB, 346576},
  {0x137FC, 346602},
  {0x137FD, 346628},
  {0x137FE, 346654},
  {0x137FF, 346680},
  {0x13800, 346706},
  {0x13801, 346732},
  {0x13802, 346758},
  {0x13803, 346784},
  {0x13804, 346810},
  {0x13805, 346836},
  {0x13806, 346862},
  {0x13807, 346888},
  {0x13808, 346914},
  {0x13809, 346940},
  {0x1380A, 346966},
  {0x1380B, 346992},
  {0x1380C, 347018},
  {0x1380D, 347044},
  {0x1380E, 347070},
  {0x1380F, 347096},
  {0x13810, 347122},
  {0x13811, 347148},
  {0x13812, 347174},
  {0x13813, 347200},
  {0x13814, 347226},
  {0x13815, 347252},
  {0x13816, 347278},
  {0x13817, 347304},
  {0x13818, 347330},
  {0x13819, 347356},
  {0x1381A, 347382},
  {0x1381B, 347408},
  {0x1381C, 347434},
  {0x1381D, 347460},
  {0x1381E, 347486},
  {0x1381F, 347512},
  {0x13820, 347538},
  {0x13821, 347564},
  {0x13822, 347590},
  {0x13823, 347616},
  {0x13824, 347642},
  {0x13825, 347668},
  {0x13826, 347694},
  {0x13827, 347720},
  {0x13828, 347746},
  {0x13829, 347772},
  {0x1382A, 347798},
  {0x1382B, 347824},
  {0x1382C, 347850},
  {0x1382D, 347876},
  {0x1382E, 347902},
  {0x1382F, 347928},
  {0x13830, 347954},
  {0x13831, 347980},
  {0x13832, 348006},
  {0x13833, 348032},
  {0x13834, 348058},
  {0x13835, 348084},
  {0x13836, 348110},
  {0x13837, 348136},
  {0x13838, 348162},
  {0x13839, 348188},
  {0x1383A, 348214},
  {0x1383B, 348240},
  {0x1383C, 348266},
  {0x1383D, 348292},
  {0x1383E, 348318},
  {0x1383F, 348344},
  {0x13840, 348370},
  {0x13841, 348396},
  {0x13842, 348422},
  {0x13843, 348448},
  {0x13844, 348474},
  {0x13845, 348500},
  {0x13846, 348526},
  {0x13847, 348552},
  {0x13848, 348578},
  {0x13849, 348604},
  {0x1384A, 348630},
  {0x1384B, 348656},
  {0x1384C, 348682},
  {0x1384D, 348708},
  {0x1384E, 348734},
  {0x1384F, 348760},
  {0x13850, 348786},
  {0x13851, 348812},
  {0x13852, 348838},
  {0x13853, 348864},
  {0x13854, 348890},
  {0x13855, 348916},
  {0x13856, 348942},
  {0x13857, 348968},
  {0x13858, 348994},
  {0x13859, 349020},
  {0x1385A, 349046},
  {0x1385B, 349072},
  {0x1385C, 349098},
  {0x1385D, 349124},
  {0x1385E, 349150},
  {0x1385F, 349176},
  {0x13860, 349202},
  {0x13861, 349228},
  {0x13862, 349254},
  {0x13863, 349280},
  {0x13864, 349306},
  {0x13865, 349332},
  {0x13866, 349358},
  {0x13867, 349384},
  {0x13868, 349410},
  {0x13869, 349436},
  {0x1386A, 349462},
  {0x1386B, 349488},
  {0x1386C, 349514},
  {0x1386D, 349540},
  {0x1386E, 349566},
  {0x1386F, 349592},
  {0x13870, 349618},
  {0x13871, 349644},
  {0x13872, 349670},
  {0x13873, 349696},
  {0x13874, 349722},
  {0x13875, 349748},
  {0x13876, 349774},
  {0x13877, 349800},
  {0x13878, 349826},
  {0x13879, 349852},
  {0x1387A, 349878},
  {0x1387B, 349904},
  {0x1387C, 349930},
  {0x1387D, 349956},
  {0x1387E, 349982},
  {0x1387F, 350008},
  {0x13880, 350034},
  {0x13881, 350060},
  {0x13882, 350086},
  {0x13883, 350112},
  {0x13884, 350138},
  {0x13885, 350164},
  {0x13886, 350190},
  {0x13887, 350216},
  {0x13888, 350242},
  {0x13889, 350268},
  {0x1388A, 350294},
  {0x1388B, 350320},
  {0x1388C, 350346},
  {0x1388D, 350372},
  {0x1388E, 350398},
  {0x1388F, 350424},
  {0x13890, 350450},
  {0x13891, 350476},
  {0x13892, 350502},
  {0x13893, 350528},
  {0x13894, 350554},
  {0x13895, 350580},
  {0x13896, 350606},
  {0x13897, 350632},
  {0x13898, 350658},
  {0x13899, 350684},
  {0x1389A, 350710},
  {0x1389B, 350736},
  {0x1389C, 350762},
  {0x1389D, 350788},
  {0x1389E, 350814},
  {0x1389F, 350840},
  {0x138A0, 350866},
  {0x138A1, 350892},
  {0x138A2, 350918},
  {0x138A3, 350944},
  {0x138A4, 350970},
  {0x138A5, 350996},
  {0x138A6, 351022},
  {0x138A7, 351048},
  {0x138A8, 351074},
  {0x138A9, 351100},
  {0x138AA, 351126},
  {0x138AB, 351152},
  {0x138AC, 351178},
  {0x138AD, 351204},
  {0x138AE, 351230},
  {0x138AF, 351256},
  {0x138B0, 351282},
  {0x138B1, 351308},
  {0x138B2, 351334},
  {0x138B3, 351360},
  {0x138B4, 351386},
  {0x138B5, 351412},
  {0x138B6, 351438},
  {0x138B7, 351464},
  {0x138B8, 351490},
  {0x138B9, 351516},
  {0x138BA, 351542},
  {0x138BB, 351568},
  {0x138BC, 351594},
  {0x138BD, 351620},
  {0x138BE, 351646},
  {0x138BF, 351672},
  {0x138C0, 351698},
  {0x138C1, 351724},
  {0x138C2, 351750},
  {0x138C3, 351776},
  {0x138C4, 351802},
  {0x138C5, 351828},
  {0x138C6, 351854},
  {0x138C7, 351880},
  {0x138C8, 351906},
  {0x138C9, 351932},
  {0x138CA, 351958},
  {0x138CB, 351984},
  {0x138CC, 352010},
  {0x138CD, 352036},
  {0x138CE, 352062},
  {0x138CF, 352088},
  {0x138D0, 352114},
  {0x138D1, 352140},
  {0x138D2, 352166},
  {0x138D3, 352192},
  {0x138D4, 352218},
  {0x138D5, 352244},
  {0x138D6, 352270},
  {0x138D7, 352296},
  {0x138D8, 352322},
  {0x138D9, 352348},
  {0x138DA, 352374},
  {0x138DB, 352400},
  {0x138DC, 352426},
  {0x138DD, 352452},
  {0x138DE, 352478},
  {0x138DF, 352504},
  {0x138E0, 352530},
  {0x138E1, 352556},
  {0x138E2, 352582},
  {0x138E3, 352608},
  {0x138E4, 352634},
  {0x138E5, 352660},
  {0x138E6, 352686},
  {0x138E7, 352712},
  {0x138E8, 352738},
  {0x138E9, 352764},
  {0x138EA, 352790},
  {0x138EB, 352816},
  {0x138EC, 352842},
  {0x138ED, 352868},
  {0x138EE, 352894},
  {0x138EF, 352920},
  {0x138F0, 352946},
  {0x138F1, 352972},
  {0x138F2, 352998},
  {0x138F3, 353024},
  {0x138F4, 353050},
  {0x138F5, 353076},
  {0x138F6, 353102},
  {0x138F7, 353128},
  {0x138F8, 353154},
  {0x138F9, 353180},
  {0x138FA, 353206},
  {0x138FB, 353232},
  {0x138FC, 353258},
  {0x138FD, 353284},
  {0x138FE, 353310},
  {0x138FF, 353336},
  {0x13900, 353362},
  {0x13901, 353388},
  {0x13902, 353414},
  {0x13903, 353440},
  {0x13904, 353466},
  {0x13905, 353492},
  {0x13906, 353518},
  {0x13907, 353544},
  {0x13908, 353570},
  {0x13909, 353596},
  {0x1390A, 353622},
  {0x1390B, 353648},
  {0x1390C, 353674},
  {0x1390D, 353700},
  {0x1390E, 353726},
  {0x1390F, 353752},
  {0x13910, 353778},
  {0x13911, 353804},
  {0x13912, 353830},
  {0x13913, 353856},
  {0x13914, 353882},
  {0x13915, 353908},
  {0x13916, 353934},
  {0x13917, 353960},
  {0x13918, 353986},
  {0x13919, 354012},
  {0x1391A, 354038},
  {0x1391B, 354064},
  {0x1391C, 354090},
  {0x1391D, 354116},
  {0x1391E, 354142},
  {0x1391F, 354168},
  {0x13920, 354194},
  {0x13921, 354220},
  {0x13922, 354246},
  {0x13923, 354272},
  {0x13924, 354298},
  {0x13925, 354324},
  {0x13926, 354350},
  {0x13927, 354376},
  {0x13928, 354402},
  {0x13929, 354428},
  {0x1392A, 354454},
  {0x1392B, 354480},
  {0x1392C, 354506},
  {0x1392D, 354532},
  {0x1392E, 354558},
  {0x1392F, 354584},
  {0x13930, 354610},
  {0x13931, 354636},
  {0x13932, 354662},
  {0x13933, 354688},
  {0x13934, 354714},
  {0x13935, 354740},
  {0x13936, 354766},
  {0x13937, 354792},
  {0x13938, 354818},
  {0x13939, 354844},
  {0x1393A, 354870},
  {0x1393B, 354896},
  {0x1393C, 354922},
  {0x1393D, 354948},
  {0x1393E, 354974},
  {0x1393F, 355000},
  {0x13940, 355026},
  {0x13941, 355052},
  {0x13942, 355078},
  {0x13943, 355104},
  {0x13944, 355130},
  {0x13945, 355156},
  {0x13946, 355182},
  {0x13947, 355208},
  {0x13948, 355234},
  {0x13949, 355260},
  {0x1394A, 355286},
  {0x1394B, 355312},
  {0x1394C, 355338},
  {0x1394D, 355364},
  {0x1394E, 355390},
  {0x1394F, 355416},
  {0x13950, 355442},
  {0x13951, 355468},
  {0x13952, 355494},
  {0x13953, 355520},
  {0x13954, 355546},
  {0x13955, 355572},
  {0x13956, 355598},
  {0x13957, 355624},
  {0x13958, 355650},
  {0x13959, 355676},
  {0x1395A, 355702},
  {0x1395B, 355728},
  {0x1395C, 355754},
  {0x1395D, 355780},
  {0x1395E, 355806},
  {0x1395F, 355832},
  {0x13960, 355858},
  {0x13961, 355884},
  {0x13962, 355910},
  {0x13963, 355936},
  {0x13964, 355962},
  {0x13965, 355988},
  {0x13966, 356014},
  {0x13967, 356040},
  {0x13968, 356066},
  {0x13969, 356092},
  {0x1396A, 356118},
  {0x1396B, 356144},
  {0x1396C, 356170},
  {0x1396D, 356196},
  {0x1396E, 356222},
  {0x1396F, 356248},
  {0x13970, 356274},
  {0x13971, 356300},
  {0x13972, 356326},
  {0x13973, 356352},
  {0x13974, 356378},
  {0x13975, 356404},
  {0x13976, 356430},
  {0x13977, 356456},
  {0x13978, 356482},
  {0x13979, 356508},
  {0x1397A, 356534},
  {0x1397B, 356560},
  {0x1397C, 356586},
  {0x1397D, 356612},
  {0x1397E, 356638},
  {0x1397F, 356664},
  {0x13980, 356690},
  {0x13981, 356716},
  {0x13982, 356742},
  {0x13983, 356768},
  {0x13984, 356794},
  {0x13985, 356820},
  {0x13986, 356846},
  {0x13987, 356872},
  {0x13988, 356898},
  {0x13989, 356924},
  {0x1398A, 356950},
  {0x1398B, 356976},
  {0x1398C, 357002},
  {0x1398D, 357028},
  {0x1398E, 357054},
  {0x1398F, 357080},
  {0x13990, 357106},
  {0x13991, 357132},
  {0x13992, 357158},
  {0x13993, 357184},
  {0x13994, 357210},
  {0x13995, 357236},
  {0x13996, 357262},
  {0x13997, 357288},
  {0x13998, 357314},
  {0x13999, 357340},
  {0x1399A, 357366},
  {0x1399B, 357392},
  {0x1399C, 357418},
  {0x1399D, 357444},
  {0x1399E, 357470},
  {0x1399F, 357496},
  {0x139A0, 357522},
  {0x139A1, 357548},
  {0x139A2, 357574},
  {0x139A3, 357600},
  {0x139A4, 357626},
  {0x139A5, 357652},
  {0x139A6, 357678},
  {0x139A7, 357704},
  {0x139A8, 357730},
  {0x139A9, 357756},
  {0x139AA, 357782},
  {0x139AB, 357808},
  {0x139AC, 357834},
  {0x139AD, 357860},
  {0x139AE, 357886},
  {0x139AF, 357912},
  {0x139B0, 357938},
  {0x139B1, 357964},
  {0x139B2, 357990},
  {0x139B3, 358016},
  {0x139B4, 358042},
  {0x139B5, 358068},
  {0x139B6, 358094},
  {0x139B7, 358120},
  {0x139B8, 358146},
  {0x139B9, 358172},
  {0x139BA, 358198},
  {0x139BB, 358224},
  {0x139BC, 358250},
  {0x139BD, 358276},
  {0x139BE, 358302},
  {0x139BF, 358328},
  {0x139C0, 358354},
  {0x139C1, 358380},
  {0x139C2, 358406},
  {0x139C3, 358432},
  {0x139C4, 358458},
  {0x139C5, 358484},
  {0x139C6, 358510},
  {0x139C7, 358536},
  {0x139C8, 358562},
  {0x139C9, 358588},
  {0x139CA, 358614},
  {0x139CB, 358640},
  {0x139CC, 358666},
  {0x139CD, 358692},
  {0x139CE, 358718},
  {0x139CF, 358744},
  {0x139D0, 358770},
  {0x139D1, 358796},
  {0x139D2, 358822},
  {0x139D3, 358848},
  {0x139D4, 358874},
  {0x139D5, 358900},
  {0x139D6, 358926},
  {0x139D7, 358952},
  {0x139D8, 358978},
  {0x139D9, 359004},
  {0x139DA, 359030},
  {0x139DB, 359056},
  {0x139DC, 359082},
  {0x139DD, 359108},
  {0x139DE, 359134},
  {0x139DF, 359160},
  {0x139E0, 359186},
  {0x139E1, 359212},
  {0x139E2, 359238},
  {0x139E3, 359264},
  {0x139E4, 359290},
  {0x139E5, 359316},
  {0x139E6, 359342},
  {0x139E7, 359368},
  {0x139E8, 359394},
  {0x139E9, 359420},
  {0x139EA, 359446},
  {0x139EB, 359472},
  {0x139EC, 359498},
  {0x139ED, 359524},
  {0x139EE, 359550},
  {0x139EF, 359576},
  {0x139F0, 359602},
  {0x139F1, 359628},
  {0x139F2, 359654},
  {0x139F3, 359680},
  {0x139F4, 359706},
  {0x139F5, 359732},
  {0x139F6, 359758},
  {0x139F7, 359784},
  {0x139F8, 359810},
  {0x139F9, 359836},
  {0x139FA, 359862},
  {0x139FB, 359888},
  {0x139FC, 359914},
  {0x139FD, 359940},
  {0x139FE, 359966},
  {0x139FF, 359992},
  {0x13A00, 360018},
  {0x13A01, 360044},
  {0x13A02, 360070},
  {0x13A03, 360096},
  {0x13A04, 360122},
  {0x13A05, 360148},
  {0x13A06, 360174},
  {0x13A07, 360200},
  {0x13A08, 360226},
  {0x13A09, 360252},
  {0x13A0A, 360278},
  {0x13A0B, 360304},
  {0x13A0C, 360330},
  {0x13A0D, 360356},
  {0x13A0E, 360382},
  {0x13A0F, 360408},
  {0x13A10, 360434},
  {0x13A11, 360460},
  {0x13A12, 360486},
  {0x13A13, 360512},
  {0x13A14, 360538},
  {0x13A15, 360564},
  {0x13A16, 360590},
  {0x13A17, 360616},
  {0x13A18, 360642},
  {0x13A19, 360668},
  {0x13A1A, 360694},
  {0x13A1B, 360720},
  {0x13A1C, 360746},
  {0x13A1D, 360772},
  {0x13A1E, 360798},
  {0x13A1F, 360824},
  {0x13A20, 360850},
  {0x13A21, 360876},
  {0x13A22, 360902},
  {0x13A23, 360928},
  {0x13A24, 360954},
  {0x13A25, 360980},
  {0x13A26, 361006},
  {0x13A27, 361032},
  {0x13A28, 361058},
  {0x13A29, 361084},
  {0x13A2A, 361110},
  {0x13A2B, 361136},
  {0x13A2C, 361162},
  {0x13A2D, 361188},
  {0x13A2E, 361214},
  {0x13A2F, 361240},
  {0x13A30, 361266},
  {0x13A31, 361292},
  {0x13A32, 361318},
  {0x13A33, 361344},
  {0x13A34, 361370},
  {0x13A35, 361396},
  {0x13A36, 361422},
  {0x13A37, 361448},
  {0x13A38, 361474},
  {0x13A39, 361500},
  {0x13A3A, 361526},
  {0x13A3B, 361552},
  {0x13A3C, 361578},
  {0x13A3D, 361604},
  {0x13A3E, 361630},
  {0x13A3F, 361656},
  {0x13A40, 361682},
  {0x13A41, 361708},
  {0x13A42, 361734},
  {0x13A43, 361760},
  {0x13A44, 361786},
  {0x13A45, 361812},
  {0x13A46, 361838},
  {0x13A47, 361864},
  {0x13A48, 361890},
  {0x13A49, 361916},
  {0x13A4A, 361942},
  {0x13A4B, 361968},
  {0x13A4C, 361994},
  {0x13A4D, 362020},
  {0x13A4E, 362046},
  {0x13A4F, 362072},
  {0x13A50, 362098},
  {0x13A51, 362124},
  {0x13A52, 362150},
  {0x13A53, 362176},
  {0x13A54, 362202},
  {0x13A55, 362228},
  {0x13A56, 362254},
  {0x13A57, 362280},
  {0x13A58, 362306},
  {0x13A59, 362332},
  {0x13A5A, 362358},
  {0x13A5B, 362384},
  {0x13A5C, 362410},
  {0x13A5D, 362436},
  {0x13A5E, 362462},
  {0x13A5F, 362488},
  {0x13A60, 362514},
  {0x13A61, 362540},
  {0x13A62, 362566},
  {0x13A63, 362592},
  {0x13A64, 362618},
  {0x13A65, 362644},
  {0x13A66, 362670},
  {0x13A67, 362696},
  {0x13A68, 362722},
  {0x13A69, 362748},
  {0x13A6A, 362774},
  {0x13A6B, 362800},
  {0x13A6C, 362826},
  {0x13A6D, 362852},
  {0x13A6E, 362878},
  {0x13A6F, 362904},
  {0x13A70, 362930},
  {0x13A71, 362956},
  {0x13A72, 362982},
  {0x13A73, 363008},
  {0x13A74, 363034},
  {0x13A75, 363060},
  {0x13A76, 363086},
  {0x13A77, 363112},
  {0x13A78, 363138},
  {0x13A79, 363164},
  {0x13A7A, 363190},
  {0x13A7B, 363216},
  {0x13A7C, 363242},
  {0x13A7D, 363268},
  {0x13A7E, 363294},
  {0x13A7F, 363320},
  {0x13A80, 363346},
  {0x13A81, 363372},
  {0x13A82, 363398},
  {0x13A83, 363424},
  {0x13A84, 363450},
  {0x13A85, 363476},
  {0x13A86, 363502},
  {0x13A87, 363528},
  {0x13A88, 363554},
  {0x13A89, 363580},
  {0x13A8A, 363606},
  {0x13A8B, 363632},
  {0x13A8C, 363658},
  {0x13A8D, 363684},
  {0x13A8E, 363710},
  {0x13A8F, 363736},
  {0x13A90, 363762},
  {0x13A91, 363788},
  {0x13A92, 363814},
  {0x13A93, 363840},
  {0x13A94, 363866},
  {0x13A95, 363892},
  {0x13A96, 363918},
  {0x13A97, 363944},
  {0x13A98, 363970},
  {0x13A99, 363996},
  {0x13A9A, 364022},
  {0x13A9B, 364048},
  {0x13A9C, 364074},
  {0x13A9D, 364100},
  {0x13A9E, 364126},
  {0x13A9F, 364152},
  {0x13AA0, 364178},
  {0x13AA1, 364204},
  {0x13AA2, 364230},
  {0x13AA3, 364256},
  {0x13AA4, 364282},
  {0x13AA5, 364308},
  {0x13AA6, 364334},
  {0x13AA7, 364360},
  {0x13AA8, 364386},
  {0x13AA9, 364412},
  {0x13AAA, 364438},
  {0x13AAB, 364464},
  {0x13AAC, 364490},
  {0x13AAD, 364516},
  {0x13AAE, 364542},
  {0x13AAF, 364568},
  {0x13AB0, 364594},
  {0x13AB1, 364620},
  {0x13AB2, 364646},
  {0x13AB3, 364672},
  {0x13AB4, 364698},
  {0x13AB5, 364724},
  {0x13AB6, 364750},
  {0x13AB7, 364776},
  {0x13AB8, 364802},
  {0x13AB9, 364828},
  {0x13ABA, 364854},
  {0x13ABB, 364880},
  {0x13ABC, 364906},
  {0x13ABD, 364932},
  {0x13ABE, 364958},
  {0x13ABF, 364984},
  {0x13AC0, 365010},
  {0x13AC1, 365036},
  {0x13AC2, 365062},
  {0x13AC3, 365088},
  {0x13AC4, 365114},
  {0x13AC5, 365140},
  {0x13AC6, 365166},
  {0x13AC7, 365192},
  {0x13AC8, 365218},
  {0x13AC9, 365244},
  {0x13ACA, 365270},
  {0x13ACB, 365296},
  {0x13ACC, 365322},
  {0x13ACD, 365348},
  {0x13ACE, 365374},
  {0x13ACF, 365400},
  {0x13AD0, 365426},
  {0x13AD1, 365452},
  {0x13AD2, 365478},
  {0x13AD3, 365504},
  {0x13AD4, 365530},
  {0x13AD5, 365556},
  {0x13AD6, 365582},
  {0x13AD7, 365608},
  {0x13AD8, 365634},
  {0x13AD9, 365660},
  {0x13ADA, 365686},
  {0x13ADB, 365712},
  {0x13ADC, 365738},
  {0x13ADD, 365764},
  {0x13ADE, 365790},
  {0x13ADF, 365816},
  {0x13AE0, 365842},
  {0x13AE1, 365868},
  {0x13AE2, 365894},
  {0x13AE3, 365920},
  {0x13AE4, 365946},
  {0x13AE5, 365972},
  {0x13AE6, 365998},
  {0x13AE7, 366024},
  {0x13AE8, 366050},
  {0x13AE9, 366076},
  {0x13AEA, 366102},
  {0x13AEB, 366128},
  {0x13AEC, 366154},
  {0x13AED, 366180},
  {0x13AEE, 366206},
  {0x13AEF, 366232},
  {0x13AF0, 366258},
  {0x13AF1, 366284},
  {0x13AF2, 366310},
  {0x13AF3, 366336},
  {0x13AF4, 366362},
  {0x13AF5, 366388},
  {0x13AF6, 366414},
  {0x13AF7, 366440},
  {0x13AF8, 366466},
  {0x13AF9, 366492},
  {0x13AFA, 366518},
  {0x13AFB, 366544},
  {0x13AFC, 366570},
  {0x13AFD, 366596},
  {0x13AFE, 366622},
  {0x13AFF, 366648},
  {0x13B00, 366674},
  {0x13B01, 366700},
  {0x13B02, 366726},
  {0x13B03, 366752},
  {0x13B04, 366778},
  {0x13B05, 366804},
  {0x13B06, 366830},
  {0x13B07, 366856},
  {0x13B08, 366882},
  {0x13B09, 366908},
  {0x13B0A, 366934},
  {0x13B0B, 366960},
  {0x13B0C, 366986},
  {0x13B0D, 367012},
  {0x13B0E, 367038},
  {0x13B0F, 367064},
  {0x13B10, 367090},
  {0x13B11, 367116},
  {0x13B12, 367142},
  {0x13B13, 367168},
  {0x13B14, 367194},
  {0x13B15, 367220},
  {0x13B16, 367246},
  {0x13B17, 367272},
  {0x13B18, 367298},
  {0x13B19, 367324},
  {0x13B1A, 367350},
  {0x13B1B, 367376},
  {0x13B1C, 367402},
  {0x13B1D, 367428},
  {0x13B1E, 367454},
  {0x13B1F, 367480},
  {0x13B20, 367506},
  {0x13B21, 367532},
  {0x13B22, 367558},
  {0x13B23, 367584},
  {0x13B24, 367610},
  {0x13B25, 367636},
  {0x13B26, 367662},
  {0x13B27, 367688},
  {0x13B28, 367714},
  {0x13B29, 367740},
  {0x13B2A, 367766},
  {0x13B2B, 367792},
  {0x13B2C, 367818},
  {0x13B2D, 367844},
  {0x13B2E, 367870},
  {0x13B2F, 367896},
  {0x13B30, 367922},
  {0x13B31, 367948},
  {0x13B32, 367974},
  {0x13B33, 368000},
  {0x13B34, 368026},
  {0x13B35, 368052},
  {0x13B36, 368078},
  {0x13B37, 368104},
  {0x13B38, 368130},
  {0x13B39, 368156},
  {0x13B3A, 368182},
  {0x13B3B, 368208},
  {0x13B3C, 368234},
  {0x13B3D, 368260},
  {0x13B3E, 368286},
  {0x13B3F, 368312},
  {0x13B40, 368338},
  {0x13B41, 368364},
  {0x13B42, 368390},
  {0x13B43, 368416},
  {0x13B44, 368442},
  {0x13B45, 368468},
  {0x13B46, 368494},
  {0x13B47, 368520},
  {0x13B48, 368546},
  {0x13B49, 368572},
  {0x13B4A, 368598},
  {0x13B4B, 368624},
  {0x13B4C, 368650},
  {0x13B4D, 368676},
  {0x13B4E, 368702},
  {0x13B4F, 368728},
  {0x13B50, 368754},
  {0x13B51, 368780},
  {0x13B52, 368806},
  {0x13B53, 368832},
  {0x13B54, 368858},
  {0x13B55, 368884},
  {0x13B56, 368910},
  {0x13B57, 368936},
  {0x13B58, 368962},
  {0x13B59, 368988},
  {0x13B5A, 369014},
  {0x13B5B, 369040},
  {0x13B5C, 369066},
  {0x13B5D, 369092},
  {0x13B5E, 369118},
  {0x13B5F, 369144},
  {0x13B60, 369170},
  {0x13B61, 369196},
  {0x13B62, 369222},
  {0x13B63, 369248},
  {0x13B64, 369274},
  {0x13B65, 369300},
  {0x13B66, 369326},
  {0x13B67, 369352},
  {0x13B68, 369378},
  {0x13B69, 369404},
  {0x13B6A, 369430},
  {0x13B6B, 369456},
  {0x13B6C, 369482},
  {0x13B6D, 369508},
  {0x13B6E, 369534},
  {0x13B6F, 369560},
  {0x13B70, 369586},
  {0x13B71, 369612},
  {0x13B72, 369638},
  {0x13B73, 369664},
  {0x13B74, 369690},
  {0x13B75, 369716},
  {0x13B76, 369742},
  {0x13B77, 369768},
  {0x13B78, 369794},
  {0x13B79, 369820},
  {0x13B7A, 369846},
  {0x13B7B, 369872},
  {0x13B7C, 369898},
  {0x13B7D, 369924},
  {0x13B7E, 369950},
  {0x13B7F, 369976},
  {0x13B80, 370002},
  {0x13B81, 370028},
  {0x13B82, 370054},
  {0x13B83, 370080},
  {0x13B84, 370106},
  {0x13B85, 370132},
  {0x13B86, 370158},
  {0x13B87, 370184},
  {0x13B88, 370210},
  {0x13B89, 370236},
  {0x13B8A, 370262},
  {0x13B8B, 370288},
  {0x13B8C, 370314},
  {0x13B8D, 370340},
  {0x13B8E, 370366},
  {0x13B8F, 370392},
  {0x13B90, 370418},
  {0x13B91, 370444},
  {0x13B92, 370470},
  {0x13B93, 370496},
  {0x13B94, 370522},
  {0x13B95, 370548},
  {0x13B96, 370574},
  {0x13B97, 370600},
  {0x13B98, 370626},
  {0x13B99, 370652},
  {0x13B9A, 370678},
  {0x13B9B, 370704},
  {0x13B9C, 370730},
  {0x13B9D, 370756},
  {0x13B9E, 370782},
  {0x13B9F, 370808},
  {0x13BA0, 370834},
  {0x13BA1, 370860},
  {0x13BA2, 370886},
  {0x13BA3, 370912},
  {0x13BA4, 370938},
  {0x13BA5, 370964},
  {0x13BA6, 370990},
  {0x13BA7, 371016},
  {0x13BA8, 371042},
  {0x13BA9, 371068},
  {0x13BAA, 371094},
  {0x13BAB, 371120},
  {0x13BAC, 371146},
  {0x13BAD, 371172},
  {0x13BAE, 371198},
  {0x13BAF, 371224},
  {0x13BB0, 371250},
  {0x13BB1, 371276},
  {0x13BB2, 371302},
  {0x13BB3, 371328},
  {0x13BB4, 371354},
  {0x13BB5, 371380},
  {0x13BB6, 371406},
  {0x13BB7, 371432},
  {0x13BB8, 371458},
  {0x13BB9, 371484},
  {0x13BBA, 371510},
  {0x13BBB, 371536},
  {0x13BBC, 371562},
  {0x13BBD, 371588},
  {0x13BBE, 371614},
  {0x13BBF, 371640},
  {0x13BC0, 371666},
  {0x13BC1, 371692},
  {0x13BC2, 371718},
  {0x13BC3, 371744},
  {0x13BC4, 371770},
  {0x13BC5, 371796},
  {0x13BC6, 371822},
  {0x13BC7, 371848},
  {0x13BC8, 371874},
  {0x13BC9, 371900},
  {0x13BCA, 371926},
  {0x13BCB, 371952},
  {0x13BCC, 371978},
  {0x13BCD, 372004},
  {0x13BCE, 372030},
  {0x13BCF, 372056},
  {0x13BD0, 372082},
  {0x13BD1, 372108},
  {0x13BD2, 372134},
  {0x13BD3, 372160},
  {0x13BD4, 372186},
  {0x13BD5, 372212},
  {0x13BD6, 372238},
  {0x13BD7, 372264},
  {0x13BD8, 372290},
  {0x13BD9, 372316},
  {0x13BDA, 372342},
  {0x13BDB, 372368},
  {0x13BDC, 372394},
  {0x13BDD, 372420},
  {0x13BDE, 372446},
  {0x13BDF, 372472},
  {0x13BE0, 372498},
  {0x13BE1, 372524},
  {0x13BE2, 372550},
  {0x13BE3, 372576},
  {0x13BE4, 372602},
  {0x13BE5, 372628},
  {0x13BE6, 372654},
  {0x13BE7, 372680},
  {0x13BE8, 372706},
  {0x13BE9, 372732},
  {0x13BEA, 372758},
  {0x13BEB, 372784},
  {0x13BEC, 372810},
  {0x13BED, 372836},
  {0x13BEE, 372862},
  {0x13BEF, 372888},
  {0x13BF0, 372914},
  {0x13BF1, 372940},
  {0x13BF2, 372966},
  {0x13BF3, 372992},
  {0x13BF4, 373018},
  {0x13BF5, 373044},
  {0x13BF6, 373070},
  {0x13BF7, 373096},
  {0x13BF8, 373122},
  {0x13BF9, 373148},
  {0x13BFA, 373174},
  {0x13BFB, 373200},
  {0x13BFC, 373226},
  {0x13BFD, 373252},
  {0x13BFE, 373278},
  {0x13BFF, 373304},
  {0x13C00, 373330},
  {0x13C01, 373356},
  {0x13C02, 373382},
  {0x13C03, 373408},
  {0x13C04, 373434},
  {0x13C05, 373460},
  {0x13C06, 373486},
  {0x13C07, 373512},
  {0x13C08, 373538},
  {0x13C09, 373564},
  {0x13C0A, 373590},
  {0x13C0B, 373616},
  {0x13C0C, 373642},
  {0x13C0D, 373668},
  {0x13C0E, 373694},
  {0x13C0F, 373720},
  {0x13C10, 373746},
  {0x13C11, 373772},
  {0x13C12, 373798},
  {0x13C13, 373824},
  {0x13C14, 373850},
  {0x13C15, 373876},
  {0x13C16, 373902},
  {0x13C17, 373928},
  {0x13C18, 373954},
  {0x13C19, 373980},
  {0x13C1A, 374006},
  {0x13C1B, 374032},
  {0x13C1C, 374058},
  {0x13C1D, 374084},
  {0x13C1E, 374110},
  {0x13C1F, 374136},
  {0x13C20, 374162},
  {0x13C21, 374188},
  {0x13C22, 374214},
  {0x13C23, 374240},
  {0x13C24, 374266},
  {0x13C25, 374292},
  {0x13C26, 374318},
  {0x13C27, 374344},
  {0x13C28, 374370},
  {0x13C29, 374396},
  {0x13C2A, 374422},
  {0x13C2B, 374448},
  {0x13C2C, 374474},
  {0x13C2D, 374500},
  {0x13C2E, 374526},
  {0x13C2F, 374552},
  {0x13C30, 374578},
  {0x13C31, 374604},
  {0x13C32, 374630},
  {0x13C33, 374656},
  {0x13C34, 374682},
  {0x13C35, 374708},
  {0x13C36, 374734},
  {0x13C37, 374760},
  {0x13C38, 374786},
  {0x13C39, 374812},
  {0x13C3A, 374838},
  {0x13C3B, 374864},
  {0x13C3C, 374890},
  {0x13C3D, 374916},
  {0x13C3E, 374942},
  {0x13C3F, 374968},
  {0x13C40, 374994},
  {0x13C41, 375020},
  {0x13C42, 375046},
  {0x13C43, 375072},
  {0x13C44, 375098},
  {0x13C45, 375124},
  {0x13C46, 375150},
  {0x13C47, 375176},
  {0x13C48, 375202},
  {0x13C49, 375228},
  {0x13C4A, 375254},
  {0x13C4B, 375280},
  {0x13C4C, 375306},
  {0x13C4D, 375332},
  {0x13C4E, 375358},
  {0x13C4F, 375384},
  {0x13C50, 375410},
  {0x13C51, 375436},
  {0x13C52, 375462},
  {0x13C53, 375488},
  {0x13C54, 375514},
  {0x13C55, 375540},
  {0x13C56, 375566},
  {0x13C57, 375592},
  {0x13C58, 375618},
  {0x13C59, 375644},
  {0x13C5A, 375670},
  {0x13C5B, 375696},
  {0x13C5C, 375722},
  {0x13C5D, 375748},
  {0x13C5E, 375774},
  {0x13C5F, 375800},
  {0x13C60, 375826},
  {0x13C61, 375852},
  {0x13C62, 375878},
  {0x13C63, 375904},
  {0x13C64, 375930},
  {0x13C65, 375956},
  {0x13C66, 375982},
  {0x13C67, 376008},
  {0x13C68, 376034},
  {0x13C69, 376060},
  {0x13C6A, 376086},
  {0x13C6B, 376112},
  {0x13C6C, 376138},
  {0x13C6D, 376164},
  {0x13C6E, 376190},
  {0x13C6F, 376216},
  {0x13C70, 376242},
  {0x13C71, 376268},
  {0x13C72, 376294},
  {0x13C73, 376320},
  {0x13C74, 376346},
  {0x13C75, 376372},
  {0x13C76, 376398},
  {0x13C77, 376424},
  {0x13C78, 376450},
  {0x13C79, 376476},
  {0x13C7A, 376502},
  {0x13C7B, 376528},
  {0x13C7C, 376554},
  {0x13C7D, 376580},
  {0x13C7E, 376606},
  {0x13C7F, 376632},
  {0x13C80, 376658},
  {0x13C81, 376684},
  {0x13C82, 376710},
  {0x13C83, 376736},
  {0x13C84, 376762},
  {0x13C85, 376788},
  {0x13C86, 376814},
  {0x13C87, 376840},
  {0x13C88, 376866},
  {0x13C89, 376892},
  {0x13C8A, 376918},
  {0x13C8B, 376944},
  {0x13C8C, 376970},
  {0x13C8D, 376996},
  {0x13C8E, 377022},
  {0x13C8F, 377048},
  {0x13C90, 377074},
  {0x13C91, 377100},
  {0x13C92, 377126},
  {0x13C93, 377152},
  {0x13C94, 377178},
  {0x13C95, 377204},
  {0x13C96, 377230},
  {0x13C97, 377256},
  {0x13C98, 377282},
  {0x13C99, 377308},
  {0x13C9A, 377334},
  {0x13C9B, 377360},
  {0x13C9C, 377386},
  {0x13C9D, 377412},
  {0x13C9E, 377438},
  {0x13C9F, 377464},
  {0x13CA0, 377490},
  {0x13CA1, 377516},
  {0x13CA2, 377542},
  {0x13CA3, 377568},
  {0x13CA4, 377594},
  {0x13CA5, 377620},
  {0x13CA6, 377646},
  {0x13CA7, 377672},
  {0x13CA8, 377698},
  {0x13CA9, 377724},
  {0x13CAA, 377750},
  {0x13CAB, 377776},
  {0x13CAC, 377802},
  {0x13CAD, 377828},
  {0x13CAE, 377854},
  {0x13CAF, 377880},
  {0x13CB0, 377906},
  {0x13CB1, 377932},
  {0x13CB2, 377958},
  {0x13CB3, 377984},
  {0x13CB4, 378010},
  {0x13CB5, 378036},
  {0x13CB6, 378062},
  {0x13CB7, 378088},
  {0x13CB8, 378114},
  {0x13CB9, 378140},
  {0x13CBA, 378166},
  {0x13CBB, 378192},
  {0x13CBC, 378218},
  {0x13CBD, 378244},
  {0x13CBE, 378270},
  {0x13CBF, 378296},
  {0x13CC0, 378322},
  {0x13CC1, 378348},
  {0x13CC2, 378374},
  {0x13CC3, 378400},
  {0x13CC4, 378426},
  {0x13CC5, 378452},
  {0x13CC6, 378478},
  {0x13CC7, 378504},
  {0x13CC8, 378530},
  {0x13CC9, 378556},
  {0x13CCA, 378582},
  {0x13CCB, 378608},
  {0x13CCC, 378634},
  {0x13CCD, 378660},
  {0x13CCE, 378686},
  {0x13CCF, 378712},
  {0x13CD0, 378738},
  {0x13CD1, 378764},
  {0x13CD2, 378790},
  {0x13CD3, 378816},
  {0x13CD4, 378842},
  {0x13CD5, 378868},
  {0x13CD6, 378894},
  {0x13CD7, 378920},
  {0x13CD8, 378946},
  {0x13CD9, 378972},
  {0x13CDA, 378998},
  {0x13CDB, 379024},
  {0x13CDC, 379050},
  {0x13CDD, 379076},
  {0x13CDE, 379102},
  {0x13CDF, 379128},
  {0x13CE0, 379154},
  {0x13CE1, 379180},
  {0x13CE2, 379206},
  {0x13CE3, 379232},
  {0x13CE4, 379258},
  {0x13CE5, 379284},
  {0x13CE6, 379310},
  {0x13CE7, 379336},
  {0x13CE8, 379362},
  {0x13CE9, 379388},
  {0x13CEA, 379414},
  {0x13CEB, 379440},
  {0x13CEC, 379466},
  {0x13CED, 379492},
  {0x13CEE, 379518},
  {0x13CEF, 379544},
  {0x13CF0, 379570},
  {0x13CF1, 379596},
  {0x13CF2, 379622},
  {0x13CF3, 379648},
  {0x13CF4, 379674},
  {0x13CF5, 379700},
  {0x13CF6, 379726},
  {0x13CF7, 379752},
  {0x13CF8, 379778},
  {0x13CF9, 379804},
  {0x13CFA, 379830},
  {0x13CFB, 379856},
  {0x13CFC, 379882},
  {0x13CFD, 379908},
  {0x13CFE, 379934},
  {0x13CFF, 379960},
  {0x13D00, 379986},
  {0x13D01, 380012},
  {0x13D02, 380038},
  {0x13D03, 380064},
  {0x13D04, 380090},
  {0x13D05, 380116},
  {0x13D06, 380142},
  {0x13D07, 380168},
  {0x13D08, 380194},
  {0x13D09, 380220},
  {0x13D0A, 380246},
  {0x13D0B, 380272},
  {0x13D0C, 380298},
  {0x13D0D, 380324},
  {0x13D0E, 380350},
  {0x13D0F, 380376},
  {0x13D10, 380402},
  {0x13D11, 380428},
  {0x13D12, 380454},
  {0x13D13, 380480},
  {0x13D14, 380506},
  {0x13D15, 380532},
  {0x13D16, 380558},
  {0x13D17, 380584},
  {0x13D18, 380610},
  {0x13D19, 380636},
  {0x13D1A, 380662},
  {0x13D1B, 380688},
  {0x13D1C, 380714},
  {0x13D1D, 380740},
  {0x13D1E, 380766},
  {0x13D1F, 380792},
  {0x13D20, 380818},
  {0x13D21, 380844},
  {0x13D22, 380870},
  {0x13D23, 380896},
  {0x13D24, 380922},
  {0x13D25, 380948},
  {0x13D26, 380974},
  {0x13D27, 381000},
  {0x13D28, 381026},
  {0x13D29, 381052},
  {0x13D2A, 381078},
  {0x13D2B, 381104},
  {0x13D2C, 381130},
  {0x13D2D, 381156},
  {0x13D2E, 381182},
  {0x13D2F, 381208},
  {0x13D30, 381234},
  {0x13D31, 381260},
  {0x13D32, 381286},
  {0x13D33, 381312},
  {0x13D34, 381338},
  {0x13D35, 381364},
  {0x13D36, 381390},
  {0x13D37, 381416},
  {0x13D38, 381442},
  {0x13D39, 381468},
  {0x13D3A, 381494},
  {0x13D3B, 381520},
  {0x13D3C, 381546},
  {0x13D3D, 381572},
  {0x13D3E, 381598},
  {0x13D3F, 381624},
  {0x13D40, 381650},
  {0x13D41, 381676},
  {0x13D42, 381702},
  {0x13D43, 381728},
  {0x13D44, 381754},
  {0x13D45, 381780},
  {0x13D46, 381806},
  {0x13D47, 381832},
  {0x13D48, 381858},
  {0x13D49, 381884},
  {0x13D4A, 381910},
  {0x13D4B, 381936},
  {0x13D4C, 381962},
  {0x13D4D, 381988},
  {0x13D4E, 382014},
  {0x13D4F, 382040},
  {0x13D50, 382066},
  {0x13D51, 382092},
  {0x13D52, 382118},
  {0x13D53, 382144},
  {0x13D54, 382170},
  {0x13D55, 382196},
  {0x13D56, 382222},
  {0x13D57, 382248},
  {0x13D58, 382274},
  {0x13D59, 382300},
  {0x13D5A, 382326},
  {0x13D5B, 382352},
  {0x13D5C, 382378},
  {0x13D5D, 382404},
  {0x13D5E, 382430},
  {0x13D5F, 382456},
  {0x13D60, 382482},
  {0x13D61, 382508},
  {0x13D62, 382534},
  {0x13D63, 382560},
  {0x13D64, 382586},
  {0x13D65, 382612},
  {0x13D66, 382638},
  {0x13D67, 382664},
  {0x13D68, 382690},
  {0x13D69, 382716},
  {0x13D6A, 382742},
  {0x13D6B, 382768},
  {0x13D6C, 382794},
  {0x13D6D, 382820},
  {0x13D6E, 382846},
  {0x13D6F, 382872},
  {0x13D70, 382898},
  {0x13D71, 382924},
  {0x13D72, 382950},
  {0x13D73, 382976},
  {0x13D74, 383002},
  {0x13D75, 383028},
  {0x13D76, 383054},
  {0x13D77, 383080},
  {0x13D78, 383106},
  {0x13D79, 383132},
  {0x13D7A, 383158},
  {0x13D7B, 383184},
  {0x13D7C, 383210},
  {0x13D7D, 383236},
  {0x13D7E, 383262},
  {0x13D7F, 383288},
  {0x13D80, 383314},
  {0x13D81, 383340},
  {0x13D82, 383366},
  {0x13D83, 383392},
  {0x13D84, 383418},
  {0x13D85, 383444},
  {0x13D86, 383470},
  {0x13D87, 383496},
  {0x13D88, 383522},
  {0x13D89, 383548},
  {0x13D8A, 383574},
  {0x13D8B, 383600},
  {0x13D8C, 383626},
  {0x13D8D, 383652},
  {0x13D8E, 383678},
  {0x13D8F, 383704},
  {0x13D90, 383730},
  {0x13D91, 383756},
  {0x13D92, 383782},
  {0x13D93, 383808},
  {0x13D94, 383834},
  {0x13D95, 383860},
  {0x13D96, 383886},
  {0x13D97, 383912},
  {0x13D98, 383938},
  {0x13D99, 383964},
  {0x13D9A, 383990},
  {0x13D9B, 384016},
  {0x13D9C, 384042},
  {0x13D9D, 384068},
  {0x13D9E, 384094},
  {0x13D9F, 384120},
  {0x13DA0, 384146},
  {0x13DA1, 384172},
  {0x13DA2, 384198},
  {0x13DA3, 384224},
  {0x13DA4, 384250},
  {0x13DA5, 384276},
  {0x13DA6, 384302},
  {0x13DA7, 384328},
  {0x13DA8, 384354},
  {0x13DA9, 384380},
  {0x13DAA, 384406},
  {0x13DAB, 384432},
  {0x13DAC, 384458},
  {0x13DAD, 384484},
  {0x13DAE, 384510},
  {0x13DAF, 384536},
  {0x13DB0, 384562},
  {0x13DB1, 384588},
  {0x13DB2, 384614},
  {0x13DB3, 384640},
  {0x13DB4, 384666},
  {0x13DB5, 384692},
  {0x13DB6, 384718},
  {0x13DB7, 384744},
  {0x13DB8, 384770},
  {0x13DB9, 384796},
  {0x13DBA, 384822},
  {0x13DBB, 384848},
  {0x13DBC, 384874},
  {0x13DBD, 384900},
  {0x13DBE, 384926},
  {0x13DBF, 384952},
  {0x13DC0, 384978},
  {0x13DC1, 385004},
  {0x13DC2, 385030},
  {0x13DC3, 385056},
  {0x13DC4, 385082},
  {0x13DC5, 385108},
  {0x13DC6, 385134},
  {0x13DC7, 385160},
  {0x13DC8, 385186},
  {0x13DC9, 385212},
  {0x13DCA, 385238},
  {0x13DCB, 385264},
  {0x13DCC, 385290},
  {0x13DCD, 385316},
  {0x13DCE, 385342},
  {0x13DCF, 385368},
  {0x13DD0, 385394},
  {0x13DD1, 385420},
  {0x13DD2, 385446},
  {0x13DD3, 385472},
  {0x13DD4, 385498},
  {0x13DD5, 385524},
  {0x13DD6, 385550},
  {0x13DD7, 385576},
  {0x13DD8, 385602},
  {0x13DD9, 385628},
  {0x13DDA, 385654},
  {0x13DDB, 385680},
  {0x13DDC, 385706},
  {0x13DDD, 385732},
  {0x13DDE, 385758},
  {0x13DDF, 385784},
  {0x13DE0, 385810},
  {0x13DE1, 385836},
  {0x13DE2, 385862},
  {0x13DE3, 385888},
  {0x13DE4, 385914},
  {0x13DE5, 385940},
  {0x13DE6, 385966},
  {0x13DE7, 385992},
  {0x13DE8, 386018},
  {0x13DE9, 386044},
  {0x13DEA, 386070},
  {0x13DEB, 386096},
  {0x13DEC, 386122},
  {0x13DED, 386148},
  {0x13DEE, 386174},
  {0x13DEF, 386200},
  {0x13DF0, 386226},
  {0x13DF1, 386252},
  {0x13DF2, 386278},
  {0x13DF3, 386304},
  {0x13DF4, 386330},
  {0x13DF5, 386356},
  {0x13DF6, 386382},
  {0x13DF7, 386408},
  {0x13DF8, 386434},
  {0x13DF9, 386460},
  {0x13DFA, 386486},
  {0x13DFB, 386512},
  {0x13DFC, 386538},
  {0x13DFD, 386564},
  {0x13DFE, 386590},
  {0x13DFF, 386616},
  {0x13E00, 386642},
  {0x13E01, 386668},
  {0x13E02, 386694},
  {0x13E03, 386720},
  {0x13E04, 386746},
  {0x13E05, 386772},
  {0x13E06, 386798},
  {0x13E07, 386824},
  {0x13E08, 386850},
  {0x13E09, 386876},
  {0x13E0A, 386902},
  {0x13E0B, 386928},
  {0x13E0C, 386954},
  {0x13E0D, 386980},
  {0x13E0E, 387006},
  {0x13E0F, 387032},
  {0x13E10, 387058},
  {0x13E11, 387084},
  {0x13E12, 387110},
  {0x13E13, 387136},
  {0x13E14, 387162},
  {0x13E15, 387188},
  {0x13E16, 387214},
  {0x13E17, 387240},
  {0x13E18, 387266},
  {0x13E19, 387292},
  {0x13E1A, 387318},
  {0x13E1B, 387344},
  {0x13E1C, 387370},
  {0x13E1D, 387396},
  {0x13E1E, 387422},
  {0x13E1F, 387448},
  {0x13E20, 387474},
  {0x13E21, 387500},
  {0x13E22, 387526},
  {0x13E23, 387552},
  {0x13E24, 387578},
  {0x13E25, 387604},
  {0x13E26, 387630},
  {0x13E27, 387656},
  {0x13E28, 387682},
  {0x13E29, 387708},
  {0x13E2A, 387734},
  {0x13E2B, 387760},
  {0x13E2C, 387786},
  {0x13E2D, 387812},
  {0x13E2E, 387838},
  {0x13E2F, 387864},
  {0x13E30, 387890},
  {0x13E31, 387916},
  {0x13E32, 387942},
  {0x13E33, 387968},
  {0x13E34, 387994},
  {0x13E35, 388020},
  {0x13E36, 388046},
  {0x13E37, 388072},
  {0x13E38, 388098},
  {0x13E39, 388124},
  {0x13E3A, 388150},
  {0x13E3B, 388176},
  {0x13E3C, 388202},
  {0x13E3D, 388228},
  {0x13E3E, 388254},
  {0x13E3F, 388280},
  {0x13E40, 388306},
  {0x13E41, 388332},
  {0x13E42, 388358},
  {0x13E43, 388384},
  {0x13E44, 388410},
  {0x13E45, 388436},
  {0x13E46, 388462},
  {0x13E47, 388488},
  {0x13E48, 388514},
  {0x13E49, 388540},
  {0x13E4A, 388566},
  {0x13E4B, 388592},
  {0x13E4C, 388618},
  {0x13E4D, 388644},
  {0x13E4E, 388670},
  {0x13E4F, 388696},
  {0x13E50, 388722},
  {0x13E51, 388748},
  {0x13E52, 388774},
  {0x13E53, 388800},
  {0x13E54, 388826},
  {0x13E55, 388852},
  {0x13E56, 388878},
  {0x13E57, 388904},
  {0x13E58, 388930},
  {0x13E59, 388956},
  {0x13E5A, 388982},
  {0x13E5B, 389008},
  {0x13E5C, 389034},
  {0x13E5D, 389060},
  {0x13E5E, 389086},
  {0x13E5F, 389112},
  {0x13E60, 389138},
  {0x13E61, 389164},
  {0x13E62, 389190},
  {0x13E63, 389216},
  {0x13E64, 389242},
  {0x13E65, 389268},
  {0x13E66, 389294},
  {0x13E67, 389320},
  {0x13E68, 389346},
  {0x13E69, 389372},
  {0x13E6A, 389398},
  {0x13E6B, 389424},
  {0x13E6C, 389450},
  {0x13E6D, 389476},
  {0x13E6E, 389502},
  {0x13E6F, 389528},
  {0x13E70, 389554},
  {0x13E71, 389580},
  {0x13E72, 389606},
  {0x13E73, 389632},
  {0x13E74, 389658},
  {0x13E75, 389684},
  {0x13E76, 389710},
  {0x13E77, 389736},
  {0x13E78, 389762},
  {0x13E79, 389788},
  {0x13E7A, 389814},
  {0x13E7B, 389840},
  {0x13E7C, 389866},
  {0x13E7D, 389892},
  {0x13E7E, 389918},
  {0x13E7F, 389944},
  {0x13E80, 389970},
  {0x13E81, 389996},
  {0x13E82, 390022},
  {0x13E83, 390048},
  {0x13E84, 390074},
  {0x13E85, 390100},
  {0x13E86, 390126},
  {0x13E87, 390152},
  {0x13E88, 390178},
  {0x13E89, 390204},
  {0x13E8A, 390230},
  {0x13E8B, 390256},
  {0x13E8C, 390282},
  {0x13E8D, 390308},
  {0x13E8E, 390334},
  {0x13E8F, 390360},
  {0x13E90, 390386},
  {0x13E91, 390412},
  {0x13E92, 390438},
  {0x13E93, 390464},
  {0x13E94, 390490},
  {0x13E95, 390516},
  {0x13E96, 390542},
  {0x13E97, 390568},
  {0x13E98, 390594},
  {0x13E99, 390620},
  {0x13E9A, 390646},
  {0x13E9B, 390672},
  {0x13E9C, 390698},
  {0x13E9D, 390724},
  {0x13E9E, 390750},
  {0x13E9F, 390776},
  {0x13EA0, 390802},
  {0x13EA1, 390828},
  {0x13EA2, 390854},
  {0x13EA3, 390880},
  {0x13EA4, 390906},
  {0x13EA5, 390932},
  {0x13EA6, 390958},
  {0x13EA7, 390984},
  {0x13EA8, 391010},
  {0x13EA9, 391036},
  {0x13EAA, 391062},
  {0x13EAB, 391088},
  {0x13EAC, 391114},
  {0x13EAD, 391140},
  {0x13EAE, 391166},
  {0x13EAF, 391192},
  {0x13EB0, 391218},
  {0x13EB1, 391244},
  {0x13EB2, 391270},
  {0x13EB3, 391296},
  {0x13EB4, 391322},
  {0x13EB5, 391348},
  {0x13EB6, 391374},
  {0x13EB7, 391400},
  {0x13EB8, 391426},
  {0x13EB9, 391452},
  {0x13EBA, 391478},
  {0x13EBB, 391504},
  {0x13EBC, 391530},
  {0x13EBD, 391556},
  {0x13EBE, 391582},
  {0x13EBF, 391608},
  {0x13EC0, 391634},
  {0x13EC1, 391660},
  {0x13EC2, 391686},
  {0x13EC3, 391712},
  {0x13EC4, 391738},
  {0x13EC5, 391764},
  {0x13EC6, 391790},
  {0x13EC7, 391816},
  {0x13EC8, 391842},
  {0x13EC9, 391868},
  {0x13ECA, 391894},
  {0x13ECB, 391920},
  {0x13ECC, 391946},
  {0x13ECD, 391972},
  {0x13ECE, 391998},
  {0x13ECF, 392024},
  {0x13ED0, 392050},
  {0x13ED1, 392076},
  {0x13ED2, 392102},
  {0x13ED3, 392128},
  {0x13ED4, 392154},
  {0x13ED5, 392180},
  {0x13ED6, 392206},
  {0x13ED7, 392232},
  {0x13ED8, 392258},
  {0x13ED9, 392284},
  {0x13EDA, 392310},
  {0x13EDB, 392336},
  {0x13EDC, 392362},
  {0x13EDD, 392388},
  {0x13EDE, 392414},
  {0x13EDF, 392440},
  {0x13EE0, 392466},
  {0x13EE1, 392492},
  {0x13EE2, 392518},
  {0x13EE3, 392544},
  {0x13EE4, 392570},
  {0x13EE5, 392596},
  {0x13EE6, 392622},
  {0x13EE7, 392648},
  {0x13EE8, 392674},
  {0x13EE9, 392700},
  {0x13EEA, 392726},
  {0x13EEB, 392752},
  {0x13EEC, 392778},
  {0x13EED, 392804},
  {0x13EEE, 392830},
  {0x13EEF, 392856},
  {0x13EF0, 392882},
  {0x13EF1, 392908},
  {0x13EF2, 392934},
  {0x13EF3, 392960},
  {0x13EF4, 392986},
  {0x13EF5, 393012},
  {0x13EF6, 393038},
  {0x13EF7, 393064},
  {0x13EF8, 393090},
  {0x13EF9, 393116},
  {0x13EFA, 393142},
  {0x13EFB, 393168},
  {0x13EFC, 393194},
  {0x13EFD, 393220},
  {0x13EFE, 393246},
  {0x13EFF, 393272},
  {0x13F00, 393298},
  {0x13F01, 393324},
  {0x13F02, 393350},
  {0x13F03, 393376},
  {0x13F04, 393402},
  {0x13F05, 393428},
  {0x13F06, 393454},
  {0x13F07, 393480},
  {0x13F08, 393506},
  {0x13F09, 393532},
  {0x13F0A, 393558},
  {0x13F0B, 393584},
  {0x13F0C, 393610},
  {0x13F0D, 393636},
  {0x13F0E, 393662},
  {0x13F0F, 393688},
  {0x13F10, 393714},
  {0x13F11, 393740},
  {0x13F12, 393766},
  {0x13F13, 393792},
  {0x13F14, 393818},
  {0x13F15, 393844},
  {0x13F16, 393870},
  {0x13F17, 393896},
  {0x13F18, 393922},
  {0x13F19, 393948},
  {0x13F1A, 393974},
  {0x13F1B, 394000},
  {0x13F1C, 394026},
  {0x13F1D, 394052},
  {0x13F1E, 394078},
  {0x13F1F, 394104},
  {0x13F20, 394130},
  {0x13F21, 394156},
  {0x13F22, 394182},
  {0x13F23, 394208},
  {0x13F24, 394234},
  {0x13F25, 394260},
  {0x13F26, 394286},
  {0x13F27, 394312},
  {0x13F28, 394338},
  {0x13F29, 394364},
  {0x13F2A, 394390},
  {0x13F2B, 394416},
  {0x13F2C, 394442},
  {0x13F2D, 394468},
  {0x13F2E, 394494},
  {0x13F2F, 394520},
  {0x13F30, 394546},
  {0x13F31, 394572},
  {0x13F32, 394598},
  {0x13F33, 394624},
  {0x13F34, 394650},
  {0x13F35, 394676},
  {0x13F36, 394702},
  {0x13F37, 394728},
  {0x13F38, 394754},
  {0x13F39, 394780},
  {0x13F3A, 394806},
  {0x13F3B, 394832},
  {0x13F3C, 394858},
  {0x13F3D, 394884},
  {0x13F3E, 394910},
  {0x13F3F, 394936},
  {0x13F40, 394962},
  {0x13F41, 394988},
  {0x13F42, 395014},
  {0x13F43, 395040},
  {0x13F44, 395066},
  {0x13F45, 395092},
  {0x13F46, 395118},
  {0x13F47, 395144},
  {0x13F48, 395170},
  {0x13F49, 395196},
  {0x13F4A, 395222},
  {0x13F4B, 395248},
  {0x13F4C, 395274},
  {0x13F4D, 395300},
  {0x13F4E, 395326},
  {0x13F4F, 395352},
  {0x13F50, 395378},
  {0x13F51, 395404},
  {0x13F52, 395430},
  {0x13F53, 395456},
  {0x13F54, 395482},
  {0x13F55, 395508},
  {0x13F56, 395534},
  {0x13F57, 395560},
  {0x13F58, 395586},
  {0x13F59, 395612},
  {0x13F5A, 395638},
  {0x13F5B, 395664},
  {0x13F5C, 395690},
  {0x13F5D, 395716},
  {0x13F5E, 395742},
  {0x13F5F, 395768},
  {0x13F60, 395794},
  {0x13F61, 395820},
  {0x13F62, 395846},
  {0x13F63, 395872},
  {0x13F64, 395898},
  {0x13F65, 395924},
  {0x13F66, 395950},
  {0x13F67, 395976},
  {0x13F68, 396002},
  {0x13F69, 396028},
  {0x13F6A, 396054},
  {0x13F6B, 396080},
  {0x13F6C, 396106},
  {0x13F6D, 396132},
  {0x13F6E, 396158},
  {0x13F6F, 396184},
  {0x13F70, 396210},
  {0x13F71, 396236},
  {0x13F72, 396262},
  {0x13F73, 396288},
  {0x13F74, 396314},
  {0x13F75, 396340},
  {0x13F76, 396366},
  {0x13F77, 396392},
  {0x13F78, 396418},
  {0x13F79, 396444},
  {0x13F7A, 396470},
  {0x13F7B, 396496},
  {0x13F7C, 396522},
  {0x13F7D, 396548},
  {0x13F7E, 396574},
  {0x13F7F, 396600},
  {0x13F80, 396626},
  {0x13F81, 396652},
  {0x13F82, 396678},
  {0x13F83, 396704},
  {0x13F84, 396730},
  {0x13F85, 396756},
  {0x13F86, 396782},
  {0x13F87, 396808},
  {0x13F88, 396834},
  {0x13F89, 396860},
  {0x13F8A, 396886},
  {0x13F8B, 396912},
  {0x13F8C, 396938},
  {0x13F8D, 396964},
  {0x13F8E, 396990},
  {0x13F8F, 397016},
  {0x13F90, 397042},
  {0x13F91, 397068},
  {0x13F92, 397094},
  {0x13F93, 397120},
  {0x13F94, 397146},
  {0x13F95, 397172},
  {0x13F96, 397198},
  {0x13F97, 397224},
  {0x13F98, 397250},
  {0x13F99, 397276},
  {0x13F9A, 397302},
  {0x13F9B, 397328},
  {0x13F9C, 397354},
  {0x13F9D, 397380},
  {0x13F9E, 397406},
  {0x13F9F, 397432},
  {0x13FA0, 397458},
  {0x13FA1, 397484},
  {0x13FA2, 397510},
  {0x13FA3, 397536},
  {0x13FA4, 397562},
  {0x13FA5, 397588},
  {0x13FA6, 397614},
  {0x13FA7, 397640},
  {0x13FA8, 397666},
  {0x13FA9, 397692},
  {0x13FAA, 397718},
  {0x13FAB, 397744},
  {0x13FAC, 397770},
  {0x13FAD, 397796},
  {0x13FAE, 397822},
  {0x13FAF, 397848},
  {0x13FB0, 397874},
  {0x13FB1, 397900},
  {0x13FB2, 397926},
  {0x13FB3, 397952},
  {0x13FB4, 397978},
  {0x13FB5, 398004},
  {0x13FB6, 398030},
  {0x13FB7, 398056},
  {0x13FB8, 398082},
  {0x13FB9, 398108},
  {0x13FBA, 398134},
  {0x13FBB, 398160},
  {0x13FBC, 398186},
  {0x13FBD, 398212},
  {0x13FBE, 398238},
  {0x13FBF, 398264},
  {0x13FC0, 398290},
  {0x13FC1, 398316},
  {0x13FC2, 398342},
  {0x13FC3, 398368},
  {0x13FC4, 398394},
  {0x13FC5, 398420},
  {0x13FC6, 398446},
  {0x13FC7, 398472},
  {0x13FC8, 398498},
  {0x13FC9, 398524},
  {0x13FCA, 398550},
  {0x13FCB, 398576},
  {0x13FCC, 398602},
  {0x13FCD, 398628},
  {0x13FCE, 398654},
  {0x13FCF, 398680},
  {0x13FD0, 398706},
  {0x13FD1, 398732},
  {0x13FD2, 398758},
  {0x13FD3, 398784},
  {0x13FD4, 398810},
  {0x13FD5, 398836},
  {0x13FD6, 398862},
  {0x13FD7, 398888},
  {0x13FD8, 398914},
  {0x13FD9, 398940},
  {0x13FDA, 398966},
  {0x13FDB, 398992},
  {0x13FDC, 399018},
  {0x13FDD, 399044},
  {0x13FDE, 399070},
  {0x13FDF, 399096},
  {0x13FE0, 399122},
  {0x13FE1, 399148},
  {0x13FE2, 399174},
  {0x13FE3, 399200},
  {0x13FE4, 399226},
  {0x13FE5, 399252},
  {0x13FE6, 399278},
  {0x13FE7, 399304},
  {0x13FE8, 399330},
  {0x13FE9, 399356},
  {0x13FEA, 399382},
  {0x13FEB, 399408},
  {0x13FEC, 399434},
  {0x13FED, 399460},
  {0x13FEE, 399486},
  {0x13FEF, 399512},
  {0x13FF0, 399538},
  {0x13FF1, 399564},
  {0x13FF2, 399590},
  {0x13FF3, 399616},
  {0x13FF4, 399642},
  {0x13FF5, 399668},
  {0x13FF6, 399694},
  {0x13FF7, 399720},
  {0x13FF8, 399746},
  {0x13FF9, 399772},
  {0x13FFA, 399798},
  {0x13FFB, 399824},
  {0x13FFC, 399850},
  {0x13FFD, 399876},
  {0x13FFE, 399902},
  {0x13FFF, 399928},
  {0x14000, 399954},
  {0x14001, 399980},
  {0x14002, 400006},
  {0x14003, 400032},
  {0x14004, 400058},
  {0x14005, 400084},
  {0x14006, 400110},
  {0x14007, 400136},
  {0x14008, 400162},
  {0x14009, 400188},
  {0x1400A, 400214},
  {0x1400B, 400240},
  {0x1400C, 400266},
  {0x1400D, 400292},
  {0x1400E, 400318},
  {0x1400F, 400344},
  {0x14010, 400370},
  {0x14011, 400396},
  {0x14012, 400422},
  {0x14013, 400448},
  {0x14014, 400474},
  {0x14015, 400500},
  {0x14016, 400526},
  {0x14017, 400552},
  {0x14018, 400578},
  {0x14019, 400604},
  {0x1401A, 400630},
  {0x1401B, 400656},
  {0x1401C, 400682},
  {0x1401D, 400708},
  {0x1401E, 400734},
  {0x1401F, 400760},
  {0x14020, 400786},
  {0x14021, 400812},
  {0x14022, 400838},
  {0x14023, 400864},
  {0x14024, 400890},
  {0x14025, 400916},
  {0x14026, 400942},
  {0x14027, 400968},
  {0x14028, 400994},
  {0x14029, 401020},
  {0x1402A, 401046},
  {0x1402B, 401072},
  {0x1402C, 401098},
  {0x1402D, 401124},
  {0x1402E, 401150},
  {0x1402F, 401176},
  {0x14030, 401202},
  {0x14031, 401228},
  {0x14032, 401254},
  {0x14033, 401280},
  {0x14034, 401306},
  {0x14035, 401332},
  {0x14036, 401358},
  {0x14037, 401384},
  {0x14038, 401410},
  {0x14039, 401436},
  {0x1403A, 401462},
  {0x1403B, 401488},
  {0x1403C, 401514},
  {0x1403D, 401540},
  {0x1403E, 401566},
  {0x1403F, 401592},
  {0x14040, 401618},
  {0x14041, 401644},
  {0x14042, 401670},
  {0x14043, 401696},
  {0x14044, 401722},
  {0x14045, 401748},
  {0x14046, 401774},
  {0x14047, 401800},
  {0x14048, 401826},
  {0x14049, 401852},
  {0x1404A, 401878},
  {0x1404B, 401904},
  {0x1404C, 401930},
  {0x1404D, 401956},
  {0x1404E, 401982},
  {0x1404F, 402008},
  {0x14050, 402034},
  {0x14051, 402060},
  {0x14052, 402086},
  {0x14053, 402112},
  {0x14054, 402138},
  {0x14055, 402164},
  {0x14056, 402190},
  {0x14057, 402216},
  {0x14058, 402242},
  {0x14059, 402268},
  {0x1405A, 402294},
  {0x1405B, 402320},
  {0x1405C, 402346},
  {0x1405D, 402372},
  {0x1405E, 402398},
  {0x1405F, 402424},
  {0x14060, 402450},
  {0x14061, 402476},
  {0x14062, 402502},
  {0x14063, 402528},
  {0x14064, 402554},
  {0x14065, 402580},
  {0x14066, 402606},
  {0x14067, 402632},
  {0x14068, 402658},
  {0x14069, 402684},
  {0x1406A, 402710},
  {0x1406B, 402736},
  {0x1406C, 402762},
  {0x1406D, 402788},
  {0x1406E, 402814},
  {0x1406F, 402840},
  {0x14070, 402866},
  {0x14071, 402892},
  {0x14072, 402918},
  {0x14073, 402944},
  {0x14074, 402970},
  {0x14075, 402996},
  {0x14076, 403022},
  {0x14077, 403048},
  {0x14078, 403074},
  {0x14079, 403100},
  {0x1407A, 403126},
  {0x1407B, 403152},
  {0x1407C, 403178},
  {0x1407D, 403204},
  {0x1407E, 403230},
  {0x1407F, 403256},
  {0x14080, 403282},
  {0x14081, 403308},
  {0x14082, 403334},
  {0x14083, 403360},
  {0x14084, 403386},
  {0x14085, 403412},
  {0x14086, 403438},
  {0x14087, 403464},
  {0x14088, 403490},
  {0x14089, 403516},
  {0x1408A, 403542},
  {0x1408B, 403568},
  {0x1408C, 403594},
  {0x1408D, 403620},
  {0x1408E, 403646},
  {0x1408F, 403672},
  {0x14090, 403698},
  {0x14091, 403724},
  {0x14092, 403750},
  {0x14093, 403776},
  {0x14094, 403802},
  {0x14095, 403828},
  {0x14096, 403854},
  {0x14097, 403880},
  {0x14098, 403906},
  {0x14099, 403932},
  {0x1409A, 403958},
  {0x1409B, 403984},
  {0x1409C, 404010},
  {0x1409D, 404036},
  {0x1409E, 404062},
  {0x1409F, 404088},
  {0x140A0, 404114},
  {0x140A1, 404140},
  {0x140A2, 404166},
  {0x140A3, 404192},
  {0x140A4, 404218},
  {0x140A5, 404244},
  {0x140A6, 404270},
  {0x140A7, 404296},
  {0x140A8, 404322},
  {0x140A9, 404348},
  {0x140AA, 404374},
  {0x140AB, 404400},
  {0x140AC, 404426},
  {0x140AD, 404452},
  {0x140AE, 404478},
  {0x140AF, 404504},
  {0x140B0, 404530},
  {0x140B1, 404556},
  {0x140B2, 404582},
  {0x140B3, 404608},
  {0x140B4, 404634},
  {0x140B5, 404660},
  {0x140B6, 404686},
  {0x140B7, 404712},
  {0x140B8, 404738},
  {0x140B9, 404764},
  {0x140BA, 404790},
  {0x140BB, 404816},
  {0x140BC, 404842},
  {0x140BD, 404868},
  {0x140BE, 404894},
  {0x140BF, 404920},
  {0x140C0, 404946},
  {0x140C1, 404972},
  {0x140C2, 404998},
  {0x140C3, 405024},
  {0x140C4, 405050},
  {0x140C5, 405076},
  {0x140C6, 405102},
  {0x140C7, 405128},
  {0x140C8, 405154},
  {0x140C9, 405180},
  {0x140CA, 405206},
  {0x140CB, 405232},
  {0x140CC, 405258},
  {0x140CD, 405284},
  {0x140CE, 405310},
  {0x140CF, 405336},
  {0x140D0, 405362},
  {0x140D1, 405388},
  {0x140D2, 405414},
  {0x140D3, 405440},
  {0x140D4, 405466},
  {0x140D5, 405492},
  {0x140D6, 405518},
  {0x140D7, 405544},
  {0x140D8, 405570},
  {0x140D9, 405596},
  {0x140DA, 405622},
  {0x140DB, 405648},
  {0x140DC, 405674},
  {0x140DD, 405700},
  {0x140DE, 405726},
  {0x140DF, 405752},
  {0x140E0, 405778},
  {0x140E1, 405804},
  {0x140E2, 405830},
  {0x140E3, 405856},
  {0x140E4, 405882},
  {0x140E5, 405908},
  {0x140E6, 405934},
  {0x140E7, 405960},
  {0x140E8, 405986},
  {0x140E9, 406012},
  {0x140EA, 406038},
  {0x140EB, 406064},
  {0x140EC, 406090},
  {0x140ED, 406116},
  {0x140EE, 406142},
  {0x140EF, 406168},
  {0x140F0, 406194},
  {0x140F1, 406220},
  {0x140F2, 406246},
  {0x140F3, 406272},
  {0x140F4, 406298},
  {0x140F5, 406324},
  {0x140F6, 406350},
  {0x140F7, 406376},
  {0x140F8, 406402},
  {0x140F9, 406428},
  {0x140FA, 406454},
  {0x140FB, 406480},
  {0x140FC, 406506},
  {0x140FD, 406532},
  {0x140FE, 406558},
  {0x140FF, 406584},
  {0x14100, 406610},
  {0x14101, 406636},
  {0x14102, 406662},
  {0x14103, 406688},
  {0x14104, 406714},
  {0x14105, 406740},
  {0x14106, 406766},
  {0x14107, 406792},
  {0x14108, 406818},
  {0x14109, 406844},
  {0x1410A, 406870},
  {0x1410B, 406896},
  {0x1410C, 406922},
  {0x1410D, 406948},
  {0x1410E, 406974},
  {0x1410F, 407000},
  {0x14110, 407026},
  {0x14111, 407052},
  {0x14112, 407078},
  {0x14113, 407104},
  {0x14114, 407130},
  {0x14115, 407156},
  {0x14116, 407182},
  {0x14117, 407208},
  {0x14118, 407234},
  {0x14119, 407260},
  {0x1411A, 407286},
  {0x1411B, 407312},
  {0x1411C, 407338},
  {0x1411D, 407364},
  {0x1411E, 407390},
  {0x1411F, 407416},
  {0x14120, 407442},
  {0x14121, 407468},
  {0x14122, 407494},
  {0x14123, 407520},
  {0x14124, 407546},
  {0x14125, 407572},
  {0x14126, 407598},
  {0x14127, 407624},
  {0x14128, 407650},
  {0x14129, 407676},
  {0x1412A, 407702},
  {0x1412B, 407728},
  {0x1412C, 407754},
  {0x1412D, 407780},
  {0x1412E, 407806},
  {0x1412F, 407832},
  {0x14130, 407858},
  {0x14131, 407884},
  {0x14132, 407910},
  {0x14133, 407936},
  {0x14134, 407962},
  {0x14135, 407988},
  {0x14136, 408014},
  {0x14137, 408040},
  {0x14138, 408066},
  {0x14139, 408092},
  {0x1413A, 408118},
  {0x1413B, 408144},
  {0x1413C, 408170},
  {0x1413D, 408196},
  {0x1413E, 408222},
  {0x1413F, 408248},
  {0x14140, 408274},
  {0x14141, 408300},
  {0x14142, 408326},
  {0x14143, 408352},
  {0x14144, 408378},
  {0x14145, 408404},
  {0x14146, 408430},
  {0x14147, 408456},
  {0x14148, 408482},
  {0x14149, 408508},
  {0x1414A, 408534},
  {0x1414B, 408560},
  {0x1414C, 408586},
  {0x1414D, 408612},
  {0x1414E, 408638},
  {0x1414F, 408664},
  {0x14150, 408690},
  {0x14151, 408716},
  {0x14152, 408742},
  {0x14153, 408768},
  {0x14154, 408794},
  {0x14155, 408820},
  {0x14156, 408846},
  {0x14157, 408872},
  {0x14158, 408898},
  {0x14159, 408924},
  {0x1415A, 408950},
  {0x1415B, 408976},
  {0x1415C, 409002},
  {0x1415D, 409028},
  {0x1415E, 409054},
  {0x1415F, 409080},
  {0x14160, 409106},
  {0x14161, 409132},
  {0x14162, 409158},
  {0x14163, 409184},
  {0x14164, 409210},
  {0x14165, 409236},
  {0x14166, 409262},
  {0x14167, 409288},
  {0x14168, 409314},
  {0x14169, 409340},
  {0x1416A, 409366},
  {0x1416B, 409392},
  {0x1416C, 409418},
  {0x1416D, 409444},
  {0x1416E, 409470},
  {0x1416F, 409496},
  {0x14170, 409522},
  {0x14171, 409548},
  {0x14172, 409574},
  {0x14173, 409600},
  {0x14174, 409626},
  {0x14175, 409652},
  {0x14176, 409678},
  {0x14177, 409704},
  {0x14178, 409730},
  {0x14179, 409756},
  {0x1417A, 409782},
  {0x1417B, 409808},
  {0x1417C, 409834},
  {0x1417D, 409860},
  {0x1417E, 409886},
  {0x1417F, 409912},
  {0x14180, 409938},
  {0x14181, 409964},
  {0x14182, 409990},
  {0x14183, 410016},
  {0x14184, 410042},
  {0x14185, 410068},
  {0x14186, 410094},
  {0x14187, 410120},
  {0x14188, 410146},
  {0x14189, 410172},
  {0x1418A, 410198},
  {0x1418B, 410224},
  {0x1418C, 410250},
  {0x1418D, 410276},
  {0x1418E, 410302},
  {0x1418F, 410328},
  {0x14190, 410354},
  {0x14191, 410380},
  {0x14192, 410406},
  {0x14193, 410432},
  {0x14194, 410458},
  {0x14195, 410484},
  {0x14196, 410510},
  {0x14197, 410536},
  {0x14198, 410562},
  {0x14199, 410588},
  {0x1419A, 410614},
  {0x1419B, 410640},
  {0x1419C, 410666},
  {0x1419D, 410692},
  {0x1419E, 410718},
  {0x1419F, 410744},
  {0x141A0, 410770},
  {0x141A1, 410796},
  {0x141A2, 410822},
  {0x141A3, 410848},
  {0x141A4, 410874},
  {0x141A5, 410900},
  {0x141A6, 410926},
  {0x141A7, 410952},
  {0x141A8, 410978},
  {0x141A9, 411004},
  {0x141AA, 411030},
  {0x141AB, 411056},
  {0x141AC, 411082},
  {0x141AD, 411108},
  {0x141AE, 411134},
  {0x141AF, 411160},
  {0x141B0, 411186},
  {0x141B1, 411212},
  {0x141B2, 411238},
  {0x141B3, 411264},
  {0x141B4, 411290},
  {0x141B5, 411316},
  {0x141B6, 411342},
  {0x141B7, 411368},
  {0x141B8, 411394},
  {0x141B9, 411420},
  {0x141BA, 411446},
  {0x141BB, 411472},
  {0x141BC, 411498},
  {0x141BD, 411524},
  {0x141BE, 411550},
  {0x141BF, 411576},
  {0x141C0, 411602},
  {0x141C1, 411628},
  {0x141C2, 411654},
  {0x141C3, 411680},
  {0x141C4, 411706},
  {0x141C5, 411732},
  {0x141C6, 411758},
  {0x141C7, 411784},
  {0x141C8, 411810},
  {0x141C9, 411836},
  {0x141CA, 411862},
  {0x141CB, 411888},
  {0x141CC, 411914},
  {0x141CD, 411940},
  {0x141CE, 411966},
  {0x141CF, 411992},
  {0x141D0, 412018},
  {0x141D1, 412044},
  {0x141D2, 412070},
  {0x141D3, 412096},
  {0x141D4, 412122},
  {0x141D5, 412148},
  {0x141D6, 412174},
  {0x141D7, 412200},
  {0x141D8, 412226},
  {0x141D9, 412252},
  {0x141DA, 412278},
  {0x141DB, 412304},
  {0x141DC, 412330},
  {0x141DD, 412356},
  {0x141DE, 412382},
  {0x141DF, 412408},
  {0x141E0, 412434},
  {0x141E1, 412460},
  {0x141E2, 412486},
  {0x141E3, 412512},
  {0x141E4, 412538},
  {0x141E5, 412564},
  {0x141E6, 412590},
  {0x141E7, 412616},
  {0x141E8, 412642},
  {0x141E9, 412668},
  {0x141EA, 412694},
  {0x141EB, 412720},
  {0x141EC, 412746},
  {0x141ED, 412772},
  {0x141EE, 412798},
  {0x141EF, 412824},
  {0x141F0, 412850},
  {0x141F1, 412876},
  {0x141F2, 412902},
  {0x141F3, 412928},
  {0x141F4, 412954},
  {0x141F5, 412980},
  {0x141F6, 413006},
  {0x141F7, 413032},
  {0x141F8, 413058},
  {0x141F9, 413084},
  {0x141FA, 413110},
  {0x141FB, 413136},
  {0x141FC, 413162},
  {0x141FD, 413188},
  {0x141FE, 413214},
  {0x141FF, 413240},
  {0x14200, 413266},
  {0x14201, 413292},
  {0x14202, 413318},
  {0x14203, 413344},
  {0x14204, 413370},
  {0x14205, 413396},
  {0x14206, 413422},
  {0x14207, 413448},
  {0x14208, 413474},
  {0x14209, 413500},
  {0x1420A, 413526},
  {0x1420B, 413552},
  {0x1420C, 413578},
  {0x1420D, 413604},
  {0x1420E, 413630},
  {0x1420F, 413656},
  {0x14210, 413682},
  {0x14211, 413708},
  {0x14212, 413734},
  {0x14213, 413760},
  {0x14214, 413786},
  {0x14215, 413812},
  {0x14216, 413838},
  {0x14217, 413864},
  {0x14218, 413890},
  {0x14219, 413916},
  {0x1421A, 413942},
  {0x1421B, 413968},
  {0x1421C, 413994},
  {0x1421D, 414020},
  {0x1421E, 414046},
  {0x1421F, 414072},
  {0x14220, 414098},
  {0x14221, 414124},
  {0x14222, 414150},
  {0x14223, 414176},
  {0x14224, 414202},
  {0x14225, 414228},
  {0x14226, 414254},
  {0x14227, 414280},
  {0x14228, 414306},
  {0x14229, 414332},
  {0x1422A, 414358},
  {0x1422B, 414384},
  {0x1422C, 414410},
  {0x1422D, 414436},
  {0x1422E, 414462},
  {0x1422F, 414488},
  {0x14230, 414514},
  {0x14231, 414540},
  {0x14232, 414566},
  {0x14233, 414592},
  {0x14234, 414618},
  {0x14235, 414644},
  {0x14236, 414670},
  {0x14237, 414696},
  {0x14238, 414722},
  {0x14239, 414748},
  {0x1423A, 414774},
  {0x1423B, 414800},
  {0x1423C, 414826},
  {0x1423D, 414852},
  {0x1423E, 414878},
  {0x1423F, 414904},
  {0x14240, 414930},
  {0x14241, 414956},
  {0x14242, 414982},
  {0x14243, 415008},
  {0x14244, 415034},
  {0x14245, 415060},
  {0x14246, 415086},
  {0x14247, 415112},
  {0x14248, 415138},
  {0x14249, 415164},
  {0x1424A, 415190},
  {0x1424B, 415216},
  {0x1424C, 415242},
  {0x1424D, 415268},
  {0x1424E, 415294},
  {0x1424F, 415320},
  {0x14250, 415346},
  {0x14251, 415372},
  {0x14252, 415398},
  {0x14253, 415424},
  {0x14254, 415450},
  {0x14255, 415476},
  {0x14256, 415502},
  {0x14257, 415528},
  {0x14258, 415554},
  {0x14259, 415580},
  {0x1425A, 415606},
  {0x1425B, 415632},
  {0x1425C, 415658},
  {0x1425D, 415684},
  {0x1425E, 415710},
  {0x1425F, 415736},
  {0x14260, 415762},
  {0x14261, 415788},
  {0x14262, 415814},
  {0x14263, 415840},
  {0x14264, 415866},
  {0x14265, 415892},
  {0x14266, 415918},
  {0x14267, 415944},
  {0x14268, 415970},
  {0x14269, 415996},
  {0x1426A, 416022},
  {0x1426B, 416048},
  {0x1426C, 416074},
  {0x1426D, 416100},
  {0x1426E, 416126},
  {0x1426F, 416152},
  {0x14270, 416178},
  {0x14271, 416204},
  {0x14272, 416230},
  {0x14273, 416256},
  {0x14274, 416282},
  {0x14275, 416308},
  {0x14276, 416334},
  {0x14277, 416360},
  {0x14278, 416386},
  {0x14279, 416412},
  {0x1427A, 416438},
  {0x1427B, 416464},
  {0x1427C, 416490},
  {0x1427D, 416516},
  {0x1427E, 416542},
  {0x1427F, 416568},
  {0x14280, 416594},
  {0x14281, 416620},
  {0x14282, 416646},
  {0x14283, 416672},
  {0x14284, 416698},
  {0x14285, 416724},
  {0x14286, 416750},
  {0x14287, 416776},
  {0x14288, 416802},
  {0x14289, 416828},
  {0x1428A, 416854},
  {0x1428B, 416880},
  {0x1428C, 416906},
  {0x1428D, 416932},
  {0x1428E, 416958},
  {0x1428F, 416984},
  {0x14290, 417010},
  {0x14291, 417036},
  {0x14292, 417062},
  {0x14293, 417088},
  {0x14294, 417114},
  {0x14295, 417140},
  {0x14296, 417166},
  {0x14297, 417192},
  {0x14298, 417218},
  {0x14299, 417244},
  {0x1429A, 417270},
  {0x1429B, 417296},
  {0x1429C, 417322},
  {0x1429D, 417348},
  {0x1429E, 417374},
  {0x1429F, 417400},
  {0x142A0, 417426},
  {0x142A1, 417452},
  {0x142A2, 417478},
  {0x142A3, 417504},
  {0x142A4, 417530},
  {0x142A5, 417556},
  {0x142A6, 417582},
  {0x142A7, 417608},
  {0x142A8, 417634},
  {0x142A9, 417660},
  {0x142AA, 417686},
  {0x142AB, 417712},
  {0x142AC, 417738},
  {0x142AD, 417764},
  {0x142AE, 417790},
  {0x142AF, 417816},
  {0x142B0, 417842},
  {0x142B1, 417868},
  {0x142B2, 417894},
  {0x142B3, 417920},
  {0x142B4, 417946},
  {0x142B5, 417972},
  {0x142B6, 417998},
  {0x142B7, 418024},
  {0x142B8, 418050},
  {0x142B9, 418076},
  {0x142BA, 418102},
  {0x142BB, 418128},
  {0x142BC, 418154},
  {0x142BD, 418180},
  {0x142BE, 418206},
  {0x142BF, 418232},
  {0x142C0, 418258},
  {0x142C1, 418284},
  {0x142C2, 418310},
  {0x142C3, 418336},
  {0x142C4, 418362},
  {0x142C5, 418388},
  {0x142C6, 418414},
  {0x142C7, 418440},
  {0x142C8, 418466},
  {0x142C9, 418492},
  {0x142CA, 418518},
  {0x142CB, 418544},
  {0x142CC, 418570},
  {0x142CD, 418596},
  {0x142CE, 418622},
  {0x142CF, 418648},
  {0x142D0, 418674},
  {0x142D1, 418700},
  {0x142D2, 418726},
  {0x142D3, 418752},
  {0x142D4, 418778},
  {0x142D5, 418804},
  {0x142D6, 418830},
  {0x142D7, 418856},
  {0x142D8, 418882},
  {0x142D9, 418908},
  {0x142DA, 418934},
  {0x142DB, 418960},
  {0x142DC, 418986},
  {0x142DD, 419012},
  {0x142DE, 419038},
  {0x142DF, 419064},
  {0x142E0, 419090},
  {0x142E1, 419116},
  {0x142E2, 419142},
  {0x142E3, 419168},
  {0x142E4, 419194},
  {0x142E5, 419220},
  {0x142E6, 419246},
  {0x142E7, 419272},
  {0x142E8, 419298},
  {0x142E9, 419324},
  {0x142EA, 419350},
  {0x142EB, 419376},
  {0x142EC, 419402},
  {0x142ED, 419428},
  {0x142EE, 419454},
  {0x142EF, 419480},
  {0x142F0, 419506},
  {0x142F1, 419532},
  {0x142F2, 419558},
  {0x142F3, 419584},
  {0x142F4, 419610},
  {0x142F5, 419636},
  {0x142F6, 419662},
  {0x142F7, 419688},
  {0x142F8, 419714},
  {0x142F9, 419740},
  {0x142FA, 419766},
  {0x142FB, 419792},
  {0x142FC, 419818},
  {0x142FD, 419844},
  {0x142FE, 419870},
  {0x142FF, 419896},
  {0x14300, 419922},
  {0x14301, 419948},
  {0x14302, 419974},
  {0x14303, 420000},
  {0x14304, 420026},
  {0x14305, 420052},
  {0x14306, 420078},
  {0x14307, 420104},
  {0x14308, 420130},
  {0x14309, 420156},
  {0x1430A, 420182},
  {0x1430B, 420208},
  {0x1430C, 420234},
  {0x1430D, 420260},
  {0x1430E, 420286},
  {0x1430F, 420312},
  {0x14310, 420338},
  {0x14311, 420364},
  {0x14312, 420390},
  {0x14313, 420416},
  {0x14314, 420442},
  {0x14315, 420468},
  {0x14316, 420494},
  {0x14317, 420520},
  {0x14318, 420546},
  {0x14319, 420572},
  {0x1431A, 420598},
  {0x1431B, 420624},
  {0x1431C, 420650},
  {0x1431D, 420676},
  {0x1431E, 420702},
  {0x1431F, 420728},
  {0x14320, 420754},
  {0x14321, 420780},
  {0x14322, 420806},
  {0x14323, 420832},
  {0x14324, 420858},
  {0x14325, 420884},
  {0x14326, 420910},
  {0x14327, 420936},
  {0x14328, 420962},
  {0x14329, 420988},
  {0x1432A, 421014},
  {0x1432B, 421040},
  {0x1432C, 421066},
  {0x1432D, 421092},
  {0x1432E, 421118},
  {0x1432F, 421144},
  {0x14330, 421170},
  {0x14331, 421196},
  {0x14332, 421222},
  {0x14333, 421248},
  {0x14334, 421274},
  {0x14335, 421300},
  {0x14336, 421326},
  {0x14337, 421352},
  {0x14338, 421378},
  {0x14339, 421404},
  {0x1433A, 421430},
  {0x1433B, 421456},
  {0x1433C, 421482},
  {0x1433D, 421508},
  {0x1433E, 421534},
  {0x1433F, 421560},
  {0x14340, 421586},
  {0x14341, 421612},
  {0x14342, 421638},
  {0x14343, 421664},
  {0x14344, 421690},
  {0x14345, 421716},
  {0x14346, 421742},
  {0x14347, 421768},
  {0x14348, 421794},
  {0x14349, 421820},
  {0x1434A, 421846},
  {0x1434B, 421872},
  {0x1434C, 421898},
  {0x1434D, 421924},
  {0x1434E, 421950},
  {0x1434F, 421976},
  {0x14350, 422002},
  {0x14351, 422028},
  {0x14352, 422054},
  {0x14353, 422080},
  {0x14354, 422106},
  {0x14355, 422132},
  {0x14356, 422158},
  {0x14357, 422184},
  {0x14358, 422210},
  {0x14359, 422236},
  {0x1435A, 422262},
  {0x1435B, 422288},
  {0x1435C, 422314},
  {0x1435D, 422340},
  {0x1435E, 422366},
  {0x1435F, 422392},
  {0x14360, 422418},
  {0x14361, 422444},
  {0x14362, 422470},
  {0x14363, 422496},
  {0x14364, 422522},
  {0x14365, 422548},
  {0x14366, 422574},
  {0x14367, 422600},
  {0x14368, 422626},
  {0x14369, 422652},
  {0x1436A, 422678},
  {0x1436B, 422704},
  {0x1436C, 422730},
  {0x1436D, 422756},
  {0x1436E, 422782},
  {0x1436F, 422808},
  {0x14370, 422834},
  {0x14371, 422860},
  {0x14372, 422886},
  {0x14373, 422912},
  {0x14374, 422938},
  {0x14375, 422964},
  {0x14376, 422990},
  {0x14377, 423016},
  {0x14378, 423042},
  {0x14379, 423068},
  {0x1437A, 423094},
  {0x1437B, 423120},
  {0x1437C, 423146},
  {0x1437D, 423172},
  {0x1437E, 423198},
  {0x1437F, 423224},
  {0x14380, 423250},
  {0x14381, 423276},
  {0x14382, 423302},
  {0x14383, 423328},
  {0x14384, 423354},
  {0x14385, 423380},
  {0x14386, 423406},
  {0x14387, 423432},
  {0x14388, 423458},
  {0x14389, 423484},
  {0x1438A, 423510},
  {0x1438B, 423536},
  {0x1438C, 423562},
  {0x1438D, 423588},
  {0x1438E, 423614},
  {0x1438F, 423640},
  {0x14390, 423666},
  {0x14391, 423692},
  {0x14392, 423718},
  {0x14393, 423744},
  {0x14394, 423770},
  {0x14395, 423796},
  {0x14396, 423822},
  {0x14397, 423848},
  {0x14398, 423874},
  {0x14399, 423900},
  {0x1439A, 423926},
  {0x1439B, 423952},
  {0x1439C, 423978},
  {0x1439D, 424004},
  {0x1439E, 424030},
  {0x1439F, 424056},
  {0x143A0, 424082},
  {0x143A1, 424108},
  {0x143A2, 424134},
  {0x143A3, 424160},
  {0x143A4, 424186},
  {0x143A5, 424212},
  {0x143A6, 424238},
  {0x143A7, 424264},
  {0x143A8, 424290},
  {0x143A9, 424316},
  {0x143AA, 424342},
  {0x143AB, 424368},
  {0x143AC, 424394},
  {0x143AD, 424420},
  {0x143AE, 424446},
  {0x143AF, 424472},
  {0x143B0, 424498},
  {0x143B1, 424524},
  {0x143B2, 424550},
  {0x143B3, 424576},
  {0x143B4, 424602},
  {0x143B5, 424628},
  {0x143B6, 424654},
  {0x143B7, 424680},
  {0x143B8, 424706},
  {0x143B9, 424732},
  {0x143BA, 424758},
  {0x143BB, 424784},
  {0x143BC, 424810},
  {0x143BD, 424836},
  {0x143BE, 424862},
  {0x143BF, 424888},
  {0x143C0, 424914},
  {0x143C1, 424940},
  {0x143C2, 424966},
  {0x143C3, 424992},
  {0x143C4, 425018},
  {0x143C5, 425044},
  {0x143C6, 425070},
  {0x143C7, 425096},
  {0x143C8, 425122},
  {0x143C9, 425148},
  {0x143CA, 425174},
  {0x143CB, 425200},
  {0x143CC, 425226},
  {0x143CD, 425252},
  {0x143CE, 425278},
  {0x143CF, 425304},
  {0x143D0, 425330},
  {0x143D1, 425356},
  {0x143D2, 425382},
  {0x143D3, 425408},
  {0x143D4, 425434},
  {0x143D5, 425460},
  {0x143D6, 425486},
  {0x143D7, 425512},
  {0x143D8, 425538},
  {0x143D9, 425564},
  {0x143DA, 425590},
  {0x143DB, 425616},
  {0x143DC, 425642},
  {0x143DD, 425668},
  {0x143DE, 425694},
  {0x143DF, 425720},
  {0x143E0, 425746},
  {0x143E1, 425772},
  {0x143E2, 425798},
  {0x143E3, 425824},
  {0x143E4, 425850},
  {0x143E5, 425876},
  {0x143E6, 425902},
  {0x143E7, 425928},
  {0x143E8, 425954},
  {0x143E9, 425980},
  {0x143EA, 426006},
  {0x143EB, 426032},
  {0x143EC, 426058},
  {0x143ED, 426084},
  {0x143EE, 426110},
  {0x143EF, 426136},
  {0x143F0, 426162},
  {0x143F1, 426188},
  {0x143F2, 426214},
  {0x143F3, 426240},
  {0x143F4, 426266},
  {0x143F5, 426292},
  {0x143F6, 426318},
  {0x143F7, 426344},
  {0x143F8, 426370},
  {0x143F9, 426396},
  {0x143FA, 426422},
  {0x14400, 9311},
  {0x14401, 9337},
  {0x14402, 9363},
  {0x14403, 9389},
  {0x14404, 9415},
  {0x14405, 9441},
  {0x14406, 9467},
  {0x14407, 9493},
  {0x14408, 9519},
  {0x14409, 9545},
  {0x1440A, 9571},
  {0x1440B, 9598},
  {0x1440C, 9624},
  {0x1440D, 9650},
  {0x1440E, 9676},
  {0x1440F, 9702},
  {0x14410, 9728},
  {0x14411, 9754},
  {0x14412, 9780},
  {0x14413, 9806},
  {0x14414, 9832},
  {0x14415, 9858},
  {0x14416, 9884},
  {0x14417, 9910},
  {0x14418, 9936},
  {0x14419, 9962},
  {0x1441A, 9988},
  {0x1441B, 10014},
  {0x1441C, 10041},
  {0x1441D, 10067},
  {0x1441E, 10093},
  {0x1441F, 10119},
  {0x14420, 10145},
  {0x14421, 10171},
  {0x14422, 10197},
  {0x14423, 10223},
  {0x14424, 10249},
  {0x14425, 10275},
  {0x14426, 10301},
  {0x14427, 10327},
  {0x14428, 10353},
  {0x14429, 10379},
  {0x1442A, 10406},
  {0x1442B, 10432},
  {0x1442C, 10458},
  {0x1442D, 10485},
  {0x1442E, 10511},
  {0x1442F, 10537},
  {0x14430, 10563},
  {0x14431, 10589},
  {0x14432, 10616},
  {0x14433, 10642},
  {0x14434, 10669},
  {0x14435, 10696},
  {0x14436, 10722},
  {0x14437, 10748},
  {0x14438, 10774},
  {0x14439, 10800},
  {0x1443A, 10826},
  {0x1443B, 10852},
  {0x1443C, 10878},
  {0x1443D, 10904},
  {0x1443E, 10930},
  {0x1443F, 10956},
  {0x14440, 10982},
  {0x14441, 11008},
  {0x14442, 11034},
  {0x14443, 11060},
  {0x14444, 11086},
  {0x14445, 11112},
  {0x14446, 11138},
  {0x14447, 11164},
  {0x14448, 11190},
  {0x14449, 11216},
  {0x1444A, 11243},
  {0x1444B, 11270},
  {0x1444C, 11297},
  {0x1444D, 11323},
  {0x1444E, 11349},
  {0x1444F, 11375},
  {0x14450, 11401},
  {0x14451, 11427},
  {0x14452, 11453},
  {0x14453, 11479},
  {0x14454, 11505},
  {0x14455, 11531},
  {0x14456, 11557},
  {0x14457, 11583},
  {0x14458, 11609},
  {0x14459, 11635},
  {0x1445A, 11661},
  {0x1445B, 11687},
  {0x1445C, 11713},
  {0x1445D, 11739},
  {0x1445E, 11765},
  {0x1445F, 11791},
  {0x14460, 11817},
  {0x14461, 11843},
  {0x14462, 11869},
  {0x14463, 11895},
  {0x14464, 11921},
  {0x14465, 11947},
  {0x14466, 11973},
  {0x14467, 11999},
  {0x14468, 12025},
  {0x14469, 12051},
  {0x1446A, 12077},
  {0x1446B, 12103},
  {0x1446C, 12130},
  {0x1446D, 12156},
  {0x1446E, 12183},
  {0x1446F, 12209},
  {0x14470, 12235},
  {0x14471, 12262},
  {0x14472, 12288},
  {0x14473, 12315},
  {0x14474, 12341},
  {0x14475, 12368},
  {0x14476, 12394},
  {0x14477, 12420},
  {0x14478, 12447},
  {0x14479, 12474},
  {0x1447A, 12501},
  {0x1447B, 12527},
  {0x1447C, 12554},
  {0x1447D, 12581},
  {0x1447E, 12607},
  {0x1447F, 12633},
  {0x14480, 12660},
  {0x14481, 12687},
  {0x14482, 12714},
  {0x14483, 12740},
  {0x14484, 12766},
  {0x14485, 12792},
  {0x14486, 12819},
  {0x14487, 12846},
  {0x14488, 12872},
  {0x14489, 12898},
  {0x1448A, 12924},
  {0x1448B, 12950},
  {0x1448C, 12976},
  {0x1448D, 13003},
  {0x1448E, 13029},
  {0x1448F, 13055},
  {0x14490, 13081},
  {0x14491, 13107},
  {0x14492, 13133},
  {0x14493, 13159},
  {0x14494, 13185},
  {0x14495, 13211},
  {0x14496, 13237},
  {0x14497, 13263},
  {0x14498, 13290},
  {0x14499, 13316},
  {0x1449A, 13342},
  {0x1449B, 13368},
  {0x1449C, 13394},
  {0x1449D, 13420},
  {0x1449E, 13446},
  {0x1449F, 13472},
  {0x144A0, 13498},
  {0x144A1, 13524},
  {0x144A2, 13550},
  {0x144A3, 13577},
  {0x144A4, 13603},
  {0x144A5, 13629},
  {0x144A6, 13655},
  {0x144A7, 13681},
  {0x144A8, 13707},
  {0x144A9, 13733},
  {0x144AA, 13759},
  {0x144AB, 13785},
  {0x144AC, 13811},
  {0x144AD, 13837},
  {0x144AE, 13863},
  {0x144AF, 13889},
  {0x144B0, 13915},
  {0x144B1, 13941},
  {0x144B2, 13967},
  {0x144B3, 13993},
  {0x144B4, 14019},
  {0x144B5, 14045},
  {0x144B6, 14071},
  {0x144B7, 14097},
  {0x144B8, 14123},
  {0x144B9, 14149},
  {0x144BA, 14175},
  {0x144BB, 14201},
  {0x144BC, 14227},
  {0x144BD, 14253},
  {0x144BE, 14279},
  {0x144BF, 14305},
  {0x144C0, 14331},
  {0x144C1, 14357},
  {0x144C2, 14383},
  {0x144C3, 14409},
  {0x144C4, 14435},
  {0x144C5, 14461},
  {0x144C6, 14487},
  {0x144C7, 14513},
  {0x144C8, 14539},
  {0x144C9, 14565},
  {0x144CA, 14591},
  {0x144CB, 14617},
  {0x144CC, 14643},
  {0x144CD, 14669},
  {0x144CE, 14695},
  {0x144CF, 14721},
  {0x144D0, 14747},
  {0x144D1, 14773},
  {0x144D2, 14799},
  {0x144D3, 14825},
  {0x144D4, 14851},
  {0x144D5, 14877},
  {0x144D6, 14903},
  {0x144D7, 14929},
  {0x144D8, 14955},
  {0x144D9, 14981},
  {0x144DA, 15007},
  {0x144DB, 15033},
  {0x144DC, 15059},
  {0x144DD, 15085},
  {0x144DE, 15111},
  {0x144DF, 15137},
  {0x144E0, 15163},
  {0x144E1, 15189},
  {0x144E2, 15215},
  {0x144E3, 15241},
  {0x144E4, 15267},
  {0x144E5, 15293},
  {0x144E6, 15319},
  {0x144E7, 15346},
  {0x144E8, 15373},
  {0x144E9, 15399},
  {0x144EA, 15425},
  {0x144EB, 15451},
  {0x144EC, 15477},
  {0x144ED, 15503},
  {0x144EE, 15530},
  {0x144EF, 15556},
  {0x144F0, 15582},
  {0x144F1, 15609},
  {0x144F2, 15635},
  {0x144F3, 15661},
  {0x144F4, 15687},
  {0x144F5, 15713},
  {0x144F6, 15739},
  {0x144F7, 15765},
  {0x144F8, 15792},
  {0x144F9, 15818},
  {0x144FA, 15845},
  {0x144FB, 15871},
  {0x144FC, 15897},
  {0x144FD, 15923},
  {0x144FE, 15949},
  {0x144FF, 15975},
  {0x14500, 16001},
  {0x14501, 16027},
  {0x14502, 16053},
  {0x14503, 16079},
  {0x14504, 16105},
  {0x14505, 16131},
  {0x14506, 16158},
  {0x14507, 16184},
  {0x14508, 16210},
  {0x14509, 16236},
  {0x1450A, 16262},
  {0x1450B, 16288},
  {0x1450C, 16314},
  {0x1450D, 16340},
  {0x1450E, 16366},
  {0x1450F, 16392},
  {0x14510, 16418},
  {0x14511, 16444},
  {0x14512, 16470},
  {0x14513, 16496},
  {0x14514, 16522},
  {0x14515, 16548},
  {0x14516, 16574},
  {0x14517, 16600},
  {0x14518, 16626},
  {0x14519, 16652},
  {0x1451A, 16678},
  {0x1451B, 16704},
  {0x1451C, 16730},
  {0x1451D, 16756},
  {0x1451E, 16782},
  {0x1451F, 16808},
  {0x14520, 16834},
  {0x14521, 16860},
  {0x14522, 16886},
  {0x14523, 16912},
  {0x14524, 16938},
  {0x14525, 16964},
  {0x14526, 16990},
  {0x14527, 17016},
  {0x14528, 17042},
  {0x14529, 17068},
  {0x1452A, 17094},
  {0x1452B, 17120},
  {0x1452C, 17146},
  {0x1452D, 17172},
  {0x1452E, 17198},
  {0x1452F, 17225},
  {0x14530, 17251},
  {0x14531, 17277},
  {0x14532, 17303},
  {0x14533, 17329},
  {0x14534, 17355},
  {0x14535, 17381},
  {0x14536, 17407},
  {0x14537, 17433},
  {0x14538, 17459},
  {0x14539, 17485},
  {0x1453A, 17511},
  {0x1453B, 17537},
  {0x1453C, 17563},
  {0x1453D, 17589},
  {0x1453E, 17615},
  {0x1453F, 17641},
  {0x14540, 17667},
  {0x14541, 17693},
  {0x14542, 17719},
  {0x14543, 17745},
  {0x14544, 17771},
  {0x14545, 17797},
  {0x14546, 17824},
  {0x14547, 17850},
  {0x14548, 17876},
  {0x14549, 17902},
  {0x1454A, 17928},
  {0x1454B, 17954},
  {0x1454C, 17981},
  {0x1454D, 18007},
  {0x1454E, 18033},
  {0x1454F, 18059},
  {0x14550, 18085},
  {0x14551, 18111},
  {0x14552, 18138},
  {0x14553, 18164},
  {0x14554, 18190},
  {0x14555, 18216},
  {0x14556, 18242},
  {0x14557, 18268},
  {0x14558, 18294},
  {0x14559, 18320},
  {0x1455A, 18346},
  {0x1455B, 18372},
  {0x1455C, 18398},
  {0x1455D, 18425},
  {0x1455E, 18451},
  {0x1455F, 18477},
  {0x14560, 18503},
  {0x14561, 18529},
  {0x14562, 18555},
  {0x14563, 18581},
  {0x14564, 18607},
  {0x14565, 18633},
  {0x14566, 18659},
  {0x14567, 18685},
  {0x14568, 18711},
  {0x14569, 18737},
  {0x1456A, 18763},
  {0x1456B, 18789},
  {0x1456C, 18815},
  {0x1456D, 18841},
  {0x1456E, 18867},
  {0x1456F, 18893},
  {0x14570, 18919},
  {0x14571, 18945},
  {0x14572, 18972},
  {0x14573, 18998},
  {0x14574, 19024},
  {0x14575, 19051},
  {0x14576, 19078},
  {0x14577, 19105},
  {0x14578, 19131},
  {0x14579, 19157},
  {0x1457A, 19183},
  {0x1457B, 19209},
  {0x1457C, 19236},
  {0x1457D, 19263},
  {0x1457E, 19290},
  {0x1457F, 19316},
  {0x14580, 19342},
  {0x14581, 19368},
  {0x14582, 19394},
  {0x14583, 19420},
  {0x14584, 19446},
  {0x14585, 19472},
  {0x14586, 19498},
  {0x14587, 19524},
  {0x14588, 19550},
  {0x14589, 19576},
  {0x1458A, 19602},
  {0x1458B, 19628},
  {0x1458C, 19654},
  {0x1458D, 19680},
  {0x1458E, 19706},
  {0x1458F, 19732},
  {0x14590, 19758},
  {0x14591, 19784},
  {0x14592, 19810},
  {0x14593, 19836},
  {0x14594, 19862},
  {0x14595, 19888},
  {0x14596, 19915},
  {0x14597, 19941},
  {0x14598, 19967},
  {0x14599, 19993},
  {0x1459A, 20019},
  {0x1459B, 20045},
  {0x1459C, 20072},
  {0x1459D, 20098},
  {0x1459E, 20124},
  {0x1459F, 20150},
  {0x145A0, 20176},
  {0x145A1, 20203},
  {0x145A2, 20229},
  {0x145A3, 20255},
  {0x145A4, 20281},
  {0x145A5, 20308},
  {0x145A6, 20334},
  {0x145A7, 20360},
  {0x145A8, 20386},
  {0x145A9, 20412},
  {0x145AA, 20438},
  {0x145AB, 20464},
  {0x145AC, 20490},
  {0x145AD, 20516},
  {0x145AE, 20542},
  {0x145AF, 20568},
  {0x145B0, 20595},
  {0x145B1, 20621},
  {0x145B2, 20656},
  {0x145B3, 20683},
  {0x145B4, 20709},
  {0x145B5, 20735},
  {0x145B6, 20761},
  {0x145B7, 20788},
  {0x145B8, 20814},
  {0x145B9, 20840},
  {0x145BA, 20866},
  {0x145BB, 20892},
  {0x145BC, 20918},
  {0x145BD, 20944},
  {0x145BE, 20976},
  {0x145BF, 21002},
  {0x145C0, 21028},
  {0x145C1, 21054},
  {0x145C2, 21080},
  {0x145C3, 21106},
  {0x145C4, 21132},
  {0x145C5, 21158},
  {0x145C6, 21184},
  {0x145C7, 21210},
  {0x145C8, 21236},
  {0x145C9, 21262},
  {0x145CA, 21288},
  {0x145CB, 21314},
  {0x145CC, 21340},
  {0x145CD, 21366},
  {0x145CE, 21392},
  {0x145CF, 21438},
  {0x145D0, 21483},
  {0x145D1, 21509},
  {0x145D2, 21535},
  {0x145D3, 21561},
  {0x145D4, 21587},
  {0x145D5, 21613},
  {0x145D6, 21639},
  {0x145D7, 21665},
  {0x145D8, 21691},
  {0x145D9, 21717},
  {0x145DA, 21743},
  {0x145DB, 21769},
  {0x145DC, 21795},
  {0x145DD, 21821},
  {0x145DE, 21847},
  {0x145DF, 21873},
  {0x145E0, 21899},
  {0x145E1, 21925},
  {0x145E2, 21951},
  {0x145E3, 21977},
  {0x145E4, 22003},
  {0x145E5, 22029},
  {0x145E6, 22055},
  {0x145E7, 22081},
  {0x145E8, 22107},
  {0x145E9, 22133},
  {0x145EA, 22159},
  {0x145EB, 22185},
  {0x145EC, 22211},
  {0x145ED, 22237},
  {0x145EE, 22263},
  {0x145EF, 22289},
  {0x145F0, 22315},
  {0x145F1, 22341},
  {0x145F2, 22367},
  {0x145F3, 22393},
  {0x145F4, 22419},
  {0x145F5, 22445},
  {0x145F6, 22471},
  {0x145F7, 22497},
  {0x145F8, 22523},
  {0x145F9, 22550},
  {0x145FA, 22576},
  {0x145FB, 22602},
  {0x145FC, 22628},
  {0x145FD, 22654},
  {0x145FE, 22680},
  {0x145FF, 22706},
  {0x14600, 22732},
  {0x14601, 22759},
  {0x14602, 22785},
  {0x14603, 22811},
  {0x14604, 22837},
  {0x14605, 22863},
  {0x14606, 22889},
  {0x14607, 22915},
  {0x14608, 22941},
  {0x14609, 22967},
  {0x1460A, 22993},
  {0x1460B, 23019},
  {0x1460C, 23045},
  {0x1460D, 23071},
  {0x1460E, 23097},
  {0x1460F, 23123},
  {0x14610, 23149},
  {0x14611, 23175},
  {0x14612, 23201},
  {0x14613, 23227},
  {0x14614, 23253},
  {0x14615, 23279},
  {0x14616, 23305},
  {0x14617, 23331},
  {0x14618, 23357},
  {0x14619, 23383},
  {0x1461A, 23409},
  {0x1461B, 23435},
  {0x1461C, 23461},
  {0x1461D, 23487},
  {0x1461E, 23513},
  {0x1461F, 23539},
  {0x14620, 23565},
  {0x14621, 23591},
  {0x14622, 23617},
  {0x14623, 23643},
  {0x14624, 23669},
  {0x14625, 23695},
  {0x14626, 23721},
  {0x14627, 23747},
  {0x14628, 23773},
  {0x14629, 23799},
  {0x1462A, 23825},
  {0x1462B, 23851},
  {0x1462C, 23877},
  {0x1462D, 23903},
  {0x1462E, 23929},
  {0x1462F, 23955},
  {0x14630, 23981},
  {0x14631, 24007},
  {0x14632, 24033},
  {0x14633, 24059},
  {0x14634, 24085},
  {0x14635, 24111},
  {0x14636, 24137},
  {0x14637, 24163},
  {0x14638, 24189},
  {0x14639, 24215},
  {0x1463A, 24241},
  {0x1463B, 24267},
  {0x1463C, 24293},
  {0x1463D, 24319},
  {0x1463E, 24345},
  {0x1463F, 24371},
  {0x14640, 24397},
  {0x14641, 24423},
  {0x14642, 24449},
  {0x14643, 24475},
  {0x14644, 24501},
  {0x14645, 24527},
  {0x14646, 24553},
  {0x16100, 486316},
  {0x16101, 486645},
  {0x16102, 486668},
  {0x16103, 486528},
  {0x16104, 486551},
  {0x16105, 486761},
  {0x16106, 486385},
  {0x16107, 486408},
  {0x16108, 486598},
  {0x16109, 486621},
  {0x1610A, 486575},
  {0x1610B, 486925},
  {0x1610C, 486949},
  {0x1610D, 486455},
  {0x1610E, 486479},
  {0x1610F, 486974},
  {0x16110, 486878},
  {0x16111, 486901},
  {0x16112, 486432},
  {0x16113, 486504},
  {0x16114, 486738},
  {0x16115, 486785},
  {0x16116, 486808},
  {0x16117, 486338},
  {0x16118, 486361},
  {0x16119, 486715},
  {0x1611A, 486997},
  {0x1611B, 486832},
  {0x1611C, 486692},
  {0x1611D, 486855},
  {0x1611E, 487105},
  {0x1611F, 487239},
  {0x16120, 487265},
  {0x16121, 487345},
  {0x16122, 487371},
  {0x16123, 487186},
  {0x16124, 487212},
  {0x16125, 487132},
  {0x16126, 487292},
  {0x16127, 487318},
  {0x16128, 487159},
  {0x16129, 487074},
  {0x1612A, 486038},
  {0x1612B, 486000},
  {0x1612C, 485924},
  {0x1612D, 487020},
  {0x1612E, 485962},
  {0x1612F, 487047},
  {0x16130, 486292},
  {0x16131, 486173},
  {0x16132, 486269},
  {0x16133, 486244},
  {0x16134, 486125},
  {0x16135, 486101},
  {0x16136, 486221},
  {0x16137, 486196},
  {0x16138, 486076},
  {0x16139, 486149},
  {0x16800, 89451},
  {0x16801, 88355},
  {0x16802, 90152},
  {0x16803, 90113},
  {0x16804, 89248},
  {0x16805, 90457},
  {0x16806, 90703},
  {0x16807, 89883},
  {0x16808, 90358},
  {0x16809, 90235},
  {0x1680A, 90194},
  {0x1680B, 88801},
  {0x1680C, 90734},
  {0x1680D, 89396},
  {0x1680E, 88863},
  {0x1680F, 88948},
  {0x16810, 89216},
  {0x16811, 90329},
  {0x16812, 89766},
  {0x16813, 88527},
  {0x16814, 89596},
  {0x16815, 90619},
  {0x16816, 89483},
  {0x16817, 88611},
  {0x16818, 88638},
  {0x16819, 89704},
  {0x1681A, 88746},
  {0x1681B, 90060},
  {0x1681C, 88420},
  {0x1681D, 90031},
  {0x1681E, 89124},
  {0x1681F, 89913},
  {0x16820, 90792},
  {0x16821, 88386},
  {0x16822, 89735},
  {0x16823, 90515},
  {0x16824, 89096},
  {0x16825, 89062},
  {0x16826, 89187},
  {0x16827, 88451},
  {0x16828, 89944},
  {0x16829, 89303},
  {0x1682A, 90088},
  {0x1682B, 88833},
  {0x1682C, 89515},
  {0x1682D, 90279},
  {0x1682E, 90765},
  {0x1682F, 88719},
  {0x16830, 90541},
  {0x16831, 90672},
  {0x16832, 88893},
  {0x16833, 88502},
  {0x16834, 89337},
  {0x16835, 88585},
  {0x16836, 89158},
  {0x16837, 90489},
  {0x16838, 88667},
  {0x16839, 88329},
  {0x1683A, 90304},
  {0x1683B, 88559},
  {0x1683C, 90569},
  {0x1683D, 88977},
  {0x1683E, 90393},
  {0x1683F, 89678},
  {0x16840, 90421},
  {0x16841, 89846},
  {0x16842, 90003},
  {0x16843, 89567},
  {0x16844, 90823},
  {0x16845, 89424},
  {0x16846, 88921},
  {0x16847, 89541},
  {0x16848, 89008},
  {0x16849, 89795},
  {0x1684A, 90644},
  {0x1684B, 90594},
  {0x1684C, 89977},
  {0x1684D, 89650},
  {0x1684E, 88775},
  {0x1684F, 89821},
  {0x16850, 88477},
  {0x16851, 89624},
  {0x16852, 90852},
  {0x16853, 88695},
  {0x16854, 89371},
  {0x16855, 89278},
  {0x16856, 89036},
  {0x16857, 91897},
  {0x16858, 92324},
  {0x16859, 92247},
  {0x1685A, 92157},
  {0x1685B, 91242},
  {0x1685C, 91360},
  {0x1685D, 91101},
  {0x1685E, 92476},
  {0x1685F, 91392},
  {0x16860, 90991},
  {0x16861, 92046},
  {0x16862, 92441},
  {0x16863, 91181},
  {0x16864, 91925},
  {0x16865, 91660},
  {0x16866, 90958},
  {0x16867, 92074},
  {0x16868, 92358},
  {0x16869, 92185},
  {0x1686A, 92216},
  {0x1686B, 91597},
  {0x1686C, 91449},
  {0x1686D, 91988},
  {0x1686E, 91044},
  {0x1686F, 90928},
  {0x16870, 91072},
  {0x16871, 91153},
  {0x16872, 92019},
  {0x16873, 92388},
  {0x16874, 91811},
  {0x16875, 91718},
  {0x16876, 92414},
  {0x16877, 92128},
  {0x16878, 92298},
  {0x16879, 91510},
  {0x1687A, 91749},
  {0x1687B, 91777},
  {0x1687C, 91569},
  {0x1687D, 92273},
  {0x1687E, 91421},
  {0x1687F, 91630},
  {0x16880, 91959},
  {0x16881, 92099},
  {0x16882, 91536},
  {0x16883, 90902},
  {0x16884, 91483},
  {0x16885, 90877},
  {0x16886, 91016},
  {0x16887, 91297},
  {0x16888, 91330},
  {0x16889, 91217},
  {0x1688A, 91690},
  {0x1688B, 91871},
  {0x1688C, 91273},
  {0x1688D, 91128},
  {0x1688E, 91845},
  {0x1688F, 94038},
  {0x16890, 94549},
  {0x16891, 95173},
  {0x16892, 95036},
  {0x16893, 93840},
  {0x16894, 93701},
  {0x16895, 94011},
  {0x16896, 92608},
  {0x16897, 93266},
  {0x16898, 92555},
  {0x16899, 94510},
  {0x1689A, 94480},
  {0x1689B, 93324},
  {0x1689C, 94651},
  {0x1689D, 93786},
  {0x1689E, 93166},
  {0x1689F, 92831},
  {0x168A0, 92582},
  {0x168A1, 92748},
  {0x168A2, 95142},
  {0x168A3, 94574},
  {0x168A4, 93487},
  {0x168A5, 94258},
  {0x168A6, 93433},
  {0x168A7, 95116},
  {0x168A8, 92863},
  {0x168A9, 92636},
  {0x168AA, 94732},
  {0x168AB, 93759},
  {0x168AC, 94838},
  {0x168AD, 94206},
  {0x168AE, 94921},
  {0x168AF, 94811},
  {0x168B0, 93893},
  {0x168B1, 95010},
  {0x168B2, 93594},
  {0x168B3, 93049},
  {0x168B4, 94398},
  {0x168B5, 93983},
  {0x168B6, 92975},
  {0x168B7, 93403},
  {0x168B8, 93513},
  {0x168B9, 94450},
  {0x168BA, 94892},
  {0x168BB, 93956},
  {0x168BC, 92530},
  {0x168BD, 93732},
  {0x168BE, 94233},
  {0x168BF, 94287},
  {0x168C0, 92504},
  {0x168C1, 94980},
  {0x168C2, 93296},
  {0x168C3, 94680},
  {0x168C4, 94098},
  {0x168C5, 92950},
  {0x168C6, 94951},
  {0x168C7, 94366},
  {0x168C8, 94151},
  {0x168C9, 92723},
  {0x168CA, 93867},
  {0x168CB, 93139},
  {0x168CC, 93459},
  {0x168CD, 93025},
  {0x168CE, 92925},
  {0x168CF, 94125},
  {0x168D0, 94072},
  {0x168D1, 95066},
  {0x168D2, 93927},
  {0x168D3, 94314},
  {0x168D4, 94424},
  {0x168D5, 92891},
  {0x168D6, 94340},
  {0x168D7, 93649},
  {0x168D8, 93081},
  {0x168D9, 95091},
  {0x168DA, 93675},
  {0x168DB, 94784},
  {0x168DC, 94704},
  {0x168DD, 93378},
  {0x168DE, 93111},
  {0x168DF, 93238},
  {0x168E0, 92802},
  {0x168E1, 93351},
  {0x168E2, 92776},
  {0x168E3, 93205},
  {0x168E4, 95198},
  {0x168E5, 94179},
  {0x168E6, 93542},
  {0x168E7, 93568},
  {0x168E8, 94758},
  {0x168E9, 92694},
  {0x168EA, 92661},
  {0x168EB, 93814},
  {0x168EC, 93624},
  {0x168ED, 93000},
  {0x168EE, 94626},
  {0x168EF, 94601},
  {0x168F0, 94867},
  {0x168F1, 95951},
  {0x168F2, 98153},
  {0x168F3, 96900},
  {0x168F4, 96163},
  {0x168F5, 96925},
  {0x168F6, 95736},
  {0x168F7, 96873},
  {0x168F8, 96979},
  {0x168F9, 96483},
  {0x168FA, 97218},
  {0x168FB, 98178},
  {0x168FC, 96739},
  {0x168FD, 97663},
  {0x168FE, 96625},
  {0x168FF, 95360},
  {0x16900, 98028},
  {0x16901, 97689},
  {0x16902, 96818},
  {0x16903, 97872},
  {0x16904, 95335},
  {0x16905, 98205},
  {0x16906, 95526},
  {0x16907, 97585},
  {0x16908, 96084},
  {0x16909, 96402},
  {0x1690A, 95864},
  {0x1690B, 96030},
  {0x1690C, 97635},
  {0x1690D, 98234},
  {0x1690E, 96844},
  {0x1690F, 95418},
  {0x16910, 96682},
  {0x16911, 98103},
  {0x16912, 95281},
  {0x16913, 96766},
  {0x16914, 95250},
  {0x16915, 96376},
  {0x16916, 96245},
  {0x16917, 95682},
  {0x16918, 96429},
  {0x16919, 96274},
  {0x1691A, 96189},
  {0x1691B, 97449},
  {0x1691C, 96217},
  {0x1691D, 96138},
  {0x1691E, 95787},
  {0x1691F, 97033},
  {0x16920, 95893},
  {0x16921, 95577},
  {0x16922, 97244},
  {0x16923, 97974},
  {0x16924, 97821},
  {0x16925, 98311},
  {0x16926, 95392},
  {0x16927, 97475},
  {0x16928, 97947},
  {0x16929, 96652},
  {0x1692A, 96056},
  {0x1692B, 97061},
  {0x1692C, 95451},
  {0x1692D, 97399},
  {0x1692E, 97371},
  {0x1692F, 97295},
  {0x16930, 95657},
  {0x16931, 96455},
  {0x16932, 96111},
  {0x16933, 96596},
  {0x16934, 98259},
  {0x16935, 97321},
  {0x16936, 98285},
  {0x16937, 96350},
  {0x16938, 97141},
  {0x16939, 97846},
  {0x1693A, 95501},
  {0x1693B, 96540},
  {0x1693C, 96711},
  {0x1693D, 98127},
  {0x1693E, 96004},
  {0x1693F, 95552},
  {0x16940, 97089},
  {0x16941, 98002},
  {0x16942, 95835},
  {0x16943, 96513},
  {0x16944, 95710},
  {0x16945, 96568},
  {0x16946, 97006},
  {0x16947, 96952},
  {0x16948, 95307},
  {0x16949, 95477},
  {0x1694A, 97531},
  {0x1694B, 97922},
  {0x1694C, 98078},
  {0x1694D, 97270},
  {0x1694E, 97560},
  {0x1694F, 95630},
  {0x16950, 95977},
  {0x16951, 95812},
  {0x16952, 97897},
  {0x16953, 96300},
  {0x16954, 97744},
  {0x16955, 97717},
  {0x16956, 95604},
  {0x16957, 97194},
  {0x16958, 97796},
  {0x16959, 97115},
  {0x1695A, 97346},
  {0x1695B, 96794},
  {0x1695C, 97770},
  {0x1695D, 97424},
  {0x1695E, 96324},
  {0x1695F, 95762},
  {0x16960, 97501},
  {0x16961, 98054},
  {0x16962, 97168},
  {0x16963, 95919},
  {0x16964, 97610},
  {0x16965, 95225},
  {0x16966, 99824},
  {0x16967, 101872},
  {0x16968, 99447},
  {0x16969, 99092},
  {0x1696A, 102053},
  {0x1696B, 99195},
  {0x1696C, 100917},
  {0x1696D, 101584},
  {0x1696E, 101324},
  {0x1696F, 100634},
  {0x16970, 99903},
  {0x16971, 101197},
  {0x16972, 101719},
  {0x16973, 100165},
  {0x16974, 98962},
  {0x16975, 102129},
  {0x16976, 101428},
  {0x16977, 100042},
  {0x16978, 100942},
  {0x16979, 101847},
  {0x1697A, 99015},
  {0x1697B, 99170},
  {0x1697C, 100579},
  {0x1697D, 100332},
  {0x1697E, 99345},
  {0x1697F, 100250},
  {0x16980, 100358},
  {0x16981, 100607},
  {0x16982, 101375},
  {0x16983, 100526},
  {0x16984, 100968},
  {0x16985, 101403},
  {0x16986, 100136},
  {0x16987, 99526},
  {0x16988, 98760},
  {0x16989, 100220},
  {0x1698A, 100439},
  {0x1698B, 99371},
  {0x1698C, 99221},
  {0x1698D, 99984},
  {0x1698E, 99877},
  {0x1698F, 101687},
  {0x16990, 101453},
  {0x16991, 101274},
  {0x16992, 99789},
  {0x16993, 100072},
  {0x16994, 102286},
  {0x16995, 98733},
  {0x16996, 101923},
  {0x16997, 102179},
  {0x16998, 101123},
  {0x16999, 101974},
  {0x1699A, 98861},
  {0x1699B, 101609},
  {0x1699C, 101819},
  {0x1699D, 102026},
  {0x1699E, 102078},
  {0x1699F, 99066},
  {0x169A0, 101173},
  {0x169A1, 101661},
  {0x169A2, 98650},
  {0x169A3, 99954},
  {0x169A4, 101478},
  {0x169A5, 99552},
  {0x169A6, 99421},
  {0x169A7, 100780},
  {0x169A8, 100553},
  {0x169A9, 98597},
  {0x169AA, 101949},
  {0x169AB, 98989},
  {0x169AC, 102154},
  {0x169AD, 100747},
  {0x169AE, 102312},
  {0x169AF, 99499},
  {0x169B0, 99850},
  {0x169B1, 101070},
  {0x169B2, 102429},
  {0x169B3, 99142},
  {0x169B4, 98546},
  {0x169B5, 98623},
  {0x169B6, 100305},
  {0x169B7, 98836},
  {0x169B8, 99320},
  {0x169B9, 100191},
  {0x169BA, 98679},
  {0x169BB, 98361},
  {0x169BC, 100688},
  {0x169BD, 100994},
  {0x169BE, 102234},
  {0x169BF, 100013},
  {0x169C0, 100714},
  {0x169C1, 100806},
  {0x169C2, 101147},
  {0x169C3, 99603},
  {0x169C4, 98571},
  {0x169C5, 99729},
  {0x169C6, 99654},
  {0x169C7, 98705},
  {0x169C8, 98489},
  {0x169C9, 99680},
  {0x169CA, 101223},
  {0x169CB, 100278},
  {0x169CC, 99041},
  {0x169CD, 101097},
  {0x169CE, 100662},
  {0x169CF, 102206},
  {0x169D0, 100109},
  {0x169D1, 100855},
  {0x169D2, 98460},
  {0x169D3, 98435},
  {0x169D4, 100468},
  {0x169D5, 98338},
  {0x169D6, 101898},
  {0x169D7, 100832},
  {0x169D8, 98813},
  {0x169D9, 99117},
  {0x169DA, 100886},
  {0x169DB, 101636},
  {0x169DC, 101744},
  {0x169DD, 99754},
  {0x169DE, 101507},
  {0x169DF, 102260},
  {0x169E0, 100384},
  {0x169E1, 102402},
  {0x169E2, 102368},
  {0x169E3, 102337},
  {0x169E4, 102480},
  {0x169E5, 102455},
  {0x169E6, 98888},
  {0x169E7, 101020},
  {0x169E8, 100494},
  {0x169E9, 101249},
  {0x169EA, 102104},
  {0x169EB, 98410},
  {0x169EC, 102001},
  {0x169ED, 99272},
  {0x169EE, 99579},
  {0x169EF, 101349},
  {0x169F0, 101299},
  {0x169F1, 100411},
  {0x169F2, 99929},
  {0x169F3, 101533},
  {0x169F4, 101558},
  {0x169F5, 98515},
  {0x169F6, 99705},
  {0x169F7, 101046},
  {0x169F8, 99247},
  {0x169F9, 98938},
  {0x169FA, 99474},
  {0x169FB, 99296},
  {0x169FC, 99396},
  {0x169FD, 101794},
  {0x169FE, 98914},
  {0x169FF, 99629},
  {0x16A00, 101769},
  {0x16A01, 98385},
  {0x16A02, 98787},
  {0x16A03, 102554},
  {0x16A04, 103717},
  {0x16A05, 102677},
  {0x16A06, 102505},
  {0x16A07, 103313},
  {0x16A08, 103667},
  {0x16A09, 103211},
  {0x16A0A, 102727},
  {0x16A0B, 103415},
  {0x16A0C, 103389},
  {0x16A0D, 102899},
  {0x16A0E, 103642},
  {0x16A0F, 102950},
  {0x16A10, 103055},
  {0x16A11, 102800},
  {0x16A12, 103617},
  {0x16A13, 103517},
  {0x16A14, 103593},
  {0x16A15, 103491},
  {0x16A16, 102701},
  {0x16A17, 102603},
  {0x16A18, 103185},
  {0x16A19, 103161},
  {0x16A1A, 103082},
  {0x16A1B, 103839},
  {0x16A1C, 103568},
  {0x16A1D, 103815},
  {0x16A1E, 103109},
  {0x16A1F, 103261},
  {0x16A20, 103135},
  {0x16A21, 103790},
  {0x16A22, 103236},
  {0x16A23, 103440},
  {0x16A24, 103031},
  {0x16A25, 103363},
  {0x16A26, 102578},
  {0x16A27, 103002},
  {0x16A28, 102976},
  {0x16A29, 103543},
  {0x16A2A, 103287},
  {0x16A2B, 102529},
  {0x16A2C, 103766},
  {0x16A2D, 102751},
  {0x16A2E, 102775},
  {0x16A2F, 102628},
  {0x16A30, 102873},
  {0x16A31, 103338},
  {0x16A32, 102823},
  {0x16A33, 102926},
  {0x16A34, 102847},
  {0x16A35, 103692},
  {0x16A36, 102652},
  {0x16A37, 103464},
  {0x16A38, 103740},
  {0x16A40, 743086},
  {0x16A41, 742943},
  {0x16A42, 743145},
  {0x16A43, 742928},
  {0x16A44, 742721},
  {0x16A45, 742750},
  {0x16A46, 742708},
  {0x16A47, 743028},
  {0x16A48, 742853},
  {0x16A49, 742821},
  {0x16A4A, 742764},
  {0x16A4B, 742735},
  {0x16A4C, 742836},
  {0x16A4D, 742986},
  {0x16A4E, 742912},
  {0x16A4F, 742958},
  {0x16A50, 743014},
  {0x16A51, 743000},
  {0x16A52, 742973},
  {0x16A53, 743057},
  {0x16A54, 743071},
  {0x16A55, 743115},
  {0x16A56, 742792},
  {0x16A57, 743131},
  {0x16A58, 742779},
  {0x16A59, 742869},
  {0x16A5A, 742897},
  {0x16A5B, 742883},
  {0x16A5C, 742806},
  {0x16A5D, 743043},
  {0x16A5E, 743100},
  {0x16A60, 742676},
  {0x16A61, 742602},
  {0x16A62, 742662},
  {0x16A63, 742646},
  {0x16A64, 742572},
  {0x16A65, 742557},
  {0x16A66, 742632},
  {0x16A67, 742616},
  {0x16A68, 742541},
  {0x16A69, 742587},
  {0x16A6E, 742531},
  {0x16A6F, 742691},
  {0x16A70, 929154},
  {0x16A71, 929103},
  {0x16A72, 929120},
  {0x16A73, 929137},
  {0x16A74, 928503},
  {0x16A75, 928380},
  {0x16A76, 928397},
  {0x16A77, 928486},
  {0x16A78, 929698},
  {0x16A79, 929647},
  {0x16A7A, 929664},
  {0x16A7B, 929681},
  {0x16A7C, 928658},
  {0x16A7D, 928607},
  {0x16A7E, 928624},
  {0x16A7F, 928641},
  {0x16A80, 928855},
  {0x16A81, 928804},
  {0x16A82, 928821},
  {0x16A83, 928838},
  {0x16A84, 929630},
  {0x16A85, 929359},
  {0x16A86, 929596},
  {0x16A87, 929613},
  {0x16A88, 928468},
  {0x16A89, 928414},
  {0x16A8A, 928432},
  {0x16A8B, 928450},
  {0x16A8C, 929578},
  {0x16A8D, 929448},
  {0x16A8E, 929466},
  {0x16A8F, 929560},
  {0x16A90, 928692},
  {0x16A91, 928924},
  {0x16A92, 929282},
  {0x16A93, 929258},
  {0x16A94, 929376},
  {0x16A95, 929430},
  {0x16A96, 929394},
  {0x16A97, 929412},
  {0x16A98, 929541},
  {0x16A99, 929484},
  {0x16A9A, 929503},
  {0x16A9B, 929522},
  {0x16A9C, 929015},
  {0x16A9D, 928964},
  {0x16A9E, 928981},
  {0x16A9F, 928998},
  {0x16AA0, 928872},
  {0x16AA1, 928889},
  {0x16AA2, 928715},
  {0x16AA3, 929049},
  {0x16AA4, 929223},
  {0x16AA5, 929749},
  {0x16AA6, 929715},
  {0x16AA7, 929171},
  {0x16AA8, 929085},
  {0x16AA9, 929188},
  {0x16AAA, 928520},
  {0x16AAB, 928947},
  {0x16AAC, 929032},
  {0x16AAD, 928750},
  {0x16AAE, 928907},
  {0x16AAF, 928767},
  {0x16AB0, 929306},
  {0x16AB1, 928572},
  {0x16AB2, 929206},
  {0x16AB3, 929067},
  {0x16AB4, 929240},
  {0x16AB5, 928537},
  {0x16AB6, 929341},
  {0x16AB7, 928732},
  {0x16AB8, 928785},
  {0x16AB9, 929323},
  {0x16ABA, 929732},
  {0x16ABB, 928675},
  {0x16ABC, 928589},
  {0x16ABD, 928554},
  {0x16ABE, 929766},
  {0x16AC0, 928362},
  {0x16AC1, 928273},
  {0x16AC2, 928345},
  {0x16AC3, 928326},
  {0x16AC4, 928237},
  {0x16AC5, 928219},
  {0x16AC6, 928309},
  {0x16AC7, 928290},
  {0x16AC8, 928200},
  {0x16AC9, 928255},
  {0x16AD0, 104909},
  {0x16AD1, 105054},
  {0x16AD2, 105176},
  {0x16AD3, 104931},
  {0x16AD4, 105096},
  {0x16AD5, 105341},
  {0x16AD6, 104951},
  {0x16AD7, 104828},
  {0x16AD8, 105074},
  {0x16AD9, 105034},
  {0x16ADA, 104993},
  {0x16ADB, 105276},
  {0x16ADC, 105362},
  {0x16ADD, 104972},
  {0x16ADE, 104850},
  {0x16ADF, 104808},
  {0x16AE0, 105235},
  {0x16AE1, 105196},
  {0x16AE2, 104788},
  {0x16AE3, 105256},
  {0x16AE4, 105319},
  {0x16AE5, 105156},
  {0x16AE6, 105296},
  {0x16AE7, 104769},
  {0x16AE8, 105117},
  {0x16AE9, 105136},
  {0x16AEA, 105216},
  {0x16AEB, 104889},
  {0x16AEC, 104870},
  {0x16AED, 105015},
  {0x16AF0, 104594},
  {0x16AF1, 104658},
  {0x16AF2, 104720},
  {0x16AF3, 104687},
  {0x16AF4, 104624},
  {0x16AF5, 104749},
  {0x16B00, 811653},
  {0x16B01, 811677},
  {0x16B02, 811772},
  {0x16B03, 811795},
  {0x16B04, 811511},
  {0x16B05, 811535},
  {0x16B06, 811960},
  {0x16B07, 811983},
  {0x16B08, 811630},
  {0x16B09, 811701},
  {0x16B0A, 811463},
  {0x16B0B, 811487},
  {0x16B0C, 811841},
  {0x16B0D, 811865},
  {0x16B0E, 811582},
  {0x16B0F, 811606},
  {0x16B10, 811912},
  {0x16B11, 811936},
  {0x16B12, 811818},
  {0x16B13, 811889},
  {0x16B14, 811724},
  {0x16B15, 811748},
  {0x16B16, 811440},
  {0x16B17, 811559},
  {0x16B18, 812006},
  {0x16B19, 812029},
  {0x16B1A, 811392},
  {0x16B1B, 811416},
  {0x16B1C, 809685},
  {0x16B1D, 809572},
  {0x16B1E, 809377},
  {0x16B1F, 809294},
  {0x16B20, 809515},
  {0x16B21, 809658},
  {0x16B22, 809487},
  {0x16B23, 809630},
  {0x16B24, 809739},
  {0x16B25, 809321},
  {0x16B26, 809404},
  {0x16B27, 809266},
  {0x16B28, 809458},
  {0x16B29, 809349},
  {0x16B2A, 809601},
  {0x16B2B, 809543},
  {0x16B2C, 809431},
  {0x16B2D, 809213},
  {0x16B2E, 809712},
  {0x16B2F, 809239},
  {0x16B30, 810164},
  {0x16B31, 810085},
  {0x16B32, 810032},
  {0x16B33, 810058},
  {0x16B34, 810110},
  {0x16B35, 810006},
  {0x16B36, 810137},
  {0x16B37, 811121},
  {0x16B38, 811148},
  {0x16B39, 810428},
  {0x16B3A, 811093},
  {0x16B3B, 810986},
  {0x16B3C, 811256},
  {0x16B3D, 811286},
  {0x16B3E, 811314},
  {0x16B3F, 811227},
  {0x16B40, 811066},
  {0x16B41, 810775},
  {0x16B42, 811039},
  {0x16B43, 810728},
  {0x16B44, 811181},
  {0x16B45, 810593},
  {0x16B50, 809982},
  {0x16B51, 809863},
  {0x16B52, 809959},
  {0x16B53, 809934},
  {0x16B54, 809815},
  {0x16B55, 809791},
  {0x16B56, 809911},
  {0x16B57, 809886},
  {0x16B58, 809766},
  {0x16B59, 809839},
  {0x16B5B, 810352},
  {0x16B5C, 810227},
  {0x16B5D, 810318},
  {0x16B5E, 810256},
  {0x16B5F, 810190},
  {0x16B60, 810285},
  {0x16B61, 810377},
  {0x16B63, 811013},
  {0x16B64, 811342},
  {0x16B65, 810683},
  {0x16B66, 810922},
  {0x16B67, 811365},
  {0x16B68, 810705},
  {0x16B69, 810854},
  {0x16B6A, 811204},
  {0x16B6B, 810877},
  {0x16B6C, 810407},
  {0x16B6D, 810956},
  {0x16B6E, 810803},
  {0x16B6F, 810900},
  {0x16B70, 810753},
  {0x16B71, 810832},
  {0x16B72, 810490},
  {0x16B73, 810456},
  {0x16B74, 810624},
  {0x16B75, 810651},
  {0x16B76, 810562},
  {0x16B77, 810531},
  {0x16B7D, 809017},
  {0x16B7E, 809185},
  {0x16B7F, 808878},
  {0x16B80, 808850},
  {0x16B81, 809129},
  {0x16B82, 808796},
  {0x16B83, 808740},
  {0x16B84, 808905},
  {0x16B85, 808961},
  {0x16B86, 808989},
  {0x16B87, 808933},
  {0x16B88, 808768},
  {0x16B89, 808713},
  {0x16B8A, 809075},
  {0x16B8B, 808686},
  {0x16B8C, 809158},
  {0x16B8D, 809047},
  {0x16B8E, 808823},
  {0x16B8F, 809102},
  {0x16D40, 577796},
  {0x16D41, 577840},
  {0x16D42, 577883},
  {0x16D43, 577140},
  {0x16D44, 577427},
  {0x16D45, 577447},
  {0x16D46, 577325},
  {0x16D47, 577345},
  {0x16D48, 577528},
  {0x16D49, 577200},
  {0x16D4A, 577220},
  {0x16D4B, 577386},
  {0x16D4C, 577406},
  {0x16D4D, 577549},
  {0x16D4E, 577713},
  {0x16D4F, 577734},
  {0x16D50, 577261},
  {0x16D51, 577282},
  {0x16D52, 577672},
  {0x16D53, 577692},
  {0x16D54, 577241},
  {0x16D55, 577304},
  {0x16D56, 577508},
  {0x16D57, 577570},
  {0x16D58, 577590},
  {0x16D59, 577159},
  {0x16D5A, 577179},
  {0x16D5B, 577488},
  {0x16D5C, 577776},
  {0x16D5D, 577611},
  {0x16D5E, 577468},
  {0x16D5F, 577756},
  {0x16D60, 577631},
  {0x16D61, 577651},
  {0x16D62, 577366},
  {0x16D63, 577926},
  {0x16D64, 578021},
  {0x16D65, 578067},
  {0x16D66, 578090},
  {0x16D67, 577998},
  {0x16D68, 577950},
  {0x16D69, 578044},
  {0x16D6A, 577974},
  {0x16D6B, 577861},
  {0x16D6C, 577820},
  {0x16D6D, 577906},
  {0x16D6E, 576891},
  {0x16D6F, 577117},
  {0x16D70, 577096},
  {0x16D71, 576992},
  {0x16D72, 577076},
  {0x16D73, 577054},
  {0x16D74, 576950},
  {0x16D75, 576929},
  {0x16D76, 577034},
  {0x16D77, 577012},
  {0x16D78, 576907},
  {0x16D79, 576971},
  {0x16E40, 705994},
  {0x16E41, 706228},
  {0x16E42, 706315},
  {0x16E43, 706344},
  {0x16E44, 705612},
  {0x16E45, 706461},
  {0x16E46, 705935},
  {0x16E47, 706141},
  {0x16E48, 706257},
  {0x16E49, 705789},
  {0x16E4A, 705760},
  {0x16E4B, 705848},
  {0x16E4C, 705906},
  {0x16E4D, 705553},
  {0x16E4E, 705877},
  {0x16E4F, 705731},
  {0x16E50, 705644},
  {0x16E51, 705673},
  {0x16E52, 706286},
  {0x16E53, 706431},
  {0x16E54, 705965},
  {0x16E55, 706170},
  {0x16E56, 705818},
  {0x16E57, 706052},
  {0x16E58, 706373},
  {0x16E59, 705702},
  {0x16E5A, 706111},
  {0x16E5B, 706023},
  {0x16E5C, 706199},
  {0x16E5D, 706082},
  {0x16E5E, 705582},
  {0x16E5F, 706402},
  {0x16E60, 707581},
  {0x16E61, 707799},
  {0x16E62, 707880},
  {0x16E63, 707907},
  {0x16E64, 707225},
  {0x16E65, 708016},
  {0x16E66, 707526},
  {0x16E67, 707718},
  {0x16E68, 707826},
  {0x16E69, 707390},
  {0x16E6A, 707363},
  {0x16E6B, 707445},
  {0x16E6C, 707499},
  {0x16E6D, 707170},
  {0x16E6E, 707472},
  {0x16E6F, 707336},
  {0x16E70, 707255},
  {0x16E71, 707282},
  {0x16E72, 707853},
  {0x16E73, 707988},
  {0x16E74, 707554},
  {0x16E75, 707745},
  {0x16E76, 707417},
  {0x16E77, 707635},
  {0x16E78, 707934},
  {0x16E79, 707309},
  {0x16E7A, 707690},
  {0x16E7B, 707608},
  {0x16E7C, 707772},
  {0x16E7D, 707663},
  {0x16E7E, 707197},
  {0x16E7F, 707961},
  {0x16E80, 706828},
  {0x16E81, 706601},
  {0x16E82, 706769},
  {0x16E83, 706706},
  {0x16E84, 706555},
  {0x16E85, 706532},
  {0x16E86, 706684},
  {0x16E87, 706660},
  {0x16E88, 706508},
  {0x16E89, 706578},
  {0x16E8A, 707093},
  {0x16E8B, 706928},
  {0x16E8C, 707144},
  {0x16E8D, 707116},
  {0x16E8E, 706981},
  {0x16E8F, 706954},
  {0x16E90, 707066},
  {0x16E91, 707037},
  {0x16E92, 706900},
  {0x16E93, 707009},
  {0x16E94, 706623},
  {0x16E95, 706791},
  {0x16E96, 706730},
  {0x16E97, 706490},
  {0x16E98, 706878},
  {0x16E99, 708043},
  {0x16E9A, 706851},
  {0x16F00, 722850},
  {0x16F01, 722327},
  {0x16F02, 723583},
  {0x16F03, 722865},
  {0x16F04, 722664},
  {0x16F05, 722679},
  {0x16F06, 722233},
  {0x16F07, 722491},
  {0x16F08, 723465},
  {0x16F09, 723480},
  {0x16F0A, 723185},
  {0x16F0B, 722358},
  {0x16F0C, 723658},
  {0x16F0D, 723601},
  {0x16F0E, 723449},
  {0x16F0F, 722373},
  {0x16F10, 722695},
  {0x16F11, 722768},
  {0x16F12, 723564},
  {0x16F13, 722256},
  {0x16F14, 722784},
  {0x16F15, 722800},
  {0x16F16, 722600},
  {0x16F17, 722648},
  {0x16F18, 722615},
  {0x16F19, 722631},
  {0x16F1A, 723215},
  {0x16F1B, 722389},
  {0x16F1C, 723232},
  {0x16F1D, 722406},
  {0x16F1E, 722585},
  {0x16F1F, 722506},
  {0x16F20, 723546},
  {0x16F21, 722881},
  {0x16F22, 722896},
  {0x16F23, 722735},
  {0x16F24, 722751},
  {0x16F25, 722279},
  {0x16F26, 722554},
  {0x16F27, 723511},
  {0x16F28, 722521},
  {0x16F29, 722537},
  {0x16F2A, 723432},
  {0x16F2B, 722474},
  {0x16F2C, 722817},
  {0x16F2D, 722833},
  {0x16F2E, 723415},
  {0x16F2F, 722440},
  {0x16F30, 723638},
  {0x16F31, 723526},
  {0x16F32, 723080},
  {0x16F33, 723137},
  {0x16F34, 723153},
  {0x16F35, 723692},
  {0x16F36, 723724},
  {0x16F37, 723383},
  {0x16F38, 722424},
  {0x16F39, 723619},
  {0x16F3A, 723122},
  {0x16F3B, 723677},
  {0x16F3C, 723708},
  {0x16F3D, 723741},
  {0x16F3E, 723757},
  {0x16F3F, 722303},
  {0x16F40, 723792},
  {0x16F41, 723774},
  {0x16F42, 723496},
  {0x16F43, 722218},
  {0x16F44, 722569},
  {0x16F45, 722342},
  {0x16F46, 723169},
  {0x16F47, 722457},
  {0x16F48, 723200},
  {0x16F49, 723399},
  {0x16F4A, 723106},
  {0x16F4F, 723830},
  {0x16F50, 722710},
  {0x16F51, 723809},
  {0x16F52, 723893},
  {0x16F53, 723863},
  {0x16F54, 723988},
  {0x16F55, 724006},
  {0x16F56, 724064},
  {0x16F57, 724103},
  {0x16F58, 724122},
  {0x16F59, 724546},
  {0x16F5A, 724623},
  {0x16F5B, 724929},
  {0x16F5C, 724911},
  {0x16F5D, 724161},
  {0x16F5E, 724217},
  {0x16F5F, 724236},
  {0x16F60, 724584},
  {0x16F61, 724295},
  {0x16F62, 724313},
  {0x16F63, 724332},
  {0x16F64, 724352},
  {0x16F65, 724450},
  {0x16F66, 724373},
  {0x16F67, 724411},
  {0x16F68, 724490},
  {0x16F69, 724430},
  {0x16F6A, 724716},
  {0x16F6B, 724734},
  {0x16F6C, 724753},
  {0x16F6D, 724773},
  {0x16F6E, 724873},
  {0x16F6F, 724794},
  {0x16F70, 724833},
  {0x16F71, 724948},
  {0x16F72, 724966},
  {0x16F73, 724025},
  {0x16F74, 724044},
  {0x16F75, 724275},
  {0x16F76, 724688},
  {0x16F77, 724256},
  {0x16F78, 724661},
  {0x16F79, 724084},
  {0x16F7A, 724198},
  {0x16F7B, 724142},
  {0x16F7C, 724642},
  {0x16F7D, 724509},
  {0x16F7E, 724527},
  {0x16F7F, 724853},
  {0x16F80, 724985},
  {0x16F81, 724604},
  {0x16F82, 724564},
  {0x16F83, 724892},
  {0x16F84, 724392},
  {0x16F85, 724179},
  {0x16F86, 724469},
  {0x16F87, 724814},
  {0x16F8F, 723952},
  {0x16F90, 723968},
  {0x16F91, 723920},
  {0x16F92, 723936},
  {0x16F93, 723250},
  {0x16F94, 723269},
  {0x16F95, 723288},
  {0x16F96, 723307},
  {0x16F97, 723326},
  {0x16F98, 723345},
  {0x16F99, 723364},
  {0x16F9A, 722912},
  {0x16F9B, 722940},
  {0x16F9C, 722968},
  {0x16F9D, 722996},
  {0x16F9E, 723024},
  {0x16F9F, 723052},
  {0x16FE0, 929783},
  {0x16FE1, 782907},
  {0x16FE2, 788069},
  {0x16FE3, 788091},
  {0x16FE4, 571018},
  {0x16FF0, 976065},
  {0x16FF1, 976102},
  {0x18B00, 554062},
  {0x18B01, 554098},
  {0x18B02, 554134},
  {0x18B03, 554170},
  {0x18B04, 554206},
  {0x18B05, 554242},
  {0x18B06, 554278},
  {0x18B07, 554314},
  {0x18B08, 554350},
  {0x18B09, 554386},
  {0x18B0A, 554422},
  {0x18B0B, 554458},
  {0x18B0C, 554494},
  {0x18B0D, 554530},
  {0x18B0E, 554566},
  {0x18B0F, 554602},
  {0x18B10, 554638},
  {0x18B11, 554674},
  {0x18B12, 554710},
  {0x18B13, 554746},
  {0x18B14, 554782},
  {0x18B15, 554818},
  {0x18B16, 554854},
  {0x18B17, 554890},
  {0x18B18, 554926},
  {0x18B19, 554962},
  {0x18B1A, 554998},
  {0x18B1B, 555034},
  {0x18B1C, 555070},
  {0x18B1D, 555106},
  {0x18B1E, 555142},
  {0x18B1F, 555178},
  {0x18B20, 555214},
  {0x18B21, 555250},
  {0x18B22, 555286},
  {0x18B23, 555322},
  {0x18B24, 555358},
  {0x18B25, 555394},
  {0x18B26, 555430},
  {0x18B27, 555466},
  {0x18B28, 555502},
  {0x18B29, 555538},
  {0x18B2A, 555574},
  {0x18B2B, 555610},
  {0x18B2C, 555646},
  {0x18B2D, 555682},
  {0x18B2E, 555718},
  {0x18B2F, 555754},
  {0x18B30, 555790},
  {0x18B31, 555826},
  {0x18B32, 555862},
  {0x18B33, 555898},
  {0x18B34, 555934},
  {0x18B35, 555970},
  {0x18B36, 556006},
  {0x18B37, 556042},
  {0x18B38, 556078},
  {0x18B39, 556114},
  {0x18B3A, 556150},
  {0x18B3B, 556186},
  {0x18B3C, 556222},
  {0x18B3D, 556258},
  {0x18B3E, 556294},
  {0x18B3F, 556330},
  {0x18B40, 556366},
  {0x18B41, 556402},
  {0x18B42, 556438},
  {0x18B43, 556474},
  {0x18B44, 556510},
  {0x18B45, 556546},
  {0x18B46, 556582},
  {0x18B47, 556618},
  {0x18B48, 556654},
  {0x18B49, 556690},
  {0x18B4A, 556726},
  {0x18B4B, 556762},
  {0x18B4C, 556798},
  {0x18B4D, 556834},
  {0x18B4E, 556870},
  {0x18B4F, 556906},
  {0x18B50, 556942},
  {0x18B51, 556978},
  {0x18B52, 557014},
  {0x18B53, 557050},
  {0x18B54, 557086},
  {0x18B55, 557122},
  {0x18B56, 557158},
  {0x18B57, 557194},
  {0x18B58, 557230},
  {0x18B59, 557266},
  {0x18B5A, 557302},
  {0x18B5B, 557338},
  {0x18B5C, 557374},
  {0x18B5D, 557410},
  {0x18B5E, 557446},
  {0x18B5F, 557482},
  {0x18B60, 557518},
  {0x18B61, 557554},
  {0x18B62, 557590},
  {0x18B63, 557626},
  {0x18B64, 557662},
  {0x18B65, 557698},
  {0x18B66, 557734},
  {0x18B67, 557770},
  {0x18B68, 557806},
  {0x18B69, 557842},
  {0x18B6A, 557878},
  {0x18B6B, 557914},
  {0x18B6C, 557950},
  {0x18B6D, 557986},
  {0x18B6E, 558022},
  {0x18B6F, 558058},
  {0x18B70, 558094},
  {0x18B71, 558130},
  {0x18B72, 558166},
  {0x18B73, 558202},
  {0x18B74, 558238},
  {0x18B75, 558274},
  {0x18B76, 558310},
  {0x18B77, 558346},
  {0x18B78, 558382},
  {0x18B79, 558418},
  {0x18B7A, 558454},
  {0x18B7B, 558490},
  {0x18B7C, 558526},
  {0x18B7D, 558562},
  {0x18B7E, 558598},
  {0x18B7F, 558634},
  {0x18B80, 558670},
  {0x18B81, 558706},
  {0x18B82, 558742},
  {0x18B83, 558778},
  {0x18B84, 558814},
  {0x18B85, 558850},
  {0x18B86, 558886},
  {0x18B87, 558922},
  {0x18B88, 558958},
  {0x18B89, 558994},
  {0x18B8A, 559030},
  {0x18B8B, 559066},
  {0x18B8C, 559102},
  {0x18B8D, 559138},
  {0x18B8E, 559174},
  {0x18B8F, 559210},
  {0x18B90, 559246},
  {0x18B91, 559282},
  {0x18B92, 559318},
  {0x18B93, 559354},
  {0x18B94, 559390},
  {0x18B95, 559426},
  {0x18B96, 559462},
  {0x18B97, 559498},
  {0x18B98, 559534},
  {0x18B99, 559570},
  {0x18B9A, 559606},
  {0x18B9B, 559642},
  {0x18B9C, 559678},
  {0x18B9D, 559714},
  {0x18B9E, 559750},
  {0x18B9F, 559786},
  {0x18BA0, 559822},
  {0x18BA1, 559858},
  {0x18BA2, 559894},
  {0x18BA3, 559930},
  {0x18BA4, 559966},
  {0x18BA5, 560002},
  {0x18BA6, 560038},
  {0x18BA7, 560074},
  {0x18BA8, 560110},
  {0x18BA9, 560146},
  {0x18BAA, 560182},
  {0x18BAB, 560218},
  {0x18BAC, 560254},
  {0x18BAD, 560290},
  {0x18BAE, 560326},
  {0x18BAF, 560362},
  {0x18BB0, 560398},
  {0x18BB1, 560434},
  {0x18BB2, 560470},
  {0x18BB3, 560506},
  {0x18BB4, 560542},
  {0x18BB5, 560578},
  {0x18BB6, 560614},
  {0x18BB7, 560650},
  {0x18BB8, 560686},
  {0x18BB9, 560722},
  {0x18BBA, 560758},
  {0x18BBB, 560794},
  {0x18BBC, 560830},
  {0x18BBD, 560866},
  {0x18BBE, 560902},
  {0x18BBF, 560938},
  {0x18BC0, 560974},
  {0x18BC1, 561010},
  {0x18BC2, 561046},
  {0x18BC3, 561082},
  {0x18BC4, 561118},
  {0x18BC5, 561154},
  {0x18BC6, 561190},
  {0x18BC7, 561226},
  {0x18BC8, 561262},
  {0x18BC9, 561298},
  {0x18BCA, 561334},
  {0x18BCB, 561370},
  {0x18BCC, 561406},
  {0x18BCD, 561442},
  {0x18BCE, 561478},
  {0x18BCF, 561514},
  {0x18BD0, 561550},
  {0x18BD1, 561586},
  {0x18BD2, 561622},
  {0x18BD3, 561658},
  {0x18BD4, 561694},
  {0x18BD5, 561730},
  {0x18BD6, 561766},
  {0x18BD7, 561802},
  {0x18BD8, 561838},
  {0x18BD9, 561874},
  {0x18BDA, 561910},
  {0x18BDB, 561946},
  {0x18BDC, 561982},
  {0x18BDD, 562018},
  {0x18BDE, 562054},
  {0x18BDF, 562090},
  {0x18BE0, 562126},
  {0x18BE1, 562162},
  {0x18BE2, 562198},
  {0x18BE3, 562234},
  {0x18BE4, 562270},
  {0x18BE5, 562306},
  {0x18BE6, 562342},
  {0x18BE7, 562378},
  {0x18BE8, 562414},
  {0x18BE9, 562450},
  {0x18BEA, 562486},
  {0x18BEB, 562522},
  {0x18BEC, 562558},
  {0x18BED, 562594},
  {0x18BEE, 562630},
  {0x18BEF, 562666},
  {0x18BF0, 562702},
  {0x18BF1, 562738},
  {0x18BF2, 562774},
  {0x18BF3, 562810},
  {0x18BF4, 562846},
  {0x18BF5, 562882},
  {0x18BF6, 562918},
  {0x18BF7, 562954},
  {0x18BF8, 562990},
  {0x18BF9, 563026},
  {0x18BFA, 563062},
  {0x18BFB, 563098},
  {0x18BFC, 563134},
  {0x18BFD, 563170},
  {0x18BFE, 563206},
  {0x18BFF, 563242},
  {0x18C00, 563278},
  {0x18C01, 563314},
  {0x18C02, 563350},
  {0x18C03, 563386},
  {0x18C04, 563422},
  {0x18C05, 563458},
  {0x18C06, 563494},
  {0x18C07, 563530},
  {0x18C08, 563566},
  {0x18C09, 563602},
  {0x18C0A, 563638},
  {0x18C0B, 563674},
  {0x18C0C, 563710},
  {0x18C0D, 563746},
  {0x18C0E, 563782},
  {0x18C0F, 563818},
  {0x18C10, 563854},
  {0x18C11, 563890},
  {0x18C12, 563926},
  {0x18C13, 563962},
  {0x18C14, 563998},
  {0x18C15, 564034},
  {0x18C16, 564070},
  {0x18C17, 564106},
  {0x18C18, 564142},
  {0x18C19, 564178},
  {0x18C1A, 564214},
  {0x18C1B, 564250},
  {0x18C1C, 564286},
  {0x18C1D, 564322},
  {0x18C1E, 564358},
  {0x18C1F, 564394},
  {0x18C20, 564430},
  {0x18C21, 564466},
  {0x18C22, 564502},
  {0x18C23, 564538},
  {0x18C24, 564574},
  {0x18C25, 564610},
  {0x18C26, 564646},
  {0x18C27, 564682},
  {0x18C28, 564718},
  {0x18C29, 564754},
  {0x18C2A, 564790},
  {0x18C2B, 564826},
  {0x18C2C, 564862},
  {0x18C2D, 564898},
  {0x18C2E, 564934},
  {0x18C2F, 564970},
  {0x18C30, 565006},
  {0x18C31, 565042},
  {0x18C32, 565078},
  {0x18C33, 565114},
  {0x18C34, 565150},
  {0x18C35, 565186},
  {0x18C36, 565222},
  {0x18C37, 565258},
  {0x18C38, 565294},
  {0x18C39, 565330},
  {0x18C3A, 565366},
  {0x18C3B, 565402},
  {0x18C3C, 565438},
  {0x18C3D, 565474},
  {0x18C3E, 565510},
  {0x18C3F, 565546},
  {0x18C40, 565582},
  {0x18C41, 565618},
  {0x18C42, 565654},
  {0x18C43, 565690},
  {0x18C44, 565726},
  {0x18C45, 565762},
  {0x18C46, 565798},
  {0x18C47, 565834},
  {0x18C48, 565870},
  {0x18C49, 565906},
  {0x18C4A, 565942},
  {0x18C4B, 565978},
  {0x18C4C, 566014},
  {0x18C4D, 566050},
  {0x18C4E, 566086},
  {0x18C4F, 566122},
  {0x18C50, 566158},
  {0x18C51, 566194},
  {0x18C52, 566230},
  {0x18C53, 566266},
  {0x18C54, 566302},
  {0x18C55, 566338},
  {0x18C56, 566374},
  {0x18C57, 566410},
  {0x18C58, 566446},
  {0x18C59, 566482},
  {0x18C5A, 566518},
  {0x18C5B, 566554},
  {0x18C5C, 566590},
  {0x18C5D, 566626},
  {0x18C5E, 566662},
  {0x18C5F, 566698},
  {0x18C60, 566734},
  {0x18C61, 566770},
  {0x18C62, 566806},
  {0x18C63, 566842},
  {0x18C64, 566878},
  {0x18C65, 566914},
  {0x18C66, 566950},
  {0x18C67, 566986},
  {0x18C68, 567022},
  {0x18C69, 567058},
  {0x18C6A, 567094},
  {0x18C6B, 567130},
  {0x18C6C, 567166},
  {0x18C6D, 567202},
  {0x18C6E, 567238},
  {0x18C6F, 567274},
  {0x18C70, 567310},
  {0x18C71, 567346},
  {0x18C72, 567382},
  {0x18C73, 567418},
  {0x18C74, 567454},
  {0x18C75, 567490},
  {0x18C76, 567526},
  {0x18C77, 567562},
  {0x18C78, 567598},
  {0x18C79, 567634},
  {0x18C7A, 567670},
  {0x18C7B, 567706},
  {0x18C7C, 567742},
  {0x18C7D, 567778},
  {0x18C7E, 567814},
  {0x18C7F, 567850},
  {0x18C80, 567886},
  {0x18C81, 567922},
  {0x18C82, 567958},
  {0x18C83, 567994},
  {0x18C84, 568030},
  {0x18C85, 568066},
  {0x18C86, 568102},
  {0x18C87, 568138},
  {0x18C88, 568174},
  {0x18C89, 568210},
  {0x18C8A, 568246},
  {0x18C8B, 568282},
  {0x18C8C, 568318},
  {0x18C8D, 568354},
  {0x18C8E, 568390},
  {0x18C8F, 568426},
  {0x18C90, 568462},
  {0x18C91, 568498},
  {0x18C92, 568534},
  {0x18C93, 568570},
  {0x18C94, 568606},
  {0x18C95, 568642},
  {0x18C96, 568678},
  {0x18C97, 568714},
  {0x18C98, 568750},
  {0x18C99, 568786},
  {0x18C9A, 568822},
  {0x18C9B, 568858},
  {0x18C9C, 568894},
  {0x18C9D, 568930},
  {0x18C9E, 568966},
  {0x18C9F, 569002},
  {0x18CA0, 569038},
  {0x18CA1, 569074},
  {0x18CA2, 569110},
  {0x18CA3, 569146},
  {0x18CA4, 569182},
  {0x18CA5, 569218},
  {0x18CA6, 569254},
  {0x18CA7, 569290},
  {0x18CA8, 569326},
  {0x18CA9, 569362},
  {0x18CAA, 569398},
  {0x18CAB, 569434},
  {0x18CAC, 569470},
  {0x18CAD, 569506},
  {0x18CAE, 569542},
  {0x18CAF, 569578},
  {0x18CB0, 569614},
  {0x18CB1, 569650},
  {0x18CB2, 569686},
  {0x18CB3, 569722},
  {0x18CB4, 569758},
  {0x18CB5, 569794},
  {0x18CB6, 569830},
  {0x18CB7, 569866},
  {0x18CB8, 569902},
  {0x18CB9, 569938},
  {0x18CBA, 569974},
  {0x18CBB, 570010},
  {0x18CBC, 570046},
  {0x18CBD, 570082},
  {0x18CBE, 570118},
  {0x18CBF, 570154},
  {0x18CC0, 570190},
  {0x18CC1, 570226},
  {0x18CC2, 570262},
  {0x18CC3, 570298},
  {0x18CC4, 570334},
  {0x18CC5, 570370},
  {0x18CC6, 570406},
  {0x18CC7, 570442},
  {0x18CC8, 570478},
  {0x18CC9, 570514},
  {0x18CCA, 570550},
  {0x18CCB, 570586},
  {0x18CCC, 570622},
  {0x18CCD, 570658},
  {0x18CCE, 570694},
  {0x18CCF, 570730},
  {0x18CD0, 570766},
  {0x18CD1, 570802},
  {0x18CD2, 570838},
  {0x18CD3, 570874},
  {0x18CD4, 570910},
  {0x18CD5, 570946},
  {0x18CFF, 570982},
  {0x1AFF0, 547752},
  {0x1AFF1, 547782},
  {0x1AFF2, 547812},
  {0x1AFF3, 547842},
  {0x1AFF5, 547872},
  {0x1AFF6, 547902},
  {0x1AFF7, 547472},
  {0x1AFF8, 547512},
  {0x1AFF9, 547552},
  {0x1AFFA, 547592},
  {0x1AFFB, 547632},
  {0x1AFFD, 547672},
  {0x1AFFE, 547712},
  {0x1B000, 546797},
  {0x1B001, 521398},
  {0x1B002, 512863},
  {0x1B003, 512885},
  {0x1B004, 512907},
  {0x1B005, 512929},
  {0x1B006, 513892},
  {0x1B007, 513914},
  {0x1B008, 513936},
  {0x1B009, 513958},
  {0x1B00A, 518458},
  {0x1B00B, 518480},
  {0x1B00C, 518502},
  {0x1B00D, 518524},
  {0x1B00E, 518546},
  {0x1B00F, 512952},
  {0x1B010, 512974},
  {0x1B011, 512996},
  {0x1B012, 513018},
  {0x1B013, 513040},
  {0x1B014, 516251},
  {0x1B015, 516273},
  {0x1B016, 516295},
  {0x1B017, 513980},
  {0x1B018, 514051},
  {0x1B019, 514074},
  {0x1B01A, 514097},
  {0x1B01B, 514120},
  {0x1B01C, 514143},
  {0x1B01D, 514166},
  {0x1B01E, 514189},
  {0x1B01F, 514212},
  {0x1B020, 514003},
  {0x1B021, 514027},
  {0x1B022, 514235},
  {0x1B023, 514397},
  {0x1B024, 514420},
  {0x1B025, 514443},
  {0x1B026, 514466},
  {0x1B027, 514489},
  {0x1B028, 514512},
  {0x1B029, 514535},
  {0x1B02A, 514558},
  {0x1B02B, 514674},
  {0x1B02C, 514697},
  {0x1B02D, 514720},
  {0x1B02E, 514743},
  {0x1B02F, 514766},
  {0x1B030, 514789},
  {0x1B031, 514812},
  {0x1B032, 514259},
  {0x1B033, 514282},
  {0x1B034, 514305},
  {0x1B035, 514328},
  {0x1B036, 514351},
  {0x1B037, 514374},
  {0x1B038, 514581},
  {0x1B039, 514604},
  {0x1B03A, 514627},
  {0x1B03B, 514650},
  {0x1B03C, 516938},
  {0x1B03D, 516961},
  {0x1B03E, 516984},
  {0x1B03F, 517007},
  {0x1B040, 517030},
  {0x1B041, 517053},
  {0x1B042, 517076},
  {0x1B043, 517099},
  {0x1B044, 517237},
  {0x1B045, 517260},
  {0x1B046, 517283},
  {0x1B047, 517306},
  {0x1B048, 517329},
  {0x1B049, 517352},
  {0x1B04A, 517536},
  {0x1B04B, 517559},
  {0x1B04C, 517582},
  {0x1B04D, 517605},
  {0x1B04E, 517628},
  {0x1B04F, 517651},
  {0x1B050, 517674},
  {0x1B051, 517697},
  {0x1B052, 517122},
  {0x1B053, 517145},
  {0x1B054, 517168},
  {0x1B055, 517191},
  {0x1B056, 517214},
  {0x1B057, 517375},
  {0x1B058, 517398},
  {0x1B059, 517421},
  {0x1B05A, 517444},
  {0x1B05B, 517467},
  {0x1B05C, 517490},
  {0x1B05D, 517513},
  {0x1B05E, 517720},
  {0x1B05F, 517743},
  {0x1B060, 517766},
  {0x1B061, 517789},
  {0x1B062, 518019},
  {0x1B063, 518042},
  {0x1B064, 518065},
  {0x1B065, 518088},
  {0x1B066, 518111},
  {0x1B067, 518134},
  {0x1B068, 518157},
  {0x1B069, 518342},
  {0x1B06A, 518365},
  {0x1B06B, 518388},
  {0x1B06C, 518411},
  {0x1B06D, 518434},
  {0x1B06E, 517812},
  {0x1B06F, 517835},
  {0x1B070, 517858},
  {0x1B071, 517881},
  {0x1B072, 517904},
  {0x1B073, 517927},
  {0x1B074, 517950},
  {0x1B075, 517973},
  {0x1B076, 517996},
  {0x1B077, 518180},
  {0x1B078, 518203},
  {0x1B079, 518226},
  {0x1B07A, 518249},
  {0x1B07B, 518272},
  {0x1B07C, 518295},
  {0x1B07D, 518318},
  {0x1B07E, 515513},
  {0x1B07F, 515536},
  {0x1B080, 515559},
  {0x1B081, 515582},
  {0x1B082, 515605},
  {0x1B083, 515628},
  {0x1B084, 515651},
  {0x1B085, 515674},
  {0x1B086, 515697},
  {0x1B087, 515882},
  {0x1B088, 515905},
  {0x1B089, 515928},
  {0x1B08A, 515951},
  {0x1B08B, 515974},
  {0x1B08C, 515997},
  {0x1B08D, 516020},
  {0x1B08E, 516043},
  {0x1B08F, 516182},
  {0x1B090, 516205},
  {0x1B091, 516228},
  {0x1B092, 515720},
  {0x1B093, 515743},
  {0x1B094, 515766},
  {0x1B095, 515789},
  {0x1B096, 515812},
  {0x1B097, 515835},
  {0x1B098, 515858},
  {0x1B099, 516067},
  {0x1B09A, 516090},
  {0x1B09B, 516113},
  {0x1B09C, 516136},
  {0x1B09D, 516159},
  {0x1B09E, 513062},
  {0x1B09F, 513133},
  {0x1B0A0, 513156},
  {0x1B0A1, 513179},
  {0x1B0A2, 513202},
  {0x1B0A3, 513225},
  {0x1B0A4, 513248},
  {0x1B0A5, 513271},
  {0x1B0A6, 513294},
  {0x1B0A7, 513085},
  {0x1B0A8, 513109},
  {0x1B0A9, 513478},
  {0x1B0AA, 513501},
  {0x1B0AB, 513524},
  {0x1B0AC, 513547},
  {0x1B0AD, 513570},
  {0x1B0AE, 513593},
  {0x1B0AF, 513616},
  {0x1B0B0, 513823},
  {0x1B0B1, 513846},
  {0x1B0B2, 513869},
  {0x1B0B3, 513317},
  {0x1B0B4, 513340},
  {0x1B0B5, 513363},
  {0x1B0B6, 513386},
  {0x1B0B7, 513409},
  {0x1B0B8, 513432},
  {0x1B0B9, 513455},
  {0x1B0BA, 513639},
  {0x1B0BB, 513662},
  {0x1B0BC, 513685},
  {0x1B0BD, 513708},
  {0x1B0BE, 513731},
  {0x1B0BF, 513754},
  {0x1B0C0, 513777},
  {0x1B0C1, 513800},
  {0x1B0C2, 514835},
  {0x1B0C3, 514858},
  {0x1B0C4, 514881},
  {0x1B0C5, 514904},
  {0x1B0C6, 514927},
  {0x1B0C7, 514950},
  {0x1B0C8, 514973},
  {0x1B0C9, 515066},
  {0x1B0CA, 515089},
  {0x1B0CB, 515112},
  {0x1B0CC, 515135},
  {0x1B0CD, 515158},
  {0x1B0CE, 515181},
  {0x1B0CF, 515204},
  {0x1B0D0, 515365},
  {0x1B0D1, 515388},
  {0x1B0D2, 515411},
  {0x1B0D3, 515434},
  {0x1B0D4, 514996},
  {0x1B0D5, 515019},
  {0x1B0D6, 515042},
  {0x1B0D7, 515227},
  {0x1B0D8, 515250},
  {0x1B0D9, 515273},
  {0x1B0DA, 515296},
  {0x1B0DB, 515319},
  {0x1B0DC, 515342},
  {0x1B0DD, 519051},
  {0x1B0DE, 519074},
  {0x1B0DF, 519097},
  {0x1B0E0, 519120},
  {0x1B0E1, 519143},
  {0x1B0E2, 519166},
  {0x1B0E3, 519328},
  {0x1B0E4, 519351},
  {0x1B0E5, 519374},
  {0x1B0E6, 519397},
  {0x1B0E7, 519190},
  {0x1B0E8, 519213},
  {0x1B0E9, 519236},
  {0x1B0EA, 519259},
  {0x1B0EB, 519282},
  {0x1B0EC, 519305},
  {0x1B0ED, 516317},
  {0x1B0EE, 516340},
  {0x1B0EF, 516363},
  {0x1B0F0, 516386},
  {0x1B0F1, 516501},
  {0x1B0F2, 516524},
  {0x1B0F3, 516547},
  {0x1B0F4, 516570},
  {0x1B0F5, 516593},
  {0x1B0F6, 516616},
  {0x1B0F7, 516639},
  {0x1B0F8, 516800},
  {0x1B0F9, 516823},
  {0x1B0FA, 516846},
  {0x1B0FB, 516869},
  {0x1B0FC, 516892},
  {0x1B0FD, 516915},
  {0x1B0FE, 516409},
  {0x1B0FF, 516432},
  {0x1B100, 516455},
  {0x1B101, 516478},
  {0x1B102, 516662},
  {0x1B103, 516685},
  {0x1B104, 516708},
  {0x1B105, 516731},
  {0x1B106, 516754},
  {0x1B107, 516777},
  {0x1B108, 518568},
  {0x1B109, 518591},
  {0x1B10A, 518614},
  {0x1B10B, 518637},
  {0x1B10C, 518660},
  {0x1B10D, 518775},
  {0x1B10E, 518798},
  {0x1B10F, 518821},
  {0x1B110, 518844},
  {0x1B111, 518867},
  {0x1B112, 518683},
  {0x1B113, 518706},
  {0x1B114, 518729},
  {0x1B115, 518752},
  {0x1B116, 518890},
  {0x1B117, 518913},
  {0x1B118, 518936},
  {0x1B119, 518959},
  {0x1B11A, 518982},
  {0x1B11B, 519005},
  {0x1B11C, 519028},
  {0x1B11D, 515457},
  {0x1B11E, 515485},
  {0x1B11F, 521371},
  {0x1B120, 546877},
  {0x1B121, 546850},
  {0x1B122, 546823},
  {0x1B132, 522531},
  {0x1B150, 522679},
  {0x1B151, 522654},
  {0x1B152, 522704},
  {0x1B155, 548595},
  {0x1B164, 549042},
  {0x1B165, 549017},
  {0x1B166, 549067},
  {0x1B167, 548670},
  {0x1B170, 774195},
  {0x1B171, 774217},
  {0x1B172, 774239},
  {0x1B173, 774261},
  {0x1B174, 774283},
  {0x1B175, 774305},
  {0x1B176, 774327},
  {0x1B177, 774349},
  {0x1B178, 774371},
  {0x1B179, 774393},
  {0x1B17A, 774415},
  {0x1B17B, 774437},
  {0x1B17C, 774459},
  {0x1B17D, 774481},
  {0x1B17E, 774503},
  {0x1B17F, 774525},
  {0x1B180, 774547},
  {0x1B181, 774569},
  {0x1B182, 774591},
  {0x1B183, 774613},
  {0x1B184, 774635},
  {0x1B185, 774657},
  {0x1B186, 774679},
  {0x1B187, 774701},
  {0x1B188, 774723},
  {0x1B189, 774745},
  {0x1B18A, 774767},
  {0x1B18B, 774789},
  {0x1B18C, 774811},
  {0x1B18D, 774833},
  {0x1B18E, 774855},
  {0x1B18F, 774877},
  {0x1B190, 774899},
  {0x1B191, 774921},
  {0x1B192, 774943},
  {0x1B193, 774965},
  {0x1B194, 774987},
  {0x1B195, 775009},
  {0x1B196, 775031},
  {0x1B197, 775053},
  {0x1B198, 775075},
  {0x1B199, 775097},
  {0x1B19A, 775119},
  {0x1B19B, 775141},
  {0x1B19C, 775163},
  {0x1B19D, 775185},
  {0x1B19E, 775207},
  {0x1B19F, 775229},
  {0x1B1A0, 775251},
  {0x1B1A1, 775273},
  {0x1B1A2, 775295},
  {0x1B1A3, 775317},
  {0x1B1A4, 775339},
  {0x1B1A5, 775361},
  {0x1B1A6, 775383},
  {0x1B1A7, 775405},
  {0x1B1A8, 775427},
  {0x1B1A9, 775449},
  {0x1B1AA, 775471},
  {0x1B1AB, 775493},
  {0x1B1AC, 775515},
  {0x1B1AD, 775537},
  {0x1B1AE, 775559},
  {0x1B1AF, 775581},
  {0x1B1B0, 775603},
  {0x1B1B1, 775625},
  {0x1B1B2, 775647},
  {0x1B1B3, 775669},
  {0x1B1B4, 775691},
  {0x1B1B5, 775713},
  {0x1B1B6, 775735},
  {0x1B1B7, 775757},
  {0x1B1B8, 775779},
  {0x1B1B9, 775801},
  {0x1B1BA, 775823},
  {0x1B1BB, 775845},
  {0x1B1BC, 775867},
  {0x1B1BD, 775889},
  {0x1B1BE, 775911},
  {0x1B1BF, 775933},
  {0x1B1C0, 775955},
  {0x1B1C1, 775977},
  {0x1B1C2, 775999},
  {0x1B1C3, 776021},
  {0x1B1C4, 776043},
  {0x1B1C5, 776065},
  {0x1B1C6, 776087},
  {0x1B1C7, 776109},
  {0x1B1C8, 776131},
  {0x1B1C9, 776153},
  {0x1B1CA, 776175},
  {0x1B1CB, 776197},
  {0x1B1CC, 776219},
  {0x1B1CD, 776241},
  {0x1B1CE, 776263},
  {0x1B1CF, 776285},
  {0x1B1D0, 776307},
  {0x1B1D1, 776329},
  {0x1B1D2, 776351},
  {0x1B1D3, 776373},
  {0x1B1D4, 776395},
  {0x1B1D5, 776417},
  {0x1B1D6, 776439},
  {0x1B1D7, 776461},
  {0x1B1D8, 776483},
  {0x1B1D9, 776505},
  {0x1B1DA, 776527},
  {0x1B1DB, 776549},
  {0x1B1DC, 776571},
  {0x1B1DD, 776593},
  {0x1B1DE, 776615},
  {0x1B1DF, 776637},
  {0x1B1E0, 776659},
  {0x1B1E1, 776681},
  {0x1B1E2, 776703},
  {0x1B1E3, 776725},
  {0x1B1E4, 776747},
  {0x1B1E5, 776769},
  {0x1B1E6, 776791},
  {0x1B1E7, 776813},
  {0x1B1E8, 776835},
  {0x1B1E9, 776857},
  {0x1B1EA, 776879},
  {0x1B1EB, 776901},
  {0x1B1EC, 776923},
  {0x1B1ED, 776945},
  {0x1B1EE, 776967},
  {0x1B1EF, 776989},
  {0x1B1F0, 777011},
  {0x1B1F1, 777033},
  {0x1B1F2, 777055},
  {0x1B1F3, 777077},
  {0x1B1F4, 777099},
  {0x1B1F5, 777121},
  {0x1B1F6, 777143},
  {0x1B1F7, 777165},
  {0x1B1F8, 777187},
  {0x1B1F9, 777209},
  {0x1B1FA, 777231},
  {0x1B1FB, 777253},
  {0x1B1FC, 777275},
  {0x1B1FD, 777297},
  {0x1B1FE, 777319},
  {0x1B1FF, 777341},
  {0x1B200, 777363},
  {0x1B201, 777385},
  {0x1B202, 777407},
  {0x1B203, 777429},
  {0x1B204, 777451},
  {0x1B205, 777473},
  {0x1B206, 777495},
  {0x1B207, 777517},
  {0x1B208, 777539},
  {0x1B209, 777561},
  {0x1B20A, 777583},
  {0x1B20B, 777605},
  {0x1B20C, 777627},
  {0x1B20D, 777649},
  {0x1B20E, 777671},
  {0x1B20F, 777693},
  {0x1B210, 777715},
  {0x1B211, 777737},
  {0x1B212, 777759},
  {0x1B213, 777781},
  {0x1B214, 777803},
  {0x1B215, 777825},
  {0x1B216, 777847},
  {0x1B217, 777869},
  {0x1B218, 777891},
  {0x1B219, 777913},
  {0x1B21A, 777935},
  {0x1B21B, 777957},
  {0x1B21C, 777979},
  {0x1B21D, 778001},
  {0x1B21E, 778023},
  {0x1B21F, 778045},
  {0x1B220, 778067},
  {0x1B221, 778089},
  {0x1B222, 778111},
  {0x1B223, 778133},
  {0x1B224, 778155},
  {0x1B225, 778177},
  {0x1B226, 778199},
  {0x1B227, 778221},
  {0x1B228, 778243},
  {0x1B229, 778265},
  {0x1B22A, 778287},
  {0x1B22B, 778309},
  {0x1B22C, 778331},
  {0x1B22D, 778353},
  {0x1B22E, 778375},
  {0x1B22F, 778397},
  {0x1B230, 778419},
  {0x1B231, 778441},
  {0x1B232, 778463},
  {0x1B233, 778485},
  {0x1B234, 778507},
  {0x1B235, 778529},
  {0x1B236, 778551},
  {0x1B237, 778573},
  {0x1B238, 778595},
  {0x1B239, 778617},
  {0x1B23A, 778639},
  {0x1B23B, 778661},
  {0x1B23C, 778683},
  {0x1B23D, 778705},
  {0x1B23E, 778727},
  {0x1B23F, 778749},
  {0x1B240, 778771},
  {0x1B241, 778793},
  {0x1B242, 778815},
  {0x1B243, 778837},
  {0x1B244, 778859},
  {0x1B245, 778881},
  {0x1B246, 778903},
  {0x1B247, 778925},
  {0x1B248, 778947},
  {0x1B249, 778969},
  {0x1B24A, 778991},
  {0x1B24B, 779013},
  {0x1B24C, 779035},
  {0x1B24D, 779057},
  {0x1B24E, 779079},
  {0x1B24F, 779101},
  {0x1B250, 779123},
  {0x1B251, 779145},
  {0x1B252, 779167},
  {0x1B253, 779189},
  {0x1B254, 779211},
  {0x1B255, 779233},
  {0x1B256, 779255},
  {0x1B257, 779277},
  {0x1B258, 779299},
  {0x1B259, 779321},
  {0x1B25A, 779343},
  {0x1B25B, 779365},
  {0x1B25C, 779387},
  {0x1B25D, 779409},
  {0x1B25E, 779431},
  {0x1B25F, 779453},
  {0x1B260, 779475},
  {0x1B261, 779497},
  {0x1B262, 779519},
  {0x1B263, 779541},
  {0x1B264, 779563},
  {0x1B265, 779585},
  {0x1B266, 779607},
  {0x1B267, 779629},
  {0x1B268, 779651},
  {0x1B269, 779673},
  {0x1B26A, 779695},
  {0x1B26B, 779717},
  {0x1B26C, 779739},
  {0x1B26D, 779761},
  {0x1B26E, 779783},
  {0x1B26F, 779805},
  {0x1B270, 779827},
  {0x1B271, 779849},
  {0x1B272, 779871},
  {0x1B273, 779893},
  {0x1B274, 779915},
  {0x1B275, 779937},
  {0x1B276, 779959},
  {0x1B277, 779981},
  {0x1B278, 780003},
  {0x1B279, 780025},
  {0x1B27A, 780047},
  {0x1B27B, 780069},
  {0x1B27C, 780091},
  {0x1B27D, 780113},
  {0x1B27E, 780135},
  {0x1B27F, 780157},
  {0x1B280, 780179},
  {0x1B281, 780201},
  {0x1B282, 780223},
  {0x1B283, 780245},
  {0x1B284, 780267},
  {0x1B285, 780289},
  {0x1B286, 780311},
  {0x1B287, 780333},
  {0x1B288, 780355},
  {0x1B289, 780377},
  {0x1B28A, 780399},
  {0x1B28B, 780421},
  {0x1B28C, 780443},
  {0x1B28D, 780465},
  {0x1B28E, 780487},
  {0x1B28F, 780509},
  {0x1B290, 780531},
  {0x1B291, 780553},
  {0x1B292, 780575},
  {0x1B293, 780597},
  {0x1B294, 780619},
  {0x1B295, 780641},
  {0x1B296, 780663},
  {0x1B297, 780685},
  {0x1B298, 780707},
  {0x1B299, 780729},
  {0x1B29A, 780751},
  {0x1B29B, 780773},
  {0x1B29C, 780795},
  {0x1B29D, 780817},
  {0x1B29E, 780839},
  {0x1B29F, 780861},
  {0x1B2A0, 780883},
  {0x1B2A1, 780905},
  {0x1B2A2, 780927},
  {0x1B2A3, 780949},
  {0x1B2A4, 780971},
  {0x1B2A5, 780993},
  {0x1B2A6, 781015},
  {0x1B2A7, 781037},
  {0x1B2A8, 781059},
  {0x1B2A9, 781081},
  {0x1B2AA, 781103},
  {0x1B2AB, 781125},
  {0x1B2AC, 781147},
  {0x1B2AD, 781169},
  {0x1B2AE, 781191},
  {0x1B2AF, 781213},
  {0x1B2B0, 781235},
  {0x1B2B1, 781257},
  {0x1B2B2, 781279},
  {0x1B2B3, 781301},
  {0x1B2B4, 781323},
  {0x1B2B5, 781345},
  {0x1B2B6, 781367},
  {0x1B2B7, 781389},
  {0x1B2B8, 781411},
  {0x1B2B9, 781433},
  {0x1B2BA, 781455},
  {0x1B2BB, 781477},
  {0x1B2BC, 781499},
  {0x1B2BD, 781521},
  {0x1B2BE, 781543},
  {0x1B2BF, 781565},
  {0x1B2C0, 781587},
  {0x1B2C1, 781609},
  {0x1B2C2, 781631},
  {0x1B2C3, 781653},
  {0x1B2C4, 781675},
  {0x1B2C5, 781697},
  {0x1B2C6, 781719},
  {0x1B2C7, 781741},
  {0x1B2C8, 781763},
  {0x1B2C9, 781785},
  {0x1B2CA, 781807},
  {0x1B2CB, 781829},
  {0x1B2CC, 781851},
  {0x1B2CD, 781873},
  {0x1B2CE, 781895},
  {0x1B2CF, 781917},
  {0x1B2D0, 781939},
  {0x1B2D1, 781961},
  {0x1B2D2, 781983},
  {0x1B2D3, 782005},
  {0x1B2D4, 782027},
  {0x1B2D5, 782049},
  {0x1B2D6, 782071},
  {0x1B2D7, 782093},
  {0x1B2D8, 782115},
  {0x1B2D9, 782137},
  {0x1B2DA, 782159},
  {0x1B2DB, 782181},
  {0x1B2DC, 782203},
  {0x1B2DD, 782225},
  {0x1B2DE, 782247},
  {0x1B2DF, 782269},
  {0x1B2E0, 782291},
  {0x1B2E1, 782313},
  {0x1B2E2, 782335},
  {0x1B2E3, 782357},
  {0x1B2E4, 782379},
  {0x1B2E5, 782401},
  {0x1B2E6, 782423},
  {0x1B2E7, 782445},
  {0x1B2E8, 782467},
  {0x1B2E9, 782489},
  {0x1B2EA, 782511},
  {0x1B2EB, 782533},
  {0x1B2EC, 782555},
  {0x1B2ED, 782577},
  {0x1B2EE, 782599},
  {0x1B2EF, 782621},
  {0x1B2F0, 782643},
  {0x1B2F1, 782665},
  {0x1B2F2, 782687},
  {0x1B2F3, 782709},
  {0x1B2F4, 782731},
  {0x1B2F5, 782753},
  {0x1B2F6, 782775},
  {0x1B2F7, 782797},
  {0x1B2F8, 782819},
  {0x1B2F9, 782841},
  {0x1B2FA, 782863},
  {0x1B2FB, 782885},
  {0x1BC00, 291521},
  {0x1BC01, 293492},
  {0x1BC02, 292385},
  {0x1BC03, 293139},
  {0x1BC04, 291443},
  {0x1BC05, 291818},
  {0x1BC06, 291897},
  {0x1BC07, 291312},
  {0x1BC08, 291330},
  {0x1BC09, 293294},
  {0x1BC0A, 291481},
  {0x1BC0B, 292475},
  {0x1BC0C, 292403},
  {0x1BC0D, 291348},
  {0x1BC0E, 291461},
  {0x1BC0F, 291836},
  {0x1BC10, 292493},
  {0x1BC11, 293199},
  {0x1BC12, 292941},
  {0x1BC13, 291368},
  {0x1BC14, 291878},
  {0x1BC15, 293041},
  {0x1BC16, 291539},
  {0x1BC17, 291915},
  {0x1BC18, 292513},
  {0x1BC19, 291980},
  {0x1BC1A, 292087},
  {0x1BC1B, 291595},
  {0x1BC1C, 292586},
  {0x1BC1D, 291998},
  {0x1BC1E, 292105},
  {0x1BC1F, 291613},
  {0x1BC20, 292604},
  {0x1BC21, 292060},
  {0x1BC22, 292167},
  {0x1BC23, 291746},
  {0x1BC24, 291773},
  {0x1BC25, 292832},
  {0x1BC26, 292859},
  {0x1BC27, 292040},
  {0x1BC28, 292147},
  {0x1BC29, 291697},
  {0x1BC2A, 292770},
  {0x1BC2B, 292018},
  {0x1BC2C, 292125},
  {0x1BC2D, 291633},
  {0x1BC2E, 292624},
  {0x1BC2F, 291717},
  {0x1BC30, 291655},
  {0x1BC31, 291675},
  {0x1BC32, 292790},
  {0x1BC33, 292810},
  {0x1BC34, 292728},
  {0x1BC35, 292748},
  {0x1BC36, 293179},
  {0x1BC37, 293157},
  {0x1BC38, 293338},
  {0x1BC39, 293415},
  {0x1BC3A, 293356},
  {0x1BC3B, 292708},
  {0x1BC3C, 292688},
  {0x1BC3D, 291856},
  {0x1BC3E, 291499},
  {0x1BC3F, 292646},
  {0x1BC40, 292666},
  {0x1BC41, 291274},
  {0x1BC42, 293090},
  {0x1BC43, 292308},
  {0x1BC44, 292290},
  {0x1BC45, 291292},
  {0x1BC46, 291558},
  {0x1BC47, 291387},
  {0x1BC48, 291576},
  {0x1BC49, 292892},
  {0x1BC4A, 293275},
  {0x1BC4B, 291405},
  {0x1BC4C, 292991},
  {0x1BC4D, 292532},
  {0x1BC4E, 292966},
  {0x1BC4F, 291934},
  {0x1BC50, 293529},
  {0x1BC51, 293218},
  {0x1BC52, 291424},
  {0x1BC53, 293510},
  {0x1BC54, 293236},
  {0x1BC55, 291957},
  {0x1BC56, 292559},
  {0x1BC57, 293256},
  {0x1BC58, 293115},
  {0x1BC59, 292327},
  {0x1BC5A, 292366},
  {0x1BC5B, 292347},
  {0x1BC5C, 293376},
  {0x1BC5D, 293453},
  {0x1BC5E, 293434},
  {0x1BC5F, 293395},
  {0x1BC60, 293472},
  {0x1BC61, 292266},
  {0x1BC62, 292242},
  {0x1BC63, 292218},
  {0x1BC64, 292194},
  {0x1BC65, 292449},
  {0x1BC66, 292423},
  {0x1BC67, 293016},
  {0x1BC68, 292916},
  {0x1BC69, 293065},
  {0x1BC6A, 293312},
  {0x1BC70, 290808},
  {0x1BC71, 291142},
  {0x1BC72, 291214},
  {0x1BC73, 291083},
  {0x1BC74, 291179},
  {0x1BC75, 290747},
  {0x1BC76, 290398},
  {0x1BC77, 290353},
  {0x1BC78, 290458},
  {0x1BC79, 290429},
  {0x1BC7A, 290291},
  {0x1BC7B, 290322},
  {0x1BC7C, 290490},
  {0x1BC80, 290527},
  {0x1BC81, 290686},
  {0x1BC82, 290604},
  {0x1BC83, 290655},
  {0x1BC84, 290580},
  {0x1BC85, 290553},
  {0x1BC86, 290630},
  {0x1BC87, 290783},
  {0x1BC88, 290718},
  {0x1BC90, 290846},
  {0x1BC91, 291024},
  {0x1BC92, 290945},
  {0x1BC93, 290994},
  {0x1BC94, 290922},
  {0x1BC95, 290896},
  {0x1BC96, 290970},
  {0x1BC97, 291118},
  {0x1BC98, 291055},
  {0x1BC99, 290871},
  {0x1BC9C, 293587},
  {0x1BC9D, 293614},
  {0x1BC9E, 291253},
  {0x1BC9F, 293548},
  {0x1BCA0, 856600},
  {0x1BCA1, 856537},
  {0x1BCA2, 856573},
  {0x1BCA3, 856632},
  {0x1CC00, 955817},
  {0x1CC01, 837186},
  {0x1CC02, 629903},
  {0x1CC03, 837379},
  {0x1CC04, 287605},
  {0x1CC05, 653369},
  {0x1CC06, 836481},
  {0x1CC07, 656375},
  {0x1CC08, 24724},
  {0x1CC09, 523891},
  {0x1CC0A, 975540},
  {0x1CC0B, 628631},
  {0x1CC0C, 725118},
  {0x1CC0D, 836061},
  {0x1CC0E, 629651},
  {0x1CC0F, 837095},
  {0x1CC10, 773882},
  {0x1CC11, 825842},
  {0x1CC12, 830530},
  {0x1CC13, 523432},
  {0x1CC14, 974794},
  {0x1CC15, 651817},
  {0x1CC16, 651692},
  {0x1CC17, 651763},
  {0x1CC18, 651790},
  {0x1CC19, 651707},
  {0x1CC1A, 651725},
  {0x1CC1B, 130140},
  {0x1CC1C, 130094},
  {0x1CC1D, 130414},
  {0x1CC1E, 127583},
  {0x1CC1F, 125039},
  {0x1CC20, 124912},
  {0x1CC21, 850775},
  {0x1CC22, 851003},
  {0x1CC23, 850802},
  {0x1CC24, 851115},
  {0x1CC25, 850918},
  {0x1CC26, 851030},
  {0x1CC27, 850830},
  {0x1CC28, 851170},
  {0x1CC29, 850975},
  {0x1CC2A, 851087},
  {0x1CC2B, 850889},
  {0x1CC2C, 851142},
  {0x1CC2D, 850946},
  {0x1CC2E, 851058},
  {0x1CC2F, 850859},
  {0x1CC30, 958126},
  {0x1CC31, 956345},
  {0x1CC32, 956448},
  {0x1CC33, 959836},
  {0x1CC34, 958281},
  {0x1CC35, 958014},
  {0x1CC36, 959653},
  {0x1CC37, 958353},
  {0x1CC38, 654935},
  {0x1CC39, 654655},
  {0x1CC3A, 656348},
  {0x1CC3B, 655007},
  {0x1CC3C, 654780},
  {0x1CC3D, 652931},
  {0x1CC3E, 653034},
  {0x1CC3F, 656567},
  {0x1CC40, 899063},
  {0x1CC41, 899086},
  {0x1CC42, 802832},
  {0x1CC43, 276382},
  {0x1CC44, 269577},
  {0x1CC45, 269555},
  {0x1CC46, 899256},
  {0x1CC47, 899277},
  {0x1CC48, 629280},
  {0x1CC49, 836735},
  {0x1CC4A, 444430},
  {0x1CC4B, 444455},
  {0x1CC4C, 8904},
  {0x1CC4D, 8878},
  {0x1CC4E, 9042},
  {0x1CC4F, 9013},
  {0x1CC50, 8838},
  {0x1CC51, 8813},
  {0x1CC52, 8970},
  {0x1CC53, 8993},
  {0x1CC54, 8928},
  {0x1CC55, 8949},
  {0x1CC56, 629844},
  {0x1CC57, 955933},
  {0x1CC58, 837318},
  {0x1CC59, 287546},
  {0x1CC5A, 948347},
  {0x1CC5B, 948242},
  {0x1CC5C, 948473},
  {0x1CC5D, 123984},
  {0x1CC5E, 123733},
  {0x1CC5F, 124122},
  {0x1CC60, 629590},
  {0x1CC61, 955752},
  {0x1CC62, 837032},
  {0x1CC63, 287403},
  {0x1CC64, 744240},
  {0x1CC65, 629795},
  {0x1CC66, 955915},
  {0x1CC67, 837267},
  {0x1CC68, 287526},
  {0x1CC69, 426577},
  {0x1CC6A, 426595},
  {0x1CC6B, 113694},
  {0x1CC6C, 113834},
  {0x1CC6D, 113787},
  {0x1CC6E, 113740},
  {0x1CC6F, 114816},
  {0x1CC70, 629409},
  {0x1CC71, 955694},
  {0x1CC72, 836869},
  {0x1CC73, 287341},
  {0x1CC74, 629368},
  {0x1CC75, 955655},
  {0x1CC76, 836827},
  {0x1CC77, 287300},
  {0x1CC78, 629713},
  {0x1CC79, 955776},
  {0x1CC7A, 837159},
  {0x1CC7B, 287429},
  {0x1CC7C, 902614},
  {0x1CC7D, 902642},
  {0x1CC7E, 902582},
  {0x1CC7F, 902551},
  {0x1CC80, 907101},
  {0x1CC81, 907164},
  {0x1CC82, 907132},
  {0x1CC83, 907070},
  {0x1CC84, 975102},
  {0x1CC85, 523473},
  {0x1CC86, 985229},
  {0x1CC87, 985254},
  {0x1CC88, 954095},
  {0x1CC89, 900275},
  {0x1CC8A, 900246},
  {0x1CC8B, 900211},
  {0x1CC8C, 899630},
  {0x1CC8D, 535728},
  {0x1CC8E, 625983},
  {0x1CC8F, 535750},
  {0x1CC90, 975220},
  {0x1CC91, 523517},
  {0x1CC92, 629301},
  {0x1CC93, 836757},
  {0x1CC94, 629318},
  {0x1CC95, 836775},
  {0x1CC96, 444218},
  {0x1CC97, 629770},
  {0x1CC98, 955867},
  {0x1CC99, 837241},
  {0x1CC9A, 287474},
  {0x1CC9B, 523869},
  {0x1CC9C, 975520},
  {0x1CC9D, 974850},
  {0x1CC9E, 629930},
  {0x1CC9F, 837407},
  {0x1CCA0, 629815},
  {0x1CCA1, 837288},
  {0x1CCA2, 629870},
  {0x1CCA3, 837345},
  {0x1CCA4, 626953},
  {0x1CCA5, 834767},
  {0x1CCA6, 957873},
  {0x1CCA7, 959508},
  {0x1CCA8, 957831},
  {0x1CCA9, 959465},
  {0x1CCAA, 654555},
  {0x1CCAB, 656245},
  {0x1CCAC, 654522},
  {0x1CCAD, 656211},
  {0x1CCAE, 654413},
  {0x1CCAF, 656099},
  {0x1CCB0, 654454},
  {0x1CCB1, 656141},
  {0x1CCB2, 957983},
  {0x1CCB3, 959621},
  {0x1CCB4, 654624},
  {0x1CCB5, 656316},
  {0x1CCB6, 957913},
  {0x1CCB7, 959549},
  {0x1CCB8, 654488},
  {0x1CCB9, 656176},
  {0x1CCBA, 957640},
  {0x1CCBB, 959268},
  {0x1CCBC, 654222},
  {0x1CCBD, 655902},
  {0x1CCBE, 957735},
  {0x1CCBF, 959366},
  {0x1CCC0, 654317},
  {0x1CCC1, 656000},
  {0x1CCC2, 957767},
  {0x1CCC3, 959399},
  {0x1CCC4, 654349},
  {0x1CCC5, 656033},
  {0x1CCC6, 957607},
  {0x1CCC7, 959234},
  {0x1CCC8, 654189},
  {0x1CCC9, 655868},
  {0x1CCCA, 957671},
  {0x1CCCB, 959300},
  {0x1CCCC, 654253},
  {0x1CCCD, 655934},
  {0x1CCCE, 957704},
  {0x1CCCF, 959334},
  {0x1CCD0, 654286},
  {0x1CCD1, 655968},
  {0x1CCD2, 957947},
  {0x1CCD3, 959584},
  {0x1CCD4, 654588},
  {0x1CCD5, 656279},
  {0x1CCD6, 807593},
  {0x1CCD7, 807625},
  {0x1CCD8, 807657},
  {0x1CCD9, 807689},
  {0x1CCDA, 807721},
  {0x1CCDB, 807753},
  {0x1CCDC, 807785},
  {0x1CCDD, 807817},
  {0x1CCDE, 807849},
  {0x1CCDF, 807881},
  {0x1CCE0, 807913},
  {0x1CCE1, 807945},
  {0x1CCE2, 807977},
  {0x1CCE3, 808009},
  {0x1CCE4, 808041},
  {0x1CCE5, 808073},
  {0x1CCE6, 808105},
  {0x1CCE7, 808137},
  {0x1CCE8, 808169},
  {0x1CCE9, 808201},
  {0x1CCEA, 808233},
  {0x1CCEB, 808265},
  {0x1CCEC, 808297},
  {0x1CCED, 808329},
  {0x1CCEE, 808361},
  {0x1CCEF, 808393},
  {0x1CCF0, 807552},
  {0x1CCF1, 807453},
  {0x1CCF2, 807533},
  {0x1CCF3, 807512},
  {0x1CCF4, 807413},
  {0x1CCF5, 807393},
  {0x1CCF6, 807493},
  {0x1CCF7, 807472},
  {0x1CCF8, 807372},
  {0x1CCF9, 807433},
  {0x1CD00, 119917},
  {0x1CD01, 118852},
  {0x1CD02, 116687},
  {0x1CD03, 120441},
  {0x1CD04, 118307},
  {0x1CD05, 117211},
  {0x1CD06, 119932},
  {0x1CD07, 117787},
  {0x1CD08, 118868},
  {0x1CD09, 120697},
  {0x1CD0A, 118595},
  {0x1CD0B, 119660},
  {0x1CD0C, 117515},
  {0x1CD0D, 118091},
  {0x1CD0E, 119172},
  {0x1CD0F, 116924},
  {0x1CD10, 120456},
  {0x1CD11, 118323},
  {0x1CD12, 119444},
  {0x1CD13, 117228},
  {0x1CD14, 119948},
  {0x1CD15, 117804},
  {0x1CD16, 118885},
  {0x1CD17, 116704},
  {0x1CD18, 120795},
  {0x1CD19, 118735},
  {0x1CD1A, 119800},
  {0x1CD1B, 117663},
  {0x1CD1C, 120324},
  {0x1CD1D, 118201},
  {0x1CD1E, 119320},
  {0x1CD1F, 117080},
  {0x1CD20, 118471},
  {0x1CD21, 119554},
  {0x1CD22, 117384},
  {0x1CD23, 120076},
  {0x1CD24, 117960},
  {0x1CD25, 119041},
  {0x1CD26, 116806},
  {0x1CD27, 120712},
  {0x1CD28, 118611},
  {0x1CD29, 119676},
  {0x1CD2A, 117532},
  {0x1CD2B, 120200},
  {0x1CD2C, 118108},
  {0x1CD2D, 119189},
  {0x1CD2E, 116942},
  {0x1CD2F, 120472},
  {0x1CD30, 118340},
  {0x1CD31, 119461},
  {0x1CD32, 117246},
  {0x1CD33, 119965},
  {0x1CD34, 117822},
  {0x1CD35, 118903},
  {0x1CD36, 118803},
  {0x1CD37, 119868},
  {0x1CD38, 117735},
  {0x1CD39, 120392},
  {0x1CD3A, 118255},
  {0x1CD3B, 119392},
  {0x1CD3C, 117156},
  {0x1CD3D, 120648},
  {0x1CD3E, 118543},
  {0x1CD3F, 119608},
  {0x1CD40, 117460},
  {0x1CD41, 120148},
  {0x1CD42, 118036},
  {0x1CD43, 119117},
  {0x1CD44, 116866},
  {0x1CD45, 118683},
  {0x1CD46, 119748},
  {0x1CD47, 117608},
  {0x1CD48, 120272},
  {0x1CD49, 119265},
  {0x1CD4A, 117022},
  {0x1CD4B, 120544},
  {0x1CD4C, 118416},
  {0x1CD4D, 117326},
  {0x1CD4E, 120021},
  {0x1CD4F, 117902},
  {0x1CD50, 118983},
  {0x1CD51, 120810},
  {0x1CD52, 118751},
  {0x1CD53, 119816},
  {0x1CD54, 117680},
  {0x1CD55, 120340},
  {0x1CD56, 118218},
  {0x1CD57, 119337},
  {0x1CD58, 117098},
  {0x1CD59, 120596},
  {0x1CD5A, 118488},
  {0x1CD5B, 119571},
  {0x1CD5C, 117402},
  {0x1CD5D, 120093},
  {0x1CD5E, 117978},
  {0x1CD5F, 119059},
  {0x1CD60, 116825},
  {0x1CD61, 120728},
  {0x1CD62, 118628},
  {0x1CD63, 119693},
  {0x1CD64, 117550},
  {0x1CD65, 120217},
  {0x1CD66, 118126},
  {0x1CD67, 119207},
  {0x1CD68, 116961},
  {0x1CD69, 120489},
  {0x1CD6A, 118358},
  {0x1CD6B, 119479},
  {0x1CD6C, 117265},
  {0x1CD6D, 119983},
  {0x1CD6E, 117841},
  {0x1CD6F, 118922},
  {0x1CD70, 116723},
  {0x1CD71, 118836},
  {0x1CD72, 119901},
  {0x1CD73, 117770},
  {0x1CD74, 120425},
  {0x1CD75, 118290},
  {0x1CD76, 119427},
  {0x1CD77, 117193},
  {0x1CD78, 120681},
  {0x1CD79, 118578},
  {0x1CD7A, 119643},
  {0x1CD7B, 117497},
  {0x1CD7C, 120183},
  {0x1CD7D, 118073},
  {0x1CD7E, 119154},
  {0x1CD7F, 116905},
  {0x1CD80, 120779},
  {0x1CD81, 118718},
  {0x1CD82, 119783},
  {0x1CD83, 117645},
  {0x1CD84, 120307},
  {0x1CD85, 118183},
  {0x1CD86, 119302},
  {0x1CD87, 117061},
  {0x1CD88, 120579},
  {0x1CD89, 118453},
  {0x1CD8A, 119536},
  {0x1CD8B, 117365},
  {0x1CD8C, 120058},
  {0x1CD8D, 117941},
  {0x1CD8E, 119022},
  {0x1CD8F, 116786},
  {0x1CD90, 118786},
  {0x1CD91, 119851},
  {0x1CD92, 117717},
  {0x1CD93, 120375},
  {0x1CD94, 119374},
  {0x1CD95, 117137},
  {0x1CD96, 120631},
  {0x1CD97, 118525},
  {0x1CD98, 117441},
  {0x1CD99, 120130},
  {0x1CD9A, 118017},
  {0x1CD9B, 119098},
  {0x1CD9C, 120745},
  {0x1CD9D, 118665},
  {0x1CD9E, 119730},
  {0x1CD9F, 117589},
  {0x1CDA0, 120254},
  {0x1CDA1, 118145},
  {0x1CDA2, 119246},
  {0x1CDA3, 117002},
  {0x1CDA4, 120526},
  {0x1CDA5, 118397},
  {0x1CDA6, 119498},
  {0x1CDA7, 117306},
  {0x1CDA8, 120002},
  {0x1CDA9, 117882},
  {0x1CDAA, 118963},
  {0x1CDAB, 116744},
  {0x1CDAC, 118819},
  {0x1CDAD, 119884},
  {0x1CDAE, 117752},
  {0x1CDAF, 120408},
  {0x1CDB0, 118272},
  {0x1CDB1, 119409},
  {0x1CDB2, 117174},
  {0x1CDB3, 120664},
  {0x1CDB4, 118560},
  {0x1CDB5, 119625},
  {0x1CDB6, 117478},
  {0x1CDB7, 120165},
  {0x1CDB8, 118054},
  {0x1CDB9, 119135},
  {0x1CDBA, 116885},
  {0x1CDBB, 120762},
  {0x1CDBC, 118700},
  {0x1CDBD, 119765},
  {0x1CDBE, 117626},
  {0x1CDBF, 120289},
  {0x1CDC0, 118164},
  {0x1CDC1, 119283},
  {0x1CDC2, 117041},
  {0x1CDC3, 120561},
  {0x1CDC4, 118434},
  {0x1CDC5, 119517},
  {0x1CDC6, 117345},
  {0x1CDC7, 120039},
  {0x1CDC8, 117921},
  {0x1CDC9, 119002},
  {0x1CDCA, 116765},
  {0x1CDCB, 120826},
  {0x1CDCC, 118768},
  {0x1CDCD, 119833},
  {0x1CDCE, 117698},
  {0x1CDCF, 120357},
  {0x1CDD0, 118236},
  {0x1CDD1, 119355},
  {0x1CDD2, 117117},
  {0x1CDD3, 120613},
  {0x1CDD4, 118506},
  {0x1CDD5, 119589},
  {0x1CDD6, 117421},
  {0x1CDD7, 120111},
  {0x1CDD8, 117997},
  {0x1CDD9, 119078},
  {0x1CDDA, 116845},
  {0x1CDDB, 118646},
  {0x1CDDC, 119711},
  {0x1CDDD, 117569},
  {0x1CDDE, 120235},
  {0x1CDDF, 119226},
  {0x1CDE0, 116981},
  {0x1CDE1, 120507},
  {0x1CDE2, 118377},
  {0x1CDE3, 117285},
  {0x1CDE4, 117861},
  {0x1CDE5, 118942},
  {0x1CDE6, 948597},
  {0x1CDE7, 124233},
  {0x1CDE8, 948501},
  {0x1CDE9, 124153},
  {0x1CDEA, 948538},
  {0x1CDEB, 124193},
  {0x1CDEC, 948374},
  {0x1CDED, 124014},
  {0x1CDEE, 948410},
  {0x1CDEF, 124053},
  {0x1CDF0, 948272},
  {0x1CDF1, 123766},
  {0x1CDF2, 123808},
  {0x1CDF3, 123850},
  {0x1CDF4, 123892},
  {0x1CDF5, 742276},
  {0x1CDF6, 948178},
  {0x1CDF7, 948210},
  {0x1CDF8, 287380},
  {0x1CDF9, 629539},
  {0x1CDFA, 893364},
  {0x1CDFB, 955800},
  {0x1CDFC, 287455},
  {0x1CDFD, 441927},
  {0x1CDFE, 441945},
  {0x1CDFF, 441963},
  {0x1CE00, 834666},
  {0x1CE01, 653093},
  {0x1CE02, 441906},
  {0x1CE03, 956595},
  {0x1CE04, 653167},
  {0x1CE05, 509324},
  {0x1CE06, 986893},
  {0x1CE07, 948706},
  {0x1CE08, 448092},
  {0x1CE09, 124840},
  {0x1CE0A, 124967},
  {0x1CE0B, 627240},
  {0x1CE0C, 835010},
  {0x1CE0D, 627173},
  {0x1CE0E, 834941},
  {0x1CE0F, 523592},
  {0x1CE10, 626994},
  {0x1CE11, 834810},
  {0x1CE12, 523554},
  {0x1CE13, 653322},
  {0x1CE14, 956803},
  {0x1CE15, 975285},
  {0x1CE16, 130969},
  {0x1CE17, 130765},
  {0x1CE18, 130928},
  {0x1CE19, 130721},
  {0x1CE1A, 581998},
  {0x1CE1B, 582030},
  {0x1CE1C, 582205},
  {0x1CE1D, 582065},
  {0x1CE1E, 580861},
  {0x1CE1F, 580495},
  {0x1CE20, 580521},
  {0x1CE21, 581918},
  {0x1CE22, 580605},
  {0x1CE23, 581449},
  {0x1CE24, 582100},
  {0x1CE25, 581384},
  {0x1CE26, 582169},
  {0x1CE27, 582133},
  {0x1CE28, 581624},
  {0x1CE29, 581487},
  {0x1CE2A, 580807},
  {0x1CE2B, 580769},
  {0x1CE2C, 580642},
  {0x1CE2D, 581411},
  {0x1CE2E, 580967},
  {0x1CE2F, 580365},
  {0x1CE30, 580887},
  {0x1CE31, 581958},
  {0x1CE32, 580394},
  {0x1CE33, 580423},
  {0x1CE34, 580452},
  {0x1CE35, 581345},
  {0x1CE36, 581545},
  {0x1CE37, 581116},
  {0x1CE38, 580717},
  {0x1CE39, 581586},
  {0x1CE3A, 581509},
  {0x1CE3B, 580680},
  {0x1CE3C, 581313},
  {0x1CE3D, 581046},
  {0x1CE3E, 581014},
  {0x1CE3F, 581081},
  {0x1CE40, 580563},
  {0x1CE41, 582237},
  {0x1CE42, 580927},
  {0x1CE43, 581165},
  {0x1CE44, 581241},
  {0x1CE45, 581198},
  {0x1CE46, 581277},
  {0x1CE47, 581893},
  {0x1CE48, 581793},
  {0x1CE49, 581869},
  {0x1CE4A, 581844},
  {0x1CE4B, 581767},
  {0x1CE4C, 581691},
  {0x1CE4D, 581820},
  {0x1CE4E, 581742},
  {0x1CE4F, 581718},
  {0x1CE50, 581666},
  {0x1CE51, 851197},
  {0x1CE52, 852109},
  {0x1CE53, 851223},
  {0x1CE54, 852557},
  {0x1CE55, 851687},
  {0x1CE56, 852135},
  {0x1CE57, 851250},
  {0x1CE58, 852777},
  {0x1CE59, 851915},
  {0x1CE5A, 852363},
  {0x1CE5B, 851486},
  {0x1CE5C, 852583},
  {0x1CE5D, 851714},
  {0x1CE5E, 852162},
  {0x1CE5F, 851278},
  {0x1CE60, 852885},
  {0x1CE61, 852027},
  {0x1CE62, 852475},
  {0x1CE63, 851602},
  {0x1CE64, 852695},
  {0x1CE65, 851830},
  {0x1CE66, 852278},
  {0x1CE67, 851398},
  {0x1CE68, 852803},
  {0x1CE69, 851942},
  {0x1CE6A, 852390},
  {0x1CE6B, 851514},
  {0x1CE6C, 852610},
  {0x1CE6D, 851742},
  {0x1CE6E, 852190},
  {0x1CE6F, 851307},
  {0x1CE70, 852938},
  {0x1CE71, 852082},
  {0x1CE72, 852530},
  {0x1CE73, 851659},
  {0x1CE74, 852750},
  {0x1CE75, 851887},
  {0x1CE76, 852335},
  {0x1CE77, 851457},
  {0x1CE78, 852858},
  {0x1CE79, 851999},
  {0x1CE7A, 852447},
  {0x1CE7B, 851573},
  {0x1CE7C, 852667},
  {0x1CE7D, 851801},
  {0x1CE7E, 852249},
  {0x1CE7F, 851368},
  {0x1CE80, 852911},
  {0x1CE81, 852054},
  {0x1CE82, 852502},
  {0x1CE83, 851630},
  {0x1CE84, 852722},
  {0x1CE85, 851858},
  {0x1CE86, 852306},
  {0x1CE87, 851427},
  {0x1CE88, 852830},
  {0x1CE89, 851970},
  {0x1CE8A, 852418},
  {0x1CE8B, 851543},
  {0x1CE8C, 852638},
  {0x1CE8D, 851771},
  {0x1CE8E, 852219},
  {0x1CE8F, 851337},
  {0x1CE90, 957528},
  {0x1CE91, 956307},
  {0x1CE92, 956409},
  {0x1CE93, 959135},
  {0x1CE94, 958243},
  {0x1CE95, 958152},
  {0x1CE96, 958197},
  {0x1CE97, 958314},
  {0x1CE98, 654897},
  {0x1CE99, 654806},
  {0x1CE9A, 654851},
  {0x1CE9B, 654968},
  {0x1CE9C, 654118},
  {0x1CE9D, 652893},
  {0x1CE9E, 652995},
  {0x1CE9F, 655817},
  {0x1CEA0, 834859},
  {0x1CEA1, 836195},
  {0x1CEA2, 628760},
  {0x1CEA3, 627094},
  {0x1CEA4, 653229},
  {0x1CEA5, 656673},
  {0x1CEA6, 959942},
  {0x1CEA7, 956710},
  {0x1CEA8, 627206},
  {0x1CEA9, 628804},
  {0x1CEAA, 836240},
  {0x1CEAB, 834975},
  {0x1CEAC, 956768},
  {0x1CEAD, 959986},
  {0x1CEAE, 656717},
  {0x1CEAF, 653287},
  {0x1CEB0, 524055},
  {0x1CEB1, 552407},
  {0x1CEB2, 795362},
  {0x1CEB3, 115030},
  {0x1CF00, 1013378},
  {0x1CF01, 1014064},
  {0x1CF02, 1014964},
  {0x1CF03, 1013285},
  {0x1CF04, 1013987},
  {0x1CF05, 1014777},
  {0x1CF06, 1013691},
  {0x1CF07, 1014379},
  {0x1CF08, 1015112},
  {0x1CF09, 1013782},
  {0x1CF0A, 1014460},
  {0x1CF0B, 1015191},
  {0x1CF0C, 1013434},
  {0x1CF0D, 1014112},
  {0x1CF0E, 1015002},
  {0x1CF0F, 1013331},
  {0x1CF10, 1014025},
  {0x1CF11, 1014816},
  {0x1CF12, 1013736},
  {0x1CF13, 1014419},
  {0x1CF14, 1015151},
  {0x1CF15, 1013838},
  {0x1CF16, 1014511},
  {0x1CF17, 1015241},
  {0x1CF18, 1015041},
  {0x1CF19, 1013895},
  {0x1CF1A, 1014856},
  {0x1CF1B, 1014563},
  {0x1CF1C, 1014605},
  {0x1CF1D, 1015292},
  {0x1CF1E, 1013942},
  {0x1CF1F, 1013491},
  {0x1CF20, 1015367},
  {0x1CF21, 1014194},
  {0x1CF22, 1014645},
  {0x1CF23, 1014901},
  {0x1CF24, 1013064},
  {0x1CF25, 1015081},
  {0x1CF26, 1014345},
  {0x1CF27, 1013661},
  {0x1CF28, 1013629},
  {0x1CF29, 1013530},
  {0x1CF2A, 1015396},
  {0x1CF2B, 1014707},
  {0x1CF2C, 1014675},
  {0x1CF2D, 1013591},
  {0x1CF30, 1015426},
  {0x1CF31, 1015470},
  {0x1CF32, 1015513},
  {0x1CF33, 1015333},
  {0x1CF34, 1013204},
  {0x1CF35, 1014221},
  {0x1CF36, 1014254},
  {0x1CF37, 1014289},
  {0x1CF38, 1013096},
  {0x1CF39, 1013128},
  {0x1CF3A, 1014161},
  {0x1CF3B, 1014737},
  {0x1CF3C, 1014933},
  {0x1CF3D, 1013250},
  {0x1CF3E, 1012974},
  {0x1CF3F, 1013168},
  {0x1CF40, 1013561},
  {0x1CF41, 1013017},
  {0x1CF42, 1019824},
  {0x1CF43, 1019858},
  {0x1CF44, 1019751},
  {0x1CF45, 1019792},
  {0x1CF46, 1019892},
  {0x1CF50, 1015995},
  {0x1CF51, 1016016},
  {0x1CF52, 1016530},
  {0x1CF53, 1015718},
  {0x1CF54, 1015951},
  {0x1CF55, 1019679},
  {0x1CF56, 1017752},
  {0x1CF57, 1017776},
  {0x1CF58, 1016574},
  {0x1CF59, 1016599},
  {0x1CF5A, 1017809},
  {0x1CF5B, 1015582},
  {0x1CF5C, 1016509},
  {0x1CF5D, 1019703},
  {0x1CF5E, 1015766},
  {0x1CF5F, 1015862},
  {0x1CF60, 1015797},
  {0x1CF61, 1015830},
  {0x1CF62, 1015893},
  {0x1CF63, 1019577},
  {0x1CF64, 1019543},
  {0x1CF65, 1019610},
  {0x1CF66, 1019643},
  {0x1CF67, 1019405},
  {0x1CF68, 1019359},
  {0x1CF69, 1019450},
  {0x1CF6A, 1019495},
  {0x1CF6B, 1015666},
  {0x1CF6C, 1016691},
  {0x1CF6D, 1015642},
  {0x1CF6E, 1015928},
  {0x1CF6F, 1015559},
  {0x1CF70, 1016639},
  {0x1CF71, 1017080},
  {0x1CF72, 1017115},
  {0x1CF73, 1017182},
  {0x1CF74, 1017149},
  {0x1CF75, 1015608},
  {0x1CF76, 1016989},
  {0x1CF77, 1016942},
  {0x1CF78, 1017035},
  {0x1CF79, 1016848},
  {0x1CF7A, 1016895},
  {0x1CF7B, 1016795},
  {0x1CF7C, 1016745},
  {0x1CF7D, 1017219},
  {0x1CF7E, 1017279},
  {0x1CF7F, 1017315},
  {0x1CF80, 1017244},
  {0x1CF81, 1016043},
  {0x1CF82, 1016726},
  {0x1CF83, 1015746},
  {0x1CF84, 1015973},
  {0x1CF85, 1019728},
  {0x1CF86, 1017349},
  {0x1CF87, 1017554},
  {0x1CF88, 1017449},
  {0x1CF89, 1017587},
  {0x1CF8A, 1017481},
  {0x1CF8B, 1017677},
  {0x1CF8C, 1017709},
  {0x1CF8D, 1017524},
  {0x1CF8E, 1017408},
  {0x1CF8F, 1017630},
  {0x1CF90, 1016665},
  {0x1CF91, 1017371},
  {0x1CF92, 1019177},
  {0x1CF93, 1019035},
  {0x1CF94, 1019002},
  {0x1CF95, 1019109},
  {0x1CF96, 1019145},
  {0x1CF97, 1019072},
  {0x1CF98, 1018444},
  {0x1CF99, 1018400},
  {0x1CF9A, 1019208},
  {0x1CF9B, 1017854},
  {0x1CF9C, 1017936},
  {0x1CF9D, 1017979},
  {0x1CF9E, 1017889},
  {0x1CF9F, 1018021},
  {0x1CFA0, 1018106},
  {0x1CFA1, 1018341},
  {0x1CFA2, 1018075},
  {0x1CFA3, 1018245},
  {0x1CFA4, 1018282},
  {0x1CFA5, 1018159},
  {0x1CFA6, 1018197},
  {0x1CFA7, 1016087},
  {0x1CFA8, 1016221},
  {0x1CFA9, 1016188},
  {0x1CFAA, 1016151},
  {0x1CFAB, 1016109},
  {0x1CFAC, 1019246},
  {0x1CFAD, 1019283},
  {0x1CFAE, 1019321},
  {0x1CFAF, 1016294},
  {0x1CFB0, 1016446},
  {0x1CFB1, 1016476},
  {0x1CFB2, 1016405},
  {0x1CFB3, 1016363},
  {0x1CFB4, 1016315},
  {0x1CFB5, 1018483},
  {0x1CFB6, 1018918},
  {0x1CFB7, 1018558},
  {0x1CFB8, 1018848},
  {0x1CFB9, 1018516},
  {0x1CFBA, 1018730},
  {0x1CFBB, 1018778},
  {0x1CFBC, 1018622},
  {0x1CFBD, 1018671},
  {0x1CFBE, 1018961},
  {0x1CFBF, 1016064},
  {0x1CFC0, 1016272},
  {0x1CFC1, 1015698},
  {0x1CFC2, 1016251},
  {0x1CFC3, 1016554},
  {0x1D000, 150675},
  {0x1D001, 144495},
  {0x1D002, 150253},
  {0x1D003, 149734},
  {0x1D004, 149697},
  {0x1D005, 152438},
  {0x1D006, 152400},
  {0x1D007, 147619},
  {0x1D008, 151626},
  {0x1D009, 150023},
  {0x1D00A, 152964},
  {0x1D00B, 152999},
  {0x1D00C, 148509},
  {0x1D00D, 143463},
  {0x1D00E, 145707},
  {0x1D00F, 151661},
  {0x1D010, 147820},
  {0x1D011, 143759},
  {0x1D012, 143795},
  {0x1D013, 151562},
  {0x1D014, 151953},
  {0x1D015, 149588},
  {0x1D016, 146849},
  {0x1D017, 150706},
  {0x1D018, 144267},
  {0x1D019, 152560},
  {0x1D01A, 147470},
  {0x1D01B, 147653},
  {0x1D01C, 147855},
  {0x1D01D, 150802},
  {0x1D01E, 150147},
  {0x1D01F, 151325},
  {0x1D020, 149812},
  {0x1D021, 152518},
  {0x1D022, 143575},
  {0x1D023, 143877},
  {0x1D024, 148030},
  {0x1D025, 150771},
  {0x1D026, 150397},
  {0x1D027, 151984},
  {0x1D028, 143348},
  {0x1D029, 150836},
  {0x1D02A, 151437},
  {0x1D02B, 151479},
  {0x1D02C, 149853},
  {0x1D02D, 151760},
  {0x1D02E, 148808},
  {0x1D02F, 145402},
  {0x1D030, 152017},
  {0x1D031, 151693},
  {0x1D032, 148392},
  {0x1D033, 143506},
  {0x1D034, 145938},
  {0x1D035, 147350},
  {0x1D036, 152193},
  {0x1D037, 147543},
  {0x1D038, 150181},
  {0x1D039, 150515},
  {0x1D03A, 148135},
  {0x1D03B, 144301},
  {0x1D03C, 150738},
  {0x1D03D, 149936},
  {0x1D03E, 150060},
  {0x1D03F, 147247},
  {0x1D040, 149558},
  {0x1D041, 150290},
  {0x1D042, 148169},
  {0x1D043, 152154},
  {0x1D044, 151399},
  {0x1D045, 147209},
  {0x1D046, 147509},
  {0x1D047, 149629},
  {0x1D048, 149777},
  {0x1D049, 150323},
  {0x1D04A, 148241},
  {0x1D04B, 150356},
  {0x1D04C, 148427},
  {0x1D04D, 150215},
  {0x1D04E, 147899},
  {0x1D04F, 147695},
  {0x1D050, 153073},
  {0x1D051, 143837},
  {0x1D052, 143656},
  {0x1D053, 153040},
  {0x1D054, 148468},
  {0x1D055, 145467},
  {0x1D056, 144234},
  {0x1D057, 149521},
  {0x1D058, 152482},
  {0x1D059, 150438},
  {0x1D05A, 150550},
  {0x1D05B, 149665},
  {0x1D05C, 143427},
  {0x1D05D, 148839},
  {0x1D05E, 150106},
  {0x1D05F, 149982},
  {0x1D060, 145663},
  {0x1D061, 148543},
  {0x1D062, 143386},
  {0x1D063, 152236},
  {0x1D064, 145431},
  {0x1D065, 151525},
  {0x1D066, 151595},
  {0x1D067, 144344},
  {0x1D068, 145583},
  {0x1D069, 150868},
  {0x1D06A, 152591},
  {0x1D06B, 152318},
  {0x1D06C, 150474},
  {0x1D06D, 152114},
  {0x1D06E, 152274},
  {0x1D06F, 150589},
  {0x1D070, 152360},
  {0x1D071, 150634},
  {0x1D072, 147078},
  {0x1D073, 143942},
  {0x1D074, 145617},
  {0x1D075, 149897},
  {0x1D076, 151829},
  {0x1D077, 151870},
  {0x1D078, 151791},
  {0x1D079, 151911},
  {0x1D07A, 147584},
  {0x1D07B, 145548},
  {0x1D07C, 152663},
  {0x1D07D, 152629},
  {0x1D07E, 151292},
  {0x1D07F, 148062},
  {0x1D080, 145362},
  {0x1D081, 148313},
  {0x1D082, 148275},
  {0x1D083, 148355},
  {0x1D084, 143618},
  {0x1D085, 143545},
  {0x1D086, 145331},
  {0x1D087, 152082},
  {0x1D088, 151726},
  {0x1D089, 148208},
  {0x1D08A, 148620},
  {0x1D08B, 148576},
  {0x1D08C, 148762},
  {0x1D08D, 148713},
  {0x1D08E, 148665},
  {0x1D08F, 146890},
  {0x1D090, 146971},
  {0x1D091, 147026},
  {0x1D092, 145083},
  {0x1D093, 145178},
  {0x1D094, 145117},
  {0x1D095, 145240},
  {0x1D096, 152047},
  {0x1D097, 143911},
  {0x1D098, 147280},
  {0x1D099, 144527},
  {0x1D09A, 143265},
  {0x1D09B, 143073},
  {0x1D09C, 143037},
  {0x1D09D, 143227},
  {0x1D09E, 143191},
  {0x1D09F, 143113},
  {0x1D0A0, 143150},
  {0x1D0A1, 143306},
  {0x1D0A2, 149180},
  {0x1D0A3, 148926},
  {0x1D0A4, 148978},
  {0x1D0A5, 148872},
  {0x1D0A6, 149428},
  {0x1D0A7, 149384},
  {0x1D0A8, 149278},
  {0x1D0A9, 149327},
  {0x1D0AA, 149027},
  {0x1D0AB, 149075},
  {0x1D0AC, 147421},
  {0x1D0AD, 143706},
  {0x1D0AE, 145838},
  {0x1D0AF, 145792},
  {0x1D0B0, 145748},
  {0x1D0B1, 149475},
  {0x1D0B2, 149227},
  {0x1D0B3, 149123},
  {0x1D0B4, 147118},
  {0x1D0B5, 147163},
  {0x1D0B6, 145500},
  {0x1D0B7, 147316},
  {0x1D0B8, 147385},
  {0x1D0B9, 145979},
  {0x1D0BA, 146271},
  {0x1D0BB, 146125},
  {0x1D0BC, 146583},
  {0x1D0BD, 146035},
  {0x1D0BE, 146719},
  {0x1D0BF, 146080},
  {0x1D0C0, 146316},
  {0x1D0C1, 146221},
  {0x1D0C2, 146172},
  {0x1D0C3, 146469},
  {0x1D0C4, 146525},
  {0x1D0C5, 145885},
  {0x1D0C6, 146665},
  {0x1D0C7, 146626},
  {0x1D0C8, 144458},
  {0x1D0C9, 144382},
  {0x1D0CA, 144420},
  {0x1D0CB, 146414},
  {0x1D0CC, 146361},
  {0x1D0CD, 152920},
  {0x1D0CE, 145001},
  {0x1D0CF, 144898},
  {0x1D0D0, 144733},
  {0x1D0D1, 144837},
  {0x1D0D2, 144783},
  {0x1D0D3, 144944},
  {0x1D0D4, 152698},
  {0x1D0D5, 152802},
  {0x1D0D6, 152748},
  {0x1D0D7, 152863},
  {0x1D0D8, 146771},
  {0x1D0D9, 146810},
  {0x1D0DA, 144606},
  {0x1D0DB, 144560},
  {0x1D0DC, 144651},
  {0x1D0DD, 144691},
  {0x1D0DE, 151096},
  {0x1D0DF, 151138},
  {0x1D0E0, 151241},
  {0x1D0E1, 151188},
  {0x1D0E2, 150904},
  {0x1D0E3, 150945},
  {0x1D0E4, 151046},
  {0x1D0E5, 150994},
  {0x1D0E6, 145294},
  {0x1D0E7, 145045},
  {0x1D0E8, 151367},
  {0x1D0E9, 144125},
  {0x1D0EA, 144161},
  {0x1D0EB, 144017},
  {0x1D0EC, 143981},
  {0x1D0ED, 144053},
  {0x1D0EE, 144198},
  {0x1D0EF, 144089},
  {0x1D0F0, 147986},
  {0x1D0F1, 147778},
  {0x1D0F2, 147942},
  {0x1D0F3, 147736},
  {0x1D0F4, 148098},
  {0x1D0F5, 146930},
  {0x1D100, 749939},
  {0x1D101, 746211},
  {0x1D102, 746673},
  {0x1D103, 749495},
  {0x1D104, 746126},
  {0x1D105, 749910},
  {0x1D106, 747868},
  {0x1D107, 749532},
  {0x1D108, 749369},
  {0x1D109, 746057},
  {0x1D10A, 746034},
  {0x1D10B, 749649},
  {0x1D10C, 744885},
  {0x1D10D, 749396},
  {0x1D10E, 749429},
  {0x1D10F, 749462},
  {0x1D110, 746621},
  {0x1D111, 746644},
  {0x1D112, 744588},
  {0x1D113, 744707},
  {0x1D114, 744544},
  {0x1D115, 744565},
  {0x1D116, 748478},
  {0x1D117, 751612},
  {0x1D118, 750889},
  {0x1D119, 746904},
  {0x1D11A, 746804},
  {0x1D11B, 749969},
  {0x1D11C, 749999},
  {0x1D11D, 746935},
  {0x1D11E, 747024},
  {0x1D11F, 747046},
  {0x1D120, 747080},
  {0x1D121, 744685},
  {0x1D122, 746530},
  {0x1D123, 746552},
  {0x1D124, 746586},
  {0x1D125, 746296},
  {0x1D126, 746323},
  {0x1D129, 748209},
  {0x1D12A, 746268},
  {0x1D12B, 746241},
  {0x1D12C, 746860},
  {0x1D12D, 746835},
  {0x1D12E, 748274},
  {0x1D12F, 748246},
  {0x1D130, 749886},
  {0x1D131, 749860},
  {0x1D132, 749268},
  {0x1D133, 749235},
  {0x1D134, 745937},
  {0x1D135, 746010},
  {0x1D136, 748873},
  {0x1D137, 748900},
  {0x1D138, 749302},
  {0x1D139, 749335},
  {0x1D13A, 748183},
  {0x1D13B, 751718},
  {0x1D13C, 747360},
  {0x1D13D, 749207},
  {0x1D13E, 746377},
  {0x1D13F, 750065},
  {0x1D140, 750855},
  {0x1D141, 750128},
  {0x1D142, 748432},
  {0x1D143, 751776},
  {0x1D144, 749070},
  {0x1D145, 744730},
  {0x1D146, 750242},
  {0x1D147, 750205},
  {0x1D148, 751401},
  {0x1D149, 751263},
  {0x1D14A, 751129},
  {0x1D14B, 751085},
  {0x1D14C, 751218},
  {0x1D14D, 751173},
  {0x1D14E, 751041},
  {0x1D14F, 750997},
  {0x1D150, 751353},
  {0x1D151, 751305},
  {0x1D152, 748148},
  {0x1D153, 748113},
  {0x1D154, 751493},
  {0x1D155, 751443},
  {0x1D156, 748928},
  {0x1D157, 751663},
  {0x1D158, 748327},
  {0x1D159, 748357},
  {0x1D15A, 744847},
  {0x1D15B, 744809},
  {0x1D15C, 744615},
  {0x1D15D, 751692},
  {0x1D15E, 747304},
  {0x1D15F, 749179},
  {0x1D160, 746350},
  {0x1D161, 750035},
  {0x1D162, 750821},
  {0x1D163, 750095},
  {0x1D164, 748386},
  {0x1D165, 745699},
  {0x1D166, 745583},
  {0x1D167, 745761},
  {0x1D168, 745796},
  {0x1D169, 745831},
  {0x1D16A, 746702},
  {0x1D16B, 746736},
  {0x1D16C, 746770},
  {0x1D16D, 744978},
  {0x1D16E, 745153},
  {0x1D16F, 745185},
  {0x1D170, 745217},
  {0x1D171, 745249},
  {0x1D172, 745281},
  {0x1D173, 744439},
  {0x1D174, 746404},
  {0x1D175, 744519},
  {0x1D176, 746507},
  {0x1D177, 744493},
  {0x1D178, 746483},
  {0x1D179, 744465},
  {0x1D17A, 746457},
  {0x1D17B, 744905},
  {0x1D17C, 745665},
  {0x1D17D, 745729},
  {0x1D17E, 745626},
  {0x1D17F, 745408},
  {0x1D180, 745441},
  {0x1D181, 744937},
  {0x1D182, 745377},
  {0x1D183, 744410},
  {0x1D184, 744379},
  {0x1D185, 745050},
  {0x1D186, 745483},
  {0x1D187, 745313},
  {0x1D188, 745512},
  {0x1D189, 745020},
  {0x1D18A, 745080},
  {0x1D18B, 745866},
  {0x1D18C, 749565},
  {0x1D18D, 750279},
  {0x1D18E, 751802},
  {0x1D18F, 749049},
  {0x1D190, 748015},
  {0x1D191, 746883},
  {0x1D192, 745964},
  {0x1D193, 746156},
  {0x1D194, 747208},
  {0x1D195, 747173},
  {0x1D196, 750979},
  {0x1D197, 751543},
  {0x1D198, 747412},
  {0x1D199, 751563},
  {0x1D19A, 751589},
  {0x1D19B, 748508},
  {0x1D19C, 748609},
  {0x1D19D, 748642},
  {0x1D19E, 748675},
  {0x1D19F, 748708},
  {0x1D1A0, 748741},
  {0x1D1A1, 748774},
  {0x1D1A2, 748807},
  {0x1D1A3, 748840},
  {0x1D1A4, 748541},
  {0x1D1A5, 748575},
  {0x1D1A6, 747385},
  {0x1D1A7, 748300},
  {0x1D1A8, 746428},
  {0x1D1A9, 746183},
  {0x1D1AA, 745119},
  {0x1D1AB, 745905},
  {0x1D1AC, 745343},
  {0x1D1AD, 745543},
  {0x1D1AE, 748964},
  {0x1D1AF, 748990},
  {0x1D1B0, 747329},
  {0x1D1B1, 747145},
  {0x1D1B2, 747115},
  {0x1D1B3, 751744},
  {0x1D1B4, 746082},
  {0x1D1B5, 746102},
  {0x1D1B6, 747993},
  {0x1D1B7, 747900},
  {0x1D1B8, 744636},
  {0x1D1B9, 749733},
  {0x1D1BA, 749670},
  {0x1D1BB, 748036},
  {0x1D1BC, 748058},
  {0x1D1BD, 749828},
  {0x1D1BE, 749765},
  {0x1D1BF, 746998},
  {0x1D1C0, 746972},
  {0x1D1C1, 747958},
  {0x1D1C2, 747921},
  {0x1D1C3, 744658},
  {0x1D1C4, 749702},
  {0x1D1C5, 748086},
  {0x1D1C6, 749797},
  {0x1D1C7, 750696},
  {0x1D1C8, 750638},
  {0x1D1C9, 750752},
  {0x1D1CA, 750580},
  {0x1D1CB, 750301},
  {0x1D1CC, 750361},
  {0x1D1CD, 750434},
  {0x1D1CE, 750507},
  {0x1D1CF, 745989},
  {0x1D1D0, 747240},
  {0x1D1D1, 747272},
  {0x1D1D2, 750181},
  {0x1D1D3, 751642},
  {0x1D1D4, 749099},
  {0x1D1D5, 744787},
  {0x1D1D6, 749592},
  {0x1D1D7, 744763},
  {0x1D1D8, 750921},
  {0x1D1D9, 749122},
  {0x1D1DA, 749147},
  {0x1D1DB, 749617},
  {0x1D1DC, 750945},
  {0x1D1DD, 749019},
  {0x1D1DE, 747441},
  {0x1D1DF, 747556},
  {0x1D1E0, 747591},
  {0x1D1E1, 747776},
  {0x1D1E2, 747814},
  {0x1D1E3, 747656},
  {0x1D1E4, 747688},
  {0x1D1E5, 747733},
  {0x1D1E6, 747470},
  {0x1D1E7, 747514},
  {0x1D1E8, 747624},
  {0x1D1E9, 750161},
  {0x1D1EA, 747847},
  {0x1D200, 479544},
  {0x1D201, 479884},
  {0x1D202, 480069},
  {0x1D203, 480099},
  {0x1D204, 480129},
  {0x1D205, 480314},
  {0x1D206, 480344},
  {0x1D207, 480374},
  {0x1D208, 480404},
  {0x1D209, 479574},
  {0x1D20A, 479605},
  {0x1D20B, 479636},
  {0x1D20C, 479667},
  {0x1D20D, 479698},
  {0x1D20E, 479729},
  {0x1D20F, 479760},
  {0x1D210, 479791},
  {0x1D211, 479822},
  {0x1D212, 479853},
  {0x1D213, 479914},
  {0x1D214, 479945},
  {0x1D215, 479976},
  {0x1D216, 480007},
  {0x1D217, 480038},
  {0x1D218, 480159},
  {0x1D219, 480190},
  {0x1D21A, 480221},
  {0x1D21B, 480252},
  {0x1D21C, 480283},
  {0x1D21D, 469517},
  {0x1D21E, 469820},
  {0x1D21F, 470313},
  {0x1D220, 470616},
  {0x1D221, 470843},
  {0x1D222, 470880},
  {0x1D223, 469554},
  {0x1D224, 469592},
  {0x1D225, 469630},
  {0x1D226, 469668},
  {0x1D227, 469706},
  {0x1D228, 469744},
  {0x1D229, 469782},
  {0x1D22A, 469857},
  {0x1D22B, 469895},
  {0x1D22C, 469933},
  {0x1D22D, 469971},
  {0x1D22E, 470009},
  {0x1D22F, 470047},
  {0x1D230, 470085},
  {0x1D231, 470123},
  {0x1D232, 470161},
  {0x1D233, 470199},
  {0x1D234, 470237},
  {0x1D235, 470275},
  {0x1D236, 470350},
  {0x1D237, 470388},
  {0x1D238, 470426},
  {0x1D239, 470464},
  {0x1D23A, 470502},
  {0x1D23B, 470540},
  {0x1D23C, 470578},
  {0x1D23D, 470653},
  {0x1D23E, 470691},
  {0x1D23F, 470729},
  {0x1D240, 470767},
  {0x1D241, 470805},
  {0x1D242, 203159},
  {0x1D243, 203125},
  {0x1D244, 203091},
  {0x1D245, 471421},
  {0x1D2C0, 540247},
  {0x1D2C1, 540015},
  {0x1D2C2, 540226},
  {0x1D2C3, 540179},
  {0x1D2C4, 539919},
  {0x1D2C5, 539897},
  {0x1D2C6, 540086},
  {0x1D2C7, 540036},
  {0x1D2C8, 539799},
  {0x1D2C9, 539967},
  {0x1D2CA, 540132},
  {0x1D2CB, 539848},
  {0x1D2CC, 540202},
  {0x1D2CD, 540153},
  {0x1D2CE, 539941},
  {0x1D2CF, 539872},
  {0x1D2D0, 540107},
  {0x1D2D1, 540059},
  {0x1D2D2, 539822},
  {0x1D2D3, 539989},
  {0x1D2E0, 704866},
  {0x1D2E1, 704664},
  {0x1D2E2, 704848},
  {0x1D2E3, 704807},
  {0x1D2E4, 704580},
  {0x1D2E5, 704561},
  {0x1D2E6, 704726},
  {0x1D2E7, 704682},
  {0x1D2E8, 704475},
  {0x1D2E9, 704622},
  {0x1D2EA, 704766},
  {0x1D2EB, 704518},
  {0x1D2EC, 704827},
  {0x1D2ED, 704784},
  {0x1D2EE, 704599},
  {0x1D2EF, 704539},
  {0x1D2F0, 704744},
  {0x1D2F1, 704702},
  {0x1D2F2, 704495},
  {0x1D2F3, 704641},
  {0x1D300, 742173},
  {0x1D301, 277311},
  {0x1D302, 277337},
  {0x1D303, 277213},
  {0x1D304, 277239},
  {0x1D305, 277196},
  {0x1D306, 932504},
  {0x1D307, 933286},
  {0x1D308, 933741},
  {0x1D309, 932424},
  {0x1D30A, 933575},
  {0x1D30B, 932740},
  {0x1D30C, 932403},
  {0x1D30D, 933788},
  {0x1D30E, 932476},
  {0x1D30F, 932817},
  {0x1D310, 932959},
  {0x1D311, 934247},
  {0x1D312, 933514},
  {0x1D313, 933857},
  {0x1D314, 933904},
  {0x1D315, 932693},
  {0x1D316, 933487},
  {0x1D317, 934205},
  {0x1D318, 933238},
  {0x1D319, 932355},
  {0x1D31A, 933924},
  {0x1D31B, 933970},
  {0x1D31C, 933025},
  {0x1D31D, 933557},
  {0x1D31E, 932715},
  {0x1D31F, 933097},
  {0x1D320, 933004},
  {0x1D321, 932525},
  {0x1D322, 932790},
  {0x1D323, 932446},
  {0x1D324, 933813},
  {0x1D325, 933676},
  {0x1D326, 932573},
  {0x1D327, 933603},
  {0x1D328, 933335},
  {0x1D329, 934128},
  {0x1D32A, 933883},
  {0x1D32B, 933312},
  {0x1D32C, 933946},
  {0x1D32D, 933649},
  {0x1D32E, 933995},
  {0x1D32F, 933386},
  {0x1D330, 933072},
  {0x1D331, 934108},
  {0x1D332, 933414},
  {0x1D333, 933121},
  {0x1D334, 933835},
  {0x1D335, 934018},
  {0x1D336, 933217},
  {0x1D337, 934171},
  {0x1D338, 932669},
  {0x1D339, 933719},
  {0x1D33A, 933147},
  {0x1D33B, 934151},
  {0x1D33C, 932910},
  {0x1D33D, 932546},
  {0x1D33E, 933438},
  {0x1D33F, 933359},
  {0x1D340, 933697},
  {0x1D341, 932328},
  {0x1D342, 933044},
  {0x1D343, 932984},
  {0x1D344, 934227},
  {0x1D345, 934063},
  {0x1D346, 933537},
  {0x1D347, 932859},
  {0x1D348, 932766},
  {0x1D349, 932937},
  {0x1D34A, 933170},
  {0x1D34B, 934039},
  {0x1D34C, 934085},
  {0x1D34D, 933464},
  {0x1D34E, 932619},
  {0x1D34F, 932597},
  {0x1D350, 933195},
  {0x1D351, 932377},
  {0x1D352, 932644},
  {0x1D353, 933761},
  {0x1D354, 932883},
  {0x1D355, 933625},
  {0x1D356, 933262},
  {0x1D360, 214646},
  {0x1D361, 214762},
  {0x1D362, 214732},
  {0x1D363, 214588},
  {0x1D364, 214559},
  {0x1D365, 214704},
  {0x1D366, 214674},
  {0x1D367, 214529},
  {0x1D368, 214617},
  {0x1D369, 214385},
  {0x1D36A, 214501},
  {0x1D36B, 214471},
  {0x1D36C, 214327},
  {0x1D36D, 214298},
  {0x1D36E, 214443},
  {0x1D36F, 214413},
  {0x1D370, 214268},
  {0x1D371, 214356},
  {0x1D372, 526400},
  {0x1D373, 526456},
  {0x1D374, 526427},
  {0x1D375, 526372},
  {0x1D376, 526344},
  {0x1D377, 925628},
  {0x1D378, 925612},
  {0x1D400, 668863},
  {0x1D401, 668923},
  {0x1D402, 668982},
  {0x1D403, 669040},
  {0x1D404, 669134},
  {0x1D405, 669226},
  {0x1D406, 669254},
  {0x1D407, 669314},
  {0x1D408, 669342},
  {0x1D409, 669401},
  {0x1D40A, 669429},
  {0x1D40B, 669489},
  {0x1D40C, 669549},
  {0x1D40D, 669606},
  {0x1D40E, 669663},
  {0x1D40F, 669757},
  {0x1D410, 669874},
  {0x1D411, 669902},
  {0x1D412, 669960},
  {0x1D413, 670020},
  {0x1D414, 670149},
  {0x1D415, 670211},
  {0x1D416, 670239},
  {0x1D417, 670267},
  {0x1D418, 670324},
  {0x1D419, 670352},
  {0x1D41A, 678441},
  {0x1D41B, 678497},
  {0x1D41C, 678552},
  {0x1D41D, 678606},
  {0x1D41E, 678694},
  {0x1D41F, 678780},
  {0x1D420, 678842},
  {0x1D421, 678898},
  {0x1D422, 678924},
  {0x1D423, 678979},
  {0x1D424, 679005},
  {0x1D425, 679061},
  {0x1D426, 679117},
  {0x1D427, 679170},
  {0x1D428, 679223},
  {0x1D429, 679311},
  {0x1D42A, 679420},
  {0x1D42B, 679446},
  {0x1D42C, 679500},
  {0x1D42D, 679556},
  {0x1D42E, 679640},
  {0x1D42F, 679698},
  {0x1D430, 679724},
  {0x1D431, 679750},
  {0x1D432, 679803},
  {0x1D433, 679829},
  {0x1D434, 683343},
  {0x1D435, 683407},
  {0x1D436, 683470},
  {0x1D437, 683532},
  {0x1D438, 683596},
  {0x1D439, 683694},
  {0x1D43A, 683724},
  {0x1D43B, 683788},
  {0x1D43C, 683818},
  {0x1D43D, 683881},
  {0x1D43E, 683911},
  {0x1D43F, 683975},
  {0x1D440, 684039},
  {0x1D441, 684100},
  {0x1D442, 684161},
  {0x1D443, 684261},
  {0x1D444, 684386},
  {0x1D445, 684416},
  {0x1D446, 684478},
  {0x1D447, 684542},
  {0x1D448, 684679},
  {0x1D449, 684745},
  {0x1D44A, 684775},
  {0x1D44B, 684805},
  {0x1D44C, 684866},
  {0x1D44D, 684896},
  {0x1D44E, 685186},
  {0x1D44F, 685246},
  {0x1D450, 685305},
  {0x1D451, 685363},
  {0x1D452, 685495},
  {0x1D453, 685587},
  {0x1D454, 685653},
  {0x1D456, 685713},
  {0x1D457, 685772},
  {0x1D458, 685800},
  {0x1D459, 685860},
  {0x1D45A, 685920},
  {0x1D45B, 685977},
  {0x1D45C, 686034},
  {0x1D45D, 686128},
  {0x1D45E, 686245},
  {0x1D45F, 686273},
  {0x1D460, 686331},
  {0x1D461, 686391},
  {0x1D462, 686481},
  {0x1D463, 686543},
  {0x1D464, 686571},
  {0x1D465, 686599},
  {0x1D466, 686656},
  {0x1D467, 686684},
  {0x1D468, 672554},
  {0x1D469, 672628},
  {0x1D46A, 672701},
  {0x1D46B, 672773},
  {0x1D46C, 672847},
  {0x1D46D, 672960},
  {0x1D46E, 672995},
  {0x1D46F, 673069},
  {0x1D470, 673104},
  {0x1D471, 673177},
  {0x1D472, 673212},
  {0x1D473, 673286},
  {0x1D474, 673360},
  {0x1D475, 673431},
  {0x1D476, 673502},
  {0x1D477, 673617},
  {0x1D478, 673762},
  {0x1D479, 673797},
  {0x1D47A, 673869},
  {0x1D47B, 673943},
  {0x1D47C, 674100},
  {0x1D47D, 674176},
  {0x1D47E, 674211},
  {0x1D47F, 674246},
  {0x1D480, 674317},
  {0x1D481, 674352},
  {0x1D482, 674687},
  {0x1D483, 674757},
  {0x1D484, 674826},
  {0x1D485, 674894},
  {0x1D486, 674964},
  {0x1D487, 675071},
  {0x1D488, 675147},
  {0x1D489, 675217},
  {0x1D48A, 675250},
  {0x1D48B, 675319},
  {0x1D48C, 675352},
  {0x1D48D, 675422},
  {0x1D48E, 675492},
  {0x1D48F, 675559},
  {0x1D490, 675626},
  {0x1D491, 675735},
  {0x1D492, 675872},
  {0x1D493, 675905},
  {0x1D494, 675973},
  {0x1D495, 676043},
  {0x1D496, 676148},
  {0x1D497, 676220},
  {0x1D498, 676253},
  {0x1D499, 676286},
  {0x1D49A, 676353},
  {0x1D49B, 676386},
  {0x1D49C, 703282},
  {0x1D49E, 703312},
  {0x1D49F, 703342},
  {0x1D4A2, 703372},
  {0x1D4A5, 703402},
  {0x1D4A6, 703432},
  {0x1D4A9, 703462},
  {0x1D4AA, 703492},
  {0x1D4AB, 703522},
  {0x1D4AC, 703552},
  {0x1D4AE, 703582},
  {0x1D4AF, 703612},
  {0x1D4B0, 703642},
  {0x1D4B1, 703672},
  {0x1D4B2, 703702},
  {0x1D4B3, 703732},
  {0x1D4B4, 703762},
  {0x1D4B5, 703792},
  {0x1D4B6, 703822},
  {0x1D4B7, 703850},
  {0x1D4B8, 703878},
  {0x1D4B9, 703906},
  {0x1D4BB, 703934},
  {0x1D4BD, 703962},
  {0x1D4BE, 703990},
  {0x1D4BF, 704018},
  {0x1D4C0, 704046},
  {0x1D4C1, 704074},
  {0x1D4C2, 704102},
  {0x1D4C3, 704130},
  {0x1D4C5, 704158},
  {0x1D4C6, 704186},
  {0x1D4C7, 704214},
  {0x1D4C8, 704242},
  {0x1D4C9, 704270},
  {0x1D4CA, 704298},
  {0x1D4CB, 704326},
  {0x1D4CC, 704354},
  {0x1D4CD, 704382},
  {0x1D4CE, 704410},
  {0x1D4CF, 704438},
  {0x1D4D0, 676673},
  {0x1D4D1, 676708},
  {0x1D4D2, 676743},
  {0x1D4D3, 676778},
  {0x1D4D4, 676813},
  {0x1D4D5, 676848},
  {0x1D4D6, 676883},
  {0x1D4D7, 676918},
  {0x1D4D8, 676953},
  {0x1D4D9, 676988},
  {0x1D4DA, 677023},
  {0x1D4DB, 677058},
  {0x1D4DC, 677093},
  {0x1D4DD, 677128},
  {0x1D4DE, 677163},
  {0x1D4DF, 677198},
  {0x1D4E0, 677233},
  {0x1D4E1, 677268},
  {0x1D4E2, 677303},
  {0x1D4E3, 677338},
  {0x1D4E4, 677373},
  {0x1D4E5, 677408},
  {0x1D4E6, 677443},
  {0x1D4E7, 677478},
  {0x1D4E8, 677513},
  {0x1D4E9, 677548},
  {0x1D4EA, 677583},
  {0x1D4EB, 677616},
  {0x1D4EC, 677649},
  {0x1D4ED, 677682},
  {0x1D4EE, 677715},
  {0x1D4EF, 677748},
  {0x1D4F0, 677781},
  {0x1D4F1, 677814},
  {0x1D4F2, 677847},
  {0x1D4F3, 677880},
  {0x1D4F4, 677913},
  {0x1D4F5, 677946},
  {0x1D4F6, 677979},
  {0x1D4F7, 678012},
  {0x1D4F8, 678045},
  {0x1D4F9, 678078},
  {0x1D4FA, 678111},
  {0x1D4FB, 678144},
  {0x1D4FC, 678177},
  {0x1D4FD, 678210},
  {0x1D4FE, 678243},
  {0x1D4FF, 678276},
  {0x1D500, 678309},
  {0x1D501, 678342},
  {0x1D502, 678375},
  {0x1D503, 678408},
  {0x1D504, 681938},
  {0x1D505, 681969},
  {0x1D507, 682000},
  {0x1D508, 682031},
  {0x1D509, 682062},
  {0x1D50A, 682093},
  {0x1D50D, 682124},
  {0x1D50E, 682155},
  {0x1D50F, 682186},
  {0x1D510, 682217},
  {0x1D511, 682248},
  {0x1D512, 682279},
  {0x1D513, 682310},
  {0x1D514, 682341},
  {0x1D516, 682372},
  {0x1D517, 682403},
  {0x1D518, 682434},
  {0x1D519, 682465},
  {0x1D51A, 682496},
  {0x1D51B, 682527},
  {0x1D51C, 682558},
  {0x1D51E, 682589},
  {0x1D51F, 682618},
  {0x1D520, 682647},
  {0x1D521, 682676},
  {0x1D522, 682705},
  {0x1D523, 682734},
  {0x1D524, 682763},
  {0x1D525, 682792},
  {0x1D526, 682821},
  {0x1D527, 682850},
  {0x1D528, 682879},
  {0x1D529, 682908},
  {0x1D52A, 682937},
  {0x1D52B, 682966},
  {0x1D52C, 682995},
  {0x1D52D, 683024},
  {0x1D52E, 683053},
  {0x1D52F, 683082},
  {0x1D530, 683111},
  {0x1D531, 683140},
  {0x1D532, 683169},
  {0x1D533, 683198},
  {0x1D534, 683227},
  {0x1D535, 683256},
  {0x1D536, 683285},
  {0x1D537, 683314},
  {0x1D538, 679915},
  {0x1D539, 679952},
  {0x1D53B, 679989},
  {0x1D53C, 680026},
  {0x1D53D, 680063},
  {0x1D53E, 680100},
  {0x1D540, 680137},
  {0x1D541, 680174},
  {0x1D542, 680211},
  {0x1D543, 680248},
  {0x1D544, 680285},
  {0x1D546, 680322},
  {0x1D54A, 680359},
  {0x1D54B, 680396},
  {0x1D54C, 680433},
  {0x1D54D, 680470},
  {0x1D54E, 680507},
  {0x1D54F, 680544},
  {0x1D550, 680581},
  {0x1D552, 680998},
  {0x1D553, 681033},
  {0x1D554, 681068},
  {0x1D555, 681103},
  {0x1D556, 681138},
  {0x1D557, 681173},
  {0x1D558, 681208},
  {0x1D559, 681243},
  {0x1D55A, 681278},
  {0x1D55B, 681313},
  {0x1D55C, 681348},
  {0x1D55D, 681383},
  {0x1D55E, 681418},
  {0x1D55F, 681453},
  {0x1D560, 681488},
  {0x1D561, 681523},
  {0x1D562, 681558},
  {0x1D563, 681593},
  {0x1D564, 681628},
  {0x1D565, 681663},
  {0x1D566, 681698},
  {0x1D567, 681733},
  {0x1D568, 681768},
  {0x1D569, 681803},
  {0x1D56A, 681838},
  {0x1D56B, 681873},
  {0x1D56C, 670734},
  {0x1D56D, 670770},
  {0x1D56E, 670806},
  {0x1D56F, 670842},
  {0x1D570, 670878},
  {0x1D571, 670914},
  {0x1D572, 670950},
  {0x1D573, 670986},
  {0x1D574, 671022},
  {0x1D575, 671058},
  {0x1D576, 671094},
  {0x1D577, 671130},
  {0x1D578, 671166},
  {0x1D579, 671202},
  {0x1D57A, 671238},
  {0x1D57B, 671274},
  {0x1D57C, 671310},
  {0x1D57D, 671346},
  {0x1D57E, 671382},
  {0x1D57F, 671418},
  {0x1D580, 671454},
  {0x1D581, 671490},
  {0x1D582, 671526},
  {0x1D583, 671562},
  {0x1D584, 671598},
  {0x1D585, 671634},
  {0x1D586, 671670},
  {0x1D587, 671704},
  {0x1D588, 671738},
  {0x1D589, 671772},
  {0x1D58A, 671806},
  {0x1D58B, 671840},
  {0x1D58C, 671874},
  {0x1D58D, 671908},
  {0x1D58E, 671942},
  {0x1D58F, 671976},
  {0x1D590, 672010},
  {0x1D591, 672044},
  {0x1D592, 672078},
  {0x1D593, 672112},
  {0x1D594, 672146},
  {0x1D595, 672180},
  {0x1D596, 672214},
  {0x1D597, 672248},
  {0x1D598, 672282},
  {0x1D599, 672316},
  {0x1D59A, 672350},
  {0x1D59B, 672384},
  {0x1D59C, 672418},
  {0x1D59D, 672452},
  {0x1D59E, 672486},
  {0x1D59F, 672520},
  {0x1D5A0, 699136},
  {0x1D5A1, 699170},
  {0x1D5A2, 699204},
  {0x1D5A3, 699238},
  {0x1D5A4, 699272},
  {0x1D5A5, 699306},
  {0x1D5A6, 699340},
  {0x1D5A7, 699374},
  {0x1D5A8, 699408},
  {0x1D5A9, 699442},
  {0x1D5AA, 699476},
  {0x1D5AB, 699510},
  {0x1D5AC, 699544},
  {0x1D5AD, 699578},
  {0x1D5AE, 699612},
  {0x1D5AF, 699646},
  {0x1D5B0, 699680},
  {0x1D5B1, 699714},
  {0x1D5B2, 699748},
  {0x1D5B3, 699782},
  {0x1D5B4, 699816},
  {0x1D5B5, 699850},
  {0x1D5B6, 699884},
  {0x1D5B7, 699918},
  {0x1D5B8, 699952},
  {0x1D5B9, 699986},
  {0x1D5BA, 702450},
  {0x1D5BB, 702482},
  {0x1D5BC, 702514},
  {0x1D5BD, 702546},
  {0x1D5BE, 702578},
  {0x1D5BF, 702610},
  {0x1D5C0, 702642},
  {0x1D5C1, 702674},
  {0x1D5C2, 702706},
  {0x1D5C3, 702738},
  {0x1D5C4, 702770},
  {0x1D5C5, 702802},
  {0x1D5C6, 702834},
  {0x1D5C7, 702866},
  {0x1D5C8, 702898},
  {0x1D5C9, 702930},
  {0x1D5CA, 702962},
  {0x1D5CB, 702994},
  {0x1D5CC, 703026},
  {0x1D5CD, 703058},
  {0x1D5CE, 703090},
  {0x1D5CF, 703122},
  {0x1D5D0, 703154},
  {0x1D5D1, 703186},
  {0x1D5D2, 703218},
  {0x1D5D3, 703250},
  {0x1D5D4, 689208},
  {0x1D5D5, 689290},
  {0x1D5D6, 689371},
  {0x1D5D7, 689451},
  {0x1D5D8, 689533},
  {0x1D5D9, 689658},
  {0x1D5DA, 689697},
  {0x1D5DB, 689779},
  {0x1D5DC, 689818},
  {0x1D5DD, 689899},
  {0x1D5DE, 689938},
  {0x1D5DF, 690020},
  {0x1D5E0, 690102},
  {0x1D5E1, 690181},
  {0x1D5E2, 690260},
  {0x1D5E3, 690387},
  {0x1D5E4, 690548},
  {0x1D5E5, 690587},
  {0x1D5E6, 690667},
  {0x1D5E7, 690749},
  {0x1D5E8, 690922},
  {0x1D5E9, 691006},
  {0x1D5EA, 691045},
  {0x1D5EB, 691084},
  {0x1D5EC, 691163},
  {0x1D5ED, 691202},
  {0x1D5EE, 697122},
  {0x1D5EF, 697200},
  {0x1D5F0, 697277},
  {0x1D5F1, 697353},
  {0x1D5F2, 697431},
  {0x1D5F3, 697550},
  {0x1D5F4, 697634},
  {0x1D5F5, 697712},
  {0x1D5F6, 697749},
  {0x1D5F7, 697826},
  {0x1D5F8, 697863},
  {0x1D5F9, 697941},
  {0x1D5FA, 698019},
  {0x1D5FB, 698094},
  {0x1D5FC, 698169},
  {0x1D5FD, 698290},
  {0x1D5FE, 698443},
  {0x1D5FF, 698480},
  {0x1D600, 698556},
  {0x1D601, 698634},
  {0x1D602, 698751},
  {0x1D603, 698831},
  {0x1D604, 698868},
  {0x1D605, 698905},
  {0x1D606, 698980},
  {0x1D607, 699017},
  {0x1D608, 700370},
  {0x1D609, 700411},
  {0x1D60A, 700452},
  {0x1D60B, 700493},
  {0x1D60C, 700534},
  {0x1D60D, 700575},
  {0x1D60E, 700616},
  {0x1D60F, 700657},
  {0x1D610, 700698},
  {0x1D611, 700739},
  {0x1D612, 700780},
  {0x1D613, 700821},
  {0x1D614, 700862},
  {0x1D615, 700903},
  {0x1D616, 700944},
  {0x1D617, 700985},
  {0x1D618, 701026},
  {0x1D619, 701067},
  {0x1D61A, 701108},
  {0x1D61B, 701149},
  {0x1D61C, 701190},
  {0x1D61D, 701231},
  {0x1D61E, 701272},
  {0x1D61F, 701313},
  {0x1D620, 701354},
  {0x1D621, 701395},
  {0x1D622, 701436},
  {0x1D623, 701475},
  {0x1D624, 701514},
  {0x1D625, 701553},
  {0x1D626, 701592},
  {0x1D627, 701631},
  {0x1D628, 701670},
  {0x1D629, 701709},
  {0x1D62A, 701748},
  {0x1D62B, 701787},
  {0x1D62C, 701826},
  {0x1D62D, 701865},
  {0x1D62E, 701904},
  {0x1D62F, 701943},
  {0x1D630, 701982},
  {0x1D631, 702021},
  {0x1D632, 702060},
  {0x1D633, 702099},
  {0x1D634, 702138},
  {0x1D635, 702177},
  {0x1D636, 702216},
  {0x1D637, 702255},
  {0x1D638, 702294},
  {0x1D639, 702333},
  {0x1D63A, 702372},
  {0x1D63B, 702411},
  {0x1D63C, 691727},
  {0x1D63D, 691823},
  {0x1D63E, 691918},
  {0x1D63F, 692012},
  {0x1D640, 692108},
  {0x1D641, 692254},
  {0x1D642, 692300},
  {0x1D643, 692396},
  {0x1D644, 692442},
  {0x1D645, 692537},
  {0x1D646, 692583},
  {0x1D647, 692679},
  {0x1D648, 692775},
  {0x1D649, 692868},
  {0x1D64A, 692961},
  {0x1D64B, 693109},
  {0x1D64C, 693298},
  {0x1D64D, 693344},
  {0x1D64E, 693438},
  {0x1D64F, 693534},
  {0x1D650, 693735},
  {0x1D651, 693833},
  {0x1D652, 693879},
  {0x1D653, 693925},
  {0x1D654, 694018},
  {0x1D655, 694064},
  {0x1D656, 694498},
  {0x1D657, 694590},
  {0x1D658, 694681},
  {0x1D659, 694771},
  {0x1D65A, 694863},
  {0x1D65B, 695003},
  {0x1D65C, 695101},
  {0x1D65D, 695193},
  {0x1D65E, 695237},
  {0x1D65F, 695328},
  {0x1D660, 695372},
  {0x1D661, 695464},
  {0x1D662, 695556},
  {0x1D663, 695645},
  {0x1D664, 695734},
  {0x1D665, 695876},
  {0x1D666, 696057},
  {0x1D667, 696101},
  {0x1D668, 696191},
  {0x1D669, 696283},
  {0x1D66A, 696421},
  {0x1D66B, 696515},
  {0x1D66C, 696559},
  {0x1D66D, 696603},
  {0x1D66E, 696692},
  {0x1D66F, 696736},
  {0x1D670, 686973},
  {0x1D671, 687006},
  {0x1D672, 687039},
  {0x1D673, 687072},
  {0x1D674, 687105},
  {0x1D675, 687138},
  {0x1D676, 687171},
  {0x1D677, 687204},
  {0x1D678, 687237},
  {0x1D679, 687270},
  {0x1D67A, 687303},
  {0x1D67B, 687336},
  {0x1D67C, 687369},
  {0x1D67D, 687402},
  {0x1D67E, 687435},
  {0x1D67F, 687468},
  {0x1D680, 687501},
  {0x1D681, 687534},
  {0x1D682, 687567},
  {0x1D683, 687600},
  {0x1D684, 687633},
  {0x1D685, 687666},
  {0x1D686, 687699},
  {0x1D687, 687732},
  {0x1D688, 687765},
  {0x1D689, 687798},
  {0x1D68A, 688171},
  {0x1D68B, 688202},
  {0x1D68C, 688233},
  {0x1D68D, 688264},
  {0x1D68E, 688295},
  {0x1D68F, 688326},
  {0x1D690, 688357},
  {0x1D691, 688388},
  {0x1D692, 688419},
  {0x1D693, 688450},
  {0x1D694, 688481},
  {0x1D695, 688512},
  {0x1D696, 688543},
  {0x1D697, 688574},
  {0x1D698, 688605},
  {0x1D699, 688636},
  {0x1D69A, 688667},
  {0x1D69B, 688698},
  {0x1D69C, 688729},
  {0x1D69D, 688760},
  {0x1D69E, 688791},
  {0x1D69F, 688822},
  {0x1D6A0, 688853},
  {0x1D6A1, 688884},
  {0x1D6A2, 688915},
  {0x1D6A3, 688946},
  {0x1D6A4, 685423},
  {0x1D6A5, 685459},
  {0x1D6A8, 668891},
  {0x1D6A9, 668951},
  {0x1D6AA, 669282},
  {0x1D6AB, 669068},
  {0x1D6AC, 669162},
  {0x1D6AD, 670380},
  {0x1D6AE, 669196},
  {0x1D6AF, 670078},
  {0x1D6B0, 669370},
  {0x1D6B1, 669457},
  {0x1D6B2, 669517},
  {0x1D6B3, 669577},
  {0x1D6B4, 669634},
  {0x1D6B5, 670295},
  {0x1D6B6, 669723},
  {0x1D6B7, 669815},
  {0x1D6B8, 669930},
  {0x1D6B9, 670110},
  {0x1D6BA, 669988},
  {0x1D6BB, 670048},
  {0x1D6BC, 670177},
  {0x1D6BD, 669785},
  {0x1D6BE, 669010},
  {0x1D6BF, 669844},
  {0x1D6C0, 669691},
  {0x1D6C1, 676524},
  {0x1D6C2, 678467},
  {0x1D6C3, 678523},
  {0x1D6C4, 678868},
  {0x1D6C5, 678632},
  {0x1D6C6, 678720},
  {0x1D6C7, 679855},
  {0x1D6C8, 678752},
  {0x1D6C9, 679610},
  {0x1D6CA, 678950},
  {0x1D6CB, 679031},
  {0x1D6CC, 679087},
  {0x1D6CD, 679143},
  {0x1D6CE, 679196},
  {0x1D6CF, 679776},
  {0x1D6D0, 679279},
  {0x1D6D1, 679365},
  {0x1D6D2, 679472},
  {0x1D6D3, 678806},
  {0x1D6D4, 679526},
  {0x1D6D5, 679582},
  {0x1D6D6, 679666},
  {0x1D6D7, 679337},
  {0x1D6D8, 678578},
  {0x1D6D9, 679392},
  {0x1D6DA, 679249},
  {0x1D6DB, 676548},
  {0x1D6DC, 670701},
  {0x1D6DD, 679884},
  {0x1D6DE, 676493},
  {0x1D6DF, 676587},
  {0x1D6E0, 676644},
  {0x1D6E1, 676616},
  {0x1D6E2, 683373},
  {0x1D6E3, 683437},
  {0x1D6E4, 683754},
  {0x1D6E5, 683562},
  {0x1D6E6, 683626},
  {0x1D6E7, 684926},
  {0x1D6E8, 683662},
  {0x1D6E9, 684604},
  {0x1D6EA, 683848},
  {0x1D6EB, 683941},
  {0x1D6EC, 684005},
  {0x1D6ED, 684069},
  {0x1D6EE, 684130},
  {0x1D6EF, 684835},
  {0x1D6F0, 684225},
  {0x1D6F1, 684323},
  {0x1D6F2, 684446},
  {0x1D6F3, 684638},
  {0x1D6F4, 684508},
  {0x1D6F5, 684572},
  {0x1D6F6, 684709},
  {0x1D6F7, 684291},
  {0x1D6F8, 683500},
  {0x1D6F9, 684354},
  {0x1D6FA, 684191},
  {0x1D6FB, 685027},
  {0x1D6FC, 685214},
  {0x1D6FD, 685274},
  {0x1D6FE, 685681},
  {0x1D6FF, 685391},
  {0x1D700, 685523},
  {0x1D701, 686712},
  {0x1D702, 685557},
  {0x1D703, 686449},
  {0x1D704, 685741},
  {0x1D705, 685828},
  {0x1D706, 685888},
  {0x1D707, 685948},
  {0x1D708, 686005},
  {0x1D709, 686627},
  {0x1D70A, 686094},
  {0x1D70B, 686186},
  {0x1D70C, 686301},
  {0x1D70D, 685615},
  {0x1D70E, 686359},
  {0x1D70F, 686419},
  {0x1D710, 686509},
  {0x1D711, 686156},
  {0x1D712, 685333},
  {0x1D713, 686215},
  {0x1D714, 686062},
  {0x1D715, 685053},
  {0x1D716, 684959},
  {0x1D717, 686743},
  {0x1D718, 684994},
  {0x1D719, 685094},
  {0x1D71A, 685155},
  {0x1D71B, 685125},
  {0x1D71C, 672589},
  {0x1D71D, 672663},
  {0x1D71E, 673030},
  {0x1D71F, 672808},
  {0x1D720, 672882},
  {0x1D721, 674387},
  {0x1D722, 672923},
  {0x1D723, 674015},
  {0x1D724, 673139},
  {0x1D725, 673247},
  {0x1D726, 673321},
  {0x1D727, 673395},
  {0x1D728, 673466},
  {0x1D729, 674281},
  {0x1D72A, 673576},
  {0x1D72B, 673689},
  {0x1D72C, 673832},
  {0x1D72D, 674054},
  {0x1D72E, 673904},
  {0x1D72F, 673978},
  {0x1D730, 674135},
  {0x1D731, 673652},
  {0x1D732, 672736},
  {0x1D733, 673725},
  {0x1D734, 673537},
  {0x1D735, 674503},
  {0x1D736, 674720},
  {0x1D737, 674790},
  {0x1D738, 675180},
  {0x1D739, 674927},
  {0x1D73A, 674997},
  {0x1D73B, 676419},
  {0x1D73C, 675036},
  {0x1D73D, 676111},
  {0x1D73E, 675283},
  {0x1D73F, 675385},
  {0x1D740, 675455},
  {0x1D741, 675525},
  {0x1D742, 675592},
  {0x1D743, 676319},
  {0x1D744, 675696},
  {0x1D745, 675803},
  {0x1D746, 675938},
  {0x1D747, 675104},
  {0x1D748, 676006},
  {0x1D749, 676076},
  {0x1D74A, 676181},
  {0x1D74B, 675768},
  {0x1D74C, 674859},
  {0x1D74D, 675837},
  {0x1D74E, 675659},
  {0x1D74F, 674534},
  {0x1D750, 674425},
  {0x1D751, 676455},
  {0x1D752, 674465},
  {0x1D753, 674580},
  {0x1D754, 674651},
  {0x1D755, 674616},
  {0x1D756, 689247},
  {0x1D757, 689329},
  {0x1D758, 689736},
  {0x1D759, 689490},
  {0x1D75A, 689572},
  {0x1D75B, 691241},
  {0x1D75C, 689617},
  {0x1D75D, 690829},
  {0x1D75E, 689857},
  {0x1D75F, 689977},
  {0x1D760, 690059},
  {0x1D761, 690141},
  {0x1D762, 690220},
  {0x1D763, 691123},
  {0x1D764, 690342},
  {0x1D765, 690467},
  {0x1D766, 690626},
  {0x1D767, 690872},
  {0x1D768, 690706},
  {0x1D769, 690788},
  {0x1D76A, 690961},
  {0x1D76B, 690426},
  {0x1D76C, 689410},
  {0x1D76D, 690507},
  {0x1D76E, 690299},
  {0x1D76F, 696918},
  {0x1D770, 697159},
  {0x1D771, 697237},
  {0x1D772, 697671},
  {0x1D773, 697390},
  {0x1D774, 697468},
  {0x1D775, 699054},
  {0x1D776, 697511},
  {0x1D777, 698710},
  {0x1D778, 697786},
  {0x1D779, 697900},
  {0x1D77A, 697978},
  {0x1D77B, 698056},
  {0x1D77C, 698131},
  {0x1D77D, 698942},
  {0x1D77E, 698247},
  {0x1D77F, 698366},
  {0x1D780, 698517},
  {0x1D781, 697587},
  {0x1D782, 698593},
  {0x1D783, 698671},
  {0x1D784, 698788},
  {0x1D785, 698327},
  {0x1D786, 697314},
  {0x1D787, 698404},
  {0x1D788, 698206},
  {0x1D789, 696953},
  {0x1D78A, 691683},
  {0x1D78B, 699094},
  {0x1D78C, 696876},
  {0x1D78D, 697003},
  {0x1D78E, 697082},
  {0x1D78F, 697043},
  {0x1D790, 691773},
  {0x1D791, 691869},
  {0x1D792, 692346},
  {0x1D793, 692058},
  {0x1D794, 692154},
  {0x1D795, 694110},
  {0x1D796, 692206},
  {0x1D797, 693628},
  {0x1D798, 692488},
  {0x1D799, 692629},
  {0x1D79A, 692725},
  {0x1D79B, 692821},
  {0x1D79C, 692914},
  {0x1D79D, 693971},
  {0x1D79E, 693057},
  {0x1D79F, 693203},
  {0x1D7A0, 693390},
  {0x1D7A1, 693678},
  {0x1D7A2, 693484},
  {0x1D7A3, 693580},
  {0x1D7A4, 693781},
  {0x1D7A5, 693155},
  {0x1D7A6, 691964},
  {0x1D7A7, 693250},
  {0x1D7A8, 693007},
  {0x1D7A9, 694259},
  {0x1D7AA, 694542},
  {0x1D7AB, 694634},
  {0x1D7AC, 695145},
  {0x1D7AD, 694815},
  {0x1D7AE, 694907},
  {0x1D7AF, 696780},
  {0x1D7B0, 694957},
  {0x1D7B1, 696373},
  {0x1D7B2, 695281},
  {0x1D7B3, 695416},
  {0x1D7B4, 695508},
  {0x1D7B5, 695600},
  {0x1D7B6, 695689},
  {0x1D7B7, 696647},
  {0x1D7B8, 695826},
  {0x1D7B9, 695966},
  {0x1D7BA, 696145},
  {0x1D7BB, 695047},
  {0x1D7BC, 696235},
  {0x1D7BD, 696327},
  {0x1D7BE, 696465},
  {0x1D7BF, 695920},
  {0x1D7C0, 694725},
  {0x1D7C1, 696011},
  {0x1D7C2, 695778},
  {0x1D7C3, 694301},
  {0x1D7C4, 694159},
  {0x1D7C5, 696827},
  {0x1D7C6, 694210},
  {0x1D7C7, 694358},
  {0x1D7C8, 694451},
  {0x1D7C9, 694405},
  {0x1D7CA, 669100},
  {0x1D7CB, 678662},
  {0x1D7CE, 670672},
  {0x1D7CF, 670528},
  {0x1D7D0, 670644},
  {0x1D7D1, 670614},
  {0x1D7D2, 670470},
  {0x1D7D3, 670441},
  {0x1D7D4, 670586},
  {0x1D7D5, 670556},
  {0x1D7D6, 670411},
  {0x1D7D7, 670499},
  {0x1D7D8, 680960},
  {0x1D7D9, 680771},
  {0x1D7DA, 680923},
  {0x1D7DB, 680884},
  {0x1D7DC, 680695},
  {0x1D7DD, 680657},
  {0x1D7DE, 680847},
  {0x1D7DF, 680808},
  {0x1D7E0, 680618},
  {0x1D7E1, 680733},
  {0x1D7E2, 700335},
  {0x1D7E3, 700161},
  {0x1D7E4, 700301},
  {0x1D7E5, 700265},
  {0x1D7E6, 700091},
  {0x1D7E7, 700056},
  {0x1D7E8, 700231},
  {0x1D7E9, 700195},
  {0x1D7EA, 700020},
  {0x1D7EB, 700126},
  {0x1D7EC, 691643},
  {0x1D7ED, 691444},
  {0x1D7EE, 691604},
  {0x1D7EF, 691563},
  {0x1D7F0, 691364},
  {0x1D7F1, 691324},
  {0x1D7F2, 691524},
  {0x1D7F3, 691483},
  {0x1D7F4, 691283},
  {0x1D7F5, 691404},
  {0x1D7F6, 688137},
  {0x1D7F7, 687968},
  {0x1D7F8, 688104},
  {0x1D7F9, 688069},
  {0x1D7FA, 687900},
  {0x1D7FB, 687866},
  {0x1D7FC, 688036},
  {0x1D7FD, 688001},
  {0x1D7FE, 687831},
  {0x1D7FF, 687934},
  {0x1D800, 864342},
  {0x1D801, 862608},
  {0x1D802, 863679},
  {0x1D803, 872769},
  {0x1D804, 871213},
  {0x1D805, 861866},
  {0x1D806, 864370},
  {0x1D807, 862638},
  {0x1D808, 869736},
  {0x1D809, 866448},
  {0x1D80A, 864448},
  {0x1D80B, 864483},
  {0x1D80C, 864518},
  {0x1D80D, 862673},
  {0x1D80E, 864658},
  {0x1D80F, 862709},
  {0x1D810, 864693},
  {0x1D811, 865710},
  {0x1D812, 865566},
  {0x1D813, 867535},
  {0x1D814, 864557},
  {0x1D815, 864788},
  {0x1D816, 865066},
  {0x1D817, 865122},
  {0x1D818, 864833},
  {0x1D819, 864951},
  {0x1D81A, 865469},
  {0x1D81B, 862796},
  {0x1D81C, 868142},
  {0x1D81D, 864403},
  {0x1D81E, 865951},
  {0x1D81F, 862927},
  {0x1D820, 865896},
  {0x1D821, 864733},
  {0x1D822, 866092},
  {0x1D823, 865608},
  {0x1D824, 867641},
  {0x1D825, 867580},
  {0x1D826, 864602},
  {0x1D827, 866389},
  {0x1D828, 866245},
  {0x1D829, 866138},
  {0x1D82A, 866341},
  {0x1D82B, 866293},
  {0x1D82C, 869244},
  {0x1D82D, 865238},
  {0x1D82E, 865355},
  {0x1D82F, 865294},
  {0x1D830, 868667},
  {0x1D831, 867136},
  {0x1D832, 865003},
  {0x1D833, 865512},
  {0x1D834, 865179},
  {0x1D835, 864885},
  {0x1D836, 868616},
  {0x1D837, 866733},
  {0x1D838, 868564},
  {0x1D839, 868508},
  {0x1D83A, 866585},
  {0x1D83B, 866531},
  {0x1D83C, 866681},
  {0x1D83D, 866187},
  {0x1D83E, 866040},
  {0x1D83F, 865992},
  {0x1D840, 868453},
  {0x1D841, 868393},
  {0x1D842, 868342},
  {0x1D843, 867081},
  {0x1D844, 870348},
  {0x1D845, 870383},
  {0x1D846, 870519},
  {0x1D847, 870423},
  {0x1D848, 870468},
  {0x1D849, 863266},
  {0x1D84A, 864265},
  {0x1D84B, 871167},
  {0x1D84C, 869984},
  {0x1D84D, 870588},
  {0x1D84E, 870073},
  {0x1D84F, 870687},
  {0x1D850, 870026},
  {0x1D851, 870635},
  {0x1D852, 870292},
  {0x1D853, 863592},
  {0x1D854, 863633},
  {0x1D855, 871119},
  {0x1D856, 872727},
  {0x1D857, 870125},
  {0x1D858, 870232},
  {0x1D859, 870174},
  {0x1D85A, 869919},
  {0x1D85B, 869941},
  {0x1D85C, 870561},
  {0x1D85D, 871063},
  {0x1D85E, 870744},
  {0x1D85F, 870994},
  {0x1D860, 871027},
  {0x1D861, 870914},
  {0x1D862, 870782},
  {0x1D863, 870817},
  {0x1D864, 870863},
  {0x1D865, 870959},
  {0x1D866, 863244},
  {0x1D867, 863538},
  {0x1D868, 863471},
  {0x1D869, 863502},
  {0x1D86A, 872313},
  {0x1D86B, 872291},
  {0x1D86C, 863886},
  {0x1D86D, 863571},
  {0x1D86E, 863987},
  {0x1D86F, 864059},
  {0x1D870, 863912},
  {0x1D871, 863856},
  {0x1D872, 863947},
  {0x1D873, 864024},
  {0x1D874, 864212},
  {0x1D875, 864091},
  {0x1D876, 862584},
  {0x1D877, 872705},
  {0x1D878, 872930},
  {0x1D879, 872829},
  {0x1D87A, 872894},
  {0x1D87B, 871899},
  {0x1D87C, 871964},
  {0x1D87D, 871096},
  {0x1D87E, 872130},
  {0x1D87F, 872006},
  {0x1D880, 872208},
  {0x1D881, 871927},
  {0x1D882, 871867},
  {0x1D883, 872242},
  {0x1D884, 872159},
  {0x1D885, 861843},
  {0x1D886, 865761},
  {0x1D887, 862885},
  {0x1D888, 871321},
  {0x1D889, 861938},
  {0x1D88A, 871651},
  {0x1D88B, 865801},
  {0x1D88C, 865846},
  {0x1D88D, 871362},
  {0x1D88E, 867731},
  {0x1D88F, 867862},
  {0x1D890, 867814},
  {0x1D891, 867765},
  {0x1D892, 868110},
  {0x1D893, 869781},
  {0x1D894, 863049},
  {0x1D895, 872797},
  {0x1D896, 862325},
  {0x1D897, 867988},
  {0x1D898, 867697},
  {0x1D899, 868067},
  {0x1D89A, 868032},
  {0x1D89B, 871759},
  {0x1D89C, 867947},
  {0x1D89D, 871717},
  {0x1D89E, 862267},
  {0x1D89F, 862213},
  {0x1D8A0, 867912},
  {0x1D8A1, 863012},
  {0x1D8A2, 871681},
  {0x1D8A3, 862177},
  {0x1D8A4, 865668},
  {0x1D8A5, 862841},
  {0x1D8A6, 871278},
  {0x1D8A7, 872045},
  {0x1D8A8, 861895},
  {0x1D8A9, 865421},
  {0x1D8AA, 862746},
  {0x1D8AB, 868835},
  {0x1D8AC, 872073},
  {0x1D8AD, 862491},
  {0x1D8AE, 869137},
  {0x1D8AF, 869062},
  {0x1D8B0, 868900},
  {0x1D8B1, 863208},
  {0x1D8B2, 872860},
  {0x1D8B3, 862549},
  {0x1D8B4, 868934},
  {0x1D8B5, 868968},
  {0x1D8B6, 869012},
  {0x1D8B7, 868867},
  {0x1D8B8, 869104},
  {0x1D8B9, 872672},
  {0x1D8BA, 866491},
  {0x1D8BB, 862970},
  {0x1D8BC, 864117},
  {0x1D8BD, 872387},
  {0x1D8BE, 872344},
  {0x1D8BF, 872431},
  {0x1D8C0, 863706},
  {0x1D8C1, 871413},
  {0x1D8C2, 862020},
  {0x1D8C3, 861979},
  {0x1D8C4, 868187},
  {0x1D8C5, 871795},
  {0x1D8C6, 868760},
  {0x1D8C7, 863174},
  {0x1D8C8, 868257},
  {0x1D8C9, 868792},
  {0x1D8CA, 872637},
  {0x1D8CB, 868718},
  {0x1D8CC, 868221},
  {0x1D8CD, 868301},
  {0x1D8CE, 863083},
  {0x1D8CF, 864164},
  {0x1D8D0, 863816},
  {0x1D8D1, 871825},
  {0x1D8D2, 862445},
  {0x1D8D3, 862400},
  {0x1D8D4, 862358},
  {0x1D8D5, 863126},
  {0x1D8D6, 863364},
  {0x1D8D7, 863415},
  {0x1D8D8, 872582},
  {0x1D8D9, 872528},
  {0x1D8DA, 872477},
  {0x1D8DB, 871242},
  {0x1D8DC, 867187},
  {0x1D8DD, 871570},
  {0x1D8DE, 867481},
  {0x1D8DF, 867426},
  {0x1D8E0, 867376},
  {0x1D8E1, 867275},
  {0x1D8E2, 867226},
  {0x1D8E3, 867325},
  {0x1D8E4, 866985},
  {0x1D8E5, 866932},
  {0x1D8E6, 867042},
  {0x1D8E7, 866784},
  {0x1D8E8, 866827},
  {0x1D8E9, 863311},
  {0x1D8EA, 866880},
  {0x1D8EB, 866640},
  {0x1D8EC, 863745},
  {0x1D8ED, 863778},
  {0x1D8EE, 871530},
  {0x1D8EF, 871489},
  {0x1D8F0, 871454},
  {0x1D8F1, 871610},
  {0x1D8F2, 862138},
  {0x1D8F3, 862100},
  {0x1D8F4, 862065},
  {0x1D8F5, 869167},
  {0x1D8F6, 869434},
  {0x1D8F7, 869647},
  {0x1D8F8, 869604},
  {0x1D8F9, 869566},
  {0x1D8FA, 869398},
  {0x1D8FB, 869195},
  {0x1D8FC, 869302},
  {0x1D8FD, 869350},
  {0x1D8FE, 869873},
  {0x1D8FF, 869521},
  {0x1D900, 869825},
  {0x1D901, 869689},
  {0x1D902, 869467},
  {0x1D903, 864243},
  {0x1D904, 864315},
  {0x1D905, 886759},
  {0x1D906, 886732},
  {0x1D907, 886706},
  {0x1D908, 861801},
  {0x1D909, 861774},
  {0x1D90A, 861748},
  {0x1D90B, 885905},
  {0x1D90C, 885877},
  {0x1D90D, 885850},
  {0x1D90E, 859602},
  {0x1D90F, 859575},
  {0x1D910, 859549},
  {0x1D911, 885494},
  {0x1D912, 885469},
  {0x1D913, 885445},
  {0x1D914, 885959},
  {0x1D915, 885931},
  {0x1D916, 885718},
  {0x1D917, 885817},
  {0x1D918, 885683},
  {0x1D919, 885782},
  {0x1D91A, 885751},
  {0x1D91B, 861473},
  {0x1D91C, 861566},
  {0x1D91D, 861440},
  {0x1D91E, 861533},
  {0x1D91F, 861504},
  {0x1D920, 885645},
  {0x1D921, 879736},
  {0x1D922, 879777},
  {0x1D923, 879818},
  {0x1D924, 879538},
  {0x1D925, 879630},
  {0x1D926, 879683},
  {0x1D927, 879581},
  {0x1D928, 881810},
  {0x1D929, 877008},
  {0x1D92A, 882735},
  {0x1D92B, 882681},
  {0x1D92C, 882573},
  {0x1D92D, 882626},
  {0x1D92E, 882788},
  {0x1D92F, 881608},
  {0x1D930, 881655},
  {0x1D931, 881497},
  {0x1D932, 881547},
  {0x1D933, 880642},
  {0x1D934, 882948},
  {0x1D935, 883004},
  {0x1D936, 882837},
  {0x1D937, 882887},
  {0x1D938, 880166},
  {0x1D939, 880123},
  {0x1D93A, 880081},
  {0x1D93B, 880598},
  {0x1D93C, 880506},
  {0x1D93D, 880462},
  {0x1D93E, 880551},
  {0x1D93F, 880419},
  {0x1D940, 880375},
  {0x1D941, 880332},
  {0x1D942, 880291},
  {0x1D943, 880249},
  {0x1D944, 880208},
  {0x1D945, 883862},
  {0x1D946, 883817},
  {0x1D947, 883773},
  {0x1D948, 882491},
  {0x1D949, 882447},
  {0x1D94A, 882404},
  {0x1D94B, 887600},
  {0x1D94C, 887545},
  {0x1D94D, 887485},
  {0x1D94E, 887429},
  {0x1D94F, 887373},
  {0x1D950, 887312},
  {0x1D951, 887655},
  {0x1D952, 887218},
  {0x1D953, 887171},
  {0x1D954, 887265},
  {0x1D955, 874779},
  {0x1D956, 874737},
  {0x1D957, 874653},
  {0x1D958, 874694},
  {0x1D959, 875365},
  {0x1D95A, 875320},
  {0x1D95B, 875230},
  {0x1D95C, 875274},
  {0x1D95D, 874970},
  {0x1D95E, 874920},
  {0x1D95F, 874820},
  {0x1D960, 874869},
  {0x1D961, 875178},
  {0x1D962, 875125},
  {0x1D963, 875019},
  {0x1D964, 875071},
  {0x1D965, 878578},
  {0x1D966, 878523},
  {0x1D967, 878413},
  {0x1D968, 878467},
  {0x1D969, 878632},
  {0x1D96A, 876776},
  {0x1D96B, 876824},
  {0x1D96C, 876663},
  {0x1D96D, 876714},
  {0x1D96E, 876163},
  {0x1D96F, 878804},
  {0x1D970, 878861},
  {0x1D971, 878682},
  {0x1D972, 878742},
  {0x1D973, 875825},
  {0x1D974, 876118},
  {0x1D975, 876072},
  {0x1D976, 876027},
  {0x1D977, 875989},
  {0x1D978, 875947},
  {0x1D979, 875904},
  {0x1D97A, 875862},
  {0x1D97B, 879493},
  {0x1D97C, 879447},
  {0x1D97D, 879402},
  {0x1D97E, 878320},
  {0x1D97F, 878275},
  {0x1D980, 878231},
  {0x1D981, 886903},
  {0x1D982, 886846},
  {0x1D983, 886784},
  {0x1D984, 887077},
  {0x1D985, 887021},
  {0x1D986, 886960},
  {0x1D987, 887133},
  {0x1D988, 881165},
  {0x1D989, 881113},
  {0x1D98A, 881009},
  {0x1D98B, 881060},
  {0x1D98C, 880847},
  {0x1D98D, 880791},
  {0x1D98E, 880679},
  {0x1D98F, 880734},
  {0x1D990, 881332},
  {0x1D991, 881267},
  {0x1D992, 882046},
  {0x1D993, 882003},
  {0x1D994, 881961},
  {0x1D995, 882313},
  {0x1D996, 882270},
  {0x1D997, 882228},
  {0x1D998, 882355},
  {0x1D999, 883164},
  {0x1D99A, 883108},
  {0x1D99B, 883053},
  {0x1D99C, 883330},
  {0x1D99D, 883274},
  {0x1D99E, 883219},
  {0x1D99F, 881216},
  {0x1D9A0, 881447},
  {0x1D9A1, 881396},
  {0x1D9A2, 885298},
  {0x1D9A3, 885151},
  {0x1D9A4, 884991},
  {0x1D9A5, 882534},
  {0x1D9A6, 880953},
  {0x1D9A7, 881906},
  {0x1D9A8, 882173},
  {0x1D9A9, 883604},
  {0x1D9AA, 885388},
  {0x1D9AB, 885241},
  {0x1D9AC, 885089},
  {0x1D9AD, 880902},
  {0x1D9AE, 881856},
  {0x1D9AF, 882123},
  {0x1D9B0, 883554},
  {0x1D9B1, 885336},
  {0x1D9B2, 885189},
  {0x1D9B3, 885032},
  {0x1D9B4, 883498},
  {0x1D9B5, 883441},
  {0x1D9B6, 883385},
  {0x1D9B7, 876308},
  {0x1D9B8, 876248},
  {0x1D9B9, 877187},
  {0x1D9BA, 877055},
  {0x1D9BB, 877253},
  {0x1D9BC, 877121},
  {0x1D9BD, 877866},
  {0x1D9BE, 877734},
  {0x1D9BF, 877800},
  {0x1D9C0, 877668},
  {0x1D9C1, 878970},
  {0x1D9C2, 878911},
  {0x1D9C3, 884883},
  {0x1D9C4, 884736},
  {0x1D9C5, 884579},
  {0x1D9C6, 876426},
  {0x1D9C7, 876368},
  {0x1D9C8, 877383},
  {0x1D9C9, 877319},
  {0x1D9CA, 877518},
  {0x1D9CB, 877447},
  {0x1D9CC, 878124},
  {0x1D9CD, 877996},
  {0x1D9CE, 878060},
  {0x1D9CF, 877932},
  {0x1D9D0, 879086},
  {0x1D9D1, 879029},
  {0x1D9D2, 884938},
  {0x1D9D3, 884791},
  {0x1D9D4, 884639},
  {0x1D9D5, 876619},
  {0x1D9D6, 876574},
  {0x1D9D7, 876484},
  {0x1D9D8, 876528},
  {0x1D9D9, 876201},
  {0x1D9DA, 877589},
  {0x1D9DB, 878188},
  {0x1D9DC, 879229},
  {0x1D9DD, 879186},
  {0x1D9DE, 879143},
  {0x1D9DF, 884844},
  {0x1D9E0, 884697},
  {0x1D9E1, 884535},
  {0x1D9E2, 878364},
  {0x1D9E3, 880026},
  {0x1D9E4, 879915},
  {0x1D9E5, 879971},
  {0x1D9E6, 879859},
  {0x1D9E7, 875756},
  {0x1D9E8, 875617},
  {0x1D9E9, 875478},
  {0x1D9EA, 875687},
  {0x1D9EB, 875547},
  {0x1D9EC, 875409},
  {0x1D9ED, 883716},
  {0x1D9EE, 883659},
  {0x1D9EF, 879337},
  {0x1D9F0, 879272},
  {0x1D9F1, 881757},
  {0x1D9F2, 881704},
  {0x1D9F3, 876941},
  {0x1D9F4, 876874},
  {0x1D9F5, 859938},
  {0x1D9F6, 859902},
  {0x1D9F7, 860011},
  {0x1D9F8, 860170},
  {0x1D9F9, 860195},
  {0x1D9FA, 860064},
  {0x1D9FB, 860092},
  {0x1D9FC, 860125},
  {0x1D9FD, 859974},
  {0x1D9FE, 860036},
  {0x1D9FF, 872963},
  {0x1DA00, 873230},
  {0x1DA01, 873144},
  {0x1DA02, 873189},
  {0x1DA03, 873056},
  {0x1DA04, 873102},
  {0x1DA05, 873013},
  {0x1DA06, 872980},
  {0x1DA07, 861112},
  {0x1DA08, 861169},
  {0x1DA09, 861221},
  {0x1DA0A, 860415},
  {0x1DA0B, 860377},
  {0x1DA0C, 860342},
  {0x1DA0D, 859783},
  {0x1DA0E, 859742},
  {0x1DA0F, 859863},
  {0x1DA10, 859824},
  {0x1DA11, 861667},
  {0x1DA12, 861638},
  {0x1DA13, 861696},
  {0x1DA14, 861002},
  {0x1DA15, 861024},
  {0x1DA16, 860922},
  {0x1DA17, 860292},
  {0x1DA18, 860261},
  {0x1DA19, 860975},
  {0x1DA1A, 861050},
  {0x1DA1B, 860946},
  {0x1DA1C, 861077},
  {0x1DA1D, 860321},
  {0x1DA1E, 860897},
  {0x1DA1F, 860837},
  {0x1DA20, 860864},
  {0x1DA21, 860701},
  {0x1DA22, 860791},
  {0x1DA23, 860740},
  {0x1DA24, 860486},
  {0x1DA25, 860578},
  {0x1DA26, 860526},
  {0x1DA27, 860664},
  {0x1DA28, 860448},
  {0x1DA29, 860625},
  {0x1DA2A, 859654},
  {0x1DA2B, 859627},
  {0x1DA2C, 859680},
  {0x1DA2D, 886186},
  {0x1DA2E, 886245},
  {0x1DA2F, 886216},
  {0x1DA30, 860221},
  {0x1DA31, 883948},
  {0x1DA32, 883923},
  {0x1DA33, 883998},
  {0x1DA34, 883973},
  {0x1DA35, 859405},
  {0x1DA36, 859470},
  {0x1DA37, 859368},
  {0x1DA38, 859433},
  {0x1DA39, 859523},
  {0x1DA3A, 859497},
  {0x1DA3B, 873877},
  {0x1DA3C, 873844},
  {0x1DA3D, 873811},
  {0x1DA3E, 874412},
  {0x1DA3F, 874465},
  {0x1DA40, 874436},
  {0x1DA41, 873936},
  {0x1DA42, 873989},
  {0x1DA43, 873960},
  {0x1DA44, 874108},
  {0x1DA45, 874138},
  {0x1DA46, 874380},
  {0x1DA47, 874169},
  {0x1DA48, 874197},
  {0x1DA49, 874234},
  {0x1DA4A, 874267},
  {0x1DA4B, 874300},
  {0x1DA4C, 874342},
  {0x1DA4D, 874022},
  {0x1DA4E, 874045},
  {0x1DA4F, 874076},
  {0x1DA50, 874498},
  {0x1DA51, 874522},
  {0x1DA52, 874554},
  {0x1DA53, 873528},
  {0x1DA54, 873462},
  {0x1DA55, 873495},
  {0x1DA56, 873910},
  {0x1DA57, 874619},
  {0x1DA58, 874585},
  {0x1DA59, 886466},
  {0x1DA5A, 886395},
  {0x1DA5B, 886502},
  {0x1DA5C, 886538},
  {0x1DA5D, 886355},
  {0x1DA5E, 886427},
  {0x1DA5F, 886316},
  {0x1DA60, 886277},
  {0x1DA61, 885987},
  {0x1DA62, 886033},
  {0x1DA63, 886121},
  {0x1DA64, 886149},
  {0x1DA65, 886060},
  {0x1DA66, 886086},
  {0x1DA67, 886005},
  {0x1DA68, 882088},
  {0x1DA69, 877632},
  {0x1DA6A, 883906},
  {0x1DA6B, 861826},
  {0x1DA6C, 860238},
  {0x1DA6D, 885574},
  {0x1DA6E, 885539},
  {0x1DA6F, 887739},
  {0x1DA70, 861597},
  {0x1DA71, 885605},
  {0x1DA72, 886661},
  {0x1DA73, 886621},
  {0x1DA74, 886583},
  {0x1DA75, 887692},
  {0x1DA76, 873251},
  {0x1DA77, 873280},
  {0x1DA78, 873306},
  {0x1DA79, 873332},
  {0x1DA7A, 873358},
  {0x1DA7B, 873384},
  {0x1DA7C, 873410},
  {0x1DA7D, 873436},
  {0x1DA7E, 861421},
  {0x1DA7F, 873774},
  {0x1DA80, 873736},
  {0x1DA81, 873620},
  {0x1DA82, 873709},
  {0x1DA83, 873562},
  {0x1DA84, 873589},
  {0x1DA85, 873682},
  {0x1DA86, 873648},
  {0x1DA87, 859724},
  {0x1DA88, 861726},
  {0x1DA89, 885517},
  {0x1DA8A, 859706},
  {0x1DA8B, 884024},
  {0x1DA9B, 861281},
  {0x1DA9C, 861309},
  {0x1DA9D, 861337},
  {0x1DA9E, 861365},
  {0x1DA9F, 861393},
  {0x1DAA1, 884279},
  {0x1DAA2, 884311},
  {0x1DAA3, 884343},
  {0x1DAA4, 884375},
  {0x1DAA5, 884407},
  {0x1DAA6, 884439},
  {0x1DAA7, 884471},
  {0x1DAA8, 884503},
  {0x1DAA9, 884048},
  {0x1DAAA, 884081},
  {0x1DAAB, 884114},
  {0x1DAAC, 884147},
  {0x1DAAD, 884180},
  {0x1DAAE, 884213},
  {0x1DAAF, 884246},
  {0x1DF00, 607944},
  {0x1DF01, 617166},
  {0x1DF02, 600946},
  {0x1DF03, 616957},
  {0x1DF04, 600353},
  {0x1DF05, 611829},
  {0x1DF06, 621139},
  {0x1DF07, 616890},
  {0x1DF08, 620769},
  {0x1DF09, 619040},
  {0x1DF0A, 599644},
  {0x1DF0B, 607258},
  {0x1DF0C, 607297},
  {0x1DF0D, 620988},
  {0x1DF0E, 599455},
  {0x1DF0F, 601260},
  {0x1DF10, 600982},
  {0x1DF11, 611238},
  {0x1DF12, 605297},
  {0x1DF13, 610884},
  {0x1DF14, 607161},
  {0x1DF15, 620876},
  {0x1DF16, 616331},
  {0x1DF17, 619430},
  {0x1DF18, 607596},
  {0x1DF19, 605347},
  {0x1DF1A, 609645},
  {0x1DF1B, 614696},
  {0x1DF1C, 619480},
  {0x1DF1D, 604182},
  {0x1DF1E, 617543},
  {0x1DF25, 604941},
  {0x1DF26, 611390},
  {0x1DF27, 613064},
  {0x1DF28, 616496},
  {0x1DF29, 617769},
  {0x1DF2A, 619127},
  {0x1E000, 201230},
  {0x1E001, 201298},
  {0x1E002, 202241},
  {0x1E003, 201502},
  {0x1E004, 201401},
  {0x1E005, 202373},
  {0x1E006, 202504},
  {0x1E008, 202469},
  {0x1E009, 201732},
  {0x1E00A, 201601},
  {0x1E00B, 201571},
  {0x1E00C, 201366},
  {0x1E00D, 201765},
  {0x1E00E, 201798},
  {0x1E00F, 201834},
  {0x1E010, 201870},
  {0x1E011, 201904},
  {0x1E012, 201936},
  {0x1E013, 201971},
  {0x1E014, 202070},
  {0x1E015, 202174},
  {0x1E016, 202209},
  {0x1E017, 201468},
  {0x1E018, 201538},
  {0x1E01B, 202037},
  {0x1E01C, 202142},
  {0x1E01D, 201331},
  {0x1E01E, 202005},
  {0x1E01F, 202340},
  {0x1E020, 202307},
  {0x1E021, 202274},
  {0x1E023, 202438},
  {0x1E024, 202104},
  {0x1E026, 202407},
  {0x1E027, 201686},
  {0x1E028, 201262},
  {0x1E029, 201642},
  {0x1E02A, 201435},
  {0x1E030, 728436},
  {0x1E031, 728509},
  {0x1E032, 729537},
  {0x1E033, 728956},
  {0x1E034, 728634},
  {0x1E035, 729058},
  {0x1E036, 729725},
  {0x1E037, 729691},
  {0x1E038, 729025},
  {0x1E039, 729126},
  {0x1E03A, 728771},
  {0x1E03B, 728805},
  {0x1E03C, 729160},
  {0x1E03D, 729233},
  {0x1E03E, 728839},
  {0x1E03F, 728873},
  {0x1E040, 729435},
  {0x1E041, 729504},
  {0x1E042, 728737},
  {0x1E043, 728991},
  {0x1E044, 729469},
  {0x1E045, 728599},
  {0x1E046, 729304},
  {0x1E047, 729571},
  {0x1E048, 728704},
  {0x1E049, 729657},
  {0x1E04A, 728668},
  {0x1E04B, 729267},
  {0x1E04C, 728543},
  {0x1E04D, 729092},
  {0x1E04E, 728469},
  {0x1E04F, 729339},
  {0x1E050, 729193},
  {0x1E051, 268161},
  {0x1E052, 268195},
  {0x1E053, 268976},
  {0x1E054, 268536},
  {0x1E055, 268323},
  {0x1E056, 268731},
  {0x1E057, 269083},
  {0x1E058, 269048},
  {0x1E059, 268697},
  {0x1E05A, 268766},
  {0x1E05B, 268466},
  {0x1E05C, 268801},
  {0x1E05D, 268835},
  {0x1E05E, 268501},
  {0x1E05F, 268942},
  {0x1E060, 268431},
  {0x1E061, 268620},
  {0x1E062, 268906},
  {0x1E063, 268287},
  {0x1E064, 268870},
  {0x1E065, 268655},
  {0x1E066, 269011},
  {0x1E067, 268572},
  {0x1E068, 268230},
  {0x1E069, 268358},
  {0x1E06A, 268394},
  {0x1E06B, 728907},
  {0x1E06C, 729607},
  {0x1E06D, 729381},
  {0x1E08F, 199025},
  {0x1E100, 783776},
  {0x1E101, 784517},
  {0x1E102, 784081},
  {0x1E103, 784484},
  {0x1E104, 783611},
  {0x1E105, 783843},
  {0x1E106, 784682},
  {0x1E107, 783910},
  {0x1E108, 783380},
  {0x1E109, 783743},
  {0x1E10A, 784451},
  {0x1E10B, 784782},
  {0x1E10C, 783876},
  {0x1E10D, 784115},
  {0x1E10E, 783710},
  {0x1E10F, 783413},
  {0x1E110, 784185},
  {0x1E111, 784047},
  {0x1E112, 784617},
  {0x1E113, 784150},
  {0x1E114, 784551},
  {0x1E115, 783545},
  {0x1E116, 784384},
  {0x1E117, 784351},
  {0x1E118, 784749},
  {0x1E119, 784013},
  {0x1E11A, 784284},
  {0x1E11B, 784715},
  {0x1E11C, 783944},
  {0x1E11D, 783446},
  {0x1E11E, 783978},
  {0x1E11F, 783644},
  {0x1E120, 783809},
  {0x1E121, 784317},
  {0x1E122, 783578},
  {0x1E123, 784417},
  {0x1E124, 783315},
  {0x1E125, 783347},
  {0x1E126, 783678},
  {0x1E127, 784585},
  {0x1E128, 784219},
  {0x1E129, 784251},
  {0x1E12A, 783480},
  {0x1E12B, 783512},
  {0x1E12C, 784650},
  {0x1E130, 785131},
  {0x1E131, 785251},
  {0x1E132, 785221},
  {0x1E133, 785311},
  {0x1E134, 785281},
  {0x1E135, 785191},
  {0x1E136, 785161},
  {0x1E137, 784977},
  {0x1E138, 785016},
  {0x1E139, 784936},
  {0x1E13A, 784852},
  {0x1E13B, 784891},
  {0x1E13C, 785054},
  {0x1E13D, 785088},
  {0x1E140, 783281},
  {0x1E141, 783112},
  {0x1E142, 783248},
  {0x1E143, 783213},
  {0x1E144, 783044},
  {0x1E145, 783010},
  {0x1E146, 783180},
  {0x1E147, 783145},
  {0x1E148, 782975},
  {0x1E149, 783078},
  {0x1E14E, 784815},
  {0x1E14F, 782941},
  {0x1E290, 949962},
  {0x1E291, 949610},
  {0x1E292, 950007},
  {0x1E293, 949754},
  {0x1E294, 949872},
  {0x1E295, 949798},
  {0x1E296, 949902},
  {0x1E297, 949917},
  {0x1E298, 949932},
  {0x1E299, 949992},
  {0x1E29A, 949738},
  {0x1E29B, 950051},
  {0x1E29C, 950036},
  {0x1E29D, 949857},
  {0x1E29E, 949813},
  {0x1E29F, 949977},
  {0x1E2A0, 949887},
  {0x1E2A1, 949828},
  {0x1E2A2, 949693},
  {0x1E2A3, 949842},
  {0x1E2A4, 949715},
  {0x1E2A5, 950022},
  {0x1E2A6, 949769},
  {0x1E2A7, 949648},
  {0x1E2A8, 949783},
  {0x1E2A9, 949670},
  {0x1E2AA, 949948},
  {0x1E2AB, 949595},
  {0x1E2AC, 949625},
  {0x1E2AD, 949581},
  {0x1E2AE, 950066},
  {0x1E2C0, 978787},
  {0x1E2C1, 978771},
  {0x1E2C2, 978876},
  {0x1E2C3, 978893},
  {0x1E2C4, 978910},
  {0x1E2C5, 978977},
  {0x1E2C6, 979480},
  {0x1E2C7, 979271},
  {0x1E2C8, 979097},
  {0x1E2C9, 979150},
  {0x1E2CA, 979254},
  {0x1E2CB, 979341},
  {0x1E2CC, 979358},
  {0x1E2CD, 978960},
  {0x1E2CE, 979306},
  {0x1E2CF, 979323},
  {0x1E2D0, 979045},
  {0x1E2D1, 979515},
  {0x1E2D2, 979463},
  {0x1E2D3, 979446},
  {0x1E2D4, 979062},
  {0x1E2D5, 979203},
  {0x1E2D6, 978859},
  {0x1E2D7, 979289},
  {0x1E2D8, 979133},
  {0x1E2D9, 979079},
  {0x1E2DA, 978994},
  {0x1E2DB, 978927},
  {0x1E2DC, 979011},
  {0x1E2DD, 979167},
  {0x1E2DE, 979412},
  {0x1E2DF, 979114},
  {0x1E2E0, 979394},
  {0x1E2E1, 979376},
  {0x1E2E2, 979236},
  {0x1E2E3, 978822},
  {0x1E2E4, 978841},
  {0x1E2E5, 979027},
  {0x1E2E6, 979219},
  {0x1E2E7, 978943},
  {0x1E2E8, 978804},
  {0x1E2E9, 979185},
  {0x1E2EA, 979428},
  {0x1E2EB, 979497},
  {0x1E2EC, 979583},
  {0x1E2ED, 979599},
  {0x1E2EE, 979549},
  {0x1E2EF, 979565},
  {0x1E2F0, 978753},
  {0x1E2F1, 978664},
  {0x1E2F2, 978736},
  {0x1E2F3, 978717},
  {0x1E2F4, 978628},
  {0x1E2F5, 978610},
  {0x1E2F6, 978700},
  {0x1E2F7, 978681},
  {0x1E2F8, 978591},
  {0x1E2F9, 978646},
  {0x1E2FF, 979532},
  {0x1E4D0, 760053},
  {0x1E4D1, 760119},
  {0x1E4D2, 760074},
  {0x1E4D3, 760141},
  {0x1E4D4, 760096},
  {0x1E4D5, 759677},
  {0x1E4D6, 759742},
  {0x1E4D7, 759698},
  {0x1E4D8, 759764},
  {0x1E4D9, 759720},
  {0x1E4DA, 759943},
  {0x1E4DB, 760009},
  {0x1E4DC, 759964},
  {0x1E4DD, 760031},
  {0x1E4DE, 759987},
  {0x1E4DF, 760163},
  {0x1E4E0, 760184},
  {0x1E4E1, 760206},
  {0x1E4E2, 760228},
  {0x1E4E3, 760250},
  {0x1E4E4, 759787},
  {0x1E4E5, 759897},
  {0x1E4E6, 759808},
  {0x1E4E7, 759853},
  {0x1E4E8, 759875},
  {0x1E4E9, 759920},
  {0x1E4EA, 759830},
  {0x1E4EB, 760317},
  {0x1E4EC, 760294},
  {0x1E4ED, 760362},
  {0x1E4EE, 760272},
  {0x1E4EF, 760339},
  {0x1E4F0, 759654},
  {0x1E4F1, 759540},
  {0x1E4F2, 759632},
  {0x1E4F3, 759608},
  {0x1E4F4, 759494},
  {0x1E4F5, 759471},
  {0x1E4F6, 759586},
  {0x1E4F7, 759562},
  {0x1E4F8, 759447},
  {0x1E4F9, 759517},
  {0x1E5D0, 787796},
  {0x1E5D1, 787813},
  {0x1E5D2, 787831},
  {0x1E5D3, 787868},
  {0x1E5D4, 787850},
  {0x1E5D5, 787887},
  {0x1E5D6, 787473},
  {0x1E5D7, 787508},
  {0x1E5D8, 787490},
  {0x1E5D9, 787526},
  {0x1E5DA, 787544},
  {0x1E5DB, 787562},
  {0x1E5DC, 787688},
  {0x1E5DD, 787759},
  {0x1E5DE, 787741},
  {0x1E5DF, 787777},
  {0x1E5E0, 787705},
  {0x1E5E1, 787723},
  {0x1E5E2, 787905},
  {0x1E5E3, 787959},
  {0x1E5E4, 787922},
  {0x1E5E5, 787941},
  {0x1E5E6, 787977},
  {0x1E5E7, 787996},
  {0x1E5E8, 787580},
  {0x1E5E9, 787670},
  {0x1E5EA, 787633},
  {0x1E5EB, 787597},
  {0x1E5EC, 787651},
  {0x1E5ED, 787615},
  {0x1E5EE, 788053},
  {0x1E5EF, 788035},
  {0x1E5F0, 788014},
  {0x1E5F1, 787454},
  {0x1E5F2, 787360},
  {0x1E5F3, 787436},
  {0x1E5F4, 787416},
  {0x1E5F5, 787322},
  {0x1E5F6, 787303},
  {0x1E5F7, 787398},
  {0x1E5F8, 787378},
  {0x1E5F9, 787283},
  {0x1E5FA, 787341},
  {0x1E5FF, 787257},
  {0x1E7E0, 434605},
  {0x1E7E1, 434745},
  {0x1E7E2, 434699},
  {0x1E7E3, 434628},
  {0x1E7E4, 434675},
  {0x1E7E5, 434652},
  {0x1E7E6, 434722},
  {0x1E7E8, 433678},
  {0x1E7E9, 434582},
  {0x1E7EA, 434558},
  {0x1E7EB, 434535},
  {0x1E7ED, 433826},
  {0x1E7EE, 433796},
  {0x1E7F0, 433973},
  {0x1E7F1, 433943},
  {0x1E7F2, 433914},
  {0x1E7F3, 433502},
  {0x1E7F4, 433472},
  {0x1E7F5, 433767},
  {0x1E7F6, 433737},
  {0x1E7F7, 433708},
  {0x1E7F8, 433649},
  {0x1E7F9, 433619},
  {0x1E7FA, 433590},
  {0x1E7FB, 433561},
  {0x1E7FC, 433531},
  {0x1E7FD, 433885},
  {0x1E7FE, 433855},
  {0x1E800, 711497},
  {0x1E801, 711528},
  {0x1E802, 711559},
  {0x1E803, 713509},
  {0x1E804, 714471},
  {0x1E805, 713858},
  {0x1E806, 712967},
  {0x1E807, 717186},
  {0x1E808, 711590},
  {0x1E809, 711621},
  {0x1E80A, 711652},
  {0x1E80B, 715474},
  {0x1E80C, 715219},
  {0x1E80D, 715090},
  {0x1E80E, 712870},
  {0x1E80F, 717673},
  {0x1E810, 716022},
  {0x1E811, 713382},
  {0x1E812, 712998},
  {0x1E813, 715890},
  {0x1E814, 711683},
  {0x1E815, 711715},
  {0x1E816, 711747},
  {0x1E817, 713317},
  {0x1E818, 714439},
  {0x1E819, 716372},
  {0x1E81A, 717511},
  {0x1E81B, 711779},
  {0x1E81C, 711810},
  {0x1E81D, 711841},
  {0x1E81E, 716246},
  {0x1E81F, 714533},
  {0x1E820, 714722},
  {0x1E821, 715859},
  {0x1E822, 711872},
  {0x1E823, 711902},
  {0x1E824, 711932},
  {0x1E825, 716669},
  {0x1E826, 714628},
  {0x1E827, 716733},
  {0x1E828, 716151},
  {0x1E829, 715828},
  {0x1E82A, 715637},
  {0x1E82B, 715765},
  {0x1E82C, 717705},
  {0x1E82D, 717155},
  {0x1E82E, 712055},
  {0x1E82F, 712086},
  {0x1E830, 712117},
  {0x1E831, 716637},
  {0x1E832, 715156},
  {0x1E833, 715796},
  {0x1E834, 713953},
  {0x1E835, 717736},
  {0x1E836, 712241},
  {0x1E837, 712272},
  {0x1E838, 712303},
  {0x1E839, 714113},
  {0x1E83A, 713764},
  {0x1E83B, 713156},
  {0x1E83C, 716341},
  {0x1E83D, 714959},
  {0x1E83E, 711962},
  {0x1E83F, 711993},
  {0x1E840, 712024},
  {0x1E841, 714277},
  {0x1E842, 717218},
  {0x1E843, 717250},
  {0x1E844, 712148},
  {0x1E845, 712179},
  {0x1E846, 712210},
  {0x1E847, 714342},
  {0x1E848, 713188},
  {0x1E849, 714754},
  {0x1E84A, 713636},
  {0x1E84B, 712334},
  {0x1E84C, 712365},
  {0x1E84D, 712396},
  {0x1E84E, 716470},
  {0x1E84F, 715059},
  {0x1E850, 716568},
  {0x1E851, 713446},
  {0x1E852, 717054},
  {0x1E853, 712427},
  {0x1E854, 712458},
  {0x1E855, 712489},
  {0x1E856, 714927},
  {0x1E857, 713984},
  {0x1E858, 715953},
  {0x1E859, 715315},
  {0x1E85A, 712520},
  {0x1E85B, 712551},
  {0x1E85C, 712582},
  {0x1E85D, 713921},
  {0x1E85E, 713827},
  {0x1E85F, 715700},
  {0x1E860, 714246},
  {0x1E861, 717768},
  {0x1E862, 714658},
  {0x1E863, 712613},
  {0x1E864, 712645},
  {0x1E865, 712677},
  {0x1E866, 715187},
  {0x1E867, 716861},
  {0x1E868, 717090},
  {0x1E869, 712741},
  {0x1E86A, 717414},
  {0x1E86B, 712709},
  {0x1E86C, 714502},
  {0x1E86D, 712772},
  {0x1E86E, 715922},
  {0x1E86F, 714080},
  {0x1E870, 715540},
  {0x1E871, 713124},
  {0x1E872, 715605},
  {0x1E873, 714214},
  {0x1E874, 713092},
  {0x1E875, 717641},
  {0x1E876, 712901},
  {0x1E877, 714309},
  {0x1E878, 712804},
  {0x1E879, 714047},
  {0x1E87A, 715122},
  {0x1E87B, 716118},
  {0x1E87C, 716436},
  {0x1E87D, 715282},
  {0x1E87E, 716534},
  {0x1E87F, 715506},
  {0x1E880, 714176},
  {0x1E881, 714824},
  {0x1E882, 717312},
  {0x1E883, 716405},
  {0x1E884, 714995},
  {0x1E885, 716830},
  {0x1E886, 717544},
  {0x1E887, 716764},
  {0x1E888, 716797},
  {0x1E889, 717350},
  {0x1E88A, 713251},
  {0x1E88B, 717121},
  {0x1E88C, 713605},
  {0x1E88D, 714597},
  {0x1E88E, 713030},
  {0x1E88F, 714015},
  {0x1E890, 713061},
  {0x1E891, 714690},
  {0x1E892, 713541},
  {0x1E893, 716086},
  {0x1E894, 713414},
  {0x1E895, 715346},
  {0x1E896, 712934},
  {0x1E897, 717478},
  {0x1E898, 713732},
  {0x1E899, 716957},
  {0x1E89A, 717022},
  {0x1E89B, 717445},
  {0x1E89C, 716600},
  {0x1E89D, 714786},
  {0x1E89E, 715985},
  {0x1E89F, 715668},
  {0x1E8A0, 714374},
  {0x1E8A1, 713795},
  {0x1E8A2, 712837},
  {0x1E8A3, 714895},
  {0x1E8A4, 715026},
  {0x1E8A5, 716502},
  {0x1E8A6, 715410},
  {0x1E8A7, 713219},
  {0x1E8A8, 716181},
  {0x1E8A9, 714406},
  {0x1E8AA, 714863},
  {0x1E8AB, 713699},
  {0x1E8AC, 713667},
  {0x1E8AD, 716926},
  {0x1E8AE, 715378},
  {0x1E8AF, 715573},
  {0x1E8B0, 715442},
  {0x1E8B1, 717576},
  {0x1E8B2, 715250},
  {0x1E8B3, 713572},
  {0x1E8B4, 713477},
  {0x1E8B5, 716309},
  {0x1E8B6, 717609},
  {0x1E8B7, 716213},
  {0x1E8B8, 715732},
  {0x1E8B9, 717281},
  {0x1E8BA, 717383},
  {0x1E8BB, 716278},
  {0x1E8BC, 716990},
  {0x1E8BD, 714145},
  {0x1E8BE, 716054},
  {0x1E8BF, 713890},
  {0x1E8C0, 716700},
  {0x1E8C1, 713284},
  {0x1E8C2, 716893},
  {0x1E8C3, 714564},
  {0x1E8C4, 713349},
  {0x1E8C7, 711373},
  {0x1E8C8, 711473},
  {0x1E8C9, 711447},
  {0x1E8CA, 711323},
  {0x1E8CB, 711298},
  {0x1E8CC, 711423},
  {0x1E8CD, 711397},
  {0x1E8CE, 711272},
  {0x1E8CF, 711348},
  {0x1E8D0, 711113},
  {0x1E8D1, 711195},
  {0x1E8D2, 711033},
  {0x1E8D3, 711231},
  {0x1E8D4, 711150},
  {0x1E8D5, 710984},
  {0x1E8D6, 711073},
  {0x1E900, 206},
  {0x1E901, 306},
  {0x1E902, 602},
  {0x1E903, 628},
  {0x1E904, 232},
  {0x1E905, 851},
  {0x1E906, 752},
  {0x1E907, 256},
  {0x1E908, 802},
  {0x1E909, 358},
  {0x1E90A, 381},
  {0x1E90B, 478},
  {0x1E90C, 729},
  {0x1E90D, 333},
  {0x1E90E, 1003},
  {0x1E90F, 954},
  {0x1E910, 679},
  {0x1E911, 527},
  {0x1E912, 979},
  {0x1E913, 907},
  {0x1E914, 501},
  {0x1E915, 281},
  {0x1E916, 454},
  {0x1E917, 776},
  {0x1E918, 405},
  {0x1E919, 704},
  {0x1E91A, 883},
  {0x1E91B, 654},
  {0x1E91C, 930},
  {0x1E91D, 552},
  {0x1E91E, 429},
  {0x1E91F, 1028},
  {0x1E920, 577},
  {0x1E921, 826},
  {0x1E922, 1411},
  {0x1E923, 1503},
  {0x1E924, 1775},
  {0x1E925, 1799},
  {0x1E926, 1435},
  {0x1E927, 2004},
  {0x1E928, 1913},
  {0x1E929, 1457},
  {0x1E92A, 1959},
  {0x1E92B, 1551},
  {0x1E92C, 1572},
  {0x1E92D, 1661},
  {0x1E92E, 1892},
  {0x1E92F, 1528},
  {0x1E930, 2144},
  {0x1E931, 2099},
  {0x1E932, 1846},
  {0x1E933, 1706},
  {0x1E934, 2122},
  {0x1E935, 2056},
  {0x1E936, 1682},
  {0x1E937, 1480},
  {0x1E938, 1639},
  {0x1E939, 1935},
  {0x1E93A, 1594},
  {0x1E93B, 1869},
  {0x1E93C, 2034},
  {0x1E93D, 1823},
  {0x1E93E, 2077},
  {0x1E93F, 1729},
  {0x1E940, 1616},
  {0x1E941, 2167},
  {0x1E942, 1752},
  {0x1E943, 1981},
  {0x1E944, 184},
  {0x1E945, 2190},
  {0x1E946, 1282},
  {0x1E947, 1304},
  {0x1E948, 1053},
  {0x1E949, 1248},
  {0x1E94A, 1399},
  {0x1E94B, 1375},
  {0x1E950, 1231},
  {0x1E951, 1147},
  {0x1E952, 1215},
  {0x1E953, 1197},
  {0x1E954, 1113},
  {0x1E955, 1096},
  {0x1E956, 1181},
  {0x1E957, 1163},
  {0x1E958, 1078},
  {0x1E959, 1130},
  {0x1E95E, 1316},
  {0x1E95F, 1347},
  {0x1EC71, 531665},
  {0x1EC72, 532617},
  {0x1EC73, 532464},
  {0x1EC74, 531324},
  {0x1EC75, 531176},
  {0x1EC76, 532203},
  {0x1EC77, 532048},
  {0x1EC78, 530964},
  {0x1EC79, 531515},
  {0x1EC7A, 532348},
  {0x1EC7B, 532556},
  {0x1EC7C, 532403},
  {0x1EC7D, 531265},
  {0x1EC7E, 531117},
  {0x1EC7F, 532289},
  {0x1EC80, 532140},
  {0x1EC81, 531056},
  {0x1EC82, 531604},
  {0x1EC83, 531688},
  {0x1EC84, 532640},
  {0x1EC85, 532489},
  {0x1EC86, 531348},
  {0x1EC87, 531200},
  {0x1EC88, 532226},
  {0x1EC89, 532073},
  {0x1EC8A, 530989},
  {0x1EC8B, 531539},
  {0x1EC8C, 531719},
  {0x1EC8D, 532671},
  {0x1EC8E, 532522},
  {0x1EC8F, 531380},
  {0x1EC90, 531232},
  {0x1EC91, 532257},
  {0x1EC92, 532106},
  {0x1EC93, 531022},
  {0x1EC94, 531571},
  {0x1EC95, 532371},
  {0x1EC96, 532582},
  {0x1EC97, 532429},
  {0x1EC98, 531290},
  {0x1EC99, 531142},
  {0x1EC9A, 532314},
  {0x1EC9B, 532167},
  {0x1EC9C, 531082},
  {0x1EC9D, 531630},
  {0x1EC9E, 531465},
  {0x1EC9F, 531489},
  {0x1ECA0, 530834},
  {0x1ECA1, 531413},
  {0x1ECA2, 531438},
  {0x1ECA3, 531884},
  {0x1ECA4, 532016},
  {0x1ECA5, 531982},
  {0x1ECA6, 531818},
  {0x1ECA7, 531785},
  {0x1ECA8, 531950},
  {0x1ECA9, 531916},
  {0x1ECAA, 531751},
  {0x1ECAB, 531851},
  {0x1ECAC, 532703},
  {0x1ECAD, 530765},
  {0x1ECAE, 530735},
  {0x1ECAF, 530798},
  {0x1ECB0, 532727},
  {0x1ECB1, 530856},
  {0x1ECB2, 530931},
  {0x1ECB3, 530889},
  {0x1ECB4, 530703},
  {0x1ED01, 806538},
  {0x1ED02, 807237},
  {0x1ED03, 807074},
  {0x1ED04, 806283},
  {0x1ED05, 806125},
  {0x1ED06, 806795},
  {0x1ED07, 806630},
  {0x1ED08, 805899},
  {0x1ED09, 806378},
  {0x1ED0A, 806950},
  {0x1ED0B, 807172},
  {0x1ED0C, 807009},
  {0x1ED0D, 806220},
  {0x1ED0E, 806062},
  {0x1ED0F, 806887},
  {0x1ED10, 806728},
  {0x1ED11, 805997},
  {0x1ED12, 806473},
  {0x1ED13, 806563},
  {0x1ED14, 807262},
  {0x1ED15, 807101},
  {0x1ED16, 806309},
  {0x1ED17, 806151},
  {0x1ED18, 806820},
  {0x1ED19, 806657},
  {0x1ED1A, 805926},
  {0x1ED1B, 806404},
  {0x1ED1C, 806596},
  {0x1ED1D, 807295},
  {0x1ED1E, 807136},
  {0x1ED1F, 806343},
  {0x1ED20, 806185},
  {0x1ED21, 806853},
  {0x1ED22, 806692},
  {0x1ED23, 805961},
  {0x1ED24, 806438},
  {0x1ED25, 806975},
  {0x1ED26, 807200},
  {0x1ED27, 807037},
  {0x1ED28, 806247},
  {0x1ED29, 806089},
  {0x1ED2A, 806914},
  {0x1ED2B, 806757},
  {0x1ED2C, 806025},
  {0x1ED2D, 806501},
  {0x1ED2E, 805876},
  {0x1ED2F, 805732},
  {0x1ED30, 805695},
  {0x1ED31, 805385},
  {0x1ED32, 805349},
  {0x1ED33, 805538},
  {0x1ED34, 805501},
  {0x1ED35, 805312},
  {0x1ED36, 805465},
  {0x1ED37, 805616},
  {0x1ED38, 805421},
  {0x1ED39, 805573},
  {0x1ED3A, 805767},
  {0x1ED3B, 805651},
  {0x1ED3C, 805811},
  {0x1ED3D, 805843},
  {0x1EE00, 70476},
  {0x1EE01, 70501},
  {0x1EE02, 72388},
  {0x1EE03, 70549},
  {0x1EE05, 75003},
  {0x1EE06, 75075},
  {0x1EE07, 71714},
  {0x1EE08, 74415},
  {0x1EE09, 75027},
  {0x1EE0A, 72413},
  {0x1EE0B, 72462},
  {0x1EE0C, 73336},
  {0x1EE0D, 73361},
  {0x1EE0E, 73556},
  {0x1EE0F, 70452},
  {0x1EE10, 71664},
  {0x1EE11, 73532},
  {0x1EE12, 73484},
  {0x1EE13, 73508},
  {0x1EE14, 73581},
  {0x1EE15, 74929},
  {0x1EE16, 74978},
  {0x1EE17, 72437},
  {0x1EE18, 74953},
  {0x1EE19, 70525},
  {0x1EE1A, 75051},
  {0x1EE1B, 71688},
  {0x1EE1C, 70573},
  {0x1EE1D, 70637},
  {0x1EE1E, 70605},
  {0x1EE1F, 70670},
  {0x1EE21, 71770},
  {0x1EE22, 71964},
  {0x1EE24, 71932},
  {0x1EE27, 71900},
  {0x1EE29, 72356},
  {0x1EE2A, 71997},
  {0x1EE2B, 72062},
  {0x1EE2C, 72094},
  {0x1EE2D, 72127},
  {0x1EE2E, 72224},
  {0x1EE2F, 71738},
  {0x1EE30, 71834},
  {0x1EE31, 72192},
  {0x1EE32, 72160},
  {0x1EE34, 72257},
  {0x1EE35, 72291},
  {0x1EE36, 72323},
  {0x1EE37, 72029},
  {0x1EE39, 71802},
  {0x1EE3B, 71866},
  {0x1EE42, 74644},
  {0x1EE47, 74613},
  {0x1EE49, 74898},
  {0x1EE4B, 74708},
  {0x1EE4D, 74739},
  {0x1EE4E, 74833},
  {0x1EE4F, 74439},
  {0x1EE51, 74802},
  {0x1EE52, 74771},
  {0x1EE54, 74865},
  {0x1EE57, 74676},
  {0x1EE59, 74470},
  {0x1EE5B, 74580},
  {0x1EE5D, 74501},
  {0x1EE5F, 74541},
  {0x1EE61, 73641},
  {0x1EE62, 73931},
  {0x1EE64, 73897},
  {0x1EE67, 73863},
  {0x1EE68, 74244},
  {0x1EE69, 74347},
  {0x1EE6A, 73966},
  {0x1EE6C, 74035},
  {0x1EE6D, 74070},
  {0x1EE6E, 74173},
  {0x1EE6F, 73607},
  {0x1EE70, 73793},
  {0x1EE71, 74139},
  {0x1EE72, 74105},
  {0x1EE74, 74208},
  {0x1EE75, 74278},
  {0x1EE76, 74312},
  {0x1EE77, 74000},
  {0x1EE79, 73675},
  {0x1EE7A, 74381},
  {0x1EE7B, 73827},
  {0x1EE7C, 73709},
  {0x1EE7E, 73751},
  {0x1EE80, 72517},
  {0x1EE81, 72549},
  {0x1EE82, 72768},
  {0x1EE83, 72611},
  {0x1EE84, 72737},
  {0x1EE85, 73211},
  {0x1EE86, 73304},
  {0x1EE87, 72706},
  {0x1EE88, 73085},
  {0x1EE89, 73242},
  {0x1EE8B, 72832},
  {0x1EE8C, 72863},
  {0x1EE8D, 72895},
  {0x1EE8E, 73020},
  {0x1EE8F, 72486},
  {0x1EE90, 72642},
  {0x1EE91, 72989},
  {0x1EE92, 72927},
  {0x1EE93, 72958},
  {0x1EE94, 73052},
  {0x1EE95, 73116},
  {0x1EE96, 73179},
  {0x1EE97, 72800},
  {0x1EE98, 73147},
  {0x1EE99, 72580},
  {0x1EE9A, 73273},
  {0x1EE9B, 72673},
  {0x1EEA1, 70740},
  {0x1EEA2, 70970},
  {0x1EEA3, 70816},
  {0x1EEA5, 71511},
  {0x1EEA6, 71625},
  {0x1EEA7, 70932},
  {0x1EEA8, 71357},
  {0x1EEA9, 71549},
  {0x1EEAB, 71048},
  {0x1EEAC, 71086},
  {0x1EEAD, 71125},
  {0x1EEAE, 71278},
  {0x1EEAF, 70702},
  {0x1EEB0, 70854},
  {0x1EEB1, 71240},
  {0x1EEB2, 71164},
  {0x1EEB3, 71202},
  {0x1EEB4, 71317},
  {0x1EEB5, 71395},
  {0x1EEB6, 71472},
  {0x1EEB7, 71009},
  {0x1EEB8, 71433},
  {0x1EEB9, 70778},
  {0x1EEBA, 71587},
  {0x1EEBB, 70892},
  {0x1EEF0, 73428},
  {0x1EEF1, 73386},
  {0x1F000, 658680},
  {0x1F001, 659464},
  {0x1F002, 659708},
  {0x1F003, 659111},
  {0x1F004, 659260},
  {0x1F005, 658976},
  {0x1F006, 659731},
  {0x1F007, 659163},
  {0x1F008, 659649},
  {0x1F009, 659558},
  {0x1F00A, 658915},
  {0x1F00B, 658825},
  {0x1F00C, 659405},
  {0x1F00D, 659314},
  {0x1F00E, 658733},
  {0x1F00F, 659050},
  {0x1F010, 659135},
  {0x1F011, 659621},
  {0x1F012, 659528},
  {0x1F013, 658886},
  {0x1F014, 658796},
  {0x1F015, 659377},
  {0x1F016, 659284},
  {0x1F017, 658703},
  {0x1F018, 659021},
  {0x1F019, 659194},
  {0x1F01A, 659680},
  {0x1F01B, 659591},
  {0x1F01C, 658947},
  {0x1F01D, 658857},
  {0x1F01E, 659436},
  {0x1F01F, 659347},
  {0x1F020, 658766},
  {0x1F021, 659082},
  {0x1F022, 659242},
  {0x1F023, 659222},
  {0x1F024, 658633},
  {0x1F025, 658653},
  {0x1F026, 659488},
  {0x1F027, 659508},
  {0x1F028, 658595},
  {0x1F029, 659757},
  {0x1F02A, 659002},
  {0x1F02B, 658615},
  {0x1F030, 282150},
  {0x1F031, 282178},
  {0x1F032, 282207},
  {0x1F033, 282236},
  {0x1F034, 282265},
  {0x1F035, 282294},
  {0x1F036, 282323},
  {0x1F037, 282352},
  {0x1F038, 282381},
  {0x1F039, 282410},
  {0x1F03A, 282439},
  {0x1F03B, 282468},
  {0x1F03C, 282497},
  {0x1F03D, 282526},
  {0x1F03E, 282555},
  {0x1F03F, 282584},
  {0x1F040, 282613},
  {0x1F041, 282642},
  {0x1F042, 282671},
  {0x1F043, 282700},
  {0x1F044, 282729},
  {0x1F045, 282758},
  {0x1F046, 282787},
  {0x1F047, 282816},
  {0x1F048, 282845},
  {0x1F049, 282874},
  {0x1F04A, 282903},
  {0x1F04B, 282932},
  {0x1F04C, 282961},
  {0x1F04D, 282990},
  {0x1F04E, 283019},
  {0x1F04F, 283048},
  {0x1F050, 283077},
  {0x1F051, 283106},
  {0x1F052, 283135},
  {0x1F053, 283164},
  {0x1F054, 283193},
  {0x1F055, 283222},
  {0x1F056, 283251},
  {0x1F057, 283280},
  {0x1F058, 283309},
  {0x1F059, 283338},
  {0x1F05A, 283367},
  {0x1F05B, 283396},
  {0x1F05C, 283425},
  {0x1F05D, 283454},
  {0x1F05E, 283483},
  {0x1F05F, 283512},
  {0x1F060, 283541},
  {0x1F061, 283570},
  {0x1F062, 283599},
  {0x1F063, 283625},
  {0x1F064, 283652},
  {0x1F065, 283679},
  {0x1F066, 283706},
  {0x1F067, 283733},
  {0x1F068, 283760},
  {0x1F069, 283787},
  {0x1F06A, 283814},
  {0x1F06B, 283841},
  {0x1F06C, 283868},
  {0x1F06D, 283895},
  {0x1F06E, 283922},
  {0x1F06F, 283949},
  {0x1F070, 283976},
  {0x1F071, 284003},
  {0x1F072, 284030},
  {0x1F073, 284057},
  {0x1F074, 284084},
  {0x1F075, 284111},
  {0x1F076, 284138},
  {0x1F077, 284165},
  {0x1F078, 284192},
  {0x1F079, 284219},
  {0x1F07A, 284246},
  {0x1F07B, 284273},
  {0x1F07C, 284300},
  {0x1F07D, 284327},
  {0x1F07E, 284354},
  {0x1F07F, 284381},
  {0x1F080, 284408},
  {0x1F081, 284435},
  {0x1F082, 284462},
  {0x1F083, 284489},
  {0x1F084, 284516},
  {0x1F085, 284543},
  {0x1F086, 284570},
  {0x1F087, 284597},
  {0x1F088, 284624},
  {0x1F089, 284651},
  {0x1F08A, 284678},
  {0x1F08B, 284705},
  {0x1F08C, 284732},
  {0x1F08D, 284759},
  {0x1F08E, 284786},
  {0x1F08F, 284813},
  {0x1F090, 284840},
  {0x1F091, 284867},
  {0x1F092, 284894},
  {0x1F093, 284921},
  {0x1F0A0, 823373},
  {0x1F0A1, 823346},
  {0x1F0A2, 825364},
  {0x1F0A3, 824800},
  {0x1F0A4, 823749},
  {0x1F0A5, 823618},
  {0x1F0A6, 824576},
  {0x1F0A7, 824465},
  {0x1F0A8, 823504},
  {0x1F0A9, 824209},
  {0x1F0AA, 824685},
  {0x1F0AB, 823862},
  {0x1F0AC, 824094},
  {0x1F0AD, 824325},
  {0x1F0AE, 823975},
  {0x1F0B1, 823319},
  {0x1F0B2, 825337},
  {0x1F0B3, 824771},
  {0x1F0B4, 823721},
  {0x1F0B5, 823590},
  {0x1F0B6, 824549},
  {0x1F0B7, 824436},
  {0x1F0B8, 823475},
  {0x1F0B9, 824181},
  {0x1F0BA, 824658},
  {0x1F0BB, 823834},
  {0x1F0BC, 824064},
  {0x1F0BD, 824296},
  {0x1F0BE, 823947},
  {0x1F0BF, 824354},
  {0x1F0C1, 823290},
  {0x1F0C2, 825308},
  {0x1F0C3, 824740},
  {0x1F0C4, 823691},
  {0x1F0C5, 823560},
  {0x1F0C6, 824520},
  {0x1F0C7, 824405},
  {0x1F0C8, 823444},
  {0x1F0C9, 824151},
  {0x1F0CA, 824629},
  {0x1F0CB, 823804},
  {0x1F0CC, 824032},
  {0x1F0CD, 824265},
  {0x1F0CE, 823917},
  {0x1F0CF, 823391},
  {0x1F0D1, 823264},
  {0x1F0D2, 825282},
  {0x1F0D3, 824712},
  {0x1F0D4, 823664},
  {0x1F0D5, 823533},
  {0x1F0D6, 824494},
  {0x1F0D7, 824377},
  {0x1F0D8, 823416},
  {0x1F0D9, 824124},
  {0x1F0DA, 824603},
  {0x1F0DB, 823777},
  {0x1F0DC, 824003},
  {0x1F0DD, 824237},
  {0x1F0DE, 823890},
  {0x1F0DF, 825391},
  {0x1F0E0, 823646},
  {0x1F0E1, 824829},
  {0x1F0E2, 825070},
  {0x1F0E3, 825135},
  {0x1F0E4, 825156},
  {0x1F0E5, 825177},
  {0x1F0E6, 825198},
  {0x1F0E7, 825219},
  {0x1F0E8, 825240},
  {0x1F0E9, 825261},
  {0x1F0EA, 824850},
  {0x1F0EB, 824872},
  {0x1F0EC, 824894},
  {0x1F0ED, 824916},
  {0x1F0EE, 824938},
  {0x1F0EF, 824960},
  {0x1F0F0, 824982},
  {0x1F0F1, 825004},
  {0x1F0F2, 825026},
  {0x1F0F3, 825048},
  {0x1F0F4, 825091},
  {0x1F0F5, 825113},
  {0x1F100, 277175},
  {0x1F101, 277158},
  {0x1F102, 276915},
  {0x1F103, 277111},
  {0x1F104, 277061},
  {0x1F105, 276818},
  {0x1F106, 276769},
  {0x1F107, 277013},
  {0x1F108, 276963},
  {0x1F109, 276718},
  {0x1F10A, 276867},
  {0x1F10B, 277764},
  {0x1F10C, 278623},
  {0x1F10D, 191710},
  {0x1F10E, 184543},
  {0x1F10F, 185111},
  {0x1F110, 815248},
  {0x1F111, 815285},
  {0x1F112, 815322},
  {0x1F113, 815359},
  {0x1F114, 815396},
  {0x1F115, 815433},
  {0x1F116, 815470},
  {0x1F117, 815507},
  {0x1F118, 815544},
  {0x1F119, 815581},
  {0x1F11A, 815618},
  {0x1F11B, 815655},
  {0x1F11C, 815692},
  {0x1F11D, 815729},
  {0x1F11E, 815766},
  {0x1F11F, 815803},
  {0x1F120, 815840},
  {0x1F121, 815877},
  {0x1F122, 815914},
  {0x1F123, 815951},
  {0x1F124, 815988},
  {0x1F125, 816025},
  {0x1F126, 816062},
  {0x1F127, 816099},
  {0x1F128, 816136},
  {0x1F129, 816173},
  {0x1F12A, 949520},
  {0x1F12B, 187351},
  {0x1F12C, 187389},
  {0x1F12D, 184716},
  {0x1F12E, 191689},
  {0x1F12F, 214155},
  {0x1F130, 905177},
  {0x1F131, 905208},
  {0x1F132, 905239},
  {0x1F133, 905270},
  {0x1F134, 905301},
  {0x1F135, 905332},
  {0x1F136, 905363},
  {0x1F137, 905394},
  {0x1F138, 905425},
  {0x1F139, 905456},
  {0x1F13A, 905487},
  {0x1F13B, 905518},
  {0x1F13C, 905549},
  {0x1F13D, 905580},
  {0x1F13E, 905611},
  {0x1F13F, 905642},
  {0x1F140, 905673},
  {0x1F141, 905704},
  {0x1F142, 905735},
  {0x1F143, 905766},
  {0x1F144, 905797},
  {0x1F145, 905828},
  {0x1F146, 905859},
  {0x1F147, 905890},
  {0x1F148, 905921},
  {0x1F149, 905952},
  {0x1F14A, 905081},
  {0x1F14B, 906082},
  {0x1F14C, 906227},
  {0x1F14D, 906360},
  {0x1F14E, 906169},
  {0x1F14F, 906512},
  {0x1F150, 762147},
  {0x1F151, 762187},
  {0x1F152, 762227},
  {0x1F153, 762267},
  {0x1F154, 762307},
  {0x1F155, 762347},
  {0x1F156, 762387},
  {0x1F157, 762427},
  {0x1F158, 762467},
  {0x1F159, 762507},
  {0x1F15A, 762547},
  {0x1F15B, 762587},
  {0x1F15C, 762627},
  {0x1F15D, 762667},
  {0x1F15E, 762707},
  {0x1F15F, 762747},
  {0x1F160, 762787},
  {0x1F161, 762827},
  {0x1F162, 762867},
  {0x1F163, 762907},
  {0x1F164, 762947},
  {0x1F165, 762987},
  {0x1F166, 763027},
  {0x1F167, 763067},
  {0x1F168, 763107},
  {0x1F169, 763147},
  {0x1F16A, 830410},
  {0x1F16B, 830425},
  {0x1F16C, 830440},
  {0x1F16D, 184705},
  {0x1F16E, 184671},
  {0x1F16F, 185940},
  {0x1F170, 763758},
  {0x1F171, 763798},
  {0x1F172, 763838},
  {0x1F173, 763878},
  {0x1F174, 763918},
  {0x1F175, 763958},
  {0x1F176, 763998},
  {0x1F177, 764038},
  {0x1F178, 764078},
  {0x1F179, 764118},
  {0x1F17A, 764158},
  {0x1F17B, 764198},
  {0x1F17C, 764238},
  {0x1F17D, 764278},
  {0x1F17E, 764318},
  {0x1F17F, 764358},
  {0x1F180, 764398},
  {0x1F181, 764438},
  {0x1F182, 764478},
  {0x1F183, 764518},
  {0x1F184, 764558},
  {0x1F185, 764598},
  {0x1F186, 764638},
  {0x1F187, 764678},
  {0x1F188, 764718},
  {0x1F189, 764758},
  {0x1F18A, 215075},
  {0x1F18B, 763738},
  {0x1F18C, 764831},
  {0x1F18D, 764916},
  {0x1F18E, 763657},
  {0x1F18F, 764967},
  {0x1F190, 899949},
  {0x1F191, 904871},
  {0x1F192, 904882},
  {0x1F193, 905030},
  {0x1F194, 905092},
  {0x1F195, 906093},
  {0x1F196, 906105},
  {0x1F197, 906116},
  {0x1F198, 906333},
  {0x1F199, 906456},
  {0x1F19A, 906501},
  {0x1F19B, 906371},
  {0x1F19C, 906238},
  {0x1F19D, 906430},
  {0x1F19E, 905015},
  {0x1F19F, 904916},
  {0x1F1A0, 904963},
  {0x1F1A1, 906260},
  {0x1F1A2, 906401},
  {0x1F1A3, 906296},
  {0x1F1A4, 906127},
  {0x1F1A5, 905983},
  {0x1F1A6, 905043},
  {0x1F1A7, 905054},
  {0x1F1A8, 905066},
  {0x1F1A9, 906051},
  {0x1F1AA, 906284},
  {0x1F1AB, 906444},
  {0x1F1AC, 906489},
  {0x1F1AD, 668825},
  {0x1F1E6, 830925},
  {0x1F1E7, 830960},
  {0x1F1E8, 830995},
  {0x1F1E9, 831030},
  {0x1F1EA, 831065},
  {0x1F1EB, 831100},
  {0x1F1EC, 831135},
  {0x1F1ED, 831170},
  {0x1F1EE, 831205},
  {0x1F1EF, 831240},
  {0x1F1F0, 831275},
  {0x1F1F1, 831310},
  {0x1F1F2, 831345},
  {0x1F1F3, 831380},
  {0x1F1F4, 831415},
  {0x1F1F5, 831450},
  {0x1F1F6, 831485},
  {0x1F1F7, 831520},
  {0x1F1F8, 831555},
  {0x1F1F9, 831590},
  {0x1F1FA, 831625},
  {0x1F1FB, 831660},
  {0x1F1FC, 831695},
  {0x1F1FD, 831730},
  {0x1F1FE, 831765},
  {0x1F1FF, 831800},
  {0x1F200, 900537},
  {0x1F201, 905123},
  {0x1F202, 905145},
  {0x1F210, 904066},
  {0x1F211, 903961},
  {0x1F212, 903681},
  {0x1F213, 905103},
  {0x1F214, 903471},
  {0x1F215, 903891},
  {0x1F216, 904696},
  {0x1F217, 903926},
  {0x1F218, 903506},
  {0x1F219, 904311},
  {0x1F21A, 904486},
  {0x1F21B, 904241},
  {0x1F21C, 903611},
  {0x1F21D, 904031},
  {0x1F21E, 903541},
  {0x1F21F, 904276},
  {0x1F220, 903576},
  {0x1F221, 904661},
  {0x1F222, 904521},
  {0x1F223, 904731},
  {0x1F224, 903856},
  {0x1F225, 903786},
  {0x1F226, 904451},
  {0x1F227, 904136},
  {0x1F228, 904206},
  {0x1F229, 903366},
  {0x1F22A, 903401},
  {0x1F22B, 904801},
  {0x1F22C, 903996},
  {0x1F22D, 903436},
  {0x1F22E, 903716},
  {0x1F22F, 904171},
  {0x1F230, 904766},
  {0x1F231, 904101},
  {0x1F232, 904591},
  {0x1F233, 904626},
  {0x1F234, 903751},
  {0x1F235, 904416},
  {0x1F236, 904381},
  {0x1F237, 904346},
  {0x1F238, 904556},
  {0x1F239, 903646},
  {0x1F23A, 903821},
  {0x1F23B, 904836},
  {0x1F240, 949364},
  {0x1F241, 949052},
  {0x1F242, 949104},
  {0x1F243, 949208},
  {0x1F244, 949416},
  {0x1F245, 949260},
  {0x1F246, 949468},
  {0x1F247, 949156},
  {0x1F248, 949312},
  {0x1F250, 185986},
  {0x1F251, 185961},
  {0x1F260, 843102},
  {0x1F261, 843124},
  {0x1F262, 843146},
  {0x1F263, 843198},
  {0x1F264, 843170},
  {0x1F265, 843079},
  {0x1F300, 251789},
  {0x1F301, 444487},
  {0x1F302, 196334},
  {0x1F303, 770508},
  {0x1F304, 910044},
  {0x1F305, 910036},
  {0x1F306, 191795},
  {0x1F307, 910067},
  {0x1F308, 830184},
  {0x1F309, 141802},
  {0x1F30A, 982109},
  {0x1F30B, 978015},
  {0x1F30C, 725239},
  {0x1F30D, 293773},
  {0x1F30E, 293725},
  {0x1F30F, 293746},
  {0x1F310, 457855},
  {0x1F311, 766574},
  {0x1F312, 982311},
  {0x1F313, 444005},
  {0x1F314, 982339},
  {0x1F315, 445173},
  {0x1F316, 979645},
  {0x1F317, 582384},
  {0x1F318, 979617},
  {0x1F319, 214838},
  {0x1F31A, 766590},
  {0x1F31B, 444031},
  {0x1F31C, 582409},
  {0x1F31D, 445190},
  {0x1F31E, 908256},
  {0x1F31F, 457883},
  {0x1F320, 856190},
  {0x1F321, 937291},
  {0x1F322, 113485},
  {0x1F323, 986398},
  {0x1F324, 986484},
  {0x1F325, 986408},
  {0x1F326, 986431},
  {0x1F327, 196445},
  {0x1F328, 196461},
  {0x1F329, 196424},
  {0x1F32A, 196477},
  {0x1F32B, 444483},
  {0x1F32C, 988605},
  {0x1F32D, 524130},
  {0x1F32E, 916311},
  {0x1F32F, 142960},
  {0x1F330, 183323},
  {0x1F331, 850481},
  {0x1F332, 441841},
  {0x1F333, 269381},
  {0x1F334, 812086},
  {0x1F335, 153105},
  {0x1F336, 524138},
  {0x1F337, 951065},
  {0x1F338, 183308},
  {0x1F339, 842890},
  {0x1F33A, 521151},
  {0x1F33B, 910026},
  {0x1F33C, 121923},
  {0x1F33D, 293673},
  {0x1F33E, 293686},
  {0x1F33F, 519420},
  {0x1F340, 444779},
  {0x1F341, 665301},
  {0x1F342, 443465},
  {0x1F343, 625659},
  {0x1F344, 744231},
  {0x1F345, 947998},
  {0x1F346, 82150},
  {0x1F347, 460024},
  {0x1F348, 710960},
  {0x1F349, 982120},
  {0x1F34A, 928190},
  {0x1F34B, 634068},
  {0x1F34C, 104408},
  {0x1F34D, 823073},
  {0x1F34E, 830882},
  {0x1F34F, 480504},
  {0x1F350, 819203},
  {0x1F351, 819181},
  {0x1F352, 183299},
  {0x1F353, 907009},
  {0x1F354, 491124},
  {0x1F355, 891956},
  {0x1F356, 705510},
  {0x1F357, 826147},
  {0x1F358, 833800},
  {0x1F359, 833790},
  {0x1F35A, 209241},
  {0x1F35B, 251710},
  {0x1F35C, 906790},
  {0x1F35D, 899020},
  {0x1F35E, 141753},
  {0x1F35F, 445011},
  {0x1F360, 841916},
  {0x1F361, 269230},
  {0x1F362, 785675},
  {0x1F363, 911863},
  {0x1F364, 445024},
  {0x1F365, 444086},
  {0x1F366, 894144},
  {0x1F367, 855182},
  {0x1F368, 524466},
  {0x1F369, 287180},
  {0x1F36A, 209253},
  {0x1F36B, 183388},
  {0x1F36C, 173635},
  {0x1F36D, 652218},
  {0x1F36E, 251761},
  {0x1F36F, 523343},
  {0x1F370, 856527},
  {0x1F371, 108993},
  {0x1F372, 826088},
  {0x1F373, 209260},
  {0x1F374, 444555},
  {0x1F375, 929843},
  {0x1F376, 846290},
  {0x1F377, 988641},
  {0x1F378, 196522},
  {0x1F379, 950990},
  {0x1F37A, 106792},
  {0x1F37B, 195071},
  {0x1F37C, 83270},
  {0x1F37D, 444570},
  {0x1F37E, 123631},
  {0x1F37F, 825987},
  {0x1F380, 833603},
  {0x1F381, 988879},
  {0x1F382, 111311},
  {0x1F383, 536189},
  {0x1F384, 184078},
  {0x1F385, 443631},
  {0x1F386, 443958},
  {0x1F387, 443940},
  {0x1F388, 87097},
  {0x1F389, 817537},
  {0x1F38A, 208830},
  {0x1F38B, 928176},
  {0x1F38C, 215061},
  {0x1F38D, 823057},
  {0x1F38E, 536241},
  {0x1F38F, 174626},
  {0x1F390, 988623},
  {0x1F391, 742288},
  {0x1F392, 850091},
  {0x1F393, 458497},
  {0x1F394, 506084},
  {0x1F395, 124703},
  {0x1F396, 725224},
  {0x1F397, 832558},
  {0x1F398, 744324},
  {0x1F399, 907193},
  {0x1F39A, 636467},
  {0x1F39B, 209209},
  {0x1F39C, 106622},
  {0x1F39D, 106653},
  {0x1F39E, 443833},
  {0x1F39F, 2221},
  {0x1F3A0, 174611},
  {0x1F3A1, 443736},
  {0x1F3A2, 841994},
  {0x1F3A3, 444122},
  {0x1F3A4, 725024},
  {0x1F3A5, 742512},
  {0x1F3A6, 184100},
  {0x1F3A7, 505953},
  {0x1F3A8, 81956},
  {0x1F3A9, 948622},
  {0x1F3AA, 191773},
  {0x1F3AB, 943633},
  {0x1F3AC, 194998},
  {0x1F3AD, 819357},
  {0x1F3AE, 976040},
  {0x1F3AF, 278748},
  {0x1F3B0, 892057},
  {0x1F3B1, 111281},
  {0x1F3B2, 448104},
  {0x1F3B3, 124768},
  {0x1F3B4, 444320},
  {0x1F3B5, 744352},
  {0x1F3B6, 743904},
  {0x1F3B7, 850061},
  {0x1F3B8, 480793},
  {0x1F3B9, 744307},
  {0x1F3BA, 951045},
  {0x1F3BB, 976157},
  {0x1F3BC, 744365},
  {0x1F3BD, 845975},
  {0x1F3BE, 932289},
  {0x1F3BF, 891562},
  {0x1F3C0, 104574},
  {0x1F3C1, 179472},
  {0x1F3C2, 894063},
  {0x1F3C3, 845968},
  {0x1F3C4, 911856},
  {0x1F3C5, 899537},
  {0x1F3C6, 950983},
  {0x1F3C7, 524095},
  {0x1F3C8, 9275},
  {0x1F3C9, 843239},
  {0x1F3CA, 911918},
  {0x1F3CB, 982430},
  {0x1F3CC, 457918},
  {0x1F3CD, 830077},
  {0x1F3CE, 830066},
  {0x1F3CF, 214860},
  {0x1F3D0, 978023},
  {0x1F3D1, 443755},
  {0x1F3D2, 524485},
  {0x1F3D3, 916282},
  {0x1F3D4, 894042},
  {0x1F3D5, 153177},
  {0x1F3D6, 106602},
  {0x1F3D7, 142871},
  {0x1F3D8, 524226},
  {0x1F3D9, 191785},
  {0x1F3DA, 270341},
  {0x1F3DB, 195032},
  {0x1F3DC, 272533},
  {0x1F3DD, 272540},
  {0x1F3DE, 762018},
  {0x1F3DF, 906558},
  {0x1F3E0, 524211},
  {0x1F3E1, 524242},
  {0x1F3E2, 785680},
  {0x1F3E3, 536322},
  {0x1F3E4, 441820},
  {0x1F3E5, 524108},
  {0x1F3E6, 104421},
  {0x1F3E7, 82187},
  {0x1F3E8, 524161},
  {0x1F3E9, 652730},
  {0x1F3EA, 209223},
  {0x1F3EB, 850084},
  {0x1F3EC, 270324},
  {0x1F3ED, 443443},
  {0x1F3EE, 536173},
  {0x1F3EF, 536225},
  {0x1F3F0, 441804},
  {0x1F3F1, 985479},
  {0x1F3F2, 114879},
  {0x1F3F3, 982245},
  {0x1F3F4, 982210},
  {0x1F3F5, 842895},
  {0x1F3F6, 115488},
  {0x1F3F7, 578333},
  {0x1F3F8, 83512},
  {0x1F3F9, 124722},
  {0x1F3FA, 9303},
  {0x1F3FB, 428414},
  {0x1F3FC, 428450},
  {0x1F3FD, 428484},
  {0x1F3FE, 428518},
  {0x1F3FF, 428552},
  {0x1F400, 830506},
  {0x1F401, 742478},
  {0x1F402, 808591},
  {0x1F403, 982071},
  {0x1F404, 214808},
  {0x1F405, 945004},
  {0x1F406, 634078},
  {0x1F407, 830039},
  {0x1F408, 174668},
  {0x1F409, 290163},
  {0x1F40A, 214881},
  {0x1F40B, 982462},
  {0x1F40C, 894016},
  {0x1F40D, 894022},
  {0x1F40E, 524078},
  {0x1F40F, 830502},
  {0x1F410, 457905},
  {0x1F411, 856125},
  {0x1F412, 742154},
  {0x1F413, 842867},
  {0x1F414, 183340},
  {0x1F415, 281069},
  {0x1F416, 822968},
  {0x1F417, 121973},
  {0x1F418, 427797},
  {0x1F419, 785667},
  {0x1F41A, 899463},
  {0x1F41B, 141947},
  {0x1F41C, 24720},
  {0x1F41D, 523353},
  {0x1F41E, 578370},
  {0x1F41F, 444081},
  {0x1F420, 951005},
  {0x1F421, 121931},
  {0x1F422, 953730},
  {0x1F423, 505425},
  {0x1F424, 83282},
  {0x1F425, 445047},
  {0x1F426, 111306},
  {0x1F427, 819252},
  {0x1F428, 578273},
  {0x1F429, 825929},
  {0x1F42A, 290204},
  {0x1F42B, 83490},
  {0x1F42C, 282142},
  {0x1F42D, 742484},
  {0x1F42E, 214812},
  {0x1F42F, 945010},
  {0x1F430, 830046},
  {0x1F431, 174672},
  {0x1F432, 290170},
  {0x1F433, 899550},
  {0x1F434, 524084},
  {0x1F435, 742161},
  {0x1F436, 281073},
  {0x1F437, 822972},
  {0x1F438, 445037},
  {0x1F439, 491199},
  {0x1F43A, 988694},
  {0x1F43B, 106734},
  {0x1F43C, 812868},
  {0x1F43D, 822981},
  {0x1F43E, 819149},
  {0x1F43F, 183372},
  {0x1F440, 442552},
  {0x1F441, 442537},
  {0x1F442, 293669},
  {0x1F443, 773265},
  {0x1F444, 742506},
  {0x1F445, 948005},
  {0x1F446, 986647},
  {0x1F447, 984454},
  {0x1F448, 985076},
  {0x1F449, 985553},
  {0x1F44A, 444144},
  {0x1F44B, 982228},
  {0x1F44C, 786278},
  {0x1F44D, 938089},
  {0x1F44E, 938072},
  {0x1F44F, 195012},
  {0x1F450, 800725},
  {0x1F451, 215153},
  {0x1F452, 988760},
  {0x1F453, 442541},
  {0x1F454, 762060},
  {0x1F455, 916274},
  {0x1F456, 538450},
  {0x1F457, 290182},
  {0x1F458, 576875},
  {0x1F459, 111263},
  {0x1F45A, 988745},
  {0x1F45B, 829457},
  {0x1F45C, 491296},
  {0x1F45D, 826141},
  {0x1F45E, 665229},
  {0x1F45F, 82124},
  {0x1F460, 521201},
  {0x1F461, 988771},
  {0x1F462, 988732},
  {0x1F463, 444529},
  {0x1F464, 142981},
  {0x1F465, 143000},
  {0x1F466, 132700},
  {0x1F467, 454708},
  {0x1F468, 663227},
  {0x1F469, 988704},
  {0x1F46A, 443611},
  {0x1F46B, 663231},
  {0x1F46C, 954073},
  {0x1F46D, 954145},
  {0x1F46E, 825914},
  {0x1F46F, 988710},
  {0x1F470, 141786},
  {0x1F471, 819627},
  {0x1F472, 663317},
  {0x1F473, 663337},
  {0x1F474, 800308},
  {0x1F475, 800318},
  {0x1F476, 83254},
  {0x1F477, 208943},
  {0x1F478, 828374},
  {0x1F479, 536308},
  {0x1F47A, 536256},
  {0x1F47B, 454664},
  {0x1F47C, 83259},
  {0x1F47D, 442331},
  {0x1F47E, 8864},
  {0x1F47F, 529629},
  {0x1F480, 891585},
  {0x1F481, 532794},
  {0x1F482, 480773},
  {0x1F483, 269223},
  {0x1F484, 650790},
  {0x1F485, 760385},
  {0x1F486, 442581},
  {0x1F487, 487425},
  {0x1F488, 104546},
  {0x1F489, 916260},
  {0x1F48A, 823015},
  {0x1F48B, 578119},
  {0x1F48C, 652741},
  {0x1F48D, 841723},
  {0x1F48E, 449686},
  {0x1F48F, 578114},
  {0x1F490, 124695},
  {0x1F491, 214790},
  {0x1F492, 982392},
  {0x1F493, 106759},
  {0x1F494, 141890},
  {0x1F495, 953937},
  {0x1F496, 899047},
  {0x1F497, 480746},
  {0x1F498, 506049},
  {0x1F499, 121950},
  {0x1F49A, 480527},
  {0x1F49B, 989276},
  {0x1F49C, 829444},
  {0x1F49D, 506066},
  {0x1F49E, 833565},
  {0x1F49F, 506020},
  {0x1F4A0, 276438},
  {0x1F4A1, 427447},
  {0x1F4A2, 24616},
  {0x1F4A3, 122122},
  {0x1F4A4, 891914},
  {0x1F4A5, 196557},
  {0x1F4A6, 899476},
  {0x1F4A7, 290248},
  {0x1F4A8, 269263},
  {0x1F4A9, 823003},
  {0x1F4AA, 444274},
  {0x1F4AB, 280647},
  {0x1F4AC, 899298},
  {0x1F4AD, 937389},
  {0x1F4AE, 984759},
  {0x1F4AF, 524286},
  {0x1F4B0, 737466},
  {0x1F4B1, 251678},
  {0x1F4B2, 506792},
  {0x1F4B3, 214826},
  {0x1F4B4, 104501},
  {0x1F4B5, 104426},
  {0x1F4B6, 104452},
  {0x1F4B7, 104476},
  {0x1F4B8, 737476},
  {0x1F4B9, 179358},
  {0x1F4BA, 850412},
  {0x1F4BB, 819740},
  {0x1F4BC, 141818},
  {0x1F4BD, 725267},
  {0x1F4BE, 444288},
  {0x1F4BF, 800958},
  {0x1F4C0, 293645},
  {0x1F4C1, 443821},
  {0x1F4C2, 800696},
  {0x1F4C3, 808652},
  {0x1F4C4, 808614},
  {0x1F4C5, 153130},
  {0x1F4C6, 929872},
  {0x1F4C7, 173705},
  {0x1F4C8, 179333},
  {0x1F4C9, 179306},
  {0x1F4CA, 104524},
  {0x1F4CB, 195107},
  {0x1F4CC, 829463},
  {0x1F4CD, 843052},
  {0x1F4CE, 812879},
  {0x1F4CF, 906981},
  {0x1F4D0, 950361},
  {0x1F4D1, 122146},
  {0x1F4D2, 625709},
  {0x1F4D3, 773842},
  {0x1F4D4, 773851},
  {0x1F4D5, 196120},
  {0x1F4D6, 800578},
  {0x1F4D7, 480516},
  {0x1F4D8, 121940},
  {0x1F4D9, 801019},
  {0x1F4DA, 122160},
  {0x1F4DB, 760408},
  {0x1F4DC, 850392},
  {0x1F4DD, 710979},
  {0x1F4DE, 930009},
  {0x1F4DF, 808667},
  {0x1F4E0, 443657},
  {0x1F4E1, 848138},
  {0x1F4E2, 829377},
  {0x1F4E3, 179440},
  {0x1F4E4, 807340},
  {0x1F4E5, 530558},
  {0x1F4E6, 808601},
  {0x1F4E7, 293649},
  {0x1F4E8, 530569},
  {0x1F4E9, 428916},
  {0x1F4EA, 196185},
  {0x1F4EB, 196218},
  {0x1F4EC, 800782},
  {0x1F4ED, 800751},
  {0x1F4EE, 826080},
  {0x1F4EF, 826039},
  {0x1F4F0, 770488},
  {0x1F4F1, 725485},
  {0x1F4F2, 725515},
  {0x1F4F3, 975999},
  {0x1F4F4, 725498},
  {0x1F4F5, 771700},
  {0x1F4F6, 24732},
  {0x1F4F7, 153152},
  {0x1F4F8, 153159},
  {0x1F4F9, 976027},
  {0x1F4FA, 930101},
  {0x1F4FB, 830117},
  {0x1F4FC, 976051},
  {0x1F4FD, 443845},
  {0x1F4FE, 825995},
  {0x1F4FF, 826283},
  {0x1F500, 953763},
  {0x1F501, 195903},
  {0x1F502, 195957},
  {0x1F503, 195732},
  {0x1F504, 24815},
  {0x1F505, 652778},
  {0x1F506, 521160},
  {0x1F507, 899136},
  {0x1F508, 899128},
  {0x1F509, 899169},
  {0x1F50A, 899197},
  {0x1F50B, 106594},
  {0x1F50C, 427467},
  {0x1F50D, 629739},
  {0x1F50E, 837209},
  {0x1F50F, 651674},
  {0x1F510, 196164},
  {0x1F511, 552364},
  {0x1F512, 651669},
  {0x1F513, 800741},
  {0x1F514, 106808},
  {0x1F515, 106837},
  {0x1F516, 122137},
  {0x1F517, 650745},
  {0x1F518, 830123},
  {0x1F519, 83322},
  {0x1F51A, 428863},
  {0x1F51B, 800346},
  {0x1F51C, 895402},
  {0x1F51D, 949023},
  {0x1F51E, 771717},
  {0x1F51F, 552396},
  {0x1F520, 532892},
  {0x1F521, 532962},
  {0x1F522, 532999},
  {0x1F523, 533024},
  {0x1F524, 532931},
  {0x1F525, 443893},
  {0x1F526, 427481},
  {0x1F527, 988910},
  {0x1F528, 491134},
  {0x1F529, 782928},
  {0x1F52A, 523247},
  {0x1F52B, 823115},
  {0x1F52C, 725035},
  {0x1F52D, 930091},
  {0x1F52E, 215208},
  {0x1F52F, 891476},
  {0x1F530, 536343},
  {0x1F531, 950387},
  {0x1F532, 115811},
  {0x1F533, 985896},
  {0x1F534, 580214},
  {0x1F535, 579842},
  {0x1F536, 580133},
  {0x1F537, 579860},
  {0x1F538, 892686},
  {0x1F539, 892122},
  {0x1F53A, 955890},
  {0x1F53B, 287499},
  {0x1F53C, 955984},
  {0x1F53D, 287572},
  {0x1F53E, 656442},
  {0x1F53F, 959680},
  {0x1F540, 184819},
  {0x1F541, 215025},
  {0x1F542, 215012},
  {0x1F543, 773633},
  {0x1F544, 773725},
  {0x1F545, 913749},
  {0x1F546, 985036},
  {0x1F547, 507486},
  {0x1F548, 176383},
  {0x1F549, 800336},
  {0x1F54A, 287189},
  {0x1F54B, 538522},
  {0x1F54C, 742316},
  {0x1F54D, 914185},
  {0x1F54E, 717800},
  {0x1F54F, 124736},
  {0x1F550, 195353},
  {0x1F551, 195631},
  {0x1F552, 195533},
  {0x1F553, 195261},
  {0x1F554, 195215},
  {0x1F555, 195445},
  {0x1F556, 195397},
  {0x1F557, 195117},
  {0x1F558, 195307},
  {0x1F559, 195489},
  {0x1F55A, 195165},
  {0x1F55B, 195581},
  {0x1F55C, 195375},
  {0x1F55D, 195653},
  {0x1F55E, 195557},
  {0x1F55F, 195284},
  {0x1F560, 195238},
  {0x1F561, 195467},
  {0x1F562, 195421},
  {0x1F563, 195141},
  {0x1F564, 195330},
  {0x1F565, 195511},
  {0x1F566, 195190},
  {0x1F567, 195606},
  {0x1F568, 835565},
  {0x1F569, 835579},
  {0x1F56A, 835613},
  {0x1F56B, 142916},
  {0x1F56C, 142925},
  {0x1F56D, 841819},
  {0x1F56E, 122132},
  {0x1F56F, 173628},
  {0x1F570, 665239},
  {0x1F571, 115557},
  {0x1F572, 771761},
  {0x1F573, 523253},
  {0x1F574, 663271},
  {0x1F575, 891942},
  {0x1F576, 269247},
  {0x1F577, 899409},
  {0x1F578, 899416},
  {0x1F579, 538491},
  {0x1F57A, 663259},
  {0x1F57B, 627264},
  {0x1F57C, 930028},
  {0x1F57D, 835035},
  {0x1F57E, 986527},
  {0x1F57F, 115977},
  {0x1F580, 929983},
  {0x1F581, 194975},
  {0x1F582, 83305},
  {0x1F583, 906603},
  {0x1F584, 428952},
  {0x1F585, 444400},
  {0x1F586, 819219},
  {0x1F587, 650757},
  {0x1F588, 114908},
  {0x1F589, 654171},
  {0x1F58A, 653400},
  {0x1F58B, 654094},
  {0x1F58C, 654149},
  {0x1F58D, 654076},
  {0x1F58E, 629227},
  {0x1F58F, 953505},
  {0x1F590, 830295},
  {0x1F591, 833285},
  {0x1F592, 833460},
  {0x1F593, 833434},
  {0x1F594, 833543},
  {0x1F595, 833165},
  {0x1F596, 830328},
  {0x1F597, 984515},
  {0x1F598, 859246},
  {0x1F599, 859281},
  {0x1F59A, 859107},
  {0x1F59B, 859142},
  {0x1F59C, 113922},
  {0x1F59D, 114967},
  {0x1F59E, 859317},
  {0x1F59F, 859211},
  {0x1F5A0, 859178},
  {0x1F5A1, 859072},
  {0x1F5A2, 116079},
  {0x1F5A3, 113228},
  {0x1F5A4, 113591},
  {0x1F5A5, 272554},
  {0x1F5A6, 552377},
  {0x1F5A7, 937581},
  {0x1F5A8, 828403},
  {0x1F5A9, 825857},
  {0x1F5AA, 113562},
  {0x1F5AB, 984840},
  {0x1F5AC, 894159},
  {0x1F5AD, 929805},
  {0x1F5AE, 988670},
  {0x1F5AF, 800487},
  {0x1F5B0, 953822},
  {0x1F5B1, 937405},
  {0x1F5B2, 950088},
  {0x1F5B3, 795340},
  {0x1F5B4, 505410},
  {0x1F5B5, 850132},
  {0x1F5B6, 828411},
  {0x1F5B7, 443648},
  {0x1F5B8, 800971},
  {0x1F5B9, 280729},
  {0x1F5BA, 280748},
  {0x1F5BB, 280707},
  {0x1F5BC, 444943},
  {0x1F5BD, 444962},
  {0x1F5BE, 444927},
  {0x1F5BF, 113525},
  {0x1F5C0, 444493},
  {0x1F5C1, 800713},
  {0x1F5C2, 173716},
  {0x1F5C3, 173691},
  {0x1F5C4, 443808},
  {0x1F5C5, 428628},
  {0x1F5C6, 428654},
  {0x1F5C7, 428639},
  {0x1F5C8, 773818},
  {0x1F5C9, 773832},
  {0x1F5CA, 773823},
  {0x1F5CB, 428602},
  {0x1F5CC, 428670},
  {0x1F5CD, 428681},
  {0x1F5CE, 280698},
  {0x1F5CF, 808609},
  {0x1F5D0, 808673},
  {0x1F5D1, 982053},
  {0x1F5D2, 899447},
  {0x1F5D3, 899427},
  {0x1F5D4, 272571},
  {0x1F5D5, 725276},
  {0x1F5D6, 704466},
  {0x1F5D7, 808482},
  {0x1F5D8, 195860},
  {0x1F5D9, 173606},
  {0x1F5DA, 530607},
  {0x1F5DB, 269449},
  {0x1F5DC, 208818},
  {0x1F5DD, 792456},
  {0x1F5DE, 841974},
  {0x1F5DF, 808629},
  {0x1F5E0, 906959},
  {0x1F5E1, 269150},
  {0x1F5E2, 650785},
  {0x1F5E3, 899228},
  {0x1F5E4, 937632},
  {0x1F5E5, 937649},
  {0x1F5E6, 937666},
  {0x1F5E7, 937682},
  {0x1F5E8, 628233},
  {0x1F5E9, 835650},
  {0x1F5EA, 954126},
  {0x1F5EB, 937723},
  {0x1F5EC, 628689},
  {0x1F5ED, 836121},
  {0x1F5EE, 626045},
  {0x1F5EF, 833885},
  {0x1F5F0, 742254},
  {0x1F5F1, 636985},
  {0x1F5F2, 636970},
  {0x1F5F3, 87161},
  {0x1F5F4, 87330},
  {0x1F5F5, 87287},
  {0x1F5F6, 87129},
  {0x1F5F7, 87211},
  {0x1F5F8, 636529},
  {0x1F5F9, 87184},
  {0x1F5FA, 988851},
  {0x1F5FB, 742404},
  {0x1F5FC, 947986},
  {0x1F5FD, 906755},
  {0x1F5FE, 887779},
  {0x1F5FF, 742525},
  {0x1F600, 480613},
  {0x1F601, 480674},
  {0x1F602, 443327},
  {0x1F603, 893629},
  {0x1F604, 893702},
  {0x1F605, 893658},
  {0x1F606, 893748},
  {0x1F607, 893546},
  {0x1F608, 893605},
  {0x1F609, 988657},
  {0x1F60A, 893801},
  {0x1F60B, 442604},
  {0x1F60C, 832544},
  {0x1F60D, 893569},
  {0x1F60E, 893935},
  {0x1F60F, 893987},
  {0x1F610, 766561},
  {0x1F611, 441981},
  {0x1F612, 954870},
  {0x1F613, 442704},
  {0x1F614, 819260},
  {0x1F615, 208860},
  {0x1F616, 208844},
  {0x1F617, 578163},
  {0x1F618, 442657},
  {0x1F619, 578206},
  {0x1F61A, 578176},
  {0x1F61B, 443206},
  {0x1F61C, 443284},
  {0x1F61D, 443233},
  {0x1F61E, 278803},
  {0x1F61F, 988866},
  {0x1F620, 24675},
  {0x1F621, 826202},
  {0x1F622, 215196},
  {0x1F623, 819429},
  {0x1F624, 442832},
  {0x1F625, 278772},
  {0x1F626, 445122},
  {0x1F627, 24700},
  {0x1F628, 443669},
  {0x1F629, 982381},
  {0x1F62A, 891930},
  {0x1F62B, 945381},
  {0x1F62C, 480562},
  {0x1F62D, 652711},
  {0x1F62E, 443015},
  {0x1F62F, 524308},
  {0x1F630, 443036},
  {0x1F631, 442634},
  {0x1F632, 82045},
  {0x1F633, 444365},
  {0x1F634, 891900},
  {0x1F635, 280636},
  {0x1F636, 443409},
  {0x1F637, 442858},
  {0x1F638, 480577},
  {0x1F639, 174681},
  {0x1F63A, 893513},
  {0x1F63B, 893473},
  {0x1F63C, 174708},
  {0x1F63D, 578129},
  {0x1F63E, 826185},
  {0x1F63F, 215180},
  {0x1F640, 982366},
  {0x1F641, 891971},
  {0x1F642, 891994},
  {0x1F643, 960066},
  {0x1F644, 443183},
  {0x1F645, 442899},
  {0x1F646, 442925},
  {0x1F647, 819446},
  {0x1F648, 850462},
  {0x1F649, 506000},
  {0x1F64A, 899107},
  {0x1F64B, 505380},
  {0x1F64C, 819569},
  {0x1F64D, 819506},
  {0x1F64E, 819715},
  {0x1F64F, 819668},
  {0x1F650, 773058},
  {0x1F651, 896905},
  {0x1F652, 772262},
  {0x1F653, 896547},
  {0x1F654, 953457},
  {0x1F655, 953673},
  {0x1F656, 953425},
  {0x1F657, 953641},
  {0x1F658, 773083},
  {0x1F659, 896930},
  {0x1F65A, 772287},
  {0x1F65B, 896572},
  {0x1F65C, 508036},
  {0x1F65D, 508779},
  {0x1F65E, 507970},
  {0x1F65F, 508713},
  {0x1F660, 773034},
  {0x1F661, 896881},
  {0x1F662, 772238},
  {0x1F663, 896523},
  {0x1F664, 508006},
  {0x1F665, 508749},
  {0x1F666, 507940},
  {0x1F667, 508683},
  {0x1F668, 523258},
  {0x1F669, 523287},
  {0x1F66A, 895292},
  {0x1F66B, 895320},
  {0x1F66C, 632707},
  {0x1F66D, 961247},
  {0x1F66E, 840301},
  {0x1F66F, 289136},
  {0x1F670, 850304},
  {0x1F671, 508478},
  {0x1F672, 636486},
  {0x1F673, 507701},
  {0x1F674, 506111},
  {0x1F675, 911893},
  {0x1F676, 847972},
  {0x1F677, 847918},
  {0x1F678, 848033},
  {0x1F679, 507440},
  {0x1F67A, 848091},
  {0x1F67B, 508440},
  {0x1F67C, 975883},
  {0x1F67D, 975806},
  {0x1F67E, 179407},
  {0x1F67F, 832701},
  {0x1F680, 841953},
  {0x1F681, 512789},
  {0x1F682, 906773},
  {0x1F683, 830153},
  {0x1F684, 521218},
  {0x1F685, 521235},
  {0x1F686, 950122},
  {0x1F687, 722212},
  {0x1F688, 636752},
  {0x1F689, 906747},
  {0x1F68A, 950128},
  {0x1F68B, 950133},
  {0x1F68C, 142968},
  {0x1F68D, 800419},
  {0x1F68E, 950972},
  {0x1F68F, 142972},
  {0x1F690, 725259},
  {0x1F691, 9265},
  {0x1F692, 443898},
  {0x1F693, 825875},
  {0x1F694, 800453},
  {0x1F695, 929838},
  {0x1F696, 800473},
  {0x1F697, 82212},
  {0x1F698, 800399},
  {0x1F699, 830541},
  {0x1F69A, 269525},
  {0x1F69B, 81938},
  {0x1F69C, 950098},
  {0x1F69D, 742227},
  {0x1F69E, 742461},
  {0x1F69F, 911869},
  {0x1F6A0, 742443},
  {0x1F6A1, 3697},
  {0x1F6A2, 856152},
  {0x1F6A3, 843220},
  {0x1F6A4, 899313},
  {0x1F6A5, 524030},
  {0x1F6A6, 975616},
  {0x1F6A7, 208925},
  {0x1F6A8, 825886},
  {0x1F6A9, 950337},
  {0x1F6AA, 284965},
  {0x1F6AB, 771686},
  {0x1F6AC, 894001},
  {0x1F6AD, 771771},
  {0x1F6AE, 829471},
  {0x1F6AF, 280677},
  {0x1F6B0, 826100},
  {0x1F6B1, 771858},
  {0x1F6B2, 111213},
  {0x1F6B3, 771665},
  {0x1F6B4, 111221},
  {0x1F6B5, 742424},
  {0x1F6B6, 819208},
  {0x1F6B7, 771746},
  {0x1F6B8, 183354},
  {0x1F6B9, 717827},
  {0x1F6BA, 988785},
  {0x1F6BB, 832653},
  {0x1F6BC, 83293},
  {0x1F6BD, 947979},
  {0x1F6BE, 982085},
  {0x1F6BF, 856691},
  {0x1F6C0, 106581},
  {0x1F6C1, 106586},
  {0x1F6C2, 817641},
  {0x1F6C3, 251769},
  {0x1F6C4, 83552},
  {0x1F6C5, 627399},
  {0x1F6C6, 950202},
  {0x1F6C7, 828437},
  {0x1F6C8, 187324},
  {0x1F6C9, 132704},
  {0x1F6CA, 454713},
  {0x1F6CB, 214229},
  {0x1F6CC, 891877},
  {0x1F6CD, 856204},
  {0x1F6CE, 106867},
  {0x1F6CF, 106788},
  {0x1F6D0, 823186},
  {0x1F6D1, 785652},
  {0x1F6D2, 856218},
  {0x1F6D3, 907229},
  {0x1F6D4, 808679},
  {0x1F6D5, 521281},
  {0x1F6D6, 524320},
  {0x1F6D7, 427806},
  {0x1F6DC, 988685},
  {0x1F6DD, 823247},
  {0x1F6DE, 982468},
  {0x1F6DF, 841739},
  {0x1F6E0, 491175},
  {0x1F6E1, 856131},
  {0x1F6E2, 786269},
  {0x1F6E3, 742395},
  {0x1F6E4, 830165},
  {0x1F6E5, 742349},
  {0x1F6E6, 955837},
  {0x1F6E7, 955731},
  {0x1F6E8, 955957},
  {0x1F6E9, 892076},
  {0x1F6EA, 773237},
  {0x1F6EB, 4897},
  {0x1F6EC, 4879},
  {0x1F6F0, 848128},
  {0x1F6F1, 800432},
  {0x1F6F2, 276669},
  {0x1F6F3, 817564},
  {0x1F6F4, 850106},
  {0x1F6F5, 742360},
  {0x1F6F6, 173653},
  {0x1F6F7, 891872},
  {0x1F6F8, 444416},
  {0x1F6F9, 891551},
  {0x1F6FA, 82173},
  {0x1F6FB, 822951},
  {0x1F6FC, 842009},
  {0x1F700, 7208},
  {0x1F701, 4941},
  {0x1F702, 6301},
  {0x1F703, 6273},
  {0x1F704, 8739},
  {0x1F705, 5285},
  {0x1F706, 5149},
  {0x1F707, 5182},
  {0x1F708, 5217},
  {0x1F709, 5250},
  {0x1F70A, 8541},
  {0x1F70B, 8613},
  {0x1F70C, 8645},
  {0x1F70D, 8273},
  {0x1F70E, 6937},
  {0x1F70F, 5517},
  {0x1F710, 6703},
  {0x1F711, 6743},
  {0x1F712, 6785},
  {0x1F713, 5762},
  {0x1F714, 7685},
  {0x1F715, 6883},
  {0x1F716, 8677},
  {0x1F717, 8707},
  {0x1F718, 7584},
  {0x1F719, 7616},
  {0x1F71A, 6328},
  {0x1F71B, 7838},
  {0x1F71C, 6506},
  {0x1F71D, 6537},
  {0x1F71E, 5947},
  {0x1F71F, 7421},
  {0x1F720, 5834},
  {0x1F721, 6570},
  {0x1F722, 8095},
  {0x1F723, 5867},
  {0x1F724, 5906},
  {0x1F725, 5793},
  {0x1F726, 7751},
  {0x1F727, 8189},
  {0x1F728, 8509},
  {0x1F729, 8362},
  {0x1F72A, 6608},
  {0x1F72B, 5114},
  {0x1F72C, 8051},
  {0x1F72D, 7712},
  {0x1F72E, 8137},
  {0x1F72F, 8571},
  {0x1F730, 7335},
  {0x1F731, 7377},
  {0x1F732, 7305},
  {0x1F733, 7459},
  {0x1F734, 7491},
  {0x1F735, 7523},
  {0x1F736, 4997},
  {0x1F737, 5026},
  {0x1F738, 6671},
  {0x1F739, 7650},
  {0x1F73A, 5318},
  {0x1F73B, 7243},
  {0x1F73C, 7273},
  {0x1F73D, 5376},
  {0x1F73E, 5483},
  {0x1F73F, 8302},
  {0x1F740, 8331},
  {0x1F741, 7175},
  {0x1F742, 5552},
  {0x1F743, 5580},
  {0x1F744, 5610},
  {0x1F745, 5057},
  {0x1F746, 6911},
  {0x1F747, 7894},
  {0x1F748, 8392},
  {0x1F749, 6355},
  {0x1F74A, 8767},
  {0x1F74B, 7011},
  {0x1F74C, 5699},
  {0x1F74D, 8453},
  {0x1F74E, 5726},
  {0x1F74F, 7800},
  {0x1F750, 5668},
  {0x1F751, 8423},
  {0x1F752, 7923},
  {0x1F753, 6639},
  {0x1F754, 7867},
  {0x1F755, 8481},
  {0x1F756, 6446},
  {0x1F757, 5348},
  {0x1F758, 6979},
  {0x1F759, 5640},
  {0x1F75A, 7040},
  {0x1F75B, 5084},
  {0x1F75C, 7961},
  {0x1F75D, 8005},
  {0x1F75E, 8239},
  {0x1F75F, 7077},
  {0x1F760, 6243},
  {0x1F761, 6179},
  {0x1F762, 6210},
  {0x1F763, 7111},
  {0x1F764, 7140},
  {0x1F765, 5984},
  {0x1F766, 6015},
  {0x1F767, 6048},
  {0x1F768, 6081},
  {0x1F769, 6114},
  {0x1F76A, 4967},
  {0x1F76B, 5410},
  {0x1F76C, 5445},
  {0x1F76D, 7555},
  {0x1F76E, 6479},
  {0x1F76F, 6855},
  {0x1F770, 6147},
  {0x1F771, 6827},
  {0x1F772, 6381},
  {0x1F773, 6413},
  {0x1F774, 652676},
  {0x1F775, 785457},
  {0x1F776, 656848},
  {0x1F77B, 505946},
  {0x1F77C, 660242},
  {0x1F77D, 457925},
  {0x1F77E, 829906},
  {0x1F77F, 801054},
  {0x1F780, 114073},
  {0x1F781, 116146},
  {0x1F782, 115149},
  {0x1F783, 113299},
  {0x1F784, 115584},
  {0x1F785, 708151},
  {0x1F786, 122086},
  {0x1F787, 509251},
  {0x1F788, 975902},
  {0x1F789, 442479},
  {0x1F78A, 983786},
  {0x1F78B, 843066},
  {0x1F78C, 115959},
  {0x1F78D, 115612},
  {0x1F78E, 636941},
  {0x1F78F, 708907},
  {0x1F790, 122104},
  {0x1F791, 509305},
  {0x1F792, 975926},
  {0x1F793, 442508},
  {0x1F794, 986003},
  {0x1F795, 985916},
  {0x1F796, 902691},
  {0x1F797, 115921},
  {0x1F798, 116386},
  {0x1F799, 114622},
  {0x1F79A, 984330},
  {0x1F79B, 984239},
  {0x1F79C, 276470},
  {0x1F79D, 115940},
  {0x1F79E, 116411},
  {0x1F79F, 114649},
  {0x1F7A0, 985280},
  {0x1F7A1, 937303},
  {0x1F7A2, 636691},
  {0x1F7A3, 708417},
  {0x1F7A4, 122031},
  {0x1F7A5, 975660},
  {0x1F7A6, 975783},
  {0x1F7A7, 442391},
  {0x1F7A8, 937320},
  {0x1F7A9, 636807},
  {0x1F7AA, 708689},
  {0x1F7AB, 122048},
  {0x1F7AC, 508426},
  {0x1F7AD, 975833},
  {0x1F7AE, 442419},
  {0x1F7AF, 636604},
  {0x1F7B0, 708237},
  {0x1F7B1, 122005},
  {0x1F7B2, 507300},
  {0x1F7B3, 975751},
  {0x1F7B4, 442354},
  {0x1F7B5, 636833},
  {0x1F7B6, 708747},
  {0x1F7B7, 122061},
  {0x1F7B8, 508634},
  {0x1F7B9, 975852},
  {0x1F7BA, 442443},
  {0x1F7BB, 636546},
  {0x1F7BC, 708208},
  {0x1F7BD, 121978},
  {0x1F7BE, 507115},
  {0x1F7BF, 975718},
  {0x1F7C0, 636859},
  {0x1F7C1, 708800},
  {0x1F7C2, 937607},
  {0x1F7C3, 708832},
  {0x1F7C4, 636661},
  {0x1F7C5, 708352},
  {0x1F7C6, 444796},
  {0x1F7C7, 708383},
  {0x1F7C8, 832723},
  {0x1F7C9, 636574},
  {0x1F7CA, 507270},
  {0x1F7CB, 708717},
  {0x1F7CC, 508605},
  {0x1F7CD, 891450},
  {0x1F7CE, 708176},
  {0x1F7CF, 507007},
  {0x1F7D0, 975682},
  {0x1F7D1, 507038},
  {0x1F7D2, 636890},
  {0x1F7D3, 508979},
  {0x1F7D4, 509011},
  {0x1F7D5, 191589},
  {0x1F7D6, 763534},
  {0x1F7D7, 191560},
  {0x1F7D8, 763510},
  {0x1F7D9, 770525},
  {0x1F7E0, 580113},
  {0x1F7E1, 582280},
  {0x1F7E2, 579964},
  {0x1F7E3, 580174},
  {0x1F7E4, 579897},
  {0x1F7E5, 580231},
  {0x1F7E6, 579879},
  {0x1F7E7, 580154},
  {0x1F7E8, 582300},
  {0x1F7E9, 579983},
  {0x1F7EA, 580194},
  {0x1F7EB, 579916},
  {0x1F7F0, 507190},
  {0x1F800, 631107},
  {0x1F801, 960519},
  {0x1F802, 838659},
  {0x1F803, 288337},
  {0x1F804, 630945},
  {0x1F805, 960395},
  {0x1F806, 838493},
  {0x1F807, 288207},
  {0x1F808, 630873},
  {0x1F809, 960351},
  {0x1F80A, 838386},
  {0x1F80B, 288161},
  {0x1F810, 631058},
  {0x1F811, 960472},
  {0x1F812, 838609},
  {0x1F813, 288288},
  {0x1F814, 630804},
  {0x1F815, 960273},
  {0x1F816, 838315},
  {0x1F817, 288079},
  {0x1F818, 507597},
  {0x1F819, 509096},
  {0x1F81A, 508284},
  {0x1F81B, 506903},
  {0x1F81C, 507646},
  {0x1F81D, 509143},
  {0x1F81E, 508334},
  {0x1F81F, 506952},
  {0x1F820, 633322},
  {0x1F821, 961788},
  {0x1F822, 840930},
  {0x1F823, 289701},
  {0x1F824, 633272},
  {0x1F825, 961740},
  {0x1F826, 840879},
  {0x1F827, 289651},
  {0x1F828, 633003},
  {0x1F829, 961478},
  {0x1F82A, 840605},
  {0x1F82B, 289379},
  {0x1F82C, 633113},
  {0x1F82D, 961584},
  {0x1F82E, 840717},
  {0x1F82F, 289489},
  {0x1F830, 633372},
  {0x1F831, 961836},
  {0x1F832, 840981},
  {0x1F833, 289751},
  {0x1F834, 631811},
  {0x1F835, 960751},
  {0x1F836, 839384},
  {0x1F837, 288614},
  {0x1F838, 632751},
  {0x1F839, 961287},
  {0x1F83A, 840347},
  {0x1F83B, 289180},
  {0x1F83C, 631571},
  {0x1F83D, 960684},
  {0x1F83E, 839136},
  {0x1F83F, 288541},
  {0x1F840, 632504},
  {0x1F841, 961170},
  {0x1F842, 840091},
  {0x1F843, 289053},
  {0x1F844, 632482},
  {0x1F845, 961150},
  {0x1F846, 840068},
  {0x1F847, 289031},
  {0x1F850, 632724},
  {0x1F851, 961262},
  {0x1F852, 840319},
  {0x1F853, 289153},
  {0x1F854, 773113},
  {0x1F855, 772317},
  {0x1F856, 896602},
  {0x1F857, 896960},
  {0x1F858, 627999},
  {0x1F859, 955506},
  {0x1F860, 987262},
  {0x1F861, 988453},
  {0x1F862, 987859},
  {0x1F863, 987067},
  {0x1F864, 987659},
  {0x1F865, 987459},
  {0x1F866, 988059},
  {0x1F867, 988259},
  {0x1F868, 987190},
  {0x1F869, 988385},
  {0x1F86A, 987785},
  {0x1F86B, 986995},
  {0x1F86C, 987585},
  {0x1F86D, 987385},
  {0x1F86E, 987985},
  {0x1F86F, 988185},
  {0x1F870, 987301},
  {0x1F871, 988490},
  {0x1F872, 987899},
  {0x1F873, 987106},
  {0x1F874, 987699},
  {0x1F875, 987499},
  {0x1F876, 988099},
  {0x1F877, 988299},
  {0x1F878, 987223},
  {0x1F879, 988416},
  {0x1F87A, 987819},
  {0x1F87B, 987028},
  {0x1F87C, 987619},
  {0x1F87D, 987419},
  {0x1F87E, 988019},
  {0x1F87F, 988219},
  {0x1F880, 987341},
  {0x1F881, 988528},
  {0x1F882, 987940},
  {0x1F883, 987146},
  {0x1F884, 987740},
  {0x1F885, 987540},
  {0x1F886, 988140},
  {0x1F887, 988340},
  {0x1F890, 632833},
  {0x1F891, 961309},
  {0x1F892, 840432},
  {0x1F893, 289204},
  {0x1F894, 634016},
  {0x1F895, 962334},
  {0x1F896, 841674},
  {0x1F897, 290047},
  {0x1F898, 630992},
  {0x1F899, 960440},
  {0x1F89A, 838541},
  {0x1F89B, 288254},
  {0x1F89C, 506136},
  {0x1F89D, 506231},
  {0x1F89E, 506164},
  {0x1F89F, 506197},
  {0x1F8A0, 631535},
  {0x1F8A1, 839099},
  {0x1F8A2, 632800},
  {0x1F8A3, 840398},
  {0x1F8A4, 632537},
  {0x1F8A5, 840265},
  {0x1F8A6, 632672},
  {0x1F8A7, 840125},
  {0x1F8A8, 631434},
  {0x1F8A9, 838995},
  {0x1F8AA, 631839},
  {0x1F8AB, 839413},
  {0x1F8AC, 982508},
  {0x1F8AD, 982536},
  {0x1F8B0, 81874},
  {0x1F8B1, 81777},
  {0x1F8B2, 838460},
  {0x1F8B3, 288476},
  {0x1F8B4, 764798},
  {0x1F8B5, 764936},
  {0x1F8B6, 764882},
  {0x1F8B7, 763705},
  {0x1F8B8, 772878},
  {0x1F8B9, 772138},
  {0x1F8BA, 896396},
  {0x1F8BB, 896781},
  {0x1F8C0, 630453},
  {0x1F8C1, 837964},
  {0x1F900, 184748},
  {0x1F901, 184784},
  {0x1F902, 184727},
  {0x1F903, 626857},
  {0x1F904, 626889},
  {0x1F905, 626922},
  {0x1F906, 626831},
  {0x1F907, 626814},
  {0x1F908, 287721},
  {0x1F909, 287772},
  {0x1F90A, 287742},
  {0x1F90B, 287801},
  {0x1F90C, 823027},
  {0x1F90D, 984869},
  {0x1F90E, 141909},
  {0x1F90F, 823043},
  {0x1F910, 1012956},
  {0x1F911, 737493},
  {0x1F912, 443350},
  {0x1F913, 765334},
  {0x1F914, 937344},
  {0x1F915, 442809},
  {0x1F916, 841937},
  {0x1F917, 524273},
  {0x1F918, 859350},
  {0x1F919, 153139},
  {0x1F91A, 830192},
  {0x1F91B, 629351},
  {0x1F91C, 836809},
  {0x1F91D, 491313},
  {0x1F91E, 491212},
  {0x1F91F, 524445},
  {0x1F920, 442725},
  {0x1F921, 196496},
  {0x1F922, 762032},
  {0x1F923, 842022},
  {0x1F924, 290220},
  {0x1F925, 657785},
  {0x1F926, 442594},
  {0x1F927, 894028},
  {0x1F928, 442946},
  {0x1F929, 480706},
  {0x1F92A, 480627},
  {0x1F92B, 442771},
  {0x1F92C, 852981},
  {0x1F92D, 893832},
  {0x1F92E, 443072},
  {0x1F92F, 856157},
  {0x1F930, 826636},
  {0x1F931, 141759},
  {0x1F932, 812109},
  {0x1F933, 850708},
  {0x1F934, 828367},
  {0x1F935, 663303},
  {0x1F936, 742332},
  {0x1F937, 856705},
  {0x1F938, 819483},
  {0x1F939, 538500},
  {0x1F93A, 443729},
  {0x1F93B, 725565},
  {0x1F93C, 988917},
  {0x1F93D, 982098},
  {0x1F93E, 491304},
  {0x1F93F, 280555},
  {0x1F940, 988591},
  {0x1F941, 290256},
  {0x1F942, 195090},
  {0x1F943, 953198},
  {0x1F944, 899531},
  {0x1F945, 457896},
  {0x1F946, 833813},
  {0x1F947, 443968},
  {0x1F948, 850417},
  {0x1F949, 937358},
  {0x1F94A, 132687},
  {0x1F94B, 666899},
  {0x1F94C, 251594},
  {0x1F94D, 578339},
  {0x1F94E, 894182},
  {0x1F94F, 444388},
  {0x1F950, 214891},
  {0x1F951, 83242},
  {0x1F952, 215231},
  {0x1F953, 83484},
  {0x1F954, 826121},
  {0x1F955, 174654},
  {0x1F956, 83566},
  {0x1F957, 480539},
  {0x1F958, 853175},
  {0x1F959, 907211},
  {0x1F95A, 293848},
  {0x1F95B, 457828},
  {0x1F95C, 819195},
  {0x1F95D, 578242},
  {0x1F95E, 812859},
  {0x1F95F, 290282},
  {0x1F960, 444623},
  {0x1F961, 924429},
  {0x1F962, 183402},
  {0x1F963, 124752},
  {0x1F964, 251564},
  {0x1F965, 196537},
  {0x1F966, 141835},
  {0x1F967, 822964},
  {0x1F968, 828339},
  {0x1F969, 251777},
  {0x1F96A, 847909},
  {0x1F96B, 173641},
  {0x1F96C, 625697},
  {0x1F96D, 663930},
  {0x1F96E, 742266},
  {0x1F96F, 83546},
  {0x1F970, 893887},
  {0x1F971, 989263},
  {0x1F972, 893964},
  {0x1F973, 443102},
  {0x1F974, 443372},
  {0x1F975, 808466},
  {0x1F976, 444979},
  {0x1F977, 770549},
  {0x1F978, 278852},
  {0x1F979, 442557},
  {0x1F97A, 443159},
  {0x1F97B, 848123},
  {0x1F97C, 578324},
  {0x1F97D, 457910},
  {0x1F97E, 521269},
  {0x1F97F, 444232},
  {0x1F980, 214821},
  {0x1F981, 650775},
  {0x1F982, 850114},
  {0x1F983, 953212},
  {0x1F984, 954922},
  {0x1F985, 293663},
  {0x1F986, 290277},
  {0x1F987, 105382},
  {0x1F988, 855176},
  {0x1F989, 808587},
  {0x1F98A, 444880},
  {0x1F98B, 143027},
  {0x1F98C, 269475},
  {0x1F98D, 457940},
  {0x1F98E, 651648},
  {0x1F98F, 833582},
  {0x1F990, 856698},
  {0x1F991, 906552},
  {0x1F992, 454695},
  {0x1F993, 1012857},
  {0x1F994, 512780},
  {0x1F995, 850052},
  {0x1F996, 916268},
  {0x1F997, 214852},
  {0x1F998, 540269},
  {0x1F999, 651655},
  {0x1F99A, 819187},
  {0x1F99B, 521294},
  {0x1F99C, 817438},
  {0x1F99D, 830058},
  {0x1F99E, 651661},
  {0x1F99F, 742323},
  {0x1F9A0, 725016},
  {0x1F9A1, 83505},
  {0x1F9A2, 911888},
  {0x1F9A3, 663219},
  {0x1F9A4, 280779},
  {0x1F9A5, 892070},
  {0x1F9A6, 805306},
  {0x1F9A7, 801044},
  {0x1F9A8, 891612},
  {0x1F9A9, 444209},
  {0x1F9AA, 808594},
  {0x1F9AB, 106773},
  {0x1F9AC, 111325},
  {0x1F9AD, 850407},
  {0x1F9AE, 480783},
  {0x1F9AF, 828424},
  {0x1F9B0, 428362},
  {0x1F9B1, 428335},
  {0x1F9B2, 428314},
  {0x1F9B3, 428387},
  {0x1F9B4, 122127},
  {0x1F9B5, 634064},
  {0x1F9B6, 444524},
  {0x1F9B7, 948020},
  {0x1F9B8, 910962},
  {0x1F9B9, 911826},
  {0x1F9BA, 846257},
  {0x1F9BB, 293698},
  {0x1F9BC, 742374},
  {0x1F9BD, 665257},
  {0x1F9BE, 705523},
  {0x1F9BF, 705538},
  {0x1F9C0, 179459},
  {0x1F9C1, 251579},
  {0x1F9C2, 846310},
  {0x1F9C3, 109060},
  {0x1F9C4, 449639},
  {0x1F9C5, 800572},
  {0x1F9C6, 443457},
  {0x1F9C7, 978584},
  {0x1F9C8, 143020},
  {0x1F9C9, 668852},
  {0x1F9CA, 524476},
  {0x1F9CB, 141921},
  {0x1F9CC, 950966},
  {0x1F9CD, 906620},
  {0x1F9CE, 578252},
  {0x1F9CF, 269369},
  {0x1F9D0, 442881},
  {0x1F9D1, 2239},
  {0x1F9D2, 183348},
  {0x1F9D3, 800296},
  {0x1F9D4, 106744},
  {0x1F9D5, 819693},
  {0x1F9D6, 819547},
  {0x1F9D7, 819467},
  {0x1F9D8, 819522},
  {0x1F9D9, 657803},
  {0x1F9DA, 443451},
  {0x1F9DB, 967491},
  {0x1F9DC, 721995},
  {0x1F9DD, 427815},
  {0x1F9DE, 449703},
  {0x1F9DF, 1019922},
  {0x1F9E0, 141747},
  {0x1F9E1, 801031},
  {0x1F9E2, 111270},
  {0x1F9E3, 850078},
  {0x1F9E4, 457876},
  {0x1F9E5, 196507},
  {0x1F9E6, 894126},
  {0x1F9E7, 830892},
  {0x1F9E8, 443928},
  {0x1F9E9, 538466},
  {0x1F9EA, 932318},
  {0x1F9EB, 819792},
  {0x1F9EC, 280660},
  {0x1F9ED, 208780},
  {0x1F9EE, 10},
  {0x1F9EF, 443910},
  {0x1F9F0, 948012},
  {0x1F9F1, 141780},
  {0x1F9F2, 657819},
  {0x1F9F3, 656840},
  {0x1F9F4, 652691},
  {0x1F9F5, 899515},
  {0x1F9F6, 87071},
  {0x1F9F7, 846246},
  {0x1F9F8, 929948},
  {0x1F9F9, 141903},
  {0x1F9FA, 104567},
  {0x1F9FB, 841960},
  {0x1F9FC, 104534},
  {0x1F9FD, 899508},
  {0x1F9FE, 830522},
  {0x1F9FF, 762047},
  {0x1FA00, 765582},
  {0x1FA01, 766165},
  {0x1FA02, 766284},
  {0x1FA03, 765387},
  {0x1FA04, 765698},
  {0x1FA05, 766049},
  {0x1FA06, 982898},
  {0x1FA07, 111706},
  {0x1FA08, 765719},
  {0x1FA09, 982786},
  {0x1FA0A, 983419},
  {0x1FA0B, 983531},
  {0x1FA0C, 982603},
  {0x1FA0D, 982944},
  {0x1FA0E, 983308},
  {0x1FA0F, 111594},
  {0x1FA10, 112227},
  {0x1FA11, 112339},
  {0x1FA12, 111411},
  {0x1FA13, 111752},
  {0x1FA14, 112116},
  {0x1FA15, 765601},
  {0x1FA16, 766185},
  {0x1FA17, 766303},
  {0x1FA18, 765408},
  {0x1FA19, 765767},
  {0x1FA1A, 766068},
  {0x1FA1B, 982986},
  {0x1FA1C, 111794},
  {0x1FA1D, 765811},
  {0x1FA1E, 983650},
  {0x1FA1F, 983724},
  {0x1FA20, 983749},
  {0x1FA21, 983624},
  {0x1FA22, 983674},
  {0x1FA23, 983700},
  {0x1FA24, 112458},
  {0x1FA25, 112532},
  {0x1FA26, 112557},
  {0x1FA27, 112432},
  {0x1FA28, 112482},
  {0x1FA29, 112508},
  {0x1FA2A, 766428},
  {0x1FA2B, 766508},
  {0x1FA2C, 766535},
  {0x1FA2D, 766400},
  {0x1FA2E, 766454},
  {0x1FA2F, 766482},
  {0x1FA30, 983157},
  {0x1FA31, 111965},
  {0x1FA32, 765988},
  {0x1FA33, 982826},
  {0x1FA34, 983460},
  {0x1FA35, 983571},
  {0x1FA36, 982645},
  {0x1FA37, 983102},
  {0x1FA38, 983348},
  {0x1FA39, 111634},
  {0x1FA3A, 112268},
  {0x1FA3B, 112379},
  {0x1FA3C, 111453},
  {0x1FA3D, 111910},
  {0x1FA3E, 112156},
  {0x1FA3F, 765643},
  {0x1FA40, 766228},
  {0x1FA41, 766345},
  {0x1FA42, 765452},
  {0x1FA43, 765931},
  {0x1FA44, 766110},
  {0x1FA45, 983045},
  {0x1FA46, 111853},
  {0x1FA47, 765872},
  {0x1FA48, 982700},
  {0x1FA49, 111508},
  {0x1FA4A, 765509},
  {0x1FA4B, 982723},
  {0x1FA4C, 111531},
  {0x1FA4D, 765534},
  {0x1FA4E, 983242},
  {0x1FA4F, 983267},
  {0x1FA50, 983216},
  {0x1FA51, 112050},
  {0x1FA52, 112075},
  {0x1FA53, 112024},
  {0x1FA60, 989180},
  {0x1FA61, 989218},
  {0x1FA62, 989159},
  {0x1FA63, 989200},
  {0x1FA64, 989139},
  {0x1FA65, 989120},
  {0x1FA66, 989239},
  {0x1FA67, 989033},
  {0x1FA68, 989075},
  {0x1FA69, 989010},
  {0x1FA6A, 989055},
  {0x1FA6B, 988988},
  {0x1FA6C, 988967},
  {0x1FA6D, 989098},
  {0x1FA70, 87084},
  {0x1FA71, 800553},
  {0x1FA72, 141828},
  {0x1FA73, 856657},
  {0x1FA74, 937376},
  {0x1FA75, 636512},
  {0x1FA76, 480551},
  {0x1FA77, 823083},
  {0x1FA78, 290234},
  {0x1FA79, 156},
  {0x1FA7A, 906827},
  {0x1FA7B, 988961},
  {0x1FA7C, 215159},
  {0x1FA80, 1010250},
  {0x1FA81, 578237},
  {0x1FA82, 812889},
  {0x1FA83, 122166},
  {0x1FA84, 657808},
  {0x1FA85, 823020},
  {0x1FA86, 765366},
  {0x1FA87, 665312},
  {0x1FA88, 444378},
  {0x1FA89, 505420},
  {0x1FA8F, 856684},
  {0x1FA90, 841805},
  {0x1FA91, 176478},
  {0x1FA92, 830516},
  {0x1FA93, 83250},
  {0x1FA94, 280626},
  {0x1FA95, 104415},
  {0x1FA96, 725208},
  {0x1FA97, 28},
  {0x1FA98, 652287},
  {0x1FA99, 196552},
  {0x1FA9A, 174640},
  {0x1FA9B, 850139},
  {0x1FA9C, 578363},
  {0x1FA9D, 523362},
  {0x1FA9E, 725466},
  {0x1FA9F, 988634},
  {0x1FAA0, 825416},
  {0x1FAA1, 853071},
  {0x1FAA2, 578268},
  {0x1FAA3, 141940},
  {0x1FAA4, 742495},
  {0x1FAA5, 948026},
  {0x1FAA6, 505963},
  {0x1FAA7, 823155},
  {0x1FAA8, 841948},
  {0x1FAA9, 725473},
  {0x1FAAA, 524593},
  {0x1FAAB, 652766},
  {0x1FAAC, 491193},
  {0x1FAAD, 444500},
  {0x1FAAE, 487404},
  {0x1FAAF, 552415},
  {0x1FAB0, 444384},
  {0x1FAB1, 988861},
  {0x1FAB2, 106801},
  {0x1FAB3, 196512},
  {0x1FAB4, 826128},
  {0x1FAB5, 988808},
  {0x1FAB6, 443682},
  {0x1FAB7, 652705},
  {0x1FAB8, 214186},
  {0x1FAB9, 428617},
  {0x1FABA, 765351},
  {0x1FABB, 524324},
  {0x1FABC, 538456},
  {0x1FABD, 988652},
  {0x1FABE, 625683},
  {0x1FABF, 457934},
  {0x1FAC0, 24579},
  {0x1FAC1, 656862},
  {0x1FAC2, 819283},
  {0x1FAC3, 826607},
  {0x1FAC4, 826620},
  {0x1FAC5, 819650},
  {0x1FAC6, 443860},
  {0x1FACE, 742310},
  {0x1FACF, 284958},
  {0x1FAD0, 121961},
  {0x1FAD1, 106813},
  {0x1FAD2, 800330},
  {0x1FAD3, 444242},
  {0x1FAD4, 925643},
  {0x1FAD5, 444517},
  {0x1FAD6, 929865},
  {0x1FAD7, 826170},
  {0x1FAD8, 106728},
  {0x1FAD9, 536372},
  {0x1FADA, 454683},
  {0x1FADB, 819160},
  {0x1FADC, 842875},
  {0x1FADF, 899499},
  {0x1FAE0, 710966},
  {0x1FAE1, 846330},
  {0x1FAE2, 442975},
  {0x1FAE3, 443137},
  {0x1FAE4, 442746},
  {0x1FAE5, 285052},
  {0x1FAE6, 111344},
  {0x1FAE7, 141932},
  {0x1FAE8, 853162},
  {0x1FAE9, 442678},
  {0x1FAF0, 491255},
  {0x1FAF1, 839458},
  {0x1FAF2, 631883},
  {0x1FAF3, 812071},
  {0x1FAF4, 812096},
  {0x1FAF5, 530656},
  {0x1FAF6, 506037},
  {0x1FAF7, 632623},
  {0x1FAF8, 840214},
  {0x1FB00, 120843},
  {0x1FB01, 121396},
  {0x1FB02, 120859},
  {0x1FB03, 121666},
  {0x1FB04, 121142},
  {0x1FB05, 121412},
  {0x1FB06, 120876},
  {0x1FB07, 121806},
  {0x1FB08, 121272},
  {0x1FB09, 121560},
  {0x1FB0A, 121011},
  {0x1FB0B, 121682},
  {0x1FB0C, 121159},
  {0x1FB0D, 121429},
  {0x1FB0E, 120894},
  {0x1FB0F, 121874},
  {0x1FB10, 121344},
  {0x1FB11, 121614},
  {0x1FB12, 121087},
  {0x1FB13, 121754},
  {0x1FB14, 121505},
  {0x1FB15, 120953},
  {0x1FB16, 121822},
  {0x1FB17, 121289},
  {0x1FB18, 121577},
  {0x1FB19, 121029},
  {0x1FB1A, 121699},
  {0x1FB1B, 121177},
  {0x1FB1C, 121447},
  {0x1FB1D, 120913},
  {0x1FB1E, 121907},
  {0x1FB1F, 121379},
  {0x1FB20, 121649},
  {0x1FB21, 121124},
  {0x1FB22, 121789},
  {0x1FB23, 121254},
  {0x1FB24, 121542},
  {0x1FB25, 120992},
  {0x1FB26, 121857},
  {0x1FB27, 121326},
  {0x1FB28, 121068},
  {0x1FB29, 121736},
  {0x1FB2A, 121216},
  {0x1FB2B, 121486},
  {0x1FB2C, 120933},
  {0x1FB2D, 121890},
  {0x1FB2E, 121361},
  {0x1FB2F, 121631},
  {0x1FB30, 121105},
  {0x1FB31, 121771},
  {0x1FB32, 121235},
  {0x1FB33, 121523},
  {0x1FB34, 120972},
  {0x1FB35, 121839},
  {0x1FB36, 121307},
  {0x1FB37, 121595},
  {0x1FB38, 121048},
  {0x1FB39, 121717},
  {0x1FB3A, 121196},
  {0x1FB3B, 121466},
  {0x1FB3C, 653425},
  {0x1FB3D, 653485},
  {0x1FB3E, 653891},
  {0x1FB3F, 654017},
  {0x1FB40, 653720},
  {0x1FB41, 655629},
  {0x1FB42, 655690},
  {0x1FB43, 655441},
  {0x1FB44, 655569},
  {0x1FB45, 655327},
  {0x1FB46, 655502},
  {0x1FB47, 655088},
  {0x1FB48, 655267},
  {0x1FB49, 655150},
  {0x1FB4A, 655381},
  {0x1FB4B, 655212},
  {0x1FB4C, 653659},
  {0x1FB4D, 653832},
  {0x1FB4E, 653544},
  {0x1FB4F, 653773},
  {0x1FB50, 653605},
  {0x1FB51, 653951},
  {0x1FB52, 958434},
  {0x1FB53, 958495},
  {0x1FB54, 958908},
  {0x1FB55, 959036},
  {0x1FB56, 958734},
  {0x1FB57, 957382},
  {0x1FB58, 957442},
  {0x1FB59, 957197},
  {0x1FB5A, 957323},
  {0x1FB5B, 957085},
  {0x1FB5C, 957257},
  {0x1FB5D, 956850},
  {0x1FB5E, 957026},
  {0x1FB5F, 956911},
  {0x1FB60, 957138},
  {0x1FB61, 956972},
  {0x1FB62, 958672},
  {0x1FB63, 958848},
  {0x1FB64, 958555},
  {0x1FB65, 958788},
  {0x1FB66, 958617},
  {0x1FB67, 958969},
  {0x1FB68, 956228},
  {0x1FB69, 625716},
  {0x1FB6A, 956100},
  {0x1FB6B, 625894},
  {0x1FB6C, 628937},
  {0x1FB6D, 960031},
  {0x1FB6E, 836392},
  {0x1FB6F, 656762},
  {0x1FB70, 975352},
  {0x1FB71, 975380},
  {0x1FB72, 975408},
  {0x1FB73, 975436},
  {0x1FB74, 975464},
  {0x1FB75, 975492},
  {0x1FB76, 523689},
  {0x1FB77, 523719},
  {0x1FB78, 523749},
  {0x1FB79, 523779},
  {0x1FB7A, 523809},
  {0x1FB7B, 523839},
  {0x1FB7C, 625773},
  {0x1FB7D, 625951},
  {0x1FB7E, 833852},
  {0x1FB7F, 833819},
  {0x1FB80, 956157},
  {0x1FB81, 523656},
  {0x1FB82, 958410},
  {0x1FB83, 959889},
  {0x1FB84, 956482},
  {0x1FB85, 959915},
  {0x1FB86, 959863},
  {0x1FB87, 835234},
  {0x1FB88, 836142},
  {0x1FB89, 834629},
  {0x1FB8A, 836168},
  {0x1FB8B, 835486},
  {0x1FB8C, 627128},
  {0x1FB8D, 834894},
  {0x1FB8E, 956744},
  {0x1FB8F, 653263},
  {0x1FB90, 535881},
  {0x1FB91, 956524},
  {0x1FB92, 956625},
  {0x1FB94, 627042},
  {0x1FB95, 179421},
  {0x1FB96, 535811},
  {0x1FB97, 507418},
  {0x1FB98, 958040},
  {0x1FB99, 959748},
  {0x1FB9A, 956190},
  {0x1FB9B, 625857},
  {0x1FB9C, 958091},
  {0x1FB9D, 959800},
  {0x1FB9E, 656531},
  {0x1FB9F, 654745},
  {0x1FBA0, 128489},
  {0x1FBA1, 128794},
  {0x1FBA2, 127835},
  {0x1FBA3, 128178},
  {0x1FBA4, 128634},
  {0x1FBA5, 128940},
  {0x1FBA6, 127891},
  {0x1FBA7, 128018},
  {0x1FBA8, 128545},
  {0x1FBA9, 128851},
  {0x1FBAA, 129013},
  {0x1FBAB, 128706},
  {0x1FBAC, 128090},
  {0x1FBAD, 128290},
  {0x1FBAE, 127658},
  {0x1FBAF, 130186},
  {0x1FBB0, 81913},
  {0x1FBB1, 535792},
  {0x1FBB2, 627151},
  {0x1FBB3, 834918},
  {0x1FBB4, 535838},
  {0x1FBB5, 630333},
  {0x1FBB6, 837841},
  {0x1FBB7, 287872},
  {0x1FBB8, 960112},
  {0x1FBB9, 626977},
  {0x1FBBA, 834792},
  {0x1FBBB, 977996},
  {0x1FBBC, 835258},
  {0x1FBBD, 763560},
  {0x1FBBE, 763610},
  {0x1FBBF, 763584},
  {0x1FBC0, 984921},
  {0x1FBC1, 628651},
  {0x1FBC2, 725140},
  {0x1FBC3, 836082},
  {0x1FBC4, 764851},
  {0x1FBC5, 906839},
  {0x1FBC6, 906905},
  {0x1FBC7, 906852},
  {0x1FBC8, 906878},
  {0x1FBC9, 906935},
  {0x1FBCA, 986704},
  {0x1FBCB, 984208},
  {0x1FBCC, 830455},
  {0x1FBCD, 115716},
  {0x1FBCE, 628971},
  {0x1FBCF, 627509},
  {0x1FBD0, 128235},
  {0x1FBD1, 129727},
  {0x1FBD2, 129421},
  {0x1FBD3, 127963},
  {0x1FBD4, 129101},
  {0x1FBD5, 128433},
  {0x1FBD6, 129545},
  {0x1FBD7, 128378},
  {0x1FBD8, 129350},
  {0x1FBD9, 129655},
  {0x1FBDA, 127694},
  {0x1FBDB, 129280},
  {0x1FBDC, 129156},
  {0x1FBDD, 129782},
  {0x1FBDE, 127765},
  {0x1FBDF, 129476},
  {0x1FBE0, 948668},
  {0x1FBE1, 835104},
  {0x1FBE2, 124302},
  {0x1FBE3, 627332},
  {0x1FBE4, 956378},
  {0x1FBE5, 652964},
  {0x1FBE6, 725057},
  {0x1FBE7, 725087},
  {0x1FBE8, 948630},
  {0x1FBE9, 835065},
  {0x1FBEA, 124261},
  {0x1FBEB, 627293},
  {0x1FBEC, 948925},
  {0x1FBED, 124404},
  {0x1FBEE, 124542},
  {0x1FBEF, 948802},
  {0x1FBF0, 850687},
  {0x1FBF1, 850583},
  {0x1FBF2, 850667},
  {0x1FBF3, 850645},
  {0x1FBF4, 850541},
  {0x1FBF5, 850520},
  {0x1FBF6, 850625},
  {0x1FBF7, 850603},
  {0x1FBF8, 850498},
  {0x1FBF9, 850562},
  {0xE0001, 578382},
  {0xE0020, 918344},
  {0xE0021, 916600},
  {0xE0022, 918208},
  {0xE0023, 918143},
  {0xE0024, 916568},
  {0xE0025, 918159},
  {0xE0026, 916316},
  {0xE0027, 916330},
  {0xE0028, 918066},
  {0xE0029, 918271},
  {0xE002A, 916345},
  {0xE002B, 918176},
  {0xE002C, 916390},
  {0xE002D, 916674},
  {0xE002E, 916621},
  {0xE002F, 918332},
  {0xE0030, 916553},
  {0xE0031, 916479},
  {0xE0032, 916539},
  {0xE0033, 916523},
  {0xE0034, 916449},
  {0xE0035, 916434},
  {0xE0036, 916509},
  {0xE0037, 916493},
  {0xE0038, 916418},
  {0xE0039, 916464},
  {0xE003A, 916380},
  {0xE003B, 918318},
  {0xE003C, 918111},
  {0xE003D, 916584},
  {0xE003E, 916652},
  {0xE003F, 918190},
  {0xE0040, 916400},
  {0xE0041, 916691},
  {0xE0042, 916718},
  {0xE0043, 916745},
  {0xE0044, 916772},
  {0xE0045, 916799},
  {0xE0046, 916826},
  {0xE0047, 916853},
  {0xE0048, 916880},
  {0xE0049, 916907},
  {0xE004A, 916934},
  {0xE004B, 916961},
  {0xE004C, 916988},
  {0xE004D, 917015},
  {0xE004E, 917042},
  {0xE004F, 917069},
  {0xE0050, 917096},
  {0xE0051, 917123},
  {0xE0052, 917150},
  {0xE0053, 917177},
  {0xE0054, 917204},
  {0xE0055, 917231},
  {0xE0056, 917258},
  {0xE0057, 917285},
  {0xE0058, 917312},
  {0xE0059, 917339},
  {0xE005A, 917366},
  {0xE005B, 918087},
  {0xE005C, 918227},
  {0xE005D, 918293},
  {0xE005E, 916358},
  {0xE005F, 918130},
  {0xE0060, 916635},
  {0xE0061, 917393},
  {0xE0062, 917418},
  {0xE0063, 917443},
  {0xE0064, 917468},
  {0xE0065, 917493},
  {0xE0066, 917518},
  {0xE0067, 917543},
  {0xE0068, 917568},
  {0xE0069, 917593},
  {0xE006A, 917618},
  {0xE006B, 917643},
  {0xE006C, 917668},
  {0xE006D, 917693},
  {0xE006E, 917718},
  {0xE006F, 917743},
  {0xE0070, 917768},
  {0xE0071, 917793},
  {0xE0072, 917818},
  {0xE0073, 917843},
  {0xE0074, 917868},
  {0xE0075, 917893},
  {0xE0076, 917918},
  {0xE0077, 917943},
  {0xE0078, 917968},
  {0xE0079, 917993},
  {0xE007A, 918018},
  {0xE007B, 918043},
  {0xE007C, 918364},
  {0xE007D, 918247},
  {0xE007E, 918354},
  {0xE007F, 173595},
  {0xE0100, 969284},
  {0xE0101, 969536},
  {0xE0102, 969788},
  {0xE0103, 970061},
  {0xE0104, 970313},
  {0xE0105, 970565},
  {0xE0106, 970817},
  {0xE0107, 971069},
  {0xE0108, 971321},
  {0xE0109, 971504},
  {0xE010A, 971526},
  {0xE010B, 971548},
  {0xE010C, 971570},
  {0xE010D, 971613},
  {0xE010E, 971635},
  {0xE010F, 971657},
  {0xE0110, 971679},
  {0xE0111, 971701},
  {0xE0112, 971723},
  {0xE0113, 971745},
  {0xE0114, 971767},
  {0xE0115, 971789},
  {0xE0116, 971811},
  {0xE0117, 971854},
  {0xE0118, 971876},
  {0xE0119, 971898},
  {0xE011A, 971920},
  {0xE011B, 971942},
  {0xE011C, 971964},
  {0xE011D, 971986},
  {0xE011E, 972008},
  {0xE011F, 972030},
  {0xE0120, 972052},
  {0xE0121, 972095},
  {0xE0122, 972117},
  {0xE0123, 972139},
  {0xE0124, 972161},
  {0xE0125, 972183},
  {0xE0126, 972205},
  {0xE0127, 972227},
  {0xE0128, 972249},
  {0xE0129, 972271},
  {0xE012A, 972293},
  {0xE012B, 972336},
  {0xE012C, 972358},
  {0xE012D, 972380},
  {0xE012E, 972402},
  {0xE012F, 972424},
  {0xE0130, 972446},
  {0xE0131, 972468},
  {0xE0132, 972490},
  {0xE0133, 972512},
  {0xE0134, 972534},
  {0xE0135, 972577},
  {0xE0136, 972599},
  {0xE0137, 972621},
  {0xE0138, 972643},
  {0xE0139, 972665},
  {0xE013A, 972687},
  {0xE013B, 972709},
  {0xE013C, 972731},
  {0xE013D, 972753},
  {0xE013E, 972775},
  {0xE013F, 972818},
  {0xE0140, 972840},
  {0xE0141, 972862},
  {0xE0142, 972884},
  {0xE0143, 972906},
  {0xE0144, 972928},
  {0xE0145, 972950},
  {0xE0146, 972972},
  {0xE0147, 972994},
  {0xE0148, 973016},
  {0xE0149, 973059},
  {0xE014A, 973081},
  {0xE014B, 973103},
  {0xE014C, 973125},
  {0xE014D, 973147},
  {0xE014E, 973169},
  {0xE014F, 973191},
  {0xE0150, 973213},
  {0xE0151, 973235},
  {0xE0152, 973257},
  {0xE0153, 967542},
  {0xE0154, 967565},
  {0xE0155, 967588},
  {0xE0156, 967611},
  {0xE0157, 967634},
  {0xE0158, 967657},
  {0xE0159, 967680},
  {0xE015A, 967703},
  {0xE015B, 967726},
  {0xE015C, 967749},
  {0xE015D, 967794},
  {0xE015E, 967817},
  {0xE015F, 967840},
  {0xE0160, 967863},
  {0xE0161, 967886},
  {0xE0162, 967909},
  {0xE0163, 967932},
  {0xE0164, 967955},
  {0xE0165, 967978},
  {0xE0166, 968001},
  {0xE0167, 968046},
  {0xE0168, 968069},
  {0xE0169, 968092},
  {0xE016A, 968115},
  {0xE016B, 968138},
  {0xE016C, 968161},
  {0xE016D, 968184},
  {0xE016E, 968207},
  {0xE016F, 968230},
  {0xE0170, 968253},
  {0xE0171, 968298},
  {0xE0172, 968321},
  {0xE0173, 968344},
  {0xE0174, 968367},
  {0xE0175, 968390},
  {0xE0176, 968413},
  {0xE0177, 968436},
  {0xE0178, 968459},
  {0xE0179, 968482},
  {0xE017A, 968505},
  {0xE017B, 968550},
  {0xE017C, 968573},
  {0xE017D, 968596},
  {0xE017E, 968619},
  {0xE017F, 968642},
  {0xE0180, 968665},
  {0xE0181, 968688},
  {0xE0182, 968711},
  {0xE0183, 968734},
  {0xE0184, 968757},
  {0xE0185, 968802},
  {0xE0186, 968825},
  {0xE0187, 968848},
  {0xE0188, 968871},
  {0xE0189, 968894},
  {0xE018A, 968917},
  {0xE018B, 968940},
  {0xE018C, 968963},
  {0xE018D, 968986},
  {0xE018E, 969009},
  {0xE018F, 969054},
  {0xE0190, 969077},
  {0xE0191, 969100},
  {0xE0192, 969123},
  {0xE0193, 969146},
  {0xE0194, 969169},
  {0xE0195, 969192},
  {0xE0196, 969215},
  {0xE0197, 969238},
  {0xE0198, 969261},
  {0xE0199, 969306},
  {0xE019A, 969329},
  {0xE019B, 969352},
  {0xE019C, 969375},
  {0xE019D, 969398},
  {0xE019E, 969421},
  {0xE019F, 969444},
  {0xE01A0, 969467},
  {0xE01A1, 969490},
  {0xE01A2, 969513},
  {0xE01A3, 969558},
  {0xE01A4, 969581},
  {0xE01A5, 969604},
  {0xE01A6, 969627},
  {0xE01A7, 969650},
  {0xE01A8, 969673},
  {0xE01A9, 969696},
  {0xE01AA, 969719},
  {0xE01AB, 969742},
  {0xE01AC, 969765},
  {0xE01AD, 969810},
  {0xE01AE, 969833},
  {0xE01AF, 969856},
  {0xE01B0, 969879},
  {0xE01B1, 969902},
  {0xE01B2, 969925},
  {0xE01B3, 969948},
  {0xE01B4, 969971},
  {0xE01B5, 969994},
  {0xE01B6, 970017},
  {0xE01B7, 970083},
  {0xE01B8, 970106},
  {0xE01B9, 970129},
  {0xE01BA, 970152},
  {0xE01BB, 970175},
  {0xE01BC, 970198},
  {0xE01BD, 970221},
  {0xE01BE, 970244},
  {0xE01BF, 970267},
  {0xE01C0, 970290},
  {0xE01C1, 970335},
  {0xE01C2, 970358},
  {0xE01C3, 970381},
  {0xE01C4, 970404},
  {0xE01C5, 970427},
  {0xE01C6, 970450},
  {0xE01C7, 970473},
  {0xE01C8, 970496},
  {0xE01C9, 970519},
  {0xE01CA, 970542},
  {0xE01CB, 970587},
  {0xE01CC, 970610},
  {0xE01CD, 970633},
  {0xE01CE, 970656},
  {0xE01CF, 970679},
  {0xE01D0, 970702},
  {0xE01D1, 970725},
  {0xE01D2, 970748},
  {0xE01D3, 970771},
  {0xE01D4, 970794},
  {0xE01D5, 970839},
  {0xE01D6, 970862},
  {0xE01D7, 970885},
  {0xE01D8, 970908},
  {0xE01D9, 970931},
  {0xE01DA, 970954},
  {0xE01DB, 970977},
  {0xE01DC, 971000},
  {0xE01DD, 971023},
  {0xE01DE, 971046},
  {0xE01DF, 971091},
  {0xE01E0, 971114},
  {0xE01E1, 971137},
  {0xE01E2, 971160},
  {0xE01E3, 971183},
  {0xE01E4, 971206},
  {0xE01E5, 971229},
  {0xE01E6, 971252},
  {0xE01E7, 971275},
  {0xE01E8, 971298},
  {0xE01E9, 971343},
  {0xE01EA, 971366},
  {0xE01EB, 971389},
  {0xE01EC, 971412},
  {0xE01ED, 971435},
  {0xE01EE, 971458},
  {0xE01EF, 971481}
};

static inline const char * font_manager_unicode_name_get_name(const FontManagerUnicodeName *entry)
{
  guint32 offset = entry->name_offset;
  return unicode_names_strings + offset;
}

#endif

